#include <stdlib.h>
#include <stdio.h>

#include "au_io.h"

void au_putdbl(double xd,Taufi *af)
{
	register long	i,x;

	if ( xd<0.0 ) x=(long) (xd-0.5); else x=(long) (xd+0.5);

	if ( (x+32768)>>16 )
	{
		if ( x<-32768 )
		{
			if ( x<af->minuf ) af->minuf=x;
			x=-32768;
		}
		if ( x>32767 )
		{
			if ( x>af->maxof ) af->maxof=x;
			x=32767;
		}
	}

	i=af->bp;
	af->buffer[i  ] = x >> 8;
	af->buffer[i+1] = x & 255;
	i+=2;
	if ( i==AUBUFSIZE )
	{
		fwrite(af->buffer,1,AUBUFSIZE,af->fd);
		af->sbcount+=AUBUFSIZE;
		i=0;
	}
	af->bp=i;
}

void au_read_buffer(Taufi *af)
{
	register long	i,j;

	if ( af->status )
	{
		af->sbcount+=AUBUFSIZE+1;
		af->bp=-1; 
		return;
	}

	i=fread(af->buffer,1,AUBUFSIZE,af->fd);

	if ( i<AUBUFSIZE )
	{
		if ( i<2 ) { af->sbcount+=AUBUFSIZE+1; af->bp=-1; return; }
		af->status=1;
		i&=-2;
		for (j=i-1; j>=0; j--) af->buffer[AUBUFSIZE-i+j]=af->buffer[j];
	}
	af->bp=AUBUFSIZE-i;
	af->sbcount+=i;
}

double au_getdbl(Taufi *af)
{
	register long i=af->bp,j=0;

	if ( i>=0 )
	{
		j =((long) ((signed char *) af->buffer)[i])<<8;
		j|= (long) af->buffer[i+1];
		i+=2;
		af->bp=i;
		if ( i==AUBUFSIZE ) au_read_buffer(af);
	}
	return (double) j;
}

static long be_getint(int *x)
{
	register long	r,h;

	h=((unsigned char *) x)[0];	r =h<<24;
	h=((unsigned char *) x)[1];	r|=h<<16;
	h=((unsigned char *) x)[2];	r|=h<< 8;
	h=((unsigned char *) x)[3];

	return r|h;
}

Taufi *au_open_read(char *name)
{
	register int	*b;
	register long	h,i,j,formc,ok=1;
	register long	ofs   = 0;
	register long	srate = 0;
	register long	nchan = 0;
	register FILE	*fd;
	register Taufi	*af;

	if ( (name[0]=='-') && (name[1]==0) ) fd=stdin; else fd=fopen(name,"r");
	if ( fd==NULL )
	{
		fprintf(stderr,"Could not open audio-file <%s>\n",name);
		exit(1);
	}

	b=(int *) malloc(sizeof(Taufi));

	do
	{
		ok=0;
		if ( fread(b,1,24,fd)<24 ) break;

		h    =be_getint(b);	if ( h!=0x2e736e64 ) break;
		ofs  =be_getint(b+1);
					/* ignore datasize	*/
		formc=be_getint(b+3);	if ( formc!=3 ) break;
		srate=be_getint(b+4);
		nchan=be_getint(b+5);	if ( nchan<1 ) break;

		h=ofs-24;
		ok=1;
		while (h>0)
		{
			i=h;
			if ( h>((long) sizeof(Taufi)) ) i=(long) sizeof(Taufi);
			if ( (j=fread(b,1,i,fd))<i ) { ok=0; break; }
			h-=j;
		}
	}
	while (0);

	if ( !ok )
	{
		fprintf(stderr,"Unsupported audio format in file <%s>\n",name);
		exit(1);
	}

	af=(Taufi *) b;

	af->name     = name;
	af->fd       = fd;
	af->bp       = AUBUFSIZE;
	af->sbcount  = -AUBUFSIZE;
	af->channels = nchan;
	af->samrate  = srate;
	af->status   = 0;
	af->minuf    = 0;
	af->maxof    = 0;

	au_read_buffer(af);

	return af;
}

static void be_putint(int *b,long x)
{
	((unsigned char *) b)[0]=(x>>24)&255;
	((unsigned char *) b)[1]=(x>>16)&255;
	((unsigned char *) b)[2]=(x>> 8)&255;
	((unsigned char *) b)[3]= x     &255;
}

Taufi *au_open_write(char *name,long samrate,long channels)
{
	register int	*b;
	register FILE	*fd;
	register Taufi	*af;

	if ( (name[0]=='-') && (name[1]==0) ) fd=stdout; else fd=fopen(name,"w");
	if ( fd==NULL )
	{
		fprintf(stderr,"Could not open audio-file <%s> for writing\n",name);
		exit(1);
	}

	b=(int *) malloc(sizeof(Taufi));

	be_putint(b  ,0x2e736e64);
	be_putint(b+1,        28);
	be_putint(b+2,        -1);
	be_putint(b+3,         3);
	be_putint(b+4,   samrate);
	be_putint(b+5,  channels);
	be_putint(b+6,         0);
	fwrite(b,1,28,fd);

	af=(Taufi *) b;
	af->name     = name;
	af->fd       = fd;
	af->sbcount  = 0;
	af->bp       = 0;
	af->channels = channels;
	af->samrate  = samrate;
	af->status   = -1;
	af->minuf    = 0;
	af->maxof    = 0;

	return af;
}

void au_close(Taufi *af)
{
	if ( af->maxof ) fprintf(stderr,"Highest overflow: %li\n",af->maxof);
	if ( af->minuf ) fprintf(stderr,"Lowest underflow: %li\n",af->minuf);

	if ( (af->status<0) && (af->bp>0) )
	{
		fwrite(af->buffer,1,af->bp,af->fd);
	}

	if ( (af->fd!=stdin) && (af->fd!=stdout) ) fclose(af->fd);

	free(af);
}
