
#include <stdio.h>

#define uchar	unsigned char
#define ushort	unsigned short
#define uint	unsigned int
#define ulong	unsigned long

#define buffersize	4096

static void be_putuint32(ulong x)
{
	putc( (x>>24) & 255, stdout );
	putc( (x>>16) & 255, stdout );
	putc( (x>> 8) & 255, stdout );
	putc(  x      & 255, stdout );
}

/*
static ulong be_getuint32()
{
	register ulong	r,h;

	h=getc(stdin);	r =h<<24;
	h=getc(stdin);	r|=h<<16;
	h=getc(stdin);	r|=h<< 8;
	h=getc(stdin);	r|=h;

	return r;
}
*/

static ulong le_getuint32()
{
	register ulong	r,h;

	r=getc(stdin);
	h=getc(stdin);	r|=h<< 8;
	h=getc(stdin);	r|=h<<16;
	h=getc(stdin);	r|=h<<24;

	return r;
}

static ulong le_getuint16()
{
	register ulong	r,h;

	r=getc(stdin);
	h=getc(stdin);	r|=h<< 8;

	return r;
}


static long convert_header()
{
	register ulong	samrate,channels;

	if ( le_getuint32()!=0x46464952UL ) return -1;	/* "RIFF" */

	le_getuint32();					/* filesize except "RIFF" and the size-field itself */

	if ( le_getuint32()!=0x45564157UL ) return -1;	/* "WAVE" */
	if ( le_getuint32()!=0x20746D66UL ) return -1;	/* "fmt " */
	if ( le_getuint32()!=        16UL ) return -1;	/* length of fmt data == 16 ! */
	if ( le_getuint16()!=         1UL ) return -1;	/* 1 -> PCM */

	channels = le_getuint16();			/* 1 or 2 */
	samrate  = le_getuint32();			/* sample rate */

	le_getuint32();					/* bytes/second - redundant */
	le_getuint16();					/* block align - redundant */

	if ( le_getuint16()!=16UL ) return -1;		/* sample size; must be 16 */


	be_putuint32( 0x2e736e64UL);	/* ".snd" */
	be_putuint32(         28UL);	/* header size */
	be_putuint32( 0xFFFFFFFFUL);	/* unlimited data size */
	be_putuint32(          3UL);	/* format code: 3 -> PCM 16 bit	*/
	be_putuint32(    samrate  );
	be_putuint32(    channels );
	be_putuint32(          0UL);	/* optional data	*/

	return 0;
}


static ulong find_data_chunk()
{
	register ulong	type,size;

	for (;;)
	{
		type=le_getuint32();
		size=le_getuint32();

		if ( type==0x61746164UL ) return size;	/* "data" -> data chunk found	*/

		while ( size-- )			/* skip other chunks	*/
		{
			if ( getc(stdin)==EOF ) return (ulong) (-1);
		}
	}

	return (ulong) (-1);
}

static void byteswap(register uchar *p,register ulong n)
{
	register ulong	a,b;

	do
	{
		a=p[0]; b=p[1];
		p[0]=b; p[1]=a;
		p+=2;
		n-=2;
	}
	while (n);
}

int main(int argc,char **argv)
{
	uchar		buffer[buffersize];
	register ulong	dsize;
	register long	i,j;

	if ( argc>1 )
	{
		fprintf(stderr,"usage: %s < <x.wav> > <x.au>\n",argv[0]);
		exit(0);
	}
	if ( convert_header()<0 )
	{
		fprintf(stderr,"invalid wave file\n");
		exit(1);
	}

	dsize=find_data_chunk();

	if ( !(~dsize) || (!dsize) )
	{
		fprintf(stderr,"no data chunk found\n");
		exit(1);
	}

	do
	{
		if ( dsize&1 )
		{
			fprintf(stderr,"odd data chunk size: %li\n",dsize);
			exit(1);
		}
		do
		{
			i=dsize;
			if ( i>buffersize ) i=buffersize;
			j=fread(buffer,1,i,stdin);
			if ( j<i )
			{
				buffer[j]=0;
				j+=(j&1);
				byteswap(buffer,j);
				if ( j>0 ) fwrite(buffer,1,j,stdout);
				fprintf(stderr,"Unexpected end of file\n");
				exit(0);
			}
			byteswap(buffer,i);
			fwrite(buffer,1,j,stdout);
			dsize-=i;
		}
		while ( dsize );

		dsize=find_data_chunk();

	}
	while ( (~dsize) && dsize );

	return 0;
}
