//---------------------------------------------------------------------------\ 
//
//               (C) copyright Fraunhofer - IIS (2000)
//                        All Rights Reserved
//
//   filename: PnsConformanceMain.h
//             
//   author  : Martin Weishart martin.weishart@iis.fhg.de
//   date    : 2000-07-05
//   contents/description: provides a class containing methods 
//                         to calculate PNS conformance criteria
//
//--------------------------------------------------------------------------/


#define SFB_1 49
#define SFB_2 14
#define FRAME_SIZE_LONG 2048
#define FRAME_SIZE_SHORT 256
#define BLOCK_LENGTH 64

typedef struct {
  float meanRatio;
  float standardDeviationRatio;
} critical_ratio;


class CPnsConformance{

public:

  CPnsConformance();

  int CalculateTemporalConformance(char * Pcmfile,
                                   char * refPcmfile,
                                   unsigned int sfbSize,
                                   unsigned int frameSize);

  int CalulateSpectralConformance(char * Pcmfile,
                                  char * refPcmfile,
                                  unsigned int sfbSize,
                                  unsigned int frameSize);

private:

  critical_ratio criticalRatio ;

  static int   sfboff_1[];
  static float hannwin_1[];

  static int   sfboff_2[];
  static float hannwin_2[];

  unsigned int wrongRatioCount ;

  
  float * m_ConformanceEnergy3_ref ;
  float * m_ConformanceEnergy3_est ;
  unsigned int m_Conformance3Counter ;

  int calculateTemporalConformance();
  int calculateSquaredEnergy( unsigned char * buffer, 
                              unsigned char * refBuffer,
                              unsigned int frameSize);
  void prepareTemporalConformance( unsigned int fileSize );

  critical_ratio  calculateCriticalRatio(float est_sfb_sum[], 
                                         float ref_sfb_sum[], 
                                         unsigned int lastFrame);

  unsigned int getWrongRatioNumber(){ return wrongRatioCount; }
  
  int calculateSpectralConformance(float              ** est_sfb_sum, 
                                   float              ** ref_sfb_sum, 
                                   unsigned int       lastFrame,
                                   unsigned int       sfbSize);

  int accumulateFrameEnergiesInSfb(unsigned char * buffer, 
                                   unsigned char * refBuffer,
                                   float est_sfb_sum[SFB_1-1], 
                                   float ref_sfb_sum[SFB_1-1],
                                   unsigned int sfbSize,
                                   unsigned int frameSize); 

  int calculateEnergyOfFrame(unsigned char * buffer, 
                             unsigned char * refBuffer,
                             float         * est_abs_time_value,
                             float         * ref_abs_time_value,
                             unsigned int    frameSize);
 
  float calculateStandardDeviation(float vector[],
                                   unsigned int maxValue) ;
  
  float calculateMeanValue(float vector[],
                           unsigned int maxValue);

  int loadPCMFile(char * refPcmfile, unsigned char * buffer, 
                  unsigned int pos, unsigned int numBytes);

  int loadWAVFile(char * refWavfile, unsigned char * buffer,
                  long int pos, unsigned int numBytes);
 
  int getFileSize(char * ofile);

};
