/*
This software module was originally developed by AT&T
and editted by 
Yasuhiro Toguri (Sony)
in the course of development of MPEG-4. 
This software module is an implementation of a part of one or 
more MPEG-4 tools as specified by MPEG-4. 
ISO/IEC gives users of MPEG-4 free license to this
software module or modifications thereof for use in hardware 
or software products claiming conformance to MPEG-4.
Those intending to use this software module in hardware or software
products are advised that its use may infringe existing patents.
The original developer of this software module and his/her company,
the subsequent editors and their companies, and ISO/IEC have no
liability for use of this software module or modifications thereof
in an implementation.
Copyright is not released for non MPEG-4 conforming
products. AT&T. retains full right to use the code for its own
purpose, assign or donate the code to a third party and to
inhibit third parties from using the code for non 
MPEG-4 conforming products.
This copyright notice must be included in all copies or
derivative works. Copyright (c) 1999.
*/

/* AAC ADIF file routines */

#define DEBUG				fprintf
#define DEBUG_BUFFER1			nofprintf
#define DEBUG_BUFFER2			nofprintf

#include <stdio.h>
#include <string.h>
#include "MP4Movies.h"
#include "adif.h"
#include "all.h"

/* for AudioSpecificInfo */
#define	lenAudioObjectType		5
#define lenSamplingFrequencyIndex	4
#define lenSamplingFrequency		24
#define lenChannelConfiguration		4
#define lenFrameLengthFlag	    	1
#define	lenDependsOnCoreCoder		1
#define lenExtensionFlag		1
#define	lenASIHeader			(5+4+0+4+1+1+1)

/* AAC parameters */
#define	bufferSizePerAudioChn		6144
#define aacFrameLength			1024	    /* AAC Main, LC, SSR only! */

#define MAX_DSI_LEN			1024	    /* bytes */
#ifdef	BIGDECODER
#define MAX_FRAME_LEN			(64*1024)   /* bytes */
#else
#define MAX_FRAME_LEN			8192	    /* bytes */
#endif
FILE	*fp;
u8	block_buffer[MAX_FRAME_LEN];
u8	*bp = block_buffer;
u32	bword, bbit; /* for buffer i/o */
long	buffer_fullness;

void	init_dec(void);
void	dec(void);

u32 sampling_rate[] = 
{
	96000, 
	88200, 
	64000, 
	48000, 
	44100, 
	32000, 
	24000, 
	22050, 
	16000, 
	12000, 
	11025, 
	8000,  
	0,
	0,
	0,
	0
};

void nofprintf() {;}

void myexit(char *s)
{
    if (s)
        fprintf(stderr,"\n%s\n", s);
    exit(s ? 1 : 0);
}

/* get byte and also copy it into block buffer */
int getbyte(void)
{
    u8 c;
    int i;
    if ((i=fread(&c, sizeof(char), 1, fp)) != 1) {
	DEBUG(stderr, "end of file\n");
	c = -1;
    }
    *bp++ = c;
    return c; 

}

long getshort(void)
{
    int c1, c2;

    c1 = getbyte();
    c2 = getbyte();
    if(c2 < 0)
        c2 = 0;
    if(c1 < 0)
        c1 = 0;
    return (c1<<8) | c2;
}

long getlongbits(int n)
{
    long v1, v2;
    if (n > 16) {
	v1 = getbits(8);
	v2 = getbits(n-8);
	v2 = (v1<<(n-8)) | v2;
    }
    else {
	v2 = getbits(n);
    }
    return v2;
}

/* writes n bits from data (msb first) to **pd if pd is not NULL
 * otherwise does nothing
 */
static void putbits( char **pd, u32 data, u32 n)
{
        u32 dbit;
        u32 word, bit;
	char *d;

	if (pd == NULL) {
		d = NULL;
	}
	else {
		d = *pd;
	}
        dbit = 1<<(n-1);
        word = bword;
        bit = bbit;
        while(dbit != 0) {
                if(data & dbit) {
                        word |= bit;
                }
                dbit >>= 1;
                bit >>= 1;
                if(bit == 0) {
			if (d)
				*d++ = word;
                        word = 0;
                        bit = 0x80;
                }
        }
        bword = word;
        bbit = bit;
	if (d)
	    *pd = d;
}

static u32 num_audio_chn( EleList *p, int cce )
{
	u32 n;
	int i, j;
	
	n = 0;
	for (i=0; i<p->num_ele; i++) {
		j = p->ele_is_cpe[i];
		if (!cce)
			n += (j) ? 2 : 1;
		else 
			n += (j) ? 1 : 0;	/* ind sw cce */
	}
	return n;
}

static void put_ele_list( char **pb, EleList *p, int flag )
{
	int i;
	for (i=0; i<p->num_ele; i++) {
		if (flag)
			putbits(pb, p->ele_is_cpe[i], LEN_ELE_IS_CPE);
		/* tag must agree with that in bitstream! */
		putbits(pb, p->ele_tag[i], LEN_TAG);
	}
}

static void put_prog_config( char **pb, int tag, ProgConfig *p )
{
	int i, j;
	
	putbits(pb, tag, LEN_TAG);
	putbits(pb, p->profile, LEN_PROFILE);
	putbits(pb, p->sampling_rate_idx, LEN_SAMP_IDX);
	putbits(pb, p->front.num_ele, LEN_NUM_ELE);
	putbits(pb, p->side.num_ele, LEN_NUM_ELE);
	putbits(pb, p->back.num_ele, LEN_NUM_ELE);
	putbits(pb, p->lfe.num_ele, LEN_NUM_LFE);
	putbits(pb, p->data.num_ele, LEN_NUM_DAT);
	putbits(pb, p->coupling.num_ele, LEN_NUM_ELE);
	putbits(pb, 0, LEN_MIX_PRES);   /* no mono mixdown */
	putbits(pb, 0, LEN_MIX_PRES);   /* no stereo mixdown */
	putbits(pb, 0, LEN_MIX_PRES);   /* no matrix mixdown */
	
	/* front, side, back sce or cpe */
	put_ele_list(pb, &p->front, 1);
	put_ele_list(pb, &p->side, 1);
	put_ele_list(pb, &p->back, 1);
	put_ele_list(pb, &p->lfe, 0);
	put_ele_list(pb, &p->data, 0);
	put_ele_list(pb, &p->coupling, 1);

	/* byte align */
	while (bbit != 0x80)
		putbits( pb, 0, 1);
	
	/* comment */
	j = strlen(p->comments);
	putbits(pb, j, LEN_COMMENT_BYTES);
	for (i=0; i<j; i++)
		putbits(pb, p->comments[i], 8);
}

/* AAC raw data block is always an integral number of bytes
 * and is byte aligned
 * returns byte count
 */
static u32 get_raw_data_block( char **p )
{
	int i, n;
	char *tp;

	if ((nbits & 3) != 0) {
		fprintf(stderr, "byte alignment error\n");
		return 0;
	}
	/* shift buffer, retaining bytes that are still in cword */
	n = (nbits>>3); /* unused bytes */
	tp = bp - n;
	bp = block_buffer;
	for (i=0; i<n; i++)
		*bp++ = *tp++;
	
	/* parse this block */
	dec();
	byte_align();

	/* set pointer */
	*p = block_buffer;
	
	/* return byte count */	
	n = (nbits>>3); /* unused bytes */
	return (bp - block_buffer) - n;
}

/* open file */
MP4Err openAdifFile( char *filename )
{
	if ((fp = fopen( filename, "r" )) == NULL) {
		DEBUG(stderr, "open\n");
		return MP4FileNotFoundErr;
	}
	return MP4NoErr;
}

MP4Err getDecoderSpecificInfo( u32 prog, MP4Handle decoderSpecificInfoH )
{
	u32 i, n;
	char *dsi, *plen;
	ProgConfig *pc;
	MP4Err err;
	
	pc = &prog_config;
	current_program = prog; /* set desired program */
	
	/* allocate default amount of memory to decoderSpecificInfo */
	err = MP4SetHandleSize( decoderSpecificInfoH, MAX_DSI_LEN );
	if (err)
		return err;
	
	/* initialize decoder */
	init_dec();
	
	/* read ADIF header */
	if (!get_adif_header()) {
		DEBUG(stderr, "adif_header\n");
		return MP4BadParamErr;
	}

	if ((nbits & 7) != 0) {
		DEBUG(stderr, "PCE read byte alignment error\n");
		return MP4BadParamErr;
	}
	
	/* write DecoderSpecificInfo */	
	bbit = 0x80; /* byte align bit stream */
	bword = 0;
	dsi = (char *) *decoderSpecificInfoH;

	/* tag is DecSpecificInfoTag, 14496-1, Sec 8.2.2.2, Table 1*/
	putbits( &dsi, 0x05, 8 );
	/* dummy sizeOfInstance, 14496-1, Sec 12.3.3 */
	plen = dsi;
	putbits( &dsi, 0, 8 );
	/* Audio Specific Configuration */
#ifdef USE_LTP
	putbits( &dsi, LTP_Profile+1, lenAudioObjectType ); /* For AAC LTP */
#else
	putbits( &dsi, pc->profile+1, lenAudioObjectType );/* For Main LC SSR */
#endif
	putbits( &dsi, pc->sampling_rate_idx, lenSamplingFrequencyIndex );
	putbits( &dsi, 0, lenChannelConfiguration );
	/* GA Specific Configurtaion */
	putbits( &dsi, 0, lenFrameLengthFlag );
	putbits( &dsi, 0, lenDependsOnCoreCoder );
	putbits( &dsi, 0, lenExtensionFlag );
	/* pce */
	put_prog_config( &dsi, 0, pc);
	if ((bbit != 0x80) != 0) {
		DEBUG(stderr, "PCE write byte alignment error\n");
		return MP4BadParamErr;
	}
	
	/* set exact handle size */
	n = dsi - *decoderSpecificInfoH;
	err = MP4SetHandleSize( decoderSpecificInfoH, n );
	if (err)
		return err;
		
	/* set length of decoderSpecificInfoH part in header,
	 * encoded as sizeOfInstance
	 */
	if (n < 128) {
		*plen = n-2;
	}
	else {
		/* encoding of size >= 128 bytes currently not supported */
		DEBUG(stderr, "decoderSpecificInfo too large\n");
		return MP4BadParamErr;
	}

	DEBUG_BUFFER1(stderr, "decoderSpecificInfo length %d\n", n);
	dsi = *decoderSpecificInfoH;
	for (i=0; i<n; i++)
		DEBUG_BUFFER2(stderr, "%3i 0x%02x\n", i, *dsi++); 

	return MP4NoErr;
}

MP4Err	getAudio_profileAndLevel( u8 *audio_profileAndLevel )
{
	ProgConfig *p;
	
	p = &prog_config;
	*audio_profileAndLevel = (u8) (p->profile);
	if (*audio_profileAndLevel <= SSR_Profile) {
		return MP4NoErr;
		   
	}
	else {
		DEBUG(stderr, "profile\n");
		return MP4BadParamErr;
	}
}

MP4Err	getSamplingRate( u32 *samplingRate )
{
	ProgConfig *p;
	
	p = &prog_config;
	*samplingRate = sampling_rate[p->sampling_rate_idx];
	if (*samplingRate != 0) {
		return MP4NoErr;
	}
	else {
		DEBUG(stderr, "samplingRate\n");
		return MP4BadParamErr;
	}
}

MP4Err	getDecoderBufferSize( u32 *decoderBufferSize )
{
	u32 nch;
	ProgConfig *p;
	
	p = &prog_config;
	nch  = num_audio_chn(&p->front, 0);
	nch += num_audio_chn(&p->side, 0);
	nch += num_audio_chn(&p->back, 0);
	nch += num_audio_chn(&p->coupling, 1);

	*decoderBufferSize = nch * bufferSizePerAudioChn;
	if ( (0 < nch) && (nch <= 48)) {
		return MP4NoErr;
	}
	else {
		DEBUG(stderr, "decoderBufferSize: %d channels\n", nch);
		return MP4BadParamErr;
	}
}

MP4Err	getBitRates( u32 *maxBitrate, u32 *avgBitrate )
{
	ADIF_Header *p;
	
	p = &adif_header;
	*maxBitrate = (u32) p->bitrate;
	*avgBitrate = (u32) p->bitrate;
	if ( avgBitrate > 0 ) {
		return MP4NoErr;
	}
	else {
		DEBUG(stderr, "avgBitrate\n");
		return MP4BadParamErr;
	}
}

MP4Err	getSampleDuration( u32 *sampleDuration )
{
	*sampleDuration = aacFrameLength; 
	return MP4NoErr;
}

MP4Err getRawDataBlock( MP4Handle sampleDataH )
{
	int i, c;
	char *p1, *p2, *rdb;
	u32 frameLength;
	MP4Err err;

	err = MP4NoErr;
	
	/* check for EOF */
	if ((c = getc(fp)) == EOF) {
	    DEBUG(stderr, "End of bitstream\n");
	    return MP4EOF;
	}
	ungetc(c, fp);
	    
	frameLength = get_raw_data_block(&rdb);
	if ( (frameLength == 0) || (frameLength > MAX_FRAME_LEN) ) {
		DEBUG(stderr, "frameLength error: %d\n", frameLength);
		return MP4BadParamErr;
	}
		
	err = MP4SetHandleSize( sampleDataH, frameLength ); if (err) goto bail;
	p1 = *sampleDataH;
	p2 = rdb;
	DEBUG_BUFFER1(stderr, "Block length %d\n", frameLength);
	for (i=0; i<frameLength; i++) {
		DEBUG_BUFFER2(stderr, "%3i 0x%02x\n", i, *p2); 
		*p1++ = *p2++;
	}
bail:
	return MP4NoErr;
}
