/*
This software module was originally developed by Apple Computer, Inc.
in the course of development of MPEG-4. 
This software module is an implementation of a part of one or 
more MPEG-4 tools as specified by MPEG-4. 
ISO/IEC gives users of MPEG-4 free license to this
software module or modifications thereof for use in hardware 
or software products claiming conformance to MPEG-4.
Those intending to use this software module in hardware or software
products are advised that its use may infringe existing patents.
The original developer of this software module and his/her company,
the subsequent editors and their companies, and ISO/IEC have no
liability for use of this software module or modifications thereof
in an implementation.
Copyright is not released for non MPEG-4 conforming
products. Apple Computer, Inc. retains full right to use the code for its own
purpose, assign or donate the code to a third party and to
inhibit third parties from using the code for non 
MPEG-4 conforming products.
This copyright notice must be included in all copies or
derivative works. Copyright (c) 1999.
*/
#include <stdio.h>
#include <stdlib.h>
#include "MP4Movies.h"
#include "adif.h"

MP4Err createMyMovie( u32 prog, char *inFile, char *outFile );
MP4Err addMySamples( MP4Track trak, MP4Media media, MP4Handle sampleH );

int main( int argc, char **argv )
{
	u32 i, prog;
	char *inFile, *outFile;
	MP4Err err;

	/* set defaults */
	prog = 0;

	if (argc < 3) {
		fprintf( stderr, "usage: %s [-p<n>] adif_file mp4_file\n", argv[0]);
		return MP4BadParamErr;
	}
	i = 1;
	if (argv[i][0] == '-') {
		if (argv[i][1] == 'p') {
			prog = atoi(&argv[i][2]);
			fprintf(stderr, "Program %d selected \n", prog);
		}
		i++;
	}
	inFile = argv[i++];
	outFile = argv[i++];
	err = createMyMovie( prog, inFile, outFile );
	fprintf( stderr, "returns %d\n", err );
	return err;
}

MP4Err createMyMovie( u32 prog, char *inFile, char *outFile )
{
	MP4Err err;
	MP4Movie moov;
	MP4Track trak;
	MP4Media media;
	MP4Handle decoderSpecificInfoH;
	u8 OD_profileAndLevel;
	u8 scene_profileAndLevel;
	u8 audio_profileAndLevel;
	u8 visual_profileAndLevel;
	u8 graphics_profileAndLevel;
	u32 initialObjectDescriptorID;
	u32 timeScale;
	u64 mediaDuration;

	err = MP4NoErr;
	err = openAdifFile( inFile ); if (err) goto bail;
	err = MP4NewHandle( 0, &decoderSpecificInfoH ); if (err) goto bail;
	err = getDecoderSpecificInfo( prog, decoderSpecificInfoH ); if (err) goto bail;
	err = getAudio_profileAndLevel( &audio_profileAndLevel ); if (err) goto bail;
	err = getSamplingRate( &timeScale ); if (err) goto bail;

	initialObjectDescriptorID = 1;
	OD_profileAndLevel = 0xff; /* none required */
	scene_profileAndLevel = 0xff; /* none required */
	visual_profileAndLevel = 0xff; /* none required */
	graphics_profileAndLevel = 0xff; /* none required */
	err = MP4NewMovie( &moov,
		initialObjectDescriptorID, 
		OD_profileAndLevel,
		scene_profileAndLevel,
		audio_profileAndLevel,
		visual_profileAndLevel,
		graphics_profileAndLevel ); if (err) goto bail;
	err = MP4NewMovieTrack( moov, MP4NewTrackIsAudio, &trak ); if (err) goto bail;
	err = MP4NewTrackMedia( trak, &media, MP4AudioHandlerType, timeScale, NULL ); if (err) goto bail;
	err = MP4BeginMediaEdits( media ); if (err) goto bail;
	err = addMySamples( trak, media, decoderSpecificInfoH ); if (err) goto bail;
	err = MP4EndMediaEdits( media ); if (err) goto bail;
	err = MP4GetMediaDuration( media, &mediaDuration ); if (err) goto bail;
	err = MP4InsertMediaIntoTrack( trak, 0, 0, mediaDuration, 1 ); if (err) goto bail;
        err = MP4SetTrackEnabled( trak, 1); if (err) goto bail;
	err = MP4WriteMovieToFile( moov, outFile ); if (err) goto bail;
	err = MP4DisposeMovie( moov ); if (err) goto bail;
bail:
	return err;
}

MP4Err addMySamples( MP4Track trak, MP4Media media, MP4Handle decoderSpecificInfoH )
{
	MP4Err err;
	MP4Handle sampleEntryH;
	MP4Handle sampleDurationH;
	MP4Handle sampleSizeH;
	MP4Handle sampleDataH;
 	u32 objectTypeIndication;
	u32 streamType;
	u32 decoderBufferSize;
	u32 maxBitrate;
	u32 avgBitrate;

	err = MP4SetMediaLanguage( media, "und" ); if (err) goto bail;
	objectTypeIndication = 0x40; /* mpeg-4 audio */
	streamType = 0x05; /* audio stream */
	err = getBitRates( &maxBitrate, &avgBitrate ); if (err) goto bail;
	err = getDecoderBufferSize( &decoderBufferSize ); if (err) goto bail;
	err = MP4NewHandle( 0, &sampleEntryH ); if (err) goto bail;
	err = MP4NewSampleDescription( trak, 
		sampleEntryH,
		1,
		objectTypeIndication,
		streamType,
		decoderBufferSize,
		maxBitrate,
		avgBitrate,
		decoderSpecificInfoH ); if (err) goto bail;

	err = MP4NewHandle( sizeof(u32), &sampleDurationH ); if (err) goto bail;
	err = getSampleDuration( (u32*) *sampleDurationH ); if (err) goto bail;
	err = MP4NewHandle( sizeof(u32), &sampleSizeH ); if (err) goto bail;
	err = MP4NewHandle( sizeof(u32), &sampleDataH ); if (err) goto bail;
	for ( ;; ) {
		err = getRawDataBlock( sampleDataH );
		if ( err ) {
			if ( err == MP4EOF )
				err = 0;
			break;
		}
		err = MP4GetHandleSize( sampleDataH, (u32*) *sampleSizeH ); if (err) goto bail;
		err = MP4AddMediaSamples( media, 
			sampleDataH,
			1,
			sampleDurationH,
			sampleSizeH, 
			sampleEntryH, 
			NULL, 
			NULL ); if (err) goto bail;
		if ( sampleEntryH ) {
			err = MP4DisposeHandle( sampleEntryH ); if (err) goto bail;
			sampleEntryH = NULL;
		}	
	}	
bail:
	return err;
}
