/************************* MPEG-2 NBC Audio Decoder **************************
 *                                                                           *
"This software module was originally developed by 
AT&T, Dolby Laboratories, Fraunhofer Gesellschaft IIS 
and edited by
Yoshiaki Oikawa (Sony Corporation),
Mitsuyuki Hatanaka (Sony Corporation)
Yasuhiro Toguri (Sony Corporation)
in the course of development of the MPEG-2 NBC/MPEG-4 Audio standard ISO/IEC 13818-7, 
14496-1,2 and 3. This software module is an implementation of a part of one or more 
MPEG-2 NBC/MPEG-4 Audio tools as specified by the MPEG-2 NBC/MPEG-4 
Audio standard. ISO/IEC  gives users of the MPEG-2 NBC/MPEG-4 Audio 
standards free license to this software module or modifications thereof for use in 
hardware or software products claiming conformance to the MPEG-2 NBC/MPEG-4
Audio  standards. Those intending to use this software module in hardware or 
software products are advised that this use may infringe existing patents. 
The original developer of this software module and his/her company, the subsequent 
editors and their companies, and ISO/IEC have no liability for use of this software 
module or modifications thereof in an implementation. Copyright is not released for 
non MPEG-2 NBC/MPEG-4 Audio conforming products.The original developer
retains full right to use the code for his/her  own purpose, assign or donate the 
code to a third party and to inhibit third party from using the code for non 
MPEG-2 NBC/MPEG-4 Audio conforming products. This copyright notice must
be included in all copies or derivative works." 
Copyright(c)1996.
 *                                                                           *
 ****************************************************************************/

static char *MP4AacBp; 

/* for AudioSpecificInfo */
#define	lenAudioObjectType		5
#define lenSamplingFrequencyIndex	4
#define lenSamplingFrequency		24
#define lenChannelConfiguration		4
#define lenFrameLengthFlag	    	1
#define	lenDependsOnCoreCoder		1
#define lenExtensionFlag		1
#define	lenASIHeader			(5+4+0+4+1+1+1)

#include "all.h"
#include <fcntl.h>
#include <sys/stat.h>

#ifdef WIN32
#include <io.h>
#else
#include <unistd.h>
#endif

#include "aiff_support.h"

#ifdef	DOLBY_MDCT
#include "block.h"
#include "dolby_def.h"
#endif
#ifdef MPEG4V1
#include "nok_lt_prediction.h"
#endif



#define PATH_LEN    128

static  int     outfd[Chans];
static  char    output_path[PATH_LEN];

static  int            useAIFF         = 0;
static  unsigned short AIFF_channels   = 1;
static  unsigned short AIFF_wordsize   = 16;
static  unsigned long  AIFF_frames     = 0;
static  unsigned long  AIFF_sampleRate = 48000;
static  MC_Info*       AIFF_MC_Info    = 0;


static void
write_fext(Ch_Info *cip, char c, int n)
{
	cip->fext = (char *)mal1(5);
	sprintf(cip->fext, "_%c%02d", c, n);
}


int
open_output_files(MC_Info *mip)
{
    int i, j;
    char file[PATH_LEN];

    if (useAIFF){
	AIFF_MC_Info = mip;
	AIFF_sampleRate = samp_rate_info[mip->sampling_rate_idx].samp_rate; 
    }	
    j=0;
    /* if FCenter == 1 (center speaker present)
     *	position    index   extension
     *	left	    1	    01
     *	center	    0	    00
     *	right	    2	    02
     * if FCenter == 0 (center speaker absent)
     *	position    index   extension
     *	left	    0	    01
     *	right	    1	    02
      */
    for (i=((FCenter==1) ? 0 : 1); i<FChans; i++) {
	write_fext(&mip->ch_info[j], 'f', i);
	j++;
    }
    for (i=0; i<SChans; i++) {
	write_fext(&mip->ch_info[j], 's', i);
	j++;
    }
    /* if BFCenter == 1 (center speaker present)
     *	position    index   extension
     *	left	    3	    00
     *	center	    5	    02
     *	right	    4	    01
     * if BCenter == 0 (center speaker absent)
     *	position    index   extension
     *	left	    3	    00
     *	right	    4	    01
      */
    for (i=0; i<BChans; i++) {
	write_fext(&mip->ch_info[j], 'b', i);
	j++;
    }
    for (i=0; i<LChans; i++) {
	write_fext(&mip->ch_info[j], 'l', i);
	j++;
    }
    
    for (i=0; i<Chans; i++) {
	if (!(mip->ch_info[i].present))
	    continue;
       
	strcpy(file, output_path);
	strcat(file, mip->ch_info[i].fext);
	if (!useAIFF)
	    strcat(file, ".pcm");
	else
	    strcat(file, ".aif");

	if (debug['o'])
	    PRINT(SE, "opening file %s at position %d\n", file, i);
	    
#ifdef WIN32
	outfd[i] = _open (file, _O_CREAT | _O_TRUNC | _O_BINARY | _O_WRONLY, _S_IWRITE);
#else
	outfd[i] = creat(file, 0666);
#endif

	if(outfd[i] < 0) {
	    PRINT(SE, "cant create %s\n", file);
	    myexit("create");
	}
	if ( useAIFF ) {
	    /* write dummy header and position for audio data */
	    if ( WriteAIFFHeader(outfd[i], AIFF_channels, AIFF_frames, AIFF_wordsize, AIFF_sampleRate) != 0 ) {
		PRINT( SE, "cannot write AIFF header for %s\n", file );
		myexit( "aiff-header" );
	    }
	}
    }
    return(1);
}

void
fmtchan(char *p, float *data, int stride)
{
    int i, c;
    float s;

    for(i=0; i<LN2; i++) {
	s = *data++;
	if(s < 0) {
	    s -= .5;
	    if(s < -0x7fff)
		s = (float) -0x7fff;
	} else {
	    s += .5;
	    if(s > 0x7fff)
		s = (float) 0x7fff;
	}
	c = (int) s;

	p[0] = (c >> 8) & 0xff;
	p[1] = c & 0xff;

	p += stride;
    }
}

void
writeout(Float **data, MC_Info *mip)
{
   int i, stride, size;
   char obuf[LN2*2];
   static int opened = 0;
   
   if (!opened) {
      open_output_files(mip);
      opened=1;
   }
   
   stride=2;
   size = sizeof(obuf);
   for (i=0; i<Chans; i++) {
      if (!(mip->ch_info[i].present))
         continue;
      
      fmtchan(obuf, data[i], stride);
      if(write(outfd[i], obuf, size) != size) {
         PRINT(SE, "write error\n");
         myexit("write");
      }
   }
   if ( useAIFF )
      AIFF_frames += LN2;
}

void
myexit(char *s)
{
   if ( useAIFF )
      closefiles( AIFF_MC_Info );
    if (debug['v'])
	PRINT(SE,"%lx %d\n", cword, nbits);
    if (s)
	PRINT(SE,"\n%s\n", s);
    exit(s ? 1 : 0);
}

void
closefiles( MC_Info *mip )
{
   int i;
   if ( mip && useAIFF )
   {
      for ( i = 0; i < Chans; i++ )
      {
         if ( !(mip->ch_info[i].present) )
            continue;
         WriteAIFFHeader( outfd[i], AIFF_channels, AIFF_frames, AIFF_wordsize, AIFF_sampleRate );
         close( outfd[i] );
      }
   }
}

#ifdef  SSR
void gain_control(float  *freqSigCh,
				  Wnd_Shape	*wnd_shape,
				  int window_sequenceCh,
				  int ch,
				  float   *timeSigCh
				  );
#endif

/* prediction */
static	PRED_STATUS	*sp_status[Chans];
#ifdef MPEG4V1
/* long term prediction */
static  NOK_LT_PRED_STATUS *nok_lt_status[Chans];
#endif


void
restartio(char *p)
{
    cword = 0;
    nbits = 0;
    MP4AacBp = p;
}

/* get byte from buffer */
int getbyte(void)
{
    int c; 
    c = (unsigned char) *MP4AacBp++;
    return c;
}

long getshort(void)
{
    int c1, c2;

    c1 = getbyte();
    c2 = getbyte();
    if(c2 < 0)
        c2 = 0;
    if(c1 < 0)
        c1 = 0;
    return (c1<<8) | c2;
}

long getlongbits(int n)
{
    long v1, v2;
    if (n > 16) {
	v1 = getbits(8);
	v2 = getbits(n-8);
	v2 = (v1<<(n-8)) | v2;
    }
    else {
	v2 = getbits(n);
    }
    return v2;
}

#if 0
void print_ele(char *s, EleList *p);	/* located in library */
#else
void
print_ele(char *s, EleList *p) /* not located in library */
{
    int i;
    PRINT(SE, "%s\n", s);    
    if (p->num_ele)
	PRINT(SE, "%6s %6s %6s\n", "idx", "cpe", "tag");    
    for (i=0; i<p->num_ele; i++)
	PRINT(SE, "%6d %6d %6d\n", i, p->ele_is_cpe[i], p->ele_tag[i]);    
}
#endif

void
print_header(ProgConfig *p)
{
	PRINT(SE, "\n");
	PRINT(SE, "%-25s %d\n", "p->profile", p->profile);
	PRINT(SE, "%-25s %d\n", "p->sampling_rate_idx", p->sampling_rate_idx);
	
	print_ele("p->front", &p->front);
	print_ele("p->side", &p->side);
	print_ele("p->back", &p->back);
	print_ele("p->lfe", &p->lfe);
	print_ele("p->coupling", &p->coupling);
	
	PRINT(SE, "%-25s %d\n", "p->data.num_ele", p->data.num_ele);
	PRINT(SE, "%-25s %d\n", "p->mono_mix.present", p->mono_mix.present);
	PRINT(SE, "%-25s %d\n", "p->stereo_mix.present", p->stereo_mix.present);
	PRINT(SE, "%-25s %d\n", "p->matrix_mix.present", p->matrix_mix.present);
	PRINT(SE, "Comment: %s\n", prog_config.comments);
}

void
init_config(char *outfile)
{
	int objtype;
	/* Audio Specific Configuration */
        objtype = getbits(lenAudioObjectType); /* from AudioSpecificConfig */
#ifdef MPEG4V1
        printf("Object type:%d\n",objtype);
        if(objtype > LTP_Profile + 1 || objtype < Main_Profile + 1 ){
           printf("Warning!!! Invalid Object Type in AudioSpecificConfig\n");
        }
#endif
	getbits(lenSamplingFrequencyIndex);
	getbits(lenChannelConfiguration);
	
	/* GA Specific Configurtaion */
	getbits(lenFrameLengthFlag);
	getbits(lenDependsOnCoreCoder);
	getbits(lenExtensionFlag);

	/* pce */
	get_prog_config(&prog_config);
 	mc_info.profile = prog_config.profile; 
        printf("ProfileID in PCE:%d\n",mc_info.profile);
        if(mc_info.profile > SSR_Profile || mc_info.profile < Main_Profile){
           printf("Warning!!! Invalid Profile ID in PCE\n");
        }
	mc_info.profile = objtype -1 ; /* Use object type for AAC Profile */
#ifdef USE_LTP
	mc_info.profile = LC_Profile; /* provisionally */
#endif
	if (outfile == NULL)
	    print_header(&prog_config);
}

void
init(void)
{
#ifndef	DOLBY_MDCT
    imdctinit();
#endif
#if (CChans > 0)
    init_cc();
#endif
#ifdef	DRC
    init_drc();
#endif
    huffbookinit();
    predinit();

    winmap[0] = win_seq_info[ONLY_LONG_WINDOW];
    winmap[1] = win_seq_info[ONLY_LONG_WINDOW];
    winmap[2] = win_seq_info[EIGHT_SHORT_WINDOW];
    winmap[3] = win_seq_info[ONLY_LONG_WINDOW];      
}

void
predinit(void)
{
    int i, ch;
    for (ch = 0; ch < Chans; ch++) {
	for (i = 0; i < LN2; i++) {
	    init_pred_stat(&sp_status[ch][i],
	        PRED_ORDER,PRED_ALPHA,PRED_A,PRED_B);
	}
    }
#ifdef MPEG4V1 
    for (ch = 0; ch < Chans; ch++) {
      nok_init_lt_pred(nok_lt_status[ch]);
    }
#endif
}

int
getdata(int *tag, int *dt_cnt, byte *data_bytes)
{
    int i, align_flag, cnt;

    *tag = getbits(LEN_TAG);
    align_flag = getbits(LEN_D_ALIGN);
    if ((cnt = getbits(LEN_D_CNT)) == (1<<LEN_D_CNT)-1)
	cnt +=  getbits(LEN_D_ESC);
    *dt_cnt = cnt;
    if (debug['x'])
	PRINT(SE, "data element %d has %d bytes\n", *tag, cnt);
    if (align_flag)
	byte_align();

    for (i=0; i<cnt; i++) {
	data_bytes[i] = getbits(LEN_BYTE);
	if (debug['X'])
	    PRINT(SE, "%6d %2x\n", i, data_bytes[i]);
    }
    
    return 0;
}

int
extension_payload(int cnt, byte *data)
{
    int type, i;
    
    /* fill bytes should not emulate any EX types below! */
    type = getbits(LEN_EX_TYPE);
    if (debug['x'])
	PRINT(SE, "fill element extension %d\n", type);
    
    switch(type) {
#ifdef	DRC
	case EX_DRC:
	    return dynamic_range_info();
#endif
	case EX_FILL_DATA:
	    getbits(LEN_NIBBLE);
	    for (i=0; i<cnt-1; i++)
		data[i] = getbits(LEN_BYTE);
	    return cnt;
	default: 
	    getbits(LEN_NIBBLE);
	    for (i=0; i<cnt-1; i++)
		getbits(LEN_BYTE);
	    return cnt;
    }
}

void
getfill(byte *data)
{
    int cnt;
    if ((cnt = getbits(LEN_F_CNT)) == (1<<LEN_F_CNT)-1)
	cnt +=  getbits(LEN_F_ESC) - 1;
    if (debug['x'])
	PRINT(SE, "fill element has %d bytes\n", cnt);
    while (cnt > 0)
	cnt -= extension_payload(cnt, data);
}

/* all storage is global */
Float *coef[Chans], *data[Chans], *state[Chans];
byte hasmask[Winds], *mask[Winds], *group[Chans],
    wnd[Chans], max_sfb[Chans],
    *cb_map[Chans], d_bytes[MAX_DBYTES];
Wnd_Shape wnd_shape[Chans];
short *factors[Chans];
int *lpflag[Chans], *prstflag[Chans];
TNS_frame_info *tns[Chans];
#ifndef MPEG4V1
    static int last_rstgrp_num[Chans] = {0};
#endif
#if (CChans > 0)
Float *cc_coef[CChans], *cc_gain[CChans][Chans];
byte cc_wnd[CChans] = {0, 0, 0, 0};
Wnd_Shape cc_wnd_shape[CChans];
#ifdef  SSR
Wnd_Shape wnd_shape_SSR[Chans];
#endif
#if (ICChans > 0)
Float *cc_state[ICChans];
#endif
#endif

void
init_dec(char *p, char *outfile)
{
    int i, j;

#ifdef   ADTS_SUPPORT
    int first_adts = 0;
#endif

    restartio(p);

    for(i=0; i<Chans; i++){
	coef[i] = (Float *)mal1(LN2*sizeof(*coef[0]));
	data[i] = (Float *)mal1(LN2*sizeof(*data[0]));
	state[i] = (Float *)mal1(LN*sizeof(*state[0]));	/* changed LN4 to LN 1/97 mfd */
	fltclr(state[i], LN);
	factors[i] = (short *)mal1(MAXBANDS*sizeof(*factors[0]));
	cb_map[i] = (byte *)mal1(MAXBANDS*sizeof(*cb_map[0]));
	group[i] = (byte *)mal1(NSHORT*sizeof(group[0]));
	lpflag[i] = (int *)mal1(MAXBANDS*sizeof(*lpflag[0]));
	prstflag[i] = (int *)mal1((LEN_PRED_RSTGRP+1)*sizeof(*prstflag[0]));
	tns[i] = (TNS_frame_info *)mal1(sizeof(*tns[0]));
	sp_status[i]  = (PRED_STATUS *)mal1(LN*sizeof(*sp_status[0]));
	wnd_shape[i].prev_bk = 0;
#ifdef MPEG4V1
        nok_lt_status[i]  = (NOK_LT_PRED_STATUS *)mal1(sizeof(*nok_lt_status[0])
);
#endif
    }
    for(i=0; i<Winds; i++) {
	mask[i] = (byte *)mal1(MAXBANDS*sizeof(mask[0]));
    }
#if (CChans > 0)
    for(i=0; i<CChans; i++){
	cc_coef[i] = (Float *)mal1(LN2*sizeof(*cc_coef[0]));
	for(j=0; j<Chans; j++)
	    cc_gain[i][j] = (Float *)mal1(MAXBANDS*sizeof(*cc_gain[0][0]));
#if (ICChans > 0)
	if (i < ICChans) {
	    cc_state[i] = (Float *)mal1(LN*sizeof(*state[0]));  /* changed LN4 to LN 1/97 mfd */
	    fltclr(cc_state[i], LN);
	    cc_wnd_shape[i].prev_bk = 0;
	}
#endif
    }
#endif


#ifdef	SSR
    for(i = 0; i < Chans; i++){
	wnd_shape_SSR[i].prev_bk = 0;
    }
#endif

#ifdef ADTS_SUPPORT
    if(adts_header_present != 1){
       init();        /* initialize data structures */
    }
#else
    init();		    /* initialize data structures */
    /* set defaults */
    adif_header_present = 0;

#ifdef   ADTS_SUPPORT
    adts_header_present = 0;
#endif
#ifdef  DRC
    set_drc(0, 0, 0);
#endif
    init_config(outfile);    /* read decoderSpecificInfo */
#endif

    if (outfile != NULL)
	strncpy(output_path, outfile, PATH_LEN);
}

/* decode a block */
void
dec(char *p)
{
    int i, j, ch, wn, ele_id, d_tag, d_cnt;
    int left, right;
    Info *info;
    MC_Info *mip = &mc_info;
    Ch_Info *cip;

    restartio(p);
    
	if(debug['n'])
	    PRINT(SE, "\rblock %ld", bno);
	if(debug['N'])
	    PRINT(SE, "\nblock %ld\n", bno);


#ifdef   ADTS_SUPPORT
   byte_align();
   if(adts_header_present == 1){
      get_adts_header();
      if(first_adts == 0){
         init();
         first_adts = 1;
      }
   }
#else
	byte_align();
#endif


	reset_mc_info(mip);
#ifdef	DRC	
	reset_drc();
#endif
	while ((ele_id=getbits(LEN_SE_ID)) != ID_END) {
	    /* get audio syntactic element */

	    if(debug['v'])
		PRINT(SE, "\nele_id %d\n", ele_id);

	    switch (ele_id) {
	    case ID_SCE:		/* single channel */
	    case ID_CPE:		/* channel pair */
	    case ID_LFE:		/* low freq effects channel */
		if (huffdecode(ele_id, mip, wnd, wnd_shape,
		    cb_map, factors, 
		    group, hasmask, mask, max_sfb,
		    lpflag, prstflag
#ifdef MPEG4V1
                    , nok_lt_status
#endif
                    , tns, coef) < 0)
		    myexit("huffdecode");
		break;
#if (CChans > 0)
	    case ID_CCE:		/* coupling channel */
		if (getcc(mip, cc_wnd, cc_wnd_shape, cc_coef, cc_gain) < 0)
		    myexit("getcc");
		break;
#endif				
	    case ID_DSE:		/* data element */
		if (getdata(&d_tag, &d_cnt, d_bytes) < 0)
		    myexit("data channel");
		break;
	    case ID_PCE:		/* program config element */
		get_prog_config(&prog_config);
		break;
	    case ID_FIL:		/* fill element */
		getfill(d_bytes);
		break;
	    default:
		PRINT(SE, "Element not supported: %d\n", ele_id);
		break;
	    }
	}

        if(debug['v']) {
	    PRINT(SE, "\nele_id %d\n", ele_id);
	    print_mc_info(mip);
	}
	check_mc_info(mip, (bno==0 && default_config));
#ifdef	DRC	
	map_drc_channels(mip);
#endif

#ifndef	PARSE_ONLY
#if (ICChans > 0)
	/* process independently switched coupling channels */
#ifdef	DRC
	/* apply dynamic range control */ 
	i=0;
	for (ch=0; ch<mip->ncch; ch++) {
	    if (mip->cc_ind[ch]) {
		apply_drc(mip, cc_coef[ch], Chans+i);
		i++;	/* index for drc_info[] */
	    }
	}
#endif    
	/* transform */
	ind_coupling(mip, wnd, wnd_shape, cc_wnd, cc_wnd_shape, cc_coef,
	    cc_state);
#endif
	
	/* m/s stereo */
	for (ch=0; ch<Chans; ch++) {
	    cip = &mip->ch_info[ch];
	    if ((cip->present) && (cip->cpe) && (cip->ch_is_left)) {
		wn = cip->widx;
		if(hasmask[wn]) {
		    left = ch;
		    right = cip->paired_ch;
		    info = winmap[wnd[wn]];
                    if (hasmask[wn] == 1)
                        map_mask(info, group[wn], mask[wn], cb_map[right]);
		    synt(info, group[wn], mask[wn], coef[right], coef[left]);
		}
	    }
	}

	/* intensity stereo and prediction */
	for (ch=0; ch<Chans; ch++) {
	    if (!(mip->ch_info[ch].present)) continue;
	    wn = mip->ch_info[ch].widx;
	    info = winmap[wnd[wn]];
#ifdef MPEG4V1
            pns( mip, info, wn, ch,
                 group[wn], cb_map[ch], factors[ch],
                 lpflag[wn], coef );
#endif
	    intensity(mip, info, wn, ch, 
		group[wn], cb_map[ch], factors[ch], 
		lpflag[wn], coef);
#ifndef MPEG4V1
	    predict(info, mip->profile, 
		lpflag[wn], sp_status[ch], coef[ch]);
#else
            nok_lt_predict (info, wnd[wn], &wnd_shape[wn],
                            nok_lt_status[ch]->sbk_prediction_used,
                            nok_lt_status[ch]->sfb_prediction_used,
                            nok_lt_status[ch], nok_lt_status[ch]->weight,
                            nok_lt_status[ch]->delay, coef[ch],
                            BLOCK_LEN_LONG, 0, BLOCK_LEN_SHORT, tns[ch]);
#endif
	}
	
	for (ch=0; ch<Chans; ch++) {
	    if (!(mip->ch_info[ch].present)) continue;
	    wn = mip->ch_info[ch].widx;
	    info = winmap[wnd[wn]];

#ifndef MPEG4V1
	    /* predictor reset */
	    left = ch;
	    right = left;
	    if ( (mip->ch_info[ch].cpe) &&
		(mip->ch_info[ch].common_window) )
		/* prstflag's shared by channel pair */
		right = mip->ch_info[ch].paired_ch;
	       predict_reset(info, prstflag[wn], sp_status, left, right, last_rstgrp_num); /* this line is required for mp4mcdec (RS 00-10-12) */
	       /* predict_reset(info, prstflag[wn], sp_status, left, right); */ /* Bug?? */ 
#endif

#ifdef MPEG4V1
            /* PNS predictor reset */
            predict_pns_reset(info, sp_status[ch], cb_map[ch]);
#endif


#if (CChans > 0)
	    /* if cc_domain indicates before TNS */
	    coupling(info, mip, coef, cc_coef, cc_gain, ch, CC_DOM, !CC_IND);
#endif

	    /* tns */
	    for (i=j=0; i<tns[ch]->n_subblocks; i++) {
		if (debug['T']) {
		    PRINT(SE, "%ld %d %d\n", bno, ch, i);
		    print_tns( &(tns[ch]->info[i]));
		}

		tns_decode_subblock(&coef[ch][j],
		    max_sfb[wn],
		    info->sbk_sfb_top[i],
		    info->islong,
		    &(tns[ch]->info[i]) );

		j += info->bins_per_sbk[i];
	    }

#if (CChans > 0)
	    /* if cc_domain indicated after TNS */
	    coupling(info, mip, coef, cc_coef, cc_gain, ch, !CC_DOM, !CC_IND);
#endif
#ifdef	DRC
	    /* apply dynamic range control to each audio channel */
	    apply_drc(mip, coef[ch], ch);
#endif    

#ifdef DOLBY_MDCT
	    /* inverse transform */
#ifdef  SSR
	    wnd_shape_SSR[ch].this_bk = wnd_shape[wn].this_bk;

	    if(mc_info.profile == SSR_Profile){
		gain_control(coef[ch], &wnd_shape_SSR[ch], wnd[wn], ch, data[ch]);
	    }
	    else {
		freq2time_adapt (wnd [wn], &wnd_shape_SSR[ch], coef [ch], state [ch], data [ch]);
	    }

	    wnd_shape_SSR[ch].prev_bk  = wnd_shape_SSR[ch].this_bk; 
#else
	    freq2time_adapt (wnd [wn], &wnd_shape [wn], coef [ch], state [ch], data [ch]);
#endif
#else 
	    imdct(wnd[wn], wnd_shape[wn].this_bk, coef[ch], state[ch], data[ch]);
	    {
		/* scale imdct output */
		Float scale = 1.0 / sqrt (2.0);
		for (i = 0; i < LN2; i++)
		    data[ch][i] *= scale;
	    }
#endif
#if (CChans > 0)
	    /* independently switched coupling */
	    coupling(info, mip, data, cc_coef, cc_gain, ch, CC_DOM, CC_IND);
#endif

#ifdef MPEG4V1
            nok_lt_update(nok_lt_status[ch], data[ch], state[ch], BLOCK_LEN_LONG
);
#endif
	} /* end chan loop */

#ifdef	ADTS_SUPPORT
   byte_align();  /* moved from above */
#endif

	/* skip first two blocks so output is time aligned with input */
	if (bno > 1)
	    writeout(data, mip);
#endif	    /* PARSE_ONLY */
	bno++;
}
