/*
This software module was originally developed by Apple Computer, Inc.
in the course of development of MPEG-4. 
This software module is an implementation of a part of one or 
more MPEG-4 tools as specified by MPEG-4. 
ISO/IEC gives users of MPEG-4 free license to this
software module or modifications thereof for use in hardware 
or software products claiming conformance to MPEG-4.
Those intending to use this software module in hardware or software
products are advised that its use may infringe existing patents.
The original developer of this software module and his/her company,
the subsequent editors and their companies, and ISO/IEC have no
liability for use of this software module or modifications thereof
in an implementation.
Copyright is not released for non MPEG-4 conforming
products. Apple Computer, Inc. retains full right to use the code for its own
purpose, assign or donate the code to a third party and to
inhibit third parties from using the code for non 
MPEG-4 conforming products.
This copyright notice must be included in all copies or
derivative works. Copyright (c) 1999.
*/
#include "MP4Movies.h"

/* should be from audio profile and level*/
#define	MAX_BUFFER_SIZE		(1024*1024)
extern int debug[256];

MP4Err playMyMovie( char *inFile, u32 trackNumber, char *outFile );
void init_dec(char *bp, char *filename);
void dec(char *bp);

int main( int argc, char **argv )
{
	MP4Err err;
	char *inFile, *outFile;
	extern int current_program;

/*******
#ifdef SHOW_DATA
debug['N']=1;
debug['v']=1;
debug['f']=1;
debug['s']=1;
debug['r']=1;
#endif
*******/

	if (argc < 1) {
		fprintf( stderr, "usage:\t%s mp4_file raw_file\n", argv[0]);
		fprintf( stderr, "or\tusage: %s mp4_file (to show header)\n", argv[0]);
		return MP4BadParamErr;
	}
	inFile = argv[1];
	outFile = (argc > 2) ? argv[2] : NULL;

	/* select first (and only) prog in decoderSpecificInfo */
	current_program = -1;
	
	err = MP4NoErr;
	err = playMyMovie( inFile, 1, outFile );
	fprintf( stderr, "playMyMovie returns %d\n", err );
	return err;
}

MP4Err playMyMovie( char *inFile, u32 trackNumber, char *outFile )
{
#ifdef	SHOW_DATA
	int i;
	char *p;
#endif
	MP4Err err;
	MP4Movie moov;
	MP4Track trak;
	MP4Media media;
	MP4TrackReader reader;
	MP4Handle initialObjectDescriptorH;
	MP4Handle decoderConfigH;
	MP4Handle decoderSpecificInfoH;
	MP4Handle sampleH;
	
	u32 handlerType;
	u32 handleSize;
	u32 sampleDescIndex;
 	u32 objectTypeIndication;
	u32 streamType;
	u32 decoderBufferSize;
	u32 frameCount;

	err = MP4NoErr;
	sampleDescIndex = 1;
	err = MP4OpenMovieFile( &moov, inFile, MP4OpenMovieNormal ); if (err) goto bail;
 	err = MP4NewHandle( 0, &initialObjectDescriptorH ); if (err) goto bail;
	err = MP4GetMovieInitialObjectDescriptor( moov, initialObjectDescriptorH ); if (err) goto bail;
	err = MP4GetHandleSize( initialObjectDescriptorH, &handleSize ); if (err) goto bail;

#ifdef	SHOW_DATA
	fprintf( stderr, "initialObjectDescriptor\n" );
	p = *initialObjectDescriptorH;
	for (i=0; i<handleSize; i++) {
		fprintf(stderr, "%3i 0x%02x\n", i, *p++); 
	}
#endif

	err = MP4GetMovieIndTrack( moov, trackNumber, &trak ); if (err) goto bail;
	err = MP4GetTrackMedia( trak, &media ); if (err) goto bail;
	err = MP4GetMediaHandlerDescription( media, &handlerType, NULL ); if (err) goto bail;
 	err = MP4NewHandle( 0, &decoderSpecificInfoH ); if (err) goto bail;
	err = MP4GetMediaDecoderType( media,
		sampleDescIndex, 
		&objectTypeIndication, 
		&streamType, 
		&decoderBufferSize, 
		decoderSpecificInfoH ); if (err) goto bail;
	if ( handlerType != MP4AudioHandlerType ) {
		fprintf(stderr, "not an audio movie\n");
		err = MP4VersionNotSupportedErr; goto bail;
	}
	if ( objectTypeIndication != 0x40 ) {
		fprintf(stderr, "not an MPEG-4 movie\n");
		err = MP4VersionNotSupportedErr; goto bail;
	}
	if ( streamType != 0x05 ) {
		fprintf(stderr, "not an audio stream\n");
		err = MP4VersionNotSupportedErr; goto bail;
	}
	if ( decoderBufferSize > MAX_BUFFER_SIZE ) {
		fprintf(stderr, "required buffer size too large\n");
		err = MP4VersionNotSupportedErr; goto bail;
	}
	err = MP4CreateTrackReader( trak, &reader ); if (err) goto bail;
	err = MP4NewHandle( 0, &decoderConfigH ); if (err) goto bail;
	err = MP4TrackReaderGetCurrentDecoderConfig( reader, decoderConfigH ); if (err) goto bail;

	err = MP4GetHandleSize( decoderConfigH, &handleSize ); if (err) goto bail;
#ifdef	SHOW_DATA
	fprintf( stderr, "decoderConfig\n" );
	p = *decoderConfigH;
	for (i=0; i<handleSize; i++) {
		fprintf(stderr, "%3i 0x%02x\n", i, *p++); 
	}
#endif

	/* instantiate proper decoder */
	err = MP4GetHandleSize( decoderSpecificInfoH, &handleSize ); if (err) goto bail;
#ifdef	SHOW_DATA
	fprintf( stderr, "send DCD to decoder: size = %d\n", handleSize );
	p = *decoderSpecificInfoH;
	for (i=0; i<handleSize; i++) {
		fprintf(stderr, "%3i 0x%02x\n", i, *p++); 
	}
#endif
	init_dec(*decoderSpecificInfoH, outFile);
	if (outFile == NULL) goto bail;
	fprintf( stderr, "instantiated decoder\n");

	err = MP4CreateTrackReader( trak, &reader ); if (err) goto bail;
	err = MP4NewHandle( 0, &sampleH ); if (err) goto bail;	
	/* play every frame */
	for (frameCount=1; ;frameCount++)
	{
		u32 unitSize;
		u32 cts;
		u32 dts;
		u32 sampleFlags;
		
		err = MP4TrackReaderGetNextAccessUnit( reader, sampleH, &unitSize, &sampleFlags, &cts, &dts );
		if ( err )
		{
			if ( err == MP4EOF )
				err = MP4NoErr;
			break;
		}
		/* send AU to decoder */
#ifdef	SHOW_DATA
		fprintf( stderr, "send AU to decoder: frame = %d, size = %d, cts = %d, dts = %d\n", frameCount, unitSize, cts, dts );
		p = *sampleH;
		for (i=0; i<unitSize; i++) {
			fprintf(stderr, "%3i 0x%02x\n", i, *p++); 
		}
#endif
		dec(*sampleH);
	}	
bail:
	return err;
}
