/**********************************************************************
MPEG-4 Audio VM
Low-overhead MPEG-4 Audio Transport Multiplex

This software module was originally developed by

Toshiyuki Nomura (NEC Corporation)

and edited by

Heiko Purnhagen (University of Hannover)

in the course of development of the MPEG-2 AAC/MPEG-4 Audio standard
ISO/IEC 13818-7, 14496-1,2 and 3. This software module is an
implementation of a part of one or more MPEG-2 AAC/MPEG-4 Audio tools
as specified by the MPEG-2 AAC/MPEG-4 Audio standard. ISO/IEC gives
users of the MPEG-2 AAC/MPEG-4 Audio standards free license to this
software module or modifications thereof for use in hardware or
software products claiming conformance to the MPEG-2 AAC/ MPEG-4 Audio
standards. Those intending to use this software module in hardware or
software products are advised that this use may infringe existing
patents. The original developer of this software module and his/her
company, the subsequent editors and their companies, and ISO/IEC have
no liability for use of this software module or modifications thereof
in an implementation. Copyright is not released for non MPEG-2
AAC/MPEG-4 Audio conforming products. The original developer retains
full right to use the code for his/her own purpose, assign or donate
the code to a third party and to inhibit third party from using the
code for non MPEG-2 AAC/MPEG-4 Audio conforming products. This
copyright notice must be included in all copies or derivative works.

Copyright (c) 1999.
**********************************************************************/

#include <assert.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#include "obj_descr.h"           /* structs */
#include "latm_handle.h"

#include "common_m4a.h"          /* common module       */
#include "cmdline.h"             /* command line module */
#include "bitstream.h"           /* bit stream module   */

#include "mp4au.h"               /* frame work common declarations */

#include "flex_mux.h"
#include "latm_modules.h"

#define PROGVER "LATM converter V1.1 12-Oct-2000"

static char *outFileName;
static int outFileNameUsed;
static char *outExtName;
static int outExtNameUsed;
static int conversionMode;
static int numSubFrames;
static int timing;
static int timingUsed;
static int debugLevel;

static char *oriExt,*bitExt;

static int*  varArgIdx;

static CmdLinePara paraList[] = {
  {&varArgIdx,NULL,"<bit stream file(s) (- = stdin)>"},
  {NULL,NULL,NULL}
};

static CmdLineSwitch switchList[] = {
  {"h",NULL,NULL,NULL,NULL,"print help"},
  {"o",&outFileName,"%s",NULL,&outFileNameUsed,"output file name (without ext)"},
  {"e",&outExtName,"%s",NULL,&outExtNameUsed,"output ext"},
  {"m",&conversionMode,"%i",NULL,NULL,"mode:\n"
   "0: conversion from FlexMux to LATM\n"
   "1: conversion from LATM to FlexMux\n"},
  {"n",&numSubFrames,"%i","1",NULL,"num sub-frames (1<=n<=8)"},
  {"t",&timing,"%i",NULL,&timingUsed,"allStreamSameTiming (0/1)"},
  {"d",&debugLevel,"%i","0",NULL,"debug level"},
  {NULL,NULL,NULL,NULL,NULL,NULL}
};

static void fl4_to_ass( char inBitFileName[MAXP][STRLEN],
			int numFile,
			int numFrame,
			char *outBitFileName )
{
  char               *info;
  char               info1[256];
  HANDLE_BSBITSTREAM inBitStream[MAXP];
  HANDLE_BSBITSTREAM outBitStream;

  FRAME_DATA         frameData[MAXF][MAXP];
  OBJECT_DESCRIPTOR  objDescr[MAXP];
  unsigned int       index;                    /* FlexMux Channel */
  unsigned long      length[MAXF][MAXP][MAXL]; /* SL-Packet length */

  unsigned long      latm_sync_word;           /* LATM: AudioSyncStream sync word */
  unsigned long      audioMuxLengthBytesLast;  /* LATM: length of audioMuxElement */
  int                muxConfigPresent;
  AUDIO_MUX_ELEMENT  audioMuxElement;

  unsigned int       x, firstFrame, endFlag;
  unsigned long      ltmp,align,dummy;
  long               st, ed;
  int i;
  int j = 0;
  int readFrame;
  
  /* init */
  BsInit(MAXREADAHEAD,0,1);
  info = NULL;
  sprintf(info1,"\nLATM converter (FlexMux -> LATM)\n");
  
  /* read input bit stream header (FlexMux) */
  for (i=0; i<numFile; i++) {
    inBitStream[i] = BsOpenFileRead(inBitFileName[i],MP4_MAGIC,&info);
    if (inBitStream[i]==NULL) {
      CommonExit(-1,"error opening bit stream file %s",inBitFileName[i]);
    }

    initObjDescr( &objDescr[i],0 );
    if (BsGetBit(inBitStream[i],&(objDescr[i].ODLength.value),objDescr[i].ODLength.length) )
      CommonExit(1,"Read: error reading bit stream header (FlexMux)");

    advanceODescr(inBitStream[i], &objDescr[i], 0);
    for (x=0;x<objDescr[i].streamCount.value;x++){
      initESDescr(&(objDescr[i].ESDescriptor[x]));
      advanceESDescr(inBitStream[i],objDescr[i].ESDescriptor[x],0);
      for (j=0; j<numFrame; j++) frameData[j][i].layer[x].bitBuf = BsAllocBuffer(64000);
    }

    align = 8 -  BsCurrentBit(inBitStream[i]) % 8;
    if (align == 8) align = 0;
    BsGetBit(inBitStream[i],&(dummy),align) ;

    for (j=0; j<numFrame; j++) frameData[j][i].od = &objDescr[i];
  }

  /* set StreamMuxConfig (AudioSyncStream) */
  outBitStream = BsOpenFileWrite(outBitFileName,MP4_MAGIC,info1);

  if (outBitStream==NULL) {
    CommonExit(-1,"error opening bit stream file %s",outBitFileName);
  }

  setStreamMuxConfig( frameData, numFile, numFrame, timing, timingUsed, &audioMuxElement );

  /* FlexMux -> LATM */
  muxConfigPresent = 1;
  latm_sync_word = 0x2B7;
  firstFrame = 1;
  endFlag = 0;
  audioMuxLengthBytesLast = 0x1FFF;
  while ( !endFlag ) {
    /* read FlexMux-stream */
    for (i=0; i<numFile; i++) {
      for (j=0; j<numFrame; j++) {
	if ( BsEof ( inBitStream[i], 8) ) break;
	for (x=0;x<objDescr[i].streamCount.value;x++){
	  length[j][i][x] = 0;
	  getAccessUnit( inBitStream[i], frameData[j][i].layer[x].bitBuf,
			 &index, &length[j][i][x],
			 objDescr[i].ESDescriptor[x] );
	  if ( x != index ) 
	    CommonExit(1,"Read: error reading bit stream frame");
	}
      }
    }
    readFrame = j;

    /* write audioSyncStream */
    if ( firstFrame ) {
      firstFrame = 0;
      audioMuxElement.useSameStreamMux = 0;
    } else {
      audioMuxElement.useSameStreamMux = 1;
    }
    if ( readFrame != numFrame ) {
      audioMuxElement.useSameStreamMux = 0;
      audioMuxElement.streamMuxConfig.numSubFrames = readFrame - 1;
    }

    setPayloadLengthInfo( &audioMuxElement, length );

    BsPutBit(outBitStream,latm_sync_word,11);
    BsPutBit(outBitStream,audioMuxLengthBytesLast,13);

    st = BsCurrentBit(outBitStream);
    writeAudioMuxElement(outBitStream, &audioMuxElement, muxConfigPresent);
    ed = BsCurrentBit(outBitStream);

    ltmp = ed - st;
    align = 8 - ltmp % 8;
    if (align == 8) align = 0;
    ltmp = ltmp / 8;
    if (align != 0) ltmp += 1;
    audioMuxLengthBytesLast = ltmp;
    BsPutBit(outBitStream,0,align);

    for (i=0; i<numFile; i++) {
      for (j=0; j<numFrame; j++) {
	for (x=0;x<objDescr[i].streamCount.value;x++){
	  BsClearBuffer(frameData[j][i].layer[x].bitBuf);
	}
      }
      if ( BsEof ( inBitStream[i], 8) ) endFlag = 1;
    }
  }

  for (i=0; i<numFile; i++) {
    for (x=0;x<objDescr[i].streamCount.value;x++){
      for (j=0; j<numFrame; j++) BsFreeBuffer(frameData[j][i].layer[x].bitBuf);
    }
  }

  /* close bit stream file */
  for (i=0; i<numFile; i++) {
    if (BsClose(inBitStream[i]))
      CommonExit(1,"error closing bit stream file");
  }
  if (BsClose(outBitStream))
    CommonExit(1,"error closing bit stream file");
}

static void ass_to_fl4( char *inBitFileName,
			char *outBitHeadName,
			char *outBitExtName )
{
  char               *info;
  char               info1[256];
  char outBitFileName[STRLEN];
  char tmpBitHeadName[STRLEN];
  HANDLE_BSBITSTREAM inBitStream;
  HANDLE_BSBITSTREAM outBitStream[MAXP];
  HANDLE_BSBITSTREAM tmpStream;
  HANDLE_BSBITBUFFER tmpBitBuffer;

  FRAME_DATA         frameData[MAXF][MAXP];
  OBJECT_DESCRIPTOR  objDescr[MAXP];

  unsigned long      latm_sync_word;           /* LATM: AudioSyncStream sync word */
  unsigned long      audioMuxLengthBytesLast;  /* LATM: length of audioMuxElement */
  int                muxConfigPresent;
  AUDIO_MUX_ELEMENT  audioMuxElement;

  unsigned int       x, y, firstFrame;
  unsigned long      ltmp,align,dummy;
  long               st, ed, cnt;
  int i,j;
  int numFile = 0;
  int numFrame;
  
  /* init */
  BsInit(MAXREADAHEAD,0,1);
  info = NULL;
  sprintf(info1,"\nLATM converter (LATM -> FlexMux)\n");
  for (i=0; i<MAXP; i++) {
    initObjDescr( &objDescr[i],0 );
    for (x=0;x<MAXL;x++) {
      initESDescr(&(objDescr[i].ESDescriptor[x]));
      for (j=0; j<MAXF; j++) {
	frameData[j][i].layer[x].bitBuf = BsAllocBuffer(64000);
	frameData[j][i].od = &objDescr[i];
      }
    }
  }
  
  /* read input bit stream header (FlexMux) */
  inBitStream = BsOpenFileRead(inBitFileName,MP4_MAGIC,&info);
  if (inBitStream==NULL) {
    CommonExit(-1,"error opening bit stream file %s",inBitFileName);
  }

  /* LATM -> FlexMux */
  muxConfigPresent = 1;
  firstFrame = 1;
  while ( ! BsEof ( inBitStream, 8) ) {
    /* read audioSyncStream (streamMuxConfig) */
    BsGetBit(inBitStream,&latm_sync_word,11);
    if ( latm_sync_word != 0x2B7 ) {
      CommonExit(1,"error read LATM sync word");
    }
    BsGetBit(inBitStream,&audioMuxLengthBytesLast,13);

    st = BsCurrentBit(inBitStream);
    readAudioMuxElementH(inBitStream, &audioMuxElement, objDescr, muxConfigPresent );

    numFrame = audioMuxElement.streamMuxConfig.numSubFrames+1;
    numFile = audioMuxElement.streamMuxConfig.numProgram+1;
    for (j=0; j<numFrame; j++) {
      cnt = 0;
      for (i=0; i<numFile; i++) {
	for ( y = 0; y < audioMuxElement.streamMuxConfig.numLayer[i]+1; y++ ) {
	  audioMuxElement.payloadMux[j].bitBuf[cnt] = frameData[j][i].layer[y].bitBuf;
	  cnt++;
	}
      }
    }

    /* read audioSyncStream (PayloadMux) */
    readAudioMuxElementP(inBitStream, &audioMuxElement );
    ed = BsCurrentBit(inBitStream);

    ltmp = ed - st;
    align = 8 -  ltmp % 8;
    if (align == 8) align = 0;
    BsGetBit(inBitStream,&dummy,align) ;

    if ( firstFrame ) {
      firstFrame = 0;

      setObjectDescriptor( frameData, &audioMuxElement );

      /* open output bit stream (FlexMux) */
      for (i=0; i<numFile; i++) {
	sprintf(tmpBitHeadName,"%s_p%d",outBitHeadName,i);

	if (ComposeFileName(tmpBitHeadName,0,NULL,outBitExtName,outBitFileName,STRLEN))
	  CommonExit(1,"composed file name too long");
      
	outBitStream[i] = BsOpenFileWrite(outBitFileName,MP4_MAGIC,info1);
	if (outBitStream[i]==NULL) {
	  CommonExit(-1,"error opening bit stream file %s",outBitFileName);
	}
      }

      /* write output bit stream header (FlexMux) */
      for (i=0; i<numFile; i++) {
	tmpBitBuffer = BsAllocBuffer(BITHEADERBUFSIZE);
	tmpStream = BsOpenBufferWrite(tmpBitBuffer);

	advanceODescr(tmpStream, &objDescr[i], 1);
	for (x=0;x<objDescr[i].streamCount.value;x++){
	  advanceESDescr(tmpStream, objDescr[i].ESDescriptor[x], 1);
	}

	ltmp=BsBufferNumBit(tmpBitBuffer)/8;
	align = 8 - BsBufferNumBit(tmpBitBuffer) % 8;
	if (align == 8) align = 0;
	if (align != 0) ltmp += 1;

	BsPutBit(outBitStream[i],ltmp,32);
	if (BsPutBuffer(outBitStream[i],tmpBitBuffer))
	  CommonExit(1,"Write: error writing bit stream header (FlexMux)");
	BsPutBit(outBitStream[i],0,align);

	BsFreeBuffer(tmpBitBuffer);
	BsClose(tmpStream);
      }
    }
    
    /* write FlexMux-stream */
    for (i=0; i<numFile; i++) {
      for (j=0; j<numFrame; j++) {
	for (x=0;x<objDescr[i].streamCount.value;x++){
	  putAccessUnit(x, outBitStream[i], frameData[j][i].layer[x].bitBuf,
			objDescr[i].ESDescriptor[x] );
	  BsClearBuffer(frameData[j][i].layer[x].bitBuf);
	}
      }
    }
  }

  for (i=0; i<MAXP; i++) {
    for (j=0; j<MAXF; j++) {
      for (x=0;x<MAXL;x++) {
	BsFreeBuffer(frameData[j][i].layer[x].bitBuf);
      }
    }
  }

  /* close bit stream file */
  if (BsClose(inBitStream))
    CommonExit(1,"error closing bit stream file");
  for (i=0; i<numFile; i++) {
    if (BsClose(outBitStream[i]))
      CommonExit(1,"error closing bit stream file");
  }
}

int main (int argc, char *argv[])
{
  char *progName = "<no program name>";
  int  result;
  int fileIdx;

  char inBitFileName[MAXP][STRLEN];
  char outBitFileName[STRLEN];
  char *outBitHeadName;

  /* evaluate command line  */
  CmdLineInit(0);
  result = CmdLineEval(argc,argv,paraList,switchList,1,&progName);
  if (result) {
    if (result==1) {
      printf("%s: %s\n",progName,PROGVER);
      CmdLineHelp(progName,paraList,switchList,stdout);
      exit (1);
    }
    else
      CommonExit(1,"command line error (\"-h\" for help)");
  }

  if (debugLevel == 1)
    fprintf(stderr,"%s: %s\n",progName,PROGVER);
  
  if ( conversionMode == 0 ) { /* FlexMux to LATM */
    oriExt = ".fl4";
    if (outExtNameUsed) bitExt = outExtName;
    else bitExt = ".ass";

    if (!outFileNameUsed) CommonExit(1,"no output file");
    if (ComposeFileName(outFileName,0,NULL,bitExt,outBitFileName,STRLEN))
      CommonExit(1,"composed file name too long");

    /* Multiplex all fl4 files on command line into one ass file */
    fileIdx = 0;
    while (varArgIdx[fileIdx] >= 0) {
      if ( fileIdx >= MAXP ) CommonExit(1,"too many files");

      if (ComposeFileName(argv[varArgIdx[fileIdx]],0,NULL,oriExt,inBitFileName[fileIdx],STRLEN))
	CommonExit(1,"composed file name too long");

      fileIdx++;
    }

    fl4_to_ass( inBitFileName, fileIdx, numSubFrames, outBitFileName );

  } else { /* LATM to FlexMux */
    oriExt = ".ass";
    if (outExtNameUsed) bitExt = outExtName;
    else bitExt = ".fl4";

    /* Process all ass files on command line */
    fileIdx = 0;
    while (varArgIdx[fileIdx] >= 0) {
      if (ComposeFileName(argv[varArgIdx[fileIdx]],0,NULL,oriExt,inBitFileName[0],STRLEN))
	CommonExit(1,"composed file name too long");

      if (!outFileNameUsed) outBitHeadName = argv[varArgIdx[fileIdx]];
      else outBitHeadName = outFileName;

      ass_to_fl4( inBitFileName[0], outBitHeadName, bitExt );

      fileIdx++;
    }
  }
  
  CmdLineEvalFree(paraList);

  return(0);
}
