/**********************************************************************
MPEG-4 Audio VM
LATM modules

This software module was originally developed by

Toshiyuki Nomura (NEC Corporation)

and edited by

Heiko Purnhagen (University of Hannover)

in the course of development of the MPEG-2 AAC/MPEG-4 Audio standard
ISO/IEC 13818-7, 14496-1,2 and 3. This software module is an
implementation of a part of one or more MPEG-2 AAC/MPEG-4 Audio tools
as specified by the MPEG-2 AAC/MPEG-4 Audio standard. ISO/IEC gives
users of the MPEG-2 AAC/MPEG-4 Audio standards free license to this
software module or modifications thereof for use in hardware or
software products claiming conformance to the MPEG-2 AAC/ MPEG-4 Audio
standards. Those intending to use this software module in hardware or
software products are advised that this use may infringe existing
patents. The original developer of this software module and his/her
company, the subsequent editors and their companies, and ISO/IEC have
no liability for use of this software module or modifications thereof
in an implementation. Copyright is not released for non MPEG-2
AAC/MPEG-4 Audio conforming products. The original developer retains
full right to use the code for his/her own purpose, assign or donate
the code to a third party and to inhibit third party from using the
code for non MPEG-2 AAC/MPEG-4 Audio conforming products. This
copyright notice must be included in all copies or derivative works.

Copyright (c) 1999.
**********************************************************************/
#include <string.h>
#include <stdio.h>

#include "obj_descr.h"           /* structs */
#include "latm_handle.h"         /* structs */

#include "bitstream.h"
#include "common_m4a.h"
#include "flex_mux.h"
#include "latm_modules.h"
#include "latm_modules.tbl"

#include "lpc_common.h"
#include "phi_cons.h"
#include "nec_abs_const.h"


static void writeStreamMuxConfig(HANDLE_BSBITSTREAM  bitStream, STREAM_MUX_CONFIG *muxConf);
static void readStreamMuxConfig(HANDLE_BSBITSTREAM  bitStream, STREAM_MUX_CONFIG *muxConf, OBJECT_DESCRIPTOR od[MAXP]);
static void writePayloadLengthInfo(HANDLE_BSBITSTREAM  bitStream, AUDIO_MUX_ELEMENT *audioMuxElement, int sf);
static void readPayloadLengthInfo(HANDLE_BSBITSTREAM  bitStream, AUDIO_MUX_ELEMENT *audioMuxElement, int sf);
static void writePayloadMux(HANDLE_BSBITSTREAM  bitStream, AUDIO_MUX_ELEMENT *audioMuxElement, int sf);
static void readPayloadMux(HANDLE_BSBITSTREAM  bitStream, AUDIO_MUX_ELEMENT *audioMuxElement, int sf);
static void writeAudioSpecificConfig(HANDLE_BSBITSTREAM  bitStream, STREAM_MUX_CONFIG *muxConf, int cnt);
static void readAudioSpecificConfig(HANDLE_BSBITSTREAM  bitStream, STREAM_MUX_CONFIG *muxConf, int cnt);
static void copyAudioSpecificConfig(STREAM_MUX_CONFIG *muxConf, int in, int out);
static void len2idx( STREAM_MUX_CONFIG   *muxConf,
		     PAYLOAD_LENGTH_INFO *lenInfo,
		     int length, int cnt );
static void idx2len( STREAM_MUX_CONFIG   *muxConf,
		     PAYLOAD_LENGTH_INFO *lenInfo, int cnt );
static void celpSpecificCnvrt( STREAM_MUX_CONFIG  *muxConf, int cnt,
			      int *index, int *frame_size );

void setStreamMuxConfig( FRAME_DATA        frameData[MAXF][MAXP],
			 int numProgram, int numFrame, int timing, int timingUsed,
			 AUDIO_MUX_ELEMENT *audioMuxElement )
{
  int i, j, k, streamCnt, frameTimeLength, index, bcnt;
  float frameTimeMS;
  float preTimeMS = 0;
  OBJECT_DESCRIPTOR  *od[MAXP];
  STREAM_MUX_CONFIG  *muxConf;

  muxConf = &(audioMuxElement->streamMuxConfig);

  /** audioMuxVersion **/
  muxConf->audioMuxVersion = 0;

  /** allStreamSameTimeFraming **/
  if ( timingUsed ) muxConf->allStreamSameTimeFraming = timing; 
  else muxConf->allStreamSameTimeFraming = 1; 

  /** numSubFrames **/
  muxConf->numSubFrames = numFrame-1;

  /** numProgram **/
  muxConf->numProgram = numProgram-1;
  for ( i = 0; i < (int)(muxConf->numProgram+1); i++ ) od[i] = frameData[0][i].od;
  
  /** numLayer **/
  for ( i = 0; i < (int)(muxConf->numProgram+1); i++ ){
    muxConf->numLayer[i] = od[i]->streamCount.value - 1;
  }

  /** otherDataPresent, otherDataLenBits **/
  muxConf->otherDataPresent = 0;
  muxConf->otherDataLenEsc = 0;
  muxConf->otherDataLenTmp = 0;
  muxConf->otherDataLenBits = 0;

  /** crcCheckPresent, crcCheckSum **/
  muxConf->crcCheckPresent = 0;
  muxConf->crcCheckSum = 0;

  streamCnt = 0;
  for ( i = 0; i < (int)(muxConf->numProgram+1); i++ ){
    for ( j = 0; j < (int)(muxConf->numLayer[i]+1); j++ ){
      muxConf->streamID[i][j] = streamCnt;
      muxConf->progSindx[streamCnt] = i;
      muxConf->laySindx[streamCnt] = j;
      bcnt = muxConf->streamID[i][0];

      /** audioSpecificConfig **/
      muxConf->audioSpecificConfig[streamCnt] = &(od[i]->ESDescriptor[j]->DecConfigDescr.audioSpecificConfig);

      /** frameLengthType **/
      switch (muxConf->audioSpecificConfig[streamCnt]->audioDecoderType.value) {
      case TWIN_VQ:
#ifdef VERSION2
      case ER_TWIN_VQ:
#endif
	muxConf->frameLengthType[streamCnt] = 1;
	muxConf->useSameConfig[streamCnt] = 0;
	if (muxConf->audioSpecificConfig[streamCnt]->specConf.TFSpecificConfig.frameLength.value==0) frameTimeLength = 1024;
	else frameTimeLength = 960;
	muxConf->frameLength[streamCnt] = (od[i]->ESDescriptor[j]->DecConfigDescr.avgBitrate.value * frameTimeLength / samplFreqIndex[muxConf->audioSpecificConfig[streamCnt]->samplingFreqencyIndex.value])/8 - 20;
	/* HP20001012 samplingFreqencyIndex==0xf not handled */
	break;
      case AAC_SCAL:
#ifdef VERSION2
      case ER_BSAC:
#endif
	muxConf->frameLengthType[streamCnt] = 0;
	muxConf->useSameConfig[streamCnt] = 0;
	if (muxConf->audioSpecificConfig[streamCnt]->specConf.TFSpecificConfig.frameLength.value==0) frameTimeLength = 1024;
	else frameTimeLength = 960;

	/** bufferFullness **/
	muxConf->bufferFullness[streamCnt] = 255; /* ??? */
	muxConf->coreFrameOffset[streamCnt] = 0; /* ??? */

	break;
      case CELP:
#ifdef VERSION2
      case ER_CELP:
#endif
#ifndef CORRIGENDUM1
	if ( streamCnt == 0 ) muxConf->useSameConfig[streamCnt] = 0;
	else if ( streamCnt == (int)(muxConf->audioSpecificConfig[0]->specConf.celpSpecificConfig.numEnhLayers.value + 1) ) {
	  if ( muxConf->audioSpecificConfig[0]->specConf.celpSpecificConfig.bandwidthScalabilityMode.value == ON )
	    muxConf->useSameConfig[streamCnt] = 0;
	} else muxConf->useSameConfig[streamCnt] = 1;
#else
	muxConf->useSameConfig[streamCnt] = 0;
#endif
	if ( muxConf->audioSpecificConfig[bcnt]->specConf.celpSpecificConfig.fineRateControl.value == 1 ) 
	  muxConf->frameLengthType[streamCnt] = 3;
#ifdef VERSION2
	else if ( muxConf->audioSpecificConfig[bcnt]->specConf.celpSpecificConfig.silenceCompressionSW.value == 1 ) 
	  muxConf->frameLengthType[streamCnt] = 5;
#endif
	else
	  muxConf->frameLengthType[streamCnt] = 4;

	celpSpecificCnvrt( muxConf, streamCnt, &index, &frameTimeLength );
	muxConf->CELPframeLengthTableIndex[streamCnt] = index;
	break;
      case HVXC :
#ifdef VERSION2
      case ER_HVXC :
#endif
#if 1	/* bug fix of frameLengthType and HVXCframeLengthTableIndex for ER_HVXC object type(AI 991129) */
	if ( streamCnt == 0 ) {
	  muxConf->useSameConfig[streamCnt] = 0;
	  if ( muxConf->audioSpecificConfig[streamCnt]->specConf.hvxcSpecificConfig.HVXCvarMode.value == 1 ) 
	    muxConf->frameLengthType[streamCnt] = 7;
	  else
	    muxConf->frameLengthType[streamCnt] = 6;
	  muxConf->HVXCframeLengthTableIndex[streamCnt] = muxConf->audioSpecificConfig[streamCnt]->specConf.hvxcSpecificConfig.HVXCrateMode.value;
	}
	else {
	  muxConf->useSameConfig[streamCnt] = 1;
	  muxConf->frameLengthType[streamCnt] = muxConf->frameLengthType[0];
	  muxConf->HVXCframeLengthTableIndex[streamCnt] = muxConf->HVXCframeLengthTableIndex[0];
	}
#else
	if ( streamCnt == 0 ) muxConf->useSameConfig[streamCnt] = 0;
	else muxConf->useSameConfig[streamCnt] = 1;
	if ( muxConf->audioSpecificConfig[streamCnt]->specConf.hvxcSpecificConfig.HVXCvarMode.value == 1 ) 
	  muxConf->frameLengthType[streamCnt] = 7;
	else
	  muxConf->frameLengthType[streamCnt] = 6;
	muxConf->HVXCframeLengthTableIndex[streamCnt] = muxConf->audioSpecificConfig[streamCnt]->specConf.hvxcSpecificConfig.HVXCrateMode.value;
#endif
	frameTimeLength = 160;
	break;
#ifdef VERSION2
      case ER_HILN: /* HP20001009 */
      case ER_PARA:
	muxConf->frameLengthType[streamCnt] = 0;
	muxConf->useSameConfig[streamCnt] = 0;
	if (muxConf->audioSpecificConfig[streamCnt]->specConf.paraSpecificConfig.PARAmode.value == 0)
	  frameTimeLength = 160;
	else if (muxConf->audioSpecificConfig[streamCnt]->specConf.paraSpecificConfig.PARAmode.value == 0)
	  frameTimeLength = muxConf->audioSpecificConfig[streamCnt]->specConf.paraSpecificConfig.HILNframeLength.value;
	else
	  frameTimeLength = 320;

	/** bufferFullness **/
	muxConf->bufferFullness[streamCnt] = 255; /* ??? */
	muxConf->coreFrameOffset[streamCnt] = 0; /* ??? */

	break;
#endif
      default :
	CommonExit(-1,"audioDecoderType not implemented");
	break;
      }

      if (muxConf->audioSpecificConfig[streamCnt]->samplingFreqencyIndex.value == 0xf)
	frameTimeMS = (float)frameTimeLength/muxConf->audioSpecificConfig[streamCnt]->samplingFrequency.value;
      else
	frameTimeMS = (float)frameTimeLength/samplFreqIndex[muxConf->audioSpecificConfig[streamCnt]->samplingFreqencyIndex.value];
      /** allStreamSameTimeFraming **/
      if ( i == 0 && j == 0 ) preTimeMS = frameTimeMS;
      if ( muxConf->allStreamSameTimeFraming == 1 ) {
	if ( preTimeMS != frameTimeMS ) {
	  muxConf->allStreamSameTimeFraming = 0;
	}
      }

      for ( k = 0; k < (int)(muxConf->numSubFrames+1); k++ ){
	audioMuxElement->payloadMux[k].bitBuf[streamCnt] = frameData[k][i].layer[j].bitBuf;
      }
      streamCnt++;
    }
  }

}

void setObjectDescriptor( FRAME_DATA        frameData[MAXF][MAXP],
			  AUDIO_MUX_ELEMENT *audioMuxElement )
{
  int i, j, cnt;
  int idx, frame_size;
  int frame_bits = 0;
  int bitRateLay = 0;
  OBJECT_DESCRIPTOR  *od;
  STREAM_MUX_CONFIG  *muxConf;

  muxConf = &(audioMuxElement->streamMuxConfig);

  for ( j = 0; j < (int)(muxConf->numProgram+1); j++ ) {
    od = frameData[0][j].od;
    od->ODLength.value=0; 
    od->ODescrId.value=1;
    od->streamCount.value = muxConf->numLayer[j]+1;
    od->extensionFlag.value=0;

    for ( i = 0; i < (int)(od->streamCount.value); i++ ) {
      od->ESDescriptor[i]->ESNumber.value=i+1;
      od->ESDescriptor[i]->streamDependence.value=((i==0)?0:1); 
      od->ESDescriptor[i]->URLFlag.value=0;
      od->ESDescriptor[i]->extensFlag.value=0;
      od->ESDescriptor[i]->dependsOn_Es_number.value=(i>0)?i:0;

      od->ESDescriptor[i]->DecConfigDescr.profileAndLevelIndication.value=0;
      od->ESDescriptor[i]->DecConfigDescr.streamType.value=5 ;
      od->ESDescriptor[i]->DecConfigDescr.upsteam.value=0;
      od->ESDescriptor[i]->DecConfigDescr.specificInfoFlag.value=1 ;
      od->ESDescriptor[i]->DecConfigDescr.bufferSizeDB.value=6144;
      od->ESDescriptor[i]->DecConfigDescr.maxBitrate.value=0;
      od->ESDescriptor[i]->DecConfigDescr.avgBitrate.value=0;
      od->ESDescriptor[i]->DecConfigDescr.specificInfoLength.value=2;

      od->ESDescriptor[i]->ALConfigDescriptor.useAccessUnitStartFlag.value=1;
      od->ESDescriptor[i]->ALConfigDescriptor.useAccessUnitEndFlag.value=1;
      od->ESDescriptor[i]->ALConfigDescriptor.useRandomAccessPointFlag.value=0;
      od->ESDescriptor[i]->ALConfigDescriptor.usePaddingFlag.value=0;
      od->ESDescriptor[i]->ALConfigDescriptor.seqNumLength.value=0;

      cnt = muxConf->streamID[j][i];

      if (muxConf->audioSpecificConfig[cnt]->samplingFreqencyIndex.value == 0xf)
	frameData[0][j].layer[i].sampleRate = muxConf->audioSpecificConfig[cnt]->samplingFrequency.value;
      else
	frameData[0][j].layer[i].sampleRate = samplFreqIndex[muxConf->audioSpecificConfig[cnt]->samplingFreqencyIndex.value];

      switch (muxConf->audioSpecificConfig[cnt]->audioDecoderType.value) {
      case TWIN_VQ:
#ifdef    VERSION2
      case ER_TWIN_VQ:
#endif
	od->ESDescriptor[i]->DecConfigDescr.bufferSizeDB.value=6122;
	if (muxConf->audioSpecificConfig[cnt]->specConf.TFSpecificConfig.frameLength.value==0) frame_size = 1024;
	else  frame_size = 960;
	frame_bits = 8*(20+muxConf->frameLength[cnt]);
	bitRateLay = (int)((float)frame_bits*frameData[0][j].layer[i].sampleRate/frame_size+0.5);
	break;
      case AAC_SCAL:
#ifdef VERSION2
      case ER_BSAC :
#endif
	if (muxConf->audioSpecificConfig[cnt]->specConf.TFSpecificConfig.frameLength.value==0) frame_size = 1024;
	else  frame_size = 960;
	frame_bits = audioMuxElement->payloadLengthInfo[0].MuxSlotLengthBits[cnt];
	bitRateLay = (int)((float)frame_bits*frameData[0][j].layer[i].sampleRate/frame_size+0.5);
      break;
#ifdef VERSION2
      case ER_CELP :
#endif
      case CELP :
	if (i > 0) od->ESDescriptor[i]->dependsOn_Es_number.value = 0;
	od->ESDescriptor[i]->DecConfigDescr.bufferSizeDB.value=0;
	celpSpecificCnvrt( muxConf, cnt, &idx, &frame_size );
	/* idx = muxConf->CELPframeLengthTableIndex[i]; */
	if ( muxConf->frameLengthType[cnt] == 4 ) {
	  if ( i == 0 ) frame_bits = celptb1[idx][0];
	  else          frame_bits = celptb2[idx][0];
	} else if ( muxConf->frameLengthType[cnt] == 5 ) {
	  if ( i == 0 ) frame_bits = celptb1[idx][1];
	  else          frame_bits = celptb2[idx][1];
	} else if ( muxConf->frameLengthType[cnt] == 3 ) {
	  frame_bits = celptb1[idx][1];
	}
	bitRateLay = (int)((float)frame_bits*frameData[0][j].layer[i].sampleRate/frame_size+0.5);
	break;
      case HVXC :
#ifdef VERSION2
      case ER_HVXC :
#endif
#if 1	/* enahanement layer dependency(AI 991129) */
	if (i > 0) od->ESDescriptor[i]->dependsOn_Es_number.value = od->ESDescriptor[0]->ESNumber.value;
#else
	if (i > 0) od->ESDescriptor[i]->dependsOn_Es_number.value = 0;
#endif
	od->ESDescriptor[i]->DecConfigDescr.bufferSizeDB.value=0;
	if ( i > 0 ) bitRateLay = 2000;
	else {
	  idx = muxConf->HVXCframeLengthTableIndex[cnt];
	  frame_bits = hvxctbl[idx][0];
	  frame_size = 160;
	  bitRateLay = (int)((float)frame_bits*frameData[0][j].layer[i].sampleRate/frame_size+0.5);
	}
	if (i==0) {
	  /* base layer */
	  od->ESDescriptor[i]->DecConfigDescr.specificInfoLength.value = 1;	/* 6 bits at maximum(AI 991129) */
	}
	else {
	  /* enhancement layer */
	  /* no specificInfo required(AI 991129) */
	  od->ESDescriptor[i]->DecConfigDescr.specificInfoFlag.value = 0;
	  od->ESDescriptor[i]->DecConfigDescr.specificInfoLength.value = 0;
	}
	break;
#ifdef VERSION2
      case ER_HILN : /* HP20001009 */
      case ER_PARA :
	if (muxConf->audioSpecificConfig[cnt]->specConf.paraSpecificConfig.PARAmode.value == 0) {
	  frame_size = 160;
	  od->ESDescriptor[i]->DecConfigDescr.specificInfoLength.value = 1;
	}
	else if (muxConf->audioSpecificConfig[cnt]->specConf.paraSpecificConfig.PARAmode.value == 1) {
	  frame_size = muxConf->audioSpecificConfig[i]->specConf.paraSpecificConfig.HILNframeLength.value;
	  od->ESDescriptor[i]->DecConfigDescr.specificInfoLength.value = 4;
	}
	else {
	  frame_size = 320;
	  od->ESDescriptor[i]->DecConfigDescr.specificInfoLength.value = 5;
	}
	if (i>0) /* enhancement/extension layer */
	  od->ESDescriptor[i]->DecConfigDescr.specificInfoLength.value = 1;
	frame_bits = audioMuxElement->payloadLengthInfo[0].MuxSlotLengthBits[cnt];
	bitRateLay = (int)((float)frame_bits*frameData[0][j].layer[i].sampleRate/frame_size+0.5);
	break;
#endif
      default :
	CommonExit(-1,"audioDecoderType not implemented");
	break;
      }
      frameData[0][j].layer[i].bitRate = bitRateLay;
      od->ESDescriptor[i]->DecConfigDescr.maxBitrate.value=bitRateLay;
      od->ESDescriptor[i]->DecConfigDescr.avgBitrate.value=bitRateLay;
    }
  }
}

void setPayloadLengthInfo( AUDIO_MUX_ELEMENT *audioMuxElement,
			   unsigned long length[MAXF][MAXP][MAXL] )
{
  int i, j, k, streamCnt, chunkCnt, streamIndx;
  STREAM_MUX_CONFIG   *muxConf;
  PAYLOAD_LENGTH_INFO *lenInfo;

  muxConf = &(audioMuxElement->streamMuxConfig);

  if ( muxConf->allStreamSameTimeFraming ) {
    for ( k = 0; k < (int)(muxConf->numSubFrames+1); k++ ){
      lenInfo = &(audioMuxElement->payloadLengthInfo[k]);

      for ( i = 0; i < (int)(muxConf->numProgram+1); i++ ){
	for ( j = 0; j < (int)(muxConf->numLayer[i]+1); j++ ){
	  streamCnt = muxConf->streamID[i][j];
	  if ( muxConf->frameLengthType[streamCnt] == 0 ) {
	    lenInfo->MuxSlotLengthBits[streamCnt] = length[k][i][j]*8;
	  } else if ( muxConf->frameLengthType[streamCnt] == 1 ) {
	    lenInfo->MuxSlotLengthBits[streamCnt] = 8*(20+muxConf->frameLength[streamCnt]);
	  } else if ( muxConf->frameLengthType[streamCnt] >= 3 &&
		      muxConf->frameLengthType[streamCnt] <= 7 ) {
	    len2idx( muxConf, lenInfo, length[k][i][j], streamCnt );
	  }
	}
      }
    }
  } else {
    for ( k = 0; k < (int)(muxConf->numSubFrames+1); k++ ){
      lenInfo = &(audioMuxElement->payloadLengthInfo[k]);

      lenInfo->numChunk = 0;
      for ( i = 0; i < (int)(muxConf->numProgram+1); i++ )
	lenInfo->numChunk += muxConf->numLayer[i]+1;
      lenInfo->numChunk = lenInfo->numChunk - 1;

      for ( chunkCnt = 0; chunkCnt < (int)(lenInfo->numChunk+1); chunkCnt++ ){
	streamIndx = lenInfo->streamIndx[chunkCnt] = chunkCnt;

	i = lenInfo->progCindx[chunkCnt] = muxConf->progSindx[streamIndx];
	j = lenInfo->layCindx[chunkCnt] = muxConf->laySindx[streamIndx];
	streamCnt = muxConf->streamID[i][j];

	if ( muxConf->frameLengthType[streamCnt] == 0 ) {
	  lenInfo->MuxSlotLengthBits[streamCnt] = length[k][i][j]*8;
	  lenInfo->AuEndFlag[streamCnt] = 1; /* ?? */
	} else if ( muxConf->frameLengthType[streamCnt] == 1 ) {
	  lenInfo->MuxSlotLengthBits[streamCnt] = 8*(20+muxConf->frameLength[streamCnt]);
	} else if ( muxConf->frameLengthType[streamCnt] >= 3 &&
		    muxConf->frameLengthType[streamCnt] <= 7 ) {
	  len2idx( muxConf, lenInfo, length[k][i][j], streamCnt );
	}
      }
    }
  }
}

void writeAudioMuxElement(HANDLE_BSBITSTREAM  bitStream, AUDIO_MUX_ELEMENT *audioMuxElement, int muxConfigPresent )
{
  int i;

  if ( muxConfigPresent ) {
    BsPutBit(bitStream,audioMuxElement->useSameStreamMux,1);
    if ( !(audioMuxElement->useSameStreamMux) ) {
      writeStreamMuxConfig(bitStream, &(audioMuxElement->streamMuxConfig));
    }
  }

  if ( audioMuxElement->streamMuxConfig.audioMuxVersion == 0 ) {
    for ( i = 0; i < (int)(audioMuxElement->streamMuxConfig.numSubFrames+1); i++ ) {
      writePayloadLengthInfo(bitStream, audioMuxElement, i);
      writePayloadMux(bitStream, audioMuxElement, i);
    }
    if ( audioMuxElement->streamMuxConfig.otherDataPresent ) {
      /* write other data */
    }
  } else {
    CommonExit(-1,"audioMuxVersion(1) not defined");
  }
}

void readAudioMuxElementH(HANDLE_BSBITSTREAM  bitStream, AUDIO_MUX_ELEMENT *audioMuxElement, OBJECT_DESCRIPTOR od[MAXP], int muxConfigPresent )
{
  if ( muxConfigPresent ) {
    BsGetBit(bitStream,&(audioMuxElement->useSameStreamMux),1);
    if ( !(audioMuxElement->useSameStreamMux) ) {
      readStreamMuxConfig(bitStream, &(audioMuxElement->streamMuxConfig), od);
    }
  }
}

void readAudioMuxElementP(HANDLE_BSBITSTREAM  bitStream, AUDIO_MUX_ELEMENT *audioMuxElement )
{
  int i;

  if ( audioMuxElement->streamMuxConfig.audioMuxVersion == 0 ) {
    for ( i = 0; i < (int)(audioMuxElement->streamMuxConfig.numSubFrames+1); i++ ) {
      readPayloadLengthInfo(bitStream, audioMuxElement, i);
      readPayloadMux(bitStream, audioMuxElement, i);
    }
    if ( audioMuxElement->streamMuxConfig.otherDataPresent ) {
      /* read other data */
    }
  } else {
    CommonExit(-1,"audioMuxVersion(1) not defined");
  }
}

static void writeStreamMuxConfig(HANDLE_BSBITSTREAM  bitStream, STREAM_MUX_CONFIG *muxConf)
{
  int i, j, streamCnt, first;
  unsigned long tmp;

  BsPutBit(bitStream,muxConf->audioMuxVersion,1);
  if ( muxConf->audioMuxVersion == 0 ) {
    BsPutBit(bitStream,muxConf->allStreamSameTimeFraming,1);
    BsPutBit(bitStream,muxConf->numSubFrames,6);
    BsPutBit(bitStream,muxConf->numProgram,4);

    for ( i = 0; i < (int)(muxConf->numProgram+1); i++ ){
      BsPutBit(bitStream,muxConf->numLayer[i],3);
      for ( j = 0; j < (int)(muxConf->numLayer[i]+1); j++ ){
	streamCnt = muxConf->streamID[i][j];
	if ( i == 0 && j == 0 ) {
	  writeAudioSpecificConfig(bitStream, muxConf, streamCnt);
	} else {
	  BsPutBit(bitStream,muxConf->useSameConfig[streamCnt],1);
	  if ( !(muxConf->useSameConfig[streamCnt]) )
	    writeAudioSpecificConfig(bitStream, muxConf, streamCnt);
	}

	BsPutBit(bitStream,muxConf->frameLengthType[streamCnt],3);
	if ( muxConf->frameLengthType[streamCnt] == 0 ) {
	  BsPutBit(bitStream,muxConf->bufferFullness[streamCnt],8);
	  if ( !muxConf->allStreamSameTimeFraming ) {
	    if ((muxConf->audioSpecificConfig[streamCnt]->audioDecoderType.value == 6 ||
		 muxConf->audioSpecificConfig[streamCnt]->audioDecoderType.value == 20 ) &&
		(muxConf->audioSpecificConfig[streamCnt-1]->audioDecoderType.value == 8 ||
		 muxConf->audioSpecificConfig[streamCnt-1]->audioDecoderType.value == 24 )) {
	      BsPutBit(bitStream,muxConf->coreFrameOffset[streamCnt],6);
	    }
	  }
	} else if ( muxConf->frameLengthType[streamCnt] == 1 ) {
	  BsPutBit(bitStream,muxConf->frameLength[streamCnt],9);
	} else if ( muxConf->frameLengthType[streamCnt] == 4 ||
		    muxConf->frameLengthType[streamCnt] == 5 ||
		    muxConf->frameLengthType[streamCnt] == 3 ) { 
	  BsPutBit(bitStream,muxConf->CELPframeLengthTableIndex[streamCnt],6);
	} else if ( muxConf->frameLengthType[streamCnt] == 6 ||
		    muxConf->frameLengthType[streamCnt] == 7 ) { 
	  BsPutBit(bitStream,muxConf->HVXCframeLengthTableIndex[streamCnt],1);
	}
      }
    }

    BsPutBit(bitStream,muxConf->otherDataPresent,1);
    if ( muxConf->otherDataPresent ) {
      tmp = muxConf->otherDataLenBits;
      first = 0;
      for ( i = 0; i < 4; i++ ) {
	muxConf->otherDataLenTmp = (tmp & 0xFF000000) >> 24;
	tmp = (tmp & 0xFFFFFF) << 8;

	if ( !first ) {
	  if ( muxConf->otherDataLenTmp == 0 ) continue;
	  first = 1;
	}
	if ( i == 3 ) muxConf->otherDataLenEsc = 0;
	else muxConf->otherDataLenEsc = 1;

	BsPutBit(bitStream,muxConf->otherDataLenEsc,1);
	BsPutBit(bitStream,muxConf->otherDataLenTmp,8);
      } 
    }
    BsPutBit(bitStream,muxConf->crcCheckPresent,1);
    if ( muxConf->crcCheckPresent )
      BsPutBit(bitStream,muxConf->crcCheckSum,8);
  } else {
    CommonExit(-1,"audioMuxVersion(1) not defined");
  }
}

static void readStreamMuxConfig(HANDLE_BSBITSTREAM  bitStream, STREAM_MUX_CONFIG *muxConf, OBJECT_DESCRIPTOR od[MAXP])
{
  int i, j, streamCnt;
  int presetcnt = 0;

  BsGetBit(bitStream,&(muxConf->audioMuxVersion),1);
  if ( muxConf->audioMuxVersion == 0 ) {
    BsGetBit(bitStream,&(muxConf->allStreamSameTimeFraming),1);
    BsGetBit(bitStream,&(muxConf->numSubFrames),6);
    BsGetBit(bitStream,&(muxConf->numProgram),4);

    streamCnt = 0;
    for ( i = 0; i < (int)(muxConf->numProgram+1); i++ ){
      BsGetBit(bitStream,&(muxConf->numLayer[i]),3);
      for ( j = 0; j < (int)(muxConf->numLayer[i]+1); j++ ){
	muxConf->streamID[i][j] = streamCnt;
	muxConf->progSindx[streamCnt] = i;
	muxConf->laySindx[streamCnt] = j;
	muxConf->audioSpecificConfig[streamCnt] = &(od[i].ESDescriptor[j]->DecConfigDescr.audioSpecificConfig);
	if ( i == 0 && j == 0 ) {
	  readAudioSpecificConfig(bitStream, muxConf, streamCnt);
	  presetcnt = streamCnt;
	} else {
	  BsGetBit(bitStream,&(muxConf->useSameConfig[streamCnt]),1);
	  if ( !(muxConf->useSameConfig[streamCnt]) ) {
	    readAudioSpecificConfig(bitStream, muxConf, streamCnt);
	    presetcnt = streamCnt;
	  } else {
	    copyAudioSpecificConfig(muxConf, presetcnt, streamCnt);
	  }
	}

	BsGetBit(bitStream,&(muxConf->frameLengthType[streamCnt]),3);
	if ( muxConf->frameLengthType[streamCnt] == 0 ) {
	  BsGetBit(bitStream,&(muxConf->bufferFullness[streamCnt]),8);
	  if ( !muxConf->allStreamSameTimeFraming ) {
	    if ((muxConf->audioSpecificConfig[streamCnt]->audioDecoderType.value == 6 ||
		 muxConf->audioSpecificConfig[streamCnt]->audioDecoderType.value == 20 ) &&
		(muxConf->audioSpecificConfig[streamCnt-1]->audioDecoderType.value == 8 ||
		 muxConf->audioSpecificConfig[streamCnt-1]->audioDecoderType.value == 24 )) {
	      BsGetBit(bitStream,&(muxConf->coreFrameOffset[streamCnt]),6);
	    }
	  }
	} else if ( muxConf->frameLengthType[streamCnt] == 1 ) {
	  BsGetBit(bitStream,&(muxConf->frameLength[streamCnt]),9);
	} else if ( muxConf->frameLengthType[streamCnt] == 4 ||
		    muxConf->frameLengthType[streamCnt] == 5 ||
		    muxConf->frameLengthType[streamCnt] == 3 ) { 
	  BsGetBit(bitStream,&(muxConf->CELPframeLengthTableIndex[streamCnt]),6);
	} else if ( muxConf->frameLengthType[streamCnt] == 6 ||
		    muxConf->frameLengthType[streamCnt] == 7 ) { 
	  BsGetBit(bitStream,&(muxConf->HVXCframeLengthTableIndex[streamCnt]),1);
	}
	streamCnt++;
      }
    }

    BsGetBit(bitStream,&(muxConf->otherDataPresent),1);
    if ( muxConf->otherDataPresent ) {
      muxConf->otherDataLenBits = 0;
      do {
	muxConf->otherDataLenBits = muxConf->otherDataLenBits << 8;
	BsGetBit(bitStream,&(muxConf->otherDataLenEsc),1);
	BsGetBit(bitStream,&(muxConf->otherDataLenTmp),8);
	muxConf->otherDataLenBits += muxConf->otherDataLenTmp;
      } while ( muxConf->otherDataLenEsc );
    }
    BsGetBit(bitStream,&(muxConf->crcCheckPresent),1);
    if ( muxConf->crcCheckPresent )
      BsGetBit(bitStream,&(muxConf->crcCheckSum),8);
  } else {
    CommonExit(-1,"audioMuxVersion(1) not defined");
  }
}

static void writePayloadLengthInfo(HANDLE_BSBITSTREAM  bitStream, AUDIO_MUX_ELEMENT *audioMuxElement, int sf)
{
  int i, j, streamCnt, chunkID;
  long tmp;
  STREAM_MUX_CONFIG   *muxConf;
  PAYLOAD_LENGTH_INFO *lenInfo;

  muxConf = &(audioMuxElement->streamMuxConfig);
  lenInfo = &(audioMuxElement->payloadLengthInfo[sf]);

  if ( muxConf->allStreamSameTimeFraming ) {
    for ( i = 0; i < (int)(muxConf->numProgram+1); i++ ){
      for ( j = 0; j < (int)(muxConf->numLayer[i]+1); j++ ){
	streamCnt = muxConf->streamID[i][j];
	if ( muxConf->frameLengthType[streamCnt] == 0 ) {
	  tmp = lenInfo->MuxSlotLengthBits[streamCnt]/8;
	  while ( tmp >= 255 ) {
	    BsPutBit(bitStream,255,8); /* tmp */
	    tmp -= 255;
	  }
	  BsPutBit(bitStream,tmp,8); /* tmp */
	} else if ( muxConf->frameLengthType[streamCnt] == 5 ||
		    muxConf->frameLengthType[streamCnt] == 7 ||
		    muxConf->frameLengthType[streamCnt] == 3 ) {
	  BsPutBit(bitStream,lenInfo->MuxSlotLengthCoded[streamCnt],2);
	}
      }
    }
  } else {
    BsPutBit(bitStream,lenInfo->numChunk,4);
    for ( i = 0; i < (int)(lenInfo->numChunk+1); i++ ){
      BsPutBit(bitStream,lenInfo->streamIndx[i],4);
      chunkID = lenInfo->streamIndx[i];
      if ( muxConf->frameLengthType[chunkID] == 0 ) {
	tmp = lenInfo->MuxSlotLengthBits[chunkID]/8;
	while ( tmp >= 255 ) {
	  BsPutBit(bitStream,255,8); /* tmp */
	  tmp -= 255;
	}
	BsPutBit(bitStream,tmp,8); /* tmp */
	BsPutBit(bitStream,lenInfo->AuEndFlag[chunkID],1);
      } else if ( muxConf->frameLengthType[chunkID] == 5 ||
	 	  muxConf->frameLengthType[chunkID] == 7 ||
		  muxConf->frameLengthType[chunkID] == 3 ) {
	BsPutBit(bitStream,lenInfo->MuxSlotLengthCoded[chunkID],2);
      }
    }
  }
}

static void readPayloadLengthInfo(HANDLE_BSBITSTREAM  bitStream, AUDIO_MUX_ELEMENT *audioMuxElement, int sf)
{
  int i, j, streamCnt, chunkID;
  unsigned long tmp;
  STREAM_MUX_CONFIG   *muxConf;
  PAYLOAD_LENGTH_INFO *lenInfo;

  muxConf = &(audioMuxElement->streamMuxConfig);
  lenInfo = &(audioMuxElement->payloadLengthInfo[sf]);

  if ( muxConf->allStreamSameTimeFraming ) {
    for ( i = 0; i < (int)(muxConf->numProgram+1); i++ ){
      for ( j = 0; j < (int)(muxConf->numLayer[i]+1); j++ ){
	streamCnt = muxConf->streamID[i][j];
	if ( muxConf->frameLengthType[streamCnt] == 0 ) {
	  lenInfo->MuxSlotLengthBits[streamCnt] = 0;
	  do {
	    BsGetBit(bitStream,&tmp,8);  /* tmp */
	    lenInfo->MuxSlotLengthBits[streamCnt] += tmp*8;
	  } while ( tmp == 255 );
	} else if ( muxConf->frameLengthType[streamCnt] == 1 ) {
	  lenInfo->MuxSlotLengthBits[streamCnt] = 8*(20+muxConf->frameLength[streamCnt]);
	} else if ( muxConf->frameLengthType[streamCnt] == 5 ||
		    muxConf->frameLengthType[streamCnt] == 7 ||
		    muxConf->frameLengthType[streamCnt] == 3 ) {
	  BsGetBit(bitStream,&(lenInfo->MuxSlotLengthCoded[streamCnt]),2);
	  idx2len( muxConf, lenInfo, streamCnt );
	} else if ( muxConf->frameLengthType[streamCnt] == 4 ||
		    muxConf->frameLengthType[streamCnt] == 6 ) {
	  idx2len( muxConf, lenInfo, streamCnt );
	}
      }
    }
  } else {
    BsGetBit(bitStream,&(lenInfo->numChunk),4);
    for ( i = 0; i < (int)(lenInfo->numChunk+1); i++ ){
      BsGetBit(bitStream,&(lenInfo->streamIndx[i]),4);

      chunkID = lenInfo->streamIndx[i];
      lenInfo->progCindx[i] = muxConf->progSindx[chunkID];
      lenInfo->layCindx[i] = muxConf->laySindx[chunkID];
      if ( muxConf->frameLengthType[chunkID] == 0 ) {
	lenInfo->MuxSlotLengthBits[chunkID] = 0;
	do {
	  BsGetBit(bitStream,&tmp,8);  /* tmp */
	  lenInfo->MuxSlotLengthBits[chunkID] += tmp*8;
	} while ( tmp == 255 );
	BsGetBit(bitStream,&(lenInfo->AuEndFlag[chunkID]),1);
      } else if ( muxConf->frameLengthType[chunkID] == 1 ) {
	lenInfo->MuxSlotLengthBits[chunkID] = 8*(20+muxConf->frameLength[chunkID]);
      } else if ( muxConf->frameLengthType[chunkID] == 5 ||
		  muxConf->frameLengthType[chunkID] == 7 ||
		  muxConf->frameLengthType[chunkID] == 3 ) {
	BsGetBit(bitStream,&(lenInfo->MuxSlotLengthCoded[chunkID]),2);
	idx2len( muxConf, lenInfo, chunkID );
      } else if ( muxConf->frameLengthType[chunkID] == 4 ||
		  muxConf->frameLengthType[chunkID] == 6 ) {
	idx2len( muxConf, lenInfo, chunkID );
      }
    }
  }
}

static void writePayloadMux(HANDLE_BSBITSTREAM  bitStream, AUDIO_MUX_ELEMENT *audioMuxElement, int sf)
{
  int i, j, streamCnt, chunkID;
  STREAM_MUX_CONFIG   *muxConf;
  PAYLOAD_LENGTH_INFO *lenInfo;
  PAYLOAD_MUX *payload;

  muxConf = &(audioMuxElement->streamMuxConfig);
  lenInfo = &(audioMuxElement->payloadLengthInfo[sf]);
  payload = &(audioMuxElement->payloadMux[sf]);

  if ( muxConf->allStreamSameTimeFraming ) {
    for ( i = 0; i < (int)(muxConf->numProgram+1); i++ ){
      for ( j = 0; j < (int)(muxConf->numLayer[i]+1); j++ ){
	streamCnt = muxConf->streamID[i][j];
        BsBufferManipulateSetNumBit(lenInfo->MuxSlotLengthBits[streamCnt],payload->bitBuf[streamCnt]);
	BsPutBuffer(bitStream,payload->bitBuf[streamCnt]);
      }
    }
  } else {
    for ( i = 0; i < (int)(lenInfo->numChunk+1); i++ ){
      chunkID = lenInfo->streamIndx[i];
      BsBufferManipulateSetNumBit(lenInfo->MuxSlotLengthBits[chunkID],payload->bitBuf[chunkID]);
      BsPutBuffer(bitStream,payload->bitBuf[chunkID]);
    }
  }
}

static void readPayloadMux(HANDLE_BSBITSTREAM  bitStream, AUDIO_MUX_ELEMENT *audioMuxElement, int sf)
{
  int i, j, streamCnt, chunkID;
  STREAM_MUX_CONFIG   *muxConf;
  PAYLOAD_LENGTH_INFO *lenInfo;
  PAYLOAD_MUX *payload;

  muxConf = &(audioMuxElement->streamMuxConfig);
  lenInfo = &(audioMuxElement->payloadLengthInfo[sf]);
  payload = &(audioMuxElement->payloadMux[sf]);

  if ( muxConf->allStreamSameTimeFraming ) {
    for ( i = 0; i < (int)(muxConf->numProgram+1); i++ ){
      for ( j = 0; j < (int)(muxConf->numLayer[i]+1); j++ ){
	streamCnt = muxConf->streamID[i][j];
	BsGetBuffer(bitStream,payload->bitBuf[streamCnt],lenInfo->MuxSlotLengthBits[streamCnt]);
      }
    }
  } else {
    for ( i = 0; i < (int)(lenInfo->numChunk+1); i++ ){
      chunkID = lenInfo->streamIndx[i];
      BsGetBuffer(bitStream,payload->bitBuf[chunkID],lenInfo->MuxSlotLengthBits[chunkID]);
    }
  }
}

static void writeAudioSpecificConfig(HANDLE_BSBITSTREAM  bitStream, STREAM_MUX_CONFIG *muxConf, int cnt)
{
  AUDIO_SPECIFIC_CONFIG *auConf;
#ifndef CORRIGENDUM1
  CELP_SPECIFIC_CONFIG *celpConf;
#endif

  auConf = muxConf->audioSpecificConfig[cnt];

  BsPutBit(bitStream,auConf->audioDecoderType.value,auConf->audioDecoderType.length) ;
  BsPutBit(bitStream,auConf->samplingFreqencyIndex.value,auConf->samplingFreqencyIndex.length) ;
  if (auConf->samplingFreqencyIndex.value == 0xf)
    BsPutBit(bitStream,auConf->samplingFrequency.value,auConf->samplingFrequency.length) ;
  BsPutBit(bitStream,auConf->channelConfiguration.value,auConf->channelConfiguration.length) ;
  switch (auConf->audioDecoderType.value) {
  case AAC_SCAL :
  case TWIN_VQ :  
#ifdef VERSION2
  case ER_BSAC :
  case ER_TWIN_VQ :  
#endif
    advanceTFspecConf(bitStream,&(auConf->specConf.TFSpecificConfig), 
		auConf->audioDecoderType.value, 1);
    break;
#ifdef VERSION2
  case ER_CELP :
#endif
  case CELP :
#ifdef CORRIGENDUM1
    advanceCelpSpecConf(bitStream,&(auConf->specConf.celpSpecificConfig),1);
#else
    celpConf = &(muxConf->audioSpecificConfig[0]->specConf.celpSpecificConfig);
    if ( cnt == 0 ) { /*CELP BaseLayer*/
      advanceCelpSpecConf(bitStream,&(auConf->specConf.celpSpecificConfig),1);
    } else if ( cnt == (int)(celpConf->numEnhLayers.value+1) ) {
      if ( celpConf->bandwidthScalabilityMode.value == ON ) {
	advanceCelpEnhSpecConf(bitStream,&(auConf->specConf.celpEnhSpecificConfig),1);
      }
    } else {
      advanceCelpSpecConf(bitStream,celpConf,1);
    }
#endif
    break;
#ifdef VERSION2
  case ER_HVXC:
#endif
  case HVXC:
    advanceHvxcSpecConf(bitStream,&(auConf->specConf.hvxcSpecificConfig),1);
    break;
#ifdef VERSION2
  case ER_HILN:
  case ER_PARA:
    if ( cnt == 0 )
      advanceParaSpecConf(bitStream,&(auConf->specConf.paraSpecificConfig),1);
    else
      advanceParaEnhSpecConf(bitStream,&(auConf->specConf.paraSpecificConfig),1);
    break;
#endif
  default :
    CommonExit(-1,"audioDecoderType not implemented");
    break;
  }
}

static void readAudioSpecificConfig(HANDLE_BSBITSTREAM  bitStream, STREAM_MUX_CONFIG *muxConf, int cnt)
{
  AUDIO_SPECIFIC_CONFIG *auConf;
#ifndef CORRIGENDUM1
  CELP_SPECIFIC_CONFIG *celpConf;
#endif

  auConf = muxConf->audioSpecificConfig[cnt];

  BsGetBit(bitStream,&(auConf->audioDecoderType.value),auConf->audioDecoderType.length) ;
  BsGetBit(bitStream,&(auConf->samplingFreqencyIndex.value),auConf->samplingFreqencyIndex.length) ;
  if (auConf->samplingFreqencyIndex.value == 0xf)
    BsGetBit(bitStream,&(auConf->samplingFrequency.value),auConf->samplingFrequency.length) ;
  BsGetBit(bitStream,&(auConf->channelConfiguration.value),auConf->channelConfiguration.length) ;

  switch (auConf->audioDecoderType.value) {
  case AAC_SCAL :
  case TWIN_VQ :  
#ifdef VERSION2
  case ER_BSAC :
  case ER_TWIN_VQ :  
#endif
    initTFspecConf(&(auConf->specConf.TFSpecificConfig));
    advanceTFspecConf(bitStream,&(auConf->specConf.TFSpecificConfig),
             auConf->audioDecoderType.value, 0);
    break;
#ifdef VERSION2
  case ER_CELP :
#endif
  case CELP :
#ifdef CORRIGENDUM1
    initCelpSpecConf(&(auConf->specConf.celpSpecificConfig)
#ifdef VERSION2
		     ,auConf->audioDecoderType.value
#endif
		     );
    advanceCelpSpecConf(bitStream,&(auConf->specConf.celpSpecificConfig),0);
#else
    celpConf = &(muxConf->audioSpecificConfig[0]->specConf.celpSpecificConfig);
    if ( cnt == 0 ) { /*CELP BaseLayer*/
      initCelpSpecConf(&(auConf->specConf.celpSpecificConfig)
#ifdef VERSION2
		       ,auConf->audioDecoderType.value
#endif
                       );
      advanceCelpSpecConf(bitStream,&(auConf->specConf.celpSpecificConfig),0);
    } else if ( cnt == (int)(celpConf->numEnhLayers.value+1) ) {
      if ( celpConf->bandwidthScalabilityMode.value == ON ) {
	initCelpEnhSpecConf(&(auConf->specConf.celpEnhSpecificConfig));
	advanceCelpEnhSpecConf(bitStream,&(auConf->specConf.celpEnhSpecificConfig),0);
      }
    } else { /* No SpecConf for CELP BRS Enh.Layer */
      initCelpSpecConf(&(auConf->specConf.celpSpecificConfig)
#ifdef VERSION2
		       ,auConf->audioDecoderType.value
#endif
);
      advanceCelpSpecConf(bitStream,&(auConf->specConf.celpSpecificConfig),0);
    }
#endif
    break;
#ifdef VERSION2
  case ER_HVXC:
#endif
  case HVXC:
    initHvxcSpecConf(&(auConf->specConf.hvxcSpecificConfig));
    advanceHvxcSpecConf(bitStream,&(auConf->specConf.hvxcSpecificConfig),0);
    break;
#ifdef VERSION2
  case ER_HILN:
  case ER_PARA:
    initParaSpecConf(&(auConf->specConf.paraSpecificConfig));
#ifndef CORRIGENDUM1
    if ( cnt == 0 )
      advanceParaSpecConf(bitStream,&(auConf->specConf.paraSpecificConfig),0);
    else
      advanceParaEnhSpecConf(bitStream,&(auConf->specConf.paraSpecificConfig),0);
#else
    advanceParaSpecConf(bitStream,&(auConf->specConf.paraSpecificConfig),0);
#endif
    break;
#endif
  default :
    CommonExit(-1,"audioDecoderType not implemented");
    break;
  }
}

static void copyAudioSpecificConfig(STREAM_MUX_CONFIG *muxConf, int in, int out)
{
  AUDIO_SPECIFIC_CONFIG *iConf, *oConf;

  iConf = muxConf->audioSpecificConfig[in];
  oConf = muxConf->audioSpecificConfig[out];

  oConf->audioDecoderType.value = iConf->audioDecoderType.value;
  oConf->samplingFreqencyIndex.value = iConf->samplingFreqencyIndex.value;
  oConf->samplingFrequency.value = iConf->samplingFrequency.value;
  oConf->channelConfiguration.value = iConf->channelConfiguration.value;

  switch (iConf->audioDecoderType.value) {
  case AAC_SCAL :
  case TWIN_VQ :  
#ifdef VERSION2
  case ER_BSAC :
  case ER_TWIN_VQ :  
#endif
    oConf->specConf.TFSpecificConfig.frameLength.value
      = iConf->specConf.TFSpecificConfig.frameLength.value;
    oConf->specConf.TFSpecificConfig.dependsOnCoreCoder.value
      = iConf->specConf.TFSpecificConfig.dependsOnCoreCoder.value;
    oConf->specConf.TFSpecificConfig.coreCoderDelay.value
      = iConf->specConf.TFSpecificConfig.coreCoderDelay.value;
    oConf->specConf.TFSpecificConfig.extension.value
      = iConf->specConf.TFSpecificConfig.extension.value;
#ifdef VERSION2
    if (oConf->specConf.TFSpecificConfig.extension.value) {
      if (iConf->audioDecoderType.value == ER_BSAC) {
        oConf->specConf.TFSpecificConfig.numOfSubFrame.value
         = iConf->specConf.TFSpecificConfig.numOfSubFrame.value;
        oConf->specConf.TFSpecificConfig.layer_length.value
         = iConf->specConf.TFSpecificConfig.layer_length.value;
      }
      oConf->specConf.TFSpecificConfig.extension3.value
        = iConf->specConf.TFSpecificConfig.extension3.value;
    }
#endif
    break;
  case CELP :
    oConf->specConf.celpSpecificConfig.excitationMode.value
      = iConf->specConf.celpSpecificConfig.excitationMode.value;
    oConf->specConf.celpSpecificConfig.sampleRateMode.value
      = iConf->specConf.celpSpecificConfig.sampleRateMode.value;
    oConf->specConf.celpSpecificConfig.fineRateControl.value
      = iConf->specConf.celpSpecificConfig.fineRateControl.value;
    oConf->specConf.celpSpecificConfig.RPE_Configuration.value
      = iConf->specConf.celpSpecificConfig.RPE_Configuration.value;
    oConf->specConf.celpSpecificConfig.MPE_Configuration.value
      = iConf->specConf.celpSpecificConfig.MPE_Configuration.value;
    oConf->specConf.celpSpecificConfig.numEnhLayers.value
      = iConf->specConf.celpSpecificConfig.numEnhLayers.value;
    oConf->specConf.celpSpecificConfig.bandwidthScalabilityMode.value
      = iConf->specConf.celpSpecificConfig.bandwidthScalabilityMode.value;
    break;
#ifdef VERSION2
  case ER_CELP :
    oConf->specConf.celpSpecificConfig.excitationMode.value
      = iConf->specConf.celpSpecificConfig.excitationMode.value;
    oConf->specConf.celpSpecificConfig.sampleRateMode.value
      = iConf->specConf.celpSpecificConfig.sampleRateMode.value;
    oConf->specConf.celpSpecificConfig.fineRateControl.value
      = iConf->specConf.celpSpecificConfig.fineRateControl.value;
    oConf->specConf.celpSpecificConfig.silenceCompressionSW.value
      = iConf->specConf.celpSpecificConfig.silenceCompressionSW.value;
    oConf->specConf.celpSpecificConfig.RPE_Configuration.value
      = iConf->specConf.celpSpecificConfig.RPE_Configuration.value;
    oConf->specConf.celpSpecificConfig.MPE_Configuration.value
      = iConf->specConf.celpSpecificConfig.MPE_Configuration.value;
    oConf->specConf.celpSpecificConfig.numEnhLayers.value
      = iConf->specConf.celpSpecificConfig.numEnhLayers.value;
    oConf->specConf.celpSpecificConfig.bandwidthScalabilityMode.value
      = iConf->specConf.celpSpecificConfig.bandwidthScalabilityMode.value;
    break;
  case ER_HVXC:
#endif
  case HVXC:
    oConf->specConf.hvxcSpecificConfig.HVXCvarMode.value
      = iConf->specConf.hvxcSpecificConfig.HVXCvarMode.value;
    oConf->specConf.hvxcSpecificConfig.HVXCrateMode.value
      = iConf->specConf.hvxcSpecificConfig.HVXCrateMode.value;
    oConf->specConf.hvxcSpecificConfig.extensionFlag.value
      = iConf->specConf.hvxcSpecificConfig.extensionFlag.value;
#ifdef VERSION2
#if 0
    oConf->specConf.hvxcSpecificConfig.packMode.value
      = iConf->specConf.hvxcSpecificConfig.packMode.value;
#endif
    oConf->specConf.hvxcSpecificConfig.vrScalFlag.value
      = iConf->specConf.hvxcSpecificConfig.vrScalFlag.value;
#endif
    break;
#ifdef VERSION2
  case ER_HILN:
  case ER_PARA:
    oConf->specConf.paraSpecificConfig.PARAmode.value
      = iConf->specConf.paraSpecificConfig.PARAmode.value;

    oConf->specConf.paraSpecificConfig.HVXCvarMode.value
      = iConf->specConf.paraSpecificConfig.HVXCvarMode.value;
    oConf->specConf.paraSpecificConfig.HVXCrateMode.value
      = iConf->specConf.paraSpecificConfig.HVXCrateMode.value;
    oConf->specConf.paraSpecificConfig.extensionFlag.value
      = iConf->specConf.paraSpecificConfig.extensionFlag.value;
    oConf->specConf.paraSpecificConfig.vrScalFlag.value
      = iConf->specConf.paraSpecificConfig.vrScalFlag.value;

    oConf->specConf.paraSpecificConfig.HILNquantMode.value
      = iConf->specConf.paraSpecificConfig.HILNquantMode.value;
    oConf->specConf.paraSpecificConfig.HILNmaxNumLine.value
      = iConf->specConf.paraSpecificConfig.HILNmaxNumLine.value;
    oConf->specConf.paraSpecificConfig.HILNsampleRateCode.value
      = iConf->specConf.paraSpecificConfig.HILNsampleRateCode.value;
    oConf->specConf.paraSpecificConfig.HILNframeLength.value
      = iConf->specConf.paraSpecificConfig.HILNframeLength.value;
    oConf->specConf.paraSpecificConfig.HILNcontMode.value
      = iConf->specConf.paraSpecificConfig.HILNcontMode.value;

    oConf->specConf.paraSpecificConfig.PARAextensionFlag.value
      = iConf->specConf.paraSpecificConfig.PARAextensionFlag.value;

    oConf->specConf.paraSpecificConfig.HILNenhaLayer.value
      = iConf->specConf.paraSpecificConfig.HILNenhaLayer.value;
    oConf->specConf.paraSpecificConfig.HILNenhaQuantMode.value
      = iConf->specConf.paraSpecificConfig.HILNenhaQuantMode.value;
    break;
#endif
  default :
    CommonExit(-1,"audioDecoderType not implemented");
    break;
  }
}

static void len2idx( STREAM_MUX_CONFIG   *muxConf,
		     PAYLOAD_LENGTH_INFO *lenInfo,
		     int length, int cnt )
{
  int i, idx, len, align, lay;

  lay = muxConf->laySindx[cnt];

  if ( muxConf->frameLengthType[cnt] == 4 ) { /* CELP fixed */
    idx = muxConf->CELPframeLengthTableIndex[cnt];
    if ( lay == 0 ) len = celptb1[idx][0];
    else            len = celptb2[idx][0];
    align = 8 - len%8;
    if (align == 8) align = 0;
    if ( (len+align) != 8*length )
      CommonExit(-1,"4 conversion error in tblconv");
    lenInfo->MuxSlotLengthBits[cnt] = len;
  }
  if ( muxConf->frameLengthType[cnt] == 6 ) { /* HVXC fixed */
    idx = muxConf->HVXCframeLengthTableIndex[cnt];
    len = hvxctbl[idx][0];
    align = 8 - len%8;
    if (align == 8) align = 0;
    if ( (len+align) != 8*length )
      CommonExit(-1,"6 conversion error in tblconv");
    lenInfo->MuxSlotLengthBits[cnt] = len;
  }
  if ( muxConf->frameLengthType[cnt] == 5 ) { /* CELP 1_of_4 */
    idx = muxConf->CELPframeLengthTableIndex[cnt];
    if ( lay == 0 ) {
      lenInfo->MuxSlotLengthCoded[cnt] = 4;
      for ( i = 0; i < 4; i ++ ) {
	len = celptb1[idx][i+1];
	align = 8 - len%8;
	if (align == 8) align = 0;
	if ( (len+align) == 8*length ) {
	  lenInfo->MuxSlotLengthBits[cnt] = len;
	  lenInfo->MuxSlotLengthCoded[cnt] = i;
	  break;
	}
      }
      if ( lenInfo->MuxSlotLengthCoded[cnt] == 4 )
	CommonExit(-1,"5-core conversion error in tblconv");
    } else {
      lenInfo->MuxSlotLengthCoded[cnt] = 4;
      for ( i = 0; i < 4; i ++ ) {
	len = celptb2[idx][i];
	align = 8 - len%8;
	if (align == 8) align = 0;
	if ( (len+align) == 8*length ) {
	  lenInfo->MuxSlotLengthBits[cnt] = len;
	  lenInfo->MuxSlotLengthCoded[cnt] = i;
	}
      }
      if ( lenInfo->MuxSlotLengthCoded[cnt] == 4 )
	CommonExit(-1,"5-enh conversion error in tblconv");
    }
  }
  if ( muxConf->frameLengthType[cnt] == 7 ) { /* HVXC 1_of_4 */
    idx = muxConf->HVXCframeLengthTableIndex[cnt];
    lenInfo->MuxSlotLengthCoded[cnt] = 4;
    for ( i = 0; i < 4; i ++ ) {
      len = hvxctbl[idx][i];
      align = 8 - len%8;
      if (align == 8) align = 0;
      if ( (len+align) == 8*length ) {
	lenInfo->MuxSlotLengthBits[cnt] = len;
	lenInfo->MuxSlotLengthCoded[cnt] = i;
      }
    }
    if ( lenInfo->MuxSlotLengthCoded[cnt] == 4 )
      CommonExit(-1,"7 conversion error in tblconv");
  }
  if ( muxConf->frameLengthType[cnt] == 3 ) { /* CELP 1_of_2 */
    idx = muxConf->CELPframeLengthTableIndex[cnt];
    if ( lay == 0 ) {
      lenInfo->MuxSlotLengthCoded[cnt] = 2;
      for ( i = 0; i < 2; i ++ ) {
	len = celptb1[idx][i*4+1];
	align = 8 - len%8;
	if (align == 8) align = 0;
	if ( (len+align) == 8*length ) {
	  lenInfo->MuxSlotLengthBits[cnt] = len;
	  lenInfo->MuxSlotLengthCoded[cnt] = i;
	}
      }
      if ( lenInfo->MuxSlotLengthCoded[cnt] == 2 )
	CommonExit(-1,"3 conversion error in tblconv");
    } else {
      CommonExit(-1,"conversion error in tblconv");
    }
  }
}

static void idx2len( STREAM_MUX_CONFIG   *muxConf,
		     PAYLOAD_LENGTH_INFO *lenInfo,
		     int cnt )
{
  int idx, len, lay;

  lay = muxConf->laySindx[cnt];

  if ( muxConf->frameLengthType[cnt] == 4 ) { /* CELP fixed */
    idx = muxConf->CELPframeLengthTableIndex[cnt];
    if ( lay == 0 ) len = celptb1[idx][0];
    else            len = celptb2[idx][0];
    lenInfo->MuxSlotLengthBits[cnt] = len;
  }
  if ( muxConf->frameLengthType[cnt] == 6 ) { /* HVXC fixed */
    idx = muxConf->HVXCframeLengthTableIndex[cnt];
    len = hvxctbl[idx][0];
    lenInfo->MuxSlotLengthBits[cnt] = len;
  }
  if ( muxConf->frameLengthType[cnt] == 5 ) { /* CELP 1_of_4 */
    idx = muxConf->CELPframeLengthTableIndex[cnt];
    if ( lay == 0 ) {
      len = celptb1[idx][lenInfo->MuxSlotLengthCoded[cnt]+1];
      lenInfo->MuxSlotLengthBits[cnt] = len;
    } else {
      len = celptb2[idx][lenInfo->MuxSlotLengthCoded[cnt]];
      lenInfo->MuxSlotLengthBits[cnt] = len;
    }
  }
  if ( muxConf->frameLengthType[cnt] == 7 ) { /* HVXC 1_of_4 */
    idx = muxConf->HVXCframeLengthTableIndex[cnt];
    len = hvxctbl[idx][lenInfo->MuxSlotLengthCoded[cnt]];
    lenInfo->MuxSlotLengthBits[cnt] = len;
  }
  if ( muxConf->frameLengthType[cnt] == 3 ) { /* CELP 1_of_2 */
    idx = muxConf->CELPframeLengthTableIndex[cnt];
    len = celptb1[idx][4*(lenInfo->MuxSlotLengthCoded[cnt])+1];
    lenInfo->MuxSlotLengthBits[cnt] = len;
  }
}

static void celpSpecificCnvrt( STREAM_MUX_CONFIG  *muxConf, int cnt,
			       int *index, int *frame_size )
{
  int prog, layer, bcnt;
  CELP_SPECIFIC_CONFIG *celpConf;
#ifdef CORRIGENDUM1
  CELP_SPECIFIC_CONFIG *celpEnhConf;
#else
  CELP_ENH_SPECIFIC_CONFIG *celpEnhConf;
#endif

  prog = muxConf->progSindx[cnt];
  layer = muxConf->laySindx[cnt];
  bcnt = muxConf->streamID[prog][0];

  celpConf = &(muxConf->audioSpecificConfig[bcnt]->specConf.celpSpecificConfig);

  if ( celpConf->excitationMode.value == RegularPulseExc ) {
    *index = 58 + celpConf->RPE_Configuration.value;
    if (celpConf->RPE_Configuration.value == 0) *frame_size = FIFTEEN_MS;
    if (celpConf->RPE_Configuration.value == 1) *frame_size = TEN_MS;
    if (celpConf->RPE_Configuration.value == 2) *frame_size = FIFTEEN_MS;
    if (celpConf->RPE_Configuration.value == 3) *frame_size = FIFTEEN_MS;
  }

  if ( celpConf->excitationMode.value == MultiPulseExc ) {
    if ( celpConf->sampleRateMode.value == fs8kHz) {
      *index = celpConf->MPE_Configuration.value;
      if (celpConf->MPE_Configuration.value < 3) *frame_size = NEC_FRAME40MS;
      if (celpConf->MPE_Configuration.value >= 3 &&
	  celpConf->MPE_Configuration.value < 6) *frame_size = NEC_FRAME30MS;
      if (celpConf->MPE_Configuration.value >= 6 &&
	  celpConf->MPE_Configuration.value < 22) *frame_size = NEC_FRAME20MS;
      if (celpConf->MPE_Configuration.value >= 22 &&
	  celpConf->MPE_Configuration.value < 27) *frame_size = NEC_FRAME10MS;
      if (celpConf->MPE_Configuration.value == 27) *frame_size = NEC_FRAME30MS;
    } else {
      if (celpConf->MPE_Configuration.value < 7) {
	*index = 28 + celpConf->MPE_Configuration.value;
	*frame_size = NEC_FRAME20MS_FRQ16;
      }
      if (celpConf->MPE_Configuration.value >= 8 &&
	  celpConf->MPE_Configuration.value < 16) {
	*index = 35 + (celpConf->MPE_Configuration.value - 8);
	*frame_size = NEC_FRAME20MS_FRQ16;
      }
      if (celpConf->MPE_Configuration.value >= 16 &&
	  celpConf->MPE_Configuration.value < 23) {
	*index = 43 + (celpConf->MPE_Configuration.value - 16);
	*frame_size = NEC_FRAME10MS_FRQ16;
      }
      if (celpConf->MPE_Configuration.value >= 24 &&
	  celpConf->MPE_Configuration.value < 32) {
	*index = 50 + (celpConf->MPE_Configuration.value - 24);
	*frame_size = NEC_FRAME10MS_FRQ16;
      }
    }
  }

  if ( layer > 0 ) {
    if ( celpConf->sampleRateMode.value == fs8kHz) {
      if ( celpConf->MPE_Configuration.value < 3 ) *index = 0;
      if ( celpConf->MPE_Configuration.value >= 3 &&
	   celpConf->MPE_Configuration.value < 6 ) *index = 1;
      if ( celpConf->MPE_Configuration.value >= 6 &&
	   celpConf->MPE_Configuration.value < 22 ) *index = 2;
      if ( celpConf->MPE_Configuration.value >= 22 &&
	   celpConf->MPE_Configuration.value < 27 ) *index = 3;
      if ( celpConf->bandwidthScalabilityMode.value == ON ) {
	if ( layer == (int)(celpConf->numEnhLayers.value+1) ) {
#ifdef CORRIGENDUM1
	  celpEnhConf = &(muxConf->audioSpecificConfig[cnt]->specConf.celpSpecificConfig);
#else
	  celpEnhConf = &(muxConf->audioSpecificConfig[cnt]->specConf.celpEnhSpecificConfig);
#endif
	  if ( celpConf->MPE_Configuration.value < 3 )
	    *index = 4 + celpEnhConf->BWS_Configuration.value;
	  if ( celpConf->MPE_Configuration.value >= 3 &&
	      celpConf->MPE_Configuration.value < 6 )
	    *index = 8 + celpEnhConf->BWS_Configuration.value;
	  if ( celpConf->MPE_Configuration.value >= 6 &&
	      celpConf->MPE_Configuration.value < 22 )
	    *index = 12 + celpEnhConf->BWS_Configuration.value;
	  if ( celpConf->MPE_Configuration.value >= 22 &&
	      celpConf->MPE_Configuration.value < 27 )
	    *index = 16 + celpEnhConf->BWS_Configuration.value;
	  *frame_size *= 2;
	}
      }
    } else {
      if ( celpConf->MPE_Configuration.value < 16 ) *index = 0;
      if ( celpConf->MPE_Configuration.value >= 16 &&
	   celpConf->MPE_Configuration.value < 32 ) *index = 2;
    }
  }
}
