/**********************************************************************
MPEG-4 Audio VM
FlexMux to MP4 converter

This software module was originally developed by

Akira Inoue (Sony Corporation)

and edited by

Heiko Purnhagen (University of Hannover)

in the course of development of the MPEG-2 AAC/MPEG-4 Audio standard
ISO/IEC 13818-7, 14496-1,2 and 3. This software module is an
implementation of a part of one or more MPEG-2 AAC/MPEG-4 Audio tools
as specified by the MPEG-2 AAC/MPEG-4 Audio standard. ISO/IEC gives
users of the MPEG-2 AAC/MPEG-4 Audio standards free license to this
software module or modifications thereof for use in hardware or
software products claiming conformance to the MPEG-2 AAC/ MPEG-4 Audio
standards. Those intending to use this software module in hardware or
software products are advised that this use may infringe existing
patents. The original developer of this software module and his/her
company, the subsequent editors and their companies, and ISO/IEC have
no liability for use of this software module or modifications thereof
in an implementation. Copyright is not released for non MPEG-2
AAC/MPEG-4 Audio conforming products. The original developer retains
full right to use the code for his/her own purpose, assign or donate
the code to a third party and to inhibit third party from using the
code for non MPEG-2 AAC/MPEG-4 Audio conforming products. This
copyright notice must be included in all copies or derivative works.

Copyright (c) 1999.

Source file: fl4_to_mp4.c

 

Required modules:
common_m4a.o            common module
cmdline.o               command line module
bitstream.o             bit stream module
flexmux.o		flexmux module

**********************************************************************/

#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#include "obj_descr.h"           /* structs */

#include "common_m4a.h"          /* common module       */
#include "cmdline.h"             /* command line module */
#include "bitstream.h"           /* bit stream module   */

#include "mp4au.h"               /* frame work common declarations */

#include "flex_mux.h"

#include "lpc_common.h"

#include "MP4Movies.h" 

#define MAXTRACK (50)
#define PROGVER "FlexMux to MP4 Converter V1.2 03-Jan-2001"
#define CVSID "$Id"

static char *outFileName;
static int outFileNameUsed;
static int profileAndLevel;
static int profileAndLevelUsed;
static int sampleDuration;
static int sampleDurationUsed;
static int sampleDurationMS;
static int sampleDurationMSUsed;
static int bufferSize;
static int bufferSizeUsed;
static int debugLevel;

static char *oriPath,*bitPath;
static char *oriExt,*bitExt;

static int*  varArgIdx;

static CmdLinePara paraList[] = {
  {&varArgIdx,NULL,"<bit stream file(s) (- = stdin)>"},
  {NULL,NULL,NULL}
};

static CmdLineSwitch switchList[] = {
  {"h",NULL,NULL,NULL,NULL,"print help"},
  {"o",&outFileName,"%s",NULL,&outFileNameUsed,
   "output file name (- = stdout)"},
  {"du",&sampleDuration,"%d",NULL,&sampleDurationUsed,
   "sample duration per frame[sample/frame]"},
  {"dm",&sampleDurationMS,"%d",NULL,&sampleDurationMSUsed,
   "sample duration in milli-seconds"},
  {"bsize",&bufferSize,"%d",NULL,&bufferSizeUsed,
   "decoder buffer size[byte]"},
  {"pl",&profileAndLevel,"%x","1",&profileAndLevelUsed,
   "audio profile and level\n"
   " 0x01: main profile L1\n"
   " 0x02: main profile L2\n"
   " 0x03: main profile L3\n"
   " 0x04: main profile L4\n"
   " 0x05: scalable profile L1\n"
   " 0x06: scalable profile L2\n"
   " 0x07: scalable profile L3\n"
   " 0x08: scalable profile L4\n"
   " 0x09: speech profile L1\n"
   " 0x0a: speech profile L2\n"
#ifdef VERSION2
   /* audio version 2 profile and level */
#endif
  },
  {"pi",&oriPath,"%s",NULL,NULL,
   "default input file path"},
  {"po",&bitPath,"%s",NULL,NULL,
   "default output file path"},
  {"ei",&oriExt,"%s",".fl4",NULL,
   "input file extension\n"},
  {"eo",&bitExt,"%s",".mp4",NULL,
   "output file extension"},
  {"d",&debugLevel,"%i","0",NULL,"debug level"},
  {NULL,NULL,NULL,NULL,NULL,NULL}
};

static int samplFreqIndex[16] = { 96000,  88200,  64000,  48000,
                                  44100,  32000,  24000,  22050,
                                  16000,  12000,  11025,  8000,
                                  7350,  -1,  -1,  -1 };

static void BsRWBitWrapper(HANDLE_BSBITSTREAM stream,         /* in: bit stream */
                      unsigned long *data,              /* out: bits read/write */
                      int numBit,               /* in: num bits to read */
                      int WriteFlag)
{
  if (WriteFlag){
    BsPutBit(stream,*data,numBit);
  } else {
    BsGetBit(stream,data,numBit);
  }
}

static void advanceAudioSpecConf (HANDLE_BSBITSTREAM  bitStream,
				  ES_DESCRIPTOR *es,
				  int WriteFlag)
{
  AUDIO_SPECIFIC_CONFIG *audSpC;
  
  audSpC = &(es->DecConfigDescr.audioSpecificConfig);

  BsRWBitWrapper(bitStream, &(audSpC->audioDecoderType.value), audSpC->audioDecoderType.length, WriteFlag);
  BsRWBitWrapper(bitStream, &(audSpC->samplingFreqencyIndex.value), audSpC->samplingFreqencyIndex.length,WriteFlag);
  if (audSpC->samplingFreqencyIndex.value == 0xf)
    BsRWBitWrapper(bitStream, &(audSpC->samplingFrequency.value), audSpC->samplingFrequency.length,WriteFlag);
  BsRWBitWrapper(bitStream, &(audSpC->channelConfiguration.value), audSpC->channelConfiguration.length,WriteFlag);
  
  switch (audSpC->audioDecoderType.value) {
  case AAC_SCAL:
  case TWIN_VQ:
#ifdef VERSION2
  case ER_BSAC : /* bsac */
  case ER_TWIN_VQ:
#endif
    initTFspecConf(&(audSpC->specConf.TFSpecificConfig));

    advanceTFspecConf(bitStream,&(audSpC->specConf.TFSpecificConfig),
                      audSpC->audioDecoderType.value, WriteFlag);
    /* added by NTT 20001121 */
#ifdef VERSION2
#ifndef EP_CONFIG_MISSING
    if(audSpC->audioDecoderType.value==ER_TWIN_VQ)
    BsRWBitWrapper(bitStream, &(audSpC->epConfig.value), audSpC->epConfig.length,WriteFlag);
#endif
#endif
    break;

  case CELP :
#ifdef CORRIGENDUM1
    initCelpSpecConf (&(audSpC->specConf.celpSpecificConfig)
#ifdef VERSION2
		      ,audSpC->audioDecoderType.value
#endif
		      ); 
    advanceCelpSpecConf(bitStream,&(audSpC->specConf.celpSpecificConfig),WriteFlag);
#else
    if ( es->streamDependence.value == 0 ) { /*CELP BaseLayer*/
      initCelpSpecConf (&(audSpC->specConf.celpSpecificConfig)
#ifdef VERSION2
			,audSpC->audioDecoderType.value
#endif
                        ); 
      advanceCelpSpecConf(bitStream,&(audSpC->specConf.celpSpecificConfig),WriteFlag);
    } else if ( es->DecConfigDescr.audioSpecificConfig.BWS_on==1 ) { /*CELP BWS Enh.Layer*/
      initCelpEnhSpecConf (&(audSpC->specConf.celpEnhSpecificConfig)); 
      advanceCelpEnhSpecConf(bitStream,&(audSpC->specConf.celpEnhSpecificConfig),WriteFlag);
    } /* No SpecConf for CELP BRS Enh.Layer */
#endif
    break;
#ifdef VERSION2
  case ER_CELP :
#ifdef CORRIGENDUM1
    initCelpSpecConf (&(audSpC->specConf.celpSpecificConfig)
		      ,audSpC->audioDecoderType.value
		      ); 
    advanceCelpSpecConf(bitStream,&(audSpC->specConf.celpSpecificConfig),WriteFlag);
#else
    if ( es->streamDependence.value == 0 ) { /*CELP BaseLayer*/
      initCelpSpecConf (&(audSpC->specConf.celpSpecificConfig)
			,audSpC->audioDecoderType.value
			); 
      advanceErCelpSpecConf(bitStream,&(audSpC->specConf.celpSpecificConfig),WriteFlag);
    } else if ( es->DecConfigDescr.audioSpecificConfig.BWS_on==1 ) { /*CELP BWS Enh.Layer*/
      initCelpEnhSpecConf (&(audSpC->specConf.celpEnhSpecificConfig)); 
      advanceCelpEnhSpecConf(bitStream,&(audSpC->specConf.celpEnhSpecificConfig),WriteFlag);
    } /* No SpecConf for CELP BRS Enh.Layer */
#endif
#ifndef EP_CONFIG_MISSING
    BsRWBitWrapper(bitStream, &(audSpC->epConfig.value), audSpC->epConfig.length,WriteFlag);
#endif
    break;
#endif
  case HVXC:
#ifdef CORRIGENDUM1
    initHvxcSpecConf (&(audSpC->specConf.hvxcSpecificConfig));
    advanceHvxcSpecConf(bitStream,&(audSpC->specConf.hvxcSpecificConfig),WriteFlag);
#else
    if (es->streamDependence.value == 0) { /* HVXC BaseLayer */
      initHvxcSpecConf (&(audSpC->specConf.hvxcSpecificConfig));
      advanceHvxcSpecConf(bitStream,&(audSpC->specConf.hvxcSpecificConfig),WriteFlag);
      }
      else {    /* HVXC Enh.Layer */
        /* No SpecConf */
      }
#endif
    break;
#ifdef VERSION2
  case ER_HVXC:
#ifdef CORRIGENDUM1
    initHvxcSpecConf (&(audSpC->specConf.hvxcSpecificConfig));
    advanceHvxcSpecConf(bitStream,&(audSpC->specConf.hvxcSpecificConfig),WriteFlag);
#else
    if (es->streamDependence.value == 0) { /* HVXC BaseLayer */
      initHvxcSpecConf (&(audSpC->specConf.hvxcSpecificConfig));
      advanceHvxcSpecConf(bitStream,&(audSpC->specConf.hvxcSpecificConfig),WriteFlag);
      }
      else {    /* HVXC Enh.Layer */
        /* No SpecConf */
      }
#endif
#ifndef EP_CONFIG_MISSING
    BsRWBitWrapper(bitStream, &(audSpC->epConfig.value), audSpC->epConfig.length,WriteFlag);
#endif
    break;
#endif
#ifdef VERSION2
  case ER_HILN:
  case ER_PARA:
      initParaSpecConf (&(audSpC->specConf.paraSpecificConfig));
#ifndef CORRIGENDUM1
    if (es->streamDependence.value == 0)
      advanceParaSpecConf(bitStream,&(audSpC->specConf.paraSpecificConfig),WriteFlag);
    else
      advanceParaEnhSpecConf(bitStream,&(audSpC->specConf.paraSpecificConfig),WriteFlag);
#else
    advanceParaSpecConf(bitStream,&(audSpC->specConf.paraSpecificConfig),WriteFlag);
#endif
#ifndef EP_CONFIG_MISSING
    BsRWBitWrapper(bitStream, &(audSpC->epConfig.value), audSpC->epConfig.length,WriteFlag);
#endif
    break;
#endif
  default:
    CommonExit(-1,"audioDecoderType not implemented");
    break;
  }
}


static MP4Err writeAudioDecoderSpecificInfo(
ES_DESCRIPTOR *es,
MP4Handle decoderSpecificInfoH
)
{
  int i;
  u8 *p;
  MP4Err err;
  HANDLE_BSBITBUFFER bitBuf;
  HANDLE_BSBITSTREAM stream;
  AUDIO_SPECIFIC_CONFIG *audSpC;
  int specificInfoLengthBit, specificInfoLengthByte, align;

  audSpC = &(es->DecConfigDescr.audioSpecificConfig);
#if 1
  /* specificInfoLength in current FlexMux bitstream are not set properly 
     so allocate maximum length ... */
  specificInfoLengthBit = 0;
  specificInfoLengthBit += audSpC->audioDecoderType.length;
  specificInfoLengthBit += audSpC->samplingFreqencyIndex.length;
  if (audSpC->samplingFreqencyIndex.value == 0xf)
    specificInfoLengthBit += audSpC->samplingFrequency.length;
  specificInfoLengthBit += audSpC->channelConfiguration.length;
  
  switch(audSpC->audioDecoderType.value) {
  case AAC_SCAL:
  case TWIN_VQ:
#ifdef VERSION2
  case ER_BSAC:
  case ER_TWIN_VQ:
#endif
    initTFspecConf(&(audSpC->specConf.TFSpecificConfig));
/* this element was removed since there is no syntax for TFCodingType specificInfoLengthBit += audSpC->specConf.TFSpecificConfig.TFCodingType.length; */
    specificInfoLengthBit += audSpC->specConf.TFSpecificConfig.frameLength.length;
    specificInfoLengthBit += audSpC->specConf.TFSpecificConfig.dependsOnCoreCoder.length;
    specificInfoLengthBit += audSpC->specConf.TFSpecificConfig.coreCoderDelay.length;
    specificInfoLengthBit += audSpC->specConf.TFSpecificConfig.extension.length;
    /* added by NTT 20001121 */
#ifdef VERSION2
#ifndef EP_CONFIG_MISSING
    if(audSpC->audioDecoderType.value==ER_TWIN_VQ)
      specificInfoLengthBit += audSpC->epConfig.length;
#endif
#endif
    if (audSpC->specConf.TFSpecificConfig.extension.value==0) break;
    if (audSpC->audioDecoderType.value == ER_BSAC) {
      specificInfoLengthBit += audSpC->specConf.TFSpecificConfig.numOfSubFrame.length;
      specificInfoLengthBit += audSpC->specConf.TFSpecificConfig.layer_length.length;
    }
    break;
  case CELP:
#ifdef CORRIGENDUM1
    initCelpSpecConf(&(audSpC->specConf.celpSpecificConfig)
#ifdef VERSION2
		     ,   audSpC->audioDecoderType.value
#endif
		     );
    specificInfoLengthBit += audSpC->specConf.celpSpecificConfig.isBaseLayer.length;
    if ( audSpC->specConf.celpSpecificConfig.isBaseLayer.value == 1 ) { /*CELP BaseLayer*/
#else
    if ( es->streamDependence.value == 0 ) { /*CELP BaseLayer*/
      initCelpSpecConf(&(audSpC->specConf.celpSpecificConfig)
#ifdef VERSION2
		       ,   audSpC->audioDecoderType.value
#endif
		       );
#endif
      specificInfoLengthBit += audSpC->specConf.celpSpecificConfig.excitationMode.length;
      specificInfoLengthBit += audSpC->specConf.celpSpecificConfig.sampleRateMode.length;
      specificInfoLengthBit += audSpC->specConf.celpSpecificConfig.fineRateControl.length;
      if (audSpC->specConf.celpSpecificConfig.excitationMode.value == RegularPulseExc ) {
	specificInfoLengthBit += audSpC->specConf.celpSpecificConfig.RPE_Configuration.length;
      } else {
	specificInfoLengthBit += audSpC->specConf.celpSpecificConfig.MPE_Configuration.length;
	specificInfoLengthBit += audSpC->specConf.celpSpecificConfig.numEnhLayers.length;
	specificInfoLengthBit += audSpC->specConf.celpSpecificConfig.bandwidthScalabilityMode.length;
      }
    } else {
#ifdef CORRIGENDUM1
      specificInfoLengthBit += audSpC->specConf.celpSpecificConfig.isBWSLayer.length;
      if ( audSpC->specConf.celpSpecificConfig.isBWSLayer.value == 1 ) { /*CELP BWS Enh.Layer*/
	specificInfoLengthBit += audSpC->specConf.celpSpecificConfig.BWS_Configuration.length;
      } else { /*CELP BRS Enh.Layer*/
	specificInfoLengthBit += audSpC->specConf.celpSpecificConfig.CELP_BRS_id.length;
      }
#else
      if ( es->DecConfigDescr.audioSpecificConfig.BWS_on==1 ) { /*CELP BWS Enh.Layer*/
	initCelpEnhSpecConf (&(audSpC->specConf.celpEnhSpecificConfig)); 
	specificInfoLengthBit += audSpC->specConf.celpEnhSpecificConfig.BWS_Configuration.length;
      }
#endif
    }
    break;
#ifdef VERSION2
  case ER_CELP:
#ifdef CORRIGENDUM1
    initCelpSpecConf(&(audSpC->specConf.celpSpecificConfig)
		     ,   audSpC->audioDecoderType.value
		     );
    specificInfoLengthBit += audSpC->specConf.celpSpecificConfig.isBaseLayer.length;
    if ( audSpC->specConf.celpSpecificConfig.isBaseLayer.value == 1 ) { /*CELP BaseLayer*/
#else
    if ( es->streamDependence.value == 0 ) { /*CELP BaseLayer*/
      initCelpSpecConf(&(audSpC->specConf.celpSpecificConfig)
		       ,audSpC->audioDecoderType.value
		       );
#endif
      specificInfoLengthBit += audSpC->specConf.celpSpecificConfig.excitationMode.length;
      specificInfoLengthBit += audSpC->specConf.celpSpecificConfig.sampleRateMode.length;
      specificInfoLengthBit += audSpC->specConf.celpSpecificConfig.fineRateControl.length;
      specificInfoLengthBit += audSpC->specConf.celpSpecificConfig.silenceCompressionSW.length;
      if (audSpC->specConf.celpSpecificConfig.excitationMode.value == RegularPulseExc ) {
	specificInfoLengthBit += audSpC->specConf.celpSpecificConfig.RPE_Configuration.length;
      } else {
	specificInfoLengthBit += audSpC->specConf.celpSpecificConfig.MPE_Configuration.length;
	specificInfoLengthBit += audSpC->specConf.celpSpecificConfig.numEnhLayers.length;
	specificInfoLengthBit += audSpC->specConf.celpSpecificConfig.bandwidthScalabilityMode.length;
      }
    } else {
#ifdef CORRIGENDUM1
      specificInfoLengthBit += audSpC->specConf.celpSpecificConfig.isBWSLayer.length;
      if ( audSpC->specConf.celpSpecificConfig.isBWSLayer.value == 1 ) { /*CELP BWS Enh.Layer*/
	specificInfoLengthBit += audSpC->specConf.celpSpecificConfig.BWS_Configuration.length;
      } else { /*CELP BRS Enh.Layer*/
	specificInfoLengthBit += audSpC->specConf.celpSpecificConfig.CELP_BRS_id.length;
      }
#else
      if ( es->DecConfigDescr.audioSpecificConfig.BWS_on==1 ) { /*CELP BWS Enh.Layer*/
	initCelpEnhSpecConf (&(audSpC->specConf.celpEnhSpecificConfig)); 
	specificInfoLengthBit += audSpC->specConf.celpEnhSpecificConfig.BWS_Configuration.length;
      }
#endif
    }
#ifndef EP_CONFIG_MISSING
    specificInfoLengthBit += audSpC->epConfig.length;
#endif
    break;
#endif
  case HVXC:
    initHvxcSpecConf(&(audSpC->specConf.hvxcSpecificConfig));
#ifdef CORRIGENDUM1
    specificInfoLengthBit += audSpC->specConf.hvxcSpecificConfig.isBaseLayer.length;
#endif
    specificInfoLengthBit += audSpC->specConf.hvxcSpecificConfig.HVXCvarMode.length;
    specificInfoLengthBit += audSpC->specConf.hvxcSpecificConfig.HVXCrateMode.length;
    specificInfoLengthBit += audSpC->specConf.hvxcSpecificConfig.extensionFlag.length;
    break;
#ifdef VERSION2
  case ER_HVXC:
    initHvxcSpecConf(&(audSpC->specConf.hvxcSpecificConfig));
#ifdef CORRIGENDUM1
    specificInfoLengthBit += audSpC->specConf.hvxcSpecificConfig.isBaseLayer.length;
#endif
    specificInfoLengthBit += audSpC->specConf.hvxcSpecificConfig.HVXCvarMode.length;
    specificInfoLengthBit += audSpC->specConf.hvxcSpecificConfig.HVXCrateMode.length;
    specificInfoLengthBit += audSpC->specConf.hvxcSpecificConfig.extensionFlag.length;
#if 0
    specificInfoLengthBit += audSpC->specConf.hvxcSpecificConfig.packMode.length;
#endif
    specificInfoLengthBit += audSpC->specConf.hvxcSpecificConfig.vrScalFlag.length;
#ifndef EP_CONFIG_MISSING
    specificInfoLengthBit += audSpC->epConfig.length;
#endif
    break;
#endif
#ifdef VERSION2
  case ER_HILN:
  case ER_PARA:
#ifdef CORRIGENDUM1
    if (audSpC->specConf.paraSpecificConfig.isBaseLayer.value == 1)
#else
    if ( es->streamDependence.value == 0 )
#endif
    {
      initParaSpecConf(&(audSpC->specConf.paraSpecificConfig));
#ifdef CORRIGENDUM1
      specificInfoLengthBit += audSpC->specConf.paraSpecificConfig.isBaseLayer.length;
#endif
      specificInfoLengthBit += audSpC->specConf.paraSpecificConfig.PARAmode.length;
      if (audSpC->specConf.paraSpecificConfig.PARAmode.value != 1) {
	specificInfoLengthBit += audSpC->specConf.paraSpecificConfig.HVXCvarMode.length;
	specificInfoLengthBit += audSpC->specConf.paraSpecificConfig.HVXCrateMode.length;
	specificInfoLengthBit += audSpC->specConf.paraSpecificConfig.extensionFlag.length;
	if (audSpC->specConf.paraSpecificConfig.extensionFlag.value)
	  specificInfoLengthBit += audSpC->specConf.paraSpecificConfig.vrScalFlag.length;
      }
      if (audSpC->specConf.paraSpecificConfig.PARAmode.value != 0) {
	specificInfoLengthBit += audSpC->specConf.paraSpecificConfig.HILNquantMode.length;
	specificInfoLengthBit += audSpC->specConf.paraSpecificConfig.HILNmaxNumLine.length;
	specificInfoLengthBit += audSpC->specConf.paraSpecificConfig.HILNsampleRateCode.length;
	specificInfoLengthBit += audSpC->specConf.paraSpecificConfig.HILNframeLength.length;
	specificInfoLengthBit += audSpC->specConf.paraSpecificConfig.HILNcontMode.length;
      }
      specificInfoLengthBit += audSpC->specConf.paraSpecificConfig.PARAextensionFlag.length;
    }
    else {
      initParaSpecConf(&(audSpC->specConf.paraSpecificConfig));
#ifdef CORRIGENDUM1
      specificInfoLengthBit += audSpC->specConf.paraSpecificConfig.isBaseLayer.length;
#endif
      specificInfoLengthBit += audSpC->specConf.paraSpecificConfig.HILNenhaLayer.length;
      if (audSpC->specConf.paraSpecificConfig.HILNenhaLayer.value)
	specificInfoLengthBit += audSpC->specConf.paraSpecificConfig.HILNenhaQuantMode.length;
    }
#ifndef EP_CONFIG_MISSING
    specificInfoLengthBit += audSpC->epConfig.length;
#endif
    break;
#endif
  default:
    CommonExit(-1,"audioDecoderType not implemented");
    break;
  }
  specificInfoLengthByte = specificInfoLengthBit / 8;
  align = 8 - specificInfoLengthBit % 8;
  if (align == 8) align = 0;
  if (align != 0) {
    specificInfoLengthByte += 1;
  }
#else
  /* in fact, signaled "specificInfoLength" should be referred */
  specificInfoLengthByte = es->DecConfigDescr.specificInfoLength.value;
  specificInfoLengthBit = 8*es->DecConfigDescr.specificInfoLength.value;
#endif
  
  if (debugLevel >= 3) {
    printf("write specificInfo %dbyte(%d bit)\n", specificInfoLengthByte, specificInfoLengthBit);
  }
  
  bitBuf = BsAllocBuffer(specificInfoLengthBit);
  stream = BsOpenBufferWrite(bitBuf);

  err = MP4SetHandleSize(decoderSpecificInfoH, 
			 (u32)(specificInfoLengthByte+2));
  if (err) CommonExit(1,"MP4 API error(%d)", err);
  p = (u8*) *decoderSpecificInfoH;
  *p++ = 0x05; 				/* DecSpecificInfoTag, 14496-1, Sec 8.2.2.2, Table 1*/
  *p++ = specificInfoLengthByte; 	/* sizeOfInstance, 14496-1, Sec 12.3.3 */

  advanceAudioSpecConf(stream, es, 1);
    
  for (i=0; i<specificInfoLengthByte; i++) {
    *p++ = BsBufferGetDataBegin(bitBuf)[i];
    if (debugLevel >= 5) printf("data[%d]=0x%02x\n", i, *(p-1));
  }
    
  BsFreeBuffer(bitBuf);

  return err;
}

static void debugPrintObjDescr(
OBJECT_DESCRIPTOR *od
)
{
  unsigned int x;
  ES_DESCRIPTOR *es;
  AUDIO_SPECIFIC_CONFIG *audSpC;
  
  printf("objDescr.ODLength = %ld\n", od->ODLength.value);
  printf("objDescr.ODescrId = %ld\n", od->ODescrId.value);
  printf("objDescr.streamCount = %ld\n", od->streamCount.value);
  printf("objDescr.extensionFlag = %ld\n", od->extensionFlag.value);
  for (x=0;x<od->streamCount.value;x++){
    es = od->ESDescriptor[x];
    
    printf("\nESDescriptor.ESNumber: %ld \n",es->ESNumber.value);
    printf("  ESDescriptor.streamDependence: %ld \n",
	   es->streamDependence.value);
    printf("  ESDescriptor.dependsOn_Es_number: %ld \n",
	   es->dependsOn_Es_number.value);
    printf("  ESDescriptor.DecConfigDescr.profileAndLevelIndication: %ld \n",
	   es->DecConfigDescr.profileAndLevelIndication.value);
    printf("  ESDescriptor.DecConfigDescr.streamType: %ld \n",
	   es->DecConfigDescr.streamType.value);
    printf("  ESDescriptor.DecConfigDescr.specificInfoFlag: %ld \n",
	   es->DecConfigDescr.specificInfoFlag.value);
    printf("  ESDescriptor.DecConfigDescr.bufferSizeDB: %ld \n",
	   es->DecConfigDescr.bufferSizeDB.value);
    printf("  ESDescriptor.DecConfigDescr.maxBitrate: %ld \n",
	   es->DecConfigDescr.maxBitrate.value);
    printf("  ESDescriptor.DecConfigDescr.avgBitrate: %ld \n",
	   es->DecConfigDescr.avgBitrate.value);
    printf("  ESDescriptor.DecConfigDescr.specificInfoLength: %ld\n",
	   es->DecConfigDescr.specificInfoLength.value);
    printf("  ESDescriptor.DecConfigDescr.audioSpecificConfig.audioDecoderType: %ld \n",
	   es->DecConfigDescr.audioSpecificConfig.audioDecoderType.value);
    printf("  ESDescriptor.DecConfigDescr.audioSpecificConfig.samplingFreqencyIndex: %ld \n",
	   es->DecConfigDescr.audioSpecificConfig.samplingFreqencyIndex.value);
    if (es->DecConfigDescr.audioSpecificConfig.samplingFreqencyIndex.value == 0xf)
      printf("  ESDescriptor.DecConfigDescr.audioSpecificConfig.samplingFrequency: %ld \n",
	     es->DecConfigDescr.audioSpecificConfig.samplingFrequency.value);
    printf("  ESDescriptor.DecConfigDescr.audioSpecificConfig.channelConfiguration: %ld \n",
	   es->DecConfigDescr.audioSpecificConfig.channelConfiguration.value);

    audSpC = &(es->DecConfigDescr.audioSpecificConfig);
    
    switch(audSpC->audioDecoderType.value) {
    case AAC_SCAL:
    case TWIN_VQ:
#ifdef VERSION2
    case ER_TWIN_VQ:
    case ER_BSAC:
#endif
      /* this elemement was removed 
      printf("  ESDescriptor.DecConfigDescr.audioSpecificConfig.TFSpecificConfig.TFCodingType: %ld \n",
	     audSpC->specConf.TFSpecificConfig.TFCodingType.value); */
      printf("  ESDescriptor.DecConfigDescr.audioSpecificConfig.TFSpecificConfig.frameLength: %ld \n",
	     audSpC->specConf.TFSpecificConfig.frameLength.value);
      printf("  ESDescriptor.DecConfigDescr.audioSpecificConfig.TFSpecificConfig.dependsOnCoreCoder: %ld \n",
	     audSpC->specConf.TFSpecificConfig.dependsOnCoreCoder.value);
      if (audSpC->specConf.TFSpecificConfig.dependsOnCoreCoder.value != 0) { 
	printf("  ESDescriptor.DecConfigDescr.audioSpecificConfig.TFSpecificConfig.coreCoderDelay: %ld \n",
	       audSpC->specConf.TFSpecificConfig.coreCoderDelay.value);
      }
      printf("  ESDescriptor.DecConfigDescr.audioSpecificConfig.TFSpecificConfig.extension: %ld \n",
	     audSpC->specConf.TFSpecificConfig.extension.value);

	  if (audSpC->specConf.TFSpecificConfig.extension.value == 0) break;
      if (audSpC->audioDecoderType.value == ER_BSAC) {
        printf("  ESDescriptor.DecConfigDescr.audioSpecificConfig.TFSpecificConfig.numOfSubFrame: %ld \n",
	     audSpC->specConf.TFSpecificConfig.numOfSubFrame.value);
        printf("  ESDescriptor.DecConfigDescr.audioSpecificConfig.TFSpecificConfig.layer_length: %ld \n",
	     audSpC->specConf.TFSpecificConfig.layer_length.value);
      }
      printf("  ESDescriptor.DecConfigDescr.audioSpecificConfig.TFSpecificConfig.extension3: %ld \n",
	     audSpC->specConf.TFSpecificConfig.extension3.value);

      break;
    case CELP:
#ifdef CORRIGENDUM1
      printf("  ESDescriptor.DecConfigDescr.audioSpecificConfig.celpSpecificConfig.isBaseLayer: %ld \n",
	     audSpC->specConf.celpSpecificConfig.isBaseLayer.value);
      if (audSpC->specConf.celpSpecificConfig.isBaseLayer.value == 1) { /*CELP BaseLayer*/
#else
      if (es->streamDependence.value == 0) { /*CELP BaseLayer*/
#endif
	printf("  ESDescriptor.DecConfigDescr.audioSpecificConfig.celpSpecificConfig.excitationMode: %ld \n",
	       audSpC->specConf.celpSpecificConfig.excitationMode.value);
	printf("  ESDescriptor.DecConfigDescr.audioSpecificConfig.celpSpecificConfig.sampleRateMode: %ld \n",
	       audSpC->specConf.celpSpecificConfig.sampleRateMode.value);
	printf("  ESDescriptor.DecConfigDescr.audioSpecificConfig.celpSpecificConfig.fineRateControl: %ld \n",
	       audSpC->specConf.celpSpecificConfig.fineRateControl.value);
	if (audSpC->specConf.celpSpecificConfig.excitationMode.value == RegularPulseExc) {
	  printf("  ESDescriptor.DecConfigDescr.audioSpecificConfig.celpSpecificConfig.RPE_Configuration: %ld \n",
		 audSpC->specConf.celpSpecificConfig.RPE_Configuration.value);
	}
	if (audSpC->specConf.celpSpecificConfig.excitationMode.value == MultiPulseExc) {
	  printf("  ESDescriptor.DecConfigDescr.audioSpecificConfig.celpSpecificConfig.MPE_Configuration: %ld \n",
		 audSpC->specConf.celpSpecificConfig.MPE_Configuration.value);
	  printf("  ESDescriptor.DecConfigDescr.audioSpecificConfig.celpSpecificConfig.numEnhLayers: %ld \n",
		 audSpC->specConf.celpSpecificConfig.numEnhLayers.value);
	  printf("  ESDescriptor.DecConfigDescr.audioSpecificConfig.celpSpecificConfig.bandwidthScalabilityMode: %ld \n",
		 audSpC->specConf.celpSpecificConfig.bandwidthScalabilityMode.value);
	}
      } else {
#ifdef CORRIGENDUM1
	printf("  ESDescriptor.DecConfigDescr.audioSpecificConfig.celpSpecificConfig.isBWSLayer: %ld \n",
	       audSpC->specConf.celpSpecificConfig.isBWSLayer.value);
	if (audSpC->specConf.celpSpecificConfig.isBWSLayer.value == 1) { /*CELP BWS Enh.Layer*/
	  printf("  ESDescriptor.DecConfigDescr.audioSpecificConfig.celpSpecificConfig.BWS_Configuration: %ld \n",
		 audSpC->specConf.celpSpecificConfig.BWS_Configuration.value);
	} else { /*CELP BRS Enh.Layer*/
	  printf("  ESDescriptor.DecConfigDescr.audioSpecificConfig.celpSpecificConfig.CELP_BRS_id: %ld \n",
		 audSpC->specConf.celpSpecificConfig.CELP_BRS_id.value);
	}
#else
	if ( es->DecConfigDescr.audioSpecificConfig.BWS_on==1 ) { /*CELP BWS Enh.Layer*/
	  printf("  ESDescriptor.DecConfigDescr.audioSpecificConfig.celpEnhSpecificConfig.BWS_Configuration: %ld \n",
		 audSpC->specConf.celpEnhSpecificConfig.BWS_Configuration.value);
	} /* No SpecConf for CELP BRS Enh.Layer */
#endif
      }
      break;
#ifdef VERSION2      
    case ER_CELP:
#ifdef CORRIGENDUM1
      printf("  ESDescriptor.DecConfigDescr.audioSpecificConfig.celpSpecificConfig.isBaseLayer: %ld \n",
	     audSpC->specConf.celpSpecificConfig.isBaseLayer.value);
      if (audSpC->specConf.celpSpecificConfig.isBaseLayer.value == 1) /*CELP BaseLayer*/
#else
      if (es->streamDependence.value == 0) /*CELP BaseLayer*/
#endif
      {
	printf("  ESDescriptor.DecConfigDescr.audioSpecificConfig.celpSpecificConfig.excitationMode: %ld \n",
	       audSpC->specConf.celpSpecificConfig.excitationMode.value);
	printf("  ESDescriptor.DecConfigDescr.audioSpecificConfig.celpSpecificConfig.sampleRateMode: %ld \n",
	       audSpC->specConf.celpSpecificConfig.sampleRateMode.value);
	printf("  ESDescriptor.DecConfigDescr.audioSpecificConfig.celpSpecificConfig.fineRateControl: %ld \n",
	       audSpC->specConf.celpSpecificConfig.fineRateControl.value);
	printf("  ESDescriptor.DecConfigDescr.audioSpecificConfig.celpSpecificConfig.silenceCompressionSW: %ld \n",
	       audSpC->specConf.celpSpecificConfig.silenceCompressionSW.value);
	if (audSpC->specConf.celpSpecificConfig.excitationMode.value == RegularPulseExc) {
	  printf("  ESDescriptor.DecConfigDescr.audioSpecificConfig.celpSpecificConfig.RPE_Configuration: %ld \n",
		 audSpC->specConf.celpSpecificConfig.RPE_Configuration.value);
	}
	if (audSpC->specConf.celpSpecificConfig.excitationMode.value == MultiPulseExc) {
	  printf("  ESDescriptor.DecConfigDescr.audioSpecificConfig.celpSpecificConfig.MPE_Configuration: %ld \n",
		 audSpC->specConf.celpSpecificConfig.MPE_Configuration.value);
	  printf("  ESDescriptor.DecConfigDescr.audioSpecificConfig.celpSpecificConfig.numEnhLayers: %ld \n",
		 audSpC->specConf.celpSpecificConfig.numEnhLayers.value);
	  printf("  ESDescriptor.DecConfigDescr.audioSpecificConfig.celpSpecificConfig.bandwidthScalabilityMode: %ld \n",
		 audSpC->specConf.celpSpecificConfig.bandwidthScalabilityMode.value);
	}
      } else {
#ifdef CORRIGENDUM1
	printf("  ESDescriptor.DecConfigDescr.audioSpecificConfig.celpSpecificConfig.isBWSLayer: %ld \n",
	       audSpC->specConf.celpSpecificConfig.isBWSLayer.value);
	if (audSpC->specConf.celpSpecificConfig.isBWSLayer.value == 1) { /*CELP BWS Enh.Layer*/
	  printf("  ESDescriptor.DecConfigDescr.audioSpecificConfig.celpSpecificConfig.BWS_Configuration: %ld \n",
		 audSpC->specConf.celpSpecificConfig.BWS_Configuration.value);
	} else { /*CELP BRS Enh.Layer*/
	  printf("  ESDescriptor.DecConfigDescr.audioSpecificConfig.celpSpecificConfig.CELP_BRS_id: %ld \n",
		 audSpC->specConf.celpSpecificConfig.CELP_BRS_id.value);
	}
#else
	if ( es->DecConfigDescr.audioSpecificConfig.BWS_on==1 ) { /*CELP BWS Enh.Layer*/
	  printf("  ESDescriptor.DecConfigDescr.audioSpecificConfig.celpEnhSpecificConfig.BWS_Configuration: %ld \n",
		 audSpC->specConf.celpEnhSpecificConfig.BWS_Configuration.value);
	} /* No SpecConf for CELP BRS Enh.Layer */
#endif
      }
#ifndef EP_CONFIG_MISSING
      printf("  ESDescriptor.DecConfigDescr.audioSpecificConfig.epConfig: %ld \n", audSpC->epConfig.value);
#endif
      break;
#endif
    case HVXC:
#ifdef VERSION2
    case ER_HVXC:
#endif
#ifdef CORRIGENDUM1
      printf("  ESDescriptor.DecConfigDescr.audioSpecificConfig.hvxcSpecificConfig.isBaseLayer: %ld \n",
	     audSpC->specConf.hvxcSpecificConfig.isBaseLayer.value);
      if (audSpC->specConf.hvxcSpecificConfig.isBaseLayer.value == 1) /*BaseLayer*/
#else
      if (es->streamDependence.value == 0)
#endif
      {
	printf("  ESDescriptor.DecConfigDescr.audioSpecificConfig.hvxcSpecificConfig.HVXCvarMode: %ld \n",
	       audSpC->specConf.hvxcSpecificConfig.HVXCvarMode.value);
	printf("  ESDescriptor.DecConfigDescr.audioSpecificConfig.hvxcSpecificConfig.HVXCrateMode: %ld \n",
	       audSpC->specConf.hvxcSpecificConfig.HVXCrateMode.value);
	printf("  ESDescriptor.DecConfigDescr.audioSpecificConfig.hvxcSpecificConfig.extensionFlag: %ld \n",
	       audSpC->specConf.hvxcSpecificConfig.extensionFlag.value);
#ifdef VERSION2
	if (audSpC->specConf.hvxcSpecificConfig.extensionFlag.value) {
#if 0
	  printf("  ESDescriptor.DecConfigDescr.audioSpecificConfig.hvxcSpecificConfig.packMode: %ld \n",
		 audSpC->specConf.hvxcSpecificConfig.packMode.value);
#endif
	  printf("  ESDescriptor.DecConfigDescr.audioSpecificConfig.hvxcSpecificConfig.vrScalFlag: %ld \n",
		 audSpC->specConf.hvxcSpecificConfig.vrScalFlag.value);
	}
#ifndef EP_CONFIG_MISSING
      printf("  ESDescriptor.DecConfigDescr.audioSpecificConfig.epConfig: %ld \n", audSpC->epConfig.value);
#endif
#endif
      }
      break;
#ifdef VERSION2
    case ER_HILN:
    case ER_PARA:
#ifdef CORRIGENDUM1
      printf("  ESDescriptor.DecConfigDescr.audioSpecificConfig.specConf.paraSpecificConfig.isBaseLayer: %ld \n",
	     audSpC->specConf.paraSpecificConfig.isBaseLayer.value);
      if (audSpC->specConf.paraSpecificConfig.isBaseLayer.value == 1) /*BaseLayer*/
#else
      if (es->streamDependence.value == 0) /*BaseLayer*/
#endif
      {
	printf("  ESDescriptor.DecConfigDescr.audioSpecificConfig.specConf.paraSpecificConfig.PARAmode: %ld \n",
	       audSpC->specConf.paraSpecificConfig.PARAmode.value);
	
	if (audSpC->specConf.paraSpecificConfig.PARAmode.value != 1) {
	  printf("  ESDescriptor.DecConfigDescr.audioSpecificConfig.specConf.paraSpecificConfig.HVXCvarMode: %ld \n",
		 audSpC->specConf.paraSpecificConfig.HVXCvarMode.value);
	  printf("  ESDescriptor.DecConfigDescr.audioSpecificConfig.specConf.paraSpecificConfig.HVXCrateMode: %ld \n",
		 audSpC->specConf.paraSpecificConfig.HVXCrateMode.value);
	  printf("  ESDescriptor.DecConfigDescr.audioSpecificConfig.specConf.paraSpecificConfig.extensionFlag: %ld \n",
		 audSpC->specConf.paraSpecificConfig.extensionFlag.value);
	  if (audSpC->specConf.paraSpecificConfig.extensionFlag.value) {
	    printf("  ESDescriptor.DecConfigDescr.audioSpecificConfig.specConf.paraSpecificConfig.vrScalFlag: %ld \n",
		   audSpC->specConf.paraSpecificConfig.vrScalFlag.value);
	  }
	}
	
	if (audSpC->specConf.paraSpecificConfig.PARAmode.value != 0) {
	  printf("  ESDescriptor.DecConfigDescr.audioSpecificConfig.specConf.paraSpecificConfig.HILNquantMode: %ld \n",
		 audSpC->specConf.paraSpecificConfig.HILNquantMode.value);
	  printf("  ESDescriptor.DecConfigDescr.audioSpecificConfig.specConf.paraSpecificConfig.HILNmaxNumLine: %ld \n",
		 audSpC->specConf.paraSpecificConfig.HILNmaxNumLine.value);
	  printf("  ESDescriptor.DecConfigDescr.audioSpecificConfig.specConf.paraSpecificConfig.HILNsampleRateCode: %ld \n",
		 audSpC->specConf.paraSpecificConfig.HILNsampleRateCode.value);
	  printf("  ESDescriptor.DecConfigDescr.audioSpecificConfig.specConf.paraSpecificConfig.HILNframeLength: %ld \n",
		 audSpC->specConf.paraSpecificConfig.HILNframeLength.value);
	  printf("  ESDescriptor.DecConfigDescr.audioSpecificConfig.specConf.paraSpecificConfig.HILNcontMode: %ld \n",
		 audSpC->specConf.paraSpecificConfig.HILNcontMode.value);
	}
	
	printf("  ESDescriptor.DecConfigDescr.audioSpecificConfig.specConf.paraSpecificConfig.PARAextensionFlag: %ld \n",
	       audSpC->specConf.paraSpecificConfig.PARAextensionFlag.value);
      }
      else {
	printf("  ESDescriptor.DecConfigDescr.audioSpecificConfig.specConf.paraSpecificConfig.HILNenhaLayer: %ld \n",
	       audSpC->specConf.paraSpecificConfig.HILNenhaLayer.value);
	if (audSpC->specConf.paraSpecificConfig.HILNenhaLayer.value) {
	  printf("  ESDescriptor.DecConfigDescr.audioSpecificConfig.specConf.paraSpecificConfig.HILNenhaQuantMode: %ld \n",
audSpC->specConf.paraSpecificConfig.HILNenhaQuantMode.value);
	}
      }
#ifndef EP_CONFIG_MISSING
      printf("  ESDescriptor.DecConfigDescr.audioSpecificConfig.epConfig: %ld \n", audSpC->epConfig.value);
#endif
      break;
#endif
    default:
      CommonExit(-1,"audioDecoderType not implemented");
      break;
    }
  }
}

static MP4Err FlexMux_to_MP4(char *inBitFileName, char *outBitFileName)
{
  char               *info;
  HANDLE_BSBITSTREAM inBitStream;

  FRAME_DATA         frameData;
  OBJECT_DESCRIPTOR  objDescr;
  unsigned int       index;               /* FlexMux Channel */
  unsigned long      length[MAXTRACK];           /* SL-Packet length */

  unsigned int x;
  unsigned long align,dummy;
  
  ES_DESCRIPTOR *es = NULL;

  unsigned int i;
  
  /* MP4 stuffs */
  MP4Err err;
  MP4Movie moov;
  MP4Track trak[MAXTRACK];
  MP4Media media[MAXTRACK];
  u8 OD_profileAndLevel;
  u8 scene_profileAndLevel;
  u8 audio_profileAndLevel;
  u8 visual_profileAndLevel;
  u8 graphics_profileAndLevel;
  u32 initialObjectDescriptorID;
  u32 timeScale[MAXTRACK];
  u64 mediaDuration;
  u32 trackID;
  u32 outRefNum;
  
  MP4Handle sampleEntryH[MAXTRACK];
  MP4Handle sampleDataH;
  MP4Handle sampleDurationH;
  MP4Handle sampleSizeH;
  MP4Handle decoderSpecificInfoH;
  u32 objectTypeIndication;
  u32 streamType;
  u32 decoderBufferSize;
  u32 maxBitrate;
  u32 avgBitrate;

  u8 *p;

  u32 trackCount;

  int ESNumberOffset = 0;	/* for FlexMux bitstreams using ESNumber = 0 */
    
  /* init */
  BsInit(MAXREADAHEAD,0,1);	/* second argument is debug level */
  info = NULL;
  
  /* open bit stream file */
  inBitStream = BsOpenFileRead(inBitFileName,MP4_MAGIC,&info);
  if (inBitStream==NULL) {
    CommonWarning("error opening bit stream file %s",inBitFileName);
    err = -1; /* HP20001012 */
    goto bail;
  }

  /* read input bit stream header (FlexMux) */
  initObjDescr(&objDescr,0);
  if (BsGetBit(inBitStream,&(objDescr.ODLength.value),objDescr.ODLength.length) )
    CommonExit(1,"Read: error reading bit stream header (FlexMux)");

  advanceODescr(inBitStream, &objDescr, 0);	/* read Object Descriptor */
  for (x=0;x<objDescr.streamCount.value;x++){
    initESDescr(&(objDescr.ESDescriptor[x]));

#ifndef CORRIGENDUM1
    if ( ( x>0 ) && (objDescr.ESDescriptor[0]->DecConfigDescr.audioSpecificConfig.specConf.celpSpecificConfig.bandwidthScalabilityMode.value==1) &&
	 (x == (objDescr.ESDescriptor[0]->DecConfigDescr.audioSpecificConfig.specConf.celpSpecificConfig.numEnhLayers.value + 1)) ){
      objDescr.ESDescriptor[x]->DecConfigDescr.audioSpecificConfig.BWS_on=1;
    }
#endif
    advanceESDescr(inBitStream,objDescr.ESDescriptor[x],0);	/* read ES Descriptor */
    frameData.layer[x].bitBuf = BsAllocBuffer(64000);
  }

  if (debugLevel >= 3) debugPrintObjDescr(&objDescr);
    
  align = 8 - BsCurrentBit(inBitStream) % 8;
  if (align == 8) align = 0;
  BsGetBit(inBitStream,&(dummy),align) ;
  frameData.od = &objDescr;

  err = MP4NoErr;
  initialObjectDescriptorID = 1;
  OD_profileAndLevel = 0xff; 		/* none required */
  scene_profileAndLevel = 0xff;	/* none required */
  if (profileAndLevelUsed)
    audio_profileAndLevel = (u8)profileAndLevel;	/* value specified by command line switch */
  else
    audio_profileAndLevel = 0x01; 	/* main profile L1 */
  visual_profileAndLevel = 0xff; 	/* none required */
  graphics_profileAndLevel = 0xff; 	/* none required */

  err = MP4NewMovie(&moov,
		    initialObjectDescriptorID, 
		    OD_profileAndLevel,
		    scene_profileAndLevel,
		    audio_profileAndLevel,
		    visual_profileAndLevel,
		    graphics_profileAndLevel); if (err) goto bail;

  for (x=0;x<objDescr.streamCount.value;x++){
    es = objDescr.ESDescriptor[x];
    /* sampling frequency */
    if (es->DecConfigDescr.audioSpecificConfig.samplingFreqencyIndex.value == 0xf)
      timeScale[x] = es->DecConfigDescr.audioSpecificConfig.samplingFrequency.value;
    else
      timeScale[x] = samplFreqIndex[es->DecConfigDescr.audioSpecificConfig.samplingFreqencyIndex.value];
    
    /* if ESNumber = 0, force trackID to be (ESNumber+1) */
    if (es->ESNumber.value == 0) ESNumberOffset = 1;	
    trackID = es->ESNumber.value + ESNumberOffset;

    if (debugLevel >= 3) printf("trackID=%d, timeScale=%d\n", trackID, timeScale[x]);

    err = MP4NewMovieTrackWithID(moov, MP4NewTrackIsAudio, trackID, &trak[x]); if (err) goto bail;
    err = MP4NewTrackMedia(trak[x], &media[x], MP4AudioHandlerType, timeScale[x], NULL); if (err) goto bail;

    if (es->streamDependence.value) {
      /* Elementary Stream dependency */
      err = MP4AddTrackReferenceWithID(trak[x], 
#if 1	/* in current FlexMux bitstream, dependsON_Es_number is not correctly signaled.
	   sometimes 0 is signaled(ES number must start from 1) */
#if 1	/* in the "legacy" FlexMux bitstresm, enhancement layer depends on the previous layer 
	   the order of layers is determined : 
	   track 1(x=0): base layer (no dependency)
	   track 2(x=1): enhancement layer 1 depending on base layer (track 1)  
	   track 3(x=2): enhancement layer 2 depending on ehn.layer 1(track 2) 
	   track 4(x=3): enhancement layer 3 depending on ehn.layer 2(track 3) 
	   and so on ...	*/
				       x,	
#else				       
				       1,	
#endif
#else
				       es->dependsOn_Es_number.value,	
#endif
				       MP4StreamDependencyReferenceType,
				       &outRefNum); if (err) goto bail;
    }
    err = MP4BeginMediaEdits(media[x]); if (err) goto bail;
    err = MP4SetMediaLanguage(media[x], "und"); if (err) goto bail;

    objectTypeIndication = 0x40; 				/* mpeg-4 audio */
    streamType = es->DecConfigDescr.streamType.value; 	/* 0x05(audio stream) */

    if (bufferSizeUsed)
      decoderBufferSize = (u32)bufferSize;	
    else
      decoderBufferSize = es->DecConfigDescr.bufferSizeDB.value;	

    maxBitrate = es->DecConfigDescr.maxBitrate.value;
    avgBitrate = es->DecConfigDescr.avgBitrate.value;

    err = MP4NewHandle(0, &decoderSpecificInfoH); if (err) goto bail;
      
    /* write decoderSpecificConfig here */
    err = writeAudioDecoderSpecificInfo(es, decoderSpecificInfoH); if (err) goto bail;

    err = MP4NewHandle(0, &sampleEntryH[x]); if (err) goto bail;

    if (debugLevel >= 3) {
      printf("objectTypeIndication=%x\n", objectTypeIndication);
      printf("streamType=%d\n", streamType);
      printf("decoderBufferSize=%d\n", decoderBufferSize);
      printf("maxBitrate=%d\n", maxBitrate);
      printf("avgBitrate=%d\n", avgBitrate);
    }
      
    err = MP4NewSampleDescription(trak[x], 
				  sampleEntryH[x],
				  1,
				  objectTypeIndication,
				  streamType,
				  decoderBufferSize,
				  maxBitrate,
				  avgBitrate,
				  decoderSpecificInfoH ); if (err) goto bail;
  }

  err = MP4NewHandle(sizeof(u32), &sampleDurationH); if (err) goto bail;

  if (sampleDurationUsed) {
    *((u32*) *sampleDurationH) = (u32)sampleDuration;	/* value specified by command line switch */
  }
  else {
    /* set duration for each sample to be added according to audioObjectType */
    /* in this implemantation, each sample corresponds to one audio frame */
    switch(es->DecConfigDescr.audioSpecificConfig.audioDecoderType.value) {
    case AAC_SCAL:
    case TWIN_VQ:
#ifdef VERSION2
    case ER_BSAC : /* bsac */
    case ER_TWIN_VQ:
#endif    
      *((u32*) *sampleDurationH) = 960;	/* change accordingly */
      break;
    case CELP:
#ifdef VERSION2
    case ER_CELP:
#endif
      *((u32*) *sampleDurationH) = 160;	/* change accordingly */
      break;
    case HVXC:
#ifdef VERSION2
    case ER_HVXC:
#endif
      *((u32*) *sampleDurationH) = 160;	/* 20ms at fs=8000Hz */
      break;
#ifdef VERSION2
    case ER_HILN: /* HP20001009 */
    case ER_PARA:
      if (objDescr.ESDescriptor[0]->DecConfigDescr.audioSpecificConfig.specConf.paraSpecificConfig.PARAmode.value == 0)
	*((u32*) *sampleDurationH) = 160;
      else if (objDescr.ESDescriptor[0]->DecConfigDescr.audioSpecificConfig.specConf.paraSpecificConfig.PARAmode.value == 1)
	*((u32*) *sampleDurationH) = objDescr.ESDescriptor[0]->DecConfigDescr.audioSpecificConfig.specConf.paraSpecificConfig.HILNframeLength.value;
      else
	*((u32*) *sampleDurationH) = 320;
      break;
#endif
    default:
      CommonExit(-1,"audioDecoderType not implemented");
      break;
    }
  }

  err = MP4NewHandle(0, &sampleDataH); if (err) goto bail;
  err = MP4NewHandle(sizeof(u32), &sampleSizeH); if (err) goto bail;

  while (! BsEof(inBitStream, 8)) {
    /* read FlexMux-stream frame by frame */
    for (x=0;x<objDescr.streamCount.value;x++){
      es = objDescr.ESDescriptor[x];

      length[x] = 0;
      /* read Access Unit from FlexMux-stream */
      getAccessUnit(inBitStream, 
		    frameData.layer[x].bitBuf,
		    &index, 
		    &length[x],
		    objDescr.ESDescriptor[x]);

      if ( x != index ) 
	CommonExit(1,"Read: error reading bit stream frame");

      err = MP4SetHandleSize(sampleDataH, (u32)length[x]); if (err) goto bail;

      /* write Access Unit to sampleDataH */
      if (debugLevel >= 5) printf("write Access Unit of ES[%ld]\n", es->ESNumber.value);
      p = (u8*) *sampleDataH;
      for (i=0; i<length[x]; i++) {
        *p++ = BsBufferGetDataBegin(frameData.layer[x].bitBuf)[i];
	if (debugLevel >= 5) printf("data[%3d]=0x%02x\n", i, *(p-1));
      }
      BsClearBuffer(frameData.layer[x].bitBuf);

      if (sampleDurationMSUsed) {
	*((u32*) *sampleDurationH) = (u32)sampleDurationMS * timeScale[x]/1000;
      }
      err = MP4GetHandleSize(sampleDataH, (u32*) *sampleSizeH); if (err) goto bail;
      if (debugLevel >= 5) 
	printf("sampleSizeH = %d, sampleDuration = %d\n", 
	       *((u32*) *sampleSizeH), *((u32*)*sampleDurationH));

      /* add samples to a media */
      err = MP4AddMediaSamples(media[x], 
			       sampleDataH, 
			       1, 
			       sampleDurationH, 
			       sampleSizeH, 
			       sampleEntryH[x], 
			       NULL, 
			       NULL); if (err) goto bail;

      if (sampleEntryH[x]) {
	err = MP4DisposeHandle(sampleEntryH[x]); if (err) goto bail;
	sampleEntryH[x] = NULL;
      }	

    }
  }

  /* dispose handles */
  err = MP4DisposeHandle(sampleDataH); if (err) goto bail;
  err = MP4DisposeHandle(sampleDurationH); if (err) goto bail;
  err = MP4DisposeHandle(sampleSizeH); if (err) goto bail;

  for (x=0;x<objDescr.streamCount.value;x++){
    err = MP4EndMediaEdits(media[x]); if (err) goto bail;
    err = MP4GetMediaDuration(media[x], &mediaDuration); if (err) goto bail;

    if (debugLevel >= 3) {
      printf("mediaDuration=%lld(timeScale=%d)\n", mediaDuration, timeScale[x]);
      err = MP4GetTrackID(trak[x], &trackID); if (err) goto bail;
      printf("trackID=%d\n", trackID);
    }
    err = MP4InsertMediaIntoTrack(trak[x], 
				  0, 
				  0, 
				  mediaDuration, 
				  1); if (err) goto bail;
    err = MP4SetTrackEnabled(trak[x], 1);       /* enable the track(AI 2000/01/18) */
  }
    
  if (debugLevel >= 3) {
    /* determine the number of tracks in a movie */
    err = MP4GetMovieTrackCount(moov, &trackCount); if (err) goto bail;
    printf("trackCount=%d\n", trackCount);
  }
    
  err = MP4WriteMovieToFile(moov, outBitFileName); if (err) goto bail;
  err = MP4DisposeMovie(moov); if (err) goto bail;

  /* close FlexMux stream file */
  if (BsClose(inBitStream))
    CommonExit(1,"error closing bit stream file");
bail:
  return err;
}

int main (int argc, char *argv[])
{
  char *progName = "<no program name>";
  int  result;
  char oriFileName[STRLEN],bitFileName[STRLEN];
  int fileIdx;
  MP4Err err;

  oriPath = NULL;
  bitPath = NULL;
  /*
  oriExt = NULL;
  bitExt = ".mp4";
  */

  /* evaluate command line  */
  CmdLineInit(0);
  result = CmdLineEval(argc,argv,paraList,switchList,1,&progName);
  if (result) {
    if (result==1) {
      printf("%s: %s\n",progName,PROGVER);
      CmdLineHelp(progName,paraList,switchList,stdout);
      exit (1);
    }
    else
      CommonExit(1,"command line error (\"-h\" for help)");
  }

  if (debugLevel == 1)
    fprintf(stderr,"%s: %s\n",progName,PROGVER);

  if (debugLevel >= 1) {
    if (profileAndLevelUsed) printf("audio_profileAndLevel=%x\n", profileAndLevel);
    if (sampleDurationUsed) printf("sampleDuration=%d\n", sampleDuration);
    if (bufferSizeUsed) printf("decoder buffer size=%d\n", bufferSize);
  }
  
  /* process all files on command line */
  fileIdx = 0;
  while (varArgIdx[fileIdx] >= 0) {

    /* compose file names */
    if (ComposeFileName(argv[varArgIdx[fileIdx]],0,oriPath,oriExt,oriFileName,
                        STRLEN))
      CommonExit(1,"composed file name too long");
    if (outFileNameUsed) {
      if (ComposeFileName(outFileName,0,bitPath,bitExt,bitFileName,
                          STRLEN))
        CommonExit(1,"composed file name too long");
    }
    else
      if (ComposeFileName(argv[varArgIdx[fileIdx]],1,bitPath,bitExt,
                          bitFileName,STRLEN))
        CommonExit(1,"composed file name too long");

    /* encode file */
    if (debugLevel >= 1)
      fprintf(stderr,"converting %s -> %s\n",oriFileName,bitFileName);
    if (debugLevel >= 2)
      printf("converting %s -> %s\n",oriFileName,bitFileName);

    /* FlexMux to MP4 conversion */
    err = FlexMux_to_MP4(oriFileName, bitFileName);
    if (err) CommonExit(1,"FlexMux_to_MP4() return %d", err);

    fileIdx++;
  }
  
  CmdLineEvalFree(paraList);

  if (debugLevel == 1)
    fprintf(stderr,"%s: finished\n",progName);
  if (debugLevel >= 2)
    printf("%s: finished\n",progName);
  return(0);
}

