/*
This software module was originally developed by AT&T
and editted by 
Yasuhiro Toguri (Sony)
in the course of development of MPEG-4. 
This software module is an implementation of a part of one or 
more MPEG-4 tools as specified by MPEG-4. 
ISO/IEC gives users of MPEG-4 free license to this
software module or modifications thereof for use in hardware 
or software products claiming conformance to MPEG-4.
Those intending to use this software module in hardware or software
products are advised that its use may infringe existing patents.
The original developer of this software module and his/her company,
the subsequent editors and their companies, and ISO/IEC have no
liability for use of this software module or modifications thereof
in an implementation.
Copyright is not released for non MPEG-4 conforming
products. AT&T. retains full right to use the code for its own
purpose, assign or donate the code to a third party and to
inhibit third parties from using the code for non 
MPEG-4 conforming products.
This copyright notice must be included in all copies or
derivative works. Copyright (c) 1999.
*/

/* AAC file routines */

#define DEBUG				nofprintf

#include <stdio.h>
#include <string.h>
#include "MP4Movies.h"
#include "adif.h"
#include "all.h"
#include "port.h"

static FILE	*fp = NULL;
static u32	bword, bbit; /* for buffer i/o */
static u8	*bp, *block_buffer;
extern MP4Handle sampleDataH;
long	byte_count;

static u8	*MP4AacBp;
static MP4Movie	mp4_input_moov;
static MP4Track	mp4_input_trak;
static MP4TrackReader	mp4_input_reader;
static MP4Handle	mp4_input_sampleH;
static u32		mp4_input_frameLen;

/* logging */
static int copy_to_output = 0;
static int output_bits_buffered = 0;
static struct {
  u8 ele_id;
  u32 bit_position;
} part_list[128];
static int part_list_len = 0;

void nofprintf(FILE* fp, ...) {fp=NULL;}

void myexit(char *s)
{
  closeFile();
  if (s) PRINT(SE, "\n%s\n", s);
  exit(s ? 1 : 0);
}

void reset_input(u8* Mp4Pos, u32 length)
{
  bword=bbit=nbits=cword=0;
  byte_count = 0;
  bp = block_buffer;
  if (Mp4Pos==NULL) {
    fseek(fp, 0, SEEK_SET);
    rdb_in_frame = -1;
  } else {
    MP4AacBp = Mp4Pos;
    mp4_input_frameLen = length;
  }
}


/* put <length> bits with a content of <value> to output */
static void feed_output_buffer(int length, u32 value)
{
  if (copy_to_output) {
    if (length<32) value &= (1<<length)-1;
    if (output_bits_buffered) {
      int write_bits=(8-output_bits_buffered);
      if (write_bits>length) write_bits=length;
      *bp |= (u8)((value>>(length-write_bits))<<(8-output_bits_buffered-write_bits));
      length -= write_bits;
      if ((output_bits_buffered +=write_bits)==8) {
        bp++;output_bits_buffered=0;
      }
    }
    while (length >= 8) {
      length-=8;
      *bp++ = (u8)(value>>length);
    }
    if (length) {
      *bp=(u8)(value<<(8-length));
      output_bits_buffered = length;
    }
  }
}


static u32 get_output_pos()
{
  return ((bp - (u8*)*sampleDataH)<<3) + output_bits_buffered;
}


/* get byte from input */
static int getbyte_conv(void)
{
  static int error_counter = 0;
  int c = getc(fp);

  byte_count++;
  if(c < 0) {
    if (debug['v']) PRINT(SE, "end of file\n");
    if (error_counter++ > 3) {
      myexit("end of file: terminating\n");
    }
    return -1;
  }

  return c; 
}

static int getbyte_conv_MP4(void)
{
  if ((u32)byte_count++<mp4_input_frameLen) {
    return *MP4AacBp++;
  }
  return -1;
}

int	(*getbyte)(void)=&getbyte_conv;

/* whole input goes through this function */
long
getbits(int n)
{
  int retval, c1, c2;

  if(nbits <= 16){
    if ((c1 = (*getbyte)()) < 0) c1=0;
    if ((c2 = (*getbyte)()) < 0) c2=0;

    cword = (cword<<16) | (c1<<8) | c2;
    nbits += 16;
  }

  nbits -= n;

  retval = (cword>>nbits) & ((1<<n)-1);
  if (n) {
    feed_CRC_calculator(n, retval);
    feed_output_buffer(n, retval);
  }

  return retval;
}


void set_log_level(int id, int add_bits_num, int add_bits)
     /*
      * to be called once for each element in the stream
      * unfortunally this additional-bits construction is necessary, because
      * these bits indicate the elements tag
      */
{
  part_list[part_list_len].ele_id = id;
  part_list[part_list_len++].bit_position = get_output_pos();
  switch(id) {
  case ID_SCE:
  case ID_CPE:
  case ID_CCE:
  case ID_LFE: /*log; CRC-log 192 bit*/
    enable_CRC(CRC_LEN_ELE, 0);
    feed_CRC_calculator(add_bits_num, add_bits);
    copy_to_output = 1;
    break;
  case ID_2ND: /*log; CRC-log additional 128 bit*/
    enable_CRC(CRC_LEN_2ND,1);
    return;
  case ID_DSE:
  case ID_PCE: /*log; CRC-log until LOG_FIN*/
    enable_CRC(CRC_LEN_ALL,0);
    feed_CRC_calculator(add_bits_num, add_bits);
    copy_to_output = 1;
    break;
  case ID_FIL: /*log; no CRC*/
    copy_to_output = !del_fill;
    if ((del_fill)&&(debug['F'])) PRINT(SE, "deleting FILL-element\n");
    break;
  case ID_END: {/*log this; no CRC*/
    unsigned long crcValue;
    copy_to_output = 1;
    feed_output_buffer(LEN_SE_ID, id);
    copy_to_output = 0;
    crcValue = reset_CRC();
    if ((debug ['C'])&&(adts_header_present)&&(!adts_header.protection_abs)) {
      if (crcValue!=(unsigned long)adts_header.crc_check)
        PRINT(SE,"CRC: 0x%04lx, but header said : 0x%04x => FAIL\n",crcValue,adts_header.crc_check);
    }
    return;
  }
  case LOG_FIN: /*dont log; CRC finish (zero-pad)*/
    fillup_CRC();
    copy_to_output = 0;
    return;
  case ID_HDR: /*dont log; CRC-log 56bit*/
    enable_CRC(CRC_LEN_HDR, 0);
    copy_to_output = 0;
    return;
  case ID_NUL: /*dont log; no CRC (cut away this element)*/
    copy_to_output = 0;
    return;
  default:
    myexit("logger: HELP!!!");
  }
  feed_output_buffer(LEN_SE_ID, id);
  if (add_bits_num) feed_output_buffer(add_bits_num, add_bits);
}


void init_output_CRC(int expect_header)
{
  reset_CRC();
  if (expect_header) enable_CRC(CRC_LEN_HDR, 0);
}


/*read the stream again for CRC-calculation, before writing to disk*/
static void feed_out_stream_CRC(u32 pos, int length, int length_in_stream)
{
  u8* out_stream_ptr = (u8*)*sampleDataH + (pos>>3);
  int bitpos = pos&7, bitsToFeed;

  if (length==-1) length = length_in_stream;
  if (length_in_stream>length) length_in_stream=length;

  bitsToFeed = length_in_stream;

  enable_CRC(length,0);

  if (bitpos) {
    int len = bitsToFeed, tmp = (u8)*out_stream_ptr++;
    if (len>(8-bitpos)) len = 8-bitpos;
    if (bitsToFeed<(8-bitpos)) tmp>>=(8-bitpos-bitsToFeed);
    feed_CRC_calculator(len, tmp&((1<<len)-1));
    bitsToFeed-=len;
  }
  while (bitsToFeed>=8) {
    feed_CRC_calculator(8,(u8)*out_stream_ptr++);
    bitsToFeed-=8;
  }
  if (bitsToFeed) {
    feed_CRC_calculator(bitsToFeed, ((u8)*out_stream_ptr)>>(8-bitsToFeed));
  }

  if (length_in_stream!=length) {
    feed_CRC_calculator(length-length_in_stream,0);
  }
}


void reset_output_CRC()
{
  part_list_len = 0;
}


u16 calculate_output_CRC()
{
  int i = 0;
  u32 pos1, pos2;

  while (i<part_list_len) {
    switch (part_list[i].ele_id) {
    case ID_SCE:
    case ID_CCE:
    case ID_LFE:
      pos1 = part_list[i].bit_position + LEN_SE_ID;
      if (part_list[++i].ele_id != LOG_FIN) myexit("logger: unexpected combination!");
      feed_out_stream_CRC(pos1, CRC_LEN_ELE, part_list[i++].bit_position - pos1);
      break;
    case ID_CPE:
      pos1 = part_list[i].bit_position + LEN_SE_ID;
      if (part_list[++i].ele_id != ID_2ND) myexit("logger: unexpected combination!");
      pos2 = part_list[i].bit_position;
      if (part_list[++i].ele_id != LOG_FIN) myexit("logger: unexpected combination!");
      feed_out_stream_CRC(pos1, CRC_LEN_ELE, part_list[i].bit_position-pos1);
      feed_out_stream_CRC(pos2, CRC_LEN_2ND, part_list[i++].bit_position-pos2);
      break;
    case ID_DSE:
    case ID_PCE:
      pos1 = part_list[i].bit_position + LEN_SE_ID;
      if (part_list[++i].ele_id != LOG_FIN) myexit("logger: unexpected combination!");
      feed_out_stream_CRC(pos1, CRC_LEN_ALL, part_list[i++].bit_position - pos1);
      break;
    case ID_FIL:
    case ID_HDR:
    case ID_NUL:
      if (part_list[++i].ele_id != LOG_FIN) myexit("logger: unexpected combination!");
    case ID_END:
      i++;
      break;
    case ID_2ND:
    default:
      myexit("can't calculate output CRC");
    }
  }
  reset_output_CRC();
  return (u16)reset_CRC();
}


/* writes n bits from data (msb first) to **pd if pd is not NULL
 * otherwise does nothing
 */
static void putbits( char **pd, u32 data, u32 n)
{
        u32 dbit;
        u32 word, bit;
	char *d;

	if (pd == NULL) {
		d = NULL;
	}
	else {
		d = *pd;
	}
        dbit = 1<<(n-1);
        word = bword;
        bit = bbit;
        while(dbit != 0) {
                if(data & dbit) {
                        word |= bit;
                }
                dbit >>= 1;
                bit >>= 1;
                if(bit == 0) {
			if (d)
				*d++ = word;
                        word = 0;
                        bit = 0x80;
                }
        }
        bword = word;
        bbit = bit;
	if (d)
	    *pd = d;
}

static u32 num_audio_chn( EleList *p, int cce )
{
	u32 n;
	int i, j;
	
	n = 0;
	for (i=0; i<p->num_ele; i++) {
		j = p->ele_is_cpe[i];
		if (!cce)
			n += (j) ? 2 : 1;
		else 
			n += (j) ? 1 : 0;	/* ind sw cce */
	}
	return n;
}

static void put_ele_list( char **pb, EleList *p, int flag )
{
	int i;
	for (i=0; i<p->num_ele; i++) {
		if (flag)
			putbits(pb, p->ele_is_cpe[i], LEN_ELE_IS_CPE);
		/* tag must agree with that in bitstream! */
		putbits(pb, p->ele_tag[i], LEN_TAG);
	}
}

static void put_prog_config( char **pb, int tag, ProgConfig *p )
{
	int i, j;
	
	putbits(pb, tag, LEN_TAG);
	putbits(pb, p->profile, LEN_PROFILE);
	putbits(pb, p->sampling_rate_idx, LEN_SAMP_IDX);
	putbits(pb, p->front.num_ele, LEN_NUM_ELE);
	putbits(pb, p->side.num_ele, LEN_NUM_ELE);
	putbits(pb, p->back.num_ele, LEN_NUM_ELE);
	putbits(pb, p->lfe.num_ele, LEN_NUM_LFE);
	putbits(pb, p->data.num_ele, LEN_NUM_DAT);
	putbits(pb, p->coupling.num_ele, LEN_NUM_ELE);
	putbits(pb, 0, LEN_MIX_PRES);   /* no mono mixdown */
	putbits(pb, 0, LEN_MIX_PRES);   /* no stereo mixdown */
	putbits(pb, 0, LEN_MIX_PRES);   /* no matrix mixdown */
	
	/* front, side, back sce or cpe */
	put_ele_list(pb, &p->front, 1);
	put_ele_list(pb, &p->side, 1);
	put_ele_list(pb, &p->back, 1);
	put_ele_list(pb, &p->lfe, 0);
	put_ele_list(pb, &p->data, 0);
	put_ele_list(pb, &p->coupling, 1);

	/* byte align */
	while (bbit != 0x80)
		putbits( pb, 0, 1);
	
	/* comment */
	j = strlen(p->comments);
	putbits(pb, j, LEN_COMMENT_BYTES);
	for (i=0; i<j; i++)
		putbits(pb, p->comments[i], 8);
}


long put_adif_header( ADIF_Header source, ProgConfig my_prog_config, char *dest )
{
  char* startpos=dest;
  int i;

  bbit = 0x80; /* byte align bit stream */
  bword = 0;

  for (i=0; i<LEN_ADIF_ID; i++)
   putbits(&dest, source.adif_id[i], LEN_BYTE); 

  putbits(&dest, source.copy_id_present, LEN_COPYRT_PRES);
  if (source.copy_id_present) {
    for (i=0; i<LEN_COPYRT_ID; i++)
      putbits(&dest, source.copy_id[i], LEN_BYTE); 
  }

  putbits(&dest, source.original_copy, LEN_ORIG);
  putbits(&dest, source.home, LEN_HOME);
  putbits(&dest, source.bitstream_type, LEN_BS_TYPE);
  putbits(&dest, source.bitrate, LEN_BIT_RATE);
  putbits(&dest, 0 /*num_program_config_elements*/, LEN_NUM_PCE);

  if (source.bitstream_type == 0)
    putbits(&dest, source.buffer_fullness, LEN_ADIF_BF);

  put_prog_config(&dest, 0 /*tag*/, &my_prog_config);

  /*byte align*/
  while (bbit!=0x80) {
    putbits(&dest, 0, 1);
  }

  return (dest-startpos);
}

long put_adts_header( ADTS_Header source, char* dest)
{
  char* startpos=dest;

  bbit = 0x80; /* byte align bit stream */
  bword = 0;

  putbits(&dest, source.syncword, LEN_SYNCWORD);
  putbits(&dest, source.id, LEN_ID);
  putbits(&dest, source.layer, LEN_LAYER);
  putbits(&dest, source.protection_abs, LEN_PROTECT_ABS);
  putbits(&dest, source.profile, LEN_PROFILE);
  putbits(&dest, source.sampling_freq_idx, LEN_SAMP_IDX);
  putbits(&dest, source.private_bit, LEN_PRIVTE_BIT);
  putbits(&dest, source.channel_config, LEN_CHANNEL_CONF);
  putbits(&dest, source.original_copy, LEN_ORIG);
  putbits(&dest, source.home, LEN_HOME);
  putbits(&dest, source.copyright_id_bit, LEN_COPYRT_ID_ADTS);
  putbits(&dest, source.copyright_id_start, LEN_COPYRT_START);
  putbits(&dest, source.frame_length, LEN_FRAME_LEN);
  putbits(&dest, source.adts_buffer_fullness, LEN_ADTS_BUF_FULLNESS);
  putbits(&dest, source.num_of_rdb, LEN_NUM_OF_RDB);
  if(source.protection_abs == 0){
    putbits(&dest, source.crc_check, LEN_CRC);
  }

  /*byte align*/
  while (bbit!=0x80) {
    putbits(&dest, 0, 1);
  }

  return (dest-startpos);
}


/* try to put <count> FILL bytes into output-stream, return achived number of bytes */
static int put_fill_bytes(int totalcount)
{
  int bitsLeft=(totalcount<<3)-(LEN_SE_ID+LEN_F_CNT),count;

  if ((bitsLeft>>3)>=((1<<LEN_F_CNT)-1)) {
    count=(1<<LEN_F_CNT)-1;
  } else {
    count=(bitsLeft>>3)+((bitsLeft&0x7)?1:0);
    if (count<0) count=0;
  }
  feed_output_buffer(LEN_SE_ID+LEN_F_CNT, (ID_FIL<<LEN_F_CNT)|count);
  bitsLeft -= (count<<3);

  if (count==(1<<LEN_F_CNT)-1) { /* more bit for length info */
    int more_cnt;
    bitsLeft -= LEN_F_ESC;
    if ((more_cnt = (bitsLeft>>3)+((bitsLeft&0x7)?1:0)+1)>(1<<LEN_F_ESC)-1)
      more_cnt=(1<<LEN_F_ESC)-1; /* will need another FILL-Element then */
    feed_output_buffer(LEN_F_ESC,more_cnt);
    count+=more_cnt-1;
    bitsLeft-=(more_cnt-1)<<3;
  }

  feed_output_buffer(count<<3,0); /* fill in zeroes */
  if (bitsLeft>0) {
    bitsLeft-=put_fill_bytes(bitsLeft>>3)<<3;
  }

  return totalcount-(bitsLeft>>3);
}


/* remove ID_END, add FILL-elements (<count> bytes) and write a new ID_END, returns number of added bytes */
int move_end(int count)
{
  int retVal=(int)bp;

  /* delete old ID_END */
  if (output_bits_buffered) bp--;
  output_bits_buffered-=LEN_SE_ID;
  if (output_bits_buffered<0) {
    output_bits_buffered+=8;
    bp--;
  }
  *bp &= ((1<<output_bits_buffered)-1)<<(8-output_bits_buffered);

  copy_to_output=1;
  put_fill_bytes(count);
  feed_output_buffer(LEN_SE_ID,ID_END);
  copy_to_output=0;

  if (output_bits_buffered) bp++;
  return (int)bp-retVal;
}


/* AAC raw data block is always an integral number of bytes
 * and is byte aligned
 * returns byte count
 */
static u32 get_raw_data_block( u8 *p )
{
        if ((nbits & 3) != 0) {
          PRINT(SE, "byte alignment error\n");
          return 0;
        }
        output_bits_buffered=0;
        bp = block_buffer = p;

	/* parse this block */
	dec();

        if (output_bits_buffered) bp++;

	/* return byte count */	
        return (bp - block_buffer);
}

/* open file */
MP4Err openInputFile( char *filename )
{
	if ((fp = fopen( filename, "r" )) == NULL) {
		PRINT(SE, "open\n");
		return MP4FileNotFoundErr;
	}
	return MP4NoErr;
}

MP4Err openMP4InputFile( char *filename )
{
  return MP4OpenMovieFile( &mp4_input_moov, filename, MP4OpenMovieNormal );
}


MP4Err getRAWinfo( MP4Handle decoderSpecificInfoH, u32 *timeScale, u8 *profile, u32 *maxBitrate, u32 *avgBitrate, u32 *bufferFullness )
{
  MP4Err err;
  char *dsi = (char*) *decoderSpecificInfoH, *plen;
  int n;
  int old_fullness;

  if (get_adts_header()) {
    PRINT(SE,"Warning: missing ADTS-syncword... trying RAW\n");
    adts_header_present = 0;
    *maxBitrate = *avgBitrate = *bufferFullness = 0;
    *profile = mc_info.profile = mc_info_cmdline.profile;
    mc_info.sampling_rate_idx=mc_info_cmdline.sampling_rate_idx;
  }
  reset_input(NULL,0);

  init_dec();
  init();
  get_raw_data_block((u8*)*sampleDataH);

  if (adts_header_present) {
    old_fullness = adts_header.adts_buffer_fullness;
    *bufferFullness = (old_fullness<<5)*channel_num;

    byte_count = (nbits>>3);
    if (get_adts_header()) {
      *maxBitrate = *avgBitrate = 0; PRINT(SE, "multiple raw-data-blocks per frame => expect bitrate problems\n");
    } else {
      *maxBitrate = *avgBitrate = (unsigned int)FRAMELEN2BITRATE((((adts_header.adts_buffer_fullness-old_fullness)<<2)*channel_num +adts_header.frame_length)<<3);
    }
  }

  init();
  reset_input(NULL,0);
  reset_output_CRC();

  if (((adts_header_present)&&(!adts_header.channel_config))||
      (prog_config.profile)||(prog_config.sampling_rate_idx)) {
    n = 0;  /* there was a PCE */
  } else {
    if (mc_info.nch==8) n = 7; /* see 13818-7, Table 8.1 */
    else n = mc_info.nch;
  }

  *timeScale = samp_rate_info[mc_info.sampling_rate_idx].samp_rate;

  bbit = 0x80; /* byte align bit stream */
  bword = 0;

  /* tag is DecSpecificInfoTag, 14496-1, Sec 8.2.2.2, Table 1 */
  putbits( &dsi, 0x05, 8 );
  /* dummy sizeOfInstance, 14496-1, Sec 12.3.3 */
  plen = dsi;
  putbits( &dsi, 0, 8 );
  /* Audio Specific Configuration */
  *profile = mc_info.profile+1;
  putbits( &dsi, *profile, lenAudioObjectType ); /* For Main LC SSR LTP */
  putbits( &dsi, mc_info.sampling_rate_idx, lenSamplingFrequencyIndex );
  putbits( &dsi, n, lenChannelConfiguration );

  /* GA Specific Configurtaion */
  putbits( &dsi, 0, lenFrameLengthFlag );
  putbits( &dsi, 0, lenDependsOnCoreCoder );
  putbits( &dsi, 0, lenExtensionFlag );

  if (!n) {  /* pce */
    put_prog_config( &dsi, 0, &prog_config);

    if (bbit != 0x80) {
      PRINT(SE, "PCE write byte alignment error\n");
      return MP4BadParamErr;
    }
  }

  /* set exact handle size */
  n = dsi - *decoderSpecificInfoH;
  err = MP4SetHandleSize( decoderSpecificInfoH, n ); if (err) return err;

  /* set length of decoderSpecificInfoH part in header,
   * encoded as sizeOfInstance
   */
  if (n < 128) {
    *plen = n-2;
  } else {
    /* encoding of size >= 128 bytes currently not supported */
    PRINT(SE, "decoderSpecificInfo too large\n");
    return MP4BadParamErr;
  }

  return MP4NoErr;
}


MP4Err getMP4info( MP4Handle decoderSpecificInfoH, u32 *timeScale, u8 *profile, u32 *maxBitrate, u32 *avgBitrate, u32 *bufferFullness )
{
  MP4Err err = MP4NoErr;
  MP4Media mp4_input_media;
  MP4Handle mp4_input_initialObjectDescriptorH;
  u32 handlerType;
  u32 objectTypeIndication;
  u32 streamType;
  u32 decoderBufferSize;
  u32 tmp;
  int nChannelConf;
  char* dsi;

  getbyte = &getbyte_conv_MP4;
  init_dec();

  err = MP4NewHandle( 0, &mp4_input_initialObjectDescriptorH ); if (err) goto bail;
  err = MP4GetMovieInitialObjectDescriptor( mp4_input_moov, mp4_input_initialObjectDescriptorH ); if (err) goto bail;
  err = MP4GetMovieIndTrack( mp4_input_moov, /*trackNumber*/ 1, &mp4_input_trak ); if (err) goto bail;
  err = MP4GetTrackMedia( mp4_input_trak, &mp4_input_media ); if (err) goto bail;
  err = MP4GetMediaHandlerDescription( mp4_input_media, &handlerType, NULL );if (err) goto bail;
  err = MP4GetMediaDecoderInformation( mp4_input_media,
                                            /*sampleDescIndex*/1, 
                                            &objectTypeIndication, 
                                            &streamType, 
                                            &decoderBufferSize, 
                                            /*upstream*/&tmp,
                                            maxBitrate,
                                            avgBitrate,
                                            decoderSpecificInfoH ); if (err) goto bail;
  if ( handlerType != MP4AudioHandlerType ) err = MP4BadParamErr;
  if ( objectTypeIndication != 0x40 ) err = MP4BadParamErr;
  if ( streamType != 0x05 ) err = MP4BadParamErr;
  if ( decoderBufferSize > MAX_FRAME_LEN ) err = MP4BadParamErr;
  if (err) goto bail;

  /* Audio Specific Configuration */
  err = MP4GetHandleSize(decoderSpecificInfoH,&tmp); if (err) goto bail;
  reset_input(*(u8**)decoderSpecificInfoH, tmp);

  mc_info.profile = getbits(lenAudioObjectType) - 1; /* from AudioSpecificConfig */
  if(mc_info.profile > LTP_Profile || mc_info.profile < Main_Profile ){
    PRINT(SE, "Warning!!! Invalid Object Type in AudioSpecificConfig\n");
  }

  mc_info.sampling_rate_idx = getbits(lenSamplingFrequencyIndex);
  *timeScale = mc_info.sampling_rate_idx;

  nChannelConf = getbits(lenChannelConfiguration);

  /* GA Specific Configurtaion */
  getbits(lenFrameLengthFlag);
  getbits(lenDependsOnCoreCoder);
  getbits(lenExtensionFlag);

  /* pce */
  if(nChannelConf==0) {
    get_prog_config(&prog_config);
    mc_info.profile = prog_config.profile; 
    if(mc_info.profile > LTP_Profile || mc_info.profile < Main_Profile){
      PRINT(SE, "Warning!!! Invalid Profile ID in PCE\n");
    }
  } else {
    infoinit(&samp_rate_info[mc_info.sampling_rate_idx]);

    if (nChannelConf==7) nChannelConf = 8;
    channel_num = (decoderBufferSize<<3)/bufferSizePerAudioChn;
  }

  *profile = mc_info.profile + 1;

  err = MP4GetHandleSize( decoderSpecificInfoH,&tmp  ); if (err) goto bail;
  err = MP4SetHandleSize( decoderSpecificInfoH, tmp+2); if (err) goto bail;
  for (dsi=(char*)*decoderSpecificInfoH+tmp+2;dsi>=(char*)*decoderSpecificInfoH+2;dsi--)
    *dsi=*(dsi-2);
  **decoderSpecificInfoH = 0x05;        /* these two bytes are missing */
  *(*decoderSpecificInfoH+1) = tmp;

  err = MP4CreateTrackReader( mp4_input_trak, &mp4_input_reader ); if (err) goto bail;
  err = MP4NewHandle( 0, &mp4_input_sampleH ); if (err) goto bail;

  init();

  *bufferFullness=0;
 bail:
  MP4DisposeHandle(mp4_input_initialObjectDescriptorH);
  return err;
}


int adif_id_present()
{
  char adif_id[4];
  fread(adif_id, sizeof(char), 4, fp);
  fseek(fp, 0, SEEK_SET);
  if(!strncmp(adif_id, "ADIF", 4)) return 1;
  return 0;
}

MP4Err getADIFinfo( int prog, MP4Handle decoderSpecificInfoH, u32* samplingRate, u8* profile, u32* maxBitrate, u32* avgBitrate, u32 *bufferFullness )
{
  u32 i, n;
  char *dsi, *plen;
  ProgConfig *pc;
  MP4Err err;


  pc = &prog_config;
  current_program = prog; /* set desired program */
	
  /* initialize decoder */
  init_dec();

  if (!get_adif_header(0)) {
    PRINT(SE, "adif_header\n");
    return MP4BadParamErr;
  }
  byte_count = (nbits>>3); /* reset bytecounter */

  init();        /* initialize data structures */

  if ((nbits & 7) != 0) {
    PRINT(SE, "PCE read byte alignment error\n");
    return MP4BadParamErr;
  }

  /* write DecoderSpecificInfo */	
  bbit = 0x80; /* byte align bit stream */
  bword = 0;
  dsi = (char *) *decoderSpecificInfoH;

  /* tag is DecSpecificInfoTag, 14496-1, Sec 8.2.2.2, Table 1*/
  putbits( &dsi, 0x05, 8 );
  /* dummy sizeOfInstance, 14496-1, Sec 12.3.3 */
  plen = dsi;
  putbits( &dsi, 0, 8 );
  /* Audio Specific Configuration */
  putbits( &dsi, pc->profile+1, lenAudioObjectType );/* For Main LC SSR LTP */
  *profile = (u8) (pc->profile);
  if((*profile > LTP_Profile)/* || (*profile < Main_Profile)*/){
    PRINT(SE, "Warning!!! Invalid Profile ID in PCE\n");
  }

  putbits( &dsi, pc->sampling_rate_idx, lenSamplingFrequencyIndex );

  *samplingRate = /*sampling_rate[pc->sampling_rate_idx]*/samp_rate_info[pc->sampling_rate_idx].samp_rate;
  if (*samplingRate == 0) {
    PRINT(SE, "samplingRate\n");
    return MP4BadParamErr;
  }

  putbits( &dsi, 0, lenChannelConfiguration );
  /* GA Specific Configurtaion */
  putbits( &dsi, 0, lenFrameLengthFlag );
  putbits( &dsi, 0, lenDependsOnCoreCoder );
  putbits( &dsi, 0, lenExtensionFlag );
  /* pce */
  put_prog_config( &dsi, 0, pc);
  if (bbit != 0x80) {
    PRINT(SE, "PCE write byte alignment error\n");
    return MP4BadParamErr;
  }

  /* set exact handle size */
  n = dsi - *decoderSpecificInfoH;
  err = MP4SetHandleSize( decoderSpecificInfoH, n ); if (err) return err;

  /* set length of decoderSpecificInfoH part in header,
   * encoded as sizeOfInstance
   */
  if (n < 128) {
    *plen = n-2;
  } else {
    /* encoding of size >= 128 bytes currently not supported */
    PRINT(SE, "decoderSpecificInfo too large\n");
    return MP4BadParamErr;
  }

  DEBUG(SE, "decoderSpecificInfo length %d\n", n);
  dsi = *decoderSpecificInfoH;
  for (i=0; i<n; i++)
    DEBUG(SE, "%3i 0x%02x\n", i, *dsi++); 

  *bufferFullness = (u32) adif_header.buffer_fullness;
  *maxBitrate = (u32) adif_header.bitrate;
  *avgBitrate = (u32) adif_header.bitrate;
  if ( *avgBitrate <= 0 ) {
    PRINT(SE, "avgBitrate\n");
    return MP4BadParamErr;
  }

  return MP4NoErr;
}

MP4Err	getDecoderBufferSize( u32 *decoderBufferSize )
{
	u32 nch;
	ProgConfig *p;
	
	p = &prog_config;
	nch  = num_audio_chn(&p->front, 0);
	nch += num_audio_chn(&p->side, 0);
	nch += num_audio_chn(&p->back, 0);
	nch += num_audio_chn(&p->coupling, 1);

        if (nch==0) { /* use decoder's speaker-configuration, if necessary */
          nch = channel_num;
        }

	*decoderBufferSize = nch * bufferSizePerAudioChn;
	if ( (0 < nch) && (nch <= 48)) {
		return MP4NoErr;
	}
	else {
		PRINT(SE, "decoderBufferSize: %d channels\n", nch);
		return MP4BadParamErr;
	}
}


MP4Err getRawDataBlock( u8 *rdb, u32 *frameLength  )
{
        int c;
	u32 i=0, unitSize;
	MP4Err err = MP4NoErr;

	/* check for EOF */
        if (!mpeg4_header_present) {
          if ((c = getc(fp)) == EOF) {
	    PRINT(SE, "End of bitstream\n");
	    return MP4EOF;
          }
          ungetc(c, fp);
        } else {
          err = MP4TrackReaderGetNextAccessUnit( mp4_input_reader, mp4_input_sampleH, &unitSize, &i, &i, &i ); if (err) {
            if (err==MP4EOF) PRINT(SE, "End of bitstream\n");
            return err;
          }
          reset_input((u8*)*mp4_input_sampleH, unitSize);
        }

	*frameLength = get_raw_data_block(rdb);
	if ( (*frameLength == 0) || (*frameLength > MAX_FRAME_LEN) ) {
	  PRINT(SE, "frameLength error: %d\n", *frameLength);
	  return MP4BadParamErr;
	}
	return MP4NoErr;
}
