/************************* MPEG-2 NBC Audio Decoder **************************
 *                                                                           *
"This software module was originally developed by 
AT&T, Dolby Laboratories, Fraunhofer Gesellschaft IIS 
and edited by
Yoshiaki Oikawa (Sony Corporation),
Mitsuyuki Hatanaka (Sony Corporation)
Yasuhiro Toguri (Sony Corporation)
in the course of development of the MPEG-2 NBC/MPEG-4 Audio standard ISO/IEC 13818-7, 
14496-1,2 and 3. This software module is an implementation of a part of one or more 
MPEG-2 NBC/MPEG-4 Audio tools as specified by the MPEG-2 NBC/MPEG-4 
Audio standard. ISO/IEC  gives users of the MPEG-2 NBC/MPEG-4 Audio 
standards free license to this software module or modifications thereof for use in 
hardware or software products claiming conformance to the MPEG-2 NBC/MPEG-4
Audio  standards. Those intending to use this software module in hardware or 
software products are advised that this use may infringe existing patents. 
The original developer of this software module and his/her company, the subsequent 
editors and their companies, and ISO/IEC have no liability for use of this software 
module or modifications thereof in an implementation. Copyright is not released for 
non MPEG-2 NBC/MPEG-4 Audio conforming products.The original developer
retains full right to use the code for his/her  own purpose, assign or donate the 
code to a third party and to inhibit third party from using the code for non 
MPEG-2 NBC/MPEG-4 Audio conforming products. This copyright notice must
be included in all copies or derivative works." 
Copyright(c)1996.
 *                                                                           *
 ****************************************************************************/

#ifdef ADIF2MP4
#define PARSE_ONLY
#endif

#include "all.h"
#include "monopred.h"
#include "port.h"
#include "block.h"
#include "dolby_def.h"
#ifdef MPEG4V1
#include "nok_lt_prediction.h"
#endif

void gain_control(Float  *freqSigCh,
                  Wnd_Shape	*wnd_shape,
                  int window_sequenceCh,
                  int ch,
                  Float   *timeSigCh
                  );

/* prediction */
static	PRED_STATUS	*sp_status[Chans];
#ifdef MPEG4V1
/* long term prediction */
static  NOK_LT_PRED_STATUS *nok_lt_status[Chans];
#endif

/* all storage is global */
Float *coef[Chans], *data[Chans], *state[Chans];
byte hasmask[Winds], *mask[Winds], *group[Chans],
    wnd[Chans], max_sfb[Chans],
    *cb_map[Chans], d_bytes[MAX_DBYTES];
Wnd_Shape wnd_shape[Chans];
short *factors[Chans];
int *lpflag[Chans], *prstflag[Chans];
TNS_frame_info *tns[Chans];
#ifndef	PARSE_ONLY
static int last_rstgrp_num[Chans] = {0};
#endif
#if (CChans > 0)
Float *cc_coef[CChans], *cc_gain[CChans][Chans];
byte cc_wnd[CChans] = {0, 0, 0, 0};
Wnd_Shape cc_wnd_shape[CChans];
Wnd_Shape wnd_shape_SSR[Chans];
#if (ICChans > 0)
Float *cc_state[ICChans];
#endif
#endif

/* variables for multiple rdb per frame */
int rdb_in_frame = -1;

void
init(void)
{
#if (CChans > 0)
  init_cc();
#endif
#ifdef	DRC
  init_drc();
#endif
  huffbookinit();
  predinit();

  winmap[0] = win_seq_info[ONLY_LONG_WINDOW];
  winmap[1] = win_seq_info[ONLY_LONG_WINDOW];
  winmap[2] = win_seq_info[EIGHT_SHORT_WINDOW];
  winmap[3] = win_seq_info[ONLY_LONG_WINDOW];
}

void
predinit(void)
{
  int i, ch;
  for (ch = 0; ch < Chans; ch++) {
    for (i = 0; i < LN2; i++) {
      init_pred_stat(&sp_status[ch][i],
                     PRED_ORDER,PRED_ALPHA,PRED_A,PRED_B);
    }
  }
#ifdef MPEG4V1
  for (ch = 0; ch < Chans; ch++) {
    nok_init_lt_pred(nok_lt_status[ch]);
  }
#endif
}

int
getdata(int *tag, int *dt_cnt, byte *data_bytes)
{
  int i, align_flag, cnt;

  *tag = getbits(LEN_TAG);

  set_log_level(ID_DSE, LEN_TAG, *tag);  /* required for CRC and ADIF2MP4 */

  align_flag = getbits(LEN_D_ALIGN);
  if ((cnt = getbits(LEN_D_CNT)) == (1<<LEN_D_CNT)-1)
    cnt +=  getbits(LEN_D_ESC);
  *dt_cnt = cnt;
  if (debug['x'])
    PRINT(SE, "data element %d has %d bytes\n", *tag, cnt);
  if (align_flag)
    byte_align();

  for (i=0; i<cnt; i++) {
    data_bytes[i] = getbits(LEN_BYTE);
    if (debug['X'])
      PRINT(SE, "%6d %2x\n", i, data_bytes[i]);
  }

  set_log_level(LOG_FIN, 0, 0);  /* required for CRC and ADIF2MP4 */

  return 0;
}

int
extension_payload(int cnt, byte *data)
{
  int type, i;
    
  /* fill bytes should not emulate any EX types below! */
  type = getbits(LEN_EX_TYPE);
  if (debug['x'])
    PRINT(SE, "fill element extension %d\n", type);
    
  switch(type) {
#ifdef	DRC
  case EX_DRC:
    return dynamic_range_info();
#endif
  case EX_FILL_DATA:
    getbits(LEN_NIBBLE);
    for (i=0; i<cnt-1; i++)
      data[i] = getbits(LEN_BYTE);
    return cnt;
  default: 
    getbits(LEN_NIBBLE);
    for (i=0; i<cnt-1; i++)
      getbits(LEN_BYTE);
    return cnt;
  }
}

void
getfill(byte *data)
{
  int cnt;

  set_log_level(ID_FIL, 0, 0);  /* required for CRC and ADIF2MP4 */

  if ((cnt = getbits(LEN_F_CNT)) == (1<<LEN_F_CNT)-1)
    cnt +=  getbits(LEN_F_ESC) - 1;
  if (debug['x'])
    PRINT(SE, "fill element has %d bytes\n", cnt);
  while (cnt > 0)
    cnt -= extension_payload(cnt, data);

  set_log_level(LOG_FIN, 0, 0);  /* required for CRC and ADIF2MP4 */
}

void
init_dec(void)
{
  int i, j;

  /*****
        debug['N']=1;
        debug['v']=1;
        debug['f']=1;
        debug['s']=1;
        debug['r']=1;
  *****/

  for(i=0; i<Chans; i++){
    coef[i] = (Float *)mal1(LN2*sizeof(*coef[0]));
    data[i] = (Float *)mal1(LN2*sizeof(*data[0]));
    state[i] = (Float *)mal1(LN*sizeof(*state[0]));	/* changed LN4 to LN 1/97 mfd */
    fltclr(state[i], LN);
    factors[i] = (short *)mal1(MAXBANDS*sizeof(*factors[0]));
    cb_map[i] = (byte *)mal1(MAXBANDS*sizeof(*cb_map[0]));
    group[i] = (byte *)mal1(NSHORT*sizeof(group[0]));
    lpflag[i] = (int *)mal1(MAXBANDS*sizeof(*lpflag[0]));
    prstflag[i] = (int *)mal1((LEN_PRED_RSTGRP+1)*sizeof(*prstflag[0]));
    tns[i] = (TNS_frame_info *)mal1(sizeof(*tns[0]));
    sp_status[i]  = (PRED_STATUS *)mal1(LN*sizeof(*sp_status[0]));
    wnd_shape[i].prev_bk = 0;
#ifdef MPEG4V1
    nok_lt_status[i]  = (NOK_LT_PRED_STATUS *)mal1(sizeof(*nok_lt_status[0])
                                                   );
#endif
  }
  for(i=0; i<Winds; i++) {
    mask[i] = (byte *)mal1(MAXBANDS*sizeof(mask[0]));
  }
#if (CChans > 0)
  for(i=0; i<CChans; i++){
    cc_coef[i] = (Float *)mal1(LN2*sizeof(*cc_coef[0]));
    for(j=0; j<Chans; j++)
      cc_gain[i][j] = (Float *)mal1(MAXBANDS*sizeof(*cc_gain[0][0]));
#if (ICChans > 0)
    if (i < ICChans) {
      cc_state[i] = (Float *)mal1(LN*sizeof(*state[0]));  /* changed LN4 to LN 1/97 mfd */
      fltclr(cc_state[i], LN);
      cc_wnd_shape[i].prev_bk = 0;
    }
#endif
  }
#endif

  for(i = 0; i < Chans; i++){
    wnd_shape_SSR[i].prev_bk = 0;
    wnd_shape[i].prev_bk = 0;
    wnd_shape[i].this_bk = 0;
  }


  if(adts_header_present != 1){
    init();        /* initialize data structures */
  }
}


/* decode a block */
void
dec()
{
  int ele_id, d_tag, d_cnt;
  MC_Info *mip = &mc_info;

#ifndef PARSE_ONLY
  int i, j, ch, wn;
  int left, right;
  Info *info;
  Ch_Info *cip;
#endif /*PARSE_ONLY*/

  if(debug['n'])
    PRINT(SE, "\rblock %ld  ", bno);
  if(debug['N'])
    PRINT(SE, "block %ld\n", bno);


  byte_align();
  if(adts_header_present == 1) {

    rdb_in_frame++;
    if (rdb_in_frame>adts_header.num_of_rdb) rdb_in_frame = 0;
    if (rdb_in_frame<=0) { /* only if ADTS frame contains no more raw data blocks */

      set_log_level(ID_HDR, 0, 0);  /* required for CRC and ADIF2MP4 */
      if (get_adts_header()) myexit("ADTS header");
      set_log_level(LOG_FIN, 0, 0);  /* required for CRC and ADIF2MP4 */

      if(debug['v'])
        PRINT(SE, "raw data blocks this frame:%i\n", adts_header.num_of_rdb +1);

      if (!bno) init();
      byte_align();
    }
  }


  reset_mc_info(mip);
#ifdef	DRC	
  reset_drc();
#endif
  while ((ele_id=getbits(LEN_SE_ID)) != ID_END) {
    /* get audio syntactic element */

    if(debug['v'])
      PRINT(SE, "\nele_id %d\n", ele_id);

    switch (ele_id) {
    case ID_SCE:		/* single channel */
    case ID_CPE:		/* channel pair */
    case ID_LFE:		/* low freq effects channel */
      if (huffdecode(ele_id, mip, wnd, wnd_shape,
                     cb_map, factors, 
                     group, hasmask, mask, max_sfb,
                     lpflag, prstflag
#ifdef MPEG4V1
                     , nok_lt_status
#endif
                     , tns, coef) < 0)
        myexit("huffdecode");
      break;
#if (CChans > 0)
    case ID_CCE:		/* coupling channel */
      if (getcc(mip, cc_wnd, cc_wnd_shape, cc_coef, cc_gain) < 0)
        myexit("getcc");
      break;
#endif				
    case ID_DSE:		/* data element */
      if (getdata(&d_tag, &d_cnt, d_bytes) < 0)
        myexit("data channel");
      break;
    case ID_PCE:		/* program config element */
      get_prog_config(&prog_config);
      break;
    case ID_FIL:		/* fill element */
      getfill(d_bytes);
      break;
    default:
      PRINT(SE, "Element not supported: %d\n", ele_id);
      break;
    }
  }
  set_log_level(ID_END, 0, 0);  /* required for CRC and ADIF2MP4 */

  if(debug['v']) {
    PRINT(SE, "\nele_id %d\n", ele_id);
    print_mc_info(mip);
  }
  check_mc_info(mip, (bno==0 && default_config));
#ifdef	DRC	
  map_drc_channels(mip);
#endif

#ifndef	PARSE_ONLY
#if (ICChans > 0)
  /* process independently switched coupling channels */
#ifdef	DRC
  /* apply dynamic range control */ 
  i=0;
  for (ch=0; ch<mip->ncch; ch++) {
    if (mip->cc_ind[ch]) {
      apply_drc(cc_coef[ch], Chans+i);
      i++;	/* index for drc_info[] */
    }
  }
#endif
  if(!bno)
    for (ch=0; ch<Chans; ch++)
      wnd_shape[ch].prev_bk = wnd_shape[ch].this_bk;


  /* transform */
  ind_coupling(mip, cc_wnd, cc_wnd_shape, cc_coef,
               cc_state);
#endif

  /* m/s stereo */
  for (ch=0; ch<Chans; ch++) {
    cip = &mip->ch_info[ch];
    if ((cip->present) && (cip->cpe) && (cip->ch_is_left)) {
      wn = cip->widx;
      if(hasmask[wn]) {
        left = ch;
        right = cip->paired_ch;
        info = winmap[wnd[wn]];
        if (hasmask[wn] == 1)
          map_mask(info, group[wn], mask[wn], cb_map[right]);
        synt(info, group[wn], mask[wn], coef[right], coef[left]);
      }
    }
  }

  /* intensity stereo and prediction */
  for (ch=0; ch<Chans; ch++) {
    if (!(mip->ch_info[ch].present)) continue;
    wn = mip->ch_info[ch].widx;
    info = winmap[wnd[wn]];
#ifdef MPEG4V1
    pns( mip, info, ch,
         group[wn], cb_map[ch], factors[ch],
         lpflag[wn], coef );
#endif
    intensity(mip, info, ch, 
              group[wn], cb_map[ch], factors[ch], 
              lpflag[wn], coef);
#ifdef MPEG4V1
    if(mc_info.profile == LTP_Profile)
      nok_lt_predict (info, wnd[wn], 
		      wnd_shape[wn].this_bk,
		      wnd_shape[ch].prev_bk,
                      nok_lt_status[ch]->sbk_prediction_used,
                      nok_lt_status[ch]->sfb_prediction_used,
                      nok_lt_status[ch], nok_lt_status[ch]->weight,
                      nok_lt_status[ch]->delay, coef[ch],
                      BLOCK_LEN_LONG, tns[ch]);
    else
#endif 
    {
      predict(info, mip->profile, lpflag[wn], sp_status[ch], coef[ch]);
    }
  }

	
  for (ch=0; ch<Chans; ch++) {
    if (!(mip->ch_info[ch].present)) continue;
    wn = mip->ch_info[ch].widx;
    info = winmap[wnd[wn]];

    if(mc_info.profile == Main_Profile) {
      /* predictor reset */
      left = ch;
      right = left;
      if ( (mip->ch_info[ch].cpe) &&
           (mip->ch_info[ch].common_window) ) {
        /* prstflag's shared by channel pair */
        right = mip->ch_info[ch].paired_ch;
      }
          predict_reset(info, prstflag[wn], sp_status, left, right, last_rstgrp_num, lpflag[wn]);
        }

#ifdef MPEG4V1
    /* PNS predictor reset */
    predict_pns_reset(info, sp_status[ch], cb_map[ch]);
#endif


#if (CChans > 0)
    /* if cc_domain indicates before TNS */
    coupling(info, mip, coef, cc_coef, cc_gain, ch, CC_DOM, !CC_IND);
#endif

    /* tns */
    for (i=j=0; i<tns[ch]->n_subblocks; i++) {
      if (debug['T']) {
        PRINT(SE, "%ld %d %d\n", bno, ch, i);
        print_tns( &(tns[ch]->info[i]));
      }

      tns_decode_subblock(&coef[ch][j],
                          max_sfb[wn],
                          info->sbk_sfb_top[i],
                          info->islong,
                          &(tns[ch]->info[i]) );

      j += info->bins_per_sbk[i];
    }

#if (CChans > 0)
    /* if cc_domain indicated after TNS */
    coupling(info, mip, coef, cc_coef, cc_gain, ch, !CC_DOM, !CC_IND);
#endif
#ifdef	DRC
    /* apply dynamic range control to each audio channel */
    apply_drc(coef[ch], ch);
#endif    

    /* inverse transform */
    wnd_shape_SSR[ch].this_bk = wnd_shape[wn].this_bk;
    wnd_shape[ch].prev_bk = wnd_shape[wn].this_bk;

    if(mc_info.profile == SSR_Profile){
      gain_control(coef[ch], &wnd_shape_SSR[ch], wnd[wn], ch, data[ch]);
    }
    else {
      freq2time_adapt (wnd [wn], &wnd_shape_SSR[ch], coef [ch], state [ch], data [ch]);
    }

    wnd_shape_SSR[ch].prev_bk  = wnd_shape_SSR[ch].this_bk; 
    /*    freq2time_adapt (wnd [wn], &wnd_shape [wn], coef [ch], state [ch], data [ch]);*/
 
#if (CChans > 0)
    /* independently switched coupling */
    coupling(info, mip, data, cc_coef, cc_gain, ch, CC_DOM, CC_IND);
#endif

#ifdef MPEG4V1
    if(mc_info.profile == LTP_Profile)
      nok_lt_update(nok_lt_status[ch], data[ch], state[ch], BLOCK_LEN_LONG);
#endif
  } /* end chan loop */

  /* skip first two blocks so output is time aligned with input */
  if (bno > 1)
    writeout(data, mip);
#endif	    /* PARSE_ONLY */

  byte_align();

}
