/*
"This software module was originally developed by Fraunhofer Gesellschaft IIS 
in the course of development of the MPEG-2 NBC/MPEG-4 Audio standard ISO/IEC 13818-7, 
14496-1,2 and 3. This software module is an implementation of a part of one or more 
MPEG-2 NBC/MPEG-4 Audio tools as specified by the MPEG-2 NBC/MPEG-4 
Audio standard. ISO/IEC  gives users of the MPEG-2 NBC/MPEG-4 Audio 
standards free license to this software module or modifications thereof for use in 
hardware or software products claiming conformance to the MPEG-2 NBC/MPEG-4
Audio  standards. Those intending to use this software module in hardware or 
software products are advised that this use may infringe existing patents. 
The original developer of this software module and his/her company, the subsequent 
editors and their companies, and ISO/IEC have no liability for use of this software 
module or modifications thereof in an implementation. Copyright is not released for 
non MPEG-2 NBC/MPEG-4 Audio conforming products.The original developer
retains full right to use the code for his/her  own purpose, assign or donate the 
code to a third party and to inhibit third party from using the code for non 
MPEG-2 NBC/MPEG-4 Audio conforming products. This copyright notice must
be included in all copies or derivative works." 
Copyright(c)2001.
*/

#include "all.h"
#include "port.h"

/********************
 * CRC-calculations *
 ********************/

/* CRC-variables */
static int bitsToLog_CRC = 0, bitsToLog_CRCbuffer = 0, bits_in_CRCbuffer = 0;
static unsigned char buffer_CRC[16]; /* maximum of 128 bit in buffer */
static unsigned long crcRegister = 0xffff;

unsigned long reset_CRC()
     /*
      * reset CRC-calculator, returns current CRC-register
      */
{
  int retval = crcRegister & 0xffff;
  crcRegister = 0xffff;
  return retval;
}


void feed_CRC_calculator(int length, unsigned long value)
     /*
      * to be called at each getbits()-call;
      * CRC-calculation shuold be controlled with enable_CRC() and fillup_CRC()
      */
{
  unsigned long flag, tmp=0;
  int bitsToFeed = length, i;

  if ((bitsToLog_CRC < length)&&(bitsToLog_CRC >= 0)) {
    bitsToFeed = bitsToLog_CRC;
    tmp = bitsToLog_CRC-length;
  }

  if (length<32) value = value & ((1<<length)-1);

  bitsToLog_CRC-=bitsToFeed;

  while (bitsToFeed) {
    flag = crcRegister & 0x8000 ? 1:0;
    flag ^= (value>>(bitsToFeed-1-tmp)) & 1;
    crcRegister<<=1;
    if (flag) crcRegister^=0x8005;

    /*PRINT(SE,"crcReg:%08lx,pos:%i (%ld=>%ld)\n",crcRegister,bitsToLog_CRC,(value>>(bitsToFeed-1-tmp))&1,flag);*/

    bitsToFeed--;
  }

  bitsToFeed = length;
  if (bitsToLog_CRCbuffer < length) bitsToFeed = bitsToLog_CRCbuffer;
  if (bitsToFeed) {
    int where_byte = bits_in_CRCbuffer>>3;
    value>>= length-bitsToFeed;
    bitsToLog_CRCbuffer-= bitsToFeed;

    if (bits_in_CRCbuffer & 0x7) { /*fill up old byte in buffer_CRC*/
      int tmp=8-(bits_in_CRCbuffer & 0x7);
      if (tmp>bitsToFeed) tmp=bitsToFeed;

      bitsToFeed-=tmp;
      buffer_CRC[where_byte++] |= (unsigned char)((value >> bitsToFeed)<<(8-(bits_in_CRCbuffer&7)-tmp));
      bits_in_CRCbuffer+=tmp;
    }

    bits_in_CRCbuffer+= bitsToFeed;
    while (bitsToFeed >= 8) { /*add the full bytes to buffer_CRC*/
      bitsToFeed-=8;
      buffer_CRC[where_byte++]=(unsigned char)(value>>bitsToFeed);
    }
    if (bitsToFeed) { /*add the remainder*/
      buffer_CRC[where_byte]=(unsigned char)(value<<(8-bitsToFeed));
    }
  }
}


void enable_CRC(int length, int second)
     /*
      * enables CRC for <length> bits. These may be logged into a buffer, if <second> is set;
      * if <length> is negative, CRC is activated until fillup_CRC() is called
      */
{
  if (bitsToLog_CRC) {
    if (!second) myexit("CRC-logger: unexpected overlap\n");
    if (bits_in_CRCbuffer) myexit("CRC-logger: unexpected overlap\n");
    if ((length>128)||(length==-1)) myexit("CRC-logger: internal buffer exceeded\n");
    bitsToLog_CRCbuffer = length;
    bits_in_CRCbuffer = 0;
    return;
  }
  bitsToLog_CRC = length;
}


void fillup_CRC()
     /*
      * fill up remaining bits for CRC-calculation with zero;
      * clear buffered bits (if necessary), feeding them into feed_CRC_calculator()
      */
{
  if (bitsToLog_CRC > 0) {
    feed_CRC_calculator(bitsToLog_CRC,0);
  } else if (bitsToLog_CRC < 0) bitsToLog_CRC=0;

  if (bits_in_CRCbuffer) {
    unsigned char *tmp = buffer_CRC;
    int missing = bitsToLog_CRCbuffer;
    bitsToLog_CRCbuffer = 0;
    enable_CRC(bits_in_CRCbuffer+missing,0);

    while (bits_in_CRCbuffer >= 8) {
      feed_CRC_calculator(8,(unsigned char)*tmp++);
      bits_in_CRCbuffer-=8;
    }
    if (bits_in_CRCbuffer) feed_CRC_calculator(bits_in_CRCbuffer,*tmp>>(8-bits_in_CRCbuffer));
    bits_in_CRCbuffer=0;
    if (missing > 0) feed_CRC_calculator(missing,0);
  }
}


/************************
 * bitbuffer simulation *
 ************************/


/* variables for bitBuffer simulation */
int calculatedBitrate;
int channel_num;
int simulated_bitBuffer_len;
static int min_bitBuffer_len, max_bitBuffer_len;
static int start_bitBuffer_len, simulation_start_bno;
static long prev_byte_count, total_byte_count;
static double internal_bitBuffer_len;
static double avgFrameLen = 0;
static int use_fixed_avgFrameLen = 0;

#define START_OF_FRAME (rdb_in_frame<=0)
#define END_OF_FRAME (rdb_in_frame>=adts_header.num_of_rdb)

static void countbytes_bitBuffer_sim()
{
  static long internal_byte_count = 0;
  static int internal_block_count = 0;
  static int prev_fullness = bufferSizePerAudioChn>>5;

  prev_byte_count = byte_count-(nbits>>3);
  byte_count = (nbits>>3);     /* reset bytePerBlock couter, but be aware of nbits/cword! */

  internal_byte_count+= prev_byte_count;
  internal_block_count++;
  if (adts_header_present && START_OF_FRAME ) {
    /* if not other known, calculate avgFrameLen */
    if (!use_fixed_avgFrameLen) {
      avgFrameLen=(((((adts_header.adts_buffer_fullness-prev_fullness)<<2)*channel_num)+internal_byte_count)<<3)/internal_block_count;
    }
    prev_fullness = adts_header.adts_buffer_fullness;
    internal_byte_count = 0;
    internal_block_count = 0;
  }

  total_byte_count+= prev_byte_count;
}


int init_bitBuffer_sim(double frameLen, int fullness)
     /*
      * to be called when finished decoding the first block, or if it returned 0 before
      * returns 1 on success
      */
{
  int retval = 1;

  avgFrameLen = frameLen;
  min_bitBuffer_len = max_bitBuffer_len = simulated_bitBuffer_len = start_bitBuffer_len = fullness;
  internal_bitBuffer_len = fullness;

  if (avgFrameLen) use_fixed_avgFrameLen = 1;
  retval = (!(adts_header_present && adts_header.num_of_rdb)^(START_OF_FRAME && bno));

  if (debug['b'] && retval) PRINT(SE, "initiating buffer_fullness at %i\n", simulated_bitBuffer_len);

  countbytes_bitBuffer_sim();

  total_byte_count = 0;
  simulation_start_bno = bno;

  return retval;
}


void do_bitBuffer_sim()
     /*
      * if init_bitBuffer_sim succeded: to be called after decoding a block and before bno++
      */
{
  static double calculated_avgFrameLen;

  countbytes_bitBuffer_sim();

  internal_bitBuffer_len+=avgFrameLen;       /* these bytes were processed */
  internal_bitBuffer_len-=prev_byte_count<<3;/* these bytes arrived in bitBuffer */
  simulated_bitBuffer_len = (int)internal_bitBuffer_len;

  if (simulated_bitBuffer_len < min_bitBuffer_len) min_bitBuffer_len = simulated_bitBuffer_len;
  if (simulated_bitBuffer_len > max_bitBuffer_len) max_bitBuffer_len = simulated_bitBuffer_len;

  if (adts_header_present) {
    if (START_OF_FRAME) {
      calculated_avgFrameLen = ((double)((total_byte_count+((adts_header.adts_buffer_fullness<<2)*channel_num))<<3)-start_bitBuffer_len)/(bno-(simulation_start_bno));
    }
  } else
    calculated_avgFrameLen = ((double)(total_byte_count<<3)-start_bitBuffer_len+simulated_bitBuffer_len)/(bno-(simulation_start_bno));
  calculatedBitrate = (int)FRAMELEN2BITRATE(calculated_avgFrameLen);

  if (debug['R']) {
    PRINT(SE, " avgFrameLen: %5i (avg. %8.2f) ==> bitrate: %8i (avg. %8i)\n",
          (int)avgFrameLen, calculated_avgFrameLen,
          (int)FRAMELEN2BITRATE(avgFrameLen), calculatedBitrate);
  }
}


int check_bitBuffer_sim()
     /*
      * writes some diagnostic output
      */
{
  int retval=0;

  if (debug['B'])
    PRINT(SE, "bitBuffer-simulation: having %i free bit(processed: %i, got: %ld)\n",
          simulated_bitBuffer_len, (int)avgFrameLen, prev_byte_count<<3);

  if (debug['b']) {
    if (adts_header_present) {
      static long frame_byte_count = 0;

      frame_byte_count+=prev_byte_count;

      if ((START_OF_FRAME) && ((simulated_bitBuffer_len/channel_num)>>5 != adts_header.adts_buffer_fullness))
        PRINT(SE, "WARNING: ADTS-Header bufferFullness might be incorrect (%i != %i)\n",
              (adts_header.adts_buffer_fullness), ((simulated_bitBuffer_len/channel_num)>>5));

      if (END_OF_FRAME) {
        if (frame_byte_count!=adts_header.frame_length)
          PRINT(SE, "WARNING: ADTS-Header frame_length incorrect (%ld!=%i)\n",
                frame_byte_count, adts_header.frame_length);
        frame_byte_count = 0;
      }
    }

    if (simulated_bitBuffer_len > channel_num*bufferSizePerAudioChn - avgFrameLen) {
      PRINT(SE, "bitBuffer-simulation: WARNING, decoder had to wait for data\n");
      retval = -1;
    }
    if (simulated_bitBuffer_len < 0) {
      PRINT(SE, "bitBuffer-simulation: WARNING, decoder recieves too much data\n");
      retval = -1;
    }
    if ((max_bitBuffer_len-min_bitBuffer_len) > channel_num*bufferSizePerAudioChn - avgFrameLen) {
      PRINT(SE, "bitBuffer-simulation: ERROR: buffer_min_size: %i, buffer_max_size %i => buffer-limits exceeded\n",
            min_bitBuffer_len, max_bitBuffer_len);
      retval = -1;
    }
  }
  return retval;
}


