/************************* MPEG-2 NBC Audio Decoder **************************
 *                                                                           *
"This software module was originally developed by 
AT&T, Dolby Laboratories, Fraunhofer Gesellschaft IIS in the course of 
development of the MPEG-2 NBC/MPEG-4 Audio standard ISO/IEC 13818-7, 
14496-1,2 and 3. This software module is an implementation of a part of one or more 
MPEG-2 NBC/MPEG-4 Audio tools as specified by the MPEG-2 NBC/MPEG-4 
Audio standard. ISO/IEC  gives users of the MPEG-2 NBC/MPEG-4 Audio 
standards free license to this software module or modifications thereof for use in 
hardware or software products claiming conformance to the MPEG-2 NBC/MPEG-4
Audio  standards. Those intending to use this software module in hardware or 
software products are advised that this use may infringe existing patents. 
The original developer of this software module and his/her company, the subsequent 
editors and their companies, and ISO/IEC have no liability for use of this software 
module or modifications thereof in an implementation. Copyright is not released for 
non MPEG-2 NBC/MPEG-4 Audio conforming products.The original developer
retains full right to use the code for his/her  own purpose, assign or donate the 
code to a third party and to inhibit third party from using the code for non 
MPEG-2 NBC/MPEG-4 Audio conforming products. This copyright notice must
be included in all copies or derivative works." 
Copyright(c)1996.
 *                                                                           *
 ****************************************************************************/

#ifdef	DRC

#define	DEBUG	0
#if (DEBUG)
#define GETBITS( n,  msg ) debug_getbits((n),(msg))
#else
#define GETBITS( n,  msg ) getbits((n))
#endif

#include "all.h"
#include "port.h"

#if (DEBUG)
static long
debug_getbits(int n, char *s)
{
    long v;
    v = getbits(n);
    PRINT(SE, "%s %lx\n", s, v);
    return v;
}
#endif

/*
 * Dynamic Range Control 
 */

#define DRC_REF_LEVEL	(27*4)	/* -27 dB below FS (typical for movies) */
#define MAX_DRC_THREADS	3	/* could be more! */
#define MAX_CHAN	(FChans + SChans + BChans + LChans + ICChans)
#define MAX_DRC_BANDS	(1<<LEN_DRC_BAND_INCR)

typedef struct {
    int excl_chn_present;
    int excl_chn_mask[MAX_CHAN];
    int num_bands;
    int band_top[MAX_DRC_BANDS];
    int prog_ref_level_present;
    int prog_ref_level;
    int drc_sgn[MAX_DRC_BANDS];
    int drc_mag[MAX_DRC_BANDS];
} DRC_Bitstream;

typedef struct {
    int num_bands;
    int band_top[MAX_DRC_BANDS];
    int drc_sgn[MAX_DRC_BANDS];
    int drc_mag[MAX_DRC_BANDS];
} DRC_Info;

static int drc_enable;
static int drc_digital_norm;
static int drc_target_ref_level;
static Float drc_hi;
static Float drc_lo;
static int drc_prog_ref_level;
static int drc_thread;
static DRC_Bitstream drc_bits[MAX_DRC_THREADS];
static DRC_Info drc_info[MAX_CHAN];

/* 
 * initialize DRC
 */

void
set_drc(int enable, Float hi, Float lo)
{
    /* enable or disable drc */
    drc_enable = enable;
    
    /* hi and lo must be between 0.0 and 1.0, inclusive*/
    if (hi < 0)
	hi = 0;
    if (hi > 1.0)
	hi = 1.0;
    if (lo < 0)
	lo = 0;
    if (hi > 1.0)
	hi = 1.0;

    /* 0.0 indicates no drc, 1.0 indicates max drc */
    drc_hi = hi;    /* compress */
    drc_lo = lo;    /* boost */

    if (debug['v'] && drc_enable)
	PRINT(SE, "setting drc compression factors to %f %f\n", hi, lo);
}

void
init_drc(void)
{
    int i;
    DRC_Info *p;
       
    /* do program level normalization in digital domain */
    drc_digital_norm = 1;
    
    /* set desired program level */
    drc_target_ref_level = DRC_REF_LEVEL;
    drc_prog_ref_level = DRC_REF_LEVEL;
    
    /* set drc info structure to default values */
    for (i=0; i<MAX_CHAN; i++) {
	p = &drc_info[i];
	p->num_bands = 1; 
	p->band_top[0] = (LN2 / DRC_BAND_MULT) - 1;
	p->drc_sgn[0] = 0;
	p->drc_mag[0] = 0;
    }
}

/* 
 * parse DRC information
 */

void
reset_drc(void)
{
    /* set threads to zero */
    drc_thread = 0;
}

static int
excluded_channels(DRC_Bitstream *p)
{
    int i, j, n;
    int *mask = &p->excl_chn_mask[0];

    n=1;	    /* bytes */
    j=0;	    /* mask bits */
    
    for (i=0; i<7; i++)
	mask[j++] = GETBITS(1, "excl_chn");
	
    /* additional_excluded_chns */
    while (GETBITS(1, "more_excl_chn")) {
	for (i=0; i<7; i++)
	    mask[j++] = GETBITS(1, "excl_chn");
	n++;
    }
    return n;
}
 
int
dynamic_range_info(void)
{
    int i, n, accept;
    DRC_Bitstream *p, tmp_drc_bits;
    
    n = 1;	/* byte count */
    
    /* pce_tag_present */
    if (GETBITS(1, "pce_tag_present")) {
	accept = (current_program == GETBITS(LEN_TAG, "drc_tag")) ? 1 : 0;
	GETBITS(LEN_DRC_PCE_RESV, "reserved");
	n++;
    }
    else {
	/* default is to apply drc to current program */
	accept = 1;
    }
    p = (accept) ? &drc_bits[drc_thread++] : &tmp_drc_bits;
    
    /* excluded_chns_present */
    if ((p->excl_chn_present = GETBITS(1, "excl_chn_present"))) {
	/* get excl_chn_mask */
	n += excluded_channels(p);
    }
    
    /* drc_bands_present */
    p->num_bands = 1;
    if (GETBITS(1, "drc_bands_present")) {
	p->num_bands += GETBITS(LEN_DRC_BAND_INCR, "band_incr");
	GETBITS(LEN_DRC_BAND_RESV, "reserved");
	for (i=0; i<p->num_bands; i++) {
	    p->band_top[i] = GETBITS(LEN_DRC_BAND_TOP, "band_top");
	}
	n += 1 + p->num_bands;
    }
    
    /* prog_ref_level_present */
    if ((p->prog_ref_level_present = GETBITS(1, "prog_ref_level_present"))) {
	p->prog_ref_level = GETBITS(LEN_DRC_PL, "prog_ref_level");
	GETBITS(LEN_DRC_PL_RESV, "reserved");
	n++;
    }
    for (i=0; i<p->num_bands; i++) {
	p->drc_sgn[i] = GETBITS(LEN_DRC_SGN, "sgn");
	p->drc_mag[i] = GETBITS(LEN_DRC_MAG, "ctl");
    }
    n += p->num_bands;
    
    return n;
}

/* 
 * apply DRC
 */

static void
cp_drc(DRC_Info *p, DRC_Bitstream *b)
{
    int i;
    if ((p->num_bands = b->num_bands) == 1) {
	p->band_top[0] = (LN2 / DRC_BAND_MULT) - 1;
	p->drc_sgn[0] = b->drc_sgn[0];
	p->drc_mag[0] = b->drc_mag[0];
if(0) {i=0; PRINT(SE,"%d %d %4d %d %3d\n", 
p-&drc_info[0], i, p->band_top[i], p->drc_sgn[i], p->drc_mag[i]);}
    }
    else {
	for (i=0; i<p->num_bands; i++) {
	    p->band_top[i] = b->band_top[i];
	    p->drc_sgn[i] = b->drc_sgn[i];
	    p->drc_mag[i] = b->drc_mag[i];
if(0)PRINT(SE,"%d %d %4d %d %3d\n", 
p-&drc_info[0], i, p->band_top[i], p->drc_sgn[i], p->drc_mag[i]);
	}
    }
}

void
map_drc_channels(MC_Info *mip)
{
    int i, j, k, valid_chn, thread, *mask;
    int valid_thread[MAX_DRC_THREADS], valid_threads;
    int num_excl_chn[MAX_DRC_THREADS]; 
    DRC_Bitstream *b;
    
    /* calculate number of valid bits in excl_chn_mask */
    valid_chn = 0;
    for (i=0; i<Chans; i++)
	if (mip->ch_info[i].present)
	    valid_chn++;
    for (i=0; i<mip->ncch; i++)
	if (mip->cc_ind[i])
	    valid_chn++;

    /* check for valid threads */
    valid_threads = 0;
    for (i=0; i<drc_thread; i++) {
	b = &drc_bits[i];
	/* calculate number of excluded channels */
	num_excl_chn[i] = 0;
	if (b->excl_chn_present) {
	    mask = &b->excl_chn_mask[0];
	    for (j=0; j<valid_chn; j++)
		if (mask[j])
		    num_excl_chn[i]++;
	}
	if (num_excl_chn[i] == valid_chn) {
	    /* all channels are excluded! */
	    valid_thread[i] = 0;
	}
	else {
	    valid_thread[i] = 1;
	    valid_threads++; 
	}
    }
    
    if (valid_threads == 0) {
	/* nothing to do */
	return;
    }
    
    if (valid_threads == 1) {
	/* point to valid thread */
	for (i=0; i<drc_thread; i++)
	    if (valid_thread[i])
		break;

	if (num_excl_chn[i] == 0) {
	    /* apply this to all channels */
	    b = &drc_bits[i];
	    
	    if (b->prog_ref_level_present)
		drc_prog_ref_level = b->prog_ref_level;
		
	    /* SCE, CPE and LFE */
	    for (i=0; i<Chans; i++) {
		if (!mip->ch_info[i].present)
		    continue;   /* chn not present */
		cp_drc(&drc_info[i], b);
	    }
	    /* CCE */
	    for (j=0; j<mip->ncch; j++) {
		if (!mip->cc_ind[j])
		    continue;   /* chn not present */
		cp_drc(&drc_info[i+j], b);
	    }
	    return;
	}
    }
    
    if (valid_threads > 1) {
	/* check consistency of excl_chn_mask amongst valid drc threads */
	for (i=0; i<valid_chn; i++) {
	    int present = 0;
if (DEBUG)     if (debug['v'] && drc_enable) PRINT(SE,"%d: ", i);
	    for (thread=0; thread<drc_thread; thread++) {
		if (!valid_thread[thread])
		    continue;
		b = &drc_bits[thread];
if (DEBUG)     if (debug['v'] && drc_enable) PRINT(SE,"%d",b->excl_chn_mask[i]);
		if ( (num_excl_chn[thread] == 0) || (!b->excl_chn_mask[i]) ) 
		    /* thread applies to this channel */
		    present++;
	    }
if (DEBUG)     if (debug['v'] && drc_enable) PRINT(SE," %d\n", present);
	    if (present > 1) {
		/* a channel can appear in only one drc thread! */
		PRINT(SE, "Error in DRC: channel present in more than one DRC thread\n");
		myexit("drc thread check");
	    }	
	}
    }
    
    /* map DRC bitstream information onto DRC channel information */
    for (thread=0; thread<drc_thread; thread++) {
	if (!valid_thread[thread])
	    continue;
	    
	b = &drc_bits[thread];
	mask = b->excl_chn_mask;

	/* last prog_ref_level transmitted is the one that is used
	 * (but it should really only be transmitted once per block!)
	 */
	if (b->prog_ref_level_present)
	    drc_prog_ref_level = b->prog_ref_level;
	
	/* SCE, CPE and LFE */
	k = 0;  /* mask bit index */
	for (i=0; i<Chans; i++) {
	    if (!mip->ch_info[i].present)
		continue;   /* chn not present */
	    if (mask[k++])
		continue;   /* chn excluded from this thread */
	    cp_drc(&drc_info[i], b);
	}
	/* CCE */
	for (j=0; j<mip->ncch; j++) {
	    if (!mip->cc_ind[j])
		continue;   /* chn not present */
	    if (mask[k++])
		continue;   /* chn excluded from this thread */
	    cp_drc(&drc_info[i+j], b);
	}
    }
    return;
}

void
apply_drc(Float *coef, int ch)
{
    int band, bin, bottom, top;
    Float norm, factor;
    DRC_Info *p = &drc_info[ch];
    
    if (!drc_enable)
	return;
	
    /* If program reference normalization is done in the digital domain,
    modify factor to perform normalization.  prog_ref_level can
    alternatively be passed to the system for modification of the level in
    the analog domain.  Analog level modification avoids problems with
    reduced DAC SNR (if signal is attenuated) or clipping (if signal is
    boosted) */
    if (drc_digital_norm) {
	norm = pow(0.5, ((drc_target_ref_level - drc_prog_ref_level) / 24.0));
    }
    else {
	norm = 1.0;
    }
    
    bottom = 0;
    for (band=0; band<p->num_bands; band++) {
	/* apply the scaled dynamic range control words to factor.
	 * if scaling drc_hi (or drc_lo), or control word drc_mag is 0.0
	 * then there is no dynamic range compression
	 *
	 * if sgn is 
	 *	1 then gain is < 1
	 *	0 then gain is > 1
	 */
	if (p->drc_sgn[band]) {
	    /* compress */
	    factor = norm * pow(2.0, ((-drc_hi * p->drc_mag[band]) / 24.0));
	}
	else {
	    /* boost */
	    factor = norm * pow(2.0, ((drc_lo * p->drc_mag[band]) / 24.0));
	}
	
	if (DEBUG)
	        if (debug['v'] && drc_enable) PRINT(SE, "Chn %d Bnd %d (%4d-%4d) has drc %f %f %f\n",
	    ch, band, bottom, (p->band_top[band]+1)*DRC_BAND_MULT - 1, 
	    norm, factor/norm, factor);

	/* apply factor to spectral lines
	 *  short blocks must take care that bands fall on 
	 *  block boundaries!
	 */
	top = (p->band_top[band]+1)*DRC_BAND_MULT;
	for (bin=bottom; bin<top; bin++) {
	    coef[bin] *= factor;
	}
	bottom = top;
    }
}
#endif	/* define DRC */
