/************************* MPEG-2 NBC Audio Decoder **************************
 *                                                                           *
"This software module was originally developed by 
AT&T, Dolby Laboratories, Fraunhofer Gesellschaft IIS 
and edited by
Yoshiaki Oikawa (Sony Corporation),
Mitsuyuki Hatanaka (Sony Corporation)
Yasuhiro Toguri (Sony Corporation)
in the course of development of the MPEG-2 NBC/MPEG-4 Audio standard ISO/IEC 13818-7, 
14496-1,2 and 3. This software module is an implementation of a part of one or more 
MPEG-2 NBC/MPEG-4 Audio tools as specified by the MPEG-2 NBC/MPEG-4 
Audio standard. ISO/IEC  gives users of the MPEG-2 NBC/MPEG-4 Audio 
standards free license to this software module or modifications thereof for use in 
hardware or software products claiming conformance to the MPEG-2 NBC/MPEG-4
Audio  standards. Those intending to use this software module in hardware or 
software products are advised that this use may infringe existing patents. 
The original developer of this software module and his/her company, the subsequent 
editors and their companies, and ISO/IEC have no liability for use of this software 
module or modifications thereof in an implementation. Copyright is not released for 
non MPEG-2 NBC/MPEG-4 Audio conforming products.The original developer
retains full right to use the code for his/her  own purpose, assign or donate the 
code to a third party and to inhibit third party from using the code for non 
MPEG-2 NBC/MPEG-4 Audio conforming products. This copyright notice must
be included in all copies or derivative works." 
Copyright(c)1996.
 *                                                                           *
 ****************************************************************************/

#ifdef MPEG4V1
#ifdef USELIB_ISOMP4

#include "all.h"
#include "port.h"
#include "MP4Movies.h"


void
init_config(int extended_show)
{
  int objtype, samplingInfo, tmp;
  int nChannelConf;

  /* Audio Specific Configuration */
  objtype = getbits(lenAudioObjectType); /* from AudioSpecificConfig */

  if (extended_show) PRINT(SE, "Object type               %d\n",objtype);
  if(objtype > LTP_Profile + 1 || objtype < Main_Profile + 1 ){
    PRINT(SE, "Warning!!! Invalid Object Type in AudioSpecificConfig\n");
  }

  samplingInfo = getbits(lenSamplingFrequencyIndex);
  if (extended_show) PRINT(SE, "SamplingFrequencyIndex    %d\n",samplingInfo);

  nChannelConf = getbits(lenChannelConfiguration);
  if (extended_show) PRINT(SE, "ChannelConfiguration      %d\n",nChannelConf);
	
  /* GA Specific Configurtaion */
  tmp = getbits(lenFrameLengthFlag);
  if (extended_show) PRINT(SE, "FrameLengthFlag           %d\n",tmp);
  getbits(lenDependsOnCoreCoder);
  getbits(lenExtensionFlag);

  /* pce */

  if(nChannelConf==0) {
    get_prog_config(&prog_config);
    if (extended_show)  {
      PRINT(SE, "\n");
      print_PCE(&prog_config);
    }
    mc_info.profile = prog_config.profile; 
    if (extended_show) PRINT(SE, "ProfileID in PCE          %d\n",mc_info.profile);
    if(mc_info.profile > LTP_Profile || mc_info.profile < Main_Profile){
      PRINT(SE, "Warning!!! Invalid Profile ID in PCE\n");
    }
  }
  else {
    mc_info.profile = objtype -1 ; /* Use object type for AAC Profile */
    mc_info.sampling_rate_idx = samplingInfo;

    infoinit(&samp_rate_info[mc_info.sampling_rate_idx]);
  }
}


void init_mp4file(MP4Track* trak, MP4Handle* decoderSpecificInfoH, u32* avgBitrate)
{
    extern MP4Movie moov;
    MP4Err err;
    MP4Media media;
    MP4Handle initialObjectDescriptorH;

    u32 handlerType;
    u32 objectTypeIndication;
    u32 streamType;
    u32 decoderBufferSize;
    u32 tmp;

    if ((err = MP4NewHandle( 0, &initialObjectDescriptorH ))) myexit("mp4lib");
    if ((err = MP4GetMovieInitialObjectDescriptor( moov, initialObjectDescriptorH ))) myexit("mp4lib-initialOD");
    if ((err = MP4GetMovieIndTrack( moov, /*trackNumber*/ 1, trak ))) myexit("mp4lib-IndTrack");
    if ((err = MP4GetTrackMedia( *trak, &media ))) myexit("mp4lib-TrackMedia");
    if ((err = MP4GetMediaHandlerDescription( media, &handlerType, NULL ))) myexit("mp4lib-MediaDesc");
    if ((err = MP4NewHandle( 0, decoderSpecificInfoH ))) myexit("mp4lib");
    if ((err = MP4GetMediaDecoderInformation( media,
                                              /*sampleDescIndex*/1, 
                                              &objectTypeIndication, 
                                              &streamType, 
                                              &decoderBufferSize, 
                                              /*upstream*/&tmp, 
                                              /*maxBitrate*/&tmp, 
                                              avgBitrate, 
                                              *decoderSpecificInfoH ))) myexit("mp4lib-MediaDec");
    if ( handlerType != MP4AudioHandlerType ) {
      myexit("not an audio movie");
    }
    if ( objectTypeIndication != 0x40 ) {
      myexit("not an MPEG-4 movie");
    }
    if ( streamType != 0x05 ) {
      myexit("not an audio stream");
    }
    if ( decoderBufferSize > MAX_FRAME_LEN ) {
      myexit("required buffer size too large");
    }
}
#endif
#endif
