/************************* MPEG-2 NBC Audio Decoder **************************
 *                                                                           *
"This software module was originally developed by 
AT&T, Dolby Laboratories, Fraunhofer Gesellschaft IIS 
and edited by
Yoshiaki Oikawa (Sony Corporation),
Mitsuyuki Hatanaka (Sony Corporation)
Mike Coleman (Five Bats Research)
in the course of development of the MPEG-2 NBC/MPEG-4 Audio standard ISO/IEC 13818-7, 
14496-1,2 and 3. This software module is an implementation of a part of one or more 
MPEG-2 NBC/MPEG-4 Audio tools as specified by the MPEG-2 NBC/MPEG-4 
Audio standard. ISO/IEC  gives users of the MPEG-2 NBC/MPEG-4 Audio 
standards free license to this software module or modifications thereof for use in 
hardware or software products claiming conformance to the MPEG-2 NBC/MPEG-4
Audio  standards. Those intending to use this software module in hardware or 
software products are advised that this use may infringe existing patents. 
The original developer of this software module and his/her company, the subsequent 
editors and their companies, and ISO/IEC have no liability for use of this software 
module or modifications thereof in an implementation. Copyright is not released for 
non MPEG-2 NBC/MPEG-4 Audio conforming products.The original developer
retains full right to use the code for his/her  own purpose, assign or donate the 
code to a third party and to inhibit third party from using the code for non 
MPEG-2 NBC/MPEG-4 Audio conforming products. This copyright notice must
be included in all copies or derivative works." 
Copyright(c)1996,1997.

2000-10-18 : <kaehleof@iis.fhg.de> started using libaudio (AFsp)
 *                                                                           *
 ****************************************************************************/

#include <stdio.h>
#include <string.h>
#include "all.h"
#include "port.h"

#ifndef USELIB_AFsp
#include "aiff_support.h"
#else
#include <libtsp.h>
#include <libtsp/AFpar.h>
#endif

#ifdef ENABLE_RTPLAY
#include <audiolib.h>
#endif

#ifdef MPEG4V1
#ifdef USELIB_ISOMP4
#include "MP4Movies.h"

MP4Movie        moov;
static  char    *MP4AacBp; 
static  int     mp4_input_frameLen;
#endif
#endif

#define	PATH_LEN    128

#ifdef USELIB_AFsp
static	AFILE*	outfd[Chans];
#else
#ifdef ENABLE_RTPLAY
static	hAudioChannel	outfd[Chans];
#else
static	FILE*	outfd[Chans];
#endif
#endif
static	char	output_path[PATH_LEN];
static	FILE*	inputfile;


#if (defined USELIB_AFsp)||(defined ENABLE_RTPLAY)
static	unsigned int   output_multifiles = 0;
#else
static  unsigned long  output_frames     = 0;
#endif
static	char*          output_format     = "raw";
static	unsigned short output_channels   = 1;
static	unsigned short output_wordsize   = 16;
static	unsigned long  output_sampleRate = 48000;
static	MC_Info*       output_MC_Info    = 0;
/*static*/  long       byte_count        = 0;

#ifdef ENABLE_RTPLAY
static  int            output_realtime   = 0;
static  AuChanInfo     outinfo;
#endif 


int              (*getbyte)(void);

void
usage(char *s)
{
#ifdef MPEG4V1
    PRINT(SE, "MPEG4V1 AAC Decoder\n");
#else
    PRINT(SE, "MPEG2 AAC Decoder\n");
#endif
    PRINT(SE, "usage:  %s [options] chan_file pcm_file\n", s);
#ifdef ENABLE_RTPLAY
    PRINT(SE, "	  if pcm_file is '-' the output is played realtime\n");
#endif
    PRINT(SE, "	options are:\n");
    PRINT(SE, "	-i program_number (ADIF header present, 0 is first program)\n");
    PRINT(SE, "	-t  (ADTS header present)\n");
    PRINT(SE, "	-s sampling_frequency, Hz   (48 kHz is default)\n");
    PRINT(SE, "	-a  (create AIFF output files)\n");
#if (defined USELIB_AFsp)||(defined ENABLE_RTPLAY)
    PRINT(SE, "	-w  (create WAV output files)\n");
    PRINT(SE, "	-3  (create 24bit output files)\n");
    PRINT(SE, "	-m  (create multiple output files)\n");
#endif
#ifdef ENABLE_RTPLAY
    PRINT(SE, "	-ole [1-100] (parameter to control volume)\n");
#endif
    PRINT(SE, "	-D [a-Z]    (enable debugging printouts)\n");
#ifndef MPEG4V1
    PRINT(SE, "	-p profile  (Main or LC or SSR, Main is default)\n");
#else
    PRINT(SE, "	-p profile  (Main or LC or SSR or LTP, Main is default)\n");
#endif
    PRINT(SE, "	-b Band For SSR Profile	(1 to 4, 4 is default)\n");
#ifdef	DRC
    PRINT(SE, "	-d cut_factor boost_factor\n");
    PRINT(SE, "    where 0.0 is no compression, 1.0 is max compression\n");
    PRINT(SE, "    %d Channels Supported\n",Chans-XChans-BCenter);
#endif
    myexit(0);
}

int
initio(int argc, char *argv[])
{
    int i, j, *p;
    char *cmd, *s;
    int getbyte_MP2(void);
    int getbyte_MP4(void);

    /* set defaults */
    getbyte = &getbyte_MP2;
    adif_header_present = 0;
    adts_header_present = 0;
#ifdef MPEG4V1
#ifdef USELIB_ISOMP4
    mpeg4_header_present = 0;
#endif
#endif

    default_config = 1;
    current_program = -1;
    mc_info.profile = Main_Profile;
    mc_info.sampling_rate_idx = Fs_48;
#ifdef	DRC
    set_drc(0, 0, 0);
#endif

    /* save cmd 
     * ARGBEGIN is so clever that it throws away argv[0] and 
     * increments the argv pointer
     */
    cmd = argv[0];

    ARGBEGIN {


    default:
	PRINT(SE, "unknown option\n");
	usage(cmd);
    case 'a':
    	output_format = "aiff";
    	break;
#if (defined USELIB_AFsp)||(defined ENABLE_RTPLAY)
    case 'w':
    	output_format = "wav";
    	break;
    case '3':
        output_wordsize = 24;
        break;
    case 'm':
        output_multifiles = 1;
        break;
#endif
#ifdef ENABLE_RTPLAY
    case 'o':
        if (!strcmp(ARGF(),"le")) {
          s = ARGF();
          outinfo.typeInfo.rt.level = atoi(s);
        }
        break;
#endif
    case 'i':
	adif_header_present = 1;
	s = ARGF();
	current_program = atoi(s);
	if((current_program < 0) || (current_program > 15)) {
	    PRINT(SE, "program number out of range: %d\n", current_program);
	    myexit("PCE");
	}
	if (debug['v'])
	    PRINT(SE, "program with tag %d selected\n", current_program);
	break;
    case 'p':
	s = ARGF();
	p = &mc_info.profile;
	if ((!strncmp(s, "Main", 4)) || (!strncmp(s, "main", 4))) {
	    *p = Main_Profile; 
	    break;
	}
	if ((!strncmp(s, "LC", 2)) || (!strncmp(s, "lc", 2))) {
	    *p = LC_Profile; 
	    break;
	}

	if ((!strncmp(s, "SSR", 3)) || (!strncmp(s, "ssr", 3))) {
	  *p = SSR_Profile;
	  break;
	}

#ifdef  MPEG4V1
	if ((!strncmp(s, "LTP", 3)) || (!strncmp(s, "ltp", 3))) {
	  *p = LTP_Profile;
	  break;
	}
#endif
	PRINT(SE, "Unsupported profile %s\n", s);
	myexit("profile");
	break;
    case 't':
        adts_header_present=1;
        break;
    case 's':
	j = atoi(ARGF());
	for (i=0; i<(1<<LEN_SAMP_IDX); i++) {
	    if (j == samp_rate_info[i].samp_rate)
		break;
	}
	if (i == (1<<LEN_SAMP_IDX)) {
	    PRINT(SE, "Unsupported sampling frequency %d\n", j);
	    myexit("samling_rate_idx");
	}
	mc_info.sampling_rate_idx = i;
	output_sampleRate = samp_rate_info[i].samp_rate;
	break;
    case 'D':
	for (s = ARGF(); *s; s++) {
	    debug[(int)(*s)] = 1;
	    PRINT(SE, "debug option %c enabled\n", *s);
	}
	break;
    case 'b':
	s = ARGF();
	ssr_decoder_band = atoi(s);
	if(ssr_decoder_band == 1)
	    PRINT(SE, "%d-band is decoded in SSR profile\n", ssr_decoder_band);
	else
	    PRINT(SE, "%d-bands are decoded in SSR profile\n", ssr_decoder_band);
	break;
#ifdef	DRC
    case 'd':
	{
	Float cut, boost;
	s = ARGF();
	cut = atof(s);
	s = ARGF();
	boost = atof(s);
	set_drc(1, cut, boost);
	}
	break;
#endif
    } 
    ARGEND

    if((argc != 1)&&(argc != 2)) {
      usage(cmd);
    }

    inputfile = fopen(argv[0], "rb");
    if(inputfile == 0) {
		 PRINT(SE, "cant open %s\n", argv[0]);
		 myexit("open");
    }
    /* check if bitstream has an ADIF header */
    {
      char adif_id[4];

      fread(adif_id, sizeof(char), 4, inputfile);
      fseek(inputfile, 0, SEEK_SET);
      if(!strncmp(adif_id, "ADIF", 4)) {
        adif_header_present = 1;
        if (debug['n'])
          PRINT(SE, "ADIF header present\n");
      }
#ifdef MPEG4V1
#ifdef USELIB_ISOMP4
      else {
        if (!MP4OpenMovieFile( &moov, argv[0], MP4OpenMovieNormal)) {
          /* seems to be a .MP4 */
          mpeg4_header_present = 1;
          getbyte = &getbyte_MP4;
          if (debug['n'])
            PRINT(SE, "MPEG4 header present\n");
          /* select first (and only) prog in decoderSpecificInfo */
          current_program = -1;
        }
      }
#endif
#endif

      if ((adif_header_present == 0)
#ifdef MPEG4V1
#ifdef USELIB_ISOMP4
          &&(mpeg4_header_present == 0)
#endif
#endif
          ) {
        if (((adif_id[0]&0xff)==0xff)&&((adif_id[1]&0xf0)==0xf0)) {
          adts_header_present = 1; /* found adts-syncword */
          if (debug['n'])
            PRINT(SE, "ADTS syncword present\n");
        }
      }
    }

    if (
#ifdef MPEG4V1
#ifdef USELIB_ISOMP4
        ((adif_header_present == 1)&&(mpeg4_header_present == 1))||
        ((adts_header_present == 1)&&(mpeg4_header_present == 1))||
#endif
#endif
        ((adif_header_present == 1)&&(adts_header_present == 1)))
    {
      PRINT(SE,"Header Error!! More than one header specified (ADIF or ADTS or MPEG4).\n");
      exit(0);
    }

    if (argc==1) {  /* only one filename on commandline => show header */
      return 1;
    }
    /* two filenames on commandline => prepare output file */

#ifdef ENABLE_RTPLAY
    if (!strcmp(argv[1],"-")) { /* prepare realtime output */
      output_realtime = 1;
      output_multifiles = 0;
      return argc;
    }
#endif

    if (strlen(argv[1]) + strlen("_f00.pcm") + 1 > PATH_LEN) {
		 PRINT(SE, "output path too long %s\n", argv[1]);
		 myexit("create");
    }

    strcpy(output_path, argv[1]);
    return argc;
}


/* -- functions for output-files -- */
static void
write_fext(Ch_Info *cip, char c, int n)
{
	cip->fext = (char *)mal1(5);
	sprintf(cip->fext, "_%c%02d", c, n);
}

int
open_output_files(MC_Info *mip)
{
    int i, j = 0;
    char file[PATH_LEN];

#ifdef USELIB_AFsp

    int format_id;

    struct {
      char *str;
      int fmt;
    } fmtstr[] = {    /* these types are also "auto-detected" from output-argument */
      /*{"au",FTW_AU*256},
        {"snd",FTW_AU*256},*/
      {"wav",FTW_WAVE*256},
      {"aiff",FTW_AIFF_C*256},
      /*{"aif",FTW_AIFF_C*256},
        {"aifc",FTW_AIFF_C*256},*/
      {"pcm",FTW_NH_EB*256},      /* no header big-endian */
      {"raw",FTW_NH_EB*256},      /* no header big-endian */
      {NULL,-1}
    };
#else
#ifdef ENABLE_RTPLAY
    AuChanType auChnType = TYPE_RT;
    AuChanError err;
    int format_id;
    struct {
      char *str;
      int fmt;
    } fmtstr[] = {
      {"wav",TYPE_RIFF},
      {"aiff",TYPE_AIFF},
      {"pcm",TYPE_PCM},
      {"raw",TYPE_PCM},
      {NULL,-1}
    };
#endif
#endif

    output_MC_Info = mip;
    output_sampleRate = samp_rate_info[mip->sampling_rate_idx].samp_rate; 

#if (defined USELIB_AFsp)||(defined ENABLE_RTPLAY)
    for (i=strlen(output_path);(i>=(signed int)(strlen(output_path)-5))&&(i>=0);i--) {
      if (output_path[i]=='.') {
        j=-1;
        while ( fmtstr[++j].str ) {
          if (!strcmp(fmtstr[j].str,(output_path+i+1))) {
            output_format=output_path+i+1;
            *(output_path+i)=0;
            break;
          }
        }
      }
    }

    i=-1;j=0;
    while ( fmtstr[++i].str )
      if (!strcmp(fmtstr[i].str, output_format)) break;

    if ( fmtstr[i].str == NULL ) {
      PRINT(SE, "Internal Error: Unknown output-format\n");
      myexit("format");
    }

    format_id = fmtstr[i].fmt;

#ifdef USELIB_AFsp
    format_id += ((output_wordsize == 24)? FD_INT24 : FD_INT16);
#endif
#endif

    /* if FCenter == 1 (center speaker present)
     *	position    index   extension
     *	left	    1	    01
     *	center	    0	    00
     *	right	    2	    02
     * if FCenter == 0 (center speaker absent)
     *	position    index   extension
     *	left	    0	    01
     *	right	    1	    02
     */
    for (i=((FCenter==1) ? 0 : 1); i<FChans; i++) {
      write_fext(&mip->ch_info[j], 'f', i);
      j++;
    }
    for (i=0; i<SChans; i++) {
      write_fext(&mip->ch_info[j], 's', i);
      j++;
    }
    /* if BFCenter == 1 (center speaker present)
     *	position    index   extension
     *	left	    3	    00
     *	center	    5	    02
     *	right	    4	    01
     * if BCenter == 0 (center speaker absent)
     *	position    index   extension
     *	left	    3	    00
     *	right	    4	    01
     */
    for (i=0; i<BChans; i++) {
      write_fext(&mip->ch_info[j], 'b', i);
      j++;
    }
    for (i=0; i<LChans; i++) {
      write_fext(&mip->ch_info[j], 'l', i);
      j++;
    }

#if (defined USELIB_AFsp)||(defined ENABLE_RTPLAY)
    if (!output_multifiles) {
      j=0;
      for (i=0; i<Chans; i++) 
        if (mip->ch_info[i].present) j++;
      if (j>2) {
        output_multifiles = 1;
        PRINT(SE, "warning: no standards for %i channels in one file... using multiple\n", j);
      } else {
        output_channels = j;
      }
    }
#endif
#ifdef ENABLE_RTPLAY
    if (output_realtime) {
      auChnType = TYPE_RT;
      if ((output_channels>2)||(output_multifiles)) {
        PRINT(SE, "error: can't play so many channels realtime\n");
        myexit(0);
      }
      outinfo.valid = 1;
      outinfo.bitsPerSample = 16;
      outinfo.sampleRate = output_sampleRate;
      outinfo.nChannels = output_channels;
      outinfo.nSamples = LN2;
      outinfo.fpScaleFactor = 32768.0;
      if (!outinfo.typeInfo.rt.level) outinfo.typeInfo.rt.level = 50; /* output Volume */
      outinfo.typeInfo.rt.outputDrain = OUT_SPEAKER;
      err = AuChannelOpen(&outfd[0], NULL/*outname*/, (AuChanMode)AU_CHAN_WRITE, &auChnType, &outinfo);
      if (err != AU_CHAN_OK ) {
        PRINT(SE, "AuChannelOpen reported an error\n" );
        myexit(0);
      }
    } else {
      auChnType = (enum AuChanType)format_id;
      outinfo.valid = 1;
      outinfo.bitsPerSample = output_wordsize;
      outinfo.sampleRate = output_sampleRate;
      outinfo.nChannels = output_channels;
      outinfo.nSamples = LN2;
      outinfo.fpScaleFactor = 32768.0;
#endif

#if (defined USELIB_AFsp)||(defined ENABLE_RTPLAY)
      if (output_multifiles) { /* output several files */
#endif
        j=0;
        for (i=0; i<Chans; i++) {
          if (!(mip->ch_info[i].present))
            continue;

          j++;

          strcpy(file, output_path);
          strcat(file, mip->ch_info[i].fext);
          strcat(file, ".");
          strcat(file, output_format);

          if (debug['o'])
            PRINT(SE, "opening file %s at position %d\n", file, i);

#ifdef ENABLE_RTPLAY
          err = AuChannelOpen(&outfd[i], file, (AuChanMode)AU_CHAN_WRITE, &auChnType, &outinfo);
          if (err != AU_CHAN_OK ) {
            PRINT(SE, "AuChannelOpen reported an error\n" );
            myexit("create");
          }
#else
#ifdef USELIB_AFsp
          outfd[i] = AFopenWrite(file, format_id, output_channels, output_sampleRate, (FILE*)NULL );
          if(outfd[i] == NULL) {
            PRINT(SE, "cant create %s\n", file);
            myexit("create");
          }
#else
          /* no AFsp-lib support */
          outfd[i] = fopen(file, "w");
          if(outfd[i] == NULL) {
            PRINT(SE, "cant create %s\n", file);
            myexit("create");
          }
          if ( !strcmp(output_format,"aiff") ) {
            /* write dummy header and position for audio data */
            if ( WriteAIFFHeader(outfd[i], output_channels, output_frames, output_wordsize, output_sampleRate) != 0 ) {
              PRINT( SE, "cannot write AIFF header for %s\n", file );
              myexit( "aiff-header" );
            }
          }
#endif
#endif
        }
#if (defined USELIB_AFsp)||(defined ENABLE_RTPLAY)
      } else {   /* place multiple channels in one file */
        strcpy(file, output_path);
        strcat(file, ".");
        strcat(file, output_format);

        if (debug['o'])
          PRINT(SE, "opening file %s with %d channels\n", file, output_channels);
#ifdef USELIB_AFsp
        outfd[0] = AFopenWrite(file, format_id, output_channels, output_sampleRate, (FILE*)NULL );

        if(outfd[0] == NULL) {
          PRINT(SE, "cant create %s\n", file);
          myexit("create");
        }
#else
        err = AuChannelOpen(&outfd[0], file, (AuChanMode)AU_CHAN_WRITE, &auChnType, &outinfo);
        if (err != AU_CHAN_OK ) {
          PRINT(SE, "AuChannelOpen reported an error\n" );
          myexit(0);
        }
#endif
      }
#endif
#ifdef ENABLE_RTPLAY
    }
#endif
    return(1);
}

#if !((defined USELIB_AFsp)||(defined ENABLE_RTPLAY))
static void
fmtchan(char *p, Float *data, int stride)
{
  int i, c;
  Float s;

  for(i=0; i<LN2; i++) {
    s = *data++;
    if(s < 0) {
      s -= .5;
      if(s < -0x7fff)
        s = (Float) -0x7fff;
    } else {
      s += .5;
      if(s > 0x7fff)
        s = (Float) 0x7fff;
    }
    c = (int) s;

    p[0] = (c >> 8) & 0xff;
    p[1] = c & 0xff;

    p += stride;
  }
}
#endif


void
writeout(Float **data, MC_Info *mip)
{
   int i;
   static int opened = 0;

#if (defined USELIB_AFsp)||(defined ENABLE_RTPLAY)
   int j;
   float fl_obuf[LN2*Chans];
#else
   char obuf[LN2*2];
   int stride = 2, size = sizeof(obuf);
#endif
   
   if (!opened) {
      open_output_files(mip);
      opened=1;
   }

#if (defined ENABLE_RTPLAY)||(defined USELIB_AFsp)
   if (!output_multifiles) { /* multiple channels into one file */
     int k=0;
     for (i=0; i<Chans; i++) {
       if (!(mip->ch_info[i].present))
         continue;
       for (j=0; j<LN2; j++)
         fl_obuf[j*output_channels+k] = data[i][j];
       k++;
     }
#ifdef ENABLE_RTPLAY
     AuChannelWriteFloat( outfd[0], fl_obuf, LN2*output_channels, &i /*written bytes*/);
#endif
#ifdef USELIB_AFsp
     AFwriteData(outfd[0], fl_obuf, LN2*output_channels);
#endif
   } else {
#endif
     for (i=0; i<Chans; i++) {
       if (!(mip->ch_info[i].present))
         continue;

       for (j=0; j<LN2; j++)
         fl_obuf[j] = data[i][j];

#ifdef USELIB_AFsp
       AFwriteData(outfd[i], fl_obuf/*data[i]*/, LN2);
#else
#ifdef ENABLE_RTPLAY
       AuChannelWriteFloat( outfd[i], fl_obuf/*data[i]*/, LN2, &j /*written bytes*/);
#else
       fmtchan(obuf, data[i], stride);
       if(fwrite(obuf, 1, size, outfd[i]) != (size_t)size) {
         PRINT(SE, "write error\n");
         myexit("write");
       }
#endif
#endif
     }
#if ((defined USELIB_AFsp)||(defined ENABLE_RTPLAY))
   }
#else
   if ( !strcmp(output_format,"aiff") )
     output_frames += LN2;
#endif
}


void set_log_level(int id, int add_bits_num, int add_bits)
{
  switch(id) {
  case ID_SCE:
  case ID_CPE:
  case ID_CCE:
  case ID_LFE: /*CRC-log 192 bit*/
    enable_CRC(CRC_LEN_ELE,0);
    feed_CRC_calculator(add_bits_num, add_bits);
    break;
  case ID_2ND: /*CRC-log additional 128 bit*/
    enable_CRC(CRC_LEN_2ND,1);
    break;
  case ID_HDR: /*CRC-log 56bit*/
    enable_CRC(CRC_LEN_HDR,0);
    break;
  case ID_DSE:
  case ID_PCE: /*CRC-log until LOG_FIN*/
    enable_CRC(CRC_LEN_ALL,0);
    feed_CRC_calculator(add_bits_num, add_bits);
    break;
  case ID_FIL:
  case ID_NUL: /*no CRC*/
    break;
  case LOG_FIN: /*CRC finish (zero-pad)*/
    fillup_CRC();
    break;
  case ID_END: {/*no CRC*/
    unsigned long crcValue;
    if (rdb_in_frame>=adts_header.num_of_rdb) {
      crcValue = reset_CRC();
      if ((debug ['C'])&&(adts_header_present)&&(!adts_header.protection_abs)) {
        if (crcValue!=(unsigned long)adts_header.crc_check)
          PRINT(SE,"CRC: 0x%04lx, but header said : 0x%04x => FAIL\n",crcValue,adts_header.crc_check);
      }
    }
    break;
  }
  default:
    myexit("logger: HELP!!!");
  }
}


/* -- functions for reading the inputfile -- */
void
restartio(
#ifdef MPEG4V1
#ifdef USELIB_ISOMP4
          char *p, int size
#endif
#endif
          )
{
    cword = 0;
    nbits = 0;
    inerror = 0;
    byte_count = 0;
#ifdef MPEG4V1
#ifdef USELIB_ISOMP4
    if ( mpeg4_header_present == 1 ) {
      MP4AacBp = p;
      mp4_input_frameLen = size;
    }
#endif
#endif
}

int 
getbyte_MP2(void)
{
  int c;
  static unsigned short errorCounter = 0;
  c = getc(inputfile);
  if(c < 0) {
    errorCounter++;
    if ( errorCounter > 3 ) {
      myexit((debug['n']) ?
             (char *)"getbyte: End of Bitstream" : (char *)0);
    }
    c = 0;
  }

  byte_count++;
  return c;
}

#ifdef MPEG4V1
#ifdef USELIB_ISOMP4
int getbyte_MP4(void)
{
  if (byte_count++<mp4_input_frameLen) {
    return (unsigned char) *MP4AacBp++;
  } else return 0;
}
#endif
#endif

/*static long
getshort(void)
{
    int c1, c2;

    c1 = (*getbyte)();
    c2 = (*getbyte)();
    if(c2 < 0)
        c2 = 0;
    if(c1 < 0)
        c1 = 0;

    if (debug['G'])
	PRINT(SE,"get: %02x %02x %ld\n", c1, c2, byte_count);
    return (c1<<8) | c2;
}*/

long
getbits(int n)
{
  int retval;
  if (!n) return 0;

  if(nbits <= 16){
    int c1, c2;
    if ((c1 = (*getbyte)()) < 0) c1=0;
    if ((c2 = (*getbyte)()) < 0) c2=0;

    cword = (cword<<16) | /*getshort()*/(c1<<8) | c2;
    nbits += 16;
  }

  nbits -= n;
  retval = (cword>>nbits) & ((1<<n)-1);
  feed_CRC_calculator(n, retval);
  return retval;
}

/* -- nice exit functions -- */
void
myexit(char *s)
{
  closefiles();
  if (debug['v'])
    PRINT(SE,"%lx %d\n", cword, nbits);
  if (s)
    PRINT(SE,"\n%s\n", s);
  exit(s ? 1 : 0);
}

void
closefiles()
{
  int i;

  for ( i = 0; i < Chans; i++ ) {
    if ( !outfd[i] )
      continue;
#ifdef USELIB_AFsp
    AFclose(outfd[i]);
#else
#ifdef ENABLE_RTPLAY
    AuChannelClose( outfd[i] );
#else
  if ( !strcmp(output_format,"aiff") ) 
    WriteAIFFHeader( outfd[i], output_channels, output_frames, output_wordsize, output_sampleRate );
  fclose( outfd[i] );
#endif
#endif
  }
}
