/**********************************************************************
MPEG-4 Audio VM
Deocoder cores (parametric, LPC-based, t/f-based)



This software module was originally developed by

Markus Werner (SEED / Software Development Karlsruhe) 

and edited by

Markus Werner (SEED / Software Development Karlsruhe) 

in the course of development of the MPEG-2 NBC/MPEG-4 Audio standard
ISO/IEC 13818-7, 14496-1,2 and 3. This software module is an
implementation of a part of one or more MPEG-2 NBC/MPEG-4 Audio tools
as specified by the MPEG-2 NBC/MPEG-4 Audio standard. ISO/IEC gives
users of the MPEG-2 NBC/MPEG-4 Audio standards free license to this
software module or modifications thereof for use in hardware or
software products claiming conformance to the MPEG-2 NBC/ MPEG-4 Audio
standards. Those intending to use this software module in hardware or
software products are advised that this use may infringe existing
patents. The original developer of this software module and his/her
company, the subsequent editors and their companies, and ISO/IEC have
no liability for use of this software module or modifications thereof
in an implementation. Copyright is not released for non MPEG-2
NBC/MPEG-4 Audio conforming products. The original developer retains
full right to use the code for his/her own purpose, assign or donate
the code to a third party and to inhibit third party from using the
code for non MPEG-2 NBC/MPEG-4 Audio conforming products. This
copyright notice must be included in all copies or derivative works.

Copyright (c) 2000.

$Id: decifc.c,v 1.1.2.13 2001/05/16 13:56:42 cvs03 Exp $ 
Decoder high level interface
**********************************************************************/
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include "decifc.h"
#include "bitstream.h"
#include "dec_tf.h"
#include "dec_lpc.h"
#include "dec_par.h"
#include "common_m4a.h"

#define MAX_AU 12
#define MAX_BITBUF 256000


static const char *codecNames [] =
{
  "Null Object - Linear",
  "AAC Main Profile",
  "AAC Low Complexity Profile",
  "AAC Scalable Sampling Rate Profile",
  "AAC Long Term Predictor (LTP)",
  "5",
  "AAC Scalable",
  "TwinVQ",
  "CELP",
  "HVXC",
  "10",
  "11",
  "TTSI (not supported)",
  "Main Synthetic (not supported)",
  "Wavetable Synthesis (not supported)",
  "General MIDI (not supported)",
  "Algorithmic Synthesis and Audio FX (not supported)",
  "Error Resilient AAC Low Complexity",
  "19",
  "Error Resilient AAC Long Term Predictor",
  "Error Resilient AAC Scalable Object",
  "Error Resilient TwinVQ Object",
  "Error Resilient BSAC Object",
  "Error Resilient AAC LD Object",
  "Error Resilient CELP Object",
  "Error Resilient HVXC Object",
  "Error Resilient HILN Object",
  "Error Resilient Parametric Object",
  "28",
  "29",
  "30",
  "31"
} ;


int audioDecInit(int  streamCount, /* in: number of ESs */
                 DecoderConfigDescr decConfig [MAX_TRACKS],
                 HANDLE_DECODER_GENERAL hFault,
                 DEC_DATA       *decData,
                 DEC_DEBUG_DATA *decDebugData)
{

  int layer ;
  int epFlag=0;
  int epDebugLevel=0;
  int delayNumSample = 0;
  FRAME_DATA *fD;
  
  /*
    clear decData
  */
  memset(decData,sizeof(DEC_DATA),0);

  /*
    Alloc FrameData
  */
  if(!(decData->frameData = calloc(1,sizeof(FRAME_DATA))))
    return(1);
  
  fD = decData->frameData;
  fD->scalOutSelect = streamCount-1;
  fD->scalOutObjectType = decConfig [streamCount-1].audioObjectType;
  fD->scalOutNumChannels = decConfig [streamCount-1].channelConfiguration;
 
  
  if(!(fD->od = calloc(1,sizeof(OBJECT_DESCRIPTOR))))
    return(1);

  initObjDescr(fD->od,decDebugData->mainDebugLevel);
  presetObjDescr(fD->od,streamCount);

  for ( layer = 0; layer < streamCount; layer++ )   {
    /* goes through all ESs, which outnumbers layers for epConfig==1 */
   
    AUDIO_SPECIFIC_CONFIG *audSpC ;
    fD->layer[layer].AULength = (unsigned int *)malloc(MAX_AU*sizeof(unsigned int));
    
    initESDescr(&(fD->od->ESDescriptor[layer]));
    
    audSpC = &fD->od->ESDescriptor[layer]->DecConfigDescr.audioSpecificConfig;

    audSpC->audioDecoderType.value = decConfig [layer].audioObjectType ;
    audSpC->channelConfiguration.value = decConfig [layer].channelConfiguration ;
    audSpC->samplingFreqencyIndex.value = decConfig [layer].samplingFrequencyIndex ;
    audSpC->samplingFrequency.value = decConfig [layer].samplingFrequency ;
    audSpC->epConfig.value = decConfig[layer].epConfig;

    
    
    fD->od->ESDescriptor[layer]->DecConfigDescr.avgBitrate.value = decConfig [layer].avgBitrate ;
   
    fD->layer [layer].bitBuf = BsAllocBuffer(MAX_BITBUF);
    fD->layer [layer].sampleRate = decConfig [layer].samplingFrequency ;
    fD->layer [layer].bitRate = decConfig [layer].avgBitrate ;

    DebugPrintf(2,"ES (Layer):%d", layer);

    if (decConfig [layer].audioObjectType < (sizeof (codecNames) / sizeof (codecNames [0])))
      DebugPrintf(2,"  audioObjType   :%s", codecNames [decConfig [layer].audioObjectType] );
    else
      DebugPrintf(2,"  audioObjType   :%d", decConfig [layer].audioObjectType );

    DebugPrintf(2,"  chanConfig     :%ld", decConfig [layer].channelConfiguration );
    DebugPrintf(2,"  samplFreqIdx   :%ld", decConfig [layer].samplingFrequencyIndex);
    DebugPrintf(2,"  samplFreq      :%ld", decConfig [layer].samplingFrequency  );
    DebugPrintf(2,"  avgBitrate     :%ld", decConfig [layer].avgBitrate );
    
    switch (decConfig[layer].audioObjectType) {
      case ER_AAC_LC:
      case ER_AAC_LTP:
      case ER_AAC_SCAL:
      case ER_TWIN_VQ:
      case ER_BSAC:
      case ER_AAC_LD:
      case ER_CELP:
      case ER_HVXC:
      case ER_HILN:
      case ER_PARA:
	      DebugPrintf(2,"  epConfig       :%ld", decConfig [layer].epConfig );
	    break;
    default:;
    }
    
    
    switch ( decConfig [layer].audioObjectType ){
      case AAC_MAIN :
      case AAC_LC :
      case AAC_SCAL :   /* aac scalable */
      case AAC_LTP:
      case TWIN_VQ :    /* tvq */
      case ER_TWIN_VQ : /* er_tvq */
      case ER_BSAC:
      case ER_AAC_LC:
      case ER_AAC_LTP:
      case ER_AAC_SCAL:
      case ER_AAC_LD:


        audSpC->specConf.TFSpecificConfig.frameLength.value = decConfig[layer].frameLengthFlag ;
        audSpC->specConf.TFSpecificConfig.dependsOnCoreCoder.value=decConfig[layer].dependsOnCoreCoder ;
        audSpC->specConf.TFSpecificConfig.coreCoderDelay.value=decConfig[layer].coreCoderDelay ;
        audSpC->specConf.TFSpecificConfig.extension.value=decConfig[layer].extensionFlag ;  
        DebugPrintf(2,"  frameLengthFlag   :%ld", decConfig[layer].frameLengthFlag );          
        DebugPrintf(2,"  dependsOnCoreCoder:%ld", decConfig[layer].dependsOnCoreCoder );          
        DebugPrintf(2,"  coreCoderDelay:%ld", decConfig[layer].coreCoderDelay );          
        DebugPrintf(2,"  extensionFlag:%ld", decConfig[layer].extensionFlag );          
        
        if (decConfig[layer].extensionFlag != 0) {
          if (decConfig[layer].audioObjectType == ER_BSAC) {
            audSpC->specConf.TFSpecificConfig.numOfSubFrame.value=decConfig[layer].numOfSubFrame;  
            audSpC->specConf.TFSpecificConfig.layer_length.value=decConfig[layer].layer_length;  
            DebugPrintf(2,"  numOfSubFrame:%ld", decConfig[layer].numOfSubFrame );          
            DebugPrintf(2,"  layer_length:%ld", decConfig[layer].layer_length );          
          }

          if(decConfig[layer].audioObjectType == ER_AAC_LC  ||
             decConfig[layer].audioObjectType == ER_AAC_LTP || decConfig[layer].audioObjectType == ER_AAC_SCAL ||
             decConfig[layer].audioObjectType == ER_TWIN_VQ || decConfig[layer].audioObjectType == ER_AAC_LD){
            
            audSpC->specConf.TFSpecificConfig.aacSectionDataResilienceFlag.value=decConfig[layer].aacSectionDataResilienceFlag;
            audSpC->specConf.TFSpecificConfig.aacScalefactorDataResilienceFlag.value=decConfig[layer].aacScalefactorDataResilienceFlag;
            audSpC->specConf.TFSpecificConfig.aacSpectralDataResilienceFlag.value=decConfig[layer].aacSpectralDataResilienceFlag;
            DebugPrintf(2,"  aacSectionDataResilienceFlag:%ld",decConfig[layer].aacSectionDataResilienceFlag);
            DebugPrintf(2,"  aacScalefactorDataResilienceFlag:%ld",decConfig[layer].aacScalefactorDataResilienceFlag);
            DebugPrintf(2,"  aacSpectralDataResilienceFlag:%ld",decConfig[layer].aacSpectralDataResilienceFlag);
          }
          audSpC->specConf.TFSpecificConfig.extension3.value=decConfig[layer].extension3Flag;  
          DebugPrintf(2,"  extension3Flag:%ld", decConfig[layer].extension3Flag );          
 
        }
        
        break;
      case CELP :    /* celp */
      case ER_CELP : /* celp */
#ifdef CORRIGENDUM1
	      audSpC->specConf.celpSpecificConfig.isBaseLayer.value = decConfig[layer].isBaseLayer;
	      if ( decConfig[layer].isBaseLayer == 1 ) {
#else
	      if (layer==0) {
#endif
	        audSpC->specConf.celpSpecificConfig.excitationMode.value = decConfig[layer].excitationMode;
	        audSpC->specConf.celpSpecificConfig.sampleRateMode.value = decConfig[layer].sampleRateMode;
	        audSpC->specConf.celpSpecificConfig.fineRateControl.value = decConfig[layer].fineRateControl;        
	        if (decConfig [layer].audioObjectType == ER_CELP) {
	          audSpC->specConf.celpSpecificConfig.silenceCompressionSW.value = decConfig[layer].silenceCompressionSW;
	        }
          /* RPE mode */
	        audSpC->specConf.celpSpecificConfig.RPE_Configuration.value = decConfig[layer].RPE_Configuration;        
          /* MPE mode */
	        audSpC->specConf.celpSpecificConfig.MPE_Configuration.value = decConfig[layer].MPE_Configuration ;
	        audSpC->specConf.celpSpecificConfig.numEnhLayers.value = decConfig[layer].numEnhLayers;
	        audSpC->specConf.celpSpecificConfig.bandwidthScalabilityMode.value = decConfig[layer].bandwidthScalabilityMode;        
	        DebugPrintf(2,"  excitationMode:%ld", decConfig[layer].excitationMode );                    
	        DebugPrintf(2,"  sampleRateMode:%ld", decConfig[layer].sampleRateMode  );                    
	        DebugPrintf(2,"  fineRateControl:%ld", decConfig[layer].fineRateControl  );                    
	        if (decConfig[layer].excitationMode == RegularPulseExc)
	          DebugPrintf(2,"  RPE_Configuration:%ld", decConfig[layer].RPE_Configuration  );                    
	        else {
	          DebugPrintf(2,"  MPE_Configuration:%ld", decConfig[layer].MPE_Configuration  );                    
	          DebugPrintf(2,"  numEnhLayers:%ld", decConfig[layer].numEnhLayers  );                    
	          DebugPrintf(2,"  bandwidthScalabilityMode:%ld", decConfig[layer].bandwidthScalabilityMode  );
	        }
	      } 
        else {
#ifdef CORRIGENDUM1
	        audSpC->specConf.celpSpecificConfig.isBWSLayer.value = decConfig[layer].isBWSLayer;
	        if ( decConfig[layer].isBWSLayer == 1 ) {
	          audSpC->specConf.celpSpecificConfig.BWS_Configuration.value = decConfig[layer].BWS_Configuration;
	          DebugPrintf(2,"  BWS_Configuration:%ld  \n", decConfig[layer].BWS_Configuration );
	        } 
          else {
	          audSpC->specConf.celpSpecificConfig.CELP_BRS_id.value = decConfig[layer].CELP_BRS_id;
	          DebugPrintf(2,"  BRS-Layer Index:%ld  \n", decConfig[layer].CELP_BRS_id);
	        }
	      

#else
	        if ( decConfig[0].bandwidthScalabilityMode == ON &&
	          layer == (int)(decConfig[0].numEnhLayers + 1) ) {
	          audSpC->specConf.celpEnhSpecificConfig.BWS_Configuration.value = decConfig[layer].BWS_Configuration;
	          DebugPrintf(2,"  BWS_Configuration:%ld", decConfig[layer].BWS_Configuration );
	        } 
          else {
	          DebugPrintf(2,"  CELP BRS-Layer: No Config.");
	        }
#endif
	      }
      break;
      case HVXC: /* HVXC */
#ifdef CORRIGENDUM1
        audSpC->specConf.hvxcSpecificConfig.isBaseLayer.value = decConfig[layer].isBaseLayer;
#endif
        audSpC->specConf.hvxcSpecificConfig.HVXCvarMode.value = decConfig[layer].HVXCvarMode;
        audSpC->specConf.hvxcSpecificConfig.HVXCrateMode.value = decConfig[layer].HVXCrateMode;
        audSpC->specConf.hvxcSpecificConfig.extensionFlag.value = decConfig[layer].extensionFlag;
        DebugPrintf(2,"  HVXCvarMode    :%ld", decConfig[layer].HVXCvarMode );
        DebugPrintf(2,"  HVXCrateMode   :%ld", decConfig[layer].HVXCrateMode );
        DebugPrintf(2,"  extensionFlag  :%ld", decConfig[layer].extensionFlag );
	      break;
      case ER_HVXC:	/* ER_HVXC */
#ifdef CORRIGENDUM1
        audSpC->specConf.hvxcSpecificConfig.isBaseLayer.value = decConfig[layer].isBaseLayer;
#endif
        audSpC->specConf.hvxcSpecificConfig.HVXCvarMode.value = decConfig[layer].HVXCvarMode;
        audSpC->specConf.hvxcSpecificConfig.HVXCrateMode.value = decConfig[layer].HVXCrateMode;
        audSpC->specConf.hvxcSpecificConfig.extensionFlag.value = decConfig[layer].extensionFlag;
	      if (decConfig[layer].extensionFlag) {
	        audSpC->specConf.hvxcSpecificConfig.vrScalFlag.value = decConfig[layer].vrScalFlag;
	      }
#ifdef CORRIGENDUM1
        DebugPrintf(2,"  isBaseLayer    :%ld  \n", decConfig[layer].isBaseLayer );
	      if (decConfig[layer].isBaseLayer) {	
#endif
        DebugPrintf(2,"  HVXCvarMode    :%ld", decConfig[layer].HVXCvarMode );
        DebugPrintf(2,"  HVXCrateMode   :%ld", decConfig[layer].HVXCrateMode );
        DebugPrintf(2,"  extensionFlag  :%ld", decConfig[layer].extensionFlag );
	      if (decConfig[layer].extensionFlag) {
	        DebugPrintf(2,"  vrScalFlag     :%ld", decConfig[layer].vrScalFlag );
        }
#ifdef CORRIGENDUM1
	      }
#endif	

	    break;
      case ER_HILN:
      case ER_PARA:
#ifdef CORRIGENDUM1
	    audSpC->specConf.paraSpecificConfig.isBaseLayer.value = decConfig[layer].isBaseLayer;
      if ( decConfig[layer].isBaseLayer == 1 ){
#else
      if (layer==0){
#endif
	      audSpC->specConf.paraSpecificConfig.PARAmode.value = decConfig[layer].PARAmode;
	      if (decConfig[layer].PARAmode != 1) {
	        audSpC->specConf.paraSpecificConfig.HVXCvarMode.value = decConfig[layer].HVXCvarMode;
	        audSpC->specConf.paraSpecificConfig.HVXCrateMode.value = decConfig[layer].HVXCrateMode;
	        audSpC->specConf.paraSpecificConfig.extensionFlag.value = decConfig[layer].extensionFlag;
	        if (decConfig[layer].extensionFlag) {
	          audSpC->specConf.paraSpecificConfig.vrScalFlag.value = decConfig[layer].vrScalFlag;
	        }
	      }
	      if (decConfig[layer].PARAmode != 0) {
	        audSpC->specConf.paraSpecificConfig.HILNquantMode.value = decConfig[layer].HILNquantMode;
	        audSpC->specConf.paraSpecificConfig.HILNmaxNumLine.value = decConfig[layer].HILNmaxNumLine;
	        audSpC->specConf.paraSpecificConfig.HILNsampleRateCode.value = decConfig[layer].HILNsampleRateCode;
	        audSpC->specConf.paraSpecificConfig.HILNframeLength.value = decConfig[layer].HILNframeLength;
	        audSpC->specConf.paraSpecificConfig.HILNcontMode.value = decConfig[layer].HILNcontMode;
	      }
	      audSpC->specConf.paraSpecificConfig.PARAextensionFlag.value = decConfig[layer].PARAextensionFlag;
	  
	  
#ifdef CORRIGENDUM1
	      DebugPrintf(2,"  isBaseLayer        :%ld", decConfig[layer].isBaseLayer );
#endif
	      DebugPrintf(2,"  PARAmode           :%ld", decConfig[layer].PARAmode );
	      if (decConfig[layer].PARAmode != 1) {
	        DebugPrintf(2,"  HVXCvarMode        :%ld", decConfig[layer].HVXCvarMode );
	        DebugPrintf(2,"  HVXCrateMode       :%ld", decConfig[layer].HVXCrateMode );
	        DebugPrintf(2,"  extensionFlag      :%ld", decConfig[layer].extensionFlag );
	        if (decConfig[layer].extensionFlag) {
		        DebugPrintf(2,"  vrScalFlag         :%ld", decConfig[layer].vrScalFlag );
	        }
	      }
	      if (decConfig[layer].PARAmode != 0) {
	        DebugPrintf(2,"  HILNquantMode      :%ld", decConfig[layer].HILNquantMode);
	        DebugPrintf(2,"  HILNmaxNumLine     :%ld", decConfig[layer].HILNmaxNumLine);
	        DebugPrintf(2,"  HILNsampleRateCode :%ld", decConfig[layer].HILNsampleRateCode);
	        DebugPrintf(2,"  HILNframeLength    :%ld", decConfig[layer].HILNframeLength);
	        DebugPrintf(2,"  HILNcontMode       :%ld", decConfig[layer].HILNcontMode);
	      }
	      DebugPrintf(2,"  PARAextensionFlag  :%ld", decConfig[layer].PARAextensionFlag);
	  
	    }
	    else { /* layer!=0 */
	      audSpC->specConf.paraSpecificConfig.HILNenhaLayer.value = decConfig[layer].HILNenhaLayer;
	      if (decConfig[layer].HILNenhaLayer) {
	        audSpC->specConf.paraSpecificConfig.HILNenhaQuantMode.value = decConfig[layer].HILNenhaQuantMode;
	      }
	  
#ifdef CORRIGENDUM1
	      DebugPrintf(2,"  isBaseLayer       :%ld", decConfig[layer].isBaseLayer );
#endif
	      DebugPrintf(2,"  HILNenhaLayer     :%ld", decConfig[layer].HILNenhaLayer );
	      if (decConfig[layer].HILNenhaLayer == 1) {
	        DebugPrintf(2,"  HILNenhaQuantMode :%ld", decConfig[layer].HILNenhaQuantMode );
	      }
	    }
	    break;

      default :
        DebugPrintf(1,"unsupported audio object type %ld" ,decConfig [layer].audioObjectType);
      break;            
      } /* end switch */
  
  } /* end for */

 
  for ( layer = 0; layer < streamCount; layer++ )  {
    /* goes through all ESs, which outnumbers layers for epConfig==1 */
    switch ( decConfig [layer].audioObjectType )
      {
      case AAC_MAIN :
      case AAC_LC :
      case AAC_SCAL :
      case AAC_LTP:
      case TWIN_VQ : 
      case ER_TWIN_VQ :
      case ER_BSAC: 
      case ER_AAC_LC:
      case ER_AAC_LTP:
      case ER_AAC_SCAL:
      case ER_AAC_LD:
        
        if(decData->tfData == NULL){
          /* init only once for all tf layers */
          if(!(decData->tfData = calloc(1,sizeof(TF_DATA))))
             return(1);
          DecTfInit(decDebugData->decPara,
                    decDebugData->aacDebugString,
                    hFault,
                    fD,
                    decData->tfData,
                    decData->lpcData,
                    layer,
                    decDebugData->infoFileName
                    );
        }
        break;
      case CELP :        
      case ER_CELP :        
        if(decData->lpcData == NULL){
          /* call it only once for all celp layers */
          if(!(decData->lpcData = calloc(1,sizeof(LPC_DATA))))
            return(1);
;
	        DecLpcInit(decDebugData->decPara,
			               fD,
			               decData->lpcData);
        }
        break;
      case HVXC:
      case ER_HVXC:
      
	      if (decData->hvxcData == NULL){ 
          /* call it only once for all hvxc layers */
	        
          decData->hvxcData= DecHvxcInit(decDebugData->decPara,
				                                 fD,
				                                 layer,
				                                 epFlag, 
				                                 epDebugLevel, 
				                                 decDebugData->infoFileName, 
				                                 &hFault->hEscInstanceData);
          delayNumSample = decData->hvxcData->delayNumSample;
	      }
	      break;
      case ER_HILN:
      case ER_PARA:
	      if (decData->paraData == NULL ){ /* call it only once for all hiln layers */
	         decData->paraData = DecParInit(decDebugData->decPara,
				                                  fD,
				                                  layer,
				                                  epFlag, 
				                                  epDebugLevel, 
				                                  decDebugData->infoFileName, 
				                                  &hFault->hEscInstanceData);
				   
	         delayNumSample = decData->paraData->delayNumSample;
	      }
	      break;

      default :
      DebugPrintf(1,"unsupported audio object type %ld" ,decConfig [layer].audioObjectType);
      break;            
      }
  
  }

  return(delayNumSample);
}




int audioDecFrame(DEC_DATA   *decData,      /* in  : Decoder Status(Handles) */
                  HANDLE_DECODER_GENERAL hFault,
                  float      ***outSamples, /* out : composition Unit (outsamples) */  
                  long        *numOutSamples)/* out : number of oputput samples/ch  */
{
    
  int numBits;


  switch (decData->frameData->scalOutObjectType)
  {
      case AAC_LC :
      case AAC_MAIN :
      case AAC_LTP:
      case AAC_SCAL : 
      case TWIN_VQ : 
      case ER_TWIN_VQ : 
      case ER_BSAC:
      case ER_AAC_LC:
      case ER_AAC_LTP:
      case ER_AAC_SCAL:
      case ER_AAC_LD:


      

          DecTfFrame(decData->frameData->scalOutNumChannels,
                     decData->frameData,
                     decData->tfData,
                     decData->lpcData,
                     hFault) ;
          *outSamples    = decData->tfData->sampleBuf;
          *numOutSamples = decData->tfData->block_size_samples;
      break;
      case CELP : 
      case ER_CELP :
        if (decData->frameData->layer[0].NoAUInBuffer>0) {
          DecLpcFrame(decData->frameData,
                      decData->lpcData,
                      &numBits);
                      
        } 
        else {
          CommonExit(-1,"\nNO more Access Units for Celp\n");
        }
        *outSamples    = decData->lpcData->sampleBuf;
        *numOutSamples = decData->lpcData->frameNumSample;
      break;

      case HVXC:
      case ER_HVXC:
        if (decData->frameData->layer[0].NoAUInBuffer>0) {
	        DecHvxcFrame(decData->frameData,
			                 decData->hvxcData,
			                 &numBits,
			                 0,/*epFlag,*/
			                 hFault->hEscInstanceData);
        } 
        else{
          CommonExit(-1,"\nNO more Access Units for Hvxc\n");
        }
        
        *outSamples    = decData->hvxcData->sampleBuf;
        *numOutSamples = decData->hvxcData->frameNumSample;
	    break;
      case ER_HILN:
      case ER_PARA:
        if (decData->frameData->layer[0].NoAUInBuffer>0) {
	         DecParFrame(decData->frameData,
		                   decData->paraData,
		                   &numBits,
		                   0,/*epFlag*/
		                   hFault->hEscInstanceData);
		       
        } 
        else {
          CommonExit(-1,"\nNO more Access Units for Para\n");
        }

       *outSamples    = decData->paraData->sampleBuf;
       *numOutSamples = decData->paraData->frameNumSample;
      break;
	    
    default :
    DebugPrintf(1,"unsupported audio object type %ld " ,decData->frameData->scalOutObjectType);
    break;            
  }
  
  return(0);
}


int audioDecFree(DEC_DATA   *decData,
                 HANDLE_DECODER_GENERAL hFault)       /* in  : Decoder Status(Handles) */
{

  if(decData->hvxcData)
    DecHvxcFree (decData->hvxcData);
  
  if(decData->lpcData){
    DecLpcFree();
    free(decData->lpcData);
  }
  if(decData->tfData){
    DecTfFree(decData->tfData,hFault);
    free(decData->tfData);
  
  }

  free(decData->frameData->od);
  free(decData->frameData);
  
  return(0);
  
}
