/**********************************************************************
MPEG-4 Audio VM
Deocoder cores (parametric, LPC-based, t/f-based)



This software module was originally developed by

Markus Werner (SEED / Software Development Karlsruhe) 

and edited by


in the course of development of the MPEG-2 NBC/MPEG-4 Audio standard
ISO/IEC 13818-7, 14496-1,2 and 3. This software module is an
implementation of a part of one or more MPEG-2 NBC/MPEG-4 Audio tools
as specified by the MPEG-2 NBC/MPEG-4 Audio standard. ISO/IEC gives
users of the MPEG-2 NBC/MPEG-4 Audio standards free license to this
software module or modifications thereof for use in hardware or
software products claiming conformance to the MPEG-2 NBC/ MPEG-4 Audio
standards. Those intending to use this software module in hardware or
software products are advised that this use may infringe existing
patents. The original developer of this software module and his/her
company, the subsequent editors and their companies, and ISO/IEC have
no liability for use of this software module or modifications thereof
in an implementation. Copyright is not released for non MPEG-2
NBC/MPEG-4 Audio conforming products. The original developer retains
full right to use the code for his/her own purpose, assign or donate
the code to a third party and to inhibit third party from using the
code for non MPEG-2 NBC/MPEG-4 Audio conforming products. This
copyright notice must be included in all copies or derivative works.

Copyright (c) 2000.

$Id: decifc.h,v 1.1.2.7 2001/05/09 14:31:02 cvs03 Exp $ 
Decoder high level interface
**********************************************************************/
#ifndef DECIFC_H
#define DECIFC_H

#include "lpc_common.h"
#include "tf_mainStruct.h"       
#include "hvxc_struct.h"	
#include "flex_mux.h"    
#include "mp4au.h"
#include "all.h"
#include "dec_par.h"


typedef struct {
  char *decPara;
  char *aacDebugString;
  char *infoFileName;
  int mainDebugLevel;
}DEC_DEBUG_DATA;



typedef struct{
    FRAME_DATA    *frameData;
    TF_DATA       *tfData ;
    LPC_DATA      *lpcData ;
    HVXC_DATA     *hvxcData;
    PARA_DATA     *paraData;
}DEC_DATA;


typedef struct
{
  unsigned long objectTypeIndication ;
  unsigned long streamType ;
  unsigned long upStream ;
  unsigned long bufferSizeDB ;
  unsigned long maxBitrate ;
  unsigned long avgBitrate ;

  /* Audio */

  AUDIO_OBJECT_TYPE_ID audioObjectType ;
  unsigned long samplingFrequencyIndex ;
  unsigned long samplingFrequency ;
  unsigned long channelConfiguration ;
  ProgConfig    progConfig;

  /* GA */

  unsigned long frameLengthFlag ;
  unsigned long dependsOnCoreCoder ;
  unsigned long coreCoderDelay ;
  unsigned long extensionFlag ;
  unsigned long layerNr ;
  unsigned long numOfSubFrame ;
  unsigned long layer_length ;
  unsigned long aacSectionDataResilienceFlag ;
  unsigned long aacScalefactorDataResilienceFlag ;
  unsigned long aacSpectralDataResilienceFlag ;
  unsigned long extension3Flag ;



  /* celp */
  unsigned long  excitationMode ;
  unsigned long  sampleRateMode ;
  unsigned long  fineRateControl ;

  unsigned long  silenceCompressionSW;

  unsigned long  RPE_Configuration ;

  unsigned long  MPE_Configuration ;
  unsigned long  numEnhLayers ;
  unsigned long  bandwidthScalabilityMode ;

  unsigned long  BWS_Configuration ;

  /* HVXC */
  unsigned long  HVXCvarMode ;
  unsigned long  HVXCrateMode ;
  unsigned long  excitationFlag ;
  unsigned long  packMode ;
  unsigned long  vrScalFlag ;

  /* HILN/PARA */
  unsigned long  PARAmode;

  unsigned long  HILNquantMode;
  unsigned long  HILNmaxNumLine;
  unsigned long  HILNsampleRateCode;
  unsigned long  HILNframeLength;
  unsigned long  HILNcontMode;

  unsigned long  PARAextensionFlag;

  unsigned long  HILNenhaLayer;
  unsigned long  HILNenhaQuantMode;
  
#ifdef CORRIGENDUM1
  unsigned long  isBaseLayer ;	/* common for CELP,HVXC and HILN */
  unsigned long  isBWSLayer ;	/* only for CELP */
  unsigned long  CELP_BRS_id ;	/* only for CELP */
#endif

  unsigned long  epConfig;

} DecoderConfigDescr ;



int audioDecInit(int  streamCount, /* in: number of Layers */
                 DecoderConfigDescr decConfig [MAX_TRACKS],
                 HANDLE_DECODER_GENERAL hFault,
                 DEC_DATA       *decData,
                 DEC_DEBUG_DATA *decDebugData);



int audioDecFrame(DEC_DATA   *decData,      /* in  : Decoder Status(Handles) */
                  HANDLE_DECODER_GENERAL hFault,
                  float      ***outSamples, /* out : composition Unit (outsamples) */  
                  long        *numOutSamples);/* out : number of oputput samples/ch  */

int audioDecFree(DEC_DATA   *decData,            /* in  : Decoder Status(Handles) */
                 HANDLE_DECODER_GENERAL hFault);

#endif
