/**********************************************************************
MPEG-4 Audio VM

This software module was originally developed by

Heiko Purnhagen     (University of Hannover / ACTS-MoMuSys)

and edited by

Markus Werner       (SEED / Software Development Karlsruhe) 

in the course of development of the MPEG-2 NBC/MPEG-4 Audio standard
ISO/IEC 13818-7, 14496-1,2 and 3. This software module is an
implementation of a part of one or more MPEG-2 NBC/MPEG-4 Audio tools
as specified by the MPEG-2 NBC/MPEG-4 Audio standard. ISO/IEC gives
users of the MPEG-2 NBC/MPEG-4 Audio standards free license to this
software module or modifications thereof for use in hardware or
software products claiming conformance to the MPEG-2 NBC/ MPEG-4 Audio
standards. Those intending to use this software module in hardware or
software products are advised that this use may infringe existing
patents. The original developer of this software module and his/her
company, the subsequent editors and their companies, and ISO/IEC have
no liability for use of this software module or modifications thereof
in an implementation. Copyright is not released for non MPEG-2
NBC/MPEG-4 Audio conforming products. The original developer retains
full right to use the code for his/her own purpose, assign or donate
the code to a third party and to inhibit third party from using the
code for non MPEG-2 NBC/MPEG-4 Audio conforming products. This
copyright notice must be included in all copies or derivative works.

Copyright (c) 2000.

$Id: mp4ifc.c,v 1.25.2.10 2001/08/15 20:02:16 uhd1 Exp $
Decoder frame work
**********************************************************************/
#include <stdio.h>
#include <string.h>

#include "MP4Movies.h"

#include "nok_ltp_common.h"      /* structs */
#include "obj_descr.h"           /* structs */
#include "tf_mainStruct.h"       /* structs */
#include "hvxc_struct.h"	       /* structs */


#include "dec_tf.h"
#include "dec_lpc.h"
#include "dec_par.h"

#include "bitstream.h"
#include "common_m4a.h"
#include "mp4ifc.h"

#include "audio.h"      /* audio i/o module */
#include "mp4au.h"
#include "port.h"
#include "decifc.h"
#ifdef  DEBUGPLOT
#include "plotmtv.h"
#endif






#ifndef min
#define min(a,b) ((a) < (b) ? (a) : (b))
#endif

#ifndef max
#define max(a,b) ((a) > (b) ? (a) : (b))
#endif

#define MAX_AU 12

static const unsigned long initialObjectDescrTag = 0x02 ;
static const unsigned long decoderConfigDescrTag = 0x04 ;
static const unsigned long decSpecificInfoTag    = 0x05 ;


static int MP4Audio_ReadInitialOD (MP4Movie theMovie, int ignoreProfiles) ;
static int MP4Audio_ReadDecoderConfig (MP4Handle decConfigH, 
                                       DecoderConfigDescr *decConfig, 
#ifndef CORRIGENDUM1
                                       DecoderConfigDescr *decConfig0, 
                                       unsigned long idx,
#endif
                                       HANDLE_DECODER_GENERAL hFault);
 
static int MP4Audio_LookForSuitableTracks (MP4Movie theMovie, unsigned long trackNumber [MAX_TRACKS]) ;
static int MP4Audio_SetupDecoders (DecoderConfigDescr decConfig [MAX_TRACKS],
                                   DEC_DATA         *decData,
                                   DEC_DEBUG_DATA   *decDebugData,
                                   int   streamCount,          /* in: number of Layers */
                                   char* audioFileName,     /* in: audio file name */
                                   char* audioFileFormat,   /* in: audio file format (au, wav, aif, raw) */
                                   int   int24flag,
                                   char* decPara ,          /* in: decoder parameter string */
                                   HANDLE_DECODER_GENERAL hFault,
                                   int   mainDebugLevel,
                                   int   audioDebugLevel,
                                   char* aacDebugString);

static HANDLE_BSBITSTREAM MP4Audio_BsFromHandle (MP4Handle theHandle);
static T_DECODERGENERAL faultData;
static HANDLE_DECODER_GENERAL hFault = &faultData;
static AudioFile *audioFile = NULL;
static int locMainDebugLevel=0;


#ifdef DEBUGPLOT
extern  int framePlot;
#endif

int MP4Audio_ProbeFile (char *inFile)
{
  MP4Movie theMovie = NULL ; 
  MP4Err err = MP4OpenMovieFile ( &theMovie, inFile, MP4OpenMovieNormal ) ; 
  if (err == MP4NoErr) 
  { 
    MP4DisposeMovie (theMovie) ; 
    return 1 ; 
  } 
  else 
    return 0 ; 
}

#define SUBTRACKSIZE  16

int MP4Audio_DecodeFile (char* inFile,          /* in: bitstream input file name */
                         char* audioFileName,   /* in: audio file name */
                         char* audioFileFormat, /* in: audio file format (au, wav, aif, raw) */
                         int   int24flag,
 						             int   maxLayer	,		    /* in  max layers         */
                         char* decPara,         /* in: decoder parameter string */
                         int   mainDebugLevel,
                         int   audioDebugLevel,
                         char* aacDebugString)
{
  MP4Err err;
  unsigned long framesDone ,EOFmask[4];
  int needNewAU;
  unsigned long trackNumber [MAX_TRACKS] ;
  DecoderConfigDescr decConfig [MAX_TRACKS] ;
  MP4TrackReader theReader [MAX_TRACKS] ;
  HANDLE_BSBITBUFFER locBitBuf = NULL;
  HANDLE_BSBITBUFFER AUBuffer  = NULL;
  HANDLE_BSBITSTREAM locBitStream = NULL;
  DEC_DATA           *decData;
  DEC_DEBUG_DATA     *decDebugData;
  MP4Movie theMovie;
  MP4Track theTrack;
  MP4Media theMedia;
  MP4Handle sampleH ;
  MP4Handle decoderConfigH = NULL ;
  int  idx, t,suitableTracks = 0;
  long sub_idx1, sub_idx2;
  int epFlag=0;
  float **outSamples;
  long  numOutSamples;
  char buffer_empty;

  locBitBuf = BsAllocBuffer(MAXREADAHEAD);
  locMainDebugLevel= mainDebugLevel;
  
  sampleH = NULL ;
  theMovie = NULL;
	err = MP4NoErr;
	err = MP4OpenMovieFile( &theMovie, inFile, MP4OpenMovieNormal ); if (err) goto bail;

  suitableTracks = MP4Audio_LookForSuitableTracks (theMovie, trackNumber) ;
  
  DebugPrintf(1,"\nFound MP4 file with  %d suitable Tracks  ",suitableTracks);

  if(maxLayer == -1)
    maxLayer = suitableTracks-1;
  
  suitableTracks = min(suitableTracks, maxLayer+1) ;

 
  if (!suitableTracks) 
    return MP4BadDataErr ;

  
  BsInit (0,0,0);
  decConfig[0].epConfig = 0; 



  /* check the iod */

  if (MP4Audio_ReadInitialOD (theMovie, 0))
    return MP4BadDataErr ;

  err = MP4NewHandle( 0, &decoderConfigH ); if (err) goto bail;

  /* read the theTrack */

  for (idx = 0 ; idx < suitableTracks ; idx++)
  {
	  err = MP4GetMovieIndTrack( theMovie, trackNumber [idx], &theTrack ); if (err) goto bail;
	  err = MP4GetTrackMedia( theTrack, &theMedia ); if (err) goto bail;
	  err = MP4CreateTrackReader( theTrack, &theReader [idx] ); if (err) goto bail;
	  err = MP4TrackReaderGetCurrentDecoderConfig( theReader [idx], decoderConfigH ); if (err) goto bail;

    if (MP4Audio_ReadDecoderConfig (decoderConfigH, 
                                    &decConfig [idx],
#ifndef CORRIGENDUM1
                                    &decConfig[0], 
                                    idx,
#endif
                                    
                                    hFault))
      return MP4BadDataErr ;
  }

	/*
	 instantiate proper decoder for decoderconfig
	*/

  decData = calloc(1,sizeof(DEC_DATA));
  decDebugData=calloc(1,sizeof(DEC_DEBUG_DATA));


  suitableTracks=MP4Audio_SetupDecoders (decConfig, 
                                         decData,
                                         decDebugData,
                                         suitableTracks,
                                         audioFileName, 
                                         audioFileFormat,
                                         int24flag,
                                         decPara,
                                         hFault,
                                         mainDebugLevel,
                                         audioDebugLevel,
                                         aacDebugString) ;

  err = MP4NewHandle( 0, &sampleH ); if (err) goto bail;	
  /* loop through the file */
  EOFmask[0] = 0; EOFmask[1] = 0; 
  EOFmask[2] = 0; EOFmask[3] = 0; 
  sub_idx1 = suitableTracks / SUBTRACKSIZE;
  sub_idx2 = suitableTracks % SUBTRACKSIZE;
  EOFmask[sub_idx1] = (1<<sub_idx2)-1;
  
  for (idx = 0; idx < sub_idx1; idx++)   
     EOFmask[idx] = (1<<SUBTRACKSIZE) - 1; 

  idx=0;
  needNewAU=1;
  for (framesDone = 0, buffer_empty=0; buffer_empty==0; framesDone++)  {
#ifdef DEBUGPLOT
    plotInit(framePlot,audioFileName,0);
#endif
    
    while (needNewAU)  {
      u32 unitSize, cts, dts, sampleFlags;
      sub_idx1 = idx / SUBTRACKSIZE; 
      sub_idx2 = idx % SUBTRACKSIZE;
   
      if (  EOFmask[sub_idx1] & (1<<sub_idx2)) {
        err = MP4TrackReaderGetNextAccessUnit( theReader [idx], sampleH, &unitSize,&sampleFlags, &cts, &dts );
        if ( err ) {
          if ( err == MP4EOF ) {
            err = MP4NoErr;
            EOFmask[sub_idx1] ^= 1<<sub_idx2;
            /* on end of file: terminate program except for AAC scalable with celp core (buffers need to be emptied)  (lzm) */
            if (EOFmask[0]==0 && EOFmask[1]==0 && EOFmask[2]==0 && EOFmask[3]==0) {
              if( !((decConfig[0].audioObjectType == 8) && (decConfig[suitableTracks-1].audioObjectType != 8)) ) {
                goto bail;
              }
            }
          } 
          else {
            goto bail;
          }
        }
        memcpy (BsBufferGetDataBegin(locBitBuf), *sampleH, unitSize) ;        
        BsBufferManipulateSetNumBit(unitSize * 8,locBitBuf);  
        locBitStream=BsOpenBufferRead (locBitBuf);	       
        if  (BsBufferNumBit(BsGetBitBufferHandle(0,locBitStream)) %8 )
          CommonExit(1,"BsOpenBufferRead: byte align error");
        BsManipulateSetNumByte(BsBufferNumBit(BsGetBitBufferHandle(0,locBitStream)) /8, locBitStream);

        AUBuffer = decData->frameData->layer[idx].bitBuf ;
        if (AUBuffer!=0 ) {
          if ((BsBufferFreeBits(AUBuffer)) > (long)unitSize*8 )  {
            BsGetBufferAppend( locBitStream, AUBuffer,1,unitSize*8);    
            decData->frameData->layer[idx].AULength[decData->frameData->layer[idx].NoAUInBuffer]= unitSize*8;
            decData->frameData->layer[idx].NoAUInBuffer++;/* each decoder must decrease this by the number of decoded AU */
            /* each decoder must remove the first element in the list  by shifting the elements down */
          }  else {
            BsGetSkip(locBitStream,unitSize*8);
            CommonWarning (" input buffer overflow for layer %d ; skiping next AU",idx);
          }
        } else {
          BsGetSkip(locBitStream,unitSize*8);
        }
        BsCloseRemove (locBitStream,1);        
      }

      needNewAU=0;        
      if (  (decConfig [0].audioObjectType == CELP) 
            && (decConfig [suitableTracks-1].audioObjectType != CELP) ) {
        /* read at least MAX_AU AU for celp if celp + aac scalable */
        for (t = 0 ; t < suitableTracks ; t++){
          if  (( decData->frameData->layer[t].NoAUInBuffer <= MAX_AU ) &&
               (  EOFmask[0] & (1<<t) )) {
            needNewAU=1;
            idx=t;
          } 
        }                
      } else {
        unsigned long s_idx1, s_idx2;
        for (t = 0 ; t < suitableTracks ; t++){
          s_idx1 = t / SUBTRACKSIZE; 
          s_idx2 = t % SUBTRACKSIZE;
          if  (( decData->frameData->layer[t].NoAUInBuffer == 0 ) &&
               (  EOFmask[s_idx1] & (1<<s_idx2) )) {
            needNewAU=1;
            idx=t;
          }
        }
      }            
    }
    
    
    /*
      send AU to decoder
    */
    if ((mainDebugLevel == 1) && (framesDone % 20 == 0)) {
      fprintf(stderr,"\rFrame [%ld] ",framesDone);
      fflush(stderr);
    }
    if (mainDebugLevel >= 2 && mainDebugLevel <= 3) {
      printf("\rFrame [%ld] ",framesDone);
      fflush(stdout);
    }
    if (mainDebugLevel > 3)
      printf("Frame [%ld]\n",framesDone);
    /* HP20010815   (no progress info on stdout as default) 
    if (mainDebugLevel >= 2) {
      fprintf(stdout,"Frame [%ld]  \r",framesDone);
    } else if (mainDebugLevel >= 1) {
      if (framesDone % 20 == 0)
        fprintf(stdout,"Frame [%ld]  \r",framesDone);
    } else {
      if (framesDone % 100 == 0)
        fprintf(stdout,"Frame [%ld]  \r",framesDone);
    }
    
    fflush(stdout);
    */


    audioDecFrame(decData,hFault,&outSamples,&numOutSamples);   
    AudioWriteData(audioFile,outSamples,numOutSamples);
   
#ifdef DEBUGPLOT    
    framePlot++;
    plotDisplay(0);
#endif
    needNewAU=0;        
    if ((decConfig [0].audioObjectType == CELP) 
          && (decConfig [suitableTracks-1].audioObjectType != CELP) ) {
      /* read at least MAX_AU AU for celp if celp + aac scalable */
      for (t = 0 ; t < suitableTracks ; t++){
        if (EOFmask[0] & (1<<t)) {
          if ( decData->frameData->layer[t].NoAUInBuffer <= MAX_AU ) {
            needNewAU=1;
            idx=t;
          }
        } else {
          /* new condition for program halt (lzm) */
          if ( decData->frameData->layer[t].NoAUInBuffer <= 1 ) {
            buffer_empty=1;
          }
        } 
      }                
      
    } else {
      unsigned long s_idx1, s_idx2;

      for (t = 0 ; t < suitableTracks ; t++){
        s_idx1 = t / SUBTRACKSIZE; 
        s_idx2 = t % SUBTRACKSIZE;
        if  (( decData->frameData->layer [t].NoAUInBuffer == 0 ) &&
             (  EOFmask[s_idx1] & (1<<s_idx2) )) {
          needNewAU=1;
          idx=t;
        }
      }
    }            
              
    
  }	
  if (mainDebugLevel >= 1)
    fprintf(stdout,"\n");
  
  bail:

  audioDecFree(decData,hFault);

  free(decData);
  free(decDebugData);


  if (audioFile)
    AudioClose(audioFile);
  
  if ( decoderConfigH) 
    MP4DisposeHandle (decoderConfigH) ;
  if ( theMovie ) 
    MP4DisposeMovie (theMovie) ;
  if ( sampleH ) 
    MP4DisposeHandle (sampleH) ;
  
  return err;
}

static int MP4Audio_LookForSuitableTracks (MP4Movie theMovie, unsigned long trackNumber [MAX_TRACKS])
{
  u32 trackCount;
  unsigned long idx ;
  unsigned long suitableTracks = 0 ;

  MP4Track baseTrack = NULL ;

  MP4Err err = MP4GetMovieTrackCount (theMovie, &trackCount) ;
  
  if (!err)
  {
    /* look for base layer */

    for (idx = 1 ; idx <= trackCount ; idx++)
    {
      u32  enabled = 0;

      err = MP4GetMovieIndTrack (theMovie, idx, &baseTrack) ; if (err) goto bail;
      err = MP4GetTrackEnabled (baseTrack, &enabled ); if (err) goto bail;

      if (enabled)
      {
        MP4Media theMedia;
      	u32    handlerType;
        u32    outRefIndex = 0 ;

	      err = MP4GetTrackMedia(baseTrack, &theMedia ); if (err) goto bail;
	      err = MP4GetMediaHandlerDescription(theMedia, &handlerType, NULL ); if (err) goto bail;
        err = MP4GetTrackReferenceCount (baseTrack, MP4StreamDependencyReferenceType, &outRefIndex) ;

        if ((handlerType == MP4AudioHandlerType) && (outRefIndex == 0))
        {
          trackNumber [suitableTracks++] = idx ;
          break ;
        }
      }
    }

    if (suitableTracks)
    {
      /* then look for extension layers */

      do
      {
        for (idx = 1 ; idx <= trackCount ; idx++)
        {
          MP4Track theTrack;
          u32    enabled = 0;

          err = MP4GetMovieIndTrack (theMovie, idx, &theTrack) ; if (err) goto bail;
          err = MP4GetTrackEnabled (theTrack, &enabled ); if (err) goto bail;

          if (enabled)
          {
            MP4Media theMedia;
      	    u32    handlerType;

	          err = MP4GetTrackMedia(theTrack, &theMedia ); if (err) goto bail;
	          err = MP4GetMediaHandlerDescription(theMedia, &handlerType, NULL ); if (err) goto bail;

            if (handlerType == MP4AudioHandlerType)
            {
              MP4Track refTrack ;
              u32 outRefIndex = 0, i ;

              err = MP4GetTrackReferenceCount (theTrack, MP4StreamDependencyReferenceType, &outRefIndex) ;

              for (i = 1 ; i <= outRefIndex ; i++)
              {
                err = MP4GetTrackReference (theTrack, MP4StreamDependencyReferenceType, i, &refTrack) ;
                if (refTrack == baseTrack)
                {
                  trackNumber [suitableTracks++] = idx ;
                  baseTrack = theTrack ;
                  break ;
                } 

              }
            }
          }
        }
      }
      while (idx < trackCount) ;
    }
  }

bail :

  return suitableTracks ;
}

static int MP4Audio_ReadInitialOD (MP4Movie theMovie, int ignoreProfiles)
{
  MP4Handle initialODH = NULL ;
  MP4Err err ;

  err = MP4NewHandle( 0, &initialODH ); 
  err = MP4GetMovieInitialObjectDescriptor (theMovie, initialODH );

  if (!err)
  {
    unsigned long tag, tmp ;
    unsigned long sizeOfClass ;

    unsigned long id ;
    unsigned long urlFlag ;
    unsigned long inlineFlag ;

    unsigned long odProfile = 0 ;
    unsigned long sceneProfile = 0 ;
    unsigned long audioProfile = 0 ;
    unsigned long visualProfile = 0 ;
    unsigned long graphicsProfile = 0 ;

    HANDLE_BSBITSTREAM bs = MP4Audio_BsFromHandle (initialODH) ;

    BsGetBit (bs, &tag, 8) ;
    if (tag != initialObjectDescrTag)
    {
      fprintf (stderr, "Failure: IOD class tag == %ld \n", tag);
      err = MP4BadDataErr ; goto bail ;
    }

    for (sizeOfClass = 0, tmp = 0x80 ; tmp &= 0x80 ;
         sizeOfClass = (sizeOfClass << 7) | (tmp & 0x7F))
         BsGetBit (bs, &tmp, 8) ;

    BsGetBit (bs, &id, 10) ;
    BsGetBit (bs, &urlFlag, 1) ;
    BsGetBit (bs, &inlineFlag, 1) ;
    BsGetBit (bs, &tmp, 4) ;

    if (!urlFlag)
    {
      BsGetBit (bs, &odProfile, 8) ;
      BsGetBit (bs, &sceneProfile, 8) ;
      BsGetBit (bs, &audioProfile, 8) ;
      BsGetBit (bs, &visualProfile, 8) ;
      BsGetBit (bs, &graphicsProfile, 8) ;
    }

    if (!ignoreProfiles)
    {
      if (odProfile != 0xFF)
      {
        fprintf (stderr, "Failure: Can't handle OD profile %ld \n", tag);
        err = MP4BadDataErr ; goto bail ;
      }

      if (sceneProfile != 0xFF)
      {
        fprintf (stderr, "Failure: Can't handle BIFS profile %ld \n", tag);
        err = MP4BadDataErr ; goto bail ;
      }

      if (visualProfile != 0xFF)
      {
        fprintf (stderr, "Failure: Can't handle VIDEO profile %ld \n", tag);
        err = MP4BadDataErr ; goto bail ;
      }

      if (graphicsProfile != 0xFF)
      {
        fprintf (stderr, "Failure: Can't handle GRAPHICS profile %ld \n", tag);
        err = MP4BadDataErr ; goto bail ;
      }
    }

bail :

    BsCloseRemove (bs, 0) ;
  }

  MP4DisposeHandle (initialODH) ;

  return err ;
}

static int MP4Audio_ReadDecoderConfig (MP4Handle decConfigH, 
                                       DecoderConfigDescr *decConfig, 
#ifndef CORRIGENDUM1                                       
                                       DecoderConfigDescr *decConfig0, 
                                       unsigned long idx,
#endif
                                       HANDLE_DECODER_GENERAL hFault
                                       )
{
  MP4Err err = MP4NoErr ;

  unsigned long tag, tmp ;
  unsigned long sizeOfClass ;

  HANDLE_BSBITSTREAM bs = MP4Audio_BsFromHandle (decConfigH) ;

  BsGetBit (bs, &tag, 8) ;
  if (tag != decoderConfigDescrTag)
  {
    fprintf (stderr, "Failure: DecoderConfig class tag == %ld\n", tag);
    err = MP4BadDataErr ; goto bail ;
  }

  for (sizeOfClass = 0, tmp = 0x80 ; tmp &= 0x80 ;
       sizeOfClass = (sizeOfClass << 7) | (tmp & 0x7F))
       BsGetBit (bs, &tmp, 8) ;

  BsGetBit (bs, &decConfig->objectTypeIndication, 8) ;
  if (decConfig->objectTypeIndication != 0x40)
  {
    fprintf (stderr, "Failure: objectTypeIndication not Audio\n");
    err = MP4BadDataErr ; goto bail ;
  }

  BsGetBit (bs, &decConfig->streamType, 6) ;
  if (decConfig->streamType != 0x5)
  {
    fprintf (stderr, "Failure: streamType not Audio\n");
    err = MP4BadDataErr ; goto bail ;
  }

  BsGetBit (bs, &decConfig->upStream, 1) ;
  BsGetBit (bs, &tmp, 1) ;
  BsGetBit (bs, &decConfig->bufferSizeDB, 24) ;
  BsGetBit (bs, &decConfig->maxBitrate, 32) ;
  BsGetBit (bs, &decConfig->avgBitrate, 32) ;

  /* DecoderSpecificInfo */

  BsGetBit (bs, &tag, 8) ;
  if (tag != decSpecificInfoTag)
  {
    fprintf (stderr, "Failure: DecoderSpecificInfo class tag == %ld\n", tag);
    err = MP4BadDataErr ; goto bail ;
  }

  for (sizeOfClass = 0, tmp = 0x80 ; tmp &= 0x80 ;
       sizeOfClass = (sizeOfClass << 7) | (tmp & 0x7F))
       BsGetBit (bs, &tmp, 8) ;

  /* HP 20000117 */
  /*BsGetBit (bs, &(long unsigned int)decConfig->audioObjectType, 5) ;*/
  BsGetBit (bs, &tmp, 5) ;
  decConfig->audioObjectType = (AUDIO_OBJECT_TYPE_ID) tmp;
  BsGetBit (bs, &decConfig->samplingFrequencyIndex, 4) ;
  if (decConfig->samplingFrequencyIndex == 0x0F)
  {
    /* BsGetBit (bs, &decConfig->samplingFrequency, 4) ; */
    BsGetBit (bs, &decConfig->samplingFrequency, 24) ;
  }
  else
  {
    static const unsigned long samplingRate [] = {
      96000, 88200, 64000, 48000, 44100, 32000, 24000, 22050, 16000, 12000, 11025, 8000, 7350, 0, 0, 0
    };

    decConfig->samplingFrequency = samplingRate [decConfig->samplingFrequencyIndex] ;
  }

  BsGetBit (bs, &decConfig->channelConfiguration, 4) ;

  switch (decConfig->audioObjectType)
  {
    case AAC_MAIN : /* AAC Main */
    case AAC_LC :   /* AAC LC */
    case AAC_SCAL : /* AAC Scalable */
    case AAC_LTP:   
    case TWIN_VQ :    /* TVQ */
    case ER_TWIN_VQ : /* ER_TVQ */
    case ER_BSAC:
    case ER_AAC_LC:
    case ER_AAC_LTP:
    case ER_AAC_SCAL:
    case ER_AAC_LD:


      BsGetBit (bs, &decConfig->frameLengthFlag, 1) ;
      BsGetBit (bs, &decConfig->dependsOnCoreCoder, 1) ;
      if (decConfig->dependsOnCoreCoder)
      {
        BsGetBit (bs, &decConfig->coreCoderDelay, 14) ;
      }
      else
        decConfig->coreCoderDelay = 0 ;

      BsGetBit (bs, &decConfig->extensionFlag, 1) ;
      if (!decConfig->channelConfiguration){
        get_prog_config_from_bs ( &decConfig->progConfig,
                                  (decConfig->frameLengthFlag ? 960:1024),
                                  hFault->hResilience, 
                                  hFault->hEscInstanceData,
                                  bs );
        decConfig->channelConfiguration = get_channel_config_from_progConfig ( &decConfig->progConfig );
      }
#ifdef CORRIGENDUM1
      if (decConfig->audioObjectType == AAC_SCAL || decConfig->audioObjectType == ER_AAC_SCAL) {
        BsGetBit (bs, &decConfig->layerNr, 3) ;   
      }
#endif
      if (decConfig->extensionFlag != 0) {
	      if (decConfig->audioObjectType == ER_BSAC) {
	        BsGetBit (bs, &decConfig->numOfSubFrame, 5) ;
	        BsGetBit (bs, &decConfig->layer_length, 11) ;
	      }
        if(decConfig->audioObjectType == ER_AAC_LC  ||
           decConfig->audioObjectType == ER_AAC_LTP || decConfig->audioObjectType == ER_AAC_SCAL ||
           decConfig->audioObjectType == ER_TWIN_VQ || decConfig->audioObjectType == ER_AAC_LD){
          
          BsGetBit (bs,&decConfig->aacSectionDataResilienceFlag,1);
          BsGetBit (bs,&decConfig->aacScalefactorDataResilienceFlag,1);
          BsGetBit (bs,&decConfig->aacSpectralDataResilienceFlag,1);
        }
	      BsGetBit (bs, &decConfig->extension3Flag, 1) ;
      }

#ifndef EP_CONFIG_MISSING
      if ( decConfig->audioObjectType == ER_TWIN_VQ ){
        BsGetBit (bs, &decConfig->epConfig, 2) ;
      }
#endif

    break ;

    case CELP : /* CELP */
    case ER_CELP : /* ER_CELP */
#ifdef CORRIGENDUM1
      BsGetBit (bs, &decConfig->isBaseLayer, 1) ;	/* 14496-3 COR1 */
      if ( decConfig->isBaseLayer == 1 ) {
#else
      if ( idx == 0 ) {
#endif
        BsGetBit (bs, &decConfig->excitationMode, 1) ;
	      BsGetBit (bs, &decConfig->sampleRateMode, 1) ;
	      BsGetBit (bs, &decConfig->fineRateControl, 1) ;
	      if (decConfig->audioObjectType == ER_CELP) {
	        BsGetBit (bs, &decConfig->silenceCompressionSW, 1) ;
	      }
	      if (decConfig->excitationMode == RegularPulseExc) {
	        BsGetBit (bs, &decConfig->RPE_Configuration, 3) ;
	      } 
	      if (decConfig->excitationMode == MultiPulseExc){
	        BsGetBit (bs, &decConfig->MPE_Configuration, 5) ;
	        BsGetBit (bs, &decConfig->numEnhLayers, 2) ;
	        BsGetBit (bs, &decConfig->bandwidthScalabilityMode, 1) ;
	      }
      } 
      else {
#ifdef CORRIGENDUM1
	      BsGetBit (bs, &decConfig->isBWSLayer, 1) ;	/* 14496-3 COR1 */
	      if ( decConfig->isBWSLayer == 1 ) {
	        BsGetBit (bs, &decConfig->BWS_Configuration, 2) ;
	      } 
        else {
	        BsGetBit (bs, &decConfig->CELP_BRS_id, 2) ;	/* 14496-3 COR1 */
	      }
#else
	      if ( decConfig0->bandwidthScalabilityMode == ON &&
	        idx == (decConfig0->numEnhLayers + 1)) {
	        BsGetBit (bs, &decConfig->BWS_Configuration, 2) ;
	      }
#endif
      }
#ifndef EP_CONFIG_MISSING
      if ( decConfig->audioObjectType == ER_CELP ) {
	      BsGetBit (bs, &decConfig->epConfig, 2) ;
      }
#endif
    break ;
    case HVXC:
#ifdef CORRIGENDUM1
      BsGetBit (bs, &decConfig->isBaseLayer, 1) ;	/* 14496-3 COR1(AI 2000/10) */
      if (decConfig->isBaseLayer) {
	      BsGetBit (bs, &decConfig->HVXCvarMode, 1) ;
	      BsGetBit (bs, &decConfig->HVXCrateMode, 2) ;
	      BsGetBit (bs, &decConfig->extensionFlag, 1) ;
      }
#else
      BsGetBit (bs, &decConfig->HVXCvarMode, 1) ;
      BsGetBit (bs, &decConfig->HVXCrateMode, 2) ;
      BsGetBit (bs, &decConfig->extensionFlag, 1) ;
#endif
      break;

    case ER_HVXC:
#ifdef CORRIGENDUM1
      BsGetBit (bs, &decConfig->isBaseLayer, 1) ;	/* 14496-3 COR1(AI 2000/10) */
      if (decConfig->isBaseLayer) {
	      BsGetBit (bs, &decConfig->HVXCvarMode, 1) ;
	      BsGetBit (bs, &decConfig->HVXCrateMode, 2) ;
	      BsGetBit (bs, &decConfig->extensionFlag, 1) ;
	      if (decConfig->extensionFlag) {
	        BsGetBit (bs, &decConfig->vrScalFlag, 1) ;
	      }
      }
#else
      BsGetBit (bs, &decConfig->HVXCvarMode, 1) ;
      BsGetBit (bs, &decConfig->HVXCrateMode, 2) ;
      BsGetBit (bs, &decConfig->extensionFlag, 1) ;
      if (decConfig->extensionFlag) {
	      BsGetBit (bs, &decConfig->vrScalFlag, 1) ;
      }
#endif
#ifndef EP_CONFIG_MISSING
      BsGetBit (bs, &decConfig->epConfig, 2) ;
#endif
      break;

  case ER_HILN:
  case ER_PARA:
#ifdef CORRIGENDUM1
    BsGetBit (bs, &decConfig->isBaseLayer, 1) ;	/* 14496-3 COR1 */
    if ( decConfig->isBaseLayer == 1 )
#else
    if ( idx == 0 )
#endif
    {
      BsGetBit (bs,  &decConfig->PARAmode, 2);
      if (decConfig->PARAmode != 1) {
	      BsGetBit (bs, &decConfig->HVXCvarMode, 1) ;
	      BsGetBit (bs, &decConfig->HVXCrateMode, 2) ;
	      BsGetBit (bs, &decConfig->extensionFlag, 1) ;
	      if (decConfig->extensionFlag) {
	        BsGetBit (bs, &decConfig->vrScalFlag, 1) ;
	      }
      }
      if (decConfig->PARAmode != 0) {
	      BsGetBit (bs, &decConfig->HILNquantMode, 1);
	      BsGetBit (bs, &decConfig->HILNmaxNumLine, 8);
	      BsGetBit (bs, &decConfig->HILNsampleRateCode, 4);
	      BsGetBit (bs, &decConfig->HILNframeLength, 12);
	      BsGetBit (bs, &decConfig->HILNcontMode, 2);
      }
      BsGetBit (bs, &decConfig->PARAextensionFlag, 1);
    }
    else {
      BsGetBit (bs, &decConfig->HILNenhaLayer, 1);
      if (decConfig->HILNenhaLayer == 1) {
	      BsGetBit (bs, &decConfig->HILNenhaQuantMode, 2);
      }
    }
#ifndef EP_CONFIG_MISSING
    BsGetBit (bs, &decConfig->epConfig, 2);
#endif
    break;

    default :

      fprintf (stderr, "Failure: audioObjectType == %d\n", decConfig->audioObjectType);
      err = MP4BadDataErr ; goto bail ;
  }

bail :

  BsCloseRemove (bs, 0) ;

  return err ;
}

static HANDLE_BSBITSTREAM MP4Audio_BsFromHandle (MP4Handle theHandle)
{
  u32    handleSize ;
  MP4Err err = MP4GetHandleSize (theHandle, &handleSize) ;
  if (!err)
  {
    HANDLE_BSBITBUFFER bb;

    bb = BsAllocPlainDirtyBuffer((unsigned char *) (*theHandle), handleSize * 8);
    return BsOpenBufferRead (bb) ;
  }

  return NULL ;
}

static int MP4Audio_SetupDecoders (DecoderConfigDescr decConfig [MAX_TRACKS],
                                   DEC_DATA         *decData,
                                   DEC_DEBUG_DATA   *decDebugData,
                                   int   streamCount,        /* in: number of Layers */
                                   char* audioFileName,   /* in: audio file name */
                                   char* audioFileFormat, /* in: audio file format (au, wav, aif, raw) */
                                   int   int24flag,
                                   char* decPara,         /* in: decoder parameter string */
                                   HANDLE_DECODER_GENERAL hFault,
                                   int   mainDebugLevel,
                                   int   audioDebugLevel,
                                   char* aacDebugString)
                            
{
  float  fSampleOut;
  int    numChannelOut;
  int    delayNumSample;

  
 
  AudioInit(NULL, int24flag, audioDebugLevel);

  /*
    init decoders
  */

  decDebugData->aacDebugString = aacDebugString;
  decDebugData->decPara = decPara;
  decDebugData->infoFileName=NULL;
  decDebugData->mainDebugLevel=mainDebugLevel;
  
  delayNumSample=audioDecInit(streamCount, decConfig,hFault,decData,decDebugData);
  
  /* for epConfig==1, frameData.od->streamCount.value might have been modified */
  if(decConfig[0].epConfig == 1){
    switch(decConfig[0].audioObjectType){
    case ER_HVXC:
    case ER_HILN:
    case ER_PARA:
    case ER_TWIN_VQ:
    case ER_CELP:
      streamCount=decData->frameData->od->streamCount.value;	
    break;
    default:; /*?*/
    }
  }
  
  
  
  fSampleOut = (float) decConfig [streamCount-1].samplingFrequency ;
  numChannelOut = decConfig [streamCount-1].channelConfiguration ;

  /* open audio file */
  /* (sample format: 16 bit twos complement, uniform quantisation) */

  audioFile = AudioOpenWrite(audioFileName,audioFileFormat,
                             numChannelOut,fSampleOut);

  /* seek to beginning of first frame (with delay compensation) */
  AudioSeek(audioFile,-delayNumSample);



  if (audioFile==NULL)
    CommonExit(1,"Decode: error opening audio file %s "
               "(maybe unknown format \"%s\")",
               audioFileName,audioFileFormat);

 
 /* seek to beginning of first frame (with delay compensation) */
  AudioSeek(audioFile,-delayNumSample);

  return streamCount ;
}

