/**********************************************************************
MPEG-4 Audio VM
Low-overhead MPEG-4 Audio Transport Multiplex

This software module was originally developed by

Toshiyuki Nomura (NEC Corporation)

and edited by

in the course of development of the MPEG-2 AAC/MPEG-4 Audio standard
ISO/IEC 13818-7, 14496-1,2 and 3. This software module is an
implementation of a part of one or more MPEG-2 AAC/MPEG-4 Audio tools
as specified by the MPEG-2 AAC/MPEG-4 Audio standard. ISO/IEC gives
users of the MPEG-2 AAC/MPEG-4 Audio standards free license to this
software module or modifications thereof for use in hardware or
software products claiming conformance to the MPEG-2 AAC/ MPEG-4 Audio
standards. Those intending to use this software module in hardware or
software products are advised that this use may infringe existing
patents. The original developer of this software module and his/her
company, the subsequent editors and their companies, and ISO/IEC have
no liability for use of this software module or modifications thereof
in an implementation. Copyright is not released for non MPEG-2
AAC/MPEG-4 Audio conforming products. The original developer retains
full right to use the code for his/her own purpose, assign or donate
the code to a third party and to inhibit third party from using the
code for non MPEG-2 AAC/MPEG-4 Audio conforming products. This
copyright notice must be included in all copies or derivative works.

Copyright (c) 1999.
**********************************************************************/

#include <assert.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#include "obj_descr.h"           /* structs */
#include "latm_handle.h"

#include "common_m4a.h"          /* common module       */
#include "bitstream.h"           /* bit stream module   */

#include "mp4au.h"               /* frame work common declarations */

#include "flex_mux.h"
#include "latm_modules.h"

static void Convert( char *inBitFileName,
		     char *outBitFileName,
		     int  mode )
{
  char               *info;
  char               info1[256];
  HANDLE_BSBITSTREAM inBitStream;
  HANDLE_BSBITSTREAM outBitStream;
  HANDLE_BSBITSTREAM tmpStream;
  HANDLE_BSBITBUFFER tmpBitBuffer;

  FRAME_DATA         frameData;
  OBJECT_DESCRIPTOR  objDescr;
  unsigned int       index;               /* FlexMux Channel */
  unsigned long      length[8];           /* SL-Packet length */

  unsigned long      latm_sync_word;      /* LATM: AudioSyncStream sync word */
  unsigned long      audioMuxLengthBytes; /* LATM: length of audioMuxElement */
  int                muxConfigPresent;
  AUDIO_MUX_ELEMENT  audioMuxElement;

  unsigned int       x, y, firstFrame;
  unsigned long      ltmp,align,dummy;
  long               st, ed, cnt;
  
  /* init */
  BsInit(MAXREADAHEAD,0,1);
  info = NULL;
  
  /* open bit stream file */
  if ( mode == 0 ) {        /* FlexMux -> LATM */
    inBitStream = BsOpenFileRead(inBitFileName,MP4_MAGIC,&info);
    sprintf(info1,"\nLATM converter (FlexMux -> LATM)\n");
    outBitStream = BsOpenFileWrite(outBitFileName,MP4_MAGIC,info1);
  } else if ( mode == 1 ) { /* LATM -> FlexMux*/
    inBitStream = BsOpenFileRead(inBitFileName,MP4_MAGIC,&info);
    sprintf(info1,"\nLATM converter (LATM -> FlexMux)\n");
    outBitStream = BsOpenFileWrite(outBitFileName,MP4_MAGIC,info1);
  }

  muxConfigPresent = 1;

  if (inBitStream==NULL) {
    CommonWarning("error opening bit stream file %s",inBitFileName);
  }
  if (outBitStream==NULL) {
    CommonWarning("error opening bit stream file %s",outBitFileName);
  }

  if ( mode == 0 ) {      /* FlexMux -> LATM */
    /* read input bit stream header (FlexMux) */
    initObjDescr( &objDescr,0 );
    if (BsGetBit(inBitStream,&(objDescr.ODLength.value),objDescr.ODLength.length) )
      CommonExit(1,"Read: error reading bit stream header (FlexMux)");

    advanceODescr(inBitStream, &objDescr, 0);
    for (x=0;x<objDescr.streamCount.value;x++){
      initESDescr(&(objDescr.ESDescriptor[x]));
      advanceESDescr(inBitStream,objDescr.ESDescriptor[x],0);
      frameData.layer[x].bitBuf = BsAllocBuffer(64000);
    }

    align = 8 -  BsCurrentBit(inBitStream) % 8;
    if (align == 8) align = 0;
    BsGetBit(inBitStream,&(dummy),align) ;
    frameData.od = &objDescr;

    setStreamMuxConfig( &frameData, &audioMuxElement );

    latm_sync_word = 0x2B7;
    firstFrame = 1;
    while ( ! BsEof ( inBitStream, 8) ) {
      /* read FlexMux-stream frame by frame */
      for (x=0;x<objDescr.streamCount.value;x++){
	length[x] = 0;
	getAccessUnit( inBitStream, frameData.layer[x].bitBuf,
		      &index, &length[x],
		      objDescr.ESDescriptor[x] );
	if ( x != index ) 
	  CommonExit(1,"Read: error reading bit stream frame");
      }

      /* write audioSyncStream frame by frame */
      if ( firstFrame ) {
	firstFrame = 0;
	audioMuxElement.useSameStreamMux = 0;
      } else {
	audioMuxElement.useSameStreamMux = 1;
      }

      setPayloadLengthInfo( &audioMuxElement, length );

      tmpBitBuffer = BsAllocBuffer(BITHEADERBUFSIZE);
      tmpStream = BsOpenBufferWrite(tmpBitBuffer);

      writeAudioMuxElement(tmpStream, &audioMuxElement, muxConfigPresent);

      BsClose(tmpStream);
      ltmp=BsBufferNumBit(tmpBitBuffer)/8;
      align = 8 - BsBufferNumBit(tmpBitBuffer) % 8;
      if (align == 8) align = 0;
      if (align != 0) ltmp += 1;
      audioMuxLengthBytes = ltmp;

      BsPutBit(outBitStream,latm_sync_word,11);
      BsPutBit(outBitStream,audioMuxLengthBytes,13);
      if (BsPutBuffer(outBitStream,tmpBitBuffer))
	CommonExit(1,"error writing LATM bit stream");
      BsPutBit(outBitStream,0,align);

      BsFreeBuffer(tmpBitBuffer);

      for (x=0;x<objDescr.streamCount.value;x++){
	BsClearBuffer(frameData.layer[x].bitBuf);
      }
    }
  }

  if ( mode == 1 ) {      /* LATM -> FlexMux */
    firstFrame = 1;
    while ( ! BsEof ( inBitStream, 8) ) {
      if ( firstFrame ) {
	initObjDescr( &objDescr,0 );
	for (x=0;x<8;x++) {
	  initESDescr(&(objDescr.ESDescriptor[x]));
	  frameData.layer[x].bitBuf = BsAllocBuffer(64000);
	}
      }
      /* read audioSyncStream frame by frame */
      BsGetBit(inBitStream,&latm_sync_word,11);
      if ( latm_sync_word != 0x2B7 ) {
	CommonExit(1,"error read LATM sync word");
      }
      BsGetBit(inBitStream,&audioMuxLengthBytes,13);

      st = BsCurrentBit(inBitStream);
      readAudioMuxElementH(inBitStream, &audioMuxElement, &objDescr, muxConfigPresent );
      cnt = 0;
      for ( x = 0; x < audioMuxElement.streamMuxConfig.numProgram+1; x++ ) {
	for ( y = 0; y < audioMuxElement.streamMuxConfig.numLayer[x]+1; y++ ) {
	  audioMuxElement.payloadMux[0].bitBuf[cnt] = frameData.layer[y].bitBuf;
	  cnt++;
	}
      }
      readAudioMuxElementP(inBitStream, &audioMuxElement );
      ed = BsCurrentBit(inBitStream);

      ltmp = ed - st;
      align = 8 -  ltmp % 8;
      if (align == 8) align = 0;
      BsGetBit(inBitStream,&dummy,align) ;
      ed = BsCurrentBit(inBitStream);
      if ( (ed-st) != (long)audioMuxLengthBytes*8 ) {
	CommonExit(1,"error reading LATM bit stream (%d %d)",
		   (ed-st),audioMuxLengthBytes*8);
      }

      if ( firstFrame ) {
	firstFrame = 0;
	frameData.od = &objDescr;
	setObjectDescriptor( &frameData, &audioMuxElement ); /**/

	/* write output bit stream header (FlexMux) */
	tmpBitBuffer = BsAllocBuffer(BITHEADERBUFSIZE);
	tmpStream = BsOpenBufferWrite(tmpBitBuffer);

	advanceODescr(tmpStream, &objDescr, 1);
	for (x=0;x<objDescr.streamCount.value;x++){
	  advanceESDescr(tmpStream, objDescr.ESDescriptor[x], 1);
	}

	BsClose(tmpStream);
	ltmp=BsBufferNumBit(tmpBitBuffer)/8;
	align = 8 - BsBufferNumBit(tmpBitBuffer) % 8;
	if (align == 8) align = 0;
	if (align != 0) ltmp += 1;

	BsPutBit(outBitStream,ltmp,32);
	if (BsPutBuffer(outBitStream,tmpBitBuffer))
	  CommonExit(1,"Write: error writing bit stream header (FlexMux)");
	BsPutBit(outBitStream,0,align);
	BsFreeBuffer(tmpBitBuffer);
      }

      /* write FlexMux-stream frame by frame */
      for (x=0;x<objDescr.streamCount.value;x++){
	putAccessUnit(x, outBitStream, frameData.layer[x].bitBuf,
		      objDescr.ESDescriptor[x] );
      }

      for (x=0;x<objDescr.streamCount.value;x++){
	BsClearBuffer(frameData.layer[x].bitBuf);
      }
    }
  }

  /* close bit stream file */
  if (BsClose(inBitStream))
    CommonExit(1,"error closing bit stream file");
  if (BsClose(outBitStream))
    CommonExit(1,"error closing bit stream file");
}

int main (int argc, char *argv[])
{
  char inBitFileName[STRLEN];
  char outBitFileName[STRLEN];
  int  mode;

  if(argc != 4){
    printf("usage: %s <mode> <infile> <outfile>\n", argv[0]);
    printf("          mode:\n");
    printf("          0: conversion from FlexMux to LATM\n");
    printf("          1: conversion from LATM to FlexMux\n");
    exit(0);
  }
  mode = atoi(argv[1]);
  strcpy(inBitFileName, argv[2]);
  strcpy(outBitFileName, argv[3]);

  Convert( inBitFileName, outBitFileName, mode );
  return(0);
}
