/**********************************************************************
MPEG-4 Audio VM
MP4 to FlexMux converter

This software module was originally developed by

Akira Inoue (Sony Corporation)

and edited by

in the course of development of the MPEG-2 AAC/MPEG-4 Audio standard
ISO/IEC 13818-7, 14496-1,2 and 3. This software module is an
implementation of a part of one or more MPEG-2 AAC/MPEG-4 Audio tools
as specified by the MPEG-2 AAC/MPEG-4 Audio standard. ISO/IEC gives
users of the MPEG-2 AAC/MPEG-4 Audio standards free license to this
software module or modifications thereof for use in hardware or
software products claiming conformance to the MPEG-2 AAC/ MPEG-4 Audio
standards. Those intending to use this software module in hardware or
software products are advised that this use may infringe existing
patents. The original developer of this software module and his/her
company, the subsequent editors and their companies, and ISO/IEC have
no liability for use of this software module or modifications thereof
in an implementation. Copyright is not released for non MPEG-2
AAC/MPEG-4 Audio conforming products. The original developer retains
full right to use the code for his/her own purpose, assign or donate
the code to a third party and to inhibit third party from using the
code for non MPEG-2 AAC/MPEG-4 Audio conforming products. This
copyright notice must be included in all copies or derivative works.

Copyright (c) 1999.
Source file: fl4_to_mp4.c

 

Required modules:
common_m4a.o            common module
cmdline.o               command line module
bitstream.o             bit stream module
flexmux.o		flexmux module

**********************************************************************/

#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#include "obj_descr.h"           /* structs */

#include "common_m4a.h"          /* common module       */
#include "cmdline.h"             /* command line module */
#include "bitstream.h"           /* bit stream module   */

#include "mp4au.h"               /* frame work common declarations */

#include "flex_mux.h"

#include "lpc_common.h"

#include "MP4Movies.h" 

#define PROGVER "MP4 to FlexMux Converter V1.0 17-Dec-1999"
#define CVSID "$Id"

static char *outFileName;
static int outFileNameUsed;
static int debugLevel;

static char *oriPath,*bitPath;
static char *oriExt,*bitExt;

static int*  varArgIdx;

static CmdLinePara paraList[] = {
  {&varArgIdx,NULL,"<bit stream file(s) (- = stdin)>"},
  {NULL,NULL,NULL}
};

static CmdLineSwitch switchList[] = {
  {"h",NULL,NULL,NULL,NULL,"print help"},
  {"o",&outFileName,"%s",NULL,&outFileNameUsed,
   "output file name (- = stdout)"},
  {"d",&debugLevel,"%i","0",NULL,"debug level"},
  {NULL,NULL,NULL,NULL,NULL,NULL}
};

static void BsRWBitWrapper(HANDLE_BSBITSTREAM stream,         /* in: bit stream */
                      unsigned long *data,              /* out: bits read/write */
                      int numBit,               /* in: num bits to read */
                      int WriteFlag)
{
  if (WriteFlag){
    BsPutBit(stream,*data,numBit);
  } else {
    BsGetBit(stream,data,numBit);
  }
}

static void advanceAudioSpecConf (HANDLE_BSBITSTREAM  bitStream,
				  ES_DESCRIPTOR *es,
				  int WriteFlag)
{
  AUDIO_SPECIFIC_CONFIG *audSpC;
  
  audSpC = &(es->DecConfigDescr.audioSpecificConfig);

  BsRWBitWrapper(bitStream, &(audSpC->audioDecoderType.value), audSpC->audioDecoderType.length, WriteFlag);
  BsRWBitWrapper(bitStream, &(audSpC->samplingFreqencyIndex.value), audSpC->samplingFreqencyIndex.length,WriteFlag);
  BsRWBitWrapper(bitStream, &(audSpC->channelConfiguration.value), audSpC->channelConfiguration.length,WriteFlag);
  
  switch (audSpC->audioDecoderType.value) {
  case AAC_SCAL:
  case TWIN_VQ:
#ifdef VERSION2
  case ER_BSAC : /* bsac */
  case ER_TWIN_VQ:
#endif
    initTFspecConf(&(audSpC->specConf.TFSpecificConfig));
    advanceTFspecConf(bitStream,&(audSpC->specConf.TFSpecificConfig),WriteFlag);
    break;

  case CELP :
    if ( es->streamDependence.value == 0 ) { /*CELP BaseLayer*/
      initCelpSpecConf (&(audSpC->specConf.celpSpecificConfig)
#ifdef VERSION2
			,audSpC->audioDecoderType.value
#endif
                        ); 
      advanceCelpSpecConf(bitStream,&(audSpC->specConf.celpSpecificConfig),WriteFlag);
    } else if ( es->DecConfigDescr.audioSpecificConfig.BWS_on==1 ) { /*CELP BWS Enh.Layer*/
      initCelpEnhSpecConf (&(audSpC->specConf.celpEnhSpecificConfig)); 
      advanceCelpEnhSpecConf(bitStream,&(audSpC->specConf.celpEnhSpecificConfig),WriteFlag);
    } /* No SpecConf for CELP BRS Enh.Layer */
    break;
#ifdef VERSION2
    case ER_CELP :
      if ( es->streamDependence.value == 0 ) { /*CELP BaseLayer*/
        initCelpSpecConf (&(audSpC->specConf.celpSpecificConfig)
#ifdef VERSION2
                          ,audSpC->audioDecoderType.value
#endif
                          ); 
        advanceErCelpSpecConf(bitStream,&(audSpC->specConf.celpSpecificConfig),WriteFlag);
      } else if ( es->DecConfigDescr.audioSpecificConfig.BWS_on==1 ) { /*CELP BWS Enh.Layer*/
        initCelpEnhSpecConf (&(audSpC->specConf.celpEnhSpecificConfig)); 
        advanceCelpEnhSpecConf(bitStream,&(audSpC->specConf.celpEnhSpecificConfig),WriteFlag);
      } /* No SpecConf for CELP BRS Enh.Layer */
      break;
#endif
  case HVXC:
    if (es->streamDependence.value == 0) { /* HVXC BaseLayer */
      initHvxcSpecConf (&(audSpC->specConf.hvxcSpecificConfig));
      advanceHvxcSpecConf(bitStream,&(audSpC->specConf.hvxcSpecificConfig),WriteFlag);
      }
      else {    /* HVXC Enh.Layer */
        /* No SpecConf */
      }
      break;
#ifdef VERSION2
  case ER_HVXC:
    if (es->streamDependence.value == 0) { /* HVXC BaseLayer */
      initHvxcSpecConf (&(audSpC->specConf.hvxcSpecificConfig));
      advanceHvxcSpecConf(bitStream,&(audSpC->specConf.hvxcSpecificConfig),WriteFlag);
      }
      else {    /* HVXC Enh.Layer */
        /* No SpecConf */
      }
      break;
#endif
  default:
    CommonExit(-1,"audioDecoderType not implemented");
    break;
  }
}


/* read decoderSpecificInfo and set them to "ES Descriptor" */
static MP4Err readAudioDecoderSpecificInfo(
ES_DESCRIPTOR *es,
MP4Handle decoderSpecificInfoH
)
{
  MP4Err err;
  HANDLE_BSBITBUFFER bitBuf;
  HANDLE_BSBITSTREAM stream;
  u32 handleSize;

  err = MP4GetHandleSize(decoderSpecificInfoH, &handleSize); if (err) goto bail;
  if (debugLevel >= 3)
    printf("send decoderSpecificInfo to decoder: size = %d\n", handleSize);

  bitBuf = BsAllocBuffer(8*handleSize);	/* allocate bit buffer */
  stream = BsOpenBufferRead(bitBuf); 

  BsDirtyDirectOnBuffer((u8*) *decoderSpecificInfoH,handleSize,debugLevel,bitBuf);

  es->DecConfigDescr.specificInfoFlag.value = 1;
  es->DecConfigDescr.specificInfoLength.value = handleSize;
  
  advanceAudioSpecConf(stream, es, 0);
    
  BsFreeBuffer(bitBuf);

bail:
  return err;
}


static void debugPrintObjDescr(
OBJECT_DESCRIPTOR *od
)
{
  unsigned int x;
  ES_DESCRIPTOR *es;
  AUDIO_SPECIFIC_CONFIG *audSpC;
  
  printf("objDescr.ODLength = %ld\n", od->ODLength.value);
  printf("objDescr.ODescrId = %ld\n", od->ODescrId.value);
  printf("objDescr.streamCount = %ld\n", od->streamCount.value);
  printf("objDescr.extensionFlag = %ld\n", od->extensionFlag.value);
  for (x=0;x<od->streamCount.value;x++){
    es = od->ESDescriptor[x];
    
    printf("\nESDescriptor.ESNumber: %ld \n",es->ESNumber.value);
    printf("  ESDescriptor.streamDependence: %ld \n",
	   es->streamDependence.value);
    printf("  ESDescriptor.dependsOn_Es_number: %ld \n",
	   es->dependsOn_Es_number.value);
    printf("  ESDescriptor.DecConfigDescr.profileAndLevelIndication: %ld \n",
	   es->DecConfigDescr.profileAndLevelIndication.value);
    printf("  ESDescriptor.DecConfigDescr.streamType: %ld \n",
	   es->DecConfigDescr.streamType.value);
    printf("  ESDescriptor.DecConfigDescr.specificInfoFlag: %ld \n",
	   es->DecConfigDescr.specificInfoFlag.value);
    printf("  ESDescriptor.DecConfigDescr.bufferSizeDB: %ld \n",
	   es->DecConfigDescr.bufferSizeDB.value);
    printf("  ESDescriptor.DecConfigDescr.maxBitrate: %ld \n",
	   es->DecConfigDescr.maxBitrate.value);
    printf("  ESDescriptor.DecConfigDescr.avgBitrate: %ld \n",
	   es->DecConfigDescr.avgBitrate.value);
    printf("  ESDescriptor.DecConfigDescr.specificInfoLength: %ld\n",
	   es->DecConfigDescr.specificInfoLength.value);
    printf("  ESDescriptor.DecConfigDescr.audioSpecificConfig.audioDecoderType: %ld \n",
	   es->DecConfigDescr.audioSpecificConfig.audioDecoderType.value);
    printf("  ESDescriptor.DecConfigDescr.audioSpecificConfig.samplingFreqencyIndex: %ld \n",
	   es->DecConfigDescr.audioSpecificConfig.samplingFreqencyIndex.value);
    printf("  ESDescriptor.DecConfigDescr.audioSpecificConfig.channelConfiguration: %ld \n",
	   es->DecConfigDescr.audioSpecificConfig.channelConfiguration.value);

    audSpC = &(es->DecConfigDescr.audioSpecificConfig);
    
    switch(audSpC->audioDecoderType.value) {
    case AAC_SCAL:
    case TWIN_VQ:
#ifdef VERSION2
    case ER_TWIN_VQ:
#endif
      printf("  ESDescriptor.DecConfigDescr.audioSpecificConfig.TFSpecificConfig.TFCodingType: %ld \n",
	     audSpC->specConf.TFSpecificConfig.TFCodingType.value);
      printf("  ESDescriptor.DecConfigDescr.audioSpecificConfig.TFSpecificConfig.frameLength: %ld \n",
	     audSpC->specConf.TFSpecificConfig.frameLength.value);
      printf("  ESDescriptor.DecConfigDescr.audioSpecificConfig.TFSpecificConfig.dependsOnCoreCoder: %ld \n",
	     audSpC->specConf.TFSpecificConfig.dependsOnCoreCoder.value);
      if (audSpC->specConf.TFSpecificConfig.dependsOnCoreCoder.value != 0) { 
	printf("  ESDescriptor.DecConfigDescr.audioSpecificConfig.TFSpecificConfig.coreCoderDelay: %ld \n",
	       audSpC->specConf.TFSpecificConfig.coreCoderDelay.value);
      }
      printf("  ESDescriptor.DecConfigDescr.audioSpecificConfig.TFSpecificConfig.extension: %ld \n",
	     audSpC->specConf.TFSpecificConfig.extension.value);
      break;
    case CELP:
      if (es->streamDependence.value == 0) { /*CELP BaseLayer*/
	printf("  ESDescriptor.DecConfigDescr.audioSpecificConfig.celpSpecificConfig.excitationMode: %ld \n",
	       audSpC->specConf.celpSpecificConfig.excitationMode.value);
	printf("  ESDescriptor.DecConfigDescr.audioSpecificConfig.celpSpecificConfig.sampleRateMode: %ld \n",
	       audSpC->specConf.celpSpecificConfig.sampleRateMode.value);
	printf("  ESDescriptor.DecConfigDescr.audioSpecificConfig.celpSpecificConfig.fineRateControl: %ld \n",
	       audSpC->specConf.celpSpecificConfig.fineRateControl.value);
	if (audSpC->specConf.celpSpecificConfig.excitationMode.value == RegularPulseExc) {
	  printf("  ESDescriptor.DecConfigDescr.audioSpecificConfig.celpSpecificConfig.RPE_Configuration: %ld \n",
		 audSpC->specConf.celpSpecificConfig.RPE_Configuration.value);
	}
	if (audSpC->specConf.celpSpecificConfig.excitationMode.value == MultiPulseExc) {
	  printf("  ESDescriptor.DecConfigDescr.audioSpecificConfig.celpSpecificConfig.MPE_Configuration: %ld \n",
		 audSpC->specConf.celpSpecificConfig.MPE_Configuration.value);
	  printf("  ESDescriptor.DecConfigDescr.audioSpecificConfig.celpSpecificConfig.numEnhLayers: %ld \n",
		 audSpC->specConf.celpSpecificConfig.numEnhLayers.value);
	  printf("  ESDescriptor.DecConfigDescr.audioSpecificConfig.celpSpecificConfig.bandwidthScalabilityMode: %ld \n",
		 audSpC->specConf.celpSpecificConfig.bandwidthScalabilityMode.value);
	}
      } else if ( es->DecConfigDescr.audioSpecificConfig.BWS_on==1 ) { /*CELP BWS Enh.Layer*/
	printf("  ESDescriptor.DecConfigDescr.audioSpecificConfig.celpEnhSpecificConfig.BWS_Configuration: %ld \n",
	       audSpC->specConf.celpEnhSpecificConfig.BWS_Configuration.value);
      } /* No SpecConf for CELP BRS Enh.Layer */
      break;
#ifdef VERSION2      
    case ER_CELP:
      if (es->streamDependence.value == 0) { /*CELP BaseLayer*/
	printf("  ESDescriptor.DecConfigDescr.audioSpecificConfig.celpSpecificConfig.excitationMode: %ld \n",
	       audSpC->specConf.celpSpecificConfig.excitationMode.value);
	printf("  ESDescriptor.DecConfigDescr.audioSpecificConfig.celpSpecificConfig.sampleRateMode: %ld \n",
	       audSpC->specConf.celpSpecificConfig.sampleRateMode.value);
	printf("  ESDescriptor.DecConfigDescr.audioSpecificConfig.celpSpecificConfig.fineRateControl: %ld \n",
	       audSpC->specConf.celpSpecificConfig.fineRateControl.value);
	printf("  ESDescriptor.DecConfigDescr.audioSpecificConfig.celpSpecificConfig.silenceCompressionSW: %ld \n",
	       audSpC->specConf.celpSpecificConfig.silenceCompressionSW.value);
	if (audSpC->specConf.celpSpecificConfig.excitationMode.value == RegularPulseExc) {
	  printf("  ESDescriptor.DecConfigDescr.audioSpecificConfig.celpSpecificConfig.RPE_Configuration: %ld \n",
		 audSpC->specConf.celpSpecificConfig.RPE_Configuration.value);
	}
	if (audSpC->specConf.celpSpecificConfig.excitationMode.value == MultiPulseExc) {
	  printf("  ESDescriptor.DecConfigDescr.audioSpecificConfig.celpSpecificConfig.MPE_Configuration: %ld \n",
		 audSpC->specConf.celpSpecificConfig.MPE_Configuration.value);
	  printf("  ESDescriptor.DecConfigDescr.audioSpecificConfig.celpSpecificConfig.numEnhLayers: %ld \n",
		 audSpC->specConf.celpSpecificConfig.numEnhLayers.value);
	  printf("  ESDescriptor.DecConfigDescr.audioSpecificConfig.celpSpecificConfig.bandwidthScalabilityMode: %ld \n",
		 audSpC->specConf.celpSpecificConfig.bandwidthScalabilityMode.value);
	}
      } else if ( es->DecConfigDescr.audioSpecificConfig.BWS_on==1 ) { /*CELP BWS Enh.Layer*/
	printf("  ESDescriptor.DecConfigDescr.audioSpecificConfig.celpEnhSpecificConfig.BWS_Configuration: %ld \n",
	       audSpC->specConf.celpEnhSpecificConfig.BWS_Configuration.value);
      } /* No SpecConf for CELP BRS Enh.Layer */
      break;
#endif
    case HVXC:
#ifdef VERSION2
    case ER_HVXC:
#endif
      if (es->streamDependence.value == 0) {
	printf("  ESDescriptor.DecConfigDescr.audioSpecificConfig.hvxcSpecificConfig.HVXCvarMode: %ld \n",
	       audSpC->specConf.hvxcSpecificConfig.HVXCvarMode.value);
	printf("  ESDescriptor.DecConfigDescr.audioSpecificConfig.hvxcSpecificConfig.HVXCrateMode: %ld \n",
	       audSpC->specConf.hvxcSpecificConfig.HVXCrateMode.value);
	printf("  ESDescriptor.DecConfigDescr.audioSpecificConfig.hvxcSpecificConfig.extensionFlag: %ld \n",
	       audSpC->specConf.hvxcSpecificConfig.extensionFlag.value);
#ifdef VERSION2
	if (audSpC->specConf.hvxcSpecificConfig.extensionFlag.value) {
#if 0
	  printf("  ESDescriptor.DecConfigDescr.audioSpecificConfig.hvxcSpecificConfig.packMode: %ld \n",
		 audSpC->specConf.hvxcSpecificConfig.packMode.value);
#endif
	  printf("  ESDescriptor.DecConfigDescr.audioSpecificConfig.hvxcSpecificConfig.vrScalFlag: %ld \n",
		 audSpC->specConf.hvxcSpecificConfig.vrScalFlag.value);
	}
#endif
      }
      break;
    default:
      CommonExit(-1,"audioDecoderType not implemented");
      break;
    }
  }
}


static MP4Err MP4_to_FlexMux(char *inBitFileName, char *outBitFileName)
{
  char               info[256];
  HANDLE_BSBITSTREAM outBitStream;
  HANDLE_BSBITSTREAM tmpStream;
  HANDLE_BSBITBUFFER tmpBitBuffer;

  FRAME_DATA         frameData;
  OBJECT_DESCRIPTOR  objDescr;

  unsigned int       x;
  unsigned long      ltmp,align;
  
  ES_DESCRIPTOR *es;

  /* MP4 stuffs */
  MP4Err err;
  MP4Movie moov;
  MP4Track trak[8];
  MP4Media media[8];
  u32 trackID;
  u32 dependsOn_ID;
  
  MP4TrackReader reader[8];

  MP4Handle initialObjectDescriptorH;
  MP4Handle decoderConfigH;
  MP4Handle decoderSpecificInfoH;
  MP4Handle sampleDataH;
  u32 handlerType;
  u32 objectTypeIndication;
  u32 streamType;
  u32 decoderBufferSize;
  u32 upStream;
  u32 maxBitrate;
  u32 avgBitrate;
  u32 trackCount;
  u32 unitSize;
  u32 cts;
  u32 dts;
  u32 sampleFlags;
  u32 decoderConfigSize;
  u32 decoderSpecificInfoSize;
  u32 enabled;

  /* init */
  BsInit(MAXREADAHEAD,0,1);	/* second argument is debug level */
  
  /* open bit stream file */
  sprintf(info,"\nMP4 converter (MP4 -> FlexMux)\n");
  outBitStream = BsOpenFileWrite(outBitFileName,MP4_MAGIC,info);
  if (outBitStream==NULL) {
    CommonWarning("error opening bit stream file %s",outBitFileName);
  }

  err = MP4OpenMovieFile(&moov, inBitFileName, MP4OpenMovieNormal); if (err) goto bail;
  err = MP4NewHandle(0, &initialObjectDescriptorH); if (err) goto bail;
  err = MP4GetMovieInitialObjectDescriptor(moov, initialObjectDescriptorH); if (err) goto bail;

  /* determine the number of tracks in a movie */
  err = MP4GetMovieTrackCount(moov, &trackCount);
  if (debugLevel >= 1) printf("trackCount=%d\n", trackCount);

  /* write output FlexMux stream header */
  initObjDescr(&objDescr,0);
  presetObjDescr(&objDescr, trackCount);

  for (x=0; x<trackCount; x++) {
    trackID = x+1;
    err = MP4GetMovieIndTrack(moov, trackID, &trak[x]); if (err) goto bail;
    err = MP4GetTrackEnabled(trak[x], &enabled);	/* check track enable (AI 2000/01/18) */
    
    if (enabled) {
      err = MP4GetTrackMedia(trak[x], &media[x]); if (err) goto bail;
      err = MP4GetMediaHandlerDescription(media[x], &handlerType, NULL); if (err) goto bail;
      err = MP4CreateTrackReader(trak[x], &reader[x]); if (err) goto bail;
      err = MP4NewHandle(0, &decoderConfigH); if (err) goto bail;
      err = MP4TrackReaderGetCurrentDecoderConfig(reader[x], decoderConfigH); if (err) goto bail;
      err = MP4GetHandleSize(decoderConfigH, &decoderConfigSize); if (err) goto bail;
    
      err = MP4NewHandle(0, &decoderSpecificInfoH); if (err) goto bail;

      /* to obtain decoder configuration and specific info */
      err = MP4GetMediaDecoderInformation(media[x], 
					  1, 
					  &objectTypeIndication, 
					  &streamType, 
					  &decoderBufferSize, 	
					  &upStream,
					  &maxBitrate,
					  &avgBitrate,
					  decoderSpecificInfoH); if (err) goto bail;
    			       
      err = MP4GetHandleSize(decoderSpecificInfoH, &decoderSpecificInfoSize); if (err) goto bail;
      if (handlerType != MP4AudioHandlerType) {
	CommonExit(1,"trackID=%d: this is not Audio Elementary Stream");
      }
      if (debugLevel >= 3) {
	printf("trackID=%d\n", trackID);
	printf("  objectTypeIndication=0x%x\n", objectTypeIndication);
	printf("  streamType=0x%x\n", streamType);
	printf("  decoderBufferSize=%d\n", decoderBufferSize);
	printf("  upstream=0x%x\n", upStream);
	printf("  maxBitrate=%d\n", maxBitrate);
	printf("  avgBitrate=%d\n", avgBitrate);
	printf("  decoderConfigSize=%d\n", decoderConfigSize);
      }

      initESDescr(&(objDescr.ESDescriptor[x]));
      presetESDescr(objDescr.ESDescriptor[x], x);
      if ( ( x>0 ) && (objDescr.ESDescriptor[0]->DecConfigDescr.audioSpecificConfig.specConf.celpSpecificConfig.bandwidthScalabilityMode.value==1) &&
	   (x == (objDescr.ESDescriptor[0]->DecConfigDescr.audioSpecificConfig.specConf.celpSpecificConfig.numEnhLayers.value + 1)) ){
	objDescr.ESDescriptor[x]->DecConfigDescr.audioSpecificConfig.BWS_on=1;
      }
      es = objDescr.ESDescriptor[x];

      /* read decoderSpecificInfo and set them to "ES Descriptor" */
      es->ESNumber.value = trackID;
      es->DecConfigDescr.streamType.value = streamType; 	/* 0x05(audio stream) */
      es->DecConfigDescr.bufferSizeDB.value = decoderBufferSize;	

      es->DecConfigDescr.maxBitrate.value = maxBitrate;
      es->DecConfigDescr.avgBitrate.value = avgBitrate;	

      es->ALConfigDescriptor.useAccessUnitStartFlag.value = 1;
      es->ALConfigDescriptor.useAccessUnitEndFlag.value = 1;
      es->ALConfigDescriptor.seqNumLength.value = 0;
      
      /* check Elementary Stream dependency */
      err = MP4GetTrackReferenceCount(trak[x],
				      MP4StreamDependencyReferenceType,
				      &dependsOn_ID);

      /* setting stream dependency is done in presetESDescr() */
#if 0	
      if (dependsOn_ID != 0) {
	/* this Elementary Stream has dependency */
	es->streamDependence.value = 1;
	es->dependsOn_Es_number.value = dependsOn_ID;
      }
      else {
	/* this Elementary Stream has no dependency */
	es->streamDependence.value = 0;
	es->dependsOn_Es_number.value = 0;
      }
#endif
      /* read audioSpecificCongig after streamDependence determined */
      err = readAudioDecoderSpecificInfo(es, decoderSpecificInfoH); if (err) goto bail;
      err = MP4DisposeHandle(decoderConfigH);
      err = MP4DisposeHandle(decoderSpecificInfoH);
    } 
    else {
      fprintf(stderr, "the Track[%d] is disabled.\n", x);
    }
  }
  
  /* write output bit stream header (FlexMux) */
  tmpBitBuffer = BsAllocBuffer(BITHEADERBUFSIZE);
  tmpStream = BsOpenBufferWrite(tmpBitBuffer);

  if (debugLevel >= 3) debugPrintObjDescr(&objDescr);
  
  advanceODescr(tmpStream, &objDescr, 1);
  for (x=0;x<objDescr.streamCount.value;x++){
    advanceESDescr(tmpStream, objDescr.ESDescriptor[x], 1);
  }

  BsClose(tmpStream);
  ltmp=BsBufferNumBit(tmpBitBuffer)/8;
  align = 8 - BsBufferNumBit(tmpBitBuffer) % 8;
  if (align == 8) align = 0;
  if (align != 0) ltmp += 1;

  BsPutBit(outBitStream,ltmp,32);
  if (BsPutBuffer(outBitStream,tmpBitBuffer))
    CommonExit(1,"Write: error writing bit stream header (FlexMux)");
  BsPutBit(outBitStream,0,align);
  BsFreeBuffer(tmpBitBuffer);

  /* read Access Unit from MP4 file */
  err = MP4NewHandle(0, &sampleDataH); if (err) goto bail;	
  frameData.od = &objDescr;
  for (x=0;x<objDescr.streamCount.value;x++)
    frameData.layer[x].bitBuf = BsAllocBuffer(64000);

  while (1) {

    for (x=0;x<objDescr.streamCount.value;x++){
      es = objDescr.ESDescriptor[x];
      err = MP4TrackReaderGetNextAccessUnit(reader[x], 
					    sampleDataH, 
					    &unitSize,
					    &sampleFlags, 
					    &cts, 
					    &dts);
    
      if (err) {
	if (err == MP4EOF) err = MP4NoErr;
	goto bail;
      }

      if (debugLevel >= 5) printf("AU of track[%ld]: size=%d, cts=%d, dts=%d\n", es->ESNumber.value, unitSize, cts, dts); 
      BsDirtyDirectOnBuffer((u8*) *sampleDataH,unitSize,debugLevel,frameData.layer[x].bitBuf);

      /* write FlexMux-stream frame by frame */
      putAccessUnit(x, 
		    outBitStream, 
		    frameData.layer[x].bitBuf,
		    objDescr.ESDescriptor[x]);

      BsClearBuffer(frameData.layer[x].bitBuf);
    }
  }
  
  for (x=0; x<trackCount; x++) {
    BsFreeBuffer(frameData.layer[x].bitBuf);
    err = MP4DisposeTrackReader(reader[x]); if (err) goto bail;
  }
  err = MP4DisposeHandle(sampleDataH); if (err) goto bail;
  
bail:
  /* close bit stream file */
  if (BsClose(outBitStream))
    CommonExit(1,"error closing bit stream file");
  return err;
}

int main (int argc, char *argv[])
{
  char *progName = "<no program name>";
  int  result;
  char oriFileName[STRLEN],bitFileName[STRLEN];
  int fileIdx;
  MP4Err err;

  /* evaluate command line  */
  CmdLineInit(0);
  result = CmdLineEval(argc,argv,paraList,switchList,1,&progName);
  if (result) {
    if (result==1) {
      printf("%s: %s\n",progName,PROGVER);
      CmdLineHelp(progName,paraList,switchList,stdout);
      exit (1);
    }
    else
      CommonExit(1,"command line error (\"-h\" for help)");
  }

  oriPath = NULL;
  oriExt = NULL;
  bitPath = NULL;
  bitExt = ".fl4";

  if (debugLevel == 1)
    fprintf(stderr,"%s: %s\n",progName,PROGVER);

  /* process all files on command line */
  fileIdx = 0;
  while (varArgIdx[fileIdx] >= 0) {

    /* compose file names */
    if (ComposeFileName(argv[varArgIdx[fileIdx]],0,oriPath,oriExt,oriFileName,
                        STRLEN))
      CommonExit(1,"composed file name too long");
    if (outFileNameUsed) {
      if (ComposeFileName(outFileName,0,bitPath,bitExt,bitFileName,
                          STRLEN))
        CommonExit(1,"composed file name too long");
    }
    else
      if (ComposeFileName(argv[varArgIdx[fileIdx]],1,bitPath,bitExt,
                          bitFileName,STRLEN))
        CommonExit(1,"composed file name too long");

    /* encode file */
    if (debugLevel >= 1)
      fprintf(stderr,"converting %s -> %s\n",oriFileName,bitFileName);
    if (debugLevel >= 2)
      printf("converting %s -> %s\n",oriFileName,bitFileName);

    /* MP4 to FlexMux conversion */
    err = MP4_to_FlexMux(oriFileName, bitFileName);
    if (err) CommonExit(1,"MP4_to_FlexMux() return %d", err);

    fileIdx++;
  }
  
  CmdLineEvalFree(paraList);

  if (debugLevel == 1)
    fprintf(stderr,"%s: finished\n",progName);
  if (debugLevel >= 2)
    printf("%s: finished\n",progName);
  return(0);
}

