//////////////////////////////////////////////////////////////////////////////
//This software module was originally developed by CSELT in the course of   //
//development of the MPEG-4 Systems (ISO/IEC 14496-1) standard.             //
//This software module is an implementation of a part of one or more MPEG-4 //
//Systems (ISO/IEC 14496-1) tools as specified by the MPEG-4 Systems        //
//(ISO/IEC 14496-1) standard. ISO/IEC gives users of the MPEG-4 Systems     //
//(ISO/IEC 14496-1) free license to this software module or modifications   //
//thereof for use in hardware or software products claiming conformance to  //
//the MPEG-4 Systems (ISO/IEC 14496-1). Those intending to use this software//
//module in hardware or software products are advised that its use may      //
//infringe existing patents.                                                //
//The original developer of this software module and his/her company, the   //
//subsequent editors and their companies, and ISO/IEC have no liability     //
//for use of this software module or modifications thereof in an            //
//implementation. Copyright is not released for non MPEG-4 Systems          //
//(ISO/IEC 14496-1) conforming products. CSELT S.pA. retains full right     //
//to use the code for his/her own purpose, assign or donate the code to a   //
//third party and to inhibit third parties from using the code for non      //
//MPEG-4 Systems (ISO/IEC 14496-1) conforming products. This copyright      //
//notice must be included in all copies or derivative works.                //
//Copyright  1999.                                                         //
//This work has been partly developed within the framework of the ESPRIT    //
//European Research Project  "MPEG-4 PC".                                   //
//////////////////////////////////////////////////////////////////////////////
//
// File:  Background2DProxy.cpp
//
// Author: Matthew Leditschke 
//
// Modified:Gianluca Di Cagno  gianluca.dicagno@cselt.it
//
// Description:
//   The implementation of the proxy object used by the presenter
//   to render the Background2D node.
//
// Created: September, 1997.
//
//
///////////////////////////////////////////////////////////////////////////


#include "defs.h"
#include "Nodes-CSELT.h"
#include "Background2DProxy.h"


// =====================================================================

//
// Presenter::InitBackground2D
//
// Initialise the proxy data structure and attach it to the render data
// pointer of the given media object node.
//

 Background2DProxy* Presenter::InitBackground2D(Background2D* pObj)
{
	return (new Background2DProxy(pObj)); 
}


// =====================================================================

//
// Background2DProxy::Background2DProxy
//
// Class constructor.
//

Background2DProxy::Background2DProxy(MediaObject* pMO) : DrawableImage(pMO)
{
	m_firstTime = TRUE;
} /* Background2DProxy::Background2DProxy */



// =====================================================================

//
// Background2DProxy::PreRender
//
// Call the PreRender() method from the base class, and then adjust the
// bounding rectangle which has just been calculated, so that the
// top-left corner of the bounding rectangle is at the top-left corner
// of the screen, if the screen height is equal to the object height,
// to the difference in the other case.

void Background2DProxy::PreRender(Effects & effects)
{
	// The first time that this is called, the stream needs to be
	// started.
	if (m_firstTime)
	{
		m_firstTime = FALSE;
		GetBackground2D()->OpenStream();
	}

   
	DrawableContext *pContext=InitPreRender(effects);
	
	if (!pContext->IsSwitchedOn()) 
		return;
	
   DrawableImage::PreRender(effects,pContext);
	    
	
	m_boundingRect.left = 0;
	m_boundingRect.top  = m_boundingRect.height; 
	
	UpdateBoundingRect(pContext);
} /* Background2DProxy::PreRender */


// =====================================================================
