// MPEG4Player.cpp : Defines the class behaviors for the application.
//
/*-------------------------------------------------------------------------
MPEGJ 
AUTHORS:    Viswanathan Swaminathan - vishy@eng.sun.com 
			Robert Schmidt(RLS)
			Praveen Alavilli(PA) 
			YUNTAI KYONG(YT) - yuntaikyong@ieee.org
-------------------------------------------------------------------------*/

#include "defs.h"
#include <assert.h>
#include "Clock.h"
#include "Presenter.h"
#include "trace.h"
#include "version.h"
#include "MPEG4Player.h"
#include "MainFrm.h"
#include "MPEG4PlayerDoc.h"
#include "MPEG4PlayerView.h"
#include "WarningMessage.h"
#include "ThreadInfoDlg.h"
#include "ThreadList.h"
#include "ViewWorldInfo.h"
#include "root.h"
#include <crtdbg.h>
#include "PlayListImpl.h"

extern FILE* traceFile;

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

// SISF Merge - PA
#ifdef SIA
 // BB: Made external to resolve link error, internal in SectionFilterEngine.cpp
 // bool M2DOk;
 extern bool M2DOk;
 extern M2DecoderPlus     M2DecObj;
 
#endif
 typedef struct programinfo
 {
	 int	programNumber ;
	 int	PMT_PID ;
	 int	PCR_PID ;
	 int	audioPID ;
	 int	videoPID ;
	 int	dataPID ;
 } PROGRAMINFO, *PPROGRAMINFO ;


 char              Dir_name[256] ;						// Gestione dello stream di ingresso 
 char              OpenFile_name[256] ;
 char			   OpenFile_title[256] ;				// Nome del file dello stream visualizzato
 char              OpenFilter[256] ;
 char			   streamFilePath[256] ;				// Path + nome dello stream
 OPENFILENAME      MPEG_SourceStream ;

 RECT			   ImgDestRect ;
 //M2DecoderPlus     M2DecObj;
 M2DECDISPLAYINFO  displayDescr;
 M2DECINITINFO     infoDescr ;
 M2DECFILTERINFO   filterInfo ;
 M2DECSECTIONDATA  sectionData ;
 PROGRAMINFO       programInfo ;
 bool bDecoderIsPlaying, bUseOverlay, bSourceEnable, startedMPEG4inMPEG2;


 void InitFileNameStruct(void) ;
 void displayError(HRESULT) ;

 void cleanup(unsigned char *fd) ;
 void look_for_PAT(PM2DECFILTERINFO) ;
 void look_for_PMT(int, PM2DECFILTERINFO) ;
 void parsePAT(PM2DECSECTIONDATA, PPROGRAMINFO) ;
 void parsePMT(PM2DECSECTIONDATA, PPROGRAMINFO) ;
 HRESULT openDefaultProgram() ;
// ENd SISF Merge

/////////////////////////////////////////////////////////////////////////////
// CMPEG4PlayerApp

BEGIN_MESSAGE_MAP(CMPEG4PlayerApp, CWinApp)
	//{{AFX_MSG_MAP(CMPEG4PlayerApp)
	ON_COMMAND(ID_APP_ABOUT, OnAppAbout)
	ON_COMMAND(ID_OPTIONS_25FPS, OnOptions25fps)
	ON_UPDATE_COMMAND_UI(ID_OPTIONS_25FPS, OnUpdateOptions25fps)
	ON_COMMAND(ID_OPTIONS_15FPS, OnOptions15fps)
	ON_UPDATE_COMMAND_UI(ID_OPTIONS_15FPS, OnUpdateOptions15fps)
	ON_COMMAND(ID_OPTIONS_10FPS, OnOptions10fps)
	ON_UPDATE_COMMAND_UI(ID_OPTIONS_10FPS, OnUpdateOptions10fps)
	ON_COMMAND(ID_OPTIONS_5FPS, OnOptions5fps)
	ON_UPDATE_COMMAND_UI(ID_OPTIONS_5FPS, OnUpdateOptions5fps)
	ON_COMMAND(ID_OPTIONS_LOOP_CONTINUOUSLY, OnOptionsLoopContinuously)
	ON_UPDATE_COMMAND_UI(ID_OPTIONS_LOOP_CONTINUOUSLY, OnUpdateOptionsLoopContinuously)
	ON_COMMAND(ID_VIEW_THREAD_INFO, OnViewThreadInfo)
	ON_COMMAND(ID_VIEW_WORLD_INFO, OnViewWorldInfo)
	ON_COMMAND(ID_OPTIONS_CLEAR_ON_END, OnOptionsClearOnEnd)
	ON_UPDATE_COMMAND_UI(ID_OPTIONS_CLEAR_ON_END, OnUpdateOptionsClearOnEnd)
	ON_COMMAND(ID_FILE_OPEN, OnFileOpenfile)
	ON_COMMAND(ID_GO_BACK, OnGoBack)
	ON_COMMAND(ID_GO_FORWARD, OnGoForward)
	ON_UPDATE_COMMAND_UI(ID_GO_BACK, OnUpdateGoBack)
	ON_UPDATE_COMMAND_UI(ID_GO_FORWARD, OnUpdateGoForward)
	ON_COMMAND(ID_GO_REPLAY, OnGoReplay)
	ON_UPDATE_COMMAND_UI(ID_GO_REPLAY, OnUpdateGoReplay)	
	ON_COMMAND(ID_FILE_MPEG2TS_OPENFILE, OnFileMpeg2tsOpenfile)
	ON_COMMAND(ID_MPEG2_PLAY, OnMpeg2Play)
	ON_COMMAND(ID_FILE_MPEG2TS_OPENNETWORK, OnFileMpeg2tsOpennetwork)
	ON_COMMAND(ID_FILE_MPEG2TS_CLOSE, OnFileMpeg2tsClose)
	//}}AFX_MSG_MAP
	// Standard file based document commands
	ON_COMMAND(ID_FILE_NEW, CWinApp::OnFileNew)
	ON_COMMAND(ID_FILE_OPEN, CWinApp::OnFileOpen)
END_MESSAGE_MAP()

// ON_COMMAND(ID_FILE_OPENFILE, OnFileOpenfile) - deleted from above message maps

/////////////////////////////////////////////////////////////////////////////
// CMPEG4PlayerApp construction

CMPEG4PlayerApp::CMPEG4PlayerApp()
{
	//BINDU - MPEG-J MERGE _ RLS
#ifdef SIA
 	getcwd(curr_dir, 256);
#endif
	//BINDU - MPEG-J MERGE _ RLS
	m_pExecutive = NULL;
}

/////////////////////////////////////////////////////////////////////////////
// The one and only CMPEG4PlayerApp object

CMPEG4PlayerApp theApp;

/////////////////////////////////////////////////////////////////////////////
// CMPEG4PlayerApp initialization

BOOL CMPEG4PlayerApp::InitInstance()
{
	// Standard initialization
	// If you are not using these features and wish to reduce the size
	//  of your final executable, you should remove from the following
	//  the specific initialization routines you do not need.
	AfxEnableControlContainer();
	
	//**Set a Breakpoint on a specific memory allocation**//
	
	//_CrtSetBreakAlloc(1405);

#ifdef _AFXDLL
	Enable3dControls();			// Call this when using MFC in a shared DLL
#else
	Enable3dControlsStatic();	// Call this when linking to MFC statically
#endif

	LoadStdProfileSettings(9);  // Load standard INI file options (including MRU)

	// Open the logfile (also send stdout to this logfile)
#ifdef WRITE_TRACE
	traceFile = fopen ("trace.log", "w");
//	traceFile = freopen ("trace.log", "w", stdout);
	assert(traceFile != NULL);
	TRACE_PRINT("Starting IM1Player FDIS:\n\n");
#endif

	AddToThreadList("FrameApp", (int)GetCurrentThread());

	// Read the flags which are stored in the application's .INI file
	m_loopContinuously = AfxGetApp()->GetProfileInt("Options", "LoopContinuously", FALSE);
	m_clearOnEnd = AfxGetApp()->GetProfileInt("Options", "ClearOnEnd", FALSE);

	// Register the application's document templates.  Document templates
	//  serve as the connection between documents, frame windows and views.

	CSingleDocTemplate* pDocTemplate;
	pDocTemplate = new CSingleDocTemplate(
		IDR_MAINFRAME1,
		RUNTIME_CLASS(CMPEG4PlayerDoc),
		RUNTIME_CLASS(CMainFrame),       // main SDI frame window
		RUNTIME_CLASS(CMPEG4PlayerView));
	AddDocTemplate(pDocTemplate);

	// Enable DDE Execute open
	EnableShellOpen();
	RegisterShellFileTypes(TRUE);

	// Parse command line for standard shell commands, DDE, file open
	CCommandLineInfo cmdInfo;
	ParseCommandLine(cmdInfo);

	// Dispatch commands specified on the command line
	if (!ProcessShellCommand(cmdInfo))
		return FALSE;

	// Enable drag/drop open
	m_pMainWnd->DragAcceptFiles();

	// Load the list of network addresses from the INI file
	GetProfileNetworkAddresses();

	// Set the default size of the window
	((CMainFrame*)m_pMainWnd)->SetSize(640,480);

	// create and initialize the playlist
	m_playList = new PlayListImpl();

	GetView()->SetFocus(); // the focus must be on the main window


	// SISF - Merge - PA
	// initialize the MPEG-2 data
	bSourceEnable = FALSE ;						 
	bDecoderIsPlaying = FALSE ;
	bUseOverlay = TRUE ;
	infoDescr.length = sizeof(M2DECINITINFO) ;
	displayDescr.length = sizeof(M2DECDISPLAYINFO) ;
	InitFileNameStruct () ;		
	// End SISF - Merge

	 return TRUE;
}

/////////////////////////////////////////////////////////////////////////////
// CAboutDlg dialog used for App About

class CAboutDlg : public CDialog
{
public:
	CAboutDlg();

// Dialog Data
	//{{AFX_DATA(CAboutDlg)
	enum { IDD = IDD_ABOUTBOX };
	CString m_version;
	CString m_date;
	//}}AFX_DATA

	// ClassWizard generated virtual function overrides
	//{{AFX_VIRTUAL(CAboutDlg)
	protected:
	virtual void DoDataExchange(CDataExchange* pDX);    // DDX/DDV support
	//}}AFX_VIRTUAL

// Implementation
protected:
	//{{AFX_MSG(CAboutDlg)
		// No message handlers
	//}}AFX_MSG
	DECLARE_MESSAGE_MAP()
};

CAboutDlg::CAboutDlg() : CDialog(CAboutDlg::IDD)
{
	//{{AFX_DATA_INIT(CAboutDlg)
	//}}AFX_DATA_INIT
}

void CAboutDlg::DoDataExchange(CDataExchange* pDX)
{
	m_version = VERSION_NUMBER
#ifdef WRITE_TRACE
		" (log)"
#else
		""
#endif
#ifdef _DEBUG
		" (debug)";
#else
		"";
#endif

	m_date = VERSION_DATE;
	CDialog::DoDataExchange(pDX);
	//{{AFX_DATA_MAP(CAboutDlg)
	DDX_Text(pDX, IDC_VERSION, m_version);
	DDX_Text(pDX, IDC_DATE, m_date);
	//}}AFX_DATA_MAP
}

BEGIN_MESSAGE_MAP(CAboutDlg, CDialog)
	//{{AFX_MSG_MAP(CAboutDlg)
		// No message handlers
	//}}AFX_MSG_MAP
END_MESSAGE_MAP()

// App command to run the dialog
void CMPEG4PlayerApp::OnAppAbout()
{
	CAboutDlg aboutDlg;
	aboutDlg.DoModal();
}

/////////////////////////////////////////////////////////////////////////////

//
// CMPEG4PlayerApp::CreateExecutive
//
// Called when the view is first initialised.
//

void CMPEG4PlayerApp::CreateExecutive(HWND hWnd)
{
	// Create the executive object
	m_pExecutive = new Executive(this, hWnd,traceFile,TRACE_PARSE);
	
	// Set the initial frame rate
	Presenter* pPres = (Presenter *) GetExecutive()->GetPresenter();
	float frameRate = GetApp()->GetProfileFloat("Options", "FrameRate", 15.0);
	pPres->SetTargetFrameRate(frameRate);
	

	// Tell the visual renderer if it is to clear the screen when
	// the end of a file is reached
	pPres->GetVisualRenderer()->SetClearOnEndFlag(m_clearOnEnd);

	// If the presenter couldn't be initialise correctly, exit
	if (!((Presenter*)m_pExecutive->GetPresenter())->InitialisedOK())
		::PostQuitMessage(-1);

	// If the file has not yet been opened, as is the case if the file
	// was opened by double clicking on it, the file is now opened.
	if (GetDocument()->DocumentNeedsOpening())
		GetDocument()->OpenDocument();
	

} /* CMPEG4PlayerApp::CreateExecutive */


/////////////////////////////////////////////////////////////////////////////

void CALLBACK EXPORT TimerProc(HWND /*hWnd*/, UINT /*nMsg*/,
							   UINT /*nIDEvent*/, DWORD /*dwTime*/)
{
	Presenter* pPres = (Presenter *) GetApp()->GetExecutive()->GetPresenter();

	if(!pPres->IsActive())
	{
		GetStatusBar()->SetWindowText("Loading..");
		return;
	}
	int ms = timeGetTime() - pPres->GetStartTime();
	float fps = pPres->GetActualFrameRate();

	// Format the time as MM.SS.HH
	char text[30];
	sprintf(text, "%02d:%02d.%02d  %.1ffps",
			((int)(ms / 1000)) / 60,
			((int)(ms / 1000)) % 60,
			(ms % 1000) / 10,
			fps);

	// Update the pane in the status bar
	GetStatusBar()->SetWindowText(text);
} /* TimerProc */

/////////////////////////////////////////////////////////////////////////////

//
// CMPEG4PlayerApp::OnConnect
//
// This is called by the Executive when a session has been established,
// indicating if it was established successfully or not.
//
// If successful, start the timer running to update the clock.
// If not, display a warning message.
//

void CMPEG4PlayerApp::OnConnect(RESPONSE result)
{
	if (result == RESPONSE_OK)
	{
		TRACE_PRINT("Session successfully started.\n");
		
	
		// Start a timer to update the running time display
		m_timerID = GetMainFrame()->SetTimer(1, 250, TimerProc);

		// Record that this file has been successfully opened.
		GetDocument()->m_fileOpened = TRUE;
		m_smpFileOpen.P();
	}

	else
	{
		
	//	OnError(result);
		
		/*// Prepare a message to give to the user as to why the connection
		// was not successful.
		char message[200];
		strcpy(message, "The session could not be established because ");
		if (result == FILE_NOT_FOUND)
			strcat(message, "the specifed file could not be found.");
		else if (result == CANNOT_OPEN_FILE)
			strcat(message, "the specifed file could not be opened.");
		else if (result == IP_ADDRESS_NOT_FOUND)
			strcat(message, "the specifed IP address could not be found.");
		else if (result == CANNOT_OPEN_SOCKET)
			strcat(message, "a socket could not be opened.");
		else if (result == MEMORY_PROBLEM)
			strcat(message, "of memory problems.");
		else if (result == UNEXPECTED_EOF)
			strcat(message, "an unexpected end of file was encountered.");
		else
			strcat(message, "an unknown error occurred.");

		CWarningMessage warningMessageDialog;
		warningMessageDialog.m_message = message;
		warningMessageDialog.DoModal();*/
	}
} /* CMPEG4PlayerApp::OnConnect */


/////////////////////////////////////////////////////////////////////////////

//
// CMPEG4PlayerApp::OnFinish
//
// This is called by the Executive when the multiplexer has finished
// reading streams.  This doesn't do anything, as the scene is assumed
// to be finished when the presenter has finished.
//

void CMPEG4PlayerApp::OnFinish(RESPONSE reason)
{
	TRACE_PRINT("End of session notification received.\n");

	// Remember the reason for finishing
	m_reason = reason;
}

void CMPEG4PlayerApp::OnError(NON_FATAL_ERROR nERROR)
{
   
#ifndef NETSCAPE_PLUGIN
//		GetView() -> PostMessage(WM_NON_FATAL_ERROR, (WPARAM)nERROR,(LPARAM)0);
		CMPEG4PlayerView *pView = GetView();
		assert(pView);
		pView -> PostMessage(WM_NON_FATAL_ERROR, (WPARAM)nERROR,(LPARAM)0);
#else 
		::PostMessage(((Presenter *)GetExecutive()->GetPresenter())->GetWindow(),
						WM_NON_FATAL_ERROR,
						(WPARAM)nERROR,
						(LPARAM)0);
#endif
}


//
// CMPEG4PlayerApp::OnSceneFinished
//
// This method is called when the presenter has indicated (by posting
// a message) that it has finished rendering the scene.  When this
// occurs, the executive is stopped, which completes the termination
// of the session.
//
// If the session terminated because of the end of the service, and
// looping enabled, then the file will be restarted.
//

void CMPEG4PlayerApp::OnSceneFinished()
{
	TRACE_PRINT("End of scene notification received.\n");

	// Stop the Executive running.
//	m_pExecutive->Stop();

	// Flag that the file is now closed
	m_smpFileOpen.V();

	// Stop the timer which is updating the display of the running time.
//	GetMainFrame()->KillTimer(m_timerID);
	if(GetMainFrame()!=NULL)
		GetMainFrame()->KillTimer(m_timerID);

	// Close the document.
//	GetDocument()->CloseDocument();

	// If the Loop Continuously option is selected,
	// send a message to the main thread to restart the file.
	// (this is done instead of calling OnFileRestart() directly,
	// because they are in different threads.
/*	if ((m_reason == END_OF_SERVICE) && (m_loopContinuously))
		GetMainFrame()->PostMessage(WM_COMMAND, ID_FILE_RESTART);
	else
		// Indicate on the status bar that the sequence has finished
	
		GetStatusBar()->SetWindowText("Finished.");*/
	//TO be viewed

} /* CMPEG4PlayerApp::OnSceneFinished */


/////////////////////////////////////////////////////////////////////////////
// CMPEG4PlayerApp commands

int CMPEG4PlayerApp::ExitInstance() 
{
	TRACE_PRINT("Entering CMPEG4PlayerApp::ExitInstance()\n");

	// Write the current frame rate to the .INI file
	if (GetExecutive() != NULL)
		GetApp()->WriteProfileFloat("Options", "FrameRate",
			((Presenter*)GetExecutive()->GetPresenter())->GetTargetFrameRate());

	// and also write the list of network addresses
	WriteProfileNetworkAddresses();

	// Delete the executive object (it has aready been stopped
	// by CloseDocument()).
	if (m_pExecutive)
		delete m_pExecutive;

	// Delete the thread info dialog box - if required
	if (pThreadInfoDlg != NULL)
		delete pThreadInfoDlg;
	EmptyThreadList();

	// Empty the list of entered network addresses
	for (int i = 0; i < m_networkAddresses.GetLength(); i++)
		free(m_networkAddresses[i]);

#ifdef SIMULATE_BIFS_DECODER
	extern char* globalFileName;

	if (globalFileName != NULL)
		free(globalFileName);
#endif

	delete m_playList;

	TRACE_PRINT("Leaving CMPEG4PlayerApp::ExitInstance()\n");
	_CrtCheckMemory();
	return CWinApp::ExitInstance();
} /* CMPEG4PlayerApp::ExitInstance */


/////////////////////////////////////////////////////////////////////////////

//
// GetDocument
//
// Return a pointer to the current document.
//

CMPEG4PlayerDoc* GetDocument()
{
	// Get the first document template
	POSITION pos = AfxGetApp()->GetFirstDocTemplatePosition();
	CDocTemplate* pDocTemplate = AfxGetApp()->GetNextDocTemplate(pos);

	// Get the first document in the template
	pos = pDocTemplate->GetFirstDocPosition();
	CDocument* pDoc = pDocTemplate->GetNextDoc(pos);

	// Return this document
	return (CMPEG4PlayerDoc*)pDoc;
} /* GetDocument */

//
// GetView
//
// Return a pointer to the current view.
//

CMPEG4PlayerView* GetView()
{
 CMPEG4PlayerDoc * pDoc = GetDocument(); 
 assert(pDoc);
 POSITION pos  = pDoc-> GetFirstViewPosition();

 return (CMPEG4PlayerView *)pDoc-> GetNextView(pos);
} /* GetView */


/////////////////////////////////////////////////////////////////////////////


/////////////////////////////////////////////////////////////////////////////

//
// These functions handle the changes to the frame rate, and the
// display of the tick marks which show which frame rate is current.
//

void CMPEG4PlayerApp::OnOptions25fps() 
{
	((Presenter*)GetExecutive()->GetPresenter())->SetTargetFrameRate(25);
}

void CMPEG4PlayerApp::OnUpdateOptions25fps(CCmdUI* pCmdUI) 
{
	pCmdUI->SetCheck((int)((Presenter*)GetExecutive()->GetPresenter())->GetTargetFrameRate() == 25);
}

void CMPEG4PlayerApp::OnOptions15fps() 
{
	((Presenter*)GetExecutive()->GetPresenter())->SetTargetFrameRate(15);
}

void CMPEG4PlayerApp::OnUpdateOptions15fps(CCmdUI* pCmdUI) 
{
	pCmdUI->SetCheck((int)((Presenter*)GetExecutive()->GetPresenter())->GetTargetFrameRate() == 15);
}

void CMPEG4PlayerApp::OnOptions10fps() 
{
	((Presenter*)GetExecutive()->GetPresenter())->SetTargetFrameRate(10);
}

void CMPEG4PlayerApp::OnUpdateOptions10fps(CCmdUI* pCmdUI) 
{
	pCmdUI->SetCheck((int)((Presenter*)GetExecutive()->GetPresenter())->GetTargetFrameRate() == 10);
}

void CMPEG4PlayerApp::OnOptions5fps() 
{
((Presenter*)GetExecutive()->GetPresenter())->SetTargetFrameRate(5);
}

void CMPEG4PlayerApp::OnUpdateOptions5fps(CCmdUI* pCmdUI) 
{
	pCmdUI->SetCheck((int)((Presenter*)GetExecutive()->GetPresenter())->GetTargetFrameRate() == 5);
}


/////////////////////////////////////////////////////////////////////////////

//
// These functions set and show the state of the m_loopContinuously flag.
//

void CMPEG4PlayerApp::OnOptionsLoopContinuously() 
{
	// Toggle the value
	m_loopContinuously = (m_loopContinuously) ? FALSE : TRUE;

	// Update this change in the .INI file
	AfxGetApp()->WriteProfileInt("Options", "LoopContinuously", m_loopContinuously);
}

void CMPEG4PlayerApp::OnUpdateOptionsLoopContinuously(CCmdUI* pCmdUI) 
{
	pCmdUI->SetCheck(m_loopContinuously);
}


/////////////////////////////////////////////////////////////////////////////

//
// These functions set and show the state of the m_clearOnEnd flag.
//

void CMPEG4PlayerApp::OnOptionsClearOnEnd() 
{
	// Toggle the value
	m_clearOnEnd = (m_clearOnEnd) ? FALSE : TRUE;

	// Update this change in the .INI file
	AfxGetApp()->WriteProfileInt("Options", "ClearOnEnd", m_clearOnEnd);
    
	// Tell the visual renderer about this change
	Presenter* pPres = (Presenter *) GetExecutive()->GetPresenter();
	pPres->GetVisualRenderer()->SetClearOnEndFlag(m_clearOnEnd);
}

void CMPEG4PlayerApp::OnUpdateOptionsClearOnEnd(CCmdUI* pCmdUI) 
{
	pCmdUI->SetCheck(m_clearOnEnd);
}


/////////////////////////////////////////////////////////////////////////////

float CMPEG4PlayerApp::GetProfileFloat(LPCTSTR lpszSection,
									   LPCTSTR lpszEntry,
									   float fDefault)
{
	CString str = GetProfileString(lpszSection, lpszEntry);
	if (str.IsEmpty())
		return fDefault;
	else
		return atof(str);
} /* CMPEG4PlayerApp::GetProfileFloat */


/////////////////////////////////////////////////////////////////////////////

BOOL CMPEG4PlayerApp::WriteProfileFloat(LPCTSTR lpszSection,
										LPCTSTR lpszEntry,
										float value)
{
	char str[20];
	sprintf(str, "%.4f", value);
	return WriteProfileString(lpszSection, lpszEntry, str);
} /* CMPEG4PlayerApp::WriteProfileFloat */


/////////////////////////////////////////////////////////////////////////////

void CMPEG4PlayerApp::GetProfileNetworkAddresses()
{
	CString str = GetProfileString("State", "NetworkAddresses");
	if (!str.IsEmpty())
	{
		char* localStr = strdup((LPCTSTR)str);
		char* pStr = localStr;

		// Break the string into separate string, using the commas
		// as separators
		while (pStr != NULL)
		{
			// NULL terminate the substring (if required)
			char* pEnd = strchr(pStr, ',');
			if (pEnd != NULL)
				*pEnd = 0;

			// Make a copy of the substring
			m_networkAddresses += strdup(pStr);

			// Move onto the next string
			if (pEnd != NULL)
				pStr = pEnd + 1;
			else
				pStr = pEnd;
		}
       
		free(localStr);
	}
} /* CMPEG4PlayerApp::GetProfileNetworkAddresses */


/////////////////////////////////////////////////////////////////////////////

void CMPEG4PlayerApp::WriteProfileNetworkAddresses()
{
	// If there are no network addresses, write an empty string
	if (m_networkAddresses.GetLength() == 0)
		WriteProfileString("State", "NetworkAddresses", "");

	// Otherwise, combine all of the network addresses into a single,
	// comma separated list in order to store it in the application's
	// profile.
	else
	{
		int requiredLength = 0;
		for (int i = 0; i < m_networkAddresses.GetLength(); i++)
			requiredLength += strlen(m_networkAddresses[i]) + 1;

		char* combinedString = new char[requiredLength];
		strcpy(combinedString, m_networkAddresses[0]);
		for (i = 1; i < m_networkAddresses.GetLength(); i++)
		{
			strcat(combinedString, ",");
			strcat(combinedString, m_networkAddresses[i]);
		}

		WriteProfileString("State", "NetworkAddresses", combinedString);
		delete combinedString;
	}
} /* CMPEG4PlayerApp::GetProfileNetworkAddresses */


/////////////////////////////////////////////////////////////////////////////

//
// CMPEG4PlayerApp::OnViewThreadInfo
//
// Show information about the threads that are running.
//

void CMPEG4PlayerApp::OnViewThreadInfo() 
{
	if (pThreadInfoDlg == NULL)
		pThreadInfoDlg = new CThreadInfoDlg;
	else
		pThreadInfoDlg->ShowWindow(SW_SHOW);
} /* CMPEG4PlayerApp::OnViewThreadInfo */


/////////////////////////////////////////////////////////////////////////////

void CMPEG4PlayerApp::OnViewWorldInfo() 
{
	CViewWorldInfo dialog;
	dialog.DoModal();
} /* CMPEG4PlayerApp::OnViewWorldInfo */


/////////////////////////////////////////////////////////////////////////////


void CMPEG4PlayerApp::OnFileOpen() 
{
	m_playList->showDialog();
}

void CMPEG4PlayerApp::OnGoBack() 
{
	// TODO: Add your command handler code here
	// Open the previous file of the playlist if possible
	if (m_playList->hasPreviousTrack())
	{
		GetView()->GetDocument()->CloseDocument();
		OpenDocumentFile(m_playList->getPreviousTrack() ); 	
	}
}

void CMPEG4PlayerApp::OnGoForward() 
{
	// TODO: Add your command handler code here
	// Open the next file of the playlist if possible
	if (m_playList->hasNextTrack())
	{
		GetView()->GetDocument()->CloseDocument();
		OpenDocumentFile(m_playList->getNextTrack() ); 
	}
	
}

void CMPEG4PlayerApp::OnUpdateGoBack(CCmdUI* pCmdUI) 
{
	// TODO: Add your command update UI handler code here
	if (m_playList && m_playList->hasPreviousTrack())
		pCmdUI->Enable(true);
	else
		pCmdUI->Enable(false);
}

void CMPEG4PlayerApp::OnUpdateGoForward(CCmdUI* pCmdUI) 
{
	// TODO: Add your command update UI handler code here
	if (m_playList && m_playList->hasNextTrack())
		pCmdUI->Enable(true);
	else
		pCmdUI->Enable(false);
	
}

void CMPEG4PlayerApp::OnGoReplay() 
{
	// TODO: Add your command handler code here
	if ( *(m_playList->getCurrentTrack()) != '\0') // playlist not empty
	{
		//CloseAllDocuments()?
		GetView()->GetDocument()->CloseDocument();
		OpenDocumentFile(m_playList->getCurrentTrack());
		GetView()->GetDocument()->CloseDocument();
		GetView()->GetDocument()->OpenDocument();
	}
}

void CMPEG4PlayerApp::OnUpdateGoReplay(CCmdUI* pCmdUI) 
{
	// TODO: Add your command update UI handler code here
	if (m_playList && *(m_playList->getCurrentTrack()) != '\0')
		pCmdUI->Enable(true);
	else
		pCmdUI->Enable(false);
	
}

void CMPEG4PlayerApp::OnFileOpenfile() 
{
	// TODO: Add your command handler code here	
	char BASED_CODE szFilter[] = "Mpeg4 file formats (*.mp4;*.trif)|*.mp4;*.trif|All files (*.*)|*.*||";
	CFileDialog fd(TRUE,NULL,NULL,OFN_HIDEREADONLY | OFN_OVERWRITEPROMPT,szFilter);

	fd.DoModal();
	OpenDocumentFile(fd.GetPathName( ) ); 
	
}

// SISF - Merge - PA
/////////////////////////////////////////////////////////////////////////////

void CMPEG4PlayerApp::OnFileMpeg2tsOpenfile() 
{
	// TODO: Add your command handler code here
	if (!bDecoderIsPlaying)
	{	
	 bSourceEnable = FALSE ;
	 if ((::GetOpenFileName (&MPEG_SourceStream)) != 0 ) 
	   {  
		 bSourceEnable = TRUE ; 

		 //createImgWin() ;

		 strcpy(infoDescr.localFileName, streamFilePath) ;
		 strcpy(infoDescr.serverIPAddress, "") ;
		 infoDescr.serverIPPort	= 0 ;
		 infoDescr.Protocol = 0 ;
		 infoDescr.hWndVideo = theApp.m_pMainWnd->m_hWnd ;

	 }
	}	
	else 
	 m_pMainWnd->MessageBox("Please,\nwait the end of stream", "Warning", MB_ICONINFORMATION | MB_OK ) ;
}

void CMPEG4PlayerApp::OnMpeg2Play() 
{
	// TODO: Add your command handler code here	

	HRESULT M2Dval ;
    POINT         point ;												// Posizione left top corner finestra di visualizzazione		

#ifdef SIA
	static HANDLE threadHandle;
	DWORD id;				   
#endif

	if (!bDecoderIsPlaying && bSourceEnable) 	
	{
		 M2DecObj.M2DecSetOverlay(bUseOverlay) ;

		 M2Dval = openDefaultProgram() ;
		 if (M2Dval != M2D_OK)
			displayError(M2Dval) ;

		m_pMainWnd->GetClientRect(&ImgDestRect);

		point.x = point.y = 0 ;												// Centra l'immagine nell'area visualizzabile
		m_pMainWnd->ClientToScreen(&point) ;

		displayDescr.dx     = ImgDestRect.left + point.x ;
		displayDescr.dy     = ImgDestRect.top + point.y ;
		displayDescr.hSize  = ImgDestRect.right - ImgDestRect.left ;
		displayDescr.vSize  = ImgDestRect.bottom - ImgDestRect.top ;
		displayDescr.chromaKey = 0x00ff00ff ;

		M2Dval = M2DecObj.M2DecSetDisplay(&displayDescr) ;
		if (M2Dval != M2D_OK)
			displayError(M2Dval) ;

		M2Dval = M2DecObj.M2DecRepaint() ;

		//m_pMainWnd->ValidateRect(NULL) ;

		//@ Dirty trick: assume that dataPID != -1 means MPEG4 
		//@ You'll have to dig deeply into PMT to see if it's really Mpeg4
		//@
		//@Now the URL contains the infoDescr structure immediately followed
		//@by the PMT pid.
		//@The length of the URL is 7+infoDescr.length+4
		if(programInfo.dataPID != -1)
		{//@There is mp4
			startedMPEG4inMPEG2 = TRUE;
			char url[4096];
			char* p;
			memcpy(url,"d-m2ts:",7); // 7 byte
			p = url + 7;
			memcpy(p,&infoDescr,infoDescr.length);
			p += infoDescr.length;
			memcpy(p,&(programInfo.PMT_PID),4); //PMT_PID is a 4-byte integer
			p += 4;
			memcpy(p,&displayDescr,displayDescr.length); //displayDescr structure
#ifndef SIA
			M2Dval = M2DecObj.M2DecClose(); //Close here the decoder.
#endif
#ifdef SIA
			//MessageBox(NULL, "M2DecoderPlus close!", "OnMpeg2Play", MB_OK);
#endif
#ifndef SIA
			if (M2Dval != M2D_OK)
				displayError(M2Dval) ;
#endif
			GetApp()->GetExecutive()->Start(url); 
		}
		else
		{//no mpeg4
			startedMPEG4inMPEG2 = FALSE;

#ifdef SIA
			 // BB: Has been used to start JVM and configure SF. Is now done differently.

//			if((threadHandle = ::CreateThread( NULL, 0, (LPTHREAD_START_ROUTINE)Myfunc, (LPVOID)NULL, 0, &id)) == NULL )
//	{
//		assert(1==0);
//	}  
#endif
			if (programInfo.audioPID != -1)
				{
					M2Dval = M2DecObj.M2DecStartAudioDecoder(programInfo.audioPID) ;
					if (M2Dval != M2D_OK)
						displayError(M2Dval) ;
				}

			if (programInfo.videoPID != -1)
				{
					M2Dval = M2DecObj.M2DecStartVideoDecoder(programInfo.videoPID) ;
					if (M2Dval != M2D_OK)
						displayError(M2Dval) ;
				}

			M2Dval = M2DecObj.M2DecPlay(programInfo.PCR_PID) ;
			bDecoderIsPlaying = TRUE;	
			if (M2Dval != M2D_OK)
					displayError(M2Dval) ;

			// BB: For test purposes only, directly call section filter engine to test SF.
/*			CstartFilter
            ( 0         // index, the number of this section filter
            , 100       // id,    uniquely identifying this filter action
            , l_cpp     // the listener to receive the events indicating a new section has arrived
            , 0         // PID,   in the case of the PAT 0
            , NULL   // mask,  which bits to check
            , NULL  // value, the value checked bits should have
			, NULL, NULL , 0 ,0
            );*/
		}

    }	
}
void CMPEG4PlayerApp::OnFileMpeg2tsOpennetwork() 
{
	// TODO: Add your command handler code here

	if (!bDecoderIsPlaying)
	{
		bSourceEnable = FALSE ;
		//if (!CreateNetConfigDlg (hInst, hWnd))
		//	break ;

		bSourceEnable = TRUE ; 

		M2DecObj.M2DecSetOverlay(bUseOverlay) ;

		//createImgWin() ;

		strcpy(infoDescr.localFileName, "") ;
		//strcpy(infoDescr.serverIPAddress, "127.0.0.1" ) ; // not used
		strcpy(infoDescr.serverIPAddress, "163.162.132.77" ) ;
		infoDescr.serverIPPort	= 25700 ;
		infoDescr.Protocol = 0 ;
		infoDescr.hWndVideo = m_pMainWnd->m_hWnd ;

/*		M2Dval = openDefaultProgram() ;
		if (M2Dval != M2D_OK)
			displayError(M2Dval) ;
*/
	}
	else 
		m_pMainWnd->MessageBox("Please,\nwait the end of stream", "Warning", MB_ICONINFORMATION | MB_OK ) ;
	
}

void  InitFileNameStruct (void)
 { 
   unsigned int  j ;


   memset (&OpenFile_name, 0, sizeof(OpenFile_name)) ;
   memset (&OpenFile_title, 0, sizeof(OpenFile_title)) ;
   memset (&Dir_name, 0, sizeof(Dir_name)) ;

   lstrcpy (OpenFilter, "MPEG server configuration files|*.ini|All files (*.*)|*.*||") ;
   for (j = 0; j < sizeof(OpenFilter); j++)  if (OpenFilter[j] == '|')  OpenFilter[j] = '\0' ;

   lstrcpy (Dir_name, "e:\\Movies\\") ;
   memset (&MPEG_SourceStream, 0, sizeof(OPENFILENAME)) ;

   MPEG_SourceStream.lStructSize     = sizeof(OPENFILENAME) ;
   MPEG_SourceStream.hwndOwner       = theApp.m_pMainWnd->m_hWnd; //hWinMain ;
   MPEG_SourceStream.lpstrFilter     = (char FAR *) &OpenFilter ;
   MPEG_SourceStream.nFilterIndex    = 1 ;
   MPEG_SourceStream.lpstrFile       = (char FAR *) &streamFilePath ;
   MPEG_SourceStream.nMaxFile        = sizeof(streamFilePath) ;
   MPEG_SourceStream.lpstrFileTitle  = (char FAR *) &OpenFile_title ;
   MPEG_SourceStream.nMaxFileTitle   = sizeof(OpenFile_title) ;
   MPEG_SourceStream.lpstrInitialDir = (char FAR *) &Dir_name ;
   MPEG_SourceStream.Flags           = OFN_PATHMUSTEXIST | OFN_FILEMUSTEXIST | OFN_SHAREAWARE ;

   return ;
 }




/* 
----------------------------------------------------------------------------------- 
	Funzione :  void  displayError ( HRESULT errorMsg)
	Note     :  Visualizza i messaggi di errore
    Input    :	Valore del messaggio di errore
	Output   :  /
----------------------------------------------------------------------------------- 
*/


void  displayError (HRESULT errorMsg) 
{
	char  msg[MAX_CHAR] ;


    M2DecObj.M2DecGetError("D:\\depetris\\M2DecoderPlus\\error.txt", errorMsg, msg) ;

	MessageBox(NULL, msg, "Error", MB_OK | MB_ICONERROR) ;
}


void cleanup(PM2DECFILTERINFO pFI)
{
	memset(pFI, 0, sizeof(M2DECFILTERINFO)) ;
}


void look_for_PAT(PM2DECFILTERINFO pFI)
{   
	cleanup(pFI);

	pFI->length = sizeof(M2DECFILTERINFO) ;
	pFI->PID = PAT_PID ;

	pFI->posMask[0]  = 0xff;			
	pFI->posValue[0] = 0x00;				// table id = 0

	pFI->posMask[5]  = 0x01;			
	pFI->posValue[5] = 0x01;				// currently applicable
	
	pFI->posMask[6]  = 0xff;			
	pFI->posValue[6] = 0x00;				// section number = 0

	pFI->posLen = 7 ;
	pFI->negLen = 0 ;
}


void look_for_PMT(int PID, PM2DECFILTERINFO pFI)
{   
	cleanup(pFI);

	pFI->length = sizeof(M2DECFILTERINFO) ;
	pFI->PID = PID ;

	pFI->posMask[0]  = 0xff;			
	pFI->posValue[0] = 0x02;				// table id = 2

	pFI->posMask[5]  = 0x01;			
	pFI->posValue[5] = 0x01;				// currently applicable
	
	pFI->posMask[6]  = 0xff;			
	pFI->posValue[6] = 0x00;				// section number = 0

	pFI->posLen = 7 ;
	pFI->negLen = 0 ;
}

/*
 *  parsePAT: return the PID of the PMT of the first program in the PAT (see MPEG-2)
 */
void parsePAT(PM2DECSECTIONDATA pSection, PPROGRAMINFO pProgInfo)
{
    unsigned char * section ;
	int program_num, PID, i;


	section = &(pSection->data[0]) ;

	pProgInfo->audioPID = -1 ;
	pProgInfo->videoPID = -1 ;
	pProgInfo->dataPID  = -1 ;
	pProgInfo->PCR_PID  = -1 ;
	pProgInfo->PMT_PID  = -1 ;

	i = 0;
	do
	{
		program_num = ((section[8+i] << 8) | (section[9+i]));			// get the program number
		pProgInfo->programNumber = program_num ;
		PID = ((section[10+i] << 8) | (section[11+i])) & PID_MASK ;		// get the PMT pid
		i += 4;															// read next 4 bytes
	} while (program_num < 1);

	pProgInfo->PMT_PID = PID;
}


void parsePMT(PM2DECSECTIONDATA pSection, PPROGRAMINFO pProgInfo)
{
	unsigned char * section ;
	int				section_length ;
	int				PID ;
    int				i, length ;
	unsigned char	stream_type;


	section = &(pSection->data[0]) ;
	section_length = pSection->sectionLength ;

	PID = ((section[8] << 8) | (section[9])) & PID_MASK ;			// get the new PCR pid	
	pProgInfo->PCR_PID = PID ;		

	length = ((section[10] << 8) | (section[11])) & 0x0FFF ;			// get the length of program info loop

	i = 12 + length;
	while (i < (section_length - 4))
	{
		stream_type = section[i];										// get the stream_type
		PID = ((section[i+1] << 8) | (section[i+2])) & PID_MASK ;		// get the stream pid
		length = ((section[i+3] << 8) | (section[i+4])) & 0x0FFF ;    // get the ES info length
		
		switch (stream_type)
		{
		case MPEG1_Video_Stream:
		case MPEG2_Video_Stream:
			pProgInfo->videoPID = PID;
			break;
			
		case MPEG1_Audio_Stream:
		case MPEG2_Audio_Stream: 
			pProgInfo->audioPID = PID;
			break;

		case Private_Data_Stream: 
			pProgInfo->dataPID = PID;
			break;
		}
		
		i += (5+length);												// increment pointer
	}
}


HRESULT openDefaultProgram()
{
	HRESULT M2Dval ;


	M2Dval = M2DecObj.M2DecInit(&infoDescr) ;
	switch(M2Dval){
		case M2D_OK:
			{
				M2DOk = true;
				break;
			}
		case M2D_ERROR:
			{
				MessageBox(NULL, "M2D_ERROR: errore generico", "Start section filter", MB_OK);
				return M2Dval;
			}
		case M2D_INVALIDPARAMETER:
			{
				MessageBox(NULL, "M2D_INVALIDPARAMETER: valori dei parametri passati al metodo non corretti", "Start section filter", MB_OK);
				return M2Dval;
			}
		case M2D_PLAYEVENTSETUP:
			{
				MessageBox(NULL, "M2D_PLAYEVENTSETUP: inizializzazione eventi di sincronizzazione del M2DecoderPlus", "Start section filter", MB_OK);
				return M2Dval;
			}
		case M2D_DDCREATION:
			{
				MessageBox(NULL, "M2D_DDCREATION: creazione istanza della classe DirectDraw", "Start section filter", MB_OK);
				return M2Dval;
			}
		case M2D_DDOVRNOYUVTORGBCONV:
			{
				MessageBox(NULL, "M2D_DDOVRNOYUVTORGBCONV: la scheda video non supporta la conversione YUV-RGB in hardware utilizzando una superficie overlay", "Start section filter", MB_OK);
				return M2Dval;
			}
		case M2D_DEMUXINIT:
			{
				MessageBox(NULL, "M2D_DEMUXINIT: inizializzazione del thread Demux", "Start section filter", MB_OK);
				return M2Dval;
			}
		case M2D_FILENOTFOUND:
			{
				MessageBox(NULL, "M2D_FILENOTFOUND: file inesistente", "Start section filter", MB_OK);
				return M2Dval;
			}
		case M2D_NOTCLOSED:
			{
				MessageBox(NULL, "M2D_NOTCLOSED: Chiamata al metodo M2DecInit prima di aver terminato il decoder tramite M2DecClose", "Start section filter", MB_OK);
				return M2Dval;
			}
		default: 
			{
				MessageBox(NULL, "Unknown return value", "Start section filter", MB_OK);
				return M2Dval;
			}
	}
	if (M2Dval != M2D_OK)
		return(M2Dval) ;


	 look_for_PAT(&filterInfo) ;
	 M2Dval = M2DecObj.M2DecStartSectionFilter(&filterInfo) ;
	 if (M2Dval != M2D_OK)
		return(M2Dval) ;

	 do 
	 {  Sleep(5) ;
		M2Dval = M2DecObj.M2DecGetNewSection(&sectionData) ;
	 } while (M2Dval != M2D_OK) ;

	 parsePAT(&sectionData, &programInfo) ;
 
	 M2Dval = M2DecObj.M2DecStopSectionFilter() ;
	 if (M2Dval != M2D_OK)
		return(M2Dval) ;

	 look_for_PMT(programInfo.PMT_PID, &filterInfo) ;
	 M2Dval = M2DecObj.M2DecStartSectionFilter(&filterInfo) ;
	 if (M2Dval != M2D_OK)
		return(M2Dval) ;

	 do 
	 {  Sleep(5) ;
		M2Dval = M2DecObj.M2DecGetNewSection(&sectionData) ;
	 } while (M2Dval != M2D_OK) ;

	 parsePMT(&sectionData, &programInfo) ;
 
	 M2Dval = M2DecObj.M2DecStopSectionFilter() ;
	 if (M2Dval != M2D_OK)
		return(M2Dval) ;

	 return(M2D_OK) ;
}

void CMPEG4PlayerApp::OnFileMpeg2tsClose() 
{
	HRESULT M2Dval ;
	// TODO: Add your command handler code here
	if (bDecoderIsPlaying || bSourceEnable)
	{
		M2Dval = M2DecObj.M2DecClose() ;
		if (M2Dval != M2D_OK)
			displayError(M2Dval) ;
		bDecoderIsPlaying = FALSE ;
	 }

}