///////////////////////////////////////////////////////////////////////////
//                                                                       //
// Copyright (c) 1997 ISO/IEC All Rights Reserved                        //
//                                                                       //
// This software module was developed by                                 //
//                                                                       //
// CSELT (Centro Studi e Laboratori Telecomunicazioni S.p.A)             //
//                                                                       //
// in the course of development of the MPEG-4 standard. This software    //
// module is an implementation of a part of one or more MPEG-4 tools as  //
// specified by the MPEG-4 standard.                                     //
//                                                                       //
// The copyright of this software belongs to ISO/IEC. ISO/IEC gives use  //
// of the MPEG-4 standard free license to use this  software module or   //
// modifications thereof for hardware or software products claiming      //
// conformance to the MPEG-4 standard.                                   //
//                                                                       //
// Those intending to use this software module in hardware or software   //
// products are advised that use may infringe existing  patents. The     //
// original developers of this software module and their companies, the  //
// subsequent editors and their companies, and ISO/IEC have no liability //
// and ISO/IEC have no liability for use of this software module or      //
// modification thereof in an implementation.                            //
//                                                                       //
///////////////////////////////////////////////////////////////////////////
//
// File: Remote.cpp
//
// Authors: Nuno Delgado, CSELT
//
// Description:
//
///////////////////////////////////////////////////////////////////////////


#include "defs.h"
#include "Remote.h"

// Prevent inclusion of winsock.h in windows.h
#ifndef _WINSOCKAPI_
#define _WINSOCKAPI_
#endif

#include "Winsock2.h"


/****************************************************************************************

****************************************************************************************/
/*RemoteService::RemoteService()
{
}
*/

/****************************************************************************************

****************************************************************************************/
RemoteService::RemoteService(ALManager* pAL) : Service(pAL)
{
	m_pALMan = pAL;
}



/****************************************************************************************

****************************************************************************************/
void RemoteService::Connect(LPCSTR szIpAddress)   
{
	int response;
	unsigned int ServiceId;
	unsigned int ClientDescriptor;

	// Fill the PeerAddress
	strcpy(mPeerAddress, szIpAddress);

	// Instantiate the DaiObject
	pDai = (Dai*) new Dai(this);


	// Initialization
	if(!pDai->Initialization())
	{
		m_pALMan -> OnConnect(CANNOT_OPEN_SOCKET,"");  //ADDEDGL
		return;
	}

	// Create the session
	pDai->DA_SessionCreate(mSessionId,response);


	// Attach to the remote peer
	ServiceId = 3366;
	ClientDescriptor = 33;
	if(!pDai->DA_SessionAttachRequest(mSessionId,mPeerAddress,ServiceId,ClientDescriptor))
	{
		m_pALMan -> OnConnect(IP_ADDRESS_NOT_FOUND,"");
		return;
	}
	
	m_pALMan -> OnConnect(CONNECTION_SUCCESS,"");
}



/****************************************************************************************

****************************************************************************************/
void RemoteService::Close()
{
	int Response;


	pDai->DA_SessionDestroy(mSessionId,0,Response);
	pDai->ReleaseResources();
	delete pDai;

	m_pALMan -> OnFinish(END_OF_SERVICE);
}



/****************************************************************************************

****************************************************************************************/
void RemoteService::OpenChannels(int nNumberOfChannels, const int* pESidList)
{
	DATA* pData;

	pData = new DATA;
	pData->NumOfElements = nNumberOfChannels;
	pData->pBuffer = new int[nNumberOfChannels];

	memcpy(pData->pBuffer,pESidList,(pData->NumOfElements)*4);
	pDai->DA_ChannelAddRequest(mSessionId,mPeerAddress,pData);

	delete pData->pBuffer;
	delete pData;
}




/****************************************************************************************

****************************************************************************************/
void RemoteService::OnChannelOpen(HANDLE TransMux, SMT& rStreamMapTable)
{
	Service::OnChannelOpen(TransMux, rStreamMapTable);
}



/****************************************************************************************

****************************************************************************************/
void RemoteService::OnDataReceived(HANDLE TransMux, LPBYTE pBuffer, int NumBytes)
{
	Service::OnDataReceived(TransMux,pBuffer,NumBytes);
}



/****************************************************************************************

****************************************************************************************/
void RemoteService::ChannelsReady(int nNumberOfChannels,const  int* pChannelAssociationTagList)
{
	DATA UuData;
	UuData.NumOfElements = nNumberOfChannels;
	UuData.pBuffer = new int[nNumberOfChannels];
	
	
	for(int j = 0; j < nNumberOfChannels; j++)
		*(UuData.pBuffer + j) = *(pChannelAssociationTagList + j);
	
	pDai->DA_ChannelReadyRequest(mSessionId,mPeerAddress,&UuData);

	delete UuData.pBuffer;
}
