///////////////////////////////////////////////////////////////////////////
//                                                                       //
// Copyright (c) 1997 ISO/IEC All Rights Reserved                        //
//                                                                       //
// This software module was developed by                                 //
//                                                                       //
// CSELT (Centro Studi e Laboratori Telecomunicazioni S.p.A)             //
//                                                                       //
// in the course of development of the MPEG-4 standard. This software    //
// module is an implementation of a part of one or more MPEG-4 tools as  //
// specified by the MPEG-4 standard.                                     //
//                                                                       //
// The copyright of this software belongs to ISO/IEC. ISO/IEC gives use  //
// of the MPEG-4 standard free license to use this  software module or   //
// modifications thereof for hardware or software products claiming      //
// conformance to the MPEG-4 standard.                                   //
//                                                                       //
// Those intending to use this software module in hardware or software   //
// products are advised that use may infringe existing  patents. The     //
// original developers of this software module and their companies, the  //
// subsequent editors and their companies, and ISO/IEC have no liability //
// and ISO/IEC have no liability for use of this software module or      //
// modification thereof in an implementation.                            //
//                                                                       //
///////////////////////////////////////////////////////////////////////////
//
// File: TransMuxChannel.cpp
//
// Authors: Nuno Delgado, CSELT
//
// Description:
//
///////////////////////////////////////////////////////////////////////////


#include "defs.h"
#include "TransMuxChannel.h"






/***************************************************************************************

****************************************************************************************/
TransMuxChannel::TransMuxChannel()
{
}


/***************************************************************************************

****************************************************************************************/
TransMuxChannel::TransMuxChannel(Dai* pDai, DaiSession* pSession, CDmifPeer* pPeer, 
					 int BuffSize, unsigned int Tag, int Type)
{
	TransTagAlreadySentToClient = FALSE;

	// Create the event tha will be associated with this socket
	m_hEvent = (WSAEVENT)CreateEvent(NULL, FALSE, FALSE, NULL);

	m_pDai = pDai;
	m_pSession = pSession;
	m_pPeer = pPeer;
	m_pBuffer = new char[BuffSize];
	m_BufferSize = BuffSize;
	m_TransMuxAssocTag = Tag;
	mTypeOfChannel = Type;
	pAuxx = m_pBuffer; 
	PacketHeaderAlreadyRead = FALSE;
	FirstByteAlreadyAvailable = FALSE;
}



/***************************************************************************************

****************************************************************************************/
TransMuxChannel::~TransMuxChannel()
{

	CloseHandle((HANDLE)m_hEvent);
	delete m_pBuffer;
}




/***************************************************************************************
	
	ReceiveData

	This method receives the data that arrived at the socket.
	If the socket is a TCP socket, the two first bytes of the packet contain  the 
	length of the complete packet and this packet will be reagruped before delivering it
	to the upper layer.
	If the socket is UDP, it is received atomicaly and will be passed to the upper layer
	
****************************************************************************************/
void TransMuxChannel::ReceiveData()
{
	int BytesReceived;
	int Flags = 0;
	Conversion_Union conv;


	if(mTypeOfChannel == TCP_CHANNEL)
	{
		if(!PacketHeaderAlreadyRead)   // lets read the first 2 bytes with the packet size
		{
			if(FirstByteAlreadyAvailable)
			{
				BytesReceived = recv(m_hSocket,pAuxx+1,1,Flags);
				if(BytesReceived == SOCKET_ERROR)
				{
					m_pDai->IdentifyError("TransMuxChannel::ReceiveData() - recv:",WSAGetLastError());
					return;
				}

				FirstByteAlreadyAvailable = FALSE;
			}
			else
			{
				BytesReceived = recv(m_hSocket,pAuxx,2,Flags);
				if(BytesReceived == SOCKET_ERROR)
				{
					m_pDai->IdentifyError("TransMuxChannel::ReceiveData() - recv:",WSAGetLastError());
					return;
				}
			
				if(BytesReceived < 2)
				{
					FirstByteAlreadyAvailable = TRUE;
					return;
				}	
			}
			memcpy(conv.data,pAuxx,2);
			PacketSize = conv.ushort_number;
			SpaceRemainingForPacket = PacketSize;
			PacketHeaderAlreadyRead = TRUE;

		}
		else
		{
			BytesReceived = recv(m_hSocket,pAuxx,SpaceRemainingForPacket,Flags);
			if(BytesReceived == SOCKET_ERROR)
			{	
				m_pDai->IdentifyError("TransMuxChannel::ReceiveData() - recv:",WSAGetLastError());
				return;
			}
			else
			{
				SpaceRemainingForPacket = SpaceRemainingForPacket - BytesReceived;
				pAuxx = pAuxx + BytesReceived;
				if(SpaceRemainingForPacket == 0)  // we have a complete packet
				{
					m_pDai->DataAvailableForClient(m_hSocket, m_pBuffer,PacketSize);
					pAuxx = m_pBuffer;
					PacketHeaderAlreadyRead = FALSE;
				}
			}
		}
	}
	else   // Is an UDP packet
	{

		MessageBox(GetActiveWindow(),"UDP packet not expected on this demo","",MB_OK);
		BytesReceived = recv(m_hSocket,m_pBuffer,m_BufferSize,Flags);
		if(BytesReceived == SOCKET_ERROR)
		{	
			m_pDai->IdentifyError("TransMuxChannel::ReceiveData() - recv:",WSAGetLastError());
			return;
		}
		m_pDai->DataAvailableForClient(m_hSocket, m_pBuffer,BytesReceived);
	}
}


/***************************************************************************************

****************************************************************************************/
BOOL TransMuxChannel::operator==(const unsigned int TagValue)
{
	return m_TransMuxAssocTag == TagValue;
}



/***************************************************************************************

****************************************************************************************/
BOOL TransMuxChannel::operator==(const WSAEVENT EventValue)
{
	return m_hEvent == EventValue;
}




/***************************************************************************************
	
	GetChannelProperties

	This method writes in the buffer pointed by pBuff the TransMux Association Tag, the
	of this TransMux (TCP or UDP) and the port number associated with it

****************************************************************************************/
void TransMuxChannel::GetChannelProperties(char* pBuff)
{
	Conversion_Union conv;

	conv.int_number = m_TransMuxAssocTag;
	memcpy(pBuff,conv.data,4);
	pBuff += 4;
	conv.int_number = mTypeOfChannel;
	memcpy(pBuff,conv.data,4);
	pBuff += 4;
	conv.int_number = m_PortNumber;
	memcpy(pBuff,conv.data,4);
}



/***************************************************************************************

****************************************************************************************/
void TransMuxChannel::UpdatePortNumber(unsigned short Port)
{
	m_PortNumber = Port;
}



/***************************************************************************************

	ConnectTransChannelToServer

	This method connects the TransMux object to a remote TransMux object.
	In the case of a TCP TransMux, no confirm is necessary because TCP is connection
	oriented. However, in the case of UDP, is sent a packet through the Dmif control
	channel to guarantee that the connection was established.
	Then, after establishing the connection will work asynchronously

****************************************************************************************/
BOOL TransMuxChannel::ConnectTransChannelToServer(struct sockaddr* RemoteSockAddr, 
								                  int RemoteSockAddrLength,
								                  LPWSAPROTOCOL_INFO pProtocolInfo)
{

	m_hSocket = WSASocket(FROM_PROTOCOL_INFO, FROM_PROTOCOL_INFO, 
			            FROM_PROTOCOL_INFO,pProtocolInfo,0,WSA_FLAG_OVERLAPPED);
	
	if(m_hSocket == INVALID_SOCKET)
	{
		m_pDai->IdentifyError("ConnectTransChannelToServer-WSASocket: ",WSAGetLastError());
		return FALSE;
	}

	// Connect the socket
	if(WSAConnect(m_hSocket,(struct sockaddr*)RemoteSockAddr,
	             RemoteSockAddrLength,NULL,NULL,NULL,NULL) == SOCKET_ERROR)
	{
		m_pDai->IdentifyError("EstablishConnectionsClientSide - WSAConnect: ",
			                                                 WSAGetLastError());
		return FALSE;
	}

	// If its a UDP socket, after this connect we have to send to the server to 
	// port number that the OS gave to this socket
	if(mTypeOfChannel == UDP_CHANNEL)
		SendUdpPortToServer(m_pPeer,m_TransMuxAssocTag,m_hSocket);
		
	// Put the sockets in asynchronous mode
	if(WSAEventSelect(m_hSocket,(WSAEVENT)m_hEvent, FD_READ) == SOCKET_ERROR)
	{
		m_pDai->IdentifyError("ConnectTransChannelToServer - WSAEventSelect",WSAGetLastError());
		return FALSE;
	}

	return TRUE;
}


/***************************************************************************************

****************************************************************************************/
WSAEVENT TransMuxChannel::GetTransMuxEvent()
{
	return (WSAEVENT)m_hEvent;
}




/***************************************************************************************

	SendUdpPortToServer

	It is used to send to the server the port number attributed to the local UDP socket. 
	This will allow the server to make a "connect" to this socket. Besides, this is a 
	way to confirm to the server that this Udp TransMux was created


****************************************************************************************/
void TransMuxChannel::SendUdpPortToServer(CDmifPeer* pPeer, unsigned int TransMuxTag, 
										                              SOCKET hSocket)
{
	struct sockaddr LocalAddr;
	int AddrLength = sizeof (struct sockaddr);
	struct sockaddr_in* LocalIpAddr;
	unsigned short Port;
	char BuffAux[CONTROL_MSG_SIZE];
	Conversion_Union conv;
	DWORD SentBytes;


	// Lets get the local port attributed to this UDP socket
	if(getsockname(hSocket,&LocalAddr, &AddrLength) == SOCKET_ERROR)
	{
		m_pDai->IdentifyError("SendUdpPortToServer - getsockname:", WSAGetLastError());
		return;
	}

	LocalIpAddr = (struct sockaddr_in*)&LocalAddr;
	Port = LocalIpAddr->sin_port;
	
	// Lets fill a message containig: UDP_PORT_CONFIRMATION + TranMuxTag + PortNumber
	conv.int_number = UDP_PORT_CONFIRMATION;
	memcpy(BuffAux,conv.data,4);
	conv.int_number = 2;  // Means that are 2 elements in the packet: TransMuxtag+PortNumber
	memcpy(BuffAux+4,conv.data,4);
	conv.int_number = TransMuxTag;
	memcpy(BuffAux+8,conv.data,4);
	conv.ushort_number = Port;
	memcpy(BuffAux+12,conv.data,2);

	m_pDai->MySend(pPeer->m_hDmifCtrlSock,BuffAux,CONTROL_MSG_SIZE,SentBytes);
}



/***************************************************************************************

	VerifySessionAndPeer

	Verifies if the TransMuxChannel object belongs to the session pSession and to the 
	CDmifPeer pPeer

****************************************************************************************/
BOOL TransMuxChannel::VerifySessionAndPeer(DaiSession* pSession, CDmifPeer* pPeer)
{
	if((m_pSession == pSession) && (m_pPeer == pPeer))
		return TRUE;
	else
		return FALSE;
}



/***************************************************************************************

	AcceptConnection

	Accepts the connection from a remote TransMuxChannel (in the case of a TCP/IP 
	connection) and updates the Table with the new socket handle after the connection 
	establishment. In the case of UDP nothing is done.

****************************************************************************************/
BOOL TransMuxChannel::AcceptConnection(int(*Table)[6], int NumElements)
{
	if(mTypeOfChannel == TCP_CHANNEL)
	{
		struct sockaddr address;
		int addr_size = sizeof(struct sockaddr);

		m_hSocket = WSAAccept(m_hListenSocket, &address, &addr_size, NULL, NULL);
		if(m_hSocket == INVALID_SOCKET)
		{
			closesocket(m_hListenSocket);
			return FALSE;
		}

		//Actualize the socket handle after accepting connection
		for(int count = 0; count < NumElements ; count ++)
			if(Table[count][5] == (int)m_hListenSocket)
				Table[count][5] = (int)m_hSocket;
	}
	
	// Note: For UDP sockets nothing is done

	return TRUE;
}