/*-------------------------------------------------------------------------
Project Im1Player
INTERNATIONAL ORGANIZATION FOR STANDARDIZATION
ISO/IEC JTC1/SC29/WG11

SUBSYSTEM:  BIFS Decoder/ Compositor
FILE:       NodeFld.cpp
AUTHOR:     Mikael Bourges-Sevenier (France Telecom)
OVERVIEW:   Implementation of animation streams methods.
-------------------------------------------------------------------------*/
#include "defs.h"
#include "Nodes.h"
#include "NodeFld.h"
#include "MediaObj.h"
#include "BifsDec.h"
#include "Route.h"
#include "Root.h"
#include "Quantize.h"
#include "AnimStreamDec.h" // MBS: added


/*-----------------------------------------------------------------
The following functions are for parsing animation streams 
(mask and frames) resp.
added by MBS
-----------------------------------------------------------------*/
// dummy functions for NodeField
void NodeField::animate(AnimationStreamDecoder *,BOOL)
{
  fprintf(stderr,"ERROR: in NodeField::parseAnimationMask\n");
}

void NodeField::parseAnimationMask(AnimationStreamDecoder *, int)
{
  fprintf(stderr,"ERROR: in NodeField::parseAnimationMask\n");
}

// dummy functions for MediaObject *
void SF<MediaObject *>::animate(AnimationStreamDecoder *,BOOL)
{
  fprintf(stderr,"ERROR: in SF<MediaObject *>::animate\n");
}

void SF<MediaObject *>::parseAnimationMask(AnimationStreamDecoder *, int)
{
  fprintf(stderr,"ERROR: in SF<MediaObject *>::parseAnimationMask\n");
}

void MF<MediaObject *>::animate(AnimationStreamDecoder *,BOOL)
{
  fprintf(stderr,"ERROR: in SF<MediaObject *>::animate\n");
}

void MF<MediaObject *>::parseAnimationMask(AnimationStreamDecoder *, int)
{
  fprintf(stderr,"ERROR: in SF<MediaObject *>::parseAnimationMask\n");
}

void SF<int>::animate(AnimationStreamDecoder *pAS,BOOL intra)
{
  if(intra) {
    if(pAS->readBit()) // if we have new quantization parameters
      qp.readSame(pAS);
  }
  m_value=qp.ParseInt(pAS,intra);
}

void MF<int>::animate(AnimationStreamDecoder *pAS,BOOL intra)
{
  if(intra) {
    if(pAS->readBit()) // if we have new quantization parameters
      qp.readSame(pAS);
  }

  if(isTotal) {
    for(int i=0;i<GetLength();i++) {
      m_array[i]=qp.ParseInt(pAS,intra);
    }
  }
  else {
    for(int i=0;i<idx.GetLength();i++) {
      m_array[idx[i]]=qp.ParseInt(pAS,intra);
    }
  }
}

void SF<int>::parseAnimationMask(AnimationStreamDecoder *, int)
{
  /* not implemented */
}

void MF<int>::parseAnimationMask(AnimationStreamDecoder *, int)
{
  /* not implemented */
}

void SF<float>::animate(AnimationStreamDecoder *pAS,BOOL intra)
{
  if(intra) {
    if(pAS->readBit()) // if we have new quantization parameters
      qp.readSame(pAS);
  }
  m_value=qp.ParseFloat(pAS,intra);
}

void MF<float>::animate(AnimationStreamDecoder *pAS,BOOL intra)
{
  if(intra) {
    if(pAS->readBit()) // if we have new quantization parameters
      qp.readSame(pAS);
  }

  if(isTotal) {
    for(int i=0;i<GetLength();i++) {
      m_array[i]=qp.ParseFloat(pAS,intra);
    }
  }
  else {
    for(int i=0;i<idx.GetLength();i++) {
      m_array[idx[i]]=qp.ParseFloat(pAS,intra);
    }
  }
}

void SF<float>::parseAnimationMask(AnimationStreamDecoder *pAS, int anim)
{
  qp.readInitialParameters(pAS,anim);
}

void MF<float>::parseAnimationMask(AnimationStreamDecoder *pAS, int anim)
{
  isTotal=pAS->readBit();
  if(!isTotal) {
    do {
      idx += pAS->readBits(32);
    } while(pAS->readBit());
  }
  qp.readInitialParameters(pAS,anim);
}

void SF<Bool>::animate(AnimationStreamDecoder *,BOOL)
{
  /* not implemented */
}

void MF<Bool>::animate(AnimationStreamDecoder *,BOOL)
{
  /* not implemented */
}

void SF<Bool>::parseAnimationMask(AnimationStreamDecoder *, int)
{
  /* not implemented */
}

void MF<Bool>::parseAnimationMask(AnimationStreamDecoder *, int)
{
  /* not implemented */
}

void SF<Vec2f>::animate(AnimationStreamDecoder *pAS,BOOL intra)
{
  if(intra) {
    if(pAS->readBit()) // if we have new quantization parameters
      qp.readSame(pAS);
  } 
  m_value=qp.ParseVec2f(pAS,intra);
}

void MF<Vec2f>::animate(AnimationStreamDecoder *pAS,BOOL intra)
{
  if(intra) {
    if(pAS->readBit()) // if we have new quantization parameters
      qp.readSame(pAS);
  }   
  if(isTotal) {
    for(int i=0;i<GetLength();i++) {
      m_array[i]=qp.ParseVec2f(pAS,intra);
    }
  }
  else {
    for(int i=0;i<idx.GetLength();i++) {
      m_array[idx[i]]=qp.ParseVec2f(pAS,intra);
    }
  }
}

void SF<Vec2f>::parseAnimationMask(AnimationStreamDecoder *pAS, int anim)
{
  qp.readInitialParameters(pAS,anim);
}

void MF<Vec2f>::parseAnimationMask(AnimationStreamDecoder *pAS, int anim)
{
  isTotal=pAS->readBit();
  if(!isTotal) {
    do {
      idx += pAS->readBits(32);
    } while(pAS->readBit());
  }
  qp.readInitialParameters(pAS,anim);
}

void SF<Vec3f>::animate(AnimationStreamDecoder *pAS,BOOL intra)
{
  if(intra) {
    if(pAS->readBit()) // if we have new quantization parameters
      qp.readSame(pAS);
  }
  m_value=qp.ParseVec3f(pAS,intra);
}

void MF<Vec3f>::animate(AnimationStreamDecoder *pAS,BOOL intra)
{
  if(intra) {
    if(pAS->readBit()) // if we have new quantization parameters
      qp.readSame(pAS);
  }
  if(isTotal) {
    for(int i=0;i<GetLength();i++) {
      m_array[i]=qp.ParseVec3f(pAS,intra);
    }
  }
  else {
    for(int i=0;i<idx.GetLength();i++) {
      m_array[idx[i]]=qp.ParseVec3f(pAS,intra);
    }
  }
}

void SF<Vec3f>::parseAnimationMask(AnimationStreamDecoder *pAS, int anim)
{
  qp.readInitialParameters(pAS,anim);
}

void MF<Vec3f>::parseAnimationMask(AnimationStreamDecoder *pAS, int anim)
{
  isTotal=pAS->readBit();
  if(!isTotal) {
    do {
      idx += pAS->readBits(32);
    } while(pAS->readBit());
  }
  qp.readInitialParameters(pAS,anim);
}

void SF<Image>::parseAnimationMask(AnimationStreamDecoder *, int )
{
  /* not implemented */
}

void SF<Image>::animate(AnimationStreamDecoder *,BOOL)
{
  /* not implemented */
}

void SF<Time>::animate(AnimationStreamDecoder *,BOOL)
{
  /* not implemented */
}

void MF<Time>::animate(AnimationStreamDecoder *,BOOL)
{
  /* not implemented */
}

void MF<Image>::animate(AnimationStreamDecoder *,BOOL)
{
  /* not implemented */
}

void SF<Time>::parseAnimationMask(AnimationStreamDecoder *, int)
{
  /* not implemented */
}

void MF<Time>::parseAnimationMask(AnimationStreamDecoder *, int)
{
  /* not implemented */
}

void MF<Image>::parseAnimationMask(AnimationStreamDecoder *, int)
{
  /* not implemented */
}

void SF<Rotation>::animate(AnimationStreamDecoder *pAS,BOOL intra)
{
  if(intra) {
    if(pAS->readBit()) // if we have new quantization parameters
      qp.readSame(pAS);
  }  
  m_value=qp.ParseRotation(pAS,intra);
}

void MF<Rotation>::animate(AnimationStreamDecoder *pAS,BOOL intra)
{
  if(intra) {
    if(pAS->readBit()) // if we have new quantization parameters
      qp.readSame(pAS);
  }
  if(isTotal) {
    for(int i=0;i<GetLength();i++) {
      m_array[i]=qp.ParseRotation(pAS,intra);
    }
  }
  else {
    for(int i=0;i<idx.GetLength();i++) {
      m_array[idx[i]]=qp.ParseRotation(pAS,intra);
    }
  }
}

void SF<Rotation>::parseAnimationMask(AnimationStreamDecoder *pAS, int anim)
{
  qp.readInitialParameters(pAS,anim);
}

void MF<Rotation>::parseAnimationMask(AnimationStreamDecoder *pAS, int anim)
{
  isTotal=pAS->readBit();
  if(!isTotal) {
    do {
      idx += pAS->readBits(32);
    } while(pAS->readBit());
  }
  qp.readInitialParameters(pAS,anim);
}

void SF<Rotation2D>::animate(AnimationStreamDecoder *,BOOL)
{
  /* not implemented */
}

void MF<Rotation2D>::animate(AnimationStreamDecoder *,BOOL)
{
  /* not implemented */
}

void SF<Rotation2D>::parseAnimationMask(AnimationStreamDecoder *, int)
{
  /* not implemented */
}

void MF<Rotation2D>::parseAnimationMask(AnimationStreamDecoder *, int)
{
  /* not implemented */
}

void SF<Color3f>::animate(AnimationStreamDecoder *pAS,BOOL intra)
{
  if(intra) {
    if(pAS->readBit()) // if we have new quantization parameters
     qp.readSame(pAS);
  } 
  m_value=qp.ParseColor(pAS,intra);
}

void MF<Color3f>::animate(AnimationStreamDecoder *pAS,BOOL intra)
{
  if(intra) {
    if(pAS->readBit()) // if we have new quantization parameters
      qp.readSame(pAS);
  }
  if(isTotal) {
    for(int i=0;i<GetLength();i++) {
      m_array[i]=qp.ParseColor(pAS,intra);
    }
  }
  else {
    for(int i=0;i<idx.GetLength();i++) {
      m_array[idx[i]]=qp.ParseColor(pAS,intra);
    }
  }
}

void SF<Color3f>::parseAnimationMask(AnimationStreamDecoder *pAS, int anim)
{
  qp.readInitialParameters(pAS,anim);
}

void MF<Color3f>::parseAnimationMask(AnimationStreamDecoder *pAS, int anim)
{
  isTotal=pAS->readBit();
  if(!isTotal) {
    do {
      idx += pAS->readBits(32);
    } while(pAS->readBit());
  }
  qp.readInitialParameters(pAS,anim);
}

void SF<ZString>::animate(AnimationStreamDecoder *,BOOL)
{
  /* not implemented */
}

void MF<ZString>::animate(AnimationStreamDecoder *,BOOL)
{
  /* not implemented */
}

void SF<ZString>::parseAnimationMask(AnimationStreamDecoder *, int)
{
  /* not implemented */
}

void MF<ZString>::parseAnimationMask(AnimationStreamDecoder *, int)
{
  /* not implemented */
}

void SF<Url>::animate(AnimationStreamDecoder *,BOOL)
{
  /* not implemented */
}

void MF<Url>::animate(AnimationStreamDecoder *,BOOL)
{
  /* not implemented */
}

void SF<Url>::parseAnimationMask(AnimationStreamDecoder *, int)
{
  /* not implemented */
}

void MF<Url>::parseAnimationMask(AnimationStreamDecoder *, int)
{
  /* not implemented */
}

