/***********************************************************************
This software module was originally developed by Zvi Lifshitz
(Triton R&D Ltd.) in the course of development of the MPEG-4 Systems
(ISO/IEC 14496-1) standard. This software module is an implementation of
a part of one or more MPEG-4 Systems (ISO/IEC 14496-1) tools as
specified by the MPEG-4 Systems (ISO/IEC 14496-1) standard. ISO/IEC
gives users of the MPEG-4 Systems (ISO/IEC 14496-1) free license to
this software module or modifications thereof for use in hardware or
software products claiming conformance to the MPEG-4 Systems (ISO/IEC
14496-1). Those intending to use this software module in hardware or
software products are advised that its use may infringe existing patents.
The original developer of this software module and his/her company, the
subsequent editors and their companies, and ISO/IEC have no liability
for use of this software module or modifications thereof in an
implementation. Copyright is not released for non MPEG-4 Systems
(ISO/IEC 14496-1) conforming products. Triton R&D Ltd. retains full right
to use the code for his/her own purpose, assign or donate the code to a
third party and to inhibit third parties from using the code for non
MPEG-4 Systems (ISO/IEC 14496-1) conforming products. This copyright
notice must be included in all copies or derivative works.
Copyright  1998.
************************************************************************

For more information or to receive an up to date version of this module,
you may call Zvi Lifshitz at zvil@csi.com.
----------------------------------------------------------------------*/

/*-------------------------------------------------------------------------
Project Im1Player
INTERNATIONAL ORGANIZATION FOR STANDARDIZATION
ISO/IEC JTC1/SC29/WG11

SUBSYSTEM:  BIFS Encoder
FILE:       BifsEnc.cpp
AUTHOR:     Zvi Lifshitz (Triton R&D Ltd.)
OVERVIEW:   Implementation of BIFSEncoder, the class that encodes date into
			bits.
-------------------------------------------------------------------------*/
#include "defs.h"
#include "BifsEnc.h"
#include "ODStrEnc.h"
#include "NodeFld.h"
#include "Token.h"
#include "bifs.h"
#include "Proto.h"
#include "..\script\encoder\VScript.h"

#include "Quantize.h" // MBS 110698: for EfficientFloat coding

/*-----------------------------------------------------------------
BIFSEncoder constructor
-----------------------------------------------------------------*/
BIFSEncoder :: BIFSEncoder (
	FILE *fBifs,
	FILE *fList,
	LPCSTR szType,
	int flags,
	ODStreamEncoder *pODEncoder
) :
	SysCoder (fBifs, fList, szType, flags, 0)
{
	m_pODEncoder = pODEncoder;
	m_pProto = NULL;
//	m_bDontWrite = FALSE;

	// MBS: inits current QP to null
	m_pCurrentQP=NULL;
}

/*-----------------------------------------------------------------
Write the encoded data to another buffer
by MBS
-----------------------------------------------------------------*/
void BIFSEncoder :: Write (SysCoder *pCoder)
{
	Align();
	pCoder -> EncodeInt (m_buffer.GetLength (), 32, "value.length32");
	for (int i = 0; i < m_buffer.GetLength (); i++)
		pCoder -> EncodeInt (m_buffer[i], 8, NULL);
	m_buffer.Cut (0);
}


#ifdef ENCODER
/*-----------------------------------------------------------------
Read and encode a scene
-----------------------------------------------------------------*/
void BIFSEncoder :: Encode ()
{
	/*-------------------------------------------------------------
	The following overloading is required for encoding script nodes
	-------------------------------------------------------------*/
	OVERLOAD_NODE (ScriptImp);		// SCRIPT

	try
	{
		UpdateScene ();
	}
	catch (ExceptionEndOfFile)
	{
		nextToken.Error ("Unexpected end of file");
	}
}

/*-----------------------------------------------------------------
Encode Routes
-----------------------------------------------------------------*/
void BIFSEncoder :: EncodeRoutesDescription ()
{
	BOOL bMoreRoutes;

	EncodeInt (1, 1, "listDescription");
	do
	{
		EncodeRoute ();
		bMoreRoutes = nextToken == TID_ROUTE;
		EncodeInt (bMoreRoutes, 1, "moreRoutes");
	}
	while (bMoreRoutes);
}

/*-----------------------------------------------------------------
Encode one Route
-----------------------------------------------------------------*/
void BIFSEncoder :: EncodeRoute ()
{
	BOOL bIsUpdateable;

	if (nextToken == TID_DEF)
		bIsUpdateable = TRUE;
	else
	{	nextToken.Scan ();
		bIsUpdateable = nextToken == TID_DEF;
	}
	EncodeInt (bIsUpdateable, 1, "isUpdateable");
	if (bIsUpdateable)
	{	int nID = 0;
		nextToken.Scan ();
		if (nextToken == TID_NAME)
		{	nID = nextToken.GetDefinedRoute ();
			if (nID < 0)
				nID = nextToken.AddPredefinedRoute ();
			else
				nextToken.Error ("Route name already defined");
		}
		else
			nextToken.Error ("Unexpected symbol");
		EncodeInt (nID, 10, "RouteID");
		if (GetFlags () & CODENAMES)
			EncodeString (nextToken.GetName ());
		nextToken.Scan ();
		if (nextToken == TID_ROUTE)
			nextToken.Scan ();
	}
		
	EncodeRouteDetails ();
}

/*-----------------------------------------------------------------
Encode update commands
-----------------------------------------------------------------*/
void BIFSEncoder :: UpdateScene ()
{
	/*-----------------------------------------------------------------
	Loop for each update frame
	-----------------------------------------------------------------*/
	while (nextToken != TID_EOF)
	{	
		DWORD dwTS = 0;
		switch (nextToken) {
		case TID_FILE:
			nextToken.Scan ();
			if (nextToken != TID_STRING)
				nextToken.Error ("File name expected");
			else
			{	FILE *fNewOutput = fopen (nextToken.GetString (), "w");
				if (fNewOutput == NULL)
					nextToken.Error ("Cannot open file");
				else
					ReplaceOutputFile (fNewOutput);
				nextToken.Scan ();
			}
			break;
		case TID_AT:
			nextToken.Scan ();
			if (nextToken != TID_INTEGER)
				nextToken.Error ("Time (in milliseconds) expected");
			dwTS = nextToken.GetInt ();
		default:
			EncodeAU (dwTS);
		}
	}
}

/*-----------------------------------------------------------------
Encode one access unit
-----------------------------------------------------------------*/
void BIFSEncoder :: EncodeAU (DWORD dwTS)
{
	fprintf (m_fList, "------------AT %d-----------\n", dwTS);
	if (nextToken == TID_LEFT_BRACE)
	{	nextToken.Scan ();
		while (nextToken != TID_RIGHT_BRACE)
			if (EncodeUpdateCommand ())
				EncodeInt (nextToken != TID_RIGHT_BRACE, 1, "continue");
		nextToken.Scan ();
	}
	else if (EncodeUpdateCommand ())
		EncodeInt (0, 1, "continue");
	Flush (dwTS);
	if (m_pODEncoder)
		m_pODEncoder -> Flush (dwTS);
}

/*-----------------------------------------------------------------
Encode an INSERT command
-----------------------------------------------------------------*/
void BIFSEncoder :: InsertCommand ()
{
	nextToken.Scan ();
	EncodeInt (0, 2, "InsertionCommand");	// changed to 2 for FDIS
	switch (nextToken) {
	case TID_AT:
		InsertToField (FALSE);
		break;
	case TID_ROUTE:
		EncodeInt (3, 2, "RouteInsertion");
		EncodeRoute ();
		break;
	default:
		nextToken.Error ("Unexpected symbol");
		nextToken.Scan ();
	}
}

/*-----------------------------------------------------------------
Encode a replacement command
-----------------------------------------------------------------*/
void BIFSEncoder :: ReplaceCommand ()
{
	nextToken.Scan ();
	if (nextToken == TID_SCENE)
	{	nextToken.Scan ();
		if (nextToken != TID_BY)
			nextToken.Error ("BY expected");
		else
			nextToken.Scan ();
		SceneReplaceCommand ();
		return;
	}

	EncodeInt (2, 2, "ReplacementCommand");	// changed to 2 for FDIS
	if (nextToken == TID_ROUTE)
		ReplaceRoute ();
	else
	{	BOOL bLast = FALSE;
		if (nextToken == TID_LAST)
		{	bLast = TRUE;
			nextToken.Scan ();
		}
		MediaObject *pNode = ScanNode ();
		if (pNode)
		{	if (!bLast && nextToken == TID_BY)
			{	EncodeInt (0, 2, "NodeReplacement");
				EncodeInt (pNode -> GetID (), 10, "NodeID");
				nextToken.Scan ();
				SFWorldNode ().Encode (this);
			}
			else
				pNode -> FieldReplacement (this, bLast);
			if (pNode ->GetNodeCode () >= 0)
				delete pNode;
		}
	}
}

/*-----------------------------------------------------------------
Encode DELETE command
-----------------------------------------------------------------*/
void BIFSEncoder :: DeleteCommand ()
{
	nextToken.Scan ();
	EncodeInt (1, 2, "DeletionCommand");	// changed to 2 for FDIS
	if (nextToken == TID_ROUTE)
	{	nextToken.Scan ();
		EncodeInt (3, 2, "RouteDeletion");
		EncodeRouteID ();
	}
	else
	{	BOOL bLast = FALSE;
		if (nextToken == TID_LAST)
		{	bLast = TRUE;
			nextToken.Scan ();
		}
		MediaObject *pNode = ScanNode ();
		if (pNode)
		{	if (nextToken != TID_PERIOD)
			{	EncodeInt (0, 2, "NodeDeletion");
				EncodeInt (pNode -> GetID (), 10, "NodeID");
			}
			else
				pNode -> IndexedValueDeletion (this, bLast);
			if (pNode ->GetNodeCode () >= 0)
				delete pNode;
		}
	}
}

/*-----------------------------------------------------------------
Encode insertion of a value to an MF node field. If bAppend is TRUE the
value is appended to the end of the field, otherwise index is given
-----------------------------------------------------------------*/
void BIFSEncoder :: InsertToField (BOOL bAppend)
{
	nextToken.Scan ();
	MediaObject *pNode = ScanNode ();
	if (pNode)
	{	pNode -> InsertToField (this, bAppend);
		if (pNode ->GetNodeCode () >= 0)
			delete pNode;
	}
}

/*-----------------------------------------------------------------
Scan a node name and instantiate the node (may be a PROTO)
-----------------------------------------------------------------*/
MediaObject * BIFSEncoder :: ScanNode ()
{
	MediaObject *pNode = NULL;
	int nNodeID = EncodeNodeID ();
	if (nNodeID >= 0)
	{	int nNodeCode = nextToken.GetNodeID (nNodeID);
		if (nNodeCode >= 0)
			pNode = SFGenericNode ().CreateNode (nNodeCode);
		else
			pNode = nextToken.GetInstance (nNodeID);
	}
	if (pNode)
		pNode -> SetID (nNodeID);
	return pNode;
}

/*-----------------------------------------------------------------
Encode and APPEND TO command
-----------------------------------------------------------------*/
void BIFSEncoder :: AppendCommand ()
{
	nextToken.Scan ();
	EncodeInt (0, 2, "InsertionCommand");	// changed to 2 in FDIS
	if (nextToken != TID_TO)
		nextToken.Error ("TO expected");
	InsertToField (TRUE);
}

/*-----------------------------------------------------------------
Next token is route name. Translate it to route ID
-----------------------------------------------------------------*/
void BIFSEncoder :: EncodeRouteID ()
{
	int nID;

	if (nextToken == TID_NAME)
	{	nID = nextToken.GetDefinedRoute ();
		if (nID < 0)
			nextToken.Error ("Undefined route name");
		EncodeInt (nID, 10, "RouteID");
	}
	else
		nextToken.Error ("Unexpected symbol");
	nextToken.Scan ();
}

/*-----------------------------------------------------------------
Next token is node name. Translate it to node ID and return the ID
-----------------------------------------------------------------*/
int BIFSEncoder :: EncodeNodeID ()
{
	int nNodeID = -1;
	if (nextToken == TID_NAME)
	{	nNodeID = nextToken.GetDefinedName ();
		if (nNodeID < 0)
			nextToken.Error ("Undefined node name");
	}
	else
		nextToken.Error ("Unexpected symbol");
	nextToken.Scan ();
	return nNodeID;
}

/*-----------------------------------------------------------------
Encode a route (after encoding the DEF part
-----------------------------------------------------------------*/
void BIFSEncoder :: EncodeRouteDetails ()
{
	for (BOOL bIn = FALSE; bIn <= TRUE; ++bIn)
	{
		MediaObject *pNode = ScanNode ();
		if (pNode != NULL)
		{	pNode -> EncodeRoute (this, bIn);
			if (pNode ->GetNodeCode () >= 0)
				delete pNode;
		}
		if (!bIn)
		{	if (nextToken == TID_TO)
				nextToken.Scan ();
			else
				nextToken.Error ("TO expected");
		}
	}
}

void BIFSEncoder :: SceneReplaceCommand ()
{
	EncodeInt (3, 2, "SceneReplaceCommand");	// changed to 2 for FDIS
	EncodeScene ();
}

/*-----------------------------------------------------------------
Encode replace route command
-----------------------------------------------------------------*/
void BIFSEncoder :: ReplaceRoute ()
{	nextToken.Scan ();
	EncodeInt (3, 2, "RouteReplacement");
	EncodeRouteID ();
	if (nextToken == TID_BY)
		nextToken.Scan ();
	else
		nextToken.Error ("BY expected");
	EncodeRouteDetails ();
}

/*-----------------------------------------------------------------
Encode an initial scene (or scene replacement
-----------------------------------------------------------------*/
void BIFSEncoder :: EncodeScene ()
{
	EncodeInt (0, 6, "reserved");
	EncodeInt ((GetFlags () & CODENAMES) != 0, 1, "CODENAMES");
	bool bMoreProtos = nextToken == TID_PROTO || nextToken == TID_EXTERNPROTO;
	EncodeInt (bMoreProtos, 1, "morePROTOs");
	while (bMoreProtos)
	{	Proto *pProto = new Proto;
		if (!pProto -> EncodeProtoDefinition (this))
			delete pProto;
		bMoreProtos = nextToken == TID_PROTO || nextToken == TID_EXTERNPROTO;
		EncodeInt (bMoreProtos, 1, "morePROTOs");
	}
	SFTopNode node (TRUE);
	node.Encode (this);
	BOOL bHasRoutes = nextToken == TID_ROUTE;
	EncodeInt (bHasRoutes, 1, "hasRoutes");
	if (bHasRoutes)
		EncodeRoutesDescription ();
}

/*-----------------------------------------------------------------
Encode one update command.
Return TRUE if the command is a BIFS command
-----------------------------------------------------------------*/
BOOL BIFSEncoder :: EncodeUpdateCommand ()
{
	switch (nextToken) {
	case TID_INSERT:
		InsertCommand ();
		break;
	case TID_REPLACE:
		ReplaceCommand ();
		break;
	case TID_DELETE:
		DeleteCommand ();
		break;
	case TID_APPEND:
		AppendCommand ();
		break;
	case TID_UPDATE:
	case TID_REMOVE:
		m_pODEncoder -> ODUpdate ();
		return FALSE;
	default:
		SceneReplaceCommand ();
		break;
	}
	return TRUE;
}

/*-------------------------------------------------------------
Encode an SFString value
-------------------------------------------------------------*/
void BIFSEncoder :: EncodeSFString (const ZString &string)
{
	int nSize = string.GetLength ();
	int nBits = 0;

	while (nSize > 0)
	{	nSize >>= 1;
		nBits++;
	}

	EncodeInt (nBits, 5, "value.lengthBits");
	EncodeInt (string.GetLength (), nBits, "value.length");
  
	for (int i = 0; i < string.GetLength (); i++)
	{
		unsigned char c = string [i];
		EncodeInt (c, 8, (ZString ("value [") += c) += ']');  
	}
}

/*-------------------------------------------------------------
Encode a String value
-------------------------------------------------------------*/
void BIFSEncoder :: EncodeString (const ZString &string)
{
	for (int i = 0; i < string.GetLength (); i++)
	{
		unsigned char c = string [i];
		EncodeInt (c, 8, (ZString ("value [") += c) += ']');  
	}
	EncodeInt (0, 8, "end of string");
}

#endif
