/***********************************************************************
This software module was originally developed by Zvi Lifshitz
(Triton R&D Ltd.) in the course of development of the MPEG-4 Systems
(ISO/IEC 14496-1) standard. This software module is an implementation of
a part of one or more MPEG-4 Systems (ISO/IEC 14496-1) tools as
specified by the MPEG-4 Systems (ISO/IEC 14496-1) standard. ISO/IEC
gives users of the MPEG-4 Systems (ISO/IEC 14496-1) free license to
this software module or modifications thereof for use in hardware or
software products claiming conformance to the MPEG-4 Systems (ISO/IEC
14496-1). Those intending to use this software module in hardware or
software products are advised that its use may infringe existing patents.
The original developer of this software module and his/her company, the
subsequent editors and their companies, and ISO/IEC have no liability
for use of this software module or modifications thereof in an
implementation. Copyright is not released for non MPEG-4 Systems
(ISO/IEC 14496-1) conforming products. Triton R&D Ltd. retains full right
to use the code for his/her own purpose, assign or donate the code to a
third party and to inhibit third parties from using the code for non
MPEG-4 Systems (ISO/IEC 14496-1) conforming products. This copyright
notice must be included in all copies or derivative works.
Copyright  1998.
************************************************************************

For more information or to receive an up to date version of this module,
you may call Zvi Lifshitz at zvil@csi.com.
----------------------------------------------------------------------*/

#if !defined(_BifsEnc_h)	// Sentry, use file only if it's not already included.
#define _BifsEnc_h

/*-------------------------------------------------------------------------
Project Im1Player
INTERNATIONAL ORGANIZATION FOR STANDARDIZATION
ISO/IEC JTC1/SC29/WG11

SUBSYSTEM:  BIFS Encoder
FILE:       BifsEnc.h
AUTHOR:     Zvi Lifshitz (Triton R&D Ltd.)
OVERVIEW:   Implementation of BIFSEncoder, the class that encodes date into
			bits.
-------------------------------------------------------------------------*/
#include "ztl.h"
#include "NodeFld.h" // MBS: added for SF<> field
#include "SysCoder.h"

class ODStreamEncoder;
class ScriptImp;	// SCRIPT
class Proto;

class BIFSEncoder : public SysCoder
{
	ODStreamEncoder *m_pODEncoder;
	Proto *m_pProto;

	// MBS : declares current QP node to be applied to fields values
	MediaObject *m_pCurrentQP;

public:
	ZArray<ScriptImp *> ScriptList;	// SCRIPT
	void Write (SysCoder *pBifs); // MBS added
	BIFSEncoder (
		FILE *fBifs,
		FILE *fList,
		LPCSTR szType,
		int flags,
		ODStreamEncoder *pODEncoder = NULL
	);
	void Encode ();
	void SetProto (Proto *pProto) {m_pProto = pProto;}
	Proto *GetProto () const {return m_pProto;}

	// MBS 
	// sets current QP node
	void SetCurrentQP(MediaObject *qp) { m_pCurrentQP=qp; }
	// gets current QP node
	MediaObject *GetCurrentQP() { return m_pCurrentQP; }
	// do we have a QP to be used ?
	BOOL hasLocalQP() { return (m_pCurrentQP ? ((Bool) *((SF<Bool> *) m_pCurrentQP->GetField(0))) : FALSE) ; }
	// end MBS

#ifndef MUX
public:
	void EncodeString (const ZString &string);
	void EncodeSFString (const ZString &string);
	void EncodeAU (DWORD dwTS = 0);
	void EncodeRoutesDescription ();

private:
	BOOL EncodeUpdateCommand ();
	void EncodeScene ();
	void ReplaceRoute ();
	void SceneReplaceCommand ();
	void EncodeRouteDetails ();
	int EncodeNodeID ();
	void EncodeRouteID ();
	void AppendCommand ();
	MediaObject *ScanNode ();
	void InsertToField (BOOL bAppend);
	void ReplaceCommand ();
	void InsertCommand ();
	void UpdateScene ();
	void EncodeRoute ();
	void DeleteCommand ();
#endif		// BIFSENC
};

#endif                                      // _BifsEnc_h sentry.
