/***********************************************************************
This software module was originally developed by Zvi Lifshitz
(Triton R&D Ltd.) in the course of development of the MPEG-4 Systems
(ISO/IEC 14496-1) standard. This software module is an implementation of
a part of one or more MPEG-4 Systems (ISO/IEC 14496-1) tools as
specified by the MPEG-4 Systems (ISO/IEC 14496-1) standard. ISO/IEC
gives users of the MPEG-4 Systems (ISO/IEC 14496-1) free license to
this software module or modifications thereof for use in hardware or
software products claiming conformance to the MPEG-4 Systems (ISO/IEC
14496-1). Those intending to use this software module in hardware or
software products are advised that its use may infringe existing patents.
The original developer of this software module and his/her company, the
subsequent editors and their companies, and ISO/IEC have no liability
for use of this software module or modifications thereof in an
implementation. Copyright is not released for non MPEG-4 Systems
(ISO/IEC 14496-1) conforming products. Triton R&D Ltd. retains full right
to use the code for his/her own purpose, assign or donate the code to a
third party and to inhibit third parties from using the code for non
MPEG-4 Systems (ISO/IEC 14496-1) conforming products. This copyright
notice must be included in all copies or derivative works.
Copyright  1998.
************************************************************************

For more information or to receive an up to date version of this module,
you may call Zvi Lifshitz at zvil@csi.com.
----------------------------------------------------------------------*/

/*-------------------------------------------------------------------------
Project Im1Player
INTERNATIONAL ORGANIZATION FOR STANDARDIZATION
ISO/IEC JTC1/SC29/WG11

SUBSYSTEM:  BIFSEncoder
FILE:       main.cpp
AUTHOR:     Zvi Lifshitz (Triton R&D Ltd.)
OVERVIEW:   main() function for the BIFS encoder.
-------------------------------------------------------------------------*/
#include "defs.h"
#include "BifsEnc.h"
#include "ODStrEnc.h"

#define MAIN
#include "Token.h"

Token nextToken;

int main (int argc, char **argv)
{
	FILE *fInput, *fBifs, *fList, *fOD;
	int nNameLength;
	if (argc < 2)
	{	printf ("Missing file name\n");
		return -1;
	}

	/*-----------------------------------------------------------------
	Source file name extension is "txt", output extension is "bif" and
	list file is "lst"
	-----------------------------------------------------------------*/
	ZString stFileName (*++argv);
	nNameLength = stFileName.GetLength ();
	LPCSTR szExt = strrchr (stFileName, '.');
	if (szExt && !strchr (szExt, '\\'))
		nNameLength -= strlen (szExt);
	else
		stFileName += ".txt";

	if ((fInput = fopen (stFileName, "r")) == NULL)
	{	printf ("Cannot open %s\n", (LPCSTR) stFileName);
		return -1;
	}
	stFileName.Cut (nNameLength);
	stFileName += ".bif";
	if ((fBifs = fopen (stFileName, "wb")) == NULL)
	{	printf ("Cannot open %s\n", (LPCSTR) stFileName);
		return -1;
	}
	stFileName.Cut (nNameLength);
	stFileName += ".od";
	if ((fOD = fopen (stFileName, "wb")) == NULL)
	{	printf ("Cannot open %s\n", (LPCSTR) stFileName);
		return -1;
	}
	stFileName.Cut (nNameLength);
	stFileName += ".lst";
	if ((fList = fopen (stFileName, "w")) == NULL)
	{	printf ("Cannot open %s\n", (LPCSTR) stFileName);
		return -1;
	}

	int flags = TRACE_NONE;

	/*-------------------------------------------------------------
	Parse the trace argument
	-------------------------------------------------------------*/
	if (argc >= 3 && (*++argv)[0] == '-')
	{	int c;
		for (int i = 1; (c = (*argv) [i]) != 0; i++)
			switch (tolower (c)) {
			case 'v':
				flags |= TRACE_VERBOSE;
				break;
			case 'n':
				flags |= CODENAMES;
				break;
			}
	}

	nextToken.Init (fInput, fList);
	ODStreamEncoder odEncoder (fOD, fList, "OD", flags);
	BIFSEncoder encoder (fBifs, fList, "BIFS", flags, &odEncoder);
	encoder.Encode ();
	fclose (fInput);
	fclose (fBifs);
	fclose (fOD);
	fclose (fList);
	return 0;
}