/***********************************************************************
This software module was originally developed by Zvi Lifshitz
(Triton R&D Ltd.) and edited by Mauro Rossi (CSELT), Guido Franceschini
(CSELT), George Tselikis (Xbind Inc.) and Michael Stepping (FTK)
in the course of development of the MPEG-4 Systems
(ISO/IEC 14496-1) standard. This software module is an implementation of
a part of one or more MPEG-4 Systems (ISO/IEC 14496-1) tools as
specified by the MPEG-4 Systems (ISO/IEC 14496-1) standard. ISO/IEC
gives users of the MPEG-4 Systems (ISO/IEC 14496-1) free license to
this software module or modifications thereof for use in hardware or
software products claiming conformance to the MPEG-4 Systems (ISO/IEC
14496-1). Those intending to use this software module in hardware or
software products are advised that its use may infringe existing patents.
The original developer of this software module and his/her company, the
subsequent editors and their companies, and ISO/IEC have no liability
for use of this software module or modifications thereof in an
implementation. Copyright is not released for non MPEG-4 Systems
(ISO/IEC 14496-1) conforming products. Triton R&D Ltd. retains full right
to use the code for his/her own purpose, assign or donate the code to a
third party and to inhibit third parties from using the code for non
MPEG-4 Systems (ISO/IEC 14496-1) conforming products. This copyright
notice must be included in all copies or derivative works.
Copyright  1998.
************************************************************************

For more information or to receive an up to date version of this module,
you may call Zvi Lifshitz at zvil@csi.com.
----------------------------------------------------------------------*/

#if !defined(_al_h)	// Sentry, use file only if it's not already included.
#define _al_h

/*-------------------------------------------------------------------------
Project Im1Player
INTERNATIONAL ORGANIZATION FOR STANDARDIZATION
ISO/IEC JTC1/SC29/WG11

SUBSYSTEM:  Core
FILE:       AL.h
AUTHOR:     Zvi Lifshitz (Triton R&D Ltd.)
OVERVIEW:   Definition of ALManager
-------------------------------------------------------------------------*/
#include "DAI.h"

class Executive;
class MediaStreamImp;
class DataChannel;
class StreamConsumer;
class SLConfigDescriptor;
class InlineImp;
class ES_Descriptor;

class ALManager
{
	Executive *m_pExecutive;
	UINT m_nServiceID;
	
// MPEGJ MERGE - RLS This is public in the MPEGJ code
	//ZArray<DataChannel *> m_channels;
#ifdef MPGJ
public:
	ZArray<DataChannel *> m_channels;
#endif
#ifndef MPGJ
private:
	ZArray<DataChannel *> m_channels;
#endif



private:
// END MERGE

	ZString m_url;
	enum tagServiceState
	{
		SS_IDLE,
		SS_CONNECTING,
		SS_CONNECTED,
		SS_UNAVAILABLE
	} m_state;
	ZMutex m_mtxChannels, m_mtxLock;
	StreamConsumer *m_pNode;

private:
	void AddChannels ();
	void ChannelsClosed ();

public:
	void OnUserCommandCnf (const DAIUserCmdParam &outParam);
	void RequestChannelDelete (DataChannel *pChannel);
	void OnServiceAttach (UINT nServiceID, const BYTE* uuData, int uuData_Length, RESPONSE response);
	void OnServiceDetach (REASON reason);
	void OnChannelDeleteCnf (const DAIChannelDeleteParam &cnfParam) ;
	void Lock () {m_mtxLock.P ();}
	void Unlock () {m_mtxLock.V ();}
	void OnChannelAdd (const DAIChannelAddParam &cnfParam);
	LPCSTR GetURL () const {return m_url;}
	UINT GetServiceID () {return m_nServiceID;}
	void StartChannel (UINT nChannelID) {SendCommand ("PLAY", nChannelID);}
	void StopChannel (UINT nChannelID) {SendCommand ("PAUSE", nChannelID);}
	void SendCommand (char *szCommand, UINT nChannelID);
	void DeleteChannel (DataChannel *pChannel);
	~ALManager ();
	void Disconnect ();
	void Connect ();
	ALManager (Executive *pExecutive, StreamConsumer *pNode, LPCSTR szURL) :
		m_pExecutive (pExecutive),
		m_pNode (pNode),
		m_nServiceID (0xffffffff),
		m_state (SS_IDLE),
		m_url (szURL)
	{}
	DataChannel *AttachStream (
		ES_Descriptor *pESD,
		MediaStreamImp *pStream,
		SLConfigDescriptor *pConfigParams,
		InlineImp *pInline,
		int nExpectedAULength = 256
	);
	void RequestChannels ();
	Executive *GetExecutive () {return m_pExecutive;}
	
	#ifdef MPGJ	
	// Bindu - NewFrameWork --- BEGIN MPEGJ MERGE - RLS
	ZArray<DataChannel *> GetChannels(){
		return m_channels;
	}		
	StreamConsumer* getNode() { return m_pNode; } 
	//BINDU END MPEGJ MERGE - RLS	
	#endif

};

#endif                                      // _al_h sentry.