/***********************************************************************
This software module was originally developed by Zvi Lifshitz
(Triton R&D Ltd.) in the course of development of the MPEG-4 Systems
(ISO/IEC 14496-1) standard. This software module is an implementation of
a part of one or more MPEG-4 Systems (ISO/IEC 14496-1) tools as
specified by the MPEG-4 Systems (ISO/IEC 14496-1) standard. ISO/IEC
gives users of the MPEG-4 Systems (ISO/IEC 14496-1) free license to
this software module or modifications thereof for use in hardware or
software products claiming conformance to the MPEG-4 Systems (ISO/IEC
14496-1). Those intending to use this software module in hardware or
software products are advised that its use may infringe existing patents.
The original developer of this software module and his/her company, the
subsequent editors and their companies, and ISO/IEC have no liability
for use of this software module or modifications thereof in an
implementation. Copyright is not released for non MPEG-4 Systems
(ISO/IEC 14496-1) conforming products. Triton R&D Ltd. retains full right
to use the code for his/her own purpose, assign or donate the code to a
third party and to inhibit third parties from using the code for non
MPEG-4 Systems (ISO/IEC 14496-1) conforming products. This copyright
notice must be included in all copies or derivative works.
Copyright  1998.
************************************************************************

For more information or to receive an up to date version of this module,
you may call Zvi Lifshitz at zvil@csi.com.
----------------------------------------------------------------------*/

#if !defined(_BifsDec_h)	// Sentry, use file only if it's not already included.
#define _BifsDec_h

/*-------------------------------------------------------------------------
Project Im1Player
INTERNATIONAL ORGANIZATION FOR STANDARDIZATION
ISO/IEC JTC1/SC29/WG11

SUBSYSTEM:  BIFS Decoder
FILE:       BifsDec.h
AUTHOR:     Zvi Lifshitz (Triton R&D Ltd.)
OVERVIEW:   Declarations of BIFSDecoder.
-------------------------------------------------------------------------*/

//#include "ztl.h" // MBS: added for SF<> field
#include "SysCoder.h"
#include "NodeFld.h" // MBS: added for SF<> field

#ifdef BIFSCAR
#define InlineImp BifsCarouselStream
#endif

class InlineImp;
class MediaObject;
class Root;
class Executive;
class Proto;

class BIFSDecoder : public SysCoder
{
private:
	virtual void ParseFrame (LPCBYTE pData, int nLength, DWORD dwTS);
	virtual void Error (int);
	void ParseSceneReplaceCommand();
	void ParseReplacementCommand ();
	void ParseDeletionCommand ();
	void ParseInsertionCommand ();
	void ParseRoute (Root *pRoot, int nID = -1);
	void ParseRoutesDescription (Root *pRoot);
	virtual void EndOfUnit ();

public:
	void ParseString (ZString &string);
	void ParseSFString (ZString &string);
	void ParseProtoCode (Proto *pProto);
#ifndef BIFSCAR
	virtual bool Configure (const BYTE *specificInfo, int nInfoLength);
#endif
	int GetNodeIDbits () {return m_nNodeIDbits;}
	int GetRouteIDbits () {return m_nRouteIDbits;}
	int GetProtoIDbits () {return m_nProtoIDbits;}
	BIFSDecoder (
		Executive *pExecutive,
		InlineImp *pSource,
		FILE *fList,
		LPCSTR szListTitle,
		int flags,
		int version
	);
	void ParseUpdateFrame ();
	Executive *GetExecutive () const {return m_pExecutive;}
	InlineImp *GetSource () const {return m_pSource;}
	Root *GetScene () const {return m_pScene;}

        // MBS: 3 methods added for quantization
        // sets current QP
        void SetCurrentQP(MediaObject *qp) { m_pCurrentQP=qp; }
        // gets current QP
        MediaObject *GetCurrentQP() { return m_pCurrentQP; }
        // do we have a local QP ?
        BOOL hasLocalQP() { return (m_pCurrentQP ? ((BOOL) *((SFBool *) m_pCurrentQP->GetField(0))) : FALSE) ; }
        // end MBS

private:
        // MBS: currently used QP node
        MediaObject *m_pCurrentQP;
  
	Executive *m_pExecutive;
	InlineImp *m_pSource;
	BOOL m_bPixelMetric, m_bUsePredictiveMFField, m_bUse3DMeshCoding;
	int m_nPixelWidth, m_nPixelHeight;

protected:
	Root *m_pScene;
	int m_nNodeIDbits, m_nRouteIDbits, m_nProtoIDbits;
};

#endif                                      // _BifsDec_h sentry.
