/***********************************************************************
This software module was originally developed by Zvi Lifshitz
(Triton R&D Ltd.) and edited by Mauro Rossi (CSELT), Guido Franceschini
(CSELT) and Michael Stepping (FTK)
in the course of development of the MPEG-4 Systems
(ISO/IEC 14496-1) standard. This software module is an implementation of
a part of one or more MPEG-4 Systems (ISO/IEC 14496-1) tools as
specified by the MPEG-4 Systems (ISO/IEC 14496-1) standard. ISO/IEC
gives users of the MPEG-4 Systems (ISO/IEC 14496-1) free license to
this software module or modifications thereof for use in hardware or
software products claiming conformance to the MPEG-4 Systems (ISO/IEC
14496-1). Those intending to use this software module in hardware or
software products are advised that its use may infringe existing patents.
The original developer of this software module and his/her company, the
subsequent editors and their companies, and ISO/IEC have no liability
for use of this software module or modifications thereof in an
implementation. Copyright is not released for non MPEG-4 Systems
(ISO/IEC 14496-1) conforming products. Triton R&D Ltd. retains full right
to use the code for his/her own purpose, assign or donate the code to a
third party and to inhibit third parties from using the code for non
MPEG-4 Systems (ISO/IEC 14496-1) conforming products. This copyright
notice must be included in all copies or derivative works.
Copyright  1998.
************************************************************************

For more information or to receive an up to date version of this module,
you may call Zvi Lifshitz at zvil@csi.com.
----------------------------------------------------------------------*/

/*-------------------------------------------------------------------------
Project Im1Player
INTERNATIONAL ORGANIZATION FOR STANDARDIZATION
ISO/IEC JTC1/SC29/WG11

SUBSYSTEM:  Adaptation Layer
FILE:       DataChan.cpp
AUTHOR:     Zvi Lifshitz (Triton R&D Ltd.)
OVERVIEW:   Implementation of DataChannel object
-------------------------------------------------------------------------*/
#include "defs.h"
#include "Channel.h"
#include "Stream.h"
#include "Except.h"
#include "ODP.h"
#include "AL.h"
#include "Executiv.h"

//BINDU --- MPEGJ MERGE_1 - RLS
#ifdef SIA
DWORD WINAPI retrieveCredits(DataChannel *foo) 
{
	int i = -1;
	DWORD absStartTime, absEndTime, diffTime = 0, creditTime = 0;

	while(i <= 0){
		absStartTime = GetTickCount();
		if(creditTime >= 3000){
			foo->retrievedCredit = foo->credits;
			foo->credits = 0;
			creditTime = 0;
		} else 
			Sleep(500);

		absEndTime = GetTickCount();

		diffTime = absEndTime - absStartTime;
		creditTime += diffTime;
	}
	return 0;
}
#endif
//BINDU --- MPEGJ MERGE_1 - RLS

/*-------------------------------------------------------------
DataChannel constructor
-------------------------------------------------------------*/
DataChannel :: DataChannel (ALManager *pAL, int nESid, ES_Descriptor *pESD) :
	m_pAL (pAL),
	m_nESid (nESid),
	m_nChannelID (0xffffffff),
	m_bStreamOpen (false),
	m_state (CS_IDLE),
	m_pStream (NULL),
	m_pConfig (NULL),
	m_nEncodedAULength (-1),
	m_uuLength (0),
	m_ODDecoder (
		pAL -> GetExecutive (),
		NULL,
		pAL -> GetExecutive () -> GetTraceFile (),
		DetermineTitle (nESid),
		DetermineFlags (pAL -> GetExecutive () -> GetTraceFlags ()),
		0
	)
{
//BINDU --- MPEGJ MERGE_2 - RLS
#ifdef SIA
		HANDLE threadHandle;
		DWORD	id;
		enable = true;
		credits = 0;
		error = 0;
 		if((threadHandle = CreateThread(NULL, 0, (LPTHREAD_START_ROUTINE)retrieveCredits, (LPVOID)this, 0, &id)) == NULL)
		{
			assert(1==0);
		} 
#endif
//BINDU --- MPEGJ MERGE_2 - RLS

	SetESDParams (pESD);
}

/*-----------------------------------------------------------------
Unregister the channel from its service.
Close the stream before deleting the object.
-----------------------------------------------------------------*/
DataChannel :: ~DataChannel ()
{
	Lock ();
	bool bOpen = m_state >= CS_READY && m_state <= CS_OPEN;
	Unlock ();		// calling ChannelStop while object is locked might cause a deadlock
	if (bOpen)
		ChannelStop ();
	Lock ();
	Close ();
	Unlock ();
	m_pAL -> DeleteChannel (this);
}

/*-----------------------------------------------------------------
Config() binds a MediaStream, copy the config params,
and store values of predefined parameters
-----------------------------------------------------------------*/
void DataChannel :: Config (
	MediaStreamImp *pStream,
	SLConfigDescriptor *pConfigParams,
	int nExpectedAULength
)
{
	m_pStream = pStream;
	m_pClock = NULL;
	if (pStream)
		m_pClock = pStream -> GetClockReferece ();
	m_nExpectedAULength = nExpectedAULength;
	m_pConfig = pConfigParams;
	m_fOCRResolutionFactor = 1000. / m_pConfig -> OCRResolution;
	m_fTSResolutionFactor = 1000. / m_pConfig -> timeStampResolution;
	m_dwPDUSeqNumMask =
		((DWORD) -1) >> (sizeof (DWORD) * 8 - m_pConfig -> packetSeqNumLength);
	m_dwAUSeqNumMask =
		((DWORD) -1) >> (sizeof (DWORD) * 8 - m_pConfig -> AU_seqNumLength);
	m_state = CS_CONFIGURED;

//BINDU --- MPEGJ MERGE_3 - RLS
#ifdef SIA
	retrievedCredit = 0; 
#endif
//BINDU --- MPEGJ MERGE_3 - RLS
}

/*-----------------------------------------------------------------
Channel is ready for input. A request to the service to add this channels
will be sent after all channels required at this moment will setup.
-----------------------------------------------------------------*/
void DataChannel :: ChannelReady ()
{
	Lock ();
	m_pBuffer = NULL;
	m_dwPreCTS = m_dwCTS = m_dwPreDTS = m_dwDTS = m_dwTimeOffset = m_dwPreOCR = 0;
	m_nAULength = m_nPreSN = 0;
	m_bWaitRandomAccessPoint = m_bNewAU = true;
	m_nExpectedPDUSequenceNumber = m_nExpectedAUSequenceNumber = 
		m_nEncodedAULength = -1;

	/*-------------------------------------------------------------
	If this stream doesn't depend on another streams's OCRs, and there
	are no OCRs in this stream, the first CTS will set the clock reference
	-------------------------------------------------------------*/
	if (m_pConfig -> OCR_ES_Id == 0  || m_pConfig -> OCR_ES_Id == m_nESid)
	{	if (m_pClock)
			m_pClock -> Reset ();
		m_bResetClockFromCTS = m_pConfig -> OCRLength == 0;
	}
	else
		m_bResetClockFromCTS = false;
	m_bOCRInit = m_bResetClockFromCTS;

	/*-------------------------------------------------------------
	Since timestamps and OCRs are received as residual values, we don't know
	what the real value of the first OCR is. This is not important, since
	sync is achieved by comparing the timestamps to the OCRs. We assume that
	the first OCR offset is OCRLength^2. This way, the following case
	should work well:
	- OCRLength and TimeStampLength are 16 bits.
	- The player logs at a point when the real OCR value is 0x40002.
	- The a stream contains a timestamp whose real value is 0x3fff7.
	- The value of the first OCR parsed by the player is 0x0002.
	- The value of the first timestamp parsed by the player is 0xfff7.
	- Since we assume initial OCR offset of 0x10000, we assume the first
	  OCR is 0x10002.
	- When we see the first timestamp, we can assume its real value is
	  0xfff7, and sync will work well.
	- If the first parsed timestamp is, for instance, 0x0009, we will
	  conclude that its real value is 0x10009, ans sync will still work
	  well.
	-------------------------------------------------------------*/
	m_dwOCROffset = 1 << m_pConfig -> OCRLength;

	if (m_pStream)
	{	m_bStreamOpen = true;
		m_pStream -> Open ();
	}
	m_state = CS_READY;
	Unlock ();
}

/*-----------------------------------------------------------------
Called from Service when channel is open. Add it to the list of open channels
and send a PLAY command
-----------------------------------------------------------------*/
void DataChannel :: OnChannelOpen (UINT nChannelID, RESPONSE eResponse)
{
	if (eResponse == RESPONSE_OK)
	{	if (m_state == CS_REQUESTED)
		{	m_nChannelID = nChannelID;
			m_state = CS_OPEN;
			m_pAL ->GetExecutive () -> AddChannel (this);
			m_pAL -> StartChannel (m_nChannelID);
		}
	}
	else
	{	m_pAL ->GetExecutive () -> Error (CHANNEL_ADD_FAILED);
		m_state = CS_UNAVAILABLE;
		Close ();
	}
}

/*-----------------------------------------------------------------
Parse an AU and post it to the stream
-----------------------------------------------------------------*/
void DataChannel :: OnPacketReceived (
	const BYTE *pStreamData,
	int nStreamDataLength,
	BOOL bErr
)
{
//BINDU --- MPEGJ MERGE_4 - The ending bracket will also be labeled 4 - RLS
#ifdef SIA
  if(enable)
  {
#endif
//BINDU
	/*-------------------------------------------------------------
	Erroneous packet is skipped. If the stream has no in-band integrity
	mechanism (i.e. PDU_seqNum), we need to wait for RAP.
	-------------------------------------------------------------*/
	if (bErr)
	{	if (m_pConfig -> packetSeqNumLength == 0)
		{	SkipTillRAP ();
			m_pAL -> GetExecutive () -> Error (DATA_TRANSMISSION_ERROR);
		}
		return;
	}

//BINDU --- MPEGJ MERGE_5 - RLS
#ifdef SIA
	error = 0;
	credits += 1;
#endif
//BINDU --- MPEGJ MERGE_5 - RLS

	/*-------------------------------------------------------------
	Parse the packet header
	-------------------------------------------------------------*/
	m_ODDecoder.Reset (pStreamData, nStreamDataLength);
	SL_PacketHeader header (m_bNewAU);
	header.SetLength (nStreamDataLength * 8);
	header.EncDec (&m_ODDecoder, DECODE, m_pConfig);

	if (header.idleFlag ||
		(header.paddingFlag && header.paddingBits == 0)
	)
		return;

	/*-------------------------------------------------------------
	Get the OCR and update the clock
	-------------------------------------------------------------*/
	if (header.OCRflag)
	{	DWORD dwOCR = header.objectClockReference;
		if (dwOCR + m_dwOCROffset < m_dwPreOCR)
			m_dwOCROffset += 1 << m_pConfig -> OCRLength;
		m_dwPreOCR = dwOCR + m_dwOCROffset;
		dwOCR = m_dwPreOCR * m_fOCRResolutionFactor;

		if (m_pClock)
			m_pClock -> SetTime (dwOCR);
	}

	/*-------------------------------------------------------------
	Now process the AU
	-------------------------------------------------------------*/
	if (header.accessUnitStartFlag)
	{
		m_nAUsequenceNumber = header.AU_sequenceNumber;
		/*-------------------------------------------------------------
		Calculate the actual composition time.
		-------------------------------------------------------------*/
		if (header.compositionTimeStampFlag)
		{	DWORD dwCTS = header.compositionTimeStamp;
			DWORD dwDTS = dwCTS;
			if (header.decodingTimeStampFlag)
				dwDTS = header.decodingTimeStamp;
			if (dwDTS + m_dwTimeOffset < m_dwPreDTS )
				if (m_pConfig -> timeStampLength < 32)
					m_dwTimeOffset += 1 << m_pConfig -> timeStampLength;

			/*-------------------------------------------------------------
			If hasn't been done before, compare the TS to the OCR in
			order to find the TS initial offset.
			-------------------------------------------------------------*/
			if (!m_bOCRInit && m_pClock && m_pClock -> IsOCRInit ())
			{	m_dwTimeOffset = m_pClock -> AdjustTimeOffset (
									dwDTS,
									m_pConfig -> timeStampLength,
									m_fTSResolutionFactor
								 );
				m_bOCRInit = true;
			}

			m_dwPreCTS = dwCTS + m_dwTimeOffset;
			m_dwPreDTS = dwDTS + m_dwTimeOffset;
			if (m_dwPreCTS < m_dwPreDTS)
				m_dwPreCTS += 1 << m_pConfig -> timeStampLength;
			m_dwCTS = m_dwPreCTS * m_fTSResolutionFactor;
			m_dwDTS = m_dwPreDTS * m_fTSResolutionFactor;
		}
		else
		{	
			/*-------------------------------------------------------------
			All times are calculated relative to the first receive
			OCR.
			-------------------------------------------------------------*/
			if (!m_bOCRInit && m_pClock && m_pClock -> IsOCRInit ())
			{	m_dwPreCTS = m_pClock -> AdjustTimeOffset (0, 0, 1);
				m_bOCRInit = true;
			}
			/*-------------------------------------------------------------
			If timestamp is not included, calculate it out of
			compositionUnitRate
			-------------------------------------------------------------*/
			if (m_pConfig -> AU_seqNumLength == 0)
			{	m_dwCTS = m_dwPreCTS;
				m_dwPreCTS += m_pConfig -> compositionUnitDuration;
			}
			else
			{	/*-------------------------------------------------------------
				If AUs are counted, calculate the CTS according to the
				SN
				-------------------------------------------------------------*/
				int n = m_nAUsequenceNumber - m_nPreSN;
				if (n < 0)
					n += m_dwAUSeqNumMask + 1;
				m_nPreSN = m_nAUsequenceNumber;
				m_dwCTS = m_dwPreCTS +=
					n * m_pConfig -> compositionUnitDuration;
			}
			m_dwCTS *= m_fTSResolutionFactor;
		}

		if (m_pClock)
		{	/*-------------------------------------------------------------
			If the attached clock has not been reset yet and no OCRs
			are available, reset it to the current CTS
			-------------------------------------------------------------*/
			if (m_bResetClockFromCTS)
			{	m_bResetClockFromCTS = false;
				m_pClock -> SetTime (m_dwCTS);
			}

			/*-------------------------------------------------------------
			If the clock of this stream hasn't been initialized yet, we store
			0 instead of the timestamp, which has no value when the clock is 
			not known. Normal 0 is converted to 1, to prevent ambiguity
			-------------------------------------------------------------*/
			if (!m_pClock -> IsOCRInit ())
				m_dwCTS = 0;
			else if (m_dwCTS == 0)
				m_dwCTS = 1;
		}
		if (m_pConfig -> AU_Length > 0)
			m_nEncodedAULength = header.accessUnitLength;
	}

	/*-------------------------------------------------------------
	Unexpected accessUnitStartFlag
	-------------------------------------------------------------*/
	if (header.accessUnitStartFlag && !m_bNewAU)
	{	SkipTillRAP ();
		m_pAL -> GetExecutive () -> Error (DATA_TRANSMISSION_ERROR);
		return;
	}

	/*-------------------------------------------------------------
	Skip AU if a repeated AU sequence number
	-------------------------------------------------------------*/
	if (m_nExpectedAUSequenceNumber == 
		(int) ((m_nAUsequenceNumber + 1) & m_dwAUSeqNumMask)
	)
		return;

	/*-------------------------------------------------------------
	Skip AU if looking for a RAP and this is not a RAP
	-------------------------------------------------------------*/
	if (m_bWaitRandomAccessPoint &&
		(!header.accessUnitStartFlag || !header.randomAccessPointFlag)
	)
		return;

	/*-----------------------------------------------------------------
	Calculate the payload length
	-----------------------------------------------------------------*/
	int nOldLength = m_nAULength;
	int nHeaderLength = nStreamDataLength - header.GetLength () / 8;
	pStreamData += nHeaderLength;
	nStreamDataLength -= nHeaderLength;
	m_nAULength += nStreamDataLength;
	bool bAUEnd = header.accessUnitEndFlag != 0;
	if (!m_pConfig -> useAccessUnitEndFlag &&
		m_nEncodedAULength == m_nAULength
	)
		bAUEnd = true;

	/*-----------------------------------------------------------------
	Integrity checks
	-----------------------------------------------------------------*/
	if (m_nExpectedPDUSequenceNumber >= 0 &&
		m_nExpectedPDUSequenceNumber != header.packetSequenceNumber ||
		m_nExpectedAUSequenceNumber >= 0 &&
		m_nExpectedAUSequenceNumber != m_nAUsequenceNumber ||
		m_bNewAU && !header.accessUnitStartFlag ||
		bAUEnd && m_nEncodedAULength >= 0 &&
		m_nEncodedAULength != m_nAULength
	)
	{	SkipTillRAP ();
		m_pAL -> GetExecutive () -> Error (DATA_TRANSMISSION_ERROR);
		return;
	}

	if (m_pConfig -> packetSeqNumLength > 0)
		m_nExpectedPDUSequenceNumber =
			header.packetSequenceNumber + 1 & m_dwPDUSeqNumMask;
	if (bAUEnd && m_pConfig -> AU_seqNumLength > 0)
		m_nExpectedAUSequenceNumber = ++m_nAUsequenceNumber & m_dwAUSeqNumMask;

	m_bWaitRandomAccessPoint = m_bNewAU = false;

	/*-----------------------------------------------------------------
	If this is a new AU, allocate buffer on the stream, otherwise,
	reallocate if necessary.
	If this is AU end, we know exactly how much to allocate. Otherwise,
	we use the size recommended by the decoder.
	-----------------------------------------------------------------*/
	if (m_bStreamOpen)
	{	if (m_pBuffer == NULL)
			m_pBuffer = (LPBYTE) m_pStream -> Allocate (
				(bAUEnd || (m_nAULength > m_nExpectedAULength)) ?
				m_nAULength : m_nExpectedAULength
			);
		else if (m_nAULength > m_nExpectedAULength)
			m_pBuffer = (LPBYTE) m_pStream -> Reallocate (m_nAULength);

		if (m_pBuffer == END_OF_STREAM)
		{	m_pBuffer = NULL;
			Close ();
			m_nAULength = 0;
		}
		else if (m_pBuffer == NULL)
		{	SkipTillRAP ();
			m_pAL -> GetExecutive () -> Error (BUFFER_TOO_SMALL);
		}
	}
		
	/*-----------------------------------------------------------------
	Copy the AU payload to the buffer
	-----------------------------------------------------------------*/
	if (m_pBuffer)
	{	memcpy (m_pBuffer + nOldLength, pStreamData, nStreamDataLength);
		/*-----------------------------------------------------------------
		If this is end of AU, dispatch the buffer into the MediaStream.
		-----------------------------------------------------------------*/
		if (bAUEnd && m_pStream && m_bStreamOpen)
			m_pStream -> Dispatch (m_nAULength, m_dwCTS);
	}
	if (bAUEnd)
	{	m_nAULength = 0;
		m_bNewAU = true;
		m_pBuffer = NULL;
	}
//BINDU --- MPEGJ MERGE_4 - Closing bracket - RLS
#ifdef SIA
  }
#endif
//BINDU --- MPEGJ MERGE_4 - Closing bracket - RLS
} 

/*-----------------------------------------------------------------
SkipTillRAP set flags and variables so decoding stops until next
random access point.
-----------------------------------------------------------------*/
void DataChannel :: SkipTillRAP ()
{
	m_nExpectedAUSequenceNumber = m_nExpectedPDUSequenceNumber = -1;
	m_bNewAU = m_bWaitRandomAccessPoint = true;
	m_nAULength = 0;
}

/*-----------------------------------------------------------------
Stop a channel from playing
-----------------------------------------------------------------*/
void DataChannel :: ChannelStop ()
{
	m_pAL -> RequestChannelDelete (this);
	Lock ();
	m_state = CS_CONFIGURED;
	Close ();
	Unlock ();
}

/*-----------------------------------------------------------------
Close the channel without deleting it. This involves closing the
stream and reseting variables
-----------------------------------------------------------------*/
inline void DataChannel :: Close ()
{
	if (m_bStreamOpen)
	{	m_bStreamOpen = false;
		if (m_pStream)
			m_pStream -> Close ();
	}
}

/*-------------------------------------------------------------
When channel is deleted by DMIF, close the attached stream, if exists.
Delete the channel if deleting was requested before, or if opened
preemptively by DMIF
-------------------------------------------------------------*/
void DataChannel :: OnChannelDelete (REASON reason)
{
	//MPEGJ
	#ifdef SIA
	error = 0; 
	#endif
	//MPEGJ

	Close ();
	m_state = CS_CONFIGURED;
	if (reason != REASON_OK)
	{	m_state = CS_UNAVAILABLE;		
		m_pAL ->GetExecutive () ->Error (UNEXPECTED_CHANNEL_DELETE);
	}
	m_pAL ->GetExecutive () ->DeleteChannel (this);
}

/*-------------------------------------------------------------
Channel shall be requested as soon as the service is connected
Return true if channel is indeed ready.
-------------------------------------------------------------*/
bool DataChannel :: Request ()
{
	if (m_state == CS_READY)
	{	m_state = CS_REQUIRED;
		return true;
	}
	return false;
}

/*-------------------------------------------------------------
If channel is required, mark it as requested and return true
-------------------------------------------------------------*/
BOOL DataChannel :: IsRequired ()
{
	if (m_state == CS_REQUIRED)
	{	m_state = CS_REQUESTED;
		return true;
	}
	else
		return false;
}

/*-------------------------------------------------------------
Display a message error in case user command rejected
-------------------------------------------------------------*/
void DataChannel :: OnUserCommandReject (int)
{
	m_pAL ->GetExecutive () ->Error (SEND_COMMAND_FAILED);
}

/*-------------------------------------------------------------
Sort out the flags that control the parse tracing
-------------------------------------------------------------*/
int DataChannel :: DetermineFlags (int flags)
{
	if (!(flags & TRACE_HEADERS))
		flags = TRACE_NONE;
	else if (!(flags & TRACE_VERBOSE))
		flags = TRACE_PARSE;
	return flags;
}

/*-------------------------------------------------------------
Sort out the title for the parse tracing.
-------------------------------------------------------------*/
LPCSTR DataChannel :: DetermineTitle (int nESid)
{
	char szTitle [140];
	sprintf (szTitle, "SL_PacketHeader (%s:%d)", m_pAL -> GetURL (), nESid);
#ifdef TRACE_ENCODING
	m_title.Copy (szTitle);
#endif
	return m_title;
}

/*-------------------------------------------------------------
Initialize values that come from the ES_Descriptor, if available.
If a QoSDescriptor exists, it is re-encoded.
[There is a chance here to manipulate the descriptor, if necessary]
-------------------------------------------------------------*/
void DataChannel :: SetESDParams (ES_Descriptor *pESD)
{
	if (pESD != NULL)
	{	if (pESD ->URLstring.GetLength () > 0)
		{	m_uuData = (LPBYTE) (LPCSTR) pESD ->URLstring;
			m_uuLength = pESD->URLstring.GetLength () + 1;
		}
		else
		{	m_esidAsuuData [0] = HIBYTE (m_nESid);
			m_esidAsuuData [1] = LOBYTE (m_nESid);
			m_uuData = m_esidAsuuData;
			m_uuLength = 2;
		}
		if (pESD -> qosDescr != NULL_DESC)
		{	ODCoder coder (NULL, NULL, NULL, 0);
			pESD -> qosDescr -> EncDec (&coder, CALC_LENGTH, NULL);
			pESD -> qosDescr -> EncDec (&coder, ENCODE, NULL);
			m_qos.Assign (coder.GetData (), coder.GetDataLength ());
		}
	}
}
