/***********************************************************************
This software module was originally developed by Zvi Lifshitz
(Triton R&D Ltd.) in the course of development of the MPEG-4 Systems
(ISO/IEC 14496-1) standard. This software module is an implementation of
a part of one or more MPEG-4 Systems (ISO/IEC 14496-1) tools as
specified by the MPEG-4 Systems (ISO/IEC 14496-1) standard. ISO/IEC
gives users of the MPEG-4 Systems (ISO/IEC 14496-1) free license to
this software module or modifications thereof for use in hardware or
software products claiming conformance to the MPEG-4 Systems (ISO/IEC
14496-1). Those intending to use this software module in hardware or
software products are advised that its use may infringe existing patents.
The original developer of this software module and his/her company, the
subsequent editors and their companies, and ISO/IEC have no liability
for use of this software module or modifications thereof in an
implementation. Copyright is not released for non MPEG-4 Systems
(ISO/IEC 14496-1) conforming products. Triton R&D Ltd. retains full right
to use the code for his/her own purpose, assign or donate the code to a
third party and to inhibit third parties from using the code for non
MPEG-4 Systems (ISO/IEC 14496-1) conforming products. This copyright
notice must be included in all copies or derivative works.
Copyright  1998.
************************************************************************

For more information or to receive an up to date version of this module,
you may call Zvi Lifshitz at zvil@csi.com.
----------------------------------------------------------------------*/

/*-------------------------------------------------------------------------
Project Im1Player
INTERNATIONAL ORGANIZATION FOR STANDARDIZATION
ISO/IEC JTC1/SC29/WG11

SUBSYSTEM:  BIFS Decoder/ Compositor / Presenter
FILE:       Group.cpp
AUTHOR:     Zvi Lifshitz (Triton R&D Ltd.)
OVERVIEW:   Implementation of Group
-------------------------------------------------------------------------*/

#include "defs.h"
#include "Group.h"
#include "bifs.h"
#include "NodeFld.h"
#include "BifsDec.h"

/*-----------------------------------------------------------------
Init searches for the field which is flagged as 'children',
stores a reference to the field, and set 'this' as the parent of the
field
-----------------------------------------------------------------*/
void GroupNode :: Init ()
{
	int i = -1;
	do
	{	++i;
		assert (i < GetFieldCount ());
	}
	while (!GetFieldTable () [i].m_bChildren);
	m_pChildren = (MFGenericNode *) GetField (i);
}

/*-----------------------------------------------------------------
Group :: Parse() first calls Init, then calls the base class Parse
function
-----------------------------------------------------------------*/
BOOL GroupNode :: Parse (BIFSDecoder *pCoder)
{
	Init ();
	return MediaObject :: Parse (pCoder);
}

/*-----------------------------------------------------------------
Similar to Parse(), this function is called by BIFSDecoder to insert
one node into the children field of the node.  The information about 
the new node and its exact position are parsed from the BIFS stream by 
this function.   Returns TRUE if successful.
-----------------------------------------------------------------*/
BOOL GroupNode :: ParseChild (BIFSDecoder *pCoder)
{
	int nPos;
	BOOL bResult = FALSE;

	switch (pCoder -> ParseInt (2, "type")) {
	case 0:				// insertion at a specified position
		nPos = pCoder -> ParseInt (8, "position");
		TRACEF ("[%d] ", nPos);
		bResult = GetChildren ().InsertElement (pCoder, this, nPos);
		break;
	case 2: 			// insertion at the beginning of the field
		TRACEF ("[0] ", 0);
		bResult = GetChildren ().InsertElement (pCoder, this, 0);
		break;
	case 3: 			// insertion at the end of the field
		TRACEF ("[end] ", 0);
		bResult = GetChildren ().AppendElement (pCoder, this);
		break;
	}
	if (bResult)
	{	SetUpdateTime ();
		OnUpdate ();
		GetChildren ().OnFieldUpdate ();
	}
	return bResult;
}