/***********************************************************************
This software module was originally developed by Zvi Lifshitz
(Triton R&D Ltd.) in the course of development of the MPEG-4 Systems
(ISO/IEC 14496-1) standard. This software module is an implementation of
a part of one or more MPEG-4 Systems (ISO/IEC 14496-1) tools as
specified by the MPEG-4 Systems (ISO/IEC 14496-1) standard. ISO/IEC
gives users of the MPEG-4 Systems (ISO/IEC 14496-1) free license to
this software module or modifications thereof for use in hardware or
software products claiming conformance to the MPEG-4 Systems (ISO/IEC
14496-1). Those intending to use this software module in hardware or
software products are advised that its use may infringe existing patents.
The original developer of this software module and his/her company, the
subsequent editors and their companies, and ISO/IEC have no liability
for use of this software module or modifications thereof in an
implementation. Copyright is not released for non MPEG-4 Systems
(ISO/IEC 14496-1) conforming products. Triton R&D Ltd. retains full right
to use the code for his/her own purpose, assign or donate the code to a
third party and to inhibit third parties from using the code for non
MPEG-4 Systems (ISO/IEC 14496-1) conforming products. This copyright
notice must be included in all copies or derivative works.
Copyright  1998.
************************************************************************

For more information or to receive an up to date version of this module,
you may call Zvi Lifshitz at zvil@csi.com.
----------------------------------------------------------------------*/

/*-------------------------------------------------------------------------
Project Im1Player
INTERNATIONAL ORGANIZATION FOR STANDARDIZATION
ISO/IEC JTC1/SC29/WG11

SUBSYSTEM:  Executive
FILE:       Inline.cpp
AUTHOR:     Zvi Lifshitz (Triton R&D Ltd.)
OVERVIEW:   Implementation of InlineImp, the objects that implements Inline
-------------------------------------------------------------------------*/
/*-------------------------------------------------------------------------
MPEGJ 
AUTHORS:    Viswanathan Swaminathan - vishy@eng.sun.com 
			Robert Schmidt(RLS)
			Praveen Alavilli(PA) 
			YUNTAI KYONG(YT) - yuntaikyong@ieee.org
-------------------------------------------------------------------------*/

#include "defs.h"
#include "Inline.h"
#include "Executiv.h"
#include "BifsDec.h"
#include "ODDec.h"
#include "StrParser.h"
#include "ODImp.h"
#include "Clock.h"
#include "CrNodes.h"
#include "IPMP.h"



/*-------------------------------------------------------------
Create a decoder for an Inline node
-------------------------------------------------------------*/
Decoder *InlineImp :: CreateDecoder (
	DecoderConfigDescriptor *pDecoderConfig,
	QoS_Descriptor *,
	Decoder *
)
{
	GetOD () -> SetInline (this);

	/*-------------------------------------------------------------
	Depending on the stream type, we create either a BIFS decoder
	or an OD decoder
	-------------------------------------------------------------*/
	SysCoder *pCoder;
	switch (pDecoderConfig -> streamType) {
	case ST_SceneDescriptionStream:
		pCoder = new BIFSDecoder (
			GetExecutive (),
			this,
			GetExecutive () -> GetTraceFile (),
			"BIFS",
			GetExecutive () -> GetTraceFlags (),
			pDecoderConfig -> objectTypeIndication == 0xff ? 0 : 
			pDecoderConfig -> objectTypeIndication
		);
		break;
	case ST_ObjectDescriptorStream:
		pCoder = new ODDecoder (
			GetExecutive (),
			this,
			GetExecutive () -> GetTraceFile (),
			"OD",
			GetExecutive () -> GetTraceFlags (),
			pDecoderConfig -> objectTypeIndication == 0xff ? 0 : 
			pDecoderConfig -> objectTypeIndication
		);
		break;
	#ifdef MPGJ
	//*** YT START (8)
	//	MPEGJ decoder is not created by InlineImp, but make it not set error
	//  to Executive

	case ST_MPEGJStream:
		return NULL;
		break;
	//*** YT END (8)
	#endif
	default:
		GetExecutive () -> Error (INVALID_STREAM_TYPE);
		return NULL;
	}
	return new StreamParser (GetExecutive (), pCoder);
}

/*-----------------------------------------------------------------
Render the object by rendering the associated Group object
-----------------------------------------------------------------*/
BOOL InlineImp :: Render (void *pContext)
{
	if (m_subScene == NULL_OBJECT)
		return TRUE;
	return m_subScene -> Render (pContext);
}

/*-------------------------------------------------------------
Initialize an Inline object after parsing it
-------------------------------------------------------------*/
BOOL InlineImp :: Parse (BIFSDecoder *pBifs)
{
	BOOL bSuccess = StreamConsumer :: Parse (pBifs);
	Init ();
	return bSuccess;
}

/*-----------------------------------------------------------------
The Destructor delete the whole sub-scene;
-----------------------------------------------------------------*/
InlineImp :: ~InlineImp ()
{
	m_subScene.SetObject (NULL);
	for (int i = m_clocks.GetLength (); --i >= 0;)
	{	ClockReference *pClock = m_clocks [i];
		delete pClock;
		pClock = NULL;
	}
	//*** YT START (3)
	#ifdef MPGJ
	// Cleanup fake InlineImp nodes which are not cleaned up by REMOVE OD command
	for (i = m_streamAssociation.GetLength (); --i >= 0;) {
		StreamConsumer *pNode = m_streamAssociation [i].m_pNode;
		// all nodes were already unregistered except fake node for MPEGJ
		// so cleanup!
		if(pNode) {
			pNode->ODRemoved();
			delete pNode;
		}
	}
	#endif MPGJ
	//*** YT END (3)
}

/*-------------------------------------------------------------
Initialize an Inline object
-------------------------------------------------------------*/
void InlineImp :: Init ()
{	
	Group *pSubScene = (Group*) m_subScene.CreateNode (1);
	pSubScene -> SetExecutive (GetExecutive ());
	m_subScene.SetObject (pSubScene, this);

	/*-------------------------------------------------------------
	Inline node is open as soon as it is created.
	-------------------------------------------------------------*/
	OpenStream ();
}

/*-------------------------------------------------------------
Return a corresponding IPMP_Descriptor, if available
-------------------------------------------------------------*/
IPMP_DescriptorImp * InlineImp :: GetIPMP_Descriptor (int nIPMPid)
{
	int i = m_IPMPDs.Find (nIPMPid);
	if (i >= 0)
		return m_IPMPDs [i].m_pIPMPD;
	return NULL;
}

/*-------------------------------------------------------------
Execute the RemoveIPMPD command. The IPMP_Descriptor is removed from
the list of IPMP descriptors and all associated systems are notified.
-------------------------------------------------------------*/
void InlineImp :: RemoveIPMPD (int nIPMPid)
{
	int i = m_IPMPDs.Find (nIPMPid);
	if (i >= 0)
	{	for (int j = 0; j < m_IPMPDs [i].m_systems.GetLength (); j++)
			m_IPMPDs [i].m_systems [j] -> RemoveDescriptor (nIPMPid);
		m_IPMPDs.DeleteFrom (i);
	}
}

/*-----------------------------------------------------------------
An update IPMPD command was detected. Locate it in the IPMPD list, or
add it if its not there.
Then notify all attached IPMP systems
-----------------------------------------------------------------*/
void InlineImp :: UpdateIPMPD (IPMP_DescriptorImp *pIPMPD)
{
	int nIPMPid = pIPMPD -> IPMP_DescriptorID;
	int i = m_IPMPDs.Find (nIPMPid);
	if (i < 0)
		(m_IPMPDs += nIPMPid).m_pIPMPD = pIPMPD;
	else
	{	delete m_IPMPDs [i].m_pIPMPD;
		m_IPMPDs [i].m_pIPMPD = pIPMPD;
		for (int j = 0; j < m_IPMPDs [i].m_systems.GetLength (); j++)
		{	const BYTE *pData;
			int nDataSize;
			if (pIPMPD -> IPMPS_Type == 0)
			{	pData = pIPMPD -> URLString;
				nDataSize = pIPMPD -> URLString.GetLength ();
			}
			else
			{	pData = pIPMPD -> IPMP_data;
				nDataSize = pIPMPD -> IPMP_data.GetLength ();
			}
			pIPMPD -> Notify (m_IPMPDs [i].m_systems [j]);
		}
	}
}

/*-------------------------------------------------------------
Return the clock reference of created by the stream whose id is
nClockID. Create one if none exists
-------------------------------------------------------------*/
ClockReference *InlineImp :: GetClock (int nClockID)
{
	for (int i = m_clocks.GetLength (); --i >= 0;)
		if (m_clocks [i] -> GetID () == nClockID)
			return m_clocks [i];
	return m_clocks += new ClockReference (nClockID);
}

/*-----------------------------------------------------------------
A remove ESD command was detected
-----------------------------------------------------------------*/
void InlineImp :: RemoveESD (int nODid, int nESDid)
{
	/*-------------------------------------------------------------
	Find the OD and check whether it contains the required ESD
	-------------------------------------------------------------*/
	ObjectDescriptorImp *pOD = NULL;
	ES_DescriptorImp *pESD = NULL;
	int i = m_streamAssociation.Find (nODid);
	if (i >= 0)
		pOD = m_streamAssociation [i].m_pOD;
	if (pOD == NULL)
	{	GetExecutive () -> Error (TRYING_TO_UPDATE_UNKNOWN_OD);
		return;
	}
	for (i = pOD -> esDescr.GetLength (); --i >= 0;)
	{	pESD = (ES_DescriptorImp *) pOD -> esDescr [i];
		if (pESD -> ES_ID == nESDid)
			break;
	}
	if (i < 0)
	{	GetExecutive () -> Error (TRYING_TO_REMOVE_UNKNOWN_ESD);
		return;
	}

	/*-------------------------------------------------------------
	Tell all the nodes associated to this OD that a the ESD has been
	removed.
	-------------------------------------------------------------*/
	pOD -> esDescr.DeleteFrom (i);
	for (int j = m_streamAssociation.GetLength (); --j >= 0;)
	{	if (nODid == m_streamAssociation [j].m_nODid)
		{	StreamConsumer *pNode = m_streamAssociation [j].m_pNode;
			if (pNode != NULL)
				pNode -> RemoveESD (pESD);
		}
	}
	delete pESD;
}

/*-----------------------------------------------------------------
An Update ESD command was detected
-----------------------------------------------------------------*/
void InlineImp :: UpdateESD (int nODid, ES_DescriptorImp *esd)
{
	/*-------------------------------------------------------------
	Find the OD and add the stream to it (ignore the stream if already
	defined)
	-------------------------------------------------------------*/
	ObjectDescriptorImp *pOD = NULL;
	int i = m_streamAssociation.Find (nODid);
	if (i >= 0)
		pOD = m_streamAssociation [i].m_pOD;
	if (pOD == NULL)
	{	GetExecutive () -> Error (TRYING_TO_UPDATE_UNKNOWN_OD);
		return;
	}
	/*-------------------------------------------------------------
	If this ESD already exists, remove it before entering the new one
	-------------------------------------------------------------*/
	for (i = pOD -> esDescr.GetLength (); --i >= 0;)
	{	if (pOD -> esDescr [i] -> ES_ID == esd -> ES_ID)
		{	RemoveESD (nODid, esd -> ES_ID);
			break;
		}
	}
	pOD -> esDescr += esd;

	/*-------------------------------------------------------------
	Tell all the nodes associated to this OD that a new ESD has been
	added
	-------------------------------------------------------------*/
	for (i = m_streamAssociation.GetLength (); --i >= 0;)
	{	if (nODid == m_streamAssociation [i].m_nODid)
		{	StreamConsumer *pNode = m_streamAssociation [i].m_pNode;
			if (pNode != NULL)
				pNode -> AddESD (esd);
		}
	}
}

/*-----------------------------------------------------------------
A remove OD command was deteced. Remove the OD from the list
-----------------------------------------------------------------*/
void InlineImp :: RemoveOD (int nODid)
{
	ObjectDescriptorImp *pOD = NULL;
	for (int i = m_streamAssociation.GetLength (); --i >= 0;)
	{	
		if (nODid == m_streamAssociation [i].m_nODid)
		{	
			pOD = m_streamAssociation [i].m_pOD;
			StreamConsumer *pNode = m_streamAssociation [i].m_pNode;
			//*** YT START (9)
			#ifdef MPGJ
			// Search MPEGJ stream to remove
			// If found cleanup mpegj stream
			// and delete the fake InlineImp node
			for(int j=0;j < pOD->esDescr.GetLength (); j++) {
				if(pOD->esDescr[j]->decConfigDescr->streamType == ST_MPEGJStream) {
					pNode->ODRemoved();
					delete pNode;
					pNode = NULL;
					break;
				}
			}
			#endif MPGJ
			//*** YT END (9)
			if (pNode != NULL)
			{	
				pNode -> ODRemoved ();
				m_streamAssociation [i].m_pOD = NULL;
			}
			else
			{	m_streamAssociation.DeleteFrom (i);
				return;
			}
		}
	}
	if (pOD == NULL) {
		// *** YT START (10)
		#ifdef MPGJ
		// make root inlineImp remove MPEGJ stream
		// but i don't think it's necessary
		// because MPEGJ stream would inside inline node's namescope
		// and it may need be destroyed when inlineImp is deleted

		// The reason I make an MPEGJ stream associate with root inline
		// node is when InlineImp is destroyed it deletes all subscene and
		// ESs. Also it will shutdown MPEGJ stream. 
		// but i think MPEGJ stream should be shutdown also...

		// so this part should be modified again, i think...
		if(this != GetExecutive()->GetInline()) 
			GetExecutive()->GetInline()->RemoveOD(nODid);
		else
		#endif MPGJ
		// *** YT END (10)
			GetExecutive () -> Error (UNKNOWN_ODID);
	}
	delete pOD;
}

/*-----------------------------------------------------------------
An update OD command was detected. Locate it in the OD list, or
add it if its not there. Once both the OD and the node which consumes
it are parsed, the proper channel(s) and decoder(s) are instantiated.
-----------------------------------------------------------------*/
void InlineImp :: UpdateOD (ObjectDescriptorImp *pOD)
{
	int nODid = pOD -> ObjectDescriptorID;
	pOD -> SetAL (GetAL ());
	pOD -> SetInline (this);
	int i = m_streamAssociation.Find (nODid);
	

	if (i < 0) //*** YT START (4) ***// 
	#ifdef MPGJ
	{

		// Search MPEGJ stream
		// 1. check OD has ES
		// 2. check i'm the root InlineImp
		// 3. associate the fake InlineImpe with MPEGJ stream in the StreamAssoication table
		//	   in the root InlineImp
		for(int j=0;j < pOD->esDescr.GetLength (); j++) {
			if(pOD->esDescr[j]->decConfigDescr->streamType == ST_MPEGJStream) {
				if(this != GetExecutive()->GetInline()) {
					GetExecutive()->GetInline()->UpdateOD(pOD);
					break;
				} 
				else {	
					// Fake node
					InlineImp *pNode = new InlineImp;
					pNode->SetExecutive (GetExecutive());
					pNode->SetInline (this);
					pNode->Init ();
					StreamAssociation& sa = (m_streamAssociation += nODid);
					sa.m_pOD = pOD;
					sa.m_pNode = pNode;
					pNode->SetStream (pOD);
					break;
				}
			}
		}
		if(j==pOD->esDescr.GetLength())
			(m_streamAssociation += nODid).m_pOD = pOD;
	}
	#else
	(m_streamAssociation += nODid).m_pOD = pOD;
	//*** YT END (4) ***//
	#endif MPGJ
	else
	{	/*-------------------------------------------------------------
		If an OD with this ID already exist, remove it before entering
		the new one.
		-------------------------------------------------------------*/
		if (m_streamAssociation [i].m_pOD != NULL)
			RemoveOD (nODid);
		m_streamAssociation [i].m_pOD = pOD;
		StreamConsumer *pNode = m_streamAssociation [i].m_pNode;
		if (pNode)
			pNode -> SetStream (pOD);
	}
	
	
	
	
}

/*-----------------------------------------------------------------
The BIFS decoder parsed a ReplaceScene command
-----------------------------------------------------------------*/
void InlineImp :: ReplaceScene (Root *pScene)
{
	MFGenericNode *pChildren = m_subScene -> GetChildrenField ();
	pChildren -> SetAt (0, pScene, m_subScene, false);
	if (pScene != NULL_OBJECT)
	{	pScene -> ResetTime ();
		GetExecutive () -> ReplaceScene ();
	}
}

/*-----------------------------------------------------------------
A node of type StreamConsumer registers here when it is instantiated
-----------------------------------------------------------------*/
void InlineImp :: RegisterStreamConsumer (StreamConsumer *pNode)
{
	int nODid = pNode -> GetObjectDescriptorID ();
	int i = m_streamAssociation.Find (nODid);
	if (i < 0)
		(m_streamAssociation += nODid).m_pNode = pNode;
	else
	{	m_streamAssociation [i].m_pNode = pNode;
		if (m_streamAssociation [i].m_pOD != NULL)
			pNode -> SetStream (m_streamAssociation [i].m_pOD);
	}
}

/*-----------------------------------------------------------------
A node of type StreamConsumer unregisters here when it is destroyed
-----------------------------------------------------------------*/
void InlineImp :: UnregisterStreamConsumer  (StreamConsumer *pNode)
{
	for (int i = m_streamAssociation.GetLength (); --i >= 0;)
		if (m_streamAssociation [i].m_pNode == pNode)
		{	m_streamAssociation [i].m_pNode = NULL;
			if (m_streamAssociation [i].m_pOD == NULL)
				m_streamAssociation.DeleteFrom (i);
			return;
		}
}


/*-----------------------------------------------------------------
A destructor for a StreamAssociation object
-----------------------------------------------------------------*/

InlineImp :: StreamAssociation :: ~StreamAssociation ()
{
	delete m_pOD;
}

/*-------------------------------------------------------------
Associate an IPMP system to an IPMPDescriptorID
-------------------------------------------------------------*/
void InlineImp :: AssociateIPMPSystem (int nID, IPMPManager * pIPMP)
{
	/*-------------------------------------------------------------
	Find the ID in the list, or create a new list item
	-------------------------------------------------------------*/
	int i = m_IPMPDs.Find (nID);
	if (i >= 0)
		m_IPMPDs [i].m_systems += pIPMP;
	else
		(m_IPMPDs += nID).m_systems += pIPMP;
}

/*-------------------------------------------------------------
The destructor for IPMPDAssociation deletes the descriptor
-------------------------------------------------------------*/
InlineImp :: IPMPDAssociation :: ~IPMPDAssociation ()
{
	delete m_pIPMPD;
}	 

/*-------------------------------------------------------------
Disassociate an IPMP system from a descriptor ID
-------------------------------------------------------------*/
void InlineImp :: RemoveIPMP (int nID, IPMPManager * pIPMP)
{
	int i = m_IPMPDs.Find (nID);
	if (i >= 0)
		m_IPMPDs [i].m_systems -= pIPMP;
}

/*-------------------------------------------------------------
When the OD of an Inline is removed, the entire branch is deleted
-------------------------------------------------------------*/
void InlineImp :: ODRemoved ()
{
	m_subScene.SetObject (NULL);
	StreamConsumer ::ODRemoved ();
}


		