class ObjectDescriptorUpdate extends BaseCommand : bit(8) tag=ObjectDescrUpdateTag {
	ObjectDescriptor OD[1 .. 255];
}

class ObjectDescriptorRemove extends BaseCommand : bit(8) tag=ObjectDescrRemoveTag {
	bit(10) objectDescriptorId[(sizeOfInstance*8)/10];
}

class ES_DescriptorUpdate extends BaseCommand : bit(8) tag=ES_DescrUpdateTag {
	bit(10) objectDescriptorId;
	ES_Descriptor ESD[1 .. 30];
}

class ES_DescriptorRemove extends BaseCommand : bit(8) tag=ES_DescrRemoveTag {
	bit(10) objectDescriptorId;
	aligned (8) bit(16) ES_ID[1..30];
}

class IPMP_DescriptorUpdate extends BaseCommand : bit(8) tag=IPMP_DescrUpdateTag {
	IPMP_Descriptor	ipmpDescr[1..255];
}

class IPMP_DescriptorRemove extends BaseCommand : bit(8) tag=IPMP_DescrRemoveTag {
	bit(8) IPMP_DescriptorID[1..255];
}

class ObjectDescriptor extends BaseDescriptor : bit(8) tag=ObjectDescrTag {
	bit(10) ObjectDescriptorID;
	bit(1) URL_Flag;
	const bit(5) reserved=0b1111.1;
	if (URL_Flag) {
		bit(8) URLlength;
		bit(8) URLstring[URLlength];
	} else {
		ES_Descriptor esDescr[1 .. 30];
		OCI_Descriptor ociDescr[0 .. 255];
		IPMP_DescriptorPointer ipmpDescrPtr[0 .. 255];
	}
	ExtensionDescriptor extDescr[0 .. 255];
}

class InitialObjectDescriptor extends BaseDescriptor : bit(8) tag=InitialObjectDescrTag {
	bit(10) ObjectDescriptorID;
	bit(1) URL_Flag;
	bit(1) includeInlineProfileLevelFlag;
	const bit(4) reserved=0b1111;
	if (URL_Flag) {
		bit(8) URLlength;
		bit(8) URLstring[URLlength];
	} else {
		bit(8) ODProfileLevelIndication;
		bit(8) sceneProfileLevelIndication;
		bit(8) audioProfileLevelIndication;
		bit(8) visualProfileLevelIndication;
		bit(8) graphicsProfileLevelIndication;
		ES_Descriptor ESD[1 .. 30];
		OCI_Descriptor ociDescr[0 .. 255];
		IPMP_DescriptorPointer ipmpDescrPtr[0 .. 255];
	}
	ExtensionDescriptor extDescr[0 .. 255];
}

class ES_Descriptor extends BaseDescriptor : bit(8) tag=ES_DescrTag {
	bit(16) ES_ID;
	bit(1) streamDependenceFlag;
	bit(1) URL_Flag;
	const bit(1) reserved=1;
	bit(5) streamPriority;
	if (streamDependenceFlag)
		bit(16) dependsOn_ES_ID;
	if (URL_Flag) {
		bit(8) URLlength;
		bit(8) URLstring[URLlength];
	}
	DecoderConfigDescriptor decConfigDescr;
	SLConfigDescriptor slConfigDescr;
	IPI_DescrPointer ipiPtr[0 .. 1];
	IP_IdentificationDataSet ipIDS[0 .. 255];
	IPMP_DescriptorPointer ipmpDescrPtr[0 .. 255];
	LanguageDescriptor langDescr[0 .. 255];
	QoS_Descriptor qosDescr[0 .. 1];
	RegistrationDescriptor regDescr[0 .. 1];
	ExtensionDescriptor extDescr[0 .. 255];
}

class DecoderConfigDescriptor extends BaseDescriptor : bit(8) tag=DecoderConfigDescrTag {
	bit(8) objectTypeIndication;
	bit(6) streamType;
	bit(1) upStream;
	const bit(1) reserved=1;
	bit(24) bufferSizeDB;
	bit(32) maxBitrate;
	bit(32) avgBitrate;
	DecoderSpecificInfo decSpecificInfo[0 .. 1];
}

abstract class DecoderSpecificInfo extends BaseDescriptor : bit(8) tag=DecSpecificInfoTag {
	// empty. To be filled by classes extending this class.
}

class JPEG_DecoderConfig extends DecoderSpecificInfo : bit(8) tag=DecSpecificInfoTag {{
	int(16) headerLength;
	int(16) Xdensity;
	int(16) Ydensity;
	int(8) numComponents;
}

abstract class IP_IdentificationDataSet extends BaseDescriptor : bit(8) tag=ContentIdentDescrTag..SupplContentIdentDescrTag {
	// empty. To be filled by classes extending this class.
}

class ContentIdentificationDescriptor extends IP_IdentificationDataSet : bit(8) tag=ContentIdentDescrTag {
	const bit(2)	compatibility=0;
	bit(1)		contentTypeFlag;
	bit(1)		contentIdentifierFlag;
	bit(1) 		protectedContent;
	bit(3)		reserved = 0b111;
	if (contentTypeFlag)
		bit(8)	contentType;
	if (contentIdentifierFlag) {
		bit(8)	contentIdentifierType;
		bit(8)	contentIdentifier[sizeOfInstance-2-contentTypeFlag];
	}
}

class SupplementaryContentIdentificationDescriptor extends IP_IdentificationDataSet : bit(8) tag= SupplContentIdentDescrTag {
	bit(24)	languageCode;
	bit(8)	supplContentIdentifierTitleLength;
	bit(8)	supplContentIdentifierTitle[supplContentIdentifierTitleLength];
	bit(8)	supplContentIdentifierValueLength;
	bit(8)	supplContentIdentifierValue[supplContentIdentifierValueLength];
}

class IPI_DescrPointer extends BaseDescriptor : bit(8) tag=IPI_DescrPointerTag {
	bit(16) IPI_ES_Id;
}

class IPMP_DescriptorPointer extends BaseDescriptor : bit(8) tag=IPMP_DescrPointerTag {
	bit(8) IPMP_DescriptorID;
}

class IPMP_Descriptor() extends BaseDescriptor :	bit(8) IPMP_DescrTag {
	bit(8) IPMP_DescriptorID;
	unsigned int(16)	 IPMPS_Type;
	if (IPMPS_Type == 0) {
		bit(8) URLString[sizeOfInstance-3];
	} else {
		bit(8) IPMP_data[sizeOfInstance-3];
	}
}

class QoS_Descriptor extends BaseDescriptor : bit(8) tag=QoS_DescrTag {
	bit(8) predefined;
	if (predefined==0) {
		QoS_Qualifier qualifiers[];
	}
}

abstract class QoS_Qualifier extends ExpandableBaseClass : bit(8) tag=0x01..0xff {
	// empty. To be filled by classes extending this class.
}

class QoS_Qualifier_MAX_DELAY extends QoS_Qualifier : bit(8) tag=0x01 {
	unsigned int(32) MAX_DELAY;
}

class QoS_Qualifier_PREF_MAX_DELAY extends QoS_Qualifier : bit(8) tag=0x02 {
	unsigned int(32) PREF_MAX_DELAY;
}

class QoS_Qualifier_LOSS_PROB extends QoS_Qualifier : bit(8) tag=0x03 {
	double(32) LOSS_PROB;
}

class QoS_Qualifier_MAX_GAP_LOSS extends QoS_Qualifier : bit(8) tag=0x04 {
	unsigned int(32) MAX_GAP_LOSS;
}

class QoS_Qualifier_MAX_AU_SIZE extends QoS_Qualifier : bit(8) tag=0x41 {
	unsigned int(32) MAX_AU_SIZE;
}

class QoS_Qualifier_AVG_AU_SIZE extends QoS_Qualifier : bit(8) tag=0x42 {
	unsigned int(32) AVG_AU_SIZE;
}

class QoS_Qualifier_MAX_AU_RATE extends QoS_Qualifier : bit(8) tag=0x43 {
	unsigned int(32) MAX_AU_RATE;
}

abstract class ExtensionDescriptor extends BaseDescriptor
: bit(8) tag = ExtDescrTagStartRange .. ExtDescrTagEndRange {
	// empty. To be filled by classes extending this class.
}

class RegistrationDescriptor extends BaseDescriptor  : bit(8) tag=RegistrationDescrTag {
	bit(32) formatIdentifier;
	bit(8) additionalIdentificationInfo[sizeOfInstance-4];
}

abstract class OCI_Descriptor extends BaseDescriptor : bit(8) tag= OCIDescrTagStartRange .. OCIDescrTagEndRange {
	// empty. To be filled by classes extending this class.
}

class ContentClassificationDescriptor extends OCI_Descriptor 
: bit(8) tag= ContentClassificationDescrTag {
	bit(32) classificationEntity; 
	bit(16) classificationTable;
	bit(8) contentClassificationData[sizeOfInstance-6];
}

class KeyWordDescriptor extends OCI_Descriptor : bit(8) tag=KeyWordDescrTag {
	int i;
	bit(24) languageCode;  
	bit(1) isUTF8_string;
	aligned(8) unsigned int(8) keyWordCount;
	for (i=0; i<keyWordCount; i++) {
		unsigned int(8) keyWordLength[[i]]; 
		if (isUTF8_string) then {
			bit(8) keyWord[[i]][keyWordLength[i]];
		} else {
			bit(16) keyWord[[i]][keyWordLength[i]];
		}
	}
}

class RatingDescriptor extends OCI_Descriptor : bit(8) tag=RatingDescrTag {
	bit(32) ratingEntity; 
	bit(16) ratingCriteria;
	bit(8)  ratingInfo[sizeOfInstance-6];
}

class LanguageDescriptor extends OCI_Descriptor : bit(8) tag=LanguageDescrTag {
	bit(24) languageCode;
}

class ShortTextualDescriptor extends OCI_Descriptor : bit(8) tag=ShortTextualDescrTag {
	bit(24) languageCode;
	bit(1) isUTF8_string;
	aligned(8) unsigned int(8) nameLength;
	if (isUTF8_string) then {
		bit(8) eventName[nameLength];
		unsigned int(8) textLength;
		bit(8) eventText[textLength]; 
	} else {
		bit(16) eventName[nameLength];
		unsigned int(8) textLength;
		bit(16) eventText[textLength];
	}
}

class ExpandedTextualDescriptor extends OCI_Descriptor : bit(8) tag=ExpandedTextualDescrTag {
	int i;
	bit(24) languageCode;
	bit(1) isUTF8_string;
	aligned(8) unsigned int(8) itemCount;
	for (i=0; i<itemCount; i++){
		unsigned int(8) itemDescriptionLength[[i]];
		if (isUTF8_string) then {
			bit(8) itemDescription[[i]][itemDescriptionLength[i];
		} else {
			bit(16) itemDescription[[i]][itemDescriptionLength[i]];
		}
		unsigned int(8) itemLength[[i]];
		if (isUTF8_string) then {
			bit(8) itemText[[i]][itemLength[i]];
		} else {
			bit(16) itemText[[i]][itemLength[i]];
		}
	}
	unsigned int(8) textLength;
	int nonItemTextLength=0;
	while( textLength == 255 ) {
		nonItemTextLength += textLength;
		bit(8) textLength;
	}
	nonItemTextLength += textLength;
	if (isUTF8_string) then {
		bit(8) nonItemText[nonItemTextLength];
	} else {
		bit(16) nonItemText[nonItemTextLength];
	}
}

class ContentCreatorNameDescriptor extends OCI_Descriptor 
: bit(8) tag= ContentCreatorNameDescrTag {
	int i;
	unsigned int(8) contentCreatorCount;
	for (i=0; i<contentCreatorCount; i++){
		bit(24) languageCode[[i]];
		bit(1) isUTF8_string[[i]];
		aligned(8) unsigned int(8) contentCreatorLength[[i]];
		if (isUTF8_string[[i]]) then {
			bit(8) contentCreatorName[[i]][contentCreatorLength[i]];
		} else {
			bit(16) contentCreatorName[[i]][contentCreatorLength[i]];
		}
	}
}

class ContentCreationDateDescriptor extends OCI_Descriptor 
: bit(8) tag= ContentCreationDateDescrTag {
	bit(40) contentCreationDate;
}

class OCICreatorNameDescriptor extends OCI_Descriptor 
: bit(8) tag=OCICreatorNameDescrTag {
	int i;
	unsigned int(8) OCICreatorCount;
	for (i=0; i<OCICreatorCount; i++) {
		bit(24) languageCode[[i]];
		bit(1) isUTF8_string;
		aligned(8) unsigned int(8) OCICreatorLength[[i]];
		if (isUTF8_string) then {
			bit(8) OCICreatorName[[i]][OCICreatorLength];
		 } else {
			bit(16) OCICreatorName[[i]][OCICreatorLength];
		}
	}
}

class OCICreationDateDescriptor extends OCI_Descriptor 
: bit(8) tag=OCICreationDateDescrTag {
	bit(40) OCICreationDate;
}

class BIFSConfig extends DecoderSpecificInfo : bit(8) tag=DecSpecificInfoTag {
	unsigned int(5) nodeIDbits;
	unsigned int(5) routeIDbits;
	bit(1) isCommandStream;
	if(isCommandStream) {
		bit(1) pixelMetric;
		bit(1) hasSize;
		if(hasSize) {
			unsigned int(16) pixelWidth;
			unsigned int(16) pixelHeight;
		}
	}
	else {
		bit(1) 			randomAccess;
		AnimationMask 	animMask();
	}
}