/***********************************************************************
This software module was originally developed by Zvi Lifshitz
(Triton R&D Ltd.) in the course of development of the MPEG-4 Systems
(ISO/IEC 14496-1) standard. This software module is an implementation of
a part of one or more MPEG-4 Systems (ISO/IEC 14496-1) tools as
specified by the MPEG-4 Systems (ISO/IEC 14496-1) standard. ISO/IEC
gives users of the MPEG-4 Systems (ISO/IEC 14496-1) free license to
this software module or modifications thereof for use in hardware or
software products claiming conformance to the MPEG-4 Systems (ISO/IEC
14496-1). Those intending to use this software module in hardware or
software products are advised that its use may infringe existing patents.
The original developer of this software module and his/her company, the
subsequent editors and their companies, and ISO/IEC have no liability
for use of this software module or modifications thereof in an
implementation. Copyright is not released for non MPEG-4 Systems
(ISO/IEC 14496-1) conforming products. Triton R&D Ltd. retains full right
to use the code for his/her own purpose, assign or donate the code to a
third party and to inhibit third parties from using the code for non
MPEG-4 Systems (ISO/IEC 14496-1) conforming products. This copyright
notice must be included in all copies or derivative works.
Copyright  1997.
************************************************************************

For more information or to receive an up to date version of this module,
you may call Zvi Lifshitz at zvil@csi.com.
----------------------------------------------------------------------*/

// -*- c++ -*-

#if !defined(_NodeFld_h)	// Sentry, use file only if it's not already included.
#define _NodeFld_h

/*-------------------------------------------------------------------------
Project Im1Player
INTERNATIONAL ORGANIZATION FOR STANDARDIZATION
ISO/IEC JTC1/SC29/WG11

SUBSYSTEM:  BIFS Decoder/ Compositor
FILE:       NodeFld.h
AUTHOR:     Zvi Lifshitz (Triton R&D Ltd.)
OVERVIEW:   Declarations of NodeField and derivations.
-------------------------------------------------------------------------*/
#include "ztl.h"
#include "ztypes.h"
#include "MediaObj.h"
#include "FldTypes.h"

//#ifndef BIFSENC	// SCRIPT
//#include "..\script\encoder\ScriptDefs.h"
//#endif

class Quantizer;

#ifdef PLAYER
#include "../Animation/AnimationQuantizer.h" // MBS: added
class AnimationStreamDecoder; // MBS: definition
#endif

#ifdef BIFSENC
class BIFSEncoder;
#endif

#ifdef MPGJ // SCENE API Integration
class Listener
{
public:
	virtual void notify( NodeField *source ) = 0;
	virtual ~Listener() {}
};
#endif MPGJ // SCENE API Integration

/*-------------------------------------------------------------------------
- NodeField is the base class all kind of node fields
- NodeField objects can parse themselves from BIFS.
- Each object must override the Parse virtual function.
-------------------------------------------------------------------------*/
class NodeField
{
	/*-------------------------------------------------------------
	Methods used by both decoder and encoder
	-------------------------------------------------------------*/
public:
	NodeField (FIELD_TYPE fieldType) :
		m_pFirstRoute (NULL), m_fieldType (fieldType) 
#ifdef MPGJ
	, m_listeners(NULL)
#endif MPGJ
	{}

	/*-------------------------------------------------------------
	Methods used by the decoder
	-------------------------------------------------------------*/
#ifdef PLAYER
public:
    // added by MBS for animation streams
    virtual void parseAnimationMask(AnimationStreamDecoder *pAS,int anim);
    virtual void animate(AnimationStreamDecoder *pAS,BOOL intra);
#endif

#ifdef BIFSDEC
public:
	virtual ~NodeField ();
	virtual BOOL Parse (
		BIFSDecoder *pBifs, 
		MediaObject *pNode, 
		Quantizer *pQuant = NULL
	) = 0;
	virtual NodeField *Clone (NodeField *pField, MediaObject *pParent) = 0;
	virtual void Dequantize (Quantizer * /*pQuant*/) {}
	virtual void Clip (Quantizer * /*pQuant*/) {}
	virtual void Copy (
		NodeField &nodeField,
		MediaObject *pParent,
		bool bIsFirst = true
	) = 0;
	void LinkRoute (Route *pRoute);
	void UnlinkRoute (Route *pRoute);
	void OnFieldUpdate (bool bIsFirst = true);
	FIELD_TYPE GetType() const {return m_fieldType;}
#ifdef MPGJ		// SCENE API Integration
	void AddListener( Listener *l );
	void RemoveListener( Listener *l );
#endif MPGJ		// SCENE API Integration
#endif // BIFSDEC

	/*-------------------------------------------------------------
	Methods used by the encoder
	-------------------------------------------------------------*/
#ifdef BIFSENC
public:
	virtual BOOL Encode (
		BIFSEncoder *pBifs, 
		Quantizer *pQuant = NULL
	) = 0;
	virtual BOOL EncodeElement (BIFSEncoder *pBifs, Quantizer *pQuant = NULL);
#endif // BIFSENC

private:
	Route *m_pFirstRoute;
	FIELD_TYPE m_fieldType;
#ifdef MPGJ // SCENE API Integration
	ZArray<Listener*> *m_listeners;
#endif MPGJ // SCENE API Integration

#ifdef PLAYER
protected:
    // MBS for animation streams 
    AnimationQuantizer qp;
#endif

#ifdef BIFSCAR
public:
	virtual BOOL Encode (
		CarouselEncoder *pBifs, 
		Quantizer *pQuant = NULL
	) = 0;
	bool NeedsEncoding () const {return m_bNeedsEncoding;}
	void SetNeedsEncoding (bool flag) {m_bNeedsEncoding = flag;}

private:
	bool m_bNeedsEncoding;
#endif
};

/*-----------------------------------------------------------------
A class template for all SF fields, except of SFNode (or specific type
SFNode fields).  T can be the classes Int, Float, Bool, Color, Time,
Rotation, ZString, Vec2f and Vec3f which are defined in ZTL. 
E.g. SFInt  would be declared as SF<int>, and SFRotation as
SF<Rotation>.
-----------------------------------------------------------------*/
template <class T> class SF : public NodeField
{
public:
	SF<T> () : m_value (0), NodeField (s_fieldType) {}
	SF<T> (T value) : m_value (value), NodeField (s_fieldType) {}
	operator T () const {return m_value;}

#ifdef PLAYER
    // added by MBS for animation streams
    virtual void parseAnimationMask(AnimationStreamDecoder *pAS,int anim);
    virtual void animate(AnimationStreamDecoder *pAS,BOOL intra);
#endif

#ifdef BIFSDEC
	/*-----------------------------------------------------------------
	Copy is activated in case of Routes. We assume that
	the nodeField argument is always of the same type as this, therefor
	allow for deliberate downcasting. This assumption is dangerous.
	Commercial application should implement safe casting or proper
	conversion mechanism.
	-----------------------------------------------------------------*/
	virtual void Copy (NodeField &nodeField, MediaObject *, bool bIsFirst)
	{
		m_value = ((SF<T>&) nodeField).m_value;
		OnFieldUpdate (bIsFirst);
	}
	virtual BOOL Parse (
		BIFSDecoder *pBifs, 
		MediaObject *pNode, 
		Quantizer *pQuant = NULL
	);
	virtual NodeField *Clone (NodeField *pField, MediaObject *)
	{
		if (pField == NULL)
			pField = new SF<T>;
		((SF<T>*) pField) ->m_value = m_value;
		return pField;
	}
	T operator = (const T value)
	{
		m_value = value;
		OnFieldUpdate (this);
		return value;
	}
	T GetValue() {return m_value;} // SCRIPT
#endif

#ifdef BIFSENC
	virtual BOOL Encode (
		BIFSEncoder *pBifs, 
		Quantizer *pQuant = NULL
	);
#endif

#ifdef ENCODER
	T operator = (const T value) {return m_value = value;}
#endif

#ifdef BIFSCAR
	virtual BOOL Encode (
		CarouselEncoder *pBifs, 
		Quantizer *pQuant = NULL
	);
#endif
protected:
	T m_value;
	static FIELD_TYPE s_fieldType;
};

/*-----------------------------------------------------------------
SFBool is a synonym for SF<Bool>
-----------------------------------------------------------------*/
class SFBool : public SF<Bool>
{
public:
	SFBool (bool b = false) {m_value = b;}
	operator bool () const {return m_value;}
};
	
/*-----------------------------------------------------------------
SFString is an extenstion of SF<ZString>.
-----------------------------------------------------------------*/
class SFString : public SF<ZString>
{
public:
	SFString () {}
	SFString (LPCSTR szString) : SF<ZString> (szString) {}
	operator const ZString & () const {return m_value;}
	const ZString &operator = (const ZString &value)
	{
		SF<ZString> :: operator = (value);
		return value;
	}
	int GetLength () const {return m_value.GetLength ();}
	operator LPCSTR () const {return m_value;}

#ifdef BIFSENC
	void EncodeString (BIFSEncoder *pBifs);
#endif

#ifdef BIFSCAR
	void EncodeString (CarouselEncoder *pBifs);
#endif
};

/*-----------------------------------------------------------------
Url
-----------------------------------------------------------------*/
struct Url
{
	int ODid;
	SFString urlValue;

	Url (int nODid = -1) : ODid (nODid) {}
	operator int () const {return ODid;}
	Url &operator = (const Url &url)
	{
		ODid = url.ODid;
		urlValue = url.urlValue;
		return *this;
	}
};

/*-----------------------------------------------------------------
SFUrl stores either URL or objectID.
-----------------------------------------------------------------*/
class SFUrl : public SF<Url>
{
public:
	int operator = (const int value)
	{
		return m_value.ODid = value;
	}
	operator int () const {return m_value.ODid;}
};

class BifsFactory
{
public:
	virtual MediaObject *CreateNode () = 0;
};

template <class T> class BifsNodeFactory : public BifsFactory
{
	virtual MediaObject *CreateNode () {return new T;}

public:
	BifsNodeFactory ()
	{
		SFGenericNode :: StoreFactory (T :: GetCode (), this);
	}
};

/*-----------------------------------------------------------------
This is a base class for all fields that are themselves nodes.  Some
fields may contain any node type, so they are defined as SFGenericNode,
but usually only a small amount of node types are allowed for a given
nodeField.  For this purpose, the nodes are classified according to Node
Data Types.  A Node Data Type is implemented by an object derived from
SFGenericNode.
-----------------------------------------------------------------*/
class SFGenericNode : public NodeField
{
private:
	virtual bool IsTypeOK (MediaObject *) const {return true;}
	virtual int GetNodeTableSize () const;
	virtual int GetV2ExtSize () const {return 0;}

public:
	SFGenericNode (BOOL bIsSceneRoot = FALSE) :
		NodeField (SFNODE),
		m_pNode (NULL),
		m_bIsSceneRoot (bIsSceneRoot)
	{}
	virtual ~SFGenericNode ();
	virtual MediaObject *CreateNode (int nodeType);
	void SetParent (MediaObject *pNode) {m_pParent = pNode;}
	static void StoreFactory (int nCode, BifsFactory *pFactory);
	MediaObject *GetNode () const {return m_pNode;}
	operator MediaObject * () const {return m_pNode == NULL ? NULL : m_pNode -> GetRenderingNode ();}
	MediaObject *operator -> () const {return m_pNode == NULL ? NULL : m_pNode -> GetRenderingNode ();}

#ifdef BIFSDEC
	/*-----------------------------------------------------------------
	Assigment operator activated in case of Routes. We assume that
	the nodeField argument is always of the same type as this, therefor
	allow for deliberate downcasting. This assumption is dangerous.
	Commercial application should implement safe casting or proper
	conversion mechanism.
	-----------------------------------------------------------------*/
public:
	MediaObject *SetObject (MediaObject *node, MediaObject *pParent = NULL, bool bIsFirst = true);
	void ReplaceNode (MediaObject *pNew);
	virtual BOOL Parse (
		BIFSDecoder *pBifs, 
		MediaObject *pNode, 
		Quantizer *pQuant = NULL
	);
	virtual NodeField *Clone (NodeField *pField, MediaObject *pParent);
	BOOL ParseNodeField (
		BIFSDecoder *pBifs, 
		MediaObject *pNode,			// the node this field belongs to
		NodeField *pContainer,		// a pointer to the container field
		BOOL bISMF					// TRUE when the field is an MF field
	);
	void RemoveNode () {m_pNode = 0;}

private:
	virtual void Copy (NodeField &nodeField, MediaObject *pParent, bool bIsFirst)
	{
		SetObject (((SFGenericNode&) nodeField).GetNode (), pParent, bIsFirst);
	}

#endif	//	BIFSDEC

#ifdef BIFSENC
public:
	virtual BOOL Encode (
		BIFSEncoder *pBifs, 
		Quantizer *pQuant = NULL
	);

protected:
	virtual MediaObject *CreateNode (LPCSTR szNodeName, int &rNodeType);

#endif	// BIFSENC

#ifndef BIFSDEC
public:
	MediaObject *operator = (MediaObject *node) {return m_pNode = node;}
#endif	// BIFSDEC

#ifdef BIFSCAR
public:
	virtual int GetNodeType (int) const {return 0;}
	virtual BOOL Encode (
		CarouselEncoder *pBifs, 
		Quantizer *pQuant = NULL
	);
#endif

private:
	BOOL m_bIsSceneRoot;
	static BifsFactory *m_bifsFactory [];
	MediaObject *m_pParent;

protected:
	MediaObject *m_pNode;
};

/*-----------------------------------------------------------------
A class template for all fields derived from SFGenericNode, when the
node is of a single known type.  E.g. SFAppearance2Dnode would be
declared as SFNode<Appearance2D>.
-----------------------------------------------------------------*/
template <class T> class SFNode : public SFGenericNode
{
	/*-------------------------------------------------------------
	Methods used by both the decoder and the encoder
	-------------------------------------------------------------*/
private:
	virtual int GetNodeTableSize () const {return 1;}
	virtual bool IsTypeOK (MediaObject *pNode) const;

public:
	virtual MediaObject *CreateNode (int);
	T* operator -> () const
	{	
		return (T*) (m_pNode == NULL ? NULL : m_pNode -> GetRenderingNode ());
	}
	operator T* () const
	{
		return (T*) (m_pNode == NULL ? NULL : m_pNode -> GetRenderingNode ());
	}

	/*-------------------------------------------------------------
	Methods used by the decoder
	-------------------------------------------------------------*/
#ifdef BIFSDEC
private:
	virtual NodeField *Clone (NodeField *pField, MediaObject *pParent)
	{
		if (pField == NULL)
			pField = new SFNode<T>;
		return SFGenericNode :: Clone (pField, pParent);
	}

public:
	T* ParseNode (BIFSDecoder *pBifs, MediaObject *pParent);
#endif

	/*-------------------------------------------------------------
	Methods used by the encoder
	-------------------------------------------------------------*/
#ifdef BIFSENC
public:
	MediaObject *operator = (MediaObject *node) {return m_pNode = node;}

private:
	virtual MediaObject *CreateNode (LPCSTR szNodeName, int &rNodeType);
#endif
};

/*-----------------------------------------------------------------
For every Node Data Type which includes more than one nodeType,
an SF...Node class is defined.
-----------------------------------------------------------------*/
//MBS 110498: added new node data types and remove old ones
typedef enum
{
  NT_2D, NT_3D, NT_APPEARANCE, NT_AUDIO, NT_BACKGROUND2D, NT_BACKGROUND3D,
  NT_COLOR, NT_COORDINATE, NT_COORDINATE2D, NT_EXPRESSION, NT_FAP, NT_FDP,
  NT_FIT, NT_FACEDEFMESH, NT_FACEDEFTABLES, NT_FACEDEFTRANSFORM, NT_FOG, 
  NT_FONTSTYLE, NT_GEOMETRY, NT_LINEPROPERTIES, NT_MATERIAL, NT_NAVIGATIONINFO,
  NT_NORMAL, NT_STREAMING, NT_TEXTURECOORDINATE, NT_TEXTURE,
  NT_TEXTURETRANSFORM, NT_TOPNODE, NT_VIEWPOINT, NT_VISEME, NT_VIEWPORT,
  NT_WORLDNODE
} NodeType;

template <NodeType type> class SFNodeType : public SFGenericNode
{
	/*-------------------------------------------------------------
	Methods used by the decoder and the encoder
	-------------------------------------------------------------*/
#ifdef MPGJ
public: //--awm 991208 added two public methods for MPEGJ
	virtual int GetNodeType (int nodeCode) const; //--awm 991208 use this method in MPEGJ
	virtual MediaObject *CreateNode (int nodeType) //--awm 991208 use this method in MPEGJ

#endif MGPJ

#ifndef MPGJ
private:
	virtual MediaObject *CreateNode (int nodeType)
#endif
	{
		if (--nodeType < 0)			// -- added for FDIS
			return NULL;
		if (nodeType >= GetNodeTableSize ())
			return NULL;
		return SFGenericNode :: CreateNode (m_nodeTable [nodeType]);
	}
	virtual int GetNodeTableSize () const;
	virtual int GetV2ExtSize () const;
	virtual bool IsTypeOK (MediaObject *pNode) const
	{
		for (int i = GetNodeTableSize (); --i >= 0;)
			if (pNode -> GetNodeCode () == m_nodeTable [i])
				return true;
		return false;
	}

public:
	SFNodeType (BOOL bIsSceneRoot = FALSE) : SFGenericNode (bIsSceneRoot) {}

	/*-------------------------------------------------------------
	Methods used by the decoder
	-------------------------------------------------------------*/
#ifdef BIFSDEC
	virtual NodeField *Clone (NodeField *pField, MediaObject *pParent)
	{
		if (pField == NULL)
			pField = new SFNodeType<type>;
		return SFGenericNode :: Clone (pField, pParent);
	}
#endif

	/*-------------------------------------------------------------
	Methods used by the encoder
	-------------------------------------------------------------*/
#ifdef BIFSENC
private:
	virtual MediaObject *CreateNode (LPCSTR szNodeName, int &rNodeType)
	{
		MediaObject *pNode;

		for (int nCode = 0; nCode < GetNodeTableSize (); nCode ++)
		{	pNode = CreateNode (nCode + 1);
			if (pNode && !stricmp (pNode -> GetName (), szNodeName))
			{	rNodeType = nCode + 1;		// +1 added for FDIS
				return pNode;
			}
			delete pNode;
		}
		pNode = SFGenericNode:: CreateNode (szNodeName, rNodeType);
		rNodeType = -1;
		return pNode;
	}
#endif	// BIFSENC

#ifdef BIFSCAR
	virtual int GetNodeType (int nodeType) const;
#endif

private:
	static const int m_nodeTable [];
};

//MBS 110498: removed old node data types, replaced by new ones
typedef SFNodeType<NT_2D> SF2DNode;
typedef SFNodeType<NT_3D> SF3DNode;
typedef SFNodeType<NT_AUDIO> SFAudioNode;
typedef SFNodeType<NT_GEOMETRY> SFGeometryNode;
typedef SFNodeType<NT_MATERIAL> SFMaterialNode;
typedef SFNodeType<NT_STREAMING> SFStreamingNode;
typedef SFNodeType<NT_TEXTURE> SFTextureNode;
typedef SFNodeType<NT_TOPNODE> SFTopNode;
typedef SFNodeType<NT_VIEWPORT> SFViewportNode;
typedef SFNodeType<NT_WORLDNODE> SFWorldNode;

/*-----------------------------------------------------------------
A base class for all MF... classes
-----------------------------------------------------------------*/
class MField : public NodeField
{
	/*-------------------------------------------------------------
	Methods used by the decoder and the encoder
	-------------------------------------------------------------*/
public:
	MField (FIELD_TYPE fieldType) : NodeField (fieldType) {}
	virtual int GetLength () const = 0;
	virtual void Clear () = 0;

	/*-------------------------------------------------------------
	Methods used by the decoder
	-------------------------------------------------------------*/
#ifdef BIFSDEC
	virtual BOOL Parse (
		BIFSDecoder *pBifs, 
		MediaObject *pNode, 
		Quantizer *pQuant = NULL
	);
	virtual void DeleteFrom (int index, BOOL bRoute = TRUE) = 0;
	virtual BOOL AppendElement (
		BIFSDecoder *pBifs, 
		MediaObject *pNode, 
		Quantizer *pQuant = NULL
	) = 0;
	virtual BOOL InsertElement (
		BIFSDecoder *pBifs, 
		MediaObject *pNode, 
		int nIndex,
		Quantizer *pQuant = NULL
	) = 0;
	virtual BOOL ReplaceElement (
		BIFSDecoder *pBifs, 
		MediaObject *pNode, 
		int nIndex,
		Quantizer *pQuant = NULL
	) = 0;
#endif	//	BIFSDEC

	/*-------------------------------------------------------------
	Methods used by the encoder
	-------------------------------------------------------------*/
#ifdef BIFSENC
public:
	virtual BOOL Encode (
		BIFSEncoder *pBifs, 
		Quantizer *pQuant = NULL
	);
	virtual BOOL EncodeElement (BIFSEncoder *pBifs, Quantizer *pQuant = NULL) = 0;
#endif		// BIFSINC

#ifdef BIFSCAR
public:
	virtual BOOL Encode (
		CarouselEncoder *pBifs, 
		Quantizer *pQuant = NULL
	);

private:
	virtual BOOL EncodeElement (
		int index,
		CarouselEncoder *pBifs,
		Quantizer *pQuant = NULL
	) = 0;
#endif

#ifdef PLAYER
    /* added by MBS for animaion streams, special case of MField:
       isTotal: TRUE if all fields are animated
       idx    : else, we specify the animated indexes
       */
protected:
    BOOL isTotal; 
    ZArray<int> idx;
#endif
};

/*-----------------------------------------------------------------
A class template for all MF fields, except of MFNode (or specific type
MFNode fields).  T can be int, float, BOOL, or the classes as Color,
Time, Rotation, ZString, Vec2f and Vec3f which are defined in ZTL.
-----------------------------------------------------------------*/
template <class T> class MF : public MField
{
	/*-------------------------------------------------------------
	Methods used by the decoder and the encoder
	-------------------------------------------------------------*/
public:
	MF<T> () : MField (s_fieldType) {}
	const T &operator [] (int index) const {return m_array [index];}
	virtual int GetLength () const {return m_array.GetLength ();}
	T operator += (T t) {return m_array += t;}
	BOOL operator -= (T t) {return m_array -= t;}
	virtual void Clear () {m_array.Clear();}

	/*-------------------------------------------------------------
	Methods used by the decoder
	-------------------------------------------------------------*/
#ifdef PLAYER
public:
    // added by MBS for animation streams
    virtual void parseAnimationMask(AnimationStreamDecoder *pAS,int anim);
    virtual void animate(AnimationStreamDecoder *pAS,BOOL intra);
#endif

#ifdef BIFSDEC
private:
	virtual BOOL AppendElement (
		BIFSDecoder *pBifs, 
		MediaObject *pNode, 
		Quantizer *pQuant = NULL
	)
	{
		SF<T> m_element;
		if (!m_element.Parse (pBifs, pNode, pQuant))
			return FALSE;
		m_array += m_element;
		return TRUE;
	}

	virtual BOOL InsertElement (
		BIFSDecoder *pBifs, 
		MediaObject *pNode, 
		int nIndex,
		Quantizer *pQuant = NULL
	)
	{
		SF<T> m_element;
		if (!m_element.Parse (pBifs, pNode, pQuant))
			return FALSE;
		m_array.InsertAt (nIndex, m_element);
		return TRUE;
	}

	virtual BOOL ReplaceElement (
		BIFSDecoder *pBifs, 
		MediaObject *pNode, 
		int nIndex,
		Quantizer *pQuant = NULL
	)
	{
		SF<T> m_element;
		if (!m_element.Parse (pBifs, pNode, pQuant))
			return FALSE;
		m_array.SetAt (nIndex, m_element);
		return TRUE;
	}
	virtual NodeField *Clone (NodeField *pField, MediaObject *)
	{
		if (pField == NULL)
			pField = new MF<T>;
		for (int i = 0; i < GetLength (); i++)
			((MF<T> *) pField) -> m_array += m_array [i];
		return pField;
	}

	/*-----------------------------------------------------------------
	Assigment operator activated in case of Routes. We assume that
	the nodeField argument is always of the same type as this, therefor
	allow for deliberate downcasting. This assumption is dangerous.
	Commercial application should implement safe casting or proper
	conversion mechanism.
	-----------------------------------------------------------------*/
	virtual void Copy (NodeField &nodeField, MediaObject *, bool bIsFirst)
	{
		MF<T> &fld = (MF<T> &) nodeField;
		m_array.Clear ();
		for (int i = 0; i < fld.GetLength (); i++)
			m_array += fld [i];
		OnFieldUpdate (bIsFirst);
	}

public:
	void InsertAt (int index, T value, BOOL bRoute = TRUE)
	{
		m_array.InsertAt (index, value);
		if (bRoute)
			OnFieldUpdate ();
	}
	void Append (T value, BOOL bRoute = TRUE)
	{
		m_array += value;
		if (bRoute)
			OnFieldUpdate ();
	}
	void SetAt (int index, T value, BOOL bRoute = TRUE)
	{
		m_array.SetAt (index, value);
		if (bRoute)
			OnFieldUpdate ();
	}
	void DeleteFrom (int index, BOOL bRoute = TRUE)
	{
		m_array.DeleteFrom (index);
		if (bRoute)
			OnFieldUpdate ();
		#ifdef BIFSCAR
		if (m_array.GetLength () == 0)
			SetNeedsEncoding (false);
		#endif
	}
#endif	//	BIFSDEC

	/*-------------------------------------------------------------
	Methods used by the encoder
	-------------------------------------------------------------*/
#ifdef BIFSENC
private:
	virtual BOOL EncodeElement (BIFSEncoder *pBifs, Quantizer *pQuant = NULL)
	{
		SF<T> m_element;
		return m_element.Encode (pBifs, pQuant);
	}
#endif		// BIFSENC

#ifdef BIFSCAR
private:
	virtual BOOL EncodeElement (
		int index,
		CarouselEncoder *pBifs,
		Quantizer *pQuant = NULL
	)
	{
		SF<T> m_element = m_array [index];
		return m_element.Encode (pBifs, pQuant);
	}
#endif

protected:
	ZArray<T> m_array;
	static FIELD_TYPE s_fieldType;
};

/*-----------------------------------------------------------------
A base class for classes implementing MFNode (or specific type
MFNode fields).
-----------------------------------------------------------------*/
class MFGenericNode : public MField
{
	/*-------------------------------------------------------------
	Methods used by the decoder and the encoder
	-------------------------------------------------------------*/
public:
	MFGenericNode () : MField (MFNODE) {m_pParent = NULL;}
	virtual ~MFGenericNode () {RemoveAll ();}
	MediaObject *operator [] (int index)
	{
		MediaObject *pObject = m_array [index];
		return pObject == NULL ? NULL : pObject -> GetRenderingNode ();
	}
	void DeleteFrom (int index, BOOL bRoute = TRUE);
	virtual int GetLength () const {return m_array.GetLength ();}
	virtual void Clear ();

private:
	void RemoveAll ();

	/*-------------------------------------------------------------
	Methods used by the decoder
	-------------------------------------------------------------*/
#ifdef BIFSDEC
	friend class MediaObject;

private:
	virtual void Copy (NodeField &nodeField, MediaObject *pParent, bool bIsFirst);
	void RemoveNode (MediaObject *pNode) {m_array -= pNode;}

protected:
	virtual NodeField *Clone (NodeField *pField, MediaObject *pParent);

public:
	MediaObject *GetObject (int index) {return m_array [index];}
	void ReplaceNode (MediaObject *pOld, MediaObject *pNew);
	void SetParent (MediaObject *pNode) {m_pParent = pNode;}
	void InsertAt (int index, MediaObject * value, MediaObject *pParent, bool bRoute = true);
	void Append (MediaObject * value, MediaObject *pParent, bool bRoute = true);
	void SetAt (int index, MediaObject * value, MediaObject *pParent, bool bRoute = true);
	virtual BOOL AppendElement (
		BIFSDecoder *pBifs, 
		MediaObject *pNode, 
		Quantizer * = NULL
	)
	{
		SFGenericNode m_element;
		if (!m_element.ParseNodeField (pBifs, pNode, this, TRUE))
			return FALSE;
		ASSERT (m_element.GetNode ());
		m_array += m_element.GetNode ();
		m_pParent = pNode;
		return TRUE;
	}

	virtual BOOL InsertElement (
		BIFSDecoder *pBifs, 
		MediaObject *pNode,
		int nIndex,	 
		Quantizer * = NULL
	)
	{
		SFGenericNode m_element;
		if (!m_element.ParseNodeField (pBifs, pNode, this, TRUE))
			return FALSE;
		m_array.InsertAt (nIndex, m_element.GetNode ());
		m_pParent = pNode;
		return TRUE;
	}

	virtual BOOL ReplaceElement (
		BIFSDecoder *pBifs, 
		MediaObject *pNode,
		int nIndex,	 
		Quantizer * = NULL
	)
	{
		SFGenericNode m_element;
		if (!m_element.ParseNodeField (pBifs, pNode, this, TRUE))
			return FALSE;
		m_array.SetAt (nIndex, m_element.GetNode ());
		m_pParent = pNode;
		return TRUE;
	}
#endif	//	BIFSDEC

	/*-------------------------------------------------------------
	Methods used by the encoder
	-------------------------------------------------------------*/
#ifdef BIFSENC
private:
	virtual BOOL EncodeElement (BIFSEncoder *pBifs, Quantizer *pQuant = NULL)
	{
		SFGenericNode m_element;
		if (m_element.Encode (pBifs, pQuant))
		{	m_array += m_element;
			m_element = NULL;
			return TRUE;
		}
		return FALSE;
	}
#endif		// BIFSENC

#ifdef BIFSCAR
private:
	virtual BOOL EncodeElement (
		int index,
		CarouselEncoder *pBifs,
		Quantizer *pQuant = NULL
	)
	{
		SFGenericNode m_element;
		m_element = m_array [index];
		return m_element.Encode (pBifs, pQuant);
	}
#endif		// BIFSCAR

protected:
	ZArray<MediaObject *> m_array;
	MediaObject *m_pParent;
};

/*-----------------------------------------------------------------
A class template for all MFNode fields which allow nodes of a single
known type.  E.g. MFAppearance2Dnode would be declared as
MFNode<Appearance2D>.
-----------------------------------------------------------------*/
template <class T> class MFNode : public MFGenericNode
{
	/*-------------------------------------------------------------
	Methods used by the decoder and the encoder
	-------------------------------------------------------------*/
public:
	T *operator [] (int index)
	{
		MediaObject *pObject = m_array [index];
		return (T*) (pObject == NULL ? NULL : pObject -> GetRenderingNode ());
	}

	/*-------------------------------------------------------------
	Methods used by the decoder
	-------------------------------------------------------------*/
#ifdef BIFSDEC
public:
	T *ParseElement (BIFSDecoder *pBifs, MediaObject *pParent);

private:
	virtual BOOL AppendElement (
		BIFSDecoder *pBifs, 
		MediaObject *pNode, 
		Quantizer * = NULL
	)
	{
		SFNode<T> m_element;
		if (!m_element.ParseNodeField (pBifs, pNode, this, TRUE))
			return FALSE;
		ASSERT (m_element.GetNode ());
		m_array += m_element;
		return TRUE;
	}

	virtual BOOL InsertElement (
		BIFSDecoder *pBifs, 
		MediaObject *pNode, 
		int nIndex,
		Quantizer * = NULL
	)
	{
		SFNode<T> m_element;
		if (!m_element.ParseNodeField (pBifs, pNode, this, TRUE))
			return FALSE;
		m_array.InsertAt (nIndex, m_element);
		return TRUE;
	}

	virtual BOOL ReplaceElement (
		BIFSDecoder *pBifs, 
		MediaObject *pNode, 
		int nIndex,
		Quantizer * = NULL
	)
	{
		SFNode<T> m_element;
		if (!m_element.ParseNodeField (pBifs, pNode, this, TRUE))
			return FALSE;
		m_array.SetAt (nIndex, m_element);
		return TRUE;
	}
	virtual NodeField *Clone (NodeField *pField, MediaObject *pParent)
	{
		if (pField == NULL)
			pField = new MFNode<T>;
		return MFGenericNode :: Clone (pField, pParent);
	}
#endif	//	BIFSDEC

	/*-------------------------------------------------------------
	Methods used by the encoder
	-------------------------------------------------------------*/
#ifdef BIFSENC
private:
	virtual BOOL EncodeElement (BIFSEncoder *pBifs, Quantizer *pQuant = NULL)
	{
		SFNode<T> m_element;
		if (m_element.Encode (pBifs, pQuant))
		{	m_array += m_element;
			#ifdef BIFSDEC
			((MediaObject*)m_element) -> SetContainer (this, TRUE);
			#endif
			m_element = NULL;
			return TRUE;
		}
		return FALSE;
	}
#endif		// BIFSENC

#ifdef BIFSCAR
private:
	virtual BOOL EncodeElement (
		int index,
		CarouselEncoder *pBifs,
		Quantizer *pQuant = NULL
	)
	{
		SFNode<T> m_element;
		m_element = (T*) m_array [index];
		return m_element.Encode (pBifs, pQuant);
	}
#endif		// BIFSCAR
};

/*-----------------------------------------------------------------
A class template for all MFNode fields which allow more than one node
type.  T can be any of the SF...Node types. E.g. MF2DNode would be
declared as MFNodeType<SF2DNode>.
-----------------------------------------------------------------*/
template <class T> class MFNodeType : public MFGenericNode
{
	/*-------------------------------------------------------------
	Methods used by the decoder
	-------------------------------------------------------------*/
#ifdef BIFSDEC
public:
	virtual BOOL AppendElement (
		BIFSDecoder *pBifs, 
		MediaObject *pNode, 
		Quantizer * = NULL
	)
	{
		T m_element;
		if (!m_element.ParseNodeField (pBifs, pNode, this, TRUE))
			return FALSE;
		ASSERT (m_element.GetNode ());
		m_array += m_element.GetNode ();
		return TRUE;
	}

private:
	virtual BOOL InsertElement (
		BIFSDecoder *pBifs, 
		MediaObject *pNode, 
		int nIndex,
		Quantizer * = NULL
	)
	{
		T m_element;
		if (!m_element.ParseNodeField (pBifs, pNode, this, TRUE))
			return FALSE;
		m_array.InsertAt (nIndex, m_element.GetNode ());
		return TRUE;
	}

	virtual BOOL ReplaceElement (
		BIFSDecoder *pBifs, 
		MediaObject *pNode, 
		int nIndex,
		Quantizer * = NULL
	)
	{
		T m_element;
		if (!m_element.ParseNodeField (pBifs, pNode, this, TRUE))
			return FALSE;
		m_array.SetAt (nIndex, m_element.GetNode ());
		return TRUE;
	}
	virtual NodeField *Clone (NodeField *pField, MediaObject *pParent)
	{
		if (pField == NULL)
			pField = new MFNodeType<T>;
		return MFGenericNode :: Clone (pField, pParent);
	}
#endif	//	BIFSDEC

	/*-------------------------------------------------------------
	Methods used by the encoder
	-------------------------------------------------------------*/
#ifdef BIFSENC
private:
	virtual BOOL EncodeElement (BIFSEncoder *pBifs, Quantizer *pQuant = NULL)
	{
		T m_element;
		if (m_element.Encode (pBifs, pQuant))
		{	m_array += m_element;
			m_element = NULL;
			return TRUE;
		}
		return FALSE;
	}
#endif		// BIFSENC

#ifdef BIFSCAR
private:
	virtual BOOL EncodeElement (
		int index,
		CarouselEncoder *pBifs,
		Quantizer *pQuant = NULL
	)
	{
		T m_element;
		m_element = m_array [index];
		return m_element.Encode (pBifs, pQuant);
	}
#endif		// BIFSCAR
};

typedef MFNodeType<SFWorldNode> MFWorldNode;

#endif                                      // _NodeFld_h sentry.
