/***********************************************************************
This software module was originally developed by Zvi Lifshitz
(Triton R&D Ltd.) in the course of development of the MPEG-4 Systems
(ISO/IEC 14496-1) standard. This software module is an implementation of
a part of one or more MPEG-4 Systems (ISO/IEC 14496-1) tools as
specified by the MPEG-4 Systems (ISO/IEC 14496-1) standard. ISO/IEC
gives users of the MPEG-4 Systems (ISO/IEC 14496-1) free license to
this software module or modifications thereof for use in hardware or
software products claiming conformance to the MPEG-4 Systems (ISO/IEC
14496-1). Those intending to use this software module in hardware or
software products are advised that its use may infringe existing patents.
The original developer of this software module and his/her company, the
subsequent editors and their companies, and ISO/IEC have no liability
for use of this software module or modifications thereof in an
implementation. Copyright is not released for non MPEG-4 Systems
(ISO/IEC 14496-1) conforming products. Triton R&D Ltd. retains full right
to use the code for his/her own purpose, assign or donate the code to a
third party and to inhibit third parties from using the code for non
MPEG-4 Systems (ISO/IEC 14496-1) conforming products. This copyright
notice must be included in all copies or derivative works.
Copyright  1998.
************************************************************************

For more information or to receive an up to date version of this module,
you may call Zvi Lifshitz at zvil@csi.com.
----------------------------------------------------------------------*/

/*-------------------------------------------------------------------------
Project Im1Player
INTERNATIONAL ORGANIZATION FOR STANDARDIZATION
ISO/IEC JTC1/SC29/WG11

SUBSYSTEM:  BIFS Decoder/ Compositor / Presenter
FILE:       OpenStr.cpp
AUTHOR:     Zvi Lifshitz (Triton R&D Ltd.)
OVERVIEW:   Implementation of ObjectDescriptorImp and ESDescriptorImp
			funtions that handle opening/closing of streams.
-------------------------------------------------------------------------*/

/*-------------------------------------------------------------------------
MPEGJ 
AUTHORS:    Viswanathan Swaminathan - vishy@eng.sun.com 
			Robert Schmidt(RLS)
			Praveen Alavilli(PA) 
			YUNTAI KYONG(YT) - yuntaikyong@ieee.org
-------------------------------------------------------------------------*/

#include "defs.h"
#include "ODImp.h"
#include "StreamCo.h"
#include "Executiv.h"
#include "Stream.h"
#include "AL.h"
#include "IPMP.h"
#include "InLine.h"
#include "Channel.h"
#include "OCIDec.h"
#include "StrParser.h"
#include "..\mpegj\MPDecoderInterface.h"

/*-----------------------------------------------------------------
Go through the ESDescriptors contained in the OD, instantiate decoders
and IPMP systems, link them through MediaStream objects, and configure
all the input channels involved. The funtion returns a pointer to the
MediaStream object that provides the media to the compositor.
-----------------------------------------------------------------*/
MediaStreamImp *ObjectDescriptorImp :: SetStream (StreamConsumer *pNode)
{
	int i;
	MediaStreamImp *pResult = NULL;

	/*-------------------------------------------------------------
	Handle the remote OD case.
	-------------------------------------------------------------*/
	if (URL_Flag != 0)
	{	char szExpandedURL [MAX_PATH + 1];
		if (ZExpandURL (m_pAL ->GetURL (), URLstring, szExpandedURL, MAX_PATH) > 0)
			pNode ->GetExecutive () -> Error (INVALID_URL);
		else
			pNode -> GetExecutive () -> CreateService (szExpandedURL, pNode);
		return NULL;
	}

	if (esDescr.GetLength () == 0)
	{	pNode -> GetExecutive () -> Error (NO_STREAM);
		return NULL;
	}

	/*-------------------------------------------------------------
	To do: reorder the ESDs according to their dependencies. 
	-------------------------------------------------------------*/

	/*-------------------------------------------------------------
	Loop through the ESDs and set up decoder streams
	-------------------------------------------------------------*/
	m_pDecoder = NULL;
	for (i = 0; i < esDescr.GetLength (); i++)
	{	MediaStreamImp *pStream = ((ES_DescriptorImp *) esDescr [i]) ->
			SetDecoder (this, pNode);
		if (pStream != NULL)
			pResult = pStream;
	}
		
	return pResult;
}

/*-------------------------------------------------------------
Start processing the streams described by this OD
-------------------------------------------------------------*/
void ObjectDescriptorImp :: Start ()
{
	/*-------------------------------------------------------------
	Loop through the ESDs and start them
	-------------------------------------------------------------*/
	for (int i = 0; i < esDescr.GetLength (); i++)
		((ES_DescriptorImp *) esDescr [i]) -> Start ();
}

/*-------------------------------------------------------------
Stop processing the streams described by this OD
-------------------------------------------------------------*/
void ObjectDescriptorImp :: Stop ()
{
	/*-------------------------------------------------------------
	Loop through the ESDs and stop them
	-------------------------------------------------------------*/
	for (int i = 0; i < esDescr.GetLength (); i++)
		((ES_DescriptorImp *) esDescr [i]) -> Stop ();
}

/*-------------------------------------------------------------
Delete all the resources created by this OD
-------------------------------------------------------------*/
void ObjectDescriptorImp :: Detach ()
{
	/*-------------------------------------------------------------
	Loop through the ESDs and detach them
	-------------------------------------------------------------*/
	for (int i = 0; i < esDescr.GetLength (); i++)
		((ES_DescriptorImp *) esDescr [i]) -> Detach ();
}

/*-------------------------------------------------------------
ES_DescriptorImp constructor
-------------------------------------------------------------*/
ES_DescriptorImp :: ES_DescriptorImp ()
{
	m_pExecutive = NULL;
	m_pOutputStream = m_pInputStream = NULL;
	m_pDecoder = NULL;
	m_pChannel = NULL;
	m_bAttached = false;
}

/*-------------------------------------------------------------
Instantiate a decoder and configure a channel for an elementary stream.
The function returns a pointer to the composition buffer.
-------------------------------------------------------------*/
MediaStreamImp *ES_DescriptorImp :: SetDecoder (
	ObjectDescriptorImp *pOD,
	StreamConsumer *pNode
)
{
	DecoderParams decoderParams;
	NON_FATAL_ERROR error = IM1_OK;
	Decoder *pDecoder = NULL;

	m_pExecutive = pNode -> GetExecutive ();
	m_pOD = pOD;
	if (decConfigDescr == NULL_DESC)
	{	m_pExecutive -> Error (NO_STREAM_TYPE);
		return NULL;
	}
	
	/*-------------------------------------------------------------
	Skip IPMP streams
	-------------------------------------------------------------*/
	if (decConfigDescr -> streamType == ST_IPMPStream)
		return NULL;

	/*-------------------------------------------------------------
	OCR streams are opened without creating a data buffer
	-------------------------------------------------------------*/
	if (decConfigDescr -> streamType == ST_ClockReferenceStream)
	{	m_pChannel = OpenStream (pOD, 0, 0);
		return NULL;
	}

	/*-------------------------------------------------------------
	If the stream depends on another stream find the decoder used by
	that stream. If no decoder is used, skip this stream as well.
	-------------------------------------------------------------*/
	if (dependsOn_ES_ID > 0)
	{	for (int i = 0; i < pOD -> esDescr.GetLength (); i++)
			if (dependsOn_ES_ID == pOD -> esDescr [i] -> ES_ID)
			{	pDecoder =
					((ES_DescriptorImp *) pOD -> esDescr [i]) -> GetDecoder ();
				if (pDecoder == NULL)
					return NULL;
				break;
			}
	}
					
	/*-------------------------------------------------------------
	First try if the node can create a decoder for this stream type.
	If the function returns the same decoder as the one used by stream
	whom this stream depends on, mark that this stream is attached to
	another decoder.
	-------------------------------------------------------------*/
	m_pDecoder =
		pNode -> CreateDecoder (decConfigDescr, qosDescr, pDecoder);
	m_bAttached = m_pDecoder != NULL && m_pDecoder == pDecoder;

	/*-------------------------------------------------------------
	ObjectContentInfoStream is a special case and we create a dedicated
	OCI parser for such stream
	-------------------------------------------------------------*/
	if (m_pDecoder == NULL && decConfigDescr -> streamType == ST_ObjectContentInfoStream)
		m_pDecoder = CreateOCIDecoder (decConfigDescr -> objectTypeIndication);
#ifdef MPGJ
	//*** YT - START (6)
	/*-------------------------------------------------------------
	MPEGJStream is a special case and we create a dedicated
	MPEGJ decoder for such stream
	-------------------------------------------------------------*/
	if (m_pDecoder == NULL && decConfigDescr -> streamType == ST_MPEGJStream)
		m_pDecoder = CreateMPEGJDecoder();
	//*** YT - END (6)
#endif MPGJ
	/*-------------------------------------------------------------
	If the node didn't create a decoder, create it now using the info
	in decConfigDescr. We skip this stream if the OD already has a
	decoder.
	-------------------------------------------------------------*/
	if (m_pDecoder == NULL)
	{	
		#ifndef MPGJ
		if(pOD -> GetDecoder () != NULL)
			return NULL;
		#else
		//*** YT START (104)
		// Original IM1Core coder ignore elementary streams
		// of which OD has already setup a decoder for the 
		// primary ES.
		// Since we want to switch decoder between ESs
		// we setup buffers and channels for secondary ESs also
	
		m_pDecoder = pOD -> GetDecoder ();
		if (m_pDecoder != NULL) {
			// here IM1COre used to return NULL;
			// but we setup channel
			m_pChannel = OpenStream (
				pOD,
				decoderParams.m_nDBSize,
				decoderParams.m_nOptimalAUSize
				);
			if (m_pChannel == NULL) {
				Detach ();
				return NULL;
			}
			
			// and setup buffers...
			if (decoderParams.m_nCBSize != 0)
				m_pOutputStream = new MediaStreamImp (
				m_pInputStream -> GetClockReferece (),
				decoderParams.m_nCBSize + m_pDecoder -> GetOptimalOutputSize ()
				);
			
			// but this ES doesn't have decoder now
			m_pDecoder = NULL;

			// input buffer is attached to DataChannel
			// Open() will be called by DataChannel
			// Datachannel will call the input buffer attached to 
			// the decoder
			// so make the buffer discard incoming AUs
			m_pInputStream->HalfClose(TRUE);
			m_pInputStream->Start();
			

			// output buffer is not attached to anybody
			// won't be called from anybody
			// so we don't call HalfClose() or Pause();
			m_pOutputStream->Open();
			m_pOutputStream->Start();
						
			//*** YT END (104)
			return NULL;
		}
		#endif
		m_pDecoder = (Decoder *) LoadFilter (
			decConfigDescr -> streamType,
			decConfigDescr -> objectTypeIndication
		);
		
//		#ifdef MPGJ // MPEGJ - PA
		m_pDecoder ->decoderType = decConfigDescr -> streamType;
	//	#endif
	}

	/*-----------------------------------------------------------------
	Verify that there is a matching capability between the decoder and
	the node, and sets up the decoder format. If no matching
	capability exists, delete the decoder and ignore the stream
	-----------------------------------------------------------------*/
	if (m_pDecoder == NULL)
		error = UNKNOWN_DECODER_TYPE;
	else
	{	DecoderSpecificInfo *pSpecificInfo = decConfigDescr -> decSpecificInfo;
		LPCBYTE pInfo = NULL;
		int nInfoLength = 0;
		if (pSpecificInfo != NULL)
		{	pInfo = pSpecificInfo -> info;
			nInfoLength = pSpecificInfo -> info.GetLength ();
		}
		if (!m_pDecoder -> Setup (pInfo, nInfoLength, 0))
			error = DECODER_SETUP_FAILURE;
		else if (!m_bAttached)
		{
			CapabilitiesImp caps;

			pNode -> GetCapabilities (&caps);
			if (m_pDecoder -> SetFormat (caps, decoderParams))
				pNode -> SetStreamFormat (decoderParams.m_format);
			else
				error = INCOMPATIBLE_DECODER_TYPE;
		}
	}
	if (error != IM1_OK)
	{	Detach ();		// will delete the decoder
		m_pExecutive -> Error (error);
		return NULL;
	}

	/*-----------------------------------------------------------------
	Set up the channel
	-----------------------------------------------------------------*/
	m_pChannel = OpenStream (
					pOD,
					decoderParams.m_nDBSize,
					decoderParams.m_nOptimalAUSize
				 );
	if (m_pChannel == NULL)
	{	Detach ();
		return NULL;
	}

	/*-----------------------------------------------------------------
	Create the composition buffer.
	If decoderParams.m_nCBSize + m_pDecoder -> GetOptimalOutputSize () 
	is zero, it means no composition buffer is used, and the decoder
	manipulates the scene graph directly.
	-----------------------------------------------------------------*/
	if (decoderParams.m_nCBSize != 0)
		m_pOutputStream = new MediaStreamImp (
			m_pInputStream -> GetClockReferece (),
			decoderParams.m_nCBSize + m_pDecoder -> GetOptimalOutputSize ()
		);


	m_pDecoder -> SetOutputStream (
		decoderParams.m_nStreamNumber,
		m_pOutputStream
	);
	
	/*-------------------------------------------------------------
	Instantiate IPMP systems for this stream. If failed, dont use this
	stream
	-------------------------------------------------------------*/
	if (!InvokeIPMP (pOD))
	{	Detach ();
		return NULL;
	}

	/*-------------------------------------------------------------
	Now everything is all right. Set connect the decoder to proper streams
	and return the topmost output stream.
	-------------------------------------------------------------*/
	pOD -> SetDecoder (m_pDecoder);
	m_pDecoder -> SetInputStream (0, m_pInputStream);

//	#ifdef MPGJ // MPEGJ - PA
	m_pDecoder -> SetExecutive(m_pExecutive); 
//	#endif

	/*-------------------------------------------------------------
	If decoderParams.m_nCBSize is zero
	and m_pDecoder -> GetOptimalOutputSize () is not zero,
	this is null decoder, i.e., stream is consumed directly by the 
	object.
	-------------------------------------------------------------*/
	if (decoderParams.m_nCBSize == 0 &&
		m_pDecoder -> GetOptimalOutputSize () != 0
	)
		return m_pInputStream;
	return m_pOutputStream;
}

/*-------------------------------------------------------------
Load the DLL and instantiate the Filter object for the given streamType
and objectTypeIndication. Returns NULL if failed.
-------------------------------------------------------------*/
Filter *ES_DescriptorImp :: LoadFilter (
	int nStreamType,
	int objectTypeIndication
)
{
	static ZRegistry registry("Software\\MPEG-4\\IM1\\Decoders");
	HMODULE hDLL = NULL;

	/*-------------------------------------------------------------
	Construct the decoder type and asks the registry object to locate it,
	load its DLL, and instantiate it.
	-------------------------------------------------------------*/
	static LPCSTR szStreamType [] =
	{
		"",			// ObjectDescriptorStream = 1
		"",			// ClockReferenceStream = 2
		"",			// SceneDescriptionStream = 3
		"Visual",	// VisualStream = 4
		"Audio",	// AudioStream = 5
		"MPEG7",	// MPEG7Stream = 6
		"IPMP",		// IPMP = 7
		"DUMMY",    // dummy, since the MPEGJ stream type is 9
		"MPEGJ"		// MPEGJ - PA
	};
	enum {TABLE_LENGTH = sizeof szStreamType / sizeof (LPCSTR)};

	if (--nStreamType < 0 || nStreamType >= TABLE_LENGTH ||
		szStreamType [nStreamType] [0] == 0
	)
		return NULL;

	ZString szFilterType (szStreamType [nStreamType]);
	if (objectTypeIndication != 0 && objectTypeIndication != 0xff && nStreamType != 8) //nStreamType != 6 MPEGJ - PA 
	{	char szBuffer [10];
		szFilterType += '-';
		szFilterType += itoa (objectTypeIndication, szBuffer, 16);
	}

	Filter *pFilter =
		(Filter *) registry.CreateInstance (szFilterType, hDLL);
	if (pFilter != NULL)
		pFilter -> SetLibrary (hDLL);
	return pFilter;
}

/*-------------------------------------------------------------
Perform the stream open procedure with this ES_Descriptor
-------------------------------------------------------------*/
DataChannel *ES_DescriptorImp :: OpenStream (
	ObjectDescriptorImp *pOD,
	int nBufferSize,
	int nExpectedAULength
)
{
	m_pAL = pOD -> GetAL ();
	if (slConfigDescr == NULL_DESC)
	{	m_pExecutive -> Error (NO_SLCONFIG_INFO);
		return NULL;
	}

	/*-------------------------------------------------------------
	Open the stream, and attach it to a MediaStream object.
	If nBufferSize is zero, open the stream without attaching to
	a MediaStream, which means "get data and discard"
	-------------------------------------------------------------*/
	if (nBufferSize != 0)
	{
		if (nBufferSize > 0)
		{	if (decConfigDescr != NULL_DESC &&
				decConfigDescr -> bufferSizeDB != 0
			)
				nBufferSize = decConfigDescr -> bufferSizeDB;

			/*-------------------------------------------------------------
			The actual buffer size is calcualted as following:
			If nExpectedAULength > 0, we use twice the declared amount of
			buffer size, to compenstate for jitter delays, and solve buffer
			slicing (units need to be in contiguneous memory).
			If nExpectedAULength is zero, we assume buffer is required for
			just one AU.
			To this we have to add space for AU headers. If nExpectedAULength
			> 0, we take 3 x buffer-size/average unit-size. If 
			nExpectedAULength is 0, there is just one unit and one header.
			-------------------------------------------------------------*/
			if (nExpectedAULength > 0)
				nBufferSize =
					nBufferSize * 2 +
					3 * (sizeof (int) + sizeof (DWORD)) *
					(nBufferSize / nExpectedAULength + 1);
			else
			{	nExpectedAULength = nBufferSize;
				nBufferSize += sizeof (int) + sizeof (DWORD);
			}
		}

		/*-------------------------------------------------------------
		Create a MediaStream object with NULL clock reference. The
		clock reference will be set by ALManager::AttachStream()
		-------------------------------------------------------------*/
		m_pInputStream = new MediaStreamImp (NULL, nBufferSize);
	}


	/*-------------------------------------------------------------
	Open the input channel and link it to the proper MediaStream object
	-------------------------------------------------------------*/
	return m_pAL -> AttachStream (
		this,
		m_pInputStream,
		slConfigDescr,
		pOD -> GetInline (),
		nExpectedAULength
	);
}

/*-------------------------------------------------------------
Attach IPMP filters to an elementary stream. Return true if succeeded 
to setup all IPMP systems.
-------------------------------------------------------------*/
bool ES_DescriptorImp :: InvokeIPMP (ObjectDescriptorImp *pOD)
{
	int i;

	/*-------------------------------------------------------------
	Loop through the ESDs to set up IPMP filters established by IPMP
	streams.
	-------------------------------------------------------------*/
	for (i = 0; i < pOD -> esDescr.GetLength (); i++)
		if (! ((ES_DescriptorImp *) pOD -> esDescr [i]) -> SetIPMP (this, pOD))
			return false;

	/*-------------------------------------------------------------
	Loop through the IPMP pointers and handle IPMP descriptors
	-------------------------------------------------------------*/
	if (!HandleIPMPDescriptors (
			ipmpDescrPtr,
			ipmpDescrPtr.GetLength (),
			pOD -> GetInline ()
		)
	)
		return false;

	/*-------------------------------------------------------------
	Loop through the IPMP pointers of the OD and handle IPMP descriptors
	-------------------------------------------------------------*/
	if (!HandleIPMPDescriptors (
			pOD -> ipmpDescrPtr,
			pOD -> ipmpDescrPtr.GetLength (),
			pOD -> GetInline ()
		)
	)
		return false;
	return true;
}

/*-------------------------------------------------------------
If the object is an IPMP stream, instantiate an IPMP system and attach
it to the given elementary stream
-------------------------------------------------------------*/
bool ES_DescriptorImp :: SetIPMP (
	ES_DescriptorImp *pES,
	ObjectDescriptorImp *pOD
)
{
	if (decConfigDescr == NULL_DESC ||
		decConfigDescr -> streamType != ST_IPMPStream
	)
		return true;

	/*-------------------------------------------------------------
	This version doesn't allow the attaching of more than IPMP stream to
	a single data stream
	-------------------------------------------------------------*/
	m_pAL = pOD -> GetAL ();
	m_pExecutive = m_pAL -> GetExecutive ();

	if (m_pChannel != NULL)
	{	m_pExecutive -> Error (IPMP_STREAM_ALREADY_IN_USE);
		return false;
	}

	/*-------------------------------------------------------------
	Load the appropriate IPMP system
	-------------------------------------------------------------*/
	IPMPManager *pIPMP = (IPMPManager *) LoadFilter (
		ST_IPMPStream,
		decConfigDescr -> objectTypeIndication
	);
	if (!pIPMP)
	{	m_pExecutive -> Error (IPMP_NOT_AVAILABLE);
		return false;
	}
	else if (!pIPMP -> Init ())
	{	m_pExecutive -> Error (NO_ACCESS_TO_PROTECTED_STREAM);
		return false;
	}

	//added by Andrea Varesio
	pIPMP -> SetAssociatedDecoder((void *) (pES -> m_pDecoder));	//the decoder is got from ES_DescriptorImp

	/*-------------------------------------------------------------
	Open the IPMP channel
	-------------------------------------------------------------*/
	int nBufferSize = decConfigDescr -> bufferSizeDB;
	if (nBufferSize == 0)
		nBufferSize = MPEG4_DEFAULT_IPMP_BUFFER_SIZE;
	m_pChannel = OpenStream (
		pOD,
		nBufferSize,
		MPEG4_OPTIMAL_IPMP_UNIT_SIZE
	);
	if (m_pChannel == NULL)
		return false;

	/*-------------------------------------------------------------
	Setup the IPMP system
	-------------------------------------------------------------*/
	DecoderSpecificInfo *pSpecificInfo = decConfigDescr -> decSpecificInfo;
	LPCBYTE pInfo = NULL;
	int nInfoLength = 0;
	if (pSpecificInfo != NULL)
	{	pInfo = pSpecificInfo -> info;
		nInfoLength = pSpecificInfo -> info.GetLength ();
	}
	pIPMP -> SetIPMPStream (
		m_pInputStream,
		decConfigDescr -> objectTypeIndication,
		pInfo,
		nInfoLength
	);

	/*-------------------------------------------------------------
	Attach the system to the elementary stream
	-------------------------------------------------------------*/
	pES -> AttachIPMP (pIPMP);
	return true;
}

/*-------------------------------------------------------------
Handle a set of IPMP descriptors linked to this stream.
Return true if all descriptors where processed successfully.
-------------------------------------------------------------*/
bool ES_DescriptorImp :: HandleIPMPDescriptors (
	IPMP_DescriptorPointer * const * descPointers,
	int nDescPointersLength,
	InlineImp *pInline
)
{
	/*-------------------------------------------------------------
	Loop through all the pointers and locate the descriptors they are
	pointing to
	-------------------------------------------------------------*/
	IPMPManager *pGenericIPMP = NULL;
	for (int i = 0; i < nDescPointersLength; i++)
	{
		int nID = descPointers [i] -> IPMP_DescriptorID;
		IPMPManager *pIPMP = NULL;
		IPMP_DescriptorImp *pIPMPD = pInline -> GetIPMP_Descriptor (nID);

		/*-------------------------------------------------------------
		If a descriptor pointed by the pointer already exists,
		walk through all the IPMP systems currently attached to this ES.
		-------------------------------------------------------------*/
		if (pIPMPD != NULL)
			for (int j = m_ipmps.GetLength (); --j >= 0;)
				if (m_ipmps [j] -> IsMyType (pIPMPD -> IPMPS_Type))
				{	pIPMP = m_ipmps [j];
					break;
				}
	
		/*-------------------------------------------------------------
		If no system can handle the descriptor, or if the descriptor is
		not known yet, instantiate a generic IPMP system, or use one if
		has already been instantiated before.
		-------------------------------------------------------------*/
		if (!pIPMP)
			pIPMP = pGenericIPMP;
		if (!pIPMP)
		{	pIPMP = pGenericIPMP =
				(IPMPManager *) LoadFilter (ST_IPMPStream, 0);
			if (!pIPMP)
			{	m_pExecutive -> Error (IPMP_NOT_AVAILABLE);
				return false;
			}
			AttachIPMP (pIPMP);
		}

		/*-------------------------------------------------------------
		If a descriptor exists pass it to the IPMP system
		-------------------------------------------------------------*/
		if (pIPMPD)
			switch (pIPMPD -> Notify (pIPMP)) {
			case 0:
				m_pExecutive -> Error (NO_ACCESS_TO_PROTECTED_STREAM);
				return false;
			case -1:
				m_pExecutive -> Error (IPMP_NOT_AVAILABLE);
				return false;
			}

		/*-------------------------------------------------------------
		Associate the IPMP system with the descriptor ID for future
		IPMPDescriptor updates
		-------------------------------------------------------------*/
		pInline -> AssociateIPMPSystem (nID, pIPMP);
	}

	return true;
}

/*-------------------------------------------------------------
Attach an IPMP filter to this elementary stream
-------------------------------------------------------------*/
void ES_DescriptorImp :: AttachIPMP (IPMPManager *pIPMP)
{
	/*-------------------------------------------------------------
	Determine whether this is a pre or post decoding IPMP system.
	Create a MediaStream object with the proper size, depending
	on previous question.
	Link before or after the decoder.
	-------------------------------------------------------------*/
	bool bPostDecoding = pIPMP -> IsPostDecoding ();
	if (bPostDecoding && m_pOutputStream == NULL)
		return;		// post decoding not applicable.
	MediaStreamImp *pStream = new MediaStreamImp (
		m_pInputStream -> GetClockReferece (),
		(bPostDecoding ? m_pOutputStream : m_pInputStream) -> GetSize ()
	);
	if (bPostDecoding)
	{	pIPMP -> SetInputStream (m_pOutputStream);
		pIPMP -> SetOutputStream (pStream);
		m_pOutputStream = pStream;
	}
	else
	{	pIPMP -> SetInputStream (m_pInputStream);
		pIPMP -> SetOutputStream (pStream);
		m_pInputStream = pStream;
	}

	/*-------------------------------------------------------------
	Add to the list of IPMP systems
	-------------------------------------------------------------*/
	m_ipmps += pIPMP;
}

/*-------------------------------------------------------------
Start processing the stream described by this ES_Descriptor
-------------------------------------------------------------*/
void ES_DescriptorImp :: Start ()
{
	if (m_pDecoder && !m_bAttached)
		m_pDecoder -> Start ();
	if (m_pChannel)
	{	m_pChannel -> ChannelReady ();
		m_pExecutive -> SetPendingChannels ();
	}

	/*-------------------------------------------------------------
	Start all IPMP systems
	-------------------------------------------------------------*/
	for (int i = 0; i < m_ipmps.GetLength (); i++)
		m_ipmps [i] -> Start ();
}

/*-------------------------------------------------------------
Stop processing the stream described by this ES_Descriptor
-------------------------------------------------------------*/
void ES_DescriptorImp :: Stop ()
{
	/*-------------------------------------------------------------
	Stop all IPMP systems
	-------------------------------------------------------------*/
	for (int i = 0; i < m_ipmps.GetLength (); i++)
		m_ipmps [i] -> Stop ();

	/*-------------------------------------------------------------
	Unlock the scene before closing for the case the decoder thread
	is blocking on scene lock. UnlockScene is called with nReserve>0,
	to prevent other threads from modifying the scene.
	-------------------------------------------------------------*/
	m_pExecutive -> UnlockScene (1);

	/*-------------------------------------------------------------
	Stop the decoder and close the channel
	-------------------------------------------------------------*/
	if (m_pDecoder && !m_bAttached)
		m_pDecoder -> Stop ();
	if (m_pChannel)
		m_pChannel -> ChannelStop ();

	m_pExecutive -> LockScene (1);
}

/*-------------------------------------------------------------
Detach all the resources created by this ESD
-------------------------------------------------------------*/
void ES_DescriptorImp :: Detach ()
{
	#ifdef MPGJ
	//*** YT START (105)
	// when the ES is not attached to decoder
	// we need to shutdown buffer
	// this task used to be done by the decoder
	if(m_pDecoder == NULL) {
		m_pOutputStream->Terminate();
		m_pOutputStream->Close();
	}
	//*** YT END (105)
	#endif
	delete m_pOutputStream;
	m_pOutputStream = NULL;
	if (m_pDecoder && !m_bAttached)
	{	HMODULE hDLL = m_pDecoder -> GetLibrary ();
		delete m_pDecoder;
		m_pDecoder = NULL;
		m_bAttached = false;
		if (hDLL)
			FreeLibrary (hDLL);
	}	
	delete m_pChannel;
	m_pChannel = NULL;
	#ifdef MPGJ
	//*** YT START (105)
	if(m_pDecoder == NULL) {
		m_pInputStream->Terminate();
		m_pInputStream->Close();
	}
	//*** YT END (105)
	#endif MPGJ
	delete m_pInputStream;
	m_pInputStream = NULL;

	/*-------------------------------------------------------------
	Delete all IPMP systems
	-------------------------------------------------------------*/
	for (int i = 0; i < m_ipmps.GetLength (); i++)
	{	IPMPManager *pIPMP = m_ipmps [i];
		/*-------------------------------------------------------------
		Loop through the IPMP pointers and disassociate the IPMP
		system from the descriptors
		-------------------------------------------------------------*/
		RemoveIPMPSystems (
			pIPMP,
			ipmpDescrPtr,
			ipmpDescrPtr.GetLength (),
			m_pOD -> GetInline ()
		);

		/*-------------------------------------------------------------
		Loop through the IPMP pointers of the OD and disassociate the IPMP
		system from the descriptors
		-------------------------------------------------------------*/
		RemoveIPMPSystems (
			pIPMP,
			m_pOD -> ipmpDescrPtr,
			m_pOD -> ipmpDescrPtr.GetLength (),
			m_pOD -> GetInline ()
		);

		delete (MediaStreamImp *)(pIPMP -> GetInputStream ());
		HMODULE hDLL = pIPMP -> GetLibrary ();
		delete pIPMP;
		if (hDLL)
			FreeLibrary (hDLL);
	}
	m_ipmps.Clear ();
}

/*-------------------------------------------------------------
Add a new ESD to an active OD
-------------------------------------------------------------*/
MediaStreamImp *ES_DescriptorImp :: AddESD (
	ObjectDescriptorImp *pOD,
	StreamConsumer *pNode
)
{
	return SetDecoder (pOD, pNode);
}

/*-------------------------------------------------------------
When an IPMP system is deleted, detach it from all the descriptors
it was linked to trough IPMP descriptor pointers.
-------------------------------------------------------------*/
void ES_DescriptorImp :: RemoveIPMPSystems (
	IPMPManager *pIPMP,
	IPMP_DescriptorPointer * const * descPointers,
	int nDescPointersLength,
	InlineImp *pInline
)
{
	for (int i = 0; i < nDescPointersLength; i++)
		pInline -> RemoveIPMP (
			descPointers [i] -> IPMP_DescriptorID,
			pIPMP
		);
}

/*-------------------------------------------------------------
Create an OCI decoder for an OCI stream
-------------------------------------------------------------*/
Decoder *ES_DescriptorImp :: CreateOCIDecoder (int version)
{
	SysCoder *pCoder;
	pCoder = new OCIDecoder (
		m_pExecutive,
		m_pExecutive -> GetTraceFile (),
		"OCI",
		m_pExecutive -> GetTraceFlags (),
		version
	);
	return new StreamParser (m_pExecutive, pCoder);
}

#ifdef MPGJ
//*** YT START (7)
/*-------------------------------------------------------------
Create an MPEGJ decoder for an MPEGJ stream
-------------------------------------------------------------*/
Decoder *ES_DescriptorImp :: CreateMPEGJDecoder ()
{
	Decoder* pDecoder = (Decoder *) LoadFilter (
			decConfigDescr -> streamType,
			decConfigDescr -> objectTypeIndication
		);
	pDecoder->decoderType = decConfigDescr -> streamType;
	return pDecoder;
}
//*** YT END (7)


/*-------------------------------------------------------------
Detach decoder from the elementary stream
-------------------------------------------------------------*/
//*** YT START (106)
// Detach decoder from ES
DecoderImp*
ES_DescriptorImp::DetachDecoder()
{
	// check decoder is already detached
	if(!m_pDecoder)
		return NULL;

	// paude input and output buffers
	((DecoderImp*)m_pDecoder)->Pause();
	Decoder* pDecoder = m_pDecoder;
	m_pDecoder = NULL;
	return (DecoderImp*)pDecoder;
}
//*** YT END (106)


/*-------------------------------------------------------------
Attach decoder from the elementary stream
-------------------------------------------------------------*/
//*** YT START (107)
// Attach decoder to ES
void ES_DescriptorImp::AttachDecoder(DecoderImp* pDecoder)
{
	// check decoder is already attached
	if(m_pDecoder)
		return;
	m_pDecoder = pDecoder;
	// change decoder assoicated with OD
	if(m_pOD->GetDecoder() != pDecoder)
		m_pOD->SetDecoder(m_pDecoder);
	// assume we are going to attach decoder between
	// MediaStreamImp already setup
	// also Decoder is already configured...

	// resume input and ouput buffers
	((DecoderImp*)m_pDecoder)->Resume();
}
//*** YT END (107)
#endif MPGJ