/***********************************************************************
This software module was originally developed by Zvi Lifshitz
(Triton R&D Ltd.) in the course of development of the MPEG-4 Systems
(ISO/IEC 14496-1) standard. This software module is an implementation of
a part of one or more MPEG-4 Systems (ISO/IEC 14496-1) tools as
specified by the MPEG-4 Systems (ISO/IEC 14496-1) standard. ISO/IEC
gives users of the MPEG-4 Systems (ISO/IEC 14496-1) free license to
this software module or modifications thereof for use in hardware or
software products claiming conformance to the MPEG-4 Systems (ISO/IEC
14496-1). Those intending to use this software module in hardware or
software products are advised that its use may infringe existing patents.
The original developer of this software module and his/her company, the
subsequent editors and their companies, and ISO/IEC have no liability
for use of this software module or modifications thereof in an
implementation. Copyright is not released for non MPEG-4 Systems
(ISO/IEC 14496-1) conforming products. Triton R&D Ltd. retains full right
to use the code for his/her own purpose, assign or donate the code to a
third party and to inhibit third parties from using the code for non
MPEG-4 Systems (ISO/IEC 14496-1) conforming products. This copyright
notice must be included in all copies or derivative works.
Copyright  1998.
************************************************************************

For more information or to receive an up to date version of this module,
you may call Zvi Lifshitz at zvil@csi.com.
----------------------------------------------------------------------*/

#if !defined(_Quantize_h)	// Sentry, use file only if it's not already included.
#define _Quantize_h

/*-------------------------------------------------------------------------
Project Im1Player
INTERNATIONAL ORGANIZATION FOR STANDARDIZATION
ISO/IEC JTC1/SC29/WG11

SUBSYSTEM:  BIFS Decoder/ Compositor
FILE:       Quantize.h
AUTHOR:     Zvi Lifshitz (Triton R&D Ltd.)
OVERVIEW:   Declarations of Quantizer
-------------------------------------------------------------------------*/

class BIFSDecoder;
// MBS
class MediaObject; // MBS: added for constructor from a QuantizationParameter
class Vec2f;
class Vec3f;
class Rotation;
class Rotation2D;
class Color3f;
// end MBS

#if defined(BIFSENC) || defined(BIFSCAR)
class BIFSEncoder;
#endif

class Quantizer
{
  const float *m_pMin, *m_pMax;
  int m_nDimension, m_nQuantizationCode;

  // MBS: added following members and enums
  int m_nBits;
  MediaObject *m_pQP; // associated QuantizationParameter node

  enum { Q_NONE=0, Q_3D, Q_2D, Q_DRAWORDER, Q_COLOR, Q_TEXTURECOORDINATE, 
	 Q_ANGLE, Q_SCALE, Q_KEY, Q_NORMAL, Q_ROTATION, Q_SIZE3D, Q_SIZE2D,
	 Q_LSQ,Q_COORDIDX
  };

  // these enums reference the field number in QuantizationParameter proto
  typedef enum
  {
    LOCAL=0,
    POS3DQ,POS3DMIN,POS3DMAX,POS3DNB,
    POS2DQ,POS2DMIN,POS2DMAX,POS2DNB,
    DOQ,DOMIN,DOMAX,DONB,
    COLORQ,COLORMIN,COLORMAX,COLORNB,
    TCQ,TCMIN,TCMAX,TCNB,
    ANGLEQ,ANGLEMIN,ANGLEMAX,ANGLENB,
    SCALEQ,SCALEMIN,SCALEMAX,SCALENB,
    KEYQ,KEYMIN,KEYMAX,KEYNB,
    NORMALQ,NORMALNB,
    SIZEQ,SIZEMIN,SIZEMAX,SIZENB,
    EFLOAT
  } QPfields;

  // end MBS

public:
	Quantizer (
		const float *pMin,  
		const float *pMax,  
		int nDimension = 1, 
		int nQuantizationCode = Q_NONE // MBS
	) :
	  m_pMin (pMin),
	  m_pMax (pMax),
	  m_nDimension (nDimension),
	  m_nQuantizationCode (nQuantizationCode),
	  m_pQP(NULL) // MBS
	{
	}

  // MBS: added constructor from a QuantizationParameter node
  Quantizer(
	    MediaObject *qp,  
	    const float *pMin,  
	    const float *pMax,  
	    int nDimension = 1, 
	    int nQuantizationCode = Q_NONE
	    ) :
  m_pQP(qp),
  m_pMin(pMin),
  m_pMax(pMax),
  m_nDimension (nDimension),
  m_nQuantizationCode (nQuantizationCode) 
  {
  }

  void SetQPNode(MediaObject *qp) { m_pQP=qp; }
  BOOL CanQuantize ();

  int ParseInt (BIFSDecoder *pBifs);
  float ParseFloat (BIFSDecoder *pBifs);
  Vec3f ParseVec3f (BIFSDecoder *pBifs);
  Vec2f ParseVec2f (BIFSDecoder *pBifs);
  Rotation ParseRotation (BIFSDecoder *pBifs);
  Rotation2D ParseRotation2D (BIFSDecoder *pBifs);
  Color3f ParseColor3f (BIFSDecoder *pBifs);
  Vec3f ParseNormal (BIFSDecoder *pBifs, int nBits);
  float ParseEfficientFloat (BIFSDecoder *pBifs);

private:
  float *ParseOnUnitSphere(BIFSDecoder *pBifs, int nbBits, int nbComp);
  
public:
  BOOL HasEfficientFloat();
  BOOL IsActive() { return (CanQuantize() && m_pQP); }
  float InvQuantize(float mini, float maxi, int nbits, int v);

#if defined(BIFSENC) || defined(BIFSCAR)
private:
  void EncodeOnUnitSphere(BIFSEncoder *pBifs, float *val, int nbComp, int nBits);

public:
  void EncodeInt (BIFSEncoder *pBifs, int nValue);
  void EncodeFloat (BIFSEncoder *pBifs, float nValue);
  void EncodeVec3f (BIFSEncoder *pBifs, Vec3f &nValue);
  void EncodeVec2f (BIFSEncoder *pBifs, Vec2f &nValue);
  void EncodeRotation (BIFSEncoder *pBifs, Rotation &nValue);
  void EncodeRotation2D (BIFSEncoder *pBifs, Rotation2D &nValue);
  void EncodeColor3f (BIFSEncoder *pBifs, Color3f &nValue);
  void EncodeNormal(BIFSEncoder *pBifs, Vec3f &norm, int nbBits);
  void EncodeEfficientFloat (BIFSEncoder *pBifs, float fValue);

  int Quantize(float mini, float maxi, int nbits, float v);
  // end MBS
#endif // BIFSENC

};

#endif                                      // _Quantize_h sentry.
