/***********************************************************************
This software module was originally developed by Zvi Lifshitz
(Triton R&D Ltd.) in the course of development of the MPEG-4 Systems
(ISO/IEC 14496-1) standard. This software module is an implementation of
a part of one or more MPEG-4 Systems (ISO/IEC 14496-1) tools as
specified by the MPEG-4 Systems (ISO/IEC 14496-1) standard. ISO/IEC
gives users of the MPEG-4 Systems (ISO/IEC 14496-1) free license to
this software module or modifications thereof for use in hardware or
software products claiming conformance to the MPEG-4 Systems (ISO/IEC
14496-1). Those intending to use this software module in hardware or
software products are advised that its use may infringe existing patents.
The original developer of this software module and his/her company, the
subsequent editors and their companies, and ISO/IEC have no liability
for use of this software module or modifications thereof in an
implementation. Copyright is not released for non MPEG-4 Systems
(ISO/IEC 14496-1) conforming products. Triton R&D Ltd. retains full right
to use the code for his/her own purpose, assign or donate the code to a
third party and to inhibit third parties from using the code for non
MPEG-4 Systems (ISO/IEC 14496-1) conforming products. This copyright
notice must be included in all copies or derivative works.
Copyright  1997.
************************************************************************

For more information or to receive an up to date version of this module,
you may call Zvi Lifshitz at zvil@csi.com.
----------------------------------------------------------------------*/

/*-------------------------------------------------------------------------
Project Im1Player
INTERNATIONAL ORGANIZATION FOR STANDARDIZATION
ISO/IEC JTC1/SC29/WG11

SUBSYSTEM:  BIFS Decoder/ Compositor / Presenter
FILE:       Root.cpp
AUTHOR:     Zvi Lifshitz (Triton R&D Ltd.)
OVERVIEW:   Implementation of a base class for root nodes
-------------------------------------------------------------------------*/
#include "defs.h"
#include "Root.h"
#include "Route.h"
#include "Nodes.h"
#include "Proto.h"
#include "BifsDec.h"

/*-------------------------------------------------------------
The Root destructor deletes all the protos
-------------------------------------------------------------*/
Root :: ~Root ()
{
	for (int i = m_protos.GetLength (); --i >= 0;)
		delete m_protos [i];
}

/*-----------------------------------------------------------------
Returns a pointer to an offspring node whose nodeID = nID, or NULL 
if none exists.
-----------------------------------------------------------------*/
MediaObject * Root :: FindNode  (int nID)
{
	for (int i = m_nodes.GetLength (); --i >= 0;)
		if (m_nodes [i] -> GetID () == nID)
			return m_nodes [i];
	return NULL;
}

#ifdef MPGJ		// SCENE API Integration
/*-----------------------------------------------------------------
Returns a pointer to an offspring node whose defName = szNameID, or NULL 
if none exists.
-----------------------------------------------------------------*/
MediaObject * Root :: FindNode  (LPCSTR szNameID)
{
	for (int i = m_nodes.GetLength (); --i >= 0;)
		if (strcmp (m_nodes [i] -> GetNameID (), szNameID) == 0)
			return m_nodes [i];
	return NULL;
}
#endif

/*-----------------------------------------------------------------
Returns a pointer to a route routeID = nID, or NULL if none exists.
-----------------------------------------------------------------*/
Route * Root :: FindRoute  (int nID)
{
	for (int i = m_routes.GetLength (); --i >= 0;)
		if (m_routes [i] -> GetID () == nID)
			return m_routes [i];
	return NULL;
}

/*-----------------------------------------------------------------
Delete the node whose ID is nNodeID.  Returns TRUE if the node was 
found and deleted successfully.
-----------------------------------------------------------------*/
BOOL Root :: DeleteNode (int nNodeID)
{
	MediaObject *pNode = FindNode (nNodeID);
	if (pNode)
	{	pNode -> DeleteObject ();
		return true;
	}
	return false;
}

/*-----------------------------------------------------------------
Find an offspring node nodeID is nNodeID, and replace it by a new node, 
parsed from the BIFS stream.  Returns TRUE if a node was found and 
replaced successfully.
-----------------------------------------------------------------*/
BOOL Root :: ReplaceNode (int nNodeID,  BIFSDecoder *pBifs)
{
	MediaObject *pNode = FindNode (nNodeID);
	if (pNode)
	{	pNode -> ReplaceNode (pBifs);
		return TRUE;
	}
	else
		return FALSE;
}

/*-----------------------------------------------------------------
Delete the Route whose ID is nRouteID.  Returns TRUE if a Route was 
found and deleted successfully.
-----------------------------------------------------------------*/
BOOL Root :: DeleteRoute (int nRouteID)
{
	Route *pRoute = FindRoute (nRouteID);
	delete pRoute;
	return pRoute != NULL;
}

/*-----------------------------------------------------------------
Replace the Route whose ID is nRouteID, then that Route will be 
deleted, and a new Route with same routeID will be inserted into the 
scene.  The new route values are parsed from the BIFS stream.  Returns 
TRUE if a Route was found and replaced successfully. 
-----------------------------------------------------------------*/
BOOL Root :: ReplaceRoute (int /*nRouteID*/,  BIFSDecoder * /*pBifs*/)
{
	return FALSE;
}

/*-------------------------------------------------------------
Set WorldInfo field of the root
-------------------------------------------------------------*/
void Root :: SetWorldInfo (WorldInfo *pNode)
{
	m_worldInfo.SetObject (pNode, this);
}

/*-------------------------------------------------------------
Store values from the BIFSDecoderConfig in the root node
-------------------------------------------------------------*/
void Root :: Configure (
	BOOL bPixelMetric,
	int nPixelWidth,
	int nPixelHeight,
	int nNodeIDbits,
	int nRouteIDbits,
	int nProtoIDbits
)
{
	m_bPixelMetric = bPixelMetric;
	m_nPixelWidth = nPixelWidth;
	m_nPixelHeight = nPixelHeight;
	m_nNodeIDbits = nNodeIDbits;
	m_nRouteIDbits = nRouteIDbits;
	m_nProtoIDbits = nProtoIDbits;
}

/*-------------------------------------------------------------
Parse the Proto declaration list
-------------------------------------------------------------*/
void Root :: ParseProtoList (BIFSDecoder *pCoder)
{
	while (pCoder -> ParseInt (1, "morePROTOs"))
	{	Proto *pProto = new Proto;
		pProto ->SetInline (GetInline ());
		m_protos += pProto;
		pProto -> ParseProtoDeclaration (pCoder);
	}
}

/*-------------------------------------------------------------
Find a Proto in the list according to its ID
-------------------------------------------------------------*/
Proto * Root :: FindProto (int nID)
{
	for (int i = m_protos.GetLength (); --i >= 0;)
		if (m_protos [i] -> GetID () == nID)
			return m_protos [i];
	return NULL;
}

