/***********************************************************************
This software module was originally developed by Zvi Lifshitz
(Triton R&D Ltd.) in the course of development of the MPEG-4 Systems
(ISO/IEC 14496-1) standard. This software module is an implementation of
a part of one or more MPEG-4 Systems (ISO/IEC 14496-1) tools as
specified by the MPEG-4 Systems (ISO/IEC 14496-1) standard. ISO/IEC
gives users of the MPEG-4 Systems (ISO/IEC 14496-1) free license to
this software module or modifications thereof for use in hardware or
software products claiming conformance to the MPEG-4 Systems (ISO/IEC
14496-1). Those intending to use this software module in hardware or
software products are advised that its use may infringe existing patents.
The original developer of this software module and his/her company, the
subsequent editors and their companies, and ISO/IEC have no liability
for use of this software module or modifications thereof in an
implementation. Copyright is not released for non MPEG-4 Systems
(ISO/IEC 14496-1) conforming products. Triton R&D Ltd. retains full right
to use the code for his/her own purpose, assign or donate the code to a
third party and to inhibit third parties from using the code for non
MPEG-4 Systems (ISO/IEC 14496-1) conforming products. This copyright
notice must be included in all copies or derivative works.
Copyright  1997.
************************************************************************

For more information or to receive an up to date version of this module,
you may call Zvi Lifshitz at zvil@csi.com.
----------------------------------------------------------------------*/

#if !defined(_root_h)	// Sentry, use file only if it's not already included.
#define _root_h

/*-------------------------------------------------------------------------
Project Im1Player
INTERNATIONAL ORGANIZATION FOR STANDARDIZATION
ISO/IEC JTC1/SC29/WG11

SUBSYSTEM:  BIFS Decoder/ Compositor / Presenter
FILE:       Root.h
AUTHOR:     Zvi Lifshitz (Triton R&D Ltd.)
OVERVIEW:   Declarations of a base class for root nodes
-------------------------------------------------------------------------*/
#include "MediaObj.h"
#include "ztl.h"
#include "NodeFld.h"
#include "..\2DPlayer-CSELT\custom.h"

class WorldInfo;
class InlineImp;
class Proto;

class Root : public MediaObject
{
	ZArray<MediaObject *> m_nodes;
	ZArray<Proto *> m_protos;
	SFNode<WorldInfo> m_worldInfo;
	InlineImp *m_pSource;
	DWORD m_dwStartTime;
	BOOL m_bPixelMetric;
	int m_nPixelWidth, m_nPixelHeight;
	int m_nNodeIDbits, m_nRouteIDbits, m_nProtoIDbits;

protected:
	ZArray<Route *> m_routes;

public:
	Proto * FindProto (int nID);
	void ParseProtoList (BIFSDecoder *pCoder);
	bool IsProto () const {return GetNodeCode () == -1;}
#if !defined(BIFSENC) || defined(BIFSCAR)
	Root () {m_bIsRoot = TRUE;}
	~Root ();
	BOOL IsPixelMetric () {return m_bPixelMetric ;}
	int GetPixelWidth () {return m_nPixelWidth;}
	int GetPixelHeight () {return m_nPixelHeight;}
	int GetNodeIDbits () {return m_nNodeIDbits;}
	int GetRouteIDbits () {return m_nRouteIDbits;}
	int GetProtoIDbits () {return m_nProtoIDbits;}
	void Configure (
		BOOL bPixelMetric,
		int nPixelWidth,
		int nPixelHeight,
		int nNodeIDbits,
		int nRouteIDbits,
		int nProtoIDbits
	);
	BOOL ReplaceRoute (int nRouteID,  BIFSDecoder *pBifs);
	BOOL DeleteRoute (int nRouteID);
	BOOL ReplaceNode (int nNodeID,  BIFSDecoder *pBifs);
	BOOL DeleteNode (int nNodeID);
	Route * FindRoute  (int nID);
	MediaObject * FindNode  (int nID);
#ifdef MPGJ		// SCENE API Integration
	MediaObject * FindNode (LPCSTR szNameID);
#endif
	int GetRouteLength () const {return m_routes.GetLength ();}
	Route * GetRoute (int n) const {return m_routes [n];}
	void UnregisterRoute (Route *pRoute) {m_routes -= pRoute;}
	void RegisterRoute (Route *pRoute) {m_routes += pRoute;}
	void UnregisterNode (MediaObject *pNode) {m_nodes -= pNode;}
	void RegisterNode (MediaObject *pNode) {m_nodes += pNode;}
	void SetWorldInfo (WorldInfo *pNode);
	WorldInfo *GetWorldInfo () {return m_worldInfo;}
	void SetInline (InlineImp *pSource) {m_pSource = pSource;}
	InlineImp *GetInline () {return m_pSource;}
	void ResetTime () {m_dwStartTime = timeGetTime ();}
	DWORD GetTime () {return timeGetTime () - m_dwStartTime;}
#endif

	CUSTOMIZE_ROOT_DECLARATION
};

#endif                                      // _root_h sentry.
