/***********************************************************************
This software module was originally developed by Zvi Lifshitz
(Triton R&D Ltd.) in the course of development of the MPEG-4 Systems
(ISO/IEC 14496-1) standard. This software module is an implementation of
a part of one or more MPEG-4 Systems (ISO/IEC 14496-1) tools as
specified by the MPEG-4 Systems (ISO/IEC 14496-1) standard. ISO/IEC
gives users of the MPEG-4 Systems (ISO/IEC 14496-1) free license to
this software module or modifications thereof for use in hardware or
software products claiming conformance to the MPEG-4 Systems (ISO/IEC
14496-1). Those intending to use this software module in hardware or
software products are advised that its use may infringe existing patents.
The original developer of this software module and his/her company, the
subsequent editors and their companies, and ISO/IEC have no liability
for use of this software module or modifications thereof in an
implementation. Copyright is not released for non MPEG-4 Systems
(ISO/IEC 14496-1) conforming products. Triton R&D Ltd. retains full right
to use the code for his/her own purpose, assign or donate the code to a
third party and to inhibit third parties from using the code for non
MPEG-4 Systems (ISO/IEC 14496-1) conforming products. This copyright
notice must be included in all copies or derivative works.
Copyright  1998.
************************************************************************

For more information or to receive an up to date version of this module,
you may call Zvi Lifshitz at zvil@csi.com.
----------------------------------------------------------------------*/

/*-------------------------------------------------------------------------
Project Im1Player
INTERNATIONAL ORGANIZATION FOR STANDARDIZATION
ISO/IEC JTC1/SC29/WG11

SUBSYSTEM:  Compositor
FILE:       ztype.cpp
AUTHOR:     Zvi Lifshitz (Triton R&D Ltd.)
OVERVIEW:   Implementation of types of node fields
-------------------------------------------------------------------------*/

#include "defs.h"
#include "ztypes.h"
#include <math.h>

/*-----------------------------------------------------------------
Image
-----------------------------------------------------------------*/
//MBS 110498: added constructor Image(w,h,components)
//Alvar 981223: Added pixel array allocation and refcount stuff

Image :: Image (int w, int h, int components) :      // Mike B-S ctor
	width (w), 
	height (h), 
	numComponents (components), 
	pixels (NULL) ,
	m_pRefCnt (NULL)
{}

Image :: Image (int) :                           // Zvi ctor
	width (0), 
	height (0),
	numComponents (0),
	pixels (NULL),
	m_pRefCnt (NULL) 
{}
	
Image :: Image (const Image &i)                      // copy ctor
{
	width         = i.width;
	height        = i.height;
	numComponents = i.numComponents;
	pixels        = i.pixels;
	m_pRefCnt     = i.m_pRefCnt;
	if( m_pRefCnt )
		++*m_pRefCnt;
}

Image& Image :: operator = (const Image &i)           // assignment
{
	if( m_pRefCnt ) 
		if( --*m_pRefCnt == 0 )
		{	delete [] pixels;
			delete m_pRefCnt;
		}
	width         = i.width;
	height        = i.height;
	numComponents = i.numComponents;
	pixels        = i.pixels;
	m_pRefCnt     = i.m_pRefCnt;
	if( m_pRefCnt )
		++*m_pRefCnt;
	return *this;
}

Image :: ~Image () 
{ 
	if (m_pRefCnt && --*m_pRefCnt == 0)
	{	delete [] pixels;
		delete m_pRefCnt;
	}
}

// SB @@@@ from Karl's code >>>>
void Rotation :: Normalize ()
{
	float mag = sqrt (x * x + y * y + z * z);
	if (mag == 0.0)
		mag = 1.0;
	x /= mag;
	y /= mag;
	z /= mag;
}
