/***********************************************************************
    This software module  was  originally  developed by Zvi  Lifshitz
    (Triton R&D Ltd.) and edited by Mauro Rossi (CSELT),
    Guido Franceschini (CSELT) and Michael Stepping (FTK),
    Reinhard Bayer (GMD Fokus) and Carsten Herpel (THOMSON multimedia)
    in the course of development of the MPEG-4 DMIF (ISO/IEC 14496-6)
    standard.  This software module is an implementation of a part of
    one or more MPEG-4  DMIF  (ISO/IEC 14496-6)  tools  as  specified
    by  the  MPEG-4 DMIF (ISO/IEC 14496-6)  standard.  ISO/IEC  gives
    users of  the  MPEG-4  DMIF  (ISO/IEC 14496-6)  free  license  to
    this software module or modifications thereof for use in hardware
    or software products claiming  conformance  to  the  MPEG-4  DMIF 
    (ISO/IEC 14496-6). Those intending to use this software module in
    hardware or software  products  are  advised  that  its  use  may
    infringe   existing  patents.  The  original  developer  of  this
    software module and his/her company, the subsequent  editors  and
    their  companies,  and  ISO/IEC have no liability for use of this
    software module or modifications thereof  in  an  implementation.
    Copyright  is not released for non MPEG-4 DMIF  (ISO/IEC 14496-6)
    conforming  products. Triton R&D Ltd.
    retains full  right to use the code for  its  own purpose, assign
    or donate the code to a third party and to inhibit third  parties
    from  using  the  code  for  non  MPEG-4  DMIF  (ISO/IEC 14496-6)
    conforming products. This copyright notice must  be  included  in
    all copies or derivative works. Copyright  1998.
************************************************************************/

// Contact: guido.franceschini@cselt.it

/**************************************************************************
 * Project Im1Player
 * INTERNATIONAL STANDARD ORGANIZATION
 * ISO/IEC JTC1/SC29/WG11
 * 
 * SUBSYSTEM: DMIF
 * FILE:      FlexDemux.cpp
 * AUTHOR:    Mauro Rossi (CSELT)
 * OVERVIEW:  Implementation of the class FlexDemux
 * 
 **************************************************************************/

#include <iostream.h>
#include "FlexDemux.h"

/*-------------------------------------------------------------
	Constructor
	Fills the m_channelTable with MAX_CHANNELS_PER_FLEXMUX NULL
	channels
-------------------------------------------------------------*/
FlexDemux :: FlexDemux (Application_Sink* pSink) :
	m_pSink (pSink)
{
	for(int i=0;i<MAX_CHANNELS_PER_FLEXMUX;i++)
		m_channelTable += (MyChannel*)0;// rba: was NULL;
}


FlexDemux :: ~FlexDemux()
{
	for(int i=0;i<m_channelTable.GetLength();i++)
		delete m_channelTable[i];
	m_channelTable.Clear();
}


/*-------------------------------------------------------------
If the TransMux channel is deleted, notify all FlexMux channels
-------------------------------------------------------------*/
void FlexDemux :: DPIC_ChannelDelete_Cnf (
	const UINT nServiceId,
	const RESPONSE eResponse)
{
	for (int i = m_channelTable.GetLength (); --i >= 0;)
	{
		if (m_channelTable [i])
		{
			m_channelTable [i] -> DPIC_ChannelDelete_Cnf (nServiceId,eResponse);
		}
	}
}

/*-----------------------------------------------------------------
A received data packet is demultiplexed and dispatched to the
proper data channel(s).
Note: Multiplex tables are not implemented yet.
-----------------------------------------------------------------*/
void FlexDemux :: DPIC_PacketReceived_Ind (
	const BYTE *pStreamData,
	int nStreamDataLength,
	const BOOL bErr)
{
	/*-------------------------------------------------------------
	If the packet is erronous, don't parse it, and acknowledge all
	FlexMux channels that their data might have been corrupted.
	-------------------------------------------------------------*/
	if (bErr)
	{	
		for (int i = m_channelTable.GetLength (); --i >= 0;)
			if (m_channelTable [i])
				m_channelTable [i] -> DPIC_PacketReceived_Ind (NULL, 0, TRUE);
		return;
	}

	while (nStreamDataLength > 0)
	{
		int nFmxChNum = (0xff /*RBA*/ & *pStreamData++);
		int nLength = (0xff /*RBA*/ & *pStreamData++);
		if (nFmxChNum == 238)	// padding stream
		{
		}
		else if (nFmxChNum == 239)	// CR stream
		{
		}
		else if (nFmxChNum > 239)	// muxcode mode
		{
			int bmxErr = 1;
			int nMCTentry = nFmxChNum-240;
			int nVersion = *pStreamData & 0xF0;

			if (m_muxCodeTable.GetLength() > nMCTentry)
				if (m_muxCodeTable [nMCTentry].version == nVersion)	// Correct version of table entry?
				{
					bmxErr = 0;
					DemuxMuxCodePacket (m_muxCodeTable [nMCTentry], pStreamData+1, nLength-1);
				}

			if (bmxErr) 
			{
			/*-------------------------------------------------------------
			if MuxCodeTable entry does not exist or is not up to date acknowledge 
			all	FlexMux channels that their data might have been corrupted.
			-------------------------------------------------------------*/
				for (int i = m_channelTable.GetLength (); --i >= 0;)
					if (m_channelTable [i])
						m_channelTable [i] -> DPIC_PacketReceived_Ind (NULL, 0, TRUE);
			}
		}
		else if ( nFmxChNum < m_channelTable.GetLength())
		{
			if (m_channelTable [nFmxChNum] != NULL)
			{
				m_channelTable [nFmxChNum] ->
					DPIC_PacketReceived_Ind (pStreamData, nLength, FALSE);
			}
		}
		nStreamDataLength -= nLength + 2;
		pStreamData += nLength;
	}
}

/*-------------------------------------------------------------
Demultiplex one MuxCode packet
-------------------------------------------------------------*/
void FlexDemux :: DemuxMuxCodePacket(
	LinearMuxCodeTableElement &mcte, 
	const BYTE *pStreamData,
	int nLength)
{
	int nSlotCount = mcte.slots.GetLength();

	while (nLength > 0)		// repeat the muxcode pattern as often as needed

		for (int iSlot = 1; iSlot < nSlotCount; iSlot++) 
		{
			int nNumBytes = mcte.slots[iSlot].numberOfBytes;
			int nFmxChId = mcte.slots[iSlot].flexMuxChannel;
			int nActBytes = nLength - nNumBytes > 0 ? nNumBytes : nLength;

			if (m_channelTable [nFmxChId] != NULL)
				m_channelTable [nFmxChId] ->
					DPIC_PacketReceived_Ind (pStreamData, nActBytes, FALSE);
			pStreamData += nActBytes;
			nLength -= nActBytes;
			if (nLength <= 0)
				return;
		}
}

/*-------------------------------------------------------------
Add or modify MuxCodeTableElements for this FlexMux stream
-------------------------------------------------------------*/	
void FlexDemux :: UpdateMuxCodeTableElements(const LinearMuxCodeTable &muxCodeTable)
{
	m_muxCodeTable = muxCodeTable;
	// update of individual MuxCode table entries is not yet supported
}

/*-------------------------------------------------------------
Add a FlexMux channel to the channel table and notify the application,
which will instantiate a Channel object for it.
-------------------------------------------------------------*/	
void FlexDemux :: AddFlexmuxChannel (
	const UINT nServiceId,
	const StreamMapElement &outParam)
{
	DAIChannelAddParam cnfParam;
	
	DAIChannelAddElement* pCnfEl = new DAIChannelAddElement();

	pCnfEl->m_nChid = outParam.m_nChid;
	pCnfEl->m_nTmpId = outParam.m_nTmpId;
	pCnfEl->m_eResponse = RESPONSE_OK;
	pCnfEl->m_pQoSStream = NULL;		// unused field
	pCnfEl->m_eDirection = UNSPECIFIED;	// useless field; just avoid random values
	pCnfEl->m_nDataLen = outParam.m_nDataLen;
	if ( pCnfEl->m_nDataLen > 0 )
	{
		pCnfEl->m_pData		= new BYTE[pCnfEl->m_nDataLen];
		memcpy(pCnfEl->m_pData,outParam.m_pData,pCnfEl->m_nDataLen);
	}
	else
		pCnfEl->m_pData = NULL ;

	cnfParam += *pCnfEl;

	m_pSink -> DAI_Channel_Add_Cnf(nServiceId,cnfParam);

	if(outParam.m_nChid)
	{
		m_channelTable.SetAt(outParam.m_nFmxInfo, (MyChannel*)outParam.m_nChid);
	}

	delete pCnfEl;
}

/*-------------------------------------------------------------
Delete a FlexMux channel from the table and notify the application.
-------------------------------------------------------------*/
void FlexDemux :: DeleteFlexmuxChannel (
	const UINT nServiceId,
	const int nFmxInfo,
	const RESPONSE eResponse )
{
	Channel *pChannel = m_channelTable [nFmxInfo];
	if (pChannel)
	{	
		pChannel -> DPIC_ChannelDelete_Cnf (nServiceId,eResponse);
		if(eResponse == RESPONSE_OK)
		{
			delete (m_channelTable [nFmxInfo]);
			m_channelTable [nFmxInfo] = NULL;
		}
	}
}

/*---------------------------------------------
Check if there is at least one valid channel.
---------------------------------------------*/
bool FlexDemux :: isEmpty()
{
	for(int i=0;i<MAX_CHANNELS_PER_FLEXMUX;i++)
	{
		if(m_channelTable[i] != NULL)
			return false;
	}
	return true;
}
