/***********************************************************************
    This software module  was  originally  developed by Zvi  Lifshitz
    (Triton R&D Ltd.), Mauro Rossi (CSELT) and Guido Franceschini
    (CSELT), and edited by Michael Stepping (FTK)
    in the course of development of the MPEG-4 DMIF (ISO/IEC 14496-6)
    standard.  This software module is an implementation of a part of
    one or more MPEG-4  DMIF  (ISO/IEC 14496-6)  tools  as  specified
    by  the  MPEG-4 DMIF (ISO/IEC 14496-6)  standard.  ISO/IEC  gives
    users of  the  MPEG-4  DMIF  (ISO/IEC 14496-6)  free  license  to
    this software module or modifications thereof for use in hardware
    or software products claiming  conformance  to  the  MPEG-4  DMIF 
    (ISO/IEC 14496-6). Those intending to use this software module in
    hardware or software  products  are  advised  that  its  use  may
    infringe   existing  patents.  The  original  developer  of  this
    software module and his/her company, the subsequent  editors  and
    their  companies,  and  ISO/IEC have no liability for use of this
    software module or modifications thereof  in  an  implementation.
    Copyright  is not released for non MPEG-4 DMIF  (ISO/IEC 14496-6)
    conforming  products. Triton R&D Ltd. and CSELT
    retain  full  right to use the code for their own purpose, assign
    or donate the code to a third party and to inhibit third  parties
    from  using  the  code  for  non  MPEG-4  DMIF  (ISO/IEC 14496-6)
    conforming products. This copyright notice must  be  included  in
    all copies or derivative works. Copyright  1998.
************************************************************************/

/**************************************************************************
 * Project Im1Player
 * INTERNATIONAL STANDARD ORGANIZATION
 * ISO/IEC JTC1/SC29/WG11
 * 
 * SUBSYSTEM: DMIF
 * FILE:      DMIFClient.h
 * OVERVIEW:  Definitions of DMIF Client interfaces.
 * 
 **************************************************************************/

#if !defined(_DMIFClient_h)	// Sentry, use file only if it's not already included.
#define _DMIFClient_h

#include "DMIF.h"

class ServiceClientSink;
class Channel;
class FlexMux;

/*-------------------------------------------------------------
Struct for passing OUT parameters in :
ServiceClientSink::DPI_ChannelAdd_Cnf
ServiceClientSink::DPI_TransmuxAdd_Cnf
-------------------------------------------------------------*/
struct StreamMapElement
{   
	int		m_nFmxInfo;
	UINT	m_nChid;
	BYTE*	m_pData;
	int		m_nDataLen;

  public: // methods	
	BOOL operator == (StreamMapElement const &el) const 
	{
		return (m_nChid == el.m_nChid);
	}
	
	StreamMapElement() : m_nFmxInfo(-1), m_nChid(0), m_pData(NULL), m_nDataLen(0) 
	{}
	
	StreamMapElement(const StreamMapElement& o)//copy constructor
	{
		m_nFmxInfo	= o.m_nFmxInfo;
		m_nChid		= o.m_nChid;
		m_nDataLen	= o.m_nDataLen;
		if ( m_nDataLen > 0 )
		{
			m_pData		= new BYTE[o.m_nDataLen];
			memcpy(m_pData,o.m_pData,o.m_nDataLen);
		}
		else
			m_pData = NULL ;
	}

	StreamMapElement& operator=(const StreamMapElement& r)
	{
		m_nFmxInfo	= r.m_nFmxInfo;
		m_nChid		= r.m_nChid;
		m_nDataLen	= r.m_nDataLen;
		if ( m_pData )
			delete [] m_pData ;
		if ( m_nDataLen > 0 )
		{
			m_pData		= new BYTE[r.m_nDataLen];
			memcpy(m_pData,r.m_pData,m_nDataLen);
		}
		else
			m_pData = NULL ;
		return ( *this ) ;
	}
	
	virtual ~StreamMapElement()
	{
		delete[] m_pData ;
	}
};

typedef ZArray<StreamMapElement>		DPI_TransmuxAdd_CnfParam;
typedef ZArray<Channel *>				ChannelsMap;
typedef ZArray<RESPONSE>				DPI_ChannelsDelete_CnfParam;

/*-------------------------------------------------------------
Structs for passing MuxCodeTable entries:
ServiceClientSink::DPI_MuxCodeTableReceived_Ind
FlexDemux::AddMuxCodeTableElements
-------------------------------------------------------------*/
struct McteSlot
{
	int flexMuxChannel;
	int numberOfBytes;
public:
	McteSlot() : flexMuxChannel(-1), numberOfBytes(-1)
	{}
};
typedef ZArray<McteSlot> McteSlotTable;

struct LinearMuxCodeTableElement
{
	int version;
	McteSlotTable slots;
public:
	LinearMuxCodeTableElement() : version(-1), slots()
	{}
};
typedef	ZArray<LinearMuxCodeTableElement> LinearMuxCodeTable;


/*-------------------------------------------------------------
Interface for the DMIF object
Interface for a service (client) handler
-------------------------------------------------------------*/
class ServiceClient
{
public:
	// Service Attach functions
	virtual void DPI_ServiceAttach_Req(
		LPCSTR		szURL, 
		const BYTE*	pUuData,
		int			nUuDataLen
	) = 0;

	// Service Detach functions
	virtual void DPI_ServiceDetach_Req(
		REASON eReason
	) = 0;
	
	// Channel Add functions
	virtual void DPI_ChannelAdd_Req(
		const DAIChannelAddParam &inParam
	) = 0;
	
	// Channel Delete functions
	virtual void DPI_ChannelDelete_Req(
		const DAIChannelDeleteParam &inParam
	) = 0;

	// User Command function
	virtual void DPI_UserCommand_Req(
		const DAIUserCmdParam& inParam
	) = 0;
	virtual	void DPI_UserCommand_Ack_Req(
		const DAIUserCmdParam& outParam
	) = 0;

	// Function for data transmission
	virtual void DPI_SendData(
		UINT		nChid,
		const char*	pData,
		int			nLength
	) = 0;

	virtual ~ServiceClient () {};
	virtual BOOL IsYourURL (LPCSTR szURL) = 0;
	virtual LPCSTR GetModuleID () = 0;
	void SetSink (ServiceClientSink* pSink) {m_pSink = pSink;};
	ServiceClientSink* GetSink () { return m_pSink; };

private:
	ServiceClientSink* m_pSink;
};

/*-------------------------------------------------------------
Interface for DMIF callbacks sink object (equivalent to	DAI callbacks,
except of Channel related events, which are reported directly to the 
Channel object.
Inteface for a service (client) sink object
-------------------------------------------------------------*/
class ServiceClientSink
{
public:
	virtual ~ServiceClientSink () {};
	virtual void DPI_ServiceAttach_Cnf (
		const BYTE*	pUuData,
		int			nUuDataLen,
		RESPONSE	eResponse
	) = 0;
	virtual void DPI_ServiceDetach_Cnf (
		RESPONSE	eResponse
	) = 0;
	virtual void DPI_ChannelAdd_Cnf (
		HANDLE		hTransmux,
		const StreamMapElement &outParam,
		RESPONSE	eResponse
	) = 0;
	virtual void DPI_ChannelDelete_Cnf (
		HANDLE		hTransmux,
		int			nFmxInfo,
		RESPONSE	eResponse
	) = 0;
	virtual void DPI_TransmuxAdd_Cnf (
		HANDLE		hTransmux,
		FLEXMUX_TYPE	eFmxType,
		const DPI_TransmuxAdd_CnfParam &outParam
	) = 0;
	virtual void DPI_TransmuxDelete_Cnf (
		HANDLE		hTransmux,
		RESPONSE	eResponse
	) = 0;
	virtual void DPI_DataReceived_Ind (
		HANDLE		hTransmux,
		const BYTE*	pStreamData,
		int			nStreamDataLength,
		BOOL		bErr
	) = 0;
	virtual void DPI_MuxCodeTableReceived_Ind (
		HANDLE hTransmux,
		const LinearMuxCodeTable &muxCodeTable,
		BOOL bErr
	) = 0;
	virtual void DPI_UserCommand_Ack_Cnf (
		const DAIUserCmdParam& outParam
	) = 0;
};

/*-------------------------------------------------------------
Interface for the Channel object.
The object that handles stream related events
-------------------------------------------------------------*/

class Channel
{
  public:
	virtual ~Channel ()
	{
		LockChannel ();
		UnlockChannel ();
	}
	virtual void DPI_ChannelDelete_Cnf (
		RESPONSE	eResponse
	) = 0;
	virtual void DPI_PacketReceived_Ind (
		const BYTE*	pStreamData,
		int			nStreamDataLength,
		BOOL		bErr
	) = 0;
	void LockChannel ( ) 
		{m_mtxLock.P ( );} 
	void UnlockChannel ( ) 
		{m_mtxLock.V ( );} 

  private: 
	ZMutex m_mtxLock; 
};

#endif                                      // _DMIFClient_h sentry.
