/*------------------------------------------------------------------
	 1999 CSELT - Centro Studi e Laboratori Telecomunicazioni.
	All rights reserved.
------------------------------------------------------------------*/


#ifndef _DMIFSERVER_H_
#define _DMIFSERVER_H_

#include "ztl.h"
#include "loopDefs.h"


class Channel;
class FlexMux;


typedef enum tagFlexmuxMode
{
	REALTIME,
	OFFLINE
} FLEXMUX_MODE;

/*-------------------------------------------------------------
Struct for passing OUT parameters in :
ServiceClientSink::OnChannelAdd
ServiceClientSink::OnTransmuxAdd
-------------------------------------------------------------*/
struct StreamMapElement
{   
	int m_nFmxInfo;
	UINT m_nChId;
	WORD m_nESid;
	BOOL operator == (const StreamMapElement &el) const {return el.m_nChId == m_nChId;}
};

typedef ZArray<StreamMapElement> StreamTable;

struct FlexMuxDataElement
{	
	HANDLE				hTAT;
	FLEXMUX_TYPE		nFmxType;
	StreamTable			*pSMEs;
	BOOL operator == (const FlexMuxDataElement &rElement) const
    {
        return (hTAT == rElement.hTAT);
    }
};

struct DN_ChannelAddCallbackInElem
{
	WORD				nCAT;
	DIRECTION			nDirection;
	QosLoop				qosDescriptor;
	LPSTR				ddDataBuf;
	int					ddDataLen;
	BOOL operator == (DN_ChannelAddCallbackInElem const &el) const 
		{return el.nCAT == nCAT;}
};

struct DN_ChannelAddCallbackOutElem
{
	HANDLE hTAT;
	LPSTR ddDataBuf;
	int ddDataLen;
	BOOL operator == (DN_ChannelAddCallbackOutElem const &el) const 
		{return el.hTAT == hTAT;}
};

struct DN_ChannelDeleteCallbackInElem
{
	WORD nCAT;
	REASON_PARAMETER nReason;
	BOOL operator == (DN_ChannelDeleteCallbackInElem const &el) const 
		{return el.nCAT == nCAT;}
};

struct DN_ChannelDeleteCallbackOutElem
{
	RESPONSE_PARAMETER nResponse;	
	BOOL operator == (DN_ChannelDeleteCallbackOutElem const &el) const 
		{return el.nResponse == nResponse;}

};

struct OnChannelsAddstruct
{
	int		m_nFmxInfo;
	WORD	m_nESid;
	UINT	m_nChId;
	RESPONSE_PARAMETER m_nResponse;
	BOOL operator == (OnChannelsAddstruct const &el) const 
		{return el.m_nESid == m_nESid;}
};

typedef ZArray<OnChannelsAddstruct> OnChannelsAddArray;

struct OnChannelsAddElem
{
	FLEXMUX_TYPE		m_nFlexMuxType;
	OnChannelsAddArray*	m_pInfo;
	BOOL operator == (OnChannelsAddElem const &/*el*/) const 
		{return FALSE;}
};

typedef ZArray<FlexMuxDataElement> FlexMuxDataParam;
typedef ZArray<OnChannelsAddElem> OnChannelsAddParam;
typedef ZArray<RESPONSE> OnChannelsDeleteParam;

class ServiceServerSink
{
	public:
		virtual ~ServiceServerSink() {};
		virtual void OnRegistService(RESPONSE_PARAMETER nResponse) = 0;
		virtual void ServiceAttachCallback(UINT ss_id,const char* serviceParam, const BYTE *pUuDataInBuffer,int nUuDataInLen) = 0;
		virtual void ChannelsAddCallback(const DAIChannelAddParam &inParam) = 0;
		virtual void ChannelsDeleteCallback(const DAIChannelDeleteParam &inParam) = 0;
		virtual void ServiceDetachCallback(REASON_PARAMETER nReason) = 0;
		virtual void UserCommandCallback(const DAIUserCmdParam& inParam) = 0;
		virtual void UserCommandAckCallback(const DAIUserCmdParam& inParam) = 0;
};

class DNI_Sink
{
	public:
		virtual ~DNI_Sink() {};
		virtual void DN_ServiceAttachCallback(const char* serviceParam, const BYTE* ddDataInBuf,int ddDataInLen, BYTE*& ddDataOutBuf, int &ddDataOutLen, RESPONSE_PARAMETER &response) = 0;
		virtual void DN_ServiceDetachCallback(REASON_PARAMETER reason, RESPONSE_PARAMETER &response) = 0;
		virtual void DN_ChannelAddCallback(const DN_ChannelAddCallbackInElem *inParamBuff,const int inParamLen, DN_ChannelAddCallbackOutElem *&outParamBuff, int &outParamLen) = 0;
		virtual void DN_ChannelDeleteCallback(const DN_ChannelDeleteCallbackInElem *inParamBuff,const int inParamLen, DN_ChannelDeleteCallbackOutElem *&outParamBuff, int &outParamLen) = 0;
		virtual void DN_UserCommandCallback(const BYTE* byteLoop) = 0;
		virtual void DN_UserCommandAckCallback(const BYTE* inByteLoop) = 0;
};

class ServiceServer
{
	public:
		virtual ~ServiceServer() {};
		virtual BOOL IsYourURL (LPCSTR szURL) = 0;
		virtual LPCSTR GetModuleID () = 0;
		virtual void RegistService(LPCSTR sURLstring) = 0;
		virtual void OnServiceAttachCallback(const BYTE *pUuDataOutBuffer, int nUuDataOutLen, RESPONSE_PARAMETER nResponse) = 0;
		virtual FlexMuxDataParam* OnChannelsAddCallback(const OnChannelsAddParam &outParam) = 0;
		virtual void OnChannelsDeleteCallback(const OnChannelsDeleteParam &outParam) = 0;
		virtual void OnTransMuxDeleteCallback(HANDLE hTransmux, RESPONSE_PARAMETER &nResponse) = 0;
		virtual void OnServiceDetachCallback(RESPONSE_PARAMETER nResponse) = 0;
		virtual void OnUserCommandAckCallback(const DAIUserCmdParam& outParam) = 0;
		virtual BOOL SendData(HANDLE hTransmux,	const BYTE *pStreamData, int nStreamDataLength) = 0;

		void SetSink (ServiceServerSink* pSink) {m_pSink = pSink;};
		ServiceServerSink* GetSink () { return m_pSink;};

		virtual void	PutChannelInChannelHandleMap(WORD scanESid,Channel* pChannel,UINT chid) = 0;

	private:
		ServiceServerSink* m_pSink;
};


class Channel
{
public:
	virtual void	ChannelDelete (RESPONSE_PARAMETER nResponse) = 0; 
	virtual BOOL	SendPacket(const BYTE *pStreamData,int nStreamDataLength) = 0;
	void			SetFlexMux(FlexMux* pFlexMux) {m_pFlexMux = pFlexMux;}
	FlexMux*	GetFlexMux() {return m_pFlexMux;}
	void		LockChannel ( ) {m_mtxLock.P ( );} 
	void		UnlockChannel ( ) {m_mtxLock.V ( );} 
	~Channel() {};
private:
	ZMutex m_mtxLock; 
	FlexMux* m_pFlexMux;
};

struct FlexMuxChannelsMapElement
{
	FlexMuxChannelsMapElement() {pChannel = NULL;}
	~FlexMuxChannelsMapElement() {}//che non venga in mente di fare delete pChannel!!!//
	Channel *pChannel;
	int nFmxInfo;
	BOOL operator == (FlexMuxChannelsMapElement const &el) const 
	{return el.pChannel == pChannel;}
};

typedef ZArray<FlexMuxChannelsMapElement> FlexMuxChannelsMap;

class FlexMux
{
	public:
		virtual ~FlexMux() {};
		virtual BOOL SendPacket(Channel *pChannel,const BYTE *pStreamData,int nStreamDataLength) = 0;
		virtual void SetFlexMuxMap(const FlexMuxChannelsMap &channelsMap) = 0;
		virtual void DeleteChannel(Channel *pChannel) = 0;
		virtual void AddChannel(const FlexMuxChannelsMapElement &element) = 0;

		BOOL IsYourChannel(Channel *pChannel) {		FlexMuxChannelsMapElement dummyEl;
													dummyEl.pChannel = pChannel;
													return ((m_ChannelsMap.Find(dummyEl) == -1) ? FALSE : TRUE);};
		void SetServiceServer (ServiceServer* pServiceServer) {m_pServiceServer = pServiceServer;};
		ServiceServer* GetServiceServer () { return m_pServiceServer; };
		friend void WaitingForDeleteFlexMuxThread(LPVOID inPar);

	private:
		ServiceServer* m_pServiceServer;
	protected:
		FlexMuxChannelsMap m_ChannelsMap;
		HANDLE m_hTAT;

};


#endif