/*
This software module was originally developed by Apple Computer, Inc.
and edited by Guido Franceschini (CSELT) in the course of development of MPEG-4. 
This software module is an implementation of a part of one or 
more MPEG-4 tools as specified by MPEG-4. 
ISO/IEC gives users of MPEG-4 free license to this
software module or modifications thereof for use in hardware 
or software products claiming conformance to MPEG-4.
Those intending to use this software module in hardware or software
products are advised that its use may infringe existing patents.
The original developer of this software module and his/her company,
the subsequent editors and their companies, and ISO/IEC have no
liability for use of this software module or modifications thereof
in an implementation.
Copyright is not released for non MPEG-4 conforming
products. Apple Computer, Inc. retains full right to use the code for its own
purpose, assign or donate the code to a third party and to
inhibit third parties from using the code for non 
MPEG-4 conforming products.
This copyright notice must be included in all copies or
derivative works. Copyright (c) 1999.
*/
/*
	$Id: MP4TrackReaderClient.cpp,v 1.1.1.1 2000/01/28 23:44:42 rls Exp $
*/
#include "MP4TrackReaderClient.h"
#include "ztl.h"

MP4TrackReaderClient::MP4TrackReaderClient( MP4Track theTrack, int ESID, int chID, int nTmpId, ServiceSink_base *sink, Service_base *pService )
: m_track(theTrack), m_id(ESID), m_chID(chID), m_nTmpId(nTmpId), m_sink(sink), m_pService (pService), m_sampleH(0), m_mode(pause),m_readerThread (readerLoop, this, THREAD_PRIORITY_BELOW_NORMAL )
{
	m_status = starting;
	m_smpMode.P();		
	RESPONSE eResponse = RESPONSE_REMOTE_ERROR;
	MP4Err err = MP4CreateTrackReader( m_track, &m_reader );
	if ( err == MP4NoErr )
	{
		err = MP4NewHandle( 4096, &m_sampleH );
	}
	if ( err == MP4NoErr )
	{
		int result = StartReaderThread();
		if ( result == -1 )
		{
			err = MP4NoMemoryErr;
			eResponse = RESPONSE_RESOURCES_EXHAUSTED;
		}
	}
	if ( err != MP4NoErr )
	{
		StreamMapElement mapElement;
		mapElement.m_nFmxInfo = 0;		// unused field
		mapElement.m_nChid    = m_chID;
		mapElement.m_nTmpId    = m_nTmpId;
		m_sink->DPI_ChannelAdd_Cnf( m_pService, mapElement, eResponse );
		m_status = finished;
	}
}

MP4TrackReaderClient::status MP4TrackReaderClient::getStatus()
{
	return m_status;
}

void MP4TrackReaderClient::requestPlay()
{
	m_mode = play;
	m_smpMode.V();		
}

void MP4TrackReaderClient::requestPause()
{
	m_mode = pause;
}

void MP4TrackReaderClient::requestStop()
{
	m_mode = stop;
	m_smpMode.V();		
}

MP4TrackReaderClient::~MP4TrackReaderClient()
{
	if ( m_sampleH )
		MP4DisposeHandle( m_sampleH );
	if ( m_reader )
		MP4DisposeTrackReader( m_reader );
}

void MP4TrackReaderClient::readerLoop()
{
	RESPONSE eResponse = RESPONSE_REMOTE_ERROR;
	DPI_TransmuxAdd_CnfParam smt;
	StreamMapElement &rMapElement = smt.ElementAt( 0 );
	rMapElement.m_nChid = m_chID;
	rMapElement.m_nTmpId = m_nTmpId;
	rMapElement.m_pData = new unsigned char[2];
	// use network byte order for the user data representing the ES_ID !!!
	*((unsigned short*)rMapElement.m_pData) = htons(m_id);
	rMapElement.m_nDataLen = 2;
	m_sink->DPI_TransmuxAdd_Cnf( m_pService, (HANDLE) this, NULL_FLEXMUX, smt );
	MP4Err err;

	m_smpMode.P();		
	do
	{
		u32 packetSize;
		err = MP4NoErr;
		if ( (m_mode == pause) || (m_status == eof) ) {
			m_smpMode.P();		
		}
		if ( m_mode == stop )
			break;
		m_status = reading;
		err = MP4TrackReaderGetNextPacket( m_reader, m_sampleH, &packetSize );
		if ( err == MP4NoErr )
		{
			m_status = writing;
			m_sink->DPI_DataReceived_Ind( m_pService, (HANDLE) this, (const BYTE *) *m_sampleH, packetSize, FALSE );
			m_status = idle;
		}
		if ( err == MP4EOF )
		{
			m_status = eof;
			err = MP4DisposeTrackReader( m_reader );
			err = MP4CreateTrackReader( m_track, &m_reader );
		}
	}
	while ( err == MP4NoErr );

	if ( err == MP4NoErr )
		eResponse = RESPONSE_OK;
	m_sink->DPI_TransmuxDelete_Cnf( m_pService, (HANDLE) this, eResponse );
	m_status = finished;
}
