/*
This software module was originally developed by Apple Computer, Inc.
in the course of development of MPEG-4. 
This software module is an implementation of a part of one or 
more MPEG-4 tools as specified by MPEG-4. 
ISO/IEC gives users of MPEG-4 free license to this
software module or modifications thereof for use in hardware 
or software products claiming conformance to MPEG-4.
Those intending to use this software module in hardware or software
products are advised that its use may infringe existing patents.
The original developer of this software module and his/her company,
the subsequent editors and their companies, and ISO/IEC have no
liability for use of this software module or modifications thereof
in an implementation.
Copyright is not released for non MPEG-4 conforming
products. Apple Computer, Inc. retains full right to use the code for its own
purpose, assign or donate the code to a third party and to
inhibit third parties from using the code for non 
MPEG-4 conforming products.
This copyright notice must be included in all copies or
derivative works. Copyright (c) 1999.
*/
/*
	$Id: MP4Movies.c,v 1.1.1.1 2000/01/28 23:44:44 rls Exp $
*/
#include "MP4Movies.h"
#include "MP4Atoms.h"
#include "MP4InputStream.h"
#include "MP4Impl.h"
#include "MovieTracks.h"
#include "FileMappingObject.h"
#include "MP4Descriptors.h"
#include "MdatDataHandler.h"

#ifdef macintosh
#include <OSUtils.h>
#endif

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <time.h>

#define GETMOOV( arg ) \
	MP4PrivateMovieRecordPtr moov; \
	MP4Err err; \
	err = MP4NoErr; \
	if ( theMovie == NULL ) \
		BAILWITHERROR( MP4BadParamErr ) \
	moov = (MP4PrivateMovieRecordPtr) arg

#define GETMOVIEATOM(arg) \
	MP4MovieAtomPtr movieAtom; \
	GETMOOV(arg); \
	movieAtom = (MP4MovieAtomPtr) moov->moovAtomPtr	

#define GETMOVIEHEADERATOM(arg) \
	MP4MovieHeaderAtomPtr movieHeaderAtom; \
	GETMOVIEATOM(arg); \
	movieHeaderAtom = (MP4MovieHeaderAtomPtr) movieAtom->mvhd	

#define GETIODATOM(arg) \
	MP4ObjectDescriptorAtomPtr iodAtom; \
	GETMOVIEATOM(arg); \
	iodAtom = (MP4ObjectDescriptorAtomPtr) movieAtom->iods;	


MP4_EXTERN ( MP4Err )
MP4DisposeMovie( MP4Movie theMovie )
{
	GETMOOV( theMovie );
	if ( moov->inputStream )
		moov->inputStream->destroy( moov->inputStream );
	if ( moov->fileMappingObject )
	{
		moov->fileMappingObject->close( moov->fileMappingObject );
		moov->fileMappingObject->destroy( moov->fileMappingObject );
	}
bail:
	TEST_RETURN( err );

	return err;
}

static MP4Err
parseMovie( MP4Movie *theMovie, MP4PrivateMovieRecordPtr moov, int openMovieFlags )
{
	MP4Err err;
	MP4AtomPtr anAtomPtr;
	int debug;
	int finished;
	
	err = MP4NoErr;
	if ( (moov == NULL) || (moov->inputStream == NULL) )
		BAILWITHERROR( MP4BadParamErr );

	debug = openMovieFlags & MP4OpenMovieDebug;

	moov->inputStream->debugging = debug;
	for ( finished = 0; !finished; )
	{
		err = MP4ParseAtom( moov->inputStream, &anAtomPtr );
		if ( err == MP4EOF )
		{
			err = MP4NoErr;
			finished = 1;
			break;
		}
		if ( err ) goto bail;
		if ( anAtomPtr->type == MP4MovieAtomType )
		{
			if ( moov->moovAtomPtr )
			{
				err = MP4BadDataErr;
				goto bail;
			}
			else
			{
				MP4MovieAtomPtr m = (MP4MovieAtomPtr) anAtomPtr;
				m->setupTracks( m, moov );
				moov->moovAtomPtr = anAtomPtr;
				
			}
		}
		else if ( anAtomPtr->type == MP4MediaDataAtomType )
		{
			if ( moov->mdat == NULL )
				moov->mdat = anAtomPtr;
		}
		/* TODO: Save other top-level atoms */
		if ( moov->inputStream->available == 0 )
			finished = 1;
	}
	if ( err == MP4NoErr )
		*theMovie = (MP4Movie) moov;
bail:
	TEST_RETURN( err );

	if ( err != MP4NoErr )
	{
		if ( moov )
			free( moov );
	}
	return err;
}

MP4_EXTERN ( MP4Err )
MP4NewMovieFromHandle( MP4Movie *theMovie, MP4Handle movieH, u32 newMovieFlags )
{
	MP4Err err;
	MP4PrivateMovieRecordPtr moov;
	u32 handleSize;

	err = MP4NoErr;
	moov = (MP4PrivateMovieRecordPtr) calloc( 1, sizeof(MP4PrivateMovieRecord) );
	TESTMALLOC( moov );
	moov->referenceCount = 1;
	moov->moovAtomPtr = NULL;
	err = MP4GetHandleSize( movieH, &handleSize ); if (err) goto bail;
	err = MP4CreateMemoryInputStream( *movieH, handleSize, &moov->inputStream ); if (err) goto bail;
	err = parseMovie( theMovie, moov, newMovieFlags ); if (err) goto bail;
bail:
	TEST_RETURN( err );
	return err;
}

MP4_EXTERN ( MP4Err )
MP4OpenMovieFile( MP4Movie *theMovie, const char *movieURL, int openMovieFlags )
{
	MP4Err err;
	MP4PrivateMovieRecordPtr moov;
	
	err = MP4NoErr;
	moov = (MP4PrivateMovieRecordPtr) calloc( 1, sizeof(MP4PrivateMovieRecord) );
	TESTMALLOC( moov );
	moov->referenceCount = 1;
	moov->moovAtomPtr = NULL;
	err = MP4CreateFileMappingObject( (char*) movieURL, (struct FileMappingObjectRecord **) &moov->fileMappingObject ); if (err) goto bail;
	err = MP4CreateFileMappingInputStream( moov->fileMappingObject, &moov->inputStream ); if (err) goto bail;
	err = parseMovie( theMovie, moov, openMovieFlags ); if (err) goto bail;
bail:
	TEST_RETURN( err );
	return err;
}



/* */

MP4_EXTERN ( MP4Err )
MP4GetMovieTrackCount( MP4Movie theMovie, u32* outTrackCount )
{
	GETMOVIEATOM( theMovie );
	*outTrackCount = movieAtom->getTrackCount( movieAtom );
bail:
	TEST_RETURN( err );

	return err;
}

MP4_EXTERN ( MP4Err )
MP4GetMovieTimeScale( MP4Movie theMovie, u32* outTimeScale )
{
	GETMOVIEHEADERATOM( theMovie );
	*outTimeScale = movieHeaderAtom->timeScale;
bail:
	TEST_RETURN( err );

	return err;
}

MP4_EXTERN ( MP4Err )
MP4GetMovieUserData( MP4Movie theMovie, MP4UserData* outUserData )
{
	MP4UserData udta;
	GETMOVIEATOM( theMovie );
	udta = (MP4UserData) movieAtom->udta;
	if ( movieAtom->udta == 0 )
	{
		err = MP4NewUserData( &udta ); if (err) goto bail;
		err = movieAtom->addAtom( movieAtom, (MP4AtomPtr) udta ); if (err) goto bail;
	}
	*outUserData = (MP4UserData) udta;
bail:
	TEST_RETURN( err );

	return err;
}

MP4Err makeESD( MP4Movie theMovie, u32 trackNumber, u64 cts, MP4DescriptorPtr *outDesc )
{
	MP4Err MP4GetMediaSampleDescIndex( MP4Media theMedia, u64 desiredTime, u32 *outSampleDescriptionIndex );
	MP4Err MP4GetMediaESD( MP4Media theMedia, u32 index, MP4ES_DescriptorPtr *outESD, u32 *outDataReferenceIndex );
	MP4Err MP4CreateSLConfigDescriptor( u32 tag, u32 size, u32 bytesRead, MP4DescriptorPtr *outDesc );
    MP4Err MP4CreateES_Descriptor( u32 tag, u32 size, u32 bytesRead, MP4DescriptorPtr *outDesc );

	MP4Err err;
	MP4ES_DescriptorPtr esd;
	MP4ES_DescriptorPtr esdInFile;
	MP4SLConfigDescriptorPtr slconfig;
	MP4TrackReferenceTypeAtomPtr dpnd;
	MP4Track theTrack;
	MP4Media theMedia;
	u32 sampleDescIndex;
	
	err = MP4GetMovieTrack( theMovie, trackNumber, &theTrack ); if (err) goto bail;
	err = MP4GetTrackMedia( theTrack, &theMedia ); if (err) goto bail;
	err = MP4GetMediaSampleDescIndex( theMedia, cts, &sampleDescIndex ); if (err) goto bail;
    err = MP4CreateES_Descriptor( MP4ES_DescriptorTag, 0, 0, (MP4DescriptorPtr*) &esd ); if (err) goto bail;
	err = MP4GetMediaESD( theMedia, sampleDescIndex, &esdInFile, 0 ); if (err) goto bail;
	*esd = *esdInFile;
	esd->ESID = trackNumber;

	err = MP4GetTrackReferenceType( theTrack, MP4StreamDependenceAtomType, &dpnd ); if (err) goto bail;
	if ( dpnd && (dpnd->trackIDCount) )
		esd->dependsOnES = dpnd->trackIDs[0];
	/*
		handle SLConfig
	*/
	slconfig = (MP4SLConfigDescriptorPtr) esd->slConfig;
	if ( slconfig == NULL )
		BAILWITHERROR( MP4InvalidMediaErr );
	if ( slconfig->predefined == SLConfigPredefinedMP4 )
	{
		u32 movieTimeScale;
		u32 mediaTimeScale;
        u32 odEncodingType;
        u32 clockReferenceID;
		
		err = MP4GetMovieTimeScale( theMovie, &movieTimeScale ); if (err) goto bail;
		err = MP4GetMediaTimeScale( theMedia, &mediaTimeScale ); if (err) goto bail;
		
		clockReferenceID = slconfig->OCRESID;
		/* make real one */
		//slconfig->destroy( (MP4DescriptorPtr) slconfig );
		err = MP4CreateSLConfigDescriptor( MP4SLConfigDescriptorTag, 0, 0,
                                           (MP4DescriptorPtr *) &slconfig ); if (err) goto bail;
        err = MP4GetMovieODEncodingType( theMovie, &odEncodingType ); if (err) goto bail;
        switch ( odEncodingType )
        {
           case MP4ODEncodingHinted:
               slconfig->predefined                   = 0;
               slconfig->useAccessUnitStartFlag       = 1;
               slconfig->useAccessUnitEndFlag         = 0;
               slconfig->useRandomAccessPointFlag     = 1;
               slconfig->useRandomAccessUnitsOnlyFlag = 0;
               slconfig->usePaddingFlag               = 0;
               slconfig->useTimestampsFlag            = 1;
               slconfig->useIdleFlag                  = 0;
               slconfig->durationFlag                 = 0;
               slconfig->timestampResolution          = mediaTimeScale;
               slconfig->OCRResolution                = movieTimeScale;
               slconfig->timestampLength              = 28;
// GUIDO               slconfig->OCRLength                    = 28;
               slconfig->OCRLength                    = 0;
               slconfig->AULength                     = 0;
               slconfig->instantBitrateLength         = 0;
               slconfig->degradationPriorityLength    = 0;
               slconfig->AUSeqNumLength               = 0;
               slconfig->packetSeqNumLength           = 0;
               slconfig->timeScale                    = mediaTimeScale;
               slconfig->AUDuration                   = mediaTimeScale;
               slconfig->CUDuration                   = mediaTimeScale;
               slconfig->startDTS                     = 0;
               slconfig->startCTS                     = 0;
               slconfig->OCRESID                      = clockReferenceID;
               break;

           default:
               slconfig->predefined                   = 0;
               slconfig->useAccessUnitStartFlag       = 0;
               slconfig->useAccessUnitEndFlag         = 0;
               slconfig->useRandomAccessPointFlag     = 1;
               slconfig->useRandomAccessUnitsOnlyFlag = 0;
               slconfig->usePaddingFlag               = 0;
               slconfig->useTimestampsFlag            = 1;
               slconfig->useIdleFlag                  = 0;
               slconfig->durationFlag                 = 0;
               slconfig->timestampResolution          = mediaTimeScale;
               slconfig->OCRResolution                = movieTimeScale;
               slconfig->timestampLength              = 32;
// GUIDO               slconfig->OCRLength                    = 32;
               slconfig->OCRLength                    = 0;
               slconfig->AULength                     = 0;
               slconfig->instantBitrateLength         = 0;
               slconfig->degradationPriorityLength    = 0;
               slconfig->AUSeqNumLength               = 0;
               slconfig->packetSeqNumLength           = 5;
               slconfig->timeScale                    = mediaTimeScale;
               slconfig->AUDuration                   = mediaTimeScale;
               slconfig->CUDuration                   = mediaTimeScale;
               slconfig->startDTS                     = 0;
               slconfig->startCTS                     = 0;
               slconfig->OCRESID                      = clockReferenceID;
               break;               
        }
	    esd->slConfig = (MP4DescriptorPtr) slconfig;
	}
	*outDesc = (MP4DescriptorPtr) esd;
bail:
	TEST_RETURN( err );

	return err;
}

MP4_EXTERN ( MP4Err )
MP4GetMovieInitialObjectDescriptor( MP4Movie theMovie, MP4Handle outDescriptorH )
{
	MP4Err MP4CreateSLConfigDescriptor( u32 tag, u32 size, u32 bytesRead, MP4DescriptorPtr *outDesc );
	MP4Err MP4GetMediaESD( MP4Media theMedia, u32 index, MP4ES_DescriptorPtr *outESD, u32 *outDataReferenceIndex );
	
	MP4InitialObjectDescriptorPtr iodDesc;
    MP4LinkedList incDescriptors;
	GETIODATOM( theMovie );

	if ( iodAtom->ODSize )
	{
		u32 count;
		u32 i;
		u32 trackCount;

		err = MP4GetMovieTrackCount( theMovie, &trackCount ); if (err) goto bail;
		iodDesc = (MP4InitialObjectDescriptorPtr) iodAtom->descriptor;
		err = MP4GetListEntryCount( iodDesc->ES_ID_IncDescriptors, &count ); if (err) goto bail;
		
		/*
			get and rewrite ES_Descriptors, placing each in iodDesc
		*/
		for ( i = 0; i < count; i++ )
		{
			
			MP4ES_ID_IncDescriptorPtr inc;
			MP4ES_DescriptorPtr       esd;
			
			err = MP4GetListEntry( iodDesc->ES_ID_IncDescriptors, i, (char **) &inc ); if (err) goto bail;				
			err = makeESD( theMovie, inc->trackID, 0, (MP4DescriptorPtr *) &esd ); if (err) goto bail;			
			/* add esd to iodDesc */
			err = iodDesc->addDescriptor( (MP4DescriptorPtr) iodDesc, (MP4DescriptorPtr) esd ); if (err) goto bail;
		}
		/*
          err = MP4DeleteLinkedList( iodDesc->ES_ID_IncDescriptors ); if (err) goto bail;
        */
        incDescriptors = iodDesc->ES_ID_IncDescriptors;
		iodDesc->ES_ID_IncDescriptors = NULL;
		iodDesc->tag = MP4InitialObjectDescriptorTag;
		err = iodDesc->calculateSize( (MP4DescriptorPtr) iodDesc ); if (err) goto bail;
		err = MP4SetHandleSize( outDescriptorH, iodDesc->size ); if (err) goto bail;
		err = iodDesc->serialize((MP4DescriptorPtr) iodDesc, *outDescriptorH ); if (err) goto bail;
        err = iodDesc->removeESDS( (MP4DescriptorPtr) iodDesc ); if (err) goto bail;
        iodDesc->ES_ID_IncDescriptors = incDescriptors;
        iodDesc->tag = MP4_IOD_Tag;
	}
bail:
	TEST_RETURN( err );

	return err;
}

MP4_EXTERN ( MP4Err )
MP4GetMovieIndTrack( MP4Movie theMovie, u32 trackIndex, MP4Track *outTrack )
{
	MP4AtomPtr aTrack;
	GETMOVIEATOM( theMovie );
	if ( (trackIndex == 0) || (trackIndex > movieAtom->getTrackCount(movieAtom)) )
		BAILWITHERROR( MP4BadParamErr )
	err = movieAtom->getIndTrack( movieAtom, trackIndex, &aTrack ); if (err) goto bail;
	if ( aTrack == NULL )
		BAILWITHERROR( MP4BadDataErr )
	*outTrack = (MP4Track) aTrack;
bail:
	TEST_RETURN( err );

	return err;
}

MP4_EXTERN ( MP4Err )
MP4GetMovieTrack( MP4Movie theMovie, u32 trackID, MP4Track *outTrack )
{
	u32 i;
	u32 trackCount;
	
	MP4Err err;
	err = MP4GetMovieTrackCount( theMovie, &trackCount ); if (err) goto bail;
	
	for ( i = 1; i <= trackCount; i++ )
	{
		MP4Track t;
		u32  id;
		err = MP4GetMovieIndTrack( theMovie, i, &t ); if (err) goto bail;
		err = MP4GetTrackID( t, &id ); if (err) goto bail;
		if ( id == trackID )
		{
			*outTrack = t;
			break;
		}
		else if ( i == trackCount )
			err = MP4BadParamErr;
	}
bail:
	TEST_RETURN( err );

	return err;
}

MP4_EXTERN ( MP4Err )
MP4SetMovieTimeScale( MP4Movie theMovie, u32 timeScale )
{
	GETMOVIEATOM( theMovie );
	if ( timeScale == 0 )
		BAILWITHERROR( MP4BadParamErr );
	assert( movieAtom->setTimeScale );
	err = movieAtom->setTimeScale( movieAtom, timeScale ); if (err) goto bail;
bail:
	return err;
}

MP4_EXTERN ( MP4Err )
MP4NewMovieTrack( MP4Movie theMovie, u32 newTrackFlags, MP4Track *outTrack )
{
	MP4TrackAtomPtr trak;
	GETMOVIEATOM( theMovie );
	err = movieAtom->newTrack( movieAtom, newTrackFlags, (MP4AtomPtr*) &trak ); if (err) goto bail;
	err = trak->setMdat( trak, moov->mdat ); if (err) goto bail;
	trak->moov = moov;
	*outTrack = (MP4Track) trak;
bail:
	TEST_RETURN( err );
	return err;
}

MP4_EXTERN ( MP4Err )
MP4NewMovieTrackWithID( MP4Movie theMovie, u32 newTrackFlags, u32 newTrackID, MP4Track *outTrack )
{
	MP4TrackAtomPtr trak;
	GETMOVIEATOM( theMovie );
	err = movieAtom->newTrackWithID( movieAtom, newTrackFlags, newTrackID, (MP4AtomPtr*) &trak ); if (err) goto bail;
	err = trak->setMdat( trak, moov->mdat ); if (err) goto bail;
	trak->moov = moov;
	*outTrack = (MP4Track) trak;
bail:
	TEST_RETURN( err );
	return err;
}

#if 0
MP4_EXTERN ( MP4Err )
MP4GetMovieInitialBIFSTrack( MP4Movie theMovie, MP4Track *outBIFSTrack )
{
	GETMOVIEATOM( theMovie );
	if ( moov->initialBIFS == NULL )
	{
		err = MP4NewMovieTrack( theMovie, MP4NewTrackIsVisual, &moov->initialBIFS ); if (err) goto bail;
	}
	*outBIFSTrack = moov->initialBIFS;
bail:
	TEST_RETURN( err );

	return err;
}

MP4_EXTERN ( MP4Err )
MP4SetMovieInitialBIFSTrack( MP4Movie theMovie, MP4Track theBIFSTrack )
{
	GETMOVIEATOM( theMovie );
	moov->initialBIFS = theBIFSTrack;
bail:
	TEST_RETURN( err );

	return err;
}

MP4_EXTERN ( MP4Err )
MP4SetMovieInitialODTrack( MP4Movie theMovie, MP4Track theODTrack )
{
	GETMOVIEATOM( theMovie );
	moov->initialOD = theODTrack;
bail:
	TEST_RETURN( err );

	return err;
}
#endif

MP4_EXTERN ( MP4Err ) MP4SetMovieODEncodingType( MP4Movie theMovie, u32 encodingType )
{
	GETMOVIEATOM( theMovie );
	moov->odEncodingType = encodingType;
bail:
	TEST_RETURN( err );

	return err;
}

MP4_EXTERN ( MP4Err ) MP4GetMovieODEncodingType( MP4Movie theMovie, u32 *encodingType )
{
	GETMOVIEATOM( theMovie );
    if ( encodingType == NULL )
    {
       BAILWITHERROR( MP4BadParamErr );
    }
    
	*encodingType = moov->odEncodingType;
bail:
	TEST_RETURN( err );

	return err;
}

MP4_EXTERN ( MP4Err )
MP4NewMovie( MP4Movie *outMovie, u32 initialODID,
						   u8 OD_profileAndLevel, u8 scene_profileAndLevel,
                           u8 audio_profileAndLevel, u8 visual_profileAndLevel,
                           u8 graphics_profileAndLevel )
{
	MP4Err MP4CreateInitialObjectDescriptor( u32 tag, u32 size, u32 bytesRead, MP4DescriptorPtr *outDesc );
	MP4Err MP4CreateMediaDataAtom( MP4MediaDataAtomPtr *outAtom );
	MP4Err MP4CreateMovieAtom( MP4MovieAtomPtr *outAtom );
	MP4Err MP4CreateMovieHeaderAtom( MP4MovieHeaderAtomPtr *outAtom );
	MP4Err MP4CreateObjectDescriptorAtom( MP4ObjectDescriptorAtomPtr *outAtom );
	MP4Err MP4GetCurrentTime( u64 *outTime );
	   
	u64 now;
	MP4Err err;
	MP4PrivateMovieRecordPtr movie;
	MP4MovieAtomPtr            moov;
	MP4MovieHeaderAtomPtr      mvhd;
	MP4ObjectDescriptorAtomPtr iods;
	MP4MediaDataAtomPtr        mdat;
	MP4InitialObjectDescriptorPtr iod;

	movie = (MP4PrivateMovieRecordPtr) calloc( 1, sizeof(MP4PrivateMovieRecord) );
	if ( movie == NULL )
	{
		err = MP4NoMemoryErr;
		goto bail;
	}
	movie->referenceCount     = 1;
	err = MP4GetCurrentTime( &now ); if (err) goto bail;
	err = MP4CreateMovieAtom( (MP4MovieAtomPtr *) &movie->moovAtomPtr ); if (err) goto bail;
	moov = (MP4MovieAtomPtr) movie->moovAtomPtr;
	err = MP4CreateMovieHeaderAtom( &mvhd ); if (err) goto bail;
	mvhd->nextTrackID = 1;
	mvhd->creationTime = now;
	mvhd->modificationTime = now;
	err = moov->addAtom( moov, (MP4AtomPtr) mvhd ); if (err) goto bail;
	moov->setTimeScale( moov, 600 );
	err = MP4CreateObjectDescriptorAtom( &iods ); if (err) goto bail;
	err = MP4CreateInitialObjectDescriptor( MP4_IOD_Tag, 0, 0, (MP4DescriptorPtr*)&iod ); if (err) goto bail;
	iod->objectDescriptorID = initialODID;
	iod->OD_profileAndLevel = OD_profileAndLevel;
	iod->scene_profileAndLevel = scene_profileAndLevel;
	iod->audio_profileAndLevel = audio_profileAndLevel;
	iod->visual_profileAndLevel = visual_profileAndLevel;
	iod->graphics_profileAndLevel = graphics_profileAndLevel;
	err = iods->setDescriptor( (MP4AtomPtr) iods, (char*) iod ); if (err) goto bail;
	err = moov->addAtom( moov, (MP4AtomPtr) iods ); if (err) goto bail;
	err = MP4CreateMediaDataAtom( &mdat );
	movie->mdat = (MP4AtomPtr) mdat;
	{
	   MP4DataHandlerPtr dh;
	      err = MP4CreateMdatDataHandler( mdat, &dh ); if (err) goto bail;
	      movie->inMemoryDataHandler = dh;
	}
	*outMovie = (MP4Movie) movie;
bail:
	TEST_RETURN( err );

	return err;	
}

MP4Err MP4MovieCreateDescriptors( MP4Movie theMovie )
{
	MP4Err MP4CreateES_ID_IncDescriptor( u32 tag, u32 size, u32 bytesRead, MP4DescriptorPtr *outDesc );
	MP4ObjectDescriptorAtomPtr iods;
	MP4InitialObjectDescriptorPtr iod;
	
	GETMOVIEATOM( theMovie );
	iods = (MP4ObjectDescriptorAtomPtr) movieAtom->iods;
	if ( iods == NULL )
		BAILWITHERROR( MP4InvalidMediaErr );
	iod = (MP4InitialObjectDescriptorPtr) iods->descriptor;
	if ( iod == NULL )
		BAILWITHERROR( MP4InvalidMediaErr );

	err = movieAtom->calculateDuration( movieAtom ); if (err) goto bail;
#if 0
	if ( moov->initialBIFS )
	{
		MP4ES_ID_IncDescriptorPtr inc;
		err = MP4CreateES_ID_IncDescriptor( MP4ES_ID_IncDescriptorTag, 0, 0, (MP4DescriptorPtr*) &inc ); if (err) goto bail;
		err = MP4GetTrackID( moov->initialBIFS, &inc->trackID ); if (err) goto bail;
		err = iod->addDescriptor( (MP4DescriptorPtr) iod, (MP4DescriptorPtr) inc ); if (err) goto bail;
	}
	if ( moov->initialOD )
	{
		MP4ES_ID_IncDescriptorPtr inc;
		err = MP4CreateES_ID_IncDescriptor( MP4ES_ID_IncDescriptorTag, 0, 0, (MP4DescriptorPtr*) &inc ); if (err) goto bail;
		err = MP4GetTrackID( moov->initialOD, &inc->trackID ); if (err) goto bail;
		err = iod->addDescriptor( (MP4DescriptorPtr) iod, (MP4DescriptorPtr) inc ); if (err) goto bail;
	}
#endif
bail:
	TEST_RETURN( err );

	return err;	
}

MP4Err MP4MovieAddTrackES_IDToIOD( MP4Movie theMovie, MP4Track theTrack )
{
	MP4Err MP4CreateES_ID_IncDescriptor( u32 tag, u32 size, u32 bytesRead, MP4DescriptorPtr *outDesc );
	MP4ObjectDescriptorAtomPtr iods;
	MP4InitialObjectDescriptorPtr iod;
	MP4ES_ID_IncDescriptorPtr inc;
	
	GETMOVIEATOM( theMovie );
	if ( theTrack == 0 )
		BAILWITHERROR( MP4BadParamErr );
	iods = (MP4ObjectDescriptorAtomPtr) movieAtom->iods;
	if ( iods == NULL )
		BAILWITHERROR( MP4InvalidMediaErr );
	iod = (MP4InitialObjectDescriptorPtr) iods->descriptor;
	if ( (iod == NULL) || (iod->ES_ID_IncDescriptors == NULL) )
		BAILWITHERROR( MP4InvalidMediaErr );

	err = MP4CreateES_ID_IncDescriptor( MP4ES_ID_IncDescriptorTag, 0, 0, (MP4DescriptorPtr*) &inc );
    if (err) goto bail;
	err = MP4GetTrackID( theTrack, &inc->trackID );
    if (err) goto bail;
	err = iod->addDescriptor( (MP4DescriptorPtr) iod, (MP4DescriptorPtr) inc );
    if (err) goto bail;
bail:
	TEST_RETURN( err );
	return err;	
}

#define MAC_EPOCH 2082758400;

MP4Err MP4GetCurrentTime( u64 *outTime )
{
	MP4Err err;
	unsigned long calctime;
	u64 ret;
	err = MP4NoErr;
	if ( outTime == NULL )
		BAILWITHERROR( MP4BadParamErr )
#ifdef macintosh
	GetDateTime( &calctime );
#else
	{
		time_t t;
		t = time( NULL );
		calctime = t + MAC_EPOCH;
	}
#endif
	ret = calctime;
	*outTime = ret;
bail:
	TEST_RETURN( err );

	return err;
}
