/*
This software module was originally developed by Apple Computer, Inc.
in the course of development of MPEG-4. 
This software module is an implementation of a part of one or 
more MPEG-4 tools as specified by MPEG-4. 
ISO/IEC gives users of MPEG-4 free license to this
software module or modifications thereof for use in hardware 
or software products claiming conformance to MPEG-4.
Those intending to use this software module in hardware or software
products are advised that its use may infringe existing patents.
The original developer of this software module and his/her company,
the subsequent editors and their companies, and ISO/IEC have no
liability for use of this software module or modifications thereof
in an implementation.
Copyright is not released for non MPEG-4 conforming
products. Apple Computer, Inc. retains full right to use the code for its own
purpose, assign or donate the code to a third party and to
inhibit third parties from using the code for non 
MPEG-4 conforming products.
This copyright notice must be included in all copies or
derivative works. Copyright (c) 1999.
*/
/*
	$Id: MediaAtom.c,v 1.1.1.1 2000/01/28 23:44:43 rls Exp $
*/

#include "MP4Atoms.h"
#include <stdlib.h>

static void destroy( MP4AtomPtr s )
{
   MP4Err err;
   MP4MediaAtomPtr self;
   u32 i;
   err = MP4NoErr;
   self = (MP4MediaAtomPtr) s;
   if ( self == NULL )	BAILWITHERROR( MP4BadParamErr )
			   DESTROY_ATOM_LIST
			   if ( self->super )
			      self->super->destroy( s );
  bail:
   TEST_RETURN( err );

   return;
}

static 	MP4Err calculateDuration( struct MP4MediaAtom *self )
{
   MP4Err err;
   u32 mediaDuration;
   MP4MediaInformationAtomPtr minf;
   MP4MediaHeaderAtomPtr      mdhd;

   err = MP4NoErr;
   minf = (MP4MediaInformationAtomPtr) self->information;
   if ( minf == NULL )
      BAILWITHERROR( MP4InvalidMediaErr );
   mdhd = (MP4MediaHeaderAtomPtr) self->mediaHeader;
   if ( mdhd == NULL )
      BAILWITHERROR( MP4InvalidMediaErr );
   err = minf->getMediaDuration( minf, &mediaDuration ); if (err) goto bail;
   mdhd->duration = mediaDuration;
  bail:
   TEST_RETURN( err );

   return err;
}

static MP4Err mdatMoved( struct MP4MediaAtom* self, s32 mdatOffset )
{
   MP4Err err;
   MP4MediaInformationAtomPtr minf;	
   err = MP4NoErr;

   minf = (MP4MediaInformationAtomPtr) self->information;
   if ( minf == NULL )
      BAILWITHERROR( MP4InvalidMediaErr );
   err = minf->mdatMoved( minf, mdatOffset ); if (err) goto bail;	
  bail:
   TEST_RETURN( err );
   return err;
}

static MP4Err addAtom( MP4MediaAtomPtr self, MP4AtomPtr atom )
{
   MP4Err err;
   err = MP4NoErr;
   if ( atom == NULL )
      BAILWITHERROR( MP4BadParamErr );
   err = MP4AddListEntry( atom, self->atomList ); if (err) goto bail;
   switch( atom->type )
   {
      case MP4MediaHeaderAtomType:
	 if ( self->mediaHeader )
	    BAILWITHERROR( MP4BadDataErr )
	       self->mediaHeader = atom;
	 break;
			
      case MP4HandlerAtomType:
	 if ( self->handler )
	    BAILWITHERROR( MP4BadDataErr )
	       self->handler = atom;
	 break;
			
      case MP4MediaInformationAtomType:
	 if ( self->information )
	    BAILWITHERROR( MP4BadDataErr )
	       self->information = atom;
	 break;
   }
  bail:
   TEST_RETURN( err );

   return err;
}

static MP4Err setupNew( struct MP4MediaAtom *self, MP4AtomPtr track, u32 mediaType, u32 timeScale, MP4Handle dataHandlerH )
{
   MP4Err MP4CreateMediaHeaderAtom( MP4MediaHeaderAtomPtr *outAtom );
   MP4Err MP4CreateMediaInformationAtom( MP4MediaInformationAtomPtr *outAtom );
   MP4Err MP4CreateHandlerAtom( MP4HandlerAtomPtr *outAtom );
   MP4Err MP4GetCurrentTime( u64 *outTime );
   void MP4TypeToString( u32 inType, char* ioStr );

   char name[ 8 ];
   MP4Err err;
   u64 currentTime;
   MP4MediaHeaderAtomPtr mdhd;
   MP4MediaInformationAtomPtr minf;
   MP4HandlerAtomPtr hdlr;
   MP4AtomPtr mdat;
	
   err = MP4NoErr;
   self->mediaTrack = track;
   mdat = ((MP4TrackAtomPtr) track)->mdat;
   err = MP4CreateMediaHeaderAtom( &mdhd ); if (err) goto bail;
   err = MP4CreateMediaInformationAtom( &minf ); if (err) goto bail;
   err = MP4CreateHandlerAtom( &hdlr ); if (err) goto bail;
	
   err = MP4GetCurrentTime( &currentTime ); if (err) goto bail;
   mdhd->timeScale = timeScale;
   mdhd->creationTime = currentTime;
   mdhd->modificationTime = currentTime;
   mdhd->packedLanguage = 21956;
	
   hdlr->handlerType = mediaType;
   MP4TypeToString( mediaType, name );
   hdlr->nameLength = 5;
   hdlr->nameUTF8 = calloc( 1, hdlr->nameLength );
   memcpy( hdlr->nameUTF8, name,  hdlr->nameLength );
	
   err = minf->setupNewMedia( minf, mediaType, dataHandlerH, mdat );

   err = addAtom( self, (MP4AtomPtr) mdhd ); if (err) goto bail;
   err = addAtom( self, (MP4AtomPtr) minf ); if (err) goto bail;
   err = addAtom( self, (MP4AtomPtr) hdlr ); if (err) goto bail;
  bail:
   TEST_RETURN( err );

   return err;
}

static MP4Err addSampleReference( struct MP4MediaAtom *self, u64 dataOffset, u32 sampleCount,
				  MP4Handle durationsH, MP4Handle sizesH, MP4Handle sampleEntryH,
				  MP4Handle decodingOffsetsH, MP4Handle syncSamplesH )
{
   MP4Err err;
   MP4MediaInformationAtomPtr minf;
	
   err = MP4NoErr;
   minf = (MP4MediaInformationAtomPtr) self->information;
   assert( minf );
   assert( minf->addSampleReference );
   err = minf->addSampleReference( minf, dataOffset, sampleCount, durationsH, sizesH, sampleEntryH, decodingOffsetsH, syncSamplesH ); if (err) goto bail;
  bail:
   TEST_RETURN( err );

   return err;
}

static 	MP4Err addSamples( struct MP4MediaAtom *self, MP4Handle sampleH, u32 sampleCount,
			   MP4Handle durationsH, MP4Handle sizesH, MP4Handle sampleEntryH,
			   MP4Handle decodingOffsetsH, MP4Handle syncSamplesH )
{
   MP4Err err;
   MP4MediaInformationAtomPtr minf;
	
   err = MP4NoErr;
   minf = (MP4MediaInformationAtomPtr) self->information;
   assert( minf );
   assert( minf->addSamples );
   err = minf->addSamples( minf, sampleH, sampleCount, durationsH, sizesH, sampleEntryH, decodingOffsetsH, syncSamplesH ); if (err) goto bail;
  bail:
   TEST_RETURN( err );

   return err;
}

static MP4Err serialize( struct MP4Atom* s, char* buffer )
{
   MP4Err err;
   MP4MediaAtomPtr self = (MP4MediaAtomPtr) s;
   err = MP4NoErr;
	
   err = MP4SerializeCommonBaseAtomFields( s, buffer ); if (err) goto bail;
   buffer += self->bytesWritten;
   SERIALIZE_ATOM_LIST( atomList );
   assert( self->bytesWritten == self->size );
  bail:
   TEST_RETURN( err );

   return err;
}

static MP4Err calculateSize( struct MP4Atom* s )
{
   MP4Err err;
   MP4MediaAtomPtr self = (MP4MediaAtomPtr) s;
   err = MP4NoErr;
	
   err = MP4CalculateBaseAtomFieldSize( s ); if (err) goto bail;
   ADD_ATOM_LIST_SIZE( atomList );
  bail:
   TEST_RETURN( err );

   return err;
}


static MP4Err createFromInputStream( MP4AtomPtr s, MP4AtomPtr proto, MP4InputStreamPtr inputStream )
{
   PARSE_ATOM_LIST(MP4MediaAtom)
      bail:
   TEST_RETURN( err );

   return err;
}

MP4Err MP4CreateMediaAtom( MP4MediaAtomPtr *outAtom )
{
   MP4Err err;
   MP4MediaAtomPtr self;
	
   self = (MP4MediaAtomPtr) calloc( 1, sizeof(MP4MediaAtom) );
   if ( self == NULL )	BAILWITHERROR( MP4NoMemoryErr )

			   err = MP4CreateBaseAtom( (MP4AtomPtr) self );
   if ( err ) goto bail;
   self->type = MP4MediaAtomType;
   self->name                = "media";
   self->createFromInputStream = (cisfunc) createFromInputStream;
   self->destroy             = destroy;
   err = MP4MakeLinkedList( &self->atomList ); if (err) goto bail;
   self->calculateSize         = calculateSize;
   self->serialize             = serialize;
   self->addSamples = addSamples;
   self->calculateDuration = calculateDuration;
   self->setupNew = setupNew;
   self->mdatMoved = mdatMoved;
   self->addSampleReference = addSampleReference;
   *outAtom = self;
  bail:
   TEST_RETURN( err );

   return err;
}
