/***********************************************************************
    This software module  was  originally  developed by Zvi  Lifshitz
    (Triton R&D Ltd.) and edited by Mauro Rossi (CSELT),
    Guido Franceschini (CSELT) and Michael Stepping (FTK)
    in the course of development of the MPEG-4 DMIF (ISO/IEC 14496-6)
    standard.  This software module is an implementation of a part of
    one or more MPEG-4  DMIF  (ISO/IEC 14496-6)  tools  as  specified
    by  the  MPEG-4 DMIF (ISO/IEC 14496-6)  standard.  ISO/IEC  gives
    users of  the  MPEG-4  DMIF  (ISO/IEC 14496-6)  free  license  to
    this software module or modifications thereof for use in hardware
    or software products claiming  conformance  to  the  MPEG-4  DMIF 
    (ISO/IEC 14496-6). Those intending to use this software module in
    hardware or software  products  are  advised  that  its  use  may
    infringe   existing  patents.  The  original  developer  of  this
    software module and his/her company, the subsequent  editors  and
    their  companies,  and  ISO/IEC have no liability for use of this
    software module or modifications thereof  in  an  implementation.
    Copyright  is not released for non MPEG-4 DMIF  (ISO/IEC 14496-6)
    conforming  products. Triton R&D Ltd.
    retains full  right to use the code for  its  own purpose, assign
    or donate the code to a third party and to inhibit third  parties
    from  using  the  code  for  non  MPEG-4  DMIF  (ISO/IEC 14496-6)
    conforming products. This copyright notice must  be  included  in
    all copies or derivative works. Copyright  1998.
************************************************************************/

// Contact: guido.franceschini@cselt.it

/**************************************************************************
 * Project Im1Player
 * INTERNATIONAL STANDARD ORGANIZATION
 * ISO/IEC JTC1/SC29/WG11
 * 
 * SUBSYSTEM: DMIF
 * FILE:      FileTrivInstance.h
 * AUTHOR:    Mauro Rossi (CSELT)
 * OVERVIEW:  Definitions of FileTrivInstance, the class that implements
 *			  the DMIF instance for local storage with Trivial File Format.
 * 
 **************************************************************************/

#if !defined(_FileTrivInstance_h)	// Sentry, use file only if it's not already included.
#define _FileTrivInstance_h

#include "DMIF.h"
#include "DMIFClient.h"

class FileTrivInstance : public ServiceClient
{
private:				// private functions
	virtual BOOL IsYourURL (LPCSTR szURL);
	virtual LPCSTR GetModuleID () {return "FileTrivService";}

	virtual RESULT DPI_ServiceAttach_Req(LPCSTR szURL, const BYTE *pUuData, int nUuDataLen) ;

	// Service Detach functions
	virtual RESULT DPI_ServiceDetach_Req(REASON nReason) ;
	
	// Channel Add functions
	virtual RESULT DPI_ChannelAdd_Req(const DAIChannelAddParam &inParam) ;
	
	// Channel Delete functions
	virtual RESULT DPI_ChannelDelete_Req(const DAIChannelDeleteParam &inParam) ;

	// User Command function
	virtual RESULT DPI_UserCommand_Req(const DAIUserCmdParam& inParam) ;
	virtual	RESULT DPI_UserCommand_Ack_Req(const DAIUserCmdParam& outParam) 
	{ outParam; return (RESULT_ERROR); }

	// Function for data transmission
	virtual RESULT DPI_SendData(UINT ch_id,const char* data,int length) 
	{ ch_id; data; length; return (RESULT_ERROR); }

	RESPONSE_PARAMETER Connect (LPCSTR szURL, ZString *initialOD = NULL);
	void Disconnect ();
	void OpenChannel (BYTE* pData, int nDataLen);
	bool IsMyFile (LPCSTR szFileName);
	bool CloseChannels ( const DAIChannelDeleteParam &inParam ) ;


	void PlayChannel (int nTag);
	void Rewind ();
	void StopChannel (int nTag, RESPONSE_PARAMETER nTerminationCode);
	void FileThread ();
	 int GetByte ();
	UINT GetNewChannelId();
	 int EsidFromChid(UINT chid);
	void ChidEsidStore(UINT chid,int esid);
	void ChidEsidRemove(UINT chid);
	 int TagFromChid(UINT chid);
	 int TagFromData(BYTE *pData, int nDataLen);

public:
	FileTrivInstance(int nID = 0);
	virtual ~FileTrivInstance();

private:				// member variables.
	ZString m_url;
	ZArray<FileTrivInstance*> m_auxFiles;
	int m_nID;


	FILE *m_fInput;
	BOOL m_bActive, m_bRewinding;
	ZMutex m_mtxChannels;
	int m_nChannelWaitedFor, m_nNotReady;
	ZSemaphore m_smpWaitForChannel, m_smpActive;

	static UINT channelIdCounter;

	struct ChannelAssociation
	{	bool m_bReady, m_bNeedRewind, m_bInProcess;
		BYTE *m_pData;
		int m_nDataLen;
		int m_nESid;
		UINT m_nChid;
		ChannelAssociation (int nESid = 0) :
			m_nESid (nESid), m_bReady (false), m_bNeedRewind (false),
			m_bInProcess (false), m_nChid (0), m_nDataLen(0), m_pData(NULL)
		{
			if(nESid)
			{	m_pData = new BYTE[2];
				memcpy(m_pData,&m_nESid,2);
				m_nDataLen = 2;
			}
		}
		~ChannelAssociation ()
		{
			if(m_pData)
				delete[] m_pData;
		}
		operator int () const {return m_nESid;}
	
		ChannelAssociation(const ChannelAssociation& o)//copy constructor
		{
			m_bReady		= o.m_bReady;
			m_bNeedRewind	= o.m_bNeedRewind;
			m_nESid			= o.m_nESid;
			m_nChid			= o.m_nChid;
			m_nDataLen		= o.m_nDataLen;
			if ( m_nDataLen > 0 )
			{
				m_pData		= new BYTE[o.m_nDataLen];
				memcpy(m_pData,o.m_pData,o.m_nDataLen);
			}
			else
				m_pData = NULL ;
		}

		ChannelAssociation& operator=(const ChannelAssociation& r)
		{
			m_bReady		= r.m_bReady;
			m_bNeedRewind	= r.m_bNeedRewind;
			m_nESid			= r.m_nESid;
			m_nChid			= r.m_nChid;
			m_nDataLen		= r.m_nDataLen;
			if ( m_pData )
				delete [] m_pData ;
			if ( m_nDataLen > 0 )
			{
				m_pData		= new BYTE[r.m_nDataLen];
				memcpy(m_pData,r.m_pData,m_nDataLen);
			}
			else
				m_pData = NULL ;
			return ( *this ) ;
		}
	};
	ZArray<ChannelAssociation> m_channelMap;
};

/*-------------------------------------------------------------
Exception object
-------------------------------------------------------------*/
struct FileException
{
	enum ErrorCodes {END_OF_FILE, MEMORY_ERROR} m_nCode;
	FileException (enum ErrorCodes nCode = END_OF_FILE) : m_nCode (nCode) {}
};

#endif                                      // _FileService_h sentry.
