/*
This software module was originally developed by Apple Computer, Inc.
in the course of development of MPEG-4. 
This software module is an implementation of a part of one or 
more MPEG-4 tools as specified by MPEG-4. 
ISO/IEC gives users of MPEG-4 free license to this
software module or modifications thereof for use in hardware 
or software products claiming conformance to MPEG-4.
Those intending to use this software module in hardware or software
products are advised that its use may infringe existing patents.
The original developer of this software module and his/her company,
the subsequent editors and their companies, and ISO/IEC have no
liability for use of this software module or modifications thereof
in an implementation.
Copyright is not released for non MPEG-4 conforming
products. Apple Computer, Inc. retains full right to use the code for its own
purpose, assign or donate the code to a third party and to
inhibit third parties from using the code for non 
MPEG-4 conforming products.
This copyright notice must be included in all copies or
derivative works. Copyright (c) 1999.
*/
/*
	$Id: MP4Service.cpp,v 1.1.1.1 2000/01/28 23:44:52 rls Exp $
*/
#include "defs.h"
#include "MP4Service.h"
#include "MP4TrackReaderClient.h"

LPCSTR MP4ServiceClient::s_moduleID = "$Id: MP4Service.cpp,v 1.1.1.1 2000/01/28 23:44:52 rls Exp $";
LPCSTR MP4ServiceClient::s_myURLScheme = "d-mp4:";

class MP4FileState 
{
	public:
		MP4Movie movie;
};

extern "C" ServiceClient *CreateInstance()
{
	return new MP4ServiceClient;
}

LPCSTR MP4ServiceClient::GetModuleID() 
{
	return s_moduleID;
}

MP4ServiceClient::MP4ServiceClient()
{
		m_movie = 0;
		m_trackCount = 0;
}


MP4ServiceClient::~MP4ServiceClient()
{
}

RESULT MP4ServiceClient::DPI_ServiceAttach_Req(LPCSTR szURL, const BYTE*, int)
{
	RESULT result;
	try
	{
		if ( (szURL == NULL) || (GetSink() == NULL) )
			throw;
		
		int schemeLen = strlen( s_myURLScheme );
		if ( _strnicmp(szURL, s_myURLScheme, schemeLen) == 0 )
			szURL += schemeLen;
		
		MP4Err err = MP4OpenMovieFile( &m_movie, szURL, MP4OpenMovieDebug );
		MP4Exception::test( err );
		err = MP4GetMovieTrackCount( m_movie, &m_trackCount );
		MP4Exception::test( err );
		err = MP4NewHandle( 0, &m_iod );
		MP4Exception::test( err );
		err = MP4GetMovieInitialObjectDescriptor( m_movie, m_iod );
		MP4Exception::test( err );
		unsigned int iodLength;
		err = MP4GetHandleSize( m_iod, &iodLength );
		MP4Exception::test( err );
		try
		{
			GetSink()->DPI_ServiceAttach_Cnf( (unsigned char *) *m_iod, iodLength, RESPONSE_OK );
			result = RESULT_OK;
		}
		catch ( ... )
		{
			if ( m_movie )
				MP4DisposeMovie( m_movie );
			m_movie = 0;
			result = RESULT_ERROR;
		}
	}
	catch ( MP4Exception& /*e*/ )
	{
		if ( m_movie )
			MP4DisposeMovie( m_movie );
		m_movie = 0;
		GetSink()->DPI_ServiceAttach_Cnf( NULL, 0, RESPONSE_REMOTE_ERROR );
		return RESULT_ERROR;
	}
	return result;
}


RESULT MP4ServiceClient::DPI_ServiceDetach_Req( REASON )
{
	GetSink()->DPI_ServiceDetach_Cnf( RESPONSE_OK );
	return RESULT_OK;
}

RESULT MP4ServiceClient::DPI_ChannelAdd_Req( const DAIChannelAddParam &inParam )
{
	for ( int i = 0; i < inParam.GetLength(); i++ )
	{
		RESPONSE_PARAMETER response = RESPONSE_REMOTE_ERROR;
		const DAIChannelAddElement &element = inParam[ i ];

		// apparently the data is either a 16-bit elementary stream
		// ID or a URL
		if ( element.m_nDataLen == 2 )
		{
			// ES id. These map directly to mp4 track ids

			MP4Track theTrack;
			unsigned short trackID = *((unsigned short*) element.m_pData);
			MP4Err err = MP4GetMovieIndTrack( m_movie, trackID, &theTrack );
			if ( err == MP4NoErr )
			{
				m_readers += new MP4TrackReaderClient( theTrack, trackID, element.ch_id, GetSink() );
				continue;
			}
		}

		DPI_TransmuxAdd_CnfParam smt;
		StreamMapElement &rMapElement = smt.ElementAt( 0 );
		rMapElement.m_nChId = element.ch_id;
		GetSink()->DPI_ChannelAdd_Cnf( NULL, rMapElement, response );
	}
	return RESULT_OK;
}

MP4TrackReaderClient* MP4ServiceClient::getClientForTrackID( int trackID )
{
	MP4TrackReaderClient* ret = 0;
	int readerCount = m_readers.GetLength();
	for ( int i = 0; i < readerCount; i++ )
	{
		MP4TrackReaderClient *aClient = m_readers[ i ];
		if ( aClient && (aClient->getID() == trackID) )
		{
			ret = aClient;
			break;
		}
	}
	return ret;
}

RESULT MP4ServiceClient::DPI_ChannelDelete_Req( const DAIChannelDeleteParam &inParam )
{
	int parameterCount = inParam.GetLength();

	for ( int i = 0; i < parameterCount; i++ )
	{
		// strange that no parameter is the handle that we
		// used when we acknowledged adding the channel...
		// so we find it using track id

		int trackID = inParam[i].m_nChId;
		MP4TrackReaderClient *client = getClientForTrackID( trackID );
		if ( client != 0 )
		{
			if ( client->getStatus() == MP4TrackReaderClient::status::finished )
			{
				GetSink()->DPI_TransmuxDelete_Cnf( (HANDLE) client, RESPONSE_OK );
			}
			else
				client->requestStop();
		}
	}
	return RESULT_OK;
}			

RESULT MP4ServiceClient::DPI_SendData( UINT ch_id, const char* data, int length ) 
{
	ch_id;
	data; 
	length; 
	return RESULT_ERROR;
}

RESULT MP4ServiceClient::DPI_UserCommand_Ack_Req( const DAIUserCmdParam& outParam ) 
{ 
	outParam; 
	return RESULT_ERROR;
}

BOOL MP4ServiceClient::IsYourURL( LPCSTR szURL )
{
	return !_strnicmp( szURL, s_myURLScheme, 4 ) || ZFileExists( szURL );
}

RESULT MP4ServiceClient::DPI_UserCommand_Req( const DAIUserCmdParam& inParam )
{
	RESULT ret = RESULT_OK;
	int parameterCount = inParam.GetLength();

	for ( int i = 0; (ret == RESULT_OK) && (i < parameterCount); i++ )
	{
		int trackID = inParam[i].m_nChId;
		MP4TrackReaderClient *client = getClientForTrackID( trackID );
		if ( client != 0 )
		{
			switch ( inParam[i].m_cData[0] )
			{
				case DMIF_UC_PLAY:
					if ( client->requestPlay() != 0 )
						ret = RESULT_ERROR;
					break;
				case DMIF_UC_STOP:
						client->requestStop();
					break;

				default:
					ret = DMIF_INTERNAL_ERROR;
			}

		}
		else
			ret = INVALID_CHANNEL_ID;
	}
	return ret;
}