/*
This software module was originally developed by Apple Computer, Inc.
in the course of development of MPEG-4. 
This software module is an implementation of a part of one or 
more MPEG-4 tools as specified by MPEG-4. 
ISO/IEC gives users of MPEG-4 free license to this
software module or modifications thereof for use in hardware 
or software products claiming conformance to MPEG-4.
Those intending to use this software module in hardware or software
products are advised that its use may infringe existing patents.
The original developer of this software module and his/her company,
the subsequent editors and their companies, and ISO/IEC have no
liability for use of this software module or modifications thereof
in an implementation.
Copyright is not released for non MPEG-4 conforming
products. Apple Computer, Inc. retains full right to use the code for its own
purpose, assign or donate the code to a third party and to
inhibit third parties from using the code for non 
MPEG-4 conforming products.
This copyright notice must be included in all copies or
derivative works. Copyright (c) 1999.
*/
/*
	$Id: MP4Service.h,v 1.1.1.1 2000/01/28 23:44:52 rls Exp $
*/

#ifndef INCLUDED_MP4SERVICE_H
#define INCLUDED_MP4SERVICE_H

#include "DMIF.h"
#include "DMIFClient.h"
#include "MP4Movies.h"

class MP4TrackReaderClient;

class MP4ServiceClient : public ServiceClient
{
	public:
		MP4ServiceClient();
		virtual ~MP4ServiceClient();
		virtual RESULT DPI_ServiceAttach_Req(LPCSTR szURL, const BYTE *pUuData, int nUuDataLen);
		virtual RESULT DPI_ServiceDetach_Req(REASON nReason);
		virtual RESULT DPI_ChannelAdd_Req(const DAIChannelAddParam &inParam);
		virtual RESULT DPI_ChannelDelete_Req(const DAIChannelDeleteParam &inParam);
		virtual RESULT DPI_UserCommand_Req(const DAIUserCmdParam& inParam);
		virtual	RESULT DPI_UserCommand_Ack_Req(const DAIUserCmdParam& outParam);
		virtual RESULT DPI_SendData(UINT ch_id,const char* data,int length);
		virtual BOOL IsYourURL( LPCSTR szURL );
		virtual LPCSTR GetModuleID();
	
	private:
		MP4TrackReaderClient *getClientForTrackID( int trackID );

		MP4Movie  m_movie;
		unsigned int m_trackCount;
		MP4Handle m_iod;
		ZArray<MP4TrackReaderClient*> m_readers;

		static LPCSTR s_moduleID;
		static LPCSTR s_myURLScheme;
};

class MP4Exception
{
	public:
		inline MP4Exception( MP4Err error );
		static inline void test( MP4Err error );
		static inline void testMalloc( void* ptr );
		inline MP4Err getErrorCode() const;
	private:
		int m_err;
};

inline MP4Exception::MP4Exception( MP4Err error )
{
		m_err = error;
}

inline MP4Err MP4Exception::getErrorCode() const
{
	return m_err;
}

inline void MP4Exception::test( MP4Err error )
{
	if ( error != MP4NoErr )
		throw MP4Exception( error );
}

inline void MP4Exception::testMalloc( void* ptr )
{
	if ( ptr == 0 )
		throw MP4Exception( MP4NoMemoryErr );
}

#endif
