/*
This software module was originally developed by Apple Computer, Inc.
in the course of development of MPEG-4. 
This software module is an implementation of a part of one or 
more MPEG-4 tools as specified by MPEG-4. 
ISO/IEC gives users of MPEG-4 free license to this
software module or modifications thereof for use in hardware 
or software products claiming conformance to MPEG-4.
Those intending to use this software module in hardware or software
products are advised that its use may infringe existing patents.
The original developer of this software module and his/her company,
the subsequent editors and their companies, and ISO/IEC have no
liability for use of this software module or modifications thereof
in an implementation.
Copyright is not released for non MPEG-4 conforming
products. Apple Computer, Inc. retains full right to use the code for its own
purpose, assign or donate the code to a third party and to
inhibit third parties from using the code for non 
MPEG-4 conforming products.
This copyright notice must be included in all copies or
derivative works. Copyright (c) 1999.
*/
/*
	$Id: MP4TrackReaderClient.cpp,v 1.1.1.1 2000/01/28 23:44:52 rls Exp $
*/
#include "MP4TrackReaderClient.h"
#include "ztl.h"

MP4TrackReaderClient::MP4TrackReaderClient( MP4Track theTrack, int ESID, int chID, ServiceClientSink *sink )
: m_track(theTrack), m_id(ESID), m_chID(chID), m_sink(sink), m_sampleH(0), m_stopRequested(false)
{
	m_status = starting;
	RESPONSE_PARAMETER response = RESPONSE_REMOTE_ERROR;
	MP4Err err = MP4CreateTrackReader( m_track, &m_reader );
	if ( err == MP4NoErr )
	{
		err = MP4NewHandle( 4096, &m_sampleH );
	}
	if ( err == MP4NoErr )
	{
		int result = ZThread<MP4TrackReaderClient>( readerLoop, this, THREAD_PRIORITY_BELOW_NORMAL ).Begin();
		if ( result == -1 )
		{
			err = MP4NoMemoryErr;
			response = RESPONSE_RESOURCES_EXHAUSTED;
		}
	}
	if ( err != MP4NoErr )
	{
		StreamMapElement mapElement;
		mapElement.m_nFmxInfo = 0;   // ???
		mapElement.m_nChId    = m_chID;
		m_sink->DPI_ChannelAdd_Cnf( NULL, mapElement, response );
		m_status = finished;
	}
}

MP4TrackReaderClient::status MP4TrackReaderClient::getStatus()
{
	return m_status;
}

void MP4TrackReaderClient::requestStop()
{
	m_stopRequested = true;
}

int MP4TrackReaderClient::requestPlay()
{
	int ret = 0;
	return ret;
}

MP4TrackReaderClient::~MP4TrackReaderClient()
{
	if ( m_sampleH )
		MP4DisposeHandle( m_sampleH );
	if ( m_reader )
		MP4DisposeTrackReader( m_reader );
}

void MP4TrackReaderClient::readerLoop()
{
	RESPONSE_PARAMETER response = RESPONSE_REMOTE_ERROR;
	DPI_TransmuxAdd_CnfParam smt;
	StreamMapElement &rMapElement = smt.ElementAt( 0 );
	rMapElement.m_nChId = m_chID;
	rMapElement.m_pData = new unsigned char[2];
	*((unsigned short*)rMapElement.m_pData) = m_id;
	rMapElement.m_nDataLen = 2;
	m_sink->DPI_TransmuxAdd_Cnf( (HANDLE) this, NULL_FLEXMUX, smt );
	MP4Err err;

	do
	{
		u32 packetSize;
		err = MP4NoErr;
		m_status = reading;
		if ( m_stopRequested == true )
			break;
		err = MP4TrackReaderGetNextPacket( m_reader, m_sampleH, &packetSize );
		if ( err == MP4NoErr )
		{
			m_status = writing;
			m_sink->DPI_DataReceived_Ind( (HANDLE) this, (const BYTE *) *m_sampleH, packetSize, FALSE );
			m_status = idle;
		}
	}
	while ( err == MP4NoErr );

	if ( err == MP4NoErr )
		response = RESPONSE_OK;
	else if ( err == MP4EOF )
		response = RESPONSE_OK; // was RESPONSE_END_OF_DATA;
	m_sink->DPI_TransmuxDelete_Cnf( (HANDLE) this, response );
	m_status = finished;
}
