/***********************************************************************
This software module was originally developed by Zvi Lifshitz
(Triton R&D Ltd.) in the course of development of the MPEG-4 Systems
(ISO/IEC 14496-1) standard. This software module is an implementation of
a part of one or more MPEG-4 Systems (ISO/IEC 14496-1) tools as
specified by the MPEG-4 Systems (ISO/IEC 14496-1) standard. ISO/IEC
gives users of the MPEG-4 Systems (ISO/IEC 14496-1) free license to
this software module or modifications thereof for use in hardware or
software products claiming conformance to the MPEG-4 Systems (ISO/IEC
14496-1). Those intending to use this software module in hardware or
software products are advised that its use may infringe existing patents.
The original developer of this software module and his/her company, the
subsequent editors and their companies, and ISO/IEC have no liability
for use of this software module or modifications thereof in an
implementation. Copyright is not released for non MPEG-4 Systems
(ISO/IEC 14496-1) conforming products. Triton R&D Ltd. retains full right
to use the code for his/her own purpose, assign or donate the code to a
third party and to inhibit third parties from using the code for non
MPEG-4 Systems (ISO/IEC 14496-1) conforming products. This copyright
notice must be included in all copies or derivative works.
Copyright  1998.
************************************************************************

For more information or to receive an up to date version of this module,
you may call Zvi Lifshitz at zvil@csi.com.
----------------------------------------------------------------------*/

/*-------------------------------------------------------------------------
Project Im1Player
INTERNATIONAL ORGANIZATION FOR STANDARDIZATION
ISO/IEC JTC1/SC29/WG11

SUBSYSTEM:  Decoder
FILE:       Decoder.cpp
AUTHOR:     Zvi Lifshitz (Triton R&D Ltd.)
OVERVIEW:   Implementation of Decoder - base class for all decoders.
-------------------------------------------------------------------------*/

/*-------------------------------------------------------------------------
MPEGJ 
AUTHORS:    Viswanathan Swaminathan - vishy@eng.sun.com 
			Robert Schmidt(RLS)
			Praveen Alavilli(PA) 
			YUNTAI KYONG(YT) - yuntaikyong@ieee.org
-------------------------------------------------------------------------*/

#include "defs.h"
#include "DecoderImp.h"
#include "MediaStream.h"
#include "..\core\ODImp.h"			// MPEGJ MERGE - RLS
#include "..\core\inline.h"			// MPEGJ MERGE - RLS
#include "..\mpegj\MPDecoderInterface.h"	// MPEGJ MERGE - RLS

#ifdef MPGJ
static int isStopped;		// MPEGJ MERGE - RLS
#endif
	
/*-----------------------------------------------------------------
Decoder constructor
-----------------------------------------------------------------*/
DecoderImp :: DecoderImp () :
	m_decoderThread (DecoderThread, this)
{
	m_pOutStream = m_pInStream = NULL;
	m_bActive = false;
	
	#ifdef MPGJ
	// MPEGJ MERGE - RLS
	// load the function pointer table to throw events - PA
	di = MPJGetTerminalDispatchTable(); 
	isStopped = 0;
	// MPEGJ MERGE - RLS
	#endif
}

/*-----------------------------------------------------------------
Decoder destructor
-----------------------------------------------------------------*/
DecoderImp :: ~DecoderImp ()
{	
	/*-----------------------------------------------------------------
	Make sure the decoder thread terminates before the object is deleted.
	-----------------------------------------------------------------*/
	ASSERT (m_decoderThread.GetThread () == NULL);
}

/*-----------------------------------------------------------------
The default implementation of Stop() aborts any "wait for time" access
on the input stream and signals the decoder thread to stop. 
-----------------------------------------------------------------*/
void DecoderImp :: Stop ()
{
//	#ifdef MPGJ
	isStopped = 1; // PA
//	#endif

	m_bActive = false;
	if (m_pInStream)
	{	m_pInStream -> Abort ();
		m_pInStream -> Terminate ();
	}
	if (m_pOutStream)
		m_pOutStream -> Close ();	
	WaitForThread ();

//	#ifdef MPGJ
	/*** terminate the event thread - PA */
	if( di != NULL )
		TerminateThread(di->hThrow, NULL);	
	/*** - end - */	
//	#endif //MPGJ

}


/*-------------------------------------------------------------
Close streams after decoder thread is terminated
-------------------------------------------------------------*/
void DecoderImp :: Terminate ()
{	
		if (m_pOutStream)
			m_pOutStream -> Close ();
		if (m_pInStream)
			m_pInStream -> Terminate ();

}


#ifdef MPGJ
//*** YT START (117)
// pause and resume decoder
// 1. pause or resume input and output buffer
// 2. if going resume state - start decoder thread
void DecoderImp :: Pause ()
{
	m_bActive = FALSE;

	if(m_pInStream)
		m_pInStream->HalfClose(TRUE);
	if(m_pOutStream)
		m_pOutStream->Pause(TRUE);
}

void DecoderImp :: Resume ()
{
	m_bActive = TRUE;

	if(m_pInStream)
		m_pInStream->HalfClose(FALSE);
	if(m_pOutStream)
		m_pOutStream->Pause(FALSE);
	
	if (StartDecoderThread () == -1)
		throw;
}


//*** YT END (117)
#endif MPGJ
/*-----------------------------------------------------------------
Start() resets the input stream and open both input and output stream
thread.
-----------------------------------------------------------------*/
void DecoderImp :: Start ()
{
	#ifdef MPGJ
	isStopped = 0; //PA
	#endif

	ASSERT (m_decoderThread.GetThread () == NULL);
	m_bActive = true;
	if (m_pInStream)
		m_pInStream -> Start ();
	if (m_pOutStream)
		m_pOutStream -> Open ();
	if (StartDecoderThread () == -1)
		throw;
}

/*-----------------------------------------------------------------
Run() retrieves units from the input stream, decode them, and dispatch
units to the output stream, till the input stream's end is exceeded.
-----------------------------------------------------------------*/
void DecoderImp :: Run ()
{
	LPBYTE pInData, pOutData;
	int nInSize, nOutSize;
	DWORD dwTime;
	BOOL bEOU;

	#ifdef MPGJ
	// MPEGJ MERGE_3 - RLS
	// to get the decoder OD ID to throw the event on to the corresponding decoder on java side - PA
    InlineImp* pInline;
	Executive *myExecutive;
	Decoder* dec;
	myExecutive = GetExecutive();
	pInline = myExecutive->GetInline();
	int len = pInline->m_streamAssociation.GetLength();
	//*** YT START (118)

	// associate the decoder with ESID also, not only OD
	for( int i=0; i < len; i++ ) {		
		for(int j = pInline->m_streamAssociation[i].m_pOD->esDescr.GetLength()-1;j>=0;j--) {
			dec =  ((ES_DescriptorImp *)(pInline->m_streamAssociation[i].m_pOD->esDescr[j]))->GetDecoder();
			if( this == dec ) {
				ESID = pInline->m_streamAssociation[i].m_pOD->esDescr[j]->ES_ID;
				ODID = pInline->m_streamAssociation[i].m_pOD->ObjectDescriptorID; 
			}
		}
	}
	//*** YT END (118)

     /**** get the instance of the function pointer table. - PA ****/ 
    di = MPJGetTerminalDispatchTable(); 
    // set the status variable 
   	if( di != NULL ) {
    	//STREAM_START
	    // go till end of list and add the new event there
	    decoderQueue* temp;
		temp = &(di->dEQ);
		if( *temp == NULL )
			//*** YT START (119)
			// (*temp) = new decoderEventQueue( 4, ODID);
			(*temp) = new decoderEventQueue( 4, ODID, ESID);
			//*** YT END (119)
		else {
			while ( (*temp)->next != NULL )
				*temp = (*temp)->next;
			//*** YT START (119)
			// (*temp) = new decoderEventQueue( 4, ODID);
			(*temp)->next = new decoderEventQueue( 4, ODID, ESID);						
			//*** YT END (119)
		}
		
	    if( di->isStartedDispatching == 1) {
		   // signal the thread to throw the event 
		   	count = ResumeThread(di->hThrow);		   
		}
	}
   	/****** end of thread resume code ****/	
	//END MPEGJ MERGE_3 - RLS
	#endif

	while (
		m_bActive &&
		(pInData =
			(LPBYTE) m_pInStream -> FetchNext (nInSize, dwTime, bEOU, TRUE)
		) !=
		END_OF_STREAM
	)
	{
		//MPEGJ MERGE_4 - RLS
		// UNDERFLOW does not exist in the new core, so I will comment this out.
		/*** check if the stream underflow flag is set - PA *****//*				
		if( m_pInStream->UNDERFLOW == true ) {
			/* if it is then throw a decoder overflow event *//*
			m_pInStream->UNDERFLOW = false;
			if( di != NULL ) {
				// stream underflow 						
				di = MPJGetTerminalDispatchTable(); 
				// go till end of list and add the new event there
				decoderQueue* temp;				
				temp = &(di->dEQ);
				if( *temp == NULL )
					(*temp) = new decoderEventQueue( 0, ODID);
				else {
					while ( (*temp)->next != NULL )
						*temp = (*temp)->next;
					(*temp)->next = new decoderEventQueue( 4, ODID);						
				}
				
				if( di->isStartedDispatching == 1) {
					// signal the thread to throw the event 					
					count = 0;
					count = ResumeThread(di->hThrow);					
				}
			}
		}				
		/*** end of thread resume code  */	
		//END MPEGJ MERGE_4 - RLS	

		if (m_bActive)
		{	nOutSize = GetOptimalOutputSize (),
			pOutData = (LPBYTE) m_pOutStream -> Allocate (nOutSize, TRUE, FALSE);

// MPEGJ MERGE_5 --- There is no decoder overflow test in this version of the core;
// this test is not in the new code, so i will comment this  piece of the old code out. - RLS
/*	
			if (m_pOutStream->DECODER_OVERFLOW ) {
				m_pOutStream->DECODER_OVERFLOW  = false;															/***** decoder overflow - PA*/ /*
				if( di != NULL ) {
					// set the status varible 
					di = MPJGetTerminalDispatchTable(); 
					// go till end of list and add the new event there
					decoderQueue* temp;					
					temp = &(di->dEQ);
					if( *temp == NULL )
						(*temp) = new decoderEventQueue( 0, ODID);
					else {
						while ( (*temp)->next != NULL )
							*temp = (*temp)->next;
						(*temp)->next = new decoderEventQueue( 0, ODID);						
					}
					
					if( di->isStartedDispatching == 1) {
						// signal the thread to throw the event 
						if ( di->isJVMStarted == 1 ) {
							count = 0;
							count = ResumeThread(di->hThrow);
						}
					}
				}
				/****** end of thread resume code ****/ /*
				throw;
			}
*/
// END MPEGJ MERGE_5 - RLS				

			if (pOutData == NULL)
				throw;
			if (pOutData == END_OF_STREAM)
				return;
			while (!Decode (pInData, nInSize, pOutData, nOutSize))
			{	pOutData = (LPBYTE) m_pOutStream -> Reallocate (nOutSize);
				if (pOutData == NULL)
				{
				#ifdef MPGJ
				//MPEGJ MERGE_6 - RLS
				/***** stream overflow - PA*/
				/*
					if( di != NULL ) {
						//stream overflow - no space for the output stream to expand 
						//                              to allocate for the decoded data
						di = MPJGetTerminalDispatchTable(); 
						// go till end of list and add the new event there
						decoderQueue* temp;
						temp = &(di->dEQ);
						if( *temp == NULL )
							//*** YT START (119)
							//(*temp) = new decoderEventQueue( 3, ODID);
							(*temp) = new decoderEventQueue( 3, ODID, ESID);
							//*** YT END (119)
						else {
							while ( (*temp)->next != NULL )
								*temp = (*temp)->next;
							//*** YT START (119)
							//(*temp) = new decoderEventQueue( 3, ODID);
							(*temp)->next = new decoderEventQueue( 3, ODID, ESID);						
							//*** YT END (119)

						}
						if( di->isStartedDispatching == 1) {
							// signal the thread to throw the event 
							if ( di->isJVMStarted == 1 ) {
								count = 0;
								count = ResumeThread(di->hThrow);
							}	
						}	
					}
					*/
					/****** end of thread resume code ****/
					//END MPEGJ MERGE_6 - RLS
					#endif

					throw;
				}
				if (pOutData == END_OF_STREAM)
					return;
			}
			m_pOutStream -> Dispatch (nOutSize, dwTime);
		}
		m_pInStream -> Release (TRUE);
	}
	#ifdef MPGJ
// MPEGJ MERGE_7 - RLS
	/***** THROW STREAM END EVENT - PA */
	// set the status varible 
	di = MPJGetTerminalDispatchTable(); 
	// go till end of list and add the new event there
	decoderQueue* temp;
	temp = &(di->dEQ);
	if( *temp == NULL )
		//*** YT START (119)
		//(*temp) = new decoderEventQueue( 5, ODID);
		(*temp) = new decoderEventQueue( 5, ODID, ESID);
		//*** YT END (119)
	else {
		while ( (*temp)->next != NULL )
			*temp = (*temp)->next;
		//*** YT START (119)
		//(*temp)->next = new decoderEventQueue( 5, ODID);						
		(*temp)->next = new decoderEventQueue( 5, ODID, ESID);						
		//*** YT END (119)
	} 					
	if( di->isStartedDispatching == 1) {
		// signal the thread to throw the event 
		if ( di->isJVMStarted == 1 ) {
			count = 0;
			count = ResumeThread(di->hThrow);
		}
	}

	/***** end of thread resume code  */
	//END  MPEGJ MERGE_7 - RLS
	#endif //MPGJ
}

/*-----------------------------------------------------------------
The Decoder thread function calls the virtual Run() function and 
release the flag semaphore when finishes.
-----------------------------------------------------------------*/
void DecoderImp :: DecoderThread ()
{
	if (Init ())
		Run ();
	Terminate ();	
}

/*-----------------------------------------------------------------
Default implementation of SetFormat, picks up the first supplied
format and returns some default values.
-----------------------------------------------------------------*/
BOOL DecoderImp :: SetFormat (
	Capabilities &formatOptions,
	DecoderParams &decoderParams
)
{
	if (formatOptions.GetFormatCapabilityLength () == 0)
		return FALSE;
	decoderParams.m_nStreamNumber = 0;
	decoderParams.m_format = formatOptions.GetFormatCapability (0);
	decoderParams.m_nOptimalAUSize = 256;
	decoderParams.m_nDBSize = 1024;
	decoderParams.m_nCBSize = 1024;
	decoderParams.m_bRequiresFullAUs = TRUE;
	return TRUE;
}
