//
// This code has been taken from that written by Mike Coleman and
// presented at Stockholm.  The changes have been to remove
// references to the OS class, add the code which writes the
// audio data format into the media stream buffer, and remove the
// Run() method, using the base class's method.
//
// Matthew Leditschke
// CSELT
//

#include "defs.h"
#include "AACDecoder.h"
#include <assert.h>
#include <limits.h>

//#include "Caps.h"
#include "Stream.h"

//#include "Executiv.h"
//#include "Presenter.h"

/*-------------------------------------------------------------
The exported function that instantiates the proper decoder
-------------------------------------------------------------*/
void *CreateInstance ()
{
	return new AACDecoder;
}

FILE *fp;

AACDecoder::AACDecoder( )
  : DecoderImp(), m_targetBitrate(64000), m_targetSampleRate(44100)
{
	m_firstDecode = TRUE;
	m_PUlen = 1024 * sizeof(short);
	m_timeStampIncrement = 1.0e6 * (1024.0 / m_targetSampleRate);
	m_timeStamp = 0.0;
	m_lastTimeStamp=0;
	const char* path;
	path = "aacTool.dll";

	m_pluginHandle = ::LoadLibrary( path );
	assert(m_pluginHandle != 0);

	m_decodeFrame     = (long(*)(void*, void*, int, void*, long))  ::GetProcAddress( (HMODULE)m_pluginHandle, "decodeFrame" );
	m_initInstance    = (int (*)(void**)) ::GetProcAddress( (HMODULE)m_pluginHandle, "initInstance" );
	m_releaseInstance = (int (*)(void*))  ::GetProcAddress( (HMODULE)m_pluginHandle, "releaseInstance" );
   	int res = (*m_initInstance)( &m_pluginInstanceData );

}

AACDecoder::~AACDecoder()
{
	if ( m_releaseInstance != 0 )
		(*m_releaseInstance)( m_pluginInstanceData );
	//m_smpActive.P ();
	if (!IsActive ())
		if(m_pluginHandle!=0)
			FreeLibrary((HINSTANCE )m_pluginHandle);
	//m_smpActive.V ();
}

int AACDecoder::GetOptimalInputSize()
{
	return (m_targetBitrate * 1024) / m_targetSampleRate;
}

int AACDecoder::GetOptimalOutputSize()
{
	return m_PUlen;
}

/*int AACDecoder::GetRecommendedInputBufferSize()
{
	return 2 * GetOptimalInputSize();
}

int AACDecoder::GetRecommendedOutputBufferSize()
{
	return 480 * GetOptimalOutputSize();
}*/

BOOL AACDecoder :: SetFormat (	Capabilities &formatOptions,DecoderParams &decoderParams)
{
		int i;
	BOOL bResult = FALSE;
	for (i = 0; !bResult; i++)
	{	if (i >= formatOptions.GetFormatCapabilityLength ())
			return FALSE;
	    
	    if(formatOptions.GetFormatCapability (i).m_code==MediaFormat :: PCM ) 
			break;
		continue;
	}
		
    decoderParams.m_nStreamNumber = 0;
	decoderParams.m_format = formatOptions.GetFormatCapability (i--);
	decoderParams.m_nOptimalAUSize = 256;
	decoderParams.m_nDBSize = 20 * GetOptimalInputSize();//was 2*
	decoderParams.m_nCBSize = 20*GetOptimalOutputSize();//was 4*
	decoderParams.m_bRequiresFullAUs = TRUE;
	return TRUE;
}		

//#define SILENCE_PLEASE

BOOL AACDecoder::Decode( LPBYTE AUptr, int AUlen, LPBYTE PUptr, int &PUlen )
{
	if ( PUlen < m_PUlen )
	{
		PUlen = m_PUlen;
		return FALSE;
	}

//  int headerSize = 0;
/*	if (m_firstDecode)
	{
		m_firstDecode = FALSE;

		DecoderAudioFormat* pFmt = (DecoderAudioFormat*)PUptr;
		pFmt->samplesPerSecond = 44100;
		pFmt->bitsPerSample	   = 16;
		pFmt->numChannels	   = 1;
		headerSize = sizeof(DecoderAudioFormat);
	}*/

#ifdef SILENCE_PLEASE
	::memset( PUptr, m_PUlen, 0 );
	PUlen = m_PUlen;
#else

	PUlen = (*m_decodeFrame)( m_pluginInstanceData, AUptr, AUlen, PUptr, PUlen);
	if ( PUlen == 0 )
	{
		::memset( PUptr, m_PUlen, 0 );
		PUlen = m_PUlen;
	}
	else
		PUlen;
#endif
	return PUlen;
}



/*-------------------------------------------------------------
In this decoder, attributes are fixed for all units
-------------------------------------------------------------*/
DWORD AACDecoder :: GetAttributes (
	ATTRIBUTE_CODE code, 
	const LPBYTE
)
{
	switch (code) {
	case AUDIO_FREQUENCY:
		return  44100;
	case BITS_PER_SAMPLE :
		return 16;
	case NUM_CHANNELS:
		return 1;
	default:
		return 0;
	}
}

void AACDecoder :: Run ()
{
	
	LPBYTE pInData, pOutData;
	int nInSize, nOutSize;
	DWORD dwTime;
	BOOL bEOU;

	while (
		(pInData =
			(LPBYTE) GetInputStream ()-> FetchNext (nInSize, dwTime, bEOU, TRUE)
		) !=
		END_OF_STREAM
	)
	{
		if (IsActive ())
		{
/*			if(m_lastTimeStamp!=0)
			{
				DWORD tmpTime = m_lastTimeStamp;
				DWORD i = 0;
				while((dwTime-tmpTime)>((1024000/m_targetSampleRate)+10))    
				{
					// If a packet is lost dispatch silence in the composition buffer.
					i++;	
					tmpTime = m_lastTimeStamp + (i*1024000/m_targetSampleRate);					
					nOutSize = GetOptimalOutputSize ();
					pOutData = (LPBYTE) GetOutputStream () -> Allocate (nOutSize, TRUE, FALSE);
					// controls for allocate.
					if (pOutData == NULL)
							throw;
					if (pOutData == END_OF_STREAM)
							return;
					//
					::memset(pOutData,nOutSize, 0 );
					GetOutputStream ()  -> Dispatch (nOutSize,m_lastTimeStamp);
				}
			}
*/
			nOutSize = GetOptimalOutputSize ();
			pOutData = (LPBYTE) GetOutputStream () -> Allocate (nOutSize, TRUE, FALSE);
			if (pOutData == NULL)
				throw;
			if (pOutData == END_OF_STREAM)
				return;
			while (!Decode (pInData, nInSize, pOutData, nOutSize))
			{	pOutData = (LPBYTE) GetOutputStream ()  -> Reallocate (nOutSize);
				if (pOutData == NULL)
					throw;
				if (pOutData == END_OF_STREAM)
					return;
			}
			GetOutputStream () -> Dispatch (nOutSize, dwTime);
			m_lastTimeStamp=dwTime;

		}
		GetInputStream () -> Release (TRUE);
	}

}
