/***********************************************************************
    This software module  was  originally  developed by Zvi  Lifshitz
    (Triton R&D Ltd.) and edited by Mauro Rossi (CSELT), George
    Tselikis (Xbind Inc.) and Michael Stepping (FTK)
    in the course of development of the MPEG-4 DMIF (ISO/IEC 14496-6)
    standard.  This software module is an implementation of a part of
    one or more MPEG-4  DMIF  (ISO/IEC 14496-6)  tools  as  specified
    by  the  MPEG-4 DMIF (ISO/IEC 14496-6)  standard.  ISO/IEC  gives
    users of  the  MPEG-4  DMIF  (ISO/IEC 14496-6)  free  license  to
    this software module or modifications thereof for use in hardware
    or software products claiming  conformance  to  the  MPEG-4  DMIF 
    (ISO/IEC 14496-6). Those intending to use this software module in
    hardware or software  products  are  advised  that  its  use  may
    infringe   existing  patents.  The  original  developer  of  this
    software module and his/her company, the subsequent  editors  and
    their  companies,  and  ISO/IEC have no liability for use of this
    software module or modifications thereof  in  an  implementation.
    Copyright  is not released for non MPEG-4 DMIF  (ISO/IEC 14496-6)
    conforming  products. Triton R&D Ltd.
    retains full  right to use the code for  its  own purpose, assign
    or donate the code to a third party and to inhibit third  parties
    from  using  the  code  for  non MPEG-4 DMIF (ISO/IEC 14496-6)
    conforming products. This copyright notice must  be  included  in
    all copies or derivative works. Copyright  1998.
************************************************************************/

/**************************************************************************
 * Project Im1Player
 * INTERNATIONAL STANDARD ORGANIZATION
 * ISO/IEC JTC1/SC29/WG11
 * 
 * SUBSYSTEM: DMIF
 * FILE:      DMIFHeader.h
 * AUTHOR:    Mauro Rossi (CSELT)
 * OVERVIEW:  Definitions for the DMIF-Application Interface.
 * 
 **************************************************************************/

#ifndef _DMIF_h_
#define _DMIF_h_

#include "defs.h"
// ZArrays for definition of Loops
#include "ztl.h"

#ifndef _WINSOCK2API_
#define ntohs(n) (((n)%256)*256 + (n)/256)
#define htons(n) (((n)%256)*256 + (n)/256)
#endif

// Response parameter
typedef enum tagResponseParameter
{
	RESPONSE_OK								= 0x0000,
	RESPONSE_SERVICEDLL_NOT_FOUND			= 0x0001,
	RESPONSE_SERVICEDLL_ERROR				= 0x0002,
	RESPONSE_URL_NOT_RECOGNIZED				= 0x0003,
	RESPONSE_CHANNEL_NOT_FOUND				= 0x0004,
	RESPONSE_RESOURCES_EXHAUSTED			= 0x0005,
	RESPONSE_REMOTE_ERROR					= 0x0006,
	RESPONSE_URL_NOT_FOUND					= 0x0007,
	RESPONSE_END_OF_DATA					= 0x0008,
	RESPONSE_UNKNOWN_ERROR					= 0xFFFF
} RESPONSE;

// Reason parameter
typedef enum tagReasonParameter
{
	REASON_OK								= 0x0000,
	REASON_REMOTE_ERROR						= 0x0001,
	REASON_REMOTE_INITIATED					= 0x0002,
	REASON_UNKNOWN_ERROR					= 0xFFFF
} REASON;

// Direction parameter for transport content
typedef enum tagDIRECTION
{
	UNSPECIFIED				= 0x00,
	DOWNSTREAM				= 0x01,
	UPSTREAM				= 0x02,
	BIDIRECTIONAL			= 0x03
} DIRECTION;

// UserComands are MPEG-4 specific
// will be defined exactly in 14496-1 V2
typedef enum tagUSERCOMMAND
{
	DMIF_UC_UNSPECIFIED		= 0x00,
	DMIF_UC_PLAY			= 0x01,
	DMIF_UC_STOP			= 0x02
} DMIF_USERCOMMAND;


typedef enum tagDATA
{
	DATA_OK					= 0x00,
	DATA_CORRUPTED			= 0xFF
} DATA_STATUS;


// enumeration of FlexMuxTypes is used
// internally of DMIF, on behalf of server and client side.
typedef enum tagFlexmuxType
{ 
	UNDEFINED_FLEXMUX	= 0x00,
	NULL_FLEXMUX		= 0x01, 
	MPEG4_FLEXMUX		= 0x02,
} FLEXMUX_TYPE;


typedef unsigned char	BYTE;
typedef unsigned short	u_short;
typedef unsigned int	UINT;


/*-------------------------------------------------------------
Structs to pass information in the interface
-------------------------------------------------------------*/
// The QoS parameters are streamed.
// MPEG-4 terminal has to negotiate the parameters with DMIF.
// Inside DMIF the Stream is decoded and used. Look at "QoSLoop"
// The exchange format is a SYSTEMS descriptor called qosDescriptor
// Inside a function call it has to be decoded before the
// function calls returns.
typedef LPCBYTE QOSSTREAM ;


/*-------------------------------------------------------------
Struct for passing IN parameters in :
ServiceClient::UserCommand
-------------------------------------------------------------*/
class DAIUserCmdElement
{
  public:
	UINT	m_nChid;
	BYTE*	m_pData;
	int		m_nLen;

	BOOL operator == (DAIUserCmdElement const &el) const 
		{return ( el.m_nChid == m_nChid) ;}
	
	DAIUserCmdElement() : m_nChid(0),m_pData(NULL),m_nLen(0) 
	{}
	DAIUserCmdElement(const DAIUserCmdElement& o)//copy constructor
	{
		m_nChid = o.m_nChid;
		m_nLen = o.m_nLen;
		m_pData = new BYTE[o.m_nLen];
		memcpy(m_pData,o.m_pData,o.m_nLen);
	}

	DAIUserCmdElement& operator=(const DAIUserCmdElement& r)
	{
		m_nChid = r.m_nChid;
		m_nLen = r.m_nLen;
		if ( m_pData )
			delete [] m_pData ;
		m_pData = new BYTE[r.m_nLen];
		memcpy(m_pData,r.m_pData,r.m_nLen);
		return ( *this ) ;
	}
	
	virtual ~DAIUserCmdElement()
	{
		delete[] m_pData;
	}
};


/*-------------------------------------------------------------
Struct for passing IN parameters in :
DAI_Channel_Add_*
-------------------------------------------------------------*/
class DAIChannelAddElement
{
  public:

	UINT		m_nChid;
	// direction for channel content. Usually from point of view of 
	// client/MPEG-4 Terminal
	DIRECTION	m_eDirection;
	
	RESPONSE	m_eResponse;
	QOSSTREAM	m_pQoSStream;
	
	// user-user data.
	BYTE*		m_pData;
	USHORT		m_nDataLen;
	
  public: // methods	
	BOOL operator == (DAIChannelAddElement const &el) const 
	{
		return (m_nChid == el.m_nChid);
	}
	
	DAIChannelAddElement() : m_nChid(0), m_pData(NULL), m_nDataLen(0), 
		m_eDirection( DOWNSTREAM ), m_pQoSStream( NULL ),
		m_eResponse ( RESPONSE_OK )
	{}
	
	DAIChannelAddElement(const DAIChannelAddElement& o)//copy constructor
	{
		m_nChid	= o.m_nChid;
		m_eDirection= o.m_eDirection;
		m_eResponse	= o.m_eResponse;
		m_pQoSStream= o.m_pQoSStream;
		m_nDataLen	= o.m_nDataLen;
		if ( m_nDataLen > 0 )
		{
			m_pData		= new BYTE[o.m_nDataLen];
			memcpy(m_pData,o.m_pData,o.m_nDataLen);
		}
		else
			m_pData = NULL ;
	}

	DAIChannelAddElement& operator=(const DAIChannelAddElement& r)
	{
		m_nChid	= r.m_nChid;
		m_eDirection	= r.m_eDirection;
		m_eResponse	= r.m_eResponse;
		m_pQoSStream= r.m_pQoSStream;
		m_nDataLen	= r.m_nDataLen;
		if ( m_pData )
			delete [] m_pData ;
		if ( m_nDataLen > 0 )
		{
			m_pData		= new BYTE[r.m_nDataLen];
			memcpy(m_pData,r.m_pData,m_nDataLen);
		}
		else
			m_pData = NULL ;
		return ( *this ) ;
	}
	
	virtual ~DAIChannelAddElement()
	{
		delete[] m_pData ;
	}
};


/*-------------------------------------------------------------
Struct for passing IN parameters in :
DAI_Channel_Delete_*
-------------------------------------------------------------*/
class DAIChannelDeleteElement
{
  public:
	UINT			m_nChid;
	REASON			m_eReason;
	RESPONSE		m_eResponse;
	BOOL operator == (DAIChannelDeleteElement const &el) const {return el.m_nChid == m_nChid;}

	DAIChannelDeleteElement& operator=(const DAIChannelDeleteElement& r)
	{
		m_nChid = r.m_nChid;
		m_eReason = r.m_eReason;
		m_eResponse = r.m_eResponse;
		return ( *this ) ;
	}
};

// construction of Loops for passing to the interface
typedef ZArray<DAIChannelAddElement>	DAIChannelAddParam;
typedef ZArray<DAIChannelDeleteElement>	DAIChannelDeleteParam;
typedef ZArray<DAIUserCmdElement>			DAIUserCmdParam;


// The following C++ class provides an abstract view of the functions that the DMIF implementation calls in order 
// to inform the application about the result of a requested service (functions with "_Cnf" extension) or about services 
// requested by the peer entity (functions with "_Ind" extension). This class defines the methods that a DMIF 
// implementation (DMIF_Session derived class) expects to be available from the application. The application should 
// define an Application_Session derived class and implement the DAI functions with "_Ind" or "_Cnf" extension. 
// Therefore, in this case the function call direction is DMIF->Application.
class Application_Session
{
public:

	// necessary for deleting concrete instance, otherwise resource leaks.
	virtual ~Application_Session() {} ;

	// Service Attach functions
	virtual void DAI_Service_Attach_Ind(
		const char*		service_name,
		const BYTE*		uuData,
		const u_short	uuData_Length
	) = 0;
	virtual void DAI_Service_Attach_Cnf(
		RESPONSE		response,
		const BYTE*		uuData,
		const u_short	uuData_Length
	) = 0;

	// Service Detach functions
	virtual void DAI_Service_Detach_Ind(
		REASON			eReason
	) = 0;
	virtual void DAI_Service_Detach_Cnf(
		RESPONSE		eResponse
		) = 0;

	// Channel Add functions
	virtual void DAI_Channel_Add_Ind(
		const DAIChannelAddParam& indParam
	) = 0;
	virtual void DAI_Channel_Add_Cnf(
		const DAIChannelAddParam& cnfParam
	) = 0;

	// Channel Delete functions
	virtual void DAI_Channel_Delete_Ind(
		const DAIChannelDeleteParam& indParam
	) = 0;
	virtual void DAI_Channel_Delete_Cnf(
		const DAIChannelDeleteParam& indParam
	) = 0;

	// User Command function
	virtual void DAI_User_Command_Ind(
		const DAIUserCmdParam& inParam
	) = 0; 
	virtual	 int DAI_User_Command_Ack_Ind(
		const DAIUserCmdParam& outParam
	) =0;
	virtual	 int DAI_User_Command_Ack_Cnf(
		const DAIUserCmdParam& outParam
	) =0;

	// Function for data reception
	virtual void DAI_Received_Data(
		UINT			nChid,
		const char*		pData,
		int				nLength,
		int				nError_flag
	) = 0;
};

// The following C++ class provides an abstract view of the functions that the application should call in order to
// request a DMIF service (functions with "_Req" extension) or to respond to a DMIF service requested by the peer entity 
// (functions with "_Rsp" extension). This class defines the methods that an application expects to be available from 
// an DMIF implementation (DMIF_Session derived class). An DMIF_Session derived class is the one that implements
// the DAI functions with "_Req" or "_Rsp" extension. Therefore, in this case the function call direction is 
// Application->DMIF.
class DMIF_Session
{
public:

	// necessary for deleting concrete instance, otherwise resource leaks.
	virtual ~DMIF_Session() {} ;

	// Service Attach functions
	virtual void DAI_Service_Attach_Req(
		const char*		dest_addr,
		const BYTE*		uuData,
		u_short			uuData_length
	) = 0;
	virtual void DAI_Service_Attach_Rsp(
		const BYTE*		uuData,
		const u_short	uuData_Length,
		RESPONSE		eResponse
	) = 0;

	// Service Detach functions
	virtual void DAI_Service_Detach_Req(
		REASON			eReason
	) = 0;
	virtual void DAI_Service_Detach_Rsp(
		RESPONSE		eResponse
	) = 0;
	
	// Channel Add functions
	virtual void DAI_Channel_Add_Req(
		const DAIChannelAddParam& reqParam
	) = 0;
	virtual void DAI_Channel_Add_Rsp(
		const DAIChannelAddParam& rspParam
	) = 0;
	
	// Channel Delete functions
	virtual void DAI_Channel_Delete_Req(
		const DAIChannelDeleteParam&
	) = 0;
	virtual void DAI_Channel_Delete_Rsp(
		const DAIChannelDeleteParam&
	) = 0;

	// User Command function
	virtual void DAI_User_Command_Req(
		const DAIUserCmdParam& inParam
	) = 0;
	virtual	void DAI_User_Command_Ack_Req(
		const DAIUserCmdParam& outParam
	) =0;
	virtual	void DAI_User_Command_Ack_Rsp(
		const DAIUserCmdParam& outParam
	) =0;

	// Function for data transmission
	virtual void DAI_Send_Data(
		UINT			nChid,
		const char*		pData,
		int				nLength,
		long			nTMin,
		long			nTMax) = 0;

	// Function for dll server registration
	virtual void RegistService(LPCSTR sURLstring) = 0;


//	class Application_Session;
	void SetSink ( Application_Session* pSink) {m_pSink = pSink;};
	Application_Session* GetSink () { return m_pSink; } ;
private:
	Application_Session* m_pSink;

};


/*-------------------------------------------------------------
Declaration of the function that instantiates DMIFClient
-------------------------------------------------------------*/
extern "C" DMIF_Session* Create_DMIF_Session(Application_Session* ptr);

#endif  //_DMIF_h_