/*
* Copyright(c) Philips Consumer Electronics B.V. 1998                         
* All rights reserved. Reproduction in whole or in part is                  
* prohibited without the written consent of the copyright holder.            
*                                                                            
*  Project             : Bali
*  File name   $Logfile: $
*       Last save $Date: 2000/06/06 18:43:02 $
*             $Revision: 1.1 $
*          Last $Author: alavilli $
*  Creation Date       : 09/02/99
*  First Author        : S.A.F.A. van den Heuvel
*                        NatLab - Eindhoven
* 
*                                                                          
*  Change history      :
*  ---------------
*
* $Log: SICacheElement.java,v $
* Revision 1.1  2000/06/06 18:43:02  alavilli
* After SISF integration into the CORE4 code
*
* Revision 1.1.2.1  1999/07/09 14:16:37  depetris
* Aggiornati i file per lo sviluppo e il rilascio a MPEG-J.
* Integra la versione 2.5 del core.
*
* Revision 1.1  1999-02-09 14:16:06+01  vdheuvel
* 39 march 1999, start of archive.
* Middleware packages DAVIC API implementation.
*
*/

package bali.cache;
import  bali.table.TableIdentifier;
import  bali.table.SITable;

/*****************
A single entry in a cache. This object allows a table to have an entry
in the cache before it is loaded. This is required to control caching policy.
@see SICache
*/
class SICacheElement
{
	/***********
	Enable/diable caching of the indicated cache lvl.
	@see SICache
	*/
	synchronized void setCacheLvl( int cacheLvl, boolean active )
	{
	    if ( active )
	    {
	        _cacheLvl = _cacheLvl|cacheLvl;
	    }
	    else
	    {
	        _cacheLvl = _cacheLvl&(~cacheLvl);
	    }
	}
	
	/***********
	Enable/diable caching of the indicated cache lvl.
	@see SICache
	*/
    synchronized boolean isSelected()
    {
        return (_cacheLvl!=0);
    }

	/***********
	Get the TableIdentifier of this SICacheElement.
	*/
	TableIdentifier getTableIdentifier()
	{
	    return _ti;
	}
	
	/***************
	Returns true when this cache element is a wildcard.
	*/
    boolean isWildCard()
    {
        return _ti.holdsWildCard();
    }

	/***********
	Replace the current SITable with table.
	*/
	synchronized void setSITable( SITable table )
	{
	    _siTable = table;
	}

	/***********
	Get the SITable of this SICacheElement (null is none is present).
	*/
	synchronized SITable getSITable()
	{
	    return _siTable;
	}
	
    /***********
	*/
	SICacheElement( TableIdentifier ti )
	{
	    _cacheLvl= 0;
	    _ti      = ti;
	    _siTable = null;
	}
	
    /////////////////////// PRIVATE ////////////////////////////
	private int             _cacheLvl;  // 0, not selected
	private TableIdentifier _ti;        // tabel identifier defining table to be cache.
	private SITable         _siTable;   // the stored table, null if none is present
	
}