/*
* Copyright(c) Philips Consumer Electronics B.V. 1998                         
* All rights reserved. Reproduction in whole or in part is                  
* prohibited without the written consent of the copyright holder.            
*                                                                            
*  Project             : Bali
*  Module              : bali.si 
*  File name   $Logfile: /func_test_platf2/bali/si/scheduler/RetrieveEvent.java $
*       Last save $Date: 2000/06/06 18:43:03 $
*             $Revision: 1.1 $
*          Last $Author: alavilli $
*  Creation Date       : 28/10/98
*  First Author        : S.A.F.A. van den Heuvel
*                        NatLab - Eindhoven
* 
*                                                                          
*  Change history      :
*  ---------------
*
* $Log: HwSfEvent.java,v $
* Revision 1.1  2000/06/06 18:43:03  alavilli
* After SISF integration into the CORE4 code
*
* Revision 1.1.2.1  1999/07/09 14:16:40  depetris
* Aggiornati i file per lo sviluppo e il rilascio a MPEG-J.
* Integra la versione 2.5 del core.
*
* Revision 1.1  1998-11-18 14:52:46+01  vdheuvel
* 39 march 1999, start of archive.
* Middleware packages DAVIC API implementation.
*
*/

package bali.hw.sectfilt;
import  bali.support.Debug;
import  java.util.EventObject;

/*********************
 This event is sent to indicate a section filter event.

 @see HwSectionFilterData
 @see HwSectionFilter
 @see HwSectionFilterListener
*/
public class  HwSfEvent
	extends EventObject
{
	/** A section was successfuly filtered. For Single: and filtering is stopped. */
	    public final static int NEW_SECTION    = 0;
    /** Filtering stopped because the source TS changed. */
    	public final static int TS_CHANGE      = 1;
    /** This filter action was canceled by the user. */
    	public final static int CANCELLED      = 2;
    /** This filter resource was revoked. */
    	public final static int RES_REVOKED    = 3;
    /** For Table filtering: a version change was detected in a table section filter, 
        filtering continues using the old version number. */
    	public final static int VERSION_CHANGE = 4;
    /** For Table filtering:a complete table has been filtered and filtering is stopped.*/
    	public final static int SUCCESS        = 5;

	/*******************
	@param source : HwSfController sending this event.
	*/
	public HwSfEvent( Object source, int result, HwSection sec, HwSfData fd )
	{ 
		super( source ); 
		_result = result;
		_sec = sec;
		_fd = fd;
	}
	
	/*******************
	Return the result of this retrieve action.
	*/
	public int getResult()
	{
		/*DBG*/ Debug.msg( Debug.SF, 5, "HwSfEvent.getResult");
		return _result;
	}

	/*******************
	Return the filtered section (NEW_SECTION).
	*/
	public HwSection getHwSection()
	{
		/*DBG*/ Debug.msg( Debug.SF, 5, "HwSfEvent.getHwSections");
		return _sec;
	}

	/*******************
	Return the FilterData of this retrieve action.
	*/
	public HwSfData getHwSfData()
	{
		/*DBG*/ Debug.msg( Debug.SF, 5, "HwSfEvent.getHwSfData");
		return _fd;
	}



	////////////////////////// PRIVATE /////////////////////////////////
	private int 			   _result;
	private HwSection   	   _sec;
	private HwSfData           _fd;
}

