/*
* Copyright(c) Philips Consumer Electronics B.V. 1998                         
* All rights reserved. Reproduction in whole or in part is                  
* prohibited without the written consent of the copyright holder.            
*                                                                            
*  Project             : Bali
*  Module              : bali.si 
*  File name   $Logfile: /func_test_platf2/bali/si/scheduler/TableSectionFilterData.java $
*       Last save $Date: 2000/06/06 18:43:04 $
*             $Revision: 1.1 $
*          Last $Author: alavilli $
*  Creation Date       : 28/10/98
*  First Author        : S.A.F.A. van den Heuvel
*                        NatLab - Eindhoven
* 
*                                                                          
*  Change history      :
*  ---------------
*
* $Log: SingleSfEnhancer.java,v $
* Revision 1.1  2000/06/06 18:43:04  alavilli
* After SISF integration into the CORE4 code
*
* Revision 1.1.2.1  1999/07/09 14:16:43  depetris
* Aggiornati i file per lo sviluppo e il rilascio a MPEG-J.
* Integra la versione 2.5 del core.
*
* Revision 1.1  1998-12-11 11:48:02+01  vdheuvel
* 39 march 1999, start of archive.
* Middleware packages DAVIC API implementation.
*
 * 
 * 1     9-11-98 13:59 Vdheuvel
 * Creation
*/

package bali.hw.sectfilt;
import  bali.support.Debug;
import  bali.support.Assert;
import  bali.support.Util;

/*******************
This HwSfEnhancer stops and sends HwSfEvent::SUCCESS when the first section has been filtered.
Any other event will result in stopping the filter and passing the event. When filtering has
been stopped, all following events are ignored.
*/
public class SingleSfEnhancer
    implements HwSfListener
{
	/**************
	May only be called with FilterData on a complete section table format.
	*/
	public SingleSfEnhancer( HwSfListener list )
	{
        /*DBG*/Debug.msg( Debug.HWSF_LE, Debug.IMPL, "SingleSfEnhancer ");
        _finished = false;
      	_list     = list;
	}
	
	/**************
	A new HwSfEvent, process the event and resend it.
	*/
    public void newHwSfEvent( HwSfEvent e)
    {
        /*DBG*/Debug.msg( Debug.HWSF_LE, Debug.IMPL, "SingleSfEnhancer newHwSfEvent");
        if ( !_finished )
        {
            HwSfController sfc = (HwSfController)e.getSource();
            _finished = true;
            sfc.stopFiltering();
            switch ( e.getResult() )
            {
            case HwSfEvent.NEW_SECTION:
                /*DBG*/Debug.msg(Debug.HWSF_LE, "SingleSfEnhancer new event NEW_SECTION");
                _sec = e.getHwSection();
                _list.newHwSfEvent( new HwSfEvent( e.getSource(), HwSfEvent.SUCCESS, null, e.getHwSfData() ) );
                break;
            default:
                /*DBG*/Debug.msg(Debug.HWSF_LE, "SingleSfEnhancer new event, error occured, stop filter");
                _list.newHwSfEvent( e );
                break;
            }
        }
        else
        {
            /*DBG*/Debug.msg(Debug.HWSF_LE, "Already finished, event ignored.");
        }
    }
    
    /**************
	Access the filtered information.
	*/
    public HwSection getSection()
    {
        /*DBG*/Debug.msg(Debug.HWSF_LE, Debug.LOW,"SingleSfEnhancer getSection "+_sec+" _finished : "+_finished);
        /*DBG*/Assert.assert( _finished );
        return _sec;
    }
	
	//////////////// PRIVATE ///////////////
	private HwSfListener _list;
    private boolean      _finished;
    private HwSection    _sec;
}







