/*
* Copyright(c) Philips Consumer Electronics B.V. 1998                         
* All rights reserved. Reproduction in whole or in part is                  
* prohibited without the written consent of the copyright holder.            
*                                                                            
*  Project             : Bali
*  Module              : bali.si 
*  File name   $Logfile: $
*       Last save $Date: 2000/06/06 18:43:07 $
*             $Revision: 1.1 $
*          Last $Author: alavilli $
*  Creation Date       : 09/02/99
*  First Author        : S.A.F.A. van den Heuvel
*                        NatLab - Eindhoven
* 
*                                                                          
*  Change history      :
*  ---------------
*
* $Log: PATMonitor.java,v $
* Revision 1.1  2000/06/06 18:43:07  alavilli
* After SISF integration into the CORE4 code
*
* Revision 1.1.2.1  1999/07/09 14:16:48  depetris
* Aggiornati i file per lo sviluppo e il rilascio a MPEG-J.
* Integra la versione 2.5 del core.
*
* Revision 1.1  1999-02-23 20:56:48+01  vdheuvel
* 39 march 1999, start of archive.
* Middleware packages DAVIC API implementation.
*
*/

package bali.retrieve;
import  bali.component.TransportStreamSelection;
import  bali.support.Debug;
import  bali.table.TableIdentifier;
import  bali.table.SITable;
import  bali.table.PAT;
import  bali.cache.SITableCache;
import  org.davic.mpeg.dvb.DvbTransportStream;

/**********************
Start monitor on PAT to support PMT filtering using TableIdentifier.
*/
class PATMonitor
    extends Monitor
{
    /*******************
    */
    static synchronized PATMonitor getInstance()
    {

        DvbTransportStream ts = TransportStreamSelection.getInstance().getCurrentTs();
        TableIdentifier    ti = new TableIdentifier( TableIdentifier.PAT, ts.getOriginalNetworkId(), ts.getTransportStreamId() );
        SITableCache.startCaching( ti, SITableCache.MONITOR );

        PAT pat  = (PAT)SITableCache.getSITable( ti );
        
        if ( pat == null )
        {   // table not present
            // filter new table
        
            /*DBG*/Debug.msg( Debug.MONITOR, "PATMoitor PAT not yet been filtered, filter PAT");
            RetrievalMediator rm;
            RetrievalEvent event;
            do
            {
                rm = RetrievalMediator.retrievePAT(null );
                event = rm.getRetrievalEvent();
            }
            while (  event.getResult() != RetrievalEvent.SUCCESS );

            pat = (PAT)event.getSITable();
            /*DBG*/Debug.msg( Debug.MONITOR, "PATMonitor PAT filtered");
        }
        // filtering and cache access done, PAT always in cache so monitor should
        // have access.
        // create PAT Monitor
        PATMonitor mon = new PATMonitor( ti );
        return mon;
    }
    
    /*******************
    */
    private PATMonitor( TableIdentifier ti )
    {
        super( null, ti );
        /*DBG*/Debug.msg( Debug.MONITOR, "PATMonitor init");

    }

    /*********************
    Access PAT to support direct TableIdentifier based PMT filtering.
    */
    public SITable getLatestSITable()
    {
        SITable pat = super.getLatestSITable();
        if ( pat == null )
        {
            /*DBG*/Debug.msg( Debug.MONITOR, "PATMoitor PAT not yet been filtered, filter PAT");
            RetrievalMediator rm;
            RetrievalEvent event;
            do
            {
                rm = RetrievalMediator.retrievePAT(null );
                event = rm.getRetrievalEvent();
                if ( event.getResult() == RetrievalEvent.SUCCESS )
        	    {
                    pat = event.getSITable();
                }
            }
            while (  event.getResult() != RetrievalEvent.SUCCESS );
        }
        return pat;
    }

}