/*
* Copyright(c) Philips Consumer Electronics B.V. 1998
* All rights reserved. Reproduction in whole or in part is
* prohibited without the written consent of the copyright holder.
*
*  Project             : Bali
*  Module              : bali.si
*  File name   $Logfile: /func_test_platf2/bali/si/scheduler/TableSectionFilterData.java $
*       Last save $Date: 2000/06/06 18:43:08 $
*             $Revision: 1.1 $
*          Last $Author: alavilli $
*  Creation Date       : 28/10/98
*  First Author        : S.A.F.A. van den Heuvel
*                        NatLab - Eindhoven
*
*
*  Change history      :
*  ---------------
*
* $Log: TableSetSfEnhancer.java,v $
* Revision 1.1  2000/06/06 18:43:08  alavilli
* After SISF integration into the CORE4 code
*
* Revision 1.1.2.1  1999/07/09 14:16:51  depetris
* Aggiornati i file per lo sviluppo e il rilascio a MPEG-J.
* Integra la versione 2.5 del core.
*
* Revision 1.1  1998-12-11 11:46:52+01  vdheuvel
* 39 march 1999, start of archive.
* Middleware packages DAVIC API implementation.
*
 *
 * 1     9-11-98 13:59 Vdheuvel
 * Creation
*/

package bali.retrieve;
import  bali.support.Debug;
import  bali.support.Util;
import  java.util.Vector;
import  java.util.Enumeration;
import  bali.hw.sectfilt.HwSfEvent;
import  bali.hw.sectfilt.HwSfListener;
import  bali.hw.sectfilt.HwSfController;
import  bali.hw.sectfilt.HwSection;
import  bali.hw.sectfilt.TableData;

/*******************
This class is the base class of all SfEnhancer classes for filtering
all tables of a single type in one PID simultaneously.
*/
abstract class TableSetSfEnhancer
    implements HwSfListener
{
	/**************
	*/
	TableSetSfEnhancer( HwSfListener list )
	{
        /*DBG*/Debug.msg(Debug.HWSF_LE, Debug.IMPL, "TableSetSfEnhancer constructor");
      	_list    = list;
      	_vecTd   = new Vector();
	}

	/**************
	A new HwSfEvent, process the event and resend it.
	*/
    public void newHwSfEvent( HwSfEvent event)
    {
        /*DBG*/Debug.msg(Debug.HWSF_LE, Debug.IMPL, "TableSetSfEnhancer newHwSfEvent");
        if ( !_finished )
        {
            HwSfController sfc = (HwSfController)event.getSource();
            switch ( event.getResult() )
            {
            case HwSfEvent.NEW_SECTION:
                /*DBG*/Debug.msg(Debug.HWSF_LE, "TableSetSfEnhancer new event NEW_SECTION");
                Enumeration e = _vecTd.elements();
                TableData td = null;
                HwSection[] arSec;
                boolean found = false;
                while ( e.hasMoreElements() && !found )
                {
                    td = (TableData)e.nextElement();
                    arSec = td.getSections();
                    if ( arSec.length>0 )
                    {
                        found = tablesAreEqual( arSec[0], (HwSection)event.getHwSection() );
                    }
                }
                if ( !found )
                {
                    td = new TableData();
                    _vecTd.addElement( td );
                }
                td.newSection( (HwSection)event.getHwSection() );
                break;

            default:
                /*DBG*/Debug.msg(Debug.HWSF_LE, "TableSetSfEnhancer new event, error occured, stop filter");
                _finished = true;
                sfc.stopFiltering();
                _list.newHwSfEvent( event );
                break;
            }
        }
        else
        {
            /*DBG*/Debug.msg(Debug.HWSF_LE, "TableSetSfEnhancer Already finished, event ignored.");
        }
    }

    /****************
    */
    synchronized Enumeration getTableData()
    {
        /*DBG*/Debug.msg(Debug.HWSF_LE, Debug.IMPL, "TableSetSfEnhancer getTableData");
        return _vecTd.elements();
    }

	//////////////// IMPLEMENTATION ///////////////
	private Vector       _vecTd = null;
	private HwSfListener _list;
	private boolean      _finished = false;

	/****************
	Return true when both sections come from the same table.
	*/
	abstract boolean tablesAreEqual( HwSection sec1, HwSection sec2 );

}







