/*
* Copyright(c) Philips Consumer Electronics B.V. 1998                         
* All rights reserved. Reproduction in whole or in part is                  
* prohibited without the written consent of the copyright holder.            
*                                                                            
*  Project             : Bali
*  File name   $Logfile: /func_test_platf2/bali/si/table/TOT.java $
*       Last save $Date: 2000/06/06 18:43:12 $
*             $Revision: 1.1 $
*          Last $Author: alavilli $
*  Creation Date       : 10/12/98
*  First Author        : S.A.F.A. van den Heuvel
*                        NatLab - Eindhoven
* 
*                                                                          
*  Change history      :
*  ---------------
*
* $Log: TOT.java,v $
* Revision 1.1  2000/06/06 18:43:12  alavilli
* After SISF integration into the CORE4 code
*
* Revision 1.1.2.1  1999/07/09 14:17:05  depetris
* Aggiornati i file per lo sviluppo e il rilascio a MPEG-J.
* Integra la versione 2.5 del core.
*
* Revision 1.2  1999-04-15 09:53:18+02  vdheuvel
* Files changed to hook the system up to the TM demultipler and the
* TM hardware. Changes include, swapping the java.util.Date class
* for the bali.tm.Date class (to prevent using 64 bit floats). Changing
* the HW sect filt used by bali.hw.SectFilt.
*
* Revision 1.1  1999-02-12 15:14:10+01  vdheuvel
* 39 march 1999, start of archive.
* Middleware packages DAVIC API implementation.
*
 * 
 * 3     10-11-98 15:54 Vdheuvel
 * Bug fix in TDT and TOT parsing. Changed SITable to support empty
 * constructor.
 * 
 * 2     10-11-98 11:26 Vdheuvel
 * Added getDescriptorList
 * 
 * 1     10-11-98 10:16 Vdheuvel
 * Time OffSet Table wrapper.
*/
package bali.table;

/*DATE_NORMAL*/ import  java.util.Date;
//DATE_TM*/     import  bali.tm.Date;
import bali.support.Debug;
import bali.support.TablePart;
import bali.hw.sectfilt.HwSection;
import org.davic.mpeg.dvb.DvbTransportStream;
import org.davic.mpeg.dvb.DvbTransportStream;

/****************************************
This class parses the filtered PAT tables and allows access to the different PAT components.
*/
public class TOT
    extends SITable
{
	/** */ public final static int PID = 0x14; 
	/** */ public final static int TABLE_ID = 0x73; 

	/***************************************
	*/
	public TOT( TOT table, DvbTransportStream ts, Date filterTime )
	{
		super( table, new SITableInfo( 0, true, ts, filterTime ) );
		_descriptorList = table._descriptorList;
	}

	/***************************************
	*/
	public TOT( HwSection[] arSec, DvbTransportStream ts, Date filterTime )
	    throws InvalidTableException
    {
		super( arSec, new SITableInfo( 0, true, ts, filterTime ) );
        /*DBG*/ Debug.msg(Debug.TABLE, "parse TOT");

       	int descLength = _arSec[0].get12bits( 8 );
	    int descStart  = 10;
        _descriptorList = new DescriptorList( new TablePart( _arSec[0], descStart, descLength ) );
    }

    /*****************
    Returns the TableIdentifier of this TOT.
    */
    public TableIdentifier getTableIdentifier()
    {
        synchronized( this )
        {
            if ( _tableIdentifier == null )
            {
                DvbTransportStream ts = _ti.getSourceTs();
                _tableIdentifier = new TableIdentifier( TableIdentifier.TDT, ts.getOriginalNetworkId(), ts.getTransportStreamId() );
            }
        }
        return _tableIdentifier;
    }

	/***************************************
	Return the UTC time of this TDT
	*/
    public Date getUTCTime()
    {
 		return _arSec[0].getUTCTime( 3 );
    }

	/***************************************
	Return the descriptors of the main loop of the TOT table.
	*/
	public DescriptorList getDescriptorList()
    {
		return _descriptorList;
    }

    /////////////// IMPLEMENTATION ////////////////
    private DescriptorList _descriptorList;	
}
