/*******************************************************************************
* Copyright(c) Philips Consumer Electronics B.V. 1998                         
* All rights reserved. Reproduction in whole or in part is                  
* prohibited without the written consent of the copyright holder.            
*                                                                            
*  Project             : Bali
*  File name   $Logfile: $
*       Last save $Date: 2000/06/06 18:26:18 $
*             $Revision: 1.1 $
*          Last $Author: alavilli $
*  Creation Date       : end 1998
*  First Author        : Bas van den Heuvel
*                        NatLab - Eindhoven
* 
*  Change history      :
*  ---------------
*
* $Log: TMSectFilt.java,v $
* Revision 1.1  2000/06/06 18:26:18  alavilli
* After SISF integration into the CORE4 code
*
* Revision 1.1.2.1  1999/07/09 14:17:09  depetris
* Aggiornati i file per lo sviluppo e il rilascio a MPEG-J.
* Integra la versione 2.5 del core.
*
* Revision 1.2  1999-04-14 11:48:31+02  vdheuvel
* Changed simulation platform and HwSectFilt module from sommit
* to tm section filter.
*
*******************************************************************************/

package bali.tm.sectfilt;
import bali.support.TablePart;

public class TMSectFilt
{

    /**
     * Starts filtering for sections with the specified PID, table_id and table_id_extension. <br>
     * It informs <code>listener</code> whenever a section is received which matches these parameters, 
     * by calling the iStore(int, Section) method. <br><br>
     *
     * The <code>identifier</code> is passed to the listener with any section received, so it can 
     * determine if the data is 'stale'. <br><br>
     *
     * If the <code>table_id_extension</code> is -1 then sections with any table_id_extension will 
     * be passed to the listener.
     *
     * @param identifier unique identifier
     * @param index which section filter to use [1 - 16]
     * @param PID PID of sections to filter
     * @param table_id table_id of sections to filter
     * @param table_id_extension table_id_extension of sections to filter [or -1]
     * @param listener listener to inform when a section is received matching the parameters
     *
     * @see SectionFilterFriendInt#iStore(int, Section)
    */
    public static void setFilter( int identifier, int index, int PID, int table_id, int table_id_extension,
    							 SectFiltListener listener)
    {
        //DBG*/System.out.println("TMSectFilt.setFilter"+index );
        initialise();
        synchronized( _lock )
        {
            FilterData fd = new FilterData( identifier, PID, table_id, table_id_extension, listener);
            _arFilter[ index ] = fd ;
        }
    }

    /**
    * Stop the specified filter from passing any more sections to the listener. <br><br>
    *
    * @param index which section filter to stop [1 - 16]
    */
    public static void clearFilter(int index)
    {
        //DBG*/System.out.println("TMSectFilt.clearFilter"+index);
       initialise();
       synchronized( _lock )
       {
            _arFilter[ index ] = null;
       }
    }
   
    /**
    * New section in transport stream
    */
    static public void newSection( int PID, byte[] sec )
    {
     //DBG System.out.println("TMSectFilt.newSection "+PID);
     initialise();
     synchronized( _lock )
     {
        for ( int i=0; i<NFILTER; i++ )
        {
          if ( _arFilter[i] != null )
          { if ( checkSection( PID, sec, _arFilter[i] ) )
            {  _arFilter[i].listener.iStore( _arFilter[i].identifier, sec );
            }
          }
        }
     }
    }
   
   /////////////// PRIVATE /////////////////
   private static        Object       _lock = new Object();
   private static        FilterData[] _arFilter = null;
   private static final  int          NFILTER = 16; 
  
   private static boolean checkSection( int PID, byte[] sec, FilterData fd )
   {
        //DBG*/ System.out.println("TMSectFilt.checkSection"+PID+","+sec.table_id()+","+sec.table_id_extension());  
        if ( fd.PID == PID )
        {  //DBG*/ System.out.println("TMSectFilt.checkSection correct PID");
          //DBG*/ System.out.println("TMSectFilt.checkSection table_id"+fd.table_id+" "+sec.table_id());
          TablePart tpSec = new TablePart( sec );
          if ( (fd.table_id==-1) || (fd.table_id== tpSec.get8bits(0) ) )
          { //DBG*/ System.out.println("TMSectFilt.checkSection table_id"+fd.table_id_extension+" "+sec.table_id_extension());
            int ti_ext = (tpSec.get8bits(3) << 8) + tpSec.get8bits(4);
            if (  (fd.table_id_extension==-1) 
               || (fd.table_id_extension==ti_ext )
               )
            { return true;
            }
          }
        }   
        
        return false;
   }

   private static void initialise()
   {
     synchronized( _lock )
     {
       if ( _arFilter == null )
       {
         //DBG*/ System.out.println("TMSectFilt : initialise");
         _arFilter = new FilterData[NFILTER];
         for ( int i=0; i<NFILTER; i++ )
         {   _arFilter[i] = null;
         }
       }
     }
   }
}