/*
* Copyright(c) Philips Consumer Electronics B.V. 1998                         
* All rights reserved. Reproduction in whole or in part is                  
* prohibited without the written consent of the copyright holder.            
*                                                                            
*  Project             : Bali
*  File name   $Logfile: /func_test_platf2/org/davic/net/dvb/si/TimeOutException.java $
*       Last save $Date: 2000/06/06 18:43:21 $
*             $Revision: 1.1 $
*          Last $Author: alavilli $
*  Creation Date       : 28/10/98
*  First Author        : S.A.F.A. van den Heuvel
*                        NatLab - Eindhoven
* 
*                                                                          
*  Change history      :
*  ---------------
*
* $Log: TimeOutException.java,v $
* Revision 1.1  2000/06/06 18:43:21  alavilli
* After SISF integration into the CORE4 code
*
* Revision 1.1.2.1  1999/07/09 14:17:28  depetris
* Aggiornati i file per lo sviluppo e il rilascio a MPEG-J.
* Integra la versione 2.5 del core.
*
* Revision 1.1  1998-11-05 12:17:56+01  vdheuvel
* start of archive, 29 march 1999
* DAVIC API implementation
*
 * 
 * 1     9-11-98 14:01 Vdheuvel
 * Creation
*/
package org.davic.net.dvb.si;
import  java.util.Vector;
import  java.util.Enumeration;

/**
 * This exception is thrown when required data could not be obtained before the time out has expired. 
 * The time out duration is at least two times the minimum repetition rate for 
 * the SI table source as specified in ETR 211. The actual value is implementation dependant.
 *
 * @see     RetrieveFailedException
 * @since   DAVIC1.3
 */

public class TimeOutException
	extends RetrieveFailedException
{
    /** 
     * Creates a new TimeOutException.
     *
     * @param  data  The part of the data filtered before the TimeOut occurred, null indicates no data.
     */
    public TimeOutException( SIIterator data )
    {
        _it = data;
    }

    /** 
     * This method returns the part of the data requested by the 
     * retrieve, that was filtered before the TimeOut occurred.
     *
     * @return  The filtered data or null.
     */
    public SIIterator getFilteredData()
    {
        if ( _vec != null )
        {   return new SIIteratorImpl( _vec );
        }
        else
        {   return ( _it );
    	}
    }

    ////////////////// IMPLEMENTATION ////////////////
    SIIterator _it  = null ;
    Vector     _vec = null ;

    /*****************
    */
    TimeOutException( Vector vec )
    {
        _vec = vec;
    }

    /*****************
    */
    TimeOutException( )
    {
        _vec = new Vector();
    }
}
