/*-------------------------------------------------------------------------
Project Im1Player
INTERNATIONAL ORGANIZATION FOR STANDARDIZATION
ISO/IEC JTC1/SC29/WG11

SUBSYSTEM:  Multiplexer
FILE:       IPMPKEY.cpp
AUTHOR:     Franco Casalino (CSELT)
OVERVIEW:   Implementation of IPMPKEYStream, the class that handles IPMPKEY
input streams.
-------------------------------------------------------------------------*/
#include "defs.h"
#include "IPMPKEY.h"
#include "assert.h"
#include "odMux.h"

/*-----------------------------------------------------------------
IPMPKEYStream constructor reads first character
-----------------------------------------------------------------*/
IPMPKEYStream :: IPMPKEYStream (FILE *fStream, ESDesc *pDescriptor) :
	Stream (fStream, pDescriptor)
{
  	m_nCount = 0;
	m_nAU = 0;
	m_cNext = getc ();
}

/*-----------------------------------------------------------------
For IPMPKEY, each AU the KEY.
The duration of each AU is then related to the audio or video sampling rate 
-----------------------------------------------------------------*/
long IPMPKEYStream :: GetDTS ()
{
  if (m_cNext == EOF)
		return -1;

  a[0] = m_cNext;
  a[1] = getc ();
  a[2] = getc ();
  a[3] = getc ();
  m_cNext = getc ();
  if (m_cNext == EOF)
	  return -1;

  m_lDTS = ((a[0]<<24) | (a[1]<<16) |(a[2]<<8) | (a[3]));
  m_nCount = 0;
  return m_lDTS;
}

/*-----------------------------------------------------------------
GetChar returns a character and reads one char ahead.  The number
of characters in an AU is determined by the data rate, which is
given in the pDescriptor -> streamData field.
-----------------------------------------------------------------*/
int IPMPKEYStream :: GetChar ()
{
	if (m_nCount >= 16)
		return -1;
	++m_nCount;
	int c = m_cNext;
	m_cNext = getc ();
	assert (c != EOF);
	return c;
}

/*----------------------------------------------------------------
Returns true if the AU referenced by the last call to GetDTS()
was a Random Access Point
-----------------------------------------------------------------*/
bool IPMPKEYStream :: IsRAP ()
{
	return true;
}

static StreamFactory<IPMPKEYStream> bifsFactory ("IPMPKEY");
