/***********************************************************************
This software module was originally developed by Franco casalino
(CSELT) in the course of development of the MPEG-4 Systems
(ISO/IEC 14496-1) standard. This software module is an implementation of
a part of one or more MPEG-4 Systems (ISO/IEC 14496-1) tools as
specified by the MPEG-4 Systems (ISO/IEC 14496-1) standard. ISO/IEC
gives users of the MPEG-4 Systems (ISO/IEC 14496-1) free license to
this software module or modifications thereof for use in hardware or
software products claiming conformance to the MPEG-4 Systems (ISO/IEC
14496-1). Those intending to use this software module in hardware or
software products are advised that its use may infringe existing patents.
The original developer of this software module and his/her company, the
subsequent editors and their companies, and ISO/IEC have no liability
for use of this software module or modifications thereof in an
implementation. Copyright is not released for non MPEG-4 Systems
(ISO/IEC 14496-1) conforming products. Triton R&D Ltd. retains full right
to use the code for his/her own purpose, assign or donate the code to a
third party and to inhibit third parties from using the code for non
MPEG-4 Systems (ISO/IEC 14496-1) conforming products. This copyright
notice must be included in all copies or derivative works.
Copyright  1998.
************************************************************************

For more information or to receive an up to date version of this module,
you may call Franco Casalino at Franco.casalino@cselt.it.
----------------------------------------------------------------------*/

#include "defs.h"
#include "Mpg4VidFDIS.h"
#include "odMux.h"

/*-----------------------------------------------------------------
Mpg4VidFDISStream constructor reads first character
-----------------------------------------------------------------*/
Mpg4VidFDISStream :: Mpg4VidFDISStream (FILE *fStream, ESDesc *pDescriptor) :
	Stream (fStream, pDescriptor)
{  
    m_SYNC = 0;
    m_nCount = 0;
	m_lFrames = 0;
	m_cNext = getc ();
	// The stream data specifies the frame rate to use (default to 15)
	if ((pDescriptor -> streamData == NULL) || (pDescriptor -> streamData[0] == 0))
		m_frameRate = 25;
	else
	{
		m_frameRate = atof(pDescriptor -> streamData);
		if (m_frameRate < 0.0001)
			throw ExceptionInvalidData (
				"The MPEG-4 frame rate must be greater than 0.\n"
			);
	}
}

/*-----------------------------------------------------------------
Time stamps are calculated according to the rate given by the
pDescriptor -> streamData field
-----------------------------------------------------------------*/
long Mpg4VidFDISStream :: GetDTS ()
{
	if (m_cNext == EOF)
		return -1;

	if (m_lFrames == 0) {
      a[0] = m_cNext;
	  a[1] = getc ();
	  m_cNext = getc ();
	  m_HEADER = ((a[1]<<8) | (a[0]));
	}
	++m_lFrames;
	m_nCount = 0;
	do
	{
      for (int i=0; i< m_HEADER; i++) { 
		a[i] = m_cNext;
	  	m_cNext = getc ();
	  }
	  if (m_cNext == EOF)
		  return -1;
	  m_nFrameSize = ((a[3]<<24) | (a[2])<<16 | (a[1]<<8) | (a[0]));
	  m_SYNC = (a[4]);
	  m_DTS = ((a[8]<<24) | (a[7])<<16 | (a[6]<<8) | (a[5]));
	  m_CTS = ((a[12]<<24) | (a[11])<<16 | (a[10]<<8) | (a[9]));
	  m_IPB_TYPE = (a[15]);
	}
	while (m_nFrameSize == 0);
	assert (m_nFrameSize != 0);
	return m_DTS;
		
/*
	m_lDTS = (long)((float)(m_lFrames * 1000) / m_frameRate);
	++m_lFrames;
	m_nCount = 0;
	do
	{
      a[0] = m_cNext;
	  a[1] = getc ();
	  a[2] = getc ();
	  a[3] = getc ();
	  a[4] = getc ();
	  a[5] = getc ();
	  a[6] = getc ();
	  a[7] = getc ();
	  a[8] = getc ();
	  a[9] = getc ();
	  a[10]= getc ();
	  a[11]= getc ();
	  a[12]= getc ();
	  a[13]= getc ();
	  a[14]= getc ();
	  a[15]= getc ();
	  m_cNext = getc ();
	  if (m_cNext == EOF)
		  return -1;
	  m_HEADER = ((a[1]<<8) | (a[0]));
	  m_nFrameSize = ((a[5]<<24) | (a[4])<<16 | (a[3]<<8) | (a[2]));
	  m_SYNC = (a[6]);
	  m_DTS = ((a[10]<<24) | (a[9])<<16 | (a[8]<<8) | (a[7]));
	  m_CTS = ((a[14]<<24) | (a[13])<<16 | (a[12]<<8) | (a[11]));
	  m_IPB_TYPE = (a[15]);
	}
	while (m_nFrameSize == 0);
	assert (m_nFrameSize != 0);
	return m_lDTS;
*/
}

long Mpg4VidFDISStream :: GetCTS ()
{
	return m_CTS;
}

/*-------------------------------------------------------------
 RAP = m_SYNC
-------------------------------------------------------------*/
bool Mpg4VidFDISStream :: IsRAP ()
{
	return (bool)m_SYNC;
}

/*-----------------------------------------------------------------
GetChar returns a character and reads one next char. We are always
4 chars ahead, which are used to check whether this is end of AU.
-----------------------------------------------------------------*/
int Mpg4VidFDISStream :: GetChar ()
{
    if (m_nCount >= m_nFrameSize)
		return -1;
	++m_nCount;
	int c = m_cNext;
	m_cNext = getc ();
	assert (c != EOF);
	return c;
}

static StreamFactory<Mpg4VidFDISStream> bifsFactory ("MPEG4-VideoFDIS");