/***********************************************************************
This software module was originally developed by Zvi Lifshitz
(Triton R&D Ltd.) in the course of development of the MPEG-4 Systems
(ISO/IEC 14496-1) standard. This software module is an implementation of
a part of one or more MPEG-4 Systems (ISO/IEC 14496-1) tools as
specified by the MPEG-4 Systems (ISO/IEC 14496-1) standard. ISO/IEC
gives users of the MPEG-4 Systems (ISO/IEC 14496-1) free license to
this software module or modifications thereof for use in hardware or
software products claiming conformance to the MPEG-4 Systems (ISO/IEC
14496-1). Those intending to use this software module in hardware or
software products are advised that its use may infringe existing patents.
The original developer of this software module and his/her company, the
subsequent editors and their companies, and ISO/IEC have no liability
for use of this software module or modifications thereof in an
implementation. Copyright is not released for non MPEG-4 Systems
(ISO/IEC 14496-1) conforming products. Triton R&D Ltd. retains full right
to use the code for his/her own purpose, assign or donate the code to a
third party and to inhibit third parties from using the code for non
MPEG-4 Systems (ISO/IEC 14496-1) conforming products. This copyright
notice must be included in all copies or derivative works.
Copyright  1998.
************************************************************************

For more information or to receive an up to date version of this module,
you may call Zvi Lifshitz at zvil@csi.com.
----------------------------------------------------------------------*/

/*-------------------------------------------------------------------------
Project Im1Player
INTERNATIONAL ORGANIZATION FOR STANDARDIZATION
ISO/IEC JTC1/SC29/WG11

SUBSYSTEM:  Multiplexer
FILE:       MuxOD.cpp
AUTHOR:     Zvi Lifshitz (Triton R&D Ltd.)
						Muxcode implementation by Carsten Herpel (THOMSON multimedia)
OVERVIEW:   Implementation of ES_DescriptorMux
-------------------------------------------------------------------------*/
#include "defs.h"
#include "odMux.h"
#include "MuxStream.h"

extern BOOL g_bErrors;

/*-------------------------------------------------------------
ES_DescriptorMux constructor
-------------------------------------------------------------*/
ES_DescriptorMux :: ES_DescriptorMux () :
	m_fStream (NULL),
	m_pStream (NULL),
	m_lTS (-1),
	m_nPayloadLength (0),
	m_bStart (TRUE),
	m_bEnd (FALSE),
	m_dwOCRrate (0xffffffff),
	m_bSlotStored (FALSE),
	m_bEndOfAU (FALSE),
	m_coder (NULL, NULL, "SLHeader", TRACE_NONE)
{}

/*-----------------------------------------------------------------
ES_DescriptorMux destructor. Close the stream file
-----------------------------------------------------------------*/
ES_DescriptorMux :: ~ES_DescriptorMux ()
{
	delete m_pStream;
	if (m_fStream)
		fclose (m_fStream);
}

/*-----------------------------------------------------------------
Array of factory objects to create proper Stream object
-----------------------------------------------------------------*/
StreamFactoryBase *ES_DescriptorMux :: m_pFactories [40];
int ES_DescriptorMux :: m_nFactoriesAvailable;

/*-----------------------------------------------------------------
Register a StreamFactory object
-----------------------------------------------------------------*/
void ES_DescriptorMux :: RegisterFactory (StreamFactoryBase *pFactory)
{
	assert (m_nFactoriesAvailable < 40);
	m_pFactories [m_nFactoriesAvailable++] = pFactory;
}

/*-----------------------------------------------------------------
Open the stream file, and create a proper Stream object
-----------------------------------------------------------------*/
int ES_DescriptorMux :: OpenFile (
	int nFMC,
	FILE *fMP4,
	FILE *fList,
	int flags
)
{
	/*-----------------------------------------------------------------
	Open the stream file
	-----------------------------------------------------------------*/
	if (muxInfo == NULL_DESC)
	{	printf ("No Mux info for stream no %d\n", ES_ID);
		return -1;
	}
	if (muxInfo -> fileName.GetLength () == 0)
		return 0;
	m_fStream = fopen (muxInfo -> fileName, "rb");
	if (!m_fStream)
	{	printf ("Cannot open %s\n", (LPCSTR) muxInfo -> fileName);
		return -1;
	}

	/*-----------------------------------------------------------------
	Instantiate a proper Stream object
	-----------------------------------------------------------------*/
	if (decConfigDescr == NULL_DESC)
	{	printf (
			"%s: decConfigDescr not available\n",
			(LPCSTR) muxInfo -> fileName
		);
		return -1;
	}

	LPCSTR szStreamType = muxInfo -> streamFormat;
	for (int i = m_nFactoriesAvailable; --i >= 0;)
		if (m_pFactories [i] -> IsTypeOf (szStreamType)
		)
		{	ESDesc *pDesc = new ESDesc;
			pDesc -> startTime = muxInfo -> startTime;
			pDesc -> fileName = muxInfo -> fileName;
			pDesc -> streamType = muxInfo -> streamFormat;
			pDesc -> streamData = muxInfo -> streamAttr;
			pDesc -> repeatRate = muxInfo -> repeatRate;
			pDesc -> sessionLength = muxInfo -> sessionLength;
			pDesc -> maxAhead = muxInfo -> maxAhead;
			m_pStream = m_pFactories [i] -> Create (m_fStream, pDesc);
			break;
		}
	if (!m_pStream)
	{	printf (
			"%s: Unknown decoder type %s\n",
			(LPCSTR) muxInfo -> fileName,
			(LPCSTR) szStreamType
		);
		return -1;
	}

	m_fMP4 = fMP4;
	m_coder.SetTraceFile (fList, muxInfo -> streamFormat, flags);
	m_nFMC = nFMC;
	m_lTS = -1;
	m_nPDUSequenceNumber = m_nAUSequenceNumber = 0;
	m_dwNextOCR = 0;
	return 1;
}

/*-----------------------------------------------------------------
Ask the stream for next time stamp
-----------------------------------------------------------------*/
long ES_DescriptorMux :: GetDTS (long lNow)
{
	if (m_lTS < 0)
		m_lTS = m_pStream -> GetDTS (lNow);
	return m_lTS;
}

/*-----------------------------------------------------------------
Store one access unit (or the remainder thereof)
-----------------------------------------------------------------*/
void ES_DescriptorMux :: Store (BOOL bContinue)
{
	BYTE packet [257];
	int nLength, nDataLength;

	m_bSlotStored = false;
	if (!bContinue) {
		m_nAULength = 0;
		m_bEndOfAU = false;
	}

	while (!m_bEndOfAU)
	{	m_bEndOfAU = BuildPacket (packet, 0, nLength, nDataLength);
	  assert(nLength>=0);
		m_nAULength += nDataLength;
		fwrite (packet, nLength, 1, m_fMP4);
	}
			
	m_lTS = -1;

	/*-------------------------------------------------------------
	Make sure requested buffer length can hold all access units
	-------------------------------------------------------------*/
	if (m_nAULength > decConfigDescr -> bufferSizeDB)
	{	printf (
			"WARNING: decoding buffer cannot hold long access units"
			". File: %s\n",
			(LPCSTR) muxInfo -> fileName
		);
		g_bErrors = TRUE;
	}
}

/*-----------------------------------------------------------------
Build one packet or muxcode slot. Return TRUE if the last packet of  
an AU was generated.
-----------------------------------------------------------------*/
BOOL ES_DescriptorMux :: BuildPacket (
	LPBYTE pPacket,
	int nMaxSize,
	int &rnLength, 
	int &rnDataLength
)
{
	BOOL bBuildFmxPacket = nMaxSize==0;
	if (bBuildFmxPacket)
		nMaxSize = 255;
	/*-----------------------------------------------------------------
	Fill the packet and return if end of AU or packet is full
	-----------------------------------------------------------------*/
	while (!m_bEnd && m_nPayloadLength < nMaxSize)
	{	int c = m_pStream -> GetChar ();
		m_bEnd = c < 0;
		if (!m_bEnd) {
			m_payload [m_nPayloadLength++] = c;
		}
	}	
	if (!m_bEnd && m_nPayloadLength == nMaxSize)
		m_bEnd = m_pStream -> IsEndOfAU(); // Caution: dont call if end already detected by GetChar!

	/*-------------------------------------------------------------
	Check if an OCR is required in this packet
	-------------------------------------------------------------*/
	bool bOCR = false;
	if (m_dwOCRrate > 0 && (DWORD) m_lTS >= m_dwNextOCR)
	{	bOCR = true;
		m_dwNextOCR += m_dwOCRrate;
	}
	if (m_bStart && m_pStream -> IsRepeated ())
			--m_nAUSequenceNumber;	

	/*-----------------------------------------------------------------
	Prepare the header. If after that it turns out that the
	header + payload is too long, truncate the payload
	-----------------------------------------------------------------*/
	PrepareHeader (m_bStart, m_bEnd, bOCR);
	if (m_bEnd && m_coder.GetDataLength () + m_nPayloadLength > nMaxSize)
		PrepareHeader (m_bStart, FALSE, bOCR);

	/*-----------------------------------------------------------------
	Store the packet.
	-----------------------------------------------------------------*/
	int nPacketLength = nMaxSize - m_coder.GetDataLength ();

	if (nPacketLength <= 0) {
		rnLength = -1;
		rnDataLength = 0;
		nPacketLength = 0;
	}
	else
	{
		if (nPacketLength > m_nPayloadLength)
			nPacketLength = m_nPayloadLength;

		int i = 0;
		int nHeaderLength = m_coder.GetDataLength ();

		if (bBuildFmxPacket) {			// add the FlexMux packet header
			pPacket [i++] = m_nFMC;
			pPacket [i++] = nHeaderLength + nPacketLength;
		}
	
		memcpy (pPacket + i, m_coder.GetData (), nHeaderLength);
		i += nHeaderLength;
		memcpy (pPacket + i, m_payload, nPacketLength);
		rnLength = i += nPacketLength;
		rnDataLength = nPacketLength;
		++m_nPDUSequenceNumber;
		m_slHeader.m_nOCROffset += 16;

	/*-----------------------------------------------------------------
	The data that was not stored is shifted to the beginning of
	the buffer
	-----------------------------------------------------------------*/
		m_nPayloadLength -= nPacketLength;
		if (m_nPayloadLength > 0)
			memcpy (m_payload, m_payload + nPacketLength, m_nPayloadLength);
	}

	if (m_bEnd && m_nPayloadLength == 0)
	{	
		m_bStart = TRUE;
		m_bEnd = FALSE;
		if (slConfigDescr != NULL_DESC &&	slConfigDescr -> AU_seqNumLength > 0)
			++m_nAUSequenceNumber;
		return TRUE;
	}
	else
	{	
		if (nPacketLength > 0)
			m_bStart = FALSE;
		return FALSE;
	}
}

/*-----------------------------------------------------------------
Prepare a packet header
-----------------------------------------------------------------*/
void ES_DescriptorMux :: PrepareHeader (bool bStart, bool bEnd, bool bOCR)
{
	/*-------------------------------------------------------------
	Header fields are saved in a sturcture for follow-up. Initially
	all is set to -1
	-------------------------------------------------------------*/
	m_slHeader.m_nAccessUnitStart = m_slHeader.m_nAccessUnitEnd =
	m_slHeader.m_nRandomAccess = m_slHeader.m_nPDUseqNum =
	m_slHeader.m_nAUseqNum = m_slHeader.m_nAULength =
	m_slHeader.m_nDTS = m_slHeader.m_nCTS = -1;
	m_slHeader.m_nOCRLength = 0;
	
	/*-------------------------------------------------------------
	Generate the header
	-------------------------------------------------------------*/
	SLConfigDescriptor configDefault;
	SLConfigDescriptor *pConfig = slConfigDescr;
	if (pConfig == NULL)
		pConfig = &configDefault;

	bool randomAccessPointFlag = bStart & m_pStream -> IsRAP ();
	bool paddingFlag =
		pConfig -> usePaddingFlag && m_pStream -> GetPaddingBits () > 0;
	bool compositionTimeStampFlag =
		pConfig -> useTimeStampsFlag && m_pStream -> GetCTS () >= 0;
	bool decodingTimeStampFlag = 
		compositionTimeStampFlag && m_pStream -> GetCTS() != m_lTS;
	bool instantBitrateFlag = FALSE;
	int	objectClockReference = 0;
	int accessUnitLength = 0;		// To be implemented correctly
	int instantBitrate = 0;
	int degradationPriority = 0;

	/*-------------------------------------------------------------
	Check if SLConfigDescriptor is suitable for this stream
	-------------------------------------------------------------*/
	if (!bEnd && !pConfig -> useAccessUnitEndFlag)
	{	printf (
			"ERROR: Long access units, must use AccessUnitEndFlag. "
			"File: %s\n",
			(LPCSTR) muxInfo -> fileName
		);
		g_bErrors = TRUE;
	}

	/*-------------------------------------------------------------
	Encode the header
	-------------------------------------------------------------*/
	SL_PacketHeader packetHeader (
		m_slHeader.m_nAccessUnitStart = bStart
	);
	m_coder.Reset ();
	packetHeader.randomAccessPointFlag = m_slHeader.m_nRandomAccess = randomAccessPointFlag;
	packetHeader.accessUnitEndFlag = m_slHeader.m_nAccessUnitEnd = bEnd;
	packetHeader.OCRflag = bOCR;
	packetHeader.paddingFlag = paddingFlag;
	packetHeader.paddingBits = m_pStream -> GetPaddingBits ();
	packetHeader.packetSequenceNumber =
		m_slHeader.m_nPDUseqNum = m_nPDUSequenceNumber;
	packetHeader.objectClockReference = objectClockReference;
	packetHeader.AU_sequenceNumber = 
		m_slHeader.m_nAUseqNum = m_nAUSequenceNumber;
	packetHeader.decodingTimeStampFlag = decodingTimeStampFlag;
	packetHeader.compositionTimeStampFlag = compositionTimeStampFlag;
	packetHeader.instantBitrateFlag = instantBitrateFlag;
	m_slHeader.m_nDTS = m_lTS;
	packetHeader.decodingTimeStamp =
		m_lTS * (pConfig -> timeStampResolution / 1000.);
	m_slHeader.m_nCTS =
		m_pStream -> GetCTS () *
		(pConfig -> timeStampResolution / 1000.);
	packetHeader.compositionTimeStamp = m_slHeader.m_nCTS;
	packetHeader.accessUnitLength = m_slHeader.m_nAULength = accessUnitLength;
	packetHeader.instantBitrate = instantBitrate;
	packetHeader.degradationPriority = degradationPriority;
	packetHeader.EncDec (&m_coder, ENCODE, pConfig);
	m_coder.Align ();
	if (bOCR)
	{	m_slHeader.m_nOCRLength = pConfig -> OCRLength;
		m_slHeader.m_nOCROffset = packetHeader.GetOCROffset ();
		m_slHeader.m_nOCRResolution = pConfig -> OCRResolution;
	}
}

/*-----------------------------------------------------------------
Store one slot in a muxcode packet
-----------------------------------------------------------------*/
BOOL ES_DescriptorMux :: StoreSlot (BYTE *packet, int nSlotSize, int &rnLength)
{
	int nDataLength;

	m_bSlotStored = true;

	if (m_bEndOfAU)
		m_nAULength = 0;

	m_bEndOfAU = BuildPacket (packet, nSlotSize, rnLength, nDataLength);

	m_nAULength += nDataLength;
	
	/*-------------------------------------------------------------
	Make sure requested buffer length can hold all access units
	-------------------------------------------------------------*/
	if (m_bEndOfAU)
	{
		if (m_nAULength > decConfigDescr -> bufferSizeDB)
		{	printf (
				"WARNING: decoding buffer cannot hold long access units"
				". File: %s\n",
				(LPCSTR) muxInfo -> fileName
			);
			g_bErrors = TRUE;
		}
		m_lTS = -1;
	}
	return m_bEndOfAU;
}

/*-----------------------------------------------------------------
The StreamFactoryBase constructor registers a factory object
-----------------------------------------------------------------*/
StreamFactoryBase :: StreamFactoryBase (LPCSTR szType)
{
	m_szType = szType;
	ES_DescriptorMux :: RegisterFactory (this);
}

/*-----------------------------------------------------------------
Check if a stream factory object is of a required type
-----------------------------------------------------------------*/
BOOL StreamFactoryBase :: IsTypeOf (LPCSTR szType)
{
	return
		(	szType != NULL && m_szType != NULL &&
			!stricmp (szType, m_szType)
		);
}
