#include "FieldValueMaker.h"
#include <string.h>

FieldValueMaker* FieldValueMaker::s_self = 0;

FieldValuePtr FieldValueMaker::createFieldValue( JNIEnv *env, jobject val )
{
	if ( s_self == 0 )
		s_self = new FieldValueMaker( env );
	return s_self->doCreateFieldValue( env, val );
}

FieldValueMaker::FieldValueMaker( JNIEnv *env )
{
	int idx = 0;
	classTypes[ idx++ ] = env->FindClass( "org.iso.mpeg.scene.SFBoolFieldValue" );
	classTypes[ idx++ ] = env->FindClass( "org.iso.mpeg.scene.SFColorFieldValue" );
	classTypes[ idx++ ] = env->FindClass( "org.iso.mpeg.scene.SFFloatFieldValue" );
	classTypes[ idx++ ] = env->FindClass( "org.iso.mpeg.scene.SFInt32FieldValue" );
	classTypes[ idx++ ] = env->FindClass( "org.iso.mpeg.scene.SFRotationFieldValue" );
	classTypes[ idx++ ] = env->FindClass( "org.iso.mpeg.scene.SFStringFieldValue" );
	classTypes[ idx++ ] = env->FindClass( "org.iso.mpeg.scene.SFTimeFieldValue" );
	classTypes[ idx++ ] = env->FindClass( "org.iso.mpeg.scene.SFVec2fFieldValue" );
	classTypes[ idx++ ] = env->FindClass( "org.iso.mpeg.scene.SFVef3fFieldValue" );
}

int FieldValueMaker::getFieldValueType( JNIEnv *env, jobject val )
{
	int idx = -1;

	for ( int i = 0; i < MPJ_FIELDVALUE_TYPE_COUNT; i++ )
	{
		if ( env->IsInstanceOf(val,classTypes[i]) == JNI_TRUE )
		{
			idx = i;
			break;
		}
	}
	return idx;
}

FieldValuePtr FieldValueMaker::doCreateFieldValue( JNIEnv *env, jobject val )
{
	FieldValuePtr f = 0;
	int myType = getFieldValueType( env, val );
	if ( myType >= 0 )
	{
		f = new FieldValue();
		f->fieldValueType = myType;
		f->fieldValueCount = 1;
	}
	jmethodID get;
	jfloatArray floatArray;
	jboolean isCopy;

	switch( myType )
	{
		case MPJ_SFBOOL_TYPE:
			get = env->GetMethodID( classTypes[myType], "getSFBoolValue", "()Z" );
			f->val.boolVal = env->CallBooleanMethod(val, get) ? true : false;
			break;

		case MPJ_SFFLOAT_TYPE:
			get = env->GetMethodID( classTypes[myType], "getSFFloatValue", "()F" );
			f->val.floatVal = env->CallFloatMethod( val, get );
			break;

		case MPJ_SFINT32_TYPE:
			get = env->GetMethodID( classTypes[myType], "getSFInt32Value", "()I" );
			f->val.int32Val = env->CallIntMethod( val, get );
			break;
			
		case MPJ_SFTIME_TYPE:
			get = env->GetMethodID( classTypes[myType], "getSFTimeValue", "()D" );
			f->val.timeVal = env->CallDoubleMethod( val, get );
			break;
			
		case MPJ_SFCOLOR_TYPE:
			get = env->GetMethodID( classTypes[myType], "getSFColorValue", "()[F" );
			floatArray = (jfloatArray) env->CallObjectMethod( val, get );
			env->GetFloatArrayRegion( floatArray, 0, 3, f->val.colorVal );
			break;

		case MPJ_SFVEC3F_TYPE:
			get = env->GetMethodID( classTypes[myType], "getSFVec3fValue", "()[F" );
			floatArray = (jfloatArray) env->CallObjectMethod( val, get );
			env->GetFloatArrayRegion( floatArray, 0, 3, f->val.vec3fVal );
			break;

		case MPJ_SFVEC2F_TYPE:
			get = env->GetMethodID( classTypes[myType], "getSFVec2fValue", "()[F" );
			floatArray = (jfloatArray) env->CallObjectMethod( val, get );
			env->GetFloatArrayRegion( floatArray, 0, 2, f->val.vec2fVal );
			break;

		case MPJ_SFROTATION_TYPE:
			get = env->GetMethodID( classTypes[myType], "getSFRotationValue", "()[F" );
			floatArray = (jfloatArray) env->CallObjectMethod( val, get );
			env->GetFloatArrayRegion( floatArray, 0, 4, f->val.rotationVal );
			break;

		case MPJ_SFSTRING_TYPE:
			get = env->GetMethodID( classTypes[myType], "getSFStringValue", "()java/lang/String;" );
			jstring js = (jstring) env->CallObjectMethod( val, get );
			const char *s = env->GetStringUTFChars( js, &isCopy );
			f->val.stringVal = new char[strlen(s)+1];
			strcpy( f->val.stringVal, s );
			if ( isCopy == JNI_TRUE )
				env->ReleaseStringUTFChars( js, s );
			break;

	}
	return f;
}
