#ifndef INCLUDED_IM1_INTERFACE_H
#define INCLUDED_IM1_INTERFACE_H

#ifdef MPEGJDLLAPI
#define MPJ_EXTERN(v) extern __declspec( dllexport ) v __cdecl
#else
#define MPJ_EXTERN(v) extern __declspec( dllimport ) v __cdecl
#endif

#ifdef __cplusplus
extern "C" {
#endif

typedef int MPJErr;

enum
{
	MPJ_SFBOOL_TYPE = 0,
	MPJ_SFCOLOR_TYPE,
	MPJ_SFFLOAT_TYPE,
	MPJ_SFINT32_TYPE,
	MPJ_SFROTATION_TYPE,
	MPJ_SFSTRING_TYPE,
	MPJ_SFTIME_TYPE,
	MPJ_SFVEC2F_TYPE,
	MPJ_SFVEC3F_TYPE,
	MPJ_FIELDVALUE_TYPE_COUNT
};

typedef struct FieldValueStruct
{
	int fieldValueType;
	int fieldValueCount;
	union
	{
		bool boolVal;
		int int32Val;
		float floatVal;
		float colorVal[3];
		float rotationVal[4];
		float vec2fVal[2];
		float vec3fVal[3];
		double timeVal;
		char *stringVal;
		struct FieldValueStruct *mfValue;
	} val;
} FieldValue, *FieldValuePtr;

enum
{
	MPJNoErr = 0,
	MPJUnspecifiedErr
};

struct IM1DispatchTable
{
	MPJErr (*getSceneHandle)( int *outSceneHandle );
	MPJErr (*findNode)( int nodeID, int *outNodeHandle );
	MPJErr (*sendEventIn)( int nodeHandle, int eventInID, FieldValuePtr fieldValue );
	MPJErr (*readEventOut)( int nodeHandle, int eventOutID, FieldValuePtr outFieldValue );
};

typedef IM1DispatchTable* IM1DispatchTablePtr;

MPJ_EXTERN( void )
MPJSetSceneDispatchTable( IM1DispatchTablePtr table );

MPJ_EXTERN( IM1DispatchTablePtr )
MPJGetSceneDispatchTable();

#ifdef __cplusplus
}
#endif

#endif
