// This software module was originally developed by Wenlong Dong(Rockwell 
// Science Center) and Wei Wu(Rockwell Science Center) in the course of 
// development of the MPEG-4 Systems standard
// (ISO/IEC 14496-1).  This software module is an implementation of a part
// of one or more MPEG-4 Systems tools as specified by the MPEG-4 Systems
// standard.  ISO/IEC gives users of the MPEG-4 Systems standard free
// license to this software module or modifications thereof for use in
// hardware or software products claiming conformance to the MPEG-4 Systems
// standard.  Those intending to use this software module in hardware or
// software products are advised that its use may infringe existing
// patents.  The original developer of this software module and his/her
// company, the subsequent editors and their companies, and ISO/IEC have no
// liability for use of this software module or modifications thereof in an
// implementation.  Copyright is not released for non MPEG-4 Systems
// conforming products.  The original developer retains full right to use
// the code for his/her own purpose, assign or donate the code to a third
// party and to inhibit third parties from using the code for non MPEG-4
// Systems conforming products.  This copyright notice must be included in
// all copies or derivative works.
//
// For more information or to receive an up to date version of this module,
// you may call Wenlong Dong at wdong@usc.edu or Wei Wu at 
// weiwu@jupiter.risc.rockwell.com.
//
// Copyright 1998.

/*-------------------------------------------------------------------------
Project Im1Player
INTERNATIONAL ORGANIZATION FOR STANDARDIZATION
ISO/IEC JTC1/SC29/WG11

SUBSYSTEM:	Script Node
FILE:       VSEvents.h
AUTHOR:     Wenlong Dong (Rockwell Science Center)
			Wei Wu		 (Rockwell Science Center)	
		    Yuval Fisher (Rockwell Science Center)
OVERVIEW:   Interface for the Derived classes: Field, EventIn, EventOut
			of the class VSNodeField.
--------------------------------------------------------------------*/
#ifndef _VSEvents_h
#define _VSEvents_h

#include "VSNodeField.h"

#ifndef BIFSENC
class DecScript;
class VsField;
class VsFunctionDef;
class VsFunctionCall;
#endif

class Field : public VSNodeField
{
public:
	Field() { hasDefault = FALSE; flags.eType = eFIELD;}
	virtual ~Field(){}

#ifdef BIFSENC
	virtual BOOL Encode(BIFSEncoder *pBifs);
	void Encode_Default(BIFSEncoder *pBifs);

#else

	virtual BOOL Parse(BIFSDecoder *pBifs);
#endif

private:

	BOOL hasDefault;
};

class EventOut : public VSNodeField
{
public:

	EventOut();
	virtual ~EventOut();

#ifndef BIFSENC
public:

	virtual void OnFieldUpdate(BOOL Onflag = FALSE);
	virtual BOOL Parse(BIFSDecoder *pBifs);
	NodeField *GetField() { return pValue; }
	virtual operator NodeField *() { return pValueCopy; }
private:

	NodeField *pValueCopy;

#endif
};

class EventIn : public VSNodeField
{
public:

	EventIn();
	virtual ~EventIn(){}

#ifndef BIFSENC

public:
	
	virtual void InitField(DecScript *p);
	virtual void OnFieldUpdate(BOOL Onflag = FALSE);

private:

	VsField *eventArg, *timeArg;
	VsFunctionDef *function;
	VSNodeField *timeStamp;
	VsFunctionCall *f_call;
	
#endif
};

#endif  /*  _VSEvents_h  */