// This software module was originally developed by Wei Wu(Rockwell Science
// Center) and Wenlong Dong(Rockwell Science Center) in the course of 
// development of the MPEG-4 Systems standard
// (ISO/IEC 14496-1).  This software module is an implementation of a part
// of one or more MPEG-4 Systems tools as specified by the MPEG-4 Systems
// standard.  ISO/IEC gives users of the MPEG-4 Systems standard free
// license to this software module or modifications thereof for use in
// hardware or software products claiming conformance to the MPEG-4 Systems
// standard.  Those intending to use this software module in hardware or
// software products are advised that its use may infringe existing
// patents.  The original developer of this software module and his/her
// company, the subsequent editors and their companies, and ISO/IEC have no
// liability for use of this software module or modifications thereof in an
// implementation.  Copyright is not released for non MPEG-4 Systems
// conforming products.  The original developer retains full right to use
// the code for his/her own purpose, assign or donate the code to a third
// party and to inhibit third parties from using the code for non MPEG-4
// Systems conforming products.  This copyright notice must be included in
// all copies or derivative works.
//
// For more information or to receive an up to date version of this module,
// you may call Wei Wu at weiwu@jupiter.risc.rockwell.com or Wenlong Dong 
// at wdong@usc.edu.
//
// Copyright 1998.

/*-------------------------------------------------------------------------
Project Im1Player
INTERNATIONAL ORGANIZATION FOR STANDARDIZATION
ISO/IEC JTC1/SC29/WG11

SUBSYSTEM:	Script Node
FILE: 	    DecScript.h
AUTHOR:     Wei Wu       (Rockwell Science Center)
		    Wenlong Dong (Rockwell Science Center)
		    Yuval Fisher (Rockwell Science Center)
OVERVIEW:   The interface of the DecScript class. Two purposes are 
			achieved here:
			(1) BIFS decoding of the a script segment
			(2) Script interpreter construction
----------------------------------------------------------------------*/
#ifndef _DecScript_h_
#define _DecScript_h_

#include "ztl.h"
#include "..\encoder\ScriptDefs.h"
#include "..\encoder\StrBuffer.h"

class CStrBuffer;
class BIFSDecoder;

class VsExpr;
class VsVar;
class VsArrayVar;
class VsFunctionCall;
class VsFunctionDef;
class VsMethodCall;
class VsMemberAccess;
class VsStatement;
class VsExprList;
class VsStatementList;

class DecScript {
public:
  DecScript();
  ~DecScript();
  LPCSTR GetTextString();
  void Decode(BIFSDecoder *pBifs);
  void InsertExID(int num);

  VsFunctionDef *beginFunction(LPCSTR name);
  VsFunctionDef *findFunction(LPCSTR name);
  VsVar *getGlobalVar(LPCSTR name);
  void addGlobalVar(LPCSTR name, VsVar *var);
  void Error(LPCSTR errmsg);

private:

  int numIdentifier;
  int flag;
  int IdCnt;
  CStrBuffer savedStrBuffer;
  ZString outBuff;

  ZArray<VsVar *> globalVars;
  ZArray<LPCSTR> globalNames;
  ZArray<VsFunctionDef *> functions;
  ZArray<VsFunctionCall *> Unimplist;
  ZArray<LPCSTR> UnimpNames; 

  void Arguments(BIFSDecoder *pBifs, VsFunctionDef *func);
  VsStatementList *StatementBlock(BIFSDecoder *pBifs);
  VsStatement *Statement(BIFSDecoder *pBifs);
  VsStatement *ifStatement(BIFSDecoder *pBifs);
  VsStatement *forStatement(BIFSDecoder *pBifs);
  VsStatement *whileStatement(BIFSDecoder *pBifs);
  VsStatement *returnStatement(BIFSDecoder *pBifs);
  VsStatement *breakStatement();
  VsStatement *continueStatement();
  VsExpr *CompoundExpression(BIFSDecoder *pBifs);
  VsExpr *OptionalExpression(BIFSDecoder *pBifs);
  VsExpr *Expression(BIFSDecoder *pBifs, BOOL n_flag = FALSE);
  VsExpr *NewObject(BIFSDecoder *pBifs);
  VsExpr *ArrayDeref(BIFSDecoder *pBifs);
  VsExpr *FunctionCall(BIFSDecoder *pBifs);
  VsExpr *ObjectMemberAccess(BIFSDecoder *pBifs);
  VsExpr *ObjectMethodCall(BIFSDecoder *pBifs);
  VsExprList *Params(BIFSDecoder *pBifs);

  LPCSTR GetIdentifier(BIFSDecoder *pBifs);
  LPCSTR GetNumber(BIFSDecoder *pBifs);
  LPCSTR GetString(BIFSDecoder *pBifs);

  void CheckUnimp();

  static char* binaryOp[];
  static char* assignmentOp[];
  static char* unaryOp[];
};

#endif // _DecScript_h_




