/*
 * VsFields.h
 *
 *      Declaration of classes:
 *	  VsField
 *	  VsSFInt   (takes care of SFInt32 and SFBool)
 *        VsSFFloat (takes care of SFFloat and SFTime)
 *	  VsSFFloatArray (takes care of SFVec2F, SFVec3F, SFColor, SFRotation)
 *	  VsMFInt   (takes care of MFInt32)
 *	  VsMFFloat   (takes care of MFFloat)
 *	  VsSFString
 *	  VsMFString
 *	  VsSFVoid
 *	  VsMFVoid
 *
 * Subclasses of VsVar that take care of interfacing with VRML
 * fields/eventOuts.
 * 
 * Copyright (C) 1996, Silicon Graphics,  Inc.
 */

/*---------------------------------------------------------------
This module is copied from the VRML Script interpreter owned by
Silicon Graphics, Inc. It is modified by Wenlong Dong, Wei Wu and
Yuval Fisher(Rockwell Science Center) in the course of development
of the MPEG-4 Systems standard(ISO/IEC 14496-1) and added to the 
subsystem: Script Node.
---------------------------------------------------------------*/

#ifndef _VS_VAR_
#define _VS_VAR_

#include "vsexpr.h"
#include "FldTypes.h"

class VSNodeField;

//
// Classes for fields/eventOuts of various sorts:
//

// Base class

class VsField : public VsVar 
{
public:

    enum Permission 
	{
		RW, // For eventOuts/fields
        RO, // For eventIns
    };

    // Creation method, given a field type Does The Right
    // Thing.
    static VsField *create(FIELD_TYPE t, Permission p);
    virtual void	setField(VSNodeField *f) = 0;
	VsValue GetValue() { return myValue; } // NODEFIELD

protected:

    Permission permission;
	FIELD_TYPE type;
	VSNodeField *myField;
	VsValue myValue;
};


class VsSFInt : public VsField 
{
public:
    VsSFInt(FIELD_TYPE t);

    virtual VsValue		evaluate(VsFunctionCall *);
    virtual VsValue		assign(const VsValue &, VsFunctionCall *);
    virtual void  		setField(VSNodeField *f);

protected:

    virtual ~VsSFInt();
};

class VsSFFloat : public VsField 
{
public:
    VsSFFloat(FIELD_TYPE t);

    virtual VsValue		evaluate(VsFunctionCall *);
    virtual VsValue		assign(const VsValue &, VsFunctionCall *);

    virtual void 		setField(VSNodeField *f);
  
protected:

    virtual ~VsSFFloat();
};

class VsSFFloatArray : public VsField 
{
public:
    VsSFFloatArray(FIELD_TYPE t);

    virtual VsValue		evaluate(VsFunctionCall *);
    virtual VsValue		assign(const VsValue &, VsFunctionCall *);
    virtual VsValue		assign(const VsValue &, int32_t index, VsFunctionCall *);
    virtual void 		setField(VSNodeField *f);
  
protected:
    virtual ~VsSFFloatArray();
};

class VsSFVoid : public VsField 
{
public:
    VsSFVoid(FIELD_TYPE t);
    virtual ~VsSFVoid();

    virtual VsValue		evaluate(VsFunctionCall *);
    virtual VsValue		assign(const VsValue &, VsFunctionCall *);

    virtual void 		setField(VSNodeField *f);
};

class VsSFString : public VsField 
{
public:
    VsSFString(FIELD_TYPE t);
    virtual ~VsSFString();

    virtual VsValue		evaluate(VsFunctionCall *);
    virtual VsValue		assign(const VsValue &, VsFunctionCall *);

    virtual void 		setField(VSNodeField *f);
};

class VsMFInt : public VsField 
{
public:
    VsMFInt(FIELD_TYPE t);

    virtual VsValue		evaluate(VsFunctionCall *);
    virtual VsValue		assign(const VsValue &, VsFunctionCall *);
    virtual VsValue		assign(const VsValue &, int32_t index, VsFunctionCall *);

    virtual void  		setField(VSNodeField *f);

protected:
    virtual ~VsMFInt();
};

class VsMFFloat : public VsField 
{
public:
    VsMFFloat(FIELD_TYPE t);

    virtual VsValue		evaluate(VsFunctionCall *);
    virtual VsValue		assign(const VsValue &, VsFunctionCall *);
    virtual VsValue		assign(const VsValue &, int32_t index, VsFunctionCall *);

    virtual void  		setField(VSNodeField *f);

protected:
    virtual ~VsMFFloat();
};

class VsMFVoid : public VsField 
{
public:
    VsMFVoid(FIELD_TYPE t);

    virtual VsValue		evaluate(VsFunctionCall *);
    virtual VsValue		assign(const VsValue &, VsFunctionCall *);
    virtual VsValue		assign(const VsValue &, int32_t index, VsFunctionCall *);

    virtual void  		setField(VSNodeField *f);

protected:
    virtual ~VsMFVoid();
};

class VsMFString : public VsField 
{
public:
    VsMFString(FIELD_TYPE t);

    virtual VsValue		evaluate(VsFunctionCall *);
    virtual VsValue		assign(const VsValue &, VsFunctionCall *);
    virtual VsValue		assign(const VsValue &, int32_t index, VsFunctionCall *);

    virtual void  		setField(VSNodeField *f);

protected:
    virtual ~VsMFString();
};

#endif /* _VS_VAR_ */
