/***********************************************************************
This software module was originally developed by Zvi Lifshitz
(Triton R&D Ltd.) in the course of development of the MPEG-4 Systems
(ISO/IEC 14496-1) standard. This software module is an implementation of
a part of one or more MPEG-4 Systems (ISO/IEC 14496-1) tools as
specified by the MPEG-4 Systems (ISO/IEC 14496-1) standard. ISO/IEC
gives users of the MPEG-4 Systems (ISO/IEC 14496-1) free license to
this software module or modifications thereof for use in hardware or
software products claiming conformance to the MPEG-4 Systems (ISO/IEC
14496-1). Those intending to use this software module in hardware or
software products are advised that its use may infringe existing patents.
The original developer of this software module and his/her company, the
subsequent editors and their companies, and ISO/IEC have no liability
for use of this software module or modifications thereof in an
implementation. Copyright is not released for non MPEG-4 Systems
(ISO/IEC 14496-1) conforming products. Triton R&D Ltd. retains full right
to use the code for his/her own purpose, assign or donate the code to a
third party and to inhibit third parties from using the code for non
MPEG-4 Systems (ISO/IEC 14496-1) conforming products. This copyright
notice must be included in all copies or derivative works.
Copyright  1999.
************************************************************************/
package org.iso.mpeg.mpegj.scene;

/**
 * An interface with inner classes defining the constants for the field
 * defIDs of each node.
 */
public interface Field {

    /**
     * An interface defining constants for the field defIDs of the Anchor node.
     */
    public interface Anchor {
        /**
         * The defID for the children field.
         */
        public static final int children = 2;
        /**
         * The defID for the description field.
         */
        public static final int description = 3;
        /**
         * The defID for the parameter field.
         */
        public static final int parameter = 4;
        /**
         * The defID for the url field.
         */
        public static final int url = 5;
    }

    /**
     * An interface defining constants for the field defIDs of the AnimationStream node.
     */
    public interface AnimationStream {
        /**
         * The defID for the loop field.
         */
        public static final int loop = 0;
        /**
         * The defID for the speed field.
         */
        public static final int speed = 1;
        /**
         * The defID for the startTime field.
         */
        public static final int startTime = 2;
        /**
         * The defID for the stopTime field.
         */
        public static final int stopTime = 3;
        /**
         * The defID for the url field.
         */
        public static final int url = 4;
    }

    /**
     * An interface defining constants for the field defIDs of the Appearance node.
     */
    public interface Appearance {
        /**
         * The defID for the material field.
         */
        public static final int material = 0;
        /**
         * The defID for the texture field.
         */
        public static final int texture = 1;
        /**
         * The defID for the textureTransform field.
         */
        public static final int textureTransform = 2;
    }

    /**
     * An interface defining constants for the field defIDs of the AudioBuffer node.
     */
    public interface AudioBuffer {
        /**
         * The defID for the loop field.
         */
        public static final int loop = 0;
        /**
         * The defID for the pitch field.
         */
        public static final int pitch = 1;
        /**
         * The defID for the startTime field.
         */
        public static final int startTime = 2;
        /**
         * The defID for the stopTime field.
         */
        public static final int stopTime = 3;
        /**
         * The defID for the children field.
         */
        public static final int children = 4;
        /**
         * The defID for the numChan field.
         */
        public static final int numChan = 5;
        /**
         * The defID for the phaseGroup field.
         */
        public static final int phaseGroup = 6;
        /**
         * The defID for the length field.
         */
        public static final int length = 7;
    }

    /**
     * An interface defining constants for the field defIDs of the AudioClip node.
     */
    public interface AudioClip {
        /**
         * The defID for the description field.
         */
        public static final int description = 0;
        /**
         * The defID for the loop field.
         */
        public static final int loop = 1;
        /**
         * The defID for the pitch field.
         */
        public static final int pitch = 2;
        /**
         * The defID for the startTime field.
         */
        public static final int startTime = 3;
        /**
         * The defID for the stopTime field.
         */
        public static final int stopTime = 4;
        /**
         * The defID for the url field.
         */
        public static final int url = 5;
    }

    /**
     * An interface defining constants for the field defIDs of the AudioDelay node.
     */
    public interface AudioDelay {
        /**
         * The defID for the children field.
         */
        public static final int children = 2;
        /**
         * The defID for the delay field.
         */
        public static final int delay = 3;
        /**
         * The defID for the numChan field.
         */
        public static final int numChan = 4;
        /**
         * The defID for the phaseGroup field.
         */
        public static final int phaseGroup = 5;
    }

    /**
     * An interface defining constants for the field defIDs of the AudioFX node.
     */
    public interface AudioFX {
        /**
         * The defID for the children field.
         */
        public static final int children = 2;
        /**
         * The defID for the orch field.
         */
        public static final int orch = 3;
        /**
         * The defID for the score field.
         */
        public static final int score = 4;
        /**
         * The defID for the params field.
         */
        public static final int params = 5;
        /**
         * The defID for the numChan field.
         */
        public static final int numChan = 6;
        /**
         * The defID for the phaseGroup field.
         */
        public static final int phaseGroup = 7;
    }

    /**
     * An interface defining constants for the field defIDs of the AudioMix node.
     */
    public interface AudioMix {
        /**
         * The defID for the children field.
         */
        public static final int children = 2;
        /**
         * The defID for the numInputs field.
         */
        public static final int numInputs = 3;
        /**
         * The defID for the matrix field.
         */
        public static final int matrix = 4;
        /**
         * The defID for the numChan field.
         */
        public static final int numChan = 5;
        /**
         * The defID for the phaseGroup field.
         */
        public static final int phaseGroup = 6;
    }

    /**
     * An interface defining constants for the field defIDs of the AudioSource node.
     */
    public interface AudioSource {
        /**
         * The defID for the children field.
         */
        public static final int children = 2;
        /**
         * The defID for the url field.
         */
        public static final int url = 3;
        /**
         * The defID for the pitch field.
         */
        public static final int pitch = 4;
        /**
         * The defID for the speed field.
         */
        public static final int speed = 5;
        /**
         * The defID for the startTime field.
         */
        public static final int startTime = 6;
        /**
         * The defID for the stopTime field.
         */
        public static final int stopTime = 7;
        /**
         * The defID for the numChan field.
         */
        public static final int numChan = 8;
        /**
         * The defID for the phaseGroup field.
         */
        public static final int phaseGroup = 9;
    }

    /**
     * An interface defining constants for the field defIDs of the AudioSwitch node.
     */
    public interface AudioSwitch {
        /**
         * The defID for the children field.
         */
        public static final int children = 2;
        /**
         * The defID for the whichChoice field.
         */
        public static final int whichChoice = 3;
        /**
         * The defID for the numChan field.
         */
        public static final int numChan = 4;
        /**
         * The defID for the phaseGroup field.
         */
        public static final int phaseGroup = 5;
    }

    /**
     * An interface defining constants for the field defIDs of the Background node.
     */
    public interface Background {
        /**
         * The defID for the groundAngle field.
         */
        public static final int groundAngle = 1;
        /**
         * The defID for the groundColor field.
         */
        public static final int groundColor = 2;
        /**
         * The defID for the backUrl field.
         */
        public static final int backUrl = 3;
        /**
         * The defID for the bottomUrl field.
         */
        public static final int bottomUrl = 4;
        /**
         * The defID for the frontUrl field.
         */
        public static final int frontUrl = 5;
        /**
         * The defID for the leftUrl field.
         */
        public static final int leftUrl = 6;
        /**
         * The defID for the rightUrl field.
         */
        public static final int rightUrl = 7;
        /**
         * The defID for the topUrl field.
         */
        public static final int topUrl = 8;
        /**
         * The defID for the skyAngle field.
         */
        public static final int skyAngle = 9;
        /**
         * The defID for the skyColor field.
         */
        public static final int skyColor = 10;
    }

    /**
     * An interface defining constants for the field defIDs of the Background2D node.
     */
    public interface Background2D {
        /**
         * The defID for the backColor field.
         */
        public static final int backColor = 1;
        /**
         * The defID for the url field.
         */
        public static final int url = 2;
    }

    /**
     * An interface defining constants for the field defIDs of the Billboard node.
     */
    public interface Billboard {
        /**
         * The defID for the children field.
         */
        public static final int children = 2;
        /**
         * The defID for the axisOfRotation field.
         */
        public static final int axisOfRotation = 3;
    }

    /**
     * An interface defining constants for the field defIDs of the Bitmap node.
     */
    public interface Bitmap {
        /**
         * The defID for the scale field.
         */
        public static final int scale = 0;
    }

    /**
     * An interface defining constants for the field defIDs of the Box node.
     */
    public interface Box {
        /**
         * The defID for the size field.
         */
        public static final int size = 0;
    }

    /**
     * An interface defining constants for the field defIDs of the Circle node.
     */
    public interface Circle {
        /**
         * The defID for the radius field.
         */
        public static final int radius = 0;
    }

    /**
     * An interface defining constants for the field defIDs of the Collision node.
     */
    public interface Collision {
        /**
         * The defID for the children field.
         */
        public static final int children = 2;
        /**
         * The defID for the collide field.
         */
        public static final int collide = 3;
        /**
         * The defID for the proxy field.
         */
        public static final int proxy = 4;
    }

    /**
     * An interface defining constants for the field defIDs of the Color node.
     */
    public interface Color {
        /**
         * The defID for the color field.
         */
        public static final int color = 0;
    }

    /**
     * An interface defining constants for the field defIDs of the ColorInterpolator node.
     */
    public interface ColorInterpolator {
        /**
         * The defID for the key field.
         */
        public static final int key = 1;
        /**
         * The defID for the keyValue field.
         */
        public static final int keyValue = 2;
    }

    /**
     * An interface defining constants for the field defIDs of the CompositeTexture2D node.
     */
    public interface CompositeTexture2D {
        /**
         * The defID for the children field.
         */
        public static final int children = 2;
        /**
         * The defID for the pixelWidth field.
         */
        public static final int pixelWidth = 3;
        /**
         * The defID for the pixelHeight field.
         */
        public static final int pixelHeight = 4;
        /**
         * The defID for the background field.
         */
        public static final int background = 5;
        /**
         * The defID for the viewport field.
         */
        public static final int viewport = 6;
    }

    /**
     * An interface defining constants for the field defIDs of the CompositeTexture3D node.
     */
    public interface CompositeTexture3D {
        /**
         * The defID for the children field.
         */
        public static final int children = 2;
        /**
         * The defID for the pixelWidth field.
         */
        public static final int pixelWidth = 3;
        /**
         * The defID for the pixelHeight field.
         */
        public static final int pixelHeight = 4;
        /**
         * The defID for the background field.
         */
        public static final int background = 5;
        /**
         * The defID for the fog field.
         */
        public static final int fog = 6;
        /**
         * The defID for the navigationInfo field.
         */
        public static final int navigationInfo = 7;
        /**
         * The defID for the viewpoint field.
         */
        public static final int viewpoint = 8;
    }

    /**
     * An interface defining constants for the field defIDs of the Conditional node.
     */
    public interface Conditional {
        /**
         * The defID for the buffer field.
         */
        public static final int buffer = 2;
    }

    /**
     * An interface defining constants for the field defIDs of the Cone node.
     */
    public interface Cone {
        /**
         * The defID for the bottomRadius field.
         */
        public static final int bottomRadius = 0;
        /**
         * The defID for the height field.
         */
        public static final int height = 1;
        /**
         * The defID for the side field.
         */
        public static final int side = 2;
        /**
         * The defID for the bottom field.
         */
        public static final int bottom = 3;
    }

    /**
     * An interface defining constants for the field defIDs of the Coordinate node.
     */
    public interface Coordinate {
        /**
         * The defID for the point field.
         */
        public static final int point = 0;
    }

    /**
     * An interface defining constants for the field defIDs of the Coordinate2D node.
     */
    public interface Coordinate2D {
        /**
         * The defID for the point field.
         */
        public static final int point = 0;
    }

    /**
     * An interface defining constants for the field defIDs of the CoordinateInterpolator node.
     */
    public interface CoordinateInterpolator {
        /**
         * The defID for the key field.
         */
        public static final int key = 1;
        /**
         * The defID for the keyValue field.
         */
        public static final int keyValue = 2;
    }

    /**
     * An interface defining constants for the field defIDs of the CoordinateInterpolator2D node.
     */
    public interface CoordinateInterpolator2D {
        /**
         * The defID for the key field.
         */
        public static final int key = 1;
        /**
         * The defID for the keyValue field.
         */
        public static final int keyValue = 2;
    }

    /**
     * An interface defining constants for the field defIDs of the Curve2D node.
     */
    public interface Curve2D {
        /**
         * The defID for the point field.
         */
        public static final int point = 0;
        /**
         * The defID for the fineness field.
         */
        public static final int fineness = 1;
        /**
         * The defID for the type field.
         */
        public static final int type = 2;
    }

    /**
     * An interface defining constants for the field defIDs of the Cylinder node.
     */
    public interface Cylinder {
        /**
         * The defID for the bottom field.
         */
        public static final int bottom = 0;
        /**
         * The defID for the height field.
         */
        public static final int height = 1;
        /**
         * The defID for the radius field.
         */
        public static final int radius = 2;
        /**
         * The defID for the side field.
         */
        public static final int side = 3;
        /**
         * The defID for the top field.
         */
        public static final int top = 4;
    }

    /**
     * An interface defining constants for the field defIDs of the CylinderSensor node.
     */
    public interface CylinderSensor {
        /**
         * The defID for the autoOffset field.
         */
        public static final int autoOffset = 0;
        /**
         * The defID for the diskAngle field.
         */
        public static final int diskAngle = 1;
        /**
         * The defID for the enabled field.
         */
        public static final int enabled = 2;
        /**
         * The defID for the maxAngle field.
         */
        public static final int maxAngle = 3;
        /**
         * The defID for the minAngle field.
         */
        public static final int minAngle = 4;
        /**
         * The defID for the offset field.
         */
        public static final int offset = 5;
    }

    /**
     * An interface defining constants for the field defIDs of the DirectionalLight node.
     */
    public interface DirectionalLight {
        /**
         * The defID for the ambientIntensity field.
         */
        public static final int ambientIntensity = 0;
        /**
         * The defID for the color field.
         */
        public static final int color = 1;
        /**
         * The defID for the direction field.
         */
        public static final int direction = 2;
        /**
         * The defID for the intensity field.
         */
        public static final int intensity = 3;
        /**
         * The defID for the on field.
         */
        public static final int on = 4;
    }

    /**
     * An interface defining constants for the field defIDs of the DiscSensor node.
     */
    public interface DiscSensor {
        /**
         * The defID for the autoOffset field.
         */
        public static final int autoOffset = 0;
        /**
         * The defID for the enabled field.
         */
        public static final int enabled = 1;
        /**
         * The defID for the maxAngle field.
         */
        public static final int maxAngle = 2;
        /**
         * The defID for the minAngle field.
         */
        public static final int minAngle = 3;
        /**
         * The defID for the offset field.
         */
        public static final int offset = 4;
    }

    /**
     * An interface defining constants for the field defIDs of the ElevationGrid node.
     */
    public interface ElevationGrid {
        /**
         * The defID for the color field.
         */
        public static final int color = 1;
        /**
         * The defID for the normal field.
         */
        public static final int normal = 2;
        /**
         * The defID for the texCoord field.
         */
        public static final int texCoord = 3;
        /**
         * The defID for the height field.
         */
        public static final int height = 4;
        /**
         * The defID for the ccw field.
         */
        public static final int ccw = 5;
        /**
         * The defID for the colorPerVertex field.
         */
        public static final int colorPerVertex = 6;
        /**
         * The defID for the creaseAngle field.
         */
        public static final int creaseAngle = 7;
        /**
         * The defID for the normalPerVertex field.
         */
        public static final int normalPerVertex = 8;
        /**
         * The defID for the solid field.
         */
        public static final int solid = 9;
        /**
         * The defID for the xDimension field.
         */
        public static final int xDimension = 10;
        /**
         * The defID for the xSpacing field.
         */
        public static final int xSpacing = 11;
        /**
         * The defID for the zDimension field.
         */
        public static final int zDimension = 12;
        /**
         * The defID for the zSpacing field.
         */
        public static final int zSpacing = 13;
    }

    /**
     * An interface defining constants for the field defIDs of the Expression node.
     */
    public interface Expression {
        /**
         * The defID for the expression_select1 field.
         */
        public static final int expression_select1 = 0;
        /**
         * The defID for the expression_intensity1 field.
         */
        public static final int expression_intensity1 = 1;
        /**
         * The defID for the expression_select2 field.
         */
        public static final int expression_select2 = 2;
        /**
         * The defID for the expression_intensity2 field.
         */
        public static final int expression_intensity2 = 3;
        /**
         * The defID for the init_face field.
         */
        public static final int init_face = 4;
        /**
         * The defID for the expression_def field.
         */
        public static final int expression_def = 5;
    }

    /**
     * An interface defining constants for the field defIDs of the Extrusion node.
     */
    public interface Extrusion {
        /**
         * The defID for the beginCap field.
         */
        public static final int beginCap = 4;
        /**
         * The defID for the ccw field.
         */
        public static final int ccw = 5;
        /**
         * The defID for the convex field.
         */
        public static final int convex = 6;
        /**
         * The defID for the creaseAngle field.
         */
        public static final int creaseAngle = 7;
        /**
         * The defID for the crossSection field.
         */
        public static final int crossSection = 8;
        /**
         * The defID for the endCap field.
         */
        public static final int endCap = 9;
        /**
         * The defID for the orientation field.
         */
        public static final int orientation = 10;
        /**
         * The defID for the scale field.
         */
        public static final int scale = 11;
        /**
         * The defID for the solid field.
         */
        public static final int solid = 12;
        /**
         * The defID for the spine field.
         */
        public static final int spine = 13;
    }

    /**
     * An interface defining constants for the field defIDs of the Face node.
     */
    public interface Face {
        /**
         * The defID for the fap field.
         */
        public static final int fap = 0;
        /**
         * The defID for the fdp field.
         */
        public static final int fdp = 1;
        /**
         * The defID for the fit field.
         */
        public static final int fit = 2;
        /**
         * The defID for the ttsSource field.
         */
        public static final int ttsSource = 3;
        /**
         * The defID for the renderedFace field.
         */
        public static final int renderedFace = 4;
    }

    /**
     * An interface defining constants for the field defIDs of the FaceDefMesh node.
     */
    public interface FaceDefMesh {
        /**
         * The defID for the faceSceneGraphNode field.
         */
        public static final int faceSceneGraphNode = 0;
        /**
         * The defID for the intervalBorders field.
         */
        public static final int intervalBorders = 1;
        /**
         * The defID for the coordIndex field.
         */
        public static final int coordIndex = 2;
        /**
         * The defID for the displacements field.
         */
        public static final int displacements = 3;
    }

    /**
     * An interface defining constants for the field defIDs of the FaceDefTables node.
     */
    public interface FaceDefTables {
        /**
         * The defID for the fapID field.
         */
        public static final int fapID = 0;
        /**
         * The defID for the highLevelSelect field.
         */
        public static final int highLevelSelect = 1;
        /**
         * The defID for the faceDefMesh field.
         */
        public static final int faceDefMesh = 2;
        /**
         * The defID for the faceDefTransform field.
         */
        public static final int faceDefTransform = 3;
    }

    /**
     * An interface defining constants for the field defIDs of the FaceDefTransform node.
     */
    public interface FaceDefTransform {
        /**
         * The defID for the faceSceneGraphNode field.
         */
        public static final int faceSceneGraphNode = 0;
        /**
         * The defID for the fieldId field.
         */
        public static final int fieldId = 1;
        /**
         * The defID for the rotationDef field.
         */
        public static final int rotationDef = 2;
        /**
         * The defID for the scaleDef field.
         */
        public static final int scaleDef = 3;
        /**
         * The defID for the translationDef field.
         */
        public static final int translationDef = 4;
    }

    /**
     * An interface defining constants for the field defIDs of the FAP node.
     */
    public interface FAP {
        /**
         * The defID for the viseme field.
         */
        public static final int viseme = 0;
        /**
         * The defID for the expression field.
         */
        public static final int expression = 1;
        /**
         * The defID for the open_jaw field.
         */
        public static final int open_jaw = 2;
        /**
         * The defID for the lower_t_midlip field.
         */
        public static final int lower_t_midlip = 3;
        /**
         * The defID for the raise_b_midlip field.
         */
        public static final int raise_b_midlip = 4;
        /**
         * The defID for the stretch_l_corner field.
         */
        public static final int stretch_l_corner = 5;
        /**
         * The defID for the stretch_r_corner field.
         */
        public static final int stretch_r_corner = 6;
        /**
         * The defID for the lower_t_lip_lm field.
         */
        public static final int lower_t_lip_lm = 7;
        /**
         * The defID for the lower_t_lip_rm field.
         */
        public static final int lower_t_lip_rm = 8;
        /**
         * The defID for the lower_b_lip_lm field.
         */
        public static final int lower_b_lip_lm = 9;
        /**
         * The defID for the lower_b_lip_rm field.
         */
        public static final int lower_b_lip_rm = 10;
        /**
         * The defID for the raise_l_cornerlip field.
         */
        public static final int raise_l_cornerlip = 11;
        /**
         * The defID for the raise_r_cornerlip field.
         */
        public static final int raise_r_cornerlip = 12;
        /**
         * The defID for the thrust_jaw field.
         */
        public static final int thrust_jaw = 13;
        /**
         * The defID for the shift_jaw field.
         */
        public static final int shift_jaw = 14;
        /**
         * The defID for the push_b_lip field.
         */
        public static final int push_b_lip = 15;
        /**
         * The defID for the push_t_lip field.
         */
        public static final int push_t_lip = 16;
        /**
         * The defID for the depress_chin field.
         */
        public static final int depress_chin = 17;
        /**
         * The defID for the close_t_l_eyelid field.
         */
        public static final int close_t_l_eyelid = 18;
        /**
         * The defID for the close_t_r_eyelid field.
         */
        public static final int close_t_r_eyelid = 19;
        /**
         * The defID for the close_b_l_eyelid field.
         */
        public static final int close_b_l_eyelid = 20;
        /**
         * The defID for the close_b_r_eyelid field.
         */
        public static final int close_b_r_eyelid = 21;
        /**
         * The defID for the yaw_l_eyeball field.
         */
        public static final int yaw_l_eyeball = 22;
        /**
         * The defID for the yaw_r_eyeball field.
         */
        public static final int yaw_r_eyeball = 23;
        /**
         * The defID for the pitch_l_eyeball field.
         */
        public static final int pitch_l_eyeball = 24;
        /**
         * The defID for the pitch_r_eyeball field.
         */
        public static final int pitch_r_eyeball = 25;
        /**
         * The defID for the thrust_l_eyeball field.
         */
        public static final int thrust_l_eyeball = 26;
        /**
         * The defID for the thrust_r_eyeball field.
         */
        public static final int thrust_r_eyeball = 27;
        /**
         * The defID for the dilate_l_pupil field.
         */
        public static final int dilate_l_pupil = 28;
        /**
         * The defID for the dilate_r_pupil field.
         */
        public static final int dilate_r_pupil = 29;
        /**
         * The defID for the raise_l_i_eyebrow field.
         */
        public static final int raise_l_i_eyebrow = 30;
        /**
         * The defID for the raise_r_i_eyebrow field.
         */
        public static final int raise_r_i_eyebrow = 31;
        /**
         * The defID for the raise_l_m_eyebrow field.
         */
        public static final int raise_l_m_eyebrow = 32;
        /**
         * The defID for the raise_r_m_eyebrow field.
         */
        public static final int raise_r_m_eyebrow = 33;
        /**
         * The defID for the raise_l_o_eyebrow field.
         */
        public static final int raise_l_o_eyebrow = 34;
        /**
         * The defID for the raise_r_o_eyebrow field.
         */
        public static final int raise_r_o_eyebrow = 35;
        /**
         * The defID for the squeeze_l_eyebrow field.
         */
        public static final int squeeze_l_eyebrow = 36;
        /**
         * The defID for the squeeze_r_eyebrow field.
         */
        public static final int squeeze_r_eyebrow = 37;
        /**
         * The defID for the puff_l_cheek field.
         */
        public static final int puff_l_cheek = 38;
        /**
         * The defID for the puff_r_cheek field.
         */
        public static final int puff_r_cheek = 39;
        /**
         * The defID for the lift_l_cheek field.
         */
        public static final int lift_l_cheek = 40;
        /**
         * The defID for the lift_r_cheek field.
         */
        public static final int lift_r_cheek = 41;
        /**
         * The defID for the shift_tongue_tip field.
         */
        public static final int shift_tongue_tip = 42;
        /**
         * The defID for the raise_tongue_tip field.
         */
        public static final int raise_tongue_tip = 43;
        /**
         * The defID for the thrust_tongue_tip field.
         */
        public static final int thrust_tongue_tip = 44;
        /**
         * The defID for the raise_tongue field.
         */
        public static final int raise_tongue = 45;
        /**
         * The defID for the tongue_roll field.
         */
        public static final int tongue_roll = 46;
        /**
         * The defID for the head_pitch field.
         */
        public static final int head_pitch = 47;
        /**
         * The defID for the head_yaw field.
         */
        public static final int head_yaw = 48;
        /**
         * The defID for the head_roll field.
         */
        public static final int head_roll = 49;
        /**
         * The defID for the lower_t_midlip_o field.
         */
        public static final int lower_t_midlip_o = 50;
        /**
         * The defID for the raise_b_midlip_o field.
         */
        public static final int raise_b_midlip_o = 51;
        /**
         * The defID for the stretch_l_cornerlip field.
         */
        public static final int stretch_l_cornerlip = 52;
        /**
         * The defID for the stretch_r_cornerlip field.
         */
        public static final int stretch_r_cornerlip = 53;
        /**
         * The defID for the lower_t_lip_lm_o field.
         */
        public static final int lower_t_lip_lm_o = 54;
        /**
         * The defID for the lower_t_lip_rm_o field.
         */
        public static final int lower_t_lip_rm_o = 55;
        /**
         * The defID for the raise_b_lip_lm_o field.
         */
        public static final int raise_b_lip_lm_o = 56;
        /**
         * The defID for the raise_b_lip_rm_o field.
         */
        public static final int raise_b_lip_rm_o = 57;
        /**
         * The defID for the raise_l_cornerlip_o field.
         */
        public static final int raise_l_cornerlip_o = 58;
        /**
         * The defID for the raise_r_cornerlip_o field.
         */
        public static final int raise_r_cornerlip_o = 59;
        /**
         * The defID for the stretch_l_nose field.
         */
        public static final int stretch_l_nose = 60;
        /**
         * The defID for the stretch_r_nose field.
         */
        public static final int stretch_r_nose = 61;
        /**
         * The defID for the raise_nose field.
         */
        public static final int raise_nose = 62;
        /**
         * The defID for the bend_nose field.
         */
        public static final int bend_nose = 63;
        /**
         * The defID for the raise_l_ear field.
         */
        public static final int raise_l_ear = 64;
        /**
         * The defID for the raise_r_ear field.
         */
        public static final int raise_r_ear = 65;
        /**
         * The defID for the pull_l_ear field.
         */
        public static final int pull_l_ear = 66;
        /**
         * The defID for the pull_r_ear field.
         */
        public static final int pull_r_ear = 67;
    }

    /**
     * An interface defining constants for the field defIDs of the FDP node.
     */
    public interface FDP {
        /**
         * The defID for the featurePointsCoord field.
         */
        public static final int featurePointsCoord = 0;
        /**
         * The defID for the textureCoord field.
         */
        public static final int textureCoord = 1;
        /**
         * The defID for the faceDefTables field.
         */
        public static final int faceDefTables = 2;
        /**
         * The defID for the faceSceneGraph field.
         */
        public static final int faceSceneGraph = 3;
        /**
         * The defID for the useOrthoTexture field.
         */
        public static final int useOrthoTexture = 4;
    }

    /**
     * An interface defining constants for the field defIDs of the FIT node.
     */
    public interface FIT {
        /**
         * The defID for the FAPs field.
         */
        public static final int FAPs = 0;
        /**
         * The defID for the Graph field.
         */
        public static final int Graph = 1;
        /**
         * The defID for the numeratorExp field.
         */
        public static final int numeratorExp = 2;
        /**
         * The defID for the denominatorExp field.
         */
        public static final int denominatorExp = 3;
        /**
         * The defID for the numeratorImpulse field.
         */
        public static final int numeratorImpulse = 4;
        /**
         * The defID for the numeratorTerms field.
         */
        public static final int numeratorTerms = 5;
        /**
         * The defID for the denominatorTerms field.
         */
        public static final int denominatorTerms = 6;
        /**
         * The defID for the numeratorCoefs field.
         */
        public static final int numeratorCoefs = 7;
        /**
         * The defID for the denominatorCoefs field.
         */
        public static final int denominatorCoefs = 8;
    }

    /**
     * An interface defining constants for the field defIDs of the Fog node.
     */
    public interface Fog {
        /**
         * The defID for the color field.
         */
        public static final int color = 0;
        /**
         * The defID for the fogType field.
         */
        public static final int fogType = 1;
        /**
         * The defID for the visibilityRange field.
         */
        public static final int visibilityRange = 2;
    }

    /**
     * An interface defining constants for the field defIDs of the FontStyle node.
     */
    public interface FontStyle {
        /**
         * The defID for the family field.
         */
        public static final int family = 0;
        /**
         * The defID for the horizontal field.
         */
        public static final int horizontal = 1;
        /**
         * The defID for the justify field.
         */
        public static final int justify = 2;
        /**
         * The defID for the language field.
         */
        public static final int language = 3;
        /**
         * The defID for the leftToRight field.
         */
        public static final int leftToRight = 4;
        /**
         * The defID for the size field.
         */
        public static final int size = 5;
        /**
         * The defID for the spacing field.
         */
        public static final int spacing = 6;
        /**
         * The defID for the style field.
         */
        public static final int style = 7;
        /**
         * The defID for the topToBottom field.
         */
        public static final int topToBottom = 8;
    }

    /**
     * An interface defining constants for the field defIDs of the Form node.
     */
    public interface Form {
        /**
         * The defID for the children field.
         */
        public static final int children = 2;
        /**
         * The defID for the size field.
         */
        public static final int size = 3;
        /**
         * The defID for the groups field.
         */
        public static final int groups = 4;
        /**
         * The defID for the constraints field.
         */
        public static final int constraints = 5;
        /**
         * The defID for the groupsIndex field.
         */
        public static final int groupsIndex = 6;
    }

    /**
     * An interface defining constants for the field defIDs of the Group node.
     */
    public interface Group {
        /**
         * The defID for the children field.
         */
        public static final int children = 2;
    }

    /**
     * An interface defining constants for the field defIDs of the ImageTexture node.
     */
    public interface ImageTexture {
        /**
         * The defID for the url field.
         */
        public static final int url = 0;
        /**
         * The defID for the repeatS field.
         */
        public static final int repeatS = 1;
        /**
         * The defID for the repeatT field.
         */
        public static final int repeatT = 2;
    }

    /**
     * An interface defining constants for the field defIDs of the IndexedFaceSet node.
     */
    public interface IndexedFaceSet {
        /**
         * The defID for the color field.
         */
        public static final int color = 4;
        /**
         * The defID for the coord field.
         */
        public static final int coord = 5;
        /**
         * The defID for the normal field.
         */
        public static final int normal = 6;
        /**
         * The defID for the texCoord field.
         */
        public static final int texCoord = 7;
        /**
         * The defID for the ccw field.
         */
        public static final int ccw = 8;
        /**
         * The defID for the colorIndex field.
         */
        public static final int colorIndex = 9;
        /**
         * The defID for the colorPerVertex field.
         */
        public static final int colorPerVertex = 10;
        /**
         * The defID for the convex field.
         */
        public static final int convex = 11;
        /**
         * The defID for the coordIndex field.
         */
        public static final int coordIndex = 12;
        /**
         * The defID for the creaseAngle field.
         */
        public static final int creaseAngle = 13;
        /**
         * The defID for the normalIndex field.
         */
        public static final int normalIndex = 14;
        /**
         * The defID for the normalPerVertex field.
         */
        public static final int normalPerVertex = 15;
        /**
         * The defID for the solid field.
         */
        public static final int solid = 16;
        /**
         * The defID for the texCoordIndex field.
         */
        public static final int texCoordIndex = 17;
    }

    /**
     * An interface defining constants for the field defIDs of the IndexedFaceSet2D node.
     */
    public interface IndexedFaceSet2D {
        /**
         * The defID for the color field.
         */
        public static final int color = 3;
        /**
         * The defID for the coord field.
         */
        public static final int coord = 4;
        /**
         * The defID for the texCoord field.
         */
        public static final int texCoord = 5;
        /**
         * The defID for the colorIndex field.
         */
        public static final int colorIndex = 6;
        /**
         * The defID for the colorPerVertex field.
         */
        public static final int colorPerVertex = 7;
        /**
         * The defID for the convex field.
         */
        public static final int convex = 8;
        /**
         * The defID for the coordIndex field.
         */
        public static final int coordIndex = 9;
        /**
         * The defID for the texCoordIndex field.
         */
        public static final int texCoordIndex = 10;
    }

    /**
     * An interface defining constants for the field defIDs of the IndexedLineSet node.
     */
    public interface IndexedLineSet {
        /**
         * The defID for the color field.
         */
        public static final int color = 2;
        /**
         * The defID for the coord field.
         */
        public static final int coord = 3;
        /**
         * The defID for the colorIndex field.
         */
        public static final int colorIndex = 4;
        /**
         * The defID for the colorPerVertex field.
         */
        public static final int colorPerVertex = 5;
        /**
         * The defID for the coordIndex field.
         */
        public static final int coordIndex = 6;
    }

    /**
     * An interface defining constants for the field defIDs of the IndexedLineSet2D node.
     */
    public interface IndexedLineSet2D {
        /**
         * The defID for the color field.
         */
        public static final int color = 2;
        /**
         * The defID for the coord field.
         */
        public static final int coord = 3;
        /**
         * The defID for the colorIndex field.
         */
        public static final int colorIndex = 4;
        /**
         * The defID for the colorPerVertex field.
         */
        public static final int colorPerVertex = 5;
        /**
         * The defID for the coordIndex field.
         */
        public static final int coordIndex = 6;
    }

    /**
     * An interface defining constants for the field defIDs of the Inline node.
     */
    public interface Inline {
        /**
         * The defID for the url field.
         */
        public static final int url = 0;
    }

    /**
     * An interface defining constants for the field defIDs of the LOD node.
     */
    public interface LOD {
        /**
         * The defID for the level field.
         */
        public static final int level = 0;
        /**
         * The defID for the center field.
         */
        public static final int center = 1;
        /**
         * The defID for the range field.
         */
        public static final int range = 2;
    }

    /**
     * An interface defining constants for the field defIDs of the Layer2D node.
     */
    public interface Layer2D {
        /**
         * The defID for the children field.
         */
        public static final int children = 2;
        /**
         * The defID for the size field.
         */
        public static final int size = 3;
        /**
         * The defID for the background field.
         */
        public static final int background = 4;
        /**
         * The defID for the viewport field.
         */
        public static final int viewport = 5;
    }

    /**
     * An interface defining constants for the field defIDs of the Layer3D node.
     */
    public interface Layer3D {
        /**
         * The defID for the children field.
         */
        public static final int children = 2;
        /**
         * The defID for the size field.
         */
        public static final int size = 3;
        /**
         * The defID for the background field.
         */
        public static final int background = 4;
        /**
         * The defID for the fog field.
         */
        public static final int fog = 5;
        /**
         * The defID for the navigationInfo field.
         */
        public static final int navigationInfo = 6;
        /**
         * The defID for the viewpoint field.
         */
        public static final int viewpoint = 7;
    }

    /**
     * An interface defining constants for the field defIDs of the Layout node.
     */
    public interface Layout {
        /**
         * The defID for the children field.
         */
        public static final int children = 2;
        /**
         * The defID for the wrap field.
         */
        public static final int wrap = 3;
        /**
         * The defID for the size field.
         */
        public static final int size = 4;
        /**
         * The defID for the horizontal field.
         */
        public static final int horizontal = 5;
        /**
         * The defID for the justify field.
         */
        public static final int justify = 6;
        /**
         * The defID for the leftToRight field.
         */
        public static final int leftToRight = 7;
        /**
         * The defID for the topToBottom field.
         */
        public static final int topToBottom = 8;
        /**
         * The defID for the spacing field.
         */
        public static final int spacing = 9;
        /**
         * The defID for the smoothScroll field.
         */
        public static final int smoothScroll = 10;
        /**
         * The defID for the loop field.
         */
        public static final int loop = 11;
        /**
         * The defID for the scrollVertical field.
         */
        public static final int scrollVertical = 12;
        /**
         * The defID for the scrollRate field.
         */
        public static final int scrollRate = 13;
    }

    /**
     * An interface defining constants for the field defIDs of the LineProperties node.
     */
    public interface LineProperties {
        /**
         * The defID for the lineColor field.
         */
        public static final int lineColor = 0;
        /**
         * The defID for the lineStyle field.
         */
        public static final int lineStyle = 1;
        /**
         * The defID for the width field.
         */
        public static final int width = 2;
    }

    /**
     * An interface defining constants for the field defIDs of the ListeningPoint node.
     */
    public interface ListeningPoint {
        /**
         * The defID for the jump field.
         */
        public static final int jump = 1;
        /**
         * The defID for the orientation field.
         */
        public static final int orientation = 2;
        /**
         * The defID for the position field.
         */
        public static final int position = 3;
        /**
         * The defID for the description field.
         */
        public static final int description = 4;
    }

    /**
     * An interface defining constants for the field defIDs of the Material node.
     */
    public interface Material {
        /**
         * The defID for the ambientIntensity field.
         */
        public static final int ambientIntensity = 0;
        /**
         * The defID for the diffuseColor field.
         */
        public static final int diffuseColor = 1;
        /**
         * The defID for the emissiveColor field.
         */
        public static final int emissiveColor = 2;
        /**
         * The defID for the shininess field.
         */
        public static final int shininess = 3;
        /**
         * The defID for the specularColor field.
         */
        public static final int specularColor = 4;
        /**
         * The defID for the transparency field.
         */
        public static final int transparency = 5;
    }

    /**
     * An interface defining constants for the field defIDs of the Material2D node.
     */
    public interface Material2D {
        /**
         * The defID for the emissiveColor field.
         */
        public static final int emissiveColor = 0;
        /**
         * The defID for the filled field.
         */
        public static final int filled = 1;
        /**
         * The defID for the lineProps field.
         */
        public static final int lineProps = 2;
        /**
         * The defID for the transparency field.
         */
        public static final int transparency = 3;
    }

    /**
     * An interface defining constants for the field defIDs of the MovieTexture node.
     */
    public interface MovieTexture {
        /**
         * The defID for the loop field.
         */
        public static final int loop = 0;
        /**
         * The defID for the speed field.
         */
        public static final int speed = 1;
        /**
         * The defID for the startTime field.
         */
        public static final int startTime = 2;
        /**
         * The defID for the stopTime field.
         */
        public static final int stopTime = 3;
        /**
         * The defID for the url field.
         */
        public static final int url = 4;
        /**
         * The defID for the repeatS field.
         */
        public static final int repeatS = 5;
        /**
         * The defID for the repeatT field.
         */
        public static final int repeatT = 6;
    }

    /**
     * An interface defining constants for the field defIDs of the NavigationInfo node.
     */
    public interface NavigationInfo {
        /**
         * The defID for the avatarSize field.
         */
        public static final int avatarSize = 1;
        /**
         * The defID for the headlight field.
         */
        public static final int headlight = 2;
        /**
         * The defID for the speed field.
         */
        public static final int speed = 3;
        /**
         * The defID for the type field.
         */
        public static final int type = 4;
        /**
         * The defID for the visibilityLimit field.
         */
        public static final int visibilityLimit = 5;
    }

    /**
     * An interface defining constants for the field defIDs of the Normal node.
     */
    public interface Normal {
        /**
         * The defID for the vector field.
         */
        public static final int vector = 0;
    }

    /**
     * An interface defining constants for the field defIDs of the NormalInterpolator node.
     */
    public interface NormalInterpolator {
        /**
         * The defID for the key field.
         */
        public static final int key = 1;
        /**
         * The defID for the keyValue field.
         */
        public static final int keyValue = 2;
    }

    /**
     * An interface defining constants for the field defIDs of the OrderedGroup node.
     */
    public interface OrderedGroup {
        /**
         * The defID for the children field.
         */
        public static final int children = 2;
        /**
         * The defID for the order field.
         */
        public static final int order = 3;
    }

    /**
     * An interface defining constants for the field defIDs of the OrientationInterpolator node.
     */
    public interface OrientationInterpolator {
        /**
         * The defID for the key field.
         */
        public static final int key = 1;
        /**
         * The defID for the keyValue field.
         */
        public static final int keyValue = 2;
    }

    /**
     * An interface defining constants for the field defIDs of the PixelTexture node.
     */
    public interface PixelTexture {
        /**
         * The defID for the image field.
         */
        public static final int image = 0;
        /**
         * The defID for the repeatS field.
         */
        public static final int repeatS = 1;
        /**
         * The defID for the repeatT field.
         */
        public static final int repeatT = 2;
    }

    /**
     * An interface defining constants for the field defIDs of the PlaneSensor node.
     */
    public interface PlaneSensor {
        /**
         * The defID for the autoOffset field.
         */
        public static final int autoOffset = 0;
        /**
         * The defID for the enabled field.
         */
        public static final int enabled = 1;
        /**
         * The defID for the maxPosition field.
         */
        public static final int maxPosition = 2;
        /**
         * The defID for the minPosition field.
         */
        public static final int minPosition = 3;
        /**
         * The defID for the offset field.
         */
        public static final int offset = 4;
    }

    /**
     * An interface defining constants for the field defIDs of the PlaneSensor2D node.
     */
    public interface PlaneSensor2D {
        /**
         * The defID for the autoOffset field.
         */
        public static final int autoOffset = 0;
        /**
         * The defID for the enabled field.
         */
        public static final int enabled = 1;
        /**
         * The defID for the maxPosition field.
         */
        public static final int maxPosition = 2;
        /**
         * The defID for the minPosition field.
         */
        public static final int minPosition = 3;
        /**
         * The defID for the offset field.
         */
        public static final int offset = 4;
    }

    /**
     * An interface defining constants for the field defIDs of the PointLight node.
     */
    public interface PointLight {
        /**
         * The defID for the ambientIntensity field.
         */
        public static final int ambientIntensity = 0;
        /**
         * The defID for the attenuation field.
         */
        public static final int attenuation = 1;
        /**
         * The defID for the color field.
         */
        public static final int color = 2;
        /**
         * The defID for the intensity field.
         */
        public static final int intensity = 3;
        /**
         * The defID for the location field.
         */
        public static final int location = 4;
        /**
         * The defID for the on field.
         */
        public static final int on = 5;
        /**
         * The defID for the radius field.
         */
        public static final int radius = 6;
    }

    /**
     * An interface defining constants for the field defIDs of the PointSet node.
     */
    public interface PointSet {
        /**
         * The defID for the color field.
         */
        public static final int color = 0;
        /**
         * The defID for the coord field.
         */
        public static final int coord = 1;
    }

    /**
     * An interface defining constants for the field defIDs of the PointSet2D node.
     */
    public interface PointSet2D {
        /**
         * The defID for the color field.
         */
        public static final int color = 0;
        /**
         * The defID for the coord field.
         */
        public static final int coord = 1;
    }

    /**
     * An interface defining constants for the field defIDs of the PositionInterpolator node.
     */
    public interface PositionInterpolator {
        /**
         * The defID for the key field.
         */
        public static final int key = 1;
        /**
         * The defID for the keyValue field.
         */
        public static final int keyValue = 2;
    }

    /**
     * An interface defining constants for the field defIDs of the PositionInterpolator2D node.
     */
    public interface PositionInterpolator2D {
        /**
         * The defID for the key field.
         */
        public static final int key = 1;
        /**
         * The defID for the keyValue field.
         */
        public static final int keyValue = 2;
    }

    /**
     * An interface defining constants for the field defIDs of the ProximitySensor2D node.
     */
    public interface ProximitySensor2D {
        /**
         * The defID for the center field.
         */
        public static final int center = 0;
        /**
         * The defID for the size field.
         */
        public static final int size = 1;
        /**
         * The defID for the enabled field.
         */
        public static final int enabled = 2;
    }

    /**
     * An interface defining constants for the field defIDs of the ProximitySensor node.
     */
    public interface ProximitySensor {
        /**
         * The defID for the center field.
         */
        public static final int center = 0;
        /**
         * The defID for the size field.
         */
        public static final int size = 1;
        /**
         * The defID for the enabled field.
         */
        public static final int enabled = 2;
    }

    /**
     * An interface defining constants for the field defIDs of the QuantizationParameter node.
     */
    public interface QuantizationParameter {
        /**
         * The defID for the isLocal field.
         */
        public static final int isLocal = 0;
        /**
         * The defID for the position3DQuant field.
         */
        public static final int position3DQuant = 1;
        /**
         * The defID for the position3DMin field.
         */
        public static final int position3DMin = 2;
        /**
         * The defID for the position3DMax field.
         */
        public static final int position3DMax = 3;
        /**
         * The defID for the position3DNbBits field.
         */
        public static final int position3DNbBits = 4;
        /**
         * The defID for the position2DQuant field.
         */
        public static final int position2DQuant = 5;
        /**
         * The defID for the position2DMin field.
         */
        public static final int position2DMin = 6;
        /**
         * The defID for the position2DMax field.
         */
        public static final int position2DMax = 7;
        /**
         * The defID for the position2DNbBits field.
         */
        public static final int position2DNbBits = 8;
        /**
         * The defID for the drawOrderQuant field.
         */
        public static final int drawOrderQuant = 9;
        /**
         * The defID for the drawOrderMin field.
         */
        public static final int drawOrderMin = 10;
        /**
         * The defID for the drawOrderMax field.
         */
        public static final int drawOrderMax = 11;
        /**
         * The defID for the drawOrderNbBits field.
         */
        public static final int drawOrderNbBits = 12;
        /**
         * The defID for the colorQuant field.
         */
        public static final int colorQuant = 13;
        /**
         * The defID for the colorMin field.
         */
        public static final int colorMin = 14;
        /**
         * The defID for the colorMax field.
         */
        public static final int colorMax = 15;
        /**
         * The defID for the colorNbBits field.
         */
        public static final int colorNbBits = 16;
        /**
         * The defID for the textureCoordinateQuant field.
         */
        public static final int textureCoordinateQuant = 17;
        /**
         * The defID for the textureCoordinateMin field.
         */
        public static final int textureCoordinateMin = 18;
        /**
         * The defID for the textureCoordinateMax field.
         */
        public static final int textureCoordinateMax = 19;
        /**
         * The defID for the textureCoordinateNbBits field.
         */
        public static final int textureCoordinateNbBits = 20;
        /**
         * The defID for the angleQuant field.
         */
        public static final int angleQuant = 21;
        /**
         * The defID for the angleMin field.
         */
        public static final int angleMin = 22;
        /**
         * The defID for the angleMax field.
         */
        public static final int angleMax = 23;
        /**
         * The defID for the angleNbBits field.
         */
        public static final int angleNbBits = 24;
        /**
         * The defID for the scaleQuant field.
         */
        public static final int scaleQuant = 25;
        /**
         * The defID for the scaleMin field.
         */
        public static final int scaleMin = 26;
        /**
         * The defID for the scaleMax field.
         */
        public static final int scaleMax = 27;
        /**
         * The defID for the scaleNbBits field.
         */
        public static final int scaleNbBits = 28;
        /**
         * The defID for the keyQuant field.
         */
        public static final int keyQuant = 29;
        /**
         * The defID for the keyMin field.
         */
        public static final int keyMin = 30;
        /**
         * The defID for the keyMax field.
         */
        public static final int keyMax = 31;
        /**
         * The defID for the keyNbBits field.
         */
        public static final int keyNbBits = 32;
        /**
         * The defID for the normalQuant field.
         */
        public static final int normalQuant = 33;
        /**
         * The defID for the normalNbBits field.
         */
        public static final int normalNbBits = 34;
        /**
         * The defID for the sizeQuant field.
         */
        public static final int sizeQuant = 35;
        /**
         * The defID for the sizeMin field.
         */
        public static final int sizeMin = 36;
        /**
         * The defID for the sizeMax field.
         */
        public static final int sizeMax = 37;
        /**
         * The defID for the sizeNbBits field.
         */
        public static final int sizeNbBits = 38;
        /**
         * The defID for the useEfficientCoding field.
         */
        public static final int useEfficientCoding = 39;
    }

    /**
     * An interface defining constants for the field defIDs of the Rectangle node.
     */
    public interface Rectangle {
        /**
         * The defID for the size field.
         */
        public static final int size = 0;
    }

    /**
     * An interface defining constants for the field defIDs of the ScalarInterpolator node.
     */
    public interface ScalarInterpolator {
        /**
         * The defID for the key field.
         */
        public static final int key = 1;
        /**
         * The defID for the keyValue field.
         */
        public static final int keyValue = 2;
    }

    /**
     * An interface defining constants for the field defIDs of the Script node.
     */
    public interface Script {
        /**
         * The defID for the url field.
         */
        public static final int url = 0;
        /**
         * The defID for the directOutput field.
         */
        public static final int directOutput = 1;
        /**
         * The defID for the mustEvaluate field.
         */
        public static final int mustEvaluate = 2;
    }

    /**
     * An interface defining constants for the field defIDs of the Shape node.
     */
    public interface Shape {
        /**
         * The defID for the appearance field.
         */
        public static final int appearance = 0;
        /**
         * The defID for the geometry field.
         */
        public static final int geometry = 1;
    }

    /**
     * An interface defining constants for the field defIDs of the Sound node.
     */
    public interface Sound {
        /**
         * The defID for the direction field.
         */
        public static final int direction = 0;
        /**
         * The defID for the intensity field.
         */
        public static final int intensity = 1;
        /**
         * The defID for the location field.
         */
        public static final int location = 2;
        /**
         * The defID for the maxBack field.
         */
        public static final int maxBack = 3;
        /**
         * The defID for the maxFront field.
         */
        public static final int maxFront = 4;
        /**
         * The defID for the minBack field.
         */
        public static final int minBack = 5;
        /**
         * The defID for the minFront field.
         */
        public static final int minFront = 6;
        /**
         * The defID for the priority field.
         */
        public static final int priority = 7;
        /**
         * The defID for the source field.
         */
        public static final int source = 8;
        /**
         * The defID for the spatialize field.
         */
        public static final int spatialize = 9;
    }

    /**
     * An interface defining constants for the field defIDs of the Sound2D node.
     */
    public interface Sound2D {
        /**
         * The defID for the intensity field.
         */
        public static final int intensity = 0;
        /**
         * The defID for the location field.
         */
        public static final int location = 1;
        /**
         * The defID for the source field.
         */
        public static final int source = 2;
        /**
         * The defID for the spatialize field.
         */
        public static final int spatialize = 3;
    }

    /**
     * An interface defining constants for the field defIDs of the Sphere node.
     */
    public interface Sphere {
        /**
         * The defID for the radius field.
         */
        public static final int radius = 0;
    }

    /**
     * An interface defining constants for the field defIDs of the SphereSensor node.
     */
    public interface SphereSensor {
        /**
         * The defID for the autoOffset field.
         */
        public static final int autoOffset = 0;
        /**
         * The defID for the enabled field.
         */
        public static final int enabled = 1;
        /**
         * The defID for the offset field.
         */
        public static final int offset = 2;
    }

    /**
     * An interface defining constants for the field defIDs of the SpotLight node.
     */
    public interface SpotLight {
        /**
         * The defID for the ambientIntensity field.
         */
        public static final int ambientIntensity = 0;
        /**
         * The defID for the attenuation field.
         */
        public static final int attenuation = 1;
        /**
         * The defID for the beamWidth field.
         */
        public static final int beamWidth = 2;
        /**
         * The defID for the color field.
         */
        public static final int color = 3;
        /**
         * The defID for the cutOffAngle field.
         */
        public static final int cutOffAngle = 4;
        /**
         * The defID for the direction field.
         */
        public static final int direction = 5;
        /**
         * The defID for the intensity field.
         */
        public static final int intensity = 6;
        /**
         * The defID for the location field.
         */
        public static final int location = 7;
        /**
         * The defID for the on field.
         */
        public static final int on = 8;
        /**
         * The defID for the radius field.
         */
        public static final int radius = 9;
    }

    /**
     * An interface defining constants for the field defIDs of the Switch node.
     */
    public interface Switch {
        /**
         * The defID for the choice field.
         */
        public static final int choice = 0;
        /**
         * The defID for the whichChoice field.
         */
        public static final int whichChoice = 1;
    }

    /**
     * An interface defining constants for the field defIDs of the TermCap node.
     */
    public interface TermCap {
        /**
         * The defID for the capability field.
         */
        public static final int capability = 1;
    }

    /**
     * An interface defining constants for the field defIDs of the Text node.
     */
    public interface Text {
        /**
         * The defID for the string field.
         */
        public static final int string = 0;
        /**
         * The defID for the length field.
         */
        public static final int length = 1;
        /**
         * The defID for the fontStyle field.
         */
        public static final int fontStyle = 2;
        /**
         * The defID for the maxExtent field.
         */
        public static final int maxExtent = 3;
    }

    /**
     * An interface defining constants for the field defIDs of the TextureCoordinate node.
     */
    public interface TextureCoordinate {
        /**
         * The defID for the point field.
         */
        public static final int point = 0;
    }

    /**
     * An interface defining constants for the field defIDs of the TextureTransform node.
     */
    public interface TextureTransform {
        /**
         * The defID for the center field.
         */
        public static final int center = 0;
        /**
         * The defID for the rotation field.
         */
        public static final int rotation = 1;
        /**
         * The defID for the scale field.
         */
        public static final int scale = 2;
        /**
         * The defID for the translation field.
         */
        public static final int translation = 3;
    }

    /**
     * An interface defining constants for the field defIDs of the TimeSensor node.
     */
    public interface TimeSensor {
        /**
         * The defID for the cycleInterval field.
         */
        public static final int cycleInterval = 0;
        /**
         * The defID for the enabled field.
         */
        public static final int enabled = 1;
        /**
         * The defID for the loop field.
         */
        public static final int loop = 2;
        /**
         * The defID for the startTime field.
         */
        public static final int startTime = 3;
        /**
         * The defID for the stopTime field.
         */
        public static final int stopTime = 4;
    }

    /**
     * An interface defining constants for the field defIDs of the TouchSensor node.
     */
    public interface TouchSensor {
        /**
         * The defID for the enabled field.
         */
        public static final int enabled = 0;
    }

    /**
     * An interface defining constants for the field defIDs of the Transform node.
     */
    public interface Transform {
        /**
         * The defID for the center field.
         */
        public static final int center = 2;
        /**
         * The defID for the children field.
         */
        public static final int children = 3;
        /**
         * The defID for the rotation field.
         */
        public static final int rotation = 4;
        /**
         * The defID for the scale field.
         */
        public static final int scale = 5;
        /**
         * The defID for the scaleOrientation field.
         */
        public static final int scaleOrientation = 6;
        /**
         * The defID for the translation field.
         */
        public static final int translation = 7;
    }

    /**
     * An interface defining constants for the field defIDs of the Transform2D node.
     */
    public interface Transform2D {
        /**
         * The defID for the children field.
         */
        public static final int children = 2;
        /**
         * The defID for the center field.
         */
        public static final int center = 3;
        /**
         * The defID for the rotationAngle field.
         */
        public static final int rotationAngle = 4;
        /**
         * The defID for the scale field.
         */
        public static final int scale = 5;
        /**
         * The defID for the scaleOrientation field.
         */
        public static final int scaleOrientation = 6;
        /**
         * The defID for the translation field.
         */
        public static final int translation = 7;
    }

    /**
     * An interface defining constants for the field defIDs of the Valuator node.
     */
    public interface Valuator {
        /**
         * The defID for the Factor1 field.
         */
        public static final int Factor1 = 32;
        /**
         * The defID for the Factor2 field.
         */
        public static final int Factor2 = 33;
        /**
         * The defID for the Factor3 field.
         */
        public static final int Factor3 = 34;
        /**
         * The defID for the Factor4 field.
         */
        public static final int Factor4 = 35;
        /**
         * The defID for the Offset1 field.
         */
        public static final int Offset1 = 36;
        /**
         * The defID for the Offset2 field.
         */
        public static final int Offset2 = 37;
        /**
         * The defID for the Offset3 field.
         */
        public static final int Offset3 = 38;
        /**
         * The defID for the viseme_def field.
         */
        public static final int viseme_def = 3;
    }

    /**
     * An interface defining constants for the field defIDs of the WorldInfo node.
     */
    public interface WorldInfo {
        /**
         * The defID for the info field.
         */
        public static final int info = 0;
        /**
         * The defID for the title field.
         */
        public static final int title = 1;
    }

}