/***********************************************************************
This software module was originally developed by Zvi Lifshitz
(Triton R&D Ltd.) in the course of development of the MPEG-4 Systems
(ISO/IEC 14496-1) standard. This software module is an implementation of
a part of one or more MPEG-4 Systems (ISO/IEC 14496-1) tools as
specified by the MPEG-4 Systems (ISO/IEC 14496-1) standard. ISO/IEC
gives users of the MPEG-4 Systems (ISO/IEC 14496-1) free license to
this software module or modifications thereof for use in hardware or
software products claiming conformance to the MPEG-4 Systems (ISO/IEC
14496-1). Those intending to use this software module in hardware or
software products are advised that its use may infringe existing patents.
The original developer of this software module and his/her company, the
subsequent editors and their companies, and ISO/IEC have no liability
for use of this software module or modifications thereof in an
implementation. Copyright is not released for non MPEG-4 Systems
(ISO/IEC 14496-1) conforming products. Triton R&D Ltd. retains full right
to use the code for his/her own purpose, assign or donate the code to a
third party and to inhibit third parties from using the code for non
MPEG-4 Systems (ISO/IEC 14496-1) conforming products. This copyright
notice must be included in all copies or derivative works.
Copyright  1999.
************************************************************************/
package org.iso.mpeg.mpegj.scene;

/**
 * An interface defining constants for node types.
 */
public interface NodeType {
    /**
     * Anchor node.
     */
    public static final int Anchor = 1;

    /**
     * AnimationStream node.
     */
    public static final int AnimationStream = 2;

    /**
     * Appearance node.
     */
    public static final int Appearance = 3;

    /**
     * AudioBuffer node.
     */
    public static final int AudioBuffer = 4;

    /**
     * AudioClip node.
     */
    public static final int AudioClip = 5;

    /**
     * AudioDelay node.
     */
    public static final int AudioDelay = 6;

    /**
     * AudioFX node.
     */
    public static final int AudioFX = 7;

    /**
     * AudioMix node.
     */
    public static final int AudioMix = 8;

    /**
     * AudioSource node.
     */
    public static final int AudioSource = 9;

    /**
     * AudioSwitch node.
     */
    public static final int AudioSwitch = 10;

    /**
     * Background node.
     */
    public static final int Background = 11;

    /**
     * Background2D node.
     */
    public static final int Background2D = 12;

    /**
     * Billboard node.
     */
    public static final int Billboard = 13;

    /**
     * Bitmap node.
     */
    public static final int Bitmap = 14;

    /**
     * Box node.
     */
    public static final int Box = 15;

    /**
     * Circle node.
     */
    public static final int Circle = 16;

    /**
     * Collision node.
     */
    public static final int Collision = 17;

    /**
     * Color node.
     */
    public static final int Color = 18;

    /**
     * ColorInterpolator node.
     */
    public static final int ColorInterpolator = 19;

    /**
     * CompositeTexture2D node.
     */
    public static final int CompositeTexture2D = 20;

    /**
     * CompositeTexture3D node.
     */
    public static final int CompositeTexture3D = 21;

    /**
     * Conditional node.
     */
    public static final int Conditional = 22;

    /**
     * Cone node.
     */
    public static final int Cone = 23;

    /**
     * Coordinate node.
     */
    public static final int Coordinate = 24;

    /**
     * Coordinate2D node.
     */
    public static final int Coordinate2D = 25;

    /**
     * CoordinateInterpolator node.
     */
    public static final int CoordinateInterpolator = 26;

    /**
     * CoordinateInterpolator2D node.
     */
    public static final int CoordinateInterpolator2D = 27;

    /**
     * Curve2D node.
     */
    public static final int Curve2D = 28;

    /**
     * Cylinder node.
     */
    public static final int Cylinder = 29;

    /**
     * CylinderSensor node.
     */
    public static final int CylinderSensor = 30;

    /**
     * DirectionalLight node.
     */
    public static final int DirectionalLight = 31;

    /**
     * DiscSensor node.
     */
    public static final int DiscSensor = 32;

    /**
     * ElevationGrid node.
     */
    public static final int ElevationGrid = 33;

    /**
     * Expression node.
     */
    public static final int Expression = 34;

    /**
     * Extrusion node.
     */
    public static final int Extrusion = 35;

    /**
     * Face node.
     */
    public static final int Face = 36;

    /**
     * FaceDefMesh node.
     */
    public static final int FaceDefMesh = 37;

    /**
     * FaceDefTables node.
     */
    public static final int FaceDefTables = 38;

    /**
     * FaceDefTransform node.
     */
    public static final int FaceDefTransform = 39;

    /**
     * FAP node.
     */
    public static final int FAP = 40;

    /**
     * FDP node.
     */
    public static final int FDP = 41;

    /**
     * FIT node.
     */
    public static final int FIT = 42;

    /**
     * Fog node.
     */
    public static final int Fog = 43;

    /**
     * FontStyle node.
     */
    public static final int FontStyle = 44;

    /**
     * Form node.
     */
    public static final int Form = 45;

    /**
     * Group node.
     */
    public static final int Group = 46;

    /**
     * ImageTexture node.
     */
    public static final int ImageTexture = 47;

    /**
     * IndexedFaceSet node.
     */
    public static final int IndexedFaceSet = 48;

    /**
     * IndexedFaceSet2D node.
     */
    public static final int IndexedFaceSet2D = 49;

    /**
     * IndexedLineSet node.
     */
    public static final int IndexedLineSet = 50;

    /**
     * IndexedLineSet2D node.
     */
    public static final int IndexedLineSet2D = 51;

    /**
     * Inline node.
     */
    public static final int Inline = 52;

    /**
     * LOD node.
     */
    public static final int LOD = 53;

    /**
     * Layer2D node.
     */
    public static final int Layer2D = 54;

    /**
     * Layer3D node.
     */
    public static final int Layer3D = 55;

    /**
     * Layout node.
     */
    public static final int Layout = 56;

    /**
     * LineProperties node.
     */
    public static final int LineProperties = 57;

    /**
     * ListeningPoint node.
     */
    public static final int ListeningPoint = 58;

    /**
     * Material node.
     */
    public static final int Material = 59;

    /**
     * Material2D node.
     */
    public static final int Material2D = 60;

    /**
     * MovieTexture node.
     */
    public static final int MovieTexture = 61;

    /**
     * NavigationInfo node.
     */
    public static final int NavigationInfo = 62;

    /**
     * Normal node.
     */
    public static final int Normal = 63;

    /**
     * NormalInterpolator node.
     */
    public static final int NormalInterpolator = 64;

    /**
     * OrderedGroup node.
     */
    public static final int OrderedGroup = 65;

    /**
     * OrientationInterpolator node.
     */
    public static final int OrientationInterpolator = 66;

    /**
     * PixelTexture node.
     */
    public static final int PixelTexture = 67;

    /**
     * PlaneSensor node.
     */
    public static final int PlaneSensor = 68;

    /**
     * PlaneSensor2D node.
     */
    public static final int PlaneSensor2D = 69;

    /**
     * PointLight node.
     */
    public static final int PointLight = 70;

    /**
     * PointSet node.
     */
    public static final int PointSet = 71;

    /**
     * PointSet2D node.
     */
    public static final int PointSet2D = 72;

    /**
     * PositionInterpolator node.
     */
    public static final int PositionInterpolator = 73;

    /**
     * PositionInterpolator2D node.
     */
    public static final int PositionInterpolator2D = 74;

    /**
     * ProximitySensor2D node.
     */
    public static final int ProximitySensor2D = 75;

    /**
     * ProximitySensor node.
     */
    public static final int ProximitySensor = 76;

    /**
     * QuantizationParameter node.
     */
    public static final int QuantizationParameter = 77;

    /**
     * Rectangle node.
     */
    public static final int Rectangle = 78;

    /**
     * ScalarInterpolator node.
     */
    public static final int ScalarInterpolator = 79;

    /**
     * Script node.
     */
    public static final int Script = 80;

    /**
     * Shape node.
     */
    public static final int Shape = 81;

    /**
     * Sound node.
     */
    public static final int Sound = 82;

    /**
     * Sound2D node.
     */
    public static final int Sound2D = 83;

    /**
     * Sphere node.
     */
    public static final int Sphere = 84;

    /**
     * SphereSensor node.
     */
    public static final int SphereSensor = 85;

    /**
     * SpotLight node.
     */
    public static final int SpotLight = 86;

    /**
     * Switch node.
     */
    public static final int Switch = 87;

    /**
     * TermCap node.
     */
    public static final int TermCap = 88;

    /**
     * Text node.
     */
    public static final int Text = 89;

    /**
     * TextureCoordinate node.
     */
    public static final int TextureCoordinate = 90;

    /**
     * TextureTransform node.
     */
    public static final int TextureTransform = 91;

    /**
     * TimeSensor node.
     */
    public static final int TimeSensor = 92;

    /**
     * TouchSensor node.
     */
    public static final int TouchSensor = 93;

    /**
     * Transform node.
     */
    public static final int Transform = 94;

    /**
     * Transform2D node.
     */
    public static final int Transform2D = 95;

    /**
     * Valuator node.
     */
    public static final int Valuator = 96;

    /**
     * Viewpoint node.
     */
    public static final int Viewpoint = 97;

    /**
     * VisibilitySensor node.
     */
    public static final int VisibilitySensor = 98;

    /**
     * Viseme node.
     */
    public static final int Viseme = 99;

    /**
     * WorldInfo node.
     */
    public static final int WorldInfo = 100;

}
