/***********************************************************************
This software module was originally developed by Zvi Lifshitz
(Triton R&D Ltd.) in the course of development of the MPEG-4 Systems
(ISO/IEC 14496-1) standard. This software module is an implementation of
a part of one or more MPEG-4 Systems (ISO/IEC 14496-1) tools as
specified by the MPEG-4 Systems (ISO/IEC 14496-1) standard. ISO/IEC
gives users of the MPEG-4 Systems (ISO/IEC 14496-1) free license to
this software module or modifications thereof for use in hardware or
software products claiming conformance to the MPEG-4 Systems (ISO/IEC
14496-1). Those intending to use this software module in hardware or
software products are advised that its use may infringe existing patents.
The original developer of this software module and his/her company, the
subsequent editors and their companies, and ISO/IEC have no liability
for use of this software module or modifications thereof in an
implementation. Copyright is not released for non MPEG-4 Systems
(ISO/IEC 14496-1) conforming products. Triton R&D Ltd. retains full right
to use the code for his/her own purpose, assign or donate the code to a
third party and to inhibit third parties from using the code for non
MPEG-4 Systems (ISO/IEC 14496-1) conforming products. This copyright
notice must be included in all copies or derivative works.
Copyright  1999.
************************************************************************/
import org.iso.mpeg.mpegj.*;
import org.iso.mpeg.mpegj.scene.*;
/**
	Gets the scene from the MpegjTerminal.
 */
public class MySceneListener implements SceneListener 
{
	private Scene m_scene;

	/**
	Waits for and returns the scene associated with the given MpegjTerminal.

	@param terminal the MpegjTerminal containing the scene
	@return the scene
	@exception MPEGJException if an MPEG-J failure occurs
	@exception InterruptedException if interrupted while waiting for the scene
	 */
	public synchronized Scene getScene(MpegjTerminal terminal)
	throws MPEGJException, InterruptedException, SceneManagerNotFoundException
	{
		SceneManager mgr = terminal.getSceneManager();
		mgr.addSceneListener(this);
		while (m_scene == null)
		{
			wait();
		}
		return m_scene;
	}
	/**
	Called back by the SceneManager to provide the scene.
	 */
	public synchronized void notify(int what, Scene scene)
	{
		if (what == SceneListener.Message.SCENE_READY)
		{
			System.out.println("MySceneListener.notify() SCENE_READY!");
			m_scene = scene;
			notify();
		}
	}
}


