/***********************************************************************
This software module was originally developed by Zvi Lifshitz
(Triton R&D Ltd.) in the course of development of the MPEG-4 Systems
(ISO/IEC 14496-1) standard. This software module is an implementation of
a part of one or more MPEG-4 Systems (ISO/IEC 14496-1) tools as
specified by the MPEG-4 Systems (ISO/IEC 14496-1) standard. ISO/IEC
gives users of the MPEG-4 Systems (ISO/IEC 14496-1) free license to
this software module or modifications thereof for use in hardware or
software products claiming conformance to the MPEG-4 Systems (ISO/IEC
14496-1). Those intending to use this software module in hardware or
software products are advised that its use may infringe existing patents.
The original developer of this software module and his/her company, the
subsequent editors and their companies, and ISO/IEC have no liability
for use of this software module or modifications thereof in an
implementation. Copyright is not released for non MPEG-4 Systems
(ISO/IEC 14496-1) conforming products. Triton R&D Ltd. retains full right
to use the code for his/her own purpose, assign or donate the code to a
third party and to inhibit third parties from using the code for non
MPEG-4 Systems (ISO/IEC 14496-1) conforming products. This copyright
notice must be included in all copies or derivative works.
Copyright  1999.
************************************************************************/
import org.iso.mpeg.mpegj.*;
import org.iso.mpeg.mpegj.scene.*;

/**
	Listens to the fraction_changed field of a DEF 2 TimeSensor.
 */
public class MyEventOutListener implements EventOutListener 
{
	/**
	Registers as a listener on the fraction_changed field.
	@param scene the scene
	@exception MPEGJException if an MPEG-J failure occurs
	 */
	public void register(Scene scene) throws MPEGJException
	{
		Node node = scene.getNode(2);

		// Check that the node is a TimeSensor.
		if (node.getNodeType() != NodeType.TimeSensor)
		{
			System.err.println("Node 2 is not a TimeSensor!");
			return;
		}

		// Add ourselves as a listener.
		node.addEventOutListener(EventOut.TimeSensor.fraction_changed, this);
	}

	/**
	Called back when the fraction_changed field changes.

	@param outID the eventOut identifier
	@param newValue the new value of the field
	 */
	public void notify(int outID, FieldValue newValue)
	{
		float fractionChanged = ((SFFloatFieldValue) newValue).getSFFloatValue();
		System.out.println("fraction_changed=" + fractionChanged);
	}
}














	