/***********************************************************************
This software module was originally developed by Zvi Lifshitz
(Triton R&D Ltd.) in the course of development of the MPEG-4 Systems
(ISO/IEC 14496-1) standard. This software module is an implementation of
a part of one or more MPEG-4 Systems (ISO/IEC 14496-1) tools as
specified by the MPEG-4 Systems (ISO/IEC 14496-1) standard. ISO/IEC
gives users of the MPEG-4 Systems (ISO/IEC 14496-1) free license to
this software module or modifications thereof for use in hardware or
software products claiming conformance to the MPEG-4 Systems (ISO/IEC
14496-1). Those intending to use this software module in hardware or
software products are advised that its use may infringe existing patents.
The original developer of this software module and his/her company, the
subsequent editors and their companies, and ISO/IEC have no liability
for use of this software module or modifications thereof in an
implementation. Copyright is not released for non MPEG-4 Systems
(ISO/IEC 14496-1) conforming products. Triton R&D Ltd. retains full right
to use the code for his/her own purpose, assign or donate the code to a
third party and to inhibit third parties from using the code for non
MPEG-4 Systems (ISO/IEC 14496-1) conforming products. This copyright
notice must be included in all copies or derivative works.
Copyright  1999.
************************************************************************/
import org.iso.mpeg.mpegj.*;
import org.iso.mpeg.mpegj.scene.*;

/**
	Translates the translation field of a DEF 1 Transform2D node.
 */
public class MyTranslate {
	/**
		Translates the translation field of node 1.

		@param scene the scene
		@exception MPEGJException if an MPEG-J failure occurs
	 */
	public void translate(Scene scene,int x, int y) throws MPEGJException {
		Node node = scene.getNode(0);

		// Check that the node is a Transform2D.
		if (node.getNodeType() != NodeType.Transform2D) {
			System.out.println("Node 0 is not a Transform2D!");
			return;
		}

		// Get the current translation and print it.
		int outID = EventOut.Transform2D.translation;
		SFVec2fFieldValue translationEventOut =
			(SFVec2fFieldValue) node.getEventOut(outID);
		float[] translation = translationEventOut.getSFVec2fValue();
		
		System.out.println("Node 0 translation is " +	translation[0] + "," + translation[1]);
		
		// Calculate a new translation.
		final float[] newTranslation = {
			translation[0] + x,
			translation[1] + y
		};

		// Set the new translation via the eventIn.
		int inID = EventIn.Transform2D.translation;
		SFVec2fFieldValue translationEventIn =
			new SFVec2fFieldValue() {
				public float[] getSFVec2fValue() {
					return newTranslation;
				}
			};
		node.sendEventIn(inID, translationEventIn);
	}
}
