/********************************************************************
 This software module was originally developed by Roberto Becchini
 (SIA - I - SOMMIT Consortium), Barbara Silano (SIA - I - SOMMIT 
Consortium) and edited by Luca Deri (Finsiel - I - SOMMIT Consortium),
 Gianluca De Petris (CSELT - I - SOMMIT Consortium) in the course of 
development of the MPEG-4 Systems (ISO/IEC 14496-1). This software 
module is an implementation of part of one or more MPEG-4 Systems 
(ISO/IEC 14496-1) tools as specified by the MPEG-4 Systems (ISO/IEC 
14496-1). ISO/IEC gives users of the MPEG-4 Systems (ISO/IEC 14496-1) 
free license to this software module or modifications thereof for use 
in hardware or software products claiming conformance to the MPEG-4 
Systems (ISO/IEC 14496-1). Those intending to use this software 
module in hardware or software products are advised that its use may 
infringe existing patents. THE ORIGINAL DEVELOPER OF THIS SOFTWARE 
MODULE AND HIS/HER COMPANY, THE SUBSEQUENT EDITORS AND THEIR 
COMPANIES, AND ISO/IEC HAVE NO LIABILITY FOR USE OF THIS SOFTWARE 
MODULE OR MODIFICATIONS THEREOF. No license to this software module 
is granted for non MPEG-4 Systems (ISO/IEC 14496-1) conforming 
products. The SOMMIT Consortium retains full right to use the 
software module for his/her own purpose, assign or donate the 
software module to a third party and to inhibit third parties from 
using the code for non MPEG-4 Systems (ISO/IEC 14496-1) conforming 
products. This copyright notice must be included in all copies or 
derivative works of the software module. Copyright 1999
********************************************************************/

import java.awt.*;
import java.awt.event.ActionListener;
import java.awt.event.ActionEvent;
import javax.swing.*;
import java.util.*;
import org.iso.mpeg.mpegj.net.*;
import java.rmi.*;


public class ResourcesMonitorRemote extends JFrame
				implements ActionListener {
	static DMIFMonitorServerInterface dmif;
	private static ResourcesMonitorRemote monitorWindow;
	private static Timer timer;
	private static MonitorSourceRemote monitorSourceRemote;
	private static JTabbedPane tabbedPaneSessions, tabbedPaneChannels;
	public static Vector sessions;
	private static JScrollPane scrollPaneChannels;
	private static JScrollPane scrollPane2Channels;
	private static String sessionNumber, currentSession;
	
	public ResourcesMonitorRemote(String title)
	{
	super(title);
	try
	{
		Object obj = Naming.lookup("rmi://"+hostname+"/DMIFMonitorServer");
	//	Object obj = Naming.lookup("rmi:/DMIFMonitorServer");//MACCHINA LOCALE

		dmif = (DMIFMonitorServerInterface)obj;
	}catch(Exception exc)
	{
		Frame f2 = new Frame(((Throwable)exc).getMessage());
		f2.resize(100, 100);
		f2.show();

		exc.printStackTrace();
		System.exit(0);
	}	
	}	
	
	static void addToCenter(JPanel gridBagPanel, JComponent component, GridBagConstraints c, int x, int y){
		GridBagLayout gridbag = (GridBagLayout)gridBagPanel.getLayout();

		c.gridy = y;
		c.gridx = x;
		c.weightx = 1.0;
		c.insets = new Insets(10,10,10,10);
		c.anchor = GridBagConstraints.CENTER;
		
		gridbag.setConstraints(component, c);
		gridBagPanel.add(component);	
	}
	static void addToWest(JPanel gridBagPanel, JComponent component, GridBagConstraints c, int x, int y){
		GridBagLayout gridbag = (GridBagLayout)gridBagPanel.getLayout();

		c.gridy = y;
		c.gridx = x;
		c.weightx = 1.0;
		c.insets = new Insets(10,30,10,10);
		c.anchor = GridBagConstraints.WEST;
		c.fill = GridBagConstraints.BOTH;
		
		gridbag.setConstraints(component, c);
		gridBagPanel.add(component);
	}

	public static void addSession(String sessionId){
		//***to be implemented***
	}
	public static void addChannel(long channelId){
		ChannelInformationsRemote channelInfos = new ChannelInformationsRemote();
		channelInfos.channelId = channelId;
		channelInfos.ESnumber = new JLabel(); 

		//extract 6 bit from ESid representing the ES number
		channelInfos.ESnumber.setText("Elementary stream number: "+(channelId & 0x0000003F) + " ");
		
		
		channelInfos.ODid = new JLabel();
		channelInfos.ODid.setText("Object descriptor id.: " + (channelId >> 6) + " ");

		channelInfos.urlIcon = new ImageIcon("icons/telnet.gif");

		channelInfos.scrollH = new JScrollPane();
		channelInfos.scrollV = new JScrollPane(); 

		channelInfos.panel = new JPanel();
		
		channelInfos.back = new JButton("BACK...");
		channelInfos.back.addActionListener(monitorWindow);
		channelInfos.back.setActionCommand("Back");
		
		
		channelInfos.status = new JLabel("Status: "); 
		channelInfos.error = new JLabel("Error: ");
		channelInfos.auRate = new JLabel("AccessUnit received rate (AU/sec)");

		channelInfos.qualifier = new QualifierUI();
		channelInfos.qualifier.metrics = new QosMetrics();
		
		GridBagLayout gridbag;
		channelInfos.panel.setLayout((gridbag = new GridBagLayout()));
		GridBagConstraints c = new GridBagConstraints();
		
		addToCenter(channelInfos.panel, channelInfos.back, c, 0, 0);
		
	
		addToWest(channelInfos.panel, channelInfos.ODid, c, 0, 1);
		addToWest(channelInfos.panel, channelInfos.ESnumber, c, 0, 2);
		addToWest(channelInfos.panel, channelInfos.status, c, 0, 3);
		addToWest(channelInfos.panel, channelInfos.auRate, c, 0, 4);
		addToWest(channelInfos.panel, channelInfos.error, c, 0, 5);
		addToWest(channelInfos.panel, channelInfos.qualifier.metrics.maxDelay, c, 0, 6);
		addToWest(channelInfos.panel, channelInfos.qualifier.metrics.avgDelay, c, 0, 7);
		addToWest(channelInfos.panel, channelInfos.qualifier.metrics.priority, c, 0, 8);
		addToWest(channelInfos.panel, channelInfos.qualifier.metrics.lossProb, c, 0, 9);
		addToWest(channelInfos.panel, channelInfos.qualifier.metrics.maxGapLoss, c, 0, 10);
		addToWest(channelInfos.panel, channelInfos.qualifier.metrics.maxAuSize, c, 0, 11);
		addToWest(channelInfos.panel, channelInfos.qualifier.metrics.maxAuRate, c, 0, 12);
		addToWest(channelInfos.panel, channelInfos.qualifier.metrics.avgAuSize, c, 0, 13);


		channelInfos.scrollV.setViewportView(channelInfos.panel);		
	

		sessionNumber = new String("0");

		tabbedPaneChannels.addTab("Channel " + channelId, channelInfos.urlIcon, channelInfos.scrollV);
		tabbedPaneChannels.setSelectedIndex(0); 

		((SessionInformations)sessions.elementAt(0)).channels.addElement(channelInfos);
	}
	static void addChannelsPane(){
		tabbedPaneChannels= new JTabbedPane(1);
		JScrollPane scrollPane = new JScrollPane();
		scrollPane.setViewportView(tabbedPaneChannels);
		monitorWindow.getContentPane().add("Channels card", scrollPane);

		
	}
	static void addSessionsPane(){

		tabbedPaneSessions= new JTabbedPane(1);
		ImageIcon mpegJIcon = new ImageIcon("icons/inet02.gif");
		SessionInformations sessionInfos = new SessionInformations();
		sessionInfos.scrollH = new JScrollPane();
		sessionInfos.scrollV = new JScrollPane();
		sessionInfos.close = new JButton("CLOSE");
		sessionInfos.channelsDetails = new JButton("Channels details ...");
		sessionInfos.urlIcon = new ImageIcon("icons/compass3b.gif");
		sessionInfos.channelsIcon = new ImageIcon("icons/telnet.gif");		
	        sessionInfos.url = new JLabel("URL: ", sessionInfos.urlIcon, SwingConstants.LEFT);		
		sessionInfos.numberOfChannels = new JLabel("No. of channels: ", sessionInfos.channelsIcon, SwingConstants.LEFT);
		sessionInfos.panel  =new JPanel();		
		sessionInfos.close.addActionListener(monitorWindow);
		sessionInfos.close.setActionCommand("Close");
		sessionInfos.channelsDetails.addActionListener(monitorWindow);
		sessionInfos.channelsDetails.setActionCommand("CHDET");
		GridBagLayout gridbag;
		sessionInfos.panel.setLayout((gridbag = new GridBagLayout()));
		GridBagConstraints c = new GridBagConstraints();

		addToCenter(sessionInfos.panel, sessionInfos.close, c, 0, 0);
		addToCenter(sessionInfos.panel, sessionInfos.channelsDetails, c, 1, 0);
		
		addToWest(sessionInfos.panel, sessionInfos.url, c, 0, 1);
		addToWest(sessionInfos.panel, sessionInfos.numberOfChannels, c, 0, 2);
		
		sessionInfos.scrollV.setViewportView(sessionInfos.panel);		
		
		sessionNumber = new String("0");
		sessionInfos.sessionId = 0;//Long.valueOf(sessionNumber);
		tabbedPaneSessions.addTab("Session "+ sessionNumber, mpegJIcon, sessionInfos.scrollV);
		tabbedPaneSessions.setSelectedIndex(0); 
		currentSession = String.valueOf(0);

		sessionInfos.scrollH.setViewportView(tabbedPaneSessions);
               monitorWindow.getContentPane().add("Sessions card", sessionInfos.scrollH);
		sessionInfos.channels = new Vector(1);
		sessions.addElement(sessionInfos);
	}

	public void updateUrl(long sessionId, String aUrl){
		((SessionInformations)sessions.elementAt((int)sessionId)).url.setText("URL: " + aUrl);
	}

	public void updateNumberOfChannels(long sessionId, int number){
		((SessionInformations)sessions.elementAt((int)sessionId)).numberOfChannels.setText("No. of channels: " + String.valueOf(number));
	}
	public void upDateStatus(long sessionId, long channelId, int status){ 
		SessionInformations sessInfos;
		ChannelInformationsRemote chInfos ;
		sessInfos = ((SessionInformations)sessions.elementAt((int)sessionId));
		
		int i;
		int flag = 0;
		String strStatus=null;
		for(i = 0; i< sessInfos.channels.size() || flag == 0; i++)
		{	
	
			chInfos = (ChannelInformationsRemote)(sessInfos.channels.elementAt(i));

			if( chInfos.channelId == channelId)
			{
				switch(status)
				{
				case 0:
					strStatus = new String("CS_IDLE");
					break;
				case 1:
					strStatus = new String("CS_CONFIGURED");
					break;
				case 2:
					strStatus = new String("CS_READY");
					break;
				case 3:
					strStatus = new String("CS_REQUIRED");
					break;
				case 4:
					strStatus = new String("CS_REQUESTED");
					break;
				case 5:
					strStatus = new String("CS_OPEN");
					break;
				case 6:
					strStatus = new String("CS_CLOSING");
					break;
				case 7:
					strStatus = new String("CS_DELETING");
					break;
				case 8:
					strStatus = new String("CS_NOT_ACTIVE");
					break;
				default:
					strStatus = new String("STATUS UNKNOWN");
					break;
				}		
				chInfos.status.setText("Status: " + strStatus);
				flag = 1;
			}
		}	
		
	}

	public void upDateAURate(long sessionId, long channelId, int rate){
		SessionInformations sessInfos;
		ChannelInformationsRemote chInfos;
		sessInfos = ((SessionInformations)sessions.elementAt((int)sessionId));
		chInfos = (ChannelInformationsRemote)(sessInfos.channels.elementAt((int)channelId));
		chInfos.auRate.setText("AccessUnit received rate: " + rate/3 + " (AU/sec)");
	}

	public void upDateError(long sessionId, long channelId, int error){
		String strErr; 

		SessionInformations sessInfos;
		ChannelInformationsRemote chInfos;
		sessInfos = ((SessionInformations)sessions.elementAt((int)sessionId));
		chInfos = (ChannelInformationsRemote)(sessInfos.channels.elementAt((int)channelId));
		switch(error)
		{
		case 0:
			strErr = new String("IM1_OK");
			break;
		case 23:
			strErr = new String("DATA_TRASMISSION_ERROR");
			break;
		case 17:
			strErr = new String("BUFFER_TOO_SMALL");
			break;
		default:
			strErr = new String("UNKNOWN_ERROR");
			break;
		}
		chInfos.error.setText("Error: " + strErr);
	}

	public void upDateQosInformations(long sessionId, long channelId, QoSData qos){ 
		SessionInformations sessInfos;
		ChannelInformationsRemote chInfos = new ChannelInformationsRemote();
		StringBuffer temp;
		sessInfos = ((SessionInformations)sessions.elementAt((int)sessionId));
		chInfos = (ChannelInformationsRemote)(sessInfos.channels.elementAt((int)channelId));
		Byte mbyte;

		//priority uptade
		temp = new StringBuffer(chInfos.qualifier.metrics.priority.getText());
		temp.setLength(10);
		temp.insert(10 , ((Qualifier)(qos.qualifiers.elementAt(0))).tagValue.toString());
		chInfos.qualifier.metrics.priority.setText(temp.toString());

		//maxDelay update
		temp = new StringBuffer(chInfos.qualifier.metrics.maxDelay.getText());
		temp.setLength(11);
		temp.insert(11 , ((Qualifier)(qos.qualifiers.elementAt(1))).tagValue.toString());
		chInfos.qualifier.metrics.maxDelay.setText(temp.toString());

		//avgDelay uptade
		temp = new StringBuffer(chInfos.qualifier.metrics.avgDelay.getText());
		temp.setLength(15);
		temp.insert(15 , ((Qualifier)(qos.qualifiers.elementAt(2))).tagValue.toString());
		chInfos.qualifier.metrics.avgDelay.setText(temp.toString());


		//lossProby uptade
		temp = new StringBuffer(chInfos.qualifier.metrics.lossProb.getText());
		temp.setLength(18);
		temp.insert(18 , ((Qualifier)(qos.qualifiers.elementAt(3))).tagValue.toString());
		chInfos.qualifier.metrics.lossProb.setText(temp.toString());

		//maxGapLoss uptade
		temp = new StringBuffer(chInfos.qualifier.metrics.maxGapLoss.getText());
		temp.setLength(14);
		temp.insert(14 , ((Qualifier)(qos.qualifiers.elementAt(4))).tagValue.toString());
		chInfos.qualifier.metrics.maxGapLoss.setText(temp.toString());

		//maxAuSize uptade
		temp = new StringBuffer(chInfos.qualifier.metrics.maxAuSize.getText());
		temp.setLength(13);
		temp.insert(13 , ((Qualifier)(qos.qualifiers.elementAt(5))).tagValue.toString());
		chInfos.qualifier.metrics.maxAuSize.setText(temp.toString());

		//maxAuRate uptade
		temp = new StringBuffer(chInfos.qualifier.metrics.maxAuRate.getText());
		temp.setLength(13);
		temp.insert(13 , ((Qualifier)(qos.qualifiers.elementAt(6))).tagValue.toString());
		chInfos.qualifier.metrics.maxAuRate.setText(temp.toString());

		//avgAuSize uptade
		temp = new StringBuffer(chInfos.qualifier.metrics.avgAuSize.getText());
		temp.setLength(17);
		temp.insert(17 , ((Qualifier)(qos.qualifiers.elementAt(7))).tagValue.toString());
		chInfos.qualifier.metrics.avgAuSize.setText(temp.toString());
	}
	private static String hostname;

	public static void mainStart(String host) 
	{
		hostname = new String(host); 
		monitorWindow = new ResourcesMonitorRemote("SOMMIT. IM1 MPEG-4 Player Resource Monitor Remote by Becchini & Silano (SIA SpA)");
		sessions = new Vector(1);
		monitorWindow.getContentPane().setLayout(new CardLayout());

		addSessionsPane();
		addChannelsPane(); 
		monitorWindow.setSize(600,300);
		monitorWindow.setVisible(true);
		monitorSourceRemote = new MonitorSourceRemote(monitorWindow);
		timer = new Timer(1000, monitorSourceRemote);		timer.setRepeats(true);
		timer.start();
	}

	public void actionPerformed(ActionEvent e){
		if(e.getActionCommand() == "Close"){
			timer.stop();
			monitorWindow.dispose();
		}
		if(e.getActionCommand() == "CHDET"){
		 //show channels details
			monitorWindow.setTitle("SOMMIT.IM1 MPEG-4 Player Resource Monitor by Becchini & Silano (SIA SpA) . Session " + currentSession);
			((CardLayout)monitorWindow.getContentPane().getLayout()).show(monitorWindow.getContentPane(), "Channels card");
		}
	

		if(e.getActionCommand() == "Back"){
		 //show sessions informations
			((CardLayout)monitorWindow.getContentPane().getLayout()).show(monitorWindow.getContentPane(), "Sessions card");
			monitorWindow.setTitle("SOMMIT.IM1 MPEG-4 Player Resource Monitor by Becchini & Silano (SIA SpA)");
		}
	}
}    