/********************************************************************
 This software module was originally developed by Luca Deri (Finsiel 
- I - SOMMIT Consortium) and edited by Roberto Becchini (SIA - I - 
SOMMIT Consortium), Barbara Silano (SIA - I - SOMMIT Consortium), 
Gianluca De Petris (CSELT - I - SOMMIT Consortium) in the course of 
development of the MPEG-4 Systems (ISO/IEC 14496-1). This software 
module is an implementation of part of one or more MPEG-4 Systems 
(ISO/IEC 14496-1) tools as specified by the MPEG-4 Systems (ISO/IEC 
14496-1). ISO/IEC gives users of the MPEG-4 Systems (ISO/IEC 14496-1) 
free license to this software module or modifications thereof for use 
in hardware or software products claiming conformance to the MPEG-4 
Systems (ISO/IEC 14496-1). Those intending to use this software 
module in hardware or software products are advised that its use may 
infringe existing patents. THE ORIGINAL DEVELOPER OF THIS SOFTWARE 
MODULE AND HIS/HER COMPANY, THE SUBSEQUENT EDITORS AND THEIR 
COMPANIES, AND ISO/IEC HAVE NO LIABILITY FOR USE OF THIS SOFTWARE 
MODULE OR MODIFICATIONS THEREOF. No license to this software module 
is granted for non MPEG-4 Systems (ISO/IEC 14496-1) conforming 
products. The SOMMIT Consortium retains full right to use the 
software module for his/her own purpose, assign or donate the 
software module to a third party and to inhibit third parties from 
using the code for non MPEG-4 Systems (ISO/IEC 14496-1) conforming 
products. This copyright notice must be included in all copies or 
derivative works of the software module. Copyright 1999
********************************************************************/

import java.util.*;
import java.awt.*;
import java.io.*;
import java.net.*;
import com.ice.jni.registry.*;
import javax.comm.*;

public class TestRegistry {
  
        
    public TestRegistry() {
    }
  
    /* ******************************************************************* */

    public static String getMouseType() {
        try {
            RegistryValue value = Registry.getTopLevelKey("HKEY_LOCAL_MACHINE").openSubKey("SOFTWARE").openSubKey("Microsoft").openSubKey("Windows NT").openSubKey("CurrentVersion").openSubKey("WOW").openSubKey("boot.description").getValue("mouse.drv");
            return(new String(value.getByteData()));
        } catch(Exception ex) {
            return(null);   
        }
    }
    
    /* ******************************************************************* */
   
    public static String getKeyboardType() {
        try {
            RegistryValue value = Registry.getTopLevelKey("HKEY_LOCAL_MACHINE").openSubKey("SOFTWARE").openSubKey("Microsoft").openSubKey("Windows NT").openSubKey("CurrentVersion").openSubKey("WOW").openSubKey("boot.description").getValue("keyboard.typ");
            return(new String(value.getByteData()));
        } catch(Exception ex) {
            return(null);   
        }
    }
   
    /* ******************************************************************* */
   
    public static String getNetworkType() {
        try {
            RegistryValue value = Registry.getTopLevelKey("HKEY_LOCAL_MACHINE").openSubKey("SOFTWARE").openSubKey("Microsoft").openSubKey("Windows NT").openSubKey("CurrentVersion").openSubKey("WOW").openSubKey("boot.description").getValue("network.drv");
            return(new String(value.getByteData()));
        } catch(Exception ex) {
            return(null);   
        }
    }
   
   /* ******************************************************************* */
    
    public static String getDisplayType() {
        try {
            RegistryValue value = Registry.getTopLevelKey("HKEY_LOCAL_MACHINE").openSubKey("SOFTWARE").openSubKey("Microsoft").openSubKey("Windows NT").openSubKey("CurrentVersion").openSubKey("WOW").openSubKey("boot.description").getValue("display.drv");
            return(new String(value.getByteData()));
        } catch(Exception ex) {
            return(null);   
        }
    }
   
   /* ******************************************************************* */
    
    public static String[] getAvailableDrivers(String[] matchingStrings) {
        try {
            String rsp[] = null;
            int idx;
            Enumeration key, val;
            Hashtable hashTable = new Hashtable();
            Stack s = new Stack();
            
            if(matchingStrings == null)
                return(rsp);
                
            RegistryKey rKey = Registry.getTopLevelKey("HKEY_LOCAL_MACHINE").openSubKey("SOFTWARE").openSubKey("Microsoft").openSubKey("Windows NT").openSubKey("CurrentVersion").openSubKey("Drivers32");
            
            for(val = rKey.valueElements(); val.hasMoreElements();) {
                String obj = (String)val.nextElement();

                for(int i=0; i<matchingStrings.length; i++)
                  if(obj.startsWith(matchingStrings[i])) {
                    hashTable.put(new String(rKey.getStringValue(obj)), new String(obj));
                    break;
                  }
            }
            
            rKey = Registry.getTopLevelKey("HKEY_LOCAL_MACHINE").openSubKey("SOFTWARE").openSubKey("Microsoft").openSubKey("Windows NT").openSubKey("CurrentVersion").openSubKey("drivers.desc");

            for(val = rKey.valueElements(); val.hasMoreElements();) {
                String obj = (String)val.nextElement();
                String value = rKey.getStringValue(obj);
                
                if((obj != null) && (hashTable.get(obj) != null)) {
                    s.push(new String(value));
                }
            }

            idx = 0;
            rsp = new String[s.size()];
            while(!s.isEmpty()) 
               rsp[idx++] = (String)s.pop();
                       
            return(rsp);
        } catch(Exception ex) {
            return(null);   
        }
    }

   /* ******************************************************************* */
    
    public static String[] getAvailableAudioEncDec() {
        String[] matchingStrings = { "msacm", "wave" };
        
        return(getAvailableDrivers(matchingStrings));
    }
   
   /* ******************************************************************* */
    
    public static String[] getAvailableVideoEncDec() {
        String[] matchingStrings = { "vidc." };
        
        return(getAvailableDrivers(matchingStrings));
    }
    
   /* ******************************************************************* */
    
    public static String[] getAvailableMIDIEncDec() {
        String[] matchingStrings = { "midi" };
        
        return(getAvailableDrivers(matchingStrings));
    }
   
   /* ******************************************************************* */
    
    public static int getNumCPUs() {
        try {
            RegistryKey rKey = Registry.getTopLevelKey("HKEY_LOCAL_MACHINE").openSubKey("HARDWARE").openSubKey("DESCRIPTION").openSubKey("System").openSubKey("CentralProcessor");
            return(rKey.getNumberSubkeys());
        } catch(Exception ex) {
            return(1);   
        }
    }
   
   /* ******************************************************************* */
    
    public static String getCPUType(int cpuIdx) {
        try {
            RegistryValue value = Registry.getTopLevelKey("HKEY_LOCAL_MACHINE").openSubKey("HARDWARE").openSubKey("DESCRIPTION").openSubKey("System").openSubKey("CentralProcessor").openSubKey(String.valueOf(cpuIdx)).getValue("Identifier");
            return(new String(value.getByteData()));
        } catch(Exception ex) {
            return(null);   
        }
    }
   

   /* ******************************************************************* */
    
    public static String getModemType() {
        try {
            RegistryValue value = Registry.getTopLevelKey("HKEY_LOCAL_MACHINE").openSubKey("System").openSubKey("ControlSet001").openSubKey("Services").openSubKey("Modem").openSubKey("Enum").getValue("0");
            StringTokenizer modemId = new StringTokenizer(new String(value.getByteData()), "\\");
            value = Registry.getTopLevelKey("HKEY_LOCAL_MACHINE").openSubKey("System").openSubKey("ControlSet001").openSubKey("Enum").openSubKey(modemId.nextToken()).openSubKey(modemId.nextToken()).openSubKey(modemId.nextToken()).getValue("FriendlyName");
            return(new String(value.getByteData()));
        } catch(Exception ex) {
            return(null);   
        }
    }
   
   /* ******************************************************************* */
    
    public static String getCPUSpeed(int cpuIdx) /* MHz */ {
        try {
            RegistryValue value = Registry.getTopLevelKey("HKEY_LOCAL_MACHINE").openSubKey("HARDWARE").openSubKey("DESCRIPTION").openSubKey("System").openSubKey("CentralProcessor").openSubKey(String.valueOf(cpuIdx)).getValue("~MHZ");
            byte speed[] = value.getByteData();
            
            return(String.valueOf(256+(int)speed[3]));
        } catch(Exception ex) {
            return(null);   
        }
    }
   
 /* ******************************************************************* */
  


    public static void main(String args[])
    {
        int idx;
	Enumeration enum;
	
	System.out.println("Display Type:      "+getDisplayType());
        System.out.println("Mouse Type:        "+getMouseType());
        System.out.println("Keyboard Type:     "+getKeyboardType());
        System.out.println("Network Type:      "+getNetworkType());
        
        int numCPUs = getNumCPUs();
        System.out.println("# CPUs:            "+numCPUs);
        for(idx=0; idx<numCPUs; idx++) {
            System.out.println("CPU Type  ["+(idx+1)+"]:     "+getCPUType(idx));
            System.out.println("CPU Speed ["+(idx+1)+"]:     "+getCPUSpeed(idx)+" MHz");
        }
    
        Dimension screenSize      = Toolkit.getDefaultToolkit().getScreenSize();
        System.out.println("Screen size:       "+screenSize.width+"x"+screenSize.height);
        /* -------------------------------------------------------------------------- */
        int       screenDepth     = Toolkit.getDefaultToolkit().getColorModel().getPixelSize();
        System.out.println("Screen depth:      "+screenDepth+" bits/pixel");
        /* -------------------------------------------------------------------------- */
        int      screenResolution = Toolkit.getDefaultToolkit().getScreenResolution();
        System.out.println("Screen resolution: "+screenResolution+" dots/inch");
        /* -------------------------------------------------------------------------- */
        long          totalMemory = Runtime.getRuntime().totalMemory();
        System.out.println("Total memory:      "+(totalMemory/1024)+" Kbytes");
        /* -------------------------------------------------------------------------- */
        long           freeMemory = Runtime.getRuntime().freeMemory();
        System.out.println("Free memory:       "+(freeMemory/1024)+" Kbytes");
        /* -------------------------------------------------------------------------- */
        Enumeration e = CommPortIdentifier.getPortIdentifiers();
        if(e == null)
            System.out.println("No serial/parallel ports found");
        else {
            idx = 0;
            
            while(e.hasMoreElements()) {
                CommPortIdentifier identifier = (CommPortIdentifier)e.nextElement();
                int portType = identifier.getPortType();
                String portTypeString;
                
                switch(portType) {
                    case CommPortIdentifier.PORT_SERIAL:
                        portTypeString = "RS-232 serial port";
                        break;
                    case CommPortIdentifier.PORT_PARALLEL:
                        portTypeString = "IEEE 1284 parallel port";
                        break;
                    default:
                        portTypeString = "Unknown type";
                }
                idx++;
                
               System.out.println("Port "+idx+":            "+identifier.getName()+" ["+portTypeString+"]");
            }
        }
        /* -------------------------------------------------------------------------- */
        System.out.println("OS Type:           "+System.getProperty("os.name")+" v."+System.getProperty("os.version"));
        /* -------------------------------------------------------------------------- */
        System.out.println("OS Architecture:   "+System.getProperty("os.arch"));
        /* -------------------------------------------------------------------------- */
        System.out.println("OS Language:       "+System.getProperty("user.region"));
        /* -------------------------------------------------------------------------- */
        String values[] = getAvailableAudioEncDec();
        System.out.println("Available Audio Enc/Dec: ");

        for(idx=0; idx<values.length; idx++)
            System.out.println("                   "+values[idx]+"");
        
        /* -------------------------------------------------------------------------- */
        values = getAvailableVideoEncDec();
        System.out.println("Available Video Enc/Dec: ");

        for(idx=0; idx<values.length; idx++)
            System.out.println("                   "+values[idx]+"");
        
        /* -------------------------------------------------------------------------- */
        values = getAvailableMIDIEncDec();
        System.out.println("Available MIDI Enc/Dec: ");

        for(idx=0; idx<values.length; idx++)
            System.out.println("                   "+values[idx]+"");
        
        /* -------------------------------------------------------------------------- */
        System.out.println("Modem Type:        "+getModemType());
        /* -------------------------------------------------------------------------- */

        /////////////////////////////////////////////////////////////

        System.exit(0);
    }


} // End class

