/***********************************************************************
This software module was originally developed by Zvi Lifshitz
(Triton R&D Ltd.) in the course of development of the MPEG-4 Systems
(ISO/IEC 14496-1) standard. This software module is an implementation of
a part of one or more MPEG-4 Systems (ISO/IEC 14496-1) tools as
specified by the MPEG-4 Systems (ISO/IEC 14496-1) standard. ISO/IEC
gives users of the MPEG-4 Systems (ISO/IEC 14496-1) free license to
this software module or modifications thereof for use in hardware or
software products claiming conformance to the MPEG-4 Systems (ISO/IEC
14496-1). Those intending to use this software module in hardware or
software products are advised that its use may infringe existing patents.
The original developer of this software module and his/her company, the
subsequent editors and their companies, and ISO/IEC have no liability
for use of this software module or modifications thereof in an
implementation. Copyright is not released for non MPEG-4 Systems
(ISO/IEC 14496-1) conforming products. Triton R&D Ltd. retains full right
to use the code for his/her own purpose, assign or donate the code to a
third party and to inhibit third parties from using the code for non
MPEG-4 Systems (ISO/IEC 14496-1) conforming products. This copyright
notice must be included in all copies or derivative works.
Copyright  1999.
************************************************************************/
import java.lang.reflect.*;
import java.util.*;
public class ClassDependencies {
	public static int MaxClasses = 250;

	public static ClassInfo getDependencies(Class currentClass, Class[] classArray,
											int currentCount, boolean addClass) {
		
		System.out.println("getDependencies called for " + currentClass.getName());
		if(currentClass.isPrimitive()) {
			System.out.println("Primitive Type: " + currentClass.getName());
			ClassInfo currentClassInfo = new ClassInfo();
			currentClassInfo.classArray = classArray;
			currentClassInfo.currentCount = currentCount;
			return currentClassInfo;
		}
		
		Class[] memberClassArray;
		Class[] methodParamClassArray;
		boolean first = true;

		ClassInfo currentClassInfo = new ClassInfo();

		currentClassInfo.classArray = classArray;
		currentClassInfo.currentCount = currentCount;
		
		while(checkIfNewClass(currentClass,currentClassInfo.classArray,
							  currentClassInfo.currentCount)) {
			if(first && addClass)
				currentClassInfo = addClassifNew(currentClass,currentClassInfo);
			
			memberClassArray = getMemberClasses(currentClass);
			
			
			for(int i=0;i<memberClassArray.length;i++){
				if(checkIfNewClass(memberClassArray[i], currentClassInfo.classArray,
								   currentClassInfo.currentCount)) {
					currentClassInfo = getDependencies(memberClassArray[i],
													   currentClassInfo.classArray,
													   currentClassInfo.currentCount, true);
					/*currentClassInfo = addClassifNew(memberClassArray[i],currentClassInfo); */
				}
			}

			methodParamClassArray = getMethodParamClasses(currentClass);

			for(int i=0;i<methodParamClassArray.length;i++){
				if(checkIfNewClass(methodParamClassArray[i], currentClassInfo.classArray,
								   currentClassInfo.currentCount)) {
					currentClassInfo = getDependencies(methodParamClassArray[i],
													   currentClassInfo.classArray,
													   currentClassInfo.currentCount,true);
					/*currentClassInfo = addClassifNew(methodParamClassArray[i],currentClassInfo); */
				}
			}
			
			currentClass = currentClass.getSuperclass();
			System.out.println("Got Super Class" + currentClass.getName());
			
			//currentClassInfo = addClassifNew(currentClass,currentClassInfo);      
			currentClassInfo = getDependencies(currentClass,
											   currentClassInfo.classArray,
											   currentClassInfo.currentCount,true); 
			first = false;
		}
		
		System.out.println("GetDepends Returning: " + currentClassInfo.currentCount);
		return(currentClassInfo);
	}

	public static boolean checkIfNewClass(Class currentClass,Class[] classArray,
										  int currentCount){
		boolean retval = false;
		if(!isSystemClass(currentClass)){
			for(int i=0;i<currentCount;i++) {
				if(currentClass.getName().equals(classArray[i].getName()))
					return(false);
			}
			retval = true;
		}
		return(retval);
	}

	public static boolean isSystemClass(Class currentClass) {
		/* From Previous */
		boolean retval = false;
		
		if(currentClass.getName().startsWith("java") || 
		   currentClass.getName().startsWith("Java") || 
		   currentClass.getName().startsWith("com") ||
		   currentClass.getName().startsWith("Com"))
			retval = true;
		return(retval);
	}
	
	public static Class[] getMemberClasses(Class currentClass) {
		Class[] MemberClassArray;
		Field[] MemberFields = currentClass.getDeclaredFields();
		int count=0;
		int[] index = new int[MaxClasses];
		Class[] MemberClasses;
		for (int i = 0; i < MemberFields.length; i++) {
			String fieldName = MemberFields[i].getName();
			Class typeClass = MemberFields[i].getType();
			String fieldType;
			
			if (typeClass.isArray()){
				typeClass = typeClass.getComponentType();
			}
			fieldType = typeClass.getName();
			if(!typeClass.isPrimitive()){
				System.out.println("Name: " + fieldName + 
								   ", Type: " + fieldType);
				index[count++] = i;
			}
		}
		MemberClasses = new Class[count];
		for(int i=0;i<count;i++){
			MemberClasses[i] = MemberFields[i].getType();
			if(MemberClasses[i].isArray())
				MemberClasses[i] = MemberClasses[i].getComponentType();
		}
		return(MemberClasses);
	}

	public static Class[] getMethodParamClasses(Class currentClass) {
		Method[] methodArray = currentClass.getDeclaredMethods();

		Class[] paramsTypeClass;
		Class[] methodClasses;
		Vector methodClassVec = new Vector();

		String fieldType;

		for(int i=0;i<methodArray.length;i++){
			paramsTypeClass = methodArray[i].getParameterTypes();
			for(int j=0;j<paramsTypeClass.length;j++) {
				if (paramsTypeClass[j].isArray()){
					paramsTypeClass[j] = paramsTypeClass[j].getComponentType();
				}
				
				fieldType = paramsTypeClass[j].getName();
				if(!paramsTypeClass[j].isPrimitive()){
					System.out.println("Name: " + methodArray[i].getName() + 
									   ", Type: " + fieldType);
					methodClassVec.addElement(paramsTypeClass[j]);
				}
			}
		}
		methodClasses = new Class[methodClassVec.size()];
		for(int i = 0;i<methodClassVec.size();i++)
			methodClasses[i] = (Class) methodClassVec.elementAt(i);
		return(methodClasses);
	}
	public static ClassInfo addClassifNew(Class currentClass, ClassInfo classInfo) {

		if(checkIfNewClass(currentClass,classInfo.classArray,classInfo.currentCount))
			if(classInfo.currentCount < MaxClasses){
				classInfo.classArray[classInfo.currentCount++] = currentClass;
				System.out.println("Adding Class " + currentClass.getName());
				System.out.println("at position : " + (classInfo.currentCount-1));
			}
			else {
				System.out.println("Max Classes Reached");
			}
		return(classInfo);
		
	}

	public static void main(String[] args) {
		/*   Tes tes = new Tes();
		ClassDependencies cd = new ClassDependencies();

		Class[] classArray = new Class[MaxClasses];
		int currentCount=0;

		ClassInfo classInfo = new ClassInfo();
		classInfo.classArray = classArray;
		classInfo.currentCount = currentCount;
		
		Class c = tes.getClass();

		classInfo = ClassDependencies.getDependencies(c,classArray,currentCount,false);
		ClassDependencies.printClassInfo(classInfo);    

		c = cd.getClass();
		classArray = new Class[MaxClasses];
		currentCount=0;
		classInfo.classArray = classArray;
		classInfo.currentCount = currentCount;
		
		classInfo = ClassDependencies.getDependencies(c,classArray,currentCount,false);
		ClassDependencies.printClassInfo(classInfo);*/
	}
	
	public static void printClassInfo(ClassInfo classInfo) {
		System.out.println("Printing Class Dependencies of : " +  classInfo.currentCount);
		System.out.println("***********************************************************");
		for(int i=0;i<classInfo.currentCount;i++) {
			System.out.println("["+ i +"]\t" + classInfo.classArray[i].getName());
		}
	}
}

