/***********************************************************************
This software module was originally developed by Zvi Lifshitz
(Triton R&D Ltd.) in the course of development of the MPEG-4 Systems
(ISO/IEC 14496-1) standard. This software module is an implementation of
a part of one or more MPEG-4 Systems (ISO/IEC 14496-1) tools as
specified by the MPEG-4 Systems (ISO/IEC 14496-1) standard. ISO/IEC
gives users of the MPEG-4 Systems (ISO/IEC 14496-1) free license to
this software module or modifications thereof for use in hardware or
software products claiming conformance to the MPEG-4 Systems (ISO/IEC
14496-1). Those intending to use this software module in hardware or
software products are advised that its use may infringe existing patents.
The original developer of this software module and his/her company, the
subsequent editors and their companies, and ISO/IEC have no liability
for use of this software module or modifications thereof in an
implementation. Copyright is not released for non MPEG-4 Systems
(ISO/IEC 14496-1) conforming products. Triton R&D Ltd. retains full right
to use the code for his/her own purpose, assign or donate the code to a
third party and to inhibit third parties from using the code for non
MPEG-4 Systems (ISO/IEC 14496-1) conforming products. This copyright
notice must be included in all copies or derivative works.
Copyright  1999.
************************************************************************/
import java.io.*;
import java.lang.*;

/** Abstracts the JavaClassId structure in the specification. It 
has accessor mutator functions to the fields. This can be used to
retrieve values from a byte array or create the byte array from the
values */
public class ClassId {

	/** Length of the buffer in bytes. This should be a multiple of 4 (32 bits) */
	private int length;
	
	/** This is the byte array which holds the length and the class name 
	aligned to a 32 bit word */
	private byte[] buffer;

	/** Name of the class of interest */
	private String className;
	
	/** Gets the Length as an Integer */
	public int getLength()
	{
		return length;
	}
	/** Gets the Name as a String*/
	public String getName()
	{
		return className;
	}
	
	public byte[] getBuffer()  
	{
		return buffer;
	}
	public int getBufferLength() 
	{
		return buffer.length;
	}
	
	/** Gets the Length from the Buffer and sets the attribute length.*/
	public void getLengthFromBuffer()
	{
		length = (((buffer[0] & 0xff) << 8)  | (buffer[1] & 0xff));
		length  = (int)(java.lang.Math.ceil((double)(length+2)/4.0))* 4;    
	}
	
	public void writeToBuf(byte[] b, int offset)
	{
		System.arraycopy(buffer,0,b,offset,buffer.length);
	}
	
	/** Gets the Name from the byte array buffer and 
	sets the attribute className.*/
	public void getNameFromBuffer()
	{
		int i;
		char[] tmpchar = new char[length];
		for(i=2;i<length;i++)
		{
			if(buffer[i] == (byte) 0xFF)
				break;
			tmpchar[i-2] = (char) (/*0x0 |*/buffer[i]);
		}
		className = new String(tmpchar,0,i-2);
	}

	/** Creates the byte[] buffer (buffer) according the 
	specification. This also sets the className and length attributes*/
	public void createBuffer()
	{
		if (className == null)  {
			length = 0;
			// aligned to 32 bits
			buffer = new byte[4];
			buffer[0] = 0x00;
			buffer[1] = 0x00;
			buffer[3] = 0x00;
			buffer[4] = 0x00;
			return;
		}
		
		int strlen = className.length();
		length  = (int)(java.lang.Math.ceil((double)(strlen+2)/4.0))*4;
	
		buffer = new byte[2 + length];
		buffer[0] = (byte) ((strlen >> 8) & 0xFF);
		buffer[1] = (byte) (strlen & 0xFF);
		
		for(int i=2;i<strlen;i++)
			buffer[i] = (byte) className.charAt(i);
		for(int i=strlen;i<length;i++)	
			buffer[i] = (byte)  0xff;
	}

	/** Constructor which takes the className as the string 
	and creates the buffer. */

	public ClassId(String className)
	{
		this.className = className;
		createBuffer();
	}

	/** Constructor which takes the byte array buffer as an argument 
	and retrieves and the className and length from it */
	public ClassId(byte buf[], int offset)
	{
		length = ((buf[0+offset] & 0xFF) <<8)| ((buf[1+offset] & 0xFF));
		length = (int) java.lang.Math.ceil((double)(length+2)/4.0)*4; /* for 32 bit alignment */

		System.out.println("Length is " + length);
		buffer = new byte[length];
		for(int i=0;i<length;i++) 
			buffer[i] = (byte) (buf[i+offset] & 0xFF); 

		getLengthFromBuffer();
		getNameFromBuffer();
	}

	/** Test main */
	public static void main(String[] args)
	{

		ClassId myJCId = new ClassId("hello");
		ClassId myJCId1 = new ClassId(myJCId.getBuffer(),0);
		System.out.println("Java Class Id of length - " + 
			myJCId.getLength() + " and \nname - " + 
			myJCId.getName() + " is saved as string - " +
			myJCId.getBuffer());
	}
}