/***********************************************************************
This software module was originally developed by Zvi Lifshitz
(Triton R&D Ltd.) in the course of development of the MPEG-4 Systems
(ISO/IEC 14496-1) standard. This software module is an implementation of
a part of one or more MPEG-4 Systems (ISO/IEC 14496-1) tools as
specified by the MPEG-4 Systems (ISO/IEC 14496-1) standard. ISO/IEC
gives users of the MPEG-4 Systems (ISO/IEC 14496-1) free license to
this software module or modifications thereof for use in hardware or
software products claiming conformance to the MPEG-4 Systems (ISO/IEC
14496-1). Those intending to use this software module in hardware or
software products are advised that its use may infringe existing patents.
The original developer of this software module and his/her company, the
subsequent editors and their companies, and ISO/IEC have no liability
for use of this software module or modifications thereof in an
implementation. Copyright is not released for non MPEG-4 Systems
(ISO/IEC 14496-1) conforming products. Triton R&D Ltd. retains full right
to use the code for his/her own purpose, assign or donate the code to a
third party and to inhibit third parties from using the code for non
MPEG-4 Systems (ISO/IEC 14496-1) conforming products. This copyright
notice must be included in all copies or derivative works.
Copyright  1999.
************************************************************************/
import java.io.*;
import java.util.zip.*;
import java.util.*;

public class JavaES 
{
	private String outFileName;
	private long lDTS;
	private boolean bZip, bDir;
	private File inPath;
	private String infoFileName;
	private boolean bClassFileOnly;
	private PrintWriter info_out;
	
	public static void main(String args[]) {
		if (args.length != 4) {
			System.out.println("Usage: JavaES <input path> <output file name> <info file name> <DTS>");
			System.out.println("input path: input directory or file name");
			System.out.println("output file name: name of output file");
			System.out.println("DTS: target DTS value");
			//System.out.println("1: zip compressed | 0: non-compressed");
			//System.out.println("1: class file only | 0");
			System.exit(-1);
		} 
		boolean bZip = true;//Integer.parseInt(args[4]) == 1;
		boolean bClassFileOnly = true;//Integer.parseInt(args[5]) == 1;
		(new JavaES(args[0],args[1],args[2],Long.parseLong(args[3]),bZip, bClassFileOnly)).make();
	}
	
	public void infoWrite(long length, long lDTS) throws IOException {
		info_out.print(length);
		info_out.print(' '); 
		info_out.print(lDTS);
		info_out.print("\r\n");
	}
	
	public JavaES(String inPathName, String outFileName, String infoFileName, long lDTS, 
						boolean bZip, boolean bClassFileOnly) {
		this.outFileName = outFileName;
		this.lDTS = lDTS;
		this.bZip = bZip;
		this.inPath = new File(inPathName);
		this.infoFileName = infoFileName;
		this.bClassFileOnly = bClassFileOnly;
			
		if (!inPath.exists() || !inPath.canRead()) {
			System.out.println("Error: Invalid path " + inPathName);
			System.exit(-1);
		}
		this.bDir = inPath.isDirectory();
		
	}
	
	public void make() {
		try
		{
			info_out = new PrintWriter(new BufferedWriter(new FileWriter(infoFileName)));
			DataOutputStream out = new DataOutputStream(new BufferedOutputStream(new FileOutputStream(outFileName)));
			String [] files = null;
			if(bDir)
			{
				if(bClassFileOnly) {
					files = inPath.list(new FilenameFilter() {
						public boolean accept(File dir, String name) {
							if(name.endsWith(".class")) 
								return true;
							else 
								return false;
						}
					});
				}
				else
					files = inPath.list();
				if(files.length == 0)
				{
					System.out.println("Error: Empty Direcotry " + inPath.getName());
					System.exit(-1);
				}
			}

			if(bZip)
			{
				boolean bContainClass = false;
				// make a zip archive with the files in the specified directory
				if(bDir)
				{
					// check the directory contains a class file
					for(int i=files.length-1;i>=0;i--)
						if(files[i].endsWith(".class"))
							bContainClass = true;
					if(bClassFileOnly)
						System.out.println("Making zip archive with class files in " + inPath.getName());
					else
						System.out.println("Making zip archive with all the files in " + inPath.getName());
					
					// make zipped byte array
					byte [] payload = makeZipByteArray(inPath,files);
					JavaStreamHeader jsh = new JavaStreamHeader(bContainClass,JavaStreamHeader.COMPRESSION);
					jsh.putHeader(out);
					out.write(payload);
					System.out.println("Header length: " + jsh.getHeaderLength() + ", payload length: " + payload.length);
					infoWrite(jsh.getHeaderLength()+payload.length, lDTS);
				}
				// make JavaStreamHeader with a given zip file
				else
				{
					if(!inPath.getName().endsWith(".zip"))
					{
						System.out.println("Error: Invalid extension(" + inPath.getName() + ")");
						System.exit(-1);
					}
					System.out.println("Making Java ES with " + inPath.getName());
					
					ZipFile zFile = new ZipFile(inPath);
					for (Enumeration e = zFile.entries() ; e.hasMoreElements() ;)
					{
						if(((ZipEntry)e.nextElement()).getName().endsWith(".class"))
						{
							System.out.println("Zip Archive " + inPath.getName() + " contains class file");
							bContainClass = true;
						}
					}
					JavaStreamHeader jsh = new JavaStreamHeader(bContainClass,JavaStreamHeader.COMPRESSION);
					jsh.putHeader(out);

					int len;
					int tot = 0;
					byte[] buf = new byte[1024];

						DataInputStream in = new DataInputStream(new BufferedInputStream(new FileInputStream(inPath.getName())));
					while ((len = in.read(buf)) != -1)
					{
						out.write(buf,0,len);
						tot += len;
					}
					infoWrite(jsh.getHeaderLength()+tot, lDTS);
					in.close();
				}
			}
			else
			{
				if(bDir)
				{
					if(bClassFileOnly)
						System.out.println("Making Java ES with all the class files in " + inPath.getName());
					else
						System.out.println("Making Java ES with all the files in " + inPath.getName());
						
					long[] lDTSA = new long[files.length];
					files = sortAU(files,lDTSA,lDTS);
					for(int i=files.length-1;i>=0;i--)
					{
						putOneFile(new File(inPath,files[i]),out,lDTSA[i]);
					}
				}
				else
				{
					System.out.println("Making Java ES with the class file " + inPath.getName());
					if(bClassFileOnly && !inPath.getName().endsWith(".class")) {
						System.out.println("Error: " + inPath.getName() + " is not a class file");
						System.exit(-1);
					}
					putOneFile(inPath,out,lDTS);	
				}
			}
			out.close();
			info_out.close();

		}
		catch(IOException e) {
			System.out.println("IOException: " + e.toString());
			e.printStackTrace();
			System.exit(-1);
		}
		catch(Exception e) {
			System.out.println("Exception: " + e.toString());
			e.printStackTrace();
			System.exit(-1);
		}
	}
	
	public void putOneFile(File _inputPath, DataOutputStream out, long _lDTS) throws IOException {
		boolean bClass = false;
		if(_inputPath.getName().endsWith(".class"))
			bClass = true;
		
		System.out.println("Making JavaStreamHeader for " + _inputPath.getName());
		JavaStreamHeader jsh = new JavaStreamHeader(bClass,_inputPath.getName());
		jsh.putHeader(out);
		
		DataInputStream in = new DataInputStream(new BufferedInputStream(new FileInputStream(_inputPath)));
		int len;
		int tot = 0;
		byte[] buf = new byte[1024];
		while ((len = in.read(buf)) != -1)  {
			out.write(buf,0,len);
			tot += len;
		}
		infoWrite(jsh.getHeaderLength()+tot, _lDTS);
		in.close();
	}
	
	public String[] sortAU(String[] classFiles,long[] lDTSA, long lDTS) {
		for(int i=lDTSA.length-1;i>=0;i--) {
			lDTSA[i] = lDTS;
		}
		return classFiles;	
	}
	
	public byte[] makeZipByteArray(File dir,String [] classFiles) {
		ByteArrayOutputStream temp = new ByteArrayOutputStream();
		try {
			ZipOutputStream out = new ZipOutputStream(new BufferedOutputStream(temp));
			//out.setComment("MPEGJ ZIP Archive");
			//out.setLevel(Deflater.DEFAULT_COMPRESSION);
			//out.setMethod(ZipOutputStream.DEFLATED);
			
			int len, total;
			byte[] buf = new byte[1024];
			
			DataInputStream in = null;
			for(int i=classFiles.length-1;i>=0;i--) {
				System.out.println("Putting " + classFiles[i] + " into the zip archive");
				in = new DataInputStream(new BufferedInputStream(new FileInputStream(new File(dir,classFiles[i]))));
				out.putNextEntry(new ZipEntry(classFiles[i]));
				while ((len = in.read(buf)) != -1)  {
					out.write(buf,0,len);
				}
				out.closeEntry();	
				in.close();
			}
			out.close();
			
		} catch(Exception e) {
			e.toString();
			System.exit(-1);
		}
		
		return temp.toByteArray();
	}
	
}
