/***********************************************************************
This software module was originally developed by Zvi Lifshitz
(Triton R&D Ltd.) in the course of development of the MPEG-4 Systems
(ISO/IEC 14496-1) standard. This software module is an implementation of
a part of one or more MPEG-4 Systems (ISO/IEC 14496-1) tools as
specified by the MPEG-4 Systems (ISO/IEC 14496-1) standard. ISO/IEC
gives users of the MPEG-4 Systems (ISO/IEC 14496-1) free license to
this software module or modifications thereof for use in hardware or
software products claiming conformance to the MPEG-4 Systems (ISO/IEC
14496-1). Those intending to use this software module in hardware or
software products are advised that its use may infringe existing patents.
The original developer of this software module and his/her company, the
subsequent editors and their companies, and ISO/IEC have no liability
for use of this software module or modifications thereof in an
implementation. Copyright is not released for non MPEG-4 Systems
(ISO/IEC 14496-1) conforming products. Triton R&D Ltd. retains full right
to use the code for his/her own purpose, assign or donate the code to a
third party and to inhibit third parties from using the code for non
MPEG-4 Systems (ISO/IEC 14496-1) conforming products. This copyright
notice must be included in all copies or derivative works.
Copyright  1999.
************************************************************************/
import java.io.*;
import java.lang.Class;
import java.lang.Class.*;


public class JavaStreamHeader {
	
	private static int VERSION = 0;
	private static int RESERVED = 0xFFF;
	
	public static int NOCOMPRESSION = 0x000;
	public static int COMPRESSION = 0x001;
	
	private int version = VERSION;
	private boolean isClassFlag;
	private int numReqClasses;
	private boolean isPackaged;
	private int compressionScheme;
	private int reserved = RESERVED;
	private ClassId classID;
	private ClassId[] reqClassId;
	private byte[] JSH;
	
	private long nHeaderSize = 4;
		
	private JavaStreamHeader(boolean bZip, boolean bClass) {
		this.isPackaged = bZip;
		this.isClassFlag= bClass;
	}
	
	// zip archive
	JavaStreamHeader(boolean bClass, int compressionScheme) {
		this(true,bClass);
		this.classID = new ClassId(null);
		this.compressionScheme = compressionScheme;
		reqClassId = null;
		numReqClasses = 0;
		nHeaderSize += classID.getBufferLength();	
	}
	
	// single file
	JavaStreamHeader(boolean bClass, String fileName) {
		this(false,bClass);
		
		
		this.compressionScheme = 0;
		if(bClass) {
			String className = fileName.substring(0,fileName.lastIndexOf(".class"));
			this.classID = new ClassId(className);
			reqClassId = evaluateRequiredClasses(className);
			if(reqClassId != null)
				numReqClasses = reqClassId.length;
			else
				numReqClasses = 0;
				
			for(int i=0;i<numReqClasses;i++) 
				nHeaderSize += reqClassId[i].getBufferLength();
		}
		else {
			this.classID = new ClassId(fileName);
			reqClassId = null;
			numReqClasses = 0;
		}
		nHeaderSize += classID.getBufferLength();
		
	}
	
	private void putHeaderInBuffer()
	{
		JSH = new byte[(int)nHeaderSize]; 
				
		int tmpint=0,writtenBytes;
		byte tmpByte = 0;
		
		version = version & 0x3;
		numReqClasses = numReqClasses & 0x1FFF;
		tmpint = (numReqClasses >> 8);

		if(isClassFlag)
			tmpByte = (byte) ((version << 6) | (1<<5)  | (tmpint));
		else 
			tmpByte = (byte) ((version << 6) | (0 << 5) | (tmpint));

		JSH[0] = tmpByte; 
		tmpByte = (byte) (numReqClasses & 0xFF); 
		JSH[1] = tmpByte;

		tmpByte = 0x00;
		tmpint =0;
		compressionScheme = compressionScheme & 0x7;
		reserved = reserved & 0xFFF;
		if(isPackaged) {
			tmpByte = (byte) ((0x1 <<7 ) | ((compressionScheme << 4)  & 0x70) | 
							  ((reserved >> 8) & 0xF));
		}
		else 
			tmpByte = (byte) ((0x0<<7) | 
							  ((compressionScheme << 4)  & 0x70) | 
							  ((reserved >> 8) & 0xF));

		JSH[2] = (byte) (tmpByte & 0xff);
		tmpByte = 0x0;
		tmpByte = (byte) (reserved & 0xff);
		JSH[3] = tmpByte;

		int offset = 4;
		classID.writeToBuf(JSH,offset);
		offset += classID.getBufferLength();
		
		for(int j=0;j<numReqClasses;j++)  {
			reqClassId[j].writeToBuf(JSH,offset);
			offset += reqClassId[j].getBufferLength();
		}
	}
		
	private ClassId[] evaluateRequiredClasses(String payloadName){
		return null;
	}
	
	public void putHeader(DataOutputStream out) throws IOException {
		putHeaderInBuffer();
		out.write(JSH);
	}
	
	public long getHeaderLength() {
		return nHeaderSize;
	}
}






