/***********************************************************************
This software module was originally developed by Zvi Lifshitz
(Triton R&D Ltd.) in the course of development of the MPEG-4 Systems
(ISO/IEC 14496-1) standard. This software module is an implementation of
a part of one or more MPEG-4 Systems (ISO/IEC 14496-1) tools as
specified by the MPEG-4 Systems (ISO/IEC 14496-1) standard. ISO/IEC
gives users of the MPEG-4 Systems (ISO/IEC 14496-1) free license to
this software module or modifications thereof for use in hardware or
software products claiming conformance to the MPEG-4 Systems (ISO/IEC
14496-1). Those intending to use this software module in hardware or
software products are advised that its use may infringe existing patents.
The original developer of this software module and his/her company, the
subsequent editors and their companies, and ISO/IEC have no liability
for use of this software module or modifications thereof in an
implementation. Copyright is not released for non MPEG-4 Systems
(ISO/IEC 14496-1) conforming products. Triton R&D Ltd. retains full right
to use the code for his/her own purpose, assign or donate the code to a
third party and to inhibit third parties from using the code for non
MPEG-4 Systems (ISO/IEC 14496-1) conforming products. This copyright
notice must be included in all copies or derivative works.
Copyright  1998.
************************************************************************

For more information or to receive an up to date version of this module,
you may call Zvi Lifshitz at zvil@csi.com.
----------------------------------------------------------------------*/

/*-------------------------------------------------------------------------
Project Im1Player
INTERNATIONAL ORGANIZATION FOR STANDARDIZATION
ISO/IEC JTC1/SC29/WG11

SUBSYSTEM:  BIFS Encoder
FILE:       EncField.cpp
AUTHOR:     Zvi Lifshitz (Triton R&D Ltd.)
OVERVIEW:   Encoding of a NodeField object
-------------------------------------------------------------------------*/
#include "defs.h"
#include "NodeFld.h"
#include "Quantize.h"
#include "Token.h"
#include "CondEnc.h"
#include "Conditional.h"
#include "NodeCode.h"

/*-----------------------------------------------------------------
Encode an SFInt
-----------------------------------------------------------------*/
BOOL SF<int> :: Encode (BIFSEncoder *pBifs,  Quantizer *pQuant)
{
	/*-----------------------------------------------------------------
	Look at the Quantizer to see if min-max values applied
	-----------------------------------------------------------------*/
	m_value = nextToken.GetInt ();
	if (!pQuant || !pQuant -> IsActive()) 
		pBifs -> EncodeInt (m_value, 32, "value");
	else
		pQuant -> EncodeInt (pBifs, m_value);
	return TRUE;
}

/*-----------------------------------------------------------------
Encode an SFBool
-----------------------------------------------------------------*/
BOOL SF<Bool> :: Encode (BIFSEncoder *pBifs, Quantizer * /*pQuant*/)
{
	m_value = nextToken.GetBool ();
	pBifs -> EncodeInt (m_value, 1, "value");
	return TRUE;
}

/*-----------------------------------------------------------------
Encode and SFFloat
MBS: modified to take into account quantization parameters if any
-----------------------------------------------------------------*/
BOOL SF<float> :: Encode (BIFSEncoder *pBifs,  Quantizer * pQuant)
{
	m_value = nextToken.GetReal ();
	if (!pQuant || !pQuant->IsActive()) 
	        pBifs -> EncodeFloat (m_value, "value");
	else 
		pQuant -> EncodeFloat(pBifs,m_value);
	return TRUE;
}

/*-----------------------------------------------------------------
Encode an SFTime
-----------------------------------------------------------------*/
BOOL SF<Time> :: Encode (BIFSEncoder *pBifs,  Quantizer * /*pQuant*/)
{
	m_value = nextToken.GetReal ();
	pBifs -> EncodeDouble (m_value, "value");
	return TRUE;
}

/*-----------------------------------------------------------------
Encode an SFVec3f
MBS: modified to take into account quantization parameters if any
-----------------------------------------------------------------*/
BOOL SF<Vec3f> :: Encode (BIFSEncoder *pBifs, Quantizer * pQuant)
{
  m_value.x = nextToken.GetReal ();
  m_value.y = nextToken.GetReal ();
  m_value.z = nextToken.GetReal ();
  if(!pQuant || !pQuant->IsActive()) {
    pBifs -> EncodeFloat (m_value.x, "x");
    pBifs -> EncodeFloat (m_value.y, "y");
    pBifs -> EncodeFloat (m_value.z, "z");
  }
  else {
    pQuant -> EncodeVec3f (pBifs,m_value);
  }
  return TRUE;
}

/*-----------------------------------------------------------------
Encode an SFVec2f
MBS: modified to take into account quantization parameters if any
-----------------------------------------------------------------*/
BOOL SF<Vec2f> :: Encode (BIFSEncoder *pBifs, Quantizer * pQuant)
{
  m_value.x = nextToken.GetReal ();
  m_value.y = nextToken.GetReal ();
  if(!pQuant || !pQuant->IsActive()) {
    pBifs -> EncodeFloat (m_value.x, "x");
    pBifs -> EncodeFloat (m_value.y, "y");
  }
  else {
    pQuant -> EncodeVec2f(pBifs,m_value);
  }
  return TRUE;
}

/*-----------------------------------------------------------------
Encode an SFRotation
MBS: modified to take into account quantization parameters if any
-----------------------------------------------------------------*/
BOOL SF<Rotation> :: Encode (BIFSEncoder *pBifs, Quantizer * pQuant)
{
  m_value.x = nextToken.GetReal ();
  m_value.y = nextToken.GetReal ();
  m_value.z = nextToken.GetReal ();
  m_value.theta = nextToken.GetReal ();
  if(!pQuant || !pQuant->IsActive()) {
    pBifs -> EncodeFloat (m_value.x, "x");
    pBifs -> EncodeFloat (m_value.y, "y");
    pBifs -> EncodeFloat (m_value.z, "z");
    pBifs -> EncodeFloat (m_value.theta, "theta");
  }
  else {
    pQuant -> EncodeRotation (pBifs,m_value);
  }

  return TRUE;
}

/*-----------------------------------------------------------------
Encode an SFRotation2D
MBS: modified to take into account quantization parameters if any
-----------------------------------------------------------------*/
BOOL SF<Rotation2D> :: Encode (BIFSEncoder *pBifs, Quantizer * pQuant)
{
  m_value.x = nextToken.GetReal ();
  m_value.y = nextToken.GetReal ();
  m_value.theta = nextToken.GetReal ();
  if(!pQuant || !pQuant->IsActive()) {
    pBifs -> EncodeFloat (m_value.x, "x");
    pBifs -> EncodeFloat (m_value.y, "y");
    pBifs -> EncodeFloat (m_value.theta, "theta");
  }
  else {
    pQuant -> EncodeRotation2D(pBifs,m_value);
  }
  return TRUE;
}

/*-----------------------------------------------------------------
Encode an SFColor
MBS: modified to take into account quantization parameters if any
-----------------------------------------------------------------*/
BOOL SF<Color3f> :: Encode (BIFSEncoder *pBifs, Quantizer * pQuant)
{
  m_value.r = nextToken.GetReal ();
  m_value.g = nextToken.GetReal ();
  m_value.b = nextToken.GetReal ();
  if(!pQuant || !pQuant->IsActive()) {
    pBifs -> EncodeFloat (m_value.r, "r");
    pBifs -> EncodeFloat (m_value.g, "g");
    pBifs -> EncodeFloat (m_value.b, "b");
  }
  else {
    pQuant->EncodeColor3f(pBifs,m_value);
  }

  return TRUE;
}

/*-----------------------------------------------------------------
Need to define empty SF<ZString> :: Encode function
-----------------------------------------------------------------*/
BOOL SF<ZString> :: Encode (BIFSEncoder *pBifs, Quantizer *)
{
	m_value = nextToken.GetString ();
	nextToken.Scan ();
	((SFString *) this) -> EncodeString (pBifs);
	return TRUE;
}

/*-----------------------------------------------------------------
Write the bits of the string after it has been parsed
MBS 110698: updated for UTF-8 encoding 
WARNING: we suppose text file uses ISO-Latin1 character set from 0x00 to 0xFF
Zvi 021799: Undo this thing, because UTF-8 is already handled in
Token::Scan.
-----------------------------------------------------------------*/
void SFString :: EncodeString (BIFSEncoder *pBifs)
{
	pBifs ->EncodeSFString (m_value);
}

/*-----------------------------------------------------------------
Encode SFUrl
-----------------------------------------------------------------*/
BOOL SF<Url> :: Encode (BIFSEncoder *pBifs, Quantizer * /*pQuant*/)
{
	BOOL isOD = nextToken == TID_INTEGER;
	pBifs -> EncodeInt (isOD, 1, "isOD");
	if (isOD)
	{	m_value.ODid = nextToken.GetInt ();
		pBifs -> EncodeInt (m_value.ODid, 10, "ODid");
	}
	else
	{	m_value.ODid = -1;
		m_value.urlValue.Encode (pBifs);
	}
	return TRUE;
}
		
/*-----------------------------------------------------------------
Encode SFImage
-----------------------------------------------------------------*/
BOOL SF<Image> :: Encode (BIFSEncoder *pBifs, Quantizer * /*pQuant*/)
{
	m_value.width = nextToken.GetInt ();
	m_value.height = nextToken.GetInt ();
	m_value.numComponents = nextToken.GetInt ();
	//Alvar 981223: Height and width are 12 bit encoded (10->12)
	pBifs -> EncodeInt (m_value.width, 12, "width");
	pBifs -> EncodeInt (m_value.height,12, "height");
	pBifs -> EncodeInt (m_value.numComponents, 2, "numComponents");

	/*-----------------------------------------------------------------
	The image is taken from a file
	-----------------------------------------------------------------*/
	LPCSTR szFileName = nextToken.GetString ();
	FILE *fImage = fopen (szFileName, "rb");
	if (!fImage)
		nextToken.Error ("File not found");
	else
	{	for (
			//Alvar 981223: numComponents is in bytes (no *8)
			int i = m_value.width * m_value.height * m_value.numComponents;
			--i >= 0;
		)
		{
			pBifs -> EncodeInt (fgetc (fImage), 8, "pixel");
		}
		fclose (fImage);
	}

	nextToken.Scan ();
	return TRUE;
}

/*-----------------------------------------------------------------
A dummy function
-----------------------------------------------------------------*/
BOOL SF<MediaObject *> :: Encode (BIFSEncoder *, Quantizer *)
{
	return FALSE;
}

/*-----------------------------------------------------------------
SFGenericNode Destructor.  Deletes the node (unless it's a proto)
-----------------------------------------------------------------*/
SFGenericNode :: ~SFGenericNode ()
{
	if (m_pNode && m_pNode ->GetNodeCode () >= 0)
		delete m_pNode;
	m_pNode = NULL;
}

/*-----------------------------------------------------------------
Encode a generic node
-----------------------------------------------------------------*/
BOOL SFGenericNode :: Encode (BIFSEncoder *pBifs, Quantizer * /*pQuant*/)
{
	/*-----------------------------------------------------------------
	Check if "USE" case
	-----------------------------------------------------------------*/
	BOOL bIsReused = nextToken == TID_USE;

	if (bIsReused)
	{	pBifs -> EncodeInt (bIsReused, 1, "isReused");
		nextToken.Scan ();
		if (nextToken == TID_NAME)
		{	int nNodeID = nextToken.GetDefinedName ();
			if (nNodeID >= 0)
				pBifs -> EncodeInt (nNodeID, 10, "nodeID");
			else
				nextToken.Error ("Name not found");
			nextToken.Scan ();
			return TRUE;
		}
		else
		{	nextToken.Error ("Unexpected symbol");
			nextToken.Scan ();
			return FALSE;
		}
	}

	/*-----------------------------------------------------------------
	Check if "DEF" is used
	-----------------------------------------------------------------*/
	BOOL bIsUpdateable = nextToken == TID_DEF;
	int nID = -1;
	ZString name;

	if (bIsUpdateable)
	{	nextToken.Scan ();
		if (nextToken == TID_NAME)
		{	nID = nextToken.GetDefinedName ();
			if (nID < 0)
				nID = nextToken.AddPredefinedName ();
			else
				nID = -1;
			if (nID < 0)
			{	nextToken.Error ("Name already defined");
				nID = -1;
			}
			name = nextToken.GetName ();
			nextToken.Scan ();
		}
		else
		{	nextToken.Error ("Unexpected symbol");
			nextToken.Scan ();
		}
	}
				
	/*-----------------------------------------------------------------
	Determine the node type, instantiate the object, and let it
	Encode itself. The node may be a PROTO
	-----------------------------------------------------------------*/
	int nNodeType = INT_MAX;
	int nProtoID = nextToken.GetDefinedProto (&m_pNode);
	if (nProtoID < 0)
	{	m_pNode = CreateNode (nextToken.GetName (), nNodeType);
		if (!m_pNode)
		{	if (nextToken != TID_NAME)
				nextToken.Error ("Unexpected symbol");
			else
				nextToken.Error ("Unrecognized node name");
			nextToken.Scan ();
			return FALSE;
		}
		if (nNodeType == -1)
			nextToken.Error ("Unexpected node type");
	}

	if (nNodeType == -2)
	{	nNodeType = 1;
		if (nextToken != TID_LEFT_BRACE)
		{	nextToken.Error ("Unexpected symbol");
			nextToken.Scan ();
		}
	}
	else
		nextToken.Scan ();

	pBifs -> EncodeInt (0, 1, "isReused");
	int nVersion1TableSize = GetNodeTableSize () - GetV2ExtSize ();
	int nFieldWidth = BitFieldWidth (nVersion1TableSize + 1);

	/*-------------------------------------------------------------
	If this is a version 2 node (or a PROTO), code with escape
	-------------------------------------------------------------*/
	if (nNodeType > nVersion1TableSize)
	{	pBifs -> EncodeInt (0, nFieldWidth, "nodeType");
		nNodeType -= nVersion1TableSize;
		++nNodeType;
		nFieldWidth = BitFieldWidth (GetV2ExtSize () + 2);
		if (nProtoID >= 0)
			nNodeType = 1;
		pBifs -> EncodeInt (nNodeType, nFieldWidth, "extLocalNodeType");
		if (nProtoID >= 0)
			pBifs -> EncodeInt (nProtoID, 10, "PROTOnodeType");
	}
	else
		pBifs -> EncodeInt (nNodeType, nFieldWidth, "nodeType");
	pBifs -> EncodeInt (bIsUpdateable, 1, "isUpdateable");
	if (bIsUpdateable)
	{	pBifs -> EncodeInt (nID, 10, "nodeID");
		if (pBifs ->GetFlags () & CODENAMES)
			pBifs -> EncodeString (name);
		if (nID >= 0)
		{	nextToken.SetNodeID (nID, m_pNode -> GetNodeCode (), m_pNode);
			m_pNode -> SetID(nID);	// SCRIPT
		}
	}
	

	// MBS: sets BIFSEncoder to current QP node just parsed
	if(m_pNode ->GetNodeCode () == CODE_QuantizationParameter) {
	  pBifs->SetCurrentQP(m_pNode);
	  m_pNode->SetID(nID);
	}
	else if (nNodeType == -1)
	{
		delete m_pNode;
	    m_pNode = NULL;
		return false;
	}
	// end MBS Q

	// BenA (7/30/99): moved this statement here from above if(isQP)
	BOOL bResult = m_pNode->Encode (pBifs);
	// end BenA

	return bResult;
}

/*-----------------------------------------------------------------
Create a node whose name is szNodeName. To do this try to instantiate
all nodes, and for each try if this is the required one.
Return the found nodeType in rNodeType.
-----------------------------------------------------------------*/
MediaObject *SFGenericNode :: CreateNode (
	LPCSTR szNodeName,
	int &rNodeType
)
{
	for (int nCode = 0; nCode < MPEG4_TOTAL_NUMBER_OF_NODES; nCode ++)
		if (m_bifsFactory [nCode] != NULL)
		{	MediaObject *pNode = m_bifsFactory [nCode] -> CreateNode ();
			if (!strcmp (pNode -> GetName (), szNodeName))
			{	rNodeType = nCode;
				return pNode;
			}
			delete pNode;
		}
	return NULL;
}

/*-----------------------------------------------------------------
Encode an MF field
-----------------------------------------------------------------*/
BOOL MField :: Encode (BIFSEncoder *pBifs, Quantizer *pQuant)
{
	pBifs -> EncodeInt (0, 1, "reserved");		// added for FDIS
	pBifs -> EncodeInt (1, 1, "isListDescription");

	// MBS 111098: must sets QP node explicitly
	if(pQuant)
	  pQuant->SetQPNode(pBifs->GetCurrentQP());
	// end MBS 111098

	if (nextToken == TID_LEFT_BRACKET)
	{	nextToken.Scan ();

		// MBS: local def of QP variables
		BOOL localQP=FALSE;
		MediaObject *qpOld = NULL;
		// end MBS
	
		// BenA (7/30/99): save quantization for this level
		MediaObject *qpForThisLevel = pBifs->GetCurrentQP();
		// end BenA

		while (nextToken != TID_RIGHT_BRACKET)
		{
			pBifs -> EncodeInt (0, 1, "endFlag");	// changed in the FDIS

			// MBS: saves old QP node
			if(!localQP) qpOld=pBifs->GetCurrentQP();
			// end MBS

			EncodeElement (pBifs, pQuant);


			// MBS: restores current QP

			if(pBifs->hasLocalQP()) {
			  if(localQP) {
			    localQP=FALSE;
			    pBifs->SetCurrentQP(qpOld);
			  }
			  else {
			    localQP=TRUE;
			  }
			}
			// end MBS


		if (nextToken == TID_COMMA)
			nextToken.Scan ();
		}

		// BenA (7/30/99): restores the right QP to use -- end of MF field
		pBifs->SetCurrentQP(qpForThisLevel);
		// end BenA


		nextToken.Scan ();	
	}
	else
	{	pBifs -> EncodeInt (0, 1, "endFlag");
		EncodeElement (pBifs,pQuant);
	}
	pBifs -> EncodeInt (1, 1, "endFlag");
	return TRUE;
}

/*-----------------------------------------------------------------
The destrcutor of MFGenericNode deletes all elements in the array (except the PROTOs)
-----------------------------------------------------------------*/
void MFGenericNode :: RemoveAll ()
{
	for (int i = m_array.GetLength (); --i >= 0;)
	{	if (m_array [i] && m_array [i] -> GetNodeCode () >= 0)
		delete m_array [i];
	}
}

/*-----------------------------------------------------------------
Delete one element from MFGenericNode
-----------------------------------------------------------------*/
void MFGenericNode :: DeleteFrom (int index, BOOL)
{
	delete m_array [index];
	m_array.DeleteFrom (index);
}

/*-----------------------------------------------------------------
The default implementation of EncodeElement can be called only by 
error
-----------------------------------------------------------------*/
BOOL NodeField :: EncodeElement (BIFSEncoder *, Quantizer *)
{
	nextToken.Error ("Not an MF field");
	return FALSE;
}

#ifndef MUX
/*-----------------------------------------------------------------
Encode the buffer of a Conditional node (which is written as an update
command
-----------------------------------------------------------------*/
BOOL SFConditionalString :: Encode (BIFSEncoder *pBifs, Quantizer *)
{
	ConditionalBIFSEncoder encoder (
		&m_value,
		pBifs -> GetListFile (),
		pBifs -> GetFlags ()
	);
	encoder.EncodeAU ();
	EncodeString (pBifs);
	return TRUE;
}
#endif		//	MUX
