/***********************************************************************
This software module was originally developed by Zvi Lifshitz
(Triton R&D Ltd.) in the course of development of the MPEG-4 Systems
(ISO/IEC 14496-1) standard. This software module is an implementation of
a part of one or more MPEG-4 Systems (ISO/IEC 14496-1) tools as
specified by the MPEG-4 Systems (ISO/IEC 14496-1) standard. ISO/IEC
gives users of the MPEG-4 Systems (ISO/IEC 14496-1) free license to
this software module or modifications thereof for use in hardware or
software products claiming conformance to the MPEG-4 Systems (ISO/IEC
14496-1). Those intending to use this software module in hardware or
software products are advised that its use may infringe existing patents.
The original developer of this software module and his/her company, the
subsequent editors and their companies, and ISO/IEC have no liability
for use of this software module or modifications thereof in an
implementation. Copyright is not released for non MPEG-4 Systems
(ISO/IEC 14496-1) conforming products. Triton R&D Ltd. retains full right
to use the code for his/her own purpose, assign or donate the code to a
third party and to inhibit third parties from using the code for non
MPEG-4 Systems (ISO/IEC 14496-1) conforming products. This copyright
notice must be included in all copies or derivative works.
Copyright  1997.
************************************************************************

For more information or to receive an up to date version of this module,
you may call Zvi Lifshitz at zvil@csi.com.
----------------------------------------------------------------------*/

/*-------------------------------------------------------------------------
Project Im1Player
INTERNATIONAL ORGANIZATION FOR STANDARDIZATION
ISO/IEC JTC1/SC29/WG11

SUBSYSTEM:  BIFS Encoder
FILE:       EncNode.cpp
AUTHOR:     Zvi Lifshitz (Triton R&D Ltd.)
OVERVIEW:   Encoding of a MediaObject object.
-------------------------------------------------------------------------*/
#include "defs.h"
#include "MediaObj.h"
#include "Token.h"
#include "BifsEnc.h"
#include "bifs.h"
#include "Quantize.h"
#include "NodeFld.h"
#include "Proto.h"

/*-----------------------------------------------------------------
Parse a MediaObject by using the field table.
-----------------------------------------------------------------*/
BOOL MediaObject :: Encode (BIFSEncoder *pBifs)
{
	pBifs -> EncodeInt (0, 1, "maskAccess");

	if (nextToken == TID_LEFT_BRACE)
	{	nextToken.Scan ();
		while (nextToken != TID_RIGHT_BRACE)
			ParseField (pBifs);
		nextToken.Scan ();
	}
	pBifs -> EncodeInt (1, 1, "endFlag");
	return TRUE;
}

/*-----------------------------------------------------------------
Parse one node field and encode it
-----------------------------------------------------------------*/
void MediaObject :: ParseField (BIFSEncoder *pBifs)
{	
	/*-----------------------------------------------------------------
	Store no-end bit
	-----------------------------------------------------------------*/
	pBifs -> EncodeInt (0, 1, "endFlag");
	if (nextToken != TID_NAME)
	{	nextToken.Error ("Unexpected symbol");
		nextToken.Scan ();
	}
	else
	{	int nFieldID;
		const BifsFieldTable *pEntry =
			GetBifsEntry (nextToken.GetName (), &nFieldID);
		if (pEntry)
		{	Proto *pProto = pBifs -> GetProto ();
			bool isedField = false;
			if (pProto != NULL)
			{	nextToken.Scan ();
				isedField = nextToken == TID_IS;
				pBifs -> EncodeInt (isedField, 1, "isedField");
				if (isedField)
				{	pBifs -> EncodeInt (
						nFieldID,
						GetBifsBounds () ->m_nBitsPerAll,
						"fieldRef"
					);
					nextToken.Scan ();
					if (nextToken != TID_NAME)
						nextToken.Error ("Unexpected symbol");
					else if (pProto -> GetBifsEntry (nextToken.GetName (), &nFieldID) == NULL)
						nextToken.Error ("Unrecognized field name");
					pBifs -> EncodeInt (
						nFieldID,
						pProto ->GetBifsBounds () ->m_nBitsPerAll,
						"protoField"
					);
					nextToken.Scan ();
				}
				else if (pEntry -> m_defID < 0)
					nextToken.Error ("Not a 'def' field");
			}
			else
			{	if (pEntry -> m_defID < 0)
					nextToken.Error ("Not a 'def' field");
				nextToken.Scan ();
			}
			if (!isedField)
			{	pBifs -> EncodeInt (
					pEntry -> m_defID,
					GetBifsBounds () -> m_nBitsPerDef,
					"fieldRef"
				);
				EncodeField (pBifs, pEntry);
			}
		}
		else
		{	nextToken.Error ("Unrecognized field name");
			nextToken.Scan ();
		}
	}
}

/*-----------------------------------------------------------------
Encode a node field, when a pointer to the BIFS table entry describing
that field is given
-----------------------------------------------------------------*/
BOOL MediaObject :: EncodeField (
	BIFSEncoder *pBifs,
	const BifsFieldTable *pEntry
)
{	
	if (pEntry == NULL)
		return FALSE;

	// MBS: init quantizer with current QuantizationParameter node
	Quantizer quant(pBifs->GetCurrentQP(),
			pEntry -> m_fMin,  
			pEntry -> m_fMax,  
			pEntry -> m_nDimension, 
			pEntry -> m_nQuantizationType);
	// end MBS

	BOOL bResult = GetField (pEntry) -> Encode (pBifs, &quant);
	return bResult;
}

/*-----------------------------------------------------------------
Encode a Route whose departure node is this node
-----------------------------------------------------------------*/
void MediaObject :: EncodeRoute (BIFSEncoder *pBifs, BOOL bIn)
{
	pBifs -> EncodeInt (m_nID, 10, "NodeID");
	if (nextToken != TID_PERIOD)
		nextToken.Error (". expected");
	else
		nextToken.Scan ();

	if (nextToken != TID_NAME)
		nextToken.Error ("Unexpected symbol");
	else
	{	const BifsFieldTable *pEntry =
			GetBifsEntry (nextToken.GetName ());
		if (pEntry)
		{	if (bIn && pEntry -> m_inID < 0)
				nextToken.Error ("Not an 'in' field");
			else if (!bIn && pEntry -> m_outID < 0)
				nextToken.Error ("Not an 'out' field");
			pBifs -> EncodeInt (
				bIn ? pEntry -> m_inID : pEntry -> m_outID ,
				bIn ?
					GetBifsBounds () -> m_nBitsPerIn :
					GetBifsBounds () -> m_nBitsPerOut,
				bIn ? "inFieldRefernce" : "outFieldReference"
			);
		}
		else
			nextToken.Error ("Unrecognized field name");
	}
	nextToken.Scan ();
}

/*-----------------------------------------------------------------
Encode an insert command. If bAppend is TRUE, a value is inserted at
the end of an MF field, otherwise an index is given.
-----------------------------------------------------------------*/
void MediaObject :: InsertToField (BIFSEncoder *pBifs, BOOL bAppend)
{
	int nIndex;
	const BifsFieldTable *pEntry = ParseMFElement (bAppend, nIndex);
	if (!bAppend && nIndex < 0)
		nextToken.Error ("[ exptected");

	/*-----------------------------------------------------------------
	Encode the command
	-----------------------------------------------------------------*/
	if (pEntry)
	{	
		if (pEntry -> m_bChildren)
		{	pBifs -> EncodeInt (0, 2, "NodeInsertion");
			pBifs -> EncodeInt (m_nID, 10, "NodeID");
		}
		else
		{	if (pEntry -> m_inID < 0)
				nextToken.Error ("Not an 'in' field");
			pBifs -> EncodeInt (2, 2, "IndexedValueInsertion");
			pBifs -> EncodeInt (m_nID, 10, "NodeID");
			pBifs -> EncodeInt (
				pEntry -> m_inID,
				GetBifsBounds () -> m_nBitsPerIn,
				"fieldID"
			);
		}
		pBifs -> EncodeInt (
			nIndex > 0 ? 0 : nIndex == 0 ? 2 : 3,
			2,
			"insertionPosition"
		);
		if (nIndex > 0)
			pBifs -> EncodeInt (
				nIndex,
				(2 - pEntry -> m_bChildren) * 8,
				"position"
			);
		EncodeFieldElement (pBifs, pEntry);
	}
}

/*-----------------------------------------------------------------
Encode an element in an MF node field, when a pointer to the BIFS table
entry describing that field is given
-----------------------------------------------------------------*/
void MediaObject :: EncodeFieldElement (
	BIFSEncoder *pBifs,
	const BifsFieldTable *pEntry
)
{	
	Quantizer quant (
		pEntry -> m_fMin,  
		pEntry -> m_fMax,  
		pEntry -> m_nDimension, 
		pEntry -> m_nQuantizationType
	);

	GetField (pEntry) -> EncodeElement (pBifs, &quant);
}

/*-----------------------------------------------------------------
Encode an IndexedValueDeletion command. If bLast is TRUE, this is DELETE
LAST ...
-----------------------------------------------------------------*/
void MediaObject :: IndexedValueDeletion (BIFSEncoder *pBifs, BOOL bLast)
{
	int nIndex;
	const BifsFieldTable *pEntry = ParseMFElement (bLast, nIndex);
	if (!bLast && nIndex < 0)
		nextToken.Error ("[ exptected");

	if (pEntry)
	{	if (pEntry -> m_inID < 0)
			nextToken.Error ("Not an 'in' field");
		pBifs -> EncodeInt (2, 2, "IndexedValueDeletion");
		pBifs -> EncodeInt (m_nID, 10, "NodeID");
		pBifs -> EncodeInt (
			pEntry -> m_inID,
			GetBifsBounds () -> m_nBitsPerIn,
			"fieldID"
		);
		pBifs -> EncodeInt (
			nIndex > 0 ? 0 : nIndex == 0 ? 2 : 3,
			2,
			"deletionPosition"
		);
		if (nIndex > 0)
			pBifs -> EncodeInt (nIndex, 16, "position");
	}
}

/*-----------------------------------------------------------------
Parse MField Element. Returns the index in rIndex
-----------------------------------------------------------------*/
const BifsFieldTable *MediaObject :: ParseMFElement (BOOL bLast, int &rIndex)
{
	const BifsFieldTable *pEntry = NULL;

	if (nextToken != TID_PERIOD)
		nextToken.Error (". expected");
	else
		nextToken.Scan ();

	if (nextToken != TID_NAME)
	{	nextToken.Error ("Unexpected symbol");
		nextToken.Scan ();
	}	
	else
	{	pEntry = GetBifsEntry (nextToken.GetName ());
		if (!pEntry)
			nextToken.Error ("Unrecognized field name");
		nextToken.Scan ();

		rIndex = -1;
		/*-----------------------------------------------------------------
		If bAppend is FALSE, field name is followed by an index
		-----------------------------------------------------------------*/
		if (!bLast && nextToken == TID_LEFT_BRACKET)
		{	rIndex = 0;
			nextToken.Scan ();
			if (nextToken != TID_INTEGER)
				nextToken.Error ("Integer index value expected");
			rIndex = nextToken.GetInt ();
			if (nextToken != TID_RIGHT_BRACKET)
				nextToken.Error ("] expected");
			nextToken.Scan ();
		}
	}
	return pEntry;
}

/*-----------------------------------------------------------------
Replace a field or an indexed value
-----------------------------------------------------------------*/
void MediaObject :: FieldReplacement (BIFSEncoder *pBifs, BOOL bLast)
{
	int nIndex;
	const BifsFieldTable *pEntry = ParseMFElement (bLast, nIndex);
	
	if (nextToken == TID_BY)
		nextToken.Scan ();
	else
		nextToken.Error ("BY expected");

	if (pEntry)
	{	
		/*-----------------------------------------------------------------
		Encode FieldReplacement
		-----------------------------------------------------------------*/
		if (!bLast && nIndex < 0)
		{	if (pEntry -> m_inID < 0)
				nextToken.Error ("Not an 'in' field");
			pBifs -> EncodeInt (1, 2, "FieldReplacement");
			pBifs -> EncodeInt (m_nID, 10, "NodeID");
			pBifs -> EncodeInt (
				pEntry -> m_inID,
				GetBifsBounds () -> m_nBitsPerIn,
				"fieldID"
			);
			EncodeField (pBifs, pEntry);
		}
		/*-----------------------------------------------------------------
-		Encode IndexedValueReplacement
		-----------------------------------------------------------------*/
		else
		{	if (pEntry -> m_inID < 0)
				nextToken.Error ("Not an 'in' field");
			pBifs -> EncodeInt (2, 2, "IndexedValueReplacement");
			pBifs -> EncodeInt (m_nID, 10, "NodeID");
			pBifs -> EncodeInt (
				pEntry -> m_inID,
				GetBifsBounds () -> m_nBitsPerIn,
				"fieldID"
			);
			pBifs -> EncodeInt (
				nIndex > 0 ? 0 : nIndex == 0 ? 2 : 3,
				2,
				"replacementPosition"
			);
			if (nIndex > 0)
				pBifs -> EncodeInt (nIndex, 16, "position");
			EncodeFieldElement (pBifs, pEntry);
		}
	}
}

/*-----------------------------------------------------------------
Returns a pointer to the BIFS table entry for the field whose name is
szName.
If pFieldID is not null, store the index of the field in the variable it points to
-----------------------------------------------------------------*/
const BifsFieldTable * MediaObject :: GetBifsEntry (LPCSTR szName, int *pFieldID)
{
	const BifsFieldTable *pTable = GetFieldTable ();
	for (int i = 0; i < GetFieldCount (); i++)
		if (pTable [i].m_szName && !stricmp (szName, pTable [i].m_szName))
		{	if (pFieldID)
				*pFieldID = i;
			return pTable + i;
		}
	return NULL;
}