/***********************************************************************
This software module was originally developed by Zvi Lifshitz
(Triton R&D Ltd.) in the course of development of the MPEG-4 Systems
(ISO/IEC 14496-1) standard. This software module is an implementation of
a part of one or more MPEG-4 Systems (ISO/IEC 14496-1) tools as
specified by the MPEG-4 Systems (ISO/IEC 14496-1) standard. ISO/IEC
gives users of the MPEG-4 Systems (ISO/IEC 14496-1) free license to
this software module or modifications thereof for use in hardware or
software products claiming conformance to the MPEG-4 Systems (ISO/IEC
14496-1). Those intending to use this software module in hardware or
software products are advised that its use may infringe existing patents.
The original developer of this software module and his/her company, the
subsequent editors and their companies, and ISO/IEC have no liability
for use of this software module or modifications thereof in an
implementation. Copyright is not released for non MPEG-4 Systems
(ISO/IEC 14496-1) conforming products. Triton R&D Ltd. retains full right
to use the code for his/her own purpose, assign or donate the code to a
third party and to inhibit third parties from using the code for non
MPEG-4 Systems (ISO/IEC 14496-1) conforming products. This copyright
notice must be included in all copies or derivative works.
Copyright  1997.
************************************************************************

For more information or to receive an up to date version of this module,
you may call Zvi Lifshitz at zvil@csi.com.
----------------------------------------------------------------------*/

#if !defined(_Token_h)	// Sentry, use file only if it's not already included.
#define _Token_h

/*-------------------------------------------------------------------------
Project Im1Player
INTERNATIONAL ORGANIZATION FOR STANDARDIZATION
ISO/IEC JTC1/SC29/WG11

SUBSYSTEM:  BIFS Encoder
FILE:       Token.h
AUTHOR:     Zvi Lifshitz (Triton R&D Ltd.)
OVERVIEW:   Implementation of Token, the class that handles text parsing.
-------------------------------------------------------------------------*/
#include "ztl.h"

typedef enum tagTokenCodes
{
	TID_UNKNOWN,
	TID_LEFT_BRACE,
	TID_RIGHT_BRACE,
	TID_LEFT_BRACKET,
	TID_RIGHT_BRACKET,
	TID_COMMA,
	TID_PERIOD,
	TID_STRING,
	TID_USE,
	TID_DEF,
	TID_FALSE,
	TID_TRUE,
	TID_ROUTE,
	TID_TO,
	TID_AT,
	TID_INSERT,
	TID_REPLACE,
	TID_DELETE,
	TID_APPEND,
	TID_LAST,
	TID_SCENE,
	TID_BY,
	TID_UPDATE,
	TID_REMOVE,
	TID_OD,
	TID_ESD,
	TID_IN,
	TID_IPMPD,
	TID_FROM,
	TID_FILE,
	TID_PROTO,
	TID_EXTERNPROTO,
	TID_FIELD, TID_EXPOSEDFIELD, TID_EVENTIN, TID_EVENTOUT,
	TID_SFBOOL, TID_SFFLOAT, TID_SFTIME, TID_SFINT32, TID_SFSTRING, TID_SFVEC3F,
	TID_SFVEC2F, TID_SFCOLOR, TID_SFROTATION, TID_SFIMAGE, TID_SFNODE,
	TID_MFBOOL, TID_MFFLOAT, TID_MFTIME, TID_MFINT32, TID_MFSTRING, TID_MFVEC3F,
	TID_MFVEC2F, TID_MFCOLOR, TID_MFROTATION, TID_MFIMAGE, TID_MFNODE,
	TID_IS,
	TID_NUMBER_OF_RESERVED,
	TID_NAME = TID_NUMBER_OF_RESERVED,
	TID_NUMBER,
	TID_INTEGER,
	TID_EOF
} TOKEN_CODE;

class MediaObject;
class Proto;

class Token
{
	TOKEN_CODE m_code;
	ZString m_string;
	int m_cNext, m_nInputOffset, m_nInputLength;
	FILE *m_fInput, *m_fList;
	char m_szLine [300];
	static LPCSTR m_reserved [];
	BOOL m_bNewSourceLine, m_bHadErrors;

	struct DefinedNode
	{
		ZString m_name;
		int m_nCode;
		MediaObject *m_pInstance;
		BOOL operator == (const DefinedNode &) const {return FALSE;}
		DefinedNode () : m_nCode (INT_MAX), m_pInstance (NULL) {}
	};

	struct DefinedProto
	{
		ZString m_name;
		Proto *m_pObject;
		BOOL operator == (const DefinedNode &) const {return FALSE;}
		DefinedProto () : m_pObject (NULL) {}
		~DefinedProto ();
	};

	ZArray<DefinedNode> m_defined;
	ZArray<ZString> m_routes;
	ZArray<DefinedProto> m_protos;

public:
	Token () : m_bHadErrors (FALSE) {}
	~Token ();
	void Init (FILE *fInput, FILE *fList = NULL);
	operator int () {return m_code;}
	int GetInt ();
	bool GetBool ();
	double GetReal ();
	const ZString &GetName () {return m_string;}
	const ZString &GetString ();
	void Scan ();
	int GetDefinedName () const;
	int GetDefinedRoute () const;
	int GetDefinedProto (MediaObject **ppObject = NULL) const;
	void Error (LPCSTR szDesc, LPCSTR szItem = NULL);
	int AddPredefinedName ();
	int AddPredefinedRoute ();
	int AddPredefinedProto (Proto *pObject);
	void SetNodeID (int nIndex, int nNodeCode, MediaObject *pInstance)
	{
		m_defined [nIndex].m_nCode = nNodeCode;
		m_defined [nIndex].m_pInstance = pInstance;
	}
	int GetNodeID (int nIndex) const {return m_defined [nIndex].m_nCode;}
	MediaObject *GetInstance (int nIndex) const {return m_defined [nIndex].m_pInstance;}

private:
	int GetChar ();

public:
	void Push ();
	void Pop ();

private:
	struct TokenStack
	{
		TOKEN_CODE m_code;
		ZString m_string;
		int m_cNext, m_nInputOffset, m_nInputLength;
		FILE *m_fInput;
		char m_szLine [300];
		BOOL m_bNewSourceLine;
	} m_stack;
};

class ExceptionEndOfFile {};

#ifndef MAIN
extern Token nextToken;
#endif	// MAIN

#endif                                      // _Token_h sentry.
