/***********************************************************************
This software module was originally developed by Zvi Lifshitz
(Triton R&D Ltd.) in the course of development of the MPEG-4 Systems
(ISO/IEC 14496-1) standard. This software module is an implementation of
a part of one or more MPEG-4 Systems (ISO/IEC 14496-1) tools as
specified by the MPEG-4 Systems (ISO/IEC 14496-1) standard. ISO/IEC
gives users of the MPEG-4 Systems (ISO/IEC 14496-1) free license to
this software module or modifications thereof for use in hardware or
software products claiming conformance to the MPEG-4 Systems (ISO/IEC
14496-1). Those intending to use this software module in hardware or
software products are advised that its use may infringe existing patents.
The original developer of this software module and his/her company, the
subsequent editors and their companies, and ISO/IEC have no liability
for use of this software module or modifications thereof in an
implementation. Copyright is not released for non MPEG-4 Systems
(ISO/IEC 14496-1) conforming products. Triton R&D Ltd. retains full right
to use the code for his/her own purpose, assign or donate the code to a
third party and to inhibit third parties from using the code for non
MPEG-4 Systems (ISO/IEC 14496-1) conforming products. This copyright
notice must be included in all copies or derivative works.
Copyright  1998.
************************************************************************

For more information or to receive an up to date version of this module,
you may call Zvi Lifshitz at zvil@csi.com.
----------------------------------------------------------------------*/

/*-------------------------------------------------------------------------
Project Im1Player
INTERNATIONAL ORGANIZATION FOR STANDARDIZATION
ISO/IEC JTC1/SC29/WG11

SUBSYSTEM:  BIFS Decoder/ Compositor / Presenter
FILE:       StreamCo.cpp
AUTHOR:     Zvi Lifshitz (Triton R&D Ltd.)
OVERVIEW:   Implementation of StreamConsumer
-------------------------------------------------------------------------*/
#include "defs.h"
#include <assert.h>
#include "StreamCo.h"
#include "Executiv.h"
#include "ODImp.h"
#include "Stream.h"
#include "InLine.h"
#include "BifsDec.h"
#include "ODDec.h"
#include "PresBase.h"
#include "AL.h"

/*
#include "Caps.h"
*/

/*-----------------------------------------------------------------
StreamConsumer constructor
-----------------------------------------------------------------*/
StreamConsumer :: StreamConsumer ()
{
	m_pStream = NULL;
	m_pOD = NULL;
	m_pAL = NULL;
	m_nODid = -1;
	m_bStreamOpen = m_bRemoteOD = false;

	CUSTOMIZE_STREAMCONSUMER_CONSTRUCTOR
}

/*-----------------------------------------------------------------
StreamConsumer destructor. First flush out everything from the stream
(to make sure no object is waiting on the stream, then delete the
associated decoder and stream.
-----------------------------------------------------------------*/
StreamConsumer :: ~StreamConsumer ()
{
	CUSTOMIZE_STREAMCONSUMER_DESTRUCTOR

	CloseStream ();
	if (m_pOD)
		m_pOD -> Detach ();
	if (m_nODid >= 0 && m_pSource != NULL)
		m_pSource -> UnregisterStreamConsumer (this);
	if (m_bRemoteOD)
		delete m_pOD;
	delete m_pAL;
}

/*-----------------------------------------------------------------
Overload Parse(). After parsing in the usual way, the ODid field
is checked. If has value, register with Executive.
-----------------------------------------------------------------*/
BOOL StreamConsumer :: Parse (BIFSDecoder *pBifs)
{
	BOOL bResult = MediaObject :: Parse (pBifs);
	m_pSource = pBifs -> GetSource ();
	for (int i = 0; i < url.GetLength (); i++)
		if (url [i] >= 0)
		{	m_nODid = url [i];
			break;
		}
	if (m_nODid >= 0)
		m_pSource -> RegisterStreamConsumer (this);
	return bResult;
}
 
/*-----------------------------------------------------------------
Instantiate the proper decoder and attach it to the object through
streams
-----------------------------------------------------------------*/
void StreamConsumer :: SetStream (ObjectDescriptorImp *pOD)
{
	bool bStreamOpen = m_bStreamOpen;
	m_bStreamOpen = false;	// don't open stream till after call to SetStream
	m_bRemoteOD = false;
	m_pOD = pOD;
	MediaStreamImp *pStream = pOD -> SetStream (this);
	if (pStream != NULL)
		m_pStream = pStream;
	if (bStreamOpen)		// if necessary open stream now.
		OpenStream ();
}

/*-------------------------------------------------------------
Stop playing from the stream and remove the OD
-------------------------------------------------------------*/
void StreamConsumer :: ODRemoved ()
{
	bool bOpen = m_bStreamOpen;	// if already open, stream remains flagges open although
								// physically closed.
	CloseStream ();
	m_pOD -> Detach ();
	delete m_pAL;
	m_pAL = NULL;
	
	/*-------------------------------------------------------------
	If the removed OD is attached to an auxliary service (i.e. this is
	an OD with URL), delete the OD object (otherwise it will be deleted by the caller
	to this function)
	-------------------------------------------------------------*/
	if (m_bRemoteOD)
		delete m_pOD;
	m_pOD = NULL;
	m_pStream = NULL;
	m_bStreamOpen = bOpen;
}

/*-------------------------------------------------------------
One ESD was added to the OD used by this node.
The consequence may be the attachment of a new stream to the current 
node.
-------------------------------------------------------------*/
void StreamConsumer :: AddESD (ES_DescriptorImp *pESD)
{
	if (m_bRemoteOD)
	{	GetExecutive () -> Error (CANNOT_ALTER_REMOTE_OD);
		return;
	}

	/*-------------------------------------------------------------
	A special treatment for IPMP streams
	-------------------------------------------------------------*/
	if (pESD -> decConfigDescr == NULL_DESC)
		return;
	if (pESD -> decConfigDescr -> streamType == ST_IPMPStream)
	{	Reopen ();
		return;
	}

	/*-------------------------------------------------------------
	Add the stream. This may result in attaching new MediaStream
	to the node
	-------------------------------------------------------------*/
	MediaStreamImp *pStream = pESD -> AddESD (m_pOD, this);
	if (pStream != NULL)
		m_pStream = pStream;
	if (m_bStreamOpen)		// if necessary open stream now.
		pESD -> Start ();
}

/*-------------------------------------------------------------
One ESD was removed from the OD used by this node.
-------------------------------------------------------------*/
void StreamConsumer :: RemoveESD (ES_DescriptorImp *pESD)
{
	if (m_bRemoteOD)
	{	GetExecutive () -> Error (CANNOT_ALTER_REMOTE_OD);
		return;
	}

	/*-------------------------------------------------------------
	A special treatment for IPMP streams
	-------------------------------------------------------------*/
	if (pESD -> decConfigDescr == NULL_DESC)
		return;
	if (pESD -> decConfigDescr -> streamType == ST_IPMPStream)
	{	Reopen ();
		return;
	}

	/*-------------------------------------------------------------
	Remove the stream. This may result in detaching the MediaStream
	assigned to this node
	-------------------------------------------------------------*/
	if (m_pStream != NULL && pESD -> IsMyStream (m_pStream))
	{	CloseStream ();
		m_pStream = NULL;
	}
	else if (m_bStreamOpen)
		pESD -> Stop ();
	pESD -> Detach ();
}

/*-----------------------------------------------------------------
Request transmission of data on the channel
-----------------------------------------------------------------*/
void StreamConsumer :: OpenStream ()
{
	if (!m_bStreamOpen)
	{	m_bStreamOpen = true;
		if (m_pOD)
		{	if (m_pStream)
				m_pStream -> Start ();
			m_pOD -> Start ();
		}
	}
}

/*-----------------------------------------------------------------
Close the stream and notify the executive
-----------------------------------------------------------------*/
void StreamConsumer :: CloseStream ()
{
	if (m_bStreamOpen)
	{	m_bStreamOpen = false;
		if (m_pOD)
		{	if (m_pStream)
				m_pStream -> Terminate ();
			m_pOD -> Stop ();
		}
	}
}

/*-----------------------------------------------------------------
The default implementation of GetCapabilities returns all the 
capabilities of the presenter
-----------------------------------------------------------------*/
void StreamConsumer :: GetCapabilities (Capabilities *pCaps)
{
	GetExecutive () -> GetPresenter () -> GetCapabilities (pCaps);
}

/*-------------------------------------------------------------
A session with remoteOD has been established
-------------------------------------------------------------*/
void StreamConsumer :: OnConnect (const BYTE *pInitOD, int nInitODLen)
{
	ODDecoder odDec (
		GetExecutive (),
		pInitOD,
		nInitODLen,
		GetExecutive () -> GetTraceFile (),
		"Initial OD",
		GetExecutive () -> GetTraceFlags (),
		0
	);
	if (m_bRemoteOD)
		delete m_pOD;
	m_bRemoteOD = true;
	m_pOD = new ObjectDescriptorImp;
	m_pOD -> DecInitialOD (&odDec);
	m_pOD -> SetAL (m_pAL);
	m_pOD -> SetInline (m_pSource);
	GetExecutive () -> LockScene ();
	SetStream (m_pOD);
	GetExecutive () -> UnlockScene ();
	GetExecutive () -> RequestChannels ();
}

/*-------------------------------------------------------------
Return the decoder attached to this node
-------------------------------------------------------------*/
Decoder *StreamConsumer :: GetDecoder () const
{
	return m_pOD == NULL ? NULL : m_pOD -> GetDecoder ();
}

/*-------------------------------------------------------------
In case an IPMP stream is added or removed to/from the OD, stop playing
the stream, close the decoder, and reopen all with the new IPMP stream
-------------------------------------------------------------*/
void StreamConsumer :: Reopen ()
{
	bool bStreamOpen = m_bStreamOpen;
	CloseStream ();
	m_pOD -> Detach ();
	m_pStream = NULL;
	m_bStreamOpen = bStreamOpen;
	SetStream (m_pOD);
}

/*-------------------------------------------------------------
When a new service is required for this node, delete the old one and store the 
address of the new.
-------------------------------------------------------------*/
void StreamConsumer :: SetAL (ALManager *pAL)
{
	delete m_pAL;
	m_pAL = pAL;
}
