/***********************************************************************
This software module was originally developed by Zvi Lifshitz
(Triton R&D Ltd.) in the course of development of the MPEG-4 Systems
(ISO/IEC 14496-1) standard. This software module is an implementation of
a part of one or more MPEG-4 Systems (ISO/IEC 14496-1) tools as
specified by the MPEG-4 Systems (ISO/IEC 14496-1) standard. ISO/IEC
gives users of the MPEG-4 Systems (ISO/IEC 14496-1) free license to
this software module or modifications thereof for use in hardware or
software products claiming conformance to the MPEG-4 Systems (ISO/IEC
14496-1). Those intending to use this software module in hardware or
software products are advised that its use may infringe existing patents.
The original developer of this software module and his/her company, the
subsequent editors and their companies, and ISO/IEC have no liability
for use of this software module or modifications thereof in an
implementation. Copyright is not released for non MPEG-4 Systems
(ISO/IEC 14496-1) conforming products. Triton R&D Ltd. retains full right
to use the code for his/her own purpose, assign or donate the code to a
third party and to inhibit third parties from using the code for non
MPEG-4 Systems (ISO/IEC 14496-1) conforming products. This copyright
notice must be included in all copies or derivative works.
Copyright  1998.
************************************************************************

For more information or to receive an up to date version of this module,
you may call Zvi Lifshitz at zvil@csi.com.
----------------------------------------------------------------------*/

#if !defined(_StreamCo_h)	// Sentry, use file only if it's not already included.
#define _StreamCo_h

/*-------------------------------------------------------------------------
Project Im1Player
INTERNATIONAL ORGANIZATION FOR STANDARDIZATION
ISO/IEC JTC1/SC29/WG11

SUBSYSTEM:  BIFS Decoder/ Compositor / Presenter
FILE:       MediaObj.h
AUTHOR:     Zvi Lifshitz (Triton R&D Ltd.)
OVERVIEW:   Declarations of StreamConsumer
-------------------------------------------------------------------------*/
#include "MediaObj.h"
#include "NodeFld.h"
#include "Decoder.h"
#include "custom.h"

class MediaStreamImp;
class DataChannel;
class Service;
class ALManager;
class ObjectDescriptorImp;
class ES_DescriptorImp;
class InlineImp;
class DecoderFactoryBase;
class Capabilities;
class IPMPManager;
class DecoderConfigDescriptor;
class QoS_Descriptor;

class StreamConsumer : public MediaObject
{
//ML - changed from protected to public
public:
	MF<Url> url;		// changed to MF in FDIS

#ifdef PLAYER
private:
	MediaFormat m_fmtStreamFormat;
	int m_nODid;
	InlineImp *m_pSource;
	MediaStreamImp *m_pStream;
	bool m_bStreamOpen;
	ObjectDescriptorImp *m_pOD;
	bool m_bRemoteOD;
	ALManager *m_pAL;

protected:
	virtual BOOL Parse (BIFSDecoder *pBifs);

public:
	StreamConsumer ();
	virtual ~StreamConsumer ();
	virtual Decoder *CreateDecoder (
		DecoderConfigDescriptor *,
		QoS_Descriptor *,
		Decoder *pCurrentDecoder
	) {return pCurrentDecoder;}
	ObjectDescriptorImp *GetOD () const {return m_pOD;}
	void SetStreamFormat (MediaFormat fmtStreamFormat)
	{
		m_fmtStreamFormat = fmtStreamFormat;
	}
	virtual void GetCapabilities (Capabilities *pCaps);
	virtual void SetStream (ObjectDescriptorImp *pOD);
	virtual void ODRemoved ();
	void AddESD (ES_DescriptorImp *pESD);
	void RemoveESD (ES_DescriptorImp *pESD);
	void Reopen ();
	void OnConnect (const BYTE *pInitOD, int nInitODLen);
	MediaStreamImp *GetStream () {return m_pStream;}
	MediaFormat GetStreamFormat () {return m_fmtStreamFormat;}
	int GetObjectDescriptorID () {return m_nODid;}
	void OpenStream ();
	void CloseStream ();
	bool IsStreamOpen () {return m_bStreamOpen;}
	Decoder *GetDecoder () const;
	void SetInline (InlineImp *pSource) {m_pSource = pSource;}
	void SetAL (ALManager *pAL);
	
protected:
	ALManager *GetAL () const {return m_pAL;}

	CUSTOMIZE_STREAMCONSUMER_DECLARATION
#endif	// BIFSENC
};

#endif                                      // _StreamCo_h sentry.
