/***********************************************************************
This software module was originally developed by Zvi Lifshitz
(Triton R&D Ltd.) in the course of development of the MPEG-4 Systems
(ISO/IEC 14496-1) standard. This software module is an implementation of
a part of one or more MPEG-4 Systems (ISO/IEC 14496-1) tools as
specified by the MPEG-4 Systems (ISO/IEC 14496-1) standard. ISO/IEC
gives users of the MPEG-4 Systems (ISO/IEC 14496-1) free license to
this software module or modifications thereof for use in hardware or
software products claiming conformance to the MPEG-4 Systems (ISO/IEC
14496-1). Those intending to use this software module in hardware or
software products are advised that its use may infringe existing patents.
The original developer of this software module and his/her company, the
subsequent editors and their companies, and ISO/IEC have no liability
for use of this software module or modifications thereof in an
implementation. Copyright is not released for non MPEG-4 Systems
(ISO/IEC 14496-1) conforming products. Triton R&D Ltd. retains full right
to use the code for his/her own purpose, assign or donate the code to a
third party and to inhibit third parties from using the code for non
MPEG-4 Systems (ISO/IEC 14496-1) conforming products. This copyright
notice must be included in all copies or derivative works.
Copyright  1998.
************************************************************************

For more information or to receive an up to date version of this module,
you may call Zvi Lifshitz at zvil@csi.com.
----------------------------------------------------------------------*/

#if !defined(_odtags_h)	// Sentry, use file only if it's not already included.
#define _odtags_h

/*-------------------------------------------------------------------------
Project Im1Player/ Mux
INTERNATIONAL ORGANIZATION FOR STANDARDIZATION
ISO/IEC JTC1/SC29/WG11

SUBSYSTEM:  OD Decoder
FILE:       odtags.h
AUTHOR:     Zvi Lifshitz (Triton R&D Ltd.)
OVERVIEW:   Definitions of tags used by the OD parser/encoder
-------------------------------------------------------------------------*/

/*-------------------------------------------------------------
Internal list of codes to uniquely identify each descriptor
-------------------------------------------------------------*/
typedef enum
{
	CODE_BaseCommand,
	CODE_ObjectDescriptorUpdate,
	CODE_ObjectDescriptorRemove,
	CODE_ES_DescriptorUpdate,
	CODE_ES_DescriptorRemove,
	CODE_IPMP_DescriptorUpdate,
	CODE_IPMP_DescriptorRemove,
	CODE_ObjectDescriptor,
	CODE_InitialObjectDescriptor,
	CODE_ES_Descriptor,
	CODE_DecoderConfigDescriptor,
	CODE_DecoderSpecificInfo,
	CODE_SLConfigDescriptor,
	CODE_ContentIdentificationDescriptor,
	CODE_SupplementaryContentIdentificationDescriptor,
	CODE_IPI_DescrPointer,
	CODE_IPMP_DescriptorPointer,
	CODE_IPMP_Descriptor,
	CODE_QoS_Descriptor,
	CODE_QoS_Qualifier,
	CODE_QoS_Qualifier_MAX_DELAY,
	CODE_QoS_Qualifier_PREF_MAX_DELAY,
	CODE_QoS_Qualifier_LOSS_PROB,
	CODE_QoS_Qualifier_MAX_GAP_LOSS,
	CODE_QoS_Qualifier_MAX_AU_SIZE,
	CODE_QoS_Qualifier_AVG_AU_SIZE,
	CODE_QoS_Qualifier_MAX_AU_RATE,
	CODE_ExtensionDescriptor,
	CODE_RegistrationDescriptor,
	CODE_OCI_Descriptor,
	CODE_ContentClassificationDescriptor,
	CODE_KeyWordDescriptor,
	CODE_RatingDescriptor,
	CODE_LanguageDescriptor,
	CODE_ShortTextualDescriptor,
	CODE_ExpandedTextualDescriptor,
	CODE_ContentCreatorNameDescriptor,
	CODE_ContentCreationDateDescriptor,
	CODE_OCICreatorNameDescriptor,
	CODE_OCICreationDateDescriptor,
	CODE_BIFSConfig,
	CODE_BIFSv2Config,
	CODE_JPEG_DecoderConfig,
	CODE_MuxInfo,
	CODE_MuxCodeSlot,
	CODE_MuxCodeTableElement,
	CODE_MuxCodeTable,
	CODE_DecoderSpecificInfoString,
	CODE_SmpteCameraPositionDescriptor,	// NHK 990702
	CODE_SL_PacketHeader,
	CODE_GeneralAudioConfig, // gew@iis.fhg.de 990512
	CODE_OCI_Event
} OD_CODES;

typedef enum
{
	TAGMIN_BaseCommand = 0,
	TAGMAX_BaseCommand = 0XFF,
	TAG_BaseCommand = 0,
	TAG_ObjectDescriptorUpdate = 0x01,
	TAG_ObjectDescriptorRemove = 0x02,
	TAG_ES_DescriptorUpdate = 0x03,
	TAG_ES_DescriptorRemove = 0x04,
	TAG_IPMP_DescriptorUpdate = 0x05,
	TAG_IPMP_DescriptorRemove = 0x06,
	TAG_ObjectDescriptor = 0x01,
	TAG_InitialObjectDescriptor = 0x02,
	TAG_ES_Descriptor = 0x03,
	TAG_DecoderConfigDescriptor = 0x04,
	TAG_DecoderSpecificInfo = 0x05,
	TAGMIN_DecoderSpecificInfo = TAG_DecoderSpecificInfo,
	TAGMAX_DecoderSpecificInfo = TAG_DecoderSpecificInfo,
	TAG_DecoderSpecificInfoString = TAG_DecoderSpecificInfo,
	TAG_BIFSConfig = TAG_DecoderSpecificInfo,
	TAG_BIFSv2Config = TAG_DecoderSpecificInfo,
	TAG_JPEG_DecoderConfig = TAG_DecoderSpecificInfo,
	TAG_GeneralAudioConfig = TAG_DecoderSpecificInfo,
	TAG_SLConfigDescriptor = 0x06,
	TAGMIN_IP_IdentificationDataSet = 0x07,
	TAGMAX_IP_IdentificationDataSet = 0x08,
	TAG_ContentIdentificationDescriptor = 0x07,
	TAG_SupplementaryContentIdentificationDescriptor = 0x08,
	TAG_IPI_DescrPointer = 0x09,
	TAG_IPMP_DescriptorPointer = 0x0A,
	TAG_IPMP_Descriptor = 0x0B,
	TAG_QoS_Descriptor = 0x0C,
	TAGMIN_QoS_Qualifier = 1,
	TAGMAX_QoS_Qualifier = 0xff,
	TAG_QoS_Qualifier_MAX_DELAY = 0x01,
	TAG_QoS_Qualifier_PREF_MAX_DELAY = 0x02,
	TAG_QoS_Qualifier_LOSS_PROB = 0x03,
	TAG_QoS_Qualifier_MAX_GAP_LOSS = 0x04,
	TAG_QoS_Qualifier_MAX_AU_SIZE = 0x41,
	TAG_QoS_Qualifier_AVG_AU_SIZE = 0x42,
	TAG_QoS_Qualifier_MAX_AU_RATE = 0x43,
	TAG_ExtensionDescriptor = 0,
	TAGMIN_ExtensionDescriptor = 0x80,
	TAGMAX_ExtensionDescriptor = 0xfe,
	TAG_RegistrationDescriptor = 0x0D,
	TAG_OCI_Descriptor = 0,
	TAGMIN_OCI_Descriptor = 0x40,
	TAGMAX_OCI_Descriptor = 0x5f,
	TAG_ContentClassificationDescriptor = 0x40,
	TAG_KeyWordDescriptor = 0x41,
	TAG_RatingDescriptor = 0x42,
	TAG_LanguageDescriptor = 0x43,
	TAG_ShortTextualDescriptor = 0x44,
	TAG_ExpandedTextualDescriptor = 0x45,
	TAG_ContentCreatorNameDescriptor = 0x46,
	TAG_ContentCreationDateDescriptor = 0x47,
	TAG_OCICreatorNameDescriptor = 0x48,
	TAG_OCICreationDateDescriptor = 0x49,
	TAG_SmpteCameraPositionDescriptor = 0x4a,	// NHK 990702
	TAG_MuxInfo,
	TAG_MuxCodeSlot,
	TAG_MuxCodeTableElement,
	TAG_MuxCodeTable,
	TAG_SL_PacketHeader,
	TAG_OCI_Event = 0
} OD_TAGS;

#endif                                      // _odtags_h sentry.
