/***********************************************************************
    This software module  was  originally  developed by Zvi  Lifshitz
    (Triton R&D Ltd.), Mauro Rossi (CSELT) and Guido Franceschini
    (CSELT), and edited by Michael Stepping (FTK), Reinhard Bayer
    (GMD Fokus) and Carsten Herpel(THOMSON multimedia).
    in the course of development of the MPEG-4 DMIF (ISO/IEC 14496-6)
    standard.  This software module is an implementation of a part of
    one or more MPEG-4  DMIF  (ISO/IEC 14496-6)  tools  as  specified
    by  the  MPEG-4 DMIF (ISO/IEC 14496-6)  standard.  ISO/IEC  gives
    users of  the  MPEG-4  DMIF  (ISO/IEC 14496-6)  free  license  to
    this software module or modifications thereof for use in hardware
    or software products claiming  conformance  to  the  MPEG-4  DMIF 
    (ISO/IEC 14496-6). Those intending to use this software module in
    hardware or software  products  are  advised  that  its  use  may
    infringe   existing  patents.  The  original  developer  of  this
    software module and his/her company, the subsequent  editors  and
    their  companies,  and  ISO/IEC have no liability for use of this
    software module or modifications thereof  in  an  implementation.
    Copyright  is not released for non MPEG-4 DMIF  (ISO/IEC 14496-6)
    conforming  products. Triton R&D Ltd. and CSELT
    retain  full  right to use the code for their own purpose, assign
    or donate the code to a third party and to inhibit third  parties
    from  using  the  code  for  non  MPEG-4  DMIF  (ISO/IEC 14496-6)
    conforming products. This copyright notice must  be  included  in
    all copies or derivative works. Copyright  1998.
************************************************************************/

// Contact: guido.franceschini@cselt.it

/**************************************************************************
 * Project Im1Player
 * INTERNATIONAL STANDARD ORGANIZATION
 * ISO/IEC JTC1/SC29/WG11
 * 
 * SUBSYSTEM: DMIF
 * FILE:      DMIFClientFilter.cpp
 * AUTHOR:    Mauro Rossi (CSELT)
 * OVERVIEW:  Implementation of DMIFClientFilter, the class that implements the
 *			  generic layer of DMIF.
 * 
 **************************************************************************/

#include <iostream.h>
#include "DMIFClientFilter.h"

#include <fstream.h>
ofstream	log0,log1,log2,log3;

/*-------------------------------------------------------------
The exported function that instantiates the proper filter
-------------------------------------------------------------*/
extern "C" DMIF_Filter* CreateInstance ()
{
	return new DMIFClientFilter();
}


DMIFClientFilter :: DMIFClientFilter ()
{
	log0.attach(stdout->_file);//screen log
	log1.attach(stdout->_file);//screen log
	log2.attach(stdout->_file);//screen log
	log3.attach(stdout->_file);//screen log
}

/*-------------------------------------------------------------
DMIFClientFilter destructor. Delete the assoicated Service, and all FlexDemux
objects (DataChannels are deleted by the application)
-------------------------------------------------------------*/
DMIFClientFilter :: ~DMIFClientFilter ()
{
	int i,j;
	struct serviceDesc* pServiceDesc;

	m_smpServices.P();
	for (i = m_loadedServices.GetLength (); --i >= 0;)
	{
		pServiceDesc = m_loadedServices[i];
		pServiceDesc->m_mtxService.P();
		if (pServiceDesc->m_nUse == 0) {
			pServiceDesc->m_mtxChannelMap.P ();
			for (j = pServiceDesc->m_channelMap.GetLength(); --j >= 0;)
			{
			// GF: it is not clear whether the following "if" is required
				if (pServiceDesc->m_channelMap [j].m_bIsFlexMux)
					delete pServiceDesc->m_channelMap [j].m_pChannel;
			}
			pServiceDesc->m_channelMap.Clear();
			pServiceDesc->m_mtxChannelMap.V ();
			delete pServiceDesc->m_pService;
			if (pServiceDesc->m_hDLL)
				FreeLibrary (pServiceDesc->m_hDLL);
			delete pServiceDesc;
			m_loadedServices.DeleteFrom(i);
		} else {
			pServiceDesc->m_mtxService.V();
		}
	}
	m_smpServices.V ();
	while (m_loadedServices.GetLength ()) {
		Sleep(1000);
		m_smpServices.P();
		for (i = m_loadedServices.GetLength (); --i >= 0;)
		{
			pServiceDesc = m_loadedServices[i];
			pServiceDesc->m_mtxService.P();
			if (pServiceDesc->m_nUse == 0) {
				pServiceDesc->m_mtxChannelMap.P ();
				for (j = pServiceDesc->m_channelMap.GetLength(); --j >= 0;)
				{
				// GF: it is not clear whether the following "if" is required
					if (pServiceDesc->m_channelMap [j].m_bIsFlexMux)
						delete pServiceDesc->m_channelMap [j].m_pChannel;
				}
				pServiceDesc->m_channelMap.Clear();
				pServiceDesc->m_mtxChannelMap.V ();
				delete pServiceDesc->m_pService;
				if (pServiceDesc->m_hDLL)
					FreeLibrary (pServiceDesc->m_hDLL);
				delete pServiceDesc;
				m_loadedServices.DeleteFrom(i);
			} else {
				pServiceDesc->m_mtxService.V();
			}
		}
		m_smpServices.V ();
	}
} 

struct serviceDesc* DMIFClientFilter :: LockService (const Service_base* pService) 
{
	int i;
	struct serviceDesc* pServiceDesc=NULL;

	m_smpServices.P();
	for (i = m_loadedServices.GetLength (); --i >= 0;)
	{
		if (m_loadedServices[i]->m_pService == pService)
		{
			pServiceDesc = m_loadedServices[i];
			m_smpServices.V();
			pServiceDesc->m_mtxService.P();
			pServiceDesc->m_nUse++;
			pServiceDesc->m_mtxService.V();
			break;
		}
	}
	if (pServiceDesc==NULL)
		m_smpServices.V();
	return (pServiceDesc);
}

struct serviceDesc* DMIFClientFilter :: LockService (const UINT nServiceId) 
{
	int i;
	struct serviceDesc* pServiceDesc=NULL;

	m_smpServices.P();
	for (i = m_loadedServices.GetLength (); --i >= 0;)
	{
		if (m_loadedServices[i]->m_nServiceId == nServiceId)
		{
			pServiceDesc = m_loadedServices[i];
			m_smpServices.V();
			pServiceDesc->m_mtxService.P();
			pServiceDesc->m_nUse++;
			pServiceDesc->m_mtxService.V();
			break;
		}
	}
	if (pServiceDesc==NULL)
		m_smpServices.V();
	return (pServiceDesc);
}

void DMIFClientFilter :: UnlockService (struct serviceDesc* pServiceDesc) 
{
	m_smpServices.P();
	pServiceDesc->m_mtxService.P();
	pServiceDesc->m_nUse--;
	pServiceDesc->m_mtxService.V();
	m_smpServices.V();
}

/*-------------------------------------------------------------
Service_Attach creates a proper Service object and calls its
Service_Attach method
-------------------------------------------------------------*/
void DMIFClientFilter::DAI_Service_Attach_Req(
	const UINT nTmpId,
	const char* dest_addr,
	const BYTE* uuData,
	const u_short uuData_length)
{
	static ZRegistry registry("Software\\MPEG-4\\IM1\\DMIFServices\\Unified"); // XXX LocalMachine
	int i;
	Service_base* pService=NULL;
	HMODULE	hDLL=NULL;
	RESPONSE eResponse = RESPONSE_SERVICEDLL_NOT_FOUND;

	/*-------------------------------------------------------------
	Find which instance can serve this URL. First try the services that
	are already loaded
	-------------------------------------------------------------*/
	m_smpServices.P ();
	for (i = m_loadedServices.GetLength (); --i >= 0;)
	{
		Service_base* pLoadedService = m_loadedServices[i]->m_pService;
		if (pLoadedService -> IsYourURL (dest_addr))
		{	pService = (Service_base*) registry.CreateInstance (
				pLoadedService -> GetModuleID (), hDLL
			);
			if (!pService)
				eResponse = RESPONSE_SERVICEDLL_ERROR;
			break;
		}
	}

	/*-------------------------------------------------------------
	If none of the existing services can handle this URL, try all registered
	services.
	-------------------------------------------------------------*/
	if (i < 0)
	{	char szBuffer [100];
		for (i = 0; registry.Enumerate (i, szBuffer, sizeof szBuffer); ++i)
		{	pService =
				(Service_base*) registry.CreateInstance (szBuffer, hDLL);
			if (pService)
			{	if (pService -> IsYourURL (dest_addr))
					break;
				else
				{	delete pService;
					FreeLibrary (hDLL);
					pService = NULL;
					hDLL = NULL;
				}
			}
			else
				eResponse = RESPONSE_SERVICEDLL_ERROR;
		}
	}
	
	/*-------------------------------------------------------------
	Add the new service to the service list, and call its ServiceAttach
	method. Or, if service could not be instantiated, report to the 
	application ServiceDetach.
	-------------------------------------------------------------*/
	if (pService)
	{
		struct serviceDesc* pServiceDesc = new(struct serviceDesc);
		pServiceDesc->m_eStatus=0;	// SERVICE_ACTIVE
		pServiceDesc->m_nUse=0;
		pServiceDesc->m_hDLL=hDLL;
		pServiceDesc->m_pService=pService;
		pServiceDesc->m_nTmpId=nTmpId;
		pServiceDesc->m_nServiceId=m_nServiceId++;
		m_loadedServices += pServiceDesc;
	}
	m_smpServices.V ();
	if (pService)
	{	
		pService -> SetSink (this);
		pService -> DPI_ServiceAttach_Req(dest_addr, uuData, uuData_length);
	}
	else
	{
		GetSink () -> DAI_Service_Attach_Cnf(nTmpId,0,NULL,0,eResponse);
	}
	return;
}

/*-------------------------------------------------------------
Pass a Service_Detach call to the service
-------------------------------------------------------------*/
void DMIFClientFilter::DAI_Service_Detach_Req(
	const UINT nServiceId,
	const REASON reason)
{
	serviceDesc* pServiceDesc=LockService(nServiceId);
	if (pServiceDesc->m_pService==NULL)
	{
		GetSink () -> DAI_Service_Detach_Cnf(pServiceDesc->m_nServiceId,RESPONSE_SERVICE_DOES_NOT_EXIST);
		return;
	}
	pServiceDesc->m_pService -> DPI_ServiceDetach_Req(reason);
	UnlockService(pServiceDesc);
	return;
}

/*-------------------------------------------------------------
Pass a Channel_Add call to the service
-------------------------------------------------------------*/
void DMIFClientFilter::DAI_Channel_Add_Req(
	const UINT nServiceId,
	const DAIChannelAddParam& reqParam)
{
	serviceDesc* pServiceDesc=LockService(nServiceId);

	if (pServiceDesc->m_pService==NULL)
	{
		DAIChannelAddParam cnfParam = reqParam;
		for(int i=0;i<cnfParam.GetLength();i++)
		{
			delete ( (MyChannel*)(cnfParam[i].m_nChid) );
			cnfParam[i].m_nChid = 0;
			cnfParam[i].m_eResponse = RESPONSE_SERVICE_DOES_NOT_EXIST;
		}
		GetSink () -> DAI_Channel_Add_Cnf(pServiceDesc->m_nServiceId,cnfParam);
		cnfParam.Clear();
		return;
	}
	for(int i=0;i<reqParam.GetLength();i++)
	{
		reqParam[i].m_nChid = (UINT)new MyChannel(GetSink());
	}

	// this local parameter loop is passed to specific service
	pServiceDesc->m_pService -> DPI_ChannelAdd_Req(reqParam);
	// after returning, acces to this loop is no longer allowed

	UnlockService(pServiceDesc);
	return;
}

/*-------------------------------------------------------------
Pass a Channel_Delete call to the service
-------------------------------------------------------------*/
void DMIFClientFilter::DAI_Channel_Delete_Req(
	const UINT nServiceId,
	const DAIChannelDeleteParam& reqParam)
{
	int i;
	DAIChannelDeleteParam cnfParam,dpiReqParam;

	serviceDesc* pServiceDesc=LockService(nServiceId);

	if (pServiceDesc->m_pService==NULL)
	{
		for(i=0;i<reqParam.GetLength();i++)
		{
			cnfParam += reqParam[i];
			cnfParam[i].m_eResponse = RESPONSE_SERVICE_DOES_NOT_EXIST;
		}
		GetSink () -> DAI_Channel_Delete_Cnf(pServiceDesc->m_nServiceId,cnfParam);
		cnfParam.Clear();
		return;
	}

	for(i=0;i<reqParam.GetLength();i++)
	{
			dpiReqParam += reqParam[i];
	}
	if(dpiReqParam.GetLength() > 0) {
		pServiceDesc->m_pService -> DPI_ChannelDelete_Req(dpiReqParam);
		dpiReqParam.Clear();
	}

	UnlockService(pServiceDesc);
	return;
}

/*-------------------------------------------------------------
Pass a User_Command call to the service
-------------------------------------------------------------*/
void DMIFClientFilter::DAI_User_Command_Req(const UINT nServiceId, const DAIUserCmdParam& inParam)
{
	serviceDesc* pServiceDesc=LockService(nServiceId);

	if (pServiceDesc->m_pService==NULL)
	{
		return;
	}
	pServiceDesc->m_pService -> DPI_UserCommand_Req(inParam);

	UnlockService(pServiceDesc);
	return;
}

/*-------------------------------------------------------------
Pass a User_Command_Ack call to the service
-------------------------------------------------------------*/
void DMIFClientFilter::DAI_User_Command_Ack_Req(const UINT nServiceId, const DAIUserCmdParam& inParam)
{
	serviceDesc* pServiceDesc=LockService(nServiceId);

	if (pServiceDesc->m_pService==NULL)
	{
		DAIUserCmdParam outParam;
		outParam.m_nChLen=0;
		outParam.m_pChid=NULL;
		outParam.m_pData=NULL;
		outParam.m_nDataLen=0;
		outParam.m_nCmdId=inParam.m_nCmdId;
		outParam.m_eResponse=RESPONSE_SERVICEDLL_NOT_FOUND;
		GetSink () -> DAI_User_Command_Ack_Cnf(pServiceDesc->m_nServiceId,outParam);
		return;
	}
	pServiceDesc->m_pService -> DPI_UserCommand_Ack_Req(inParam);

	UnlockService(pServiceDesc);
	return;
}

/*-------------------------------------------------------------
Pass Service_Attach callback to the application
-------------------------------------------------------------*/
void DMIFClientFilter :: DPI_ServiceAttach_Cnf (
	const Service_base* pService,
	const BYTE *pUuData,
	const int nUuDataLen,
	const RESPONSE eResponse
)
{
	serviceDesc* pServiceDesc = LockService(pService);

	GetSink () -> DAI_Service_Attach_Cnf(pServiceDesc->m_nTmpId,pServiceDesc->m_nServiceId,pUuData,nUuDataLen,eResponse);

	UnlockService(pServiceDesc);
	return;
}

/*-------------------------------------------------------------
Pass Service_Detach callback to the application
-------------------------------------------------------------*/
void DMIFClientFilter :: DPI_ServiceDetach_Cnf (
	const Service_base* pService,
	const RESPONSE eResponse)
{
	serviceDesc* pServiceDesc=LockService(pService);

	GetSink () -> DAI_Service_Detach_Cnf(pServiceDesc->m_nServiceId,eResponse);

	UnlockService(pServiceDesc);
	return;
}

/*-------------------------------------------------------------
A FlexMux channel is added to an existing TransMux channel
-------------------------------------------------------------*/
void DMIFClientFilter :: DPI_ChannelAdd_Cnf (
	const Service_base* pService,
	const StreamMapElement &outParam,
	const RESPONSE eResponse
)
{
	serviceDesc* pServiceDesc=LockService(pService);

	DAIChannelAddParam cnfParam;
	DAIChannelAddElement* pCnfEl = new DAIChannelAddElement();
	if (outParam.m_nChid)
		delete (void *)(outParam.m_nChid);
	pCnfEl->m_nChid = NULL;
	pCnfEl->m_nTmpId = outParam.m_nTmpId;
	pCnfEl->m_eResponse = eResponse;
	pCnfEl->m_pQoSStream = NULL;		// unused field
	pCnfEl->m_eDirection = UNSPECIFIED;	// useless field; just avoid random values
	pCnfEl->m_nDataLen = outParam.m_nDataLen;
	pCnfEl->m_pData = new BYTE[pCnfEl->m_nDataLen];
	memcpy(pCnfEl->m_pData,outParam.m_pData,pCnfEl->m_nDataLen);
	cnfParam += *pCnfEl;
	delete pCnfEl;
	GetSink() -> DAI_Channel_Add_Cnf(pServiceDesc->m_nServiceId,cnfParam);

	UnlockService(pServiceDesc);
	return;
}

/*-------------------------------------------------------------
A FlexMux channel is removed from an existing TransMux channel
-------------------------------------------------------------*/
void DMIFClientFilter :: DPI_ChannelDelete_Cnf (
	const Service_base* pService,
	const HANDLE hTransmux, //TAT
	const int nFmxInfo,
	const RESPONSE eResponse
)
{
	serviceDesc* pServiceDesc=LockService(pService);

	pServiceDesc->m_mtxChannelMap.P ();
	int i = pServiceDesc->m_channelMap.Find (hTransmux);
	if (i >= 0)
	{
		FlexDemux* pFlexDemux = (FlexDemux *)(pServiceDesc->m_channelMap [i].m_pChannel);
		pFlexDemux->DeleteFlexmuxChannel(pServiceDesc->m_nServiceId,nFmxInfo,eResponse);

		if(pFlexDemux->isEmpty())
		{//if empty, delete the transmux.
			delete pFlexDemux;
			pFlexDemux = NULL;
			pServiceDesc->m_channelMap.DeleteFrom(i);
		}
	}
	else
	{
		DAIChannelDeleteParam cnfParam;
		DAIChannelDeleteElement cnfEl;
		cnfEl.m_nChid = 0;
		cnfEl.m_eResponse = RESPONSE_OK; // RESPONSE_CHANNEL_NOT_FOUND;
		cnfEl.m_eReason = REASON_UNKNOWN_ERROR ;
		cnfParam += cnfEl;
		GetSink () -> DAI_Channel_Delete_Cnf(pServiceDesc->m_nServiceId,cnfParam);
		cnfParam.Clear();
	}
	pServiceDesc->m_mtxChannelMap.V ();

	UnlockService(pServiceDesc);
	return;
}

/*-------------------------------------------------------------
The answer to the UserCommand is received
-------------------------------------------------------------*/
void DMIFClientFilter :: DPI_UserCommand_Ack_Cnf(
	const Service_base* pService,
	const DAIUserCmdParam& outParam)
{
	serviceDesc* pServiceDesc=LockService(pService);

	GetSink()->DAI_User_Command_Ack_Cnf(pServiceDesc->m_nServiceId,outParam);

	UnlockService(pServiceDesc);
	return;
}

/*-------------------------------------------------------------
A new TransMux channel is opened by the server
-------------------------------------------------------------*/
void DMIFClientFilter :: DPI_TransmuxAdd_Cnf (
	const Service_base* pService,
	const HANDLE hTransmux,
	const FLEXMUX_TYPE eFmxType,
	const DPI_TransmuxAdd_CnfParam &outParam
)
{
	int i;

	serviceDesc* pServiceDesc=LockService(pService);

	/*-----------------------------------------------------------------
	If this is a null-FlexMux channel, the map table contains just one
	entry.
	-----------------------------------------------------------------*/
	if (eFmxType == NULL_FLEXMUX)
	{
		OnChannelOpen (pServiceDesc,hTransmux,outParam[0]);

		UnlockService(pServiceDesc);
		return;
	}

	/*-----------------------------------------------------------------
	Create a FlexDemux object for this channel, and update its SMT.
	-----------------------------------------------------------------*/
	pServiceDesc->m_mtxChannelMap.P ();
	int h = pServiceDesc->m_channelMap.Find (hTransmux);
	if (h >= 0)
	{	//hTransmux exists already.
		for (i = outParam.GetLength (); --i >= 0;)
		{
			((FlexDemux *)(pServiceDesc->m_channelMap)[h].m_pChannel)->AddFlexmuxChannel(pServiceDesc->m_nServiceId,outParam[i]);
		}
	}
	else
	{
		FlexDemux *pFlexDemux = new FlexDemux (GetSink ());
		pServiceDesc->m_channelMap += ChannelMapEntry (hTransmux, pFlexDemux, TRUE);
		for (i = outParam.GetLength (); --i >= 0;)
		{
			pFlexDemux -> AddFlexmuxChannel (pServiceDesc->m_nServiceId,outParam[i]);
		}
	}
	pServiceDesc->m_mtxChannelMap.V ();

	UnlockService(pServiceDesc);
	return;
}

/*-------------------------------------------------------------
An existing TransMux channel is deleted
-------------------------------------------------------------*/
void DMIFClientFilter :: DPI_TransmuxDelete_Cnf (
	const Service_base* pService,
	const HANDLE hTransmux,
	const RESPONSE eResponse)
{
	serviceDesc* pServiceDesc=LockService(pService);

	pServiceDesc->m_mtxChannelMap.P ();
	int i = pServiceDesc->m_channelMap.Find (hTransmux);
	if (i >= 0)
	{	
		Channel *pChannel = pServiceDesc->m_channelMap [i].m_pChannel;
		pChannel -> DPIC_ChannelDelete_Cnf (pServiceDesc->m_nServiceId,eResponse);

		delete pChannel;	
		pServiceDesc->m_channelMap.DeleteFrom (i);
	}
	else
	{
		DAIChannelDeleteParam cnfParam;
		DAIChannelDeleteElement cnfEl;
		cnfEl.m_nChid = 0;
		cnfEl.m_eResponse = RESPONSE_OK; // RESPONSE_CHANNEL_NOT_FOUND;
		cnfEl.m_eReason = REASON_UNKNOWN_ERROR ;
		cnfParam += cnfEl;
		GetSink () -> DAI_Channel_Delete_Cnf(pServiceDesc->m_nServiceId,cnfParam);
		cnfParam.Clear();
	}
	pServiceDesc->m_mtxChannelMap.V ();

	UnlockService(pServiceDesc);
	return;
}

/*-----------------------------------------------------------------
Pass a received TransMux data packet to the proper channel object
-----------------------------------------------------------------*/
void DMIFClientFilter :: DPI_DataReceived_Ind (
	const Service_base* pService,
	const HANDLE hTransmux,
	const BYTE *pStreamData,
	const int nStreamDataLength,
	const BOOL bErr)
{
	serviceDesc* pServiceDesc=LockService(pService);

	pServiceDesc->m_mtxChannelMap.P ();
	int i = pServiceDesc->m_channelMap.Find (hTransmux);
	if (i >= 0)
	{	
		Channel *pChannel = pServiceDesc->m_channelMap [i].m_pChannel;
		pChannel -> LockChannel ();
		pServiceDesc->m_mtxChannelMap.V ();
		pChannel -> DPIC_PacketReceived_Ind (pStreamData, nStreamDataLength, bErr);
		pChannel -> UnlockChannel ();
	}
	else
		pServiceDesc->m_mtxChannelMap.V ();

	UnlockService(pServiceDesc);
	return;
}

/*-------------------------------------------------------------------
Pass received MuxCodeTableElement(s) to the proper FlexDemux object
-------------------------------------------------------------------*/
void DMIFClientFilter :: DPI_MuxCodeTableReceived_Ind (
	const Service_base* pService,
	const HANDLE hTransmux,
	const LinearMuxCodeTable &muxCodeTable,
	const BOOL bErr)
{
	serviceDesc* pServiceDesc=LockService(pService);

	if (bErr)	// no handling of erroneous mcte's implemented (can't do much anyway)
		return;

#ifdef DEBUG
	printf("DMIF received muxcode table with %d entries (ignored)\n",muxCodeTable.GetLength());
#endif

	// the code below is yet to be tested with a suitable DMIF instance *CH* 990621
	pServiceDesc->m_mtxChannelMap.P ();
	int i = pServiceDesc->m_channelMap.Find (hTransmux);
	if (i >= 0)
	{	
		FlexDemux *pChannel = (FlexDemux*) (pServiceDesc->m_channelMap [i].m_pChannel);
		pChannel -> LockChannel ();
		pServiceDesc->m_mtxChannelMap.V ();
		pChannel -> UpdateMuxCodeTableElements (muxCodeTable);
		pChannel -> UnlockChannel ();
	}
	else
		pServiceDesc->m_mtxChannelMap.V ();

	UnlockService(pServiceDesc);
	return;
}

/*-------------------------------------------------------------
A null-FlexMux channel is opened
-------------------------------------------------------------*/
void DMIFClientFilter :: OnChannelOpen (
	struct serviceDesc* pServiceDesc,
	const HANDLE hTransmux,
	const StreamMapElement& sme)
{
	/*-----------------------------------------------------------------
	Request the application to instantiate a DataChannel object.
	Saves an association between the handle and the object.
	-----------------------------------------------------------------*/
	pServiceDesc->m_mtxChannelMap.P ();

	DAIChannelAddParam cnfParam;
	 
	DAIChannelAddElement* pCnfEl = new DAIChannelAddElement();
	pCnfEl->m_nChid = sme.m_nChid;
	pCnfEl->m_nTmpId = sme.m_nTmpId;
	pCnfEl->m_eResponse = RESPONSE_OK;
	pCnfEl->m_pQoSStream = NULL;		// unused field
	pCnfEl->m_eDirection = UNSPECIFIED;	// useless field; just avoid random values
	pCnfEl->m_nDataLen = sme.m_nDataLen;
	pCnfEl->m_pData = new BYTE[pCnfEl->m_nDataLen];
	memcpy(pCnfEl->m_pData,sme.m_pData,pCnfEl->m_nDataLen);
	cnfParam += *pCnfEl;

	GetSink() -> DAI_Channel_Add_Cnf(pServiceDesc->m_nServiceId,cnfParam);

	Channel *pChannel =  (MyChannel*)sme.m_nChid;

	if (pChannel)
	{
		ChannelMapEntry *pEntry = new ChannelMapEntry (hTransmux, pChannel, FALSE);
		pServiceDesc->m_channelMap += *pEntry;
		delete pEntry;
	}
	delete pCnfEl;
	pServiceDesc->m_mtxChannelMap.V ();
}


///////////////////////////////////////////
/// UTILITY FUNCTIONs
///////////////////////////////////////////

int GetQOSSTREAMLength(QOSSTREAM qos)
{
	int ret;

	if( qos == NULL)
	{
		log2<<"qos NULL"<<endl;
		return 0;
	}

	if( qos[0] != 0xC )
	{
		log0<<"qos[0] != 0xC"<<endl;
		return -1;  // It is not a QOS descriptor;
	}

	if( qos[1] & 0x80 ) 
	{
		log0<<"qos TOO long"<<endl;
		return -1; // Qos length greater than 127+2 bytes. Not yet supported.
	}

	ret = 2 + (int)( qos[1] & 0x7F );

	return ret;
}

void PrintQOSSTREAMString(QOSSTREAM qos)
{
	int length = GetQOSSTREAMLength(qos);
	int i;

	if(qos==NULL)
	{
		log2<<" EMPTY (QOS == NULL)"<<endl;
		return;
	}

	if(length == -1)
	{
		log2<<" EMPTY (NOT QOS DESCRIPTOR)"<<endl;
		return;
	}
	if(length==0)
	{
		log2<<" EMPTY (LENGTH == 0)"<<endl;
		return;
	}
	log2<<hex;

	for(i=0;i<length;i++) // qos[0] and qos[1] = 2bytes
	{
		log2<<(unsigned int) (qos[i])<<".";
	}
	log2<<dec<<endl;
	for(i=3;i<length;i++) // qos[0] and qos[1] = 2bytes
	{
/*		log2 << i<< endl;;
		log2 << "qos = " << (unsigned int) qos[i] << endl;*/
		switch (qos[i])
		{
		case 0x01:{
				log2 << "MAX_DELAY = ";
				union vv {
					BYTE		 bvalue[4];
					unsigned int ivalue;
				}v;
				v.bvalue[0] = 0x00;
				v.ivalue = 0;
				int l = qos[++i];
				for (int t = 0; t<l; t++)
					v.bvalue[l-1-t] = qos[i+1+t];
				i += l;
				log2 << "*"<< (unsigned int )v.ivalue << "*"<< endl;
/*				for (t = 0; t<4; t++)
					log2 << (unsigned int )v.bvalue[t] << "-";*/
				break;}
		case 0x02:{
				log2 << "AVG_DELAY = ";
				union vv {
					BYTE		 bvalue[4];
					unsigned int ivalue;
				}v;
				v.bvalue[0] = 0x00;
				v.ivalue = 0;
				int l = qos[++i];
				for (int t = 0; t<l; t++)
					v.bvalue[l-1-t] = qos[i+1+t];
				i += l;
				log2 << "*"<< (unsigned int )v.ivalue << "*"<< endl;
				break;}
		case 0x03:{
				log2 << "LOSS_PROB = ";
				union vv {
					BYTE	 bvalue[4];
					float	 ivalue;
				}v;
				v.bvalue[0] = 0x00;
				v.ivalue = 0;
				int l = qos[++i];
				for (int t = 0; t<l; t++)
					v.bvalue[l-1-t] = qos[i+1+t];
				i += l;
				log2 << "*"<< (double )v.ivalue << "*"<< endl;
				break;}
		case 0x04:{
				log2 << "MAX_GAP_LOSS = ";
				union vv {
					BYTE		 bvalue[4];
					unsigned int ivalue;
				}v;
				v.bvalue[0] = 0x00;
				v.ivalue = 0;
				int l = qos[++i];
				for (int t = 0; t<l; t++)
					v.bvalue[l-1-t] = qos[i+1+t];
				i += l;
				log2 << "*"<< (unsigned int )v.ivalue << "*"<< endl;
				break;}
		case 0x41:{
				log2 << "MAX_AU_SIZE = ";
				union vv {
					BYTE		 bvalue[4];
					unsigned int ivalue;
				}v;
				v.bvalue[0] = 0x00;
				v.ivalue = 0;
				int l = qos[++i];
				for (int t = 0; t<l; t++)
					v.bvalue[l-1-t] = qos[i+1+t];
				i += l;
				log2 << "*"<< (unsigned int )v.ivalue << "*"<< endl;
				break;}
		case 0x42:{
				log2 << "AVG_AU_SIZE = ";
				union vv {
					BYTE		 bvalue[4];
					unsigned int ivalue;
				}v;
				v.bvalue[0] = 0x00;
				v.ivalue = 0;
				int l = qos[++i];
				for (int t = 0; t<l; t++)
					v.bvalue[l-1-t] = qos[i+1+t];
				i += l;
				log2 << "*"<< (unsigned int )v.ivalue << "*"<< endl;
				break;}
		case 0x43:{
				log2 << "MAX_AU_RATE = ";
				union vv {
					BYTE		 bvalue[4];
					unsigned int ivalue;
				}v;
				v.bvalue[0] = 0x00;
				v.ivalue = 0;
				int l = qos[++i];
				for (int t = 0; t<l; t++)
					v.bvalue[l-1-t] = qos[i+1+t];
				i += l;
				log2 << "*"<< (unsigned int )v.ivalue << "*"<< endl;
				break;}
		default:{
				log2 << "UNKNOWN QUALIFIER " << endl;
				int l = qos[++i];
				i = i+l;
				break;}
		}
	}
}
