/***********************************************************************
    This software module  was  originally  developed by Zvi  Lifshitz
    (Triton R&D Ltd.) and edited by Mauro Rossi (CSELT),
    Guido Franceschini (CSELT) and Michael Stepping (FTK),
    Reinhard Bayer (GMD Fokus) and Carsten Herpel (THOMSON multimedia)
    in the course of development of the MPEG-4 DMIF (ISO/IEC 14496-6)
    standard.  This software module is an implementation of a part of
    one or more MPEG-4  DMIF  (ISO/IEC 14496-6)  tools  as  specified
    by  the  MPEG-4 DMIF (ISO/IEC 14496-6)  standard.  ISO/IEC  gives
    users of  the  MPEG-4  DMIF  (ISO/IEC 14496-6)  free  license  to
    this software module or modifications thereof for use in hardware
    or software products claiming  conformance  to  the  MPEG-4  DMIF 
    (ISO/IEC 14496-6). Those intending to use this software module in
    hardware or software  products  are  advised  that  its  use  may
    infringe   existing  patents.  The  original  developer  of  this
    software module and his/her company, the subsequent  editors  and
    their  companies,  and  ISO/IEC have no liability for use of this
    software module or modifications thereof  in  an  implementation.
    Copyright  is not released for non MPEG-4 DMIF  (ISO/IEC 14496-6)
    conforming  products. Triton R&D Ltd.
    retains full  right to use the code for  its  own purpose, assign
    or donate the code to a third party and to inhibit third  parties
    from  using  the  code  for  non  MPEG-4  DMIF  (ISO/IEC 14496-6)
    conforming products. This copyright notice must  be  included  in
    all copies or derivative works. Copyright  1998.
************************************************************************/

// Contact: guido.franceschini@cselt.it

/**************************************************************************
 * Project Im1Player
 * INTERNATIONAL STANDARD ORGANIZATION
 * ISO/IEC JTC1/SC29/WG11
 * 
 * SUBSYSTEM: DMIF
 * FILE:      FlexDemux.h
 * AUTHOR:    Mauro Rossi (CSELT)
 * OVERVIEW:  Declarations of FlexDemux.
 * 
 **************************************************************************/

#if !defined(_FlexDemux_h)	// Sentry, use file only if it's not already included.
#define _FlexDemux_h

#include "Channel.h"
#include "MyChannel.h"
#include "FlexMuxTool.h"
#include "DPI.h"

class FlexDemux : public Channel
{
private:
	virtual void DPIC_ChannelDelete_Cnf (
		const UINT nServiceId,
		const RESPONSE eResponse);
	virtual void DPIC_PacketReceived_Ind (
		const BYTE *pStreamData,
		const int nStreamDataLength,
		const BOOL bErr
	);
	virtual void DemuxMuxCodePacket(
		LinearMuxCodeTableElement &mcte, 
		const BYTE *pStreamData,
		const int nLength
	);

//--- dummies:
	virtual void DPIC_ChannelDelete_Ind	(
		const Service_base* pService,
		const RESPONSE eResponse)
		{pService;eResponse;};
	virtual BOOL ReceivePacket (
		BYTE *pDataStream,
		int &nDataStreamLength)
		{pDataStream;nDataStreamLength;return FALSE;}
	virtual BOOL SendPacket (
		const BYTE *pStreamData,
		const int nStreamDataLength,
		const long nTMin,
		const long nTMax)
		{pStreamData;nStreamDataLength;nTMin;nTMax;return FALSE;}
public:
	FlexDemux (Application_Sink* pSink);
	~FlexDemux();
	void AddFlexmuxChannel (
		const UINT nServiceId,
		const StreamMapElement &outParam);
	void DeleteFlexmuxChannel (
		const UINT nServiceId,
		const int nFmxInfo,
		const RESPONSE eResponse);
	bool isEmpty();
	void UpdateMuxCodeTableElements(
		const LinearMuxCodeTable &muxCodeTable);
	ZArray<MyChannel*> m_channelTable;
private:
	Application_Sink* m_pSink;
	LinearMuxCodeTable m_muxCodeTable;
};

#endif                                      // _FlexDemux_h sentry.
