/***********************************************************************
    This software module  was  originally  developed by Zvi  Lifshitz
    (Triton R&D Ltd.), Mauro Rossi (CSELT) and Guido Franceschini
    (CSELT), and edited by Michael Stepping (FTK)
    in the course of development of the MPEG-4 DMIF (ISO/IEC 14496-6)
    standard.  This software module is an implementation of a part of
    one or more MPEG-4  DMIF  (ISO/IEC 14496-6)  tools  as  specified
    by  the  MPEG-4 DMIF (ISO/IEC 14496-6)  standard.  ISO/IEC  gives
    users of  the  MPEG-4  DMIF  (ISO/IEC 14496-6)  free  license  to
    this software module or modifications thereof for use in hardware
    or software products claiming  conformance  to  the  MPEG-4  DMIF 
    (ISO/IEC 14496-6). Those intending to use this software module in
    hardware or software  products  are  advised  that  its  use  may
    infringe   existing  patents.  The  original  developer  of  this
    software module and his/her company, the subsequent  editors  and
    their  companies,  and  ISO/IEC have no liability for use of this
    software module or modifications thereof  in  an  implementation.
    Copyright  is not released for non MPEG-4 DMIF  (ISO/IEC 14496-6)
    conforming  products. Triton R&D Ltd. and CSELT
    retain  full  right to use the code for their own purpose, assign
    or donate the code to a third party and to inhibit third  parties
    from  using  the  code  for  non  MPEG-4  DMIF  (ISO/IEC 14496-6)
    conforming products. This copyright notice must  be  included  in
    all copies or derivative works. Copyright  1998.
************************************************************************/

/**************************************************************************
 * Project Im1Player
 * INTERNATIONAL STANDARD ORGANIZATION
 * ISO/IEC JTC1/SC29/WG11
 * 
 * SUBSYSTEM: DMIF
 * FILE:      Channel.h
 * OVERVIEW:  Definitions of the Channel base object used in the DMIF Filter.
 * 
 **************************************************************************/

#ifndef _Channel_h_
#define _Channel_h_

#include "DAI.h"
#include "ChSemList.h"

class Service_base;

class Channel
{
public:

	virtual ~Channel ()
	{}
	virtual void DPIC_ChannelDelete_Cnf(
		const UINT nServiceId,
		const RESPONSE eResponse
	) = 0;
	virtual void DPIC_PacketReceived_Ind(
		const BYTE *pStreamData,
		const int nStreamDataLength,
		const BOOL bErr
	) = 0;
//----
	virtual void	DPIC_ChannelDelete_Ind(
		const Service_base* pService,
		const RESPONSE eResponse
	) = 0;
	virtual BOOL	ReceivePacket(
		BYTE* pDataStream,
		int& nDataStreamLength
	) = 0;
	virtual BOOL	SendPacket(
		const BYTE *pStreamData,
		const int nStreamDataLength,
		const long nTMin,
		const long nTMax
	) = 0;
	void LockChannel() {m_mtxLock.P ();}
	void UnlockChannel () {m_mtxLock.V ();}
	void SetRTSemaPtr(ChannelSemaphoreListElement* t) {m_RTSemaPtr = t;}

private:
	ZMutex m_mtxLock;
protected:
	ChannelSemaphoreListElement*	m_RTSemaPtr;
};

#endif _Channel_h_
