/***********************************************************************
    This software module  was  originally  developed by Zvi  Lifshitz
    (Triton R&D Ltd.), Mauro Rossi (CSELT) and Guido Franceschini
    (CSELT), and edited by Michael Stepping (FTK)
    in the course of development of the MPEG-4 DMIF (ISO/IEC 14496-6)
    standard.  This software module is an implementation of a part of
    one or more MPEG-4  DMIF  (ISO/IEC 14496-6)  tools  as  specified
    by  the  MPEG-4 DMIF (ISO/IEC 14496-6)  standard.  ISO/IEC  gives
    users of  the  MPEG-4  DMIF  (ISO/IEC 14496-6)  free  license  to
    this software module or modifications thereof for use in hardware
    or software products claiming  conformance  to  the  MPEG-4  DMIF 
    (ISO/IEC 14496-6). Those intending to use this software module in
    hardware or software  products  are  advised  that  its  use  may
    infringe   existing  patents.  The  original  developer  of  this
    software module and his/her company, the subsequent  editors  and
    their  companies,  and  ISO/IEC have no liability for use of this
    software module or modifications thereof  in  an  implementation.
    Copyright  is not released for non MPEG-4 DMIF  (ISO/IEC 14496-6)
    conforming  products. Triton R&D Ltd. and CSELT
    retain  full  right to use the code for their own purpose, assign
    or donate the code to a third party and to inhibit third  parties
    from  using  the  code  for  non  MPEG-4  DMIF  (ISO/IEC 14496-6)
    conforming products. This copyright notice must  be  included  in
    all copies or derivative works. Copyright  1998.
************************************************************************/

/**************************************************************************
 * Project Im1Player
 * INTERNATIONAL STANDARD ORGANIZATION
 * ISO/IEC JTC1/SC29/WG11
 * 
 * SUBSYSTEM: DMIF
 * FILE:      DPI.h
 * OVERVIEW:  Definitions of DMIF-Plugin Interface.
 * 
 **************************************************************************/

#ifndef _DPI_h_
#define _DPI_h_

#include "FlexMuxTool.h"

/*-------------------------------------------------------------
Struct for passing OUT parameters in :
ServiceClientSink::DPI_ChannelAdd_Cnf
ServiceClientSink::DPI_TransmuxAdd_Cnf
-------------------------------------------------------------*/
struct StreamMapElement
{   
	int		m_nFmxInfo;
	UINT	m_nChid;
	UINT	m_nTmpId;
	BYTE*	m_pData;
	int		m_nDataLen;

  public: // methods	
	BOOL operator == (StreamMapElement const &el) const 
	{
		return (m_nChid == el.m_nChid);
	}
	
	StreamMapElement() : m_nFmxInfo(-1), m_nChid(0), m_nTmpId(0), m_pData(NULL), m_nDataLen(0) 
	{}
	
	StreamMapElement(const StreamMapElement& o)//copy constructor
	{
		m_nFmxInfo	= o.m_nFmxInfo;
		m_nChid		= o.m_nChid;
		m_nTmpId		= o.m_nTmpId;
		m_nDataLen	= o.m_nDataLen;
		if ( m_nDataLen > 0 )
		{
			m_pData		= new BYTE[o.m_nDataLen];
			memcpy(m_pData,o.m_pData,o.m_nDataLen);
		}
		else
			m_pData = NULL ;
	}

	StreamMapElement& operator=(const StreamMapElement& r)
	{
		m_nFmxInfo	= r.m_nFmxInfo;
		m_nChid		= r.m_nChid;
		m_nTmpId		= r.m_nTmpId;
		m_nDataLen	= r.m_nDataLen;
		if ( m_pData )
			delete [] m_pData ;
		if ( m_nDataLen > 0 )
		{
			m_pData		= new BYTE[r.m_nDataLen];
			memcpy(m_pData,r.m_pData,m_nDataLen);
		}
		else
			m_pData = NULL ;
		return ( *this ) ;
	}
	
	virtual ~StreamMapElement()
	{
		delete[] m_pData ;
	}
};

typedef ZArray<StreamMapElement>		DPI_TransmuxAdd_CnfParam;
typedef ZArray<RESPONSE>				DPI_ChannelsDelete_CnfParam;


//--------------------------

struct OnChannelsAddstruct
{
	int			m_nFmxInfo;
	WORD		m_nESid;
	UINT		m_nChid;
	QOSSTREAM	m_sQosStream;
	RESPONSE	m_eResponse;
	BOOL operator == (OnChannelsAddstruct const &el) const 
		{return el.m_nESid == m_nESid;}
};

typedef ZArray<OnChannelsAddstruct> OnChannelsAddArray;

struct OnChannelsAddElem
{
	bool				m_bOpenTransmux;
	HANDLE				m_hTAT;
	FLEXMUX_TYPE		m_eFmxType;
	QOSSTREAM			m_sQosStream;
	DIRECTION			m_eDirection;
	OnChannelsAddArray*	m_pInfo;
	BOOL operator == (OnChannelsAddElem const &/*el*/) const 
		{return FALSE;}
};

struct OnChannelsDeleteElem
{
	HANDLE		m_hTAT;
	RESPONSE	m_eResponse;
	BOOL operator == (OnChannelsDeleteElem const &el) const 
		{return(m_hTAT == el.m_hTAT);}
};

typedef ZArray<OnChannelsAddElem> OnChannelsAddParam;
typedef ZArray<OnChannelsDeleteElem> OnChannelsDeleteParam;

class Service_base;

/*-------------------------------------------------------------
Interface for DMIF callbacks sink object (equivalent to	DAI callbacks,
except of Channel related events, which are reported directly to the 
Channel object.
Inteface for a service sink object
-------------------------------------------------------------*/
class ServiceSink_base
{
public:
	virtual ~ServiceSink_base () {};
	virtual void DPI_ServiceAttach_Cnf (
		const Service_base*		pService,
		const BYTE*				pUuData,
		const int				nUuDataLen,
		const RESPONSE			Response
	) = 0;
	virtual void DPI_ServiceDetach_Cnf (
		const Service_base*		pService,
		const RESPONSE			Response
	) = 0;
	virtual void DPI_ChannelAdd_Cnf (
		const Service_base*			pService,
		const StreamMapElement	&outParam,
		const RESPONSE			eResponse
	) = 0;
	virtual void DPI_ChannelDelete_Cnf (
		const Service_base*		pService,
		const HANDLE			hTransmux,
		const int				nFmxInfo,
		const RESPONSE			Response
	) = 0;
	virtual void DPI_TransmuxAdd_Cnf (
		const Service_base*		pService,
		const HANDLE			hTransmux,
		const FLEXMUX_TYPE		FmxType,
		const DPI_TransmuxAdd_CnfParam &outParam
	) = 0;
	virtual void DPI_TransmuxDelete_Cnf (
		const Service_base*		pService,
		const HANDLE			hTransmux,
		const RESPONSE			Response
	) = 0;
	virtual void DPI_DataReceived_Ind (
		const Service_base*		pService,
		const HANDLE			hTransmux,
		const BYTE*				pStreamData,
		const int				nStreamDataLength,
		const BOOL				bErr
	) = 0;
	virtual void DPI_UserCommand_Ack_Cnf (
		const Service_base*		pService,
		const DAIUserCmdParam&	outParam
	) = 0;
	virtual void DPI_MuxCodeTableReceived_Ind (
		const Service_base*		pService,
		const HANDLE			hTransmux,
		const LinearMuxCodeTable& muxCodeTable,
		const BOOL				bErr
	) = 0;
};

/*-------------------------------------------------------------
Interface for the DMIF object
Interface for a service handler
-------------------------------------------------------------*/
class Service_base
{
private:
	ServiceSink_base* m_pSink;
public:
	// Management functions
	Service_base () { m_pSink = NULL; }
	virtual ~Service_base () {}
	void SetSink (ServiceSink_base* pSink) { m_pSink = pSink; }
	ServiceSink_base* GetSink () { return m_pSink; }
	virtual BOOL IsYourURL (LPCSTR szURL) = 0;
	virtual LPCSTR GetModuleID () = 0;

	// Service Attach functions
	virtual void DPI_ServiceAttach_Req(
		const LPCSTR	szURL, 
		const BYTE*		pUuData,
		const int		nUuDataLen
	) = 0;

	// Service Detach functions
	virtual void DPI_ServiceDetach_Req(
		const REASON	Reason
	) = 0;
	
	// Channel Add functions
	virtual void DPI_ChannelAdd_Req(
		const DAIChannelAddParam &inParam
	) = 0;
	
	// Channel Delete functions
	virtual void DPI_ChannelDelete_Req(
		const DAIChannelDeleteParam &inParam
	) = 0;

	// User Command function
	virtual void DPI_UserCommand_Req(
		const DAIUserCmdParam& inParam
	) = 0;
	virtual	void DPI_UserCommand_Ack_Req(
		const DAIUserCmdParam& outParam
	) = 0;

	// Function for data transmission
	virtual void DPI_SendData(
		const UINT		nChid,
		const char*		pData,
		const int		nLength
	) = 0;
};

#endif _DPI_h_
