/*
This software module was originally developed by Apple Computer, Inc.
and edited by Guido Franceschini (CSELT) in the course of development of MPEG-4. 
This software module is an implementation of a part of one or 
more MPEG-4 tools as specified by MPEG-4. 
ISO/IEC gives users of MPEG-4 free license to this
software module or modifications thereof for use in hardware 
or software products claiming conformance to MPEG-4.
Those intending to use this software module in hardware or software
products are advised that its use may infringe existing patents.
The original developer of this software module and his/her company,
the subsequent editors and their companies, and ISO/IEC have no
liability for use of this software module or modifications thereof
in an implementation.
Copyright is not released for non MPEG-4 conforming
products. Apple Computer, Inc. retains full right to use the code for its own
purpose, assign or donate the code to a third party and to
inhibit third parties from using the code for non 
MPEG-4 conforming products.
This copyright notice must be included in all copies or
derivative works. Copyright (c) 1999.
*/
/*
	$Id: FileMP4Instance.h,v 1.2 2000/01/20 00:15:18 mleditsc Exp $
*/

#ifndef INCLUDED_FileMP4Instance_H
#define INCLUDED_FileMP4Instance_H

#include "DPI.h"
#include "MP4Movies.h"

class MP4TrackReaderClient;

class FileMP4Instance : public Service_base
{
public:
	FileMP4Instance();
	virtual ~FileMP4Instance();
	
private:
	virtual BOOL IsYourURL( LPCSTR szURL );
	virtual LPCSTR GetModuleID();
	
	virtual void DPI_ServiceAttach_Req(
		const LPCSTR	szURL, 
		const BYTE*		pUuData,
		const int		nUuDataLen
		);
	virtual void DPI_ServiceDetach_Req(
		const REASON	eReason
		);
	virtual void DPI_ChannelAdd_Req(
		const DAIChannelAddParam	&inParam
		);
	virtual void DPI_ChannelDelete_Req(
		const DAIChannelDeleteParam	&inParam
		);
	virtual void DPI_UserCommand_Req(
		const DAIUserCmdParam&	inParam
		);
	virtual	void DPI_UserCommand_Ack_Req(
		const DAIUserCmdParam&	outParam
		);
	virtual void DPI_SendData(
		const UINT	nChid,
		const char*	pData,
		const int	nLength
		)	{nChid;pData;nLength;return;};
private:
	void ExtractIOD(LPCSTR serviceURL, char*& initialOD, int* pIodLength);
	MP4TrackReaderClient *getClientForChID( int chID );
	
	MP4Movie  m_movie;
	unsigned int m_trackCount;
	MP4Handle m_iod;
	ZArray<MP4TrackReaderClient*> m_readers;
	
	static LPCSTR s_moduleID;
	static LPCSTR s_myURLScheme;
};

class MP4Exception
{
public:
	inline MP4Exception( MP4Err error );
	static inline void test( MP4Err error );
	static inline void testMalloc( void* ptr );
	inline MP4Err getErrorCode() const;
private:
	int m_err;
};

inline MP4Exception::MP4Exception( MP4Err error )
{
	m_err = error;
}

inline MP4Err MP4Exception::getErrorCode() const
{
	return m_err;
}

inline void MP4Exception::test( MP4Err error )
{
	if ( error != MP4NoErr )
		throw MP4Exception( error );
}

inline void MP4Exception::testMalloc( void* ptr )
{
	if ( ptr == 0 )
		throw MP4Exception( MP4NoMemoryErr );
}

#endif
