/***********************************************************************
This software module was originally developed by Zvi Lifshitz
(Triton R&D Ltd.) in the course of development of the MPEG-4 Systems
(ISO/IEC 14496-1) standard. This software module is an implementation of
a part of one or more MPEG-4 Systems (ISO/IEC 14496-1) tools as
specified by the MPEG-4 Systems (ISO/IEC 14496-1) standard. ISO/IEC
gives users of the MPEG-4 Systems (ISO/IEC 14496-1) free license to
this software module or modifications thereof for use in hardware or
software products claiming conformance to the MPEG-4 Systems (ISO/IEC
14496-1). Those intending to use this software module in hardware or
software products are advised that its use may infringe existing patents.
The original developer of this software module and his/her company, the
subsequent editors and their companies, and ISO/IEC have no liability
for use of this software module or modifications thereof in an
implementation. Copyright is not released for non MPEG-4 Systems
(ISO/IEC 14496-1) conforming products. Triton R&D Ltd. retains full right
to use the code for his/her own purpose, assign or donate the code to a
third party and to inhibit third parties from using the code for non
MPEG-4 Systems (ISO/IEC 14496-1) conforming products. This copyright
notice must be included in all copies or derivative works.
Copyright  1998.
************************************************************************

For more information or to receive an up to date version of this module,
you may call Zvi Lifshitz at zvil@csi.com.
----------------------------------------------------------------------*/

#if !defined(_DecoderImp_h)	// Sentry, use file only if it's not already included.
#define _DecoderImp_h

/*-------------------------------------------------------------------------
Project Im1Player
INTERNATIONAL ORGANIZATION FOR STANDARDIZATION
ISO/IEC JTC1/SC29/WG11

SUBSYSTEM:  Decoder
FILE:       DecoderImp.h
AUTHOR:     Zvi Lifshitz (Triton R&D Ltd.)
OVERVIEW:   Definition of DecoderImp, a base class implementation of
			decoder objects
-------------------------------------------------------------------------*/
#include "Decoder.h"
#include "ztl.h"

class DecoderImp : public Decoder
{
private:
	/*-------------------------------------------------------------------
	Interface methods
	-------------------------------------------------------------*/
	virtual void Start ();
	virtual void Stop ();
	virtual BOOL Setup (
		const BYTE *,		// pSpecificInfo,
		int,				// nSpecificInfoLength,
		int					// nESNumber
	) {return TRUE;}
	virtual BOOL SetFormat (
		Capabilities &formatOptions,
		DecoderParams &decoderParams
	);
	virtual int GetOptimalOutputSize () {return 256;}
	virtual void SetInputStream (int, MediaStream *pStream)
	{
		m_pInStream = pStream;
	};
	virtual void SetOutputStream (int, MediaStream *pStream)
	{
		m_pOutStream = pStream;
	}
	virtual void SetStream (
		MediaStream *pStream, 
		Executive *pExecutive, 
		Root *pNode
	)
	{
		m_pInStream = pStream;
		m_pExecutive = pExecutive;
		m_pScene = pNode;
	}
	virtual bool Init () {return TRUE;};
	virtual int GetDataOffset  (const LPBYTE) {return 0;}
	virtual DWORD GetAttributes (ATTRIBUTE_CODE, const LPBYTE) {return 0;}

protected:
	/*-------------------------------------------------------------
	Protected functions to be used by derived objects
	-------------------------------------------------------------*/
	MediaStream *GetInputStream () {return m_pInStream;}
	MediaStream *GetOutputStream () {return m_pOutStream;}
	Root *GetRoot () {return m_pScene;}
	BOOL IsActive () {return m_bActive;}
	virtual void Run ();
	virtual void Terminate () {}
	int StartDecoderThread () {return m_decoderThread.Begin ();}
	void WaitForThread () {m_decoderThread.Wait ();}

private:
	/*-------------------------------------------------------------
	Member variables
	-------------------------------------------------------------*/
	Executive *m_pExecutive;
	MediaStream *m_pInStream, *m_pOutStream;
	BOOL m_bActive, m_bClosing;
	Root *m_pScene;
	ZThread<DecoderImp> m_decoderThread;

private:
	/*-------------------------------------------------------------
	Private methods, some may be overloaded
	-------------------------------------------------------------*/
	void DecoderThread ();
	virtual BOOL Decode (
		LPBYTE,		// pInput,
		int,		// nInputLength,
		LPBYTE,		// pOutput,
		int &		// rnOutputLength
	) {return FALSE;}

public:
	/*-------------------------------------------------------------
	Constructor and destructor
	-------------------------------------------------------------*/
	DecoderImp ();
	virtual ~DecoderImp ();
	Executive *GetExecutive () {return m_pExecutive;}
};

#endif                                      // _DecoderImp_h sentry.
