/*************************************************************************

Module Name:	FBA.cpp

Abstract:	source file for face and body frames decoding

Authors:		Dominique Nerriere (Bell-Labs)
                Tolga K. Capin (EPFL)
				Fabrice Vergnenegre (EPFL)

Remarks:        

*************************************************************************/
#include "defs.h"
#include "FBA.h"
#include "FbaClass.h"

// Tolga: It would be nice to pur these var's to a struct. We cannot
// animate more than one fba as it is now.
static int first_pbap=0;
static int first_pfap=0;
FOP_DATA moFopData;
BOP_DATA moBopData;
ArithModel models[NFAP];        // an ArithModel for each FAP value
int temp_arr1[250];  // This array is required because models[] has memory leak
ArithModel bapModels[NUM_BAPS];
int temp_arr2[250]; // This array is required because bapModels[] has memory leak

#define CALL_FROM_BAP 0
#define CALL_FROM_FAP 1

static BOOL bap_call=FALSE;
static BOOL fap_call=FALSE;

// Ensures the fba object plane is not called
// by both FAP::animate and BAP::animate
// Hypothesis: function called by both
void make_sure_fba_op_called_once(FOP_DATA * moFopData, BOP_DATA * moBopData, unsigned int * uiPts, AnimationStreamDecoder *pAnimStr, int caller)
{
	switch(caller) {
	case CALL_FROM_BAP : if (!fap_call) 
							 fba_object_plane_data(moFopData, moBopData, uiPts, pAnimStr);
						 bap_call=TRUE;
						 break;

	case CALL_FROM_FAP : if (!bap_call) 
							 fba_object_plane_data(moFopData, moBopData, uiPts, pAnimStr);
						 fap_call=TRUE;
						 break;
	}

	if (bap_call && fap_call) {
		bap_call=FALSE;
		fap_call=FALSE;
	}
}

void FAP::animate(AnimationStreamDecoder *pAnimStr,BOOL intra) 
{
  static int initialised=0;
  TRACE("[FAP::Decode] buffer %08x size %d \n",pAnimStr->getInputBuffer(),pAnimStr->getCurrentBufSize());

  unsigned int uiPts = 0;
  int bits = 0;

  /* copy the stream data into the buffer */
  fop.stream.buf = pAnimStr->getInputBuffer();
  fop.stream.bufsize = pAnimStr->getCurrentBufSize();
  fop.stream.p = 0;
  fop.stream.incnt = pAnimStr->getBitsIn();
  fop.stream.bitcnt = pAnimStr->getBitsIn();

    if (!initialised) {
	init();
	initialised = 1;
  }

  static int suggested_gender=0;
  static int coding_type=0; /* 3-3-98 */
  static int first_intra_segment=1; /* 6-26-98 */
  int is_frame_rate = 0;
  int is_time_code = 0;
  int skip_frames = 0;
  static double real_frame_rate = 30.0;
  static double NEXT_PTS=0.0;

  /*if(nextbits_bytealigned() == FBA_OBJECT_PLANE_START_CODE) {
    next_start_code();
    int sc = pAS->readBits(32);
    assert(sc==FBA_OBJECT_PLANE_START_CODE);
  }*/

  fop.is_intra = pAnimStr->getIsIntra();
  fop.fba_paramset = (pAnimStr->getNodeMask(0) << 1) | (int) pAnimStr->getNodeMask(1);

  if (fop.is_intra) {
    if (fop.fba_paramset & FAP_PARAMSET_MASK) {
      if(pAnimStr->getAnimTime().hasFrameRate()) {
		fop.fo.frame_rate = pAnimStr->getAnimTime().getFrameRate();
		fop.fo.seconds = pAnimStr->getAnimTime().getFrameRateSeconds();
		fop.fo.frequency_offset = pAnimStr->getAnimTime().getFrameRateOffset(); 
		if(fop.fo.frequency_offset)
			real_frame_rate = ((double)fop.fo.frame_rate +\
			     ((double)fop.fo.seconds/16))*(1000/1001);
		else
			real_frame_rate = (double)fop.fo.frame_rate + ((double)fop.fo.seconds/16);

      }
      if(fop.fo.is_time_code=pAnimStr->getAnimTime().hasTimeCode()) {
		fop.fo.time_code_hours = pAnimStr->getAnimTime().getTimeCodeHours();
		fop.fo.time_code_minutes = pAnimStr->getAnimTime().getTimeCodeMinutes();
		fop.fo.marker_bit = pAnimStr->getAnimTime().getTimeCodeMarkerBit();
		fop.fo.time_code_seconds = pAnimStr->getAnimTime().getTimeCodeSeconds();
      }
      if(skip_frames=(pAnimStr->getAnimTime().getNumSkipFrames()!=0)) {
		fop.fo.number_of_frames_to_skip = pAnimStr->getAnimTime().getNumSkipFrames();
	  } 
	  else 
	  {
		fop.fo.number_of_frames_to_skip = 0;
      }
    }
  
    if (fop.fba_paramset & BAP_PARAMSET_MASK) 
	{
		TRACE("BAP shouldn\'t be decoded here\n");
    }
  }
  
  
  make_sure_fba_op_called_once(&moFopData, &moBopData, &uiPts, pAnimStr,CALL_FROM_FAP);

  /*
  open_jaw			= moFopData.fap[2];
  lower_t_midlip	= moFopData.fap[3];
  raise_b_midlip	= moFopData.fap[4];
  stretch_l_corner	= moFopData.fap[5];
  stretch_r_corner	= moFopData.fap[6];
  lower_t_lip_lm	= moFopData.fap[7];
  lower_t_lip_rm	= moFopData.fap[8];
  lower_b_lip_lm	= moFopData.fap[9];
  lower_b_lip_rm	= moFopData.fap[10];
  raise_l_cornerlip = moFopData.fap[11];
  raise_r_cornerlip = moFopData.fap[12];
  thrust_jaw		= moFopData.fap[13];
  shift_jaw			= moFopData.fap[14];
  push_b_lip		= moFopData.fap[15];
  push_t_lip		= moFopData.fap[16];
  depress_chin		= moFopData.fap[17];
  close_t_l_eyelid	= moFopData.fap[18];
  close_t_r_eyelid	= moFopData.fap[19];
  close_b_l_eyelid	= moFopData.fap[20];
  close_b_r_eyelid	= moFopData.fap[21];
  yaw_l_eyeball		= moFopData.fap[22];
  yaw_r_eyeball		= moFopData.fap[23];
  pitch_l_eyeball	= moFopData.fap[24];
  pitch_r_eyeball	= moFopData.fap[25];
  thrust_l_eyeball	= moFopData.fap[26];
  thrust_r_eyeball	= moFopData.fap[27];
  dilate_l_pupil	= moFopData.fap[28];
  dilate_r_pupil	= moFopData.fap[29];
  raise_l_i_eyebrow = moFopData.fap[30];
  raise_r_i_eyebrow = moFopData.fap[31];
  raise_l_m_eyebrow = moFopData.fap[32];
  raise_r_m_eyebrow = moFopData.fap[33];
  raise_l_o_eyebrow = moFopData.fap[34];
  raise_r_o_eyebrow = moFopData.fap[35];
  squeeze_l_eyebrow = moFopData.fap[36];
  squeeze_r_eyebrow = moFopData.fap[37];
  puff_l_cheek		= moFopData.fap[38];
  puff_r_cheek		= moFopData.fap[39];
  lift_l_cheek		= moFopData.fap[40];
  lift_r_cheek		= moFopData.fap[41];
  shift_tongue_tip	= moFopData.fap[42];
  raise_tongue_tip	= moFopData.fap[43];
  thrust_tongue_tip = moFopData.fap[44];
  raise_tongue		= moFopData.fap[45];
  tongue_roll		= moFopData.fap[46];
  head_pitch		= moFopData.fap[47];
  head_yaw			= moFopData.fap[48];
  head_roll			= moFopData.fap[49];
  lower_t_midlip_o	= moFopData.fap[50];
  raise_b_midlip_o	= moFopData.fap[51];
  stretch_l_cornerlip = moFopData.fap[52];
  stretch_r_cornerlip = moFopData.fap[53];
  lower_t_lip_lm_o	= moFopData.fap[54];
  lower_t_lip_rm_o	= moFopData.fap[55];
  raise_b_lip_lm_o	= moFopData.fap[56];
  raise_b_lip_rm_o	= moFopData.fap[57];
  raise_l_cornerlip_o = moFopData.fap[58];
  raise_r_cornerlip_o = moFopData.fap[59];
  stretch_l_nose	= moFopData.fap[60];
  stretch_r_nose	= moFopData.fap[61];
  raise_nose		= moFopData.fap[62];
  bend_nose			= moFopData.fap[63];
  raise_l_ear		= moFopData.fap[64];
  raise_r_ear		= moFopData.fap[65];
  pull_l_ear		= moFopData.fap[66];
  pull_r_ear		= moFopData.fap[67];*/
   for (int i=0;i<68;i++)
	{
		FAPValues[i]=moFopData.fap[i];
   		FAPMask[i]=moFopData.fap_mask[i];
	}
//  TRACE("[FAP::Decode] decoded %4d bytes with PTS %8d \n",cntbits(), uiPts);
	TRACE("[FAP::Decode end] \n");
}

void FAP::parseAnimationMask(AnimationStreamDecoder *pAS) 
{
	TRACE("FAP parse mask\n");
}


/************************************************************************************************/



void BAP::animate(AnimationStreamDecoder *pAnimStr,BOOL intra) 
{
  static int initialised=0;
  TRACE("[BAP::animate] buffer %08x size %d \n",pAnimStr->getInputBuffer(),pAnimStr->getCurrentBufSize());

  unsigned int uiPts = 0;
  int bits = 0;

  /* copy the stream data into the buffer */
  fop.stream.buf = pAnimStr->getInputBuffer();
  fop.stream.bufsize = pAnimStr->getCurrentBufSize();
  fop.stream.p = 0;
  fop.stream.incnt = pAnimStr->getBitsIn();
  fop.stream.bitcnt = pAnimStr->getBitsIn();

    if (!initialised) {
	init();
	initialised = 1;
  }

  static int suggested_gender=0;
  static int coding_type=0; /* 3-3-98 */
  static int first_intra_segment=1; /* 6-26-98 */
  int is_frame_rate = 0;
  int is_time_code = 0;
  int skip_frames = 0;
  static double real_frame_rate = 30.0;
  static double NEXT_PTS=0.0;

  /*if(nextbits_bytealigned() == FBA_OBJECT_PLANE_START_CODE) {
    next_start_code();
    int sc = pAS->readBits(32);
    assert(sc==FBA_OBJECT_PLANE_START_CODE);
  }*/

  fop.is_intra = pAnimStr->getIsIntra();

  // OK, here we are sure there are BAPS and not FAPS
  int test1 = pAnimStr->getNodeMask(0) << 1;
  int test2 = (int) pAnimStr->getNodeMask(1);
  fop.fba_paramset = (pAnimStr->getNodeMask(0) << 1) | (int) pAnimStr->getNodeMask(1);

  if (fop.is_intra) {
    if (fop.fba_paramset & BAP_PARAMSET_MASK) {
      if(pAnimStr->getAnimTime().hasFrameRate()) {
		fop.fo.frame_rate = pAnimStr->getAnimTime().getFrameRate();
		fop.fo.seconds = pAnimStr->getAnimTime().getFrameRateSeconds();
		fop.fo.frequency_offset = pAnimStr->getAnimTime().getFrameRateOffset(); 
		if(fop.fo.frequency_offset)
			real_frame_rate = ((double)fop.fo.frame_rate +\
			     ((double)fop.fo.seconds/16))*(1000/1001);
		else
			real_frame_rate = (double)fop.fo.frame_rate + ((double)fop.fo.seconds/16);

      }
      if(fop.fo.is_time_code=pAnimStr->getAnimTime().hasTimeCode()) {
		fop.fo.time_code_hours = pAnimStr->getAnimTime().getTimeCodeHours();
		fop.fo.time_code_minutes = pAnimStr->getAnimTime().getTimeCodeMinutes();
		fop.fo.marker_bit = pAnimStr->getAnimTime().getTimeCodeMarkerBit();
		fop.fo.time_code_seconds = pAnimStr->getAnimTime().getTimeCodeSeconds();
      }
      if(skip_frames=(pAnimStr->getAnimTime().getNumSkipFrames()!=0)) {
		fop.fo.number_of_frames_to_skip = pAnimStr->getAnimTime().getNumSkipFrames();
	  } 
	  else 
	  {
		fop.fo.number_of_frames_to_skip = 0;
      }
    }
  
    if (fop.fba_paramset & FAP_PARAMSET_MASK) 
	{
		TRACE("FAP shouldn\'t be decoded here\n");
    }
  }  
  
	make_sure_fba_op_called_once(&moFopData, &moBopData, &uiPts, pAnimStr,CALL_FROM_BAP);
    //fba_object_plane_data(&moFopData, &moBopData, &uiPts, pAnimStr);

	for (int i=0 ; i<NUM_BAPS ; i++) {
		BAPValues[i]=moBopData.bap[i];
		BAPMask[i]=moBopData.bap_mask[i];
	}

	TRACE("[BAP::Decode end] \n");
}

void BAP::parseAnimationMask(AnimationStreamDecoder *pAS) 
{
	TRACE("BAP parse mask\n");
}


/************************************************************************************************/




int fba_object_plane_data(FOP_DATA* fop_data, BOP_DATA *bop_data, unsigned int *PTS,AnimationStreamDecoder *pAs) 
{
  int i, j;
  int marker;
  int hquant;
  int bitcnt;
  static int suggested_gender=0;
  static int coding_type=0; /* 3-3-98 */
  static int first_intra_segment=1; /* 6-26-98 */
  static int real_frame_rate = 30.0;
  static double NEXT_PTS=0.0;
  static int first_action=1;
  static int fr_no=0;
  int minn,maxn;

  static int no_frames=0;

  TRACE("TRACEDECODING:fba_object_plane_data start\n");

  if (fop.fba_paramset & FAP_PARAMSET_MASK) {
	TRACE("TRACEDECODING:FAP_PARAMSET_MASK\n");
	if(fop.is_intra) {
	  fop.fo.fap_quant = pAs->readBits(5);
	  hquant = fop.fo.fap_quant/2;

	  if(fop.fo.Irange_max[3] == 0 && fop.fo.Irange_min[3] == 0) {
	    if(fop.fo.fap_quant)
	      for(i=2;i<7;i+=2) {
		fop.fo.Irange_ve_max[i] = (63+hquant)/fop.fo.fap_quant;
		fop.fo.range_ve_max[i] = (63+hquant)/fop.fo.fap_quant;
		fop.fo.range_ve_min[i] = -(63+hquant)/fop.fo.fap_quant;
	      }
	    for(i=2;i<NFAP;i++) {
	      maxn = (fop.fo.Idefault_max[i]+RANGE_DELTA-1)/RANGE_DELTA;
	      maxn *= RANGE_DELTA;
	      if(fop.fo.fap_quant)
		fop.fo.Irange_max[i] = maxn/fop.fo.fap_quant;
	      else fop.fo.Irange_max[i] = maxn;
		    
	      minn = (fop.fo.Idefault_min[i]-RANGE_DELTA+1)/RANGE_DELTA;
	      minn *= RANGE_DELTA;
	      if(fop.fo.fap_quant)
		fop.fo.Irange_min[i] = minn/fop.fo.fap_quant;
	      else fop.fo.Irange_min[i] = minn;
			  
	      maxn = (fop.fo.default_max[i]+RANGE_DELTA-1)/RANGE_DELTA;
	      maxn *= RANGE_DELTA;
	      if(fop.fo.fap_quant)
		fop.fo.range_max[i] = maxn/fop.fo.fap_quant;
	      else fop.fo.range_max[i] = maxn;
	      
	      minn = (fop.fo.default_min[i]-RANGE_DELTA+1)/RANGE_DELTA;
	      minn *= RANGE_DELTA;
	      if(fop.fo.fap_quant)
		fop.fo.range_min[i] = minn/fop.fo.fap_quant;
	      else fop.fo.range_min[i] = minn;
	    }
	  }

	  for(i=0; i<N_FAP_GROUP; i++) {
	    marker = pAs->readBits(1);
		assert(marker);
	    fop.fo.fap_mask_type[i] = pAs->readBits(2);
	    if(fop.fo.fap_mask_type[i]==1 || fop.fo.fap_mask_type[i]==2) {
	      decode_fap_group_mask(i,pAs);
	    } else if(fop.fo.fap_mask_type[i]==0) {
	      for(j=fop.fo.mask_index[i]; j<fop.fo.mask_index[i+1]; j++) {
		fop.fo.fap_mask[j] = 0;
	      }
	    } else {
	      for(j=fop.fo.mask_index[i]; j<fop.fo.mask_index[i+1]; j++) {
		fop.fo.fap_mask[j] = 1;
	      }
	    }
	  }

	  suggested_gender = pAs->readBits(1); 
	  coding_type = pAs->readBits(1);/*changed from 2 to 1 -edp*/
	  fop.fo.coding_type = coding_type;
	  assert(coding_type==PREDICTIVE_CODING_TYPE || coding_type==DCT_CODING_TYPE);
	  
	  if (coding_type == PREDICTIVE_CODING_TYPE) {

	    fop.fo.is_i_new_max = pAs->readBits(1);
	    fop.fo.is_i_new_min = pAs->readBits(1);
	    fop.fo.is_p_new_max = pAs->readBits(1);
	    fop.fo.is_p_new_min = pAs->readBits(1);
	    
	    decode_new_minmax(pAs);
	    
	    if(fop.fo.fap_quant) {
	      fop.fo.Irange_ve_max[2] = (63+hquant)/fop.fo.fap_quant;
	      fop.fo.Irange_ve_max[4] = (63+hquant)/fop.fo.fap_quant;
	      fop.fo.Irange_ve_max[6] = (63+hquant)/fop.fo.fap_quant;
	      fop.fo.range_ve_max[2] = (63+hquant)/fop.fo.fap_quant;
	      fop.fo.range_ve_max[4] = (63+hquant)/fop.fo.fap_quant;
	      fop.fo.range_ve_max[6] = (63+hquant)/fop.fo.fap_quant;
	      fop.fo.range_ve_min[2] = -(63+hquant)/fop.fo.fap_quant;
	      fop.fo.range_ve_min[4] = -(63+hquant)/fop.fo.fap_quant;
	      fop.fo.range_ve_min[6] = -(63+hquant)/fop.fo.fap_quant;
	    }

	    // Tolga 00/04/04: commented out the following
		// init_arith(/*fop.fo.fap_mask*/);
	    for(i=0; i<NFAP; i++) {
	      fop_data->fap_mask[i] = fop.fo.fap_mask[i];
	    }
	    decode_ifap(fop_data,pAs);
	  } else {
	    if (first_intra_segment) {
	      DCT_initialize();
	      first_intra_segment = 0;
	    }
	    decode_i_segment();
	  }
	} else {
	  if (coding_type == PREDICTIVE_CODING_TYPE) {
	    for(i=0; i<NFAP; i++) {
	      fop_data->fap_mask[i] = fop.fo.fap_mask[i];
	    }
	    decode_pfap(fop_data,pAs);
	  }  else {
	    decode_p_segment();
	  }
	}

	if (coding_type == PREDICTIVE_CODING_TYPE) { /* 6-26-98 */
	  pAs->adjustBits();
	  
	  if(fop.fo.is_time_code) {
	    *PTS = (3600*fop.fo.time_code_hours+60*fop.fo.time_code_minutes+fop.fo.time_code_seconds)*1000;
	  } else {
	    *PTS = (int) NEXT_PTS;
	  }
	  /* Modified on 6/24/98
	     NEXT_PTS += (fop.fo.number_of_frames_to_skip+1)*1000.0/(double) real_frame_rate;
	     */
	  NEXT_PTS += (fop.fo.number_of_frames_to_skip+1)*1000.0/(double) real_frame_rate;
	}
	bitcnt=pAs->getBitsIn();
  }

	if (fop.fba_paramset & BAP_PARAMSET_MASK) {
		TRACE("TRACEDECODING:BAP_PARAMSET_MASK\n");
		if(fop.is_intra) {
			TRACE("TRACEDECODING:fop_is_intra\n");
			TRACE("TRACEDECODING:fop.bo.bap_quant\n");
			fop.bo.bap_quant = pAs->readBits(5);
			for(i=0; i<BAP_NUM_GROUPS; i++) {
				TRACE("TRACEDECODING:marker\n");
				marker = pAs->readBits(1);
				assert(marker);
				TRACE("TRACEDECODING:fop.bo.bap_mask_type[%d]\n",i);
                fop.bo.bap_mask_type[i] = pAs->readBits(2);
                if(fop.bo.bap_mask_type[i]==1)
					bap_decode_group_mask(i,pAs);
				else 
					if (fop.bo.bap_mask_type[i]==3 || fop.bo.bap_mask_type[i]==0) {
						int j;
						int temp_index;

						for (j=0;j<fop.bo.baps_in_group[i]; j++) {
							temp_index = fop.bo.bap_mask_index[i][j];
							if(fop.bo.bap_mask_type[i]==3)
								fop.bo.bap_mask[temp_index] = 1;
							else
								fop.bo.bap_mask[temp_index] = 0;
						} // end of for
					} // end of if
					else {
						fprintf( stderr, "Error:\n");
						fprintf( stderr, "stream contains wrong BAP mask type for group %d\n", i);
						exit(0);
					} // end of else
			} // end of for

			TRACE("TRACEDECODING:fop.bo.suggested_gender\n");
			fop.bo.suggested_gender = pAs->readBits(1);
			TRACE("TRACEDECODING:fop.bo.body_object_coding_type\n");
			fop.bo.body_object_coding_type = pAs->readBits(1);
			TRACE("TRACEDECODING:fop.bo.is_i_new_max\n");
			fop.bo.is_i_new_max = pAs->readBits(1);
			TRACE("TRACEDECODING:fop.bo.is_i_new_min\n");
			fop.bo.is_i_new_min = pAs->readBits(1);
			TRACE("TRACEDECODING:fop.bo.is_p_new_max\n");
			fop.bo.is_p_new_max = pAs->readBits(1);
			TRACE("TRACEDECODING:fop.bo.is_p_new_min\n");
			fop.bo.is_p_new_min = pAs->readBits(1);

		/*
		fop.bo.has_extendable_baps = pAs->readBits(1);

		if (fop.bo.has_extendable_baps) {
		  marker = pAs->readBits(1);
		  assert(marker);
		  fop.bo.num_extendable_baps = pAs->readBits(8);
		  for (i=0; i<fop.bo.num_extendable_baps; i++) {
		    marker = pAs->readBits(1);
		    assert(marker);
		    extendable_bap_id = pAs->readBits(8);
		    marker = pAs->readBits(1);
		    assert(marker);
		    fop.bo.extendable_baps[extendable_bap_id] = pAs->readBits(18);
		  }
		}*/

			if (fop.bo.is_i_new_max||fop.bo.is_i_new_min||fop.bo.is_p_new_max||fop.bo.is_p_new_min)
				bap_decode_new_minmax(pAs);

			// Strange
			// Tolga 00/03/29: commented out below
			// bap_init_arith();

			/*
			for(i=0; i<NUM_BAPS; i++) {
				if (fop.bo.bap_mask[i])
					pAs->getAAD().init(bapModels[i]);
			}*/

			for(i=0; i<NUM_BAPS; i++)
				bop_data->bap_mask[i] = fop.bo.bap_mask[i];
			decode_ibap(bop_data,pAs);

		} // end of if (is_intra)

		else {

			for(i=0; i<NUM_BAPS; i++)
			    bop_data->bap_mask[i] = fop.bo.bap_mask[i];
			decode_pbap(bop_data,pAs);
		}

		if (coding_type == PREDICTIVE_CODING_TYPE) { /* 6-26-98 */
			TRACE("TRACEDECODING:adjustbits");
			pAs->adjustBits(); /* get ready to code next frame */

			if(fop.bo.is_time_code) {
				*PTS = (3600*fop.bo.time_code_hours+60*fop.bo.time_code_minutes+fop.bo.time_code_seconds)*1000;
			} 
			else {
				*PTS = (int) NEXT_PTS;
			}
	  
			NEXT_PTS += (fop.bo.number_of_frames_to_skip+1)*1000.0/(double) real_frame_rate;
		} // end of if (coding_type == PREDICTIVE_CODING_TYPE)
		TRACE("TRACEDECODING:getBitsin\n");
		bitcnt = pAs->getBitsIn();

	} // end of if (fop.fba_paramset & BAP_PARAMSET_MASK) {

	return bitcnt;
}

void decode_pfap(FOP_DATA *fop_data,AnimationStreamDecoder *pAS) 
{
  int i;
  int skip_frames;
  int symbol, qstep;
  void decode_viseme(FOP_DATA*, int);
  void decode_expression(FOP_DATA*, int);
  int dequant(int, int);
  int recon(int, int);

  skip_frames=(pAS->getAnimTime().getNumSkipFrames()!=0);
  if(!skip_frames) 
	fop.fo.number_of_frames_to_skip = 0;
  
  if(fop.fo.fap_mask[0]) {
	//	decoder_reset();
	decode_viseme(fop_data, 0);
  }
  if(fop.fo.fap_mask[1]) {
	//	decoder_reset();
	decode_expression(fop_data, 0);
  }

  
  for(i=2; i<NFAP; i++) {
	if(fop.fo.fap_mask[i]) {
		// must be init only for the first pfap after one ifap
		if (first_pfap==0)
		    models[i].init(fop.fo.range_max[i]-fop.fo.range_min[i]+1);
		symbol = pAS->getAAD().decode(models[i]);
		models[i].update(symbol);

		fop.fo.fap_Q[i] = symbol + fop.fo.range_min[i];
		qstep = fop.fo.QP[i]*fop.fo.fap_quant;
		fop.fo.fap_diff[i] = dequant(fop.fo.fap_Q[i], qstep);
		fop.fo.fap_prev[i] = recon(fop.fo.fap_diff[i], fop.fo.fap_prev[i]);
		fop_data->fap[i] = fop.fo.fap_prev[i];
		//printf("%d:%d %d, ",i,symbol,fop_data->fap[i]);
		
	}
  }
  if (first_pfap==0) first_pfap++;
  procvisexp(fop_data);
  //printf("\n");

}

void decode_ifap(FOP_DATA *fop_data,AnimationStreamDecoder *pAS) 
{
  int i;
  int symbol, qstep;
  void decode_viseme(FOP_DATA*, int);
  void decode_expression(FOP_DATA*, int);
  int dequant(int, int);

  if(fop.fo.fap_mask[0]) {
	pAS->getAAD().reset();
	decode_viseme(fop_data, 1);
  }
  if(fop.fo.fap_mask[1]) {
	pAS->getAAD().reset();
	decode_expression(fop_data, 1);
  }

  pAS->getAAD().reset();
  first_pfap=0;
  for(i=2; i<NFAP; i++) {
	if(fop.fo.fap_mask[i]) {
		models[i].init(fop.fo.Irange_max[i]-fop.fo.Irange_min[i]+1);
  		symbol = pAS->getAAD().decode(models[i]);
		fop.fo.fap_Q[i] = symbol + fop.fo.Irange_min[i];
		qstep = fop.fo.IQP[i]*fop.fo.fap_quant;
		fop.fo.fap_prev[i] = dequant(fop.fo.fap_Q[i], qstep);
		fop_data->fap[i] = fop.fo.fap_prev[i];
		//printf("%d %d,",i,fop_data->fap[i]);
	}
  }
  procvisexp(fop_data);/*-edp*/
}

void decode_new_minmax(AnimationStreamDecoder *pAS) 
{
  int i, j;
  int minmax;
  int marker;
  int scaler;

  if (fop.fo.fap_quant) scaler = fop.fo.fap_quant;
  else scaler = 1;
  
  if(fop.fo.is_i_new_max) {
    for(i=2, j=0; i<NFAP; i++, j++) {
      if(i==18 || i==30 || i==38 || i== 42 || 
	 i==47 || i==50 || i==60 || i== 64) {
	j=0;
      }
      if(!(j&3)) {
		marker = pAS->readBits(1);
		assert(marker);
      }
      if(fop.fo.fap_mask[i]) {
	minmax = pAS->readBits(5);
	fop.fo.Irange_max[i] = minmax*RANGE_DELTA/scaler;
      }
    }
  }
  if(fop.fo.is_i_new_min) {
    for(i=2, j=0; i<NFAP; i++, j++) {
      if(i==18 || i==30 || i==38 || i== 42 || 
	 i==47 || i==50 || i==60 || i== 64) {
	j=0;
      }
      if(!(j&3)) {
		marker = pAS->readBits(1);
		assert(marker);
      }
      if(fop.fo.fap_mask[i]) {
	minmax = pAS->readBits(5);
	fop.fo.Irange_min[i] = -minmax*RANGE_DELTA/scaler;
      }
    }
  }
  if(fop.fo.is_p_new_max) {
    for(i=2, j=0; i<NFAP; i++, j++) {
      if(i==18 || i==30 || i==38 || i== 42 || 
	 i==47 || i==50 || i==60 || i== 64) {
	j=0;
      }
      if(!(j&3)) {
	marker = pAS->readBits(1);
		assert(marker);
      }
      if(fop.fo.fap_mask[i]) {
	minmax = pAS->readBits(5);
	fop.fo.range_max[i] = minmax*RANGE_DELTA/scaler;
	
      }
    }
  }
  if(fop.fo.is_p_new_min) {
    for(i=2, j=0; i<NFAP; i++, j++) {
      if(i==18 || i==30 || i==38 || i== 42 || 
	 i==47 || i==50 || i==60 || i== 64) {
	j=0;
      }
      if(!(j&3)) {
	marker = pAS->readBits(1);
		// SB 2000/03/24
	    // 	assert(marker);
      }
      if(fop.fo.fap_mask[i]) {
	minmax = pAS->readBits(5);
	fop.fo.range_min[i] = -minmax*RANGE_DELTA/scaler;
	
      }
    }
  }

	/*
    for(i=2; i<NFAP; i++) printf("%d ",fop.fo.Irange_max[i]);
	printf("\n");
	for(i=2; i<NFAP; i++) printf("%d ",fop.fo.Irange_min[i]);
	printf("\n");
	for(i=2; i<NFAP; i++) printf("%d ",fop.fo.range_max[i]);
	printf("\n");
	for(i=2; i<NFAP; i++) printf("%d ",fop.fo.range_min[i]);
	printf("\n");*/
}

void decode_fap_group_mask(int n,AnimationStreamDecoder *pAS) 
{
  int i;

  for(i=fop.fo.mask_index[n]; i<fop.fo.mask_index[n+1]; i++) {
	fop.fo.fap_mask[i] = pAS->readBits(1);
  }
}

void procvisexp(FOP_DATA *fop_data) 
{/*-edp*/
  int i;
  static int numnew;  
  /*if def bits set then update viseme/expression LUT else lookup vis/exp in LUT and replace interp FAPs for this frame -edp*/
  /*form packed indices and expressiondata*/
  if(fop_data->fap_mask[0]) {
    vsptrind = fop_data->viseme_2;
    vsptrind |= fop_data->viseme_1 <<4;
    vbdptrind = fop_data->viseme_blend;
  }
  else {
    vsptrind=0;vbdptrind=0;
  }
  if(fop_data->fap_mask[1]) {
    expvals = fop_data->init_face;
    expvals |= fop_data->expression_2_intensity<<1;
    expvals |= fop_data->expression_2 <<7;
    expvals |= fop_data->expression_1_intensity <<10;
    expvals |= fop_data->expression_1 <<13;
  }
  else {
    expvals = 0;
  }
  /*compare vis/exp to LUT. create new entries if needed and if one or both defbits are set*/
  lastexplinkptr = 0;/*0 indicates only one link*/
  curexplinkptr = 0;

  if(vsptr[vsptrind]!=NULL) 
  {/*test existance of viseme select pair in LUT*/
    if(vsptr[vsptrind][vbdptrind]) {/*test existance of viseme_blend and viseme_def values in LUT*/
      curexplinkptr = vsptr[vsptrind][vbdptrind];
      do {
	if(curexplinkptr->expvals == expvals) {/*test existance of expression values*/
	  break;
	}
	if(!(curexplinkptr->next)) {
	  if(fop_data->expression_def ||fop_data->viseme_def) {
	    newexplinkptr = (explink *)malloc(sizeof(explink));
#if DEBUG
	    printf("numnew1 = %d\n",numnew++);
#endif
	    if(expvals < curexplinkptr->expvals) {
	      if(!lastexplinkptr) {/*insert before first link*/
		vsptr[vsptrind][vbdptrind] = newexplinkptr;
		newexplinkptr->next = curexplinkptr;
		curexplinkptr = newexplinkptr;
		break;
	      }
	      else {
		lastexplinkptr->next = newexplinkptr;
		newexplinkptr->next = curexplinkptr;
		curexplinkptr = newexplinkptr;
		break;
	      }
	    }
	    else {
	      curexplinkptr->next = newexplinkptr;
	      newexplinkptr->next = 0;
	      lastexplinkptr = curexplinkptr;
	      curexplinkptr = newexplinkptr;
	      break;
	    }
	  }
	  else {
	    curexplinkptr = 0;/*no LUT entry found*/
	    break;
	  }
	}
	else {
	  if(fop_data->expression_def ||fop_data->viseme_def) {/*is next, def set*/
            if(expvals < curexplinkptr->expvals) {
	      newexplinkptr = (explink *)malloc(sizeof(explink));
#if DEBUG
	      printf("numnew2 = %d\n",numnew++);
#endif
              if(!lastexplinkptr) {/*insert before first link*/
                vsptr[vsptrind][vbdptrind] = newexplinkptr;
                newexplinkptr->next = curexplinkptr;
                curexplinkptr = newexplinkptr;
                break;
              }
	      else {
                lastexplinkptr->next = newexplinkptr;
                newexplinkptr->next = curexplinkptr;
                curexplinkptr = newexplinkptr;
                break;
              }
	    }
	    else {
	      lastexplinkptr = curexplinkptr;
	      curexplinkptr = curexplinkptr->next;
	    }
	  }
	  else {/*is next,no def*/
	    if(expvals < curexplinkptr->expvals) {
	      curexplinkptr = 0;/*no LUT entry found*/
	      break;
	    }
	    else {
	      lastexplinkptr = curexplinkptr;
	      curexplinkptr = curexplinkptr->next;
	    }
	  }
	}
      } while(1);
    }
    else if(fop_data->expression_def ||fop_data->viseme_def) {
      vsptr[vsptrind][vbdptrind] = (explink *)malloc(sizeof(explink));
#if DEBUG
      printf("numnew3 = %d\n",numnew++);
#endif
      curexplinkptr = vsptr[vsptrind][vbdptrind];
      curexplinkptr->next = 0;
    }
  }
  else if(fop_data->expression_def ||fop_data->viseme_def){
    vsptr[vsptrind] = (vbdptr *)malloc(64*sizeof(explink *));
    for(i = 0;i < 64;i++) vsptr[vsptrind][i] = 0;
    vsptr[vsptrind][vbdptrind] = (explink *)malloc(sizeof(explink));
#if DEBUG
    printf("numnew4 = %d\n",numnew++);
#endif
    curexplinkptr = vsptr[vsptrind][vbdptrind];
    curexplinkptr->next = 0;
  }
  /*assume the curexplinkptr references the either the correct LUT entry or a new empty LUT entry*/
  if(curexplinkptr) {
    if(fop_data->viseme_def && !(fop_data->expression_def)) {/*replace LUT entry only for mouth related FAPs*/
      curexplinkptr->expvals = expvals;
      for(i=0; i<NFAP; i++)
	if(fop_data->fap_mask[i] && ((i>1 && i<18) ||(i>37 && i<47) || (i>49 && i <60))) curexplinkptr->faps[i] = fop_data->fap[i];
	else curexplinkptr->faps[i] = MAXINT;/*set to interpolate*/
    }
    else if(fop_data->expression_def) {/*replace LUT entry for all but headrot FAPs*/
      curexplinkptr->expvals = expvals;
      for(i=0; i<NFAP; i++)
	if(fop_data->fap_mask[i] && ((i>1 && i<47) || (i>49 && i <NFAP))) curexplinkptr->faps[i] = fop_data->fap[i];
	else curexplinkptr->faps[i] = MAXINT;/*set to interpolate*/
    }
    else if(!(fop_data->viseme_def) && !(fop_data->expression_def)) {
      for(i=0; i<NFAP; i++) if(!fop_data->fap_mask[i] && (curexplinkptr->faps[i] != MAXINT)) {/*this needs to change when MAXINT is accepted to mean interpolate by face models*/
	fop_data->fap[i] = curexplinkptr->faps[i];/*update interp faps with LUT faps*/
	fop_data->fap_mask[i] = 1;
      }
    }  
  }
}

void bap_decode_frame_rate(AnimationStreamDecoder *pAS)
{
	TRACE("TRACEDECODING:bap_decode_frame_rate()\n");
	TRACE("TRACEDECODING:fop.bo.frame_rate\n");
	fop.bo.frame_rate = pAS->readBits(8);
	TRACE("TRACEDECODING:fop.bo.seconds\n");
	fop.bo.seconds = pAS->readBits(4);
	TRACE("TRACEDECODING:fop.bo.frequency_offset\n");
	fop.bo.frequency_offset = pAS->readBits(1);
}

void bap_decode_time_code(AnimationStreamDecoder *pAS)
{
	TRACE("TRACEDECODING:bap_decode_time_code()\n");
	TRACE("TRACEDECODING:fop.bo.time_code_hours\n");
	fop.bo.time_code_hours = pAS->readBits(5);
	TRACE("TRACEDECODING:fop.bo.time_code_minutes\n");
	fop.bo.time_code_minutes = pAS->readBits(6);
	TRACE("TRACEDECODING:fop.bo.marker_bit\n");
	fop.bo.marker_bit = pAS->readBits(1);
	TRACE("TRACEDECODING:fop.bo.time_code_seconds\n");
	fop.bo.time_code_seconds = pAS->readBits(6);
}

void bap_decode_skip_frames(AnimationStreamDecoder *pAS)
{
	int ivalue;
	TRACE("TRACEDECODING:bap_decode_skip_frames()\n");
	fop.bo.number_of_frames_to_skip = 0;
	do {
		TRACE("TRACEDECODING:ivalue\n");
        ivalue = pAS->readBits(4);
        fop.bo.number_of_frames_to_skip += ivalue;
	} while (ivalue==15);
}

void bap_decode_group_mask(int n, AnimationStreamDecoder *pAS) 
{
  int i;
  int temp_index;

  TRACE("TRACEDECODING:bap_decode_group_mask()\n");

  for(i=0;i<fop.bo.baps_in_group[n]; i++) {
    temp_index = fop.bo.bap_mask_index[n][i];
	TRACE("TRACEDECODING:fop.bo.bap_mask[temp_index]\n");
    fop.bo.bap_mask[temp_index] = pAS->readBits(1);
  }
}

void bap_decode_new_minmax(AnimationStreamDecoder *pAS)
{
  int i, j;
  int minmax;
  int marker;
  int scaler;
  int temp_index;

	TRACE("TRACEDECODING:bap_decode_new_minmax()\n");

  if (fop.bo.bap_quant) scaler = fop.bo.bap_quant;
  else scaler = 1;

  if(fop.bo.is_i_new_max) {
    for (i=0; i<BAP_NUM_GROUPS; i++) {
        for(j=0;j<fop.bo.baps_in_group[i]; j++) {
          temp_index = fop.bo.bap_mask_index[i][j];
		if(!(j&3)) {
			marker = pAS->readBits(1);
		// SB 2000/03/24
	    // 			assert(marker);
		}
                if(fop.bo.bap_mask[temp_index]) {
                        minmax = pAS->readBits(5);
                        fop.bo.Irange_max[temp_index] = minmax*RANGE_DELTA/scaler;
                }
        }
    }
  }
  if(fop.bo.is_i_new_min) {
    for (i=0; i<BAP_NUM_GROUPS; i++) {
        for(j=0;j<fop.bo.baps_in_group[i]; j++) {
          temp_index = fop.bo.bap_mask_index[i][j];

		if(!(j&3)) {
			marker = pAS->readBits(1);
		// SB 2000/03/24
	    // 			assert(marker);
		}
                if(fop.bo.bap_mask[temp_index]) {
                        minmax = pAS->readBits(5);
                        fop.bo.Irange_min[temp_index] = -minmax*RANGE_DELTA/scaler;
                }
        }
    }
  }
  if(fop.bo.is_p_new_max) {
    for (i=0; i<BAP_NUM_GROUPS; i++) {
        for(j=0;j<fop.bo.baps_in_group[i]; j++) {
          temp_index = fop.bo.bap_mask_index[i][j];

		if(!(j&3)) {
			marker = pAS->readBits(1);
		// SB 2000/03/24
	    // 			assert(marker);
		}
                if(fop.bo.bap_mask[temp_index]) {
                        minmax = pAS->readBits(5);
                        fop.bo.range_max[temp_index] = minmax*RANGE_DELTA/scaler;
                }
        }
    }
  }
  if(fop.bo.is_p_new_min) {
    for (i=0; i<BAP_NUM_GROUPS; i++) {
        for(j=0;j<fop.bo.baps_in_group[i]; j++) {
          temp_index = fop.bo.bap_mask_index[i][j];

		if(!(j&3)) {
			marker = pAS->readBits(1);
		// SB 2000/03/24
	    // 			assert(marker);
		}
                if(fop.bo.bap_mask[temp_index]) {
                        minmax = pAS->readBits(5);
                        fop.bo.range_min[temp_index] = -minmax*RANGE_DELTA/scaler;
                }
        }
    }
  }

}

void decode_ibap(BOP_DATA *bop_data, AnimationStreamDecoder *pAS)
{
  int i, j, k;
  int symbol, qstep;
  int dequant(int, int);

  /*
  FILE * o = fopen("baps.trace","at");
  fprintf(o,"DECODE_IBAP\n");*/
	
  pAS->getAAD().reset();
  first_pbap=0;

  for (k=0; k<BAP_NUM_GROUPS; k++) {
    for(j=0;j<fop.bo.baps_in_group[k]; j++) {
	  i = fop.bo.bap_mask_index[k][j];
	if(fop.bo.bap_mask[i]) {
		bapModels[i].init(fop.bo.Irange_max[i]-fop.bo.Irange_min[i]+1);
		symbol = pAS->getAAD().decode(bapModels[i]);
		fop.bo.bap_Q[i] = symbol + fop.bo.Irange_min[i];
		qstep = fop.bo.IBQP[i]*fop.bo.bap_quant;
		fop.bo.bap_prev[i] = dequant(fop.bo.bap_Q[i], qstep);
		bop_data->bap[i] = fop.bo.bap_prev[i];
		//fprintf(o,"IBAPID=%d SYMBOL=%d\n", i, symbol);
	}
    }
  }

  /*fprintf(o,"\n");
  fclose(o);*/
}

void decode_pbap(BOP_DATA *bop_data, AnimationStreamDecoder *pAS)
{
  int i, j, k;
  int skip_frames; 
  int symbol, qstep;
  int dequant(int, int);
  int recon(int, int);

  /*
  FILE * o = fopen("baps.trace","at");
  fprintf(o,"DECODE_PBAP\n");*/

  /*TRACE("TRACEDECODING:decode_pbap()\n");
  // TRACE("TRACEBITS->read pbap skip_frames\n");
  // skip_frames = pAS->readBits(1);
  // if(skip_frames) {
  //	bap_decode_skip_frames(pAS);
  // } else {
  //	fop.bo.number_of_frames_to_skip = 0;
  // }
  */

  skip_frames=(pAS->getAnimTime().getNumSkipFrames()!=0);
  //skip_frames = pAS->readBits(1);
  if(!skip_frames) 
	fop.bo.number_of_frames_to_skip = 0;

  // Tolga: Do we need this? --> decoder_reset();
  //pAS->getAAD().reset();

  for (k=0; k<BAP_NUM_GROUPS; k++) {
    for(j=0;j<fop.bo.baps_in_group[k]; j++) {
	  i = fop.bo.bap_mask_index[k][j];
	  if(fop.bo.bap_mask[i]) {
		if (first_pbap==0)
		    models[i].init(fop.bo.range_max[i]-fop.bo.range_min[i]+1);
		symbol = pAS->getAAD().decode(bapModels[i]);
	    bapModels[i].update(symbol);
	    fop.bo.bap_Q[i] = symbol + fop.bo.range_min[i];
	    qstep = fop.bo.BQP[i]*fop.bo.bap_quant;
	    fop.bo.bap_diff[i] = dequant(fop.bo.bap_Q[i], qstep);
	    fop.bo.bap_prev[i] = recon(fop.bo.bap_diff[i], fop.bo.bap_prev[i]);
	    bop_data->bap[i] = fop.bo.bap_prev[i];
		//TRACE("TRACEDECODING:PFRAME BAPID=%d symbol=%d\n", i, symbol);
		//fprintf(o,"PBAPID=%d SYMBOL=%d\n", i, symbol);
	  }
    }
  }

  /*
  fprintf(o,"\n");
  fclose(o);*/
  
  if (first_pbap==0) first_pbap++;
}
