/*-------------------------------------------------------------------------
Project MoMuSys
ACTS

SUBSYSTEM:  Decoder
FILE:       MPEG4Dec.cpp
AUTHOR:     Angel Pacheco (Univ.Politecnica de Mardrid - GTI)
OVERVIEW:   Implementation of MPEG4Decoder.
-------------------------------------------------------------------------*/

#ifndef _IM1_CD_DECODER_
#define _IM1_CD_DECODER_
#endif

#ifndef _OPTIMIZE_
#define _OPTIMIZE_
#endif

#include "Include/Defs.h"
#include "MPEG4Dec.h"
#include "Core/Stream.h"

// static members

/* static */ BOOL MPEG4Decoder::initialised = FALSE;
/* static */ // struct MPEG4Decoder::yuvconvtable MPEG4Decoder::convtab[256];
/* static */ // int MPEG4Decoder::cliptab[768];

#ifndef WIN32

#include <sys/time.h>
#include <unistd.h>

inline DWORD timeGetTime ()
{
	struct timeval time;

	::gettimeofday(&time, NULL);

	return (DWORD) (time.tv_sec * 1000 + time.tv_usec / 1000);
}

#endif

Int MPEG4Decoder :: num_cd_decoders=0;


MediaFormat MPEG4Decoder :: SetFormat (Capabilities *)
{
#ifdef _MPEG4_RGBA_
  return MediaFormat::RGBA;
#else
	return MediaFormat::RGB;
#endif
}
   

MediaFormat MPEG4Decoder :: GetFormat ()
{
#ifdef _MPEG4_RGBA_
	return MediaFormat::RGBA;
#else
	return MediaFormat::RGB;
#endif
}

/*** >>>> comm out SB

int MPEG4Decoder :: GetOptimalInputSize ()
{
	return CD_OPTIMAL_INPUT_SIZE;
}
int MPEG4Decoder :: GetOptimalOutputSize ()
{
	return optimal_output_size;
}
int MPEG4Decoder :: GetRecommendedInputBufferSize ()
{
	return CD_RECOMMENDED_INPUT_SIZE;
}
int MPEG4Decoder :: GetRecommendedOutputBufferSize ()
{
	return recommended_output_size;
}

bool MPEG4Decoder::SetOutputBufferFormat(int format)
{
	int maps;

	//by now, simply set the format
	maps=(GetFormat()==FORMAT_RGB)?3:4;
	switch (format){
	case 2: output_format=format;
			size=352*288*maps+10;
			if(pOutput_aux!=NULL)
				free(pOutput_aux);
			pOutput_aux=(unsigned char*)calloc(size,sizeof(unsigned char));
			break;
	case 1: output_format=format;
	        break;
	case 0: output_format=format;
			size=176*144*maps+10;
			if(pOutput_aux!=NULL)
				free(pOutput_aux);
			pOutput_aux=(unsigned char*)calloc(size,sizeof(unsigned char));
	default: output_format=0;
	        break;
	}
	return true;
}

>>>> comm out SB ***/

void MPEG4Decoder ::PositionateVO(
    int format,
    MediaFormat rgb_rgba,
    Vop* decoded_vop,
		unsigned char * pOut_aux,
    unsigned char * pOutput)
{
	int w_orig,h_orig,w_dest,h_dest;
	int info_offset,inc,orig_pos;
	int w_pos,h_pos,size,maps,i,j;
	unsigned char*poutaux;

  maps = (rgb_rgba == MediaFormat::RGB) ? 3 : 4;

	if(format==2)
		{w_dest=352;h_dest=288;}
	else//format==0
		{w_dest=176;h_dest=144;}

	size=w_dest*h_dest*maps+4;
	w_pos=decoded_vop->hor_spat_ref;
	h_pos=decoded_vop->ver_spat_ref;
	w_orig=decoded_vop->width;
	h_orig=decoded_vop->height;


	info_offset=0;
	pOutput=(unsigned char*)memset((void*)pOutput,(unsigned char)0,size);
	pOutput[info_offset++] = w_dest >> 8;
	pOutput[info_offset++] = w_dest;
	pOutput[info_offset++] = h_dest >> 8;
	pOutput[info_offset++] = h_dest;
	pOut_aux+=info_offset;

	orig_pos=info_offset+h_pos*w_dest*maps;
	inc=w_dest*maps;
	w_pos*=maps;
	w_orig*=maps;

	for(i=0;i<h_orig;i++)
	{
		poutaux=pOutput+orig_pos+w_pos;
		for(j=0;j<w_orig;j++)
			(*poutaux++)=(*pOut_aux++);
		orig_pos+=inc;
	}

}

/*** >>>> comm out SB

// APA: for MPE4 YUVA 422 short int vops
void
MPEG4Decoder :: Yuva4222Rgba(unsigned char* rgbdata,
		 short int* yptr,
		 short int* uptr,
		 short int* vptr,
		 short int* aptr,
		 int lumcols, int lumrows)
{
  if (!initialised) {
    int i;

    for (i = 0; i < 256; ++i) {
      convtab[i].u1 = int(float(i - 128) * 1.37f);
      convtab[i].u2 = int(float(i - 128) * 0.698f);
      convtab[i].v1 = int(float(i - 128) * 0.336f);
      convtab[i].v2 = int(float(i - 128) * 1.73f);
    }
   
    for (i = 0; i < 256; ++i)
      cliptab[i] = 0;
    for (; i < 512; ++i)
      cliptab[i] = i - 256;
    for (; i < 768; ++i)
      cliptab[i] = 255;

    initialised = TRUE;
  }

  // row loop

  for (int j = lumrows >> 1; j > 0; --j) {
    short int* tuptr = uptr;
    short int* tvptr = vptr;
    int i;

    // loop chrominance columns 

    for (i = lumcols >> 1; i > 0; --i) {
      int u = *tuptr++;
      int v = *tvptr++;

      if (u < 0)
	u = 0;
      if (u > 255)
	u = 255;

      if (v < 0)
	v = 0;
      if (v > 255)
	v = 255;

      // horizontal downsampling

      for (int k = 2; k > 0; --k) { 
	int y = *yptr++;
	
	if (*aptr) {
	  *rgbdata++ = (cliptab + 256)[y                 + convtab[v].v2]; // r
	  *rgbdata++ = (cliptab + 256)[y - convtab[u].u2 - convtab[v].v1]; // g
	  *rgbdata++ = (cliptab + 256)[y + convtab[u].u1                ]; // b
	  *rgbdata++ = *aptr++;
	} else {
	  *rgbdata++ = 0;
	  *rgbdata++ = 0;
	  *rgbdata++ = 0;
	  *rgbdata++ = 0;
	  aptr++;
	}
      }
    }

    for (i = lumcols >> 1; i > 0; --i) {
      // calculate intermediate values that can be recycled

      int u = *uptr++;
      int v = *vptr++;

      if (u < 0)
	u = 0;
      if (u > 255)
	u = 255;

      if (v < 0)
	v = 0;
      if (v > 255)
	v = 255;

      // horizontal downsampling   

      for (int k = 2; k > 0; --k) { 
	int y = *yptr++;
	
	if (*aptr) {
	  *rgbdata++ = (cliptab + 256)[y                 + convtab[v].v2]; // r
	  *rgbdata++ = (cliptab + 256)[y - convtab[u].u2 - convtab[v].v1]; // g
	  *rgbdata++ = (cliptab + 256)[y + convtab[u].u1                ]; // b
	  *rgbdata++ = *aptr++;
	} else {
	  *rgbdata++ = 0;
	  *rgbdata++ = 0;
	  *rgbdata++ = 0;
	  *rgbdata++ = 0;
	  aptr++;
	}
      }
    }
  }
}

// APA: for MPE4 YUV 422 short int vops
void
MPEG4Decoder :: Yuv4222Rgb(unsigned char* rgbdata,
		 short int* yptr,
		 short int* uptr,
		 short int* vptr,
		 int lumcols, int lumrows)
{
  if (!initialised) {
    int i;

    for (i = 0; i < 256; ++i) {
      convtab[i].u1 = int(float(i - 128) * 1.37f);
      convtab[i].u2 = int(float(i - 128) * 0.698f);
      convtab[i].v1 = int(float(i - 128) * 0.336f);
      convtab[i].v2 = int(float(i - 128) * 1.73f);
    }
   
    for (i = 0; i < 256; ++i)
      cliptab[i] = 0;
    for (; i < 512; ++i)
      cliptab[i] = i - 256;
    for (; i < 768; ++i)
      cliptab[i] = 255;

    initialised = TRUE;
  }

  // row loop

  for (int j = lumrows >> 1; j > 0; --j) {
    short int* tuptr = uptr;
    short int* tvptr = vptr;
    int i;

    // loop chrominance columns 

    for (i = lumcols >> 1; i > 0; --i) {
      int u = *tuptr++;
      int v = *tvptr++;

      if (u < 0)
	u = 0;

      if (v < 0)
	v = 0;

      // horizontal downsampling

      for (int k = 2; k > 0; --k) { 
	int y = *yptr++;

	*rgbdata++ = (cliptab + 256)[y                 + convtab[v].v2]; // r
	*rgbdata++ = (cliptab + 256)[y - convtab[u].u2 - convtab[v].v1]; // g
	*rgbdata++ = (cliptab + 256)[y + convtab[u].u1                ]; // b
      }
    }

    for (i = lumcols >> 1; i > 0; --i) {
      // calculate intermediate values that can be recycled

      int u = *uptr++;
      int v = *vptr++;

      if (u < 0)
	u = 0;

      if (v < 0)
	v = 0;

      // horizontal downsampling   

      for (int k = 2; k > 0; --k) { 
	int y = *yptr++;

	*rgbdata++ = (cliptab + 256)[y                 + convtab[v].v2]; // r
	*rgbdata++ = (cliptab + 256)[y - convtab[u].u2 - convtab[v].v1]; // g
	*rgbdata++ = (cliptab + 256)[y + convtab[u].u1                ]; // b
      }
    }
  }
}

<<<< comm out SB ***/

void MPEG4Decoder ::MPEG4UpdateBuffer(Vop *decoded_vop,LPBYTE pOutput)
{
  
  int info_offset=0;
  LPBYTE pOut_aux;
  
  
  if(output_format==1)
    pOut_aux=pOutput;
  else
    pOut_aux=pOutput_aux;	
	 
#ifdef _WITH_POSITION_
  pOut_aux[info_offset++] = decoded_vop->hor_spat_ref >> 8;
  pOut_aux[info_offset++] = decoded_vop->hor_spat_ref;
  pOut_aux[info_offset++] = decoded_vop->ver_spat_ref >> 8;
  pOut_aux[info_offset++] = decoded_vop->ver_spat_ref;
#endif
  pOut_aux[info_offset++] = decoded_vop->width >> 8;
  pOut_aux[info_offset++] = decoded_vop->width;
  pOut_aux[info_offset++] = decoded_vop->height >> 8;
  pOut_aux[info_offset++] = decoded_vop->height;
  
/*** >>>>

#ifdef _MPEG4_RGBA_
  Yuva4222Rgba((unsigned char*) (pOut_aux + info_offset),
  decoded_vop->y_chan->data->s, decoded_vop->u_chan->data->s, 
  decoded_vop->v_chan->data->s, decoded_vop->a_chan->data->s,
  decoded_vop->width, decoded_vop->height);
#else
  Yuv4222Rgb((unsigned char*) (pOut_aux + info_offset),
  (SInt*)decoded_vop->y_chan->data->s,(SInt*) decoded_vop->u_chan->data->s,
  (SInt*) decoded_vop->v_chan->data->s,
  decoded_vop->width, decoded_vop->height);
#endif
  
<<<< ***/
  
  if(output_format==0)
    PositionateVO(
      0,
      GetFormat (),
      decoded_vop,
      pOut_aux,
      (unsigned char*)pOutput);
  else if(output_format==1)
    ; // do nothing, the output buffer has already the right data
  else // output_format==2
    PositionateVO(
      2,
      GetFormat (),
      decoded_vop,
      pOut_aux,
      (unsigned char*)pOutput);
  
}

BOOL MPEG4Decoder :: Decode (
		LPBYTE pInput,
		int nInputLength,
		LPBYTE pOutput,
		int &rnOutputLength

)
{
  int keep_decoding=0;
  Int skipped=0;
#ifdef _WITHOUT_PERFORMANCE_
  DWORD timeoriginal, timeBefore, timeAfter, timea, timeb;
  static long orig_time=0;
#endif
  
#ifdef _WITHOUT_DECODING_
  if(init==0)		
  {
    init=1;
    timeoriginal=timeBefore=GetTickCount();
  }
  else
    timeBefore=GetTickCount();
  
  Sleep(20);
  if(cont++%2)
    pOutput=(unsigned char*)memset((void*)pOutput,(unsigned char)255,size);
  else
    pOutput=(unsigned char*)memset((void*)pOutput,(unsigned char)1,size);
  pOutput[0] = 176 >> 8;
  pOutput[1] = 176;
  pOutput[2] = 144 >> 8;
  pOutput[3] = 144;
  timeAfter=GetTickCount();
  orig_time+=(timeAfter - timeBefore);
  TRACE ("\n\nMPEG4 %d decoding time: this frame %d = %d ms\n (total= %d ms)\n\n",
    num_cd_decoders, cont, (timeAfter - timeBefore),orig_time);
  
  return TRUE;
#endif
  
  /*** >>>>
  if (rnOutputLength < GetOptimalOutputSize ())
  {	
    rnOutputLength = GetOptimalOutputSize ();
		return FALSE;
  }
  <<<< ***/

 	stream[0][0]->apabuffer->buff       = 
    stream[0][0]->apabuffer->orig_buff  = pInput;
  stream[0][0]->apabuffer->rest_bytes =
    stream[0][0]->apabuffer->orig_bytes = nInputLength;
#ifdef _OPTIMIZE_
  if(stream[0][0]->apabuffer->rest_bytes<= BFR_SIZE)
    memcpy(stream[0][0]->rdbfr,stream[0][0]->apabuffer->buff,
    stream[0][0]->apabuffer->rest_bytes);
  else
  {
    fprintf(stderr,"[MPEG4Dec]:There is no enought buffer to work in a right way.\n\n");
    fprintf(stderr,"Recompile the MPEG4Dec dll and the IM1 without the _OPTIMIZE_ option or");
    fprintf(stderr,"increase the value of BFR_SIZE (65536)\n\n");
    fflush(stderr);
    exit(1);
  }
#else
  stream[0][0]->rdbfr = stream[0][0]->apabuffer->buff;//first pos, first buff
#endif
  
  stream[0][0]->incnt = 0;
  stream[0][0]->rdptr = stream[0][0]->rdbfr + BFR_SIZE;
  stream[0][0]->bitcnt = 0;
  
  
  if(init==0)		
  {
    DecodeMPEG4CDHeaders(width,height,vols,framerate, 
      filter_type,&temporal_scalability,readen_bits,stop_decoding_vol,
      display_vop,&time_between_two_frames,&next_display_time,
      random,stream,display_vol_list,trace,trace_filename,
      start_time,&prev_ref,&next_ref,MB_TYPE2,PRE_H_SIZE2,PRE_V_SIZE2,PRE_MB_TYPE2,
      bs_vol,alpha_rec_packet,alpha_mode_packet,Px_for,Py_for,Px_back,Py_back,
      prev_j,zero_edge,prev_time_base,stock_mb_type2,stock_h_size2,stock_v_size2);
    init=1;
#ifdef _WITHOUT_PERFORMANCE_
    timeoriginal=timeBefore=GetTickCount();
#endif
  }
  else
  {
#ifdef _WITHOUT_PERFORMANCE_
    timeBefore=GetTickCount();
#endif
    if(vols==1)
    {
      stop_decoding_vol[0][0]=0;
      readen_bits[0][0]=0;
    }else /*vols==2*/
    {
      stop_decoding_vol[0][0]=stop_decoding_vol[0][1]=0;
      readen_bits[0][0]=readen_bits[0][1]=0;
    }
    
  }
#ifdef _WITHOUT_PERFORMANCE_
  timea=GetTickCount();
#endif
  DecodeMPEG4CDVol(width,height,vols,framerate,filter_type,
    temporal_scalability,random,
    readen_bits,stop_decoding_vol,display_vop,time_between_two_frames,
    &next_display_time,stream,display_vol_list,trace,trace_filename,
    base_output_vop,display_framerate_TPS,last_base_decoding_frame,
    &last_enh_decoding_frame,&TPS_base_count,&prev_ref,&next_ref,
    MB_TYPE2,PRE_H_SIZE2,PRE_V_SIZE2,PRE_MB_TYPE2,bs_vol,
    alpha_rec_packet,alpha_mode_packet,Px_for,Py_for,Px_back,Py_back,
    prev_j,zero_edge,prev_time_base,prev_predic_type,
    back_shape_vop,for_shape_vop,stock_mb_type2,stock_h_size2,stock_v_size2);
#ifdef _WITHOUT_PERFORMANCE_
  timeb=GetTickCount();
#endif
  //S		if(vol->iReadbits==0)
  //S			vol->decode_flag = 0; /* bitstream exhausted */
  /* this DOEST NOT work, the systenm is not able to reallocate the memory
  size=(display_vol_list[0]->pvop->width*display_vol_list[0]->pvop->height)*MULT_FACTOR+INC_POS;
  if(size>optimal_output_size)
  {
		optimal_output_size=recommended_output_size=size;
    rnOutputLength = GetOptimalOutputSize (); //correcto despues de decodificar algo??
    return FALSE;
    }
  */
  
  keep_decoding=1;//cambiar a una comprobacion de lo que salga, eof, readen_bits, etc
  if(keep_decoding == 1)
  {
    MPEG4UpdateBuffer(display_vol_list[0]->pvop,pOutput);
    next_display_time+=time_between_two_frames; //??

#ifdef _WITHOUT_PERFORMANCE_
    timeAfter=GetTickCount();
    orig_time+=(timeAfter - timeBefore);
    TRACE ("\n\nMPEG4 %d decoding time: this frame %d\n",num_cd_decoders, cont);
    TRACE ("%d ms\n (total= %d ms)\n", (timeAfter - timeBefore),orig_time);
    TRACE ("dll %d ms\n\n",	(timeb - timea));
#endif

    // SB rnOutputLength = GetOptimalOutputSize ();

#if 0
    if(skipped==1)
      TRACE ("\n\n last VOL skipped\n\n");
    else
      TRACE ("\n\nMPEG4thread %d, decoded one more VOL (instant %03d)\n",num_cd_decoders, cont++);
#endif
    return TRUE;
  }
  else
  {
    //ended, STOP the decoder
    //setNotActive(); se hace en el metodo DecoderThread ()	 :	{Run ();	m_smpActive.V ();}
    
    TRACE("Mpeg4Dec %d, end of sequence reached (time %03d) \n",
      num_cd_decoders, cont++);
    rnOutputLength=4+1;
#ifdef _WITH_POSITION_
    rnOutputLength+=4;
#endif
    for(int i=0;i<rnOutputLength;i++) pOutput[i] = 0;
    return TRUE;//FALSE;
  }
  
}

/*-----------------------------------------------------------------
-----------------------------------------------------------------*/

/*** >>>>

void MPEG4Decoder :: Run ()
{
	LPBYTE pInData, pOutData;
	int nInSize, nOutSize;
	DWORD dwTime;

  // APA Init Decoder
	initcddecoder();

	while (
		(pInData =
			(LPBYTE) m_pInStream -> FetchNext (nInSize, dwTime, TRUE)
		) !=
		END_OF_STREAM
	)
	{
		if (m_bActive)
		{	nOutSize = GetOptimalOutputSize (),
			pOutData = (LPBYTE) m_pOutStream -> Allocate (nOutSize, TRUE, FALSE);
			if (pOutData == NULL)
				throw;
			if (pOutData == END_OF_STREAM)
			{
				m_pInStream -> Release (TRUE);

				// apa: free data
				FreeMPEG4CD(display_vop,vols,temporal_scalability,cont,
					display_vol_list,stream,PRE_MB_TYPE2,PRE_V_SIZE2,
					num_cd_decoders,back_shape_vop,for_shape_vop,
					stock_mb_type2,stock_v_size2);
				return;
			}
			while (!Decode (pInData, nInSize, pOutData, nOutSize))
			{	pOutData = (LPBYTE) m_pOutStream -> Reallocate (nOutSize);
				if (pOutData == NULL)
					throw;
				if (pOutData == END_OF_STREAM)
					{
					m_pInStream -> Release (TRUE);

					// apa: free data
					FreeMPEG4CD(display_vop,vols,temporal_scalability,cont,
						display_vol_list,stream,PRE_MB_TYPE2,PRE_V_SIZE2,
						num_cd_decoders,back_shape_vop,for_shape_vop,
						stock_mb_type2,stock_v_size2);
					return;
					}

			}
			m_pOutStream -> Dispatch (nOutSize, dwTime);
		}
		m_pInStream -> Release (TRUE);
	}
	
	// apa: free data
  // this code is never called
	FreeMPEG4CD(display_vop,vols,temporal_scalability,cont,
		display_vol_list,stream,PRE_MB_TYPE2,PRE_V_SIZE2,
		num_cd_decoders,back_shape_vop,for_shape_vop,
		stock_mb_type2,stock_v_size2);
}

/*** >>>>

static DecoderFactory<MPEG4Decoder> MPEG4Decoder (5);

<<<< ***/
