// -*- c++ -*-

#if !defined(_mpeg4cddec_h)	// Sentry, use file only if it's not already included.
#define _mpeg4cddec_h

/*-------------------------------------------------------------------------
Project MoMuSys
ACTS

SUBSYSTEM:  Decoder
FILE:       MPEG4Dec.h
AUTHOR:     Angel Pacheco (Univ.Politecnica de Madrid-GTI)
OVERVIEW:   Definition of MPEG4Decoder
-------------------------------------------------------------------------*/

#include "Decoder/DecoderImp.h"
#include "Core/Nodes.h"

#define _MPEG4_RGBA_

//MPEG4 CD-971201 includes
#include "Mpeg4Decoder/include/vm_mpegcddec.h"

/**
 * external C functions for the Mpeg4 Video Decoder (Momusys)
 */

extern "C"
{
  /*** >>>>
  Void CallingConvention InitMPEG4CDDecoder(
    Int width, Int height, 
    Int vols,  Int framerate, 
    Int filter_type,
    Int        readen_bits[MAX_NUM_VOS][MAX_NUM_VOLS],
    Int        stop_decoding_vol[MAX_NUM_VOS][MAX_NUM_VOLS],
    Vop      **display_vop, 
    Float     *time_between_two_frames,
    Float     *next_display_time,
    Int		  *random_access_start_time,
    Bitstream *stream[MAX_NUM_VOS][MAX_NUM_VOLS],
    Vol       *display_vol_list[MAX_NUM_VOS],
    Trace     *trace[MAX_NUM_VOS][MAX_NUM_VOLS],
    Char       trace_filename[MAX_NUM_VOS][MAX_NUM_VOLS][300],
    Vop	      *base_output_vop[3],
    UChar **PRE_MB_TYPE2[MAX_NUM_VOS][MAX_NUM_VOLS],
    Int bs_vo[MAX_NUM_VOS],
    Int bs_vol[MAX_NUM_VOS][MAX_NUM_VOLS],
    Float      *display_framerate_TPS,
    Int			num_cd_decoders,
    UChar **stock_mb_type2[3][MAX_NUM_VOS][MAX_NUM_VOLS]
  );

  Void CallingConvention DecodeMPEG4CDVol(
    Int        width,
    Int        height, 
    Int        vols,
    Int        framerate, 
    Int        post_filter_type,
    Int        temporal_scalability,
    Int        random_access_start_time,
    Int        readen_bits[MAX_NUM_VOS][MAX_NUM_VOLS],
    Int        stop_decoding_vol[MAX_NUM_VOS][MAX_NUM_VOLS],
    Vop       *display_vop, 
    Float      time_between_two_frames,
    Float     *next_display_time,
    Bitstream *stream[MAX_NUM_VOS][MAX_NUM_VOLS],
    Vol       *display_vol_list[MAX_NUM_VOS],
    Trace     *trace[MAX_NUM_VOS][MAX_NUM_VOLS], 
    Char       trace_filename[MAX_NUM_VOS][MAX_NUM_VOLS][300],
    Vop	      *base_output_vop[3],
    Float      display_framerate_TPS,
    Int last_base_decoding_frame[3],
    Int *last_enh_decoding_frame,
    Int *TPS_base_count,
    Int    *prev_ref,
    Int *next_ref,
    UChar **MB_TYPE2[MAX_NUM_VOS][MAX_NUM_VOLS],
    Int PRE_H_SIZE2[MAX_NUM_VOS][MAX_NUM_VOLS],
    Int PRE_V_SIZE2[MAX_NUM_VOS][MAX_NUM_VOLS],
    UChar **PRE_MB_TYPE2[MAX_NUM_VOS][MAX_NUM_VOLS],
    Int bs_vol[MAX_NUM_VOS][MAX_NUM_VOLS],
    Image   *alpha_rec_packet,
    Image   *alpha_mode_packet,
    SInt Px_for,SInt Py_for, SInt Px_back, SInt Py_back,
    Int prev_j,Int zero_edge,Int prev_time_base,
    Int prev_predic_type,
    Vop	*back_shape_vop,
    Vop	*for_shape_vop,
    UChar **stock_mb_type2[3][MAX_NUM_VOS][MAX_NUM_VOLS],
    Int   stock_h_size2[3][MAX_NUM_VOS][MAX_NUM_VOLS],
    Int   stock_v_size2[3][MAX_NUM_VOS][MAX_NUM_VOLS]
  );

  Void CallingConvention DecodeMPEG4CDHeaders(
  	Int        width, Int height, 
	  Int        vols,  Int framerate, 
	  Int        filter_type,
	  Int		  *temporal_scalability,
	  Int        readen_bits[MAX_NUM_VOS][MAX_NUM_VOLS],
	  Int        stop_decoding_vol[MAX_NUM_VOS][MAX_NUM_VOLS],
	  Vop		  *display_vop, 
	  Float     *time_between_two_frames,
	  Float     *next_display_time,
	  Int		   random_access_start_time,
	  Bitstream *stream[MAX_NUM_VOS][MAX_NUM_VOLS],
    Vol       *display_vol_list[MAX_NUM_VOS],
    Trace     *trace[MAX_NUM_VOS][MAX_NUM_VOLS],
	  Char       trace_filename[MAX_NUM_VOS][MAX_NUM_VOLS][300],
	  Int        start_time[MAX_NUM_VOLS][2],
	  Int   *prev_ref,Int   *next_ref,
	  UChar **MB_TYPE2[MAX_NUM_VOS][MAX_NUM_VOLS],
	  Int PRE_H_SIZE2[MAX_NUM_VOS][MAX_NUM_VOLS],
	  Int PRE_V_SIZE2[MAX_NUM_VOS][MAX_NUM_VOLS],
	  UChar **PRE_MB_TYPE2[MAX_NUM_VOS][MAX_NUM_VOLS],
	  Int bs_vol[MAX_NUM_VOS][MAX_NUM_VOLS],
	  Image   *alpha_rec_packet,
	  Image   *alpha_mode_packet,
	  SInt Px_for,SInt Py_for, SInt Px_back, SInt Py_back,
	  Int prev_j,Int zero_edge,Int prev_time_base,
	  UChar **stock_mb_type2[3][MAX_NUM_VOS][MAX_NUM_VOLS],
	  Int   stock_h_size2[3][MAX_NUM_VOS][MAX_NUM_VOLS],
	  Int   stock_v_size2[3][MAX_NUM_VOS][MAX_NUM_VOLS]
  );
  <<<< ***/
}


/**
 * a bunch of defines for the in out formats
 */

#define	CD_OPTIMAL_INPUT_SIZE			10000 /* change to 256*/		/* to de defined */
#ifdef _WITH_POSITION_
	#ifdef _MPEG4_RGBA_
	#define	OPTIMAL_OUTPUT_SIZE_QCIF		101385	/*176 x 144 x 4 ; qcif RGBA format + 8 + 1*/
	#define OPTIMAL_OUTPUT_SIZE_CIF			405513 /*352 x 288 x 4 ;  cif RGBA format + 8 + 1*/
	#define MULT_FACTOR						4
	#else
	#define	OPTIMAL_OUTPUT_SIZE_QCIF		76041	/*176 x 144 x 3 ; qcif RGB  format + 8 + 1*/
	#define OPTIMAL_OUTPUT_SIZE_CIF			304137  /*352 x 288 x 3 ;  cif RGBA format + 8 + 1*/
	#define MULT_FACTOR						3
	#endif
	#define INC_POS							9
#else
	#ifdef _MPEG4_RGBA_
	#define	OPTIMAL_OUTPUT_SIZE_QCIF		101381	/*176 x 144 x 4 ; qcif RGBA format + 4 + 1*/
	#define OPTIMAL_OUTPUT_SIZE_CIF			405409  /*352 x 288 x 4 ;  cif RGBA format + 4 + 1*/
	#define MULT_FACTOR						4
	#else
	#define	OPTIMAL_OUTPUT_SIZE_QCIF		76037	/*176 x 144 x 3 ; qcif RGB  format + 4 + 1*/
	#define OPTIMAL_OUTPUT_SIZE_CIF			304133  /*352 x 288 x 4 ;  cif RGBA format + 4 + 1*/
	#define MULT_FACTOR						3
	#define INC_POS							5
#endif
#endif

#define	CD_OPTIMAL_OUTPUT_SIZE			OPTIMAL_OUTPUT_SIZE_CIF
#define	CD_RECOMMENDED_INPUT_SIZE		10000	/* to be defined */
#define	CD_RECOMMENDED_OUTPUT_SIZE		CD_OPTIMAL_OUTPUT_SIZE	/* to be defined */
#define TIME_INCREMENT_MSEC			100		/* display rate in ms */

//class MediaStream;	 

class MPEG4Decoder 
// : public Decoder
{
public:

  /// Constructor
	MPEG4Decoder() 
	{
		// SB initcddecoder();
#if _MPEG4_2_DISK_
		if( (stream_out = fopen( "VM4buffer", "w+" )) == NULL )
			printf( "Problem opening the file %s\n","VM4buffer" );
#endif
#if _MPEG4_FROM_DISK_
		if( (stream_in = fopen( "CDbuffer", "r" )) == NULL )
			printf( "Problem opening the file %s\n","CDbuffer" );
#endif
	}

  /// Destructor
  ~MPEG4Decoder ()
	{
	  // FreeMPEG4 has been already called in the Run() method of the thread
	  num_cd_decoders--;
	}

public:

  /// Decode one VOP
	virtual BOOL Decode (
		LPBYTE pInput,
		int nInputLength,
		LPBYTE pOutput,
		int &rnOutputLength
	);

  /// Update the Picture buffer
	void MPEG4UpdateBuffer(Vop *decoded_vop,LPBYTE pOutput);

  /// Skips P and B vops
	void FastDecoding(){fast_forward=1;};

  /// Decodes IPB vops
	void NormalDecoding(){fast_forward=0;};

	// puts the decoded VO in a output buffer with the size and position given
	// by the input parameter:
	//  0 -> QCIF buffer, relative position given in the video bitstream (DEFAULT)
	//  1 -> bounding_box-sized buffer, no relative position.
	//  2 -> CIF buffer, relative position given in the video bitstream

	void PositionateVO(
    int format,
    MediaFormat rgb_rgba,
    Vop* decoded_vop,
		unsigned char *pOut_aux,
    unsigned char*pOutput);

	MediaFormat GetFormat ();
  virtual MediaFormat SetFormat (Capabilities *pPresentationCapabilities);

  inline int GetWidth() { return width; }
  inline int GetHeight() { return height; }

  /*** >>>>
	virtual void Run();
  bool SetOutputBufferFormat(int format);
	virtual int GetOptimalInputSize ();
	virtual int GetOptimalOutputSize ();
	virtual int GetRecommendedInputBufferSize ();
	virtual int GetRecommendedOutputBufferSize ();
	static void Yuv4222Rgb(unsigned char* rgbdata,
		 short int* yptr, short int* uptr,short int* vptr,
		 int lumcols, int lumrows);
	static void Yuva4222Rgba(unsigned char* rgbdata,
		 short int* yptr, short int* uptr,short int* vptr,short int* aptr,
		 int lumcols, int lumrows);
  <<<< ***/

  /*** >>>>
  static struct yuvconvtable {
	  int u1;
	  int u2;
	  int v1;
	  int v2;
	} convtab[256];
  static int cliptab[768];
  <<<< ***/

private:
	static BOOL initialised;

  Vol	*vol;
	Int init;
	Int cont;
	Int fast_forward;
	int i,j,k;

	// apa begin: surely, to be deleted
#if _MPEG4_2_DISK_
	FILE*	stream_out;
#endif
#if _MPEG4_FROM_DISK_
	FILE*	stream_in;
#endif
	// apa end  : surely, to be deleted

  // >>>> A.Pacheco:03/11/98:
  // added for CD MPEG4 decoder

  Bitstream *stream[MAX_NUM_VOS][MAX_NUM_VOLS];
  Int width, height;
  Int size, optimal_output_size,recommended_output_size;
  Int vols,  framerate; 
  Int filter_type, random;
  Int        readen_bits[MAX_NUM_VOS][MAX_NUM_VOLS];
  Int        stop_decoding_vol[MAX_NUM_VOS][MAX_NUM_VOLS];
  Vop       *display_vop; 
  Float      time_between_two_frames;
  Float      next_display_time;
  int			start_time1,time_now;
  Vol       *display_vol_list[MAX_NUM_VOS];
  Trace     *trace[MAX_NUM_VOS][MAX_NUM_VOLS]; 
  Char       trace_filename[MAX_NUM_VOS][MAX_NUM_VOLS][300];
  Int        temporal_scalability;
  
  Vop	   *base_output_vop[3];
  Float	display_framerate_TPS;
  Int		last_base_decoding_frame[3];
  Int		last_enh_decoding_frame;
  Int		TPS_base_count;
  Int     prev_ref;
  Int		next_ref;
  UChar **MB_TYPE2[MAX_NUM_VOS][MAX_NUM_VOLS];
  Int		PRE_H_SIZE2[MAX_NUM_VOS][MAX_NUM_VOLS];
  Int		PRE_V_SIZE2[MAX_NUM_VOS][MAX_NUM_VOLS];
  UChar **PRE_MB_TYPE2[MAX_NUM_VOS][MAX_NUM_VOLS];
  Int		bs_vo[MAX_NUM_VOS];
  Int		bs_vol[MAX_NUM_VOS][MAX_NUM_VOLS];
  Int     start_time[MAX_NUM_VOLS][2];
  
  Image   *alpha_rec_packet;
  Image   *alpha_mode_packet;
  SInt Px_for,Py_for, Px_back, Py_back;
  Int prev_j;
  Int zero_edge;
  Int prev_time_base;
  Int prev_predic_type;
  static Int num_cd_decoders;
  Vop	*back_shape_vop;
  Vop	*for_shape_vop;
  UChar **stock_mb_type2[3][MAX_NUM_VOS][MAX_NUM_VOLS];
  Int   stock_h_size2[3][MAX_NUM_VOS][MAX_NUM_VOLS];
  Int   stock_v_size2[3][MAX_NUM_VOS][MAX_NUM_VOLS];
  
	int output_format;
	unsigned char *pOutput_aux;

  // <<<< A.Pacheco:03/11/98:
  // added for CD MPEG4 decoder

public:
	void initcddecoder()
	{
		init=0;cont=0;fast_forward=0;

		width=176;height=144;size=176*144;
		optimal_output_size=CD_OPTIMAL_OUTPUT_SIZE;
		recommended_output_size=CD_RECOMMENDED_OUTPUT_SIZE;

		initialised=false;
		vols=1;framerate=60; 
		filter_type=0;/*0=disabled; 1/2/3 */
		random=0;
		time_between_two_frames=1.0/framerate;
		start_time1=time_now=next_display_time=0.0;
		stream[0][0]=stream[0][1]=NULL;
		readen_bits[0][0]=readen_bits[0][1]=0;
		stop_decoding_vol[0][0]=stop_decoding_vol[0][1]=1;
		display_vop=NULL; 
		display_vol_list[0]=NULL;
		trace[0][0]=trace[0][1]=NULL;
		temporal_scalability=0;

		last_base_decoding_frame[0] = last_base_decoding_frame[1] =
		last_base_decoding_frame[2] = -1;
		last_enh_decoding_frame  = 0;
		TPS_base_count           = 0;
		display_framerate_TPS    = 0;
		prev_ref= -1;
		next_ref= -1;
		alpha_rec_packet=NULL;
		alpha_mode_packet=NULL;
		prev_j=-1;

		num_cd_decoders++;
		pOutput_aux=NULL;

    /*** >>>>
		SetOutputBufferFormat(0); //QCIF+relative position output buffer.
    <<<< ***/

		for(k=0;k<3;k++)
		  for(i=0;i<MAX_NUM_VOS;i++)
			for(j=0;j<MAX_NUM_VOLS;j++)		
			{
			stock_mb_type2[k][i][j]=NULL;
			stock_h_size2[k][i][j]=0;
			stock_v_size2[k][i][j]=0;
			}

		/*from MPEG4 decoder DLL */
		InitMPEG4CDDecoder(width,height,vols,framerate,filter_type,
			readen_bits,stop_decoding_vol,&display_vop,&time_between_two_frames,
			&next_display_time,&random,stream,display_vol_list,trace,trace_filename,
			base_output_vop,PRE_MB_TYPE2,bs_vo,bs_vol,&display_framerate_TPS,
			num_cd_decoders,stock_mb_type2);

	}

};

#endif                                      // _mpeg4cddec_h sentry.
