/*****************************************************************************
 *
 * This software module was originally developed by
 *
 *   Michael Wollborn (TUH / ACTS-MoMuSyS)
 *
 * and edited by
 * 
 *   Robert Danielsen (Telenor / ACTS-MoMuSyS)
 *   Bruno Loret (CNET / ACTS-MoMuSyS)
 *
 * in the course of development of the MPEG-4 Video (ISO/IEC 14496-2) standard.
 * This software module is an implementation of a part of one or more MPEG-4
 * Video (ISO/IEC 14496-2) tools as specified by the MPEG-4 Video (ISO/IEC
 * 14496-2) standard.
 *
 * ISO/IEC gives users of the MPEG-4 Video (ISO/IEC 14496-2) standard free
 * license to this software module or modifications thereof for use in hardware
 * or software products claiming conformance to the MPEG-4 Video (ISO/IEC
 * 14496-2) standard.
 *
 * Those intending to use this software module in hardware or software products
 * are advised that its use may infringe existing patents. The original
 * developer of this software module and his/her company, the subsequent
 * editors and their companies, and ISO/IEC have no liability for use of this
 * software module or modifications thereof in an implementation. Copyright is
 * not released for non MPEG-4 Video (ISO/IEC 14496-2) Standard conforming
 * products.
 *
 * ACTS-MoMuSys partners retain full right to use the code for his/her own
 * purpose, assign or donate the code to a third party and to inhibit third
 * parties from using the code for non MPEG-4 Video (ISO/IEC 14496-2) Standard
 * conforming products. This copyright notice must be included in all copies or
 * derivative works.
 *
 * Copyright (c) 1996
 *
 *****************************************************************************/

/***********************************************************HeaderBegin*******
 *                                                                         
 * File:	mom_access.h
 *
 * Author:	Michael Wollborn
 * Created:	19-Feb-96
 *                                                                         
 * Description: This is the header file for the common access functions.
 *     		This file should be included in each source file where
 *     		the access functions are used. 
 *
 * Notes: 	
 *
 * Modified:	V 2.0, 19-FEB-1996, TUH (M. Wollborn)
 * 		21.04.96 Robert Danielsen: New header.
 *              23.04.96 Bruno Loret: Non reentrant ifndef
 *
 ***********************************************************HeaderEnd*********/


#ifndef _MOM_ACCESS_H_
#define _MOM_ACCESS_H_


#include "momusys.h"

#ifdef _OPTIMIZE_

/****************************************************************************/
INLINE Char *
GetImageData(Image *image)
{
#ifndef _OPTIMIZE_
    switch(GetImageType(image))
    {
    case SHORT_TYPE:
      return((Char *)image->data->s);
      break;
    case FLOAT_TYPE:
      return((Char *)image->data->f);
      break;
    case UCHAR_TYPE:
      return((Char *)image->data->u);
      break;
    default:
      printf("Image type >>%d<< not supported\n",image->type);
      return(NULL);
    }
#else
    return((Char *)image->f);
#endif

}

INLINE Int
GetImageSize(Image *image)
{
  return(image->x * image->y);
}

INLINE Int
GetImageSizeX(Image *image)
{
  return(image->x);
}

INLINE Int
GetImageSizeY(Image *image)
{
  return(image->y);
}
  
INLINE Int
GetImageVersion(Image *image)
{
  return(image->version);
}

INLINE ImageType
GetImageType(Image *image)
{
  return(image->type);
}
/****************************************************************************/
INLINE SInt *
GetImageIData(ImageI *image)		/* ImageI */
{
#ifndef _OPTIMIZE_
	return(image->data->s);
#else
	return(image->f);
#endif
}

INLINE UInt
GetImageISize(ImageI *image)
{
  return(image->x * image->y);
}

INLINE UInt
GetImageISizeX(ImageI *image)
{
  return(image->x);
}

INLINE UInt
GetImageISizeY(ImageI *image)
{
  return(image->y);
}

INLINE Float *
GetImageFData(ImageF *image)		/* Image F */
{
  return(image->data->f);
}

INLINE UInt
GetImageFSize(ImageF *image)
{
  return(image->x * image->y);
}

INLINE UInt
GetImageFSizeX(ImageF *image)
{
  return(image->x);
}

INLINE UInt
GetImageFSizeY(ImageF *image)
{
  return(image->y);
}
/****************************************************************************/
INLINE Int
GetVopIsObjectLayerIdentifier(Vop *vop)
{
  return (vop->is_object_layer_identifier);
}

INLINE Int
GetVopVisualObjectLayerVerid(Vop *vop)
{
  return (vop->visual_object_layer_verid);
}

INLINE Int
GetVopVisualObjectLayerPriority(Vop *vop)
{
  return (vop->visual_object_layer_priority);
}

INLINE Int
GetVopVolControlParameters(Vop *vop)
{
  return (vop->vol_control_parameters);
}

INLINE Int
GetVopFixedVopRate(Vop *vop)
{
  return (vop->fixed_vop_rate);
}

INLINE Int
GetVopNot8Bit(Vop *vop)
{
  return (vop->not_8_bit);
}

INLINE Int
GetVopQuantPrecision(Vop *vop)
{
  return (vop->quant_precision);
}

INLINE Int
GetVopBitsPerPixel(Vop *vop)
{
  return (vop->bits_per_pixel);
}

INLINE Int
GetVopComplexityEstimationDisable(Vop *vop)
{
  return (vop->complexity_estimation_disable);
}

INLINE Int
GetVopRandomAccessibleVol(Vop *vop)
{
  return (vop->random_accessible_vol);
}

INLINE Int
GetVopConstantAlpha(Vop *vop)
{
  return (vop->constant_alpha);
}

INLINE Int
GetVopConstantAlphaValue(Vop *vop)
{
  return (vop->constant_alpha_value);
}

INLINE Int
GetVopInterlaced(Vop *vop)
{
  return (vop->interlaced);
}

INLINE Int
GetVopTopFieldFirst(Vop *vop)
{
  return (vop->top_field_first);
}

INLINE Int
GetVopAlternateScan(Vop *vop)
{
  return (vop->alternate_scan);
}

INLINE Int
GetVopTimeIncrementResolution(Vop *vop)
{
 return (vop->time_increment_resolution);
 }

INLINE Int
GetVopCoded(Vop *vop)
{
  return(vop->vop_coded);
} 

INLINE Int
GetVopModTimeBase(Vop *vop)
{
  return(vop->mod_time_base);
}

INLINE Float 
GetVopTimeInc(Vop *vop)
{
  return(vop->time_inc);
}

INLINE Int
GetVopPredictionType(Vop *vop)
{
  return(vop->prediction_type);
}

INLINE Int
GetVopIntraDCVlcThr(Vop *vop) 
 {
  return (vop->intra_dc_vlc_thr);
 } 

INLINE Int
GetVopRoundingType(Vop *vop)
{
  return(vop->rounding_type);
}

INLINE Int
GetVopWidth(Vop *vop)
{
  return(vop->width);
}

INLINE Int
GetVopHeight(Vop *vop)
{
  return(vop->height);
}

INLINE Int
GetVopHorSpatRef(Vop *vop)
{
  return(vop->hor_spat_ref);
}

INLINE Int
GetVopVerSpatRef(Vop *vop)
{
  return(vop->ver_spat_ref);
}

INLINE Int
GetVopBackComp(Vop *vop)
{
  return(vop->back_comp);
}

INLINE Int
GetVopQuantizer(Vop *vop)
{
  return(vop->quantizer);
}

INLINE Int
GetVopBQuantizer(Vop *vop)
{
  return(vop->B_quantizer);
}

INLINE Int
GetVopGLQuantizer(Vop *vop)
{
	return(vop->gl_quantizer);
}

INLINE Int
GetVopLoadBackShape(Vop *vop)
{
  return(vop->load_back_shape);
}

INLINE Int
GetVopLoadForShape(Vop *vop)
{
  return(vop->load_for_shape);
}

INLINE Int
GetVopRefSelCode(Vop *vop)
{
  return(vop->ref_sel_code);
}

INLINE Int
GetVopForTempRef(Vop *vop)
{
  return(vop->for_temp_ref);
}

INLINE Int
GetVopBackTempRef(Vop *vop)
{
  return(vop->back_temp_ref);
}

INLINE Int
GetVopId(Vop *vop)
{
  return(vop->ident);
}

INLINE Int
GetVopVOId(Vop *vop)
{
  return(vop->vo_id);
}

INLINE Int
GetVopArbitraryShape(Vop *vop)
{
  return(vop->arbitrary_shape);
}

INLINE Int
GetVopShape(Vop *vop)
{
  return(vop->arbitrary_shape);
}

INLINE Int
GetVopShapeEffects(Vop *vop)
{
  return(vop->shape_effects);
}

INLINE Int
GetVopChangeCRDisable(Vop *vop)
{
	return (vop->change_CR_disable);
}

INLINE Int
GetVopQuantType(Vop *vop)
{
  return(vop->quant_type);
}

INLINE Int
GetVopLoadIntraQuantMat(Vop *vop)
{
  return(vop->load_intra_quant_mat);
}

INLINE Int *
GetVopIntraQuantMat(Vop *vop)
{
  return(vop->intra_quant_mat);
}

INLINE Int
GetVopLoadNonintraQuantMat(Vop *vop)
{
  return(vop->load_nonintra_quant_mat);
}

INLINE Int *
GetVopNonintraQuantMat(Vop *vop)
{
  return(vop->nonintra_quant_mat);
}

INLINE Int
GetVopDisableGrayQuantUpdate(Vop *vop)
{
  return(vop->disable_gray_quant_update);
}

INLINE Int
GetVopLoadGrayIntraQuantMat(Vop *vop)
{
  return(vop->load_gray_intra_quant_mat);
}

INLINE Int *
GetVopGrayIntraQuantMat(Vop *vop)
{
  return(vop->gray_intra_quant_mat);
}

INLINE Int
GetVopLoadGrayNonintraQuantMat(Vop *vop)
{
  return(vop->load_gray_nonintra_quant_mat);
}

INLINE Int *
GetVopGrayNonintraQuantMat(Vop *vop)
{
  return(vop->gray_nonintra_quant_mat);
}

INLINE Int 
GetVopIntraQuantizer(Vop *vop)
{
	return(vop->intra_quantizer);
}

INLINE Int
GetVopErrorResDisable(Vop *vop)
{
	return(vop->error_res_disable);
}

INLINE Int
GetVopDataPartEnable(Vop *vop)
{
        return(vop->data_partitioning);
}

INLINE Int
GetVopReverseVlc(Vop *vop)
{
	return(vop->reverse_vlc);
}

INLINE Int
GetVopIntraACDCPredDisable(Vop *vop)
{
  return(vop->intra_acdc_pred_disable);
}

INLINE Int
GetVopSADCTDisable(Vop *vop)
{
	return(vop->sadct_disable);
}

INLINE Int 
GetVopOBMCDisable(Vop *vop)
{
      return(vop->obmc_disable);
} 

INLINE Int
GetVopFCodeFor(Vop *vop)
{
  return(vop->fcode_for);
}

INLINE Int
GetVopFCodeBack(Vop *vop)
{
  return(vop->fcode_back);
}

INLINE Int
GetVopSearchRangeFor(Vop *vop)
{
  return(vop->sr_for);
}

INLINE Int
GetVopSearchRangeBack(Vop *vop)
{
  return(vop->sr_back);
}

INLINE Int
GetVopShapeCodingType(Vop *vop)
{
	return(vop->shape_coding_type);
}

INLINE Int
GetVopSepMotText(Vop *vop)
{
  return(vop->sep_mot_text);
}

INLINE Int
GetVopSepMotShapeText(Vop *vop)
{
  return(vop->sep_mot_text);
}

INLINE Int
GetVopScalability(Vop *vop)
{
  return(vop->scalability);
}

INLINE Int
GetVopRefId(Vop *vop)
{
  return(vop->ref_ident);
}

INLINE Int
GetVopRefSampDir(Vop *vop)
{
  return(vop->ref_samp_dir);
}

INLINE Int
GetVopHorSampN(Vop *vop)
{
  return(vop->hor_samp_n);
}

INLINE Int
GetVopHorSampM(Vop *vop)
{
  return(vop->hor_samp_m);
}

INLINE Int
GetVopVerSampN(Vop *vop)
{
  return(vop->ver_samp_n);
}

INLINE Int
GetVopVerSampM(Vop *vop)
{
  return(vop->ver_samp_m);
}

INLINE Int
GetVopEnhanceType(Vop *vop)
{
  return(vop->enhancement_type);
}

INLINE Int
GetVopScaling(Vop *vop)
{
  return(vop->scaling);
}

INLINE Int
GetVopVisibility(Vop *vop)
{
  return(vop->visibility);
}

INLINE Int
GetVopBinaryShape(Vop *vop)
{
  return(vop->binary_shape);
}

INLINE Int
GetVopCompOrder(Vop *vop)
{
  return(vop->comp_order);
}

INLINE Int     
GetVopSpriteUsage(Vop *vop)
{
  return(vop->sprite_usage);
}

INLINE Int     
GetVopNoOfSpritePoints(Vop *vop)
{
  return(vop->no_of_sprite_points);
}

INLINE TrajPoint *    
GetVopRefPointCoord(Vop *vop)
{
  return(vop->ref_point_coord);
}

INLINE TrajPoint *    
GetVopTrajPointCoord(Vop *vop)
{
  return(vop->traj_point_coord);
}

INLINE TrajPoint *    
GetVopDiffTrajPointCoord(Vop *vop)
{
  return(vop->difftraj_point_coord);
}

INLINE Int     
GetVopLightingChangeInSprite(Vop *vop)
{
  return(vop->lighting_change_in_sprite);
}

INLINE Int    
GetVopBlendFact(Vop *vop)
{
  return(vop->blend_fact);
}

INLINE Int    
GetVopSpriteHdim(Vop *vop)
{
  return(vop->sprite_hdim);
}

INLINE Int    
GetVopSpriteVdim(Vop *vop)
{
  return(vop->sprite_vdim);
}

INLINE Int    
GetVopSpriteLeftEdge(Vop *vop)
{
  return(vop->sprite_left_edge);
}

INLINE Int    
GetVopSpriteTopEdge(Vop *vop)
{
  return(vop->sprite_top_edge);
}

INLINE Int    
GetVopWarpingAccuracy(Vop *vop)
{
  return(vop->warping_accuracy);
}

INLINE Vop *
GetVopSprite(Vop *vop)
{
  return(vop->rec_sprite);
}

INLINE Sprite_motion *
GetVopWarpParam(Vop *vop)
{
  return(vop->warp_param);
}

INLINE Float
GetVopLightingChangeFactor(Vop *vop)
{
  return(vop->lighting_change_factor);
}

INLINE Int
GetVopSpriteTransmitMode(Vop *vop)
{
	return(vop->sprite_transmit_mode);
}

INLINE Int *
GetVopSpriteTabTransmit(Vop *vop)
{
	return(vop->tab_transmit);
}

INLINE Int *
GetVopSpriteTabAmbType(Vop *vop)
{
	return(vop->tab_amb_type);
}

INLINE Int *
GetVopSpriteTabMvda(Vop *vop)
{
	return(vop->tab_mvda);
}

INLINE SInt *
GetVopSpriteTabQPStore(Vop *vop)
{
	return(vop->tab_QP_store);
}

INLINE Int *
GetVopSpriteTabDQUANTStore(Vop *vop)
{
	return(vop->tab_DQUANT_store);
}

INLINE Int ***
GetVopSpriteTabDCStore(Vop *vop)
{
	return(vop->tab_DC_store);
}

INLINE Image *
GetVopA(Vop *vop)
{
  return(vop->a_chan);
}

INLINE Image *
GetVopAuv(Vop *vop)
{
  return(vop->a_uv_chan);
}

INLINE Image *
GetVopY(Vop *vop)
{
  return(vop->y_chan);
}

INLINE Image *
GetVopU(Vop *vop)
{
  return(vop->u_chan);
}

INLINE Image *
GetVopV(Vop *vop)
{
  return(vop->v_chan);
}

INLINE Image *
GetVopQP(Vop *vop)
{
  return(vop->QP_chan);
}

INLINE Vop *
GetVopNextSpat(Vop *vop)
{
  return(vop->pnext_spat);
}

INLINE Vop *
GetVopNextTemp(Vop *vop)
{
  return(vop->pnext_temp);
}

INLINE Vop *
GetVopPrevTemp(Vop *vop)
{
  return(vop->pprev_temp);
}

INLINE VopExtend *
GetVopExtend(Vop *vop)
{
  return(vop->pextend);
}

INLINE Vop *
GetVopById(Vop *vop, Int vop_id)
{
  Vop *locvop;
  Int id;
  
  locvop = vop;
  
  while(locvop != NULL)
    {
      id = GetVopId(locvop);
      if(id != vop_id)
	locvop = GetVopNextSpat(locvop);
      else
	break;
    }
  
  return(locvop);
}


INLINE Vop *
GetVopByCompOrder(Vop *list, Int comp_order)
{
  Vop          *curr_vop;

  Int          vop_comp_order;

  curr_vop = list;

  while(curr_vop != NULL) 
    {   
   
      vop_comp_order = GetVopCompOrder(curr_vop);
   
      if(vop_comp_order == comp_order)
	{
	  return(curr_vop);
	}
      else
	{
	  curr_vop = GetVopNextSpat(curr_vop);
	}
    }

  return NULL;
}
/****************************************************************************/
INLINE Void
PutVopVolControlParameters(Int vol_control_parameters,Vop *vop)
{
   vop->vol_control_parameters = vol_control_parameters;
}

INLINE Void
PutVopQuantPrecision(Int quant_precision,Vop *vop)
{
   vop->quant_precision = quant_precision;
} 

INLINE Void
PutVopComplexityEstimationDisable(Int complexity_estimation_disable,Vop *vop)
{
   vop->complexity_estimation_disable = complexity_estimation_disable;
} 

INLINE Void
PutVopRandomAccessibleVol(Int random_accessible_vol,Vop *vop)
{
   vop->random_accessible_vol = random_accessible_vol;
}

INLINE Void
PutVopIsObjectLayerIdentifier(Int is_object_layer_identifier,Vop *vop)
{
   vop->is_object_layer_identifier = is_object_layer_identifier;
}

INLINE Void
PutVopVisualObjectLayerVerid(Int visual_object_layer_verid,Vop *vop)
{
   vop->visual_object_layer_verid = visual_object_layer_verid;
} 

INLINE Void
PutVopVisualObjectLayerPriority(Int visual_object_layer_priority,Vop *vop)
{
   vop->visual_object_layer_priority = visual_object_layer_priority;
} 

INLINE Void
PutVopFixedVopRate(Int fixed_vop_rate,Vop *vop)
{
   vop->fixed_vop_rate = fixed_vop_rate;
} 

INLINE Void
PutVopNot8Bit(Int not_8_bit,Vop *vop)
{
   vop->not_8_bit = not_8_bit;
} 

INLINE Void
PutVopBitsPerPixel(Int bits_per_pixel,Vop *vop)
{
   vop->bits_per_pixel = bits_per_pixel;
} 

INLINE Void
PutVopConstantAlpha(Int constant_alpha,Vop *vop)
{
   vop->constant_alpha = constant_alpha;
}

INLINE Void
PutVopConstantAlphaValue(Int constant_alpha_value,Vop *vop)
{
   vop->constant_alpha_value = constant_alpha_value;
}

INLINE Void
PutVopInterlaced(Int interlaced,Vop *vop)
{
   vop->interlaced = interlaced;
}

INLINE Void
PutVopTopFieldFirst(Int top_field_first,Vop *vop)
{
   vop->top_field_first = top_field_first;
}

INLINE Void
PutVopAlternateScan(Int alternate_scan,Vop *vop)
{
   vop->alternate_scan = alternate_scan;
}

INLINE Void
PutVopTimeIncrementResolution(Int time_incre_res, Vop *vop)
{
  vop->time_increment_resolution=time_incre_res;
 }

INLINE Void 
PutVopCoded(Int vop_coded, Vop *vop)
{
  vop->vop_coded = vop_coded;
}

INLINE Void
PutVopModTimeBase(Int mod_time_base, Vop *vop)
{
  vop->mod_time_base = mod_time_base;
}

INLINE Void
PutVopTimeInc(Float time_inc, Vop *vop)
{
  vop->time_inc = time_inc;
}

INLINE Void
PutVopPredictionType(Int prediction_type, Vop *vop)
{
  vop->prediction_type = prediction_type;
}

INLINE Void
PutVopIntraDCVlcThr(Int intra_dc_vlc_thr,Vop *vop) 
 {
  vop->intra_dc_vlc_thr=intra_dc_vlc_thr;
 } 

INLINE Void
PutVopRoundingType(Int rounding_type, Vop *vop)
{
  vop->rounding_type = rounding_type;
}

INLINE Void
PutVopWidth(Int width, Vop *vop)
{
  vop->width = width;
}

INLINE Void
PutVopHeight(Int height, Vop *vop)
{
  vop->height = height;
}

INLINE Void
PutVopHorSpatRef(Int hor_spat_ref, Vop *vop)
{
  vop->hor_spat_ref = hor_spat_ref;
}

INLINE Void
PutVopVerSpatRef(Int ver_spat_ref, Vop *vop)
{
  vop->ver_spat_ref = ver_spat_ref;
}

INLINE Void
PutVopBackComp(Int back_comp, Vop *vop)
{
  vop->back_comp = back_comp;
}

INLINE Void
PutVopQuantizer(Int quantizer, Vop *vop)
{
  vop->quantizer = quantizer;
}

INLINE Void
PutVopBQuantizer(Int B_quantizer, Vop *vop)
{
  vop->B_quantizer = B_quantizer;
}

INLINE Void
PutVopGLQuantizer(Int gl_quant, Vop *vop)
{
	vop->gl_quantizer = gl_quant;
}

INLINE Void
PutVopLoadBackShape(Int load_back_shape, Vop *vop)
{
  vop->load_back_shape = load_back_shape;
}

INLINE Void
PutVopLoadForShape(Int load_for_shape, Vop *vop)
{
  vop->load_for_shape = load_for_shape;
}

INLINE Void
PutVopRefSelCode(Int ref_sel_code, Vop *vop)
{
  vop->ref_sel_code = ref_sel_code;
}

INLINE Void
PutVopForTempRef(Int for_temp_ref, Vop *vop)
{
  vop->for_temp_ref = for_temp_ref;
}

INLINE Void
PutVopBackTempRef(Int back_temp_ref, Vop *vop)
{
  vop->back_temp_ref = back_temp_ref;
}

INLINE Void
PutVopId(Int ident, Vop *vop)
{
  vop->ident = ident;
}

INLINE Void
PutVopVOId(Int vo_id, Vop *vop)
{
  vop->vo_id = vo_id;
}

INLINE Void
PutVopArbitraryShape(Int arbitrary_shape, Vop *vop)
{
  vop->arbitrary_shape = arbitrary_shape;
}

INLINE Void
PutVopShape(Int arbitrary_shape, Vop *vop)
{
  vop->arbitrary_shape = arbitrary_shape;
}

INLINE Void
PutVopShapeEffects(Int shape_effects, Vop *vop)
{
	vop->shape_effects = shape_effects;
}

INLINE Void
PutVopChangeCRDisable(Int change_CR_disable, Vop *vop)
{
	vop->change_CR_disable = change_CR_disable;
}

INLINE Void
PutVopQuantType(Int quant_type, Vop *vop)
{
  vop->quant_type = quant_type;
}

INLINE Void
PutVopLoadIntraQuantMat(Int load_intra_quant_mat, Vop *vop)
{
  vop->load_intra_quant_mat = load_intra_quant_mat;
}

#if 0
INLINE Void
PutVopIntraQuantMat(Int *intra_quant_mat, Vop *vop)
{
  vop->intra_quant_mat = intra_quant_mat;
}
#endif

INLINE Void
PutVopLoadNonintraQuantMat(Int load_nonintra_quant_mat, Vop *vop)
{
  vop->load_nonintra_quant_mat = load_nonintra_quant_mat;
}

#if 0
INLINE Void
PutVopNonintraQuantMat(Int *nonintra_quant_mat, Vop *vop)
{
  vop->nonintra_quant_mat = nonintra_quant_mat;
}
#endif

INLINE Void
PutVopDisableGrayQuantUpdate(Int disable_gray_quant_update, Vop *vop)
{
  vop->disable_gray_quant_update = disable_gray_quant_update;
}

INLINE Void
PutVopLoadGrayIntraQuantMat(Int load_gray_intra_quant_mat, Vop *vop)
{
  vop->load_gray_intra_quant_mat = load_gray_intra_quant_mat;
}

#if 0
INLINE Void
PutVopGrayIntraQuantMat(Int *gray_intra_quant_mat, Vop *vop)
{
  vop->gray_intra_quant_mat = gray_intra_quant_mat;
}
#endif

INLINE Void
PutVopLoadGrayNonintraQuantMat(Int load_gray_nonintra_quant_mat, Vop *vop)
{
  vop->load_gray_nonintra_quant_mat = load_gray_nonintra_quant_mat;
}

#if 0
INLINE Void
PutVopGrayNonintraQuantMat(Int *gray_nonintra_quant_mat, Vop *vop)
{
  vop->gray_nonintra_quant_mat = gray_nonintra_quant_mat;
}
#endif

INLINE Void
PutVopErrorResDisable(Int flag,Vop *vop)
{
	vop->error_res_disable = flag;
}

INLINE Void
PutVopDataPartEnable(Int flag, Vop *vop)
{
        vop->data_partitioning = flag;
}

INLINE Void
PutVopOBMCDisable(Int flag, Vop *vop)
{
        vop->obmc_disable = flag;
}

INLINE Void
PutVopReverseVlc(Int reverse_vlc,Vop *vop)
{
	vop->reverse_vlc = reverse_vlc;
}

INLINE Void
PutVopIntraACDCPredDisable(Int intra_acdc_pred_disable, Vop *vop)
{
  vop->intra_acdc_pred_disable = intra_acdc_pred_disable;
}

INLINE Void
PutVopSADCTDisable(Int sadct_disable, Vop *vop)
{
	vop->sadct_disable = sadct_disable;
}

INLINE Void
PutVopFCodeFor(Int fcode_for, Vop *vop)
{
  vop->fcode_for = fcode_for;
}

INLINE Void
PutVopFCodeBack(Int fcode_back, Vop *vop)
{
  vop->fcode_back = fcode_back;
}

INLINE Void
PutVopSearchRangeFor(Int sr_for, Vop *vop)
{
  vop->sr_for = sr_for;
}

INLINE Void
PutVopSearchRangeBack(Int sr_back, Vop *vop)
{
  vop->sr_back = sr_back;
}

INLINE Void
PutVopShapeCodingType(Int shape_coding_type, Vop *vop)
{
  vop->shape_coding_type = shape_coding_type;
}

INLINE Void
PutVopSepMotText(Int sep_mot_text, Vop *vop)
{
  vop->sep_mot_text = sep_mot_text;
}

INLINE Void
PutVopSepMotShapeText(Int sep_mot_text, Vop *vop)
{
  vop->sep_mot_text = sep_mot_text;
}

INLINE Void
PutVopScalability(Int scalability, Vop *vop)
{
  vop->scalability = scalability;
}

INLINE Void
PutVopRefId(Int id,Vop *vop)
{
  vop->ref_ident = id;
}

INLINE Void
PutVopRefSampDir(Int dir,Vop *vop)
{
  vop->ref_samp_dir = dir;
}

INLINE Void
PutVopHorSampN(Int n,Vop *vop)
{
  vop->hor_samp_n = n;
}

INLINE Void
PutVopHorSampM(Int m,Vop *vop)
{
  vop->hor_samp_m = m;
}

INLINE Void
PutVopVerSampN(Int n,Vop *vop)
{
  vop->ver_samp_n = n;
}

INLINE Void
PutVopVerSampM(Int m,Vop *vop)
{
  vop->ver_samp_m = m;
}

INLINE Void
PutVopEnhanceType(Int type,Vop *vop)
{
  vop->enhancement_type = type;
}

INLINE Void
PutVopScaling(Int scaling, Vop *vop)
{
  vop->scaling = scaling;
}

INLINE Void
PutVopVisibility(Int visibility, Vop *vop)
{
  vop->visibility = visibility;
}

INLINE Void
PutVopBinaryShape(Int binary_shape, Vop *vop)
{
  vop->binary_shape = binary_shape;
}

INLINE Void
PutVopCompOrder(Int comp_order, Vop *vop)
{
  vop->comp_order = comp_order;
}

INLINE Void
PutVopSpriteUsage(Int sprite_usage, Vop *vop)
{
  vop->sprite_usage = sprite_usage;
}

INLINE Void
PutVopNoOfSpritePoints(Int no_of_sprite_points, Vop *vop)
{
  vop->no_of_sprite_points = no_of_sprite_points;
}

INLINE Void
PutVopRefPointCoord(TrajPoint *ref_point_coord, Vop *vop)
{
  vop->ref_point_coord = ref_point_coord;
}

INLINE Void
PutVopTrajPointCoord(TrajPoint *traj_point_coord, Vop *vop)
{
  vop->traj_point_coord = traj_point_coord;
}

INLINE Void
PutVopDiffTrajPointCoord(TrajPoint *difftraj_point_coord, Vop *vop)
{
  vop->difftraj_point_coord = difftraj_point_coord;
}

INLINE Void
PutVopLightingChangeInSprite(Int lighting_change_in_sprite, Vop *vop)
{
  vop->lighting_change_in_sprite = lighting_change_in_sprite;
}

INLINE Void
PutVopBlendFact(Int blend_fact, Vop *vop)
{
  vop->blend_fact = blend_fact;
}

INLINE Void
PutVopSpriteHdim(Int sprite_hdim, Vop *vop)
{
  vop->sprite_hdim = sprite_hdim;
}

INLINE Void
PutVopSpriteVdim(Int sprite_vdim, Vop *vop)
{
  vop->sprite_vdim = sprite_vdim;
}

INLINE Void
PutVopSpriteLeftEdge(Int sprite_left_edge, Vop *vop)
{
  vop->sprite_left_edge = sprite_left_edge;
}

INLINE Void
PutVopSpriteTopEdge(Int sprite_top_edge, Vop *vop)
{
  vop->sprite_top_edge = sprite_top_edge;
}

INLINE Void
PutVopWarpingAccuracy(Int warping_accuracy, Vop *vop)
{
  vop->warping_accuracy = warping_accuracy;
}

INLINE Void
PutVopSprite(Vop *rec_sprite, Vop *vop)
{
  vop->rec_sprite = rec_sprite;
}

INLINE Void
PutVopWarpParam(Sprite_motion *warp_param, Vop *vop)
{
  vop->warp_param = warp_param;
}

INLINE Void
PutVopLightingChangeFactor(Float lighting_change_factor, Vop *vop)
{
  vop->lighting_change_factor = lighting_change_factor;
}

INLINE Void
PutVopSpriteTransmitMode(Int sprite_transmit_mode, Vop *vop)
{
  vop->sprite_transmit_mode = sprite_transmit_mode;
}

INLINE Void
PutVopSpriteTabTransmit(Int *tab_transmit, Vop *vop)
{
	vop->tab_transmit = tab_transmit;
}

INLINE Void
PutVopSpriteTabAmbType(Int *tab_amb_type, Vop *vop)
{
	vop->tab_amb_type = tab_amb_type;
}

INLINE Void
PutVopSpriteTabMvda(Int *tab_mvda, Vop *vop)
{
	vop->tab_mvda = tab_mvda;
}

INLINE Void
PutVopSpriteTabQPStore(SInt *tab_QP_store, Vop *vop)
{
	vop->tab_QP_store = tab_QP_store;
}

INLINE Void
PutVopSpriteTabDQUANTStore(Int *tab_DQUANT_store, Vop *vop)
{
	vop->tab_DQUANT_store = tab_DQUANT_store;
}

INLINE Void
PutVopSpriteTabDCStore(Int ***tab_DC_store, Vop *vop)
{
	vop->tab_DC_store = tab_DC_store;
}

#ifndef _IM1_CD_DECODER_
INLINE Void
PutVopA(Image *a_chan, Vop *vop)
{
  FreeImage(vop->a_chan);
  vop->a_chan = a_chan;
}

INLINE Void 
PutVopAuv(Image *a_uv_chan, Vop *vop)
{
  FreeImage(vop->a_uv_chan);
  vop->a_uv_chan = a_uv_chan;
}

INLINE Void
PutVopY(Image *y_chan, Vop *vop)
{
  FreeImage(vop->y_chan);
  vop->y_chan = y_chan;
}

INLINE Void
PutVopU(Image *u_chan, Vop *vop)
{
  FreeImage(vop->u_chan);
  vop->u_chan = u_chan;
}

INLINE Void
PutVopV(Image *v_chan, Vop *vop)
{
  FreeImage(vop->v_chan);
  vop->v_chan = v_chan;
}

INLINE Void
PutVopQP(Image *QP_chan, Vop *vop)
{
  FreeImage(vop->QP_chan);
  vop->QP_chan = QP_chan;
}
#endif

INLINE Void
PutVopNextSpat(Vop *pnext_spat, Vop *vop)
{
  vop->pnext_spat = pnext_spat;
}

INLINE Void
PutVopNextTemp(Vop *pnext_temp, Vop *vop)
{
  vop->pnext_temp = pnext_temp;
}

INLINE Void
PutVopPrevTemp(Vop *pprev_temp, Vop *vop)
{
  vop->pprev_temp = pprev_temp;
}

INLINE Void
PutVopExtend(VopExtend *pextend, Vop *vop)
{
  vop->pextend = pextend;
}

INLINE Void
PutVopIntraQuantizer(Int Q,Vop *vop)
{
	vop->intra_quantizer = Q;
}
/****************************************************************************/
INLINE Int
GetMotionId(Motion *motion)
{
  return(motion->ident);
}

INLINE Float *
GetMotionAffine(Motion *motion)
{
  return(motion->affine_pars);
}

INLINE Image *
GetMotionVectorFieldX(Motion *motion, Int num)
{
  return(motion->dvf_x[num]);
}

INLINE Image *
GetMotionVectorFieldY(Motion *motion, Int num)
{
  return(motion->dvf_y[num]);
}

INLINE Int
GetMotionUnderX(Motion *motion, Int num)
{
  return(motion->under_x[num]);
}

INLINE Int
GetMotionUnderY(Motion *motion, Int num)
{
  return(motion->under_y[num]);
}

INLINE Int
GetMotionResX(Motion *motion, Int num)
{
  return(motion->resolution_x[num]);
}

INLINE Int
GetMotionResY(Motion *motion, Int num)
{
  return(motion->resolution_y[num]);
}
/****************************************************************************/
INLINE Int
GetVolIsObjectLayerIdentifier(Vol *vol)
{
  return (vol->is_object_layer_identifier);
}

INLINE Int
GetVolVisualObjectLayerVerid(Vol *vol)
{
  return (vol->visual_object_layer_verid);
}

INLINE Int
GetVolVisualObjectLayerPriority(Vol *vol)
{
  return (vol->visual_object_layer_priority);
}

INLINE Int
GetVolVolControlParameters(Vol *vol)
{
  return (vol->vol_control_parameters);
}

INLINE Int
GetVolFixedVopRate(Vol *vol)
{
  return (vol->fixed_vop_rate);
}

INLINE Int
GetVolNot8Bit(Vol *vol)
{
  return (vol->not_8_bit);
}

INLINE Int
GetVolQuantPrecision(Vol *vol)
{
  return (vol->quant_precision);
}

INLINE Int
GetVolBitsPerPixel(Vol *vol)
{
  return (vol->bits_per_pixel);
}

INLINE Int
GetVolComplexityEstimationDisable(Vol *vol)
{
  return (vol->complexity_estimation_disable);
}

INLINE Int
GetVolRandomAccessibleVol(Vol *vol)
{
  return (vol->random_accessible_vol);
}

INLINE Int
GetVolTimeIncrementResolution(Vol *vol)
{
  return vol->time_increment_resolution;
 }

INLINE Int
GetVolId(Vol *vol)
{
	return(vol->ident);
}

INLINE Int
GetVolVOId(Vol *vol)
{
	return(vol->vo_id);
}

INLINE Int
GetVolShape(Vol *vol)
{
	return(vol->shape);
}

INLINE Int
GetVolShapeEffects(Vol *vol)
{
	return(vol->shape_effects);
}

INLINE Int
GetVolOBMCDisable(Vol *vol)
{
	return(vol->obmc_disable);
}

INLINE Int
GetVolWidth(Vol *vol)
{
	return(vol->width);
}

INLINE Int
GetVolHeight(Vol *vol)
{
	return(vol->height);
}

INLINE Int
GetVolQuantType(Vol *vol)
{
	return(vol->quant_type);
}

INLINE Int
GetVolLoadIntraQuantMat(Vol *vol)
{
  return(vol->load_intra_quant_mat);
}

INLINE Int *
GetVolIntraQuantMat(Vol *vol)
{
  return(vol->intra_quant_mat);
}

INLINE Int
GetVolLoadNonintraQuantMat(Vol *vol)
{
  return(vol->load_nonintra_quant_mat);
}

INLINE Int *
GetVolNonintraQuantMat(Vol *vol)
{
  return(vol->nonintra_quant_mat);
}

INLINE Int
GetVolDisableGrayQuantUpdate(Vol *vol)
{
  return(vol->disable_gray_quant_update);
}

INLINE Int
GetVolLoadGrayIntraQuantMat(Vol *vol)
{
  return(vol->load_gray_intra_quant_mat);
}

INLINE Int *
GetVolGrayIntraQuantMat(Vol *vol)
{
  return(vol->gray_intra_quant_mat);
}

INLINE Int
GetVolLoadGrayNonintraQuantMat(Vol *vol)
{
  return(vol->load_gray_nonintra_quant_mat);
}

INLINE Int *
GetVolGrayNonintraQuantMat(Vol *vol)
{
  return(vol->gray_nonintra_quant_mat);
}

INLINE Int
GetVolErrorResDisable(Vol *vol)
{
	return(vol->error_res_disable);
}

INLINE Int
GetVolDataPartEnable(Vol *vol)
{
        return(vol->data_partitioning);
}

INLINE Int
GetVolReverseVlc(Vol *vol)
{
	return(vol->reverse_vlc);
}

INLINE Int
GetVolACDCPredDisable(Vol *vol)
{
	return(vol->intra_acdc_pred_disable);
}

INLINE Int 
GetVolSADCTDisable(Vol *vol)
{
	return(vol->sadct_disable);
}

INLINE Int
GetVolSepMotShapeText(Vol *vol)
{
	return(vol->sep_mot_shape_text);
}

INLINE Int
GetVolScalability(Vol *vol)
{
	return(vol->scalability);
}

INLINE Int
GetVolScalType(Vol *vol)
{
	return(vol->scal_type);
}

INLINE Int
GetVolRefId(Vol *vol)
{
	return(vol->ref_ident);
}

INLINE Int
GetVolRefSampDir(Vol *vol)
{
	return(vol->ref_samp_dir);
}

INLINE Int
GetVolHorSampN(Vol *vol)
{
	return(vol->hor_samp_n);
}

INLINE Int
GetVolHorSampM(Vol *vol)
{
	return(vol->hor_samp_m);
}

INLINE Int
GetVolVerSampN(Vol *vol)
{
	return(vol->ver_samp_n);
}

INLINE Int
GetVolVerSampM(Vol *vol)
{
	return(vol->ver_samp_m);
}

INLINE Int
GetVolEnhanceType(Vol *vol)
{
	return(vol->enhancement_type);
}

INLINE Int 
GetVolSpriteHdim(Vol *vol)
{
	return(vol->sprite_hdim);
}

INLINE Int
GetVolSpriteVdim(Vol *vol)
{
	return(vol->sprite_vdim);
}

INLINE Int
GetVolSpriteLeftEdge(Vol *vol)
{
	return(vol->sprite_left_edge);
}

INLINE Int
GetVolSpriteTopEdge(Vol *vol)
{
	return(vol->sprite_top_edge);
}

INLINE Int
GetVolSpriteInitPieceWidth(Vol *vol)
{
	return(vol->init_piece_width);
}

INLINE Int
GetVolSpriteInitPieceHeight(Vol *vol)
{
	return(vol->init_piece_height);
}

INLINE Int
GetVolSpriteInitPieceHorRef(Vol *vol)
{
	return(vol->init_piece_hor_ref);
}

INLINE Int
GetVolSpriteInitPieceVerRef(Vol *vol)
{
	return(vol->init_piece_ver_ref);
}

INLINE Int 
GetVolBlendFact(Vol *vol)
{
	return(vol->blend_fact);
}

INLINE Int 
GetVolSpriteUsage(Vol *vol)
{
	return(vol->sprite_usage);
}

INLINE Int 
GetVolWarpingAccuracy(Vol *vol)
{
	return(vol->warping_accuracy);
}

INLINE Int 
GetVolLightingChangeInSprite(Vol *vol)
{
	return(vol->lighting_change_in_sprite);
}

INLINE Int 
GetVolNoOfSpritePoints(Vol *vol)
{
	return(vol->no_of_sprite_points );
}

INLINE TrajPoint *
GetVolRefPointCoord(Vol *vol)
{
	return(vol->ref_point_coord );
}

INLINE TrajPoint *
GetVolTrajPointCoord(Vol *vol)
{
	return(vol->traj_point_coord );
}

INLINE Sprite_motion *
GetVolPrevWarpParam(Vol *vol)
{
	return(vol->prev_warp_param);
}

INLINE Vop *
GetVolSprite(Vol *vol)
{
	return(vol->psprite);
}

INLINE Int
GetVolSpriteTransmitMode(Vol *vol)
{
	return(vol->sprite_transmit_mode);
}

INLINE Int *
GetVolSpriteTabTransmit(Vol *vol)
{
	return(vol->tab_transmit);
}

INLINE Int *
GetVolSpriteTabAmbType(Vol *vol)
{
	return(vol->tab_amb_type);
}

INLINE Int *
GetVolSpriteTabMvda(Vol *vol)
{
	return(vol->tab_mvda);
}

INLINE SInt *
GetVolSpriteTabQPStore(Vol *vol)
{
	return(vol->tab_QP_store);
}

INLINE Int *
GetVolSpriteTabDQUANTStore(Vol *vol)
{
	return(vol->tab_DQUANT_store);
}

INLINE Int ***
GetVolSpriteTabDCStore(Vol *vol)
{
	return(vol->tab_DC_store);
}

INLINE Vop *
GetVolVop(Vol *vol)
{
	return(vol->pvop);
}

INLINE Vol *
GetVolNext(Vol *vol)
{
	return(vol->pnext);
}
/****************************************************************************/
INLINE Void
PutVolIsObjectLayerIdentifier(Int is_object_layer_identifier,Vol *vol)
{
   vol->is_object_layer_identifier = is_object_layer_identifier;
} 

INLINE Void
PutVolVisualObjectLayerVerid(Int visual_object_layer_verid,Vol *vol)
{
   vol->visual_object_layer_verid = visual_object_layer_verid;
} 

INLINE Void
PutVolVisualObjectLayerPriority(Int visual_object_layer_priority,Vol *vol)
{
   vol->visual_object_layer_priority = visual_object_layer_priority;
}

INLINE Void
PutVolVolControlParameters(Int vol_control_parameters,Vol *vol)
{
   vol->vol_control_parameters = vol_control_parameters;
} 

INLINE Void
PutVolFixedVopRate(Int fixed_vop_rate,Vol *vol)
{
   vol->fixed_vop_rate = fixed_vop_rate;
}

INLINE Void
PutVolNot8Bit(Int not_8_bit,Vol *vol)
{
   vol->not_8_bit = not_8_bit;
}

INLINE Void
PutVolQuantPrecision(Int quant_precision,Vol *vol)
{
   vol->quant_precision = quant_precision;
} 

INLINE Void
PutVolBitsPerPixel(Int bits_per_pixel,Vol *vol)
{
   vol->bits_per_pixel = bits_per_pixel;
}

INLINE Void
PutVolComplexityEstimationDisable(Int complexity_estimation_disable,Vol *vol)
{
   vol->complexity_estimation_disable = complexity_estimation_disable;
} 

INLINE Void
PutVolRandomAccessibleVol(Int random_accessible_vol,Vol *vol)
{
   vol->random_accessible_vol = random_accessible_vol;
}

INLINE Void
PutVolTimeIncrementResolution(Int time_incre_res, Vol *vol)
{
  vol->time_increment_resolution=time_incre_res;
 }

INLINE Void
PutVolId(Int id,Vol *vol)
{
	vol->ident = id;
}

INLINE Void
PutVolVOId(Int vo_id,Vol *vol)
{
	vol->vo_id = vo_id;
}

INLINE Void
PutVolShape(Int shape,Vol *vol)
{
	vol->shape = shape;
}

INLINE Void
PutVolShapeEffects(Int shape_effects,Vol *vol)
{
	vol->shape_effects = shape_effects;
}

INLINE Void
PutVolOBMCDisable(Int OBMCDisable,Vol *vol)
{
	vol->obmc_disable = OBMCDisable;
}

INLINE Void
PutVolWidth(Int width,Vol *vol)
{
	vol->width = width;
}

INLINE Void
PutVolHeight(Int height, Vol *vol)
{
	vol->height = height;
}

INLINE Void
PutVolQuantType(Int quant_type, Vol *vol)
{
  vol->quant_type = quant_type;
}

INLINE Void
PutVolLoadIntraQuantMat(Int load_intra_quant_mat, Vol *vol)
{
  vol->load_intra_quant_mat = load_intra_quant_mat;
}

#if 0
INLINE Void
PutVolIntraQuantMat(Int *intra_quant_mat, Vol *vol)
{
  vol->intra_quant_mat = intra_quant_mat;
}
#endif

INLINE Void
PutVolLoadNonintraQuantMat(Int load_nonintra_quant_mat, Vol *vol)
{
  vol->load_nonintra_quant_mat = load_nonintra_quant_mat;
}

#if 0
INLINE Void
PutVolNonintraQuantMat(Int *nonintra_quant_mat, Vol *vol)
{
  vol->nonintra_quant_mat = nonintra_quant_mat;
}
#endif

INLINE Void
PutVolDisableGrayQuantUpdate(Int disable_gray_quant_update, Vol *vol)
{
  vol->disable_gray_quant_update = disable_gray_quant_update;
}

INLINE Void
PutVolLoadGrayIntraQuantMat(Int load_gray_intra_quant_mat, Vol *vol)
{
  vol->load_gray_intra_quant_mat = load_gray_intra_quant_mat;
}

#if 0
INLINE Void
PutVolGrayIntraQuantMat(Int *gray_intra_quant_mat, Vol *vol)
{
  vol->gray_intra_quant_mat = gray_intra_quant_mat;
}
#endif

INLINE Void
PutVolLoadGrayNonintraQuantMat(Int load_gray_nonintra_quant_mat, Vol *vol)
{
  vol->load_gray_nonintra_quant_mat = load_gray_nonintra_quant_mat;
}

#if 0
INLINE Void
PutVolGrayNonintraQuantMat(Int *gray_nonintra_quant_mat, Vol *vol)
{
  vol->gray_nonintra_quant_mat = gray_nonintra_quant_mat;
}
#endif

INLINE Void
PutVolErrorResDisable(Int flag,Vol *vol)
{
	vol->error_res_disable = flag;
}

INLINE Void
PutVolDataPartEnable(Int flag,Vol *vol)
{
        vol->data_partitioning = flag;
}

INLINE Void
PutVolReverseVlc(Int reverse_vlc,Vol *vol)
{
	vol->reverse_vlc = reverse_vlc;
}

INLINE Void
PutVolACDCPredDisable(Int flag,Vol *vol)
{
	vol->intra_acdc_pred_disable = flag;
}

INLINE Void
PutVolSADCTDisable(Int flag,Vol *vol)
{
	vol->sadct_disable = flag;
}

INLINE Void
PutVolSepMotShapeText(Int flag,Vol *vol)
{
	vol->sep_mot_shape_text = flag;
}

INLINE Void
PutVolScalability(Int scale,Vol *vol)
{
	vol->scalability = scale;
}

INLINE Void
PutVolScalType(Int scale,Vol *vol)
{
	vol->scal_type = scale;
}

INLINE Void
PutVolRefId(Int id,Vol *vol)
{
	vol->ref_ident = id;
}

INLINE Void
PutVolRefSampDir(Int dir,Vol *vol)
{
	vol->ref_samp_dir = dir;
}

INLINE Void
PutVolHorSampN(Int n,Vol *vol)
{
	vol->hor_samp_n = n;
}

INLINE Void
PutVolHorSampM(Int m,Vol *vol)
{
	vol->hor_samp_m = m;
}

INLINE Void
PutVolVerSampN(Int n,Vol *vol)
{
	vol->ver_samp_n = n;
}

INLINE Void
PutVolVerSampM(Int m,Vol *vol)
{
	vol->ver_samp_m = m;
}

INLINE Void
PutVolEnhanceType(Int type,Vol *vol)
{
	vol->enhancement_type = type;
}

INLINE Void
PutVolSpriteHdim(Int sprite_hdim,Vol *vol)
{
	vol->sprite_hdim = sprite_hdim;
}

INLINE Void
PutVolSpriteVdim(Int sprite_vdim,Vol *vol)
{
	vol->sprite_vdim = sprite_vdim;
}

INLINE Void
PutVolSpriteLeftEdge(Int sprite_left_edge, Vol *vol)
{
	vol->sprite_left_edge = sprite_left_edge;
}

INLINE Void
PutVolSpriteTopEdge(Int sprite_top_edge, Vol *vol)
{
	vol->sprite_top_edge = sprite_top_edge;
}

INLINE Void
PutVolSpriteInitPieceWidth( Int init_piece_width, Vol *vol)
{
	vol->init_piece_width = init_piece_width;
}

INLINE Void
PutVolSpriteInitPieceHeight( Int init_piece_height, Vol *vol)
{
	vol->init_piece_height = init_piece_height;
}

INLINE Void
PutVolSpriteInitPieceHorRef( Int init_piece_hor_ref, Vol *vol)
{
	vol->init_piece_hor_ref = init_piece_hor_ref;
}

INLINE Void
PutVolSpriteInitPieceVerRef( Int init_piece_ver_ref, Vol *vol)
{
	vol->init_piece_ver_ref = init_piece_ver_ref;
}

INLINE Void
PutVolBlendFact(Int blend_fact,Vol *vol)
{
	vol->blend_fact = blend_fact;
}

INLINE Void
PutVolSpriteUsage(Int sprite_usage, Vol *vol )
{
	vol->sprite_usage = sprite_usage;
}

INLINE Void
PutVolWarpingAccuracy(Int warping_accuracy, Vol *vol )
{
	vol->warping_accuracy = warping_accuracy;
}

INLINE Void
PutVolNoOfSpritePoints(Int no_of_sprite_points, Vol *vol )
{
	vol->no_of_sprite_points = no_of_sprite_points ;
}

INLINE Void
PutVolRefPointCoord(TrajPoint *ref_point_coord, Vol *vol )
{
	vol->ref_point_coord = ref_point_coord ;
}

INLINE Void
PutVolTrajPointCoord(TrajPoint *traj_point_coord, Vol *vol )
{
	vol->traj_point_coord = traj_point_coord ;
}

INLINE Void
PutVolPrevWarpParam(Sprite_motion *prev_warp_param, Vol *vol)
{
	vol->prev_warp_param = prev_warp_param;
}

INLINE Void
PutVolLightingChangeInSprite( Int lighting_change_in_sprite, Vol *vol )
{
	vol->lighting_change_in_sprite = lighting_change_in_sprite;
}

INLINE Void
PutVolSprite(Vop *sprite,Vol *vol)
{
	vol->psprite = sprite;
}

INLINE Void
PutVolSpriteTransmitMode(Int sprite_transmit_mode, Vol *vol)
{
	vol->sprite_transmit_mode = sprite_transmit_mode;
}

INLINE Void
PutVolSpriteTabTransmit(Int *tab_transmit, Vol *vol)
{
	vol->tab_transmit = tab_transmit;
}

INLINE Void
PutVolSpriteTabAmbType(Int *tab_amb_type, Vol *vol)
{
	vol->tab_amb_type = tab_amb_type;
}

INLINE Void
PutVolSpriteTabMvda(Int *tab_mvda, Vol *vol)
{
	vol->tab_mvda = tab_mvda;
}

INLINE Void
PutVolSpriteTabQPStore(SInt *tab_QP_store, Vol *vol)
{
	vol->tab_QP_store = tab_QP_store;
}

INLINE Void
PutVolSpriteTabDQUANTStore(Int *tab_DQUANT_store, Vol *vol)
{
	vol->tab_DQUANT_store = tab_DQUANT_store;
}

INLINE Void
PutVolSpriteTabDCStore(Int ***tab_DC_store, Vol *vol)
{
	vol->tab_DC_store = tab_DC_store;
}

INLINE Void
PutVolVop(Vop *vop,Vol *vol)
{
	vol->pvop = vop;
}

INLINE Void
PutVolNext(Vol *next,Vol *vol)
{
	vol->pnext = next;
}
/****************************************************************************/
INLINE Void
PutVOId(Int id,VO *vo)
{
	vo->id = id;
}

INLINE Void
PutVOLayers(Vol *layers,VO *vo)
{
	vo->layers = layers;
}

INLINE Void
PutVONext(VO *next,VO *vo)
{
	vo->pnext = next;
}
/****************************************************************************/
INLINE Int
GetVOId(VO *vo)
{
	return(vo->id);
}

INLINE Vol *
GetVOLayers(VO *vo)
{
	return(vo->layers);
}

INLINE VO *
GetVONext(VO *vo)
{
	return(vo->pnext);
}
/****************************************************************************/
#endif

/* #include "mom_access.p" */
       
#endif /* _MOM_ACCESS_H_ */
