#ifndef _VM_MPEG4CDDEC_H_
#define _VM_MPEG4CDDEC_H_


#ifdef __cplusplus
extern "C" {
#endif /* __cplusplus */


/****************************************************/
/* A.Pacheco: 2/2/98:                               */
/* Inteface of the MPEG4 CD decoder for the IM1 api */
/****************************************************/

#include <string.h>				   
#include "momusys.im1cd.h"
#include "mom_structs.im1cd.h"
#include "vm_common_defs.h"
#include "mom_bitstream_d.im1cd.h"

#ifdef WIN32
#define APADLLIMPORTS		__declspec( dllimport )
#else
#define APADLLIMPORTS
#endif

/***********************************************************CommentBegin******
 *
 * -- InitMPEG4CDDecoder -- 
 *
 *          Initializates the MPEG4 CD decoder.
 *			(=> no data is needed in the buffer for this operation
 ***********************************************************CommentEnd********/

APADLLIMPORTS 
Void CallingConvention InitMPEG4CDDecoder(
	Int width, Int height, 
	Int vols,  Int framerate, 
	Int filter_type,
	Int        readen_bits[MAX_NUM_VOS][MAX_NUM_VOLS],
	Int        stop_decoding_vol[MAX_NUM_VOS][MAX_NUM_VOLS],
	Vop      **display_vop, 
	Float     *time_between_two_frames,
	Float     *next_display_time,
	Int		  *random_access_start_time,
	Bitstream *stream[MAX_NUM_VOS][MAX_NUM_VOLS],
    Vol       *display_vol_list[MAX_NUM_VOS],
    Trace     *trace[MAX_NUM_VOS][MAX_NUM_VOLS],
	Char       trace_filename[MAX_NUM_VOS][MAX_NUM_VOLS][300],
	Vop	      *base_output_vop[3],
	UChar **PRE_MB_TYPE2[MAX_NUM_VOS][MAX_NUM_VOLS],
	Int bs_vo[MAX_NUM_VOS],
	Int bs_vol[MAX_NUM_VOS][MAX_NUM_VOLS],
	Float      *display_framerate_TPS,
	Int			num_cd_decoders,
	UChar **stock_mb_type2[3][MAX_NUM_VOS][MAX_NUM_VOLS]
	);


/***********************************************************CommentBegin******
 *
 * -- DecodeMPEG4CDHeaders -- 
 *
 *          Decodes the data from the MPEG4 CD headers.
 *          (=>there must be enought data in the buffer to do this)
 *
 ***********************************************************CommentEnd********/

APADLLIMPORTS
Void CallingConvention DecodeMPEG4CDHeaders(
  	Int        width, Int height, 
	Int        vols,  Int framerate, 
	Int        filter_type,
	Int		  *temporal_scalability,
	Int        readen_bits[MAX_NUM_VOS][MAX_NUM_VOLS],
	Int        stop_decoding_vol[MAX_NUM_VOS][MAX_NUM_VOLS],
	Vop		  *display_vop, 
	Float     *time_between_two_frames,
	Float     *next_display_time,
	Int		   random_access_start_time,
	Bitstream *stream[MAX_NUM_VOS][MAX_NUM_VOLS],
    Vol       *display_vol_list[MAX_NUM_VOS],
    Trace     *trace[MAX_NUM_VOS][MAX_NUM_VOLS],
	Char       trace_filename[MAX_NUM_VOS][MAX_NUM_VOLS][300],
	Int        start_time[MAX_NUM_VOLS][2],
	Int   *prev_ref,Int   *next_ref,
	UChar **MB_TYPE2[MAX_NUM_VOS][MAX_NUM_VOLS],
	Int PRE_H_SIZE2[MAX_NUM_VOS][MAX_NUM_VOLS],
	Int PRE_V_SIZE2[MAX_NUM_VOS][MAX_NUM_VOLS],
	UChar **PRE_MB_TYPE2[MAX_NUM_VOS][MAX_NUM_VOLS],
	Int bs_vol[MAX_NUM_VOS][MAX_NUM_VOLS],
	Image   *alpha_rec_packet,
	Image   *alpha_mode_packet,
	SInt Px_for,SInt Py_for, SInt Px_back, SInt Py_back,
	Int prev_j,Int zero_edge,Int prev_time_base,
	UChar **stock_mb_type2[3][MAX_NUM_VOS][MAX_NUM_VOLS],
	Int   stock_h_size2[3][MAX_NUM_VOS][MAX_NUM_VOLS],
	Int   stock_v_size2[3][MAX_NUM_VOS][MAX_NUM_VOLS]
);

/***********************************************************CommentBegin******
 *
 * -- DecodeCDMPEG4Vop -- Decodes the VOP information from the bitstream
 *
 ***********************************************************CommentEnd********/

APADLLIMPORTS
Void CallingConvention DecodeMPEG4CDVol(
	Int        width,
	Int        height, 
	Int        vols,
	Int        framerate, 
	Int        post_filter_type,
	Int        temporal_scalability,
	Int        random_access_start_time,
	Int        readen_bits[MAX_NUM_VOS][MAX_NUM_VOLS],
	Int        stop_decoding_vol[MAX_NUM_VOS][MAX_NUM_VOLS],
	Vop       *display_vop, 
	Float      time_between_two_frames,
	Float     *next_display_time,
	Bitstream *stream[MAX_NUM_VOS][MAX_NUM_VOLS],
    Vol       *display_vol_list[MAX_NUM_VOS],
    Trace     *trace[MAX_NUM_VOS][MAX_NUM_VOLS], 
	Char       trace_filename[MAX_NUM_VOS][MAX_NUM_VOLS][300],
	Vop	      *base_output_vop[3],
	Float      display_framerate_TPS,
	Int last_base_decoding_frame[3],
	Int *last_enh_decoding_frame,
	Int *TPS_base_count,
	Int    *prev_ref,
	Int *next_ref,
	UChar **MB_TYPE2[MAX_NUM_VOS][MAX_NUM_VOLS],
	Int PRE_H_SIZE2[MAX_NUM_VOS][MAX_NUM_VOLS],
	Int PRE_V_SIZE2[MAX_NUM_VOS][MAX_NUM_VOLS],
	UChar **PRE_MB_TYPE2[MAX_NUM_VOS][MAX_NUM_VOLS],
	Int bs_vol[MAX_NUM_VOS][MAX_NUM_VOLS],
	Image   *alpha_rec_packet,
	Image   *alpha_mode_packet,
	SInt Px_for,SInt Py_for, SInt Px_back, SInt Py_back,
	Int prev_j,Int zero_edge,Int prev_time_base,
	Int prev_predic_type,
	Vop	*back_shape_vop,
	Vop	*for_shape_vop,
	UChar **stock_mb_type2[3][MAX_NUM_VOS][MAX_NUM_VOLS],
	Int   stock_h_size2[3][MAX_NUM_VOS][MAX_NUM_VOLS],
	Int   stock_v_size2[3][MAX_NUM_VOS][MAX_NUM_VOLS]
	);

APADLLIMPORTS
Void CallingConvention FreeMPEG4CD(
		Vop		*display_vop,
		Int		 vols,
		Int		 temporal_scalability,
		Int		num_frames,
		Vol		*display_vol_list[MAX_NUM_VOS],
		Bitstream *stream[MAX_NUM_VOS][MAX_NUM_VOLS],
		UChar **PRE_MB_TYPE2[MAX_NUM_VOS][MAX_NUM_VOLS],
		Int PRE_V_SIZE2[MAX_NUM_VOS][MAX_NUM_VOLS],
		Int		num_cd_decoders,
		Vop		*back_shape_vop,
		Vop		*for_shape_vop,
		UChar **stock_mb_type2[3][MAX_NUM_VOS][MAX_NUM_VOLS],
		Int   stock_v_size2[3][MAX_NUM_VOS][MAX_NUM_VOLS]
		);

#ifdef __cplusplus
}
#endif /* __cplusplus */

#endif
