/*
 * $RCSfile: Ring.c,v $           $Author: mleditsc $
 * $Revision: 1.1 $          $Date: 2000/01/24 05:38:27 $ 
 * $Locker:  $      
 */

/******************************************************************************/
/*                                                                            */
/* Copyright (C) 1990,1991,1992,1993,1994,1995,1996,1997  FRANCE TELECOM/CNET */
/* All Rights Reserved.                                                       */
/*                                                                            */
/* CNET DSE/SGV  -   Fiche d'etude STC 95-17                                  */
/*                                                                            */
/******************************************************************************/


/***********************************************************HeaderBegin*******
 *
 * File :        ring.c
 *
 * Author :       <Loret@issy.cnet.fr>
 *
 * Created :    23-Dec-1997
 *
 * Description :
 *
 * Notes :
 *
 * Modified :
 *
 ***********************************************************HeaderEnd*********/

/*****************************************************************************/
/*         This software module has been speed optimized                     */
/*                                                                           */
/*         by  Bruno Loret - France Telecom / CNET                           */
/*                                                                           */
/*         in the framework of the ACTS Project MoMuSys                      */
/*****************************************************************************/

/***********/
/* HISTORY */
/***********/
/*
 * $Log: Ring.c,v $
 * Revision 1.1  2000/01/24 05:38:27  mleditsc
 * *** empty log message ***
 *
 * Revision 1.2  2000/01/19 03:09:17  mleditsc
 * Fixed up a problem with lines ending with CR CR LF
 *
 * Revision 1.1.1.1  2000/01/13 06:05:59  mleditsc
 * Import of 3D player 990924 2.55
 *
 *
 */

/******************************************************************************/

/*******************        INCLUDE FILES       *******************************/
#include "ring.h"

/****************************************************************************/
void
init_more_FLOAT (int size_FLOAT);
void
init_more_BIG (int size_BIG);
void
init_more_SMALL (int size_SMALL);
void
init_more_MEDIUM (int size_MEDIUM);
/****************************************************************************/
#define LOCKED   1
#define UNLOCKED 0

/****************************************************************************/
/* For rectangular
   
   --- INT ---
   
   11 *   9 :      99
   13 *  11 :     143
   22 *  18 :     396
   88 *  72 :    6336   8192 * 17 :   188 416
   
   104 *  88 :    9152
   176 * 144 :   25344
   208 * 176 :   36608  65536 * 19 : 1 179 648
   
   416 * 352 :  146432 262144 *  1 :   262 144
   
   1 630 208
   
   --- FLOAT ---
   22 *  18 :     396    512 * 2 :      1 024
   
   */
/* For akiyo binary shape
   
   --- INT ---
   
   11 *   9 :      99
   13 *  11 :     143
   22 *  18 :     396
   88 *  72 :    6336   8192 * 17 :   188 416
   
   104 *  88 :    9152
   176 * 144 :   25344
   208 * 176 :   36608  65536 * 19 : 1 179 648
   
   416 * 352 :  146432 262144 *  1 :   262 144
   
   1 630 208
   
   --- FLOAT ---
   22 *  18 :     396    512 * 2 :      1 024
   
   */

/***********************************/
/* Ring for akiyo_qcif rectangular */
/***********************************/

#   define RING_SMALL_MAX_SIZE  1000  /* orig 40 */
#   define RING_MEDIUM_MAX_SIZE 1000  /* orig 30 */
#   define RING_BIG_MAX_SIZE       4  /* orig 1 */
#   define RING_FLOAT_MAX_SIZE     8  /* orig 2 */

#   define RING_SMALL_MORE      10  
#   define RING_MEDIUM_MORE     10  
#   define RING_BIG_MORE         1  
#   define RING_FLOAT_MORE       2  

static int ring_small_size  =  0;
static int ring_medium_size =  0;
static int ring_big_size    =  0;
static int ring_float_size  =  0;

/* SHORT_TYPE ring -- used almost always (in the decoder at least) */

/* used for small "images "
   Macroblocks, MAcroblocks flags, etc
   */
static Image    *ring_SMALL[RING_SMALL_MAX_SIZE];
static int      ring_SMALL_status[RING_SMALL_MAX_SIZE];

/* used for real images */
static Image    *ring_MEDIUM[RING_MEDIUM_MAX_SIZE];
static int      ring_MEDIUM_status[RING_MEDIUM_MAX_SIZE];

/* used for the interpolated images */
static Image    *ring_BIG[RING_BIG_MAX_SIZE];
static int      ring_BIG_status[RING_BIG_MAX_SIZE];

/* FLOAT_TYPE ring -- used for the motion vectors */
static Image    *ring_float[RING_FLOAT_MAX_SIZE];
static int      ring_float_status[RING_FLOAT_MAX_SIZE];

static SInt     *mem_SMALL[RING_SMALL_MAX_SIZE];
static SInt     *mem_MEDIUM[RING_MEDIUM_MAX_SIZE];
static SInt     *mem_BIG[RING_BIG_MAX_SIZE];
static Float    *mem_float[RING_FLOAT_MAX_SIZE];


static Int  *mem_DC_store;

/*A.Pacheco:1/04/98:mutually exclusive access to the ring data*/
#if defined(_IM1_CD_DECODER_) && defined(WIN32)
#include <windows.h>
static HANDLE m_hMutexMemoryRing=NULL;

__inline void StartRingMutex()
{
	if(m_hMutexMemoryRing==NULL)
		m_hMutexMemoryRing = CreateMutex (NULL, FALSE, NULL);
}

__inline void BeginRingMutex()
{
        WaitForSingleObject (m_hMutexMemoryRing, INFINITE);
}

__inline void EndRingMutex()
{
	ReleaseMutex (m_hMutexMemoryRing);
}

#elif defined(PTHREADS)

#include <pthread.h>

static int initialised = 0;
static pthread_mutex_t mutexMemoryRing;

void StartRingMutex()
{
        if(!initialised) {
	        pthread_mutex_init(&mutexMemoryRing, NULL);
		initialised = 1;
	}
}

void BeginRingMutex()
{
	pthread_mutex_lock(&mutexMemoryRing);
}

void EndRingMutex()
{
	pthread_mutex_unlock(&mutexMemoryRing);
}

#else

#include <ulocks.h>

static ulock_t mutexMemoryRing;
static usptr_t* arenaMemoryRing = NULL;

void StartRingMutex()
{
        if(!arenaMemoryRing) {
                usconfig(CONF_ARENATYPE, US_SHAREDONLY);
                if (!(arenaMemoryRing = usinit("/dev/zero")))
                        perror("usinit");
                mutexMemoryRing = usnewlock(arenaMemoryRing);
        }
}

void BeginRingMutex()
{
        ussetlock(mutexMemoryRing);
}

void EndRingMutex()
{
        usunsetlock(mutexMemoryRing);
}
#endif


/****************************************************************************/

void
CallingConvention
init_ring(int qcif)
{
    int size_SMALL = 0, size_MEDIUM = 0, size_BIG = 0;
    int size_FLOAT = 0;

#ifdef _IM1_CD_DECODER_
	StartRingMutex();
	BeginRingMutex();
#endif
    
    
    if(qcif)
    {
        size_SMALL  =   8192;
        size_MEDIUM =  65536;
        size_BIG    = 262144;
        size_FLOAT  =    512;
    }
    else
    {
        /* For further studies */
    }
    
    init_more_SMALL (size_SMALL);
    init_more_MEDIUM(size_MEDIUM);
    init_more_BIG   (size_BIG);
    init_more_FLOAT (size_FLOAT);
    
    mem_DC_store = (Int *) calloc(65536, sizeof(Int));
    printf("Init_ring DC_store : Allocation of %12d bytes for DC store %d*%d \n\n",
           65536*sizeof(Int), 1, 65536);
    
#ifdef _IM1_CD_DECODER_
	EndRingMutex();
#endif
    
}

void
CallingConvention
free_ring()
{   
    int i;
    
#ifdef _IM1_CD_DECODER_
    BeginRingMutex();
#endif   
    
    if(ring_SMALL[0] == NULL)
    {
        for (i = 0; i < ring_small_size; i++)
        {
            free(mem_SMALL[i]);
            free(ring_SMALL[i]->data);
            free(ring_SMALL[i]);
            ring_SMALL_status[i] = UNLOCKED;                  
        }
    }
    
    if(ring_MEDIUM[0] == NULL)
    {
        for (i = 0; i < ring_medium_size; i++)
        {
            free(mem_MEDIUM[i]);
            free(ring_MEDIUM[i]->data);
            free(ring_MEDIUM[i]);
            ring_MEDIUM_status[i] = UNLOCKED;                  
        }
    }
    
    free(mem_DC_store);
    
    if(ring_BIG[0] == NULL)
    {
        for (i = 0; i < ring_big_size; i++)
        {
            free(mem_BIG[i]);
            free(ring_BIG[i]->data);
            free(ring_BIG[i]);
            ring_BIG_status[i] = UNLOCKED;                  
        }
    }
    
    if(ring_float[0] == NULL)
    {
        for (i = 0; i < ring_float_size; i++)
        {
            free(mem_float[i]);
            free(ring_float[i]->data);
            free(ring_float[i]);
            ring_float_status[i] = UNLOCKED;                  
        }
    }
    
    
#ifdef _IM1_CD_DECODER_
    EndRingMutex();
#endif
}

Image *
CallingConvention
get_picture(int size_x, int size_y, ImageType type, int qcif)
{
    int i;
    
    int xy;    
    
    int size_SMALL = 0, size_MEDIUM = 0, size_BIG = 0;
    int size_FLOAT;
    
#ifdef _IM1_CD_DECODER_
    BeginRingMutex();
#endif

    if(qcif)
    {
        size_SMALL  =   8192;
        size_MEDIUM =  65536;
        size_BIG    = 262144;
        size_FLOAT  =    512;
    }
    else
    {
        /* For further studies */
    }
    
    i = 0;
    if(type == SHORT_TYPE)
    {
        xy = size_x*size_y;
        
        if(xy < size_SMALL)
        {
            while(ring_SMALL_status[i] == LOCKED)
            {
                if(++i >= ring_small_size)
                {
                    init_more_SMALL(size_SMALL);
                }
            }
            
            ring_SMALL_status[i] = LOCKED;
            
            ring_SMALL[i]->version = qcif;
            ring_SMALL[i]->x = size_x;
            ring_SMALL[i]->y = size_y;
            
   
#ifdef _IM1_CD_DECODER_
	    EndRingMutex();
#endif
            return( ring_SMALL[i] );
        }
        else if(xy < size_MEDIUM)
        {
            while(ring_MEDIUM_status[i] == LOCKED)
            {
                if(++i >= ring_medium_size)
                {
                    init_more_MEDIUM(size_MEDIUM);
                }
            }
            
            ring_MEDIUM_status[i] = LOCKED;
            
            ring_MEDIUM[i]->version = qcif;
            ring_MEDIUM[i]->x = size_x;
            ring_MEDIUM[i]->y = size_y;        
            

#ifdef _IM1_CD_DECODER_
	    EndRingMutex();
#endif
            return( ring_MEDIUM[i] );
        }
        else if(xy < size_BIG)
        {
            while(ring_BIG_status[i] == LOCKED)
            {
                if(++i >= ring_big_size)
                {
                    init_more_BIG(size_BIG);
                }
            }
            
            ring_BIG_status[i] = LOCKED;
            
            ring_BIG[i]->version = qcif;
            ring_BIG[i]->x = size_x;
            ring_BIG[i]->y = size_y;        
            

#ifdef _IM1_CD_DECODER_
	    EndRingMutex();
#endif
            return( ring_BIG[i] );
        }        
        
        else
        {
            printf("Error in get_picture : memories size too small ! \n");
            printf("Check vm_common/src/ring.c for more informations \n");
#ifdef _IM1_CD_DECODER_
	    EndRingMutex();
#endif
            exit(0);
        }
    }
    else if(type == FLOAT_TYPE)
    {
        xy = size_x*size_y;
        
        while(ring_float_status[i] == LOCKED)
        {
            if(++i >= ring_float_size)
            {
                init_more_FLOAT(size_FLOAT);
            }
        }
        
        ring_float_status[i] = LOCKED;
        
        ring_float[i]->version = qcif;
        ring_float[i]->x = size_x;
        ring_float[i]->y = size_y;
        

#ifdef _IM1_CD_DECODER_
	EndRingMutex();
#endif
        return( ring_float[i] );
    }
    else
    {
        printf("Image type not supported in OPTIMIZE mode \n");
#ifdef _IM1_CD_DECODER_
	EndRingMutex();
#endif
        exit(999);
    }
    
    
}


Int *
CallingConvention
Get_DC_store()
{
    return(mem_DC_store);
}


void
CallingConvention
release_picture(int i, int x, int y, ImageType type, int qcif)
{
    int xy;
    
    int size_SMALL = 0, size_MEDIUM = 0, size_BIG = 0;
    int size_FLOAT;
    
#ifdef _IM1_CD_DECODER_
    BeginRingMutex();
#endif
    
    if(qcif)
    {
        size_SMALL  =   8192;
        size_MEDIUM =  65536;
        size_BIG    = 262144;
        size_FLOAT  =    512;
    }
    else
    {
        /* For further studies */
    }
    
    if(type == SHORT_TYPE)
    {
        xy = x*y;
        
        if(xy < size_SMALL)
            ring_SMALL_status[i] = UNLOCKED;
        else if(xy < size_MEDIUM)
            ring_MEDIUM_status[i] = UNLOCKED;
        else if(xy < size_BIG)
            ring_BIG_status[i] = UNLOCKED;
        else
        {
            printf("Error in release_picture : memories size too small ! \n");
            printf("Check vm_common/src/ring.c for more informations \n");
#ifdef _IM1_CD_DECODER_
	    EndRingMutex();
#endif
            exit(0);
        }
    }
    else if(type == FLOAT_TYPE)
        ring_float_status[i] = UNLOCKED;
    else
    {
        printf("Image type not supported in OPTIMIZE mode \n");
#ifdef _IM1_CD_DECODER_
	EndRingMutex();
#endif
        exit(999);
    }
#ifdef _IM1_CD_DECODER_
    EndRingMutex();
#endif
}

/****************************************************************************/
void
init_more_SMALL (int size_SMALL)
{
    int i;

    int a, b;

    a = ring_small_size;
    b = a + RING_SMALL_MORE;
    
    if(b >= RING_SMALL_MAX_SIZE)
    {
        printf("Error in init_more_SMALL : ring to small !! \n");
#ifdef _IM1_CD_DECODER_
        EndRingMutex();
#endif
                exit(0);
    }

    ring_small_size = b;

    for (i = a; i < b; i++)
        mem_SMALL[i] = (SInt *) calloc(size_SMALL, sizeof(SInt));
    
    printf("Init_ring SMALL    : Allocation of %12d bytes for Ring of size %d*%d \n\n",
           RING_SMALL_MORE*size_SMALL*sizeof(SInt), RING_SMALL_MORE,size_SMALL);

    
    for (i = a; i < b; i++)
    {
        ring_SMALL[i] = (Image *) emalloc(sizeof(Image));

        ring_SMALL_status[i]      = UNLOCKED;
            
        ring_SMALL[i]->version    = 1;           /* indicate QCIF(1) or CIF(0) */
        ring_SMALL[i]->x          = 1;              /* Modified in get_picture */
        ring_SMALL[i]->y          = 1;
        ring_SMALL[i]->upperodd   = i;              /* index of the picture in the ring */
        ring_SMALL[i]->grid       = 's';
        ring_SMALL[i]->type       = SHORT_TYPE;
        ring_SMALL[i]->data       = (ImageData *) emalloc(sizeof(ImageData)); 
            
        ring_SMALL[i]->data->s    = mem_SMALL[i];
            
        ring_SMALL[i]->f          = ring_SMALL[i]->data->s; /* For compatibility with KHOROS */
    }
    

    return;
}

/****************************************************************************/
void
init_more_MEDIUM (int size_MEDIUM)
{
    int i;

    int a, b;

    a = ring_medium_size;
    b = a + RING_MEDIUM_MORE;
    
    if(b >= RING_MEDIUM_MAX_SIZE)
    {
        printf("Error in init_more_MEDIUM : ring to small !! \n");
#ifdef _IM1_CD_DECODER_
        EndRingMutex();
#endif
                exit(0);
    }

    ring_medium_size = b;

    for (i = a; i < b; i++)
        mem_MEDIUM[i] = (SInt *) calloc(size_MEDIUM, sizeof(SInt));
    
    printf("Init_ring MEDIUM   : Allocation of %12d bytes for Ring of size %d*%d \n\n",
           RING_MEDIUM_MORE*size_MEDIUM*sizeof(SInt), RING_MEDIUM_MORE,size_MEDIUM);

    
    for (i = a; i < b; i++)
    {
        ring_MEDIUM[i] = (Image *) emalloc(sizeof(Image));
        
        ring_MEDIUM_status[i]      = UNLOCKED;
            
        ring_MEDIUM[i]->version    = 1;           /* indicate QCIF(1) or CIF(0) */
        ring_MEDIUM[i]->x          = 1;              /* Modified in get_picture */
        ring_MEDIUM[i]->y          = 1;
        ring_MEDIUM[i]->upperodd   = i;              /* index of the picture in the ring */
        ring_MEDIUM[i]->grid       = 's';
        ring_MEDIUM[i]->type       = SHORT_TYPE;
        ring_MEDIUM[i]->data       = (ImageData *) emalloc(sizeof(ImageData)); 
            
        ring_MEDIUM[i]->data->s    = mem_MEDIUM[i];
            
        ring_MEDIUM[i]->f          = ring_MEDIUM[i]->data->s; /* For compatibility with KHOROS */
    }
    

    return;
}
    
/****************************************************************************/
void
init_more_BIG (int size_BIG)
{
    int i;

    int a, b;

    a = ring_big_size;
    b = a + RING_BIG_MORE;
    
    if(b >= RING_BIG_MAX_SIZE)
    {
        printf("Error in init_more_BIG : ring to small !! \n");
#ifdef _IM1_CD_DECODER_
        EndRingMutex();
#endif
                exit(0);
    }

    ring_big_size = b;

    for (i = a; i < b; i++)
        mem_BIG[i] = (SInt *) calloc(size_BIG, sizeof(SInt));
    
    printf("Init_ring BIG      : Allocation of %12d bytes for Ring of size %d*%d \n\n",
           RING_BIG_MORE*size_BIG*sizeof(SInt), RING_BIG_MORE,size_BIG);

    
    for (i = a; i < b; i++)
    {
        ring_BIG[i] = (Image *) emalloc(sizeof(Image));
        
        ring_BIG_status[i]      = UNLOCKED;
            
        ring_BIG[i]->version    = 1;           /* indicate QCIF(1) or CIF(0) */
        ring_BIG[i]->x          = 1;              /* Modified in get_picture */
        ring_BIG[i]->y          = 1;
        ring_BIG[i]->upperodd   = i;              /* index of the picture in the ring */
        ring_BIG[i]->grid       = 's';
        ring_BIG[i]->type       = SHORT_TYPE;
        ring_BIG[i]->data       = (ImageData *) emalloc(sizeof(ImageData)); 
            
        ring_BIG[i]->data->s    = mem_BIG[i];
            
        ring_BIG[i]->f          = ring_BIG[i]->data->s; /* For compatibility with KHOROS */
    }
    

    return;
}

/****************************************************************************/
void
init_more_FLOAT (int size_FLOAT)
{
    int i;

    int a, b;

    a = ring_float_size;
    b = a + RING_FLOAT_MORE;
    
    if(b >= RING_FLOAT_MAX_SIZE)
    {
        printf("Error in init_more_float : ring to small !! \n");
#ifdef _IM1_CD_DECODER_
        EndRingMutex();
#endif
                exit(0);
    }

    ring_float_size = b;

    for (i = a; i < b; i++)
        mem_float[i] = (Float *) calloc(size_FLOAT, sizeof(Float));
    
    printf("Init_ring float    : Allocation of %12d bytes for Ring of size %d*%d \n\n",
           RING_FLOAT_MORE*size_FLOAT*sizeof(SInt), RING_FLOAT_MORE,size_FLOAT);

    
    for (i = a; i < b; i++)
    {
        ring_float[i] = (Image *) emalloc(sizeof(Image));
        
        ring_float_status[i]      = UNLOCKED;
            
        ring_float[i]->version    = 1;           /* indicate QCIF(1) or CIF(0) */
        ring_float[i]->x          = 1;              /* Modified in get_picture */
        ring_float[i]->y          = 1;
        ring_float[i]->upperodd   = i;              /* index of the picture in the ring */
        ring_float[i]->grid       = 's';
        ring_float[i]->type       = FLOAT_TYPE;
        ring_float[i]->data       = (ImageData *) emalloc(sizeof(ImageData)); 
            
        ring_float[i]->data->s    = (SInt *) mem_float[i];
            
        ring_float[i]->f          = (SInt *) ring_float[i]->data->s; /* For compatibility with KHOROS */
    }
    

    return;
}
