/*****************************************************************************
  "This software module was originally developed by:
  
  Noboru Yamaguchi (TOSHIBA CORPORATION), 
  Takashi Ida (TOSHIBA CORPORATION) 
  
  and edited by:
  
  Toshiaki Watanabe (TOSHIBA CORPORATION), 
  Yoshihiro Kikuchi (TOSHIBA CORPORATION), 
  Noel Brady (TELTEC IRELAND)
  
  in the course of development of the <MPEG-4 Video(ISO/IEC 14496-2)>. This
  software module is an implementation of a part of one or more <MPEG-4 Video
  (ISO/IEC 14496-2)> tools as specified by the <MPEG-4 Video(ISO/IEC 14496-2)
  >. ISO/IEC gives users of the <MPEG-4 Video(ISO/IEC 14496-2)> free license
  to this software module or modifications thereof for use in hardware or
  software products claiming conformance to the <MPEG-4 Video(ISO/IEC 14496-2
  )>. Those intending to use this software module in hardware or software
  products are advised that its use may infringe existing patents. The
  original developer of this software module and his/her company, the
  subsequent editors and their companies, and ISO/IEC have no liability for
  use of this software module or modifications thereof in an implementation.
  Copyright is not released for non <MPEG-4 Video(ISO/IEC 14496-2)>
  conforming products. TOSHIBA CORPORATION retains full right to use the code
  for his/her own purpose, assign or donate the code to a third party and to
  inhibit third parties from using the code for non <MPEG-4 Video(ISO/IEC
  14496-2)> conforming products. This copyright notice must be included in
  all copies or derivative works. Copyright (c)1996".
  *****************************************************************************/

/*****************************************************************************/
/*         This software module has been speed optimized                     */
/*                                                                           */
/*         by  Bruno Loret - France Telecom / CNET                           */
/*                                                                           */
/*         in the framework of the ACTS Project MoMuSys                      */
/*****************************************************************************/

/***********************************************************HeaderBegin*******
 *                                                                         
 * File:        alp_dec_header.c
 *
 * Author:      TOSHIBA + Noel Brady Teltec Irl.
 * Created:     11-04-97
 *                                                                         
 * Description: 
 *                              Functions for decoding first_shape_code in I/P/B VOps
 *
 *                                 
 ***********************************************************HeaderEnd*********/


#include <stdio.h>
#ifndef WIN32
#include <values.h>
#endif
#include <string.h>
#include "momusys.h"
#include "vm_dec_defs.h"
#include "mom_bitstream_d.h"
#include "alp_common_def.h"
#include "alp_dec_util.h"
#include "alp_mode_tables.h"
#include "alp_dec_header.h"

/*A.Pacheco: 1/04/98: global variables are nor allowed*/
#if 0
/*ok*/UChar **PRE_MB_TYPE2[MAX_NUM_VOS][MAX_NUM_VOLS]/*[22][18]*/;
/*ok*/UChar **MB_TYPE2[MAX_NUM_VOS][MAX_NUM_VOLS]/*[22][18]*/;
/*ok*/Int PRE_H_SIZE2[MAX_NUM_VOS][MAX_NUM_VOLS];
/*ok*/Int PRE_V_SIZE2[MAX_NUM_VOS][MAX_NUM_VOLS];
#endif

/***********************************************************CommentBegin******
 *
 * -- DecodeIntraMbtype -- decodes first_shape_code for a BAB in I-VOP
 *
 * Author :             
 *      Noel Brady Teltec Irl.
 *
 * Created :            
 *      11-04-97
 * 
 * Arguments In:        
 *                                      stream - the bitstream
 *                                      i,j - the BAB coordinate
 *                                      width - number of BABs per row of VOP
 *                                      vo_id - 
 *                                      vol_id - 
 *
 * Arguments In/Out:
 *
 * Arguments Out:
 *                                      mb_type - the decoded BAB mode
 *
 * Return values :
 *                                      number of bits in first_shape_code
 *
 * Side effects :       
 *
 * Description : As per VM7
 *
 * See also :
 *
 ***********************************************************CommentEnd********/

Int
CallingConvention
DecodeIntraMBtype(Bitstream *stream, Int *mb_type, Int i,
                  Int j, Int width, Int vo_id, Int vol_id, Trace *trace,
				  UChar **MB_TYPE2[MAX_NUM_VOS][MAX_NUM_VOLS])
{
    Int context;
    Int ret=0, k;
    Int a=3,b=3,c=3,d=3;
    
#ifndef _OPTIMIZE_
    Int last_col = width - 1;
#else
    const Int last_col = width - 1;
#endif
    
    Int i1,j1,len;
    
    if (i>0)
    {
        if (j>0)
            a = MB_TYPE2[vo_id][vol_id][i-1][j-1]+1;
        b = MB_TYPE2[vo_id][vol_id][i-1][j]+1;
        
        if (j<last_col)
            c = MB_TYPE2[vo_id][vol_id][i-1][j+1]+1;
    }
    
    if (j>0)
        d = MB_TYPE2[vo_id][vol_id][i][j-1]+1;
    
    context = 27*(a-3) +        9*(b-3) +       3*(c-3) + d-3;
    
    k=0;
    do 
    {
        len = strlen(IMB_TYPE[context][k]);
        i1 = BitstreamShowBits(stream, len);
        j1 = ConvertStoI(IMB_TYPE[context][k],len);
        
        if (i1==j1) 
        {  
            ret = len;
            switch (k)
            {
                case 0: *mb_type = 1;
                    break;
                case 1: *mb_type = 0;
                    break;
                case 2: *mb_type = 2;
                    break;
                default: exit(-1);
            }
        }
        k++;
    }
    while (k<3);
    
    if (ret == 0) 
    {
        printf("Illegal VLC for INTRA shape mode\n");
        exit(-1);
    }
    else 
    {
        BitstreamReadBits(stream,ret,"first_shape_code",trace,CODE);
    }
    
#ifdef _DEBUG
    if (trace->trace) 
        switch (*mb_type)
        {
            case 0: fprintf(trace->fp_trace, "bab_type = all_opaque\n");
                break;
            case 1: fprintf(trace->fp_trace, "bab_type = all_transparent\n");
                break;
            case 2: fprintf(trace->fp_trace, "bab_type = intra_coded\n");
                break;
        }
#endif
    
    return ret;
}


#if 0
Int
CallingConvention
DecodeIntraMBtype_ER(Bitstream *stream, Int *mb_type, Int i,
                     Int j, Int width, Int vo_id, Int vol_id, Image *alpha_mode_packet,
                     Trace *trace)
{
    Int context;
    Int ret=0, k;
    Int a=3,b=3,c=3,d=3;
    Int i1,j1,len;
    
#ifndef _OPTIMIZE_
    Int last_col = width - 1;
    Int w = GetImageSizeX(alpha_mode_packet);
#else
    const Int last_col = width - 1;
    const Int w = GetImageSizeX(alpha_mode_packet);
#endif
    
    SInt *p = (SInt *)GetImageData(alpha_mode_packet);
    
    if (i>0)
    {
        if (j>0 && p[(i-1)*w+j-1])
            a = MB_TYPE2[vo_id][vol_id][i-1][j-1]+1;
        if (p[(i-1)*w+j])
            b = MB_TYPE2[vo_id][vol_id][i-1][j]+1;
        if (j<last_col && p[(i-1)*w+j+1])
            c = MB_TYPE2[vo_id][vol_id][i-1][j+1]+1;
    }
    if (j>0 && p[i*w+j-1])
        d = MB_TYPE2[vo_id][vol_id][i][j-1]+1;
    
    p[i*w+j] = 1;
    
    context = 27*(a-3) +        9*(b-3) +       3*(c-3) + d-3;
    
    k=0;
    do 
    {
        len = strlen(IMB_TYPE[context][k]);
        i1 = BitstreamShowBits(stream, len);
        j1 = ConvertStoI(IMB_TYPE[context][k],len);
        
        if (i1==j1) 
        {  
            ret = len;
            switch (k)
            {
                case 0: *mb_type = 1;
                    break;
                case 1: *mb_type = 0;
                    break;
                case 2: *mb_type = 2;
                    break;
                default: exit(-1);
            }
        }
        k++;
    }
    while (k<3);
    
    if (ret == 0) 
    {
        printf("Illegal VLC for INTRA shape mode\n");
        exit(-1);
    }
    else 
    {
        BitstreamReadBits(stream,ret,"first_shape_code",trace,CODE);
    }
    
#ifdef _DEBUG
    if (trace->trace) 
        switch (*mb_type)
        {
            case 0: fprintf(trace->fp_trace, "bab_type = all_opaque\n");
                break;
            case 1: fprintf(trace->fp_trace, "bab_type = all_transparent\n");
                break;
            case 2: fprintf(trace->fp_trace, "bab_type = intra_coded\n");
                break;
        }
#endif
    
    return ret;
}
#endif

/***********************************************************CommentBegin******
 *
 * -- DecodeInterMbtype -- decodes first_shape_code for a BAB in P/B-VOP
 *
 * Author :             
 *      Noel Brady Teltec Irl.
 *
 * Created :            
 *      11-04-97
 * 
 * Arguments In:        
 *                                      stream - the bitstream
 *                                      i,j - the BAB coordinate
 *                                      vo_id - 
 *                                      vol_id - 
 *
 * Arguments In/Out:
 *
 * Arguments Out:
 *                                      mb_type - the decoded BAB mode
 *                                      mvda - flag indicating whether the MVDs == 0 or not
 *
 * Return values :
 *                                      number of bits in first_shape_code
 *
 * Side effects :       
 *
 * Description : As per VM7
 *
 * See also :
 *
 ***********************************************************CommentEnd********/
Int
CallingConvention
DecodeInterMBtype(
    Int *mb_type,
    Int *mvda,
    Int i,
    Int j,
    Bitstream *stream,
    Int vo_id, Int vol_id,
    Trace *trace,
	UChar **MB_TYPE2[MAX_NUM_VOS][MAX_NUM_VOLS],
	Int PRE_H_SIZE2[MAX_NUM_VOS][MAX_NUM_VOLS],
	Int PRE_V_SIZE2[MAX_NUM_VOS][MAX_NUM_VOLS],
	UChar **PRE_MB_TYPE2[MAX_NUM_VOS][MAX_NUM_VOLS])
{
    Int
        prei,
        prej,
        bits=0;
    Int
        prev_mode,
        decoded,
        k;
    
    Char **vlc_table;
    
    Int i1,j1,len;
    
    if(i<PRE_V_SIZE2[vo_id][vol_id])
        prei=i; /* PRE_V_SIZE: v_size in previous frame */
    else
        prei=PRE_V_SIZE2[vo_id][vol_id]-1;
    
    if(j<PRE_H_SIZE2[vo_id][vol_id])
        prej=j; /* PRE_H_SIZE: h_size in previous frame */
    else
        prej=PRE_H_SIZE2[vo_id][vol_id]-1;
    
    prev_mode = PRE_MB_TYPE2[vo_id][vol_id][prei][prej];
    
    switch (prev_mode)
    {
        case 0: vlc_table = MMB_TYPE0;  
            break;
        case 1: vlc_table = MMB_TYPE1;  
            break;
        case 2: vlc_table = MMB_TYPE2;  
            break;
        case 3: vlc_table = MMB_TYPE3;  
            break;
        case 4: vlc_table = MMB_TYPE4;  
            break;
        case 5: vlc_table = MMB_TYPE5;  
            break;
        case 6: vlc_table = MMB_TYPE6;  
            break;
        default: printf("Error in PRE_MB_TYPE\n");
            exit(-1);
    }
    
    decoded = 0; k = 0; *mvda = 1;
    do 
    {
        len = strlen(vlc_table[k]);
        i1 = BitstreamShowBits(stream, len);
        j1 = ConvertStoI(vlc_table[k],len);
        
        if(i1==j1)
        {
            MB_TYPE2[vo_id][vol_id][i][j] = k;
            switch (k)
            {
                case 0: *mb_type = -1;  /* not coded && mvds == 0 */
                    *mvda = 1;
                    break;
                case 1: *mb_type = -1;  /* not coded && mvds !=0 */
                    *mvda = 0;
                    break;
                case 2: *mb_type = 1;   /* all_0 */
                    break;
                case 3: *mb_type = 0;           /* all_255*/
                    break;
                case 4: *mb_type = 2;           /* INTRA coded */
                    break;
                case 5: *mb_type = 6;  /* INTER coded && mvds==0 */
                    *mvda = 1;
                    break;
                case 6: *mb_type = 5;   /* INTER coded && mvds!=0 */
                    *mvda = 0;
                    break;                                                      
                default:        printf("Error decoding first_shape_code of VOP %d\n", vo_id);
                    exit(-1);
            }
            decoded = 1;
            bits = len;
        }
        k++;
    } while     (k<7 && !decoded);
    
    if (!decoded) 
    {
        printf("Error decoding first_shape_code of VOP %d\n", vo_id);
#ifdef _DEBUG
        printf("in image %d \n", trace->counter);
#endif
        exit(-1);
    }
    else 
    {
        BitstreamReadBits(stream,bits,"first_shape_code",trace,CODE);
    }
    
#ifdef _DEBUG
    if (trace->trace) 
    {   
        switch (*mb_type)
        {
            case 0: fprintf(trace->fp_trace, "bab_type = all_opaque\n");
                break;
            case 1: fprintf(trace->fp_trace, "bab_type = all_transparent\n");
                break;
            case 2: fprintf(trace->fp_trace, "bab_type = intra_coded\n");
                break;
            case -1:    if (*mvda)
                fprintf(trace->fp_trace, "bab_type = inter_not_coded (mvds==0)\n");
            else 
                fprintf(trace->fp_trace, "bab_type = inter_not_coded (mvds!=0)\n");
            
            break;
            case 6: fprintf(trace->fp_trace, "bab_type = inter_coded (mvds==0)\n");
                break;
            case 5: fprintf(trace->fp_trace, "bab_type = inter_coded (mvds!=0)\n");
                break;
        }
    }
#endif
    
    return bits;
}


Void
CallingConvention
MakeMBtype2(
    Int mb_type,
    Int i,
    Int j,
    Int vo_id,
    Int vol_id,
	UChar **MB_TYPE2[MAX_NUM_VOS][MAX_NUM_VOLS])
{
    /*
      MB_TYPE: for I-VOP
      1: MMR
      2: all0
      3: all255
      */
    
    if(mb_type==0)
    {
        /* all255 */
        MB_TYPE2[vo_id][vol_id][i][j]=3;
    }
    else if(mb_type==1)
    {
        /* all0  */
        MB_TYPE2[vo_id][vol_id][i][j]=2;
    }
    else if(mb_type>=2 && mb_type<=6)
    {
        /* MMR */
        MB_TYPE2[vo_id][vol_id][i][j]=4;
    }
    else{
        printf("wrong mb_type=%d: i j=%d %d\n",mb_type,i,j);
        exit(0);
    }
    
}

Void
CallingConvention
AllocMBtype2(Int width, Int height, Int vo_id, Int vol_id,
			 UChar **MB_TYPE2[MAX_NUM_VOS][MAX_NUM_VOLS])
{
    Int i;
    
#ifndef _OPTIMIZE_
    Int w = width/MB_SIZE,              
        h = height/MB_SIZE;
#else
    const Int   w = width/MB_SIZE;
    const Int   h = height/MB_SIZE;
#endif
    
    MB_TYPE2[vo_id][vol_id] = (UChar **) malloc(h*sizeof(UChar*));
    
    for (i=0;i<h;i++)
        MB_TYPE2[vo_id][vol_id][i] = (UChar *) malloc(w*sizeof(UChar));
    
/*** 10/31 TEST */
/*  printf("mb_type2 alloc %2d %10d\n",h,MB_TYPE2[vo_id][vol_id][0]); */
/* 10/31 TEST ***/
}

Void
CallingConvention
AllocPRE_MBtype2(Int width, Int height, Int vo_id, Int vol_id,
				 UChar **PRE_MB_TYPE2[MAX_NUM_VOS][MAX_NUM_VOLS])
{
    Int i;
    
#ifndef _OPTIMIZE_
    Int w = width/MB_SIZE,              
        h = height/MB_SIZE;
#else
    const Int   w = width/MB_SIZE;
    const Int   h = height/MB_SIZE;
#endif
    
    PRE_MB_TYPE2[vo_id][vol_id] = (UChar **) malloc(h*sizeof(UChar*));
    
    for (i=0;i<h;i++)
        PRE_MB_TYPE2[vo_id][vol_id][i] = (UChar *) malloc(w*sizeof(UChar));
    
/*** 10/31 TEST */
/*  printf("pre_mb_type2 alloc %2d %10d\n",h,PRE_MB_TYPE2[vo_id][vol_id][0]);*/
/* 10/31 TEST ***/
}

Void
CallingConvention
FreeMBtype2(Int height, Int vo_id, Int vol_id,
			UChar **MB_TYPE2[MAX_NUM_VOS][MAX_NUM_VOLS])
{
#ifndef _OPTIMIZE_
    Int h = height/MB_SIZE;
#else
    const Int h = height/MB_SIZE;
#endif
    
    Int i;
    
/*** 10/31 TEST */
/*  printf("mb_type2 free %2d %10d\n",h,MB_TYPE2[vo_id][vol_id][0]); */
/* 10/31 TEST ***/
    for (i=0;i<h;i++)
        free(MB_TYPE2[vo_id][vol_id][i]);
    
    free(MB_TYPE2[vo_id][vol_id]);
    MB_TYPE2[vo_id][vol_id] = NULL;
}

Void
CallingConvention
FreePRE_MBtype2(Int height, Int vo_id, Int vol_id,
				UChar **PRE_MB_TYPE2[MAX_NUM_VOS][MAX_NUM_VOLS])
{
#ifndef _OPTIMIZE_
    Int h = height/MB_SIZE;
#else
    const Int h = height/MB_SIZE;
#endif
    
    Int i;
/*** 10/31 TEST */
/*  printf("pre_mb_type2 free %2d %10d\n",h,PRE_MB_TYPE2[vo_id][vol_id][0]);*/
/* 10/31 TEST ***/
    for (i=0;i<h;i++)
        free(PRE_MB_TYPE2[vo_id][vol_id][i]);
    
    free(PRE_MB_TYPE2[vo_id][vol_id]);
    PRE_MB_TYPE2[vo_id][vol_id] = NULL;
}


Void
CallingConvention
InitAllPRE_MBtype2(UChar **PRE_MB_TYPE2[MAX_NUM_VOS][MAX_NUM_VOLS])
{
    Int i,j;
    for (i=0;i<MAX_NUM_VOS;i++)
        for (j=0;j<MAX_NUM_VOLS;j++)
            PRE_MB_TYPE2[i][j] = NULL;
}

Void
CallingConvention
FreeAllPRE_MBtype2(Int PRE_V_SIZE2[MAX_NUM_VOS][MAX_NUM_VOLS],
				   UChar **PRE_MB_TYPE2[MAX_NUM_VOS][MAX_NUM_VOLS])
{
    Int i,j;
    for (i=0;i<MAX_NUM_VOS;i++)
        for (j=0;j<MAX_NUM_VOLS;j++)
            if (PRE_MB_TYPE2[i][j] != NULL) 
                FreePRE_MBtype2(PRE_V_SIZE2[i][j]*MB_SIZE,i,j,PRE_MB_TYPE2);
    
}

Void
CallingConvention
ChangeMBtype2(
    Int h_size,
    Int v_size,
    Int vo_id,
    Int vol_id,
	Int PRE_H_SIZE2[MAX_NUM_VOS][MAX_NUM_VOLS],
	Int PRE_V_SIZE2[MAX_NUM_VOS][MAX_NUM_VOLS],
	UChar **PRE_MB_TYPE2[MAX_NUM_VOS][MAX_NUM_VOLS],
	UChar **MB_TYPE2[MAX_NUM_VOS][MAX_NUM_VOLS]
	)
{
    Int i,j;
    
#ifndef _OPTIMIZE_
    Int h,v,;
    h = h_size/MB_SIZE;
    v = v_size/MB_SIZE;
#else    
    const Int h = h_size/MB_SIZE;
    const Int v = v_size/MB_SIZE;
#endif
    
#ifdef MB_TYPE_DEBUG
    printf("MB_TYPE2[vo_id][vol_id]\tPRE_MB_TYPE2[vo_id][vol_id]\n");
    for(i=0;i<v;i++){
        for(j=0;j<h;j++) printf("%d",MB_TYPE2[vo_id][vol_id][i][j]);
        printf("\t");
        if(i<PRE_V_SIZE2[vo_id][vol_id])
            for(j=0;j<PRE_H_SIZE2[vo_id][vol_id];j++) printf("%d",PRE_MB_TYPE2[vo_id][vol_id][i][j]);
        printf("\n");
    }
    printf("\n");
#endif
    
    if (PRE_MB_TYPE2[vo_id][vol_id] != NULL)    
        FreePRE_MBtype2(PRE_V_SIZE2[vo_id][vol_id]*MB_SIZE,vo_id,vol_id,PRE_MB_TYPE2);
    AllocPRE_MBtype2(h_size,v_size,vo_id,vol_id,PRE_MB_TYPE2);
    
    PRE_H_SIZE2[vo_id][vol_id]=h;
    PRE_V_SIZE2[vo_id][vol_id]=v;
    
    
    for(i=0;i<v;i++)
        for(j=0;j<h;j++)
        {
            PRE_MB_TYPE2[vo_id][vol_id][i][j]=MB_TYPE2[vo_id][vol_id][i][j];
        }
    
    FreeMBtype2(v_size,vo_id,vol_id,MB_TYPE2);
}







