/*****************************************************************************
  "This software module was originally developed by:
  
  Noboru Yamaguchi (TOSHIBA CORPORATION), 
  Takashi Ida (TOSHIBA CORPORATION) 
  
  and edited by:
  
  Toshiaki Watanabe (TOSHIBA CORPORATION), 
  Yoshihiro Kikuchi (TOSHIBA CORPORATION), 
  Noel Brady (TELTEC IRELAND)
  
  in the course of development of the <MPEG-4 Video(ISO/IEC 14496-2)>. This
  software module is an implementation of a part of one or more <MPEG-4 Video
  (ISO/IEC 14496-2)> tools as specified by the <MPEG-4 Video(ISO/IEC 14496-2)
  >. ISO/IEC gives users of the <MPEG-4 Video(ISO/IEC 14496-2)> free license
  to this software module or modifications thereof for use in hardware or
  software products claiming conformance to the <MPEG-4 Video(ISO/IEC 14496-2
  )>. Those intending to use this software module in hardware or software
  products are advised that its use may infringe existing patents. The
  original developer of this software module and his/her company, the
  subsequent editors and their companies, and ISO/IEC have no liability for
  use of this software module or modifications thereof in an implementation.
  Copyright is not released for non <MPEG-4 Video(ISO/IEC 14496-2)>
  conforming products. TOSHIBA CORPORATION retains full right to use the code
  for his/her own purpose, assign or donate the code to a third party and to
  inhibit third parties from using the code for non <MPEG-4 Video(ISO/IEC
  14496-2)> conforming products. This copyright notice must be included in
  all copies or derivative works. Copyright (c)1996".
  *****************************************************************************/

/*****************************************************************************/
/*         This software module has been speed optimized                     */
/*                                                                           */
/*         by  Bruno Loret - France Telecom / CNET                           */
/*                                                                           */
/*         in the framework of the ACTS Project MoMuSys                      */
/*****************************************************************************/

/***********************************************************HeaderBegin*******
 *                                                                         
 * File:        alp_dec_mom.c
 *
 * Author:      TOSHIBA + Noel Brady Teltec Irl.
 * Created:     11-04-97
 *                                                                         
 * Description: Contains high level functions for binary shape decoding.
 *
 *
 *                                 
 ***********************************************************HeaderEnd*********/

/********* header ********/

#include<stdio.h>
#ifndef WIN32
#include<values.h>
#endif


#include <string.h>
#include "momusys.h"
#include "mom_structs.h"
#include "mom_bitstream_d.h"
#include "vm_dec_defs.h"
#include "alp_common_def.h"
#include "alp_dec_mom.h"
#include "alp_dec_header.h"
#include "alp_dec_cae.h"
#include "alp_dec_mc.h"
#include "alp_common_cae.h"


/*A.Pacheco:1/04/98: static variables are not allowed in MTS code*/
/*
ok static Image   *alpha_rec_packet = NULL;
ok static Image   *alpha_mode_packet=NULL;
*/

/***********************************************************CommentBegin******
 *
 * -- DecodeFirst -- decodes first_shape_codes for single BAB
 *
 * Author :             TOSHIBA/Noel Brady (TELTEC IRELAND)
 *
 * Created :            
 *      11-04-97
 * 
 * Arguments In:        
 *                                      stream - the bitstream
 *                                      vo_id - 
 *                                      vol_id - 
 *                                      adr_i,adr_j - address of BAB in BAB units
 *                                      vop_type - 0=I_VOP, 1=P-VOP/B_VOP
 *
 * Arguments In/Out:
 *
 * Arguments Out:
 *                                      mb_type - the decoded BAB modes
 *                                      mvda - the decoded flags indicting whether MVDs=0 or not.
 *
 * Return values :
 *
 * Side effects :       
 *
 * Description : As per VM7
 *
 * See also :
 *
 *
 *
 ***********************************************************CommentEnd********/
Int
CallingConvention
DecodeFirst(Bitstream *stream,
            Int vo_id,
            Int vol_id,
            Int adr_i,
            Int adr_j,
            Int *mb_type,
            Int *mvda,
            Int vop_type,
            Int start_of_packet,
            Vop *vop,
            Trace *trace,
			UChar **MB_TYPE2[MAX_NUM_VOS][MAX_NUM_VOLS],
			Int PRE_H_SIZE2[MAX_NUM_VOS][MAX_NUM_VOLS],
			Int PRE_V_SIZE2[MAX_NUM_VOS][MAX_NUM_VOLS],
			UChar **PRE_MB_TYPE2[MAX_NUM_VOS][MAX_NUM_VOLS]
            )
{
    Int
        mb_bits,
        flag;
    
#ifndef _OPTIMIZE_
    Int
        width = GetVopWidth(vop)/MB_SIZE,
        error_res_disable = GetVopErrorResDisable(vop);
#else
    const Int width = GetVopWidth(vop)>>4;
    const Int error_res_disable = GetVopErrorResDisable(vop);
#endif
    
    if( vop_type == 1 && GetVopShapeCodingType(vop)) 
        flag = 1;
    else 
        flag = 0;
    
#if(__ERROR_RESILIENCE__)
    if (flag == 0)
    {
        if (!error_res_disable && start_of_packet==1)
        {       
            SetConstantImage(alpha_mode_packet,0);
        }
    }
#endif
    
    if( flag == 1 )
    {
        mb_bits = DecodeInterMBtype( mb_type, mvda, 
                                     adr_i, adr_j, stream, vo_id, vol_id, trace,
									 MB_TYPE2,PRE_H_SIZE2,PRE_V_SIZE2,
									 PRE_MB_TYPE2);
    }
    else 
    {
#if(__ERROR_RESILIENCE__)
        if (!error_res_disable)
            mb_bits = DecodeIntraMBtype_ER( stream, mb_type, adr_i, adr_j,
                                            width, vo_id, vol_id, 
                                            alpha_mode_packet, trace);
        else
#endif
            mb_bits = DecodeIntraMBtype( stream, mb_type, adr_i, adr_j,
                                         width, vo_id, vol_id, trace,MB_TYPE2);
        
        MakeMBtype2( *mb_type, adr_i, adr_j,vo_id, vol_id,MB_TYPE2);
    }
    
    return mb_bits;
}

Int
CallingConvention
AlphaDecodeMB(Vop *rec_curr,
              Vop *rec_prev,
              Int vo_id,
              Int vol_id,
              Int adr_i,
              Int adr_j,
              Int *mb_type,
              Int *mvdA,
              Int vop_type,
              Image *MB_decisions,
              Image *alpha_decisions,
              Image *mot_x,
              Image *mot_y,
              Image *modeA2,
              Image *motA2_x,
              Image *motA2_y,
              Int start_of_packet,
              Bitstream *stream,
              Trace *trace,
			  Image   *alpha_rec_packet,
				Image   *alpha_mode_packet
              )
{
    
    Image *rec_curr_alpha;
    Image *rec_prev_alpha;
    
#ifndef _OPTIMIZE_
    Image *mblock_rec;
    SInt *a_mb_rec;
    
    Image *block_rec;
    SInt *a_b_rec;
#else
    SInt block_rec[64];
    SInt mblock_rec[256];
#endif
    
    Int
        mb_bits=0,
        tmp_bits,
        mvda=1,
        flag,
        ox,oy,
        h,v;
    
    
#ifndef _OPTIMIZE_
    UChar
        alpha_mb_comp[MB_SIZE*MB_SIZE],
        alpha_dmb[MB_SIZE*MB_SIZE];
#endif
    
    SInt pred[MB_SIZE][MB_SIZE];
    
#ifndef _OPTIMIZE_
    Int width = GetVopWidth(rec_curr);
    Int fixed_cr;
    
    Int
        x,y,
        error_res_disable = GetVopErrorResDisable(rec_curr);
    
    x = adr_j * MB_SIZE;
    y = adr_i * MB_SIZE;
    
    fixed_cr = GetVopChangeCRDisable(rec_curr);
#else
    const Int width = GetVopWidth(rec_curr);
    const Int error_res_disable = GetVopErrorResDisable(rec_curr);
    
    const Int x = adr_j << 4;
    const Int y = adr_i << 4;
    
    const Int fixed_cr = GetVopChangeCRDisable(rec_curr);
#endif    
    
    if( vop_type == 1 && GetVopShapeCodingType(rec_curr) ) 
        flag = 1;
    else 
        flag = 0;
    
#if(__ERROR_RESILIENCE__)
    if (!error_res_disable && start_of_packet==1)
    {   
        if (flag)
            SetConstantImage(modeA2,0);
        SetConstantImage(alpha_rec_packet,0);
    }
#endif
    
#ifndef _OPTIMIZE_
    mblock_rec = AllocImage(16,16,SHORT_TYPE);
    a_mb_rec = (SInt *) GetImageData(mblock_rec);
#endif
    
    rec_curr_alpha = GetVopA(rec_curr);
    if (GetVopPredictionType(rec_curr) != INTRA_VOP)
        rec_prev_alpha = GetVopA(rec_prev);
    else
        rec_prev_alpha = NULL;
    
    
    if (flag) 
        mvda = *mvdA;
    
    if( *mb_type<=4 )
    {
        /* inter or all0 or all255 or MMR */
        
        if(*mb_type==-1) 
        {
#if 0
#  ifdef _OPTIMIZE_
            memset(pred, 0, MB_SIZE*MB_SIZE*sizeof(SInt));
#  else
            for (x=0;x<MB_SIZE;x++)
                for (y=0;y<MB_SIZE;y++)
                    pred[x][y]=0;
#  endif
#endif
            *((SInt *)GetImageData(modeA2)+adr_i*(width/MB_SIZE)+adr_j) = 1;
            
            ox = GetVopHorSpatRef(rec_curr) - GetVopHorSpatRef(rec_prev);
            oy = GetVopVerSpatRef(rec_curr) - GetVopVerSpatRef(rec_prev);
            tmp_bits = DecodeAlphaAndMVmei (rec_curr,x,y,ox,oy,mot_x,mot_y,
                                            motA2_x,motA2_y,
                                            MB_decisions,alpha_decisions,modeA2,
                                            (SInt *)GetImageData(rec_prev_alpha),
                                            (SInt *)pred,mvda,width,
                                            GetImageSizeX(rec_prev_alpha),
                                            GetImageSizeY(rec_prev_alpha),
                                            MB_SIZE,stream,trace);
            mb_bits += tmp_bits;
        }   
        
        
#ifndef _OPTIMIZE_
        for( v=0; v<MB_SIZE; v++ )
            for( h=0; h<MB_SIZE; h++ ) 
            {
                alpha_mb_comp[v*MB_SIZE+h] = (UChar)pred[v][h];
            }
        
        mb_bits += ShapeDecodingCAE(stream, 
                                    alpha_dmb, 
                                    alpha_mb_comp,
                                    rec_curr_alpha,
                                    alpha_rec_packet,
                                    x,
                                    y,
                                    fixed_cr,
                                    mb_type,
                                    error_res_disable,
                                    trace);
#else
        mb_bits += ShapeDecodingCAE(stream, 
                                    (SInt *)mblock_rec, 
                                    (SInt *)pred,
                                    rec_curr_alpha,
                                    alpha_rec_packet,
                                    x,
                                    y,
                                    fixed_cr,
                                    mb_type,
                                    error_res_disable,
                                    trace);
#endif  
    }
    else
    {
#if 0
#  ifdef _OPTIMIZE_
        memset(pred, 0, MB_SIZE*MB_SIZE*sizeof(SInt));
#  else
        for (x=0;x<MB_SIZE;x++)
            for (y=0;y<MB_SIZE;y++)
                pred[x][y]=0;
#  endif
#endif
        *((SInt *)GetImageData(modeA2)+adr_i*(width/MB_SIZE)+adr_j) = 1;
        
        ox = GetVopHorSpatRef(rec_curr) - GetVopHorSpatRef(rec_prev);
        oy = GetVopVerSpatRef(rec_curr) - GetVopVerSpatRef(rec_prev);
        tmp_bits = DecodeAlphaAndMVmei (rec_curr,x,y,ox,oy,
                                        mot_x,mot_y, motA2_x,motA2_y,
                                        MB_decisions, alpha_decisions, modeA2,
                                        (SInt *)GetImageData(rec_prev_alpha),
                                        (SInt *)pred,mvda,width,
                                        GetImageSizeX(rec_prev_alpha),
                                        GetImageSizeY(rec_prev_alpha),
                                        MB_SIZE,stream,trace);
        mb_bits += tmp_bits;
        
#ifndef _OPTIMIZE_
        ox = GetVopHorSpatRef(rec_curr) - GetVopHorSpatRef(rec_prev);
        oy = GetVopVerSpatRef(rec_curr) - GetVopVerSpatRef(rec_prev);
#endif
        
#ifndef _OPTIMIZE_
        for( v=0; v<MB_SIZE; v++ ) for( h=0; h<MB_SIZE; h++ ) 
        {
            alpha_mb_comp[v*MB_SIZE+h] = (UChar)pred[v][h];
        }
        
        
        mb_bits += ShapeDecodingInterCAE(stream, 
                                         alpha_dmb, 
                                         alpha_mb_comp,
                                         rec_curr_alpha,
                                         rec_prev_alpha,
                                         alpha_rec_packet,
                                         motA2_x,
                                         motA2_y,
                                         x,
                                         y,
                                         ox,
                                         oy,
                                         fixed_cr,
                                         mb_type,
                                         error_res_disable,
                                         trace);
#else
        mb_bits += ShapeDecodingInterCAE(stream, 
                                         (SInt *)mblock_rec,
                                         (SInt *)pred,
                                         rec_curr_alpha,
                                         rec_prev_alpha,
                                         alpha_rec_packet,
                                         motA2_x,
                                         motA2_y,
                                         x,
                                         y,
                                         ox,
                                         oy,
                                         fixed_cr,
                                         mb_type,
                                         error_res_disable,
                                         trace);
#endif
    }
    
#ifndef _OPTIMIZE_
    for( v=0; v<MB_SIZE; v++ )
        for( h=0; h<MB_SIZE; h++ )
        {
            a_mb_rec[v*MB_SIZE+h] = (SInt)alpha_dmb[v*MB_SIZE+h];
        }
#endif
    
#ifndef _OPTIMIZE_
    PutSubImage(GetVopA(rec_curr),mblock_rec,x,y);
#else
    PutSubImage_opt(rec_curr_alpha->f, mblock_rec,
                    rec_curr_alpha->x, rec_curr_alpha->y,
                    16,16,
                    x, y);
#endif
    
#if (__ERROR_RESILIENCE__)
    if (!error_res_disable)
        PutSubImage(alpha_rec_packet,mblock_rec,x,y);
#endif
    
#ifndef _OPTIMIZE_
    block_rec = AllocImage(8,8,SHORT_TYPE);
    a_b_rec = (SInt *) GetImageData(block_rec);
    for( v=0; v<8; v++ )
        for( h=0; h<8; h++ )
        {
            Int index = (v+v)*MB_SIZE+h+h;
            a_b_rec[v*8+h] = (a_mb_rec[index]||a_mb_rec[index+1]
                              ||a_mb_rec[index+MB_SIZE]||a_mb_rec[index+MB_SIZE+1])?255:0;
        }
#else
    flag = 0;
    for( v=0; v<8; v++ )
        for( h=0; h<8; h++ )
        {
            Int index = ((v+v)<<4) + h+h;
            block_rec[flag++] = (mblock_rec[index]||
                                 mblock_rec[index+1]||
                                 mblock_rec[index+MB_SIZE]||
                                 mblock_rec[index+MB_SIZE+1])  ? 255:0;
        }    
#endif
    
#ifndef _OPTIMIZE_
    PutSubImage(GetVopAuv(rec_curr),block_rec,x/2,y/2);
#else
    rec_curr_alpha = GetVopAuv(rec_curr);
    PutSubImage_opt(rec_curr_alpha->f, &block_rec[0],
                    rec_curr_alpha->x, rec_curr_alpha->y,
                    8,8,
                    x>>1, y>>1);
#endif
    
#ifndef _OPTIMIZE_
    FreeImage (block_rec);    
    FreeImage (mblock_rec);
#endif
    
    return mb_bits;
}

#if 0
Void
CallingConvention
BitstreamReadMtoT(Bitstream *stream, UChar *shape_stream, Int nbits)
{
    UInt i, code=0;
    
    if (nbits > 0)
    {
        code = BitstreamShowBits(stream, nbits);
    }
    for (i = 0; i < (UInt)nbits; i++)
        shape_stream[i] = (UChar)((code >> (nbits - 1 - i)) & 1);
    
}
#endif

Void
CallingConvention
AllocShapePacket2(	Vop *vop,
					Image   **alpha_rec_packet,
					Image   **alpha_mode_packet)
{
    Int         width = GetVopWidth(vop)/MB_SIZE,
				height = GetVopHeight(vop)/MB_SIZE;
    
    *alpha_rec_packet = AllocSameImage(GetVopA(vop));
    *alpha_mode_packet = AllocImage(width,height,SHORT_TYPE);
}

Void
CallingConvention
FreeShapePacket2(	Image   *alpha_rec_packet,
					Image   *alpha_mode_packet)
{
    FreeImage(alpha_rec_packet);
    FreeImage(alpha_mode_packet);
}

/***********************************************************CommentBegin******
 *
 * -- VopFirstDecode -- decodes all first_shape_codes for a given VOP 
 *                      ,used only in separate mode decoding
 *
 * Author :             TOSHIBA
 *
 * Created :            
 *      11-04-97
 * 
 * Arguments In:        
 *                                      stream - the bitstream
 *                                      rec_curr - reconstructed VOP
 *                                      vo_id - 
 *                                      vol_id - 
 *
 * Arguments In/Out:
 *
 * Arguments Out:
 *                                      mb_type - the decoded BAB modes
 *                                      mvda - the decoded flags indicting whether MVDs=0 or not.
 *
 * Return values :
 *
 * Side effects :       
 *
 * Description : As per VM7
 *
 * See also :
 *
 *
 ***********************************************************CommentEnd********/
#if 0
Void
CallingConvention
VopFirstDecode(
    Bitstream *stream,
    Int vo_id,
    Int vol_id,
    Vop *rec_curr,
    Int *mb_type,
    Int *mvda,
    Trace *trace,
	Int PRE_H_SIZE2[MAX_NUM_VOS][MAX_NUM_VOLS],
	Int PRE_V_SIZE2[MAX_NUM_VOS][MAX_NUM_VOLS],
	UChar **PRE_MB_TYPE2[MAX_NUM_VOS][MAX_NUM_VOLS]
    )
{
    Int i, j;
#ifndef _OPTIMIZE_
    Int width, height, vop_type;
    
    if(GetVopPredictionType(rec_curr) == INTRA_VOP)
        vop_type = 0;
    else
        vop_type = 1;
    
    width = GetVopWidth(rec_curr)/MB_SIZE;
    height = GetVopHeight(rec_curr)/MB_SIZE;
#else
    const Int vop_type = (GetVopPredictionType(rec_curr) == INTRA_VOP) ? 0:1;
    
    const Int width = GetVopWidth(rec_curr)/MB_SIZE;
    const Int height = GetVopHeight(rec_curr)/MB_SIZE;    
#endif
    
    for(i=0; i<height; i++)
        for(j=0; j<width; j++)
            (Void)DecodeFirst(stream, vo_id, vol_id, i, j, &mb_type[i*width+j], 
                              &mvda[i*width+j], vop_type, 0, rec_curr, trace,
							  MB_TYPE2,PRE_H_SIZE2,PRE_V_SIZE2,PRE_MB_TYPE2);
}
#endif

/***********************************************************CommentBegin******
 *
 * -- VopShapeDecode -- decodes all the binary shape fields 
 *                      (except first_shape_code)  for a given VOP 
 *                      ,used only in separate mode decoding
 *
 * Author :             TOSHIBA
 *
 * Created :            
 *      11-04-97
 * 
 * Arguments In:        
 *                                      stream - the bitstream
 *                                      rec_prev - previously reconstructed VOP
 *                                      mb_type - the decoded BAB modes
 *                                      mvda - the decoded flags indicting whether MVDs=0 or not.
 *                                      vo_id - 
 *                                      vol_id - 
 *
 * Arguments In/Out:
 *                                      rec_curr - reconstructed VOP
 *
 * Arguments Out:
 *
 * Return values :
 *
 * Side effects :       
 *
 * Description : As per VM7
 *
 * See also :
 *
 *
 ***********************************************************CommentEnd********/
#if 0
Void
CallingConvention
VopShapeDecode(
    Vop *rec_curr,
    Vop *rec_prev,
    Int vo_id,
    Int vol_id,
    Int *mb_type,
    Int *mvda,
    Bitstream *stream,
    Trace *trace,
	Image   *alpha_rec_packet,
	Image   *alpha_mode_packet
    )
{
    Image *modeA2; /* the flag of current alpha block is inter coded or not */
    Image *motA2_x; /* the x compornent of shape MV */
    Image *motA2_y; /* the y compornent of shape MV */
    Int i, j, width, height, vop_type;
    
    /* Allocate shape ME/MC */
    modeA2 = AllocImage(GetVopWidth(rec_curr)/MB_SIZE,
                        GetVopHeight(rec_curr)/MB_SIZE,SHORT_TYPE);
    motA2_x = AllocImage(GetVopWidth(rec_curr)/MB_SIZE,
                         GetVopHeight(rec_curr)/MB_SIZE,SHORT_TYPE);
    motA2_y = AllocImage(GetVopWidth(rec_curr)/MB_SIZE,
                         GetVopHeight(rec_curr)/MB_SIZE,SHORT_TYPE);
    
    if(GetVopPredictionType(rec_curr) == INTRA_VOP) vop_type = 0;
    else                                            vop_type = 1;
    width = GetVopWidth(rec_curr)/MB_SIZE;
    height = GetVopHeight(rec_curr)/MB_SIZE;
    
    for(i=0; i<height; i++)
        for(j=0; j<width; j++)
            (Void)AlphaDecodeMB(rec_curr,
                                rec_prev,
                                vo_id,
                                vol_id,
                                i,
                                j,
                                &mb_type[i*width+j],
                                &mvda[i*width+j],
                                vop_type,
                                (Image *)NULL,
                                (Image *)NULL,
                                (Image *)NULL,
                                (Image *)NULL,
                                modeA2,
                                motA2_x,
                                motA2_y,
                                0,
                                stream,
                                trace,
								alpha_rec_packet,alpha_mode_packet
                                );
    /* Free shape ME/MC */
    FreeImage(modeA2);
    FreeImage(motA2_x);
    FreeImage(motA2_y);
}
#endif
