/*****************************************************************************/
/*         This software module has been speed optimized                     */
/*                                                                           */
/*         by  Bruno Loret - France Telecom / CNET                           */
/*                                                                           */
/*         in the framework of the ACTS Project MoMuSys                      */
/*****************************************************************************/

/*
 * $RCSfile: chrono.c,v $           $Author: mleditsc $
 * $Revision: 1.1 $          $Date: 2000/01/24 05:38:31 $ 
 * $Locker:  $      
 */

/***********/
/* HISTORY */
/***********/
/*
 * $Log: chrono.c,v $
 * Revision 1.1  2000/01/24 05:38:31  mleditsc
 * *** empty log message ***
 *
 * Revision 1.2  2000/01/19 03:09:20  mleditsc
 * Fixed up a problem with lines ending with CR CR LF
 *
 * Revision 1.1.1.1  2000/01/13 06:05:56  mleditsc
 * Import of 3D player 990924 2.55
 *
 *
 */


#include "momusys.h"
#include "chrono.h"

#ifndef WIN32

#include <sys/time.h>
#include <unistd.h>

Int
start_chrono(Struct_Chrono *c)
{
    struct timeval tstart;

    gettimeofday(&tstart,(struct timezone *)NULL);
    c->start   = 100*tstart.tv_sec+tstart.tv_usec/10000;
    c->stop    = 0;
    c->runtime = 0;
    
    return(0);
}



Int
stop_chrono(Struct_Chrono *c)
{
    struct timeval tstop;
    
    gettimeofday(&tstop,(struct timezone *)NULL);
    
    c->stop   = 100*tstop.tv_sec+tstop.tv_usec/10000;
    c->runtime = c->stop - c->start;    

    return(c->runtime);
}

Int
display_chrono(FILE *fp, Int runtime, Int framenum)
{
    fprintf(fp, "%d.%02d seconds, %d frames, %d.%02d fps\n",
            runtime/100, runtime%100,
            framenum, ((10000*framenum+runtime/2)/runtime)/100,
            ((10000*framenum+runtime/2)/runtime)%100);

    
    return(0);
}

#else

#include <windows.h>

Int
CallingConvention
start_chrono(Struct_Chrono *c)
{
    c->start   = (Int) GetTickCount();
    c->stop    = 0;
    c->runtime = 0;
    
    return(0);
}

Int
CallingConvention
stop_chrono(Struct_Chrono *c)
{
    
    c->stop   = (Int) GetTickCount();
    c->runtime = c->stop - c->start;    
    
    return(0);
}

Int
CallingConvention
display_chrono(FILE *fp, Int runtime, Int framenum)
{    
    fprintf(fp, "%d.%02d seconds, %d frames, %f fps\n",
            runtime/1000, runtime%1000,
            framenum, ((float)(1000*framenum))/((float)(runtime)) );
    return(0);
}


#endif
