/*****************************************************************************
 *
 * This software module was originally developed by
 *
 *   Isabelle Corset (Philips / ACTS-MoMuSyS)                        
 *   Michael Wollborn (TUH / ACTS-MoMuSyS)                                
 *   Cor Quist (KPN / ACTS-MoMuSyS)                                       
 *   Angel Pacheco (UPM / ACTS-MoMuSyS)                                   
 *   J.Ignacio Ronda (UPM / ACTS-MoMuSyS)                         
 *
 * and edited by
 * 
 *   Paulo Nunes (IST / ACTS-MoMuSyS)                                    
 *   Robert Danielsen (Telenor / ACTS-MoMuSyS)                            
 *   Jan De Lameillieure (Heinrich-Hertz-Institut Berlin / ACTS-MoMuSyS)     
 *   Aasmund Sandvand (Telenor / ACTS-MoMuSyS)                                 
 *   Fernando Jaureguizar (UPM / ACTS-MoMuSyS)                       
 *
 * in the course of development of the MPEG-4 Video (ISO/IEC 14496-2) standard.
 * This software module is an implementation of a part of one or more MPEG-4
 * Video (ISO/IEC 14496-2) tools as specified by the MPEG-4 Video (ISO/IEC
 * 14496-2) standard.
 *
 * ISO/IEC gives users of the MPEG-4 Video (ISO/IEC 14496-2) standard free
 * license to this software module or modifications thereof for use in hardware
 * or software products claiming conformance to the MPEG-4 Video (ISO/IEC
 * 14496-2) standard.
 *
 * Those intending to use this software module in hardware or software products
 * are advised that its use may infringe existing patents. The original
 * developer of this software module and his/her company, the subsequent
 * editors and their companies, and ISO/IEC have no liability for use of this
 * software module or modifications thereof in an implementation. Copyright is
 * not released for non MPEG-4 Video (ISO/IEC 14496-2) Standard conforming
 * products.
 *
 * ACTS-MoMuSys partners retain full right to use the code for his/her own
 * purpose, assign or donate the code to a third party and to inhibit third
 * parties from using the code for non MPEG-4 Video (ISO/IEC 14496-2) Standard
 * conforming products. This copyright notice must be included in all copies or
 * derivative works.
 *
 * Copyright (c) 1996
 *
 *****************************************************************************/

/*****************************************************************************/
/*         This software module has been speed optimized                     */
/*                                                                           */
/*         by  Bruno Loret - France Telecom / CNET                           */
/*                                                                           */
/*         in the framework of the ACTS Project MoMuSys                      */
/*****************************************************************************/

/***********************************************************HeaderBegin*******
 *                                                                         
 * File:        combined_decode.c
 *
 * Author:      Jan De Lameillieure (HHI) <jan@hhi.de>
 * Created:     18-Jan-97
 *                                                                         
 * Description: Decoding VOP in combined mode
 *
 * Notes:       
 *              starting point of this file was text_decode.c of 16-Jan-97
 *
 * Modified: 04.02.97 Noel O'Connor : removed all calls to assert()
 *           07-Feb-1997, M.Wollborn: changed in order to treat skipped MBs
 *                            correctly
 *           15-APR-97 Jan De Lameillieure : removed unused variables which were 
 *                       obsolete since upgrade from MMR to M4R
 *           15-APR-97 Jan De Lameillieure : added the inverse SA-DCT 
 *           22-APR-97 Michael Wollborn: changed name of BlockDequant to 
 *                            BlockDequantH263 and changed
 *                            stuff for MPEG-like quantization
 *           01.05.97 Luis Ducla-Soares:added functions for the error resilient
 *                       combined mode with data partitioning:
 *                       DecodeVopCombinedShapeTextureIntraErrRes(),
 *                       DecodeVopCombinedMotionShapeTextureInterErrRes(), 
 *                       GetMBheaderErrRes(), GetMBheaderMotionErrRes(),
 *                       GetMBheaderTextErrRes(), GetMBvectorsErrRes()
 *                                 and GetMBblockdataErrRes().
 *           15.05.97 Luis Ducla-Soares: corrected the remultiplexing of the DCT data
 *                       in the combined error resilient mode with 
 *                       data partitioning. Cleaned up the code.
 *           21.05.97 A. Pacheco (UPM): Modifications in several functions
 *                       to have just one function to compute the MVs prediction
 *                       in both encoder/decoder (find_pmvs)
 *           26.05.97 Minhua Zhou: added mot_padding.h and VopPadding() 
 *           16.06.97 Angel Pacheco: unified the TRANSPARENT modes.
 *      16-JUN-97 Jan De Lameillieure : renaming some include files to sadct_*
 *      16-JUN-97 Jan De Lameillieure : added Kaup-SADCT
 *           24.07.97 Minhua Zhou: added B-VOP decoding
 *           04.08.97 Minhua Zhou: renamed advanced_prediction_disable
 *      04.08.97 Minhua Zhou: added intra_dc_vlc_thr (switched)
 *      12.08.97 Minhua Zhou: added decoding of DQUANT in combined mode 
 *      05.09.97 Fernando Jaureguizar: deleted unnecessary code with aux_vop
 *
 ***********************************************************HeaderEnd*********/

/************************    INCLUDE FILES    ********************************/

#include <assert.h>

#include "io_generic.h"
#include "vm_common_defs.h"
#include "combined_decode.h"
#include "text_decode_mb.h"
#include "zigzag.h"
#include "mot_decode.h"
#include "mot_util.h"
#include "mot_comp.h"
#include "alp_common_def.h"
#include "alp_dec_mom.h" 
#include "alp_dec_header.h" 

#ifndef _WD_MODE_
#  include "sadct_momusys.h"
#  include "sadct_momusys_kaup.h"
#endif

#include "mot_padding.h"

#include "ring.h"

/*** 8/18 TPS */
#include "vm_dec_main.h"



/*ok*/static void
CallingConvention
FillAlphaForTrace(Int comp, Short *alpha, UChar shape, Short nrpix, Int x1, Int y1, Int trace, SInt mb_shape[][64]); 


Void
CallingConvention
DecodeVopInter(       Bitstream * stream,
                      Int vo_id,
                      Trace *trace,
                      Vop *rec_vop,
                      Image **mot_x_out,
                      Image **mot_y_out,
                      Image **MB_decisions_out,
					  UChar **MB_TYPE2[MAX_NUM_VOS][MAX_NUM_VOLS],
					  Int PRE_H_SIZE2[MAX_NUM_VOS][MAX_NUM_VOLS],
					  Int PRE_V_SIZE2[MAX_NUM_VOS][MAX_NUM_VOLS],
					  UChar **PRE_MB_TYPE2[MAX_NUM_VOS][MAX_NUM_VOLS],
					  Image   *alpha_rec_packet,
						Image   *alpha_mode_packet);

/*ok*/static Void
CallingConvention
VopMotionCompensate_opt (Vop *rec_prev,
                         Image *mot_x,
                         Image *mot_y,
                         Image *MB_decisions,
                         Image *alpha_decisions,
                         UInt  *CBP_store,
                         Vop *comp,
                         Int obmc_disable, 
                         Vop *sprite);
/*ok*/static void
CallingConvention
CompY(Int i, Int j, Int mode,
      Int prev_x_min, Int prev_y_min, Int prev_x_max, Int prev_y_max,
      Image *mot_x, Image *mot_y,
      Image *MB_decisions, Image *alpha_decisions,
      SInt *prev_ipol, Image *comp,
      Int width,
      Int width_prev, Int overlap);

/*ok*/static Void
CallingConvention
CompC(Int i, Int j, Int mode,
      Int prev_x_min, Int prev_y_min, Int prev_x_max, Int prev_y_max,
      Image *mot_x, Image *mot_y,
      Image *MB_decisions, Image *alpha_decisions,
      Int width,
      Int width_prev,
      Vop *prev_image,
      UInt  *CBP_store,
      Vop *comp_Vop);
/*ok*/static Void
CallingConvention
GetMBblockdata( Int transp_pattern[],
                Int intra_dcpred_disable,
                Int quant_step,
                Bitstream * texture_bits,
                Int mbnum,
                Trace *trace,
                Macroblock * mblock,
                Int * skipped_flag,
                Int * error_flag,
                Int MB_width,
                Int *DC_store,
                SInt *QP_store,
                Int QP,
                UInt CBP,
                Int ACpred_flag,
                UInt mode,
                Short *alpha,
                Vop *rec_vop);
/*ok*/static Void
CallingConvention
GetMBheader(Int vop_type,
            UInt *new_vop,
            Int intra_dcpred_disable,
            Int *quant_step,
            Bitstream * texture_bits,
            Int mbnum,
            Trace *trace,
            Image * motion_mode,
            Macroblock * mblock,
            Int * skipped_flag,
            Int * error_flag,
            Int *DC_store,
            SInt *QP_store,
            Int QP,
            Int *coded,
            UInt *CBP,
            Int *ACpred_flag,
            UInt *mode,
            Int *MB_transp_pattern);
/*ok*/static Void
CallingConvention
GetMBvectors (Int vop_type,
              Int intra_dcpred_disable,
              Int *quant_step,
              Bitstream * bits,
              Int mbnum,
              Int MB_width,
              Int f_code,
              Trace *trace,
              Image * motion_mode,
              Image * block_decis,
              Image * mot_x,
              Image * mot_y,
              Int * skipped_flag,
              Int * error_flag,
              Int *DC_store,
              SInt *QP_store,
              Int QP,
              Int *coded,
              Int *ACpred_flag,
              UInt mode);
/*ok*/static Void
CallingConvention
PrintMBShapeData(Vop *vop, Int x_pos, Int y_pos, Trace *trace);

/* For correct rounding of chrominance vectors */
/*ok*/static Int roundtab4[] = {0,1,1,1};
/*ok*/static Int roundtab8[] = {0,0,1,1,1,1,1,2};
/*** 10/30 for TPS */
/*ok*/static Int roundtab12[] = {0,0,0,1,1,1,1,1,1,1,2,2}; 
/* 10/30 for TPS ***/
/*ok*/static Int roundtab16[] = {0,0,0,1,1,1,1,1,1,1,1,1,1,1,2,2};

/***********************************************************CommentBegin******
 *
 * -- DecodeVopCombinedShapeTextureIntra -- 
 *
 * Author :             
 *      Jan De Lameillieure (HHI) <jan@hhi.de>
 *
 * Created :            
 *      18-Jan-97
 *
 * Purpose :            
 *      Decodes combined shape / texture for INTRA VOPs
 *
 * Arguments in :       
 *      Bitstream *stream       combined shape / texture bitstream
 *      Trace  *trace           flag for tracing the process.
 *
 *
 * Arguments in/out :   
 *      Vop *rec_vop            alpha plane is supposed to be filled
 *                              already / YUV components.
 *
 * Arguments out :      
 *
 *
 * Return values :      
 *      
 *
 * Side effects :       
 *      
 *
 * Description :        
 *
 *
 * Note : 
 *      Starting point for this function was DecodeVopTextureIntra() as of 16-Jan-97
 *
 * See also :
 *      
 *
 * Modified : 
 *                      06.08.97: Noel Brady: Added mods for BINARY_SHAPE_ONLY
 *                      04.11.97: Minhua Zhou: updated DC/AC prediction 
 *
 ***********************************************************CommentEnd********/

Void
CallingConvention
DecodeVopCombinedShapeTextureIntra(
		Bitstream * stream, Int vo_id, 
        Trace *trace, Vop * rec_vop,
		UChar **MB_TYPE2[MAX_NUM_VOS][MAX_NUM_VOLS],
		Int PRE_H_SIZE2[MAX_NUM_VOS][MAX_NUM_VOLS],
		Int PRE_V_SIZE2[MAX_NUM_VOS][MAX_NUM_VOLS],
		UChar **PRE_MB_TYPE2[MAX_NUM_VOS][MAX_NUM_VOLS],
		Image   *alpha_rec_packet,
		Image   *alpha_mode_packet
		)
{
    Int                 vol_id = GetVopId(rec_vop);
    Int                 ef=0, *error_flag = &ef; 
    Int                 mbnum, x_pos, y_pos, i, j;
    Int                 rec_height;
    Int                 rec_width;
    Int                 MB_in_height, MB_in_width;
    Int                 B_in_width, B_in_height;
    Macroblock          *mblock;
    Image               *Y_rec, *U_rec, *V_rec, *A_rec;
    Short               *alpha;
    Int                 vop_quant,intra_dcpred_disable;
    Int                 skipped_flag=0;
    Image               *MB_decisions;
    UInt                new_vop;
    SInt                *pMB_decisions;
    Int                 MB_transp_pattern[4];
#ifndef _OPTIMIZE_
    Int                 ***DC_store;
    Int                 n;
#else
    Int                 *DC_store;
    Int                 *p;
#endif
    
    Int                 m, QP;
    Int                 coded;
    UInt CBP;
    Int  ACpred_flag=-1;
    UInt mode;
    Int amb_type=0,mvda=0;
    SInt *QP_store;
    Int MB_non_trans_count=0;
    
#if (__SPRITES__)
    Int u, comp;
#endif
    Int vop_quant_prev;
    
#if (__SPRITES__)
    /* for Low Latency Static SPRITE */ 
    Int *tab_transmit,numblocks_x, *tab_amb_type, ***tab_DC_store;
    Int *tab_mvda;
    SInt *tab_QP_store;
    
    if (GetVopSpriteUsage(rec_vop)!=SPRITE_NOT_USED)
    { 
        tab_transmit = GetVopSpriteTabTransmit(rec_vop);
        tab_amb_type = GetVopSpriteTabAmbType(rec_vop);
        tab_mvda = GetVopSpriteTabMvda(rec_vop);
        tab_DC_store = GetVopSpriteTabDCStore(rec_vop);
        tab_QP_store = GetVopSpriteTabQPStore(rec_vop);
        numblocks_x = GetVopSpriteHdim(rec_vop)/16;
    }
#endif    
    
    /*  if(!GetVopShape(rec_vop)) {
        printf("The function DecodeVopCombinedShapeTextureIntra() should not have \n");
        printf("been called because there is no shape information !\n");
        return;
        } */
    
    /*****
     *
     *  Set some useful parameters
     *
     *****/
    rec_height = GetVopHeight (rec_vop);
    rec_width = GetVopWidth (rec_vop);
    MB_in_height = rec_height>>4;
    MB_in_width = rec_width>>4;
    B_in_width = MB_in_width<<1;
    B_in_height = MB_in_height<<1;
    QP_store = (SInt *) GetImageData(GetVopQP(rec_vop));
    
    /* allocate space for 3D matrix to keep track of prediction values
       for DC/AC prediction */
    
#ifndef _OPTIMIZE_
    DC_store = (Int ***)calloc(MB_in_width*MB_in_height,
                               sizeof(Int **));
    for (i = 0; i < MB_in_width*MB_in_height; i++)
    {
        DC_store[i] = (Int **)calloc(6, sizeof(Int *));
        for (j = 0; j < 6; j++)
            DC_store[i][j] = (Int *)calloc(15, sizeof(Int));
    }
#else
    DC_store = Get_DC_store();
#endif
    
    intra_dcpred_disable = GetVopIntraACDCPredDisable(rec_vop);
    
    MB_decisions    = AllocImage(MB_in_width,MB_in_height,SHORT_TYPE);
    
    /*****
     *
     *  Get the image fields of the VOP
     *
     *****/
    Y_rec = GetVopY (rec_vop);
    U_rec = GetVopU (rec_vop);
    V_rec = GetVopV (rec_vop);
    A_rec = GetVopA (rec_vop);
    alpha=(Short *)GetImageData(A_rec);
    
    /*****
     *
     *  Get VOP quantizer and allocate MB structure
     *
     *****/
    vop_quant = GetVopQuantizer (rec_vop);
    QP = vop_quant;
    vop_quant_prev = vop_quant;
    
    mblock = (Macroblock *) malloc (sizeof (Macroblock));
    
    if (mblock == NULL)
    {
        printf (" Error allocating macroblock\n");
        exit(1);
    }
#ifdef __WRITE_IM__    
#  if(__SPRITES__)
    if (GetVopSpriteUsage(rec_vop)==STATIC_SPRITE)
        printf("Start I VOP decoding (object-piece of static sprite)\n");
    else
#  endif
        printf("Start I VOP decoding \n");
#endif
    
    /*****
     *
     *  Loop over all MBs, decode texture of each MB
     *
     *****/
    new_vop = 1;
    pMB_decisions = (SInt *)GetImageData(MB_decisions);
    
    for (y_pos = 0; y_pos < MB_in_height; y_pos++)
        for (x_pos = 0; x_pos < MB_in_width; x_pos++)
        {    
            coded=0;
            mbnum = y_pos * MB_in_width + x_pos;
            
            if (GetVopShape(rec_vop) != BINARY_SHAPE_ONLY)
            {
                if (MB_non_trans_count>=MB_trace_thres)
                    SetTraceLevelsZero(trace);
                else
                    SetTraceLevelsDefault(trace);
            }
            
            
            /* Store the QP value for later use in AC prediction */
            QP_store[mbnum] = (SInt) QP;
            
#ifdef _DEBUG
            trace->trace = trace->MB_header;
            if (trace->trace)
            {
                fprintf (trace->fp_trace, "\n>>>   Decoding Macroblock %d ........- Combined Shape/Text\n",(int)mbnum);
            }
            
            trace->trace = trace->MB_shape;
#endif
            
            if (GetVopShape(rec_vop))
            {
                if (GetVopSpriteUsage(rec_vop)!=STATIC_SPRITE
#if (__SPRITES__)
                    || tab_transmit[y_pos*numblocks_x+x_pos] ==0
#endif
                    )
                {
                    DecodeFirst(stream, 
                                vo_id, 
                                vol_id, 
                                y_pos, 
                                x_pos, 
                                &amb_type, 
                                &mvda, 
                                0,
                                0, 
                                rec_vop,
                                trace,
								MB_TYPE2,
								PRE_H_SIZE2,
								PRE_V_SIZE2,
								PRE_MB_TYPE2);
                    
                    /* fill the alpha map with the decoded pixels for this MB */
                    AlphaDecodeMB(rec_vop, 
                                  (Vop *)NULL, 
                                  vo_id,
                                  vol_id,
                                  y_pos, 
                                  x_pos, 
                                  &amb_type,
                                  &mvda, 
                                  0,
                                  MB_decisions,
                                  (Image *)NULL,
                                  (Image *)NULL,
                                  (Image *)NULL,
                                  (Image *)NULL,        
                                  (Image *)NULL,
                                  (Image *)NULL,
                                  0,
                                  stream,
                                  trace,
								  alpha_rec_packet,alpha_mode_packet);
#if (__SPRITES__)
                    if (GetVopSpriteUsage(rec_vop)==STATIC_SPRITE)
                    {
                        tab_amb_type[y_pos*numblocks_x+x_pos] = amb_type;
                        tab_mvda[y_pos*numblocks_x+x_pos] = mvda;
                    }
#endif
                } /* STATIC_SPRITE */
            }/* GetVopShape */
            
#if (__SPRITES__)
            if (GetVopSpriteUsage(rec_vop)==STATIC_SPRITE&&
                tab_transmit[y_pos*numblocks_x+x_pos] !=0)
            {
                if (GetVopShape(rec_vop)) 
                {
                    amb_type=tab_amb_type[y_pos*numblocks_x+x_pos];
                    mvda=tab_mvda[y_pos*numblocks_x+x_pos];
                    MakeMBtype2(amb_type,y_pos,x_pos,vo_id,vol_id,MB_TYPE2);
                }
                for (comp=0 ; comp<6 ; comp++)
                    for (u=0 ; u<15 ; u++)
                        DC_store[mbnum][comp][u] = tab_DC_store[y_pos*numblocks_x+x_pos][comp][u];
                if (amb_type != 1) /* if this is MB not all_0 */
                    vop_quant = (Int)tab_QP_store[y_pos*numblocks_x+x_pos];
            }
#endif
            
#ifdef  _DEBUG
            if(trace->trace)
                fprintf(trace->fp_trace, "----------\n");
            trace->trace = trace->MB_header;
#endif
            
            if (GetVopShape(rec_vop) != BINARY_SHAPE_ONLY)   /* BSO_NOEL */
            {    
                if (amb_type != 1) /* if this is MB not all_0 */
                {
                    /* calculating MB_transp_pattern */
                    for(i=0;i<4;i++)
                        MB_transp_pattern[i]=1;
                    
                    for(i=0;i<B_SIZE;i++)
                        for(j=0;j<B_SIZE;j++)
                            if (alpha[((y_pos * MB_SIZE) +i)*rec_width + 
                                     (x_pos * MB_SIZE) +j] != 0)
                                MB_transp_pattern[0]=0;
                    
                    for(i=0;i<B_SIZE;i++)
                        for(j=B_SIZE;j<2*B_SIZE;j++)
                            if (alpha[((y_pos * MB_SIZE) +i)*rec_width + 
                                     (x_pos * MB_SIZE) +j] != 0)
                                MB_transp_pattern[1]=0;
                    
                    for(i=B_SIZE;i<2*B_SIZE;i++)
                        for(j=0;j<B_SIZE;j++)
                            if (alpha[((y_pos * MB_SIZE) +i)*rec_width + 
                                     (x_pos * MB_SIZE) +j] != 0)
                                MB_transp_pattern[2]=0;
                    
                    for(i=B_SIZE;i<2*B_SIZE;i++)
                        for(j=B_SIZE;j<2*B_SIZE;j++)
                            if (alpha[((y_pos * MB_SIZE) +i)*rec_width + 
                                     (x_pos * MB_SIZE) +j] != 0)
                                MB_transp_pattern[3]=0;
                    
                    
                    /* set MB_decisions for this MB on intra */
                    *((SInt *) GetImageData (MB_decisions) + mbnum) = INTRA;
                    
                    
                    if (GetVopSpriteUsage(rec_vop)!=STATIC_SPRITE
#if (__SPRITES__)
                        || tab_transmit[y_pos*numblocks_x+x_pos] ==0
#endif
                        )
                    {
                        vop_quant = vop_quant_prev;
                        
                        GetMBheader(INTRA_VOP,&new_vop,
                                    intra_dcpred_disable,
                                    &vop_quant,stream,mbnum,trace,MB_decisions,
                                    mblock,&skipped_flag, error_flag,
                                    DC_store, QP_store, QP, &coded, 
                                    &CBP, &ACpred_flag, &mode,
                                    MB_transp_pattern);
                        
                        vop_quant_prev = vop_quant;
                        
                        if (*error_flag == RETURN)
                            exit(1);
                        
#if (__SPRITES__)
                        if (GetVopSpriteUsage(rec_vop)==STATIC_SPRITE)
                            tab_QP_store[y_pos*numblocks_x+x_pos] = (SInt) vop_quant;
#endif
                    }
                    QP_store[mbnum] = (SInt) vop_quant;
                    
                }
                
                if (amb_type == 2) 
                { /* if this MB is a multilevel MB */
                    
                }
                
                if (amb_type != 1) /* if it is not a transparent MB */
                {
                    MB_non_trans_count++;
                    if (GetVopSpriteUsage(rec_vop)!=STATIC_SPRITE
#if (__SPRITES__)
                        || tab_transmit[y_pos*numblocks_x+x_pos] ==0
#endif
                        )
                    {
                        if (coded)
                            GetMBblockdata(MB_transp_pattern,
                                           intra_dcpred_disable,
                                           vop_quant,stream,mbnum,trace,
                                           mblock,&skipped_flag, error_flag, 
                                           MB_in_width, 
                                           DC_store, QP_store, QP,
                                           CBP, ACpred_flag, mode,
                                           alpha, rec_vop);
                        
                        if (*error_flag == RETURN)
                            exit(1);
                    }
                    
                    QP =vop_quant;
                    
                    
                    if (skipped_flag == 1)
                        /* (skipped_flag==1) Clear the skipped MB */
                        MBzero (mblock);
                }
                else 
                {                       /* transparent MB */
                    MBzero (mblock);
                    /* set all alpha block pixels to zero */
                    /* Fill DC_store with default coeff values */
#ifndef _OPTIMIZE_
                    for (m = 0; m < 6; m++) 
                    {
                        DC_store[mbnum][m][0] = 128*8;
                        for (n = 1; n < 15; n++)
                            DC_store[mbnum][m][n] = 0;
                    }
#else
                    p = DC_store + mbnum*90; /* 15*6 */
                    memset(p, 0, 15*6*sizeof(Int));
                    for (m = 0; m < 6; m++) 
                    {
                        *(p + m*15) = 1024; /* 128*8 */;                    
                    }
#endif                    
                }
                
#if (__SPRITES__)
                if (GetVopSpriteUsage(rec_vop)==STATIC_SPRITE&&
                    tab_transmit[y_pos*numblocks_x+x_pos] ==0)
                    for (comp=0 ; comp<6 ; comp++)
                        for (u=0 ; u<15 ; u++)
                            tab_DC_store[y_pos*numblocks_x+x_pos][comp][u] = 
                                DC_store[mbnum][comp][u];
#endif
                
                if (GetVopSpriteUsage(rec_vop)!=STATIC_SPRITE
#if (__SPRITES__)
                    || tab_transmit[y_pos*numblocks_x+x_pos] ==0
#endif
                    )
                {
#ifndef _OPTIMIZE_
                    if(amb_type != 1 && skipped_flag == 0 && coded)
                        FillVop (mblock, mbnum, Y_rec, U_rec, V_rec);
#endif   
                }
                
#if (__SPRITES__)
                if (GetVopSpriteUsage(rec_vop)==STATIC_SPRITE)
                    if (tab_transmit[y_pos*numblocks_x+x_pos]==0)
                        tab_transmit[y_pos*numblocks_x+x_pos] = 1;
#endif                
            } /* BSO_NOEL */
            else 
            {
                if (trace->MB_sdata)
                    PrintMBShapeData(rec_vop,x_pos,y_pos,trace);
            }
        }
    
    /* Free allocated memory for 3D matrix */
    
#ifndef _OPTIMIZE_
    for (i = 0; i < MB_in_width*MB_in_height; i++)
    {
        for (j = 0; j < 6; j++) 
            free((Char *)DC_store[i][j]);
        free((Char *)DC_store[i]);
    }
    free((Char *)DC_store);
#endif
    
    FreeImage(MB_decisions);
    free ((Char *)mblock);
    
    return;
}


/***********************************************************CommentBegin******
 *
 * -- DecodeVopCombinedMotionShapeTextureInter -- 
 *
 * Author :             
 *      Jan De Lameillieure (HHI) <jan@hhi.de>
 *
 * Created :            
 *      19-Jan-97
 *
 * Purpose :            
 *      Decodes combined motion / shape / texture for INTER VOPs
 *
 * Arguments in :       
 *      Bitstream *stream       combined shape / texture bitstream
 *      Trace  *trace           flag for tracing the process.
 *
 *
 * Arguments in/out :   
 *      Vop *rec_vop            alpha plane is supposed to be filled
 *                              already / YUV components.
 *
 * Arguments out :      
 *
 *
 * Return values :      
 *      
 *
 * Side effects :       
 *      
 *
 * Description :        
 *
 *
 * Note : 
 *      Starting point for this function was 
 *      DecodeVopCombinedShapeTextureIntra() as of 18-Jan-97
 *
 * See also :
 *      
 *
 * Modified : 
 *      24-Jan-97 Jan De Lameillieure : moved alpha clampling after motion 
 *                                      compensation inside the MB loop
 *      07-Feb-1997, M.Wollborn: changed in order to treat skipped MBs
 *                               correctly
 *      30.07.97 F. Jaureguizar: changed in order to treat spatial
 *          scalability (SpSc)
 * 05.09.97 Fernando Jaureguizar: deleted unnecessary code with aux_vop to
 *          solve a memory leak
 *      04.11.97 Minhua Zhou: updated DC/AC prediction
 *
 ***********************************************************CommentEnd********/
#ifndef _OPTIMIZE_
Void
CallingConvention
DecodeVopCombinedMotionShapeTextureInter(       
		Bitstream * stream,
        Int vo_id,
        Trace *trace,
        Vop *rec_vop,
        Image **mot_x_out,
        Image **mot_y_out,
        Image **MB_decisions_out,
		UChar **MB_TYPE2[MAX_NUM_VOS][MAX_NUM_VOLS],
		Int PRE_H_SIZE2[MAX_NUM_VOS][MAX_NUM_VOLS],
		Int PRE_V_SIZE2[MAX_NUM_VOS][MAX_NUM_VOLS],
		UChar **PRE_MB_TYPE2[MAX_NUM_VOS][MAX_NUM_VOLS],
		Image   *alpha_rec_packet,
		Image   *alpha_mode_packet)
{
    Int         vol_id = GetVopId(rec_vop);
    Int         ef=0, *error_flag = &ef;
    Int         mbnum, x_pos, y_pos;
    Int         i, j, k, a;
    Int         rec_height;
    Int         rec_width, rec_prev_width;
    Int         MB_in_height, MB_in_width;
    Int         B_in_width, B_in_height;
    Macroblock  *mblock;
    Image       *Y_rec, *U_rec, *V_rec;
    Image       *Y_rec_prev, *U_rec_prev, *V_rec_prev;
    Short       *alpha, *alpha_prev;
    Int         vop_quant,intra_dcpred_disable;
    Int         skipped_flag=0;
    Int         boundary_flag;
    Image       *alpha_decisions,*MB_decisions;
    UInt        new_vop;
    SInt        *palpha_decisions,*pMB_decisions;
    Int         MB_transp_pattern[4];
#ifndef _OPTIMIZE_
    Int         ***DC_store;
#else
    Int         *DC_store;
    Int         *p;
    Short       *ps;
#endif
    
    Int         m, n, QP;
    Int         coded;
    UInt        CBP;
    UInt        *CBP_store;
    
    Int         ACpred_flag=-1;
    UInt        mode;
    Int         f_code=GetVopFCodeFor(rec_vop);
    Image       *mot_x, *mot_y;
    Float       *motx_ptr, *moty_ptr;
    Vop         *tmp_vop;
    SInt        *QP_store;
    /*>SpSc*/
    Int         spatial_scalability=0;
    Int         scalability  = GetVopScalability(rec_vop);
    Int         ref_sel_code = GetVopRefSelCode(rec_vop);
    Int         MB_non_trans_count=0;     
    UChar       *skipped;    
    /*SpSc<*/
#ifdef _DEBUG_SPSC_
/*ok*/    static Int count=0;
    Char       name[81];
#endif
    
    /*****
     *
     *   Stuff for motion compensation of alpha plane (copied by JDL 
     *   from AlphaMotionCompensate() in vm_common/src/mot_comp.c,
     *   a function written by Noel Brady
     *
     *****/
    Image       *motA2_x=NULL,*motA2_y=NULL,*modeA2=NULL;
    Int         amb_type=0,mvda=0;
    
    Int         rectangular_vop = !(GetVopShape(rec_vop));
    
    Vop         *sprite=NULL;
    Vop         *rec_prev;
    Int         ox, oy;
    
    
    
#if (__SPRITES__)
/* Online SPRITE */     
/* Low Latency Static SPRITE */ 
    Int *tab_transmit,numblocks_x, *tab_amb_type,*tab_mvda;
    
    if (GetVopSpriteUsage(rec_vop)==STATIC_SPRITE)
    { 
        tab_transmit = GetVopSpriteTabTransmit(rec_vop);
        tab_amb_type = GetVopSpriteTabAmbType(rec_vop);
        tab_mvda = GetVopSpriteTabMvda(rec_vop);
        numblocks_x = GetVopSpriteHdim(rec_vop)/16;
    }
    
    if (GetVopSpriteUsage(rec_vop)!=SPRITE_NOT_USED)
    { 
        sprite = GetVopSprite(rec_vop);
    }
#endif    
    
    /*****
     *
     *  Set some useful parameters
     *
     *****/
    rec_height = GetVopHeight (rec_vop);
    rec_width = GetVopWidth (rec_vop);
    MB_in_height = rec_height>>4;
    MB_in_width = rec_width>>4;
    B_in_width = MB_in_width + MB_in_width;
    B_in_height = MB_in_height + MB_in_height;
    QP_store = (SInt *) GetImageData(GetVopQP(rec_vop));
    /*>SpSc*/
    /* P-VOP with Sp.Sc. => ref_sel_code = 3 */
    if (scalability == 1 && ref_sel_code == 3)
        spatial_scalability = 1;
    /*SpSc<*/
    
    mot_x = AllocImage((UInt) B_in_width,(UInt) B_in_height,FLOAT_TYPE);
    mot_y = AllocImage((UInt) B_in_width,(UInt) B_in_height,FLOAT_TYPE);
    motx_ptr = (Float *)GetImageData(mot_x);
    moty_ptr = (Float *)GetImageData(mot_y);
    
    ZeroImageF(mot_x);
    ZeroImageF(mot_y);
    
    
    skipped   = (UChar *)calloc(MB_in_width*MB_in_height,sizeof(UChar));
    CBP_store = (UInt  *)calloc(MB_in_width*MB_in_height,sizeof(UInt));
    
    
    if (GetVopShape(rec_vop))
    {
        /* Allocate shape ME/MC */
        modeA2 = AllocImage(GetVopWidth(rec_vop)>>4,
                            GetVopHeight(rec_vop)>>4,SHORT_TYPE);
        ZeroImageI(modeA2);
        
        motA2_x = AllocImage(GetVopWidth(rec_vop)>>4,
                             GetVopHeight(rec_vop)>>4,SHORT_TYPE);
        motA2_y = AllocImage(GetVopWidth(rec_vop)>>4,
                             GetVopHeight(rec_vop)>>4,SHORT_TYPE);
    }
    
    /* allocate space for 3D matrix to keep track of prediction values
       for DC/AC prediction */
    
#ifndef _OPTIMIZE_
    DC_store = (Int ***)calloc(MB_in_width*MB_in_height,
                               sizeof(Int **));
    for (i = 0; i < MB_in_width*MB_in_height; i++)
    {
        DC_store[i] = (Int **)calloc(6, sizeof(Int *));
        for (j = 0; j < 6; j++)
            DC_store[i][j] = (Int *)calloc(15, sizeof(Int));
    }
#else
    DC_store = Get_DC_store();
#endif
    
    intra_dcpred_disable = GetVopIntraACDCPredDisable(rec_vop);
    
    alpha_decisions = AllocImage(B_in_width,B_in_height,SHORT_TYPE);
    MB_decisions    = AllocImage(MB_in_width,MB_in_height,SHORT_TYPE);
    
    tmp_vop = AllocVop(GetVopWidth(rec_vop),GetVopHeight(rec_vop));
    CopyVopNonImageField(rec_vop,tmp_vop);
    
    /*****
     *
     *  Get the image fields of the VOP
     *
     *****/
    Y_rec = GetVopY (rec_vop);
    U_rec = GetVopU (rec_vop);
    V_rec = GetVopV (rec_vop);
    
    rec_prev = GetVopNextTemp(rec_vop);
    Y_rec_prev = GetVopY (rec_prev);
    U_rec_prev = GetVopU (rec_prev);
    V_rec_prev = GetVopV (rec_prev);
    rec_prev_width = GetVopWidth (rec_prev);    
    
    
    /*
      SetConstantImage(Y_rec, 0);
      SetConstantImage(U_rec, 0);
      SetConstantImage(V_rec, 0);
      */
    
    alpha = (Short *) GetImageData( GetVopA(rec_vop) );
    /* for inter shape */
    alpha_prev = (Short *) GetImageData( GetVopA( GetVopNextTemp(rec_vop) ) ); 
    
    /*****
     *
     *  Get VOP quantizer and allocate MB structure
     *
     *****/
    vop_quant = GetVopQuantizer (rec_vop);
    QP = vop_quant;
    
    mblock = (Macroblock *) malloc (sizeof (Macroblock));
    
    if (mblock == NULL)
    {
        printf (" Error allocating macroblock\n");
        exit(1);
    }
    
/*>SpSc*/
    if (scalability == 1)
        printf("Enhancement Layer: \n");
/*SpSc<*/
    
#ifdef __WRITE_IM__    
    if (GetVopSpriteUsage(rec_vop)==ONLINE_SPRITE)
        printf("Start SPRITE VOP decoding\n");
    else
        if (GetVopSpriteUsage(rec_vop)==GMC_SPRITE)
            printf("Start GMC VOP decoding\n");
        else
            if (GetVopSpriteUsage(rec_vop)==STATIC_SPRITE)
                printf("Start P VOP decoding (update-piece of static sprite)\n");
            else
                printf("Start P VOP decoding\n");
#endif
    
    
    /* If rectangular VOP
       amb_type = 0;
       MB_transp_pattern = 0;
       Alpha = BINARY_ALPHA;
       alpha_decisions = 0;
       */
    
    if(rectangular_vop)
    {
        MB_transp_pattern[0]=0;
        MB_transp_pattern[1]=0;
        MB_transp_pattern[2]=0;
        MB_transp_pattern[3]=0;
        
        /* Already done before */
        /*memset(alpha, BINARY_ALPHA, (rec_width*rec_height)*sizeof(SInt));*/
        
        SetConstantImageI(alpha_decisions, 0);
    }
    
    SetConstantImageI(MB_decisions, 0);
    
    /*****
     *
     *  Loop over all MBs, decode texture of each MB
     *
     *****/
    new_vop = 1;
    pMB_decisions = (SInt *)GetImageData(MB_decisions);
    palpha_decisions = (SInt *)GetImageData(alpha_decisions);
    
    if(!rectangular_vop)
    {
        /* initialise values that indicate blockwise whether */
        /* the block is transparent                          */
#ifndef _OPTIMIZE_
        memset(palpha_decisions, MBM_TRANSPARENT, (B_in_width*B_in_height)*sizeof(SInt));
#else
        for (i = 0; i < B_in_width*B_in_height; i++)
            palpha_decisions[i] = MBM_TRANSPARENT;
#endif
    }
    
    
    /*****
     *
     * the macroblock loop
     *
     *****/
    
    for (y_pos = 0; y_pos < MB_in_height; y_pos++)
        for (x_pos = 0; x_pos < MB_in_width; x_pos++)
        {
            mbnum = y_pos * MB_in_width + x_pos;
            
            /* Store the QP value for later use in AC prediction */
            QP_store[mbnum] = (SInt) QP;
            
#ifdef _DEBUG
            if (GetVopShape(rec_vop) != BINARY_SHAPE_ONLY)
            {
                if (MB_non_trans_count>=MB_trace_thres)
                    SetTraceLevelsZero(trace);
                else
                    SetTraceLevelsDefault(trace);
            }
            
            
            trace->trace = trace->MB_header;
            
            if (trace->trace)
            {
                fprintf (trace->fp_trace,"\n>>>   Decoding Macroblock %d ........- Combined Motion/Shape/Text\n",(int)mbnum);
            }
#endif
            
            if (GetVopShape(rec_vop) && GetVopSpriteUsage(rec_vop)!=STATIC_SPRITE) 
            {
#ifdef _DEBUG
                trace->trace = trace->MB_shape;
#endif
                DecodeFirst(stream, vo_id, vol_id, y_pos, x_pos, &amb_type, 
                            &mvda, 1, 0, rec_vop, trace,MB_TYPE2,PRE_H_SIZE2,PRE_V_SIZE2,
							PRE_MB_TYPE2);
                
                boundary_flag = 0;
                if(amb_type >= 2)
                    boundary_flag = 1;
                
                
                /* fill the alpha map with the decoded pixels for this MB */
                AlphaDecodeMB(rec_vop,GetVopNextTemp(rec_vop),vo_id,vol_id,y_pos,x_pos,
                              &amb_type,&mvda,1,MB_decisions,alpha_decisions,mot_x,mot_y,modeA2,
                              motA2_x,motA2_y,0,stream,trace,alpha_rec_packet,alpha_mode_packet);
                
#ifdef _DEBUG
                if(trace->trace)
                    fprintf(trace->fp_trace, "----------\n");
                trace->trace = trace->MB_header;
#endif
                
                if (amb_type == 1) 
                {
                    /* if the alpha MV is all_0 */
                    /* for a correct motion vector prediction in the MB's */
                    /* below the actual MB   --- JDL                      */
                    i = (y_pos<<1)*B_in_width + (x_pos<<1);
                    palpha_decisions[i] = MBM_TRANSPARENT;
                    palpha_decisions[i + 1] = MBM_TRANSPARENT;
                    palpha_decisions[i + B_in_width] = MBM_TRANSPARENT;
                    palpha_decisions[i + B_in_width + 1] = MBM_TRANSPARENT;
                }
            }
            
#if (__SPRITES__)
            if (GetVopSpriteUsage(rec_vop)==STATIC_SPRITE)
            {
                if (GetVopShape(rec_vop)) 
                {
                    amb_type = tab_amb_type[y_pos*numblocks_x+x_pos];
                    mvda = tab_mvda[y_pos*numblocks_x+x_pos];
                    if (amb_type == 1) 
                    {   /* if the alpha MV is all_0 */
                        /* for a correct motion vector prediction in the MB's */
                        /* below the actual MB   --- JDL                      */
                        palpha_decisions[2*y_pos*B_in_width + 2*x_pos] = MBM_TRANSPARENT;
                        palpha_decisions[2*y_pos*B_in_width + 2*x_pos + 1] = MBM_TRANSPARENT;
                        palpha_decisions[(2*y_pos + 1)*B_in_width + 2*x_pos] = MBM_TRANSPARENT;
                        palpha_decisions[(2*y_pos + 1)*B_in_width + 2*x_pos + 1] = MBM_TRANSPARENT;
                    }
                }
            }
#endif            
            
            if (GetVopShape(rec_vop) != BINARY_SHAPE_ONLY)                      /* BSO_NOEL */
            {    
                /* decode COD, MCPC, ACpred_flag, CPBY and DQUANT */
                coded = 0;
                skipped_flag = 1;
                
                if (amb_type != 1)  /* if not transparent */
                {                        
                    if(!rectangular_vop)
                    {
                        MB_transp_pattern[0]=1;
                        MB_transp_pattern[1]=1;
                        MB_transp_pattern[2]=1;
                        MB_transp_pattern[3]=1;
                        
                        ps = alpha + (y_pos * MB_SIZE) *rec_width + (x_pos * MB_SIZE);
                        for(i=0;i<B_SIZE;i++)
                            for(j=0;j<B_SIZE;j++)
                                if (*(ps + i*rec_width + j))
                                {
                                    MB_transp_pattern[0]=0;
                                    goto out1;
                                }
                      out1:
                        ps = alpha + (y_pos * MB_SIZE) *rec_width + (x_pos * MB_SIZE);
                        for(i=0;i<B_SIZE;i++)           
                            for(j=B_SIZE;j<2*B_SIZE;j++)
                                if (*(ps + i*rec_width + j))                                
                                {
                                    MB_transp_pattern[1]=0;
                                    goto out2;
                                }
                      out2:
                        ps = alpha + (y_pos * MB_SIZE) *rec_width + (x_pos * MB_SIZE);
                        for(i=B_SIZE;i<2*B_SIZE;i++)
                            for(j=0;j<B_SIZE;j++)
                                if (*(ps + i*rec_width + j))                                
                                {
                                    MB_transp_pattern[2]=0;
                                    goto out3;
                                }
                      out3:
                        ps = alpha + (y_pos * MB_SIZE) *rec_width + (x_pos * MB_SIZE);
                        for(i=B_SIZE;i<2*B_SIZE;i++)
                            for(j=B_SIZE;j<2*B_SIZE;j++)
                                if (*(ps + i*rec_width + j))                                
                                {
                                    MB_transp_pattern[3]=0;
                                    goto out4;
                                }
                        
                      out4:
                        i=0; /* ???? */
                    }
                    
                    /* set MB_decisions for this MB to MBM_SKIPPED => mot. comp. */ 
                    /* with zero vector when MB skipped                         */
                    /* *((SInt *) GetImageData (MB_decisions) + mbnum) = MBM_SKIPPED; */
#if (__SPRITES__)
                    if (GetVopPredictionType(rec_vop) == SPRITE_VOP)
                        /* set MB_decisions for this MB to MBM_SPRITE => mot. comp. with zero vector when MB skipped */
                        *((SInt *) GetImageData (MB_decisions) + mbnum) = MBM_SPRITE;
                    else   /* P_VOP */
                        /* Temporarily changed to ONEVEC. RID 970211. */
#endif                        
                        *((SInt *) GetImageData (MB_decisions) + mbnum) = MBM_INTER16;
                    
                    GetMBheader(GetVopPredictionType(rec_vop),&new_vop,
                                intra_dcpred_disable,
                                &vop_quant,stream,mbnum,trace,MB_decisions,
                                mblock,&skipped_flag, error_flag,
                                DC_store, QP_store, QP, &coded, 
                                &CBP, &ACpred_flag, &mode,MB_transp_pattern);
                    
                    skipped[mbnum] = skipped_flag;
                    if(skipped_flag)
                        *((SInt *) GetImageData (MB_decisions) + mbnum) = MBM_SKIPPED;
                    
                    QP_store[mbnum]  = (SInt) vop_quant;
                    CBP_store[mbnum] = (UInt) CBP;
                    
#if (__SPRITES__)
                    if (GetVopSpriteUsage(rec_vop)== STATIC_SPRITE)
                        if (coded==1)
                            tab_transmit[y_pos*numblocks_x+x_pos]=2;
                        else
                            tab_transmit[y_pos*numblocks_x+x_pos]=1;
#endif                    
                    if (*error_flag == RETURN)
                        exit(1);
                }
                
                if(!rectangular_vop)
                {
                    /* if the MB is multilevel */
                    if ((amb_type != 0) || (amb_type != 1))
                        SetAlphaDecisions(GetVopA(rec_vop),x_pos,y_pos,alpha_decisions);
                }
                
                /* decode the MV for texture */
                if (amb_type != 1)  /* if not transparent */
                {
                    MB_non_trans_count++;
                    if ((coded) && (mode == MODE_INTER || mode == MODE_INTER_Q || 
                                    mode == MODE_INTER4V || mode == MODE_DYN_SP) &&
                        !spatial_scalability /*SpSc*/&&
                        GetVopSpriteUsage(rec_vop)!=STATIC_SPRITE
                        )
                        GetMBvectors(GetVopPredictionType(rec_vop),
                                     intra_dcpred_disable, &vop_quant,stream,mbnum,
                                     MB_in_width,f_code,trace,MB_decisions,
                                     alpha_decisions,mot_x,mot_y,&skipped_flag, 
                                     error_flag,DC_store, QP_store, QP, &coded,
                                     &ACpred_flag, mode);
                }
                
                /* calculate MB_transp_pattern */
                
                if(!rectangular_vop)
                {
#ifndef _OPTIMIZE_
                    if (amb_type == 0) /* alpha MB is all_255 */
                        for(i=0;i<4;i++) 
                            MB_transp_pattern[i]=0;
                    else if (amb_type==1) 
                        for(i=0;i<4;i++) 
                            MB_transp_pattern[i]=1;
#else
                    MB_transp_pattern[0]=amb_type;
                    MB_transp_pattern[1]=amb_type;
                    MB_transp_pattern[2]=amb_type;
                    MB_transp_pattern[3]=amb_type;
#endif
                }
                
                
                /* decode the DCT coeficients for the MB */
                if (amb_type!=1 && coded)
                {
                    GetMBblockdata(MB_transp_pattern,
                                   intra_dcpred_disable,
                                   vop_quant,stream,mbnum,trace,
                                   mblock,&skipped_flag, error_flag, 
                                   MB_in_width, 
                                   DC_store, QP_store, QP,
                                   CBP, ACpred_flag, mode,
                                   alpha, rec_vop);
                    QP = vop_quant;
                    if (*error_flag == RETURN)
                        exit(1);
                    
                    /*FillVop (mblock, mbnum, Y_rec, U_rec, V_rec);*/
                }      
                
                /*if (skipped_flag == 1)*/
                    /*    MBzero (mblock);*/
                
                if (amb_type == 1) /* transparent MB */
                {                       
                    /*MBzero (mblock);*/
                    /* Fill DC_store with default coeff values */
#ifndef _OPTIMIZE_
                    for (m = 0; m < 6; m++) 
                    {
                        DC_store[mbnum][m][0] = 128*8;
                        for (n = 1; n < 15; n++)
                            DC_store[mbnum][m][n] = 0;
                    }
#else
                    p = DC_store + mbnum*90; /* 15*6 */
                    memset(p, 0, 15*6*sizeof(Int));
                    for (m = 0; m < 6; m++) 
                    {
                        *(p + m*15) = 1024; /* 128*8 */;                    
                    }
#endif                    
                }
                
                
                if(!rectangular_vop)
                {
                    if(skipped_flag && boundary_flag)
                    {
                        /* boundary block skipped */
                        /* Only the shape changed */
                        /* rec_vop = alpha * prev */
                        ox = GetVopHorSpatRef(rec_vop) - GetVopHorSpatRef(rec_prev);
                        oy = GetVopVerSpatRef(rec_vop) - GetVopVerSpatRef(rec_prev);
                        
                        k = (y_pos * MB_SIZE)      * rec_width      + x_pos * MB_SIZE;
                        n = (y_pos * MB_SIZE + oy) * rec_prev_width + x_pos * MB_SIZE + ox;
                        
                        for(i=0;i<MB_SIZE;i++)
                        {
                            for(j=0;j<MB_SIZE;j++)
                            {
                                m = i*rec_width + j;
                                
                                if(alpha[k + m])
                                    Y_rec->f[k + m] = Y_rec_prev->f[n + i*rec_prev_width + j];
                            }
                        }
                        
                        
                        a  = (y_pos * B_SIZE)      * rec_width/2      + x_pos * B_SIZE;
                        n  = (y_pos * B_SIZE + oy/2) * rec_prev_width/2 + x_pos * B_SIZE + ox/2;
                        
                        for(i=0;i<B_SIZE;i++)
                        {
                            for(j=0;j<B_SIZE;j++)
                            {
                                m = i*rec_width/2 + j;
                                
                                if(alpha[k + 2*i*rec_width])
                                {
                                    U_rec->f[a + m] = U_rec_prev->f[n + i*rec_prev_width/2 + j];
                                    V_rec->f[a + m] = V_rec_prev->f[n + i*rec_prev_width/2 + j];
                                }
                            }
                        }
                        
                    }
                }
                
                
                
                /*FillVop (mblock, mbnum, Y_rec, U_rec, V_rec);*/
            }                                                                                                                                                                                           /* BSO_NOEL */
            else 
            {
#ifdef _DEBUG
                if (trace->MB_sdata)
                    PrintMBShapeData(rec_vop, x_pos, y_pos, trace);
#endif
            }
        }
    
    
    
    if (GetVopShape(rec_vop) != BINARY_SHAPE_ONLY)  /* BSO_NOEL */
    {
        
        /* mapping MB_decisions to alpha_decisions in order to make */
        /* VopMotionCompensate() work                               */
        
        RemapAlphaDecisions(alpha_decisions);
        
        
        VopMotionCompensate(GetVopNextTemp(rec_vop),mot_x,mot_y,MB_decisions,
                            alpha_decisions, CBP_store, tmp_vop, 
                            GetVopOBMCDisable(rec_vop),
                            sprite); 
        
        
#ifdef _DEBUG_SPSC2_
        if(spatial_scalability) {
            if(tmp_cnt==0) {
                WriteVopGeneric( tmp_vop, "Dcomp_vop", "Dcomp_vop", "Dcomp_vop",
                                 "tmpppu", 0, IO_FORMAT, IO_OVERWRITE, 0);
                WriteVopGeneric( rec_vop, "Drec_vop", "Drec_vop", "Drec_vop",
                                 "tmpppu", 0, IO_FORMAT, IO_OVERWRITE, 0);
            } else {
                WriteVopGeneric( tmp_vop, "Dcomp_vop", "Dcomp_vop", "Dcomp_vop",
                                 "tmpppu", tmp_cnt, IO_FORMAT, IO_APPEND, 0);
                WriteVopGeneric( rec_vop, "Drec_vop", "Drec_vop", "Drec_vop",
                                 "tmpppu", tmp_cnt, IO_FORMAT, IO_APPEND, 0);
            }
        }
#endif
        
        
#ifdef _DEBUG_SPSC_
/*** 8/18 TPS */
        if(!temporal_scalability && GetVopScalability(rec_vop))
/* 8/18 TPS ***/
        {
            sprintf(name,"-p-Pred+Comp%d",count);
            WriteVopDebug(rec_vop,name,"","",0);
            sprintf(name,"-p-Pred+Comp.N.%d",count);
            WriteVopDebug(GetVopNextTemp(rec_vop),name,"","",0);
            sprintf(name,"-p-Pred+Comp.P.%d",count);
            WriteVopDebug(GetVopPrevTemp(rec_vop),name,"","",0);
            
            sprintf(name,"-p-XXXX+Comp%d",count);
            WriteVopDebug(tmp_vop,name,"","",0);
        }
#endif
        
        VopTextureUpdate(tmp_vop,rec_vop,trace);
        
#ifdef _DEBUG_SPSC2_
        if(spatial_scalability) {
            if(tmp_cnt==0) {
                WriteVopGeneric( rec_vop, "D_vop", "D_vop", "D_vop",
                                 "tmpppu", 0, IO_FORMAT, IO_OVERWRITE, 0);
            } else {
                WriteVopGeneric( rec_vop, "D_vop", "D_vop", "D_vop",
                                 "tmpppu", tmp_cnt, IO_FORMAT, IO_APPEND, 0);
            }
            tmp_cnt++;
        }
#endif
        
        
#ifdef _DEBUG_SPSC_
/*** 8/18 TPS */
        if(!temporal_scalability && GetVopScalability(rec_vop))
/* 8/18 TPS ***/
        {
            sprintf(name,"-p-XXX+TxUpd%d",count);
            WriteVopDebug(rec_vop,name,"","",0);
            
            count++;
        }
#endif
    }                                   /* BSO_NOEL */
    
    /* Free allocated memory for 3D matrix */
    
#ifndef _OPTIMIZE_
    for (i = 0; i < MB_in_width*MB_in_height; i++)
    {
        for (j = 0; j < 6; j++)
            free((Char *)DC_store[i][j]);
        free((Char *)DC_store[i]);
    }
    free((Char *)DC_store);
#endif
    
    if (GetVopShape(rec_vop))
    {
        /* Deallocate shape ME/MC */
        /* Removed cast to (Char *), 05-FEB-1997 MW */
        FreeImage(modeA2);                        
        FreeImage(motA2_x);                       
        FreeImage(motA2_y);                       
    }
    
    FreeImage(alpha_decisions);
    free ((Char *)mblock);
    *mot_x_out=mot_x;
    *mot_y_out=mot_y;
    
    for (i = 0; i < MB_in_width*MB_in_height; i++)
        if (skipped[i])
            pMB_decisions[i] = MBM_SKIPPED;
    
    free(skipped); 
    free(CBP_store); 
    *MB_decisions_out = MB_decisions;
    FreeVop(tmp_vop);
    
#ifdef _WD_MODE_
    FreeImage(mot_x);
    FreeImage(mot_y);
    FreeImage(MB_decisions);
#endif
    
    return;
}
#endif

Void
CallingConvention
RemapAlphaDecisions(Image *alpha)
{
    SInt *p = (SInt *) GetImageData(alpha),
        *pe = p + GetImageSize(alpha);
    
    while (p<pe)
    {
        if (*p == 0)
            *p = 1;
        if (*p == 2)
            *p = MBM_TRANSPARENT; /*MBM_TRANSPARENT;*/ /*2*/
        p++;
    }
}


Void
CallingConvention
SetAlphaDecisions(      Image *alpha, Int mbx_pos, Int mby_pos,
                        Image *alpha_decisions)
{
    const Int bx_pos = mbx_pos + mbx_pos;
    const Int by_pos = mby_pos + mby_pos;
    const Int x_pos = mbx_pos<<4;
    const Int y_pos = mby_pos<<4;
    
    Int
        x,
        y,
        w2;
    
    const Int width_out = GetImageSizeX(alpha_decisions);
    const Int width_in = GetImageSizeX(alpha);
    
    SInt *p_out = (SInt *) GetImageData(alpha_decisions) 
        + width_out*by_pos +bx_pos;
    
    SInt *p_in_mb = (SInt *)GetImageData(alpha)+y_pos*width_in+x_pos,
        *p_in;
    
    w2 = width_in - B_SIZE;
    
    
    
    p_in = p_in_mb;
    
    /*white_pel = 0;*/
    *p_out = MBM_TRANSPARENT; /*apa*/
    
    for (y=0;y<B_SIZE;y++,p_in+=w2)
        for (x=0;x<B_SIZE;x++,p_in++)
            if (*p_in)
            {
                /*white_pel=1;*/
                *p_out = 0;
                goto out1;
            }
    
    
  out1:  
    p_in = p_in_mb+B_SIZE;
    p_out++;
    
    /*white_pel = 0;*/
    *p_out = MBM_TRANSPARENT; /*apa*/
    
    for (y=0;y<B_SIZE;y++,p_in+=w2)
        for (x=0;x<B_SIZE;x++,p_in++)
            if (*p_in)
            {
                /*white_pel=1;*/
                *p_out = 0;
                goto out2;
            }
    
  out2:
    p_in = p_in_mb + (width_in<<3);
    p_out--;p_out+=width_out;
    
    /*white_pel = 0;*/
    *p_out = MBM_TRANSPARENT; /*apa*/
    
    for (y=0;y<B_SIZE;y++,p_in+=w2)
        for (x=0;x<B_SIZE;x++,p_in++)
            if (*p_in)
            {
                /*white_pel=1;*/
                *p_out = 0;
                goto out3;
            }
    
  out3:
    p_out++;
    p_in = p_in_mb + (width_in<<3) + B_SIZE;
    
    /*white_pel = 0;*/
    *p_out = MBM_TRANSPARENT; /*apa*/
    
    for (y=0;y<B_SIZE;y++,p_in+=w2)
        for (x=0;x<B_SIZE;x++,p_in++)
            if (*p_in)
            {
                /*white_pel=1;*/
                *p_out = 0;
                goto out4;
            }
  out4:
    return;
}

/***********************************************************CommentBegin******
 *
 * -- GetMBheader --  
 *
 * Author :
 *      Jan De Lameillieure (HHI) <jan@hhi.de>
 *
 * Created :
 *      18-Jan-97
 *
 * Purpose :
 *       decodes the MCBPC, ACpred_flag, CBPY and DQUANT of a MB (for combined mode)
 *
 * Arguments in :
 *      Int vop_type,                   vop INTRA/INTER.
 *      Int intra_dcpred_disable        enable/disable pred. of INTRA DC
 *      Int *quant_step,                 vop quantizer step (used for all MBs).
 *      Bitstream * texture_bits,       Image made of texture parameters.
 *      Int mbnum,                      macroblock number.
 *      Int trace,                      flag for tracing the process.
 *      Int *motion_mode,               already filled by MotionDecode.
 *      Int DC_store[][6][15],          Stores DCAC prediction values
 *      SInt *QP_store,                 Stores QPs for every MB
 *      Int QP                          Current QPn
 *
 * Arguments in/out :
 *
 *
 * Arguments out :
 *      Macroblock *mblock,             decoded macroblock.
 *      Int *skipped_flag,              1 = macroblock is skipped.
 *      Int *error_flag,                set if an error occured.
 *      Int *coded,                     set if the MB contains coded texture
 *      UInt *CBP,                      Coded Block Pattern derived from MB header data
 *      Int *ACpred_flag                flag indicating the the use of AC prediction
 *      UInt *mode                      macroblock mode
 *
 * Return values :
 *
 *
 * Side effects :
 *
 *
 * Description :
 *      Starting point for this function was the first part of GetMBtext() of 
 *      the file text_decode_mb.c of 16-Jan-97
 *
 * See also :
 *
 *
 * Modified : 
 *           13.05.97 Minhua Zhou: added MB_transp_pattern
 *           12.08.97 Minhua Zhou: assigned skipped MB to mode=MODE_INTER
 *
 ***********************************************************CommentEnd********/


/*ok*/static Void
CallingConvention
GetMBheader(Int vop_type,
            UInt *new_vop,
            Int intra_dcpred_disable,
            Int *quant_step,
            Bitstream * texture_bits,
            Int mbnum,
            Trace *trace,
            Image * motion_mode,
            Macroblock * mblock,
            Int * skipped_flag,
            Int * error_flag,
            Int *DC_store,
            SInt *QP_store,
            Int QP,
            Int *coded,
            UInt *CBP,
            Int *ACpred_flag,
            UInt *mode,
            Int *MB_transp_pattern)
{
    UInt CBPY, CBPC;
    Int  MBtype;
    SInt *decis_ptr;
    Int  m;
    Int  MCBPC;
    Int  DQUANT;
    UInt no_dct_flag;
    Int  DQ_tab[4] = {-1, -2, 1, 2};
    Int  *p;
    
    Int flag;
    
    Int MCSEL;
    
    if(vop_type == INTRA_VOP)
        no_dct_flag = 0;
    else
        no_dct_flag = BitstreamReadBits (texture_bits, 1,"NO DCT FLAG", 
                                         trace, FLAG);
    
    decis_ptr = (SInt *) GetImageData (motion_mode);
    
    if (no_dct_flag)
    {
        /* skipped macroblock */
        *skipped_flag = 1;
        *coded=0;
        *mode=MODE_INTER; 
        /* Fill DC_store with default coeff values */
#ifndef _OPTIMIZE_
        for (m = 0; m < 6; m++)
        {
            DC_store[mbnum][m][0] = 128*8;
            for (n = 1; n < 15; n++)
                DC_store[mbnum][m][n] = 0;
        }
#else
        p = DC_store + mbnum*90; /* 15*6 */
        memset(p, 0, 15*6*sizeof(Int));
        for (m = 0; m < 6; m++) 
        {
            *(p + m*15) = 1024; /* 128*8 */;                    
        }
#endif                    
    }
    else
    {
        /* coded macroblock */
        *coded=1;
        *skipped_flag = 0;
        
        if (vop_type == INTRA_VOP)
            MCBPC = VlcDecMCBPC_com_intra (texture_bits, error_flag, trace);
        else
            MCBPC = VlcDecMCBPC_com_inter (texture_bits, error_flag, trace);
        
        if (*error_flag == RETURN)
        {
            printf ("Error decoding MCBPC of macroblock %d\n", (int)mbnum);
            return;
        }
        
        MBtype = MCBPC & 7;
        if ( MBtype == 0 ) *mode = MODE_INTER;
        if ( MBtype == 1 ) *mode = MODE_INTER_Q;
        if ( MBtype == 2 ) *mode = MODE_INTER4V;
        if ( MBtype == 3 ) *mode = MODE_INTRA;
        if ( MBtype == 4 ) *mode = MODE_INTRA_Q;
        
        /* MCSEL syntax */
        if (((*mode == MODE_INTER || *mode == MODE_INTER_Q)) && (vop_type == SPRITE_VOP))
        {
            MCSEL = BitstreamReadBits(texture_bits, 1, "MCSEL",
                                      trace, FLAG);
            
            if (MCSEL) {
                if (*mode == MODE_INTER)
                    *mode = MODE_DYN_SP;
                if (*mode == MODE_INTER_Q)
                    *mode = MODE_DYN_SP_Q;
            }
        }
        CBPC = (MCBPC >> 4) & 3;
        
        if ((*mode == MODE_INTRA || *mode == MODE_INTRA_Q) && intra_dcpred_disable == 0)
        {
            *ACpred_flag = BitstreamReadBits(texture_bits, 1, "ACpred_flag",
                                             trace, FLAG);
        }
        
        flag = (*mode == MODE_INTRA || *mode == MODE_INTRA_Q);
        CBPY = VlcDecCBPY (texture_bits,flag,
                           MB_transp_pattern, error_flag, trace);
        if (*error_flag == RETURN)
        {
            printf ("Error decoding CBPY of macroblock %d\n", (int)mbnum);
            return;
        }
        
        if (*mode == MODE_INTRA || *mode == MODE_INTRA_Q)
        {
            *(decis_ptr + mbnum) = MBM_INTRA; /* added 30-JAN-1997 MW */
        } else if (*mode==MODE_INTER4V) *(decis_ptr + mbnum) = MBM_INTER16;
        
        
        
        *CBP = CBPY << 2 | CBPC;
        
        if (*mode == MODE_INTER_Q || *mode == MODE_INTRA_Q|| *mode == MODE_DYN_SP_Q) {
            DQUANT = BitstreamReadBits(texture_bits, 2, "DQUANT", trace, NUM);
            *quant_step = *quant_step + DQ_tab[DQUANT];
            
#ifdef _DEBUG
            if (trace->trace)
                fprintf (trace->fp_trace, "DQUANT = %d   QUANT = %d\n",
                         (int)DQ_tab[DQUANT], (int)*quant_step);
#endif        
            
            if (*quant_step > 31 || *quant_step < 1)
            {
#ifdef _DEBUG
                if (trace->trace)
                    fprintf(trace->fp_trace,"Quantizer out of range %d:clipping\n"
                            , (int)*quant_step);
#endif        
                
#ifdef _OPTIMIZE_
                *quant_step = (*quant_step > 31 ? 31 : *quant_step);
                *quant_step = (*quant_step > 1 ? *quant_step : 1);
#else                
                *quant_step = mmax (1, mmin (31, (*quant_step)));
#endif                
                
            }
        }
        
#ifdef _DEBUG
        trace->trace = trace->MB_texture;
        if(trace->trace)
            fprintf(trace->fp_trace, "----------\n");
#endif        
    }
}


/***********************************************************CommentBegin******
 *
 * -- GetMBvectors --  
 *
 * Author :
 *      Jan De Lameillieure (HHI) <jan@hhi.de>
 *
 * Created :
 *      19-Jan-97
 *
 * Purpose :
 *       decodes the vectors of a MB (for combined mode)
 *
 * Arguments in :
 *      Int vop_type,                   vop INTRA/INTER.
 *      Int intra_dcpred_disable        enable/disable pred. of INTRA DC
 *      Int *quant_step,                 vop quantizer step (used for all MBs).
 *      Bitstream * texture_bits,       Image made of texture parameters.
 *      Int mbnum,                      macroblock number.
 *      Int trace,                      flag for tracing the process.
 *      Int *motion_mode,               already filled by MotionDecode.
 *      Int DC_store[][6][15],          Stores DCAC prediction values
 *      SInt *QP_store,                 Stores QPs for every MB
 *      Int QP                          Current QPn
 *
 * Arguments in/out :
 *
 *
 * Arguments out :
 *      Int *skipped_flag,              1 = macroblock is skipped.
 *      Int *error_flag,                set if an error occured.
 *      Int *coded,                     set if the MB contains coded texture
 *      Int *ACpred_flag                flag indicating the the use of AC prediction
 *      UInt *mode                      macroblock mode
 *
 * Return values :
 *
 *
 * Side effects :
 *
 *
 * Description :
 *      Starting point for this function was the first part of GetMBmottext() of 
 *      the file text_decode_mb.c of 16-Jan-97
 *
 * See also :
 *
 *
 * Modified : 08.10.1997 Minhua Zhou: added vector padding 
 *
 ***********************************************************CommentEnd********/

/*ok*/static Void
CallingConvention
GetMBvectors (Int vop_type,
              Int intra_dcpred_disable,
              Int *quant_step,
              Bitstream * bits,
              Int mbnum,
              Int MB_width,
              Int f_code,
              Trace *trace,
              Image * motion_mode,
              Image * block_decis,
              Image * mot_x,
              Image * mot_y,
              Int * skipped_flag,
              Int * error_flag,
              Int *DC_store,
              SInt *QP_store,
              Int QP,
              Int *coded,
              Int *ACpred_flag,
              UInt mode)
{
    Int         k;
    SInt        *decis_ptr, *bdecis_ptr;
    Int         x_pos = mbnum % MB_width;
    Int         y_pos = mbnum / MB_width;
    Float       *motx_ptr, *moty_ptr, mvx=0, mvy=0;
    Int         B_in_width = MB_width<<1;
    
    decis_ptr = (SInt *) GetImageData (motion_mode);
    bdecis_ptr = (SInt *) GetImageData (block_decis);
    motx_ptr = (Float *) GetImageData (mot_x);
    moty_ptr = (Float *) GetImageData (mot_y);
    
    
    /* Obtaining whether a block is transparent */
    /* #define B_TRANSP(p,hdim,h,v,h2,v2,value)\
       ((p)[(2*(v)+(v2))*(hdim)+2*(h)+(h2)]==(value))
       */
    
    
    /* Decoding of motion vectors */
    
    if (mode == MODE_INTER || mode == MODE_INTER_Q || mode == MODE_INTER4V)
    {
        if (mode == MODE_INTER4V)
        {
            for (k = 0; k < 4; k++)
            {
                if (!B_TRANSP (bdecis_ptr,B_in_width,x_pos,y_pos,k & 0x1,k >> 1,MBM_TRANSPARENT))
                {
                    *(decis_ptr+mbnum) = FOURVEC;
/* MVP/Noel */
                    DecodeMBVec (mot_x, mot_y, motion_mode, block_decis, 
                                 bits, trace, x_pos, y_pos, 
                                 k + 1, f_code, 1, 0, error_flag, &mvx, &mvy, (Int **)NULL);
                    
                    if (*error_flag == RETURN)
                        return;
                    
                    BV (motx_ptr, B_in_width, x_pos, y_pos, k & 0x1, k >> 1) = mvx;
                    BV (moty_ptr, B_in_width, x_pos, y_pos, k & 0x1, k >> 1) = mvy;
                }
                else
                {
                    BV (motx_ptr, B_in_width, x_pos, y_pos, k & 0x1, k >> 1) = 0.0;
                    BV (moty_ptr, B_in_width, x_pos, y_pos, k & 0x1, k >> 1) = 0.0;
                }
#ifdef _DEBUG
                if (trace->trace)
                {
                    fprintf (trace->fp_trace, "  >> mvx: %g\n", mvx);
                    fprintf (trace->fp_trace, "  >> mvy: %g\n", mvy);
                }
#endif
            }
            
/* VPT / Minhua */
            if (B_TRANSP(bdecis_ptr, B_in_width, x_pos, y_pos, 0, 0, MBM_TRANSPARENT)) {
                if (!B_TRANSP(bdecis_ptr, B_in_width, x_pos, y_pos, 1, 0, MBM_TRANSPARENT)) {
                    BV(motx_ptr, B_in_width, x_pos, y_pos, 0, 0) = BV(motx_ptr, B_in_width, x_pos, y_pos, 1, 0); 
                    BV(moty_ptr, B_in_width, x_pos, y_pos, 0, 0) = BV(moty_ptr, B_in_width, x_pos, y_pos, 1, 0); 
                } else {
                    BV(motx_ptr, B_in_width, x_pos, y_pos, 0, 0) = BV(motx_ptr, B_in_width, x_pos, y_pos, 0, 1); 
                    BV(moty_ptr, B_in_width, x_pos, y_pos, 0, 0) = BV(moty_ptr, B_in_width, x_pos, y_pos, 0, 1); 
                }
            }
            if (B_TRANSP(bdecis_ptr, B_in_width, x_pos, y_pos, 1, 0, MBM_TRANSPARENT)) {
                if (!B_TRANSP(bdecis_ptr, B_in_width, x_pos, y_pos, 0, 0, MBM_TRANSPARENT)) {
                    BV(motx_ptr, B_in_width, x_pos, y_pos, 1, 0) = BV(motx_ptr, B_in_width, x_pos, y_pos, 0, 0); 
                    BV(moty_ptr, B_in_width, x_pos, y_pos, 1, 0) = BV(moty_ptr, B_in_width, x_pos, y_pos, 0, 0); 
                } else {
                    BV(motx_ptr, B_in_width, x_pos, y_pos, 1, 0) = BV(motx_ptr, B_in_width, x_pos, y_pos, 1, 1); 
                    BV(moty_ptr, B_in_width, x_pos, y_pos, 1, 0) = BV(moty_ptr, B_in_width, x_pos, y_pos, 1, 1); 
                }
            }
            if (B_TRANSP(bdecis_ptr, B_in_width, x_pos, y_pos, 0, 1, MBM_TRANSPARENT)) {
                if (!B_TRANSP(bdecis_ptr, B_in_width, x_pos, y_pos, 1, 1, MBM_TRANSPARENT)) {
                    BV(motx_ptr, B_in_width, x_pos, y_pos, 0, 1) = BV(motx_ptr, B_in_width, x_pos, y_pos, 1, 1); 
                    BV(moty_ptr, B_in_width, x_pos, y_pos, 0, 1) = BV(moty_ptr, B_in_width, x_pos, y_pos, 1, 1); 
                } else {
                    BV(motx_ptr, B_in_width, x_pos, y_pos, 0, 1) = BV(motx_ptr, B_in_width, x_pos, y_pos, 0, 0); 
                    BV(moty_ptr, B_in_width, x_pos, y_pos, 0, 1) = BV(moty_ptr, B_in_width, x_pos, y_pos, 0, 0); 
                }
            }
            if (B_TRANSP(bdecis_ptr, B_in_width, x_pos, y_pos, 1, 1, MBM_TRANSPARENT)) {
                if (!B_TRANSP(bdecis_ptr, B_in_width, x_pos, y_pos, 0, 1, MBM_TRANSPARENT)) {
                    BV(motx_ptr, B_in_width, x_pos, y_pos, 1, 1) = BV(motx_ptr, B_in_width, x_pos, y_pos, 0, 1); 
                    BV(moty_ptr, B_in_width, x_pos, y_pos, 1, 1) = BV(moty_ptr, B_in_width, x_pos, y_pos, 0, 1); 
                } else {
                    BV(motx_ptr, B_in_width, x_pos, y_pos, 1, 1) = BV(motx_ptr, B_in_width, x_pos, y_pos, 1, 0); 
                    BV(moty_ptr, B_in_width, x_pos, y_pos, 1, 1) = BV(moty_ptr, B_in_width, x_pos, y_pos, 1, 0); 
                }
            }
            
            
        }
        else
        {
/* MVP/Noel */
            DecodeMBVec (mot_x, mot_y, motion_mode, block_decis, 
                         bits, trace, x_pos, y_pos, 
                         0, f_code, 1, 0, error_flag, &mvx, &mvy, (Int **)NULL); 
            
            if (*error_flag == RETURN)
                return;
#ifdef _DEBUG
            if (trace->trace)
            {
                fprintf (trace->fp_trace, "  >> mvx: %g\n", mvx);
                fprintf (trace->fp_trace, "  >> mvy: %g\n", mvy);
            }
#endif
            for (k = 0; k < 4; k++)
            {
                BV (motx_ptr, B_in_width, x_pos, y_pos, k & 0x1, k >> 1) = mvx;
                BV (moty_ptr, B_in_width, x_pos, y_pos, k & 0x1, k >> 1) = mvy;
                *(decis_ptr + mbnum) = ONEVEC;
            }
        }
    }
    else   /* INTRA Macroblock or Sprite Macroblock*/
    {
        if (mode == MODE_INTRA) 
        {
            if (vop_type != INTRA)
                for (k = 0; k < 4; k++)
                {
                    BV (motx_ptr, B_in_width, x_pos, y_pos, k & 0x1, k >> 1) = 0.0;
                    BV (moty_ptr, B_in_width, x_pos, y_pos, k & 0x1, k >> 1) = 0.0;
                    *(decis_ptr + mbnum) = INTRA;
                }
        }
#ifndef _WD_MODE_
        else
            if (mode == MODE_DYN_SP)
            {
                for (k = 0; k < 4; k++)
                {
                    BV (motx_ptr, B_in_width, x_pos, y_pos, k & 0x1, k >> 1) = 0.0;
                    BV (moty_ptr, B_in_width, x_pos, y_pos, k & 0x1, k >> 1) = 0.0;
                    *(decis_ptr + mbnum) = MBM_SPRITE;
                }
            }
#endif        
    }
    
#ifdef _DEBUG
    trace->trace = trace->MB_texture;
    if(trace->trace)
        fprintf(trace->fp_trace, "----------\n");
#endif
}

/***********************************************************CommentBegin******
 *
 * -- GetMBblockdata --  
 *
 * Author :
 *      Jan De Lameillieure (HHI) <jan@hhi.de>
 *
 * Created :
 *      18-Jan-97
 *
 * Purpose :
 *       decodes the block data (DCT coefficientsm, alias "texture") of a MB (for combined mode)
 *
 * Arguments in :
 *      Int intra_dcpred_disable        enable/disable pred. of INTRA DC
 *      Int quant_step,                 vop quantizer step (used for all MBs).
 *      Bitstream * texture_bits,       Image made of texture parameters.
 *      Int mbnum,                      macroblock number.
 *      Int trace,                      flag for tracing the process.
 *      Int *motion_mode,               already filled by MotionDecode.
 *      Int MB_width,
 *      Int DC_store[][6][15],          Stores DCAC prediction values
 *      SInt *QP_store,                 Stores QPs for every MB
 *      Int QP                          Current QPn
 *      UInt CBP                        Coded Block Pattern derived from MB header data
 *      UChar disable_sadct             Flag indicating whether SADCT is disabled or not
 *      Short *alpha                    The object plane with alpha values
 *      UChar shape                     Indicator of shape type of the video object being decoded
 *
 * Arguments in/out :
 *
 *
 * Arguments out :
 *      Macroblock *mblock,             decoded macroblock.
 *      Int *skipped_flag,              1 = macroblock is skipped.
 *      Int *error_flag,                set if an error occured.
 *
 *
 * Return values :
 *
 *
 * Side effects :
 *
 *
 * Description :
 *      Starting point for this function was the second part of GetMBtext() of 
 *      the file text_decode_mb.c of 16-Jan-97
 *
 * See also :
 *
 *
 * Modified :
 *      28.01.97 Robert Danielsen: Changed zigzag-scanning.
 *      27.02.97 Robert Danielsen: Added parameter to signal luma or
 *                      chroma to the nonlinear quantization.
 *      15-APR-97 Jan De Lameillieure : called BlockIDCT() instead of idctref()
 *      15-APR-97 Jan De Lameillieure : added the inverse SADCT
 *      22-APR-97 Michael Wollborn: changed name of BlockDequant to 
 *                      BlockDequantH263 and changed
 *                      stuff for MPEG-like quantization
 *       14-05-97 Minhua Zhou: disable SADCT for rectangular VOP
 *      11-08-97 Minhua Zhou: added clipping for MODE_INTRA_Q mode
 *
 ***********************************************************CommentEnd********/
#if 0
Void
CallingConvention
GetMBblockdata( Int transp_pattern[],
                Int intra_dcpred_disable,
                Int quant_step,
                Bitstream * texture_bits,
                Int mbnum,
                Trace *trace,
                Macroblock * mblock,
                Int * skipped_flag,
                Int * error_flag,
                Int MB_width,
                Int *DC_store,
                SInt *QP_store,
                Int QP,
                UInt CBP,
                Int ACpred_flag,
                UInt mode,
                Short *alpha,
                Vop *rec_vop)
{
    Image       *Y_rec, *U_rec, *V_rec;
    
    Int  comp;
    Int  DCT_block[B_SIZE * B_SIZE];
    Int  q_block[B_SIZE * B_SIZE];
    Int  r_block[B_SIZE * B_SIZE];
    Int  r_block_2D[B_SIZE][B_SIZE];
    Int  pcoeff[B_SIZE * B_SIZE];
    UInt DC_coeff;
    Int  pred_DC_coeff;
    
    Int  x_pos = mbnum % MB_width;
    Int  y_pos = mbnum / MB_width;
    Int  x1, y1;
    Int  m, n;
    Int  i, j;
    Char level_3_trace;
    Int switched;
    
    UChar disable_sadct = GetVopSADCTDisable(rec_vop);
    UChar shape         = GetVopShape(rec_vop);
    
    /* variables because of contour-adaptive DCT */
    Short alpha_block[B_SIZE][B_SIZE];
    Short nrpix = MB_width * MB_SIZE;
    Short tmp; 
    SInt mb_shape[6][64],mb_dct_bf_iq[6][64],mb_dct_iq[6][64],mb_idct[6][64]; 
    
    Int last_non_null_coeff;
    /*****
     *
     *  Get the image fields of the VOP
     *
     *****/
    Y_rec = GetVopY (rec_vop);
    U_rec = GetVopU (rec_vop);
    V_rec = GetVopV (rec_vop);
    
    /*added by Minhua Zhou 04.08.97 */
    switched = IntraDCSwitch_Decision(mode,
                                      GetVopIntraDCVlcThr(rec_vop),
                                      QP);
    QP = quant_step;
    
#if 0
    /* Because of Bzero done always below*/
           MBzero (mblock);
#endif
    
#ifdef _DEBUG
    level_3_trace = trace->MB_sdata||trace->MB_tdata_before_IQ|| 
        trace->MB_tdata_after_IQ||trace->MB_tdata_after_IDCT||
        trace->MB_tdata_reconstructed;
    if (level_3_trace)
        for (i=0;i<6;i++)
            for (j=0;j<64;j++)
                mb_shape[i][j]=0;  
#endif
    
    if (shape==0)
        disable_sadct = 1;
    
    
#ifdef DEBUG_COMBINED
/* control print for JDL 18-Jan-97 */
    printf("transp_pattern[] = (%d,%d,%d,%d) \n", 
           transp_pattern[0], transp_pattern[1], transp_pattern[2], transp_pattern[3]);
#endif
    
    x1 = x_pos*MB_SIZE;
    y1 = y_pos*MB_SIZE;
    
    
    /*****
     *
     *    Decoding of the 4 luminance blocks (depending on transparent pattern)
     *
     *****/
    for (comp=0; comp<4; comp++)
    {        
        if(transp_pattern[comp]==0) /* No decoding of transparent Y blocks */
        {
#ifndef _WD_MODE_
/* alpha_block needed only for SADCT */
            if ((disable_sadct==0)||(level_3_trace))
            {
                switch(comp)
                {
                    case 0 :
                        for (i=0; i<B_SIZE; i++)
                            for (j=0; j<B_SIZE; j++)
                                alpha_block[i][j] = alpha[(y1 + i)*nrpix + (x1 + j)];
                        break;
                    case 1 :
                        for (i=0; i<B_SIZE; i++)
                            for (j=0; j<B_SIZE; j++)
                                alpha_block[i][j] = alpha[(y1 + i)*nrpix + (x1 + 8 + j)];
                        break;
                    case 2 :
                        for (i=0; i<B_SIZE; i++)
                            for (j=0; j<B_SIZE; j++)
                                alpha_block[i][j] = alpha[(y1 + 8 + i)*nrpix + (x1 + j)];
                        break;
                    case 3 :
                        for (i=0; i<B_SIZE; i++)
                            for (j=0; j<B_SIZE; j++)
                                alpha_block[i][j] = alpha[(y1 + 8 + i)*nrpix + (x1 + 8 + j)];
                        break;
                    case 4 :
                    case 5 :
                        for (i=0; i<B_SIZE; i++)
                            for (j=0; j<B_SIZE; j++)
                            {
                                if (shape == GREY_SCALE)
                                {
                                    tmp =  alpha[(y1 + 2*i)*nrpix   + (x1 + 2*j)];
                                    tmp += alpha[(y1 + 2*i)*nrpix   + (x1 + 2*j+1)];
                                    tmp += alpha[(y1 + 2*i+1)*nrpix + (x1 + 2*j)];
                                    tmp += alpha[(y1 + 2*i+1)*nrpix + (x1 + 2*j+1)];
                                    tmp += 2;
                                    tmp /= 4;
                                    alpha_block[i][j] = (tmp > BINARY_ALPHA/2) ? BINARY_ALPHA : 0;
                                }
                                else
                                { /* so-called "conservative" subsampling (JDL - 20-MAR-97) */
                                    alpha_block[i][j] = 
                                        ((alpha[(y1 + 2*i)*nrpix + (x1 + 2*j)] == BINARY_ALPHA) ||
                                         (alpha[(y1 + 2*i)*nrpix + (x1 + 2*j+1)] == BINARY_ALPHA) ||
                                         (alpha[(y1 + 2*i+1)*nrpix + (x1 + 2*j)] == BINARY_ALPHA) ||
                                         (alpha[(y1 + 2*i+1)*nrpix + (x1 + 2*j+1)] == BINARY_ALPHA))
                                        ? BINARY_ALPHA : 0 ; 
                                }
                            }
                        break;
                    default :
                        for (i=0; i<B_SIZE; i++)
                            for (j=0; j<B_SIZE; j++)
                                alpha_block[i][j] = BINARY_ALPHA;
                }
                
#ifdef _DEBUG
                if (trace->trace)
                    for (i=0;i<8;i++)
                        for (j=0;j<8;j++)
                            mb_shape[comp][i*8+j]=alpha_block[i][j];
#endif                
            }
#endif
            
            if (mode == MODE_INTRA || mode == MODE_INTRA_Q)
            {
                Bzero (q_block);
                
                if (switched==0)
                {
                    if(intra_dcpred_disable)    /* No INTRA DC prediction */
                    {
                        DC_coeff = BitstreamReadBits (texture_bits, 8, "DC coeff",
                                                      trace, NUM);
                        
                        if (DC_coeff == 128)
                        {
                            printf ("Illegal DC coeff: 1000000\n");
                            exit(1);
                        }
                        if (DC_coeff == 255)
                            DC_coeff = 128;
                        
                        q_block[0] = DC_coeff;
                        
                    }
                    else                                /* INTRA DC prediction (I-MBs) */
                    {
                        /* This must be Int not UInt MW 15-NOV-1996 */
                        pred_DC_coeff = DecodePredictedIntraDC(comp,
                                                               texture_bits,
                                                               trace, 
                                                               NUM);
                        q_block[0] = pred_DC_coeff;
                    }
                }
                
                
                if (CBP & (1 << (BLOCK_CNT - 1 - comp)))
                {
                    VlcGetBlock(texture_bits, q_block, INTRA,
                                trace, error_flag, 1,
                                GetVopReverseVlc(rec_vop),switched,
                                &last_non_null_coeff);   
                }
                
                /* Do the DC/AC prediction, changing the qcoeff values as
                   appropriate */
                if (intra_dcpred_disable == 0)
                {
                    doDCACrecon(q_block, 64, DC_store, QP_store, QP,
                                x_pos, y_pos, MB_width, ACpred_flag, comp);
                }
                else
                {
                    /* Ordinary inverse zigzag-scanning */
                    for (m = 0; m <64; m++)
                        pcoeff[m] = q_block[zigzag[m]];
                    
                    for (m = 0; m < 8; m++)
                    {
                        tmp = m<<3;
                        for (n = 0; n < 8; n++)
                            q_block[tmp+n] = pcoeff[tmp+n];
                    }
                }
                
#ifdef _DEBUG
                if (trace->MB_tdata_before_IQ)
                    for (i=0;i<64;i++)
                        mb_dct_bf_iq[comp][i]=q_block[i];
#endif                
                
                /* Store the qcoeff-values needed later for prediction */
#ifndef _OPTIMIZE_
                for (m = 0; m < 8; m++) {
                    DC_store[mbnum][comp][m] = q_block[m];
                }
                for (m = 0; m < 7; m++) {
                    DC_store[mbnum][comp][m+8] = q_block[(m+1)*8];
                }
#else
                for (m = 0; m < 8; m++) {
                    *(DC_store + mbnum*90 + comp*15 + m) = q_block[m];
                }
                for (m = 0; m < 7; m++) {
                    *(DC_store + mbnum*90 + comp*15 + m + 8) = q_block[(m+1)*8];
                }              
#endif
                
                /* MW 22-APR-1997 Changes for MPEG dequantization */
                if(GetVopQuantType(rec_vop))
                    BlockDequantMPEG(q_block,quant_step,INTRA,1,
                                     GetVopIntraQuantMat(rec_vop),DCT_block,63);
                else
                    BlockDequantH263(q_block,quant_step,INTRA,1,DCT_block,63);
                
#ifndef _OPTIMIZE_
                DC_store[mbnum][comp][0] = DCT_block[0];
#else
                *(DC_store + mbnum*90 + comp*15) = DCT_block[0];
#endif
                
#ifdef _DEBUG
                if (trace->MB_tdata_after_IQ)
                    for (i=0;i<64;i++)
                        mb_dct_iq[comp][i]=DCT_block[i];
#endif                
                
                if (disable_sadct)
                {
                    BlockIDCT(DCT_block, r_block_2D);
                }
                
#ifndef _WD_MODE_
                else 
                    BlockSAIDCT (DCT_block, r_block_2D, alpha_block); 
#endif
                for (m = 0; m < B_SIZE; m++)
                {
                    tmp = m*B_SIZE;
                    for (n = 0; n < B_SIZE; n++)
                        r_block[tmp+n] = r_block_2D[m][n];
                }
                
                
#ifdef _DEBUG
                if (trace->MB_tdata_after_IDCT)
                    for (i=0;i<64;i++)
                        mb_idct[comp][i]=r_block[i];
#endif
                PutBlock (comp, r_block, mblock);
            }
            /****************************************************************************/
            else /* MODE INTER LUMI */
            {
                if (CBP & (1 << (BLOCK_CNT - 1 - comp)))
                {
                    Bzero (q_block);
                    Bzero (pcoeff);
                    
                    VlcGetBlock (texture_bits, q_block, INTER, trace,
                                 error_flag, 0,
                                 GetVopReverseVlc(rec_vop),switched,
                                 &last_non_null_coeff);
                    
                    /*last_non_null_coeff = zigzag[last_non_null_coeff];*/
                    
                    
#ifndef _OPTIMIZE_
                    for (m = 0; m < 8; m++)
                    {
                        tmp = m<<3;
                        for (n = 0; n < 8; n++)
                            q_block[tmp+n] = pcoeff[tmp+n];
                    }
#endif
                    
#ifdef _DEBUG
                    if (trace->MB_tdata_before_IQ)
                        for (i=0;i<64;i++)
                            mb_dct_bf_iq[comp][i]=q_block[i];
#endif                
                    
                    /* MW 22-APR-1997 Changes for MPEG dequantization */
                    if(GetVopQuantType(rec_vop))
                        BlockDequantMPEG(q_block,quant_step,INTER,1,
                                         GetVopNonintraQuantMat(rec_vop),DCT_block,
                                         last_non_null_coeff);
                    else
                        BlockDequantH263(q_block,quant_step,INTER,1,DCT_block,
                                         last_non_null_coeff);
                    
                    /* OK to do the zz after the dequant in H.263 mode */
                    /* in MPEG mode : do zz of quant matrix */
                    
                    /* Inverse zigzag-scanning also here 12-11-96 MW */
                    for (m = 0; m <=last_non_null_coeff; m++)
                        pcoeff[zigzag_i[m]] = DCT_block[m];
                    
#ifdef _DEBUG
                    if (trace->MB_tdata_after_IQ)
                        for (i=0;i<64;i++)
                            mb_dct_iq[comp][i]=DCT_block[i]; 
#endif                
                    
                    if (disable_sadct)
                        BlockIDCT(pcoeff, r_block_2D);
#ifndef _WD_MODE_
                    else
                        BlockSAIDCT_Kaup (DCT_block, r_block_2D, alpha_block);
#endif
                    
#ifndef _OPTIMIZE_
                    for (m = 0; m < B_SIZE; m++)
                    {
                        tmp = m*B_SIZE;
                        for (n = 0; n < B_SIZE; n++)
                            r_block[tmp+n] = r_block_2D[m][n];
                    }
#endif
                    
#ifdef _DEBUG
                    if (trace->MB_tdata_after_IDCT)
                        for (i=0;i<64;i++)
                            mb_idct[comp][i]=r_block[i]; 
#endif
                    /*PutBlock (comp, r_block, mblock);*/
                    /*FillVop (mblock, mbnum, Y_rec, U_rec, V_rec);*/
                    {
                        Int                 lumstart;
                        Int                 n, m;
                        UInt                MB_in_width;
                        UInt                image_x;
                        SInt               *y_point;
                        
                        image_x = GetImageSizeX (Y_rec);
                        
                        MB_in_width = image_x / MB_SIZE;
                        
                        lumstart = (mbnum / MB_in_width) * image_x * MB_SIZE +
                            (mbnum % MB_in_width) * MB_SIZE;
                        
                        lumstart += ((comp / 2) * B_SIZE) * image_x + (comp % 2) * B_SIZE;
                        
                        y_point = (SInt *) GetImageData (Y_rec) + lumstart;
                        
                        for (n = 0; n < B_SIZE; n++)
                            for (m = 0; m < B_SIZE; m++)
                                *(y_point + n * image_x + m) = r_block_2D[n][m];
                    }
                }
                
                
                /* Fill DC_store with default coeff values */
#ifndef _OPTIMIZE_
                DC_store[mbnum][comp][0] = 128*8;
                for (n = 1; n < 15; n++)
                    DC_store[mbnum][comp][n] = 0;
#else
                p = DC_store + mbnum*90 + comp*15; /* 15*6 */
                memset(p, 0, 15*sizeof(Int));
                *p = 1024; /* 128*8 */;                    
#endif                
            }
        }
        else /* clear r_block if transparent block */
        {
            /*Bzero(r_block);*/
            /* Fill DC_store with default coeff values */
#ifndef _OPTIMIZE_
            DC_store[mbnum][comp][0] = 128*8;
            for (n = 1; n < 15; n++)
                DC_store[mbnum][comp][n] = 0;
#else
            p = DC_store + mbnum*90 + comp*15; /* 15*6 */
            memset(p, 0, 15*sizeof(Int));
            *p = 1024; /* 128*8 */;                    
#endif
        }
        
    }
    
    /*****
     *
     *    Decoding of 2 chrominance blocks (not dep. on 
     *        transparent pattern)
     *
     *****/
    for (comp=4; comp<6; comp++)
    {        
#ifndef _WD_MODE_
/* alpha_block needed only for SADCT, so the disable_sadct is wrong !!! */
        if ((disable_sadct==0)||(level_3_trace))
        {
            switch(comp)
            {
                case 0 :
                    for (i=0; i<B_SIZE; i++)
                        for (j=0; j<B_SIZE; j++)
                            alpha_block[i][j] = alpha[(y1 + i)*nrpix + (x1 + j)];
                    break;
                case 1 :
                    for (i=0; i<B_SIZE; i++)
                        for (j=0; j<B_SIZE; j++)
                            alpha_block[i][j] = alpha[(y1 + i)*nrpix + (x1 + 8 + j)];
                    break;
                case 2 :
                    for (i=0; i<B_SIZE; i++)
                        for (j=0; j<B_SIZE; j++)
                            alpha_block[i][j] = alpha[(y1 + 8 + i)*nrpix + (x1 + j)];
                    break;
                case 3 :
                    for (i=0; i<B_SIZE; i++)
                        for (j=0; j<B_SIZE; j++)
                            alpha_block[i][j] = alpha[(y1 + 8 + i)*nrpix + (x1 + 8 + j)];
                    break;
                case 4 :
                case 5 :
                    for (i=0; i<B_SIZE; i++)
                        for (j=0; j<B_SIZE; j++) {
                            if (shape == GREY_SCALE) {
                                tmp = alpha[(y1 + 2*i)*nrpix + (x1 + 2*j)];
                                tmp += alpha[(y1 + 2*i)*nrpix + (x1 + 2*j+1)];
                                tmp += alpha[(y1 + 2*i+1)*nrpix + (x1 + 2*j)];
                                tmp += alpha[(y1 + 2*i+1)*nrpix + (x1 + 2*j+1)];
                                tmp += 2;
                                tmp /= 4;
                                alpha_block[i][j] = (tmp > BINARY_ALPHA/2) ? BINARY_ALPHA : 0;
                            }
                            else { /* so-called "conservative" subsampling (JDL - 20-MAR-97) */
                                alpha_block[i][j] = 
                                    ((alpha[(y1 + 2*i)*nrpix + (x1 + 2*j)] == BINARY_ALPHA) ||
                                     (alpha[(y1 + 2*i)*nrpix + (x1 + 2*j+1)] == BINARY_ALPHA) ||
                                     (alpha[(y1 + 2*i+1)*nrpix + (x1 + 2*j)] == BINARY_ALPHA) ||
                                     (alpha[(y1 + 2*i+1)*nrpix + (x1 + 2*j+1)] == BINARY_ALPHA))
                                    ? BINARY_ALPHA : 0 ; 
                            }
                        }
                    break;
                default :
                    for (i=0; i<B_SIZE; i++)
                        for (j=0; j<B_SIZE; j++)
                            alpha_block[i][j] = BINARY_ALPHA;
            }
            
#ifdef _DEBUG
            if (trace->trace)
                for (i=0;i<8;i++)
                    for (j=0;j<8;j++)
                        mb_shape[comp][i*8+j]=alpha_block[i][j];
#endif
        }
#endif
        
        
        if (mode == MODE_INTRA || mode == MODE_INTRA_Q)
        {
            Bzero (q_block);
            
            if (switched==0) {
                if(intra_dcpred_disable)        /* No INTRA DC prediction */
                {
                    DC_coeff = BitstreamReadBits (texture_bits, 8, "DC coeff",
                                                  trace, NUM);
                    
                    if (DC_coeff == 128)
                    {
                        printf ("Illegal DC coeff: 1000000\n");
                        exit(1);
                    }
                    
                    if (DC_coeff == 255)
                        DC_coeff = 128;
                }
                else             /* INTRA DC prediction (I-MBs) */
                {
                    DC_coeff = DecodePredictedIntraDC(comp,
                                                      texture_bits,
                                                      trace,
                                                      NUM);
                }       
                q_block[0] = DC_coeff;
            } 
            if (CBP & (1 << (BLOCK_CNT - 1 - comp)))
            {
                VlcGetBlock (texture_bits, q_block, INTRA, trace,
                             error_flag, 1, GetVopReverseVlc(rec_vop),switched,
                             &last_non_null_coeff);
            }
            
            
            /* Do the DC/AC prediction, changing the qcoeff values as
               appropriate */
            if (intra_dcpred_disable == 0)
                doDCACrecon(q_block, 64, DC_store, QP_store, QP,
                            x_pos, y_pos, MB_width, ACpred_flag, comp);
            else
            {
                /* Ordinary inverse zigzag-scanning */
                for (m = 0; m <64; m++)
                    pcoeff[m] = q_block[zigzag[m]];
                
                for (m = 0; m < 8; m++)
                {
                    tmp = m<<3;
                    for (n = 0; n < 8; n++)
                        q_block[tmp+n] = pcoeff[m*8+n];
                }
                
            }
            
#ifdef _DEBUG
            if (trace->MB_tdata_before_IQ)
                for (i=0;i<64;i++)
                    mb_dct_bf_iq[comp][i]=q_block[i];
#endif
            
            /* Store the qcoeff-values needed later for prediction */
            for (m = 0; m < 8; m++) {
                DC_store[mbnum][comp][m] = q_block[m];
            }
            for (m = 0; m < 7; m++) {
                DC_store[mbnum][comp][m+8] = q_block[(m+1)*8];
            }
            
            /* MW 22-APR-1997 Changes for MPEG dequantization */
            if(GetVopQuantType(rec_vop))
                BlockDequantMPEG(q_block,quant_step,INTRA,2,
                                 GetVopIntraQuantMat(rec_vop),DCT_block,63);
            else
                BlockDequantH263(q_block,quant_step,INTRA,2,DCT_block,63);
            
            DC_store[mbnum][comp][0] = DCT_block[0];
            
#ifdef _DEBUG
            if (trace->MB_tdata_after_IQ)
                for (i=0;i<64;i++)
                    mb_dct_iq[comp][i]=DCT_block[i]; 
#endif
            
            
            if (disable_sadct)
                BlockIDCT(DCT_block, r_block_2D);
#ifndef _WD_MODE_
            else
                BlockSAIDCT (DCT_block, r_block_2D, alpha_block);
#endif
            for (m = 0; m < B_SIZE; m++)
                for (n = 0; n < B_SIZE; n++)
                    r_block[m*B_SIZE+n] = r_block_2D[m][n];
            
#ifdef _DEBUG
            if (trace->MB_tdata_after_IDCT)
                for (i=0;i<64;i++)
                    mb_idct[comp][i]=r_block[i]; 
#endif
            PutBlock (comp, r_block, mblock);
        }
        /****************************************************************************/
        else            /* MODE_INTER CHROMA */
        {
            if (CBP & (1 << (BLOCK_CNT - 1 - comp)))
            {
                Bzero (q_block);
                Bzero (DCT_block);
                
                VlcGetBlock (texture_bits, q_block, INTER, trace,
                             error_flag, 0, GetVopReverseVlc(rec_vop),switched,
                             &last_non_null_coeff);
                
                last_non_null_coeff = 64;
                
                /* Inverse zigzag-scanning also here 12-11-96 MW */
                for (m = 0; m <= last_non_null_coeff; m++)
                    pcoeff[m] = q_block[zigzag[m]];
                
#ifndef _OPTIMIZE_
                for (m = 0; m < 8; m++)
                {
                    tmp = m<<3;
                    for (n = 0; n < 8; n++)
                        q_block[tmp+n] = pcoeff[m*8+n];
                }
#endif
                
#ifdef _DEBUG
                if (trace->MB_tdata_before_IQ)
                    for (i=0;i<64;i++)
                        mb_dct_bf_iq[comp][i]=q_block[i]; 
#endif
                
                /* MW 22-APR-1997 Changes for MPEG dequantization */
                if(GetVopQuantType(rec_vop))
                    BlockDequantMPEG(pcoeff,quant_step,INTER,2,
                                     GetVopNonintraQuantMat(rec_vop),DCT_block,
                                     last_non_null_coeff);
                else
                    BlockDequantH263(pcoeff,quant_step,INTER,2,DCT_block,
                                     last_non_null_coeff);
                
#ifdef _DEBUG
                if (trace->MB_tdata_after_IQ)
                    for (i=0;i<64;i++)
                        mb_dct_iq[comp][i]=DCT_block[i]; 
#endif
                
                if (disable_sadct)
                    BlockIDCT(DCT_block, r_block_2D);
#ifndef _WD_MODE_
                else
                    BlockSAIDCT_Kaup (DCT_block, r_block_2D, alpha_block);
#endif 
                
#ifndef _OPTIMIZE_
                for (m = 0; m < B_SIZE; m++)
                {
                    tmp = m*B_SIZE;
                    for (n = 0; n < B_SIZE; n++)
                        r_block[tmp+n] = r_block_2D[m][n];
                }
#endif
                
#ifdef _DEBUG
                if (trace->MB_tdata_after_IDCT)
                    for (i=0;i<64;i++)
                        mb_idct[comp][i]=r_block[i]; 
#endif
                /*PutBlock (comp, r_block, mblock);*/
                /*FillVop (mblock, mbnum, Y_rec, U_rec, V_rec);*/
                {
                    
                    Int                 chrstart;
                    Int                 n, m;
                    UInt                MB_in_width;
                    UInt                image_x;
                    SInt                *u_point, *v_point;
                    
                    image_x = GetImageSizeX (Y_rec);
                    
                    MB_in_width = image_x / MB_SIZE;
                    
                    chrstart = (mbnum / MB_in_width) * image_x / 2 * MB_SIZE / 2 +
                        (mbnum % MB_in_width) * MB_SIZE / 2;
                    
                    
                    if(comp == 4)
                    {
                        u_point = (SInt *) GetImageData (U_rec) + chrstart;
                        
                        for (n = 0; n < B_SIZE; n++)
                            for (m = 0; m < B_SIZE; m++)
                            {
                                *(u_point + n * image_x / 2 + m) = r_block_2D[n][m];
                            }
                    }
                    else
                    {
                        v_point = (SInt *) GetImageData (V_rec) + chrstart;
                        
                        for (n = 0; n < B_SIZE; n++)
                            for (m = 0; m < B_SIZE; m++)
                            {
                                *(v_point + n * image_x / 2 + m) = r_block_2D[n][m];
                            }
                    }
                    
                }
                
            }
            
            
            /* Fill DC_store with default coeff values */
            DC_store[mbnum][comp][0] = 128*8;
            for (n = 1; n < 15; n++)
                DC_store[mbnum][comp][n] = 0;
        } 
    }
    
    /*****
     *
     *    Clip reconstructed intra MBs
     *
     *****/
    if ((mode == MODE_INTRA)||(mode == MODE_INTRA_Q))
        MB_clip (mblock);
    
#ifdef _DEBUG
    
    if (trace->MB_sdata) 
        PrintOutMBData("MB_shape_data",mb_shape,mb_shape,trace->fp_trace);
    if (trace->MB_tdata_before_IQ)
        PrintOutMBData("MB_tdata_before_IQ",mb_shape,mb_dct_bf_iq,trace->fp_trace); 
    if (trace->MB_tdata_after_IQ)
        PrintOutMBData("MB_tdata_after_IQ",mb_shape,mb_dct_iq,trace->fp_trace);
    if (trace->MB_tdata_after_IDCT)
        PrintOutMBData("MB_tdata_after_IDCT",mb_shape,mb_idct,trace->fp_trace);
    if ((mode == MODE_INTRA)&&(trace->MB_tdata_reconstructed))
    {
        for (i=0;i<6;i++)
            for (j=0;j<64;j++)
                if (mb_idct[i][j]<0) mb_idct[i][j]=0;
                else if (mb_idct[i][j]>255) mb_idct[i][j]=255;
        
        PrintOutMBData("MB_tdata_reconstructed",mb_shape,mb_idct,trace->fp_trace);
    }
#endif 
}                               /* GetMBblockdata */
#endif

/****************************************************************************/
/****************************************************************************/
/****************************************************************************/
/****************************************************************************/
/****************************************************************************/
/****************************************************************************/
/*ok*/static Void
CallingConvention
GetMBblockdata( Int transp_pattern[],
                Int intra_dcpred_disable,
                Int quant_step,
                Bitstream * texture_bits,
                Int mbnum,
                Trace *trace,
                Macroblock * mblock,
                Int * skipped_flag,
                Int * error_flag,
                Int MB_width,
                Int *DC_store,
                SInt *QP_store,
                Int QP,
                UInt CBP,
                Int ACpred_flag,
                UInt mode,
                Short *alpha,
                Vop *rec_vop)
{
    Image       *Y_rec, *U_rec, *V_rec;
    
    Int  comp;
    Int  DCT_block[B_SIZE_2];
    Int  q_block[B_SIZE_2];
#ifdef _DEBUG
    Int  r_block[B_SIZE_2];
#endif
    Int  r_block_2D[B_SIZE][B_SIZE];
    Int  pcoeff[B_SIZE_2];
    UInt DC_coeff;
    Int  pred_DC_coeff;
    
    Int  x_pos = mbnum % MB_width;
    Int  y_pos = mbnum / MB_width;
    Int  m, n;
    Int switched;
    
#ifdef _DEBUG
    UChar shape = GetVopShape(rec_vop);
    Int  x1, y1;
    Int i, j;
    Char level_3_trace;
    Short nrpix = MB_width * MB_SIZE;
    SInt mb_shape[6][64],mb_dct_bf_iq[6][64],mb_dct_iq[6][64],mb_idct[6][64];
#endif
    
    Int *p;
    Short tmp; 
    
    Int last_non_null_coeff;
    
    Int lumi_or_chroma;
    Int transparent[6];
    
    
    Int         lumstart;
    Int         chrstart;
    
    Int         MB_in_width;
    Int         image_x, comp_size;
    SInt        *y_point, *u_point, *v_point;
    SInt        *comp_point;

    assert(rec_vop);
    
#define LUMI   1
#define CHROMA 2
    
    
#ifdef _DEBUG
    x1 = x_pos*MB_SIZE;
    y1 = y_pos*MB_SIZE;
    
    level_3_trace = trace->MB_sdata||trace->MB_tdata_before_IQ|| 
        trace->MB_tdata_after_IQ||trace->MB_tdata_after_IDCT||
        trace->MB_tdata_reconstructed;
    if (level_3_trace)
    {
        memset(mb_shape,     0, 6*64*sizeof(SInt));
        memset(mb_dct_bf_iq, 0, 6*64*sizeof(SInt));
        memset(mb_dct_iq,    0, 6*64*sizeof(SInt));
        memset(mb_idct,      0, 6*64*sizeof(SInt));
    }
#endif
    
    /*****
     *
     *  Get the image fields of the VOP
     *
     *****/
    Y_rec = GetVopY (rec_vop);
    U_rec = GetVopU (rec_vop);
    V_rec = GetVopV (rec_vop);
    
    image_x = GetImageSizeX (Y_rec);
    
    MB_in_width = image_x>>4;
    
    lumstart = (mbnum / MB_in_width) * (image_x << 4) +
        ((mbnum % MB_in_width) << 4);
    
    chrstart = (mbnum / MB_in_width) * (image_x << 2)  +
        ((mbnum % MB_in_width) << 3);
    
    y_point = (SInt *) GetImageData (Y_rec) + lumstart;
    u_point = (SInt *) GetImageData (U_rec) + chrstart;
    v_point = (SInt *) GetImageData (V_rec) + chrstart;
    
    
    /* Fill DC_store with default coeff values */
#ifndef _OPTIMIZE_
    for (comp=0; comp<6; comp++)
    {        
        DC_store[mbnum][comp][0] = 128*8;
        for (n = 1; n < 15; n++)
            DC_store[mbnum][comp][n] = 0;
    }
#else
    p = DC_store + mbnum*90; /* 15*6 */
    memset(p, 0, 90*sizeof(Int));
    for (comp=0; comp<6; comp++)
        *(p + comp*15) = 1024; /* 128*8 */;                        
#endif
    
    for (comp=0; comp<6; comp++)
    {    
        transparent[comp] = 0;
        if(comp < 4)
        {
            if(transp_pattern[comp]==1) /* No decoding of transparent Y blocks */
                transparent[comp] = 1;
        }
    }
    
    /*added by Minhua Zhou 04.08.97 */
    switched = IntraDCSwitch_Decision(mode,
                                      GetVopIntraDCVlcThr(rec_vop),
                                      QP);
    QP = quant_step;   
    
    
    
    lumi_or_chroma = LUMI;
    comp_size = image_x;
    
    for (comp=0; comp<6; comp++)
    {
        if (!transparent[comp])
        {
            if(comp < 4)
            {
                comp_point = y_point +  ((comp>>1) << 3) * image_x + ((comp % 2)<<3);
            }
            else
            {
                lumi_or_chroma = CHROMA;
                
                comp_size = image_x >> 1;
                
                if(comp == 4)
                    comp_point = u_point;
                else
                    comp_point = v_point;
            }       
            
#ifdef _DEBUG
            if(level_3_trace)
                FillAlphaForTrace(comp, alpha, shape, nrpix, x1, y1, trace->trace, mb_shape);
#endif
            
            /************************************************************/
            /*          INTRA                                           */
            /************************************************************/
            if (mode == MODE_INTRA || mode == MODE_INTRA_Q)
            {
                Bzero (q_block);
                
                if (switched==0)
                {
                    if(intra_dcpred_disable)    /* No INTRA DC prediction */
                    {
                        DC_coeff = BitstreamReadBits (texture_bits, 8, "DC coeff",
                                                      trace, NUM);
                        
                        if (DC_coeff == 128)
                        {
                            printf ("Illegal DC coeff: 1000000\n");
                            exit(1);
                        }
                        if (DC_coeff == 255)
                            DC_coeff = 128;
                        
                        q_block[0] = DC_coeff;
                        
                    }
                    else                                /* INTRA DC prediction (I-MBs) */
                    {
                        /* This must be Int not UInt MW 15-NOV-1996 */
                        pred_DC_coeff = DecodePredictedIntraDC(comp,
                                                               texture_bits,
                                                               trace, 
                                                               NUM);
                        q_block[0] = pred_DC_coeff;
                    }
                }
                
                
                if (CBP & (1 << (BLOCK_CNT - 1 - comp)))
                {
                    VlcGetBlock(texture_bits, q_block, INTRA,
                                trace, error_flag, 1,
                                GetVopReverseVlc(rec_vop),switched,
                                &last_non_null_coeff);   
                }
                
                /* Do the DC/AC prediction, changing the qcoeff values as
                   appropriate */
                if (intra_dcpred_disable == 0)
                {
                    doDCACrecon(q_block, 64, DC_store, QP_store, QP,
                                x_pos, y_pos, MB_width, ACpred_flag, comp);
                }
                else
                {
                    /* Ordinary inverse zigzag-scanning */
                    for (m = 0; m <64; m++)
                        pcoeff[m] = q_block[zigzag[m]];
                    
                    for (m = 0; m < 8; m++)
                    {
                        tmp = m<<3;
                        for (n = 0; n < 8; n++)
                            q_block[tmp+n] = pcoeff[tmp+n];
                    }
                }
#ifdef _DEBUG
                if (trace->MB_tdata_before_IQ)
                    for (i=0;i<64;i++)
                        mb_dct_bf_iq[comp][i]=q_block[i];
#endif
                /* Store the qcoeff-values needed later for prediction */
#ifndef _OPTIMIZE_
                for (m = 0; m < 8; m++) {
                    DC_store[mbnum][comp][m] = q_block[m];
                }
                for (m = 0; m < 7; m++) {
                    DC_store[mbnum][comp][m+8] = q_block[(m+1)*8];
                }
#else
                for (m = 0; m < 8; m++) {
                    *(DC_store + mbnum*90 + comp*15 + m) = q_block[m];
                }
                for (m = 0; m < 7; m++) {
                    *(DC_store + mbnum*90 + comp*15 + m + 8) = q_block[(m+1)*8];
                }              
#endif
                
                /* MW 22-APR-1997 Changes for MPEG dequantization */
                if(GetVopQuantType(rec_vop))
                    BlockDequantMPEG(q_block,quant_step,INTRA,lumi_or_chroma,
                                     GetVopIntraQuantMat(rec_vop),DCT_block,63);
                else
                    BlockDequantH263(q_block,quant_step,INTRA,lumi_or_chroma,DCT_block,63);
                
#ifndef _OPTIMIZE_
                DC_store[mbnum][comp][0] = DCT_block[0];
#else
                *(DC_store + mbnum*90 + comp*15) = DCT_block[0];
#endif
                
#ifdef _DEBUG
                if (trace->MB_tdata_after_IQ)
                    for (i=0;i<64;i++)
                        mb_dct_iq[comp][i]=DCT_block[i];
#endif                
                BlockIDCT(DCT_block, r_block_2D);
#ifdef _DEBUG
                for (m = 0; m < B_SIZE; m++)
                {
                    tmp = m*B_SIZE;
                    for (n = 0; n < B_SIZE; n++)
                        r_block[tmp+n] = r_block_2D[m][n];
                }
                
                if (trace->MB_tdata_after_IDCT)
                    for (i=0;i<64;i++)
                        mb_idct[comp][i]=r_block[i];        
#endif
                /* Filling the VOP */
                for (n = 0; n < B_SIZE; n++)
                {
                    for (m = 0; m < B_SIZE; m++)
                    {
                        /* r_block_2D[n][m] = MIN(255,MAX(0,  r_block_2D->Cr[n][m])); */
                        /**(comp_point + n * comp_size + m) = r_block_2D[n][m];*/
                        *(comp_point + n * comp_size + m) = MIN(255,MAX(0,r_block_2D[n][m]));
                    }
                }
            }
            else
            {
                /************************************************************/
                /*          INTER                                           */
                /************************************************************/
                if (CBP & (1 << (BLOCK_CNT - 1 - comp)))
                {
                    Bzero (q_block);
                    Bzero (pcoeff);
                    
                    VlcGetBlock (texture_bits, q_block, INTER, trace,
                                 error_flag, 0,
                                 GetVopReverseVlc(rec_vop),switched,
                                 &last_non_null_coeff);
                    
#ifdef _DEBUG
                    if (trace->MB_tdata_before_IQ)
                        for (i=0;i<64;i++)
                            mb_dct_bf_iq[comp][i]=q_block[zigzag[i]];
#endif
                    
                    if(GetVopQuantType(rec_vop))
                        BlockDequantMPEG(q_block,quant_step,INTER,lumi_or_chroma,
                                         GetVopNonintraQuantMat(rec_vop),DCT_block,
                                         last_non_null_coeff);
                    else
                        BlockDequantH263(q_block,quant_step,INTER,lumi_or_chroma,
                                         DCT_block,last_non_null_coeff);
                    
                    
                    /***************************************************/
                    /* OK to do the zz after the dequant in H.263 mode */
                    /* in MPEG mode : do zz of quant matrix            */
                    /***************************************************/
                    
                    /* Inverse zigzag-scanning also here 12-11-96 MW */
                    for (m = 0; m <last_non_null_coeff; m++)
                        pcoeff[zigzag_i[m]] = DCT_block[m];
                    
#ifdef _DEBUG
                    if (trace->MB_tdata_after_IQ)
                        for (i=0;i<64;i++)
                            mb_dct_iq[comp][i]=pcoeff[i];
#endif
                    
                    BlockIDCT(pcoeff, r_block_2D);
                    
                    
#ifdef _DEBUG
                    for (m = 0; m < B_SIZE; m++)
                    {
                        tmp = m*B_SIZE;
                        for (n = 0; n < B_SIZE; n++)
                            r_block[tmp+n] = r_block_2D[m][n];
                    }
                    
                    if (trace->MB_tdata_after_IDCT)
                        for (i=0;i<64;i++)
                            mb_idct[comp][i]=r_block[i];        
#endif
                    
                    /* Filling the VOP */
                    for (n = 0; n < B_SIZE; n++)
                        for (m = 0; m < B_SIZE; m++)
                            *(comp_point + n * comp_size + m) = r_block_2D[n][m];
                }
                else
                {
                    /* Filling the VOP */
                    for (n = 0; n < B_SIZE; n++)
                        for (m = 0; m < B_SIZE; m++)
                            *(comp_point + n * comp_size + m) = 0;
                }
                
            }
        }
    }
    
    
    
#ifdef _DEBUG    
    if (trace->MB_sdata) 
        PrintOutMBData("MB_shape_data",mb_shape,mb_shape,trace->fp_trace);
    if (trace->MB_tdata_before_IQ)
        PrintOutMBData("MB_tdata_before_IQ",mb_shape,mb_dct_bf_iq,trace->fp_trace); 
    if (trace->MB_tdata_after_IQ)
        PrintOutMBData("MB_tdata_after_IQ",mb_shape,mb_dct_iq,trace->fp_trace);
    if (trace->MB_tdata_after_IDCT)
        PrintOutMBData("MB_tdata_after_IDCT",mb_shape,mb_idct,trace->fp_trace);
    if ((mode == MODE_INTRA)&&(trace->MB_tdata_reconstructed))
    {
        for (i=0;i<6;i++)
            for (j=0;j<64;j++)
                if (mb_idct[i][j]<0)
                    mb_idct[i][j]=0;
                else if (mb_idct[i][j]>255)
                    mb_idct[i][j]=255;
        
        PrintOutMBData("MB_tdata_reconstructed",mb_shape,mb_idct,trace->fp_trace);
    }
#endif
    
    
}

#if 0
/***********************************************************CommentBegin******
 *
 * -- receive_MODB_MTYPE -- receive_CBPB--- receive_DBQUANT---
 *
 * Author :             
 *      Minhua Zhou (HHI) <
 *
 * Created :            
 *      24-July-97
 *
 * Purpose :            
 *      decodes the MB header codes of a B-MB
 *
 * Arguments in :       
 
 *
 * Arguments in/out :   
 * Arguments out :      
 *
 * Return values :      
 *      
 *
 * Side effects :       
 *      
 *
 * Description :        
 *
 *
 * Note : 
 *
 * See also :
 *      
 *
 * Modified : 
 *   30.07.97 Fernando Jaureguizar: Added spatial_scalability
 *
 ***********************************************************CommentEnd********/

Void
CallingConvention
receive_MODB_MTYPE(Bitstream *stream, 
                   Int *CBPB,
                   Int *Mode,
                   Int spatial_scalability, /*SpSc*/
                   Trace *trace
                   )
{
    Int code;
    /* receive MODB */
    code = BitstreamShowBits (stream, 2);
    BitstreamFlushBits (stream, 2);
    if (code==2)  
        BitstreamTrace("10" , "MODB", trace);
    else
        BitstreamTrace("11" , "MODB", trace);
    *CBPB=(code==2)?0:1;
    
    code = BitstreamShowBits (stream, 4);
    if (code==1) {
        *Mode=0; /* forward MC */
        BitstreamFlushBits (stream, 4);
        BitstreamTrace("0001" , "MBTYPE", trace);
        /*>SpSc*/
        if(spatial_scalability)
        {
            fprintf(stderr,
                    "ERROR: MBTYPE '0001' unknown for Spat. Scal.\n"); 
            exit(-1);
        }
        /*SpSc<*/
    } else if (code<4) {
        *Mode=1; /* backward MC */
        BitstreamFlushBits (stream, 3);
        BitstreamTrace("001" , "MBTYPE", trace);
    }  else if (code<8) {
        *Mode=2; /* interploative MC */
        BitstreamFlushBits (stream, 2);
        BitstreamTrace("01" , "MBTYPE", trace);
    }  else {
        BitstreamFlushBits (stream, 1);
        BitstreamTrace("1" , "MBTYPE", trace);
        /*>SpSc*/
        if(spatial_scalability)
            *Mode=0; /* forward MC */
        else
            /*SpSc<*/
            *Mode=3; /* Direct MC */
    }
    return;
}  

Void
CallingConvention
receive_CBPB(Bitstream *stream,
             Int *MB_transp_pattern,
             Int *CBPB,
             Trace *trace)
{
    Int code=0,k;
    for (k=0;k<6;k++) {
        code+=code;
        if ((k < 4 && MB_transp_pattern[k] != 1) || (k > 3)) 
            code += BitstreamReadBits (stream, 1, "CBPB", trace, CODE);
    } 
    
    *CBPB=code; 
}

Void
CallingConvention
receive_B_DQUANT(Bitstream *stream,
                 Int Mode,
                 Int *DQUANT,
                 Trace *trace)
{
    Int code;
    if (Mode==3) {*DQUANT=0;return;}
    code = BitstreamShowBits (stream, 2);
    if (code<2) {
        *DQUANT=0;
        BitstreamFlushBits (stream, 1);
        BitstreamTrace("0" , "DQUANT", trace);
    } else if (code==2) {
        BitstreamFlushBits (stream, 2);
        BitstreamTrace("10" , "DQUANT", trace);
        *DQUANT=-2;
    } else if (code==3) {
        BitstreamFlushBits (stream, 2);
        BitstreamTrace("11" , "DQUANT", trace);
        *DQUANT=2;
    }
}

/***********************************************************CommentBegin******
 *
 * -- receive_B_Motion_Vector -- 
 *
 * Author :             
 *      Minhua Zhou (HHI) <
 *
 * Created :            
 *      24-July-97
 *
 * Purpose :            
 *      Forms the motion vector predictor and then decodes vectors for a MB
 *
 * Arguments in :       
 *              Bitstream *stream,      bitstream
 *              Image *modes,          prediction modes
 *              Int i, Int j,          position of MB 
 *              Int f_code_for,
 *              Int f_code_back,       f_codes
 *              Int spatial_scalability  flag for Spat. Scal. (SpSc)
 *              Trace *trace           trace control
 
 *
 * Arguments in/out :   
 *              Image *mot_x,           horizontal motion vector field 
 *              Image *mot_y,           vertical motion vector field
 * Arguments out :      
 *
 * Return values :      
 *      
 *
 * Side effects :       
 *      
 *
 * Description :        
 *
 *
 * Note : 
 *
 * See also :
 *      
 *
 * Modified : 
 *   30.07.97 Fernando Jaureguizar: Added spatial_scalability
 *
 ***********************************************************CommentEnd********/

Void
CallingConvention
receive_B_Motion_Vector(Bitstream *stream,
                        Image *mot_x, 
                        Image *mot_y, 
                        Image *modes,
                        Int i, Int j, 
                        Int f_code_for, Int f_code_back,
                        Int spatial_scalability, /*SpSc*/
                        Trace *trace,
						SInt *Px_for,SInt *Py_for, SInt *Px_back, SInt *Py_back,
						Int *prev_j
						)
{
    Int     MB_num_x;  /* Dimensions in macroblocks */
    Float   *mv_x, *mv_y;    /* Motion vectors            */
    SInt    *pred_m;         /* Modes                     */
    Int     B_num_x,index,Mode,Mode_prevs;
    SInt vx,vy;
/* A.Pacheco:1/04/98: static variables are not allowed for MTSafe code*/
/*
ok    static SInt Px_for,Py_for,Px_back,Py_back;
ok    static Int  prev_j = -1;
*/
    if (*prev_j != j){
        *Px_for  = *Py_for  = 0;
        *Px_back = *Py_back = 0;
        *prev_j = j;
    }
    
    
    MB_num_x=(Int)GetImageSizeX(modes);
    mv_x=(Float*)GetImageData(mot_x);
    mv_y=(Float*)GetImageData(mot_y);
    pred_m=(SInt*)GetImageData(modes);
    B_num_x=2*MB_num_x; 
    index = (j+j)*B_num_x+i+i;
    Mode = pred_m[j*MB_num_x+i];
    if (i>0)
        Mode_prevs =  pred_m[j*MB_num_x+i-1];
    else Mode_prevs = 2;
    
    
    /*   if ((Mode_prevs ==-1)||(Mode_prevs ==3)||(Mode_prevs ==4)) {
         Px_for  = Py_for  = 0;
         Px_back = Py_back = 0;
         } */
    
    if ((Mode==0)||(Mode==2)) {        /* forward + interplo. */
        receive_vector(&vx,&vy,*Px_for,*Py_for,f_code_for,f_code_for,trace,stream);
/*** 10/28 TEST */
/* printf("for pmvy= %3d pmvx= %3d mvy= %3d mvx= %3d\n",Py_for,Px_for,vy,vx); */
/* 10/28 TEST ***/
        mv_x[index]=(Float)(vx/2.0);
        mv_y[index]=(Float)(vy/2.0);
        *Px_for = vx; *Py_for  = vy;
    } 
    
    if ((Mode==1)||(Mode==2)) { /* backward + interplo. */
        if(!spatial_scalability) /*SpSc*/{
            receive_vector(&vx,&vy,*Px_back,*Py_back,f_code_back,f_code_back,trace,stream);
            
/*** 10/28 TEST */
/* printf("back pmvy= %3d pmvx= %3d mvy= %3d mvx= %3d\n",Py_back,Px_back,vy,vx); */
/* 10/28 TEST ***/
            mv_x[index+1]=(Float)(vx/2.0);
            mv_y[index+1]=(Float)(vy/2.0);
            *Px_back = vx; *Py_back = vy;
        }
        
    }  
    if (Mode==3) { /* direct mode */       
        receive_vector(&vx,&vy,0,0,1,1,trace,stream);
        mv_x[index+1+B_num_x]=(Float)(vx/2.0);
        mv_y[index+1+B_num_x]=(Float)(vy/2.0);
        /* Px_for  = Py_for  = 0;
           Px_back = Py_back = 0;*/
    }
}


/***********************************************************CommentBegin******
 *
 * -- receive_vector -- 
 *
 * Author :             
 *      Minhua Zhou (HHI) <
 *
 * Created :            
 *      24-July-97
 *
 * Purpose :            
 *      Decodes a vector of  B-VOPs
 *
 * Arguments in :       
 *       SInt Px,SInt Py,                       vector predictor
 *       Int f_code_horizontal,                 horizontal f_code
 *       Int f_code_vertical,                   vertical f_code
 *       Trace *trace,                          trace control
 *       Bitstream *stream                      bitstream
 *
 * Arguments in/out :   
 *
 * Arguments out :      
 *       SInt *vx,SInt *vy                      decoded vector
 *
 * Return values :      
 *      
 *
 * Side effects :       
 *      
 *
 * Description :        
 *
 *
 * Note : 
 *
 * See also :
 *      
 *
 * Modified : 
 *
 ***********************************************************CommentEnd********/

Void
CallingConvention
receive_vector(SInt *vx,SInt *vy,SInt Px,SInt Py,
               Int f_code_horizontal,
               Int f_code_vertical,
               Trace *trace,
               Bitstream *stream)
{
    Int diff_vector,residual,vlc_code_magnitude;
    Int f_code,i,sign,*error_flag=NULL;
/*ok*/    static Int range[7]={32,64,128,256,512,1024,2048};  /* 2*range*/
/*ok*/    static Int scale_factor[7]={1,2,4,8,16,32,64};
    for (i=0;i<2;i++) {
        f_code=((i==0)?f_code_horizontal-1:f_code_vertical-1);
        vlc_code_magnitude = VlcDecMV(stream, error_flag,trace);
        if (vlc_code_magnitude==0) diff_vector=0;
        else {
            residual=BitstreamReadBits(stream, f_code, "MV residual",
                                       trace, CODE);
            if (vlc_code_magnitude<0) sign=1;else sign=0;
            if (sign)  vlc_code_magnitude=-vlc_code_magnitude;
            vlc_code_magnitude--;
            diff_vector=vlc_code_magnitude*scale_factor[f_code]+residual+1;
            if (sign) diff_vector=-diff_vector;
        } 
        diff_vector+=((i==0)?Px:Py);
        if (diff_vector<-range[f_code]) diff_vector+=range[f_code]+range[f_code];
        else if (diff_vector>range[f_code]-1)  diff_vector-=range[f_code]+range[f_code];
        if (i==0) *vx=diff_vector;
        *vy=diff_vector;
    }
    
}
#endif

/***********************************************************CommentBegin******
 *
 * -- B_DecodeVopCombinedMotionShapeTextureInter -- 
 *
 * Author :             
 *      Minhua Zhou (HHI) <
 *
 * Created :            
 *      24-July-97
 *
 * Purpose :            
 *      Decodes combined motion / shape / texture for B-VOPs
 *
 * Arguments in :       
 *      Bitstream *stream       combined shape / texture bitstream
 *      Trace  *trace           flag for tracing the process.
 *      Image *mot_x_P,
 *      Image *mot_y_P          motion vector of the next P-VOP or I-VOP
 *
 *
 * Arguments in/out :   
 *      Vop *rec_vop            alpha plane is supposed to be filled
 *                              already / YUV components.
 *
 * Arguments out :      
 *
 *
 * Return values :      
 *      
 *
 * Side effects :       
 *      
 *
 * Description :        
 *
 *
 * Note : 
 *
 * See also :
 *      
 *
 * Modified : 
 *   30.07.97 Fernando Jaureguizar: Added spatial_scalability
 *       14.10.97 shape MC for B-VOPs uses the backward reference i.e. the 
 *                                              most recently decoded I/P VOP.
 *
 ***********************************************************CommentEnd********/
#ifndef __SIMPLE_PROFILE__
Void B_DecodeVopCombinedMotionShapeTextureInter(
    Bitstream * stream,
    Int vo_id,
    Trace *trace,
    Vop *rec_vop,
    Image *mot_x_P,
    Image *mot_y_P,
    Image *MB_decisions_P,
	Int		*prev_ref,
	Int		*next_ref,
	UChar **MB_TYPE2[MAX_NUM_VOS][MAX_NUM_VOLS],
	Int PRE_H_SIZE2[MAX_NUM_VOS][MAX_NUM_VOLS],
	Int PRE_V_SIZE2[MAX_NUM_VOS][MAX_NUM_VOLS],
	UChar **PRE_MB_TYPE2[MAX_NUM_VOS][MAX_NUM_VOLS],
	Image   *alpha_rec_packet,
	Image   *alpha_mode_packet,
	SInt Px_for,SInt Py_for, SInt Px_back, SInt Py_back,
	Int prev_j
	)
{
    Int         vol_id = GetVopId(rec_vop);
    Int         ef=0, *error_flag = &ef; 
    Int         mbnum, x_pos, y_pos, i, j;
    Int         rec_height;
    Int         rec_width;
    Int         MB_in_height, MB_in_width;
    Int         B_in_width, B_in_height;
    Macroblock  *mblock;
    Image       *Y_rec, *U_rec, *V_rec;
    Short       *alpha, *alpha_prev;
    Int         vop_quant,intra_dcpred_disable;
    Int         skipped_flag;
    Image       *alpha_decisions,*MB_decisions;
    UInt        new_vop;
    SInt        *palpha_decisions,*pMB_decisions;
    SInt        *mode_P = (SInt *)GetImageData(MB_decisions_P);
    Int         MB_transp_pattern[4];
    Int         ***DC_store;
    Int         QP;
    Int         COD,TRB,TRD;
    Int         CBP;
    Int         ACpred_flag=-1;
    Int         mode,DQUANT;
    Int         f_code_for=GetVopFCodeFor(rec_vop);
    Int         f_code_back=GetVopFCodeBack(rec_vop);
    Image       *mot_x, *mot_y;
    Float       *motx_ptr, *moty_ptr;
    Vop         *tmp_vop;
    SInt        *QP_store;
    /*>SpSc*/
    Int         spatial_scalability=0;
    Int         scalability  = GetVopScalability(rec_vop);
    Int         ref_sel_code = GetVopRefSelCode(rec_vop);
    Int         offset_x,offset_y;
    Int         co_mbx,co_mby,edge=16;
    Int         MB_non_trans_count=0;
    /*SpSc<*/
#ifdef _DEBUG_SPSC_
/*ok*/    static Int count=0;
    Char       name[81];
#endif
    
    /*****
     *
     *   Stuff for motion compensation of alpha plane (copied by JDL 
     *   from AlphaMotionCompensate() in vm_common/src/mot_comp.c,
     *   a function written by Noel Brady
     *
     *****/
    Image *motA2_x=NULL,*motA2_y=NULL,*modeA2=NULL;
    
    Int amb_type=0,mvda=0;
    
    
    
#ifdef _DEBUG_SPSC_
    PrintVopInfo(rec_vop,"-b-Rec--Comp");
    PrintVopInfo(GetVopNextTemp(rec_vop),"-b-Rec--Comp->N");
    PrintVopInfo(GetVopPrevTemp(rec_vop),"-b-Rec--Comp->P");
    sprintf(name,"-b-Rec--Comp%d",count);
    WriteVopDebug(rec_vop,name,name,name,0);
#endif
    
/* SPRITE */   
    
    /*****
     *
     *  Set some useful parameters
     *
     *****/
    rec_height = GetVopHeight (rec_vop);
    rec_width = GetVopWidth (rec_vop);
    MB_in_height = rec_height / MB_SIZE;
    MB_in_width = rec_width / MB_SIZE;
    B_in_width = 2 * MB_in_width;
    B_in_height = 2 * MB_in_height;
    QP_store = (SInt *) GetImageData(GetVopQP(rec_vop));
    /*>SpSc*/
    /* B-VOP with Sp.Sc. => ref_sel_code = 0 */
    if (scalability == 1 && ref_sel_code == 0)
        spatial_scalability = 1;
    /*SpSc<*/
    
    DC_store = (Int ***)calloc(MB_in_width*MB_in_height,
                               sizeof(Int **));
    for (i = 0; i < MB_in_width*MB_in_height; i++)
    {
        DC_store[i] = (Int **)calloc(6, sizeof(Int *));
        for (j = 0; j < 6; j++)
            DC_store[i][j] = (Int *)calloc(15, sizeof(Int));
    }
    
    mot_x = AllocImage((UInt) B_in_width,(UInt) B_in_height,FLOAT_TYPE);
    mot_y = AllocImage((UInt) B_in_width,(UInt) B_in_height,FLOAT_TYPE);
    motx_ptr = (Float *)GetImageData(mot_x);
    moty_ptr = (Float *)GetImageData(mot_y);
    
    SetConstantImage(mot_x,0.0);
    SetConstantImage(mot_y,0.0);
    
    if (GetVopShape(rec_vop))
    {
        /* Allocate shape ME/MC */
        modeA2 = AllocImage(GetVopWidth(rec_vop)/MB_SIZE,
                            GetVopHeight(rec_vop)/MB_SIZE,SHORT_TYPE);
        motA2_x = AllocImage(GetVopWidth(rec_vop)/MB_SIZE,
                             GetVopHeight(rec_vop)/MB_SIZE,SHORT_TYPE);
        motA2_y = AllocImage(GetVopWidth(rec_vop)/MB_SIZE,
                             GetVopHeight(rec_vop)/MB_SIZE,SHORT_TYPE);
    }
    
    
    intra_dcpred_disable = GetVopIntraACDCPredDisable(rec_vop);
    
    alpha_decisions = AllocImage(B_in_width,B_in_height,SHORT_TYPE);
    MB_decisions    = AllocImage(MB_in_width,MB_in_height,SHORT_TYPE);
    
    tmp_vop = AllocVop(GetVopWidth(rec_vop),GetVopHeight(rec_vop));
    CopyVopNonImageField(rec_vop,tmp_vop);
    
    /*****
     *
     *  Get the image fields of the VOP
     *
     *****/
    Y_rec = GetVopY (rec_vop);
    U_rec = GetVopU (rec_vop);
    V_rec = GetVopV (rec_vop);
    alpha = (Short *) GetImageData( GetVopA(rec_vop) );
    /* for inter shape */
    alpha_prev = (Short *) GetImageData( GetVopA( GetVopPrevTemp(rec_vop) ) ); 
    
    /*****
     *
     *  Get VOP quantizer and allocate MB structure
     *
     *****/
/*** 10/28 for TPS */
    /*if (GetVopScalability(rec_vop)==0)*/
    vop_quant = GetVopQuantizer(rec_vop);
    
    QP = vop_quant;
    
    
    mblock = (Macroblock *) malloc (sizeof (Macroblock));
    
    if (mblock == NULL)
    {
        printf (" Error allocating macroblock\n");
        exit(1);
    }
    
    /*>SpSc*/
/*** 8/18 TPS */
    if(!temporal_scalability && GetVopScalability(rec_vop))
/* 8/18 TPS ***/
        printf("Enhancement Layer: \n");
    /*SpSc<*/
    
    printf("Start B VOP decoding \n");
    
    /*****
     *
     *  Loop over all MBs, decode texture of each MB
     *
     *****/
    new_vop = 1;
    pMB_decisions = (SInt *)GetImageData(MB_decisions);
    palpha_decisions = (SInt *)GetImageData(alpha_decisions);
    
    /* initialise values that indicate blockwise whether */
    /* the block is transparent                          */
    for (i = 0; i < B_in_width*B_in_height; i++)
        palpha_decisions[i] = MBM_TRANSPARENT;
    
    /*****
     *
     * the macroblock loop
     *
     *****/
    
    offset_x = GetVopHorSpatRef(rec_vop)-GetVopHorSpatRef(GetVopNextTemp(rec_vop))+8-edge;
    offset_y = GetVopVerSpatRef(rec_vop)-GetVopVerSpatRef(GetVopNextTemp(rec_vop))+8-edge;
    for (y_pos = 0; y_pos < MB_in_height; y_pos++)
        for (x_pos = 0; x_pos < MB_in_width; x_pos++)
        {
/*** 10/28 TEST */
/* printf("MB NUM(Y,X)= %3d %3d\n",y_pos,x_pos); */
/* 10/28TEST ***/
            mbnum = y_pos * MB_in_width + x_pos;
            MBzero (mblock);
            
            if (GetVopShape(rec_vop) != BINARY_SHAPE_ONLY) {
                if (MB_non_trans_count>=MB_trace_thres)
                    SetTraceLevelsZero(trace);
                else
                    SetTraceLevelsDefault(trace);
            }
            
            
            /* Store the QP value for later use in AC prediction */
            QP_store[mbnum] = (SInt) QP;
            pMB_decisions[mbnum]=-1;
            
            
            trace->trace = trace->MB_header;
            
            if (trace->trace)
            {
                fprintf (trace->fp_trace,"\n>>>   Decoding Macroblock %d ........- Combined Motion/Shape/Text\n",(int)mbnum);
            }
            
            if (GetVopShape(rec_vop)) 
            {
                trace->trace = trace->MB_shape;
                
                DecodeFirst(stream, vo_id, vol_id, y_pos, x_pos, &amb_type, 
                            &mvda, 1, 0, rec_vop, trace,MB_TYPE2,PRE_H_SIZE2,PRE_V_SIZE2,
							PRE_MB_TYPE2);
                /* fill the alpha map with the decoded pixels for this MB */
                /*      Noel(B) - shape MC is done from most 
                        recently decoded P-VOP i.e. GetVopNextTemp
                        */
                
                
                /*** 10/22 SHARP FIX */
                AlphaDecodeMB(rec_vop,GetVopPrevTemp(rec_vop),vo_id,vol_id,y_pos,x_pos,
                              &amb_type,&mvda,1,MB_decisions,alpha_decisions,mot_x,mot_y,modeA2,
                              motA2_x,motA2_y,0,stream,trace,alpha_rec_packet,alpha_mode_packet);
/* 10/22 SHARP FIX ***/
                
                if(trace->trace)
                    fprintf(trace->fp_trace, "----------\n");
                trace->trace = trace->MB_header;
                
                if (amb_type == 1) 
                {       /* if the alpha MV is all_0 */
                    /* for a correct motion vector prediction in the MB's */
                    /* below the actual MB   --- JDL                      */
                    palpha_decisions[2*y_pos*B_in_width + 2*x_pos] = MBM_TRANSPARENT;
                    palpha_decisions[2*y_pos*B_in_width + 2*x_pos + 1] = MBM_TRANSPARENT;
                    palpha_decisions[(2*y_pos + 1)*B_in_width + 2*x_pos] = MBM_TRANSPARENT;
                    palpha_decisions[(2*y_pos + 1)*B_in_width + 2*x_pos + 1] = MBM_TRANSPARENT;
                }
            }
            
            /* decode COD, MCPC, ACpred_flag, CPBY and DQUANT */
            if (amb_type==1) for (i=0;i<4;i++) MB_transp_pattern[i]=1;
            if (!GetVopShape(rec_vop))
            {   
                /* no shape ("rectangular" shape) */
                for(i=0;i<MB_SIZE;i++)
                    for(j=0;j<MB_SIZE;j++)
                        alpha[((y_pos * MB_SIZE) +i)*rec_width 
                             + (x_pos * MB_SIZE) +j] = BINARY_ALPHA;
            }
            
            if (amb_type != 1)  /* if not transparent */
            {
                MB_non_trans_count++;
                if (amb_type==0)  for (i=0;i<4;i++) MB_transp_pattern[i]=0;
                else {
                    for (i=0;i<4;i++) MB_transp_pattern[i]=1;
                    for(i=0;i<B_SIZE;i++)
                        for(j=0;j<B_SIZE;j++)
                            if (alpha[((y_pos * MB_SIZE) +i)*rec_width + 
                                     (x_pos * MB_SIZE) +j] != 0)
                                MB_transp_pattern[0]=0;
                    
                    for(i=0;i<B_SIZE;i++)               
                        for(j=B_SIZE;j<2*B_SIZE;j++)
                            if (alpha[((y_pos * MB_SIZE) +i)*rec_width + 
                                     (x_pos * MB_SIZE) +j] != 0)
                                MB_transp_pattern[1]=0;
                    
                    for(i=B_SIZE;i<2*B_SIZE;i++)
                        for(j=0;j<B_SIZE;j++)
                            if (alpha[((y_pos * MB_SIZE) +i)*rec_width + 
                                     (x_pos * MB_SIZE) +j] != 0)
                                MB_transp_pattern[2]=0;
                    
                    for(i=B_SIZE;i<2*B_SIZE;i++)
                        for(j=B_SIZE;j<2*B_SIZE;j++)
                            if (alpha[((y_pos * MB_SIZE) +i)*rec_width + 
                                     (x_pos * MB_SIZE) +j] != 0)
                                MB_transp_pattern[3]=0;
                }
                
                
                /* co_mbx = (x_pos*16+offset_x)>>4;
                   co_mby = (y_pos*16+offset_y)>>4; */
                co_mbx = x_pos;
                co_mby = y_pos;
                
                if ((co_mbx>=0)&&(co_mbx<GetImageSizeX(MB_decisions_P))&&
                    (co_mby>=0)&&(co_mby<GetImageSizeY(MB_decisions_P))&& 
                    (mode_P[co_mby*GetImageSizeX(MB_decisions_P)+co_mbx]==MBM_SKIPPED)&&(!spatial_scalability)
                    
                    && (GetVopScalability(rec_vop)==0 ||
                        (GetVopScalability(rec_vop)==1 && ref_sel_code==3 && GetVopEnhanceType(rec_vop)==0)) 
/* 10/30 for TPS ***/
                    ) {
/* 10/28 for TPS ***/
                    pMB_decisions[mbnum]=4;
                    COD =1;CBP =0;QP_store[mbnum]=QP;
                } else {           
                    COD = BitstreamShowBits(stream,1);
                    COD = COD?0:1;
                    
                    if (COD==1) {
                        BitstreamFlushBits(stream,1);
                        CBP =0;
                        if (spatial_scalability) 
                            pMB_decisions[mbnum]=0;
                        else  pMB_decisions[mbnum]=3;
                        
                        QP_store[mbnum]=QP;
                    } else {
                        receive_MODB_MTYPE(stream,&CBP,&mode,
                                           spatial_scalability, /*SpSc*/
                                           trace);
                        pMB_decisions[mbnum]=mode;
                        DQUANT=0;
                        if (CBP) {
                            receive_CBPB(stream,MB_transp_pattern,&CBP,trace);
                            receive_B_DQUANT(stream,mode,&DQUANT,trace);    
                        } 
                        QP+=DQUANT;
                        QP_store[mbnum]=QP;
                        
                        receive_B_Motion_Vector(stream,mot_x,mot_y,MB_decisions,
                                                x_pos,y_pos,f_code_for, f_code_back,
                                                spatial_scalability, /*SpSc*/
                                                trace,
												&Px_for,&Py_for,&Px_back,&Py_back,&prev_j); 
                        if (CBP) 
                            GetMBblockdata(MB_transp_pattern,
                                           1,
                                           QP,stream,mbnum,trace,
                                           mblock,&skipped_flag, error_flag, 
                                           MB_in_width, 
                                           DC_store, QP_store, QP,
                                           CBP, ACpred_flag, MODE_INTER,
                                           alpha, rec_vop);
                    }
                } 
            } else pMB_decisions[mbnum]=-1; /* transparent */
            if (pMB_decisions[mbnum]!=-1)
                /*printf("(%d,%d),Mode=%d,CBP=%d(%d,%d,%d,%d)\n",x_pos,y_pos,pMB_decisions[mbnum],CBP,MB_transp_pattern[0],MB_transp_pattern[1],MB_transp_pattern[2],MB_transp_pattern[3]);*/
                
                /* if the MB is multilevel */
                if ((amb_type != 0) || (amb_type != 1))
                    SetAlphaDecisions(GetVopA(rec_vop),x_pos,y_pos,alpha_decisions);              
            FillVop (mblock, mbnum, Y_rec, U_rec, V_rec);
        }
    
    
    RemapAlphaDecisions(alpha_decisions);
/*** 10/29 for TPS */
    {
        Float tmf;
        Int   a;
        
        if (temporal_scalability && GetVopScalability(rec_vop)==1) {
            TRD=*next_ref-*prev_ref;
            tmf=(Float)(0.001+CalcVopDisplayTime(rec_vop)*1000.0/(Double)GetVopTimeIncrementResolution(rec_vop));
/*     printf("TEST tmf= %3.2f\n",tmf);  */
            
            a = (Int)( tmf / 33.33);
            if( tmf - a * 33.33 > 30) a = a + 1;
            TRB=a-*prev_ref;
            
/*     TRB=TimetoFrameNumeber(tmf)-prev_ref; */
/*     printf("TRD= %3d TRB= %3d\n",TRD,TRB); */
        }
        else {
            TRD = CalcVopDisplayTime(GetVopNextTemp(rec_vop))-CalcVopDisplayTime(GetVopPrevTemp(rec_vop));
            TRB =CalcVopDisplayTime(rec_vop)-CalcVopDisplayTime(GetVopPrevTemp(rec_vop));
        } 
/*   TRD = CalcVopDisplayTime(GetVopNextTemp(rec_vop))-CalcVopDisplayTime(GetVopPrevTemp(rec_vop));
     TRB =CalcVopDisplayTime(rec_vop)-CalcVopDisplayTime(GetVopPrevTemp(rec_vop)); */
    }
/* 10/29 for TPS ***/
    
/*printf("TRD= %3d TRB= %3d\n",TRD,TRB); */
/* printf("TRB=%d,TRD=%d\n",TRB,TRD); */
#ifdef _DEBUG_SPSC_
/*** 8/18 TPS */
    if(!temporal_scalability && GetVopScalability(rec_vop))
/* 8/18 TPS ***/
    {
        sprintf(name,"-b-Pred-Comp%d",count);
        WriteVopDebug(rec_vop,name,"","",0);
        sprintf(name,"-b-Pred-Comp.N.%d",count);
        WriteVopDebug(GetVopNextTemp(rec_vop),name,"","",0);
        sprintf(name,"-b-Pred-Comp.P.%d",count);
        WriteVopDebug(GetVopPrevTemp(rec_vop),name,"","",0);
    }
#endif
    
    B_VopMotionCompensation(
        rec_vop,GetVopPrevTemp(rec_vop),GetVopNextTemp(rec_vop),
        mot_x,mot_y,mot_x_P,mot_y_P,MB_decisions_P,
        MB_decisions,alpha_decisions,
        TRB,TRD,tmp_vop);
    
    
#ifdef _DEBUG_SPSC2_
    if(spatial_scalability) {
        if(tmp_cnt==0){    
            WriteVopGeneric (tmp_vop, "Dcomp_vop","tmp_vop","tmp_vop",
                             "up_vop.shape",0, IO_FORMAT, IO_OVERWRITE, 0);
            WriteVopGeneric (rec_vop, "Drec_vop","tmp_vop","tmp_vop",
                             "up_vop.shape",0, IO_FORMAT, IO_OVERWRITE, 0);
        }else{
            WriteVopGeneric (tmp_vop,
                             "Dcomp_vop","tmp_vop","tmp_vop",
                             "up_vop.shape",tmp_cnt, IO_FORMAT, IO_APPEND, 0);
            WriteVopGeneric (rec_vop,
                             "Drec_vop","tmp_vop","tmp_vop",
                             "up_vop.shape",tmp_cnt, IO_FORMAT, IO_APPEND, 0);
        }
    }
#endif
    
    
#ifdef _DEBUG_SPSC_
/*** 8/18 TPS */
    if(!temporal_scalability && GetVopScalability(rec_vop))
/* 8/18 TPS ***/
    {
        sprintf(name,"-b-Pred+Comp%d",count);
        WriteVopDebug(rec_vop,name,"","",0);
        sprintf(name,"-b-Pred+Comp.N.%d",count);
        WriteVopDebug(GetVopNextTemp(rec_vop),name,"","",0);
        sprintf(name,"-b-Pred+Comp.P.%d",count);
        WriteVopDebug(GetVopPrevTemp(rec_vop),name,"","",0);
        
        sprintf(name,"-b-XXXX+Comp%d",count);
        WriteVopDebug(tmp_vop,name,"","",0);
    }
#endif
    
    
    VopTextureUpdate(tmp_vop,rec_vop,trace);
    
#ifdef _DEBUG_SPSC2_
    if(spatial_scalability) {
        if(tmp_cnt==0){    
            WriteVopGeneric (rec_vop, "D_vop","tmp_vop","tmp_vop",
                             "up_vop.shape",0, IO_FORMAT, IO_OVERWRITE, 0);
        }else{
            WriteVopGeneric (rec_vop,"D_vop","tmp_vop","tmp_vop",
                             "up_vop.shape",tmp_cnt, IO_FORMAT, IO_APPEND, 0);
        }
        tmp_cnt++;
    }
#endif
    
#ifdef _DEBUG_SPSC_
/*** 8/18 TPS */
    if(!temporal_scalability && GetVopScalability(rec_vop))
/* 8/18 TPS ***/
    {
        sprintf(name,"-b-XXX+TxUpd%d",count);
        WriteVopDebug(rec_vop,name,"","",0);
        
        count++;
    }
#endif
    
    
    if (GetVopShape(rec_vop))
    {
        /* Allocate shape ME/MC */
        /* Removed cast to (Char *), 05-FEB-1997 MW */
        FreeImage(modeA2);                        
        FreeImage(motA2_x);                       
        FreeImage(motA2_y);                       
    }
    
    for (i = 0; i < MB_in_width*MB_in_height; i++)
    {
        for (j = 0; j < 6; j++) 
            free((Char *)DC_store[i][j]);
        free((Char *)DC_store[i]);
    }
    free((Char *)DC_store);
    
    
    FreeImage(alpha_decisions);
    FreeImage(MB_decisions);
    free ((Char *)mblock);
    FreeImage(mot_x);
    FreeImage(mot_y);
    FreeVop(tmp_vop);
    
    return;
}
#endif

/*ok*/static Void
CallingConvention
PrintMBShapeData(Vop *vop, Int x_pos, Int y_pos, Trace *trace)
{
    Int i,j,k1,k2;
    
    SInt mb_shape[6][64];
    
    Image       *alpha = GetVopA(vop),
        *alpha_uv = GetVopAuv(vop),
        *block = AllocImage(8,8,SHORT_TYPE);            
    
    SInt    *p=(SInt *)GetImageData(block);     
    
    for (i=0;i<4;i++)
    {
        k1 = i&1;
        k2 = (i>>1)&1;
        GetSubImage(alpha,block,x_pos*MB_SIZE+k1*8,y_pos*MB_SIZE+k2*8);
        for (j=0;j<64;j++)
            mb_shape[i][j] = p[j];
    }
    
    GetSubImage(alpha_uv,block,x_pos*8,y_pos*8);
    
    for (j=0;j<64;j++)
        mb_shape[4][j] = mb_shape[5][j] = p[j];
    
    PrintOutMBData("MB_shape_data",mb_shape,mb_shape,trace->fp_trace);
    FreeImage(block);
}

/*ok*/static void
CallingConvention
FillAlphaForTrace(Int comp, Short *alpha, UChar shape, Short nrpix, Int x1, Int y1, Int trace, SInt mb_shape[][64])
{
    Int i,j;
    Int tmp;
    
    Short alpha_block[B_SIZE][B_SIZE];
    
    switch(comp)
    {
        case 0 :
            for (i=0; i<B_SIZE; i++)
                for (j=0; j<B_SIZE; j++)
                    alpha_block[i][j] = alpha[(y1 + i)*nrpix + (x1 + j)];
            break;
        case 1 :
            for (i=0; i<B_SIZE; i++)
                for (j=0; j<B_SIZE; j++)
                    alpha_block[i][j] = alpha[(y1 + i)*nrpix + (x1 + 8 + j)];
            break;
        case 2 :
            for (i=0; i<B_SIZE; i++)
                for (j=0; j<B_SIZE; j++)
                    alpha_block[i][j] = alpha[(y1 + 8 + i)*nrpix + (x1 + j)];
            break;
        case 3 :
            for (i=0; i<B_SIZE; i++)
                for (j=0; j<B_SIZE; j++)
                    alpha_block[i][j] = alpha[(y1 + 8 + i)*nrpix + (x1 + 8 + j)];
            break;
        case 4 :
        case 5 :
            for (i=0; i<B_SIZE; i++)
                for (j=0; j<B_SIZE; j++)
                {
                    if (shape == GREY_SCALE)
                    {
                        tmp =  alpha[(y1 + 2*i)*nrpix   + (x1 + 2*j)];
                        tmp += alpha[(y1 + 2*i)*nrpix   + (x1 + 2*j+1)];
                        tmp += alpha[(y1 + 2*i+1)*nrpix + (x1 + 2*j)];
                        tmp += alpha[(y1 + 2*i+1)*nrpix + (x1 + 2*j+1)];
                        tmp += 2;
                        tmp /= 4;
                        alpha_block[i][j] = (tmp > BINARY_ALPHA/2) ? BINARY_ALPHA : 0;
                    }
                    else
                    { /* so-called "conservative" subsampling (JDL - 20-MAR-97) */
                        alpha_block[i][j] = 
                            ((alpha[(y1 + 2*i)*nrpix + (x1 + 2*j)] == BINARY_ALPHA) ||
                             (alpha[(y1 + 2*i)*nrpix + (x1 + 2*j+1)] == BINARY_ALPHA) ||
                             (alpha[(y1 + 2*i+1)*nrpix + (x1 + 2*j)] == BINARY_ALPHA) ||
                             (alpha[(y1 + 2*i+1)*nrpix + (x1 + 2*j+1)] == BINARY_ALPHA))
                            ? BINARY_ALPHA : 0 ; 
                    }
                }
            break;
        default :
            for (i=0; i<B_SIZE; i++)
                for (j=0; j<B_SIZE; j++)
                    alpha_block[i][j] = BINARY_ALPHA;
    }
    
    if (trace)
        for (i=0;i<8;i++)
            for (j=0;j<8;j++)
                mb_shape[comp][i*8+j]=alpha_block[i][j];
}
/******************************************************************/
/******************************************************************/
/******************************************************************/
/******************************************************************/
/******************************************************************/
/******************************************************************/

Void
CallingConvention
DecodeVopInter(       Bitstream         *stream,
                      Int               vo_id,
                      Trace             *trace,
                      Vop               *rec_vop,
                      Image             **mot_x_out,
                      Image             **mot_y_out,
                      Image             **MB_decisions_out,
					  UChar **MB_TYPE2[MAX_NUM_VOS][MAX_NUM_VOLS],
					  Int PRE_H_SIZE2[MAX_NUM_VOS][MAX_NUM_VOLS],
					  Int PRE_V_SIZE2[MAX_NUM_VOS][MAX_NUM_VOLS],
					  UChar **PRE_MB_TYPE2[MAX_NUM_VOS][MAX_NUM_VOLS],
					  Image   *alpha_rec_packet,Image   *alpha_mode_packet
                      )
{
    const Int   vol_id = GetVopId(rec_vop);
    
    Int         ef=0, *error_flag = &ef;
    
    Int         mbnum, x_pos, y_pos;
    Int         i, j, k;
    Int         rec_prev_height;
    Int         rec_prev_width;
    
    Macroblock  *mblock;
    
    Vop         *compensated_vop;
    Image       *Y_rec, *U_rec, *V_rec;
    Image       *Y_comp, *U_comp, *V_comp;
    Image       *Y_rec_prev, *U_rec_prev, *V_rec_prev;
    Short       *alpha, *alpha_prev;
    Short       *alpha_uv;
    
    Int         vop_quant,intra_dcpred_disable;
    Int         skipped_flag=0;
    Int         boundary_flag;
    Image       *alpha_decisions,*MB_decisions;
    UInt        new_vop;
    SInt        *palpha_decisions,*pMB_decisions;
    Int         MB_transp_pattern[4];
    
    Int         *DC_store;
    Int         *p;
    
    SInt        *ps;
    
    
    Int         m, n, QP;
    Int         coded;
    UInt        CBP;
    UInt        *CBP_store;
    
    Int         ACpred_flag=-1;
    UInt        mode;
    Int         f_code=GetVopFCodeFor(rec_vop);
    Image       *mot_x, *mot_y;
    Float       *motx_ptr, *moty_ptr;
    SInt        *QP_store;
    /*>SpSc*/
    Int         spatial_scalability=0;
    Int         scalability  = GetVopScalability(rec_vop);
    Int         ref_sel_code = GetVopRefSelCode(rec_vop);
    Int         MB_non_trans_count=0;     
    UChar       *skipped;    
    
    /*****
     *
     *   Stuff for motion compensation of alpha plane (copied by JDL 
     *   from AlphaMotionCompensate() in vm_common/src/mot_comp.c,
     *   a function written by Noel Brady
     *
     *****/
    Image       *motA2_x=NULL,*motA2_y=NULL,*modeA2=NULL;
    Int         amb_type=0,mvda=0;
    
    Int         rectangular_vop = !(GetVopShape(rec_vop));
    
    Vop         *sprite=NULL;
    Vop         *rec_prev;
    Int         ox, oy;
    
    /*****
     *
     *  Set some useful parameters
     *
     *****/
    const Int rec_height          = GetVopHeight (rec_vop);
    const Int rec_width           = GetVopWidth (rec_vop);
    const Int MB_in_height        = rec_height >> 4;
    const Int MB_in_width         = rec_width  >> 4;
    const Int B_in_width          = MB_in_width<<1;
    const Int B_in_height         = MB_in_height<<1;
    
    QP_store            = (SInt *) GetImageData(GetVopQP(rec_vop));
    
    mot_x = AllocImage((UInt) B_in_width,(UInt) B_in_height,FLOAT_TYPE);
    mot_y = AllocImage((UInt) B_in_width,(UInt) B_in_height,FLOAT_TYPE);
    motx_ptr = (Float *)GetImageData(mot_x);
    moty_ptr = (Float *)GetImageData(mot_y);
    
    ZeroImageF(mot_x);
    ZeroImageF(mot_y);    
    
    skipped   = (UChar *)calloc(MB_in_width*MB_in_height,sizeof(UChar));
    CBP_store = (UInt  *)calloc(MB_in_width*MB_in_height,sizeof(UInt));
    
    
    if (GetVopShape(rec_vop))
    {
        /* Allocate shape ME/MC */
        modeA2 = AllocImage(MB_in_width,MB_in_height,SHORT_TYPE);
        ZeroImageI(modeA2);
        
        motA2_x = AllocImage(MB_in_width,MB_in_height,SHORT_TYPE);
        motA2_y = AllocImage(MB_in_width,MB_in_height,SHORT_TYPE);
    }
    
    DC_store = Get_DC_store();
    
    intra_dcpred_disable = GetVopIntraACDCPredDisable(rec_vop);
    
    alpha_decisions = AllocImage(B_in_width,B_in_height,SHORT_TYPE);
    MB_decisions    = AllocImage(MB_in_width,MB_in_height,SHORT_TYPE);
    
    
    
    compensated_vop = AllocVop(GetVopWidth(rec_vop),GetVopHeight(rec_vop));
    CopyVopNonImageField(rec_vop,compensated_vop);
    
    Y_rec = GetVopY (rec_vop);
    U_rec = GetVopU (rec_vop);
    V_rec = GetVopV (rec_vop);
    
    Y_comp = GetVopY (compensated_vop);
    U_comp = GetVopU (compensated_vop);
    V_comp = GetVopV (compensated_vop);
    
    rec_prev    = GetVopNextTemp(rec_vop);
    Y_rec_prev  = GetVopY (rec_prev);
    U_rec_prev  = GetVopU (rec_prev);
    V_rec_prev  = GetVopV (rec_prev);
    
    rec_prev_width  = GetVopWidth (rec_prev);
    rec_prev_height = GetVopHeight(rec_prev);
    
    alpha       = (Short *) GetImageData( GetVopA(rec_vop) );
    alpha_prev  = (Short *) GetImageData( GetVopA( GetVopNextTemp(rec_vop) ) ); 
    alpha_uv    = (Short *) GetImageData( GetVopAuv(rec_vop) );
    
    
    vop_quant = GetVopQuantizer (rec_vop);
    QP = vop_quant;
    
    mblock = (Macroblock *) malloc (sizeof (Macroblock));
    
    if (mblock == NULL)
    {
        printf (" Error allocating macroblock\n");
        exit(1);
    }
    
    /* If rectangular VOP
       amb_type = 0;
       MB_transp_pattern = 0;
       Alpha = BINARY_ALPHA;
       alpha_decisions = 0;
       */
    
    if(rectangular_vop)
    {
        MB_transp_pattern[0]=0;
        MB_transp_pattern[1]=0;
        MB_transp_pattern[2]=0;
        MB_transp_pattern[3]=0;
        
        ZeroImageI(alpha_decisions);
    }
    
    ZeroImageI(MB_decisions);
    
    new_vop = 1;
    pMB_decisions = (SInt *)GetImageData(MB_decisions);
    palpha_decisions = (SInt *)GetImageData(alpha_decisions);
    
    if(!rectangular_vop)
    {
        for (i = 0; i < B_in_width*B_in_height; i++)
            palpha_decisions[i] = MBM_TRANSPARENT;
    }
    
    
    /*****
     *
     * the macroblock loop
     *
     *****/
    
#ifdef __WRITE_IM__    
    printf("VOL %2d - VO %2d : Start P VOP decoding\n", vol_id, vo_id);
#endif
    
    for (y_pos = 0; y_pos < MB_in_height; y_pos++)
    {
        for (x_pos = 0; x_pos < MB_in_width; x_pos++)
        {
            mbnum = y_pos * MB_in_width + x_pos;
            
            /* Store the QP value for later use in AC prediction */
            QP_store[mbnum] = (SInt) QP;
            
#ifdef _DEBUG
            if (GetVopShape(rec_vop) != BINARY_SHAPE_ONLY)
            {
                if (MB_non_trans_count>=MB_trace_thres)
                    SetTraceLevelsZero(trace);
                else
                    SetTraceLevelsDefault(trace);
            }
            
            
            trace->trace = trace->MB_header;
            
            if (trace->trace)
            {
                fprintf (trace->fp_trace,"\n>>>   Decoding Macroblock %d ........- Combined Motion/Shape/Text\n",(int)mbnum);
            }
#endif
            
            if (GetVopShape(rec_vop) && GetVopSpriteUsage(rec_vop)!=STATIC_SPRITE) 
            {
#ifdef _DEBUG
                trace->trace = trace->MB_shape;
#endif
                DecodeFirst(stream, vo_id, vol_id, y_pos, x_pos, &amb_type, 
                            &mvda, 1, 0, rec_vop, trace,MB_TYPE2,PRE_H_SIZE2,PRE_V_SIZE2,
							PRE_MB_TYPE2);
                
                boundary_flag = 0;
                if(amb_type >= 2)
                    boundary_flag = 1;
                else if(amb_type == -1)
                    boundary_flag = 2;
                
                /* fill the alpha map with the decoded pixels for this MB */
                AlphaDecodeMB(rec_vop,rec_prev,vo_id,vol_id,y_pos,x_pos,
                              &amb_type,&mvda,1,MB_decisions,alpha_decisions,mot_x,mot_y,modeA2,
                              motA2_x,motA2_y,0,stream,trace,alpha_rec_packet,alpha_mode_packet);
                
#ifdef _DEBUG
                if(trace->trace)
                    fprintf(trace->fp_trace, "----------\n");
                trace->trace = trace->MB_header;
#endif
                
                if (amb_type == 1) 
                {
                    /* if the alpha MV is all_0 */
                    /* for a correct motion vector prediction in the MB's */
                    /* below the actual MB   --- JDL                      */
                    i = 2*y_pos*B_in_width + 2*x_pos;
                    palpha_decisions[i] = MBM_TRANSPARENT;
                    palpha_decisions[i + 1] = MBM_TRANSPARENT;
                    palpha_decisions[i + B_in_width] = MBM_TRANSPARENT;
                    palpha_decisions[i + B_in_width + 1] = MBM_TRANSPARENT;
                }
            }
            
            
            if (GetVopShape(rec_vop) != BINARY_SHAPE_ONLY)                      /* BSO_NOEL */
            {    
                /* decode COD, MCPC, ACpred_flag, CPBY and DQUANT */
                coded = 0;
                skipped_flag = 1;
                
                if (amb_type != 1)  /* if not transparent */
                {                        
                    if(!rectangular_vop)
                    {
                        MB_transp_pattern[0]=1;
                        MB_transp_pattern[1]=1;
                        MB_transp_pattern[2]=1;
                        MB_transp_pattern[3]=1;
                        
                        ps = alpha + (y_pos * MB_SIZE)*rec_width + (x_pos * MB_SIZE);
                        for(i=0;i<B_SIZE;i++)
                            for(j=0;j<B_SIZE;j++)
                                if (*(ps + i*rec_width + j))
                                {
                                    MB_transp_pattern[0]=0;
                                    goto out1;
                                }
                      out1:
                        ps = alpha + (y_pos * MB_SIZE)*rec_width + (x_pos * MB_SIZE);
                        for(i=0;i<B_SIZE;i++)           
                            for(j=B_SIZE;j<2*B_SIZE;j++)
                                if (*(ps + i*rec_width + j))
                                {
                                    MB_transp_pattern[1]=0;
                                    goto out2;
                                }
                      out2:
                        ps = alpha + (y_pos * MB_SIZE)*rec_width + (x_pos * MB_SIZE);
                        for(i=B_SIZE;i<2*B_SIZE;i++)
                            for(j=0;j<B_SIZE;j++)
                                if (*(ps + i*rec_width + j))
                                {
                                    MB_transp_pattern[2]=0;
                                    goto out3;
                                }
                      out3:
                        ps = alpha + (y_pos * MB_SIZE)*rec_width + (x_pos * MB_SIZE);
                        for(i=B_SIZE;i<2*B_SIZE;i++)
                            for(j=B_SIZE;j<2*B_SIZE;j++)
                                if (*(ps + i*rec_width + j))
                                {
                                    MB_transp_pattern[3]=0;
                                    goto out4;
                                }
                        
                      out4:
                        i=0; /* ???? */
                    }
                    
                    *((SInt *) GetImageData (MB_decisions) + mbnum) = MBM_INTER16;
                    
                    GetMBheader(GetVopPredictionType(rec_vop),&new_vop,
                                intra_dcpred_disable,
                                &vop_quant,stream,mbnum,trace,MB_decisions,
                                mblock,&skipped_flag, error_flag,
                                DC_store, QP_store, QP, &coded, 
                                &CBP, &ACpred_flag, &mode,MB_transp_pattern);
                    
                    skipped[mbnum] = skipped_flag;
                    if(skipped_flag)
                        *((SInt *) GetImageData (MB_decisions) + mbnum) = MBM_SKIPPED;
                    
                    QP_store[mbnum]  = (SInt) vop_quant;
                    CBP_store[mbnum] = (UInt) CBP;
                    
                    if (*error_flag == RETURN)
                        exit(1);
                }
                
                if(!rectangular_vop)
                {
                    /* if the MB is multilevel */
                    if ((amb_type != 0) || (amb_type != 1))
                        SetAlphaDecisions(GetVopA(rec_vop),x_pos,y_pos,alpha_decisions);
                }
                
                /* decode the MV for texture */
                if (amb_type != 1)  /* if not transparent */
                {
                    MB_non_trans_count++;
                    if ((coded) && (mode == MODE_INTER || mode == MODE_INTER_Q || 
                                    mode == MODE_INTER4V || mode == MODE_DYN_SP) &&
                        !spatial_scalability /*SpSc*/&&
                        GetVopSpriteUsage(rec_vop)!=STATIC_SPRITE
                        )
                        GetMBvectors(GetVopPredictionType(rec_vop),
                                     intra_dcpred_disable, &vop_quant,stream,mbnum,
                                     MB_in_width,f_code,trace,MB_decisions,
                                     alpha_decisions,mot_x,mot_y,&skipped_flag, 
                                     error_flag,DC_store, QP_store, QP, &coded,
                                     &ACpred_flag, mode);
                }
                
                /* calculate MB_transp_pattern */
                
                if(!rectangular_vop)
                {
                    if (amb_type == 0) /* alpha MB is all_255 */
                        for(i=0;i<4;i++) 
                            MB_transp_pattern[i]=0;
                    else if (amb_type==1) 
                        for(i=0;i<4;i++) 
                            MB_transp_pattern[i]=1;
                }
                
                
                /* decode the DCT coeficients for the MB */
                if (amb_type!=1 && coded)
                {
                    GetMBblockdata(MB_transp_pattern,
                                   intra_dcpred_disable,
                                   vop_quant,stream,mbnum,trace,
                                   mblock,&skipped_flag, error_flag, 
                                   MB_in_width, 
                                   DC_store, QP_store, QP,
                                   CBP, ACpred_flag, mode,
                                   alpha, rec_vop);
                    QP = vop_quant;
                    if (*error_flag == RETURN)
                        exit(1);
                    
                }      
                
                if (amb_type == 1) /* transparent MB */
                {                       
                    p = DC_store + mbnum*90; /* 15*6 */
                    memset(p, 0, 15*6*sizeof(Int));
                    for (m = 0; m < 6; m++) 
                    {
                        *(p + m*15) = 1024; /* 128*8 */;                    
                    }
                }
                
                
                if(!rectangular_vop)
                {
                    if(skipped_flag && boundary_flag)
                    {
                        /* boundary block skipped */
                        /* Only the shape changed */
                        /* rec_vop = alpha * prev */
                        ox = GetVopHorSpatRef(rec_vop) - GetVopHorSpatRef(rec_prev);
                        oy = GetVopVerSpatRef(rec_vop) - GetVopVerSpatRef(rec_prev);
                        
                        k =  (y_pos <<4)       * rec_width      + (x_pos<<4);
                        n = ((y_pos <<4) + oy) * rec_prev_width + (x_pos<<4) + ox;
                        
                        for(i=0;i<MB_SIZE;i++)
                        {
                            for(j=0;j<MB_SIZE;j++)
                            {
                                m = i*rec_width + j;
                                
                                if(alpha[k + m])
                                    Y_rec->f[k + m] = Y_rec_prev->f[n + i*rec_prev_width + j];
                            }
                        }
                        
                        k  =  (y_pos<<3)            * (rec_width>>1)      + (x_pos<<3);
                        n  = ((y_pos<<3) + (oy>>1)) * (rec_prev_width>>1) + (x_pos<<3) + (ox>>1);
                        
                        for(i=0;i<B_SIZE;i++)
                        {
                            for(j=0;j<B_SIZE;j++)
                            {
                                m = i*(rec_width>>1) + j;
                                
                                if(alpha_uv[k + m])
                                {
                                    U_rec->f[k + m] = U_rec_prev->f[n + i*(rec_prev_width>>1) + j];
                                    V_rec->f[k + m] = V_rec_prev->f[n + i*(rec_prev_width>>1) + j];
                                }
                            }
                        }
                        
                    }
                }
                
            }                                                                                                                                                                                           /* BSO_NOEL */
            else 
            {
#ifdef _DEBUG
                if (trace->MB_sdata)
                    PrintMBShapeData(rec_vop, x_pos, y_pos, trace);
#endif
            }
        }
    }
    
    
    if (GetVopShape(rec_vop) != BINARY_SHAPE_ONLY)  /* BSO_NOEL */
    {
        /* mapping MB_decisions to alpha_decisions in order to make */
        /* VopMotionCompensate() work                               */
        
        RemapAlphaDecisions(alpha_decisions);
        
        
        VopMotionCompensate_opt(rec_prev,mot_x,mot_y,MB_decisions,
                                alpha_decisions, CBP_store, compensated_vop, 
                                GetVopOBMCDisable(rec_vop),
                                sprite); 
        
#  ifdef _DEBUG
        VopTextureUpdate(compensated_vop,rec_vop,trace);
#  endif        
        
        AddVopWithClipWithModes(compensated_vop, rec_vop, MB_decisions);
    }
    
    
    if (GetVopShape(rec_vop))
    {
        /* Deallocate shape ME/MC */
        /* Removed cast to (Char *), 05-FEB-1997 MW */
        FreeImage(modeA2);                        
        FreeImage(motA2_x);                       
        FreeImage(motA2_y);                       
    }
    
    FreeImage(alpha_decisions);
    free ((Char *)mblock);
    *mot_x_out=mot_x;
    *mot_y_out=mot_y;
    
    for (i = 0; i < MB_in_width*MB_in_height; i++)
        if (skipped[i])
            pMB_decisions[i] = MBM_SKIPPED;
    
    free(skipped); 
    free(CBP_store); 
    *MB_decisions_out = MB_decisions;
    
    FreeVop(compensated_vop);
    
    
#ifdef _WD_MODE_
    FreeImage(mot_x);
    FreeImage(mot_y);
    FreeImage(MB_decisions);
#endif
    
    return;
}
/*
  
********************************************


*/
/*ok*/static Void
CallingConvention
VopMotionCompensate_opt (Vop *rec_prev,
                         Image *mot_x,
                         Image *mot_y,
                         Image *MB_decisions,
                         Image *alpha_decisions,
                         UInt  *CBP_store,
                         Vop *compensated_vop,
                         Int obmc_disable, 
                         Vop *sprite) 
{
    Image       *pi;
    const Int width = GetVopWidth(compensated_vop);
    const Int height = GetVopHeight(compensated_vop);
    Int mode;
    Int i, j;
    Int offset_x, offset_y;
    SInt *prev_ipol;
    Int overlap;        
    Int prev_x_min,prev_x_max,prev_y_min,prev_y_max;
    
    SInt *BackupRef_Y, *BackupRef_U, *BackupRef_V
        ;
    /* Make padding witdh dependent on fcode; MW, 07-Feb-1997 */
    const Int edge = 16;
    
    const Int width_prev = GetImageSizeX(GetVopY(rec_prev));
    const Int height_prev = GetImageSizeY(GetVopY(rec_prev));
    
#if 0
#ifdef _DEBUG_
    /* Check MB-modes */
    fprintf (stdout, "Checking MB-modes...\n");
    for (i = 0; i < height/MB_SIZE; i++)
    {
        for (j = 0; j < width/MB_SIZE; j++)
        {
            fprintf (stdout, "%d ", MB_decisions->data->s[i*width/MB_SIZE+j]);
        }
        fprintf (stdout, "\n");
    } 
    /* Check MB-modes */
    fprintf (stdout, "Checking B-modes...\n");
    for (i = 0; i < height/B_SIZE; i++)
    {
        for (j = 0; j < width/B_SIZE; j++)
        {
            fprintf (stdout, "%d ", alpha_decisions->data->s[i*width/B_SIZE+j]);
        }
        fprintf (stdout, "\n");
    } 
#endif
#endif
    
    BackupRef_Y = rec_prev->y_chan->f;
    BackupRef_U = rec_prev->u_chan->f;
    BackupRef_V = rec_prev->v_chan->f;
    
    /*rec_prev->y_chan->f += width_prev*compensated_vop->ver_spat_shift*/
          /*    + compensated_vop->hor_spat_shift;*/
    
    if(GetVopShape(compensated_vop))
    {
        rec_prev->hor_spat_shift = compensated_vop->hor_spat_shift;
        rec_prev->ver_spat_shift = compensated_vop->ver_spat_shift;
    }
    else
    {
        rec_prev->hor_spat_shift = 0;
        rec_prev->ver_spat_shift = 0;        
    }
    
    
    /* First we have to set all 4 vectors equal for
       the MBM_INTER16 case. This could/should have
       been done before entering this routine. */
    Equalise1MVMBs(MB_decisions, mot_x, mot_y);
    
    
    pi = AllocImage(width_prev*2, height_prev*2, SHORT_TYPE);
    
    
    /* interpolate the Y component */
#ifndef _OPTIMIZE_
    InterpolateImage(GetVopY(rec_prev), pi,GetVopRoundingType(compensated_vop));
#else
    if(compensated_vop->ver_spat_shift || compensated_vop->hor_spat_shift)
        InterpolateImage(GetVopY(rec_prev), pi,GetVopRoundingType(compensated_vop));
    else
        InterpolateImageIfNecessary(rec_prev, pi,GetVopRoundingType(compensated_vop),
                                    mot_x, mot_y, MB_decisions);
#endif
    
    /* Compensate for the offset in spatial references
       between comp and rec_prev. This is done offseting the pointers
       into the MC routines and hence tricking them.
       */
    
    offset_x = GetVopHorSpatRef(compensated_vop) - GetVopHorSpatRef(rec_prev);
    offset_y = GetVopVerSpatRef(compensated_vop) - GetVopVerSpatRef(rec_prev);
    
    prev_ipol = (SInt *) GetImageData(pi) + 2*width_prev * 2 * offset_y + 2 * offset_x;
    
    rec_prev->y_chan->f = BackupRef_Y + width_prev * offset_y + offset_x;
    rec_prev->u_chan->f += width_prev/2 * (offset_y/2) + (offset_x/2);
    rec_prev->v_chan->f += width_prev/2 * (offset_y/2) + (offset_x/2);
    
    /* Do the motion compensation */
    prev_x_min = -2*offset_x + 2*edge;
    prev_y_min = -2*offset_y + 2*edge;
    prev_x_max = prev_x_min + 2*width_prev  - 4*edge;
    prev_y_max = prev_y_min + 2*height_prev - 4*edge; 
    
    
    overlap = (obmc_disable)?0:1;
    
    
#ifdef _DEBUG   /* Necessary for trace */
    ZeroImageI (GetVopY (compensated_vop));
    ZeroImageI (GetVopU (compensated_vop));
    ZeroImageI (GetVopV (compensated_vop));
#endif
    
    for (i = 0; i < (height>>4); i++)
    {
        for (j = 0; j < (width>>4); j++)
        {
            mode = ModeMB(MB_decisions, j, i);
            /*CBP  = *(CBP_store + i*(width/MB_SIZE) + j);*/
            
            if (
                mode == MBM_INTER16 || 
                mode == MBM_INTER8 || 
                mode == MBM_SPRITE
#ifndef _OPTIMIZE_
                || mode == MBM_SKIPPED
#endif
                ) 
            {
                CompY(i, j, mode,
                      prev_x_min,prev_y_min,prev_x_max,prev_y_max,
                      mot_x, mot_y,
                      MB_decisions, alpha_decisions,
                      prev_ipol, compensated_vop->y_chan,
                      width, width_prev, overlap);
                
                CompC(i, j, mode,
                      prev_x_min>>2, prev_y_min>>2, prev_x_max>>2, prev_y_max>>2,
                      mot_x, mot_y,
                      MB_decisions, alpha_decisions,
                      width, width_prev,
                      rec_prev,
                      CBP_store,
                      compensated_vop);
            }            
        }
    }
    
    
    rec_prev->y_chan->f = BackupRef_Y;
    rec_prev->u_chan->f = BackupRef_U;
    rec_prev->v_chan->f = BackupRef_V;
    
#if 0
    rec_prev->y_chan->f = rec_prev->y_chan->data->s;
    rec_prev->u_chan->f = rec_prev->u_chan->data->s;
    rec_prev->v_chan->f = rec_prev->v_chan->data->s;
#endif
    
    FreeImage (pi);
}
/*ok*/static void
CallingConvention
CompY(Int i, Int j, Int mode,
      Int prev_x_min, Int prev_y_min, Int prev_x_max, Int prev_y_max,
      Image *mot_x, Image *mot_y,
      Image *MB_decisions, Image *alpha_decisions,
      SInt *prev_ipol, Image *comp,
      Int width,
      Int width_prev, Int overlap)
{
    SInt pred[16][16];
    Float mvx, mvy;
    
    Int x, y, a, b;
    
    x = j << 4;
    y = i << 4;
    
    if (mode == MBM_INTER8
#ifndef _OPTIMIZE_
        || mode == MBM_INTER16
#endif
        
#ifndef _OPTIMIZE_
        || mode == MBM_SKIPPED
#endif
        )
    {
        /* Do overlapped motion compensation 8x8 */
        
        /* Do Y-component first */
        
        if(overlap)
        {
            /* Minhua Zhou 14.04.97 -- added alpha_decisions */
            memset(pred,0,256*sizeof(SInt));
            
            GetPred_Advanced (x, y, 
                              prev_x_min,prev_y_min,prev_x_max,prev_y_max,
                              mot_x, mot_y, MB_decisions,
                              alpha_decisions,
                              prev_ipol, &pred[0][0], 0, width,
                              width_prev, MB_SIZE,overlap);
            
            GetPred_Advanced (x, y, 
                              prev_x_min,prev_y_min,prev_x_max,prev_y_max,
                              mot_x, mot_y, MB_decisions,
                              alpha_decisions,
                              prev_ipol, &pred[0][8], 1, width,
                              width_prev, MB_SIZE, overlap);
            
            GetPred_Advanced (x, y, 
                              prev_x_min,prev_y_min,prev_x_max,prev_y_max,
                              mot_x, mot_y, MB_decisions,
                              alpha_decisions,
                              prev_ipol, &pred[8][0], 2, width,
                              width_prev, MB_SIZE, overlap);
            
            GetPred_Advanced (x, y, 
                              prev_x_min,prev_y_min,prev_x_max,prev_y_max,
                              mot_x, mot_y, MB_decisions,
                              alpha_decisions,
                              prev_ipol, &pred[8][8], 3, width,
                              width_prev, MB_SIZE, overlap);
            
            
            PutMB (x, y, pred, comp, width, MB_SIZE);
        }
        else
        {
            a = ((i*width)>>2) + j + j;
            b = (i+i+1)*(width>>3) + j + j;                        
            
            mvx = mot_x->data->f[a];
            mvy = mot_y->data->f[a];
            
            GetPred_Default_With_Unrestricted_MC (x, y,
                                                  prev_x_min,prev_y_min,
                                                  prev_x_max,prev_y_max,
                                                  mvx, mvy, prev_ipol,
                                                  comp->f,
                                                  width, width_prev, B_SIZE);
            
            
            mvx = mot_x->data->f[a + 1];
            mvy = mot_y->data->f[a + 1];
            
            GetPred_Default_With_Unrestricted_MC (x+8, y,
                                                  prev_x_min,prev_y_min,
                                                  prev_x_max,prev_y_max,
                                                  mvx, mvy, prev_ipol,
                                                  comp->f,
                                                  width, width_prev, B_SIZE);
            
            
            mvx = mot_x->data->f[b];
            mvy = mot_y->data->f[b];
            
            GetPred_Default_With_Unrestricted_MC (x, y+8,
                                                  prev_x_min,prev_y_min,
                                                  prev_x_max,prev_y_max,
                                                  mvx, mvy, prev_ipol,
                                                  comp->f,
                                                  width, width_prev, B_SIZE);
            
            
            mvx = mot_x->data->f[b + 1];
            mvy = mot_y->data->f[b + 1];
            
            GetPred_Default_With_Unrestricted_MC (x+8, y+8,
                                                  prev_x_min,prev_y_min,
                                                  prev_x_max,prev_y_max,
                                                  mvx, mvy, prev_ipol,
                                                  comp->f,
                                                  width, width_prev, B_SIZE);                        
        }
    }
    else 
    {           
        /* Do default motion compensation 16x16 */
        mvx = mot_x->data->f[((i*width)>>2) + j + j];
        mvy = mot_y->data->f[((i*width)>>2) + j + j];
        
        GetPred_Default_With_Unrestricted_MC (x, y,
                                              prev_x_min,prev_y_min,
                                              prev_x_max,prev_y_max,
                                              mvx, mvy, prev_ipol,
                                              comp->f,
                                              width, width_prev, MB_SIZE);
    }
}

/*ok*/static Void
CallingConvention
CompC(Int i, Int j, Int mode,
      Int prev_x_min, Int prev_y_min, Int prev_x_max, Int prev_y_max,
      Image *mot_x, Image *mot_y,
      Image *MB_decisions, Image *alpha_decisions,
      Int width,
      Int width_prev,
      Vop *prev_image,
      UInt  *CBP_store,
      Vop *comp_Vop)
{
    Int dx=0, dy=0;
    Int xsum, ysum, divider;
    
    Int a, b;
    Float mvx, mvy;
    
    Int bmode;    
    SInt *alpha;
    
    alpha = (SInt *) GetImageData (alpha_decisions);
    
    a = ((i*width)>>2) + j + j;
    b = (((2*i+1)*width)>>3) + j + j;                        
    
    /* 16 by 16 vectors or skipped MBs */
    if (mode == MBM_INTER16
#ifndef _OPTIMIZE_
        || mode == MBM_SKIPPED
#endif
        )
    {
        dx = (Int)(2 * mot_x->data->f[a]);
        dy = (Int)(2 * mot_y->data->f[a]);
        dx = ( dx % 4 == 0 ? dx >> 1 : (dx>>1)|1 );
        dy = ( dy % 4 == 0 ? dy >> 1 : (dy>>1)|1 );
        GetPred_Chroma (j<<4, i<<4, dx, dy, prev_image,
                        comp_Vop, width, width_prev,
                        prev_x_min,prev_y_min,prev_x_max,prev_y_max, CBP_store);
    }
    /* 8 by 8 vectors */
    else if (mode == MBM_INTER8)
    {
        divider = xsum = ysum = 0;
        
        bmode = alpha[a];
        if (bmode == 1 || bmode == 3 || bmode == 4 || bmode == 5)
        {
            mvx = mot_x->data->f[a];
            mvy = mot_y->data->f[a];
            
            xsum += (Int)(2 * mvx);
            ysum += (Int)(2 * mvy);
            divider++;
        }
        
        bmode = alpha[a+1];
        if (bmode == 1 || bmode == 3 || bmode == 4 || bmode == 5)
        {
            mvx = mot_x->data->f[a+1];
            mvy = mot_y->data->f[a+1];
            xsum += (Int)(2 * mvx);
            ysum += (Int)(2 * mvy);
            divider++;
        }
        
        bmode = alpha[b];
        if (bmode == 1 || bmode == 3 || bmode == 4 || bmode == 5)
        {
            mvx = mot_x->data->f[b];
            mvy = mot_y->data->f[b];
            xsum += (Int)(2 * mvx);
            ysum += (Int)(2 * mvy);
            divider++;
        }
        
        bmode = alpha[b+1];
        if (bmode == 1 || bmode == 3 || bmode == 4 || bmode == 5)
        {
            mvx = mot_x->data->f[b+1];
            mvy = mot_y->data->f[b+1];
            xsum += (Int)(2 * mvx);
            ysum += (Int)(2 * mvy);
            divider++;
        }
        
        switch (divider)
        {
            case 1:
                dx = SIGN (xsum) * (roundtab4[ABS (xsum) % 4]
                                    + (ABS (xsum) / 4) * 2);
                dy = SIGN (ysum) * (roundtab4[ABS (ysum) % 4]
                                    + (ABS (ysum) / 4) * 2);
                break;
            case 2:
                dx = SIGN (xsum) * (roundtab8[ABS (xsum) % 8]
                                    + (ABS (xsum) / 8) * 2);
                dy = SIGN (ysum) * (roundtab8[ABS (ysum) % 8]
                                    + (ABS (ysum) / 8) * 2);
                break;
            case 3:
                dx = SIGN (xsum) * (roundtab12[ABS (xsum) % 12]
                                    + (ABS (xsum) / 12) * 2);
                dy = SIGN (ysum) * (roundtab12[ABS (ysum) % 12]
                                    + (ABS (ysum) / 12) * 2);
                break;
            case 4:
                dx = SIGN (xsum) * (roundtab16[ABS (xsum) % 16]
                                    + (ABS (xsum) / 16) * 2);
                dy = SIGN (ysum) * (roundtab16[ABS (ysum) % 16]
                                    + (ABS (ysum) / 16) * 2);
                break;
            default:
                fprintf (stderr, "Error in Compensate_Chroma (): No blocks inside the Vop for this macroblock %d,%d\n", (int)j, (int)i);
                exit (-1);
                break;
        }
        
        GetPred_Chroma (j<<4, i<<4, dx, dy, prev_image,
                        comp_Vop, width, width_prev,
                        prev_x_min,prev_y_min,prev_x_max,prev_y_max,
                        CBP_store);
    }
}

