/*****************************************************************************
 *
 * This software module was originally developed by
 *
 *   Isabelle Corset (Philips / ACTS-MoMuSyS)       	       	     
 *   Michael Wollborn (TUH / ACTS-MoMuSyS)                    	          
 *   Cor Quist (KPN / ACTS-MoMuSyS)        	                          
 *   Angel Pacheco (UPM / ACTS-MoMuSyS)        	                          
 *   J.Ignacio Ronda (UPM / ACTS-MoMuSyS)                    	  
 *
 * and edited by
 * 
 *   Paulo Nunes (IST / ACTS-MoMuSyS)        	                         
 *   Robert Danielsen (Telenor / ACTS-MoMuSyS)                            
 *   Jan De Lameillieure (Heinrich-Hertz-Institut Berlin / ACTS-MoMuSyS)     
 *   Aasmund Sandvand (Telenor / ACTS-MoMuSyS)                                 
 *   Fernando Jaureguizar (UPM / ACTS-MoMuSyS)                 	     
 *
 * in the course of development of the MPEG-4 Video (ISO/IEC 14496-2) standard.
 * This software module is an implementation of a part of one or more MPEG-4
 * Video (ISO/IEC 14496-2) tools as specified by the MPEG-4 Video (ISO/IEC
 * 14496-2) standard.
 *
 * ISO/IEC gives users of the MPEG-4 Video (ISO/IEC 14496-2) standard free
 * license to this software module or modifications thereof for use in hardware
 * or software products claiming conformance to the MPEG-4 Video (ISO/IEC
 * 14496-2) standard.
 *
 * Those intending to use this software module in hardware or software products
 * are advised that its use may infringe existing patents. The original
 * developer of this software module and his/her company, the subsequent
 * editors and their companies, and ISO/IEC have no liability for use of this
 * software module or modifications thereof in an implementation. Copyright is
 * not released for non MPEG-4 Video (ISO/IEC 14496-2) Standard conforming
 * products.
 *
 * ACTS-MoMuSys partners retain full right to use the code for his/her own
 * purpose, assign or donate the code to a third party and to inhibit third
 * parties from using the code for non MPEG-4 Video (ISO/IEC 14496-2) Standard
 * conforming products. This copyright notice must be included in all copies or
 * derivative works.
 *
 * Copyright (c) 1996
 *
 *****************************************************************************/

/***********************************************************HeaderBegin*******
 *                                                                         
 * File:	combined_decode.c
 *
 * Author:	Jan De Lameillieure (HHI) <jan@hhi.de>
 * Created:	18-Jan-97
 *                                                                         
 * Description: Decoding VOP in combined mode
 *
 * Notes: 	
 *              starting point of this file was text_decode.c of 16-Jan-97
 *
 * Modified: 04.02.97 Noel O'Connor : removed all calls to assert()
 *           07-Feb-1997, M.Wollborn: changed in order to treat skipped MBs
 *	                      correctly
 *           15-APR-97 Jan De Lameillieure : removed unused variables which were 
 *                       obsolete since upgrade from MMR to M4R
 *           15-APR-97 Jan De Lameillieure : added the inverse SA-DCT 
 *           22-APR-97 Michael Wollborn: changed name of BlockDequant to 
 *	                      BlockDequantH263 and changed
 *	                      stuff for MPEG-like quantization
 *           01.05.97 Luis Ducla-Soares:added functions for the error resilient
 *                       combined mode with data partitioning:
 *                       DecodeVopCombinedShapeTextureIntraErrRes(),
 *                       DecodeVopCombinedMotionShapeTextureInterErrRes(), 
 *                       GetMBheaderErrRes(), GetMBheaderMotionErrRes(),
 *                       GetMBheaderTextErrRes(), GetMBvectorsErrRes()
 *	 	                   and GetMBblockdataErrRes().
 *           15.05.97 Luis Ducla-Soares: corrected the remultiplexing of the DCT data
 *                       in the combined error resilient mode with 
 *                       data partitioning. Cleaned up the code.
 *           21.05.97 A. Pacheco (UPM): Modifications in several functions
 *                       to have just one function to compute the MVs prediction
 *                       in both encoder/decoder (find_pmvs)
 *           26.05.97 Minhua Zhou: added mot_padding.h and VopPadding() 
 *           16.06.97 Angel Pacheco: unified the TRANSPARENT modes.
 *      16-JUN-97 Jan De Lameillieure : renaming some include files to sadct_*
 *      16-JUN-97 Jan De Lameillieure : added Kaup-SADCT
 *           24.07.97 Minhua Zhou: added B-VOP decoding
 *           04.08.97 Minhua Zhou: renamed advanced_prediction_disable
 *      04.08.97 Minhua Zhou: added intra_dc_vlc_thr (switched)
 *      12.08.97 Minhua Zhou: added decoding of DQUANT in combined mode 
 *      05.09.97 Fernando Jaureguizar: deleted unnecessary code with aux_vop
 *
 ***********************************************************HeaderEnd*********/

/************************    INCLUDE FILES    ********************************/

#include "io_generic.h"
#include "vm_common_defs.h"
#include "combined_decode.h"
#include "text_decode_mb.h"
#include "zigzag.h"
#include "mot_decode.h"
#include "mot_util.h"
#include "mot_comp.h"
#include "alp_common_def.h"
#include "alp_dec_mom.h" 
#include "alp_dec_header.h" 

#ifndef _WD_MODE_
#  include "sadct_momusys.h"
#  include "sadct_momusys_kaup.h"
#endif

#include "mot_padding.h"

#include "ring.h"

/*** 8/18 TPS */
#include "vm_dec_main.h"

/*A.Pacheco:1/04/98: global variables are not allowed*/
/* ok extern Int   temporal_scalability;*/
/* 8/18 TPS ***/

/*** 10/27 TPS */
/*
ok extern Int prev_ref;
ok extern Int next_ref;
*/
/* 10/27 TPS ***/

#ifdef _DEBUG_SPSC2_
/*ok*/static Int tmp_cnt=0;
#endif


/***********************************************************CommentBegin******
 *
 * -- DecodeVopCombinedShapeTextureIntraErrRes -- 
 *
 * Author :		
 *	Luis Ducla-Soares (IST) - lds@lx.it.pt
 *
 * Created :		
 *	1-May-97
 *
 * Purpose :		
 *      Decodes shape / texture for INTRA VOPs for the combined error
 *      resilient mode with data partitioning
 *
 * Arguments in : 	
 *	Bitstream *stream	combined shape / texture bitstream
 *	Trace  *trace		flag for tracing the process.
 *
 *
 * Arguments in/out :	
 *	Vop *rec_vop		alpha plane is supposed to be filled
 *				already / YUV components.
 *
 * Arguments out :	
 *
 *
 * Return values :	
 *	
 *
 * Side effects :	
 *	
 *
 * Description :	
 *
 *
 * Note : 
 *      based on DecodeVopCombinedShapeTextureIntra() as of 01.05.97
 *
 * See also :
 *	
 *
 * Modified : 
 *      15.05.97 Luis Ducla-Soares: corrected the remultiplexing of the DCT data
 *                                  in the combined error resilient mode with 
 *                                  data partitioning. Cleaned up the code.
 *      04.11.97 Minhua Zhou: updated DC/AC prediction
 *			19.11.97 Noel Brady: quant scale is not present when in shape only mode
 *
 ***********************************************************CommentEnd********/

Void
CallingConvention
DecodeVopCombinedShapeTextureIntraErrRes(Bitstream * stream, Int vo_id, 
                                         Trace *trace, Vop * rec_vop,
										 UChar **MB_TYPE2[MAX_NUM_VOS][MAX_NUM_VOLS],
										 Int PRE_H_SIZE2[MAX_NUM_VOS][MAX_NUM_VOLS],
										Int PRE_V_SIZE2[MAX_NUM_VOS][MAX_NUM_VOLS],
										UChar **PRE_MB_TYPE2[MAX_NUM_VOS][MAX_NUM_VOLS],
										Image   *alpha_rec_packet,
										Image   *alpha_mode_packet)
{
    Int 	      	ef=0, *error_flag = &ef; 
    Int           mbnum, x_pos, y_pos, i, j;
    Int           rec_height;
    Int 		rec_width;
    Int		MB_in_height, MB_in_width;
    Int           B_in_width, B_in_height;
    Macroblock    *mblock;
    Image         *Y_rec, *U_rec, *V_rec, *A_rec;
    Short		*alpha;
    Int           vop_quant, intra_dcpred_disable;
    Image         *MB_decisions;
    UInt		new_vop;
    SInt		*pMB_decisions;
    Int 		MB_transp_pattern[4];
    Int           ***DC_store;
    Int           **slice_nb;
    Int           slice_counter;
    Int           amb_type;
    SInt          *QP_store;
    Int           QP;
    Int           counter, tmpvar, tmpvar1, tmpvar2;
    Int           next_MB;
    Int           MB_in_VOP;
    Int           MB_in_VOP_length;
    HeaderInfo    *header_data;
    Int           after_marker;
    Int           skip_texture;
    Int           bits;
    Int 					start_of_packet = 0;
    Int 		vol_id = GetVopId(rec_vop);
    Int 		n,m;
    
    
    printf("%d,%d\n",GetVopHorSpatRef(rec_vop),GetVopVerSpatRef(rec_vop));
    
    /*****
     *
     *	Set some useful parameters
     *
     *****/
    rec_height = GetVopHeight (rec_vop);
    rec_width = GetVopWidth (rec_vop);
    MB_in_height = rec_height / MB_SIZE;
    MB_in_width = rec_width / MB_SIZE;
    B_in_width = 2 * MB_in_width;
    B_in_height = 2 * MB_in_height;
    QP_store = (SInt *) GetImageData(GetVopQP(rec_vop));
    MB_in_VOP = MB_in_width * MB_in_height;
    MB_in_VOP_length = (Int) ceil(log(MB_in_VOP)/log(2));
    
    /* allocate space for 3D matrix to keep track of prediction values
       for DC/AC prediction */
    
    DC_store = (Int ***)calloc(MB_in_width*MB_in_height,
                               sizeof(Int **));
    for (i = 0; i < MB_in_width*MB_in_height; i++)
    {
        DC_store[i] = (Int **)calloc(6, sizeof(Int *));
        for (j = 0; j < 6; j++)
            DC_store[i][j] = (Int *)calloc(15, sizeof(Int));
    }
    
    /* allocate space for 2D matrix to keep track of the slice numbers */
    
    slice_nb = (Int **)calloc(MB_in_width,sizeof(Int *));
    
    for (i = 0; i < MB_in_width; i++)
        slice_nb[i] = (Int *)calloc(MB_in_height, sizeof(Int));
    
    
    intra_dcpred_disable = GetVopIntraACDCPredDisable(rec_vop);
    
    header_data = (HeaderInfo *)calloc(MB_in_width*MB_in_height,sizeof(HeaderInfo));
    
    MB_decisions    = AllocImage(MB_in_width,MB_in_height,SHORT_TYPE);
    
    /* set all blocks to opaque */
    for(i=0;i<4;i++) 
        MB_transp_pattern[i]=0;
    
    /*****
     *
     *	Get the image fields of the VOP
     *
     *****/
    Y_rec = GetVopY (rec_vop);
    U_rec = GetVopU (rec_vop);
    V_rec = GetVopV (rec_vop);
    A_rec = GetVopA (rec_vop);
    alpha=(Short *)GetImageData(A_rec);
    
    /*****
     *
     *	Get VOP quantizer and allocate MB structure
     *
     *****/
    vop_quant = GetVopQuantizer (rec_vop);
    QP = vop_quant;
    
    mblock = (Macroblock *) malloc (sizeof (Macroblock));
    
    if (mblock == NULL)
    {
        printf (" Error allocating macroblock\n");
        exit(1);
    }
    
#ifdef __WRITE_IM__    
    printf("Start I VOP decoding \n");
#endif
    
    /*****
     *
     *	Loop over all MBs, decode texture of each MB
     *
     *****/
    new_vop = 1;
    pMB_decisions = (SInt *)GetImageData(MB_decisions);
    
    amb_type = 0;  /* No shape coding: opaque block */
    mbnum = 0;
    counter = 0;
    after_marker = 0;
    skip_texture = 0;
    slice_counter = 0;
    
    if (GetVopShape(rec_vop)) AllocShapePacket2(rec_vop,&alpha_rec_packet,&alpha_mode_packet);
    
    while(mbnum < MB_in_width*MB_in_height)
    {
        start_of_packet = 1;
        while (((BitstreamShowBits(stream,DC_MARKER_LENGTH))!=DC_MARKER) &&
               (mbnum < MB_in_width * MB_in_height)){
            x_pos = mbnum % MB_in_width;
            y_pos = mbnum / MB_in_width;
            
            slice_nb[x_pos][y_pos] = slice_counter;
            
            trace->trace = trace->MB_header;
            if (trace->trace)
            {
                fprintf (trace->fp_trace, "\n>>>   Decoding Macroblock %d ........- Combined Shape/Text\n",(int)mbnum);
            }
            
            /* Store the QP value for later use in AC prediction */
            QP_store[mbnum] = (SInt) QP;
            
            /* decode COD, MCPC, ACpred_flag, CPBY and DQUANT */
            header_data[mbnum].coded = 0;
            header_data[mbnum].skipped_flag = 1;
            
            amb_type = 0;
            
            /* set all blocks to opaque */
            for(i=0;i<4;i++) 
		header_data[mbnum].MB_transp_pattern[i]=0;
            
            if (GetVopShape(rec_vop)) 
            {
                GetMBIntraShapeErrRes(stream,vo_id,vol_id,y_pos,x_pos,
                                      MB_decisions,start_of_packet,&amb_type,
                                      header_data[mbnum].MB_transp_pattern,
                                      error_flag,rec_vop,trace,MB_TYPE2,
									  PRE_H_SIZE2,PRE_V_SIZE2,PRE_MB_TYPE2,
									  alpha_rec_packet,alpha_mode_packet);
                start_of_packet = 0;
            }
            
            if (amb_type != 1)		
                /* no further data if the MB
                   is fully transparent */
            {      
                GetMBheaderErrRes(intra_dcpred_disable,&vop_quant,
                                  stream,mbnum,trace,MB_decisions,
                                  &header_data[mbnum].skipped_flag, error_flag,
                                  &header_data[mbnum].coded, 
                                  &header_data[mbnum].CBP,
                                  &header_data[mbnum].mode,
                                  header_data[mbnum].IntraDC,
                                  header_data[mbnum].MB_transp_pattern,
                                  rec_vop);
                
                QP_store[mbnum] = (SInt) vop_quant;
            }
            else
            {			/* transparent MB */
                MBzero (mblock);
                /* set all alpha block pixels to zero */
                /* Fill DC_store with default coeff values */
                for (m = 0; m < 6; m++) 
                {
                    DC_store[mbnum][m][0] = 128*8;
                    for (n = 1; n < 15; n++)
                        DC_store[mbnum][m][n] = 0;
                }
            }
            
            if (*error_flag){
                while (((BitstreamShowBits(stream,DC_MARKER_LENGTH)) != DC_MARKER) &&
                       (((BitstreamShowBitsByteAlign(stream,RESYNC_MARKER_LENGTH)) != RESYNC_MARKER) || (!CheckBitStuffing(stream))) &&
                       (((BitstreamShowBitsByteAlign(stream,VOP_START_CODE_LENGTH)) !=VOP_START_CODE) || (!CheckBitStuffing(stream))) &&
                       (((BitstreamShowBitsByteAlign(stream,24)) != 252) || (!CheckBitStuffing(stream))))
                    BitstreamFlushBits(stream,1);
                
                if (((BitstreamShowBitsByteAlign(stream,RESYNC_MARKER_LENGTH)) == RESYNC_MARKER) || 
                    ((BitstreamShowBitsByteAlign(stream,VOP_START_CODE_LENGTH)) == VOP_START_CODE) ||
                    ((BitstreamShowBitsByteAlign(stream,24)) == 252)){
                    skip_texture = 1;
                    break;
                }
            }
            else {
                mbnum++;
                counter++;
            }
        }
        
        if (!skip_texture)
	{
            while ((BitstreamShowBits(stream,DC_MARKER_LENGTH)!=DC_MARKER))
                BitstreamFlushBits(stream,1);
            BitstreamReadBits(stream,DC_MARKER_LENGTH,"dc marker",trace,NUM); 
	}
        
        
        
        tmpvar1 = mbnum - counter;
        tmpvar2 = tmpvar1;
        counter = 0;
        
        
        if (!skip_texture)
	{
            if (!*error_flag)
	    {
                while (tmpvar1 < mbnum)
		{
                    if (header_data[tmpvar1].coded)
		    {
                        GetMBheaderTextErrRes(intra_dcpred_disable,/*&vop_quant,*/stream,
                                              tmpvar1,trace,MB_decisions,
                                              error_flag,&header_data[tmpvar1].coded, 
                                              &header_data[tmpvar1].CBP, 
                                              &header_data[tmpvar1].ACpred_flag,
                                              &header_data[tmpvar1].mode,
                                              header_data[tmpvar1].MB_transp_pattern);      
		    }
                    tmpvar1++;
		}
                
                while (tmpvar2 < mbnum)
		{
                    if (header_data[tmpvar2].coded)
		    {
                        if (*error_flag != RETURN)
                            GetMBblockdataErrRes(header_data[tmpvar2].MB_transp_pattern,
                                                 intra_dcpred_disable,vop_quant,
                                                 stream,tmpvar2,trace,
                                                 mblock,error_flag, MB_in_width,
                                                 DC_store,QP_store,QP,
                                                 header_data[tmpvar2].CBP,
                                                 header_data[tmpvar2].ACpred_flag,
                                                 header_data[tmpvar2].mode,
                                                 header_data[tmpvar2].IntraDC,
                                                 rec_vop,slice_nb);
                        QP = vop_quant;
		    }
                    
                    
                    /* (skipped_flag==1) Clear the skipped MB */
                    if (header_data[tmpvar2].skipped_flag == 1)
                        MBzero (mblock);
                    
                    
                    FillVop (mblock, tmpvar2, Y_rec, U_rec, V_rec);
                    tmpvar2++;
		}
	    }
	}
        else
            skip_texture = 0;
        
        if (*error_flag){
            while (((BitstreamShowBitsByteAlign(stream,RESYNC_MARKER_LENGTH) != RESYNC_MARKER) || (!CheckBitStuffing(stream))) &&
                   ((BitstreamShowBitsByteAlign(stream,VOP_START_CODE_LENGTH) != VOP_START_CODE) || (!CheckBitStuffing(stream))) &&
                   ((BitstreamShowBitsByteAlign(stream,24) != 252) || (!CheckBitStuffing(stream))))
                BitstreamFlushBits(stream,1);
            *error_flag = 0;
        }
        
        
        
        if ((BitstreamShowBitsByteAlign(stream,RESYNC_MARKER_LENGTH)) == RESYNC_MARKER){
            BitstreamByteAlign(stream);
            BitstreamReadBits(stream,RESYNC_MARKER_LENGTH,"resync_marker",trace,NUM);
            next_MB = BitstreamReadBits(stream,MB_in_VOP_length,"macro_block_num",trace,NUM);
            if (GetVopShape(rec_vop) != BINARY_SHAPE_ONLY) 
		BitstreamReadBits(stream,5,"quant_scale",trace,NUM);
            if (BitstreamReadBits(stream,1,"header_extension_code",trace,NUM)){
                /* modulo_time_base (? bits) */
                tmpvar = (Int) BitstreamReadBits(stream,1,"modulo_time_base",trace,FLAG);
                
                while(tmpvar == 1)
                {
                    tmpvar = (Int) BitstreamReadBits(stream,1,"modulo_time_base",trace,FLAG);
                }
                
                /* vop_time_increment (1-15 bits) */
                bits = (Int)ceil(log((double)GetVopTimeIncrementResolution(rec_vop))/log(2.0));
                if (bits<1) bits=1;
                BitstreamReadBits(stream,1,"marker_bit",trace,NUM);
                tmpvar = (Int) BitstreamReadBits(stream,bits,"vop_time_increment",trace,NUM);
                BitstreamReadBits(stream,1,"marker_bit",trace,NUM);
                
                /* vop_prediction_type (2 bits) */
                BitstreamReadBits(stream,2,"vop_prediction_type",trace,NUM);
            }
            after_marker = 1;
            slice_counter++;
        }
        else
            next_MB = MB_in_width * MB_in_height;
        
        if ((mbnum != next_MB) && (next_MB < mbnum))
            mbnum = next_MB;
        else if (next_MB > mbnum){
            while ((((BitstreamShowBitsByteAlign(stream,VOP_START_CODE_LENGTH) !=VOP_START_CODE)) || (!CheckBitStuffing(stream))) &&
                   (((BitstreamShowBitsByteAlign(stream,24)) != 252) || (!CheckBitStuffing(stream))))
                BitstreamFlushBits(stream,1);
        }
        
        if ((BitstreamShowBitsByteAlign(stream,24)) == 252){
            BitstreamByteAlign(stream);
        }
    }
    
    /* Free allocated memory for 3D matrix */
    
    for (i = 0; i < MB_in_width*MB_in_height; i++)
    {
        for (j = 0; j < 6; j++) 
            free((Char *)DC_store[i][j]);
        free((Char *)DC_store[i]);
    }
    free((Char *)DC_store);
    
    
    if (GetVopShape(rec_vop)) FreeShapePacket2(alpha_rec_packet,alpha_mode_packet);
    
    /* Free allocated memory for 2D matrix */
    
    for (i = 0; i < MB_in_width; i++)
        free(slice_nb[i]);
    free(slice_nb);
    
    FreeImage(MB_decisions);
    free ((Char *)mblock);
    
    free(header_data);
    
    return;
}


/***********************************************************CommentBegin******
 *
 * -- DecodeVopCombNoDataPartShapeTextureIntraErrRes -- 
 *
 * Author :		
 *	Luis Ducla-Soares (IST) - lds@lx.it.pt
 *
 * Created :		
 *	1-May-97
 *
 * Purpose :		
 *      Decodes shape / texture for INTRA VOPs for the combined error
 *      resilient mode with data partitioning
 *
 * Arguments in : 	
 *	Bitstream *stream	combined shape / texture bitstream
 *	Trace  *trace		flag for tracing the process.
 *
 *
 * Arguments in/out :	
 *	Vop *rec_vop		alpha plane is supposed to be filled
 *				already / YUV components.
 *
 * Arguments out :	
 *
 *
 * Return values :	
 *	
 *
 * Side effects :	
 *	
 *
 * Description :	
 *
 *
 * Note : 
 *      based on DecodeVopCombinedShapeTextureIntra() as of 01.05.97
 *
 * See also :
 *	
 *
 * Modified : 
 *      15.05.97 Luis Ducla-Soares: corrected the remultiplexing of the DCT data
 *                                  in the combined error resilient mode with 
 *                                  data partitioning. Cleaned up the code.
 *			19.11.97 Noel Brady: quant_scale not present when in shape only mode
 *
 ***********************************************************CommentEnd********/

Void
CallingConvention
DecodeVopCombNoDataPartShapeTextureIntraErrRes(Bitstream * stream, Int vo_id, 
                                               Trace *trace, Vop * rec_vop,
											   UChar **MB_TYPE2[MAX_NUM_VOS][MAX_NUM_VOLS],
											   Int PRE_H_SIZE2[MAX_NUM_VOS][MAX_NUM_VOLS],
												Int PRE_V_SIZE2[MAX_NUM_VOS][MAX_NUM_VOLS],
												UChar **PRE_MB_TYPE2[MAX_NUM_VOS][MAX_NUM_VOLS],
												Image   *alpha_rec_packet,
												Image   *alpha_mode_packet)
{
    Int 	      	ef=0, *error_flag = &ef; 
    Int           mbnum, x_pos, y_pos, i, j;
    Int           rec_height;
    Int 		rec_width;
    Int		MB_in_height, MB_in_width;
    Int           B_in_width, B_in_height;
    Macroblock    *mblock;
    Image         *Y_rec, *U_rec, *V_rec, *A_rec;
    Short		*alpha;
    Int           vop_quant, intra_dcpred_disable;
    Image         *MB_decisions;
    UInt		new_vop;
    SInt		*pMB_decisions;
    Int 		MB_transp_pattern[4];
    Int           ***DC_store;
    Int           **slice_nb;
    Int           slice_counter;
    Int           amb_type;
    SInt          *QP_store;
    Int           QP;
    Int           counter, tmpvar;
    Int           next_MB;
    Int           MB_in_VOP;
    Int           MB_in_VOP_length;
    HeaderInfo    *header_data;
    Int           after_marker;
    Int           skip_texture;
    Int           bits;
    Int start_of_packet;
    Int vol_id = GetVopId(rec_vop);
    Int m,n;
    
    
    /*****
     *
     *	Set some useful parameters
     *
     *****/
    rec_height = GetVopHeight (rec_vop);
    rec_width = GetVopWidth (rec_vop);
    MB_in_height = rec_height / MB_SIZE;
    MB_in_width = rec_width / MB_SIZE;
    B_in_width = 2 * MB_in_width;
    B_in_height = 2 * MB_in_height;
    QP_store = (SInt *) GetImageData(GetVopQP(rec_vop));
    MB_in_VOP = MB_in_width * MB_in_height;
    MB_in_VOP_length = (Int) ceil(log(MB_in_VOP)/log(2));
    
    
    /* allocate space for 3D matrix to keep track of prediction values
       for DC/AC prediction */
    
    DC_store = (Int ***)calloc(MB_in_width*MB_in_height,
                               sizeof(Int **));
    for (i = 0; i < MB_in_width*MB_in_height; i++)
    {
        DC_store[i] = (Int **)calloc(6, sizeof(Int *));
        for (j = 0; j < 6; j++)
            DC_store[i][j] = (Int *)calloc(15, sizeof(Int));
    }
    
    
    /* allocate space for 2D matrix to keep track of the slice numbers */
    
    slice_nb = (Int **)calloc(MB_in_width,sizeof(Int *));
    
    for (i = 0; i < MB_in_width; i++)
        slice_nb[i] = (Int *)calloc(MB_in_height, sizeof(Int));
    
    
    intra_dcpred_disable = GetVopIntraACDCPredDisable(rec_vop);
    
    header_data = (HeaderInfo *)calloc(MB_in_width*MB_in_height,sizeof(HeaderInfo));
    
    MB_decisions    = AllocImage(MB_in_width,MB_in_height,SHORT_TYPE);
    
    /* set all blocks to opaque */
    for(i=0;i<4;i++) 
        MB_transp_pattern[i]=0;
    
    /*****
     *
     *	Get the image fields of the VOP
     *
     *****/
    Y_rec = GetVopY (rec_vop);
    U_rec = GetVopU (rec_vop);
    V_rec = GetVopV (rec_vop);
    A_rec = GetVopA (rec_vop);
    alpha=(Short *)GetImageData(A_rec);
    
    /*****
     *
     *	Get VOP quantizer and allocate MB structure
     *
     *****/
    vop_quant = GetVopQuantizer (rec_vop);
    QP = vop_quant;
    
    mblock = (Macroblock *) malloc (sizeof (Macroblock));
    
    if (mblock == NULL)
    {
        printf (" Error allocating macroblock\n");
        exit(1);
    }
    
#ifdef __WRITE_IM__    
    printf("Start I VOP decoding \n");
#endif
    
    /*****
     *
     *	Loop over all MBs, decode texture of each MB
     *
     *****/
    new_vop = 1;
    pMB_decisions = (SInt *)GetImageData(MB_decisions);
    
    amb_type = 0;  /* No shape coding: opaque block */
    mbnum = 0;
    counter = 0;
    after_marker = 0;
    skip_texture = 0;
    slice_counter = 0;
    start_of_packet = 1;
    
    if (GetVopShape(rec_vop)) AllocShapePacket2(rec_vop,&alpha_rec_packet,&alpha_mode_packet);
    
    while(mbnum < MB_in_width*MB_in_height)
    {
        
        x_pos = mbnum % MB_in_width;
        y_pos = mbnum / MB_in_width;
        
        slice_nb[x_pos][y_pos] = slice_counter;
	
        trace->trace = trace->MB_header;
        if (trace->trace)
	{
            fprintf (trace->fp_trace, "\n>>>   Decoding Macroblock %d ........- Combined Shape/Text\n",(int)mbnum);
	}
	
        /* Store the QP value for later use in AC prediction */
        QP_store[mbnum] = (SInt) QP;
        
        /* decode COD, MCPC, ACpred_flag, CPBY and DQUANT */
        header_data[mbnum].coded = 0;
        header_data[mbnum].skipped_flag = 1;
        
        amb_type = 0;
        
        /* set all blocks to opaque */
        for(i=0;i<4;i++) 
            header_data[mbnum].MB_transp_pattern[i]=0;
        
        if (GetVopShape(rec_vop)) 
        {
            GetMBIntraShapeErrRes(stream,vo_id,vol_id,y_pos,x_pos,
                                  MB_decisions,start_of_packet,&amb_type,
                                  header_data[mbnum].MB_transp_pattern,
                                  error_flag,rec_vop,trace,MB_TYPE2,
								  PRE_H_SIZE2,PRE_V_SIZE2,PRE_MB_TYPE2,
								  alpha_rec_packet,alpha_mode_packet);
            start_of_packet = 0;
        }
        
        if (GetVopShape(rec_vop) != BINARY_SHAPE_ONLY)  
        { 
            if (amb_type != 1) 
            {
                GetMBheaderNoDataPartIntraErrRes(stream,intra_dcpred_disable,
                                                 mbnum,trace,MB_decisions,
                                                 &header_data[mbnum].skipped_flag, error_flag,
                                                 &header_data[mbnum].coded, 
                                                 &header_data[mbnum].CBP,
                                                 &header_data[mbnum].ACpred_flag,
                                                 &header_data[mbnum].mode,
                                                 header_data[mbnum].MB_transp_pattern);
                
                QP_store[mbnum] = (SInt) vop_quant;
                
                if (header_data[mbnum].coded)
                {
                    if (*error_flag != RETURN)
                        GetMBblockdataNoDataPartErrRes(
                            header_data[mbnum].MB_transp_pattern,
                            intra_dcpred_disable,
                            vop_quant,stream,mbnum,
                            trace,mblock,error_flag, 
                            MB_in_width,
                            DC_store,QP_store,QP,
                            header_data[mbnum].CBP,
                            header_data[mbnum].ACpred_flag,
                            header_data[mbnum].mode,
                            rec_vop, slice_nb);
                    QP = vop_quant;
                }
                
                
                /* (skipped_flag==1) Clear the skipped MB */
                if (header_data[mbnum].skipped_flag == 1)
                    MBzero (mblock);
            }
            else
            {
                MBzero (mblock);
                /* set all alpha block pixels to zero */
                /* Fill DC_store with default coeff values */
                for (m = 0; m < 6; m++) 
                {
                    DC_store[mbnum][m][0] = 128*8;
                    for (n = 1; n < 15; n++)
                        DC_store[mbnum][m][n] = 0;
                }
            }
            
            
            FillVop (mblock, mbnum, Y_rec, U_rec, V_rec);
        }					/* BSO_ONLY */
        mbnum++;
        
        
        
        if (((BitstreamShowBitsByteAlign(stream,RESYNC_MARKER_LENGTH)) ==
             RESYNC_MARKER) && (CheckBitStuffing(stream))){
            start_of_packet = 1;
            BitstreamByteAlign(stream);
            BitstreamReadBits(stream,RESYNC_MARKER_LENGTH,"resync_marker",trace,NUM);
            next_MB = BitstreamReadBits(stream,MB_in_VOP_length,"macro_block_num",trace,NUM);
            if (GetVopShape(rec_vop) != BINARY_SHAPE_ONLY)
		BitstreamReadBits(stream,5,"quant_scale",trace,NUM);
            if (BitstreamReadBits(stream,1,"header_extension_code",trace,NUM)){
                /* modulo_time_base (? bits) */
                tmpvar = (Int) BitstreamReadBits(stream,1,"modulo_time_base",trace,FLAG);
                
                while(tmpvar == 1)
                {
                    tmpvar = (Int) BitstreamReadBits(stream,1,"modulo_time_base",trace,FLAG);
                }
                
                /* vop_time_increment (1-15 bits) */
                bits = (Int)ceil(log((double)GetVopTimeIncrementResolution(rec_vop))/log(2.0));
                if (bits<1) bits=1;
                BitstreamReadBits(stream,1,"marker_bit",trace,NUM);
                tmpvar = (Int) BitstreamReadBits(stream,bits,"vop_time_increment",trace,NUM);
                BitstreamReadBits(stream,1,"marker_bit",trace,NUM);
                
                /* vop_prediction_type (2 bits) */
                BitstreamReadBits(stream,2,"vop_prediction_type",trace,NUM);
            }
            after_marker = 1;
            slice_counter++;
        }
        
        if ((BitstreamShowBitsByteAlign(stream,24)) == 252){
            BitstreamByteAlign(stream);
        }
    }
    
    
    /* Free allocated memory for 3D matrix */
    
    for (i = 0; i < MB_in_width*MB_in_height; i++)
    {
        for (j = 0; j < 6; j++) 
            free((Char *)DC_store[i][j]);
        free((Char *)DC_store[i]);
    }
    free((Char *)DC_store);
    
    
    if (GetVopShape(rec_vop)) FreeShapePacket2(alpha_rec_packet,alpha_mode_packet);
    
    /* Free allocated memory for 2D matrix */
    
    for (i = 0; i < MB_in_width; i++)
        free(slice_nb[i]);
    free(slice_nb);
    
    
    FreeImage(MB_decisions);
    free ((Char *)mblock);
    
    free(header_data);
    
    return;
}


/***********************************************************CommentBegin******
 *
 * -- DecodeVopCombinedMotionShapeTextureInterErrRes -- 
 *
 * Author :		
 *	Luis Ducla-Soares (IST) - lds@lx.it.pt
 *
 * Created :		
 *	1-May-97
 *
 * Purpose :		
 *      Decodes error resilient combined motion / shape / texture for INTER
 *      VOPs with data partitioning
 *
 * Arguments in : 	
 *	Bitstream *stream	combined shape / texture bitstream
 *	Trace  *trace		flag for tracing the process.
 *
 *
 * Arguments in/out :	
 *	Vop *rec_vop		alpha plane is supposed to be filled
 *				already / YUV components.
 *
 * Arguments out :	
 *
 *
 * Return values :	
 *	
 *
 * Side effects :	
 *	
 *
 * Description :	
 *
 *
 * Note : based on DecodeVopCombinedMotionShapeTextureInter() as of 01.05.97
 *
 *
 * See also :
 *	
 *
 * Modified : 
 *      15.05.97 Luis Ducla-Soares: corrected the remultiplexing of the DCT data
 *                                  in the combined error resilient mode with 
 *                                  data partitioning. Cleaned up the code.
 *      04.11.97 Minhua Zhou: updated DC/AC prediction
 *			19.11.97 Noel Brady: quant_scale not present when in shape only mode
 *			19.11.97 Noel Brady: fcodes are now in HEC (except for shape only mode)
 *			19.11.97 Noel Brady: added support for shape decoding
 *
 ***********************************************************CommentEnd********/
Void
CallingConvention
DecodeVopCombinedMotionShapeTextureInterErrRes(
			Bitstream * stream,Int vo_id, Trace *trace, Vop *rec_vop,
			UChar **MB_TYPE2[MAX_NUM_VOS][MAX_NUM_VOLS],
			Int PRE_H_SIZE2[MAX_NUM_VOS][MAX_NUM_VOLS],
			Int PRE_V_SIZE2[MAX_NUM_VOS][MAX_NUM_VOLS],
			UChar **PRE_MB_TYPE2[MAX_NUM_VOS][MAX_NUM_VOLS],
			Image   *alpha_rec_packet,
			Image   *alpha_mode_packet)
{    
    Int           ef=0, *error_flag = &ef; 
    Int           mbnum, x_pos, y_pos, i, j;
    Int           rec_height;
    Int         rec_width;
    Int		MB_in_height, MB_in_width;
    Int           B_in_width, B_in_height;
    Macroblock    *mblock;
    Image         *Y_rec, *U_rec, *V_rec;
    Int           vop_quant, intra_dcpred_disable;
    Image         *alpha_decisions,*MB_decisions;
    SInt		*palpha_decisions;
    Int 		MB_transp_pattern[4];
    Int           ***DC_store;
    Int           **slice_nb;
    Int           slice_counter;
    UInt          mode;
    UInt        *CBP_store;
    Int           f_code=GetVopFCodeFor(rec_vop);
    Image         *mot_x, *mot_y;
    Vop           *tmp_vop;
    SInt          *QP_store;
    Int           QP;
    Int           counter, tmpvar, tmpvar1, tmpvar2;
    Int           next_MB;
    Int           MB_in_VOP;
    Int           MB_in_VOP_length;
    HeaderInfo    *header_data;
    Int           after_marker;
    Int           skip_texture;
    Int           bits;
    Int           resync_marker_length;
    
    Image *modeA,*motA_x,*motA_y;
    Int start_of_packet,m,n;
    Int amb_type,vol_id = GetVopId(rec_vop);
    SInt *mb_mode;
    
    /*****
     *
     *   Stuff for motion compensation of alpha plane (copied by JDL 
     *   from AlphaMotionCompensate() in vm_common/src/mot_comp.c,
     *   a function written by Noel Brady
     *
     *****/
    
    
/* SPRITE */	
    Vop *sprite=NULL;
    
    if (GetVopSpriteUsage(rec_vop)!=SPRITE_NOT_USED)
    { 
	sprite = GetVopSprite(rec_vop);
    }
    
    
    /*****
     *
     *	Set some useful parameters
     *
     *****/
    rec_height = GetVopHeight (rec_vop);
    rec_width = GetVopWidth (rec_vop);
    MB_in_height = rec_height / MB_SIZE;
    MB_in_width = rec_width / MB_SIZE;
    B_in_width = 2 * MB_in_width;
    B_in_height = 2 * MB_in_height;
    QP_store = (SInt *) GetImageData(GetVopQP(rec_vop));
    MB_in_VOP = MB_in_width * MB_in_height;
    MB_in_VOP_length = (Int) ceil(log(MB_in_VOP)/log(2));
    resync_marker_length = 16 + f_code;
    
    
    mot_x = AllocImage((UInt) B_in_width,(UInt) B_in_height,FLOAT_TYPE);
    mot_y = AllocImage((UInt) B_in_width,(UInt) B_in_height,FLOAT_TYPE);
    
    if (GetVopShape(rec_vop))
    {
        /* Allocate shape ME/MC */
        modeA = AllocImage(GetVopWidth(rec_vop)/MB_SIZE,
                           GetVopHeight(rec_vop)/MB_SIZE,SHORT_TYPE);
        motA_x = AllocImage(GetVopWidth(rec_vop)/MB_SIZE,
                            GetVopHeight(rec_vop)/MB_SIZE,SHORT_TYPE);
        motA_y = AllocImage(GetVopWidth(rec_vop)/MB_SIZE,
                            GetVopHeight(rec_vop)/MB_SIZE,SHORT_TYPE);
        if (GetVopShape(rec_vop)) AllocShapePacket2(rec_vop,&alpha_rec_packet,&alpha_mode_packet);
    }
    
    header_data = (HeaderInfo *)calloc(MB_in_width*MB_in_height,sizeof(HeaderInfo));
    
    CBP_store = (UInt  *)calloc(MB_in_width*MB_in_height,sizeof(UInt));
    
    ZeroImageF(mot_x);
    ZeroImageF(mot_y);

    /* allocate space for 3D matrix to keep track of prediction values
       for DC/AC prediction */
    
    DC_store = (Int ***)calloc(MB_in_width*MB_in_height,
			       sizeof(Int **));
    for (i = 0; i < MB_in_width*MB_in_height; i++)
    {
	DC_store[i] = (Int **)calloc(6, sizeof(Int *));
	for (j = 0; j < 6; j++)
            DC_store[i][j] = (Int *)calloc(15, sizeof(Int));
    }
    
    
    /* allocate space for 2D matrix to keep track of the slice numbers */
    
    slice_nb = (Int **)calloc(MB_in_width,sizeof(Int *));
    
    for (i = 0; i < MB_in_width; i++)
        slice_nb[i] = (Int *)calloc(MB_in_height, sizeof(Int));
    
    
    intra_dcpred_disable = GetVopIntraACDCPredDisable(rec_vop);
    
    alpha_decisions = AllocImage(B_in_width,B_in_height,SHORT_TYPE);
    MB_decisions    = AllocImage(MB_in_width,MB_in_height,SHORT_TYPE);
    mb_mode = (SInt *) GetImageData(MB_decisions);
    
    tmp_vop = AllocVop(GetVopWidth(rec_vop),GetVopHeight(rec_vop));
    CopyVopNonImageField(rec_vop,tmp_vop);
    
    /*****
     *
     *	Get the image fields of the VOP
     *
     *****/
    Y_rec = GetVopY (rec_vop);
    U_rec = GetVopU (rec_vop);
    V_rec = GetVopV (rec_vop);
    
    
    /*****
     *
     *	Get VOP quantizer and allocate MB structure
     *
     *****/
    vop_quant = GetVopQuantizer (rec_vop);
    QP = vop_quant;
    
    mblock = (Macroblock *) malloc (sizeof (Macroblock));
    
    if (mblock == NULL)
    {
	printf (" Error allocating macroblock\n");
	exit(1);
    }
    
#ifdef __WRITE_IM__    
    if (GetVopSpriteUsage(rec_vop)==ONLINE_SPRITE)
	printf("Start SPRITE VOP decoding\n");
    else
        if (GetVopSpriteUsage(rec_vop)==GMC_SPRITE)
            printf("Start GMC VOP decoding\n");
        else
            printf("Start P VOP decoding\n");
#endif
    
    /*****
     *
     *	Loop over all MBs, decode texture of each MB
     *
     *****/
    palpha_decisions = (SInt *)GetImageData(alpha_decisions);
    
    /* initialise values that indicate blockwise whether */
    /* the block is transparent                          */
    for (i = 0; i < B_in_width*B_in_height; i++)
        palpha_decisions[i] = 0; /*MBM_TRANSPARENT;*/
    
    for(i=0;i<4;i++) 
        MB_transp_pattern[i]=0;
    
    /*****
     *
     * the macroblock loop
     *
     *****/
    mbnum = 0;
    counter = 0;
    after_marker = 0;
    skip_texture = 0;
    slice_counter = 0;
    
    
    while(mbnum < MB_in_width*MB_in_height)
    {
        start_of_packet = 1;
        
	while (((BitstreamShowBits(stream,MOTION_MARKER_COMB_LENGTH))!=MOTION_MARKER_COMB) &&
	       (mbnum < MB_in_width * MB_in_height)){
            x_pos = mbnum % MB_in_width;
            y_pos = mbnum / MB_in_width;
            
            slice_nb[x_pos][y_pos] = slice_counter;
            
            trace->trace = trace->MB_header;
            
            if (trace->trace)
	    {
                fprintf (trace->fp_trace,"\n>>>   Decoding Macroblock %d ........- Combined Motion/Shape/Text\n",(int)mbnum);
	    }
            
            /* Store the QP value for later use in AC prediction */
            QP_store[mbnum] = (SInt) QP;
            
            /* decode COD, MCPC, ACpred_flag, CPBY and DQUANT */
            header_data[mbnum].coded = 0;
            header_data[mbnum].skipped_flag = 1;
            amb_type = 0;
            
            /* set all blocks to opaque */
            for(i=0;i<4;i++) 
                header_data[mbnum].MB_transp_pattern[i]=0;
            
            if (GetVopPredictionType(rec_vop) == SPRITE_VOP)
                mb_mode[mbnum] = MBM_SPRITE;
            else   /* P_VOP */
                mb_mode[mbnum] = MBM_OPAQUE;
            
            if (GetVopShape(rec_vop)) 
            {
#if 0
                printf("\n\nMB: %d,%d (%d)\n",x_pos,y_pos,mbnum);
#endif
                GetMBInterShapeErrRes(stream,GetVopNextTemp(rec_vop),
                                      vo_id,vol_id,y_pos,x_pos,mbnum,
                                      MB_decisions,alpha_decisions,mot_x,mot_y,
                                      modeA,motA_x,motA_y,start_of_packet,&amb_type,
                                      header_data[mbnum].MB_transp_pattern,
                                      error_flag,rec_vop,trace,MB_TYPE2,
									  PRE_H_SIZE2,PRE_V_SIZE2,PRE_MB_TYPE2);
#if 0
                printf("transp pattern: %d %d %d %d\n",
                       header_data[mbnum].MB_transp_pattern[0],
                       header_data[mbnum].MB_transp_pattern[1],
                       header_data[mbnum].MB_transp_pattern[2],
                       header_data[mbnum].MB_transp_pattern[3]);
#endif
                start_of_packet = 0;
            }
            
            if (!*error_flag)
            {
                if (mb_mode[mbnum] != MBM_TRANSPARENT) 
                {
                    GetMBheaderMotionErrRes(stream,mbnum,trace,  
                                            &header_data[mbnum].skipped_flag, 
                                            error_flag,DC_store,
                                            &header_data[mbnum].coded, 
                                            &header_data[mbnum].CBP,
                                            &header_data[mbnum].mode,
                                            header_data[mbnum].MB_transp_pattern);
                    
                    QP_store[mbnum] = (SInt) vop_quant;
                    mode = header_data[mbnum].mode;
                    if (!header_data[mbnum].coded) mb_mode[mbnum] = SKIPPED;
                }
                else
                {
                    MBzero (mblock);
                    /* Fill DC_store with default coeff values */
                    for (m = 0; m < 6; m++) 
                    {
                        DC_store[mbnum][m][0] = 128*8;
                        for (n = 1; n < 15; n++)
                            DC_store[mbnum][m][n] = 0;
                    }
                }
            } 
            
            if (!*error_flag)
	    {
                /* decode the MV for texture */
                
                if ((header_data[mbnum].coded) && (mode == MODE_INTER || mode == MODE_INTER_Q || 
                                                   mode == MODE_INTER4V || mode == MODE_INTRA || mode == MODE_DYN_SP))
                    GetMBvectorsErrRes(intra_dcpred_disable,stream,mbnum,  
                                       MB_in_width,f_code,trace,MB_decisions,
                                       alpha_decisions, mot_x, mot_y, error_flag,
                                       header_data[mbnum].mode, 
                                       GetVopErrorResDisable(rec_vop), 
                                       after_marker, slice_nb);
                
                after_marker = 0;
                
	    }
	    
            
            if (*error_flag){
                while (((BitstreamShowBits(stream,MOTION_MARKER_COMB_LENGTH)) != MOTION_MARKER_COMB) &&
                       (((BitstreamShowBitsByteAlign(stream,resync_marker_length)) != RESYNC_MARKER) || (!CheckBitStuffing(stream))) &&
                       (((BitstreamShowBitsByteAlign(stream,VOP_START_CODE_LENGTH)) !=VOP_START_CODE) || (!CheckBitStuffing(stream))) &&
                       (((BitstreamShowBitsByteAlign(stream,24)) != 252) || (!CheckBitStuffing(stream))))
                    BitstreamFlushBits(stream,1);
                
                if (((BitstreamShowBitsByteAlign(stream,resync_marker_length)) == RESYNC_MARKER) || 
                    ((BitstreamShowBitsByteAlign(stream,VOP_START_CODE_LENGTH)) == VOP_START_CODE) ||
                    ((BitstreamShowBitsByteAlign(stream,24)) == 252)){
                    skip_texture = 1;
                    break;
		}
            }
            else {
                mbnum++;
                counter++;
            }
	}
	
        
	if (!skip_texture)
        {
	    while ((BitstreamShowBits(stream,MOTION_MARKER_COMB_LENGTH)!=MOTION_MARKER_COMB)) 
            {
                BitstreamFlushBits(stream,1);
            }
	    BitstreamReadBits(stream,MOTION_MARKER_COMB_LENGTH,"motion marker",trace,NUM); 
        }
        
	tmpvar1 = mbnum - counter;
	tmpvar2 = tmpvar1;
	counter = 0;
        
	if (!skip_texture)
        {
	    if (!*error_flag)
            {
		while (tmpvar1 < mbnum)
                {
		    /* decode the DCT coeficients for the MB */
		    if (header_data[tmpvar1].coded)
                    {
			
			GetMBheaderTextInterErrRes(intra_dcpred_disable,&vop_quant, 
						   stream,tmpvar1,trace,MB_decisions,
						   error_flag,&header_data[tmpvar1].coded, 
						   &header_data[tmpvar1].CBP, 
						   &header_data[tmpvar1].ACpred_flag,
						   &header_data[tmpvar1].mode,
                                                   header_data[tmpvar1].MB_transp_pattern,
						   header_data[tmpvar1].IntraDC,
						   rec_vop);
                        
                    }
		    tmpvar1++;
                }
                
		while (tmpvar2 < mbnum)
                {
		    if (header_data[tmpvar2].coded)
                    {
			if (*error_flag != RETURN)
                            GetMBblockdataErrRes(header_data[tmpvar2].MB_transp_pattern,  
                                                 intra_dcpred_disable,vop_quant,
                                                 stream,tmpvar2,trace,
                                                 mblock,error_flag,MB_in_width,
                                                 DC_store,QP_store,QP,
                                                 header_data[tmpvar2].CBP,
                                                 header_data[tmpvar2].ACpred_flag,
                                                 header_data[tmpvar2].mode,
                                                 header_data[tmpvar2].IntraDC,
                                                 rec_vop,slice_nb);
			
			
                    }
		    
		    /* (skipped_flag==1) Clear the skipped MB */
		    if (header_data[tmpvar2].skipped_flag == 1)
                        MBzero (mblock);
		    
                    
		    FillVop (mblock, tmpvar2, Y_rec, U_rec, V_rec);
		    tmpvar2++;
                }
            }
        }
	else
            skip_texture = 0;
        
	if (*error_flag){
            while (((BitstreamShowBitsByteAlign(stream,resync_marker_length) != RESYNC_MARKER) || (!CheckBitStuffing(stream))) &&
                   ((BitstreamShowBitsByteAlign(stream,VOP_START_CODE_LENGTH) != VOP_START_CODE) || (!CheckBitStuffing(stream))) &&
                   ((BitstreamShowBitsByteAlign(stream,24) != 252) || (!CheckBitStuffing(stream))))
                BitstreamFlushBits(stream,1);
            *error_flag = 0;
	}
        
        
        
	if ((BitstreamShowBitsByteAlign(stream,resync_marker_length)) == RESYNC_MARKER){
            BitstreamByteAlign(stream);
            BitstreamReadBits(stream,resync_marker_length,"resync_marker",trace,NUM);
            next_MB = BitstreamReadBits(stream,MB_in_VOP_length,"macro_block_num",trace,NUM);
            if (GetVopShape(rec_vop) != BINARY_SHAPE_ONLY) 
	  	BitstreamReadBits(stream,5,"quant_scale",trace,NUM);
            if (BitstreamReadBits(stream,1,"header_extension_code",trace,NUM)){
                /* modulo_time_base (? bits) */
                tmpvar = (Int) BitstreamReadBits(stream,1,"modulo_time_base",trace,FLAG);
                
                while(tmpvar == 1)
                {
                    tmpvar = (Int) BitstreamReadBits(stream,1,"modulo_time_base",trace,FLAG);
                }
                
                /* vop_time_increment (1-15 bits) */
                bits = (Int)ceil(log((double)GetVopTimeIncrementResolution(rec_vop))/log(2.0));
                if (bits<1) bits=1;
                BitstreamReadBits(stream,1,"marker_bit",trace,NUM);
                tmpvar = (Int) BitstreamReadBits(stream,bits,"vop_time_increment",trace,NUM);
                BitstreamReadBits(stream,1,"marker_bit",trace,NUM);
                
                /* vop_prediction_type (2 bits) */
                BitstreamReadBits(stream,2,"vop_prediction_type",trace,NUM);
                
                /* fcodes */
                if (GetVopShape(rec_vop) != BINARY_SHAPE_ONLY) /* Noel/E14 */
                {
                    BitstreamReadBits(stream,3,"fcode_for",trace,NUM);
                    if (GetVopPredictionType(rec_vop) == B_VOP) 
                        BitstreamReadBits(stream,3,"fcode_back",trace,NUM);
                }
            }
            after_marker = 1;
            slice_counter++;
	}
	else
            next_MB = MB_in_width * MB_in_height;
	
	if ((mbnum != next_MB) && (next_MB < mbnum))
            mbnum = next_MB;
	else if (next_MB > mbnum){
            while ((((BitstreamShowBitsByteAlign(stream,VOP_START_CODE_LENGTH) !=VOP_START_CODE)) || (!CheckBitStuffing(stream))) &&
                   (((BitstreamShowBitsByteAlign(stream,24)) != 252) || (!CheckBitStuffing(stream))))
                BitstreamFlushBits(stream,1);
	}
	
	if ((BitstreamShowBitsByteAlign(stream,24)) == 252){
            BitstreamByteAlign(stream);
	}
    }
    
    
    
    /* mapping MB_decisions to alpha_decisions in order to make */
    /* VopMotionCompensate() work                               */
    
    RemapAlphaDecisions(alpha_decisions);
    
    VopMotionCompensate(GetVopNextTemp(rec_vop),mot_x,mot_y,MB_decisions,
			alpha_decisions,
                        CBP_store,
                        tmp_vop, 
			GetVopOBMCDisable(rec_vop),
			sprite); 
    
    
    VopTextureUpdate(tmp_vop,rec_vop,trace);
    
    /* Free allocated memory for 3D matrix */
    
    for (i = 0; i < MB_in_width*MB_in_height; i++)
    {
	for (j = 0; j < 6; j++)
            free((Char *)DC_store[i][j]);
	free((Char *)DC_store[i]);
    }
    free((Char *)DC_store);
    
    
    /* Free allocated memory for 2D matrix */
    
    for (i = 0; i < MB_in_width; i++)
        free(slice_nb[i]);
    free(slice_nb);
    
    
    if (GetVopShape(rec_vop))
    {
        FreeImage(modeA);
        FreeImage(motA_x);
        FreeImage(motA_y);
        FreeShapePacket2(alpha_rec_packet,alpha_mode_packet);
    }
    FreeImage(alpha_decisions);
    FreeImage(MB_decisions);
    free ((Char *)mblock);
    FreeImage(mot_x);
    FreeImage(mot_y);
    FreeVop(tmp_vop);
    
    free(header_data);
    
    return;
}


/***********************************************************CommentBegin******
 *
 * -- DecodeVopCombNoDataPartMotionShapeTextureInterErrRes -- 
 *
 * Author :		
 *	Luis Ducla-Soares (IST) - lds@lx.it.pt
 *
 * Created :		
 *	18-Sep-97
 *
 * Purpose :		
 *      Decodes error resilient combined motion / shape / texture for INTER
 *      VOPs with data partitioning
 *
 * Arguments in : 	
 *	Bitstream *stream	combined shape / texture bitstream
 *	Trace  *trace		flag for tracing the process.
 *
 *
 * Arguments in/out :	
 *	Vop *rec_vop		alpha plane is supposed to be filled
 *				already / YUV components.
 *
 * Arguments out :	
 *
 *
 * Return values :	
 *	
 *
 * Side effects :	
 *	
 *
 * Description :	
 *
 *
 * Note : based on DecodeVopCombinedMotionShapeTextureInter() as of 01.05.97
 *
 *
 * See also :
 *	
 *
 * Modified : 
 *      15.05.97 Luis Ducla-Soares: corrected the remultiplexing of the DCT data
 *                                  in the combined error resilient mode with 
 *                                  data partitioning. Cleaned up the code.
 *			19.09.97 Noel Brady: quant_scale not present when in shape only mode
 *			19.09.97 Noel Brady: fcodes are now in HEC (except for shape only mode)
 *
 ***********************************************************CommentEnd********/
Void
CallingConvention
DecodeVopCombNoDataPartMotionShapeTextureInterErrRes(
			Bitstream * stream, Int vo_id, Trace *trace, Vop *rec_vop,
			UChar **MB_TYPE2[MAX_NUM_VOS][MAX_NUM_VOLS],
			Int PRE_H_SIZE2[MAX_NUM_VOS][MAX_NUM_VOLS],
			Int PRE_V_SIZE2[MAX_NUM_VOS][MAX_NUM_VOLS],
			UChar **PRE_MB_TYPE2[MAX_NUM_VOS][MAX_NUM_VOLS],
			Image   *alpha_rec_packet,
			Image   *alpha_mode_packet)
{    
    Int           ef=0, *error_flag = &ef; 
    Int           mbnum, x_pos, y_pos, i, j;
    Int           rec_height;
    Int 		rec_width;
    Int		MB_in_height, MB_in_width;
    Int           B_in_width, B_in_height;
    Macroblock    *mblock;
    Image         *Y_rec, *U_rec, *V_rec;
    Int           vop_quant,intra_dcpred_disable;
    Image         *alpha_decisions,*MB_decisions;
    SInt		*palpha_decisions,*mb_mode;
    Int 		MB_transp_pattern[4];
    Int           ***DC_store;
    Int           **slice_nb;
    Int           slice_counter;
    UInt          mode;
    Int           f_code=GetVopFCodeFor(rec_vop);
    Image         *mot_x, *mot_y;
    Vop           *tmp_vop;
    SInt          *QP_store;
    UInt          *CBP_store;
    Int           QP;
    Int           counter, tmpvar;
    Int           next_MB;
    Int           MB_in_VOP;
    Int           MB_in_VOP_length;
    HeaderInfo    *header_data;
    Int           after_marker;
    Int           skip_texture;
    Int           bits;
    Int           resync_marker_length;
    
    Int start_of_packet,
        vol_id = GetVopId(rec_vop),
        amb_type,
        m,n;
    
    Image *modeA; /* the flag of current alpha block is inter coded or not */
    Image *motA_x; /* the x compornent of shape MV */
    Image *motA_y; /* the y compornent of shape MV */
    
    
/* SPRITE */	
    Vop *sprite=NULL;
    
    if (GetVopSpriteUsage(rec_vop)!=SPRITE_NOT_USED)
    { 
	sprite = GetVopSprite(rec_vop);
    }
    
    
    /*****
     *
     *	Set some useful parameters
     *
     *****/
    rec_height = GetVopHeight (rec_vop);
    rec_width = GetVopWidth (rec_vop);
    MB_in_height = rec_height / MB_SIZE;
    MB_in_width = rec_width / MB_SIZE;
    B_in_width = 2 * MB_in_width;
    B_in_height = 2 * MB_in_height;
    QP_store = (SInt *) GetImageData(GetVopQP(rec_vop));
    MB_in_VOP = MB_in_width * MB_in_height;
    MB_in_VOP_length = (Int) ceil(log(MB_in_VOP)/log(2));
    
    if (GetVopShape(rec_vop) == BINARY_SHAPE_ONLY)
    	resync_marker_length = 17;
    else
    	resync_marker_length = 16 + f_code;
    
    
    mot_x = AllocImage((UInt) B_in_width,(UInt) B_in_height,FLOAT_TYPE);
    mot_y = AllocImage((UInt) B_in_width,(UInt) B_in_height,FLOAT_TYPE);
    
    if (GetVopShape(rec_vop))
    {
        /* Allocate shape ME/MC */
        modeA = AllocImage(GetVopWidth(rec_vop)/MB_SIZE,
                           GetVopHeight(rec_vop)/MB_SIZE,SHORT_TYPE);
        motA_x = AllocImage(GetVopWidth(rec_vop)/MB_SIZE,
                            GetVopHeight(rec_vop)/MB_SIZE,SHORT_TYPE);
        motA_y = AllocImage(GetVopWidth(rec_vop)/MB_SIZE,
                            GetVopHeight(rec_vop)/MB_SIZE,SHORT_TYPE);
    }
    
    header_data = (HeaderInfo *)calloc(MB_in_width*MB_in_height,sizeof(HeaderInfo));
    
    CBP_store = (UInt  *)calloc(MB_in_width*MB_in_height,sizeof(UInt));
    
    ZeroImageF(mot_x);
    ZeroImageF(mot_y);
    
    /* allocate space for 3D matrix to keep track of prediction values
       for DC/AC prediction */
    
    DC_store = (Int ***)calloc(MB_in_width*MB_in_height,
			       sizeof(Int **));
    for (i = 0; i < MB_in_width*MB_in_height; i++)
    {
	DC_store[i] = (Int **)calloc(6, sizeof(Int *));
	for (j = 0; j < 6; j++)
            DC_store[i][j] = (Int *)calloc(15, sizeof(Int));
    }
    
    
    /* allocate space for 2D matrix to keep track of the slice numbers */
    
    slice_nb = (Int **)calloc(MB_in_width,sizeof(Int *));
    
    for (i = 0; i < MB_in_width; i++)
        slice_nb[i] = (Int *)calloc(MB_in_height, sizeof(Int));
    
    
    intra_dcpred_disable = GetVopIntraACDCPredDisable(rec_vop);
    
    alpha_decisions = AllocImage(B_in_width,B_in_height,SHORT_TYPE);
    MB_decisions    = AllocImage(MB_in_width,MB_in_height,SHORT_TYPE);
    
    tmp_vop = AllocVop(GetVopWidth(rec_vop),GetVopHeight(rec_vop));
    CopyVopNonImageField(rec_vop,tmp_vop);
    
    /*****
     *
     *	Get the image fields of the VOP
     *
     *****/
    Y_rec = GetVopY (rec_vop);
    U_rec = GetVopU (rec_vop);
    V_rec = GetVopV (rec_vop);
    
    
    /*****
     *
     *	Get VOP quantizer and allocate MB structure
     *
     *****/
    vop_quant = GetVopQuantizer (rec_vop);
    QP = vop_quant;
    
    mblock = (Macroblock *) malloc (sizeof (Macroblock));
    
    if (mblock == NULL)
    {
	printf (" Error allocating macroblock\n");
	exit(1);
    }
    
#ifdef __WRITE_IM__    
    if (GetVopSpriteUsage(rec_vop)==ONLINE_SPRITE)
	printf("Start SPRITE VOP decoding\n");
    else
        if (GetVopSpriteUsage(rec_vop)==GMC_SPRITE)
            printf("Start GMC VOP decoding\n");
        else
            printf("Start P VOP decoding\n");
#endif
    
    /*****
     *
     *	Loop over all MBs, decode texture of each MB
     *
     *****/
    mb_mode = (SInt *)GetImageData(MB_decisions);
    palpha_decisions = (SInt *)GetImageData(alpha_decisions);
    
    /* initialise values that indicate blockwise whether */
    /* the block is transparent                          */
    for (i = 0; i < B_in_width*B_in_height; i++)
        palpha_decisions[i] = 0; /*MBM_TRANSPARENT;*/
    
    for(i=0;i<4;i++) 
        MB_transp_pattern[i]=0;
    
    /*****
     *
     * the macroblock loop
     *
     *****/
    mbnum = 0;
    counter = 0;
    after_marker = 0;
    skip_texture = 0;
    slice_counter = 0;
    start_of_packet = 1;
    
    if (GetVopShape(rec_vop)) AllocShapePacket2(rec_vop,&alpha_rec_packet,&alpha_mode_packet);
    
    while(mbnum < MB_in_width*MB_in_height)
    {
        
	x_pos = mbnum % MB_in_width;
	y_pos = mbnum / MB_in_width;
        
	slice_nb[x_pos][y_pos] = slice_counter;
        
	trace->trace = trace->MB_header;
	
	if (trace->trace)
        {
	    fprintf (trace->fp_trace,"\n>>>   Decoding Macroblock %d ........- Combined Motion/Shape/Text\n",(int)mbnum);
        }
        
	/* Store the QP value for later use in AC prediction */
	QP_store[mbnum] = (SInt) QP;
	
        
        /* decode COD, MCPC, ACpred_flag, CPBY and DQUANT */
        header_data[mbnum].coded = 0;
        header_data[mbnum].skipped_flag = 1;
        
        amb_type = 0;
        
  	/* set all blocks to opaque */
  	for(i=0;i<4;i++) 
            header_data[mbnum].MB_transp_pattern[i]=0;
        
        if (GetVopPredictionType(rec_vop) == SPRITE_VOP)
            mb_mode[mbnum] = MBM_SPRITE;
        else   /* P_VOP */
	    mb_mode[mbnum] = MBM_OPAQUE;
        
	if (GetVopShape(rec_vop)) 
        {
            GetMBInterShapeErrRes(stream,GetVopNextTemp(rec_vop),
                                  vo_id,vol_id,y_pos,x_pos,mbnum,
                                  MB_decisions,alpha_decisions,mot_x,mot_y,
                                  modeA,motA_x,motA_y,start_of_packet,&amb_type,
                                  header_data[mbnum].MB_transp_pattern,
                                  error_flag,rec_vop,trace,MB_TYPE2,
								  PRE_H_SIZE2,PRE_V_SIZE2,PRE_MB_TYPE2);
            start_of_packet = 0;
        }
        
	if (GetVopShape(rec_vop) != BINARY_SHAPE_ONLY)
	{
            if (mb_mode[mbnum] != MBM_TRANSPARENT)
            {
                GetMBheaderNoDataPartInterErrRes(stream,intra_dcpred_disable,
                                                 mbnum,trace,
                                                 &header_data[mbnum].skipped_flag,
                                                 error_flag,DC_store,
                                                 &header_data[mbnum].coded, 
                                                 &header_data[mbnum].CBP,
                                                 &header_data[mbnum].ACpred_flag,
                                                 &header_data[mbnum].mode,
                                                 header_data[mbnum].MB_transp_pattern);
                
                QP_store[mbnum] = (SInt) vop_quant;
                mode = header_data[mbnum].mode;
                if (!header_data[mbnum].coded) mb_mode[mbnum] = SKIPPED;
            }
            else
            {
                MBzero (mblock);
                /* Fill DC_store with default coeff values */
                for (m = 0; m < 6; m++) 
                {
                    DC_store[mbnum][m][0] = 128*8;
                    for (n = 1; n < 15; n++)
                        DC_store[mbnum][m][n] = 0;
                }
            }
            
            
            
            if (!*error_flag)
	    {
                /* decode the MV for texture */
                
                if ((header_data[mbnum].coded) && (mode == MODE_INTER || mode == MODE_INTER_Q || 
                                                   mode == MODE_INTER4V || mode == MODE_INTRA || mode == MODE_DYN_SP))
                    GetMBvectorsNoDataPartErrRes(stream,mbnum, MB_in_width,f_code,trace,MB_decisions,
                                                 alpha_decisions, mot_x, mot_y, error_flag,
                                                 header_data[mbnum].mode, header_data[mbnum].IntraDC,
                                                 GetVopErrorResDisable(rec_vop), after_marker,slice_nb);
                
                after_marker = 0;
                
	    }
	    
            
            
            if (header_data[mbnum].coded)
	    {
                if (*error_flag != RETURN)
                    GetMBblockdataNoDataPartErrRes(
                        header_data[mbnum].MB_transp_pattern,
                        intra_dcpred_disable,
                        vop_quant,stream,mbnum,
                        trace,mblock,error_flag, 
                        MB_in_width,
                        DC_store,QP_store,QP,
                        header_data[mbnum].CBP,
                        header_data[mbnum].ACpred_flag,
                        header_data[mbnum].mode,
                        rec_vop, slice_nb);
                
                QP = vop_quant;
                
	    }
            
            /* (skipped_flag==1) Clear the skipped MB */
            if (header_data[mbnum].skipped_flag == 1)
                MBzero (mblock);
            
            
            FillVop (mblock, mbnum, Y_rec, U_rec, V_rec);
        }
        mbnum++;
        
	if (((BitstreamShowBitsByteAlign(stream,resync_marker_length)) ==
	     RESYNC_MARKER) && (CheckBitStuffing(stream))){
            BitstreamByteAlign(stream);
            BitstreamReadBits(stream,resync_marker_length,"resync_marker",trace,NUM);
            next_MB = BitstreamReadBits(stream,MB_in_VOP_length,"macro_block_num",trace,NUM);
            if (GetVopShape(rec_vop) != BINARY_SHAPE_ONLY)
                BitstreamReadBits(stream,5,"quant_scale",trace,NUM);
            if (BitstreamReadBits(stream,1,"header_extension_code",trace,NUM)){
                /* modulo_time_base (? bits) */
                tmpvar = (Int) BitstreamReadBits(stream,1,"modulo_time_base",trace,FLAG);
                
                while(tmpvar == 1)
                {
                    tmpvar = (Int) BitstreamReadBits(stream,1,"modulo_time_base",trace,FLAG);
                }
                
                /* vop_time_increment (1-15 bits) */
                bits = (Int)ceil(log((double)GetVopTimeIncrementResolution(rec_vop))/log(2.0));
                if (bits<1) bits=1;
                BitstreamReadBits(stream,1,"marker_bit",trace,NUM);
                tmpvar = (Int) BitstreamReadBits(stream,bits,"vop_time_increment",trace,NUM);
                BitstreamReadBits(stream,1,"marker_bit",trace,NUM);
                
                /* vop_prediction_type (2 bits) */
                BitstreamReadBits(stream,2,"vop_prediction_type",trace,NUM);
                
                /* fcodes */
                if (GetVopShape(rec_vop) != BINARY_SHAPE_ONLY) /* Noel/E14 */
                {
                    BitstreamReadBits(stream,3,"fcode_for",trace,NUM);
                    if (GetVopPredictionType(rec_vop) == B_VOP) 
                        BitstreamReadBits(stream,3,"fcode_back",trace,NUM);
                }
            }
            after_marker = 1;
            slice_counter++;
            start_of_packet = 1;
	}
        
	
	if ((BitstreamShowBitsByteAlign(stream,24)) == 252){
            BitstreamByteAlign(stream);
	}
    }
    
    
    
    /* mapping MB_decisions to alpha_decisions in order to make */
    /* VopMotionCompensate() work                               */
    
    RemapAlphaDecisions(alpha_decisions);
    
    VopMotionCompensate(GetVopNextTemp(rec_vop),mot_x,mot_y,MB_decisions,
			alpha_decisions,CBP_store, tmp_vop, 
			GetVopOBMCDisable(rec_vop),
			sprite); 
    
    
    VopTextureUpdate(tmp_vop,rec_vop,trace);
    
    if (GetVopShape(rec_vop)) FreeShapePacket2(alpha_rec_packet,alpha_mode_packet);
    
    /* Free allocated memory for 3D matrix */
    
    for (i = 0; i < MB_in_width*MB_in_height; i++)
    {
	for (j = 0; j < 6; j++)
            free((Char *)DC_store[i][j]);
	free((Char *)DC_store[i]);
    }
    free((Char *)DC_store);
    
    
    /* Free allocated memory for 2D matrix */
    
    for (i = 0; i < MB_in_width; i++)
        free(slice_nb[i]);
    free(slice_nb);
    
    if (GetVopShape(rec_vop))
    {
        FreeImage(modeA);
        FreeImage(motA_x);
        FreeImage(motA_y);
    }
    FreeImage(alpha_decisions);
    FreeImage(MB_decisions);
    free ((Char *)mblock);
    FreeImage(mot_x);
    FreeImage(mot_y);
    FreeVop(tmp_vop);
    
    free(header_data);
    
    return;
}

Void
CallingConvention
GetMBIntraShapeErrRes(  Bitstream *stream, Int vo_id, 
                        Int vol_id, Int y_pos, Int x_pos,
                        Image *MB_decisions, Int start_of_packet, 	
                        Int *amb_type, Int *MB_transp_pattern,
                        Int *error_flag, Vop *rec_vop, Trace *trace,
						UChar **MB_TYPE2[MAX_NUM_VOS][MAX_NUM_VOLS],
						Int PRE_H_SIZE2[MAX_NUM_VOS][MAX_NUM_VOLS],
						Int PRE_V_SIZE2[MAX_NUM_VOS][MAX_NUM_VOLS],
						UChar **PRE_MB_TYPE2[MAX_NUM_VOS][MAX_NUM_VOLS],
						Image   *alpha_rec_packet,Image   *alpha_mode_packet)
{
    Int *mvda=NULL;
    SInt *alpha;
    Int rec_width;
    SInt i,j;
    
    /* decode the shape codes */
    
    DecodeFirst(stream, 
                vo_id, 
                vol_id, 
                y_pos, 
                x_pos, 
                amb_type, 
                mvda, 
                0, 
                start_of_packet,
                rec_vop,
                trace,MB_TYPE2,
				PRE_H_SIZE2,PRE_V_SIZE2,PRE_MB_TYPE2);
    
    /* fill the alpha map with the decoded pixels for this MB */
    AlphaDecodeMB(rec_vop, 
                  (Vop *)NULL, 
                  vo_id,
                  vol_id,
                  y_pos, 
                  x_pos, 
                  amb_type,
                  mvda, 
                  0,
                  MB_decisions,
                  (Image *)NULL,
                  (Image *)NULL,
                  (Image *)NULL,
                  (Image *)NULL,	
                  (Image *)NULL,
                  (Image *)NULL,
                  start_of_packet,
                  stream,
                  trace,
				  alpha_rec_packet,
				  alpha_mode_packet);
    
    /* calculating MB_transp_pattern */
    for(i=0;i<4;i++)
        MB_transp_pattern[i]=1;
    
    if (*amb_type != 1) /* if not a transparent MB */
    {
        alpha = (SInt *) GetImageData(GetVopA(rec_vop));
        rec_width = GetVopWidth(rec_vop);
        
        for(i=0;i<B_SIZE;i++)
            for(j=0;j<B_SIZE;j++)
                if (alpha[((y_pos * MB_SIZE) +i)*rec_width + 
                         (x_pos * MB_SIZE) +j] != 0)
                    MB_transp_pattern[0]=0;
        
        for(i=0;i<B_SIZE;i++)
            for(j=B_SIZE;j<2*B_SIZE;j++)
                if (alpha[((y_pos * MB_SIZE) +i)*rec_width + 
                         (x_pos * MB_SIZE) +j] != 0)
                    MB_transp_pattern[1]=0;
        
        for(i=B_SIZE;i<2*B_SIZE;i++)
            for(j=0;j<B_SIZE;j++)
                if (alpha[((y_pos * MB_SIZE) +i)*rec_width + 
                         (x_pos * MB_SIZE) +j] != 0)
                    MB_transp_pattern[2]=0;
        
        for(i=B_SIZE;i<2*B_SIZE;i++)
            for(j=B_SIZE;j<2*B_SIZE;j++)
                if (alpha[((y_pos * MB_SIZE) +i)*rec_width + 
                         (x_pos * MB_SIZE) +j] != 0)
                    MB_transp_pattern[3]=0;
    }
    
}

Void
CallingConvention
GetMBInterShapeErrRes(  Bitstream *stream, Vop *prev_vop,Int vo_id, 
                        Int vol_id, Int y_pos, Int x_pos, Int mbnum,
                        Image *MB_decisions, Image *alpha_decisions,
                        Image *mot_x, Image *mot_y, Image *modeA,
                        Image *motA_x, Image *motA_y, Int start_of_packet, 	
                        Int *amb_type, Int *MB_transp_pattern,
                        Int *error_flag, Vop *rec_vop, Trace *trace,
						UChar **MB_TYPE2[MAX_NUM_VOS][MAX_NUM_VOLS],
						Int PRE_H_SIZE2[MAX_NUM_VOS][MAX_NUM_VOLS],
						Int PRE_V_SIZE2[MAX_NUM_VOS][MAX_NUM_VOLS],
						UChar **PRE_MB_TYPE2[MAX_NUM_VOS][MAX_NUM_VOLS],
						Image   *alpha_rec_packet,Image   *alpha_mode_packet)
{
    Int mvda;
    SInt *alpha;
    Int rec_width;
    SInt i,j;
    SInt *mb_mode = (SInt *) GetImageData(MB_decisions);
    
    /* decode the shape codes */
    
    DecodeFirst(stream, 
                vo_id, 
                vol_id, 
                y_pos, 
                x_pos, 
                amb_type, 
                &mvda, 
                1, 
                start_of_packet,
                rec_vop,
                trace,MB_TYPE2,
				PRE_H_SIZE2,PRE_V_SIZE2,
				PRE_MB_TYPE2);
    
    /* fill the alpha map with the decoded pixels for this MB */
    AlphaDecodeMB(rec_vop, 
                  prev_vop, 
                  vo_id,
                  vol_id,
                  y_pos, 
                  x_pos, 
                  amb_type,
                  &mvda, 
                  1,
                  MB_decisions,
                  alpha_decisions,
                  mot_x,
                  mot_y,
                  modeA,
                  motA_x,
                  motA_y,		
                  start_of_packet,
                  stream,
                  trace,
				  alpha_rec_packet,
				  alpha_mode_packet);
    
    /* calculating MB_transp_pattern */
    for(i=0;i<4;i++)
        MB_transp_pattern[i]=1;
    
    if (*amb_type != 1) /* if not a transparent MB */
    {
        alpha = (SInt *) GetImageData(GetVopA(rec_vop));
        rec_width = GetVopWidth(rec_vop);
        
        for(i=0;i<B_SIZE;i++)
            for(j=0;j<B_SIZE;j++)
                if (alpha[((y_pos * MB_SIZE) +i)*rec_width + 
                         (x_pos * MB_SIZE) +j] != 0)
                    MB_transp_pattern[0]=0;
        
        for(i=0;i<B_SIZE;i++)
            for(j=B_SIZE;j<2*B_SIZE;j++)
                if (alpha[((y_pos * MB_SIZE) +i)*rec_width + 
                         (x_pos * MB_SIZE) +j] != 0)
                    MB_transp_pattern[1]=0;
        
        for(i=B_SIZE;i<2*B_SIZE;i++)
            for(j=0;j<B_SIZE;j++)
                if (alpha[((y_pos * MB_SIZE) +i)*rec_width + 
                         (x_pos * MB_SIZE) +j] != 0)
                    MB_transp_pattern[2]=0;
        
        for(i=B_SIZE;i<2*B_SIZE;i++)
            for(j=B_SIZE;j<2*B_SIZE;j++)
                if (alpha[((y_pos * MB_SIZE) +i)*rec_width + 
                         (x_pos * MB_SIZE) +j] != 0)
                    MB_transp_pattern[3]=0;
    }
    
    if ((MB_transp_pattern[0] + 	
         MB_transp_pattern[1] +
         MB_transp_pattern[2] +
         MB_transp_pattern[3]) == 4)
        mb_mode[mbnum] = MBM_TRANSPARENT;
    
    SetAlphaDecisions(GetVopA(rec_vop),x_pos,y_pos,alpha_decisions);  	  
}


/***********************************************************CommentBegin******
 *
 * -- GetMBheaderErrRes --  
 *
 * Author :
 *       Luis Ducla-Soares (IST) - lds@lx.it.pt
 *
 * Created :
 *      1-May-97
 *
 * Purpose :
 *       decodes the MCBPC and Intra DC coeffs for an Intra MB in an I-VOP
 *       for the combined error resilient mode with data partitioning.
 *
 * Arguments in :
 *      Int vop_type,                   vop INTRA/INTER.
 *	Int intra_dcpred_disable	enable/disable pred. of INTRA DC
 *      Int *quant_step,                 vop quantizer step (used for all MBs).
 *      Bitstream * texture_bits,       Image made of texture parameters.
 *      Int mbnum,                      macroblock number.
 *      Int trace,                      flag for tracing the process.
 *      Int *motion_mode,               already filled by MotionDecode.
 *	Int DC_store[][6][15],		Stores DCAC prediction values
 *	SInt *QP_store,			Stores QPs for every MB
 *	Int QP				Current QPn
 *
 * Arguments in/out :
 *
 *
 * Arguments out :
 *      Macroblock *mblock,             decoded macroblock.
 *      Int *skipped_flag,              1 = macroblock is skipped.
 *      Int *error_flag,                set if an error occured.
 *      Int *coded,                     set if the MB contains coded texture
 *      UInt *CBP,                      Coded Block Pattern derived from MB header data
 *      Int *ACpred_flag                flag indicating the the use of AC prediction
 *      UInt *mode                      macroblock mode
 *
 * Return values :
 *
 *
 * Side effects :
 *
 *
 * Description : based on GetMBheader; 
 *               called by DecodeVopCombinedShapeTextureIntraErrRes.
 *
 *
 * See also :
 *
 *
 * Modified : 
 *          13.05.97 Minhua Zhou: added MB_transp_pattern
 *          15.05.97 Luis Ducla-Soares: cleaned up the code.
 *
 ***********************************************************CommentEnd********/

Void
CallingConvention
GetMBheaderErrRes (Int intra_dcpred_disable,
		   Int *quant_step,
		   Bitstream * texture_bits,
		   Int mbnum,
		   Trace *trace,
		   Image * motion_mode,
		   Int * skipped_flag,
		   Int * error_flag,
		   Int *coded,
		   Int *CBP,
		   Int *mode,
		   Int *DC_coeff,
                   Int *MB_transp_pattern,
		   Vop *rec_vop)
{
    UInt CBPC;
    Int  MBtype;
    SInt *decis_ptr;
    Int  k;
    Int  MCBPC;
    Int  DQUANT=0;
    Int  DQ_tab[4] = {-1, -2, 1, 2};
    Int  switched;
    
    decis_ptr = (SInt *) GetImageData (motion_mode);
    
    switched = IntraDCSwitch_Decision(*mode,
				      GetVopIntraDCVlcThr(rec_vop),
				      *quant_step);
    
    /* coded macroblock */
    *coded=1;
    *skipped_flag = 0;
    
    MCBPC = VlcDecMCBPC_com_intra (texture_bits, error_flag, trace);
    
    if (*error_flag == RETURN)
    {
	printf ("Error decoding MCBPC of macroblock %d\n", (int)mbnum);
	return;
    }
    
    MBtype = MCBPC & 7;
    if ( MBtype == 0 ) *mode = MODE_INTER;
    if ( MBtype == 1 ) *mode = MODE_INTER_Q;
    if ( MBtype == 2 ) *mode = MODE_INTER4V;
    if ( MBtype == 3 ) *mode = MODE_INTRA;
    if ( MBtype == 4 ) *mode = MODE_INTRA_Q;
    if ( MBtype == 5 ) *mode = MODE_DYN_SP;
    if ( MBtype == 6 ) *mode = MODE_DYN_SP_Q;
    
    
    
    CBPC = (MCBPC >> 4) & 3;
    
    *CBP = CBPC;
    
    if (*mode == MODE_INTER_Q || *mode == MODE_INTRA_Q|| *mode == MODE_DYN_SP_Q) {
        DQUANT = BitstreamReadBits(texture_bits, 2, "DQUANT", trace, NUM);
        *quant_step = *quant_step + DQ_tab[DQUANT];
        if (trace->trace)
            fprintf (trace->fp_trace, "DQUANT = %d   QUANT = %d\n",
                     (int)DQ_tab[DQUANT], (int)*quant_step);
        if (*quant_step > 31 || *quant_step < 1)
	{
            if (trace->trace)
                fprintf(trace->fp_trace,"Quantizer out of range %d:clipping\n"
                        , (int)*quant_step);
            *quant_step = mmax (1, mmin (31, (*quant_step)));
	}
    }
    
    
    if (*mode == MODE_INTRA || *mode == MODE_INTRA_Q) 
    {
	for (k = 0; k < 6; k++)
        {
            if ((k>3) || (MB_transp_pattern[k]!=1))
            {
                if (switched==0)
                {
                    if (intra_dcpred_disable)   /* No INTRA DC prediction */
                    {
                        DC_coeff[k] = BitstreamReadBits (texture_bits, 8, "DC coeff",
                                                         trace, NUM);
                        
                        if (DC_coeff[k] == 128) 
                        {
                            printf ("Illegal DC coeff: 1000000\n");
                            *error_flag = RETURN;
                            return;
                        }
                        
                        if (DC_coeff[k] == 255) DC_coeff[k] = 128;
                        
                    }
                    else
                        DC_coeff[k] = DecodePredictedIntraDC(k, texture_bits, 
                                                             trace, NUM);
                }
            }
        }
	
	*(decis_ptr + mbnum) = INTRA;
	
    }
    
    trace->trace = trace->MB_texture;
    if(trace->trace)
        fprintf(trace->fp_trace, "----------\n");
    
}

/***********************************************************CommentBegin******
 *
 * -- GetMBheaderNoDataPartIntraErrRes --  
 *
 * Author :
 *       Luis Ducla-Soares (IST) - lds@lx.it.pt
 *
 * Created :
 *      8-Sep-97
 *
 * Purpose :
 *       decodes the MCBPC and Intra DC coeffs for an Intra MB in an I-VOP
 *       for the combined error resilient mode with data partitioning.
 *
 * Arguments in :
 *      Int vop_type,                   vop INTRA/INTER.
 *	Int intra_dcpred_disable	enable/disable pred. of INTRA DC
 *      Int *quant_step,                 vop quantizer step (used for all MBs).
 *      Bitstream * texture_bits,       Image made of texture parameters.
 *      Int mbnum,                      macroblock number.
 *      Int trace,                      flag for tracing the process.
 *      Int *motion_mode,               already filled by MotionDecode.
 *	Int DC_store[][6][15],		Stores DCAC prediction values
 *	SInt *QP_store,			Stores QPs for every MB
 *	Int QP				Current QPn
 *
 * Arguments in/out :
 *
 *
 * Arguments out :
 *      Macroblock *mblock,             decoded macroblock.
 *      Int *skipped_flag,              1 = macroblock is skipped.
 *      Int *error_flag,                set if an error occured.
 *      Int *coded,                     set if the MB contains coded texture
 *      UInt *CBP,                      Coded Block Pattern derived from MB header data
 *      Int *ACpred_flag                flag indicating the the use of AC prediction
 *      UInt *mode                      macroblock mode
 *
 * Return values :
 *
 *
 * Side effects :
 *
 *
 * Description : based on GetMBheader; 
 *               called by DecodeVopCombinedShapeTextureIntraErrRes.
 *
 *
 * See also :
 *
 *
 * Modified : 
 *          13.05.97 Minhua Zhou: added MB_transp_pattern
 *          15.05.97 Luis Ducla-Soares: cleaned up the code.
 *
 ***********************************************************CommentEnd********/

Void
CallingConvention
GetMBheaderNoDataPartIntraErrRes (Bitstream * texture_bits,
				  Int intra_dcpred_disable,
				  Int mbnum,
				  Trace *trace,
				  Image * motion_mode,
				  Int * skipped_flag,
				  Int * error_flag,
				  Int *coded,
				  Int *CBP,
				  Int *ACpred_flag,
				  Int *mode,
				  Int *MB_transp_pattern)
{
    UInt CBPY;
    UInt CBPC;
    Int  MBtype;
    SInt *decis_ptr;
    Int  MCBPC;
    
    Int flag;

    decis_ptr = (SInt *) GetImageData (motion_mode);
    
    
    /* coded macroblock */
    *coded=1;
    *skipped_flag = 0;
    
    MCBPC = VlcDecMCBPC_com_intra (texture_bits, error_flag, trace);
    
    if (*error_flag == RETURN)
    {
	printf ("Error decoding MCBPC of macroblock %d\n", (int)mbnum);
	return;
    }
    
    MBtype = MCBPC & 7;
    if ( MBtype == 0 ) *mode = MODE_INTER;
    if ( MBtype == 1 ) *mode = MODE_INTER_Q;
    if ( MBtype == 2 ) *mode = MODE_INTER4V;
    if ( MBtype == 3 ) *mode = MODE_INTRA;
    if ( MBtype == 4 ) *mode = MODE_INTRA_Q;
    if ( MBtype == 5 ) *mode = MODE_DYN_SP;
    if ( MBtype == 6 ) *mode = MODE_DYN_SP_Q;
    
    
    
    CBPC = (MCBPC >> 4) & 3;
    
    if((*mode == MODE_INTRA || *mode == MODE_INTRA_Q) && intra_dcpred_disable == 0)
    {
	*ACpred_flag = BitstreamReadBits(texture_bits, 1, "ACpred_flag",
					 trace, FLAG);
    }
    
	flag = (*mode == MODE_INTRA || *mode == MODE_INTRA_Q);
    CBPY = VlcDecCBPY (texture_bits, flag,
		       MB_transp_pattern,error_flag, trace);
    
    if (*error_flag == RETURN)
    {
	printf ("Error decoding CBPY of macroblock %d\n", (int)mbnum);
	return;
    }
    
    if (*mode == MODE_INTRA || *mode == MODE_INTRA_Q)
    {
	*(decis_ptr + mbnum) = INTRA; /* added 30-JAN-1997 MW */
	/* CBPY = CBPY ^ 15; removed by Minhua Zhou 15-05-97 */
    }
    
    
    *CBP = CBPY << 2 | (CBPC);
    
    
    
    trace->trace = trace->MB_texture;
    if(trace->trace)
        fprintf(trace->fp_trace, "----------\n");
    
}

/***********************************************************CommentBegin******
 *
 * -- GetMBheaderNoDataPartInterErrRes --  
 *
 * Author :
 *      Luis Ducla-Soares (IST) - lds@lx.it.pt
 *
 * Created :
 *      8-Sep-97
 *
 * Purpose :
 *       decodes the COD and MCBPC of a MB in a P-VOP for combined error resilient mode
 *       with data partitioning.
 *
 * Arguments in :
 *      Int vop_type,                   vop INTRA/INTER.
 *	Int intra_dcpred_disable	enable/disable pred. of INTRA DC
 *      Int *quant_step,                 vop quantizer step (used for all MBs).
 *      Bitstream * texture_bits,       Image made of texture parameters.
 *      Int mbnum,                      macroblock number.
 *      Int trace,                      flag for tracing the process.
 *      Int *motion_mode,               already filled by MotionDecode.
 *	Int DC_store[][6][15],		Stores DCAC prediction values
 *	SInt *QP_store,			Stores QPs for every MB
 *	Int QP				Current QPn
 *
 * Arguments in/out :
 *
 *
 * Arguments out :
 *      Macroblock *mblock,             decoded macroblock.
 *      Int *skipped_flag,              1 = macroblock is skipped.
 *      Int *error_flag,                set if an error occured.
 *      Int *coded,                     set if the MB contains coded texture
 *      UInt *CBP,                      Coded Block Pattern derived from MB header data
 *      Int *ACpred_flag                flag indicating the the use of AC prediction
 *      UInt *mode                      macroblock mode
 *
 * Return values :
 *
 *
 * Side effects :
 *
 *
 * Description : the function GetMBheader() was used as a starting point;
 *               called by DecodeVopCombinedMotionShapeTextureInterErrRes().
 *
 *
 * See also :
 *
 *
 * Modified : 
 *           13.05.97 Minhua Zhou: added MB_transp_pattern
 *           15.05.97 Luis Ducla-Soares: cleaned up the code.
 *
 ***********************************************************CommentEnd********/

Void
CallingConvention
GetMBheaderNoDataPartInterErrRes (Bitstream * texture_bits,
				  Int intra_dcpred_disable,
				  Int mbnum,
				  Trace *trace,
				  Int * skipped_flag,
				  Int * error_flag,
				  Int ***DC_store,
				  Int *coded,
				  Int *CBP,
				  Int *ACpred_flag,
				  Int *mode,
				  Int *MB_transp_pattern)
{
    UInt CBPY;
    UInt CBPC;
    Int  MBtype;
    Int  MCBPC;
    UInt no_dct_flag;
    Int  m,n;
	Int flag;    
    
    no_dct_flag = BitstreamReadBits (texture_bits, 1,"NO DCT FLAG", 
                                     trace, FLAG);
    
    if (no_dct_flag)
    {
	/* skipped macroblock */
	*skipped_flag = 1;
	*coded=0;
        *mode=MODE_INTER; 
	/* Fill DC_store with default coeff values */
	for (m = 0; m < 6; m++) {
            DC_store[mbnum][m][0] = 128*8;
            for (n = 1; n < 15; n++)
                DC_store[mbnum][m][n] = 0;
	}
    }
    else
    {
	/* coded macroblock */
        *coded=1;
	*skipped_flag = 0;
	
	MCBPC = VlcDecMCBPC_com_inter (texture_bits, error_flag, trace);
	
	if (*error_flag == RETURN)
        {
	    printf ("Error decoding MCBPC of macroblock %d\n", (int)mbnum);
	    return;
        }
	
	MBtype = MCBPC & 7;
	if ( MBtype == 0 ) *mode = MODE_INTER;
	if ( MBtype == 1 ) *mode = MODE_INTER_Q;
	if ( MBtype == 2 ) *mode = MODE_INTER4V;
	if ( MBtype == 3 ) *mode = MODE_INTRA;
	if ( MBtype == 4 ) *mode = MODE_INTRA_Q;
	if ( MBtype == 5 ) *mode = MODE_DYN_SP;
	if ( MBtype == 6 ) *mode = MODE_DYN_SP_Q;
	
        
	CBPC = (MCBPC >> 4) & 3;
        
	if ((*mode == MODE_INTRA || *mode == MODE_INTRA_Q) && intra_dcpred_disable == 0)
        {
	    *ACpred_flag = BitstreamReadBits(texture_bits, 1, "ACpred_flag",
					     trace, FLAG);
        }
        
	flag = (*mode == MODE_INTRA || *mode == MODE_INTRA_Q);
	CBPY = VlcDecCBPY (texture_bits, flag, 
			   MB_transp_pattern,error_flag, trace);
        
	if (*error_flag == RETURN)
        {
	    printf ("Error decoding CBPY of macroblock %d\n", (int)mbnum);
	    return;
        }
	
	
	*CBP = CBPY << 2 | (CBPC);
	
	trace->trace = trace->MB_texture;
	if(trace->trace)
	    fprintf(trace->fp_trace, "----------\n");
    }
}


/***********************************************************CommentBegin******
 *
 * -- GetMBheaderMotionErrRes --  
 *
 * Author :
 *      Luis Ducla-Soares (IST) - lds@lx.it.pt
 *
 * Created :
 *      1-May-97
 *
 * Purpose :
 *       decodes the COD and MCBPC of a MB in a P-VOP for combined error resilient mode
 *       with data partitioning.
 *
 * Arguments in :
 *      Int vop_type,                   vop INTRA/INTER.
 *	Int intra_dcpred_disable	enable/disable pred. of INTRA DC
 *      Int *quant_step,                 vop quantizer step (used for all MBs).
 *      Bitstream * texture_bits,       Image made of texture parameters.
 *      Int mbnum,                      macroblock number.
 *      Int trace,                      flag for tracing the process.
 *      Int *motion_mode,               already filled by MotionDecode.
 *	Int DC_store[][6][15],		Stores DCAC prediction values
 *	SInt *QP_store,			Stores QPs for every MB
 *	Int QP				Current QPn
 *
 * Arguments in/out :
 *
 *
 * Arguments out :
 *      Macroblock *mblock,             decoded macroblock.
 *      Int *skipped_flag,              1 = macroblock is skipped.
 *      Int *error_flag,                set if an error occured.
 *      Int *coded,                     set if the MB contains coded texture
 *      UInt *CBP,                      Coded Block Pattern derived from MB header data
 *      Int *ACpred_flag                flag indicating the the use of AC prediction
 *      UInt *mode                      macroblock mode
 *
 * Return values :
 *
 *
 * Side effects :
 *
 *
 * Description : the function GetMBheader() was used as a starting point;
 *               called by DecodeVopCombinedMotionShapeTextureInterErrRes().
 *
 *
 * See also :
 *
 *
 * Modified : 
 *           13.05.97 Minhua Zhou: added MB_transp_pattern
 *           15.05.97 Luis Ducla-Soares: cleaned up the code.
 *
 ***********************************************************CommentEnd********/

Void
CallingConvention
GetMBheaderMotionErrRes (Bitstream * texture_bits,
			 Int mbnum,
			 Trace *trace,
			 Int * skipped_flag,
			 Int * error_flag,
			 Int ***DC_store,
			 Int *coded,
			 Int *CBP,
			 Int *mode,
			 Int *transp_pattern)
{
    UInt CBPC;
    Int  MBtype;
    Int  MCBPC;
    UInt no_dct_flag=1;
    Int  m,n;
    
    no_dct_flag = BitstreamReadBits (texture_bits, 1,"NO DCT FLAG", 
                                     trace, FLAG);
    
    if (no_dct_flag)
    {
	/* skipped macroblock */
	*skipped_flag = 1;
	*mode=INTER; 
	/* Fill DC_store with default coeff values */
	for (m = 0; m < 6; m++) {
            DC_store[mbnum][m][0] = 128*8;
            for (n = 1; n < 15; n++)
                DC_store[mbnum][m][n] = 0;
	}
    }
    else
    {
	/* coded macroblock */
        *coded=1;
	*skipped_flag = 0;
	
	MCBPC = VlcDecMCBPC_com_inter (texture_bits, error_flag, trace);
	
	if (*error_flag == RETURN)
        {
	    printf ("Error decoding MCBPC of macroblock %d\n", (int)mbnum);
	    return;
        }
	
	MBtype = MCBPC & 7;
	if ( MBtype == 0 ) *mode = MODE_INTER;
	if ( MBtype == 1 ) *mode = MODE_INTER_Q;
	if ( MBtype == 2 ) *mode = MODE_INTER4V;
	if ( MBtype == 3 ) *mode = MODE_INTRA;
	if ( MBtype == 4 ) *mode = MODE_INTRA_Q;
	if ( MBtype == 5 ) *mode = MODE_DYN_SP;
	if ( MBtype == 6 ) *mode = MODE_DYN_SP_Q;
	
        
	CBPC = (MCBPC >> 4) & 3;
        
	*CBP =  CBPC;
        
        
	trace->trace = trace->MB_texture;
	if(trace->trace)
	    fprintf(trace->fp_trace, "----------\n");
    }
}

/***********************************************************CommentBegin******
 *
 * -- GetMBheaderTextErrRes --  
 *
 * Author :
 *      Luis Ducla-Soares (IST) - lds@lx.it.pt
 *
 * Created :
 *      28-April-97
 *
 * Purpose :
 *       decodes the CBPY and DQUANT of a MB in a P-VOP for combined error resilient
 *       mode with data partitioning.
 *
 * Arguments in :
 *      Int vop_type,                   vop INTRA/INTER.
 *	Int intra_dcpred_disable	enable/disable pred. of INTRA DC
 *      Int *quant_step,                 vop quantizer step (used for all MBs).
 *      Bitstream * texture_bits,       Image made of texture parameters.
 *      Int mbnum,                      macroblock number.
 *      Int trace,                      flag for tracing the process.
 *      Int *motion_mode,               already filled by MotionDecode.
 *	Int DC_store[][6][15],		Stores DCAC prediction values
 *	SInt *QP_store,			Stores QPs for every MB
 *	Int QP				Current QPn
 *
 * Arguments in/out :
 *
 *
 * Arguments out :
 *      Macroblock *mblock,             decoded macroblock.
 *      Int *skipped_flag,              1 = macroblock is skipped.
 *      Int *error_flag,                set if an error occured.
 *      Int *coded,                     set if the MB contains coded texture
 *      UInt *CBP,                      Coded Block Pattern derived from MB header data
 *      Int *ACpred_flag                flag indicating the the use of AC prediction
 *      UInt *mode                      macroblock mode
 *
 * Return values :
 *
 *
 * Side effects :
 *
 *
 * Description : based on GetMBheader;
 *               called by DecodeVopCombinedMotionShapeTextureInterErrRes()
 *               and DecodeVopCombinedShapeTextureIntraErrRes()
 *
 *
 * See also :
 *
 *
 * Modified : 
 *           13.05.97 Minhua Zhou: added MB_transp_pattern
 *           15.05.97 Luis Ducla-Soares: cleaned up the code.
 *
 ***********************************************************CommentEnd********/

Void
CallingConvention
GetMBheaderTextErrRes (Int intra_dcpred_disable,
		       /*Int *quant_step,*/
		       Bitstream * texture_bits,
		       Int mbnum,
		       Trace *trace,
		       Image * motion_mode,
		       Int * error_flag,
		       Int *coded,
		       Int *CBP,
		       Int *ACpred_flag,
		       Int *mode,
                       Int *MB_transp_pattern)
{
    UInt CBPY;
    SInt *decis_ptr;
    /* Int  DQUANT=0;
       Int  DQ_tab[4] = {-1, -2, 1, 2};*/
	Int flag;    
    
    decis_ptr = (SInt *) GetImageData (motion_mode);
    
    
    if ((*mode == MODE_INTRA || *mode == MODE_INTRA_Q) && intra_dcpred_disable == 0)
    {
	*ACpred_flag = BitstreamReadBits(texture_bits, 1, "ACpred_flag",
					 trace, FLAG);
    }
    
	flag = (*mode == MODE_INTRA || *mode == MODE_INTRA_Q);
    CBPY = VlcDecCBPY (texture_bits, flag,MB_transp_pattern,error_flag, trace);
    if (*error_flag == RETURN)
    {
	printf ("Error decoding CBPY of macroblock %d\n", (int)mbnum);
	return;
    }
    
    if (*mode == MODE_INTRA || *mode == MODE_INTRA_Q)
    {
	*(decis_ptr + mbnum) = INTRA; /* added 30-JAN-1997 MW */
    }
    
    
    *CBP = CBPY << 2 | (*CBP);
    
#if 0
    if (*mode == MODE_INTER_Q || *mode == MODE_INTRA_Q|| *mode == MODE_DYN_SP_Q) {
        DQUANT = BitstreamReadBits(texture_bits, 2, "DQUANT", trace, NUM);
        *quant_step = *quant_step + DQ_tab[DQUANT];
        if (trace->trace)
            fprintf (trace->fp_trace, "DQUANT = %d   QUANT = %d\n",
                     (int)DQ_tab[DQUANT], (int)*quant_step);
        if (*quant_step > 31 || *quant_step < 1)
	{
            if (trace->trace)
                fprintf(trace->fp_trace,"Quantizer out of range %d:clipping\n"
                        , (int)*quant_step);
            *quant_step = mmax (1, mmin (31, (*quant_step)));
	}
    }
#endif
    
    trace->trace = trace->MB_texture;
    if(trace->trace)
        fprintf(trace->fp_trace, "----------\n");
    
}

/***********************************************************CommentBegin******
 *
 * -- GetMBheaderTextInterErrRes --  
 *
 * Author :
 *      Luis Ducla-Soares (IST) - lds@lx.it.pt
 *
 * Created :
 *      09-Oct-97
 *
 * Purpose :
 *       decodes the CBPY and DQUANT of a MB in a P-VOP for combined error resilient
 *       mode with data partitioning.
 *
 * Arguments in :
 *      Int vop_type,                   vop INTRA/INTER.
 *	Int intra_dcpred_disable	enable/disable pred. of INTRA DC
 *      Int *quant_step,                 vop quantizer step (used for all MBs).
 *      Bitstream * texture_bits,       Image made of texture parameters.
 *      Int mbnum,                      macroblock number.
 *      Int trace,                      flag for tracing the process.
 *      Int *motion_mode,               already filled by MotionDecode.
 *	Int DC_store[][6][15],		Stores DCAC prediction values
 *	SInt *QP_store,			Stores QPs for every MB
 *	Int QP				Current QPn
 *
 * Arguments in/out :
 *
 *
 * Arguments out :
 *      Macroblock *mblock,             decoded macroblock.
 *      Int *skipped_flag,              1 = macroblock is skipped.
 *      Int *error_flag,                set if an error occured.
 *      Int *coded,                     set if the MB contains coded texture
 *      UInt *CBP,                      Coded Block Pattern derived from MB header data
 *      Int *ACpred_flag                flag indicating the the use of AC prediction
 *      UInt *mode                      macroblock mode
 *
 * Return values :
 *
 *
 * Side effects :
 *
 *
 * Description : based on GetMBheader;
 *               called by DecodeVopCombinedMotionShapeTextureInterErrRes()
 *               and DecodeVopCombinedShapeTextureIntraErrRes()
 *
 *
 * See also :
 *
 *
 * Modified : 
 *           13.05.97 Minhua Zhou: added MB_transp_pattern
 *           15.05.97 Luis Ducla-Soares: cleaned up the code.
 *					 19.11.97 Noel Brady: added support of shape
 *
 ***********************************************************CommentEnd********/

Void
CallingConvention
GetMBheaderTextInterErrRes (Int intra_dcpred_disable,
			    Int *quant_step,
			    Bitstream * texture_bits,
			    Int mbnum,
			    Trace *trace,
			    Image * motion_mode,
			    Int * error_flag,
			    Int *coded,
			    Int *CBP,
			    Int *ACpred_flag,
			    Int *mode,
			    Int *MB_transp_pattern,
			    Int *DC_coeff,
			    Vop *rec_vop)
{
    UInt CBPY;
    SInt *decis_ptr;
    Int  DQUANT=0;
    Int  DQ_tab[4] = {-1, -2, 1, 2};
    Int  k;
    Int  switched;
	Int flag;
    
    decis_ptr = (SInt *) GetImageData (motion_mode);
    
    switched = IntraDCSwitch_Decision(*mode,
				      GetVopIntraDCVlcThr(rec_vop),
				      *quant_step);
    
    
    if ((*mode == MODE_INTRA || *mode == MODE_INTRA_Q) && intra_dcpred_disable == 0)
    {
	*ACpred_flag = BitstreamReadBits(texture_bits, 1, "ACpred_flag",
					 trace, FLAG);
    }
    
    
	flag = (*mode == MODE_INTRA || *mode == MODE_INTRA_Q);
    CBPY = VlcDecCBPY (texture_bits, flag,MB_transp_pattern,error_flag, trace);
    if (*error_flag == RETURN)
    {
	printf ("Error decoding CBPY of macroblock %d\n", (int)mbnum);
	return;
    }
    
    if (*mode == MODE_INTRA || *mode == MODE_INTRA_Q)
    {
	*(decis_ptr + mbnum) = INTRA; /* added 30-JAN-1997 MW */
    }
    
    
    *CBP = CBPY << 2 | (*CBP);
    if (*mode == MODE_INTER_Q || *mode == MODE_INTRA_Q|| *mode == MODE_DYN_SP_Q) {
        DQUANT = BitstreamReadBits(texture_bits, 2, "DQUANT", trace, NUM);
        *quant_step = *quant_step + DQ_tab[DQUANT];
        if (trace->trace)
            fprintf (trace->fp_trace, "DQUANT = %d   QUANT = %d\n",
                     (int)DQ_tab[DQUANT], (int)*quant_step);
        if (*quant_step > 31 || *quant_step < 1)
	{
            if (trace->trace)
                fprintf(trace->fp_trace,"Quantizer out of range %d:clipping\n"
                        , (int)*quant_step);
            *quant_step = mmax (1, mmin (31, (*quant_step)));
	}
    }
    
    if (*mode == MODE_INTRA || *mode == MODE_INTRA_Q) 
    {
	for (k = 0; k < 6; k++)
        {
            if (k>3 || MB_transp_pattern[k]!=1)
            {
                if (switched==0)
                {
                    if (intra_dcpred_disable)  /* No INTRA DC prediction */
                    {
                        DC_coeff[k] = BitstreamReadBits (texture_bits, 8, "DC coeff",
                                                         trace, NUM);
                        
                        if (DC_coeff[k] == 128) {
                            printf ("Illegal DC coeff: 1000000\n");
                            *error_flag = RETURN;
                            return;
                        }
                        if (DC_coeff[k] == 255)
                            DC_coeff[k] = 128;
                    }
                    else
                        DC_coeff[k] = DecodePredictedIntraDC(k, texture_bits, trace, NUM);
                }
            }
        }
        
	*(decis_ptr + mbnum) = INTRA;
	
    }
    
    
    trace->trace = trace->MB_texture;
    if(trace->trace)
        fprintf(trace->fp_trace, "----------\n");
    
}


/***********************************************************CommentBegin******
 *
 * -- GetMBvectorsErrRes --  
 *
 * Author :
 *      Luis Ducla-Soares (IST) - lds@lx.it.pt
 *
 * Created :
 *      01.05.97
 *
 * Purpose :
 *       decodes the vectors of a MB for the combined error resilient mode
 *       with data partitioning.
 *
 * Arguments in :
 *      Int vop_type,                   vop INTRA/INTER.
 *	Int intra_dcpred_disable	enable/disable pred. of INTRA DC
 *      Int *quant_step,                 vop quantizer step (used for all MBs).
 *      Bitstream * texture_bits,       Image made of texture parameters.
 *      Int mbnum,                      macroblock number.
 *      Int trace,                      flag for tracing the process.
 *      Int *motion_mode,               already filled by MotionDecode.
 *	Int DC_store[][6][15],		Stores DCAC prediction values
 *	SInt *QP_store,			Stores QPs for every MB
 *	Int QP				Current QPn
 *
 * Arguments in/out :
 *
 *
 * Arguments out :
 *      Int *skipped_flag,              1 = macroblock is skipped.
 *      Int *error_flag,                set if an error occured.
 *      Int *coded,                     set if the MB contains coded texture
 *      Int *ACpred_flag                flag indicating the the use of AC prediction
 *      UInt *mode                      macroblock mode
 *
 * Return values :
 *
 *
 * Side effects :
 *
 *
 * Description : based on GetMBvectors();
 *               called by DecodeVopCombinedMotionShapeTextureInterErrRes().
 *     
 *
 * See also :
 *
 *
 * Modified : 
 *      15.05.97 Luis Ducla-Soares: cleaned up the code.
 *
 ***********************************************************CommentEnd********/

Void
CallingConvention
GetMBvectorsErrRes (Int intra_dcpred_disable,
		    Bitstream * bits,
		    Int mbnum,
		    Int MB_width,
		    Int f_code,
		    Trace *trace,
		    Image * motion_mode,
		    Image * block_decis,
		    Image * mot_x,
		    Image * mot_y,
		    Int * error_flag,
		    UInt mode,
		    Int error_res_disable,
		    Int after_marker,
		    Int **slice_nb)
{
    Int k;
    SInt *decis_ptr, *bdecis_ptr;
    Int  x_pos = mbnum % MB_width;
    Int  y_pos = mbnum / MB_width;
    Float               *motx_ptr, *moty_ptr, mvx=0, mvy=0;
    Int                 B_in_width = 2 * MB_width;
    Int   tmp_pred;
    
    decis_ptr = (SInt *) GetImageData (motion_mode);
    bdecis_ptr = (SInt *) GetImageData (block_decis);
    motx_ptr = (Float *) GetImageData (mot_x);
    moty_ptr = (Float *) GetImageData (mot_y);
    
    
    /* Decoding of motion vectors */
    
    if (mode == MODE_INTER || mode == MODE_INTER_Q || mode == MODE_INTER4V)
    {
        
        if (mode == MODE_INTER4V)
        {
            for (k = 0; k < 4; k++)
            {
                if (!B_TRANSP (bdecis_ptr,B_in_width,x_pos,y_pos,k & 0x1,k >> 1,MBM_TRANSPARENT))
                {
                    *(decis_ptr+mbnum) = FOURVEC;
                    
                    if (after_marker && ((k == 0) || (k == 2)))
                        tmp_pred = 1;
                    else
                        tmp_pred = 0;
                    
                    
                    /* MVP/Noel */
                    DecodeMBVec (mot_x, mot_y, motion_mode, block_decis, 
                                 bits, trace, x_pos, y_pos, 
                                 k + 1, f_code, error_res_disable, tmp_pred,
                                 error_flag, &mvx, &mvy, slice_nb); 
                    
                    if (*error_flag == RETURN)
                        return;
                    
                    BV (motx_ptr, B_in_width, x_pos, y_pos, k & 0x1, k >> 1) = mvx;
                    BV (moty_ptr, B_in_width, x_pos, y_pos, k & 0x1, k >> 1) = mvy;
                }
                else
                {
                    BV (motx_ptr, B_in_width, x_pos, y_pos, k & 0x1, k >> 1) = 0.0;
                    BV (moty_ptr, B_in_width, x_pos, y_pos, k & 0x1, k >> 1) = 0.0;
                }
                if (trace->trace)
                {
                    fprintf (trace->fp_trace, "  >> mvx: %g\n", mvx);
                    fprintf (trace->fp_trace, "  >> mvy: %g\n", mvy);
                }
            }
        }
        else
        {
            
            /* MVP/Noel */
            DecodeMBVec (mot_x, mot_y, motion_mode, block_decis, 
                         bits, trace, x_pos, y_pos, 
                         0, f_code, error_res_disable, after_marker,
                         error_flag, &mvx, &mvy, slice_nb); 
            
            if (*error_flag == RETURN)
                return;
            
            if (trace->trace)
            {
                fprintf (trace->fp_trace, "  >> mvx: %g\n", mvx);
                fprintf (trace->fp_trace, "  >> mvy: %g\n", mvy);
            }
            for (k = 0; k < 4; k++)
            {
                BV (motx_ptr, B_in_width, x_pos, y_pos, k & 0x1, k >> 1) = mvx;
                BV (moty_ptr, B_in_width, x_pos, y_pos, k & 0x1, k >> 1) = mvy;
                *(decis_ptr + mbnum) = ONEVEC;
            }
        }
    }
#if 0
    else   /* INTRA Macroblock or Sprite Macroblock*/
    {
        if (mode == MODE_INTRA) 
     	{
            for (k = 0; k < 6; k++)
	    {
                
                DC_coeff[k] = BitstreamReadBits (bits, 8, "DC coeff",
                                                 trace, NUM);
                
                if (DC_coeff[k] == 128) {
                    printf ("Illegal DC coeff: 1000000\n");
                    exit(1);
                }
                if (DC_coeff[k] == 255)
                    DC_coeff[k] = 128;
                
	    	
	    }
            
            for (k = 0; k < 4; k++)
            {
                BV (motx_ptr, B_in_width, x_pos, y_pos, k & 0x1, k >> 1) = 0.0;
                BV (moty_ptr, B_in_width, x_pos, y_pos, k & 0x1, k >> 1) = 0.0;
                *(decis_ptr + mbnum) = INTRA;
            }
	}
        else
            if (mode == MODE_DYN_SP)
            {
                for (k = 0; k < 4; k++)
                {
                    BV (motx_ptr, B_in_width, x_pos, y_pos, k & 0x1, k >> 1) = 0.0;
                    BV (moty_ptr, B_in_width, x_pos, y_pos, k & 0x1, k >> 1) = 0.0;
                    *(decis_ptr + mbnum) = MBM_SPRITE;
                }
            }
        
    }
#endif  
    
    
    trace->trace = trace->MB_texture;
    if(trace->trace)
        fprintf(trace->fp_trace, "----------\n");
}

/***********************************************************CommentBegin******
 *
 * -- GetMBvectorsNoDataPartErrRes --  
 *
 * Author :
 *      Luis Ducla-Soares (IST) - lds@lx.it.pt
 *
 * Created :
 *      10.09.97
 *
 * Purpose :
 *       decodes the vectors of a MB for the combined error resilient mode
 *       with data partitioning.
 *
 * Arguments in :
 *      Int vop_type,                   vop INTRA/INTER.
 *	Int intra_dcpred_disable	enable/disable pred. of INTRA DC
 *      Int *quant_step,                 vop quantizer step (used for all MBs).
 *      Bitstream * texture_bits,       Image made of texture parameters.
 *      Int mbnum,                      macroblock number.
 *      Int trace,                      flag for tracing the process.
 *      Int *motion_mode,               already filled by MotionDecode.
 *	Int DC_store[][6][15],		Stores DCAC prediction values
 *	SInt *QP_store,			Stores QPs for every MB
 *	Int QP				Current QPn
 *
 * Arguments in/out :
 *
 *
 * Arguments out :
 *      Int *skipped_flag,              1 = macroblock is skipped.
 *      Int *error_flag,                set if an error occured.
 *      Int *coded,                     set if the MB contains coded texture
 *      Int *ACpred_flag                flag indicating the the use of AC prediction
 *      UInt *mode                      macroblock mode
 *
 * Return values :
 *
 *
 * Side effects :
 *
 *
 * Description : based on GetMBvectors();
 *               called by DecodeVopCombinedMotionShapeTextureInterErrRes().
 *     
 *
 * See also :
 *
 *
 * Modified : 
 *      15.05.97 Luis Ducla-Soares: cleaned up the code.
 *
 ***********************************************************CommentEnd********/

Void
CallingConvention
GetMBvectorsNoDataPartErrRes (Bitstream * bits,
			      Int mbnum,
			      Int MB_width,
			      Int f_code,
			      Trace *trace,
			      Image * motion_mode,
			      Image * block_decis,
			      Image * mot_x,
			      Image * mot_y,
			      Int * error_flag,
			      UInt mode,
			      Int *DC_coeff,
			      Int error_res_disable,
			      Int after_marker,
			      Int **slice_nb)
{
    Int k;
    SInt *decis_ptr, *bdecis_ptr;
    Int  x_pos = mbnum % MB_width;
    Int  y_pos = mbnum / MB_width;
    Float               *motx_ptr, *moty_ptr, mvx=0, mvy=0;
    Int                 B_in_width = 2 * MB_width;
    Int   tmp_pred;
    
    decis_ptr = (SInt *) GetImageData (motion_mode);
    bdecis_ptr = (SInt *) GetImageData (block_decis);
    motx_ptr = (Float *) GetImageData (mot_x);
    moty_ptr = (Float *) GetImageData (mot_y);
    
    
    /* Decoding of motion vectors */
    
    if (mode == MODE_INTER || mode == MODE_INTER_Q || mode == MODE_INTER4V)
    {
        
        if (mode == MODE_INTER4V)
        {
            for (k = 0; k < 4; k++)
            {
                if (!B_TRANSP (bdecis_ptr,B_in_width,x_pos,y_pos,k & 0x1,k >> 1,MBM_TRANSPARENT))
                {
                    *(decis_ptr+mbnum) = FOURVEC;
                    
                    if (after_marker && ((k == 0) || (k == 2))) /* Watch out for this !!! */
                        tmp_pred = 1;
                    else
                        tmp_pred = 0;
                    
                    
                    DecodeMBVec (mot_x, mot_y, motion_mode, block_decis, bits, trace, x_pos, y_pos, 
                                 k + 1, f_code, error_res_disable, tmp_pred,
                                 error_flag, &mvx, &mvy, slice_nb); 
                    
                    if (*error_flag == RETURN)
                        return;
                    
                    BV (motx_ptr, B_in_width, x_pos, y_pos, k & 0x1, k >> 1) = mvx;
                    BV (moty_ptr, B_in_width, x_pos, y_pos, k & 0x1, k >> 1) = mvy;
                }
                else
                {
                    BV (motx_ptr, B_in_width, x_pos, y_pos, k & 0x1, k >> 1) = 0.0;
                    BV (moty_ptr, B_in_width, x_pos, y_pos, k & 0x1, k >> 1) = 0.0;
                }
                if (trace->trace)
                {
                    fprintf (trace->fp_trace, "  >> mvx: %g\n", mvx);
                    fprintf (trace->fp_trace, "  >> mvy: %g\n", mvy);
                }
            }
        }
        else
        {
            DecodeMBVec (mot_x, mot_y, motion_mode, block_decis, bits, trace, x_pos, y_pos, 
                         0, f_code, error_res_disable, after_marker,
                         error_flag, &mvx, &mvy, slice_nb); 
            
            if (*error_flag == RETURN)
                return;
            
            if (trace->trace)
            {
                fprintf (trace->fp_trace, "  >> mvx: %g\n", mvx);
                fprintf (trace->fp_trace, "  >> mvy: %g\n", mvy);
            }
            for (k = 0; k < 4; k++)
            {
                BV (motx_ptr, B_in_width, x_pos, y_pos, k & 0x1, k >> 1) = mvx;
                BV (moty_ptr, B_in_width, x_pos, y_pos, k & 0x1, k >> 1) = mvy;
                *(decis_ptr + mbnum) = ONEVEC;
            }
        }
    }
    else   /* INTRA Macroblock or Sprite Macroblock*/
    {
        if (mode == MODE_INTRA) 
     	{
#if 0
            for (k = 0; k < 6; k++)
	    {
                
                DC_coeff[k] = BitstreamReadBits (bits, 8, "DC coeff",
                                                 trace, NUM);
                
                if (DC_coeff[k] == 128) {
                    printf ("Illegal DC coeff: 1000000\n");
                    exit(1);
                }
                if (DC_coeff[k] == 255)
                    DC_coeff[k] = 128;
                
	    	
	    }
#endif	
            
            for (k = 0; k < 4; k++)
            {
                BV (motx_ptr, B_in_width, x_pos, y_pos, k & 0x1, k >> 1) = 0.0;
                BV (moty_ptr, B_in_width, x_pos, y_pos, k & 0x1, k >> 1) = 0.0;
                *(decis_ptr + mbnum) = INTRA;
            }
	}
        else
            if (mode == MODE_DYN_SP)
            {
                for (k = 0; k < 4; k++)
                {
                    BV (motx_ptr, B_in_width, x_pos, y_pos, k & 0x1, k >> 1) = 0.0;
                    BV (moty_ptr, B_in_width, x_pos, y_pos, k & 0x1, k >> 1) = 0.0;
                    *(decis_ptr + mbnum) = MBM_SPRITE;
                }
            }
        
    }
    
    
    trace->trace = trace->MB_texture;
    if(trace->trace)
        fprintf(trace->fp_trace, "----------\n");
}


/***********************************************************CommentBegin******
 *
 * -- GetMBblockdataErrRes --  
 *
 * Author :
 *      Luis Ducla-Soares (IST) - lds@lx.it.pt
 *
 * Created :
 *      01.05.97
 *
 * Purpose :
 *       decodes the block data (DCT coefficients, alias "texture") of a MB
 *       for the combined error resilient mode with data partitioning.
 *
 * Arguments in :
 *	Int intra_dcpred_disable	enable/disable pred. of INTRA DC
 *      Int quant_step,                 vop quantizer step (used for all MBs).
 *      Bitstream * texture_bits,       Image made of texture parameters.
 *      Int mbnum,                      macroblock number.
 *      Int trace,                      flag for tracing the process.
 *      Int *motion_mode,               already filled by MotionDecode.
 *	Int MB_width,
 *	Int DC_store[][6][15],		Stores DCAC prediction values
 *	SInt *QP_store,			Stores QPs for every MB
 *	Int QP				Current QPn
 *	UInt CBP			Coded Block Pattern derived from MB header data
 *      UChar disable_sadct             Flag indicating whether SADCT is disabled or not
 *      Short *alpha                    The object plane with alpha values
 *      UChar shape                     Indicator of shape type of the video object being decoded
 *
 * Arguments in/out :
 *
 *
 * Arguments out :
 *      Macroblock *mblock,             decoded macroblock.
 *      Int *skipped_flag,              1 = macroblock is skipped.
 *      Int *error_flag,                set if an error occured.
 *
 *
 * Return values :
 *
 *
 * Side effects :
 *
 *
 * Description :
 *        This function was based on GetMBblockdata().
 *
 * See also :
 *
 *
 * Modified :
 *       14-05-97 Minhua Zhou: disable SADCT for rectangular VOP
 *       15.05.97 Luis Ducla-Soares: cleaned up the code.
 *				19.11.97 Noel Brady: Added traces for shape
 *
 ***********************************************************CommentEnd********/

Void
CallingConvention
GetMBblockdataErrRes(Int *transp_pattern,
                     Int intra_dcpred_disable,
                     Int quant_step,
                     Bitstream * texture_bits,
                     Int mbnum,
                     Trace *trace,
                     Macroblock * mblock,
                     Int * error_flag,
                     Int MB_width,
                     Int ***DC_store,
                     SInt *QP_store,
                     Int QP,
                     UInt CBP,
                     Int ACpred_flag,
                     UInt mode,
                     Int *DC_coeff,
                     Vop *rec_vop,
                     Int **slice_nb)
{       
    Int  comp;
    Int  DCT_block[B_SIZE * B_SIZE];
    Int  q_block[B_SIZE * B_SIZE];
    Int  r_block[B_SIZE * B_SIZE];
    Int  r_block_2D[B_SIZE][B_SIZE];
    Int  pcoeff[B_SIZE * B_SIZE];
    Int  m, n;
    Int  i,j;
    Int  x_pos = mbnum % MB_width;
    Int  y_pos = mbnum / MB_width;
    Int switched;

    Int last_non_null_coeff;
    
    
    UChar disable_sadct = 1;  /* Noel/E14 */
    UChar shape         = GetVopShape(rec_vop);
    
    SInt *alpha = (SInt *) GetImageData(GetVopA(rec_vop));
    
    /* variables because of contour-adaptive DCT */
    Short alpha_block[B_SIZE][B_SIZE];
    Short nrpix = MB_width * MB_SIZE;
    Short tmp; 
    
    Char level_3_trace;
    SInt mb_shape[6][64],mb_dct_bf_iq[6][64],mb_dct_iq[6][64],mb_idct[6][64]; 
    
    MBzero (mblock);
    level_3_trace = trace->MB_sdata||trace->MB_tdata_before_IQ|| 
        trace->MB_tdata_after_IQ||trace->MB_tdata_after_IDCT||
        trace->MB_tdata_reconstructed;
    if (level_3_trace)
        for (i=0;i<6;i++)
            for (j=0;j<64;j++) mb_shape[i][j]=0;
    /*added by Minhua Zhou 04.08.97 */
    switched = IntraDCSwitch_Decision(mode,
                                      GetVopIntraDCVlcThr(rec_vop),
                                      quant_step);
    
    
    
    /*****
     *
     *	  Decoding of the 4 luminance blocks (depending on transparent pattern)
     *
     *****/
    for (comp=0; comp<4; comp++)
    {
        Bzero (q_block);
        
        if(transp_pattern[comp]==0) /* No decoding of transparent Y blocks */
      	{
            
            if ((disable_sadct==0)||(level_3_trace)) {		
                switch(comp) {
                    case 0 :
                        for (i=0; i<B_SIZE; i++)
                            for (j=0; j<B_SIZE; j++)
                                alpha_block[i][j] = alpha[(y_pos*MB_SIZE + i)*nrpix + (x_pos*MB_SIZE + j)];
                        break;
                    case 1 :
                        for (i=0; i<B_SIZE; i++)
                            for (j=0; j<B_SIZE; j++)
                                alpha_block[i][j] = alpha[(y_pos*MB_SIZE + i)*nrpix + (x_pos*MB_SIZE + 8 + j)];
                        break;
                    case 2 :
                        for (i=0; i<B_SIZE; i++)
                            for (j=0; j<B_SIZE; j++)
                                alpha_block[i][j] = alpha[(y_pos*MB_SIZE + 8 + i)*nrpix + (x_pos*MB_SIZE + j)];
                        break;
                    case 3 :
                        for (i=0; i<B_SIZE; i++)
                            for (j=0; j<B_SIZE; j++)
                                alpha_block[i][j] = alpha[(y_pos*MB_SIZE + 8 + i)*nrpix + (x_pos*MB_SIZE + 8 + j)];
                        break;
                    case 4 :
                    case 5 :
                        for (i=0; i<B_SIZE; i++)
                            for (j=0; j<B_SIZE; j++) {
                                if (shape == GREY_SCALE) {
                                    tmp = alpha[(y_pos*MB_SIZE + 2*i)*nrpix + (x_pos*MB_SIZE + 2*j)];
                                    tmp += alpha[(y_pos*MB_SIZE + 2*i)*nrpix + (x_pos*MB_SIZE + 2*j+1)];
                                    tmp += alpha[(y_pos*MB_SIZE + 2*i+1)*nrpix + (x_pos*MB_SIZE + 2*j)];
                                    tmp += alpha[(y_pos*MB_SIZE + 2*i+1)*nrpix + (x_pos*MB_SIZE + 2*j+1)];
                                    tmp += 2;
                                    tmp /= 4;
                                    alpha_block[i][j] = (tmp > BINARY_ALPHA/2) ? BINARY_ALPHA : 0;
                                }
                                else { /* so-called "conservative" subsampling (JDL - 20-MAR-97) */
                                    alpha_block[i][j] = 
                                        ((alpha[(y_pos*MB_SIZE + 2*i)*nrpix + (x_pos*MB_SIZE + 2*j)] == BINARY_ALPHA) ||
                                         (alpha[(y_pos*MB_SIZE + 2*i)*nrpix + (x_pos*MB_SIZE + 2*j+1)] == BINARY_ALPHA) ||
                                         (alpha[(y_pos*MB_SIZE + 2*i+1)*nrpix + (x_pos*MB_SIZE + 2*j)] == BINARY_ALPHA) ||
                                         (alpha[(y_pos*MB_SIZE + 2*i+1)*nrpix + (x_pos*MB_SIZE + 2*j+1)] == BINARY_ALPHA))
                                        ? BINARY_ALPHA : 0 ; 
                                }
                            }
                        break;
                    default :
                        for (i=0; i<B_SIZE; i++)
                            for (j=0; j<B_SIZE; j++)
                                alpha_block[i][j] = BINARY_ALPHA;
                }
                if (trace->trace)
                    for (i=0;i<8;i++)
                        for (j=0;j<8;j++)
                            mb_shape[comp][i*8+j]=alpha_block[i][j]; 
            }
            
            if (mode == MODE_INTRA || mode == MODE_INTRA_Q) {
                
                q_block[0] = DC_coeff[comp];
                
                if (CBP & (1 << (BLOCK_CNT - 1 - comp))) {
                    VlcGetBlock(texture_bits, q_block, INTRA,
                                trace, error_flag, 1, GetVopReverseVlc(rec_vop),switched,&last_non_null_coeff);
                }
                
                if (*error_flag == RETURN)
                    return;
                
                
                if (intra_dcpred_disable == 0)
                    doDCACreconErrRes(q_block, 64, DC_store, QP_store, QP,
                                      x_pos, y_pos, MB_width, ACpred_flag, comp, slice_nb);
                else
                {
                    /* Ordinary inverse zigzag-scanning */
                    for (m = 0; m < 64; m++)
                        pcoeff[m] = q_block[zigzag[m]];
                    for (m = 0; m < 8; m++)
                        for (n = 0; n < 8; n++)
                            q_block[m*8+n] = pcoeff[m*8+n];
                }
                
                if (trace->MB_tdata_before_IQ)
                    for (i=0;i<64;i++)
                        mb_dct_bf_iq[comp][i]=q_block[i]; 
                
                /* Store the qcoeff-values needed later for prediction */
                for (m = 0; m < 8; m++) {
                    DC_store[mbnum][comp][m] = q_block[m];
                }
                for (m = 0; m < 7; m++) {
                    DC_store[mbnum][comp][m+8] = q_block[(m+1)*8];
                }
                
                
                /* MW 22-APR-1997 Changes for MPEG dequantization */
                if(GetVopQuantType(rec_vop))
                    BlockDequantMPEG(q_block,quant_step,INTRA,1,
                                     GetVopIntraQuantMat(rec_vop),DCT_block,last_non_null_coeff);
                else
                    BlockDequantH263(q_block,quant_step,INTRA,1,DCT_block,last_non_null_coeff);
                
                DC_store[mbnum][comp][0] = DCT_block[0];
                
                if (trace->MB_tdata_after_IQ)
                    for (i=0;i<64;i++)
                        mb_dct_iq[comp][i]=DCT_block[i]; 
                
                
                
                BlockIDCT(DCT_block, r_block_2D);
                
                for (m = 0; m < B_SIZE; m++)
                    for (n = 0; n < B_SIZE; n++)
                        r_block[m*B_SIZE+n] = r_block_2D[m][n];
                
                if (trace->MB_tdata_after_IDCT)
                    for (i=0;i<64;i++)
                        mb_idct[comp][i]=r_block[i]; 
                
                
            }
            else /* MODE INTER */
            {
                if (CBP & (1 << (BLOCK_CNT - 1 - comp))) {
                    VlcGetBlock (texture_bits, q_block, INTER, trace,
                                 error_flag, 0, GetVopReverseVlc(rec_vop),switched,&last_non_null_coeff);
                }
                
                if (*error_flag == RETURN)
                    return;
                
                /* Inverse zigzag-scanning also here 12-11-96 MW */
                for (m = 0; m < 64; m++)
                    pcoeff[m] = q_block[zigzag[m]];
                
                for (m = 0; m < 8; m++)
                    for (n = 0; n < 8; n++)
                        q_block[m*8+n] = pcoeff[m*8+n];
                if (trace->MB_tdata_before_IQ)
                    for (i=0;i<64;i++)
                        mb_dct_bf_iq[comp][i]=q_block[i]; 
                
                /* Fill DC_store with default coeff values */
                DC_store[mbnum][comp][0] = 128*8;
                for (n = 1; n < 15; n++)
                    DC_store[mbnum][comp][n] = 0;
                
                
                /* MW 22-APR-1997 Changes for MPEG dequantization */
                if(GetVopQuantType(rec_vop))
                    BlockDequantMPEG(q_block,quant_step,INTER,1,
                                     GetVopNonintraQuantMat(rec_vop),DCT_block,last_non_null_coeff);
                else
                    BlockDequantH263(q_block,quant_step,INTER,1,DCT_block,last_non_null_coeff);
                
                
                if (trace->MB_tdata_after_IQ)
                    for (i=0;i<64;i++)
                        mb_dct_iq[comp][i]=DCT_block[i]; 
                
                BlockIDCT(DCT_block, r_block_2D);
                
                for (m = 0; m < B_SIZE; m++)
                    for (n = 0; n < B_SIZE; n++)
                        r_block[m*B_SIZE+n] = r_block_2D[m][n];
                
                if (trace->MB_tdata_after_IDCT)
                    for (i=0;i<64;i++)
                        mb_idct[comp][i]=r_block[i]; 
                
                
            }
	}
	else
        {
            Bzero(r_block);
            DC_store[mbnum][comp][0] = 128*8;
            for (n = 1; n < 15; n++)
                DC_store[mbnum][comp][n] = 0;
            
        }
        PutBlock (comp, r_block, mblock);
    }
    
    /*****
     *
     *	  Decoding of 2 chrominance blocks (not dep. on 
     *        transparent pattern)
     *
     *****/
    for (comp=4; comp<6; comp++)
    {
        Bzero (q_block);
        
        if ((disable_sadct==0)||(level_3_trace)) { /* Noel/E14 */
            switch(comp) {
                case 0 :
                    for (i=0; i<B_SIZE; i++)
                        for (j=0; j<B_SIZE; j++)
                            alpha_block[i][j] = alpha[(y_pos*MB_SIZE + i)*nrpix + (x_pos*MB_SIZE + j)];
                    break;
                case 1 :
                    for (i=0; i<B_SIZE; i++)
                        for (j=0; j<B_SIZE; j++)
                            alpha_block[i][j] = alpha[(y_pos*MB_SIZE + i)*nrpix + (x_pos*MB_SIZE + 8 + j)];
                    break;
                case 2 :
                    for (i=0; i<B_SIZE; i++)
                        for (j=0; j<B_SIZE; j++)
                            alpha_block[i][j] = alpha[(y_pos*MB_SIZE + 8 + i)*nrpix + (x_pos*MB_SIZE + j)];
                    break;
                case 3 :
                    for (i=0; i<B_SIZE; i++)
                        for (j=0; j<B_SIZE; j++)
                            alpha_block[i][j] = alpha[(y_pos*MB_SIZE + 8 + i)*nrpix + (x_pos*MB_SIZE + 8 + j)];
                    break;
                case 4 :
                case 5 :
                    for (i=0; i<B_SIZE; i++)
                        for (j=0; j<B_SIZE; j++) {
                            if (shape == GREY_SCALE) {
                                tmp = alpha[(y_pos*MB_SIZE + 2*i)*nrpix + (x_pos*MB_SIZE + 2*j)];
                                tmp += alpha[(y_pos*MB_SIZE + 2*i)*nrpix + (x_pos*MB_SIZE + 2*j+1)];
                                tmp += alpha[(y_pos*MB_SIZE + 2*i+1)*nrpix + (x_pos*MB_SIZE + 2*j)];
                                tmp += alpha[(y_pos*MB_SIZE + 2*i+1)*nrpix + (x_pos*MB_SIZE + 2*j+1)];
                                tmp += 2;
                                tmp /= 4;
                                alpha_block[i][j] = (tmp > BINARY_ALPHA/2) ? BINARY_ALPHA : 0;
                            }
                            else { /* so-called "conservative" subsampling (JDL - 20-MAR-97) */
                                alpha_block[i][j] = 
                                    ((alpha[(y_pos*MB_SIZE + 2*i)*nrpix + (x_pos*MB_SIZE + 2*j)] == BINARY_ALPHA) ||
                                     (alpha[(y_pos*MB_SIZE + 2*i)*nrpix + (x_pos*MB_SIZE + 2*j+1)] == BINARY_ALPHA) ||
                                     (alpha[(y_pos*MB_SIZE + 2*i+1)*nrpix + (x_pos*MB_SIZE + 2*j)] == BINARY_ALPHA) ||
                                     (alpha[(y_pos*MB_SIZE + 2*i+1)*nrpix + (x_pos*MB_SIZE + 2*j+1)] == BINARY_ALPHA))
                                    ? BINARY_ALPHA : 0 ; 
                            }
                        }
                    break;
                default :
                    for (i=0; i<B_SIZE; i++)
                        for (j=0; j<B_SIZE; j++)
                            alpha_block[i][j] = BINARY_ALPHA;
            }
            if (trace->trace)
                for (i=0;i<8;i++)
                    for (j=0;j<8;j++)
                        mb_shape[comp][i*8+j]=alpha_block[i][j];
        }
        if (mode == MODE_INTRA || mode == MODE_INTRA_Q)
	{
            q_block[0] = DC_coeff[comp];
            
            
            if (CBP & (1 << (BLOCK_CNT - 1 - comp)))
                VlcGetBlock (texture_bits, q_block, INTRA, trace,
                             error_flag, 1, GetVopReverseVlc(rec_vop),switched,&last_non_null_coeff);
            
            if (*error_flag == RETURN)
                return;
            
            
            /* Do the DC/AC prediction, changing the qcoeff values as
               appropriate */
            if (intra_dcpred_disable == 0)
                doDCACreconErrRes(q_block, 64, DC_store, QP_store, QP,
                                  x_pos, y_pos, MB_width, ACpred_flag, comp, slice_nb);
            else
	    {
                /* Ordinary inverse zigzag-scanning */
                for (m = 0; m < 64; m++)
                    pcoeff[m] = q_block[zigzag[m]];
                for (m = 0; m < 8; m++)
                    for (n = 0; n < 8; n++)
                        q_block[m*8+n] = pcoeff[m*8+n];
	    }
            
            if (trace->MB_tdata_before_IQ)
                for (i=0;i<64;i++)
                    mb_dct_bf_iq[comp][i]=q_block[i];
            
            /* Store the qcoeff-values needed later for prediction */
            for (m = 0; m < 8; m++) {
                DC_store[mbnum][comp][m] = q_block[m];
            }
            for (m = 0; m < 7; m++) {
                DC_store[mbnum][comp][m+8] = q_block[(m+1)*8];
            }
            
            /* MW 22-APR-1997 Changes for MPEG dequantization */
            if(GetVopQuantType(rec_vop))
                BlockDequantMPEG(q_block,quant_step,INTRA,2,
                                 GetVopIntraQuantMat(rec_vop),DCT_block,last_non_null_coeff);
            else
                BlockDequantH263(q_block,quant_step,INTRA,2,DCT_block,last_non_null_coeff);
            
            DC_store[mbnum][comp][0] = DCT_block[0];
            
            if (trace->MB_tdata_after_IQ)
                for (i=0;i<64;i++)
                    mb_dct_iq[comp][i]=DCT_block[i]; 
            
            
            BlockIDCT(DCT_block, r_block_2D);
            
            for (m = 0; m < B_SIZE; m++)
                for (n = 0; n < B_SIZE; n++)
                    r_block[m*B_SIZE+n] = r_block_2D[m][n];
            if (trace->MB_tdata_after_IDCT)
                for (i=0;i<64;i++)
                    mb_idct[comp][i]=r_block[i]; 
            
            
	}
        else		/* MODE_INTER */
	{
            if (CBP & (1 << (BLOCK_CNT - 1 - comp)))
                VlcGetBlock (texture_bits, q_block, INTER, trace,
                             error_flag, 0, GetVopReverseVlc(rec_vop),switched,&last_non_null_coeff);
            
            if (*error_flag == RETURN)
                return;
            
            
            /* Inverse zigzag-scanning also here 12-11-96 MW */
            for (m = 0; m < 64; m++)
                pcoeff[m] = q_block[zigzag[m]];
            
            for (m = 0; m < 8; m++)
                for (n = 0; n < 8; n++)
                    q_block[m*8+n] = pcoeff[m*8+n];
            if (trace->MB_tdata_before_IQ)
                for (i=0;i<64;i++)
                    mb_dct_bf_iq[comp][i]=q_block[i]; 
            
            /* Fill DC_store with default coeff values */
            DC_store[mbnum][comp][0] = 128*8;
            for (n = 1; n < 15; n++)
                DC_store[mbnum][comp][n] = 0;
            
            
            /* MW 22-APR-1997 Changes for MPEG dequantization */
            if(GetVopQuantType(rec_vop))
                BlockDequantMPEG(q_block,quant_step,INTER,2,
                                 GetVopNonintraQuantMat(rec_vop),DCT_block,last_non_null_coeff);
            else
                BlockDequantH263(q_block,quant_step,INTER,2,DCT_block,last_non_null_coeff);
            
            if (trace->MB_tdata_after_IQ)
                for (i=0;i<64;i++)
                    mb_dct_iq[comp][i]=DCT_block[i]; 
            
            
            BlockIDCT(DCT_block, r_block_2D);
            
            for (m = 0; m < B_SIZE; m++)
                for (n = 0; n < B_SIZE; n++)
                    r_block[m*B_SIZE+n] = r_block_2D[m][n];
            if (trace->MB_tdata_after_IDCT)
                for (i=0;i<64;i++)
                    mb_idct[comp][i]=r_block[i];
            
	}
        
        PutBlock (comp, r_block, mblock);
    }
    
    /*****
     *
     *	  Clip reconstructed intra MBs
     *
     *****/
    if (mode == MODE_INTRA)
        MB_clip (mblock);
    
    if (trace->MB_sdata) 
        PrintOutMBData("MB_shape_data",mb_shape,mb_shape,trace->fp_trace);
    if (trace->MB_tdata_before_IQ)
        PrintOutMBData("MB_tdata_before_IQ",mb_shape,mb_dct_bf_iq,trace->fp_trace); 
    if (trace->MB_tdata_after_IQ)
        PrintOutMBData("MB_tdata_after_IQ",mb_shape,mb_dct_iq,trace->fp_trace);
    if (trace->MB_tdata_after_IDCT)
        PrintOutMBData("MB_tdata_after_IDCT",mb_shape,mb_idct,trace->fp_trace);
    if ((mode == MODE_INTRA)&&(trace->MB_tdata_reconstructed)) {
        for (i=0;i<6;i++)
            for (j=0;j<64;j++)
                if (mb_idct[i][j]<0) mb_idct[i][j]=0;
                else if (mb_idct[i][j]>255) mb_idct[i][j]=255;
        
        PrintOutMBData("MB_tdata_reconstructed",mb_shape,mb_idct,trace->fp_trace);
    }
    
}                               /* GetMBblockdataErrRes */


/***********************************************************CommentBegin******
 *
 * -- GetMBblockdataNoDataPartErrRes --  
 *
 * Author :
 *      Luis Ducla-Soares (IST) - lds@lx.it.pt
 *
 * Created :
 *      08.09.97
 *
 * Purpose :
 *       decodes the block data (DCT coefficients, alias "texture") of a MB
 *       for the combined error resilient mode with data partitioning.
 *
 * Arguments in :
 *	Int intra_dcpred_disable	enable/disable pred. of INTRA DC
 *      Int quant_step,                 vop quantizer step (used for all MBs).
 *      Bitstream * texture_bits,       Image made of texture parameters.
 *      Int mbnum,                      macroblock number.
 *      Int trace,                      flag for tracing the process.
 *      Int *motion_mode,               already filled by MotionDecode.
 *	Int MB_width,
 *	Int DC_store[][6][15],		Stores DCAC prediction values
 *	SInt *QP_store,			Stores QPs for every MB
 *	Int QP				Current QPn
 *	UInt CBP			Coded Block Pattern derived from MB header data
 *      UChar disable_sadct             Flag indicating whether SADCT is disabled or not
 *      Short *alpha                    The object plane with alpha values
 *      UChar shape                     Indicator of shape type of the video object being decoded
 *
 * Arguments in/out :
 *
 *
 * Arguments out :
 *      Macroblock *mblock,             decoded macroblock.
 *      Int *skipped_flag,              1 = macroblock is skipped.
 *      Int *error_flag,                set if an error occured.
 *
 *
 * Return values :
 *
 *
 * Side effects :
 *
 *
 * Description :
 *        This function was based on GetMBblockdata().
 *
 * See also :
 *
 *
 * Modified :
 *       14-05-97 Minhua Zhou: disable SADCT for rectangular VOP
 *       15.05.97 Luis Ducla-Soares: cleaned up the code.
 *			 19.11.97 Noel Brady: Added trace stuff for shape
 ***********************************************************CommentEnd********/

Void
CallingConvention
GetMBblockdataNoDataPartErrRes(Int *transp_pattern,
                               Int intra_dcpred_disable,
                               Int quant_step,
                               Bitstream * texture_bits,
                               Int mbnum,
                               Trace *trace,
                               Macroblock * mblock,
                               Int * error_flag,
                               Int MB_width,
                               Int ***DC_store,
                               SInt *QP_store,
                               Int QP,
                               UInt CBP,
                               Int ACpred_flag,
                               UInt mode,
                               Vop *rec_vop,
                               Int **slice_nb
                               )
{
    Int  comp;
    Int  DCT_block[B_SIZE * B_SIZE];
    Int  q_block[B_SIZE * B_SIZE];
    Int  r_block[B_SIZE * B_SIZE];
    Int  r_block_2D[B_SIZE][B_SIZE];
    Int  pcoeff[B_SIZE * B_SIZE];
    UInt DC_coeff;
    Int  m, n;
    Int  i,j;
    Int  x_pos = mbnum % MB_width;
    Int  y_pos = mbnum / MB_width;
    Int  switched;
    
    Int last_non_null_coeff;

    UChar disable_sadct = 1;  /* Noel/E14 */
    UChar shape         = GetVopShape(rec_vop);
    
    SInt *alpha = (SInt *) GetImageData(GetVopA(rec_vop));
    
    /* variables because of contour-adaptive DCT */
    Short alpha_block[B_SIZE][B_SIZE];
    Short nrpix = MB_width * MB_SIZE;
    Short tmp; 
    
    Char level_3_trace;
    SInt mb_shape[6][64],mb_dct_bf_iq[6][64],mb_dct_iq[6][64],mb_idct[6][64]; 
    
    MBzero (mblock);
    level_3_trace = trace->MB_sdata||trace->MB_tdata_before_IQ|| 
        trace->MB_tdata_after_IQ||trace->MB_tdata_after_IDCT||
        trace->MB_tdata_reconstructed;
    if (level_3_trace)
        for (i=0;i<6;i++)
            for (j=0;j<64;j++) mb_shape[i][j]=0;
    /*added by Minhua Zhou 04.08.97 */
    switched = IntraDCSwitch_Decision(mode,
                                      GetVopIntraDCVlcThr(rec_vop),
                                      quant_step);
    
    
    
    /*****
     *
     *	  Decoding of the 4 luminance blocks (depending on transparent pattern)
     *
     *****/
    for (comp=0; comp<4; comp++)
    {
        Bzero (q_block);
        
        if(transp_pattern[comp]==0) /* No decoding of transparent Y blocks */
      	{
            if ((disable_sadct==0)||(level_3_trace)) {		
                switch(comp) {
                    case 0 :
                        for (i=0; i<B_SIZE; i++)
                            for (j=0; j<B_SIZE; j++)
                                alpha_block[i][j] = alpha[(y_pos*MB_SIZE + i)*nrpix + (x_pos*MB_SIZE + j)];
                        break;
                    case 1 :
                        for (i=0; i<B_SIZE; i++)
                            for (j=0; j<B_SIZE; j++)
                                alpha_block[i][j] = alpha[(y_pos*MB_SIZE + i)*nrpix + (x_pos*MB_SIZE + 8 + j)];
                        break;
                    case 2 :
                        for (i=0; i<B_SIZE; i++)
                            for (j=0; j<B_SIZE; j++)
                                alpha_block[i][j] = alpha[(y_pos*MB_SIZE + 8 + i)*nrpix + (x_pos*MB_SIZE + j)];
                        break;
                    case 3 :
                        for (i=0; i<B_SIZE; i++)
                            for (j=0; j<B_SIZE; j++)
                                alpha_block[i][j] = alpha[(y_pos*MB_SIZE + 8 + i)*nrpix + (x_pos*MB_SIZE + 8 + j)];
                        break;
                    case 4 :
                    case 5 :
                        for (i=0; i<B_SIZE; i++)
                            for (j=0; j<B_SIZE; j++) {
                                if (shape == GREY_SCALE) {
                                    tmp = alpha[(y_pos*MB_SIZE + 2*i)*nrpix + (x_pos*MB_SIZE + 2*j)];
                                    tmp += alpha[(y_pos*MB_SIZE + 2*i)*nrpix + (x_pos*MB_SIZE + 2*j+1)];
                                    tmp += alpha[(y_pos*MB_SIZE + 2*i+1)*nrpix + (x_pos*MB_SIZE + 2*j)];
                                    tmp += alpha[(y_pos*MB_SIZE + 2*i+1)*nrpix + (x_pos*MB_SIZE + 2*j+1)];
                                    tmp += 2;
                                    tmp /= 4;
                                    alpha_block[i][j] = (tmp > BINARY_ALPHA/2) ? BINARY_ALPHA : 0;
                                }
                                else { /* so-called "conservative" subsampling (JDL - 20-MAR-97) */
                                    alpha_block[i][j] = 
                                        ((alpha[(y_pos*MB_SIZE + 2*i)*nrpix + (x_pos*MB_SIZE + 2*j)] == BINARY_ALPHA) ||
                                         (alpha[(y_pos*MB_SIZE + 2*i)*nrpix + (x_pos*MB_SIZE + 2*j+1)] == BINARY_ALPHA) ||
                                         (alpha[(y_pos*MB_SIZE + 2*i+1)*nrpix + (x_pos*MB_SIZE + 2*j)] == BINARY_ALPHA) ||
                                         (alpha[(y_pos*MB_SIZE + 2*i+1)*nrpix + (x_pos*MB_SIZE + 2*j+1)] == BINARY_ALPHA))
                                        ? BINARY_ALPHA : 0 ; 
                                }
                            }
                        break;
                    default :
                        for (i=0; i<B_SIZE; i++)
                            for (j=0; j<B_SIZE; j++)
                                alpha_block[i][j] = BINARY_ALPHA;
                }
                if (trace->trace)
                    for (i=0;i<8;i++)
                        for (j=0;j<8;j++)
                            mb_shape[comp][i*8+j]=alpha_block[i][j]; 
            }
            
            if (mode == MODE_INTRA || mode == MODE_INTRA_Q) {
                if (switched==0) {
                    if (intra_dcpred_disable) /* No INTRA DC prediction */
                    {
                        DC_coeff = BitstreamReadBits(texture_bits, 8, "DC coeff", trace, NUM);
                        
                        if (DC_coeff == 128){
                            printf("Illegal DC coeff: 1000000\n");
                            exit(1);
                        }
                        if (DC_coeff == 255)
                            DC_coeff = 128;
                        
                        q_block[0] = DC_coeff;
                    }
                    else   /* INTRA DC prediction (I-MBs) */
                    {
                        /* This must be Int not UInt MW 15-Nov-1996 */
                        q_block[0] = DecodePredictedIntraDC(comp,texture_bits,
                                                            trace,NUM);
                        
                    }
                }
                
                
                if (CBP & (1 << (BLOCK_CNT - 1 - comp))) {
                    VlcGetBlock(texture_bits, q_block, INTRA,
                                trace, error_flag, 1, GetVopReverseVlc(rec_vop),switched,&last_non_null_coeff);
                }
                
                if (*error_flag == RETURN)
                    return;
                
                if (intra_dcpred_disable == 0)
                    doDCACreconErrRes(q_block, 64, DC_store, QP_store, QP,
                                      x_pos, y_pos, MB_width, ACpred_flag, comp, slice_nb);
                else
                {
                    /* Ordinary inverse zigzag-scanning */
                    for (m = 0; m < 64; m++)
                        pcoeff[m] = q_block[zigzag[m]];
                    for (m = 0; m < 8; m++)
                        for (n = 0; n < 8; n++)
                            q_block[m*8+n] = pcoeff[m*8+n];
                }	  
                
                if (trace->MB_tdata_before_IQ)
                    for (i=0;i<64;i++)
                        mb_dct_bf_iq[comp][i]=q_block[i]; 
                
                /* Store the qcoeff-values needed later for prediction */
                for (m = 0; m < 8; m++) {
                    DC_store[mbnum][comp][m] = q_block[m];
                }
                for (m = 0; m < 7; m++) {
                    DC_store[mbnum][comp][m+8] = q_block[(m+1)*8];
                }
                
                
                /* MW 22-APR-1997 Changes for MPEG dequantization */
                if(GetVopQuantType(rec_vop))
                    BlockDequantMPEG(q_block,quant_step,INTRA,1,
                                     GetVopIntraQuantMat(rec_vop),DCT_block,last_non_null_coeff);
                else
                    BlockDequantH263(q_block,quant_step,INTRA,1,DCT_block,last_non_null_coeff);
                
                DC_store[mbnum][comp][0] = DCT_block[0];
                
                if (trace->MB_tdata_after_IQ)
                    for (i=0;i<64;i++)
                        mb_dct_iq[comp][i]=DCT_block[i]; 
                
                
                BlockIDCT(DCT_block, r_block_2D);
                for (m = 0; m < B_SIZE; m++)
                    for (n = 0; n < B_SIZE; n++)
                        r_block[m*B_SIZE+n] = r_block_2D[m][n];
                
                if (trace->MB_tdata_after_IDCT)
                    for (i=0;i<64;i++)
                        mb_idct[comp][i]=r_block[i]; 
                
                
            }
            else /* MODE INTER */
            {
                if (CBP & (1 << (BLOCK_CNT - 1 - comp))) {
                    VlcGetBlock (texture_bits, q_block, INTER, trace,
                                 error_flag, 0, GetVopReverseVlc(rec_vop), switched,&last_non_null_coeff);
                }
                
                if (*error_flag == RETURN)
                    return;
                
                /* Inverse zigzag-scanning also here 12-11-96 MW */
                for (m = 0; m < 64; m++)
                    pcoeff[m] = q_block[zigzag[m]];
                
                for (m = 0; m < 8; m++)
                    for (n = 0; n < 8; n++)
                        q_block[m*8+n] = pcoeff[m*8+n];
                if (trace->MB_tdata_before_IQ)
                    for (i=0;i<64;i++)
                        mb_dct_bf_iq[comp][i]=q_block[i]; 
                
                
                /* Fill DC_store with default coeff values */
                DC_store[mbnum][comp][0] = 128*8;
                for (n = 1; n < 15; n++)
                    DC_store[mbnum][comp][n] = 0;
                
                
                /* MW 22-APR-1997 Changes for MPEG dequantization */
                if(GetVopQuantType(rec_vop))
                    BlockDequantMPEG(q_block,quant_step,INTER,1,
                                     GetVopNonintraQuantMat(rec_vop),DCT_block,last_non_null_coeff);
                else
                    BlockDequantH263(q_block,quant_step,INTER,1,DCT_block,last_non_null_coeff);
                
                for (i=0;i<64;i++)
                    DCT_block[i] =  MIN(2047,MAX(-2048,DCT_block[i]));  /* VM31 Clp */
                
                if (trace->MB_tdata_after_IQ)
                    for (i=0;i<64;i++)
                        mb_dct_iq[comp][i]=DCT_block[i]; 
                
                BlockIDCT(DCT_block, r_block_2D);
                
                for (m = 0; m < B_SIZE; m++)
                    for (n = 0; n < B_SIZE; n++)
                        r_block[m*B_SIZE+n] = r_block_2D[m][n];
                
                if (trace->MB_tdata_after_IDCT)
                    for (i=0;i<64;i++)
                        mb_idct[comp][i]=r_block[i]; 
                
                
                
            }
        }
	else
        {
            Bzero(r_block);
            DC_store[mbnum][comp][0] = 128*8;
            for (n = 1; n < 15; n++)
                DC_store[mbnum][comp][n] = 0;
            
        }
        PutBlock (comp, r_block, mblock);
    }
    
    /*****
     *
     *	  Decoding of 2 chrominance blocks (not dep. on 
     *        transparent pattern)
     *
     *****/
    for (comp=4; comp<6; comp++)
    {
        Bzero (q_block);
        
        if ((disable_sadct==0)||(level_3_trace)) { /* Noel/E14 */
            switch(comp) {
                case 0 :
                    for (i=0; i<B_SIZE; i++)
                        for (j=0; j<B_SIZE; j++)
                            alpha_block[i][j] = alpha[(y_pos*MB_SIZE + i)*nrpix + (x_pos*MB_SIZE + j)];
                    break;
                case 1 :
                    for (i=0; i<B_SIZE; i++)
                        for (j=0; j<B_SIZE; j++)
                            alpha_block[i][j] = alpha[(y_pos*MB_SIZE + i)*nrpix + (x_pos*MB_SIZE + 8 + j)];
                    break;
                case 2 :
                    for (i=0; i<B_SIZE; i++)
                        for (j=0; j<B_SIZE; j++)
                            alpha_block[i][j] = alpha[(y_pos*MB_SIZE + 8 + i)*nrpix + (x_pos*MB_SIZE + j)];
                    break;
                case 3 :
                    for (i=0; i<B_SIZE; i++)
                        for (j=0; j<B_SIZE; j++)
                            alpha_block[i][j] = alpha[(y_pos*MB_SIZE + 8 + i)*nrpix + (x_pos*MB_SIZE + 8 + j)];
                    break;
                case 4 :
                case 5 :
                    for (i=0; i<B_SIZE; i++)
                        for (j=0; j<B_SIZE; j++) {
                            if (shape == GREY_SCALE) {
                                tmp = alpha[(y_pos*MB_SIZE + 2*i)*nrpix + (x_pos*MB_SIZE + 2*j)];
                                tmp += alpha[(y_pos*MB_SIZE + 2*i)*nrpix + (x_pos*MB_SIZE + 2*j+1)];
                                tmp += alpha[(y_pos*MB_SIZE + 2*i+1)*nrpix + (x_pos*MB_SIZE + 2*j)];
                                tmp += alpha[(y_pos*MB_SIZE + 2*i+1)*nrpix + (x_pos*MB_SIZE + 2*j+1)];
                                tmp += 2;
                                tmp /= 4;
                                alpha_block[i][j] = (tmp > BINARY_ALPHA/2) ? BINARY_ALPHA : 0;
                            }
                            else { /* so-called "conservative" subsampling (JDL - 20-MAR-97) */
                                alpha_block[i][j] = 
                                    ((alpha[(y_pos*MB_SIZE + 2*i)*nrpix + (x_pos*MB_SIZE + 2*j)] == BINARY_ALPHA) ||
                                     (alpha[(y_pos*MB_SIZE + 2*i)*nrpix + (x_pos*MB_SIZE + 2*j+1)] == BINARY_ALPHA) ||
                                     (alpha[(y_pos*MB_SIZE + 2*i+1)*nrpix + (x_pos*MB_SIZE + 2*j)] == BINARY_ALPHA) ||
                                     (alpha[(y_pos*MB_SIZE + 2*i+1)*nrpix + (x_pos*MB_SIZE + 2*j+1)] == BINARY_ALPHA))
                                    ? BINARY_ALPHA : 0 ; 
                            }
                        }
                    break;
                default :
                    for (i=0; i<B_SIZE; i++)
                        for (j=0; j<B_SIZE; j++)
                            alpha_block[i][j] = BINARY_ALPHA;
            }
            if (trace->trace)
                for (i=0;i<8;i++)
                    for (j=0;j<8;j++)
                        mb_shape[comp][i*8+j]=alpha_block[i][j];
        }
        if (mode == MODE_INTRA || mode == MODE_INTRA_Q)
	{
            if (switched==0) {
                if (intra_dcpred_disable)    /* No INTRA DC prediction */
                {
                    DC_coeff = BitstreamReadBits(texture_bits, 8, "DC coeff", trace,
                                                 NUM);
                    
                    if (DC_coeff == 128){
                        printf("Illegal DC coeff: 1000000\n");
                        exit(1);
                    }
                    if (DC_coeff == 255)
                        DC_coeff = 128;
                    
                }
                else            /* INTRA DC prediction (I-MBs) */
                {
                    DC_coeff = DecodePredictedIntraDC(comp, texture_bits,
                                                      trace, NUM);
                }
                
                q_block[0] = DC_coeff;
            }	     
	    
            if (CBP & (1 << (BLOCK_CNT - 1 - comp)))
                if (GetVopReverseVlc(rec_vop))
                    VlcGetBlock (texture_bits, q_block, INTRA, trace,
                                 error_flag, 1, GetVopReverseVlc(rec_vop), switched,&last_non_null_coeff);
                else
                    VlcGetBlock (texture_bits, q_block, INTRA, trace,
                                 error_flag, 1, GetVopReverseVlc(rec_vop), switched,&last_non_null_coeff); /*it should be 1 */
            
            if (*error_flag == RETURN)
                return;
            
            /* Do the DC/AC prediction, changing the qcoeff values as
               appropriate */
            if (intra_dcpred_disable == 0)
                doDCACreconErrRes(q_block, 64, DC_store, QP_store, QP,
                                  x_pos, y_pos, MB_width, ACpred_flag, comp, slice_nb);
            else
	    {
                /* Ordinary inverse zigzag-scanning */
                for (m = 0; m < 64; m++)
                    pcoeff[m] = q_block[zigzag[m]];
                for (m = 0; m < 8; m++)
                    for (n = 0; n < 8; n++)
                        q_block[m*8+n] = pcoeff[m*8+n];
	    }
            
            if (trace->MB_tdata_before_IQ)
                for (i=0;i<64;i++)
                    mb_dct_bf_iq[comp][i]=q_block[i];
            
            /* Store the qcoeff-values needed later for prediction */
            for (m = 0; m < 8; m++) {
                DC_store[mbnum][comp][m] = q_block[m];
            }
            for (m = 0; m < 7; m++) {
                DC_store[mbnum][comp][m+8] = q_block[(m+1)*8];
            }
            
            /* MW 22-APR-1997 Changes for MPEG dequantization */
            if(GetVopQuantType(rec_vop))
                BlockDequantMPEG(q_block,quant_step,INTRA,2,
                                 GetVopIntraQuantMat(rec_vop),DCT_block,last_non_null_coeff);
            else
                BlockDequantH263(q_block,quant_step,INTRA,2,DCT_block,last_non_null_coeff);
            
            DC_store[mbnum][comp][0] = DCT_block[0];
            
            if (trace->MB_tdata_after_IQ)
                for (i=0;i<64;i++)
                    mb_dct_iq[comp][i]=DCT_block[i]; 
            
            
            BlockIDCT(DCT_block, r_block_2D);
            
            for (m = 0; m < B_SIZE; m++)
                for (n = 0; n < B_SIZE; n++)
                    r_block[m*B_SIZE+n] = r_block_2D[m][n];
            if (trace->MB_tdata_after_IDCT)
                for (i=0;i<64;i++)
                    mb_idct[comp][i]=r_block[i]; 
            
            
	}
        else		/* MODE_INTER */
	{
            if (CBP & (1 << (BLOCK_CNT - 1 - comp)))
                VlcGetBlock (texture_bits, q_block, INTER, trace,
                             error_flag, 0, GetVopReverseVlc(rec_vop),switched,&last_non_null_coeff);
            
            if (*error_flag == RETURN)
                return;
            
            
            /* Inverse zigzag-scanning also here 12-11-96 MW */
            for (m = 0; m < 64; m++)
                pcoeff[m] = q_block[zigzag[m]];
            
            for (m = 0; m < 8; m++)
                for (n = 0; n < 8; n++)
                    q_block[m*8+n] = pcoeff[m*8+n];
            if (trace->MB_tdata_before_IQ)
                for (i=0;i<64;i++)
                    mb_dct_bf_iq[comp][i]=q_block[i]; 
            
            /* Fill DC_store with default coeff values */
            DC_store[mbnum][comp][0] = 128*8;
            for (n = 1; n < 15; n++)
                DC_store[mbnum][comp][n] = 0;
            
            
            /* MW 22-APR-1997 Changes for MPEG dequantization */
            if(GetVopQuantType(rec_vop))
                BlockDequantMPEG(q_block,quant_step,INTER,2,
                                 GetVopNonintraQuantMat(rec_vop),DCT_block,last_non_null_coeff);
            else
                BlockDequantH263(q_block,quant_step,INTER,2,DCT_block,last_non_null_coeff);
            
#if 0	  
            for (i=0;i<64;i++)
                DCT_block[i] =  MIN(2047,MAX(-2048,DCT_block[i]));  /* VM31 Clp */
#endif     
            
            if (trace->MB_tdata_after_IQ)
                for (i=0;i<64;i++)
                    mb_dct_iq[comp][i]=DCT_block[i]; 
            
            
            BlockIDCT(DCT_block, r_block_2D);
            
            for (m = 0; m < B_SIZE; m++)
                for (n = 0; n < B_SIZE; n++)
                    r_block[m*B_SIZE+n] = r_block_2D[m][n];
            if (trace->MB_tdata_after_IDCT)
                for (i=0;i<64;i++)
                    mb_idct[comp][i]=r_block[i];
            
	}
        
        PutBlock (comp, r_block, mblock);
    }
    
    /*****
     *
     *	  Clip reconstructed intra MBs
     *
     *****/
    if ((mode == MODE_INTRA)||(mode == MODE_INTRA_Q))
        MB_clip (mblock);
    
    if (trace->MB_sdata) 
        PrintOutMBData("MB_shape_data",mb_shape,mb_shape,trace->fp_trace);
    if (trace->MB_tdata_before_IQ)
        PrintOutMBData("MB_tdata_before_IQ",mb_shape,mb_dct_bf_iq,trace->fp_trace); 
    if (trace->MB_tdata_after_IQ)
        PrintOutMBData("MB_tdata_after_IQ",mb_shape,mb_dct_iq,trace->fp_trace);
    if (trace->MB_tdata_after_IDCT)
        PrintOutMBData("MB_tdata_after_IDCT",mb_shape,mb_idct,trace->fp_trace);
    if ((mode == MODE_INTRA)&&(trace->MB_tdata_reconstructed)) {
        for (i=0;i<6;i++)
            for (j=0;j<64;j++)
                if (mb_idct[i][j]<0) mb_idct[i][j]=0;
                else if (mb_idct[i][j]>255) mb_idct[i][j]=255;
        
        PrintOutMBData("MB_tdata_reconstructed",mb_shape,mb_idct,trace->fp_trace);
    }
    
}                               /* GetMBblockdataNoDataPartErrRes */


