#ifndef _DEC_INTERFACE_C_
#define _DEC_INTERFACE_C_


#ifdef _IM1_CD_DECODER_
/******************************************************/
/* A.Pacheco: 03/11/98:                               */
/*   Inteface of the MPEG4 CD decoder for the IM1 api */
/******************************************************/

#include <string.h>
#include "momusys.h"
#include "mom_bitstream_d.h"
#include "vm_compos.h"
#include "text_decode.h"
#include "mot_padding.h"
#include "io_generic.h"
#include "mom_vol.h"
#include "mom_access.h"
#include "mot_comp.h"
#include "zigzag.h"
#include "post_filter.h"
#include "mom_vo.h"
#include "combined_decode.h"
#include "vm_common_defs.h"
#include "sprite_util.h"
#include "sprite_dec_util.h"
#include "sprite_dec_piece.h"
#include "alp_dec_mom.h"
#include "alp_dec_header.h"
#include "text_quant_mat_def.h"
#include "text_quant.h"
/* #include "tpsdec.h"*/ /*SpSc*/
#include "do_bgc.h" 
#include "scal_dec.h" /*SpSc*/

#include "vm_dec_main.h"

#define TIME_INC 40   /* display rate in ms */

#if defined(_IM1_CD_DECODER_) && defined(WIN32)
#define APADLLEXPORTS      __declspec( dllexport )
#include <windows.h>
static HANDLE m_hMutexTest=NULL;

__inline void StartMutex()
{
	if(m_hMutexTest==NULL)
		m_hMutexTest = CreateMutex (NULL, FALSE, NULL);
}

__inline void BeginMutex()
{
        WaitForSingleObject (m_hMutexTest, INFINITE);
}

__inline void EndMutex()
{
	ReleaseMutex (m_hMutexTest);
}

#else
#define APADLLEXPORTS
#if defined(PTHREADS)

#include <pthread.h>

static int initialised = 0;
static pthread_mutex_t mutexTest;

void StartMutex()
{
        if(!initialised) {
	        pthread_mutex_init(&mutexTest, NULL);
		initialised = 1;
	}
}

void BeginMutex()
{
	pthread_mutex_lock(&mutexTest);
}

void EndMutex()
{
	pthread_mutex_unlock(&mutexTest);
}
#else

#include <ulocks.h>

static ulock_t mutex;
static usptr_t* arena = NULL;

void StartMutex()
{
        if(!arena) {
		usconfig(CONF_ARENATYPE, US_SHAREDONLY);
		if (!(arena = usinit("/dev/zero")))
			perror("usinit");
	        mutex = usnewlock(arena);
	}
}

void BeginMutex()
{
	ussetlock(mutex);
}

void EndMutex()
{
	usunsetlock(mutex);
}
#endif
#endif

/****************************************************************************/
#include "idct.h"
#include "chrono.h"
#include "ring.h"
/****************************************************************************/

Void
CallingConvention
DecodeVolHeader(Bitstream *stream, Trace *trace, Vol *curr_vol, 
                Int *readen_bits, Int vo_id, Int file_vol_id
				,Int temporal_scalability,Int *prev_ref,Int *next_ref,
				UChar **MB_TYPE2[MAX_NUM_VOS][MAX_NUM_VOLS],
				Int PRE_H_SIZE2[MAX_NUM_VOS][MAX_NUM_VOLS],
				Int PRE_V_SIZE2[MAX_NUM_VOS][MAX_NUM_VOLS],
				UChar **PRE_MB_TYPE2[MAX_NUM_VOS][MAX_NUM_VOLS],
				Int bs_vol[MAX_NUM_VOS][MAX_NUM_VOLS],
				Image   *alpha_rec_packet,
				Image   *alpha_mode_packet,
				SInt Px_for,SInt Py_for, SInt Px_back, SInt Py_back,
				Int prev_j,Int zero_edge,Int prev_time_base,
				UChar **stock_mb_type2[3][MAX_NUM_VOS][MAX_NUM_VOLS],
				Int   stock_h_size2[3][MAX_NUM_VOS][MAX_NUM_VOLS],
				Int   stock_v_size2[3][MAX_NUM_VOS][MAX_NUM_VOLS]);
Void
CallingConvention
DecodeVolHeaderList(Int number_of_vols, Bitstream *stream[], 
                    Trace *trace[], Vol *vol_list, Int readen_bits[],
                    Int vo_id,Int temporal_scalability,Int *prev_ref,Int *next_ref,
					UChar **MB_TYPE2[MAX_NUM_VOS][MAX_NUM_VOLS],
					Int PRE_H_SIZE2[MAX_NUM_VOS][MAX_NUM_VOLS],
					Int PRE_V_SIZE2[MAX_NUM_VOS][MAX_NUM_VOLS],
					UChar **PRE_MB_TYPE2[MAX_NUM_VOS][MAX_NUM_VOLS],
					Int bs_vol[MAX_NUM_VOS][MAX_NUM_VOLS],
					Image   *alpha_rec_packet,
					Image   *alpha_mode_packet,
					SInt Px_for,SInt Py_for, SInt Px_back, SInt Py_back,
					Int prev_j,Int zero_edge,Int prev_time_base,
					UChar **stock_mb_type2[3][MAX_NUM_VOS][MAX_NUM_VOLS],
					Int   stock_h_size2[3][MAX_NUM_VOS][MAX_NUM_VOLS],
					Int   stock_v_size2[3][MAX_NUM_VOS][MAX_NUM_VOLS]);
Void
CallingConvention
DecodeVolList(Int number_of_vols, Bitstream *stream[], Trace *trace[], 
              Float next_display_time, Int random_access_start_time,
              Vol *vol_list, Int readen_bits[], 
              Int stop_decoding_vol[], Int vo_id, Int post_filter_type,
			  Vop	      *base_output_vop[3],Float display_framerate_TPS
			  ,Int temporal_scalability,Int last_base_decoding_frame[3],
			  Int *last_enh_decoding_frame,Int *TPS_base_count,Int   *prev_ref,
			  Int *next_ref,UChar **MB_TYPE2[MAX_NUM_VOS][MAX_NUM_VOLS],
			  Int PRE_H_SIZE2[MAX_NUM_VOS][MAX_NUM_VOLS],
			  Int PRE_V_SIZE2[MAX_NUM_VOS][MAX_NUM_VOLS],
			  UChar **PRE_MB_TYPE2[MAX_NUM_VOS][MAX_NUM_VOLS],
			  Int bs_vol[MAX_NUM_VOS][MAX_NUM_VOLS],
			  Image   *alpha_rec_packet,
			  Image   *alpha_mode_packet,
			  SInt Px_for,SInt Py_for, SInt Px_back, SInt Py_back,
			  Int prev_j,Int zero_edge,Int prev_time_base,
			  Int prev_predic_type,
			  Int image_width,Int image_height,
			  Vop	*back_shape_vop,
				Vop	*for_shape_vop,
				UChar **stock_mb_type2[3][MAX_NUM_VOS][MAX_NUM_VOLS],
				Int   stock_h_size2[3][MAX_NUM_VOS][MAX_NUM_VOLS],
				Int   stock_v_size2[3][MAX_NUM_VOS][MAX_NUM_VOLS]
			  );
Void
CallingConvention
FindStartGOV(Int number_of_vols,Bitstream *stream[], 
             Trace *trace[],Vol *vol_list, Int readen_bits[], Int random_access_start_time);

Vol *
CallingConvention
InitVolList(Int number_of_vols);

/****************************************************************************/
Int CallingConvention 
StopDecoding1VO(Int number_of_vols, Int stop_decoding_vol[][MAX_NUM_VOLS]);


/***********************************************************CommentBegin******
 *
 * -- InitMPEG4CDDecoder -- 
 *
 *          Initializates the MPEG4 CD decoder.
 *			(=> no data is needed in the buffer for this operation
 ***********************************************************CommentEnd********/
    

APADLLEXPORTS Void CallingConvention InitMPEG4CDDecoder(
	Int width, Int height, 
	Int vols,  Int framerate, 
	Int filter_type,
	Int        readen_bits[MAX_NUM_VOS][MAX_NUM_VOLS],
	Int        stop_decoding_vol[MAX_NUM_VOS][MAX_NUM_VOLS],
	Vop      **display_vop, 
	Float     *time_between_two_frames,
	Float     *next_display_time,
	Int		  *random_access_start_time,
	Bitstream *stream[MAX_NUM_VOS][MAX_NUM_VOLS],
    Vol       *display_vol_list[MAX_NUM_VOS],
    Trace     *trace[MAX_NUM_VOS][MAX_NUM_VOLS],
	Char       trace_filename[MAX_NUM_VOS][MAX_NUM_VOLS][300],
	Vop	      *base_output_vop[3],
	UChar **PRE_MB_TYPE2[MAX_NUM_VOS][MAX_NUM_VOLS],
	Int bs_vo[MAX_NUM_VOS],
	Int bs_vol[MAX_NUM_VOS][MAX_NUM_VOLS],
	Float      *display_framerate_TPS,
	Int			num_cd_decoders,
	UChar **stock_mb_type2[3][MAX_NUM_VOS][MAX_NUM_VOLS]
	)
{
  Int i,k;
  Vol *next_vol = NULL;
  Int display_framerate;
  Int display_width,display_height;

#if defined(_IM1_CD_DECODER_)
  StartMutex();
#endif
    
#if _OPTIMIZE_
    i = start_chrono( &(chrono[num_cd_decoders-1]) );
    
    if(num_cd_decoders==1)
	{
		init_idct();
    
		init_ring(1); /* 1 for QCIF, 0 for CIF */
    
		init_ClipTab(1024);
    
		init_ClipTab_for_Dequant();
	}
    
    i = stop_chrono( &(chrono[num_cd_decoders-1]) );
    
    printf("Init phase : %d.%02d seconds.\n",
           chrono[num_cd_decoders-1].runtime/1000, chrono[num_cd_decoders-1].runtime%1000 );    
#endif
    
    i = start_chrono( &(chrono[num_cd_decoders-1]) );

  /* default values, originally from the decoder.ctl file*/
  display_framerate		=	framerate;
  display_width			=	width;
  display_height		=	height;

		       
  /* display_vop, used for composition purposes */
  *display_vop = AllocVop(display_width,display_height);

  /*	Calculate the time between two frames to be displayed in ms
   *	(from display_frame_rate) and allocate the necessary VOL, bitstream
   *	and trace structures */
  *time_between_two_frames = (Float)1000 / display_framerate;

#if(__TEMPORAL_SCALABILITY__)
    /* for TPS */
    for (i=0; i<MAX_NUM_VOS; i++)
    {
        bs_vo[i] = -1;
        for (k=0; k<MAX_NUM_VOLS; k++)
            bs_vol[i][k] = -1;
    }
    /**/
#endif

  /* 1 VO */
#if(__TEMPORAL_SCALABILITY__)
  /* for TPS */
  bs_vo[0] = 0; /* This part needs to change when VO header is decoded. */
  /**/
#endif

  display_vol_list[0] = InitVolList(vols);

  for(k=0; k<vols; k++)
  {
      stream[0][k] = BitstreamOpen();
      readen_bits[0][k] = 0;
      stop_decoding_vol[0][k] = 0;

      if(k==0) strcpy(trace_filename[0][k], "buffer_mem_bas");
	  else     strcpy(trace_filename[0][k], "buffer_mem_enh");
	  switch (num_cd_decoders)
	  {
	  case 1:strcat(trace_filename[0][k], ".1");
			 break;
	  case 2:strcat(trace_filename[0][k], ".2");
			 break;
	  case 3:strcat(trace_filename[0][k], ".3");
			 break;
	  case 4:strcat(trace_filename[0][k], ".4");
			 break;
	  case 5:strcat(trace_filename[0][k], ".5");
			 break;
	  case 6:strcat(trace_filename[0][k], ".6");
			 break;
	  case 7:strcat(trace_filename[0][k], ".7");
			 break;
	  case 8:strcat(trace_filename[0][k], ".8");
			 break;
	  case 9:strcat(trace_filename[0][k], ".9");
			 break;
	  }
      strcat(trace_filename[0][k], ".dec_log");
     
      /*****
       *
       *    Trace capabilities. The settings by the users should be
       *    done on top of this file!
       *
       *    Please do NOT MODIFY any values below here !
       *
       *****/
      trace[0][k] = (Trace *) malloc( sizeof(Trace) );

      trace[0][k]->counter              = 0; 
      trace[0][k]->trace                = 0; 

      /* Set frames for full trace (see README for details) */
      trace[0][k]->full_trace_start     = 0; 
      trace[0][k]->full_trace_end       = 0; 
      trace[0][k]->full_trace_period    = 0; 

      /* Set default trace levels (see README for details) */

      /* Level 1 data */
      trace[0][k]->VO_header_def  = 0;
      trace[0][k]->VOL_header_def = 0;
      trace[0][k]->VOP_header_def = 0;

      /* Level 2 data */
      trace[0][k]->MB_header_def  = 0;
      trace[0][k]->MB_motion_def  = 0;
      trace[0][k]->MB_shape_def   = 0;
      trace[0][k]->MB_texture_def = 0;

      /* Level 3 data */
      trace[0][k]->MB_sdata_def               = 0;
      trace[0][k]->MB_tdata_before_IQ_def     = 0;
      trace[0][k]->MB_tdata_after_IQ_def      = 0;
      trace[0][k]->MB_tdata_after_IDCT_def    = 0;
      trace[0][k]->MB_tdata_predicted_def     = 0;
      trace[0][k]->MB_tdata_reconstructed_def = 0;

      /* we need the buffer of the input bitstream ready
	  DecodeVolHeaderList(vols,stream[0],trace[0],
			display_vol_list[0],readen_bits[0],0);*/
    }

  /*****
   *
   *	Initializations
   *
   *****/

  /* for random access */
  if (*random_access_start_time != 0)
    *random_access_start_time = 
      ((Int)(*random_access_start_time + 0.5*(*time_between_two_frames))/ (Int) (*time_between_two_frames))
      * (*time_between_two_frames);
  if (*random_access_start_time != 0)
    printf("Adjusted random_access_start_time=%i\n",*random_access_start_time);

  *next_display_time = 0;
#if(__TEMPORAL_SCALABILITY__)
  /* comp_image_counter = 0;*/
  for(i = 0; i < 3 ; i++) 
	  base_output_vop[i] = NULL;
  *display_framerate_TPS = display_framerate;


#endif  

  /*apa1*/
    InitAllPRE_PRE_MBtype2(stock_mb_type2);
  /*apaimage_width  = display_width;
  image_height = display_height;*/
  /**/
  InitAllPRE_MBtype2(PRE_MB_TYPE2);


}

/***********************************************************CommentBegin******
 *
 * -- DecodeMPEG4CDHeaders -- 
 *
 *          Decodes the data from the MPEG4 CD headers.
 *          (=>there must be enought data in the buffer to do this)
 *
 ***********************************************************CommentEnd********/


APADLLEXPORTS Void CallingConvention DecodeMPEG4CDHeaders(
  	Int        width, Int height, 
	Int        vols,  Int framerate, 
	Int        filter_type,
	Int		  *temporal_scalability,
	Int        readen_bits[MAX_NUM_VOS][MAX_NUM_VOLS],
	Int        stop_decoding_vol[MAX_NUM_VOS][MAX_NUM_VOLS],
	Vop		  *display_vop, 
	Float     *time_between_two_frames,
	Float     *next_display_time,
	Int		   random_access_start_time,
	Bitstream *stream[MAX_NUM_VOS][MAX_NUM_VOLS],
    Vol       *display_vol_list[MAX_NUM_VOS],
    Trace     *trace[MAX_NUM_VOS][MAX_NUM_VOLS],
	Char       trace_filename[MAX_NUM_VOS][MAX_NUM_VOLS][300],
	Int        start_time[MAX_NUM_VOLS][2],
	Int  *prev_ref,Int  *next_ref,
	UChar **MB_TYPE2[MAX_NUM_VOS][MAX_NUM_VOLS],
	Int PRE_H_SIZE2[MAX_NUM_VOS][MAX_NUM_VOLS],
	Int PRE_V_SIZE2[MAX_NUM_VOS][MAX_NUM_VOLS],
	UChar **PRE_MB_TYPE2[MAX_NUM_VOS][MAX_NUM_VOLS],
	Int bs_vol[MAX_NUM_VOS][MAX_NUM_VOLS],
	Image   *alpha_rec_packet,
	Image   *alpha_mode_packet,
	SInt Px_for,SInt Py_for, SInt Px_back, SInt Py_back,
	Int prev_j,Int zero_edge,Int prev_time_base,
	UChar **stock_mb_type2[3][MAX_NUM_VOS][MAX_NUM_VOLS],
	Int   stock_h_size2[3][MAX_NUM_VOS][MAX_NUM_VOLS],
	Int   stock_v_size2[3][MAX_NUM_VOS][MAX_NUM_VOLS]
)
{
int k;

  for(k=0; k<vols; k++)
	DecodeVolHeaderList(vols,stream[0],trace[0],
			display_vol_list[0],readen_bits[0],0,*temporal_scalability,
			prev_ref,next_ref,MB_TYPE2,PRE_H_SIZE2,PRE_V_SIZE2,PRE_MB_TYPE2,bs_vol,
			alpha_rec_packet,alpha_mode_packet,
			Px_for,Py_for,Px_back,Py_back,prev_j,zero_edge,prev_time_base,
			stock_mb_type2,stock_h_size2,stock_v_size2);
  /*if random_access, find time_between_govs*/
  if (random_access_start_time != 0){
    *next_display_time = (Float) random_access_start_time;
    FindStartGOV(vols,stream[0],trace[0],display_vol_list[0],
		         readen_bits[0],random_access_start_time);
  }
#if(__TEMPORAL_SCALABILITY__)
  Check_TPS(vols, stream[0], trace[0],
		    random_access_start_time,  display_vol_list[0],*temporal_scalability,
			start_time);
#endif
}


/***********************************************************CommentBegin******
 *
 * -- DecodeCDMPEG4Vop -- Decodes the VOP information from the bitstream
 *
 ***********************************************************CommentEnd********/

APADLLEXPORTS Void CallingConvention DecodeMPEG4CDVol(
	Int        width,
	Int        height, 
	Int        vols,
	Int        framerate, 
	Int        post_filter_type,
	Int        temporal_scalability,
	Int        random_access_start_time,
	Int        readen_bits[MAX_NUM_VOS][MAX_NUM_VOLS],
	Int        stop_decoding_vol[MAX_NUM_VOS][MAX_NUM_VOLS],
	Vop       *display_vop, 
	Float      time_between_two_frames,
	Float     *next_display_time,
	Bitstream *stream[MAX_NUM_VOS][MAX_NUM_VOLS],
    Vol       *display_vol_list[MAX_NUM_VOS],
    Trace     *trace[MAX_NUM_VOS][MAX_NUM_VOLS], 
	Char       trace_filename[MAX_NUM_VOS][MAX_NUM_VOLS][300],
	Vop	      *base_output_vop[3],
	Float      display_framerate_TPS,
	Int last_base_decoding_frame[3],
	Int *last_enh_decoding_frame,
	Int *TPS_base_count,
	Int    *prev_ref,
	Int *next_ref,
	UChar **MB_TYPE2[MAX_NUM_VOS][MAX_NUM_VOLS],
	Int PRE_H_SIZE2[MAX_NUM_VOS][MAX_NUM_VOLS],
	Int PRE_V_SIZE2[MAX_NUM_VOS][MAX_NUM_VOLS],
	UChar **PRE_MB_TYPE2[MAX_NUM_VOS][MAX_NUM_VOLS],
	Int bs_vol[MAX_NUM_VOS][MAX_NUM_VOLS],
	Image   *alpha_rec_packet,
	Image   *alpha_mode_packet,
	SInt Px_for,SInt Py_for, SInt Px_back, SInt Py_back,
	Int prev_j,Int zero_edge,Int prev_time_base,
	Int prev_predic_type,
	Vop	*back_shape_vop,
	Vop	*for_shape_vop,
	UChar **stock_mb_type2[3][MAX_NUM_VOS][MAX_NUM_VOLS],
	Int   stock_h_size2[3][MAX_NUM_VOS][MAX_NUM_VOLS],
	Int   stock_v_size2[3][MAX_NUM_VOS][MAX_NUM_VOLS]
	)
{	  
	Int image_width;
	Int image_height;

#if defined(_IM1_CD_DECODER_)
	BeginMutex();
#endif  

	image_width=width;
	image_height=height;

  while(!StopDecoding1VO(vols,stop_decoding_vol))
    {
#if(__TEMPORAL_SCALABILITY__)
    if (temporal_scalability)
      fprintf(stderr,"===( next [%d] )===\n",(Int)*next_display_time);
#endif

    DecodeVolList(vols,stream[0],trace[0],
		    (Float)*next_display_time, random_access_start_time,
		    display_vol_list[0],readen_bits[0],stop_decoding_vol[0],
			0/*VOs*/,post_filter_type,base_output_vop,
			display_framerate_TPS,temporal_scalability,
			last_base_decoding_frame,last_enh_decoding_frame,TPS_base_count,
			prev_ref,next_ref,MB_TYPE2,PRE_H_SIZE2,PRE_V_SIZE2,PRE_MB_TYPE2,
			bs_vol,alpha_rec_packet,alpha_mode_packet,
			Px_for,Py_for,Px_back,Py_back,prev_j,zero_edge,prev_time_base,prev_predic_type,
			image_width,image_height,back_shape_vop,for_shape_vop,
			stock_mb_type2,stock_h_size2,stock_v_size2);

#ifdef _DEBUG
    fprintf(stderr, "======================================\n");
    fprintf(stderr, "{curr_time}:%f ->" , *next_display_time );
#endif
    *next_display_time += time_between_two_frames;    
#ifdef _DEBUG
    fprintf(stderr,"{next_time} = %f\n" , *next_display_time );
#endif
    }

#if defined(_IM1_CD_DECODER_)
	EndMutex();
#endif
}

									  
									  
APADLLEXPORTS Void CallingConvention FreeMPEG4CD(
		Vop		*display_vop,
		Int		 vols,
		Int		 temporal_scalability,
		Int		num_frames,
		Vol		*display_vol_list[MAX_NUM_VOS],
		Bitstream *stream[MAX_NUM_VOS][MAX_NUM_VOLS],
		UChar **PRE_MB_TYPE2[MAX_NUM_VOS][MAX_NUM_VOLS],
		Int PRE_V_SIZE2[MAX_NUM_VOS][MAX_NUM_VOLS],
		Int		num_cd_decoders,
		Vop		*back_shape_vop,
		Vop		*for_shape_vop,
		UChar **stock_mb_type2[3][MAX_NUM_VOS][MAX_NUM_VOLS],
		Int   stock_v_size2[3][MAX_NUM_VOS][MAX_NUM_VOLS]
		)
{
Int k,i;

#if(__TEMPORAL_SCALABILITY__)
  FreeAllPRE_PRE_MBtype2(stock_mb_type2,stock_v_size2
); 
#endif
  FreeVop(display_vop);  
  FreeVolList(display_vol_list[0]);
    
  for(k=0; k<vols; k++)
      BitstreamClose(stream[0][k]);

  FreeAllPRE_MBtype2(PRE_V_SIZE2,PRE_MB_TYPE2);
    
    i = stop_chrono( &(chrono[num_cd_decoders-1]) );
    
#if 0
    printf("\n\n\n");
    printf("Start   : %d \n", chrono[num_cd_decoders-1].start);
    printf("Stop    : %d \n", chrono[num_cd_decoders-1].stop);
    printf("Runtime : %d \n", chrono[num_cd_decoders-1].runtime);
    
    printf("\n\n\n");
#endif
    
    i = display_chrono(stdout, chrono[num_cd_decoders-1].runtime, num_frames);

#if(__TEMPORAL_SCALABILITY__)
  for (i=0;i<3;i++) 
    if (base_output_vop[i]!=NULL) FreeVop(base_output_vop[i]);

  if (temporal_scalability) FreeShapeVop(&back_shape_vop,&for_shape_vop); 
#endif

    if (num_cd_decoders==1)
		free_ring();
}



#endif /*_IM1_CD_DECODER_*/

#endif
