/***************************************************************
  COPYRIGHT (1996)  DAVID SARNOFF RESEARCH CENTER, INC.
  ALL RIGHTS RESERVED
  SARNOFF PROPRIETARY INFORMATION
  PERMISSION IS GRANTED FOR USE WITHIN MPEG4 STANDARDIZATION PROCESS
  ****************************************************************/

/*****************************************************************************/
/*         This software module has been speed optimized                     */
/*                                                                           */
/*         by  Bruno Loret - France Telecom / CNET                           */
/*                                                                           */
/*         in the framework of the ACTS Project MoMuSys                      */
/*****************************************************************************/

#include <math.h>
#include <malloc.h>
#include <stdio.h>
#include "rc_matrix.h"

/*****/
Double **
CallingConvention
transpose_matrix(Double **src, Int nrow, Int ncol)
{
    Int i, j;
    Double **dest;
    
    dest=matrix(1,ncol,1,nrow);
    for (i = 1; i <= nrow; i++)
        for (j = 1; j <= ncol; j++)
            dest[j][i] = src[i][j];
    return(dest);
}

/*****/
Void
CallingConvention
print_matrix(File *file, Double **matrix, Int nrow, Int ncol)
{
    Int i, j;
    for (i = 1; i <= nrow; i++)
    {
        for (j = 1; j <= ncol; j++)
            fprintf(stderr, "matrix[%d][%d]= %f\n", i, j, matrix[i][j]);
        fprintf(stderr, "\n");
    }
    
}
/*****/
Double **
CallingConvention
multiply_matrix(Double **ma, Double **mb, Int m, Int n, Int p)
{
    Int i, j, k;
    Double **mab;
    
    mab=matrix(1,m,1,p);
    for (i = 1; i <= m; i++)
        for (j = 1; j <= p; j++) {
            mab[i][j] = 0.0;
            for (k = 1; k <= n; k++)
                mab[i][j] += (ma[i][k] * mb[k][j]);
        }
    return(mab);
}

/*****/
Double **
CallingConvention
inverse(Double **a, Int N)
{
    Double **y;
    Double det;
    
    y=matrix(1,N,1,N);
    det = a[1][1]*a[2][2] - a[2][1]*a[1][2];
    /***/
    if (det==0)
    {
        fprintf(stderr, "*****inverse: det=%f\n", det);
        exit(1);
    }
    y[2][2] = a[1][1]/det; 
    y[1][1] = a[2][2]/det;
    y[1][2] = -a[1][2]/det;
    y[2][1] = -a[2][1]/det;
    return (y);
    
}

/*****/
Double **
CallingConvention
matrix(Int nrl, Int nrh, Int ncl, Int nch)
{
    Int i, j;
    Double **m;
    
    m = (Double **) malloc((U_Int) (nrh - nrl + 2) * sizeof(Double *));
    if (!m)
        nrerror("allocation failure 1 in matrix()");
    m -= nrl;
    m++;
    
    m[nrl-1]=(Double *) malloc((2 * sizeof(Double)));
    m[nrl-1][0]= (Double) (nrh - nrl + 1);
    m[nrl-1][1]= (Double) (nch - ncl + 1);
    
    for (i = nrl; i <= nrh; i++) {
        m[i] = (Double *) malloc((U_Int) (nch - ncl + 1) * sizeof(Double));
        if (!m[i])
            nrerror("allocation failure 2 in matrix()");
        m[i] -= ncl;
    }
    
    for (i = nrl; i <= nrh; i++)
        for (j = ncl; j <= nch; j++)
            m[i][j] = 0.0;
    
    return(m);
}


/*****/
Void
CallingConvention
free_matrix(Double **m, Int nrl, Int nrh, Int ncl, Int nch)
{
    Int i;
    
    if((m[nrl-1][0] != (Double)(nrh - nrl + 1)) ||
       (m[nrl-1][1] != (Double)(nch - ncl + 1)))
    {
        fprintf(stderr,"original h=%d w=%d release h=%d w=%d\n",
                (Int)m[nrl-1][0],(Int)m[ncl-1][1], 
                nrh - nrl + 1, nch - ncl + 1);
        perror("Inconsistency");
    }
    m[nrl-1][1]=m[nrl-1][0]=0;
    free((Double *) (m[nrl-1]));
    
    for (i = nrh; i >= nrl; i--)
        free((Double *) (m[i] + ncl)), m[i]=NULL;
    free((Double *) (m + nrl-1)), m=NULL;
}

/*****/
Void
CallingConvention
nrerror(Char *error_text)
{
    fprintf(stderr, "Numerical Recipes run-time error...\n");
    fprintf(stderr, "%s\n", error_text);
    fprintf(stderr, "...now exiting to system...\n");
    exit(1);
}

/*****/
Double *
CallingConvention
dvector(nl,nh)
Int nl,nh;
{
    Double *v;
    
    v=(Double *)malloc((U_Int) (nh-nl+1)*sizeof(Double));
    if (!v) nrerror("allocation failure in dvector()");
    return v-nl;
}

/*****/
Int *
CallingConvention
ivector(nl,nh)
Int nl,nh;
{
    Int *v;
    
    v=(Int *)malloc((U_Int) (nh-nl+1)*sizeof(Int));
    if (!v) nrerror("allocation failure in ivector()");
    return v-nl;
}

/*****/
Double **
CallingConvention
dmatrix(nrl,nrh,ncl,nch)
Int nrl,nrh,ncl,nch;
{
    Int i;
    Double **m;
    
    m=(Double **) malloc((U_Int) (nrh-nrl+1)*sizeof(Double*));
    if (!m) nrerror("allocation failure 1 in dmatrix()");
    m -= nrl;
    
    for(i=nrl;i<=nrh;i++) {
        m[i]=(Double *) malloc((unsigned) (nch-ncl+1)*sizeof(Double));
        if (!m[i]) nrerror("allocation failure 2 in dmatrix()");
        m[i] -= ncl;
    }
    return m;
}

/*****/
Void
CallingConvention
lubksb(a,n,indx,b)
Double **a,b[];
Int n,*indx;
{
    Int i,ii=0,ip,j;
    Double sum;
    
    for (i=1;i<=n;i++) {
        ip=indx[i];
        sum=b[ip];
        b[ip]=b[i];
        if (ii)
            for (j=ii;j<=i-1;j++) sum -= a[i][j]*b[j];
        else if (sum) ii=i;
        b[i]=sum;
    }
    for (i=n;i>=1;i--) {
        sum=b[i];
        for (j=i+1;j<=n;j++) sum -= a[i][j]*b[j];
        b[i]=sum/a[i][i];
    }
}

/*****/
Void
CallingConvention
free_dvector(v,nl,nh)
Double *v;
Int nl,nh;
{
    free((Char*) (v+nl));
}

Void
CallingConvention
free_ivector(v,nl,nh)
Int *v,nl,nh;
{
    free((Char*) (v+nl));
}


#define TINY 1.0e-20;

/*****/
Void
CallingConvention
ludcmp(a,n,indx,d)
Int n,*indx;
Double **a,*d;
{
    Int i,imax=0,j,k;
    Double big,dum,sum,temp;
    Double *vv;
    Double * CallingConvention dvector();
    Void CallingConvention nrerror();
    Void CallingConvention free_dvector();
    
    vv=dvector(1,n);
    *d=1.0;
    for (i=1;i<=n;i++)
    {
        big=0.0;
        for (j=1;j<=n;j++)
            if ((temp=fabs(a[i][j])) > big) big=temp; 
        if (big == 0.0) nrerror("Singular matrix in routine LUDCMP");
        vv[i]=1.0/big;
    }
    
    for (j=1;j<=n;j++) 
    {
        for (i=1;i<j;i++) 
        {
            sum=a[i][j];
            for (k=1;k<i;k++) sum -= a[i][k]*a[k][j];
            a[i][j]=sum;
        }
        big=0.0;
        for (i=j;i<=n;i++) 
        {
            sum=a[i][j];
            for (k=1;k<j;k++)
            {
                sum -= a[i][k]*a[k][j];
            }
            a[i][j]=sum;
            if ( (dum=vv[i]*fabs(sum)) >= big) 
            {
                big=dum;
                imax=i;
            }
        }
        if (j != imax) 
        {
            for (k=1;k<=n;k++) 
            {
                dum=a[imax][k];
                a[imax][k]=a[j][k];
                a[j][k]=dum;
            }
            *d = -(*d);
            vv[imax]=vv[j];
        }
        indx[j]=imax;
        if (a[j][j] == 0.0) a[j][j]=TINY; 
        if (j != n) 
        {
            dum=1.0/(a[j][j]);
            for (i=j+1;i<=n;i++) 
            {
                a[i][j] *= dum;
            }
        }
    }
    free_dvector(vv,1,n);
    
}

#undef TINY


/*****/

Void
CallingConvention
invert_mat (Double **m, Double **m_inv, Int  n)
{
    Int i,j,*indx;
    Double      det, d, *col;
    
    indx = ivector(0,n);
    col  =  dvector(0,n);
    
    ludcmp (m, n, indx, &d); 
    
    det = 1.;
    for ( i=1; i<=n; i++) det = det * m[i][i];
    if (det == 0.) printf("\n Warning : Determinant is zero \n");
    
    for ( j=1; j<=n; j++) {
        for ( i=1; i<=n; i++) col[i] = 0.0;
        col[j] = 1.0;
        lubksb ( m, n, indx, col);
        for ( i=1; i<=n; i++) 
        {
            m_inv[i][j]=col[i];
        }
    }
    
    free_ivector( indx, 0, n);
    free_dvector (  col, 0, n);
    
}

