/*****************************************************************************
 *
 * This software module was originally developed by
 *
 *   Ryuji Kitaura  (Sharp Corporation)                                  
 *   Hisashi Saiga  (Sharp Corporation)                                  
 *   Tomoko Aono    (Sharp Corporation)                                  
 *   Hiroyuki Katata(Sharp Corporation)                                  
 *
 * and edited by
 * 
 *   Fernando Jaureguizar (UPM / ACTS-MoMuSyS)
 *
 * in the course of development of the MPEG-4 Video (ISO/IEC 14496-2) standard.
 * This software module is an implementation of a part of one or more MPEG-4
 * Video (ISO/IEC 14496-2) tools as specified by the MPEG-4 Video (ISO/IEC
 * 14496-2) standard.
 *
 * ISO/IEC gives users of the MPEG-4 Video (ISO/IEC 14496-2) standard free
 * license to this software module or modifications thereof for use in hardware
 * or software products claiming conformance to the MPEG-4 Video (ISO/IEC
 * 14496-2) standard.
 *
 * Those intending to use this software module in hardware or software products
 * are advised that its use may infringe existing patents. The original
 * developer of this software module and his/her company, the subsequent
 * editors and their companies, and ISO/IEC have no liability for use of this
 * software module or modifications thereof in an implementation. Copyright is
 * not released for non MPEG-4 Video (ISO/IEC 14496-2) Standard conforming
 * products.
 *
 * Sharp Corporation retains full right to use the code for his/her own
 * purpose, assign or donate the code to a third party and to inhibit third
 * parties from using the code for non MPEG-4 Video (ISO/IEC 14496-2) Standard
 * conforming products. This copyright notice must be included in all copies or
 * derivative works.
 *
 * Copyright (c) 1997
 *
 *****************************************************************************/

/* All functions in this file were written by
   Hisashi Saiga & Ryuji Kitaura of Sharp Corporation, 1997
   utilizing existing subroutines and data structures.
   */

/*****************************************************************************/
/*         This software module has been speed optimized                     */
/*                                                                           */
/*         by  Bruno Loret - France Telecom / CNET                           */
/*                                                                           */
/*         in the framework of the ACTS Project MoMuSys                      */
/*****************************************************************************/

/***********************************************************HeaderBegin*******
 *
 * File:   scal_dec.c 
 *
 * Author:   Hisashi Saiga & Ryuji Kitaura  (Sharp Corporation)
 * Created:  00-xxx-1997
 *
 * Description:
 *   Originally created by the authors utilizing existing subroutines
 *   and data structures.
 *   The original file tpsdec.c had the modules necessary for decoding
 *   an MPEG-4 video VM7.0 bitstream with temporal scalability.
 *
 * Notes:
 *   Some parts of the original file have been modified to have in the same
 *   file the temporal and the spatial scalability.
 *   The main routine DecodeVopScalable() was created by Michael Wollborn (TUH)
 *   on 12-AUG-1996 as an empty function.
 *   This function was filled by two companies outside the ACTS-MoMuSys
 *   consortium; Sharp (temporal scalability) and Sony (spatial scalability
 *   integrated into MoMuSyS VM6 version on 26.May.97),
 *   but using the same function name.
 *   F. Jaureguizar from (UMC / ACTS-MoMuSyS) has integrated and modified both
 *   scalabilities in this DecodeVopScalable() function.
 *
 * Modified :
 *   30.07.97 F. Jaureguizar: integration of spatial scalability (SpSc)
 *            Some format modification to make the code more readeable.
 *   29.08.97 Osamu Sunohara(Sony): modified for spatial scalable coding
 *   05.09.97 F. Jaureguizar: deleted unnecessary "base_vop = SallocVop();"
 *            line in DecodeVopScalable()
 *
 *      23.10.97 Minhua Zhou: changed deblock_filter_disable to 
 *                            post_filter_type
 ***********************************************************HeaderEnd*********/

#include <string.h>
#include "momusys.h"
#include "mom_bitstream_d.h"
#include "vm_compos.h"
#include "text_decode.h"
#include "mot_padding.h"
#include "io_generic.h"
#include "vm_dec_main.h"
#include "mom_vol.h"
#include "mom_access.h"
#include "mot_comp.h"
#include "zigzag.h"
#include "post_filter.h"
#include "mom_vo.h"
#include "combined_decode.h"
#include "vm_common_defs.h"
#include <limits.h>
/* #include "tpsdec.h"*/ /*SpSc*/
#include "scal_dec.h"    /*SpSc*/
#include "vm_midproc.h"  /*SpSc*/
/* changed 6/16 */
#include "do_bgc.h"
#include "alp_dec_mom.h"
#include "alp_dec_header.h"
/**/

#ifdef _DEBUG_SPSC2_
/*ok*/static tmp_cnt = 0;
#endif

/*A.Pacheco: global variables are not allowed*/
#if 0
/*** 10/27 for TPS */
/*ok*/Int prev_ref= -1;
/*ok*/Int next_ref= -1;
/* 10/27 for TPS ***/
#endif
/***********************************************************CommentBegin******
 *
 * -- DecodeVopScalable -- Decodes the VOP information from the bitstream
 *    in scalable mode
 *
 * Author :
 *    Michael Wollborn (TUH)
 *
 * Created :
 *    12-AUG-1996
 *
 * Purpose :
 *   Decode the vop information from the bitstream in scalable mode
 *
 * Arguments in :
 *   Bitstream *stream   pointer on the decoder bitstream structure,
 *                       should be at the right position.
 *   Trace *trace        pointer to trace information
 *
 * Arguments in/out :
 *   Vop *curr_vop   pointer on considered vop - the previous VOP
 *                   can be accessed by the structure of the current
 *                   VOP (GetVopNextTemp())
 *
 * Arguments out :
 *
 *
 * Return values :
 *
 *
 * Side effects :
 *
 *
 * Description :
 *   It is supposed the following returns in case of Spatial Scalability:
 *      GetVopRefSelCode(curr_vop)  = 3 for P_VOPs and 0 for B_VOPs.
 *      GetVopBackCom(curr_vop)     = 0 -- No Background composition
 *      GetVopEnhanceType(curr_vop) = 0 -- All base frame is enhanced
 *
 * See also :
 *
 *
 * Modified :
 *   00.??.97 Sharp Corp.: implementation of the temporal scalability.
 *   30.07.97 F. Jaureguizar: integration of the spatial scalability.
 *   29.08.97 Osamu Sunohara: modified for spatial scalable coding
 *   05.09.97 F. Jaureguizar: deleted unnecessary "base_vop = SallocVop();"
 *            line in DecodeVopScalable()
 *
 ***********************************************************CommentEnd********/

/* changed 6/16 */
Void
CallingConvention
DecodeVopScalable(
    Bitstream **stream ,
    Int vo_id ,
    Trace *trace,
    Vop *curr_vop ,
    Vol *vol_list ,
    Int next_display_time,
    Int post_filter_type,
	Vop	      *base_output_vop[3],
	Int        temporal_scalability,
	Int     *prev_ref,
	Int		*next_ref,
	UChar **MB_TYPE2[MAX_NUM_VOS][MAX_NUM_VOLS],
	Int PRE_H_SIZE2[MAX_NUM_VOS][MAX_NUM_VOLS],
	Int PRE_V_SIZE2[MAX_NUM_VOS][MAX_NUM_VOLS],
	UChar **PRE_MB_TYPE2[MAX_NUM_VOS][MAX_NUM_VOLS],
	Int bs_vol[MAX_NUM_VOS][MAX_NUM_VOLS],
	Image   *alpha_rec_packet,
	Image   *alpha_mode_packet,
	SInt Px_for,SInt Py_for, SInt Px_back, SInt Py_back,
	Int prev_j,
	Int zero_edge,
	Int image_width,
	Int image_height,
	Vop	**back_shape_vop,
	Vop	**for_shape_vop,
	UChar **stock_mb_type2[3][MAX_NUM_VOS][MAX_NUM_VOLS],
	Int   stock_h_size2[3][MAX_NUM_VOS][MAX_NUM_VOLS],
	Int   stock_v_size2[3][MAX_NUM_VOS][MAX_NUM_VOLS]
	)
/**/
{
    Vol        *ref_vol;
    Int        tmpvar;
    
/*** 8/18 TPS */
/*  Int        offset0;
    Bitstream  *ref_stream;*/
/*** 8/18 TPS */
    
    Int        vop_exist[ 2 ];
    Vop        *vop_list[ 2 ];
    Int        vop_display_time[ 2 ];
    /* changed 6/17 */
    Int        vol_id, nvol_id;
    Int        ref_vol_id, nref_vol_id;
    /**/
    Int        *mvda;
    Vop        *file_vop;
    /*>SpSc*/
    Int        predic_type  = GetVopPredictionType(curr_vop);
    Int        ref_sel_code;
    Vop        *up_vop=NULL;
    Vop        *base_vop=NULL;
    Int        spatial_scalability = 0;
    Int        aux_width;
    Int        aux_height;
    Int        aux_hor_spat_ref;
    Int        aux_ver_spat_ref;
    
    
    /*SpSc<*/
#ifdef _DEBUG_SPSC_
/*ok*/    static Int count=0;
    Char       name[81];
#endif
    
/*** 10/27 for TPS changed by kitaura */
    Vop   *ref_vop0 = NULL;
    Vop   *ref_vop1 = NULL;
    Vop   *tmp_vop = NULL;
/* 10/27 for TPS ***/
    
    /* changed 6/17 */
    nvol_id = GetVopId( curr_vop );
    vol_id = bs_vol[vo_id][nvol_id];
    /*****
     *
     * Free image channels of VOP and allocate the image channels
     * for the new sizes (only necessary if arbitrary shape)
     *
     *****/
    if(GetVopShape(curr_vop))
    {
        FreeVopChannels(curr_vop);
        AllocVopChannels(curr_vop,GetVopWidth(curr_vop),GetVopHeight(curr_vop));
    }
    
    
    if ( GetVopBackComp( curr_vop ) )
    {
        tmpvar = (Int) BitstreamReadBits( stream[ vol_id ] , 1 ,
                                          "load_backward_shape" , trace , CODE );
        PutVopLoadBackShape( tmpvar , curr_vop );
        if ( GetVopLoadBackShape( curr_vop ) )
        {
            Int   back_shape_width;
            Int   back_shape_height;
            Int   back_shape_hor_spat_ref;
            Int   back_shape_ver_spat_ref;
            Vop   *dup_back_shape_vop;
            Int   *for_mb_type;
            Int   *back_mb_type;
            
            back_shape_width = ( Int )BitstreamReadBits( stream[ vol_id ] , 13 ,
                                                         "back_shape_width" , trace , NUM );
            back_shape_height = ( Int )BitstreamReadBits( stream[ vol_id ] , 13 ,
                                                          "back_shape_height" , trace , NUM );
            back_shape_hor_spat_ref = (Int)BitstreamReadBits( stream[ vol_id ], 13 ,
                                                              "back_shape_hor_spat_ref" , trace , NUM );
            tmpvar = (Int) BitstreamReadBits( stream[ vol_id ] , 1 ,
                                              "marker_bit" , trace , FLAG );
            back_shape_ver_spat_ref = (Int)BitstreamReadBits( stream[ vol_id ], 13 ,
                                                              "back_shape_ver_spat_ref" , trace , NUM );
            if ( *back_shape_vop != ( Vop * )NULL )
            {
                dup_back_shape_vop = CloneVop( *back_shape_vop );
                FreeVop( *back_shape_vop );
            }
            else
                dup_back_shape_vop = ( Vop * )NULL;
            if ( back_shape_width ==0 )
            {
                *back_shape_vop = AllocVop( GetVopWidth( curr_vop ) ,
                                           GetVopHeight( curr_vop ) );
                
/*** 8/18 TPS */
                PutVopChangeCRDisable(GetVopChangeCRDisable(curr_vop), *back_shape_vop);
                PutVopShapeEffects(GetVopShapeEffects(curr_vop), *back_shape_vop);
/* 8/18 TPS ***/
                
                ZeroImageI( GetVopA (*back_shape_vop) );
                PutVopHorSpatRef( back_shape_hor_spat_ref , *back_shape_vop );
                PutVopVerSpatRef( back_shape_ver_spat_ref , *back_shape_vop );
            }
            else
            {
                *back_shape_vop = AllocVop( back_shape_width , back_shape_height );
                
/*** 8/18 TPS */
                PutVopChangeCRDisable(GetVopChangeCRDisable(curr_vop), *back_shape_vop);
                PutVopShapeEffects(GetVopShapeEffects(curr_vop), *back_shape_vop);
/* 8/18 TPS ***/
                
                PutVopHorSpatRef( back_shape_hor_spat_ref , *back_shape_vop );
                PutVopVerSpatRef( back_shape_ver_spat_ref , *back_shape_vop );
                back_mb_type = ( Int * )calloc( GetVopWidth( *back_shape_vop ) /
                                                MB_SIZE * GetVopHeight( *back_shape_vop ) / MB_SIZE , sizeof(Int) );
                mvda = ( Int * )calloc( GetVopWidth( *back_shape_vop ) /
                                        MB_SIZE * GetVopHeight( *back_shape_vop ) / MB_SIZE , sizeof(Int) );
                PutVopPredictionType(INTRA_VOP,*back_shape_vop);
                
                /*** 8/18 TPS */
                PutVopShape(BINARY_SHAPE_ONLY, *back_shape_vop);
                
/*** 10/9 TPS */
                {
                    Int height = 0;
                    height = set_MB_TYPE2_type1_TPS(*back_shape_vop, vo_id, vol_id,MB_TYPE2);
                    DecodeVopCombinedShapeTextureIntra(stream[ vol_id ], vo_id, trace, *back_shape_vop,
														MB_TYPE2,PRE_H_SIZE2,PRE_V_SIZE2,PRE_MB_TYPE2,
														alpha_rec_packet,alpha_mode_packet);
                    if(MB_TYPE2[ vo_id ][ vol_id ] != NULL)
                        FreeMBtype2(height, vo_id, vol_id,MB_TYPE2);
                }    
/* 10/9 TPS ***/
                
                free( back_mb_type );
                free( mvda );
            }
            tmpvar = (Int) BitstreamReadBits( stream[ vol_id ] , 1 ,
                                              "load_forward_shape" , trace , CODE );
            PutVopLoadForShape( tmpvar , curr_vop );
            if ( GetVopLoadForShape( curr_vop ) )
            {
                Int   for_shape_width;
                Int   for_shape_height;
                Int   for_shape_hor_spat_ref;
                Int   for_shape_ver_spat_ref;
                
                for_shape_width = ( Int )BitstreamReadBits( stream[ vol_id ] , 13 ,
                                                            "for_shape_width" , trace , NUM );
                for_shape_height = ( Int )BitstreamReadBits( stream[ vol_id ] , 13 ,
                                                             "for_shape_height" , trace , NUM );
                for_shape_hor_spat_ref = (Int)BitstreamReadBits( stream[vol_id], 13 ,
                                                                 "for_shape_hor_spat_ref" , trace , NUM );
                tmpvar = (Int) BitstreamReadBits( stream[ vol_id ] , 1 ,
                                                  "marker_bit" , trace , FLAG );
                for_shape_ver_spat_ref = (Int)BitstreamReadBits( stream[vol_id], 13 ,
                                                                 "for_shape_ver_spat_ref" , trace , NUM );
                if ( *for_shape_vop != ( Vop * )NULL )
                    FreeVop( *for_shape_vop );
                if ( for_shape_width == 0 )
                {
                    *for_shape_vop = AllocVop( GetVopWidth( curr_vop ),
                                              GetVopHeight( curr_vop ) );
                    
/*** 8/18 TPS */
                    PutVopChangeCRDisable(GetVopChangeCRDisable(curr_vop), *for_shape_vop);
                    PutVopShapeEffects(GetVopShapeEffects(curr_vop), *for_shape_vop);
/* 8/18 TPS ***/
                    
                    ZeroImageI( GetVopA( *for_shape_vop ) );
                    PutVopHorSpatRef( for_shape_hor_spat_ref , *for_shape_vop );
                    PutVopVerSpatRef( for_shape_ver_spat_ref , *for_shape_vop );
                }
                else
                {
                    *for_shape_vop = AllocVop( for_shape_width , for_shape_height );
                    
/*** 8/18 TPS */
                    PutVopChangeCRDisable(GetVopChangeCRDisable(curr_vop), *for_shape_vop);
                    PutVopShapeEffects(GetVopShapeEffects(curr_vop), *for_shape_vop);
/* 8/18 TPS ***/
                    
                    PutVopHorSpatRef( for_shape_hor_spat_ref , *for_shape_vop );
                    PutVopVerSpatRef( for_shape_ver_spat_ref , *for_shape_vop );
                    for_mb_type = ( Int * )calloc( GetVopWidth( *for_shape_vop ) /
                                                   MB_SIZE * GetVopHeight(*for_shape_vop) / MB_SIZE , sizeof(Int) );
                    mvda = ( Int * )calloc( GetVopWidth( *for_shape_vop ) /
                                            MB_SIZE * GetVopHeight(*for_shape_vop) / MB_SIZE , sizeof(Int) );
                    PutVopPredictionType(INTRA_VOP,*for_shape_vop);
                    
                    /*** 8/18 TPS */
                    PutVopShape(BINARY_SHAPE_ONLY, *for_shape_vop);
/*** 10/9 TPS */
                    {
                        Int height = 0;
                        height = set_MB_TYPE2_type1_TPS(*for_shape_vop, vo_id, vol_id,MB_TYPE2);
                        DecodeVopCombinedShapeTextureIntra(stream[ vol_id ], vo_id, trace, *for_shape_vop,
															MB_TYPE2,PRE_H_SIZE2,PRE_V_SIZE2,PRE_MB_TYPE2,
															alpha_rec_packet,alpha_mode_packet);
                        if(MB_TYPE2[ vo_id ][ vol_id ] != NULL)
                            FreeMBtype2(height, vo_id, vol_id,MB_TYPE2);
                    }    
/* 10/9 TPS ***/
                    free( for_mb_type );
                    free ( mvda );
                }
            }
            else
            {
                if ( *for_shape_vop != ( Vop * )NULL )
                    FreeVop( *for_shape_vop );
                if ( dup_back_shape_vop != ( Vop * )NULL )
                    *for_shape_vop = CloneVop( dup_back_shape_vop );
            }
            if ( dup_back_shape_vop != ( Vop * )NULL )
                FreeVop( dup_back_shape_vop );
        }
    }
    else  /* No Background Composition */
    {
        vop_list[ 0 ] = ( Vop * )NULL;
        vop_list[ 1 ] = ( Vop * )NULL;
        vop_exist[ 0 ] = vop_exist[ 1 ] = 0;
    }
    
    tmpvar = ( Int )BitstreamReadBits( stream[ vol_id ] , 2 ,
                                       "ref_select_code" , trace , CODE );
    PutVopRefSelCode( tmpvar , curr_vop );
    
/*** 10/31 for TPS */
/*  if ( ( GetVopPredictionType( curr_vop ) == B_VOP ) ||
    ( GetVopPredictionType( curr_vop ) == P_VOP ) )
    {
    Int bits;
    if (0)
    bits = GetRefTimeBits_TPS(curr_vop); 
    else
    bits=10;
    
    tmpvar = ( Int )BitstreamReadBits( stream[ vol_id ] , bits ,
    "forward_temporal_ref" , trace , NUM );
    PutVopForTempRef( tmpvar , curr_vop );
    if ( GetVopPredictionType( curr_vop ) == B_VOP )
    {
    tmpvar = ( Int )BitstreamReadBits( stream[ vol_id ] , 1 ,
    "marker_bit" , trace , FLAG );
    tmpvar = ( Int )BitstreamReadBits( stream[ vol_id ] , bits ,
    "backward_temporal_ref" , trace , NUM );
    PutVopBackTempRef( tmpvar , curr_vop ); 
    } 
    } */
/* 10/31 for TPS ***/
    
    nref_vol_id = GetVopRefId( curr_vop );
    ref_vol_id = bs_vol[vo_id][nref_vol_id];
    ref_vol = GetVolById( vol_list , nref_vol_id );
    
    /*>SpSc*/
    ref_sel_code = GetVopRefSelCode(curr_vop);
    
    
    if ( (predic_type == P_VOP  &&  ref_sel_code == 3) ||
         (predic_type == B_VOP  &&  ref_sel_code == 0) )
        spatial_scalability = 1;
    
    if (spatial_scalability )
    {
#ifdef _DEBUG_SPSC_
        PrintVopInfo(curr_vop,"currA.1S");
        PrintVopInfo(GetVopNextTemp(curr_vop),"currA.1S->N");
        PrintVopInfo(GetVopPrevTemp(curr_vop),"currA.1S->P");
        sprintf(name,"A.1S%d",count);
        WriteVopDebug(curr_vop,name,name,name,0);
        
        PrintVopInfo(GetVolVop(ref_vol),"currA.1S-ref");
        PrintVopInfo(GetVopNextTemp(GetVolVop(ref_vol)),"currA.1S-ref->N");
        PrintVopInfo(GetVopPrevTemp(GetVolVop(ref_vol)),"currA.1S-ref->P");
        sprintf(name,"A.1S-ref%d",count);
        WriteVopDebug(GetVolVop(ref_vol),name,name,name,0);
#endif
        
        if (predic_type == P_VOP)
        {
            /* modified by Sony 290897 */
            if( GetVopRefSelCode(curr_vop)==3 )
            {
                if( (GetVopModTimeBase(curr_vop)==
                     GetVopModTimeBase(GetVopPrevTemp(GetVolVop(ref_vol))))
                    &&(GetVopTimeInc(curr_vop)==
                       GetVopTimeInc(GetVopPrevTemp(GetVolVop(ref_vol)))) )
                {
                    base_vop = CloneVop_TMP(GetVopPrevTemp(GetVolVop(ref_vol)));
                }
                else if( (GetVopModTimeBase(curr_vop)==
                          GetVopModTimeBase(GetVopNextTemp(GetVolVop(ref_vol))))
                         &&(GetVopTimeInc(curr_vop)==
                            GetVopTimeInc(GetVopNextTemp(GetVolVop(ref_vol)))) )
                {
                    base_vop = CloneVop_TMP(GetVopNextTemp(GetVolVop(ref_vol)));
                }
                else
                    fprintf( stderr,
                             "No Base VOP which have the same Modulo_Time_Base and Time_Incriment!!\n");
            }
            else
                /* 290897 */
                base_vop = CloneVop_TMP(GetVopPrevTemp(GetVolVop(ref_vol)));
            
        }
        else
        {
            Int time_inc=(Int)GetVopTimeInc(curr_vop);
            Int tmp_time_inc=(Int)GetVopTimeInc(GetVopNextTemp(GetVolVop(ref_vol)));
            
            if(time_inc == tmp_time_inc)
            {
                base_vop = CloneVop_TMP(GetVopNextTemp(GetVolVop(ref_vol)));
                printf("Base VOP for upsampling in the NextTemp of the Ref. VOP\n");
            }
            else
            {
                tmp_time_inc=(Int)GetVopTimeInc(GetVopPrevTemp(GetVolVop(ref_vol)));
                if(time_inc == tmp_time_inc)
                {
                    base_vop = CloneVop_TMP(GetVopPrevTemp(GetVolVop(ref_vol)));
                    printf("Base VOP for upsampling in the PrevTemp of the Ref. VOP\n");
                }
                else
                {
                    printf("ERROR: Base VOP for upsampling not in the Ref. VOP\n");
                    exit(-1);
                }
            }
        }
        
#ifdef _DEBUG_SPSC_
        PrintVopInfo(base_vop,"base");
#endif
        
        up_vop = Midproc(base_vop);
        
#ifdef _DEBUG_SPSC_
        PrintVopInfo(up_vop,"up-0");
        sprintf(name,"Sup-0%d",count);
        WriteVopDebug(up_vop,name,"","",0);
#endif
        
        aux_width = GetVopWidth(up_vop);
        aux_height = GetVopHeight(up_vop);
        aux_hor_spat_ref = GetVopHorSpatRef(up_vop);
        aux_ver_spat_ref = GetVopVerSpatRef(up_vop);
        
        CopyVopNonImageField(GetVopPrevTemp(GetVolVop(ref_vol)),up_vop);
        PutVopWidth(aux_width, up_vop);
        PutVopHeight(aux_height,up_vop);
        PutVopHorSpatRef(aux_hor_spat_ref,up_vop);
        PutVopVerSpatRef(aux_ver_spat_ref,up_vop);
        
        FreeVop(GetVopNextTemp(up_vop));
        PutVopNextTemp(NULL,up_vop);
        
        FreeVop(GetVopPrevTemp(up_vop));
        PutVopPrevTemp(NULL,up_vop);
        
#ifdef _DEBUG_SPSC_
        PrintVopInfo(up_vop,"up-modif");
        sprintf(name,"Sup-modif%d",count);
        WriteVopDebug(up_vop,name,"","",0);
#endif
        
        VopPadding(up_vop,&zero_edge);
        
#ifdef _DEBUG_SPSC_
        PrintVopInfo(up_vop,"up-padd");
        sprintf(name,"Sup-padd%d",count);
        WriteVopDebug(up_vop,name,"","",0);
#endif
        
        if (predic_type == P_VOP)
        {
            FreeVop(GetVopNextTemp(curr_vop));
            PutVopNextTemp(NULL,curr_vop);
            
            FreeVop(GetVopPrevTemp(curr_vop));
            PutVopPrevTemp(NULL,curr_vop);
            
#ifdef _DEBUG_SPSC_
            PrintVopInfo(curr_vop,"currA.2-p-S");
            PrintVopInfo(GetVopNextTemp(curr_vop),"currA.2-p-S->N");
            PrintVopInfo(GetVopPrevTemp(curr_vop),"currA.2-p-S->P");
#endif
            
            PutVopNextTemp(up_vop,curr_vop);
        }
        else if (predic_type == B_VOP)
        {
            /* FreeVop(GetVopNextTemp(curr_vop));*/
            FreeVop(GetVopPrevTemp(curr_vop));
            PutVopPrevTemp(NULL,curr_vop);
            
#ifdef _DEBUG_SPSC_
            PrintVopInfo(curr_vop,"currA.2-b-S");
            PrintVopInfo(GetVopNextTemp(curr_vop),"currA.2-b-S->N");
            PrintVopInfo(GetVopPrevTemp(curr_vop),"currA.2-b-S->P");
#endif
            
            /*PutVopPrevTemp(CloneVop_TMP(GetVopNextTemp(curr_vop)),curr_vop);*/
            PutVopPrevTemp(GetVopNextTemp(curr_vop),curr_vop);
            PutVopNextTemp(up_vop,curr_vop);
        }
        
#ifdef _DEBUG_SPSC_
        PrintVopInfo(curr_vop,"currA.3S");
        PrintVopInfo(GetVopNextTemp(curr_vop),"currA.3S->N");
        PrintVopInfo(GetVopPrevTemp(curr_vop),"currA.3S->P");
        sprintf(name,"A.3S%d",count);
        WriteVopDebug(curr_vop,name,name,name,0);
        
        count++;
#endif
        
        DecodeVopSpatScalable(stream[ vol_id ], vo_id, trace, curr_vop,
								prev_ref,next_ref,MB_TYPE2,PRE_H_SIZE2,PRE_V_SIZE2,
								PRE_MB_TYPE2,bs_vol,alpha_rec_packet,alpha_mode_packet,
								Px_for,Py_for,Px_back,Py_back,prev_j,
								stock_mb_type2,stock_h_size2,stock_v_size2);
        
        FreeVop(base_vop);
        /* FreeVop(up_vop);*/
    }
    
    /*SpSc<*/
    
/*** 10/27 for TPS */
    search_vop_for_bgc(vop_list, vop_exist, curr_vop, vop_display_time,base_output_vop);
    *prev_ref=(Int)CheckTime((Float)(vop_display_time[ 0 ]/33.33));
    *next_ref=(Int)CheckTime((Float)(vop_display_time[ 1 ]/33.33));
    
    ref_sel_code=GetVopRefSelCode(curr_vop);
    if ((predic_type==P_VOP && ref_sel_code==1) /* for vop : prev.base vop */
        || (predic_type==B_VOP && (ref_sel_code==1 /* for vop : prev.enh vop and
                                                      back vop : prev.base vop */
                                   || ref_sel_code==3))) {                    /* for vop : prev.base vop and
                                                                                 back vop : next.base vop */
        /* The forward and backward prediction reference VOP belongs to the reference layer */
        
        if (vop_list[0]!=NULL)
            ref_vop0 = CloneVop(vop_list[0]);
        if (predic_type==B_VOP && vop_list[1]!=NULL)
            ref_vop1 = CloneVop(vop_list[1]);
        
        SetRefVop(ref_vop0, ref_vop1, curr_vop,zero_edge);
        
        if (predic_type==B_VOP && ref_sel_code==1) ;
        else 
            SetRefMB_TYPE2(curr_vop, vo_id, nvol_id, nref_vol_id, vop_exist[ 1 ],
							PRE_MB_TYPE2,PRE_H_SIZE2,PRE_V_SIZE2,
							stock_mb_type2,stock_h_size2,stock_v_size2);
    } /* End of predic_type == P_VOP && ref_select_code == 1 ... */
/* 10/27 for TPS ***/
    
/*** 10/27 for TPS comment out 
  if(GetVopEnhanceType( curr_vop )){
  if(last_base_decoding_frame[1] > last_enh_decoding_frame &&
  last_base_decoding_frame[0] != last_base_decoding_frame[1]){
  vop_exist[ 0 ]        = 1;
  vop_exist[ 1 ]        = 1;
  
  vop_list[ 0 ]         = CloneVop(GetVopPrevTemp(base_vop_not_padding_for_bgc));
  vop_display_time[ 0 ] = last_base_decoding_frame[0];
  
  vop_list[ 1 ]         = CloneVop(GetVopNextTemp(base_vop_not_padding_for_bgc));
  vop_display_time[ 1 ] = last_base_decoding_frame[1];
  }
  else{
  vop_exist[ 0 ]        = 1;
  vop_exist[ 1 ]        = 0;
  
  vop_list[ 0 ]         = CloneVop(GetVopNextTemp(base_vop_not_padding_for_bgc));
  vop_display_time[ 0 ] = last_base_decoding_frame[1];
  }
  }
  
  if ( GetVopRefSelCode( curr_vop ) == 1 )
  {
  Vop   *ref_vop = NULL;
  
  if(GetVopEnhanceType( curr_vop )){
  ref_vop = CloneVop(vop_list[ 0 ]);
  SetRefVop(ref_vop, curr_vop);
  }
  else{
  if(last_base_decoding_frame[1] > last_enh_decoding_frame &&
  last_base_decoding_frame[0] != last_base_decoding_frame[1]){
  vop_exist[ 1 ]        = 1;
  if(GetVopPrevTemp(base_vop_not_padding_for_bgc) != NULL)
  ref_vop = CloneVop(GetVopPrevTemp(base_vop_not_padding_for_bgc));
  }
  else{
  if(GetVopNextTemp(base_vop_not_padding_for_bgc) != NULL)
  ref_vop = CloneVop(GetVopNextTemp(base_vop_not_padding_for_bgc));
  }
  if(ref_vop != NULL)
  SetRefVop(ref_vop, curr_vop);
  }
  SetRefMB_TYPE2(curr_vop, vo_id, nvol_id, nref_vol_id, vop_exist[ 1 ]);
  }
/* 10/27 for  TPS comment out ***/
    
    if (!spatial_scalability) {/*SpSc*/
/*** 10/28 for TPS */
/*    if (GetVopShape(GetVopPrevTemp(curr_vop))!=BINARY_SHAPE)
      if (GetVopA(GetVopPrevTemp(curr_vop))==NULL) {
      Image *tmp_image;
      Vop *tmp_prev;
      
      tmp_prev=GetVopPrevTemp(curr_vop);
      tmp_image=AllocImage(GetVopWidth(tmp_prev),GetVopHeight(tmp_prev),SHORT_TYPE);
      PutVopA(tmp_image,tmp_prev);
      SetConstantImage(GetVopA(tmp_prev),255);
      }*/
/* 10/28 for TPS ***/
        
        DecodeVopNonScalable( stream[ vol_id ] , vo_id , trace , curr_vop,
							prev_ref,next_ref,MB_TYPE2,PRE_H_SIZE2,PRE_V_SIZE2,PRE_MB_TYPE2,
							bs_vol,alpha_rec_packet,alpha_mode_packet,
							Px_for,Py_for,Px_back,Py_back,prev_j,
							stock_mb_type2,stock_h_size2,stock_v_size2);
    }
/*** 10/27 for TPS */
    if (predic_type==B_VOP) {
        if (GetVopNextTemp(curr_vop)!=NULL) FreeVop(GetVopNextTemp(curr_vop));
        PutVopNextTemp(NULL,curr_vop);
        
        tmp_vop =CloneVop(curr_vop);
        if (GetVopPrevTemp(tmp_vop)!=NULL) FreeVop(GetVopPrevTemp(tmp_vop));
        PutVopPrevTemp(NULL,tmp_vop);  
        PutVopNextTemp(tmp_vop,curr_vop);
        VopPadding(GetVopNextTemp(curr_vop),&zero_edge);
    } 
/* 10/27 for TPS ***/
    if ( GetVopBackComp( curr_vop ) )
    {
        CopyAlpha( *for_shape_vop , vop_list[ 0 ] );
        if ( vop_exist[ 1 ] )
            CopyAlpha( *back_shape_vop , vop_list[ 1 ] );
        else
            CopyAlpha( *back_shape_vop , vop_list[ 0 ] );
    }
    
/* TEST */
/* printf("BGC--------------- %3d %3d %3d ----------------\n",(Int)(CalcFrameNumber( curr_vop )*33.33),vop_display_time[ 0 ],vop_display_time[ 1 ]); */
/* TEST */
    
    /* for type 2 */
    if ( GetVopBackComp( curr_vop ) == 0 && GetVopEnhanceType( curr_vop ) ==0 )
    {   
        
/*** 10/27 for TPS */
        file_vop = PostFilter_TPS(curr_vop, post_filter_type, image_width , image_height,zero_edge);
/* 10/27 for TPS ***/
        
    }
    /* for type 1 */
    else
    {
        if ( vop_exist[ 1 ] == 0 )
        {
            Vop   *local, *base0;
            
            file_vop = AllocVop( image_width , image_height );
            
            /* for deblock */
/*** 10/27 for TPS */
            local = PostFilter_TPS( curr_vop,      post_filter_type, image_width , image_height,zero_edge);
            base0 = PostFilter_TPS( vop_list[ 0 ], post_filter_type, image_width , image_height,zero_edge);
            PutVopShape( 1 , base0 ); 
/* 10/27 for TPS ***/
            
/*** 10/27 for TPS */
            do_bgc(local ,
                   base0 ,
                   base0 ,
                   file_vop ,
                   (Int)CheckTime((Float)(33.333*CalcFrameNumber( curr_vop ))) ,
                   /* 33.333* */ (Int)CheckTime((Float)(vop_display_time[ 0 ])),
                   /* 33.333* */ (Int)CheckTime((Float)(vop_display_time[ 0 ]))
                   );
/* 10/27 for TPS ***/
            
            FreeVop(local);
            FreeVop(base0);
        }
        else
        {
            Vop   *local, *base0, *base1;
            
            file_vop = AllocVop( image_width , image_height );
            
            /* for deblock */
/*** 10/27 for TPS */
            local = PostFilter_TPS( curr_vop,      post_filter_type, image_width , image_height,zero_edge);
            base0 = PostFilter_TPS( vop_list[ 0 ], post_filter_type, image_width , image_height,zero_edge);
            base1 = PostFilter_TPS( vop_list[ 1 ], post_filter_type, image_width , image_height,zero_edge);
            PutVopShape( 1 , base0 ); 
            PutVopShape( 1 , base1 ); 
/* 10/27 for TPS ***/
            
/*** 10/27 for TPS */
            do_bgc( local ,
                    base0 ,
                    base1 ,
                    file_vop ,
                    (Int)CheckTime((Float)(33.333*CalcFrameNumber( curr_vop ))) ,
                    (Int)CheckTime((Float)(vop_display_time[ 0 ])),
                    (Int)CheckTime((Float)(vop_display_time[ 1 ])));
/* 10/27 for TPS ***/
            
            FreeVop(local);
            FreeVop(base0);
            FreeVop(base1);
        }
    }
 
/*A.Pacheco:1/04/98: just for writting, static is not allowed*/
#if 0 
    if ( GetVopWidth( curr_vop ) != 0 )
    {
/*ok*/        static Int first = 1;
        
/*** 8/18 TPS */
        
/********** comment out 
  #ifdef SEPARATE_OUTPUT
  WriteVopSHARP( file_vop , comp_output_filename[ 0 ] ,
  comp_output_filename[ 1 ] , comp_output_filename[ 2 ] ,
  comp_output_filename[ 3 ] , comp_image_counter ,
  IO_FORMAT , IO_APPEND ,FALSE );
  #else 
  comment out **********/
        
        /*  if (first) */
        
/* modified for spatial scalable coding by Sony 290897 */
        if(spatial_scalability)
        {
/*ok*/            static Int spsc_count=0;
            if(spsc_count==0)
                WriteVopGeneric( file_vop , comp_output_filename[ 0 ] ,
                                 comp_output_filename[ 1 ] , comp_output_filename[ 2 ] ,
                                 comp_output_filename[ 3 ] , spsc_count,
                                 IO_FORMAT , IO_OVERWRITE ,FALSE );
            else
                WriteVopGeneric( file_vop , comp_output_filename[ 0 ] ,
                                 comp_output_filename[ 1 ] , comp_output_filename[ 2 ] ,
                                 comp_output_filename[ 3 ] , spsc_count,
                                 IO_FORMAT , IO_APPEND ,FALSE );
            spsc_count++;
        } else {
            if (!temporal_scalability && first)
                WriteVopGeneric( file_vop , comp_output_filename[ 0 ] ,
                                 comp_output_filename[ 1 ] , comp_output_filename[ 2 ] ,
                                 comp_output_filename[ 3 ] , CalcFrameNumber(curr_vop),
                                 IO_FORMAT , IO_OVERWRITE ,FALSE );
            else
                WriteVopGeneric( file_vop , comp_output_filename[ 0 ] ,
                                 comp_output_filename[ 1 ] , comp_output_filename[ 2 ] ,
                                 comp_output_filename[ 3 ] , CalcFrameNumber(curr_vop),
                                 IO_FORMAT , IO_APPEND ,FALSE );
        }
/* 290897 */
        
/*    if(temporal_scalability)
      fprintf(stderr," Write ENHANCE\n");*/
        
        if(temporal_scalability)
            fprintf(stderr,"[E]frame[%d] Write ENHANCE\n",CalcFrameNumber(curr_vop));
/*#endif*/
        
/* 8/18 TPS ***/
        
        first = 0;
    }
#endif
    FreeVop( file_vop );
    
/*** 8/18 TPS */
    /*comp_image_counter++;
      bgc_output = 1;*/
    
    if ( vop_exist[ 0 ] != 0 ){
        if ( GetVopNextTemp( vop_list[ 0 ] ) != ( Vop * )NULL )
            FreeVop(GetVopNextTemp( vop_list[ 0 ] ));
        if(vop_list[ 0 ] != ( Vop * )NULL)
            FreeVop( vop_list[ 0 ] );
    }
/*** 10/30 for TPS */
/*  if ( GetVopEnhanceType( curr_vop ) && vop_exist[ 1 ] ){ */
    if ( vop_exist[ 1 ] ){
/* 10/30 for TPS ***/
        if ( GetVopNextTemp( vop_list[ 1 ] ) != ( Vop * )NULL )
            FreeVop(GetVopNextTemp( vop_list[ 1 ] ));
        if(vop_list[ 1 ] != ( Vop * )NULL)
            FreeVop( vop_list[ 1 ] );
    }
/* 8/18 TPS ***/
    
    return;
}

/***********************************************************CommentBegin******
 *
 * -- GetBitstreamPosition -- Get the position of the next input on the file
 *                associated with the bitstream
 *
 * Author :
 *      Hisashi Saiga (SHARP)
 *
 * Created :
 *   25-FEB-1997
 *
 *
 * Purpose :
 *   Get the position of the next input on the file
 *   associated with the bitstream
 *
 *
 * Arguments in :
 *   int      offset0     offset
 *
 * Arguments in/out :
 *   long      *stream    the bitstream
 *
 *
 * Arguments out :
 *
 *
 * Return values :
 *
 *
 * Side effects :
 *
 *
 * Description :
 *
 *
 * See also :
 *
 *
 * Modified :
 *
 *
 ***********************************************************CommentEnd********/

Int
CallingConvention
GetBitstreamPosition(
    Int *offset0 ,
    Bitstream *stream0 )
{
    
#ifdef _IM1_CD_DECODER_  /*A.Pacheco:03/11/98*/
	*offset0 = APAftell(stream0->apabuffer);
#else
    FILE   *fp0;
    
	fp0 = stream0 -> fptr;
	*offset0 = ftell( fp0 );
#endif
    /* I don't know what value ftell() returns on error */
    return( 0 );
}

/***********************************************************CommentBegin******
 *
 * -- SeekBitstream -- Seek the file associated with the bitstream
 *
 * Author :
 *      Hisashi Saiga (SHARP)
 *
 * Created :
 *   25-FEB-1997
 *
 * Purpose :
 *   Seek the file associated with the bitstream
 *
 * Arguments in :
 *   long      offset0      offset
 *   Bitstream   *stream    a bitstream that contains the original status
 * Arguments in/out :
 *
 *
 * Arguments out :
 *
 *
 * Return values :
 *
 *
 * Side effects :
 *
 *
 * Description :
 *
 *
 * See also :
 *
 *
 * Modified :
 *
 *
 ***********************************************************CommentEnd********/

Int
CallingConvention
SeekBitstream(
    Int offset0 ,
    Bitstream *stream0 )
{

#ifdef _IM1_CD_DECODER_  /*A.Pacheco:03/11/98*/
	if ( APAfseek( stream0->apabuffer , ( long )offset0) == -1 )
#else
    FILE   *fp0;
    
    fp0 = stream0 -> fptr;
	if ( fseek( fp0 , ( long )offset0 , 0 ) == -1 ) 
#endif
		return( 1);
	else return( 0 );
}

/***********************************************************CommentBegin******
 *
 * -- write_vop_shape -- ??? but is comented out
 *
 * Author :
 *   ???
 *
 * Created :
 *   ???
 *
 * Purpose :
 *  ???
 *
 ***********************************************************CommentEnd********/

/*
  #include   <fcntl.h>
  
  int   write_vop_shape(
  Vop *vop0 ,
  char *filename0 )
  {
  int   fd0;
  
  fd0 = open( filename0 , O_RDWR | O_TRUNC | O_CREAT , 0666 );
  if ( fd0 == -1 ) {
  perror( filename0 );
  return( 1 );
  }
  write( fd0 , vop0 -> a_chan -> data -> s ,
  GetVopWidth( vop0 ) * GetVopHeight( vop0 ) );
  close( fd0 );
  return( 0 );
  }
  */

/***********************************************************CommentBegin******
 *
 * -- clone_bitstream -- Clones a bitstream???
 *
 * Author :
 *   SHARP???
 *
 * Created :
 *   ???
 *
 * Purpose :
 *  ???
 *
 ***********************************************************CommentEnd********/

Bitstream   *
CallingConvention
clone_bitstream(Bitstream *stream0)
{
    Bitstream   *stream1;
    
    stream1 = ( Bitstream * )malloc ( sizeof( Bitstream ) );
    if ( stream1 == NULL ) return( NULL );
    memcpy( stream1 , stream0 , sizeof( Bitstream ) );
    /* Do not stop here; there is some more to do. */
    stream1 -> rdptr = stream1 -> rdbfr + ( stream0 -> rdptr -
                                            &( stream0 -> rdbfr[ 0 ] ) );
    return( stream1 );
}

/***********************************************************CommentBegin******
 *
 * -- preview_vop_display_time -- ???
 *
 * Author :s
 *   ???
 *
 * Created :
 *   ???
 *
 * Purpose :
 *  ???
 *
 ***********************************************************CommentEnd********/

int
CallingConvention
preview_vop_display_time(
    Bitstream *stream0 ,
    Trace *trace ,
    Vop *curr_vop ,
    Int next_display_time )
{
    Bitstream   *preview_stream;
    Vop   *preview_vop;
    Int   preview_display_time;
    Int   offset0,bits;
    Int   tmpvar,time_base=0,time_base0,tmpvar2;
    Float tmpvar1;
    
/*   printf( "next_display_time = %d\n" , next_display_time ); */
    preview_stream = clone_bitstream( stream0 );
    /* changed 6/11 */
    if ( preview_stream == NULL ) return( 1 );
    /**/
    if ( GetBitstreamPosition( &offset0 , preview_stream ) ) return( 1 );
    tmpvar = ( Int )BitstreamReadBits( preview_stream , VOP_START_CODE_LENGTH ,
                                       "vop_start_code" , trace , CODE );
    if(tmpvar != VOP_START_CODE) {
        printf ( "Bitstream does not start with VOP_START_CODE\n" );
        exit( 1 );
    }
    tmpvar2 = (Int) BitstreamReadBits(preview_stream,2,
                                      "vop_prediction_type",trace,CODE);
    
    /* modulo_time_base (? bits) */
    tmpvar = (Int) BitstreamReadBits( preview_stream , 1 ,
                                      "modulo_time_base" , trace , FLAG );
    preview_vop = CloneVop( curr_vop );
    while( tmpvar == 1 ) {
        tmpvar = ( Int )BitstreamReadBits( preview_stream , 1 ,
                                           "modulo_time_base" , trace , FLAG );
        time_base++;
    }
    /* vop_time_increment (10 bits) */
    
    /* marker_bit (1 bit) */
    tmpvar = (Int) BitstreamReadBits(preview_stream,1,"marker_bit",trace,FLAG);
    
    bits = (Int)ceil(log((double)GetVopTimeIncrementResolution(curr_vop))/log(2.0));
    if (bits<1) bits=1;
    tmpvar = (Int) BitstreamReadBits(preview_stream,bits,"vop_time_increment",trace,NUM); 
    tmpvar1 =(Float)(tmpvar*1000.0)/(Float)GetVopTimeIncrementResolution(curr_vop);
    PutVopTimeInc((Float)tmpvar, preview_vop );
    
    if (tmpvar2==B_VOP) {
        time_base0=GetVopModTimeBase(GetVopPrevTemp(curr_vop));
        if (time_base0<0) time_base0=0;
        PutVopModTimeBase(time_base0+time_base,preview_vop);
    }
    else {
        time_base0=GetVopModTimeBase(GetVopNextTemp(curr_vop));
        if (time_base0<0) time_base0=0;
        PutVopModTimeBase(time_base0+time_base,preview_vop);
    }
    
    
    preview_display_time = CalcVopDisplayTime( preview_vop );
    /* restore everything */
    if ( SeekBitstream( offset0 , stream0 ) ) return( 1 );
    free( preview_stream );
    FreeVop( preview_vop );
    /* restore everything */
/*   printf( "preview_display_time = %d / next_display_time = %d\n" ,
     preview_display_time , next_display_time ); */
    /*if ( preview_display_time > next_display_time ) return( 1 );
      else Minhua Zhouxxx */ return( 0 );
}

/*>SpSc*/
/***********************************************************CommentBegin******
 *
 * -- DecodeVopSpatScalable -- Decodes the VOP information from the bitstream
 *    in spatial scalable mode
 *
 * Author :
 *   Fernando Jaureguizar (UPM)
 *
 * Created :
 *   30-JUL-1997
 *
 * Purpose :
 *    Decode the vop information from the bitstream in spatial scalable mode
 *
 * Arguments in :
 *   Bitstream *stream   pointer on the decoder bitstream structure,
 *                       should be at the right position.
 *   Int vo_id           video object id number
 *   Trace *trace        pointer to trace information
 *
 * Arguments in/out :
 *   Vop *curr_vop   pointer on considered vop - the previous VOP
 *                   can be accessed by the structure of the current
 *                   VOP (GetVopNextTemp())
 *
 * Arguments out :
 *
 *
 * Return values :
 *
 *
 * Side effects :
 *
 *
 * Description :
 *
 *
 * See also :
 *
 *
 * Modified :
 *
 *
 ***********************************************************CommentEnd********/

Void
CallingConvention
DecodeVopSpatScalable(
    Bitstream *stream,
    Int vo_id,
    Trace *trace,
    Vop *curr_vop,
	Int    *prev_ref,
	Int		*next_ref,
	UChar **MB_TYPE2[MAX_NUM_VOS][MAX_NUM_VOLS],
	Int PRE_H_SIZE2[MAX_NUM_VOS][MAX_NUM_VOLS],
	Int PRE_V_SIZE2[MAX_NUM_VOS][MAX_NUM_VOLS],
	UChar **PRE_MB_TYPE2[MAX_NUM_VOS][MAX_NUM_VOLS],
	Int bs_vol[MAX_NUM_VOS][MAX_NUM_VOLS],
	Image   *alpha_rec_packet,
	Image   *alpha_mode_packet,
	SInt Px_for,SInt Py_for, SInt Px_back, SInt Py_back,
	Int prev_j,
	UChar **stock_mb_type2[3][MAX_NUM_VOS][MAX_NUM_VOLS],
	Int   stock_h_size2[3][MAX_NUM_VOS][MAX_NUM_VOLS],
	Int   stock_v_size2[3][MAX_NUM_VOS][MAX_NUM_VOLS]
	)
{
    Image  *MB_mot_decis,*B_mot_decis,*mot_x,*mot_y;
    Int    MB_in_width,MB_in_height,B_in_width,B_in_height;
    Int    *mb_type;
    Int    vol_id = GetVopId(curr_vop);
    
    /*****
     *
     *  Free image channels of VOP and allocate the image channels
     *  for the new sizes (only necessary if arbitrary shape)
     *
     *****/
    
    if(GetVopShape(curr_vop))
    {
        fprintf(stderr,"Can't decode Spatial Scalability with Objects\n");
        exit(1);
    }
    
    /*****
     *
     *  Set local parameters and allocate local memory
     *
     *****/
    
    MB_in_width  = GetVopWidth(curr_vop)  / MB_SIZE;
    MB_in_height = GetVopHeight(curr_vop) / MB_SIZE;
    B_in_width   = 2 * MB_in_width;
    B_in_height  = 2 * MB_in_height;
    
    MB_mot_decis = AllocImage((UInt) MB_in_width,(UInt) MB_in_height,SHORT_TYPE);
    B_mot_decis = AllocImage ((UInt) B_in_width,(UInt) B_in_height,SHORT_TYPE);
    mot_x = AllocImage((UInt) B_in_width,(UInt) B_in_height,FLOAT_TYPE);
    mot_y = AllocImage((UInt) B_in_width,(UInt) B_in_height,FLOAT_TYPE);
    
    mb_type=(Int *)calloc((MB_in_width*MB_in_height), sizeof(Int));
    
    ZeroImageI(MB_mot_decis);
    ZeroImageF(mot_x);
    ZeroImageF(mot_y);
    
    /*****
     *
     *  Do combined or separate decoding
     *
     *****/
    
    if(GetVopSepMotShapeText(curr_vop) == 0)
    {
        if(GetVopShape(curr_vop)==0)
        {
            SetConstantImageI(GetVopA(curr_vop),255);
            SetConstantImageI(GetVopAuv(curr_vop),255);
        }
        if(GetVopErrorResDisable(curr_vop))
        {             
            
#ifdef _DEBUG_SPSC2_
            if(tmp_cnt==0) {
                WriteVopGeneric( GetVopNextTemp(curr_vop), "Dnext_vop", "Dnext_vop", "Dnext_vop",
                                 "tmpppu", 0, IO_FORMAT, IO_OVERWRITE, 0);
            } else {
                WriteVopGeneric( GetVopNextTemp(curr_vop), "Dnext_vop", "Dnext_vop", "Dnext_vop",
                                 "tmpppu", tmp_cnt, IO_FORMAT, IO_APPEND, 0);
            }
            tmp_cnt++;
#endif
            
            
            
            
            DecodeVopCombMotionShapeTexture(stream,vo_id,trace,curr_vop,prev_ref,next_ref,
						MB_TYPE2,PRE_H_SIZE2,PRE_V_SIZE2,PRE_MB_TYPE2,bs_vol,
						alpha_rec_packet,alpha_mode_packet,
						Px_for,Py_for,Px_back,Py_back,prev_j);
            
/*** 10/27 for TPS */
            ChangeMBtype2(GetVopWidth(curr_vop),GetVopHeight(curr_vop),vo_id,vol_id,
					PRE_H_SIZE2,PRE_V_SIZE2,PRE_MB_TYPE2,MB_TYPE2);
/*      if(temporal_scalability) ChangeMBtype2_for_TPS(vo_id, vol_id); */
            ChangeMBtype2_for_TPS(vo_id, vol_id, GetVopPredictionType( curr_vop ),
					PRE_MB_TYPE2,PRE_H_SIZE2,PRE_V_SIZE2,
					stock_mb_type2,stock_h_size2,stock_v_size2); 
/* 10/27 for TPS ***/
        }
        else
        {
            fprintf(stderr,"Can't decode Spatial Scalability with Error Resil.\n");
            exit(1);
        }
    }
    else
    {
        fprintf(stderr,"Can't decode Spatial Scalability with Separate Mode\n");
        exit(1);
    }
    
    /*****
     *
     *  Free local memory and return
     *
     *****/
    free((Char *)mb_type);
    
    FreeImage(MB_mot_decis);
    FreeImage(B_mot_decis);
    FreeImage(mot_x);
    FreeImage(mot_y);
    
    return;
}

/*** 8/18 TPS */
/***********************************************************CommentBegin******
 *
 * -- CheckBaseVop
 *
 * Author :             
 *      Ryuji KITAURA (SHARP)
 *
 ***********************************************************CommentBegin******/
Int
CallingConvention
CheckBaseVop(Vop *curr_vop)
{
    Int vop_id;
    Int ref_vop_id;
    Int scalability;
    Int base;
    
    scalability = GetVopScalability( curr_vop );
    ref_vop_id  = GetVopRefId( curr_vop );
    vop_id      = GetVopId( curr_vop );
    
    if(scalability == 0)
        base = 1;
    else base = 0;
    
    return(base);
}


/***********************************************************CommentBegin******
 *
 * -- SetRefVop
 *
 * Author :             
 *      Ryuji KITAURA (SHARP)
 *
 * Modified: 18.9.97 T. Aono(Sharp): Simplify by removing malloc and free in some part
 *
 ***********************************************************CommentBegin******/
Void
CallingConvention
SetRefVop(Vop *ref_vop0, Vop *ref_vop1, Vop *curr_vop,Int zero_edge)
{
    Int ref_sel_code, predic_type;
    
    ref_sel_code = GetVopRefSelCode( curr_vop );
    predic_type  = GetVopPredictionType( curr_vop );
    
    /* Padding for reference vop ( BASE vop ) */
    if (ref_vop0 != NULL) {
        PutVopNextTemp(NULL, ref_vop0);
        PutVopPrevTemp(NULL, ref_vop0);
        PutVopShape( GetVopShape(curr_vop) , ref_vop0 ); 
        VopPadding(ref_vop0,&zero_edge);
    }
/*** 10/27 for TPS */
    if (ref_vop1 != NULL) {
        PutVopNextTemp(NULL, ref_vop1);
        PutVopPrevTemp(NULL, ref_vop1);
        PutVopShape( GetVopShape(curr_vop) , ref_vop1 ); 
        VopPadding(ref_vop1,&zero_edge);
    }
/* 10/27 for TPS ***/
    
    /* PrevTemp Free */
    if(GetVopPrevTemp(curr_vop) != NULL){
        FreeVop(GetVopPrevTemp(curr_vop));
        PutVopPrevTemp(NULL, curr_vop);
    }
    
    /* for enhance frame only I or P picture */ 
    if ( predic_type == P_VOP && ref_sel_code == 1 ) {
        if(GetVopNextTemp(curr_vop) !=  NULL){
            FreeVop(GetVopNextTemp(curr_vop));
            PutVopNextTemp(NULL, curr_vop);
        }
        PutVopNextTemp( ref_vop0 , curr_vop );
    }
/*** 10/27 for TPS */
    /* for enhance frame B picture */ 
    else if ( predic_type == B_VOP ) {
        if ( ref_sel_code == 3 ) {
            PutVopPrevTemp( ref_vop0 , curr_vop );
            if( GetVopNextTemp(curr_vop) !=  NULL){
                FreeVop(GetVopNextTemp(curr_vop));
                PutVopNextTemp(NULL, curr_vop);
            }
            PutVopNextTemp( ref_vop1 , curr_vop );
        }
        else if ( ref_sel_code == 1 ) {
            PutVopPrevTemp(GetVopNextTemp(curr_vop),curr_vop);
            PutVopNextTemp( ref_vop0 , curr_vop );
            
            if (ref_vop1!=NULL) {
                FreeVop(ref_vop1);
                ref_vop1=NULL;
            }
        }
    }
/* 10/27 for TPS ***/
    return;
}

/*** 10/27 for TPS changed by kitaura */
/***********************************************************CommentBegin******
 *
 * -- SetRefMB_TYPE2
 *
 * Author :             
 *      Ryuji KITAURA (SHARP)
 *
 ***********************************************************CommentBegin******/
Void
CallingConvention
SetRefMB_TYPE2(Vop *curr_vop,
               Int vo_id,
               Int nvol_id,
               Int nref_vol_id ,
               Int last_base_exist,
			   UChar **PRE_MB_TYPE2[MAX_NUM_VOS][MAX_NUM_VOLS],
			   Int PRE_H_SIZE2[MAX_NUM_VOS][MAX_NUM_VOLS],
				Int PRE_V_SIZE2[MAX_NUM_VOS][MAX_NUM_VOLS],
				UChar **stock_mb_type2[3][MAX_NUM_VOS][MAX_NUM_VOLS],
				Int   stock_h_size2[3][MAX_NUM_VOS][MAX_NUM_VOLS],
				Int   stock_v_size2[3][MAX_NUM_VOS][MAX_NUM_VOLS]
)       
{
    Int j,k;
    Int ref_sel_code, predic_type;
    
    ref_sel_code=GetVopRefSelCode( curr_vop );
    predic_type=GetVopPredictionType( curr_vop );
    
    if(PRE_MB_TYPE2[vo_id][nvol_id]!=NULL)
        FreePRE_MBtype2(PRE_V_SIZE2[ vo_id ][ nvol_id ]*MB_SIZE,vo_id,nvol_id,PRE_MB_TYPE2); 
    
    if (GetVopScalability(curr_vop)==0) {
        if (predic_type==P_VOP|| predic_type==SPRITE_VOP) {
            PRE_H_SIZE2[vo_id][ nvol_id ]=stock_h_size2[ 1 ][ vo_id ][ nref_vol_id ];
            PRE_V_SIZE2[vo_id][ nvol_id ]=stock_v_size2[ 1 ][ vo_id ][ nref_vol_id ];
            AllocPRE_MBtype2(PRE_H_SIZE2[ vo_id ][ nvol_id ]*MB_SIZE,
                             PRE_V_SIZE2[ vo_id ][ nvol_id ]*MB_SIZE,vo_id, nvol_id,
							 PRE_MB_TYPE2);
            for ( j = 0 ; j < PRE_V_SIZE2[vo_id][ nvol_id ] ; j++ )
                for ( k = 0 ; k < PRE_H_SIZE2[vo_id][ nvol_id ] ; k++ ) 
                    PRE_MB_TYPE2[ vo_id ][ nvol_id ][ j ][ k ] =
                        stock_mb_type2[ 1 ][ vo_id ][ nvol_id ][ j ][ k ];
        }
        else if (predic_type==B_VOP) {
            PRE_H_SIZE2[vo_id][ nvol_id ]=stock_h_size2[ 0 ][ vo_id ][ nref_vol_id ];
            PRE_V_SIZE2[vo_id][ nvol_id ]=stock_v_size2[ 0 ][ vo_id ][ nref_vol_id ];
            AllocPRE_MBtype2(PRE_H_SIZE2[ vo_id ][ nvol_id ]*MB_SIZE,
                             PRE_V_SIZE2[ vo_id ][ nvol_id ]*MB_SIZE,vo_id, nvol_id,
							 PRE_MB_TYPE2);
            for ( j = 0 ; j < PRE_V_SIZE2[ vo_id ][ nvol_id ] ; j++ )
                for ( k = 0 ; k < PRE_H_SIZE2[ vo_id ][ nvol_id ] ; k++ ) 
                    PRE_MB_TYPE2[ vo_id ][ nvol_id ][ j ][ k ] =
                        stock_mb_type2[ 0 ][ vo_id ][ nvol_id ][ j ][ k ];
        }
        return; 
    }
    
    if (GetVopEnhanceType( curr_vop ) || (last_base_exist ==1)){
        PRE_H_SIZE2[ vo_id ][ nvol_id ]=stock_h_size2[ 0 ][ vo_id ][ nref_vol_id ];
        PRE_V_SIZE2[ vo_id ][ nvol_id ]=stock_v_size2[ 0 ][ vo_id ][ nref_vol_id ];
    }
    else if(last_base_exist == 0){
        PRE_H_SIZE2[ vo_id ][ nvol_id ]=stock_h_size2[ 1 ][ vo_id ][ nref_vol_id ];
        PRE_V_SIZE2[ vo_id ][ nvol_id ]=stock_v_size2[ 1 ][ vo_id ][ nref_vol_id ];
    }  
    AllocPRE_MBtype2(PRE_H_SIZE2[ vo_id ][ nvol_id ]*MB_SIZE,
                     PRE_V_SIZE2[ vo_id ][ nvol_id ]*MB_SIZE,vo_id, nvol_id,
					 PRE_MB_TYPE2); 
    
    if (GetVopEnhanceType( curr_vop ) ||
        (GetVopEnhanceType( curr_vop ) && predic_type == B_VOP )){ /* type 1 */
        for ( j = 0 ; j < PRE_V_SIZE2[ vo_id ][ nvol_id ] ; j++ )
            for ( k = 0 ; k < PRE_H_SIZE2[ vo_id ][ nvol_id ] ; k++ ) 
                PRE_MB_TYPE2[ vo_id ][ nvol_id ][ j ][ k ] = ( unsigned char )3;
    }
    else{ /* type 2 */
        if(last_base_exist == 1){
            for ( j = 0 ; j < PRE_V_SIZE2[ vo_id ][ nvol_id ] ; j++ )
                for ( k = 0 ; k < PRE_H_SIZE2[ vo_id ][ nvol_id ] ; k++ ) 
                    PRE_MB_TYPE2[ vo_id ][ nvol_id ][ j ][ k ] =
                        stock_mb_type2[ 0 ][ vo_id ][ nref_vol_id ][ j ][ k ];
        }
        else if(last_base_exist == 0){
            for ( j = 0 ; j < PRE_V_SIZE2[ vo_id ][ nvol_id ] ; j++ )
                for ( k = 0 ; k < PRE_H_SIZE2[ vo_id ][ nvol_id ] ; k++ ) 
                    PRE_MB_TYPE2[ vo_id ][ nvol_id ][ j ][ k ] =
                        stock_mb_type2[ 1 ][ vo_id ][ nref_vol_id ][ j ][ k ];
        }
    }
}
/* 10/27 for TPS  ***/

/***********************************************************CommentBegin******
 *
 * -- ChangeMBtype2_for_TPS
 *
 * Author :             
 *      Ryuji KITAURA (SHARP)
 *
 ***********************************************************CommentBegin******/
Void
CallingConvention
ChangeMBtype2_for_TPS(	Int vo_id, Int vol_id, Int predic_type,
						UChar **PRE_MB_TYPE2[MAX_NUM_VOS][MAX_NUM_VOLS],
						Int PRE_H_SIZE2[MAX_NUM_VOS][MAX_NUM_VOLS],
						Int PRE_V_SIZE2[MAX_NUM_VOS][MAX_NUM_VOLS],
						UChar **stock_mb_type2[3][MAX_NUM_VOS][MAX_NUM_VOLS],
						Int   stock_h_size2[3][MAX_NUM_VOS][MAX_NUM_VOLS],
						Int   stock_v_size2[3][MAX_NUM_VOS][MAX_NUM_VOLS]
						)
{
    Int
        i,j;
    
/*** 10/27 for TPS */
    if (predic_type==I_VOP || predic_type==P_VOP || predic_type==SPRITE_VOP) {
        if(stock_mb_type2[0][vo_id][vol_id] != NULL)
            FreePRE_PRE_MBtype2(stock_v_size2[0][vo_id][vol_id]*MB_SIZE,vo_id,vol_id,0,
								stock_mb_type2);
		if(stock_mb_type2[1][vo_id][vol_id] != NULL) {
            stock_h_size2[0][vo_id][vol_id]=stock_h_size2[1][vo_id][vol_id];
			stock_v_size2[0][vo_id][vol_id]=stock_v_size2[1][vo_id][vol_id];
			AllocPRE_PRE_MBtype2(stock_h_size2[0][vo_id][vol_id]*MB_SIZE,
				                 stock_v_size2[0][vo_id][vol_id]*MB_SIZE,vo_id,vol_id,0,
								 stock_mb_type2);
			for (i=0;i<stock_v_size2[0][vo_id][vol_id];i++)
				for (j=0;j<stock_h_size2[0][vo_id][vol_id];j++)
					stock_mb_type2[0][vo_id][vol_id][i][j]=stock_mb_type2[1][vo_id][vol_id][i][j]; /* stock for most recent decoded I/P VOP */
            FreePRE_PRE_MBtype2(stock_v_size2[1][vo_id][vol_id]*MB_SIZE,vo_id,vol_id,1,
								stock_mb_type2);
		} else
			stock_mb_type2[0][vo_id][vol_id] = NULL;
        stock_h_size2[1][vo_id][vol_id]=PRE_H_SIZE2[vo_id][vol_id];
        stock_v_size2[1][vo_id][vol_id]=PRE_V_SIZE2[vo_id][vol_id];
        AllocPRE_PRE_MBtype2(stock_h_size2[1][vo_id][vol_id]*MB_SIZE,
                             stock_v_size2[1][vo_id][vol_id]*MB_SIZE,vo_id,vol_id,1,
							 stock_mb_type2);
        for (i=0;i<stock_v_size2[1][vo_id][vol_id];i++)
            for (j=0;j<stock_h_size2[1][vo_id][vol_id];j++)
                stock_mb_type2[1][vo_id][vol_id][i][j]=PRE_MB_TYPE2[vo_id][vol_id][i][j]; /* stock for 2nd recent decoded I/P VOP */
    }
    else if (predic_type==B_VOP) {
        if(stock_mb_type2[2][vo_id][vol_id] != NULL)
            FreePRE_PRE_MBtype2(stock_v_size2[2][vo_id][vol_id]*MB_SIZE,vo_id,vol_id,2,
								stock_mb_type2);
        stock_h_size2[2][vo_id][vol_id]=PRE_H_SIZE2[vo_id][vol_id];
        stock_v_size2[2][vo_id][vol_id]=PRE_V_SIZE2[vo_id][vol_id];
        AllocPRE_PRE_MBtype2(stock_h_size2[2][vo_id][vol_id]*MB_SIZE,
                             stock_v_size2[2][vo_id][vol_id]*MB_SIZE,vo_id,vol_id,2,
							 stock_mb_type2);
        for (i=0;i<stock_v_size2[2][vo_id][vol_id];i++)
            for (j=0;j<stock_h_size2[2][vo_id][vol_id];j++)
                stock_mb_type2[2][vo_id][vol_id][i][j]=PRE_MB_TYPE2[vo_id][vol_id][i][j]; /* stock for decoded B VOP */
    }
/* 10/27 for TPS ***/
}

/***********************************************************CommentBegin******
 *
 * -- CheckMBType2
 *
 * Author :             
 *      Ryuji KITAURA (SHARP)
 *
 ***********************************************************CommentBegin******/
/* void
   CheckMBtype2( Int h_size,
   Int v_size,
   Int vo_id,
   Int vol_id,
   Int frame)
   {
   Int h,v,i,j;
   
   h = h_size/MB_SIZE;
   v = v_size/MB_SIZE;
   
/*** 10/9 TPS */
/*  fprintf(stdout,"[%d]MB_TYPE\t PRE_MB_TYPE\t PRE_PRE_MB_TYPE\n",frame);
    
    for(i=0;i<v;i++){
    for(j=0;j<h;j++)
    fprintf(stdout,"%d",MB_TYPE2[vo_id][vol_id][i][j]);
    fprintf(stdout,"\t");
    
    if(i<PRE_V_SIZE2[vo_id][vol_id])
    for(j=0;j<PRE_H_SIZE2[vo_id][vol_id];j++)
    fprintf(stdout,"%d",PRE_MB_TYPE2[vo_id][vol_id][i][j]);
    fprintf(stdout,"\t");
    
    if(i<PRE_PRE_V_SIZE2[vo_id][vol_id])
    for(j=0;j<PRE_PRE_H_SIZE2[vo_id][vol_id];j++)
    fprintf(stdout,"%d",PRE_PRE_MB_TYPE2[vo_id][vol_id][i][j]);
    
    fprintf(stdout,"\n");
    }
    fprintf(stdout,"\n");
    /* 10/9 TPS ***/
/*} */
/***********************************************************CommentBegin******
 *
 * -- Deblock_TPS
 *
 * Author :             
 *      Ryuji KITAURA (SHARP)
 *
 ***********************************************************CommentBegin******/
/*
  Vop
  *Deblock_TPS(Vop *curr_vop, Int post_filter_type, Int image_width, Int image_height)
  {
  Vop *local,
  *file_vop;
  
  file_vop = AllocVop( image_width , image_height );
  
  SetConstantImage(GetVopA(file_vop),0);
  SetConstantImage(GetVopY(file_vop),0);
  SetConstantImage(GetVopU(file_vop),128);
  SetConstantImage(GetVopV(file_vop),128);
  
  if(GetVopShape(curr_vop) == 0)
  SetConstantImage(GetVopA(curr_vop),255);
  
  if(GetVopShape(curr_vop)!=0 && GetVopWidth(curr_vop)==0) ;
  else {
  if(post_filter_type)
  BlendVops(curr_vop, file_vop);
  else {
  local = DeblockVop( curr_vop,      post_filter_type);
  BlendVops(local, file_vop);
  FreeVop(local);
  }
  }
  return(file_vop);
  }
  */
/***********************************************************CommentBegin******
 *
 * -- LastFrameInput
 *
 * Author :             
 *      Ryuji KITAURA (SHARP)
 *
 ***********************************************************CommentBegin******/
Void
CallingConvention
LastFrameInput(Vop *curr_vop,Int last_base_decoding_frame[3],Int *last_enh_decoding_frame)
{
    if(CheckBaseVop(curr_vop)){
/*** 10/27 for TPS */
        if(last_base_decoding_frame[0] < 0 || last_base_decoding_frame[2] < 0) {
            last_base_decoding_frame[0] = last_base_decoding_frame[2];
            last_base_decoding_frame[2] = CalcFrameNumber( curr_vop );
        }
        else{
            Int predic_type=GetVopPredictionType(curr_vop);
            if (last_base_decoding_frame[1] < 0 && predic_type==B_VOP)
                last_base_decoding_frame[1] = CalcFrameNumber( curr_vop );
            else {
                if (last_base_decoding_frame[1] < 0) {
                    last_base_decoding_frame[0] = last_base_decoding_frame[2];
                    last_base_decoding_frame[2] = CalcFrameNumber( curr_vop );
                }
                else {
                    last_base_decoding_frame[0] = last_base_decoding_frame[1];
                    last_base_decoding_frame[1] = last_base_decoding_frame[2];
/*** 11/5 for TPS */
/*        if (predic_type==P_VOP) */
                    if (predic_type!=B_VOP)
/* 11/5 for TPS ***/
                        last_base_decoding_frame[2] = CalcFrameNumber( curr_vop );
                    else
                        last_base_decoding_frame[1] = CalcFrameNumber( curr_vop );
                }
            }
        }
/* 10/27 for TPS ***/
    }
    else  
        *last_enh_decoding_frame = CalcFrameNumber( curr_vop );
/* TEST */
/*  printf("last_base[0] = %3d\n",last_base_decoding_frame[0]);
    printf("last_base[1] = %3d\n",last_base_decoding_frame[1]);
    printf("last_base[2] = %3d\n",last_base_decoding_frame[2]);
    printf("last_enh     = %3d\n",last_enh_decoding_frame); */
/* TEST */
}
/***********************************************************CommentBegin******
 *
 * -- BASEImageInput
 *
 * Author :             
 *      Ryuji KITAURA (SHARP)
 *
 ***********************************************************CommentBegin******/
void
CallingConvention
BASEImageInput(Vop *curr_vop, Vop *base_output_vop[3],Int *TPS_base_count)
{
/*** 10/27 for TPS */
    Int i, j, fg, time[3], min;
    
    i=0; fg=0; min=10000000;
    while (base_output_vop[i]!=NULL && i<3) i++; 
    if (i<3) j=i;
    else {
        for (i=0;i<3;i++) {
            time[i]=CalcVopDisplayTime(base_output_vop[i]);
            if (time[i]<min) {
                min=time[i];
                j=i;
            }
        }
        FreeVop(base_output_vop[j]);
        base_output_vop[j] = NULL;
    }
    base_output_vop[j]=CloneVop(curr_vop);
    *TPS_base_count=j;
/*  printf("TPS_base_count= %3d\n : scal_dec.c",TPS_base_count); */
/* 10/27 for TPS ***/
}


/***********************************************************CommentBegin******
 *
 * -- WriteOutputImage_TPS_for_base
 *
 * Author :             
 *      Ryuji KITAURA (SHARP)
 *
 ***********************************************************CommentBegin******/
Void
CallingConvention
WriteOutputImage_TPS_for_base(Vop *base_output_vop[3],
                              Int next_display_time,
                              Int post_filter_type,
							  Int zero_edge,
							  Int image_width,Int image_height)
{
    Int i;
    Int frame = -1;
    Vop *file_vop = NULL;
    Int next_display_frame = -1;
    
/*** 10/29 for TPS */
    next_display_frame = TimetoFrameNumeber((Float)next_display_time);
/* 10/29 for TPS ***/
    
    for(i = 0; i < 3 ; i++){
        if(base_output_vop[i] != NULL)
            frame = CalcFrameNumber(base_output_vop[i]);
        
        if(frame == next_display_frame){
            if ( GetVopWidth( base_output_vop[i] ) != 0 ){
                fprintf(stderr,"   Write BASE\n");
/*** 10/27 for TPS */
                file_vop = PostFilter_TPS(base_output_vop[i], post_filter_type, image_width , image_height,zero_edge);
/* 10/27 for TPS ***/
/*A.Pacheco:1/04/98*/
#if 0
                WriteVopGeneric(file_vop,
                                comp_output_filename[ 0 ] ,
                                comp_output_filename[ 1 ] ,
                                comp_output_filename[ 2 ] ,
                                comp_output_filename[ 3 ] ,
                                CalcFrameNumber(base_output_vop[i]),
                                IO_FORMAT , IO_APPEND ,FALSE );
#endif
/* TEST */
                printf("**************write frame = %3d\n",CalcFrameNumber(base_output_vop[i]));
/* TEST */
                FreeVop(file_vop);
/*** 10/27 for TPS */
/*             FreeVop(base_output_vop[i]);
               base_output_vop[i] = NULL; */
/* 10/27 for TPS ***/
                i = 3;
            }
        }
    } 
    return;
}

/***********************************************************CommentBegin******
 *
 * -- Check_TPS
 *
 * Author :             
 *      Ryuji KITAURA (SHARP)
 *
 * Modified: 15.9.97 A. Sandvand: Removed variables for Random Access
 * Modified: 16.9.97 T. Aono(Sharp): Add argument "random_access_start_time"
 *
 ***********************************************************CommentBegin******/
Void
CallingConvention
Check_TPS(Int number_of_vols,
          Bitstream *stream[],
          Trace *trace[],
          Int random_access_start_time,
          Vol *vol_list,Int temporal_scalability,
		  Int start_time[MAX_NUM_VOLS][2])

{
    
    Vol *next_vol;
    Int k;
    next_vol = vol_list;
    
    for(k=0; k<number_of_vols; k++){
        if(!temporal_scalability)
            Check_TPS_decode(stream[k],
                             next_vol,
                             random_access_start_time,
                             trace[k],
                             k,temporal_scalability,
							 start_time);
        next_vol = GetVolNext(next_vol);
    }
}

/***********************************************************CommentBegin******
 *
 * -- Check_TPS_decode
 *
 * Author :             
 *      Ryuji KITAURA (SHARP)
 *
 * Modified: 15.9.97 A. Sandvand: Removed variables for Random Access
 * Modified: 16.9.97 T. Aono(Sharp): Add argument "random_access_start_time"
 *                                   Simplify by removing malloc and free in some part
 ***********************************************************CommentBegin******/
Void
CallingConvention
Check_TPS_decode(Bitstream *stream,
                 Vol *next_vol,
                 Int random_access_start_time,
                 Trace *trace,
                 Int nvol_id,Int temporal_scalability,
				 Int start_time[MAX_NUM_VOLS][2])

{
    Bitstream *ref_stream;
    
    Vop *tmp_vop;
    Int tmpvar;
    Int time_base = 0;
    Int time_inc  = 0;
    Int readen_bits;
    //FILE *fp_TMP;
    Int bits;
    /*A.Pacheco:1/04/98 static Int start_time[MAX_NUM_VOLS][2];*/
    Int t0;
/*** 10/27 for TPS */
    Int offset;
/* 10/27 for TPS ***/
    
    tmp_vop        = CloneVop(GetVolVop(next_vol));
    ref_stream     = clone_bitstream(stream);
    
/*** 10/27 TPS */  
    GetBitstreamPosition(&offset,stream);        
/* 10/27 TPS ***/  
    tmpvar         = (Int) BitstreamShowBits(ref_stream,VOP_START_CODE_LENGTH);
    
    if (tmpvar == GROUP_START_CODE) {
        DecodeGOV(ref_stream,trace,tmp_vop, &readen_bits);
        tmpvar = (Int) BitstreamShowBits(ref_stream,VOP_START_CODE_LENGTH);
    }
    if(tmpvar == VOP_START_CODE) {
        tmpvar = (Int) BitstreamReadBits(ref_stream,VOP_START_CODE_LENGTH,"vop_start_code",trace,CODE);
        tmpvar = (Int) BitstreamReadBits(ref_stream,2,"vop_prediction_type",trace,FLAG);
        tmpvar = (Int) BitstreamReadBits(ref_stream,1,"modulo_time_base",trace,FLAG);
        
        while(tmpvar == 1){
            tmpvar = (Int) BitstreamReadBits(ref_stream,1,"modulo_time_base",trace,FLAG);
            time_base++;
        }
        
        /* marker_bit (1 bit) */
        tmpvar = (Int) BitstreamReadBits(ref_stream,1,"marker_bit",trace,FLAG);
        
        bits = (Int)ceil(log((double)GetVolTimeIncrementResolution(next_vol))/log(2.0));
        if (bits<1) bits=1;
        
        time_inc = (Int) BitstreamReadBits(ref_stream,bits,"vop_time_increment",trace,NUM); 
    }
    
    t0=GetVopModTimeBase(tmp_vop);
    if (t0<0) t0=0;
    time_base=time_base+t0;
    start_time[nvol_id][0]=time_base;
    start_time[nvol_id][1]=time_inc;
    if(nvol_id!=0 && (start_time[nvol_id][0]!=start_time[nvol_id-1][0] ||
                      start_time[nvol_id][1]!=start_time[nvol_id-1][1])) {
        fprintf(stderr,"temporal_scalability is ON\n");
        temporal_scalability = 1;
/*A.Pacheco:1/04/98*/
#if 0
        if(NULL == (fp_TMP = fopen(comp_output_filename[ 0 ],"w"))){
            fprintf(stderr,"cannot open %s\n",comp_output_filename[ 0 ]);
            exit(1);
        }
        fclose(fp_TMP);
        if(NULL == (fp_TMP = fopen(comp_output_filename[ 1 ],"w"))){
            fprintf(stderr,"cannot open %s\n",comp_output_filename[ 1 ]);
            exit(1);
        }
        fclose(fp_TMP);
        if(NULL == (fp_TMP = fopen(comp_output_filename[ 2 ],"w"))){
            fprintf(stderr,"cannot open %s\n",comp_output_filename[ 2 ]);
            exit(1);
        }
        fclose(fp_TMP);
        if(NULL == (fp_TMP = fopen(comp_output_filename[ 3 ],"w"))){
            fprintf(stderr,"cannot open %s\n",comp_output_filename[ 3 ]);
            exit(1);
        }
        fclose(fp_TMP);
#endif
    }
    
    FreeVop(tmp_vop);
/*** 10/27 for TPS */  
    SeekBitstream(offset,stream);        
/* 10/27 for TPS ***/  
    free(ref_stream);
    return;
}


/***********************************************************CommentBegin******
 *
 * -- CalcFrameNumber
 *
 * Author :             
 *      Ryuji KITAURA (SHARP)
 *
 ***********************************************************CommentBegin******/
Int
CallingConvention
CalcFrameNumber(Vop *curr_vop)
{
    Int modulo_time_base, time_increment, display_time;
    
    modulo_time_base = GetVopModTimeBase(curr_vop);
    time_increment = (Int)GetVopTimeInc(curr_vop);
/*** 10/27 for TPS */
/*  display_time = modulo_time_base*1000 + time_increment; */
    display_time = (Int)(modulo_time_base*1000 + time_increment*1000.0/(double)GetVopTimeIncrementResolution(curr_vop));
/* 10/27 for TPS ***/
    
/*** 10/29 for TPS */
    return(TimetoFrameNumeber((Float)display_time));
/* 10/29 for TPS ***/
}

/***********************************************************CommentBegin******
 *
 * -- TimetoFrameNumeber
 *
 * Author :             
 *      Ryuji KITAURA (SHARP)
 *
 ***********************************************************CommentBegin******/
Int
CallingConvention
TimetoFrameNumeber(Float display_time)
{
    Int a;
    a = (Int)( display_time / 33.33);
    if( display_time - a * 33.33 > 30)
        a = a + 1;
    return(a);
}
/***********************************************************CommentBegin******
 *
 * -- SetBGCImage
 *
 * Author :             
 *      Ryuji KITAURA (SHARP)
 *
 ***********************************************************CommentBegin******/
Void
CallingConvention
SetBGCImage(Vop *curr_vop)
{
    Vop *tmp_vop;
    
    if (GetVopPredictionType(curr_vop)!=B_VOP) {
        if (GetVopPrevTemp(curr_vop)!=NULL) 
            FreeVop(GetVopPrevTemp(curr_vop));
        PutVopPrevTemp(GetVopNextTemp(curr_vop),curr_vop);
        PutVopNextTemp(NULL,curr_vop);
        tmp_vop = CloneVop(curr_vop);
        if (GetVopPrevTemp(tmp_vop)!=NULL)
            FreeVop(GetVopPrevTemp(tmp_vop));
        PutVopPrevTemp(NULL,tmp_vop);
        PutVopNextTemp(tmp_vop,curr_vop);
    }
    if(CheckBaseVop(curr_vop))
        fprintf(stderr,"[B]frame[%d] NOT decoding\n",CalcFrameNumber(curr_vop));
    else
        fprintf(stderr,"[E]frame[%d] NOT decoding\n",CalcFrameNumber(curr_vop));
/*** 10/27 for TPS */
/*  LastFrameInput(curr_vop); */
/* 10/27 for TPS ***/
}     
/* 8/18 TPS ***/

/*** 10/27 for TPS */
/***********************************************************CommentBegin******
 *
 * -- search_vop_for_bgc
 *
 * Author :             
 *      Tomoko Aono (SHARP)
 *
 ***********************************************************CommentBegin******/
Void 
CallingConvention
search_vop_for_bgc(Vop *vop_list[], Int vop_exist[], Vop *curr_vop, Int vop_display_time[],Vop	      *base_output_vop[3])
{
    Int count, time[3], time_enh, dtime_f[3], dtime_b[3];
    Int bgcfor, bgcback, kfor, kback;
    Int i, k, kk[2], minf, minb;
    
    count=0; bgcfor=bgcback=0;
    vop_exist[0]=vop_exist[1]=0;
    
    for (i=0;i<3;i++) {
        if (base_output_vop[i]!=NULL) {
            time[i]=CalcVopDisplayTime(base_output_vop[i]); 
            time[i]=(Int)(0.001+time[i]*1000.0/(Double)GetVopTimeIncrementResolution(base_output_vop[i]));
            count++;
        }
        else time[i]= -1;
    }
    time_enh=CalcVopDisplayTime(curr_vop); 
    time_enh=(Int)(0.001+time_enh*1000.0/(Double)GetVopTimeIncrementResolution(curr_vop));
/* TEST */
/* printf("base time %3d, %3d, %3d enh time %3d\n",time[0],time[1],time[2],time_enh);
   fprintf(stderr,"base time %3d, %3d, %3d enh time %3d\n",time[0],time[1],time[2],time_enh); */
/* TEST */
    if (count==0) { /* no base vop exists. */
        printf("error base vop for bgc! : scale_dec.c\n");
        return;
    }
    else if (count==1) { /* 1 base vop exists! */
        for (i=0;i<3;i++)
            if (time[i]!= -1) {
                if (time[i]<time_enh) {
                    kfor=i; bgcfor=1;
                }
                break;
            }
        if (bgcfor==0) {
            printf("error base vop for bgc! : scale_dec.c\n");
            return;
        }
    }
    else if (count==2) { /* 2 base vop exists! */
        for (i=0;i<3;i++) {
            if (time[i]!= -1) {
                if (time[i]<time_enh) bgcfor++;
                else if (time[i]>time_enh) bgcback++;
            }
        }
        if (bgcfor==0) printf("error base vop for bgc! : scale_dec.c\n");
        else if (bgcfor==1) { /* base->enh->base */
            for (i=0;i<3;i++) {
                if (time[i]!= -1) {
                    if (time[i]<time_enh) kfor=i;
                    else if (time[i]>time_enh) kback=i;
                }
            }
        }
        else if (bgcfor==2) { /* base->enh */
            k=0;
            for (i=0;i<3;i++) if (time[i]!= -1) kk[k++]=i;
            if (time[kk[0]]>=time[kk[1]]) kfor=kk[0];
            else if (time[kk[0]]<time[kk[1]]) kfor=kk[1];
        }
    }
    else { /* 3 base exists! */
        for (i=0;i<3;i++) {
            dtime_f[i]=time_enh-time[i];
            dtime_b[i]=time[i]-time_enh;
        }
        minf=100000; minb=100000;
        for (i=0;i<3;i++) {
            if (dtime_f[i]>0 && dtime_f[i]<minf) {
                minf=dtime_f[i];
                kfor=i;
                bgcfor++;
            }
            else if (dtime_b[i]>0 && dtime_b[i]<minb) {
                minb=dtime_b[i];
                kback=i;
                bgcback++;
            }
        }
    }
    vop_list[0]=NULL;  vop_list[1]=NULL; 
/* TEST */
/* printf("bgc time "); */
/* TEST */
    if (bgcfor!=0) {
        vop_list[0]=CloneVop(base_output_vop[kfor]);
        vop_display_time[0]=time[kfor];
        vop_exist[0]=1;
/* TEST */
/* printf("for= %3d",vop_display_time[0]); */
/* TEST */
    }
    if (bgcback!=0) {
        vop_list[1]=CloneVop(base_output_vop[kback]);
        vop_display_time[1]=time[kback];
        vop_exist[1]=1;
/* TEST */
/* printf("back= %3d",vop_display_time[1]); */
/* TEST */
    }
/* TEST */
/* printf("\n"); */
/* TEST */
    return;
}

/***********************************************************CommentBegin******
 *
 * -- preview_gov_header
 *
 * Author :             
 *      Tomoko Aono (SHARP)
 *
 ***********************************************************CommentBegin******/
Void
CallingConvention
preview_gov_header(Bitstream *stream, Trace *trace)
{
    Int tmpvar;
    
    tmpvar = (Int) BitstreamReadBits(stream,32,"group_start_code",trace,CODE);
    tmpvar = (Int) BitstreamReadBits(stream,5,"Hour",trace,NUM);
    tmpvar = (Int) BitstreamReadBits(stream,6,"Minute",trace,NUM);
    tmpvar = (Int) BitstreamReadBits(stream,1,"Marker_bit",trace,NUM);
    tmpvar = (Int) BitstreamReadBits(stream,6,"Second",trace,NUM);
    tmpvar = (Int) BitstreamReadBits(stream,1,"closed_gov",trace,NUM);
    tmpvar = (Int) BitstreamReadBits(stream,1,"broken_link",trace,NUM);
    return;
} 

/***********************************************************CommentBegin******
 *
 * -- preview_vop_header
 *
 * Author :             
 *      Tomoko Aono (SHARP)
 *
 ***********************************************************CommentBegin******/
Int
CallingConvention
preview_vop_header(Bitstream *stream, Trace *trace)
{
    Bitstream *ref_stream;
    Int tmpvar;
    Int offset;
    
    ref_stream = clone_bitstream(stream);
    GetBitstreamPosition(&offset,stream);        
    
    tmpvar = (Int) BitstreamShowBits(ref_stream,VOP_START_CODE_LENGTH);
    if (tmpvar == GROUP_START_CODE) {
        preview_gov_header(ref_stream,trace);
        tmpvar = (Int) BitstreamShowBits(ref_stream,VOP_START_CODE_LENGTH);
    }  
    if (tmpvar == VOP_START_CODE) {
        tmpvar=(Int)BitstreamReadBits(ref_stream,VOP_START_CODE_LENGTH,"vop_start_code",trace,CODE);
        tmpvar=(Int)BitstreamReadBits(ref_stream,2,"vop_prediction_type",trace,FLAG);
    }
    else tmpvar= -1;
    SeekBitstream(offset,stream);        
    free(ref_stream);
    return(tmpvar);
}

Void
CallingConvention
FreeShapeVop(Vop **back_shape_vop,Vop **for_shape_vop)
{
    if ( *back_shape_vop != ( Vop * )NULL ) FreeVop( *back_shape_vop );
    if ( *for_shape_vop != ( Vop * )NULL ) FreeVop( *for_shape_vop );
}

/***********************************************************CommentBegin******
 *
 * -- CheckTime
 *
 ***********************************************************CommentEnd********/

Float
CallingConvention
CheckTime(Float time)
{
    Float     new_time;
    Int       int_value;
    Float     float_value;
    
    int_value   = (Int)time;
    float_value = time - (Float)int_value;
    
    if(float_value > 0.9 )
        new_time = (Float)((float)int_value + 1.00);
    else if( float_value <= 0.9 && float_value > 0.6)
        new_time = (Float)((float)int_value + 0.66);
    else if( float_value <= 0.6 && float_value > 0.3)
        new_time = (Float)((float)int_value + 0.33);
    else
        new_time = ((float)int_value);
    
    return new_time;
}
/* 10/27 for TPS */

/*** 10/9 TPS ***/
/***********************************************************CommentBegin******
 *
 * -- set_MB_TYPE2_type1_TPS
 *
 ***********************************************************CommentEnd********/
Int
CallingConvention
set_MB_TYPE2_type1_TPS(Vop* vop, Int vo_id,Int vol_id,
					   UChar **MB_TYPE2[MAX_NUM_VOS][MAX_NUM_VOLS])
{
    if(MB_TYPE2[ vo_id ][ vol_id ] != NULL)
        FreeMBtype2(GetVopHeight(vop), vo_id, vol_id,MB_TYPE2);
    AllocMBtype2(GetVopWidth(vop), GetVopHeight(vop),vo_id, vol_id,MB_TYPE2);
    PutVopId(vol_id, vop);
    
    return(GetVopHeight(vop));
}

/*** 10/27 for TPS */
/***********************************************************CommentBegin******
 *
 * -- AllocPRE_PRE_MBtype2
 *
 ***********************************************************CommentEnd********/
Void
CallingConvention
AllocPRE_PRE_MBtype2(Int width, Int height, Int vo_id, Int vol_id, Int fg,
					 UChar **stock_mb_type2[3][MAX_NUM_VOS][MAX_NUM_VOLS])
{
    Int                 w = width/MB_SIZE,              
        h = height/MB_SIZE,
        i;
    
    stock_mb_type2[fg][vo_id][vol_id] = (UChar **) malloc(h*sizeof(UChar*));
    for (i=0;i<h;i++)
        stock_mb_type2[fg][vo_id][vol_id][i]=(UChar *) malloc(w*sizeof(UChar));
}

Void
CallingConvention
FreePRE_PRE_MBtype2(Int height, Int vo_id, Int vol_id, Int fg,
					UChar **stock_mb_type2[3][MAX_NUM_VOS][MAX_NUM_VOLS])
{
    Int                 h = height/MB_SIZE,
        i;
    
/*** 10/31 TEST */
/*  printf("stock[%d] free %2d %10d\n",fg,h,stock_mb_type2[fg][vo_id][vol_id][0]); */
/* 10/31 TEST ***/
    for (i=0;i<h;i++)
/*    free(PRE_PRE_MB_TYPE2[vo_id][vol_id][i]); */
        free(stock_mb_type2[fg][vo_id][vol_id][i]); 
    
/*  free(PRE_PRE_MB_TYPE2[vo_id][vol_id]);
    PRE_PRE_MB_TYPE2[vo_id][vol_id] = NULL; */
    free(stock_mb_type2[fg][vo_id][vol_id]);
    stock_mb_type2[fg][vo_id][vol_id] = NULL;
}

Void
CallingConvention
InitAllPRE_PRE_MBtype2(UChar **stock_mb_type2[3][MAX_NUM_VOS][MAX_NUM_VOLS])
{
    Int i,j,fg;
    
    for (fg=0;fg<3;fg++)
        for (i=0;i<MAX_NUM_VOS;i++)
            for (j=0;j<MAX_NUM_VOLS;j++)
/*      PRE_PRE_MB_TYPE2[i][j] = NULL; */
                stock_mb_type2[fg][i][j] = NULL;
}

Void
CallingConvention
FreeAllPRE_PRE_MBtype2(	UChar **stock_mb_type2[3][MAX_NUM_VOS][MAX_NUM_VOLS],
						Int   stock_v_size2[3][MAX_NUM_VOS][MAX_NUM_VOLS]) 
{
    Int i,j,fg;
    for (fg=0;fg<3;fg++)
        for (i=0;i<MAX_NUM_VOS;i++)
            for (j=0;j<MAX_NUM_VOLS;j++)
/*      if (PRE_PRE_MB_TYPE2[i][j] != NULL) 
        FreePRE_MBtype2(PRE_V_SIZE2[i][j]*MB_SIZE,i,j); */
                if (stock_mb_type2[fg][i][j] != NULL) 
                    FreePRE_PRE_MBtype2(stock_v_size2[fg][i][j]*MB_SIZE,i,j,fg,
										stock_mb_type2);
}
/* 10/27 TPS ***/

/*** 10/22 SHARP FIX */
/***********************************************************CommentBegin******
 *
 * -- ChangeMBtype2_for_B_vop
 *
 * Author :             
 *      Ryuji KITAURA (SHARP)
 *
 ***********************************************************CommentBegin******/
/*Void
  ChangeMBtype2_for_B_vop(Int vo_id, Int vol_id, Int mode)
  {
  Int i,j;
  UChar **tmp_MB_TYPE2;
  Int   tmp_width, tmp_height;
  
  
  if(mode == 0){
  tmp_width  = PRE_PRE_H_SIZE2[vo_id][vol_id];
  tmp_height = PRE_PRE_V_SIZE2[vo_id][vol_id];
  
  /* alloc tmp_MB_TYPE2 */
/*      tmp_MB_TYPE2 = (UChar **) malloc(tmp_height*sizeof(UChar*));
        for (i = 0;i < tmp_height; i++)
        tmp_MB_TYPE2[i] = (UChar *) malloc(tmp_width*sizeof(UChar));*/
        
        /* save PRE_PRE_MB_TYPE2 */
/*      for(i = 0; i < tmp_height; i++)
        for(j = 0; j < tmp_width; j++)
        tmp_MB_TYPE2[i][j] = PRE_PRE_MB_TYPE2[vo_id][vol_id][i][j];*/
        
        /* change PRE_MB_TYPE2 to PRE_PRE_MB_TYPE2 */
/*      if(PRE_PRE_MB_TYPE2[ vo_id ][ vol_id ] != NULL)
        FreePRE_PRE_MBtype2(PRE_PRE_V_SIZE2[vo_id][vol_id], vo_id, vol_id);
        PRE_PRE_H_SIZE2[vo_id][vol_id] = PRE_H_SIZE2[vo_id][vol_id];
        PRE_PRE_V_SIZE2[vo_id][vol_id] = PRE_V_SIZE2[vo_id][vol_id];
        AllocPRE_PRE_MBtype2(PRE_PRE_H_SIZE2[ vo_id ][ vol_id ]*MB_SIZE,
        PRE_PRE_V_SIZE2[ vo_id ][ vol_id ]*MB_SIZE,vo_id, vol_id);
        for(i = 0; i<PRE_PRE_V_SIZE2[vo_id][vol_id]; i++)
        for(j = 0; j<PRE_PRE_H_SIZE2[vo_id][vol_id]; j++)
        PRE_PRE_MB_TYPE2[vo_id][vol_id][i][j] = PRE_MB_TYPE2[vo_id][vol_id][i][j];*/
        
        /* change tmp_MB_TYPE2 to PRE_MB_TYPE2 */
/*      if(PRE_MB_TYPE2[ vo_id ][ vol_id ] != NULL)
        FreePRE_MBtype2(PRE_V_SIZE2[vo_id][vol_id], vo_id, vol_id);
        PRE_H_SIZE2[vo_id][vol_id] = tmp_width;
        PRE_V_SIZE2[vo_id][vol_id] = tmp_height;
        AllocPRE_MBtype2(PRE_H_SIZE2[ vo_id ][ vol_id ]*MB_SIZE,
        PRE_V_SIZE2[ vo_id ][ vol_id ]*MB_SIZE,vo_id, vol_id);
        for(i = 0; i < tmp_height; i++)
        for(j = 0; j < tmp_width; j++)
        PRE_MB_TYPE2[vo_id][vol_id][i][j] = tmp_MB_TYPE2[i][j];*/
        
        /* free tmp_MB_TYPE2 */ 
/*      for (i = 0; i < tmp_height; i++)
        free(tmp_MB_TYPE2[i]);
        free(tmp_MB_TYPE2);
        }
        
        else if(mode == 1){
        tmp_width  = PRE_H_SIZE2[vo_id][vol_id];
        tmp_height = PRE_V_SIZE2[vo_id][vol_id];*/
        
        /* alloc tmp_MB_TYPE2 */
/*      tmp_MB_TYPE2 = (UChar **) malloc(tmp_height*sizeof(UChar*));
        for (i = 0;i < tmp_height; i++)
        tmp_MB_TYPE2[i] = (UChar *) malloc(tmp_width*sizeof(UChar));*/
        
        /* save PRE_MB_TYPE2 */
/*      for(i = 0; i < tmp_height; i++)
        for(j = 0; j < tmp_width; j++)
        tmp_MB_TYPE2[i][j] = PRE_MB_TYPE2[vo_id][vol_id][i][j];*/
        
        /* change PRE_PRE_MB_TYPE2 to PRE_MB_TYPE2 */
/*      if(PRE_MB_TYPE2[ vo_id ][ vol_id ] != NULL)
        FreePRE_MBtype2(PRE_V_SIZE2[vo_id][vol_id], vo_id, vol_id);
        PRE_H_SIZE2[vo_id][vol_id] = PRE_PRE_H_SIZE2[vo_id][vol_id];
        PRE_V_SIZE2[vo_id][vol_id] = PRE_PRE_V_SIZE2[vo_id][vol_id];
        AllocPRE_MBtype2(PRE_H_SIZE2[ vo_id ][ vol_id ]*MB_SIZE,
        PRE_V_SIZE2[ vo_id ][ vol_id ]*MB_SIZE,vo_id, vol_id);
        for(i = 0; i<PRE_V_SIZE2[vo_id][vol_id]; i++)
        for(j = 0; j<PRE_H_SIZE2[vo_id][vol_id]; j++)
        PRE_MB_TYPE2[vo_id][vol_id][i][j] = PRE_PRE_MB_TYPE2[vo_id][vol_id][i][j];*/
        
        /* change tmp_MB_TYPE2 to PRE_PRE_MB_TYPE2 */
/*      if(PRE_PRE_MB_TYPE2[ vo_id ][ vol_id ] != NULL)
        FreePRE_PRE_MBtype2(PRE_PRE_V_SIZE2[vo_id][vol_id], vo_id, vol_id);
        PRE_PRE_H_SIZE2[vo_id][vol_id] = tmp_width;
        PRE_PRE_V_SIZE2[vo_id][vol_id] = tmp_height;
        AllocPRE_PRE_MBtype2(PRE_PRE_H_SIZE2[ vo_id ][ vol_id ]*MB_SIZE,
        PRE_PRE_V_SIZE2[ vo_id ][ vol_id ]*MB_SIZE,vo_id, vol_id);
        for(i = 0; i < tmp_height; i++)
        for(j = 0; j < tmp_width; j++)
        PRE_PRE_MB_TYPE2[vo_id][vol_id][i][j] = tmp_MB_TYPE2[i][j];*/
        
        /* free tmp_MB_TYPE2 */ 
/*      for (i = 0; i < tmp_height; i++)
        free(tmp_MB_TYPE2[i]);
        free(tmp_MB_TYPE2);
        }
        }*/
/* 10/22 SHARP FIX ***/
