/*****************************************************************************
 *
 * This software module was originally developed by
 *
 *   Isabelle Corset (Philips / ACTS-MoMuSyS)
 *   Michael Wollborn (TUH / ACTS-MoMuSyS)
 *   Cor Quist (KPN / ACTS-MoMuSyS)
 *   Angel Pacheco (UPM / ACTS-MoMuSyS)
 *   J.Ignacio Ronda (UPM / ACTS-MoMuSyS)
 *
 * and edited by
 * 
 *   Paulo Nunes (IST / ACTS-MoMuSyS) 
 *   Robert Danielsen (Telenor / ACTS-MoMuSyS)
 *   Aasmund Sandvand (Telenor / ACTS-MoMuSyS)
 *
 * in the course of development of the MPEG-4 Video (ISO/IEC 14496-2) standard.
 * This software module is an implementation of a part of one or more MPEG-4
 * Video (ISO/IEC 14496-2) tools as specified by the MPEG-4 Video (ISO/IEC
 * 14496-2) standard.
 *
 * ISO/IEC gives users of the MPEG-4 Video (ISO/IEC 14496-2) standard free
 * license to this software module or modifications thereof for use in hardware
 * or software products claiming conformance to the MPEG-4 Video (ISO/IEC
 * 14496-2) standard.
 *
 * Those intending to use this software module in hardware or software products
 * are advised that its use may infringe existing patents. The original
 * developer of this software module and his/her company, the subsequent
 * editors and their companies, and ISO/IEC have no liability for use of this
 * software module or modifications thereof in an implementation. Copyright is
 * not released for non MPEG-4 Video (ISO/IEC 14496-2) Standard conforming
 * products.
 *
 * ACTS-MoMuSys partners retain full right to use the code for his/her own
 * purpose, assign or donate the code to a third party and to inhibit third
 * parties from using the code for non MPEG-4 Video (ISO/IEC 14496-2) Standard
 * conforming products. This copyright notice must be included in all copies or
 * derivative works.
 *
 * Copyright (c) 1996
 *
 *****************************************************************************/

/*****************************************************************************/
/*         This software module has been speed optimized                     */
/*                                                                           */
/*         by  Bruno Loret - France Telecom / CNET                           */
/*                                                                           */
/*         in the framework of the ACTS Project MoMuSys                      */
/*****************************************************************************/

/***********************************************************HeaderBegin*******
 *                                                                         
 * File:        text_decode.c
 *
 * Author:      Isabelle Corset (LEP) <corset@lep-philips.fr>
 * Created:     5-Mar-96
 *                                                                         
 * Description: Decoding VOP texture (and motion when combined mode)
 *              includes the Motion Compensation of the VOP in INTER.
 *
 * Notes:       
 *
 * Modified:
 *      09-May-1996 Paulo Nunes: Reformatted. New headers.
 *      13-AUG-1996 M.Wollborn: Changed for VM3.0 compliance
 *      16-SEP-1996 M.Wollborn: Changed INTRADCPREDDISABLE from
 *                      compiler flag to bitstream level
 *      23.10.96 Robert Danielsen: Added DC/AC prediction. Not tested.
 *      04.02.97 Noel O'Connor: removed all calls to assert()
 *      07-Feb-1997, M.Wollborn: changed in order to treat skipped MBs
 *                               correctly
 * 16.06.97 Angel Pacheco: unified the TRANSPARENT modes.
 *
 ***********************************************************HeaderEnd*********/

/************************    INCLUDE FILES    ********************************/

#include "io_generic.h"
#include "text_decode.h"
#include "text_decode_mb.h"
#include "mot_decode.h"
#include "mot_comp.h"
#include "vm_common_defs.h"

/***********************************************************CommentBegin******
 *
 * -- DecodeVopTextureIntra -- Decodes texture for INTRA VOPs
 *
 * Author :             
 *      Isabelle Corset (LEP) <corset@lep-philips.fr>
 *
 * Created :            
 *      26-Feb-96
 *
 * Purpose :            
 *      To perform texture decoding for INTRA coded VOPs.
 * 
 * Arguments in :       
 *      Bitstream *texture_bitstream    texture bitstream
 *      Trace  *trace                   flag for tracing the process.
 *
 *
 * Arguments in/out :   
 *      Vop * rec_vop                   alpha plane is supposed to be filled
 *                                      already / YUV components.
 *
 * Arguments out :      
 *
 *
 * Return values :      
 *      
 *
 * Side effects :       
 *      
 *
 * Description :        
 *      Notes: Separate Motion/Texture syntax is used for INTRA (skipped MBs
 *             available). 
 *
 *
 * See also :
 *      
 *
 * Modified : Cor Quist (KPN), parameter rec_width added to the call to
 *            GetMBText.
 *            11-06-96 Cor Quist (KPN) Added support for combined mot/text
 *                                     bitstream mode for I-VOP's.
 *            13-AUG-1996 M.Wollborn: Changed for VM3.0 compliance
 *            16-SEP-1996 M.Wollborn: Changed INTRADCPREDDISABLE from
 *                                    compiler flag to bitstream level
 *      23.10.96 Robert Danielsen: Added DC/AC prediction. Not tested.
 *      28.11.96 Robert Danielsen: Moved updating of QP_store.
 *
 ***********************************************************CommentEnd********/
#ifndef _WD_MODE_
Void DecodeVopTextureIntra(Bitstream * texture_bitstream, Trace *trace,
                           Vop * rec_vop)
{
    Int                 ef=0, *error_flag = &ef; 
    Int                 mbnum, x_pos, y_pos, i, j;
    Int                 rec_height;
    Int                 rec_width;
    Int                 MB_in_height, MB_in_width;
    Int                 B_in_width, B_in_height;
    Macroblock          *mblock;
    Image               *Y_rec, *U_rec, *V_rec, *A_rec;
    Int                 vop_quant,vop_shape,intra_dcpred_disable;
    Int                 skipped_flag;
    Image               *alpha_decisions,*MB_decisions;
    UInt                new_vop;
    SInt                *palpha_decisions=NULL,*pMB_decisions;
    Int                 MB_transp_pattern[4];
    Int                 ***DC_store;
    Int                 m, n, QP;
    SInt *QP_store;
    
    /*****
     *
     *  Set some useful parameters
     *
     *****/
    rec_height = GetVopHeight (rec_vop);
    rec_width = GetVopWidth (rec_vop);
    MB_in_height = rec_height / MB_SIZE;
    MB_in_width = rec_width / MB_SIZE;
    B_in_width = 2 * MB_in_width;
    B_in_height = 2 * MB_in_height;
    QP_store = (SInt *) GetImageData(GetVopQP(rec_vop));
    
    /* allocate space for 3D matrix to keep track of prediction values
       for DC/AC prediction */
    
    DC_store = (Int ***)calloc(MB_in_width*MB_in_height,
                               sizeof(Int **));
    for (i = 0; i < MB_in_width*MB_in_height; i++)
    {
        DC_store[i] = (Int **)calloc(6, sizeof(Int *));
        for (j = 0; j < 6; j++)
            DC_store[i][j] = (Int *)calloc(15, sizeof(Int));
    }
    
    intra_dcpred_disable = GetVopIntraACDCPredDisable(rec_vop);
    vop_shape = GetVopShape(rec_vop);
    
    alpha_decisions = AllocImage(B_in_width,B_in_height,SHORT_TYPE);
    MB_decisions    = AllocImage(MB_in_width,MB_in_height,SHORT_TYPE);
    
    /*****
     *
     *  Get the image fields of the VOP
     *
     *****/
    Y_rec = GetVopY (rec_vop);
    U_rec = GetVopU (rec_vop);
    V_rec = GetVopV (rec_vop);
    A_rec = GetVopA (rec_vop);
    
    /*****
     *
     *  Get VOP quantizer and allocate MB structure
     *
     *****/
    vop_quant = GetVopQuantizer (rec_vop);
    QP = vop_quant;
    
    mblock = (Macroblock *) malloc (sizeof (Macroblock));
    
    if (mblock == NULL)
    {
        printf (" Error allocating macroblock\n");
        exit(1);
    }
    
    printf("Start I VOP decoding \n");
    
    /*****
     *
     *  Get subsampled alpha decisions (if arbitrary shape),
     *  set motion decisions to "0" (since INTRA-VOP)
     *
     *****/
    if(vop_shape)
    {
        FillMB_decisions((SInt *)GetImageData(A_rec),rec_width,rec_height, 
                         MB_decisions,alpha_decisions);
        palpha_decisions = (SInt *)GetImageData(alpha_decisions);
    }
    else
    {
        SetConstantImage(MB_decisions,0);
        for(i=0; i<4; i++)
            MB_transp_pattern[i] = 0;
    }
    
    /*****
     *
     *  Loop over all MBs, decode texture of each MB
     *
     *****/
    new_vop = 1;
    pMB_decisions = (SInt *)GetImageData(MB_decisions);
    
    for (y_pos = 0; y_pos < MB_in_height; y_pos++)
        for (x_pos = 0; x_pos < MB_in_width; x_pos++)
        {
            mbnum = y_pos * MB_in_width + x_pos;
            
            /* Store the QP value for later use in AC prediction */
            QP_store[mbnum] = (SInt) QP;
            
            if(*(pMB_decisions + mbnum) == 0) /* not transparent MB */
            {
                if(vop_shape)
                {
                    GetTranspPattern(palpha_decisions,rec_width,rec_height,
                                     x_pos,y_pos,MB_transp_pattern);
                }
                GetMBText(INTRA_VOP,&new_vop,MB_transp_pattern,
                          intra_dcpred_disable,
                          vop_quant,texture_bitstream,mbnum,trace,MB_decisions,
                          mblock,&skipped_flag, error_flag, rec_width,
                          MB_in_width, MB_in_height,
                          DC_store, QP_store, QP, rec_vop);
                
                if (*error_flag == RETURN)
                    exit(1);
                
                if (skipped_flag == 1)
                    /* (skipped_flag==1) Clear the skipped MB */
                    MBzero (mblock);
                
            }
            else {
                MBzero (mblock);
                /* Fill DC_store with default coeff values */
                for (m = 0; m < 6; m++) {
                    DC_store[mbnum][m][0] = 128;
                    for (n = 1; n < 15; n++)
                        DC_store[mbnum][m][n] = 0;
                }
            }
            
            /* Store the QP value for later use in AC prediction */
/*          QP_store[mbnum] = QP; */
            FillVop (mblock, mbnum, Y_rec, U_rec, V_rec);
            
            
        }
    
    /* Free allocated memory for 3D matrix */
    
    for (i = 0; i < MB_in_width*MB_in_height; i++)
    {
        for (j = 0; j < 6; j++)
            free(DC_store[i][j]);
        free(DC_store[i]);
    }
    free(DC_store);
    
    FreeImage(alpha_decisions);
    FreeImage(MB_decisions);
    free ((Char *)mblock);
    
    return;
}
#endif




/***********************************************************CommentBegin******
 *
 * -- DecodeVopTextureInter -- Performs texture decoding for I-VOPs
 *
 * Author :             
 *      Isabelle Corset (LEP) <corset@lep-philips.fr>
 *
 * Created :            
 *      26-Feb-96
 *
 * Purpose :            
 *      To perform texture decoding for I-VOPs. The rsult is written into a
 *      temporal VOP which is lateron used for texture update (after MC, which
 *      is performed *after* the texture decoding since the skipped MB info.
 *      is required for MC)
 * 
 * Arguments in :       
 *      Bitstream * bitstream,  texture bitstream.
 *      Int trace,              flag for tracing the process.
 *
 *
 * Arguments in/out :   
 *      Image *MB_mot_decis     Image with MB modes for motion
 *      Vop *tmp_vop,           temporal VOP for texture information
 *      
 *
 * Arguments out :      
 *
 *
 * Return values :      
 *      
 *
 * Side effects :       
 *      
 *
 * Description :        
 *      
 *
 * See also :
 *      
 *
 * Modified : Cor Quist (KPN), parameter rec_width added to the call to
 *            GetMBText and GetMBMotText.
 *            15-AUG-1996 M.Wollborn: Changed for VM3.0 compliance
 *            16-SEP-1996 M.Wollborn: Changed INTRADCPREDDISABLE from
 *                                    compiler flag to bitstream level
 *      23.10.96 Robert Danielsen: Added DC/AC prediction. Not tested.
 *      28.11.96 Robert Danielsen: Moved updating of QP_store.
 *
 ***********************************************************CommentEnd********/
#ifndef _WD_MODE_
Void DecodeVopTextureInter(Bitstream * bitstream, Trace *trace, 
                           Image *MB_mot_decis, Vop *tmp_vop)
{
    Int                 x_pos, y_pos, mbnum;
    Int                 MB_in_height, MB_in_width, B_in_width, B_in_height;
    Int                 tmp_height;
    Int                 tmp_width;
    Macroblock          *mblock;
    Int                 skipped_flag, ef=0, *error_flag=&ef;
    Image               *Y_tmp, *U_tmp, *V_tmp, *A_tmp;
    SInt                *MBdecis_ptr;
    Int                 vop_quant, vop_type, vop_shape, i, j,
        intra_dcpred_disable;
    UInt                new_vop;
    Int                 MB_transp_pattern[4];
    Image               *alpha_decisions,*MB_decisions;
    SInt                *palpha_decisions=NULL,*pMB_decisions;
    Int                 ***DC_store;
    Int                 QP;
    Int                 m, n;
    SInt *QP_store;
    
    /*****
     *
     *  Get the images from the temporal VOP
     *
     *****/
    Y_tmp = GetVopY (tmp_vop);
    U_tmp = GetVopU (tmp_vop);
    V_tmp = GetVopV (tmp_vop);
    A_tmp = GetVopA (tmp_vop);
    
    /*****
     *
     *  Some useful initializations
     *
     *****/
    tmp_height = GetImageSizeY(GetVopY(tmp_vop));
    tmp_width  = GetImageSizeX(GetVopY(tmp_vop));
    MB_in_height = tmp_height / MB_SIZE;
    MB_in_width  = tmp_width / MB_SIZE;
    B_in_width = 2 * MB_in_width;
    B_in_height = 2 * MB_in_height;
    QP_store = (SInt *) GetImageData(GetVopQP(tmp_vop));
    
    /* allocate space for 3D matrix to keep track of prediction values
       for DC/AC prediction */
    
    DC_store = (Int ***)calloc(MB_in_width*MB_in_height,
                               sizeof(Int **));
    for (i = 0; i < MB_in_width*MB_in_height; i++)
    {
        DC_store[i] = (Int **)calloc(6, sizeof(Int *));
        for (j = 0; j < 6; j++)
            DC_store[i][j] = (Int *)calloc(15, sizeof(Int));
    }
    
    MBdecis_ptr = (SInt *) GetImageData (MB_mot_decis);
    
    intra_dcpred_disable = GetVopIntraACDCPredDisable(tmp_vop);
    
    vop_quant = GetVopQuantizer(tmp_vop);
    QP = vop_quant;
    vop_type  = GetVopPredictionType(tmp_vop);
    vop_shape = GetVopShape(tmp_vop);
    
    mblock    = (Macroblock *) malloc (sizeof (Macroblock));
    
    alpha_decisions = AllocImage(B_in_width,B_in_height,SHORT_TYPE);
    MB_decisions    = AllocImage(MB_in_width,MB_in_height,SHORT_TYPE);
    
    /*****
     *
     *  Get subsampled alpha decisions (if arbitrary shape),
     *  set motion decisions to "0" (since INTRA-VOP)
     *
     *****/
    if(vop_shape)
    {
        FillMB_decisions((SInt *)GetImageData(A_tmp),tmp_width,tmp_height, 
                         MB_decisions,alpha_decisions);
        palpha_decisions = (SInt *)GetImageData(alpha_decisions);
    }
    else
    {
        SetConstantImage(MB_decisions,0);
        for(i=0; i<4; i++)
            MB_transp_pattern[i] = 0;
    }
    
    /*****
     *
     *  Decode texture for all MBs and put in temporal VOP images
     *
     *****/
    mbnum   = 0;
    new_vop = 1;
    pMB_decisions = (SInt *)GetImageData(MB_decisions);
    
    while(mbnum < MB_in_width * MB_in_height)
    {
        x_pos = mbnum % MB_in_width;
        y_pos = mbnum / MB_in_width;
        
        /* Store the QP value for later use in AC prediction */
        QP_store[mbnum] = (SInt) QP;
        
        if(*(pMB_decisions + mbnum) != MBM_TRANSPARENT) /* apa transparent MB */
        {
            if(vop_shape)
                GetTranspPattern(palpha_decisions,tmp_width,tmp_height,
                                 x_pos,y_pos,MB_transp_pattern);
            
            GetMBText (vop_type,&new_vop,MB_transp_pattern,
                       intra_dcpred_disable,
                       vop_quant,bitstream,mbnum,trace,MB_mot_decis,mblock,
                       &skipped_flag,error_flag,tmp_width, MB_in_width,
                       MB_in_height, DC_store, QP_store, QP, tmp_vop);
            
            if(*error_flag == RETURN)
                exit(1);
            
            if (skipped_flag == 1){
                /* (skipped_flag == 1) */
                /* Clear the skipped MB */
                MBzero (mblock);
                *(MBdecis_ptr + mbnum) = MBM_SKIPPED;
            }
        }
        else {
            MBzero (mblock);
            /* Fill DC_store with default coeff values */
            for (m = 0; m < 6; m++) {
                DC_store[mbnum][m][0] = 128;
                for (n = 1; n < 15; n++)
                    DC_store[mbnum][m][n] = 0;
            }
        }    
        
        FillVop(mblock,mbnum,Y_tmp,U_tmp,V_tmp);
        
        
        mbnum++;
    }
    
    /* Free allocated memory for 3D matrix */
    
    for (i = 0; i < MB_in_width*MB_in_height; i++)
    {
        for (j = 0; j < 6; j++)
            free(DC_store[i][j]);
        free(DC_store[i]);
    }
    free(DC_store);
    
    FreeImage(alpha_decisions);
    FreeImage(MB_decisions);
    free((Char *)mblock);
    return;
}
#endif




/***********************************************************CommentBegin******
 *
 * -- VopTextureUpdate -- Add the decoded texture on the MC image
 *
 * Author :             
 *      Michael Wollborn (TUH)
 *
 * Created :            
 *      15-Aug-1996
 *
 * Purpose :            
 *      The decoded texture from the temporal VOP is added to the motion
 *      compensated image
 *
 * 
 * Arguments in : 
 *      Vop *tmp_vop    VOP with the decoded texture    
 *
 *
 * Arguments in/out :   
 *      Vop *curr_vop   motion compensated VOP to reconstruct
 *      
 *
 * Arguments out :      
 *
 *
 * Return values :      
 *      
 *
 * Side effects :       
 *      
 *
 * Description :        
 *      
 *
 * See also :
 *      
 *
 * Modified : 
 *          23.06.97 Minhua Zhou: added Trace
 *
 ***********************************************************CommentEnd********/

Void
CallingConvention
VopTextureUpdate(Vop *tmp_vop, Vop *curr_vop,Trace *trace)
{
    Image *Y_tmp, *U_tmp, *V_tmp, *Y_curr, *U_curr, *V_curr;
    
#ifdef _DEBUG
    //SInt mb_pred[6][64],mb_rec[6][64],mb_shape[6][64];
    Char level_3_trace=0;
    //SInt *y_data,*u_data,*v_data;
    Int npix=GetVopWidth(curr_vop);
    Int nlin=GetVopHeight(curr_vop);
    Int /*sum,*/MB_nb=-1;
    //Int i,j,k,i_mb,j_mb;
    Int MB_non_trans_count=0;
#endif
    
    /*****
     *
     *  Get the images from the VOPs and add texture update to 
     *  the motion compensated VOP images. Then clip the result
     *
     *****/
    trace->MB_tdata_predicted = 1;
    trace->MB_tdata_reconstructed = 1;
    
    
#ifdef _DEBUG_
    SetTraceLevelsDefault(trace);
    /*if (GetVopPredictionType(curr_vop)!=I_VOP)*/
        level_3_trace = trace->MB_tdata_predicted||trace->MB_tdata_reconstructed;
    if (level_3_trace)
    {
#if 0
        fprintf(trace->fp_trace,"\n\n *****************************************************\n");  
        fprintf(trace->fp_trace,"\n\n * Predictions and  reconstructions at time inc. %3d *\n",curr_vop->time_inc);
        fprintf(trace->fp_trace,"\n\n *****************************************************\n\n"); 
#endif
        fprintf(trace->fp_trace,"\n\n **************************************************\n");  
        fprintf(trace->fp_trace,"\n\n *        Predictions and  reconstructions        *\n");
        fprintf(trace->fp_trace,"\n\n **************************************************\n\n");
        for (i_mb=0;i_mb<nlin/16;i_mb++)
            for (j_mb=0;j_mb<npix/16;j_mb++)
            {
                if (MB_non_trans_count>=MB_trace_thres)
                    break;
                
                
                y_data = (SInt *)GetImageData(GetVopA(curr_vop));
                u_data = (SInt *)GetImageData(GetVopAuv(curr_vop));
                for (i=0;i<4;i++)
                    for (j=0;j<8;j++)
                        for (k=0;k<8;k++)
                            mb_shape[i][j*8+k] = y_data[(i_mb*16+(i/2)*8+j)*npix+j_mb*16+(i%2)*8+k];
                sum=0;
                for (i=0;i<4;i++)
                    for(j=0;j<64;j++) sum+=mb_shape[i][j];
                MB_nb++;
                if (sum==0)
                    continue;
                
                MB_non_trans_count++;
                for (j=0;j<8;j++)
                    for (k=0;k<8;k++)
                        mb_shape[4][j*8+k]=mb_shape[5][j*8+k]
                            = u_data[(i_mb*8+j)*npix/2+j_mb*8+k];
                
                y_data = (SInt *)GetImageData(GetVopY(tmp_vop));
                u_data = (SInt *)GetImageData(GetVopU(tmp_vop));
                v_data = (SInt *)GetImageData(GetVopV(tmp_vop));
                for (i=0;i<4;i++)
                    for (j=0;j<8;j++)
                        for (k=0;k<8;k++)
                            mb_pred[i][j*8+k] = y_data[(i_mb*16+(i/2)*8+j)*npix+j_mb*16+(i%2)*8+k];
                for (j=0;j<8;j++)
                    for (k=0;k<8;k++) {
                        mb_pred[4][j*8+k]= u_data[(i_mb*8+j)*npix/2+j_mb*8+k]; 
                        mb_pred[5][j*8+k]= v_data[(i_mb*8+j)*npix/2+j_mb*8+k]; 
                    }
                
                y_data = (SInt *)GetImageData(GetVopY(curr_vop));
                u_data = (SInt *)GetImageData(GetVopU(curr_vop));
                v_data = (SInt *)GetImageData(GetVopV(curr_vop));
                for (i=0;i<4;i++)
                    for (j=0;j<8;j++)
                        for (k=0;k<8;k++)
                            mb_rec[i][j*8+k] = y_data[(i_mb*16+(i/2)*8+j)*npix+j_mb*16+(i%2)*8+k];
                for (j=0;j<8;j++)
                    for (k=0;k<8;k++) {
                        mb_rec[4][j*8+k]= u_data[(i_mb*8+j)*npix/2+j_mb*8+k]; 
                        mb_rec[5][j*8+k]= v_data[(i_mb*8+j)*npix/2+j_mb*8+k]; 
                    }
                
                for (i=0;i<6;i++)
                    for (j=0;j<64;j++) {
                        mb_rec[i][j]+=mb_pred[i][j];
                        if (mb_rec[i][j]<0) mb_rec[i][j]=0;
                        else if (mb_rec[i][j]>255) mb_rec[i][j]=255;
                    }
                if (level_3_trace)
                    fprintf(trace->fp_trace,"\n\n____MB%d_____ at (%d,%d)\n",MB_nb,j_mb*16,i_mb*16);
                
                /*
                  if (trace->MB_tdata_predicted)
                  PrintOutMBData("MB_tdata_predicted",mb_shape,mb_pred,trace->fp_trace);
                  */
                
                if (trace->MB_tdata_reconstructed)
                    PrintOutMBData("MB_tdata_reconstructed",mb_shape,mb_rec,trace->fp_trace);
                
                fflush(trace->fp_trace);
            }
    }
#endif   
    
    
    Y_tmp = GetVopY(tmp_vop);
    U_tmp = GetVopU(tmp_vop);
    V_tmp = GetVopV(tmp_vop);
    
    Y_curr = GetVopY(curr_vop);
    U_curr = GetVopU(curr_vop);
    V_curr = GetVopV(curr_vop);
    
#if 0
#ifndef _OPTIMIZE_
    AddImage(Y_tmp,Y_curr,Y_curr);
    AddImage(U_tmp,U_curr,U_curr);
    AddImage(V_tmp,V_curr,V_curr);
    
    ClipImage (Y_curr);
    ClipImage (U_curr);
    ClipImage (V_curr);
#else
    /*printf("Add with Clip : Y \n");*/
    /*printf("Adresse : Y_tmp : %d \n",&Y_tmp);*/
    /*printf("Adresse : Y_curr : %d \n",&Y_curr);*/
    AddWithClipImage(Y_tmp,Y_curr,Y_curr);
    
    /*printf("Add with Clip : U \n");*/
    /*printf("Adresse : U_tmp : %d \n",&U_tmp);*/
    /*printf("Adresse : U_curr : %d \n",&U_curr);*/
    AddWithClipImage(U_tmp,U_curr,U_curr);
    
    /*printf("Add with Clip : V \n");*/
    /*printf("Adresse : V_tmp : %d \n",&V_tmp);*/
    /*printf("Adresse : V_curr : %d \n",&V_curr);*/
    AddWithClipImage(V_tmp,V_curr,V_curr);
#endif
#endif
    
    return;
}


/***********************************************************CommentBegin******
 *
 * -- FillVop -- Fills the mbnum-th macroblock of VOP y,u,v components
 *
 * Author :             
 *      Isabelle Corset (LEP) <corset@lep-philips.fr>
 *
 * Created :            
 *      26-Feb-96
 *
 * Purpose :            
 *      To fills the mbnum-th macroblock of VOP y,u,v components.
 * 
 * Arguments in :       
 *      Macroblock * mblock,    decoded macroblock.
 *      Int mbnum,              macroblock number.
 
 *
 * Arguments in/out :   
 *      
 *
 * Arguments out :      
 *      Image *y_chan,          Y component of the VOP to fill.
 *      Image *u_chan,          U component of the VOP to fill.
 *      Image *v_chan,          V component of the VOP to fill.
 *
 *
 * Return values :      
 *      
 *
 * Side effects :       
 *      
 *
 * Description :        
 *      
 *
 * See also :
 *      
 *
 * Modified :           
 *      
 *
 ***********************************************************CommentEnd********/
#if 0
Void
CallingConvention
FillVop (Macroblock * mblock,
         Int mbnum,
         Image * y_chan,
         Image * u_chan,
         Image * v_chan)

{
    Int                 lumstart, chrstart;
    Int                 n, m;
    UInt                MB_in_width;
    UInt                image_x;
    SInt               *y_point, *u_point, *v_point;
    
    image_x = GetImageSizeX (y_chan);
    
    MB_in_width = image_x / MB_SIZE;
    lumstart = (mbnum / MB_in_width) * image_x * MB_SIZE +
        (mbnum % MB_in_width) * MB_SIZE;
    
    chrstart = (mbnum / MB_in_width) * image_x / 2 * MB_SIZE / 2 +
        (mbnum % MB_in_width) * MB_SIZE / 2;
    
    y_point = (SInt *) GetImageData (y_chan) + lumstart;
    u_point = (SInt *) GetImageData (u_chan) + chrstart;
    v_point = (SInt *) GetImageData (v_chan) + chrstart;
    
    for (n = 0; n < MB_SIZE; n++)
        for (m = 0; m < MB_SIZE; m++)
            *(y_point + n * image_x + m) = mblock->lum[n][m];
    
    for (n = 0; n < (MB_SIZE >> 1); n++)
        for (m = 0; m < (MB_SIZE >> 1); m++)
        {
            *(u_point + n * image_x / 2 + m) = mblock->Cr[n][m];
            *(v_point + n * image_x / 2 + m) = mblock->Cb[n][m];
        }
    
}                               /* FillVop */
#endif

/***********************************************************CommentBegin******
 *
 * -- subsamp_alpha_dec --
 *
 * Author :
 *      UPM - J.Ignacio Ronda / Angel Pacheco
 *
 * Created :
 *      2/2/96
 *
 * Purpose :
 *      generates an array with the alpha plane subsampled in block
 *      pixel units. This array is used to see if the block/macroblock
 *      is transparent
 *
 * Arguments in :
 *      SInt*  alpha,          alpha plane data
 *      Int     br_width,        width of the alpha plane data
 *      Int     br_height        height  of the alpha plane data
 *
 * Arguments in/out :
 *      SInt*  alpha_sub,      subsampled alpha data
 *
 * Arguments out :
 *
 *
 * Return values :
 *      none
 *
 * Side effects :
 *
 *
 * Description :
 *      It does not allocate memory for the subsampled array
 *
 * See also :
 *
 *
 * Modified :
 *
 *
 ***********************************************************CommentEnd********/
#if 0
Void
CallingConvention
subsamp_alpha_dec(
    SInt  *alpha,       /* <-- alpha plane data                */
    Int    br_width,     /* <-- width of the alpha plane data   */
    Int    br_height,    /* <-- height  of the alpha plane data */
    SInt  *alpha_sub    /* --> subsampled alpha data           */
    )   
{
    Int  h, v;
    
    for (v=0;v<br_height/B_SIZE;v++)
        for (h=0;h<br_width/B_SIZE;h++)
            alpha_sub[v*br_width/B_SIZE+h] = 0;
    
    for (v=0;v<br_height;v++)
        for (h=0;h<br_width;h++)
            alpha_sub[(v/8)*br_width/B_SIZE+(h/B_SIZE)] =
                alpha_sub[(v/B_SIZE)*br_width/B_SIZE+(h/B_SIZE)] ||
                alpha[v*br_width+h];
    
    for (v=0;v<br_height/B_SIZE;v++)
        for (h=0;h<br_width/B_SIZE;h++)
            if (alpha_sub[v*br_width/B_SIZE+h]==0)
                alpha_sub[v*br_width/B_SIZE+h] = MBM_TRANSPARENT;
            else 
                alpha_sub[v*br_width/B_SIZE+h] = 1;
    
}
#endif

/***********************************************************CommentBegin******
 *
 * -- BoutVop -- Determines which blocks contains points of the VOP
 *
 * Author :             
 *      Isabelle Corset (LEP) <corset@lep-philips.fr>
 *
 * Created :            
 *      27-Feb-96
 *
 * Purpose :            
 *      To determine which blocks contains points of the VOP.
 * 
 * Arguments in :       
 *      Image * A_chan,          alpha channel of a VOP.
 *
 *
 * Arguments in/out :   
 *      
 *
 * Arguments out :      
 *      Image * B_mot_decis,     block-based motion decision (2: MBM_TRANSPARENT).
 *
 * Return values :      
 *      
 *
 * Side effects :       
 *      
 *
 * Description :        
 *      
 *
 * See also :
 *      
 *
 * Modified :           
 *      
 *
 ***********************************************************CommentEnd********/

#if 0
Void
CallingConvention
BoutVop (Image * B_mot_decis,
         Image * A_chan)

{
    UInt                vop_size;
    Int                 i, j, b_ind, total_B, alpha_x, B_in_width;
    SInt               *decis_ptr;
    SInt               *alpha_ptr, *begin_add;
    SInt                one_point, one_point_flag;
    
    vop_size = GetImageSize (A_chan);
    total_B = vop_size / (B_SIZE * B_SIZE);
    alpha_ptr = (SInt *) GetImageData (A_chan);
    decis_ptr = (SInt *) GetImageData (B_mot_decis);
    alpha_x = GetImageSizeX (A_chan);
    B_in_width = alpha_x / B_SIZE;
    
    for (b_ind = 0; b_ind < total_B; b_ind ++)
    {
        i = 0;
        one_point_flag = 0;
        begin_add = alpha_ptr + (b_ind / B_in_width) * alpha_x * B_SIZE
            + (b_ind % B_in_width) * B_SIZE;
        for (i = 0; i< B_SIZE; i++)
        {
            for (j = 0; j< B_SIZE; j++)
            {
                one_point = *(begin_add + j);
                if (one_point != 0)
                {
                    one_point_flag = 1;
                    break;
                }
            }
            if (one_point_flag) break;
            begin_add += alpha_x; /* next line ! */
        }
        
        if (!one_point_flag) *(decis_ptr + b_ind) = MBM_TRANSPARENT;
    }
}                               /* BoutVop */
#endif

/***********************************************************CommentBegin******
 *
 * -- BToMBdecis -- Converts block-based decisions into MB-based decisions
 *
 * Author :             
 *      Isabelle Corset (LEP) <corset@lep-philips.fr>
 *
 * Created :            
 *      27-Feb-96
 *
 * Purpose :            
 *      To convert block-based decisions into MB-based decisions for texture
 *      decoding in the separate mode.
 * 
 * Arguments in :       
 *      Image * B_mot_decis,     block-based motion decision.
 *
 *
 * Arguments in/out :   
 *      
 *
 * Arguments out :      
 *      Image * MB_mot_decis,   MB-based motion decision (MBM_TRANSPARENT not
 *                              marked).
 *
 *
 * Return values :      
 *      
 *
 * Side effects :       
 *      
 *
 * Description :        
 *      
 *
 * See also :
 *      
 *
 * Modified :           
 *      
 *
 ***********************************************************CommentEnd********/

#if 0
Void
CallingConvention
BToMBdecis (Image * B_decis,
            Image * MB_decis)

{
    Int                 mbnum, B_in_width, MB_in_width, totalMB, block, ypos,
        xpos, bpos, offset;
    SInt               *MBdecis_ptr, *MBdecis_ini, *Bdecis_ptr, *Bdecis_ini;
    
    totalMB = GetImageSize (MB_decis);
    B_in_width = GetImageSizeX (B_decis);
    MB_in_width = GetImageSizeX (MB_decis);
    MBdecis_ini = (SInt *) GetImageData (MB_decis);
    Bdecis_ini = (SInt *) GetImageData (B_decis);
    
    for (mbnum = 0; mbnum < totalMB; mbnum++)
    {
        ypos = (mbnum / MB_in_width) * 2;
        xpos = (mbnum % MB_in_width) * 2;
        bpos = ypos * B_in_width + xpos;
        Bdecis_ptr = Bdecis_ini + bpos;
        MBdecis_ptr = MBdecis_ini + mbnum;
        block = 0;
        while (block < 4)
        {
            offset = (block >> 1) * B_in_width + (block & 0x1);
            if (*(Bdecis_ptr + offset) != MBM_TRANSPARENT)
            {
                *MBdecis_ptr = *(Bdecis_ptr + offset);
                block = 4;
            }
            else
            {
                block++;
            }
        }
        
    }
    
}                               /* BToMBdecis */
#endif

/***********************************************************CommentBegin******
 *
 * -- MB_BToMBdecis -- Converts block-based decisions into MB-based decisions
 *
 * Author : Cor Quist (KPN)             
 *
 *
 * Created :            
 *      13-Jun-96
 *
 * Purpose :            
 *      To convert block-based decisions into MB-based decisions for texture
 *      decoding in the combined motion/texture mode.
 * 
 * Arguments in :       
 *      Image * B_mot_decis,     block-based motion decision.
 *      Int mbnum, macroblock number
 *
 * Arguments in/out :   
 *      
 *
 * Arguments out :      
 *      Image * MB_mot_decis,   MB-based motion decision (MBM_TRANSPARENT not
 *                              marked).
 *
 *
 * Return values :      
 *      
 *
 * Side effects :       
 *      
 *
 * Description :        
 *      
 *
 * See also :
 *      
 *
 * Modified :           
 *      
 *
 ***********************************************************CommentEnd********/

#if 0
Void
CallingConvention
MB_BToMBdecis (Image * B_decis,
               Image * MB_decis,
               Int mbnum)

{
    Int                 B_in_width, MB_in_width, totalMB, block, ypos,
        xpos, bpos, offset;
    SInt               *MBdecis_ptr, *MBdecis_ini, *Bdecis_ptr, *Bdecis_ini;
    
    totalMB = GetImageSize (MB_decis);
    B_in_width = GetImageSizeX (B_decis);
    MB_in_width = GetImageSizeX (MB_decis);
    MBdecis_ini = (SInt *) GetImageData (MB_decis);
    Bdecis_ini = (SInt *) GetImageData (B_decis);
    
    /* check mbnum out of range */
    MOMCHECK(mbnum >= 0 && mbnum < totalMB);
    
    ypos = (mbnum / MB_in_width) * 2;
    xpos = (mbnum % MB_in_width) * 2;
    bpos = ypos * B_in_width + xpos;
    Bdecis_ptr = Bdecis_ini + bpos;
    MBdecis_ptr = MBdecis_ini + mbnum;
    block = 0;
    while (block < 4)
    {
        offset = (block >> 1) * B_in_width + (block & 0x1);
        if (*(Bdecis_ptr + offset) != MBM_TRANSPARENT)
        {
            *MBdecis_ptr = *(Bdecis_ptr + offset);
            block = 4;
        }
        else
        {
            block++;
        }
    }
    
    
}                               /* MB_BToMBdecis */
#endif
