/*****************************************************************************
 *
 * This software module was originally developed by
 *
 *   Isabelle Corset (Philips / ACTS-MoMuSyS)
 *
 * and edited by
 * 
 *   Paulo Nunes (IST / ACTS-MoMuSyS)
 *   Cor Quist (KPN / ACTS-MoMuSyS)
 *   Jan De Lameillieure (HHI / ACTS-MoMuSyS)
 *   Michael Wollborn (TUH / ACTS-MoMuSyS)
 *   Robert Danielsen (Telenor / ACTS-MoMuSyS)
 *
 * in the course of development of the MPEG-4 Video (ISO/IEC 14496-2) standard.
 * This software module is an implementation of a part of one or more MPEG-4
 * Video (ISO/IEC 14496-2) tools as specified by the MPEG-4 Video (ISO/IEC
 * 14496-2) standard.
 *
 * ISO/IEC gives users of the MPEG-4 Video (ISO/IEC 14496-2) standard free
 * license to this software module or modifications thereof for use in hardware
 * or software products claiming conformance to the MPEG-4 Video (ISO/IEC
 * 14496-2) standard.
 *
 * Those intending to use this software module in hardware or software products
 * are advised that its use may infringe existing patents. The original
 * developer of this software module and his/her company, the subsequent
 * editors and their companies, and ISO/IEC have no liability for use of this
 * software module or modifications thereof in an implementation. Copyright is
 * not released for non MPEG-4 Video (ISO/IEC 14496-2) Standard conforming
 * products.
 *
 * ACTS-MoMuSys partners retain full right to use the code for his/her own
 * purpose, assign or donate the code to a third party and to inhibit third
 * parties from using the code for non MPEG-4 Video (ISO/IEC 14496-2) Standard
 * conforming products. This copyright notice must be included in all copies or
 * derivative works.
 *
 * Copyright (c) 1996
 *
 *****************************************************************************/

/*****************************************************************************/
/*         This software module has been speed optimized                     */
/*                                                                           */
/*         by  Bruno Loret - France Telecom / CNET                           */
/*                                                                           */
/*         in the framework of the ACTS Project MoMuSys                      */
/*****************************************************************************/

/***********************************************************HeaderBegin*******
 *
 * File:        text_decode_mb.c
 *
 * Author:      Isabelle Corset (LEP) <corset@lep-philips.fr>
 * Created:     5-Mar-96
 *
 * Description: Decoding of the Macroblock Layer
 *
 * Notes:
 *
 * Modified:    9-May-96 Paulo Nunes: Reformatted. New headers.
 *             23-May-96 Cor Quist : Added the INTRA DC prediction
 *             13-Jun-96 Cor Quist : Bug fixes for combined mode
 *             31-Jul-96 Jan De Lameillieure (HHI) : added MPEG-1/-2 
 *                       like inverse quantisation
 *             04-OCT-96 M.Wollborn: Removed reading of NO_DCT_FLAG from the
 *                                   bitstream for I-VOPs (GetMBText)
 *      23.10.96 Robert Danielsen: Added DC/AC prediction. Not tested.
 *                      Two new functions: doDCACrecon(), nullfill().
 *      22-APR-97 Michael Wollborn: changed name of BlockDequant to 
 *                      BlockDequantH263 and changed
 *                      stuff for MPEG-like quantization
 *      23-APR-97 Michael Wollborn: Removed function "GetMBMotText()"
 *                                  since it is no longer used
 *      04.08.97 Minhua Zhou: added intra_dc_vlc_thr (switched)
 *
 ***********************************************************HeaderEnd*********/

/************************    INCLUDE FILES    ********************************/

#include "text_decode_mb.h"
#include "text_quant.h"
#include "zigzag.h"             /* Added 14-NOV-1996, MW */

/*ok*/static UInt
CallingConvention
VlcDecIntraDCPredSize (Bitstream *stream, Int *error_flag, Int compnum);
/*ok*/static Void
CallingConvention
nullfill(Int pred[]);

/***********************************************************CommentBegin******
 *
 * -- GetMBText --  Decode one macroblock of texture (INTRA or INTER in separate
 *                  motion/texture mode).
 *
 * Author :
 *      Isabelle Corset (LEP) <corset@lep-philips.fr>
 *
 * Created :
 *      28-Feb-96  VM 1.1 Texture decoding
 *
 * Purpose :
 *       To decode one macroblock of texture (INTRA or INTER in separate
 *       motion/texture mode).
 *
 * Arguments in :
 *      Int vop_type,                   vop INTRA/INTER.
 *      Int intra_dcpred_disable        enable/disable pred. of INTRA DC
 *      Int quant_step,                 vop quantizer step (used for all MBs).
 *      Bitstream * texture_bits,       Image made of texture parameters.
 *      Int mbnum,                      macroblock number.
 *      Int trace,                      flag for tracing the process.
 *      Int *motion_mode,               already filled by MotionDecode.
 *      Int MB_width,
 *      Int MB_height,
 *      Int DC_store[][6][15],          Stores DCAC prediction values
 *      SInt *QP_store,                 Stores QPs for every MB
 *      Int QP                          Current QPn
 *
 * Arguments in/out :
 *
 *
 * Arguments out :
 *      Macroblock *mblock,             decoded macroblock.
 *      Int *skipped_flag,              1 = macroblock is skipped.
 *      Int *error_flag,                set if an error occured.
 *
 *
 * Return values :
 *
 *
 * Side effects :
 *
 *
 * Description :
 *      Notes: skipped macroblock for the transparent MBs and skipped MBs
 *
 * See also :
 *
 *
 * Modified : Cor Quist, INTRA DC prediction added. 23-May-96
 *      31-JUL-96 Jan De Lameillieure (HHI) - added MPEG-1/-2 like inverse quantisation
 *      03-Sep-96 : added clipping before IDCT (VM3.1)
 *      12-SEP-96 : fixed clipping bug (8x8 loop instead of 16x16 loop)
 *                  (M.Wollborn)
 *      16-SEP-96 : (M.Wollborn) Changed INTRADCPREDDISABLE from
 *                               compiler flag to bitstream level
 *      04-OCT-96 : (M.Wollborn) Removed reading of NO_DCT_FLAG from the
 *                               bitstream for I-VOPs
 *      23.10.96 Robert Danielsen: Added DC/AC prediction. Not tested.
 *      04.11.96 Robert Danielsen: Added parameter to VlcGetBlock call, for
 *                      signalling intra luma block.
 *      06.11.96 Robert Danielsen: Added reading of MCBPC for separate mode,
 *                              with DQUANT & CBPC.
 *      09.12.96 Robert Danielsen: Fixed a small bug. The filling of
 *                      DC_store with default values for INTER MB's was
 *                      done in the INTRA-loop, thus not having any effect...
 *      09.01.97 Noel Brady/Robert Danielsen: Added lines to fill DC-store
 *                      with default values when no_dct_flag == 1
 *      28.01.97 Robert Danielsen: Changed zigzag-scanning.
 *      27.02.97 Robert Danielsen: Changed to using Intra VLC table for chroma
 *                      coefficients.
 *      27.02.97 Robert Danielsen: Adding parameter to calls to quant-routines
 *                      needed for nonlinear quantization.
 *      22-APR-97 Michael Wollborn: changed name of BlockDequant to 
 *                      BlockDequantH263 and changed
 *                      stuff for MPEG-like quantization
 *      21-May-97 Minhua Zhou: idctref is relpaced by BlockIDCT
 *
 *      04.11.97 Minhua Zhou: updated ACDC prediction
 ***********************************************************CommentEnd********/
#ifndef _WD_MODE_
Void
GetMBText (Int vop_type,
           UInt *new_vop,
           Int transp_pattern[],
           Int intra_dcpred_disable,
           Int quant_step,
           Bitstream * texture_bits,
           Int mbnum,
           Trace *trace,
           Image * motion_mode,
           Macroblock * mblock,
           Int * skipped_flag,
           Int * error_flag,
           Int x_width,
           Int MB_width,
           Int MB_height,
           Int ***DC_store,
           SInt *QP_store,
           Int QP,
           Vop *rec_vop
           )
{
    UInt                no_dct_flag, CBPY, CBPC, CBP, mode, DC_coeff;
    Int                 comp, curr_motion, pred_DC_coeff;
    Int                 DCT_block[B_SIZE * B_SIZE], q_block[B_SIZE * B_SIZE],
        r_block[B_SIZE*B_SIZE];
    Int block_buf[B_SIZE][B_SIZE];
    SInt                *decis_ptr;
    Int                 x_pos = mbnum % MB_width;
    Int                 y_pos = mbnum / MB_width;
    Int                 m, n;
    Int ACpred_flag=-1;
    Int DQUANT_flag;
    Int MCBPC;
    Int DQUANT;
    Int pcoeff[B_SIZE * B_SIZE];
    Int switched;
    
    Int last_non_null_coeff;
    
    
    trace->trace = trace->MB_header;
    
    if (trace->trace)
    {
        fprintf (trace->fp_trace, "\n>>>   Decoding Macroblock %d ........- Separate Mot/Text\n",
                 (Int)mbnum);
    }
    
    if(vop_type == INTRA_VOP)
        no_dct_flag = 0;
    else
        no_dct_flag = BitstreamReadBits (texture_bits, 1,"NO DCT FLAG", 
                                         trace, FLAG);
    
    
    decis_ptr = (SInt *) GetImageData (motion_mode);
    curr_motion = *(decis_ptr + mbnum);
    
    /*added by Minhua Zhou 04.08.97 */
    switched = IntraDCSwitch_Decision(curr_motion? MODE_INTER:MODE_INTRA,
                                      GetVopIntraDCVlcThr(rec_vop),
                                      QP);
    
    if ((curr_motion == INTRA) && (no_dct_flag))
    {
        /* skipped macroblock */
        *skipped_flag = 1;
        /* Fill DC_store with default coeff values */
        for (m = 0; m < 6; m++) {
            DC_store[mbnum][m][0] = 128*8;
            for (n = 1; n < 15; n++)
                DC_store[mbnum][m][n] = 0;
        }
    }
    else if(no_dct_flag == 1)
    {
        *skipped_flag = 0;
        MBzero (mblock);
        /* Fill DC_store with default coeff values */
        for (m = 0; m < 6; m++) {
            DC_store[mbnum][m][0] = 128*8;
            for (n = 1; n < 15; n++)
                DC_store[mbnum][m][n] = 0;
        }
    }
    else
    {
        /* coded macroblock */
        *skipped_flag = 0;
        if (curr_motion == 0)
        {
            mode = MODE_INTRA;
        }
        else
        {
            mode = MODE_INTER;
        }
        
        MCBPC = VlcDecMCBPC_sep(texture_bits, error_flag, trace);
        
        DQUANT_flag = (MCBPC & 4) >> 2;
        CBPC = MCBPC & 3;
        
        if (mode == MODE_INTRA && intra_dcpred_disable == 0)
        {
            ACpred_flag = BitstreamReadBits(texture_bits, 1, "ACpred_flag",
                                            trace, FLAG);
        }
        
        CBPY = VlcDecCBPY (texture_bits, (mode == MODE_INTRA), transp_pattern,error_flag, trace);
        if (*error_flag == RETURN)
        {
            printf ("Error decoding CBPY of macroblock %d\n", (Int)mbnum);
            return;
        }
        
        /*if (mode == MODE_INTRA)
          {
          CBPY = CBPY ^ 15;
          }  removed by minhua Zhou 15-05-97 */
        
        CBP = CBPY << 2 | CBPC;
        
        if (DQUANT_flag == 1)
            DQUANT = BitstreamReadBits(texture_bits, 2, "DQUANT", trace, NUM);
        
        trace->trace = trace->MB_texture;
        if(trace->trace)
            fprintf(trace->fp_trace, "----------\n");
        
        MBzero (mblock);
        
        /*****
         *
         *        Decoding of the 4 luminance blocks (depending on transparent pattern)
         *
         *****/
        for (comp=0; comp<4; comp++)
        {
            Bzero (q_block);
            
            if(transp_pattern[comp]==0) /* No decoding of transparent Y blocks */
            {
                if (mode == MODE_INTRA) {
                    if (switched==0) {
                        if(intra_dcpred_disable)        /* No INTRA DC prediction */
                        {
                            DC_coeff = BitstreamReadBits (texture_bits, 8, "DC coeff",
                                                          trace, NUM);
                            
                            if (DC_coeff == 128) {
                                printf ("Illegal DC coeff: 1000000\n");
                                exit(1);
                            }
                            if (DC_coeff == 255)
                                DC_coeff = 128;
                            
                            q_block[0] = DC_coeff;
                            
                        }
                        else                            /* INTRA DC prediction (I-MBs) */
                        {
                            /* This must be Int not UInt MW 15-NOV-1996 */
                            pred_DC_coeff = DecodePredictedIntraDC(comp,
                                                                   texture_bits,
                                                                   trace, 
                                                                   NUM);
                            q_block[0] = pred_DC_coeff;
                        }
                    }
                    
                    if (CBP & (1 << (BLOCK_CNT - 1 - comp)))
                        VlcGetBlock(texture_bits, q_block, INTRA,
                                    trace, error_flag, 1, GetVopReverseVlc(rec_vop),switched,&last_non_null_coeff);
                    
                    /* Do the DC/AC prediction, changing the qcoeff values as
                       appropriate */
                    if (intra_dcpred_disable == 0)
                        doDCACrecon(q_block, 64, DC_store, QP_store, QP,
                                    x_pos, y_pos, MB_width, ACpred_flag, comp);
                    else
                    {
                        /* Ordinary inverse zigzag-scanning */
                        for (m = 0; m < 64; m++)
                            pcoeff[m] = q_block[zigzag[m]];
                        for (m = 0; m < 8; m++)
                            for (n = 0; n < 8; n++)
                                q_block[m*8+n] = pcoeff[m*8+n];
                    }
                    
                    /* Store the qcoeff-values needed later for prediction */
                    for (m = 0; m < 8; m++) {
                        DC_store[mbnum][comp][m] = q_block[m];
                    }
                    for (m = 0; m < 7; m++) {
                        DC_store[mbnum][comp][m+8] = q_block[(m+1)*8];
                    }
                    
                    /* MW 22-APR-1997 Changes for MPEG dequantization */
                    if(GetVopQuantType(rec_vop))
                        BlockDequantMPEG(q_block,quant_step,INTRA,1,
                                         GetVopIntraQuantMat(rec_vop),DCT_block, last_non_null_coeff
                                         );
                    else
                        BlockDequantH263(q_block,quant_step,INTRA,1,DCT_block, last_non_null_coeff
                                         );
                    
                    DC_store[mbnum][comp][0] = DCT_block[0];
                    BlockIDCT (DCT_block, block_buf);
                    for (m=0;m<8;m++)
                        for (n=0;n<8;n++)
                            r_block[m*8+n]= block_buf[m][n];
                    
                    
                }
                else /* MODE INTER */
                {
                    if (CBP & (1 << (BLOCK_CNT - 1 - comp)))
                        VlcGetBlock (texture_bits, q_block, INTER, trace,
                                     error_flag, 0, GetVopReverseVlc(rec_vop),0,&last_non_null_coeff);
                    
                    /* Inverse zigzag-scanning also here 12-11-96 MW */
                    for (m = 0; m < 64; m++)
                        pcoeff[m] = q_block[zigzag[m]];
                    
                    for (m = 0; m < 8; m++)
                        for (n = 0; n < 8; n++)
                            q_block[m*8+n] = pcoeff[m*8+n];
                    
                    /* Fill DC_store with default coeff values */
                    DC_store[mbnum][comp][0] = 128*8;
                    for (n = 1; n < 15; n++)
                        DC_store[mbnum][comp][n] = 0;
                    
                    /* MW 22-APR-1997 Changes for MPEG dequantization */
                    if(GetVopQuantType(rec_vop))
                        BlockDequantMPEG(q_block,quant_step,INTER,1,
                                         GetVopNonintraQuantMat(rec_vop),
                                         DCT_block,last_non_null_coeff);
                    else
                        BlockDequantH263(q_block,quant_step,INTER,1,DCT_block,last_non_null_coeff);
                    
                    
                    
                    BlockIDCT (DCT_block, block_buf);
                    for (m=0;m<8;m++)
                        for (n=0;n<8;n++)
                            r_block[m*8+n]= block_buf[m][n];
                    
                }
            }
            else /* clear r_block if transparent block */
            {
                Bzero(r_block);
                /* Fill DC_store with default coeff values */
                DC_store[mbnum][comp][0] = 128*8;
                for (n = 1; n < 15; n++)
                    DC_store[mbnum][comp][n] = 0;
            }
            
            PutBlock (comp, r_block, mblock);
        }
        
        /*****
         *
         *        Decoding of 2 chrominance blocks (not dep. on 
         *        transparent pattern)
         *
         *****/
        for (comp=4; comp<6; comp++)
        {
            Bzero (q_block);
            
            if (mode == MODE_INTRA)
            {
                if (switched==0) {
                    if(intra_dcpred_disable)    /* No INTRA DC prediction */
                    {
                        DC_coeff = BitstreamReadBits (texture_bits, 8, "DC coeff",
                                                      trace, NUM);
                        
                        if (DC_coeff == 128)
                        {
                            printf ("Illegal DC coeff: 1000000\n");
                            exit(1);
                        }
                        
                        if (DC_coeff == 255)
                            DC_coeff = 128;
                    }
                    else                 /* INTRA DC prediction (I-MBs) */
                    {
                        DC_coeff = DecodePredictedIntraDC(comp,
                                                          texture_bits,
                                                          trace,
                                                          NUM);
                    }   
                    q_block[0] = DC_coeff;
                }
                if (CBP & (1 << (BLOCK_CNT - 1 - comp)))
                    VlcGetBlock (texture_bits, q_block, INTRA, trace,
                                 error_flag, 1, GetVopReverseVlc(rec_vop),switched,&last_non_null_coeff);
                
                /* Do the DC/AC prediction, changing the qcoeff values as
                   appropriate */
                if (intra_dcpred_disable == 0)
                    doDCACrecon(q_block, 64, DC_store, QP_store, QP,
                                x_pos, y_pos, MB_width, ACpred_flag, comp);
                else
                {
                    /* Ordinary inverse zigzag-scanning */
                    for (m = 0; m < 64; m++)
                        pcoeff[m] = q_block[zigzag[m]];
                    for (m = 0; m < 8; m++)
                        for (n = 0; n < 8; n++)
                            q_block[m*8+n] = pcoeff[m*8+n];
                }
                /* Store the qcoeff-values needed later for prediction */
                for (m = 0; m < 8; m++) {
                    DC_store[mbnum][comp][m] = q_block[m];
                }
                for (m = 0; m < 7; m++) {
                    DC_store[mbnum][comp][m+8] = q_block[(m+1)*8];
                }
                
                /* MW 22-APR-1997 Changes for MPEG dequantization */
                if(GetVopQuantType(rec_vop))
                    BlockDequantMPEG(q_block,quant_step,INTRA,2,
                                     GetVopIntraQuantMat(rec_vop),
                                     DCT_block,last_non_null_coeff);
                else
                    BlockDequantH263(q_block,quant_step,INTRA,2,DCT_block,last_non_null_coeff);
                DC_store[mbnum][comp][0] = DCT_block[0];
                
                BlockIDCT (DCT_block, block_buf);
                for (m=0;m<8;m++)
                    for (n=0;n<8;n++)
                        r_block[m*8+n]= block_buf[m][n];
                
            }
            else                /* MODE_INTER */
            {
                if (CBP & (1 << (BLOCK_CNT - 1 - comp)))
                    VlcGetBlock (texture_bits, q_block, INTER, trace,
                                 error_flag, 0, GetVopReverseVlc(rec_vop),0,&last_non_null_coeff);
                
                /* Inverse zigzag-scanning also here 12-11-96 MW */
                for (m = 0; m < 64; m++)
                    pcoeff[m] = q_block[zigzag[m]];
                
                for (m = 0; m < 8; m++)
                    for (n = 0; n < 8; n++)
                        q_block[m*8+n] = pcoeff[m*8+n];
                
                /* Fill DC_store with default coeff values */
                DC_store[mbnum][comp][0] = 128*8;
                for (n = 1; n < 15; n++)
                    DC_store[mbnum][comp][n] = 0;
                
                /* MW 22-APR-1997 Changes for MPEG dequantization */
                if(GetVopQuantType(rec_vop))
                    BlockDequantMPEG(q_block,quant_step,INTER,2,
                                     GetVopNonintraQuantMat(rec_vop),
                                     DCT_block,last_non_null_coeff);
                else
                    BlockDequantH263(q_block,quant_step,INTER,2,DCT_block,last_non_null_coeff);
                
                
                BlockIDCT (DCT_block, block_buf);
                for (m=0;m<8;m++)
                    for (n=0;n<8;n++)
                        r_block[m*8+n]= block_buf[m][n];
                
            }
            
            PutBlock (comp, r_block, mblock);
        }
        
        /*****
         *
         *        Clip reconstructed intra MBs
         *
         *****/
        if (mode == MODE_INTRA)
            MB_clip (mblock);
    }
}                               /* GetMBText */
#endif


/***********************************************************CommentBegin******
 *
 * -- MBzero -- Fills one macrocblock with zeros
 *
 * Author :
 *      Karl.Lillevold@nta.no
 *
 * Created :
 *      29-Aug-94
 *
 * Purpose :
 *      To fill one macrocblock with zeros.
 *
 * Arguments in :
 *
 *
 * Arguments in/out :
 *      Macroblock     *mblock : macroblock (cleared)
 *
 * Arguments out :
 *
 *
 * Return values :
 *
 *
 * Side effects :
 *
 *
 * Description :
 *
 *
 * See also :
 *
 *
 * Modified :
 *      28-Nov-95 Robert Inge Danielsen - C++-ifying the code. Is now a
 *                                        member function.
 *
 *
 ***********************************************************CommentEnd********/

Void
CallingConvention
MBzero (Macroblock * mblock)

{
    Int                 m;
#ifndef _OPTIMIZE_
    Int                 n;
#endif    
    
#ifdef _OPTIMIZE_
    m = (MB_SIZE>>1) * (MB_SIZE>>1);
    memset(mblock->lum, 0, (MB_SIZE * MB_SIZE)*sizeof(Int));
    memset(mblock->Cr,  0, (m)*sizeof(Int));
    memset(mblock->Cb,  0, (m)*sizeof(Int));
#else
    for (n = 0; n < MB_SIZE; n++)
        for (m = 0; m < MB_SIZE; m++)
            mblock->lum[n][m] = 0;
    
    for (n = 0; n < (MB_SIZE >> 1); n++)
        for (m = 0; m < (MB_SIZE >> 1); m++)
        {
            mblock->Cr[n][m] = 0;
            mblock->Cb[n][m] = 0;
        }
#endif
    
}                               /* MBzero */

/***********************************************************CommentBegin******
 *
 * -- Bzero -- Fills one block with zeros
 *
 * Author :
 *      Isabelle Corset (LEP) <corset@lep-philips.fr>
 *
 * Created :
 *      5-Mar-96
 *
 * Purpose :
 *      To fill one block with zeros.
 *
 * Arguments in :
 *
 *
 * Arguments in/out :
 *      Int *block,              block (already allocated).
 *
 *
 * Arguments out :
 *
 *
 * Return values :
 *
 *
 * Side effects :
 *
 *
 * Description :
 *
 *
 * See also :
 *
 *
 * Modified :
 *
 *
 ***********************************************************CommentEnd********/

Void
CallingConvention
Bzero (Int * block)
{
#ifdef _OPTIMIZE_
    memset(block, 0, (B_SIZE * B_SIZE)*sizeof(Int));
#else
    Int                 n;
    for (n = 0; n < B_SIZE * B_SIZE; n++)
        block[n] = 0;
#endif   
}                               /* Bzero */



/***********************************************************CommentBegin******
 *
 * -- PutBlock -- Copy a block into a macroblock
 *
 * Author :
 *      Isabelle Corset (LEP) <corset@lep-philips.fr>
 *
 * Created :
 *      27-Feb-96
 *
 * Purpose :
 *      To copy the blocks of texture component into a macroblock structure.
 *
 *
 * Arguments in :
 *      Int comp,               the component number.
 *      Int * block,            the block data.
 *
 *
 * Arguments in/out :
 *      Macroblock *mblock,     the filled macroblock structure.
 *
 *
 * Arguments out :
 *
 *
 * Return values :
 *
 *
 * Side effects :
 *
 *
 * Description :
 *
 *
 * See also :
 *
 *
 * Modified :
 *
 *
 ***********************************************************CommentEnd********/
#if 0
Void
CallingConvention
PutBlock (Int comp,
          Int * block,
          Macroblock * mblock)
{
    Int                 m, n;
    Int                 begin_l, begin_c;
    
    if (comp < 4)
    {
        begin_l = (comp / 2) * B_SIZE;
        begin_c = (comp % 2) * B_SIZE;
        
        for (n = 0; n < B_SIZE; n++)
            for (m = 0; m < B_SIZE; m++)
                mblock->lum[n + begin_l][m + begin_c] = *(block + n * B_SIZE + m);
    }
    else
    {
        if (comp == 4)
        {
            for (n = 0; n < B_SIZE; n++)
                for (m = 0; m < B_SIZE; m++)
                    mblock->Cr[n][m] = *(block + n * B_SIZE + m);
        }
        else
        {
            for (n = 0; n < B_SIZE; n++)
                for (m = 0; m < B_SIZE; m++)
                    mblock->Cb[n][m] = *(block + n * B_SIZE + m);
        }
    }
    
}                               /* PutBlock */
#endif

/***********************************************************CommentBegin******
 *
 * -- UInt DecodePredictedIntraDC --
 *
 * Author : Cor Quist, KPN Research - The Netherlands
 *
 *
 * Created : 23-May-1996
 *
 *
 * Purpose : To decode the predicted intra dc coefficient for intra
 *           macroblocks
 *
 *
 * Arguments in : Int mbnum : macroblock number
 *                Int compnum : block number (0..5)
 *                Bitstream *stream : a pointer to the bitstream
 *                Trace *trace : trace info
 *                Int x_width : horizontal dimension of the VOP
 *                UInt *new_vop : indicates if this MB is the first MB for
 *                                this VOP
 *                Int trace_mode : mode info. for trace
 *
 * Arguments in/out :
 *
 *
 * Arguments out :
 *
 *
 * Return values : UInt DC_coeff : the dc coefficient for one block
 *
 *
 * Side effects :
 *
 *
 * Description :
 *
 *
 * See also : This algorithm is based upon VM2.0.
 *            Not compatible with VM2.1
 *
 *
 * Modified :
 *
 *
 ***********************************************************CommentEnd********/
/* This should be "Int" since prediction can be negative!!! MW 15-NOV-1996 */
Int
CallingConvention
DecodePredictedIntraDC(Int compnum,
                       Bitstream *stream,
                       Trace *trace,
                       Int trace_mode)
{
    UInt DC_size;
    UInt code;
    Int INTRADC_delta;
    Int error_flag=0;
    Int first_bit;
    
    /* read DC size 2 - 8 bits */
    
    DC_size = VlcDecIntraDCPredSize (stream, &error_flag, compnum);
    
    if ( error_flag == RETURN)
    {
        fprintf(stderr, "Error decoding INTRADC pred. size\n");
        return -1;
    }
    
    if (DC_size == 0)
    {
        INTRADC_delta = 0;
    }
    else
    {
        /* read delta DC 0 - 8 bits */
        
        code = BitstreamReadBits (stream, DC_size, "DC coeff", trace,
                                  trace_mode);
        
        first_bit = code >> (DC_size-1);
        
        if (first_bit == 0 )
        { /* negative delta INTRA DC */
            INTRADC_delta  = -1 * (code ^( (Int)pow(2.0,(Double)DC_size)-1) );
        }
        else
        { /* positive delta INTRA DC */
            INTRADC_delta = code;
        }
    }
    
    return INTRADC_delta;
}

/***********************************************************CommentBegin******
 *
 * -- UInt VlcDecIntraDCPredSize --
 *
 * Author : Cor Quist, KPN Research - The Netherlands.
 *
 *
 * Created : 23-May-1996
 *
 *
 * Purpose : VLC decoding of the SIZE field off the predicted INTRA DC
 *           coefficient.
 *
 *
 * Arguments in : Bitstream *stream : a pointer to the bitstream
 *                Int *error_flag : indicates the occurence of an error
 *                Int compnum : the block number (0..5)
 *
 * Arguments in/out :
 *
 *
 * Arguments out :
 *
 *
 * Return values : UInt DC_size : The number of bits of the INTRA DC
 *                                difference
 *
 *
 * Side effects :
 *
 *
 * Description :
 *
 *
 * See also : This algorithm is based upon VM2.0.
 *            Not compatible with VM2.1
 *
 *
 * Modified :
 *
 *
 ***********************************************************CommentEnd********/

/*ok*/static UInt
CallingConvention
VlcDecIntraDCPredSize (Bitstream *stream, Int *error_flag, Int compnum)
{
    UInt  code;
    UInt  DC_size=0;
    
    if( compnum >=0 && compnum < 4 ) /* luminance block */
    {
        code = BitstreamShowBits (stream, 11);
        if ( code == 1)
        {
            DC_size = 12;
            BitstreamFlushBits(stream, 11);
            return DC_size;
        }
        
        code = BitstreamShowBits (stream, 10);
        if ( code == 1)
        {
            DC_size = 11;
            BitstreamFlushBits(stream, 10);
            return DC_size;
        }
        
        code = BitstreamShowBits (stream, 9);
        if ( code == 1)
        {
            DC_size = 10;
            BitstreamFlushBits(stream, 9);
            return DC_size;
        }
        
        code = BitstreamShowBits (stream, 8);
        if ( code == 1)
        {
            DC_size = 9;
            BitstreamFlushBits(stream, 8);
            return DC_size;
        }
        
        code = BitstreamShowBits (stream, 7);
        if ( code == 1)
        {
            DC_size = 8;
            BitstreamFlushBits(stream, 7);
            return DC_size;
        }
        code = BitstreamShowBits (stream, 6);
        if ( code == 1)
        {
            DC_size = 7;
            BitstreamFlushBits(stream, 6);
            return DC_size;
        }
        code = BitstreamShowBits (stream, 5);
        if ( code == 1)
        {
            DC_size = 6;
            BitstreamFlushBits(stream, 5);
            return DC_size;
        }
        code = BitstreamShowBits (stream, 4);
        if ( code == 1)
        {
            DC_size = 5;
            BitstreamFlushBits(stream, 4);
            return DC_size;
        }
        code = BitstreamShowBits (stream, 3);
        if ( code == 1)
        {
            DC_size = 4;
            BitstreamFlushBits(stream, 3);
            return DC_size;
        } else if (code == 2) {
            DC_size = 3;
            BitstreamFlushBits(stream, 3);
            return DC_size;
        } else if (code ==3) {
            DC_size = 0;
            BitstreamFlushBits(stream, 3);
            return DC_size;
        }
        code = BitstreamShowBits (stream, 2);
        if ( code == 2)
        {
            DC_size = 2;
            BitstreamFlushBits(stream, 2);
            return DC_size;
        } else if (code == 3) {
            DC_size = 1;
            BitstreamFlushBits(stream, 2);
            return DC_size;
        }     
        
    }
    else /* chrominance block */
    {
        code = BitstreamShowBits (stream, 12);
        if ( code == 1)
        {
            DC_size = 12;
            BitstreamFlushBits(stream, 12);
            return DC_size;
        }
        
        code = BitstreamShowBits (stream, 11);
        if ( code == 1)
        {
            DC_size = 11;
            BitstreamFlushBits(stream, 11);
            return DC_size;
        }
        
        code = BitstreamShowBits (stream, 10);
        if ( code == 1)
        {
            DC_size = 10;
            BitstreamFlushBits(stream, 10);
            return DC_size;
        }
        
        code = BitstreamShowBits (stream, 9);
        if ( code == 1)
        {
            DC_size = 9;
            BitstreamFlushBits(stream, 9);
            return DC_size;
        }
        
        code = BitstreamShowBits (stream, 8);
        if ( code == 1)
        {
            DC_size = 8;
            BitstreamFlushBits(stream, 8);
            return DC_size;
        }
        code = BitstreamShowBits (stream, 7);
        if ( code == 1)
        {
            DC_size = 7;
            BitstreamFlushBits(stream, 7);
            return DC_size;
        }
        code = BitstreamShowBits (stream, 6);
        if ( code == 1)
        {
            DC_size = 6;
            BitstreamFlushBits(stream, 6);
            return DC_size;
        }
        code = BitstreamShowBits (stream, 5);
        if ( code == 1)
        {
            DC_size = 5;
            BitstreamFlushBits(stream, 5);
            return DC_size;
        }
        code = BitstreamShowBits (stream, 4);
        if ( code == 1)
        {
            DC_size = 4;
            BitstreamFlushBits(stream, 4);
            return DC_size;
        } 
        code = BitstreamShowBits (stream, 3);
        if ( code == 1)
        {
            DC_size = 3;
            BitstreamFlushBits(stream, 3);
            return DC_size;
        } 
        code = BitstreamShowBits (stream, 2);
        {
            DC_size = 3-code;
            BitstreamFlushBits(stream, 2);
            return DC_size;
        } 
    }
    
    return DC_size;
    
}     /* VlcDecIntraDCPredSize */

/***********************************************************CommentBegin******
 *
 * -- doDCACrecon -- Does DC/AC reconstruction. Changes qcoeff values as
 *                   appropriate.
 *
 * Author :             
 *      Robert Danielsen, Telenor <Robert.Danielsen@nta.no>
 *
 * Created :            
 *      23.10.96
 *
 * Purpose :            
 *      Does DC/AC reconstruction. Changes qcoeff values as appropriate. 
 * 
 * Arguments in :       
 *      Int ncoeffs
 *      Int x_pos
 *      Int y_pos
 *      Int DC_store[][6][15]   Stores coefficient values per MB for
 *                              prediction (for one Vop)
 *      SInt *QP_store          Stores QP values for MBs (for one Vop)
 *      Int QP                  QP value for this MB
 *      Int MB_width
 *
 * Arguments in/out :   
 *      Int *q_block            Coefficients
 *
 * Arguments out :      
 *      
 *
 * Return values :      
 *
 * Side effects :       
 *      Modifies q_block if needed for the reconstruction.
 *
 * Description :        
 *      
 *
 * See also :
 *      
 *
 * Modified :           
 *      28.01.97 Robert Danielsen: Changed zigzag-scanning.
 *      04.11.97 Minhua Zhou     : updated DC/AC prediction
 *
 ***********************************************************CommentEnd********/

Void
CallingConvention
doDCACrecon(Int *q_block, Int ncoeffs, Int *DC_store,
            SInt *QP_store, Int QP, Int x_pos, Int y_pos, Int MB_width,
            Int ACpred_flag, Int comp)
{
    Int m, n;
    Int block_A, block_B, block_C;
    Int Xpos[6] = {-1, 0, -1, 0, -1, -1};
    Int Ypos[6] = {-1, -1, 0, 0, -1, -1};
    Int Xtab[6] = {1, 0, 3, 2, 4, 5};
    Int Ytab[6] = {2, 3, 0, 1, 4, 5};
    Int Ztab[6] = {3, 2, 1, 0, 4, 5};
    Int grad_hor, grad_ver, DC_pred;
    Int direction;              /* 0: HORIZONTAL, 1: VERTICAL */
    Int pred_A[15], pred_C[15];
    Int pcoeff[64];
    
#ifdef _OPTIMIZE_
    Int *a, *b, *c;
#endif    
    
    /* Find the direction of prediction and the DC prediction */
#ifndef _OPTIMIZE_
    if ((x_pos == 0) && y_pos == 0) {   /* top left corner */
        block_A = (comp == 1 || comp == 3) ? DC_store[y_pos*MB_width+(x_pos+Xpos[comp])][Xtab[comp]][0] : 128*8;
        block_B = (comp == 3) ? DC_store[(y_pos+Ypos[comp])*MB_width+(x_pos+Xpos[comp])][Ztab[comp]][0] : 128*8;
        block_C = (comp == 2 || comp == 3) ? DC_store[(y_pos+Ypos[comp])*MB_width+x_pos][Ytab[comp]][0] : 128*8;
    }
    else if (x_pos == 0) {      /* left edge */
        block_A = (comp == 1 || comp == 3) ? DC_store[y_pos*MB_width+(x_pos+Xpos[comp])][Xtab[comp]][0] : 128*8;
        block_B = (comp == 1 || comp == 3) ? DC_store[(y_pos+Ypos[comp])*MB_width+(x_pos+Xpos[comp])][Ztab[comp]][0] : 128*8;
        block_C = DC_store[(y_pos+Ypos[comp])*MB_width+x_pos][Ytab[comp]][0];
    }
    else if (y_pos == 0) { /* top row */
        block_A = DC_store[y_pos*MB_width+(x_pos+Xpos[comp])][Xtab[comp]][0];
        block_B = (comp == 2 || comp == 3) ? DC_store[(y_pos+Ypos[comp])*MB_width+(x_pos+Xpos[comp])][Ztab[comp]][0] : 128*8;
        block_C = (comp == 2 || comp == 3) ? DC_store[(y_pos+Ypos[comp])*MB_width+x_pos][Ytab[comp]][0] : 128*8;
    }
    else {
        block_A = DC_store[y_pos*MB_width+(x_pos+Xpos[comp])][Xtab[comp]][0];
        block_B = (DC_store[(y_pos+Ypos[comp])*MB_width+(x_pos+Xpos[comp])]
                   [Ztab[comp]][0]);
        block_C = DC_store[(y_pos+Ypos[comp])*MB_width+x_pos][Ytab[comp]][0];
    }
#else
    a = DC_store + 90*(y_pos*MB_width+(x_pos+Xpos[comp]))              + 15*Xtab[comp];
    b = DC_store + 90*((y_pos+Ypos[comp])*MB_width+(x_pos+Xpos[comp])) + 15*Ztab[comp];
    c = DC_store + 90*((y_pos+Ypos[comp])*MB_width+x_pos)              + 15*Ytab[comp];
    if ((x_pos == 0) && y_pos == 0)
    {   /* top left corner */
        
        block_A = (comp == 1 || comp == 3) ? *a : 1024;
        block_B = (comp == 3) ? *b: 1024;
        block_C = (comp == 2 || comp == 3) ? *c : 1024;
    }
    else if (x_pos == 0)
    {   /* left edge */
        block_A = (comp == 1 || comp == 3) ? *a : 1024;
        block_B = (comp == 1 || comp == 3) ? *b : 1024;
        block_C = *c;
    }
    else if (y_pos == 0)
    { /* top row */
        block_A = *a;
        block_B = (comp == 2 || comp == 3) ? *b : 1024;
        block_C = (comp == 2 || comp == 3) ? *c : 1024;
    }
    else
    {
        block_A = *a;
        block_B = *b;
        block_C = *c;
    }    
#endif
    
    grad_hor = block_B - block_C;
    grad_ver = block_A - block_B;
    
    if ((ABS(grad_ver)) < (ABS(grad_hor))) {
        DC_pred = block_C;
        direction = 1;
    }
    else {
        DC_pred = block_A;
        direction = 0;
    }
    
    /* Do inverse zigzag-scanning */
    if (ACpred_flag == 1)
        if (direction == 0)
            for (m = 0; m < 64; m++)
                pcoeff[m] = q_block[zigzag_v[m]];
        else
            for (m = 0; m < 64; m++)
                pcoeff[m] = q_block[zigzag_h[m]];
    else
        for (m = 0; m < 64; m++)
            pcoeff[m] = q_block[zigzag[m]];
    
    /* Now reconstruct the DC coefficient */
    q_block[0] = pcoeff[0] + (DC_pred+cal_dc_scaler(QP,(comp<4)?1:2)/2)/cal_dc_scaler(QP,(comp<4)?1:2);
    
    
    /* Do AC prediction if required */
    if (ACpred_flag == 1)
    {
#ifndef _OPTIMIZE_
        /* Find AC predictions */
        if ((x_pos == 0) && y_pos == 0) {       /* top left corner */
            if (comp == 1 || comp == 3)
                for (m = 0; m < 15; m++) 
                    pred_A[m] = (((DC_store[y_pos*MB_width+(x_pos+Xpos[comp])][Xtab[comp]][m]) * (QP_store[y_pos*MB_width+(x_pos+Xpos[comp])])) / QP);
            else
                nullfill(pred_A);
            if (comp == 2 || comp == 3)
                for (m = 0; m < 15; m++) 
                    pred_C[m] = (((DC_store[(y_pos+Ypos[comp])*MB_width+x_pos][Ytab[comp]][m]) * (QP_store[(y_pos+Ypos[comp])*MB_width+x_pos])) / QP);
            else
                nullfill(pred_C);
        }
        else if (x_pos == 0) {  /* left edge */
            if (comp == 1 || comp == 3)
                for (m = 0; m < 15; m++) 
                    pred_A[m] = (((DC_store[y_pos*MB_width+(x_pos+Xpos[comp])][Xtab[comp]][m]) * (QP_store[y_pos*MB_width+(x_pos+Xpos[comp])])) / QP);
            else
                nullfill(pred_A);
            for (m = 0; m < 15; m++) 
                pred_C[m] = (((DC_store[(y_pos+Ypos[comp])*MB_width+x_pos][Ytab[comp]][m]) * (QP_store[(y_pos+Ypos[comp])*MB_width+x_pos])) / QP);
        }
        else if (y_pos == 0) { /* top row */
            for (m = 0; m < 15; m++) 
                pred_A[m] = (((DC_store[y_pos*MB_width+(x_pos+Xpos[comp])][Xtab[comp]][m]) * (QP_store[y_pos*MB_width+(x_pos+Xpos[comp])])) / QP);
            if (comp == 2 || comp == 3)
                for (m = 0; m < 15; m++) 
                    pred_C[m] = (((DC_store[(y_pos+Ypos[comp])*MB_width+x_pos][Ytab[comp]][m]) * (QP_store[(y_pos+Ypos[comp])*MB_width+x_pos])) / QP);
            else
                nullfill(pred_C);
        }
        else {
            for (m = 0; m < 15; m++) {
                pred_A[m] = (((DC_store[y_pos*MB_width+(x_pos+Xpos[comp])][Xtab[comp]][m]) * (QP_store[y_pos*MB_width+(x_pos+Xpos[comp])])) / QP);
                pred_C[m] = (((DC_store[(y_pos+Ypos[comp])*MB_width+x_pos][Ytab[comp]][m]) * (QP_store[(y_pos+Ypos[comp])*MB_width+x_pos])) / QP);
            }
        }
#else
        a = DC_store + 90*(y_pos*MB_width+(x_pos+Xpos[comp]))              + 15*Xtab[comp];
        b = DC_store + 90*((y_pos+Ypos[comp])*MB_width+(x_pos+Xpos[comp])) + 15*Ztab[comp];
        c = DC_store + 90*((y_pos+Ypos[comp])*MB_width+x_pos)              + 15*Ytab[comp];
        /* Find AC predictions */
        if ((x_pos == 0) && y_pos == 0)
        {       /* top left corner */
            if (comp == 1 || comp == 3)
                for (m = 0; m < 15; m++) 
                    pred_A[m] = ( ( *(a + m) * (QP_store[y_pos*MB_width+(x_pos+Xpos[comp])])) / QP);
            else
                nullfill(pred_A);
            
            if (comp == 2 || comp == 3)
                for (m = 0; m < 15; m++) 
                    pred_C[m] = ( ( *(c + m) * (QP_store[(y_pos+Ypos[comp])*MB_width+x_pos])) / QP);
            else
                nullfill(pred_C);
        }
        else if (x_pos == 0)
        {       /* left edge */
            if (comp == 1 || comp == 3)
                for (m = 0; m < 15; m++) 
                    pred_A[m] = ( ( *(a + m) * (QP_store[y_pos*MB_width+(x_pos+Xpos[comp])])) / QP);
            else
                nullfill(pred_A);
            
            for (m = 0; m < 15; m++) 
                pred_C[m] = ( ( *(c + m) * (QP_store[(y_pos+Ypos[comp])*MB_width+x_pos])) / QP);
        }
        else if (y_pos == 0)
        { /* top row */
            for (m = 0; m < 15; m++) 
                pred_A[m] = ( ( *(a + m) * (QP_store[y_pos*MB_width+(x_pos+Xpos[comp])])) / QP);
            
            if (comp == 2 || comp == 3)
                for (m = 0; m < 15; m++) 
                    pred_C[m] = ( ( *(c + m) * (QP_store[(y_pos+Ypos[comp])*MB_width+x_pos])) / QP);
            else
                nullfill(pred_C);
        }
        else
        {
            for (m = 0; m < 15; m++)
            {
                pred_A[m] = ( ( *(a + m) * (QP_store[y_pos*MB_width+(x_pos+Xpos[comp])])) / QP);
                pred_C[m] = ( ( *(c + m) * (QP_store[(y_pos+Ypos[comp])*MB_width+x_pos])) / QP);
            }
        }        
#endif
        
        /* Now reconstruct the AC coefficients */
        if (direction == 0) { /* Horizontal, left COLUMN of block A */
            for (m = 0; m < 7; m++) {
                q_block[(m+1)*8]= pcoeff[(m+1)*8] + pred_A[m+8];
            }
            for (m = 1; m < 8; m++) {
                q_block[m] = pcoeff[m];
            }
        }
        else { /* Vertical, top ROW of block C */
            for (m = 0; m < 7; m++) {
                q_block[(m+1)*8]= pcoeff[(m+1)*8];
            }
            for (m = 1; m < 8; m++) {
                q_block[m] = pcoeff[m] + pred_C[m];
            }
        }
        
        /* Copy rest of coefficients */
        for (m = 1; m < 8; m++)
            for (n = 1; n < 8; n++)
                q_block[m*8+n] = pcoeff[m*8+n];
    }
    else
        /* Copy all AC coefficients directly */
        for (m = 1; m < 64; m++)
            q_block[m] = pcoeff[m];
}

/***********************************************************CommentBegin******
 *
 * -- doDCACreconErrRes -- Does DC/AC reconstruction. Changes qcoeff values as
 *                         appropriate.
 *
 * Author :             
 *      Luis Ducla Soares (IST) - lds@lx.it.pt
 *
 * Created :            
 *      23.10.96
 *
 * Purpose :            
 *      Does DC/AC reconstruction. Changes qcoeff values as appropriate. 
 * 
 * Arguments in :       
 *      Int ncoeffs
 *      Int x_pos
 *      Int y_pos
 *      Int DC_store[][6][15]   Stores coefficient values per MB for
 *                              prediction (for one Vop)
 *      SInt *QP_store          Stores QP values for MBs (for one Vop)
 *      Int QP                  QP value for this MB
 *      Int MB_width
 *
 * Arguments in/out :   
 *      Int *q_block            Coefficients
 *
 * Arguments out :      
 *      
 *
 * Return values :      
 *
 * Side effects :       
 *      Modifies q_block if needed for the reconstruction.
 *
 * Description :        
 *      
 *
 * See also :
 *      
 *
 * Modified :           
 *      28.01.97 Robert Danielsen: Changed zigzag-scanning.
 *
 ***********************************************************CommentEnd********/

#if(__ERROR_RESILIENCE__)
Void
CallingConvention
doDCACreconErrRes(Int *q_block, Int ncoeffs, Int ***DC_store,
                  SInt *QP_store, Int QP, Int x_pos, Int y_pos, Int MB_width,
                  Int ACpred_flag, Int comp, Int **slice_nb)
{
    Int m, n;
    Int block_A, block_B, block_C;
    Int Xpos[6] = {-1, 0, -1, 0, -1, -1};
    Int Ypos[6] = {-1, -1, 0, 0, -1, -1};
    Int Xtab[6] = {1, 0, 3, 2, 4, 5};
    Int Ytab[6] = {2, 3, 0, 1, 4, 5};
    Int Ztab[6] = {3, 2, 1, 0, 4, 5};
    Int grad_hor, grad_ver, DC_pred;
    Int direction;              /* 0: HORIZONTAL, 1: VERTICAL */
    Int pred_A[15], pred_C[15];
    Int pcoeff[64];
    
    /* Find the direction of prediction and the DC prediction */
    if ((x_pos == 0) && y_pos == 0) {   /* top left corner */
        block_A = (comp == 1 || comp == 3) ? DC_store[y_pos*MB_width+(x_pos+Xpos[comp])][Xtab[comp]][0] : 128*8;
        block_B = (comp == 3) ? DC_store[(y_pos+Ypos[comp])*MB_width+(x_pos+Xpos[comp])][Ztab[comp]][0] : 128*8;
        block_C = (comp == 2 || comp == 3) ? DC_store[(y_pos+Ypos[comp])*MB_width+x_pos][Ytab[comp]][0] : 128*8;
    }
    else if (x_pos == 0) {      /* left edge */
        block_A = (comp == 1 || comp == 3) ? DC_store[y_pos*MB_width+(x_pos+Xpos[comp])][Xtab[comp]][0] : 128*8;
        block_B = ((comp == 1 && slice_nb[x_pos][y_pos] == slice_nb[x_pos][y_pos-1]) || comp == 3) ? 
            DC_store[(y_pos+Ypos[comp])*MB_width+(x_pos+Xpos[comp])][Ztab[comp]][0] : 128*8;
        block_C = (comp == 2 || comp == 3 || ((comp == 0 || comp == 1 || comp == 4 || comp == 5) &&
                                              slice_nb[x_pos][y_pos] == slice_nb[x_pos][y_pos-1])) ?
            DC_store[(y_pos+Ypos[comp])*MB_width+x_pos][Ytab[comp]][0] : 128*8;
    }
    else if (y_pos == 0) { /* top row */
        block_A = (comp == 1 || comp == 3 || ((comp == 0 || comp == 2 || comp == 4 || comp == 5) &&
                                              slice_nb[x_pos][y_pos] == slice_nb[x_pos-1][y_pos])) ?
            DC_store[y_pos*MB_width+(x_pos+Xpos[comp])][Xtab[comp]][0] : 128*8;
        block_B = ((comp == 2 && slice_nb[x_pos][y_pos] == slice_nb[x_pos-1][y_pos]) || comp == 3) ? 
            DC_store[(y_pos+Ypos[comp])*MB_width+(x_pos+Xpos[comp])][Ztab[comp]][0] : 128*8;
        block_C = (comp == 2 || comp == 3) ? DC_store[(y_pos+Ypos[comp])*MB_width+x_pos][Ytab[comp]][0] : 128*8;
    }
    else {
        block_A = (comp == 1 || comp == 3 || ((comp == 0 || comp == 2 || comp == 4 || comp == 5 ) &&
                                              slice_nb[x_pos][y_pos] == slice_nb[x_pos-1][y_pos])) ?
            DC_store[y_pos*MB_width+(x_pos+Xpos[comp])][Xtab[comp]][0] : 128*8;
        block_B = (((comp == 0 || comp == 4 || comp == 5) && slice_nb[x_pos][y_pos] == slice_nb[x_pos-1][y_pos-1]) ||
                   (comp == 1 && slice_nb[x_pos][y_pos] == slice_nb[x_pos][y_pos-1]) || 
                   (comp == 2 && slice_nb[x_pos][y_pos] == slice_nb[x_pos-1][y_pos]) || (comp == 3)) ? 
            (DC_store[(y_pos+Ypos[comp])*MB_width+(x_pos+Xpos[comp])][Ztab[comp]][0]) : 128*8;
        block_C = (comp == 2 || comp == 3 || ((comp == 0 || comp == 1 || comp == 4 || comp == 5) &&
                                              slice_nb[x_pos][y_pos] == slice_nb[x_pos][y_pos-1])) ?
            DC_store[(y_pos+Ypos[comp])*MB_width+x_pos][Ytab[comp]][0] : 128*8;
    }
    grad_hor = block_B - block_C;
    grad_ver = block_A - block_B;
    
    if ((ABS(grad_ver)) < (ABS(grad_hor))) {
        DC_pred = block_C;
        direction = 1;
    }
    else {
        DC_pred = block_A;
        direction = 0;
    }
    
    /* Do inverse zigzag-scanning */
    if (ACpred_flag == 1)
        if (direction == 0)
            for (m = 0; m < 64; m++)
                pcoeff[m] = q_block[zigzag_v[m]];
        else
            for (m = 0; m < 64; m++)
                pcoeff[m] = q_block[zigzag_h[m]];
    else
        for (m = 0; m < 64; m++)
            pcoeff[m] = q_block[zigzag[m]];
    
    /* Now reconstruct the DC coefficient */
    q_block[0] = pcoeff[0] + (DC_pred+cal_dc_scaler(QP,(comp<4)?1:2)/2)/cal_dc_scaler(QP,(comp<4)?1:2);
    
    /* Do AC prediction if required */
    if (ACpred_flag == 1)
    {
        /* Find AC predictions */
        if ((x_pos == 0) && y_pos == 0) {       /* top left corner */
            if (comp == 1 || comp == 3)
                for (m = 0; m < 15; m++) 
                    pred_A[m] = (((DC_store[y_pos*MB_width+(x_pos+Xpos[comp])][Xtab[comp]][m]) * (QP_store[y_pos*MB_width+(x_pos+Xpos[comp])])) / QP);
            else
                nullfill(pred_A);
            if (comp == 2 || comp == 3)
                for (m = 0; m < 15; m++) 
                    pred_C[m] = (((DC_store[(y_pos+Ypos[comp])*MB_width+x_pos][Ytab[comp]][m]) * (QP_store[(y_pos+Ypos[comp])*MB_width+x_pos])) / QP);
            else
                nullfill(pred_C);
        }
        else if (x_pos == 0) {  /* left edge */
            if (comp == 1 || comp == 3)
                for (m = 0; m < 15; m++) 
                    pred_A[m] = (((DC_store[y_pos*MB_width+(x_pos+Xpos[comp])][Xtab[comp]][m]) * (QP_store[y_pos*MB_width+(x_pos+Xpos[comp])])) / QP);
            else
                nullfill(pred_A);
            if (((comp == 0 || comp == 1 || comp == 4 || comp == 5) && slice_nb[x_pos][y_pos] == slice_nb[x_pos][y_pos-1]) || 
                comp == 2 || comp == 3)
                for (m = 0; m < 15; m++) 
                    pred_C[m] =
                        (((DC_store[(y_pos+Ypos[comp])*MB_width+x_pos][Ytab[comp]][m]) * (QP_store[(y_pos+Ypos[comp])*MB_width+x_pos])) / QP);
            else 
                nullfill(pred_C);
        }
        else if (y_pos == 0) { /* top row */
            if (((comp == 0 || comp == 2 || comp == 4 || comp == 5) && slice_nb[x_pos][y_pos] == slice_nb[x_pos-1][y_pos]) ||
                comp == 1 || comp == 3)
                for (m = 0; m < 15; m++) 
                    pred_A[m] =
                        (((DC_store[y_pos*MB_width+(x_pos+Xpos[comp])][Xtab[comp]][m]) * (QP_store[y_pos*MB_width+(x_pos+Xpos[comp])])) / QP);
            else
                nullfill(pred_A);
            if (comp == 2 || comp == 3)
                for (m = 0; m < 15; m++) 
                    pred_C[m] = (((DC_store[(y_pos+Ypos[comp])*MB_width+x_pos][Ytab[comp]][m]) * (QP_store[(y_pos+Ypos[comp])*MB_width+x_pos])) / QP);
            else
                nullfill(pred_C);
        }
        else {
            if (((comp == 0 || comp == 2 || comp == 4 || comp == 5) && slice_nb[x_pos][y_pos] == slice_nb[x_pos-1][y_pos]) || 
                comp == 1 || comp == 3)
                for (m = 0; m < 15; m++) 
                    pred_A[m] =
                        (((DC_store[y_pos*MB_width+(x_pos+Xpos[comp])][Xtab[comp]][m]) * (QP_store[y_pos*MB_width+(x_pos+Xpos[comp])])) / QP);
            else
                nullfill(pred_A);
            if (((comp == 0 || comp == 1 || comp == 4 || comp == 5) && slice_nb[x_pos][y_pos] == slice_nb[x_pos][y_pos-1]) ||
                comp  == 2 || comp == 3)
                for (m = 0; m < 15; m++)
                    pred_C[m] =
                        (((DC_store[(y_pos+Ypos[comp])*MB_width+x_pos][Ytab[comp]][m]) * (QP_store[(y_pos+Ypos[comp])*MB_width+x_pos])) / QP);
            else
                nullfill(pred_C);
            
        }
        
        /* Now reconstruct the AC coefficients */
        if (direction == 0) { /* Horizontal, left COLUMN of block A */
            for (m = 0; m < 7; m++) {
                q_block[(m+1)*8]= pcoeff[(m+1)*8] + pred_A[m+8];
            }
            for (m = 1; m < 8; m++) {
                q_block[m] = pcoeff[m];
            }
        }
        else { /* Vertical, top ROW of block C */
            for (m = 0; m < 7; m++) {
                q_block[(m+1)*8]= pcoeff[(m+1)*8];
            }
            for (m = 1; m < 8; m++) {
                q_block[m] = pcoeff[m] + pred_C[m];
            }
        }
        
        /* Copy rest of coefficients */
        for (m = 1; m < 8; m++)
            for (n = 1; n < 8; n++)
                q_block[m*8+n] = pcoeff[m*8+n];
    }
    else
        /* Copy all AC coefficients directly */
        for (m = 1; m < 64; m++)
            q_block[m] = pcoeff[m];
}
#endif

/**
 * Small routine to fill default prediction values into a DC_store entry
 *
 * Author:      Robert Danielsen
 * Created:     23.10.96
 */

/*ok*/static Void
CallingConvention
nullfill(Int pred[])
{
    Int i;
    
    pred[0] = 128*8;
    for (i = 1; i < 15; i++) {
        pred[i] = 0;
    }
}    
