/*****************************************************************************
 *
 * This software module was originally developed by
 *
 *   Ulrike Pestel (TUH / ACTS-MoMuSyS)
 *
 * and edited by
 * 
 *   Fernando Jaureguizar (UPM / ACTS-MoMuSyS)
 *
 * in the course of development of the MPEG-4 Video (ISO/IEC 14496-2) standard.
 * This software module is an implementation of a part of one or more MPEG-4
 * Video (ISO/IEC 14496-2) tools as specified by the MPEG-4 Video (ISO/IEC
 * 14496-2) standard.
 *
 * ISO/IEC gives users of the MPEG-4 Video (ISO/IEC 14496-2) standard free
 * license to this software module or modifications thereof for use in hardware
 * or software products claiming conformance to the MPEG-4 Video (ISO/IEC
 * 14496-2) standard.
 *
 * Those intending to use this software module in hardware or software products
 * are advised that its use may infringe existing patents. The original
 * developer of this software module and his/her company, the subsequent
 * editors and their companies, and ISO/IEC have no liability for use of this
 * software module or modifications thereof in an implementation. Copyright is
 * not released for non MPEG-4 Video (ISO/IEC 14496-2) Standard conforming
 * products.
 *
 * ACTS-MoMuSys partners retain full right to use the code for his/her own
 * purpose, assign or donate the code to a third party and to inhibit third
 * parties from using the code for non MPEG-4 Video (ISO/IEC 14496-2) Standard
 * conforming products. This copyright notice must be included in all copies or
 * derivative works.
 *
 * Copyright (c) 1996
 *
 *****************************************************************************/

/*****************************************************************************/
/*         This software module has been speed optimized                     */
/*                                                                           */
/*         by  Bruno Loret - France Telecom / CNET                           */
/*                                                                           */
/*         in the framework of the ACTS Project MoMuSys                      */
/*****************************************************************************/

/***********************************************************HeaderBegin*******
 *
 * File:        vm_midproc.c
 *
 * Author:      Ulrike Pestel-Schiller
 * Created:     21.05.97
 *
 * Description: * Midprocessor
 *
 * Notes:       
 *
 * Modified:    
 ***********************************************************HeaderEnd*********/

/************************    INCLUDE FILES    ********************************/

#include <stdio.h>
#include <assert.h>

#include "momusys.h"
#include "mom_access.h"
#include "mom_structs.h"
#include "vm_common_defs.h"
#include "mom_image.h"
#include "vm_midproc.h"




/***********************************************************CommentBegin******
 *
 * -- Midproc -- Midprocessor in the case of scalability
 *
 * Author :
 *      pestel@tnt.uni-hannover.de
 *
 * Created :
 *      97.05.21    
 *
 * Purpose :
 *      Builds the block Midprocessor of VM
 *
 * Arguments in :
 *      Vop   *rec_vop,    pointer to Vop of reconstructed image BASE Layer
 *      
 * Arguments in/out :
 *
 *
 * Arguments out :
 *      Vop   *rec_vop_int, pointer to Vop of reconstructed interpolated image
 *
 * Return values :
 *      none
 *
 * Side effects :
 *      Allocates memory to interpolated image
 *
 * Description :
 *
 *
 * See also :
 *
 *
 * Modified :
 *     30.07.97 F. Jaureguizar: added the all_255 alpha_uv mask for the
 *              upsampled vop. (SpSc)
 *
 ***********************************************************CommentEnd********/

Vop *
CallingConvention
Midproc(Vop *vop)
{
    Vop *up_vop=NULL;
    Int error_flag;
    
    /* VOP for upsampled reconstructed base layer allocated*/
    
    
    up_vop=AllocVop(GetVopWidth(vop)*2, GetVopHeight(vop)*2);
    
    UpsampleTexture(GetVopY(vop), GetVopY(up_vop), &error_flag);
    UpsampleTexture(GetVopU(vop), GetVopU(up_vop), &error_flag);
    UpsampleTexture(GetVopV(vop), GetVopV(up_vop), &error_flag);
    SetConstantImage(GetVopA(up_vop), 255); /* only rectangular VOPS */
    SetConstantImage(GetVopAuv(up_vop), 255); /* only rectangular VOPS *//*SpSc*/
    
    return(up_vop);
}

/********************************CommentBegin**************************
 *
 * -- UpsampleTexture -- Upsamples an image to double size
 *
 * Author :    Per Thorell (ETX)
 *
 * Created :   Jul 25 1996
 *
 * Description :         Upsamples an image to double size
 *
 * Arguments in :        Image  *small_image,  
 *
 * Arguments in/out :    
 *
 * Arguments out :       Image  *big_image
 *                       Int  *error_flag) 
 *
 * Return values :       none 
 *
 * Side effects :        Modifies output arguments
 *
 * See also : 
 *
 * Modified : 
 *
 *********************************CommentEnd***************************/

Void
CallingConvention
UpsampleTexture( Image *small_image,  
                 Image *big_image,
                 Int *error_flag)

{
    Int  i, x, y;
    SInt a[4]; 
    SInt b[4];
    
    SInt      *bim_pointer;
    SInt      *sim_pointer;
    
    Int       big_height, big_width;
    Int       small_height, small_width;
    
    
    small_height = GetImageSizeY (small_image); 
    small_width  = GetImageSizeX (small_image);
    
    big_height = GetImageSizeY (big_image); 
    big_width  = GetImageSizeX (big_image);
    
    assert(big_height == 2*small_height);
    assert(big_width == 2*small_width);
    
    bim_pointer = (SInt *) GetImageData(big_image); 
    sim_pointer = (SInt *) GetImageData(small_image); 
    
    
    
    /* Start with the corners */
    
    bim_pointer[0] = 
        sim_pointer[0];
    
    bim_pointer[big_width-1] = 
        sim_pointer[small_width - 1];
    
    bim_pointer[big_width * (big_height - 1 )] = 
        sim_pointer[small_width * (small_height - 1)];
    
    bim_pointer[big_width * big_height - 1] = 
        sim_pointer[small_width * small_height -1];
    
    
    
    /* then the edges */
    
    for (i=1 ; i < (big_width - 2) ; i+=2){
        bim_pointer[i]   = 
            (3*sim_pointer[i/2] 
             + sim_pointer[i/2+1] ) / 4;
        
        bim_pointer[i+1] = 
            (sim_pointer[i/2] 
             + 3*sim_pointer[i/2+1] ) / 4;
        
        bim_pointer[big_width * (big_height - 1 ) + i] =
            (3*sim_pointer[small_width * (small_height - 1) + i/2 ] 
             + sim_pointer[small_width * (small_height - 1) + i/2 + 1]) / 4;
        
        bim_pointer[big_width * (big_height - 1) + i + 1] =
            (sim_pointer[small_width * (small_height - 1) + i/2 ]
             + 3*sim_pointer[small_width * (small_height - 1) + i/2 + 1]) / 4;
    }
    
    
    for (i=1 ; i < (big_height - 2) ; i+=2){
        bim_pointer[big_width * i]   = 
            (3*sim_pointer[small_width * (i/2) ] 
             + sim_pointer[small_width * (i/2+1)] ) / 4;
        
        bim_pointer[big_width * (i + 1)]   =   
            (  sim_pointer[small_width * (i/2)] 
               + 3*sim_pointer[small_width * (i/2+1)] ) / 4;
        
        
        bim_pointer[big_width * (i+1) - 1] = 
            (3*sim_pointer[small_width * (i/2+1) - 1] 
             + sim_pointer[small_width * (i/2+2) - 1]) / 4; 
        
        bim_pointer[big_width * (i+2) - 1] = 
            (sim_pointer[small_width * (i/2+1) - 1] 
             + 3*sim_pointer[small_width * (i/2+2) - 1]) / 4; 
        
    }
    
    
    /* and finally calculate the pels in the middle of the image */
    
    
    for ( y=0 ; y < big_height-2 ; y+=2)
        for ( x=0 ; x < big_width-2 ; x+=2){
            
            /* Pick four pels, a[i],  from small image              */
            /*                                                      */
            /*  a[0]      a[1]                                      */
            /*                                                      */
            /*                                                      */
            /*                                                      */
            /*  a[3]      a[2]                                      */
            
            
            a[0] = sim_pointer[ (y/2)     * small_width + (x/2) ];
            a[1] = sim_pointer[ (y/2)     * small_width + (x/2 + 1)];
            a[2] = sim_pointer[ (y/2 + 1) * small_width + (x/2 + 1) ];
            a[3] = sim_pointer[ (y/2 + 1) * small_width + (x/2) ];      
            
            /* Calculate the values of pels, b[i], in the big image. */
            /*                                                       */
            /*  a[0]       a[1]                                      */
            /*     b[0] b[1]                                         */
            /*                                                       */
            /*     b[3] b[2]                                         */
            /*  a[3]       a[2]                                      */
            
            
            
            for (i = 0 ; i < 4 ; i++)
                b[i] = (9*a[i] + 3*a[(i+1)%4] + a[(i+2)%4] + 3*a[(i+3)%4]) / 16;
            
            /* Insert the calculated values in the image */   
            
            bim_pointer [(y+1) * big_width + x + 1] = b[0];
            bim_pointer [(y+1) * big_width + x + 2] = b[1];
            bim_pointer [(y+2) * big_width + x + 1] = b[3];
            bim_pointer [(y+2) * big_width + x + 2] = b[2];
            
        }
    
}





