/***********************************************************************
This software module was originally developed by Zvi Lifshitz
(Triton R&D Ltd.) in the course of development of the MPEG-4 Systems
(ISO/IEC 14496-1) standard. This software module is an implementation of
a part of one or more MPEG-4 Systems (ISO/IEC 14496-1) tools as
specified by the MPEG-4 Systems (ISO/IEC 14496-1) standard. ISO/IEC
gives users of the MPEG-4 Systems (ISO/IEC 14496-1) free license to
this software module or modifications thereof for use in hardware or
software products claiming conformance to the MPEG-4 Systems (ISO/IEC
14496-1). Those intending to use this software module in hardware or
software products are advised that its use may infringe existing patents.
The original developer of this software module and his/her company, the
subsequent editors and their companies, and ISO/IEC have no liability
for use of this software module or modifications thereof in an
implementation. Copyright is not released for non MPEG-4 Systems
(ISO/IEC 14496-1) conforming products. Triton R&D Ltd. retains full right
to use the code for his/her own purpose, assign or donate the code to a
third party and to inhibit third parties from using the code for non
MPEG-4 Systems (ISO/IEC 14496-1) conforming products. This copyright
notice must be included in all copies or derivative works.
Copyright  1998.
************************************************************************

For more information or to receive an up to date version of this module,
you may call Zvi Lifshitz at zvil@csi.com.
----------------------------------------------------------------------*/

/*-------------------------------------------------------------------------
Project Im1Player
INTERNATIONAL ORGANIZATION FOR STANDARDIZATION
ISO/IEC JTC1/SC29/WG11

SUBSYSTEM:  Decoder
FILE:       NullDec.cpp
AUTHOR:     Zvi Lifshitz (Triton R&D Ltd.)
OVERVIEW:   Implementation of NullDecoder - an object that servers as
			a template for devlopment of decoder plug-ins.
-------------------------------------------------------------------------*/
#include "Defs.h"
#include "NullDec.h"

/*-------------------------------------------------------------
The exported function that instantiates the proper decoder
-------------------------------------------------------------*/
//MBS 110498: changed returned type to "void *"
void *CreateInstance ()
{
	return (void *) new NullDecoder;
}

/*-------------------------------------------------------------
Sets the decoder to CIF or QCIF
-------------------------------------------------------------*/
BOOL NullDecoder :: Setup (
	const BYTE *,			// pSpecificInfo,
	int,					// nSpecificInfoLength,
	int
)
{
	return TRUE;
}

/*-----------------------------------------------------------------
Find a compatible format and initialize the decoder.
In this template we just pick up the first format.
-----------------------------------------------------------------*/
BOOL NullDecoder :: SetFormat (
	Capabilities &formatOptions,
	DecoderParams &decoderParams
)
{
	if (formatOptions.GetFormatCapabilityLength () == 0)
		return FALSE;
	decoderParams.m_nStreamNumber = 0;
	decoderParams.m_format = formatOptions.GetFormatCapability (0);
	decoderParams.m_nOptimalAUSize = 256;
	decoderParams.m_nDBSize = 1024;
	decoderParams.m_nCBSize = 100000;
	decoderParams.m_bRequiresFullAUs = TRUE;
	return TRUE;
}

/*-----------------------------------------------------------------
Calculate the output size in bytes
-----------------------------------------------------------------*/
int NullDecoder :: GetOptimalOutputSize ()
{
	return 256;
}

/*-------------------------------------------------------------
Decode one unit.
This template just copy input buffer to output buffer
-------------------------------------------------------------*/
BOOL NullDecoder :: Decode (
		LPBYTE pInput,
		int nInputLength,
		LPBYTE pOutput,
		int &rnOutputLength
)
{
	if (rnOutputLength < nInputLength)
	{	rnOutputLength = nInputLength;
		return FALSE;
	}

	memcpy (pOutput, pInput, nInputLength);
	return TRUE;
}

/*-------------------------------------------------------------
In this decoder, the decoded data always strats at beginning of output
block
-------------------------------------------------------------*/
int NullDecoder :: GetDataOffset  (const LPBYTE)
{
	return 0;
}

/*-------------------------------------------------------------
In this decoder, attributes are fixed for all units
-------------------------------------------------------------*/
DWORD NullDecoder :: GetAttributes (
	ATTRIBUTE_CODE,		// code, 
	const LPBYTE
)
{
	return 0;
}
