// -*- c++ -*-

/***********************************************************************
This software module was originally developed by Riitta Vaananen
(Helsinki University of Technology) in the course of development of
the MPEG-4 Systems (ISO/IEC 14496-1) standard.  This software module
is an implementation of a part of one or more MPEG-4 Systems (ISO/IEC
14496-1) tools as specified by the MPEG-4 Systems (ISO/IEC 14496-1)
standard. ISO/IEC gives users of the MPEG-4 Systems (ISO/IEC 14496-1)
free license to this software module or modifications thereof for use
in hardware or software products claiming conformance to the MPEG-4
Systems (ISO/IEC 14496-1). Those intending to use this software module
in hardware or software products are advised that its use may infringe
existing patents.  The original developer of this software module and
his/her company, the subsequent editors and their companies, and
ISO/IEC have no liability for use of this software module or
modifications thereof in an implementation.  Copyright is not released
for non MPEG-4 Systems (ISO/IEC 14496- 1) conforming products.
Helsinki University of Technology retains full right to use the code
for their own purpose, assign or donate the code to a third party and
to inhibit third parties from using the code for non MPEG-4 Systems
(ISO/IEC 14496-1) conforming products. This copyright notice must be
included in all copies or derivative works. Copyright  1999.
************************************************************************/


#ifndef __ACOUSTICSCENEPROXY_H__
#define __ACOUSTICSCENEPROXY_H__

#include "OpenGl/group/GroupProxy.h"

// SB %%%% 99/05/10
#include "opengl/AdvancedAudio/AudioProcess/ProcessSound.h"

class AcousticScene;
class AcousticSurface;
class Auralize;

/** AcousticScene renderer node.*/
class AcousticSceneProxy
  : public Node3DProxy
{
	friend class ProcessDirectiveSound;

public:

  /// Constructor

                    AcousticSceneProxy(AcousticScene* pAcousticScene);

					~AcousticSceneProxy();

  // Render object and its children

  virtual BOOL		Render(const Matrix& rTransformation, const CullFlag& rCullFlag,
			       OpenGLVisualRenderer* pRenderer, DWORD dwTime);

 /// Render object description

  virtual void		RenderDescription(OpenGLVisualRenderer* pRenderer, DWORD dwTime);

	virtual int		FindAcousticScenes(
						ZArray<AcousticScene*>& rAcousticScenes, 
						ZArray<Matrix>& rASceneTransformations,
						const Matrix& rTransform);

	void FindAcousticShapes(Group* pGroup);
	
	inline AcousticScene * GetAcousticScene()
		{ return (AcousticScene *) GetMediaObject(); }

	void MakeAcousticPolygons();
	void AddToAcousticSurfaceArray(Shape* acoustic_shape);
	void MakeAcousticSurfaceArray(BOOL is_update);
	void FindTransformsInParents(MediaObject* AS);
	void UpdateAcousticPolygons();
	Matrix GetASTransform(void);

	BOOL polygons_done;
	ZArray<Shape*> acousticshapes;
	ZArray<Matrix> trAShape;
//	ZArray<Transform*> transforms;
	ZArray<Transform*> transforms_AS;

	Matrix AS_transf;
	ZArray<AcousticSurface*> a_surfaces;
//	ZArray<int*> ashape_IDs;
//	ZArray<int*> transform_IDs;

};

/** class for handling one AcousticSurface. */

class 
AcousticSurface
{
	friend AcousticSceneProxy;
	friend ProcessDirectiveSound;
	friend Auralize;
	friend ImageSource;
	friend SoundSource;
	//friend SoundSources;

	/// Constructor
	AcousticSurface::AcousticSurface(AcousticMaterial* a_mat, Vec3f* poly_coords, int nof_v);

	/** Destructor.
	Deletes filter coefficient arrays.*/
	~AcousticSurface();

private:

	/** Puts filter coefficients to arrays.
	And sets the number of coefficients variable*/
	void SetFilters(AcousticMaterial* a_mat);

	/** Computes normals for a polygon*/
	void SetNormals(Vec3f* coor);

	/** Updates surface if it has changed. */
	void UpdateSurface(AcousticMaterial* a_mat, Vec3f* poly_coords, int nof_v);	
	
	/** vertices of the polygon */
	Vec3f* vertices;
	int nof_vert;

	/** Number of reflection filter coefficients */
	int nof_refcoeffs;

	/** Number of transmission filter coefficients. */
	int nof_transcoeffs;


	/** a coefficients of reflectivity filter*/
	float* ref_a;

	/** b coefficients of reflectivity filter*/
	float* ref_b;

	/** a coefficients of transmission filter*/
	float* trans_a;

	/** b coefficients of transmission filter*/
	float* trans_b;

	/** Normal of the polygon*/
	Vec3f normal;

	/** A point on the surface*/
	Vec3f surface_point;

	/** Pointer to a corresponding IIRFilter */
	IIRFilt* iirfilt;

};



#include "AcousticSceneProxy.i"

#endif // __ACOUSTICSCENEPROXY_H__

