#ifdef ADV_AUDIO

// -*- c++ -*-

/***********************************************************************
This software module was originally developed by Riitta Vaananen
(Helsinki University of Technology) in the course of development of
the MPEG-4 Systems (ISO/IEC 14496-1) standard.  This software module
is an implementation of a part of one or more MPEG-4 Systems (ISO/IEC
14496-1) tools as specified by the MPEG-4 Systems (ISO/IEC 14496-1)
standard. ISO/IEC gives users of the MPEG-4 Systems (ISO/IEC 14496-1)
free license to this software module or modifications thereof for use
in hardware or software products claiming conformance to the MPEG-4
Systems (ISO/IEC 14496-1). Those intending to use this software module
in hardware or software products are advised that its use may infringe
existing patents.  The original developer of this software module and
his/her company, the subsequent editors and their companies, and
ISO/IEC have no liability for use of this software module or
modifications thereof in an implementation.  Copyright is not released
for non MPEG-4 Systems (ISO/IEC 14496- 1) conforming products.
Helsinki University of Technology retains full right to use the code
for their own purpose, assign or donate the code to a third party and
to inhibit third parties from using the code for non MPEG-4 Systems
(ISO/IEC 14496-1) conforming products. This copyright notice must be
included in all copies or derivative works. Copyright  1997.
************************************************************************/


#include "Include/defs.h"
#include "AcousticSceneProxy.h"
#include "opengl/base/Matrix.h"
#include "opengl/base/OpenGLPresenter.h"
#include <GL/gl.h>

#include "opengl/group/TransformProxy.h"

// Add node to draw list

/* virtual */BOOL
AcousticSceneProxy::Render(
    const Matrix& /* rTransformation */, 
    const CullFlag& /* rCullFlag */,
		OpenGLVisualRenderer * /* pRenderer */, 
    DWORD /* dwTime */)
{
	return TRUE;
}



// Render object description

/* virtual */ void
AcousticSceneProxy::RenderDescription(OpenGLVisualRenderer* pRenderer, DWORD dwTime)
{

  RenderDescriptionBox(pRenderer, "AcousticScene");

}

int
AcousticSceneProxy::FindAcousticScenes(
						ZArray<AcousticScene*>& rAcousticScenes, 
						ZArray<Matrix>& rASceneTransformations,
						const Matrix& rTransform)
{
	rAcousticScenes += (AcousticScene*) GetMediaObject();
	rASceneTransformations +=rTransform;
	return 1;
}

// The following is taken from AcousticScene.cpp // riitta 10029


// ***********************************************************
// Here is what is called from the ProcessDirectiveSound
// **********************************************************

void 
AcousticSceneProxy::MakeAcousticPolygons()
{
	int i=0;
	if(polygons_done == FALSE){ 
	//	int depth = 0;	
	//	AcousticScene* pAcousticScene = (AcousticScene*) GetMediaObject();
		MediaObject* pGroup = GetMediaObject()->GetParent(0);

		FindTransformsInParents(pGroup);
		
		// Concatenate the transforms above the AcousticScene
		for(i=(transforms_AS.GetLength()-1);i>=0;i--){
			((TransformProxy*)transforms_AS[i]->GetRenderingObject())->ConcatenateTransform(AS_transf);
		}

		FindAcousticShapes((Group*)pGroup);
		MakeAcousticSurfaceArray(FALSE);
		polygons_done = TRUE;
	}
	else
		UpdateAcousticPolygons();

}


void 
AcousticSceneProxy::UpdateAcousticPolygons()
{

	int i=0; // crash here
	acousticshapes.Clear();
	transforms_AS.Clear();

	MediaObject* pGroup = GetMediaObject()->GetParent(0);

	FindTransformsInParents(pGroup);
	Matrix tmp;
	AS_transf = tmp;
	for(i=(transforms_AS.GetLength()-1);i>=0;i--){
		((TransformProxy*)transforms_AS[i]->GetRenderingObject())->ConcatenateTransform(AS_transf); // AS_tr on jo transformoitu edellisella kierroksella!!
	}

	FindAcousticShapes((Group*)pGroup);

	MakeAcousticSurfaceArray(TRUE);	
}

Matrix AcousticSceneProxy::GetASTransform(void)
{
	return AS_transf;
}

void AcousticSceneProxy::FindAcousticShapes(Group* pGroupnode)
{
	acousticshapes.Clear();
	trAShape.Clear();

	MediaObjectProxy* pProxy = (MediaObjectProxy*)(pGroupnode->GetRenderingObject());
	pProxy->FindAcousticSurfaces(acousticshapes, trAShape, AS_transf);
}


void 
AcousticSceneProxy::FindTransformsInParents(MediaObject* AS)
{
	MediaObject* par = AS->GetParent(0); //riitta 100299

	if(par) {
		if(par->GetNodeCode() == Transform::GetCode()){
		 transforms_AS += (Transform*)(par);
		}
			FindTransformsInParents((MediaObject*)par);
	}
}

void
AcousticSceneProxy::MakeAcousticSurfaceArray(BOOL is_update){
	
	int i=0;
	int j=0;
	int k=0;
	int l=0;
	int m=0;
	
	m=0;

	for(i=0;i<acousticshapes.GetLength();i++){ 

		MediaObject* mo = (acousticshapes[i])->geometry;
		IndexedFaceSet* ifs = (IndexedFaceSet*)mo;
		const MF<Vec3f>& coords = (ifs->coord)->point;
	
		MF<Vec3f> transformed_coords;

#ifndef BIFSENC
		for(k=0;k<coords.GetLength();k++){
			transformed_coords += ProcessSound::TransformPoints(trAShape[i], coords[k]);
		}
#endif
		mo = acousticshapes[i]->geometry;
		const MF<int>& coordindex = ((IndexedFaceSet*)mo)->coordIndex;		
		mo = acousticshapes[i]->appearance;
		Appearance* app = (Appearance*)mo;
		mo = app->material;
		AcousticMaterial* a_mat = (AcousticMaterial*)mo;

		for(j=0;j<coordindex.GetLength();){
		Vec3f* poly = new Vec3f[14];
			int index = 0;
			while(coordindex[j] != -1){
				poly[index++] = transformed_coords[coordindex[j]];
				j++;
			}//	while : j = nof indices in this polygon

			if(is_update == FALSE){
				a_surfaces += new AcousticSurface(a_mat, poly, index);
				m++;
			}
			if(is_update == TRUE){
				a_surfaces[m]->UpdateSurface(a_mat, poly, index);
				m++;				
			}

			j++; 

			delete [] poly;
		} //for
	} // for
}

AcousticSurface::AcousticSurface(AcousticMaterial* a_mat, Vec3f* poly_coords, int nof_v):
ref_b(NULL),
ref_a(NULL),
trans_b(NULL),
trans_a(NULL),
vertices(NULL),
nof_vert(nof_v)
{
	
	int i=0;
	vertices = new Vec3f[nof_vert];
	for(i=0;i<nof_vert;i++){
		vertices[i] = poly_coords[i];
	}
	SetFilters(a_mat); // ok?!
	SetNormals(poly_coords);

}


AcousticSurface::~AcousticSurface()
{
	COMPLAIN("deleting acoustic surface");
	if(ref_b){
		delete [] ref_b;
		ref_b = NULL;
	}
	if(ref_a){
		delete [] ref_a;
		ref_a = NULL;
	}
	if(trans_b){
		delete [] trans_b;
		trans_b = NULL;
	}
	if(trans_a){
		delete [] trans_a;
		trans_a = NULL;
	}
	if(vertices){
		delete [] vertices;
		vertices = NULL;
	}
}

void 
AcousticSurface::SetFilters(AcousticMaterial* a_mat)
{
	int i=0;
	int ref_len = (a_mat->reffunc).GetLength();
	float* ref_filt_coeffs = new float[ref_len];

	for(i=0;i<ref_len;i++)
		ref_filt_coeffs[i] = a_mat->reffunc[i];

	nof_refcoeffs = (ref_len-1)/2 + 1;

	int trans_len = (a_mat->transfunc).GetLength();
	float* trans_filt_coeffs = new float[trans_len];

	for(i=0;i<trans_len;i++)
		trans_filt_coeffs[i] = a_mat->transfunc[i];


	nof_transcoeffs = (trans_len-1)/2 +1;
	
	if(ref_len > 0)
	{
		if(nof_refcoeffs == 1){
			ref_b = new float[1];
			ref_b[0] = ref_filt_coeffs[0];
		}

		else{
			ref_b = new float[nof_refcoeffs];
			ref_a = new float[nof_refcoeffs];
			for(i=0;i<nof_refcoeffs;i++){
				ref_b[i] = ref_filt_coeffs[i];
			}
			ref_a[0] = 1.0;
			for(i=0;i<nof_refcoeffs-1;i++){
				ref_a[i+1] = ref_filt_coeffs[i+nof_refcoeffs];
			}
		}
}
else{
	nof_refcoeffs = 1;
	ref_b = new float[1];
	ref_b[0] = 0.0;
}

if(trans_len > 0)
{
		if(nof_transcoeffs == 1){
			trans_b = new float[1];
			trans_b[0] = trans_filt_coeffs[0];
		}
		else{
			trans_a = new float[nof_transcoeffs];
			trans_b = new float[nof_transcoeffs];
			for(i=0;i<nof_transcoeffs;i++){
				trans_b[i] = trans_filt_coeffs[i]; 
				//trans_b += trans_filt_coeffs[i]; 
			}
			//trans_a += 1.0;
			trans_a[0] = 1.0;
			for(i=0;i<nof_transcoeffs-1;i++){
				trans_a[i+1] = trans_filt_coeffs[i+nof_transcoeffs];
			}
		}
}
else{
	trans_b = new float[1];
	nof_transcoeffs = 1;
	trans_b[0] = 1.0;
}

delete [] ref_filt_coeffs;
delete [] trans_filt_coeffs;
}

void 
AcousticSurface::SetNormals(Vec3f* coor)
{
		int l;
		int i=0;

		surface_point.x = 0.0;
		surface_point.y = 0.0;
		surface_point.z = 0.0;
		for(i=0;i<nof_vert;i++){
			surface_point += coor[i];
		}
	
		surface_point.x /= nof_vert;
		surface_point.y /= nof_vert;
		surface_point.z /= nof_vert;

		Vec3f v1;
		Vec3f v2;

		v1.x = coor[0].x - surface_point.x;
		v1.y = coor[0].y - surface_point.y;
		v1.z = coor[0].z - surface_point.z;

		v2.x = coor[1].x - surface_point.x;
		v2.y = coor[1].y - surface_point.y;
		v2.z = coor[1].z - surface_point.z;

		normal.x = v1.y * v2.z - v1.z * v2.y;
		normal.y = v1.z * v2.x - v1.x * v2.z;
		normal.z = v1.x * v2.y - v1.y * v2.x; 

		float n = sqrt(pow(normal.x,2) + pow(normal.y,2) + pow(normal.z,2)); 
		normal.x /= n;
		normal.y /= n;
		normal.z /= n;

}


void 
AcousticSurface::UpdateSurface(AcousticMaterial* a_mat, Vec3f* poly_coords, int nof_v)
{

	int i=0;
	int len1 = nof_vert; 
	int len2 = nof_v;  
	if(len1 == len2){
		for(i=0;i<len1;i++){
			if(vertices[i] == poly_coords[i]){}
			else{
				SetNormals(poly_coords);
				i=len1; 
			}	
		}	
	}
	else{
		SetNormals(poly_coords);	
	}
	nof_vert = nof_v;  
	for(i=0;i<nof_vert;i++){
		vertices[i] = poly_coords[i];
	}
}

#endif
