// -*- c++ -*-

// This software module was originally developed by Karl Anders ygard
// (Telenor R&D) in the course of development of the MPEG-4 Systems
// standard (ISO/IEC 14496-1).  This software module is an implementation
// of a part of one or more MPEG-4 Systems tools as specified by the MPEG-4
// Systems standard.  ISO/IEC gives users of the MPEG-4 Systems standard
// free license to this software module or modifications thereof for use in
// hardware or software products claiming conformance to the MPEG-4 Systems
// standard.  Those intending to use this software module in hardware or
// software products are advised that its use may infringe existing
// patents.  The original developer of this software module and his/her
// company, the subsequent editors and their companies, and ISO/IEC have no
// liability for use of this software module or modifications thereof in an
// implementation.  Copyright is not released for non MPEG-4 Systems
// conforming products.  The original developer retains full right to use
// the code for his/her own purpose, assign or donate the code to a third
// party and to inhibit third parties from using the code for non MPEG-4
// Systems conforming products.  This copyright notice must be included in
// all copies or derivative works.
//
// Copyright 1997, 1998.

#include "OpenGl/OpenGl.h"
#include "MovieTextureProxy.h"
#include "opengl/base/OpenGLPresenter.h"
#include "opengl/base/OpenGLVisualRenderer.h"

#include "Stream.h"
//#include "Executiv.h"
#include "Presenter.h"

// Render object description

/* virtual */ void
MovieTextureProxy::RenderDescription(OpenGLVisualRenderer* pRenderer, DWORD /* dwTime */)
{
  RenderDescriptionBox(pRenderer, "ImageTexture");
}

// Enable texture

/* virtual */ BOOL
MovieTextureProxy::Enable(
    OpenGLVisualRenderer * pRenderer, 
    DWORD /* dwTime */)
{
  MovieTexture* pTexture = (MovieTexture*) GetMediaObject();
  MediaStream* pStream = pTexture->GetStream();
  
  if (pStream)
  {
    /* pTexture-> */ TestActive();
    if (/* pTexture-> */ GetActive())
    {
      int nSize;
      DWORD dwTime;
      BOOL bEOU;
      int nStatus;
      BOOL bSkip;
      BOOL bWait;
      long lDelta;
      BOOL bDontClose;
      BOOL bRecurrent;
      
      pRenderer->RegisterStreamConsumer(pTexture);

      /***
      unsigned char* pData = (unsigned char*) pStream->
          Fetch(nSize,
          dwTime,
          bRecurrent,
          TRUE,
          FALSE);
      ***/
      // SB @@@@ I am calling Fetch without using nRecurrent!!!
      // what is th edefault? guess: FALSE
      bSkip = TRUE;
      bWait = FALSE;
      lDelta = 0;
      bDontClose = TRUE;
      unsigned char * pData = (unsigned char *) pStream->Fetch(
        nSize, dwTime, bEOU, nStatus, bSkip, bWait, lDelta, bDontClose);
      bRecurrent = FALSE;
      
      if (pData != END_OF_STREAM) {
        if (pData)
          if (!bRecurrent) {
            int nWidth = (((unsigned short) pData[0]) << 8) | pData[1];
            int nHeight = (((unsigned short) pData[2]) << 8) | pData[3];
            BOOL bRet;
            
            // Enable texture object
            
            bRet = TextureNodeProxy::Enable(pRenderer,
              pTexture->GetStreamFormat() == /* FORMAT_RGBA */ MediaFormat::RGBA ? RGBA : RGB,
              nWidth, nHeight, pData + 4);
            
            pStream->Release(TRUE);
            
            return bRet;
          } else
            pStream->Release(TRUE);
      }
      else
      {
        if (/* pTexture-> */ GetLoop())
        {
          pTexture->CloseStream();
        }
      }
    }
    
    return TextureNodeProxy::Enable(pRenderer);
  }
  
  return TRUE;
}


// Stefano Battista
// moved from MovieTexture to Proxy

// Test whether movie should be activated or deactivated

void MovieTextureProxy::TestActive()
{
  MovieTexture * poMovieTexture = GetMovieTexture();
  Time currentTime = poMovieTexture->GetRoot()->GetTime() / 1000.0;
  
  if (m_bActive)
  {
    if ((Time) poMovieTexture->startTime > currentTime)
    {
      Stop();
    }
    else
    {
      if (!(Bool) poMovieTexture->loop)
      {
        if ((Time) poMovieTexture->stopTime <= currentTime && 
            poMovieTexture->stopTime > poMovieTexture->startTime)
        {
          Stop();
        }
      }
    }
  }
  else
  {
    if ((Time) poMovieTexture->startTime <= currentTime)
    {
      if (poMovieTexture->stopTime < poMovieTexture->startTime)
      {
        Start();
      }
      else
      {
        if ((Bool) poMovieTexture->loop && 
            poMovieTexture->stopTime == poMovieTexture->startTime)
        {
          Start();
        }
        else
        {
          if (poMovieTexture->stopTime > poMovieTexture->startTime && 
              poMovieTexture->stopTime > currentTime)
          {
            Start();
          }
        }
      }
    }
  }
}


// Start movie

void MovieTextureProxy::Start()
{
#ifndef BIFSENC
  MovieTexture * poMovieTexture = GetMovieTexture();
  poMovieTexture->isActive = /* m_bActive = */ TRUE;
  if (poMovieTexture->GetStream())
  {
    OpenGLPresenter * poPresenter;
    poPresenter = (OpenGLPresenter *) poMovieTexture->GetExecutive()->GetPresenter();
    poPresenter->GetVisualRenderer()->RegisterStreamConsumer(poMovieTexture);
  }
#endif
}


// Stop movie

void MovieTextureProxy::Stop()
{
#ifndef BIFSENC
  MovieTexture * poMovieTexture = GetMovieTexture();
  poMovieTexture->isActive = /* m_bActive = */ FALSE;
  {
    OpenGLPresenter * poPresenter;
    poPresenter = (OpenGLPresenter *) poMovieTexture->GetExecutive()->GetPresenter();
    poPresenter->GetVisualRenderer()->RegisterStreamConsumer(poMovieTexture);
  }
  poMovieTexture->CloseStream();
  // PauseStream();
#endif
}
