// -*- c++ -*-

// This software module was originally developed by Karl Anders ygard
// (Telenor R&D), in the course of development of the MPEG-4 Systems
// standard (ISO/IEC 14496-1).  This software module is an implementation
// of a part of one or more MPEG-4 Systems tools as specified by the MPEG-4
// Systems standard.  ISO/IEC gives users of the MPEG-4 Systems standard
// free license to this software module or modifications thereof for use in
// hardware or software products claiming conformance to the MPEG-4 Systems
// standard.  Those intending to use this software module in hardware or
// software products are advised that its use may infringe existing
// patents.  The original developer of this software module and his/her
// company, the subsequent editors and their companies, and ISO/IEC have no
// liability for use of this software module or modifications thereof in an
// implementation.  Copyright is not released for non MPEG-4 Systems
// conforming products.  The original developer retains full right to use
// the code for his/her own purpose, assign or donate the code to a third
// party and to inhibit third parties from using the code for non MPEG-4
// Systems conforming products.  This copyright notice must be included in
// all copies or derivative works.
//
// Copyright 1997, 1998.

#include "OpenGl/OpenGl.h"
#include "OpenGLPresenter.h"

#include "OpenGLVisualRenderer.h"
#include "Executiv.h"

#include "opengl/base/MediaObjectProxy.h"
#include "opengl/base/NodeProxy.h"
#include "opengl/base/Node2DProxy.h"
#include "opengl/base/Node3DProxy.h"

#include "opengl/base/AnimationStreamProxy.h"
#include "opengl/base/ConditionalProxy.h"
#include "opengl/base/TermCapProxy.h"

#include "opengl/group/AnchorProxy.h"
#include "opengl/group/BillboardProxy.h"
#include "opengl/group/LayoutProxy.h"
#include "opengl/group/FormProxy.h"
#include "opengl/group/GroupProxy.h"
#include "opengl/group/InlineProxy.h"
#include "opengl/group/LODProxy.h"
#include "opengl/group/SwitchProxy.h"
#include "opengl/group/TransformProxy.h"
#include "opengl/group/Transform2DProxy.h"
#ifdef PRE_FDIS
#include "opengl/group/Anchor2DProxy.h"
#include "opengl/group/Group2DProxy.h"
#include "opengl/group/Inline2DProxy.h"
#include "opengl/group/Switch2DProxy.h"
#endif // PRE_FDIS

#include "opengl/geometry/ShapeProxy.h"
#include "opengl/geometry/BoxProxy.h"
#include "opengl/geometry/SphereProxy.h"
#include "opengl/geometry/CylinderProxy.h"
#include "opengl/geometry/ConeProxy.h"
#include "opengl/geometry/IndexedFaceSetProxy.h"
#include "opengl/geometry/IndexedLineSetProxy.h"
#include "opengl/geometry/PointSetProxy.h"
#include "opengl/geometry/TextProxy.h"
#include "opengl/geometry/CircleProxy.h"
#include "opengl/geometry/RectangleProxy.h"
#include "opengl/geometry/IndexedFaceSet2DProxy.h"
#include "opengl/geometry/IndexedLineSet2DProxy.h"
#include "opengl/geometry/PointSet2DProxy.h"
#include "opengl/geometry/ElevationGridProxy.h"
#ifdef PRE_FDIS
#include "opengl/geometry/Image2DProxy.h"
#include "opengl/geometry/StreamingTextProxy.h"
#include "opengl/geometry/VideoObject2DProxy.h"
#endif // PRE_FDIS

#include "opengl/appearance/AppearanceProxy.h"
#include "opengl/appearance/MaterialProxy.h"
#include "opengl/appearance/TextureNodeProxy.h"
#include "opengl/appearance/ImageTextureProxy.h"
#include "opengl/appearance/MovieTextureProxy.h"
#include "opengl/appearance/PixelTextureProxy.h"
#include "opengl/appearance/TextureTransformProxy.h"
#include "opengl/appearance/Material2DProxy.h"
#include "opengl/appearance/LinePropertiesProxy.h"

#include "opengl/bindable/BindableProxy.h"
#include "opengl/bindable/BackgroundProxy.h"
#include "opengl/bindable/ViewpointProxy.h"
#include "opengl/bindable/FogProxy.h"

#include "opengl/sensors/DragSensorProxy.h"
#include "opengl/sensors/TouchSensorProxy.h"
#include "opengl/sensors/TimeSensorProxy.h"
#include "opengl/sensors/ProximitySensorProxy.h"
#include "opengl/sensors/VisibilitySensorProxy.h"

#include "opengl/interpolators/InterpolatorNodeProxy.h"

#include "opengl/light/DirectionalLightProxy.h"
#include "opengl/light/PointLightProxy.h"
#include "opengl/light/SpotLightProxy.h"

//#ifdef ML_CANNOT_FIX
#include "opengl/fba/FaceProxy.h"
#include "opengl/fba/FDPProxy.h"
//#endif

// Tolga 00/03/28: body animation
#include "opengl/fba/BodyProxy.h"
#include "opengl/fba/BAPProxy.h"
#include "opengl/fba/BDPProxy.h"


#include "opengl/audio/SoundProxy.h"
#include "opengl/audio/Sound2DProxy.h"
#include "opengl/audio/AudioSourceProxy.h"
#ifdef PRE_FDIS
#include "opengl/audio/AudioObjectProxy.h"
#endif // PRE_FDIS

#ifdef ADV_AUDIO
#include "OpenGl/AdvancedAudio/AcousticMaterialProxy.h"
#include "OpenGl/AdvancedAudio/AcousticSceneProxy.h"
#include "OpenGl/AdvancedAudio/DirectiveSoundProxy.h"
#include "OpenGl/AdvancedAudio/PerceptualParametersProxy.h"
#endif // ADV_AUDIO

#if defined(_IRIX)
#include "AudioRendererSgi.h"
#elif defined(_SOLARIS)
#include "AudioRendererSol.h"
#elif defined(linux)
#include "AudioRendererLinux.h"
#else
#include "AudioRenderer.h"
#endif

// Constructor

OpenGLPresenter::OpenGLPresenter(Executive* pExecutive, HWND hWnd)
  : Presenter(pExecutive, hWnd),
    m_dwLoop(0),
    m_nMouseButtons(0)
{
  m_pVisualRenderer = new OpenGLVisualRenderer(this);

#if defined(_IRIX)
  m_pAudioRenderer = new AudioRendererSgi(this, hWnd);
#elif defined(_SOLARIS)
  m_pAudioRenderer = new AudioRendererSol(this, hWnd);
#elif defined(linux)
  m_pAudioRenderer = new AudioRendererLinux(this, hWnd);
#else
  m_pAudioRenderer = new AudioRenderer(this, hWnd);
#endif

  m_fLogFile = fopen ("PresenterLog.Txt", "w");

}

// Destructor

/* virtual */
OpenGLPresenter::~OpenGLPresenter()
{
  if (m_pVisualRenderer)
    delete m_pVisualRenderer;
    
  if (m_pAudioRenderer)
    delete m_pAudioRenderer;
}

// Terminate

/* virtual */ void
OpenGLPresenter::Terminate()
{
  TRACE("[OpenGLPresenter::Terminate] begin\n");
  m_pVisualRenderer->Terminate();
  m_pAudioRenderer->Terminate();

  //ML - added the next line
  Presenter::Terminate();

  TRACE("[OpenGLPresenter::Terminate] end\n");
}

// Initialise presenter
  
/* virtual */ 
// BOOL
void
OpenGLPresenter::Init()
{
  TRACE("[OpenGLPresenter::Init] begin\n");

  //ML - added the next line
  Presenter::Init();

  // if (Presenter::Init()) {
  m_pVisualRenderer->Init();
  m_pAudioRenderer->Init();
  
  m_clock.Reset();
  TRACE("[OpenGLPresenter::Init] end\n");
  return;
  
  //   return TRUE;
  // } else
  // return FALSE;
}

// Handle mouse events

/* virtual */ void
OpenGLPresenter::OnUserEvent(USER_EVENT event, int nX, int nY, UINT)
{
  switch (event) {
  case EVENT_LEFT_BUTTON_UP:
    m_nMouseButtons &= ~VisualRenderer::LEFT_BUTTON;
    break;
  case EVENT_MIDDLE_BUTTON_UP:
    m_nMouseButtons &= ~VisualRenderer::MIDDLE_BUTTON;
    break;
  case EVENT_RIGHT_BUTTON_UP:
    m_nMouseButtons &= ~VisualRenderer::RIGHT_BUTTON;
    break;
  case EVENT_LEFT_BUTTON_DOWN:
    m_nMouseButtons |= VisualRenderer::LEFT_BUTTON;
    break;
  case EVENT_MIDDLE_BUTTON_DOWN:
    m_nMouseButtons |= VisualRenderer::MIDDLE_BUTTON;
    break;
  case EVENT_RIGHT_BUTTON_DOWN:
    m_nMouseButtons |= VisualRenderer::RIGHT_BUTTON;
    break;
  case EVENT_MOUSE_MOVE:
    break;
  default:
    assert(!"Unknown user event");
    break;
  }

  m_pVisualRenderer->MouseState(m_nMouseButtons, nX, nY);
}

// init MediaObject

RenderingObject* OpenGLPresenter::CreateMediaObjectProxy(MediaObject* pObject)
{
  return new MediaObjectProxy(pObject);
}

// init Node

RenderingObject* OpenGLPresenter::CreateNodeProxy(/* Node */ MediaObject * pObject)
{
  return new NodeProxy(pObject);
}

// init Node3D

RenderingObject* OpenGLPresenter::CreateNode3DProxy(/* Node3D */ MediaObject * pObject)
{
  return new Node3DProxy(pObject);
}

// init Node2D

RenderingObject* OpenGLPresenter::CreateNode2DProxy(/* Node2D */ MediaObject * pObject)
{
  return new Node2DProxy(pObject);
}

// init Bindable

RenderingObject* OpenGLPresenter::CreateBindableProxy(/* Bindable */ MediaObject * pObject)
{
  return new BindableProxy(pObject);
}

// init InterpolatorNode

RenderingObject* OpenGLPresenter::CreateInterpolatorNodeProxy(/* InterpolatorNode */ MediaObject * pObject)
{
  return new InterpolatorNodeProxy(pObject);
}

// init Anchor

RenderingObject* OpenGLPresenter::CreateAnchorProxy(Anchor* pObject)
{
  return new AnchorProxy(pObject);
}

// init Anchor2D

#ifdef PRE_FDIS
RenderingObject* OpenGLPresenter::CreateAnchor2DProxy(Anchor2D* pObject)
{
  return new Anchor2DProxy(pObject);
}
#endif // PRE_FDIS

// init AnimationStream

RenderingObject* OpenGLPresenter::CreateAnimationStreamProxy(AnimationStream* pObject)
{
  return new AnimationStreamProxy(pObject);
}

// init Appearance

RenderingObject* OpenGLPresenter::CreateAppearanceProxy(Appearance* pObject)
{
  return new AppearanceProxy(pObject);
}

// init AudioObject

#ifdef PRE_FDIS
RenderingObject* OpenGLPresenter::CreateAudioObjectProxy(AudioObject * pObject)
{
  return new MediaObjectProxy(pObject);
}
#endif PRE_FDIS

// init AudioSource

RenderingObject* OpenGLPresenter::CreateAudioSourceProxy(AudioSource* pObject)
{
  return new AudioSourceProxy(pObject);
}

// init Billboard

RenderingObject* OpenGLPresenter::CreateBackgroundProxy(Background * poObject)
{
  return new BackgroundProxy(poObject);
}

// init Billboard

RenderingObject* OpenGLPresenter::CreateBillboardProxy(Billboard* pObject)
{
  return new BillboardProxy(pObject);
}

// init Box

RenderingObject* OpenGLPresenter::CreateBoxProxy(Box* pObject)
{
  return new BoxProxy(pObject);
}

// init Circle

RenderingObject* OpenGLPresenter::CreateCircleProxy(Circle* pObject)
{
  return new CircleProxy(pObject);
}

// init Conditional

RenderingObject* OpenGLPresenter::CreateConditionalProxy(Conditional* pObject)
{
  return new ConditionalProxy(pObject);
}

// init Cone

RenderingObject* OpenGLPresenter::CreateConeProxy(Cone* pObject)
{
  return new ConeProxy(pObject);
}

// init Cylinder

RenderingObject* OpenGLPresenter::CreateCylinderProxy(Cylinder* pObject)
{
  return new CylinderProxy(pObject);
}

// init DirectionalLight

RenderingObject* OpenGLPresenter::CreateDirectionalLightProxy(DirectionalLight* pObject)
{
  return new DirectionalLightProxy(pObject);
}

// init DragSensor

RenderingObject* OpenGLPresenter::CreateDragSensorProxy(/* DragSensor */ MediaObject * pObject)
{
  return new DragSensorProxy(pObject);
}

// init ElevationGrid

RenderingObject* OpenGLPresenter::CreateElevationGridProxy(ElevationGrid* pObject)
{
  return new ElevationGridProxy(pObject);
}

//#ifdef ML_CANNOT_FIX
// init Face

RenderingObject* OpenGLPresenter::CreateFaceProxy(Face* pObject)
{
  return new FaceProxy(pObject);
}

// init FDP

RenderingObject* OpenGLPresenter::CreateFDPProxy(FDP* pObject)
{
  return new FDPProxy(pObject);
}
//#endif

// init Fog

RenderingObject* OpenGLPresenter::CreateFogProxy(Fog* pObject)
{
  return new FogProxy(pObject);
}

// init Form

RenderingObject* OpenGLPresenter::CreateFormProxy(Form* pObject)
{
  return new FormProxy(pObject);
}

// init Group

RenderingObject* OpenGLPresenter::CreateGroupProxy(Group* pObject)
{
  return new GroupProxy(pObject);
}

// init Group2D

#ifdef PRE_FDIS
RenderingObject* OpenGLPresenter::CreateGroup2DProxy(Group2D* pObject)
{
  return new Group2DProxy(pObject);
}
#endif // PRE_FDIS

// init Image2D

#ifdef PRE_FDIS
RenderingObject* OpenGLPresenter::CreateImage2DProxy(Image2D* pObject)
{
  return new Image2DProxy(pObject);
}
#endif PRE_FDIS

// init ImageTexture

RenderingObject* OpenGLPresenter::CreateImageTextureProxy(ImageTexture * pObject)
{
  return new ImageTextureProxy(pObject);
}

// init IndexedFaceSet

RenderingObject* OpenGLPresenter::CreateIndexedFaceSetProxy(IndexedFaceSet* pObject)
{
  return new IndexedFaceSetProxy(pObject);
}

// init IndexedFaceSet2D

RenderingObject* OpenGLPresenter::CreateIndexedFaceSet2DProxy(IndexedFaceSet2D* pObject)
{
  return new IndexedFaceSet2DProxy(pObject);
}

// init IndexedLineSet

RenderingObject* OpenGLPresenter::CreateIndexedLineSetProxy(IndexedLineSet* pObject)
{
  return new IndexedLineSetProxy(pObject);
}

// init IndexedLineSet2D

RenderingObject* OpenGLPresenter::CreateIndexedLineSet2DProxy(IndexedLineSet2D* pObject)
{
  return new IndexedLineSet2DProxy(pObject);
}

// init Inline

RenderingObject* OpenGLPresenter::CreateInlineProxy(Inline* pObject)
{
  //ML - commented out next line
  //((GroupNode *) pObject)->SetRenderingObject(new InlineProxy(pObject));
  return new InlineProxy(pObject);
}

// init Inline2D

#ifdef PRE_FDIS
RenderingObject* OpenGLPresenter::CreateInline2DProxy(Inline2D* pObject)
{
  //ML - commented out next line
  //((GroupNode *) pObject)->SetRenderingObject(new Inline2DProxy(pObject));
  pObject->SetRenderingObject(new Inline2DProxy(pObject));
}
#endif // PRE_FDIS

// init Layout

RenderingObject* OpenGLPresenter::CreateLayoutProxy(Layout* pObject)
{
  return new LayoutProxy(pObject);
}

// init LineProperties

RenderingObject* OpenGLPresenter::CreateLinePropertiesProxy(LineProperties* pObject)
{
  return new LinePropertiesProxy(pObject);
}

// init LOD

RenderingObject* OpenGLPresenter::CreateLODProxy(LOD* pObject)
{
  return new LODProxy(pObject);
}

// init Material

RenderingObject* OpenGLPresenter::CreateMaterialProxy(Material* pObject)
{
  return new MaterialProxy(pObject);
}

// init Material2D

RenderingObject* OpenGLPresenter::CreateMaterial2DProxy(Material2D* pObject)
{
  return new Material2DProxy(pObject);
}

// init MovieTexture

RenderingObject* OpenGLPresenter::CreateMovieTextureProxy(MovieTexture* pObject)
{
  return new MovieTextureProxy(pObject);
}

// init PixelTexture

RenderingObject* OpenGLPresenter::CreatePixelTextureProxy(PixelTexture* pObject)
{
  return new PixelTextureProxy(pObject);
}

// init PointLight

RenderingObject* OpenGLPresenter::CreatePointLightProxy(PointLight* pObject)
{
  return new PointLightProxy(pObject);
}

// init PointSet

RenderingObject* OpenGLPresenter::CreatePointSetProxy(PointSet* pObject)
{
  return new PointSetProxy(pObject);
}

// init PointSet2D

RenderingObject* OpenGLPresenter::CreatePointSet2DProxy(PointSet2D* pObject)
{
  return new PointSet2DProxy(pObject);
}

// init ProximitySensor

RenderingObject* OpenGLPresenter::CreateProximitySensorProxy(ProximitySensor* pObject)
{
  return new ProximitySensorProxy(pObject);
}

// init Rectangle

RenderingObject* OpenGLPresenter::CreateRectangleProxy(class Rectangle* pObject)
{
  return new RectangleProxy(pObject);
}

// init Shape

RenderingObject* OpenGLPresenter::CreateShapeProxy(Shape* pObject)
{
  return new ShapeProxy(pObject);
}

// init Sound

RenderingObject* OpenGLPresenter::CreateSoundProxy(Sound* pObject)
{
  return new SoundProxy(pObject);
}

// init Sound2D

RenderingObject* OpenGLPresenter::CreateSound2DProxy(Sound2D* pObject)
{
  return new Sound2DProxy(pObject);
}

// init Sphere

RenderingObject* OpenGLPresenter::CreateSphereProxy(Sphere* pObject)
{
  return new SphereProxy(pObject);
}

// init SpotLight

RenderingObject* OpenGLPresenter::CreateSpotLightProxy(SpotLight* pObject)
{
  return new SpotLightProxy(pObject);
}

// init StreamingText

#ifdef PRE_FDIS
RenderingObject* OpenGLPresenter::CreateStreamingTextProxy(StreamingText* pObject)
{
  return new StreamingTextProxy(pObject);
}
#endif // PRE_FDIS

// init Switch

RenderingObject* OpenGLPresenter::CreateSwitchProxy(Switch* pObject)
{
  return new SwitchProxy(pObject);
}

// init Switch2D

#ifdef PRE_FDIS
RenderingObject* OpenGLPresenter::CreateSwitch2DProxy(Switch2D* pObject)
{
  return new Switch2DProxy(pObject);
}
#endif // PRE_FDIS

// init TermCap

RenderingObject* OpenGLPresenter::CreateTermCapProxy(TermCap* pObject)
{
  return new TermCapProxy(pObject);
}

// init Text

RenderingObject* OpenGLPresenter::CreateTextProxy(Text* pObject)
{
  return new TextProxy(pObject);
}

// init TimeSensor

RenderingObject* OpenGLPresenter::CreateTimeSensorProxy(TimeSensor* pObject)
{
  return new TimeSensorProxy(pObject);
}

// init TouchSensor

RenderingObject* OpenGLPresenter::CreateTouchSensorProxy(TouchSensor* pObject)
{
  return new TouchSensorProxy(pObject);
}

// init Transform

RenderingObject* OpenGLPresenter::CreateTransformProxy(Transform* pObject)
{
  return new TransformProxy(pObject);
}

// init Transform2D

RenderingObject* OpenGLPresenter::CreateTransform2DProxy(Transform2D* pObject)
{
  return new Transform2DProxy(pObject);
}

// init VisibilitySensorProxy

RenderingObject* OpenGLPresenter::CreateVisibilitySensorProxy(VisibilitySensor* pObject)
{
  return new VisibilitySensorProxy(pObject);
}

// init VideoObject2D

#ifdef PRE_FDIS
RenderingObject* OpenGLPresenter::CreateVideoObject2DProxy(VideoObject2D* pObject)
{
  return new VideoObject2DProxy(pObject);
}
#endif // PRE_FDIS

// init Viewpoint

RenderingObject* OpenGLPresenter::CreateViewpointProxy(Viewpoint* pObject)
{
  return new ViewpointProxy(pObject);
}

// init TextureTransform

RenderingObject* OpenGLPresenter::CreateTextureTransformProxy(TextureTransform* pObject)
{
  return new TextureTransformProxy(pObject);
}

#ifdef ADV_AUDIO

RenderingObject* OpenGLPresenter::CreateAcousticMaterialProxy(AcousticMaterial * poNode)
{
  return new AcousticMaterialProxy(poNode);
}

RenderingObject* OpenGLPresenter::CreateAcousticSceneProxy(AcousticScene * poNode)
{
  return new AcousticSceneProxy(poNode);
}

RenderingObject* OpenGLPresenter::CreateDirectiveSoundProxy(DirectiveSound * poNode)
{
  return new DirectiveSoundProxy(poNode);
}

RenderingObject* OpenGLPresenter::CreatePerceptualParametersProxy(PerceptualParameters * poNode)
{
  return new PerceptualParametersProxy(poNode); 
}

#endif // ADV_AUDIO

// delete object

/* virtual */ void
OpenGLPresenter::DeleteObject(MediaObject* pObject)
{
  if (pObject->GetRenderingObject())
    delete (MediaObjectProxy*) pObject->GetRenderingObject();
}


// Tolga 00/03/28: Added body proxies

RenderingObject* OpenGLPresenter::CreateBAPProxy(BAP* pObject)
{
  return new BAPProxy(pObject);
}

RenderingObject* OpenGLPresenter::CreateBDPProxy(BDP* pObject)
{
  return new BDPProxy(pObject);
}

RenderingObject* OpenGLPresenter::CreateBodyProxy(Body* pObject)
{
  return new BodyProxy(pObject);
}

// RenderingObject* OpenGLPresenter::CreateBodyDefTableProxy(BodyDefTable* pObject)
// {
//   return new BodyDefTableProxy(pObject);
// }

// RenderingObject* OpenGLPresenter::CreateBodySegmentConnectionHintProxy(BodySegmentConnectionHint* pObject)
// {
//   return new BodySegmentConnectionHintProxy(pObject);
// }


