// -*- c++ -*-

// This software module was originally developed by Karl Anders ygard
// (Telenor R&D), in the course of development of the MPEG-4 Systems
// standard (ISO/IEC 14496-1).  This software module is an implementation
// of a part of one or more MPEG-4 Systems tools as specified by the MPEG-4
// Systems standard.  ISO/IEC gives users of the MPEG-4 Systems standard
// free license to this software module or modifications thereof for use in
// hardware or software products claiming conformance to the MPEG-4 Systems
// standard.  Those intending to use this software module in hardware or
// software products are advised that its use may infringe existing
// patents.  The original developer of this software module and his/her
// company, the subsequent editors and their companies, and ISO/IEC have no
// liability for use of this software module or modifications thereof in an
// implementation.  Copyright is not released for non MPEG-4 Systems
// conforming products.  The original developer retains full right to use
// the code for his/her own purpose, assign or donate the code to a third
// party and to inhibit third parties from using the code for non MPEG-4
// Systems conforming products.  This copyright notice must be included in
// all copies or derivative works.
//
// Copyright 1997, 1998.

#ifndef __OPENGLPRESENTER_H__
#define __OPENGLPRESENTER_H__

//#include <stdio.h>

#include "Presenter.h"
#include "Clock.h"

#include "OpenGLVisualRenderer.h"
//#include "AudioRenderer.h"


/// OpenGL Presenter

class OpenGLPresenter
  : public Presenter
{
public:

  /// Constructor

			OpenGLPresenter(Executive* pExecutive, HWND hWnd);

  /// Destructor

  inline		~OpenGLPresenter();

  /// Get pointer to visual renderer

  inline virtual VisualRenderer*
			GetVisualRenderer() const;

  /// Get pointer to visual renderer

  inline virtual OpenGLVisualRenderer*
			GetOpenGLVisualRenderer() const;

  /// Get pointer to audio renderer

  inline virtual AudioRenderer*
			GetAudioRenderer() const;

  /// Get current time of presenter

  virtual DWORD		GetTime() const;

  /// Get current time of presenter

  const ClockReference&	GetClock() const;

  /** Initialise presenter.

      Initialises the renderers, starts the object thread and starts
      presenting the scene. */

  // virtual BOOL		Init();
  virtual void Init();

  /** Terminate renderer.

      Stops rendering, performs cleanup procedures and terminates the
      object thread.  */

  inline virtual void	Terminate();

  /// Paint

  virtual void		Paint(void* rectangle);

  /// Resize the display

  virtual void		ReSize(int nWidth, int nHeight);

  // SB 99/05/14: Logf defined in Presenter (Player3d)
  /// Output message
  // virtual void	Logf(LPCSTR szFormat, ...);

  /// Handle mouse events

  virtual void		OnUserEvent(USER_EVENT event, int nX, int nY, UINT uModifierState);

  // init MediaObject
  virtual RenderingObject* CreateMediaObjectProxy(MediaObject* pObject);

  // init Node
  virtual RenderingObject* CreateNodeProxy(/* Node */ MediaObject * pObject);

  // init Node3D
  virtual RenderingObject* CreateNode3DProxy(/* Node3D */ MediaObject * pObject);

  // init Node2D
  virtual RenderingObject* CreateNode2DProxy(/* Node2D */ MediaObject * pObject);

  // init Bindable
  virtual RenderingObject* CreateBindableProxy(/* Bindable */ MediaObject * pObject);

  // init InterpolatorNode
  virtual RenderingObject* CreateInterpolatorNodeProxy(/* InterpolatorNode */ MediaObject * pObject);

  // init Anchor
  virtual RenderingObject* CreateAnchorProxy(Anchor* pObject);

  // init Anchor2D
#ifdef PRE_FDIS
  virtual RenderingObject* CreateAnchor2DProxy(Anchor2D* pObject);
#endif // PRE_FDIS

  // init AnimationStream
  virtual RenderingObject* CreateAnimationStreamProxy(AnimationStream* pObject);

  // init Appearance
  virtual RenderingObject* CreateAppearanceProxy(Appearance* pObject);

  // init AudioSource
  virtual RenderingObject* CreateAudioSourceProxy(AudioSource* pObject);

  // init AudioObject
#ifdef PRE_FDIS
  virtual RenderingObject* CreateAudioObjectProxy(AudioObject * poObject);
#endif // PRE_FDIS

  // init Background
  virtual RenderingObject* CreateBackgroundProxy(Background * poObject);

  // init Billboard
  virtual RenderingObject* CreateBillboardProxy(Billboard* pObject);

  // init Box
  virtual RenderingObject* CreateBoxProxy(Box* pObject);

  // init Circle
  virtual RenderingObject* CreateCircleProxy(Circle* pObject);

  // init Cone
  virtual RenderingObject* CreateConeProxy(Cone* pObject);

  // init Conditional
  virtual RenderingObject* CreateConditionalProxy(Conditional* pObject);

  // init Cylinder
  virtual RenderingObject* CreateCylinderProxy(Cylinder* pObject);

  // init DirectionalLight
  virtual RenderingObject* CreateDirectionalLightProxy(DirectionalLight* pObject);

  // init DragSensor
  virtual RenderingObject* CreateDragSensorProxy(/* DragSensor */ MediaObject * pObject);

  // init ElevationGrid
  virtual RenderingObject* CreateElevationGridProxy(ElevationGrid* pObject);

//#ifdef ML_CANNOT_FIX
  // init Face
  virtual RenderingObject* CreateFaceProxy(Face* pObject);

  // init FDP
  virtual RenderingObject* CreateFDPProxy(FDP* pObject);
//#endif

  // init Fog
  virtual RenderingObject* CreateFogProxy(Fog* pObject);

  // init Form
  virtual RenderingObject* CreateFormProxy(Form* pObject);

  // init Group
  virtual RenderingObject* CreateGroupProxy(Group* pObject);

  // init Group2D
#ifdef PRE_FDIS
  virtual RenderingObject* CreateGroup2DProxy(Group2D* pObject);
#endif // PRE_FDIS

  // init Image2D
#ifdef PRE_FDIS
  virtual RenderingObject* CreateImage2DProxy(Image2D* pObject);
#endif // PRE_FDIS

  // init ImageTexture
  virtual RenderingObject* CreateImageTextureProxy(ImageTexture* pObject);

  // init IndexedFaceSet
  virtual RenderingObject* CreateIndexedFaceSetProxy(IndexedFaceSet* pObject);

  // init IndexedFaceSet2D
  virtual RenderingObject* CreateIndexedFaceSet2DProxy(IndexedFaceSet2D* pObject);

  // init IndexedLineSet
  virtual RenderingObject* CreateIndexedLineSetProxy(IndexedLineSet* pObject);

  // init IndexedLineSet2D
  virtual RenderingObject* CreateIndexedLineSet2DProxy(IndexedLineSet2D* pObject);

  // init Inline
  virtual RenderingObject* CreateInlineProxy(Inline* pObject);

  // init Inline2D
#ifdef PRE_FDIS
  virtual RenderingObject* CreateInline2DProxy(Inline2D* pObject);
#endif // PRE_FDIS

  // init Layout
  virtual RenderingObject* CreateLayoutProxy(Layout* pObject);

  // init LineProperties
  virtual RenderingObject* CreateLinePropertiesProxy(LineProperties* pObject);

  // init LOD
  virtual RenderingObject* CreateLODProxy(LOD* pObject);

  // init Material
  virtual RenderingObject* CreateMaterialProxy(Material* pObject);

  // init Material2D
  virtual RenderingObject* CreateMaterial2DProxy(Material2D* pObject);

  // init MovieTexture
  virtual RenderingObject* CreateMovieTextureProxy(MovieTexture* pObject);

  // init PixelTexture
  virtual RenderingObject* CreatePixelTextureProxy(PixelTexture* pObject);

  // init PointLight
  virtual RenderingObject* CreatePointLightProxy(PointLight* pObject);

  // init PointSet
  virtual RenderingObject* CreatePointSetProxy(PointSet* pObject);

  // init PointSet2D
  virtual RenderingObject* CreatePointSet2DProxy(PointSet2D* pObject);

  // init ProximitySensor
  virtual RenderingObject* CreateProximitySensorProxy(ProximitySensor* pObject);

  // init Rectangle
  virtual RenderingObject* CreateRectangleProxy(class Rectangle* pObject);

  // init Shape
  virtual RenderingObject* CreateShapeProxy(Shape* pObject);

  // init Sound
  virtual RenderingObject* CreateSoundProxy(Sound* pObject);

  // init Sound2D
  virtual RenderingObject* CreateSound2DProxy(Sound2D* pObject);

  // init Sphere
  virtual RenderingObject* CreateSphereProxy(Sphere* pObject);

  // init SpotLight
  virtual RenderingObject* CreateSpotLightProxy(SpotLight* pObject);

  // init StreamingText
#ifdef PRE_FDIS
  virtual RenderingObject* CreateStreamingTextProxy(StreamingText* pObject);
#endif // PRE_FDIS

  // init Switch
  virtual RenderingObject* CreateSwitchProxy(Switch* pObject);

  // init Switch2D
#ifdef PRE_FDIS
  virtual RenderingObject* CreateSwitch2DProxy(Switch2D* pObject);
#endif // PRE_FDIS

  // init TermCap
  virtual RenderingObject* CreateTermCapProxy(TermCap* pObject);

  // init Text
  virtual RenderingObject* CreateTextProxy(Text* pObject);

  // init TextureTransform
  virtual RenderingObject* CreateTextureTransformProxy(TextureTransform* pObject);

  // init TimeSensor
  virtual RenderingObject* CreateTimeSensorProxy(TimeSensor* pObject);

  // init Transform
  virtual RenderingObject* CreateTransformProxy(Transform* pObject);

  // init Transform2D
  virtual RenderingObject* CreateTransform2DProxy(Transform2D* pObject);

  // init TouchSensor
  virtual RenderingObject* CreateTouchSensorProxy(TouchSensor* pObject);

  // init Viewpoint
  virtual RenderingObject* CreateViewpointProxy(Viewpoint* pObject);

  // init VideoObject2D
#ifdef PRE_FDIS
  virtual RenderingObject* CreateVideoObject2DProxy(VideoObject2D* pObject);
#endif // PRE_FDIS

  // init VisibilitySensor
  virtual RenderingObject* CreateVisibilitySensorProxy(VisibilitySensor* pObject);

#ifdef ADV_AUDIO
  virtual RenderingObject* CreateAcousticMaterialProxy(AcousticMaterial * poNode);
  virtual RenderingObject* CreateAcousticSceneProxy(AcousticScene * poNode);
  virtual RenderingObject* CreateDirectiveSoundProxy(DirectiveSound * poNode);
  virtual RenderingObject* CreatePerceptualParametersProxy(PerceptualParameters * poNode);
#endif // ADV_AUDIO

  // Delete object
  virtual void		DeleteObject(MediaObject* pObject);

  // Tolga 00/03/28: For body animation
  virtual RenderingObject* CreateBAPProxy(BAP* pObject);
  virtual RenderingObject* CreateBDPProxy(BDP* pObject);
  virtual RenderingObject* CreateBodyProxy(Body* pObject);
private:

  /** How many times we have traversed the scene graph.

      This should always increase. */

  DWORD			m_dwLoop;

  /// Clock reference

  ClockReference	m_clock;

  /// mouse buttons

  int			m_nMouseButtons;

  /// Visual renderer

  OpenGLVisualRenderer*	m_pVisualRenderer;

  /// Audio renderer

  AudioRenderer*	m_pAudioRenderer;
};

#include "OpenGLPresenter.inl"

#endif // __OPENGLPRESENTER_H__
