// -*- c++ -*-

// This software module was originally developed by Edward Cooke (Teltec
// Ireland) and Karl Anders ygard (Telenor R&D), in the course of
// development of the MPEG-4 Systems standard (ISO/IEC 14496-1).  This
// software module is an implementation of a part of one or more MPEG-4
// Systems tools as specified by the MPEG-4 Systems standard.  ISO/IEC
// gives users of the MPEG-4 Systems standard free license to this software
// module or modifications thereof for use in hardware or software products
// claiming conformance to the MPEG-4 Systems standard.  Those intending to
// use this software module in hardware or software products are advised
// that its use may infringe existing patents.  The original developer of
// this software module and his/her company, the subsequent editors and
// their companies, and ISO/IEC have no liability for use of this software
// module or modifications thereof in an implementation.  Copyright is not
// released for non MPEG-4 Systems conforming products.  The original
// developer retains full right to use the code for his/her own purpose,
// assign or donate the code to a third party and to inhibit third parties
// from using the code for non MPEG-4 Systems conforming products.  This
// copyright notice must be included in all copies or derivative works.
//
// Copyright 1997, 1998.

// Stefano Battista 98/11/09
// #include "group/Group.h"
#include "Caps.h"

#include <stdio.h>
#if defined(GLTT)
#include <gltt/GLTTFont.h>
#include <gltt/FTFont.h>
#include <gltt/FTGlyph.h>
#include <gltt/FTInstance.h>
#elif defined(GLC)
#include <GL/glc.h>
#endif

// Destructor

inline /* virtual */
OpenGLVisualRenderer::~OpenGLVisualRenderer()
{
  // make sure we don't delete an active renderer

  m_mtxRendererActive.P();
  m_mtxRendererActive.V();

#if defined(GLTT)
  for (int i = m_generatedFonts.GetLength() - 1; i >= 0; --i) {
    delete m_generatedFonts[i]->font;
    delete m_generatedFonts[i]->face;
	delete m_generatedFonts[i];
  }
#elif defined(GLC)
  ::glcDeleteContext(m_hGLCContext);
#endif
}

// Get presenter

inline OpenGLPresenter*
OpenGLVisualRenderer::GetPresenter() const
{
  return m_pPresenter;
}

// Register light

inline int
OpenGLVisualRenderer::RegisterLight()
{
  return m_nNextGLLight++;
}

// Toggle drawing of scene graph

inline int
OpenGLVisualRenderer::DrawSceneGraph(int nDrawSceneGraph)
{
  return m_nDrawSceneGraph = nDrawSceneGraph;
}

// Get whether to draw scene graph

inline int
OpenGLVisualRenderer::DrawSceneGraph() const
{
  return m_nDrawSceneGraph;
}

// Toggle drawing of bounding boxes

inline int
OpenGLVisualRenderer::DrawBBoxes(BOOL bDrawBBoxes)
{
  return m_bDrawBBoxes = bDrawBBoxes;
}

// Get whether to draw bounding boxes

inline int
OpenGLVisualRenderer::DrawBBoxes() const
{
  return m_bDrawBBoxes;
}

// Toggle clearing of screen

inline BOOL
OpenGLVisualRenderer::ClearScreen(BOOL bClearScreen)
{
  return m_bClearScreen = bClearScreen;
}

// Toggle use of Z buffer

inline BOOL
OpenGLVisualRenderer::UseZBuffer(BOOL bZBuffer)
{
  return m_bZBuffer = bZBuffer;
}

// Set current texture

inline void
OpenGLVisualRenderer::SetCurrentTexture(TextureNodeProxy* pTexture)
{
  m_pCurrentTexture = pTexture;
}

// Get current texture

inline TextureNodeProxy*
OpenGLVisualRenderer::GetCurrentTexture() const
{
  return m_pCurrentTexture;
}

// Set current material

inline void
OpenGLVisualRenderer::SetCurrentMaterial2D(Material2DProxy* pMaterial)
{
  m_pCurrentMaterial = pMaterial;
}

// Get current material

inline Material2DProxy*
OpenGLVisualRenderer::GetCurrentMaterial2D() const
{
  return m_pCurrentMaterial;
}

// Get texture mag function

inline GLint
OpenGLVisualRenderer::GetTextureMagFunc() const
{
  return m_nMagFunc;
}

// Set texture min function

inline void
OpenGLVisualRenderer::SetTextureMinFunc(GLint nFunc)
{
  m_nMinFunc = nFunc;
}

// Get texture min function

inline GLint
OpenGLVisualRenderer::GetTextureMinFunc() const
{
  return m_nMinFunc;
}

// Set texture min function to use with mipmapping

inline void
OpenGLVisualRenderer::SetTextureMipmapMinFunc(GLint nFunc)
{
  m_nMipmapMinFunc = nFunc;
}

// Get texture min function to use with mipmapping

inline GLint
OpenGLVisualRenderer::GetTextureMipmapMinFunc() const
{
  return m_nMipmapMinFunc;
}

// Set texture mag function

inline void
OpenGLVisualRenderer::SetTextureMagFunc(GLint nFunc)
{
  m_nMagFunc = nFunc;
}

// Get capabilities

inline /* virtual */ void
OpenGLVisualRenderer::GetCapabilities(Capabilities* pCapabilities)
{
  // Stefano Battista 
  // @@@@ 98/11/10 changed from tagFormat to MediaFormat
  pCapabilities->AddFormatCapability((MediaFormat) (MediaFormat :: RGBA), 0);
  pCapabilities->AddFormatCapability((MediaFormat) (MediaFormat :: RGB), 0);
  pCapabilities->AddFormatCapability((MediaFormat) (MediaFormat :: FBA), 0);
/***
  pCapabilities->AddFormatCapability(FORMAT_RGBA, 0);
  pCapabilities->AddFormatCapability(FORMAT_RGB, 0);
  pCapabilities->AddFormatCapability(FORMAT_FBA, 0);
***/
  /* if (m_bColorMatrixExtension) {
    pCapabilities->AddFormatCapability(FORMAT_YUV411, 0);
    pCapabilities->AddFormatCapability(FORMAT_YUV422, 0);
    pCapabilities->AddFormatCapability(FORMAT_YUV444, 0);
  } */
}

// Register active TouchSensor

inline void
OpenGLVisualRenderer::RegisterTouchSensor(TouchSensorProxy* pTouchSensorProxy)
{
  m_pActiveTouchSensors += pTouchSensorProxy;
}

// Unregister previously active TouchSensor

inline void
OpenGLVisualRenderer::UnregisterTouchSensor(TouchSensorProxy* pTouchSensorProxy)
{
  m_pActiveTouchSensors -= pTouchSensorProxy;
}

// Register active Anchor

inline void
OpenGLVisualRenderer::RegisterAnchor(AnchorProxy* pAnchorProxy)
{
  m_pActiveAnchors += pAnchorProxy;
}

// Unregister previously active Anchor

inline void
OpenGLVisualRenderer::UnregisterAnchor(AnchorProxy* pAnchorProxy)
{
  m_pActiveAnchors -= pAnchorProxy;
}

// Register active Anchor2D

#ifdef PRE_FDIS
inline void
OpenGLVisualRenderer::RegisterAnchor2D(Anchor2DProxy* pAnchor2DProxy)
{
  m_pActiveAnchors2D += pAnchor2DProxy;
}
#endif // PRE_FDIS

// Unregister previously active Anchor2D

#ifdef PRE_FDIS
inline void
OpenGLVisualRenderer::UnregisterAnchor2D(Anchor2DProxy* pAnchor2DProxy)
{
  m_pActiveAnchors2D -= pAnchor2DProxy;
}
#endif // PRE_FDIS

// Register active DragSensor

inline void
OpenGLVisualRenderer::RegisterDragSensor(DragSensorProxy* pDragSensorProxy)
{
  m_pActiveDragSensors += pDragSensorProxy;
}

// Unregister previously active DragSensor

inline void
OpenGLVisualRenderer::UnregisterDragSensor(DragSensorProxy* pDragSensorProxy)
{
  m_pActiveDragSensors -= pDragSensorProxy;
}

// Get current frame rate in FPS

inline float
OpenGLVisualRenderer::GetFrameRate() const
{
  return m_fFrameRate;
}

// Get left frustum plane

inline const Vec3f&
OpenGLVisualRenderer::GetLeftFrustumPlane() const
{
  return m_leftFrustumPlane;
}

// Get right frustum plane

inline const Vec3f&
OpenGLVisualRenderer::GetRightFrustumPlane() const
{
  return m_rightFrustumPlane;
}

// Get top frustum plane

inline const Vec3f&
OpenGLVisualRenderer::GetTopFrustumPlane() const
{
  return m_topFrustumPlane;
}

// Get bottom frustum plane

inline const Vec3f&
OpenGLVisualRenderer::GetBottomFrustumPlane() const
{
  return m_bottomFrustumPlane;
}

// Toggle collision detection

inline BOOL
OpenGLVisualRenderer::CollisionDetection(BOOL bCollisionDetection)
{
  return m_bCollisionDetection = bCollisionDetection;
}

// Get whether to use collision detection

inline BOOL
OpenGLVisualRenderer::CollisionDetection() const
{
  return m_bCollisionDetection;
}  

// Get OpenGL version

inline float
OpenGLVisualRenderer::GetOpenGLVersion() const
{
  return m_fOpenGLVersion;
}

