// This software module was originally developed by Karl Anders ygard
// (Telenor R&D) in the course of development of the MPEG-4 Systems
// standard (ISO/IEC 14496-1).  This software module is an implementation
// of a part of one or more MPEG-4 Systems tools as specified by the MPEG-4
// Systems standard.  ISO/IEC gives users of the MPEG-4 Systems standard
// free license to this software module or modifications thereof for use in
// hardware or software products claiming conformance to the MPEG-4 Systems
// standard.  Those intending to use this software module in hardware or
// software products are advised that its use may infringe existing
// patents.  The original developer of this software module and his/her
// company, the subsequent editors and their companies, and ISO/IEC have no
// liability for use of this software module or modifications thereof in an
// implementation.  Copyright is not released for non MPEG-4 Systems
// conforming products.  The original developer retains full right to use
// the code for his/her own purpose, assign or donate the code to a third
// party and to inhibit third parties from using the code for non MPEG-4
// Systems conforming products.  This copyright notice must be included in
// all copies or derivative works.
//
// Copyright 1998.

#include "OpenGl/OpenGl.h"
#include <math.h>
#include "BackgroundProxy.h"
#include "opengl/base/Matrix.h"
#include "opengl/base/OpenGLVisualRenderer.h"


// Constructor

BackgroundProxy::BackgroundProxy(Background * poBackground)
  : BindableProxy(poBackground)
{
  SetRoot(poBackground->GetRoot());
}


// Render background

void
BackgroundProxy::RenderBackground(OpenGLVisualRenderer* pRenderer, DWORD /* dwTime */)
{
  // Background* pBackground = (Background*) GetMediaObject();

  const MF<Color3f>& skyColor = /* pBackground-> */ GetSkyColor();
  const MF<float>& skyAngle = /* pBackground-> */ GetSkyAngle();
  int skyColors = /* pBackground-> */ GetSkyColor().GetLength();
  
  if (skyColors > 0 && skyColors != skyAngle.GetLength() + 1)
  {
    COMPLAIN("Not correct number of angles in skyAngle\n");
    ::glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);
  } 
  else
  {
    if (skyColors == 1)
    {
      ::glClearColor(skyColor[0].r, skyColor[0].g, skyColor[0].b, 1.0f);
      ::glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);
    }
    else
    {
      const GLfloat* upperColor = &skyColor[0].r;
      float upperAngle = 0.0f;
      
      ::glPushAttrib(GL_DEPTH_BUFFER_BIT | GL_CURRENT_BIT);
      
      ::glDisable(GL_DEPTH_TEST);
      ::glDepthMask(GL_FALSE);
      
      for (int i = 1; i < skyColors; ++i)
      {
        const GLfloat* lowerColor = &skyColor[i].r;
        float lowerAngle = skyAngle[i - 1];
        
        float upperCos = ::cos(upperAngle);
        float lowerCos = ::cos(lowerAngle);
        float upperSin = ::fabs(::sin(upperAngle));
        float lowerSin = ::fabs(::sin(lowerAngle));
        
        ::glBegin(GL_QUAD_STRIP);
        
        for (float f = 0; f <= 360.0f; f += 30.0f)
        {
          ::glColor3fv(upperColor);
          ::glVertex3f(20.0f * ::cos(f * M_PI / 180.0f) * upperSin,
            20.0f * upperCos,
            20.0f * ::sin(f * M_PI / 180.0f) * upperSin);
          ::glColor3fv(lowerColor);
          ::glVertex3f(20.0f * ::cos(f * M_PI / 180.0f) * lowerSin,
            20.0f * lowerCos,
            20.0f * ::sin(f * M_PI / 180.0f) * lowerSin);
        }
        
        ::glEnd();
        
        upperColor = lowerColor;
        upperAngle = lowerAngle;
      }
      
      if (::fabs(M_PI - upperAngle) > 0.01)
      {
        float upperCos = ::cos(upperAngle);
        float upperSin = ::fabs(::sin(upperAngle));
        
        ::glBegin(GL_TRIANGLE_FAN);
        
        ::glVertex3f(0.0f, -20.0f, 0.0f);
        
        for (float f = 360.0f; f >= 0.0f; f -= 30.0f)
          ::glVertex3f(20.0f * ::cos(f * M_PI / 180.0f) * upperSin,
          20.0f * upperCos,
          20.0f * ::sin(f * M_PI / 180.0f) * upperSin);
        
        ::glEnd();
      }
      
      ::glPopAttrib();
      
      ::glClear(GL_DEPTH_BUFFER_BIT);
    } 
  }
#ifdef _DEBUG
  if (pRenderer->CheckGLError("BackgroundProxy:"))
    TRACE("BackgroundProxy::RenderBackground() [%08x]: ", this);
#endif
}

// Render object description

/* virtual */ void
BackgroundProxy::RenderDescription(OpenGLVisualRenderer* pRenderer, DWORD)
{
  GLfloat color[] = { 1.0f, 0.0f, 0.0f, 0.75f};

  ::glColor4fv(color);

  RenderDescriptionBox(pRenderer, "Background");
}


void BackgroundProxy::SetRoot(Root* pRoot)
{ 
  TRACE("[Background::SetRoot] \n");
  GetMediaObject()->SetRoot(pRoot);
  if (! GetMediaObject() -> GetRoot() -> GetBoundBackground())
  {
    TRACE("[Background::SetRoot] Push Background \n");
    GetMediaObject() -> GetRoot() -> PushBackground(GetBackground());
  }
}
