// This software module was originally developed by Kevin Barron (KPN Research), 
// Mike Coleman (Five Bats Research), Edward Cooke (Teltec Ireland), In-Dai Ha 
// (Hyundai Electronics Industries, Co.), Hae-Kwan Kim (Hyundai Electronics 
// Industries, Co.), Matthew Leditschke (CSELT), Zvi Lifshitz (VDOnet Corp.), 
// Karl Anders ygard (Telenor R&D), in the course of development of the MPEG-4 
// Systems standard (ISO/IEC 14496-1).  This software module is an 
// implementation of a part of one or more MPEG-4 Systems tools as specified by 
// the MPEG-4 Systems standard.  ISO/IEC gives users of the MPEG-4 Systems 
// standard free license to this software module or modifications thereof for 
// use in hardware or software products claiming conformance to the MPEG-4 
// Systems standard.  Those intending to use this software module in hardware or 
// software products are advised that its use may infringe existing patents.  
// The original developer of this software module and his/her company, the 
// subsequent editors and their companies, and ISO/IEC have no liability for use 
// of this software module or modifications thereof in an implementation.  
// Copyright is not released for non MPEG-4 Systems conforming products.  The 
// original developer retains full right to use the code for his/her own 
// purpose, assign or donate the code to a third party and to inhibit third 
// parties from using the code for non MPEG-4 Systems conforming products.  This 
// copyright notice must be included in all copies or derivative works.
//
// Copyright 1997.


#include "defs.h"
#include "StreamingTextProxy.h"
#include "opengl/base/OpenGLPresenter.h"
#include "Stream.h"

// Render object description

/* virtual */ void
StreamingTextProxy::RenderDescription(OpenGLVisualRenderer* pRenderer, DWORD /* dwTime */)
{
  GLfloat color[] = { 0.0f, 0.0f, 1.0f, 0.75f};

  ::glColor4fv(color);

  RenderDescriptionBox(pRenderer, "StreamingText");
}

// Render object

/* virtual */ BOOL
StreamingTextProxy::Render(OpenGLVisualRenderer* pRenderer, DWORD /* dwTime */)
{
  MediaStream* pStream = ((StreamConsumer *) GetMediaObject())->GetStream();
  StreamingText* pStreamingText = (StreamingText*) GetMediaObject();

  if (pStream) {
    int nSize;
    DWORD dwTime;
    BOOL bRecurrent;

    /***
    const char* pData = (const char*) pStream->Fetch(nSize,
						     dwTime,
						     bRecurrent);
    ***/
    // SB @@@@ I am calling Fetch without using nRecurrent!!!
    // what is th edefault? guess: FALSE
    BOOL bSkip = TRUE;
    BOOL bWait = FALSE;
    BOOL bEOU = FALSE;
    int nStatus = 0;
    const char * pData = (const char *) pStream->Fetch(
      nSize, dwTime, bEOU, nStatus, bSkip, bWait);
    bRecurrent = FALSE;
 
    if (pData != (const char*) END_OF_STREAM) {
      if (pData) {
        if (!bRecurrent) {
	  FontStyle* pFontStyle = /* pStreamingText-> */ GetFontStyle();
	  float height = 10.0f;
	  float scale;
	  char* string = (char*) ::malloc(nSize + 1);
	  
	  ::strncpy(string, pData, nSize);
	  string[nSize] = 0;

	  if (pFontStyle)
	    height = pFontStyle->size /* GetSize() */ ;
	  
	  scale = height / pRenderer->GetFontHeight();

	  ::glNewList(m_nDisplayList, GL_COMPILE_AND_EXECUTE);
	  ::glPushMatrix();
	  ::glTranslatef(0.0f, -height, 0.0f);
	  ::glScalef(scale, scale, 1.0f);
	  pRenderer->RenderString(string);
	  ::glPopMatrix();
	  ::glEndList();
	} else
	  ::glCallList(m_nDisplayList);

        pStream->Release();
      
#ifdef _DEBUG
	if (pRenderer->CheckGLError("StreamingTextProxy:"))
	  TRACE ("StreamingTextProxy::Render() [%08x]: ", this);
#endif
      } else
        ::glCallList(m_nDisplayList);
    }
  } else
    pStream->Terminate();

  return TRUE;
}

// Traverse scene graph, checking for intersection

/* virtual */ BOOL
StreamingTextProxy::Select(
    OpenGLVisualRenderer* pRenderer, 
    DWORD dwTime, 
    unsigned int& depth)
{
  GLuint selectBuffer[4];

  ::glSelectBuffer(4, selectBuffer);
  ::glRenderMode(GL_SELECT);

  Render(pRenderer, dwTime);

  if (::glRenderMode(GL_RENDER) > 0)
    if (depth >= selectBuffer[1]) {
      depth = selectBuffer[1];

      return TRUE;
    }

  return FALSE;
}
