// This software module was originally developed by Karl Anders ygard
// (Telenor R&D) in the course of development of the MPEG-4 Systems
// standard (ISO/IEC 14496-1).  This software module is an implementation
// of a part of one or more MPEG-4 Systems tools as specified by the MPEG-4
// Systems standard.  ISO/IEC gives users of the MPEG-4 Systems standard
// free license to this software module or modifications thereof for use in
// hardware or software products claiming conformance to the MPEG-4 Systems
// standard.  Those intending to use this software module in hardware or
// software products are advised that its use may infringe existing
// patents.  The original developer of this software module and his/her
// company, the subsequent editors and their companies, and ISO/IEC have no
// liability for use of this software module or modifications thereof in an
// implementation.  Copyright is not released for non MPEG-4 Systems
// conforming products.  The original developer retains full right to use
// the code for his/her own purpose, assign or donate the code to a third
// party and to inhibit third parties from using the code for non MPEG-4
// Systems conforming products.  This copyright notice must be included in
// all copies or derivative works.
//
// Copyright 1998.

#include "defs.h"
// Stefano Battista 98/10/28
#include "Nodes.h"

#include "Anchor2DProxy.h"
#include "opengl/base/OpenGLVisualRenderer.h"

#include <GL/gl.h>

// Render object description

/* virtual */ void
Anchor2DProxy::RenderDescription(OpenGLVisualRenderer* pRenderer, DWORD dwTime)
{
  Anchor2D * pAnchor = (Anchor2D *) GetMediaObject();
  GLfloat color[] = { 0.0f, 1.0f, 0.0f, 0.75f};

  ::glColor4fv(color);

  RenderDescriptionBox(pRenderer, "Anchor2D");

  // Stefano Battista 98/11/10
  // derived from GroupNode to call GetChildCount etc...
  // GroupNode * instead of Node * (see Nodes.h)

  ::glPushMatrix();
  ::glScalef(1.0f / pAnchor->GetChildCount(), 1.0f, 1.0f);
  ::glTranslatef(0.0f, -1.0f, 0.0f);

  for (MediaObject * pObject = pAnchor->GetFirstChild(); pObject; 
      pObject = pAnchor->GetNextSibling())
  {
    ((NodeProxy *) pObject->GetRenderingObject())->
        RenderDescription(pRenderer, dwTime);
    ::glTranslatef(1.0f, 0.0f, 0.0f);
  }

  ::glPopMatrix();
}

// Activate sensor

/* virtual */ bool
Anchor2DProxy::Activate(
    OpenGLVisualRenderer* pRenderer,
		float /* fObjectX */, 
    float /* fObjectY */, 
    float /* fObjectZ */,
		int /* nX */, 
    int /* nY */, 
    bool bMouseActivated)
{
  Anchor2D* pAnchor = (Anchor2D*) GetMediaObject();

  pRenderer->RegisterAnchor2D(this);
  /* pAnchor-> */ ActivateNode(bMouseActivated);

  return TRUE;
}

// Trigger sensor

void
Anchor2DProxy::Trigger(OpenGLVisualRenderer * /* pRenderer */)
{
  Anchor2D* pAnchor = (Anchor2D*) GetMediaObject();

  /* pAnchor-> */ DeactivateNode(TRUE);
}

// Deactivate sensor

void
Anchor2DProxy::Deactivate(OpenGLVisualRenderer* pRenderer, BOOL bMouseActive)
{
  Anchor2D* pAnchor = (Anchor2D*) GetMediaObject();

  if (!bMouseActive)
    pRenderer->UnregisterAnchor2D(this);

  /* pAnchor-> */ DeactivateNode(FALSE);
}

// Activate sensor
bool Anchor2DProxy::ActivateNode(
    bool bMouseActivated)
{
  Anchor2D * poAnchor = GetAnchor2D();

  /* poAnchor-> */ activated = bMouseActivated;

  if (::strcmp(poAnchor->description, "") == 0)
    TRACE("url: %s\n", (LPCSTR) (const ZString&) poAnchor->url);
  else
    TRACE("url: %s\n", (LPCSTR) (const ZString&) poAnchor->description);

  return TRUE;
}

// Deactivate sensor
void Anchor2DProxy::DeactivateNode(BOOL bMouseOver)
{
  Anchor2D * poAnchor = GetAnchor2D();

  if ((Bool) (/* poAnchor-> */ activated) && bMouseOver) {
#if defined(HAVE_SNPRINTF) || defined(WIN32)
    int size = 128;
    char* command;
    command = (char*) ::malloc(size);
#if defined(unix)
    while (::snprintf(command, size, "netscape -remote 'openURL(%s)' &", (LPCSTR) (const ZString&) url) == -1)
    {
      size *= 2;
      command = (char*) ::realloc(command, size);
      if (!command)
      {
        TRACE("Warning: ran out of memory trying to realloc() with size %d\n", size);
        return;
      }
    }
    ::system(command);
#elif defined(WIN32) && !defined(BIFSENC)
    while (::_snprintf(command, size, "%s", (LPCSTR) (const ZString&) poAnchor->url) == -1)
    {
      size *= 2;
      command = (char*) ::realloc(command, size);
      if (!command)
      {
        TRACE("Warning: ran out of memory trying to realloc() with size %d\n", size);
        return;
      }
    }
		if (! m_pNetObj->startLoading((CString) command, (UINT) m_nMode)) {
			//const char* args = "netscape";
			//const char* args1 = " -h ";

			::_flushall();
			ShellExecute(NULL, "open", command , NULL, NULL, SW_SHOWNORMAL);
			
			/*
			m_nBrowserHandle = ::_spawnlp( _P_NOWAIT, args, args1, command, NULL );
			switch (errno)   
			{
			case E2BIG:
				COMPLAIN("Argument list exceeds 1024 bytes");
				break;
			case EINVAL :
				COMPLAIN("mode argument is invalid");
				break;
			case ENOENT :
				COMPLAIN("File or path is not found");
				break;
			case ENOEXEC :
				COMPLAIN("Specified file is not executable or has invalid executable-file format");
				break;
			case ENOMEM :
				COMPLAIN("Not enough memory is available to execute new process");
				break;
			}
			*/
		}
#endif
#else
    char command[512];
    ::sprintf(command, "netscape -remote 'openURL(%s)' &", (LPCSTR) (const ZString&) url);
#endif

    ::free(command);
  } 
  else
    COMPLAIN("\n");

  /* poAnchor-> */ activated = FALSE;
}
