// This software module was originally developed by Karl Anders ygard
// (Telenor R&D) in the course of development of the MPEG-4 Systems
// standard (ISO/IEC 14496-1).  This software module is an implementation
// of a part of one or more MPEG-4 Systems tools as specified by the MPEG-4
// Systems standard.  ISO/IEC gives users of the MPEG-4 Systems standard
// free license to this software module or modifications thereof for use in
// hardware or software products claiming conformance to the MPEG-4 Systems
// standard.  Those intending to use this software module in hardware or
// software products are advised that its use may infringe existing
// patents.  The original developer of this software module and his/her
// company, the subsequent editors and their companies, and ISO/IEC have no
// liability for use of this software module or modifications thereof in an
// implementation.  Copyright is not released for non MPEG-4 Systems
// conforming products.  The original developer retains full right to use
// the code for his/her own purpose, assign or donate the code to a third
// party and to inhibit third parties from using the code for non MPEG-4
// Systems conforming products.  This copyright notice must be included in
// all copies or derivative works.
//
// For more information or to receive an up to date version of this module,
// mail karlo@nta.no or check <http://televr.fou.telenor.no/compositor/>.
//
// Copyright 1997, 1998.

#include "defs.h"
// Stefano Battista 98/10/28
#include "Nodes.h"
#include "Group2DProxy.h"
#include <GL/gl.h>

// add node to draw list

/* virtual */ BOOL
Group2DProxy::Render(const Matrix& rTransformation, const CullFlag& rCullFlag,
		     OpenGLVisualRenderer* pRenderer, DWORD dwTime)
{
  Group2D* pGroup = (Group2D*) GetMediaObject();
  ZArray<MediaObjectProxy*> nodes;
  
  for (MediaObject * pObject = pGroup->GetFirstChild(); 
      pObject; pObject = pGroup->GetNextSibling())
  {
    MediaObjectProxy* pProxy = (MediaObjectProxy*) pObject->GetRenderingObject();

    int i = 0;
    float drawingOrder = pProxy->GetDrawingOrder();
    
    while (nodes.GetLength() > i && nodes[i]->GetDrawingOrder() >= drawingOrder)
      ++i;
    
    nodes.InsertAt(i, pProxy);
  }

  for (int j = 0; j < nodes.GetLength(); ++j)
    nodes[j]->CullOrRender(rTransformation, rCullFlag,
			   pRenderer, dwTime);
  
  return Node2DProxy::Render(rTransformation, rCullFlag,
			     pRenderer, dwTime);
}

// traverse scene graph, checking for intersection

/* virtual */ void
Group2DProxy::Select(
    OpenGLVisualRenderer* pRenderer, 
    DWORD dwTime, 
    MediaObjectProxy*& intersectingObject, 
    unsigned int& depth)
{
  Group2D* pGroup = (Group2D*) GetMediaObject();

  for (MediaObject * pObject = pGroup->GetFirstChild(); 
      pObject; pObject = pGroup->GetNextSibling())
    ((MediaObjectProxy*) pObject->GetRenderingObject())->Select(pRenderer, dwTime, intersectingObject, depth);
}

// Traverse scene graph, checking for activation of sensors

/* virtual */ void
Group2DProxy::Select(
    const Matrix& rTransformation, 
    const CullFlag& rCullFlag,
		OpenGLVisualRenderer* pRenderer, 
    DWORD dwTime,
		MediaObjectProxy*& intersectingObject,
		ZArray<MediaObjectProxy*>& activatedSensors, 
    unsigned int& depth,
		const ZArray<MediaObjectProxy*>& localSensors)
{
  Group2D* pGroup = (Group2D*) GetMediaObject();
  ZArray<MediaObjectProxy*> sensors;
  ZArray<MediaObjectProxy*> nodes;

  // sort objects by drawing order and find pointing device sensors

  for (MediaObject * pObject = pGroup->GetFirstChild(); 
      pObject; pObject = pGroup->GetNextSibling())
  {
    MediaObjectProxy* pProxy = (MediaObjectProxy*) pObject->GetRenderingObject();

    int i = 0;
    float drawingOrder = pProxy->GetDrawingOrder();
    
    while (nodes.GetLength() > i && nodes[i]->GetDrawingOrder() >= drawingOrder)
      ++i;
    
    nodes.InsertAt(i, pProxy);

    if (pProxy->IsPointingDeviceSensor())
      sensors += pProxy;
  }

  // then traverse scene graph further down

  for (int j = 0; j < nodes.GetLength(); ++j)
    nodes[j]->CullOrSelect(
      rTransformation, 
      rCullFlag,
			pRenderer, 
      dwTime,
			intersectingObject,
			activatedSensors, 
      depth,
			sensors.GetLength() ? sensors : localSensors);
}

// add node to draw list

/* virtual */ BOOL
Group2DProxy::RenderLights(OpenGLVisualRenderer* pRenderer, DWORD dwTime)
{
  Group2D* pGroup = (Group2D*) GetMediaObject();

  for (MediaObject * pObject = pGroup->GetFirstChild(); 
      pObject; pObject = pGroup->GetNextSibling())
    ((MediaObjectProxy*) pObject->GetRenderingObject())->RenderLights(pRenderer, dwTime);

  return TRUE;
}

// Render object description

/* virtual */ void
Group2DProxy::RenderDescription(OpenGLVisualRenderer* pRenderer, DWORD dwTime)
{
  Group2D* pGroup = (Group2D*) GetMediaObject();
  GLfloat color[] = { 0.0f, 1.0f, 0.0f, 0.75f};

  ::glColor4fv(color);

  RenderDescriptionBox(pRenderer, "Group2D");

  ::glPushMatrix();
  // Stefano Battista 98/11/10
  ::glScalef(1.0f / ((Group *) pGroup)->GetChildCount(), 1.0f, 1.0f);
  ::glTranslatef(0.0f, -1.0f, 0.0f);

  for (MediaObject * pObject = pGroup->GetFirstChild(); 
      pObject; pObject = pGroup->GetNextSibling())
  {
    ((MediaObjectProxy*) pObject->GetRenderingObject())->RenderDescription(pRenderer, dwTime);
    ::glTranslatef(1.0f, 0.0f, 0.0f);
  }

  ::glPopMatrix();
}

// Find sound sources

/* virtual */ int
Group2DProxy::FindSoundSources(ZArray<Sound*>& rSources3d, ZArray<Matrix>& rTransformations3d,
			       ZArray<Sound2D*>& rSources2d, ZArray<Matrix>& rTransformations2d,
			       const Matrix& rTransform)
{
  Group2D* pGroup = (Group2D*) GetMediaObject();
  int ret = 0;

  for (MediaObject * pObject = pGroup->GetFirstChild(); 
      pObject; pObject = pGroup->GetNextSibling())
    ret += ((MediaObjectProxy*) pObject->GetRenderingObject())->FindSoundSources(rSources3d, rTransformations3d,
									rSources2d, rTransformations2d,
									rTransform);

  return ret;
}
