// This software module was originally developed by Karl Anders ygard
// (Telenor R&D) in the course of development of the MPEG-4 Systems
// standard (ISO/IEC 14496-1).  This software module is an implementation
// of a part of one or more MPEG-4 Systems tools as specified by the MPEG-4
// Systems standard.  ISO/IEC gives users of the MPEG-4 Systems standard
// free license to this software module or modifications thereof for use in
// hardware or software products claiming conformance to the MPEG-4 Systems
// standard.  Those intending to use this software module in hardware or
// software products are advised that its use may infringe existing
// patents.  The original developer of this software module and his/her
// company, the subsequent editors and their companies, and ISO/IEC have no
// liability for use of this software module or modifications thereof in an
// implementation.  Copyright is not released for non MPEG-4 Systems
// conforming products.  The original developer retains full right to use
// the code for his/her own purpose, assign or donate the code to a third
// party and to inhibit third parties from using the code for non MPEG-4
// Systems conforming products.  This copyright notice must be included in
// all copies or derivative works.
//
// Copyright 1997, 1998.

#include "defs.h"
#include "Switch2DProxy.h"
#include <GL/gl.h>

// add node to draw list

/* virtual */ BOOL
Switch2DProxy::Render(const Matrix& rTransformation, const CullFlag& rCullFlag,
		      OpenGLVisualRenderer* pRenderer, DWORD dwTime)
{
  Switch2D* pSwitch = (Switch2D*) GetMediaObject();
  int whichChoice = /* pSwitch-> */ GetWhichChoice();

  if (whichChoice >= 0 &&
      whichChoice < pSwitch->GetChildCount())
    ((MediaObjectProxy*) /* pSwitch-> */ GetChild(whichChoice)->GetRenderingObject())->CullOrRender(rTransformation, rCullFlag,
										      pRenderer, dwTime);

  return MediaObjectProxy::Render(rTransformation, rCullFlag,
			     pRenderer, dwTime);
}

// traverse scene graph, checking for intersection

/* virtual */ void
Switch2DProxy::Select(
  OpenGLVisualRenderer* pRenderer, 
  DWORD dwTime, 
  MediaObjectProxy*& intersectingObject, 
  unsigned int& depth)
{
  Switch2D* pSwitch = (Switch2D*) GetMediaObject();
  int whichChoice = /* pSwitch-> */ GetWhichChoice();

  if (whichChoice >= 0 &&
      whichChoice < pSwitch->GetChildCount())
    ((MediaObjectProxy*) /* pSwitch-> */ GetChild(whichChoice)->GetRenderingObject())->Select(pRenderer, dwTime, intersectingObject, depth);
}

// add node to draw list

/* virtual */ BOOL
Switch2DProxy::RenderLights(OpenGLVisualRenderer* pRenderer, DWORD dwTime)
{
  Switch2D* pSwitch = (Switch2D*) GetMediaObject();
  int whichChoice = /* pSwitch-> */ GetWhichChoice();

  if (whichChoice >= 0 &&
      whichChoice < pSwitch->GetChildCount())
    ((MediaObjectProxy*) /* pSwitch-> */ GetChild(whichChoice)->GetRenderingObject())->RenderLights(pRenderer, dwTime);

  return TRUE;
}

// Render object description

/* virtual */ void
Switch2DProxy::RenderDescription(OpenGLVisualRenderer* pRenderer, DWORD dwTime)
{
  Switch2D* pSwitch = (Switch2D*) GetMediaObject();
  GLfloat color[] = { 0.0f, 1.0f, 0.0f, 0.75f};

  ::glColor4fv(color);

  RenderDescriptionBox(pRenderer, "Switch2D");

  ::glPushMatrix();
  ::glScalef(1.0f / pSwitch->GetChildCount(), 1.0f, 1.0f);
  ::glTranslatef(0.0f, -1.0f, 0.0f);

  for (int nIndex = 0; nIndex < pSwitch->GetChildCount(); ++nIndex) {
    ((MediaObjectProxy*) /* pSwitch-> */ GetChild(nIndex)->GetRenderingObject())->RenderDescription(pRenderer, dwTime);
    
    ::glTranslatef(1.0f, 0.0f, 0.0f);
  }
  
  ::glPopMatrix();
}

// Traverse scene graph, checking for activation of sensors

/* virtual */ void
Switch2DProxy::Select(
    const Matrix& rTransformation, 
    const CullFlag& rCullFlag,
		OpenGLVisualRenderer* pRenderer, 
    DWORD dwTime,
		MediaObjectProxy*& intersectingObject,
		ZArray<MediaObjectProxy*>& activatedSensors, 
    unsigned int& depth,
		const ZArray<MediaObjectProxy*>& localSensors)
{
  Switch2D* pSwitch = (Switch2D*) GetMediaObject();
  ZArray<MediaObjectProxy*> sensors;
  int whichChoice = /* pSwitch-> */ GetWhichChoice();

  // first check children to see if any of them are pointing device sensors

  for (int nIndex = 0; nIndex < pSwitch->GetChildCount(); ++nIndex)
    if (((MediaObjectProxy*) /* pSwitch-> */ GetChild(nIndex)->GetRenderingObject())->IsPointingDeviceSensor())
      sensors += (MediaObjectProxy*) /* pSwitch-> */ GetChild(nIndex)->GetRenderingObject();

  // then traverse scene graph further down

  if (whichChoice >= 0 &&
      whichChoice < pSwitch->GetChildCount())
    ((MediaObjectProxy*) /* pSwitch-> */ GetChild(whichChoice)->GetRenderingObject())->CullOrSelect(rTransformation, rCullFlag,
										      pRenderer, dwTime,
										      intersectingObject,
										      activatedSensors, depth,
										      sensors.GetLength() ? sensors : localSensors);
}

// Find sound sources

/* virtual */ int
Switch2DProxy::FindSoundSources(ZArray<Sound*>& rSources3d, ZArray<Matrix>& rTransformations3d,
				ZArray<Sound2D*>& rSources2d, ZArray<Matrix>& rTransformations2d,
				const Matrix& rTransform)
{
  Switch2D* pSwitch = (Switch2D*) GetMediaObject();
  int ret = 0;

  if (/* pSwitch-> */ GetWhichChoice() >= 0)
    for (int nIndex = 0; nIndex < pSwitch->GetChildCount(); ++nIndex)
      if (/* pSwitch-> */ GetChild(nIndex))
	((MediaObjectProxy*) /* pSwitch-> */ GetChild(nIndex)->GetRenderingObject())->FindSoundSources(rSources3d, rTransformations3d,
											 rSources2d, rTransformations2d,
											 rTransform);

  return ret;
}
