// -*- c++ -*-

// This software module was originally developed by Karl Anders ygard
// (Telenor R&D) in the course of development of the MPEG-4 Systems
// standard (ISO/IEC 14496-1).  This software module is an implementation
// of a part of one or more MPEG-4 Systems tools as specified by the MPEG-4
// Systems standard.  ISO/IEC gives users of the MPEG-4 Systems standard
// free license to this software module or modifications thereof for use in
// hardware or software products claiming conformance to the MPEG-4 Systems
// standard.  Those intending to use this software module in hardware or
// software products are advised that its use may infringe existing
// patents.  The original developer of this software module and his/her
// company, the subsequent editors and their companies, and ISO/IEC have no
// liability for use of this software module or modifications thereof in an
// implementation.  Copyright is not released for non MPEG-4 Systems
// conforming products.  The original developer retains full right to use
// the code for his/her own purpose, assign or donate the code to a third
// party and to inhibit third parties from using the code for non MPEG-4
// Systems conforming products.  This copyright notice must be included in
// all copies or derivative works.
//
// Copyright 1998.

#ifndef __SWITCH2DPROXY_H__
#define __SWITCH2DPROXY_H__

#include "opengl/base/Node2DProxy.h"

//#include "Nodes.h"

/// Switch renderering node

class Switch2DProxy
  : public Node2DProxy
{
public:

  /// Constructor

			Switch2DProxy(Switch2D* pSwitch2D);

  /// Traverse scene graph, activating lights

  virtual BOOL          RenderLights(OpenGLVisualRenderer* pRenderer, DWORD dwTime);

  /// Traverse scene graph, checking for intersection

  virtual void Select(
    OpenGLVisualRenderer* pRenderer, 
    DWORD dwTime,
		MediaObjectProxy*& intersectingObject, 
    unsigned int& depth);

  /// Render object description

  virtual void		RenderDescription(OpenGLVisualRenderer* pRenderer, DWORD dwTime);

  /// Find sound sources

  virtual int		FindSoundSources(ZArray<Sound*>& rSources3d, ZArray<Matrix>& rTransformations3d,
					 ZArray<Sound2D*>& rSources2d, ZArray<Matrix>& rTransformations2d,
					 const Matrix& rTransform);

protected:

  /// Render object and its children
  virtual BOOL		Render(const Matrix& rTransformation, const CullFlag& rCullFlag,
			       OpenGLVisualRenderer* pRenderer, DWORD dwTime);

  /// Traverse scene graph, checking for activation of sensors
  virtual void Select(
    const Matrix& rTransformation, 
    const CullFlag& rCullFlag,
		OpenGLVisualRenderer* pRenderer, 
    DWORD dwTime,
		MediaObjectProxy*& intersectingObject,
		ZArray<MediaObjectProxy*>& activatedSensors, 
    unsigned int& depth,
		const ZArray<MediaObjectProxy*>& localSensors);

  // SB 98/12/04
public:
  Switch2D * GetSwitch2D()
  { return (Switch2D *) GetMediaObject(); }
  /// Get active choice
  inline int GetWhichChoice() /* const */
  { return GetSwitch2D()->whichChoice; }
  /// Get child in this switch
  inline MediaObject * GetChild(int nIndex) /* const */
  { return (MediaObject *) GetSwitch2D()->choice[nIndex]; }

};

#include "Switch2DProxy.i"

#endif // __SWITCH2DPROXY_H__
