#include "OpenGl/OpenGl.h"
#include "CoordinateInterpolatorProxy.h"

CoordinateInterpolatorProxy::CoordinateInterpolatorProxy(MediaObject * poNode) :
InterpolatorNodeProxy(poNode)
{}

inline BOOL CoordinateInterpolatorProxy::SetValue(
    CoordinateInterpolator * poCI, int iIndex)
{
  int arraySize = poCI->keyValue.GetLength() / poCI->key.GetLength();
  int offset = arraySize * iIndex;

  // sanity test
  assert(poCI->keyValue.GetLength() % poCI->key.GetLength() == 0);

  // Get rid of all old values
  poCI->value_changed.Clear();

  // Set coordinates directly
  for (int j = 0; j < arraySize; ++j)
    poCI->value_changed.Append(poCI->keyValue[offset + j]);

  return TRUE;
}

BOOL CoordinateInterpolatorProxy::Evaluate()
{
  CoordinateInterpolator * poCI = 
    (CoordinateInterpolator *) GetMediaObject();

  // Check if we have any keys
  if (poCI->key.GetLength() > 0)
  {
    // If we only have one key
    if (poCI->key.GetLength() == 1)
    {
      return SetValue(poCI, 0);
    }
    else // more keys
    {
      // If set_fraction is smaller than smallest key
      if (poCI->set_fraction <= poCI->key[0])
      {
        return SetValue(poCI, 0);
      }
      else
      {
        // If set_fraction is larger than largest key
        if (poCI->set_fraction >= poCI->key[poCI->key.GetLength() - 1])
        {
          return SetValue(poCI, poCI->key.GetLength() - 1);
        }
        else
        {
          // Search the key space for fraction
          for (int i = poCI->key.GetLength() - 2; i >= 0 ; --i)
          {
            if (poCI->key[i] == poCI->set_fraction)
            {
              // Found exact match
              return SetValue(poCI, i);
            }
            else
            {
              if (poCI->key[i] < poCI->set_fraction)
              {
                // Didn't find exact match; must interpolate
                return InterpolateValue( 
                  i, i + 1, 
                  ((poCI->key[i + 1] - poCI->set_fraction) / 
                  (poCI->key[i + 1] - poCI->key[i])) );
              }
            }
          } // for
        }
      }
    } // one or more keys
  } // some key

  return FALSE;
}

BOOL CoordinateInterpolatorProxy::InterpolateValue(
    int nMinIndex, int nMaxIndex, float fFraction)
{
  CoordinateInterpolator * poCI = 
    (CoordinateInterpolator *) GetMediaObject();

  int arraySize = poCI->keyValue.GetLength() / poCI->key.GetLength();
  int min = arraySize * nMinIndex;
  int max = arraySize * nMaxIndex;

  // Sanity test

  assert(poCI->keyValue.GetLength() % poCI->key.GetLength() == 0);

  // Get rid of all old values

  poCI->value_changed.Clear();

  // Interpolate coordinates

  for (int i = 0; i < arraySize; ++i) {
    Vec3f tempCoordinate;

    tempCoordinate.x = (poCI->keyValue[max + i].x + 
      (poCI->keyValue[min + i].x - poCI->keyValue[max + i].x) * fFraction);
    tempCoordinate.y = (poCI->keyValue[max + i].y + 
      (poCI->keyValue[min + i].y - poCI->keyValue[max + i].y) * fFraction);
    tempCoordinate.z = (poCI->keyValue[max + i].z + 
      (poCI->keyValue[min + i].z - poCI->keyValue[max + i].z) * fFraction);

    poCI->value_changed.Append(tempCoordinate);
  }

  return TRUE;
}
