#include "OpenGl/OpenGl.h"
#include "PositionInterpolatorProxy.h"

PositionInterpolatorProxy::PositionInterpolatorProxy(MediaObject * poNode) :
InterpolatorNodeProxy(poNode)
{}

inline BOOL PositionInterpolatorProxy::SetValue(
    PositionInterpolator * poPI, int iIndex)
{
  poPI->value_changed = poPI->keyValue[iIndex];
  return TRUE;
}

BOOL PositionInterpolatorProxy::Evaluate()
{
  PositionInterpolator * poPI = 
    (PositionInterpolator *) GetMediaObject();

  // Check if we have any keys
  if (poPI->key.GetLength() > 0)
  {
    // If we only have one key
    if (poPI->key.GetLength() == 1)
    {
      return SetValue(poPI, 0);
    }
    else // more keys
    {
      // If set_fraction is smaller than smallest key
      if (poPI->set_fraction <= poPI->key[0])
      {
        return SetValue(poPI, 0);
      }
      else
      {
        // If set_fraction is larger than largest key
        if (poPI->set_fraction >= poPI->key[poPI->key.GetLength() - 1])
        {
          return SetValue(poPI, poPI->key.GetLength() - 1);
        }
        else
        {
          // Search the key space for fraction
          for (int i = poPI->key.GetLength() - 2; i >= 0 ; --i)
          {
            if (poPI->key[i] == poPI->set_fraction)
            {
              // Found exact match
              return SetValue(poPI, i);
            }
            else
            {
              if (poPI->key[i] < poPI->set_fraction)
              {
                // Didn't find exact match; must interpolate
                return InterpolateValue( 
                  i, i + 1, 
                  ((poPI->key[i + 1] - poPI->set_fraction) / 
                  (poPI->key[i + 1] - poPI->key[i])) );
              }
            }
          } // for
        }
      }
    } // one or more keys
  } // some key

  return FALSE;
}

BOOL PositionInterpolatorProxy::InterpolateValue(
    int nMinIndex, int nMaxIndex, float fFraction)
{
  PositionInterpolator * poPI = 
    (PositionInterpolator *) GetMediaObject();

  Vec3f tempPosition;

  assert(poPI->keyValue.GetLength() == poPI->key.GetLength());

  // Interpolate positions

  tempPosition.x = (poPI->keyValue[nMaxIndex].x + 
    (poPI->keyValue[nMinIndex].x - poPI->keyValue[nMaxIndex].x) * fFraction);
  tempPosition.y = (poPI->keyValue[nMaxIndex].y + 
    (poPI->keyValue[nMinIndex].y - poPI->keyValue[nMaxIndex].y) * fFraction);
  tempPosition.z = (poPI->keyValue[nMaxIndex].z + 
    (poPI->keyValue[nMinIndex].z - poPI->keyValue[nMaxIndex].z) * fFraction);
  
  poPI->value_changed = tempPosition;

  return TRUE;
}
