// This software module was originally developed by Matthew Leditschke
// (CSELT), Zvi Lifshitz (VDOnet Corp.) and Karl Anders ygard (Telenor
// R&D) in the course of development of the MPEG-4 Systems standard
// (ISO/IEC 14496-1).  This software module is an implementation of a part
// of one or more MPEG-4 Systems tools as specified by the MPEG-4 Systems
// standard.  ISO/IEC gives users of the MPEG-4 Systems standard free
// license to this software module or modifications thereof for use in
// hardware or software products claiming conformance to the MPEG-4 Systems
// standard.  Those intending to use this software module in hardware or
// software products are advised that its use may infringe existing
// patents.  The original developer of this software module and his/her
// company, the subsequent editors and their companies, and ISO/IEC have no
// liability for use of this software module or modifications thereof in an
// implementation.  Copyright is not released for non MPEG-4 Systems
// conforming products.  The original developer retains full right to use
// the code for his/her own purpose, assign or donate the code to a third
// party and to inhibit third parties from using the code for non MPEG-4
// Systems conforming products.  This copyright notice must be included in
// all copies or derivative works.
//
// For more information or to receive an up to date version of this module,
// mail karlo@nta.no or check <http://televr.fou.telenor.no/compositor/>.
//
// Copyright 1997.

//ML - added defs.h
#include "defs.h"
#include "Presenter.h"

#include "Caps.h"

#include "opengl/base/VisualRenderer.h"
#include "opengl/base/AudioRenderer.h"

#include "MediaObj.h"

// Stefano Battista 98/10/28
//#include "Nodes.h"

// #include "audio/AudioObject.h"

/***
#include "Node.h"
#include "Node3D.h"

#include "group/Switch.h"
#include "group/Group.h"
#include "group/Inline.h"
#include "group/Transform.h"
#include "group/Billboard.h"
#include "group/Anchor.h"
#include "group/Collision.h"
#include "group/Layout.h"
#include "group/Form.h"
#include "group/LOD.h"

#include "Node2D.h"
#include "Conditional.h"

#include "group/Switch2D.h"
#include "group/Group2D.h"
#include "group/Anchor2D.h"
#include "group/Inline2D.h"
#include "group/Transform2D.h"
#include "geometry/TextureNode2D.h"
#include "geometry/VideoObject2D.h"
#include "geometry/Image2D.h"

#include "bindable/Bindable.h"
#include "bindable/Viewpoint.h"
#include "bindable/WorldInfo.h"
#include "bindable/Fog.h"
#include "bindable/Background.h"
#include "bindable/NavigationInfo.h"

#include "interpolators/InterpolatorNode.h"

#include "AnimationStream.h"

#include "TermCap.h"

#include "sensors/TouchSensor.h"
#include "sensors/DragSensor.h"
#include "sensors/PlaneSensor.h"
#include "sensors/PlaneSensor2D.h"
#include "sensors/SphereSensor.h"
#include "sensors/CylinderSensor.h"
#include "sensors/DiscSensor.h"
#include "sensors/TimeSensor.h"
#include "sensors/ProximitySensor.h"
#include "sensors/VisibilitySensor.h"

#include "appearance/Appearance.h"
#include "appearance/Material.h"
#include "appearance/TextureNode.h"
#include "appearance/ImageTexture.h"
#include "appearance/MovieTexture.h"
#include "appearance/PixelTexture.h"
#include "appearance/TextureTransform.h"
#include "appearance/FontStyle.h"

#include "appearance/Material2D.h"
#include "appearance/LineProperties.h"

#include "geometry/Coordinate.h"
#include "geometry/Coordinate2D.h"
#include "geometry/Normal.h"
#include "geometry/TextureCoordinate.h"
#include "appearance/Color.h"

#include "geometry/Shape.h"
#include "geometry/Geometry.h"
#include "geometry/Box.h"
#include "geometry/Sphere.h"
#include "geometry/Cylinder.h"
#include "geometry/Cone.h"
#include "geometry/IndexedFaceSet.h"
#include "geometry/IndexedLineSet.h"
#include "geometry/PointSet.h"
#include "geometry/Text.h"
#include "geometry/StreamingText.h"
#include "geometry/Circle.h"
#include "geometry/Rectangle.h"
#include "geometry/IndexedFaceSet2D.h"
#include "geometry/IndexedLineSet2D.h"
#include "geometry/PointSet2D.h"
#include "geometry/ElevationGrid.h"
#include "geometry/Extrusion.h"

#include "light/Light.h"
#include "light/DirectionalLight.h"
#include "light/PointLight.h"
#include "light/SpotLight.h"

#include "fba/Face.h"
#include "fba/FIT.h"
#include "fba/FDP.h"
#include "fba/FAP.h"
#include "fba/Viseme.h"
#include "fba/Expression.h"
#include "fba/FaceDefTables.h"
#include "fba/FaceDefMesh.h"
#include "fba/FaceDefTransform.h" 

#include "audio/Sound.h"
#include "audio/Sound2D.h"
#include "audio/AudioSource.h"
#include "audio/AudioSwitch.h"
#include "audio/AudioFX.h"
#include "audio/AudioMix.h"
#include "audio/AudioDelay.h"
***/

#include "script/encoder/VScript.h"

//*****************************************************************
// SB @@@@ added from Zvi 98/11/22 >>>>

/*-----------------------------------------------------------------
Save system time as an offset, and set the clock on.
-----------------------------------------------------------------*/
void Presenter :: SetClock (BOOL bOn)
{
	if (bOn)
		m_dwOffset = timeGetTime ();
	m_bClockActive = bOn;
}

/*-----------------------------------------------------------------
Return the sysmem time minus the clocko offset of the Presenter's clock
-----------------------------------------------------------------*/
DWORD Presenter :: GetClockTime ()
{
	return timeGetTime () - m_dwOffset;
}

/*-----------------------------------------------------------------
Format a string into the log file
-----------------------------------------------------------------*/
void Presenter :: Logf (LPCSTR szFormat, ...)
{
	fprintf (
		m_fLogFile,
		"Time: %6lu(%6lu): ",
		timeGetTime () - m_dwStartTime,
		GetClockTime ()
	);
	va_list argptr;
	va_start (argptr, szFormat);
	vfprintf (m_fLogFile, szFormat, argptr);
	va_end (argptr);
}

// SB @@@@ added from Zvi 98/11/22 <<<<
//*****************************************************************

// The capabilities of the OpenGLPresenter combines the capabilities of the
// visual and audio renderers.

void
Presenter::GetCapabilities(Capabilities *pCaps)
{
  int j;
  // Capabilities caps;
  CapabilitiesImp caps;

  GetVisualRenderer()->GetCapabilities(pCaps); 
  GetAudioRenderer()->GetCapabilities(&caps); 

  j = pCaps->GetFormatCapabilityLength();

  for (int i = 0; i < caps.GetFormatCapabilityLength(); i++)
    pCaps->AddFormatCapability(caps.GetFormatCapability(i), j++);

  pCaps->AddFormatCapability(MediaFormat::ANIMSTREAM /* FORMAT_ANIMSTREAM */ , 0);
}


//ML - changed void Presenter::InitCLASS to RenderingObject* Presenter::CreateCLASSProxy
// init MediaObject

RenderingObject* Presenter::CreateMediaObjectProxy(MediaObject* /* pObject */)
{
  return NULL;
}


//ML - removed this function
// init GroupNode

/*
virtual void
Presenter::InitGroupNode(GroupNode * pObject)
{
  InitMediaObject(pObject);
}
*/

// init StreamConsumer

RenderingObject* Presenter::CreateStreamConsumerProxy(StreamConsumer * pObject)
{
  return CreateMediaObjectProxy(pObject);
}

// init Node

/* virtual */ 
/*** 
void
Presenter::InitNode(Node * pObject)
{
  printf("ding\n");

  InitMediaObject(pObject);
} 
***/

// init Node3D

/* virtual */ 
/*** 
void
Presenter::InitNode3D(Node3D * pObject)
{
  InitNode(pObject);
} 
***/

// init Switch

RenderingObject* Presenter::CreateSwitchProxy(Switch* pObject)
{
  // InitGroupNode(pObject);
  return CreateMediaObjectProxy(pObject);
}

// init Group

RenderingObject* Presenter::CreateGroupProxy(Group * pObject)
{
  //ML - commented out next line
  //InitGroupNode(pObject);
  return CreateMediaObjectProxy(pObject);
}

// init Transform

RenderingObject* Presenter::CreateTransformProxy(Transform* pObject)
{
  //ML - commented out next line
  //InitGroupNode(pObject);
  return CreateMediaObjectProxy(pObject);
}

// init Billboard

RenderingObject* Presenter::CreateBillboardProxy(Billboard* pObject)
{
  //ML - commented out next line
  //InitGroupNode(pObject);
  return CreateMediaObjectProxy(pObject);
}

// init Anchor

RenderingObject* Presenter::CreateAnchorProxy(Anchor* pObject)
{
  //ML - commented out next line
  //InitGroupNode(pObject);
  return CreateMediaObjectProxy(pObject);
}

// init Collision

RenderingObject* Presenter::CreateCollisionProxy(Collision* pObject)
{
  //ML - commented out next line
  //InitGroupNode(pObject);
  return CreateMediaObjectProxy(pObject);
}

// init Node2D

/* virtual */ 
/*** 
void
Presenter::InitNode2D(Node2D* pObject)
{
  InitNode(pObject);
} 
***/

// init Conditional

RenderingObject* Presenter::CreateConditionalProxy(Conditional* pObject)
{
  return CreateMediaObjectProxy(pObject);
}

// init Switch2D

#ifdef PRE_FDIS
RenderingObject* Presenter::CreateSwitch2DProxy(Switch2D* pObject)
{
  //ML - commented out next line
  //InitGroupNode(pObject);
  return CreateMediaObjectProxy(pObject);
}
#endif // PRE_FDIS

// init Group2D

#ifdef PRE_FDIS
RenderingObject* Presenter::CreateGroup2DProxy(Group2D* pObject)
{
  //ML - commented out next line
  //InitGroupNode(pObject);
  return CreateMediaObjectProxy(pObject);
}
#endif // PRE_FDIS

// init Transform2D

RenderingObject* Presenter::CreateTransform2DProxy(Transform2D* pObject)
{
  //ML - commented out next line
  //InitGroupNode(pObject);
  return CreateMediaObjectProxy(pObject);
}

// init Anchor2D

#ifdef PRE_FDIS
RenderingObject* Presenter::CreateAnchor2DProxy(Anchor2D* pObject)
{
  //ML - commented out next line
  //InitGroupNode(pObject);
  return CreateMediaObjectProxy(pObject);
}
#endif // PRE_FDIS

// init Layout

RenderingObject* Presenter::CreateLayoutProxy(Layout* pObject)
{
  //ML - commented out next line
  //InitGroupNode(pObject);
  return CreateMediaObjectProxy(pObject);
}

// init Form

RenderingObject* Presenter::CreateFormProxy(Form* pObject)
{
  //ML - commented out next line
  //InitGroupNode(pObject);
  return CreateMediaObjectProxy(pObject);
}

// init LOD

RenderingObject* Presenter::CreateLODProxy(LOD* pObject)
{
  // InitGroupNode(pObject);
  return CreateMediaObjectProxy(pObject);
}

// init TextureNode2D

/* virtual */ 
/*** 
void
Presenter::InitTextureNode2D(TextureNode2D* pObject)
{
  InitNode2D(pObject);
} 
***/

// init VideoObject2D

#ifdef PRE_FDIS
RenderingObject* Presenter::CreateVideoObject2DProxy(VideoObject2D* pObject)
{
  return CreateStreamConsumerProxy(pObject);
}
#endif // PRE_FDIS

// init Image2D

#ifdef PRE_FDIS
RenderingObject* Presenter::CreateImage2DProxy(Image2D* pObject)
{
  return CreateStreamConsumerProxy(pObject);
}
#endif // PRE_FDIS

// init Inline2D

#ifdef PRE_FDIS
RenderingObject* Presenter::CreateInline2DProxy(Inline2D* pObject)
{
  return CreateStreamConsumerProxy(pObject);
}
#endif // PRE_FDIS

// init Bindable

/* virtual */ 
/*** 
void
Presenter::InitBindable(Bindable* pObject)
{
  InitNode(pObject);
} 
***/

// init Viewpoint

RenderingObject* Presenter::CreateViewpointProxy(Viewpoint* pObject)
{
  return CreateMediaObjectProxy(pObject);
}

// init WorldInfo

RenderingObject* Presenter::CreateWorldInfoProxy(WorldInfo* pObject)
{
  return CreateMediaObjectProxy(pObject);
}

// init Fog

RenderingObject* Presenter::CreateFogProxy(Fog* pObject)
{
  return CreateMediaObjectProxy(pObject);
}

// init Background

RenderingObject* Presenter::CreateBackgroundProxy(Background* pObject)
{
  return CreateMediaObjectProxy(pObject);
}

// init NavigationInfo

RenderingObject* Presenter::CreateNavigationInfoProxy(NavigationInfo* pObject)
{
  return CreateMediaObjectProxy(pObject);
}

// init InterpolatorNode

/* virtual */ 
/*** 
void
Presenter::InitInterpolatorNode(InterpolatorNode* pObject)
{
  InitNode(pObject);
} 
***/

// init Inline

RenderingObject* Presenter::CreateInlineProxy(Inline* pObject)
{
  return CreateStreamConsumerProxy(pObject);
}

// init AnimationStream

RenderingObject* Presenter::CreateAnimationStreamProxy(AnimationStream* pObject)
{
  return CreateStreamConsumerProxy(pObject);
}

// init TermCap

RenderingObject* Presenter::CreateTermCapProxy(TermCap* pObject)
{
  return CreateMediaObjectProxy(pObject);
}

// init TimeSensor

RenderingObject* Presenter::CreateTimeSensorProxy(TimeSensor* pObject)
{
  return CreateMediaObjectProxy(pObject);
}

// init ProximitySensor

RenderingObject* Presenter::CreateProximitySensorProxy(ProximitySensor* pObject)
{
  return CreateMediaObjectProxy(pObject);
}

// init VisibilitySensor

RenderingObject* Presenter::CreateVisibilitySensorProxy(VisibilitySensor* pObject)
{
  return CreateMediaObjectProxy(pObject);
}

// init DragSensor

/* virtual */ 
/*** 
void
Presenter::InitDragSensor(DragSensor* pObject)
{
  InitMediaObject(pObject);
} 
***/

// init PlaneSensor

RenderingObject* Presenter::CreatePlaneSensorProxy(PlaneSensor* pObject)
{
  return CreateMediaObjectProxy(pObject);
}

// init PlaneSensor2D

RenderingObject* Presenter::CreatePlaneSensor2DProxy(PlaneSensor2D* pObject)
{
  return CreateMediaObjectProxy(pObject);
}

// init SphereSensor

RenderingObject* Presenter::CreateSphereSensorProxy(SphereSensor* pObject)
{
  return CreateMediaObjectProxy(pObject);
}

// init CylinderSensor

RenderingObject* Presenter::CreateCylinderSensorProxy(CylinderSensor* pObject)
{
  return CreateMediaObjectProxy(pObject);
}

// init DiscSensor

RenderingObject* Presenter::CreateDiscSensorProxy(DiscSensor* pObject)
{
  return CreateMediaObjectProxy(pObject);
}

// init TouchSensor

RenderingObject* Presenter::CreateTouchSensorProxy(TouchSensor* pObject)
{
  return CreateMediaObjectProxy(pObject);
}

// init Appearance

RenderingObject* Presenter::CreateAppearanceProxy(Appearance* pObject)
{
  return CreateMediaObjectProxy(pObject);
}

// init MaterialNode

/* virtual */ 
/*** 
void
Presenter::InitMaterialNode(MaterialNode* pObject)
{
  InitMediaObject(pObject);
} 
***/

// init Material

RenderingObject* Presenter::CreateMaterialProxy(Material* pObject)
{
  return CreateMediaObjectProxy(pObject);
}

// init TextureNode

/* virtual */ 
/*** 
void
Presenter::InitTextureNode(TextureNode* pObject)
{
  InitMediaObject(pObject);
} 
***/

// init ImageTexture

RenderingObject* Presenter::CreateImageTextureProxy(ImageTexture* pObject)
{
  return CreateMediaObjectProxy(pObject);
}

// init FontStyle

RenderingObject* Presenter::CreateFontStyleProxy(FontStyle* pObject)
{
  return CreateMediaObjectProxy(pObject);
}


// init MovieTexture

RenderingObject* Presenter::CreateMovieTextureProxy(MovieTexture* pObject)
{
  return CreateStreamConsumerProxy(pObject);
}

// init PixelTexture

RenderingObject* Presenter::CreatePixelTextureProxy(PixelTexture* pObject)
{
  //ML - changed the next line (PixelTexture derives from MediaObject)
  //InitStreamConsumer(pObject);
  return CreateMediaObjectProxy(pObject);
}

// init Material2D

RenderingObject* Presenter::CreateMaterial2DProxy(Material2D* pObject)
{
  return CreateMediaObjectProxy(pObject);
}

// init LineProperties

RenderingObject* Presenter::CreateLinePropertiesProxy(LineProperties* pObject)
{
  return CreateMediaObjectProxy(pObject);
}

// init Coordinate

RenderingObject* Presenter::CreateCoordinateProxy(Coordinate* pObject)
{
  return CreateMediaObjectProxy(pObject);
}

// init Coordinate2D

RenderingObject* Presenter::CreateCoordinate2DProxy(Coordinate2D* pObject)
{
  return CreateMediaObjectProxy(pObject);
}

// init Normal

RenderingObject* Presenter::CreateNormalProxy(Normal* pObject)
{
  return CreateMediaObjectProxy(pObject);
}

// init TextureCoordinate

RenderingObject* Presenter::CreateTextureCoordinateProxy(TextureCoordinate* pObject)
{
  return CreateMediaObjectProxy(pObject);
}

// init Color

RenderingObject* Presenter::CreateColorProxy(Color* pObject)
{
  return CreateMediaObjectProxy(pObject);
}

// init TextureTransform

RenderingObject* Presenter::CreateTextureTransformProxy(TextureTransform* pObject)
{
  return CreateMediaObjectProxy(pObject);
}

// init Shape

RenderingObject* Presenter::CreateShapeProxy(Shape* pObject)
{
  return CreateMediaObjectProxy(pObject);
}

// init Geometry

/* virtual */ 
/*** 
void
Presenter::InitGeometry(Geometry* pObject)
{
  InitMediaObject(pObject);
} 
***/

// init Circle

RenderingObject* Presenter::CreateCircleProxy(Circle* pObject)
{
  return CreateMediaObjectProxy(pObject);
}

// init Rectangle

RenderingObject* Presenter::CreateRectangleProxy(class Rectangle* pObject)
{
  return CreateMediaObjectProxy(pObject);
}

// init IndexedFaceSet2D

RenderingObject* Presenter::CreateIndexedFaceSet2DProxy(IndexedFaceSet2D* pObject)
{
  return CreateMediaObjectProxy(pObject);
}

// init IndexedLineSet2D

RenderingObject* Presenter::CreateIndexedLineSet2DProxy(IndexedLineSet2D* pObject)
{
  return CreateMediaObjectProxy(pObject);
}

// init PointSet2D

RenderingObject* Presenter::CreatePointSet2DProxy(PointSet2D* pObject)
{
  return CreateMediaObjectProxy(pObject);
}

// init ElevationGrid

RenderingObject* Presenter::CreateElevationGridProxy(ElevationGrid* pObject)
{
  return CreateMediaObjectProxy(pObject);
}

// init Extrusion

RenderingObject* Presenter::CreateExtrusionProxy(Extrusion* pObject)
{
  return CreateIndexedFaceSetProxy((IndexedFaceSet *) (MediaObject *) pObject);
}

// init Box

RenderingObject* Presenter::CreateBoxProxy(Box* pObject)
{
  return CreateMediaObjectProxy(pObject);
}

// init Sphere

RenderingObject* Presenter::CreateSphereProxy(Sphere* pObject)
{
  return CreateMediaObjectProxy(pObject);
}

// init Cylinder

RenderingObject* Presenter::CreateCylinderProxy(Cylinder* pObject)
{
  return CreateMediaObjectProxy(pObject);
}

// init Cone

RenderingObject* Presenter::CreateConeProxy(Cone* pObject)
{
  return CreateMediaObjectProxy(pObject);
}

// init IndexedFaceSet

RenderingObject* Presenter::CreateIndexedFaceSetProxy(IndexedFaceSet* pObject)
{
  return CreateMediaObjectProxy(pObject);
}

// init IndexedLineSet

RenderingObject* Presenter::CreateIndexedLineSetProxy(IndexedLineSet* pObject)
{
  return CreateMediaObjectProxy(pObject);
}

// init PointSet

RenderingObject* Presenter::CreatePointSetProxy(PointSet* pObject)
{
  return CreateMediaObjectProxy(pObject);
}

// init Text

RenderingObject* Presenter::CreateTextProxy(Text* pObject)
{
  return CreateMediaObjectProxy(pObject);
}

// init StreamingText

#ifdef PRE_FDIS
RenderingObject* Presenter::CreateStreamingTextProxy(StreamingText* pObject)
{
  return CreateStreamConsumerProxy(pObject);
}
#endif // PRE_FDIS

// init Light

/* virtual */ 
/*** 
void
Presenter::InitLight(Light* pObject)
{
  InitNode3D(pObject);
} 
***/

// init DirectionalLight

RenderingObject* Presenter::CreateDirectionalLightProxy(DirectionalLight* pObject)
{
  return CreateMediaObjectProxy(pObject);
}

// init PointLight

RenderingObject* Presenter::CreatePointLightProxy(PointLight* pObject)
{
  return CreateMediaObjectProxy(pObject);
}

// init SpotLight

RenderingObject* Presenter::CreateSpotLightProxy(SpotLight* pObject)
{
  return CreateMediaObjectProxy(pObject);
}

// init Face

RenderingObject* Presenter::CreateFaceProxy(Face* pObject)
{
  return CreateMediaObjectProxy(pObject);
}

// init FIT

RenderingObject* Presenter::CreateFITProxy(FIT* pObject)
{
  return CreateMediaObjectProxy(pObject);
}

// init FDP

RenderingObject* Presenter::CreateFDPProxy(FDP* pObject)
{
  // InitGroupNode(pObject);
  return CreateMediaObjectProxy(pObject);
}

// init FAP

RenderingObject* Presenter::CreateFAPProxy(FAP* pObject)
{
  //ML - changed the next line (FAP derives from MediaObject)
  //InitStreamConsumer(pObject);
  return CreateMediaObjectProxy(pObject);
}

// init Expression

RenderingObject* Presenter::CreateExpressionProxy(Expression* pObject)
{
  return CreateMediaObjectProxy(pObject);
}

// init Viseme

RenderingObject* Presenter::CreateVisemeProxy(Viseme* pObject)
{
  return CreateMediaObjectProxy(pObject);
}

// init FaceDefTables

RenderingObject* Presenter::CreateFaceDefTablesProxy(FaceDefTables* pObject)
{
  return CreateMediaObjectProxy(pObject);
}

// init FaceDefMesh

RenderingObject* Presenter::CreateFaceDefMeshProxy(FaceDefMesh* pObject)
{
  return CreateMediaObjectProxy(pObject);
}

// init FaceDefTransform

RenderingObject* Presenter::CreateFaceDefTransformProxy(FaceDefTransform* pObject)
{
  return CreateMediaObjectProxy(pObject);
}

// init Sound

RenderingObject* Presenter::CreateSoundProxy(Sound* pObject)
{
  return CreateMediaObjectProxy(pObject);
}

// init Sound2D

RenderingObject* Presenter::CreateSound2DProxy(Sound2D* pObject)
{
  return CreateMediaObjectProxy(pObject);
}

// init AudioObject

/***
/& virtual &/ void
Presenter::InitAudioObject(AudioObject* pObject)
{
  InitMediaObject(pObject);
}
***/

// init AudioSource

RenderingObject* Presenter::CreateAudioSourceProxy(AudioSource* pObject)
{
  // InitAudioObject(pObject);
  return CreateMediaObjectProxy(pObject);
}

// init AudioSwitch

RenderingObject* Presenter::CreateAudioSwitchProxy(AudioSwitch* pObject)
{
  // InitAudioObject(pObject);
  return CreateMediaObjectProxy(pObject);
}

// init AudioFX

RenderingObject* Presenter::CreateAudioFXProxy(AudioFX* pObject)
{
  // InitAudioObject(pObject);
  return CreateMediaObjectProxy(pObject);
}

// init AudioMix

RenderingObject* Presenter::CreateAudioMixProxy(AudioMix* pObject)
{
  // InitAudioObject(pObject);
  return CreateMediaObjectProxy(pObject);
}

// init AudioDelay

RenderingObject* Presenter::CreateAudioDelayProxy(AudioDelay* pObject)
{
  // InitAudioObject(pObject);
  return CreateMediaObjectProxy(pObject);
}

// init Script

RenderingObject* Presenter::CreateScriptProxy(Script* pObject)
{
  return CreateMediaObjectProxy(pObject);
}

// delete object

void Presenter::DeleteObject(MediaObject* /* pObject */)
{
}


// **************************************************************************

// SB @@@@ added on 99/01/20 for new Nodes >>>>

RenderingObject* Presenter::CreateAudioBufferProxy(AudioBuffer * poNode)
{
  //ML - commented out next line
  //InitGroupNode(poNode);
  return CreateMediaObjectProxy(poNode);
}

RenderingObject* Presenter::CreateLayer2DProxy(Layer2D * poNode)
{
  //ML - commented out next line
  //InitGroupNode(poNode);
  return CreateMediaObjectProxy(poNode);
}

RenderingObject* Presenter::CreateLayer3DProxy(Layer3D * poNode)
{
  //ML - commented out next line
  //InitGroupNode(poNode);
  return CreateMediaObjectProxy(poNode);
}

RenderingObject* Presenter::CreateOrderedGroupProxy(OrderedGroup * poNode)
{
  //ML - commented out next line
  //InitGroupNode(poNode);
  return CreateMediaObjectProxy(poNode);
}

RenderingObject* Presenter::CreateAudioClipProxy(AudioClip * poNode)
{ return CreateStreamConsumerProxy(poNode); }

RenderingObject* Presenter::CreateCompositeTexture2DProxy(CompositeTexture2D * poNode)
{
  //ML - changed the next line (CompositeTexture2D derives from MediaObject)
  //InitStreamConsumer(poNode);
  return CreateMediaObjectProxy(poNode);
}

RenderingObject* Presenter::CreateCompositeTexture3DProxy(CompositeTexture3D * poNode)
{
  //ML - changed the next line (CompositeTexture3D derives from MediaObject)
  //InitStreamConsumer(poNode);
  return CreateMediaObjectProxy(poNode);
}

RenderingObject* Presenter::CreateBackground2DProxy(Background2D * poNode)
{ return CreateMediaObjectProxy(poNode); }
RenderingObject* Presenter::CreateBitmapProxy(Bitmap * poNode)
{ return CreateMediaObjectProxy(poNode); }
RenderingObject* Presenter::CreateColorInterpolatorProxy(ColorInterpolator * poNode)
{ return CreateMediaObjectProxy(poNode); }
RenderingObject* Presenter::CreateCoordinateInterpolatorProxy(CoordinateInterpolator * poNode)
{ return CreateMediaObjectProxy(poNode); }
RenderingObject* Presenter::CreateCoordinateInterpolator2DProxy(CoordinateInterpolator2D * poNode)
{ return CreateMediaObjectProxy(poNode); }
RenderingObject* Presenter::CreateCurve2DProxy(Curve2D * poNode)
{ return CreateMediaObjectProxy(poNode); }
RenderingObject* Presenter::CreateListeningPointProxy(ListeningPoint * poNode)
{ return CreateMediaObjectProxy(poNode); }
RenderingObject* Presenter::CreateNormalInterpolatorProxy(NormalInterpolator * poNode)
{ return CreateMediaObjectProxy(poNode); }
RenderingObject* Presenter::CreateOrientationInterpolatorProxy(OrientationInterpolator * poNode)
{ return CreateMediaObjectProxy(poNode); }
RenderingObject* Presenter::CreatePositionInterpolatorProxy(PositionInterpolator * poNode)
{ return CreateMediaObjectProxy(poNode); }
RenderingObject* Presenter::CreatePositionInterpolator2DProxy(PositionInterpolator2D * poNode)
{ return CreateMediaObjectProxy(poNode); }
RenderingObject* Presenter::CreateProximitySensor2DProxy(ProximitySensor2D * poNode)
{ return CreateMediaObjectProxy(poNode); }
RenderingObject* Presenter::CreateQuantizationParameterProxy(QuantizationParameter * poNode)
{ return CreateMediaObjectProxy(poNode); }
RenderingObject* Presenter::CreateScalarInterpolatorProxy(ScalarInterpolator * poNode)
{ return CreateMediaObjectProxy(poNode); }
RenderingObject* Presenter::CreateValuatorProxy(Valuator * poNode)
{ return CreateMediaObjectProxy(poNode); }

// SB @@@@ added on 99/01/20 for new Nodes <<<<


//ML - removed the following descriptor nodes
#if 0
// SB @@@@ added on 99/01/20 for descriptors Nodes >>>>
void Presenter::InitObjectDescriptor(MediaObject * /* poNode */) {}
void Presenter::InitES_Descriptor(MediaObject * /* poNode */) {}
void Presenter::InitDecoderConfigDescriptor(MediaObject * /* poNode */) {}
void Presenter::InitBIFSDecoderConfig(MediaObject * /* poNode */) {}
void Presenter::InitIPI_Descriptor(MediaObject * /* poNode */) {}
void Presenter::InitIP_InformationDataSet(MediaObject * /* poNode */) {}
void Presenter::InitSupplementaryContentIdentifier(MediaObject * /* poNode */) {}
void Presenter::InitQoS_Descriptor(MediaObject * /* poNode */) {}
void Presenter::InitQoS_Qualifier(MediaObject * /* poNode */) {}
void Presenter::InitExtensionDescriptor(MediaObject * /* poNode */) {}
void Presenter::InitShortExtensionDescriptor(MediaObject * /* poNode */) {}
void Presenter::InitIPMP_DescriptorPointer(MediaObject * /* poNode */) {}
void Presenter::InitALConfigDescriptor(MediaObject * /* poNode */) {}
void Presenter::InitIPMPDescriptor(MediaObject * /* poNode */) {}
// SB @@@@ added on 99/01/20 for descriptors Nodes <<<<
#endif


#ifdef ADV_AUDIO
RenderingObject* Presenter::CreateAcousticMaterialProxy(AcousticMaterial * poNode)
{ return CreateMediaObjectProxy(poNode); }
RenderingObject* Presenter::CreateAcousticSceneProxy(AcousticScene * poNode)
{ return CreateMediaObjectProxy(poNode); }
RenderingObject* Presenter::CreateDirectiveSoundProxy(DirectiveSound * poNode)
{ return CreateMediaObjectProxy(poNode); }
RenderingObject* Presenter::CreatePerceptualParametersProxy(PerceptualParameters * poNode)
{ return CreateMediaObjectProxy(poNode); }

#endif // ADV_AUDIO


//ML - added the following nodes
RenderingObject* Presenter::CreateBAPProxy(BAP * /* poNode */)
	{ return NULL; }
RenderingObject* Presenter::CreateBDPProxy(BDP * /* poNode */)
	{ return NULL; }
RenderingObject* Presenter::CreateBodyProxy(Body * /* poNode */)
	{ return NULL; }
RenderingObject* Presenter::CreateBodyDefTableProxy(BodyDefTable * /* poNode */)
	{ return NULL; }
RenderingObject* Presenter::CreateBodySegmentConnectionHintProxy(BodySegmentConnectionHint * /* poNode */)
	{ return NULL; }
RenderingObject* Presenter::CreateHierarchical3DMeshProxy(Hierarchical3DMesh * /* poNode */)
	{ return NULL; }
RenderingObject* Presenter::CreateMaterialKeyProxy(MaterialKey * /* poNode */)
	{ return NULL; }
RenderingObject* Presenter::CreateServerCommandProxy(ServerCommand * /* poNode */)
	{ return NULL; }


// **************************************************************************
