// This software module was originally developed by Wenlong Dong(Rockwell 
// Science Center) and Wei Wu(Rockwell Science Center) in the course of 
// development of the MPEG-4 Systems standard
// (ISO/IEC 14496-1).  This software module is an implementation of a part
// of one or more MPEG-4 Systems tools as specified by the MPEG-4 Systems
// standard.  ISO/IEC gives users of the MPEG-4 Systems standard free
// license to this software module or modifications thereof for use in
// hardware or software products claiming conformance to the MPEG-4 Systems
// standard.  Those intending to use this software module in hardware or
// software products are advised that its use may infringe existing
// patents.  The original developer of this software module and his/her
// company, the subsequent editors and their companies, and ISO/IEC have no
// liability for use of this software module or modifications thereof in an
// implementation.  Copyright is not released for non MPEG-4 Systems
// conforming products.  The original developer retains full right to use
// the code for his/her own purpose, assign or donate the code to a third
// party and to inhibit third parties from using the code for non MPEG-4
// Systems conforming products.  This copyright notice must be included in
// all copies or derivative works.
//
// For more information or to receive an up to date version of this module,
// you may call Wenlong Dong at wdong@usc.edu or Wei Wu at 
// weiwu@jupiter.risc.rockwell.com.
//
// Copyright 1998.

/*-------------------------------------------------------------------------
Project Im1Player
INTERNATIONAL ORGANIZATION FOR STANDARDIZATION
ISO/IEC JTC1/SC29/WG11

SUBSYSTEM:	Script Node
FILE:       VSEvnets.cpp
AUTHOR:     Wenlong Dong (Rockwell Science Center)
			Wei Wu		 (Rockwell Science Center)	
		    Yuval Fisher (Rockwell Science Center)
OVERVIEW:   Implementations for the Derived classes: Field, EventIn, 
			EventOut of the class VSNodeField.
--------------------------------------------------------------------*/
#include "defs.h"
#include "..\..\Core\Quantize.h"
#include <float.h>
#include "VSEvents.h"
#ifdef BIFSENC
#include "..\..\BifsEnc\BifsEnc.h"
#include "..\..\BifsEnc\Token.h"
#else
#include "..\..\Core\BifsDec.h"
#include "..\interpreter\DecScript.h"
#include "..\interpreter\vsfields.h"
#include "..\interpreter\vsfunction.h"
#include "VScript.h"	// NODEFIELD
#include "..\..\Core\Root.h"	// NODEFIELD
#include "..\..\Core\NodeFld.h"	// NODEFIELD
#include "..\..\Core\Executiv.h"	// ROK1999
#include "..\..\Core\PresBase.h"	// ROK1999
#endif

EventIn::EventIn()
{ 
	flags.eType = eEVENTIN; 
#ifndef BIFSENC
	
	timeStamp = new VSNodeField(SFTIME);
#endif
}

EventOut::EventOut()
{ 
	flags.eType = eEVENTOUT; 

#ifndef BIFSENC
	beenSet = FALSE; 
#endif
}

EventOut::~EventOut()
{
#ifndef BIFSENC
	 if(pValueCopy != NULL) 
		 delete pValueCopy; 
#endif
}

#ifndef BIFSENC
BOOL EventOut::Parse(BIFSDecoder *pBifs)
{
	VSNodeField::Parse(pBifs);
	pValueCopy = CreateField(flags.fType);
	return TRUE;
}
#endif

#ifdef BIFSENC
BOOL Field::Encode(BIFSEncoder *pBifs)
{
	VSNodeField::Encode(pBifs);
	Encode_Default(pBifs);
	return TRUE;
}

void Field::Encode_Default(BIFSEncoder *pBifs)
{
	if((nextToken == TID_INTEGER) ||(nextToken == TID_NUMBER)
		|| (nextToken == TID_STRING) || (nextToken == TID_LEFT_BRACKET)
		|| (nextToken == TID_TRUE) || (nextToken == TID_FALSE)
		|| (nextToken == TID_USE))	// NODEFIELD
		hasDefault = TRUE;

	pBifs -> EncodeInt(hasDefault, 1, "hasDefault");
	if(hasDefault)
	{
		if (nextToken == TID_USE) { // NODEFIELD
			nextToken.Scan();
			if (nextToken == TID_NAME) {
				nNodeID = nextToken.GetDefinedName();
				if (nNodeID >= 0)
					pBifs->EncodeInt(nNodeID, 10, "nodeID");
				else
					nextToken.Error("Name not found");
				nNodeID = nextToken.GetNodeID(nNodeID);
				nextToken.Scan();
			} else {
				nNodeID = -1;
				nextToken.Error("Unexpected symbol");
				nextToken.Scan();
			}
//			pValue = new SFGenericNode();
		} else {
			float fmin = FLT_MIN, fmax = FLT_MAX;
			Quantizer quant(&fmin, &fmax, 1, 0);
			pValue = CreateField(flags.fType);
			pValue -> Encode(pBifs, &quant);
		}
	}
}

#else
BOOL Field::Parse(BIFSDecoder *pBifs)
{
	VSNodeField::Parse(pBifs);
	hasDefault = pBifs -> ParseInt(1);

	if(hasDefault)
	{
		if (flags.fType == SFNODE) { // NODEFIELD
			int nID = pBifs->ParseInt(10);
			SFGenericNode *pField = (SFGenericNode *) pValue;
			pField -> SetObject (pSNode->GetRoot()->FindNode(nID), pSNode);
		} else {
			float fmin = FLT_MIN, fmax = FLT_MAX;
			Quantizer quant(&fmin, &fmax, 1, 0);
			pValue -> Parse(pBifs, NULL, &quant);
		}
	}

	return TRUE;
}

void EventIn::InitField(DecScript *p)
{
	VsExprList arguments;

	parent = p;
	function = p -> findFunction(name);
	if(function == NULL)
		exit(-1);
	eventArg = VsField::create(flags.fType, VsField::RO);
	timeArg = VsField::create(SFTIME, VsField::RO);

	arguments += eventArg;
	arguments += timeArg;

	f_call = new VsFunctionCall(function, arguments);
}

void EventOut::OnFieldUpdate(BOOL Onflag)
{
	if(!Onflag)
		beenSet = FALSE;
	else
	{
		if(beenSet)
			pValue -> Copy (*pValueCopy, NULL);
	}
}

void EventIn::OnFieldUpdate(BOOL)
{
	 // set the time stamp later??
//	(Time)*(SF<Time> *)(NodeField *)*timeStamp = 0;
//	timeStamp = 0;
	*(SF<Time> *)(NodeField *)*timeStamp =
		GetSNode()->GetExecutive()->GetPresenter()->GetTime();	// ROK1999
	eventArg -> setField(this);
	timeArg -> setField(timeStamp);
	f_call->evaluate(NULL); 
}

#endif