// This software module was originally developed by Wenlong Dong(Rockwell 
// Science Center) and Wei Wu(Rockwell Science Center) in the course of 
// development of the MPEG-4 Systems standard
// (ISO/IEC 14496-1).  This software module is an implementation of a part
// of one or more MPEG-4 Systems tools as specified by the MPEG-4 Systems
// standard.  ISO/IEC gives users of the MPEG-4 Systems standard free
// license to this software module or modifications thereof for use in
// hardware or software products claiming conformance to the MPEG-4 Systems
// standard.  Those intending to use this software module in hardware or
// software products are advised that its use may infringe existing
// patents.  The original developer of this software module and his/her
// company, the subsequent editors and their companies, and ISO/IEC have no
// liability for use of this software module or modifications thereof in an
// implementation.  Copyright is not released for non MPEG-4 Systems
// conforming products.  The original developer retains full right to use
// the code for his/her own purpose, assign or donate the code to a third
// party and to inhibit third parties from using the code for non MPEG-4
// Systems conforming products.  This copyright notice must be included in
// all copies or derivative works.
//
// For more information or to receive an up to date version of this module,
// you may call Wenlong Dong at wdong@usc.edu or Wei Wu at 
// weiwu@jupiter.risc.rockwell.com.
//
// Copyright 1998.

/*-------------------------------------------------------------------------
Project Im1Player
INTERNATIONAL ORGANIZATION FOR STANDARDIZATION
ISO/IEC JTC1/SC29/WG11

SUBSYSTEM:	Script Node
FILE:       VSNodeField.h
AUTHOR:     Wenlong Dong (Rockwell Science Center)
			Wei Wu       (Rockwell Science Center)
		    Yuval Fisher (Rockwell Science Center)
OVERVIEW:   Implementations for the class VSNodeField.
--------------------------------------------------------------------*/
#include "defs.h"
#include "VSNodeField.h"
#ifdef BIFSENC
#include "..\..\BifsEnc\Token.h"
#include "..\..\BifsEnc\BifsEnc.h"
#else
#include "..\..\Core\BifsDec.h"
#include "..\..\Core\NodeFld.h"
#include "..\..\Core\MediaObj.h"
#include "..\interpreter\DecScript.h"
#include "..\interpreter\vsfields.h"
#endif

VSNodeField::VSNodeField(FIELD_TYPE type)
{
	flags.eType = eUNKNOWN_TYPE;
	flags.fType = type;

#ifndef BIFSENC
	beenSet = TRUE;

	if(type != UNKNOWN_TYPE)
	{
		pValue = CreateField(type);
	}
#endif
}

VSNodeField::~VSNodeField()
{
	if(pValue)
		delete pValue;
}

#ifdef BIFSENC
BOOL VSNodeField::Encode(BIFSEncoder *pBifs)
{
	pBifs -> EncodeInt(flags.eType, 2, "Event type");

	if(nextToken != TID_NAME)
		nextToken.Error("Not an appropriate field type");
	else
	{
		//for(int type = SFWORLDNODE; type <= MFVEC3F; type++)
		for(int type = SFNODE; type <= MFVEC3F; type++) // NODEFIELD
		{
			if(!stricmp(nextToken.GetName(), field_reserved[type]))
			{
				flags.fType = (FIELD_TYPE)type;
				break;
			}
		}
	}

	if(flags.fType == UNKNOWN_TYPE) 
		nextToken.Error("Unrecognized Field type");
	pBifs -> EncodeInt (flags.fType, 6, "Field type");

	nextToken.Scan();
	if(nextToken == TID_NAME)
		name = _strdup(nextToken.GetName());
	else 
		nextToken.Error("Not a valid name");

	nextToken.Scan();

	return TRUE;
}

#else
void VSNodeField::SetID(int nIndex)
{
	Id = nIndex;
	char var[20];

	sprintf(var, "var%d", Id);
	name = _strdup(var);
}

BOOL VSNodeField::Parse(BIFSDecoder *pBifs)
{
	flags.fType = (FIELD_TYPE)pBifs -> ParseInt(6);

	pValue = CreateField(flags.fType);

	return TRUE;
}


void VSNodeField::InitField(DecScript *p)
{	
	parent = p;
	VsField *Gbfield = VsField::create(flags.fType, VsField::RW);
	Gbfield -> setField(this);
	p -> addGlobalVar(name, Gbfield);
}
#endif

NodeField *VSNodeField::CreateField(FIELD_TYPE type)
{
	switch(type)
	{
	/*
	case SFWORLDNODE:
	case SF3DNODE:
	case SF2DNODE:
	case SFSTREAMINGNODE:
	case SFAUDIONODE:
	case SFGEOMETRYNODE:
	case SFAPPEARANCENODE:
	case SFCOLORNODE:
	case SFFONTSTYLENODE:
	case SFTEXTURENODE:
	case SFTEXTURECOORDINATENODE:
	case SFTEXTURETRANSFORMNODE:
	case SFTIMESENSORNODE:
	case SFWORLDINFONODE:
	case SFCOORDINATE2DNODE:
	case SFTOPNODE:
	case SFLINEPROPERTIESNODE:
	case SFMATERIALNODE:
	case SFSHADOWPROPERTIESNODE:
	case SFFACENODE:
	case SFBODYNODE:
	case SFFITNODE:
	case SFFAPNODE:
	case SFFDPNODE:
	case SFFBADEFNODE:
	case SFFBADEFTABLENODE:
	case SFCOORDINATENODE:
	case SFNORMALNODE:
	case SFLAYERNODE:
	*/
	case SFNODE:
//		return NULL; // Not implement right now ??
		return new SFGenericNode(); // NODEFIELD
	case SFBOOL:
        return new SF<Bool>;
	case SFTIME:
		return new SF<Time>;
    case SFCOLOR:
		return new SF<Color3f>;
	case MFCOLOR:
		return new MF<Color3f>;
	case SFFLOAT:
		return new SF<float>;
	case MFFLOAT:
		return new MF<float>;
    case SFINT32:
		return new SF<int>;
	case MFINT32:
		return new MF<int>;
    case SFROTATION:
		return new SF<Rotation>;
	case MFROTATION:  
		return new MF<Rotation>;
	case SFSTRING:  
		return new SFString;
	case MFSTRING:
		return new MF<ZString>;
    case SFVEC2F:
		return new SF<Vec2f>;
	case MFVEC2F:     
		return new MF<Vec2f>;
	case SFVEC3F:   
		return new SF<Vec3f>;
	case MFVEC3F:
		return new MF<Vec3f>;
	default:
		return NULL;
	}
}

#ifdef BIFSENC
LPCSTR VSNodeField::field_reserved [] =
{
	/*
	"SFWorldNode",
	"SF3DNode",
	"SF2DNode",
	"SFStreamingNode",
	"SFAudioNode",
	"SFGeometryNode",
	"SFAppearanceNode",
	"SFColorNode",
	"SFFontStyleNode",
	"SFTextureNode",
	"SFTextureCoordinateNode",
	"SFTextureTransformNode",
	"SFTimeSensorNode",
	"SFWorldInfoNode",
	"SFCoordinate2DNode",
	"SFTopNode",
	"SFLinePropertiesNode",
	"SFMaterialNode",
	"SFShadowPropertiesNode",
	"SFFaceNode",
	"SFBodyNode",
	"SFFITNode",
	"SFFAPNode",
	"SFFDPNode",
	"SFFBADefNode",
	"SFFBADefTableNode",
	"SFCoordinateNode",
	"SFNormalNode",
	"SFLayerNode",
	*/
	"SFNode", // NODEFIELD
	"SFBool",        
	"SFTime",
    "SFColor",   
	"MFColor",     
	"SFFloat",     
	"MFFloat",
    "SFInt32",    
	"MFInt32",     
    "SFRotation", 
	"MFRotation",  
	"SFString",    
	"MFString",
    "SFVec2f",    
	"MFVec2f",     
	"SFVec3f",     
	"MFVec3f"
};
#endif


