// This software module was originally developed by Wenlong Dong(Rockwell 
// Science Center) and Wei Wu(Rockwell Science Center) in the course of 
// development of the MPEG-4 Systems standard
// (ISO/IEC 14496-1).  This software module is an implementation of a part
// of one or more MPEG-4 Systems tools as specified by the MPEG-4 Systems
// standard.  ISO/IEC gives users of the MPEG-4 Systems standard free
// license to this software module or modifications thereof for use in
// hardware or software products claiming conformance to the MPEG-4 Systems
// standard.  Those intending to use this software module in hardware or
// software products are advised that its use may infringe existing
// patents.  The original developer of this software module and his/her
// company, the subsequent editors and their companies, and ISO/IEC have no
// liability for use of this software module or modifications thereof in an
// implementation.  Copyright is not released for non MPEG-4 Systems
// conforming products.  The original developer retains full right to use
// the code for his/her own purpose, assign or donate the code to a third
// party and to inhibit third parties from using the code for non MPEG-4
// Systems conforming products.  This copyright notice must be included in
// all copies or derivative works.
//
// For more information or to receive an up to date version of this module,
// you may call Wenlong Dong at wdong@usc.edu or Wei Wu at 
// weiwu@jupiter.risc.rockwell.com.
//
// Copyright 1998.

/*-------------------------------------------------------------------------
Project Im1Player
INTERNATIONAL ORGANIZATION FOR STANDARDIZATION
ISO/IEC JTC1/SC29/WG11

SUBSYSTEM:	ScriptImp Node
FILE:       VScript.cpp
AUTHOR:     Wenlong Dong (Rockwell Science Center)
			Wei Wu       (Rockwell Science Center)
		    Yuval Fisher (Rockwell Science Center)
OVERVIEW:   Implementation of the ScriptImp class.
--------------------------------------------------------------------*/
#include "defs.h"
#include "VScript.h"
#include "VSEvents.h"
#include "..\..\Core\NodeCode.h"
#ifdef BIFSENC
#include "..\..\BifsEnc\Token.h"
#include "..\..\BifsEnc\BifsEnc.h"
#include "CParser.h"
#else
#include "..\..\Core\BifsDec.h"
#include "..\..\Core\Route.h"
#include "..\..\Core\Root.h"
#include "..\interpreter\DecScript.h"
#endif

ScriptImp::ScriptImp()
{
	directOutput = FALSE;
	mustEvaluate = FALSE;
	EncUrl = FALSE;
	NumIn = 0;
	NumOut = 0;
	NumField = 0;
}

ScriptImp::~ScriptImp()
{
}

#ifdef BIFSENC
EVENT_TYPE ScriptImp::Scan_eType()
{
	const char *str = nextToken.GetName();
	if(!strcmp(str, "eventIn"))
		return eEVENTIN;
	else if(!strcmp(str, "eventOut"))
		return eEVENTOUT;
	else if(!strcmp(str, "field"))
		return eFIELD;
	else {
		nextToken.Error("Unrecognized event type");
		return eUNKNOWN_TYPE;
	}
}

/*-----------------------------------------------------------------
Encode the ScriptImp Node. Left problems: 
(1) How to deal with initialize and shutdown ???
(2) A field of type SFNode and MFNode is not implemented ???
(3) In script, the "new" method is not implemented ???
-----------------------------------------------------------------*/
BOOL ScriptImp::Encode(BIFSEncoder *pCoder)
{
	if(nextToken != TID_LEFT_BRACE) 
	{
		nextToken.Error("Left brace { is expected!");
		return FALSE;
	}
		
	nextToken.Scan();
	while(nextToken != TID_RIGHT_BRACE) 
	{
		if(!stricmp(nextToken.GetName(),"url")) 
		{
			nextToken.Scan();
			if(nextToken != TID_STRING)
				nextToken.Error("Not a url string");

			ZString string = nextToken.GetString();
			int pos;
			char *str = GetHeader(string, pos);
			if(!stricmp(str,"javascript:"))
			{
				EncUrl = TRUE;
				string.Cut(0,pos);
			}

			delete [] str;
			url += string;
			nextToken.Scan();
		}
		else if(!stricmp(nextToken.GetName(), "directOutput"))
		{
			nextToken.Scan();
			if(nextToken == TID_TRUE)
				directOutput = TRUE;
			else if(nextToken == TID_FALSE)
				directOutput = FALSE;
			else
				nextToken.Error("Unrecognized boolean value");
		}
		else if(!stricmp(nextToken.GetName(), "mustEvaluate"))
		{
			nextToken.Scan();
			if(nextToken == TID_TRUE)
				mustEvaluate = TRUE;
			else if(nextToken == TID_FALSE)
				mustEvaluate = FALSE;
			else
				nextToken.Error("Unrecognized boolean value");			
		}
		else
		{
			pCoder -> EncodeInt(1, 1, "list description");

			EVENT_TYPE eType = Scan_eType();
			VSNodeField *event = NULL;
			switch(eType)
			{
			case eEVENTIN:
				event = new EventIn;
				EventInList += event;
				NumIn ++;
				break;
			case eEVENTOUT:
				event = new EventOut;
				EventOutList += event;
				NumOut ++;
				break;
			case eFIELD:
				event = new Field;
				FieldList += event;
				NumField ++;
				break;
			}
			
			nextToken.Scan();
			event -> Encode(pCoder);
		}
	}

	pCoder -> EncodeInt(0, 1, "endFlag");
	pCoder -> EncodeInt((Bool) directOutput, 1, "directOutput");
	pCoder -> EncodeInt((Bool) mustEvaluate, 1, "mustEvaluate");
	pCoder -> EncodeInt (EncUrl, 1, "Encode URL string");
	if(!EncUrl)
		url.Encode(pCoder);
	else
	{
		int i;
		pCParser = new CParser((LPCSTR)url [0]);
		for(i = 0; i < NumIn; i++)
			pCParser -> InsertExID (EventInList[i] -> GetName());
		for(i = 0; i < NumOut; i++)
			pCParser -> InsertExID (EventOutList[i] -> GetName());
		for(i = 0; i < NumField; i++)
			pCParser -> InsertExID (FieldList[i] -> GetName());
		pCParser -> SetSNode(this);	// NODEFIELD
		pCParser -> Encode(pCoder);
		delete pCParser;
	}

	// Save this ScriptImp object in BifsEncoder for Route encoding
	if(GetID() >= 0)
		pCoder -> ScriptList += this;

	nextToken.Scan();
	return TRUE;
}

void ScriptImp::EncodeRoute(BIFSEncoder *pCoder, BOOL bIn)
{
	pCoder -> EncodeInt(GetID(), 10, "NodeID");
	if(nextToken != TID_PERIOD)
		nextToken.Error("period . expected");
	else
		nextToken.Scan();
		
	if(nextToken == TID_NAME)
	{
		for(int i=0; i < pCoder->ScriptList.GetLength(); i++) 
		{	

			ScriptImp *pScript = pCoder -> ScriptList[i];
			if(GetID() == pScript -> GetID()) 
			{
				int j;
				if(bIn) 
				{
					for(j = 0; j < pScript -> NumIn; j++) 
					{
						if(!strcmp(nextToken.GetName(), 
							pScript ->EventInList[j]->GetName()))
						{
							pCoder -> EncodeInt(j, BitFieldWidth(pScript -> NumIn), 
								"inFieldReference");							
							break;
						}
					}

					if(j >= pScript -> NumIn)
						nextToken.Error("No eventIn with this name");
				}
				else 
				{
					for(j = 0; j < pScript -> NumOut; j++) 
					{
						if(!strcmp(nextToken.GetName(), 
							pScript ->EventOutList[j]->GetName())) 
						{
							pCoder -> EncodeInt(j, BitFieldWidth(pScript -> NumOut), 
								"outFieldReference");
							break;
						}
					}

					if(j >= pScript -> NumOut)
						nextToken.Error("No eventOut with this name");
				} // if...else...
			} // if
		} // for
	} 
	else
		nextToken.Error("Unexpected symbol");
	
	nextToken.Scan();
}

char *ScriptImp::GetHeader(ZString string, int &pos)
{
	int pos1 = 0;
	char *header;

	while((string[pos1]==' ')||(string[pos1]=='\n')||(string[pos1]=='\t'))
		pos1++;

	pos = pos1;
	for(int i=0; i<15; i++)
	{
		char c = string[pos];
		pos++;
		if(!isalpha(c))
			break;
	}
	
	header = _strdup((LPCSTR)(string.Substr(pos1, pos-pos1)));
	return header;
}

#else
/*--------------------------------------------------------------------
Parse the ScriptImp Node. Problems left:
(1) directOutput and mustEvaluate are not implemented
(2) If the URL field is not an inline javascript, it is not 
	implemented
--------------------------------------------------------------------*/
BOOL ScriptImp::Parse(BIFSDecoder *pCoder)
{
	TRACEF ("%s {", GetName());
	TRACENEWLINE;
	TRACEINDENT (1);

	while(pCoder -> ParseInt(1))
	{
		EVENT_TYPE eType = (EVENT_TYPE)pCoder -> ParseInt(2);
		switch(eType)
		{
		case eEVENTIN:
			EventInList += new EventIn;
			EventInList[NumIn]->SetSNode(this);	// ROK1999
			EventInList[NumIn++] -> Parse(pCoder);
			break;
		case eEVENTOUT:
			EventOutList += new EventOut;
			EventOutList[NumOut++] -> Parse(pCoder);
			break;
		case eFIELD:
			FieldList += new Field;
			FieldList[NumField]->SetSNode(this); // NODEFIELD
			FieldList[NumField++] -> Parse(pCoder);
			break;
		default:
			break;
		}
	}

	int i;
	for(i = 0; i < NumIn; i++)
		EventInList[i] -> SetID(i);

	for(i = 0; i < NumOut; i++)
		EventOutList[i] -> SetID(i + NumIn);

	for(i = 0; i < NumField; i++)
		FieldList[i] -> SetID(i + NumIn + NumOut);

	directOutput = pCoder -> ParseInt(1) != 0;
	mustEvaluate = pCoder -> ParseInt(1) != 0;

	EncUrl = pCoder -> ParseInt(1);
	if(!EncUrl)
		url.Parse(pCoder, NULL);
	else
	{
		pDecScript = new DecScript;
		
		pDecScript -> InsertExID(NumIn + NumOut + NumField);

		int i;
		for(i = 0; i < NumOut; i++)
			EventOutList[i] -> InitField(pDecScript);

		for(i = 0; i < NumField; i++)
			FieldList[i] -> InitField(pDecScript);

		pDecScript -> Decode(pCoder);

		for(i = 0; i < NumIn; i++)
			EventInList[i] -> InitField(pDecScript);
	}

	TRACEINDENT (-1);
	TRACEF ("}", 0);
	TRACENEWLINE;
	return TRUE;
}

/*---------------------------------------------------------------------
The function AddRoute is called for eventOuts.
(1) A Route is created and attached to the eventOut. 
(2) The eventOut is also bound to the Route.
(3) The TO field(eventIn) of the Route has the ID number in the Route
---------------------------------------------------------------------*/
BOOL ScriptImp::AddRoute(int nRouteID, BIFSDecoder *pCoder, LPCSTR name)
{
	int nIndex = pCoder -> ParseInt(BitFieldWidth(NumOut));

	NodeField *pField = (*(EventOut *)EventOutList[nIndex]).GetField();
	if(pField == NULL)
		return FALSE;

	TRACEF("var%d TO ", nIndex);
	Route *pRoute = new Route (nRouteID, pField, nIndex, this, name);

	int nNodeID = pCoder -> ParseInt (10);
	MediaObject *pNode = GetRoot () -> FindNode (nNodeID);
	if(pNode == NULL)
		return FALSE;
	TRACEF ("%s.", pNode ->GetNameID ());

	return pNode -> BindRoute (pRoute, pCoder);
}

BOOL ScriptImp::BindRoute(Route *pRoute, BIFSDecoder *pCoder)
{
	int nIndex = pCoder -> ParseInt(BitFieldWidth(NumIn));

	TRACEF ("var%d", nIndex);
	pRoute -> Bind (this, nIndex);

	return TRUE;
}

/*---------------------------------------------------------------------
The function RouteEvent is called by the Update procedure of the eventOut 
of another Node in the same ROUTE. This node is bound to the Route in the
this -> BindRoute() function:
(1) The eventIn with the fixed index is evaluated by the input Field value 
(2) All eventOuts are set to HasBeenSet = FALSE
(3) The corresponding function of the eventIn is evaluated
(4) Update all eventOuts at last
---------------------------------------------------------------------*/
void ScriptImp::RouteEvent (NodeField *pField, int nFieldIndex)
{
	int i;

	((NodeField *)*EventInList[nFieldIndex]) -> Copy (*pField, this, false);

	for(i = 0; i < NumOut; i++)
		EventOutList[i] -> OnFieldUpdate(FALSE);

	EventInList[nFieldIndex] -> OnFieldUpdate();

	for(i = 0; i < NumOut; i++)
		EventOutList[i] -> OnFieldUpdate(TRUE);
}

#endif