//////////////////////////////////////////////////////////////////////////////
//This software module was originally developed by CSELT in the course of   //
//development of the MPEG-4 Systems (ISO/IEC 14496-1) standard.             //
//This software module is an implementation of a part of one or more MPEG-4 //
//Systems (ISO/IEC 14496-1) tools as specified by the MPEG-4 Systems        //
//(ISO/IEC 14496-1) standard. ISO/IEC gives users of the MPEG-4 Systems     //
//(ISO/IEC 14496-1) free license to this software module or modifications   //
//thereof for use in hardware or software products claiming conformance to  //
//the MPEG-4 Systems (ISO/IEC 14496-1). Those intending to use this software//
//module in hardware or software products are advised that its use may      //
//infringe existing patents.                                                //
//The original developer of this software module and his/her company, the   //
//subsequent editors and their companies, and ISO/IEC have no liability     //
//for use of this software module or modifications thereof in an            //
//implementation. Copyright is not released for non MPEG-4 Systems          //
//(ISO/IEC 14496-1) conforming products. CSELT S.pA. retains full right     //
//to use the code for his/her own purpose, assign or donate the code to a   //
//third party and to inhibit third parties from using the code for non      //
//MPEG-4 Systems (ISO/IEC 14496-1) conforming products. This copyright      //
//notice must be included in all copies or derivative works.                //
//Copyright  1999.                                                         //
//This work has been partly developed within the framework of the ESPRIT    //
//European Research Project  "MPEG-4 PC".                                   //
//////////////////////////////////////////////////////////////////////////////
//
// File:  AudioSourceProxy.cpp
//
// Author: Matthew Leditschke
//
// Modified: Gianluca Di Cagno  gianluca.dicagno@cselt.it
//
// Description:
//   The implementation of the proxy object used by the presenter
//   to render the AudioSource node.
//
// Created: August, 1997.
//
//
///////////////////////////////////////////////////////////////////////////


#include "defs.h"
#include <assert.h>
#include "Stream.h"
#include "AudioSourceProxy.h"
#include "Clock.h"
#include "trace.h"

//FILE* outfile;


// =====================================================================

//
// Presenter::InitAudioSource
//
// Initialise the proxy data structure and attach it to the render data
// pointer of the given media object node.
//

AudioSourceProxy * Presenter::InitAudioSource(AudioSource* pObj)
{
	return (new AudioSourceProxy(pObj));
}


// ======================================================================

//
// AudioSourceProxy::AudioSourceProxy
//
// Class constructor.
//

AudioSourceProxy::AudioSourceProxy(MediaObject* pMO) : Traversable(pMO)
{
	m_registeredWithRenderer = FALSE;
	m_skippedHeader = FALSE;
	m_partialStoreAllocatedSize = 0;
	m_partialStore = NULL;
	m_partialLen = 0;
	m_isFinished = FALSE;
	m_firstTimeAtStartTime = TRUE;
	m_firstTimeAtStopTime  = TRUE;
	m_timesStarted = 0;
} /* AudioSourceProxy::AudioSourceProxy */


// ======================================================================

//
// AudioSourceProxy::~AudioSourceProxy
//
// Class destructor.
//

AudioSourceProxy::~AudioSourceProxy()
{
	
//Added because sometimes the denstructor is called before stop time
// has been reached.

	if(!m_isFinished)
		if (GetAudioSource()->GetStream() != NULL)
			{
				GetAudioSource()->CloseStream();
			}

	if (m_registeredWithRenderer)
	{
		GetAudioRenderer()->RemoveSourceNode(this);
	 
	}

	if (m_partialStore)
		delete m_partialStore;
} /* AudioSourceProxy::~AudioSourceProxy */




// ======================================================================

//
// AudioSourceProxy::PreRender
//
// The first time that this is called, the output of the decoder
// will be read to determine the format of the audio, and then
// register this node with the audio renderer.  This method also
// supports the startTime field.
//

void AudioSourceProxy::PreRender(Effects & /*effects*/)
{
	// This gets the time since the start of the scene
	// (convert the time to seconds, as used by BIFS)
	double currentTime = (double)GetPresenter()->GetTime() / 1000;

	// Get the current values of the start and stop time from the
	// AudioSource node
	double startTime = (double)GetAudioSource()->startTime;
	double stopTime  = (double)GetAudioSource()->stopTime;

	// Only proceed once a stream has been attached
	if (GetStream() == NULL)
		return;

	// Determine if the stop time has been reached
	if ((currentTime >= stopTime) && (stopTime >= startTime))
	{
		// If this is the first time, and the stream is currently
		// not active, then close the stream and remove this node
		// from the audio renderer.
		if ((m_firstTimeAtStopTime) && (m_registeredWithRenderer))
		{
			TRACE_PRINT1("AudioSourceProxy: Stopping at %lf\n", currentTime);
			m_firstTimeAtStopTime = FALSE;
			m_firstTimeAtStartTime = TRUE;

			if (GetAudioSource()->GetStream() != NULL)
			{
			//	GetAudioSource()->GetStream()->Terminate();
				GetAudioSource()->CloseStream();
			}
			m_isFinished = TRUE;

			if (m_registeredWithRenderer)
			{
				TRACE_PRINT("AudioSourceProxy: removing from audio renderer\n");
				GetAudioRenderer()->RemoveSourceNode(this);
				m_registeredWithRenderer = FALSE;
			}
		}
	}

	// Only proceed if the start time has been reached
	if ((currentTime < startTime) || (stopTime > startTime))
		return;

	// If this is the first time that the start time has been reached
	if (m_firstTimeAtStartTime)
	{
		m_firstTimeAtStartTime = FALSE;
		m_firstTimeAtStopTime = TRUE;
		m_isFinished = FALSE;
		m_bIsFirstFrame = true;
		m_timesStarted++;

		// If this is not the first time that this has been
		// started, we need to reconnect this stream to the demux
		AudioSource* pAS = GetAudioSource();

	
		// Start the stream flowing.
		pAS->OpenStream();
		GetPresenter()->GetExecutive()->RequestChannels();     
		// Read the output of the decoder to get the audio format
		int dataSize;
		DWORD rdwTime;
	    BOOL bEOU;

		
	
		LPVOID 	dataPtr= GetStream()->FetchNext(dataSize, rdwTime, bEOU, FALSE);
        

			

		// If the data couldn't be obtained, then don't proceed
		if ((dataPtr == NULL) || (dataPtr == END_OF_STREAM) || (dataSize == 0))
			return;

		
		Decoder * pIstDec =((StreamConsumer*)GetObject())->GetDecoder(); 
		
		// The audio format is fixed or passed by the following structure
				
		DecoderAudioFormat* pFmt = (DecoderAudioFormat*)dataPtr;
		
		DWORD nSamplesPerSec, nBitsPerSample, nNumChannels;
	
	 	if(pIstDec->GetDataOffset(NULL)==0)
		{
			nSamplesPerSec=pIstDec->GetAttributes(AUDIO_FREQUENCY,(LPBYTE)dataPtr);

			nBitsPerSample=pIstDec->GetAttributes(BITS_PER_SAMPLE,(LPBYTE)dataPtr);

			nNumChannels=pIstDec->GetAttributes(NUM_CHANNELS,(LPBYTE)dataPtr);
		}
		else
		{
		
			nSamplesPerSec=pFmt->samplesPerSecond;

			nBitsPerSample=pFmt->bitsPerSample;

			nNumChannels=pFmt->numChannels;
		}

		// Record when we started playing (in msec)
		m_startUpDelay =  1000 * startTime;
//		TRACE_PRINT1("AudioSourceProxy: Starting at %d\n", m_startUpDelay);

		// Tell the audio renderer about this node
		m_registeredWithRenderer = TRUE;
		GetAudioRenderer()->AddSourceNode(this, startTime,
										  nSamplesPerSec,
										  nBitsPerSample,
										  nNumChannels);
		// Release the media stream buffer (but don't discard)
		GetStream()->Release(FALSE);
	}

} /* AudioSourceProxy::PreRender */


// ============================================================================

//
// AudioSourceProxy::DataUsed
//
// This function is used by the audio renderer to indicate that it
// has finished with some data given to it by GetData().  The
// parameters given to this function will be the same as those
// returned by GetData().
//

void AudioSourceProxy::DataUsed(LPBYTE /*pData*/, DWORD /*nBytesGiven*/)
{
	if (!m_isFinished)
		GetStream()->Release();
} /* AudioSourceProxy::DataUsed */

// ============================================================================

void AudioSourceProxy::GetData(DWORD nBytesRequired, LPBYTE& rpData,
							   DWORD& rnBytesGiven,DWORD& timeStamp)
{
	// If the stream has already been closed down, then return zero bytes.
	
	if ((GetStream() == NULL)||m_isFinished)
	{
		rpData = NULL;
		rnBytesGiven = 0;
		return;
	}
	//TobeCorrected;
	timeStamp=0;

	int dataSize;
	DWORD rdwTime;
	int  Status;
	LPVOID dataPtr;
    BOOL bEOU;
	// If there is data left over from a previous call, use it
	// instead of reading from the stream
	if (m_partialLen > nBytesRequired)
	{
		rpData = NULL;
		rnBytesGiven = 0;
		return;
	}
	else if (m_partialLen > 0)
	{
		dataSize		 = m_partialLen;
		m_partialLen	 = 0;
		rdwTime			 = m_partialTime;
		Status  		 = 0;
		dataPtr			 = m_partialStore;
	}
	else
	{
		// Read the output of the decoder to get the audio format

		/*-------------------------------------------------------------
		Zvi: first frame shall be fetched with Fetch()
		-------------------------------------------------------------*/
		if (m_bIsFirstFrame)
		{	dataPtr = GetStream () -> Fetch (
										dataSize,
										rdwTime,
										bEOU,
										Status,
										FALSE,
										FALSE,
										0,
										FALSE
			);
			if (dataPtr != NULL && rdwTime != 0)
				m_bIsFirstFrame	= false;
		}
		else
			dataPtr = GetStream () -> FetchNext (dataSize, rdwTime, bEOU, FALSE);
		
		if (dataPtr) {
			TRACE_PRINT3("AudioSourceProxy: fetching for time %ld at %ld  StrClock %ld \n", rdwTime,
				 GetPresenter()->GetTime(),GetStream()->GetClock());
		}
		else
			TRACE_PRINT("AudioSourceProxy: No more CUs available\n");
	}
	
	// If end of stream has been reached, close the stream and return
	// zero bytes
	if (dataPtr == END_OF_STREAM)
	{
		TRACE_PRINT("AudioSourceProxy::GetData(): End of stream detected.\n");

		GetAudioSource()->CloseStream();
		rpData = NULL;
		rnBytesGiven = 0;
		m_isFinished = TRUE;
	}

	// If there is no data, or the data returned has already been played,
	// return zero bytes
	else if (dataPtr == NULL)
	{
		rpData = NULL;
		rnBytesGiven = 0;
	}

	// If data block returned has 0 size, return zero bytes
	else if (dataSize == 0)
	{
		GetStream()->Release();
		rpData = NULL;
		rnBytesGiven = 0;
	}
	else
	{
		// Skip the header data if this is the first time data has been
		// read from this buffer by this function
		timeStamp=rdwTime;	
		Decoder * pIstDec =((StreamConsumer*)GetObject())->GetDecoder();
		
		if (!m_skippedHeader&&pIstDec->GetDataOffset(NULL)!=0)
		{
			m_skippedHeader = TRUE;
			rpData = (LPBYTE)dataPtr + sizeof(DecoderAudioFormat);
			rnBytesGiven = dataSize - sizeof(DecoderAudioFormat);
		}
		else
		{
			rpData = (LPBYTE)dataPtr;
			rnBytesGiven = dataSize;
		}

	}

	// If the number of bytes requested is less than the number read
	// from the media stream, don't return data, but store it for the
	// next call.
	if (rnBytesGiven > nBytesRequired)
	{
		m_partialLen = rnBytesGiven;
		m_partialTime = rdwTime;

		// Make the partial store large enough
		if (m_partialStoreAllocatedSize < m_partialLen)
		{
			if (m_partialStore)
				delete m_partialStore;
			m_partialStore = new unsigned char [m_partialLen];
			m_partialStoreAllocatedSize = m_partialLen;
		}

		// Copy over the remaining samples
		memcpy(m_partialStore, rpData, m_partialLen);
		rnBytesGiven = 0;
//		m_needsReleasing = FALSE;
	}
	else
		m_partialLen = 0;

} /* AudioSourceProxy::GetData */

float AudioSourceProxy::GetIntensity() {

if (GetSound2DProxy())
return ((Sound2D*)GetSound2DProxy()->GetObject())->intensity;
else return 1;
//GetSoundProxy() ? return pSound->->GetObject()->intesity : return 1;

}