///////////////////////////////////////////////////////////////////////////
//                                                                       //
// Copyright (c) 1997 ISO/IEC All Rights Reserved                        //
//                                                                       //
// This software module was developed by                                 //
//                                                                       //
// CSELT (Centro Studi e Laboratori Telecomunicazioni S.p.A)             //
//                                                                       //
// in the course of development of the MPEG-4 standard. This software    //
// module is an implementation of a part of one or more MPEG-4 tools as  //
// specified by the MPEG-4 standard.                                     //
//                                                                       //
// The copyright of this software belongs to ISO/IEC. ISO/IEC gives use  //
// of the MPEG-4 standard free license to use this  software module or   //
// modifications thereof for hardware or software products claiming      //
// conformance to the MPEG-4 standard.                                   //
//                                                                       //
// Those intending to use this software module in hardware or software   //
// products are advised that use may infringe existing  patents. The     //
// original developers of this software module and their companies, the  //
// subsequent editors and their companies, and ISO/IEC have no liability //
// and ISO/IEC have no liability for use of this software module or      //
// modification thereof in an implementation.                            //
//                                                                       //
///////////////////////////////////////////////////////////////////////////
//
// File: CopyYUVA.cpp
//
// Author: Matthew Leditschke, Gianluca Di Cagno  gianluca.dicagno@cselt.it
//
// Description:
//   The implementation of the functions which copy a YUV image
//   to the offscreen RGB surface in the visual renderer.
//
// Created: April, 1997
//
// Modified:
//
///////////////////////////////////////////////////////////////////////////

#define  YUVRGB_CSELT 
#undef  YUVRGB_LEP

#include "defs.h"
#include <assert.h>
#include "VisRend.h"
#include "Copy.h"
#include "trace.h"

// ===============================================================================

unsigned char clip_lut[1256] = {
0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
0x00, 0x00, 0x00, 0x00, 0x00, 0x01, 0x02, 0x03,
0x04, 0x05, 0x06, 0x07, 0x08, 0x09, 0x0a, 0x0b,
0x0c, 0x0d, 0x0e, 0x0f, 0x10, 0x11, 0x12, 0x13,
0x14, 0x15, 0x16, 0x17, 0x18, 0x19, 0x1a, 0x1b,
0x1c, 0x1d, 0x1e, 0x1f, 0x20, 0x21, 0x22, 0x23,
0x24, 0x25, 0x26, 0x27, 0x28, 0x29, 0x2a, 0x2b,
0x2c, 0x2d, 0x2e, 0x2f, 0x30, 0x31, 0x32, 0x33,
0x34, 0x35, 0x36, 0x37, 0x38, 0x39, 0x3a, 0x3b,
0x3c, 0x3d, 0x3e, 0x3f, 0x40, 0x41, 0x42, 0x43,
0x44, 0x45, 0x46, 0x47, 0x48, 0x49, 0x4a, 0x4b,
0x4c, 0x4d, 0x4e, 0x4f, 0x50, 0x51, 0x52, 0x53,
0x54, 0x55, 0x56, 0x57, 0x58, 0x59, 0x5a, 0x5b,
0x5c, 0x5d, 0x5e, 0x5f, 0x60, 0x61, 0x62, 0x63,
0x64, 0x65, 0x66, 0x67, 0x68, 0x69, 0x6a, 0x6b,
0x6c, 0x6d, 0x6e, 0x6f, 0x70, 0x71, 0x72, 0x73,
0x74, 0x75, 0x76, 0x77, 0x78, 0x79, 0x7a, 0x7b,
0x7c, 0x7d, 0x7e, 0x7f, 0x80, 0x81, 0x82, 0x83,
0x84, 0x85, 0x86, 0x87, 0x88, 0x89, 0x8a, 0x8b,
0x8c, 0x8d, 0x8e, 0x8f, 0x90, 0x91, 0x92, 0x93,
0x94, 0x95, 0x96, 0x97, 0x98, 0x99, 0x9a, 0x9b,
0x9c, 0x9d, 0x9e, 0x9f, 0xa0, 0xa1, 0xa2, 0xa3,
0xa4, 0xa5, 0xa6, 0xa7, 0xa8, 0xa9, 0xaa, 0xab,
0xac, 0xad, 0xae, 0xaf, 0xb0, 0xb1, 0xb2, 0xb3,
0xb4, 0xb5, 0xb6, 0xb7, 0xb8, 0xb9, 0xba, 0xbb,
0xbc, 0xbd, 0xbe, 0xbf, 0xc0, 0xc1, 0xc2, 0xc3,
0xc4, 0xc5, 0xc6, 0xc7, 0xc8, 0xc9, 0xca, 0xcb,
0xcc, 0xcd, 0xce, 0xcf, 0xd0, 0xd1, 0xd2, 0xd3,
0xd4, 0xd5, 0xd6, 0xd7, 0xd8, 0xd9, 0xda, 0xdb,
0xdc, 0xdd, 0xde, 0xdf, 0xe0, 0xe1, 0xe2, 0xe3,
0xe4, 0xe5, 0xe6, 0xe7, 0xe8, 0xe9, 0xea, 0xeb,
0xec, 0xed, 0xee, 0xef, 0xf0, 0xf1, 0xf2, 0xf3,
0xf4, 0xf5, 0xf6, 0xf7, 0xf8, 0xf9, 0xfa, 0xfb,
0xfc, 0xfd, 0xfe, 0xff, 0xff, 0xff, 0xff, 0xff,
0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff,
0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff,
0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff,
0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff,
0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff,
0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff,
0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff,
0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff,
0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff,
0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff,
0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff,
0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff,
0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff,
0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff,
0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff,
0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff,
0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff,
0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff,
0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff,
0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff,
0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff,
0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff,
0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff,
0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff,
0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff,
0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff,
0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff,
0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff,
0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff,
0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff,
0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff,
0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff,
0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff,
0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff,
0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff,
0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff,
0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff,
0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff,
0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff,
0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff,
0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff,
0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff,
0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff,
0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff,
0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff,
0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff,
0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff,
0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff,
0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff,
0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff,
0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff,
0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff,
0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff,
0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff,
0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff,
0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff,
0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff,
0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff,
0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff,
0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff,
0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff,
0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff};

#define	clip(x)	*(&clip_lut[500] + (x))	/* clip using clipping table */

/*	write BGR instead of RGB if picture is viewed on a PC	*/

// this is a PC!!! little endian
#define LITTLE_ENDIAN

#ifdef	LITTLE_ENDIAN
#define	PERMUTE_RGB	TRUE
#else
#define	PERMUTE_RGB	FALSE
#endif
#define	R_MATRIX	0x250033c9
#define	G_MATRIX	0x25f4e622
#define	B_MATRIX	0x254000bb

#define dematrix \
R = clip(((Y * 37 + V * 51 ) >> 5) - 223); \
G = clip(((Y * 37 - U * 13 - V * 26 ) >> 5) + 135); \
B = clip(((Y * 37 + U * 65) >> 5) - 277);

#define	PACK_RGB16(a) \
(((R >> 3) << (10 + (a))) | ((G >> 3) << (5 + (a))) | ((B >> 3) << (a)))

#if	(PERMUTE_RGB == TRUE)
#define	PACK_RGB24 {*rgb_ptr++ = B; *rgb_ptr++ = G; *rgb_ptr++ = R;}
#else
#define	PACK_RGB24 {*rgb_ptr++ = R; *rgb_ptr++ = G; *rgb_ptr++ = B;}
#endif

#define	PACK_RGB32 \
((0xff << 24) | (R << 16) | (G << 8) | B)

// ===============================================================================

//
// SetUpSrcYUVpointers
//
// Set up the source pointers for a YUV image.
//

static void SetUpSrcYUVpointers(SrcYUVAImage const& srcImage,
								CopyParameters const& cpParams,
								unsigned int srcY,
								unsigned char const*& pSrcY,
								unsigned char const*& pSrcU,
								unsigned char const*& pSrcV,
								unsigned char const*& pSrcA,
								unsigned char const*& pEndRowSrcY)
{
	// Ensure that the source image has valid pointers
	assert(srcImage.pY != NULL);
	assert(srcImage.pU != NULL);
	assert(srcImage.pV != NULL);

	// Get pointers to the start of this line in the source frame
	//  - source frame is YUV 4:2:0
	pSrcY = srcImage.pY + srcY * srcImage.pitch + cpParams.srcStartX;
	pSrcU = srcImage.pU
			+ (unsigned int)(srcY / 2) * (unsigned int)(srcImage.pitch / 2)
			+ (unsigned int)(cpParams.srcStartX / 2);
	pSrcV = srcImage.pV
			+ (unsigned int)(srcY / 2) * (unsigned int)(srcImage.pitch / 2)
			+ (unsigned int)(cpParams.srcStartX / 2);

	// Get a pointer to the start of line in the alpha mask, if required
	if (srcImage.pA != NULL)
		pSrcA = srcImage.pA + srcY * srcImage.pitch + cpParams.srcStartX;
	else
		pSrcA = NULL;

	// Get a pointer to the end of the srcY row - terminate for loop
	pEndRowSrcY = srcImage.pY + srcY * srcImage.pitch + cpParams.copyWidth
			+ cpParams.srcStartX;
} /* SetUpSrcYUVpointers */


// =============================================================================

//
// ConvertYUVtripleToRGB:
//
// Convert the given YUV triple to RGB.  The YUV values are assumed to all lie
// in the range 0 .. 255 and the RGB values will be returned in this range as
// well.
//
// The following conversion matrix is used:
//		R = Y + 1.40200 * U
//		G = Y - 0.71414 * U - 0.34414 * V
//		B = Y + 1.77200 * V
//
// For the purpose of speed, the arithmetic is all done using integers, with
// values right shifted by 16 bits.
//
// For some reason beyond my comprehension, this function is not compiled inline,
// even when the \Ob2 compiler option is given.  Thus, a #define style macro
// is used instead. (about 20% faster).
//

#define ConvertYUVtripleToRGB(Y, U, V, R, G, B) \
	/* U and V must be adjusted to lie in the range -128 .. 127 */ \
	int tempU = U - 128; \
	int tempV = V - 128; \
\
	/* Convert YUV to RGB */ \
	int tempY = Y << 16; \
	R = tempY +  91881 * tempV; \
	G = tempY -  46802 * tempV - 22554 * tempU; \
	B = tempY + 116130 * tempU; \
\
	/* Scale down the values and clamp values to the range 0 .. 255 */ \
	if (R < 0x10000) R = 0; else if (R > 0xff0000) R = 255; else R >>= 16; \
	if (G < 0x10000) G = 0; else if (G > 0xff0000) G = 255; else G >>= 16; \
	if (B < 0x10000) B = 0; else if (B > 0xff0000) B = 255; else B >>= 16


inline void XX_ConvertYUVtripleToRGB (int Y, int U, int V, int& R, int& G, int& B)
{
	// U and V must be adjusted to lie in the range -128 .. 127
	U -= 128;
	V -= 128;

	// Convert YUV to RGB
	Y <<= 16;
	R = Y +  91881 * V;
	G = Y -  46802 * V - 22554 * U;
	B = Y + 116130 * U;

	// Scale down the values and clamp values to the range 0 .. 255
	if (R < 0x10000) R = 0; else if (R > 0xff0000) R = 255; else R >>= 16;
	if (G < 0x10000) G = 0; else if (G > 0xff0000) G = 255; else G >>= 16;
	if (B < 0x10000) B = 0; else if (B > 0xff0000) B = 255; else B >>= 16;
} /* ConvertYUVtripleToRGB */


// ===============================================================================

//
// CopyYUVToRGB555
//
// Copy the given Y, U and V bands to the given surface, in RGB555 format, using
// the alpha mask to determine which pixels are copied.  If the alpha mask is NULL,
// then the entire image is used.
//
// The top-left corner of the given image is copied to the point given by
// (top,left), where (0,0) is the top-left corner of the given surface.
//

void CopyYUVToRGB555(SrcYUVAImage const& srcImage,
					 SurfaceDesc const& surfaceDesc)
{
	CopyParameters cpParams;
	if (SetUpCopyParameters(cpParams, srcImage, surfaceDesc))
		return;

	// For every row of pixels...
	for (unsigned int srcY = cpParams.srcStartY; srcY < cpParams.srcStartY + cpParams.copyHeight; srcY++)
	{
		// Get pointers to the start of this row in each source band,
		// and another for the end of the Y row.
		unsigned char const *pSrcY, *pSrcU, *pSrcV, *pSrcA, *pEndRowSrcY;
		SetUpSrcYUVpointers(srcImage, cpParams, srcY, pSrcY, pSrcU, pSrcV,
							pSrcA, pEndRowSrcY);

		// Get a pointer to the start of this line in the destination frame
		unsigned short* pDest = (unsigned short*)((unsigned char*)((unsigned char*)surfaceDesc.lpSurface + (cpParams.destStartY + srcY - cpParams.srcStartY) * surfaceDesc.lPitch
								+ 2 * cpParams.destStartX));

		// Copy across the line of data, performing alpha masking if required
		unsigned int pixelCount = cpParams.srcStartX;
		while (pSrcY != pEndRowSrcY)
		{
			// Get the alpha mask value
			int alpha = (pSrcA == NULL) ? 255 : *pSrcA++;

			// This means that I can divide by 256 not 255
			if (alpha > 0)
				alpha++;	

			// Convert this YUV triple to RGB
			int red, green, blue;
			ConvertYUVtripleToRGB (*pSrcY++, *pSrcU, *pSrcV, red, green, blue);

			if (alpha < 256)
			{
				// Get the RGB values underneath this one
				int oldRed   = (*pDest & 0x7c00) >> 7;
				int oldGreen = (*pDest & 0x03e0) >> 2;
				int oldBlue  = (*pDest & 0x001f) << 3;

				// Apply the alpha mask factor
				ApplyAlphaMaskToRGB(red, green, blue, alpha, oldRed, oldGreen, oldBlue);
			}

			// Combine the values, 5 bits red, 5 bits green, 5 bits blue
			*pDest++ = MAKE_RGB555_PIXEL(red, green, blue);

			// Only advance the U and V pointers every second time
			if ((pixelCount % 2) == 1)
			{
				pSrcU++;
				pSrcV++;
			}
			pixelCount++;
		}
	}
} /* CopyYUVToRGB555 */


// ===============================================================================

//
// CopyYUVToRGB565
//
// Copy the given Y, U and V bands to the given surface, in RGB565 format, using
// the alpha mask to determine which pixels are copied.  If the alpha mask is NULL,
// then the entire image is used.
//
// The top-left corner of the given image is copied to the point given by
// (top,left), where (0,0) is the top-left corner of the given surface.
//

#ifdef YUVRGB_CSELT

void CopyYUVToRGB565(SrcYUVAImage const& srcImage,
					 SurfaceDesc const& surfaceDesc)
{
	CopyParameters cpParams;
	if (SetUpCopyParameters(cpParams, srcImage, surfaceDesc))
		return;

	// For every row of pixels...
	for (unsigned int srcY = cpParams.srcStartY; srcY < cpParams.srcStartY + cpParams.copyHeight; srcY++)
	{
		// Get pointers to the start of this row in each source band,
		// and another for the end of the Y row.
		unsigned char const *pSrcY, *pSrcU, *pSrcV, *pSrcA, *pEndRowSrcY;
		SetUpSrcYUVpointers(srcImage, cpParams, srcY, pSrcY, pSrcU, pSrcV,
							pSrcA, pEndRowSrcY);
        
		// Get a pointer to the start of this line in the destination frame
		unsigned short* pDest = (unsigned short*)((unsigned char*)((unsigned char*)surfaceDesc.lpSurface + (cpParams.destStartY + srcY - cpParams.srcStartY) * surfaceDesc.lPitch
								+ 2 * cpParams.destStartX));

		// Copy across the line of data, performing alpha masking if required
		unsigned int pixelCount = cpParams.srcStartX;
		while (pSrcY != pEndRowSrcY)
		{
			// Get the alpha mask value
			int alpha = (pSrcA == NULL) ? 255 : (*pSrcA++);

			// This means that I can divide by 256 not 255
			if (alpha > 0)
				alpha++;	

			// Convert this YUV triple to RGB
			int red, green, blue;
			//TRACE_PRINT3("Y %d, U %d, i: %d\n",*pSrcY,*pSrcU,srcY);
			ConvertYUVtripleToRGB(*pSrcY++, *pSrcU, *pSrcV, red, green, blue);

			if (alpha < 256)
			{
				// Get the RGB values underneath this one
				int oldRed   = (*pDest & 0xf800) >> 8;
				int oldGreen = (*pDest & 0x07e0) >> 3;
				int oldBlue  = (*pDest & 0x001f) << 3;

				// Apply the alpha mask factor
				ApplyAlphaMaskToRGB(red, green, blue, alpha, oldRed, oldGreen, oldBlue);
			}

			// Combine the values, 5 bits red, 6 bits green, 5 bits blue
			*pDest++ = MAKE_RGB565_PIXEL(red, green, blue);

			// Only advance the U and V pointers every second time
			if ((pixelCount % 2) == 1)
			{
				pSrcU++;
				pSrcV++;
			}
			pixelCount++;
		}
	}
} /* CopyYUVToRGB565 */

#endif // YUVRGB_CSELT

/***************************************************************************
Proto:
void			YuvToRgb(Vscene *s, UInt8 *buf, UInt8 pix_size);
Author:		Jean Gobert	LEP			
Purpose:		dematrix from YUV to RGB
Abstract:	does only crude approximation, no filtering. pix_size
				determines the output format and can take values between
				2 and 4. If pixel size is 2, the output is RGB 5:5:5.
				If pixel size is 3, the output is RGB or BGR on 24 bits. 
				If pixel size is 4 the output is ARGB, where A is an 
				alpha value set to 255. 
In:			s, pix_size
InOut:		none
Out:			buf updated
Modified:    Stefano Battista
***************************************************************************/

#ifdef YUVRGB_LEP

/**
void			YuvToRgb(Vscene *s, UInt8 *buf, UInt8 pix_size)
**/

void CopyYUVToRGB565(SrcYUVAImage const& srcImage,
					   SurfaceDesc const& surfaceDesc)
{
	unsigned int	x, y;
	unsigned char * y_ptr, * u_ptr, * v_ptr, * a_ptr;
	unsigned short * rgb_ptr_start, * rgb_ptr;
	long * argb_ptr;
	int	Y, U, V, R, G, B;
//	unsigned int u_l_rgb;
	int	rgb_line_size = surfaceDesc.lPitch / 2;

	CopyParameters cpParams;
	if (SetUpCopyParameters(cpParams, srcImage, surfaceDesc))
		return;

	// destination (DDraw surface)
	rgb_ptr_start = (unsigned short *) surfaceDesc.lpSurface +
        cpParams.destStartY * surfaceDesc.lPitch +
        2 * cpParams.destStartX;

	unsigned int uiOffsetY, uiOffsetUV, uiOffsetRGB;
	for	(y = cpParams.srcStartY; 
	    y < cpParams.srcStartY + cpParams.copyHeight; 
		y++)
	{
		uiOffsetY = y * srcImage.pitch + cpParams.srcStartX;
		uiOffsetUV = ((y / 2) * (srcImage.pitch / 2)) + (cpParams.srcStartX / 2);
		uiOffsetRGB = (y * rgb_line_size);
		y_ptr = srcImage.pY + uiOffsetY;
		u_ptr = srcImage.pU + uiOffsetUV;
		v_ptr = srcImage.pV + uiOffsetUV;
		rgb_ptr = rgb_ptr_start + uiOffsetRGB;
		if (srcImage.pA != NULL)
    		a_ptr = srcImage.pA + uiOffsetY;
		else
			a_ptr = NULL;
	    for	(x = cpParams.srcStartX; 
		    x < cpParams.srcStartX + cpParams.copyWidth; 
			x += 2)
		{
			Y = *y_ptr++;
			U = *u_ptr++;
			V = *v_ptr++;
			dematrix
			*rgb_ptr++ = MAKE_RGB565_PIXEL(R, G, B);
			Y = *y_ptr++;
			dematrix
			*rgb_ptr++ = MAKE_RGB565_PIXEL(R, G, B);
		}
	}
}

#endif // YUVRGB_LEP

// ===============================================================================

//
// CopyYUVToRGB24bpp
//
// Copy the given Y, U and V bands to the given surface, in RGB24bpp format, using
// the alpha mask to determine which pixels are copied.  If the alpha mask is NULL,
// then the entire image is used.
//
// The top-left corner of the given image is copied to the point given by
// (top,left), where (0,0) is the top-left corner of the given surface.
//

#ifdef YUVRGB_CSELT

void CopyYUVToRGB24bpp(SrcYUVAImage const& srcImage,
					   SurfaceDesc const& surfaceDesc)
{
	CopyParameters cpParams;
	if (SetUpCopyParameters(cpParams, srcImage, surfaceDesc))
		return;

	// For every row of pixels...
	for (unsigned int srcY = cpParams.srcStartY; srcY < cpParams.srcStartY + cpParams.copyHeight; srcY++)
	{
		// Get pointers to the start of this row in each source band,
		// and another for the end of the Y row.
		unsigned char const *pSrcY, *pSrcU, *pSrcV, *pSrcA, *pEndRowSrcY;
		SetUpSrcYUVpointers(srcImage, cpParams, srcY, pSrcY, pSrcU, pSrcV,
							pSrcA, pEndRowSrcY);

		// Get a pointer to the start of this line in the destination frame
		unsigned char* pDest = (unsigned char*) surfaceDesc.lpSurface
								+ (cpParams.destStartY + srcY - cpParams.srcStartY) * surfaceDesc.lPitch
								+ 3 * cpParams.destStartX;

		// Copy across the line of data, performing alpha masking if required
		unsigned int pixelCount = cpParams.srcStartX;
		while (pSrcY != pEndRowSrcY)
		{
			// Get the alpha mask value
			int alpha = (pSrcA == NULL) ? 255 : *pSrcA++;

			// This means that I can divide by 256 not 255
			if (alpha > 0)
				alpha++;	

			// Convert this YUV triple to RGB
			int red, green, blue;
			ConvertYUVtripleToRGB (*pSrcY++, *pSrcU, *pSrcV, red, green, blue);

			if (alpha < 256)
			{
				// Get the RGB values underneath this one
				int oldRed   = pDest[2];
				int oldGreen = pDest[1];
				int oldBlue  = pDest[0];

				// Apply the alpha mask factor
				ApplyAlphaMaskToRGB(red, green, blue, alpha, oldRed, oldGreen, oldBlue);
			}

			// Store the values in the output frame store
			*pDest++ = (unsigned char) blue;
			*pDest++ = (unsigned char) green;
			*pDest++ = (unsigned char) red;

			// Only advance the U and V pointers every second time
			if ((pixelCount % 2) == 1)
			{
				pSrcU++;
				pSrcV++;
			}
			pixelCount++;
		}
	}
} /* CopyYUVToRGB24bpp */

#endif // YUVRGB_CSELT

/***************************************************************************
Proto:
void			YuvToRgb(Vscene *s, UInt8 *buf, UInt8 pix_size);
Author:		Jean Gobert	LEP			
Purpose:		dematrix from YUV to RGB
Abstract:	does only crude approximation, no filtering. pix_size
				determines the output format and can take values between
				2 and 4. If pixel size is 2, the output is RGB 5:5:5.
				If pixel size is 3, the output is RGB or BGR on 24 bits. 
				If pixel size is 4 the output is ARGB, where A is an 
				alpha value set to 255. 
In:			s, pix_size
InOut:		none
Out:			buf updated
Modified:    Stefano Battista
***************************************************************************/

#ifdef YUVRGB_LEP

/**
void			YuvToRgb(Vscene *s, UInt8 *buf, UInt8 pix_size)
**/

void CopyYUVToRGB24bpp(SrcYUVAImage const& srcImage,
					   SurfaceDesc const& surfaceDesc)
{
	unsigned int	x, y;
	unsigned char * y_ptr, * u_ptr, * v_ptr, * a_ptr;
	unsigned char * rgb_ptr_start, * rgb_ptr;
	long * argb_ptr;
	int	Y, U, V, R, G, B;
//	unsigned int u_l_rgb;
	int	rgb_line_size = surfaceDesc.lPitch;

	CopyParameters cpParams;
	if (SetUpCopyParameters(cpParams, srcImage, surfaceDesc))
		return;

	// destination (DDraw surface)
	rgb_ptr_start = (unsigned char *) surfaceDesc.lpSurface +
        cpParams.destStartY * surfaceDesc.lPitch +
        3 * cpParams.destStartX;

	unsigned int uiOffsetY, uiOffsetUV, uiOffsetRGB;
	for	(y = cpParams.srcStartY; 
	    y < cpParams.srcStartY + cpParams.copyHeight; 
		y++)
	{
		uiOffsetY = y * srcImage.pitch + cpParams.srcStartX;
		uiOffsetUV = ((y / 2) * (srcImage.pitch / 2)) + (cpParams.srcStartX / 2);
		uiOffsetRGB = (y * rgb_line_size);
		y_ptr = srcImage.pY + uiOffsetY;
		u_ptr = srcImage.pU + uiOffsetUV;
		v_ptr = srcImage.pV + uiOffsetUV;
		rgb_ptr = rgb_ptr_start + uiOffsetRGB;
		if (srcImage.pA != NULL)
    		a_ptr = srcImage.pA + uiOffsetY;
		else
			a_ptr = NULL;
	    for	(x = cpParams.srcStartX; 
		    x < cpParams.srcStartX + cpParams.copyWidth; 
			x += 2)
		{
			Y = *y_ptr++;
			U = *u_ptr++;
			V = *v_ptr++;
			dematrix
			PACK_RGB24;
			Y = *y_ptr++;
			dematrix
			PACK_RGB24;
		}
	}
}

#endif // YUVRGB_LEP

// ===============================================================================

//
// CopyYUVToRGB32bpp
//
// Copy the given Y, U and V bands to the given surface, in RGB32bpp format, using
// the alpha mask to determine which pixels are copied.  If the alpha mask is NULL,
// then the entire image is used.
//
// The top-left corner of the given image is copied to the point given by
// (top,left), where (0,0) is the top-left corner of the given surface.
//

void CopyYUVToRGB32bpp(SrcYUVAImage const& srcImage,
					   SurfaceDesc const& surfaceDesc)
{
	CopyParameters cpParams;
	if (SetUpCopyParameters(cpParams, srcImage, surfaceDesc))
		return;

	// For every row of pixels...
	for (unsigned int srcY = cpParams.srcStartY; srcY < cpParams.srcStartY + cpParams.copyHeight; srcY++)
	{
		// Get pointers to the start of this row in each source band,
		// and another for the end of the Y row.
		unsigned char const *pSrcY, *pSrcU, *pSrcV, *pSrcA, *pEndRowSrcY;
		SetUpSrcYUVpointers(srcImage, cpParams, srcY, pSrcY, pSrcU, pSrcV,
							pSrcA, pEndRowSrcY);

		// Get a pointer to the start of this line in the destination frame
		unsigned long* pDest = (unsigned long*)((unsigned char*)((unsigned char*)surfaceDesc.lpSurface + (cpParams.destStartY + srcY - cpParams.srcStartY) * surfaceDesc.lPitch
								+ 4 * cpParams.destStartX));

		// Copy across the line of data, performing alpha masking if required
		unsigned int pixelCount = cpParams.srcStartX;
		while (pSrcY != pEndRowSrcY)
		{
			// Get the alpha mask value
			int alpha = (pSrcA == NULL) ? 255 : *pSrcA++;

			// This means that I can divide by 256 not 255
			if (alpha > 0)
				alpha++;	

			// Convert this YUV triple to RGB
			int red, green, blue;
			ConvertYUVtripleToRGB (*pSrcY++, *pSrcU, *pSrcV, red, green, blue);

			if (alpha < 256)
			{
				// Get the RGB values underneath this one
				int oldRed   = (*pDest & 0xff0000) >> 16;
				int oldGreen = (*pDest & 0xff00) >> 8;
				int oldBlue  = (*pDest & 0xff);

				// Apply the alpha mask factor
				ApplyAlphaMaskToRGB(red, green, blue, alpha, oldRed, oldGreen, oldBlue);
			}

			// Store the values in the output frame store
			*pDest++ = MAKE_RGB32bpp_PIXEL(red, green, blue);

			// Only advance the U and V pointers every second time
			if ((pixelCount % 2) == 1)
			{
				pSrcU++;
				pSrcV++;
			}
			pixelCount++;
		}
	}
} /* CopyYUVToRGB32bpp */


// ========================================================================

//
// CopyYUVToBGR32bpp
//
// Copy the given Y, U and V bands to the given surface, in BGR32bpp format, using
// the alpha mask to determine which pixels are copied.  If the alpha mask is NULL,
// then the entire image is used.
//
// The top-left corner of the given image is copied to the point given by
// (top,left), where (0,0) is the top-left corner of the given surface.
//

void CopyYUVToBGR32bpp(SrcYUVAImage const& srcImage,
					   SurfaceDesc const& surfaceDesc)
{
	CopyParameters cpParams;
	if (SetUpCopyParameters(cpParams, srcImage, surfaceDesc))
		return;

	// For every row of pixels...
	for (unsigned int srcY = cpParams.srcStartY; srcY < cpParams.srcStartY + cpParams.copyHeight; srcY++)
	{
		// Get pointers to the start of this row in each source band,
		// and another for the end of the Y row.
		unsigned char const *pSrcY, *pSrcU, *pSrcV, *pSrcA, *pEndRowSrcY;
		SetUpSrcYUVpointers(srcImage, cpParams, srcY, pSrcY, pSrcU, pSrcV,
							pSrcA, pEndRowSrcY);

		// Get a pointer to the start of this line in the destination frame
		unsigned long* pDest = (unsigned long*)((unsigned char*)((unsigned char*)surfaceDesc.lpSurface + (cpParams.destStartY + srcY - cpParams.srcStartY) * surfaceDesc.lPitch
								+ 4 * cpParams.destStartX));

		// Copy across the line of data, performing alpha masking if required
		unsigned int pixelCount = cpParams.srcStartX;
		while (pSrcY != pEndRowSrcY)
		{
			// Get the alpha mask value
			int alpha = (pSrcA == NULL) ? 255 : *pSrcA++;

			// This means that I can divide by 256 not 255
			if (alpha > 0)
				alpha++;	

			// Convert this YUV triple to RGB
			int red, green, blue;
			ConvertYUVtripleToRGB (*pSrcY++, *pSrcU, *pSrcV, red, green, blue);

			if (alpha < 256)
			{
				// Get the RGB values underneath this one
				int oldRed   = (*pDest & 0xff);
				int oldGreen = (*pDest & 0xff00) >> 8;
				int oldBlue  = (*pDest & 0xff0000) >> 16;

				// Apply the alpha mask factor
				ApplyAlphaMaskToRGB(red, green, blue, alpha, oldRed, oldGreen, oldBlue);
			}

			// Store the values in the output frame store
			*pDest++ = MAKE_BGR32bpp_PIXEL(red, green, blue);

			// Only advance the U and V pointers every second time
			if ((pixelCount % 2) == 1)
			{
				pSrcU++;
				pSrcV++;
			}
			pixelCount++;
		}
	}
} /* CopyYUVToBGR32bpp */


// ========================================================================
