/***********************************************************************
This software module was originally developed by Zvi Lifshitz
(Triton R&D Ltd.) in the course of development of the MPEG-4 Systems
(ISO/IEC 14496-1) standard. This software module is an implementation of
a part of one or more MPEG-4 Systems (ISO/IEC 14496-1) tools as
specified by the MPEG-4 Systems (ISO/IEC 14496-1) standard. ISO/IEC
gives users of the MPEG-4 Systems (ISO/IEC 14496-1) free license to
this software module or modifications thereof for use in hardware or
software products claiming conformance to the MPEG-4 Systems (ISO/IEC
14496-1). Those intending to use this software module in hardware or
software products are advised that its use may infringe existing patents.
The original developer of this software module and his/her company, the
subsequent editors and their companies, and ISO/IEC have no liability
for use of this software module or modifications thereof in an
implementation. Copyright is not released for non MPEG-4 Systems
(ISO/IEC 14496-1) conforming products. Triton R&D Ltd. retains full right
to use the code for his/her own purpose, assign or donate the code to a
third party and to inhibit third parties from using the code for non
MPEG-4 Systems (ISO/IEC 14496-1) conforming products. This copyright
notice must be included in all copies or derivative works.
Copyright  1998.
************************************************************************

For more information or to receive an up to date version of this module,
you may call Zvi Lifshitz at zvil@csi.com.
----------------------------------------------------------------------*/

#if !defined(_BifsEnc_h)	// Sentry, use file only if it's not already included.
#define _BifsEnc_h

/*-------------------------------------------------------------------------
Project Im1Player
INTERNATIONAL ORGANIZATION FOR STANDARDIZATION
ISO/IEC JTC1/SC29/WG11

SUBSYSTEM:  BIFS Encoder
FILE:       BifsEnc.h
AUTHOR:     Zvi Lifshitz (Triton R&D Ltd.)
OVERVIEW:   Implementation of BIFSEncoder, the class that encodes date into
			bits.
-------------------------------------------------------------------------*/
//$Log: BifsEnc.h,v $
//Revision 1.3.2.2  2000/03/10 15:59:31  Alex Cotarmanac'h
//deuxime envoi
//
//Revision 1.2  2000/03/03 18:41:20  Alex Cotarmanac'h
//Version 1 -- Alexandre Cotarmanac_h
//
#include "ztl.h"
#include "NodeFld.h" // MBS: added for SF<> field
#include "SysCoder.h"

class ODStreamEncoder;
class ScriptImp;	// SCRIPT
class Proto;

class BIFSEncoder : public SysCoder
{
	ODStreamEncoder *m_pODEncoder, *m_pOCIEncoder;
	Proto *m_pProto;
	SFTopNode m_scene;

	// MBS : declares current QP node to be applied to fields values
	MediaObject *m_pCurrentQP;
//AC 19991121 : isPredictive. AC 20000227 Quantization Scheme 14 stuff.
private:
	bool m_isPredictive;
	void setPredictive( int flags ) { m_isPredictive = (flags & PREDICTIVE_FLAG)?true:false; }
	// Quantization Scheme 14 stuff.
	bool m_coordActive; // can we use m_lengthCoord ?
	bool m_coordStore;  // do we have to store m_lengthCoord ?
	int  m_lengthCoord;
	bool m_scriptCoding;
	ZMap<int> m_IdToLength; // maps DEF_id to length

public:
	bool isPredictive() { return m_isPredictive; }
	// Quantization Scheme 14 stuff.
	// remember to store number of points
	void setScriptCoding(bool state) {m_scriptCoding = state;}
	// coding a script
	bool isCodingScript(){return m_scriptCoding; }
	void setCoordStore() { m_coordStore=true;}
	// remember to use number of points
	void setCoordActive() { m_coordActive=true; }
	// can I use the recorded number of points ?
	bool isCoordActive() { return m_coordActive; }
	// do I have to store the number of points ?
	bool isCoordStore() { return m_coordStore; }
	// store the number of points
	void setLengthCoord(int l);
	// retrieve the number of points
	int getLengthCoord();
	// reset the flags
	void resetCoord();
	// store in case it's used somewhere else USE CoordName
	void setIdToLength(int kId, int Length);
	// retrieve stored value and set as current length.
	void setCurrentId(int kId);
//AC end

public:
	//ZArray<ScriptImp *> ScriptList;	// SCRIPT  //BF Suppressed
	void Write (SysCoder *pBifs); // MBS added
	BIFSEncoder (
		FILE *fBifs,
		FILE *fList,
        FILE *fLogs, // mp conformance
		LPCSTR szType,
		int flags,
		ODStreamEncoder *pODEncoder = NULL,
		ODStreamEncoder *pOCIEncoder = NULL
	);
	void Encode ();
	void SetProto (Proto *pProto) {m_pProto = pProto;}
	Proto *GetProto () const {return m_pProto;}

	// MBS
	// sets current QP node
	void SetCurrentQP(MediaObject *qp) { m_pCurrentQP=qp; }
	// gets current QP node
	MediaObject *GetCurrentQP() { return m_pCurrentQP; }
	// do we have a QP to be used ?
	BOOL hasLocalQP() { return (m_pCurrentQP ? ((Bool) *((SF<Bool> *) m_pCurrentQP->GetField(0))) : FALSE) ; }
	// end MBS

#ifndef MUX
public:
	void EncodeString (const ZString &string);
	void EncodeAU (DWORD dwTS = 0);
	void EncodeRoutesDescription ();

private:
	BOOL EncodeUpdateCommand ();
	void EncodeScene ();
	void ReplaceRoute ();
	void SceneReplaceCommand ();
	void EncodeRouteDetails ();
	int EncodeNodeID ();
	void EncodeRouteID ();
	void AppendCommand ();
	MediaObject *ScanNode ();
	void InsertToField (BOOL bAppend);
	void ReplaceCommand ();
	void InsertCommand ();
	void UpdateScene ();
	void EncodeRoute ();
	void DeleteCommand ();
#endif		// BIFSENC
};

#endif                                      // _BifsEnc_h sentry.
