/***********************************************************************
This software module was originally developed by Zvi Lifshitz
(Triton R&D Ltd.) and edited by Mauro Rossi (CSELT), Guido Franceschini
(CSELT), George Tselikis (Xbind Inc.) and Michael Stepping (FTK)
in the course of development of the MPEG-4 Systems
(ISO/IEC 14496-1) standard. This software module is an implementation of
a part of one or more MPEG-4 Systems (ISO/IEC 14496-1) tools as
specified by the MPEG-4 Systems (ISO/IEC 14496-1) standard. ISO/IEC
gives users of the MPEG-4 Systems (ISO/IEC 14496-1) free license to
this software module or modifications thereof for use in hardware or
software products claiming conformance to the MPEG-4 Systems (ISO/IEC
14496-1). Those intending to use this software module in hardware or
software products are advised that its use may infringe existing patents.
The original developer of this software module and his/her company, the
subsequent editors and their companies, and ISO/IEC have no liability
for use of this software module or modifications thereof in an
implementation. Copyright is not released for non MPEG-4 Systems
(ISO/IEC 14496-1) conforming products. Triton R&D Ltd. retains full right
to use the code for his/her own purpose, assign or donate the code to a
third party and to inhibit third parties from using the code for non
MPEG-4 Systems (ISO/IEC 14496-1) conforming products. This copyright
notice must be included in all copies or derivative works.
Copyright  1998.
************************************************************************

For more information or to receive an up to date version of this module,
you may call Zvi Lifshitz at zvil@csi.com.
----------------------------------------------------------------------*/

#if !defined(_al_h)	// Sentry, use file only if it's not already included.
#define _al_h

/*-------------------------------------------------------------------------
Project Im1Player
INTERNATIONAL ORGANIZATION FOR STANDARDIZATION
ISO/IEC JTC1/SC29/WG11

SUBSYSTEM:  Core
FILE:       AL.h
AUTHOR:     Zvi Lifshitz (Triton R&D Ltd.)
OVERVIEW:   Definition of ALManager
-------------------------------------------------------------------------*/
#include "DAI.h"
#include "Clock.h"

class Executive;
class MediaStreamImp;
class DataChannel;
class StreamConsumer;
class SLConfigDescriptor;
class ObjectDescriptorImp;
class ES_DescriptorImp;

class ALManager
{
	Executive *m_pExecutive;
	UINT m_nServiceID, m_nParentServiceID;
	ZArray<DataChannel *> m_channels;
	ZString m_url;
	enum tagServiceState
	{
		SS_IDLE,
		SS_CONNECTING,
		SS_CONNECTED,
		SS_UNAVAILABLE
	} m_state;
	ZMutex m_mtxChannels, m_mtxLock;
	ObjectDescriptorImp *m_pOD;
	int m_nInUse;
	ClockNamespace m_clocks;

private:
	void AddChannels ();
	void ChannelsClosed ();

public:
	void Resume () {m_clocks.Resume ();}
	void Pause () {m_clocks.Pause ();}
	ClockReference *GetClock (int nClockID) {return m_clocks.GetClock (nClockID);}
	ClockReference *AttachClock (int nClockID, int nHasOCR)
	{
		return m_clocks.AttachClock (nClockID, nHasOCR);
	}
	void OnUserCommandCnf (const DAIUserCmdParam &outParam);
	void RequestChannelDelete (DataChannel *pChannel);
	void OnServiceAttach (UINT nServiceID, const BYTE* uuData, int uuData_Length, RESPONSE response);
	void OnServiceDetach (REASON reason);
	void OnChannelDeleteCnf (const DAIChannelDeleteParam &cnfParam) ;
	void Lock () {m_mtxLock.P ();}
	void Unlock () {m_mtxLock.V (); --m_nInUse;}
	void InUse () {++m_nInUse;}
	void OnChannelAdd (const DAIChannelAddParam &cnfParam);
	LPCSTR GetURL () const {return m_url;}
	UINT GetServiceID () {return m_nServiceID;}
	void StartChannel (
		UINT nChannelID,
		double dStart,
		double dStop,
		double dSpeed,
		bool bPlay = true
	);
	void StopChannel (UINT nChannelID) {SendCommand ("STOP", nChannelID);}
	void PauseChannel (UINT nChannelID) {SendCommand ("PAUSE", nChannelID);}
	void MuteChannel (UINT nChannelID) {SendCommand ("MUTE", nChannelID);}
	void SendCommand (char *szCommand, UINT nChannelID);
	void DeleteChannel (DataChannel *pChannel);
	void DataSend (LPBYTE pData, int nLength, UINT nChannelID, int nAUnumber);
	~ALManager ();
	void Disconnect ();
	void Connect ();
	ALManager (
		Executive *pExecutive,
		ObjectDescriptorImp *pOD,
		LPCSTR szURL,
		UINT nParentServiceID
	) :
		m_pExecutive (pExecutive),
		m_pOD (pOD),
		m_nServiceID (0xffffffff),
		m_state (SS_IDLE),
		m_url (szURL),
		m_nParentServiceID (nParentServiceID),
		m_nInUse (0)
	{}
	DataChannel *AttachStream (
		ES_DescriptorImp *pESD,
		MediaStreamImp *pStream,
		SLConfigDescriptor *pConfigParams,
		int nExpectedAULength = 256,
		bool bBackChannel = false
	);
	void RequestChannels ();
	Executive *GetExecutive () {return m_pExecutive;}
};

#endif                                      // _al_h sentry.