/***********************************************************************
This software module was originally developed by Zvi Lifshitz
(Triton R&D Ltd.) in the course of development of the MPEG-4 Systems
(ISO/IEC 14496-1) standard. This software module is an implementation of
a part of one or more MPEG-4 Systems (ISO/IEC 14496-1) tools as
specified by the MPEG-4 Systems (ISO/IEC 14496-1) standard. ISO/IEC
gives users of the MPEG-4 Systems (ISO/IEC 14496-1) free license to
this software module or modifications thereof for use in hardware or
software products claiming conformance to the MPEG-4 Systems (ISO/IEC
14496-1). Those intending to use this software module in hardware or
software products are advised that its use may infringe existing patents.
The original developer of this software module and his/her company, the
subsequent editors and their companies, and ISO/IEC have no liability
for use of this software module or modifications thereof in an
implementation. Copyright is not released for non MPEG-4 Systems
(ISO/IEC 14496-1) conforming products. Triton R&D Ltd. retains full right
to use the code for his/her own purpose, assign or donate the code to a
third party and to inhibit third parties from using the code for non
MPEG-4 Systems (ISO/IEC 14496-1) conforming products. This copyright
notice must be included in all copies or derivative works.
Copyright  1997.
************************************************************************

For more information or to receive an up to date version of this module,
you may call Zvi Lifshitz at zvil@csi.com.
----------------------------------------------------------------------*/
//$Log: BifsDec.cpp,v $
//
//Revision 1.2.2.1  2000/03/10 15:59:31  Alex Cotarmanac'h
//deuxime envoi
//
//Revision 1.2  2000/03/03 18:41:21  Alex Cotarmanac'h
//Version 1 -- Alexandre Cotarmanac_h
//
//Modified 2001/05/08 Boris Felts boris.felts@polytechnique.org

/*-------------------------------------------------------------------------
Project Im1Player
INTERNATIONAL ORGANIZATION FOR STANDARDIZATION
ISO/IEC JTC1/SC29/WG11

SUBSYSTEM:  BIFS Decoder
FILE:       BifsDec.cpp
AUTHOR:     Zvi Lifshitz (Triton R&D Ltd.)
OVERVIEW:   Implementation of BIFSDecoder.
-------------------------------------------------------------------------*/
#include "defs.h"
#include <assert.h>
#include <Executiv.h>
#include "BifsDec.h"
#include "NodeFld.h"
#include "Root.h"
#include "Nodes.h"
#include "Route.h"
#include "Except.h"
#include "MemBifs.h"
#include "Proto.h"
#include "InLine.h"
#include "ODP.h"

#define pCoder this

/*-----------------------------------------------------------------
BIFSDecoder constructor
-----------------------------------------------------------------*/
BIFSDecoder :: BIFSDecoder (
	Executive *pExecutive,
	InlineImp *pSource,
	FILE *fList,
	LPCSTR szListTitle,
	int flags,
	int objectTypeIndication,
	bool bExternProto
) :
	SysCoder (fList, szListTitle, flags, objectTypeIndication),
	m_pExecutive (pExecutive),
	m_pSource (pSource),
	m_bExternProtoLib (bExternProto),
	m_nNodeIDbits (10),
	m_nRouteIDbits (10),
	m_nProtoIDbits (10),
	m_bPixelMetric (FALSE),
	m_nPixelWidth (0),
	m_nPixelHeight (0),
	m_bUse3DMeshCoding (false),
	m_bUsePredictiveMFField (false),
#ifdef PREDICTIVE	//AC
	m_buffer(0),
	m_bufferSize(0),
#endif // BF 043101
	m_coordActive(false),
	m_coordStore(false),
	m_lengthCoord(-1),
	m_IdToLength(),
	m_pUICoder (NULL),
//#endif	//AC end  //BF
	m_pScene (pSource ? pSource -> GetScene () : NULL)
{
	// MBS: inits current QP node to null
	m_pCurrentQP=NULL;
	// end MBS
}




#ifdef PREDICTIVE	//AC 19991211
/*-------------------------------------------------------------
GetBit with a (15 bit) buffer so as to rewind when predictive done (AC)
-------------------------------------------------------------*/
int BIFSDecoder::GetBit () {
	if (m_bufferSize==0) return  SysCoder::GetBit();
	m_bufferSize--;
	int nResult= (m_buffer>>(m_bufferSize)) & 1;
	return nResult;
}
/*-------------------------------------------------------------
setIntBuffer					 so as to rewind when predictive done (AC)
Parameters : buffer and size
-------------------------------------------------------------*/
void BIFSDecoder::setIntBuffer(int buffer, int size)
{
	m_buffer=buffer;
	m_bufferSize=size;
}
#endif	//AC end


/*-------------------------------------------------------------
Report an error to the executive
-------------------------------------------------------------*/
void BIFSDecoder :: Error (int nError)
{
	m_pExecutive -> Error ((NON_FATAL_ERROR) nError);
}

/*-------------------------------------------------------------
This virtual function is called if the parser exausted the unit at the
middle of decoding
-------------------------------------------------------------*/
void BIFSDecoder :: EndOfUnit ()
{
	throw BifsException (BIFX_END_OF_STREAM);
}

/*-----------------------------------------------------------------
Parses ObjectDescriptors and scene updates from the stream, and update
the scene accordingly.
-----------------------------------------------------------------*/
void BIFSDecoder :: ParseFrame (LPCBYTE pData, int nLength, DWORD dwTS)
{
	Reset (pData, nLength);
	dwTS;
	TRACEF ("AT %d", dwTS);
	TRACENEWLINE;
	try
	{
		ParseUpdateFrame ();
		m_pExecutive -> RequestChannels ();
	}
	catch (BifsException err)
	{
		err;
		TRACEF ("ERROR", 0);
		TRACENEWLINE;
		Error (SCENE_PARSING_ERROR);
	}
}

/*-----------------------------------------------------------------
Parse RoutesDescription
-----------------------------------------------------------------*/
void BIFSDecoder :: ParseRoutesDescription (Root *pRoot)
{
	if (ParseInt (1, "ListDescription"))		// if ListDescription
		do
			ParseRoute (pRoot);
		while (ParseInt (1, "moreROUTEs"));		// while moreROUTEs
	else
	{	int nBits = ParseInt (5, "nBits"); // mp : conformance
		int nSize = ParseInt (nBits, "length");
		for (int i = nSize; --i >= 0;)
			ParseRoute (pRoot);
	}
}

/*-----------------------------------------------------------------
Parse one Route and insert it into the scene whose root is pRoot
-----------------------------------------------------------------*/
void BIFSDecoder :: ParseRoute (Root *pRoot, int nID)
{
	/*-----------------------------------------------------------------
	if nID >= we already know the ID
	-----------------------------------------------------------------*/
	ZString routeName;
	if (nID < 0)
	{	if (ParseInt (1, "isUpdateable"))	// if isUpdatable // mp conformance
		{	nID = ParseInt (GetRouteIDbits (), "routeID"); //BF Conformance
			if (pRoot -> GetInline () ->IsCodeNames ())
			{	ParseString (routeName);
				TRACEF ("DEF %s ", (LPCSTR) routeName);
			}
			else
				TRACEF ("DEF %d ", nID);
		}
	}

	int nNodeID = ParseInt (GetNodeIDbits (), "outNodeID"); //BF Conformance
	MediaObject *pNode = pRoot -> FindNode (nNodeID);
	if (pNode == NULL)
		throw BifsException (BIFX_NODEID_NOT_FOUND);
	TRACEF ("ROUTE %s.", pNode ->GetNameID ());
	if (!pNode -> AddRoute (nID, this, routeName))
		throw BifsException (BIFX_PARSING_ERROR);
	TRACENEWLINE;
}

/*-----------------------------------------------------------------
Parse an UpdateFrame
-----------------------------------------------------------------*/
void BIFSDecoder :: ParseUpdateFrame ()
{
	TRACEF ("UpdateScene {", 0);
	TRACENEWLINE;
	TRACEINDENT (1);

	do
	{
		int nCommandCode = ParseInt (2, "code");	//BF Conformance // changed to 2 in FDIS
		/*-------------------------------------------------------------
		Only ReplaceScene with Protos is allowed if this is an Extern Proto library
		-------------------------------------------------------------*/
		if (m_bExternProtoLib && nCommandCode != 3)
			throw BifsException (BIFX_ILLEGAL_CODE_IN_EXTERNPROTO_LIB);

		switch (nCommandCode) {
		case 0:
			ParseInsertionCommand ();
			break;
		case 1:
			ParseDeletionCommand ();
			break;
		case 2:
			ParseReplacementCommand ();
			break;
		case 3:
			ParseSceneReplaceCommand ();
			break;
		}

		TRACENEWLINE;
	}
	while (ParseInt (1, "continue"));  //BF Conformance

	TRACEINDENT (-1);
	TRACEF ("}", 0);
	TRACENEWLINE;
}

/*-----------------------------------------------------------------
Parse Insertion Command
-----------------------------------------------------------------*/
void BIFSDecoder :: ParseInsertionCommand ()
{
	int nID;
	MediaObject *pNode;

	if (!m_pScene)
		throw BifsException (BIFX_NODEID_NOT_FOUND);

	switch (ParseInt (2, "type")) {
	case 0:
		nID = ParseInt (GetNodeIDbits (), "nodeID"); // mp : conformance
		pNode = m_pScene -> FindNode (nID);
		if (!pNode)
			throw BifsException (BIFX_NODEID_NOT_FOUND);
		TRACEF ("Insert node into %s.children ", pNode ->GetNameID ());
		pNode -> ParseChild (this);
		break;
	case 2:
		nID = ParseInt (GetNodeIDbits (), "nodeID"); // mp : conformance
		pNode = m_pScene -> FindNode (nID);
		if (!pNode)
			throw BifsException (BIFX_NODEID_NOT_FOUND);
		TRACEF ("Insert into %s.", pNode ->GetNameID ());
		pNode -> InsertValue (this);
		break ;
	case 3:
		ParseRoute (m_pScene);
		break;
	}
}

/*-----------------------------------------------------------------
Parse DeletionCommand
-----------------------------------------------------------------*/
void BIFSDecoder :: ParseDeletionCommand ()
{
	int nID = 0;
	MediaObject *pNode;

	if (!m_pScene)
		throw BifsException (BIFX_NODEID_NOT_FOUND);

	int nType = ParseInt (2, "type");
	if (nType != 3)
		nID = ParseInt (GetNodeIDbits (), "nodeID"); // mp : conformance
	switch (nType) {
	case 0:
		pNode = m_pScene -> FindNode (nID);
		if (!pNode)
			throw BifsException (BIFX_NODEID_NOT_FOUND);
		TRACEF ("Delete %s", pNode ->GetNameID ());
		pNode -> Delete ();
		break;
	case 2:
		pNode = m_pScene -> FindNode (nID);
		if (!pNode)
			throw BifsException (BIFX_NODEID_NOT_FOUND);
		TRACEF ("Delete %s.", pNode ->GetNameID ());
		pNode -> DeleteValue (this);
		break ;
	case 3:
		nID = ParseInt (GetRouteIDbits (), "RouteID");
		Route *pRoute = m_pScene ->FindRoute (nID);
		if (pRoute)
		{
			TRACEF ("Delete ROUTE %s", pRoute ->GetName ());
			pRoute -> Remove ();
		}
		else
			TRACEF ("Delete ROUTE %d", nID);
		break ;
	}
}

/*-----------------------------------------------------------------
Parse replacement command
-----------------------------------------------------------------*/
void BIFSDecoder :: ParseReplacementCommand ()
{
	int nID = 0;
	MediaObject *pNode;

	if (!m_pScene)
		throw BifsException (BIFX_NODEID_NOT_FOUND);

	int nType = ParseInt (2, "type");
	if (nType != 3)
		nID = ParseInt (GetNodeIDbits (), "nodeID"); // mp : conformance
	switch (nType) {
	case 0:
		pNode = m_pScene -> FindNode (nID);
		if (!pNode)
			throw BifsException (BIFX_NODEID_NOT_FOUND);
		TRACEF ("REPLACE %s BY ", pNode ->GetNameID ());
		pNode -> ReplaceNode (this);
		break;
	case 1:
		pNode = m_pScene -> FindNode (nID);
		if (!pNode)
			throw BifsException (BIFX_NODEID_NOT_FOUND);
		TRACEF ("REPLACE %s.", pNode ->GetNameID ());
		pNode -> ReplaceValue (this);
		break;
	case 2:
		pNode = m_pScene -> FindNode (nID);
		if (!pNode)
			throw BifsException (BIFX_NODEID_NOT_FOUND);
		TRACEF ("REPLACE %s.", pNode ->GetNameID ());
		pNode -> ReplaceIndexedValue (this);
		break ;
	case 3:
		nID = ParseInt (GetRouteIDbits (), "RouteID");
		Route *pRoute = m_pScene ->FindRoute (nID);
		if (pRoute)
		{
			TRACEF ("REPLACE ROUTE %s BY ", pRoute ->GetName ());
			pRoute ->Remove ();
		}
		else
			TRACEF ("REPLACE ROUTE %d BY ", nID);
		ParseRoute (m_pScene, nID);
		break ;
	}
}

/*-----------------------------------------------------------------
Replace the scene (or generate the first scene
-----------------------------------------------------------------*/
void BIFSDecoder :: ParseSceneReplaceCommand ()
{
	/*-------------------------------------------------------------
	Delete the old scene (if this is not an ExternProto library
	-------------------------------------------------------------*/
	m_pScene = m_pSource -> GetScene ();
	if (!m_bExternProtoLib)
	{	m_pSource -> ReplaceScene (NULL);
		m_pScene -> SetInline (m_pSource);
	}

	/*-----------------------------------------------------------------
	The BIFS parser architecture is based on the principle that every object,
	node or node field, parses itself.  So to parse the scene, we
	instantiate an SFNode object, and call the object to parse itself.
	The parse function will throw an exception if end of stream is
	unexpectedly encountered.
	-----------------------------------------------------------------*/
	ParseInt (6, "reserved");
	m_pSource -> SetCodeNamesFlags (ParseInt (1, "useNames"));
	m_pScene -> ParseProtoList (this);
	if (m_bExternProtoLib)
		return;

	SFTopNode node;
	if (node.ParseNodeField (this, m_pScene, NULL, TRUE))
	{	/*-----------------------------------------------------------------
		If parse was successful, "node" now points to the scene root.
		-----------------------------------------------------------------*/
		m_pScene -> Configure (
			m_bPixelMetric,
			m_nPixelWidth,
			m_nPixelHeight,
			m_nNodeIDbits,
			m_nRouteIDbits,
			m_nProtoIDbits
		);

		/*-----------------------------------------------------------------
		Parse Routes
		-----------------------------------------------------------------*/
		if (ParseInt (1, "hasROUTEs"))	// if hasROUTEs
			ParseRoutesDescription (m_pScene);
		m_pSource -> ReplaceScene ((Root *) node.GetNode ());
	}
	else
		throw BifsException (BIFX_PARSING_ERROR);
}

/*-------------------------------------------------------------
Parse the data in the specificInfo field of the ES descriptor,
and configure the decoder appropriately.
-------------------------------------------------------------*/
bool BIFSDecoder :: Configure (const BYTE *specificInfo, int nInfoLength)
{
	BIFSConfig *pBifsConfig = NULL;
	bool bResult;
	try
	{	if (nInfoLength > 0)
		{	ODDecoder odDec (
				m_pExecutive,
				specificInfo,
				nInfoLength,
				m_pExecutive -> GetTraceFile (),
				"BIFS",
				m_pExecutive -> GetTraceFlags (),
				GetVersion ()
			); //BF Conformance
#ifdef AC_DEBUG
#ifdef PREDICTIVE	//AC toErase
			printf("GetVersion() : %d\n",GetVersion());
#endif	//AC end
#endif
			if (GetVersion () <= 1)
				pBifsConfig = new BIFSConfig;
			else
				pBifsConfig = new BIFSv2Config;
			pBifsConfig -> SetLength (nInfoLength * 8);
			pBifsConfig -> EncDec (&odDec, DECODE, NULL);
			if (pBifsConfig -> nodeIDbits > 0)
				m_nNodeIDbits = pBifsConfig -> nodeIDbits;
			if (pBifsConfig -> routeIDbits > 0)
				m_nRouteIDbits = pBifsConfig -> routeIDbits;
			if (pBifsConfig -> protoIDbits > 0)
				m_nProtoIDbits = pBifsConfig -> protoIDbits;
			m_bPixelMetric = pBifsConfig -> pixelMetric;
			m_nPixelWidth = pBifsConfig -> pixelWidth;
			m_nPixelHeight = pBifsConfig -> pixelHeight;
			m_bUse3DMeshCoding = pBifsConfig -> use3DMeshCoding;
			m_bUsePredictiveMFField = pBifsConfig -> usePredictiveMFField;
		}
		bResult = true;
	}
	catch (BifsException)
	{
		m_pExecutive -> Error (INVALID_BIFS_CONFIG);
		bResult = false;
	}
	delete pBifsConfig;
	return bResult;
}

/*-------------------------------------------------------------
Parse PROTOCode
-------------------------------------------------------------*/
void BIFSDecoder :: ParseProtoCode (Proto *pProto)
{
	TRACEF ("{", 0);
	TRACENEWLINE;
	TRACEINDENT (1);

	/*-------------------------------------------------------------
	Parse nexted Proto, the Proto's nodes and the Routes
	-------------------------------------------------------------*/
	pProto -> ParseProtoList (this);
	pProto -> ParseNodes (this);
	if (ParseInt (1, "hasROUTEs"))	// if hasROUTEs
		ParseRoutesDescription (pProto);
	TRACEINDENT (-1);
	TRACEF ("}", 0);
	TRACENEWLINE;
}

/*-------------------------------------------------------------
Parse a String value
-------------------------------------------------------------*/
void BIFSDecoder :: ParseString (ZString &string)
{
	string.Cut (0);
	char c;
	while ((c = ParseInt (8, "char")) != 0)  //BF Conformance
		string += c;
}
