/***********************************************************************
This software module was originally developed by Zvi Lifshitz
(Triton R&D Ltd.) in the course of development of the MPEG-4 Systems
(ISO/IEC 14496-1) standard. This software module is an implementation of
a part of one or more MPEG-4 Systems (ISO/IEC 14496-1) tools as
specified by the MPEG-4 Systems (ISO/IEC 14496-1) standard. ISO/IEC
gives users of the MPEG-4 Systems (ISO/IEC 14496-1) free license to
this software module or modifications thereof for use in hardware or
software products claiming conformance to the MPEG-4 Systems (ISO/IEC
14496-1). Those intending to use this software module in hardware or
software products are advised that its use may infringe existing patents.
The original developer of this software module and his/her company, the
subsequent editors and their companies, and ISO/IEC have no liability
for use of this software module or modifications thereof in an
implementation. Copyright is not released for non MPEG-4 Systems
(ISO/IEC 14496-1) conforming products. Triton R&D Ltd. retains full right
to use the code for his/her own purpose, assign or donate the code to a
third party and to inhibit third parties from using the code for non
MPEG-4 Systems (ISO/IEC 14496-1) conforming products. This copyright
notice must be included in all copies or derivative works.
Copyright  1998.
************************************************************************

For more information or to receive an up to date version of this module,
you may call Zvi Lifshitz at zvil@csi.com.
----------------------------------------------------------------------*/
//$Log: BifsDec.h,v $
//Revision 1.3.2.1  2000/03/10 15:59:31  Alex Cotarmanac'h
//deuxime envoi
//
//Revision 1.2  2000/03/03 18:41:21  Alex Cotarmanac'h
//Version 1 -- Alexandre Cotarmanac_h
//
#if !defined(_BifsDec_h)	// Sentry, use file only if it's not already included.
#define _BifsDec_h

/*-------------------------------------------------------------------------
Project Im1Player
INTERNATIONAL ORGANIZATION FOR STANDARDIZATION
ISO/IEC JTC1/SC29/WG11

SUBSYSTEM:  BIFS Decoder
FILE:       BifsDec.h
AUTHOR:     Zvi Lifshitz (Triton R&D Ltd.)
OVERVIEW:   Declarations of BIFSDecoder.
-------------------------------------------------------------------------*/

//Modified 2001/05/08 Boris Felts boris.felts@polytechnique.org

//#include "ztl.h" // MBS: added for SF<> field
#include "SysCoder.h"
#include "NodeFld.h" // MBS: added for SF<> field

class InlineImp;
class MediaObject;
class Root;
class Executive;
class Proto;

class BIFSDecoder : public SysCoder
{
private:
	virtual void ParseFrame (LPCBYTE pData, int nLength, DWORD dwTS);
	virtual void Error (int);
	void ParseSceneReplaceCommand();
	void ParseReplacementCommand ();
	void ParseDeletionCommand ();
	void ParseInsertionCommand ();
	void ParseRoute (Root *pRoot, int nID = -1);
	void ParseRoutesDescription (Root *pRoot);
	virtual void EndOfUnit ();

public:
	void ParseString (ZString &string);
	void ParseProtoCode (Proto *pProto);
	virtual bool Configure (const BYTE *specificInfo, int nInfoLength);
	int GetNodeIDbits () {return m_nNodeIDbits;}
	int GetRouteIDbits () {return m_nRouteIDbits;}
	int GetProtoIDbits () {return m_nProtoIDbits;}
	BIFSDecoder (
		Executive *pExecutive,
		InlineImp *pSource,
		FILE *fList,
		LPCSTR szListTitle,
		int flags,
		int version,
		bool bExternProto = false
	);
	void ParseUpdateFrame ();
	Executive *GetExecutive () const {return m_pExecutive;}
	InlineImp *GetSource () const {return m_pSource;}
	Root *GetScene () const {return m_pScene;}
	SysCoder *GetUICoder () const {return m_pUICoder;}
	void SetUICoder (SysCoder *pCoder) {m_pUICoder = pCoder;}

        // MBS: 3 methods added for quantization
        // sets current QP
        void SetCurrentQP(MediaObject *qp) { m_pCurrentQP=qp; }
        // gets current QP
        MediaObject *GetCurrentQP() { return m_pCurrentQP; }
        // do we have a local QP ?
        BOOL hasLocalQP() { return (m_pCurrentQP ? ((BOOL) *((SFBool *) m_pCurrentQP->GetField(0))) : FALSE) ; }
        // end MBS
//******************************************************************************
//#ifdef PREDICTIVE	//AC 19991197 : UsePredictiveMFField - B AC 20000227 Quantization Scheme 14 stuff. //BF
public:
#ifdef PREDICTIVE	//BF
	bool UsePredictiveMFField () { return ( m_bUsePredictiveMFField ? true : false );}
	void setIntBuffer(int buffer, int size); // set the buffer for ParseInt
#endif

	// Quantization Scheme 14 stuff.
	// remember to store number of points
	void setCoordStore() { m_coordStore=true;}
	// remember to use number of points
	void setCoordActive() { m_coordActive=true; }
	// can I use the recorded number of points ?
	bool isCoordActive() { return m_coordActive; }
	// do I have to store the number of points ?
	bool isCoordStore() { return m_coordStore; }
	// store the number of points
	void setLengthCoord(int l) ;
	// retrieve the number of points
	int getLengthCoord();
	// reset the flags
	void resetCoord();
	// store in case it's used somewhere else USE CoordName
	void setIdToLength(int kId, int Length);
	// retrieve stored value and set as current length.
	void setCurrentId(int kId);
#ifdef PREDICTIVE  //BF
protected:
	virtual int GetBit ();
private:
	int m_buffer;
	int m_bufferSize;
#endif
	// Quantization Scheme 14 stuff.
	bool m_coordActive; // can we use m_lengthCoord ?
	bool m_coordStore;  // do we have to store m_lengthCoord ?
	int  m_lengthCoord;
	ZMap<int> m_IdToLength; // maps DEF_id to length
//#endif	//AC end  //BF
//******************************************************************************
private:
        // MBS: currently used QP node
        MediaObject *m_pCurrentQP;

	Executive *m_pExecutive;
	InlineImp *m_pSource;
	BOOL m_bPixelMetric, m_bUsePredictiveMFField, m_bUse3DMeshCoding;
	int m_nPixelWidth, m_nPixelHeight;
	bool m_bExternProtoLib;
	SysCoder *m_pUICoder;

protected:
	Root *m_pScene;
	int m_nNodeIDbits, m_nRouteIDbits, m_nProtoIDbits;
};

#endif                                      // _BifsDec_h sentry.
