/***********************************************************************
This software module was originally developed by Zvi Lifshitz
(Optibase Ltd.) in the course of development of the MPEG-4 Systems
(ISO/IEC 14496-1) standard. This software module is an implementation of
a part of one or more MPEG-4 Systems (ISO/IEC 14496-1) tools as
specified by the MPEG-4 Systems (ISO/IEC 14496-1) standard. ISO/IEC
gives users of the MPEG-4 Systems (ISO/IEC 14496-1) free license to
this software module or modifications thereof for use in hardware or
software products claiming conformance to the MPEG-4 Systems (ISO/IEC
14496-1). Those intending to use this software module in hardware or
software products are advised that its use may infringe existing patents.
The original developer of this software module and his/her company, the
subsequent editors and their companies, and ISO/IEC have no liability
for use of this software module or modifications thereof in an
implementation. Copyright is not released for non MPEG-4 Systems
(ISO/IEC 14496-1) conforming products. Optibase Ltd. retains full right
to use the code for his/her own purpose, assign or donate the code to a
third party and to inhibit third parties from using the code for non
MPEG-4 Systems (ISO/IEC 14496-1) conforming products. This copyright
notice must be included in all copies or derivative works.
Copyright  1997.
************************************************************************

For more information or to receive an up to date version of this module,
you may call Zvi Lifshitz at zvil@csi.com.
----------------------------------------------------------------------*/

/*-------------------------------------------------------------------------
Project Im1Player
INTERNATIONAL ORGANIZATION FOR STANDARDIZATION
ISO/IEC JTC1/SC29/WG11

SUBSYSTEM:	Compositor (simulation)
FILE:       MediaControl.cpp
AUTHORS:    Zvi Lifshitz (Optibase Ltd.)
OVERVIEW:   Implementation of MediaControlImp - the node that implements MediaControl
-------------------------------------------------------------------------*/

#include "defs.h"
#include "MediaControl.h"
#include "Stream.h"
#include "BifsDec.h"
#include "Inline.h"
#include "ODImp.h"
#include "AL.h"
#include "Stream.h"

/*-------------------------------------------------------------
MediaControlImp constructor
-------------------------------------------------------------*/
MediaControlImp :: MediaControlImp () :
	m_bStart (true),
	m_bChanged (true),
	m_bPaused (false),
	m_saveMediaStartTime (-1),
	m_saveMediaStopTime (-1),
	m_saveMediaSpeed (1.0),
	m_bEnabled (false),
	m_bPreRolled (false)
{}

/*-------------------------------------------------------------
The Render function examines the status of the media and update the fields accodringly
-------------------------------------------------------------*/
BOOL MediaControlImp :: Render (PresenterContext *)
{
	int i;

	/*-------------------------------------------------------------
	Check whether a field in the node has changed and the node is enabled and act accordingly
	-------------------------------------------------------------*/
	if (!m_bChanged || !(Bool) enabled)
		return true;
	m_bChanged = false;

	/*-------------------------------------------------------------
	If node is enabled now and another MediaControl node controling same stream was enabled
	before, disable the previous one and copy its status to the "save" fields in the current
	node
	-------------------------------------------------------------*/
	if (!m_bEnabled)
	{	m_bEnabled = true;
		if (GetOD () != NULL)
		{	MediaControlImp *pPreControl = GetOD () -> SetMediaControl (this);
			if (GetStream () != NULL)
				GetStream () ->GetClockReference () -> SetMediaControl (this);
			if (pPreControl != NULL && pPreControl != this)
				pPreControl -> SwitchControl (this);
		}
	}

	/*-------------------------------------------------------------
	If the url field has changed copy the url field to the base StreamConsumer URL field
	and re-register the node.
	-------------------------------------------------------------*/
	bool bURLchanged = m_saveUrl.GetLength () != url.GetLength ();
	if (!bURLchanged)
		for (i = url.GetLength (); --i >= 0;)
			if (m_saveUrl [i] != url [i])
			{	bURLchanged = true;
				break;
			}
	if (bURLchanged)
	{	for (i = 0; i < url.GetLength (); i++)
			m_saveUrl += url [i];
		if (!m_bStart)		// not needed first time after node instantiation
		{	RemoveStream ();
			GetInline () -> UnregisterStreamConsumer (this);
			Register ();
		}
	}
	m_bStart = false;

	/*-------------------------------------------------------------
	Handle change of speed command
	-------------------------------------------------------------*/
	if (m_saveMediaSpeed != mediaSpeed)
	{	/*-------------------------------------------------------------
		Pause/resume the clock if necessary
		-------------------------------------------------------------*/
		if (GetStream () != NULL)
			if (mediaSpeed == 0 && !m_bPaused)
			{	GetStream () ->GetClockReference () -> Pause ();
				m_bPaused = true;
			}
			else if (mediaSpeed != 0 && m_bPaused)
			{	GetStream () ->GetClockReference () -> Resume ();
				m_bPaused = false;
			}
	}

	/*-------------------------------------------------------------
	If the media start/stop time or play time has changed, and the stream is currently
	playing, send server commands to change play parameters
	-------------------------------------------------------------*/
	if (m_saveMediaStartTime != mediaStartTime ||
		m_saveMediaStopTime != mediaStopTime ||
		m_saveMediaSpeed != mediaSpeed
	)
	{	if ((Bool) preRoll || GetOD () -> IsPlaying ())
		{	/*-------------------------------------------------------------
			Pause if necessary, or, if only speed has changed, send a play command with the new speed
			-------------------------------------------------------------*/
			if (mediaSpeed != m_saveMediaSpeed)
			{	if (mediaSpeed == 0)
					GetOD () -> Pause ();
				else if (mediaStartTime == -1 && mediaStopTime == m_saveMediaStopTime)
				{	GetOD () -> Play (mediaSpeed);
					if ((Bool) mute)
						GetOD () -> Mute ();
				}
			}

			/*-------------------------------------------------------------
			If play parameters have changed stop the playing and restart with the new
			parameters
			-------------------------------------------------------------*/
			if (mediaSpeed != 0 &&
				(mediaStartTime >= 0 || mediaStopTime != m_saveMediaStopTime)
			)
				GetOD () -> Restart ();
		}
		m_saveMediaSpeed = mediaSpeed;
		m_saveMediaStartTime = mediaStartTime;
		m_saveMediaStopTime = mediaStopTime;
		m_saveMute = (bool) (Bool) mute;
	}
	else if ((Bool) mute != m_saveMute)
	{	/*-------------------------------------------------------------
		if only mute has changed, then mute/demute
		-------------------------------------------------------------*/
		if ((Bool) mute)
			GetOD () -> Mute ();
		else
			GetOD () -> Play (1);
		m_saveMute = (bool) (Bool) mute;
	}

	/*-------------------------------------------------------------
	Preroll the stream if necessary
	-------------------------------------------------------------*/
	if ((Bool) preRoll && !m_bPreRolled && GetOD () != NULL)
	{	m_bPreRolled = true;
		GetOD () -> PreRoll ();
	}
	return true;
}

/*-------------------------------------------------------------
Mark that a field in the node is changed so that when we render the node we check which
field has changed and act accordingly
-------------------------------------------------------------*/
void MediaControlImp :: OnUpdate ()
{
	m_bChanged = true;
}

/*-------------------------------------------------------------
SetStream is a virtual fucntion that is called when the node is associated with an OD.
In this case first execute the base SetStream function, then store node address
on the associated ClockReference object so it affect all objects using same time base
-------------------------------------------------------------*/
void MediaControlImp :: SetStream (ObjectDescriptorImp *pOD)
{
	StreamConsumer ::SetStream (pOD);
	if ((Bool) enabled)
	{	GetOD () -> SetMediaControl (this);
		if (GetStream () != NULL)
			GetStream () ->GetClockReference () -> SetMediaControl (this);
	}
}

/*-------------------------------------------------------------
RemoveStream is a virtual fucntion that is called when the node is deassociated from its OD.
In this case first remove the node address from the associated ClockReference then 
execute the base RemoveStream function
-------------------------------------------------------------*/
void MediaControlImp :: RemoveStream ()
{
	if ((Bool) enabled)
	{	GetOD () -> SetMediaControl (NULL);
		if (GetStream () != NULL)
		{	GetStream () ->GetClockReference () -> SetMediaControl (NULL);
			if (m_bPaused)		// resume the clock of the associated stream
			{	GetStream () ->GetClockReference () -> Resume ();
				m_bPaused = false;
			}
		}
	}
	StreamConsumer :: RemoveStream ();
}

/*-------------------------------------------------------------
Send a PLAY command (or a series of commands if there are multiple URLs) to the server
-------------------------------------------------------------*/
void MediaControlImp :: Play (int nChannel)
{
	if (mediaSpeed != 0)
	{	if (m_segments.GetLength () == 0)
			GetOD () ->GetAL () -> StartChannel (
				nChannel,
				mediaStartTime,
				mediaStopTime,
				mediaSpeed,
				GetOD () -> IsPlaying ()
			);
		else
			for (int i = 0; i < m_segments.GetLength (); i++)
			{	double dStart, dStop;
				CalculatePlayInterval (
					m_segments [i].m_dStart,
					m_segments [i].m_dDuration,
					dStart,
					dStop
				);
				GetOD () ->GetAL () -> StartChannel (
					nChannel,
					dStart,
					dStop,
					mediaSpeed,
					GetOD () -> IsPlaying ()
				);
			}

		/*-------------------------------------------------------------
		If necessary mute the stream
		-------------------------------------------------------------*/
		if ((Bool) mute)
			GetOD () ->GetAL () -> MuteChannel (nChannel);
	}

	/*-------------------------------------------------------------
	If necessary pause or resume the clock
	-------------------------------------------------------------*/
	if (mediaSpeed == 0 && !m_bPaused)
	{	GetStream () ->GetClockReference () -> Pause ();
		m_bPaused = true;
	}
	else if (mediaSpeed != 0 && m_bPaused)
	{	GetStream () ->GetClockReference () -> Resume ();
		m_bPaused = false;
	}
}

/*-------------------------------------------------------------
Calculate the play interval for one URL element
-------------------------------------------------------------*/
void MediaControlImp :: CalculatePlayInterval (
	double dSegmentStart,
	double dSegmentDuration,
	double &dStart,
	double &dStop
) const
{
	if (mediaSpeed >= 0)
	{	dStart = dSegmentStart;
		dStop = -1;
		if (dSegmentDuration >= 0)
			dStop = dStart + dSegmentDuration;
		if (dStart < 0)
		{	dStart = mediaStartTime;
			if (mediaStopTime >= mediaStartTime)
				dStop = mediaStopTime;
		}
		else
		{	if (mediaStopTime >= 0 && mediaStopTime >= mediaStartTime &&
				(dStop < 0 || mediaStopTime < dSegmentDuration)
			)
				dStop = dStart + mediaStopTime;
			if (mediaStartTime > 0)
				dStart += mediaStartTime;
		}
	}
	else
	{	dStart = FLT_MAX;
		dStop = 0;
		if (dSegmentStart >= 0)
		{	dStop = dSegmentStart;
			if (dSegmentDuration >= 0)
				dStart = dSegmentStart + dSegmentDuration;
		}
		if (dStart == FLT_MAX)
		{	dStart = mediaStartTime;
			if (mediaStartTime < 0 || mediaStartTime >= mediaStopTime)
				dStop = mediaStopTime;
		}
		else
		{	if (mediaStartTime >= 0 &&
				(dSegmentDuration < 0 || mediaStartTime < dSegmentDuration)
			)
				dStart = dStop + mediaStartTime;
			double d = dStop;
			if (mediaStopTime > 0)
				d += mediaStopTime;
			if (d <= dStart)
				dStop = d;
		}
	}	
}

/*-------------------------------------------------------------
Given a timestamp (expressed in media time), determine whether the unit with this time stamp
is playable according to the MediaControl parameters
-------------------------------------------------------------*/
bool MediaControlImp :: IsUnitPlayable (double dTS)
{
	if (mediaSpeed == 0 || (Bool) mute)
		return false;
	if (mediaSpeed > 0)
	{	if (m_segments.GetLength () == 0)
			return mediaStartTime <= dTS && (mediaStopTime < 0 || mediaStopTime > dTS);
		for (int i = 0; i < m_segments.GetLength (); i++)
		{	double dStart, dStop;
			CalculatePlayInterval (
				m_segments [i].m_dStart,
				m_segments [i].m_dDuration,
				dStart,
				dStop
			);
			if (dStart <= dTS && (dStop < 0 || dStop > dTS))
				return true;
		}
	}
	else
	{	if (m_segments.GetLength () == 0)
			return mediaStopTime <= dTS && (mediaStartTime < 0 || mediaStartTime >= dTS);
		for (int i = 0; i < m_segments.GetLength (); i++)
		{	double dStart, dStop;
			CalculatePlayInterval (
				m_segments [i].m_dStart,
				m_segments [i].m_dDuration,
				dStart,
				dStop
			);
			if (dStop <= dTS && (dStart < 0 || dStart >= dTS))
				return true;
		}
	}
	return false;
}

/*-------------------------------------------------------------
Called when a new MediaControl controling same stream is enabled. Disable this node
and copy its status to the new node
-------------------------------------------------------------*/
void MediaControlImp :: SwitchControl (MediaControlImp *pNew)
{
	enabled = m_bEnabled = false;
	m_bPreRolled = false;
	pNew ->m_saveMute = m_saveMute;
	pNew ->m_bPaused = m_bPaused;
	m_bPaused = false;
	pNew ->m_saveUrl.Clear ();
	for (int i = 0; i < m_saveUrl.GetLength (); i++)
		pNew ->m_saveUrl += m_saveUrl [i];
	pNew ->m_saveMediaStartTime = m_saveMediaStartTime;
	pNew ->m_saveMediaStopTime = m_saveMediaStopTime;
	pNew ->m_saveMediaSpeed = m_saveMediaSpeed;
}

/*-------------------------------------------------------------
Place holder for a function be executed when a stream finished playing
-------------------------------------------------------------*/
void MediaControlImp :: StreamStopped ()
{

}
