/***********************************************************************
This software module was originally developed by Zvi Lifshitz
(Optibase Ltd.) in the course of development of the MPEG-4 Systems
(ISO/IEC 14496-1) standard. This software module is an implementation of
a part of one or more MPEG-4 Systems (ISO/IEC 14496-1) tools as
specified by the MPEG-4 Systems (ISO/IEC 14496-1) standard. ISO/IEC
gives users of the MPEG-4 Systems (ISO/IEC 14496-1) free license to
this software module or modifications thereof for use in hardware or
software products claiming conformance to the MPEG-4 Systems (ISO/IEC
14496-1). Those intending to use this software module in hardware or
software products are advised that its use may infringe existing patents.
The original developer of this software module and his/her company, the
subsequent editors and their companies, and ISO/IEC have no liability
for use of this software module or modifications thereof in an
implementation. Copyright is not released for non MPEG-4 Systems
(ISO/IEC 14496-1) conforming products. Optibase Ltd. retains full right
to use the code for his/her own purpose, assign or donate the code to a
third party and to inhibit third parties from using the code for non
MPEG-4 Systems (ISO/IEC 14496-1) conforming products. This copyright
notice must be included in all copies or derivative works.
Copyright  1997.
************************************************************************

For more information or to receive an up to date version of this module,
you may call Zvi Lifshitz at zvil@csi.com.
----------------------------------------------------------------------*/

/*-------------------------------------------------------------------------
Project Im1Player
INTERNATIONAL ORGANIZATION FOR STANDARDIZATION
ISO/IEC JTC1/SC29/WG11

SUBSYSTEM:	Compositor (simulation)
FILE:       MediaSensor.cpp
AUTHORS:    Zvi Lifshitz (Optibase Ltd.)
OVERVIEW:   Implementation of MediaSensorImp - the node that implements MediaSensor
-------------------------------------------------------------------------*/

#include "defs.h"
#include "MediaSensor.h"
#include "Stream.h"
#include "ODImp.h"

/*-------------------------------------------------------------
The Render function examines the status of the media and update the fields accodringly
-------------------------------------------------------------*/
BOOL MediaSensorImp :: Render (PresenterContext *)
{
	/*-------------------------------------------------------------
	Loop through the URLs to see if the current media position is in one of them
	-------------------------------------------------------------*/
	bool bActive = false;
	double dTS = m_dMediaTime;
	int nSize, nStatus, bEOU;
	DWORD dwTime;
	if (GetStream () != NULL &&
		GetStream () ->Fetch (nSize, dwTime, bEOU, nStatus, true, false, 0, -1) != NULL
	)
	{	dTS =
			dwTime / (double) 1000 +
			GetStream () ->GetClockReference () ->GetMediaTimeOffset ();
		do
		{	/*-------------------------------------------------------------
			If media time precedes the end of previous segment restart from first URL
			-------------------------------------------------------------*/
			if (dTS < m_dEndOfLastSegment)
			{	ResetURL ();
				m_dEndOfLastSegment = 0;
			}
			/*-------------------------------------------------------------
			If media time precedes current segment go out of the loop
			-------------------------------------------------------------*/
			if (dTS < GetSegmentStart ())
				break;
			/*-------------------------------------------------------------
			If media time is within the segment go out of the loop with bActive = true
			-------------------------------------------------------------*/
			if (GetSegmentStart () < 0 || GetSegmentDuration () < 0 ||
				dTS < GetSegmentStart () + GetSegmentDuration ()
			)
			{	bActive = true;
				break;
			}
			/*-------------------------------------------------------------
			Media time exceeds the current segment. So try next URL and keep looping
			-------------------------------------------------------------*/
			m_dEndOfLastSegment = GetSegmentStart () + GetSegmentDuration ();
		}
		while (NextURL ());
	}
	/*-------------------------------------------------------------
	Update node fields
	-------------------------------------------------------------*/
	if (bActive != m_bActive)
		isActive = m_bActive = bActive;
	if (bActive)
	{	if (dTS != m_dMediaTime)
			mediaCurrentTime = m_dMediaTime = dTS;
		if (m_dSegmentEnd >= 0 && (dTS < m_dSegmentStart || dTS >= m_dSegmentEnd))
		{	LPCSTR szSegment;
			if (GetOD () -> FindSegment (dTS, m_dSegmentStart, m_dSegmentEnd, szSegment))
				info.SetAt (0, szSegment);
		}
	}
	return true;
}
