/***********************************************************************
This software module was originally developed by Zvi Lifshitz
(Triton R&D Ltd.) in the course of development of the MPEG-4 Systems
(ISO/IEC 14496-1) standard. This software module is an implementation of
a part of one or more MPEG-4 Systems (ISO/IEC 14496-1) tools as
specified by the MPEG-4 Systems (ISO/IEC 14496-1) standard. ISO/IEC
gives users of the MPEG-4 Systems (ISO/IEC 14496-1) free license to
this software module or modifications thereof for use in hardware or
software products claiming conformance to the MPEG-4 Systems (ISO/IEC
14496-1). Those intending to use this software module in hardware or
software products are advised that its use may infringe existing patents.
The original developer of this software module and his/her company, the
subsequent editors and their companies, and ISO/IEC have no liability
for use of this software module or modifications thereof in an
implementation. Copyright is not released for non MPEG-4 Systems
(ISO/IEC 14496-1) conforming products. Triton R&D Ltd. retains full right
to use the code for his/her own purpose, assign or donate the code to a
third party and to inhibit third parties from using the code for non
MPEG-4 Systems (ISO/IEC 14496-1) conforming products. This copyright
notice must be included in all copies or derivative works.
Copyright  1997.
************************************************************************

For more information or to receive an up to date version of this module,
you may call Zvi Lifshitz at zvil@csi.com.
----------------------------------------------------------------------*/
//$Log: NodeFld.cpp,v $
//
//Revision 1.x 2000/10/12  Pascal Belin
//QP bug fix
//
//Revision 1.2  2000/03/03 18:41:21  Alex Cotarmanac'h
//Version 1 -- Alexandre Cotarmanac_h
//
//Modified 2001/05/08 Boris Felts boris.felts@polytechnique.org
//
/*-------------------------------------------------------------------------
Project Im1Player
INTERNATIONAL ORGANIZATION FOR STANDARDIZATION
ISO/IEC JTC1/SC29/WG11

SUBSYSTEM:  BIFS Decoder/ Compositor
FILE:       NodeFld.cpp
AUTHOR:     Zvi Lifshitz (Triton R&D Ltd.)
OVERVIEW:   Implementation of NodeField and derivations.
-------------------------------------------------------------------------*/
#include "defs.h"
#include "Nodes.h"
#include "NodeFld.h"
#include "MediaObj.h"
#include "BifsDec.h"
#include "Route.h"
#include "Root.h"
#include "Quantize.h"
#include "NodeCode.h"
#include "Proto.h"
#include "Inline.h"
#include "SFScript.h" //BF Script

#if PREDICTIVE
#include "Predictive.h"//AC: do not forget to add Predictive/Decoder to include path of this file
#endif

/*-----------------------------------------------------------------
NodeField destructor. Delete all routes
-----------------------------------------------------------------*/
NodeField :: ~NodeField ()
{
	while (m_pFirstRoute)
		m_pFirstRoute ->Remove ();  // this unlinks the route automatically
}

/*-----------------------------------------------------------------
Add a Route to the nodeField
-----------------------------------------------------------------*/
void NodeField :: LinkRoute (Route *pRoute)
{
	pRoute -> Link (m_pFirstRoute);
}

/*-----------------------------------------------------------------
Unlink a Route from the list of routes
-----------------------------------------------------------------*/
void NodeField :: UnlinkRoute (Route *pRoute)
{
	pRoute -> Unlink (m_pFirstRoute);
}

/*-----------------------------------------------------------------
Activate attached routes on nodeField update
The paramter bFirst is used to eliminate loops.
-----------------------------------------------------------------*/
void NodeField :: OnFieldUpdate (bool bIsFirst)
{
	for (Route *pRoute = m_pFirstRoute;
		pRoute != NULL;
		pRoute = pRoute -> GetNext ()
	)
		pRoute -> Activate (bIsFirst);
}

/*-----------------------------------------------------------------
Parse an SFInt
-----------------------------------------------------------------*/
BOOL SF<int> :: Parse (BIFSDecoder *pCoder, MediaObject*, Quantizer *pQuant)
{
	if(!pQuant || !pQuant -> IsActive ())
		m_value = pCoder -> ParseInt (32, "intValue");
	else
		m_value = pQuant -> ParseInt (pCoder);

	if (pCoder -> GetUICoder ())
		m_value = pCoder -> GetUICoder () -> ParseInt (32, "intValue");

	TRACEF ("%d", m_value);
	return TRUE;
}

/*-----------------------------------------------------------------
Parse an SFBool
-----------------------------------------------------------------*/
BOOL SF<Bool> :: Parse (
	BIFSDecoder *pCoder,
	MediaObject * /*pNode*/,
	Quantizer * /*pQuant*/
)
{
	m_value = pCoder -> ParseInt (1, "value") != 0; // mp : conformance
	if (pCoder -> GetUICoder ())
		m_value = pCoder -> GetUICoder () -> ParseInt (1, "value") != 0;

	TRACEF ("%s", m_value ? "TRUE" : "FALSE");
	return TRUE;
}

/*-----------------------------------------------------------------
Parse and SFFloat
MBS: modified to take into account quantization parameters if any
-----------------------------------------------------------------*/
BOOL SF<float> :: Parse (
	BIFSDecoder *pCoder,
	MediaObject * /*pNode*/,
	Quantizer * pQuant
)
{
	if(!pQuant || !pQuant -> IsActive ())
		m_value = pCoder -> ParseFloat ("value"); // mp conformance
	else
		m_value = pQuant->ParseFloat(pCoder);

	if (pCoder -> GetUICoder ())
		m_value = pCoder -> GetUICoder () -> ParseFloat ("value");
	TRACEF ("%.2f", m_value);
	return TRUE;
}

/*-----------------------------------------------------------------
Parse an SFTime
-----------------------------------------------------------------*/
BOOL SF<Time> :: Parse (
	BIFSDecoder *pCoder,
	MediaObject * /*pNode*/,
	Quantizer * /*pQuant*/
)
{
	m_value = pCoder -> ParseDouble("value");
	if (pCoder -> GetUICoder ())
		m_value = pCoder -> GetUICoder () -> ParseDouble("value");

	TRACEF ("%.2lf", m_value);
	return TRUE;
}

/*-----------------------------------------------------------------
Parse an SFVec3f
MBS: modified to take into account quantization parameters if any
-----------------------------------------------------------------*/
BOOL SF<Vec3f> :: Parse (
	BIFSDecoder *pCoder,
	MediaObject * /*pNode*/,
	Quantizer * pQuant
)
{
	if(!pQuant || !pQuant->IsActive())
	{	m_value.x = pCoder -> ParseFloat("float"); //BF Conformance
		m_value.y = pCoder -> ParseFloat("float"); //BF Conformance
		m_value.z = pCoder -> ParseFloat("float"); //BF Conformance
	}
	else
		m_value = pQuant -> ParseVec3f(pCoder);

	if (pCoder -> GetUICoder ())
	{
		m_value.x = pCoder -> GetUICoder () -> ParseFloat("float");
		m_value.y = pCoder -> GetUICoder () -> ParseFloat("float");
		m_value.z = pCoder -> GetUICoder () -> ParseFloat("float");
	}

	TRACEF3 ("%.2f %.2f %.2f", m_value.x, m_value.y, m_value.z);
	return TRUE;
}

/*-----------------------------------------------------------------
Parse an SFVec2f
MBS: modified to take into account quantization parameters if any
-----------------------------------------------------------------*/
BOOL SF<Vec2f> :: Parse (
	BIFSDecoder *pCoder,
	MediaObject * /*pNode*/,
	Quantizer * pQuant
)
{
	if(!pQuant || !pQuant->IsActive())
	{	m_value.x = pCoder -> ParseFloat("float");  //BF Conformance
		m_value.y = pCoder -> ParseFloat("float");  //BF Conformance
	}
	else
		m_value = pQuant -> ParseVec2f(pCoder);

	if (pCoder -> GetUICoder ())
	{
		m_value.x = pCoder -> GetUICoder () -> ParseFloat("float");
		m_value.y = pCoder -> GetUICoder () -> ParseFloat("float");
	}
	
	TRACEF2 ("%.2f %.2f", m_value.x, m_value.y);
	return TRUE;
}

/*-----------------------------------------------------------------
Parse an SFRotation
MBS: modified to take into account quantization parameters if any
-----------------------------------------------------------------*/
BOOL SF<Rotation> :: Parse (
	BIFSDecoder *pCoder,
	MediaObject * /*pNode*/,
	Quantizer * pQuant
)
{
	if(!pQuant || !pQuant->IsActive())
	{	m_value.x = pCoder -> ParseFloat("float");    //BF Conformance
		m_value.y = pCoder -> ParseFloat("float");    //BF Conformance
		m_value.z = pCoder -> ParseFloat("float");    //BF Conformance
		m_value.theta = pCoder -> ParseFloat("float");//BF Conformance
	}
	else
		m_value = pQuant -> ParseRotation(pCoder);

	if (pCoder -> GetUICoder ())
	{
		m_value.x = pCoder -> GetUICoder () -> ParseFloat("float");
		m_value.y = pCoder -> GetUICoder () -> ParseFloat("float");
		m_value.z = pCoder -> GetUICoder () -> ParseFloat("float");
		m_value.theta = pCoder -> GetUICoder () -> ParseFloat("float");
	}

	TRACEF4 ("%.2f %.2f %.2f %.2f", m_value.x, m_value.y, m_value.z, m_value.theta);
	return TRUE;
}

/*-----------------------------------------------------------------
Parse an SFRotation2D
MBS: modified to take into account quantization parameters if any
-----------------------------------------------------------------*/
BOOL SF<Rotation2D> :: Parse (
	BIFSDecoder *pCoder,
	MediaObject * /*pNode*/,
	Quantizer * pQuant
)
{
  if(!pQuant || !pQuant->IsActive()) {
    m_value.x = pCoder -> ParseFloat("float");  //BF Conformance
    m_value.y = pCoder -> ParseFloat("float");  //BF Conformance
    m_value.theta = pCoder -> ParseFloat("float");//BF Conformance
  }
  else
    m_value = pQuant -> ParseRotation2D(pCoder);

  if (pCoder -> GetUICoder ())
  {
	  m_value.x = pCoder -> GetUICoder () -> ParseFloat("float");
	  m_value.y = pCoder -> GetUICoder () -> ParseFloat("float");
	  m_value.theta = pCoder -> GetUICoder () -> ParseFloat("float");
  }

  TRACEF3 ("%2.f %.2f %.2f", m_value.x, m_value.y, m_value.theta);
  return TRUE;
}

/*-----------------------------------------------------------------
Parse an SFColor
MBS: modified to take into account quantization parameters if any
-----------------------------------------------------------------*/
BOOL SF<Color3f> :: Parse (
	BIFSDecoder *pCoder,
	MediaObject * /*pNode*/,
	Quantizer * pQuant
)
{
	if(!pQuant || !pQuant->IsActive())
	{	m_value.r = pCoder -> ParseFloat("float");  //BF Conformance
		m_value.g = pCoder -> ParseFloat("float");  //BF Conformance
		m_value.b = pCoder -> ParseFloat("float");  //BF Conformance
	}
	else
		m_value = pQuant -> ParseColor3f(pCoder);

	if (pCoder -> GetUICoder ())
	{
		m_value.r = pCoder -> GetUICoder () -> ParseFloat("float");
		m_value.g = pCoder -> GetUICoder () -> ParseFloat("float");
		m_value.b = pCoder -> GetUICoder () -> ParseFloat("float");
	}

	TRACEF3 ("%.2f %.2f %.2f", m_value.r, m_value.g, m_value.b);
	return TRUE;
}

/*-----------------------------------------------------------------
Parse an SFObjectID
-----------------------------------------------------------------*/
BOOL SF<Url> :: Parse (
	BIFSDecoder *pCoder,
	MediaObject * pNode,
	Quantizer * /*pQuant*/
)
{

	if (pCoder -> ParseInt (1, "isOD"))
	{	m_value.ODid = pCoder -> ParseInt (10, "ODid");
		TRACEF ("%d", m_value.ODid);
	}
	else
	{	m_value.ODid = -1;
		m_value.urlValue.Parse (pCoder,pNode, NULL); //BF Script 
	}
	return TRUE;
}

/*-----------------------------------------------------------------
Parse SFString
MBS 110698: updated for UTF-8 String
Zvi 021799: redo MBS change. The string length is given in bytes, not
in characters. The handling of UTF-8 coding shall be done by the compositor -
it is not the matter of the parser.
-----------------------------------------------------------------*/
BOOL SF<ZString> :: Parse (
	BIFSDecoder *pCoder,
	MediaObject *pNode,    //  BF Script
	Quantizer * /*pQuant*/
)
{
 	if (pNode->GetNodeCode()== CODE_Script) {		// BF Script
		SFScript *mine = new SFScript((ScriptImp *)pNode);
		mine->Parse(pCoder);
		return TRUE;
	}
	pCoder ->ParseSFString (m_value);
	if (pCoder -> GetUICoder ())
		pCoder -> GetUICoder () ->ParseSFString (m_value);

	TRACEF ("\"%s\"", (LPCSTR) m_value);	//AC 20000229 inserted quotation marks.
	return TRUE;
}

/*-----------------------------------------------------------------
Parse SFImage
MBS 110498: this is a new type
BF  310401: fix the coding and the output
-----------------------------------------------------------------*/
BOOL SF<Image> :: Parse (
	BIFSDecoder *pCoder,
	MediaObject * /*pNode*/,
	Quantizer * /*pQuant*/
)
{
	m_value.width = pCoder -> ParseInt (12, "width");
	m_value.height = pCoder -> ParseInt (12, "height");
	m_value.numComponents = pCoder -> ParseInt (2, "numComponents");

	TRACEF3 (" %d %d %d", m_value.width, m_value.height, m_value.numComponents + 1);  // BF 010209: write pixel values
	TRACENEWLINE;
	//Alvar 981223: Added pixel array allocation and refcount stuff
	int nPixels = (m_value.numComponents + 1)* m_value.width * m_value.height;             // BF 010209: SFimage fix according to 9.3.7.23.1
	BYTE *p = m_value.pixels = new BYTE [nPixels];
	p;
	m_value.m_pRefCnt = new int;
	*(m_value.m_pRefCnt) = 1;
	int j_max= m_value.height *(m_value.numComponents + 1);
	for (int i = 0; i < m_value.width; i++ ){
		for (int j = 0; j < j_max; j++ ){
			TRACEF("0x%x ", *p++= pCoder -> ParseInt (8, "pixels"));                        // BF 010209: write pixel values
		}
		TRACENEWLINE;
	}
	// end of Alvar changes


	return TRUE;
}

/*-----------------------------------------------------------------
A dummy function
-----------------------------------------------------------------*/
BOOL SF<MediaObject *> :: Parse (BIFSDecoder *, MediaObject *, Quantizer *)
{
	return FALSE;
}

/*-----------------------------------------------------------------
SFGenericNode Destructor.  Deletes the MediaObject, unless it is defined
by other node and only USEd here.
-----------------------------------------------------------------*/
void SFGenericNode :: Remove ()
{
	if (m_pNode)
	{	m_pNode -> RemoveContainer (this);
		m_pNode = NULL;
	}
}

/*-----------------------------------------------------------------
Assign a value into a generic-node. If the old value is not a copy
of another nodeField, delete the old node
-----------------------------------------------------------------*/
MediaObject *SFGenericNode :: SetObject (MediaObject *node, MediaObject *pParent, bool bIsFirst)
{
	if (m_pNode)
		m_pNode -> RemoveContainer (this);
	m_pNode = node;
	if (node)
		node -> SetContainer (this, false, pParent);
	OnFieldUpdate (bIsFirst);
	return node;
}

/*-----------------------------------------------------------------
Parse a node field which is a sub-node
-----------------------------------------------------------------*/
BOOL SFGenericNode :: Parse (
	BIFSDecoder *pCoder,
	MediaObject *pNode,
	Quantizer * /*pQuant*/
)
{
	m_pParent = pNode;
	return ParseNodeField (pCoder, pNode, this, FALSE);
}

/*-------------------------------------------------------------
This function does the actual work of node parsing when the node
is either a scalar field in a parent node (in which case the function
is called from SFGenericNode :: Parse), an element in a vector field,
or when it is parsed in ReplaceNode command. The later two cases ParseNode
is called directly, not through Parse().
-------------------------------------------------------------*/
BOOL SFGenericNode :: ParseNodeField (
	BIFSDecoder *pCoder,
	MediaObject *pNode,			// the node this field belongs to
	NodeField *pContainer,		// a pointer to the container field
								// (this is not always 'this' because
								// a temporary NodeField object can be
								// used to parse the node, and then the
								// node is assigned or inserted in another
								// NodeField object.
	BOOL bISMF					// TRUE when the field is an MF field
)
{
//#ifdef PREDICTIVE	//AC 20000227 Quantization Scheme 14 Stuff  //BF
	bool resetWhenFinished=false;
	bool recordData=false;
//#endif	//AC end  //BF
	bool isUse=false;
	int nID=-1;

	/*-------------------------------------------------------------
	m_pNode may already point to another node if the function is called
	during a ReplaceField command.
	-------------------------------------------------------------*/
	if (m_pNode)
	{	m_pNode -> RemoveContainer (pContainer, pNode);
		m_pNode = NULL;
	}

	if (pCoder -> ParseInt (1, "isReused"))	// if isReused
	{	nID = pCoder -> ParseInt (pCoder -> GetNodeIDbits (), "nodeID"); // mp : conformance

		/*-------------------------------------------------------------
		If the node ID is all ones, this actually means "null node"
		-------------------------------------------------------------*/
		if (nID == (1 << pCoder -> GetNodeIDbits ()) - 1)
		{	TRACEF ("NULL", 0);
			return true;
		}

		if (pNode &&
			(m_pNode = pNode -> GetRoot () -> FindNode (nID)) != NULL
		)
		{
			m_pNode -> SetContainer (pContainer, bISMF, pNode);
			// PB added this if{} to have the QP working with DEF and USE
			if(m_pNode->GetNodeCode() == QuantizationParameter :: GetCode())
			{
				pCoder->SetCurrentQP(m_pNode);
			}
			// PB end
		}
		else
			return FALSE;
		TRACEF ("USE %s", m_pNode ->GetNameID ());
		isUse=true; //AC 20000227
	}
	else
	{
		/*-----------------------------------------------------------------
		Get object type, instantiate the object, and let it parse itself
		-----------------------------------------------------------------*/
		nID = -1;
		int nNodeType =
			pCoder -> ParseInt (
				BitFieldWidth (GetNodeTableSize () - GetV2ExtSize () - GetV3ExtSize () - GetV4ExtSize() + 1),
				"localNodeType"
			); //BF Conformance
		int nProtoID = -1;
		/*-------------------------------------------------------------
		If nodeType is zero this is an escape to V2 or V3
		-------------------------------------------------------------*/
		if (nNodeType == 0)
		{	int nExt = pCoder -> ParseInt (BitFieldWidth (GetV2ExtSize () + 2), "nodeTypeV2");
			if (nExt == 1)
			{	nNodeType = -1;		// negative value signals a PROTO
				nProtoID = pCoder -> ParseInt (pCoder -> GetProtoIDbits (), "PROTOnodeType");
			}
			else
			{	if (nExt == 0)
				{	nExt = pCoder -> ParseInt (BitFieldWidth (GetV3ExtSize () + 1), "nodeTypeV3");
					if (nExt == 0)
					{	nExt = pCoder -> ParseInt (BitFieldWidth (GetV4ExtSize () + 1), "nodeTypeV4");
						if (nExt == 0)
						{	nExt = pCoder -> ParseInt (BitFieldWidth (GetV5ExtSize () + 1), "nodeTypeV5");
							if (nExt == 0)
								return false;		// another escape which we don't support in V5
							nExt += GetV4ExtSize();
						}
						nExt += GetV3ExtSize();
					}
					nExt += GetV2ExtSize () + 1;
				}
				nNodeType = GetNodeTableSize () - GetV2ExtSize () - GetV3ExtSize () - GetV4ExtSize () + GetV5ExtSize () + nExt - 1;
			}
		}

		/*-----------------------------------------------------------------
		The root of the new object is the same as of the container
		object. If the container object is itslef a root, the root is
		the container object.
		-----------------------------------------------------------------*/
		Root *pRoot = pNode -> GetRoot () ? pNode -> GetRoot () : (Root *) pNode;

		/*-------------------------------------------------------------
		Create a new node. If this is a proto, instantiate the proto
		-------------------------------------------------------------*/
		if (nNodeType >= 0)
			m_pNode = CreateNode (nNodeType);
		else
		{	/*-------------------------------------------------------------
			Locate the Proto, and check whether its first node is of appropriate type.
			-------------------------------------------------------------*/
			Proto *pProto = pRoot -> FindProto (nProtoID);
			if (pProto == NULL)
				return NULL;
			m_pNode = pProto -> Instantiate (pRoot);
			if (m_pNode == NULL)
				return false;
			MediaObject *pFirst = ((Proto *)m_pNode) ->GetFirstNode ();
			if (pFirst && !IsTypeOK (pFirst))
			{	m_pNode -> Remove ();
				m_pNode = NULL;
			}
		}
		if (!m_pNode)
			return FALSE;

		/*-------------------------------------------------------------
		Parse the DEF id, if any
		-------------------------------------------------------------*/
		if (pCoder -> ParseInt (1, "isUpdateable")) // if isUpdatable  //BF Conformance
		{
			nID = pCoder -> ParseInt (pCoder -> GetNodeIDbits (), "nodeID"); // mp : conformance
			if (pRoot -> GetInline () ->IsCodeNames ())
			{	m_pNode ->ParseNameID (pCoder);
				TRACEF ("DEF %s ", m_pNode ->GetNameID ());
			}
			else
				TRACEF ("DEF ID%d ", nID);//AC20000228 just to get it running in a Vrml Browser
		}

		m_pNode -> SetID (nID);
		m_pNode -> SetContainer (pContainer, bISMF, pNode);
		m_pNode -> SetRoot (pRoot);
		m_pNode -> SetExecutive (pCoder -> GetExecutive ());

		//***********************************
//#ifdef PREDICTIVE	//AC 20000227 QP14 Stuff  //BF

		switch (m_pNode->GetNodeCode())
		{
		case CODE_Coordinate:
		case CODE_Coordinate2D:
			pCoder->setCoordStore();
			if (nID>=0) recordData=true;
			break;
		case CODE_IndexedFaceSet:
		case CODE_IndexedFaceSet2D:
		case CODE_IndexedLineSet:
		case CODE_IndexedLineSet2D:
			pCoder->setCoordActive();
			resetWhenFinished=true;
			break;
		default:
			break;
		}
// #endif	//AC end  //BF
		/*-------------------------------------------------------------
		Parse the node
		-------------------------------------------------------------*/

		// PB Sept 1. 2000 - The following line used to be after the if
		//  According to the standard, QPs don't quantize themselves, but only their nodes.
		m_pNode -> Parse (pCoder);

		// MBS: sets current QP in decoder if we have a QP node
		if(m_pNode->GetNodeCode() == QuantizationParameter :: GetCode())
		{
			pCoder->SetCurrentQP(m_pNode);
		}
		// end MBS

	    // PB: Cancelled the change made by BenA in july 99.

		// PB end
	}


	//***********************************
// #ifdef PREDICTIVE	//AC 20000227 Quantization Scheme 14  //BF
//	QP14 Use Case
	if (isUse)
	{
		//do we have to retrieve any data ?
		// we try and find when isCoordActive.
		switch (m_pNode->GetNodeCode())
		{
		case CODE_Coordinate:
		case CODE_Coordinate2D:
			pCoder->setCurrentId(nID);
		default:
			break;
		}
	}

	if (recordData)
	{
		if(pCoder->isCoordActive()) // PB put this into comment: assert (used to check this was active)
		if (nID>=0) pCoder->setIdToLength(nID,pCoder->getLengthCoord());
	}
	if (resetWhenFinished) pCoder->resetCoord();

//#endif	//AC end  //BF


	return TRUE;
}

/*-------------------------------------------------------------
Clone a node field when a Proto is instantiated
-------------------------------------------------------------*/
NodeField *SFGenericNode :: Clone (NodeField *pField, MediaObject *pParent)
{
	if (pField == NULL)
		pField = new SFGenericNode;
	if (m_pNode)
	{	((SFGenericNode*) pField) ->m_pNode =  m_pNode -> Clone (pParent -> GetRoot ());
		((SFGenericNode*) pField) -> m_pNode -> SetContainer (pField, false, pParent);
	}
	return pField;
}

/*-----------------------------------------------------------------
Parse an MF nodeField
-----------------------------------------------------------------*/
BOOL MField :: Parse (
	BIFSDecoder *pCoder,
	MediaObject *pNode,
	Quantizer *pQuant
)
{
	bool bFirst = true;
	if (pNode == NULL || pNode -> GetNodeCode () != CODE_Script || GetType() != MFSTRING) // mp
	{
	    TRACEF ("[", 0);
    }
	else
	{
        bFirst = false; // mp don't indent the first line
    }

	Remove ();
	/*-----------------------------------------------------------------
	MF nodeField may be coded as ListMFDescription or VectorMFDescription
	-----------------------------------------------------------------*/
	// MBS: variables for quantization
	BOOL localQP=FALSE;
	MediaObject *qpOld = NULL;
	// end MBS

	// BenA (7/30/99): save qp for this level
	MediaObject *qpForThisLevel = pCoder->GetCurrentQP();
	// end BenA

#ifdef PREDICTIVE	//AC 19991117
	/*-----------------------------------------------------------------
	MF nodeField may be coded also Predictive Mode
	-----------------------------------------------------------------*/

	// First check whether Predictive Mode is enabled
	if ( pCoder->UsePredictiveMFField() ) {
		if ( pCoder -> ParseInt(1,"usePredictive") )
		{
			if (bFirst) {
				TRACENEWLINE;
				TRACEINDENT (1);
				bFirst=false;
			}
			PredictiveMField a_PMField = PredictiveMField(pCoder, pNode, pQuant, this );
			return a_PMField.Parse ();
		}
	}
	// Disabled
#endif	//AC end
	if (!pCoder -> ParseInt (1, "reserved"))		// reserved, added in FDIS
	{	if (pCoder -> ParseInt (1, "isListDescription"))	// isListDescription
		{
//#ifdef PREDICTIVE	//AC  //BF
			int nElements=0;
//#endif	//AC end    //BF
			while (pCoder -> ParseInt (1, "endFlag") == 0)	// changed for  FDIS // mp : conformance
			{
				if (bFirst)
				{	bFirst = false;
					TRACENEWLINE;
					TRACEINDENT (1);
				}
				// MBS: saves old QP
				if(!localQP) qpOld=pCoder->GetCurrentQP();
				// end MBS

//#ifdef PREDICTIVE	//AC  //BF
				nElements++;
//#endif	//AC end  //BF

				if (!AppendElement (pCoder, pNode, pQuant))
					return FALSE;

				// MBS: restores old QP

				if(pCoder->hasLocalQP()) {
					if(localQP) {
						localQP=FALSE;
						pCoder->SetCurrentQP(qpOld);
					}
					else {
						localQP=TRUE;
					}
				}
				// end MBS

				TRACENEWLINE;
			}
//#ifdef PREDICTIVE	//AC  //BF
			pCoder->setLengthCoord(nElements);
//#endif	//AC end  //BF
		}
		else
		{
			int nElements = pCoder -> ParseInt (pCoder -> ParseInt (5, "nbBits"), "numberOfFields"); // mp conformance
//#ifdef PREDICTIVE	//AC  //BF
			pCoder->setLengthCoord(nElements);
//#endif	//AC end    //BF
			for (int i = nElements; --i >= 0;)
			{
				if (bFirst)
				{	bFirst = false;
					TRACENEWLINE;
					TRACEINDENT (1);
				}

				// MBS: saves old QP
				if(!localQP) qpOld=pCoder->GetCurrentQP();
				// end MBS

				if (!AppendElement (pCoder, pNode, pQuant))
					return FALSE;

				// MBS: restores old QP

				if(pCoder->hasLocalQP()) {
				  if(localQP) {
					localQP=FALSE;
					pCoder->SetCurrentQP(qpOld);
				  }
				  else {
					localQP=TRUE;
				  }
				}
				// end MBS

				TRACENEWLINE;
			}
		}

		// BenA (7/30/99): restore quantization for this level
		pCoder->SetCurrentQP(qpForThisLevel);
		// end BenA
	}

#ifdef TRACE_ENCODING
	if (!bFirst &&
		(pNode == NULL || pNode -> GetNodeCode () != CODE_Script || GetType () != MFSTRING)
	) // mp
		TRACEINDENT (-1);
	TRACEF ("]", 0);
	TRACENEWLINE;
#endif

	return TRUE;
}

/*-----------------------------------------------------------------
Deletes all nodes
-----------------------------------------------------------------*/
void MFGenericNode :: Remove ()
{
	for (int i = GetLength (); --i >= 0;)
		if (m_array [i])
			m_array [i] -> RemoveContainer (this, m_pParent);
	m_array.Clear ();
}

/*-----------------------------------------------------------------
Manipulation of MFGenericNode arrays
-----------------------------------------------------------------*/
void MFGenericNode :: InsertAt (int index, MediaObject *value, MediaObject *pParent, bool bRoute)
{
	m_array.InsertAt (index, value);
	value ->SetContainer (this, true, pParent);
	if (bRoute)
		OnFieldUpdate ();
}

void MFGenericNode :: Append (MediaObject *value, MediaObject *pParent, bool bRoute)
{
	if (value -> GetNodeCode () != CODE_Script)	// SCRIPT
	{	m_array += value;
		value ->SetContainer (this, true, pParent);
	}
	if (bRoute)
		OnFieldUpdate ();
}

void MFGenericNode :: SetAt (int index, MediaObject *value, MediaObject *pParent, bool bRoute)
{
	if (index < GetLength () && m_array [index] != NULL)
		m_array [index] -> RemoveContainer (this, m_pParent);
	m_array.SetAt (index, value);
	if (value != NULL)
		value ->SetContainer (this, true, pParent);
	if (bRoute)
		OnFieldUpdate ();
}

void MFGenericNode :: DeleteFrom (int index, BOOL bRoute)
{
	if (index < GetLength () && m_array [index] != NULL)
		m_array [index] -> RemoveContainer (this, m_pParent);
	m_array.DeleteFrom (index);
	if (bRoute)
		OnFieldUpdate ();
}

/*-----------------------------------------------------------------
Assigment operator activated in case of Routes. We assume that
the nodeField argument is always of the same type as this, therefor
allow for deliberate downcasting. This assumption is dangerous.
Commercial application should implement safe casting or proper
conversion mechanism.
-----------------------------------------------------------------*/
void MFGenericNode :: Copy (NodeField &nodeField, MediaObject *pParent, bool bIsFirst)
{
	MFGenericNode &fld = (MFGenericNode &) nodeField;
	for (int i = GetLength (); --i >= 0;)
		DeleteFrom (i, false);
	for (i = 0; i < fld.GetLength (); i++)
	{	m_array += fld.m_array [i];
		fld.m_array [i] -> SetContainer (this, true, pParent);
	}
	OnFieldUpdate (bIsFirst);
}

/*-------------------------------------------------------------
The node in this field has been replaced by ReplaceNode command
-------------------------------------------------------------*/
void SFGenericNode :: ReplaceNode (MediaObject *pNew)
{
	m_pNode = pNew;
	pNew -> SetContainer (this, FALSE, m_pParent);
}

/*-------------------------------------------------------------
Node pOld in this MFxxxNode field has been replaced by a
ReplaceNode command.
-------------------------------------------------------------*/
void MFGenericNode :: ReplaceNode (MediaObject *pOld, MediaObject *pNew)
{
	for (int i = m_array.GetLength (); --i >= 0; i)
		if (m_array [i] == pOld)
		{	m_array [i] = pNew;
			pNew -> SetContainer (this, TRUE, m_pParent);
			return;
		}
}

/*-------------------------------------------------------------
Clone a vector node field when a Proto is instantiated
-------------------------------------------------------------*/
NodeField *MFGenericNode :: Clone (NodeField *pField, MediaObject *pParent)
{
	if (pField == NULL)
		pField = new MFGenericNode;
	for (int i = 0; i < m_array.GetLength (); i++)
	{	if (m_array [i])
		{	MediaObject *pNode = m_array [i] -> Clone (pParent -> GetRoot ());
			((MFGenericNode *) pField) -> m_array += pNode;
			pNode -> SetContainer (pField, true, pParent);
		}
	}
	return pField;
}