/***********************************************************************
    This software module  was  originally  developed by Zvi  Lifshitz
    (Triton R&D Ltd.) and  Mikael Bourges-Sevenier (France Telecom CNET)
    in the course of development of the MPEG-4 Systems (ISO/IEC 14496-1)
    standard.  This software module is an implementation of a part of
    one or more MPEG-4 Systems (ISO/IEC 14496-1) tools  as  specified
    by  the  MPEG-4 Systems (ISO/IEC 14496-1) standard. ISO/IEC gives
    users of the MPEG-4 Systems (ISO/IEC 14496-1)  free  license  to
    this software module or modifications thereof for use in hardware
    or software products claiming conformance to the  MPEG-4  Systems
    (ISO/IEC 14496-1). Those intending to use this software module in
    hardware or software  products  are  advised  that  its  use  may
    infringe   existing  patents.  The  original  developer  of  this
    software module and his/her company, the subsequent  editors  and
    their  companies,  and  ISO/IEC have no liability for use of this
    software module or modifications thereof  in  an  implementation.
    Copyright  is not released for non MPEG-4 Systems (ISO/IEC 14496-
    1) conforming  products.  Triton R&D Ltd. and France  Telecom  CNET
    retain  full  right to use the code for their own purpose, assign
    or donate the code to a third party and to inhibit third  parties
    from  using  the  code  for  non MPEG-4 Systems (ISO/IEC 14496-1)
    conforming products. This copyright notice must  be  included  in
    all copies or derivative works. Copyright  1997.
************************************************************************/

/**************************************************************************
 * Project Im1Player
 * INTERNATIONAL STANDARD ORGANIZATION
 * ISO/IEC JTC1/SC29/WG11
 * 
 * SUBSYSTEM: BIFS Encoder/Decoder
 * FILE:      Nodes.h
 * AUTHOR:    Mikael Bourges-Sevenier (France Telecom CNET/DIH/HDM)
 * OVERVIEW:  Declaration of nodes and fields.
 * 
 * Comment:   Automatically generated on Wed Jul 04 19:40:31 PDT 2001
 *            at Melbourne, Australia
 *            by parsing the MPEG-4 templates.txt file.
 *            Contact: mikeseven@hotmail.com
 *
 **************************************************************************/
#include "float.h"
#include "StreamCo.h"
#include "NodeFld.h"
#include "bifs.h"
#include "Root.h"
#include "Conditional.h"

#if !defined(_Nodes_h)  // Sentry, use file only if it's not already included.
#define _Nodes_h

#pragma warning(disable: 4305)  // truncation from 'const double' to 'const float'

/* Currently, we have 120 nodes defined. */

class Anchor : public StreamConsumer
{
         BIFS_DECLARE_NODE

         eventIn MFNodeType<SF3DNode> addChildren;
         eventIn MFNodeType<SF3DNode> removeChildren;
         exposedField MFNodeType<SF3DNode> children;
         exposedField SFString description;
         exposedField MF<ZString> parameter;
         void OnaddChildren();
         void OnremoveChildren();

         public:
              Anchor() {
              }
};

class AnimationStream : public StreamConsumer
{
         BIFS_DECLARE_NODE

         exposedField SF<Bool> loop;
         exposedField SF<float> speed;
         exposedField SF<Time> startTime;
         exposedField SF<Time> stopTime;
         eventOut SF<Time> duration_changed;
         eventOut SF<Bool> isActive;

         public:
              AnimationStream() {
                   loop = FALSE;
                   speed = 1;
                   startTime = 0;
                   stopTime = 0;
              }
};

class Appearance : public MediaObject
{
         BIFS_DECLARE_NODE

         exposedField SFMaterialNode material;
         exposedField SFTextureNode texture;
         exposedField SFNode<class TextureTransform> textureTransform;

         public:
              Appearance() {
              }
};

class AudioBuffer : public MediaObject
{
         BIFS_DECLARE_NODE

         exposedField SF<Bool> loop;
         exposedField SF<float> pitch;
         exposedField SF<Time> startTime;
         exposedField SF<Time> stopTime;
         exposedField MFNodeType<SFAudioNode> children;
         exposedField SF<int> numChan;
         exposedField MF<int> phaseGroup;
         field SF<float> length;
         eventOut SF<Time> duration_changed;
         eventOut SF<Bool> isActive;

         public:
              AudioBuffer() {
                   loop = FALSE;
                   pitch = 1;
                   startTime = 0;
                   stopTime = 0;
                   numChan = 1;
                   phaseGroup += 1;
                   length = 0;
              }
};

class AudioClip : public StreamConsumer
{
         BIFS_DECLARE_NODE

         exposedField SFString description;
         exposedField SF<Bool> loop;
         exposedField SF<float> pitch;
         exposedField SF<Time> startTime;
         exposedField SF<Time> stopTime;
         eventOut SF<Time> duration_changed;
         eventOut SF<Bool> isActive;

         public:
              AudioClip() {
                   loop = FALSE;
                   pitch = 1;
                   startTime = 0;
                   stopTime = 0;
              }
};

class AudioDelay : public MediaObject
{
         BIFS_DECLARE_NODE

         eventIn MFNodeType<SFAudioNode> addChildren;
         eventIn MFNodeType<SFAudioNode> removeChildren;
         exposedField MFNodeType<SFAudioNode> children;
         exposedField SF<Time> delay;
         field SF<int> numChan;
         field MF<int> phaseGroup;
         void OnaddChildren();
         void OnremoveChildren();

         public:
              AudioDelay() {
                   delay = 0;
                   numChan = 1;
              }
};

class AudioFX : public MediaObject
{
         BIFS_DECLARE_NODE

         eventIn MFNodeType<SFAudioNode> addChildren;
         eventIn MFNodeType<SFAudioNode> removeChildren;
         exposedField MFNodeType<SFAudioNode> children;
         exposedField SFString orch;
         exposedField SFString score;
         exposedField MF<float> params;
         field SF<int> numChan;
         field MF<int> phaseGroup;
         void OnaddChildren();
         void OnremoveChildren();

         public:
              AudioFX() {
                   numChan = 1;
              }
};

class AudioMix : public MediaObject
{
         BIFS_DECLARE_NODE

         eventIn MFNodeType<SFAudioNode> addChildren;
         eventIn MFNodeType<SFAudioNode> removeChildren;
         exposedField MFNodeType<SFAudioNode> children;
         exposedField SF<int> numInputs;
         exposedField MF<float> matrix;
         field SF<int> numChan;
         field MF<int> phaseGroup;
         void OnaddChildren();
         void OnremoveChildren();

         public:
              AudioMix() {
                   numInputs = 1;
                   numChan = 1;
              }
};

class AudioSource : public StreamConsumer
{
         BIFS_DECLARE_NODE

         eventIn MFNodeType<SFAudioNode> addChildren;
         eventIn MFNodeType<SFAudioNode> removeChildren;
         exposedField MFNodeType<SFAudioNode> children;
         exposedField SF<float> pitch;
         exposedField SF<float> speed;
         exposedField SF<Time> startTime;
         exposedField SF<Time> stopTime;
         field SF<int> numChan;
         field MF<int> phaseGroup;
         void OnaddChildren();
         void OnremoveChildren();

         public:
              AudioSource() {
                   pitch = 1;
                   speed = 1;
                   startTime = 0;
                   stopTime = 0;
                   numChan = 1;
              }
};

class AudioSwitch : public MediaObject
{
         BIFS_DECLARE_NODE

         eventIn MFNodeType<SFAudioNode> addChildren;
         eventIn MFNodeType<SFAudioNode> removeChildren;
         exposedField MFNodeType<SFAudioNode> children;
         exposedField MF<int> whichChoice;
         field SF<int> numChan;
         field MF<int> phaseGroup;
         void OnaddChildren();
         void OnremoveChildren();

         public:
              AudioSwitch() {
                   numChan = 1;
              }
};

class Background : public MediaObject
{
         BIFS_DECLARE_NODE

         eventIn SF<Bool> set_bind;
         exposedField MF<float> groundAngle;
         exposedField MF<Color3f> groundColor;
         exposedField MF<Url> backUrl;
         exposedField MF<Url> bottomUrl;
         exposedField MF<Url> frontUrl;
         exposedField MF<Url> leftUrl;
         exposedField MF<Url> rightUrl;
         exposedField MF<Url> topUrl;
         exposedField MF<float> skyAngle;
         exposedField MF<Color3f> skyColor;
         eventOut SF<Bool> isBound;
         void Onset_bind();

         public:
              Background() {
                   skyColor += 0;
                   skyColor += 0;
                   skyColor += 0;
              }
};

class Background2D : public StreamConsumer
{
         BIFS_DECLARE_NODE

         eventIn SF<Bool> set_bind;
         exposedField SF<Color3f> backColor;
         eventOut SF<Bool> isBound;
         void Onset_bind();

         public:
              Background2D() {
                   backColor = Color3f(0, 0, 0);
              }
};

class Billboard : public MediaObject
{
         BIFS_DECLARE_NODE

         eventIn MFNodeType<SF3DNode> addChildren;
         eventIn MFNodeType<SF3DNode> removeChildren;
         exposedField MFNodeType<SF3DNode> children;
         exposedField SF<Vec3f> axisOfRotation;
         void OnaddChildren();
         void OnremoveChildren();

         public:
              Billboard() {
                   axisOfRotation = Vec3f(0, 1, 0);
              }
};

class Bitmap : public MediaObject
{
         BIFS_DECLARE_NODE

         exposedField SF<Vec2f> scale;

         public:
              Bitmap() {
                   scale = Vec2f(-1, -1);
              }
};

class Box : public MediaObject
{
         BIFS_DECLARE_NODE

         field SF<Vec3f> size;

         public:
              Box() {
                   size = Vec3f(2, 2, 2);
              }
};

class Circle : public MediaObject
{
         BIFS_DECLARE_NODE

         exposedField SF<float> radius;

         public:
              Circle() {
                   radius = 1;
              }
};

class Collision : public MediaObject
{
         BIFS_DECLARE_NODE

         eventIn MFNodeType<SF3DNode> addChildren;
         eventIn MFNodeType<SF3DNode> removeChildren;
         exposedField MFNodeType<SF3DNode> children;
         exposedField SF<Bool> collide;
         field SF3DNode proxy;
         eventOut SF<Time> collideTime;
         void OnaddChildren();
         void OnremoveChildren();

         public:
              Collision() {
                   collide = TRUE;
              }
};

class Color : public MediaObject
{
         BIFS_DECLARE_NODE

         exposedField MF<Color3f> color;

         public:
              Color() {
              }
};

class ColorInterpolator : public MediaObject
{
         BIFS_DECLARE_NODE

         eventIn SF<float> set_fraction;
         exposedField MF<float> key;
         exposedField MF<Color3f> keyValue;
         eventOut SF<Color3f> value_changed;
         void Onset_fraction();

         public:
              ColorInterpolator() {
              }
};

class CompositeTexture2D : public MediaObject
{
         BIFS_DECLARE_NODE

         eventIn MFNodeType<SF2DNode> addChildren;
         eventIn MFNodeType<SF2DNode> removeChildren;
         exposedField MFNodeType<SF2DNode> children;
         exposedField SF<int> pixelWidth;
         exposedField SF<int> pixelHeight;
         exposedField SFNode<class Background2D> background;
         exposedField SFViewportNode viewport;
         void OnaddChildren();
         void OnremoveChildren();

         public:
              CompositeTexture2D() {
                   pixelWidth = -1;
                   pixelHeight = -1;
              }
};

class CompositeTexture3D : public MediaObject
{
         BIFS_DECLARE_NODE

         eventIn MFNodeType<SF3DNode> addChildren;
         eventIn MFNodeType<SF3DNode> removeChildren;
         exposedField MFNodeType<SF3DNode> children;
         exposedField SF<int> pixelWidth;
         exposedField SF<int> pixelHeight;
         exposedField SFNode<class Background> background;
         exposedField SFNode<class Fog> fog;
         exposedField SFNode<class NavigationInfo> navigationInfo;
         exposedField SFNode<class Viewpoint> viewpoint;
         void OnaddChildren();
         void OnremoveChildren();

         public:
              CompositeTexture3D() {
                   pixelWidth = -1;
                   pixelHeight = -1;
              }
};

class Conditional : public MediaObject
{
         BIFS_DECLARE_NODE

         eventIn SF<Bool> activate;
         eventIn SF<Bool> reverseActivate;
         exposedField SFConditionalString buffer;
         eventOut SF<Bool> isActive;
         void Onactivate();
         void OnreverseActivate();

         public:
              Conditional() {
                   reverseActivate = FALSE;
              }
};

class Cone : public MediaObject
{
         BIFS_DECLARE_NODE

         field SF<float> bottomRadius;
         field SF<float> height;
         field SF<Bool> side;
         field SF<Bool> bottom;

         public:
              Cone() {
                   bottomRadius = 1;
                   height = 2;
                   side = TRUE;
                   bottom = TRUE;
              }
};

class Coordinate : public MediaObject
{
         BIFS_DECLARE_NODE

         exposedField MF<Vec3f> point;

         public:
              Coordinate() {
              }
};

class Coordinate2D : public MediaObject
{
         BIFS_DECLARE_NODE

         exposedField MF<Vec2f> point;

         public:
              Coordinate2D() {
              }
};

class CoordinateInterpolator : public MediaObject
{
         BIFS_DECLARE_NODE

         eventIn SF<float> set_fraction;
         exposedField MF<float> key;
         exposedField MF<Vec3f> keyValue;
         eventOut MF<Vec3f> value_changed;
         void Onset_fraction();

         public:
              CoordinateInterpolator() {
              }
};

class CoordinateInterpolator2D : public MediaObject
{
         BIFS_DECLARE_NODE

         eventIn SF<float> set_fraction;
         exposedField MF<float> key;
         exposedField MF<Vec2f> keyValue;
         eventOut MF<Vec2f> value_changed;
         void Onset_fraction();

         public:
              CoordinateInterpolator2D() {
              }
};

class Curve2D : public MediaObject
{
         BIFS_DECLARE_NODE

         exposedField SFNode<class Coordinate2D> point;
         exposedField SF<float> fineness;
         exposedField MF<int> type;

         public:
              Curve2D() {
                   fineness = 0.5;
              }
};

class Cylinder : public MediaObject
{
         BIFS_DECLARE_NODE

         field SF<Bool> bottom;
         field SF<float> height;
         field SF<float> radius;
         field SF<Bool> side;
         field SF<Bool> top;

         public:
              Cylinder() {
                   bottom = TRUE;
                   height = 2;
                   radius = 1;
                   side = TRUE;
                   top = TRUE;
              }
};

class CylinderSensor : public MediaObject
{
         BIFS_DECLARE_NODE

         exposedField SF<Bool> autoOffset;
         exposedField SF<float> diskAngle;
         exposedField SF<Bool> enabled;
         exposedField SF<float> maxAngle;
         exposedField SF<float> minAngle;
         exposedField SF<float> offset;
         eventOut SF<Bool> isActive;
         eventOut SF<Rotation> rotation_changed;
         eventOut SF<Vec3f> trackPoint_changed;

         public:
              CylinderSensor() {
                   autoOffset = TRUE;
                   diskAngle = 0.262;
                   enabled = TRUE;
                   maxAngle = -1;
                   minAngle = 0;
                   offset = 0;
              }
};

class DirectionalLight : public MediaObject
{
         BIFS_DECLARE_NODE

         exposedField SF<float> ambientIntensity;
         exposedField SF<Color3f> color;
         exposedField SF<Vec3f> direction;
         exposedField SF<float> intensity;
         exposedField SF<Bool> on;

         public:
              DirectionalLight() {
                   ambientIntensity = 0;
                   color = Color3f(1, 1, 1);
                   direction = Vec3f(0, 0, -1);
                   intensity = 1;
                   on = TRUE;
              }
};

class DiscSensor : public MediaObject
{
         BIFS_DECLARE_NODE

         exposedField SF<Bool> autoOffset;
         exposedField SF<Bool> enabled;
         exposedField SF<float> maxAngle;
         exposedField SF<float> minAngle;
         exposedField SF<float> offset;
         eventOut SF<Bool> isActive;
         eventOut SF<float> rotation_changed;
         eventOut SF<Vec2f> trackPoint_changed;

         public:
              DiscSensor() {
                   autoOffset = TRUE;
                   enabled = TRUE;
                   maxAngle = -1;
                   minAngle = -1;
                   offset = 0;
              }
};

class ElevationGrid : public MediaObject
{
         BIFS_DECLARE_NODE

         eventIn MF<float> set_height;
         exposedField SFNode<class Color> color;
         exposedField SFNode<class Normal> normal;
         exposedField SFNode<class TextureCoordinate> texCoord;
         field MF<float> height;
         field SF<Bool> ccw;
         field SF<Bool> colorPerVertex;
         field SF<float> creaseAngle;
         field SF<Bool> normalPerVertex;
         field SF<Bool> solid;
         field SF<int> xDimension;
         field SF<float> xSpacing;
         field SF<int> zDimension;
         field SF<float> zSpacing;
         void Onset_height();

         public:
              ElevationGrid() {
                   ccw = TRUE;
                   colorPerVertex = TRUE;
                   creaseAngle = 0;
                   normalPerVertex = TRUE;
                   solid = TRUE;
                   xDimension = 0;
                   xSpacing = 1;
                   zDimension = 0;
                   zSpacing = 1;
              }
};

class Expression : public MediaObject
{
         BIFS_DECLARE_NODE

         exposedField SF<int> expression_select1;
         exposedField SF<int> expression_intensity1;
         exposedField SF<int> expression_select2;
         exposedField SF<int> expression_intensity2;
         exposedField SF<Bool> init_face;
         exposedField SF<Bool> expression_def;

         public:
              Expression() {
                   expression_select1 = 0;
                   expression_intensity1 = 0;
                   expression_select2 = 0;
                   expression_intensity2 = 0;
                   init_face = FALSE;
                   expression_def = FALSE;
              }
};

class Extrusion : public MediaObject
{
         BIFS_DECLARE_NODE

         eventIn MF<Vec2f> set_crossSection;
         eventIn MF<Rotation> set_orientation;
         eventIn MF<Vec2f> set_scale;
         eventIn MF<Vec3f> set_spine;
         field SF<Bool> beginCap;
         field SF<Bool> ccw;
         field SF<Bool> convex;
         field SF<float> creaseAngle;
         field MF<Vec2f> crossSection;
         field SF<Bool> endCap;
         field MF<Rotation> orientation;
         field MF<Vec2f> scale;
         field SF<Bool> solid;
         field MF<Vec3f> spine;
         void Onset_crossSection();
         void Onset_orientation();
         void Onset_scale();
         void Onset_spine();

         public:
              Extrusion() {
                   beginCap = TRUE;
                   ccw = TRUE;
                   convex = TRUE;
                   creaseAngle = 0;
                   crossSection += 1;
                   crossSection += 1;
                   crossSection += -1;
                   crossSection += -1;
                   crossSection += -1;
                   crossSection += -1;
                   crossSection += 1;
                   crossSection += 1;
                   crossSection += 1;
                   crossSection += 1;
                   endCap = TRUE;
                   orientation += 0;
                   orientation += 0;
                   orientation += 1;
                   orientation += 0;
                   scale += 1;
                   scale += 1;
                   solid = TRUE;
                   spine += 0;
                   spine += 0;
                   spine += 0;
                   spine += 1;
                   spine += 1;
                   spine += 0;
              }
};

class Face : public MediaObject
{
         BIFS_DECLARE_NODE

         exposedField SFNode<class FAP> fap;
         exposedField SFNode<class FDP> fdp;
         exposedField SFNode<class FIT> fit;
         exposedField SFAudioNode ttsSource;
         exposedField MFNodeType<SF3DNode> renderedFace;

         public:
              Face() {
              }
};

class FaceDefMesh : public MediaObject
{
         BIFS_DECLARE_NODE

         field SF3DNode faceSceneGraphNode;
         field MF<int> intervalBorders;
         field MF<int> coordIndex;
         field MF<Vec3f> displacements;

         public:
              FaceDefMesh() {
              }
};

class FaceDefTables : public MediaObject
{
         BIFS_DECLARE_NODE

         field SF<int> fapID;
         field SF<int> highLevelSelect;
         exposedField MFNode<class FaceDefMesh> faceDefMesh;
         exposedField MFNode<class FaceDefTransform> faceDefTransform;

         public:
              FaceDefTables() {
                   fapID = 1;
                   highLevelSelect = 1;
              }
};

class FaceDefTransform : public MediaObject
{
         BIFS_DECLARE_NODE

         field SF3DNode faceSceneGraphNode;
         field SF<int> fieldId;
         field SF<Rotation> rotationDef;
         field SF<Vec3f> scaleDef;
         field SF<Vec3f> translationDef;

         public:
              FaceDefTransform() {
                   fieldId = 1;
                   rotationDef = Rotation(0, 0, 1, 0);
                   scaleDef = Vec3f(1, 1, 1);
                   translationDef = Vec3f(0, 0, 0);
              }
};

class FAP : public MediaObject
{
         BIFS_DECLARE_NODE

         exposedField SFNode<class Viseme> viseme;
         exposedField SFNode<class Expression> expression;
         exposedField SF<int> open_jaw;
         exposedField SF<int> lower_t_midlip;
         exposedField SF<int> raise_b_midlip;
         exposedField SF<int> stretch_l_corner;
         exposedField SF<int> stretch_r_corner;
         exposedField SF<int> lower_t_lip_lm;
         exposedField SF<int> lower_t_lip_rm;
         exposedField SF<int> lower_b_lip_lm;
         exposedField SF<int> lower_b_lip_rm;
         exposedField SF<int> raise_l_cornerlip;
         exposedField SF<int> raise_r_cornerlip;
         exposedField SF<int> thrust_jaw;
         exposedField SF<int> shift_jaw;
         exposedField SF<int> push_b_lip;
         exposedField SF<int> push_t_lip;
         exposedField SF<int> depress_chin;
         exposedField SF<int> close_t_l_eyelid;
         exposedField SF<int> close_t_r_eyelid;
         exposedField SF<int> close_b_l_eyelid;
         exposedField SF<int> close_b_r_eyelid;
         exposedField SF<int> yaw_l_eyeball;
         exposedField SF<int> yaw_r_eyeball;
         exposedField SF<int> pitch_l_eyeball;
         exposedField SF<int> pitch_r_eyeball;
         exposedField SF<int> thrust_l_eyeball;
         exposedField SF<int> thrust_r_eyeball;
         exposedField SF<int> dilate_l_pupil;
         exposedField SF<int> dilate_r_pupil;
         exposedField SF<int> raise_l_i_eyebrow;
         exposedField SF<int> raise_r_i_eyebrow;
         exposedField SF<int> raise_l_m_eyebrow;
         exposedField SF<int> raise_r_m_eyebrow;
         exposedField SF<int> raise_l_o_eyebrow;
         exposedField SF<int> raise_r_o_eyebrow;
         exposedField SF<int> squeeze_l_eyebrow;
         exposedField SF<int> squeeze_r_eyebrow;
         exposedField SF<int> puff_l_cheek;
         exposedField SF<int> puff_r_cheek;
         exposedField SF<int> lift_l_cheek;
         exposedField SF<int> lift_r_cheek;
         exposedField SF<int> shift_tongue_tip;
         exposedField SF<int> raise_tongue_tip;
         exposedField SF<int> thrust_tongue_tip;
         exposedField SF<int> raise_tongue;
         exposedField SF<int> tongue_roll;
         exposedField SF<int> head_pitch;
         exposedField SF<int> head_yaw;
         exposedField SF<int> head_roll;
         exposedField SF<int> lower_t_midlip_o;
         exposedField SF<int> raise_b_midlip_o;
         exposedField SF<int> stretch_l_cornerlip;
         exposedField SF<int> stretch_r_cornerlip;
         exposedField SF<int> lower_t_lip_lm_o;
         exposedField SF<int> lower_t_lip_rm_o;
         exposedField SF<int> raise_b_lip_lm_o;
         exposedField SF<int> raise_b_lip_rm_o;
         exposedField SF<int> raise_l_cornerlip_o;
         exposedField SF<int> raise_r_cornerlip_o;
         exposedField SF<int> stretch_l_nose;
         exposedField SF<int> stretch_r_nose;
         exposedField SF<int> raise_nose;
         exposedField SF<int> bend_nose;
         exposedField SF<int> raise_l_ear;
         exposedField SF<int> raise_r_ear;
         exposedField SF<int> pull_l_ear;
         exposedField SF<int> pull_r_ear;

         public:
              FAP() {
                   open_jaw = +INT_MAX;
                   lower_t_midlip = +INT_MAX;
                   raise_b_midlip = +INT_MAX;
                   stretch_l_corner = +INT_MAX;
                   stretch_r_corner = +INT_MAX;
                   lower_t_lip_lm = +INT_MAX;
                   lower_t_lip_rm = +INT_MAX;
                   lower_b_lip_lm = +INT_MAX;
                   lower_b_lip_rm = +INT_MAX;
                   raise_l_cornerlip = +INT_MAX;
                   raise_r_cornerlip = +INT_MAX;
                   thrust_jaw = +INT_MAX;
                   shift_jaw = +INT_MAX;
                   push_b_lip = +INT_MAX;
                   push_t_lip = +INT_MAX;
                   depress_chin = +INT_MAX;
                   close_t_l_eyelid = +INT_MAX;
                   close_t_r_eyelid = +INT_MAX;
                   close_b_l_eyelid = +INT_MAX;
                   close_b_r_eyelid = +INT_MAX;
                   yaw_l_eyeball = +INT_MAX;
                   yaw_r_eyeball = +INT_MAX;
                   pitch_l_eyeball = +INT_MAX;
                   pitch_r_eyeball = +INT_MAX;
                   thrust_l_eyeball = +INT_MAX;
                   thrust_r_eyeball = +INT_MAX;
                   dilate_l_pupil = +INT_MAX;
                   dilate_r_pupil = +INT_MAX;
                   raise_l_i_eyebrow = +INT_MAX;
                   raise_r_i_eyebrow = +INT_MAX;
                   raise_l_m_eyebrow = +INT_MAX;
                   raise_r_m_eyebrow = +INT_MAX;
                   raise_l_o_eyebrow = +INT_MAX;
                   raise_r_o_eyebrow = +INT_MAX;
                   squeeze_l_eyebrow = +INT_MAX;
                   squeeze_r_eyebrow = +INT_MAX;
                   puff_l_cheek = +INT_MAX;
                   puff_r_cheek = +INT_MAX;
                   lift_l_cheek = +INT_MAX;
                   lift_r_cheek = +INT_MAX;
                   shift_tongue_tip = +INT_MAX;
                   raise_tongue_tip = +INT_MAX;
                   thrust_tongue_tip = +INT_MAX;
                   raise_tongue = +INT_MAX;
                   tongue_roll = +INT_MAX;
                   head_pitch = +INT_MAX;
                   head_yaw = +INT_MAX;
                   head_roll = +INT_MAX;
                   lower_t_midlip_o = +INT_MAX;
                   raise_b_midlip_o = +INT_MAX;
                   stretch_l_cornerlip = +INT_MAX;
                   stretch_r_cornerlip = +INT_MAX;
                   lower_t_lip_lm_o = +INT_MAX;
                   lower_t_lip_rm_o = +INT_MAX;
                   raise_b_lip_lm_o = +INT_MAX;
                   raise_b_lip_rm_o = +INT_MAX;
                   raise_l_cornerlip_o = +INT_MAX;
                   raise_r_cornerlip_o = +INT_MAX;
                   stretch_l_nose = +INT_MAX;
                   stretch_r_nose = +INT_MAX;
                   raise_nose = +INT_MAX;
                   bend_nose = +INT_MAX;
                   raise_l_ear = +INT_MAX;
                   raise_r_ear = +INT_MAX;
                   pull_l_ear = +INT_MAX;
                   pull_r_ear = +INT_MAX;
              }
};

class FDP : public MediaObject
{
         BIFS_DECLARE_NODE

         exposedField SFNode<class Coordinate> featurePointsCoord;
         exposedField SFNode<class TextureCoordinate> textureCoord;
         exposedField MFNode<class FaceDefTables> faceDefTables;
         exposedField MFNodeType<SF3DNode> faceSceneGraph;
         field SF<Bool> useOrthoTexture;

         public:
              FDP() {
                   useOrthoTexture = FALSE;
              }
};

class FIT : public MediaObject
{
         BIFS_DECLARE_NODE

         exposedField MF<int> FAPs;
         exposedField MF<int> Graph;
         exposedField MF<int> numeratorExp;
         exposedField MF<int> denominatorExp;
         exposedField MF<int> numeratorImpulse;
         exposedField MF<int> numeratorTerms;
         exposedField MF<int> denominatorTerms;
         exposedField MF<float> numeratorCoefs;
         exposedField MF<float> denominatorCoefs;

         public:
              FIT() {
              }
};

class Fog : public MediaObject
{
         BIFS_DECLARE_NODE

         exposedField SF<Color3f> color;
         exposedField SFString fogType;
         exposedField SF<float> visibilityRange;
         eventIn SF<Bool> set_bind;
         eventOut SF<Bool> isBound;
         void Onset_bind();

         public:
              Fog() {
                   color = Color3f(1, 1, 1);
                   fogType = ZString("LINEAR");
                   visibilityRange = 0;
              }
};

class FontStyle : public MediaObject
{
         BIFS_DECLARE_NODE

         field MF<ZString> family;
         field SF<Bool> horizontal;
         field MF<ZString> justify;
         field SFString language;
         field SF<Bool> leftToRight;
         field SF<float> size;
         field SF<float> spacing;
         field SFString style;
         field SF<Bool> topToBottom;

         public:
              FontStyle() {
                   family += ZString("SERIF");
                   horizontal = TRUE;
                   justify += ZString("BEGIN");
                   leftToRight = TRUE;
                   size = 1;
                   spacing = 1;
                   style = ZString("PLAIN");
                   topToBottom = TRUE;
              }
};

class Form : public MediaObject
{
         BIFS_DECLARE_NODE

         eventIn MFNodeType<SF2DNode> addChildren;
         eventIn MFNodeType<SF2DNode> removeChildren;
         exposedField MFNodeType<SF2DNode> children;
         exposedField SF<Vec2f> size;
         exposedField MF<int> groups;
         exposedField MF<ZString> constraints;
         exposedField MF<int> groupsIndex;
         void OnaddChildren();
         void OnremoveChildren();

         public:
              Form() {
                   size = Vec2f(-1, -1);
              }
};

class Group : public Root
{
         BIFS_DECLARE_NODE

         eventIn MFNodeType<SF3DNode> addChildren;
         eventIn MFNodeType<SF3DNode> removeChildren;
         exposedField MFNodeType<SF3DNode> children;
         void OnaddChildren();
         void OnremoveChildren();

         public:
              Group() {
              }
};

class ImageTexture : public StreamConsumer
{
         BIFS_DECLARE_NODE

         field SF<Bool> repeatS;
         field SF<Bool> repeatT;

         public:
              ImageTexture() {
                   repeatS = TRUE;
                   repeatT = TRUE;
              }
};

class IndexedFaceSet : public MediaObject
{
         BIFS_DECLARE_NODE

         eventIn MF<int> set_colorIndex;
         eventIn MF<int> set_coordIndex;
         eventIn MF<int> set_normalIndex;
         eventIn MF<int> set_texCoordIndex;
         exposedField SFNode<class Color> color;
         exposedField SFNode<class Coordinate> coord;
         exposedField SFNode<class Normal> normal;
         exposedField SFNode<class TextureCoordinate> texCoord;
         field SF<Bool> ccw;
         field MF<int> colorIndex;
         field SF<Bool> colorPerVertex;
         field SF<Bool> convex;
         field MF<int> coordIndex;
         field SF<float> creaseAngle;
         field MF<int> normalIndex;
         field SF<Bool> normalPerVertex;
         field SF<Bool> solid;
         field MF<int> texCoordIndex;
         void Onset_colorIndex();
         void Onset_coordIndex();
         void Onset_normalIndex();
         void Onset_texCoordIndex();

         public:
              IndexedFaceSet() {
                   ccw = TRUE;
                   colorPerVertex = TRUE;
                   convex = TRUE;
                   creaseAngle = 0;
                   normalPerVertex = TRUE;
                   solid = TRUE;
              }
};

class IndexedFaceSet2D : public MediaObject
{
         BIFS_DECLARE_NODE

         eventIn MF<int> set_colorIndex;
         eventIn MF<int> set_coordIndex;
         eventIn MF<int> set_texCoordIndex;
         exposedField SFNode<class Color> color;
         exposedField SFNode<class Coordinate2D> coord;
         exposedField SFNode<class TextureCoordinate> texCoord;
         field MF<int> colorIndex;
         field SF<Bool> colorPerVertex;
         field SF<Bool> convex;
         field MF<int> coordIndex;
         field MF<int> texCoordIndex;
         void Onset_colorIndex();
         void Onset_coordIndex();
         void Onset_texCoordIndex();

         public:
              IndexedFaceSet2D() {
                   colorPerVertex = TRUE;
                   convex = TRUE;
              }
};

class IndexedLineSet : public MediaObject
{
         BIFS_DECLARE_NODE

         eventIn MF<int> set_colorIndex;
         eventIn MF<int> set_coordIndex;
         exposedField SFNode<class Color> color;
         exposedField SFNode<class Coordinate> coord;
         field MF<int> colorIndex;
         field SF<Bool> colorPerVertex;
         field MF<int> coordIndex;
         void Onset_colorIndex();
         void Onset_coordIndex();

         public:
              IndexedLineSet() {
                   colorPerVertex = TRUE;
              }
};

class IndexedLineSet2D : public MediaObject
{
         BIFS_DECLARE_NODE

         eventIn MF<int> set_colorIndex;
         eventIn MF<int> set_coordIndex;
         exposedField SFNode<class Color> color;
         exposedField SFNode<class Coordinate2D> coord;
         field MF<int> colorIndex;
         field SF<Bool> colorPerVertex;
         field MF<int> coordIndex;
         void Onset_colorIndex();
         void Onset_coordIndex();

         public:
              IndexedLineSet2D() {
                   colorPerVertex = TRUE;
              }
};

class Inline : public StreamConsumer
{
         BIFS_DECLARE_NODE


         public:
              Inline() {
              }
};

class LOD : public MediaObject
{
         BIFS_DECLARE_NODE

         exposedField MFNodeType<SF3DNode> level;
         field SF<Vec3f> center;
         field MF<float> range;

         public:
              LOD() {
                   center = Vec3f(0, 0, 0);
              }
};

class Layer2D : public Root
{
         BIFS_DECLARE_NODE

         eventIn MFNodeType<SF2DNode> addChildren;
         eventIn MFNodeType<SF2DNode> removeChildren;
         exposedField MFNodeType<SF2DNode> children;
         exposedField SF<Vec2f> size;
         exposedField SFNode<class Background2D> background;
         exposedField SFViewportNode viewport;
         void OnaddChildren();
         void OnremoveChildren();

         public:
              Layer2D() {
                   size = Vec2f(-1, -1);
              }
};

class Layer3D : public Root
{
         BIFS_DECLARE_NODE

         eventIn MFNodeType<SF3DNode> addChildren;
         eventIn MFNodeType<SF3DNode> removeChildren;
         exposedField MFNodeType<SF3DNode> children;
         exposedField SF<Vec2f> size;
         exposedField SFNode<class Background> background;
         exposedField SFNode<class Fog> fog;
         exposedField SFNode<class NavigationInfo> navigationInfo;
         exposedField SFNode<class Viewpoint> viewpoint;
         void OnaddChildren();
         void OnremoveChildren();

         public:
              Layer3D() {
                   size = Vec2f(-1, -1);
              }
};

class Layout : public MediaObject
{
         BIFS_DECLARE_NODE

         eventIn MFNodeType<SF2DNode> addChildren;
         eventIn MFNodeType<SF2DNode> removeChildren;
         exposedField MFNodeType<SF2DNode> children;
         exposedField SF<Bool> wrap;
         exposedField SF<Vec2f> size;
         exposedField SF<Bool> horizontal;
         exposedField MF<ZString> justify;
         exposedField SF<Bool> leftToRight;
         exposedField SF<Bool> topToBottom;
         exposedField SF<float> spacing;
         exposedField SF<Bool> smoothScroll;
         exposedField SF<Bool> loop;
         exposedField SF<Bool> scrollVertical;
         exposedField SF<float> scrollRate;
         void OnaddChildren();
         void OnremoveChildren();

         public:
              Layout() {
                   wrap = FALSE;
                   size = Vec2f(-1, -1);
                   horizontal = TRUE;
                   justify += ZString("BEGIN");
                   leftToRight = TRUE;
                   topToBottom = TRUE;
                   spacing = 1;
                   smoothScroll = FALSE;
                   loop = FALSE;
                   scrollVertical = TRUE;
                   scrollRate = 0;
              }
};

class LineProperties : public MediaObject
{
         BIFS_DECLARE_NODE

         exposedField SF<Color3f> lineColor;
         exposedField SF<int> lineStyle;
         exposedField SF<float> width;

         public:
              LineProperties() {
                   lineColor = Color3f(0, 0, 0);
                   lineStyle = 0;
                   width = 1;
              }
};

class ListeningPoint : public MediaObject
{
         BIFS_DECLARE_NODE

         eventIn SF<Bool> set_bind;
         exposedField SF<Bool> jump;
         exposedField SF<Rotation> orientation;
         exposedField SF<Vec3f> position;
         field SFString description;
         eventOut SF<Time> bindTime;
         eventOut SF<Bool> isBound;
         void Onset_bind();

         public:
              ListeningPoint() {
                   jump = TRUE;
                   orientation = Rotation(0, 0, 1, 0);
                   position = Vec3f(0, 0, 10);
              }
};

class Material : public MediaObject
{
         BIFS_DECLARE_NODE

         exposedField SF<float> ambientIntensity;
         exposedField SF<Color3f> diffuseColor;
         exposedField SF<Color3f> emissiveColor;
         exposedField SF<float> shininess;
         exposedField SF<Color3f> specularColor;
         exposedField SF<float> transparency;

         public:
              Material() {
                   ambientIntensity = 0.2;
                   diffuseColor = Color3f(0.8, 0.8, 0.8);
                   emissiveColor = Color3f(0, 0, 0);
                   shininess = 0.2;
                   specularColor = Color3f(0, 0, 0);
                   transparency = 0;
              }
};

class Material2D : public MediaObject
{
         BIFS_DECLARE_NODE

         exposedField SF<Color3f> emissiveColor;
         exposedField SF<Bool> filled;
         exposedField SFNode<class LineProperties> lineProps;
         exposedField SF<float> transparency;

         public:
              Material2D() {
                   emissiveColor = Color3f(0.8, 0.8, 0.8);
                   filled = FALSE;
                   transparency = 0;
              }
};

class MovieTexture : public StreamConsumer
{
         BIFS_DECLARE_NODE

         exposedField SF<Bool> loop;
         exposedField SF<float> speed;
         exposedField SF<Time> startTime;
         exposedField SF<Time> stopTime;
         field SF<Bool> repeatS;
         field SF<Bool> repeatT;
         eventOut SF<Time> duration_changed;
         eventOut SF<Bool> isActive;

         public:
              MovieTexture() {
                   loop = FALSE;
                   speed = 1;
                   startTime = 0;
                   stopTime = 0;
                   repeatS = TRUE;
                   repeatT = TRUE;
              }
};

class NavigationInfo : public MediaObject
{
         BIFS_DECLARE_NODE

         eventIn SF<Bool> set_bind;
         exposedField MF<float> avatarSize;
         exposedField SF<Bool> headlight;
         exposedField SF<float> speed;
         exposedField MF<ZString> type;
         exposedField SF<float> visibilityLimit;
         eventOut SF<Bool> isBound;
         void Onset_bind();

         public:
              NavigationInfo() {
                   avatarSize += 0.25;
                   avatarSize += 0.75;
                   avatarSize += 0.75;
                   headlight = TRUE;
                   speed = 1;
                   type += ZString("WALK");
                   visibilityLimit = 0;
              }
};

class Normal : public MediaObject
{
         BIFS_DECLARE_NODE

         exposedField MF<Vec3f> vector;

         public:
              Normal() {
              }
};

class NormalInterpolator : public MediaObject
{
         BIFS_DECLARE_NODE

         eventIn SF<float> set_fraction;
         exposedField MF<float> key;
         exposedField MF<Vec3f> keyValue;
         eventOut MF<Vec3f> value_changed;
         void Onset_fraction();

         public:
              NormalInterpolator() {
              }
};

class OrderedGroup : public Root
{
         BIFS_DECLARE_NODE

         eventIn MFNodeType<SF3DNode> addChildren;
         eventIn MFNodeType<SF3DNode> removeChildren;
         exposedField MFNodeType<SF3DNode> children;
         exposedField MF<float> order;
         void OnaddChildren();
         void OnremoveChildren();

         public:
              OrderedGroup() {
              }
};

class OrientationInterpolator : public MediaObject
{
         BIFS_DECLARE_NODE

         eventIn SF<float> set_fraction;
         exposedField MF<float> key;
         exposedField MF<Rotation> keyValue;
         eventOut SF<Rotation> value_changed;
         void Onset_fraction();

         public:
              OrientationInterpolator() {
              }
};

class PixelTexture : public MediaObject
{
         BIFS_DECLARE_NODE

         exposedField SF<Image> image;
         field SF<Bool> repeatS;
         field SF<Bool> repeatT;

         public:
              PixelTexture() {
                   image = Image(0, 0, 0);
                   repeatS = TRUE;
                   repeatT = TRUE;
              }
};

class PlaneSensor : public MediaObject
{
         BIFS_DECLARE_NODE

         exposedField SF<Bool> autoOffset;
         exposedField SF<Bool> enabled;
         exposedField SF<Vec2f> maxPosition;
         exposedField SF<Vec2f> minPosition;
         exposedField SF<Vec3f> offset;
         eventOut SF<Bool> isActive;
         eventOut SF<Vec3f> trackPoint_changed;
         eventOut SF<Vec3f> translation_changed;

         public:
              PlaneSensor() {
                   autoOffset = TRUE;
                   enabled = TRUE;
                   maxPosition = Vec2f(-1, -1);
                   minPosition = Vec2f(0, 0);
                   offset = Vec3f(0, 0, 0);
              }
};

class PlaneSensor2D : public MediaObject
{
         BIFS_DECLARE_NODE

         exposedField SF<Bool> autoOffset;
         exposedField SF<Bool> enabled;
         exposedField SF<Vec2f> maxPosition;
         exposedField SF<Vec2f> minPosition;
         exposedField SF<Vec2f> offset;
         eventOut SF<Bool> isActive;
         eventOut SF<Vec2f> trackPoint_changed;
         eventOut SF<Vec2f> translation_changed;

         public:
              PlaneSensor2D() {
                   autoOffset = TRUE;
                   enabled = TRUE;
                   maxPosition = Vec2f(0, 0);
                   minPosition = Vec2f(0, 0);
                   offset = Vec2f(0, 0);
              }
};

class PointLight : public MediaObject
{
         BIFS_DECLARE_NODE

         exposedField SF<float> ambientIntensity;
         exposedField SF<Vec3f> attenuation;
         exposedField SF<Color3f> color;
         exposedField SF<float> intensity;
         exposedField SF<Vec3f> location;
         exposedField SF<Bool> on;
         exposedField SF<float> radius;

         public:
              PointLight() {
                   ambientIntensity = 0;
                   attenuation = Vec3f(1, 0, 0);
                   color = Color3f(1, 1, 1);
                   intensity = 1;
                   location = Vec3f(0, 0, 0);
                   on = TRUE;
                   radius = 100;
              }
};

class PointSet : public MediaObject
{
         BIFS_DECLARE_NODE

         exposedField SFNode<class Color> color;
         exposedField SFNode<class Coordinate> coord;

         public:
              PointSet() {
              }
};

class PointSet2D : public MediaObject
{
         BIFS_DECLARE_NODE

         exposedField SFNode<class Color> color;
         exposedField SFNode<class Coordinate2D> coord;

         public:
              PointSet2D() {
              }
};

class PositionInterpolator : public MediaObject
{
         BIFS_DECLARE_NODE

         eventIn SF<float> set_fraction;
         exposedField MF<float> key;
         exposedField MF<Vec3f> keyValue;
         eventOut SF<Vec3f> value_changed;
         void Onset_fraction();

         public:
              PositionInterpolator() {
              }
};

class PositionInterpolator2D : public MediaObject
{
         BIFS_DECLARE_NODE

         eventIn SF<float> set_fraction;
         exposedField MF<float> key;
         exposedField MF<Vec2f> keyValue;
         eventOut SF<Vec2f> value_changed;
         void Onset_fraction();

         public:
              PositionInterpolator2D() {
              }
};

class ProximitySensor2D : public MediaObject
{
         BIFS_DECLARE_NODE

         exposedField SF<Vec2f> center;
         exposedField SF<Vec2f> size;
         exposedField SF<Bool> enabled;
         eventOut SF<Bool> isActive;
         eventOut SF<Vec2f> position_changed;
         eventOut SF<float> orientation_changed;
         eventOut SF<Time> enterTime;
         eventOut SF<Time> exitTime;

         public:
              ProximitySensor2D() {
                   center = Vec2f(0, 0);
                   size = Vec2f(0, 0);
                   enabled = TRUE;
              }
};

class ProximitySensor : public MediaObject
{
         BIFS_DECLARE_NODE

         exposedField SF<Vec3f> center;
         exposedField SF<Vec3f> size;
         exposedField SF<Bool> enabled;
         eventOut SF<Bool> isActive;
         eventOut SF<Vec3f> position_changed;
         eventOut SF<Rotation> orientation_changed;
         eventOut SF<Time> enterTime;
         eventOut SF<Time> exitTime;

         public:
              ProximitySensor() {
                   center = Vec3f(0, 0, 0);
                   size = Vec3f(0, 0, 0);
                   enabled = TRUE;
              }
};

class QuantizationParameter : public MediaObject
{
         BIFS_DECLARE_NODE

         field SF<Bool> isLocal;
         field SF<Bool> position3DQuant;
         field SF<Vec3f> position3DMin;
         field SF<Vec3f> position3DMax;
         field SF<int> position3DNbBits;
         field SF<Bool> position2DQuant;
         field SF<Vec2f> position2DMin;
         field SF<Vec2f> position2DMax;
         field SF<int> position2DNbBits;
         field SF<Bool> drawOrderQuant;
         field SF<float> drawOrderMin;
         field SF<float> drawOrderMax;
         field SF<int> drawOrderNbBits;
         field SF<Bool> colorQuant;
         field SF<float> colorMin;
         field SF<float> colorMax;
         field SF<int> colorNbBits;
         field SF<Bool> textureCoordinateQuant;
         field SF<float> textureCoordinateMin;
         field SF<float> textureCoordinateMax;
         field SF<int> textureCoordinateNbBits;
         field SF<Bool> angleQuant;
         field SF<float> angleMin;
         field SF<float> angleMax;
         field SF<int> angleNbBits;
         field SF<Bool> scaleQuant;
         field SF<float> scaleMin;
         field SF<float> scaleMax;
         field SF<int> scaleNbBits;
         field SF<Bool> keyQuant;
         field SF<float> keyMin;
         field SF<float> keyMax;
         field SF<int> keyNbBits;
         field SF<Bool> normalQuant;
         field SF<int> normalNbBits;
         field SF<Bool> sizeQuant;
         field SF<float> sizeMin;
         field SF<float> sizeMax;
         field SF<int> sizeNbBits;
         field SF<Bool> useEfficientCoding;

         public:
              QuantizationParameter() {
                   isLocal = FALSE;
                   position3DQuant = FALSE;
                   position3DMin = Vec3f(-FLT_MAX, -FLT_MAX, -FLT_MAX);
                   position3DMax = Vec3f(+FLT_MAX, +FLT_MAX, +FLT_MAX);
                   position3DNbBits = 16;
                   position2DQuant = FALSE;
                   position2DMin = Vec2f(-FLT_MAX, -FLT_MAX);
                   position2DMax = Vec2f(+FLT_MAX, +FLT_MAX);
                   position2DNbBits = 16;
                   drawOrderQuant = FALSE;
                   drawOrderMin = -FLT_MAX;
                   drawOrderMax = +FLT_MAX;
                   drawOrderNbBits = 8;
                   colorQuant = TRUE;
                   colorMin = 0;
                   colorMax = 1;
                   colorNbBits = 8;
                   textureCoordinateQuant = TRUE;
                   textureCoordinateMin = 0;
                   textureCoordinateMax = 1;
                   textureCoordinateNbBits = 16;
                   angleQuant = TRUE;
                   angleMin = 0;
                   angleMax = 6.2831853;
                   angleNbBits = 16;
                   scaleQuant = FALSE;
                   scaleMin = 0;
                   scaleMax = +FLT_MAX;
                   scaleNbBits = 8;
                   keyQuant = TRUE;
                   keyMin = 0;
                   keyMax = 1;
                   keyNbBits = 8;
                   normalQuant = TRUE;
                   normalNbBits = 8;
                   sizeQuant = FALSE;
                   sizeMin = 0;
                   sizeMax = +FLT_MAX;
                   sizeNbBits = 8;
                   useEfficientCoding = FALSE;
              }
};

class Rectangle : public MediaObject
{
         BIFS_DECLARE_NODE

         exposedField SF<Vec2f> size;

         public:
              Rectangle() {
                   size = Vec2f(2, 2);
              }
};

class ScalarInterpolator : public MediaObject
{
         BIFS_DECLARE_NODE

         eventIn SF<float> set_fraction;
         exposedField MF<float> key;
         exposedField MF<float> keyValue;
         eventOut SF<float> value_changed;
         void Onset_fraction();

         public:
              ScalarInterpolator() {
              }
};

class Script : public MediaObject
{
         BIFS_DECLARE_NODE

         exposedField MF<ZString> url;
         field SF<Bool> directOutput;
         field SF<Bool> mustEvaluate;

         public:
              Script() {
                   directOutput = FALSE;
                   mustEvaluate = FALSE;
              }
};

class Shape : public MediaObject
{
         BIFS_DECLARE_NODE

         exposedField SFNode<class Appearance> appearance;
         exposedField SFGeometryNode geometry;

         public:
              Shape() {
              }
};

class Sound : public MediaObject
{
         BIFS_DECLARE_NODE

         exposedField SF<Vec3f> direction;
         exposedField SF<float> intensity;
         exposedField SF<Vec3f> location;
         exposedField SF<float> maxBack;
         exposedField SF<float> maxFront;
         exposedField SF<float> minBack;
         exposedField SF<float> minFront;
         exposedField SF<float> priority;
         exposedField SFAudioNode source;
         field SF<Bool> spatialize;

         public:
              Sound() {
                   direction = Vec3f(0, 0, 1);
                   intensity = 1;
                   location = Vec3f(0, 0, 0);
                   maxBack = 10;
                   maxFront = 10;
                   minBack = 1;
                   minFront = 1;
                   priority = 0;
                   spatialize = TRUE;
              }
};

class Sound2D : public MediaObject
{
         BIFS_DECLARE_NODE

         exposedField SF<float> intensity;
         exposedField SF<Vec2f> location;
         exposedField SFAudioNode source;
         field SF<Bool> spatialize;

         public:
              Sound2D() {
                   intensity = 1;
                   location = Vec2f(0, 0);
                   spatialize = TRUE;
              }
};

class Sphere : public MediaObject
{
         BIFS_DECLARE_NODE

         field SF<float> radius;

         public:
              Sphere() {
                   radius = 1;
              }
};

class SphereSensor : public MediaObject
{
         BIFS_DECLARE_NODE

         exposedField SF<Bool> autoOffset;
         exposedField SF<Bool> enabled;
         exposedField SF<Rotation> offset;
         eventOut SF<Bool> isActive;
         eventOut SF<Rotation> rotation_changed;
         eventOut SF<Vec3f> trackPoint_changed;

         public:
              SphereSensor() {
                   autoOffset = TRUE;
                   enabled = TRUE;
                   offset = Rotation(0, 1, 0, 0);
              }
};

class SpotLight : public MediaObject
{
         BIFS_DECLARE_NODE

         exposedField SF<float> ambientIntensity;
         exposedField SF<Vec3f> attenuation;
         exposedField SF<float> beamWidth;
         exposedField SF<Color3f> color;
         exposedField SF<float> cutOffAngle;
         exposedField SF<Vec3f> direction;
         exposedField SF<float> intensity;
         exposedField SF<Vec3f> location;
         exposedField SF<Bool> on;
         exposedField SF<float> radius;

         public:
              SpotLight() {
                   ambientIntensity = 0;
                   attenuation = Vec3f(1, 0, 0);
                   beamWidth = 1.570796;
                   color = Color3f(1, 1, 1);
                   cutOffAngle = 0.785398;
                   direction = Vec3f(0, 0, -1);
                   intensity = 1;
                   location = Vec3f(0, 0, 0);
                   on = TRUE;
                   radius = 100;
              }
};

class Switch : public MediaObject
{
         BIFS_DECLARE_NODE

         exposedField MFNodeType<SF3DNode> choice;
         exposedField SF<int> whichChoice;

         public:
              Switch() {
                   whichChoice = -1;
              }
};

class TermCap : public MediaObject
{
         BIFS_DECLARE_NODE

         eventIn SF<Time> evaluate;
         exposedField SF<int> capability;
         eventOut SF<int> value;
         void Onevaluate();

         public:
              TermCap() {
                   capability = 0;
                   value = 0;
              }
};

class Text : public MediaObject
{
         BIFS_DECLARE_NODE

         exposedField MF<ZString> string;
         exposedField MF<float> length;
         exposedField SFNode<class FontStyle> fontStyle;
         exposedField SF<float> maxExtent;

         public:
              Text() {
                   maxExtent = 0;
              }
};

class TextureCoordinate : public MediaObject
{
         BIFS_DECLARE_NODE

         exposedField MF<Vec2f> point;

         public:
              TextureCoordinate() {
              }
};

class TextureTransform : public MediaObject
{
         BIFS_DECLARE_NODE

         exposedField SF<Vec2f> center;
         exposedField SF<float> rotation;
         exposedField SF<Vec2f> scale;
         exposedField SF<Vec2f> translation;

         public:
              TextureTransform() {
                   center = Vec2f(0, 0);
                   rotation = 0;
                   scale = Vec2f(1, 1);
                   translation = Vec2f(0, 0);
              }
};

class TimeSensor : public MediaObject
{
         BIFS_DECLARE_NODE

         exposedField SF<Time> cycleInterval;
         exposedField SF<Bool> enabled;
         exposedField SF<Bool> loop;
         exposedField SF<Time> startTime;
         exposedField SF<Time> stopTime;
         eventOut SF<Time> cycleTime;
         eventOut SF<float> fraction_changed;
         eventOut SF<Bool> isActive;
         eventOut SF<Time> time;

         public:
              TimeSensor() {
                   cycleInterval = 1;
                   enabled = TRUE;
                   loop = FALSE;
                   startTime = 0;
                   stopTime = 0;
              }
};

class TouchSensor : public MediaObject
{
         BIFS_DECLARE_NODE

         exposedField SF<Bool> enabled;
         eventOut SF<Vec3f> hitNormal_changed;
         eventOut SF<Vec3f> hitPoint_changed;
         eventOut SF<Vec2f> hitTexCoord_changed;
         eventOut SF<Bool> isActive;
         eventOut SF<Bool> isOver;
         eventOut SF<Time> touchTime;

         public:
              TouchSensor() {
                   enabled = TRUE;
              }
};

class Transform : public MediaObject
{
         BIFS_DECLARE_NODE

         eventIn MFNodeType<SF3DNode> addChildren;
         eventIn MFNodeType<SF3DNode> removeChildren;
         exposedField SF<Vec3f> center;
         exposedField MFNodeType<SF3DNode> children;
         exposedField SF<Rotation> rotation;
         exposedField SF<Vec3f> scale;
         exposedField SF<Rotation> scaleOrientation;
         exposedField SF<Vec3f> translation;
         void OnaddChildren();
         void OnremoveChildren();

         public:
              Transform() {
                   center = Vec3f(0, 0, 0);
                   rotation = Rotation(0, 0, 1, 0);
                   scale = Vec3f(1, 1, 1);
                   scaleOrientation = Rotation(0, 0, 1, 0);
                   translation = Vec3f(0, 0, 0);
              }
};

class Transform2D : public MediaObject
{
         BIFS_DECLARE_NODE

         eventIn MFNodeType<SF2DNode> addChildren;
         eventIn MFNodeType<SF2DNode> removeChildren;
         exposedField MFNodeType<SF2DNode> children;
         exposedField SF<Vec2f> center;
         exposedField SF<float> rotationAngle;
         exposedField SF<Vec2f> scale;
         exposedField SF<float> scaleOrientation;
         exposedField SF<Vec2f> translation;
         void OnaddChildren();
         void OnremoveChildren();

         public:
              Transform2D() {
                   center = Vec2f(0, 0);
                   rotationAngle = 0;
                   scale = Vec2f(1, 1);
                   scaleOrientation = 0;
                   translation = Vec2f(0, 0);
              }
};

class Valuator : public MediaObject
{
         BIFS_DECLARE_NODE

         eventIn SF<Bool> inSFBool;
         eventIn SF<Color3f> inSFColor;
         eventIn MF<Color3f> inMFColor;
         eventIn SF<float> inSFFloat;
         eventIn MF<float> inMFFloat;
         eventIn SF<int> inSFInt32;
         eventIn MF<int> inMFInt32;
         eventIn SF<Rotation> inSFRotation;
         eventIn MF<Rotation> inMFRotation;
         eventIn SFString inSFString;
         eventIn MF<ZString> inMFString;
         eventIn SF<Time> inSFTime;
         eventIn SF<Vec2f> inSFVec2f;
         eventIn MF<Vec2f> inMFVec2f;
         eventIn SF<Vec3f> inSFVec3f;
         eventIn MF<Vec3f> inMFVec3f;
         eventOut SF<Bool> outSFBool;
         eventOut SF<Color3f> outSFColor;
         eventOut MF<Color3f> outMFColor;
         eventOut SF<float> outSFFloat;
         eventOut MF<float> outMFFloat;
         eventOut SF<int> outSFInt32;
         eventOut MF<int> outMFInt32;
         eventOut SF<Rotation> outSFRotation;
         eventOut MF<Rotation> outMFRotation;
         eventOut SFString outSFString;
         eventOut MF<ZString> outMFString;
         eventOut SF<Time> outSFTime;
         eventOut SF<Vec2f> outSFVec2f;
         eventOut MF<Vec2f> outMFVec2f;
         eventOut SF<Vec3f> outSFVec3f;
         eventOut MF<Vec3f> outMFVec3f;
         exposedField SF<float> factor1;
         exposedField SF<float> factor2;
         exposedField SF<float> factor3;
         exposedField SF<float> factor4;
         exposedField SF<float> offset1;
         exposedField SF<float> offset2;
         exposedField SF<float> offset3;
         exposedField SF<float> offset4;
         exposedField SF<Bool> sum;
         void OninSFBool();
         void OninSFColor();
         void OninMFColor();
         void OninSFFloat();
         void OninMFFloat();
         void OninSFInt32();
         void OninMFInt32();
         void OninSFRotation();
         void OninMFRotation();
         void OninSFString();
         void OninMFString();
         void OninSFTime();
         void OninSFVec2f();
         void OninMFVec2f();
         void OninSFVec3f();
         void OninMFVec3f();

         public:
              Valuator() {
                   factor1 = 1;
                   factor2 = 1;
                   factor3 = 1;
                   factor4 = 1;
                   offset1 = 0;
                   offset2 = 0;
                   offset3 = 0;
                   offset4 = 0;
                   sum = FALSE;
              }
};

class Viewpoint : public MediaObject
{
         BIFS_DECLARE_NODE

         eventIn SF<Bool> set_bind;
         exposedField SF<float> fieldOfView;
         exposedField SF<Bool> jump;
         exposedField SF<Rotation> orientation;
         exposedField SF<Vec3f> position;
         field SFString description;
         eventOut SF<Time> bindTime;
         eventOut SF<Bool> isBound;
         void Onset_bind();

         public:
              Viewpoint() {
                   fieldOfView = 0.785398;
                   jump = TRUE;
                   orientation = Rotation(0, 0, 1, 0);
                   position = Vec3f(0, 0, 10);
              }
};

class VisibilitySensor : public MediaObject
{
         BIFS_DECLARE_NODE

         exposedField SF<Vec3f> center;
         exposedField SF<Bool> enabled;
         exposedField SF<Vec3f> size;
         eventOut SF<Time> enterTime;
         eventOut SF<Time> exitTime;
         eventOut SF<Bool> isActive;

         public:
              VisibilitySensor() {
                   center = Vec3f(0, 0, 0);
                   enabled = TRUE;
                   size = Vec3f(0, 0, 0);
              }
};

class Viseme : public MediaObject
{
         BIFS_DECLARE_NODE

         exposedField SF<int> viseme_select1;
         exposedField SF<int> viseme_select2;
         exposedField SF<int> viseme_blend;
         exposedField SF<Bool> viseme_def;

         public:
              Viseme() {
                   viseme_select1 = 0;
                   viseme_select2 = 0;
                   viseme_blend = 0;
                   viseme_def = FALSE;
              }
};

class WorldInfo : public MediaObject
{
         BIFS_DECLARE_NODE

         field MF<ZString> info;
         field SFString title;

         public:
              WorldInfo() {
              }
};

class AcousticMaterial : public MediaObject
{
         BIFS_DECLARE_NODE

         exposedField SF<float> ambientIntensity;
         exposedField SF<Color3f> diffuseColor;
         exposedField SF<Color3f> emissiveColor;
         exposedField SF<float> shininess;
         exposedField SF<Color3f> specularColor;
         exposedField SF<float> transparency;
         field MF<float> reffunc;
         field MF<float> transfunc;
         field MF<float> refFrequency;
         field MF<float> transFrequency;

         public:
              AcousticMaterial() {
                   ambientIntensity = 0.2;
                   diffuseColor = Color3f(0.8, 0.8, 0.8);
                   emissiveColor = Color3f(0, 0, 0);
                   shininess = 0.2;
                   specularColor = Color3f(0, 0, 0);
                   transparency = 0;
                   reffunc += 0;
                   transfunc += 1;
                   refFrequency += 0;
                   transFrequency += 0;
              }
};

class AcousticScene : public MediaObject
{
         BIFS_DECLARE_NODE

         field SF<Vec3f> center;
         field SF<Vec3f> Size;
         field MF<Time> reverbTime;
         field MF<float> reverbFreq;
         exposedField SF<float> reverbLevel;
         exposedField SF<Time> reverbDelay;

         public:
              AcousticScene() {
                   center = Vec3f(0, 0, 0);
                   Size = Vec3f(-1, -1, -1);
                   reverbTime += 0;
                   reverbFreq += 1000;
                   reverbLevel = 0.4;
                   reverbDelay = 0.5;
              }
};

class ApplicationWindow : public StreamConsumer
{
         BIFS_DECLARE_NODE

         exposedField SF<Bool> isActive;
         exposedField SF<Time> startTime;
         exposedField SF<Time> stopTime;
         exposedField SFString description;
         exposedField MF<ZString> parameter;
         exposedField SF<Vec2f> size;

         public:
              ApplicationWindow() {
                   isActive = FALSE;
                   startTime = 0;
                   stopTime = 0;
                   size = Vec2f(0, 0);
              }
};

class BAP : public MediaObject
{
         BIFS_DECLARE_NODE

         exposedField SF<int> sacroiliac_tilt;
         exposedField SF<int> sacroiliac_torsion;
         exposedField SF<int> sacroiliac_roll;
         exposedField SF<int> l_hip_flexion;
         exposedField SF<int> r_hip_flexion;
         exposedField SF<int> l_hip_abduct;
         exposedField SF<int> r_hip_abduct;
         exposedField SF<int> l_hip_twisting;
         exposedField SF<int> r_hip_twisting;
         exposedField SF<int> l_knee_flexion;
         exposedField SF<int> r_knee_flexion;
         exposedField SF<int> l_knee_twisting;
         exposedField SF<int> r_knee_twisting;
         exposedField SF<int> l_ankle_flexion;
         exposedField SF<int> r_ankle_flexion;
         exposedField SF<int> l_ankle_twisting;
         exposedField SF<int> r_ankle_twisting;
         exposedField SF<int> l_subtalar_flexion;
         exposedField SF<int> r_subtalar_flexion;
         exposedField SF<int> l_midtarsal_flexion;
         exposedField SF<int> r_midtarsal_flexion;
         exposedField SF<int> l_metatarsal_flexion;
         exposedField SF<int> r_metatarsal_flexion;
         exposedField SF<int> l_sternoclavicular_abduct;
         exposedField SF<int> r_sternoclavicular_abduct;
         exposedField SF<int> l_sternoclavicular_rotate;
         exposedField SF<int> r_sternoclavicular_rotate;
         exposedField SF<int> l_acromioclavicular_abduct;
         exposedField SF<int> r_acromioclavicular_abduct;
         exposedField SF<int> l_acromioclavicular_rotate;
         exposedField SF<int> r_acromioclavicular_rotate;
         exposedField SF<int> l_shoulder_flexion;
         exposedField SF<int> r_shoulder_flexion;
         exposedField SF<int> l_shoulder_abduct;
         exposedField SF<int> r_shoulder_abduct;
         exposedField SF<int> l_shoulder_twisting;
         exposedField SF<int> r_shoulder_twisting;
         exposedField SF<int> l_elbow_flexion;
         exposedField SF<int> r_elbow_flexion;
         exposedField SF<int> l_elbow_twisting;
         exposedField SF<int> r_elbow_twisting;
         exposedField SF<int> l_wrist_flexion;
         exposedField SF<int> r_wrist_flexion;
         exposedField SF<int> l_wrist_pivot;
         exposedField SF<int> r_wrist_pivot;
         exposedField SF<int> l_wrist_twisting;
         exposedField SF<int> r_wrist_twisting;
         exposedField SF<int> skullbase_roll;
         exposedField SF<int> skullbase_torsion;
         exposedField SF<int> skullbase_tilt;
         exposedField SF<int> vc1roll;
         exposedField SF<int> vc1torsion;
         exposedField SF<int> vc1tilt;
         exposedField SF<int> vc2roll;
         exposedField SF<int> vc2torsion;
         exposedField SF<int> vc2tilt;
         exposedField SF<int> vc3roll;
         exposedField SF<int> vc3torsion;
         exposedField SF<int> vc3tilt;
         exposedField SF<int> vc4roll;
         exposedField SF<int> vc4torsion;
         exposedField SF<int> vc4tilt;
         exposedField SF<int> vc5roll;
         exposedField SF<int> vc5torsion;
         exposedField SF<int> vc5tilt;
         exposedField SF<int> vc6roll;
         exposedField SF<int> vc6torsion;
         exposedField SF<int> vc6tilt;
         exposedField SF<int> vc7roll;
         exposedField SF<int> vc7torsion;
         exposedField SF<int> vc7tilt;
         exposedField SF<int> vt1roll;
         exposedField SF<int> vt1torsion;
         exposedField SF<int> vt1tilt;
         exposedField SF<int> vt2roll;
         exposedField SF<int> vt2torsion;
         exposedField SF<int> vt2tilt;
         exposedField SF<int> vt3roll;
         exposedField SF<int> vt3torsion;
         exposedField SF<int> vt3tilt;
         exposedField SF<int> vt4roll;
         exposedField SF<int> vt4torsion;
         exposedField SF<int> vt4tilt;
         exposedField SF<int> vt5roll;
         exposedField SF<int> vt5torsion;
         exposedField SF<int> vt5tilt;
         exposedField SF<int> vt6roll;
         exposedField SF<int> vt6torsion;
         exposedField SF<int> vt6tilt;
         exposedField SF<int> vt7roll;
         exposedField SF<int> vt7torsion;
         exposedField SF<int> vt7tilt;
         exposedField SF<int> vt8roll;
         exposedField SF<int> vt8torsion;
         exposedField SF<int> vt8tilt;
         exposedField SF<int> vt9roll;
         exposedField SF<int> vt9torsion;
         exposedField SF<int> vt9tilt;
         exposedField SF<int> vt10roll;
         exposedField SF<int> vt10torsion;
         exposedField SF<int> vt10tilt;
         exposedField SF<int> vt11roll;
         exposedField SF<int> vt11torsion;
         exposedField SF<int> vt11tilt;
         exposedField SF<int> vt12roll;
         exposedField SF<int> vt12torsion;
         exposedField SF<int> vt12tilt;
         exposedField SF<int> vl1roll;
         exposedField SF<int> vl1torsion;
         exposedField SF<int> vl1tilt;
         exposedField SF<int> vl2roll;
         exposedField SF<int> vl2torsion;
         exposedField SF<int> vl2tilt;
         exposedField SF<int> vl3roll;
         exposedField SF<int> vl3torsion;
         exposedField SF<int> vl3tilt;
         exposedField SF<int> vl4roll;
         exposedField SF<int> vl4torsion;
         exposedField SF<int> vl4tilt;
         exposedField SF<int> vl5roll;
         exposedField SF<int> vl5torsion;
         exposedField SF<int> vl5tilt;
         exposedField SF<int> l_pinky0_flexion;
         exposedField SF<int> r_pinky0_flexion;
         exposedField SF<int> l_pinky1_flexion;
         exposedField SF<int> r_pinky1_flexion;
         exposedField SF<int> l_pinky1_pivot;
         exposedField SF<int> r_pinky1_pivot;
         exposedField SF<int> l_pinky1_twisting;
         exposedField SF<int> r_pinky1_twisting;
         exposedField SF<int> l_pinky2_flexion;
         exposedField SF<int> r_pinky2_flexion;
         exposedField SF<int> l_pinky3_flexion;
         exposedField SF<int> r_pinky3_flexion;
         exposedField SF<int> l_ring0_flexion;
         exposedField SF<int> r_ring0_flexion;
         exposedField SF<int> l_ring1_flexion;
         exposedField SF<int> r_ring1_flexion;
         exposedField SF<int> l_ring1_pivot;
         exposedField SF<int> r_ring1_pivot;
         exposedField SF<int> l_ring1_twisting;
         exposedField SF<int> r_ring1_twisting;
         exposedField SF<int> l_ring2_flexion;
         exposedField SF<int> r_ring2_flexion;
         exposedField SF<int> l_ring3_flexion;
         exposedField SF<int> r_ring3_flexion;
         exposedField SF<int> l_middle0_flexion;
         exposedField SF<int> r_middle0_flexion;
         exposedField SF<int> l_middle1_flexion;
         exposedField SF<int> r_middle1_flexion;
         exposedField SF<int> l_middle1_pivot;
         exposedField SF<int> r_middle1_pivot;
         exposedField SF<int> l_middle1_twisting;
         exposedField SF<int> r_middle1_twisting;
         exposedField SF<int> l_middle2_flexion;
         exposedField SF<int> r_middle2_flexion;
         exposedField SF<int> l_middle3_flexion;
         exposedField SF<int> r_middle3_flexion;
         exposedField SF<int> l_index0_flexion;
         exposedField SF<int> r_index0_flexion;
         exposedField SF<int> l_index1_flexion;
         exposedField SF<int> r_index1_flexion;
         exposedField SF<int> l_index1_pivot;
         exposedField SF<int> r_index1_pivot;
         exposedField SF<int> l_index1_twisting;
         exposedField SF<int> r_index1_twisting;
         exposedField SF<int> l_index2_flexion;
         exposedField SF<int> r_index2_flexion;
         exposedField SF<int> l_index3_flexion;
         exposedField SF<int> r_index3_flexion;
         exposedField SF<int> l_thumb1_flexion;
         exposedField SF<int> r_thumb1_flexion;
         exposedField SF<int> l_thumb1_pivot;
         exposedField SF<int> r_thumb1_pivot;
         exposedField SF<int> l_thumb1_twisting;
         exposedField SF<int> r_thumb1_twisting;
         exposedField SF<int> l_thumb2_flexion;
         exposedField SF<int> r_thumb2_flexion;
         exposedField SF<int> l_thumb3_flexion;
         exposedField SF<int> r_thumb3_flexion;
         exposedField SF<int> HumanoidRoot_tr_vertical;
         exposedField SF<int> HumanoidRoot_tr_lateral;
         exposedField SF<int> HumanoidRoot_tr_frontal;
         exposedField SF<int> HumanoidRoot_rt_body_turn;
         exposedField SF<int> HumanoidRoot_rt_body_roll;
         exposedField SF<int> HumanoidRoot_rt_body_tilt;
         exposedField SF<int> extensionBap187;
         exposedField SF<int> extensionBap188;
         exposedField SF<int> extensionBap189;
         exposedField SF<int> extensionBap190;
         exposedField SF<int> extensionBap191;
         exposedField SF<int> extensionBap192;
         exposedField SF<int> extensionBap193;
         exposedField SF<int> extensionBap194;
         exposedField SF<int> extensionBap195;
         exposedField SF<int> extensionBap196;
         exposedField SF<int> extensionBap197;
         exposedField SF<int> extensionBap198;
         exposedField SF<int> extensionBap199;
         exposedField SF<int> extensionBap200;
         exposedField SF<int> extensionBap201;
         exposedField SF<int> extensionBap202;
         exposedField SF<int> extensionBap203;
         exposedField SF<int> extensionBap204;
         exposedField SF<int> extensionBap205;
         exposedField SF<int> extensionBap206;
         exposedField SF<int> extensionBap207;
         exposedField SF<int> extensionBap208;
         exposedField SF<int> extensionBap209;
         exposedField SF<int> extensionBap210;
         exposedField SF<int> extensionBap211;
         exposedField SF<int> extensionBap212;
         exposedField SF<int> extensionBap213;
         exposedField SF<int> extensionBap214;
         exposedField SF<int> extensionBap215;
         exposedField SF<int> extensionBap216;
         exposedField SF<int> extensionBap217;
         exposedField SF<int> extensionBap218;
         exposedField SF<int> extensionBap219;
         exposedField SF<int> extensionBap220;
         exposedField SF<int> extensionBap221;
         exposedField SF<int> extensionBap222;
         exposedField SF<int> extensionBap223;
         exposedField SF<int> extensionBap224;
         exposedField SF<int> extensionBap225;
         exposedField SF<int> extensionBap226;
         exposedField SF<int> extensionBap227;
         exposedField SF<int> extensionBap228;
         exposedField SF<int> extensionBap229;
         exposedField SF<int> extensionBap230;
         exposedField SF<int> extensionBap231;
         exposedField SF<int> extensionBap232;
         exposedField SF<int> extensionBap233;
         exposedField SF<int> extensionBap234;
         exposedField SF<int> extensionBap235;
         exposedField SF<int> extensionBap236;
         exposedField SF<int> extensionBap237;
         exposedField SF<int> extensionBap238;
         exposedField SF<int> extensionBap239;
         exposedField SF<int> extensionBap240;
         exposedField SF<int> extensionBap241;
         exposedField SF<int> extensionBap242;
         exposedField SF<int> extensionBap243;
         exposedField SF<int> extensionBap244;
         exposedField SF<int> extensionBap245;
         exposedField SF<int> extensionBap246;
         exposedField SF<int> extensionBap247;
         exposedField SF<int> extensionBap248;
         exposedField SF<int> extensionBap249;
         exposedField SF<int> extensionBap250;
         exposedField SF<int> extensionBap251;
         exposedField SF<int> extensionBap252;
         exposedField SF<int> extensionBap253;
         exposedField SF<int> extensionBap254;
         exposedField SF<int> extensionBap255;
         exposedField SF<int> extensionBap256;
         exposedField SF<int> extensionBap257;
         exposedField SF<int> extensionBap258;
         exposedField SF<int> extensionBap259;
         exposedField SF<int> extensionBap260;
         exposedField SF<int> extensionBap261;
         exposedField SF<int> extensionBap262;
         exposedField SF<int> extensionBap263;
         exposedField SF<int> extensionBap264;
         exposedField SF<int> extensionBap265;
         exposedField SF<int> extensionBap266;
         exposedField SF<int> extensionBap267;
         exposedField SF<int> extensionBap268;
         exposedField SF<int> extensionBap269;
         exposedField SF<int> extensionBap270;
         exposedField SF<int> extensionBap271;
         exposedField SF<int> extensionBap272;
         exposedField SF<int> extensionBap273;
         exposedField SF<int> extensionBap274;
         exposedField SF<int> extensionBap275;
         exposedField SF<int> extensionBap276;
         exposedField SF<int> extensionBap277;
         exposedField SF<int> extensionBap278;
         exposedField SF<int> extensionBap279;
         exposedField SF<int> extensionBap280;
         exposedField SF<int> extensionBap281;
         exposedField SF<int> extensionBap282;
         exposedField SF<int> extensionBap283;
         exposedField SF<int> extensionBap284;
         exposedField SF<int> extensionBap285;
         exposedField SF<int> extensionBap286;
         exposedField SF<int> extensionBap287;
         exposedField SF<int> extensionBap288;
         exposedField SF<int> extensionBap289;
         exposedField SF<int> extensionBap290;
         exposedField SF<int> extensionBap291;
         exposedField SF<int> extensionBap292;
         exposedField SF<int> extensionBap293;
         exposedField SF<int> extensionBap294;
         exposedField SF<int> extensionBap295;
         exposedField SF<int> extensionBap296;

         public:
              BAP() {
                   sacroiliac_tilt = +INT_MAX;
                   sacroiliac_torsion = +INT_MAX;
                   sacroiliac_roll = +INT_MAX;
                   l_hip_flexion = +INT_MAX;
                   r_hip_flexion = +INT_MAX;
                   l_hip_abduct = +INT_MAX;
                   r_hip_abduct = +INT_MAX;
                   l_hip_twisting = +INT_MAX;
                   r_hip_twisting = +INT_MAX;
                   l_knee_flexion = +INT_MAX;
                   r_knee_flexion = +INT_MAX;
                   l_knee_twisting = +INT_MAX;
                   r_knee_twisting = +INT_MAX;
                   l_ankle_flexion = +INT_MAX;
                   r_ankle_flexion = +INT_MAX;
                   l_ankle_twisting = +INT_MAX;
                   r_ankle_twisting = +INT_MAX;
                   l_subtalar_flexion = +INT_MAX;
                   r_subtalar_flexion = +INT_MAX;
                   l_midtarsal_flexion = +INT_MAX;
                   r_midtarsal_flexion = +INT_MAX;
                   l_metatarsal_flexion = +INT_MAX;
                   r_metatarsal_flexion = +INT_MAX;
                   l_sternoclavicular_abduct = +INT_MAX;
                   r_sternoclavicular_abduct = +INT_MAX;
                   l_sternoclavicular_rotate = +INT_MAX;
                   r_sternoclavicular_rotate = +INT_MAX;
                   l_acromioclavicular_abduct = +INT_MAX;
                   r_acromioclavicular_abduct = +INT_MAX;
                   l_acromioclavicular_rotate = +INT_MAX;
                   r_acromioclavicular_rotate = +INT_MAX;
                   l_shoulder_flexion = +INT_MAX;
                   r_shoulder_flexion = +INT_MAX;
                   l_shoulder_abduct = +INT_MAX;
                   r_shoulder_abduct = +INT_MAX;
                   l_shoulder_twisting = +INT_MAX;
                   r_shoulder_twisting = +INT_MAX;
                   l_elbow_flexion = +INT_MAX;
                   r_elbow_flexion = +INT_MAX;
                   l_elbow_twisting = +INT_MAX;
                   r_elbow_twisting = +INT_MAX;
                   l_wrist_flexion = +INT_MAX;
                   r_wrist_flexion = +INT_MAX;
                   l_wrist_pivot = +INT_MAX;
                   r_wrist_pivot = +INT_MAX;
                   l_wrist_twisting = +INT_MAX;
                   r_wrist_twisting = +INT_MAX;
                   skullbase_roll = +INT_MAX;
                   skullbase_torsion = +INT_MAX;
                   skullbase_tilt = +INT_MAX;
                   vc1roll = +INT_MAX;
                   vc1torsion = +INT_MAX;
                   vc1tilt = +INT_MAX;
                   vc2roll = +INT_MAX;
                   vc2torsion = +INT_MAX;
                   vc2tilt = +INT_MAX;
                   vc3roll = +INT_MAX;
                   vc3torsion = +INT_MAX;
                   vc3tilt = +INT_MAX;
                   vc4roll = +INT_MAX;
                   vc4torsion = +INT_MAX;
                   vc4tilt = +INT_MAX;
                   vc5roll = +INT_MAX;
                   vc5torsion = +INT_MAX;
                   vc5tilt = +INT_MAX;
                   vc6roll = +INT_MAX;
                   vc6torsion = +INT_MAX;
                   vc6tilt = +INT_MAX;
                   vc7roll = +INT_MAX;
                   vc7torsion = +INT_MAX;
                   vc7tilt = +INT_MAX;
                   vt1roll = +INT_MAX;
                   vt1torsion = +INT_MAX;
                   vt1tilt = +INT_MAX;
                   vt2roll = +INT_MAX;
                   vt2torsion = +INT_MAX;
                   vt2tilt = +INT_MAX;
                   vt3roll = +INT_MAX;
                   vt3torsion = +INT_MAX;
                   vt3tilt = +INT_MAX;
                   vt4roll = +INT_MAX;
                   vt4torsion = +INT_MAX;
                   vt4tilt = +INT_MAX;
                   vt5roll = +INT_MAX;
                   vt5torsion = +INT_MAX;
                   vt5tilt = +INT_MAX;
                   vt6roll = +INT_MAX;
                   vt6torsion = +INT_MAX;
                   vt6tilt = +INT_MAX;
                   vt7roll = +INT_MAX;
                   vt7torsion = +INT_MAX;
                   vt7tilt = +INT_MAX;
                   vt8roll = +INT_MAX;
                   vt8torsion = +INT_MAX;
                   vt8tilt = +INT_MAX;
                   vt9roll = +INT_MAX;
                   vt9torsion = +INT_MAX;
                   vt9tilt = +INT_MAX;
                   vt10roll = +INT_MAX;
                   vt10torsion = +INT_MAX;
                   vt10tilt = +INT_MAX;
                   vt11roll = +INT_MAX;
                   vt11torsion = +INT_MAX;
                   vt11tilt = +INT_MAX;
                   vt12roll = +INT_MAX;
                   vt12torsion = +INT_MAX;
                   vt12tilt = +INT_MAX;
                   vl1roll = +INT_MAX;
                   vl1torsion = +INT_MAX;
                   vl1tilt = +INT_MAX;
                   vl2roll = +INT_MAX;
                   vl2torsion = +INT_MAX;
                   vl2tilt = +INT_MAX;
                   vl3roll = +INT_MAX;
                   vl3torsion = +INT_MAX;
                   vl3tilt = +INT_MAX;
                   vl4roll = +INT_MAX;
                   vl4torsion = +INT_MAX;
                   vl4tilt = +INT_MAX;
                   vl5roll = +INT_MAX;
                   vl5torsion = +INT_MAX;
                   vl5tilt = +INT_MAX;
                   l_pinky0_flexion = +INT_MAX;
                   r_pinky0_flexion = +INT_MAX;
                   l_pinky1_flexion = +INT_MAX;
                   r_pinky1_flexion = +INT_MAX;
                   l_pinky1_pivot = +INT_MAX;
                   r_pinky1_pivot = +INT_MAX;
                   l_pinky1_twisting = +INT_MAX;
                   r_pinky1_twisting = +INT_MAX;
                   l_pinky2_flexion = +INT_MAX;
                   r_pinky2_flexion = +INT_MAX;
                   l_pinky3_flexion = +INT_MAX;
                   r_pinky3_flexion = +INT_MAX;
                   l_ring0_flexion = +INT_MAX;
                   r_ring0_flexion = +INT_MAX;
                   l_ring1_flexion = +INT_MAX;
                   r_ring1_flexion = +INT_MAX;
                   l_ring1_pivot = +INT_MAX;
                   r_ring1_pivot = +INT_MAX;
                   l_ring1_twisting = +INT_MAX;
                   r_ring1_twisting = +INT_MAX;
                   l_ring2_flexion = +INT_MAX;
                   r_ring2_flexion = +INT_MAX;
                   l_ring3_flexion = +INT_MAX;
                   r_ring3_flexion = +INT_MAX;
                   l_middle0_flexion = +INT_MAX;
                   r_middle0_flexion = +INT_MAX;
                   l_middle1_flexion = +INT_MAX;
                   r_middle1_flexion = +INT_MAX;
                   l_middle1_pivot = +INT_MAX;
                   r_middle1_pivot = +INT_MAX;
                   l_middle1_twisting = +INT_MAX;
                   r_middle1_twisting = +INT_MAX;
                   l_middle2_flexion = +INT_MAX;
                   r_middle2_flexion = +INT_MAX;
                   l_middle3_flexion = +INT_MAX;
                   r_middle3_flexion = +INT_MAX;
                   l_index0_flexion = +INT_MAX;
                   r_index0_flexion = +INT_MAX;
                   l_index1_flexion = +INT_MAX;
                   r_index1_flexion = +INT_MAX;
                   l_index1_pivot = +INT_MAX;
                   r_index1_pivot = +INT_MAX;
                   l_index1_twisting = +INT_MAX;
                   r_index1_twisting = +INT_MAX;
                   l_index2_flexion = +INT_MAX;
                   r_index2_flexion = +INT_MAX;
                   l_index3_flexion = +INT_MAX;
                   r_index3_flexion = +INT_MAX;
                   l_thumb1_flexion = +INT_MAX;
                   r_thumb1_flexion = +INT_MAX;
                   l_thumb1_pivot = +INT_MAX;
                   r_thumb1_pivot = +INT_MAX;
                   l_thumb1_twisting = +INT_MAX;
                   r_thumb1_twisting = +INT_MAX;
                   l_thumb2_flexion = +INT_MAX;
                   r_thumb2_flexion = +INT_MAX;
                   l_thumb3_flexion = +INT_MAX;
                   r_thumb3_flexion = +INT_MAX;
                   HumanoidRoot_tr_vertical = +INT_MAX;
                   HumanoidRoot_tr_lateral = +INT_MAX;
                   HumanoidRoot_tr_frontal = +INT_MAX;
                   HumanoidRoot_rt_body_turn = +INT_MAX;
                   HumanoidRoot_rt_body_roll = +INT_MAX;
                   HumanoidRoot_rt_body_tilt = +INT_MAX;
                   extensionBap187 = +INT_MAX;
                   extensionBap188 = +INT_MAX;
                   extensionBap189 = +INT_MAX;
                   extensionBap190 = +INT_MAX;
                   extensionBap191 = +INT_MAX;
                   extensionBap192 = +INT_MAX;
                   extensionBap193 = +INT_MAX;
                   extensionBap194 = +INT_MAX;
                   extensionBap195 = +INT_MAX;
                   extensionBap196 = +INT_MAX;
                   extensionBap197 = +INT_MAX;
                   extensionBap198 = +INT_MAX;
                   extensionBap199 = +INT_MAX;
                   extensionBap200 = +INT_MAX;
                   extensionBap201 = +INT_MAX;
                   extensionBap202 = +INT_MAX;
                   extensionBap203 = +INT_MAX;
                   extensionBap204 = +INT_MAX;
                   extensionBap205 = +INT_MAX;
                   extensionBap206 = +INT_MAX;
                   extensionBap207 = +INT_MAX;
                   extensionBap208 = +INT_MAX;
                   extensionBap209 = +INT_MAX;
                   extensionBap210 = +INT_MAX;
                   extensionBap211 = +INT_MAX;
                   extensionBap212 = +INT_MAX;
                   extensionBap213 = +INT_MAX;
                   extensionBap214 = +INT_MAX;
                   extensionBap215 = +INT_MAX;
                   extensionBap216 = +INT_MAX;
                   extensionBap217 = +INT_MAX;
                   extensionBap218 = +INT_MAX;
                   extensionBap219 = +INT_MAX;
                   extensionBap220 = +INT_MAX;
                   extensionBap221 = +INT_MAX;
                   extensionBap222 = +INT_MAX;
                   extensionBap223 = +INT_MAX;
                   extensionBap224 = +INT_MAX;
                   extensionBap225 = +INT_MAX;
                   extensionBap226 = +INT_MAX;
                   extensionBap227 = +INT_MAX;
                   extensionBap228 = +INT_MAX;
                   extensionBap229 = +INT_MAX;
                   extensionBap230 = +INT_MAX;
                   extensionBap231 = +INT_MAX;
                   extensionBap232 = +INT_MAX;
                   extensionBap233 = +INT_MAX;
                   extensionBap234 = +INT_MAX;
                   extensionBap235 = +INT_MAX;
                   extensionBap236 = +INT_MAX;
                   extensionBap237 = +INT_MAX;
                   extensionBap238 = +INT_MAX;
                   extensionBap239 = +INT_MAX;
                   extensionBap240 = +INT_MAX;
                   extensionBap241 = +INT_MAX;
                   extensionBap242 = +INT_MAX;
                   extensionBap243 = +INT_MAX;
                   extensionBap244 = +INT_MAX;
                   extensionBap245 = +INT_MAX;
                   extensionBap246 = +INT_MAX;
                   extensionBap247 = +INT_MAX;
                   extensionBap248 = +INT_MAX;
                   extensionBap249 = +INT_MAX;
                   extensionBap250 = +INT_MAX;
                   extensionBap251 = +INT_MAX;
                   extensionBap252 = +INT_MAX;
                   extensionBap253 = +INT_MAX;
                   extensionBap254 = +INT_MAX;
                   extensionBap255 = +INT_MAX;
                   extensionBap256 = +INT_MAX;
                   extensionBap257 = +INT_MAX;
                   extensionBap258 = +INT_MAX;
                   extensionBap259 = +INT_MAX;
                   extensionBap260 = +INT_MAX;
                   extensionBap261 = +INT_MAX;
                   extensionBap262 = +INT_MAX;
                   extensionBap263 = +INT_MAX;
                   extensionBap264 = +INT_MAX;
                   extensionBap265 = +INT_MAX;
                   extensionBap266 = +INT_MAX;
                   extensionBap267 = +INT_MAX;
                   extensionBap268 = +INT_MAX;
                   extensionBap269 = +INT_MAX;
                   extensionBap270 = +INT_MAX;
                   extensionBap271 = +INT_MAX;
                   extensionBap272 = +INT_MAX;
                   extensionBap273 = +INT_MAX;
                   extensionBap274 = +INT_MAX;
                   extensionBap275 = +INT_MAX;
                   extensionBap276 = +INT_MAX;
                   extensionBap277 = +INT_MAX;
                   extensionBap278 = +INT_MAX;
                   extensionBap279 = +INT_MAX;
                   extensionBap280 = +INT_MAX;
                   extensionBap281 = +INT_MAX;
                   extensionBap282 = +INT_MAX;
                   extensionBap283 = +INT_MAX;
                   extensionBap284 = +INT_MAX;
                   extensionBap285 = +INT_MAX;
                   extensionBap286 = +INT_MAX;
                   extensionBap287 = +INT_MAX;
                   extensionBap288 = +INT_MAX;
                   extensionBap289 = +INT_MAX;
                   extensionBap290 = +INT_MAX;
                   extensionBap291 = +INT_MAX;
                   extensionBap292 = +INT_MAX;
                   extensionBap293 = +INT_MAX;
                   extensionBap294 = +INT_MAX;
                   extensionBap295 = +INT_MAX;
                   extensionBap296 = +INT_MAX;
              }
};

class BDP : public MediaObject
{
         BIFS_DECLARE_NODE

         exposedField MFNode<class BodyDefTable> bodyDefTables;
         exposedField MFNodeType<SF3DNode> bodySceneGraph;

         public:
              BDP() {
              }
};

class Body : public MediaObject
{
         BIFS_DECLARE_NODE

         exposedField SFNode<class BDP> bdp;
         exposedField SFNode<class BAP> bap;
         exposedField MFNodeType<SF3DNode> renderedBody;

         public:
              Body() {
              }
};

class BodyDefTable : public MediaObject
{
         BIFS_DECLARE_NODE

         exposedField SFString bodySceneGraphNodeName;
         exposedField MF<int> bapIDs;
         exposedField MF<int> vertexIds;
         exposedField MF<int> bapCombinations;
         exposedField MF<Vec3f> displacements;
         exposedField SF<int> numInterpolateKeys;

         public:
              BodyDefTable() {
                   numInterpolateKeys = 2;
              }
};

class BodySegmentConnectionHint : public MediaObject
{
         BIFS_DECLARE_NODE

         exposedField SFString firstSegmentNodeName;
         exposedField SFString secondSegmentNodeName;
         exposedField MF<int> firstVertexIdList;
         exposedField MF<int> secondVertexIdList;

         public:
              BodySegmentConnectionHint() {
              }
};

class DirectiveSound : public MediaObject
{
         BIFS_DECLARE_NODE

         exposedField SF<Vec3f> direction;
         exposedField SF<float> intensity;
         exposedField SF<Vec3f> location;
         exposedField SFAudioNode source;
         exposedField SFNode<class PerceptualParameters> perceptualParameters;
         exposedField SF<Bool> roomEffect;
         exposedField SF<Bool> spatialize;
         field MF<float> directivity;
         field MF<float> angles;
         field MF<float> frequency;
         field SF<float> speedOfSound;
         field SF<float> distance;
         field SF<Bool> useAirabs;

         public:
              DirectiveSound() {
                   direction = Vec3f(0, 0, -1);
                   intensity = 1;
                   location = Vec3f(0, 0, 0);
                   roomEffect = FALSE;
                   spatialize = TRUE;
                   directivity += 1;
                   angles += 1;
                   speedOfSound = 340;
                   distance = 100;
                   useAirabs = FALSE;
              }
};

class Hierarchical3DMesh : public StreamConsumer
{
         BIFS_DECLARE_NODE

         eventIn SF<int> triangleBudget;
         exposedField SF<float> level;
         eventOut SF<Bool> doneLoading;
         void OntriangleBudget();

         public:
              Hierarchical3DMesh() {
                   triangleBudget = 1000;
                   level = 1;
              }
};

class MaterialKey : public MediaObject
{
         BIFS_DECLARE_NODE

         exposedField SF<Bool> isKeyed;
         exposedField SF<Bool> isRGB;
         exposedField SF<Color3f> keyColor;
         exposedField SF<float> lowThreshold;
         exposedField SF<float> highThreshold;
         exposedField SF<float> transparency;

         public:
              MaterialKey() {
                   isKeyed = TRUE;
                   isRGB = TRUE;
                   keyColor = Color3f(0, 0, 0);
                   lowThreshold = 0;
                   highThreshold = 0;
                   transparency = 0;
              }
};

class PerceptualParameters : public MediaObject
{
         BIFS_DECLARE_NODE

         exposedField SF<float> sourcePresence;
         exposedField SF<float> sourceWarmth;
         exposedField SF<float> sourceBrilliance;
         exposedField SF<float> roomPresence;
         exposedField SF<float> runningReverberance;
         exposedField SF<float> envelopment;
         exposedField SF<float> lateReverberance;
         exposedField SF<float> heavyness;
         exposedField SF<float> liveness;
         exposedField MF<float> omniDirectivity;
         exposedField MF<float> directFilterGains;
         exposedField MF<float> inputFilterGains;
         exposedField SF<float> refDistance;
         exposedField SF<float> freqLow;
         exposedField SF<float> freqHigh;
         exposedField SF<Time> timeLimit1;
         exposedField SF<Time> timeLimit2;
         exposedField SF<Time> timeLimit3;
         exposedField SF<Time> modalDensity;

         public:
              PerceptualParameters() {
                   sourcePresence = 1;
                   sourceWarmth = 1;
                   sourceBrilliance = 1;
                   roomPresence = 1;
                   runningReverberance = 1;
                   envelopment = 0;
                   lateReverberance = 1;
                   heavyness = 1;
                   liveness = 1;
                   omniDirectivity += 1;
                   directFilterGains += 1;
                   directFilterGains += 1;
                   directFilterGains += 1;
                   inputFilterGains += 1;
                   inputFilterGains += 1;
                   inputFilterGains += 1;
                   refDistance = 1;
                   freqLow = 250;
                   freqHigh = 4000;
                   timeLimit1 = 0.02;
                   timeLimit2 = 0.04;
                   timeLimit3 = 0.1;
                   modalDensity = 0.8;
              }
};

class TemporalTransform : public StreamConsumer
{
         BIFS_DECLARE_NODE

         eventIn MFNodeType<SF3DNode> addChildren;
         eventIn MFNodeType<SF3DNode> removeChildren;
         exposedField MFNodeType<SF3DNode> children;
         exposedField SF<Time> startTime;
         exposedField SF<Time> optimalDuration;
         exposedField SF<Bool> active;
         exposedField SF<float> speed;
         exposedField SF<Vec2f> scalability;
         exposedField MF<int> stretchMode;
         exposedField MF<int> shrinkMode;
         exposedField SF<Time> maxDelay;
         eventOut SF<Time> actualDuration;
         void OnaddChildren();
         void OnremoveChildren();

         public:
              TemporalTransform() {
                   startTime = -1;
                   optimalDuration = -1;
                   active = FALSE;
                   speed = 1;
                   scalability = Vec2f(1, 1);
                   stretchMode += 0;
                   shrinkMode += 0;
                   maxDelay = 0;
              }
};

class TemporalGroup : public MediaObject
{
         BIFS_DECLARE_NODE

         eventIn MFNodeType<SFTemporalNode> addChildren;
         eventIn MFNodeType<SFTemporalNode> removeChildren;
         exposedField MFNodeType<SFTemporalNode> children;
         field SF<Bool> costart;
         field SF<Bool> coend;
         field SF<Bool> meet;
         exposedField MF<float> priority;
         eventOut SF<Bool> isActive;
         eventOut SF<int> activeChild;
         void OnaddChildren();
         void OnremoveChildren();

         public:
              TemporalGroup() {
              }
};

class ServerCommand : public StreamConsumer
{
         BIFS_DECLARE_NODE

         eventIn SF<Bool> trigger;
         exposedField SFString command;
         exposedField SF<Bool> enable;
         void Ontrigger();

         public:
              ServerCommand() {
                   trigger = FALSE;
                   enable = FALSE;
              }
};

class MediaSensor : public StreamConsumer
{
         BIFS_DECLARE_NODE

         eventOut SF<Time> mediaCurrentTime;
         eventOut SF<Time> streamObjectStartTime;
         eventOut SF<Time> mediaDuration;
         eventOut SF<Bool> isActive;
         eventOut MF<ZString> info;

         public:
              MediaSensor() {
              }
};

class MediaControl : public StreamConsumer
{
         BIFS_DECLARE_NODE

         exposedField SF<Time> mediaStartTime;
         exposedField SF<Time> mediaStopTime;
         exposedField SF<float> mediaSpeed;
         exposedField SF<Bool> loop;
         exposedField SF<Bool> preRoll;
         exposedField SF<Bool> mute;
         exposedField SF<Bool> enabled;
         eventOut SF<Bool> isPreRolled;

         public:
              MediaControl() {
                   mediaStartTime = -1;
                   mediaStopTime = +FLT_MAX;
                   mediaSpeed = 1;
                   loop = FALSE;
                   preRoll = TRUE;
                   mute = FALSE;
                   enabled = TRUE;
              }
};

class MediaBuffer : public StreamConsumer
{
         BIFS_DECLARE_NODE

         exposedField SF<float> bufferSize;
         exposedField SF<Time> mediaStartTime;
         exposedField SF<Time> mediaStopTime;
         eventOut SF<Bool> isBuffered;
         exposedField SF<Bool> enabled;

         public:
              MediaBuffer() {
                   bufferSize = 0;
                   mediaStartTime = -1;
                   mediaStopTime = +FLT_MAX;
                   enabled = TRUE;
              }
};

class MatteTexture : public MediaObject
{
         BIFS_DECLARE_NODE

         field SFTextureNode surface1;
         field SFTextureNode surface2;
         field SFTextureNode alphaSurface;
         exposedField SFString operation;
         field SF<Bool> overwrite;
         exposedField SF<float> fraction;
         exposedField MF<float> parameter;

         public:
              MatteTexture() {
                   overwrite = FALSE;
                   fraction = 0;
                   parameter += 0;
              }
};

class InputSensor : public StreamConsumer 
{
         BIFS_DECLARE_NODE

         exposedField SF<Bool> enabled;
         exposedField SFConditionalString buffer;
         eventOut SF<Bool> isActive;

         public:
              InputSensor() {
                   enabled = TRUE;
              }
};

#endif // _Nodes_h sentry.
/* End of automatically generated file */
