/***********************************************************************
    This software module  was  originally  developed by Zvi  Lifshitz
    (Optibase Ltd.) and  Mikael Bourges-Sevenier (iVAST Inc.)
    in the course of development of the MPEG-4 Systems (ISO/IEC 14496-1)
    standard.  This software module is an implementation of a part of
    one or more MPEG-4 Systems (ISO/IEC 14496-1) tools  as  specified
    by  the  MPEG-4 Systems (ISO/IEC 14496-1) standard. ISO/IEC gives
    users of the MPEG-4 Systems (ISO/IEC 14496-1)  free  license  to
    this software module or modifications thereof for use in hardware
    or software products claiming conformance to the  MPEG-4  Systems
    (ISO/IEC 14496-1). Those intending to use this software module in
    hardware or software  products  are  advised  that  its  use  may
    infringe   existing  patents.  The  original  developer  of  this
    software module and his/her company, the subsequent  editors  and
    their  companies,  and  ISO/IEC have no liability for use of this
    software module or modifications thereof  in  an  implementation.
    Copyright  is not released for non MPEG-4 Systems (ISO/IEC 14496-
    1) conforming  products.  Optibase Ltd. and iVAST Inc.
    retain  full  right to use the code for their own purpose, assign
    or donate the code to a third party and to inhibit third  parties
    from  using  the  code  for  non MPEG-4 Systems (ISO/IEC 14496-1)
    conforming products. This copyright notice must  be  included  in
    all copies or derivative works. Copyright  1997-2000.
************************************************************************/

/**************************************************************************
 * Project Im1Player
 * INTERNATIONAL STANDARD ORGANIZATION
 * ISO/IEC JTC1/SC29/WG11
 * 
 * SUBSYSTEM: BIFS Encoder/Decoder
 * FILE:      Nodes2.h
 * AUTHORS:   Zvi Lifshitz (Optibase Ltd.), Mikael Bourges-Sevenier (France Telecom CNET/DIH/HDM)
 * OVERVIEW:  Declaration of nodes and fields for version 2.
 * 
 * Comment:   Automatically generated on Fri Jun 09 09:58:24 PDT 2000
 *            at Santa Clara, CA, USA
 *            by parsing the MPEG-4 templates.txt file.
 *            Contact: mikael@ivast.com
 *
 **************************************************************************/
#include "float.h"
#include "StreamCo.h"
#include "NodeFld.h"
#include "bifs.h"
#include "Root.h"
#include "Conditional.h"

#if !defined(_Nodes2_h)  // Sentry, use file only if it's not already included.
#define _Nodes2_h

#pragma warning(disable: 4305)  // truncation from 'const double' to 'const float'

class AcousticMaterial : public MediaObject
{
         BIFS_DECLARE_NODE

         exposedField SF<float> ambientIntensity;
         exposedField SF<Color3f> diffuseColor;
         exposedField SF<Color3f> emissiveColor;
         exposedField SF<float> shininess;
         exposedField SF<Color3f> specularColor;
         exposedField SF<float> transparency;
         field MF<float> reffunc;
         field MF<float> transfunc;
         field MF<float> refFrequency;
         field MF<float> transFrequency;

         public:
              AcousticMaterial() {
                   ambientIntensity = 0.2;
                   diffuseColor = Color3f(0.8, 0.8, 0.8);
                   emissiveColor = Color3f(0, 0, 0);
                   shininess = 0.2;
                   specularColor = Color3f(0, 0, 0);
                   transparency = 0;
                   reffunc += 0;
                   transfunc += 1;
                   refFrequency += 0;
                   transFrequency += 0;
              }
};

class AcousticScene : public MediaObject
{
         BIFS_DECLARE_NODE

         field SF<Vec3f> center;
         field SF<Vec3f> Size;
         field MF<Time> reverbTime;
         field MF<float> reverbFreq;
         exposedField SF<float> reverbLevel;
         exposedField SF<Time> reverbDelay;

         public:
              AcousticScene() {
                   center = Vec3f(0, 0, 0);
                   Size = Vec3f(-1, -1, -1);
                   reverbTime += 0;
                   reverbFreq += 1000;
                   reverbLevel = 0.4;
                   reverbDelay = 0.5;
              }
};

class ApplicationWindow : public StreamConsumer
{
         BIFS_DECLARE_NODE

         exposedField SF<Bool> isActive;
         exposedField SF<Time> startTime;
         exposedField SF<Time> stopTime;
         exposedField SFString description;
         exposedField MF<ZString> parameter;
         exposedField SF<Vec2f> size;

         public:
              ApplicationWindow() {
                   isActive = FALSE;
                   startTime = 0;
                   stopTime = 0;
                   parameter += ZString("");
                   size = Vec2f(0, 0);
              }
};

class BAP : public MediaObject
{
         BIFS_DECLARE_NODE

         exposedField SF<int> sacroiliac_tilt;
         exposedField SF<int> sacroiliac_torsion;
         exposedField SF<int> sacroiliac_roll;
         exposedField SF<int> l_hip_flexion;
         exposedField SF<int> r_hip_flexion;
         exposedField SF<int> l_hip_abduct;
         exposedField SF<int> r_hip_abduct;
         exposedField SF<int> l_hip_twisting;
         exposedField SF<int> r_hip_twisting;
         exposedField SF<int> l_knee_flexion;
         exposedField SF<int> r_knee_flexion;
         exposedField SF<int> l_knee_twisting;
         exposedField SF<int> r_knee_twisting;
         exposedField SF<int> l_ankle_flexion;
         exposedField SF<int> r_ankle_flexion;
         exposedField SF<int> l_ankle_twisting;
         exposedField SF<int> r_ankle_twisting;
         exposedField SF<int> l_subtalar_flexion;
         exposedField SF<int> r_subtalar_flexion;
         exposedField SF<int> l_midtarsal_flexion;
         exposedField SF<int> r_midtarsal_flexion;
         exposedField SF<int> l_metatarsal_flexion;
         exposedField SF<int> r_metatarsal_flexion;
         exposedField SF<int> l_sternoclavicular_abduct;
         exposedField SF<int> r_sternoclavicular_abduct;
         exposedField SF<int> l_sternoclavicular_rotate;
         exposedField SF<int> r_sternoclavicular_rotate;
         exposedField SF<int> l_acromioclavicular_abduct;
         exposedField SF<int> r_acromioclavicular_abduct;
         exposedField SF<int> l_acromioclavicular_rotate;
         exposedField SF<int> r_acromioclavicular_rotate;
         exposedField SF<int> l_shoulder_flexion;
         exposedField SF<int> r_shoulder_flexion;
         exposedField SF<int> l_shoulder_abduct;
         exposedField SF<int> r_shoulder_abduct;
         exposedField SF<int> l_shoulder_twisting;
         exposedField SF<int> r_shoulder_twisting;
         exposedField SF<int> l_elbow_flexion;
         exposedField SF<int> r_elbow_flexion;
         exposedField SF<int> l_elbow_twisting;
         exposedField SF<int> r_elbow_twisting;
         exposedField SF<int> l_wrist_flexion;
         exposedField SF<int> r_wrist_flexion;
         exposedField SF<int> l_wrist_pivot;
         exposedField SF<int> r_wrist_pivot;
         exposedField SF<int> l_wrist_twisting;
         exposedField SF<int> r_wrist_twisting;
         exposedField SF<int> skullbase_roll;
         exposedField SF<int> skullbase_torsion;
         exposedField SF<int> skullbase_tilt;
         exposedField SF<int> vc1roll;
         exposedField SF<int> vc1torsion;
         exposedField SF<int> vc1tilt;
         exposedField SF<int> vc2roll;
         exposedField SF<int> vc2torsion;
         exposedField SF<int> vc2tilt;
         exposedField SF<int> vc3roll;
         exposedField SF<int> vc3torsion;
         exposedField SF<int> vc3tilt;
         exposedField SF<int> vc4roll;
         exposedField SF<int> vc4torsion;
         exposedField SF<int> vc4tilt;
         exposedField SF<int> vc5roll;
         exposedField SF<int> vc5torsion;
         exposedField SF<int> vc5tilt;
         exposedField SF<int> vc6roll;
         exposedField SF<int> vc6torsion;
         exposedField SF<int> vc6tilt;
         exposedField SF<int> vc7roll;
         exposedField SF<int> vc7torsion;
         exposedField SF<int> vc7tilt;
         exposedField SF<int> vt1roll;
         exposedField SF<int> vt1torsion;
         exposedField SF<int> vt1tilt;
         exposedField SF<int> vt2roll;
         exposedField SF<int> vt2torsion;
         exposedField SF<int> vt2tilt;
         exposedField SF<int> vt3roll;
         exposedField SF<int> vt3torsion;
         exposedField SF<int> vt3tilt;
         exposedField SF<int> vt4roll;
         exposedField SF<int> vt4torsion;
         exposedField SF<int> vt4tilt;
         exposedField SF<int> vt5roll;
         exposedField SF<int> vt5torsion;
         exposedField SF<int> vt5tilt;
         exposedField SF<int> vt6roll;
         exposedField SF<int> vt6torsion;
         exposedField SF<int> vt6tilt;
         exposedField SF<int> vt7roll;
         exposedField SF<int> vt7torsion;
         exposedField SF<int> vt7tilt;
         exposedField SF<int> vt8roll;
         exposedField SF<int> vt8torsion;
         exposedField SF<int> vt8tilt;
         exposedField SF<int> vt9roll;
         exposedField SF<int> vt9torsion;
         exposedField SF<int> vt9tilt;
         exposedField SF<int> vt10roll;
         exposedField SF<int> vt10torsion;
         exposedField SF<int> vt10tilt;
         exposedField SF<int> vt11roll;
         exposedField SF<int> vt11torsion;
         exposedField SF<int> vt11tilt;
         exposedField SF<int> vt12roll;
         exposedField SF<int> vt12torsion;
         exposedField SF<int> vt12tilt;
         exposedField SF<int> vl1roll;
         exposedField SF<int> vl1torsion;
         exposedField SF<int> vl1tilt;
         exposedField SF<int> vl2roll;
         exposedField SF<int> vl2torsion;
         exposedField SF<int> vl2tilt;
         exposedField SF<int> vl3roll;
         exposedField SF<int> vl3torsion;
         exposedField SF<int> vl3tilt;
         exposedField SF<int> vl4roll;
         exposedField SF<int> vl4torsion;
         exposedField SF<int> vl4tilt;
         exposedField SF<int> vl5roll;
         exposedField SF<int> vl5torsion;
         exposedField SF<int> vl5tilt;
         exposedField SF<int> l_pinky0_flexion;
         exposedField SF<int> r_pinky0_flexion;
         exposedField SF<int> l_pinky1_flexion;
         exposedField SF<int> r_pinky1_flexion;
         exposedField SF<int> l_pinky1_pivot;
         exposedField SF<int> r_pinky1_pivot;
         exposedField SF<int> l_pinky1_twisting;
         exposedField SF<int> r_pinky1_twisting;
         exposedField SF<int> l_pinky2_flexion;
         exposedField SF<int> r_pinky2_flexion;
         exposedField SF<int> l_pinky3_flexion;
         exposedField SF<int> r_pinky3_flexion;
         exposedField SF<int> l_ring0_flexion;
         exposedField SF<int> r_ring0_flexion;
         exposedField SF<int> l_ring1_flexion;
         exposedField SF<int> r_ring1_flexion;
         exposedField SF<int> l_ring1_pivot;
         exposedField SF<int> r_ring1_pivot;
         exposedField SF<int> l_ring1_twisting;
         exposedField SF<int> r_ring1_twisting;
         exposedField SF<int> l_ring2_flexion;
         exposedField SF<int> r_ring2_flexion;
         exposedField SF<int> l_ring3_flexion;
         exposedField SF<int> r_ring3_flexion;
         exposedField SF<int> l_middle0_flexion;
         exposedField SF<int> r_middle0_flexion;
         exposedField SF<int> l_middle1_flexion;
         exposedField SF<int> r_middle1_flexion;
         exposedField SF<int> l_middle1_pivot;
         exposedField SF<int> r_middle1_pivot;
         exposedField SF<int> l_middle1_twisting;
         exposedField SF<int> r_middle1_twisting;
         exposedField SF<int> l_middle2_flexion;
         exposedField SF<int> r_middle2_flexion;
         exposedField SF<int> l_middle3_flexion;
         exposedField SF<int> r_middle3_flexion;
         exposedField SF<int> l_index0_flexion;
         exposedField SF<int> r_index0_flexion;
         exposedField SF<int> l_index1_flexion;
         exposedField SF<int> r_index1_flexion;
         exposedField SF<int> l_index1_pivot;
         exposedField SF<int> r_index1_pivot;
         exposedField SF<int> l_index1_twisting;
         exposedField SF<int> r_index1_twisting;
         exposedField SF<int> l_index2_flexion;
         exposedField SF<int> r_index2_flexion;
         exposedField SF<int> l_index3_flexion;
         exposedField SF<int> r_index3_flexion;
         exposedField SF<int> l_thumb1_flexion;
         exposedField SF<int> r_thumb1_flexion;
         exposedField SF<int> l_thumb1_pivot;
         exposedField SF<int> r_thumb1_pivot;
         exposedField SF<int> l_thumb1_twisting;
         exposedField SF<int> r_thumb1_twisting;
         exposedField SF<int> l_thumb2_flexion;
         exposedField SF<int> r_thumb2_flexion;
         exposedField SF<int> l_thumb3_flexion;
         exposedField SF<int> r_thumb3_flexion;
         exposedField SF<int> HumanoidRoot_tr_vertical;
         exposedField SF<int> HumanoidRoot_tr_lateral;
         exposedField SF<int> HumanoidRoot_tr_frontal;
         exposedField SF<int> HumanoidRoot_rt_body_turn;
         exposedField SF<int> HumanoidRoot_rt_body_roll;
         exposedField SF<int> HumanoidRoot_rt_body_tilt;
         exposedField SF<int> extensionBap187;
         exposedField SF<int> extensionBap188;
         exposedField SF<int> extensionBap189;
         exposedField SF<int> extensionBap190;
         exposedField SF<int> extensionBap191;
         exposedField SF<int> extensionBap192;
         exposedField SF<int> extensionBap193;
         exposedField SF<int> extensionBap194;
         exposedField SF<int> extensionBap195;
         exposedField SF<int> extensionBap196;
         exposedField SF<int> extensionBap197;
         exposedField SF<int> extensionBap198;
         exposedField SF<int> extensionBap199;
         exposedField SF<int> extensionBap200;
         exposedField SF<int> extensionBap201;
         exposedField SF<int> extensionBap202;
         exposedField SF<int> extensionBap203;
         exposedField SF<int> extensionBap204;
         exposedField SF<int> extensionBap205;
         exposedField SF<int> extensionBap206;
         exposedField SF<int> extensionBap207;
         exposedField SF<int> extensionBap208;
         exposedField SF<int> extensionBap209;
         exposedField SF<int> extensionBap210;
         exposedField SF<int> extensionBap211;
         exposedField SF<int> extensionBap212;
         exposedField SF<int> extensionBap213;
         exposedField SF<int> extensionBap214;
         exposedField SF<int> extensionBap215;
         exposedField SF<int> extensionBap216;
         exposedField SF<int> extensionBap217;
         exposedField SF<int> extensionBap218;
         exposedField SF<int> extensionBap219;
         exposedField SF<int> extensionBap220;
         exposedField SF<int> extensionBap221;
         exposedField SF<int> extensionBap222;
         exposedField SF<int> extensionBap223;
         exposedField SF<int> extensionBap224;
         exposedField SF<int> extensionBap225;
         exposedField SF<int> extensionBap226;
         exposedField SF<int> extensionBap227;
         exposedField SF<int> extensionBap228;
         exposedField SF<int> extensionBap229;
         exposedField SF<int> extensionBap230;
         exposedField SF<int> extensionBap231;
         exposedField SF<int> extensionBap232;
         exposedField SF<int> extensionBap233;
         exposedField SF<int> extensionBap234;
         exposedField SF<int> extensionBap235;
         exposedField SF<int> extensionBap236;
         exposedField SF<int> extensionBap237;
         exposedField SF<int> extensionBap238;
         exposedField SF<int> extensionBap239;
         exposedField SF<int> extensionBap240;
         exposedField SF<int> extensionBap241;
         exposedField SF<int> extensionBap242;
         exposedField SF<int> extensionBap243;
         exposedField SF<int> extensionBap244;
         exposedField SF<int> extensionBap245;
         exposedField SF<int> extensionBap246;
         exposedField SF<int> extensionBap247;
         exposedField SF<int> extensionBap248;
         exposedField SF<int> extensionBap249;
         exposedField SF<int> extensionBap250;
         exposedField SF<int> extensionBap251;
         exposedField SF<int> extensionBap252;
         exposedField SF<int> extensionBap253;
         exposedField SF<int> extensionBap254;
         exposedField SF<int> extensionBap255;
         exposedField SF<int> extensionBap256;
         exposedField SF<int> extensionBap257;
         exposedField SF<int> extensionBap258;
         exposedField SF<int> extensionBap259;
         exposedField SF<int> extensionBap260;
         exposedField SF<int> extensionBap261;
         exposedField SF<int> extensionBap262;
         exposedField SF<int> extensionBap263;
         exposedField SF<int> extensionBap264;
         exposedField SF<int> extensionBap265;
         exposedField SF<int> extensionBap266;
         exposedField SF<int> extensionBap267;
         exposedField SF<int> extensionBap268;
         exposedField SF<int> extensionBap269;
         exposedField SF<int> extensionBap270;
         exposedField SF<int> extensionBap271;
         exposedField SF<int> extensionBap272;
         exposedField SF<int> extensionBap273;
         exposedField SF<int> extensionBap274;
         exposedField SF<int> extensionBap275;
         exposedField SF<int> extensionBap276;
         exposedField SF<int> extensionBap277;
         exposedField SF<int> extensionBap278;
         exposedField SF<int> extensionBap279;
         exposedField SF<int> extensionBap280;
         exposedField SF<int> extensionBap281;
         exposedField SF<int> extensionBap282;
         exposedField SF<int> extensionBap283;
         exposedField SF<int> extensionBap284;
         exposedField SF<int> extensionBap285;
         exposedField SF<int> extensionBap286;
         exposedField SF<int> extensionBap287;
         exposedField SF<int> extensionBap288;
         exposedField SF<int> extensionBap289;
         exposedField SF<int> extensionBap290;
         exposedField SF<int> extensionBap291;
         exposedField SF<int> extensionBap292;
         exposedField SF<int> extensionBap293;
         exposedField SF<int> extensionBap294;
         exposedField SF<int> extensionBap295;
         exposedField SF<int> extensionBap296;

         public:
              BAP() {
                   sacroiliac_tilt = INT_MAX;
                   sacroiliac_torsion = INT_MAX;
                   sacroiliac_roll = INT_MAX;
                   l_hip_flexion = INT_MAX;
                   r_hip_flexion = INT_MAX;
                   l_hip_abduct = INT_MAX;
                   r_hip_abduct = INT_MAX;
                   l_hip_twisting = INT_MAX;
                   r_hip_twisting = INT_MAX;
                   l_knee_flexion = INT_MAX;
                   r_knee_flexion = INT_MAX;
                   l_knee_twisting = INT_MAX;
                   r_knee_twisting = INT_MAX;
                   l_ankle_flexion = INT_MAX;
                   r_ankle_flexion = INT_MAX;
                   l_ankle_twisting = INT_MAX;
                   r_ankle_twisting = INT_MAX;
                   l_subtalar_flexion = INT_MAX;
                   r_subtalar_flexion = INT_MAX;
                   l_midtarsal_flexion = INT_MAX;
                   r_midtarsal_flexion = INT_MAX;
                   l_metatarsal_flexion = INT_MAX;
                   r_metatarsal_flexion = INT_MAX;
                   l_sternoclavicular_abduct = INT_MAX;
                   r_sternoclavicular_abduct = INT_MAX;
                   l_sternoclavicular_rotate = INT_MAX;
                   r_sternoclavicular_rotate = INT_MAX;
                   l_acromioclavicular_abduct = INT_MAX;
                   r_acromioclavicular_abduct = INT_MAX;
                   l_acromioclavicular_rotate = INT_MAX;
                   r_acromioclavicular_rotate = INT_MAX;
                   l_shoulder_flexion = INT_MAX;
                   r_shoulder_flexion = INT_MAX;
                   l_shoulder_abduct = INT_MAX;
                   r_shoulder_abduct = INT_MAX;
                   l_shoulder_twisting = INT_MAX;
                   r_shoulder_twisting = INT_MAX;
                   l_elbow_flexion = INT_MAX;
                   r_elbow_flexion = INT_MAX;
                   l_elbow_twisting = INT_MAX;
                   r_elbow_twisting = INT_MAX;
                   l_wrist_flexion = INT_MAX;
                   r_wrist_flexion = INT_MAX;
                   l_wrist_pivot = INT_MAX;
                   r_wrist_pivot = INT_MAX;
                   l_wrist_twisting = INT_MAX;
                   r_wrist_twisting = INT_MAX;
                   skullbase_roll = INT_MAX;
                   skullbase_torsion = INT_MAX;
                   skullbase_tilt = INT_MAX;
                   vc1roll = INT_MAX;
                   vc1torsion = INT_MAX;
                   vc1tilt = INT_MAX;
                   vc2roll = INT_MAX;
                   vc2torsion = INT_MAX;
                   vc2tilt = INT_MAX;
                   vc3roll = INT_MAX;
                   vc3torsion = INT_MAX;
                   vc3tilt = INT_MAX;
                   vc4roll = INT_MAX;
                   vc4torsion = INT_MAX;
                   vc4tilt = INT_MAX;
                   vc5roll = INT_MAX;
                   vc5torsion = INT_MAX;
                   vc5tilt = INT_MAX;
                   vc6roll = INT_MAX;
                   vc6torsion = INT_MAX;
                   vc6tilt = INT_MAX;
                   vc7roll = INT_MAX;
                   vc7torsion = INT_MAX;
                   vc7tilt = INT_MAX;
                   vt1roll = INT_MAX;
                   vt1torsion = INT_MAX;
                   vt1tilt = INT_MAX;
                   vt2roll = INT_MAX;
                   vt2torsion = INT_MAX;
                   vt2tilt = INT_MAX;
                   vt3roll = INT_MAX;
                   vt3torsion = INT_MAX;
                   vt3tilt = INT_MAX;
                   vt4roll = INT_MAX;
                   vt4torsion = INT_MAX;
                   vt4tilt = INT_MAX;
                   vt5roll = INT_MAX;
                   vt5torsion = INT_MAX;
                   vt5tilt = INT_MAX;
                   vt6roll = INT_MAX;
                   vt6torsion = INT_MAX;
                   vt6tilt = INT_MAX;
                   vt7roll = INT_MAX;
                   vt7torsion = INT_MAX;
                   vt7tilt = INT_MAX;
                   vt8roll = INT_MAX;
                   vt8torsion = INT_MAX;
                   vt8tilt = INT_MAX;
                   vt9roll = INT_MAX;
                   vt9torsion = INT_MAX;
                   vt9tilt = INT_MAX;
                   vt10roll = INT_MAX;
                   vt10torsion = INT_MAX;
                   vt10tilt = INT_MAX;
                   vt11roll = INT_MAX;
                   vt11torsion = INT_MAX;
                   vt11tilt = INT_MAX;
                   vt12roll = INT_MAX;
                   vt12torsion = INT_MAX;
                   vt12tilt = INT_MAX;
                   vl1roll = INT_MAX;
                   vl1torsion = INT_MAX;
                   vl1tilt = INT_MAX;
                   vl2roll = INT_MAX;
                   vl2torsion = INT_MAX;
                   vl2tilt = INT_MAX;
                   vl3roll = INT_MAX;
                   vl3torsion = INT_MAX;
                   vl3tilt = INT_MAX;
                   vl4roll = INT_MAX;
                   vl4torsion = INT_MAX;
                   vl4tilt = INT_MAX;
                   vl5roll = INT_MAX;
                   vl5torsion = INT_MAX;
                   vl5tilt = INT_MAX;
                   l_pinky0_flexion = INT_MAX;
                   r_pinky0_flexion = INT_MAX;
                   l_pinky1_flexion = INT_MAX;
                   r_pinky1_flexion = INT_MAX;
                   l_pinky1_pivot = INT_MAX;
                   r_pinky1_pivot = INT_MAX;
                   l_pinky1_twisting = INT_MAX;
                   r_pinky1_twisting = INT_MAX;
                   l_pinky2_flexion = INT_MAX;
                   r_pinky2_flexion = INT_MAX;
                   l_pinky3_flexion = INT_MAX;
                   r_pinky3_flexion = INT_MAX;
                   l_ring0_flexion = INT_MAX;
                   r_ring0_flexion = INT_MAX;
                   l_ring1_flexion = INT_MAX;
                   r_ring1_flexion = INT_MAX;
                   l_ring1_pivot = INT_MAX;
                   r_ring1_pivot = INT_MAX;
                   l_ring1_twisting = INT_MAX;
                   r_ring1_twisting = INT_MAX;
                   l_ring2_flexion = INT_MAX;
                   r_ring2_flexion = INT_MAX;
                   l_ring3_flexion = INT_MAX;
                   r_ring3_flexion = INT_MAX;
                   l_middle0_flexion = INT_MAX;
                   r_middle0_flexion = INT_MAX;
                   l_middle1_flexion = INT_MAX;
                   r_middle1_flexion = INT_MAX;
                   l_middle1_pivot = INT_MAX;
                   r_middle1_pivot = INT_MAX;
                   l_middle1_twisting = INT_MAX;
                   r_middle1_twisting = INT_MAX;
                   l_middle2_flexion = INT_MAX;
                   r_middle2_flexion = INT_MAX;
                   l_middle3_flexion = INT_MAX;
                   r_middle3_flexion = INT_MAX;
                   l_index0_flexion = INT_MAX;
                   r_index0_flexion = INT_MAX;
                   l_index1_flexion = INT_MAX;
                   r_index1_flexion = INT_MAX;
                   l_index1_pivot = INT_MAX;
                   r_index1_pivot = INT_MAX;
                   l_index1_twisting = INT_MAX;
                   r_index1_twisting = INT_MAX;
                   l_index2_flexion = INT_MAX;
                   r_index2_flexion = INT_MAX;
                   l_index3_flexion = INT_MAX;
                   r_index3_flexion = INT_MAX;
                   l_thumb1_flexion = INT_MAX;
                   r_thumb1_flexion = INT_MAX;
                   l_thumb1_pivot = INT_MAX;
                   r_thumb1_pivot = INT_MAX;
                   l_thumb1_twisting = INT_MAX;
                   r_thumb1_twisting = INT_MAX;
                   l_thumb2_flexion = INT_MAX;
                   r_thumb2_flexion = INT_MAX;
                   l_thumb3_flexion = INT_MAX;
                   r_thumb3_flexion = INT_MAX;
                   HumanoidRoot_tr_vertical = INT_MAX;
                   HumanoidRoot_tr_lateral = INT_MAX;
                   HumanoidRoot_tr_frontal = INT_MAX;
                   HumanoidRoot_rt_body_turn = INT_MAX;
                   HumanoidRoot_rt_body_roll = INT_MAX;
                   HumanoidRoot_rt_body_tilt = INT_MAX;
                   extensionBap187 = INT_MAX;
                   extensionBap188 = INT_MAX;
                   extensionBap189 = INT_MAX;
                   extensionBap190 = INT_MAX;
                   extensionBap191 = INT_MAX;
                   extensionBap192 = INT_MAX;
                   extensionBap193 = INT_MAX;
                   extensionBap194 = INT_MAX;
                   extensionBap195 = INT_MAX;
                   extensionBap196 = INT_MAX;
                   extensionBap197 = INT_MAX;
                   extensionBap198 = INT_MAX;
                   extensionBap199 = INT_MAX;
                   extensionBap200 = INT_MAX;
                   extensionBap201 = INT_MAX;
                   extensionBap202 = INT_MAX;
                   extensionBap203 = INT_MAX;
                   extensionBap204 = INT_MAX;
                   extensionBap205 = INT_MAX;
                   extensionBap206 = INT_MAX;
                   extensionBap207 = INT_MAX;
                   extensionBap208 = INT_MAX;
                   extensionBap209 = INT_MAX;
                   extensionBap210 = INT_MAX;
                   extensionBap211 = INT_MAX;
                   extensionBap212 = INT_MAX;
                   extensionBap213 = INT_MAX;
                   extensionBap214 = INT_MAX;
                   extensionBap215 = INT_MAX;
                   extensionBap216 = INT_MAX;
                   extensionBap217 = INT_MAX;
                   extensionBap218 = INT_MAX;
                   extensionBap219 = INT_MAX;
                   extensionBap220 = INT_MAX;
                   extensionBap221 = INT_MAX;
                   extensionBap222 = INT_MAX;
                   extensionBap223 = INT_MAX;
                   extensionBap224 = INT_MAX;
                   extensionBap225 = INT_MAX;
                   extensionBap226 = INT_MAX;
                   extensionBap227 = INT_MAX;
                   extensionBap228 = INT_MAX;
                   extensionBap229 = INT_MAX;
                   extensionBap230 = INT_MAX;
                   extensionBap231 = INT_MAX;
                   extensionBap232 = INT_MAX;
                   extensionBap233 = INT_MAX;
                   extensionBap234 = INT_MAX;
                   extensionBap235 = INT_MAX;
                   extensionBap236 = INT_MAX;
                   extensionBap237 = INT_MAX;
                   extensionBap238 = INT_MAX;
                   extensionBap239 = INT_MAX;
                   extensionBap240 = INT_MAX;
                   extensionBap241 = INT_MAX;
                   extensionBap242 = INT_MAX;
                   extensionBap243 = INT_MAX;
                   extensionBap244 = INT_MAX;
                   extensionBap245 = INT_MAX;
                   extensionBap246 = INT_MAX;
                   extensionBap247 = INT_MAX;
                   extensionBap248 = INT_MAX;
                   extensionBap249 = INT_MAX;
                   extensionBap250 = INT_MAX;
                   extensionBap251 = INT_MAX;
                   extensionBap252 = INT_MAX;
                   extensionBap253 = INT_MAX;
                   extensionBap254 = INT_MAX;
                   extensionBap255 = INT_MAX;
                   extensionBap256 = INT_MAX;
                   extensionBap257 = INT_MAX;
                   extensionBap258 = INT_MAX;
                   extensionBap259 = INT_MAX;
                   extensionBap260 = INT_MAX;
                   extensionBap261 = INT_MAX;
                   extensionBap262 = INT_MAX;
                   extensionBap263 = INT_MAX;
                   extensionBap264 = INT_MAX;
                   extensionBap265 = INT_MAX;
                   extensionBap266 = INT_MAX;
                   extensionBap267 = INT_MAX;
                   extensionBap268 = INT_MAX;
                   extensionBap269 = INT_MAX;
                   extensionBap270 = INT_MAX;
                   extensionBap271 = INT_MAX;
                   extensionBap272 = INT_MAX;
                   extensionBap273 = INT_MAX;
                   extensionBap274 = INT_MAX;
                   extensionBap275 = INT_MAX;
                   extensionBap276 = INT_MAX;
                   extensionBap277 = INT_MAX;
                   extensionBap278 = INT_MAX;
                   extensionBap279 = INT_MAX;
                   extensionBap280 = INT_MAX;
                   extensionBap281 = INT_MAX;
                   extensionBap282 = INT_MAX;
                   extensionBap283 = INT_MAX;
                   extensionBap284 = INT_MAX;
                   extensionBap285 = INT_MAX;
                   extensionBap286 = INT_MAX;
                   extensionBap287 = INT_MAX;
                   extensionBap288 = INT_MAX;
                   extensionBap289 = INT_MAX;
                   extensionBap290 = INT_MAX;
                   extensionBap291 = INT_MAX;
                   extensionBap292 = INT_MAX;
                   extensionBap293 = INT_MAX;
                   extensionBap294 = INT_MAX;
                   extensionBap295 = INT_MAX;
                   extensionBap296 = INT_MAX;
              }
};

class BDP : public MediaObject
{
         BIFS_DECLARE_NODE

         exposedField MFNode<class BodyDefTable> bodyDefTables;
         exposedField MFNodeType<SF3DNode> bodySceneGraph;

         public:
              BDP() {
              }
};

class Body : public MediaObject
{
         BIFS_DECLARE_NODE

         exposedField SFNode<class BDP> bdp;
         exposedField SFNode<class BAP> bap;
         exposedField MFNodeType<SF3DNode> renderedBody;

         public:
              Body() {
              }
};

class BodyDefTable : public MediaObject
{
         BIFS_DECLARE_NODE

         exposedField SFString bodySceneGraphNodeName;
         exposedField MF<int> bapIDs;
         exposedField MF<int> vertexIds;
         exposedField MF<int> bapCombinations;
         exposedField MF<Vec3f> displacements;
         exposedField SF<int> numInterpolateKeys;

         public:
              BodyDefTable() {
                   numInterpolateKeys = 2;
              }
};

class BodySegmentConnectionHint : public MediaObject
{
         BIFS_DECLARE_NODE

         exposedField SFString firstSegmentNodeName;
         exposedField SFString secondSegmentNodeName;
         exposedField MF<int> firstVertexIdList;
         exposedField MF<int> secondVertexIdList;

         public:
              BodySegmentConnectionHint() {
                   firstVertexIdList += NULL;
                   secondVertexIdList += NULL;
              }
};

class DirectiveSound : public MediaObject
{
         BIFS_DECLARE_NODE

         exposedField SF<Vec3f> direction;
         exposedField SF<float> intensity;
         exposedField SF<Vec3f> location;
         exposedField SFAudioNode source;
         exposedField SFNode<class PerceptualParameters> perceptualParameters;
         exposedField SF<Bool> roomEffect;
         exposedField SF<Bool> spatialize;
         field MF<float> directivity;
         field MF<float> angles;
         field MF<float> frequency;
         field SF<float> speedOfSound;
         field SF<float> distance;
         field SF<Bool> useAirabs;

         public:
              DirectiveSound() {
                   direction = Vec3f(0, 0, -1);
                   intensity = 1;
                   location = Vec3f(0, 0, 0);
                   roomEffect = FALSE;
                   spatialize = TRUE;
                   directivity += 1;
                   angles += 1;
                   speedOfSound = 340;
                   distance = 100;
                   useAirabs = FALSE;
              }
};

class Hierarchical3DMesh : public StreamConsumer
{
         BIFS_DECLARE_NODE

         eventIn SF<int> triangleBudget;
         exposedField SF<float> level;
         eventOut SF<Bool> doneLoading;
         void OntriangleBudget();

         public:
              Hierarchical3DMesh() {
                   triangleBudget = 1000;
                   level = 1;
              }
};

class MaterialKey : public MediaObject
{
         BIFS_DECLARE_NODE

         exposedField SF<Bool> isKeyed;
         exposedField SF<Bool> isRGB;
         exposedField SF<Color3f> keyColor;
         exposedField SF<float> lowThreshold;
         exposedField SF<float> highThreshold;
         exposedField SF<float> transparency;

         public:
              MaterialKey() {
                   isKeyed = TRUE;
                   isRGB = TRUE;
                   keyColor = Color3f(0, 0, 0);
                   lowThreshold = 0;
                   highThreshold = 0;
                   transparency = 0;
              }
};

class PerceptualParameters : public MediaObject
{
         BIFS_DECLARE_NODE

         exposedField SF<float> sourcePresence;
         exposedField SF<float> sourceWarmth;
         exposedField SF<float> sourceBrilliance;
         exposedField SF<float> roomPresence;
         exposedField SF<float> runningReverberance;
         exposedField SF<float> envelopment;
         exposedField SF<float> lateReverberance;
         exposedField SF<float> heavyness;
         exposedField SF<float> liveness;
         exposedField MF<float> omniDirectivity;
         exposedField MF<float> directFilterGains;
         exposedField MF<float> inputFilterGains;
         exposedField SF<float> refDistance;
         exposedField SF<float> freqLow;
         exposedField SF<float> freqHigh;
         exposedField SF<Time> timeLimit1;
         exposedField SF<Time> timeLimit2;
         exposedField SF<Time> timeLimit3;
         exposedField SF<Time> modalDensity;

         public:
              PerceptualParameters() {
                   sourcePresence = 1;
                   sourceWarmth = 1;
                   sourceBrilliance = 1;
                   roomPresence = 1;
                   runningReverberance = 1;
                   envelopment = 0;
                   lateReverberance = 1;
                   heavyness = 1;
                   liveness = 1;
                   omniDirectivity += 1;
                   directFilterGains += 1;
                   directFilterGains += 1;
                   directFilterGains += 1;
                   inputFilterGains += 1;
                   inputFilterGains += 1;
                   inputFilterGains += 1;
                   refDistance = 1;
                   freqLow = 250;
                   freqHigh = 4000;
                   timeLimit1 = 0.02;
                   timeLimit2 = 0.04;
                   timeLimit3 = 0.1;
                   modalDensity = 0.8;
              }
};

#endif // _Nodes2_h sentry.
/* End of automatically generated file */
