/***********************************************************************
    This software module  was  originally  developed by Zvi  Lifshitz
    (Optibase Ltd.) and  Mikael Bourges-Sevenier (iVAST Inc.)
    in the course of development of the MPEG-4 Systems (ISO/IEC 14496-1)
    standard.  This software module is an implementation of a part of
    one or more MPEG-4 Systems (ISO/IEC 14496-1) tools  as  specified
    by  the  MPEG-4 Systems (ISO/IEC 14496-1) standard. ISO/IEC gives
    users of the MPEG-4 Systems (ISO/IEC 14496-1)  free  license  to
    this software module or modifications thereof for use in hardware
    or software products claiming conformance to the  MPEG-4  Systems
    (ISO/IEC 14496-1). Those intending to use this software module in
    hardware or software  products  are  advised  that  its  use  may
    infringe   existing  patents.  The  original  developer  of  this
    software module and his/her company, the subsequent  editors  and
    their  companies,  and  ISO/IEC have no liability for use of this
    software module or modifications thereof  in  an  implementation.
    Copyright  is not released for non MPEG-4 Systems (ISO/IEC 14496-
    1) conforming  products.  Optibase Ltd. and iVAST Inc.
    retain  full  right to use the code for their own purpose, assign
    or donate the code to a third party and to inhibit third  parties
    from  using  the  code  for  non MPEG-4 Systems (ISO/IEC 14496-1)
    conforming products. This copyright notice must  be  included  in
    all copies or derivative works. Copyright  1997-2000.
************************************************************************/

/**************************************************************************
 * Project Im1Player
 * INTERNATIONAL STANDARD ORGANIZATION
 * ISO/IEC JTC1/SC29/WG11
 * 
 * SUBSYSTEM: BIFS Encoder/Decoder
 * FILE:      Nodes3.h
 * AUTHORS:   Zvi Lifshitz (Optibase Ltd.), Mikael Bourges-Sevenier (France Telecom CNET/DIH/HDM)
 * OVERVIEW:  Declaration of nodes and fields for version 3.
 * 
 * Comment:   Automatically generated on Fri Jun 09 09:58:24 PDT 2000
 *            at Santa Clara, CA, USA
 *            by parsing the MPEG-4 templates.txt file.
 *            Contact: mikael@ivast.com
 *
 **************************************************************************/

#if !defined(_Nodes3_h)  // Sentry, use file only if it's not already included.
#define _Nodes3_h

#include "StreamCo.h"
#include "bifs.h"

#pragma warning(disable: 4305)  // truncation from 'const double' to 'const float'

/* Currently, we have 5 nodes defined. */

class TemporalTransform : public StreamConsumer
{
         BIFS_DECLARE_NODE

         eventIn MFNodeType<SF3DNode> addChildren;
         eventIn MFNodeType<SF3DNode> removeChildren;
         exposedField MFNodeType<SF3DNode> children;
         exposedField SF<Time> startTime;
         exposedField SF<Time> optimalDuration;
         exposedField SF<Bool> active;
         exposedField SF<float> speed;
         exposedField SF<Vec2f> scalability;
         exposedField MF<int> stretchMode;
         exposedField MF<int> shrinkMode;
         exposedField SF<Time> maxDelay;
         eventOut SF<Time> actualDuration;
         void OnaddChildren();
         void OnremoveChildren();

         public:
              TemporalTransform() {
                   startTime = -1;
                   optimalDuration = -1;
                   active = FALSE;
                   speed = 1;
                   scalability = Vec2f(1, 1);
                   stretchMode += 0;
                   shrinkMode += 0;
                   maxDelay = 0;
              }
};

class TemporalGroup : public MediaObject
{
         BIFS_DECLARE_NODE

         eventIn MFNodeType<SF3DNode> addChildren;
         eventIn MFNodeType<SF3DNode> removeChildren;
         exposedField MF<float> priority;
         exposedField MFNodeType<SF3DNode> children;
         field SF<Bool> costart;
         field SF<Bool> coend;
         field SF<Bool> meet;
         eventOut SF<Bool> isActive;
         eventOut SF<int> activeChild;
         void OnaddChildren();
         void OnremoveChildren();

         public:
              TemporalGroup() {
              }
};

class MediaSensor : public StreamConsumer
{
         BIFS_DECLARE_NODE

         eventOut SF<Time> mediaCurrentTime;
         eventOut SF<Bool> isActive;
         eventOut SF<Time> compositionStartTime;
         eventOut SF<Time> mediaDuration;
         eventOut MF<ZString> info;

         public:
              MediaSensor() {
              }
};

class ServerCommand : public StreamConsumer
{
         BIFS_DECLARE_NODE

         eventIn SF<Bool> trigger;
         exposedField SF<Bool> enable;
         exposedField SFString command;
         void Ontrigger();

         public:
              ServerCommand() {
                   trigger = FALSE;
                   enable = FALSE;
              }
};

class MediaControl : public MediaObject
{
         BIFS_DECLARE_NODE

         exposedField SF<Time> duration;
         exposedField SF<Bool> enabled;
         exposedField SF<Bool> loop;
         exposedField MFNodeType<SFWorldNode> media;
         exposedField SF<float> mediaRate;
         exposedField SF<Time> mediaStartTime;
         exposedField SF<Time> mediaStopTime;
         exposedField SF<Time> playTime;
         exposedField SF<Time> startTime;
         exposedField SF<Bool> postroll;
         eventOut SF<Bool> isActive;
         eventOut SF<Time> currentMediaTime;

         public:
              MediaControl() {
                   duration = -1;
                   enabled = TRUE;
                   loop = FALSE;
                   mediaRate = 1;
                   mediaStartTime = -1;
                   mediaStopTime = -1;
                   playTime = 0;
                   startTime = 0;
                   postroll = FALSE;
              }
};

#endif // _Nodes_h sentry.
/* End of automatically generated file */
