/***********************************************************************
This software module was originally developed by Zvi Lifshitz
(Triton R&D Ltd.) in the course of development of the MPEG-4 Systems
(ISO/IEC 14496-1) standard. This software module is an implementation of
a part of one or more MPEG-4 Systems (ISO/IEC 14496-1) tools as
specified by the MPEG-4 Systems (ISO/IEC 14496-1) standard. ISO/IEC
gives users of the MPEG-4 Systems (ISO/IEC 14496-1) free license to
this software module or modifications thereof for use in hardware or
software products claiming conformance to the MPEG-4 Systems (ISO/IEC
14496-1). Those intending to use this software module in hardware or
software products are advised that its use may infringe existing patents.
The original developer of this software module and his/her company, the
subsequent editors and their companies, and ISO/IEC have no liability
for use of this software module or modifications thereof in an
implementation. Copyright is not released for non MPEG-4 Systems
(ISO/IEC 14496-1) conforming products. Triton R&D Ltd. retains full right
to use the code for his/her own purpose, assign or donate the code to a
third party and to inhibit third parties from using the code for non
MPEG-4 Systems (ISO/IEC 14496-1) conforming products. This copyright
notice must be included in all copies or derivative works.
Copyright  1998.
************************************************************************

For more information or to receive an up to date version of this module,
you may call Zvi Lifshitz at zvil@csi.com.
----------------------------------------------------------------------*/

/*-------------------------------------------------------------------------
Project Im1Player/ Mux
INTERNATIONAL ORGANIZATION FOR STANDARDIZATION
ISO/IEC JTC1/SC29/WG11

SUBSYSTEM:  OD Decoder/ Encoder / Mux
FILE:       ODEncDec.cpp
AUTHOR:     Zvi Lifshitz (Triton R&D Ltd.)
OVERVIEW:   Implementation of parsing/encoding of descriptors of OD stream
-------------------------------------------------------------------------*/

#include "defs.h"
#include "odp.h"
#include "ODCoder.h"

void ObjectDescriptor :: DescEncDec (ODCoder *pCoder, OPTYPE op)
{
	ObjectDescriptorID.EncDec (pCoder, this, op);
	URL_Flag.EncDec (pCoder, this, op);
	reserved.EncDec (pCoder, this, op);
	if (URL_Flag) {
		URLstring.EncDec (pCoder, this, op);
	} else {
		esDescr.EncDec (pCoder, this, op);
		ociDescr.EncDec (pCoder, this, op);
		ipmpDescrPtr.EncDec (pCoder, this, op);
	}
	extDescr.EncDec (pCoder, this, op);
}

void InitialObjectDescriptor :: DescEncDec (ODCoder *pCoder, OPTYPE op)
{
	/*-------------------------------------------------------------
	InitialObjectDescriptor is derived from ObjectDescriptor. They
	differ by the value of the 'reserved' field.
	-------------------------------------------------------------*/
	if (reserved != 0xf)
	{	ObjectDescriptor :: DescEncDec (pCoder, op);
		return;
	}
	ObjectDescriptorID.EncDec (pCoder, this, op);
	URL_Flag.EncDec (pCoder, this, op);
	includeInlineProfilesFlag.EncDec (pCoder, this, op);
	reserved.EncDecVarSize (pCoder, this, op, 4);
	if (URL_Flag) {
		URLstring.EncDec (pCoder, this, op);
	} else {
		ODProfileLevelIndication.EncDec (pCoder, this, op);
		sceneProfileLevelIndication.EncDec (pCoder, this, op);
		audioProfileLevelIndication.EncDec (pCoder, this, op);
		visualProfileLevelIndication.EncDec (pCoder, this, op);
		graphicsProfileLevelIndication.EncDec (pCoder, this, op);
		esDescr.EncDec (pCoder, this, op);
		ociDescr.EncDec (pCoder, this, op);
		ipmpDescrPtr.EncDec (pCoder, this, op);
	}
	extDescr.EncDec (pCoder, this, op);
}

void ES_Descriptor :: DescEncDec (ODCoder *pCoder, OPTYPE op)
{
	ES_ID.EncDec (pCoder, this, op);
	streamDependenceFlag.EncDec (pCoder, this, op);
	URL_Flag.EncDec (pCoder, this, op);
	OCRstreamFlag.EncDec (pCoder, this, op);
	streamPriority.EncDec (pCoder, this, op);
	if (streamDependenceFlag)
		dependsOn_ES_ID.EncDec (pCoder, this, op);
	if (URL_Flag) {
		URLstring.EncDec (pCoder, this, op);
	}
	if (OCRstreamFlag)
		OCR_ES_Id.EncDec (pCoder, this, op);
	decConfigDescr.EncDec (pCoder, this, op);
	slConfigDescr.EncDec (pCoder, this, op);
	ipiPtr.EncDec (pCoder, this, op);
	ipIDS.EncDec (pCoder, this, op);
	ipmpDescrPtr.EncDec (pCoder, this, op);
	langDescr.EncDec (pCoder, this, op);
	qosDescr.EncDec (pCoder, this, op);
	regDescr.EncDec (pCoder, this, op);
	extDescr.EncDec (pCoder, this, op);
}

void ContentIdentificationDescriptor :: DescEncDec (ODCoder *pCoder, OPTYPE op)
{
	compatibility.EncDec (pCoder, this, op);
	contentTypeFlag.EncDec (pCoder, this, op);
	contentIdentifierFlag.EncDec (pCoder, this, op);
	protectedContent.EncDec (pCoder, this, op);
	reserved.EncDec (pCoder, this, op);
	if (contentTypeFlag)
		contentType.EncDec (pCoder, this, op);
	if (contentIdentifierFlag) {
		contentIdentifierType.EncDec (pCoder, this, op);
		contentIdentifier.EncDec (pCoder, this, op);
	}
}

void IPMP_Descriptor :: DescEncDec (ODCoder *pCoder, OPTYPE op)
{
	IPMP_DescriptorID.EncDec (pCoder, this, op);
	IPMPS_Type.EncDec (pCoder, this, op);
	if (IPMPS_Type == 0) {
		URLString.EncDec (pCoder, this, op);
	} else {
		IPMP_data.EncDec (pCoder, this, op);
	}
}

void QoS_Descriptor :: DescEncDec (ODCoder *pCoder, OPTYPE op)
{
	predefined.EncDec (pCoder, this, op);
	if (predefined==0)
		qualifiers.EncDec (pCoder, this, op);
}

void KeyWordDescriptor :: DescEncDec (ODCoder *pCoder, OPTYPE op)
{
	languageCode.EncDec (pCoder, this, op);  
	isUTF8_string.EncDec (pCoder, this, op);
	keyWordCount.EncDec (pCoder, this, op);
	keyWord.EncDecStringArray (pCoder, this, op, keyWordCount, !isUTF8_string);
}  

void ShortTextualDescriptor :: DescEncDec (ODCoder *pCoder, OPTYPE op)
{
	languageCode.EncDec (pCoder, this, op);  
	isUTF8_string.EncDec (pCoder, this, op);
	align.EncDec (pCoder, this, op);
	eventName.EncDecString (pCoder, this, op, !isUTF8_string);
	eventText.EncDecString (pCoder, this, op, !isUTF8_string);
}  

void ExpandedTextualDescriptor :: DescEncDec (ODCoder *pCoder, OPTYPE op)
{
	int i;
	languageCode.EncDec (pCoder, this, op);
	isUTF8_string.EncDec (pCoder, this, op);
	itemCount.EncDec (pCoder, this, op);
	for (i=0; i<itemCount; i++){  
		itemDescription.EncDecStringArrayElement (pCoder, this, op, i, !isUTF8_string);
		itemText.EncDecStringArrayElement (pCoder, this, op, i, !isUTF8_string);
	}  
	nonItemText.EncDecLongString (pCoder, this, op, !isUTF8_string);
}  

void ContentCreatorNameDescriptor :: DescEncDec (ODCoder *pCoder, OPTYPE op)
{
	int i;
	contentCreatorCount.EncDec (pCoder, this, op);
	for (i=0; i<contentCreatorCount; i++){  
		languageCode.EncDecArrayElement (pCoder, this, op, i);
		isUTF8_string.EncDecArrayElement (pCoder, this, op, i);
		align.EncDec (pCoder, this, op);
		contentCreatorName.EncDecStringArrayElement (pCoder, this, op, i, !isUTF8_string [i]);
	}  
}  

void OCICreatorNameDescriptor :: DescEncDec (ODCoder *pCoder, OPTYPE op)
{
	int i;
	OCICreatorCount.EncDec (pCoder, this, op);
	for (i=0; i<OCICreatorCount; i++) {  
		languageCode.EncDecArrayElement (pCoder, this, op, i);
		isUTF8_string.EncDecArrayElement (pCoder, this, op, i);
		align.EncDec (pCoder, this, op);
		OCICreatorName.EncDecStringArrayElement (pCoder, this, op, i, !isUTF8_string [i]);
	}  
}  

// NHK 990702
void SmpteCameraPositionDescriptor :: DescEncDec (ODCoder *pCoder, OPTYPE op)
{
	int	i;
	cameraID.EncDec(pCoder, this, op);
	parameterCount.EncDec(pCoder, this, op);
	for (i=0; i<parameterCount; i++) {
		parameterID.EncDecArrayElement (pCoder, this, op, i);
		parameter.EncDecArrayElement(pCoder, this, op, i);
	}
}

void SLConfigDescriptor :: DescEncDec (ODCoder *pCoder, OPTYPE op)
{
	predefined.EncDec (pCoder, this, op);
	if (predefined==0) {
		useAccessUnitStartFlag.EncDec (pCoder, this, op);
		useAccessUnitEndFlag.EncDec (pCoder, this, op);
		useRandomAccessPointFlag.EncDec (pCoder, this, op);
		hasRandomAccessUnitsOnlyFlag.EncDec (pCoder, this, op);
		usePaddingFlag.EncDec (pCoder, this, op);
		useTimeStampsFlag.EncDec (pCoder, this, op);
		useIdleFlag.EncDec (pCoder, this, op);
		durationFlag.EncDec (pCoder, this, op);
		timeStampResolution.EncDec (pCoder, this, op);
		OCRResolution.EncDec (pCoder, this, op);
		timeStampLength.EncDec (pCoder, this, op);	// must be  64
		OCRLength.EncDec (pCoder, this, op);		// must be  64
		AU_Length.EncDec (pCoder, this, op);		// must be  32
		instantBitrateLength.EncDec (pCoder, this, op);
		degradationPriorityLength.EncDec (pCoder, this, op);
		AU_seqNumLength.EncDec (pCoder, this, op); // must be  16
		packetSeqNumLength.EncDec (pCoder, this, op); // must be  16
		reserved.EncDec (pCoder, this, op);
	}
	else if (predefined == 1)
	{	timeStampLength = 32;
		timeStampResolution = 1000;
	}
	if (durationFlag) {
		timeScale.EncDec (pCoder, this, op);
		accessUnitDuration.EncDec (pCoder, this, op);
		compositionUnitDuration.EncDec (pCoder, this, op);
	}
	if (!useTimeStampsFlag) {
		startDecodingTimeStamp.EncDecVarSize (pCoder, this, op, timeStampLength);
		startCompositionTimeStamp.EncDecVarSize (pCoder, this, op, timeStampLength);
	}
}

void SL_PacketHeader :: EncDec (ODCoder *pCoder, OPTYPE op, SLConfigDescriptor *pConfig)
{
	if (op == DECODE)
	{	TRACEF ("SL_PacketHeader {", NULL);
		TRACENEWLINE;
		TRACEINDENT (1);
	}

	if (pConfig -> useAccessUnitStartFlag)
		accessUnitStartFlag.EncDec (pCoder, this, op);
	if (pConfig -> useAccessUnitEndFlag)
		accessUnitEndFlag.EncDec (pCoder, this, op);
	if (pConfig -> OCRLength>0)
		OCRflag.EncDec (pCoder, this, op);
	if (pConfig -> useIdleFlag)
		idleFlag.EncDec (pCoder, this, op);
	if (pConfig -> usePaddingFlag)
		paddingFlag.EncDec (pCoder, this, op);
	if (paddingFlag)
		paddingBits.EncDec (pCoder, this, op);

	if (!idleFlag && (!paddingFlag || paddingBits!=0)) {
		if (pConfig -> packetSeqNumLength>0)
			packetSequenceNumber.EncDecVarSize (pCoder, this, op, pConfig -> packetSeqNumLength);
		if (pConfig -> degradationPriorityLength>0)
		{	DegPrioflag.EncDec (pCoder, this, op);
			if (DegPrioflag)
				degradationPriority.EncDecVarSize (pCoder, this, op, pConfig -> degradationPriorityLength);
		}
		if (OCRflag)
		{	m_nOCROffset = pCoder -> GetOffset ();
			objectClockReference.EncDecVarSize (pCoder, this, op, pConfig -> OCRLength);
		}

		if (accessUnitStartFlag) {
			if (pConfig -> useRandomAccessPointFlag)
				randomAccessPointFlag.EncDec (pCoder, this, op);
			AU_sequenceNumber.EncDecVarSize (pCoder, this, op, pConfig -> AU_seqNumLength);
			if (pConfig -> useTimeStampsFlag) {
				decodingTimeStampFlag.EncDec (pCoder, this, op);
				compositionTimeStampFlag.EncDec (pCoder, this, op);
			}
			if (pConfig -> instantBitrateLength>0)
				instantBitrateFlag.EncDec (pCoder, this, op);
			if (decodingTimeStampFlag)
				decodingTimeStamp.EncDecVarSize (pCoder, this, op, pConfig -> timeStampLength);
			if (compositionTimeStampFlag)
				compositionTimeStamp.EncDecVarSize (pCoder, this, op, pConfig -> timeStampLength);
			if (pConfig -> AU_Length > 0)
				accessUnitLength.EncDecVarSize (pCoder, this, op, pConfig -> AU_Length);
			if (instantBitrateFlag)
				instantBitrate.EncDecVarSize (pCoder, this, op, pConfig -> instantBitrateLength);
		}
	}

	if (op == DECODE)
	{	TRACEINDENT (-1);
		TRACEF ("}", NULL);
		TRACENEWLINE;
	}
}

void BIFSConfig :: DescEncDec (ODCoder *pCoder, OPTYPE op)
{
	nodeIDbits.EncDec (pCoder, this, op);
	routeIDbits.EncDec (pCoder, this, op);
	isCommandStream.EncDec (pCoder, this, op);
	if(isCommandStream) {
		pixelMetric.EncDec (pCoder, this, op);
		hasSize.EncDec (pCoder, this, op);
		if(hasSize) {
			pixelWidth.EncDec (pCoder, this, op);
			pixelHeight.EncDec (pCoder, this, op);
		}
	}
	else {
		randomAccess.EncDec (pCoder, this, op);
		animMask.EncDec (pCoder, this, op);
	}
}

void BIFSv2Config :: DescEncDec (ODCoder *pCoder, OPTYPE op)
{
	use3DMeshCoding.EncDec (pCoder, this, op);
	usePredictiveMFField.EncDec (pCoder, this, op);
	nodeIDbits.EncDec (pCoder, this, op);
	routeIDbits.EncDec (pCoder, this, op);
	protoIDbits.EncDec (pCoder, this, op);
	isCommandStream.EncDec (pCoder, this, op);
	if(isCommandStream) {
		pixelMetric.EncDec (pCoder, this, op);
		hasSize.EncDec (pCoder, this, op);
		if(hasSize) {
			pixelWidth.EncDec (pCoder, this, op);
			pixelHeight.EncDec (pCoder, this, op);
		}
	}
	else {
		randomAccess.EncDec (pCoder, this, op);
		animMask.EncDec (pCoder, this, op);
	}
}

// AudioSpecificConfig added by giorgio.zoia@epfl.ch 2000-01-20
// GeneralAudioConfig added by gew@iis.fhg.de 990512

void AudioSpecificConfig :: DescEncDec (ODCoder *pCoder, OPTYPE op)
{

	audioObjectType.EncDec (pCoder, this, op);
	samplingFrequencyIndex.EncDec (pCoder, this, op);
	if (samplingFrequencyIndex == 0x0F) {
		samplingFrequency.EncDec (pCoder, this, op);
	}
	channelConfiguration.EncDec (pCoder, this, op);

	switch(audioObjectType) {
	case 0x01:
	case 0x02:
	case 0x03:
	case 0x04:
	case 0x06:
	case 0x07:
		GASpecificConfig.EncDec (pCoder, this, op);
		break;
	case 0x08:
		CELPSpecificConfig.EncDec (pCoder, this, op);
		break;
	case 0x0D:
	case 0x0E:
	case 0x0F:
		SASpecificConfig.EncDec (pCoder, this, op);
		break;
	default:
		break;
	} 
}


void GeneralAudioConfig :: DescEncDec (ODCoder *pCoder, OPTYPE op)
{

	// GA_SpecificConfig()

	frameLengthFlag.EncDec (pCoder, this, op);
	dependsOnCoreCoder.EncDec (pCoder, this, op);
	if (dependsOnCoreCoder) {
		coreCoderDelay.EncDec (pCoder, this, op);
	}
	extensionFlag.EncDec (pCoder, this, op);
	if (extensionFlag) {
		// tbd for version 2
	}
}
