/***********************************************************************
This software module was originally developed by Zvi Lifshitz
(Triton R&D Ltd.) in the course of development of the MPEG-4 Systems
(ISO/IEC 14496-1) standard. This software module is an implementation of
a part of one or more MPEG-4 Systems (ISO/IEC 14496-1) tools as
specified by the MPEG-4 Systems (ISO/IEC 14496-1) standard. ISO/IEC
gives users of the MPEG-4 Systems (ISO/IEC 14496-1) free license to
this software module or modifications thereof for use in hardware or
software products claiming conformance to the MPEG-4 Systems (ISO/IEC
14496-1). Those intending to use this software module in hardware or
software products are advised that its use may infringe existing patents.
The original developer of this software module and his/her company, the
subsequent editors and their companies, and ISO/IEC have no liability
for use of this software module or modifications thereof in an
implementation. Copyright is not released for non MPEG-4 Systems
(ISO/IEC 14496-1) conforming products. Triton R&D Ltd. retains full right
to use the code for his/her own purpose, assign or donate the code to a
third party and to inhibit third parties from using the code for non
MPEG-4 Systems (ISO/IEC 14496-1) conforming products. This copyright
notice must be included in all copies or derivative works.
Copyright  1998.
************************************************************************

For more information or to receive an up to date version of this module,
you may call Zvi Lifshitz at zvil@csi.com.
----------------------------------------------------------------------*/

#if !defined(_ODImp_h)	// Sentry, use file only if it's not already included.
#define _ODImp_h

/*-------------------------------------------------------------------------
Project Im1Player/ Mux
INTERNATIONAL ORGANIZATION FOR STANDARDIZATION
ISO/IEC JTC1/SC29/WG11

SUBSYSTEM:  OD Decoder/ Encoder
FILE:       ODImp.h
AUTHOR:     Zvi Lifshitz (Triton R&D Ltd.)
OVERVIEW:   Declarations of classes derived from standard OD descriptors.
			These extension are used to implement OD in the IM1 player.
-------------------------------------------------------------------------*/

#include "odp3.h"
#include "odtags.h"
#include "Caps.h"
#include "Im1defs.h"
#include "Decoder.h"

class ALManager;
class InlineImp;
class StreamConsumer;
class DataChannel;
class IPMPManager;
class MediaStreamImp;
class Executive;
class Decoder;
class Filter;
class ES_DescriptorImp;
class ClockReference;
class MediaControlImp;

class ObjectDescriptorUpdateImp : public ObjectDescriptorUpdate
{
private:
	virtual void Commit (InlineImp *pSource);
};

class ObjectDescriptorRemoveImp : public ObjectDescriptorRemove
{
private:
	virtual void Commit (InlineImp *pSource);
};

class ES_DescriptorUpdateImp : public ES_DescriptorUpdate
{
private:
	virtual void Commit (InlineImp *pSource);
};

class ES_DescriptorRemoveImp : public ES_DescriptorRemove
{
private:
	virtual void Commit (InlineImp *pSource);
};

class IPMP_DescriptorUpdateImp : public IPMP_DescriptorUpdate
{
private:
	virtual void Commit (InlineImp *pSource);
};

class IPMP_DescriptorRemoveImp : public IPMP_DescriptorRemove
{
private:
	virtual void Commit (InlineImp *pSource);
};

class IPMP_DescriptorImp : public IPMP_Descriptor
{
public:
	int Notify (IPMPManager *pIPMP);
	~IPMP_DescriptorImp ()
	{
	}
};

class ObjectDescriptorExecuteImp : public ObjectDescriptorExecute
{
private:
	virtual void Commit (InlineImp *pSource);
};

class ObjectDescriptorImp : public InitialObjectDescriptor
{
public: static int GetCode () {return CODE_ObjectDescriptor;}

public:
	bool FindSegment (
		double dTS,
		double &rdSegmentStart,
		double &rdSegmentEnd,
		LPCSTR &rszSegment
	);
	ObjectDescriptorImp () :
		m_pAL (NULL), m_pInline (NULL), m_pDecoder (NULL), m_pChannel (NULL), m_nUse (0),
		m_nInstance (0), m_nOpen (0), m_pRemoteAL (NULL), m_pRemoteOD (NULL),
		m_pControl (NULL), m_bPreRolled (false)
	{}
	~ObjectDescriptorImp ();
	ALManager *GetAL () const {return m_pAL;}
	void SetAL (ALManager *pAL) {m_pAL = pAL;}
	void SetRemoteAL (ALManager *pAL) {m_pRemoteAL = pAL;}
	void SetInline (InlineImp *pInline) {m_pInline = pInline;}
	InlineImp *GetInline () {return m_pInline;}
	void DecInitialOD (ODCoder *pODCoder);
	MediaStreamImp *SetStream (StreamConsumer *pNode);
	void Start ();
	void Stop ();
	void Detach ();
	Decoder *GetDecoder () {return m_pDecoder;}
	void SetDecoder (Decoder *pDecoder) {m_pDecoder = pDecoder;}
	void SetChannel (DataChannel *pChannel) {m_pChannel = pChannel;}
	void SetClocks ();
	void DataSend (LPBYTE pData, int nLength);
	int SetUseCount (int n) {return m_nUse += n;}
	void OnConnect (const BYTE *pInitOD, int nInitODLen);
	void AttachNode (StreamConsumer *pNode);
	bool UpdateESD (ES_DescriptorImp *pESD);
	void RemoveESD (int nESid);
	ES_DescriptorImp *FindESD (int nESid);
	bool IsPlaying () const {return m_nOpen > 0;}
	void Play (double dSpeed);
	void PreRoll ();
	void Pause ();
	void Mute ();
	void Restart ();
	MediaControlImp *SetMediaControl (MediaControlImp *pControl)
	{
		MediaControlImp *p = m_pControl;
		m_pControl = pControl;
		return p;
	}
	
private:
	ALManager *m_pAL, *m_pRemoteAL;
	InlineImp *m_pInline;
	Decoder *m_pDecoder;
	DataChannel *m_pChannel;
	int m_nUse, m_nInstance, m_nOpen;
	ObjectDescriptorImp *m_pRemoteOD;
	MediaControlImp *m_pControl;
	bool m_bPreRolled;
};

class ES_DescriptorImp : public ES_Descriptor
{
public:
	ES_DescriptorImp ();
	MediaStreamImp *SetDecoder (
		ObjectDescriptorImp *pOD,
		StreamConsumer *pNode
	);
	void Start (bool bPreRolled);
	void Stop ();
	void Detach ();
	void Play (double dSpeed);
	void Pause ();
	void Mute ();
	void Restart ();
	Decoder *GetDecoder () const {return m_pDecoder;}
	ALManager *GetAL () const {return m_pAL;}
	MediaStreamImp *AddESD (ObjectDescriptorImp *pOD, StreamConsumer *pNode);
	bool IsMyStream (MediaStreamImp *pStream) const
	{
		return pStream == m_pInputStream || pStream == m_pOutputStream;
	}
	void SetClock (ALManager *pAL);
	ClockReference *GetClock () const {return m_pClock;}

private:
	Filter *LoadFilter (
		int nStreamType,
		int objectTypeIndication
	);
	DataChannel *OpenStream (
		ObjectDescriptorImp *pOD,
		int nBufferSize,
		int nExpectedAULength,
		bool bBackChannel = false,
		float fThreshold = 0
	);
	bool InvokeIPMP (ObjectDescriptorImp *pOD);
	bool SetIPMP (
		ES_DescriptorImp *pES,
		ObjectDescriptorImp *pOD
	);
	bool HandleIPMPDescriptors (
		IPMP_DescriptorPointer * const * descPointers,
		int nDescPointersLength,
		InlineImp *pInline
	);
	void AttachIPMP (IPMPManager *pIPMP);
	void RemoveIPMPSystems (
		IPMPManager *pIPMP,
		IPMP_DescriptorPointer * const * descPointers,
		int nDescPointersLength,
		InlineImp *pInline
	);
	Decoder *CreateOCIDecoder (int version);
	
private:
	ALManager *m_pAL;
	Executive *m_pExecutive;
	ObjectDescriptorImp *m_pOD;
	MediaStreamImp *m_pOutputStream, *m_pInputStream;
	Decoder *m_pDecoder;
	DataChannel *m_pChannel;
	bool m_bAttached;
	ZArray<IPMPManager*> m_ipmps;
	ClockReference *m_pClock;
	NON_FATAL_ERROR m_error;
	int m_nStart, m_nDecoderStart, m_nInstantiated;
	DecoderParams m_decoderParams;
};			

#endif                                      // _ODImp_h sentry.
