/***********************************************************************
This software module was originally developed by Zvi Lifshitz
(Triton R&D Ltd.) in the course of development of the MPEG-4 Systems
(ISO/IEC 14496-1) standard. This software module is an implementation of
a part of one or more MPEG-4 Systems (ISO/IEC 14496-1) tools as
specified by the MPEG-4 Systems (ISO/IEC 14496-1) standard. ISO/IEC
gives users of the MPEG-4 Systems (ISO/IEC 14496-1) free license to
this software module or modifications thereof for use in hardware or
software products claiming conformance to the MPEG-4 Systems (ISO/IEC
14496-1). Those intending to use this software module in hardware or
software products are advised that its use may infringe existing patents.
The original developer of this software module and his/her company, the
subsequent editors and their companies, and ISO/IEC have no liability
for use of this software module or modifications thereof in an
implementation. Copyright is not released for non MPEG-4 Systems
(ISO/IEC 14496-1) conforming products. Triton R&D Ltd. retains full right
to use the code for his/her own purpose, assign or donate the code to a
third party and to inhibit third parties from using the code for non
MPEG-4 Systems (ISO/IEC 14496-1) conforming products. This copyright
notice must be included in all copies or derivative works.
Copyright  1998.
************************************************************************

For more information or to receive an up to date version of this module,
you may call Zvi Lifshitz at zvil@csi.com.
----------------------------------------------------------------------*/

#if !defined(_odp_h)	// Sentry, use file only if it's not already included.
#define _odp_h

/*-------------------------------------------------------------------------
Project Im1Player/ Mux
INTERNATIONAL ORGANIZATION FOR STANDARDIZATION
ISO/IEC JTC1/SC29/WG11

SUBSYSTEM:  OD Decoder/ Encoder
FILE:       odp.h
AUTHOR:     Zvi Lifshitz (Triton R&D Ltd.)
OVERVIEW:   Declarations of descriptors of OD stream
-------------------------------------------------------------------------*/

#include "sdl.h"
#include "sdldef.h"

class InlineImp;
class ClockReference;

class BaseCommand : public BaseDescriptor
{
	OD_DECLARE_ABSTRACT_DESCRIPTOR

public:
	virtual void Commit (InlineImp *) {}
};

class ObjectDescriptorUpdate : public BaseCommand
{
	OD_DECLARE_DESCRIPTOR
public:
	SDLClassArray<class ObjectDescriptor, 1, 255> OD;
};

class ObjectDescriptorRemove : public BaseCommand
{
	OD_DECLARE_DESCRIPTOR
public:
	SDLArray<10> objectDescriptorId;
};

class ES_DescriptorUpdate : public BaseCommand
{
	OD_DECLARE_DESCRIPTOR
public:
	SDLInt<10> objectDescriptorId;
	SDLClassArray<class ES_Descriptor, 1, 255> esDescr;
};

class ES_DescriptorRemove : public BaseCommand
{
	OD_DECLARE_DESCRIPTOR
public:
	SDLInt<10> objectDescriptorId;
	SDLAlignedInt<0> align;
	SDLArray<16> ES_ID;
};

class IPMP_DescriptorUpdate : public BaseCommand
{
	OD_DECLARE_DESCRIPTOR
public:
	SDLClassArray<class IPMP_Descriptor, 1, 255> d;
};

class IPMP_DescriptorRemove : public BaseCommand
{
	OD_DECLARE_DESCRIPTOR
public:
	SDLArray<8> IPMP_DescriptorID;
};

class ObjectDescriptor : public BaseDescriptor
{
	OD_DECLARE_DESCRIPTOR
public:
	SDLInt<10> ObjectDescriptorID;
	SDLInt<1> URL_Flag;
	SDLInt<5> reserved;
	SDLString URLstring;
	SDLClassArray<class ES_Descriptor, 1, 255> esDescr;
	SDLClassArray<class OCI_Descriptor, 0, 255> ociDescr;
	SDLClassArray<class IPMP_DescriptorPointer, 0, 255> ipmpDescrPtr;
	SDLClassArray<class ExtensionDescriptor, 0, 255> extDescr;

	/*-------------------------------------------------------------
	Pseudo fields for BifsEnc/Mux
	-------------------------------------------------------------*/
	SDLString muxScript;

public:
	ObjectDescriptor () : reserved (0x1f) {}

protected:
	virtual void DescEncDec (ODCoder *pODCoder, OPTYPE op);
};

class InitialObjectDescriptor : public ObjectDescriptor
{
	OD_DECLARE_DESCRIPTOR
public:
	/*-------------------------------------------------------------
	The fields that were commented out are derived from ObjectDescriptor
	-------------------------------------------------------------*/
	SDLInt<1> includeInlineProfilesFlag;
	SDLInt<8> ODProfileLevelIndication;
	SDLInt<8> sceneProfileLevelIndication;
	SDLInt<8> audioProfileLevelIndication;
	SDLInt<8> visualProfileLevelIndication;
	SDLInt<8> graphicsProfileLevelIndication;

private:
	virtual void DescEncDec (ODCoder *pODCoder, OPTYPE op);
};

class ES_Descriptor : public BaseDescriptor
{
	OD_DECLARE_DESCRIPTOR
public:
	SDLInt<16> ES_ID;
	SDLInt<1> streamDependenceFlag;
	SDLInt<1> URL_Flag;
	SDLInt<1> OCRstreamFlag;
	SDLInt<5> streamPriority;
	SDLInt<16> dependsOn_ES_ID;
	SDLString URLstring;
	SDLInt<16> OCR_ES_Id;
	SDLClass<class DecoderConfigDescriptor, 1> decConfigDescr;
	SDLClass<class SLConfigDescriptor, 1> slConfigDescr;
	SDLClass<class IPI_DescrPointer, 0> ipiPtr;
	SDLClassArray<class IP_IdentificationDataSet, 0, 255> ipIDS;
	SDLClassArray<class IPMP_DescriptorPointer, 0, 255> ipmpDescrPtr;
	SDLClassArray<class LanguageDescriptor, 0, 255> langDescr;
	SDLClass<class QoS_Descriptor, 0> qosDescr;
	SDLClass<class RegistrationDescriptor, 0> regDescr;
	SDLClassArray<class ExtensionDescriptor, 0, 255> extDescr;

	/*-------------------------------------------------------------
	Pseudo field for Mux
	-------------------------------------------------------------*/
	SDLClass<class MuxInfo, 0> muxInfo;
	SDLClass<class MuxCodeTable, 0> muxCodeTable;

public:

private:
	virtual void DescEncDec (ODCoder *pODCoder, OPTYPE op);
};

class DecoderConfigDescriptor : public BaseDescriptor
{
	OD_DECLARE_DESCRIPTOR
public:
	SDLInt<8> objectTypeIndication;
	SDLInt<6> streamType;
	SDLInt<1> upStream;
	SDLInt<1> reserved;	
	SDLInt<24> bufferSizeDB;
	SDLInt<32> maxBitrate;
	SDLInt<32> avgBitrate;
	SDLClass<class DecoderSpecificInfo, 0> decSpecificInfo;
	SDLClassArray<class profileLevelIndicationIndexDescriptor, 0, 255> profileLevelIndicationIndexDescr;

public:
	DecoderConfigDescriptor () : reserved (1) {}
};

class profileLevelIndicationIndexDescriptor : public BaseDescriptor
{
	OD_DECLARE_DESCRIPTOR
public:
	SDLInt<8> profileLevelIndicationIndex;
};

class DecoderSpecificInfo : public BaseDescriptor
{
	OD_DECLARE_ABSTRACT_DESCRIPTOR
public:
	SDLArray<8> info;
};

class DecoderSpecificInfoString : public DecoderSpecificInfo
{
	OD_DECLARE_DESCRIPTOR
public:
};

class IP_IdentificationDataSet : public BaseDescriptor
{
	OD_DECLARE_ABSTRACT_DESCRIPTOR
public:
};

class ContentIdentificationDescriptor : public IP_IdentificationDataSet
{
	OD_DECLARE_DESCRIPTOR
public:
	SDLInt<2> compatibility;
	SDLInt<1> contentTypeFlag;
	SDLInt<1> contentIdentifierFlag;
	SDLInt<1> protectedContent;
	SDLInt<3> reserved;
	SDLInt<8> contentType;
	SDLInt<8> contentIdentifierType;
	SDLArray<8> contentIdentifier;

public:
	ContentIdentificationDescriptor () :
		compatibility (0), reserved (0x3)
	{}

private:
	virtual void DescEncDec (ODCoder *pODCoder, OPTYPE op);
};

class SupplementaryContentIdentificationDescriptor : public IP_IdentificationDataSet
{
	OD_DECLARE_DESCRIPTOR
public:
	SDLInt<24> languageCode;
	SDLString supplContentIdentifierTitle;
	SDLString supplContentIdentifierValue;
};

class IPI_DescrPointer : public BaseDescriptor
{
	OD_DECLARE_DESCRIPTOR
public:
	SDLInt<16> IPI_ES_Id;
};

class IPMP_DescriptorPointer : public BaseDescriptor
{
	OD_DECLARE_DESCRIPTOR
public:
	SDLInt<8> IPMP_DescriptorID;
};

class IPMP_Descriptor : public BaseDescriptor
{
	OD_DECLARE_DESCRIPTOR
public:
	SDLInt<8> IPMP_DescriptorID;
	SDLInt<16> IPMPS_Type;
	SDLArray<8> URLString;
	SDLArray<8> IPMP_data;

private:
	virtual void DescEncDec (ODCoder *pODCoder, OPTYPE op);
};

class QoS_Descriptor : public BaseDescriptor
{
	OD_DECLARE_DESCRIPTOR
public:
	SDLInt<8> predefined;
	SDLClassArray<class QoS_Qualifier> qualifiers;

private:
	virtual void DescEncDec (ODCoder *pODCoder, OPTYPE op);
};

class QoS_Qualifier : public BaseDescriptor
{
	OD_DECLARE_ABSTRACT_DESCRIPTOR
};

class QoS_Qualifier_MAX_DELAY : public QoS_Qualifier
{
	OD_DECLARE_DESCRIPTOR

public:
	SDLInt<32> MAX_DELAY;
};

class QoS_Qualifier_PREF_MAX_DELAY : public QoS_Qualifier
{
	OD_DECLARE_DESCRIPTOR

public:
	SDLInt<32> PREF_MAX_DELAY;
};

class QoS_Qualifier_LOSS_PROB : public QoS_Qualifier
{
	OD_DECLARE_DESCRIPTOR

public:
	SDLFloat LOSS_PROB;
};

class QoS_Qualifier_MAX_GAP_LOSS : public QoS_Qualifier
{
	OD_DECLARE_DESCRIPTOR

public:
	SDLInt<32> MAX_GAP_LOSS;
};

class QoS_Qualifier_MAX_AU_SIZE : public QoS_Qualifier
{
	OD_DECLARE_DESCRIPTOR

public:
	SDLInt<32> MAX_AU_SIZE;
};

class QoS_Qualifier_AVG_AU_SIZE : public QoS_Qualifier
{
	OD_DECLARE_DESCRIPTOR

public:
	SDLInt<32> AVG_AU_SIZE;
};

class QoS_Qualifier_MAX_AU_RATE : public QoS_Qualifier
{
	OD_DECLARE_DESCRIPTOR

public:
	SDLInt<32> MAX_AU_RATE;
};

class ExtensionDescriptor : public BaseDescriptor 
{
	OD_DECLARE_ABSTRACT_DESCRIPTOR
public:
};

class RegistrationDescriptor : public BaseDescriptor 
{
	OD_DECLARE_DESCRIPTOR
public:
	SDLInt<32> formatIdentifier;
	SDLArray<8> additionalIdentificationInfo;
};

class OCI_Descriptor : public BaseDescriptor 
{
	OD_DECLARE_ABSTRACT_DESCRIPTOR

public:
	virtual void Commit (int, bool, long, long, ClockReference *, DWORD) {}
};

class ContentClassificationDescriptor : public OCI_Descriptor 
{
	OD_DECLARE_DESCRIPTOR
public:
	SDLInt<32> classificationEntity; 
	SDLInt<16> classificationTable;
	SDLArray<8> contentClassificationData;
};

class KeyWordDescriptor : public OCI_Descriptor 
{
	OD_DECLARE_DESCRIPTOR
public:
	SDLInt<24> languageCode;  
	SDLInt<1> isUTF8_string;
	SDLAlignedInt<8> keyWordCount;
	SDLStringArray keyWord;

private:
	virtual void DescEncDec (ODCoder *pODCoder, OPTYPE op);
};  

class RatingDescriptor : public OCI_Descriptor 
{
	OD_DECLARE_DESCRIPTOR
public:
	SDLInt<32> ratingEntity; 
	SDLInt<16> ratingCriteria;
	SDLArray<8> ratingInfo;
};

class LanguageDescriptor : public OCI_Descriptor 
{
	OD_DECLARE_DESCRIPTOR
public:
	SDLInt<24> languageCode;
};

class ShortTextualDescriptor : public OCI_Descriptor 
{
	OD_DECLARE_DESCRIPTOR
public:
	SDLInt<24> languageCode;
	SDLInt<1> isUTF8_string;
	SDLAlignedInt<0> align;
	SDLString eventName;
	SDLString eventText; 

private:
	virtual void DescEncDec (ODCoder *pODCoder, OPTYPE op);
};

class ExpandedTextualDescriptor : public OCI_Descriptor 
{
	OD_DECLARE_DESCRIPTOR
public:
	SDLInt<24> languageCode;
	SDLInt<1> isUTF8_string;
	SDLAlignedInt<8> itemCount;
	SDLStringArray itemDescription;
	SDLStringArray itemText;
	SDLString nonItemText;

private:
	virtual void DescEncDec (ODCoder *pODCoder, OPTYPE op);
};  

class ContentCreatorNameDescriptor : public OCI_Descriptor 
{
	OD_DECLARE_DESCRIPTOR
public:
	SDLInt<8> contentCreatorCount;
	SDLArray<24> languageCode;
	SDLArray<1> isUTF8_string;
	SDLAlignedInt<0> align;
	SDLStringArray contentCreatorName;

private:
	virtual void DescEncDec (ODCoder *pODCoder, OPTYPE op);
};  

class ContentCreationDateDescriptor : public OCI_Descriptor 
{
	OD_DECLARE_DESCRIPTOR
public:
	SDLInt<40> contentCreationDate;
};  

class OCICreatorNameDescriptor : public OCI_Descriptor 
{
	OD_DECLARE_DESCRIPTOR
public:
	SDLInt<8> OCICreatorCount;
	SDLArray<24> languageCode;
	SDLArray<1> isUTF8_string;
	SDLAlignedInt<0> align;
	SDLStringArray OCICreatorName;

private:
	virtual void DescEncDec (ODCoder *pODCoder, OPTYPE op);
};  

class OCICreationDateDescriptor : public OCI_Descriptor 
{
	OD_DECLARE_DESCRIPTOR
public:
	SDLInt<40> OCICreationDate;
};  

// NHK 990702
class SmpteCameraPositionDescriptor : public OCI_Descriptor
{
	OD_DECLARE_DESCRIPTOR
public:
	SDLInt<8> cameraID;
	SDLInt<8> parameterCount;
	SDLArray<8> parameterID;
	SDLArray<32> parameter;

private:
	virtual void DescEncDec (ODCoder *pODCoder, OPTYPE op);
};

class MediaTimeDescriptor : public OCI_Descriptor
{
	OD_DECLARE_DESCRIPTOR
public:
	SDLDouble mediaTimeStamp;
};

class SLConfigDescriptor : public BaseDescriptor
{
	OD_DECLARE_DESCRIPTOR
public:
	SDLInt<8> predefined;
	SDLInt<1> useAccessUnitStartFlag;
	SDLInt<1> useAccessUnitEndFlag;
	SDLInt<1> useRandomAccessPointFlag;
	SDLInt<1> hasRandomAccessUnitsOnlyFlag;
	SDLInt<1> usePaddingFlag;
	SDLInt<1> useTimeStampsFlag;
	SDLInt<1> useIdleFlag;
	SDLInt<1> durationFlag;
	SDLInt<32> timeStampResolution;
	SDLInt<32> OCRResolution;
	SDLInt<8> timeStampLength;	// must be  64
	SDLInt<8> OCRLength;		// must be  64
	SDLInt<8> AU_Length;		// must be  32
	SDLInt<8> instantBitrateLength;
	SDLInt<4> degradationPriorityLength;
	SDLInt<5> AU_seqNumLength; // must be  16
	SDLInt<5> packetSeqNumLength; // must be  16
	SDLInt<2> reserved;
	SDLInt<32> timeScale;
	SDLInt<16> accessUnitDuration;
	SDLInt<16> compositionUnitDuration;
	SDLInt<0> startDecodingTimeStamp;
	SDLInt<0> startCompositionTimeStamp;

public:
	SLConfigDescriptor () : reserved (0x3) {}

private:
	virtual void DescEncDec (ODCoder *pODCoder, OPTYPE op);
};

class BIFSConfig : public DecoderSpecificInfo
{
	OD_DECLARE_DESCRIPTOR
public:
 	SDLInt<1> use3DMeshCoding;
 	SDLInt<1> usePredictiveMFField;
	SDLInt<5> nodeIDbits;
	SDLInt<5> routeIDbits;
	SDLInt<5> protoIDbits;
	SDLInt<1> isCommandStream;
	SDLInt<1> pixelMetric;
	SDLInt<1> hasSize;
	SDLInt<16> pixelWidth;
	SDLInt<16> pixelHeight;
	SDLInt<1> randomAccess;
	SDLArray<8> animMask;

private:
	virtual void DescEncDec (ODCoder *pODCoder, OPTYPE op);
};

class BIFSv2Config : public BIFSConfig
{
	OD_DECLARE_DESCRIPTOR

private:
	virtual void DescEncDec (ODCoder *pODCoder, OPTYPE op);
};

class OCIDecoderConfiguration : public DecoderSpecificInfo
{
	OD_DECLARE_DESCRIPTOR

public:
	SDLInt<8> versionLabel;
	OCIDecoderConfiguration () : versionLabel (1) {}
};

class JPEG_DecoderConfig : public DecoderSpecificInfo
{
	OD_DECLARE_DESCRIPTOR
public:
	SDLInt<16> headerLength;
	SDLInt<16> Xdensity;
	SDLInt<16> Ydensity;
	SDLInt<8> numComponents;
};

// AudioSpecificConfig added by giorgio.zoia@epfl.ch 2000-01-20
// GeneralAudioConfig added by gew@iis.fhg.de 990512

class AudioSpecificConfig : public DecoderSpecificInfo
{
	OD_DECLARE_DESCRIPTOR
public:
	SDLInt<5> audioObjectType;
	SDLInt<4> samplingFrequencyIndex;
	SDLInt<24> samplingFrequency;
	SDLInt<4> channelConfiguration;

	SDLClass<class GeneralAudioConfig, 0> GASpecificConfig;
	SDLClass<class StructuredAudioConfig, 0> SASpecificConfig;
	SDLClass<class CELP_DecoderConfig, 0> CELPSpecificConfig;

private:
	virtual void DescEncDec (ODCoder *pODCoder, OPTYPE op);
};


class GeneralAudioConfig : public DecoderSpecificInfo
{
	OD_DECLARE_DESCRIPTOR
public:

	SDLInt<1> frameLengthFlag;
	SDLInt<1> dependsOnCoreCoder;
	SDLInt<14> coreCoderDelay;
	SDLInt<1> extensionFlag;

private:
	virtual void DescEncDec (ODCoder *pODCoder, OPTYPE op);
};

// SA and CELP Config added by giorgio.zoia@epfl.ch 2000-01-20
class StructuredAudioConfig : public DecoderSpecificInfo
{
	OD_DECLARE_DESCRIPTOR
public:
};

class CELP_DecoderConfig : public DecoderSpecificInfo
{
	OD_DECLARE_DESCRIPTOR
public:
};

class SL_PacketHeader : public BaseDescriptor
{
#ifdef TRACE_ENCODING
	OD_DECLARE_DESCRIPTOR
#endif

public:
	SDLInt<1> accessUnitStartFlag;
	SDLInt<1> accessUnitEndFlag;
	SDLInt<1> OCRflag;
	SDLInt<1> idleFlag;
	SDLInt<1> paddingFlag;
	SDLInt<3> paddingBits;
	SDLInt<0> packetSequenceNumber;
	SDLInt<1> DegPrioflag;
	SDLInt<0> degradationPriority;
	SDLInt<0> objectClockReference;
	SDLInt<1> randomAccessPointFlag;
	SDLInt<0> AU_sequenceNumber;
	SDLInt<1> decodingTimeStampFlag;
	SDLInt<1> compositionTimeStampFlag;
	SDLInt<1> instantBitrateFlag;
	SDLInt<0> decodingTimeStamp;
	SDLInt<0> compositionTimeStamp;
	SDLInt<0> accessUnitLength;
	SDLInt<0> instantBitrate;

public:
	SL_PacketHeader (bool bStart = true) :
		accessUnitStartFlag (bStart),
		randomAccessPointFlag (true),
		accessUnitEndFlag (true)
	{}
	void EncDec (ODCoder *pODCoder, OPTYPE op, SLConfigDescriptor *pConfig);
	int GetOCROffset () const {return m_nOCROffset;}

private:
	int m_nOCROffset;
};

/*-------------------------------------------------------------
A pseudo class used by the Mux tool
-------------------------------------------------------------*/
class MuxInfo : public BaseDescriptor
{
	OD_DECLARE_DESCRIPTOR
public:
	SDLString fileName;
	SDLString streamFormat;
	SDLString streamAttr;
	SDLInteger repeatRate;
	SDLInteger sessionLength;
	SDLInteger maxAhead;
	SDLInteger startTime;

	SDLInteger muxCode;		// additions for muxcode implementation
	SDLInteger maxDeltaT;	// permissible time difference for a single muxcode packet
	
	// additions for MP4Enc implementation
	SDLInteger GroupID;
	SDLInt<1> UseDataReference;
public:
	MuxInfo() : muxCode(0,FALSE,-1), maxDeltaT(0), GroupID(1, FALSE, 1), UseDataReference(false)
	{}
};

/*-------------------------------------------------------------
Pseudo classes used by the Mux tool for mux code proecessing
-------------------------------------------------------------*/
class MuxCodeSlot : public BaseDescriptor
{
	OD_DECLARE_DESCRIPTOR
public:
	SDLInt<5> slotCount;
	SDLInt<3> repetitionCount;
	SDLArray<16> flexMuxChannel;  // 16 bit: this reads the ES_ID from file!!
	SDLArray<8> numberOfBytes;
};

class MuxCodeTableElement : public BaseDescriptor
{
	OD_DECLARE_DESCRIPTOR
public:
	SDLInt<8> length;
	SDLInt<4> muxCode;
	SDLInt<4> version;
	SDLInt<8> subStrucCount;
	SDLClassArray<class MuxCodeSlot, 1, 31> slots;
};

class MuxCodeTable : public BaseDescriptor
{
	OD_DECLARE_DESCRIPTOR
public:
	SDLClassArray<class MuxCodeTableElement, 0, 16> elements;
};

class OCI_Event : public BaseDescriptor
{
	OD_DECLARE_DESCRIPTOR
public:
	SDLInt<15> eventID;
	SDLInt<1>  absoluteTimeFlag;
	SDLInt<32> startingTime;
	SDLInt<32> duration;
	SDLClassArray<class OCI_Descriptor, 1, 255> OCI_Descr;
};

/*-------------------------------------------------------------
streamType Values
-------------------------------------------------------------*/
typedef enum
{
	ST_ObjectDescriptorStream = 0x01,
	ST_ClockReferenceStream = 0x02,
	ST_SceneDescriptionStream = 0x03,
	ST_VisualStream = 0x04,
	ST_AudioStream = 0x05,
	ST_MPEG7Stream = 0x06,
	ST_IPMPStream = 0x07,
	ST_ObjectContentInfoStream = 0x08,
	ST_UserInteractionStream = 0x09 // CC 010110
} STREAM_TYPE;

#endif                                      // _odp_h sentry.
