/***********************************************************************
This software module was originally developed by Zvi Lifshitz
(Triton R&D Ltd.) in the course of development of the MPEG-4 Systems
(ISO/IEC 14496-1) standard. This software module is an implementation of
a part of one or more MPEG-4 Systems (ISO/IEC 14496-1) tools as
specified by the MPEG-4 Systems (ISO/IEC 14496-1) standard. ISO/IEC
gives users of the MPEG-4 Systems (ISO/IEC 14496-1) free license to
this software module or modifications thereof for use in hardware or
software products claiming conformance to the MPEG-4 Systems (ISO/IEC
14496-1). Those intending to use this software module in hardware or
software products are advised that its use may infringe existing patents.
The original developer of this software module and his/her company, the
subsequent editors and their companies, and ISO/IEC have no liability
for use of this software module or modifications thereof in an
implementation. Copyright is not released for non MPEG-4 Systems
(ISO/IEC 14496-1) conforming products. Triton R&D Ltd. retains full right
to use the code for his/her own purpose, assign or donate the code to a
third party and to inhibit third parties from using the code for non
MPEG-4 Systems (ISO/IEC 14496-1) conforming products. This copyright
notice must be included in all copies or derivative works.
Copyright  1998.
************************************************************************/

/*------------------------------------------
File: MP4Enc.cpp
Author: Jean Le Feuvre, E-Vue Inc, March 2000

Content: Parsing of script file and writing of the movie
------------------------------------------*/

#include "defs.h"
#include "MP4Enc.h"
#include "Token.h"
#include "odMP4.h"
#include "MuxODEnc.h"

//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////

MP4Enc::MP4Enc(LPCSTR stMP4Name)
{
	m_stMP4Movie = (char *) stMP4Name;
	m_pMovie = NULL;
}

MP4Enc::~MP4Enc()
{

}

/*-----------------------------------------------
Parses the same script file as the Mux one (uses also Mux struct as File info )
-------------------------------------------------*/
void MP4Enc:: Start (FILE *fList, int flags)
{
	MuxODEncoder odEncoder (NULL, fList, "MP4", flags);

	// We separate ODs and IODs to insert the IOD streams in the movie's IOD
	SDLClassArray<ObjectDescriptor, 1, 255> ods;
	SDLClassArray<InitialObjectDescriptor, 1, 255> IODs;

	MP4Err err;
	BOOL bError = FALSE;
	int i, j;
	ZArray<int> esIDs;

	/*-------------------------------------------------------------
	ES_Descriptor is overloaded with a special version used by the Mux
	-------------------------------------------------------------*/		
	OVERLOAD_OD_CLASS (ES_DescriptorMP4);

	try
	{
		/*-------------------------------------------------------------
		Parse the initial object descritpor, if exists, and encode
		it at the beginning of the file.
		-------------------------------------------------------------*/

		if (stricmp (nextToken.GetString (), "InitialObjectDescriptor") == 0)
		{	SDLClass<InitialObjectDescriptor> IOD;
			IOD.Scan ();
			if (IOD != NULL_DESC)
			{
				IODs += IOD;
				IOD = NULL_DESC;
			}
		}

		/*-----------------------------------------------------------------
		Parse the rest of the object descriptors
		-----------------------------------------------------------------*/
		while (nextToken != TID_EOF)
		{
			SDLClass<ObjectDescriptor> od;
			od.Scan ();
			if (od != NULL_DESC)
			{	ods += od;
				od = NULL_DESC;
			}
		}
	}
	catch (...)
	{
		printf ("Script file error. Hit enter to continue\x7\n");
		getchar ();
		return;
	}

	//Check we have one and only one IOD 
	switch (IODs.GetLength()) {
	case 0:
		printf ("Script file error: No Initial Object Descriptor.\n Hit enter to continue\x7\n");
		return;

	case 1:
		break;
	
	default:
		printf ("Script file error: Initial Object Descriptor must be unique.\n Hit enter to continue\x7\n");
		return;
	}

	//Now we are sure there is only one IOD
	InitialObjectDescriptor *IOD = IODs [0];
	err = (MP4Err) NewMovie(IOD); if (err) return;


	/*-----------------------------------------------------------------
	Traverse the esDescrs list. For	each descriptor which points to a file
	that shall be added to the movie, open that file. 
	Other esDescrs are NOT deleted ( if URLFlag == TRUE	we have to tell
	it in the ES track);
	-----------------------------------------------------------------*/

	/*-------------------------------------------------------------
	Open the IOD streams
	--------------------------------------------------------------*/
	for (j = 0; j < IOD-> esDescr.GetLength ();) {
		switch (
			((ES_DescriptorMP4 *) IOD-> esDescr [j]) ->
			OpenFile (fList, flags)
			) {
			//Stream couldn't be opened
		case -1:
			bError = TRUE;
			//Add the track
		default:
			esIDs += IOD-> esDescr [j] -> ES_ID;
			j++;
		}
	}


	/*-------------------------------------------------------------
	Open the other streams
	--------------------------------------------------------------*/
	for (i = 0; i < ods.GetLength ();)
	{	for (j = 0; j < ods [i] -> esDescr.GetLength ();)
			switch (
				((ES_DescriptorMP4 *) ods [i] -> esDescr [j]) ->
					OpenFile (fList, flags)
			) {
			//Stream couldn't be opened
			case -1:
				bError = TRUE;
			//Add the track
			default:
				esIDs += ods [i] -> esDescr [j] -> ES_ID;
				j++;
			}

		if (ods [i] -> esDescr.GetLength () == 0)
			ods.DeleteFrom (i);
		else
			i++;
	}
	
	/*-----------------------------------------------------------------
	Terminate here if had an error while opening the files
	-----------------------------------------------------------------*/
	if (bError)
	{	printf ("Hit Enter to continue\x07\n");
		getchar ();
		return;
	}

	int IODLength = IOD -> esDescr.GetLength ();

	//Now add the streams to the tracks
	//Warning : in order to write the OD Commands to the OD track, we need to write each non-OD stream
	//then create the OD track, add the dependancies and then write the OD track.
	//The API assumes that trackID == ES_ID (used to writes the dependancies on the fly)

	//non-IOD Streams

	for (i = 0; i < ods.GetLength() ; i++) {
		for (j = 0; j < ods[i]-> esDescr.GetLength (); j++) {

			ES_Descriptor *pESDesc = (ES_Descriptor *) ods [i] -> esDescr [j];
			ES_DescriptorMP4 *p = (ES_DescriptorMP4 *) ods [i] -> esDescr [j];

			if (pESDesc->decConfigDescr->streamType != 0x01) {
				if (p->AddTrackToMovie(m_pMovie, FALSE)) {
					printf ("%s: Error writing track %d.\n"	, m_stMP4Movie, pESDesc->ES_ID);
					goto bail;
				}
			}
		}
	}

	//IOD BIFS stream
	for (j = 0; j < IODLength ; j++) {

		ES_Descriptor *pESDesc = (ES_Descriptor *) IOD->esDescr [j];
		ES_DescriptorMP4 *p = (ES_DescriptorMP4 *) IOD-> esDescr [j];

		
		if (pESDesc->decConfigDescr->streamType != 0x01) {
			if (p->AddTrackToMovie(m_pMovie, TRUE)) {
				printf ("%s: Error writing track %d.\n"	, m_stMP4Movie, (int) pESDesc->ES_ID);
				goto bail;
			}
		}

		/*-----------------------------------------------------------------
		Now we have to check if the stream is an OD or BIFS one
		If it is, store the ES_ID of this stream
		-----------------------------------------------------------------*/
		//BIFS
		if (p->GetStreamType() == 0x03) {
			if (m_IODBifsID) {
				printf ("Error: Only one BIFS stream is allowed in the IODescriptor.\n Hit enter to continue\x7\n");
				goto bail;
			}
			m_IODBifsID = p->ES_ID;
		}
	}


	//non-IOD OD-Streams
	for (i = 0; i < ods.GetLength(); i++) {
		for (j = 0; j < ods[i]-> esDescr.GetLength (); j++) {

			ES_Descriptor *pESDesc = (ES_Descriptor *) ods [i] -> esDescr [j];
			ES_DescriptorMP4 *p = (ES_DescriptorMP4 *) ods [i] -> esDescr [j];

			if (pESDesc->decConfigDescr->streamType == 0x01) {
				if (p->AddTrackToMovie(m_pMovie, FALSE)) {
					printf ("%s: Error writing track %d.\n"	, m_stMP4Movie, pESDesc->ES_ID);
					goto bail;
				}			
			}
		}
	}

	//IOD OD stream
	for (j = 0; j < IODLength; j++) {

		ES_Descriptor *pESDesc = (ES_Descriptor *) IOD->esDescr [j];
		ES_DescriptorMP4 *p = (ES_DescriptorMP4 *) IOD-> esDescr [j];

		if (pESDesc->decConfigDescr->streamType == 0x01) {
			if (p->AddTrackToMovie(m_pMovie, TRUE)) {
				printf ("%s: Error writing track %d.\n"	, m_stMP4Movie, pESDesc->ES_ID);
				goto bail;
			}
		}
	}

	//All tracks have been added-> set dependencies and save the movie to a file
	if (m_pMovie != NULL) {
		//Write the movie to a file and free the handle
		err = (MP4Err) MP4WriteMovieToFile( m_pMovie, m_stMP4Movie); if (err) goto bail;
		err = (MP4Err) MP4DisposeMovie(m_pMovie); if (err) goto bail;
	}
	return;

bail:
	printf("Has Error");
	return;
}


BOOL MP4Enc::NewMovie(InitialObjectDescriptor *pIODescr)
{
	MP4Err err = MP4NoErr;

	//No Bifs and No ODstream
	m_IODBifsID = 0;
	m_IODOdID = 0;

	err = MP4NewMovie( &m_pMovie, pIODescr->ObjectDescriptorID, 
		pIODescr->ODProfileLevelIndication,
		pIODescr->sceneProfileLevelIndication,
		pIODescr->audioProfileLevelIndication,
		pIODescr->visualProfileLevelIndication,
		pIODescr->graphicsProfileLevelIndication); if (err) return err;

	return MP4SetMovieTimeScale(m_pMovie, DEF_MOVIE_TIMESCALE); if (err) return err;

}


