/***********************************************************************
This software module was originally developed by Zvi Lifshitz
(Triton R&D Ltd.) in the course of development of the MPEG-4 Systems
(ISO/IEC 14496-1) standard. This software module is an implementation of
a part of one or more MPEG-4 Systems (ISO/IEC 14496-1) tools as
specified by the MPEG-4 Systems (ISO/IEC 14496-1) standard. ISO/IEC
gives users of the MPEG-4 Systems (ISO/IEC 14496-1) free license to
this software module or modifications thereof for use in hardware or
software products claiming conformance to the MPEG-4 Systems (ISO/IEC
14496-1). Those intending to use this software module in hardware or
software products are advised that its use may infringe existing patents.
The original developer of this software module and his/her company, the
subsequent editors and their companies, and ISO/IEC have no liability
for use of this software module or modifications thereof in an
implementation. Copyright is not released for non MPEG-4 Systems
(ISO/IEC 14496-1) conforming products. Triton R&D Ltd. retains full right
to use the code for his/her own purpose, assign or donate the code to a
third party and to inhibit third parties from using the code for non
MPEG-4 Systems (ISO/IEC 14496-1) conforming products. This copyright
notice must be included in all copies or derivative works.
Copyright  1998.
************************************************************************/

/*------------------------------------------
File: MP4OD.cpp
Author: Jean Le Feuvre, E-Vue Inc, March 2000

Content: stream->track writer
------------------------------------------*/


#include "defs.h"
#include "odMP4.h"
#include "MP4Stream.h"

extern BOOL g_bErrors;

/*-------------------------------------------------------------
ES_DescriptorMux constructor
-------------------------------------------------------------*/
ES_DescriptorMP4 :: ES_DescriptorMP4 () :
	m_fStream (NULL),
	m_pStream (NULL),
	m_lTS (-1),
	m_nScale (0),	// Guido 4/8/00
	m_nPayloadLength (0),
	m_bStart (TRUE),
	m_bEnd (FALSE),
	m_bEndOfAU (FALSE),
	m_coder (NULL, NULL, "DecoderSpecificInfos", TRACE_NONE, 0)
{
		m_pSampleEntry = NULL;
}

/*-----------------------------------------------------------------
ES_DescriptorMP4 destructor. Close the stream file
-----------------------------------------------------------------*/
ES_DescriptorMP4 :: ~ES_DescriptorMP4 ()
{
	if (m_pStream)
		delete m_pStream;
	if (m_fStream)
		fclose (m_fStream);

}

/*-----------------------------------------------------------------
Array of factory objects to create proper Stream object
-----------------------------------------------------------------*/
StreamFactoryBase *ES_DescriptorMP4 :: m_pFactories [40];
int ES_DescriptorMP4 :: m_nFactoriesAvailable;

/*-----------------------------------------------------------------
Register a StreamFactory object
-----------------------------------------------------------------*/
void ES_DescriptorMP4 :: RegisterFactory (StreamFactoryBase *pFactory)
{
	assert (m_nFactoriesAvailable < 40);
	m_pFactories [m_nFactoriesAvailable++] = pFactory;
}

/*-----------------------------------------------------------------
Open the stream file, and create a proper Stream object
-----------------------------------------------------------------*/
int ES_DescriptorMP4 :: OpenFile (FILE *fList, int flags)
{
	/*-----------------------------------------------------------------
	Open the stream file
	-----------------------------------------------------------------*/
	if (muxInfo == NULL_DESC)
	{	printf ("No Mux info for stream no %d\n", ES_ID);
		return -1;
	}
	if (muxInfo -> fileName.GetLength () == 0)
		return 0;
	m_fStream = fopen (muxInfo -> fileName, "rb");
	if (!m_fStream)
	{	printf ("Cannot open %s\n", (LPCSTR) muxInfo -> fileName);
		return -1;
	}

	/*-----------------------------------------------------------------
	Instantiate a proper Stream object
	-----------------------------------------------------------------*/
	if (decConfigDescr == NULL_DESC)
	{	printf (
			"%s: decConfigDescr not available\n",
			(LPCSTR) muxInfo -> fileName
		);
		return -1;
	}


	//No hint file precised, use stream
	LPCSTR szStreamType = muxInfo -> streamFormat;
	for (int i = m_nFactoriesAvailable; --i >= 0;)
		if (m_pFactories [i] -> IsTypeOf (szStreamType)
		)
		{	ESDesc *pDesc = new ESDesc;
			pDesc -> startTime = muxInfo -> startTime;
			pDesc -> fileName = muxInfo -> fileName;
			pDesc -> streamType = muxInfo -> streamFormat;
			pDesc -> streamData = muxInfo -> streamAttr;
			pDesc -> repeatRate = muxInfo -> repeatRate;
			pDesc -> sessionLength = muxInfo -> sessionLength;
			pDesc -> maxAhead = muxInfo -> maxAhead;
			m_pStream = m_pFactories [i] -> Create (m_fStream, pDesc);
			break;
		}
	if (!m_pStream)
	{	printf (
			"%s: Unknown decoder type %s\n",
			(LPCSTR) muxInfo -> fileName,
			(LPCSTR) szStreamType
		);
		return -1;
	}

	// Stream Track in the movie
	m_coder.SetTraceFile (fList, muxInfo -> streamFormat, flags);
	m_lTS = -1;

	//If the file is empty, return.
	if (GetDTS() == -1) {
		printf("Error! Stream file %s is empty !\n", (LPCSTR)muxInfo->fileName);
		return -1;
	}
	
	return 1;
}

/*-----------------------------------------------------------------
Ask the stream for its scale (Guido 4/8/00)
-----------------------------------------------------------------*/
int ES_DescriptorMP4 :: GetScale ()
{
	if (m_nScale == 0)
		m_nScale = m_pStream -> GetScale ();
	return m_nScale;
}

/*-----------------------------------------------------------------
Ask the stream for next time stamp
-----------------------------------------------------------------*/
long ES_DescriptorMP4 :: GetDTS (long lNow)
{
	if (m_lTS < 0)
		m_lTS = m_pStream -> GetDTS (lNow);
	GetScale (); // Guido 4/8/00
	return m_lTS;
}

int ES_DescriptorMP4 :: GetStreamType()
{
	return (int) decConfigDescr -> streamType;
}


/*------------------------------------------------------------
Create a track with trackID = ES_ID, create its media container
Check if the stream has to be in the IOD
------------------------------------------------------------*/
BOOL ES_DescriptorMP4 :: AddTrackToMovie(MP4Movie Movie, BOOL bIsInIOD)
{
	m_pMovie = Movie;
	MP4Err err = MP4NoErr;

	err = CreateTrack(); if (err) return err;
	err = CreateMedia(); if (err) return err;
	return WriteToTrack(bIsInIOD);
}


/*------------------------------------------------------------
Get the decoder & decoder specific infos, creates the sample entry
------------------------------------------------------------*/
MP4Err ES_DescriptorMP4::InsertStreamInMedia()
{
	MP4Err err;
	
	MP4Handle decoderSpecificInfo;	

	int iDecSpecInfoSize = 0;
	BYTE *p, *buffer;
	u32 OCRId;

	err = MP4NoErr;

	/*-------------------------------------------------------------
	Get the SLConfigDescr
	-------------------------------------------------------------*/
	SLConfigDescriptor configDefault;
	SLConfigDescriptor *pConfig = slConfigDescr;
	if (pConfig == NULL)
		pConfig = &configDefault;

	//Get the SpecificInfo
	if (decConfigDescr->decSpecificInfo != NULL_DESC) {
		//First, encode the DecSpecInfo
		decConfigDescr->decSpecificInfo -> EncDec (&m_coder, CALC_LENGTH, NULL);
		decConfigDescr->decSpecificInfo -> EncDec (&m_coder, ENCODE, NULL);
		m_coder.Align ();

		iDecSpecInfoSize = m_coder.GetDataLength();
		
		
		err = MP4NewHandle(iDecSpecInfoSize, &decoderSpecificInfo);  if (err) return err;
		p = (BYTE*) *decoderSpecificInfo;

		//Then write the DecSpecInfo
		buffer = (BYTE *) calloc(iDecSpecInfoSize, 1);
		memcpy(buffer, m_coder.GetData(), iDecSpecInfoSize);

		for (int i=0; i < iDecSpecInfoSize; i++) {
			*p++ = buffer[i];
		}
		free(buffer);
	} else {
		decoderSpecificInfo = NULL;
	}

	//Check for Object Clock References
	//Are we COR1 now, so OCR_ES_ID is in ESDESCR, not in SLConf
#ifdef PRECOR1
	if (pConfig ->OCR_ES_Id != NULL) {
		OCRId = pConfig ->OCR_ES_Id ;
	} else {
		OCRId = 0;
	}
#else
	ES_Descriptor *pESD = (ES_Descriptor *) this;
	OCRId = pESD->OCR_ES_Id;
#endif
	
	err = MP4NewHandle(0, &m_pSampleEntry); if (err) goto bail;
	//Create the sample description
	err = MP4NewSampleDescriptionWithOCRAssociation(m_Track, m_pSampleEntry, 1,
						decConfigDescr -> objectTypeIndication, 
					 	decConfigDescr -> streamType,
						decConfigDescr -> bufferSizeDB,
					 	decConfigDescr -> maxBitrate,
						decConfigDescr -> avgBitrate,
						decoderSpecificInfo,
						OCRId); if (err) goto bail;

	err = WriteUsingStreamFactory(pConfig); if (err) goto bail;


bail:
	if (decoderSpecificInfo) MP4DisposeHandle(decoderSpecificInfo);
	if (m_pSampleEntry) MP4DisposeHandle(m_pSampleEntry);

	return err;
}


/*------------------------------------------------------------
Insert the stream in the media using StreamFactory 
to calculate the DTS and CTS of each AU
------------------------------------------------------------*/
MP4Err ES_DescriptorMP4::WriteUsingStreamFactory(SLConfigDescriptor *pSLConfig)
{
	MP4Err err;
	u32 size = 0, maxDuration, duration;
	u32 DTSNext, DTS, CTS;

	MP4Handle sampleSize;
	MP4Handle sampleDuration;
	MP4Handle compositionOffset, compOff;
	MP4Handle sampEnt = m_pSampleEntry;


	maxDuration = DEF_SAMPLE_DURATION;
	err = MP4NoErr;

	err = MP4NewHandle(sizeof(u32), &sampleDuration); if (err) goto bail;
	err = MP4NewHandle(sizeof(u32), &m_pSample);if (err) goto bail;
	err = MP4NewHandle(sizeof(u32), &sampleSize);if (err) goto bail;
	err = MP4NewHandle(sizeof(u32), &compositionOffset);if (err) goto bail;

	// Store AUs till EOF (GetDTS() == -1)
	while (GetDTS() != -1) {
	
		//Write the AU
		Store();

		err = MP4GetHandleSize(m_pSample, &size); if (err) return err;

//		Guido 4/8/00 : allow for timescales other than ms !
//		DTS = GetDTS () * (pSLConfig -> timeStampResolution / 1000.);
//		CTS = m_pStream -> GetCTS () * (pSLConfig -> timeStampResolution / 1000.);
		DTS = GetDTS () * (pSLConfig -> timeStampResolution / (float)(m_pStream -> GetScale ()));
		CTS = m_pStream -> GetCTS () * (pSLConfig -> timeStampResolution / (float)(m_pStream -> GetScale ()));

		//Get the composition offset ( = CTS - DTS) if needed
		if ( pSLConfig -> useTimeStampsFlag
			&& m_pStream -> GetCTS () >= 0
			) {

		*(u32 *) *compositionOffset = ( CTS > DTS ) ? CTS - DTS : 0;
			compOff = compositionOffset;
		} else {
			compOff = NULL;
		}


		// Get the duration of the AU (ie DTSNext - DTS)
		// First, reset TS to go to the next AU
		m_lTS = -1;

//		Guido 4/8/00 : allow for timescales other than ms !
//		DTSNext = GetDTS () * (pSLConfig -> timeStampResolution / 1000.);
		DTSNext = GetDTS () * (pSLConfig -> timeStampResolution / (float)(m_pStream -> GetScale ()));
		if (GetDTS() == -1) {
			//Last AU -> duration = maxDuration
			duration = maxDuration;
		} else {
			if (DTSNext > DTS) {
				duration = DTSNext - DTS;
				if (maxDuration < DTSNext - DTS) {
					maxDuration = DTSNext - DTS;
				}
			} else {
				duration = DEF_SAMPLE_DURATION;
			}
		}

		*(u32 *) *sampleSize = size;
		*(u32 *) *sampleDuration = duration;
		err = MP4AddMediaSamples(m_Media, m_pSample, 1, sampleDuration, sampleSize, sampEnt, compOff, NULL);	if (err) goto bail;

		//We use the same description entry for each sample
		if (sampEnt) {
			sampEnt = NULL;
		}
	}

	//If the associated file is empty :(
	if (!m_nAULength || !size) {
		err = MP4EOF;
	}
bail:

	MP4DisposeHandle(sampleDuration);
	MP4DisposeHandle(m_pSample);
	MP4DisposeHandle(sampleSize);
	MP4DisposeHandle(compositionOffset);
	return err;
}

/*-----------------------------------------------------------------
Write one access unit
-----------------------------------------------------------------*/
void ES_DescriptorMP4::Store ()
{
	BYTE packet[DEF_PACKET_SIZE];
	int nLength, nDataLength;

	m_nAULength = 0;
	m_bEndOfAU = false;
	int i;
	BYTE *bufferPrev = NULL, *buffer = NULL, *p;
	int prevSize = 0;

	if (prevSize) {
		bufferPrev = (BYTE *) calloc(prevSize , 1);
	}

	while (!m_bEndOfAU)
	{
		m_bEndOfAU = BuildPacket (packet, 0, nLength, nDataLength);
		assert(nLength>=0);
		m_nAULength += nDataLength;

		buffer = (BYTE *) calloc(prevSize+nLength, 1);
		memcpy(buffer, bufferPrev, prevSize);
		memcpy(buffer+prevSize, packet, nLength);
		if (bufferPrev) free(bufferPrev);
		prevSize += nLength;
		bufferPrev = (BYTE *) calloc(prevSize, 1);
		memcpy(bufferPrev, buffer, prevSize);
		free(buffer);

	}

	MP4SetHandleSize(m_pSample, prevSize);
	p = (BYTE*) *m_pSample;

	for (i=0; i<prevSize; i++) {
		*p++ = bufferPrev[i];
	}

	free(bufferPrev);

	/*-------------------------------------------------------------
	Make sure requested buffer length can hold all access units
	-------------------------------------------------------------*/
	if (m_nAULength > decConfigDescr -> bufferSizeDB)
	{	printf (
			"WARNING: decoding buffer cannot hold long access units"
			". File: %s\n",
			(LPCSTR) muxInfo -> fileName
		);
		g_bErrors = TRUE;
	}
}

/*-----------------------------------------------------------------
Fill the packet with AU datas. Return TRUE if the last packet of  
an AU was generated.
Note: We don't use any Sync Layer stuff here (the current varsion of the API
has its own SLConfig and does not handle any other one)
-----------------------------------------------------------------*/
BOOL ES_DescriptorMP4 :: BuildPacket (
	LPBYTE pPacket,
	int nMaxSize,
	int &rnLength, 
	int &rnDataLength
)
{
	if (!nMaxSize)
		nMaxSize = DEF_PACKET_SIZE;

	/*-----------------------------------------------------------------
	Fill the packet and return if end of AU or packet is full
	-----------------------------------------------------------------*/

	while (!m_bEnd && m_nPayloadLength <= nMaxSize)
	{	int c = m_pStream -> GetChar ();
		m_bEnd = c < 0;
		if (!m_bEnd) {
			m_payload [m_nPayloadLength++] = c;
		}
	}

	/*-----------------------------------------------------------------
	Store the packet.
	-----------------------------------------------------------------*/
	int nPacketLength = nMaxSize - m_coder.GetDataLength ();

	if (nPacketLength <= 0) {
		rnLength = -1;
		rnDataLength = 0;
		nPacketLength = 0;
	}
	else
	{
		if (nPacketLength > m_nPayloadLength)
			nPacketLength = m_nPayloadLength;

		int i = 0;

		memcpy (pPacket + i, m_payload, nPacketLength);
		rnLength = i += nPacketLength;
		rnDataLength = nPacketLength;

	/*-----------------------------------------------------------------
	The data that was not stored is shifted to the beginning of
	the buffer
	-----------------------------------------------------------------*/
		m_nPayloadLength -= nPacketLength;
		if (m_nPayloadLength > 0)
			memcpy (m_payload, m_payload + nPacketLength, m_nPayloadLength);
	}

	if (m_bEnd && m_nPayloadLength == 0)
	{	
		m_bStart = TRUE;
		m_bEnd = FALSE;
		return TRUE;
	}
	else
	{	
		if (nPacketLength > 0)
			m_bStart = FALSE;
		return FALSE;
	}
}


u32 ES_DescriptorMP4::GetHandlerType(u32 streamType)
{
	switch (streamType) {
	case 0x01 :
		return MP4ObjectDescriptorHandlerType;

	case 0x02:
		return MP4ClockReferenceHandlerType;

	case 0x03:
		return MP4SceneDescriptionHandlerType;

	case 0x04:
		return MP4VisualHandlerType;

	case 0x05:
		return MP4AudioHandlerType;

	case 0x06:
		return MP4MPEG7HandlerType;

	case 0x07:
		return MP4IPMPHandlerType;

	case 0x08:
		return MP4OCIHandlerType;

	case 0x09:
		return MP4MPEGJHandlerType;

	default:
		return MP4HintHandlerType;
	}
}




MP4Err ES_DescriptorMP4::CreateTrack()
{
	return MP4NewMovieTrackWithID(m_pMovie, GetTrackType(decConfigDescr -> streamType), 
					(u32) ES_ID, 
					&m_Track);
}

MP4Err ES_DescriptorMP4::CreateMedia()
{
	u32 timeScale = 0;
	MP4Err err = MP4NoErr;

	/*-------------------------------------------------------------
	Get the SLConfigDescr
	-------------------------------------------------------------*/
	SLConfigDescriptor configDefault;
	SLConfigDescriptor *pSLConfig = slConfigDescr;
	if (pSLConfig == NULL)
		pSLConfig = &configDefault;

	//In the API, media timeScale == timestampResolution (default DEF_TRACK_TIMESCALE)
	timeScale = (u32) pSLConfig->timeStampResolution;
	if (!timeScale) {
		timeScale = DEF_TRACK_TIMESCALE;
	}
	
	//Add a movie container to that track (it may be an URL)

	err = MP4NewTrackMedia(m_Track, &m_Media, 
			GetHandlerType(decConfigDescr -> streamType),
			timeScale, 
			NULL); if (err) return err;
	return MP4SetMediaLanguage(m_Media, "und");


}

u32 ES_DescriptorMP4::GetTrackType(u32 streamType)
{
	switch (streamType) {
	case 0x03:	//BIFS is Visual
	case 0x04:	//Visual is Visual
		return MP4NewTrackIsVisual;

	case 0x05:
		return MP4NewTrackIsAudio;

	default:
		return MP4NewTrackIsPrivate;
	}
}


MP4Err ES_DescriptorMP4::WriteToTrack(BOOL bInIOD)
{
	MP4Err err = MP4NoErr;
	u32 uRef = 0;
	u64 mediaDuration;

	if (URL_Flag) {
		//Add a reference to the URL (not tested yet)
		MP4Handle URLHandle;
		err = MP4NewHandle(sizeof(SDLString), &URLHandle); if (err) return err;
		*((LPCSTR*) *URLHandle) = URLstring;
		err = MP4AddMediaDataReference (m_Media, &uRef, URLHandle, NULL); if (err) return err;
		MP4DisposeHandle(URLHandle);
		if (err) return err;
	} else {
		//Add the stream to the track
		err = InsertStreamInMedia(); if (err) return err;
	}

	/* ***************************************************
	Insert media in the track
	if bIsInIOD, this stream is in the initial Object descriptor 
	-> we have to add it
	Then set the track enable
	************************************************** */

	//EndMediaEdit -> compute the duration
	err = MP4EndMediaEdits(m_Media); if (err) return err;
	err = MP4GetMediaDuration(m_Media, &mediaDuration ); if (err) return err;
	err = MP4InsertMediaIntoTrack( m_Track, muxInfo->startTime, 0, mediaDuration, 1 ); if (err) return err;
	if (bInIOD) 
		err = MP4AddTrackToMovieIOD(m_Track); if (err) return err;
	return MP4SetTrackEnabled( m_Track, 1); 

}


/*-----------------------------------------------------------------
The StreamFactoryBase constructor registers a factory object
-----------------------------------------------------------------*/
StreamFactoryBase :: StreamFactoryBase (LPCSTR szType)
{
	m_szType = szType;
	ES_DescriptorMP4 :: RegisterFactory (this);
}

/*-----------------------------------------------------------------
Check if a stream factory object is of a required type
-----------------------------------------------------------------*/
BOOL StreamFactoryBase :: IsTypeOf (LPCSTR szType)
{
	return
		(	szType != NULL && m_szType != NULL &&
			!stricmp (szType, m_szType)
		);
}



