/***********************************************************************
This software module was originally developed by Jean Le Feuvre
(e-Vue Inc.) in the course of development of the MPEG-4 Systems
(ISO/IEC 14496-1) standard. This software module is an implementation of
a part of one or more MPEG-4 Systems (ISO/IEC 14496-1) tools as
specified by the MPEG-4 Systems (ISO/IEC 14496-1) standard. ISO/IEC
gives users of the MPEG-4 Systems (ISO/IEC 14496-1) free license to
this software module or modifications thereof for use in hardware or
software products claiming conformance to the MPEG-4 Systems (ISO/IEC
14496-1). Those intending to use this software module in hardware or
software products are advised that its use may infringe existing patents.
The original developer of this software module and his/her company, the
subsequent editors and their companies, and ISO/IEC have no liability
for use of this software module or modifications thereof in an
implementation. Copyright is not released for non MPEG-4 Systems
(ISO/IEC 14496-1) conforming products. e-Vue Inc. retains full right
to use the code for his/her own purpose, assign or donate the code to a
third party and to inhibit third parties from using the code for non
MPEG-4 Systems (ISO/IEC 14496-1) conforming products. This copyright
notice must be included in all copies or derivative works.
Copyright  2000.
************************************************************************/

/*-------------------------------------------------------------------------
Project Im1Player
INTERNATIONAL ORGANIZATION FOR STANDARDIZATION
ISO/IEC JTC1/SC29/WG11

SUBSYSTEM:  MP4 File Producer
FILE:       main.cpp
AUTHOR:     Jean Le Feuvre (e-Vue Inc , jlefeuvre@e-vue.com), March 2000
OVERVIEW:   main: main() function for the MP4 File Creator
-------------------------------------------------------------------------*/


#include "defs.h"
#define MAIN
#include "MP4Enc.h"
#include "Token.h"
#include "SysCoder.h"
#include "VrmlPars.h"

BOOL g_bErrors;
Token nextToken;

int main (int argc, char **argv)
{
	FILE *fList;
	int nNameLength;

	if (argc < 2)
	{	printf ("No input means no output :)\n");
		return -1;
	}



	/*-----------------------------------------------------------------
	Source : ".scr", output ".mp4" and list is ".trc"
	-----------------------------------------------------------------*/
	ZString stFileName (*++argv);
	ZString stMovieName ("");
	nNameLength = stFileName.GetLength ();
	LPCSTR szExt = strrchr (stFileName, '.');

	if (szExt && !strchr (szExt, '\\'))
		nNameLength -= strlen (szExt);
	else
		stFileName += ".scr";

	VRMLParser parser;
	if (!parser.Init (stFileName))
	{	printf ("Cannot open %s\n", (LPCSTR) stFileName);
		parser.Shutdown ();
		return -1;		
	}

	stFileName.Cut (nNameLength);
	stFileName += ".mp4";
	if (argc == 3 ) {
		stMovieName = *++argv;
	} else {
		stMovieName = stFileName;
	}

	stFileName.Cut (nNameLength);
	stFileName += ".trc";
	if ((fList = fopen (stFileName, "w")) == NULL)
	{	printf ("Cannot open %s\n", (LPCSTR) stFileName);
		return -1;
	}
	
	

	//Init text parsing
	nextToken.Init (&parser, stFileName, fList);
	//Start encoding
	MP4Enc((LPCSTR) stMovieName).Start (fList, TRACE_NONE);

	parser.Shutdown ();
	fclose(fList);

	if (g_bErrors)
	{	printf ("Hit any key to continue\n");
		getchar ();
		return -1;
	}

	return 0;
}