/***********************************************************************
This software module was originally developed by Zvi Lifshitz
(Triton R&D Ltd.) in the course of development of the MPEG-4 Systems
(ISO/IEC 14496-1) standard. This software module is an implementation of
a part of one or more MPEG-4 Systems (ISO/IEC 14496-1) tools as
specified by the MPEG-4 Systems (ISO/IEC 14496-1) standard. ISO/IEC
gives users of the MPEG-4 Systems (ISO/IEC 14496-1) free license to
this software module or modifications thereof for use in hardware or
software products claiming conformance to the MPEG-4 Systems (ISO/IEC
14496-1). Those intending to use this software module in hardware or
software products are advised that its use may infringe existing patents.
The original developer of this software module and his/her company, the
subsequent editors and their companies, and ISO/IEC have no liability
for use of this software module or modifications thereof in an
implementation. Copyright is not released for non MPEG-4 Systems
(ISO/IEC 14496-1) conforming products. Triton R&D Ltd. retains full right
to use the code for his/her own purpose, assign or donate the code to a
third party and to inhibit third parties from using the code for non
MPEG-4 Systems (ISO/IEC 14496-1) conforming products. This copyright
notice must be included in all copies or derivative works.
Copyright  1998.
************************************************************************

For more information or to receive an up to date version of this module,
you may call Zvi Lifshitz at zvil@csi.com.
----------------------------------------------------------------------*/

/*-------------------------------------------------------------------------
Project Im1Player
INTERNATIONAL ORGANIZATION FOR STANDARDIZATION
ISO/IEC JTC1/SC29/WG11

SUBSYSTEM:  Multiplexer
FILE:       main.cpp
AUTHOR:     Zvi Lifshitz (Triton R&D Ltd.)
OVERVIEW:   main() function for the off-line Mux
-------------------------------------------------------------------------*/
#include "defs.h"
#define MAIN
#include "Token.h"
#include "Mux.h"
#include "SysCoder.h"
#include "VrmlPars.h"

Token nextToken;
BOOL g_bErrors;

int main (int argc, char **argv)
{
	FILE *fTRIF, *fList;
	int nNameLength;

	if (argc < 2)
	{	printf ("Missing file name\n");
		return -1;
	}

	/*-----------------------------------------------------------------
	Source file name extension is "scr", output extension is "trif" and
	list file is "trc"
	-----------------------------------------------------------------*/
	ZString stFileName (*++argv);
	nNameLength = stFileName.GetLength ();
	LPCSTR szExt = strrchr (stFileName, '.');
	if (szExt && !strchr (szExt, '\\'))
		nNameLength -= strlen (szExt);
	else
		stFileName += ".scr";

	VRMLParser parser;
	if (!parser.Init (stFileName))
	{	printf ("Cannot open %s\n", (LPCSTR) stFileName);
		parser.Shutdown ();
		return -1;		
	}

	stFileName.Cut (nNameLength);
	stFileName += ".trif";
	if ((fTRIF = fopen (stFileName, "wb")) == NULL)
	{	printf ("Cannot open %s\n", (LPCSTR) stFileName);
		return -1;
	}
	stFileName.Cut (nNameLength);
	stFileName += ".trc";
	if ((fList = fopen (stFileName, "w")) == NULL)
	{	printf ("Cannot open %s\n", (LPCSTR) stFileName);
		return -1;
	}

	stFileName.Cut (nNameLength);
	stFileName += ".trif.mct";  // must be last in list of files.
	/*-------------------------------------------------------------
	Parse the trace argument
	-------------------------------------------------------------*/
	int flags =  TRACE_NONE;
	if (argc >= 3 && (*++argv)[0] == '-')
	{	int c;
		for (int i = 1; (c = (*argv) [i]) != 0; i++)
			switch (tolower (c)) {
			case 'v':
				flags |= TRACE_VERBOSE;
				break;
			case 'h':
				flags |= TRACE_HEADERS;
				break;
			}
	}

	/*-----------------------------------------------------------------
	The Multiplex work is done by the Mux object
	-----------------------------------------------------------------*/
	nextToken.Init (&parser, stFileName, fList);
	Mux (fTRIF, (LPCSTR) stFileName).Start (fList, flags, stFileName);
	parser.Shutdown ();
	fclose (fList);
	fclose (fTRIF);
	if (g_bErrors)
	{	printf ("Hit any key to continue\n");
		getchar ();
		return -1;
	}
	return 0;
}