/*-------------------------------------------------------------------------
Project Im1Player
INTERNATIONAL ORGANIZATION FOR STANDARDIZATION
ISO/IEC JTC1/SC29/WG11

SUBSYSTEM:  UIStream Decoder
FILE:       UIStreamDec.cpp
AUTHOR:     Cyril Concolato (ENST/COMELEC)
OVERVIEW:   Implementation of UIStreamDecoder.
-------------------------------------------------------------------------*/
#include "defs.h"
#include <assert.h>
#include <Executiv.h>
#include "Presenter.h"
#include "UIStreamDec.h"
#include "NodeFld.h"
#include "Root.h"
#include "Nodes.h"
#include "Except.h"
#include "Inline.h"
#include "ODP4.h"
#include "InputSensorImp.h"
#include "MemBifs.h"

#define pCoder this

//-----------------------------------------------------------------
// UIStreamDecoder constructor
//-----------------------------------------------------------------
UIStreamDecoder :: UIStreamDecoder( 
		Executive *pExecutive, 
		InlineImp *pSource, 
		FILE *fList,
		LPCSTR szListTitle,
		int flags,
		int objectTypeIndication
) : SysCoder (fList, szListTitle, flags, objectTypeIndication),
	m_pExecutive(pExecutive), 
	m_pSource(pSource),
	m_pScene (pSource ? pSource -> GetScene () : NULL),
	m_nNodeIDbits(10), m_nProtoIDbits(10),
	m_pBufferDecoder(NULL),
	m_pNode(NULL)
{
}

UIStreamDecoder :: ~UIStreamDecoder() {
	m_pDeviceFilter->DeviceListener_Detach_Req(m_nDeviceId);
	if (m_pBufferDecoder)
		delete m_pBufferDecoder;
}

/*-----------------------------------------------------------------
Parses updates from the stream, and update the scene accordingly.
-----------------------------------------------------------------*/
void UIStreamDecoder :: ParseFrame (LPCBYTE pData, int nLength, DWORD dwTS)
{
	Reset (pData, nLength);
	dwTS;
	TRACEF ("AT %d", dwTS);
	TRACENEWLINE;
	try
	{
		if (m_pBufferDecoder) {
			m_pNode -> UpdateBuffer();
			m_pBufferDecoder->ParseUpdateFrame ();
		}
	}
	catch (BifsException err)
	{
		err;
		Error (SCENE_PARSING_ERROR);
	}
}

/*----------------------------------------------------------------
Parse the data in the specificInfo field of the device descriptor,
and configure the decoder appropriately.
-----------------------------------------------------------------*/
bool UIStreamDecoder :: Configure (const BYTE *specificInfo, int nInfoLength)
{
	UIConfig *pUiConfig = NULL;
	bool bResult = true;
	try
	{	if (nInfoLength > 0)
		{	ODDecoder odDec (
				m_pExecutive,
				specificInfo,
				nInfoLength,
				m_pExecutive -> GetTraceFile (),
				"UIConfig",
				m_pExecutive -> GetTraceFlags (),
				GetVersion ()
			);

			pUiConfig = new UIConfig;
			pUiConfig -> SetLength (nInfoLength * 8);
			pUiConfig -> EncDec (&odDec, DECODE, NULL);
			if (pUiConfig -> nodeIDbits > 0)
				m_nNodeIDbits = pUiConfig -> nodeIDbits;
			if (pUiConfig -> protoIDbits > 0)
				m_nProtoIDbits = pUiConfig -> protoIDbits;
			if (pUiConfig -> hasSize) {
				m_nPixelWidth = pUiConfig -> pixelWidth;
				m_nPixelHeight = pUiConfig -> pixelHeight;
			}
			m_szDeviceURL = pUiConfig -> deviceURL;
			m_pDeviceFilter = GetExecutive()->GetDeviceFilter();
			m_pDeviceFilter -> DeviceListener_Attach_Req (this,m_szDeviceURL);

		}
		bResult = true;
	}
	catch (BifsException)
	{
		m_pExecutive -> Error (INVALID_UI_CONFIG);
		bResult = false;
	}
	delete pUiConfig;

	if (bResult) {
		// Register this decoder as a DeviceListener
	}
	return bResult;
}

void UIStreamDecoder::UpdateBuffer(const BYTE *nodeBuffer, int nodeBufferLength, InputSensorImp* pNode) {
	
	m_pNode = pNode;
	try
	{	
		if (m_pBufferDecoder) {
			delete m_pBufferDecoder;
			m_pBufferDecoder = NULL;
		}
		
		if (nodeBufferLength) {
			m_pBufferDecoder =
				new MemBIFSDecoder (
					GetExecutive (),
					nodeBuffer,
					nodeBufferLength,
					GetExecutive () -> GetTraceFile (),
					"InputSensor",
					GetExecutive () -> GetTraceFlags (),
					0,
					m_pNode->GetRoot ()
			);
			m_pBufferDecoder -> SetUICoder (this);
		}

	} catch (BifsException)	{
		#ifdef PLAYER
		GetExecutive () -> Error (INVALID_UI_BUFFER);
		#endif
	}
}

void UIStreamDecoder::DataReceive_Ind(const BYTE* pStreamData, const int nStreamDataLength, const int nErrorFlag) {
	nErrorFlag;
	ParseFrame(pStreamData, nStreamDataLength, GetExecutive()->GetPresenter() ->GetTime()/1000);
}
