/*-------------------------------------------------------------------------
Project Im1Player
INTERNATIONAL ORGANIZATION FOR STANDARDIZATION
ISO/IEC JTC1/SC29/WG11

  SUBSYSTEM:  Decoder
  FILE:       UIStreamDec.h
  AUTHOR:     Cyril Concolato (ENST/COMELEC)
  OVERVIEW:   Definition of UIStreamDecoder  
-------------------------------------------------------------------------*/
#ifndef UISTREAMDEC_H // Sentry, use file only if it's not already included.
#define UISTREAMDEC_H

#include "SysCoder.h"
#include "Nodes.h"
#include "Device.h"

class InlineImp;
class InputSensorImp;
class Root;
class Executive;
class MemBIFSDecoder;

class UIStreamDecoder : public SysCoder, public DeviceListener
{
public:
	UIStreamDecoder(Executive *pExecutive, 
					InlineImp *pSource, 
					FILE *fList,
					LPCSTR szListTitle,
					int flags,
					int objectTypeIndication);
	~UIStreamDecoder();
	bool Configure (const BYTE *specificInfo, int nInfoLength);

	Executive *GetExecutive () const {return m_pExecutive;}
	InlineImp *GetSource () const {return m_pSource;}
	Root *GetScene () const {return m_pScene;}
	int GetNodeIDbits () {return m_nNodeIDbits;}
	void UpdateBuffer(const BYTE *nodeBuffer, int nodeBufferLength, InputSensorImp* pNode);
	void SetNode(InputSensorImp *pNode) { m_pNode = pNode; }
	InputSensorImp* GetNode() { return m_pNode; }
	
	virtual void DataReceive_Ind(const BYTE* pStreamData, const int	nStreamDataLength, const int nErrorFlag);
	void Terminate() {
		m_pDeviceFilter->DeviceListener_Detach_Req(m_nDeviceId);
	}

private:
	virtual void ParseFrame (LPCBYTE pData, int nLength, DWORD dwTS);

	int m_nNodeIDbits, m_nProtoIDbits;
	int m_nPixelWidth, m_nPixelHeight;
	LPCSTR m_szDeviceURL;

	Root *m_pScene;
	Executive *m_pExecutive;
	InlineImp *m_pSource;
	MemBIFSDecoder *m_pBufferDecoder;
	InputSensorImp *m_pNode;
};

#endif                                      // UISTREAMDEC_H sentry.
