/***********************************************************************
This software module was originally developed by Zvi Lifshitz
(Triton R&D Ltd.) in the course of development of the MPEG-4 Systems
(ISO/IEC 14496-1) standard. This software module is an implementation of
a part of one or more MPEG-4 Systems (ISO/IEC 14496-1) tools as
specified by the MPEG-4 Systems (ISO/IEC 14496-1) standard. ISO/IEC
gives users of the MPEG-4 Systems (ISO/IEC 14496-1) free license to
this software module or modifications thereof for use in hardware or
software products claiming conformance to the MPEG-4 Systems (ISO/IEC
14496-1). Those intending to use this software module in hardware or
software products are advised that its use may infringe existing patents.
The original developer of this software module and his/her company, the
subsequent editors and their companies, and ISO/IEC have no liability
for use of this software module or modifications thereof in an
implementation. Copyright is not released for non MPEG-4 Systems
(ISO/IEC 14496-1) conforming products. Triton R&D Ltd. retains full right
to use the code for his/her own purpose, assign or donate the code to a
third party and to inhibit third parties from using the code for non
MPEG-4 Systems (ISO/IEC 14496-1) conforming products. This copyright
notice must be included in all copies or derivative works.
Copyright  1997.
************************************************************************

For more information or to receive an up to date version of this module,
you may call Mikael Bourges-Sevenier at mikael.bourgessevenier@cnet.francetelecom.fr.
----------------------------------------------------------------------*/

/*-------------------------------------------------------------------------
Project Im1Player
INTERNATIONAL ORGANIZATION FOR STANDARDIZATION
ISO/IEC JTC1/SC29/WG11

SUBSYSTEM:  Presenter (simulation)
FILE:       MNodes.cpp
AUTHOR:     Mikael Bourges-Sevenier (France Telecom CNET/DIH/ATI)
OVERVIEW:   Implementation of my overloading nodes
-------------------------------------------------------------------------*/

#include "defs.h"
#include "MNodes.h"
#include "Executiv.h"
#include "Presenter.h"
#include "Stream.h"
#include "Clock.h"
#include "ODImp.h"
#include "Decoder.h"
#include "AnimStreamDec.h"
#include "Anim.h"

void MikeOverloads ()
{
	OVERLOAD_NODE (MSphere);
	OVERLOAD_NODE (MTouchSensor);
	OVERLOAD_NODE (MTimeSensor);
	OVERLOAD_NODE (MOrientationInterpolator);
	OVERLOAD_NODE (MScalarInterpolator);
	OVERLOAD_NODE (MGroup);
	OVERLOAD_NODE (MMaterial);
	OVERLOAD_NODE (MTransform);
	OVERLOAD_NODE (MTransform2D);
	OVERLOAD_NODE (MViewpoint);
}

BOOL MSphere :: Render (PresenterContext *pContext)
{
  ZPresenterContext context = *(ZPresenterContext*) pContext;
  if (GetUpdateTime () == GetPresenter () -> GetTime ())
    GetPresenter () -> Logf (context.GetTime (), 
			     "%*c%d:Sphere - radius: %f\n",
			     context.m_nTabs*3,
			     ' ',
			     GetID (),
			     (float) radius
			     );
  return TRUE;
}

BOOL MOrientationInterpolator :: Render (PresenterContext *pContext)
{
  ZPresenterContext context = *(ZPresenterContext*) pContext;
  if (GetUpdateTime () == GetPresenter () -> GetTime ())
    GetPresenter () -> Logf (context.GetTime (), 
			     "%*c%d:OrientationInterpolator \n",
			     context.m_nTabs*3,
			     ' ',
			     GetID ()
			     );
  return TRUE;
}

BOOL MScalarInterpolator :: Render (PresenterContext *pContext)
{
  ZPresenterContext context = *(ZPresenterContext*) pContext;
  if (GetUpdateTime () == GetPresenter () -> GetTime ())
    GetPresenter () -> Logf (context.GetTime (), 
			     "%*c%d:ScalarInterpolator \n",
			     context.m_nTabs*3,
			     ' ',
			     GetID ()
			     );
  return TRUE;
}

BOOL MTouchSensor :: Render (PresenterContext *pContext)
{
  ZPresenterContext context = *(ZPresenterContext*) pContext;
  if (GetUpdateTime () == GetPresenter () -> GetTime ())
    GetPresenter () -> Logf (context.GetTime (), 
			     "%*c%d:TouchSensor \n",
			     context.m_nTabs*3,
			     ' ',
			     GetID ()
			     );
  return TRUE;
}

BOOL MTimeSensor :: Render (PresenterContext *pContext)
{
  ZPresenterContext context = *(ZPresenterContext*) pContext;
  if (GetUpdateTime () == GetPresenter () -> GetTime ())
    GetPresenter () -> Logf (context.GetTime (), 
			     "%*c%d:TimeSensor \n",
			     context.m_nTabs*3,
			     ' ',
			     GetID ()
			     );
  return TRUE;
}

BOOL MGroup :: Render (PresenterContext *pContext)
{
  ZPresenterContext context = *(ZPresenterContext*) pContext;
  if (GetUpdateTime () == GetPresenter () -> GetTime ())
    GetPresenter () -> Logf (context.GetTime (), 
			     "%*c%d:Group\n\tbboxCenter: %f %f %f\n\tbboxSize: %f %f %f\n",
			     context.m_nTabs*3,
			     ' ',
			     GetID () //,
			     //MBS 110498 ((Vec3f)bboxCenter).x, ((Vec3f)bboxCenter).y, ((Vec3f)bboxCenter).z,
			     //MBS 110498 ((Vec3f)bboxSize).x, ((Vec3f)bboxSize).y, ((Vec3f)bboxSize).z
			     );
  ++context.m_nTabs;
  for (int i = 0; i < children.GetLength (); i++)
    children [i] -> Render (&context);
  return TRUE;
}

BOOL MMaterial :: Render (PresenterContext *pContext)
{
  ZPresenterContext context = *(ZPresenterContext*) pContext;
  if (GetUpdateTime () == GetPresenter () -> GetTime ())
    GetPresenter () -> Logf (context.GetTime (), 
			     "%*c%d:Material\n\tdiffuseColor: %f %f %f\n\ttransparency: %f\n",
			     context.m_nTabs*3,
			     ' ',
			     GetID (),
			     ((Color3f)diffuseColor).r,
			     ((Color3f)diffuseColor).g,
			     ((Color3f)diffuseColor).b,
			     ((float)transparency)
			     );
  return TRUE;
}

BOOL MTransform :: Render (PresenterContext *pContext)
{
  ZPresenterContext context = *(ZPresenterContext*) pContext;
  if (GetUpdateTime () == GetPresenter () -> GetTime ())
    GetPresenter () -> Logf (context.GetTime (), 
			     "%*c%d:Transform\n\torientation: (%f, %f, %f), %f\n\tposition: %f, %f, %f\n",
			     context.m_nTabs*3,
			     ' ',
			     GetID (),
			     ((Rotation)rotation).x,
			     ((Rotation)rotation).y,
			     ((Rotation)rotation).z,
			     ((Rotation)rotation).theta,
			     ((Vec3f)translation).x,
			     ((Vec3f)translation).y,
			     ((Vec3f)translation).z
			     );
  ++context.m_nTabs;
  for (int i = 0; i < children.GetLength (); i++)
    children [i] -> Render (&context);
  return TRUE;
}

BOOL MTransform2D :: Render (PresenterContext *pContext)
{
  ZPresenterContext context = *(ZPresenterContext*) pContext;
  if (GetUpdateTime () == GetPresenter () -> GetTime ())
    GetPresenter () -> Logf (context.GetTime (), 
			     "%*c%d:Transform2D\n\trotationAngle: %f\n\ttranslation: %f, %f\n",
			     context.m_nTabs*3,
			     ' ',
			     GetID (),
			     ((float)rotationAngle),
			     ((Vec2f)translation).x,
			     ((Vec2f)translation).y
			     );
  ++context.m_nTabs;
  for (int i = 0; i < children.GetLength (); i++)
    children [i] -> Render (&context);
  return TRUE;
}

BOOL MViewpoint :: Render (PresenterContext *pContext)
{
  ZPresenterContext context = *(ZPresenterContext*) pContext;
  if (GetUpdateTime () == GetPresenter () -> GetTime ())
    GetPresenter () -> Logf (context.GetTime (), 
			     "%*c%d:Viewpoint\n\torientation: (%f, %f, %f), %f\n\tposition: %f, %f, %f\n",
			     context.m_nTabs*3,
			     ' ',
			     GetID (),
			     ((Rotation)orientation).x,
			     ((Rotation)orientation).y,
			     ((Rotation)orientation).z,
			     ((Rotation)orientation).theta,
			     ((Vec3f)position).x,
			     ((Vec3f)position).y,
			     ((Vec3f)position).z
			     );
  return TRUE;
}

