// This software module was originally developed by Wei Wu(Rockwell Science
// Center) and Wenlong Dong(Rockwell Science Center) in the course of
// development of the MPEG-4 Systems standard
// (ISO/IEC 14496-1).  This software module is an implementation of a part
// of one or more MPEG-4 Systems tools as specified by the MPEG-4 Systems
// standard.  ISO/IEC gives users of the MPEG-4 Systems standard free
// license to this software module or modifications thereof for use in
// hardware or software products claiming conformance to the MPEG-4 Systems
// standard.  Those intending to use this software module in hardware or
// software products are advised that its use may infringe existing
// patents.  The original developer of this software module and his/her
// company, the subsequent editors and their companies, and ISO/IEC have no
// liability for use of this software module or modifications thereof in an
// implementation.  Copyright is not released for non MPEG-4 Systems
// conforming products.  The original developer retains full right to use
// the code for his/her own purpose, assign or donate the code to a third
// party and to inhibit third parties from using the code for non MPEG-4
// Systems conforming products.  This copyright notice must be included in
// all copies or derivative works.
//
// For more information or to receive an up to date version of this module,
// you may call Wei Wu at weiwu@jupiter.risc.rockwell.com or Wenlong Dong
// at wdong@usc.edu.
//
// Copyright 1998.

/*-------------------------------------------------------------------------
Project Im1Player
INTERNATIONAL ORGANIZATION FOR STANDARDIZATION
ISO/IEC JTC1/SC29/WG11

SUBSYSTEM:	Script Node
FILE: 	    DecScript.cpp
AUTHOR:     Wei Wu       (Rockwell Science Center)
		    Wenlong Dong (Rockwell Science Center)
		    Yuval Fisher (Rockwell Science Center)
OVERVIEW:   Implementation of the DecScript class
----------------------------------------------------------------------*/
//$Log: DecScript.cpp,v $
//Revision 1.1.1.1.2.1  2000/03/10 15:59:34  Alex Cotarmanac'h
//
// Revision 2001/06/30 Marc Patoureaux marc@envivio.com
/******************************************************************
Object Method Call, New, Object_memberAccess and ET_FunctionCall
are not implemented. Member Acces is especially usefull for Nodes, 
but can be avoided with Routes.
*******************************************************************/

#include "defs.h"
#include "DecScript.h"
#include "BifsDec.h"
#include "NodeFld.h"
# include "vsfields.h"
#include "vsexpr.h"
#include "vsfunction.h"
#include "vsstatement.h"
#include "VsSwitch.h"

DecScript::DecScript(SFScript *script)
{
	pScript = script;
}

DecScript::~DecScript()
{
}

//AC200005
const char *DecScript::exprDesc[] ={
"ET_CURVED_EXPR",
"ET_NEGATIVE",
"ET_NOT",
"ET_ONESCOMP",
"ET_INCREMENT",
"ET_DECREMENT",
"ET_POST_INCREMENT",
"ET_POST_DECREMENT",
"ET_CONDTEST",
"ET_STRING",
"ET_NUMBER",
"ET_IDENTIFIER",
"ET_FUNCTION_CALL",
"ET_NEW",
"ET_OBJECT_MEMBER_ACCESS",
"ET_OBJECT_METHOD_CALL",
"ET_ARRAY_DEREFERENCE",
"ET_ASSIGN",
"ET_PLUSEQ",
"ET_MINUSEQ",
"ET_MULTIPLYEQ",
"ET_DIVIDEEQ",
"ET_MODEQ",
"ET_ANDEQ",
"ET_OREQ",
"ET_XOREQ",
"ET_LSHIFTEQ",
"ET_RSHIFTEQ",
"ET_RSHIFTFILLEQ",
"ET_EQ",
"ET_NE",
"ET_LT",
"ET_LE",
"ET_GT",
"ET_GE",
"ET_PLUS",
"ET_MINUS",
"ET_MULTIPLY",
"ET_DIVIDE",
"ET_MOD",
"ET_LAND",
"ET_LOR",
"ET_AND",
"ET_OR",
"ET_XOR",
"ET_LSHIFT",
"ET_RSHIFT",
"ET_RSHIFTFILL",
"ET_BOOLEAN",
"NUMBER_OF_EXPR_TYPE"
};
#define BINOP_MINVAL ET_EQ

char* DecScript::binaryOp[] =
{

	"==", "!=","<", "<=",
	">", ">=", "+", "-",
	"*", "/", "%",
	"&&", "||",
	"&", "|", "^",
	"<<", ">>", ">>>"
};

char* DecScript::assignmentOp[] =
{
	"=", "+=", "-=", "*=", "/=", "%=", "<<=", ">>=", ">>>=",
	"&=", "^=", "|="
};

char* DecScript::unaryOp[] =
{
	"!", "~", "-", "++", "--"
};

/*
void
DecScript::InsertExID(int num)  // FAUX !!
{
	for(int i=0; i<num; i++) {
		ZString ret;
		ret = "var"+('0'+ i);
		m_identifierBuffer += ret;
		IdCnt ++;
	}
}
*/
/*
ZString
DecScript::GetTextString()
{
	return m_outBuff;
}
*/
void
DecScript::Decode(BIFSDecoder *pCoder)
{

	VsStatementList *StList;

	TRACEF("\"javascript:", 0);
	TRACENEWLINE;
	TRACEINDENT(1);
	TRACENEWLINE;
	while (pCoder -> ParseInt(1,"hasFunction")) {
		TRACEF("function ",0);
		LPCSTR str = GetIdentifier(pCoder);
		VsFunctionDef *func = beginFunction(str); // Verifier
		Arguments(pCoder, func);
		TRACEINDENT(1);
		StList = StatementBlock(pCoder);
		TRACEINDENT(-1);
		func -> endDefinition(StList);
		functions += func;
	}

	TRACENEWLINE;
	TRACEINDENT(-1);
	TRACEF("\"", 0);
	TRACENEWLINE;
//end AC

	CheckUnimp();
}

void
DecScript::Arguments(BIFSDecoder *pCoder, VsFunctionDef *f)
{
	int val = pCoder -> ParseInt(1,"hasArgument");
	ASSERT (f != NULL);
	f;
	//m_outBuff += "(";
	TRACEF("(",0);
	while(val) {
		LPCSTR str = GetIdentifier(pCoder);
		f -> addArg(str);
		val = pCoder -> ParseInt(1, "hasArgument");
		if (val) {
			//m_outBuff += ", ";
			TRACEF(", ",0);
		}
	}
	//m_outBuff += ")\n";
	TRACEF(")",0);
	TRACENEWLINE;
}

VsStatementList *
DecScript::StatementBlock(BIFSDecoder *pCoder)
{
	VsStatementList *stlist = new VsStatementList;
	VsStatement *stmnt;

    if(pCoder -> ParseInt(1,"isCompoundStatement")) { // mp : conformance
		//m_outBuff += "{\n";
		TRACEF("{",0);
		TRACENEWLINE;
		TRACEINDENT(1);
		while (pCoder -> ParseInt(1, "hasStatement")) {
			stmnt = Statement(pCoder);
			*stlist += stmnt;
		}
		//m_outBuff += "}\n";
		TRACEINDENT(-1);
		TRACEF("}",0);
		TRACENEWLINE;

	}
	else
	{
		stmnt = Statement(pCoder);
		*stlist += stmnt;
	}

	return stlist;
}

VsStatement *
DecScript::Statement(BIFSDecoder *pCoder)
{
	int val;
	VsStatement *stmnt;

	val = pCoder -> ParseInt(NUMBITS_STATEMENT,"statementType");
	switch (val) {
	case ST_IF:
		stmnt = ifStatement(pCoder);
		break;
	case ST_FOR:
		stmnt = forStatement(pCoder);
		break;
	case ST_WHILE:
		stmnt = whileStatement(pCoder);
		break;
	case ST_RETURN:
		stmnt = returnStatement(pCoder);
		TRACEF(";",0);
		TRACENEWLINE;
		//m_outBuff += ";\n";
		break;
	case ST_BREAK:
		stmnt = breakStatement(pCoder);
		TRACEF(";",0);
		TRACENEWLINE;
		//m_outBuff += ";\n";
		break;
	case ST_CONTINUE:
		stmnt = continueStatement(pCoder);
		TRACEF(";",0);
		TRACENEWLINE;
		//m_outBuff += ";\n";
		break;
	case ST_COMPOUND_EXPR:
		stmnt = CompoundExpression(pCoder);
		TRACEF(";",0);
		TRACENEWLINE;
		//m_outBuff += ";\n";
		break;
	case ST_SWITCH:
		stmnt = switchStatement(pCoder);

		break;
	default:
		stmnt = NULL;
		fprintf(stderr, "Error: Unrecognized statement type %d\n", val);
	}

	return stmnt;
}

VsStatement *
DecScript::ifStatement(BIFSDecoder *pCoder)
{
	int val;
	VsExpr *stmnt_expr;
	VsStatementList *stmnt_else, *stmnt_blk;

	//m_outBuff += "if (";
	TRACEF("if (",0);
	stmnt_expr = CompoundExpression(pCoder);
	//m_outBuff += ")\n";
	TRACEF(")",0);
	TRACENEWLINE;
	stmnt_blk = StatementBlock(pCoder);

	val = pCoder -> ParseInt(1, "hasELSEStatement");
	if (val) {
		//m_outBuff += "else\n";
		TRACEF("else",0);
		TRACENEWLINE;
		stmnt_else = StatementBlock(pCoder);
	}
	else
		stmnt_else = NULL;

	return new VsIf(stmnt_expr, stmnt_blk, stmnt_else);

}


VsStatement *
DecScript::switchStatement(BIFSDecoder *pCoder)
{
	int numBits, caseValue;

	TRACEF("switch (",0);
	VsExpr * switched_expr = CompoundExpression(pCoder);
	VsSwitch * switch_stmt = new VsSwitch(switched_expr);
	TRACEF(")",0);
	TRACENEWLINE;
	TRACEF("{",0);
	TRACENEWLINE;
	TRACEINDENT(1);
	do {
		TRACEF("case ",0);
		numBits = pCoder -> ParseInt(5, "numbits"); // mp conformance
		caseValue = pCoder -> ParseInt(numBits, "caseValue");
		TRACEF("%d ",caseValue);
		//CompoundExpression(pCoder);
		//VsExpr * caselabel = CompoundExpression(pCoder);
		TRACEF(":",0);
		TRACENEWLINE;
		//m_outBuff += " :\n";
		StatementBlock(pCoder);
		//switch_stmt->addCase(caselabel,StatementBlock(pCoder));

	}
	while (pCoder->ParseInt(1,"hasMoreCases")==1);

	if (pCoder->ParseInt(1,"hasDefault") == 1)
	{
		TRACEF("default :",0);
		TRACENEWLINE;
		//m_outBuff += "default :\n";
		StatementBlock(pCoder);
		//switch_stmt->addCase(NULL,StatementBlock(pCoder));

	}
	TRACEINDENT(-1);
	TRACEF("}",0);
	TRACENEWLINE;
	//m_outBuff += "}\n";


	return switch_stmt;
}//end of switchStatement

VsStatement *
DecScript::forStatement(BIFSDecoder *pCoder)
{
	VsExpr *expr1, *expr2, *expr3;
	VsStatementList *stmntlist;

	TRACEF("for (",0);
	//m_outBuff += "for (";
	expr1 = OptionalExpression(pCoder);
	TRACEF("; ",0);
	//m_outBuff += "; ";
	expr2 = OptionalExpression(pCoder);
	TRACEF("; ",0);
	//m_outBuff += "; ";
	expr3 = OptionalExpression(pCoder);
	TRACEF(")",0);
	TRACENEWLINE;
	//m_outBuff += ")\n";
	stmntlist = StatementBlock(pCoder);

	return new VsFor(expr1, expr2, expr3, stmntlist);
}

VsStatement *
DecScript::whileStatement(BIFSDecoder *pCoder)
{
	VsExpr *expr;
	VsStatementList *stmntlist;
	TRACEF("while (",0);
	//m_outBuff += "while (";
	expr = CompoundExpression(pCoder);
	TRACEF(")",0);
	TRACENEWLINE;
	//m_outBuff += ")\n";
	stmntlist = StatementBlock(pCoder);

	return new VsWhile(expr, stmntlist);
}

VsStatement *
DecScript::returnStatement(BIFSDecoder *pCoder)
{
	VsExpr *expr = NULL;
	TRACEF("return ",0);
	//m_outBuff += "return ";

	if (pCoder -> ParseInt(1,"returnValue")) {
		expr = CompoundExpression(pCoder);
	}

	return new VsReturn(expr);
}

VsStatement *
DecScript::breakStatement(BIFSDecoder *pCoder)
{
	pCoder;
	TRACEF("break",0);
	//m_outBuff += "break";
	return new VsBreak();
}

VsStatement *
DecScript::continueStatement(BIFSDecoder *pCoder)
{	
	pCoder;
	TRACEF("continue",0);
	//m_outBuff += "continue";
	return new VsContinue();
}

VsExpr *
DecScript::CompoundExpression(BIFSDecoder *pCoder)
{
	VsExpr *expr1, *expr2;

	expr1 = Expression(pCoder);
	if(!pCoder -> ParseInt(1,"hasExpression"))
		return expr1;
	else
	{
		TRACEF(", ",0);
		//m_outBuff += ", ";
		expr2 = CompoundExpression(pCoder);
		return new VsComma(expr1, expr2);
	}

}

VsExpr *
DecScript::OptionalExpression(BIFSDecoder *pCoder)
{
	if (pCoder -> ParseInt(1,"hasCompoundExpression")) {
		return CompoundExpression(pCoder);
	}

	return NULL;
}

VsExpr *
DecScript::Expression(BIFSDecoder *pCoder, bool n_flag)
{
	VsExpr *expr1, *expr2, *expr3, *expr = NULL;
	//VsExprList *exprlist;
	int val = pCoder -> ParseInt(NUMBITS_EXPR_TYPE,"expressionType");

	LPCSTR str;

	switch(val)
	{
	case ET_CURVED_EXPR:
		TRACEF("(",0);
		//m_outBuff += "(";
		expr = CompoundExpression(pCoder);
		TRACEF(")",0);
		//m_outBuff += ")";
		break;
	case ET_NEGATIVE:
		TRACEF("-",0);
		//m_outBuff += "-";
		expr1 = Expression(pCoder);
		expr = new VsUArithOp(expr1, ET_MINUS);
		break;
	case ET_NOT:
		TRACEF("!",0);
		//m_outBuff += "!";
		expr1 = Expression(pCoder);
		expr = new VsULogicOp(expr1, ET_NOT);
		break;
	case ET_ONESCOMP:
		TRACEF("~",0);
		//m_outBuff += "~";
		expr1 = Expression(pCoder);
		expr = new VsUArithOp(expr1, ET_ONESCOMP);
		break;
	case ET_INCREMENT:
		TRACEF("++",0);
		//m_outBuff += "++";
		expr1 = Expression(pCoder);
		expr = new VsIncDec(expr1, TRUE, TRUE);
		break;
	case ET_DECREMENT:
		TRACEF("--",0);
		//m_outBuff += "--";
		expr1 = Expression(pCoder);
		expr = new VsIncDec(expr1, FALSE, TRUE);
		break;
	case ET_POST_INCREMENT:
		expr1 = Expression(pCoder);
		TRACEF("++",0);
		//m_outBuff += "++";
		expr = new VsIncDec(expr1, TRUE, FALSE);
		break;
	case ET_POST_DECREMENT:
		expr1 = Expression(pCoder);
		TRACEF("--",0);
		//m_outBuff += "--";
		expr = new VsIncDec(expr1, FALSE, FALSE);
		break;
	case ET_CONDTEST:
		expr1 = Expression(pCoder);
		TRACEF(" ? ",0);
		//m_outBuff += " ? ";
		expr2 = Expression(pCoder);
		TRACEF(" : ",0);
		//m_outBuff += " : ";
		expr3 = Expression(pCoder);
		expr = new VsCondExpr(expr1, expr2, expr3);
		break;
	case ET_STRING:
		TRACEF("'",0);
		//m_outBuff += "'";
		str = GetString(pCoder);
		expr = new VsConstant(str);
		TRACEF("'",0);
        //m_outBuff += "'";
		break;
	case ET_NUMBER:
		str = GetNumber(pCoder);
		expr = new VsConstant(atof(str));
		//m_outBuff += str;
		break;
	case ET_IDENTIFIER:
		str = GetIdentifier(pCoder);
		expr = VsFunctionDef::findIdentifier(str, n_flag);
		break;
	case ET_FUNCTION_CALL:
		expr = FunctionCall(pCoder);
		/*str = GetIdentifier(pCoder);
		expr1 = new VsConstant(str);
		TRACEF(" (",0);
		//m_outBuff += " (";
		exprlist = Params(pCoder);
		TRACEF(")",0);
		//m_outBuff += ")";
        */
		break;
	case ET_NEW:
        expr = NewObject(pCoder);
        /*
		TRACEF("new ",0);
		//m_outBuff += "new ";
		str = GetIdentifier(pCoder);
		expr1 = VsFunctionDef::findIdentifier(str);
		TRACEF("(",0);
		//m_outBuff += "(";
		exprlist = Params(pCoder);
		TRACEF(")",0);
		//m_outBuff += ")";
		Error("The new method has not been implemented yet");
        */
		break;
	case ET_OBJECT_MEMBER_ACCESS: 
   		expr = ObjectMemberAccess(pCoder);
		/*expr1 = Expression(pCoder);
		TRACEF(".",0);
		//m_outBuff += ".";
		str = GetIdentifier(pCoder);
		expr = VsFunctionDef::findIdentifier(str, expr1); // BF TO BE IMPLEMENTED !!*/
		break;
	case ET_OBJECT_METHOD_CALL:
        expr = ObjectMethodCall(pCoder);

		/*expr1 = Expression(pCoder);
		TRACEF(".",0);
		str = GetIdentifier(pCoder);
		expr2 = VsFunctionDef::findIdentifier(str);
		TRACEF("(",0);
		//m_outBuff += ".";
		exprlist = Params(pCoder);
		expr = new VsMethodCall(expr1, str, *exprlist);  /// faux!!
		TRACEF(")",0);*/
		break;
	case ET_ARRAY_DEREFERENCE:
        expr = ArrayDeref(pCoder);
        /*
		expr1 = Expression(pCoder);
		TRACEF("[",0);
		//m_outBuff += "[";
		expr2 = CompoundExpression(pCoder);
		TRACEF("]",0);
		//m_outBuff += "]";
		expr = new VsArrayVar(expr1,expr2);
        */
		break;

	case ET_MULTIPLY:
	case ET_DIVIDE:
	case ET_MOD:
	case ET_PLUS:
	case ET_MINUS:
	case ET_LSHIFT:
	case ET_RSHIFT:
	case ET_RSHIFTFILL:
	case ET_AND:
	case ET_XOR:
	case ET_OR:
		expr1 = Expression(pCoder);
		TRACEF("%s",(LPCSTR)binaryOp[val - BINOP_MINVAL]);
		//m_outBuff += binaryOp[val - BINOP_MINVAL];//AC
		expr2 = Expression(pCoder);
		expr = new VsArith(expr1, expr2, val);
		break;
	case ET_LT:
	case ET_LE:
	case ET_GT:
	case ET_GE:
	case ET_EQ:
	case ET_NE:
	case ET_LAND:
	case ET_LOR:
		expr1 = Expression(pCoder);
		TRACEF("%s",(LPCSTR)binaryOp[val - BINOP_MINVAL]);
		//m_outBuff += binaryOp[val - BINOP_MINVAL];//AC
		expr2 = Expression(pCoder);
		expr = new VsLogicOp(expr1, expr2, val);
		break;
	case ET_ASSIGN:
		expr1 = Expression(pCoder);
		TRACEF("%s",(LPCSTR)assignmentOp[0]);
		//m_outBuff += assignmentOp[0];
		expr2 = Expression(pCoder);
		expr = new VsAssign((VsVar *)expr1, expr2);
		break;
	case ET_PLUSEQ:
		expr1 = Expression(pCoder);
		TRACEF("%s",(LPCSTR)assignmentOp[1]);
		//m_outBuff += assignmentOp[1];
		expr2 = Expression(pCoder);
		expr3 = new VsArith(expr1, expr2, ET_PLUS);
		expr = new VsAssign((VsVar *)expr1, expr3);
		break;
	case ET_MINUSEQ:
		expr1 = Expression(pCoder);
		TRACEF("%s",(LPCSTR)assignmentOp[2]);
		//m_outBuff += assignmentOp[2];
		expr2 = Expression(pCoder);
		expr3 = new VsArith(expr1, expr2, ET_MINUS);
		expr = new VsAssign((VsVar *)expr1, expr3);
		break;
	case ET_MULTIPLYEQ:
		expr1 = Expression(pCoder);
		TRACEF("%s",(LPCSTR)assignmentOp[3]);
		//m_outBuff += assignmentOp[3];
		expr2 = Expression(pCoder);
		expr3 = new VsArith(expr1, expr2, ET_MULTIPLY);
		expr = new VsAssign((VsVar *)expr1, expr3);
		break;
	case ET_DIVIDEEQ:
		expr1 = Expression(pCoder);
		TRACEF("%s",(LPCSTR)assignmentOp[4]);
		//m_outBuff += assignmentOp[4];
		expr2 = Expression(pCoder);
		expr3 = new VsArith(expr1, expr2, ET_DIVIDE);
		expr = new VsAssign((VsVar *)expr1, expr3);
		break;
	case ET_MODEQ:
		expr1 = Expression(pCoder);
		TRACEF("%s",(LPCSTR)assignmentOp[5]);
		//m_outBuff += assignmentOp[5];
		expr2 = Expression(pCoder);
		expr3 = new VsArith(expr1, expr2, ET_MOD);
		expr = new VsAssign((VsVar *)expr1, expr3);
		break;
	case ET_LSHIFTEQ:
		expr1 = Expression(pCoder);
		TRACEF("%s",(LPCSTR)assignmentOp[6]);
		//m_outBuff += assignmentOp[6];
		expr2 = Expression(pCoder);
		expr3 = new VsArith(expr1, expr2, ET_LSHIFT);
		expr = new VsAssign((VsVar *)expr1, expr3);
		break;
	case ET_RSHIFTEQ:
		expr1 = Expression(pCoder);
		TRACEF("%s",(LPCSTR)assignmentOp[7]);
		//m_outBuff += assignmentOp[7];
		expr2 = Expression(pCoder);
		expr3 = new VsArith(expr1, expr2, ET_RSHIFT);
		expr = new VsAssign((VsVar *)expr1, expr3);
		break;
	case ET_RSHIFTFILLEQ:
		expr1 = Expression(pCoder);
		TRACEF("%s",(LPCSTR)assignmentOp[8]);
		//m_outBuff += assignmentOp[8];
		expr2 = Expression(pCoder);
		expr3 = new VsArith(expr1, expr2,  ET_RSHIFTFILL);
		expr = new VsAssign((VsVar *)expr1, expr3);
		break;
	case ET_ANDEQ:
		expr1 = Expression(pCoder);
		TRACEF("%s",(LPCSTR)assignmentOp[9]);
		//m_outBuff += assignmentOp[9];
		expr2 = Expression(pCoder);
		expr3 = new VsArith(expr1, expr2, ET_AND);
		expr = new VsAssign((VsVar *)expr1, expr3);
		break;
	case ET_XOREQ:
		expr1 = Expression(pCoder);
		TRACEF("%s",(LPCSTR)assignmentOp[10]);
		expr2 = Expression(pCoder);
		expr3 = new VsArith(expr1, expr2, ET_XOR);
		expr = new VsAssign((VsVar *)expr1, expr3);
		break;
	case ET_OREQ:
		expr1 = Expression(pCoder);
		TRACEF("%s",(LPCSTR)assignmentOp[11]);
		//m_outBuff += assignmentOp[11];
		expr2 = Expression(pCoder);
		expr3 = new VsArith(expr1, expr2, ET_OR);
		expr = new VsAssign((VsVar *)expr1, expr3);
		break;
    case ET_BOOLEAN: {
		bool boolValue = GetBoolean(pCoder);
        expr = new VsConstant(boolValue);
        }
        break;
	default:
		Error("Error: illegal expression type");
	}

	return expr;
}

VsExpr *
DecScript::NewObject(BIFSDecoder *pCoder)
{
	VsExpr *expr = NULL;
	VsExprList *exprlist;

	//m_outBuff += "new ";
	TRACEF("new ",0);
    LPCSTR str = GetIdentifier(pCoder);
	//m_outBuff += "(";
	TRACEF("(",0);
	exprlist = Params(pCoder);
	//m_outBuff += ")";
	TRACEF(") ",0);

	//Error("The new method has not been implemented yet");
	expr = new VsNew(str, *exprlist);

	return expr;
}

VsExpr * DecScript::ArrayDeref(BIFSDecoder *pCoder)
{
	VsExpr *expr1, *expr2;
	expr1 = Expression(pCoder);
	//m_outBuff += "[";
    TRACEF("[",0);
	expr2 = CompoundExpression(pCoder);
	//m_outBuff += "]";
	TRACEF("] ",0);

	return new VsArrayVar(expr1, expr2);
}

VsExpr *
DecScript::FunctionCall(BIFSDecoder *pCoder)
{
	VsExpr *expr1, *expr = NULL;
	VsExprList *expr2;
	LPCSTR str;
	str = GetIdentifier(pCoder);
	expr1 = new VsConstant(str);
	TRACEF(" (",0);
	//m_outBuff += " (";
	expr2 = Params(pCoder);
	TRACEF(")",0);
	//m_outBuff += ")";

	VsFunctionDef *funcDef = findFunction(str);
	if(funcDef != NULL)
	{
		expr = new VsFunctionCall(funcDef, *expr2);
	}
	else if (VsFunctionCall::isAnUnimplementedFunction(str))
	{
		char msg[256];

		sprintf(msg, "Built-in function \'%s\' is not implemented yet", str);
		Error(msg);
	}
	else
	{
		expr = new VsFunctionCall(NULL, *expr2);
		Unimplist += (VsFunctionCall *)expr;
		UnimpNames += str;
	}

	return expr;
}


VsExpr *
DecScript::ObjectMemberAccess(BIFSDecoder *pCoder)
{
	VsExpr *expr1, *expr = NULL;
	LPCSTR str;

	expr1 = Expression(pCoder, true);
	TRACEF(".",0);
//	m_outBuff += ".";
    str = GetIdentifier(pCoder);
//	m_outBuff += str;
	expr = new VsMemberAccess(expr1, str); // NODEFIELD
	return expr;
}


VsExpr *
DecScript::ObjectMethodCall(BIFSDecoder *pCoder)
{
	VsExpr *expr1, *expr = NULL;
	VsExprList *exprlist;
	LPCSTR str;

	expr1 = Expression(pCoder, true);

	//m_outBuff += ".";
	str = GetIdentifier(pCoder);
	TRACEF(".",0);
    //m_outBuff += str;
	//m_outBuff += "(";
	TRACEF("(",0);
	exprlist = Params(pCoder);
	TRACEF(")",0);
	//m_outBuff += ")";

	if(expr1 == NULL)
		expr = new VsMethodCall(expr1, str, *exprlist);
	else
	{
		char msg[256];
		sprintf(msg, "Object method call \'%s\' is not implemented yet", str);
		Error(msg);
	}

	return expr;
}

VsExprList * DecScript::Params(BIFSDecoder *pCoder)
{
	int val;
	VsExprList *exprlist = new VsExprList;
	VsExpr *expr;

	val = pCoder -> ParseInt(1,"hasParam");
	while (val) {
		expr = Expression(pCoder);
		*exprlist += expr;

		val = pCoder -> ParseInt(1,"hasParam");
		if(val) {
			TRACEF(", ",0);
			//m_outBuff += ", ";
		}
	}

	return exprlist;
}

LPCSTR DecScript::GetIdentifier(BIFSDecoder *pCoder)
{
	ZString ret;

	if (pCoder -> ParseInt(1, "received")) {
		int nbits, index, length;
		nbits = 0;
		length = pScript->pScriptNode->identifierLength() - 1;
		while (length > 0) {
			length >>= 1;
			nbits ++;
		}
		index = pCoder->ParseInt(nbits,"identifierCode");
		ret = pScript->pScriptNode->identifierAt(index);
	}else{
		pCoder->ParseString (ret);
		pScript->pScriptNode->addIdentifier(ret);
	}
	TRACEF("%s",(LPCSTR)ret);
	//m_outBuff += ret;
	return strdup((LPCSTR)ret);
}

LPCSTR DecScript::GetNumber(BIFSDecoder *pCoder)
{
	int val;
	char *ret;
	ZString str;

	ret = (char *) malloc(40);
	if (pCoder -> ParseInt(1,"isInteger")) { // integer
		int nbits;
		nbits = pCoder -> ParseInt(5,"numbits");
		val = pCoder -> ParseInt(nbits,"value");
		sprintf(ret, "%d", val);
		str = ret;
	} else { // real number
		val = pCoder -> ParseInt(4,"floatChar");
		while ( val!= 15) {
			if (val>=0 && val<=9) {
				str += (char) (val + '0');
			} else if (val==10) {
				str += '.';
			} else if (val==11) {
				str += 'E';
			} else if (val==12){
				str += '-';
			} else
			{
				fprintf(stderr, "Error: %d is not a valid real number character code\n", val);
			}
			val = pCoder -> ParseInt(4,"floatChar");
		}
		strcpy(ret, (LPCSTR) str);
	}
	TRACEF("%s",(LPCSTR)ret);
	//m_outBuff += ret;
	return strdup((LPCSTR)ret);
}

LPCSTR DecScript::GetString(BIFSDecoder *pCoder)
{
	ZString string;
	LPCSTR str;
	pCoder->ParseString(string);
	TRACEF("%s",(LPCSTR)string);
	str = _strdup((LPCSTR)string);
	return strdup((LPCSTR) str);
}

bool
DecScript::GetBoolean(BIFSDecoder *pCoder)
{
	if (pCoder->ParseInt(1,"value") == 1)
	{
		TRACEF ("true",0);
		return true;
	}
	else
	{
		TRACEF ("false",0);
		return false;
	}
}

void
DecScript::addGlobalVar(LPCSTR name, VsVar *var)
{
    globalNames += name;
    globalVars += var;

    var->ref();
}

VsVar *
DecScript::getGlobalVar(LPCSTR name)
{
    // Look through globals:
    for (int i = 0; i < globalVars.GetLength(); i++) {
		if(!strcmp(globalNames[i], name))
			return globalVars[i];
    }

    return NULL;
}

VsFunctionDef *
DecScript::beginFunction(LPCSTR name)
{
    VsFunctionDef *result = new VsFunctionDef(name, this);

    // ??? Warn if two functions with same name ???
    // Call findFunction to test if the current function name is
    // already used.    --Dong
    if(findFunction(name) != NULL) {
      fprintf(stderr, "Error: The function: \"%s\" is already defined\n",name);
      exit(1);
    //  return NULL;
    }

    functions += result;
    return result;
}

VsFunctionDef *
DecScript::findFunction(LPCSTR name)
{
    for (int i = 0; i < functions.GetLength(); i++) {
	VsFunctionDef *f = (VsFunctionDef *)functions[i];
	if (strcmp(f->getName(), name) == 0)
	    return f;
    }
    return NULL;
}

void
DecScript::CheckUnimp()
{
	for(int i = 0; i < Unimplist.GetLength(); i++)
	{
		VsFunctionDef *f = findFunction(UnimpNames[i]);

		if(f == NULL)
		{
			char msg[256];
			sprintf(msg, "Function: %s is not defined", UnimpNames[i]);
			Error(msg);
		}
		else
			Unimplist[i]->SetFuncDef(f);
	}

	UnimpNames.Clear();
}

void
DecScript::Error(LPCSTR errmsg)
{
	fprintf(stderr, "Script node: %s\n", errmsg);
}








