// This software module was originally developed by Wenlong Dong(Rockwell
// Science Center) and Wei Wu(Rockwell Science Center) in the course of
// development of the MPEG-4 Systems standard
// (ISO/IEC 14496-1).  This software module is an implementation of a part
// of one or more MPEG-4 Systems tools as specified by the MPEG-4 Systems
// standard.  ISO/IEC gives users of the MPEG-4 Systems standard free
// license to this software module or modifications thereof for use in
// hardware or software products claiming conformance to the MPEG-4 Systems
// standard.  Those intending to use this software module in hardware or
// software products are advised that its use may infringe existing
// patents.  The original developer of this software module and his/her
// company, the subsequent editors and their companies, and ISO/IEC have no
// liability for use of this software module or modifications thereof in an
// implementation.  Copyright is not released for non MPEG-4 Systems
// conforming products.  The original developer retains full right to use
// the code for his/her own purpose, assign or donate the code to a third
// party and to inhibit third parties from using the code for non MPEG-4
// Systems conforming products.  This copyright notice must be included in
// all copies or derivative works.
//
// For more information or to receive an up to date version of this module,
// you may call Wenlong Dong at wdong@usc.edu or Wei Wu at
// weiwu@jupiter.risc.rockwell.com.
//
// Copyright 1998.

/*-------------------------------------------------------------------------
Project Im1Player
INTERNATIONAL ORGANIZATION FOR STANDARDIZATION
ISO/IEC JTC1/SC29/WG11

SUBSYSTEM:	SFScript Node
FILE:       VScript.cpp
AUTHOR:     Wenlong Dong (Rockwell Science Center)
			Wei Wu       (Rockwell Science Center)
		    Yuval Fisher (Rockwell Science Center)
OVERVIEW:   Implementation of the SFScript class.
--------------------------------------------------------------------*/
//$Log: VScript.cpp,v $
//Revision 1.1.1.1.2.1  2000/03/10 15:59:34  Alex Cotarmanac'h
//deuxime envoi
//
//Modified 2001/05/08 Boris Felts boris.felts@polytechnique.org
//

#include "defs.h"
#include "SFScript.h"
#include "VSEvents.h"
#include "NodeCode.h"
#ifdef BIFSENC
#include "Token.h"
#include "BifsEnc.h"
#include "CParser.h"
#else
#include "BifsDec.h"
#include "Route.h"
#include "Root.h"
#include "DecScript.h"
#include "VsFields.h"
#endif
// #include "ScriptInterface.h"  //BF

SFScript::SFScript(ScriptImp *node)
{
	pScriptNode = node;
	pDecScript = new DecScript(this);
}

SFScript::~SFScript()
{
	delete (pDecScript);
}

#ifdef BIFSENC

SCRIPT_EVENT_TYPE SFScript::Scan_eType()
{
//AC
	switch (nextToken)
	{
	case TID_FIELD:
		return SCRIPT_FIELD;
		break;
	case TID_EVENTIN:
		return SCRIPT_EVENTIN;
		break;
	case TID_EVENTOUT:
		return SCRIPT_EVENTOUT;
		break;
	default:
		nextToken.Error("Unrecognized event type");
		printf("[Script] Error bad fieldtype \n");
		return SCRIPT_UNDEFINED;
	}
//end AC
/*
	const char *str = nextToken.GetName();
	if(!strcmp(str, "eventIn"))
		return eEVENTIN;
	else if(!strcmp(str, "eventOut"))
		return eEVENTOUT;
	else if(!strcmp(str, "field"))
		return eFIELD;
	else {
		nextToken.Error("Unrecognized event type");
		return eUNKNOWN_TYPE;
	}
*/
}

/*-----------------------------------------------------------------
Encode the SFScript Node. Left problems:
(1) How to deal with initialize and shutdown ???
(2) A field of type SFNode and MFNode is not implemented ???
(3) In script, the "new" method is not implemented ???
-----------------------------------------------------------------*/
BOOL SFScript::Encode(BIFSEncoder *pCoder)
{
//AC 20000304
	// we define this array to contain references to events in the order they're encoded.
	// as a consequence, their id is going to be the "natural reading" id
	// this makes decoding and debugging easier
	// formerly, ids where given in the following way : eventIns, eventOuts, fields.
	// I checked in the spec and this does not seem to be excluded.
	ZArray<VSNodeField *> eventArray;
//end AC
	if(nextToken != TID_LEFT_BRACE)
	{
		nextToken.Error("Left brace { is expected!");
		return FALSE;
	}

	nextToken.Scan();
	while(nextToken != TID_RIGHT_BRACE)
	{
		if(!stricmp(nextToken.GetName(),"url"))
		{
			nextToken.Scan();
			if(nextToken != TID_STRING)
				nextToken.Error("Not a url string");

			ZString string = nextToken.GetString();
			int pos;
			char *str = GetHeader(string, pos);
			if(!stricmp(str,"javascript:"))
			{
				EncUrl = TRUE;
				string.Cut(0,pos);
			}

			delete [] str;
			url += string;
			nextToken.Scan();
		}
		else if(!stricmp(nextToken.GetName(), "directOutput"))
		{
			nextToken.Scan();
			if(nextToken == TID_TRUE)
				directOutput = TRUE;
			else if(nextToken == TID_FALSE)
				directOutput = FALSE;
			else
				nextToken.Error("Unrecognized boolean value");
		}
		else if(!stricmp(nextToken.GetName(), "mustEvaluate"))
		{
			nextToken.Scan();
			if(nextToken == TID_TRUE)
				mustEvaluate = TRUE;
			else if(nextToken == TID_FALSE)
				mustEvaluate = FALSE;
			else
				nextToken.Error("Unrecognized boolean value");
		}
		else
		{
			pCoder -> EncodeInt(1, 1, "list description");

			EVENT_TYPE eType = Scan_eType();
			VSNodeField *event = NULL;
			switch(eType)
			{
			case eEVENTIN:
				event = new EventIn;
				m_fields += event;
				eventArray +=event;//AC
				NumIn ++;
				break;
			case eEVENTOUT:
				event = new EventOut;
				m_fields += event;
				eventArray +=event;//AC
				NumOut ++;
				break;
			case eFIELD:
				event = new Field;
				m_fields += event;
				eventArray +=event;//AC
				NumField ++;
				break;
			default:
				return FALSE;
			}

			nextToken.Scan();
			event -> Encode(pCoder);
		}
	}

	pCoder -> EncodeInt(0, 1, "endFlag");
	pCoder -> EncodeInt((Bool) directOutput, 1, "directOutput");
	pCoder -> EncodeInt((Bool) mustEvaluate, 1, "mustEvaluate");
	pCoder -> EncodeInt (EncUrl, 1, "Encode URL string");

	//AC 200003
	// since url is also a NumIn and NumOut (exposedField)
	NumIn++;
	NumOut++;
	// there's going to be a bug if we change url !!!!!!!
	// create a ScriptInterface.
	printf("[Nom :: %s %s]\n",getDefName(),GetName());
	ScriptInterface * mon_interface= new ScriptInterface(getDefName());
	mon_interface->StoreScriptDefinition(eventArray);
	nextToken.AddPredefinedScript(mon_interface);
	//AC end

	if(!EncUrl)
		url.Encode(pCoder);
	else
	{
		int i;
		pCParser = new CParser((LPCSTR)url [0]);
		for(i = 0; i < eventArray.GetLength(); i++)
			pCParser -> InsertExID (eventArray[i] -> GetName());
/*		//AC 20000304 : modified just above.
		for(i = 0; i < NumIn; i++)
			pCParser -> InsertExID (m_fields[i] -> GetName());
		for(i = 0; i < NumOut; i++)
			pCParser -> InsertExID (m_fields[i] -> GetName());
		for(i = 0; i < NumField; i++)
			pCParser -> InsertExID (m_fields[i] -> GetName());
*/
		pCParser -> SetSNode(this);	// NODEFIELD
		pCParser -> Encode(pCoder);
		delete pCParser;
	}

	// Save this SFScript object in BifsEncoder for Route encoding
	if(GetID() >= 0)
		pCoder -> ScriptList += this;

	nextToken.Scan();
	return TRUE;
}

void SFScript::EncodeRoute(BIFSEncoder *pCoder, BOOL bIn)
{
	pCoder -> EncodeInt(GetID(), 10, "nodeID"); // mp : conformance
	if(nextToken != TID_PERIOD)
		nextToken.Error("period . expected");
	else
		nextToken.Scan();

	if(nextToken == TID_NAME)
	{
		for(int i=0; i < pCoder->ScriptList.GetLength(); i++)
		{

			SFScript *pScript = pCoder -> ScriptList[i];
			if(GetID() == pScript -> GetID())
			{
				int j;
				if(bIn)
				{
					for(j = 0; j < pScript -> NumIn; j++)
					{
						if(!strcmp(nextToken.GetName(),
							pScript ->m_fields[j]->GetName()))
						{
							pCoder -> EncodeInt(j, BitFieldWidth(pScript -> NumIn),
								"inFieldReference");
							break;
						}
					}

					if(j >= pScript -> NumIn)
						nextToken.Error("No eventIn with this name");
				}
				else
				{
					for(j = 0; j < pScript -> NumOut; j++)
					{
						if(!strcmp(nextToken.GetName(),
							pScript ->m_fields[j]->GetName()))
						{
							pCoder -> EncodeInt(j, BitFieldWidth(pScript -> NumOut),
								"outFieldReference");
							break;
						}
					}

					if(j >= pScript -> NumOut)
						nextToken.Error("No eventOut with this name");
				} // if...else...
			} // if
		} // for
	}
	else
		nextToken.Error("Unexpected symbol");

	nextToken.Scan();
}

char *SFScript::GetHeader(ZString string, int &pos)
{
	int pos1 = 0;
	char *header;

	while((string[pos1]==' ')||(string[pos1]=='\n')||(string[pos1]=='\t'))
		pos1++;

	pos = pos1;
	for(int i=0; i<15; i++)
	{
		char c = string[pos];
		pos++;
		if(!isalpha(c))
			break;
	}

	header = _strdup((LPCSTR)(string.Substr(pos1, pos-pos1)));
	return header;
}

#else

BOOL ScriptImp::ParseScriptField(BIFSDecoder *pCoder){
	NodeField *pField;

	SCRIPT_EVENT_TYPE eType = (SCRIPT_EVENT_TYPE)pCoder -> ParseInt(2, "eventType");
	FIELD_TYPE fieldType = (FIELD_TYPE)pCoder -> ParseInt(6,"fieldType");
	ZString fieldName;
	pCoder->ParseString (fieldName);
	pField = CreateInterfaceField (pCoder,eType,fieldType,fieldName);
	TRACEF("%s ",(LPCSTR)fieldName);
	addIdentifier(fieldName) ;  // save the name in the list of identifiers

	if (eType == SCRIPT_FIELD)
	{
		if (pCoder->ParseInt(1,"hasInitialValue"))
		{
			pField->Parse(pCoder,this);
		}
	}
	//m_fields += pField;
	TRACENEWLINE;
	return TRUE;
}

/*--------------------------------------------------------------------
Parse the SFScript Node.
--------------------------------------------------------------------*/
BOOL SFScript::Parse(BIFSDecoder *pCoder)
{
	//TRACENEWLINE; // mp
	//TRACEINDENT (1); // mp

	if (pCoder -> ParseInt(1,"isListDescription")){
		while(!pCoder -> ParseInt(1, "endFlag")){
			pScriptNode->ParseScriptField(pCoder);
		}
	}
	else{
		int fieldBits = pCoder -> ParseInt(4,"fieldBits");

		int numFields = pCoder -> ParseInt(fieldBits,"numFields");
		for (int i=0;i<numFields;i++){
			pScriptNode->ParseScriptField(pCoder);
		}
	}

    TRACEF("url [", 0); // mp this has to be the url field as it is the only MFScript field
    TRACENEWLINE; // mp
	TRACEINDENT (1); // mp

	if(pCoder -> ParseInt(1,"reserved") != 1){
		printf("Error: reserved should be 1\n");
	}

    pScriptNode->initOutFields(pDecScript);

  	pDecScript -> Decode(pCoder);

    pScriptNode->initInFields(pDecScript);

	TRACEINDENT (-1);
	TRACENEWLINE;
	return TRUE;
}

#endif