// This software module was originally developed by Wenlong Dong(Rockwell
// Science Center) and Wei Wu(Rockwell Science Center) in the course of
// development of the MPEG-4 Systems standard
// (ISO/IEC 14496-1).  This software module is an implementation of a part
// of one or more MPEG-4 Systems tools as specified by the MPEG-4 Systems
// standard.  ISO/IEC gives users of the MPEG-4 Systems standard free
// license to this software module or modifications thereof for use in
// hardware or software products claiming conformance to the MPEG-4 Systems
// standard.  Those intending to use this software module in hardware or
// software products are advised that its use may infringe existing
// patents.  The original developer of this software module and his/her
// company, the subsequent editors and their companies, and ISO/IEC have no
// liability for use of this software module or modifications thereof in an
// implementation.  Copyright is not released for non MPEG-4 Systems
// conforming products.  The original developer retains full right to use
// the code for his/her own purpose, assign or donate the code to a third
// party and to inhibit third parties from using the code for non MPEG-4
// Systems conforming products.  This copyright notice must be included in
// all copies or derivative works.
//
// For more information or to receive an up to date version of this module,
// you may call Wenlong Dong at wdong@usc.edu or Wei Wu at
// weiwu@jupiter.risc.rockwell.com.
//
// Copyright 1998.

/*-------------------------------------------------------------------------
Project Im1Player
INTERNATIONAL ORGANIZATION FOR STANDARDIZATION
ISO/IEC JTC1/SC29/WG11

SUBSYSTEM:	ScriptImp Node
FILE:       VScript.cpp
AUTHOR:     Wenlong Dong (Rockwell Science Center)
			Wei Wu       (Rockwell Science Center)
		    Yuval Fisher (Rockwell Science Center)
OVERVIEW:   Implementation of the ScriptImp class.
--------------------------------------------------------------------*/
//$Log: VScript.cpp,v $
//Revision 1.1.1.1.2.1  2000/03/10 15:59:34  Alex Cotarmanac'h
//
// Modified 2001/06/30 Marc Patoureaux marc@envivio.com

#include "defs.h"
#include "ScriptImp.h" 
#include "VSEvents.h"
#include "NodeCode.h"
#ifdef BIFSENC
#include "Token.h"
#include "BifsEnc.h"
#include "CParser.h"

#else
#include "BifsDec.h"
#include "Route.h"
#include "Root.h"
#include "DecScript.h"
#include "VSFields.h"
#endif
//#include "ScriptInterface.h"  //BF

//******************************************************
//TODO : The new method is not implemented yet ( it is decoded though )
//******************************************************

ScriptImp::ScriptImp()
{
	BifsFieldTable table;
	// initialize url field
	table.m_offset = (WORD) ((size_t) ((NodeField *) & ((ScriptImp *)0) -> url) - (size_t) (MediaObject *)((ScriptImp *)0));
	table.m_function = NULL;
	table.m_bChildren = FALSE;
	table.m_defID = 0;
	table.m_inID = 0;
	table.m_outID = 0;
	table.m_dynID = -1;
	table.m_nQuantizationType = 0;
	table.m_quantParams = -1;
	table.m_AnimationType = 0;
	table.m_nDimension = 0;
	table.m_fMin [0] = -FLT_MAX;
	table.m_fMax [0]= FLT_MAX;
	table.m_szName = "url";
	m_fieldTable += table;
	// initialize directOutput field
	table.m_offset = (WORD) ((size_t) ((NodeField *) & ((ScriptImp *)0) -> directOutput) - (size_t) (MediaObject *)((ScriptImp *)0));
	table.m_function = NULL;
	table.m_bChildren = FALSE;
	table.m_defID = 1;
	table.m_inID = -1;
	table.m_outID = -1;
	table.m_dynID = -1;
	table.m_nQuantizationType = 0;
	table.m_quantParams = -1;
	table.m_AnimationType = 0;
	table.m_nDimension = 0;
	table.m_fMin [0] = -FLT_MAX;
	table.m_fMax [0]= FLT_MAX;
	table.m_szName = "directOutput";
	m_fieldTable += table;
	// initialize mustEvaluate field
	table.m_offset = (WORD) ((size_t) ((NodeField *) & ((ScriptImp *)0) -> mustEvaluate) - (size_t) (MediaObject *)((ScriptImp *)0));
	table.m_function = NULL;
	table.m_bChildren = FALSE;
	table.m_defID = 2;
	table.m_inID = -1;
	table.m_outID = -1;
	table.m_dynID = -1;
	table.m_nQuantizationType = 0;
	table.m_quantParams = -1;
	table.m_AnimationType = 0;
	table.m_nDimension = 0;
	table.m_fMin [0] = -FLT_MAX;
	table.m_fMax [0]= FLT_MAX;
	table.m_szName = "mustEvaluate";
	m_fieldTable += table;

	// ajust bifsBounds
	m_bifsBounds.m_nTotalAll = 3;
	m_bifsBounds.m_nTotalDefs = 3;
	m_bifsBounds.m_nTotalIns = 1;
	m_bifsBounds.m_nTotalOuts = 1;
	m_bifsBounds.m_nTotalDyns = 0;

	m_bifsBounds.CalculateBounds ();

	// ajust offset
	m_nChildrenOffset = -1;
	m_fieldCount = 3;
}


ScriptImp::~ScriptImp()
{
}

/*-------------------------------------------------------------
Create a script field. This involves instantiation of the proper NodeField object,
and adding proper values to the field table for this proto node
-------------------------------------------------------------*/
NodeField *ScriptImp :: CreateInterfaceField (
	SysCoder *pCoder,
	int eventType,
	FIELD_TYPE fieldType,
	LPCSTR szName
)
{
	/*-------------------------------------------------------------
	Add an entry to the BifsFieldTable
	-------------------------------------------------------------*/
	int nOffset = m_fieldTable.GetLength ();
	BifsFieldTable rEntry ;
	rEntry.m_AnimationType = -1;
	rEntry.m_bChildren = FALSE;
	rEntry.m_dynID = -1;
	rEntry.m_defID = rEntry.m_inID = rEntry.m_outID = -1;
	rEntry.m_fMax [0] = FLT_MAX;
	rEntry.m_fMin [0] = -FLT_MAX;
	rEntry.m_function = NULL;
	rEntry.m_nDimension = 0;
	rEntry.m_nQuantizationType = -1;
	rEntry.m_offset = -nOffset;
	rEntry.m_quantParams = -1;
	
	rEntry.m_szName = strdup(szName);

	/*-------------------------------------------------------------
	Increment counter in the BifsBounds structure
	-------------------------------------------------------------*/
	pCoder;
	m_bifsBounds.m_nTotalAll++;

    	/*-------------------------------------------------------------
	Instantiate the proper field object and add it to the list
	-------------------------------------------------------------*/
	NodeField *pField = NULL;
	switch (fieldType) {
		case SFBOOL:
			pField = new SFBool;
			TRACEF ("SFBool ", 0);
			break;
		case SFFLOAT:
			pField = new SF<float>;
			TRACEF ("SFFloat ", 0);
			break;
		case SFTIME:
			pField = new SF<Time>;
			TRACEF ("SFTime ", 0);
			break;
		case SFINT32:
			pField = new SF<int>;
			TRACEF ("SFInt32 ", 0);
			break;
		case SFSTRING:
			pField = new SFString;
			TRACEF ("SFString ", 0);
			break;
		case SFVEC3F:
			pField = new SF<Vec3f>;
			TRACEF ("SFVec3f ", 0);
			break;
		case SFVEC2F:
			pField = new SF<Vec2f>;
			TRACEF ("SFVec2f ", 0);
			break;
		case SFCOLOR:
			pField = new SF<Color3f>;
			TRACEF ("SFColor ", 0);
			break;
		case SFROTATION:
			pField = new SF<Rotation>;
			TRACEF ("SFRotation ", 0);
			break;
		case SFIMAGE:
			pField = new SF<Image>;
			TRACEF ("SFImage ", 0);
			break;
		case SFNODE:
			pField = new SFWorldNode;
			TRACEF ("SFNode ", 0);
			break;
		case MFBOOL:
			pField = new MF<Bool>;
			TRACEF ("MFBool ", 0);
			break;
		case MFFLOAT:
			pField = new MF<float>;
			TRACEF ("MFFloat ", 0);
			break;
		case MFTIME:
			pField = new MF<Time>;
			TRACEF ("MFTime ", 0);
			break;
		case MFINT32:
			pField = new MF<int>;
			TRACEF ("MFInt32 ", 0);
			break;
		case MFSTRING:
			pField = new MF<ZString>;
			TRACEF ("MFString ", 0);
			break;
		case MFVEC3F:
			pField = new MF<Vec3f>;
			TRACEF ("MFVec3f ", 0);
			break;
		case MFVEC2F:
			pField = new MF<Vec2f>;
			TRACEF ("MFVec2f ", 0);
			break;
		case MFCOLOR:
			pField = new MF<Color3f>;
			TRACEF ("MFColor ", 0);
			break;
		case MFROTATION:
			pField = new MF<Rotation>;
			TRACEF ("MFRotation ", 0);
			break;
		case MFIMAGE:
			pField = new MF<Image>;
			TRACEF ("MFImage ", 0);
			break;
		case MFNODE:
			pField = new MFWorldNode;
			TRACE ("MFNode ", 0);
			break;
		default:
			return NULL;
	}

	switch (eventType) {
    case SCRIPT_FIELD: {
		rEntry.m_defID = m_bifsBounds.m_nTotalDefs++;
		TRACEF ("field ", 0);
# ifndef BIFSENC
        ScriptField * a_field = new ScriptField(fieldType, pField, szName);
        a_field->SetSNode(this);
        FieldList += a_field;
# endif
        break; }
    case SCRIPT_EVENTIN: {
		rEntry.m_inID = m_bifsBounds.m_nTotalIns++;
		TRACEF ("eventIn ", 0);
# ifndef BIFSENC
        ScriptEventIn * a_field = new ScriptEventIn(fieldType, pField, szName);
        a_field->SetSNode(this);
        EventInList += a_field;
# endif
        break; }
    case SCRIPT_EVENTOUT: {
		rEntry.m_outID = m_bifsBounds.m_nTotalOuts++;
		TRACEF ("eventOut ", 0);
# ifndef BIFSENC
        ScriptEventOut * a_field = new ScriptEventOut(fieldType, pField, szName);
        a_field->SetSNode(this);
        EventOutList += a_field;
# endif
        break; }
	}
	m_fieldTable += rEntry;
	m_bifsBounds.CalculateBounds();
	m_fieldCount ++;

    m_fields += pField; // mp add
	return pField;
}

# ifndef BIFSENC
void ScriptImp::RouteEvent (NodeField *pField, int nFieldIndex)
{
	int i;

    MediaObject::GetField (nFieldIndex) -> Copy (*pField, this, false);
	MediaObject::GetField (nFieldIndex) -> OnFieldUpdate();
    EventInList[nFieldIndex-3]->OnFieldUpdate();

    for (i = GetFieldCount (); --i >= 0;) {
        if (m_fieldTable [i].m_outID >= 0) { // if it's an outnode
			MediaObject::GetField(i) -> OnFieldUpdate(true);
		}
    }
}

void ScriptImp::initOutFields(DecScript * pDecScript) {
    int i;
    for ( i = 0; i < EventOutList.GetLength(); i++) { 
        EventOutList[i]->InitField(pDecScript);
    }
    for ( i = 0; i < FieldList.GetLength(); i++) { 
        FieldList[i]->InitField(pDecScript);
    }
}

void ScriptImp::initInFields(DecScript * pDecScript) {
    for (int i = 0; i < EventInList.GetLength(); i++) {
        EventInList[i]->InitField(pDecScript);
    }
}

/*void ScriptImp::OnUpdate() {
    int i = 0;
    fprintf(stdout,"test");
}*/

# endif
