// This software module was originally developed by Wenlong Dong(Rockwell
// Science Center) and Wei Wu(Rockwell Science Center) in the course of
// development of the MPEG-4 Systems standard
// (ISO/IEC 14496-1).  This software module is an implementation of a part
// of one or more MPEG-4 Systems tools as specified by the MPEG-4 Systems
// standard.  ISO/IEC gives users of the MPEG-4 Systems standard free
// license to this software module or modifications thereof for use in
// hardware or software products claiming conformance to the MPEG-4 Systems
// standard.  Those intending to use this software module in hardware or
// software products are advised that its use may infringe existing
// patents.  The original developer of this software module and his/her
// company, the subsequent editors and their companies, and ISO/IEC have no
// liability for use of this software module or modifications thereof in an
// implementation.  Copyright is not released for non MPEG-4 Systems
// conforming products.  The original developer retains full right to use
// the code for his/her own purpose, assign or donate the code to a third
// party and to inhibit third parties from using the code for non MPEG-4
// Systems conforming products.  This copyright notice must be included in
// all copies or derivative works.
//
// For more information or to receive an up to date version of this module,
// you may call Wenlong Dong at wdong@usc.edu or Wei Wu at
// weiwu@jupiter.risc.rockwell.com.
//
// Copyright 1998.

/*-------------------------------------------------------------------------
Project Im1Player
INTERNATIONAL ORGANIZATION FOR STANDARDIZATION
ISO/IEC JTC1/SC29/WG11

SUBSYSTEM:	Script Node
FILE:       VSEvnets.cpp
AUTHOR:     Wenlong Dong (Rockwell Science Center)
			Wei Wu		 (Rockwell Science Center)
		    Yuval Fisher (Rockwell Science Center)
OVERVIEW:   Implementations for the Derived classes: Field, EventIn,
			EventOut of the class VSNodeField.
--------------------------------------------------------------------*/
#include "defs.h"
#include "Quantize.h"
#include <float.h>
#include "VSEvents.h"
#ifdef BIFSENC
#include "BifsEnc.h"
#include "Token.h"
#else
#include "BifsDec.h"
#include "DecScript.h"
#include "vsfields.h"
#include "vsfunction.h"
#include "SFScript.h"	// NODEFIELD
#include "Root.h"	// NODEFIELD
#include "NodeFld.h"	// NODEFIELD
#include "Executiv.h"	// ROK1999
#include "PresBase.h"	// ROK1999
#endif


ScriptField::ScriptField (FIELD_TYPE fieldType, NodeField * reffield, const char * szName)
{
	m_scriptEventType = SCRIPT_FIELD;
    m_fieldType = fieldType;
    m_NodeField = reffield;
    m_name = szName;
}

ScriptEventIn::ScriptEventIn (FIELD_TYPE fieldType, NodeField * reffield, const char * szName)
{
	m_scriptEventType = SCRIPT_EVENTIN;
    m_fieldType = fieldType;
    m_NodeField = reffield;
    m_name = szName;

#ifndef BIFSENC

	timeStamp = new ScriptField(SFTIME, new SF<Time>,"t");
#endif
}



/*#ifndef BIFSENC
BOOL ScriptEventIn::Parse(BIFSDecoder *pCoder)
{
	TRACEF("eventIn ",0);
	return VSNodeField::Parse(pCoder);
}
#endif*/


ScriptEventOut::ScriptEventOut(FIELD_TYPE fieldType, NodeField * reffield, const char * szName)
{
	m_scriptEventType = SCRIPT_EVENTOUT;
    m_fieldType = fieldType;
    m_NodeField = reffield;
    m_name = szName;

#ifndef BIFSENC
	beenSet = FALSE;
#endif
}

ScriptEventOut::~ScriptEventOut()
{
#ifndef BIFSENC
	 if(pValueCopy != NULL)
		 delete pValueCopy;
#endif
}

/*#ifndef BIFSENC
BOOL ScriptEventOut::Parse(BIFSDecoder *pCoder)
{
	TRACEF("eventOut ",0);
	return VSNodeField::Parse(pCoder);
}
#endif*/

#ifdef BIFSENC
/*BOOL ScriptField::Encode(BIFSEncoder *pCoder)
{
	TRACEF("field ",0);
	VSNodeField::Encode(pCoder);
	Encode_Default(pCoder);
	return TRUE;
}

void ScriptField::Encode_Default(BIFSEncoder *pCoder)
{
	if((nextToken == TID_INTEGER) ||(nextToken == TID_NUMBER)
		|| (nextToken == TID_STRING) || (nextToken == TID_LEFT_BRACKET)
		|| (nextToken == TID_TRUE) || (nextToken == TID_FALSE)
		|| (nextToken == TID_USE))	// NODEFIELD
		hasDefault = TRUE;

	pCoder -> EncodeInt(hasDefault, 1, "hasDefault");
	if(hasDefault)
	{
		if (nextToken == TID_USE) { // NODEFIELD
			nextToken.Scan();
			if (nextToken == TID_NAME) {
				nNodeID = nextToken.GetDefinedName();
				if (nNodeID >= 0)
					pCoder->EncodeInt(nNodeID, 10, "nodeID");
				else
					nextToken.Error("Name not found");
				nNodeID = nextToken.GetNodeID(nNodeID);
				nextToken.Scan();
			} else {
				nNodeID = -1;
				nextToken.Error("Unexpected symbol");
				nextToken.Scan();
			}
//			pValue = new SFGenericNode();
		} else {
			float fmin = FLT_MIN, fmax = FLT_MAX;
			Quantizer quant(&fmin, &fmax, 1, 0);
			pValue = CreateField(m_fieldType);
			pValue -> Encode(pCoder, &quant);
		}
	}
}*/

#else
/*BOOL ScriptField::Parse(BIFSDecoder *pCoder)
{
	VSNodeField::Parse(pCoder);
	hasDefault = pCoder -> ParseInt(1,"hasInitialValue");

	if(hasDefault)
	{
		pValue->Parse(pCoder,pSNode);
	/*
		if (m_fieldType == SFNODE) { // NODEFIELD
			int nID = pCoder->ParseInt(10);
			SFGenericNode *pField = (SFGenericNode *) pValue;
			pField -> SetObject (pSNode->GetRoot()->FindNode(nID), pSNode);
		} else {
			float fmin = FLT_MIN, fmax = FLT_MAX;
			Quantizer quant(&fmin, &fmax, 1, 0);
			pValue -> Parse(pCoder, NULL, &quant);
		}
*/
	/*}

	return TRUE;
}*/

void ScriptEventIn::InitField(DecScript *p)
{
	VsExprList arguments;

//	parent = p;
	function = p -> findFunction(m_name);

    if(function == NULL)
		return;
	
	eventArg = VsField::create(m_fieldType, VsField::RO);
	
	timeArg = VsField::create(SFTIME, VsField::RO);

	arguments += eventArg;
	arguments += timeArg;

	f_call = new VsFunctionCall(function, arguments);
}

void ScriptEventOut::OnFieldUpdate(BOOL Onflag)
{
	if(!Onflag)
		beenSet = FALSE;
	else
	{
		/*
		if(beenSet)
			pValue -> Copy (*pValueCopy, NULL);
			*/
	}
}

void ScriptEventIn::OnFieldUpdate(BOOL)
{
    *(SF<Time> *) (timeStamp->m_NodeField) = GetSNode()->GetExecutive()->GetPresenter()->GetTime();
	eventArg -> setField(this);
	timeArg -> setField(timeStamp);
	f_call->evaluate(NULL);
}

#endif