// This software module was originally developed by Wenlong Dong(Rockwell
// Science Center) and Wei Wu(Rockwell Science Center) in the course of
// development of the MPEG-4 Systems standard
// (ISO/IEC 14496-1).  This software module is an implementation of a part
// of one or more MPEG-4 Systems tools as specified by the MPEG-4 Systems
// standard.  ISO/IEC gives users of the MPEG-4 Systems standard free
// license to this software module or modifications thereof for use in
// hardware or software products claiming conformance to the MPEG-4 Systems
// standard.  Those intending to use this software module in hardware or
// software products are advised that its use may infringe existing
// patents.  The original developer of this software module and his/her
// company, the subsequent editors and their companies, and ISO/IEC have no
// liability for use of this software module or modifications thereof in an
// implementation.  Copyright is not released for non MPEG-4 Systems
// conforming products.  The original developer retains full right to use
// the code for his/her own purpose, assign or donate the code to a third
// party and to inhibit third parties from using the code for non MPEG-4
// Systems conforming products.  This copyright notice must be included in
// all copies or derivative works.
//
// For more information or to receive an up to date version of this module,
// you may call Wenlong Dong at wdong@usc.edu or Wei Wu at
// weiwu@jupiter.risc.rockwell.com.
//
// Copyright 1998.

/*-------------------------------------------------------------------------
Project Im1Player
INTERNATIONAL ORGANIZATION FOR STANDARDIZATION
ISO/IEC JTC1/SC29/WG11

SUBSYSTEM:	Script Node
FILE:       VSNodeField.h
AUTHOR:     Wenlong Dong (Rockwell Science Center)
			Wei Wu       (Rockwell Science Center)
		    Yuval Fisher (Rockwell Science Center)
OVERVIEW:   Implementations for the class VSNodeField.
--------------------------------------------------------------------*/
//$Log: VSNodeField.cpp,v $
//Revision 1.1.1.1.2.1  2000/03/10 15:59:34  Alex Cotarmanac'h
//deuxime envoi
//
#include "defs.h"
#include "VSNodeField.h"
#ifdef BIFSENC
#include "Token.h"
#include "BifsEnc.h"
#else // End BIFSENC
#include "BifsDec.h"
#include "NodeFld.h"
#include "MediaObj.h"
#include "DecScript.h"
#include "vsfields.h"
#endif


//#include "ScriptOutput.h"


/*VSNodeField::VSNodeField(FIELD_TYPE type)
{
	m_scriptEventType = UNDEFINED;
	m_fieldType = type;

#ifndef BIFSENC
	beenSet = TRUE;

	if(type != UNKNOWN_TYPE)
	{
		pValue = CreateField(type);
	}
#endif
}*/


VSNodeField::~VSNodeField()
{
//	if(pValue)
//		delete pValue;
}

#ifdef BIFSENC
//AC 200003
/*
		Encodes the Field + sets the isSF field
*/
BOOL VSNodeField::Encode(BIFSEncoder *pBifs)
{
	char * name;
	pBifs -> EncodeInt(m_scriptEventType, 2, "Event type");
	//AC 20000304 :same as in EncProto
	int fieldType=UNKNOWN_TYPE;
	switch(nextToken)
	{
	case TID_SFBOOL:		fieldType= SFBOOL ; break;
	case TID_SFFLOAT:		fieldType= SFFLOAT ; break;
	case TID_SFTIME:		fieldType= SFTIME ; break;
	case TID_SFINT32:		fieldType= SFINT32 ; break;
	case TID_SFSTRING:		fieldType= SFSTRING ; break;
	case TID_SFVEC3F:		fieldType= SFVEC3F ; break;
	case TID_SFVEC2F:		fieldType= SFVEC2F ; break;
	case TID_SFCOLOR:		fieldType= SFCOLOR ; break;
	case TID_SFROTATION:	fieldType= SFROTATION ; break;
	case TID_SFIMAGE:		fieldType= SFIMAGE ; break;
	case TID_SFNODE:		fieldType= SFNODE ; break;
	case TID_MFBOOL:		fieldType= MFBOOL ; break;
	case TID_MFFLOAT:		fieldType= MFFLOAT ; break;
	case TID_MFTIME:		fieldType= MFTIME ; break;
	case TID_MFINT32:		fieldType= MFINT32 ; break;
	case TID_MFSTRING:		fieldType= MFSTRING ; break;
	case TID_MFVEC3F:		fieldType= MFVEC3F ; break;
	case TID_MFVEC2F:		fieldType= MFVEC2F ; break;
	case TID_MFCOLOR:		fieldType= MFCOLOR ; break;
	case TID_MFROTATION:	fieldType= MFROTATION ; break;
	case TID_MFIMAGE:		fieldType= MFIMAGE ; break;
	default:
		nextToken.Error ("Unexpected symbol");
		printf("[VSNodeField::Encode] Error :Unexpected symbol\n");
		nextToken.Scan ();
		break;
	}
	m_fieldType = (FIELD_TYPE)fieldType;

/*	if(nextToken != TID_NAME)
	{
		//AC
		printf("[VSNodeField::Encode] Error :Not an appropriate field type\n");
		nextToken.Error("Not an appropriate field type");
	}
	else
	{
		//for(int type = SFWORLDNODE; type <= MFVEC3F; type++)
		for(int type = SFNODE; type <= MFVEC3F; type++) // NODEFIELD
		{
			if(!stricmp(nextToken.GetName(), field_reserved[type]))
			{
				m_fieldType = (FIELD_TYPE)type;
				break;
			}
		}
	}
*/
	if(m_fieldType == UNKNOWN_TYPE)
		nextToken.Error("Unrecognized Field type");
	pBifs -> EncodeInt (m_fieldType, 6, "Field type");

	nextToken.Scan();
	if(nextToken == TID_NAME)
		name = _strdup(nextToken.GetName());
	else
	{
		nextToken.Error("Not a valid name");
		printf("[VSNodeField::Encode] Error :Not a valid name\n");
	}

	nextToken.Scan();

	return TRUE;
}

#else
void VSNodeField::SetID(int nIndex)
{
	nNodeID = nIndex;
	char var[20];

//	sprintf(var, "var%d", Id);

	m_name = _strdup(var);
}


/*BOOL VSNodeField::Parse(BIFSDecoder *pBifs, MediaObject *, Quantizer *)
{
	m_fieldType = (FIELD_TYPE)pBifs -> ParseInt(6,"fieldType");
	pBifs->ParseString (m_name);
	pSNode->addIdentifier(m_name) ;  // save the name in the list of identifiers
	pValue = CreateField(m_fieldType);
	return TRUE;
}*/



void VSNodeField::InitField(DecScript *p)
{
	//parent = p;
	VsField *Gbfield = VsField::create(m_fieldType, VsField::RW);
    if (!Gbfield) {
        ERROR("Invalid Field Type in script");
        return;
    }
	Gbfield -> setField(this);
	p -> addGlobalVar(m_name, Gbfield);

    /*if (m_fieldType == SFNODE) {
        MediaObject * a_node = ((SFWorldNode *)m_NodeField)->GetNode();
        ZArray<BifsFieldTable> * a_table = a_node->GetFieldTable();
        for (int i = 0; i < a_table.GetLength(); i++) {
            Gbfield = VsField::create(a_node->GetField(i)->GetType(), VsField::RW);
            if (!Gbfield)
                continue;
            Gbfield->setField(a_node->GetField(i));
            p->addGlobalVar(a_table[i].m_szName, Gbfield);
        }
    }*/
}

#endif

NodeField *VSNodeField::CreateField(FIELD_TYPE type)
{
	switch(type)
	{
	case SFNODE:
//		return NULL; // Not implement right now ??
		return new SFGenericNode(); // NODEFIELD
	case SFBOOL:
        return new SF<Bool>;
	case SFTIME:
		return new SF<Time>;
    case SFCOLOR:
		return new SF<Color3f>;
	case MFCOLOR:
		return new MF<Color3f>;
	case SFFLOAT:
		return new SF<float>;
	case MFFLOAT:
		return new MF<float>;
    case SFINT32:
		return new SF<int>;
	case MFINT32:
		return new MF<int>;
    case SFROTATION:
		return new SF<Rotation>;
	case MFROTATION:
		return new MF<Rotation>;
	case SFSTRING:
		return new SFString;
	case MFSTRING:
		return new MF<ZString>;
    case SFVEC2F:
		return new SF<Vec2f>;
	case MFVEC2F:
		return new MF<Vec2f>;
	case SFVEC3F:
		return new SF<Vec3f>;
	case MFVEC3F:
		return new MF<Vec3f>;
	default:
		return NULL;
	}
}

#ifdef BIFSENC
LPCSTR VSNodeField::field_reserved [] =
{
	"SFNode", // NODEFIELD
	"SFBool",
	"SFTime",
    "SFColor",
	"MFColor",
	"SFFloat",
	"MFFloat",
    "SFInt32",
	"MFInt32",
    "SFRotation",
	"MFRotation",
	"SFString",
	"MFString",
    "SFVec2f",
	"MFVec2f",
	"SFVec3f",
	"MFVec3f"
};
#endif


