// This software module was originally developed by Wenlong Dong(Rockwell
// Science Center) and Wei Wu(Rockwell Science Center) in the course of
// development of the MPEG-4 Systems standard
// (ISO/IEC 14496-1).  This software module is an implementation of a part
// of one or more MPEG-4 Systems tools as specified by the MPEG-4 Systems
// standard.  ISO/IEC gives users of the MPEG-4 Systems standard free
// license to this software module or modifications thereof for use in
// hardware or software products claiming conformance to the MPEG-4 Systems
// standard.  Those intending to use this software module in hardware or
// software products are advised that its use may infringe existing
// patents.  The original developer of this software module and his/her
// company, the subsequent editors and their companies, and ISO/IEC have no
// liability for use of this software module or modifications thereof in an
// implementation.  Copyright is not released for non MPEG-4 Systems
// conforming products.  The original developer retains full right to use
// the code for his/her own purpose, assign or donate the code to a third
// party and to inhibit third parties from using the code for non MPEG-4
// Systems conforming products.  This copyright notice must be included in
// all copies or derivative works.
//
// For more information or to receive an up to date version of this module,
// you may call Wenlong Dong at wdong@usc.edu or Wei Wu at
// weiwu@jupiter.risc.rockwell.com.
//
// Copyright 1998.

/*-------------------------------------------------------------------------
Project Im1Player
INTERNATIONAL ORGANIZATION FOR STANDARDIZATION
ISO/IEC JTC1/SC29/WG11

SUBSYSTEM:	ScriptImp Node
FILE:       VSNodeField.h
AUTHOR:     Wenlong Dong (Rockwell Science Center)
			Wei Wu       (Rockwell Science Center)
		    Yuval Fisher (Rockwell Science Center)
OVERVIEW:   Interface for the class VSNodeField.
--------------------------------------------------------------------*/
//$Log: VSNodeField.h,v $
//Revision 1.1.1.1.2.1  2000/03/10 15:59:34  Alex Cotarmanac'h
//deuxime envoi
//
//Modified 2001/05/08 Boris Felts boris.felts@polytechnique.org
//

#if !defined(_VSNodeField_H__)
#define _VSNodeField_H__

#include "ScriptDefs.h"
#include "defs.h"
#include "NodeFld.h"
#include "ScriptImp.h"


#ifdef BIFSENC
class BIFSEncoder;
#else
class BIFSDecoder;
class DecScript;
#endif

class ScriptImp; // NODEFIELD //BF Script

enum SCRIPT_EVENT_TYPE  //BF : eventTypes for Scripts.
{
	SCRIPT_FIELD,         // 0b00
	SCRIPT_EVENTIN,       // 0b01
	SCRIPT_EVENTOUT,      // 0b10
    SCRIPT_UNDEFINED
};

class ScriptOutput;
//end AC
class VSNodeField
{
	//AC 20000303
	friend class ScriptOutput;
	//end AC
public:

	/*VSNodeField(FIELD_TYPE fieldType = UNKNOWN_TYPE,
		        SCRIPT_EVENT_TYPE  scriptEventType = SCRIPT_UNDEFINED) :
	m_scriptEventType (scriptEventType),
	pValue (NULL)
	{}*/
    VSNodeField() {}

    NodeField * getNodeField() { return m_NodeField; }

	virtual ~VSNodeField();
	NodeField *CreateField(FIELD_TYPE type);

	SCRIPT_EVENT_TYPE GetEventType() { return m_scriptEventType;}
	void SetEventType(SCRIPT_EVENT_TYPE type) {m_scriptEventType = type;}

    NodeField * m_NodeField;
    FIELD_TYPE GetType() { return m_fieldType;}
    FIELD_TYPE m_fieldType;

#ifdef BIFSENC
public:

	LPCSTR GetName() { return m_name; }
	int GetID() { return nNodeID; } // NODEFIELD
	virtual BOOL Encode(BIFSEncoder *pBifs) = 0;
	static LPCSTR field_reserved [];

#else
public:

	//virtual BOOL Parse (BIFSDecoder *pBifs, MediaObject *pNode = NULL, Quantizer *pQuant = NULL);
	// CC 010629
	//virtual BOOL Parse (BIFSDecoder *pBifs, UIStreamDecoder *pUi, MediaObject *pNode=NULL, Quantizer *pQuant = NULL) {pQuant; pNode; pBifs; pUi;  return false;};

	/*virtual void Copy (NodeField &nodeField, MediaObject *, bool bIsFirst)
	{
		pValue = ((VSNodeField&) nodeField).pValue;
		OnFieldUpdate (bIsFirst);
	}*/

	/*virtual NodeField *Clone (NodeField *pField, MediaObject *)
	{
		if (pField == NULL)
			pField = new VSNodeField;
		((VSNodeField*) pField) ->pValue = pValue;
		return pField;
	}*/

	//virtual void Copy (NodeField &nodeField, MediaObject *pParent,bool bIsFirst = true);
	void SetID(int nIndex);
	virtual void InitField(DecScript *p);
	//virtual operator NodeField *() { return pValue; }
	//virtual NodeField * getNF() { return pValue; } //AC debug only
	void setHasBeenSet(BOOL val) { beenSet = val; }
	BOOL hasBeenSet() { return beenSet; }

	virtual void OnFieldUpdate(BOOL = FALSE){}
	void SetSNode(ScriptImp *node) { pSNode = node; } // NODEFIELD//  // BF Script
	ScriptImp *GetSNode() { return pSNode; }	// ROK1999  // BF Script
#endif

protected:

	int nNodeID;
	BOOL beenSet;
	//DecScript *parent;
	ScriptImp *pSNode; // NODEFIELD  // BF Script
	ZString m_name;
	//NodeField *pValue;
	//int nNodeID; // NODEFIELD
	SCRIPT_EVENT_TYPE m_scriptEventType;
		// the synonym used in IM1
};


#endif // _VSNodeField_H__


