// This software module was originally developed by Wei Wu(Rockwell Science
// Center) and Wenlong Dong(Rockwell Science Center) in the course of 
// development of the MPEG-4 Systems standard
// (ISO/IEC 14496-1).  This software module is an implementation of a part
// of one or more MPEG-4 Systems tools as specified by the MPEG-4 Systems
// standard.  ISO/IEC gives users of the MPEG-4 Systems standard free
// license to this software module or modifications thereof for use in
// hardware or software products claiming conformance to the MPEG-4 Systems
// standard.  Those intending to use this software module in hardware or
// software products are advised that its use may infringe existing
// patents.  The original developer of this software module and his/her
// company, the subsequent editors and their companies, and ISO/IEC have no
// liability for use of this software module or modifications thereof in an
// implementation.  Copyright is not released for non MPEG-4 Systems
// conforming products.  The original developer retains full right to use
// the code for his/her own purpose, assign or donate the code to a third
// party and to inhibit third parties from using the code for non MPEG-4
// Systems conforming products.  This copyright notice must be included in
// all copies or derivative works.
//
// For more information or to receive an up to date version of this module,
// you may call Wei Wu at weiwu@jupiter.risc.rockwell.com or Wenlong Dong 
// at wdong@usc.edu.
//
// Copyright 1998.

/*-------------------------------------------------------------------------
Project Im1Player
INTERNATIONAL ORGANIZATION FOR STANDARDIZATION
ISO/IEC JTC1/SC29/WG11

SUBSYSTEM:	Script Node
FILE: 	    DecScript.cpp
AUTHOR:     Wei Wu       (Rockwell Science Center)
		    Wenlong Dong (Rockwell Science Center)
		    Yuval Fisher (Rockwell Science Center)
OVERVIEW:   Implementation of the DecScript class
----------------------------------------------------------------------*/
//$Log: DecScript.cpp,v $
//Revision 1.1.1.1.2.1  2000/03/10 15:59:34  Alex Cotarmanac'h
//deuxime envoi
//
#include "defs.h"
#include "DecScript.h"
#include "..\..\Core\BifsDec.h"
#include "..\..\Core\NodeFld.h"
#include "vsexpr.h"
#include "vsfunction.h"
#include "vsstatement.h"
#include "VsSwitch.h"
//AC200005
const char *DecScript::exprDesc[] ={
"ET_CURVED_EXPR",
"ET_NEGATIVE",
"ET_NOT",
"ET_ONESCOMP",
"ET_INCREMENT",
"ET_DECREMENT",
"ET_POST_INCREMENT",
"ET_POST_DECREMENT",
"ET_CONDTEST",
"ET_STRING",
"ET_NUMBER",
"ET_IDENTIFIER",
"ET_FUNCTION_CALL",
"ET_NEW",
"ET_OBJECT_MEMBER_ACCESS",
"ET_OBJECT_METHOD_CALL",
"ET_ARRAY_DEREFERENCE",
"ET_ASSIGN",
"ET_PLUSEQ",
"ET_MINUSEQ",
"ET_MULTIPLYEQ",
"ET_DIVIDEEQ",
"ET_MODEQ",
"ET_ANDEQ",
"ET_OREQ",
"ET_XOREQ",
"ET_LSHIFTEQ",
"ET_RSHIFTEQ",
"ET_RSHIFTFILLEQ",
"ET_EQ",
"ET_NE",
"ET_LT",
"ET_LE",
"ET_GT",
"ET_GE",
"ET_PLUS",
"ET_MINUS",
"ET_MULTIPLY",
"ET_DIVIDE",
"ET_MOD",
"ET_LAND",
"ET_LOR",
"ET_AND",
"ET_OR",
"ET_XOR",
"ET_LSHIFT",
"ET_RSHIFT",
"ET_RSHIFTFILL",
"NUMBER_OF_EXPR_TYPE"
};
#define BINOP_MINVAL ET_EQ
//char * forced_enter="forced_enter";
//#define ASSIGNFORCED_ENTER(X) new VsAssign((VsVar *)VsFunctionDef::findIdentifier("forced_enter"),new VsConstant(X))
//#define LOGICOPFORCED_ENTER(X,OP) new VsLogicOp((VsVar *)VsFunctionDef::findIdentifier("forced_enter"),new VsConstant(X), OP)
char* DecScript::binaryOp[] = 
{
	
	"==", "!=","<", "<=",
	">", ">=", "+", "-",
	"*", "/", "%",
	"&&", "||",
	"&", "|", "^",
	"<<", ">>", ">>>"


//	"*", "/", "%", "+", "-", "<<", ">>", ">>>", "<", "<=",
//	">", ">=", "==", "!=", "&", "^", "|", "&&", "||"
	//AC end
};

char* DecScript::assignmentOp[] =
{
	"=", "+=", "-=", "*=", "/=", "%=", "<<=", ">>=", ">>>=",
	"&=", "^=", "|="
};

char* DecScript::unaryOp[] =
{
	"!", "~", "-", "++", "--"
};

DecScript::DecScript()
{
  numIdentifier=0;
  flag=0;
  IdCnt=0;
}

DecScript::~DecScript()
{
}

void 
DecScript::InsertExID(int num)
{
	for(int i=0; i<num; i++) {
		char ret[10];
		sprintf(ret, "var%d", i);
		savedStrBuffer.insert(ret);
		IdCnt ++;
	}
}

LPCSTR
DecScript::GetTextString()
{
	return outBuff;
}

void
DecScript::Decode(BIFSDecoder *pCoder)
{
	VsStatementList *StList;
	while (pCoder -> ParseInt(1)) {
		outBuff += "function ";
		LPCSTR str = GetIdentifier(pCoder);
		VsFunctionDef *func = beginFunction(str);
		outBuff += str;
		outBuff += "(";
		Arguments(pCoder, func); 
		outBuff += ")\n";
		StList = StatementBlock(pCoder);
		func -> endDefinition(StList);
		functions += func;
	}

	outBuff += "\0";
//AC
	TRACEF("url \"javascript:", 0);
	TRACENEWLINE;
	TRACEINDENT(1);
	TRACENEWLINE;
	TRACEF("%s",(LPCSTR)outBuff); 
	TRACENEWLINE;
	TRACEINDENT(-1);
	TRACEF("\"", 0);
	TRACENEWLINE;
//end AC

	CheckUnimp();
}

void 
DecScript::Arguments(BIFSDecoder *pCoder, VsFunctionDef *f)
{
	int val = pCoder -> ParseInt(1);
	ASSERT (f != NULL);
	f;	// avoid compiler warning
	while(val) {
		LPCSTR str = GetIdentifier(pCoder);
		f -> addArg(str);

		outBuff += str;
		val = pCoder -> ParseInt(1);
		if (val) {
			outBuff += ", ";
		}
	}
}

VsStatementList *
DecScript::StatementBlock(BIFSDecoder *pCoder)
{
	VsStatementList *stlist = new VsStatementList;
	VsStatement *stmnt;

	if(pCoder -> ParseInt(1)) {
		outBuff += "{\n";
		while (pCoder -> ParseInt(1)) {
			stmnt = Statement(pCoder);
			*stlist += stmnt;
		}
		outBuff += "}\n";
	} 
	else 
	{
		stmnt = Statement(pCoder);
		*stlist += stmnt;
	}

	return stlist;
}

VsStatement *
DecScript::Statement(BIFSDecoder *pCoder)
{
	int val;
	VsStatement *stmnt;

	val = pCoder -> ParseInt(NUMBITS_STATEMENT);
	switch (val) {
	case ST_IF:
		stmnt = ifStatement(pCoder);
		break;
	case ST_FOR:
		stmnt = forStatement(pCoder);
		break;
	case ST_WHILE:
		stmnt = whileStatement(pCoder);
		break;
	case ST_RETURN:
		stmnt = returnStatement(pCoder);
		outBuff += ";\n";
		break;
	case ST_BREAK:
		stmnt = breakStatement();
		outBuff += ";\n";
		break;
	case ST_CONTINUE:
		stmnt = continueStatement();
		outBuff += ";\n";
		break;
	case ST_COMPOUND_EXPR:
		stmnt = CompoundExpression(pCoder);
		outBuff += ";\n";
		break;
	case ST_SWITCH:
		stmnt = switchStatement(pCoder);

		break;
	default:
		stmnt = NULL;
		fprintf(stderr, "Error: Unrecognized statement type %d\n", val);
	}

	return stmnt;
}

VsStatement *
DecScript::ifStatement(BIFSDecoder *pCoder)
{
	int val;
	VsExpr *stmnt_expr;
	VsStatementList *stmnt_else, *stmnt_blk;

	outBuff += "if (";
	stmnt_expr = CompoundExpression(pCoder);
	outBuff += ")\n";
	stmnt_blk = StatementBlock(pCoder);
	
	val = pCoder -> ParseInt(1);
	if (val) {
		outBuff += "else\n";
		stmnt_else = StatementBlock(pCoder);
	}
	else
		stmnt_else = NULL;

	return new VsIf(stmnt_expr, stmnt_blk, stmnt_else);

}
// SEMANTIQUE LITIGIEUSE ...
VsStatement *
DecScript::switchStatement(BIFSDecoder *pCoder)
{
	outBuff += "switch (";
	VsExpr * switched_expr = CompoundExpression(pCoder);
	// NO NUMBITS
	VsSwitch * switch_stmt = new VsSwitch(switched_expr);
	outBuff += ")\n{\n";
	do {
		outBuff += "case ";
		VsExpr * caselabel = CompoundExpression(pCoder);
		outBuff += " :\n";
		switch_stmt->addCase(caselabel,StatementBlock(pCoder));

	}
	while (pCoder->ParseInt(1,"hasMoreCases")==1);

	if (pCoder->ParseInt(1,"hasDefault") == 1)
	{
		outBuff += "default :\n";
		switch_stmt->addCase(NULL,StatementBlock(pCoder));

	}
	outBuff += "}\n";


	return switch_stmt;
}//end of switchStatement

VsStatement *
DecScript::forStatement(BIFSDecoder *pCoder)
{
	VsExpr *expr1, *expr2, *expr3;
	VsStatementList *stmntlist;

	outBuff += "for (";
	expr1 = OptionalExpression(pCoder);
	outBuff += "; ";
	expr2 = OptionalExpression(pCoder);
	outBuff += "; ";
	expr3 = OptionalExpression(pCoder);
	outBuff += ")\n";
	stmntlist = StatementBlock(pCoder);

	return new VsFor(expr1, expr2, expr3, stmntlist);
}

VsStatement *
DecScript::whileStatement(BIFSDecoder *pCoder)
{
	VsExpr *expr;
	VsStatementList *stmntlist;
	outBuff += "while (";
	expr = CompoundExpression(pCoder);
	outBuff += ")\n";
	stmntlist = StatementBlock(pCoder);

	return new VsWhile(expr, stmntlist);
}

VsStatement *
DecScript::returnStatement(BIFSDecoder *pCoder)
{
	VsExpr *expr = NULL;

	outBuff += "return ";

	if (pCoder -> ParseInt(1)) {
		expr = CompoundExpression(pCoder);
	}

	return new VsReturn(expr);
}

VsStatement *
DecScript::breakStatement()
{
	outBuff += "break";

	return new VsBreak();
}

VsStatement *
DecScript::continueStatement()
{
	outBuff += "continue";

	return new VsContinue();
}

VsExpr *
DecScript::CompoundExpression(BIFSDecoder *pCoder)
{
	VsExpr *expr1, *expr2;

	expr1 = Expression(pCoder);
	if(!pCoder -> ParseInt(1))
		return expr1;
	else
	{
		outBuff += ", ";
		expr2 = CompoundExpression(pCoder);
		return new VsComma(expr1, expr2);
	}
}

VsExpr *
DecScript::OptionalExpression(BIFSDecoder *pCoder)
{
	if (pCoder -> ParseInt(1)) {
		return CompoundExpression(pCoder);
	}

	return NULL;
}

VsExpr *
DecScript::Expression(BIFSDecoder *pCoder, BOOL n_flag)
{
	VsExpr *expr1, *expr2, *expr3, *expr = NULL;
	int val = pCoder -> ParseInt(NUMBITS_EXPR_TYPE);
	
	LPCSTR str;

	switch(val)
	{
	case ET_IDENTIFIER:
		if(n_flag)
		{
			if(pCoder->ParseInt(1))
				outBuff += "Math"; // In this case, expr = NULL
			else
			{
				str = GetIdentifier(pCoder);
				expr = VsFunctionDef::findIdentifier(str);
				outBuff += str;
			}
		}
		else
		{
			str = GetIdentifier(pCoder);	
			expr = VsFunctionDef::findIdentifier(str);
			outBuff += str;
		}
		break;
	case ET_STRING:
		outBuff += "'";
		str = GetString(pCoder);
		expr = new VsConstant(str);
		outBuff += str;
        outBuff += "'";// ???
		break;
	case ET_NUMBER:
		str = GetNumber(pCoder);
		expr = new VsConstant(atof(str)); 
		outBuff += str;
		break;
	case ET_CURVED_EXPR:
		outBuff += "(";
		expr = CompoundExpression(pCoder);
		outBuff += ")";
		break;
	case ET_OBJECT_METHOD_CALL:
		expr = ObjectMethodCall(pCoder);
		break;
	case ET_OBJECT_MEMBER_ACCESS:
		expr = ObjectMemberAccess(pCoder);
		break;
	case ET_FUNCTION_CALL:
		expr = FunctionCall(pCoder);
		break;
	case ET_ARRAY_DEREFERENCE:
		expr = ArrayDeref(pCoder);
		break;
	case ET_NEW:
		expr = NewObject(pCoder);
		break;
	case ET_NOT:
		str = unaryOp[0];
		outBuff += str;
		expr1 = Expression(pCoder);
		expr = new VsULogicOp(expr1, ET_NOT);
		break;
	case ET_ONESCOMP:
		str = unaryOp[1];
		outBuff += str;
		expr1 = Expression(pCoder);
		expr = new VsUArithOp(expr1, ET_ONESCOMP);
		break;
	case ET_NEGATIVE:
		str = unaryOp[2];
		outBuff += str;
		expr1 = Expression(pCoder);
		expr = new VsUArithOp(expr1, ET_MINUS);
		break;
	case ET_INCREMENT:
		str = unaryOp[3];
		outBuff += str;
		expr1 = Expression(pCoder);
		expr = new VsIncDec(expr1, TRUE, TRUE);
		break;
	case ET_DECREMENT:
		str = unaryOp[4];
		outBuff += str;
		expr1 = Expression(pCoder);
		expr = new VsIncDec(expr1, FALSE, TRUE);
		break;
	case ET_POST_INCREMENT:
		expr1 = Expression(pCoder);
		outBuff += "++";
		expr = new VsIncDec(expr1, TRUE, FALSE);
		break;
	case ET_POST_DECREMENT:
		expr1 = Expression(pCoder);
		outBuff += "--";
		expr = new VsIncDec(expr1, FALSE, FALSE);
		break;
	case ET_MULTIPLY:
	case ET_DIVIDE:
	case ET_MOD:
	case ET_PLUS:
	case ET_MINUS:
	case ET_LSHIFT:
	case ET_RSHIFT:
	case ET_RSHIFTFILL:
	case ET_AND:
	case ET_XOR:
	case ET_OR:
		expr1 = Expression(pCoder);
		fprintf(stderr,"Parsed : %s -> %s \n",exprDesc[val],binaryOp[val - BINOP_MINVAL]);
		outBuff += binaryOp[val - BINOP_MINVAL];//AC
		expr2 = Expression(pCoder);
		expr = new VsArith(expr1, expr2, val);
		break;
	case ET_LT:
	case ET_LE:
	case ET_GT:
	case ET_GE:
	case ET_EQ:
	case ET_NE:
	case ET_LAND:
	case ET_LOR:
		expr1 = Expression(pCoder);
		fprintf(stderr,"Parsed : %s -> %s \n",exprDesc[val],binaryOp[val - BINOP_MINVAL]);
		outBuff += binaryOp[val - BINOP_MINVAL];//AC
		expr2 = Expression(pCoder);
		expr = new VsLogicOp(expr1, expr2, val);
		break;	
	case ET_CONDTEST:
		expr1 = Expression(pCoder);
		outBuff += " ? ";
		expr2 = Expression(pCoder);
		outBuff += " : ";
		expr3 = Expression(pCoder);
		expr = new VsCondExpr(expr1, expr2, expr3);
		break;
	case ET_ASSIGN:
		expr1 = Expression(pCoder);
		outBuff += assignmentOp[0];
		expr2 = Expression(pCoder);
		expr = new VsAssign((VsVar *)expr1, expr2);
		break;
	case ET_PLUSEQ:
		expr1 = Expression(pCoder);
		outBuff += assignmentOp[1];
		expr2 = Expression(pCoder);
		expr3 = new VsArith(expr1, expr2, ET_PLUS);
		expr = new VsAssign((VsVar *)expr1, expr3);
		break;
	case ET_MINUSEQ:
		expr1 = Expression(pCoder);
		outBuff += assignmentOp[2];
		expr2 = Expression(pCoder);
		expr3 = new VsArith(expr1, expr2, ET_MINUS);
		expr = new VsAssign((VsVar *)expr1, expr3);
		break;
	case ET_MULTIPLYEQ:
		expr1 = Expression(pCoder);
		outBuff += assignmentOp[3];
		expr2 = Expression(pCoder);
		expr3 = new VsArith(expr1, expr2, ET_MULTIPLY);
		expr = new VsAssign((VsVar *)expr1, expr3);
		break;
	case ET_DIVIDEEQ:
		expr1 = Expression(pCoder);
		outBuff += assignmentOp[4];
		expr2 = Expression(pCoder);
		expr3 = new VsArith(expr1, expr2, ET_DIVIDE);
		expr = new VsAssign((VsVar *)expr1, expr3);
		break;
	case ET_MODEQ:
		expr1 = Expression(pCoder);
		outBuff += assignmentOp[5];
		expr2 = Expression(pCoder);
		expr3 = new VsArith(expr1, expr2, ET_MOD);
		expr = new VsAssign((VsVar *)expr1, expr3);
		break;
	case ET_LSHIFTEQ:
		expr1 = Expression(pCoder);
		outBuff += assignmentOp[6];
		expr2 = Expression(pCoder);
		expr3 = new VsArith(expr1, expr2, ET_LSHIFT);
		expr = new VsAssign((VsVar *)expr1, expr3);
		break;
	case ET_RSHIFTEQ:
		expr1 = Expression(pCoder);
		outBuff += assignmentOp[7];
		expr2 = Expression(pCoder);
		expr3 = new VsArith(expr1, expr2, ET_RSHIFT);
		expr = new VsAssign((VsVar *)expr1, expr3);
		break;
	case ET_RSHIFTFILLEQ:
		expr1 = Expression(pCoder);
		outBuff += assignmentOp[8];
		expr2 = Expression(pCoder);
		expr3 = new VsArith(expr1, expr2,  ET_RSHIFTFILL);
		expr = new VsAssign((VsVar *)expr1, expr3);
		break;
	case ET_ANDEQ:
		expr1 = Expression(pCoder);
		outBuff += assignmentOp[9];
		expr2 = Expression(pCoder);
		expr3 = new VsArith(expr1, expr2, ET_AND);
		expr = new VsAssign((VsVar *)expr1, expr3);
		break;
	case ET_XOREQ:
		expr1 = Expression(pCoder);
		outBuff += assignmentOp[10];
		expr2 = Expression(pCoder);
		expr3 = new VsArith(expr1, expr2, ET_XOR);
		expr = new VsAssign((VsVar *)expr1, expr3);
		break;
	case ET_OREQ:
		expr1 = Expression(pCoder);
		outBuff += assignmentOp[11];
		expr2 = Expression(pCoder);
		expr3 = new VsArith(expr1, expr2, ET_OR);
		expr = new VsAssign((VsVar *)expr1, expr3);
		break;
	default:
		Error("Error: illegal expression type");
	}

	return expr;
}

VsExpr *
DecScript::NewObject(BIFSDecoder *pCoder)
{
	VsExpr *expr = NULL;
	VsExprList *exprlist;
	FIELD_TYPE fType;

	outBuff += "new ";
	fType = (FIELD_TYPE)pCoder -> ParseInt(6);
	outBuff += "Field"; // do not care the output
	outBuff += "(";
	exprlist = Params(pCoder);
	outBuff += ")";

	Error("The new method has not been implemented yet");
	expr = new VsNew(fType, *exprlist);

	return expr;
}

VsExpr *
DecScript::ArrayDeref(BIFSDecoder *pCoder)
{
	VsExpr *expr1, *expr2;
	expr1 = Expression(pCoder);
	outBuff += "[";
	expr2 = CompoundExpression(pCoder);
	outBuff += "]";

	return new VsArrayVar(expr1, expr2);
}

VsExpr *
DecScript::FunctionCall(BIFSDecoder *pCoder)
{
	VsExpr *expr1, *expr = NULL;
	VsExprList *expr2;
	LPCSTR str;
	str = GetIdentifier(pCoder);
	expr1 = new VsConstant(str);
	outBuff += str;
	outBuff += " (";
	expr2 = Params(pCoder);
	outBuff += ")";

	VsFunctionDef *funcDef = findFunction(str);
	if(funcDef != NULL) 
	{
		expr = new VsFunctionCall(funcDef, *expr2);
	}
	else if (VsFunctionCall::isAnUnimplementedFunction(str))
	{
		char msg[256];

		sprintf(msg, "Built-in function \'%s\' is not implemented yet", str);
		Error(msg); 
	}
	else
	{
		/*
		char msg[256];

		sprintf(msg, "Function \'%s\' must be defined before it can be used", str);
		Error(msg);
		*/
		expr = new VsFunctionCall(NULL, *expr2);
		Unimplist += (VsFunctionCall *)expr;
		UnimpNames += str; 
	}

	return expr;
}		

VsExpr *
DecScript::ObjectMemberAccess(BIFSDecoder *pCoder)
{
	VsExpr *expr1, *expr = NULL;
	LPCSTR str;

	expr1 = Expression(pCoder, TRUE); // TRUE for "Math" checking
	outBuff += ".";
	if(expr1==NULL) { // NODEFIELD
		str = GetString(pCoder);
	} else {
		str = GetNumber(pCoder);
	}
	outBuff += str;	
//	if(expr1 == NULL)
		expr = new VsMemberAccess(expr1, str); // NODEFIELD
//	else
//	{
//		char msg[256];

//		sprintf(msg, "Access to object member \'%s\' is not implemented yet", str);
//		Error(msg);
//	}

	return expr;
}

VsExpr *
DecScript::ObjectMethodCall(BIFSDecoder *pCoder)
{
	VsExpr *expr1, *expr = NULL;
	VsExprList *exprlist;
	LPCSTR str;

	expr1 = Expression(pCoder, TRUE); // TRUE for "Math" checking

	outBuff += ".";
	str = GetString(pCoder);
	outBuff += str;
	outBuff += "(";
	exprlist = Params(pCoder);
	outBuff += ")";

	if(expr1 == NULL)
		expr = new VsMethodCall(expr1, str, *exprlist);
	else 
	{
		char msg[256];

		sprintf(msg, "Object method call \'%s\' is not implemented yet", str);
		Error(msg);
	}

	return expr;
}

VsExprList *
DecScript::Params(BIFSDecoder *pCoder)
{
	int val;
	VsExprList *exprlist = new VsExprList;
	VsExpr *expr;

	val = pCoder -> ParseInt(1);
	while (val) {
		expr = Expression(pCoder);
		*exprlist += expr;

		val = pCoder -> ParseInt(1);
		if(val) {
			outBuff += ", ";		
		}
	}

	return exprlist;
}

LPCSTR
DecScript::GetIdentifier(BIFSDecoder *pCoder)
{
	int calNbits(int num);
	char *str, *ret;

	if (pCoder -> ParseInt(1)) {
		int nbits, index;

		if (flag) {
			nbits = calNbits(savedStrBuffer.getPos()-1);
			index = pCoder -> ParseInt(nbits);
			str = savedStrBuffer[index];
			ret = _strdup(str);
		}
		else {
			nbits = calNbits(IdCnt-1);
			index = pCoder -> ParseInt(nbits);
			ret = (char *) malloc(10);
			sprintf(ret, "var%d", index);
		}
		return ret;
	} else {
		if (flag) {
			ret = (char *)GetString(pCoder);
			savedStrBuffer.insert(ret);
		} else {
			ret = (char *) malloc(10);
			sprintf(ret, "var%d", IdCnt);
			IdCnt++;
		}
		return ret;
	}
}

LPCSTR
DecScript::GetNumber(BIFSDecoder *pCoder)
{
	int val;
	char *ret;

	ret = (char *) malloc(40);
	if (pCoder -> ParseInt(1)) { // integer
		int nbits;

		nbits = pCoder -> ParseInt(5);
		val = pCoder -> ParseInt(nbits);
		sprintf(ret, "%d", val);
	} else { // real number
		ZString str;
		str = "";
		val = pCoder -> ParseInt(4);
		while (val != 15) {
			if (val>=0 && val<=9) {
				str += (char) (val + '0');
			} else if (val==10) {
				str += '.';
			} else if (val==11) {
				str += 'E';
			} else {
				fprintf(stderr, "Error: %d is not a valid \
real number character code\n", val);
				exit (-1);
			}
			val = pCoder -> ParseInt(4);
		}
		strcpy(ret, (LPCSTR) str);
	}
	return ret;
}
// AC 20000304 : we change this so that it parses directly the string
//				--> we don't have any output.
LPCSTR
DecScript::GetString(BIFSDecoder *pCoder)
{
//	SFString string;
	ZString string;
	LPCSTR str;
	pCoder->ParseSFString(string);
//	string.Parse(pCoder, NULL);
	str = _strdup((LPCSTR)string);
	return str;
}
//AC end
void
DecScript::addGlobalVar(LPCSTR name, VsVar *var)
{
    globalNames += name;
    globalVars += var;

    var->ref();
}

VsVar *
DecScript::getGlobalVar(LPCSTR name)
{
    // Look through globals:
    for (int i = 0; i < globalVars.GetLength(); i++) {
		if(!strcmp(globalNames[i], name))
			return globalVars[i];
    }

    return NULL;
}

VsFunctionDef *
DecScript::beginFunction(LPCSTR name)
{
    VsFunctionDef *result = new VsFunctionDef(name, this);

    // ??? Warn if two functions with same name ???
    // Call findFunction to test if the current function name is 
    // already used.    --Dong
    if(findFunction(name) != NULL) {
      fprintf(stderr, "Error: The function: \"%s\" is already defined\n",name);
      exit(1);
    //  return NULL;
    }      

    functions += result;
    return result;
}

VsFunctionDef *
DecScript::findFunction(LPCSTR name)
{
    for (int i = 0; i < functions.GetLength(); i++) {
	VsFunctionDef *f = (VsFunctionDef *)functions[i];
	if (strcmp(f->getName(), name) == 0)
	    return f;
    }
    return NULL;
}

void 
DecScript::CheckUnimp()
{
	for(int i = 0; i < Unimplist.GetLength(); i++)
	{
		VsFunctionDef *f = findFunction(UnimpNames[i]);

		if(f == NULL)
		{
			char msg[256];
			sprintf(msg, "Function: %s is not defined", UnimpNames[i]);
			Error(msg);
		}
		else
			Unimplist[i]->SetFuncDef(f);
	}

	UnimpNames.Clear();
}

void
DecScript::Error(LPCSTR errmsg)
{
//    char *msg = (char *)malloc(strlen(errmsg)+50);

//    sprintf(msg, "Script node: %s\n",errmsg);
//    ::MessageBox((HWND)NULL, msg, "VrmlScript syntax error", MB_OK );

//    free(msg);
	fprintf(stderr, "Script node: %s\n", errmsg);
}








