// VsSwitch.cpp: implementation of the VsSwitch class.
//
//////////////////////////////////////////////////////////////////////
#include "defs.h"
#include "VsSwitch.h"

//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////

VsSwitch::VsSwitch()
{

}

VsSwitch::~VsSwitch()
{
	m_switched_expr->unref();
	int i;
	for (i=0; i<m_caseLabelArray.GetLength();i++)
	{
		m_caseLabelArray[i]->unref();
		m_caseLabelArray[i]=NULL;
	}
	for (i=0; i<m_caseBodyArray.GetLength();i++)
	{
		delete m_caseBodyArray[i];
		m_caseBodyArray[i]=NULL;
	}
}

VsStatement::Status	VsSwitch::execute(VsFunctionCall *f)
{
	//First :evaluate switched_expr.
	VsValue swe_val= m_switched_expr->execute(f);
	//Now find the corresponding label.
	for (int i=0; i<m_caseLabelArray.GetLength();i++)
	{
		if (m_caseLabelArray[i]==NULL) 
		{
			break;
		}
		VsValue lbl_val = m_caseLabelArray[i]->execute(f);
		if (areEqual(swe_val,lbl_val)) break;
	}
	if (i==m_caseLabelArray.GetLength())
	{
		fprintf(stderr,"[VsSwitch::execute] WARNING: didn't find matching label\n");	
		return VsStatement::NORMAL;
	}
	//execute casebodys starting from i.
	while (i< m_caseBodyArray.GetLength())
	{
		VsStatement::Status status = m_caseBodyArray[i]->execute(f);
		switch(status)
		{
		case VsStatement::BREAK: //get out of here
			i=m_caseBodyArray.GetLength();
			break;
		case VsStatement::CONTINUE:
		case VsStatement::RETURN:
			return status; // return from execute.
			break;
		case VsStatement::NORMAL:
			i++; // move to the next statement.
			break;
		default:
			fprintf(stderr,"[VsSwitch::execute] ERROR: should not be here\n");	
		}
	}
	return VsStatement::NORMAL;
}//end of execute.

VsSwitch::VsSwitch(VsExpr *switched_expr):
m_switched_expr(switched_expr)
{	
}

void VsSwitch::addCase(VsExpr *switch_against, VsStatementList *case_block)
{
	m_caseLabelArray += switch_against;
	m_caseBodyArray  += case_block;
}

bool VsSwitch::areEqual(VsValue v1, VsValue v2)
{
    if (v1.type == VsValue::vNUMBER && v2.type == VsValue::vNUMBER) 
		return (v1.number == v2.number);
	if (v1.type == VsValue::vSTRING || v2.type == VsValue::vSTRING) 
	{
		const char *s1 = NULL;
		const char *s2 = NULL;
		char scratch[26];
		if (v1.type == VsValue::vNUMBER) 
		{
			sprintf(scratch, "%d", v1.number);
			s1 = scratch; s2 = v2.string;
		} 
		else if (v2.type == VsValue::vNUMBER) 
		{
			sprintf(scratch, "%d", v2.number);
			s2 = scratch; s1 = v1.string;
		} 
		else 
		{
			fprintf(stderr,"[VsSwitch::areEqual] WARNING: didn't know how to promote to string...(boolean maybe)\n");
			return false;
		}
		int cmp = strcmp(s1, s2);
		return(cmp == 0);
    }
	fprintf(stderr,"[VsSwitch::areEqual] WARNING: didn't know how to compare(boolean maybe)\n");
    return false;
}
