/*
This software module was originally developed by Apple Computer, Inc.
in the course of development of MPEG-4. 
This software module is an implementation of a part of one or 
more MPEG-4 tools as specified by MPEG-4. 
ISO/IEC gives users of MPEG-4 free license to this
software module or modifications thereof for use in hardware 
or software products claiming conformance to MPEG-4.
Those intending to use this software module in hardware or software
products are advised that its use may infringe existing patents.
The original developer of this software module and his/her company,
the subsequent editors and their companies, and ISO/IEC have no
liability for use of this software module or modifications thereof
in an implementation.
Copyright is not released for non MPEG-4 conforming
products. Apple Computer, Inc. retains full right to use the code for its own
purpose, assign or donate the code to a third party and to
inhibit third parties from using the code for non 
MPEG-4 conforming products.
This copyright notice must be included in all copies or
derivative works. Copyright (c) 1999.
*/
#include "ISOMovies.h"

#ifndef INCLUDED_MP4ATOMS_H
#include "MP4Atoms.h"
#endif


ISOErr createMyMovie( char *filename );
ISOErr createQTMovie( char *filename );
ISOErr addMySamples( ISOTrack theTrack, ISOMedia theMedia );
ISOErr getNextAudioFrame( ISOHandle sampleH );

int main( int argc, char **argv )
{
	MP4Err err;

	err = createMyMovie( "myMovie.mj2" );
/*	err = createQTMovie( "myMovie.mov" );	*/
	fprintf( stderr, "createMyMovie returns %d\n", err );
	return err;
}

ISOErr createMyMovie( char *filename )
{
	ISOErr err;
	ISOMovie moov;
	ISOTrack trak;
	ISOMedia media;
    u64 mediaDuration;
	
	err = ISONoErr;
	err = MJ2NewMovie( &moov ); if (err) goto bail;
	err = ISONewMovieTrack( moov, MP4NewTrackIsAudio, &trak ); if (err) goto bail;
	err = ISONewTrackMedia( trak, &media, MP4AudioHandlerType, 48000, NULL ); if (err) goto bail;
	err = ISOBeginMediaEdits( media ); if (err) goto bail;
	err = addMySamples( trak, media ); if (err) goto bail;
	err = ISOEndMediaEdits( media ); if (err) goto bail;
	err = ISOGetMediaDuration( media, &mediaDuration ); if (err) goto bail;
	err = ISOInsertMediaIntoTrack( trak, 0, 0, mediaDuration, 1 ); if (err) goto bail;
	err = ISOWriteMovieToFile( moov, filename ); if (err) goto bail;
	err = ISODisposeMovie( moov ); if (err) goto bail;
bail:
	return err;
}

/* this is not finished yet... */
ISOErr createQTMovie( char *filename )
{
	ISOErr err;
	ISOMovie moov;
	ISOTrack trak;
	ISOMedia media;
    u64 mediaDuration;
	
	err = ISONoErr;
	err = QTNewMovie( &moov ); if (err) goto bail;
	err = ISONewMovieTrack( moov, MP4NewTrackIsAudio, &trak ); if (err) goto bail;
	err = ISONewTrackMedia( trak, &media, MP4AudioHandlerType, 48000, NULL ); if (err) goto bail;
	err = ISOBeginMediaEdits( media ); if (err) goto bail;
	err = addMySamples( trak, media ); if (err) goto bail;
	err = ISOEndMediaEdits( media ); if (err) goto bail;
	err = ISOGetMediaDuration( media, &mediaDuration ); if (err) goto bail;
	
	if ( media )
	{
 		MP4MediaAtomPtr	mdia;
		MP4HandlerAtomPtr hdlr;
		
 		mdia = (MP4MediaAtomPtr) media;
 		hdlr = (MP4HandlerAtomPtr) mdia->handler;
  		hdlr->qt_componentType = ISOMediaHandlerType;
  		hdlr->qt_componentManufacturer = MP4_FOUR_CHAR_CODE( 'a', 'p', 'p', 'l' );
  		hdlr->setName( (MP4AtomPtr) hdlr, "Apple Sound Media Handler");
	}
	
	err = ISOInsertMediaIntoTrack( trak, 0, 0, mediaDuration, 1 ); if (err) goto bail;
	err = ISOWriteMovieToFile( moov, filename ); if (err) goto bail;
	err = ISODisposeMovie( moov ); if (err) goto bail;
bail:
	return err;
}

ISOErr addMySamples( ISOTrack trak, ISOMedia media )
{
	ISOErr err;
	ISOHandle sampleEntryH;
	ISOHandle sampleDataH;
	ISOHandle sampleDurationH;
	ISOHandle sampleSizeH;
	ISOHandle decoderSpecificInfoH;
 	u32 objectTypeIndication;
	u32 streamType;
	u32 decoderBufferSize;
	u32 maxBitrate;
	u32 avgBitrate;
	
	err = ISONoErr;
	err = ISOSetMediaLanguage( media, "und" ); if (err) goto bail;

	objectTypeIndication = 0x40; /* mpeg-4 audio */
	streamType = 0x05; /* audio stream */
	decoderBufferSize = 8192;
	maxBitrate = 64000;
	avgBitrate = 64000;
	decoderSpecificInfoH = NULL;
	err = ISONewHandle( 0, &sampleEntryH ); if (err) goto bail;
	err = MP4NewSampleDescription( trak, sampleEntryH,
								   1,
					     		   objectTypeIndication,
					               streamType,
					               decoderBufferSize,
					               maxBitrate,
					               avgBitrate,
					               decoderSpecificInfoH ); if (err) goto bail;

	err = ISONewHandle( sizeof(u32), &sampleDurationH ); if (err) goto bail;
	*((u32*) *sampleDurationH) = 1024;
	err = ISONewHandle( 0, &sampleDataH ); if (err) goto bail;
	err = ISONewHandle( sizeof(u32), &sampleSizeH ); if (err) goto bail;
	for ( ;; )
	{
		err = getNextAudioFrame( sampleDataH );
		if ( err )
		{
			if ( err == ISOEOF )
				err = 0;
			break;
		}
		err = ISOGetHandleSize( sampleDataH, (u32*) *sampleSizeH ); if (err) goto bail;
		err = ISOAddMediaSamples( media, sampleDataH, 1, sampleDurationH, sampleSizeH, sampleEntryH, NULL, NULL ); if (err) goto bail;
		if ( sampleEntryH )
		{
			err = ISODisposeHandle( sampleEntryH ); if (err) goto bail;
			sampleEntryH = NULL;
		}	
	}
bail:
	if ( sampleDataH )
		ISODisposeHandle( sampleDataH );
	
	if ( sampleDurationH )
		ISODisposeHandle( sampleDurationH );
	
	if ( sampleSizeH )
		ISODisposeHandle( sampleSizeH );
	
	if ( decoderSpecificInfoH )
		ISODisposeHandle( decoderSpecificInfoH );

	return err;
}

ISOErr getNextAudioFrame( ISOHandle sampleH )
{
	ISOErr err;
	u32 frameLength;
	static u32 frameCount = 1;
	
	err = ISONoErr;
	if ( frameCount >= 4 )
		return ISOEOF;
	frameLength = 1;
	err = ISOSetHandleSize( sampleH, frameLength ); if (err) goto bail;
	**sampleH = (u8) (frameCount++ & 0xff);
bail:
	return err;
}
