/*
This software module was originally developed by Apple Computer, Inc.
in the course of development of MPEG-4. 
This software module is an implementation of a part of one or 
more MPEG-4 tools as specified by MPEG-4. 
ISO/IEC gives users of MPEG-4 free license to this
software module or modifications thereof for use in hardware 
or software products claiming conformance to MPEG-4.
Those intending to use this software module in hardware or software
products are advised that its use may infringe existing patents.
The original developer of this software module and his/her company,
the subsequent editors and their companies, and ISO/IEC have no
liability for use of this software module or modifications thereof
in an implementation.
Copyright is not released for non MPEG-4 conforming
products. Apple Computer, Inc. retains full right to use the code for its own
purpose, assign or donate the code to a third party and to
inhibit third parties from using the code for non 
MPEG-4 conforming products.
This copyright notice must be included in all copies or
derivative works. Copyright (c) 1999.
*/
#include "ISOMovies.h"

ISOErr playMyMovie( char *filename, u32 trackNumber );

int main( int argc, char **argv )
{
	ISOErr err;
	char *filename = "myMovie.mj2";

	err = MP4NoErr;
	if ( argc == 2 )
		filename = argv[1];
	err = playMyMovie( filename, 1 );
	fprintf( stderr, "playMyMovie returns %d\n", err );
	return err;
}

ISOErr playMyMovie( char *filename, u32 trackNumber )
{
	ISOErr err;
	u32 handlerType;
	ISOMovie moov;
	ISOTrack trak;
	ISOMedia media;
	MP4TrackReader reader;
	ISOHandle sampleH;
	ISOHandle decoderConfigH;
	
	err = ISONoErr;
	err = ISOOpenMovieFile( &moov, filename, MP4OpenMovieNormal ); if (err) goto bail;
	err = ISOGetMovieIndTrack( moov, trackNumber, &trak ); if (err) goto bail;
	err = ISOGetTrackMedia( trak, &media ); if (err) goto bail;
	err = ISOGetMediaHandlerDescription( media, &handlerType, NULL ); if (err) goto bail;
	err = ISONewHandle( 0, &decoderConfigH ); if (err) goto bail;
	err = ISOCreateTrackReader( trak, &reader ); if (err) goto bail;
/*	err = ISOTrackReaderGetCurrentDecoderConfig( reader, decoderConfigH ); if (err) goto bail; */

	/*
	 instantiate proper decoder for mediatype / decoderconfig
	 (not shown)
	*/
	err = ISONewHandle( 0, &sampleH ); if (err) goto bail;	
	/* play every frame */
	for (;;)
	{
		u32 unitSize;
		u32 cts;
		u32 dts;
		u32 sampleFlags;
		
		err = MP4TrackReaderGetNextAccessUnit( reader, sampleH, &unitSize, &sampleFlags, &cts, &dts );
		if ( err )
		{
			if ( err == MP4EOF )
				err = MP4NoErr;
			break;
		}
		/*
		  send AU to decoder
		  (simulated)
		*/
		fprintf( stderr, "send AU to decoder: size = %d, cts = %d, dts = %d\n", unitSize, cts, dts );
	}
	
bail:
	// close track reader
	ISODisposeTrackReader( reader );
	
	if ( decoderConfigH )
		ISODisposeHandle( decoderConfigH );
	
	if ( sampleH )
		ISODisposeHandle( sampleH );
	
	err = ISODisposeMovie( moov );
	
	return err;
}
