/*
CONTRIBUTION TO THE WG11 PROGRAM MPEG 4 PLAYER
______________________________________________________________________

STANDARDS PROJECT:  MPEG 4 Player
______________________________________________________________________

TITLE:  Animation of 3D Face Mpdels using Animation Definition Tables
______________________________________________________________________

AUTHORS:  	Erich Haratsch, Joern Ostermann

SOURCE:      	AT&T
 
CONTACT:	Joern Ostermann
		AT&T Labs - Research
		Room NSL 3-231
		100 Schultz Drive - Middletown
		Red Bank, NJ  07701-7033
		USA
		email: osterman@research.att.com   
		fax:   +1 732 345 3033
		phone:  +1 732 345 3311		   
______________________________________________________________________

DATE:  July 14, Middletown, New Jersey, USA
______________________________________________________________________

DISTRIBUTION:  MPEG 4 WG11
______________________________________________________________________

ABSTRACT:  fapfile_anim_mpeg_release takes as input an decoded 
downloaded model and decoded AnimationDefinitionTables and animates 
the model with decoded FAP sequences


______________________________________________________________________
Notice

This document has been prepared by AT&T
to assist the Standards Committee WG 11.  It is proposed to the Committee
as a basis for discussion and is not to be construed as a binding proposal
on AT&T. AT&T specifically reserves the right to amend or modify the
material contained herein and nothing herein shall be construed as conferring
or offering licenses or rights with respect to any intellectual property of
AT&T.

AT&T Proprietary
*/
#include <Inventor/Xt/SoXt.h>

#include <Inventor/actions/SoWriteAction.h>

#include <Inventor/nodes/SoGroup.h>

#include <Inventor/nodes/SoSeparator.h>
#include <Inventor/Xt/viewers/SoXtExaminerViewer.h>
#include <Inventor/nodes/SoDirectionalLight.h>
#include <Inventor/nodes/SoPerspectiveCamera.h>
#include <Inventor/nodes/SoSeparator.h>
#include <Inventor/nodes/SoFaceSet.h>
#include <Inventor/nodes/SoCoordinate3.h>
#include <Inventor/nodes/SoMaterial.h>
#include <Inventor/nodes/SoTransform.h>
#include <Inventor/nodes/SoTexture2.h>
#include <Inventor/nodes/SoTexture2Transform.h>
#include <Inventor/SoOffscreenRenderer.h>

#include "Face_mpeg.h"
#include "DisplayManager_mpeg.h"

#define MAXLINE 10000 /*original is 1000*/


extern char file_to_be_read[MAXCHAR];  /* ostermann */
extern int FIXRATE, frate;


extern int outputfile;
extern char outputfilename[MAXCHAR];

const float dirLightPosition[3] = {-0.497468, 0.497468, 0.710669};
const float dirLightColor[3] = {1,1,1};
const float perspectivePosition[3] = {309.120117, 300, 2500};
const float perspectiveAspectRatio = 0.375;
const float perspectiveNearDistance = 1837;
const float perspectiveFarDistance = 3233;
const float perspectiveFocalDistance = 2200;
const float perspectiveHeightAngle = 0.8;
const SbVec3f perspectiveTarget(0,0,0);

// *****************************************************************
DisplayManager:: DisplayManager()
{
  topWindow = NULL;
  viewer = NULL;
 
  displaySceneGraph = new SoSeparator;
  displaySceneGraph->ref();
}
// *****************************************************************
DisplayManager::~DisplayManager()
{
}

/*void DisplayManager::addBaldie(Baldie *_baldie)
{
  baldie = _baldie;
}
*/

// ******************************************************************
int DisplayManager::createDisplay(const char *appName) 
{

  topWindow = SoXt::init(appName);
  if (topWindow == NULL) return 1;
  else {
    addPerspectiveCamera();
    addLogo();
    viewer = new SoXtExaminerViewer(topWindow);
//    unsigned long redrawpriority;
//    redrawpriority = viewer->getRedrawPriority();
//    printf("RedrawPriority:%d\n", redrawpriority);
//    viewer->setRedrawPriority(0);
    viewer->setHeadlight(FALSE);
//    addDirectionalLight();
//    viewer->setViewing(FALSE);
    viewer->setSceneGraph(displaySceneGraph);
    viewer->setTitle("facetts");
    viewer->setAutoRedraw(TRUE);
//    viewer->setAntialiasing(TRUE,2);
//    viewer->setBackgroundColor(SbColor(0.,1.,1.));
    viewer->setBackgroundColor(SbColor(0.,0.,0.));
    SoXt::show(topWindow);
    viewer->show();
    XtAppAddInput(SoXt::getAppContext(), fileno(stdin), 
		  (XtPointer)XtInputReadMask, &DisplayManager::getFileXt, NULL);

    return 0;
  }
}
// ******************************************************************
void DisplayManager::addPerspectiveCamera()
{ 
  perspectiveCamera = new SoPerspectiveCamera;
  perspectiveCamera->position.setValue(perspectivePosition);
  perspectiveCamera->pointAt(perspectiveTarget);
  perspectiveCamera->aspectRatio.setValue(perspectiveAspectRatio);
  perspectiveCamera->nearDistance.setValue(perspectiveNearDistance);
  perspectiveCamera->farDistance.setValue(perspectiveFarDistance);
  perspectiveCamera->heightAngle.setValue(perspectiveHeightAngle);
  perspectiveCamera->focalDistance.setValue(perspectiveFocalDistance);
  displaySceneGraph->addChild(perspectiveCamera);
}

// ******************************************************************
void DisplayManager::addDirectionalLight()
{
  
  dirLight = new SoDirectionalLight;
  dirLight->direction.setValue(dirLightPosition);
  dirLight->color.setValue(dirLightColor);
  displaySceneGraph->addChild(dirLight);
  viewer->setHeadlight(FALSE);
}
// ******************************************************************
void DisplayManager::mainLoop() 
{
  SoXt::mainLoop();
}

// ******************************************************************
void DisplayManager::render() 
{
  viewer->render();
}

void DisplayManager::render2RGB(char *filename)
{
  FILE *fp;
  SoOffscreenRenderer offScreenRenderer(viewer->getViewportRegion());
  fp = fopen(filename, "w");
  offScreenRenderer.render(displaySceneGraph);
  offScreenRenderer.writeToRGB(fp);
  fclose(fp);
}

// ******************************************************************
void DisplayManager::addSceneGraph(SoSeparator *root)
{
//root->unrefNoDelete();
  displaySceneGraph->addChild(root);
 

}
// ******************************************************************
void DisplayManager::writeSceneGraph(char *output_iv)
{
  SoWriteAction writeBaldie;
  writeBaldie.getOutput()->openFile(output_iv);
  writeBaldie.getOutput()->setBinary(FALSE);
  writeBaldie.apply(displaySceneGraph);
  writeBaldie.getOutput()->closeFile();   
  printf("wrote SceneGraph into %s\n", output_iv);
}
// ******************************************************************
void getFile(void);
void DisplayManager::getFileXt(XtPointer, int *fid, XtInputId *)
  {
    char finline[MAXLINE];
    read(*fid, finline, MAXLINE);
    getFile();
}

void DisplayManager::addLogo()
{ 
  float logoVertices[4][3] =
  {{-81,-36,-100}, {81,-36,-100}, {81,36,-100}, {-81,36,-100} };
  const int32_t numLogoVertices[1] = {4};
  SoSeparator *logoSeparator = new SoSeparator;
  displaySceneGraph->addChild(logoSeparator);

  SoTexture2Transform *texture2Transform = new SoTexture2Transform;
  logoSeparator->addChild(texture2Transform);
  texture2Transform->scaleFactor.setValue(1.0,2.0);

  SoTexture2 *texture = new SoTexture2;
  logoSeparator->addChild(texture);
  texture->filename.setValue("logo.rgb");
  texture->model=SoTexture2::DECAL; 

  SoTransform *logoTransform = new SoTransform;
  logoTransform->translation.setValue(950,-700,600);
  logoTransform->scaleFactor.setValue(2,2,1);
//  logoTransform->rotation.setValue(0,1,0,2.2);
  logoSeparator->addChild(logoTransform); 


  SoCoordinate3 *logoCoords = new SoCoordinate3;
  logoCoords->point.setValues(0,4,logoVertices);
  logoSeparator->addChild(logoCoords); 
  SoFaceSet *logoFaceSet = new SoFaceSet;
  logoFaceSet->numVertices.setValues(0,1,numLogoVertices);
  logoSeparator->addChild(logoFaceSet);
}










