#ifndef DISPL_MANAGER_H
#define DISPL_MANAGER_H

/*
CONTRIBUTION TO THE WG11 PROGRAM MPEG 4 PLAYER
______________________________________________________________________

STANDARDS PROJECT:  MPEG 4 Player
______________________________________________________________________

TITLE:  Animation of 3D Face Mpdels using Animation Definition Tables
______________________________________________________________________

AUTHORS:  	Erich Haratsch, Joern Ostermann

SOURCE:      	AT&T
 
CONTACT:	Joern Ostermann
		AT&T Labs - Research
		Room NSL 3-231
		100 Schultz Drive - Middletown
		Red Bank, NJ  07701-7033
		USA
		email: osterman@research.att.com   
		fax:   +1 732 345 3033
		phone:  +1 732 345 3311		   
______________________________________________________________________

DATE:  July 14, Middletown, New Jersey, USA
______________________________________________________________________

DISTRIBUTION:  MPEG 4 WG11
______________________________________________________________________

ABSTRACT:  fapfile_anim_mpeg_release takes as input an decoded 
downloaded model and decoded AnimationDefinitionTables and animates 
the model with decoded FAP sequences


______________________________________________________________________
Notice

This document has been prepared by AT&T
to assist the Standards Committee WG 11.  It is proposed to the Committee
as a basis for discussion and is not to be construed as a binding proposal
on AT&T. AT&T specifically reserves the right to amend or modify the
material contained herein and nothing herein shall be construed as conferring
or offering licenses or rights with respect to any intellectual property of
AT&T.

AT&T Proprietary
*/

/* ######################################################################################
   #                                                                                    #
   # Class for facial animation. Author: Erich Haratsch, December 1996                  #
   #  --------------------------------------------------------------------------------  #
   #                                                                                    #
   #                              (c) 1996 AT&T Research                                #
   #                                                                                    #
   #                                                                                    #
   #  Library:                                                                          #
   #  File:          displ_manager.h                                                    #
   #  Class:         DisplayManager                                                     #
   #  Description:   manages the Display                                                #
   #                                                                                    #
   ###################################################################################### */


// *******************************************************************************

#include "Face_mpeg.h"

class DisplayManager
{
public:

  DisplayManager();

  ~DisplayManager();

  int  createDisplay(const char *appName); //creates the Viewer and initializes the OpenInventor database

  void addSceneGraph(SoSeparator *root); //adds the scene graph under root to the scene graph of the viewer (called displaySceneGraph)

  void mainLoop(); //starts main loop for event handling

  void render(); //renders the scene graph of the viewer

  void render2RGB(char *filename);
  //renders the scene graph off screen and saves the image as .rgb file to the given filename		     

  void writeSceneGraph(char *output_iv); //writes the scene graph of the viewer to a file

  void setAutoRedraw(const int autoredraw) {viewer->setAutoRedraw(autoredraw);}; //When autoredraw is TRUE, the render area in the viewer will automatically redraw whenever something in the scene graph changes. Passing FALSE will disable this feature

private:

  Widget topWindow;

  SoXtExaminerViewer *viewer; //OpenInventor viewer

  SoSeparator *displaySceneGraph; //root scene graph of the viewer

  SoPerspectiveCamera *perspectiveCamera; //perspective camera

  SoDirectionalLight *dirLight; //directional light source

  void addPerspectiveCamera(); //adds a perspective camera to the scene graph of the viewer

  void addLogo(); //adds the AT&T Logo to the scene graph of the viewer

  void addDirectionalLight(); //adds a directional light to the scene graph of the viewer 

  static void getFileXt(XtPointer, int *fid, XtInputId *); //if RETURN is pressed, the file with the text for visual speech is read
};


#endif











































