/*
CONTRIBUTION TO THE WG11 PROGRAM MPEG 4 PLAYER
______________________________________________________________________

STANDARDS PROJECT:  MPEG 4 Player
______________________________________________________________________

TITLE:  Animation of 3D Face Mpdels using Animation Definition Tables
______________________________________________________________________

AUTHORS:  	Erich Haratsch, Joern Ostermann

SOURCE:      	AT&T
 
CONTACT:	Joern Ostermann
		AT&T Labs - Research
		Room NSL 3-231
		100 Schultz Drive - Middletown
		Red Bank, NJ  07701-7033
		USA
		email: osterman@research.att.com   
		fax:   +1 732 345 3033
		phone:  +1 732 345 3311		   
______________________________________________________________________

DATE:  July 14, Middletown, New Jersey, USA
______________________________________________________________________

DISTRIBUTION:  MPEG 4 WG11
______________________________________________________________________

ABSTRACT:  fapfile_anim_mpeg_release takes as input an decoded 
downloaded model and decoded AnimationDefinitionTables and animates 
the model with decoded FAP sequences


______________________________________________________________________
Notice

This document has been prepared by AT&T
to assist the Standards Committee WG 11.  It is proposed to the Committee
as a basis for discussion and is not to be construed as a binding proposal
on AT&T. AT&T specifically reserves the right to amend or modify the
material contained herein and nothing herein shall be construed as conferring
or offering licenses or rights with respect to any intellectual property of
AT&T.

AT&T Proprietary
*/
#include <Inventor/SbLinear.h>

#include <Inventor/nodes/SoGroup.h>
#include <Inventor/nodes/SoShapeHints.h>
#include <Inventor/nodes/SoDirectionalLight.h>
#include <Inventor/nodes/SoIndexedFaceSet.h>
#include <Inventor/nodes/SoLightModel.h>
#include <Inventor/nodes/SoMaterial.h>
#include <Inventor/nodes/SoMatrixTransform.h>
#include <Inventor/nodes/SoSeparator.h>
#include <Inventor/nodes/SoTranslation.h>
#include <Inventor/actions/SoWriteAction.h>
#include <Inventor/nodes/SoCoordinate3.h>
#include <Inventor/fields/SoMFFloat.h>

//needed for displ_manager.h:

#include <Inventor/Xt/viewers/SoXtExaminerViewer.h>
#include <Inventor/nodes/SoPerspectiveCamera.h>
#include <Inventor/nodes/SoDirectionalLight.h>
#include <Inventor/nodes/SoRotationXYZ.h>
#include <stdio.h>
#include "time_mpeg.h"
#include "PartInfo_mpeg.h"

#include "DisplayManager_mpeg.h"

#include "Face_mpeg.h"

#define MAX_LINE_LENGTH 200
#define TPF       0.0333333       /*time per frame*/
int frate;
int FIXRATE=TRUE;


Face::Face()
{



  sceneGraph = new SoSeparator;
  sceneGraph->ref();


  fapValue=NULL;
  fapMask=NULL;


 
}

Face::~Face()
{
  delete[] fapValue;
  delete[] fapMask;

}

void Face::readFapFileNew(const char *filename)
{
  FILE *fp;
 
  char sequenceName[MAXCHAR];
  float version;
  int fps;
  int i, frame;
  
  if((fp=fopen(filename,"r")) == NULL) {
      fprintf(stderr,"Face::readFapFile: Can't open %s.\n",filename);
      exit(0);
  }

//   fgets(line,MAXCHAR,fp);
   fscanf(fp,"%f %s %d %d", &version, sequenceName, &fps, &fapNumFrames);

   frate = fps;

   fapMask = new int[fapNumFrames][68];
   fapValue = new int[fapNumFrames][68];

   for (frame=0; frame<fapNumFrames; frame++) {
//     fgets(line,MAXCHAR,fp);

     for(i=0;i<68;i++){
       fscanf(fp, "%d", &fapMask[frame][i]);
     }


//     fgets(line,MAXCHAR,fp);
     
     fscanf(fp, "%d", &frame);
     for(i=0;i<68;i++){
       if(fapMask[frame][i]==TRUE){
	 fscanf(fp, "%d", &fapValue[frame][i]);
       }
     }
   }
   for (frame=1; frame<fapNumFrames; frame++) {
     for(i=0;i<68;i++){
       if((fapMask[frame][i]==FALSE)&&(fapMask[frame-1][i]==TRUE)){
	 fapValue[frame][i]=fapValue[frame-1][i];
	 fapMask[frame][i]=TRUE;
       }
     }
   }
}




void Face::readModel(const char *inventor_model_file){
  model.readFile(inventor_model_file);
  sceneGraph->addChild(model.getSceneGraph());
  modelNumTransformInfos=0;
  modelNumIndexedFaceSetInfos=0;
}


void Face::calcFAPUsModel(char *featurePointFile){

  FILE *fp;
  if((fp=fopen(featurePointFile,"r")) == NULL) {
    fprintf(stderr,"Can't open %s.\n",featurePointFile);
      exit(0);
  }
  SbVec3f fp_2_2;
  SbVec3f fp_2_3;
  SbVec3f fp_3_1;
  SbVec3f fp_3_3;
  SbVec3f fp_3_5;
  SbVec3f fp_3_6;
  SbVec3f fp_8_3;
  SbVec3f fp_8_4;
  SbVec3f fp_9_4;
  char line[MAXCHAR];
  
  float x, y, z;
   
  fgets(line,MAXCHAR,fp);
  sscanf(line, "%f %f %f", &x, &y, &z);
  fp_2_2.setValue(x, y, z);
  
  fgets(line,MAXCHAR,fp);
  sscanf(line, "%f %f %f", &x, &y, &z);
  fp_2_3.setValue(x, y, z);
  
  fgets(line,MAXCHAR,fp);
  sscanf(line, "%f %f %f", &x, &y, &z);
  fp_3_1.setValue(x, y, z);
  
  fgets(line,MAXCHAR,fp);
  sscanf(line, "%f %f %f", &x, &y, &z);
  fp_3_3.setValue(x, y, z);
  
  fgets(line,MAXCHAR,fp);
  sscanf(line, "%f %f %f", &x, &y, &z);
  fp_3_5.setValue(x, y, z);
  
  fgets(line,MAXCHAR,fp);
  sscanf(line, "%f %f %f", &x, &y, &z);
  fp_3_6.setValue(x, y, z);
  
  fgets(line,MAXCHAR,fp);
  sscanf(line, "%f %f %f", &x, &y, &z);
  fp_8_3.setValue(x, y, z);
  
  fgets(line,MAXCHAR,fp);
  sscanf(line, "%f %f %f", &x, &y, &z);
  fp_8_4.setValue(x, y, z);
  
  fgets(line,MAXCHAR,fp);
  sscanf(line, "%f %f %f", &x, &y, &z);
  fp_9_4.setValue(x, y, z);
  
  ES = (fp_3_5.getValue()[0]-fp_3_6.getValue()[0])/1024;
  printf("ES: %f\n", ES);
  ENS = (0.5*(fp_3_5.getValue()[1]+fp_3_6.getValue()[1])
	 -fp_9_4.getValue()[1])/1024;
  printf("ENS: %f\n", ENS);
  MNS = (fp_9_4.getValue()[1]-
	 0.5*(fp_2_2.getValue()[1]+fp_2_3.getValue()[1]))/1024;
  printf("MNS: %f\n", MNS);
  MW = (fp_8_3.getValue()[0]-fp_8_4.getValue()[0])/1024;
  printf("MW: %f\n", MW);
  IRISD = (fp_3_1.getValue()[1]-fp_3_3.getValue()[1])/1024;
  printf("IRISD: %f\n", IRISD);
  setFAPUs();
  fclose(fp);
}





void Face::makeFAPTable(AnimationDefinitionTable &table, FILE *fp)
{

  int nNodes;
  fscanf(fp, "%d", &nNodes);
  int indexTransformTables = 0;
  int indexFaceSetTables = 0;
  for(int i=0; i<nNodes; i++){
    char nodeId[MAXCHAR];
    char nodeType[MAXCHAR];
    fscanf(fp, "%s %s", nodeId, nodeType);
    if(!(strcmp(nodeType, "Transform"))){
      TransformSTable transSTable;
      transSTable.setName(nodeId);
      int fieldRef;
      fscanf(fp, "%d", &fieldRef);
      transSTable.setFieldRef(fieldRef);
      float x, y, z, angle;
      if (fieldRef==2){    //AnimDefTransform defines rotation
        fscanf(fp, "%f %f %f %f", &x, &y, &z, &angle);
        SbVec3f axis(x, y, z);
        SbRotation rotation;
	rotation.setValue(axis, angle);
	transSTable.setRotation(rotation);
      }
      else {   //AnimDefTransform defines translation or scaling
	fscanf(fp, "%f %f %f", &x, &y, &z);
        SbVec3f translationOrScalevector(x, y, z);
	transSTable.setTranslationOrScalefactor(translationOrScalevector);
      }
   
      table.set1Value(indexTransformTables, transSTable);
      indexTransformTables++;
    }
    if(!(strcmp(nodeType, "IndexedFaceSet"))){
      int nVertices, nIntervals;
      fscanf(fp, "%d %d", &nVertices, &nIntervals);
      int *intervalBorders = new int[nIntervals+1];
      for(int j=0;j<nIntervals+1;j++){
	fscanf(fp, "%d", &intervalBorders[j]);
      }
      IndexedFaceSetSTable faceSetSTable(nVertices, nIntervals, intervalBorders);
      faceSetSTable.setName(nodeId);
      for(j=0; j<nVertices; j++){
	VertexDisplacementsInfo displacementsInfo;
	int vertexIndex;
	fscanf(fp, "%d", &vertexIndex);
	SbVec3f *displacements = new SbVec3f[nIntervals];
	for(int k=0;k<nIntervals; k++){
	  float displacement[3];
	  fscanf(fp, "%f %f %f", 
		 &displacement[0], &displacement[1], &displacement[2]);
	  displacements[k].setValue(displacement);
	}
	displacementsInfo.setValues(vertexIndex, nIntervals, 
				    displacements);
	faceSetSTable.set1VertexDisplacements(j, displacementsInfo);
	delete[] displacements;
      }
      table.set1Value(indexFaceSetTables, faceSetSTable);
      indexFaceSetTables++;
    }
  }
}


void Face::updateAnimFapTables(int frame)
{
  for(int i=0;i<68;i++){
    if(fapMask[frame][i]==TRUE){
      calcValuesFapTables(i , fapValue[frame][i]);
    }
  }
  for(int infoIndex=0;infoIndex<modelNumIndexedFaceSetInfos;infoIndex++){
    modelIndexedFaceSetInfos[infoIndex].setLocalVerticesValues();  
    modelIndexedFaceSetInfos[infoIndex].initUpdateMask();
  }
}

void Face::calcValuesFapTables(int FAPId, int fap_value)
{
  if(FapTables[FAPId].getNumNodes()>0){
    int numTransformTables = FapTables[FAPId].getNumTransformTables();
    for(int i=0; i<numTransformTables; i++){

      const TransformSTable *transformTable = FapTables[FAPId].getTransformSTable(i);
      const char *transformTableName = transformTable->getName();
      int infoExists = FALSE;
      for(int infoIndex=0; infoIndex<modelNumTransformInfos; infoIndex++){
	if(!(strcmp(modelTransformInfos[infoIndex].getName(), transformTableName))){
	  infoExists = TRUE;
	  break;
	}
      }
      if(infoExists == FALSE){
	modelNumTransformInfos++;
//	SoNode *modelNode = model.getNode(transformTableName);
        SoTransform *modelTransformNode = model.getTransformNode(transformTableName);


        if(modelTransformNode!=NULL){
//	  modelTransformInfos[infoIndex].setNode((SoTransform *)modelNode);
          modelTransformInfos[infoIndex].setNode(modelTransformNode);
	  modelTransformInfos[infoIndex].setNeutralValues(transformTable);
	  modelTransformInfos[infoIndex].setName(transformTableName);
	} else{
	  printf("Face::calcValuesFapTables: Transform node %s does not exit in downloaded model\n", transformTableName);
	}
      }
      modelTransformInfos[infoIndex].setTransformValue(fap_value*fapFAPU[FAPId]);
    }
	  
    int numIndexedFaceSetTables = FapTables[FAPId].getNumIndexedFaceSetTables();
    for(i=0; i<numIndexedFaceSetTables; i++){
      const IndexedFaceSetSTable *faceSetTable = FapTables[FAPId].getIndexedFaceSetSTable(i);
      const char *faceSetTableName = faceSetTable->getName(); 
      int infoExists = FALSE;
      for(int infoIndex=0; infoIndex<modelNumIndexedFaceSetInfos; infoIndex++){
	if(!(strcmp(modelIndexedFaceSetInfos[infoIndex].getName(), faceSetTableName))){
	  infoExists = TRUE;
	  break;
	}
      }
      if(infoExists == FALSE){
	modelNumIndexedFaceSetInfos++;
	SoCoordinate3 *modelCoordNode = model.getPartCoordNode(faceSetTableName);
	if(modelCoordNode!=NULL){
	  modelIndexedFaceSetInfos[infoIndex].setNode(modelCoordNode);
	  modelIndexedFaceSetInfos[infoIndex].setNeutralValues();
	  modelIndexedFaceSetInfos[infoIndex].initLocalVertexPositions();
	  modelIndexedFaceSetInfos[infoIndex].makeUpdateMask();
	  modelIndexedFaceSetInfos[infoIndex].setName(faceSetTableName);
	} else{
	  printf("Face::calcValuesFapTables: Coordinate node %s does not exit in downloaded model\n", faceSetTableName);
	}
      }
    
      
//      modelIndexedFaceSetInfos[infoIndex].initLocalVertexPositions();

      int numVertices = faceSetTable->getNumVertices();
      int numIntervals = faceSetTable->getNumIntervals();
      const int *intervalBorders = faceSetTable->getIntervalBorders();

      int zeroIndexInterval;
      int fapIndexInterval;

       if(fap_value!=0){
	 if(fap_value<0){
	   fapIndexInterval=0;
	 }else{
	   fapIndexInterval=numIntervals-1;
	 }
 
	 for(int j=0;j<numIntervals+1; j++){
	   if(intervalBorders[j]==0){
	     zeroIndexInterval=j;
	   }
	 }
	 for(j=0;j<numIntervals; j++){
	   int indexIntervalTest = (fap_value>=intervalBorders[j])&&(fap_value<intervalBorders[j+1]);
	   if(indexIntervalTest){
	     fapIndexInterval=j;
	   }
	 }
       }

       for(int j=0; j<numVertices; j++){
	 SbVec3f displacement(0, 0, 0);
	 const VertexDisplacementsInfo *displacementsInfo = faceSetTable->getValue(j); 
	 if(fap_value==0){
	   modelIndexedFaceSetInfos[infoIndex].addVertexDisplacement(displacementsInfo->getVertexIndex(), displacement.getValue());
	 } else {
	   if(fapIndexInterval>=zeroIndexInterval){
	     for(int Ik=zeroIndexInterval; Ik<fapIndexInterval; Ik++){
	       displacement += (intervalBorders[Ik+1]-intervalBorders[Ik])*displacementsInfo->getValues(Ik)[0];
	     }
	     displacement += (fap_value-intervalBorders[Ik])*displacementsInfo->getValues(Ik)[0];
	     
	   }else{
	     for(int Ik=zeroIndexInterval; Ik>fapIndexInterval+1; Ik--){
	       displacement+=(intervalBorders[Ik]-intervalBorders[Ik-1])*displacementsInfo->getValues(Ik-1)[0];
	     }
	     displacement +=(intervalBorders[Ik]-fap_value)*displacementsInfo->getValues(Ik-1)[0];
	   }
	  
	   modelIndexedFaceSetInfos[infoIndex].addVertexDisplacement(displacementsInfo->getVertexIndex(), (displacement*fapFAPU[FAPId]).getValue());
	 }
       }
    }
  }
}




void Face::makeFAPTablesFromADTs(const char *ADTFile)
{
  FILE *fp;
  int nFAPTables, FAPId;

  if((fp=fopen(ADTFile,"r")) == NULL) {
      fprintf(stderr,"Face::makeFAPTablesFromADTs: Can't open %s.\n",ADTFile);
      exit(0);
  }

  fscanf(fp, "%d", &nFAPTables);

  for(int i=0;i<nFAPTables;i++){
    fscanf(fp, "%d", &FAPId);
    printf("Creating ADT for FAP %d\n", FAPId);
    makeFAPTable(FapTables[FAPId-1], fp);
  }
  fclose(fp);
}

void Face::setFAPUs()
{
  for (int i=0; i<2; i++){
    fapFAPU[i]=1;
  }

  for (i=2; i<5; i++){
    fapFAPU[i]=MNS;
  }

  for (i=5; i<7; i++){
    fapFAPU[i]=MW;
  }

  for (i=7; i<18; i++){
    fapFAPU[i]=MNS;
  }

  for (i=18; i<22; i++){
    fapFAPU[i]=IRISD;
  }

  for (i=22; i<26; i++){
    fapFAPU[i]=1;
  }

  for (i=26; i<30; i++){
    fapFAPU[i]=IRISD;
  }

//  for (i=28; i<30; i++){
//    fapFAPU[i]=ES;
//  }

  for (i=30; i<36; i++){
    fapFAPU[i]=ENS;
  }

  for (i=36; i<40; i++){
    fapFAPU[i]=ES;
  }

  for (i=40; i<42; i++){
    fapFAPU[i]=ENS;
  }

  for (i=42; i<46; i++){
    fapFAPU[i]=MW;
  }

  for (i=46; i<50; i++){
    fapFAPU[i]=1;
  }

  for (i=50; i<52; i++){
    fapFAPU[i]=MNS;
  }

  for (i=52; i<54; i++){
    fapFAPU[i]=MW;
  }

  for (i=54; i<60; i++){
    fapFAPU[i]=MNS;
  }

  for (i=60; i<68; i++){
    fapFAPU[i]=ENS;
  }
}




SoSeparator *Face::getSceneGraph()
{
  return sceneGraph;
}

extern DisplayManager dispm;
extern char output_filename[MAXCHAR];

void Face::dorendFAPTablesOffScreen()
{
 
  char filename[MAXCHAR];
  char frameChar[MAXCHAR];
  int framei = 0;
  while(framei<(fapNumFrames-1)){
    sprintf(frameChar, "%d", framei);
    strcpy(filename,output_filename);
    strcat(filename, frameChar);
    strcat(filename, ".rgb");
    updateAnimFapTables(framei);
    dispm.render2RGB(filename);
    framei++;
  }
}

void Face::dorendFAPTables()
{
  int start = 0, stop = fapNumFrames-1;
  int nf;
  register double afps, rt, tpf = TPF;
  static int framei_step=3;   /* make default 10 frames per second */
  int time_offset=0;
  int ntimer=0; //Counts no. of calls to timer   
//  drstop = min(stop, maxframe);
  int drstop = stop;
  framei_step=FIXRATE ? 30/frate : framei_step;
  int framei=start;
  nf=0;
  start_timer();
  fprintf(stdout,"%d images will be computed\n", 
	  (drstop-start)/framei_step+1); fflush(stdout);
	  

  while(framei <= drstop) {
    register double ft=(framei-1.)*tpf-0.1*time_offset;

    updateAnimFapTables(framei);

    rt=read_timer();
    sginap((ft-rt)*100-1); // wakes up 1/100 of a second early
    while((rt=read_timer()) < ft)
      ntimer++;       
      dispm.setAutoRedraw(FALSE);
      dispm.render();
      dispm.setAutoRedraw(TRUE);

      framei=framei+framei_step; 

      nf++;
  }
  rt = read_timer();	/*rendering time*/
  nf = drstop - start + 1;
  afps = (double)nf / rt;	/*number of frames rendered per second*/
  if(afps < 29.0)
    if(!FIXRATE) {
      framei_step=29.5/afps+1;
      fprintf(stderr, " did %d frames in %f sec, %f fps \n", nf, rt, afps);
      fprintf(stderr, "skip %d frame. frame rate is now %d per sec.\n", 
	      framei_step, 30/framei_step); 
    }
  if (ntimer==0)
    printf("TIMER READ %d TIMES!\n",ntimer);
}

















