/*
CONTRIBUTION TO THE WG11 PROGRAM MPEG 4 PLAYER
______________________________________________________________________

STANDARDS PROJECT:  MPEG 4 Player
______________________________________________________________________

TITLE:  Animation of 3D Face Mpdels using Animation Definition Tables
______________________________________________________________________

AUTHORS:  	Erich Haratsch, Joern Ostermann

SOURCE:      	AT&T
 
CONTACT:	Joern Ostermann
		AT&T Labs - Research
		Room NSL 3-231
		100 Schultz Drive - Middletown
		Red Bank, NJ  07701-7033
		USA
		email: osterman@research.att.com   
		fax:   +1 732 345 3033
		phone:  +1 732 345 3311		   
______________________________________________________________________

DATE:  July 14, Middletown, New Jersey, USA
______________________________________________________________________

DISTRIBUTION:  MPEG 4 WG11
______________________________________________________________________

ABSTRACT:  fapfile_anim_mpeg_release takes as input an decoded 
downloaded model and decoded AnimationDefinitionTables and animates 
the model with decoded FAP sequences


______________________________________________________________________
Notice

This document has been prepared by AT&T
to assist the Standards Committee WG 11.  It is proposed to the Committee
as a basis for discussion and is not to be construed as a binding proposal
on AT&T. AT&T specifically reserves the right to amend or modify the
material contained herein and nothing herein shall be construed as conferring
or offering licenses or rights with respect to any intellectual property of
AT&T.

AT&T Proprietary
*/
#ifndef Face_H
#define Face_H


/* ######################################################################################
   #                                                                                    #
   # Class for facial animation. Author: Erich Haratsch, February 1997                  #
   #  --------------------------------------------------------------------------------  #
   #                                                                                    #
   #                              (c) 1997 AT&T Research                                #
   #                                                                                    #
   #                                                                                    #
   #  Library:                                                                          #
   #  File:          Face.h                                                           #
   #  Class:         Face                                                             #
   #  Description:   encapsulates all rendering functions for the model Face          #
   #                                                                                    #
   ###################################################################################### */
#include "PartInfo_mpeg.h"
#include "Model_mpeg.h"
#include "TableIndexedFaceSet_mpeg.h"

#define MAXCHAR 256
#define MAXMODELNUMPARTS 20
class DisplayManager;

class Face
{

public:

  Face();

  /*  Face(const char *FaceParts[], const int numFaceParts);*/
  ~Face();





  void readModel(const char *inventor_model_file); //not used



  SoSeparator *getSceneGraph(); 
  //get the scene graph of the current face model

 

 
  void calcFAPUsModel(char *featurePointFile);


			      
  void makeFAPTablesFromADTs(const char *ADTFile);
  //reads the ADTs from a file named ADTFile);
									


  void readFapFileNew(const char *filename); //reads an FAP sequence in the MPEG-4 raw data format; the header (all lines starting with #) must be deleted in this file prior to reading

 

  void dorendFAPTables(); //renders the frames of an MPEG-4 FAP sequence
  void dorendFAPTablesOffScreen();
			      
 

private:

 
  int (*fapValue)[68];//array for all 68 FAP values

  int (*fapMask)[68];//array for the mask for the FAPs

  float fapFAPU[68];//for each FAP, the appropriate mask is stored here
 
  void makeFAPTable(AnimationDefinitionTable &table, FILE *fp);
  //reads an AnimationDefinitionTable from the file pointed to by fp

  int fapNumFrames; //number of frames to be rendered in dorendFAPTables
  
  float ES, ENS, MNS, MW, IRISD; //FAPUs

  SoSeparator *sceneGraph; //scene graph of face model

  Model model; //manages a Inventor model

  AnimationDefinitionTable FapTables[68]; //array for the ADTs of all FAPs

  IndexedFaceSetInfo modelIndexedFaceSetInfos[MAXMODELNUMPARTS]; 
  //array of IndexedFaceSetInfos; used for animation with FAP sequences; initialized in calcValuesFapTables()

  TransformInfo modelTransformInfos[MAXMODELNUMPARTS]; 
  //array of TransformInfos; used for animation with FAP sequences; initialized in calcValuesFapTables()

  int modelNumTransformInfos; 
  //number of initialized TransformInfos in the array modelTransformInfos

  int modelNumIndexedFaceSetInfos; 
  //number of initialized IndexedFaceSetInfos in the array modelIndexedFaceSetInfos



  SoDirectionalLight *makeDirectionalLight(); //creates a directional light
 
  void setFAPUs(); //sets the FAPU for all FAPs in fapFAPU; used in calcFAPUsModel()

  void updateAnimFapTables(int frame); 
  //for animation with an FAP sequence: updates all IndexedFaceSet and Transforms according to the ADTs for a frame
  void calcValuesFapTables(int FAPId, int fap_value); 
  //called by updateAnimFapTables; updates the nodes deformed by the FAP FAPId according to the ADT

};



#endif


