/*
CONTRIBUTION TO THE WG11 PROGRAM MPEG 4 PLAYER
______________________________________________________________________

STANDARDS PROJECT:  MPEG 4 Player
______________________________________________________________________

TITLE:  Animation of 3D Face Mpdels using Animation Definition Tables
______________________________________________________________________

AUTHORS:  	Erich Haratsch, Joern Ostermann

SOURCE:      	AT&T
 
CONTACT:	Joern Ostermann
		AT&T Labs - Research
		Room NSL 3-231
		100 Schultz Drive - Middletown
		Red Bank, NJ  07701-7033
		USA
		email: osterman@research.att.com   
		fax:   +1 732 345 3033
		phone:  +1 732 345 3311		   
______________________________________________________________________

DATE:  July 14, Middletown, New Jersey, USA
______________________________________________________________________

DISTRIBUTION:  MPEG 4 WG11
______________________________________________________________________

ABSTRACT:  fapfile_anim_mpeg_release takes as input an decoded 
downloaded model and decoded AnimationDefinitionTables and animates 
the model with decoded FAP sequences


______________________________________________________________________
Notice

This document has been prepared by AT&T
to assist the Standards Committee WG 11.  It is proposed to the Committee
as a basis for discussion and is not to be construed as a binding proposal
on AT&T. AT&T specifically reserves the right to amend or modify the
material contained herein and nothing herein shall be construed as conferring
or offering licenses or rights with respect to any intellectual property of
AT&T.

AT&T Proprietary
*/
#include <Inventor/SbLinear.h>
#include <Inventor/nodes/SoSeparator.h>
#include <Inventor/SoPath.h>
#include <Inventor/SoInput.h>
#include <Inventor/actions/SoSearchAction.h>
#include <Inventor/nodes/SoCoordinate3.h>
#include <Inventor/nodes/SoDirectionalLight.h>
#include <Inventor/nodes/SoShapeHints.h>
#include <Inventor/nodes/SoIndexedFaceSet.h>
#include <Inventor/nodes/SoNode.h>
#include <Inventor/nodes/SoNormal.h>
#include <Inventor/nodes/SoTransform.h>


#include "Model_mpeg.h"
#include "Face_mpeg.h"

Model::Model(){
  root = NULL;
  //partPath = new SoPath;

}

Model::~Model(){
}

void Model::readFile(const char *filename){
  SoInput mySceneInput;
  if (!mySceneInput.openFile(filename)){
    fprintf(stderr, "Cannot open file %s\n", filename);
    exit(0);
  }
  root = SoDB::readAll(&mySceneInput);
  if(root==NULL){
    fprintf(stderr, "Problem reading file\n");
    exit(0);
  }
  root->ref();
  mySceneInput.closeFile();
}

 

SoCoordinate3 *Model::getPartCoordNode(const char* partName){
//  char pathfile[100];
//  strcpy(pathfile,partName);
//  strcat(pathfile, "searchNode");
  //setPartPath(partName);

  SoSearchAction partSearchAction;
  partSearchAction.setName(partName);
  partSearchAction.apply(root);
 
  if (partSearchAction.getPath() != NULL) {
    SoSeparator *partSeparator = 
      (SoSeparator *) partSearchAction.getPath()->getTail();

    SoPath *partPath = new SoPath;
    partPath->ref();
    partPath->setHead(partSeparator);
    SoSearchAction partCoordsSearchAction;
    partCoordsSearchAction.setType(SoCoordinate3::getClassTypeId());
    partCoordsSearchAction.setInterest(SoSearchAction::FIRST);

    partCoordsSearchAction.apply(partPath);
//    partPath->unref();
    return (SoCoordinate3 *) partCoordsSearchAction.getPath()->getTail();



  } else {
    printf("Model::findPartPath:Could not find: %s\n", partName);
    return NULL;
  }

}

SoNode *Model::getNode(const char *name)
{

  SoSearchAction partSearchAction;
  partSearchAction.setName(name);
  partSearchAction.apply(root);
 
  if (partSearchAction.getPath() != NULL) {
    return partSearchAction.getPath()->getTail();
  }else{
    printf("Model::getNode:Could not find: %s\n", name);
    return NULL;
  }
}

SoTransform *Model::getTransformNode(const char *name)
{

  SoSearchAction partSearchAction;
  partSearchAction.setName(name);
  partSearchAction.apply(root);
 
  if (partSearchAction.getPath() != NULL) {
    return (SoTransform *) partSearchAction.getPath()->getTail();
  }else{
    printf("Model::getNode:Could not find: %s\n", name);
    return NULL;
  }
}


void Model::clean(){
  SoSearchAction normalSearchAction;
  normalSearchAction.setType(SoNormal::getClassTypeId());
  normalSearchAction.setInterest(SoSearchAction::ALL);
  normalSearchAction.apply(root);

  SoPathList normalPathList = normalSearchAction.getPaths();
  int numPaths = normalPathList.getLength();
  int i;
  for(i=0; i<numPaths; i++){
    SoNormal *normalNode =
     (SoNormal *) normalPathList[i]->getTail();
    normalPathList[i]->pop();
    ((SoSeparator *) normalPathList[i]->getTail())->removeChild(normalNode);
  }

  SoSearchAction indexedFaceSetSearchAction;
  indexedFaceSetSearchAction.setType(SoIndexedFaceSet::getClassTypeId());
  indexedFaceSetSearchAction.setInterest(SoSearchAction::ALL);
  indexedFaceSetSearchAction.apply(root);

  SoPathList indexedFaceSetPathList = indexedFaceSetSearchAction.getPaths();
  numPaths = indexedFaceSetPathList.getLength();

  for(i=0; i<numPaths; i++){
    SoIndexedFaceSet *indexedFaceSetNode =
      (SoIndexedFaceSet *) indexedFaceSetPathList[i]->getTail();
    indexedFaceSetNode->normalIndex.setValue(-1);
  }

  SoSearchAction shapeHintsSearchAction;
  shapeHintsSearchAction.setType(SoShapeHints::getClassTypeId());
  shapeHintsSearchAction.setInterest(SoSearchAction::ALL);
  shapeHintsSearchAction.apply(root);

  SoPathList shapeHintsPathList = shapeHintsSearchAction.getPaths();
  numPaths = shapeHintsPathList.getLength();

  for(i=0; i<numPaths; i++){
    SoShapeHints *shapeHintsNode =
      (SoShapeHints *) shapeHintsPathList[i]->getTail();
    shapeHintsNode->vertexOrdering = SoShapeHints::UNKNOWN_ORDERING;
    shapeHintsNode->shapeType = SoShapeHints::SOLID;
//    shapeHintsNode->shapeType = SoShapeHints::UNKNOWN_SHAPE_TYPE;
    shapeHintsNode->creaseAngle = (M_PI);
  }

 
} 

SoSeparator *Model::getSceneGraph(){
  return root;
}

const SbVec3f *Model::getPartVertexPositions(const char *partName){
  SoCoordinate3 *aliasPartCoords=getPartCoordNode(partName);
  return aliasPartCoords->point.getValues(0);
}

int Model::getPartNumVertices(const char *partName){
  SoCoordinate3 *aliasPartCoords=getPartCoordNode(partName);
  if(aliasPartCoords==NULL){
    exit(0);
  }
  return aliasPartCoords->point.getNum();
}


SbVec3f Model::getFeaturePoint(const char *feature_point){
  SoCoordinate3 *fpCoords=getPartCoordNode(feature_point);
  if(fpCoords==0){
    exit(0);
  }
  SbVec3f fpVec3f;
  fpVec3f.setValue(fpCoords->point.getValues(0)[0].getValue());
  return fpVec3f;
}


float Model::calcFAPU(const char *FAPUName) {
  float FAPUValue;
  if (!strcmp(FAPUName, "ES")) {
    SbVec3f fp_3_5;
    SbVec3f fp_3_6;
    fp_3_5.setValue(getFeaturePoint("fp_3_5").getValue());
    fp_3_6.setValue(getFeaturePoint("fp_3_6").getValue());
    FAPUValue = (fp_3_5.getValue()[0]-fp_3_6.getValue()[0])/1024;
    return FAPUValue;
  }else if (!strcmp(FAPUName, "ENS")){
    SbVec3f fp_3_5;
    SbVec3f fp_3_6;
    SbVec3f fp_9_4;
    fp_3_5.setValue(getFeaturePoint("fp_3_5").getValue());
    fp_3_6.setValue(getFeaturePoint("fp_3_6").getValue());
    fp_9_4.setValue(getFeaturePoint("fp_9_4").getValue());
    FAPUValue = (0.5*(fp_3_5.getValue()[1]+fp_3_6.getValue()[1])
		 -fp_9_4.getValue()[1])/1024;
    return FAPUValue;
  }else if (!strcmp(FAPUName, "MNS")){
    SbVec3f fp_9_4;
    SbVec3f fp_2_2;
    SbVec3f fp_2_3;
    fp_9_4.setValue(getFeaturePoint("fp_9_4").getValue());
    fp_2_2.setValue(getFeaturePoint("fp_2_2").getValue());
    fp_2_3.setValue(getFeaturePoint("fp_2_3").getValue());
    FAPUValue = (fp_9_4.getValue()[1]-
		 0.5*(fp_2_2.getValue()[1]+fp_2_3.getValue()[1]))/1024;
    return FAPUValue;
  }else if (!strcmp(FAPUName, "MW")){
    SbVec3f fp_8_3;
    SbVec3f fp_8_4;
    fp_8_3.setValue(getFeaturePoint("fp_8_3").getValue());
    fp_8_4.setValue(getFeaturePoint("fp_8_4").getValue());
    FAPUValue = (fp_8_3.getValue()[0]-fp_8_4.getValue()[0])/1024;
    return FAPUValue;
  }else if(!strcmp(FAPUName, "IRISD")){
    SbVec3f fp_3_1;
    SbVec3f fp_3_3;
    fp_3_1.setValue(getFeaturePoint("fp_3_1").getValue());
    fp_3_3.setValue(getFeaturePoint("fp_3_3").getValue());
    FAPUValue = (fp_3_1.getValue()[1]-fp_3_3.getValue()[1])/1024;
    return FAPUValue;
  }
  else{
    printf("Model:calcFAPU: FAPU %s does not exist!\n", FAPUName);
    exit(0);
  }
  return 0;
}

void Model::unreadFile(){
  root->unref();
  root = NULL;
  //partPath = NULL;
}
