/*
CONTRIBUTION TO THE WG11 PROGRAM MPEG 4 PLAYER
______________________________________________________________________

STANDARDS PROJECT:  MPEG 4 Player
______________________________________________________________________

TITLE:  Animation of 3D Face Mpdels using Animation Definition Tables
______________________________________________________________________

AUTHORS:  	Erich Haratsch, Joern Ostermann

SOURCE:      	AT&T
 
CONTACT:	Joern Ostermann
		AT&T Labs - Research
		Room NSL 3-231
		100 Schultz Drive - Middletown
		Red Bank, NJ  07701-7033
		USA
		email: osterman@research.att.com   
		fax:   +1 732 345 3033
		phone:  +1 732 345 3311		   
______________________________________________________________________

DATE:  July 14, Middletown, New Jersey, USA
______________________________________________________________________

DISTRIBUTION:  MPEG 4 WG11
______________________________________________________________________

ABSTRACT:  fapfile_anim_mpeg_release takes as input an decoded 
downloaded model and decoded AnimationDefinitionTables and animates 
the model with decoded FAP sequences


______________________________________________________________________
Notice

This document has been prepared by AT&T
to assist the Standards Committee WG 11.  It is proposed to the Committee
as a basis for discussion and is not to be construed as a binding proposal
on AT&T. AT&T specifically reserves the right to amend or modify the
material contained herein and nothing herein shall be construed as conferring
or offering licenses or rights with respect to any intellectual property of
AT&T.

AT&T Proprietary
*/
#ifndef Model_H
#define Model_H

/* ######################################################################################
   #                                                                                    #
   # Class for facial animation. Author: Erich Haratsch, May 1997                       #
   #  --------------------------------------------------------------------------------  #
   #                                                                                    #
   #                              (c) 1997 AT&T Research                                #
   #                                                                                    #
   #                                                                                    #
   #  Library:                                                                          #
   #  File:          Model.h                                                    #
   #  Class:         Model                                                      #
   #  Description:   offers functions to read and manipulate an Inventor model          #
   #                                                                                    #
   ###################################################################################### */

class Model
{
public:
  
  Model();
  ~Model();
  void readFile(const char *aliasFile); //reads an Inventor model from a file
  void readSceneGraph(SoSeparator *sceneGraph){root=sceneGraph;}; //reads a scene graph
  void clean(); //cleans a scene graph; eg deletes normals, manipulates shape hints
  const SbVec3f *getPartVertexPositions(const char *partName); //gets the vertex positions of an IndexedFaceSet node
  int getPartNumVertices(const char *partName); //gets the number of vertices stored in an IndexedFaceSet
  SbVec3f getFeaturePoint(const char *feature_point); //gets the coordinate positions of a feature point; a feature point is a one-polygon IndexedFaceSet; the coordinates of the first vertex in this polygon are taken as feature point coordinates
  SoCoordinate3 *getPartCoordNode(const char* partName); //get the pointer to a Coordinate node
  SoTransform *getTransformNode(const char *name); //get the pointer to a Transform node
  SoNode *getNode(const char *name); //get the pointer to a node
  SoSeparator *getSceneGraph(); //return a pointer to the scene graph of the Inventor model
  float calcFAPU(const char *FAPUName); //calc the FAPUs of the model; expects that the feature points fp_3_6, fp_3_6, fp_9_4, fp_2_2, fp_2_3, fp_8_3, fp_8_4, fp_3_1, fp_3_3 are contained as one-polygon IndexedFaceSets in the Inventor model
  void unreadFile(); //delete the imported Inventor model

private:

  SoSeparator *root; //pointer to the scene graph of the imported Inventor model
  //SoPath *partPath;
  void setPartPath(const char *partName); //not used
};

#endif


