/*
CONTRIBUTION TO THE WG11 PROGRAM MPEG 4 PLAYER
______________________________________________________________________

STANDARDS PROJECT:  MPEG 4 Player
______________________________________________________________________

TITLE:  Animation of 3D Face Mpdels using Animation Definition Tables
______________________________________________________________________

AUTHORS:  	Erich Haratsch, Joern Ostermann

SOURCE:      	AT&T
 
CONTACT:	Joern Ostermann
		AT&T Labs - Research
		Room NSL 3-231
		100 Schultz Drive - Middletown
		Red Bank, NJ  07701-7033
		USA
		email: osterman@research.att.com   
		fax:   +1 732 345 3033
		phone:  +1 732 345 3311		   
______________________________________________________________________

DATE:  July 14, Middletown, New Jersey, USA
______________________________________________________________________

DISTRIBUTION:  MPEG 4 WG11
______________________________________________________________________

ABSTRACT:  fapfile_anim_mpeg_release takes as input an decoded 
downloaded model and decoded AnimationDefinitionTables and animates 
the model with decoded FAP sequences


______________________________________________________________________
Notice

This document has been prepared by AT&T
to assist the Standards Committee WG 11.  It is proposed to the Committee
as a basis for discussion and is not to be construed as a binding proposal
on AT&T. AT&T specifically reserves the right to amend or modify the
material contained herein and nothing herein shall be construed as conferring
or offering licenses or rights with respect to any intellectual property of
AT&T.

AT&T Proprietary
*/
#include <stdlib.h>

#include <Inventor/actions/SoSearchAction.h>
#include <Inventor/nodes/SoIndexedFaceSet.h>
#include <Inventor/nodes/SoSeparator.h>
#include <Inventor/Xt/viewers/SoXtExaminerViewer.h>
#include <Inventor/nodes/SoDirectionalLight.h>
#include <Inventor/nodes/SoPerspectiveCamera.h>

#include "PartInfo_mpeg.h"

void PartInfo::setName(const char *name)
{
  strcpy(_name, name);
}



TransformInfo::TransformInfo()
{
  node = new SoTransform;
}

TransformInfo::~TransformInfo(){}

void TransformInfo::setNode(SoTransform *transformNode)
{
  node = transformNode;
}

void TransformInfo::setNeutralValues(const TransformSTable *transformTable)
{
  fieldRef = transformTable->getFieldRef();
  if(fieldRef ==2){
    transformTable->getRotation(neutralAxis, neutralAngle);
  }
  else{
    transformTable->getTranslationOrScalefactor(neutralTranslationOrScalefactor);
  }
}

void TransformInfo::setTransformValue(const float value)
{
  float angle;
  switch(fieldRef){
  case 2:
    angle = value*M_PI/180;
    node->rotation.setValue(neutralAxis, angle + neutralAngle);
    break;
  case 4:
    node->scaleFactor.setValue(value*neutralTranslationOrScalefactor);
    break;
  case 5:
    node->translation.setValue(value*neutralTranslationOrScalefactor);
    break;
  }
}

SoTransform *TransformInfo::getNode()
{
  return node;
}

IndexedFaceSetInfo::IndexedFaceSetInfo()
{
  numLocalVertices=0;
  localVertexPositions = NULL;
  neutralLocalVertexPositions=NULL;
  updateMask=NULL;
  local2Global = NULL;
  partCoordNode = new SoCoordinate3;
 
}  

IndexedFaceSetInfo::~IndexedFaceSetInfo()
{
  delete[] localIndices;
  delete[] localVertexPositions;
  delete[] neutralLocalVertexPositions;
  delete[] updateMask;
  delete[] local2Global;
}

		       
void IndexedFaceSetInfo::setNode(SoCoordinate3 *coordNode){
  partCoordNode = coordNode;  
}

void IndexedFaceSetInfo::setNeutralValues(){
  if(partCoordNode==NULL){
    numLocalVertices = 0;
  } else{
    numLocalVertices=partCoordNode->point.getNum();
    delete[] neutralLocalVertexPositions;
    delete[] localVertexPositions;
    neutralLocalVertexPositions = new float[numLocalVertices][3];
    localVertexPositions = new float[numLocalVertices][3];
    int i,j;
    for (i=0; i<numLocalVertices;i++){
      for (j=0;j<3;j++){
	neutralLocalVertexPositions[i][j] = 
	  (partCoordNode->point.getValues(0)[i].getValue())[j];
      }
    }
  }
}

void IndexedFaceSetInfo::makeUpdateMask(){
  updateMask = new int[numLocalVertices];
  initUpdateMask();
}

void IndexedFaceSetInfo::initUpdateMask(){
  for(int i=0; i<numLocalVertices;i++){
    updateMask[i]=FALSE;
  }
}

void IndexedFaceSetInfo::setLocalVerticesValues(){
  partCoordNode->point.setValues(0, numLocalVertices, localVertexPositions);
}



void IndexedFaceSetInfo::initLocalVertexPositions(){
  for (int i=0; i<numLocalVertices;i++){
    for (int j=0;j<3;j++){
      localVertexPositions[i][j]=neutralLocalVertexPositions[i][j];
    }
  }
}

void IndexedFaceSetInfo::addVertexDisplacement(const int index, const float displacement[3]){
  if(updateMask[index]==FALSE){
    for(int j=0; j<3; j++){
      localVertexPositions[index][j]=neutralLocalVertexPositions[index][j] + displacement[j];
    }
    updateMask[index]=TRUE;
  }else{
    for(int j=0; j<3; j++){
      localVertexPositions[index][j]+=displacement[j];
    }    
  }
}


SoCoordinate3 *IndexedFaceSetInfo::getNode()
{
  return partCoordNode;
}





