/*
CONTRIBUTION TO THE WG11 PROGRAM MPEG 4 PLAYER
______________________________________________________________________

STANDARDS PROJECT:  MPEG 4 Player
______________________________________________________________________

TITLE:  Animation of 3D Face Mpdels using Animation Definition Tables
______________________________________________________________________

AUTHORS:  	Erich Haratsch, Joern Ostermann

SOURCE:      	AT&T
 
CONTACT:	Joern Ostermann
		AT&T Labs - Research
		Room NSL 3-231
		100 Schultz Drive - Middletown
		Red Bank, NJ  07701-7033
		USA
		email: osterman@research.att.com   
		fax:   +1 732 345 3033
		phone:  +1 732 345 3311		   
______________________________________________________________________

DATE:  July 14, Middletown, New Jersey, USA
______________________________________________________________________

DISTRIBUTION:  MPEG 4 WG11
______________________________________________________________________

ABSTRACT:  fapfile_anim_mpeg_release takes as input an decoded 
downloaded model and decoded AnimationDefinitionTables and animates 
the model with decoded FAP sequences


______________________________________________________________________
Notice

This document has been prepared by AT&T
to assist the Standards Committee WG 11.  It is proposed to the Committee
as a basis for discussion and is not to be construed as a binding proposal
on AT&T. AT&T specifically reserves the right to amend or modify the
material contained herein and nothing herein shall be construed as conferring
or offering licenses or rights with respect to any intellectual property of
AT&T.

AT&T Proprietary
*/
#ifndef PARTINFO_H
#define PARTINFO_H

/* ######################################################################################
   #                                                                                    #
   # Class for facial animation. Author: Erich Haratsch, March 1997               #
   #  --------------------------------------------------------------------------------  #
   #                                                                                    #
   #                              (c) 1997 AT&T Research                                #
   #                                                                                    #
   #                                                                                    #
   #  Library:                                                                          #
   #  File:          PartInfo.h                                                         #
   #  Class:         PartInfo, TransformInfo, IndexedFaceSetInfo                        #
   #  Description:   classes to manipulate Transforms and IndexedFaceSets nodes
                     in a scene graph #
   #                                                                                    #
   ###################################################################################### */

#include <inttypes.h>
#include <Inventor/nodes/SoCoordinate3.h>
#include <Inventor/nodes/SoTransform.h>
#include "TableIndexedFaceSet_mpeg.h"
#define MAXCHAR 256

class PartInfo //base class for TransformInfo and IndexedFaceSetInfo
{
public:

  PartInfo(){};
  ~PartInfo(){};
  void setName(const char *name); //sets the name of the node

  const char *getName(){return _name;}; //get the name of the node

private:

  char _name[MAXCHAR]; //name of the node

protected:

  SoSeparator *readFile(const char *filename); //reads a scene graph from a file
};

class TransformInfo : public virtual PartInfo //class for the manipulation of a Transform node
{

public:
  TransformInfo();
  ~TransformInfo();

  void setNode(SoTransform *transformNode); //set the internal pointer to transformNode
  void setNeutralValues(const TransformSTable *transformTable); //set neutral values for the Transform node
  void setTransformValue(const float transformValue); //set a value in the Transform node; fieldRef specifies which field of the Transform node is accessed
  SoTransform *getNode(); //get a pointer to the Transform node, to which this class is pointing

private:

  SoTransform *node; //pointer to the Transform node
  int fieldRef;			 
  SbVec3f neutralAxis; //neutral axis
  float neutralAngle;			   
  SbVec3f neutralTranslationOrScalefactor; //neutral scale factor or neutral translation vector
  
};


class IndexedFaceSetInfo : public virtual PartInfo //class for the manipulation of an Coordinate node of an IndexedFaceSet
{

public:

  IndexedFaceSetInfo();
  ~IndexedFaceSetInfo();
//  void setName(const char *name);
  void setValues(const int numGlobalVertices, 
		 const SoMFVec3f *globalVertexPositions, 
		 const int numIndices, const int32_t *globalIndices); //uses globalVertexPositions and globalIndices to determine the relationship between globalIndices and localIndices (local2Global) and set the coordinates for the local vertices in localVertexPositions 
  void setNode(SoCoordinate3 *coordNode); //set the internal Coordinate pointer to coordNode
  void setNeutralValues(); //pass the coordinates of the partCoordNode to neutralLocalVertexPositions
  void makeUpdateMask(); //make a mask for all vertices in localVertexPositions and set all mask values FALSE; if a displacement is added to a vertex in addVertexDisplacement, then the mask for this vertex is set TRUE
  void initUpdateMask(); //set all values for updateMask to FALSE
  void setLocalVerticesValues(); //pass the values from localVertexPositions to partCoordNode
  void initLocalVertexPositions(); //pass the values in neutralLocalVertexPositions to localVertexPositions
  void addVertexDisplacement(const int index, const float displacement[3]); //add a displacents to the vertex in localVertexPositions referenced by index
  void setGlobalVertexPositions(SoMFVec3f *globalVertexPositions, int & no_vert); //pass the values of partCoordNode to globalVertexPositions; the arrays globalVertexPositions and local2Global are enlarged if necessary
  void updateGlobalVertexPositions(float (*globalVertexPositions)[3]); //pass the values of partCoordNode to globalVertexPositions; the arrays globalVertexPositions and local2Global are not modified
  void readAlias(const char *filename, const char *partName, 
		 const int no_vert, SoMFVec3f *globalVertexPositions); //old

  int32_t *getLocalIndices(); //returns pointer to localIndices
//  float **getLocalVertexPositions();
  void update(const SoMFVec3f *globalVertexPositions); //pass the values in globalVertexPositions to partCoordNode
//  int getNumLocalVertices();
  SoCoordinate3 *getNode(); //returns pointer to partCoordNode

private:

  
  int numLocalVerticesInit;//number of vertices in partCoordNode after setValues has been called
  int numLocalVertices; //actual number of vertices in partCoordNode; may be changed in setNeutralValues
  int32_t *localIndices; //all global indices for vertices stored in partCoordNode are stored here
  float (*localVertexPositions)[3]; //array for the vertex positions of the IndexedFaceSet
  float (*neutralLocalVertexPositions)[3]; //array for the neutral vertex positions of the IndexedFaceSet
  int *updateMask; //mask for each vertex in localVertexPositions, which says whether a displacement has been added to a specific mask
  int *local2Global; //for each vertex stored in localVertexPositions, the global index is stored here
  SoCoordinate3 *partCoordNode; //pointer to the Coordinate node of the IndexedFaceSet

  static int cmp(const int *val1, const int *val2); //compare function, needed for qsort in setValues(..)
};

#endif
