/*
CONTRIBUTION TO THE WG11 PROGRAM MPEG 4 PLAYER
______________________________________________________________________

STANDARDS PROJECT:  MPEG 4 Player
______________________________________________________________________

TITLE:  Animation of 3D Face Mpdels using Animation Definition Tables
______________________________________________________________________

AUTHORS:  	Erich Haratsch, Joern Ostermann

SOURCE:      	AT&T
 
CONTACT:	Joern Ostermann
		AT&T Labs - Research
		Room NSL 3-231
		100 Schultz Drive - Middletown
		Red Bank, NJ  07701-7033
		USA
		email: osterman@research.att.com   
		fax:   +1 732 345 3033
		phone:  +1 732 345 3311		   
______________________________________________________________________

DATE:  July 14, Middletown, New Jersey, USA
______________________________________________________________________

DISTRIBUTION:  MPEG 4 WG11
______________________________________________________________________

ABSTRACT:  fapfile_anim_mpeg_release takes as input an decoded 
downloaded model and decoded AnimationDefinitionTables and animates 
the model with decoded FAP sequences


______________________________________________________________________
Notice

This document has been prepared by AT&T
to assist the Standards Committee WG 11.  It is proposed to the Committee
as a basis for discussion and is not to be construed as a binding proposal
on AT&T. AT&T specifically reserves the right to amend or modify the
material contained herein and nothing herein shall be construed as conferring
or offering licenses or rights with respect to any intellectual property of
AT&T.

AT&T Proprietary
*/
#include <Inventor/SbLinear.h>
#include "TableIndexedFaceSet_mpeg.h"
#include <stdio.h>

void VertexDisplacementsInfo::setValues(const int vertexIndex, 
				    const int nIntervals, 
				    SbVec3f *displacements)
{
  _nIntervals=nIntervals;
  _vertexIndex = vertexIndex;
  delete[] _displacements;
  _displacements = new SbVec3f[nIntervals];
  int i;
  for(i=0;i<nIntervals;i++){
      _displacements[i].setValue(displacements[i].getValue());
  }
}

int VertexDisplacementsInfo::getVertexIndex() const{
  return _vertexIndex;
}

const SbVec3f *VertexDisplacementsInfo::getValues(const int index) const{
  return _displacements+index;
}

VertexDisplacementsInfo & VertexDisplacementsInfo::operator = (const VertexDisplacementsInfo &v){
  _vertexIndex = v._vertexIndex;
  _nIntervals = v._nIntervals;
  delete[] _displacements;
  _displacements = new SbVec3f[_nIntervals];
  int i;
  for(i=0;i<_nIntervals;i++){
    _displacements[i].setValue(v._displacements[i].getValue());
  }
  return *this;
}

TransformSTable & TransformSTable::operator = (const TransformSTable &t)
{
  setName(t.getName());
  _fieldRef = t._fieldRef;

  if (_fieldRef==2){    //AnimDefTransform defines rotation
     SbVec3f axis;
     float angle;
     t._rotation.getValue(axis, angle);
     _rotation.setValue(axis, angle);
  }
  else {   //AnimDefTransform defines translation or scaling
     _translationOrScalefactor.setValue(t._translationOrScalefactor.getValue());
  }

  return *this;
}

void TransformSTable::setRotation(const SbRotation rotation)
{
  SbVec3f axis;
  float angle;
  rotation.getValue(axis, angle);
  _rotation.setValue(axis, angle);
}

void TransformSTable::setTranslationOrScalefactor(const SbVec3f translationOrScalefactor)
{
  _translationOrScalefactor.setValue(translationOrScalefactor.getValue());
}
  
IndexedFaceSetSTable::IndexedFaceSetSTable()
{
  _intervalBorders = NULL;
  _vertexDisplacementsTable = NULL;
  _nVertices = _nIntervals = 0;
}

IndexedFaceSetSTable::IndexedFaceSetSTable(const int nVertices, const int nIntervals, int *intervalBorders)
{
  _nVertices = nVertices;
  _nIntervals = nIntervals;
  _intervalBorders = new int[nIntervals+1];
  _vertexDisplacementsTable = new VertexDisplacementsInfo[nVertices];
  for(int i=0; i<nIntervals+1;i++){
    _intervalBorders[i]= intervalBorders[i];
  }
}

IndexedFaceSetSTable & IndexedFaceSetSTable::operator = (const IndexedFaceSetSTable &t)
{
  setName(t.getName());
  delete[] _intervalBorders;
  delete[] _vertexDisplacementsTable;
  _nVertices = t._nVertices;
  _nIntervals = t._nIntervals;
  _intervalBorders = new int[_nIntervals+1];
  _vertexDisplacementsTable = new VertexDisplacementsInfo[_nVertices];
  int i;
  for(i=0; i<_nIntervals+1;i++){
    _intervalBorders[i]= t._intervalBorders[i];
  }
  for(i=0;i<_nVertices;i++){
    _vertexDisplacementsTable[i] = t._vertexDisplacementsTable[i];
  }
  return *this;
}

/*void TableIndexedFaceSet::setIntervalBorders(const int *intervalBorders)
{
  int i;
  for (i=0; i<nIntervals+1;i++){
    _intervalBorders[i]= intervalBorders[i];
  }
}*/

void IndexedFaceSetSTable::set1VertexDisplacements(const int index, const VertexDisplacementsInfo & displacements)
{
  _vertexDisplacementsTable[index] = displacements;
}
  
int IndexedFaceSetSTable::getNumVertices() const
{
  return _nVertices;
}

int IndexedFaceSetSTable::getNumIntervals() const
{
  return _nIntervals;
}

const int *IndexedFaceSetSTable::getIntervalBorders() const
{
  return _intervalBorders;
}

const VertexDisplacementsInfo *IndexedFaceSetSTable::getValue(const int index) const{
  return _vertexDisplacementsTable+index;
}

IndexedFaceSetSTable::~IndexedFaceSetSTable()
{
  delete[] _intervalBorders;
  delete[] _vertexDisplacementsTable;
}

void AnimationDefinitionTable:: set1Value(const int index, const IndexedFaceSetSTable  & newValue)
{
  if (index > (maxNum-1)){
    allocValues(index+1);
  }
  tables[index]=newValue;
}

void AnimationDefinitionTable:: set1Value(const int index, const TransformSTable  & newValue)
{
  if (index > (maxNumTransformTables-1)){
    allocValuesTransformTables(index+1);
  }
  transformTables[index]=newValue;
}

const IndexedFaceSetSTable *AnimationDefinitionTable:: getIndexedFaceSetSTable(const int index) const
{
  return tables+index;
}

const TransformSTable *AnimationDefinitionTable:: getTransformSTable(const int index) const
{
  return transformTables+index;
}

void AnimationDefinitionTable::allocValues(const int num)
{
  IndexedFaceSetSTable *tablesOld = tables;
  tables = new IndexedFaceSetSTable[num];
  for (int i=0; i<maxNum;i++) tables[i]=tablesOld[i];
  delete[] tablesOld;
  maxNum = num;
  nNodes = maxNum + maxNumTransformTables;
}

void AnimationDefinitionTable::allocValuesTransformTables(const int num)
{
  TransformSTable *tablesOld = transformTables;
  transformTables = new TransformSTable[num];
  for (int i=0; i<maxNumTransformTables;i++) transformTables[i]=tablesOld[i];
  delete[] tablesOld;
  maxNumTransformTables = num;
  nNodes = maxNum + maxNumTransformTables;
}






