/*
CONTRIBUTION TO THE WG11 PROGRAM MPEG 4 PLAYER
______________________________________________________________________

STANDARDS PROJECT:  MPEG 4 Player
______________________________________________________________________

TITLE:  Animation of 3D Face Mpdels using Animation Definition Tables
______________________________________________________________________

AUTHORS:  	Erich Haratsch, Joern Ostermann

SOURCE:      	AT&T
 
CONTACT:	Joern Ostermann
		AT&T Labs - Research
		Room NSL 3-231
		100 Schultz Drive - Middletown
		Red Bank, NJ  07701-7033
		USA
		email: osterman@research.att.com   
		fax:   +1 732 345 3033
		phone:  +1 732 345 3311		   
______________________________________________________________________

DATE:  July 14, Middletown, New Jersey, USA
______________________________________________________________________

DISTRIBUTION:  MPEG 4 WG11
______________________________________________________________________

ABSTRACT:  fapfile_anim_mpeg_release takes as input an decoded 
downloaded model and decoded AnimationDefinitionTables and animates 
the model with decoded FAP sequences


______________________________________________________________________
Notice

This document has been prepared by AT&T
to assist the Standards Committee WG 11.  It is proposed to the Committee
as a basis for discussion and is not to be construed as a binding proposal
on AT&T. AT&T specifically reserves the right to amend or modify the
material contained herein and nothing herein shall be construed as conferring
or offering licenses or rights with respect to any intellectual property of
AT&T.

AT&T Proprietary
*/
#ifndef TableIndexedFaceSet_H
#define TableIndexedFaceSet_H

/* ######################################################################################
   #                                                                                    #
   # Class for facial animation. Author: Erich Haratsch, May 1997               #
   #  --------------------------------------------------------------------------------  #
   #                                                                                    #
   #                              (c) 1997 AT&T Research                                #
   #                                                                                    #
   #                                                                                    #
   #  Library:                                                          #
   #  File:          TableIndexedFaceSet.h                                            #
   #  Class:         VetexDisplacementsInfo, STable, TransformSTable, 
                     IndexedFaceSetSTable, AnimationDefinitionTable                     #
   #  Description:                    #
   #                                                                                    #
   ###################################################################################### */
#include <string.h>
#define MAXCHAR 256
class VertexDisplacementsInfo { //class for storing the displacements for a vertex

public:

  VertexDisplacementsInfo(){_displacements = NULL; _vertexIndex = 0; _nIntervals = 0;};

  void setValues(const int vertexIndex, const int nIntervals, 
		 SbVec3f *displacements); //sets the index and the displacements for the vertex; nIntervals is the number of intervals
  int getVertexIndex() const; //return the index of the vertex
  const SbVec3f *getValues(const int index) const; //returns a pointer into the array of displacements in the field, starting at index index
  VertexDisplacementsInfo & operator = (const VertexDisplacementsInfo &v); //assignment operator
  ~VertexDisplacementsInfo(){delete[] _displacements;};

private:
  int _vertexIndex; //index of the vertex
  int _nIntervals; //number of intervals for the displacements
  SbVec3f *_displacements; //array of displacements
};


class STable //base class for TransformsTable and IndexedFaceSetSTable
{
public:
  STable(){strcpy(_name,"");};
  void setName(const char* name){strcpy(_name, name);}; //set the name
  const char *getName() const {return _name;}; //get the name
  STable & operator = (const STable &t){strcpy(_name, t._name);
                                        return *this;}; //assignment operator
private:
  
  char _name[MAXCHAR]; //name of the table
};

class TransformSTable : public STable //class for AnimationDefTransform
{

public:
  TransformSTable(){_fieldRef = -1;};
  TransformSTable &operator = (const TransformSTable &t); //assignment operator
  void setFieldRef(const int fieldRef){_fieldRef = fieldRef;}; //set the field reference
   void setRotation(const SbRotation rotation);				  
   void setTranslationOrScalefactor(const SbVec3f TranslationOrScalefactor);		   
  int getFieldRef() const{return _fieldRef;}; //get the field reference
  void getRotation(SbVec3f &axis, float &angle) const {_rotation.getValue(axis, angle);};
  void getTranslationOrScalefactor(SbVec3f &vector) const {vector.setValue(_translationOrScalefactor.getValue());};
  ~TransformSTable(){};
  
private:
  int _fieldRef;
  SbRotation _rotation;
  SbVec3f _translationOrScalefactor;
};
    

class IndexedFaceSetSTable : public STable //class for AnimationDefIndexedFaceSet
{

public:

  IndexedFaceSetSTable(const int nVertices, const int nIntervals, int *intervalBorders);
  IndexedFaceSetSTable();
  IndexedFaceSetSTable & operator = (const IndexedFaceSetSTable &t); //assignment operator
//  void setIntervalBorders(const int *intervalBorders);
  void set1VertexDisplacements(const int index, const VertexDisplacementsInfo & displacements); //set the indexth value to the given displacements
  int getNumVertices() const; //get the number of vertices, for which displacements are stored
  int getNumIntervals() const; //get the number of intervals
  const VertexDisplacementsInfo *getValue(const int index) const; //get the value of the indexth displacement
  const int *getIntervalBorders() const; //get the interval borders
  ~IndexedFaceSetSTable();


private:

  int _nVertices; //number of vertices, for which displacements are given
  int _nIntervals; //number of intervals
  int *_intervalBorders; //interval borders
  
  VertexDisplacementsInfo *_vertexDisplacementsTable; //vertex displacments

};

class AnimationDefinitionTable //class for AnimationDefinitionTable
{

public:
  AnimationDefinitionTable(){tables = NULL; maxNum = 0; transformTables=NULL;
  maxNumTransformTables = 0; nNodes = 0;};
//  void setValues(int start, int num, const IndexedFaceSetSTable *newValues);
  void set1Value(const int index, const IndexedFaceSetSTable & newValue); //set the indexth IndexFaceSetTable to have the given newValue
  void set1Value(const int index, const TransformSTable &newValue); //set the indexth TransformTable to have the indexth newValue
  const TransformSTable *getTransformSTable(const int index) const; //return a pointer into the array of TransformDefTables, starting at index
  const IndexedFaceSetSTable *getIndexedFaceSetSTable(const int index) const; //return a pointer into the array of IndexedFaceSetDefTables starting at index
  ~AnimationDefinitionTable(){delete[] tables; delete[] transformTables;};
  int getNumNodes() const {return nNodes;}; //get total number of TransformDefTables and IndexedFaceSetDefTables
  int getNumTransformTables()const { return maxNumTransformTables;}; //
  int getNumIndexedFaceSetTables()const{return maxNum;};

private:

  IndexedFaceSetSTable *tables; //array of IndexedFaceSetDefTables
  TransformSTable *transformTables; //array of TransformDefTables
  int maxNum; //number of IndexedFaceSetDefTables
  int maxNumTransformTables; //number TransformDefTables
  void allocValuesTransformTables(const int num); //allocate resources for num TransformDefTables
  void allocValues(const int num); //allocate resources for num IndexedFaceSetDefTables
  int nNodes; //total number of TransformDefTables and IndexedFaceSetDefTables
};



#endif
