/* batio.C */

/************** I/O for Body deformation tables ************************/


#include <bddef.h>

/* external */
int  index_from_body_part_sequence (char body_part[]);    /* invio.C */
void Read_bap_frames (char file[]);                       /* bapio.C */
void Read_default_displacements (char file[]);            /* invio.C */


/**********************************************************************/



/* template function for reading int/float
 * within brackets;
 * store the values in val_arr and record number of values;
 */

template <class T> void read_values_between_brackets (T val_arr[],
						      int & nvalues,
						      ifstream & inf)

{
  char   c;
  T      value;
  inf.get(c);
  while (c != '[') 
    inf.get(c);
  
  /* read integer Bap Ids */
  nvalues = 0;
  while (c != ']') {
    inf >> value;
    val_arr[nvalues] = value;
    nvalues++;
    //cout << value << '\n';
    /* skip till delimiter */
    inf.get(c);
    while (isspace(c)) inf.get(c);
  }
}


/* copy value arrays from source to destination */

template <class T> void copy_value_arrays (T dst[], T src[], int nvalues)

{
  int i;

  for (i = 0; i < nvalues; i++) 
    dst[i] = src[i];
}


/* read text until (and including) string "bapIDs" */

void read_text_until_bapIDs (ifstream & inf, char bodySceneGraphNode[])

{
  char  str[100];
  int i;

  for (i = 1; i <= 4; i++) 
    inf >> str;
  inf >> bodySceneGraphNode; /* body part name */
  for (i = 1; i <= 3; i++) {
    inf >> str;
    //cout << str << '\n';
  }
}


/* read text until (and including) string "vertexIds" 
 * beginning from "exposedField"
 */

void read_text_until_vertexIds (ifstream & inf)

{
  char  str[100];
  int i;

  for (i = 1; i <= 3; i++) {
    inf >> str;
    //cout << str << '\n';
  }
}


/* read text until (and including) string "bapCombinations" 
 * beginning from previous "exposedField"
 */

void read_text_until_bapCombinations (ifstream & inf)

{
  char  str[100];
  int i;

  for (i = 1; i <= 3; i++) {
    inf >> str;
    //cout << str << '\n';
  }
}


/* read text until (and including) string "displacements" 
 * beginning from previous "exposedField"
 */

void read_text_until_displacements (ifstream & inf)

{
  char  str[100];
  int i;

  for (i = 1; i <= 3; i++) {
    inf >> str;
    //cout << str << '\n';
  }
}


/* read Bat table */

void Bat::read_bat (ifstream & inf, char body_part_name[])

{
  char str[50];

  /* read bapIDs */
  strcpy (bodySceneGraphNode, body_part_name);
  read_values_between_brackets (Pad_bap_ids, N_bap_ids, inf);
  /* assign and store in bap_id storage */
  //cout << "number of values read " << N_bap_ids << '\n';
  bap_ids = x_alloc (int, N_bap_ids); 
  copy_value_arrays (bap_ids, Pad_bap_ids, N_bap_ids);
  /*for (int i = 0; i < N_bap_ids; i++)
    cout << bap_ids[i] << '\n';*/

  /* read vertexIds */
  read_text_until_vertexIds (inf);
  read_values_between_brackets (Pad_Vertex_ids, N_vertex_ids, inf);
  /* assign and store in vertex_id storage */
  //cout << "number of values read " << N_vertex_ids << '\n';
  vertex_ids = x_alloc (int, N_vertex_ids); 
  copy_value_arrays (vertex_ids, Pad_Vertex_ids, N_vertex_ids);

  /* read bapCombinations */
  read_text_until_bapCombinations(inf);
  read_values_between_brackets (Pad_bap_comb_entries, N_bap_combs, inf);
  if (N_bap_combs < 2) {
    cout << "ERROR (read_bat): should have at least 2 BAP keys \n";
    return;
  }
  
  /* assign and store in bap_combinations storage */
  //cout << "number of values read " << N_bap_combs << '\n';
  bap_comb_entries = x_alloc (Bap_comb_t, N_bap_combs);
  copy_value_arrays (bap_comb_entries, Pad_bap_comb_entries, N_bap_combs);
  /* actual no of combs: total/bap dimension */
  N_bap_combs = N_bap_combs/N_bap_ids; 

  /* read displacements */
  read_text_until_displacements(inf);
  read_values_between_brackets (Pad_displacements, N_displacements, inf);
  /* assign and store in displacements storage */
  //cout << "number of values read " << N_displacements << '\n';
  displacements = x_alloc (float, N_displacements);
  copy_value_arrays (displacements, Pad_displacements, N_displacements);

  /* read terminating brace */
  inf >> str;

  if ( (3*N_vertex_ids*N_bap_combs != N_displacements) ) {
    cout << "ERROR (read_bat): BAT table incorrectly read\n";
    cout.flush();
  }

  /* derive other quantities */
  derive_bap_comb_cg ();
  derive_bap_comb_vectors ();
}


void Bat::read_all_bats (ifstream & inf, char body_part_name[])

{
  if (N_bap_ids > 0) {
    cout << "WARNING (read_bats): BAT table for body part '" 
	 << body_part_name << "' already read; creating another NEW table\n";
    /* create new BAT table and read data into it */
    Bat *bat = new Bat;
    bat->read_bat (inf, body_part_name);
  }
  else
    /* read data into current BAT table */
    read_bat (inf, body_part_name);
}


void Read_bat_tables (char file[])

{
  ifstream  inf; /* input file stream */
  int       bap_no = 0;
  char      str[50], body_part_name[100];

  inf.open(file);
  
  if (!inf) {
    cout << "ERROR (Read_bat_tables): can not open " << file << '\n';
    cout.flush();
  }
  /* while eof not reached, read all Bat tables */
  while (!inf.eof()) {
    /* try to read first string (expected to be "BodyDefTables") */
    inf >> str;
    if (inf.eof()) break;
    /* read until bapIDs to read body part name */
    read_text_until_bapIDs (inf, body_part_name);
    //cout << "Body part : " << body_part_name << '\n';
    /* get body part index in array */
    bap_no = index_from_body_part_sequence (body_part_name);
    if (bap_no < 0) 
      cout << "Body part '" << body_part_name <<"' does not exist\n";
    /* read bat table for this body part */
    Bap_arr[bap_no].read_all_bats (inf, body_part_name);
  }
  inf.close();
}


main (int argc, char *argv[])

{
  Bat  bat;

  /* read bat tables */
  Read_bat_tables ("batfile");
  /* read default displacements */
  Read_default_displacements ("body/naomi_smart.inf.default.iv");
  /* read animation file (bap) and produce deformations */
  if (argc > 1) /* read frame no for which deformation needed */
    sscanf (argv[1], "%d", &Deformation_frame_no);
  else {
    cout << "usage: " << argv[0] << " <frame no>\n";
    Deformation_frame_no = 0; /* default */
    cout << "default frame number " << Deformation_frame_no << '\n';
  }
  Read_bap_frames ("bapfile");
}

