/* bddef.h */

/********************** body deformations (header) ***********************/

#include <iostream.h>
#include <fstream.h>
#include <ctype.h>
#include <malloc.h>

#ifndef  EXTERN
#define  EXTERN extern
#endif

#define  TRUE                      1
#define  FALSE                     0
#define  LARGE_INT                 1000000/* large integer value */
#define  MAX_BUF                   1000   /* max buffer size */
#define  MAX_BODY_PARTS            16     /* max body animation parts */
#define  MAX_BAP_NAME_LEN          30     /* max length of body part name */
#define  MAX_BAPIDS                10     /* max no of bap ids for tables */
#define  MAX_VERTEX_IDS            5000
#define  MAX_BAP_COMBS             10
#define  MAX_BAP_COMB_ENTRIES      100    /* should be 
					     MAX_BAPIDS*MAX_BAP_COMBS */
#define  MAX_DISPLACEMENTS         150000 /* should be maximum of 
					     3*MAX_VERTEX_IDS*MAX_BAP_COMBS and
					     4*MAX_VERTEX_IDS */
#define  N_BAP_IDS                 186    /* total number of MPEG4 BAP ids */ 

#define  x_alloc(t, q) ((t *) calloc (q, sizeof (t)));
                                          /* generic memory allocator */
typedef  long   Bap_comb_t;               /* bap combination type */
typedef  float  Bap_vector_t;             /* bap vector type */

/* define scratch pad variables */
EXTERN int      Pad_bap_ids[ MAX_BAPIDS];        
                                          /* stores bapID input */
EXTERN int      Pad_Vertex_ids[MAX_VERTEX_IDS];  
                                          /* stores vertexId input */
EXTERN Bap_comb_t      
                Pad_bap_comb_entries[MAX_BAP_COMB_ENTRIES];    
                                          /* stores bapCombinations input */
EXTERN float    Pad_displacements[MAX_DISPLACEMENTS];
                                          /* stores displacement input */
EXTERN char     Pad_buf[MAX_BUF];         /* scratch pad buffer */

/* other variables */
EXTERN int      Deformation_frame_no;     /* no of frame for which deforamation
					     needed; default 0 */
EXTERN int      No_of_bat_keys_to_interpolate;
                                          /* no of bat KEY combinations 
					     considered to get displacements 
					     at a given combination;
					     default 1 */
EXTERN Bap_comb_t 
                Prv_frame_bap_points[N_BAP_IDS];        
                                          /* all Bap point values
					     from previous frame */

static char     Body_part_sequence[MAX_BODY_PARTS][MAX_BAP_NAME_LEN] = {
                        "pelvis", "l5", "c1", "r_thigh", "r_calf",
			"l_thigh", "l_calf", "r_upperarm", "r_forearm",
			"l_upperarm", "l_forerarm"};
                                          /* sequence of part names whose 
					     values appear in .iv files */

/* Body animation table: Base class */

class Bat {
  /*---- private/protected area -----*/
  Bat    *next;                           /* reference to next BAT table of
					     same body part */
  void   add_bat_displacements (int n, float total_disp[], float bat_disp[]);
                                          /* add this BAT's displacements to 
					     total displacement */
  void   deform_bap_using_bat (int n, float total_disp[], Bap_comb_t p[]);
                                          /* deform body part using this 
					     BAT table at given BAP point */
  void   read_bat (ifstream & inf, char body_part_name[]);       
                                          /* read SINGLE body animation table*/
  /* data read from file */
protected:
  char   bodySceneGraphNode[MAX_BAP_NAME_LEN];
                                          /* body part name */
  int    N_bap_ids;                       /* number of bapIDs */
  int    N_vertex_ids;                    /* number of vertexIds */
  int    N_bap_combs;                     /* number of key bapCombinations */
  int    N_displacements;                 /* number of displacements:
					     3*N_vertex_ids*N_bap_combs */
  int   *bap_ids, *vertex_ids;            /* storage */
  Bap_comb_t *bap_comb_entries;           /* sequential list of bap key
					     combinations */
  float *displacements;                   /* displacements w.r.t normal */
  /* data derived */
  Bap_comb_t   *bap_comb_cg;              /* center of gravity of 
					     bap combinations (UNUSED) */
  Bap_vector_t **bap_comb_v;              /* bap combination vectors */
  void   derive_bap_comb_cg();            /* derive cg of bap combinations */
  void   derive_bap_comb_vectors();       /* derive bap combination vectors */
  void   closest_points_to_bap_point (Bap_comb_t p[], 
				      int &n, int v[], float dist[]);
                                          /* get bap points closest to a given
					     point using bap vectors */
public:
  void   read_all_bats (ifstream & inf, char body_part_name[]);       
                                          /* read body animation tables for
					     this body part
					   */
  void   deform_bap_using_all_bats (int n, float total_disp[], Bap_comb_t p[]);
                                          /* deform body part using all its
					     BAT tables  at given BAP point and
					     create total displacement */
  Bat ();
  ~Bat();
};


/* Body animation part: derived from base class Bat */

class Bap: public Bat {
  /* private/protected values */
  int    Bap_no;                          /* body part number */
  int    N_vertices;                      /* number of vertices in body part */
  int    N_polygons;                      /* number of polygons in body part */
  char   bap_transf[MAX_BUF];             /* BAP transformation info
					     (valid in cur frame only) */
  float *normal_displacements;            /* normal/default displacements:
					     3*N_vertices */
  float *total_displacements;             /* total displacement:
					     normal + BAT displacement */
  int   *vertex_connectivity;             /* specifies connectivity among
					     above displacements */
  void   generate_total_bap_displacement_UNUSED (float bat_disp[]);
                                          /* generate total deformation: 
					     bat + normal displacement */
public:
  void   read_normal_bap_displacements (ifstream & inf, int bap_no);
                                          /* read default displacements 
					     from a file */
  void   extract_bap_transformation_from_file_iv (ifstream & inf);
                                          /* read transformation of BAP from 
					     inventor file */
  void   deform_bap_UNUSED (Bap_comb_t comb[]);  
                                          /* produce deformation for
					     given bap combination */
  void   bap_deformation (unsigned char active_bap_flags[], 
			  Bap_comb_t bap_values[]);
                                          /* animate body parts using active 
					     bap ids indicated by flags and 
					     corresponding values */
  void   output_bap ();                   /* output BAP on screen */
  void   output_bap_iv (ofstream & ouf);  /* output BAP in inventor file */
  Bap ();
  ~Bap ();
};


/* overloading i/o operators */

ostream  &operator<<(ostream &,  Bap &);
ofstream &operator<<(ofstream &, Bap &);

EXTERN Bap      Bap_arr[MAX_BODY_PARTS];  /* array of body animation parts */
