/************************************************************************
   This software module was originally developed by

   Hai Tao    (tao@ifp.uiuc.edu), University of Illinois at Urbana-Champaign
   Homer Chen (homer@risc.rockwell.com), Rockwell International Corporation
   (date: October, 1997)

in the course of development of the MPEG-4 Video (ISO/IEC 14496-2).
This software module is an implementation of a part of one or more
MPEG-4 Video tools as specified by the MPEG-4 Video. ISO/IEC gives users
of the MPEG-4 Video free license to this software module or
modifications thereof for use in hardware or software products claiming
conformance to the MPEG-4 Video.  Those intending to use this software
module in hardware or software products are advised that its use may
infringe existing patents.  The original developer of this software
module and his/her company, the subsequent editors and their companies,
and ISO/IEC have no liability for use of this software module or
modifications thereof in an implementation. Copyright is not released
for non MPEG-4 Video conforming products. UIUC/Rockwell retains full right
to use the code for his/her own purpose, assign or donate the code to a
third party and to inhibit third parties from using the code for non
MPEG-4 Video conforming products. This copyright notice must be
included in all copies or derivative works.

Copyright (c) 1996, 1997.
***********************************************************************/


#include <stdio.h>
#include <stdlib.h>
#include <math.h>
#include "util.h"
#include "dctencode1D.h"


BYTE sc[24] = {0,0,0,0, 0,0,0,0,
	       0,0,0,0, 0,0,0,0,
	       0,0,0,0, 0,0,0,1,
};
/* fba_sequence_start_code */
BYTE fss[8] = {0,0,0,0, 0,0,0,1,};
/* fba_sequence_end_code */
BYTE fse[8] = {0,0,0,0, 0,0,1,0,};
/* fba_segment_start_code */
BYTE fsegs[8] = {0,0,0,0, 0,0,1,1,};

int mask_index[11] = {-1, 1, 17, 29, 37, 41, 46, 49, 59, 63, 67};


/*#define M_DEBUG 1
 */


int code_viseme(BYTE intra, int *sel1, int *sel2,int *blend, int *vis_prev,
		HUFFMAN_TABLE *vis_sel_table,
		HUFFMAN_TABLE *vis_blend_table,
		FAP_STREAM_MEM *stream);
int code_expression(BYTE intra, int *sel1, int *sel2,
		    int *intensity1,int *intensity2,int *exp_prev,
		    HUFFMAN_TABLE *exp_sel_table,
		    HUFFMAN_TABLE *exp_intensity_table,
		    FAP_STREAM_MEM *stream);

main(int argc, char **argv) {

  int i, j, k, n,m,fr, tmpi, frm_no;
  int mask[NGROUP];
  int  nframe, data_dim,total_bit, no_of_segment;
  float data[NFAP][SEG_LENGTH], tmpdata[SEG_LENGTH][NFAP], version;
  int f_mask[NFAP],tmpf_mask[SEG_LENGTH][NFAP],lf_mask[NFAP];
  char basename[12];
  BYTE tmpb,is_intra, frameRate, tc_hour, tc_min, marker_bit, tc_sec, fap_quant;
  FILE *fap_fp, *stream_fp, *out_fp;
  int prev_dc[NFAP];
  int expression_select1[SEG_LENGTH], expression_select2[SEG_LENGTH], 
    expression_intensity1[SEG_LENGTH], expression_intensity2[SEG_LENGTH], expression_prev[4];
  int viseme_select1[SEG_LENGTH], viseme_select2[SEG_LENGTH], 
    viseme_blend[SEG_LENGTH], viseme_prev[3];
  /* the HUFFMAN table for DCT DC coefficients coding */
  HUFFMAN_TABLE  dc_table;
  /* the HUFFMAN table for position run-length coding*/
  HUFFMAN_TABLE  runlength_table;
  /* the HUFFMAN table for DCT AC coefficients coding */
  HUFFMAN_TABLE  ac_table;
  /* the HUFFMAN table for viseme_select1/2*/
  HUFFMAN_TABLE  v_sel_table;
  /* the HUFFMAN table for viseme_blend */
  HUFFMAN_TABLE  v_blend_table;
  /* the HUFFMAN table for expression_select1/2*/
  HUFFMAN_TABLE  e_sel_table;
  /* the HUFFMAN table for viseme_intensity1/2 */
  HUFFMAN_TABLE  e_intensity_table;
  
  /* the out put stream */
  FAP_STREAM_MEM out_stream;

  
  if(argc != 4) {
    fprintf(stderr, "Usage: %s input_fap output_stream fap_quant\n", argv[0]);
    exit(-1);
  }

  total_bit = 0;
  if ( (fap_fp = fopen(argv[1], "r")) == NULL) {
    printf("The file %s can not be open !\n", argv[1]);
    exit(1);
  }
  
  if (( stream_fp = fopen(argv[2], "wb")) == NULL) {
    printf("The file %s can not be open !\n", argv[2]);
    exit(1);
  }
   
  fap_quant = (BYTE)(atoi(argv[3]));
  if (fap_quant <1 || fap_quant >31) {
    printf("fap_quant must be a integer in rage [1, 31]\n");
    exit(1);
  }
  
  /* Read Header Information */
  nextValidLine(fap_fp);
  fscanf(fap_fp, "%f", &version);
  fscanf(fap_fp, "%s", basename);
  fscanf(fap_fp, "%d", &tmpi);
  frameRate = (BYTE)(tmpi);
  fscanf(fap_fp, "%d", &nframe);
   
  /* read mask */
  if (version <1.5) {
    for(i=0; i<NGROUP; i++)
      fscanf(fap_fp, "%d", &tmpi);
    
    finishLine(fap_fp);
    for (i=0; i<NFAP; i++){
      fscanf(fap_fp, "%d", &f_mask[i]);
    }
  }
  finishLine(fap_fp);

#ifdef M_DEBUG
  printf("snhc vm%2.1f\n", version);
  printf("%s Sequence\n", basename);
  printf("Frame rate: %d\n", frameRate);
  printf("Sequence length: %d frames\n", nframe);

  if(version < 1.5) {
    printf("FAP group mask:\n");
    for(i=0; i<NGROUP; i++)
      printf("%d ", mask[i]);
    printf("\n");
  } 
#endif
  printf("%d ", fap_quant);
  
  /* out stream initialization */
  output_stream_init(&out_stream);

  /* output fba_sequence_start_code (fss), byte aligned */
  total_bit += output_stream_aligned(&out_stream);
  total_bit += output_stream_bits(sc, 24, &out_stream);
  total_bit += output_stream_bits(fss, 8, &out_stream);
  /* the following nframe is for FAP format, in the formal
     implementation, there should be no such bits */
  total_bit +=   output_stream_ushort((unsigned short)nframe, &out_stream);

 
  /* initialize DCT transform */
  init_dct(SEG_LENGTH);

  /* initialize Huffman tables*/
  for (i=0; i<512; i++)
    runlength_table.freq[i] = ac_table.freq[i]= dc_table.freq[i] =0;
  /* read in Huffman tables from files */
  assign_dc_ac_runlength_vis_exp_table(&dc_table, &ac_table, &runlength_table,
				       &v_sel_table, &v_blend_table, 
				       &e_sel_table, &e_intensity_table);
  
  /* find the number of segment */
  no_of_segment = nframe/SEG_LENGTH;
  if (nframe % SEG_LENGTH != 0) no_of_segment++;
 
   /* for each segment */
  for (n=0; n<no_of_segment; n++) {
    /* fba_paramset_mask, 2 bits,  '01' */
    total_bit +=   output_stream_bit(0, &out_stream);
    total_bit +=   output_stream_bit(1, &out_stream);

    /* find the real length of this segment -> frm_no */
    if (n != no_of_segment-1) {
      frm_no = SEG_LENGTH;
    }
    else {
      frm_no = nframe % SEG_LENGTH;
      if (frm_no == 0) frm_no =SEG_LENGTH;
    } 
    
    /* read one segment of mask and data -> f_mask[NFAP], 
       data_dim, data[data_dim][SEG_LENGTH]*/
    if (version > 1.5) {
      for (i=0; i<NFAP; i++)  f_mask[i]=0;
      for (fr=0; fr<frm_no; fr++){
	/* skip line start with '#' */
	nextValidLine(fap_fp);
	
	/* read in mask of this frame  -> tmpf_mask */
	for (i=0; i<NFAP; i++) {
	  fscanf(fap_fp, "%d", &tmpf_mask[fr][i]);
	  /* f_mask is the "OR" of all tmpf_mask */
	  if (tmpf_mask[fr][i]) f_mask[i] = 1; 
	}
	
	/* find how many valid FAP in this frame */
	data_dim = 0;
	for (i=2; i<NFAP; i++)
	  if (tmpf_mask[fr][i]) data_dim++;
	
	/* read the frame number */
	fscanf(fap_fp, "%d", &tmpi);
	/* read the visemes and expressions */
	if(tmpf_mask[0]) { /* there is viseme */
	  /* read to viseme_select1[fr], 
	     viseme_select2[fr],
	     viseme_blend[fr] */
	  /* I am not sure about the file format, so just put them as 0s */
	   viseme_select1[fr] = 0;
	   viseme_select2[fr] = 0;
	   viseme_blend [fr] = 0;
	}
	if(tmpf_mask[1]) { /* there is expression */
	  /* read to expression_select1[fr], 
	     expression_select2[fr],
	     expression_intensity1[fr] 
	     expression_intensity2[fr] */
	  /* I am not sure about the file format, so just put them as 0s */
	   expression_select1[fr] = 0;
	   expression_select2[fr] = 0;
	   expression_intensity1[fr] = 0;
	   expression_intensity2[fr] = 0;
	}
	/* read FAP data to tmpdata[][] */
	for (i=0; i<data_dim; i++) {
	  fscanf(fap_fp, "%d", &tmpi);
	  tmpdata[fr][i] = (float)tmpi;
	}
	
	/* read the rest of the line */
	finishLine(fap_fp);
      }
      
      /* for visemes and expressions */
      for (fr=0; fr<frm_no; fr++) {
	if (f_mask[0] && !tmpf_mask[0]) {
	  viseme_select1[fr] = 0;
	  viseme_select2[fr] = 0;
	  viseme_blend [fr] = 0;
	}
	if (f_mask[1] && !tmpf_mask[1]) {
	  expression_select1[fr] = 0;
	  expression_select2[fr] = 0;
	  expression_intensity1[fr] = 0;
	  expression_intensity2[fr] = 0;
	}
	
      }
      /* find how many valid FAPs appear in this segment */
      data_dim = 0;
      for (i=2; i<NFAP; i++)
	if (f_mask[i]) data_dim++;
      /* find in data[][] with tmpdata[][] */ 
      for (fr=0; fr<frm_no; fr++) {
	k=0; m=0;
	for (i=0; i<NFAP; i++) 
	  if (f_mask[i]) {
	    if (tmpf_mask[fr][i]) {
	      data[k][fr] = tmpdata[fr][m];
	      m++, k++;
	    }
	    else {
	      data[k++][fr] =0;
	    }
	  }
      }
    }
    else {  /* version 1.1 */
      /* find how many valid FAPs in this frame */
      data_dim = 0;
      for (i=2; i<NFAP; i++)
	if (f_mask[i]) data_dim++;
      /* for each frame */
      for (fr = 0; fr <frm_no; fr ++) {
	/* read in frame number */
	fscanf(fap_fp, "%d", &tmpi);
	/* read viseme and expression to */
	if(f_mask[0]) { /* there is viseme */
	  /* read to viseme_select1[fr], 
	     viseme_select2[fr],
	     viseme_blend[fr] */
	  /* I am not sure about the file format, so just put them as 0s */
	   viseme_select1[fr] = 0;
	   viseme_select2[fr] = 0;
	   viseme_blend [fr] = 0;
	}
	if(f_mask[1]) { /* there is expression */
	  /* read to expression_select1[fr], 
	     expression_select2[fr],
	     expression_intensity1[fr] 
	     expression_intensity2[fr] */
	  /* I am not sure about the file format, so just put them as 0s */
	   expression_select1[fr] = 0;
	   expression_select2[fr] = 0;
	   expression_intensity1[fr] = 0;
	   expression_intensity2[fr] = 0;
	}
	/* read data to data[][] */
	for (j=0; j<data_dim; j++) {
	  fscanf(fap_fp, "%d", &tmpi);
	  data[j][fr] = (float)tmpi;
	}
      }
    }
    /* if number of frames in this segment is less than 
       SEG_LENGTH, pad it with 0 */
    if (n ==  no_of_segment - 1) {
      for (fr=frm_no; fr<SEG_LENGTH; fr++) {
	/* for viseme and expression */
	if(f_mask[0]) { /* there is viseme */
	  viseme_select1[fr] = 0;
	   viseme_select2[fr] = 0;
	   viseme_blend [fr] = 0;
	}
	if(f_mask[1]) { /* there is expression */
	  expression_select1[fr] = 0;
	  expression_select2[fr] = 0;
	  expression_intensity1[fr] = 0;
	  expression_intensity2[fr] = 0;
	}
	/* for other FAPs */
	for (j=0; j<data_dim; j++)
	  data[j][fr] = 0;
      }
    }
    
    /* store the mask bit for each segment */
    if (n == 0){
      is_intra = 1;
    }
    else {
      /* if it is not the first segment, out put bit '0' if the mask 
	 doesn't change, or bit '1' plus mask if the mask changes */
      tmpi = 0;
      for (i=0; i<NFAP; i++)
	if (f_mask[i] != lf_mask[i]) {
	  tmpi= 1;
	  break;
	}
      if (tmpi)
	is_intra = 1;
      else 
	is_intra = 0;
    }     
    /* store current mask for future use */
    for (i=0; i<NFAP; i++)
      lf_mask[i] = f_mask[i];
    
    /* is_intra, 1 bit*/
    total_bit +=   output_stream_bit(is_intra, &out_stream);
    /* is_last_segment, 1 bits*/
    if (n== no_of_segment-1)
      total_bit +=   output_stream_bit(1, &out_stream);
    else
      total_bit +=   output_stream_bit(0, &out_stream);
    
    /* if is last segment */
    if (n== no_of_segment-1)
      total_bit += output_stream_bits_from_byte((BYTE)(frm_no-1), 4,&out_stream);
    /* if is_intra */
    if (is_intra) {
      /* fap_segment_start_code */
      total_bit += output_stream_bits(sc, 24, &out_stream);
      total_bit += output_stream_bits(fsegs, 8, &out_stream);
      /* is_frame_rate */
      total_bit += output_stream_bit(1, &out_stream);
      /* if (is_frame_rate) */
      /* frame_rate */
      total_bit += output_stream_bits_from_byte((BYTE)(frameRate), 8,&out_stream);
     
      /* seconds 4 bits*/
      for (i=0; i<4; i++)
	total_bit += output_stream_bit(0, &out_stream);
      /* frequency_offset 1 bit */
      total_bit += output_stream_bit(0, &out_stream);
      /* is_time_code = 1 */
      total_bit += output_stream_bit(1, &out_stream);
      tc_hour = 0; tc_min = 0; marker_bit = 0; tc_sec = 0;
     
      total_bit += output_stream_bits_from_byte((BYTE)(tc_hour), 5,&out_stream);
      total_bit += output_stream_bits_from_byte((BYTE)(tc_min), 6,&out_stream);
      total_bit += output_stream_bit(0, &out_stream);
      total_bit += output_stream_bits_from_byte((BYTE)(tc_sec), 6,&out_stream);
     
    }
    /* skip_segment 1bit, no skip */
    total_bit += output_stream_bit(0, &out_stream);
    /* if (fba_paramset_mask & 1) */
    /* if (is_intra) */
    if (is_intra) {
      for (i=0; i<NGROUP; i++) {
	/* fap_mask_type = '01' */
	total_bit += output_stream_bit(0, &out_stream);
	total_bit += output_stream_bit(1, &out_stream);
	for (j=mask_index[i]+1; j<= mask_index[i+1]; j++)
	  total_bit += output_stream_bit((BYTE)(f_mask[j]), &out_stream);
      }
      /* fap_quant */
      total_bit += output_stream_bits_from_byte((BYTE)(fap_quant), 5,&out_stream);
    }
    /* for each FAP, perform DCT coding */
    k=0;
    for (j=0; j<NFAP; j++) {
      if (f_mask[j]) {
	if (j<=1) {
	  switch(j) {
	  case 0: 
	    total_bit += code_viseme(is_intra, viseme_select1, viseme_select2, viseme_blend, viseme_prev,
				     &v_sel_table,
				     &v_blend_table,
				     &out_stream);
	    break;
	  case 1: 
	    total_bit += code_expression(is_intra, expression_select1,expression_select2,
					 expression_intensity1, expression_intensity2, expression_prev,
					 &e_sel_table,
					 &e_intensity_table,
					 &out_stream);
	    break;
	  }
	}
	else {
	  total_bit += dctencode1seg(data[k], is_intra,
				     (float)(MSCALE*fap_quant*DCTQP[j]), 
				     &prev_dc[k],
				     &dc_table, &runlength_table, 
				     &ac_table, &out_stream);
	}
	/*if (n == no_of_segment-1 && j == 49)
	  printf("      used bits = %d\n", out_stream.byte_count*8+out_stream.bit_count);;
	  */
	k++;
      }

    }
   
    /*if (n%10 == 9 )
      output_stream_flush(stream_fp, &out_stream);
      */
    /*printf("total_bit = %d  ", out_stream.byte_count*8+out_stream.bit_count);;
     */
  }
  /* output fba_sequence_end_code (fse), byte aligned with next_start_code */
  if (out_stream.bit_count != 0) {
    total_bit += output_stream_bit(0, &out_stream);
    if (out_stream.bit_count != 0) {
      tmpi  = out_stream.bit_count;
      for (i=0; i< 8-tmpi; i++)
	total_bit += output_stream_bit(1, &out_stream);
    }
 
  } 
  
  total_bit += output_stream_bits(sc, 24, &out_stream);
  total_bit += output_stream_bits(fse, 8, &out_stream);
  output_stream_flush(stream_fp, &out_stream);
  
  /* stream file close */
  fclose(stream_fp);
  fclose(fap_fp);  
  printf("    %d  ", total_bit);

  /* print huffman table */
  /*  printf("The dc_table:\n");
  for (i=0; i<512; i++)
    printf(" %7.5f ", (float)(dc_table.freq[i])/dc_table.total_freq);
  printf("\n total freq = %d\n", dc_table.total_freq);

  printf("The ac_table:\n");
  for (i=0; i<512; i++)
    printf(" %7.5f ", (float)(ac_table.freq[i])/ac_table.total_freq);
  printf("\n total freq = %d\n", ac_table.total_freq);
  printf("\n\nThe runlength_table:\n");
  for (i=0; i<DCT_LENGTH; i++)
    printf(" %7.5f ", (float)(runlength_table.freq[i])/
	   runlength_table.total_freq );
  printf("\n total freq = %d\n", runlength_table.total_freq);
  */

}


int code_viseme(BYTE intra, int *sel1, int *sel2,int *blend, int *vis_prev,
		HUFFMAN_TABLE *vis_sel_table,
		HUFFMAN_TABLE *vis_blend_table,
		FAP_STREAM_MEM *stream)
{
  int bit_c = 0;
  int i, start_index, diff;
  if (intra){
    /* intra segment */
    /* first frame */
    bit_c += output_stream_bits_from_byte((BYTE)(sel1[0]), 4, stream);
    bit_c += output_stream_bits_from_byte((BYTE)(sel2[0]), 4, stream);
    bit_c += output_stream_bits_from_byte((BYTE)(blend[0]), 6, stream);
    
    vis_prev[0] = sel1[0];
    vis_prev[1] = sel2[0];
    vis_prev[2] = blend[0];
    start_index = 1;
  }
  else start_index = 0;
  for (i=start_index; i<SEG_LENGTH; i++) {
    diff = sel1[i] - vis_prev[0];
    bit_c += output_stream_huffman_symbol(diff+14,vis_sel_table, stream);
    vis_prev[0] = sel1[i];
    
    diff = sel2[i] - vis_prev[1];
    bit_c += output_stream_huffman_symbol(diff+14,vis_sel_table, stream);
    vis_prev[1] = sel2[i];
    
    diff = blend[i] - vis_prev[2];
    bit_c += output_stream_huffman_symbol(diff+63,vis_blend_table, stream);
    vis_prev[1] = sel2[i];
  }
  /* viseme_def 1bit = 0 */
  bit_c +=   output_stream_bit(0, stream);
  return(bit_c);
}
int code_expression(BYTE intra, int *sel1, int *sel2,
		    int *intensity1,int *intensity2,int *exp_prev,
		    HUFFMAN_TABLE *exp_sel_table,
		    HUFFMAN_TABLE *exp_intensity_table,
		    FAP_STREAM_MEM *stream)
{
  int bit_c = 0;
  int i, start_index, diff;
  if (intra){
    /* intra segment */
    /* first frame */
    bit_c += output_stream_bits_from_byte((BYTE)(sel1[0]), 3, stream);
    bit_c += output_stream_bits_from_byte((BYTE)(sel2[0]), 3, stream);
    bit_c += output_stream_bits_from_byte((BYTE)(intensity1[0]), 6, stream);
    bit_c += output_stream_bits_from_byte((BYTE)(intensity2[0]), 6, stream);
    
    exp_prev[0] = sel1[0];
    exp_prev[1] = sel2[0];
    exp_prev[2] = intensity1[0];
    exp_prev[3] = intensity2[0];
    start_index = 1;
  }
  else start_index = 0;
  
  for (i=start_index; i<SEG_LENGTH; i++) {
    diff = sel1[i] - exp_prev[0];
    bit_c += output_stream_huffman_symbol(diff+6,exp_sel_table, stream);
    exp_prev[0] = sel1[i];
    
    diff = sel2[i] - exp_prev[1];
    bit_c += output_stream_huffman_symbol(diff+6,exp_sel_table, stream);
    exp_prev[1] = sel2[i];
    
    diff = intensity1[i] - exp_prev[2];
    bit_c += output_stream_huffman_symbol(diff+63,exp_intensity_table, stream);
    exp_prev[2] = intensity1[i];
    
    diff = intensity2[i] - exp_prev[3];
    bit_c += output_stream_huffman_symbol(diff+63,exp_intensity_table, stream);
    exp_prev[3] = intensity2[i];
    
  }
  /* expression_def 1bit = 0 */
  bit_c +=   output_stream_bit(0, stream);
  return(bit_c);
}
