/************************************************************************
   This software module was originally developed by

   Hai Tao    (tao@ifp.uiuc.edu), University of Illinois at Urbana-Champaign
   Homer Chen (homer@risc.rockwell.com), Rockwell International Corporation
   (date: October, 1997)

in the course of development of the MPEG-4 Video (ISO/IEC 14496-2).
This software module is an implementation of a part of one or more
MPEG-4 Video tools as specified by the MPEG-4 Video. ISO/IEC gives users
of the MPEG-4 Video free license to this software module or
modifications thereof for use in hardware or software products claiming
conformance to the MPEG-4 Video.  Those intending to use this software
module in hardware or software products are advised that its use may
infringe existing patents.  The original developer of this software
module and his/her company, the subsequent editors and their companies,
and ISO/IEC have no liability for use of this software module or
modifications thereof in an implementation. Copyright is not released
for non MPEG-4 Video conforming products. UIUC/Rockwell retains full right
to use the code for his/her own purpose, assign or donate the code to a
third party and to inhibit third parties from using the code for non
MPEG-4 Video conforming products. This copyright notice must be
included in all copies or derivative works.

Copyright (c) 1996, 1997.
***********************************************************************/

#include <stdio.h>
#include <stdlib.h>
#include <math.h>
#include <sys/stat.h>
#include "util.h"


/*#define M_DEBUG 1
 */

int mask_index[11] = {-1, 1, 17, 29, 37, 41, 46, 49, 59, 63, 67};

void decode_viseme(BYTE intra, int *sel1, int *sel2,int *blend, int *vis_prev,
		   HUFFMAN_TABLE *vis_sel_table,
		   HUFFMAN_TABLE *vis_blend_table,
		   FAP_STREAM_MEM *stream);

void decode_expression(BYTE intra, int *sel1, int *sel2,
		       int *intensity1, int *intensity2, int *exp_prev,
		       HUFFMAN_TABLE *exp_sel_table,
		       HUFFMAN_TABLE *exp_intensity_table,
		       FAP_STREAM_MEM *stream);

main(int argc, char **argv) {

  int i, j, n,m, tmpi, leap, last_dc[NFAP], frm_no, f_mask[NFAP];
  int skip_num,frameRate, nframe,total_bit, no_of_segment;
  int remain_file_length;
  float data[NFAP][SEG_LENGTH], quantization, qdc;
  FILE *stream_fp, *out_fp;
  BYTE fba_paramset_mask, is_intra,is_last_segment,tmpb,tmpbytes[100];
  BYTE is_frame_rate, is_time_code, skip_segments, fap_quant;
  int tmps;
  float  coeff_sequence[SEG_LENGTH], output_sequence[SEG_LENGTH];
  int expression_select1[SEG_LENGTH], expression_select2[SEG_LENGTH], 
    expression_intensity1[SEG_LENGTH], expression_intensity2[SEG_LENGTH], expression_prev[4];
  int viseme_select1[SEG_LENGTH], viseme_select2[SEG_LENGTH], 
    viseme_blend[SEG_LENGTH], viseme_prev[3];
  /* the HUFFMAN table for DCT DC coefficients coding */
  HUFFMAN_TABLE  dc_table;
  /* the HUFFMAN table for position run-length coding*/
  HUFFMAN_TABLE  runlength_table;
  /* the HUFFMAN table for DCT AC coefficients coding */
  HUFFMAN_TABLE  ac_table;
  /* the HUFFMAN table for viseme_select1/2*/
  HUFFMAN_TABLE  v_sel_table;
  /* the HUFFMAN table for viseme_blend */
  HUFFMAN_TABLE  v_blend_table;
  /* the HUFFMAN table for expression_select1/2*/
  HUFFMAN_TABLE  e_sel_table;
  /* the HUFFMAN table for viseme_intensity1/2 */
  HUFFMAN_TABLE  e_intensity_table;
 
  /* the out put stream */
  FAP_STREAM_MEM in_stream;
  struct stat stbuf;


  if(argc != 3) {
    fprintf(stderr, "Usage: %s input_stream output_fap\n", argv[0]);
    exit(-1);
  }
  if (stat(argv[1],&stbuf) == -1) {
    printf("The file %s can not be found !\n", argv[1]);
    exit(1);
  }
  remain_file_length = stbuf.st_size;
  if (( stream_fp = fopen(argv[1], "r")) == NULL) {
    printf("The file %s can not be open !\n", argv[1]);
    exit(1);
  }
  
  if ( (out_fp = fopen(argv[2], "w")) == NULL) {
    printf("The file %s can not be open !\n", argv[2]);
    exit(1);
  }
  /* init, read in some data */
  input_stream_init(&in_stream);
  /* aligned and input some data to memory */
  input_stream_aligned(&in_stream);
  input_stream_load(stream_fp, &remain_file_length, &in_stream);
    
  /* input fap_sequence_start_code */
  input_stream_bits(tmpbytes, 32, &in_stream);
  

  /* the following nframe is for FAP format, in the formal
     implementation, there should be no such bits */
  nframe    = input_stream_ushort(&in_stream);
 

 
#ifdef M_DEBUG
  printf("Basename: rec\n");
  printf("Frame rate: %d\n", frameRate);
  printf("Sequence length: %d frames\n", nframe);
  printf("FAP_quant: %d\n", fap_quant);
  printf("FAP mask:\n");
  printf("\n");
#endif

  for (i=0; i<512; i++)
    runlength_table.freq[i] = ac_table.freq[i]= dc_table.freq[i] =0;
  /* read in Huffman tables from files */
  assign_dc_ac_runlength_vis_exp_table(&dc_table, &ac_table, &runlength_table,
				       &v_sel_table, &v_blend_table, 
				       &e_sel_table, &e_intensity_table);
  
  /* initialize DCT transform */
  init_dct(SEG_LENGTH);
  
  /*no_of_segment = nframe/SEG_LENGTH;
  if (nframe % SEG_LENGTH != 0) no_of_segment++;
  */

  /* initialize the values for DC */
  for (j=0; j<NFAP; j++) last_dc[j] = 0;
  
  is_last_segment = 0;
  n = 0;
  while(!is_last_segment) {
    /* read fba_paramset_mask, 2 bits */
    input_stream_bits(tmpbytes, 2, &in_stream);
    if (tmpbytes[0] == 0 || tmpbytes[0] == 1)
      fba_paramset_mask = 1;
    else fba_paramset_mask = 0;
    /* read in is_intra, 1 bit */
    is_intra = input_stream_bit(&in_stream);
    /* read in is_last_segment, 1 bit */
    is_last_segment = input_stream_bit(&in_stream);
    /* if is_last_segment */
    if (is_last_segment) {
      input_stream_bits(tmpbytes, 4, &in_stream);
      frm_no = tmpbytes[0]*8 +tmpbytes[1]*4 +tmpbytes[2]*2 + 
	tmpbytes[3];
      frm_no++;
    }
    else 
      frm_no = SEG_LENGTH;
    
    /* if is_intra */
    if (is_intra) {
      /* fap_segment_start_code */
      input_stream_bits(tmpbytes, 32, &in_stream);
      /* is_frame_rate */
      is_frame_rate = input_stream_bit(&in_stream);
      if (is_frame_rate) {
	input_stream_bits(tmpbytes, 8, &in_stream);
	frameRate = 0;
	for(i=0; i<8; i++)
	  frameRate |= tmpbytes[i]<< (7-i);
	input_stream_bits(tmpbytes, 4, &in_stream);
	input_stream_bits(tmpbytes, 1, &in_stream);
      }
      else frameRate = -1;

      /* is_time_code*/
      is_time_code = input_stream_bit(&in_stream);
      if (is_time_code) {
	input_stream_bits(tmpbytes, 5, &in_stream);
	input_stream_bits(tmpbytes, 6, &in_stream);
	input_stream_bits(tmpbytes, 1, &in_stream);
	input_stream_bits(tmpbytes, 6, &in_stream);
      }
    }
    /* skip_segments */
    skip_segments = input_stream_bit(&in_stream);
    if (skip_segments) {
      skip_num = 0;
      do {
	input_stream_bits(tmpbytes, 4, &in_stream);
	tmpb = 0;
	for (i=0; i<4; i++)
	  tmpb |= tmpbytes[i] << (3-i);
	skip_num = skip_num*16 + tmpb; 
      } while (tmpb == 15);
    }
    
    if(fba_paramset_mask&1) {
      if (is_intra) {
	for (i=0; i<NGROUP; i++) {
	  /* fap_mask_type = '01' */
	  input_stream_bits(tmpbytes, 2, &in_stream);
	  if (tmpbytes[0] + tmpbytes[1] == 1)
	    for (j=mask_index[i]+1; j<= mask_index[i+1]; j++)
	      f_mask[j] = input_stream_bit(&in_stream);
	  
	}
	/* fap_quant */
	tmpb = input_stream_bits_to_byte(5, &in_stream);
	fap_quant = tmpb;
	if (n == 0) {
	  /* ouput Header Information */
	  fprintf(out_fp, "# reconstructed fap file\n");
	  fprintf(out_fp, "# fap_quant = %d\n", fap_quant);
	  fprintf(out_fp, "%2.1f ", 2.0);
	  fprintf(out_fp, "rec ");
	  fprintf(out_fp, "%d ", frameRate);
	  fprintf(out_fp, "%d ", nframe);
	  fprintf(out_fp, "\n");
	}
      }
    }  
    
    
    /* for each parameter, decoding */
    for (j=0; j<NFAP; j++) {
      if (f_mask[j]) {
	
	if (j<=1){
	  switch(j) {
	  case 0: 
	    decode_viseme(is_intra, viseme_select1, viseme_select2, viseme_blend, viseme_prev,
				     &v_sel_table,
				     &v_blend_table,
				     &in_stream);
	    break;
	  case 1:
	    decode_expression(is_intra, expression_select1,expression_select2,
			      expression_intensity1, expression_intensity2, expression_prev,
			      &e_sel_table,
			      &e_intensity_table,
			      &in_stream); 
	    break;
	  }
	}
	else {
	  quantization = fap_quant*MSCALE*DCTQP[j];
	  qdc =  quantization/3.0;
	  /* get dc */
	  if ( is_intra) { /* intra-segment */
	    tmps = input_stream_short(&in_stream);
	    if (tmps == SHORT_MIN) /* ESC */
	      coeff_sequence[0] = (float)(input_stream_int(&in_stream));
	    else coeff_sequence[0] = (float)(tmps);
	    last_dc[j] = round(coeff_sequence[0]);
	  }
	  else { /* inter-segment */
	    tmps = (short)input_stream_huffman_symbol(&dc_table, &in_stream);
	    if (tmps == 0) { /* ESC */
	      tmps = input_stream_short(&in_stream);
	      if (tmps == SHORT_MIN) /* ESC */
		tmps = input_stream_int(&in_stream);
	    }
	    else 
	      tmps -= 256;
	    coeff_sequence[0] = last_dc[j]+tmps;
	    last_dc[j] = round(coeff_sequence[0]);
	  }
	  
	  coeff_sequence[0] *= qdc;
	  
	  /* get ac */
	  for (m=1; m<SEG_LENGTH; m++)
	    coeff_sequence[m] = 0;
	  leap = 1; m=0;
	  while (leap && m <(SEG_LENGTH-1)) { 
	    leap = input_stream_huffman_symbol(&runlength_table, &in_stream);
	    if (leap != 0) {
	      m += leap;
	      tmps = input_stream_huffman_symbol(&ac_table, &in_stream)-256;
	      if (tmps == 0) { 
		tmps = input_stream_short(&in_stream);
		if (tmps == SHORT_MIN) /* ESC */
		  tmps = input_stream_int(&in_stream);
	      }
	      coeff_sequence[m] = tmps*quantization;
	    }
	  }
	  /* inverse DCT */
	  idct(coeff_sequence, output_sequence);
	  for (m=0; m<SEG_LENGTH; m++)
	    data[j][m] = output_sequence[m];
	} /* if is expression/viseme end */
	/*if (is_last_segment && j== 49)
	  printf("      used bits = %d\n", in_stream.byte_count*8+in_stream.bit_count);
	  */
      } /* if f_mask[j] */
    } /* for j FAP */
    /* write out the segment */
    for (i=0; i<frm_no; i++) {
      for (j=0; j<NFAP; j++)
	fprintf(out_fp, "%d ", f_mask[j]);
      fprintf(out_fp, "\n");
      fprintf(out_fp, "%d ", n*SEG_LENGTH+i);
      for (j=0; j<NFAP; j++)
	if (f_mask[j])
	  fprintf(out_fp, "%d ",round(data[j][i]));
      fprintf(out_fp, "\n");
    }
    
    /* refill the in_stream buffer, this way, 
       we can handle infinite long sequence */
    if (n%10 == 9) {
      input_stream_load(stream_fp, &remain_file_length,&in_stream);
    }
    /*fprintf(stderr,"used bits = %d  ", in_stream.byte_count*8+in_stream.bit_count);
     */
    n++;
    
  } /* while(!last_segment) end */
  /* aligned and input some data to memory */
  input_stream_aligned(&in_stream);
  /*input_stream_load(stream_fp, &remain_file_length, &in_stream);
   */
  /* input fap_sequence_end_code */
  input_stream_bits(tmpbytes, 32, &in_stream);
   
  fclose(stream_fp);
  fclose(out_fp);
}



void decode_viseme(BYTE intra, int *sel1, int *sel2,int *blend, int *vis_prev,
		   HUFFMAN_TABLE *vis_sel_table,
		   HUFFMAN_TABLE *vis_blend_table,
		   FAP_STREAM_MEM *stream)
{
  int i, start_index, diff;
  BYTE tmpb;
  if (intra){
    /* intra segment */
    /* first frame */
    tmpb =input_stream_bits_to_byte(4, stream);
    sel1[0] =tmpb;
    tmpb =input_stream_bits_to_byte(4, stream);
     sel2[0] =tmpb;
    tmpb =input_stream_bits_to_byte(6, stream);
    blend[0] =tmpb;
    
    vis_prev[0] = sel1[0];
    vis_prev[1] = sel2[0];
    vis_prev[2] = blend[0];
    start_index = 1;
  }
  else start_index = 0;
  
  for (i=start_index; i<SEG_LENGTH; i++) {
    diff = (int)input_stream_huffman_symbol(vis_sel_table, stream);
    sel1[i] = vis_prev[0]+diff;
    vis_prev[0] = sel1[i];
    
    diff = (int)input_stream_huffman_symbol(vis_sel_table, stream);
    sel2[i] = vis_prev[1]+diff;
    vis_prev[1] = sel2[i];
    
    diff = (int)input_stream_huffman_symbol(vis_blend_table, stream);
    blend[i] = vis_prev[2]+diff;
    vis_prev[2] = blend[i];
  }
  /* viseme_def 1bit = 0 */
  tmpb = input_stream_bit(stream);
}

void decode_expression(BYTE intra, int *sel1, int *sel2,
		       int *intensity1, int *intensity2, int *exp_prev,
		       HUFFMAN_TABLE *exp_sel_table,
		       HUFFMAN_TABLE *exp_intensity_table,
		       FAP_STREAM_MEM *stream)
{
  int i, start_index, diff;
  BYTE tmpb;
  if (intra){
    /* intra segment */
    /* first frame */
    tmpb = input_stream_bits_to_byte(3, stream);
    sel1[0] =tmpb;
    tmpb = input_stream_bits_to_byte(3, stream);
    sel2[0] =tmpb;
    tmpb = input_stream_bits_to_byte(6, stream);
    intensity1[0] =tmpb;
    tmpb = input_stream_bits_to_byte(6, stream);
    intensity2[0] =tmpb;
    
    exp_prev[0] = sel1[0];
    exp_prev[1] = sel2[0];
    exp_prev[2] = intensity1[0];
    exp_prev[3] = intensity2[0];
    start_index = 1;
  }
  else start_index = 0;
  

  for (i=start_index; i<SEG_LENGTH; i++) {
    diff = (int)input_stream_huffman_symbol(exp_sel_table, stream);
    sel1[i] = exp_prev[0]+diff;
    exp_prev[0] = sel1[i];
    
    diff = (int)input_stream_huffman_symbol(exp_sel_table, stream);
    sel2[i] = exp_prev[1]+diff;
    exp_prev[1] = sel2[i];
    
    diff = (int)input_stream_huffman_symbol(exp_intensity_table, stream);
    intensity1[i] = exp_prev[2]+diff;
    exp_prev[2] = intensity1[i];

    diff = (int)input_stream_huffman_symbol(exp_intensity_table, stream);
    intensity2[i] = exp_prev[3]+diff;
    exp_prev[3] = intensity2[i];
  }
  /* expression_def 1bit = 0 */
  tmpb = input_stream_bit(stream);
}
