/************************************************************************
   This software module was originally developed by

   Hai Tao    (tao@ifp.uiuc.edu), University of Illinois at Urbana-Champaign
   Homer Chen (homer@risc.rockwell.com), Rockwell International Corporation
   (date: October, 1997)

in the course of development of the MPEG-4 Video (ISO/IEC 14496-2).
This software module is an implementation of a part of one or more
MPEG-4 Video tools as specified by the MPEG-4 Video. ISO/IEC gives users
of the MPEG-4 Video free license to this software module or
modifications thereof for use in hardware or software products claiming
conformance to the MPEG-4 Video.  Those intending to use this software
module in hardware or software products are advised that its use may
infringe existing patents.  The original developer of this software
module and his/her company, the subsequent editors and their companies,
and ISO/IEC have no liability for use of this software module or
modifications thereof in an implementation. Copyright is not released
for non MPEG-4 Video conforming products. UIUC/Rockwell retains full right
to use the code for his/her own purpose, assign or donate the code to a
third party and to inhibit third parties from using the code for non
MPEG-4 Video conforming products. This copyright notice must be
included in all copies or derivative works.

Copyright (c) 1996, 1997.
***********************************************************************/

#include <stdio.h>
#include <math.h>
#include "util.h"
#include "dctencode1D.h"

/*#define MY_DEBUG
 */

int dctencode1seg(float *input,
		  int intra_flag,
		  float quantization,
		  int *l_dc,
		  HUFFMAN_TABLE *dc_table,
		  HUFFMAN_TABLE *run_table,
		  HUFFMAN_TABLE *ac_table, FAP_STREAM_MEM *outs) 
{
  float input_sequence[SEG_LENGTH], coeff_sequence[SEG_LENGTH], qdc;
  int bit_count;
  int i,j, k, m, n, tmpi, leap;
   
  /* bits used in this segment and this FAP*/
  bit_count = 0;
  
  /* input[] -> input_sequence[j]*/
  for (j=0; j<SEG_LENGTH; j++)
    input_sequence[j] = input[j];
 

  /* DCT:  input_sequence -> coeff_sequence */
  dct(input_sequence, coeff_sequence);
 

  /* find quantization step for DC */
  qdc =  quantization/3.0;

  /* DC -> outs */
  coeff_sequence[0] = coeff_sequence[0]/qdc;
  if (intra_flag) { /* intra frame */
    if (round(coeff_sequence[0]) > SHORT_MAX ||
	round(coeff_sequence[0]) < SHORT_MIN+1) {
      bit_count +=  output_stream_short(SHORT_MIN, outs);
      bit_count +=  output_stream_int(round(coeff_sequence[0]), outs);
    }
    else {
      bit_count +=  output_stream_short((short)(round(coeff_sequence[0])),outs);
    }
    *l_dc = round(coeff_sequence[0]);
  }
  else { /* inter frame */
    tmpi = round(coeff_sequence[0])- (*l_dc);
     
    if (tmpi >255 || tmpi< -255) {
      bit_count += output_stream_huffman_symbol(0 ,dc_table, outs);
      if (tmpi  > SHORT_MAX || tmpi < SHORT_MIN+1) {
	bit_count +=  output_stream_short(SHORT_MIN, outs);
	bit_count += output_stream_int(tmpi, outs);
      }
      else {
	bit_count += output_stream_short((short)(tmpi), outs);
      }
    }
    else {
      bit_count += output_stream_huffman_symbol(tmpi+256,dc_table, outs);
    }
    (*l_dc) = round(coeff_sequence[0]);
  }
  coeff_sequence[0] *= qdc;
  
  /* runlength, value -> outs */
  n = 0; leap = 1;
  for (m=1; m< SEG_LENGTH; m++) {
    tmpi = round(coeff_sequence[m]/quantization);
    if(tmpi != 0 ) {
      bit_count += output_stream_huffman_symbol(leap, run_table, outs);
      leap =1;
      if (tmpi >255 || tmpi< -256) {
	bit_count += output_stream_huffman_symbol(0+256,ac_table, outs);
	if (tmpi  > SHORT_MAX || tmpi < SHORT_MIN+1) {
	  bit_count +=  output_stream_short(SHORT_MIN, outs);
	  bit_count += output_stream_int(tmpi, outs);
	}
	else {
	  bit_count += output_stream_short((short)tmpi, outs);
	}
      }
      else {
	bit_count += output_stream_huffman_symbol(tmpi+256,ac_table, outs);
      }
      n++; 
    }
    else {
      leap ++;
    }
    coeff_sequence[m]= tmpi*quantization;
  }
  
  /* output the segment end runlength symbol */
  if (leap != 1)
    bit_count += output_stream_huffman_symbol(0, run_table, outs);       
  
#ifdef MY_DEBUG
  printf("(%d) --", n);
#endif
  return (bit_count);
}
