/************************************************************************
   This software module was originally developed by

   Hai Tao    (tao@ifp.uiuc.edu), University of Illinois at Urbana-Champaign
   Homer Chen (homer@risc.rockwell.com), Rockwell International Corporation
   (date: October, 1997)

in the course of development of the MPEG-4 Video (ISO/IEC 14496-2).
This software module is an implementation of a part of one or more
MPEG-4 Video tools as specified by the MPEG-4 Video. ISO/IEC gives users
of the MPEG-4 Video free license to this software module or
modifications thereof for use in hardware or software products claiming
conformance to the MPEG-4 Video.  Those intending to use this software
module in hardware or software products are advised that its use may
infringe existing patents.  The original developer of this software
module and his/her company, the subsequent editors and their companies,
and ISO/IEC have no liability for use of this software module or
modifications thereof in an implementation. Copyright is not released
for non MPEG-4 Video conforming products. UIUC/Rockwell retains full right
to use the code for his/her own purpose, assign or donate the code to a
third party and to inhibit third parties from using the code for non
MPEG-4 Video conforming products. This copyright notice must be
included in all copies or derivative works.

Copyright (c) 1996, 1997.
***********************************************************************/

#include <stdio.h>
#include <stdlib.h>
#include <math.h>

#include "util.h"

int ndim, nfrm;
int FAP_MASK_TYPE=0;	/* different from encode.c */
int fap_group_mask=0;
char fap_mask[NFAP], ifap_mask[NFAP];
int  Irange_max[NFAP], Irange_min[NFAP];


main(int argc, char **argv) {

  int i,j,k, n;
  int maxn, minn, qstep;
  int nframe;
  int frameRate;
  int frame_num;
  char basename[100];
  float version, tmpf,tmpff, *mse, *psnr;
  FILE *fap_fp, *rfap_fp;
  int **fap_o, **fap_r;
  int ndimi, tmpi;

  if(argc!=3) {
    fprintf(stderr, "Usage: %s original_fap reconstructed_fap \n", argv[0]);
    exit(-1);
  }

  if ((fap_fp = fopen(argv[1], "r")) == NULL){
    printf("file %s can not be open\n", argv[1]);
    exit(1);
  }
  if ((rfap_fp = fopen(argv[2], "r")) == NULL){
    printf("file %s can not be open\n", argv[2]);
    exit(1);
  }
    
  /* Read Header Information from original faps file */
  nextValidLine(fap_fp);
  fscanf(fap_fp, "%f", &version);
  fscanf(fap_fp, "%s", basename);
  fscanf(fap_fp, "%d", &frameRate);
  fscanf(fap_fp, "%d", &nframe);
  nfrm = nframe;
  
  if (version < 1.5) {
    for(i=0; i<NGROUP; i++) {
      int mask;
      fscanf(fap_fp, "%d", &mask);
    }
  }
  
  for (i=0; i<NFAP; i++) {
	int mask;
	fscanf(fap_fp, "%d", &mask);
	fap_mask[i] = mask;
  }
  
  ndim = 0;
  for (i=0;  i<NFAP; i++)
    if (fap_mask[i]) ndim++;
  
  mse = float_array(ndim);
  psnr = float_array(ndim);
  fap_o = int_2d_array(nfrm, ndim);
  fap_r = int_2d_array(nfrm, ndim);
  
  if (version > 1.5) {
    for (i=0;  i<NFAP; i++) ifap_mask[i] = fap_mask[i];
    for(n=0; n<nfrm; n++) {
      fscanf(fap_fp, "%d", &frame_num);
      for(i=0,j=0; i<NFAP; i++) {
	if (ifap_mask[i] && (!fap_mask[i]))
	fscanf(fap_fp, "%d", &tmpi);
	if (ifap_mask[i] && fap_mask[i]){
	  fscanf(fap_fp, "%d", &tmpi);
	  fap_o[n][j++] =  tmpi;
	}
	if ((! ifap_mask[i]) && fap_mask[i])
	  fap_o[n][j++]=0;
      }
      for (i=0;  i<NFAP; i++) {
	fscanf(fap_fp, "%d", &tmpi);
	ifap_mask[i] = tmpi;
      }
    }
  }
  else {
    for(n=0; n<nfrm; n++) {
      fscanf(fap_fp, "%d", &frame_num);
      for(i=0,j=0; i<NFAP; i++)
	if (fap_mask[i]) 
	  fscanf(fap_fp, "%d", &fap_o[n][j++]);
    }  
  }
  fclose(fap_fp);
  
  /* Read Header Information from reconstructed faps file */
  nextValidLine(rfap_fp);
  fscanf(rfap_fp, "%f", &version);
  fscanf(rfap_fp, "%s", basename);
  fscanf(rfap_fp, "%d", &frameRate);
  fscanf(rfap_fp, "%d", &nframe);
  if (version < 1.5){
    for(i=0; i<NGROUP; i++) {
      int mask;
      fscanf(rfap_fp, "%d", &mask);
    }
  }
  
  for (i=0; i<NFAP; i++) {
    int mask;
    fscanf(rfap_fp, "%d", &mask);
    fap_mask[i] = mask;
  }
  
  if (version > 1.5) {
    for (i=0;  i<NFAP; i++)ifap_mask[i] = fap_mask[i];
    for(n=0; n<nfrm; n++) {
      fscanf(rfap_fp, "%d", &frame_num);
      for(i=0,j=0; i<NFAP; i++) {
	if (ifap_mask[i] && (!fap_mask[i]))
	  fscanf(rfap_fp, "%d", &tmpi);
	if (ifap_mask[i] && fap_mask[i]){
	  fscanf(rfap_fp, "%d", &tmpi);
	  fap_r[n][j++] =  tmpi;
	}
	if ((! ifap_mask[i]) && fap_mask[i])
	  fap_r[n][j++]=0;
      }
      for (i=0;  i<NFAP; i++) {
	fscanf(rfap_fp, "%d", &tmpi);
	ifap_mask[i] = tmpi;
      }
    }
  }
  else {
    for(n=0; n<nfrm; n++) {
      fscanf(rfap_fp, "%d", &frame_num);
      for(i=0,j=0; i<NFAP; i++)
	if (fap_mask[i]) 
	  fscanf(rfap_fp, "%d", &fap_r[n][j++]);
    }  
  }
  fclose(rfap_fp);
  
  /* find the range fap_o[nfrm][ndim] ->Irange_max[ndim], Irange_min[ndim] */
  for (j=0; j< ndim; j++) {
    Irange_max[j] = 0-100000;
    Irange_min[j] = 0+100000;
    for (i=0; i<nfrm; i++) {
      if (fap_o[i][j] > Irange_max[j])
	Irange_max[j] = fap_o[i][j];
      if (fap_o[i][j] < Irange_min[j])
	Irange_min[j] = fap_o[i][j];
    }
  }
  
  /* computer MSE/PSNR */
  for (j=0; j<ndim; j++)
    mse[j] = 0.0;
  for(n=0; n<nfrm; n++) {
    for(j=0; j<ndim; j++)
      mse[j] += (fap_o[n][j]-fap_r[n][j])*(fap_o[n][j]-fap_r[n][j]);
  }
  for(j=0; j<ndim; j++)
    mse[j] = mse[j]/nfrm;
  
  /*
    printf("MSE = \n");
    for (j=0; j<ndim; j++)
    printf("%f, ", mse[j]);
    printf("\n");
  */

  for(j=0; j<ndim; j++) {
    tmpf = Irange_max[j]- Irange_min[j];
    tmpf *= tmpf;
    if (mse[j] >0.0001)
      psnr[j] = 10.0*(float)(log10(tmpf/mse[j]));
    else 
      psnr[j] = 10000.0;
  }
  
  tmpff = 0;
  ndimi = 0;
  for(j=0; j<ndim; j++) {
    if (Irange_max[j]- Irange_min[j] <3.0) continue;
    tmpf = Irange_max[j]- Irange_min[j];
    tmpf *= tmpf;
    if (mse[j] != 0)
      tmpff += tmpf/mse[j];
    else 
      tmpff += 10000.0;
    ndimi++;
  }

  printf("    %9.4f\n", (float)(10*log10(tmpff/ndimi)));
  /*  printf("data dimension is %d, actual dimension is %d\n",ndim, ndimi);
    
  
  printf("PSNR = \n");
  j=0;
  for(i=0; i<NFAP; i++) 
    if (fap_mask[i]) 
      printf("(%d)%9.4f, ",i+1,  psnr[j++]);
  printf("\n");
  */
  

  
}
