/************************************************************************
   This software module was originally developed by

   Hai Tao    (tao@ifp.uiuc.edu), University of Illinois at Urbana-Champaign
   Homer Chen (homer@risc.rockwell.com), Rockwell International Corporation
   (date: October, 1997)

in the course of development of the MPEG-4 Video (ISO/IEC 14496-2).
This software module is an implementation of a part of one or more
MPEG-4 Video tools as specified by the MPEG-4 Video. ISO/IEC gives users
of the MPEG-4 Video free license to this software module or
modifications thereof for use in hardware or software products claiming
conformance to the MPEG-4 Video.  Those intending to use this software
module in hardware or software products are advised that its use may
infringe existing patents.  The original developer of this software
module and his/her company, the subsequent editors and their companies,
and ISO/IEC have no liability for use of this software module or
modifications thereof in an implementation. Copyright is not released
for non MPEG-4 Video conforming products. UIUC/Rockwell retains full right
to use the code for his/her own purpose, assign or donate the code to a
third party and to inhibit third parties from using the code for non
MPEG-4 Video conforming products. This copyright notice must be
included in all copies or derivative works.

Copyright (c) 1996, 1997.
***********************************************************************/

#include <malloc.h>
#include <math.h>
#define  UTLC
#include "util.h"

/* #define MY_DEBUG 1
 */

  
float *float_array(int len){
  float *ptr;
  ptr = (float *) calloc (len, sizeof(float));
  return ptr;
}

int *int_array(int len){
  int *ptr;
  ptr = (int *) calloc (len, sizeof(int));
  return ptr;
}

double *double_array(int len){
  double *ptr;
  ptr = (double *) calloc (len, sizeof(double));
  return ptr;
}

float **float_2d_array(int row, int col){
  int i;
  float *ptr, **pptr;
  ptr = (float *) calloc (row*col, sizeof(float));
  pptr = (float **) calloc (row, sizeof(float *));
  for (i=0; i<row; i++)
    pptr[i] = ptr+i*col;
  return pptr;
}
double **double_2d_array(int row, int col){
  int i;
  double *ptr, **pptr;
  ptr = (double *) calloc (row*col, sizeof(double));
  pptr = (double **) calloc (row, sizeof(double *));
  for (i=0; i<row; i++)
    pptr[i] = ptr+i*col;
  return pptr;
}

int **int_2d_array(int row, int col){
  int i;
  int *ptr, **pptr;
  ptr = (int *) calloc (row*col, sizeof(int));
  pptr = (int **) calloc (row, sizeof(int *));
  for (i=0; i<row; i++)
    pptr[i] = ptr+i*col;
  return pptr;
}


void init_dct(int size){
  float a;
  int i,j;
  dct_size = size;
  
  dct_matrix = float_2d_array(size, size);
  idct_matrix = float_2d_array(size, size);
  
  for(i=0; i<size; i++) {
    if(i==0) a = (float)(1.0/sqrt(2));
    else a = 1;
    for(j=0; j<size; j++)
      dct_matrix[i][j]=(float)(sqrt(2./size))*a*
	(float)(cos(PI*(float)(2*j+1)*i/2.0/size));
  }
  for(i=0; i<size; i++)
    for(j=0; j<size; j++)
      idct_matrix[i][j] = dct_matrix[j][i];

}

void dct(float *input, float *output) {
  int i,j;
  for (i=0; i<dct_size; i++){
    output[i] = 0;
    for (j=0; j<dct_size; j++)
      output[i] += dct_matrix[i][j]*input[j];
  }
}

void idct(float *input, float *output) {
  int i,j;
  for (i=0; i<dct_size; i++){
    output[i] = 0;
    for (j=0; j<dct_size; j++)
      output[i] += idct_matrix[i][j]*input[j];
  }
}


int input_stream_huffman_symbol(HUFFMAN_TABLE *table,FAP_STREAM_MEM *stream ){
  
  int j,k,found;
  BYTE tmpb;
  int no[512];
  int symbol;
  for(k=0; k<table->no_of_symbol; k++) 
    no[k] = 0;

  j=0;
  found = 0;
  while(!found) {
    tmpb = input_stream_bit(stream);
    
    for (k=0; k<table->no_of_symbol; k++) {
      if (no[k] != 1) {
	if (table->no_of_bits[k] == j+1){
	  if (tmpb == table->bit[k][j]) {
	    /* the symbol is k */
	     symbol = k;
	    found = 1;
	    break;
	  }
	  else {  /* k is not the symbol */
	     no[k] = 1;
	  }
	}
	else if (table->no_of_bits[k] >j+1){
	  if (tmpb != table->bit[k][j])
	    no[k] = 1;
	}
      }
    }
    j++;
  }
  return(symbol);
}


BYTE input_stream_bit(FAP_STREAM_MEM *stream ){
  BYTE tmpb = 1;
  tmpb <<= (7-stream->bit_count);
  tmpb &= stream->stream[stream->byte_count];
  tmpb >>= (7-stream->bit_count);
  tmpb &= 1;
  
  stream->bit_count ++;
  if (stream->bit_count == 8) {
    stream->byte_count ++; 
    stream->bit_count = 0;
  }
  return (tmpb);
}

void input_stream_bits(BYTE *data, int length, FAP_STREAM_MEM *stream ) {
  int i;
  for (i=0; i<length; i++)
    data[i] = input_stream_bit(stream);
}

BYTE input_stream_bits_to_byte(int length, FAP_STREAM_MEM *stream ) {
  int i;
  BYTE tmpb, data;
  if(length >8) {
    fprintf(stderr, "input_stream_bits_to_byte(), length >8 \n");
    exit(1);
  }
  data = 0;
  for (i=0; i<length; i++) {
    tmpb = input_stream_bit(stream);
    data |= (tmpb & (BYTE)(1))<<(length-i-1);
  }
  return(data);
}

BYTE input_stream_byte(FAP_STREAM_MEM *stream ){
  BYTE tmpc = 0;
  BYTE tmpb;
  int i;
  
  for (i=0; i<8; i++) {
    tmpb= input_stream_bit(stream );
    tmpb <<= 7-i;
    tmpc |= tmpb;
  }
  return(tmpc);
}

int input_stream_int(FAP_STREAM_MEM *stream ){
  /* int tmpi;
  BYTE *ptr;
  int i;
  
  ptr = (BYTE *)(&tmpi);
  for (i=0; i<4; i++){
    *ptr = input_stream_byte(stream);
    ptr++;
  }
  */
  int tmpi, *ip;
  unsigned int tmpui;
  tmpui = input_stream_uint(stream);
  ip = (int *)(&tmpui);
  return(*ip);
}
unsigned int input_stream_uint(FAP_STREAM_MEM *stream ){
  unsigned short tmpus1, tmpus2;
  tmpus1 = input_stream_ushort(stream);
  tmpus2 = input_stream_ushort(stream);
  return(tmpus1*256*256+tmpus2);
}

short input_stream_short(FAP_STREAM_MEM *stream ){
  /*
    short tmpi;
  BYTE *ptr;
  int i;
  
  ptr = (BYTE *)(&tmpi);
  for (i=0; i<2; i++){
    *ptr = input_stream_byte(stream);
    ptr++;
  }
  return(tmpi);
  */
  
  short tmps, *sp;
  unsigned short tmpus;
  tmpus = input_stream_ushort(stream);
  sp = (short *)(&tmpus);
  return(*sp);

}
unsigned short input_stream_ushort(FAP_STREAM_MEM *stream ){
  /*  unsigned short tmpi;
  BYTE *ptr;
  int i;
  
  ptr = (BYTE *)(&tmpi);
  for (i=0; i<2; i++){
    *ptr = input_stream_byte(stream);
    ptr++;
  }
  return(tmpi);
  */
  unsigned char tmpub1, tmpub2;
  tmpub1 = input_stream_byte(stream);
  tmpub2 = input_stream_byte(stream);
  return(tmpub1*256+tmpub2);
}

int input_stream_aligned(FAP_STREAM_MEM *stream){
  if (stream->bit_count != 0){
    stream->bit_count = 0;
    stream->byte_count ++;
  }
}

int input_stream_load(FILE *fp, int *remain_len, FAP_STREAM_MEM *stream){
  int i, space_left, space_used;
  /* move bytes_in_stream- stream->byte_count*/ 
  for (i=stream->byte_count; i<stream->bytes_in_stream; i++)
    stream->stream[i-stream->byte_count] = stream->stream[i];

  space_used = stream->bytes_in_stream- stream->byte_count;
  space_left = STREAM_SIZE -  space_used;

  stream->byte_count = 0;
  stream->bytes_in_stream = space_used;
  
  /* load in space_left*/
  if (*remain_len >= space_left) {
    fread(&(stream->stream[space_used]),space_left, 1, fp);
    *remain_len -= space_left;
    stream->bytes_in_stream = STREAM_SIZE;
  }
  else {
    if (*remain_len != 0) {
      fread(&(stream->stream[space_used]), *remain_len, 1, fp);
      stream->bytes_in_stream = space_used+(*remain_len);
      *remain_len = 0;
    }
  }
  return(1);
}


int output_stream_huffman_symbol(int symbol_index, 
				 HUFFMAN_TABLE *table,FAP_STREAM_MEM *stream ){
  int i;
  
  for (i=0; i<table->no_of_bits[symbol_index]; i++)
    output_stream_bit(table->bit[symbol_index][i], stream);
  table->freq[symbol_index]++;
  table->total_freq ++;
  return(table->no_of_bits[symbol_index]);
  /*
  for (i=0; i<8; i++)
    output_stream_bit(1, stream);
  return(8);*/
  
}

int output_stream_int(int input, FAP_STREAM_MEM *stream ){
  /* BYTE *ptr;
  int i;
  ptr = (BYTE *)(&input);
  for (i=0; i<4; i++){
    output_stream_byte(*ptr, stream);
    ptr++;
  }
  */
  unsigned int *uip;
  uip = (unsigned int *)(&input);
  return(output_stream_uint(*uip, stream));
}

int output_stream_uint(unsigned int input, FAP_STREAM_MEM *stream ){
  output_stream_ushort((unsigned short)(input/(256*256)), stream);
  output_stream_ushort((unsigned short)(input%(256*256)), stream);
  return(32);
}

int output_stream_short(short input, FAP_STREAM_MEM *stream ){
  /*BYTE *ptr; 
  ptr = (BYTE *)(&input);
  output_stream_byte(*ptr, stream);
  ptr++;
  output_stream_byte(*ptr, stream);
  */
  unsigned short *usp;
  usp = (unsigned short *)(&input);
  return(output_stream_ushort(*usp, stream));
}

int output_stream_ushort(unsigned short input, FAP_STREAM_MEM *stream ){
  output_stream_byte((BYTE)(input/256), stream);
  output_stream_byte((BYTE)(input%256), stream);
  return(16);
}

int output_stream_byte(BYTE input, FAP_STREAM_MEM *stream ){
  BYTE tmpc = 0;
  BYTE mask = 128;
  int i;
  for (i=0; i<8; i++) {
    tmpc = input & mask;
    tmpc >>= 7-i;
    output_stream_bit(tmpc, stream );
    mask >>= 1;
  }
  return(8);
}

/* bit in the last bit of BYTE input */
int output_stream_bit(BYTE input, FAP_STREAM_MEM *stream ){
  input <<= (7-stream->bit_count);
  stream->stream[stream->byte_count] |= input;
  stream->bit_count ++;
  if (stream->bit_count == 8) {
    stream->byte_count ++; 
    stream->bit_count = 0;
  }
  return (1);
}
int output_stream_bits(BYTE *input, int length, FAP_STREAM_MEM *stream ){
  int i;
  for(i=0; i<length; i++)
    output_stream_bit(input[i], stream);
  return(length);
}

int output_stream_bits_from_byte(BYTE input, int length, FAP_STREAM_MEM *stream ){
  int i;
  if (length >8) {
    fprintf(stderr, "output_stream_bits() tries to output more than 8 bits \n");
    exit(1);
  }
  for(i=0; i<length; i++)
    output_stream_bit( ((BYTE)(input>>(length-i-1))&(BYTE)(1)), stream);
  return(length);
}

int output_stream_aligned(FAP_STREAM_MEM *stream){
  int tmpi;
  if (stream->bit_count != 0) {
    tmpi = 8 - stream->bit_count;
    stream->bit_count = 0;
    stream->byte_count ++;
    return(tmpi);
  }
  else {
    return(0);
  }
}
void output_stream_flush(FILE *fp, FAP_STREAM_MEM *stream){
  int tmpi;
  BYTE tmpb;
 
  if (stream->byte_count != 0) {
    /* output stream->byte_count data */
    fwrite(stream->stream, stream->byte_count, 1, fp);
    tmpb = stream->stream[stream->byte_count];
    tmpi = stream->bit_count;
    output_stream_init(stream);
    stream->bit_count = tmpi;
    stream->stream[0] = tmpb;
  }
}

int output_stream_init(FAP_STREAM_MEM *stream){
  int i;
  for (i=0; i< STREAM_SIZE; i++)
    stream->stream[i] = 0;
  stream->bit_count = 0;
  stream->byte_count = 0;
}

int input_stream_init(FAP_STREAM_MEM *stream){
  int i;
  stream->bytes_in_stream = 0;
  stream->bit_count = 0;
  stream->byte_count = 0;
}





#define MaxCharPerLine	300

void  nextValidLine(FILE *ifp) {
  char firstChar;
  char buf[MaxCharPerLine];

  while(feof(ifp) == 0) {
    firstChar = fgetc(ifp);
    if (firstChar == '#') { /* skip comments */
      fgets(buf, MaxCharPerLine, ifp); 
    } else if(firstChar == '\n') { /* skip empty line */
      ;
    } else {
      ungetc (firstChar, ifp);
      return;
    }
  }
}

void  finishLine(FILE *ifp) {
  char buf[MaxCharPerLine];
  if(feof(ifp) == 0) fgets(buf, MaxCharPerLine, ifp); 
}


void assign_dc_ac_runlength_vis_exp_table(HUFFMAN_TABLE  *dc_t, 
					  HUFFMAN_TABLE  *ac_t, 
					  HUFFMAN_TABLE  *rl_t,
					  HUFFMAN_TABLE  *v_s,
					  HUFFMAN_TABLE  *v_b,
					  HUFFMAN_TABLE  *e_s,
					  HUFFMAN_TABLE  *e_i )
{
 
  read_a_huffman_table(dc_table_name, dc_t);
  read_a_huffman_table(ac_table_name, ac_t);
  read_a_huffman_table(rl_table_name, rl_t);
  read_a_huffman_table(v_s_table_name, v_s);
  read_a_huffman_table(v_b_table_name, v_b);
  read_a_huffman_table(e_s_table_name, e_s);
  read_a_huffman_table(e_i_table_name, e_i);
  
}
void read_a_huffman_table(char *filename, HUFFMAN_TABLE  *table){
  FILE *fp;
  int i,j,tmpi;
  if ((fp = fopen(filename, "r")) == NULL) {
    printf("the file %s can not be open !\n", filename);
    exit(1);
  }
  fscanf(fp, "%d", &(table->no_of_symbol));
  for (i=0; i<table->no_of_symbol; i++) {
    fscanf(fp, "%d", &(table->no_of_bits[i]));
    for (j=0; j<table->no_of_bits[i]; j++) {
      fscanf(fp, "%d", &tmpi);
      table->bit[i][j] = (BYTE)tmpi;
    }
  }
  fclose(fp);
}


