/***********************************************************************
    This software module  was  originally  developed by Zvi  Lifshitz
    (Triton R&D Ltd.) and  Mikael Bourges-Sevenier (France Telecom CNET)
    in the course of development of the MPEG-4 Systems (ISO/IEC 14496-1)
    standard.  This software module is an implementation of a part of
    one or more MPEG-4 Systems (ISO/IEC 14496-1) tools  as  specified
    by  the  MPEG-4 Systems (ISO/IEC 14496-1) standard. ISO/IEC gives
    users of the MPEG-4 Systems (ISO/IEC 14496-1)  free  license  to
    this software module or modifications thereof for use in hardware
    or software products claiming conformance to the  MPEG-4  Systems
    (ISO/IEC 14496-1). Those intending to use this software module in
    hardware or software  products  are  advised  that  its  use  may
    infringe   existing  patents.  The  original  developer  of  this
    software module and his/her company, the subsequent  editors  and
    their  companies,  and  ISO/IEC have no liability for use of this
    software module or modifications thereof  in  an  implementation.
    Copyright  is not released for non MPEG-4 Systems (ISO/IEC 14496-
    1) conforming  products.  Triton R&D Ltd. and France  Telecom  CNET
    retain  full  right to use the code for their own purpose, assign
    or donate the code to a third party and to inhibit third  parties
    from  using  the  code  for  non MPEG-4 Systems (ISO/IEC 14496-1)
    conforming products. This copyright notice must  be  included  in
    all copies or derivative works. Copyright  1997.
************************************************************************/

/**************************************************************************
 * Project Im1Player
 * INTERNATIONAL STANDARD ORGANIZATION
 * ISO/IEC JTC1/SC29/WG11
 * 
 * SUBSYSTEM: BIFS Encoder/Decoder
 * FILE:      Nodes.h
 * AUTHOR:    Mikael Bourges-Sevenier (France Telecom CNET/DIH/HDM)
 * OVERVIEW:  BIFS macros for node objects.
 * 
 * Comment:   Automatically generated on Wed Nov 18 18:14:13 CET 1998
 *            at France Telecom CCETT, Rennes, FRANCE
 *            by parsing the MPEG-4 templates.wrl file.
 *            Contact: mikael.bourgessevenier@cnet.francetelecom.fr
 * 
 **************************************************************************/

#include "float.h"
#include "StreamCo.h"
#include "NodeFld.h"
#include "bifs.h"
#include "Root.h"
#include "Conditional.h"

extern "C"
{
  #include "../FbaDecoder/sim.h"
  #include "../Animation/ArithModel.h"
}

#if !defined(_Nodes_h)  // Sentry, use file only if it's not already included.
#define _Nodes_h

#pragma warning(disable: 4305)  // truncation from 'const double' to 'const float'

// Riitta 99/02/23
// added nodes under ifdef

#ifdef ADV_AUDIO 

class AcousticScene : public MediaObject // GroupNode changed 042999
{
     BIFS_DECLARE_NODE

		 field MF<float> reverbtime;
		 field SF<Vec3f> size;
		 field SF<Vec3f> center;
         public:
			 AcousticScene()
				:  center(Vec3f(0.0f, 0.0f, 0.0f)),
				size(Vec3f(-1.0f, -1.0f, -1.0f))
			{
			} 
};


class DirectiveSound : public MediaObject
{
         BIFS_DECLARE_NODE

		 exposedField SF<Vec3f> direction;
         exposedField SF<float> intensity;
         exposedField SF<Vec3f> location;
         field SF<BOOL> useAirabs;
         exposedField SFAudioNode source;
         field SF<BOOL> spatialize;
		 field SF<BOOL> roomEffect;
		 field MF<float>	directivity;
		 field SF<float>	speedOfSound;
		 field SF<float>	distance;
#ifdef PERCEPTUAL_AUDIO // J-B, here's what you need (riitta 050299)
		 exposedField SFNode<class PerceptualParameters> perceptualParameters;
#endif		 

public:
	DirectiveSound()
		: direction(Vec3f(0, 0, 1)),
		intensity(1.0f),
		location(Vec3f(0, 0, 0)),
		speedOfSound(344.0f),
		distance(100.0f),
		useAirabs(FALSE),
		spatialize(TRUE),
		roomEffect(FALSE)
	{
		#ifdef PERCEPTUAL_AUDIO
			percpetualParameters = NULL;
		#endif
	}
};

class AcousticMaterial : public MediaObject
{
         BIFS_DECLARE_NODE

         exposedField SF<float> ambientIntensity;
         exposedField SF<Color3f> diffuseColor;
         exposedField SF<Color3f> emissiveColor;
         exposedField SF<float> shininess;
         exposedField SF<Color3f> specularColor;
         exposedField SF<float> transparency;
		 field MF<float> reffunc;
		 field MF<float> transfunc;

         public:
			 AcousticMaterial() {
				   ambientIntensity = 0.2;
                   diffuseColor = Color3f(0.8, 0.8, 0.8);
                   emissiveColor = Color3f(0, 0, 0);
                   shininess = 0.2;
                   specularColor = Color3f(0, 0, 0);
                   transparency = 0;
				}
};

#endif // ADV_AUDIO

 class Anchor : public GroupNode
 {
         BIFS_DECLARE_NODE

         eventIn MFNodeType<SF3DNode> addChildren;
         eventIn MFNodeType<SF3DNode> removeChildren;
         exposedField MFNodeType<SF3DNode> children;
         exposedField SFString description;
         exposedField MF<ZString> parameter;
         exposedField MF<Url> url;
         void OnaddChildren();
         void OnremoveChildren();

         public:
              Anchor() {
              }
 };

 class AnimationStream : public StreamConsumer
 {
         BIFS_DECLARE_NODE

         exposedField SF<Bool> loop;
         exposedField SF<float> speed;
         exposedField SF<Time> startTime;
         exposedField SF<Time> stopTime;
//         exposedField MF<Url> url;
         eventOut SF<Time> duration_changed;
         eventOut SF<Bool> isActive;

         public:
              AnimationStream() {
                   loop = FALSE;
                   speed = 1;
                   startTime = 0;
                   stopTime = 0;
              }
 };

 class Appearance : public MediaObject
 {
         BIFS_DECLARE_NODE

         exposedField SFMaterialNode material;
         exposedField SFTextureNode texture;
         exposedField SFNode<class TextureTransform> textureTransform;

         public:
              Appearance() {
              }
 };

 class AudioBuffer : public GroupNode
 {
         BIFS_DECLARE_NODE

         exposedField SF<Bool> loop;
         exposedField SF<float> pitch;
         exposedField SF<Time> startTime;
         exposedField SF<Time> stopTime;
         exposedField MFNodeType<SFAudioNode> children;
         exposedField SF<int> numChan;
         exposedField MF<int> phaseGroup;
         field SF<float> length;
         eventOut SF<Time> duration_changed;
         eventOut SF<Bool> isActive;

         public:
              AudioBuffer() {
                   loop = FALSE;
                   pitch = 1;
                   startTime = 0;
                   stopTime = 0;
                   numChan = 1;
                   phaseGroup += 1;
                   length = 0;
              }
 };

 class AudioClip : public StreamConsumer
 {
         BIFS_DECLARE_NODE

         exposedField SFString description;
         exposedField SF<Bool> loop;
         exposedField SF<float> pitch;
         exposedField SF<Time> startTime;
         exposedField SF<Time> stopTime;
//         exposedField MF<Url> url;
         eventOut SF<Time> duration_changed;
         eventOut SF<Bool> isActive;

         public:
              AudioClip() {
                   loop = FALSE;
                   pitch = 1;
                   startTime = 0;
                   stopTime = 0;
              }
 };

 class AudioDelay : public GroupNode
 {
         BIFS_DECLARE_NODE

         eventIn MFNodeType<SFAudioNode> addChildren;
         eventIn MFNodeType<SFAudioNode> removeChildren;
         exposedField MFNodeType<SFAudioNode> children;
         exposedField SF<Time> delay;
         field SF<int> numChan;
         field MF<int> phaseGroup;
         void OnaddChildren();
         void OnremoveChildren();

         public:
              AudioDelay() {
                   delay = 0;
                   numChan = 1;
              }
 };

 class AudioFX : public GroupNode
 {
         BIFS_DECLARE_NODE

         eventIn MFNodeType<SFAudioNode> addChildren;
         eventIn MFNodeType<SFAudioNode> removeChildren;
         exposedField MFNodeType<SFAudioNode> children;
         exposedField SFString orch;
         exposedField SFString score;
         exposedField MF<float> params;
         field SF<int> numChan;
         field MF<int> phaseGroup;
         void OnaddChildren();
         void OnremoveChildren();

         public:
              AudioFX() {
                   numChan = 1;
              }
 };

 class AudioMix : public GroupNode
 {
         BIFS_DECLARE_NODE

         eventIn MFNodeType<SFAudioNode> addChildren;
         eventIn MFNodeType<SFAudioNode> removeChildren;
         exposedField MFNodeType<SFAudioNode> children;
         exposedField SF<int> numInputs;
         exposedField MF<float> matrix;
         field SF<int> numChan;
         field MF<int> phaseGroup;
         void OnaddChildren();
         void OnremoveChildren();

         public:
              AudioMix() {
                   numInputs = 1;
                   numChan = 1;
              }
 };

 class AudioSource : public StreamConsumer
 {
         BIFS_DECLARE_NODE

         eventIn MFNodeType<SFAudioNode> addChildren;
         eventIn MFNodeType<SFAudioNode> removeChildren;
         exposedField MFNodeType<SFAudioNode> children;
//         exposedField MF<Url> url;
         exposedField SF<float> pitch;
         exposedField SF<Time> startTime;
         exposedField SF<Time> stopTime;
         field SF<int> numChan;
         field MF<int> phaseGroup;
         void OnaddChildren();
         void OnremoveChildren();

         public:
              AudioSource() {
                   pitch = 1;
                   startTime = 0;
                   stopTime = 0;
                   numChan = 1;
              }
 };

 class AudioSwitch : public GroupNode
 {
         BIFS_DECLARE_NODE

         eventIn MFNodeType<SFAudioNode> addChildren;
         eventIn MFNodeType<SFAudioNode> removeChildren;
         exposedField MFNodeType<SFAudioNode> children;
         exposedField MF<int> whichChoice;
         field SF<int> numChan;
         field MF<int> phaseGroup;
         void OnaddChildren();
         void OnremoveChildren();

         public:
              AudioSwitch() {
                   numChan = 1;
              }
 };

 class Background : public MediaObject
 {
         BIFS_DECLARE_NODE

         eventIn SF<Bool> set_bind;
         exposedField MF<float> groundAngle;
         exposedField MF<Color3f> groundColor;
         exposedField MF<Url> backUrl;
         exposedField MF<Url> bottomUrl;
         exposedField MF<Url> frontUrl;
         exposedField MF<Url> leftUrl;
         exposedField MF<Url> rightUrl;
         exposedField MF<Url> topUrl;
         exposedField MF<float> skyAngle;
         exposedField MF<Color3f> skyColor;
         eventOut SF<Bool> isBound;
         void Onset_bind();

         public:
              Background() {
                   skyColor += 0;
                   skyColor += 0;
                   skyColor += 0;
              }
 };

 class Background2D : public StreamConsumer
 {
         BIFS_DECLARE_NODE

         eventIn SF<Bool> set_bind;
         exposedField SF<Color3f> backColor;
//         exposedField MF<Url> url;
         eventOut SF<Bool> isBound;
         void Onset_bind();

         public:
              Background2D() {
                   backColor = Color3f(0, 0, 0);
              }
 };

 class Billboard : public GroupNode
 {
         BIFS_DECLARE_NODE

         eventIn MFNodeType<SF3DNode> addChildren;
         eventIn MFNodeType<SF3DNode> removeChildren;
         exposedField MFNodeType<SF3DNode> children;
         exposedField SF<Vec3f> axisOfRotation;
         void OnaddChildren();
         void OnremoveChildren();

         public:
              Billboard() {
                   axisOfRotation = Vec3f(0, 1, 0);
              }
 };

 class Bitmap : public MediaObject
 {
         BIFS_DECLARE_NODE

         exposedField SF<Vec2f> scale;

         public:
              Bitmap() {
                   scale = Vec2f(-1, -1);
              }
 };

 class Box : public MediaObject
 {
         BIFS_DECLARE_NODE

         field SF<Vec3f> size;

         public:
              Box() {
                   size = Vec3f(2, 2, 2);
              }
 };

 class Circle : public MediaObject
 {
         BIFS_DECLARE_NODE

         exposedField SF<float> radius;

         public:
              Circle() {
                   radius = 1;
              }
 };

 class Collision : public GroupNode
 {
         BIFS_DECLARE_NODE

         eventIn MFNodeType<SF3DNode> addChildren;
         eventIn MFNodeType<SF3DNode> removeChildren;
         exposedField MFNodeType<SF3DNode> children;
         exposedField SF<Bool> collide;
         field SF3DNode proxy;
         eventOut SF<Time> collideTime;
         void OnaddChildren();
         void OnremoveChildren();

         public:
              Collision() {
                   collide = TRUE;
              }
 };

 class Color : public MediaObject
 {
         BIFS_DECLARE_NODE

         exposedField MF<Color3f> color;

         public:
              Color() {
              }
 };

 class ColorInterpolator : public MediaObject
 {
         BIFS_DECLARE_NODE

         eventIn SF<float> set_fraction;
         exposedField MF<float> key;
         exposedField MF<Color3f> keyValue;
         eventOut SF<Color3f> value_changed;
         void Onset_fraction();

         public:
              ColorInterpolator() {
              }
 };

 class CompositeTexture2D : public StreamConsumer
 {
         BIFS_DECLARE_NODE

         eventIn MFNodeType<SF2DNode> addChildren;
         eventIn MFNodeType<SF2DNode> removeChildren;
         exposedField MFNodeType<SF2DNode> children;
         exposedField SF<int> pixelWidth;
         exposedField SF<int> pixelHeight;
         exposedField SFNode<class Background2D> background;
         exposedField SF2DNode viewport;
         void OnaddChildren();
         void OnremoveChildren();

         public:
              CompositeTexture2D() {
                   pixelWidth = -1;
                   pixelHeight = -1;
              }
 };

 class CompositeTexture3D : public StreamConsumer
 {
         BIFS_DECLARE_NODE

         eventIn MFNodeType<SF3DNode> addChildren;
         eventIn MFNodeType<SF3DNode> removeChildren;
         exposedField MFNodeType<SF3DNode> children;
         exposedField SF<int> pixelWidth;
         exposedField SF<int> pixelHeight;
         exposedField SFNode<class Background> background;
         exposedField SFNode<class Fog> fog;
         exposedField SFNode<class NavigationInfo> navigationInfo;
         exposedField SFNode<class Viewpoint> viewpoint;
         void OnaddChildren();
         void OnremoveChildren();

         public:
              CompositeTexture3D() {
                   pixelWidth = -1;
                   pixelHeight = -1;
              }
 };

 class Conditional : public MediaObject
 {
         BIFS_DECLARE_NODE

         eventIn SF<Bool> activate;
         eventIn SF<Bool> reverseActivate;
         exposedField SFConditionalString buffer;
         eventOut SF<Bool> isActive;
         void Onactivate();
         void OnreverseActivate();

         public:
              Conditional() {
                   reverseActivate = FALSE;
              }
 };

 class Cone : public MediaObject
 {
         BIFS_DECLARE_NODE

         field SF<float> bottomRadius;
         field SF<float> height;
         field SF<Bool> side;
         field SF<Bool> bottom;

         public:
              Cone() {
                   bottomRadius = 1;
                   height = 2;
                   side = TRUE;
                   bottom = TRUE;
              }
 };

 class Coordinate : public MediaObject
 {
         BIFS_DECLARE_NODE

         exposedField MF<Vec3f> point;

         public:
              Coordinate() {
              }
 };

 class Coordinate2D : public MediaObject
 {
         BIFS_DECLARE_NODE

         exposedField MF<Vec2f> point;

         public:
              Coordinate2D() {
              }
 };

 class CoordinateInterpolator : public MediaObject
 {
         BIFS_DECLARE_NODE

         eventIn SF<float> set_fraction;
         exposedField MF<float> key;
         exposedField MF<Vec3f> keyValue;
         eventOut MF<Vec3f> value_changed;
         void Onset_fraction();

         public:
              CoordinateInterpolator() {
              }
 };

 class CoordinateInterpolator2D : public MediaObject
 {
         BIFS_DECLARE_NODE

         eventIn SF<float> set_fraction;
         exposedField MF<float> key;
         exposedField MF<Vec2f> keyValue;
         eventOut MF<Vec2f> value_changed;
         void Onset_fraction();

         public:
              CoordinateInterpolator2D() {
              }
 };

 class Curve2D : public MediaObject
 {
         BIFS_DECLARE_NODE

         exposedField SFNode<class Coordinate2D> point;
         exposedField SF<float> fineness;
         exposedField MF<int> type;

         public:
              Curve2D() {
                   fineness = 0.5;
              }
 };

 class Cylinder : public MediaObject
 {
         BIFS_DECLARE_NODE

         field SF<Bool> bottom;
         field SF<float> height;
         field SF<float> radius;
         field SF<Bool> side;
         field SF<Bool> top;

         public:
              Cylinder() {
                   bottom = TRUE;
                   height = 2;
                   radius = 1;
                   side = TRUE;
                   top = TRUE;
              }
 };

 class CylinderSensor : public MediaObject
 {
         BIFS_DECLARE_NODE

         exposedField SF<Bool> autoOffset;
         exposedField SF<float> diskAngle;
         exposedField SF<Bool> enabled;
         exposedField SF<float> maxAngle;
         exposedField SF<float> minAngle;
         exposedField SF<float> offset;
         eventOut SF<Bool> isActive;
         eventOut SF<Rotation> rotation_changed;
         eventOut SF<Vec3f> trackPoint_changed;

         public:
              CylinderSensor() {
                   autoOffset = TRUE;
                   diskAngle = 0.262;
                   enabled = TRUE;
                   maxAngle = -1;
                   minAngle = 0;
                   offset = 0;
              }
 };

 class DirectionalLight : public MediaObject
 {
         BIFS_DECLARE_NODE

         exposedField SF<float> ambientIntensity;
         exposedField SF<Color3f> color;
         exposedField SF<Vec3f> direction;
         exposedField SF<float> intensity;
         exposedField SF<Bool> on;

         public:
              DirectionalLight() {
                   ambientIntensity = 0;
                   color = Color3f(1, 1, 1);
                   direction = Vec3f(0, 0, -1);
                   intensity = 1;
                   on = TRUE;
              }
 };

 class DiscSensor : public MediaObject
 {
         BIFS_DECLARE_NODE

         exposedField SF<Bool> autoOffset;
         exposedField SF<Bool> enabled;
         exposedField SF<float> maxAngle;
         exposedField SF<float> minAngle;
         exposedField SF<float> offset;
         eventOut SF<Bool> isActive;
         eventOut SF<float> rotation_changed;
         eventOut SF<Vec2f> trackPoint_changed;

         public:
              DiscSensor() {
                   autoOffset = TRUE;
                   enabled = TRUE;
                   maxAngle = -1;
                   minAngle = -1;
                   offset = 0;
              }
 };

 class ElevationGrid : public MediaObject
 {
         BIFS_DECLARE_NODE

         eventIn MF<float> set_height;
         exposedField SFNode<class Color> color;
         exposedField SFNode<class Normal> normal;
         exposedField SFNode<class TextureCoordinate> texCoord;
         field MF<float> height;
         field SF<Bool> ccw;
         field SF<Bool> colorPerVertex;
         field SF<float> creaseAngle;
         field SF<Bool> normalPerVertex;
         field SF<Bool> solid;
         field SF<int> xDimension;
         field SF<float> xSpacing;
         field SF<int> zDimension;
         field SF<float> zSpacing;
         void Onset_height();

         public:
              ElevationGrid() {
                   ccw = TRUE;
                   colorPerVertex = TRUE;
                   creaseAngle = 0;
                   normalPerVertex = TRUE;
                   solid = TRUE;
                   xDimension = 0;
                   xSpacing = 1;
                   zDimension = 0;
                   zSpacing = 1;
              }
 };

 class Expression : public MediaObject
 {
         BIFS_DECLARE_NODE

         exposedField SF<int> expression_select1;
         exposedField SF<int> expression_intensity1;
         exposedField SF<int> expression_select2;
         exposedField SF<int> expression_intensity2;
         exposedField SF<Bool> init_face;
         exposedField SF<Bool> expression_def;

         public:
              Expression() {
                   expression_select1 = 0;
                   expression_intensity1 = 0;
                   expression_select2 = 0;
                   expression_intensity2 = 0;
                   init_face = FALSE;
                   expression_def = FALSE;
              }
 };

 class Extrusion : public MediaObject
 {
         BIFS_DECLARE_NODE

         eventIn MF<Vec2f> set_crossSection;
         eventIn MF<Rotation> set_orientation;
         eventIn MF<Vec2f> set_scale;
         eventIn MF<Vec3f> set_spine;
         field SF<Bool> beginCap;
         field SF<Bool> ccw;
         field SF<Bool> convex;
         field SF<float> creaseAngle;
         field MF<Vec2f> crossSection;
         field SF<Bool> endCap;
         field MF<Rotation> orientation;
         field MF<Vec2f> scale;
         field SF<Bool> solid;
         field MF<Vec3f> spine;
         void Onset_crossSection();
         void Onset_orientation();
         void Onset_scale();
         void Onset_spine();

         public:
              Extrusion() {
                   beginCap = TRUE;
                   ccw = TRUE;
                   convex = TRUE;
                   creaseAngle = 0;
                   crossSection += 1;
                   crossSection += 1;
                   crossSection += 1;
                   crossSection += -1;
                   crossSection += -1;
                   crossSection += -1;
                   crossSection += -1;
                   crossSection += 1;
                   crossSection += 1;
                   crossSection += 1;
                   endCap = TRUE;
                   orientation += 0;
                   orientation += 0;
                   orientation += 1;
                   orientation += 0;
                   scale += 1;
                   scale += 1;
                   solid = TRUE;
                   spine += 0;
                   spine += 0;
                   spine += 0;
                   spine += 0;
                   spine += 1;
                   spine += 0;
              }
 };

 class Face : public Grouping
 {
         BIFS_DECLARE_NODE

         exposedField SFNode<class FAP> fap;
         exposedField SFNode<class FDP> fdp;
         exposedField SFNode<class FIT> fit;
         exposedField SFAudioNode ttsSource;
         exposedField MFNodeType<SF3DNode> renderedFace;

         public:
              Face() {
              }
 };

 class FaceDefMesh : public MediaObject
 {
         BIFS_DECLARE_NODE

         field SF3DNode faceSceneGraphNode;
         field MF<int> intervalBorders;
         field MF<int> coordIndex;
         field MF<Vec3f> displacements;

         public:
              FaceDefMesh() {
              }
 };

 class FaceDefTables : public MediaObject
 {
         BIFS_DECLARE_NODE

         field SF<int> fapID;
         field SF<int> highLevelSelect;
         exposedField MFNodeType<SFFaceDefMeshNode> faceDefMesh;
         exposedField MFNodeType<SFFaceDefTransformNode> faceDefTransform;

         public:
              FaceDefTables() {
                   fapID = 1;
                   highLevelSelect = 1;
              }
 };

 class FaceDefTransform : public MediaObject
 {
         BIFS_DECLARE_NODE

         field SF3DNode faceSceneGraphNode;
         field SF<int> fieldId;
         field SF<Rotation> rotationDef;
         field SF<Vec3f> scaleDef;
         field SF<Vec3f> translationDef;

         public:
              FaceDefTransform() {
                   fieldId = 1;
                   rotationDef = Rotation(0, 0, 1, 0);
                   scaleDef = Vec3f(1, 1, 1);
                   translationDef = Vec3f(0, 0, 0);
              }
 };

#ifdef  PLAYER3D  
 
 class FAP : public StreamConsumer
 {
   BIFS_DECLARE_NODE
     
 public:
   /// Constructor
   FAP() {};
 public:
   /// The viseme to use
   SFNode<class Viseme>	viseme;
   /// The expression to use
   SFNode<class Expression>	expression;
   /// FAP values
   SF<int>		FAPValues[68];
   // FAP mask
   BOOL			FAPMask[68];
   
   //==========================================================
   // Dominique Nerriere 99/09 >>>>
   
   
   
   void animate(AnimationStreamDecoder *pAS,BOOL intra);
   void parseAnimationMask(AnimationStreamDecoder *pAS) ;

   
   
 };
 
#else 
 
 class FAP : public MediaObject
 {
   BIFS_DECLARE_NODE
     
   exposedField SFNode<class Viseme> viseme;
   exposedField SFNode<class Expression> expression;
   exposedField SF<int> open_jaw;
   exposedField SF<int> lower_t_midlip;
   exposedField SF<int> raise_b_midlip;
   exposedField SF<int> stretch_l_corner;
   exposedField SF<int> stretch_r_corner;
   exposedField SF<int> lower_t_lip_lm;
   exposedField SF<int> lower_t_lip_rm;
   exposedField SF<int> lower_b_lip_lm;
   exposedField SF<int> lower_b_lip_rm;
   exposedField SF<int> raise_l_cornerlip;
   exposedField SF<int> raise_r_cornerlip;
   exposedField SF<int> thrust_jaw;
   exposedField SF<int> shift_jaw;
   exposedField SF<int> push_b_lip;
   exposedField SF<int> push_t_lip;
   exposedField SF<int> depress_chin;
   exposedField SF<int> close_t_l_eyelid;
   exposedField SF<int> close_t_r_eyelid;
   exposedField SF<int> close_b_l_eyelid;
   exposedField SF<int> close_b_r_eyelid;
   exposedField SF<int> yaw_l_eyeball;
   exposedField SF<int> yaw_r_eyeball;
   exposedField SF<int> pitch_l_eyeball;
   exposedField SF<int> pitch_r_eyeball;
   exposedField SF<int> thrust_l_eyeball;
   exposedField SF<int> thrust_r_eyeball;
   exposedField SF<int> dilate_l_pupil;
   exposedField SF<int> dilate_r_pupil;
   exposedField SF<int> raise_l_i_eyebrow;
   exposedField SF<int> raise_r_i_eyebrow;
   exposedField SF<int> raise_l_m_eyebrow;
   exposedField SF<int> raise_r_m_eyebrow;
   exposedField SF<int> raise_l_o_eyebrow;
   exposedField SF<int> raise_r_o_eyebrow;
   exposedField SF<int> squeeze_l_eyebrow;
   exposedField SF<int> squeeze_r_eyebrow;
   exposedField SF<int> puff_l_cheek;
   exposedField SF<int> puff_r_cheek;
   exposedField SF<int> lift_l_cheek;
   exposedField SF<int> lift_r_cheek;
   exposedField SF<int> shift_tongue_tip;
   exposedField SF<int> raise_tongue_tip;
   exposedField SF<int> thrust_tongue_tip;
   exposedField SF<int> raise_tongue;
   exposedField SF<int> tongue_roll;
   exposedField SF<int> head_pitch;
   exposedField SF<int> head_yaw;
   exposedField SF<int> head_roll;
   exposedField SF<int> lower_t_midlip_o;
   exposedField SF<int> raise_b_midlip_o;
   exposedField SF<int> stretch_l_cornerlip;
   exposedField SF<int> stretch_r_cornerlip;
   exposedField SF<int> lower_t_lip_lm_o;
   exposedField SF<int> lower_t_lip_rm_o;
   exposedField SF<int> raise_b_lip_lm_o;
   exposedField SF<int> raise_b_lip_rm_o;
   exposedField SF<int> raise_l_cornerlip_o;
   exposedField SF<int> raise_r_cornerlip_o;
   exposedField SF<int> stretch_l_nose;
   exposedField SF<int> stretch_r_nose;
   exposedField SF<int> raise_nose;
   exposedField SF<int> bend_nose;
   exposedField SF<int> raise_l_ear;
   exposedField SF<int> raise_r_ear;
   exposedField SF<int> pull_l_ear;
   exposedField SF<int> pull_r_ear;
   
 public:
   FAP() {
     open_jaw = 0;
     lower_t_midlip = 0;
     raise_b_midlip = 0;
     stretch_l_corner = 0;
     stretch_r_corner = 0;
     lower_t_lip_lm = 0;
     lower_t_lip_rm = 0;
     lower_b_lip_lm = 0;
     lower_b_lip_rm = 0;
     raise_l_cornerlip = 0;
     raise_r_cornerlip = 0;
     thrust_jaw = 0;
     shift_jaw = 0;
     push_b_lip = 0;
     push_t_lip = 0;
     depress_chin = 0;
     close_t_l_eyelid = 0;
     close_t_r_eyelid = 0;
     close_b_l_eyelid = 0;
     close_b_r_eyelid = 0;
     yaw_l_eyeball = 0;
     yaw_r_eyeball = 0;
     pitch_l_eyeball = 0;
     pitch_r_eyeball = 0;
     thrust_l_eyeball = 0;
     thrust_r_eyeball = 0;
     dilate_l_pupil = 0;
     dilate_r_pupil = 0;
     raise_l_i_eyebrow = 0;
     raise_r_i_eyebrow = 0;
     raise_l_m_eyebrow = 0;
     raise_r_m_eyebrow = 0;
     raise_l_o_eyebrow = 0;
     raise_r_o_eyebrow = 0;
     squeeze_l_eyebrow = 0;
     squeeze_r_eyebrow = 0;
     puff_l_cheek = 0;
     puff_r_cheek = 0;
     lift_l_cheek = 0;
     lift_r_cheek = 0;
     shift_tongue_tip = 0;
     raise_tongue_tip = 0;
     thrust_tongue_tip = 0;
     raise_tongue = 0;
     tongue_roll = 0;
     head_pitch = 0;
     head_yaw = 0;
     head_roll = 0;
     lower_t_midlip_o = 0;
     raise_b_midlip_o = 0;
     stretch_l_cornerlip = 0;
     stretch_r_cornerlip = 0;
     lower_t_lip_lm_o = 0;
     lower_t_lip_rm_o = 0;
     raise_b_lip_lm_o = 0;
     raise_b_lip_rm_o = 0;
     raise_l_cornerlip_o = 0;
     raise_r_cornerlip_o = 0;
     stretch_l_nose = 0;
     stretch_r_nose = 0;
     raise_nose = 0;
     bend_nose = 0;
     raise_l_ear = 0;
     raise_r_ear = 0;
     pull_l_ear = 0;
     pull_r_ear = 0;
   }
   
 };

#endif // PLAYER3D
 
 // SB 99/03: derived from Grouping
 class FDP : public Grouping
 /*** >>>>
 class FDP : public MediaObject
 <<<< ***/
 {
         BIFS_DECLARE_NODE

         exposedField SFNode<class Coordinate> featurePointsCoord;
         exposedField SFNode<class TextureCoordinate> textureCoord;
         exposedField MFNodeType<SFFaceDefTablesNode> faceDefTables;
         exposedField MFNodeType<SF3DNode> faceSceneGraph;
         field SF<Bool> useOrthoTexture;

         public:
              FDP() {
                   useOrthoTexture = FALSE;
              }
 };

 class FIT : public MediaObject
 {
         BIFS_DECLARE_NODE

         exposedField MF<int> FAPs;
         exposedField MF<int> Graph;
         exposedField MF<int> numeratorExp;
         exposedField MF<int> denominatorExp;
         exposedField MF<int> numeratorImpulse;
         exposedField MF<int> numeratorTerms;
         exposedField MF<int> denominatorTerms;
         exposedField MF<float> numeratorCoefs;
         exposedField MF<float> denominatorCoefs;

         public:
              FIT() {
              }
 };

 class Fog : public MediaObject
 {
         BIFS_DECLARE_NODE

         exposedField SF<Color3f> color;
         exposedField SFString fogType;
         exposedField SF<float> visibilityRange;
         eventIn SF<Bool> set_bind;
         eventOut SF<Bool> isBound;
         void Onset_bind();

         public:
              Fog() {
                   color = Color3f(1, 1, 1);
                   fogType = ZString("LINEAR");
                   visibilityRange = 0;
              }
 };

 class FontStyle : public MediaObject
 {
         BIFS_DECLARE_NODE

         field MF<ZString> family;
         field SF<Bool> horizontal;
         field MF<ZString> justify;
         field SFString language;
         field SF<Bool> leftToRight;
         field SF<float> size;
         field SF<float> spacing;
         field SFString style;
         field SF<Bool> topToBottom;

         public:
              FontStyle() {
                   family += ZString("SERIF");
                   horizontal = TRUE;
                   justify += ZString("BEGIN");
                   leftToRight = TRUE;
                   size = 1;
                   spacing = 1;
                   style = ZString("PLAIN");
                   topToBottom = TRUE;
              }
 };

 class Form : public GroupNode
 {
         BIFS_DECLARE_NODE

         eventIn MFNodeType<SF2DNode> addChildren;
         eventIn MFNodeType<SF2DNode> removeChildren;
         exposedField MFNodeType<SF2DNode> children;
         exposedField SF<Vec2f> size;
         exposedField MF<int> groups;
         exposedField MF<ZString> constraints;
         exposedField MF<int> groupsIndex;
         void OnaddChildren();
         void OnremoveChildren();

         public:
              Form() {
                   size = Vec2f(-1, -1);
              }
 };

 class Group : public Root
 {
         BIFS_DECLARE_NODE

         eventIn MFNodeType<SF3DNode> addChildren;
         eventIn MFNodeType<SF3DNode> removeChildren;
         exposedField MFNodeType<SF3DNode> children;
         void OnaddChildren();
         void OnremoveChildren();

         public:
              Group() {
              }
 };

 class ImageTexture : public StreamConsumer
 {
         BIFS_DECLARE_NODE

//         exposedField MF<Url> url;
         field SF<Bool> repeatS;
         field SF<Bool> repeatT;

         public:
              ImageTexture() {
                   repeatS = TRUE;
                   repeatT = TRUE;
              }
 };

 class IndexedFaceSet : public MediaObject
 {
         BIFS_DECLARE_NODE

         eventIn MF<int> set_colorIndex;
         eventIn MF<int> set_coordIndex;
         eventIn MF<int> set_normalIndex;
         eventIn MF<int> set_texCoordIndex;
         exposedField SFNode<class Color> color;
         exposedField SFNode<class Coordinate> coord;
         exposedField SFNode<class Normal> normal;
         exposedField SFNode<class TextureCoordinate> texCoord;
         field SF<Bool> ccw;
         field MF<int> colorIndex;
         field SF<Bool> colorPerVertex;
         field SF<Bool> convex;
         field MF<int> coordIndex;
         field SF<float> creaseAngle;
         field MF<int> normalIndex;
         field SF<Bool> normalPerVertex;
         field SF<Bool> solid;
         field MF<int> texCoordIndex;
         void Onset_colorIndex();
         void Onset_coordIndex();
         void Onset_normalIndex();
         void Onset_texCoordIndex();

         public:
              IndexedFaceSet() {
                   ccw = TRUE;
                   colorPerVertex = TRUE;
                   convex = TRUE;
                   creaseAngle = 0;
                   normalPerVertex = TRUE;
                   solid = TRUE;
              }
 };

 class IndexedFaceSet2D : public MediaObject
 {
         BIFS_DECLARE_NODE

         eventIn MF<int> set_colorIndex;
         eventIn MF<int> set_coordIndex;
         eventIn MF<int> set_texCoordIndex;
         exposedField SFNode<class Color> color;
         exposedField SFNode<class Coordinate2D> coord;
         exposedField SFNode<class TextureCoordinate> texCoord;
         field MF<int> colorIndex;
         field SF<Bool> colorPerVertex;
         field SF<Bool> convex;
         field MF<int> coordIndex;
         field MF<int> texCoordIndex;
         void Onset_colorIndex();
         void Onset_coordIndex();
         void Onset_texCoordIndex();

         public:
              IndexedFaceSet2D() {
                   colorPerVertex = TRUE;
                   convex = TRUE;
              }
 };

 class IndexedLineSet : public MediaObject
 {
         BIFS_DECLARE_NODE

         eventIn MF<int> set_colorIndex;
         eventIn MF<int> set_coordIndex;
         exposedField SFNode<class Color> color;
         exposedField SFNode<class Coordinate> coord;
         field MF<int> colorIndex;
         field SF<Bool> colorPerVertex;
         field MF<int> coordIndex;
         void Onset_colorIndex();
         void Onset_coordIndex();

         public:
              IndexedLineSet() {
                   colorPerVertex = TRUE;
              }
 };

 class IndexedLineSet2D : public MediaObject
 {
         BIFS_DECLARE_NODE

         eventIn MF<int> set_colorIndex;
         eventIn MF<int> set_coordIndex;
         exposedField SFNode<class Color> color;
         exposedField SFNode<class Coordinate2D> coord;
         field MF<int> colorIndex;
         field SF<Bool> colorPerVertex;
         field MF<int> coordIndex;
         void Onset_colorIndex();
         void Onset_coordIndex();

         public:
              IndexedLineSet2D() {
                   colorPerVertex = TRUE;
              }
 };

 class Inline : public StreamConsumer
 {
         BIFS_DECLARE_NODE

//         exposedField MF<Url> url;

         public:
              Inline() {
              }
 };

 // SB 99/03: derived from Grouping
 class LOD : public Grouping
 /*** >>>>
 class LOD : public MediaObject
 <<<< ***/
 {
         BIFS_DECLARE_NODE

         exposedField MFNodeType<SF3DNode> level;
         field SF<Vec3f> center;
         field MF<float> range;

         public:
              LOD() {
                   center = Vec3f(0, 0, 0);
              }
 };

 class Layer2D : public Root
 {
         BIFS_DECLARE_NODE

         eventIn MFNodeType<SF2DNode> addChildren;
         eventIn MFNodeType<SF2DNode> removeChildren;
         exposedField MFNodeType<SF2DNode> children;
         exposedField SF<Vec2f> size;
         exposedField SFNode<class Background2D> background;
         exposedField SF2DNode viewport;
         void OnaddChildren();
         void OnremoveChildren();

         public:
              Layer2D() {
                   size = Vec2f(-1, -1);
              }
 };

 class Layer3D : public Root
 {
         BIFS_DECLARE_NODE

         eventIn MFNodeType<SF3DNode> addChildren;
         eventIn MFNodeType<SF3DNode> removeChildren;
         exposedField MFNodeType<SF3DNode> children;
         exposedField SF<Vec2f> size;
         exposedField SFNode<class Background> background;
         exposedField SFNode<class Fog> fog;
         exposedField SFNode<class NavigationInfo> navigationInfo;
         exposedField SFNode<class Viewpoint> viewpoint;
         void OnaddChildren();
         void OnremoveChildren();

         public:
              Layer3D() {
                   size = Vec2f(-1, -1);
              }
 };

 class Layout : public GroupNode
 {
         BIFS_DECLARE_NODE

         eventIn MFNodeType<SF2DNode> addChildren;
         eventIn MFNodeType<SF2DNode> removeChildren;
         exposedField MFNodeType<SF2DNode> children;
         exposedField SF<Bool> wrap;
         exposedField SF<Vec2f> size;
         exposedField SF<Bool> horizontal;
         exposedField MF<ZString> justify;
         exposedField SF<Bool> leftToRight;
         exposedField SF<Bool> topToBottom;
         exposedField SF<float> spacing;
         exposedField SF<Bool> smoothScroll;
         exposedField SF<Bool> loop;
         exposedField SF<Bool> scrollVertical;
         exposedField SF<float> scrollRate;
         void OnaddChildren();
         void OnremoveChildren();

         public:
              Layout() {
                   wrap = FALSE;
                   size = Vec2f(-1, -1);
                   horizontal = TRUE;
                   justify += ZString("BEGIN");
                   leftToRight = TRUE;
                   topToBottom = TRUE;
                   spacing = 1;
                   smoothScroll = FALSE;
                   loop = FALSE;
                   scrollVertical = TRUE;
                   scrollRate = 0;
              }
 };

 class LineProperties : public MediaObject
 {
         BIFS_DECLARE_NODE

         exposedField SF<Color3f> lineColor;
         exposedField SF<int> lineStyle;
         exposedField SF<float> width;

         public:
              LineProperties() {
                   lineColor = Color3f(0, 0, 0);
                   lineStyle = 0;
                   width = 1;
              }
 };

 class ListeningPoint : public MediaObject
 {
         BIFS_DECLARE_NODE

         eventIn SF<Bool> set_bind;
         exposedField SF<Bool> jump;
         exposedField SF<Rotation> orientation;
         exposedField SF<Vec3f> position;
         field SFString description;
         eventOut SF<Time> bindTime;
         eventOut SF<Bool> isBound;
         void Onset_bind();

         public:
              ListeningPoint() {
                   jump = TRUE;
                   orientation = Rotation(0, 0, 1, 0);
                   position = Vec3f(0, 0, 10);
              }
 };

 class Material : public MediaObject
 {
         BIFS_DECLARE_NODE

         exposedField SF<float> ambientIntensity;
         exposedField SF<Color3f> diffuseColor;
         exposedField SF<Color3f> emissiveColor;
         exposedField SF<float> shininess;
         exposedField SF<Color3f> specularColor;
         exposedField SF<float> transparency;

         public:
              Material() {
                   ambientIntensity = 0.2;
                   diffuseColor = Color3f(0.8, 0.8, 0.8);
                   emissiveColor = Color3f(0, 0, 0);
                   shininess = 0.2;
                   specularColor = Color3f(0, 0, 0);
                   transparency = 0;
              }
 };

 class Material2D : public MediaObject
 {
         BIFS_DECLARE_NODE

         exposedField SF<Color3f> emissiveColor;
         exposedField SF<Bool> filled;
         exposedField SFNode<class LineProperties> lineProps;
         exposedField SF<float> transparency;

         public:
              Material2D() {
                   emissiveColor = Color3f(0.8, 0.8, 0.8);
                   filled = FALSE;
                   transparency = 0;
              }
 };

 class MovieTexture : public StreamConsumer
 {
         BIFS_DECLARE_NODE

         exposedField SF<Bool> loop;
         exposedField SF<float> speed;
         exposedField SF<Time> startTime;
         exposedField SF<Time> stopTime;
//         exposedField MF<Url> url;
         field SF<Bool> repeatS;
         field SF<Bool> repeatT;
         eventOut SF<Time> duration_changed;
         eventOut SF<Bool> isActive;

         public:
              MovieTexture() {
                   loop = FALSE;
                   speed = 1;
                   startTime = 0;
                   stopTime = 0;
                   repeatS = TRUE;
                   repeatT = TRUE;
              }
 };

 class NavigationInfo : public MediaObject
 {
         BIFS_DECLARE_NODE

         eventIn SF<Bool> set_bind;
         exposedField MF<float> avatarSize;
         exposedField SF<Bool> headlight;
         exposedField SF<float> speed;
         exposedField MF<ZString> type;
         exposedField SF<float> visibilityLimit;
         eventOut SF<Bool> isBound;
         void Onset_bind();

         public:
              NavigationInfo() {
                   avatarSize += 0.25;
                   avatarSize += 1.6;
                   avatarSize += 0.75;
                   headlight = TRUE;
                   speed = 1;
                   type += ZString("WALK");
                   type += ZString("ANY");
                   visibilityLimit = 0;
              }
 };

 class Normal : public MediaObject
 {
         BIFS_DECLARE_NODE

         exposedField MF<Vec3f> vector;

         public:
              Normal() {
              }
 };

 class NormalInterpolator : public MediaObject
 {
         BIFS_DECLARE_NODE

         eventIn SF<float> set_fraction;
         exposedField MF<float> key;
         exposedField MF<Vec3f> keyValue;
         eventOut MF<Vec3f> value_changed;
         void Onset_fraction();

         public:
              NormalInterpolator() {
              }
 };

 class OrderedGroup : public GroupNode
 {
         BIFS_DECLARE_NODE

         eventIn MFNodeType<SF3DNode> addChildren;
         eventIn MFNodeType<SF3DNode> removeChildren;
         exposedField MFNodeType<SF3DNode> children;
         exposedField MF<float> order;
         void OnaddChildren();
         void OnremoveChildren();

         public:
              OrderedGroup() {
              }
 };

 class OrientationInterpolator : public MediaObject
 {
         BIFS_DECLARE_NODE

         eventIn SF<float> set_fraction;
         exposedField MF<float> key;
         exposedField MF<Rotation> keyValue;
         eventOut SF<Rotation> value_changed;
         void Onset_fraction();

         public:
              OrientationInterpolator() {
              }
 };

 class PixelTexture : public StreamConsumer
 {
         BIFS_DECLARE_NODE

         exposedField SF<Image> image;
         field SF<Bool> repeatS;
         field SF<Bool> repeatT;

         public:
              PixelTexture() {
                   image = Image(0, 0, 0);
                   repeatS = TRUE;
                   repeatT = TRUE;
              }
 };

 class PlaneSensor : public MediaObject
 {
         BIFS_DECLARE_NODE

         exposedField SF<Bool> autoOffset;
         exposedField SF<Bool> enabled;
         exposedField SF<Vec2f> maxPosition;
         exposedField SF<Vec2f> minPosition;
         exposedField SF<Vec3f> offset;
         eventOut SF<Bool> isActive;
         eventOut SF<Vec3f> trackPoint_changed;
         eventOut SF<Vec3f> translation_changed;

         public:
              PlaneSensor() {
                   autoOffset = TRUE;
                   enabled = TRUE;
                   maxPosition = Vec2f(-1, -1);
                   minPosition = Vec2f(0, 0);
                   offset = Vec3f(0, 0, 0);
              }
 };

 class PlaneSensor2D : public MediaObject
 {
         BIFS_DECLARE_NODE

         exposedField SF<Bool> autoOffset;
         exposedField SF<Bool> enabled;
         exposedField SF<Vec2f> maxPosition;
         exposedField SF<Vec2f> minPosition;
         exposedField SF<Vec2f> offset;
         eventOut SF<Bool> isActive;
         eventOut SF<Vec2f> trackPoint_changed;
         eventOut SF<Vec2f> translation_changed;

         public:
              PlaneSensor2D() {
                   autoOffset = TRUE;
                   enabled = TRUE;
                   maxPosition = Vec2f(0, 0);
                   minPosition = Vec2f(0, 0);
                   offset = Vec2f(0, 0);
              }
 };

 class PointLight : public MediaObject
 {
         BIFS_DECLARE_NODE

         exposedField SF<float> ambientIntensity;
         exposedField SF<Vec3f> attenuation;
         exposedField SF<Color3f> color;
         exposedField SF<float> intensity;
         exposedField SF<Vec3f> location;
         exposedField SF<Bool> on;
         exposedField SF<float> radius;

         public:
              PointLight() {
                   ambientIntensity = 0;
                   attenuation = Vec3f(1, 0, 0);
                   color = Color3f(1, 1, 1);
                   intensity = 1;
                   location = Vec3f(0, 0, 0);
                   on = TRUE;
                   radius = 100;
              }
 };

 class PointSet : public MediaObject
 {
         BIFS_DECLARE_NODE

         exposedField SFNode<class Color> color;
         exposedField SFNode<class Coordinate> coord;

         public:
              PointSet() {
              }
 };

 class PointSet2D : public MediaObject
 {
         BIFS_DECLARE_NODE

         exposedField SFNode<class Color> color;
         exposedField SFNode<class Coordinate2D> coord;

         public:
              PointSet2D() {
              }
 };

 class PositionInterpolator : public MediaObject
 {
         BIFS_DECLARE_NODE

         eventIn SF<float> set_fraction;
         exposedField MF<float> key;
         exposedField MF<Vec3f> keyValue;
         eventOut SF<Vec3f> value_changed;
         void Onset_fraction();

         public:
              PositionInterpolator() {
              }
 };

 class PositionInterpolator2D : public MediaObject
 {
         BIFS_DECLARE_NODE

         eventIn SF<float> set_fraction;
         exposedField MF<float> key;
         exposedField MF<Vec2f> keyValue;
         eventOut SF<Vec2f> value_changed;
         void Onset_fraction();

         public:
              PositionInterpolator2D() {
              }
 };

 class ProximitySensor2D : public MediaObject
 {
         BIFS_DECLARE_NODE

         exposedField SF<Vec2f> center;
         exposedField SF<Vec2f> size;
         exposedField SF<Bool> enabled;
         eventOut SF<Bool> isActive;
         eventOut SF<Vec2f> position_changed;
         eventOut SF<float> orientation_changed;
         eventOut SF<Time> enterTime;
         eventOut SF<Time> exitTime;

         public:
              ProximitySensor2D() {
                   center = Vec2f(0, 0);
                   size = Vec2f(0, 0);
                   enabled = TRUE;
              }
 };

 class ProximitySensor : public MediaObject
 {
         BIFS_DECLARE_NODE

         exposedField SF<Vec3f> center;
         exposedField SF<Vec3f> size;
         exposedField SF<Bool> enabled;
         eventOut SF<Bool> isActive;
         eventOut SF<Vec3f> position_changed;
         eventOut SF<Rotation> orientation_changed;
         eventOut SF<Time> enterTime;
         eventOut SF<Time> exitTime;

         public:
              ProximitySensor() {
                   center = Vec3f(0, 0, 0);
                   size = Vec3f(0, 0, 0);
                   enabled = TRUE;
              }
 };

 class QuantizationParameter : public MediaObject
 {
         BIFS_DECLARE_NODE

         field SF<Bool> isLocal;
         field SF<Bool> position3DQuant;
         field SF<Vec3f> position3DMin;
         field SF<Vec3f> position3DMax;
         field SF<int> position3DNbBits;
         field SF<Bool> position2DQuant;
         field SF<Vec2f> position2DMin;
         field SF<Vec2f> position2DMax;
         field SF<int> position2DNbBits;
         field SF<Bool> drawOrderQuant;
         field SF<float> drawOrderMin;
         field SF<float> drawOrderMax;
         field SF<int> drawOrderNbBits;
         field SF<Bool> colorQuant;
         field SF<float> colorMin;
         field SF<float> colorMax;
         field SF<int> colorNbBits;
         field SF<Bool> textureCoordinateQuant;
         field SF<float> textureCoordinateMin;
         field SF<float> textureCoordinateMax;
         field SF<int> textureCoordinateNbBits;
         field SF<Bool> angleQuant;
         field SF<float> angleMin;
         field SF<float> angleMax;
         field SF<int> angleNbBits;
         field SF<Bool> scaleQuant;
         field SF<float> scaleMin;
         field SF<float> scaleMax;
         field SF<int> scaleNbBits;
         field SF<Bool> keyQuant;
         field SF<float> keyMin;
         field SF<float> keyMax;
         field SF<int> keyNbBits;
         field SF<Bool> normalQuant;
         field SF<int> normalNbBits;
         field SF<Bool> sizeQuant;
         field SF<float> sizeMin;
         field SF<float> sizeMax;
         field SF<int> sizeNbBits;
         field SF<Bool> useEfficientCoding;

         public:
              QuantizationParameter() {
                   isLocal = FALSE;
                   position3DQuant = FALSE;
                   position3DMin = Vec3f(-FLT_MAX, -FLT_MAX, -FLT_MAX);
                   position3DMax = Vec3f(+FLT_MAX, +FLT_MAX, +FLT_MAX);
                   position3DNbBits = 16;
                   position2DQuant = FALSE;
                   position2DMin = Vec2f(-FLT_MAX, -FLT_MAX);
                   position2DMax = Vec2f(+FLT_MAX, +FLT_MAX);
                   position2DNbBits = 16;
                   drawOrderQuant = FALSE;
                   drawOrderMin = -FLT_MAX;
                   drawOrderMax = +FLT_MAX;
                   drawOrderNbBits = 8;
                   colorQuant = TRUE;
                   colorMin = 0;
                   colorMax = 1;
                   colorNbBits = 8;
                   textureCoordinateQuant = TRUE;
                   textureCoordinateMin = 0;
                   textureCoordinateMax = 1;
                   textureCoordinateNbBits = 16;
                   angleQuant = TRUE;
                   angleMin = 0;
                   angleMax = 6.2831855;
                   angleNbBits = 16;
                   scaleQuant = TRUE;
                   scaleMin = 0;
                   scaleMax = +FLT_MAX;
                   scaleNbBits = 8;
                   keyQuant = TRUE;
                   keyMin = 0;
                   keyMax = 1;
                   keyNbBits = 8;
                   normalQuant = TRUE;
                   normalNbBits = 8;
                   sizeQuant = FALSE;
                   sizeMin = 0;
                   sizeMax = +FLT_MAX;
                   sizeNbBits = 8;
                   useEfficientCoding = FALSE;
              }
 };

 class Rectangle : public MediaObject
 {
         BIFS_DECLARE_NODE

         exposedField SF<Vec2f> size;

         public:
              Rectangle() {
                   size = Vec2f(2, 2);
              }
 };

 class ScalarInterpolator : public MediaObject
 {
         BIFS_DECLARE_NODE

         eventIn SF<float> set_fraction;
         exposedField MF<float> key;
         exposedField MF<float> keyValue;
         eventOut SF<float> value_changed;
         void Onset_fraction();

         public:
              ScalarInterpolator() {
              }
 };

 class Script : public MediaObject
 {
         BIFS_DECLARE_NODE

         exposedField MF<ZString> url;
         field SF<Bool> directOutput;
         field SF<Bool> mustEvaluate;

         public:
              Script() {
                   directOutput = FALSE;
                   mustEvaluate = FALSE;
              }
 };

 class Shape : public MediaObject
 {
         BIFS_DECLARE_NODE

         exposedField SFNode<class Appearance> appearance;
         exposedField SFGeometryNode geometry;

         public:
              Shape() {
              }
 };

 class Sound : public MediaObject
 {
         BIFS_DECLARE_NODE

         exposedField SF<Vec3f> direction;
         exposedField SF<float> intensity;
         exposedField SF<Vec3f> location;
         exposedField SF<float> maxBack;
         exposedField SF<float> maxFront;
         exposedField SF<float> minBack;
         exposedField SF<float> minFront;
         exposedField SF<float> priority;
         exposedField SFAudioNode source;
         field SF<Bool> spatialize;

         public:
              Sound() {
                   direction = Vec3f(0, 0, 1);
                   intensity = 1;
                   location = Vec3f(0, 0, 0);
                   maxBack = 10;
                   maxFront = 10;
                   minBack = 1;
                   minFront = 1;
                   priority = 0;
                   spatialize = TRUE;
              }
 };

 class Sound2D : public MediaObject
 {
         BIFS_DECLARE_NODE

         exposedField SF<float> intensity;
         exposedField SF<Vec2f> location;
         exposedField SFAudioNode source;
         field SF<Bool> spatialize;

         public:
              Sound2D() {
                   intensity = 1;
                   location = Vec2f(0, 0);
                   spatialize = TRUE;
              }
 };

 class Sphere : public MediaObject
 {
         BIFS_DECLARE_NODE

         field SF<float> radius;

         public:
              Sphere() {
                   radius = 1;
              }
 };

 class SphereSensor : public MediaObject
 {
         BIFS_DECLARE_NODE

         exposedField SF<Bool> autoOffset;
         exposedField SF<Bool> enabled;
         exposedField SF<Rotation> offset;
         eventOut SF<Bool> isActive;
         eventOut SF<Rotation> rotation_changed;
         eventOut SF<Vec3f> trackPoint_changed;

         public:
              SphereSensor() {
                   autoOffset = TRUE;
                   enabled = TRUE;
                   offset = Rotation(0, 1, 0, 0);
              }
 };

 class SpotLight : public MediaObject
 {
         BIFS_DECLARE_NODE

         exposedField SF<float> ambientIntensity;
         exposedField SF<Vec3f> attenuation;
         exposedField SF<float> beamWidth;
         exposedField SF<Color3f> color;
         exposedField SF<float> cutOffAngle;
         exposedField SF<Vec3f> direction;
         exposedField SF<float> intensity;
         exposedField SF<Vec3f> location;
         exposedField SF<Bool> on;
         exposedField SF<float> radius;

         public:
              SpotLight() {
                   ambientIntensity = 0;
                   attenuation = Vec3f(1, 0, 0);
                   beamWidth = 1.570796;
                   color = Color3f(1, 1, 1);
                   cutOffAngle = 0.785398;
                   direction = Vec3f(0, 0, -1);
                   intensity = 1;
                   location = Vec3f(0, 0, 0);
                   on = TRUE;
                   radius = 100;
              }
 };

 class Switch : public Grouping
 {
         BIFS_DECLARE_NODE

         exposedField MFNodeType<SF3DNode> choice;
         exposedField SF<int> whichChoice;

         public:
              Switch() {
                   whichChoice = -1;
              }
 };

 class TermCap : public MediaObject
 {
         BIFS_DECLARE_NODE

         eventIn SF<Time> evaluate;
         exposedField SF<int> capability;
         eventOut SF<int> value;
         void Onevaluate();

         public:
              TermCap() {
                   capability = 0;
                   value = 0;
              }
 };

 class Text : public MediaObject
 {
         BIFS_DECLARE_NODE

         exposedField MF<ZString> string;
         exposedField MF<float> length;
         exposedField SFNode<class FontStyle> fontStyle;
         exposedField SF<float> maxExtent;

         public:
              Text() {
                   maxExtent = 0;
              }
 };

 class TextureCoordinate : public MediaObject
 {
         BIFS_DECLARE_NODE

         exposedField MF<Vec2f> point;

         public:
              TextureCoordinate() {
              }
 };

 class TextureTransform : public MediaObject
 {
         BIFS_DECLARE_NODE

         exposedField SF<Vec2f> center;
         exposedField SF<float> rotation;
         exposedField SF<Vec2f> scale;
         exposedField SF<Vec2f> translation;

         public:
              TextureTransform() {
                   center = Vec2f(0, 0);
                   rotation = 0;
                   scale = Vec2f(1, 1);
                   translation = Vec2f(0, 0);
              }
 };

 class TimeSensor : public MediaObject
 {
         BIFS_DECLARE_NODE

         exposedField SF<Time> cycleInterval;
         exposedField SF<Bool> enabled;
         exposedField SF<Bool> loop;
         exposedField SF<Time> startTime;
         exposedField SF<Time> stopTime;
         eventOut SF<Time> cycleTime;
         eventOut SF<float> fraction_changed;
         eventOut SF<Bool> isActive;
         eventOut SF<Time> time;

         public:
              TimeSensor() {
                   cycleInterval = 1;
                   enabled = TRUE;
                   loop = FALSE;
                   startTime = 0;
                   stopTime = 0;
              }
 };

 class TouchSensor : public MediaObject
 {
         BIFS_DECLARE_NODE

         exposedField SF<Bool> enabled;
         eventOut SF<Vec3f> hitNormal_changed;
         eventOut SF<Vec3f> hitPoint_changed;
         eventOut SF<Vec2f> hitTexCoord_changed;
         eventOut SF<Bool> isActive;
         eventOut SF<Bool> isOver;
         eventOut SF<Time> touchTime;

         public:
              TouchSensor() {
                   enabled = TRUE;
              }
 };

 class Transform : public GroupNode
 {
         BIFS_DECLARE_NODE

         eventIn MFNodeType<SF3DNode> addChildren;
         eventIn MFNodeType<SF3DNode> removeChildren;
         exposedField SF<Vec3f> center;
         exposedField MFNodeType<SF3DNode> children;
         exposedField SF<Rotation> rotation;
         exposedField SF<Vec3f> scale;
         exposedField SF<Rotation> scaleOrientation;
         exposedField SF<Vec3f> translation;
         void OnaddChildren();
         void OnremoveChildren();

         public:
              Transform() {
                   center = Vec3f(0, 0, 0);
                   rotation = Rotation(0, 0, 1, 0);
                   scale = Vec3f(1, 1, 1);
                   scaleOrientation = Rotation(0, 0, 1, 0);
                   translation = Vec3f(0, 0, 0);
              }
 };

 class Transform2D : public GroupNode
 {
         BIFS_DECLARE_NODE

         eventIn MFNodeType<SF2DNode> addChildren;
         eventIn MFNodeType<SF2DNode> removeChildren;
         exposedField MFNodeType<SF2DNode> children;
         exposedField SF<Vec2f> center;
         exposedField SF<float> rotationAngle;
         exposedField SF<Vec2f> scale;
         exposedField SF<float> scaleOrientation;
         exposedField SF<Vec2f> translation;
         void OnaddChildren();
         void OnremoveChildren();

         public:
              Transform2D() {
                   center = Vec2f(0, 0);
                   rotationAngle = 0;
                   scale = Vec2f(1, 1);
                   scaleOrientation = 0;
                   translation = Vec2f(0, 0);
              }
 };

 class Valuator : public MediaObject
 {
         BIFS_DECLARE_NODE

         eventIn SF<Bool> inSFBool;
         eventIn SF<Color3f> inSFColor;
         eventIn MF<Color3f> inMFColor;
         eventIn SF<float> inSFFloat;
         eventIn MF<float> inMFFloat;
         eventIn SF<int> inSFInt32;
         eventIn MF<int> inMFInt32;
         eventIn SF<Rotation> inSFRotation;
         eventIn MF<Rotation> inMFRotation;
         eventIn SFString inSFString;
         eventIn MF<ZString> inMFString;
         eventIn SF<Time> inSFTime;
         eventIn SF<Vec2f> inSFVec2f;
         eventIn MF<Vec2f> inMFVec2f;
         eventIn SF<Vec3f> inSFVec3f;
         eventIn MF<Vec3f> inMFVec3f;
         eventOut SF<Bool> outSFBool;
         eventOut SF<Color3f> outSFColor;
         eventOut MF<Color3f> outMFColor;
         eventOut SF<float> outSFFloat;
         eventOut MF<float> outMFFloat;
         eventOut SF<int> outSFInt32;
         eventOut MF<int> outMFInt32;
         eventOut SF<Rotation> outSFRotation;
         eventOut MF<Rotation> outMFRotation;
         eventOut SFString outSFString;
         eventOut MF<ZString> outMFString;
         eventOut SF<Time> outSFTime;
         eventOut SF<Vec2f> outSFVec2f;
         eventOut MF<Vec2f> outMFVec2f;
         eventOut SF<Vec3f> outSFVec3f;
         eventOut MF<Vec3f> outMFVec3f;
         exposedField SF<float> Factor1;
         exposedField SF<float> Factor2;
         exposedField SF<float> Factor3;
         exposedField SF<float> Factor4;
         exposedField SF<float> Offset1;
         exposedField SF<float> Offset2;
         exposedField SF<float> Offset3;
         exposedField SF<float> Offset4;
         exposedField SF<Bool> Sum;
         void OninSFBool();
         void OninSFColor();
         void OninMFColor();
         void OninSFFloat();
         void OninMFFloat();
         void OninSFInt32();
         void OninMFInt32();
         void OninSFRotation();
         void OninMFRotation();
         void OninSFString();
         void OninMFString();
         void OninSFTime();
         void OninSFVec2f();
         void OninMFVec2f();
         void OninSFVec3f();
         void OninMFVec3f();

         public:
              Valuator() {
                   Factor1 = 1;
                   Factor2 = 1;
                   Factor3 = 1;
                   Factor4 = 1;
                   Offset1 = 0;
                   Offset2 = 0;
                   Offset3 = 0;
                   Offset4 = 0;
                   Sum = FALSE;
              }
 };

 class Viewpoint : public MediaObject
 {
         BIFS_DECLARE_NODE

         eventIn SF<Bool> set_bind;
         exposedField SF<float> fieldOfView;
         exposedField SF<Bool> jump;
         exposedField SF<Rotation> orientation;
         exposedField SF<Vec3f> position;
         field SFString description;
         eventOut SF<Time> bindTime;
         eventOut SF<Bool> isBound;
         void Onset_bind();

         public:
              Viewpoint() {
                   fieldOfView = 0.785398;
                   jump = TRUE;
                   orientation = Rotation(0, 0, 1, 0);
                   position = Vec3f(0, 0, 10);
              }
 };

 class VisibilitySensor : public MediaObject
 {
         BIFS_DECLARE_NODE

         exposedField SF<Vec3f> center;
         exposedField SF<Bool> enabled;
         exposedField SF<Vec3f> size;
         eventOut SF<Time> enterTime;
         eventOut SF<Time> exitTime;
         eventOut SF<Bool> isActive;

         public:
              VisibilitySensor() {
                   center = Vec3f(0, 0, 0);
                   enabled = TRUE;
                   size = Vec3f(0, 0, 0);
              }
 };

 class Viseme : public MediaObject
 {
         BIFS_DECLARE_NODE

         exposedField SF<int> viseme_select1;
         exposedField SF<int> viseme_select2;
         exposedField SF<int> viseme_blend;
         exposedField SF<Bool> viseme_def;

         public:
              Viseme() {
                   viseme_select1 = 0;
                   viseme_select2 = 0;
                   viseme_blend = 0;
                   viseme_def = FALSE;
              }
 };

 class WorldInfo : public MediaObject
 {
         BIFS_DECLARE_NODE

         field MF<ZString> info;
         field SFString title;

         public:
              WorldInfo() {
              }
 };

// -------[ Body Animation Part Added ]-------
// By :	* Tolga Capin
//      * Fabrice Vergnenegre
// On : 09/25/1999
class BAP : public StreamConsumer {

		BIFS_DECLARE_NODE
		
		/*
		exposedField	SFInt32	sacroiliac_tilt ;
		exposedField	SFInt32	sacroiliac_torsion
		exposedField	SFInt32	sacroiliac_roll 
		exposedField	SFInt32	l_hip_flexion 
		exposedField	SFInt32	r_hip_flexion 
		exposedField	SFInt32	l_hip_abduct  
		exposedField	SFInt32	r_hip_abduct  
		exposedField	SFInt32	l_hip_twisting 
		exposedField	SFInt32	r_hip_twisting 
		exposedField	SFInt32	l_knee_flexion 
		exposedField	SFInt32	r_knee_flexion 
		exposedField	SFInt32	l_knee_twisting 
		exposedField	SFInt32	r_knee_twisting 
		exposedField	SFInt32	l_ankle_flexion 
		exposedField	SFInt32	r_ankle_flexion 
		exposedField	SFInt32	l_ankle_twisting
		exposedField	SFInt32	r_ankle_twisting
		exposedField	SFInt32	l_subtalar_flexion
		exposedField	SFInt32	r_subtalar_flexion
		exposedField	SFInt32	l_midtarsal_flexion
		exposedField	SFInt32	r_midtarsal_flexion
		exposedField	SFInt32	l_metatarsal_flexion 
		exposedField	SFInt32	r_metatarsal_flexion 
		exposedField	SFInt32	l_sternoclavicular_abduct 
		exposedField	SFInt32	r_sternoclavicular_abduct 
		exposedField	SFInt32	l_sternoclavicular_rotate 
		exposedField	SFInt32	r_sternoclavicular_rotate 
		exposedField	SFInt32	l_acromioclavicular_abduct 
		exposedField	SFInt32	r_acromioclavicular_abduct 
		exposedField	SFInt32	l_acromioclavicular_rotate 
		exposedField	SFInt32	r_acromioclavicular_rotate 
		exposedField	SFInt32	l_shoulder_flexion 
		exposedField	SFInt32	r_shoulder_flexion 
		exposedField	SFInt32	l_shoulder_abduct 
		exposedField	SFInt32	r_shoulder_abduct 
		exposedField	SFInt32	l_shoulder_twisting 
		exposedField	SFInt32	r_shoulder_twisting 
		exposedField	SFInt32	l_elbow_flexion 
		exposedField	SFInt32	r_elbow_flexion 
		exposedField	SFInt32	l_elbow_twisting 
		exposedField	SFInt32	r_elbow_twisting 
		exposedField	SFInt32	l_wrist_flexion 
		exposedField	SFInt32	r_wrist_flexion 
		exposedField	SFInt32	l_wrist_pivot  
		exposedField	SFInt32	r_wrist_pivot  
		exposedField	SFInt32	l_wrist_twisting
		exposedField	SFInt32	r_wrist_twisting
		exposedField	SFInt32	skullbase_roll
		exposedField	SFInt32	skullbase_torsion
		exposedField	SFInt32	skullbase_tilt
		exposedField	SFInt32	vc1roll
		exposedField	SFInt32	vc1torsion
		exposedField	SFInt32	vc1tilt
		exposedField	SFInt32	vc2roll
		exposedField	SFInt32	vc2torsion
		exposedField	SFInt32	vc2tilt
		exposedField	SFInt32	vc3roll
		exposedField	SFInt32	vc3torsion
		exposedField	SFInt32	vc3tilt
		exposedField	SFInt32	vc4roll
		exposedField	SFInt32	vc4torsion
		exposedField	SFInt32	vc4tilt
		exposedField	SFInt32	vc5roll
		exposedField	SFInt32	vc5torsion
		exposedField	SFInt32	vc5tilt
		exposedField	SFInt32	vc6roll
		exposedField	SFInt32	vc6torsion
		exposedField	SFInt32	vc6tilt
		exposedField	SFInt32	vc7roll
		exposedField	SFInt32	vc7torsion
		exposedField	SFInt32	vc7tilt
		exposedField	SFInt32	vt1roll
		exposedField	SFInt32	vt1torsion
		exposedField	SFInt32	vt1tilt
		exposedField	SFInt32	vt2roll
		exposedField	SFInt32	vt2torsion
		exposedField	SFInt32	vt2tilt
		exposedField	SFInt32	vt3roll
		exposedField	SFInt32	vt3torsion
		exposedField	SFInt32	vt3tilt
		exposedField	SFInt32	vt4roll
		exposedField	SFInt32	vt4torsion
		exposedField	SFInt32	vt4tilt
		exposedField	SFInt32	vt5roll
		exposedField	SFInt32	vt5torsion
		exposedField	SFInt32	vt5tilt
		exposedField	SFInt32	vt6roll
		exposedField	SFInt32	vt6torsion
		exposedField	SFInt32	vt6tilt
		exposedField	SFInt32	vt7roll
		exposedField	SFInt32	vt7torsion
		exposedField	SFInt32	vt7tilt
		exposedField	SFInt32	vt8roll
		exposedField	SFInt32	vt8torsion
		exposedField	SFInt32	vt8tilt
		exposedField	SFInt32	vt9roll
		exposedField	SFInt32	vt9torsion
		exposedField	SFInt32	vt9tilt
		exposedField	SFInt32	vt10roll
		exposedField	SFInt32	vt10torsion
		exposedField	SFInt32	vt10tilt
		exposedField	SFInt32	vt11roll
		exposedField	SFInt32	vt11torsion
		exposedField	SFInt32	vt11tilt
		exposedField	SFInt32	vt12roll
		exposedField	SFInt32	vt12torsion
		exposedField	SFInt32	vt12tilt
		exposedField	SFInt32	vl1roll
		exposedField	SFInt32	vl1torsion
		exposedField	SFInt32	vl1tilt
		exposedField	SFInt32	vl2roll
		exposedField	SFInt32	vl2torsion
		exposedField	SFInt32	vl2tilt
		exposedField	SFInt32	vl3roll
		exposedField	SFInt32	vl3torsion
		exposedField	SFInt32	vl3tilt
		exposedField	SFInt32	vl4roll
		exposedField	SFInt32	vl4torsion
		exposedField	SFInt32	vl4tilt
		exposedField	SFInt32	vl5roll
		exposedField	SFInt32	vl5torsion
		exposedField	SFInt32	vl5tilt
		exposedField	SFInt32	l_pinky0_flexion
		exposedField	SFInt32	r_pinky0_flexion
		exposedField	SFInt32	l_pinky1_flexion
		exposedField	SFInt32	r_pinky1_flexion
		exposedField	SFInt32	l_pinky1_pivot
		exposedField	SFInt32	r_pinky1_pivot
		exposedField	SFInt32	l_pinky1_twisting
		exposedField	SFInt32	r_pinky1_twisting
		exposedField	SFInt32	l_pinky2_flexion
		exposedField	SFInt32	r_pinky2_flexion
		exposedField	SFInt32	l_pinky3_flexion
		exposedField	SFInt32	r_pinky3_flexion
		exposedField	SFInt32	l_ring0_flexion
		exposedField	SFInt32	r_ring0_flexion
		exposedField	SFInt32	l_ring1_flexion
		exposedField	SFInt32	r_ring1_flexion
		exposedField	SFInt32	l_ring1_pivot
		exposedField	SFInt32	r_ring1_pivot
		exposedField	SFInt32	l_ring1_twisting
		exposedField	SFInt32	r_ring1_twisting
		exposedField	SFInt32	l_ring2_flexion
		exposedField	SFInt32	r_ring2_flexion
		exposedField	SFInt32	l_ring3_flexion
		exposedField	SFInt32	r_ring3_flexion
		exposedField	SFInt32	l_middle0_flexion
		exposedField	SFInt32	r_middle0_flexion
		exposedField	SFInt32	l_middle1_flexion
		exposedField	SFInt32	r_middle1_flexion
		exposedField	SFInt32	l_middle1_pivot
		exposedField	SFInt32	r_middle1_pivot
		exposedField	SFInt32	l_middle1_twisting
		exposedField	SFInt32	r_middle1_twisting
		exposedField	SFInt32	l_middle2_flexion
		exposedField	SFInt32	r_middle2_flexion
		exposedField	SFInt32	l_middle3_flexion
		exposedField	SFInt32	r_middle3_flexion
		exposedField	SFInt32	l_index0_flexion
		exposedField	SFInt32	r_index0_flexion
		exposedField	SFInt32	l_index1_flexion
		exposedField	SFInt32	r_index1_flexion
		exposedField	SFInt32	l_index1_pivot
		exposedField	SFInt32	r_index1_pivot
		exposedField	SFInt32	l_index1_twisting
		exposedField	SFInt32	r_index1_twisting
		exposedField	SFInt32	l_index2_flexion
		exposedField	SFInt32	r_index2_flexion
		exposedField	SFInt32	l_index3_flexion
		exposedField	SFInt32	r_index3_flexion
		exposedField	SFInt32	l_thumb1_flexion
		exposedField	SFInt32	r_thumb1_flexion
		exposedField	SFInt32	l_thumb1_pivot
		exposedField	SFInt32	r_thumb1_pivot
		exposedField	SFInt32	l_thumb1_twisting
		exposedField	SFInt32	r_thumb1_twisting
		exposedField	SFInt32	l_thumb2_flexion
		exposedField	SFInt32	r_thumb2_flexion
		exposedField	SFInt32	l_thumb3_flexion
		exposedField	SFInt32	r_thumb3_flexion
		exposedField	SFInt32	HumanoidRoot_tr_vertical
		exposedField	SFInt32	HumanoidRoot_tr_lateral
		exposedField	SFInt32	HumanoidRoot_tr_frontal
		exposedField	SFInt32	HumanoidRoot_rt_body_turn
		exposedField	SFInt32	HumanoidRoot_rt_body_roll
		exposedField	SFInt32	HumanoidRoot_rt_body_tilt
		exposedField	SFInt32	extensionBap187
		exposedField	SFInt32	extensionBap188
		exposedField	SFInt32	extensionBap189
		exposedField	SFInt32	extensionBap190
		exposedField	SFInt32	extensionBap191
		exposedField	SFInt32	extensionBap192
		exposedField	SFInt32	extensionBap193
		exposedField	SFInt32	extensionBap194
		exposedField	SFInt32	extensionBap195
		exposedField	SFInt32	extensionBap196
		exposedField	SFInt32	extensionBap197
		exposedField	SFInt32	extensionBap198
		exposedField	SFInt32	extensionBap199
		exposedField	SFInt32	extensionBap200
		exposedField	SFInt32	extensionBap201
		exposedField	SFInt32	extensionBap202
		exposedField	SFInt32	extensionBap203
		exposedField	SFInt32	extensionBap204
		exposedField	SFInt32	extensionBap205
		exposedField	SFInt32	extensionBap206
		exposedField	SFInt32	extensionBap207
		exposedField	SFInt32	extensionBap208
		exposedField	SFInt32	extensionBap209
		exposedField	SFInt32	extensionBap210
		exposedField	SFInt32	extensionBap211
		exposedField	SFInt32	extensionBap212
		exposedField	SFInt32	extensionBap213
		exposedField	SFInt32	extensionBap214
		exposedField	SFInt32	extensionBap215
		exposedField	SFInt32	extensionBap216
		exposedField	SFInt32	extensionBap217
		exposedField	SFInt32	extensionBap218
		exposedField	SFInt32	extensionBap219
		exposedField	SFInt32	extensionBap220
		exposedField	SFInt32	extensionBap221
		exposedField	SFInt32	extensionBap222
		exposedField	SFInt32	extensionBap223
		exposedField	SFInt32	extensionBap224
		exposedField	SFInt32	extensionBap225
		exposedField	SFInt32	extensionBap226
		exposedField	SFInt32	extensionBap227
		exposedField	SFInt32	extensionBap228
		exposedField	SFInt32	extensionBap229
		exposedField	SFInt32	extensionBap230
		exposedField	SFInt32	extensionBap231
		exposedField	SFInt32	extensionBap232
		exposedField	SFInt32	extensionBap233
		exposedField	SFInt32	extensionBap234
		exposedField	SFInt32	extensionBap235
		exposedField	SFInt32	extensionBap236
		exposedField	SFInt32	extensionBap237
		exposedField	SFInt32	extensionBap238
		exposedField	SFInt32	extensionBap239
		exposedField	SFInt32	extensionBap240
		exposedField	SFInt32	extensionBap241
		exposedField	SFInt32	extensionBap242
		exposedField	SFInt32	extensionBap243
		exposedField	SFInt32	extensionBap244
		exposedField	SFInt32	extensionBap245
		exposedField	SFInt32	extensionBap246
		exposedField	SFInt32	extensionBap247
		exposedField	SFInt32	extensionBap248
		exposedField	SFInt32	extensionBap249
		exposedField	SFInt32	extensionBap250
		exposedField	SFInt32	extensionBap251
		exposedField	SFInt32	extensionBap252
		exposedField	SFInt32	extensionBap253
		exposedField	SFInt32	extensionBap254
		exposedField	SFInt32	extensionBap255
		exposedField	SFInt32	extensionBap256
		exposedField	SFInt32	extensionBap257
		exposedField	SFInt32	extensionBap258
		exposedField	SFInt32	extensionBap259
		exposedField	SFInt32	extensionBap250
		exposedField	SFInt32	extensionBap251
		exposedField	SFInt32	extensionBap262
		exposedField	SFInt32	extensionBap263
		exposedField	SFInt32	extensionBap264
		exposedField	SFInt32	extensionBap265
		exposedField	SFInt32	extensionBap266
		exposedField	SFInt32	extensionBap267
		exposedField	SFInt32	extensionBap268
		exposedField	SFInt32	extensionBap269
		exposedField	SFInt32	extensionBap270
		exposedField	SFInt32	extensionBap271
		exposedField	SFInt32	extensionBap272
		exposedField	SFInt32	extensionBap273
		exposedField	SFInt32	extensionBap274
		exposedField	SFInt32	extensionBap275
		exposedField	SFInt32	extensionBap276
		exposedField	SFInt32	extensionBap277
		exposedField	SFInt32	extensionBap278
		exposedField	SFInt32	extensionBap279
		exposedField	SFInt32	extensionBap280
		exposedField	SFInt32	extensionBap281
		exposedField	SFInt32	extensionBap282
		exposedField	SFInt32	extensionBap283
		exposedField	SFInt32	extensionBap284
		exposedField	SFInt32	extensionBap285
		exposedField	SFInt32	extensionBap286
		exposedField	SFInt32	extensionBap287
		exposedField	SFInt32	extensionBap288
		exposedField	SFInt32	extensionBap289
		exposedField	SFInt32	extensionBap290
		exposedField	SFInt32	extensionBap291
		exposedField	SFInt32	extensionBap292
		exposedField	SFInt32	extensionBap293
		exposedField	SFInt32	extensionBap294
		exposedField	SFInt32	extensionBap295
		exposedField	SFInt32	extensionBap296
		*/
public :
		// BAP values
		SF<int>		BAPValues[NUM_BAPS];
		// BAP mask
		BOOL		BAPMask[NUM_BAPS];

		AnimationStreamDecoder *pAS;	// keep a pointer to parse the data with the AnimationStreamDecoder functions
		ArithModel models[NUM_BAPS];        // an ArithModel for each FAP value
		int first_pbap;

		void animate(AnimationStreamDecoder *pAS,BOOL intra);
		void parseAnimationMask(AnimationStreamDecoder *pAS) ;

		BAP::BAP() {
			for (int i=0 ; i<NUM_BAPS ; i++) {
				BAPMask[i]=FALSE;
				BAPValues[i]=0;
			}
		}

		// functions for parsing the FAP data that I had to group under the FAP class
		// to take advantage of the AnimationStreamDecoder pointer: pAS and its methods

		//int fba_object_plane_data(FOP_DATA* fop_data,BOP_DATA *bop_data, unsigned int *PTS) ;
		//void decode_pbap(BOP_DATA *bop_data) ;
		//void decode_ibap(BOP_DATA *bop_data) ;
		//void decode_new_minmax() ;
		//void decode_bap_group_mask(int n) ;
 };

  class Body : public Grouping {
         BIFS_DECLARE_NODE
		
  public:
		 exposedField SFNode<class BDP> bdp;
		 exposedField SFNode<class BAP> bap;
		 exposedField MFNodeType<SF3DNode> renderedBody ;

  public:
         Body() {}
 };

  class BDP : public Grouping {

		BIFS_DECLARE_NODE

		exposedField MFNodeType<SFBodyDefTableNode> bodyDefTables;
		exposedField MFNodeType<SF3DNode> bodySceneGraph;

		public :
			BDP()
			{}
  };

  class BodyDefTable : public Grouping {

		BIFS_DECLARE_NODE

		exposedField SFString bodySceneGraphNodeName;
		exposedField MF<int> bapIDs;
		exposedField MF<int> vertexIds;
		exposedField MF<int> bapCombinations;
		exposedField MF<Vec3f> displacements;
		exposedField SF<int> numInterpolateKeys;

		public :
			BodyDefTable()
			{}
  };

  class BodySegmentConnectionHint : public Grouping {

		BIFS_DECLARE_NODE

		exposedField SFString firstSegmentNodeName;
		exposedField SFString secondSegmentNodeName;
		exposedField MF<int> firstVertexIdList;
		exposedField MF<int> secondVertexIdList;

		public :
			BodySegmentConnectionHint()
			{}

  };
// -------[ End Of Added Part ]-------



#endif                                     // _Nodes_h sentry.
/* End of automatically generated file */
