#include "Vector3d.h"

// Default constructor
Vector3d::Vector3d()
  : x(0.f),
    y(0.f),
    z(0.f)
{
}

// Explicit members constructor
Vector3d::Vector3d(float cx, float cy, float cz)
  : x(cx),
    y(cy),
    z(cz)
{
}

// Pointed copy constructor
Vector3d::Vector3d(Vector3d * p)
  : x(p->GetX()),
    y(p->GetY()),
    z(p->GetZ())
{
}

// Copy constructor
Vector3d::Vector3d(const Vector3d & p)
  : x(p.GetX()),
    y(p.GetY()),
    z(p.GetZ())
{
}

// Equality operator
BOOL Vector3d::operator == (const Vector3d & v) const
{
	return ( (x==v.GetX()) && (y==v.GetY()) && (z==v.GetZ()) );
}

// Dot product
float Vector3d::operator * (const Vector3d & v) const 
{
	return (x*v.GetX() + y*v.GetY() + z*v.GetZ() );
}

// Scalar product
const Vector3d & Vector3d::operator * (float a) const
{
	return Vector3d(x*a , y*a , z*a );
}

// Scalar division
const Vector3d & Vector3d::operator / (float a) const
{
	return Vector3d(x/a , y/a , z/a );
}

// Vector addition
const Vector3d & Vector3d::operator + (const Vector3d & v) const
{
	return Vector3d(x+v.GetX() , y+v.GetY() , z+v.GetZ() );
}

// Vector subtraction
const Vector3d & Vector3d::operator - (const Vector3d & v) const
{
	return Vector3d(x-v.GetX() , y-v.GetY() , z-v.GetZ() );
}

// Cross product
const Vector3d & Vector3d::operator ^ (const Vector3d & v) const
{
	return Vector3d(y*v.GetZ() - z*v.GetY(),
					z*v.GetX() - x*v.GetZ(),
					x*v.GetY() - y*v.GetX() );
}