/* bapio.C */


/*************************** I/O for body animation parts ********************/

#include <bddef.h>

/*****************************************************************************/



/* skip commented portion in a file (i.e. those starting with '#') */

void skip_comments (ifstream & inf)

{
  char c;

  inf.get (c);
  while (c == '#') {
    inf.get (Pad_buf, MAX_BUF, '\n'); /* read until (excluding) '\n' */
    inf.get (c);               /* read '\n' */
    //cout << Pad_buf << c; 
    inf.get (c);
  }
}



/* read a frame from bap file */

int read_frame (ifstream & inf, int & frameno, 
		unsigned char flags[], Bap_comb_t values[])

{
  int   flagno, buf_ix, active_ids;
  char  c;

  /* read frame line: every alternate char is bap id flag */
  inf.get (Pad_buf, MAX_BUF, '\n'); 
  if (inf.eof()) 
    return FALSE;
  inf >> frameno;  /* read frame no */
  //cout << "Frame number : " << frameno << '\n';
  //cout << "Values of ids: ";
  /* set flags and if any active bap ids are present, read their values */
  flagno = 0; buf_ix = 0; active_ids = 0;
  while (flagno < N_BAP_IDS) {
    if (Pad_buf[buf_ix] == '1')
      flags[flagno] = TRUE;
    else
      flags[flagno] = FALSE;
    if (flags[flagno]) {
      inf >> values[flagno];
      //cout << values[flagno] << "    ";
      active_ids++;
    }
    else
      values[flagno] = 0;
    flagno++; 
    buf_ix += 2; /* skip alternate char, which is whitespace */
  }
  //cout << "\nActive bap ids: " << active_ids << '\n';

  /* read till end of line */
  inf.get (Pad_buf, MAX_BUF, '\n');
  inf.get (c); /* eat up '\n' */
  //cout << '\n';
  return TRUE;
}

/* read all frames from a given BAP file 
 * for animating using BAT tables;
 * read transformation of BAP from inventor file of corresponding frame:
 * Deformation_frame_no (global) specifies number 
 * of frame for which deformation
 * is to be simulated
 */

void Read_bap_frames (char file[])

{
  ifstream   inf; /* input file stream */
  ofstream   ouf; /* output file stream */
  int        frameno, i; 
  unsigned   char       bap_id_flags[N_BAP_IDS];
  Bap_comb_t bap_id_vals[N_BAP_IDS];
  char       c;
  char       frame_file[1000]; /* file with geometry for single frame */
  ifstream   frame_inf;        /* frame file */

  inf.open(file);

  if (!inf) {
    cout << "ERROR (Read_bap_frames): can not open " << file << '\n';
    cout.flush();
  }

  //cout << "Producing deformation for frame " << Deformation_frame_no << '\n';
  /* while eof not reached, read all Bat tables */
  skip_comments (inf);
  /* read the line following comments */
  inf.get (Pad_buf, MAX_BUF, '\n');
  inf.get (c); 
  while (read_frame (inf, frameno, bap_id_flags, bap_id_vals)) {
    if (frameno == Deformation_frame_no) {
      /* read transformations of BAPs for this frame from .iv file */
      sprintf (frame_file, "body/naomi_smart.inf%d.iv", frameno);
      frame_inf.open (frame_file);
      if (!frame_inf) {
	cout << "ERROR (Read_bap_frames): cannot open " << frame_file << '\n';
	cout.flush();
      }
      else {
	skip_comments (frame_inf);
	for (i = 0; i < MAX_BODY_PARTS; i++) 
	  Bap_arr[i].extract_bap_transformation_from_file_iv (frame_inf);
	frame_inf.close();
      }
      
      /* animate all body animation parts */
      for (i = 0; i < MAX_BODY_PARTS; i++) 
	Bap_arr[i].bap_deformation (bap_id_flags, bap_id_vals);

      /* output inventor file */
      ouf.open ("output.iv");
      ouf << "#Inventor V2.1 ascii\n\n";
      for (i = 0; i < MAX_BODY_PARTS; i++) 
	ouf << Bap_arr[i];

      ouf.close();
      break; 
    }
    /* set Prv_frame_bap_points components which are set in this
     frame just executed */
    for (i = 0; i < N_BAP_IDS; i++) {
      if (bap_id_flags[i])
	/* this component has value; set it for next frame */
	Prv_frame_bap_points[i] =  bap_id_vals[i];
    }
  } /* while read frame */

  inf.close();
}

