/* invio.C */

/*********************** I/O for Inventor files *****************************/

#include <bddef.h>

/* external */

void skip_comments (ifstream & inf);                     /* bapio.C */

/****************************************************************************/


/* get index of body part from sequence */

int index_from_body_part_sequence (char body_part[])

{
  for (int i = 0; i < MAX_BODY_PARTS; i++) 
    if (strcmp (body_part, Body_part_sequence[i]) == 0)
      return i;
  cout << "ERROR (Index_from_body_part_sequence): body part does not exist\n";
  cout.flush();
  return -1;
}


/* read normal/default displacements for existing baps */

void Bap::read_normal_bap_displacements (ifstream & inf, int bap_no)

{
  int  i;
  int  n_disps, n_indices;
  char c, c1, c2;

  /* try reading "Separator" */
  inf >> Pad_buf;
  if (inf.eof())
    return;
  Bap_no = bap_no;
  /* skip till '[' ignoring 'Transform' segment and ending at start of
   * point sequence
   */
  inf.ignore (LARGE_INT, '['); 
  /* read all vertices */
  n_disps = 0;
  inf >> c;
  while (c != ']') {
    inf.putback(c);
    /* read 3 floats followed by comma */
    inf >> Pad_displacements[n_disps] >> Pad_displacements[n_disps+1]
	>> Pad_displacements[n_disps+2] >> c;
    n_disps += 3;
    inf >> c;
  }

  /* transfer values to 'normal_displacements' after allocating storage;
     also allocate storage for total displacement, though unused now.
     */
  normal_displacements = new float[n_disps]; /* default displacement */
  total_displacements  = new float[n_disps]; /* total; unused now */
  N_vertices = n_disps/3;
  if (3*N_vertices != n_disps) {
    cout << "ERROR (read_normal_bap_displacements): 3D points missing\n";
    return;
  }
  for (i = 0; i < n_disps; i++)
    normal_displacements[i] = Pad_displacements[i];
  /* read connectivity indices */
  inf.ignore (LARGE_INT, '[');  /* skip till start of indices */
  /* read all indices */
  n_indices = 0; N_polygons = 0;
  inf >> c;
  while (c != ']') {
    inf.putback(c);
    /* read ints until (including) -1 and then read comma */
    inf >> Pad_Vertex_ids[n_indices] >> c;   /* read int and comma */
    while (Pad_Vertex_ids[n_indices] != -1) {
      n_indices++;
      inf >> Pad_Vertex_ids[n_indices] >> c; /* read int and comma */
    }
    n_indices++;
    N_polygons++;
    inf >> c; /* next char */
  }
  /* transfer values to 'vertex_connectivity' after allocating storage */
  vertex_connectivity = new int[n_indices];
  for (i = 0; i < n_indices; i++)
    vertex_connectivity[i] = Pad_Vertex_ids[i];

  inf >> c1 >> c2;           /* read two '}' that terminate Separator */
}


/* read default displacements of geometry of all body parts 
   from inventor file */

void Read_default_displacements (char file[])

{
  ifstream   inf; /* input file stream */
  int        bap_no = 0;

  inf.open(file);
  if (!inf) {
    cout << "ERROR(Read_default_displacements): can not open " << file << '\n';
    cout.flush();
  }

  /* read inventor file and assign displacements to each part */
  skip_comments (inf);
  while (!inf.eof()) {
    Bap_arr[bap_no].read_normal_bap_displacements (inf, bap_no);
    bap_no++;
  }
}


/* read transformation of BAP from the given file;
 * Assumed to be in Inventor format;
 * the file pointer is assumed to be at the beginning
 * of corresponding "Separator"
 */

void Bap::extract_bap_transformation_from_file_iv (ifstream &inf)

{
  char c1, c2;

  /* skip "Separator", "{", "Transform", "{" */
  inf >> Pad_buf >> Pad_buf >> Pad_buf >> Pad_buf;
  /* read transformation info until (and excluding) closing '}' */
  inf.get (bap_transf, MAX_BUF, '}');
  /* skip rest of Separator parts */
  inf.ignore (LARGE_INT, ']');   /* skip coordinates values */
  inf.ignore (LARGE_INT, ']');   /* skip index values */
  inf >> c1 >> c2;            /* read two '}' terminating Separator */
}



/*------------------------ inventor output ---------------------------*/


/* print bap total deformation info in inventor file format */

void Bap::output_bap_iv (ofstream & ouf)

{
  int i, j;

  ouf << "Separator {\n"; 
  // trsnaformation 
  ouf << " Transform {"; 
  ouf << bap_transf << "}\n"; 
  // coordinates
  ouf << " Coordinate3 {\n"; 
  ouf << "  point [\n"; 
  // points
  j = 0;
  for (i = 0; i < N_vertices; i++) {
    ouf << "   " << total_displacements[j] << ' ' 
	<< total_displacements[j+1] << ' ' << total_displacements[j+2] 
	<< ",\n"; 
    j += 3;
  }
  ouf << "  ]\n" << " }\n"; 
  // indices
  ouf << " IndexedFaceSet {\n"; 
  ouf << "   coordIndex [\n"; 
  j = 0;
  for (i = 0; i < N_polygons; i++) {
    ouf << "   "; 
    for (; vertex_connectivity[j] >= 0; j++) {
      ouf << vertex_connectivity[j] << ','; 
    }
    ouf <<  vertex_connectivity[j] << ",\n";   // terminator
    j++;
  }
  ouf << "   ]\n" << "  }\n"; 
  
  // separator terminator
  ouf << "}\n";   
}



