
/*------------------------------------------------------------------------------*/
/* 										*/
/* 										*/
/*  This software module was originally developed by Joaquim Esmerado (EPFL-LIG)*/
/*  in the course of development of the MPEG-4 SNHC 				*/
/*  VM. This software module is an implementation of a part of one or more 	*/
/*  MPEG-4 SNHC VM tools as specified by the MPEG-4 SNHC VM. ISO/IEC gives 	*/
/*  users of the MPEG-4 SNHC VM free license to this software module or 	*/
/*  modifications thereof for use in hardware or software products 		*/
/*  claiming conformance to the MPEG-4 SNHC VM. Those intending to use 		*/
/*  this software module in hardware or software products are advised that 	*/
/*  its use may infringe existing patents. The original developer of this 	*/
/*  software module and his/her company, the subsequent editors and their 	*/
/*  companies, and ISO/IEC have no liability for use of this software 		*/
/*  module or modifications thereof. Copyright is not released for non 		*/
/*  MPEG-4 SNHC VM conforming products. EPFL-LIG 				*/
/*  retains full right to use the code for its own purpose, assign or 		*/
/*  donate the code to a third party and to inhibit third parties from 		*/
/*  using the code for non MPEG-4 SNHC VM conforming products. This 		*/
/*  copyright notice must be included in all copies or derivative 		*/
/*  works. Copyright (1997).  							*/






/* -------------------------------------------------------------------

Deals with Masks interpretation and rule implementation routines. It
has a bit compromized version which should be amplified soon.

The ImplementRules routine seems to me to be the star

------------------------------------------------------------------- */

#include "scn1.h"
#include "etall.h"
#include "private.h"


#include "params.h"
#include "mais.h"


#define         UPD_VT          1
#define         UPD_EUL         2
#define         UPD_ALL         3





/* This routine is supposed to get the first 8 bits of a mask */

static JOINT * joint_current;



static FILE *inputBap, *outputBap;

FILE *OpenBapFile( name )
  char *name;
{
  FILE *inputBap;

  if (!(inputBap = fopen( name, "rb" )) ) {
    fprintf( stderr, "Could not open file %s\n", name);
    return ( inputBap );
  }

  return ( inputBap );
}

void CloseBapFile( bap_file )
  FILE *bap_file;
{
  fclose( bap_file );
}


int TestHeader( line )
  char *line;
{
  int retval = 1, i, l, ok;
  float version;
  int fps,numframes;
  char basename[50];

  i = sscanf( line, "%f%s%d%d", &version, basename, &fps, &numframes );
  if ( i<4 ) {
    fprintf( stderr, "Error in header parsing :\n\t[%s]\n", line );
    retval = 0;
  }
  else
     retval = 1;
  if (fps <= 0) retval = 0;
  else
     if (numframes  <= 0) retval = 0;
     else
        if (version  <= 0.) retval = 0;
        else
	   for ( l=0; l<strlen(line); l++) {
	     if (line[l] != ' ' && line[l] != '0' && line[l] != '1') {
	       ok = 1; retval =1 ; break;}
	     retval = 0;
	   }
#ifdef VRML2_OUTPUT
  vrml2_noframes = numframes;
#endif
  return retval;

}



int GetHeaderInfo( bap_file )
  FILE *bap_file;
{
  char line[MAX_LINE_SIZE];

  if( bap_file )
     while ( (fgets( line, MAX_LINE_SIZE, bap_file ) ) ) {
       line[strlen(line)-1] = '\0'; /* get rid of \n */
       if ( line[0] == '#' || line[0] == '\0' )
	  continue;
       else if (! TestHeader( line ) )
	  return ( 0 );
       else {
	 return ( 1 );
       }
     }
  return ( 0 );
}



#define 	BLANK 		1
#define 	NONBLANK 	0
BlanksTest( line )
  char *line;
{
  short stat = BLANK, i;
  for ( i=0; line[i] != '\0'; i++) {
    switch (line[i]) {
      case ' ':
      case '\t':
      case '\n': break;
      default: stat = NONBLANK;		
    }
  }
  return stat;
}

int GetBapMask( bap_file , mask, intFile )
  FILE *bap_file;
  char *mask;
  int *intFile;
{
  char line[MAX_LINE_SIZE];
  int flagValidMask = 0;
  *intFile = 1;


  if( bap_file )

     while ( (fgets( line, MAX_LINE_SIZE, bap_file ) ) ) {
       line[strlen(line)-1] = '\0'; /* get rid of \n */
       if ( line[0] == '#' ) {
	 if ( line[1] == '@' && line[2] == '!' && line[3] == '%' )
	    *intFile = 0;
	 continue;}
       if ( (BlanksTest(line)) == BLANK )
	  continue;
       else if (! TestMask( line ) )
	  return ( 0 );
       else {
	 strcpy( mask, line);
	 return ( 1 );
       }
     }
  return ( EOF );
}

int ConvertToScaledFloat( strNumber, outFloat )
  char strNumber[30];
  float *outFloat;
{
  float flo;

  if ( (sscanf( strNumber, "%f", &flo )) ) {
    *outFloat = (flo / 100000.);
    return ( 1 );
  }
  else
     return ( 0 );
}


int ConvertToFloat( strNumber, flo )
  char strNumber[30];
  float *flo;
{
  if ( (sscanf( strNumber, "%f", flo )) )
     return ( 1 );

  return ( 0 );
}

int ConvertToInt( strNumber, Frame )
  char strNumber[30];
  int *Frame;
{
  if ( (sscanf( strNumber, "%d", Frame )) )
     return ( 1 );

  return ( 0 );
}




/* ------------------------------------------------------------------- */

#define EPSILON 1e-4

#define NUM_FLOAT       250


int ReadParameters( bap_file, Frame, flo ,numPar, intFile)
  FILE *bap_file;
  int *Frame, *numPar;
  float flo[NUM_FLOAT];
  int intFile;
{
  int c, j=0, i=0, FirstParam=1;
  int pointIn= 0, insideNum=0, inComment=0, inUsefulLine=0;
  char strNumber[30];

  for ( c=fgetc( bap_file );/*c!='\n' &&*/ c != EOF; c=fgetc( bap_file ))
  {
	    
    if ( (c >= '0' && c <= '9') || c == '.' || c == '-' || c == '+' ) {
      if ( !inComment )
	 switch ( insideNum ) {
	   case 0: /* Start a new number */
	   i = 0; insideNum = 1; inUsefulLine = 1;
	   strNumber[i++] = c;
	   if ( c == '.' ) 
	      if ( pointIn == 0 ) {
		pointIn = 1; }
	      else {
		fprintf( stderr, "Error in parameter\n" );
		return ( 0 );
	      }
	   break;
	   default: /* insideNum != 0 -> Try to continue a number */
	   if ( c == '-' || c == '+' )
	   {
	     fprintf( stderr, "Error in signal position\n" );
	     return ( 0 );
	   }
	   if ( c == '.' ) 
	      if ( pointIn == 0 ) {
		strNumber[i++] = c;
		pointIn = 1; }
	      else {
		fprintf( stderr, "Error in parameter\n" );
		return ( 0 );
	      }
	   else {
	     strNumber[i++] = c;
	   }
	   break;
		
	 }
    }
    else { /* It's not a number char. Let's hope it's a space...*/
      if ( c == ' ' || c == '\t' || c == '\n' ) {
	if ( inComment ) {
	  if ( c == '\n' ) 
	     inComment = 0; 
	  continue; 
	}
	/*		    else*/
	/*		    if ( c == '\n' && inUsefulLine ) break;*/

	switch ( insideNum ) {
	  case 0:
	  break;
	  default:
	  insideNum = 0;
	  strNumber[i] = '\0'; i=0; pointIn = 0;
	  if ( !FirstParam ) {
	    if ( intFile ) {
	      if (!(ConvertToScaledFloat( strNumber, &flo[j++] ))){
		fprintf( stderr, "Error in param. conv.\n" );
		return ( 0 ); }
	    }
	    else
	       if (!(ConvertToFloat( strNumber, &flo[j++] ))) {
		 fprintf( stderr, "Error in param. conv.\n" );
		 return ( 0 ); }
	  }
	  else {
	    if (!(ConvertToInt( strNumber, Frame ))) {
	      fprintf( stderr, "Error in Frame conv.\n" );
	      return ( 0 ); }
	    FirstParam = 0;
	  }
	  break;
	}

	if ( c == '\n' && inUsefulLine ) break;
      }
      else {
	/* Ok, so it's not a space. Is it a comment? */
	if ( c == '#' ) inComment = 1;
	if ( !inComment ) {
	  fprintf( stderr, "Illegal character\n" );
	  return ( 0 ); }
      }
    }
  }

  *numPar = j;

  return (c);

}





int ConvertToMaskFormat( mask, bytemask, maskrest, bytecounter, maskrestlen ,
			 num1Bits)
  /* num1Bits= number of 1s in suplementary mask */
  char *mask;
  unsigned char *bytemask;
  unsigned char *maskrest;
  int *bytecounter, *maskrestlen, *num1Bits;
{
  int len = strlen( mask );
  int bytepos = 0, i, restpos = 0;

  *bytecounter = 0, *bytemask = 0;
  *maskrestlen = 0;


  for ( *bytecounter=0, i=/*i+1*/0 ; i <len; i++ ) {

    switch ( mask[i] ) {
      case '1':
      *( maskrest + *bytecounter ) <<= 1;
      *( maskrest + *bytecounter ) += 1;
      restpos ++;
      if ( restpos == 8 ) {
	(*bytecounter) ++; restpos = 0;
	*(maskrest + *bytecounter) = 0; }
      break;
      case '0':
      *( maskrest + *bytecounter ) <<= 1;
      restpos ++;
      if ( restpos == 8 ) {
	(*bytecounter) ++; restpos = 0;
	*(maskrest + *bytecounter) = 0; }
      break;
      default: break;
    }

  }

  *maskrestlen = *bytecounter * BYTELEN + restpos /* +1*/;

  *( maskrest + *bytecounter ) <<= (BYTELEN - restpos);

  return ( 1 );
}




int TestMask( mask )
  char *mask;
{
  int retval = 1, i;

  /* Test 1: Teste if all the characters are 1, 0 or space */

  for ( i=0; mask[i] != '\0'; i++) {
    switch ( mask[i] ) {
      case ' ':
      case '\t':
      case '1':
	  case '\n':
      case '0': break;
      default:  fprintf( stderr,
			 "Error in mask: illegal character: %c\n %d %d ", mask[i], mask[i], '\n' );
      retval = 0;
      break;
    }
    if ( !retval ) return retval;
  }
  if (!i) retval = 0; 
  return retval;
}




float ChangeJointVal( body, joint_ind, new_val )
  BODY *body;
  int joint_ind;
  float new_val;
{

  int i, j;

  joint_current = body->tjoint[joint_ind];

  h3d_actual_ref(unit_get_root(body->golem),DIRECT_PROPAGATION,FALSE);/* TO REMOVE */

  joint_val_actual( joint_current, new_val );
  h3d_actual_ref(unit_get_root(body->golem),DIRECT_PROPAGATION,FALSE);
  return ( joint_get_val(joint_current) );
}


float ChangeRHandJointVal( body, joint_ind, new_val )
  BODY *body;
  int joint_ind;
  float new_val;
{
  joint_current = body->tjoint_rhand[joint_ind];
  joint_val_actual( joint_current, new_val );
  h3d_actual_ref(unit_get_root(body->golem),DIRECT_PROPAGATION,FALSE);
  return ( joint_get_val(joint_current) );
}



float ChangeLHandJointVal( body, joint_ind, new_val )
  BODY *body;
  int joint_ind;
  float new_val;
{
  joint_current = body->tjoint_lhand[joint_ind];
  joint_val_actual( joint_current, new_val );
  h3d_actual_ref(unit_get_root(body->golem),DIRECT_PROPAGATION,FALSE);
  return ( joint_get_val(joint_current) );
}




/* To compute size:*/
static int UpdtJointList[NUM_BAPS] = {
  BODY_JOINT_PELVIC_TILT,
  BODY_JOINT_PELVIC_TORSION,
  BODY_JOINT_PELVIC_ROLL,
  BODY_JOINT_L_HIP_FLEXION,
  BODY_JOINT_R_HIP_FLEXION,
  BODY_JOINT_L_HIP_ABDUCT,
  BODY_JOINT_R_HIP_ABDUCT,
  BODY_JOINT_L_HIP_TWISTING,
  BODY_JOINT_R_HIP_TWISTING,
  BODY_JOINT_L_KNEE_FLEXION,
  BODY_JOINT_R_KNEE_FLEXION,
  BODY_JOINT_L_KNEE_TWISTING,
  BODY_JOINT_R_KNEE_TWISTING,
  BODY_JOINT_L_ANKLE_FLEXION,
  BODY_JOINT_R_ANKLE_FLEXION,
  BODY_JOINT_L_KNEE_TWISTING,
  BODY_JOINT_R_KNEE_TWISTING,
  BODY_JOINT_L_ANKLE_SUBTALAR,
  BODY_JOINT_R_ANKLE_SUBTALAR,
  BODY_JOINT_L_MID_FOOT,
  BODY_JOINT_R_MID_FOOT,
  BODY_JOINT_L_TOE_FLEXION,
  BODY_JOINT_R_TOE_FLEXION,
  BODY_JOINT_L_CLAV_ABDUCT,
  BODY_JOINT_R_CLAV_ABDUCT,
  BODY_JOINT_L_CLAV_ROTATE,
  BODY_JOINT_R_CLAV_ROTATE,
  BODY_JOINT_L_SCAP_ABDUCT,
  BODY_JOINT_R_SCAP_ABDUCT,
  BODY_JOINT_L_SCAP_ROTATE,
  BODY_JOINT_R_SCAP_ROTATE,

  BODY_JOINT_L_SHOULDER_FLEXION,
  BODY_JOINT_R_SHOULDER_FLEXION,
  BODY_JOINT_L_SHOULDER_ABDUCT,
  BODY_JOINT_R_SHOULDER_ABDUCT,
  BODY_JOINT_L_SHOULDER_TWISTING,
  BODY_JOINT_R_SHOULDER_TWISTING,
  BODY_JOINT_L_ELBOW_FLEXION,
  BODY_JOINT_R_ELBOW_FLEXION,
  BODY_JOINT_L_ELBOW_TWISTING,
  BODY_JOINT_R_ELBOW_TWISTING,
  BODY_JOINT_L_WRIST_FLEXION,
  BODY_JOINT_R_WRIST_FLEXION,
  BODY_JOINT_L_WRIST_PIVOT,
  BODY_JOINT_R_WRIST_PIVOT,
  BODY_JOINT_L_ELBOW_TWISTING,
  BODY_JOINT_R_ELBOW_TWISTING,

/* Added 31/08/98 :*/
-1, /*  BODY_JOINT_SKULL_ROLL,*/
-1, /*  BODY_JOINT_SKULL_TORSION,*/
-1, /*  BODY_JOINT_SKULL_TILT,*/

  -1, /* c1 roll */
  -1, /* c1 torsion */
  -1, /* c1 tilt */
  
  BODY_JOINT_VC8_ROLL,
  BODY_JOINT_VC8_TORSION,
  BODY_JOINT_VC8_TILT,
  
  -1, /* c3 roll */
  -1, /* c3 torsion */
  -1, /* c3 tilt */
  
  BODY_JOINT_VC7_ROLL,
  BODY_JOINT_VC7_TORSION,
  BODY_JOINT_VC7_TILT,
  
  -1, /* c5 roll */
  -1, /* c5 torsion */
  -1, /* c5 tilt */
  -1, /* c6 roll */
  -1, /* c6 torsion */
  -1, /* c6 tilt */
  -1, /* c7 roll */
  -1, /* c7 torsion */
  -1, /* c7 tilt */
  
  BODY_JOINT_VT6_ROLL,
  BODY_JOINT_VT6_TORSION,
  BODY_JOINT_VT6_TILT,
  
  -1, /* t2 roll */
  -1, /* t2 torsion */
  -1, /* t2 tilt */
  -1, /* t3 roll */
  -1, /* t3 torsion */
  -1, /* t3 tilt */
  -1, /* t4 roll */
  -1, /* t4 torsion */
  -1, /* t4 tilt */
  -1, /* t5 roll */
  -1, /* t5 torsion */
  -1, /* t5 tilt */
  
  BODY_JOINT_VT5_ROLL,
  BODY_JOINT_VT5_TORSION,
  -1, /* t6 tilt */

  -1, /* t7 roll */
  -1, /* t7 torsion */
  -1, /* t7 tilt */
  -1, /* t8 roll */
  -1, /* t8 torsion */
  -1, /* t8 tilt */
  -1, /* t9 roll */
  -1, /* t9 torsion */
  -1, /* t9 tilt */
  
  BODY_JOINT_VT4_ROLL,
  BODY_JOINT_VT4_TORSION,
  BODY_JOINT_VT4_TILT,
  
  -1, /* t11 roll */
  -1, /* t11 torsion */
  -1, /* t11 tilt */
  -1, /* t12 roll */
  -1, /* t12 torsion */
  -1, /* t12 tilt */
  
  BODY_JOINT_VL3_ROLL,
  -1, /* l1 torsion */
  BODY_JOINT_VL3_TILT,
  
  -1, /* l2 roll */
  -1, /* l2 torsion */
  -1, /* l2 tilt */
  
  BODY_JOINT_VL2_ROLL,
  -1, /* l3 torsion */
  BODY_JOINT_VL2_TILT,
  
  -1, /* l4 roll */
  -1, /* l4 torsion */
  -1, /* l4 tilt */
  
  -1, /* l5 roll */
  -1, /* l5 torsion */
  BODY_JOINT_VL1_TILT,

/* Added 31/08/98 :*/
-1, /*  BODY_JOINT_L_PINKY_FL0 + 100,*/
-1, /*  BODY_JOINT_R_PINKY_FL0 + 200,*/

  /* Hands */
  BODY_JOINT_L_PINKY_FL1 + 100,
  BODY_JOINT_R_PINKY_FL1 + 200,
  BODY_JOINT_L_PINKY_PIVOT + 100,
  BODY_JOINT_R_PINKY_PIVOT + 200,
  BODY_JOINT_L_PINKY_TWIST + 100,
  BODY_JOINT_R_PINKY_TWIST + 200,
  BODY_JOINT_L_PINKY_FL2 + 100,
  BODY_JOINT_R_PINKY_FL2 + 200,
  BODY_JOINT_L_PINKY_FL3 + 100,
  BODY_JOINT_R_PINKY_FL3 + 200,

/* Added 31/08/98 :*/
-1, /*  BODY_JOINT_L_RING_FL0 + 100,*/
-1, /*  BODY_JOINT_R_RING_FL0 + 200,*/

  BODY_JOINT_L_RING_FL1 + 100,
  BODY_JOINT_R_RING_FL1 + 200,
  BODY_JOINT_L_RING_PIVOT + 100,
  BODY_JOINT_R_RING_PIVOT + 200,
  BODY_JOINT_L_RING_TWIST + 100,
  BODY_JOINT_R_RING_TWIST + 200,
  BODY_JOINT_L_RING_FL2 + 100,
  BODY_JOINT_R_RING_FL2 + 200,
  BODY_JOINT_L_RING_FL3 + 100,
  BODY_JOINT_R_RING_FL3 + 200,
  
/* Added 31/08/98 :*/
-1, /*  BODY_JOINT_L_MIDDLE_FL0 + 100,*/
-1, /*  BODY_JOINT_R_MIDDLE_FL0 + 200,*/

  BODY_JOINT_L_MIDDLE_FL1 + 100,
  BODY_JOINT_R_MIDDLE_FL1 + 200,
  BODY_JOINT_L_MIDDLE_PIVOT + 100,
  BODY_JOINT_R_MIDDLE_PIVOT + 200,
  BODY_JOINT_L_MIDDLE_TWIST + 100,
  BODY_JOINT_R_MIDDLE_TWIST + 200,
  BODY_JOINT_L_MIDDLE_FL2 + 100,
  BODY_JOINT_R_MIDDLE_FL2 + 200,
  BODY_JOINT_L_MIDDLE_FL3 + 100,
  BODY_JOINT_R_MIDDLE_FL3 + 200,

/* Added 31/08/98 :*/
-1, /*  BODY_JOINT_L_INDEX_FL0 + 100,*/
-1, /*  BODY_JOINT_R_INDEX_FL0 + 200,*/

  BODY_JOINT_L_INDEX_FL1 + 100,
  BODY_JOINT_R_INDEX_FL1 + 200,
  BODY_JOINT_L_INDEX_PIVOT + 100,
  BODY_JOINT_R_INDEX_PIVOT + 200,
  BODY_JOINT_L_INDEX_TWIST + 100,
  BODY_JOINT_R_INDEX_TWIST + 200,
  BODY_JOINT_L_INDEX_FL2 + 100,
  BODY_JOINT_R_INDEX_FL2 + 200,
  BODY_JOINT_L_INDEX_FL3 + 100,
  BODY_JOINT_R_INDEX_FL3 + 200,
  
  BODY_JOINT_L_THUMB_FL1 + 100,
  BODY_JOINT_R_THUMB_FL1 + 200,
  BODY_JOINT_L_THUMB_ABDUCT + 100,
  BODY_JOINT_R_THUMB_ABDUCT + 200,
  BODY_JOINT_L_THUMB_TW + 100,
  BODY_JOINT_R_THUMB_TW + 200,
  BODY_JOINT_L_THUMB_FL2 + 100,
  BODY_JOINT_R_THUMB_FL2 + 200,
  BODY_JOINT_L_THUMB_FL3 + 100,
  BODY_JOINT_R_THUMB_FL3 + 200,

  /* GLOBAL POSITION */ 
  BODY_JOINT_TR_VERTICAL,
  BODY_JOINT_TR_LATERAL,
  BODY_JOINT_TR_FRONTAL,
  BODY_JOINT_BODY_TURN,
  BODY_JOINT_BODY_ROLL,
  BODY_JOINT_BODY_TILT

};




void
ImplementRules(  maskrest, bytecounter, ParVal, numPar, num1Bits, Frame, body  )
  unsigned char maskrest[15];
  int bytecounter;
  float ParVal[250];
  int numPar, num1Bits, Frame ;
  BODY *body;
{
  unsigned char currentByte = 0;
  int locallen = 0;
  int i=0, bit=0, bitpos=0, pInd = 0, d, j;
  VT3D vt;
  VT3D eulang;



#ifdef VRML2_OUTPUT
  {
    int i;
    if (Frame>0) {
      for (i=0; i<VRML2_NUM_BAPS; i++) {
	vrml2_bapvalues[Frame][i] = vrml2_bapvalues[Frame-1][i];
      }
    }
    else {
      for (i=0; i<VRML2_NUM_BAPS; i++) {
	vrml2_bapvalues[Frame][i] = 0.0;
      }
    }
  }
		     

#endif
  /* To compute size:*/
  for (j=0; j < NUM_BAPS ; j++ ) {
    if ( (bitpos++ % 8) == 0 /*&& bitpos !=0*/ )
       currentByte = maskrest[i++];
    bit = currentByte & 0x80; currentByte <<= 1;	
    if ( bit ) {	     
      if (UpdtJointList[j] == -1)
	 pInd++;
      else {
	if (UpdtJointList[j] >= 100 && UpdtJointList[j] <= 200)
	   ChangeLHandJointVal(body, UpdtJointList[j]-100, ParVal[pInd++]);
	else if (UpdtJointList[j] >= 200)
	   ChangeRHandJointVal(body, UpdtJointList[j]-200, ParVal[pInd++]);
	else
	   ChangeJointVal( body, UpdtJointList[j]
			   , ParVal[pInd++] );
      }

#ifdef VRML2_OUTPUT
      vrml2_bapvalues[Frame][j] = ParVal[pInd-1];
#endif		
    }

  }

}

