
/*------------------------------------------------------------------------------*/
/* 										*/
/*										*/
/* This software module was originally developed by Ronan Boulic and Tom Molet 	*/
/*  (EPFL-LIG) and edited by Joaquim Esmerado (EPFL-LIG) 			*/
/*  in the course of development of the MPEG-4 SNHC 				*/
/*  VM. This software module is an implementation of a part of one or more 	*/
/*  MPEG-4 SNHC VM tools as specified by the MPEG-4 SNHC VM. ISO/IEC gives 	*/
/*  users of the MPEG-4 SNHC VM free license to this software module or 	*/
/*  modifications thereof for use in hardware or software products 		*/
/*  claiming conformance to the MPEG-4 SNHC VM. Those intending to use 		*/
/*  this software module in hardware or software products are advised that 	*/
/*  its use may infringe existing patents. The original developer of this 	*/
/*  software module and his/her company, the subsequent editors and their 	*/
/*  companies, and ISO/IEC have no liability for use of this software 		*/
/*  module or modifications thereof. Copyright is not released for non 		*/
/*  MPEG-4 SNHC VM conforming products. EPFL-LIG 				*/
/*  retains full right to use the code for its own purpose, assign or 		*/
/*  donate the code to a third party and to inhibit third parties from 		*/
/*  using the code for non MPEG-4 SNHC VM conforming products. This 		*/
/*  copyright notice must be included in all copies or derivative 		*/
/*  works. Copyright (1997).  							*/




/* -----------------------------------------------------------------------

Main file:

----------------------------------------------------------------------- */

#include "scn1.h"
#include "etall.h"
#include "private.h"

#include "mais.h"
#include "Node.h"

/*#include <unistd.h>*/

#ifndef EPSILON
#define EPSILON 1e-4
#endif

#ifdef VRML2_OUTPUT
void hanim_generate_animation();
void set_matrices( BODY *bodyview, int Frame, MatriX bodyMatrices[]);
#endif

extern BODY * bodyview, * bodtview;
extern PSCENE scene;

/******************************/

HierarchieInfo Hinfo[VRML_NB_JOINTS+2*VRML_NB_HAND_JOINTS] = {
  /* --------- Lower Part Top Hierarchy */

  /*  pelvis = sacroiliac */
  {"sacroiliac","pelvis",
   BODY_XYZ,BODY_N3D_PELVIC_TILT,BODY_N3D_PELVIC_TORSION,BODY_N3D_PELVIC_ROLL,
   1,"_hip",
   2,{VRML_L_HIP,VRML_R_HIP,-1,-1,-1}},
 
  {"l_hip","l_thigh",
   BODY_XYZ,BODY_N3D_L_HIP_FLEXION,BODY_N3D_L_HIP_ABDUCT,BODY_N3D_L_HIP_TWISTING,
   1,"_left_thigh",
   1,{VRML_L_KNEE,-1,-1,-1,-1}},
  {"l_knee","l_calf",
   BODY_XY,BODY_N3D_L_KNEE_FLEXION,BODY_N3D_L_KNEE_TWISTING,0,
   1,"_left_leg",
   1,{VRML_L_ANKLE,-1,-1,-1,-1}},
  {"l_ankle","l_hindfoot",
   BODY_X,BODY_N3D_L_ANKLE_FLEXION,0,0,
   1,"_left_foot",
   1,{VRML_L_SUBTALAR,-1,-1,-1,-1}},
  {"l_subtalar","l_midproximal",
   BODY_UNDEF,BODY_N3D_L_ANKLE_SUBTALAR,0,0,
   0,"",
   1,{VRML_L_MIDFOOT,-1,-1,-1,-1}},
  /*  midfoot = midtarsal  */
  {"l_midtarsal","l_middistral",
   BODY_Z,BODY_N3D_L_MID_FOOT,0,0,
   0,"",
   1,{VRML_L_TOE,-1,-1,-1,-1}},
  /*  toe = metatarsal  */
  {"l_metatarsal","l_forefoot",
   BODY_Z,BODY_N3D_L_TOE_FLEXION,0,0,
   0,"",
   0,{-1,-1,-1,-1,-1}},

  {"r_hip","r_thigh",
   BODY_XYZ,BODY_N3D_R_HIP_FLEXION,BODY_N3D_R_HIP_ABDUCT,BODY_N3D_R_HIP_TWISTING,
   1,"_right_thigh",
   1,{VRML_R_KNEE,-1,-1,-1,-1}},
  {"r_knee","r_calf",
   BODY_XY,BODY_N3D_R_KNEE_FLEXION,BODY_N3D_R_KNEE_TWISTING,0,
   1,"_right_leg",
   1,{VRML_R_ANKLE,-1,-1,-1,-1}},
  {"r_ankle","r_hindfoot",
   BODY_X,BODY_N3D_R_ANKLE_FLEXION,0,0,
   1,"_right_foot",
   1,{VRML_R_SUBTALAR,-1,-1,-1,-1}},
  {"r_subtalar","r_midproximal",
   BODY_UNDEF,BODY_N3D_R_ANKLE_SUBTALAR,0,0,
   0,"",
   1,{VRML_R_MIDFOOT,-1,-1,-1,-1}},
  /*  midfoot = midtarsal  */
  {"r_midtarsal","r_middistral",
   BODY_Z,BODY_N3D_R_MID_FOOT,0,0,
   0,"",
   1,{VRML_R_TOE,-1,-1,-1,-1}},
  /*  toe = metatarsal  */
  {"r_metatarsal","r_forefoot",
   BODY_Z,BODY_N3D_R_TOE_FLEXION,0,0,
   0,"",
   0,{-1,-1,-1,-1,-1}},
  
  {"vl5","l5",
   BODY_X,BODY_N3D_VL1_TILT,0,0,
   0,"",
   1,{VRML_VL4,-1,-1,-1,-1}},
  {"vl4","l4",
   BODY_UNDEF,BODY_N3D_VL1_TILT,0,0, /*  BODY_N3D_VL1_TILT because it is a Jack N3D  */
   0,"",
   1,{VRML_VL3,-1,-1,-1,-1}},
  {"vl3","l3",
   BODY_XY,BODY_N3D_VL2_ROLL,BODY_N3D_VL2_TILT,0,
   1,"_lower_torso",
   1,{VRML_VL2,-1,-1,-1,-1}},
  {"vl2","l2",
   BODY_UNDEF,BODY_N3D_VL2_ROLL,0,0, /*  BODY_N3D_VL2_ROLL because it is a Jack N3D  */
   0,"",
   1,{VRML_VL1,-1,-1,-1,-1}},
  {"vl1","l1",
   BODY_XY,BODY_N3D_VL3_ROLL,BODY_N3D_VL3_TILT,0,
   0,"",
   1,{VRML_VT10,-1,-1,-1,-1}},
  {"vt10","t10",
   BODY_XYZ,BODY_N3D_VT4_ROLL,BODY_N3D_VT4_TORSION,BODY_N3D_VT4_TILT,
   1,"_upper_torso",
   1,{VRML_VT9,-1,-1,-1,-1}},
  {"vt9","t9",
   BODY_UNDEF,BODY_N3D_VT4_ROLL,0,0, /*  BODY_N3D_VT4_ROLL because it is a Jack N3D  */
   0,"",
   1,{VRML_VT8,-1,-1,-1,-1}},
  {"vt8","t8",
   BODY_UNDEF,BODY_N3D_VT4_ROLL,0,0, /*  BODY_N3D_VT4_ROLL because it is a Jack N3D  */
   0,"",
   1,{VRML_VT7,-1,-1,-1,-1}},
  {"vt7","t7",
   BODY_UNDEF,BODY_N3D_VT4_ROLL,0,0, /*  BODY_N3D_VT4_ROLL because it is a Jack N3D  */
   0,"",
   1,{VRML_VT6,-1,-1,-1,-1}},
  {"vt6","t6",
   BODY_YZ,BODY_N3D_VT5_ROLL,BODY_N3D_VT5_TORSION,0,
   0,"",
   1,{VRML_VT5,-1,-1,-1,-1}},
  {"vt5","t5",
   BODY_UNDEF,BODY_N3D_VT5_ROLL,0,0, /* BODY_N3D_VT5_ROLL because it is a Jack N3D  */
   0,"",
   1,{VRML_VT4,-1,-1,-1,-1}},
  {"vt4","t4",
   BODY_UNDEF,BODY_N3D_VT5_ROLL,0,0, /*  BODY_N3D_VT5_ROLL because it is a Jack N3D  */
   0,"",
   1,{VRML_VT3,-1,-1,-1,-1}},
  {"vt3","t3",
   BODY_UNDEF,BODY_N3D_VT5_ROLL,0,0, /*  BODY_N3D_VT5_ROLL because it is a Jack N3D  */
   0,"",
   1,{VRML_VT2,-1,-1,-1,-1}},
  {"vt2","t2",
   BODY_UNDEF,BODY_N3D_VT5_ROLL,0,0, /*  BODY_N3D_VT5_ROLL because it is a Jack N3D  */
   0,"",
   1,{VRML_VT1,-1,-1,-1,-1}},
  {"vt1","t1",
   BODY_XYZ,BODY_N3D_VT6_ROLL,BODY_N3D_VT6_TORSION,BODY_N3D_VT6_TILT,
   0,"",
   3,{VRML_VC7,VRML_L_CLAVICULE,VRML_R_CLAVICULE,-1,-1}},
  {"vc7","c7",
   BODY_UNDEF,BODY_N3D_VT6_ROLL,0,0, /*  BODY_N3D_VT6_ROLL because it is a Jack N3D  */
   0,"",
   1,{VRML_VC6,-1,-1,-1,-1}},
  {"vc6","c6",
   BODY_UNDEF,BODY_N3D_VT6_ROLL,0,0, /*  BODY_N3D_VT6_ROLL because it is a Jack N3D  */
   0,"",
   1,{VRML_VC5,-1,-1,-1,-1}},
  {"vc5","c5",
   BODY_UNDEF,BODY_N3D_VT6_ROLL,0,0, /*  BODY_N3D_VT6_ROLL because it is a Jack N3D  */
   0,"",
   1,{VRML_VC4,-1,-1,-1,-1}},
  {"vc4","c4",
   BODY_XYZ,BODY_N3D_VC7_ROLL,BODY_N3D_VC7_TORSION,BODY_N3D_VC7_TILT,
   1,"_neck",
   1,{VRML_VC3,-1,-1,-1,-1}},
  {"vc3","c3",
   BODY_UNDEF,BODY_N3D_VC7_ROLL,0,0, /*  BODY_N3D_VC7_ROLL because it is a Jack N3D  */
   0,"",
   1,{VRML_VC2,-1,-1,-1,-1}},
  {"vc2","c2",
   BODY_XYZ,BODY_N3D_VC8_ROLL,BODY_N3D_VC8_TORSION,BODY_N3D_VC8_TILT,
   1,"_head",
   1,{VRML_VC1,-1,-1,-1,-1}},
  {"vc1","c1",
   BODY_UNDEF,BODY_N3D_VC8_ROLL,0,0, /*  BODY_N3D_VC8_ROLL because it is a Jack N3D  */
   0,"",
   1,{VRML_SKULLBASE,-1,-1,-1,-1}},
  {"skullbase","skull",
   BODY_UNDEF,0,0,0,
   0,"",
   0,{-1,-1,-1,-1,-1}},

  /*  calvicule = sternoclavicular  */
  {"l_sternoclavicular","l_clavicule",
   BODY_XY,BODY_N3D_L_CLAV_ABDUCT,BODY_N3D_L_CLAV_ROTATE,0,
   0,"",
   1,{VRML_L_SCAPULA,-1,-1,-1,-1}},
  /*  scapula = acromioclavicular  */
  {"l_acromioclavicular","l_scapula",
   BODY_XY,BODY_N3D_L_SCAP_ABDUCT,BODY_N3D_L_SCAP_ROTATE,0,
   0,"",
   1,{VRML_L_SHOULDER,-1,-1,-1,-1}},
  {"l_shoulder","l_upperarm",
   BODY_XYZ,BODY_N3D_L_SHOULDER_FLEXION,BODY_N3D_L_SHOULDER_ABDUCT,BODY_N3D_L_SHOULDER_TWISTING,
   1,"_left_upper_arm",
   1,{VRML_L_ELBOW,-1,-1,-1,-1}},
  {"l_elbow","l_forearm",
   BODY_XY,BODY_N3D_L_ELBOW_FLEXION,BODY_N3D_L_ELBOW_TWISTING,0,
   1,"_left_lower_arm",
   1,{VRML_L_WRIST,-1,-1,-1,-1}},
  {"l_wrist","l_hand",
   BODY_XY,BODY_N3D_L_WRIST_FLEXION,BODY_N3D_L_WRIST_PIVOT,0,
   1,"_l_palm",
   5,{VRML_L_THUMB1,VRML_L_INDEX1,VRML_L_MIDDLE1,VRML_L_RING1,VRML_L_PINKY1}},

  /*  calvicule = sternoclavicular  */
  {"r_sternoclavicular","r_clavicule",
   BODY_XY,BODY_N3D_R_CLAV_ABDUCT,BODY_N3D_R_CLAV_ROTATE,0,
   0,"",
   1,{VRML_R_SCAPULA,-1,-1,-1,-1}},
  /*  scapula = acromioclavicular  */
  {"r_acromioclavicular","r_scapula",
   BODY_XY,BODY_N3D_R_SCAP_ABDUCT,BODY_N3D_R_SCAP_ROTATE,0,
   0,"",
   1,{VRML_R_SHOULDER,-1,-1,-1,-1}},
  {"r_shoulder","r_upperarm",
   BODY_XYZ,BODY_N3D_R_SHOULDER_FLEXION,BODY_N3D_R_SHOULDER_ABDUCT,BODY_N3D_R_SHOULDER_TWISTING,
   1,"_right_upper_arm",
   1,{VRML_R_ELBOW,-1,-1,-1,-1}},
  {"r_elbow","r_forearm",
   BODY_XY,BODY_N3D_R_ELBOW_FLEXION,BODY_N3D_R_ELBOW_TWISTING,0,
   1,"_right_lower_arm",
   1,{VRML_R_WRIST,-1,-1,-1,-1}},
  {"r_wrist","r_hand",
   BODY_XY,BODY_N3D_R_WRIST_FLEXION,BODY_N3D_R_WRIST_PIVOT,0,
   1,"_r_palm",
   5,{VRML_R_THUMB1,VRML_R_INDEX1,VRML_R_MIDDLE1,VRML_R_RING1,VRML_R_PINKY1}},


/*  Additional HumanoidRoot (missing) node  */
  {"HumanoidRoot","VRML_Root",
   BODY_TR,BODY_N3D_TR_VERTICAL,BODY_N3D_TR_LATERAL,BODY_N3D_TR_FRONTAL/* BODY_N3D_PELVIC_ROLL */,
   1,"",
   1,{VRML_ROOT,-1,-1,-1,-1}},
  {"HumanoidRoot","VRML_Root",
   BODY_ZYX,BODY_N3D_BODY_TURN,BODY_N3D_BODY_ROLL,BODY_N3D_BODY_TILT,
   1,"",
   5,{VRML_ROOT,-1,-1,-1,-1}},





  /*  Left Fingers  */
  {"l_thumb1","l_thumb_metacarpal",
   BODY_XY,BODY_N3D_L_THUMB_FL1,BODY_N3D_L_THUMB_ABDUCT,BODY_N3D_L_THUMB_TW,
   1,"_l_thumb_base",
   1,{VRML_L_THUMB2,-1,-1,-1,-1}},
  {"l_thumb2","l_thumb_proximal",
   BODY_XY,BODY_N3D_L_THUMB_FL2,0,0,
   1,"_l_thumb_mid",
   1,{VRML_L_THUMB3,-1,-1,-1,-1}},
  {"l_thumb3","l_thumb_distal",
   BODY_XY,BODY_N3D_L_THUMB_FL3,0,0,
   1,"_l_thumb_ext",
   0,{-1,-1,-1,-1,-1}},
  {"l_index1","l_index_proximal",
   BODY_XY,BODY_N3D_L_INDEX_FL1,BODY_N3D_L_INDEX_PIVOT,BODY_N3D_L_INDEX_TWIST,
   1,"_l_index_base",
   1,{VRML_L_INDEX2,-1,-1,-1,-1}},
  {"l_index2","l_index_middle",
   BODY_XY,BODY_N3D_L_INDEX_FL2,0,0,
   1,"_l_index_mid",
   1,{VRML_L_INDEX3,-1,-1,-1,-1}},
  {"l_index3","l_index_distal",
   BODY_XY,BODY_N3D_L_INDEX_FL3,0,0,
   1,"_l_index_ext",
   0,{-1,-1,-1,-1,-1}},
  {"l_middle1","l_middle_proximal",
   BODY_XY,BODY_N3D_L_MIDDLE_FL1,BODY_N3D_L_MIDDLE_PIVOT,BODY_N3D_L_MIDDLE_TWIST,
   1,"_l_middle_base",
   1,{VRML_L_MIDDLE2,-1,-1,-1,-1}},
  {"l_middle2","l_middle_middle",
   BODY_XY,BODY_N3D_L_MIDDLE_FL2,0,0,
   1,"_l_middle_mid",
   1,{VRML_L_MIDDLE3,-1,-1,-1,-1}},
  {"l_middle3","l_middle_distal",
   BODY_XY,BODY_N3D_L_MIDDLE_FL3,0,0,
   1,"_l_middle_ext",
   0,{-1,-1,-1,-1,-1}},
  {"l_ring1","l_ring_proximal",
   BODY_XY,BODY_N3D_L_RING_FL1,BODY_N3D_L_RING_PIVOT,BODY_N3D_L_RING_TWIST,
   1,"_l_ring_base",
   1,{VRML_L_RING2,-1,-1,-1,-1}},
  {"l_ring2","l_ring_middle",
   BODY_XY,BODY_N3D_L_RING_FL2,0,0,
   1,"_l_ring_mid",
   1,{VRML_L_RING3,-1,-1,-1,-1}},
  {"l_ring3","l_ring_distal",
   BODY_XY,BODY_N3D_L_RING_FL3,0,0,
   1,"_l_ring_ext",
   0,{-1,-1,-1,-1,-1}},
  {"l_pinky1","l_pinky_proximal",
   BODY_XY,BODY_N3D_L_PINKY_FL1,BODY_N3D_L_PINKY_PIVOT,BODY_N3D_L_PINKY_TWIST,
   1,"_l_pinky_base",
   1,{VRML_L_PINKY2,-1,-1,-1,-1}},
  {"l_pinky2","l_pinky_middle",
   BODY_XY,BODY_N3D_L_PINKY_FL2,0,0,
   1,"_l_pinky_mid",
   1,{VRML_L_PINKY3,-1,-1,-1,-1}},
  {"l_pinky3","l_pinky_distal",
   BODY_XY,BODY_N3D_L_PINKY_FL3,0,0,
   1,"_l_pinky_ext",
   0,{-1,-1,-1,-1,-1}},

  /*  Right Fingers  */
  {"r_thumb1","r_thumb_metacarpal",
   BODY_XY,BODY_N3D_R_THUMB_FL1,BODY_N3D_R_THUMB_ABDUCT,BODY_N3D_R_THUMB_TW,
   1,"_r_thumb_base",
   1,{VRML_R_THUMB2,-1,-1,-1,-1}},
  {"r_thumb2","r_thumb_proximal",
   BODY_XY,BODY_N3D_R_THUMB_FL2,0,0,
   1,"_r_thumb_mid",
   1,{VRML_R_THUMB3,-1,-1,-1,-1}},
  {"r_thumb3","r_thumb_distal",
   BODY_XY,BODY_N3D_R_THUMB_FL3,0,0,
   1,"_r_thumb_ext",
   0,{-1,-1,-1,-1,-1}},
  {"r_index1","r_index_proximal",
   BODY_XY,BODY_N3D_R_INDEX_FL1,BODY_N3D_R_INDEX_PIVOT,BODY_N3D_R_INDEX_TWIST,
   1,"_r_index_base",
   1,{VRML_R_INDEX2,-1,-1,-1,-1}},
  {"r_index2","r_index_middle",
   BODY_XY,BODY_N3D_R_INDEX_FL2,0,0,
   1,"_r_index_mid",
   1,{VRML_R_INDEX3,-1,-1,-1,-1}},
  {"r_index3","r_index_distal",
   BODY_XY,BODY_N3D_R_INDEX_FL3,0,0,
   1,"_r_index_ext",
   0,{-1,-1,-1,-1,-1}},
  {"r_middle1","r_middle_proximal",
   BODY_XY,BODY_N3D_R_MIDDLE_FL1,BODY_N3D_R_MIDDLE_PIVOT,BODY_N3D_R_MIDDLE_TWIST,
   1,"_r_middle_base",
   1,{VRML_R_MIDDLE2,-1,-1,-1,-1}},
  {"r_middle2","r_middle_middle",
   BODY_XY,BODY_N3D_R_MIDDLE_FL2,0,0,
   1,"_r_middle_mid",
   1,{VRML_R_MIDDLE3,-1,-1,-1,-1}},
  {"r_middle3","r_middle_distal",
   BODY_XY,BODY_N3D_R_MIDDLE_FL3,0,0,
   1,"_r_middle_ext",
   0,{-1,-1,-1,-1,-1}},
  {"r_ring1","r_ring_proximal",
   BODY_XY,BODY_N3D_R_RING_FL1,BODY_N3D_R_RING_PIVOT,BODY_N3D_R_RING_TWIST,
   1,"_r_ring_base",
   1,{VRML_R_RING2,-1,-1,-1,-1}},
  {"r_ring2","r_ring_middle",
   BODY_XY,BODY_N3D_R_RING_FL2,0,0,
   1,"_r_ring_mid",
   1,{VRML_R_RING3,-1,-1,-1,-1}},
  {"r_ring3","r_ring_distal",
   BODY_XY,BODY_N3D_R_RING_FL3,0,0,
   1,"_r_ring_ext",
   0,{-1,-1,-1,-1,-1}},
  {"r_pinky1","r_pinky_proximal",
   BODY_XY,BODY_N3D_R_PINKY_FL1,BODY_N3D_R_PINKY_PIVOT,BODY_N3D_R_PINKY_TWIST,
   1,"_r_pinky_base",
   1,{VRML_R_PINKY2,-1,-1,-1,-1}},
  {"r_pinky2","r_pinky_middle",
   BODY_XY,BODY_N3D_R_PINKY_FL2,0,0,
   1,"_r_pinky_mid",
   1,{VRML_R_PINKY3,-1,-1,-1,-1}},
  {"r_pinky3","r_pinky_distal",
   BODY_XY,BODY_N3D_R_PINKY_FL3,0,0,
   1,"_r_pinky_ext",
   0,{-1,-1,-1,-1,-1}}
};

/******************************/


#define BYTELEN         8

FILE *Bap_file;

FILE *DebugFile;

N3D * n3d_current;
char * BuffPath;
char * BdyDataPath;
char bodyfile[80];
char BdyDataName[60];
char HndDataName[60];

MatriX bodyMatrices[2000][VRML_NB_JOINTS+2*VRML_NB_HAND_JOINTS+2];

#define	NUM_FLOAT	250

#ifdef VRML2_OUTPUT
#define VRML_NB_EXTRA_FRAMES 40
FILE *animation;
void mr3dI_to_axis_angle(VT3D axis, MR3D mr, float *theta);
int vrml2_noframes;
float vrml2_bapvalues[VRML2_MAX_NUMFRAMES][VRML2_NUM_BAPS];
#endif
char *AnimFile = NULL;


void main(argc, argv)
  int argc;
  char **argv;
{
  int i, j, VisualEnhance;
int kl=0;
  int Frame, numPar=0 ;
  float flo[NUM_FLOAT];


  unsigned char bytemask = 0;
  unsigned char maskrest[30];
  int res = 0, num1Bits;
  int bytecounter = 0, maskrestlen = 0;
  int masktestval = 0;
  char line[MAX_LINE_SIZE];

  char file_base[40];
  char file_ext[20];
  char next_out_file[40];
  int k, l;
  int f_counter=0;

  FILE *OutFil;

  UNIT *unit1;
  char n3d_name1[60];

  float Height, racio, sp_orig;

  int intFile=0;
  int c;
  extern char *optarg;
  extern int optind;
  int errflg = 0;
  

  if (argc < 3) { 
    fprintf( stderr,"Missing input, output file names\n\n" );
    fprintf( stderr,
	     "Usage: %s inputfile outputreffile [-v] [-a animVRML2.0file] \n",
	     argv[0] );
    exit(0); 
  }
  
  if (!(Bap_file = fopen(argv[1], "rb"))) {
    fprintf( stderr, "Error: could not open file %s\n", argv[1] );
    exit( 1 );
  }

  VisualEnhance = 0;

  if (argc >= 4 && argv[3][0] == '-' && argv[3][1] == 'v' ) {
     kl += 1;
     VisualEnhance = 1;
  }

  if (argc >= 3+kl+2 && argv[3+kl][0] == '-' && argv[3+kl][1] == 'a' ) {
     kl +=1;
     AnimFile = argv[3+kl];
  }
  
  for ( k=0; argv[2][k] != '\0' && argv[2][k] != '.'; k++ )
     file_base[k] = argv[2][k];
  file_base[k] = '\0';
  for ( k=k+1, l=0; argv[2][k] != '\0'; k++, l++ )
     file_ext[l] = argv[2][k];
  file_ext[l] = '\0';

  
  BdyDataPath = getenv( "BDYDATAPATH" );
   
  sprintf( BdyDataName, "%s%s%s", 
	   (BdyDataPath)?BdyDataPath:"",
	   (BdyDataPath)?"/":"",
	   "body.dat");
    
  sprintf( HndDataName, "%s%s%s", 
	   (BdyDataPath)?BdyDataPath:"",
	   (BdyDataPath)?"/":"",
	   "hand.dat");
    

/*
  if (!(SkelDataFile = fopen ( BdyDataName, "rb" ) )) {
    fprintf( stderr, "Error: could not open file %s\n", BdyDataName );
    fprintf( stderr, "Check BDYDATAPATH env variable also\n" );
    exit( 1 );
  }

  if (!(HandDataFile = fopen ( HndDataName, "rb" ) )) {
    fprintf( stderr, "Error: could not open file %s\n", HndDataName );
    fprintf( stderr, "Check BDYDATAPATH env variable also\n" );
    exit( 1 );
  }
*/

  sceneBodyIni();
  
  /* Entry to account for bap file header line */
  GetHeaderInfo( Bap_file );

  masktestval = GetBapMask( Bap_file , line, &intFile );

  if ( masktestval ) {
    if ( masktestval == EOF ) {
      fprintf( stderr, "%s has no mask. Quitting\n", line );
      exit( 1 ); }

    res = ConvertToMaskFormat( line, &bytemask, maskrest,
			       &bytecounter, &maskrestlen, &num1Bits );

    if ( !res )
       fprintf( stderr, " MASK INCOMPLETE\n" );

  }
  else {
    fprintf( stderr, "%s is not a valid mask. Quitting\n", line );
    exit( 1 );
  }



  BuffPath = getenv( "BUFFPATH" );


  /*----------------------------------------------------
    PARAMETERS READING & PROCESSING 
    -----------------------------------------------------*/

#ifdef VRML2_OUTPUT
  set_matrices( bodyview, -1, bodyMatrices[0]);
#endif

  while ( (ReadParameters( Bap_file, &Frame, flo, &numPar, intFile ) ) 
	  != EOF )
  {
    sprintf( next_out_file, "%s_%04d.%s", 
	     file_base, f_counter++, file_ext);

    /* verify if frames are in order ...? */
    if (!(OutFil = fopen ( next_out_file, "wb" ) )) {
      fprintf( stderr, "Error: could not open file %s\n", next_out_file );
      exit( 1 );
    }

    ImplementRules(  maskrest, bytecounter, flo, numPar, 
		     num1Bits, Frame, bodyview  );

#ifdef VRML2_OUTPUT
    set_matrices( bodyview, Frame, bodyMatrices[Frame]);
#endif

    if (! (unit1 = scene_get_top( scene )) )
       fprintf( stderr, "ERROR in scene_get_top\n" );
    n3d_current = unit_get_scene_root( unit1 );
    n3d_get_type_name( n3d_current, n3d_name1 );

    unit1 = unit_get_child( unit1, 0 );
    n3d_current = unit_get_root( unit1 );
    n3d_get_type_name( n3d_current, n3d_name1 );

    Height = bodyview->total_height;

    racio = bodyview->spine_origin_ratio;


    sp_orig = 1000.0;

    VrmlFileHead( OutFil );
    if (VisualEnhance) 
       Vrml_initial_position( OutFil );

    RunBodyThrough( OutFil, n3d_current, 0, sp_orig ,Frame);


    VrmlFileTail( OutFil );

    fclose( OutFil );


    h3d_actual_ref(unit_get_root(bodyview->golem),DIRECT_PROPAGATION,FALSE);

    /* ---------------------------------------------------- */
    /*	Replicate Mask acquisition code here:		*/
    /* ---------------------------------------------------- */

    masktestval = GetBapMask( Bap_file , line, &intFile );

    if ( masktestval ) {
      if ( masktestval == EOF ) break;

      res = ConvertToMaskFormat( line, &bytemask, maskrest,
				 &bytecounter, &maskrestlen, &num1Bits );

      if ( !res )
	 fprintf( stderr, " MASK INCOMPLETE\n" );

    }
    else {
      fprintf( stderr, "%s is not a valid mask. Quitting\n", line );
      exit( 1 );
    }

  }


#ifdef VRML2_OUTPUT
  hanim_generate_animation();
#endif

}


#ifdef VRML2_OUTPUT

#define HANIM_NUM_NODES 78

/*
sacroiliac : pelvis
    l_hip : l_thigh
      l_knee : l_calf
        l_ankle : l_hindfoot
          l_subtalar : l_midproximal
            l_midtarsal : l_middistal
              l_metatarsal : l_forefoot
    r_hip : r_thigh
      r_knee : r_calf
        r_ankle : r_hindfoot
          r_subtalar :  r_midproximal
            r_midtarsal : r_middistal
              r_metatarsal : r_forefoot
  vl5 * : l5
    vl4 : l4
      vl3 * : l3
        vl2 : l2
          vl1 * : l1
            vt12 : t12
              vt11 : t11
                vt10 * : t10
                  vt9 : t9
                    vt8 : t8
                      vt7 : t7
                        vt6 * : t6
                          vt5 : t5
                            vt4 : t4
                              vt3 : t3
                                vt2 : t2
                                  vt1 : t1
                                    vc7 * : c7
                                    | vc6 : c6
                                    |   vc5 : c5
                                    |     vc4 * : c4
                                    |       vc3 : c3
                                    |         vc2 * : c2
                                    |           vc1 : c1
                                    |             skullbase : skull
                                    l_sternoclavicular : l_clavicle
                                    | l_acromioclavicular : l_scapula
                                    |   l_shoulder : l_upperarm
                                    |     l_elbow : l_forearm
                                    |       l_wrist : l_hand
                                    |         l_thumb1 : l_thumb_metacarpal
                                    |           l_thumb2 : l_thumb_proximal
                                    |             l_thumb3 : l_thumb_distal
                                    |         l_index1 : l_index_proximal
                                    |           l_index2 : l_index_middle
                                    |             l_index3 : l_index_distal
                                    |         l_middle1 : l_middle_proximal
                                    |           l_middle2 : l_middle_middle
                                    |             l_middle3 : l_middle_distal
                                    |         l_ring1 : l_ring_proximal
                                    |           l_ring2 l_ring_middle
                                    |             l_ring3 : l_ring_distal
                                    |         l_pinky1 : l_pinky_proximal
                                    |           l_pinky2 : l_pinky_middle
                                    |             l_pinky3 : l_pinky_distal
                                    r_sternoclavicular : r_clavicle
                                      r_acromioclavicular : r_scapula
                                        r_shoulder : r_upperarm
                                          r_elbow : r_forearm
                                            r_wrist : r_hand
                                              r_thumb1 : r_thumb_metacarpal
                                                r_thumb2 : r_thumb_proximal
                                                  r_thumb3 : r_thumb_distal
                                              r_index1 : r_index_proximal
                                                r_index2 : r_index_middle
                                                  r_index3 : r_index_distal
                                              r_middle1 : r_middle_proximal
                                                r_middle2 : r_middle_middle
                                                  r_middle3 : r_middle_distal
                                              r_ring1 : r_ring_proximal
                                                r_ring2 : r_ring_middle
                                                  r_ring3 : r_ring_distal
                                              r_pinky1 : r_pinky_proximal
                                                r_pinky2 : r_pinky_middle
                                                  r_pinky3 : r_pinky_distal
						  */

/*-------------------------------------------------------------------*/
void mr3dI_to_axis_angle(VT3D axis, MR3D mr, float *theta)
{
  float angle, norm, ww, xx, yy, four_w, two_x;
  double scalar ;
  VHOM  quat ;
  
  ww = 0.25 * ( 1.0 + mr[0][0] + mr[1][1] + mr[2][2]);
  
  if (ww > EPSILON)
  {
    quat[3] = fsqrt(ww);
    four_w  = 4.0 * quat[3];
    quat[0] = (mr[1][2] - mr[2][1]) / four_w;
    quat[1] = (mr[2][0] - mr[0][2]) / four_w;
    quat[2] = (mr[0][1] - mr[1][0]) / four_w;
          
  }
  else
  {
    quat[3] = 0.0;
    xx = -0.5 * (mr[1][1] + mr[2][2]);

    if (xx > EPSILON)
    {
      quat[0] = fsqrt(xx);
      two_x = quat[0] * 2.0;
      quat[1] = mr[0][1] / two_x;
      quat[2] = mr[0][2] / two_x;
    }
    else
    {
      quat[0] = 0.0;
      yy = 0.5 * (1.0 - mr[2][2]);

      if (yy > EPSILON)
      {
	quat[1] = fsqrt(yy);
	quat[2] = mr[1][2] / (2.0 * quat[1]);
      }
      else
      {
	quat[1] = 0.0;
	quat[2] = 1.0;
      }
    }
  }


  /* normalize the quaternion */
  norm = fsqrt(
    quat[0] * quat[0] +
    quat[1] * quat[1] +
    quat[2] * quat[2] +
    quat[3] * quat[3]);

  if (norm > EPSILON)
  {
    quat[0] /= norm;
    quat[1] /= norm;
    quat[2] /= norm;
    quat[3] /= norm;
  }
  else
  {
    quat[0] = 0.0;
    quat[1] = 0.0;
    quat[2] = 0.0;
    quat[3] = 1.0;
  }

  /* calculate angle */
  angle = 2.0 * facos(quat[3]);

  axis[0] = quat[0];
  axis[1] = quat[1];
  axis[2] = quat[2];

  norm = sqrt((double)(axis[0]*axis[0] + axis[1]*axis[1] + axis[2]*axis[2])) ;
  if (norm > EPSILON)
  {
    *theta = (float) angle;
    axis[0] /= norm;
    axis[1] /= norm;
    axis[2] /= norm;
  }
  else
  {
    *theta = angle;
    axis[0] = 1 ;
    axis[1] = 0. ;
    axis[2] = 0. ;
     
  }
}

/*--------------------------------------------------------*/

static MatriX WorldRefMat = {
  { -1, 0, 0, 0},
  {  0, 1, 0, 0},
  {  0, 0,-1, 0},
  {  0, 0, 0, 1}
};



void
GetWorldNodeMat( N3D_t *self1 , N3D_t *self2, MatriX TT1, MatriX TT2 )
{
  MatriX P1_ref_inv;
  MatriX WrldInv;

  /* Get 1st Parent's ref_inv */
  n3d_get_ref_inv( n3d_get_parent( self1 ), P1_ref_inv );

  matrix_copy( P1_ref_inv, TT1);
  n3d_get_ref_dir( self2, TT2 );

}

/*-------------------------------------------------------------------*/
void    matrix_transpose(MatriX a,
                         MatriX at)
{
  MatriX  m;
  int     i,j;

  for(i=0 ; i<4 ; i++)
     for(j=0 ; j<4 ; j++)
	m[i][j] = a[j][i];

  for(i=0 ; i<4 ; i++)
     for(j=0 ; j<4 ; j++)
	at[i][j] = m[i][j];
}


void
ComputeJointWorldTransf( N3D_t *self1, N3D_t *self2, MatriX TT1, MatriX TT2, MatriX TT3 )
{
  MatriX P1_ref_dir, ref_inv2, X1, X2;

  /* Get 1st DOF Parent ref_dir */
  n3d_get_ref_dir( n3d_get_parent( self1 ), P1_ref_dir );

  /* Get last DOF's ref_inv */
  n3d_get_ref_inv( self2, ref_inv2 );

  matrix_mult( WorldRefMat, TT1, X1 );
  matrix_mult( X1, P1_ref_dir, X2 );
  matrix_mult( X2, ref_inv2, X1 );
  matrix_mult( X1, TT2, X2 );
  matrix_mult( X2, WorldRefMat, X1);
  matrix_transpose( X1, TT3 );
}

void
set_matrices( BODY *bodyview, int Frame, MatriX bodyMatrices[])
{
  int i;
  MatriX mat1, mat2, res, temp;
  N3D *node1, *node2;
  static MatriX initial_matrices1[VRML_NB_JOINTS+2*VRML_NB_HAND_JOINTS];
  static MatriX initial_matrices2[VRML_NB_JOINTS+2*VRML_NB_HAND_JOINTS];
  int node2_id;

  for (i=0; i<VRML_NB_JOINTS+2*VRML_NB_HAND_JOINTS; i++) {

    if (Hinfo[i].order == BODY_UNDEF) {
      matrix_idt_( bodyMatrices[i]);
      continue;
    }

    if (i<VRML_NB_JOINTS)
      node1 = bodyview->tn3d[Hinfo[i].JointIndex1];
    else if (i<VRML_NB_JOINTS+VRML_NB_HAND_JOINTS)
      node1 = bodyview->tlhand[Hinfo[i].JointIndex1];
    else
      node1 = bodyview->trhand[Hinfo[i].JointIndex1];

    if (Hinfo[i].JointIndex3)
      node2_id = Hinfo[i].JointIndex3;
    else if (Hinfo[i].JointIndex2)
      node2_id = Hinfo[i].JointIndex2;
    else
      node2_id = Hinfo[i].JointIndex1;  

    if (i<VRML_NB_JOINTS)
      node2 = bodyview->tn3d[node2_id];
    else if (i<VRML_NB_JOINTS+VRML_NB_HAND_JOINTS)
      node2 = bodyview->tlhand[node2_id];
    else
      node2 = bodyview->trhand[node2_id];

    if (Frame == -1) {
      GetWorldNodeMat( node1, node2, initial_matrices1[i], initial_matrices2[i] );

    }
    else {
      ComputeJointWorldTransf( 
		 node1, node2, initial_matrices1[i], initial_matrices2[i], res );
      matrix_copy( res, bodyMatrices[i]);
    }

  }

}


static MatriX Compensate = {
  { -1, 0, 0, 0},
  {  0,-1, 0, 0},
  {  0, 0, -1, 0},
  {  0, 0, 0, 1}
};

void
hanim_generate_animation()
{
  FILE *ofp;
  int i,j;
  float angle;
  MR3D mx;
  VT3D axis, pos, v;
  MatriX R,XX;

  if (! AnimFile )
     ofp = fopen("animation.wrl", "w");
  else
     ofp = fopen(AnimFile, "w");

  fprintf( ofp, "DEF TS TimeSensor {\n");
  fprintf( ofp, "cycleInterval %f\n", vrml2_noframes/25.0);
  fprintf( ofp, "loop TRUE }\n");

  for (i=0; i<VRML_NB_JOINTS+2*VRML_NB_HAND_JOINTS; i++) {

    if (Hinfo[i].order != BODY_TR )
       fprintf( ofp, "DEF INTER%s OrientationInterpolator {\n", Hinfo[i].VRML_name );
    else
       fprintf( ofp, "DEF INTER_TR%s PositionInterpolator {\n", Hinfo[i].VRML_name );

    fprintf( ofp, "key [ ");
    for (j=0; j<vrml2_noframes+VRML_NB_EXTRA_FRAMES; j++) {
      fprintf( ofp, "%f ", (float)j/(float)(vrml2_noframes+VRML_NB_EXTRA_FRAMES));
    }
    fprintf( ofp, "] \n");

    fprintf( ofp, "keyValue [ ");
    for (j=0; j<vrml2_noframes; j++) {
      matrix_copy( bodyMatrices[j][i], R);

      mx[0][0] = (float) R[0][0];  mx[0][1] = (float) R[0][1];  mx[0][2] = (float) R[0][2];
      mx[1][0] = (float) R[1][0];  mx[1][1] = (float) R[1][1];  mx[1][2] = (float) R[1][2];
      mx[2][0] = (float) R[2][0];  mx[2][1] = (float) R[2][1];  mx[2][2] = (float) R[2][2];

      v[0] = R[0][3]; 
      v[1] = R[1][3];
      v[2] = R[2][3];

      matrix_mult_vt3d( Compensate, v , pos );

      pos[0] /= 1000.; 
      pos[1] /= 1000.;
      pos[2] /= 1000.;

      mr3dI_to_axis_angle( axis, mx, &angle);

      if (Hinfo[i].order != BODY_TR )
	 fprintf( ofp, " %f %f %f %f ", axis[0], axis[1], axis[2], angle);
      else 
	 fprintf( ofp, " %f %f %f\n ", pos[0], pos[1], pos[2] );

      /*   if (i!=vrml2_noframes-1) */
      fprintf( ofp, ",");
    }
    for (j=vrml2_noframes; j<vrml2_noframes+VRML_NB_EXTRA_FRAMES; j++)
       if (Hinfo[i].order != BODY_TR )
	  fprintf( ofp, " %f %f %f %f , ", axis[0], axis[1], axis[2], angle);
       else 
	  fprintf( ofp, " %f %f %f \n", pos[0], pos[1], pos[2] );
    fprintf( ofp, "] \n");
    fprintf( ofp, "} \n");
	
    if (Hinfo[i].order != BODY_TR ) {
      fprintf( ofp, "ROUTE TS.fraction_changed TO INTER%s.set_fraction\n", Hinfo[i].VRML_name);
      fprintf( ofp, "ROUTE INTER%s.value_changed TO hanim_%s.set_rotation\n", Hinfo[i].VRML_name, Hinfo[i].VRML_name);
    }
    else {
      fprintf( ofp, "ROUTE TS.fraction_changed TO INTER_TR%s.set_fraction\n", Hinfo[i].VRML_name);
      fprintf( ofp, "ROUTE INTER_TR%s.value_changed TO hanim_%s.set_translation\n", Hinfo[i].VRML_name, Hinfo[i].VRML_name);
    }

  }

}

#endif
