
/*------------------------------------------------------------------------------*/
/* 										*/
/* 										*/
/* This software module was originally developed by Joaquim Esmerado (EPFL-LIG)	*/
/*  in the course of development of the MPEG-4 SNHC 				*/
/*  VM. This software module is an implementation of a part of one or more 	*/
/*  MPEG-4 SNHC VM tools as specified by the MPEG-4 SNHC VM. ISO/IEC gives 	*/
/*  users of the MPEG-4 SNHC VM free license to this software module or 	*/
/*  modifications thereof for use in hardware or software products 		*/
/*  claiming conformance to the MPEG-4 SNHC VM. Those intending to use 		*/
/*  this software module in hardware or software products are advised that 	*/
/*  its use may infringe existing patents. The original developer of this 	*/
/*  software module and his/her company, the subsequent editors and their 	*/
/*  companies, and ISO/IEC have no liability for use of this software 		*/
/*  module or modifications thereof. Copyright is not released for non 		*/
/*  MPEG-4 SNHC VM conforming products. EPFL-LIG 				*/
/*  retains full right to use the code for its own purpose, assign or 		*/
/*  donate the code to a third party and to inhibit third parties from 		*/
/*  using the code for non MPEG-4 SNHC VM conforming products. This 		*/
/*  copyright notice must be included in all copies or derivative 		*/
/*  works. Copyright (1997).  							*/





/* -------------------------------------------------------------------------

RunBodyThrough - dedicated file.

------------------------------------------------------------------------- */
#include "scn1.h"
#include "etall.h"
#include "mais.h"




extern MR3D mr3d_z_on_x;

STRING my_body_parts[NB_MY_BODY_PARTS] =
{
  "Buffman_upper_torso.wrl",
  "Buffman_lower_torso.wrl",
  "Buffman_hip.wrl",
  "Buffman_right_hand.wrl",
  "Buffman_left_thigh.wrl",
  "Buffman_left_lower_arm.wrl",
  "Buffman_right_thigh.wrl",
  "Buffman_left_upper_arm.wrl",
  "Buffman_right_lower_arm.wrl",
  "Buffman_left_leg.wrl",
  "Buffman_right_upper_arm.wrl",
  "Buffman_right_leg.wrl",
  "Buffman_left_hand.wrl",
  "Buffman_left_foot.wrl",
  "Buffman_right_foot.wrl",
  "Buffman_head.wrl"

  /* Added the following 11/02/98 */
  ,
  "l_palm.wrl",
  "l_pinky_base.wrl",
  "l_pinky_mid.wrl",
  "l_pinky_ext.wrl",
  "l_ring_base.wrl",
  "l_ring_mid.wrl",
  "l_ring_ext.wrl",
  "l_middle_base.wrl",
  "l_middle_mid.wrl",
  "l_middle_ext.wrl",
  "l_index_base.wrl",
  "l_index_mid.wrl",
  "l_index_ext.wrl",
  "l_thumb_base.wrl",
  "l_thumb_mid.wrl",
  "l_thumb_ext.wrl",

  "r_palm.wrl",
  "r_pinky_base.wrl",
  "r_pinky_mid.wrl",
  "r_pinky_ext.wrl",
  "r_ring_base.wrl",
  "r_ring_mid.wrl",
  "r_ring_ext.wrl",
  "r_middle_base.wrl",
  "r_middle_mid.wrl",
  "r_middle_ext.wrl",
  "r_index_base.wrl",
  "r_index_mid.wrl",
  "r_index_ext.wrl",
  "r_thumb_base.wrl",
  "r_thumb_mid.wrl",
  "r_thumb_ext.wrl",

};

STRING corresp_body_parts[NB_MY_BODY_PARTS] =
{
  "vt4_tilt",
  "vl1_tilt",
  "pelvic_roll",
  "R_WRIST_PIVOT", 
  "l_hip_twisting",
  "l_elbow_twisting",
  "r_hip_twisting",
  "l_shoulder_twisting",
  "r_elbow_twisting",
  "l_knee_twisting",
  "r_shoulder_twisting",
  "r_knee_twisting",
  "L_WRIST_PIVOT", 
  "l_ankle_subtalar",
  "r_ankle_subtalar",
  "vc8_torsion"

  ,
  "l_hand_root",
  "l_pinky_tw",
  "l_pinky_fl2",
  "l_pinky_fl3",
  "l_ring_tw",
  "l_ring_fl2",
  "l_ring_fl3",
  "l_middle_tw",
  "l_middle_fl2",
  "l_middle_fl3",
  "l_index_tw",
  "l_index_fl2",
  "l_index_fl3",
  "l_thumb_tw",
  "l_thumb_fl2",
  "l_thumb_fl3",

  "r_hand_root",
  "r_pinky_tw",
  "r_pinky_fl2",
  "r_pinky_fl3",
  "r_ring_tw",
  "r_ring_fl2",
  "r_ring_fl3",
  "r_middle_tw",
  "r_middle_fl2",
  "r_middle_fl3",
  "r_index_tw",
  "r_index_fl2",
  "r_index_fl3",
  "r_thumb_tw",
  "r_thumb_fl2",
  "r_thumb_fl3",

};




/* ---------------------------------------------------------------------*/
/* Special case: the suppression of rt_body_tilt & some others forced
one such solution: Get the parent nodes Ref transf & the node Ini dir:

Jref_dir = Jini_dir x Pref_dir

The only issue is that we want to alter Jini_dir -> Z_ON_X. Take the 
translation from the inspirational node and change the rotation. Then
generate back the Jref_dir, to output later.
*/

void
SpecialCaseMatrix1( n3d_current, refdir )
  N3D * n3d_current;
  MatriX refdir;
{
  N3D * n3d_Pcurrent;
  MR3D mr_in;
  VT3D vt_in;
  MatriX inidir, Prefdir;

  n3d_Pcurrent = n3d_get_parent( n3d_current );

  n3d_get_ini_dir( n3d_current, inidir );
  n3d_get_ref_dir( n3d_Pcurrent, Prefdir );
  matrix_decompose( mr_in, vt_in, inidir );

  matrix_compose( mr3d_z_on_x, vt_in, inidir );
  matrix_mult( inidir, Prefdir, refdir );
}


/* ---------------------------------------------------------------------*/
void
SpecialCaseMatrix2( n3d_current, refdir )
  N3D * n3d_current;
  MatriX refdir;
{
  N3D * n3d_Pcurrent, * n3d_chl;
  MR3D mr_in, mr_in_c, mr_in_r;
  /* VT3D vt_in, vt_in_c, vt_in_r; */
  MatriX inidir,inidir_c, inidir_r,  Prefdir;
  MatriX refdir1,refdir_c, refdir_r;

  MatriX m45x;


  n3d_Pcurrent = n3d_get_parent( n3d_current );

  /* Correcting matrix 35o + arbitrary 20 on Y -10 on Z transl. 
     see frame for subtalar changed to 45 deg. the changes are according
     to this frame */
  m45x[0][0] = 1.;
  m45x[0][1] = 0.;
  m45x[0][2] = 0.;
  m45x[0][3] = 0.;
  m45x[1][0] = 0.;
  m45x[1][1] = 0.81915204;
  m45x[1][2] = -0.57357644;
  m45x[1][3] = 0.;
  m45x[2][0] = 0.;
  m45x[2][1] = 0.57357644;
  m45x[2][2] = 0.81915204;
  m45x[2][3] = 0.;
  m45x[3][0] = 0.;
  m45x[3][1] = 20.;
  m45x[3][2] = -10.;
  m45x[3][3] = 1.;


  /*     vt_in_r[0] = (vt_in[0]); */
  /*     vt_in_r[1] = (vt_in[1]); */
  /*     vt_in_r[2] = (vt_in[2]); */

  n3d_get_ini_dir( n3d_current, inidir );
  matrix_mult( m45x, inidir, inidir_r);

  n3d_get_ref_dir( n3d_Pcurrent, Prefdir );

  matrix_mult( inidir_r, Prefdir, refdir );
}

void
VrmlFileTail( file )
  FILE * file;
{
  fprintf( file, "}" );
}

void
VrmlFileHead( file )
  FILE * file;
{
  fprintf( file, "#VRML V1.0 ascii\n\nSeparator {\n" );
  fprintf( file, "           ShapeHints {\n" );
  fprintf( file, "                       vertexOrdering  COUNTERCLOCKWISE\n" );
  fprintf( file, "                       shapeType UNKNOWN_SHAPE_TYPE\n" );
  fprintf( file, "                       faceType UNKNOWN_FACE_TYPE\n" );
  fprintf( file,"                      }\n      DEF InfoSeparator Separator {\n" );
  fprintf( file,"               DEF Viewer Info {\n" );
  fprintf( file,"                    string \"walk\"\n                }\n" );
  fprintf( file,"               }\n" );

}

void
Vrml_matrix_print( file, n3d_current, height )
  FILE * file;
  N3D * n3d_current;
  float height;
{
  MatriX dir, specialRef;
  char Name[30];


  if ( n3d_ref_is_todate( n3d_current ) ) {

    n3d_get_ref_dir( n3d_current, dir );

    strcpy(Name, n3d_get_local_name( n3d_current ) );

    if ( !(strcmp( Name, "skeleton_root" ) ) ) {
      SpecialCaseMatrix1( n3d_current, dir );
    }

    if ( (!(strcmp( Name, "r_ankle_subtalar" ) )) 
	 || (!(strcmp( Name, "l_ankle_subtalar" ) )) ) {
      /*	    SpecialCaseMatrix2( n3d_current, dir );*/
    }

    fprintf( file, "%.6f %.6f %.6f %.6f\n",
	     dir[0][0], dir[0][1],dir[0][2], dir[0][3]);
    fprintf( file, "%.6f %.6f %.6f %.6f\n",
	     dir[1][0], dir[1][1], dir[1][2], dir[1][3]);
    fprintf( file, "%.6f %.6f %.6f %.6f\n",
	     dir[2][0], dir[2][1], dir[2][2], dir[2][3]);
    fprintf( file, "%.6f %.6f %.6f %.6f\n",
	     dir[3][0]/height, dir[3][1]/height, dir[3][2]/height, dir[3][3]);
  }
  else printf( "ERROR: Node has no global\n" );
}


#define MaxCharPerLine  200

void nextValidLine(ifp) 
  FILE *ifp;
{
  int firstChar;
  char buf[MaxCharPerLine];

  while(feof(ifp) == 0) {
    firstChar = fgetc(ifp);
    if (firstChar == '#') { /* skip comments */
      fgets(buf, MaxCharPerLine, ifp);
    } else if(firstChar == '\n') { /* skip empty line */
      ;
    } else {
      ungetc (firstChar, ifp);
      return;
    }
  }
}

#define 	MAXC		200


Vrml_initial_position( file )
  FILE * file;
{
  /*        fprintf( file, "\tDEF Cameras Switch {\n" );*/
  /*        fprintf( file, "\t  DEF ForVideo  PerspectiveCamera {\n" );*/
  /*        fprintf( file, "\t      position    -1.3 1.5 -2.00\n" );*/
  /*        fprintf( file, "\t      orientation .1 1 0.1 3.80\n" );*/
  /*        fprintf( file, "\t  }\n" );*/
  /*        fprintf( file, "\t}\n" );*/

  FILE * in_file;
  char line[MAXC];
  int ichar;

  if (!(in_file = fopen( "ViewPoint.dat", "r" ) ) ) {
    fprintf( stderr, " Error: Could not open file ViewPoint.dat\n" );
    return -1;
  }


  while ( (ichar = fgetc(in_file)) != EOF ) {
    ungetc(ichar, in_file);
    nextValidLine( in_file );
    /*	if (feof(in_file) == 0) {*/
    if ( (ichar = fgetc(in_file)) != EOF ) {
      ungetc(ichar, in_file);
      fgets(line, MAXC, in_file);
      fprintf( file, "%s", line );
    }
  }
  fclose(in_file);
  return 0;
}




void
RunBodyThrough( file, n3d_current, ind, height, Frame )
  FILE * file;
  N3D * n3d_current;
  int ind, Frame;
  float height;
{
  int i, nb, j ;
  N3D * n3d_chl;
  char nd_name1[60];
  char my_nd_name[60];
  short n3dtype;


  nb = n3d_get_nb_children( n3d_current );

  n3d_chl = n3d_get_down( n3d_current );

  for (i=1; i<=nb && ( n3d_chl = n3d_get_left( n3d_chl )) != FALSE ; i++) {
    strcpy( nd_name1, n3d_get_local_name( n3d_chl ) );

    n3dtype = n3d_get_type( n3d_chl );

    if ( n3dtype==N3D_JOINT  || n3dtype==N3D_NEUTRAL) {
      for ( j=0; j<NB_MY_BODY_PARTS; j++ ) {
        if ( !(strcmp(corresp_body_parts[j], nd_name1)) ) {
	  sprintf( my_nd_name, "%s%s%s", 
		   (BuffPath)?BuffPath:"",
		   (BuffPath)?"/":"",
		   my_body_parts[j]);
	  /*                strcpy( my_nd_name, my_body_parts[j] );*/
          fprintf( file,
		   "Separator {\n MatrixTransform\n    {\n matrix\n" );
          Vrml_matrix_print( file, n3d_chl, height );
          fprintf( file,
		   "    }\n   WWWInline {\n              name \"%s\"\n",
		   my_nd_name );
          fprintf( file, "             }\n          }\n" );
          break;
        }
      }
    }

    RunBodyThrough( file, n3d_chl, ind*10+i, height, Frame  );
  }
}





