/********************************************************************/

/* This software module was updated for BAP coding by Marius Preda   */
/* and Titus  Zaharia (Institut National des Telecommunications -    */
/* ARTEMIS Project Unit), and  is based                              */
/* on  Gabriel Antunes Abrantes (Instituto Superior Tcnico)         */
/* original soft  (implemented for FAP only)	                     */	
/* in the course of development of the MPEG-4 standard.	             */
/* This software module is an implementation of a part of one or     */
/* more MPEG-4 tools as specified by the	                     */
/* MPEG-4 standard. ISO/IEC gives users of the MPEG-4 standard       */
/* free license to this software module or modifications thereof     */
/* for use in hardware or software products claiming conformance     */
/* to the MPEG-4 standard. Those intending to use this software      */
/* module in hardware or software products are advised that its      */
/* use may infringe existing patents. The original developer of      */
/* this software module and his company, the subsequent              */
/* editors and their companies, and ISO/IEC have no liability        */
/* for use of this software module or modifications thereof in       */
/* an implementation. Copyright is not released for non              */
/* MPEG-4 standard conforming products. ACTS-MoMuSys partners        */
/* retain full right to use the code for their own purposes,         */
/* assign or donate the code to a third party and to inhibit         */
/* third parties from using the code for non MPEG-4 standard         */
/* conforming products. This copyright notice must be included       */
/* in all copies or derivative works.    		             */
/* Copyright (c)1999				                     */
/*   						                     */

/********************************************************************/

#include <stdio.h>
#include <math.h>
#include <stdlib.h>
#include <ctype.h>
#include <string.h>

/* DCT coding */

#define SEGMENT_LENGTH   16
#define PI               3.1415926
#define DC_MAX	 32767	
#define DC_MIN	-32767

#define RANGE_MAX 3720

#define RUN 0
#define DC  1
#define AC  2

/* arithmetic coding */
#define NO_BAPS       296

#define   IrangeMin		  -1860
#define   IrangeMax		   1860
#define   rangeMin		  -1860
#define   rangeMax		   1860


static int IrangeMin1[NO_BAPS] =
   {  
      -960,  -960,  -960, -1260, -1260,  -960,  -960,  -960,  -960, -1500,
     -1500,  -960,  -960,  -780,  -780,  -960,  -960,  -780,  -780,  -180,
      -180,  -780,  -780,   -60,  -240,  -120,  -120,   -60,  -360,  -360,
      -360, -1080, -1080,  -240, -1860,  -960,  -960,   -60,   -60,  -960,
      -960,  -960,  -960,  -660,  -660,  -660,  -660, -1860, -1860, -1860,
      -240,  -240,  -240,  -660,  -660,  -660,  -240,  -240,  -240,  -660,
      -660,  -660,  -240,  -240,  -240,  -240,  -240,  -240,  -240,  -240, 
      -240,  -660,  -660,  -660,  -240,  -240,  -240,  -240,  -240,  -240,
      -240,  -240,  -240,  -240,  -240,  -240,  -660,  -660,  -660,  -240,
      -240,  -240,  -240,  -240,  -240,  -240,  -240,  -240,  -660,  -660,
      -660,  -240,  -240,  -240,  -240,  -240,  -240,  -660,  -660,  -660,
      -240,  -240,  -240,  -660,  -660,  -660,  -240,  -240,  -240,  -660,
      -660,  -660,  -240,  -240, -1140, -1140,  -420,  -120,  -180,  -360,
     -1380, -1380,  -960,  -960,  -180,  -180, -1140, -1140,  -240,  -120,
      -240,  -240, -1380, -1380,  -960,  -960,  -120,  -120, -1140, -1140,
      -180,  -180,  -180,  -180, -1380, -1380,  -960,  -960,   -60, -  60,
     -1140, -1140,  -120,  -240,  -240,  -180, -1380, -1380,  -960,  -960,
      -480,  -960,  -120, -1080,  -720,  -720,  -780,  -780,  -960,  -960,
     -1860, -1860, -1860, -1860, -1860, -1860,
     -1860, -1860, -1860, -1860, -1860, -1860, -1860, -1860, -1860, -1860,
     -1860, -1860, -1860, -1860, -1860, -1860, -1860, -1860, -1860, -1860, 
     -1860, -1860, -1860, -1860, -1860, -1860, -1860, -1860, -1860, -1860,
     -1860, -1860, -1860, -1860, -1860, -1860, -1860, -1860, -1860, -1860,
     -1860, -1860, -1860, -1860, -1860, -1860, -1860, -1860, -1860, -1860,
     -1860, -1860, -1860, -1860, -1860, -1860, -1860, -1860, -1860, -1860,
     -1860, -1860, -1860, -1860, -1860, -1860, -1860, -1860, -1860, -1860,
     -1860, -1860, -1860, -1860, -1860, -1860, -1860, -1860, -1860, -1860,
     -1860, -1860, -1860, -1860, -1860, -1860, -1860, -1860, -1860, -1860,
     -1860, -1860, -1860, -1860, -1860, -1860, -1860, -1860, -1860, -1860,
     -1860, -1860, -1860, -1860, -1860, -1860, -1860, -1860, -1860, -1860
   };

static int IrangeMax1[NO_BAPS] =
   {  
       960,   960,   960,  1260,  1260,   480,   480,   960,   960,   180,
       180,   960,   960,   780,   780,   960,   960,   780,   780,   180,
       180,   780,   780,   240,    60,   120,   120,   360,    60,   360,
       360,  1860,  1860,  1860,   240,   960,   960,  1560,  1560,   960,
       960,   960,   960,   660,   660,   660,   660,  1860,  1860,  1860,
       240,   240,   240,   660,   660,   660,   240,   240,   240,   660,
       660,   660,   240,   240,   240,   240,   240,   240,   240,   240, 
       240,   660,   660,   660,   240,   240,   240,   240,   240,   240,
       240,   240,   240,   240,   240,   240,   660,   660,   660,   240,
       240,   240,   240,   240,   240,   240,   240,   240,   660,   660,
       660,   240,   240,   240,   240,   240,   240,   660,   660,   660,
       240,   240,   240,   660,   660,   660,   240,   240,   240,   660,
       660,   660,   240,   240,   240,   240,   120,   420,   360,   180,
        60,    60,    60,    60,   180,   180,   360,   360,   120,   240,
       240,   240,    60,    60,    60,    60,   120,   120,   360,   360,
       180,   180,   180,   180,    60,    60,    60,    60,    60,    60,
       360,   360,   240,   120,   180,   240,    60,    60,    60,    60,
       960,   480,  1080,   120,   240,   240,    60,    60,    60,    60,
      1860,  1860,  1860,  1860,  1860,  1860,
     1860, 1860, 1860, 1860, 1860, 1860, 1860, 1860, 1860, 1860,
     1860, 1860, 1860, 1860, 1860, 1860, 1860, 1860, 1860, 1860, 
     1860, 1860, 1860, 1860, 1860, 1860, 1860, 1860, 1860, 1860,
     1860, 1860, 1860, 1860, 1860, 1860, 1860, 1860, 1860, 1860,
     1860, 1860, 1860, 1860, 1860, 1860, 1860, 1860, 1860, 1860,
     1860, 1860, 1860, 1860, 1860, 1860, 1860, 1860, 1860, 1860,
     1860, 1860, 1860, 1860, 1860, 1860, 1860, 1860, 1860, 1860,
     1860, 1860, 1860, 1860, 1860, 1860, 1860, 1860, 1860, 1860,
     1860, 1860, 1860, 1860, 1860, 1860, 1860, 1860, 1860, 1860,
     1860, 1860, 1860, 1860, 1860, 1860, 1860, 1860, 1860, 1860,
     1860, 1860, 1860, 1860, 1860, 1860, 1860, 1860, 1860, 1860
   };

static int rangeMin1[NO_BAPS] =
   {  
      -600,  -600,  -600,  -600,  -600,  -600,  -600,  -360,  -360,  -600,
      -600,  -360,  -360,  -600,  -600,  -360,  -360,  -600,  -600,  -120,
      -120,  -600,  -600,  -120,  -120,   -60,   -60,  -120,  -120,  -120,
      -120,  -600,  -600,  -600,  -600,  -360,  -360,  -600,  -600,  -360,
      -360,  -600,  -600,  -360,  -360,  -360,  -360,  -600,  -600,  -600,
      -120,  -120,  -120,  -360,  -360,  -360,  -120,  -120,  -120,  -360,
      -360,  -360,  -120,  -120,  -120,  -120,  -120,  -120,  -120,  -120,
      -120,  -360,  -360,  -360,  -120,  -120,  -120,  -120,  -120,  -120,
      -120,  -120,  -120,  -120,  -120,  -120,  -360,  -360,  -360,  -120,
      -120,  -120,  -120,  -120,  -120,  -120,  -120,  -120,  -360,  -360,
      -360,  -120,  -120,  -120,  -120,  -120,  -120,  -360,  -360,  -360,
      -120,  -120,  -120,  -360,  -360,  -360,  -120,  -120,  -120,  -360,
      -360,  -360,  -120,  -120,  -240,  -240,  -120,  -120,  -120,  -120,
      -240,  -240,  -240,  -240,  -120,  -120,  -240,  -240,  -120,  -120,
      -120,  -120,  -240,  -240,  -240,  -240,  -120,  -120,  -240,  -240,
      -120,  -120,  -120,  -120,  -240,  -240,  -240,  -240,   -60,   -60,
      -240,  -240,  -60,   -60,   -120,  -120,  -240,  -240,  -240,  -240,
      -240,  -240,  -240,  -240,  -240,  -240,  -240,  -240,  -240,  -240,
      -600,  -600,  -600,  -600,  -600,  -600,
     -600, -600, -600, -600, -600, -600, -600, -600, -600, -600,
     -600, -600, -600, -600, -600, -600, -600, -600, -600, -600, 
     -600, -600, -600, -600, -600, -600, -600, -600, -600, -600,
     -600, -600, -600, -600, -600, -600, -600, -600, -600, -600,
     -600, -600, -600, -600, -600, -600, -600, -600, -600, -600,
     -600, -600, -600, -600, -600, -600, -600, -600, -600, -600,
     -600, -600, -600, -600, -600, -600, -600, -600, -600, -600,
     -600, -600, -600, -600, -600, -600, -600, -600, -600, -600,
     -600, -600, -600, -600, -600, -600, -600, -600, -600, -600,
     -600, -600, -600, -600, -600, -600, -600, -600, -600, -600,
     -600, -600, -600, -600, -600, -600, -600, -600, -600, -600

   };

static int rangeMax1[NO_BAPS] =
   {  
       600,   600,   600,   600,   600,   600,   600,   360,   360,   600,
       600,   360,   360,   600,   600,   360,   360,   600,   600,   120,
       120,   600,   600,   120,   120,    60,    60,   120,   120,   120,
       120,   600,   600,   600,   600,   360,   360,   600,   600,   360,
       360,   600,   600,   360,   360,   360,   360,   600,   600,   600,
       120,   120,   120,   360,   360,   360,   120,   120,   120,   360,
       360,   360,   120,   120,   120,   120,   120,   120,   120,   120,
       120,   360,   360,   360,   120,   120,   120,   120,   120,   120,
       120,   120,   120,   120,   120,   120,   360,   360,   360,   120,
       120,   120,   120,   120,   120,   120,   120,   120,   360,   360,
       360,   120,   120,   120,   120,   120,   120,   360,   360,   360,
       120,   120,   120,   360,   360,   360,   120,   120,   120,   360,
       360,   360,   120,   120,   240,   240,   120,   120,   120,   120,
       240,   240,   240,   240,   120,   120,   240,   240,   120,   120,
       120,   120,   240,   240,   240,   240,   120,   120,   240,   240,
       120,   120,   120,   120,   240,   240,   240,   240,    60,    60,
       240,   240,   60,    60,    120,   120,   240,   240,   240,   240,
       240,   240,   240,   240,   240,   240,   240,   240,   240,   240,
       600,   600,   600,   600,   600,   600,
        600, 600, 600, 600, 600, 600, 600, 600, 600, 600,
     600, 600, 600, 600, 600, 600, 600, 600, 600, 600, 
     600, 600, 600, 600, 600, 600, 600, 600, 600, 600,
     600, 600, 600, 600, 600, 600, 600, 600, 600, 600,
     600, 600, 600, 600, 600, 600, 600, 600, 600, 600,
     600, 600, 600, 600, 600, 600, 600, 600, 600, 600,
     600, 600, 600, 600, 600, 600, 600, 600, 600, 600,
     600, 600, 600, 600, 600, 600, 600, 600, 600, 600,
     600, 600, 600, 600, 600, 600, 600, 600, 600, 600,
     600, 600, 600, 600, 600, 600, 600, 600, 600, 600,
     600, 600, 600, 600, 600, 600, 600, 600, 600, 600,
   };

#define   q1             16384
#define   q2             32768
#define   q3             49152
#define   top            65535
#define   Max_frequency  16383
#define   RANGE_DELTA       60

#define   ZEROMAX           22

#define BAP_NUM_GROUPS     24
#define MAX_BAPS           22

int NBAP_GROUP[BAP_NUM_GROUPS]={3, 4, 4, 6, 6, 5, 5, 7, 7, 12, 15, 18, 18, 12, 16, 
				16, 13, 13, 6,22,22,22,22,22};
int BAPS_IN_GROUP[BAP_NUM_GROUPS][MAX_BAPS]=
                   {{ 1, 2, 3, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0 ,0,0,0,0,0},  
		    { 4, 6, 10, 14, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,0,0,0,0},
		    { 5, 7, 11, 15, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,0,0,0,0},
		    {8, 12, 16, 18, 20, 22, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,0,0,0,0},
		    {9, 13, 17, 19, 21, 23, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,0,0,0,0}, 
		    {32, 34, 36, 38, 42, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,0,0,0,0},
		    {33, 35, 37, 39, 43, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,0,0,0,0},
		    {24,26,28,30,40,44,46,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0},
		    {25,27,29,31,41,45,47,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0},
		    {48,49,50,60,61,62,87,88,89,114,115,116,0,0,0,0,0,0,0,0,0,0},
		    {54,55,56,72,73,74,99,100,101,108,109,110,120,121,122,0,0,0,0,0,0,0},
		    {57,58,59,66,67,68,81,82,83,93,94,95,105,106,107,117,118,119,0,0,0,0},
		    {63,64,65,69,70,71,75,76,77,90,91,92,102,103,104,111,112,113,0,0,0,0},
		    {51,52,53,78,79,80,84,85,86,96,97,98,0,0,0,0,0,0},
		    {125,131,133,137,143,145,149,155,157,161,167,169,171,173,177,179,0,0,0,0,0,0},
		    {126,132,134,138,144,146,150,156,158,162,168,170,172,174,178,180,0,0,0,0,0,0},
		    {123,127,129,135,139,141,147,151,153,159,163,165,175,0,0,0,0,0,0,0,0,0},
		    {124,128,130,136,140,142,148,152,154,160,164,166,176,0,0,0,0,0,0,0,0,0},
		    {181,182,183,184,185,186,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0},
		    {187,188,189,190,191,192,193,194,195,196,197,198,199,200,201,202,203,204,205,206,207,208},
		    {209,210,211,212,213,214,215,216,217,218,219,220,221,222,223,224,225,226,227,228,229,230},
		    {231,232,233,234,235,236,237,238,239,240,241,242,243,244,245,246,247,248,249,250,251,252},
		    {253,254,255,256,257,258,259,260,261,262,263,264,265,266,267,268,269,270,271,272,273,274},
		    {275,276,277,278,279,280,281,282,283,284,285,286,287,288,289,290,291,292,293,294,295,296}};
		    


/* up-date doc Soeul*/

int BapQuantSteps[NO_BAPS]={64,   64,  64, 128, 128, 128, 128, 256, 256, 128,
     128, 256, 256,  23,  23, 256, 256, 256, 256, 256,
     256, 256, 256, 128, 128, 128, 128, 128, 128, 128,
     128,  64,  64,  64,  64, 256, 256,  64,  64, 256,
     256, 128, 128, 128, 128, 256, 256, 128, 128, 128,
     256, 256, 256, 128, 128, 128, 256, 256, 256, 128,
     128, 128, 256, 256, 256, 256, 256, 256, 256, 256,
     256, 128, 128, 128, 256, 256, 256, 256, 256, 256,
     256, 256, 256, 256, 256, 256, 128, 128, 128, 256,
     256, 256, 256, 256, 256, 256, 256, 256, 128, 128,
     128, 256, 256, 256, 256, 256, 256, 128, 128, 128,
     256, 256, 256, 128, 128, 128, 256, 256, 256, 128,
     128, 128, 512, 512, 128, 128, 128, 128, 256, 256,
     128, 128, 128, 128, 256, 256, 128, 128, 128, 128,
     256, 256, 128, 128, 128, 128, 256, 256, 128, 128,
     128, 128, 256, 256, 128, 128, 128, 128, 256, 256,
     128, 128, 128, 128, 256, 256, 128, 128, 128, 128,
     128, 128, 128, 128, 256, 256, 128, 128, 128, 128,
     64,   64,  64,  64,  64,  64,   1,  1,  1,  1,
      1,  1,  1,  1,  1,  1,  1,  1,  1,  1,   
      1,  1,  1,  1,  1,  1,  1,  1,  1,  1,   
      1,  1,  1,  1,  1,  1,  1,  1,  1,  1,   
      1,  1,  1,  1,  1,  1,  1,  1,  1,  1,   
      1,  1,  1,  1,  1,  1,  1,  1,  1,  1,   
      1,  1,  1,  1,  1,  1,  1,  1,  1,  1,   
      1,  1,  1,  1,  1,  1,  1,  1,  1,  1,   
      1,  1,  1,  1,  1,  1,  1,  1,  1,  1,   
      1,  1,  1,  1,  1,  1,  1,  1,  1,  1,   
      1,  1,  1,  1,  1,  1,  1,  1,  1,  1,   
      1,  1,  1,  1,  1,  1};


			     
/* DCT coding */

#define SEGMENT_LENGTH   16
#define PI               3.1415926
#define DC_MAX			 32767
#define DC_MIN			-32767

#define RUN 0
#define DC  1
#define AC  2

/* fba object syntax */

/*#define BAP_QUANT            1*/

#define FBA_OBJECT_START_CODE        257
#define FBA_OBJECT_END_CODE          258
#define FBA_OBJECT_PLANE_START_CODE  259

#define FAP                  1
#define BAP                  2
#define FBAP                 3

#define ABORT                2

#define BUFFER_SIZE 256
#define sqr(x) pow(x,2)


#define DEBUG          1
#define MY_DEBUG       0

#define GROUP_MASK_SIZE 10

#define BAP_MASK_TYPE0       0
#define BAP_MASK_TYPE1       1


#define BAP_MASK_TYPE2       2
#define BAP_MASK_TYPE3       3


#define ARITH_TYPE			 0
#define DCT_TYPE			 1

#define DEC_ERROR			-1


#define OFF 0
#define  ON 1

#define sign(x) ((x>=0)?1:-1)


int   baps[NO_BAPS];
int   baps_mask[NO_BAPS];
float fps;



float DCTQuantStepsBAPS[NO_BAPS]={1,1,1,15,15,15,15,30,30,15,15,30,30,
				  23,23,30,30,30,30,30,30,30,30,
				  15,15,15,15,15,15,15,15,10,10,10,10,30,30,10,10,30,30,
				  15,15,15,15,30,30,15,15,15,30,30,30,
				  15,15,15,30,30,30,15,15,15,30,30,30,30,30,30,30,30,30,15,15,15,30,30,30,30,30,30,30,30,30,
				  30,30,30,15,15,15,30,30,30,30,30,30,30,30,30,15,15,15,30,30,30,30,
				  30,30,15,15,15,30,30,30,15,15,15,30,30,30,15,15,15,50,50,15,15,15,
				  15,30,30,15,15,15,15,30,30,15,15,15,15,30,30,15,15,15,
				  15,30,30,15,15,15,15,30,30,15,15,15,15,15,15,15,15,30,30,15,15,15,15,1,1,1,1,1,1};

			    

int bap_scale[32] = { 1, 1, 2, 3, 5, 7, 8, 10, 12, 15, 18, 21, 25, 30, 35, 42, 
                      50, 60, 72, 87, 105, 128, 156, 191, 234, 288, 355, 439, 543, 
					  674, 836, 1039 };

static int bap_pred_scale[32] =
  { 0,  1,   2,  3,   5,   7,   9,  11,  14,  17,  20,  23,  27,  31,  35,  39,
    43, 47, 52, 57,  62,  67,  72,  77,  82,  88,  94, 100, 106, 113, 120, 127};

int end = OFF;

FILE *fp_stream = NULL;
FILE *bap_control = NULL;



/******************************************************************


 - Memory allocation


******************************************************************/

float **FloatMatrixAllocation(int vs,int hs)
{
  float **new_matrix;
  int i;
  
  new_matrix=(float **) calloc(vs,sizeof(float*));
  if ( (new_matrix==NULL) ) {
    printf("Out of Memory \n");
    exit(1);
  }
  
  for(i=0;i<vs;i++) {
    new_matrix[i]=(float *) calloc(hs,sizeof(float));
    if ( (new_matrix[i]==NULL)  ){
      printf("Out of Memory \n");
      exit(1);
    }
  }
  
  return new_matrix;
}



void MatrixDealoc(float **matrix,int vs)
{
  int i;
  
  for(i=0;i<vs;i++) 
    free(matrix[i]);
  
  free(matrix);
}




int **IntMatrixAllocation(int vs,int hs)
{
  int **new_matrix;
  int i;
  
  new_matrix=(int **) calloc(vs,sizeof(int*));
  if ( (new_matrix==NULL) ) {
    printf("Out of Memory \n");
    exit(1);
  }
  
  for(i=0;i<vs;i++) {
    new_matrix[i]=(int *) calloc(hs,sizeof(int));
    if ( (new_matrix[i]==NULL)  ){
      printf("Out of Memory \n");
      exit(1);
    }
  }
  
  return new_matrix;
}




void IntMatrixDealoc(int **matrix,int vs)
{
  int i;
  
  for(i=0;i<vs;i++) 
    free(matrix[i]);
  
  free(matrix);
}


int quantize(int bap, int quantStep) 
{
	
	if(!quantStep)
		return bap;
	else  if( abs(bap)<=quantStep/2 )
		return 0;
	else 
		return (bap + sign(bap)*quantStep/2 ) / quantStep;
	
}


int dequantize(int qbap, int qstep)

{

  if(qstep) {
  	return qbap*qstep;
  } else {
	return qbap;
  }
}


/***********************************************************************

  Get bits of stream -   
	
***********************************************************************/

int bitsin,bitcount=0;


int GetBits(int no_bits,FILE *fp_stream) 
{
	int	counter;
	int	value = 0;
	int	bit;
	static unsigned char byte;
	
	bitcount=bitcount+no_bits;
	for(counter=0; counter<no_bits; counter++) {
        
		if(bitsin==0) {
		  byte = getc(fp_stream);
		  bitsin = 8;
		}

		bitsin -= 1;
		bit = (byte&0200);
		bit >>=7;
		byte <<=1;
		
		value <<= 1;
        value |= bit;
	}
	
	return value;
}



int ShortGetBits(FILE *fp_stream) 
{
	int counter;
	short value = 0;	
	
	bitcount+=16;
	for(counter=0; counter<16; counter++) {
        value <<= 1;
        value += GetBits(1,fp_stream);
	}
	
	return (int)value;
}





/***********************************************************************
									
 Huffman-Tables and decoding
 
***********************************************************************/




void ReadHuffmanTables(int **run_table,int **dc_table,int **ac_table)
{
  FILE *fp_table;
  char word[BUFFER_SIZE];
  int counter,bitcounter;
  
  sprintf(word,"run.table");
  if( (fp_table  = fopen(word,"rb"))==NULL ) {
    printf("\n File - %s - does not exist. \n",word);
    exit(0);
  }
  
	for(counter=0;counter<16;counter++) {
		fscanf(fp_table,"%d",&run_table[counter][0]);
		
		for(bitcounter=1;bitcounter<=run_table[counter][0];bitcounter++) 
			fscanf(fp_table,"%d",&run_table[counter][bitcounter]);
		fgets(word,BUFFER_SIZE,fp_table);
	}

	fclose(fp_table);


	sprintf(word,"dc.table");
	if( (fp_table  = fopen(word,"rb"))==NULL ) {
		printf("\n File - %s - does not exist. \n",word);
		exit(0);
	}

	for(counter=0;counter<512;counter++) {
		fscanf(fp_table,"%d",&dc_table[counter][0]);
		
		for(bitcounter=1;bitcounter<=dc_table[counter][0];bitcounter++) 
			fscanf(fp_table,"%d",&dc_table[counter][bitcounter]);
		fgets(word,BUFFER_SIZE,fp_table);
	}

	fclose(fp_table);


	sprintf(word,"ac.table");
	if( (fp_table  = fopen(word,"rb"))==NULL ) {
		printf("\n File - %s - does not exist. \n",word);
		exit(0);
	}

	for(counter=0;counter<512;counter++) {
		fscanf(fp_table,"%d",&ac_table[counter][0]);
		
		for(bitcounter=1;bitcounter<=ac_table[counter][0];bitcounter++) 
			fscanf(fp_table,"%d",&ac_table[counter][bitcounter]);
		fgets(word,BUFFER_SIZE,fp_table);
	}

	

	fclose(fp_table);
}




int GetHuffmanSymbol(int **table,int type,FILE *bap_stream,FILE *bap_control)
{
	int symbol,bit;
	int size,no_bits,counter;
	int found = OFF;
	int *check;

	if(!type) size = 16;
	else size = 512;

	check = (int *)calloc(size,sizeof(int));
	
	no_bits = 1;

	while(!found) {

	  bit = GetBits(1,bap_stream);
	  
	  for(counter=0;counter<size;counter++) {
	    if(!check[counter]) {
	      
	      if( (table[counter][0]==no_bits) && (table[counter][no_bits]==bit) ) {
		symbol = counter;
		found=ON;
		break;
	      }
	      else if( table[counter][no_bits]!=bit )
		check[counter]=ON;
	    }
	  }
	  no_bits++;
	}
	
	free(check);
	if (MY_DEBUG) fprintf(bap_control,"\n huffman symbol %d",symbol);
	return symbol;
}




/***********************************************************************

 
  Arithmetic Coder - as specified in the MPEG4-Visual FCD
  
   
***********************************************************************/


static long low, high, code_value, Bit, length, sacindex, cum;

int zerocount = 0;


int bit_out_psc_layer(FILE *fp_stream, FILE *bap_control)
{	
  if(zerocount == ZEROMAX) {
    if(!GetBits(1,fp_stream)) {	
      printf("MARKER BIT error in arithmetic decoding\n");
      exit(-1);
      return OFF;
    }
    zerocount = 0;
  }
  
  Bit = GetBits(1,fp_stream);
  if (MY_DEBUG) fprintf(bap_control,"\n bit %ld, bitsin %d",Bit,bitsin);
  
  if(!Bit) zerocount++;
  else zerocount = 0;
  
  return ON;
}



int decoder_reset(FILE *fp_stream,FILE *bap_control )
{
  int i;
  
  code_value = 0;
  low = 0;
  high = top;
  zerocount = 0;
  
  for (i = 1;   i <= 16;   i++) {
    if(!bit_out_psc_layer(fp_stream,bap_control))
      return OFF; 
    code_value = 2 * code_value + Bit;
  }
  
  return ON;
}



int aa_decode(int cumul_freq[ ],FILE *fp_stream,FILE *bap_control)
{
	
  length = high - low + 1;
  cum = (-1 + (code_value - low + 1) * cumul_freq[0]) / length;
  for (sacindex = 1; cumul_freq[sacindex] > cum; sacindex++);
  high = low - 1 + (length * cumul_freq[sacindex-1]) / cumul_freq[0];
  low += (length * cumul_freq[sacindex]) / cumul_freq[0];
  for ( ; ; ) {  
    if (high < q2) ;
    else if (low >= q2) {
      code_value -= q2; 
      low -= q2; 
      high -= q2;
    }
    
    else if (low >= q1 && high < q3) {
      code_value -= q1; 
      low -= q1; 
      high -= q1;
    }
    
    else {
      break;
    }
    
    low *= 2; 
    high = 2*high + 1;
    if(!bit_out_psc_layer(fp_stream,bap_control))
      return DEC_ERROR; 
    code_value = 2*code_value + Bit;
    
  }
  
  return (sacindex-1);
}




void update(int symbol, int index,int *range_max, int *range_min,int **cumul_freq, int **freq) 
{
	int n;
	int *cumul_tmp, *freq_tmp;
	
	n = range_max[index]-range_min[index]+1;
	cumul_tmp = cumul_freq[index];
	freq_tmp = freq[index];
	
	if (cumul_tmp[0] == Max_frequency) {
		int i, cum;
		cum = 0;
		for(i=n-1; i>=0; i--) {
			freq_tmp[i] = (freq_tmp[i]+1)/2;
			cum += freq_tmp[i];
			cumul_tmp[i] = cum;
		}
		cumul_tmp[n] = 0;
	}
	
	if(symbol < 0 || symbol >= n){
		printf("Error in symbol%d : %d ",index+1,symbol);
		exit(0);
	}

	freq_tmp[symbol] += 1;
	cumul_tmp[symbol] += 1;
	while(symbol>0)
		cumul_tmp[--symbol] += 1;
}




void init_arith(int *Irange_max,int *Irange_min, 
		int **Icumul_freq, int *range_max, 
		int *range_min, int **cumul_freq, int **freq, 
		int mask[NO_BAPS])
{
  int i, j, n;


  for(i=0; i<NO_BAPS; i++) {
    if(mask[i]) {
      free(Icumul_freq[i]);
      n = Irange_max[i]-Irange_min[i]+1;
      Icumul_freq[i] = (int *) calloc(RANGE_MAX+1,sizeof(int));
      for(j=0; j<n; j++)
	Icumul_freq[i][j] = n-j;
      Icumul_freq[i][n] = 0;
    }
  }
  
  for(i=0; i<NO_BAPS; i++) {
    if(mask[i]) {
      if((freq[i]) != NULL)
	free(freq[i]);
      if((cumul_freq[i]) != NULL)
	free(cumul_freq[i]);      
      n = range_max[i]-range_min[i]+1;
     
      

      freq[i] = (int *) calloc(RANGE_MAX,sizeof(int));
     
      cumul_freq[i] = (int *) calloc(RANGE_MAX+1,sizeof(int));

      for(j=0; j<n; j++) {
	freq[i][j] = 1;
	cumul_freq[i][j] = n-j;
      }
      
      cumul_freq[i][n] = 0;
    }
  }
}



void AdjustBits(FILE *fp_stream) 
{
  int bitin;
  
  bitin = 8-bitsin;
  
  
  if(bitin<=5) {
    fseek(fp_stream, -3L, SEEK_CUR);
    bitsin = 0;
    GetBits(bitin+2,fp_stream);
  }
  else {
    fseek(fp_stream, -2L, SEEK_CUR);
    bitsin = 0;
    GetBits(bitin-6,fp_stream);
  }
  
}





/***********************************************************************

 
  Common parts of Bit-stream Decoder
  
   
***********************************************************************/




void Get_Fba_Object_Start_Code(FILE *fp_stream, FILE *bap_control)
{
  int dec_sym;
  
  dec_sym = GetBits(32,fp_stream);
  
  if (MY_DEBUG) fprintf(bap_control,"\nFBA_OBJECT_START_CODE %d",FBA_OBJECT_START_CODE);
  if(dec_sym != FBA_OBJECT_START_CODE) {
    printf("\n Wrong FBA-Sequence start code\n");
    exit(-1);
  }  
	
}



void Check_Fba_Paramset_Mask(int fba_paramset_mask)
{
  
  if(fba_paramset_mask!=BAP) {
    printf("\n Only BA-frames decoding implemented \n");
    exit(-1);
  }  
}




void  DeCode_Frame_Rate(float *frm_rate,FILE *fp_stream, FILE *bap_control)
{
  int frame_rate,seconds,frequency_offset;
  
  frame_rate = GetBits(8,fp_stream);
  if (MY_DEBUG) fprintf(bap_control,"\n frame_rate %d",frame_rate);
  seconds = GetBits(4,fp_stream);
  if (MY_DEBUG) fprintf(bap_control,"\n seconds %d",seconds);
  frequency_offset = GetBits(1,fp_stream);
  if (MY_DEBUG) fprintf(bap_control,"\n frequency_offset %d",frequency_offset);
 
  *frm_rate = (float)1000/(1000+frequency_offset)*(float)(frame_rate+(float)seconds/16);
}




void  DeCode_Time_Code(FILE *fp_stream,FILE *bap_control)
{
  int time_code_hours,time_code_minutes,marker_bit,time_code_seconds;
  
  time_code_hours   = GetBits(5,fp_stream);
  if (MY_DEBUG) fprintf(bap_control,"\n time_code_hours %d",time_code_hours);  
  time_code_minutes = GetBits(6,fp_stream);
  if (MY_DEBUG) fprintf(bap_control,"\n time_code_minutes %d",time_code_minutes);  
  marker_bit  = GetBits(1,fp_stream);
  if (MY_DEBUG) fprintf(bap_control,"\nmarker_bit  %d",marker_bit);  
  time_code_seconds = GetBits(6,fp_stream);
  if (MY_DEBUG) fprintf(bap_control,"\n time_code_seconds %d",time_code_seconds);  
  
 	
}




void DeCode_Skip_Frames(int *no_frames_skip,FILE *fp_stream,FILE *bap_control)
{
  int nfs,value;
  
  nfs = GetBits(4,fp_stream);
	
  if(nfs<15) {
    *no_frames_skip = nfs;
    return;
  }
  
  value = nfs;
  
  do {
    value = GetBits(4,fp_stream);
    nfs+=value;
  }
  while (value == 15);
  
  if(nfs>254) {
    printf("\n Maximum allowable frames skip is 254\n");
    exit(-1);
  }
  if (MY_DEBUG) fprintf(bap_control,"\n nr_skip_frames %d",nfs);
  *no_frames_skip = nfs;
}



void Decode_BAP_Mask( FILE *fp_stream,FILE *bap_control)
{
  int counter,bap_mask_type,mask,mark_bit;

  for(counter=0;counter<NO_BAPS;counter++)
    baps_mask[counter] = 0;  
    
  for(counter=0;counter<BAP_NUM_GROUPS;counter++) {
    
    mark_bit=GetBits(1,fp_stream);
    if (MY_DEBUG) fprintf(bap_control,"\n mark_bit %d",mark_bit);
    
    bap_mask_type = GetBits(2,fp_stream);
    if (MY_DEBUG) fprintf(bap_control,"\n bap_mask_type %d",bap_mask_type);
    
    if (MY_DEBUG) fprintf(bap_control,"\n mask: ");
    switch(bap_mask_type) {
			
    case BAP_MASK_TYPE0:
      break;
      
    case BAP_MASK_TYPE1:
      for(mask=0;mask<NBAP_GROUP[counter];mask++) 
	{
	  baps_mask[BAPS_IN_GROUP[counter][mask]-1] = GetBits(1,fp_stream);
	  if (MY_DEBUG) fprintf(bap_control,"%d ",baps_mask[BAPS_IN_GROUP[counter][mask]-1]);
	}
      break;
      
    case BAP_MASK_TYPE2:
      break;
      
    case BAP_MASK_TYPE3:
     for(mask=0;mask<NBAP_GROUP[counter];mask++) 
       {
	 baps_mask[BAPS_IN_GROUP[counter][mask]-1] = ON;
	 if (MY_DEBUG) fprintf(bap_control,"%d ",baps_mask[BAPS_IN_GROUP[counter][mask]-1]);
       }
      break;
		}
	}
}




int Check_Fba_Object_End(FILE *fp_stream,FILE *bap_control)
{
  int  b0,b1,b2,b3;
  long  position = ftell(fp_stream);
  

  if(!bitsin) getc(fp_stream);
	
  b0 = getc(fp_stream);
  b1 = getc(fp_stream);
  b2 = getc(fp_stream);
  b3 = getc(fp_stream);
  
  if( b0==0 && b1==0 && b2==1 && b3==3 ) {
    fseek(fp_stream,position,SEEK_SET);
    return OFF;
  }
  else if( b0==0 && b1==0 && (b2==1 || b2==0) ) return ON;
  else fseek(fp_stream,position,SEEK_SET);
  
  return OFF;
}



int Check_Fba_Object_Plane_Start_Code(FILE *fp_stream,FILE *bap_control)
{
  int   b0,b1,b2,b3;
  char a1;
  long  position = ftell(fp_stream);
  

  if(!bitsin) 
    {
      a1=getc(fp_stream);
      bitcount+=8;
      if (MY_DEBUG) fprintf(bap_control,"\n next_start_code %c",a1);
    }

  
  b0 = getc(fp_stream);
  b1 = getc(fp_stream);
  b2 = getc(fp_stream);
  b3 = getc(fp_stream);
  

  if( b0==0 && b1==0 && b2==1 && b3==3 ) 
    {
      bitsin = 0;
      if (MY_DEBUG) fprintf(bap_control,"\n Fba_Object_Plane_Start_Code %d",b2*256+b3);
      bitcount+=32;
    }
  else 
    {
      fseek(fp_stream,position,SEEK_SET);
      
      if (MY_DEBUG) fprintf(bap_control,"\n Fba_Object_Plane_Start_Code nu aici %d",b2*256+b3);
    }
  

  return ON;
}





/****************************************************************************


	Frame Based Decoding


*****************************************************************************/


void ResetIntra(int *quant_steps,int bap_quant)
{
  int counter;
  
  for(counter=0;counter<NO_BAPS;counter++) {
    baps[counter] = 0;
    quant_steps[counter] = bap_pred_scale[bap_quant]*BapQuantSteps[counter];
	}
}




int CheckGroupBegin(int pos)
{
  int counter;
  
  for(counter=0;counter<BAP_NUM_GROUPS;counter++) {
    if(BAPS_IN_GROUP[counter][0]==pos) return ON;
  }
  
  return OFF;
}



void DeCode_New_Minmax(	int is_I_new_max,int is_I_new_min,int is_P_new_max,
			int is_P_new_min,int *i_new_max,int *i_new_min,int *p_new_max,
			int *p_new_min,int *Irange_max,int  *Irange_min,int *range_max,
			int *range_min,FILE *fp_stream,FILE *bap_control)
{
  int counter,i,j;
  
  if (MY_DEBUG) fprintf(bap_control,"\n Decode_new_Minmax");
  if(is_I_new_max)
    for (i=0; i<BAP_NUM_GROUPS; i++) {
      for(j=0;j<NBAP_GROUP[i]; j++) {
	counter=BAPS_IN_GROUP[i][j]-1;
	 if (!(j&3)) 
	   if(!GetBits(1,fp_stream)) {
	     printf("WRONG MARKERBIT in CodeNewMinMax\n");
	     exit(-1);
	   }
	   else
	     {
	       if (MY_DEBUG) fprintf(bap_control,"\n marker bit 1");
	     }
	 if(baps_mask[counter]) { 
	   i_new_max[counter] = GetBits(5,fp_stream);
	   if (MY_DEBUG) fprintf (bap_control,"\ni_new_max[%d] %d ",counter,i_new_max[counter]);
	   Irange_max[counter] = RANGE_DELTA*i_new_max[counter];
	 }
      }
    }
  if(is_I_new_min) 
    for (i=0; i<BAP_NUM_GROUPS; i++) {
      for(j=0;j<NBAP_GROUP[i]; j++) {
	counter=BAPS_IN_GROUP[i][j]-1;
	if (!(j&3)) 
	  if(!GetBits(1,fp_stream)) {
	    printf("WRONG MARKERBIT in CodeNewMinMax\n");
	    exit(-1);
	  }
	  else
	    {
	      if (MY_DEBUG) fprintf(bap_control,"\n marker bit 1");
	    }
	if(baps_mask[counter]) { 
	  i_new_min[counter] = GetBits(5,fp_stream);
	  if (MY_DEBUG) fprintf (bap_control,"\ni_new_min[%d] %d ",counter,i_new_min[counter]);
	  Irange_min[counter] = RANGE_DELTA*i_new_min[counter];
	}
      }
    }

  
  if(is_P_new_max) 
    for (i=0; i<BAP_NUM_GROUPS; i++) {
      for(j=0;j<NBAP_GROUP[i]; j++) {
	counter=BAPS_IN_GROUP[i][j]-1;
	if (!(j&3)) 
	  if(!GetBits(1,fp_stream)) {
	    printf("WRONG MARKERBIT in CodeNewMinMax\n");
	    exit(-1);
	  }
	  else
	    {
	      if (MY_DEBUG) fprintf(bap_control,"\n marker bit 1");
	    }
	if(baps_mask[counter]) { 
	  p_new_max[counter] = GetBits(5,fp_stream);
	  if (MY_DEBUG) fprintf (bap_control,"\np_new_max[%d] %d ",counter,p_new_max[counter]);
	   range_max[counter] = RANGE_DELTA*p_new_max[counter];
	 }
      }
    }

  if(is_P_new_min) 
    for (i=0; i<BAP_NUM_GROUPS; i++) {
      for(j=0;j<NBAP_GROUP[i]; j++) {
	counter=BAPS_IN_GROUP[i][j]-1;
	if (!(j&3)) 
	  if(!GetBits(1,fp_stream)) {
	    printf("WRONG MARKERBIT in CodeNewMinMax\n");
	    exit(-1);
	  }
	  else
	    {
	      if (MY_DEBUG) fprintf(bap_control,"\n marker bit 1");
	    }
	if(baps_mask[counter]) { 
	  p_new_min[counter] = GetBits(5,fp_stream);
	  if (MY_DEBUG) fprintf (bap_control,"\np_new_min[%d] %d ",counter,p_new_min[counter]);
	  range_max[counter] = RANGE_DELTA*p_new_min[counter];
	}
      }
    }
  if (MY_DEBUG) fprintf(bap_control,"\n end Decode_new_Minmax");
 
}





int DeCode_IBap(int *bapQuantSteps,
		int *Irange_max, int *Irange_min, int **Icumul_freq, 
		int *range_max, int *range_min, 
		int **cumul_freq, int **freq,int scale,FILE *fp_stream,FILE *bap_control)
{
  int counter1,counter,group_number;
  int out_dec,dec_bap,qstep;
  
  
  if(!decoder_reset(fp_stream,bap_control))
    return OFF;

  for(group_number=0;group_number<BAP_NUM_GROUPS;group_number++) {
    for(counter1=0;counter1<NBAP_GROUP[group_number];counter1++) 
      if(baps_mask[BAPS_IN_GROUP[group_number][counter1]-1]) {
	counter=BAPS_IN_GROUP[group_number][counter1]-1;
    	out_dec = aa_decode(Icumul_freq[counter],fp_stream,bap_control);
	if (MY_DEBUG) fprintf(bap_control,"\n out arth codec: %d",out_dec);
	if(out_dec == DEC_ERROR)
	  return OFF;
	else
	  {
	    update(out_dec,counter,Irange_max,Irange_min,Icumul_freq,freq );
	    out_dec+=Irange_min[counter];
	  }
	qstep=bapQuantSteps[counter];
	dec_bap = dequantize(out_dec,qstep);
	baps[counter] = dec_bap;
	if (MY_DEBUG) fprintf(bap_control,"\nbaps[%d]=%d",counter,baps[counter]);
      }
  }
  
  AdjustBits(fp_stream);
  if (MY_DEBUG) fprintf(bap_control,"\n BITSIN=%d,posCur=%ld",bitsin,ftell(fp_stream));
  
  
  return ON;
}




int DeCode_PBap(int *bapQuantSteps,
		int *Irange_max, int *Irange_min, int **Icumul_freq, 
		int *range_max, int *range_min, 
		int **cumul_freq, int **freq,int scale,FILE *fp_stream,FILE *bap_control)
{
  int counter1,counter,group_number;
  int out_dec,dec_bap,qstep;
  
  
  if(!decoder_reset(fp_stream,bap_control))
    {
      return OFF;
    }
	
 
  for(group_number=0;group_number<BAP_NUM_GROUPS;group_number++) {
    for(counter1=0;counter1<NBAP_GROUP[group_number];counter1++) 
      if(baps_mask[BAPS_IN_GROUP[group_number][counter1]-1]) {
	counter=BAPS_IN_GROUP[group_number][counter1]-1;
	out_dec = aa_decode(cumul_freq[counter],fp_stream,bap_control);
	if (MY_DEBUG) fprintf(bap_control,"\n out arth codec %d",out_dec);
	if(out_dec == DEC_ERROR)
	  return OFF;
	update(out_dec,counter,range_max,range_min,cumul_freq,freq);
	out_dec += range_min[counter];
	qstep=bapQuantSteps[counter];


	if (MY_DEBUG) fprintf(bap_control,"\n qstep =%d",qstep);
	dec_bap = dequantize(out_dec,qstep) + baps[counter];
	baps[counter] = dec_bap;
	if (MY_DEBUG) fprintf(bap_control,"\nbaps[%d]=%d",counter,baps[counter]);  
      }
  }
  AdjustBits(fp_stream);
  if (MY_DEBUG) fprintf(bap_control,"\n BITSIN=%d",bitsin);
  return ON;
}




void Print_Dec_BAPfile(int last,char *bap_file)
{
  int    j;
  static int n =  0;
  static FILE *fp_dec;
  static first = ON;
  char   word[BUFFER_SIZE];
  
  if(first) {
		
    sprintf(word,"%sDEC.bap",bap_file);
    if( (fp_dec  = fopen(word,"wb"))==NULL ) {
      printf("\n File - %s - not possible to open. \n",word);
      return;
    }
    
    for(j=0;j<200;j++)
      fprintf(fp_dec," ");
    fprintf(fp_dec,"\n");
    
    first = OFF;
  }
  
  if(last) {
    rewind(fp_dec);
    sprintf(word,"%sDEC.bap",bap_file);
    fprintf(fp_dec,"2.0 %s %d %d ",word,(int)fps,n);	
    fclose(fp_dec);
    return;
  }
  
  for(j=0;j<NO_BAPS;j++)
    fprintf(fp_dec,"%d ",baps_mask[j]);
  fprintf(fp_dec,"\n%d",n);
  
  
  
  for(j=0;j<NO_BAPS;j++) {
    if(baps_mask[j])
      fprintf(fp_dec," %d",(int)(baps[j]));
  }
  
	n++;
	fprintf(fp_dec,"\n");
	
}






/****************************************************************************
									     

	DCT Based Decoding


*****************************************************************************/


void MakeIDCTMatrix(float **idct)
{
  float idct_coef;
  int   line,colunm;
  
  for(line=0; line<SEGMENT_LENGTH; line++) {
    
    if(line==0) 
      idct_coef = (float)(1.0/sqrt(2));
    else idct_coef = 1;
    
    for(colunm=0; colunm<SEGMENT_LENGTH; colunm++)
      idct[colunm][line] = (float)(sqrt(2.0/SEGMENT_LENGTH))*idct_coef*
	(float)(cos(PI*(float)(2*colunm+1)*line/2.0/SEGMENT_LENGTH));
  }
}



void Obtain_IDCT(float *baps, float *baps_coeff, float **dct)
{
  int line,colunm;
  
  for (line=0; line<SEGMENT_LENGTH; line++) {
    baps_coeff[line]=0;
    for (colunm=0; colunm<SEGMENT_LENGTH; colunm++)
      baps_coeff[line] += dct[line][colunm]*baps[colunm];
  }
}



void Dequantize(int dc_Q,float *dc_pred,float quant_step)
{
  *dc_pred = (float)dc_Q*quant_step;
}



void DC_I_Decode(int *dc_Q,FILE *bap_stream,FILE *bap_control)
{
  *dc_Q = ShortGetBits(bap_stream);
  if (MY_DEBUG) fprintf(bap_control,"\n dc_Q %d bitsin %d",*dc_Q,bitsin);
  if(*dc_Q == DC_MIN-1)
    {
      if (bitsin!=0) GetBits(bitsin,bap_stream);
      *dc_Q = GetBits(32,bap_stream);
      if (MY_DEBUG) fprintf(bap_control,"\n dc_Q %d bitsin %d",*dc_Q,bitsin);
       }
  
  
}





void AC_Decode(float *baps_coeff, float quant_step, int **run_table,
	       int **ac_table,FILE *bap_stream,FILE *bap_control)
{
  int *coeff_Q,counter;
  int run_length,run_counter,symbol;

  coeff_Q = (int *)calloc(SEGMENT_LENGTH,sizeof(int));

  
  run_length = 1;	
  run_counter = 0;
  
  while(run_length ) {
    
    if( run_counter+1 >= SEGMENT_LENGTH )
      break;
    
    run_length = GetHuffmanSymbol(run_table,RUN,bap_stream, bap_control);
    
    if(run_length!=0) {
      run_counter+=run_length;			
      symbol = GetHuffmanSymbol(ac_table,AC,bap_stream,bap_control);
      if(symbol==256)
	DC_I_Decode(&coeff_Q[run_counter],bap_stream,bap_control);
      else coeff_Q[run_counter] = symbol-256;
    }
  }
  
  
  for(counter=1;counter<SEGMENT_LENGTH;counter++)
    Dequantize(coeff_Q[counter],&baps_coeff[counter],quant_step);	
  
  free(coeff_Q);
}


 

void Predict(float dc,float dc_pred,float *dc_diff)
{
  *dc_diff = dc - dc_pred;
}
					 



void Reconstruct(float dc_diff,float *dc_pred )
{
  *dc_pred +=dc_diff;
} 



void DC_P_Decode(int *dc_Q,int **dc_table,FILE *bap_stream,FILE *bap_control)
{
  
  *dc_Q = GetHuffmanSymbol(dc_table,DC,bap_stream,bap_control);
  
  if(*dc_Q == 0)
    DC_I_Decode(dc_Q,bap_stream,bap_control);
  else
    *dc_Q-=256;
  
}




void DeCode_ISegment( int *bap_mask, float **baps,
		      float *bapQuantSteps,float *dc_pred,float **idct, int **run_table,
		      int **ac_table,FILE *bap_stream,FILE *bap_control)
{
  int counter,i;
  int group_number;
  
  float **baps_coeff;
  int   dc_Q;
  
  baps_coeff = FloatMatrixAllocation(NO_BAPS,SEGMENT_LENGTH);
  

  
  for(group_number=0;group_number<BAP_NUM_GROUPS;group_number++)
    {
      for(i=0;i<NBAP_GROUP[group_number];i++)
	if(bap_mask[BAPS_IN_GROUP[group_number][i]-1]){
	  counter=BAPS_IN_GROUP[group_number][i]-1;
  	  DC_I_Decode(&dc_Q,bap_stream,bap_control);
	  Dequantize(dc_Q,&baps_coeff[counter][0],bapQuantSteps[counter]/3);
	  AC_Decode(baps_coeff[counter],bapQuantSteps[counter],run_table,ac_table,bap_stream,bap_control);
	  if (MY_DEBUG) fprintf(bap_control,"\n group_number %d",group_number);
	  Obtain_IDCT(baps_coeff[counter],baps[counter],idct);
	}
      
  }
  
  for(counter = 0; counter<NO_BAPS; counter++)
    dc_pred[counter] = baps_coeff[counter][0];
  
 
  MatrixDealoc(baps_coeff,NO_BAPS);
}




void DeCode_PSegment(int *bap_mask,float **baps,
		     float *bapQuantSteps,float *dc_pred,float **idct,int **run_table,
		     int **dc_table,int **ac_table,FILE *bap_stream,FILE *bap_control)
{
  int counter,i;
  int group_number;
  
  float **baps_coeff;
  int   dc_Q;
  float *dc_diff;
  
  baps_coeff = FloatMatrixAllocation(NO_BAPS,SEGMENT_LENGTH);
  dc_diff    = (float *)calloc(NO_BAPS,sizeof(float));
	

  for(group_number=0;group_number<BAP_NUM_GROUPS;group_number++) {
    for(i=0;i<NBAP_GROUP[group_number];i++)
      if(bap_mask[BAPS_IN_GROUP[group_number][i]-1]){
	counter=BAPS_IN_GROUP[group_number][i]-1;		
	DC_P_Decode(&dc_Q,dc_table,bap_stream,bap_control);	
	Dequantize(dc_Q,&dc_diff[counter],bapQuantSteps[counter]/3);
	Reconstruct(dc_diff[counter],&dc_pred[counter]);
	baps_coeff[counter][0] = dc_pred[counter];
	AC_Decode(baps_coeff[counter],bapQuantSteps[counter],run_table,ac_table,bap_stream,bap_control);
	Obtain_IDCT(baps_coeff[counter],baps[counter],idct);
      }
  }
  
  
  
  MatrixDealoc(baps_coeff,NO_BAPS);
  free(dc_diff);
}



void ResetDCTIntra(float **baps,float *dctquant_steps,int bap_quant_index)
{
	int line,colunm,counter;

	for(line=0;line<NO_BAPS;line++) {
	  for(colunm=0;colunm<SEGMENT_LENGTH;colunm++)
	    {
	      baps[line][colunm]=0;
	     
	    }
	  
	}
	
	for(counter=0;counter<NO_BAPS;counter++) 
	  dctquant_steps[counter] = bap_scale[bap_quant_index]*DCTQuantStepsBAPS[counter];

}


void Print_DCTDec_BAPfile(float **baps,int *bap_mask,int last,char *bap_file,int fps)
{
  int    i,j;
  static int n =  0;
  static FILE *fp_dec;
  static int first = ON;
  char   word[BUFFER_SIZE];


  if(last) {
    rewind(fp_dec);
    sprintf(word,"%sDCTDEC.bap",bap_file);
    fprintf(fp_dec,"2.0 %s %d %d ",word,fps,n);	
    fclose(fp_dec);
    return;
  }
  
  if(first) {
    
    sprintf(word,"%sDCTDEC.bap",bap_file);
    if( (fp_dec  = fopen(word,"wb"))==NULL ) {
      printf("\n File - %s - not possible to open. \n",word);
      return;
    }
    
    for(i=0;i<200;i++)
      fprintf(fp_dec," ");
    fprintf(fp_dec,"\n");
    
    first = OFF;
  }
  
  for(i=0;i<SEGMENT_LENGTH;i++) 
    {
      for(j=0;j<NO_BAPS;j++)
	{
	  fprintf(fp_dec,"%d ",bap_mask[j]);
	}
      fprintf(fp_dec,"\n%d",n);
      
      for(j=0;j<NO_BAPS;j++)
	if(bap_mask[j])
	  fprintf(fp_dec," %d",(int)baps[j][i]);
      n++;
    fprintf(fp_dec,"\n");
  }


}


/******************************************************************************


 Decode_Fba_Object_Plane()


******************************************************************************/

static int fba_object_coding_type;

int Decode_Fba_Object_Plane(int frm_number,int *no_frames_skip,int *quant_steps,
			    int *is_rate,
			    int *i_new_max,int *i_new_min,int *p_new_max,
			    int *p_new_min,int *Irange_max,int *Irange_min,
			    int *range_max,int *range_min,int **freq,
			    int **cumul_freq,int **Icumul_freq,
			    float **dctbaps,int **bap_mask_mare,
			    float *dctquant_steps, float *dc_pred,float **idct,
			    int **run_table,int **dc_table,int **ac_table,
			    char *bap_file)
{	
  int   counter;
  int   fba_paramset_mask;
  int   is_time_code,is_frame_rate;
  int   skip_frames;
  int   is_I_new_max,is_I_new_min,is_P_new_max,is_P_new_min;
  int   is_intra;
  int   fba_suggested_gender;
  int bap_quant;
  
 
  
  long positions;
  
  positions=ftell (fp_stream);
 
  
  if( Check_Fba_Object_Plane_Start_Code(fp_stream,bap_control) ) {
        
   
    positions=ftell (fp_stream);
  
    is_intra = GetBits(1,fp_stream);
    if (MY_DEBUG) fprintf(bap_control,"\n is_intra %d bitcount= %d",is_intra,bitcount);
    fba_paramset_mask = GetBits(2,fp_stream);
    if (MY_DEBUG) fprintf(bap_control,"\n fba_paramset_mask %d",fba_paramset_mask);
    Check_Fba_Paramset_Mask(fba_paramset_mask);
   
    if(is_intra) {
      is_frame_rate = GetBits(1,fp_stream);
      if (MY_DEBUG) fprintf(bap_control,"\n is_frame_rate %d",is_frame_rate);
      *is_rate = is_frame_rate;
      if (is_frame_rate) 
	DeCode_Frame_Rate(&fps,fp_stream,bap_control);
			
      is_time_code = GetBits(1,fp_stream);
      if (MY_DEBUG) fprintf(bap_control,"\n is_time_code %d",is_time_code);
      if (is_time_code) 
	DeCode_Time_Code(fp_stream,bap_control);
    }
    
    skip_frames = GetBits(1,fp_stream);
    if (MY_DEBUG) fprintf(bap_control,"\n skip_frames %d",skip_frames);
    if (skip_frames) 
      DeCode_Skip_Frames(no_frames_skip,fp_stream, bap_control);
	
	
    if( fba_paramset_mask == BAP) {
      if(is_intra) {
	bap_quant = GetBits(5,fp_stream);

	if (MY_DEBUG) fprintf(bap_control,"\nbap_quant global %d",bap_quant);	
	Decode_BAP_Mask(fp_stream,bap_control);
	fba_suggested_gender=GetBits(1,fp_stream);
	fba_object_coding_type = GetBits(1,fp_stream);
	if (MY_DEBUG) fprintf(bap_control,"\n fba_object_coding_type %d",fba_object_coding_type);
	if(fba_object_coding_type == ARITH_TYPE) {
	  ResetIntra(quant_steps,bap_quant);
	  is_I_new_max = GetBits(1,fp_stream);
	  is_I_new_min = GetBits(1,fp_stream);
	  is_P_new_max = GetBits(1,fp_stream);
	  is_P_new_min = GetBits(1,fp_stream);
	
	  if (MY_DEBUG) fprintf(bap_control,"\n is_I_new_max %d",is_I_new_max);
	  if (MY_DEBUG) fprintf(bap_control,"\n is_I_new_min %d",is_I_new_min);
	  if (MY_DEBUG) fprintf(bap_control,"\n is_P_new_max %d",is_P_new_max);
	  if (MY_DEBUG) fprintf(bap_control,"\n is_P_new_min %d",is_P_new_min);

    

      if (frm_number==0){
	for(counter=0; counter<NO_BAPS; counter++) {
	  i_new_max[counter]=(int)fabs(((float)Irange_max[counter])/(float)RANGE_DELTA);
	  i_new_min[counter]=(int)fabs(((float)Irange_min[counter])/(float)RANGE_DELTA);
	  p_new_max[counter]=(int)fabs(((float)range_max[counter])/(float)RANGE_DELTA);
	  p_new_min[counter]=(int)fabs(((float)range_min[counter])/(float)RANGE_DELTA);

	  
	  Irange_max[counter] =(int)(RANGE_DELTA*(float)i_new_max[counter]/((float)bap_pred_scale[bap_quant]));
	  Irange_min[counter] = -(int)(RANGE_DELTA*(float)i_new_min[counter]/((float)bap_pred_scale[bap_quant]));
	  range_max[counter]  = (int) (RANGE_DELTA*(float)p_new_max[counter]/((float)bap_pred_scale[bap_quant]));
	  range_min[counter]  = -(int)(RANGE_DELTA*(float)p_new_min[counter]/((float)bap_pred_scale[bap_quant]));
	  
	}
      }
     
      DeCode_New_Minmax(is_I_new_max,is_I_new_min,is_P_new_max,is_P_new_min,
			i_new_max,i_new_min, p_new_max,p_new_min,
			Irange_max,Irange_min,range_max,range_min,fp_stream,bap_control);
      if(is_I_new_max || is_I_new_min || is_P_new_max || is_P_new_min) {
	for(counter=0; counter<NO_BAPS; counter++) {
	  if (baps_mask[counter]) {
	    if(is_I_new_max) Irange_max[counter] =(int)(RANGE_DELTA*(float)i_new_max[counter]/bap_quant);
	    if(is_I_new_min) Irange_min[counter] = -(int)(RANGE_DELTA*(float)i_new_min[counter]/bap_quant);
	    if(is_P_new_max) range_max[counter]  =(int) (RANGE_DELTA*(float)p_new_max[counter]/bap_quant);
	    if(is_P_new_min) range_min[counter]  = -(int)(RANGE_DELTA*(float)p_new_min[counter]/bap_quant);
	  }
	}
      }
   
      
      init_arith(Irange_max,Irange_min,Icumul_freq,
		 range_max,range_min,cumul_freq,
		 freq,baps_mask);
      if(!DeCode_IBap(quant_steps,Irange_max,Irange_min,Icumul_freq,
		      range_max, range_min,cumul_freq,freq,bap_quant,fp_stream,bap_control))
	return DEC_ERROR;
	}
	else if(fba_object_coding_type == DCT_TYPE) {
	  ResetDCTIntra(dctbaps,dctquant_steps,bap_quant);
	  DeCode_ISegment(baps_mask,dctbaps,dctquant_steps,dc_pred,idct,run_table,ac_table,fp_stream,bap_control);	  if (MY_DEBUG) fprintf(bap_control,"\n Am facut un segment");
	}
	else {
	  printf("\n Unknown coding type!");
	  return DEC_ERROR;
	}
      }
      else {
	if(fba_object_coding_type == ARITH_TYPE) {
	  if(!DeCode_PBap(quant_steps,Irange_max,Irange_min,
			  Icumul_freq,range_max,range_min,cumul_freq,freq,
			  bap_quant,fp_stream,bap_control))
	    return DEC_ERROR;
	}
	else if(fba_object_coding_type == DCT_TYPE) {
	  DeCode_PSegment(baps_mask,dctbaps,
			  dctquant_steps,dc_pred,idct,run_table,dc_table,
			  ac_table, fp_stream,bap_control);
	}
      }
     
      }
      
    }
  
  
  if(fba_object_coding_type == ARITH_TYPE) 
    Print_Dec_BAPfile(OFF,bap_file);
   

  if(fba_object_coding_type == DCT_TYPE)
    Print_DCTDec_BAPfile(dctbaps,baps_mask,OFF,bap_file,(int)fps);
	
  return Check_Fba_Object_End(fp_stream,bap_control);
  }




							   								   

void Decode_FBA_Object(char *bap_file)
{
  int  counter,frame_number,segment_counter;
  int  *quant_steps;
  
  int  no_frames_skip;
  int  end_code;
  
  int  is_frame_rate;
  float *dctquant_steps;
  float **idct;
  float **dctbaps;
  float *dc_pred;
  int   **dc_table,**ac_table,**run_table;	
  int **bap_mask_mare;
  
  
  int  *i_new_max,*i_new_min,*p_new_max,*p_new_min;
  int  *Irange_max, *Irange_min,*range_max, *range_min;
  int  **freq,**cumul_freq,**Icumul_freq;
	

 
  quant_steps = (int *)calloc(NO_BAPS,sizeof(int));
  
  
  dctquant_steps = (float *)calloc(NO_BAPS,sizeof(float));
  dctbaps  = FloatMatrixAllocation(NO_BAPS,SEGMENT_LENGTH);
  
  bap_mask_mare=IntMatrixAllocation(NO_BAPS,SEGMENT_LENGTH);

  dc_pred    = (float *)calloc(NO_BAPS,sizeof(float));
  
  
  run_table = IntMatrixAllocation(16,10);
  dc_table  = IntMatrixAllocation(512,18);
  ac_table  = IntMatrixAllocation(512,18);
  
 
  ReadHuffmanTables(run_table,dc_table,ac_table);
  

  idct  = FloatMatrixAllocation(SEGMENT_LENGTH,SEGMENT_LENGTH);
  MakeIDCTMatrix(idct);
 
  Icumul_freq = (int **)calloc(NO_BAPS,sizeof(int *));
  freq		= (int **)calloc(NO_BAPS,sizeof(int *));
  cumul_freq	= (int **)calloc(NO_BAPS,sizeof(int *));

  i_new_max = (int *)calloc(NO_BAPS,sizeof(int));
  i_new_min = (int *)calloc(NO_BAPS,sizeof(int));
  p_new_max = (int *)calloc(NO_BAPS,sizeof(int));
  p_new_min = (int *)calloc(NO_BAPS,sizeof(int));
  
  Irange_max = (int *)calloc(NO_BAPS,sizeof(int));
  Irange_min = (int *)calloc(NO_BAPS,sizeof(int));
  range_max  = (int *)calloc(NO_BAPS,sizeof(int));
  range_min  = (int *)calloc(NO_BAPS,sizeof(int));
  
  
  for(counter=0; counter<NO_BAPS; counter++) {
    Irange_min[counter] = IrangeMin1[counter]; 
    Irange_max[counter] = IrangeMax1[counter];
    range_min[counter]  = rangeMin1[counter];
    range_max[counter]  = rangeMax1[counter];
  }
 
       

  bitsin = 0;
  

  frame_number = 0;
  segment_counter = 0;
  end_code = OFF;	
  
  Get_Fba_Object_Start_Code(fp_stream,bap_control);
  

  
  do {
    if (MY_DEBUG) 
      {
	if (fba_object_coding_type == ARITH_TYPE)
	  fprintf(bap_control,"  FRAME: %d",frame_number);
	if (fba_object_coding_type == DCT_TYPE)
	  fprintf(bap_control,"  SEGMENT: %d",segment_counter);
      } 
   
    end_code=Decode_Fba_Object_Plane(frame_number,&no_frames_skip,quant_steps,&is_frame_rate,
				     i_new_max,i_new_min, p_new_max,p_new_min,
				     Irange_max,Irange_min,range_max,range_min,freq,
				     cumul_freq,Icumul_freq,
				     dctbaps,bap_mask_mare,dctquant_steps,dc_pred,idct,
				     run_table, dc_table,ac_table,
				     bap_file);
 
    if(DEBUG  && (fba_object_coding_type == ARITH_TYPE)) {
      printf("\n - Frame %d decoded - ",frame_number-1);
    }
    if (fba_object_coding_type == ARITH_TYPE) frame_number++;
    else if(DEBUG  && (fba_object_coding_type == DCT_TYPE)) {
    
      segment_counter++;
      printf("\n - Segment %d decoded - ",segment_counter);
      
    }
    
  }
  while(!end_code); 
  
  if(fba_object_coding_type == ARITH_TYPE)
    Print_Dec_BAPfile(ON,bap_file);
  else if(fba_object_coding_type == DCT_TYPE)
    Print_DCTDec_BAPfile(dctbaps,baps_mask,ON,bap_file,(int)fps);

  if(end_code!=DEC_ERROR) 
    printf("   - END of FBA-sequence - ");
  else 
    printf("Error occured, exiting.");
	
	
  free(quant_steps);
	
  free(i_new_max);
  free(i_new_min);
  free(p_new_max);
  free(p_new_min);
  free(Irange_max);
  free(Irange_min);
  free(range_max);
  free(range_min);
  for(counter=0; counter<NO_BAPS; counter++) {
    free(Icumul_freq[counter]);
    free(freq[counter]);
    free(cumul_freq[counter]);
  }
  
  free(Icumul_freq);
  free(freq);
  free(cumul_freq);
 
  MatrixDealoc(idct,SEGMENT_LENGTH);
  MatrixDealoc(dctbaps,NO_BAPS);
  IntMatrixDealoc(bap_mask_mare,NO_BAPS);
  IntMatrixDealoc(run_table,16);
  IntMatrixDealoc(dc_table,512);
  IntMatrixDealoc(ac_table,512);
  
  free(dctquant_steps);
  free(dc_pred);  
}




int OpenStreamFile(char *stream_file)
{
  char  word[BUFFER_SIZE];
  
  sprintf(word,"%s.stream",stream_file);
  if( (fp_stream  = fopen(word,"rb"))==NULL ) {
    printf("\n File %s does not exist.\n",word);
    exit(0);
  }
  
  if (MY_DEBUG)
    {
      sprintf(word,"%s.ctrl-dec",stream_file);
      if( (bap_control  = fopen(word,"wb"))==NULL ) {
	printf("\n File %s does not exist.\n",word);
	exit(0);
      }
    }
  
  
  return ON;
}



/***********************************************************************

  
	Main
	
	  
***********************************************************************/


int main(int argc,char *argv[])
{
	
	
  if ( argc!=2 ) {
    printf("Usage:  <decoder> <location+bapfilename>  \n");
    return 0;
  }

  OpenStreamFile(argv[1]);	
  
  Decode_FBA_Object(argv[1]);
  fclose(fp_stream);
  if (MY_DEBUG) fclose(bap_control);

  return 0;
}


