
/*************************************************************************

This software module was originally developed by 

	Wei Wu (weiwu@stallion.risc.rockwell.com),  Rockwell Science Center 
        Tolga K. Capin (capin@lig.di.epfl.ch), EPFL-LIG,
        Joaquim Esmerado (jle@lig.di.epfl.ch), EPFL-LIG.

in the course of development of the MPEG-4 Video (ISO/IEC 14496-2). 
This software module is an implementation of a part of one or more MPEG-4 
Video tools as specified by the MPEG-4 Video. ISO/IEC gives users of the 
MPEG-4 Video free license to this software module or modifications thereof 
for use in hardware or software products claiming conformance to the MPEG-4 
Video. Those intending to use this software module in hardware or software 
products are advised that its use may infringe existing patents. The original
developer of this software module and his/her company, the subsequent editors
and their companies, and ISO/IEC have no liability for use of this software 
module or modifications thereof in an implementation. Copyright is not 
released for non MPEG-4 Video conforming products. Rockwell Science Center 
and EPFL-LIG retain full right to use the code for his/her own purpose, assign
or donate  the code to a third party and to inhibit third parties from using the
code  for non <MPEG standard> conforming products. This copyright notice must be 
included in all copies or derivative works. Copyright (c) 1997.

Module Name:	$Id: encode.c,v 1.5 1998/02/20 17:24:05 capin Exp $

Abstract:	main module for FAP/BAP encoding 

Update:		
		April-1997
		1. using new fap format
		2. using new bit-stream syntax according to SNHC vm4.0
		July-1997
		August-1997: 
		1. new syntax according to Working Draft 4.0
		2. use both epf and fap files
		Oct-1997:
		1. new syntax according to WD4.0Rev
		2. start code emulation
		3. AA code viseme and expression
		Nov.20-1997
		1. Kayvon cleans up code
		2. starts changes to add BAP capability
		June 24, 1998
		1. integrate DCT scheme
                November - 1998
		1.updated to fdis:
		fba_suggested_gender bit
		fba_object_coding_type reduced to 1 bit
		update minmax defaults

Change Log:     See end of file for RCS log

*************************************************************************/

#include <assert.h>
#include <string.h>
#include "sim.h"

#define BAP_OUT_FNAME  "bap.out"

/* global variables */
static int head_cnt = 0;
static int ibit_cnt = 0;
static int pbit_cnt = 0;
static int total_bit_cnt = 0;
fba_object_plane_t fop;





/* function prototypes */
void initialize(int argc, char **argv, int* nframes);
void set_ve_ranges(void);
int expand_mask(void);
void cleanup(void);
void reset_frame(void);
void finish_frame(void);
bool_t get_line(FILE *);
void assure_fap_frame(void);
void assure_bap_frame(void);


/* module variables */
static FILE *fap_fp;
static FILE *bap_fp;
static FILE *epf_fp;



int main(int argc, char **argv) 
{
  int i, j;
  int maxn, minn;
  int hquant;
  char tag[4];
  bool_t frame_started = false;
  int nframes, curFrame, segLength;
  int temp_index;

  initialize(argc, argv, &nframes);
  initbits();

  /* fba_sequence() function is embedded here */
  head_cnt += putbits(32, FBA_OBJECT_START_CODE);

  if (fop.fo.coding_type == DCT_CODE_TYPE) {
    int first = 1;
    curFrame = 0;
    while (curFrame < nframes) {
	if (curFrame + SEG_LENGTH > nframes) {
	    segLength = nframes - curFrame;
	} else {
	    segLength = SEG_LENGTH;
	}
	read1Segment(segLength, fap_fp);
	if (first) {
		first = 0;
		fop.is_intra = 1;
	}
	fop.cur_frame_bit_cnt = 0;
	fba_object_plane_header();
	fba_object_plane_data();
	write1Segment(curFrame, fop.fo.fap_out_fp);
	if (fop.is_intra)
	    ibit_cnt += fop.cur_frame_bit_cnt;
 	else
	    pbit_cnt += fop.cur_frame_bit_cnt;
	curFrame += segLength;
    }

  } else { /* predictive coding type */
    while(get_line(epf_fp)) {

    fscanf(epf_fp, "%s", tag);
    
    if( strcmp(tag, "STC")==0) {/*startcode -edp*/
      fop.is_startcode = 1;
    }
    else if( strcmp(tag, "FFR")==0 ) {
      assure_fap_frame();
      fscanf(epf_fp, "%d", &fop.fo.frame_num);
    } else if( strcmp(tag, "BFR")==0 ) {
      assure_bap_frame();
      fscanf(epf_fp, "%d", &fop.bo.frame_num);
    } else if( strcmp(tag, "ISI")==0 ) {
      fscanf(epf_fp, "%d", &fop.is_intra);
      assert(fop.is_intra==0 || fop.is_intra==1);
    } else if( strcmp(tag, "FRR")==0 ) {
      fop.fo.is_frame_rate = true;
      fscanf(epf_fp, "%d%d%d",
	     &fop.fo.frame_rate, &fop.fo.fr_seconds, &fop.fo.fr_frequency_offset);
      assert(fop.fo.frame_rate>=0 && fop.fo.frame_rate<=255);
      assert(fop.fo.fr_seconds>=0 && fop.fo.fr_seconds<=15);
      assert(fop.fo.fr_frequency_offset==0 || fop.fo.fr_frequency_offset==1);
    } else if( strcmp(tag, "BRR")==0 ) {
      fop.bo.is_frame_rate = true;
      fscanf(epf_fp, "%d%d%d",
	     &fop.bo.frame_rate, &fop.bo.fr_seconds, &fop.bo.fr_frequency_offset);
      assert(fop.bo.frame_rate>=0 && fop.bo.frame_rate<=255);
      assert(fop.bo.fr_seconds>=0 && fop.bo.fr_seconds<=15);
      assert(fop.bo.fr_frequency_offset==0 || fop.bo.fr_frequency_offset==1);
    } else if( strcmp(tag, "TC")==0 ) {
      fop.fo.is_time_code = true;
      fscanf(epf_fp, "%d%d%d%d", &fop.fo.time_code_hours,
	     &fop.fo.time_code_minutes, &fop.marker_bit, &fop.fo.time_code_seconds);
    } else if( strcmp(tag, "BTC")==0) {
      fop.bo.is_time_code = true;
      fscanf(epf_fp, "%d%d%d%d", &fop.bo.time_code_hours,
	     &fop.bo.time_code_minutes, &fop.marker_bit, &fop.bo.time_code_seconds);
    } else if( strcmp(tag, "SKN")==0 ) {
      fop.fo.skip_frames = true;
      fscanf(epf_fp, "%d", &fop.fo.number_of_frames_to_skip);
    } else if( strcmp(tag, "BSK")==0 ) {
      fop.bo.skip_frames = true;
      fscanf(epf_fp, "%d", &fop.bo.number_of_frames_to_skip);
    } else if( strcmp(tag, "FMT")==0 ) {
      assure_fap_frame();
      for(i=0; i<N_FAP_GROUP; i++) {
	fscanf(epf_fp, "%d", &fop.fo.fap_mask_type[i]);
	assert(fop.fo.fap_mask_type[i]>=0 && fop.fo.fap_mask_type[i]<=3);
	if(fop.fo.fap_mask_type[i]==1 || fop.fo.fap_mask_type[i]==2) {
	  fop.fo.new_mask_flag = true;
	  for(j=fop.fo.mask_index[i]; j<fop.fo.mask_index[i+1]; j++) {
	    fscanf(epf_fp, "%d", &fop.fo.fap_mask[j]);
	    assert(fop.fo.fap_mask[j]==0 || fop.fo.fap_mask[j]==1);
	  }
	} else if(fop.fo.fap_mask_type[i]==0) {
	  for(j=fop.fo.mask_index[i]; j<fop.fo.mask_index[i+1]; j++) {
	    fop.fo.fap_mask[j] = 0;
	  }
	} else {
	  fop.fo.new_mask_flag = true;
	  for(j=fop.fo.mask_index[i]; j<fop.fo.mask_index[i+1]; j++) {
	    fop.fo.fap_mask[j] = 1;
	  }
	}
      }
    } else if( strcmp(tag, "BMT")==0 ) {
      assure_bap_frame();
      for(i=0; i<BAP_NUM_GROUPS; i++) {
	fscanf(epf_fp, "%d", &fop.bo.bap_mask_type[i]);
	assert(fop.bo.bap_mask_type[i]>=0 && fop.bo.bap_mask_type[i]<=3);
	if(fop.bo.bap_mask_type[i]==1) {
	  fop.bo.new_mask_flag = true;
	  for(j=0;j<fop.bo.baps_in_group[i]; j++) {
	    temp_index = fop.bo.bap_mask_index[i][j];
	    fscanf(epf_fp, "%d", &fop.bo.bap_mask[temp_index]);
	    assert(fop.bo.bap_mask[temp_index]==0 || fop.bo.bap_mask[temp_index]==1);
	  }
	}
	else if (fop.bo.bap_mask_type[i]==0) {
	  for(j=0;j<fop.bo.baps_in_group[i]; j++) {
	    temp_index = fop.bo.bap_mask_index[i][j];
	    fop.bo.bap_mask[temp_index] = 0;
	  }
	}
	else if (fop.bo.bap_mask_type[i]==3) {
	  fop.bo.new_mask_flag = true;
	  for(j=0;j<fop.bo.baps_in_group[i]; j++) {
	    temp_index = fop.bo.bap_mask_index[i][j];
	    fop.bo.bap_mask[temp_index] = 1;
	  }
	}
	else {
	  fprintf( stderr, "Error:\n");
	  fprintf( stderr,
		   "EPF file contains wrong BAP mask type for group %d\n", i);
	  exit(0);
	}
      }
    } else if( strcmp(tag, "FQU")==0 ) {
      assure_fap_frame();
      fop.fo.is_fap_quant = true;
      fscanf(epf_fp, "%d", &fop.fo.quant);
      hquant = fop.fo.quant/2;
      assert(fop.fo.quant>=0 && fop.fo.quant<=31);
      if(fop.fo.quant) 
	for(i=2;i<7;i+=2) {
	  fop.fo.Irange_ve_max[i] = (63+hquant)/fop.fo.quant;
	  fop.fo.range_ve_max[i] = (63+hquant)/fop.fo.quant;
	  fop.fo.range_ve_min[i] = -(63+hquant)/fop.fo.quant;
	}
    } else if( strcmp(tag, "BQU")==0 ) {
      assure_bap_frame();
      fop.bo.is_bap_quant = true;
      fscanf(epf_fp, "%d", &fop.bo.quant);
      assert(fop.bo.quant>=0 && fop.bo.quant<=31);
    } else if( strcmp(tag, "FIX")==0 ) {
      assure_fap_frame();
      fop.fo.is_i_new_max = true;
      /*fscanf(epf_fp, "%d%d%d%d%d", &maxn, &maxn, &maxn, &maxn, &maxn); -edp*/
      /*fscanf(epf_fp, "%d %d %d", &maxn, &maxn, &maxn); remove faps12 min/max*-edp*/
      for(i=2; i<NFAP; i++) {
	fscanf(epf_fp, "%d", &maxn);
	fop.fo.i_new_max[i] = (maxn+RANGE_DELTA-1)/RANGE_DELTA;
	maxn = fop.fo.i_new_max[i]*RANGE_DELTA; 
	if(fop.fo.quant) {
	  fop.fo.Irange_max[i] = maxn/fop.fo.quant;
	} else {
	  fop.fo.Irange_max[i] = maxn;
	}
      }
    } else if( strcmp(tag, "FIN")==0 ) {
      assure_fap_frame();
      fop.fo.is_i_new_min = true;
      /*fscanf(epf_fp, "%d%d%d%d%d", &minn, &minn, &minn, &minn, &minn);-edp*/
      /*fscanf(epf_fp, "%d %d %d", &minn, &minn, &minn);remove faps12 min/max -edp*/
      for(i=2; i<NFAP; i++) {
	fscanf(epf_fp, "%d", &minn);
	fop.fo.i_new_min[i] = -(minn-RANGE_DELTA+1)/RANGE_DELTA;
	minn = -fop.fo.i_new_min[i]*RANGE_DELTA; 
	if(fop.fo.quant) {
	  fop.fo.Irange_min[i] = minn/fop.fo.quant;
	} else {
	  fop.fo.Irange_min[i] = minn;
	}
      }
    } else if( strcmp(tag, "FPX")==0 ) {
      assure_fap_frame();
      fop.fo.is_p_new_max = true;
      /*fscanf(epf_fp, "%d%d%d%d%d", &maxn, &maxn, &maxn, &maxn, &maxn);-edp*/
      /*fscanf(epf_fp, "%d %d %d", &maxn, &maxn, &maxn);remove faps12 min/max -edp*/
      for(i=2; i<NFAP; i++) {
	fscanf(epf_fp, "%d", &maxn);
	fop.fo.p_new_max[i] = (maxn+RANGE_DELTA-1)/RANGE_DELTA;
	maxn = fop.fo.p_new_max[i]*RANGE_DELTA; 
	if(fop.fo.quant) {
	  fop.fo.range_max[i] = maxn/fop.fo.quant;
	} else {
	  fop.fo.range_max[i] = maxn;
	}
      }
    } else if( strcmp(tag, "FPN")==0 ) {
      assure_fap_frame();
      fop.fo.is_p_new_min = true;
      /*fscanf(epf_fp, "%d%d%d%d%d", &minn, &minn, &minn, &minn, &minn); -edp*/
      /*fscanf(epf_fp, "%d %d %d", &minn, &minn, &minn);remove faps12 min/max -edp*/
      for(i=2; i<NFAP; i++) {
	fscanf(epf_fp, "%d", &minn);
	fop.fo.p_new_min[i] = -(minn-RANGE_DELTA+1)/RANGE_DELTA;
	minn = -fop.fo.p_new_min[i]*RANGE_DELTA; 
	if(fop.fo.quant) {
	  fop.fo.range_min[i] = minn/fop.fo.quant;
	} else {
	  fop.fo.range_min[i] = minn;
	}
      }
    } else if( strcmp(tag, "BIX")==0 ) {
      assure_bap_frame();
      fop.bo.is_i_new_max = true;
      for(i=0; i<NUM_BAPS; i++) {
	fscanf(epf_fp, "%d", &maxn);
	fop.bo.i_new_max[i] = (maxn+RANGE_DELTA-1)/RANGE_DELTA;
	maxn = fop.bo.i_new_max[i]*RANGE_DELTA; 
	if(fop.bo.quant) {
	  fop.bo.Irange_max[i] = maxn/fop.bo.quant;
	} else {
	  fop.bo.Irange_max[i] = maxn;
	}
      }
    } else if( strcmp(tag, "BIN")==0 ) {
      assure_bap_frame();
      fop.bo.is_i_new_min = true;
      for(i=0; i<NUM_BAPS; i++) {
	fscanf(epf_fp, "%d", &minn);
	fop.bo.i_new_min[i] = -(minn-RANGE_DELTA+1)/RANGE_DELTA;
	minn = -fop.bo.i_new_min[i]*RANGE_DELTA; 
	if(fop.bo.quant) {
	  fop.bo.Irange_min[i] = minn/fop.bo.quant;
	} else {
	  fop.bo.Irange_min[i] = minn;
	}
      }
    } else if( strcmp(tag, "BPX")==0 ) {
      assure_bap_frame();
      fop.bo.is_p_new_max = true;
      for(i=0; i<NUM_BAPS; i++) {
	fscanf(epf_fp, "%d", &maxn);
	fop.bo.p_new_max[i] = (maxn+RANGE_DELTA-1)/RANGE_DELTA;
	maxn = fop.bo.p_new_max[i]*RANGE_DELTA; 
	if(fop.bo.quant) {
	  fop.bo.range_max[i] = maxn/fop.bo.quant;
	} else {
	  fop.bo.range_max[i] = maxn;
	}
      }
    } else if( strcmp(tag, "BPN")==0 ) {
      assure_bap_frame();
      fop.bo.is_p_new_min = true;
      for(i=0; i<NUM_BAPS; i++) {
	fscanf(epf_fp, "%d", &minn);
	fop.bo.p_new_min[i] = -(minn-RANGE_DELTA+1)/RANGE_DELTA;
	minn = -fop.bo.p_new_min[i]*RANGE_DELTA; 
	if(fop.bo.quant) {
	  fop.bo.range_min[i] = minn/fop.bo.quant;
	} else {
	  fop.bo.range_min[i] = minn;
	}
      }
    } else if( strcmp(tag, "FBM")==0) {
      if(frame_started)
	finish_frame();
      reset_frame();

      /* read in FBM line */
      frame_started = true;
      fscanf(epf_fp, "%d", &fop.fba_paramset);
      assert(fop.fba_paramset>=0 && fop.fba_paramset<=3);      
    }
    else 
      err_exit(LP, "tag %s unknown\n", tag);
  } 
  

  finish_frame();

  } /* predictive coding type */

  head_cnt += next_start_code();
  head_cnt += putbits(32, FBA_OBJECT_END_CODE);

  LOG(LP, "head_cnt = %d\n", head_cnt);
  LOG(LP, "ibit_cnt = %d\n", ibit_cnt);
  LOG(LP, "pbit_cnt = %d\n", pbit_cnt);
  LOG(LP, "total_bit_cnt = %d\n", head_cnt+ibit_cnt+pbit_cnt);

  cleanup();
  return 1;
}


void cleanup(void) 
{
  int i;
  for(i=0; i<NFAP; i++) {
    free(fop.fo.Ifreq[i]);
    free(fop.fo.Icumul_freq[i]);
    free(fop.fo.freq[i]);
    free(fop.fo.cumul_freq[i]);
  }
  for(i=0; i<7; i++) {
    free(fop.fo.Ifreq_ve[i]);
    free(fop.fo.Icumul_ve[i]);
    free(fop.fo.freq_ve[i]);
    free(fop.fo.cumul_ve[i]);
  }
  free(fop.fo.fap_mask);
  free(fop.fo.fap_mask2);
  free(fop.fo.prev_fap_mask);

  fclose(fap_fp);
  fclose(epf_fp);
  fclose(fop.stream_fp);
  fclose(fop.fo.fap_out_fp);
  fclose(bap_fp);
  for (i=0; i<NUM_BAPS; i++) {
    free(fop.bo.Ifreq[i]);
    free(fop.bo.Icumul_freq[i]);
    free(fop.bo.freq[i]);
    free(fop.bo.cumul_freq[i]);
  }
  fclose(fop.bo.bap_out_fp);
  if (fop.fo.coding_type == DCT_CODE_TYPE) {
    free(fop.fo.dctQP);
    for (i=0; i<NFAP; i++) {
      free(fop.fo.faps[i]);
    }
    free(fop.fo.faps);
    for (i=0; i<4; i++) {
      free(fop.fo.visemes[i]);
    }
    free(fop.fo.visemes);
    for (i=0; i<6; i++) {
      free(fop.fo.expressions[i]);
    }
    free(fop.fo.expressions);
    free(fop.fo.dcPred);
    for (i=0; i<SEG_LENGTH; i++) {
      free(fop.fo.dctMatrix[i]);
      free(fop.fo.idctMatrix[i]);
    }
    free(fop.fo.dctMatrix);
    free(fop.fo.idctMatrix);
  }
}


/* return false on EOF */
bool_t get_line(FILE *ifp) 
{
  int firstChar; /* XXX must be int, not char */
  char buf[BUFSIZ];

  while(feof(ifp) == 0) {
    if ((firstChar = fgetc(ifp)) == EOF) return false;
    if (firstChar == '#') { /* skip comments */
      if(!fgets(buf, BUFSIZ, ifp))
	return false;
    } else if(firstChar == '\t' || firstChar == ' ' ||
	      firstChar == '\n') { /* skip tracing empty space */
      ;
    } else {
      ungetc (firstChar, ifp);
      if(feof(ifp)) return false;
      return true;
    }
  }
  return false;
}



int expand_mask() 
{
  int i;
  for(i=0; i<NFAP; i++) 
    if(fop.fo.fap_mask[i]==1 && fop.fo.prev_fap_mask[i]==0)
      return 1;
  return 0;
}

/*
void finish_frame_bap(void)
{
  int i, j, curframe_num, dummy;

  if(fop.is_intra) 
    bap_init_arith();
  
  do {
    if(!get_line(bap_fp)) 
      err_exit(LP, "finish_frame_bap: expected line in .bap\n");

    for(j=0;j<NUM_BAPS;j++) {
	fscanf(bap_fp, "%d", &fop.bo.bap_mask2[j]);
    }

    if(!get_line(bap_fp))
      err_exit(LP, "finish_frame_bap: (1) expected line in .bap\n");
    
    / * 
    ** Read frame number
    * /

    fscanf(bap_fp, "%d", &curframe_num);
    assert(curframe_num<=fop.bo.frame_num);

    / *
    ** Read in data 
    * /

    for(i=0; i<NUM_BAPS; i++) {
	if(fop.bo.bap_mask2[i])
	  fscanf(bap_fp, "%d", &fop.bo.bap_orig[i]);
    }
  } while(curframe_num < fop.bo.frame_num);
  
  for(i=0; i<NUM_BAPS; i++) {
    fprintf(fop.bo.bap_out_fp, "%d ", fop.bo.bap_mask[i]);
  }
  fprintf(fop.bo.bap_out_fp, "\n");
  fprintf(fop.bo.bap_out_fp, "%d ", fop.bo.frame_num);

  fop.cur_frame_bit_cnt = 0;

  / * 
  ** Create the bitstream
  * /
  fba_object_plane_data();

  if(fop.is_intra) {
    ibit_cnt += fop.cur_frame_bit_cnt;
    for (i=0; i<NUM_BAPS; i++) {
      fop.bo.prev_bap_mask[i] = fop.bo.bap_mask[i];
    }
  }
  else 
    pbit_cnt += fop.cur_frame_bit_cnt;
}
*/

void code_frame(void) 
{
  int i, n;
  int j, curframe_num, dummy;

  if(fop.is_intra) {
    init_arith();
    bap_init_arith();

  }

  if(fop.fba_paramset & FAP_PARAMSET_MASK) {
    /* Read FAPs */
    do {
      if(!get_line(fap_fp)) err_exit(LP, "finish_frame_fap: expected line in .fap\n");
      
      for(i=0; i<NFAP; i++) 
	fscanf(fap_fp, "%d", &fop.fo.fap_mask2[i]);
      
      fscanf(fap_fp, "%d", &n);
      assert(n<=fop.fo.frame_num);
      if(fop.fo.fap_mask2[0]) 
	fscanf(fap_fp, "%d%d%d%d", &fop.fo.ve_orig[0],
	       &fop.fo.ve_orig[1], &fop.fo.ve_orig[2], &fop.fo.viseme_def);
      if(fop.fo.fap_mask2[1]) {
	fscanf(fap_fp, "%d%d%d%d%d%d", &fop.fo.ve_orig[3],
	       &fop.fo.ve_orig[4], &fop.fo.ve_orig[5], 
	       &fop.fo.ve_orig[6], &fop.fo.init_face, 
	       &fop.fo.expression_def);
      }
      for(i=2; i<NFAP; i++) {
	if(fop.fo.fap_mask2[i]) 
	  fscanf(fap_fp, "%d", &fop.fo.fap_orig[i]);
      }
    } while(n<fop.fo.frame_num);
    
    for(i=0; i<NFAP; i++) {
      fprintf(fop.fo.fap_out_fp, "%d ", fop.fo.fap_mask[i]);
    }
    fprintf(fop.fo.fap_out_fp, "\n");
    fprintf(fop.fo.fap_out_fp, "%d ", fop.fo.frame_num);
  }

  if(fop.fba_paramset & BAP_PARAMSET_MASK) {
  /* Read BAPs */
    do {
      if(!get_line(bap_fp)) 
	err_exit(LP, "finish_frame_bap: expected line in .bap\n");
      
      for(j=0;j<NUM_BAPS;j++) {
	fscanf(bap_fp, "%d", &fop.bo.bap_mask2[j]);
      }
      
      if(!get_line(bap_fp))
	err_exit(LP, "finish_frame_bap: (1) expected line in .bap\n");
      
      /* 
      ** Read frame number
      */
      
      fscanf(bap_fp, "%d", &curframe_num);
      assert(curframe_num<=fop.bo.frame_num);
      
      /*
      ** Read in data 
      */
      
      for(i=0; i<NUM_BAPS; i++) {
	if(fop.bo.bap_mask2[i])
	  fscanf(bap_fp, "%d", &fop.bo.bap_orig[i]);
      }
    } while(curframe_num < fop.bo.frame_num);
    
    for(i=0; i<NUM_BAPS; i++) {
      fprintf(fop.bo.bap_out_fp, "%d ", fop.bo.bap_mask[i]);
    }
    fprintf(fop.bo.bap_out_fp, "\n");
    fprintf(fop.bo.bap_out_fp, "%d ", fop.bo.frame_num);
  }

  /* 
  ** Create the bitstream
  */
  fop.cur_frame_bit_cnt = 0;
  fba_object_plane_data();

  if(fop.is_intra) {
    ibit_cnt += fop.cur_frame_bit_cnt;
    if(fop.fo.new_mask_flag && expand_mask()) {
      for(i=0; i<NFAP; i++) {
	fop.fo.prev_fap_mask[i] = fop.fo.fap_mask[i];
      }
      for (i=0; i<NUM_BAPS; i++) {
	fop.bo.prev_bap_mask[i] = fop.bo.bap_mask[i];
      }
    }
  } else 
    pbit_cnt += fop.cur_frame_bit_cnt;
}

void finish_frame(void)
{
  int i,minn,maxn,hquant;/*-edp*/

  if(fop.fo.p_new_max[3] == 0 && fop.fo.p_new_min[3] == 0) {/*init min/max values if not set before*/
    hquant = fop.fo.quant/2;
    if(fop.fo.quant)
      for(i=2;i<7;i+=2) {
	fop.fo.Irange_ve_max[i] = (63+hquant)/fop.fo.quant;
	fop.fo.range_ve_max[i] = (63+hquant)/fop.fo.quant;
	fop.fo.range_ve_min[i] = -(63+hquant)/fop.fo.quant;
      }
    for(i=2;i<NFAP;i++) {/*set init minmax values -edp*/
      fop.fo.i_new_max[i] = (DEFAULTMAX+RANGE_DELTA-1)/RANGE_DELTA;
      maxn = fop.fo.i_new_max[i]*RANGE_DELTA;
      if(fop.fo.quant) fop.fo.Irange_max[i] = maxn/fop.fo.quant;
      else fop.fo.Irange_max[i] = maxn;
      
      fop.fo.i_new_min[i] = -(DEFAULTMIN-RANGE_DELTA+1)/RANGE_DELTA;
      minn = -fop.fo.i_new_min[i]*RANGE_DELTA;
      if(fop.fo.quant) fop.fo.Irange_min[i] = minn/fop.fo.quant;
      else fop.fo.Irange_min[i] = minn;
      
      fop.fo.p_new_max[i] = (DEFAULTMAX+RANGE_DELTA-1)/RANGE_DELTA;
      maxn = fop.fo.p_new_max[i]*RANGE_DELTA;
      if(fop.fo.quant) fop.fo.range_max[i] = maxn/fop.fo.quant;
      else fop.fo.range_max[i] = maxn;
      
      fop.fo.p_new_min[i] = -(DEFAULTMIN-RANGE_DELTA+1)/RANGE_DELTA;
      minn = -fop.fo.p_new_min[i]*RANGE_DELTA;
      if(fop.fo.quant) fop.fo.range_min[i] = minn/fop.fo.quant;
      else fop.fo.range_min[i] = minn;
    }
    
  }

  fba_object_plane_header();
  code_frame();

  /*
  if(fop.fba_paramset & FAP_PARAMSET_MASK)
    finish_frame_fap();
  if(fop.fba_paramset & BAP_PARAMSET_MASK)
    finish_frame_bap();
    */
}



void reset_frame(void)
{
  fop.is_startcode = 0;/*edp*/
  fop.fo.is_frame_rate = false;
  fop.fo.skip_frames = false;

  fop.fo.new_mask_flag = false;
  fop.fo.is_i_new_max = false;
  fop.fo.is_i_new_min = false;
  fop.fo.is_p_new_max = false;
  fop.fo.is_p_new_min = false;
  fop.fo.is_fap_quant = false;

  fop.bo.is_frame_rate = false;
  fop.bo.skip_frames = false;

  fop.bo.new_mask_flag = false;
  fop.bo.is_i_new_max = false;
  fop.bo.is_i_new_min = false;
  fop.bo.is_p_new_max = false;
  fop.bo.is_p_new_min = false;
  fop.bo.is_bap_quant = false;
}

void initialize_fop(void)
{
  int  bap_mask_index[BAP_NUM_GROUPS][MAX_BAPS]={
                   {1,2,3},
                   {4,6,10,14},
                   {5,7,11,15},
                   {8,12,16,18,20,22},
                   {9,13,17,19,21,23},
                   {32,34,36,38,42},
                   {33,35,37,39,43},
                   {24,26,28,30,40,44,46},
                   {25,27,29,31,41,45,47},
                   {48,49,50,60,61,62,87,88,89,114,115,116},
                   {54,55,56,72,73,74,99,100,101,108,109,110,120,121,122},
                   {57,58,59,66,67,68,81,82,83,93,94,95,105,106,107,117,118,119},
                   {63,64,65,69,70,71,75,76,77,90,91,92,102,103,104,111,112,113},
                   {51,52,53,78,79,80,84,85,86,96,97,98},
                   {125,131,133,137,143,145,149,155,157,161,167,169,171,173,177,179},
                   {126,132,134,138,144,146,150,156,158,162,168,170,172,174,178,180},
                   {123,127,129,135,139,141,147,151,153,159,163,165,175},
                   {124,128,130,136,140,142,148,152,154,160,164,166,176},
                   {181,182,183,184,185,186}
/*     {1,2,3},*/
/*     {4,6,10,14},*/
/*     {5,7,11,15},*/
/*     {8,12,16,18,20,22},*/
/*     {9,13,17,19,21,23},*/
/*     {32,34,36,38,42},*/
/*     {33,35,37,39,43},*/
/*     {24,26,28,30,40,44,46},*/
/*     {25,27,29,31,41,45,47},*/
/*     {57,58,59,84,85,86,111,112,113},*/
/*     {51,52,53,69,70,71,96,97,98,105,106,107,117,118,119},*/
/*     {54,55,56,63,64,65,78,79,80,90,91,92,102,103,104,114,115,116},*/
/*     {60,61,62,66,67,68,72,73,74,87,88,89,99,100,101,108,109,110},*/
/*     {48,49,50,75,76,77,81,82,83,93,94,95},*/
/*     {120,126,128,130,136,138,140,146,148,150,156,158,160,162,166,168},*/
/*     {121,127,129,131,137,139,141,147,149,151,157,159,161,163,167,169},*/
/*     {124,122,134,132,144,142,154,152,164},*/
/*     {125,123,135,133,145,143,155,153,165},*/
/*     {170,171,172,173,174,175}*/
  };
  int baps_in_group[BAP_NUM_GROUPS] = {
/*    3, 4, 4, 6, 6, 5, 5, 7, 7, 9, 15, 18, 18, 12, 16, 16, 9, 9, 6 */
    3, 4, 4, 6, 6, 5, 5, 7, 7, 12, 15, 18, 18, 12, 16, 16, 13, 13, 6 
  };
  int i,j;

  reset_frame();
  fop.is_startcode = 0;/*-edp*/
  fop.fba_paramset = 1;
  fop.fo.quant = 1;
  fop.is_intra = true;
  fop.marker_bit = true;

  fop.fo.is_time_code = false;
  fop.fo.time_code_hours = 0;
  fop.fo.time_code_minutes = 0;
  fop.fo.time_code_seconds = 0;
  fop.marker_bit = true;

  fop.bo.quant = 1;
  fop.bo.is_time_code = false;
  fop.bo.time_code_hours = 0;
  fop.bo.time_code_minutes = 0;
  fop.bo.time_code_seconds = 0;

  /* Following replaces 
     int mask_index[] = {0, 2, 18, 30, 38, 42, 47, 50, 60, 64, 68}; */
  fop.fo.mask_index[0] = 0;
  fop.fo.mask_index[1] = 2;
  fop.fo.mask_index[2] = 18;
  fop.fo.mask_index[3] = 30;
  fop.fo.mask_index[4] = 38;
  fop.fo.mask_index[5] = 42;
  fop.fo.mask_index[6] = 47;
  fop.fo.mask_index[7] = 50;
  fop.fo.mask_index[8] = 60;
  fop.fo.mask_index[9] = 64;
  fop.fo.mask_index[10] = 68;

  for (i=0; i<BAP_NUM_GROUPS; i++) {
    fop.bo.baps_in_group[i] = baps_in_group[i];
    for (j=0; j<baps_in_group[i]; j++) {
      fop.bo.bap_mask_index[i][j] = bap_mask_index[i][j]-1;
    }
  
  }

}



void initialize(int argc, char **argv, int *nframes)
{
  int i, bap_nframes, fap_nframes;
  int fap_frame_rate, bap_frame_rate;
  float fap_version, bap_version;
  char fap_basename[128], bap_basename[128];
  
  if(argc != 7)
    err_exit(LP, "Usage: %s input_epf input_fap input_bap coding_type out_stream out_fap\n",
	     argv[0]);

  set_ve_ranges();
  initialize_fop();

  if((epf_fp = fopen(argv[1], "r")) == NULL)
    err_exit(LP, "initialize: cannot open EPF file %s\n", argv[1]);
  if((fap_fp = fopen(argv[2], "r")) == NULL)
    err_exit(LP, "initialize: cannot open FAP file %s\n", argv[2]);
  if((bap_fp = fopen(argv[3], "r")) == NULL)
    err_exit(LP, "initialize: cannot open BAP file %s\n", argv[3]);
  fop.fo.suggested_gender = 0;
  fop.fo.coding_type = atoi(argv[4]);
  if((fop.stream_fp = fopen(argv[5], "wb")) == NULL)
    err_exit(LP, "initialize: cannot open Stream file %s\n", argv[5]);
  if((fop.fo.fap_out_fp = fopen(argv[6], "w")) == NULL)
    err_exit(LP, "initialize: cannot open FAP out file %s\n", argv[6]);
  if((fop.bo.bap_out_fp = fopen(BAP_OUT_FNAME, "w")) == NULL)
    err_exit(LP, "initialize: cannot open BAP out file %s\n", BAP_OUT_FNAME);

  /* Read Header Information */
  fprintf(fop.fo.fap_out_fp, "# reconstructed fap file\n");
  fprintf(fop.bo.bap_out_fp, "# reconstructed bap file\n");

  /* version */
  if(!get_line(fap_fp)) err_exit(LP, "main: expected line in .fap\n");
  fscanf(fap_fp, "%f", &fap_version);
  if(fabsf(fap_version - 2.2) > 0.01)
    LOG(LP, "WARNING: FAP file version %2.1f != 2.2\n", fap_version);
  else
    LOG(LP, "FAP file version %2.1f\n", fap_version);
  if(!get_line(bap_fp)) err_exit(LP, "main: expected line in .bap\n");
  fscanf(bap_fp, "%f", &bap_version);
  LOG(LP, "BAP version %2.1f\n", bap_version);
  fprintf(fop.fo.fap_out_fp, "%2.1f ", fap_version);
  fprintf(fop.bo.bap_out_fp, "%2.1f ", bap_version);

  /* basename */
  fscanf(fap_fp, "%s", fap_basename);
  LOG(LP, "basename: %s (FAP)\n", fap_basename);
  fscanf(bap_fp, "%s", bap_basename);
  LOG(LP, "basename: %s (BAP)\n", bap_basename);
  fprintf(fop.fo.fap_out_fp, "%s ", fap_basename);
  fprintf(fop.bo.bap_out_fp, "%s ", bap_basename);

  /* frame rate */
  fscanf(fap_fp, "%d", &fap_frame_rate);
  LOG(LP, "Frame rate: %d (FAP)\n", fap_frame_rate);

  fprintf(fop.fo.fap_out_fp, "%d ", fap_frame_rate);

  fscanf(bap_fp, "%d", &bap_frame_rate);
  LOG(LP, "Frame rate: %d (BAP)\n", bap_frame_rate);
  fprintf(fop.bo.bap_out_fp, "%d ", bap_frame_rate);

  /* number of frames */
  fscanf(fap_fp, "%d", &fap_nframes);
  LOG(LP, "Sequence length: %d frames (FAP)\n", fap_nframes);
  fprintf(fop.fo.fap_out_fp, "%d\n", fap_nframes);
  *nframes = fap_nframes;

  fscanf(bap_fp, "%d", &bap_nframes);
  LOG(LP, "Sequence length: %d frames (BAP)\n", bap_nframes);
  fprintf(fop.bo.bap_out_fp, "%d\n", bap_nframes);

  fop.fo.fap_mask = (int*) malloc(NFAP*sizeof(int));
  fop.fo.fap_mask2 = (int*) malloc(NFAP*sizeof(int));
  fop.fo.prev_fap_mask = (int*) malloc(NFAP*sizeof(int));
  for(i=0; i<NFAP; i++) {
    fop.fo.prev_fap_mask[i] = 0;
    fop.fo.freq[i] = fop.fo.cumul_freq[i] = NULL;
    fop.fo.Ifreq[i] = fop.fo.Icumul_freq[i] = NULL;
  }
  for(i=0; i<7; i++) {
    fop.fo.Ifreq_ve[i] = fop.fo.Icumul_ve[i] = NULL;
    fop.fo.freq_ve[i] = fop.fo.cumul_ve[i] = NULL;
  }


  if (fop.fo.coding_type == DCT_CODE_TYPE) {
    fop.fo.dctQP = (float *) malloc(NFAP*sizeof(float));
    init_dctQP();
    fop.fo.faps = (float **) malloc(NFAP*sizeof(float*));
    for (i=0; i<NFAP; i++) {
	fop.fo.faps[i] = (float *) malloc(SEG_LENGTH*sizeof(float));
    }
    fop.fo.visemes = (int **) malloc(4*sizeof(int*));
    for (i=0; i<4; i++) {
	fop.fo.visemes[i] = (int*) malloc((SEG_LENGTH+1)*sizeof(int));
    }
    fop.fo.expressions = (int **) malloc(6*sizeof(int*));
    for (i=0; i<6; i++) {
	fop.fo.expressions[i] = (int*) malloc((SEG_LENGTH+1)*sizeof(int));
    }
    fop.fo.dcPred = (float*) malloc(NFAP*sizeof(float));
    init_htables();
    fop.fo.dctMatrix = (float**) malloc(SEG_LENGTH*sizeof(float*));
    fop.fo.idctMatrix = (float**) malloc(SEG_LENGTH*sizeof(float*));
    for (i=0; i<SEG_LENGTH; i++) {
	fop.fo.dctMatrix[i] = (float*) malloc(SEG_LENGTH*sizeof(float));
	fop.fo.idctMatrix[i] = (float*) malloc(SEG_LENGTH*sizeof(float));
    }
    init_dct();
  }
  for(i=2;i<NFAP;i++) {/*set init minmax values -edp*/
    fop.fo.i_new_max[i] = 0;
    fop.fo.Irange_max[i] = 0;

    fop.fo.i_new_min[i] = 0;
    fop.fo.Irange_min[i] = 0;

    fop.fo.p_new_max[i] = 0;
    fop.fo.range_max[i] = 0;

    fop.fo.p_new_min[i] = 0;
    fop.fo.range_min[i] = 0;
  }

  fop.bo.bap_mask = (int*) malloc(NUM_BAPS*sizeof(int));
  fop.bo.bap_mask2 = (int*) malloc(NUM_BAPS*sizeof(int));
  fop.bo.prev_bap_mask = (int*) malloc(NUM_BAPS*sizeof(int));
  for(i=0; i<NUM_BAPS; i++) {
    fop.bo.prev_bap_mask[i] = 0;
    fop.bo.freq[i] = fop.bo.cumul_freq[i] = NULL;
    fop.bo.Ifreq[i] = fop.bo.Icumul_freq[i] = NULL;
  }

}


void assure_fap_frame(void)
{
  if(fop.fba_paramset & FAP_PARAMSET_MASK)
    return;
  else
    err_exit(LP, "got a FAP tag, unexpected\n");
}

void assure_bap_frame(void)
{
  if(fop.fba_paramset & BAP_PARAMSET_MASK)
    return;
  else
    err_exit(LP, "got a BAP tag, unexpected\n");
}


/* XXX this should be changed back... */
void set_ve_ranges(void)
{
  int i;
  
  fop.fo.Irange_ve_max[0] = 14;
  fop.fo.Irange_ve_max[1] = 14;
  fop.fo.Irange_ve_max[2] = 63;
  fop.fo.Irange_ve_max[3] = 6;
  fop.fo.Irange_ve_max[4] = 63;
  fop.fo.Irange_ve_max[5] = 6;
  fop.fo.Irange_ve_max[6] = 63;

  i = 7;
  while(i--)
    fop.fo.Irange_ve_min[i] = 0;

  fop.fo.range_ve_max[0] = 14;
  fop.fo.range_ve_max[1] = 14;
  fop.fo.range_ve_max[2] = 63;
  fop.fo.range_ve_max[3] = 6;
  fop.fo.range_ve_max[4] = 63;
  fop.fo.range_ve_max[5] = 6;
  fop.fo.range_ve_max[6] = 63;

  fop.fo.range_ve_min[0] = -14;
  fop.fo.range_ve_min[1] = -14;
  fop.fo.range_ve_min[2] = -63;
  fop.fo.range_ve_min[3] = -6;
  fop.fo.range_ve_min[4] = -63;
  fop.fo.range_ve_min[5] = -6;
  fop.fo.range_ve_min[6] = -63;
}


