/************************************************************************
 *
 *  dctcode.c, part of MPEG4-SNHC DCT FAP coding
 *
 *  Author: Hai Tao <tao@ifp.uiuc.edu>
 *  Date:   Sept. 1st, 1997
 *  Tel:  1-(217)-244-1089	
 *  Mail:     405, N. Mathews Ave.
 *            Urbana, IL 61801, USA 
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY.
 ***********************************************************************/


#include <stdio.h>
#include <stdlib.h>
#include <math.h>
#include "util.h"




/* fba_sequence_start_code */
BYTE fss[8] = {0,0,0,0, 0,0,0,1,};
/* fba_sequence_end_code */
BYTE fse[8] = {0,0,0,0, 0,0,1,0,};
/* fba_segment_start_code */
BYTE fsegs[8] = {0,0,0,0, 0,0,1,1,};

typedef struct {
  BYTE fap_num;
  BYTE fap_id[NFAP];
}NODE;

/*#define M_DEBUG 1
 */
int output_stream_levels1(unsigned int number, FAP_STREAM_MEM *stream);
int output_stream_levels2(unsigned int number, FAP_STREAM_MEM *stream);
int output_stream_levels3(unsigned int number, FAP_STREAM_MEM *stream);
int output_stream_levels4(unsigned int number, FAP_STREAM_MEM *stream);
int output_stream_levels5(unsigned int number, FAP_STREAM_MEM *stream);

main(int argc, char **argv) {

  int i, j, k, l,n,m,t,v,tmpi, total_bit, node_num, tmpi1, tmpi2, tmpi3;
  int link_num, from, to, prio, func_num, var_num, term_num, denum;
  FILE *fit_fp, *stream_fp, *out_fp;
  /* the out put stream */
  FAP_STREAM_MEM out_stream;
  NODE *nodes;
  
  if(argc != 3) {
    fprintf(stderr, "Usage: %s input_fit output_stream\n", argv[0]);
    exit(-1);
  }
  
  total_bit = 0;
  if ( (fit_fp = fopen(argv[1], "r")) == NULL) {
    printf("The file %s can not be open !\n", argv[1]);
    exit(1);
  }
  
  if (( stream_fp = fopen(argv[2], "wb")) == NULL) {
    printf("The file %s can not be open !\n", argv[2]);
    exit(1);
  }
  
  /* out stream initialization */
  output_stream_init(&out_stream);
  /* output fba_sequence_start_code (fss), byte aligned */
  total_bit += output_stream_aligned(&out_stream);

  /* read nodes from .fit file */
  nextValidLine(fit_fp);
  fscanf(fit_fp, "%d", &node_num);
  finishLine(fit_fp);
  total_bit +=output_stream_levels4((unsigned int)node_num, &out_stream);
  
  nodes = (NODE *) calloc(node_num, sizeof(NODE));
  for (n=0; n<node_num; n++) {
    nextValidLine(fit_fp);
    fscanf(fit_fp, "%d", &tmpi),nodes[n].fap_num = (BYTE)(tmpi);
    total_bit +=output_stream_levels3((unsigned int)nodes[n].fap_num, &out_stream);
    
    for (i=0; i<nodes[n].fap_num; i++) {
      fscanf(fit_fp, "%d", &tmpi),nodes[n].fap_id[i] = (BYTE)(tmpi);
      total_bit += output_stream_bits_from_byte(nodes[n].fap_id[i],7, &out_stream);
    }
    finishLine(fit_fp);
  }
  
  /* read links from .fit file*/
  nextValidLine(fit_fp);
  fscanf(fit_fp, "%d", &link_num);
  finishLine(fit_fp);
  total_bit +=output_stream_levels4((unsigned int)link_num, &out_stream);
  printf("link_num = %d node bitcount = %d\n", link_num, total_bit);
  
  for(l=0; l<link_num; l++){
    fscanf(fit_fp, "%d%d%d", &from, &to, &prio);
    printf("(%d,%d,%d)\n", from, to, prio);
    total_bit +=output_stream_levels4((unsigned int)from, &out_stream);
    total_bit +=output_stream_levels4((unsigned int)to, &out_stream);
    total_bit +=output_stream_levels4((unsigned int)prio, &out_stream);
    /* read in functions */
    func_num = nodes[to].fap_num;
    for (j=0; j<func_num; j++) {
      fscanf(fit_fp, "%d", &term_num);
      total_bit +=output_stream_levels1((unsigned int)term_num, &out_stream);
      if (term_num != 0) {
	fscanf(fit_fp, "%d", &denum);
	total_bit +=output_stream_levels1((unsigned int)denum, &out_stream);
      }
      for(t=0; t<term_num; t++) {
	/* read in coefficient */
	/* sign */
	fscanf(fit_fp, "%d", &tmpi);  
	total_bit +=output_stream_bit((BYTE)(tmpi),&out_stream);
	/* value */
	fscanf(fit_fp, "%d", &tmpi);
	total_bit +=output_stream_levels1((unsigned int)tmpi, &out_stream);
	/* number of variables */ 
	fscanf(fit_fp, "%d", &var_num);
	total_bit +=output_stream_levels1((unsigned int)var_num, &out_stream);
	for (v=0; v<var_num; v++) {
	  fscanf(fit_fp, "%d%d%d", &tmpi1, &tmpi2, &tmpi3);
	  total_bit +=output_stream_levels2((unsigned int)tmpi1, &out_stream);
	  total_bit +=output_stream_levels1((unsigned int)tmpi2, &out_stream);
	  total_bit +=output_stream_levels1((unsigned int)tmpi3, &out_stream);
	}
      }
      fscanf(fit_fp, "%d", &term_num);
      total_bit +=output_stream_levels5((unsigned int)term_num, &out_stream);
       
      if (term_num){
	fscanf(fit_fp, "%d", &denum);
	total_bit +=output_stream_levels1((unsigned int)denum, &out_stream);
      }
      for(t=0; t<term_num; t++) {
	/* read in coefficient */
	/* sign */
	fscanf(fit_fp, "%d", &tmpi);  
	total_bit +=output_stream_bit((BYTE)(tmpi),&out_stream);
	/* value */
	fscanf(fit_fp, "%d", &tmpi);
	total_bit +=output_stream_levels1((unsigned int)tmpi, &out_stream);
	/* number of variables */ 
	fscanf(fit_fp, "%d", &var_num);
	total_bit +=output_stream_levels1((unsigned int)var_num, &out_stream);
	for (v=0; v<var_num; v++) {
	  fscanf(fit_fp, "%d%d%d", &tmpi1, &tmpi2, &tmpi3);
	  total_bit +=output_stream_levels2((unsigned int)tmpi1, &out_stream);
	     total_bit +=output_stream_levels1((unsigned int)tmpi2, &out_stream);
	     total_bit +=output_stream_levels1((unsigned int)tmpi3, &out_stream);
	}
      }
    }
    finishLine(fit_fp);  
  }
  total_bit += output_stream_aligned(&out_stream);
  output_stream_flush(stream_fp, &out_stream);
  
  /* stream file close */
  fclose(stream_fp);
  fclose(fit_fp);  
  printf("Totally %d bits\n", total_bit);
}


int output_stream_levels1(unsigned int number, FAP_STREAM_MEM *stream) {
  int count=0;
  if (number == 1) {
    count +=output_stream_bit(0,stream);
  }
  else {
    count +=output_stream_bit(1,stream);
    if (number <7)
      count +=output_stream_bits_from_byte((BYTE)number, 3,stream);
    else {
      count +=output_stream_bits_from_byte(7, 3,stream);
      if (number <31)
	count +=output_stream_bits_from_byte((BYTE)number, 5,stream);
      else {
	count +=output_stream_bits_from_byte(31, 5,stream);
	if (number <255)
	  count +=output_stream_byte((BYTE)number,stream);
	else {
	  count +=output_stream_byte(255,stream);
	  if(number <256*256-1)
	     count +=output_stream_ushort((unsigned short)number,stream);
	  else {
	    count +=output_stream_ushort(256*256-1,stream);
	    count +=output_stream_uint(number,stream);
	  }
	}
	  
      }
    }
  }
  return(count);
}

int output_stream_levels2(unsigned int number, FAP_STREAM_MEM *stream) {
    int count=0;

  if (number <7)
    count +=output_stream_bits_from_byte((BYTE)number, 3,stream);
  else {
    count +=output_stream_bits_from_byte(7, 3,stream);
    count +=output_stream_bits_from_byte((BYTE)number, 6,stream);
  }
  return(count);
}

int output_stream_levels3(unsigned int number, FAP_STREAM_MEM *stream) {
  int count=0;
  
  if (number <7)
    count +=output_stream_bits_from_byte((BYTE)number, 3,stream);
  else {
    count +=output_stream_bits_from_byte(7, 3,stream);
    count +=output_stream_bits_from_byte((BYTE)number, 8,stream);
  }
  return(count);
}

int output_stream_levels4(unsigned int number, FAP_STREAM_MEM *stream) {
  int count=0;
  
  if (number <31)
    count +=output_stream_bits_from_byte((BYTE)number, 5,stream);
  else {
    count +=output_stream_bits_from_byte(31, 5,stream);
    count +=output_stream_bits_from_byte((BYTE)number, 8,stream);
  }
  return(count);
}


int output_stream_levels5(unsigned int number, FAP_STREAM_MEM *stream) {
  int count=0;
  if (number == 0) {
    count +=output_stream_bit(0,stream);
  }
  else {
    count +=output_stream_bit(1,stream);
    if (number <3)
      count +=output_stream_bits_from_byte((BYTE)number, 2,stream);
    else {
      count +=output_stream_bits_from_byte(7, 3,stream);
      if (number <31)
	count +=output_stream_bits_from_byte((BYTE)number, 5,stream);
      else {
	count +=output_stream_bits_from_byte(31, 5,stream);
	if (number <255)
	  count +=output_stream_byte((BYTE)number,stream);
	else {
	  count +=output_stream_byte(255,stream);
	  if(number <256*256-1)
	     count +=output_stream_ushort((unsigned short)number,stream);
	  else {
	    count +=output_stream_ushort(256*256-1,stream);
	    count +=output_stream_uint(number,stream);
	  }
	}
	  
      }
    }
  }
  return(count);
}
