/************************************************************************
 *
 *  dctcode.c, part of MPEG4-SNHC DCT FAP coding
 *
 *  Author: Hai Tao <tao@ifp.uiuc.edu>
 *  Date:   Sept. 1st, 1997
 *  Tel:  1-(217)-244-1089	
 *  Mail:     405, N. Mathews Ave.
 *            Urbana, IL 61801, USA 
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY.
 ***********************************************************************/


#include <stdio.h>
#include <stdlib.h>
#include <math.h>
#include "util.h"

typedef struct {
  BYTE fap_num;
  BYTE fap_id[NFAP];
}FITNODE;

typedef struct {
  int var_index;
  int pow_nume;
  int pow_denom;
}ATOM;

typedef struct {
  int sign;
  int coeff;
  int num_of_var;
  ATOM *atoms;
}TERM;

typedef struct {
  int num_of_term;
  int denom_coeff;
  TERM  *terms;
}POLY;

typedef struct {
  POLY  nume_poly;
  POLY  denom_poly;
}IFUN;

typedef struct {
  int from;
  int to;
  int prio;
  IFUN  *functions;
}FITLINK;

/*
#define M_DEBUG 1
*/

void read_poly(POLY *polynomial, FILE *fp);
void print_poly(POLY *polynomial);
int   evaluate_poly(POLY *polynomial, float *var, int *mask, float *ret);
int  interpolate(FITLINK *link, FITNODE *from, FITNODE *to, float *var, int *mask);
main(int argc, char **argv) {

  int i, j, k, l,m,n, tmpi;
  FILE *fit_fp, *in_file_fp, *out_file_fp;
  int nframe, frameRate, frm_no;
  char basename[100];
  float version;
  int fap_mask[NFAP],ofap_mask[NFAP];
  float fap[NFAP];
  int link_num, func_num, node_num, parent_prio;
  int complete,inter_count, ototal_fap, total_fap;
  FITNODE *nodes;
  FITLINK *links;
  int *node_priority;
  
  if(argc != 4) {
    fprintf(stderr, "Usage: %s input_fit input_fap_file out_put_file\n", argv[0]);
    exit(-1);
  }
  if ( (fit_fp = fopen(argv[1], "r")) == NULL) {
    fprintf(stderr, "The file %s can not be open !\n", argv[1]);
    exit(1);
  }
  
  if (( in_file_fp = fopen(argv[2], "r")) == NULL) {
    fprintf(stderr, "The file %s can not be open !\n", argv[2]);
    exit(1);
  }
  
  if (( out_file_fp = fopen(argv[3], "wb")) == NULL) {
    fprintf(stderr, "The file %s can not be open !\n", argv[3]);
    exit(1);
  }
  
  /* read from .fit file, the nodes */
  nextValidLine(fit_fp);
  fscanf(fit_fp, "%d", &node_num);
  finishLine(fit_fp);
  
  nodes = (FITNODE *) calloc(node_num, sizeof(FITNODE));
  node_priority = (int *) calloc(node_num, sizeof(int));
  for (n=0; n<node_num; n++) {
    nextValidLine(fit_fp);
    fscanf(fit_fp, "%d", &tmpi),nodes[n].fap_num = (BYTE)(tmpi);
    
    for (i=0; i<nodes[n].fap_num; i++) {
      fscanf(fit_fp, "%d", &tmpi),nodes[n].fap_id[i] = (BYTE)(tmpi-1);
    }
    finishLine(fit_fp);
  }
  /* read from .fit file, the links */
  nextValidLine(fit_fp);
  fscanf(fit_fp, "%d", &link_num);
  finishLine(fit_fp);
#ifdef M_DEBUG
  printf("%d\n", link_num);
#endif
  links = (FITLINK *) calloc(link_num, sizeof(FITLINK));
  
  for(l=0; l<link_num; l++){
    fscanf(fit_fp, "%d%d%d", &links[l].from, &links[l].to, &links[l].prio);
#ifdef M_DEBUG
    printf("%d %d %d\n", links[l].from, links[l].to, links[l].prio);
#endif
    if (node_priority[links[l].to] < links[l].prio)
      node_priority[links[l].to] = links[l].prio;
    /* read in functions */
    func_num = nodes[links[l].to].fap_num;
    links[l].functions =  (IFUN *)calloc(func_num, sizeof(IFUN));
    /* read all functions for a link*/
    for (j=0; j<func_num; j++) {
      /* for each function, read in its numerator polynomial*/
      read_poly(&(links[l].functions[j].nume_poly), fit_fp);
      /* for each function, read in its denominator polynomial*/
      read_poly(&(links[l].functions[j].denom_poly), fit_fp);
#ifdef M_DEBUG
      print_poly(&(links[l].functions[j].nume_poly));
      printf("\n");
      print_poly(&(links[l].functions[j].denom_poly));
      printf("\n");
#endif
    }
  }
  fclose(fit_fp); 
  /* fit file close */

  /* interpolate in_file_fp to out_file_fp */
  /* Read Header Information from original faps file */
  nextValidLine(in_file_fp);
  fscanf(in_file_fp, "%f", &version);
  fscanf(in_file_fp, "%s", basename);
  fscanf(in_file_fp, "%d", &frameRate);
  fscanf(in_file_fp, "%d", &nframe);
  fprintf(out_file_fp, "%3.1f ", version);
  fprintf(out_file_fp, "cut ");
  fprintf(out_file_fp, "%d ", frameRate);
  fprintf(out_file_fp, "%d ", nframe);
  if (version < 1.5) {
    for(i=0; i<NGROUP; i++) {
      int mask;
      fscanf(in_file_fp, "%d", &mask);
      fprintf(out_file_fp, "%d ", mask);
    }
  }
  fprintf(out_file_fp, "\n");
  for (i=0; i<NFAP; i++)
    fscanf(in_file_fp, "%d", &ofap_mask[i]);
 
  /* read each frame of faps*/
  for(n=0; n<nframe; n++) {
    for (i=0; i<NFAP; i++)
      fap_mask[i] =ofap_mask[i];
    fscanf(in_file_fp, "%d", &frm_no); /* frame_number */
    for(i=0; i<NFAP; i++)
      if (ofap_mask[i])
	fscanf(in_file_fp, "%f", &fap[i]);
    /* interpolate fap[i] to fap[i], fap_mask[] to ofap_mask[] */
    ototal_fap = 0;
    for(i=0; i<NFAP; i++)
      ototal_fap += fap_mask[i];    
    do{
      for (i=0; i< node_num; i++) {
	/* check if this node is complete */
	complete = 1;
	for (m=0; m< nodes[i].fap_num; m++) {
	  tmpi = nodes[i].fap_id[m];
	  if (fap_mask[tmpi]==0) {
	    complete = 0;
	    break;
	  }
	}
	if (complete) continue;
	
      	for(parent_prio=1; parent_prio<= node_priority[i]; parent_prio++)   /* for all priority */
	  for (k=0; k<link_num; k++)                              /* for all links */
	    if (links[k].to == i && links[k].prio == parent_prio) /* if prio */
	      interpolate(&links[k], &nodes[links[k].from], &nodes[links[k].to], fap, fap_mask);
      }/* for all nodes end */
      total_fap = 0;
      for(i=0; i<NFAP; i++)
	total_fap += fap_mask[i];
      inter_count = total_fap - ototal_fap;
      ototal_fap = total_fap;
      /* printf("inter_count = %d\n",total_fap); */
    } while(inter_count > 0);


    if (n==0 || version >1.5) {
      for(i=0; i<NFAP; i++)
	fprintf(out_file_fp, "%d ", fap_mask[i]);
      fprintf(out_file_fp, "\n");
    }
    fprintf(out_file_fp, "%d ", frm_no);
    for(i=0; i<NFAP; i++)
      if (fap_mask[i])
	fprintf(out_file_fp, "%d ", round(fap[i]));
    fprintf(out_file_fp, "\n");
    if (version >1.5 && n<nframe-1)
      for (i=0; i<NFAP; i++)
	fscanf(in_file_fp, "%d", &ofap_mask[i]);
  }
  fclose(in_file_fp);
  fclose(out_file_fp);
}
void read_poly(POLY *polynomial, FILE *fp) {
  int i,j;
  TERM *termpt;

  fscanf(fp, "%d", &(polynomial->num_of_term));
  if (polynomial->num_of_term != 0) {
    fscanf(fp, "%d", &(polynomial->denom_coeff));
    polynomial->terms = (TERM *)calloc(polynomial->num_of_term, sizeof(TERM));
    for (i=0; i<polynomial->num_of_term; i++) {
      termpt = &(polynomial->terms[i]);
      /* read in term to polynomial->terms[i] */
      fscanf(fp, "%d", &(termpt->sign));
      fscanf(fp, "%d", &(termpt->coeff));
      fscanf(fp, "%d", &(termpt->num_of_var));
      termpt->atoms = (ATOM *)calloc(termpt->num_of_var, sizeof(ATOM));
      for (j=0; j<termpt->num_of_var; j++) {
	/* read each atom */
	fscanf(fp, "%d", &(termpt->atoms[j].var_index));
	fscanf(fp, "%d", &(termpt->atoms[j].pow_nume));
	fscanf(fp, "%d", &(termpt->atoms[j].pow_denom));
      }
    }
  }
}

void print_poly(POLY *polynomial) {
  int i,j;
  TERM *termpt;
  
  printf("%d ", polynomial->num_of_term);
  if (polynomial->num_of_term != 0) {
    printf("%d ", polynomial->denom_coeff);
    for (i=0; i<polynomial->num_of_term; i++) {
      termpt = &(polynomial->terms[i]);
      /*  print polynomial->terms[i] */
      printf("%d ", termpt->sign);
      printf("%d ", termpt->coeff);
      printf("%d ", termpt->num_of_var);
      for (j=0; j<termpt->num_of_var; j++) {
	/* print each atom */
	printf("%d ", termpt->atoms[j].var_index);
	printf("%d ", termpt->atoms[j].pow_nume);
	printf("%d ", termpt->atoms[j].pow_denom);
      }
    }
  }
}
     
int evaluate_poly(POLY *polynomial, float *var, int *mask, float *ret) {
  int i,j;
  TERM *termpt;
  float product;
  *ret = 0;
  if (polynomial->num_of_term != 0) {
    for (i=0; i<polynomial->num_of_term; i++) {
      termpt = &(polynomial->terms[i]);
      product = termpt->coeff;
      /*  evaluate polynomial->terms[i] */
      for (j=0; j<termpt->num_of_var; j++) {
	/* evaluate each atom */
	if(mask[termpt->atoms[j].var_index] != 1) return(-1);
	product *= (float)(pow(var[termpt->atoms[j].var_index], 
			       termpt->atoms[j].pow_nume/(double)(termpt->atoms[j].pow_denom)));
      }
      if (termpt->sign) product = 0-product;
      *ret += product;
    }
    *ret /= polynomial->denom_coeff;
  }
  else *ret = 1.0;
  return(0);
}
     

int  interpolate(FITLINK *link, FITNODE *from, FITNODE *to, float *var, int *mask) {
  float input_v[NFAP], output_v[NFAP], nvalue, dvalue;
  int input_mask[NFAP];
  int i, func_num;
  int get_some_thing = 0;
  
  for (i=0; i<from->fap_num; i++)
    input_v[i] = var[from->fap_id[i]];
  for (i=0; i<from->fap_num; i++)
    input_mask[i] = mask[from->fap_id[i]];
  
  func_num = to->fap_num;
  for (i=0; i<func_num; i++) {
    if (link->functions[i].nume_poly.num_of_term == 0){
      var[to->fap_id[i]] = 0;
      mask[to->fap_id[i]] = 1;
      get_some_thing = 1;
      continue;
    }
    
    if(evaluate_poly(&link->functions[i].nume_poly, input_v, input_mask, &nvalue) != -1)
      if(evaluate_poly(&link->functions[i].denom_poly, input_v, input_mask, &dvalue) != -1) {
	if (dvalue== 0)
	  var[to->fap_id[i]] = nvalue;
	else 
	  var[to->fap_id[i]] = nvalue/dvalue;
	mask[to->fap_id[i]] = 1;
	get_some_thing = 1;
      }
  }
  return(get_some_thing);
}

