/************************************************************************
 *
 *  util.h, part of MPEG4-SNHC DCT FAP coding
 *
 *  Author: Hai Tao <tao@ifp.uiuc.edu>
 *  Date:   Sept. 1st, 1997
 *  Tel:  1-(217)-244-1089	
 *  Mail:     405, N. Mathews Ave.
 *            Urbana, IL 61801, USA 
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY. 
 ***********************************************************************/

#ifndef   UTIL_H
#define  UTIL_H 1.0



#ifndef NFAP
#define NFAP	68
#endif

#ifndef NGROUP
#define NGROUP  10
#endif
#define SEG_LENGTH  16
#define MSCALE      5.0
#define SHORT_MAX   32767
#define SHORT_MIN   -32768

#define STREAM_SIZE 100000
#define PI 3.1415926
#define round(f) ((f) > 0.0 ? (int)((f)+0.499) : (int)((f)-0.499) )
typedef unsigned char BYTE;

typedef struct {       
  int bytes_in_stream;
  int byte_count;
  int bit_count;
  BYTE  stream[STREAM_SIZE];
} FAP_STREAM_MEM;

typedef struct {
  int no_of_symbol;
  int no_of_bits[512];
  int freq[512];
  int total_freq;
  BYTE bit[512][50];
} HUFFMAN_TABLE;


#ifdef UTLC
float **dct_matrix;
float ** idct_matrix;
int dct_size;

/*
float DCTQP[NFAP] = {1, 1, 1.5, 1.5, 1.5, 1.5, 1.5, 1.5, 1.5, 1.5, 
		    1.5, 1.5, 1.5, 3, 3, 3, 3, 1, 2, 2, 
                    2, 2, 0.05, 0.05, 0.05, 0.05, 1, 1, 1, 1, 
		    1.5, 1.5, 1.5, 1.5, 1.5, 1.5, 1.5, 1.5, 4, 4, 
		    4, 4, 2, 2, 2, 2, 0.03, 0.02, 0.03, 0.03, 
		    1.5, 1.5, 1.5, 1.5, 1.5, 1.5, 1.5, 1.5, 1.5, 1.5, 
                    3, 3, 3, 3, 2, 2, 2, 2};
		    */
float DCTQP[NFAP] = {1, 1, 1.5, 1.5, 1.5, 1.5, 1.5, 1.5, 1.5, 1.5, 
		    1.5, 1.5, 1.5, 3, 3, 3, 3, 1, 2, 2, 
                    2, 2, 85, 85, 85, 85, 1, 1, 1, 1, 
		    1.5, 1.5, 1.5, 1.5, 1.5, 1.5, 1.5, 1.5, 4, 4, 
		    4, 4, 2, 2, 2, 2, 51, 34, 51, 51, 
		    1.5, 1.5, 1.5, 1.5, 1.5, 1.5, 1.5, 1.5, 1.5, 1.5, 
                    3, 3, 3, 3, 2, 2, 2, 2};
	
char dc_table_name[] = "./TABLE/dc.table";
char ac_table_name[] = "./TABLE/ac.table";
char rl_table_name[] = "./TABLE/runlength.table";
char v_s_table_name[] = "./TABLE/v_sel.table";
char v_b_table_name[] = "./TABLE/v_blend.table";
char e_s_table_name[] = "./TABLE/e_sel.table";
char e_i_table_name[] = "./TABLE/e_intensity.table";
#else
extern float **dct_matrix;
extern float ** idct_matrix;
extern int dct_size;

extern float DCTQP[NFAP] ;
extern char dc_table_name[];
extern char ac_table_name[];
extern char rl_table_name[];
extern char v_s_table_name[];
extern char v_b_table_name[];
extern char e_s_table_name[];
extern char e_i_table_name[];
#endif
  int output_stream_huffman_symbol(int symbol_index, 
			     HUFFMAN_TABLE *table,FAP_STREAM_MEM *stream );
  int output_stream_int(int input, FAP_STREAM_MEM *stream );
  int output_stream_uint(unsigned int input, FAP_STREAM_MEM *stream );
  int output_stream_ushort(unsigned short input, FAP_STREAM_MEM *stream );
  int output_stream_short(short input, FAP_STREAM_MEM *stream );
  int output_stream_byte(BYTE input, FAP_STREAM_MEM *stream );
  int output_stream_bit(BYTE input, FAP_STREAM_MEM *stream );
  int output_stream_bits(BYTE *input,int length,  FAP_STREAM_MEM *stream );
/* the last length<=8 bits of BYTE input */
  int output_stream_bits_from_byte(BYTE input,int length,  FAP_STREAM_MEM *stream );
/* align byte */
  int output_stream_aligned(FAP_STREAM_MEM *stream);
/*output to output file*/
  void output_stream_flush(FILE *fp, FAP_STREAM_MEM *stream); 
  int output_stream_init(FAP_STREAM_MEM *stream);

  int input_stream_init(FAP_STREAM_MEM *stream);
/* get symbol numbers from stream */
  int input_stream_huffman_symbol(HUFFMAN_TABLE *table,
				 FAP_STREAM_MEM *stream );
  unsigned int input_stream_uint(FAP_STREAM_MEM *stream );
  int input_stream_int(FAP_STREAM_MEM *stream );
  unsigned short input_stream_ushort(FAP_STREAM_MEM *stream );
  short input_stream_short(FAP_STREAM_MEM *stream );
  BYTE input_stream_byte(FAP_STREAM_MEM *stream );
  BYTE input_stream_bit(FAP_STREAM_MEM *stream );
  void input_stream_bits(BYTE *data, int length, FAP_STREAM_MEM *stream );
/* input length bits to the last section of BYTE data */
  BYTE input_stream_bits_to_byte(int length, FAP_STREAM_MEM *stream );

/* align input stream (discard several bits if necessary) */
  int input_stream_aligned(FAP_STREAM_MEM *stream);
/* input data from file */
  int input_stream_load(FILE *fp, int *remain_len, FAP_STREAM_MEM *stream); 


/* allocation of 1D or 2D array */
  float *float_array(int len);
  double *double_array(int len);
  int *int_array(int len);

  float **float_2d_array(int row, int col);
  double **double_2d_array(int row, int col);
  int **int_2d_array(int row, int col);

/* DCT initialization and DCT/IDCT */
  void init_dct(int size);
  void dct(float *input, float *output);
  void idct(float *input, float *output);

/* file read manipulaiton */
  void  nextValidLine(FILE *ifp);
  void  finishLine(FILE *ifp);
 
/* read huffman tables */
void assign_dc_ac_runlength_vis_exp_table(HUFFMAN_TABLE  *dc_t, 
					  HUFFMAN_TABLE  *ac_t, 
					  HUFFMAN_TABLE  *rl_t,
					  HUFFMAN_TABLE  *v_s,
					  HUFFMAN_TABLE  *v_b,
					  HUFFMAN_TABLE  *e_s,
					  HUFFMAN_TABLE  *e_i);
void read_a_huffman_table(char *filename, HUFFMAN_TABLE  *table);


#endif
