/********************************************************************/
/* 																	*/
/* This software module was originally developed and edited by		*/
/* Gabriel Antunes Abrantes (Instituto Superior Tcnico)			*/	
/* 																	*/
/* in the course of development of the MPEG-4 standard.	 			*/
/* This software module is an implementation of a part of one or 	*/
/* more MPEG-4 tools as specified by the	 						*/
/* MPEG-4 standard. ISO/IEC gives users of the MPEG-4 standard		*/
/* free license to this software module or modifications thereof 	*/
/* for use in hardware or software products claiming conformance 	*/
/* to the MPEG-4 standard. Those intending to use this software 	*/
/* module in hardware or software products are advised that its 	*/
/* use may infringe existing patents. The original developer of		*/
/* this software module and his company, the subsequent 			*/
/* editors and their companies, and ISO/IEC have no liability		*/
/* for use of this software module or modifications thereof in 		*/
/* an implementation. Copyright is not released for non				*/
/* MPEG-4 standard conforming products. ACTS-MoMuSys partners	 	*/
/* retain full right to use the code for their own purposes, 		*/
/* assign or donate the code to a third party and to inhibit 		*/
/* third parties from using the code for non MPEG-4 standard		*/
/* conforming products. This copyright notice must be included 		*/
/* in all copies or derivative works. 								*/
/* Copyright (c)1997 												*/
/* 																	*/
/********************************************************************/





/***********************************HeaderBegin****************************
*   File : ISTFAPcoder.c                              
*   
*   Author  : Gabriel Antunes Abrantes
*   Created : 23-July-1997 
*
*   Description : Face Object Coder
*
*   Modified:
*   Version 5.0 / October-1998
*   Updated for MPEG-4 Atlantic City Reunion, includes frame based and DCT 
*						based coding modes, and gender bit
*
****************************************HeaderEnd**************************/



#include <stdio.h>
#include <math.h>
#include <stdlib.h>
#include <ctype.h>
#include <string.h>

#define ON  1
#define OFF 0

#define NO_FAPS       68

#define GROUP_MASK_SIZE     10
#define SINGLE_MASK_SIZE    68

/* arithmetic coding */

#define   IrangeMin		  -900
#define   IrangeMax		   900
#define   rangeMin		  -900
#define   rangeMax		   900

#define   q1             16384
#define   q2             32768
#define   q3             49152
#define   top            65535
#define   MAX_FREQUENCY  16383
#define   RANGE_DELTA       60

#define   ZEROMAX           22
#define   ARITH_TYPE		 0
#define   DCT_TYPE			 1


/*DCT coding */

#define SEGMENT_LENGTH  16
#define PI              3.1415926
#define DC_MAX 32767
#define DC_MIN -32767


/* EPF parameters */

#define FBM         "FBM"
#define FFR         "FFR"
#define BFR         "BFR"
#define ISI         "ISI"
#define FRR         "FRR"
#define TC          "TC"
#define SKN         "SKN"
#define FMT         "FMT"
#define FQU         "FQU"
#define FIX         "FIX"
#define FIN         "FIN"
#define FPX         "FPX"
#define FPN         "FPN"



/* face object syntax */

#define FACE_OBJECT_START_CODE        257
#define FACE_OBJECT_END_CODE          258
#define FACE_OBJECT_PLANE_START_CODE  259


#define FAP                  1
#define BAP                  2
#define FBAP                 3

#define FAP_MASK_TYPE0       0
#define FAP_MASK_TYPE1       1
#define FAP_MASK_TYPE2       2
#define FAP_MASK_TYPE3       3


#define MARKER_BIT 1

#define BUFFER_SIZE 256
#define sign(x) ((x>=0)?1:-1)


#define DEBUG    0


int DefaultArithLimits[4*NO_FAPS] = { 
	63,63,1080,600,1860,600,600, 600, 600,1860, 1860, 600, 600,600, 1080, 1080, 
	1080, 420, 1080,1080, 600, 600, 1200, 1200, 900, 900, 600, 600, 420, 420, 
	900, 900, 900, 900, 900,900, 900, 900, 900, 900, 600, 600, 1080, 1080, 1080, 
	1080,300, 1860, 1860, 1860, 600,1860, 600, 600, 600, 600, 1860, 1860, 600, 
	600, 540, 540,680,900,900, 900, 900,900,

	0,0,0,-600,-1860,-600,-600,-600,-600,-1860,-1860,-600,-600,0,-1080,-1080,-1080,
	-420,-1080,-1080,-600,-600,-1200,-1200,-900,-900,-600,-600,-420,-420,-900,-900,
	-900,-900,-900,-900,-900,-900,-900,-900,-600,-600,-1080,-1080,-1080,-1080,0,-1860,
	-1860,-1860,-600,-1860,-600,-600,-600,-600,-1860,-1860,-600,-600,-540,-540,-680,
	-900,-900,-900,-900,-900,

	63,63,360,180,600,180,180,180,
	180,600,600,180,180,180,360,360,360,180,600,600,240,240,420,420,300,300,180,180,
	120,120,360,360,360,360,360,360,300,300,300,300,180,180,420,420,420,420,60,600,
	600,600,180,600,180,180,180,180,600,600,180,180,120,120,180,180,240,240,300,300,
	
	-63,-63,0,-180,-600,-180,
	-180,-180,-180,-600,-600,-180,-180,0,-360,-360,-360,-180,-600,-600,-240,-240,
	-420,-420,-300,-300,-180,-180,-120,-120,-360,-360,-360,-360,-360,-360,-300,
	-300,-300,-300,-180,-180,-420,-420,-420,-420,0,-600,-600,-600,-180,-600,-180,
	-180,-180,-180,-600,-600,-180,-180,-120,-120,-180,-180,-240,-240,-300,-300
};



int FapQuantSteps[NO_FAPS] ={ 1, 1, 4, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 1, 1, 1, 1, 1,
							  1, 1, 1, 1, 128, 128, 128, 128, 8, 8, 1, 1, 2, 2, 
							  2, 2, 2, 2, 1, 1, 2, 2, 2, 2, 1, 1, 1, 1, 512, 170,
							  170, 170, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 1, 1, 1, 1, 
							  1, 1, 1, 1 };



int GroupMaskLimits[11] = 
                   { 0,2,18,30,38,42,47,50,60,64,68 };




float DCTQuantSteps[NO_FAPS]  = { 1, 1, 7.5, 7.5, 7.5, 7.5, 7.5, 7.5, 7.5, 7.5,
								  7.5, 7.5, 7.5,  15, 15, 15, 15, 5, 10,10,
								  10, 10, 425, 425, 425, 425, 5, 5, 5, 5,
								  7.5, 7.5, 7.5, 7.5, 7.5, 7.5, 7.5, 7.5, 20, 20,
								  20, 20, 10, 10, 10, 10, 255, 170, 255, 255,
								  7.5, 7.5, 7.5, 7.5, 7.5, 7.5, 7.5, 7.5, 7.5, 7.5,
								  15, 15, 15, 15, 10, 10, 10, 10 };


int fap_scale[32] = { 1, 1, 2, 3, 5, 7, 8, 10, 12, 15, 18, 21, 25, 30, 35, 42, 
                      50, 60, 72, 87, 105, 128, 156, 191, 234, 288, 355, 439, 543, 
					  674, 836, 1039 };


int quantize(int, int);


/***********************************************************************


  ReadFile - Utilities


***********************************************************************/



int ReadWord(FILE *fp,char *s)
{
  int c;
  
  while(c = getc(fp),!isspace(c)) 
    *s++ = c;
  *s='\0';
  
  return ungetc(c,fp);
}




int PassBlank(FILE *fp) 
{
  int c;
  
  while(c = getc(fp),isspace(c));
  
  return ungetc(c,fp);
}


void TestforTrash(FILE *fp_fap)
{
  char  trash[BUFFER_SIZE];
  
  while(ungetc(getc(fp_fap),fp_fap) == '#') 
    fgets(trash,BUFFER_SIZE,fp_fap);
  
}




int PassToNumber(FILE *fp) 
{
	int c;
	
	while(c = getc(fp),(!isdigit(c)) && (c!=EOF) && (c!='-') );
	
	return ungetc(c,fp);
}


int GetInt(FILE *fp)
{
	int get;

	PassToNumber(fp);
	fscanf(fp,"%d", &get);
	return get;
}




/******************************************************************
					  
						
						  - Memory allocation
						  
							
******************************************************************/

float **FloatMatrixAllocation(int vs,int hs)
{
	float **new_matrix;
	int i;
	
	new_matrix=(float **) calloc(vs,sizeof(float*));
	if ( (new_matrix==NULL) ) {
		printf("Out of Memory \n");
		exit(1);
	}
	
	for(i=0;i<vs;i++) {
		new_matrix[i]=(float *) calloc(hs,sizeof(float));
		if ( (new_matrix[i]==NULL)  ){
			printf("Out of Memory \n");
			exit(1);
		}
	}
	
	return new_matrix;
}



void MatrixDealoc(float **matrix,int vs)
{
	int i;
	
	for(i=0;i<vs;i++) 
		free(matrix[i]);
	
	free(matrix);
}




int **IntMatrixAllocation(int vs,int hs)
{
	int **new_matrix;
	int i;
	
	new_matrix=(int **) calloc(vs,sizeof(int*));
	if ( (new_matrix==NULL) ) {
		printf("Out of Memory \n");
		exit(1);
	}
	
	for(i=0;i<vs;i++) {
		new_matrix[i]=(int *) calloc(hs,sizeof(int));
		if ( (new_matrix[i]==NULL)  ){
			printf("Out of Memory \n");
			exit(1);
		}
	}
	
	return new_matrix;
}




void IntMatrixDealoc(int **matrix,int vs)
{
	int i;
	
	for(i=0;i<vs;i++) 
		free(matrix[i]);
	
	free(matrix);
}



/***********************************************************************

   Put bits in stream

***********************************************************************/

static int bitsout,bitcount=0;
static int framebits;

unsigned char byte;

void PutBits (int nobits, int value, FILE *fap_stream)
{
	int counter;
	unsigned int mask;
	int bit;
	
	bitcount+=nobits;
	framebits+=nobits;
	
	mask = (int)pow(2,nobits-1); 
	
	for (counter=0; counter<nobits; counter++) {
		
		bitsout--;
		byte <<= 1;
		
		bit = mask&value;
		bit>>=(nobits-1-counter);
		
		byte|=bit;
		
		mask >>= 1; 
		
		if (bitsout==0)  {
			putc(byte,fap_stream);
			bitsout = 8;
		}
		
	}
}


void ShortPutBits (short symbol, FILE *fap_stream)
{
	int counter;
	unsigned int mask,value;
	int bit;
	
	bitcount+=16;
	framebits+=16;
	
	value = (unsigned int)symbol;
	
	mask = (int)pow(2,15); 
	
	for (counter=0; counter<16; counter++) {
		
		bitsout--;
		byte <<= 1;
		
		bit = mask&value;
		bit>>=(15-counter);
		
		byte|=bit;
		
		mask >>= 1; 
		
		if (bitsout==0)  {
			putc(byte,fap_stream);
			bitsout = 8;
		}
		
	}
}



/***********************************************************************


  Huffman-Tables file reading


***********************************************************************/




void ReadHuffmanTables(int **run_table,int **dc_table,int **ac_table)
{
	FILE *fp_table;
	char word[BUFFER_SIZE];
	int counter,bitcounter;

	sprintf(word,"run.table");
	if( (fp_table  = fopen(word,"rb"))==NULL ) {
		printf("\n File - %s - does not exist. \n",word);
		exit(0);
	}

	for(counter=0;counter<16;counter++) {
		fscanf(fp_table,"%d",&run_table[counter][0]);
		
		for(bitcounter=1;bitcounter<=run_table[counter][0];bitcounter++) 
			fscanf(fp_table,"%d",&run_table[counter][bitcounter]);
		fgets(word,BUFFER_SIZE,fp_table);
	}

	fclose(fp_table);


	sprintf(word,"dc.table");
	if( (fp_table  = fopen(word,"rb"))==NULL ) {
		printf("\n File - %s - does not exist. \n",word);
		exit(0);
	}

	for(counter=0;counter<512;counter++) {
		fscanf(fp_table,"%d",&dc_table[counter][0]);
		
		for(bitcounter=1;bitcounter<=dc_table[counter][0];bitcounter++) 
			fscanf(fp_table,"%d",&dc_table[counter][bitcounter]);
		fgets(word,BUFFER_SIZE,fp_table);
	}

	fclose(fp_table);


	sprintf(word,"ac.table");
	if( (fp_table  = fopen(word,"rb"))==NULL ) {
		printf("\n File - %s - does not exist. \n",word);
		exit(0);
	}

	for(counter=0;counter<512;counter++) {
		fscanf(fp_table,"%d",&ac_table[counter][0]);
		
		for(bitcounter=1;bitcounter<=ac_table[counter][0];bitcounter++) 
			fscanf(fp_table,"%d",&ac_table[counter][bitcounter]);
		fgets(word,BUFFER_SIZE,fp_table);
	}

	

	fclose(fp_table);
}




/***********************************************************************


  DCT Coder 


***********************************************************************/



void MakeDCTMatrix(float **dct)
{
	float dct_coef;
	int   line,colunm;
	
	for(line=0; line<SEGMENT_LENGTH; line++) {
		
		if(line==0) 
			dct_coef = (float)(1.0/sqrt(2));
		else dct_coef = 1;
		
		for(colunm=0; colunm<SEGMENT_LENGTH; colunm++)
			dct[line][colunm] = (float)(sqrt(2.0/SEGMENT_LENGTH))*dct_coef*
							(float)(cos(PI*(float)(2*colunm+1)*line/2.0/SEGMENT_LENGTH));
		
	}

}



void Reset(float **faps, int **fap_expression,int *mask_type)
{
	int line,colunm;


	for(line=0;line<NO_FAPS;line++) {
		for(colunm=0;colunm<SEGMENT_LENGTH;colunm++)
			faps[line][colunm]=0;
	}

	for(line=0;line<SEGMENT_LENGTH;line++) {
		for(colunm=0;colunm<6;colunm++)
			fap_expression[line][colunm]=0;
	}


	mask_type[0] = ON;
	for(line=1;line<10;line++)
		mask_type[line]=ON;

}



/***********************************************************************
*
*	The modules Ar_encode(), encoder_flush(), bit_in_psc_layer(), 
*   bit_opp_bits(), are described in the MPEG4 FCD-Visual text 
*	and were modified for this module by Gabriel Antunes Abrantes
*
*  These modules are part of
*  sac.c, part of tmn (TMN encoder) 
*  Copyright (C) 1996  Telenor R&D, Norway
*        Karl Olav Lillevold <Karl.Lillevold@nta.no>
*
*  The program(sac.c) is free software; you can redistribute it and/or modify
*  it under the terms of the GNU General Public License as published by
*  the Free Software Foundation; either version 2 of the License, or
*  (at your option) any later version.
*
*  The program(sac.c) is distributed in the hope that it will be useful,
*  but WITHOUT ANY WARRANTY; without even the implied warranty of
*  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
*  GNU General Public License for more details.
*
*  You should have received a copy of the GNU General Public License
*  along with this program; if not, write to the Free Software
*  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*
*  Karl Olav Lillevold               <Karl.Lillevold@nta.no>
*  Telenor Research and Development
*  P.O.Box 83                        tel.:   +47 63 84 84 00
*  N-2007 Kjeller, Norway            fax.:   +47 63 81 00 76
*
*  Robert Danielsen                  e-mail: Robert.Danielsen@nta.no
*  Telenor Research and Development  www:    http://www.nta.no/brukere/DVC/
*  P.O.Box 83                        tel.:   +47 63 84 84 00
*  N-2007 Kjeller, Norway            fax.:   +47 63 81 00 76
*
*
* SAC Encoder Module
* Algorithm as specified in H263 (Annex E)
*         (c) BT Labs 1995
*
* Author: Pat Mulroy <pmulroy@visual.bt.co.uk>
*
***********************************************************************/



static long   low=0, high=top, opposite_bits=0, length=0;

int zerocount = 0;

void bit_in_psc_layer(int bit, FILE *fap_stream)
{
	if(zerocount==ZEROMAX) {
		PutBits(1,MARKER_BIT,fap_stream);
		zerocount=0;
	}
	
	PutBits(1, bit, fap_stream);
	
	if(!bit) zerocount++;
	else zerocount = 0;
}



void bit_opp_bits(int bit, FILE *fap_stream) 
{                                   
  bit_in_psc_layer(bit,fap_stream);

  while(opposite_bits > 0){
    bit_in_psc_layer(!bit,fap_stream);
    opposite_bits--;
  }

}



void update(int symbol, int index, int is_ifap, int *Irange_max,
	    int *Irange_min, int **Icumul_freq, int **Ifreq, 
	    int *range_max, int *range_min, int **cumul_freq, int **freq) 
{
  int n;
  int *cumul_tmp, *freq_tmp;

  if(is_ifap) {
    n = Irange_max[index]-Irange_min[index]+1;
    cumul_tmp = Icumul_freq[index];
    freq_tmp = Ifreq[index];
  } 
  else {
    n = range_max[index]-range_min[index]+1;
    cumul_tmp = cumul_freq[index];
    freq_tmp = freq[index];
  }

  if (cumul_tmp[0] == MAX_FREQUENCY) {
    int i, cum;
    cum = 0;

    for(i=n-1; i>=0; i--) {
      freq_tmp[i] = (freq_tmp[i]+1)/2;
      cum += freq_tmp[i];
      cumul_tmp[i] = cum;
    }
    cumul_tmp[n] = 0;
  }

  freq_tmp[symbol] += 1;
  cumul_tmp[symbol] += 1;
  while(symbol>0) {
    symbol -= 1;
    cumul_tmp[symbol] += 1;
  }
}



void update_viseme(int symbol, int index,int **Vis_cumul_freq, int **Vis_freq,int scale) 
{
  int n;
  int *cumul_tmp, *freq_tmp;
//##
	if( (index==0) || (index==1) )
		n = 14+14+1;
	else n = 2*quantize(63,scale)+1;
    
	cumul_tmp = Vis_cumul_freq[index];
    freq_tmp = Vis_freq[index];

  if (cumul_tmp[0] == MAX_FREQUENCY) {
    int i, cum;
    cum = 0;

    for(i=n-1; i>=0; i--) {
      freq_tmp[i] = (freq_tmp[i]+1)/2;
      cum += freq_tmp[i];
      cumul_tmp[i] = cum;
    }
    cumul_tmp[n] = 0;
  }

  freq_tmp[symbol] += 1;
  cumul_tmp[symbol] += 1;

  while(symbol>0) {
    symbol -= 1;
    cumul_tmp[symbol] += 1;
  }

}



void update_expression(int symbol, int index,int **Exp_cumul_freq, int **Exp_freq,int scale) 
{
  int n;
  int *cumul_tmp, *freq_tmp;

	if( (index==0) || (index==2) )
		n = 6+6+1;
	else n = 2*quantize(63,scale)+1;
    
	cumul_tmp = Exp_cumul_freq[index];
    freq_tmp = Exp_freq[index];

  if (cumul_tmp[0] == MAX_FREQUENCY) {
    int i, cum;
    cum = 0;

    for(i=n-1; i>=0; i--) {
      freq_tmp[i] = (freq_tmp[i]+1)/2;
      cum += freq_tmp[i];
      cumul_tmp[i] = cum;
    }
    cumul_tmp[n] = 0;
  }

  freq_tmp[symbol] += 1;
  cumul_tmp[symbol] += 1;
 
  while(symbol>0) {
    symbol -= 1;
    cumul_tmp[symbol] += 1;
  }

}



void AR_Encode(int index, int cumul_freq[ ], FILE *fap_stream)
{
  
  if (index<0) {
    printf("\n Error ARcoding, symbol is negative");
    exit(-1);
  }
  

  length = high - low + 1;
  high = low - 1 + (length * cumul_freq[index]) / cumul_freq[0];
  low += (length * cumul_freq[index+1]) / cumul_freq[0];

  for ( ; ; ) {
    if (high < q2) {
      bit_opp_bits(0,fap_stream);
    }
    else if (low >= q2) {
      bit_opp_bits(1,fap_stream);	
      low -= q2; 
      high -= q2;
    }
    else if (low >= q1 && high < q3) {
      opposite_bits += 1; 
      low -= q1; 
      high -= q1;
    }
    else break;
 
    low *= 2; 
    high = 2*high+1;
  }

}




void init_arith(int *Irange_max,int *Irange_min, 
				int **Icumul_freq, int **Ifreq,  int *range_max, 
				int *range_min, int **cumul_freq, int **freq,
				int	**Exp_freq,int **Exp_cumul_freq,int **IExp_cumul_freq,
				int	**Vis_freq,int **Vis_cumul_freq,int **IVis_cumul_freq,
				int *fap_mask,int scale)
{
	
	int i, j, n;

	if(fap_mask[0]) {
		// visemes values fields 0&1
		for(i=0;i<2;i++) {
			
			free(IVis_cumul_freq[i]);
//##
			n = 14+1;
			IVis_cumul_freq[i] = (int *) calloc(n+1,sizeof(int));
			
			for(j=0; j<n; j++) 
				IVis_cumul_freq[i][j] = n-j;
			
			IVis_cumul_freq[i][n] = 0;			
		}
		
		for(i=0;i<2;i++) {
			
			free(Vis_freq[i]);
			free(Vis_cumul_freq[i]);
//##			
			n = 2*14+1;
			Vis_freq[i] = (int *) calloc(n,sizeof(int));
			Vis_cumul_freq[i] = (int *) calloc(n+1,sizeof(int));
			
			for(j=0; j<n; j++) {
				Vis_freq[i][j] = 1;
				Vis_cumul_freq[i][j] = n-j;
			}
			
			Vis_cumul_freq[i][n] = 0;
		}
		
		
		// viseme blend field
		
		free(IVis_cumul_freq[2]);
		
		n = quantize(63,scale)+1;
		IVis_cumul_freq[2] = (int *) calloc(n+1,sizeof(int));
		
		for(j=0; j<n; j++) 
			IVis_cumul_freq[2][j] = n-j;
		
		IVis_cumul_freq[2][n] = 0;
		
		
		free(Vis_freq[2]);
		free(Vis_cumul_freq[2]);
		
		n = 2*quantize(63,scale)+1;
		Vis_freq[2] = (int *) calloc(n,sizeof(int));
		Vis_cumul_freq[2] = (int *) calloc(n+1,sizeof(int));
		
		for(j=0; j<n; j++) {
			Vis_freq[2][j] = 1;
			Vis_cumul_freq[2][j] = n-j;
		}
		
		Vis_cumul_freq[2][n] = 0;

	}


	if(fap_mask[1]) {
		// expression values fields 0&2
		for(i=0;i<3;i+=2) {
			
			free(IExp_cumul_freq[i]);
			
			n = 6+1;
			IExp_cumul_freq[i] = (int *) calloc(n+1,sizeof(int));
			
			for(j=0; j<n; j++) 
				IExp_cumul_freq[i][j] = n-j;
			
			IExp_cumul_freq[i][n] = 0;			
		}
		
		for(i=0;i<3;i+=2) {
			
			free(Exp_freq[i]);
			free(Exp_cumul_freq[i]);
			
			n = 2*6+1;
			Exp_freq[i] = (int *) calloc(n,sizeof(int));
			Exp_cumul_freq[i] = (int *) calloc(n+1,sizeof(int));
			
			for(j=0; j<n; j++) {
				Exp_freq[i][j] = 1;
				Exp_cumul_freq[i][j] = n-j;
			}
			
			Exp_cumul_freq[i][n] = 0;
		}
		
		
		// expressions intensities fields 1&3
		for(i=1;i<4;i+=2) {
			
			free(IExp_cumul_freq[i]);
			
			n = quantize(63,scale)+1;
			IExp_cumul_freq[i] = (int *) calloc(n+1,sizeof(int));
			
			for(j=0; j<n; j++) 
				IExp_cumul_freq[i][j] = n-j;
			
			IExp_cumul_freq[i][n] = 0;
		}
		
		for(i=1;i<4;i+=2) {
			
			free(Exp_freq[i]);
			free(Exp_cumul_freq[i]);
			
			n = 2*quantize(63,scale)+1;
			Exp_freq[i] = (int *) calloc(n,sizeof(int));
			Exp_cumul_freq[i] = (int *) calloc(n+1,sizeof(int));
			
			for(j=0; j<n; j++) {
				Exp_freq[i][j] = 1;
				Exp_cumul_freq[i][j] = n-j;
			}
			
			Exp_cumul_freq[i][n] = 0;
		}
	}

	// rest of faps
	
	for(i=2; i<NO_FAPS; i++) if(fap_mask[i]){
		
		free(Ifreq[i]);
		free(Icumul_freq[i]);
		n = Irange_max[i]-Irange_min[i]+1;
		Ifreq[i] = (int *) malloc(n*sizeof(int));
		Icumul_freq[i] = (int *) malloc((n+1)*sizeof(int));
		
		for(j=0; j<n; j++) {
			Ifreq[i][j] = 1;
			Icumul_freq[i][j] = n-j;
		}
		
		Icumul_freq[i][n] = 0;
	}
	
	for(i=2; i<NO_FAPS; i++) if(fap_mask[i]) {
		
		free(freq[i]);
		free(cumul_freq[i]);
		n = range_max[i]-range_min[i]+1;
		freq[i] = (int *) malloc(n*sizeof(int));
		cumul_freq[i] = (int *) malloc((n+1)*sizeof(int));
		
		for(j=0; j<n; j++) {
			freq[i][j] = 1;
			cumul_freq[i][j] = n-j;
		}
		
		cumul_freq[i][n] = 0;
	}
}




void encoder_flush(FILE *fap_stream)
{
  opposite_bits++;

  if (low < q1) {
    bit_opp_bits(0,fap_stream);
  }
  else {
    bit_opp_bits(1,fap_stream);
  }
  low = 0; 
  high = top;
  zerocount = 0;
}



void Check_Symbol(int *input_symbol, int fap, int is_intra,int *Irange_max,
				  int *Irange_min,int *range_max,int *range_min)
{
	// if a symbol is out of range, it is clipped

	int symbol = *input_symbol;
	
	if(is_intra)  {
		symbol += Irange_min[fap];
		if(symbol < Irange_min[fap] || symbol > Irange_max[fap]) {
			printf("\n Symbol out of range: IFAP%d : %d\n",fap+1,symbol*FapQuantSteps[fap]);
			printf(" Change Irange value to %d\n", (int)(symbol/60+1)*60 );

			if(symbol < Irange_min[fap]) *input_symbol = Irange_min[fap];
			else *input_symbol = Irange_max[fap];
			printf(" Changing value of FAP %d to %d\n",fap+1,*input_symbol);
			*input_symbol -= Irange_min[fap];
		}
	}
	else {
		symbol += range_min[fap];
		if(symbol < range_min[fap] || symbol > range_max[fap]) {
			printf("\n Symbol out of range: PFAP%d : %d\n",fap+1,symbol*FapQuantSteps[fap]);
			printf(" Change range value to %d\n",(int)(symbol/60+1)*60 );

			if(symbol < range_min[fap]) *input_symbol = range_min[fap];
			else *input_symbol = range_max[fap];
			printf(" Changing value of FAP %d to %d\n",fap+1,*input_symbol);
			*input_symbol -= range_min[fap];
		}
	}
}




/***********************************************************************


  Arith FAP & EPF file reading


***********************************************************************/


int Read1Parameter(FILE *fp_epf,char *par)
{
  char  word[BUFFER_SIZE];
  int   number;
  
  TestforTrash(fp_epf);    

  ReadWord(fp_epf,word);
  if(strcmp(word,par)!=0) {
    printf("\n EPF Reading Error, %s expected,not '%s'.",par,word);
    exit(-1);
  }
  else {
    fscanf(fp_epf, "%d", &number);
    fgets(word,BUFFER_SIZE,fp_epf);
  }

  return number;
}




void MakeFapMask(int group,int *mask_type,int *fap_mask,FILE *fp_epf)
{
  int counter;
    
    if(mask_type[group] == FAP_MASK_TYPE3)
      for(counter=GroupMaskLimits[group];counter<GroupMaskLimits[group+1];
	  counter++) 
	fap_mask[counter] = 1;
    
    if(mask_type[group] == FAP_MASK_TYPE1 ||
       mask_type[group] == FAP_MASK_TYPE2)
      for(counter=GroupMaskLimits[group];counter<GroupMaskLimits[group+1];
	  counter++) 
	fscanf(fp_epf,"%d",&fap_mask[counter]);  

}




void ReadNewRangeValues(int *range,int *newrange,FILE *fp_epf)
{
  int counter;
  int visex[3];
  
  fscanf(fp_epf,"%d",&visex[0]);
  fscanf(fp_epf,"%d",&visex[1]);
  fscanf(fp_epf,"%d",&visex[2]);

  for(counter=2;counter<NO_FAPS;counter++){
    fscanf(fp_epf,"%d",&range[counter]);
    newrange[counter]=(int)fabs(range[counter])/RANGE_DELTA;
	if(newrange[counter]>31) {
		printf("\nError for range values of FAP%d: %d, resetting to 31",counter+1,
			newrange[counter]);
		newrange[counter] = 31;
	}
	else if(newrange[counter]==0) {
		printf("\nError for range values of FAP%d: %d, resetting to 1",counter+1,
			newrange[counter]);
		newrange[counter] = 1;
	}
  }
  
}




void Read1FrameEPF(int frame_counter, int *fba_paramset_mask, int *is_intra,
				   int *is_frame_rate, int *frame_rate, int *fr_seconds,
				   int *fr_offset, int *is_time_code, int *tc_hours,
				   int *tc_minutes, int *tc_mbit, int *tc_seconds,
				   int *skip_frames, int *no_frames_skip, int *mask_type,
				   int *fap_mask, int *fap_quant, int *is_I_new_max, 
				   int *is_I_new_min, int *is_P_new_max, int *is_P_new_min,
				   int *Irange_max, int *Irange_min, int *range_max, 
				   int *range_min, int *i_new_max, int *i_new_min,
				   int *p_new_max, int *p_new_min, int Nframe,FILE *fp_epf)
{
	char   word[BUFFER_SIZE];
	int    test,counter;
	long   start;
	
	TestforTrash(fp_epf);
	
	*fba_paramset_mask = Read1Parameter(fp_epf,FBM);
	
	if(Read1Parameter(fp_epf,FFR) != frame_counter) {
		printf("\n EPF Reading Error, different frame-numbers.");
		exit(-1);
	}    
	
	TestforTrash(fp_epf);
	
	*is_intra          = Read1Parameter(fp_epf,ISI);
	
	if(DEBUG)
		printf("\nEPF readout:\n- FBM: %d\n- ISI: %d\n",*fba_paramset_mask,
		*is_intra);
	
	
	if(*is_intra) {
		
		test = ON;
		*is_frame_rate = OFF;
		*is_time_code  = OFF;
		*skip_frames   = OFF;
		*is_I_new_max      = OFF;
		*is_I_new_min      = OFF;
		*is_P_new_max      = OFF;
		*is_P_new_min      = OFF;
		
		do {
			
			TestforTrash(fp_epf);
			
			start = ftell(fp_epf);
			
			ReadWord(fp_epf,word);    
			
			if(strcmp(word,FRR)==0) { 
				*is_frame_rate = ON;
				fscanf(fp_epf, "%d %d %d",frame_rate,fr_seconds,fr_offset );
				if(DEBUG)
					printf("- FRR : %d %d %d \n",*frame_rate,*fr_seconds,*fr_offset);
				fgets(word,BUFFER_SIZE,fp_epf);
			} 
			
			else if(strcmp(word,TC) ==0) {
				*is_time_code  = ON;
				fscanf(fp_epf, "%d %d %d %d",tc_hours,tc_minutes,tc_mbit,
					tc_seconds);
				if(DEBUG)
					printf("- TC  : %d %d %d %d \n",*tc_hours,*tc_minutes,*tc_mbit,
					*tc_seconds);
				fgets(word,BUFFER_SIZE,fp_epf);
			}
			
			else if(strcmp(word,SKN)==0) {
				*skip_frames   = ON;
				fscanf(fp_epf, "%d",no_frames_skip);
				if(DEBUG)
					printf("- SKN : %d \n",*no_frames_skip);
				fgets(word,BUFFER_SIZE,fp_epf);
			}
			
			else if(strcmp(word,FMT)==0) {
				
				for(counter=0;counter<NO_FAPS;counter++) 
					fap_mask[counter]=0;
				
				if(DEBUG)
					printf("- FMT :");
				for(counter=0;counter<GROUP_MASK_SIZE;counter++) {
					
					fscanf(fp_epf, "%d",&mask_type[counter]);  
					
					if(DEBUG)	 
						printf(" %d",mask_type[counter]);
					
					MakeFapMask(counter,mask_type,fap_mask,fp_epf);
					
				}
				if(DEBUG)
					printf("\n");
				fgets(word,BUFFER_SIZE,fp_epf);
			}
			
			else if(strcmp(word,FQU)==0) {
				fscanf(fp_epf, "%d", fap_quant);
				if(DEBUG)
					printf("- FQU : %d \n",*fap_quant);
				fgets(word,BUFFER_SIZE,fp_epf);	
			}
			
			else if(strcmp(word,FIX)==0) {
				if(DEBUG)
					printf("- FIX ON\n");
				*is_I_new_max      = ON;
				ReadNewRangeValues(Irange_max,i_new_max,fp_epf);
				fgets(word,BUFFER_SIZE,fp_epf);
			}
			
			else if(strcmp(word,FIN)==0) {
				if(DEBUG)
					printf("- FIN ON\n");
				*is_I_new_min      = ON;
				ReadNewRangeValues(Irange_min,i_new_min,fp_epf);
				fgets(word,BUFFER_SIZE,fp_epf);
			}
			
			else if(strcmp(word,FPX)==0) {
				if(DEBUG)
					printf("- FPX ON\n");
				*is_P_new_max      = ON;
				ReadNewRangeValues(range_max,p_new_max,fp_epf);
				fgets(word,BUFFER_SIZE,fp_epf);
			}
			
			else if(strcmp(word,FPN)==0) {
				if(DEBUG)
					printf("- FPN ON\n");
				*is_P_new_min      = ON;
				ReadNewRangeValues(range_min,p_new_min,fp_epf);
				fgets(word,BUFFER_SIZE,fp_epf);
				test=OFF;
			}
			
			else {
				test = OFF;
				fseek(fp_epf,start,SEEK_SET);
			}
			
    }
    while(test);
  }
  else if( (*is_intra==OFF) && (frame_counter!=Nframe-1) ) {
		
	  start = ftell(fp_epf);
	  TestforTrash(fp_epf);
	  
	  ReadWord(fp_epf,word);    
	  
	  if(strcmp(word,SKN)==0) {
		  *skip_frames   = ON;
		  fscanf(fp_epf, "%d",no_frames_skip);
		  if(DEBUG)
			  printf("- SKN : %d \n",*no_frames_skip);
		  fgets(word,BUFFER_SIZE,fp_epf);
	  }
	  else fseek(fp_epf,start,SEEK_SET);
  }
}



void ReadFAPInfo(int *Nframe,FILE *fp_fap)
{
  char  word[BUFFER_SIZE];
  float fr;
  
  TestforTrash(fp_fap);
  ReadWord(fp_fap,word);
  if(DEBUG)
    printf("\n## Version %s",word);
  PassBlank(fp_fap);
  ReadWord(fp_fap,word);
  if(DEBUG)
    printf(" of sequence '%s' has",word);
  fscanf(fp_fap,"%f",&fr);
  if(DEBUG)
    printf(" %f frames per second,",fr);
  fscanf(fp_fap,"%d",Nframe);
  if(DEBUG)
    printf("\n## and a total of %d frames. \n ",*Nframe);
  fgets(word,BUFFER_SIZE,fp_fap);
  
}



void Read1FrameFAP(int frame_counter,int *fap_mask,float *faps,int *fap_expr,
				   int *fap_vis,FILE *fp_fap)
{
	int   counter,frame_number;
	float fap;
	char  trash[BUFFER_SIZE];
	int   FAPfile_mask[NO_FAPS];	
	
	/* read mask and FAPs from fapfile version 2.1*/
    
	TestforTrash(fp_fap);
	
	for(counter=0;counter<NO_FAPS;counter++) 
		fscanf(fp_fap,"%d",&FAPfile_mask[counter]);
	fgets(trash,BUFFER_SIZE,fp_fap);
	
	/* read frame number */
	
	TestforTrash(fp_fap);    
	fscanf(fp_fap,"%d",&frame_number);
	
	if(frame_number != frame_counter) {
		printf("\n FAP-file Reading Error, different frame-numbers.");
		exit(-1);
	}    

	if(fap_mask[0]) {
		for(counter=0;counter<4;counter++)
			fap_vis[counter] = GetInt(fp_fap);	
	}

	if(fap_mask[1]) {
		for(counter=0;counter<6;counter++)
			fap_expr[counter] = GetInt(fp_fap);	
	}


	/* read faps */
	
	for(counter=2;counter<NO_FAPS;counter++) {
		if(FAPfile_mask[counter])
			fscanf(fp_fap,"%f",&fap);
		if(fap_mask[counter])
			faps[counter] = fap;
	}
		
	fgets(trash,BUFFER_SIZE,fp_fap);
		
}




/***********************************************************************


  DCT FAP file reading


***********************************************************************/





void ReadDCTFAPInfo(int *Nframe,FILE *fp_fap,int *frt)
{
	char  word[BUFFER_SIZE];
	float fr;
	
	TestforTrash(fp_fap);
	ReadWord(fp_fap,word);
	if(DEBUG)
		printf("\n## Version %s",word);
	PassBlank(fp_fap);
	ReadWord(fp_fap,word);
	if(DEBUG)
		printf(" of sequence '%s' has",word);
	fscanf(fp_fap,"%f",&fr);
	*frt = (int)fr;
	if(DEBUG)
		printf(" %f frames per second,",fr);
	fscanf(fp_fap,"%d",Nframe);
	if(DEBUG)
		printf("\n## and a total of %d frames. \n ",*Nframe);
	fgets(word,BUFFER_SIZE,fp_fap);
	
}



void Read1DCTFrameFAP(int *fap_mask,float *fap,int **fap_expression,int **fap_visemes,
					  int frame,FILE *fp_fap)
{
	int counter,frame_number;
	char  trash[BUFFER_SIZE];
	
	for(counter=0;counter<NO_FAPS;counter++) 
		fap[counter]=0;
	

	/* read mask and FAPs from fapfile version 2.0*/
    
	TestforTrash(fp_fap);
	
	for(counter=0;counter<NO_FAPS;counter++) 
		fscanf(fp_fap,"%d",&fap_mask[counter]);
	fgets(trash,BUFFER_SIZE,fp_fap);
	
	// obtain fapmask & fap_group_mask
	
	
	/* read frame number */
	
	TestforTrash(fp_fap);    
	fscanf(fp_fap,"%d",&frame_number);

	if(fap_mask[0]) {
		for(counter=0;counter<4;counter++)
			fap_visemes[frame][counter] = GetInt(fp_fap);	
	}
    

	if(fap_mask[1]) {
		for(counter=0;counter<6;counter++)
			fap_expression[frame][counter] = GetInt(fp_fap);	
	}
		
	/* read faps */
	
	for(counter=2;counter<NO_FAPS;counter++)
		if(fap_mask[counter]==ON)
			fscanf(fp_fap,"%f",&fap[counter]);
		fgets(trash,BUFFER_SIZE,fp_fap);
		
}


void PutInFaps(float *fap,float **faps,int frame)
{
	int counter;

	for(counter=0;counter<NO_FAPS;counter++)
		faps[counter][frame]=fap[counter];

}



int CheckFapMask(int *masknew,int *maskold,float *fap,int first)
{
	int i,test = OFF;
	
	if(first) {
		for(i=0;i<NO_FAPS;i++) {
			if(masknew[i]!=maskold[i]) {
				test = ON;
				maskold[i] = masknew[i];
			}	
		}
		return test;
	}


	for(i=0;i<NO_FAPS;i++) {
		if(masknew[i]!=maskold[i]) {
			test = ON;
			if(masknew[i])
				maskold[i] = masknew[i];
		}
	}

	return test;
}



void MakeGroupMask(int *fap_mask,int *mask_type)
{
	int i,j;

	for(i=0;i<GROUP_MASK_SIZE;i++) {
		mask_type[i] = OFF;
		for(j=GroupMaskLimits[i];j<GroupMaskLimits[i+1];j++)
			if(fap_mask[j]) mask_type[i] = FAP_MASK_TYPE2;
	}

	if(mask_type[1] && !mask_type[7])
		mask_type[7] = FAP_MASK_TYPE2;

	if(mask_type[7] && !mask_type[1])
		mask_type[1] = FAP_MASK_TYPE2;

}



void Read1Segment(int size,int *fap_mask,int *mask_type,float **faps,int **fap_expression,
				  int **fap_visemes,int *is_intra,FILE *fp_fap)
{
	int   frame_counter,i;
	int   *trash,*mask_old;
	float *fap;

	if(size==0) 
		return;

	trash = (int *)calloc(NO_FAPS,sizeof(int));
	fap   = (float *)calloc(NO_FAPS,sizeof(float));

	mask_old = (int *)calloc(NO_FAPS,sizeof(int));

	for(i=0;i<NO_FAPS;i++)
		mask_old[i] = fap_mask[i];

	Read1DCTFrameFAP(fap_mask,fap,fap_expression,fap_visemes,0,fp_fap);
	PutInFaps(fap,faps,0);
	if(CheckFapMask(fap_mask,mask_old,fap,ON)) *is_intra = ON;
	
	for(frame_counter=1;frame_counter<size;frame_counter++) {
		
		Read1DCTFrameFAP(fap_mask,fap,fap_expression,fap_visemes,frame_counter,fp_fap);
		PutInFaps(fap,faps,frame_counter);
		if(CheckFapMask(fap_mask,mask_old,fap,OFF)) *is_intra = ON;
	
	}

	for(i=0;i<NO_FAPS;i++)
		fap_mask[i] = mask_old[i];

	if(*is_intra) MakeGroupMask(fap_mask,mask_type);


	free(trash);
	free(fap);
	free(mask_old);
}







/***********************************************************************


  Common parts of Encoder


***********************************************************************/



void  Next_Start_Code(FILE *fap_stream)
{

  if(bitsout!=8){
    PutBits(1,0,fap_stream);
    while(bitsout!=8)
      PutBits(1,1,fap_stream);
  }
  else 
    PutBits(8,127,fap_stream);
}




void Put_Fba_Paramset_Mask(int fpm,FILE *fap_stream)
{
  if(fpm==OFF)
    PutBits(2,0,fap_stream);
  else if(fpm==FAP)
    PutBits(2,1,fap_stream);
  else if(fpm==BAP)
    PutBits(2,2,fap_stream);
  else if(fpm==FBAP)
    PutBits(2,3,fap_stream);
}



void  Code_Frame_Rate(int frame_rate,int seconds,int frequency_offset,
		      FILE *fap_stream)
{
  
  /* frame_rate */

  if(frame_rate>=0 && frame_rate<256)
    PutBits(8,frame_rate,fap_stream);
  else {
    printf("\n Error: Frame_rate range from 0 to 255 \n");
    exit(-1);
  }

  /* seconds */

  seconds = 0;
  if(seconds>=0 && seconds<16)
    PutBits(4,seconds,fap_stream);
  else {
    printf("\n Error: Seconds(framerate) range from 0 to 15 \n");
    exit(-1);
  }

  /* frequency_offset */

  frequency_offset = OFF;
  PutBits(1,frequency_offset,fap_stream);
}





void  Code_Time_Code(FILE *fap_stream,int time_code_hours,
		     int time_code_minutes,int marker_bit,
		     int time_code_seconds)
{
  
  if(time_code_hours>=0 && time_code_hours<24)
    PutBits(5,time_code_hours,fap_stream);
  else {
    printf("\n Error: Hours(timecode) range from 0 to 23 \n");
    exit(-1);
  }

  
  if(time_code_minutes>=0 && time_code_minutes<60)
  PutBits(6,time_code_minutes,fap_stream);
  else {
    printf("\n Error: Minutes(timecode) range from 0 to 59 \n");
    exit(-1);
  }

  
  PutBits(1,marker_bit,fap_stream);

  
  if(time_code_seconds>=0 && time_code_seconds<60)
  PutBits(6,time_code_seconds,fap_stream);
  else {
    printf("\n Error: Seconds(timecode) range from 0 to 59 \n");
    exit(-1);
  }

}




void Code_Skip_Frames(int no_frames_skip,FILE *fap_stream)
{
  int nfs;
  
  if(no_frames_skip>254) {
    printf("\n Maximum allowable frames skip is 254.\n");
    exit(-1);
  }

  if(no_frames_skip > 14) {
    PutBits(4,15,fap_stream);
    nfs = no_frames_skip - 15;
  }
  else {
    nfs = no_frames_skip;
    PutBits(4,nfs,fap_stream);
    return;
  }

  if(nfs<15) {
      PutBits(4,nfs,fap_stream);
      return;
  }
  
  do {
    PutBits(4,15,fap_stream);
    nfs-=15;
  }
  while (nfs > 14);
  
  nfs+=16;
  PutBits(4,nfs,fap_stream);
}






void Temporal_Header(int is_intra,int is_frame_rate,int frame_rate,int fr_seconds,int fr_offset,
					 int is_time_code,int tc_hours,int tc_minutes,int tc_mbit,
					 int tc_seconds,int skip_frames,int no_frames_skip,FILE *fap_stream)
{
		if(is_intra) {
			PutBits(1,is_frame_rate,fap_stream);
			if (is_frame_rate) 
				Code_Frame_Rate(frame_rate,fr_seconds,fr_offset,fap_stream);
			
			PutBits(1,is_time_code,fap_stream);
			if (is_time_code) 
				Code_Time_Code(fap_stream,tc_hours,tc_minutes,tc_mbit,tc_seconds);
		}
			
		PutBits(1,skip_frames,fap_stream);
		if (skip_frames) 
			Code_Skip_Frames(no_frames_skip,fap_stream);

}



void FBA_Object_Plane_Header(int is_intra,int fba_paramset_mask,int is_frame_rate,
							 int frame_rate,int fr_seconds,int fr_offset,int is_time_code,
							 int tc_hours,int tc_minutes,int tc_mbit,int tc_seconds,
							 int skip_frames,int no_frames_skip,FILE *fap_stream)
{
	if(is_intra) {
		Next_Start_Code(fap_stream);
		PutBits(32,FACE_OBJECT_PLANE_START_CODE,fap_stream);	
	}
	
	PutBits(1,is_intra,fap_stream);
	Put_Fba_Paramset_Mask(fba_paramset_mask,fap_stream);
	
	//temporal header 
	Temporal_Header(is_intra,is_frame_rate,	frame_rate,fr_seconds,fr_offset,
		is_time_code,tc_hours,tc_minutes,tc_mbit,tc_seconds,skip_frames,
		no_frames_skip,fap_stream);
}





void Code_Fap_Group_Mask(int group_number,int *fap_mask,FILE *fap_stream)
{
  int mask;

  for(mask=GroupMaskLimits[group_number];mask<GroupMaskLimits[group_number+1];mask++)
    PutBits(1,fap_mask[mask],fap_stream);
}




/**************************************************************************



	Arithmetic Coding Parts


**************************************************************************/


void ResetIntra(int *fap_pred)
{
  int counter;
  
  for(counter=0;counter<NO_FAPS;counter++) 
    fap_pred[counter]=0;	
	  
}



int TestNewFapMask(int frame_counter,int *fap_mask,int *old_fap_mask) 
{
  int counter,test;

  if(frame_counter==0) {

    for(counter=0;counter<NO_FAPS;counter++)
      old_fap_mask[counter]=fap_mask[counter];
    
      return OFF;    
  }
  
  else {

    test=OFF;
        
    for(counter=0;counter<NO_FAPS;counter++)
      if(fap_mask[counter])
	if(old_fap_mask[counter]!=fap_mask[counter])
	  test=ON;
            
    for(counter=0;counter<NO_FAPS;counter++)
      old_fap_mask[counter]=fap_mask[counter];
  
  }

  return test;
  
}



int CheckGroupBegin(int pos)
{
	int counter;

	for(counter=0;counter<11;counter++) {
		if(GroupMaskLimits[counter]==pos) return ON;
	}
	
	return OFF;
}



void Code_New_Minmax(int is_I_new_max,int is_I_new_min,int is_P_new_max,
					 int is_P_new_min,int *fap_mask,int *i_new_max, 
					 int  *i_new_min,int *p_new_max,int *p_new_min,
					 FILE *fap_stream)
{
	int counter,aux;
	
	if(is_I_new_max)
		for(counter=2;counter<NO_FAPS;counter++) {

			if(CheckGroupBegin(counter)) aux=0;

			if(!(aux&3)) 
				PutBits(1,MARKER_BIT,fap_stream);

			if(fap_mask[counter])
				PutBits(5,i_new_max[counter],fap_stream);
			
			aux++;
		}


	if(is_I_new_min)
		for(counter=2;counter<NO_FAPS;counter++){

			if(CheckGroupBegin(counter)) aux=0;

			if(!(aux&3)) 
				PutBits(1,MARKER_BIT,fap_stream);

			if(fap_mask[counter])
				PutBits(5,i_new_min[counter],fap_stream);
			
			aux++;
		}
			
	if(is_P_new_max)
		for(counter=2;counter<NO_FAPS;counter++) {

			if(CheckGroupBegin(counter)) aux=0;

			if(!(aux&3)) 
				PutBits(1,MARKER_BIT,fap_stream);

			if(fap_mask[counter])
				PutBits(5,p_new_max[counter],fap_stream);
			
			aux++;
		}
			
	if(is_P_new_min)
		for(counter=2;counter<NO_FAPS;counter++) {

			if(CheckGroupBegin(counter)) aux=0;

			if(!(aux&3)) 
				PutBits(1,MARKER_BIT,fap_stream);

			if(fap_mask[counter])
				PutBits(5,p_new_min[counter],fap_stream);
			
			aux++;
		}
}



int quantize(int fap, int quantStep) 
{
	
	if(!quantStep)
		return fap;
	else  if( abs(fap)<=quantStep/2 )
		return 0;
	else 
		return (fap + sign(fap)*quantStep/2 ) / quantStep;
	
}





void Code_Viseme(FILE *fap_stream,int *fap_vis, int **Vis_freq,int **Vis_cumul_freq,
					 int **IVis_cumul_freq,int is_intra,int fap_quant)
{
	static int vis_prev[3];
	int qvis,vis_diff;

	if(is_intra) {

		//vis 1
		AR_Encode(fap_vis[0],IVis_cumul_freq[0],fap_stream);
		vis_prev[0] = fap_vis[0];
		
		//vis 2
		AR_Encode(fap_vis[1],IVis_cumul_freq[1],fap_stream);
		vis_prev[1] = fap_vis[1];
		
		//blend 
		qvis = quantize(fap_vis[2],fap_quant);
		AR_Encode(qvis,IVis_cumul_freq[2],fap_stream);
		vis_prev[2] = qvis*fap_quant;
	
	}
	else {
		//vis 1
		vis_diff = fap_vis[0] - vis_prev[0] + 14;
		AR_Encode(vis_diff,Vis_cumul_freq[0],fap_stream);
		update_viseme(vis_diff,0,Vis_cumul_freq,Vis_freq,fap_quant);	
		vis_prev[0] = fap_vis[0];

		//vis 2
		vis_diff = fap_vis[1] - vis_prev[1] + 14;
		AR_Encode(vis_diff,Vis_cumul_freq[1],fap_stream);
		update_viseme(vis_diff,1,Vis_cumul_freq,Vis_freq,fap_quant);
		vis_prev[1] = fap_vis[1];

		//blend
		vis_diff = fap_vis[2] - vis_prev[2] ;
		qvis = quantize(vis_diff,fap_quant) + quantize(63,fap_quant);
		AR_Encode(qvis,Vis_cumul_freq[2],fap_stream);
		update_viseme(qvis,2,Vis_cumul_freq,Vis_freq,fap_quant);
		vis_diff = (qvis-quantize(63,fap_quant))*fap_quant;
		vis_prev[2] +=vis_diff;
	}
	
	encoder_flush(fap_stream);
	PutBits(1,fap_vis[3],fap_stream);
}






void Code_Expression(FILE *fap_stream,int *fap_expr, int **Exp_freq,int **Exp_cumul_freq,
					 int **IExp_cumul_freq,int is_intra,int fap_quant)
{
	static int expr_prev[4];
	int qexpr,expr_diff;

	if(is_intra) {

		//expr 1
		AR_Encode(fap_expr[0],IExp_cumul_freq[0],fap_stream);
		expr_prev[0] = fap_expr[0];

		// intensity 1
		qexpr = quantize(fap_expr[1],fap_quant);
		AR_Encode(qexpr,IExp_cumul_freq[1],fap_stream);
		expr_prev[1] = qexpr*fap_quant;
		
		//expr 2
		AR_Encode(fap_expr[2],IExp_cumul_freq[2],fap_stream);
		expr_prev[2] = fap_expr[2];
		
		//intensity 2
		qexpr = quantize(fap_expr[3],fap_quant);
		AR_Encode(qexpr,IExp_cumul_freq[3],fap_stream);
		expr_prev[3] = qexpr*fap_quant;
	
	}
	else {
		//expr 1
		expr_diff = fap_expr[0] - expr_prev[0] + 6;
		AR_Encode(expr_diff,Exp_cumul_freq[0],fap_stream);
		update_expression(expr_diff,0,Exp_cumul_freq,Exp_freq,fap_quant);	
		expr_prev[0] = fap_expr[0];

		//intensity 1
		expr_diff = fap_expr[1] - expr_prev[1] ;
		qexpr = quantize(expr_diff,fap_quant) + quantize(63,fap_quant);
		AR_Encode(qexpr,Exp_cumul_freq[1],fap_stream);
		update_expression(qexpr,1,Exp_cumul_freq,Exp_freq,fap_quant);
		expr_diff = (qexpr-quantize(63,fap_quant))*fap_quant;
		expr_prev[1] +=expr_diff;

		//expr 2
		expr_diff = fap_expr[2] - expr_prev[2] + 6;
		AR_Encode(expr_diff,Exp_cumul_freq[2],fap_stream);
		update_expression(expr_diff,2,Exp_cumul_freq,Exp_freq,fap_quant);
		expr_prev[2] = fap_expr[2];

		//intensity 2
		expr_diff = fap_expr[3] - expr_prev[3] ;
		qexpr = quantize(expr_diff,fap_quant) + quantize(63,fap_quant);
		AR_Encode(qexpr,Exp_cumul_freq[3],fap_stream);
		update_expression(qexpr,3,Exp_cumul_freq,Exp_freq,fap_quant);
		expr_diff = (qexpr-quantize(63,fap_quant))*fap_quant;
		expr_prev[3] +=expr_diff;
	}
	
	encoder_flush(fap_stream);
	PutBits(1,fap_expr[4],fap_stream);
	PutBits(1,fap_expr[5],fap_stream);
}




void Code_IFap(int *fap_mask,int *fap_pred, float *fap,int *fap_expr,int *fap_vis,
			   int *fapQuantSteps,
			   int *Irange_max, int *Irange_min, int **Icumul_freq, 
			   int **Ifreq, int *range_max, int *range_min, int **cumul_freq, int **freq,
			   int	**Exp_freq,int **Exp_cumul_freq,int **IExp_cumul_freq,
			   int	**Vis_freq,int **Vis_cumul_freq,int **IVis_cumul_freq,
			   int fapq,FILE *fap_stream )
{
	int counter;
	int fap_quant;
	int group_number;

	if(fap_mask[0])
		Code_Viseme(fap_stream,fap_vis,Vis_freq,Vis_cumul_freq,IVis_cumul_freq,ON,fapq);
	if(fap_mask[1])
		Code_Expression(fap_stream,fap_expr,Exp_freq,Exp_cumul_freq,IExp_cumul_freq,ON,fapq);
	
	for(group_number=1;group_number<GROUP_MASK_SIZE;group_number++) {

		for(counter=GroupMaskLimits[group_number];counter<
				GroupMaskLimits[group_number+1];counter++) 
				if(fap_mask[counter]) {
					
					fap_quant = quantize((int)fap[counter],fapQuantSteps[counter]) - 
						Irange_min[counter];
					
					Check_Symbol(&fap_quant,counter,ON,Irange_max,Irange_min,range_max,
						range_min);
					
					AR_Encode(fap_quant,Icumul_freq[counter],fap_stream);
					
					fap_pred[counter] = (fap_quant + Irange_min[counter]) *
						fapQuantSteps[counter];
					
					if(DEBUG)
						printf("\n IFAP%d: %d -> send FAPQ%d: %d (%d)",counter+1,
						(int)fap[counter],counter+1,fap_pred[counter],
						fap_quant);
				
				}
	}
}





void Code_PFap(int *fap_mask,int *fap_pred, float *fap,int *fap_expr,int *fap_vis,
			   int *fapQuantSteps,
			   int *Irange_max, int *Irange_min, int **Icumul_freq, 
			   int **Ifreq,int *range_max, int *range_min, int **cumul_freq,
			   int **freq, int **Exp_freq,int **Exp_cumul_freq,int **IExp_cumul_freq,
			   int	**Vis_freq,int **Vis_cumul_freq,int **IVis_cumul_freq,
			   int fapq,FILE *fap_stream)
{
	int   counter;
	int   fap_quant;
	int   fap_diff;
	int   group_number; 
	

	if(fap_mask[0])	
		Code_Viseme(fap_stream,fap_vis,Vis_freq,Vis_cumul_freq,IVis_cumul_freq,OFF,fapq);
	if(fap_mask[1])
		Code_Expression(fap_stream,fap_expr,Exp_freq,Exp_cumul_freq,IExp_cumul_freq,OFF,fapq);
	
	for(group_number=1;group_number<GROUP_MASK_SIZE;group_number++) {	
				
			for(counter=GroupMaskLimits[group_number];counter<
				GroupMaskLimits[group_number+1];counter++) 
				if(fap_mask[counter]) {
					
					fap_diff = (int)fap[counter] - fap_pred[counter];
					
					fap_quant = quantize(fap_diff,fapQuantSteps[counter]) - range_min[counter];
					
					Check_Symbol(&fap_quant,counter,OFF,Irange_max,Irange_min,range_max,
						range_min);
					
					AR_Encode(fap_quant, cumul_freq[counter],fap_stream);
					
					update(fap_quant,counter,OFF,Irange_max, Irange_min,Icumul_freq,Ifreq, 
						range_max,range_min,cumul_freq,freq);
					
					fap_diff = (fap_quant + range_min[counter])*fapQuantSteps[counter];
					
					fap_pred[counter]+=fap_diff;
					
					if(DEBUG)
						printf(" \nPFAP%d: %d -> send FAPQ%d: %d (%d) ",counter+1,
						(int)fap[counter],counter+1,fap_pred[counter],
						fap_quant);
				}	
	}
}

	



/***********************************************************************


  DCT Encoder


***********************************************************************/




void Obtain_DCT(float *faps, float *faps_coeff, float **dct)
{
	int line,colunm;
	
	for (line=0; line<SEGMENT_LENGTH; line++) {
		faps_coeff[line]=0;
		for (colunm=0; colunm<SEGMENT_LENGTH; colunm++)
			faps_coeff[line] += dct[line][colunm]*faps[colunm];
	}
}




void DCTQuantize(float dc, int *dc_Q,float quant_step)
{
	if(dc>0.0)
		*dc_Q = (int)((dc/quant_step)+0.5);
	else
		*dc_Q = (int)((dc/quant_step)-0.5);
}



void Dequantize(int dc_Q,float *dc_pred,float quant_step)
{
	*dc_pred = (float)dc_Q*quant_step;	
}



void DC_I_Encode(int dc_Q,FILE *fap_stream)
{
	if( (dc_Q > DC_MAX) || (dc_Q < DC_MIN) ) {

		ShortPutBits((short)(DC_MIN-1),fap_stream);
		PutBits(32,dc_Q,fap_stream);
	}
	else 
		ShortPutBits((short)dc_Q,fap_stream);

}
					


void HuffmanCoding(int **table,int symbol,FILE *fap_stream)
{
	int counter,bitcounter;
	
	bitcounter = table[symbol][0];
	
	for(counter=1;counter<=bitcounter;counter++)
		PutBits(1,table[symbol][counter],fap_stream);


}



void AC_Encode(float *faps_coeff, float quant_step, int **run_table,
			   int **ac_table,FILE *fap_stream)
{
	int counter,*coeff_Q;
	int run_length;

	coeff_Q = (int *)calloc(SEGMENT_LENGTH,sizeof(int));

	for(counter=1;counter<SEGMENT_LENGTH;counter++)
		DCTQuantize(faps_coeff[counter],&coeff_Q[counter],quant_step);	

	run_length = 1;
	

	for(counter=1;counter<SEGMENT_LENGTH;counter++) {
	
		if(coeff_Q[counter] != 0) {

			HuffmanCoding(run_table,run_length,fap_stream);
			
			if( (coeff_Q[counter]>255) || (coeff_Q[counter]<-256) ){
				HuffmanCoding(ac_table,256,fap_stream);
				DC_I_Encode(coeff_Q[counter],fap_stream);			
			}
			else 
				HuffmanCoding(ac_table,coeff_Q[counter]+256,fap_stream);
			
			run_length = 1;
		}
	
		else run_length++;

	}

	if(run_length != 1)
		HuffmanCoding(run_table,0,fap_stream);

	free(coeff_Q);

}


 
					
void Predict(float dc,float dc_pred,float *dc_diff)
{
	*dc_diff = dc - dc_pred;
}
					 

					
void DC_P_Encode(int dc_Q,int **dc_table,FILE *fap_stream)
{
	if( (dc_Q>255) || (dc_Q<-255) ){
		HuffmanCoding(dc_table,0,fap_stream);
		DC_I_Encode(dc_Q,fap_stream);
	}
	else 
		HuffmanCoding(dc_table,dc_Q+256,fap_stream);

}
					 
					

void Reconstruct(float dc_diff,float *dc_pred )
{
	*dc_pred +=dc_diff;
} 




int last_viseme[3];


void Code_I_Viseme_Segment(FILE *fap_stream, int **fap_viseme)
{
	int counter;
	
	PutBits(5,fap_viseme[0][0],fap_stream);
	PutBits(5,fap_viseme[0][1],fap_stream);
	PutBits(7,fap_viseme[0][2],fap_stream);
	PutBits(1,fap_viseme[0][3],fap_stream);

	for(counter=1;counter<SEGMENT_LENGTH;counter++) {
		PutBits(5,fap_viseme[counter][0]-fap_viseme[counter-1][0]+14,fap_stream);
		PutBits(5,fap_viseme[counter][1]-fap_viseme[counter-1][1]+14,fap_stream);
		PutBits(7,fap_viseme[counter][2]-fap_viseme[counter-1][2]+63,fap_stream);
		PutBits(1,fap_viseme[counter][3],fap_stream);
	}

	for(counter=0;counter<3;counter++)
		last_viseme[counter] = fap_viseme[15][counter];

}


void Code_P_Viseme_Segment(FILE *fap_stream, int **fap_viseme)
{
	int counter;
	
	PutBits(5,fap_viseme[0][0]-last_viseme[0]+14,fap_stream);
	PutBits(5,fap_viseme[0][1]-last_viseme[1]+14,fap_stream);
	PutBits(7,fap_viseme[0][2]-last_viseme[2]+63,fap_stream);
	PutBits(1,fap_viseme[0][3],fap_stream);


	for(counter=1;counter<SEGMENT_LENGTH;counter++) {
		PutBits(5,fap_viseme[counter][0]-fap_viseme[counter-1][0]+14,fap_stream);
		PutBits(5,fap_viseme[counter][1]-fap_viseme[counter-1][1]+14,fap_stream);
		PutBits(7,fap_viseme[counter][2]-fap_viseme[counter-1][2]+63,fap_stream);
		PutBits(1,fap_viseme[counter][3],fap_stream);
	}

	for(counter=0;counter<3;counter++)
		last_viseme[counter] = fap_viseme[15][counter];
}




int last_expression[4];


void Code_I_Expression_Segment(FILE *fap_stream, int **fap_expression)
{
	int counter;
	
	PutBits(4,fap_expression[0][0],fap_stream);
	PutBits(4,fap_expression[0][2],fap_stream);
	PutBits(7,fap_expression[0][1],fap_stream);
	PutBits(7,fap_expression[0][3],fap_stream);
	PutBits(1,fap_expression[0][4],fap_stream);
	PutBits(1,fap_expression[0][5],fap_stream);

	for(counter=1;counter<SEGMENT_LENGTH;counter++) {
		PutBits(4,fap_expression[counter][0]-fap_expression[counter-1][0]+6,fap_stream);
		PutBits(4,fap_expression[counter][2]-fap_expression[counter-1][2]+6,fap_stream);
		PutBits(7,fap_expression[counter][1]-fap_expression[counter-1][1]+63,fap_stream);
		PutBits(7,fap_expression[counter][3]-fap_expression[counter-1][3]+63,fap_stream);
		PutBits(1,fap_expression[counter][4],fap_stream);
		PutBits(1,fap_expression[counter][5],fap_stream);
	}

	for(counter=0;counter<4;counter++)
		last_expression[counter] = fap_expression[15][counter];

}


void Code_P_Expression_Segment(FILE *fap_stream, int **fap_expression)
{
	int counter;
	
	PutBits(4,fap_expression[0][0]-last_expression[0]+6,fap_stream);
	PutBits(4,fap_expression[0][2]-last_expression[2]+6,fap_stream);
	PutBits(7,fap_expression[0][1]-last_expression[1]+63,fap_stream);
	PutBits(7,fap_expression[0][3]-last_expression[3]+63,fap_stream);
	PutBits(1,fap_expression[0][4],fap_stream);
	PutBits(1,fap_expression[0][5],fap_stream);


	for(counter=1;counter<SEGMENT_LENGTH;counter++) {
		PutBits(4,fap_expression[counter][0]-fap_expression[counter-1][0]+6,fap_stream);
		PutBits(4,fap_expression[counter][2]-fap_expression[counter-1][2]+6,fap_stream);
		PutBits(7,fap_expression[counter][1]-fap_expression[counter-1][1]+63,fap_stream);
		PutBits(7,fap_expression[counter][3]-fap_expression[counter-1][3]+63,fap_stream);
		PutBits(1,fap_expression[counter][4],fap_stream);
		PutBits(1,fap_expression[counter][5],fap_stream);
	}

	for(counter=0;counter<4;counter++)
		last_expression[counter] = fap_expression[15][counter];

}





void Code_ISegment( int *fap_mask, float **faps, int **fap_expression,int **fap_visemes,
				   float *fapQuantSteps,float *dc_pred,float **dct, int **run_table,
				   int **ac_table,FILE *fap_stream)
{
	int counter;
	int group_number;
	float step;

	float **faps_coeff;
	int   dc_Q;

	faps_coeff = FloatMatrixAllocation(NO_FAPS,SEGMENT_LENGTH);
	
	for(group_number=0;group_number<10;group_number++) {
		
		if( group_number+1 == 1) {
			if(fap_mask[1]) 
				Code_I_Expression_Segment(fap_stream,fap_expression);
			if(fap_mask[0]) 
				Code_I_Viseme_Segment(fap_stream,fap_visemes);
		}
		
		else {
			
			for(counter=GroupMaskLimits[group_number];
				counter<GroupMaskLimits[group_number+1];counter++) 

				if(fap_mask[counter]) {
					step = fapQuantSteps[counter];

					Obtain_DCT(faps[counter],faps_coeff[counter],dct);
					DCTQuantize(faps_coeff[counter][0],&dc_Q,step/3.0f);
					Dequantize(dc_Q,&dc_pred[counter],step/3.0f);
					DC_I_Encode(dc_Q,fap_stream);
					AC_Encode(faps_coeff[counter],step,run_table,ac_table,fap_stream);
										
				}
		}
		
	}

	
	MatrixDealoc(faps_coeff,NO_FAPS);
}



void Code_PSegment( int *fap_mask, float **faps, int **fap_expression,int **fap_visemes,
					float *fapQuantSteps,float *dc_pred,float **dct, int **run_table, 
					int **dc_table, int **ac_table,FILE *fap_stream)
{
	int counter;
	int group_number;
	float step;

	float **faps_coeff;
	int   dc_Q;
	float *dc_diff;

	faps_coeff = FloatMatrixAllocation(NO_FAPS,SEGMENT_LENGTH);
	dc_diff    = (float *)calloc(NO_FAPS,sizeof(float));
	

	for(group_number=0;group_number<10;group_number++) {
		
		if( group_number+1 == 1 ) {
			if(fap_mask[1]) 
				Code_P_Expression_Segment(fap_stream,fap_expression);
			if(fap_mask[0]) 
				Code_P_Viseme_Segment(fap_stream,fap_visemes);
		}
		
		else {
			
			for(counter=GroupMaskLimits[group_number];
				counter<GroupMaskLimits[group_number+1];counter++) 

				if(fap_mask[counter]) {

					step = fapQuantSteps[counter];
					
					Obtain_DCT(faps[counter],faps_coeff[counter],dct);
			
					Predict(faps_coeff[counter][0],dc_pred[counter],&dc_diff[counter]);
					
					DCTQuantize(dc_diff[counter],&dc_Q,step/3.0f);
					
					DC_P_Encode(dc_Q,dc_table,fap_stream);
					
					Dequantize(dc_Q,&dc_diff[counter],step/3.0f);
					
					Reconstruct(dc_diff[counter],&dc_pred[counter]);
					
					AC_Encode(faps_coeff[counter],step,run_table,ac_table,fap_stream);
										
				}
		}
		
	}

	MatrixDealoc(faps_coeff,NO_FAPS);
	free(dc_diff);
}




/*****************************************************************************



  Face Object()


*****************************************************************************/




void Face_Object(char *fap_file,int face_object_coding_type)
{
	FILE  *fp_fap,*fp_epf,*fap_stream,*fp_val;
	float *fap;
	int	  *fap_expr,*fap_vis;
	int   *fap_pred;
	int   frame_counter,counter;
	int   *fap_mask,old_fap_mask[NO_FAPS];
	int   Nframe;
	char  word[BUFFER_SIZE];
    
	int   frame_rate,fr_seconds,fr_offset;
	int   tc_hours,tc_minutes,tc_mbit,tc_seconds;
	int   fba_paramset_mask;
	int   is_intra,is_time_code,is_frame_rate;
	int   skip_frames,no_frames_skip;
	int   group_number;
	int   fap_quant;
	int   *quant_steps,*mask_type;
	int   fap_mask_type;
	int   is_I_new_max,is_I_new_min,is_P_new_max,is_P_new_min;

	int   segment_counter;
	
	int	  *i_new_max,*i_new_min,*p_new_max,*p_new_min;
	int   *Irange_max, *Irange_min,*range_max, *range_min;
	int   **freq,**cumul_freq,**Ifreq,**Icumul_freq;
	int	  **Exp_freq,**Exp_cumul_freq,**IExp_cumul_freq;		
	int	  **Vis_freq,**Vis_cumul_freq,**IVis_cumul_freq;		
	
	float **dct;
	float **faps;
	int   **fap_expression,**fap_visemes;
	float *dc_pred,*dctquant_steps;
	int   **dc_table,**ac_table,**run_table;	
	int   size;

	int gender_bit = 0;
	
	/* FAPs */

	if(face_object_coding_type == ARITH_TYPE) {
		
		fap         = (float *)calloc(NO_FAPS,sizeof(float));
		fap_expr    = (int *)calloc(6,sizeof(int));
		fap_vis     = (int *)calloc(4,sizeof(int));
		fap_pred    = (int *)calloc(NO_FAPS,sizeof(int));
		quant_steps = (int *)calloc(NO_FAPS,sizeof(int));
		Icumul_freq = (int **)calloc(NO_FAPS,sizeof(int *));
		Ifreq		= (int **)calloc(NO_FAPS,sizeof(int *));
		freq		= (int **)calloc(NO_FAPS,sizeof(int *));
		cumul_freq	= (int **)calloc(NO_FAPS,sizeof(int *));
		
		IExp_cumul_freq = (int **)calloc(4,sizeof(int *));
		Exp_freq		= (int **)calloc(4,sizeof(int *));
		Exp_cumul_freq	= (int **)calloc(4,sizeof(int *));

		IVis_cumul_freq = (int **)calloc(3,sizeof(int *));
		Vis_freq		= (int **)calloc(3,sizeof(int *));
		Vis_cumul_freq	= (int **)calloc(3,sizeof(int *));

		/* initialize arithmetic coding range */
				
		i_new_max = (int *)calloc(NO_FAPS,sizeof(int));
		i_new_min = (int *)calloc(NO_FAPS,sizeof(int));
		p_new_max = (int *)calloc(NO_FAPS,sizeof(int));
		p_new_min = (int *)calloc(NO_FAPS,sizeof(int));
		
		Irange_max = (int *)calloc(NO_FAPS,sizeof(int));
		Irange_min = (int *)calloc(NO_FAPS,sizeof(int));
		range_max  = (int *)calloc(NO_FAPS,sizeof(int));
		range_min  = (int *)calloc(NO_FAPS,sizeof(int));
		
		for(counter=0; counter<NO_FAPS; counter++) {
			Irange_max[counter] = DefaultArithLimits[counter];
			Irange_min[counter] = DefaultArithLimits[NO_FAPS+counter];
			range_max[counter]  = DefaultArithLimits[2*NO_FAPS+counter];
			range_min[counter]  = DefaultArithLimits[3*NO_FAPS+counter];
		}

		for(counter=2; counter<NO_FAPS; counter++) {
			i_new_min[counter]  = Irange_min[counter]/RANGE_DELTA;
			i_new_max[counter]  = Irange_max[counter]/RANGE_DELTA;
			p_new_min[counter]  = range_min[counter]/RANGE_DELTA;
			p_new_max[counter]  = range_max[counter]/RANGE_DELTA;
		}

		
	}	
	else if(face_object_coding_type == DCT_TYPE) {

		/* dct faps*/
		dctquant_steps = (float *)calloc(NO_FAPS,sizeof(float));
		faps		   = FloatMatrixAllocation(NO_FAPS,SEGMENT_LENGTH);
		fap_expression = IntMatrixAllocation(SEGMENT_LENGTH+1,6);
		fap_visemes    = IntMatrixAllocation(SEGMENT_LENGTH+1,4);
		
		dc_pred    = (float *)calloc(NO_FAPS,sizeof(float));
		
		/* huffman tables */
		run_table = IntMatrixAllocation(16,10);
		dc_table  = IntMatrixAllocation(512,18);
		ac_table  = IntMatrixAllocation(512,18);

		/* read dct - tables */
		ReadHuffmanTables(run_table,dc_table,ac_table);
		
		/* make dct matrix */
		
		dct  = FloatMatrixAllocation(SEGMENT_LENGTH,SEGMENT_LENGTH);
		MakeDCTMatrix(dct);
		
	}
	
	fap_mask  = (int *)calloc(NO_FAPS,sizeof(int));
	mask_type = (int *)calloc(10,sizeof(int));
	


	/* FAP file */
	
	sprintf(word,"%s.fap",fap_file);
	if( (fp_fap  = fopen(word,"rb"))==NULL ) {
		printf("\n File - %s - does not exist. \n",word);
		exit(0);
	}
					
	
	/* EPF file */

	if(face_object_coding_type == ARITH_TYPE) {
		sprintf(word,"%s.epf",fap_file);
		if( (fp_epf  = fopen(word,"rb"))==NULL ) {
			printf("\n File - %s - does not exist. \n",word);
			exit(0);
		}
	}	
	
		
	/* stream file */
	
	sprintf(word,"%s.stream",fap_file);
	if( (fap_stream  = fopen(word,"wb"))==NULL ) {
		printf("\n File - %s - impossible to create. \n",word);
		exit(0);
	}
	

	/* bitvalues files */
	sprintf(word,"%s.val",fap_file);
	if( (fp_val  = fopen(word,"w"))==NULL ) {
		printf("\n File - %s - does not exist. \n",word);
		exit(0);
	}


	/* initialize bitcounter */
	
	bitcount = 0;
	bitsout = 8;
	
	
	/* face_object_plane class */
	
	frame_counter   = 0;			
	segment_counter = 0;

	if(face_object_coding_type == ARITH_TYPE) {
		ReadFAPInfo(&Nframe,fp_fap);

		Read1FrameEPF(frame_counter,&fba_paramset_mask,&is_intra,&is_frame_rate,
			&frame_rate,&fr_seconds,&fr_offset,&is_time_code,
			&tc_hours,&tc_minutes,&tc_mbit,&tc_seconds,&skip_frames,
			&no_frames_skip,mask_type,fap_mask,&fap_quant,
			&is_I_new_max,&is_I_new_min,&is_P_new_max,&is_P_new_min,
			Irange_max,Irange_min,range_max,range_min,
			i_new_max,i_new_min,p_new_max,p_new_min,Nframe,fp_epf);
		
		Read1FrameFAP(frame_counter,fap_mask,fap,fap_expr,fap_vis,fp_fap);
	
	}
	else if(face_object_coding_type == DCT_TYPE) {

		ReadDCTFAPInfo(&Nframe,fp_fap,&frame_rate);

		fr_seconds = 0;
		fr_offset  = 0;
		is_frame_rate = ON;
		fap_quant = 1;

		is_time_code = OFF;
		tc_hours=0;
		tc_minutes=0;
		tc_mbit=0;
		tc_seconds=0;

		skip_frames = 0;

		fba_paramset_mask = FAP;

		Reset(faps,fap_expression,mask_type);

		size = SEGMENT_LENGTH;
		if( frame_counter>(Nframe-SEGMENT_LENGTH) )
			size = Nframe  - frame_counter;

		Read1Segment(size,fap_mask,mask_type,faps,fap_expression,fap_visemes,&is_intra,fp_fap);
//		frame_counter+=size;

		is_intra   = ON;		
	}


	/* face object class */
	
	PutBits(32,FACE_OBJECT_START_CODE,fap_stream);
	
	if(!is_intra) {
		printf("\n Error: first frame must be intra!\n");
		exit(-1);
	}
	
	
	while(frame_counter<Nframe) {
		
		framebits = 0;

		if(face_object_coding_type == DCT_TYPE) 
			frame_counter+=size;


//		if(!(frame_counter%30))
//			is_intra = ON;


		//fba_object_plane_header 
		FBA_Object_Plane_Header(is_intra,fba_paramset_mask,is_frame_rate,frame_rate,fr_seconds,
								fr_offset,is_time_code,tc_hours,tc_minutes,tc_mbit,tc_seconds,
								skip_frames,no_frames_skip,fap_stream);

		
		if( fba_paramset_mask == FAP) {
			
			if(is_intra) {

				PutBits(5,fap_quant,fap_stream);
				
				for(group_number=0;group_number<GROUP_MASK_SIZE;group_number++) {
					
					PutBits(1,MARKER_BIT,fap_stream);
					fap_mask_type = mask_type[group_number];
					PutBits(2,fap_mask_type,fap_stream);
					if(fap_mask_type == FAP_MASK_TYPE1 || fap_mask_type == FAP_MASK_TYPE2)
						Code_Fap_Group_Mask(group_number,fap_mask,fap_stream);
				}
				
				
				PutBits(1,gender_bit,fap_stream);
				PutBits(1,face_object_coding_type,fap_stream);

				/* arithmetic coder */
				if(face_object_coding_type == ARITH_TYPE) {

					for(counter=0;counter<NO_FAPS;counter++) 
						quant_steps[counter] = FapQuantSteps[counter]*fap_quant;

					ResetIntra(fap_pred);
					
					for(counter=2;counter<NO_FAPS;counter++) {
						Irange_max[counter] = i_new_max[counter]*RANGE_DELTA/fap_quant;
						Irange_min[counter] = -i_new_min[counter]*RANGE_DELTA/fap_quant;
						range_max[counter]  = p_new_max[counter]*RANGE_DELTA/fap_quant;
						range_min[counter]  = -p_new_min[counter]*RANGE_DELTA/fap_quant;
					}		
					
					if( TestNewFapMask(frame_counter,fap_mask,old_fap_mask) ) {
						is_I_new_max = ON;
						is_I_new_min = ON;
						is_P_new_max = ON;
						is_P_new_min = ON;
					}
					
					PutBits(1,is_I_new_max,fap_stream);
					PutBits(1,is_I_new_min,fap_stream);
					PutBits(1,is_P_new_max,fap_stream);
					PutBits(1,is_P_new_min,fap_stream);
					
					
					init_arith(Irange_max,Irange_min,Icumul_freq,Ifreq,range_max, 
						range_min,cumul_freq,freq,Exp_freq,Exp_cumul_freq,IExp_cumul_freq,
						Vis_freq,Vis_cumul_freq,IVis_cumul_freq,fap_mask,fap_quant);
					
					Code_New_Minmax(is_I_new_max,is_I_new_min,is_P_new_max,
						is_P_new_min,fap_mask,i_new_max,i_new_min,
						p_new_max,p_new_min,fap_stream);
					
					Code_IFap(fap_mask,fap_pred,fap,fap_expr,fap_vis,quant_steps,
						Irange_max,Irange_min,Icumul_freq,Ifreq,range_max,
						range_min,cumul_freq,freq,Exp_freq,Exp_cumul_freq,IExp_cumul_freq,
						Vis_freq,Vis_cumul_freq,IVis_cumul_freq,fap_quant,fap_stream);
					
					encoder_flush(fap_stream);

					is_I_new_max = OFF;
					is_I_new_min = OFF;
					is_P_new_max = OFF;
					is_P_new_min = OFF;

				}
				else if(face_object_coding_type == DCT_TYPE) {

					for(counter=0;counter<NO_FAPS;counter++)
						dctquant_steps[counter] = fap_scale[fap_quant]*DCTQuantSteps[counter];

					Code_ISegment(fap_mask,faps,fap_expression,fap_visemes,dctquant_steps,
									dc_pred,dct,run_table,ac_table,fap_stream);

				}
				
			}
			else {
			//P frame/segment
				if(face_object_coding_type == ARITH_TYPE) {
					Code_PFap(fap_mask,fap_pred,fap,fap_expr,fap_vis,quant_steps,
						Irange_max,Irange_min,Icumul_freq,Ifreq,range_max,
						range_min,cumul_freq,freq,Exp_freq,Exp_cumul_freq,IExp_cumul_freq,
						Vis_freq,Vis_cumul_freq,IVis_cumul_freq,fap_quant,fap_stream);
					encoder_flush(fap_stream);
				}
				else if(face_object_coding_type == DCT_TYPE) {
					Code_PSegment(fap_mask,faps,fap_expression,fap_visemes,dctquant_steps,
						dc_pred,dct,run_table,dc_table,ac_table,fap_stream);
				}
			}

		}/* end fba_param_mask */
		else {
			printf("\n Only FAP encoding is performed.\n");
			return;
		}

		if(DEBUG  && (face_object_coding_type == ARITH_TYPE))       
			printf("\n\n ----- Frame %d encoded (%d bits)-----\n",frame_counter,framebits);
		else if(DEBUG  && (face_object_coding_type == DCT_TYPE))
			printf("\n\n ----- Segment(%d) %d encoded (%d bits)-----\n",is_intra,
				segment_counter,framebits);

		fprintf(fp_val,"%d \n",framebits);


		if(face_object_coding_type == ARITH_TYPE) 
			frame_counter++;
		else if(face_object_coding_type == DCT_TYPE) {
 			segment_counter++;		
		}

		if(frame_counter<Nframe) {
			
			if(face_object_coding_type == ARITH_TYPE) {
				Read1FrameEPF(frame_counter,&fba_paramset_mask,&is_intra,&is_frame_rate,
					&frame_rate,&fr_seconds,&fr_offset,&is_time_code,
					&tc_hours,&tc_minutes,&tc_mbit,&tc_seconds,&skip_frames,
					&no_frames_skip,mask_type,fap_mask,&fap_quant,
					&is_I_new_max,&is_I_new_min,&is_P_new_max,&is_P_new_min,
					Irange_max,Irange_min,range_max,range_min,
					i_new_max,i_new_min,p_new_max,p_new_min,Nframe,fp_epf);
				
				Read1FrameFAP(frame_counter,fap_mask,fap,fap_expr,fap_vis,fp_fap);

				// check FAPs
			}
			else if(face_object_coding_type == DCT_TYPE) {

				is_intra = OFF;
				Reset(faps,fap_expression,mask_type);

				size = SEGMENT_LENGTH;
				if( frame_counter>(Nframe-SEGMENT_LENGTH) )
					size = Nframe  - frame_counter;
	
				Read1Segment(size,fap_mask,mask_type,faps,fap_expression,fap_visemes,&is_intra,
							fp_fap);
			}

		}

	}


	Next_Start_Code(fap_stream);
	PutBits(32,FACE_OBJECT_END_CODE,fap_stream);
		
	if(face_object_coding_type == ARITH_TYPE) 
		printf("\n Frame rate: %d Quant step: %d - Bits used: %d bits (%.3f kbps)\n",
				(int)frame_rate,fap_quant,bitcount,
				(float)(bitcount*frame_rate)/(float)(Nframe*1000) );
	else if(face_object_coding_type == DCT_TYPE) 
		printf("\nBits used: %d bits (%.3f kbps)\n",bitcount,
		(float)(bitcount*frame_rate)/(float)(Nframe*1000) );

	fprintf(fp_val,"\n fr = %d q= %d : bits = %d (%.3f kbps)",(int)(frame_rate),fap_quant,
			bitcount,(float)(bitcount*frame_rate)/(float)(Nframe*1000) );

	
	free(mask_type);
	free(fap_mask);
	
	if(face_object_coding_type == ARITH_TYPE) {

		free(fap);
		free(fap_pred);
		free(quant_steps);
		free(fap_expr);
		free(fap_vis);

		free(i_new_max);
		free(i_new_min);
		free(p_new_max);
		free(p_new_min);
		free(Irange_max);
		free(Irange_min);
		free(range_max);
		free(range_min);
		
		
		for(counter=2; counter<NO_FAPS; counter++) {
			free(Ifreq[counter]);
			free(Icumul_freq[counter]);
			free(freq[counter]);
			free(cumul_freq[counter]);
		}
		
		free(Ifreq);
		free(Icumul_freq);
		free(freq);
		free(cumul_freq);
		
		for(counter=0; counter<4; counter++) {
			free(IExp_cumul_freq[counter]);
			free(Exp_freq[counter]);
			free(Exp_cumul_freq[counter]);
		}
		
		free(IExp_cumul_freq);
		free(Exp_freq);
		free(Exp_cumul_freq);

		for(counter=0; counter<3; counter++) {
			free(IVis_cumul_freq[counter]);
			free(Vis_freq[counter]);
			free(Vis_cumul_freq[counter]);
		}
		
		free(IVis_cumul_freq);
		free(Vis_freq);
		free(Vis_cumul_freq);

		fclose (fp_epf);
	}
	else if(face_object_coding_type == DCT_TYPE) {
		
		MatrixDealoc(dct,SEGMENT_LENGTH);
		MatrixDealoc(faps,NO_FAPS);
		IntMatrixDealoc(fap_expression,SEGMENT_LENGTH+1);
		IntMatrixDealoc(fap_visemes,SEGMENT_LENGTH+1);
		
		IntMatrixDealoc(run_table,16);
		IntMatrixDealoc(dc_table,512);
		IntMatrixDealoc(ac_table,512);

		free(dctquant_steps);
		free(dc_pred);
	}

	fclose (fp_fap);
	fclose (fap_stream);
	fclose (fp_val);
}






/***********************************************************************


  Main


***********************************************************************/


int main(int argc,char *argv[])
{
	int coding_type;
	
	if ( argc!=3 ) {
		printf("Usage:  <ISTFAPcoder> <fapfilename>  <coding type=> Arith=0, DCT=1>\n");
		return 0;
	}
	
	sscanf(argv[2],"%d",&coding_type);
	if(coding_type>1 || coding_type<0) {
		printf("rror in coding type: Arith=0, DCT=1");
		return 0;
	}

	Face_Object(argv[1],coding_type);
	
	return 0;
}




