/********************************************************************/
/* 																	*/
/* This software module was originally developed and edited by		*/
/* Gabriel Antunes Abrantes (Instituto Superior Tcnico)			*/	
/* 																	*/
/* in the course of development of the MPEG-4 standard.	 			*/
/* This software module is an implementation of a part of one or 	*/
/* more MPEG-4 tools as specified by the	 						*/
/* MPEG-4 standard. ISO/IEC gives users of the MPEG-4 standard		*/
/* free license to this software module or modifications thereof 	*/
/* for use in hardware or software products claiming conformance 	*/
/* to the MPEG-4 standard. Those intending to use this software 	*/
/* module in hardware or software products are advised that its 	*/
/* use may infringe existing patents. The original developer of		*/
/* this software module and his company, the subsequent 			*/
/* editors and their companies, and ISO/IEC have no liability		*/
/* for use of this software module or modifications thereof in 		*/
/* an implementation. Copyright is not released for non				*/
/* MPEG-4 standard conforming products. ACTS-MoMuSys partners	 	*/
/* retain full right to use the code for their own purposes, 		*/
/* assign or donate the code to a third party and to inhibit 		*/
/* third parties from using the code for non MPEG-4 standard		*/
/* conforming products. This copyright notice must be included 		*/
/* in all copies or derivative works. 								*/
/* Copyright (c)1997 												*/
/* 																	*/
/********************************************************************/







/***********************************HeaderBegin****************************
*   File : ISTFAPdecoder.cpp
*   
*   Author : Gabriel Antunes Abrantes(gabriel.abrantes@lx.it.pt)
*   Created : 25-July-1997 
*
*   Description : Face Object Decoder
*
*   Modified:
*   Version 5.0 / November-1998
*   Updated for after MPEG-4 Atlantic City Meeting for inclusion in
*				 the IM1 3-D player
*
****************************************HeaderEnd**************************/


#include <stdio.h>
#include <math.h>
#include <stdlib.h>
#include <ctype.h>
#include <string.h>

#include "ISTFAPdecoder.h"


// constructor

FAP_Decoder::FAP_Decoder(
#ifndef IM1
		unsigned char *stbuffer
#endif 						 
		)
{
#ifndef IM1
		stream_buffer = stbuffer;
#else
		stream_buffer = NULL;
#endif 		

	position = 0;
	end = OFF;
	zerocount = 0;
	firstframe = ON;

	/* fap masks */
	quant_steps = (int *)calloc(NO_FAPS,sizeof(int));
		
	/* initialize arithmetic coding */
	Icumul_freq = (int **)calloc(NO_FAPS,sizeof(int *));
	freq		= (int **)calloc(NO_FAPS,sizeof(int *));
	cumul_freq	= (int **)calloc(NO_FAPS,sizeof(int *));
	IExp_cumul_freq = (int **)calloc(4,sizeof(int *));
	Exp_freq		= (int **)calloc(4,sizeof(int *));
	Exp_cumul_freq	= (int **)calloc(4,sizeof(int *));
	IVis_cumul_freq = (int **)calloc(3,sizeof(int *));
	Vis_freq		= (int **)calloc(3,sizeof(int *));
	Vis_cumul_freq	= (int **)calloc(3,sizeof(int *));

	i_new_max = (int *)calloc(NO_FAPS,sizeof(int));
	i_new_min = (int *)calloc(NO_FAPS,sizeof(int));
	p_new_max = (int *)calloc(NO_FAPS,sizeof(int));
	p_new_min = (int *)calloc(NO_FAPS,sizeof(int));
	
	Irange_max = (int *)calloc(NO_FAPS,sizeof(int));
	Irange_min = (int *)calloc(NO_FAPS,sizeof(int));
	range_max  = (int *)calloc(NO_FAPS,sizeof(int));
	range_min  = (int *)calloc(NO_FAPS,sizeof(int));
	
	
	for(int counter=0; counter<NO_FAPS; counter++) {
		Irange_max[counter] = DefaultArithLimits[counter];
		Irange_min[counter] = DefaultArithLimits[NO_FAPS+counter];
		range_max[counter]  = DefaultArithLimits[2*NO_FAPS+counter];
		range_min[counter]  = DefaultArithLimits[3*NO_FAPS+counter];
	}
	
	
	/* DCT FAPs */
	dctquant_steps	  = (float *)calloc(NO_FAPS,sizeof(float));
	dctfaps			  = FloatMatrixAllocation(NO_FAPS,SEGMENT_LENGTH);
	dctfap_expression = IntMatrixAllocation(SEGMENT_LENGTH,6);
	dctfap_viseme     = IntMatrixAllocation(SEGMENT_LENGTH,4);

	/* DCT faps*/
	dc_pred    = (float *)calloc(NO_FAPS,sizeof(float));

	/* huffman tables */
	run_table = IntMatrixAllocation(16,10);
	dc_table  = IntMatrixAllocation(512,18);
	ac_table  = IntMatrixAllocation(512,18);
	
	/* read DCT - tables */
	ReadHuffmanTables(run_table,dc_table,ac_table);

	/* make DCT matrix */
	idct  = FloatMatrixAllocation(SEGMENT_LENGTH,SEGMENT_LENGTH);
	MakeIDCTMatrix(idct);
	
	/* initialize bitcounter */
	bitsin = 0;
	end_code = OFF;	
}



// destructor

FAP_Decoder::~FAP_Decoder()
{
	free(quant_steps);
	
	free(i_new_max);
	free(i_new_min);
	free(p_new_max);
	free(p_new_min);
	free(Irange_max);
	free(Irange_min);
	free(range_max);
	free(range_min);
	
	for(int counter=0; counter<NO_FAPS; counter++) {
		free(Icumul_freq[counter]);
		free(freq[counter]);
		free(cumul_freq[counter]);
	}
	
	free(Icumul_freq);
	free(freq);
	free(cumul_freq);
	
	for(counter=0; counter<4; counter++) {
		free(IExp_cumul_freq[counter]);
		free(Exp_freq[counter]);
		free(Exp_cumul_freq[counter]);
	}

	for(counter=0; counter<3; counter++) {
		free(IVis_cumul_freq[counter]);
		free(Vis_freq[counter]);
		free(Vis_cumul_freq[counter]);
	}
	
	free(IExp_cumul_freq);
	free(Exp_freq);
	free(Exp_cumul_freq);

	free(IVis_cumul_freq);
	free(Vis_freq);
	free(Vis_cumul_freq);
		
	MatrixDealoc(idct,SEGMENT_LENGTH);
	MatrixDealoc(dctfaps,NO_FAPS);
	IntMatrixDealoc(dctfap_expression,SEGMENT_LENGTH);
	IntMatrixDealoc(dctfap_viseme,SEGMENT_LENGTH);

	IntMatrixDealoc(run_table,16);
	IntMatrixDealoc(dc_table,512);
	IntMatrixDealoc(ac_table,512);
	
	free(dctquant_steps);
	free(dc_pred);
}


/******************************************************************


 - Memory allocation


******************************************************************/

float** FAP_Decoder::FloatMatrixAllocation(int vs,int hs)
{
  float **new_matrix;
  int i;
  
  new_matrix=(float **) calloc(vs,sizeof(float*));
  if ( (new_matrix==NULL) ) {
    printf("Out of Memory \n");
    exit(1);
  }
  
  for(i=0;i<vs;i++) {
    new_matrix[i]=(float *) calloc(hs,sizeof(float));
    if ( (new_matrix[i]==NULL)  ){
      printf("Out of Memory \n");
      exit(1);
    }
  }
  
  return new_matrix;
}



void FAP_Decoder::MatrixDealoc(float **matrix,int vs)
{
  int i;
  
  for(i=0;i<vs;i++) 
    free(matrix[i]);
  
  free(matrix);
}




int **FAP_Decoder::IntMatrixAllocation(int vs,int hs)
{
  int **new_matrix;
  int i;
  
  new_matrix=(int **) calloc(vs,sizeof(int*));
  if ( (new_matrix==NULL) ) {
    printf("Out of Memory \n");
    exit(1);
  }
  
  for(i=0;i<vs;i++) {
    new_matrix[i]=(int *) calloc(hs,sizeof(int));
    if ( (new_matrix[i]==NULL)  ){
      printf("Out of Memory \n");
      exit(1);
    }
  }
  
  return new_matrix;
}




void FAP_Decoder::IntMatrixDealoc(int **matrix,int vs)
{
  int i;
  
  for(i=0;i<vs;i++) 
    free(matrix[i]);
  
  free(matrix);
}


int FAP_Decoder::quantize(int fap, int quantStep) 
{
	
	if(!quantStep)
		return fap;
	else  if( abs(fap)<=quantStep/2 )
		return 0;
	else 
		return (fap + sign(fap)*quantStep/2 ) / quantStep;
	
}



/***********************************************************************

  Get bits of stream -   
	
***********************************************************************/

int FAP_Decoder::GetBits(int no_bits) 
{
	int	counter;
	int	value = 0;
	int	bit;
	static unsigned char byte;
	
	for(counter=0; counter<no_bits; counter++) {
        
		if(bitsin==0) {
			byte = stream_buffer[position++];
			bitsin = 8;
		}

		bitsin -= 1;
		bit = (byte&0200);
		bit >>=7;
		byte <<=1;
		
		value <<= 1;
        value |= bit;
	}
	
	return value;
}



int FAP_Decoder::ShortGetBits() 
{
	int counter;
	short value = 0;	
	
	for(counter=0; counter<16; counter++) {
        value <<= 1;
        value += GetBits(1);
	}
	
	return (int)value;
}




/***********************************************************************

 
  Arithmetic Coder - as specified in the MPEG4-Visual FCD
  
   
***********************************************************************/


int FAP_Decoder::bit_out_psc_layer()
{	
	if(zerocount == ZEROMAX) {
		if(!GetBits(1)) {	
			printf("MARKER BIT error in arithmetic decoding\n");
			exit(-1);
			return OFF;
		}
		zerocount = 0;
	}
	
	Bit = GetBits(1);
	
	if(!Bit) zerocount++;
	else zerocount = 0;

	return ON;
}



int FAP_Decoder::decoder_reset( )
{
	int i;
	
	code_value = 0;
	low = 0;
	high = top;
	zerocount = 0;
	
	for (i = 1;   i <= 16;   i++) {
		if(!bit_out_psc_layer())
			return OFF; 
		code_value = 2 * code_value + Bit;
	}
	
	return ON;
}



int FAP_Decoder::aa_decode(int cumul_freq[ ])
{
	
	length = high - low + 1;
	cum = (-1 + (code_value - low + 1) * cumul_freq[0]) / length;
	for (sacindex = 1; cumul_freq[sacindex] > cum; sacindex++);
	high = low - 1 + (length * cumul_freq[sacindex-1]) / cumul_freq[0];
	low += (length * cumul_freq[sacindex]) / cumul_freq[0];
	
	for ( ; ; ) {  
		if (high < q2) ;
		else if (low >= q2) {
			code_value -= q2; 
			low -= q2; 
			high -= q2;
		}
		
		else if (low >= q1 && high < q3) {
			code_value -= q1; 
			low -= q1; 
			high -= q1;
		}
		
		else {
			break;
		}
		
		low *= 2; 
		high = 2*high + 1;
		if(!bit_out_psc_layer())
			return DEC_ERROR; 
		code_value = 2*code_value + Bit;
		
	}
	
	return (sacindex-1);
}




void FAP_Decoder::update(int symbol, int index,int *range_max, int *range_min,int **cumul_freq, int **freq) 
{
	int n;
	int *cumul_tmp, *freq_tmp;
	
	n = range_max[index]-range_min[index]+1;
	cumul_tmp = cumul_freq[index];
	freq_tmp = freq[index];
	
	if (cumul_tmp[0] == Max_frequency) {
		int i, cum;
		cum = 0;
		for(i=n-1; i>=0; i--) {
			freq_tmp[i] = (freq_tmp[i]+1)/2;
			cum += freq_tmp[i];
			cumul_tmp[i] = cum;
		}
		cumul_tmp[n] = 0;
	}
	
	if(symbol < 0 || symbol >= n){
		printf("Error in symbol%d : %d ",index+1,symbol);
		exit(0);
	}

	freq_tmp[symbol] += 1;
	cumul_tmp[symbol] += 1;
	while(symbol>0)
		cumul_tmp[--symbol] += 1;
}




void FAP_Decoder::update_viseme(int symbol, int index,int **Vis_cumul_freq, int **Vis_freq,int scale) 
{
	int n;
	int *cumul_tmp, *freq_tmp;
//##	
	if( (index==0) || (index==1) )
		n = 2*14+1;
	else n = 2*quantize(63,scale)+1;
    
	cumul_tmp = Vis_cumul_freq[index];
    freq_tmp = Vis_freq[index];
	
	if (cumul_tmp[0] == Max_frequency) {
		int i, cum;
		cum = 0;
		
		for(i=n-1; i>=0; i--) {
			freq_tmp[i] = (freq_tmp[i]+1)/2;
			cum += freq_tmp[i];
			cumul_tmp[i] = cum;
		}
		cumul_tmp[n] = 0;
	}
	
	freq_tmp[symbol] += 1;
	cumul_tmp[symbol] += 1;
	while(symbol>0) {
		symbol -= 1;
		cumul_tmp[symbol] += 1;
	}
	
}



void FAP_Decoder::update_expression(int symbol, int index,int **Exp_cumul_freq, int **Exp_freq,int scale) 
{
	int n;
	int *cumul_tmp, *freq_tmp;
	
	if( (index==0) || (index==2) )
		n = 2*6+1;
	else n = 2*quantize(63,scale)+1;
    
	cumul_tmp = Exp_cumul_freq[index];
    freq_tmp = Exp_freq[index];
	
	if (cumul_tmp[0] == Max_frequency) {
		int i, cum;
		cum = 0;
		
		for(i=n-1; i>=0; i--) {
			freq_tmp[i] = (freq_tmp[i]+1)/2;
			cum += freq_tmp[i];
			cumul_tmp[i] = cum;
		}
		cumul_tmp[n] = 0;
	}
	
	freq_tmp[symbol] += 1;
	cumul_tmp[symbol] += 1;
	while(symbol>0) {
		symbol -= 1;
		cumul_tmp[symbol] += 1;
	}
	
}



void FAP_Decoder::init_arith(int *Irange_max,int *Irange_min, 
				int **Icumul_freq, int *range_max, 
				int *range_min, int **cumul_freq, int **freq, 
				int	**Exp_freq,int **Exp_cumul_freq,int **IExp_cumul_freq,
				int	**Vis_freq,int **Vis_cumul_freq,int **IVis_cumul_freq,
				int mask[NO_FAPS+1],int scale)
{
	int i, j, n;
	

	if(mask[0]) {
		// visemes values fields 0&1
		for(i=0;i<2;i++) {
			
			free(IVis_cumul_freq[i]);
//##			
			n = 14+1;
			IVis_cumul_freq[i] = (int *) calloc(n+1,sizeof(int));
			
			for(j=0; j<n; j++) 
				IVis_cumul_freq[i][j] = n-j;
			
			IVis_cumul_freq[i][n] = 0;			
		}
		
		for(i=0;i<2;i++) {
			
			free(Vis_freq[i]);
			free(Vis_cumul_freq[i]);
//##						
			n = 2*14+1;
			Vis_freq[i] = (int *) calloc(n,sizeof(int));
			Vis_cumul_freq[i] = (int *) calloc(n+1,sizeof(int));
			
			for(j=0; j<n; j++) {
				Vis_freq[i][j] = 1;
				Vis_cumul_freq[i][j] = n-j;
			}
			
			Vis_cumul_freq[i][n] = 0;
		}
		
		
		// viseme blend field
		
		free(IVis_cumul_freq[2]);
		
		n = quantize(63,scale)+1;
		IVis_cumul_freq[2] = (int *) calloc(n+1,sizeof(int));
		
		for(j=0; j<n; j++) 
			IVis_cumul_freq[2][j] = n-j;
		
		IVis_cumul_freq[2][n] = 0;
		
		
		free(Vis_freq[2]);
		free(Vis_cumul_freq[2]);
		
		n = 2*quantize(63,scale)+1;
		Vis_freq[2] = (int *) calloc(n,sizeof(int));
		Vis_cumul_freq[2] = (int *) calloc(n+1,sizeof(int));
		
		for(j=0; j<n; j++) {
			Vis_freq[2][j] = 1;
			Vis_cumul_freq[2][j] = n-j;
		}
		
		Vis_cumul_freq[2][n] = 0;

	}


	if(mask[1]) {
		// expression values fields 0&2
		for(i=0;i<3;i+=2) {
			
			free(IExp_cumul_freq[i]);
			
			n = 6+1;
			IExp_cumul_freq[i] = (int *) calloc(n+1,sizeof(int));
			
			for(j=0; j<n; j++) 
				IExp_cumul_freq[i][j] = n-j;
						
			IExp_cumul_freq[i][n] = 0;			
		}
		
		for(i=0;i<3;i+=2) {
			
			free(Exp_freq[i]);
			free(Exp_cumul_freq[i]);
			
			n = 2*6+1;
			Exp_freq[i] = (int *) calloc(n,sizeof(int));
			Exp_cumul_freq[i] = (int *) calloc(n+1,sizeof(int));
			
			for(j=0; j<n; j++) {
				Exp_freq[i][j] = 1;
				Exp_cumul_freq[i][j] = n-j;
			}
			
			Exp_cumul_freq[i][n] = 0;
		}


		// expressions intensities fields 1&3
		for(i=1;i<4;i+=2) {

			free(IExp_cumul_freq[i]);
			
			n = quantize(63,scale)+1;
			IExp_cumul_freq[i] = (int *) calloc(n+1,sizeof(int));
			
			for(j=0; j<n; j++) 
				IExp_cumul_freq[i][j] = n-j;

			IExp_cumul_freq[i][n] = 0;
		}

		for(i=1;i<4;i+=2) {

			free(Exp_freq[i]);
			free(Exp_cumul_freq[i]);

			n = 2*quantize(63,scale)+1;
			Exp_freq[i] = (int *) calloc(n,sizeof(int));
			Exp_cumul_freq[i] = (int *) calloc(n+1,sizeof(int));
			
			for(j=0; j<n; j++) {
				Exp_freq[i][j] = 1;
				Exp_cumul_freq[i][j] = n-j;
			}
			
			Exp_cumul_freq[i][n] = 0;
		}
	}


	for(i=2; i<NO_FAPS; i++) {
		if(mask[i]) {

			free(Icumul_freq[i]);
			
			n = Irange_max[i]-Irange_min[i]+1;

			Icumul_freq[i] = (int *) calloc(n+1,sizeof(int));
			
			for(j=0; j<n; j++)
				Icumul_freq[i][j] = n-j;
			
			Icumul_freq[i][n] = 0;
		}
	}
    
	for(i=2; i<NO_FAPS; i++) {
		if(mask[i]) {

			free(freq[i]);
			free(cumul_freq[i]);
			
			n = range_max[i]-range_min[i]+1;
			freq[i] = (int *) calloc(n,sizeof(int));
			cumul_freq[i] = (int *) calloc(n+1,sizeof(int));
			
			for(j=0; j<n; j++) {
				freq[i][j] = 1;
				cumul_freq[i][j] = n-j;
			}
			
			cumul_freq[i][n] = 0;
		}
	}
}



void FAP_Decoder::AdjustBits() 
{
	int bitin = 8-bitsin;

	position-=3;
	bitsin = 0;
	GetBits(bitin+2);
}



/***********************************************************************


  Huffman-Tables and decoding


***********************************************************************/




void FAP_Decoder::ReadHuffmanTables(int **run_table,int **dc_table,int **ac_table)
{
	FILE *fp_table;
	char word[BUFFER_SIZE];
	int counter,bitcounter;

	sprintf(word,"run.table");
	if( (fp_table  = fopen(word,"rb"))==NULL ) {
		printf("\n File - %s - does not exist. \n",word);
		exit(0);
	}

	for(counter=0;counter<16;counter++) {
		fscanf(fp_table,"%d",&run_table[counter][0]);
		
		for(bitcounter=1;bitcounter<=run_table[counter][0];bitcounter++) 
			fscanf(fp_table,"%d",&run_table[counter][bitcounter]);
		fgets(word,BUFFER_SIZE,fp_table);
	}

	fclose(fp_table);


	sprintf(word,"dc.table");
	if( (fp_table  = fopen(word,"rb"))==NULL ) {
		printf("\n File - %s - does not exist. \n",word);
		exit(0);
	}

	for(counter=0;counter<512;counter++) {
		fscanf(fp_table,"%d",&dc_table[counter][0]);
		
		for(bitcounter=1;bitcounter<=dc_table[counter][0];bitcounter++) 
			fscanf(fp_table,"%d",&dc_table[counter][bitcounter]);
		fgets(word,BUFFER_SIZE,fp_table);
	}

	fclose(fp_table);


	sprintf(word,"ac.table");
	if( (fp_table  = fopen(word,"rb"))==NULL ) {
		printf("\n File - %s - does not exist. \n",word);
		exit(0);
	}

	for(counter=0;counter<512;counter++) {
		fscanf(fp_table,"%d",&ac_table[counter][0]);
		
		for(bitcounter=1;bitcounter<=ac_table[counter][0];bitcounter++) 
			fscanf(fp_table,"%d",&ac_table[counter][bitcounter]);
		fgets(word,BUFFER_SIZE,fp_table);
	}

	

	fclose(fp_table);
}




int FAP_Decoder::GetHuffmanSymbol(int **table,int type)
{
	int symbol,bit;
	int size,no_bits,counter;
	int found = OFF;
	int *check;

	if(!type) size = 16;
	else size = 512;

	check = (int *)calloc(size,sizeof(int));
	
	no_bits = 1;

	while(!found) {

		bit = GetBits(1);
		
		for(counter=0;counter<size;counter++) {
			if(!check[counter]) {

				if( (table[counter][0]==no_bits) && (table[counter][no_bits]==bit) ) {
					symbol = counter;
					found=ON;
					break;
				}
				else if( table[counter][no_bits]!=bit )
					check[counter]=ON;
			}
		}
		no_bits++;
	}

	free(check);
	return symbol;
}





/***********************************************************************

 
  Common parts of Bit-stream Decoder
  
   
***********************************************************************/




void FAP_Decoder::NextStartCode() 
{
	/*GetBits(1);*/
	bitsin = 0;
}



int FAP_Decoder::Get_Face_Object_Start_Code()
{
	int dec_sym;
	
	dec_sym = GetBits(32);
	
	if(dec_sym != FACE_OBJECT_START_CODE) {
		printf("\n Wrong FBA-Sequence start code\n");
		return DEC_ERROR;
	}  
	
	return ON;
}




int FAP_Decoder::Check_Face_Paramset_Mask(int face_paramset_mask)
{
	if(face_paramset_mask!=FAP) {
		printf("\n Only FA-frames decoding implemented \n");
		return DEC_ERROR;
	}  
	return ON;
}




void  FAP_Decoder::DeCode_Frame_Rate(float *frm_rate)
{
	int frame_rate,seconds,frequency_offset;
	
	frame_rate = GetBits(8);
		
	seconds = GetBits(4);
	
	frequency_offset = GetBits(1);    	
	
	*frm_rate = (float)1000/(1000+frequency_offset)*(float)(frame_rate+(float)seconds/16);
}




void  FAP_Decoder::DeCode_Time_Code()
{
	int time_code_hours,time_code_minutes,marker_bit,time_code_seconds;
	
	time_code_hours   = GetBits(5);
	
	time_code_minutes = GetBits(6);
	
	marker_bit  = GetBits(1);
	
	time_code_seconds = GetBits(6);	
}




void FAP_Decoder::DeCode_Skip_Frames(int *no_frames_skip)
{
	int nfs,value;
	
	nfs = GetBits(4);
	
	if(nfs<15) {
		*no_frames_skip = nfs;
		return;
	}
	
	value = nfs;
	
	do {
		value = GetBits(4);
		nfs+=value;
	}
	while (value == 15);
	
	if(nfs>254) {
		printf("\n Maximum allowable frames skip is 254\n");
		exit(-1);
	}
	
	*no_frames_skip = nfs;
}



void FAP_Decoder::Decode_FAP_Mask( )
{
	int counter,fap_mask_type,mask;

	for(counter=0;counter<NO_FAPS;counter++)
		faps_mask[counter] = 0;  
    
	for(counter=0;counter<GROUP_MASK_SIZE;counter++) {
		
		GetBits(1);
		
		fap_mask_type = GetBits(2);
		
		switch(fap_mask_type) {
			
		case FAP_MASK_TYPE0:
			break;
			
		case FAP_MASK_TYPE1:
			for(mask=GroupMaskLimits[counter];mask<GroupMaskLimits[counter+1];mask++) 
				faps_mask[mask] = GetBits(1);
			break;
			
		case FAP_MASK_TYPE2:
			for(mask=GroupMaskLimits[counter];mask<GroupMaskLimits[counter+1];mask++) 
				faps_mask[mask] = GetBits(1);
			break;
			
		case FAP_MASK_TYPE3:
			for(mask=GroupMaskLimits[counter];mask<GroupMaskLimits[counter+1];mask++)
				faps_mask[mask]=ON;
			break;
		}
	}
}




int FAP_Decoder::Check_Face_Object_End()
{
	int  b0,b1,b2,b3;

	int pos = position;
	if(bitsin)pos--;

	b0 = stream_buffer[pos+1];
	b1 = stream_buffer[pos+2];
	b2 = stream_buffer[pos+3];
	b3 = stream_buffer[pos+4];
	
	if( b0==0 && b1==0 && b2==1 && b3==3 ) return OFF;
	else if( b0==0 && b1==0 && (b2==1 || b2==0) ) return ON;
		
	return OFF;
}



int FAP_Decoder::Check_Face_Object_Plane_Start_Code()
{
	int   b0,b1,b2,b3;

	int pos = position;
	if(bitsin)pos--;

	b0 = stream_buffer[pos+1];
	b1 = stream_buffer[pos+2];
	b2 = stream_buffer[pos+3];
	b3 = stream_buffer[pos+4];

	if( b0==0 && b1==0 && b2==1 && b3==3 ) {
		position = pos+5;
		bitsin = 0;
	}

	return ON;
}


int FAP_Decoder::Check_If_Face_Object_Plane_Start_Code()
{
	int   b0,b1,b2,b3;

	int pos = position;
	if(bitsin)pos--;

	b0 = stream_buffer[pos+1];
	b1 = stream_buffer[pos+2];
	b2 = stream_buffer[pos+3];
	b3 = stream_buffer[pos+4];

	if( b0==0 && b1==0 && b2==1 && b3==3 ) return ON;

	return OFF;
}




/****************************************************************************


	Frame Based Decoding


*****************************************************************************/


void FAP_Decoder::ResetIntra(int *quant_steps,int fap_quant)
{
	int counter;
	
	for(counter=0;counter<NO_FAPS;counter++) {
		faps[counter] = 0;
		quant_steps[counter] = fap_quant*FapQuantSteps[counter];
	}
}




int FAP_Decoder::CheckGroupBegin(int pos)
{
	int counter;

	for(counter=0;counter<11;counter++) {
		if(GroupMaskLimits[counter]==pos) return ON;
	}
	
	return OFF;
}



void FAP_Decoder::DeCode_New_Minmax(	int is_I_new_max,int is_I_new_min,int is_P_new_max,
					   int is_P_new_min,int *i_new_max,int *i_new_min,int *p_new_max,
					   int *p_new_min,int *Irange_max,int  *Irange_min,int *range_max,
					   int *range_min)
{
	int counter,aux;
	
	if(is_I_new_max) {
		for(counter=2;counter<NO_FAPS;counter++) {  

			if(CheckGroupBegin(counter)) aux=0;

			if(!(aux&3)) {
				if(!GetBits(1)) {
					printf("WRONG MARKERBIT in CodeNewMinMax\n");
					exit(-1);
				}
			}
						
			if(faps_mask[counter]) {
				i_new_max[counter] = GetBits(5);
				Irange_max[counter] = RANGE_DELTA*i_new_max[counter];
			}
			
			aux++;
		}
	}	
	
	if(is_I_new_min) {
		for(counter=2;counter<NO_FAPS;counter++) {  
			
			if(CheckGroupBegin(counter)) aux=0;

			if(!(aux&3)) {
				if(!GetBits(1)) {
					printf("WRONG MARKERBIT in CodeNewMinMax\n");
					exit(-1);
				}
			}

			if(faps_mask[counter]) {
				i_new_min[counter] = GetBits(5);
				Irange_min[counter] = -RANGE_DELTA*i_new_min[counter];
			}

			aux++;
		}
	}
	
	if(is_P_new_max) {
		for(counter=2;counter<NO_FAPS;counter++) {  

			if(CheckGroupBegin(counter)) aux=0;

			if(!(aux&3)) {
				if(!GetBits(1)) {
					printf("WRONG MARKERBIT in CodeNewMinMax\n");
					exit(-1);
				}
			}

			if(faps_mask[counter]) {
				p_new_max[counter] = GetBits(5);
				range_max[counter] = RANGE_DELTA*p_new_max[counter];
			}

			aux++;
		}
	}
				
	if(is_P_new_min) {
		for(counter=2;counter<NO_FAPS;counter++) {  

			if(CheckGroupBegin(counter)) aux=0;

			if(!(aux&3)) {
				if(!GetBits(1)) {
					printf("WRONG MARKERBIT in CodeNewMinMax\n");
					exit(-1);
				}
			}

			if(faps_mask[counter]) {
				p_new_min[counter] = GetBits(5);
				range_min[counter] = -RANGE_DELTA*p_new_min[counter];
			}

			aux++;
		}
	}

}




int FAP_Decoder::Decode_Viseme(int is_intra, int **Vis_freq,int **Vis_cumul_freq,int **IVis_cumul_freq,
					int scale)
{
	static int vis_prev[3];
	int vis_diff;

	if(!decoder_reset())
		return OFF;	

	if(!scale) scale = 1; // lossless 

	if(is_intra) {
		
		fap_vis[0] = aa_decode(IVis_cumul_freq[0]);
		vis_prev[0] = fap_vis[0];
		fap_vis[1] = aa_decode(IVis_cumul_freq[1]);
		vis_prev[1] = fap_vis[1];
		fap_vis[2] = aa_decode(IVis_cumul_freq[2])*scale;
		vis_prev[2] = fap_vis[2];

	}
	else {

		vis_diff = aa_decode(Vis_cumul_freq[0]);
		update_viseme(vis_diff,0,Vis_cumul_freq,Vis_freq,scale);
		vis_diff-=14;
		fap_vis[0]+=vis_diff;
		vis_prev[0] = fap_vis[0];

		vis_diff = aa_decode(Vis_cumul_freq[1]);
		update_viseme(vis_diff,1,Vis_cumul_freq,Vis_freq,scale);
		vis_diff-=14;
		fap_vis[1]+=vis_diff;
		vis_prev[1] = fap_vis[2];

		vis_diff = aa_decode(Vis_cumul_freq[2]);
		update_viseme(vis_diff,2,Vis_cumul_freq,Vis_freq,scale);
		vis_diff-=quantize(63,scale);
		vis_diff*=scale;
		fap_vis[2]+=vis_diff;
		vis_prev[2] = fap_vis[3];

	}
	
	AdjustBits();

	fap_vis[3] = GetBits(1);


	return ON;
}





int FAP_Decoder::Decode_Expression(int is_intra, int **Exp_freq,int **Exp_cumul_freq,int **IExp_cumul_freq,
					  int scale)
{
	static int expr_prev[4];
	int expr_diff;


	if(!decoder_reset())
		return OFF;	

	if(!scale) scale = 1; // lossless 

	if(is_intra) {
		
		fap_expr[0] = aa_decode(IExp_cumul_freq[0]);
		expr_prev[0] = fap_expr[0];
		fap_expr[1] = aa_decode(IExp_cumul_freq[1]);
		expr_prev[1] = fap_expr[1]*scale;
		fap_expr[2] = aa_decode(IExp_cumul_freq[2]);
		expr_prev[2] = fap_expr[2];
		fap_expr[3] = aa_decode(IExp_cumul_freq[3]);
		expr_prev[3] = fap_expr[3]*scale;

	}
	else {

		expr_diff = aa_decode(Exp_cumul_freq[0]);
		update_expression(expr_diff,0,Exp_cumul_freq,Exp_freq,scale);
		expr_diff-=6;
		fap_expr[0]+=expr_diff;
		expr_prev[0] = fap_expr[0];

		expr_diff = aa_decode(Exp_cumul_freq[1]);
		update_expression(expr_diff,1,Exp_cumul_freq,Exp_freq,scale);
		expr_diff-=quantize(63,scale);
		expr_diff*=scale;
		fap_expr[1]+=expr_diff;
		expr_prev[1] = fap_expr[1];

		expr_diff = aa_decode(Exp_cumul_freq[2]);
		update_expression(expr_diff,2,Exp_cumul_freq,Exp_freq,scale);
		expr_diff-=6;
		fap_expr[2]+=expr_diff;
		expr_prev[2] = fap_expr[2];

		expr_diff = aa_decode(Exp_cumul_freq[3]);
		update_expression(expr_diff,3,Exp_cumul_freq,Exp_freq,scale);
		expr_diff-=quantize(63,scale);
		expr_diff*=scale;
		fap_expr[3]+=expr_diff;
		expr_prev[3] = fap_expr[3];

	}
	
	AdjustBits();

	fap_expr[4] = GetBits(1);
	fap_expr[5] = GetBits(1);


	return ON;
}




int FAP_Decoder::DeCode_IFap(int *fapQuantSteps,
				int *Irange_max, int *Irange_min, int **Icumul_freq, 
				int *range_max, int *range_min, 
				int **cumul_freq, int **freq,int **Exp_freq,int **Exp_cumul_freq,
				int **IExp_cumul_freq,int **Vis_freq,int **Vis_cumul_freq,
				int **IVis_cumul_freq, int scale)
{
	int counter;
	int fap_quant,dec_fap;
		
	if(faps_mask[0]){
		if(!Decode_Viseme(ON,Vis_freq,Vis_cumul_freq,IVis_cumul_freq,scale))
			return OFF;	
	}

	if(faps_mask[1]) {
		if(!Decode_Expression(ON,Exp_freq,Exp_cumul_freq,IExp_cumul_freq,scale))
			return OFF;	
	}
	
	if(!decoder_reset()) return OFF;
		
	for(counter=2;counter<NO_FAPS;counter++) {
		if(faps_mask[counter]) {
			fap_quant = aa_decode(Icumul_freq[counter]);
			if(fap_quant == DEC_ERROR) return OFF;
			else fap_quant+=Irange_min[counter];
			if(fapQuantSteps[counter]) dec_fap = fapQuantSteps[counter]*fap_quant;
			else dec_fap =  fap_quant;
			faps[counter] = dec_fap;
			
		}
	}

	AdjustBits();
	return ON;
}




int FAP_Decoder::DeCode_PFap(int *fapQuantSteps,
				int *Irange_max, int *Irange_min, int **Icumul_freq, 
				int *range_max, int *range_min, 
				int **cumul_freq, int **freq, int **Exp_freq,int **Exp_cumul_freq,
				int **IExp_cumul_freq,int **Vis_freq,int **Vis_cumul_freq,
				int **IVis_cumul_freq,int scale)
{
	int counter;
	int fap_quant,dec_fap;

	if(faps_mask[0]) {
		if(!Decode_Viseme(OFF,Vis_freq,Vis_cumul_freq,IVis_cumul_freq,scale))
			return OFF;	
	}
	if(faps_mask[1]) {
		if(!Decode_Expression(OFF,Exp_freq,Exp_cumul_freq,IExp_cumul_freq,scale))
			return OFF;	
	}

	if(!decoder_reset()) return OFF;
	
	for(counter=2;counter<NO_FAPS;counter++) {
		if(faps_mask[counter]) {
			fap_quant = aa_decode(cumul_freq[counter]);
			if(fap_quant == DEC_ERROR) return OFF;
			update(fap_quant,counter,range_max,range_min,cumul_freq,freq );
			fap_quant += range_min[counter];
			if(fapQuantSteps[counter]) 
				dec_fap =  fapQuantSteps[counter]*fap_quant + faps[counter];
			else dec_fap =  fap_quant + faps[counter];
			faps[counter] = dec_fap;
		}
	}

	AdjustBits();
	return ON;
}




void FAP_Decoder::Print_Dec_FAPfile(int last,char *fap_file)
{
	int    j;
	static int n =  0;
	static FILE *fp_dec;
	static first = ON;
	char   word[BUFFER_SIZE];
	
	if(first) {
		
		sprintf(word,"%sDEC.fap",fap_file);
		if( (fp_dec  = fopen(word,"wb"))==NULL ) {
			printf("\n File - %s - not possible to open. \n",word);
			return;
		}
		
		for(j=0;j<200;j++)
			fprintf(fp_dec," ");
		fprintf(fp_dec,"\n");
		
		first = OFF;
	}
	
	if(last) {
		rewind(fp_dec);
		sprintf(word,"%sDEC.fap",fap_file);
		fprintf(fp_dec,"2.0 %s %d %d ",word,(int)fps,n);	
		fclose(fp_dec);
		return;
	}

	for(j=0;j<NO_FAPS;j++)
		fprintf(fp_dec,"%d ",faps_mask[j]);
	fprintf(fp_dec,"\n%d",n);

	if(faps_mask[0])
		fprintf(fp_dec," %d %d %d %d ",fap_vis[0],fap_vis[1],fap_vis[2],fap_vis[3]);

	if(faps_mask[1])
		fprintf(fp_dec," %d %d %d %d %d %d ",fap_expr[0],fap_expr[1],fap_expr[2],fap_expr[3],
											 fap_expr[4],fap_expr[5]);

	for(j=2;j<NO_FAPS;j++) {
		if(faps_mask[j])
			fprintf(fp_dec," %d",(int)faps[j]);
	}

	n++;
	fprintf(fp_dec,"\n");
	
}




/****************************************************************************


	DCT Based Decoding


*****************************************************************************/


void FAP_Decoder::MakeIDCTMatrix(float **idct)
{
	float idct_coef;
	int   line,colunm;
	
	for(line=0; line<SEGMENT_LENGTH; line++) {
		
		if(line==0) 
			idct_coef = (float)(1.0/sqrt(2));
		else idct_coef = 1;
		
		for(colunm=0; colunm<SEGMENT_LENGTH; colunm++)
			idct[colunm][line] = (float)(sqrt(2.0/SEGMENT_LENGTH))*idct_coef*
							(float)(cos(PI*(float)(2*colunm+1)*line/2.0/SEGMENT_LENGTH));
	}
}



void FAP_Decoder::Obtain_IDCT(float *faps, float *faps_coeff, float **dct)
{
	int line,colunm;
	
	for (line=0; line<SEGMENT_LENGTH; line++) {
		faps_coeff[line]=0;
		for (colunm=0; colunm<SEGMENT_LENGTH; colunm++)
			faps_coeff[line] += dct[line][colunm]*faps[colunm];
	}
}



void FAP_Decoder::Dequantize(int dc_Q,float *dc_pred,float quant_step)
{
	*dc_pred = (float)dc_Q*quant_step;
}


					
void FAP_Decoder::DC_I_Decode(int *dc_Q)
{
	*dc_Q = ShortGetBits();

	if(*dc_Q == DC_MIN-1)
		*dc_Q = GetBits(32);
}
					




void FAP_Decoder::AC_Decode(float *faps_coeff, float quant_step, int **run_table,
			   int **ac_table)
{
	int *coeff_Q,counter;
	int run_length,run_counter,symbol;

	coeff_Q = (int *)calloc(SEGMENT_LENGTH,sizeof(int));


	run_length = 1;	
	run_counter = 0;

	while(run_length ) {
		 
		if( run_counter+1 >= SEGMENT_LENGTH )
			break;
		
		run_length = GetHuffmanSymbol(run_table,RUN);
		
		if(run_length!=0) {
			run_counter+=run_length;			
			symbol = GetHuffmanSymbol(ac_table,AC);
			if(symbol==256)
				DC_I_Decode(&coeff_Q[run_counter]);
			else coeff_Q[run_counter] = symbol-256;
		}
	}

	
	for(counter=1;counter<SEGMENT_LENGTH;counter++)
		Dequantize(coeff_Q[counter],&faps_coeff[counter],quant_step);	

	free(coeff_Q);
}


 
					
void FAP_Decoder::Predict(float dc,float dc_pred,float *dc_diff)
{
	*dc_diff = dc - dc_pred;
}
					 

					

void FAP_Decoder::Reconstruct(float dc_diff,float *dc_pred )
{
	*dc_pred +=dc_diff;
} 



void FAP_Decoder::DC_P_Decode(int *dc_Q,int **dc_table)
{

	*dc_Q = GetHuffmanSymbol(dc_table,DC);

	if(*dc_Q == 0)
		DC_I_Decode(dc_Q);
	else
		*dc_Q-=256;

}


void FAP_Decoder::DeCode_I_Viseme_Segment(int **fap_viseme)
{
	int counter;
	
	fap_viseme[0][0] = GetBits(5);
	fap_viseme[0][1] = GetBits(5);
	fap_viseme[0][2] = GetBits(7);
	fap_viseme[0][3] = GetBits(1);


	for(counter=1;counter<SEGMENT_LENGTH;counter++) {
		fap_viseme[counter][0] = GetBits(5)+fap_viseme[counter-1][0]-14;
		fap_viseme[counter][1] = GetBits(5)+fap_viseme[counter-1][1]-14;
		fap_viseme[counter][2] = GetBits(7)+fap_viseme[counter-1][2]-63;
		fap_viseme[counter][3] = GetBits(1);
	}

	for(counter=0;counter<3;counter++)
		last_viseme[counter] = fap_viseme[15][counter];
}



void FAP_Decoder::DeCode_P_Viseme_Segment(int **fap_viseme)
{
	int counter;
	
	fap_viseme[0][0] = GetBits(5) + last_viseme[0] - 14;
	fap_viseme[0][1] = GetBits(5) + last_viseme[1] - 14;
	fap_viseme[0][2] = GetBits(7) + last_viseme[2] - 63;
	fap_viseme[0][3] = GetBits(1);


	for(counter=1;counter<SEGMENT_LENGTH;counter++) {
		fap_viseme[counter][0] = GetBits(5)+fap_viseme[counter-1][0]-14;
		fap_viseme[counter][1] = GetBits(5)+fap_viseme[counter-1][1]-14;
		fap_viseme[counter][2] = GetBits(7)+fap_viseme[counter-1][2]-63;
		fap_viseme[counter][3] = GetBits(1);
	}

	for(counter=0;counter<3;counter++)
		last_viseme[counter] = fap_viseme[15][counter];
}



void FAP_Decoder::DeCode_I_Expression_Segment(int **fap_expression)
{
	int counter;
	
	fap_expression[0][0] = GetBits(4);
	fap_expression[0][2] = GetBits(4);
	fap_expression[0][1] = GetBits(7);
	fap_expression[0][3] = GetBits(7);
	fap_expression[0][4] = GetBits(1);
	fap_expression[0][5] = GetBits(1);


	for(counter=1;counter<SEGMENT_LENGTH;counter++) {
		fap_expression[counter][0] = GetBits(4)+fap_expression[counter-1][0]-6;
		fap_expression[counter][2] = GetBits(4)+fap_expression[counter-1][2]-6;
		fap_expression[counter][1] = GetBits(7)+fap_expression[counter-1][1]-63;
		fap_expression[counter][3] = GetBits(7)+fap_expression[counter-1][3]-63;
		fap_expression[counter][4] = GetBits(1);
		fap_expression[counter][5] = GetBits(1);
	}

	for(counter=0;counter<4;counter++)
		last_expression[counter] = fap_expression[15][counter];
}



void FAP_Decoder::DeCode_P_Expression_Segment(int **fap_expression)
{
	int counter;
	
	fap_expression[0][0] = GetBits(4) + last_expression[0] - 6;
	fap_expression[0][2] = GetBits(4) + last_expression[2] - 6;
	fap_expression[0][1] = GetBits(7) + last_expression[1] - 63;
	fap_expression[0][3] = GetBits(7) + last_expression[3] - 63;
	fap_expression[0][4] = GetBits(1);
	fap_expression[0][5] = GetBits(1);


	for(counter=1;counter<SEGMENT_LENGTH;counter++) {
		fap_expression[counter][0] = GetBits(4)+fap_expression[counter-1][0]-6;
		fap_expression[counter][2] = GetBits(4)+fap_expression[counter-1][2]-6;
		fap_expression[counter][1] = GetBits(7)+fap_expression[counter-1][1]-63;
		fap_expression[counter][3] = GetBits(7)+fap_expression[counter-1][3]-63;
		fap_expression[counter][4] = GetBits(1);
		fap_expression[counter][5] = GetBits(1);
	}

	for(counter=0;counter<4;counter++)
		last_expression[counter] = fap_expression[15][counter];
}




void FAP_Decoder::DeCode_ISegment( int *fap_mask, float **faps, int	**fap_expression,int **fap_viseme,
					 float *fapQuantSteps,float *dc_pred,float **idct, int **run_table,
					 int **ac_table)
{
	int counter;
	int group_number;

	float **faps_coeff;
	int   dc_Q;

	faps_coeff = FloatMatrixAllocation(NO_FAPS,SEGMENT_LENGTH);
	
	
	for(group_number=0;group_number<10;group_number++) {
		
		if( group_number+1 == 1)  {
			if(fap_mask[1]) 
				DeCode_I_Expression_Segment(fap_expression);
			if(fap_mask[0]) 
				DeCode_I_Viseme_Segment(fap_viseme);
		}
		
		else {
			
			for(counter=GroupMaskLimits[group_number];
				counter<GroupMaskLimits[group_number+1];counter++) 

				if(fap_mask[counter]) {

					DC_I_Decode(&dc_Q);
					Dequantize(dc_Q,&faps_coeff[counter][0],fapQuantSteps[counter]/3);
					AC_Decode(faps_coeff[counter],fapQuantSteps[counter],
								run_table,ac_table);
					
					Obtain_IDCT(faps_coeff[counter],faps[counter],idct);
				}
		}
	}

	for(counter = 0; counter<NO_FAPS; counter++)
		dc_pred[counter] = faps_coeff[counter][0];

	MatrixDealoc(faps_coeff,NO_FAPS);
}




void FAP_Decoder::DeCode_PSegment(int *fap_mask,float **faps,int	**fap_expression,int **fap_viseme,
					 float *fapQuantSteps,float *dc_pred,float **idct,int **run_table,
					 int **dc_table,int **ac_table)
{
	int counter;
	int group_number;

	float **faps_coeff;
	int   dc_Q;
	float *dc_diff;

	faps_coeff = FloatMatrixAllocation(NO_FAPS,SEGMENT_LENGTH);
	dc_diff    = (float *)calloc(NO_FAPS,sizeof(float));
	

	for(group_number=0;group_number<10;group_number++) {
		
		if( group_number+1 == 1 ) {
			if(fap_mask[1]) 
				DeCode_P_Expression_Segment(fap_expression);
			if(fap_mask[0]) 
				DeCode_P_Viseme_Segment(fap_viseme);
		}
		
		else {
			
			for(counter=GroupMaskLimits[group_number];
				counter<GroupMaskLimits[group_number+1];counter++) 

				if(fap_mask[counter]) {

					DC_P_Decode(&dc_Q,dc_table);	

					Dequantize(dc_Q,&dc_diff[counter],fapQuantSteps[counter]/3);
					
					Reconstruct(dc_diff[counter],&dc_pred[counter]);
					
					faps_coeff[counter][0] = dc_pred[counter];

					AC_Decode(faps_coeff[counter],fapQuantSteps[counter],run_table,ac_table);
					
					Obtain_IDCT(faps_coeff[counter],faps[counter],idct);
				}
		}
		
	}

	MatrixDealoc(faps_coeff,NO_FAPS);
	free(dc_diff);
}



void FAP_Decoder::ResetDCTIntra(float **faps,int **fap_expression,float *dctquant_steps,int fap_quant_index)
{
	int line,colunm,counter;

	for(line=0;line<NO_FAPS;line++) {
		for(colunm=0;colunm<SEGMENT_LENGTH;colunm++)
			faps[line][colunm]=0;
	}

	for(line=0;line<SEGMENT_LENGTH;line++) {
		for(colunm=0;colunm<6;colunm++)
			fap_expression[line][colunm]=0;
	}

	for(counter=0;counter<NO_FAPS;counter++) 
		dctquant_steps[counter] = fap_scale[fap_quant_index]*DCTQuantSteps[counter];

}


void FAP_Decoder::Print_DCTDec_FAPfile(float **faps,int	**fap_expression,int **fap_viseme,int *fap_mask, 
						  int last,char *fap_file,int fps)
{
	int    i,j;
	static int n =  0;
	static FILE *fp_dec;
	static first = ON;
	char   word[BUFFER_SIZE];

	if(last) {
		rewind(fp_dec);
		sprintf(word,"%sDCTDEC.fap",fap_file);
		fprintf(fp_dec,"2.0 %s %d %d ",word,fps,n);	
		fclose(fp_dec);
		return;
	}
	
	if(first) {
		
		sprintf(word,"%sDCTDEC.fap",fap_file);
		if( (fp_dec  = fopen(word,"wb"))==NULL ) {
			printf("\n File - %s - not possible to open. \n",word);
			return;
		}
		
		for(i=0;i<200;i++)
			fprintf(fp_dec," ");
		fprintf(fp_dec,"\n");

		first = OFF;
	}

	for(i=0;i<16;i++) {

		for(j=0;j<NO_FAPS;j++)
			fprintf(fp_dec,"%d ",fap_mask[j]);
		fprintf(fp_dec,"\n%d",n);
		
		if(fap_mask[0]) {
			for(j=0;j<4;j++)
				fprintf(fp_dec," %d",fap_viseme[i][j]);
		}

		if(fap_mask[1]) {
			for(j=0;j<6;j++)
				fprintf(fp_dec," %d",fap_expression[i][j]);
		}

		for(j=2;j<NO_FAPS;j++)
			if(fap_mask[j])
				fprintf(fp_dec," %d",(int)faps[j][i]);
			n++;
			fprintf(fp_dec,"\n");
	}
}




/******************************************************************************


	Decode_Face_Object_Plane()


******************************************************************************/

static int face_object_coding_type;

int FAP_Decoder::Decode_Face_Object_Plane(int *no_frames_skip,int *quant_steps,
							 int *is_rate,
							 int *i_new_max,int *i_new_min,int *p_new_max,
							 int *p_new_min,int *Irange_max,int *Irange_min,
							 int *range_max,int *range_min,int **freq,
							 int **cumul_freq,int **Icumul_freq,
							 int **Exp_freq,int **Exp_cumul_freq,int **IExp_cumul_freq,
							 int **Vis_freq,int **Vis_cumul_freq,int **IVis_cumul_freq,
							 float **dctfaps,int **dctfap_expression,int **dctfap_viseme,
							 float *dctquant_steps, float *dc_pred,float **idct,
							 int **run_table,int **dc_table,int **ac_table,
							 char *fap_file,int file)
{	
	int   counter;
	int   face_paramset_mask;
	int   is_time_code,is_frame_rate;
	int   skip_frames;
	int   is_I_new_max,is_I_new_min,is_P_new_max,is_P_new_min;
	int   is_intra;

	int fap_quant;

	if( Check_Face_Object_Plane_Start_Code() ) {
		
		is_intra = GetBits(1);

		face_paramset_mask = GetBits(2);
		
		if(Check_Face_Paramset_Mask(face_paramset_mask)==DEC_ERROR)
			return DEC_ERROR;

		if(is_intra) {
			is_frame_rate = GetBits(1);
			*is_rate = is_frame_rate;
			if (is_frame_rate) 
				DeCode_Frame_Rate(&fps);
			
			is_time_code = GetBits(1);
			if (is_time_code) 
				DeCode_Time_Code();
		}

		skip_frames = GetBits(1);
		if (skip_frames) 
			DeCode_Skip_Frames(no_frames_skip);
	
	
		if( face_paramset_mask == FAP) {
		
			if(is_intra) {

				fap_quant = GetBits(5);

				Decode_FAP_Mask();

				gender = GetBits(1);
				face_object_coding_type = GetBits(1);
				

				if(face_object_coding_type == ARITH_TYPE) {
	
					ResetIntra(quant_steps,fap_quant);
					
					is_I_new_max = GetBits(1);
					is_I_new_min = GetBits(1);
					is_P_new_max = GetBits(1);
					is_P_new_min = GetBits(1);
					
					
					DeCode_New_Minmax(is_I_new_max,is_I_new_min,is_P_new_max,is_P_new_min,
						i_new_max,i_new_min, p_new_max,p_new_min,
						Irange_max,Irange_min,range_max,range_min);	
					
					
					if(is_I_new_max || is_I_new_min || is_P_new_max || is_P_new_min) {
						for(counter=2; counter<NO_FAPS; counter++) {
							if (faps_mask[counter]) {
								if(is_I_new_max) Irange_max[counter] = 
									RANGE_DELTA*i_new_max[counter]/fap_quant;
								if(is_I_new_min) Irange_min[counter] = 
									-RANGE_DELTA*i_new_min[counter]/fap_quant;
								if(is_P_new_max) range_max[counter]  = 
									RANGE_DELTA*p_new_max[counter]/fap_quant;
								if(is_P_new_min) range_min[counter]  = 
									-RANGE_DELTA*p_new_min[counter]/fap_quant;
							}
						}
					}

				init_arith(Irange_max,Irange_min,Icumul_freq,range_max,range_min,cumul_freq,
							freq,Exp_freq,Exp_cumul_freq,IExp_cumul_freq,
							Vis_freq,Vis_cumul_freq,IVis_cumul_freq,faps_mask,fap_quant);
				
				if(!DeCode_IFap(quant_steps,Irange_max,Irange_min,Icumul_freq,range_max,
								range_min,cumul_freq,freq,Exp_freq,Exp_cumul_freq,
								IExp_cumul_freq,Vis_freq,Vis_cumul_freq,
								IVis_cumul_freq,fap_quant))
					return DEC_ERROR;
				
				}
				else if(face_object_coding_type == DCT_TYPE) {

					ResetDCTIntra(dctfaps,dctfap_expression,dctquant_steps,fap_quant);

					DeCode_ISegment(faps_mask,dctfaps,dctfap_expression,dctfap_viseme,
									dctquant_steps,dc_pred,idct,run_table,ac_table);
										
				}
				else {
					printf("\n Unknown coding type!");
					return DEC_ERROR;
				}
	
			} /* Intra-frame decoding */

			else {
				if(face_object_coding_type == ARITH_TYPE) {

					if(!DeCode_PFap(quant_steps,Irange_max,Irange_min,
						Icumul_freq,range_max,range_min,cumul_freq,freq,
						Exp_freq,Exp_cumul_freq,IExp_cumul_freq,
						Vis_freq,Vis_cumul_freq,IVis_cumul_freq,fap_quant))
						return DEC_ERROR;
				}
				else if(face_object_coding_type == DCT_TYPE) {

					DeCode_PSegment(faps_mask,dctfaps,dctfap_expression,dctfap_viseme,
									dctquant_steps,dc_pred,idct,run_table,dc_table,ac_table);
				}
			} /* Inter-frame decoding */

		} // fba paramset mask

	}
	
	if(file) {
		if(face_object_coding_type == ARITH_TYPE) 
			Print_Dec_FAPfile(OFF,fap_file);
		else if(face_object_coding_type == DCT_TYPE)
			Print_DCTDec_FAPfile(dctfaps,dctfap_expression,dctfap_viseme,faps_mask,OFF,
			fap_file,(int)fps);
	}
    return Check_Face_Object_End();
}




								   								   

void FAP_Decoder::Decode_Face_Object(char *fap_file)
{
	int  frame_number,segment_counter;
	
	int  no_frames_skip;	
	int  is_frame_rate;

	frame_number = 0;
	segment_counter = 0;

	if(firstframe) {
		Get_Face_Object_Start_Code();
		firstframe = OFF;
	}

	/* decode face object plane class */
	
	do {                
		end_code=Decode_Face_Object_Plane(&no_frames_skip,
			quant_steps,&is_frame_rate,i_new_max,i_new_min, p_new_max,p_new_min,
			Irange_max,Irange_min,range_max,range_min,freq,cumul_freq,
			Icumul_freq,Exp_freq,Exp_cumul_freq,IExp_cumul_freq,
			Vis_freq,Vis_cumul_freq,IVis_cumul_freq,
			dctfaps,dctfap_expression,dctfap_viseme,dctquant_steps,dc_pred,idct,run_table,
			dc_table,ac_table,fap_file,ON);


		if(end_code!=DEC_ERROR) {	
			if(face_object_coding_type == ARITH_TYPE) {
				frame_number++;
				printf("\n - Frame %d decoded - ",frame_number);
			}
			else if(face_object_coding_type == DCT_TYPE) {
				segment_counter++;
				printf("\n - Segment %d decoded - ",segment_counter);
			}
		}
		
	} /* FA frame decoding */
	while(!end_code); /* FBA Sequence decoding */
	
	if(face_object_coding_type == ARITH_TYPE)
		Print_Dec_FAPfile(ON,fap_file);
	else if(face_object_coding_type == DCT_TYPE)
		Print_DCTDec_FAPfile(dctfaps,dctfap_expression,dctfap_viseme,faps_mask,ON,fap_file,
							(int)fps);

	if(end_code!=DEC_ERROR) printf("\n   - END of FBA-sequence - \n\n");
	else printf("\n\n ---> Error occured, exiting. <--- \n\n\n");	
}





int FAP_Decoder::Decode_FA_Frame(unsigned char *AUptr, 
								  int bitoffset,
								  int AUbitcount,
								  FOP_DATA *fop_data, 
								  int fba_bytecount, 
								  unsigned int *PTS)
{
	int  no_frames_skip;	
	int  is_frame_rate;

	position = 0;
	bitsin   = 0;

	stream_buffer = AUptr;
	GetBits(bitoffset);
	
	if(firstframe) {
		if(!Get_Face_Object_Start_Code()) return OFF;
		firstframe = OFF;
	}

	/* decode face object plane class */
	
	end_code=Decode_Face_Object_Plane(&no_frames_skip,
		quant_steps,&is_frame_rate,i_new_max,i_new_min, p_new_max,p_new_min,
		Irange_max,Irange_min,range_max,range_min,freq,cumul_freq,
		Icumul_freq,Exp_freq,Exp_cumul_freq,IExp_cumul_freq,
		Vis_freq,Vis_cumul_freq,IVis_cumul_freq,
		dctfaps,dctfap_expression,dctfap_viseme,dctquant_steps,dc_pred,idct,run_table,
		dc_table,ac_table,NULL,OFF);
	

	for(int i=0;i<68;i++) {
		fop_data->fap_mask[i] = faps_mask[i];
		fop_data->fap[i] = faps[i];
	}

	fop_data->viseme_1 = 0;
	fop_data->viseme_2 = 0;
	fop_data->viseme_blend = 0;
	fop_data->viseme_def = 0;
	fop_data->expression_1 = 0;
	fop_data->expression_1_intensity = 0;
	fop_data->expression_2 = 0;
	fop_data->expression_2_intensity = 0;
	fop_data->init_face = 0;
	fop_data->expression_def = 0;		
	fop_data->gender = gender;		
	
	*PTS = 0; // TBD!!

	if(Check_If_Face_Object_Plane_Start_Code() && bitsin) bitsin=8;

	if( (end_code==DEC_ERROR) || end_code) return OFF;
	else return (position*8 - bitsin - bitoffset); // number of bits read	

}




/***********************************************************************

***********************************************************************/

FILE *OpenStreamFile(char *stream_file)
{
	char  word[BUFFER_SIZE];
	FILE *fp;

	sprintf(word,"%s",stream_file);
	if( (fp = fopen(word,"rb"))==NULL ) {
		printf("\n File %s does not exist.\n",word);
		exit(0);
	}
	return fp;
}



int GetSize(FILE *fp)
{
	int size=0;
	
	while(getc(fp)!=EOF)size++;
	fseek(fp,0,SEEK_SET);

	return size;
}


void FillBuffer(FILE *fp,unsigned char *stream_buffer)
{
	int pos=0;
	while( (ungetc(getc(fp),fp))!=EOF) stream_buffer[pos++]=getc(fp);
}

/***********************************************************************

  
	Main
	
	  
***********************************************************************/


int main(int argc,char *argv[])
{
	if ( argc!=2 ) {
		printf("Usage:  <ISTFAPdecoder> <location+fapfilename>  \n");
		return 0;
	}
	
	FILE *fp = OpenStreamFile(argv[1]);	
	
	int	size = GetSize(fp);
	unsigned char *stream_buffer = (unsigned char *)calloc (size,sizeof(unsigned char));
	FillBuffer(fp,stream_buffer);

#ifndef IM1
	FAP_Decoder *FA_Dec = new FAP_Decoder(stream_buffer);
	FA_Dec->Decode_Face_Object(argv[1]);
#else 
	FAP_Decoder *FA_Dec = new FAP_Decoder();
	FOP_DATA *fop_data = (FOP_DATA *)calloc(1,sizeof(FOP_DATA));
	unsigned char *tmp = stream_buffer;

	unsigned int ts;
	int bitoffset = 0;
	int i=0;
	int bit_sum=0;

	int bits = FA_Dec->Decode_FA_Frame(	stream_buffer,bitoffset,
										size,fop_data,sizeof(FOP_DATA),&ts);
	printf("\nFA-frame %d decoded\n",i++);

	while(bits) {

		bit_sum+=bits;
		bitoffset = bit_sum%8;
		stream_buffer = tmp + bit_sum/8;

		bits = FA_Dec->Decode_FA_Frame(stream_buffer,bitoffset,
										size-=bits,fop_data,sizeof(FOP_DATA),&ts);

		printf("FA-frame %d decoded\n",i++);
	}

	free(fop_data);
	free(tmp);

#endif

	delete FA_Dec;
	fclose(fp);

	return 0;
}

