/********************************************************************/
/* 																	*/
/* This software module was originally developed and edited by		*/
/* Gabriel Antunes Abrantes (Instituto Superior Tcnico)			*/	
/* 																	*/
/* in the course of development of the MPEG-4 standard.	 			*/
/* This software module is an implementation of a part of one or 	*/
/* more MPEG-4 tools as specified by the	 						*/
/* MPEG-4 standard. ISO/IEC gives users of the MPEG-4 standard		*/
/* free license to this software module or modifications thereof 	*/
/* for use in hardware or software products claiming conformance 	*/
/* to the MPEG-4 standard. Those intending to use this software 	*/
/* module in hardware or software products are advised that its 	*/
/* use may infringe existing patents. The original developer of		*/
/* this software module and his company, the subsequent 			*/
/* editors and their companies, and ISO/IEC have no liability		*/
/* for use of this software module or modifications thereof in 		*/
/* an implementation. Copyright is not released for non				*/
/* MPEG-4 standard conforming products. ACTS-MoMuSys partners	 	*/
/* retain full right to use the code for their own purposes, 		*/
/* assign or donate the code to a third party and to inhibit 		*/
/* third parties from using the code for non MPEG-4 standard		*/
/* conforming products. This copyright notice must be included 		*/
/* in all copies or derivative works. 								*/
/* Copyright (c)1997 												*/
/* 																	*/
/********************************************************************/







/***********************************HeaderBegin****************************
*   File : ISTFAPdecoder.h
*   
*   Author : Gabriel Antunes Abrantes(gabriel.abrantes@lx.it.pt)
*   Created : 25-July-1997 
*
*   Description : Face Object Decoder
*
*   Modified:
*   Version 4.0 / February-1997
*   Updated for  MPEG-4 Tokyo Reunion, includes frame based and DCT 
*						based coding modes
*
****************************************HeaderEnd**************************/

#if !defined(_fap_decoder_h)	// Sentry, use file only if it's not already included.
#define _fap_decoder_h


#include "fadecoder_defs.h"

typedef struct {
	unsigned char fap_mask[68];
	int viseme_1, viseme_2, viseme_blend, viseme_def;
	int expression_1, expression_1_intensity;
	int expression_2, expression_2_intensity;
	int init_face, expression_def;
	int fap[68];
	int gender;
} FOP_DATA;



class FAP_Decoder
{
	
public:
	
	FAP_Decoder(
#ifndef IM1
		unsigned char *stbuffer
#endif 						 
		);
	~FAP_Decoder();
	void Decode_Face_Object(char *fap_file);

	int FAP_Decoder::Decode_FA_Frame(unsigned char *AUptr, 
								  int bitoffset,
								  int AUbitcount,
								  FOP_DATA *fop_data, 
								  int fba_bytecount, 
								  unsigned int *PTS);
private:
	
	int   faps[NO_FAPS+1];
	int   faps_mask[NO_FAPS+1];
	float fps;
	int	  fap_expr[6];
	int	  fap_vis[4];
		
	int	 end;
	int  position;
	int  gender;

	int  firstframe;

	int  *quant_steps;
	int  end_code;
	
	int  *i_new_max,*i_new_min,*p_new_max,*p_new_min;
	int  *Irange_max, *Irange_min,*range_max, *range_min;
	int  **freq,**cumul_freq,**Icumul_freq;
	int	 **Exp_freq,**Exp_cumul_freq,**IExp_cumul_freq;		
	int	 **Vis_freq,**Vis_cumul_freq,**IVis_cumul_freq;		

	float **dctfaps;
	float *dctquant_steps;
	float **idct;
	float *dc_pred;
	int   **dc_table,**ac_table,**run_table;	
	int	  **dctfap_expression,**dctfap_viseme;


	unsigned char *stream_buffer;
	
	float **FloatMatrixAllocation(int vs,int hs);
	void MatrixDealoc(float **matrix,int vs);
	int **IntMatrixAllocation(int vs,int hs);
	void IntMatrixDealoc(int **matrix,int vs);
	int quantize(int fap, int quantStep) ;
	int bitsin;
	
	
	int GetBits(int no_bits) ;
	int ShortGetBits() ;
	long low, high, code_value, Bit, length, sacindex, cum;
	
	int zerocount;	
	int bit_out_psc_layer();
	int decoder_reset();
	int aa_decode(int cumul_freq[ ]);
	void update(int symbol, int index,int *range_max, int *range_min,int **cumul_freq, int **freq);
	void update_viseme(int symbol, int index,int **Vis_cumul_freq, int **Vis_freq,int scale);
	void update_expression(int symbol, int index,int **Exp_cumul_freq, int **Exp_freq,int scale);

	void init_arith(int *Irange_max,int *Irange_min, 
		int **Icumul_freq, int *range_max, 
		int *range_min, int **cumul_freq, int **freq, 
		int	**Exp_freq,int **Exp_cumul_freq,int **IExp_cumul_freq,
		int	**Vis_freq,int **Vis_cumul_freq,int **IVis_cumul_freq,
		int mask[NO_FAPS+1],int scale);
	void AdjustBits();
	
	void ReadHuffmanTables(int **run_table,int **dc_table,int **ac_table);
	int GetHuffmanSymbol(int **table,int type);
	
	
	
	//  Common parts of Bit-stream Decoder
	
	void NextStartCode() ;
	int  Get_Face_Object_Start_Code();
	int  Check_Face_Paramset_Mask(int face_paramset_mask);
	void DeCode_Frame_Rate(float *frm_rate);
	
	void DeCode_Time_Code();
	void DeCode_Skip_Frames(int *no_frames_skip);
	void Decode_FAP_Mask();
	int  Check_Face_Object_End();
	int  Check_Face_Object_Plane_Start_Code();
	int  Check_If_Face_Object_Plane_Start_Code();
	
	//	Frame Based Decoding
	
	void ResetIntra(int *quant_steps,int fap_quant);
	int  CheckGroupBegin(int pos);
	void DeCode_New_Minmax(	int is_I_new_max,int is_I_new_min,int is_P_new_max,
					   int is_P_new_min,int *i_new_max,int *i_new_min,int *p_new_max,
					   int *p_new_min,int *Irange_max,int  *Irange_min,int *range_max,
					   int *range_min);
	int Decode_Viseme(int is_intra, int **Vis_freq,int **Vis_cumul_freq,int **IVis_cumul_freq,
		int scale);
	int Decode_Expression(int is_intra, int **Exp_freq,int **Exp_cumul_freq,int **IExp_cumul_freq,
		int scale);
	int DeCode_IFap(int *fapQuantSteps,
		int *Irange_max, int *Irange_min, int **Icumul_freq, 
		int *range_max, int *range_min, 
		int **cumul_freq, int **freq,int **Exp_freq,int **Exp_cumul_freq,
		int **IExp_cumul_freq,int **Vis_freq,int **Vis_cumul_freq,
		int **IVis_cumul_freq, int scale);
	int DeCode_PFap(int *fapQuantSteps,
		int *Irange_max, int *Irange_min, int **Icumul_freq, 
		int *range_max, int *range_min, 
		int **cumul_freq, int **freq, int **Exp_freq,int **Exp_cumul_freq,
		int **IExp_cumul_freq,int **Vis_freq,int **Vis_cumul_freq,
		int **IVis_cumul_freq,int scale);
	void Print_Dec_FAPfile(int last,char *fap_file);
	
	
	//	DCT Based Decoding
	
	void MakeIDCTMatrix(float **idct);
	void Obtain_IDCT(float *faps, float *faps_coeff, float **dct);
	void Dequantize(int dc_Q,float *dc_pred,float quant_step);
	void DC_I_Decode(int *dc_Q);
	void AC_Decode(float *faps_coeff, float quant_step, int **run_table,int **ac_table);
	void Predict(float dc,float dc_pred,float *dc_diff);
	void Reconstruct(float dc_diff,float *dc_pred );
	void DC_P_Decode(int *dc_Q,int **dc_table);
	
	int last_viseme[3];
		
	void DeCode_I_Viseme_Segment(int **fap_viseme);
	void DeCode_P_Viseme_Segment(int **fap_viseme);
	
	int last_expression[4];
	
	void DeCode_I_Expression_Segment(int **fap_expression);
	void DeCode_P_Expression_Segment(int **fap_expression);
	
	
	void DeCode_ISegment( int *fap_mask, float **faps, int	**fap_expression,int **fap_viseme,
		float *fapQuantSteps,float *dc_pred,float **idct, int **run_table,
		int **ac_table);
	
	void DeCode_PSegment(int *fap_mask,float **faps,int	**fap_expression,int **fap_viseme,
		float *fapQuantSteps,float *dc_pred,float **idct,int **run_table,
		int **dc_table,int **ac_table);

	void ResetDCTIntra(float **faps,int **fap_expression,float *dctquant_steps,int fap_quant_index);
	void Print_DCTDec_FAPfile(float **faps,int	**fap_expression,int **fap_viseme,int *fap_mask, 
						  int last,char *fap_file,int fps);
	
	
	//	Decode_Face_Object_Plane()
	
	int face_object_coding_type;
	int Decode_Face_Object_Plane(int *no_frames_skip,int *quant_steps,
							 int *is_rate,
							 int *i_new_max,int *i_new_min,int *p_new_max,
							 int *p_new_min,int *Irange_max,int *Irange_min,
							 int *range_max,int *range_min,int **freq,
							 int **cumul_freq,int **Icumul_freq,
							 int **Exp_freq,int **Exp_cumul_freq,int **IExp_cumul_freq,
							 int **Vis_freq,int **Vis_cumul_freq,int **IVis_cumul_freq,
							 float **dctfaps,int **dctfap_expression,int **dctfap_viseme,
							 float *dctquant_steps, float *dc_pred,float **idct,
							 int **run_table,int **dc_table,int **ac_table,
							 char *fap_file,int file);
};

#endif                                      // _fap_decoder_h sentry.
