/********************************************************************/
/* 																	*/
/* This software module was originally developed and edited by		*/
/* Gabriel Antunes Abrantes (Instituto Superior Tcnico)			*/	
/* 																	*/
/* in the course of development of the MPEG-4 standard.	 			*/
/* This software module is an implementation of a part of one or 	*/
/* more MPEG-4 tools as specified by the	 						*/
/* MPEG-4 standard. ISO/IEC gives users of the MPEG-4 standard		*/
/* free license to this software module or modifications thereof 	*/
/* for use in hardware or software products claiming conformance 	*/
/* to the MPEG-4 standard. Those intending to use this software 	*/
/* module in hardware or software products are advised that its 	*/
/* use may infringe existing patents. The original developer of		*/
/* this software module and his company, the subsequent 			*/
/* editors and their companies, and ISO/IEC have no liability		*/
/* for use of this software module or modifications thereof in 		*/
/* an implementation. Copyright is not released for non				*/
/* MPEG-4 standard conforming products. ACTS-MoMuSys partners	 	*/
/* retain full right to use the code for their own purposes, 		*/
/* assign or donate the code to a third party and to inhibit 		*/
/* third parties from using the code for non MPEG-4 standard		*/
/* conforming products. This copyright notice must be included 		*/
/* in all copies or derivative works. 								*/
/* Copyright (c)1997 												*/
/* 																	*/
/********************************************************************/







/***********************************HeaderBegin****************************
*   File : FAdecoder_defs.h                              
*   
*   Author : Gabriel Antunes Abrantes(gabriel.abrantes@lx.it.pt)
*   Created : 25-July-1997 
*
*   Description : Face Object Decoder
*
*   Modified:
*   Version 5.0 / November-1997
*   Updated for  MPEG-4 Atlantic City Meeting
*
****************************************HeaderEnd**************************/


/* arithmetic coding */

#define   IrangeMin		  -900
#define   IrangeMax		   900
#define   rangeMin		  -900
#define   rangeMax		   900

#define   q1             16384
#define   q2             32768
#define   q3             49152
#define   top            65535
#define   Max_frequency  16383
#define   RANGE_DELTA       60

#define   ZEROMAX           22


/* DCT coding */

#define SEGMENT_LENGTH   16
#define PI               3.1415926
#define DC_MAX			 32767
#define DC_MIN			-32767

#define RUN 0
#define DC  1
#define AC  2

/* face object syntax */

#define FAP_QUANT            1

#define FACE_OBJECT_START_CODE        257
#define FACE_OBJECT_END_CODE          258
#define FACE_OBJECT_PLANE_START_CODE  259

#define FAP                  1
#define BAP                  2
#define FBAP                 3

#define ABORT                2

#define BUFFER_SIZE 256
#define sqr(x) pow(x,2)


#define NO_FAPS       68
#define GROUP_MASK_SIZE 10

#define FAP_MASK_TYPE0       0
#define FAP_MASK_TYPE1       1
#define FAP_MASK_TYPE2       2
#define FAP_MASK_TYPE3       3


#define ARITH_TYPE			 0
#define DCT_TYPE			 1

#define DEC_ERROR			-1


#define OFF 0
#define  ON 1

#define sign(x) ((x>=0)?1:-1)


int DefaultArithLimits[4*NO_FAPS] = { 
	63,63,1080,600,1860,600,600, 600, 600,1860, 1860, 600, 600,600, 1080, 1080, 
	1080, 420, 1080,1080, 600, 600, 1200, 1200, 900, 900, 600, 600, 420, 420, 
	900, 900, 900, 900, 900,900, 900, 900, 900, 900, 600, 600, 1080, 1080, 1080, 
	1080,300, 1860, 1860, 1860, 600,1860, 600, 600, 600, 600, 1860, 1860, 600, 
	600, 540, 540,680,900,900, 900, 900,900,

	0,0,0,-600,-1860,-600,-600,-600,-600,-1860,-1860,-600,-600,0,-1080,-1080,-1080,
	-420,-1080,-1080,-600,-600,-1200,-1200,-900,-900,-600,-600,-420,-420,-900,-900,
	-900,-900,-900,-900,-900,-900,-900,-900,-600,-600,-1080,-1080,-1080,-1080,0,-1860,
	-1860,-1860,-600,-1860,-600,-600,-600,-600,-1860,-1860,-600,-600,-540,-540,-680,
	-900,-900,-900,-900,-900,

	63,63,360,180,600,180,180,180,
	180,600,600,180,180,180,360,360,360,180,600,600,240,240,420,420,300,300,180,180,
	120,120,360,360,360,360,360,360,300,300,300,300,180,180,420,420,420,420,60,600,
	600,600,180,600,180,180,180,180,600,600,180,180,120,120,180,180,240,240,300,300,
	
	-63,-63,0,-180,-600,-180,
	-180,-180,-180,-600,-600,-180,-180,0,-360,-360,-360,-180,-600,-600,-240,-240,
	-420,-420,-300,-300,-180,-180,-120,-120,-360,-360,-360,-360,-360,-360,-300,
	-300,-300,-300,-180,-180,-420,-420,-420,-420,0,-600,-600,-600,-180,-600,-180,
	-180,-180,-180,-600,-600,-180,-180,-120,-120,-180,-180,-240,-240,-300,-300
};


int FapQuantSteps[NO_FAPS] ={ 1, 1, 4, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 1, 1, 1, 1, 1,
							  1, 1, 1, 1, 128, 128, 128, 128, 8, 8, 1, 1, 2, 2, 
							  2, 2, 2, 2, 1, 1, 2, 2, 2, 2, 1, 1, 1, 1, 512, 170,
							  170, 170, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 1, 1, 1, 1, 
							  1, 1, 1, 1 };

int GroupMaskLimits[11] = { 0,2,18,30,38,42,47,50,60,64,68 };


float DCTQuantSteps[NO_FAPS]  = { 1, 1, 7.5, 7.5, 7.5, 7.5, 7.5, 7.5, 7.5, 7.5,
								  7.5, 7.5, 7.5,  15, 15, 15, 15, 5, 10,10,
								  10, 10, 425, 425, 425, 425, 5, 5, 5, 5,
								  7.5, 7.5, 7.5, 7.5, 7.5, 7.5, 7.5, 7.5, 20, 20,
								  20, 20, 10, 10, 10, 10, 255, 170, 255, 255,
								  7.5, 7.5, 7.5, 7.5, 7.5, 7.5, 7.5, 7.5, 7.5, 7.5,
								  15, 15, 15, 15, 10, 10, 10, 10 };

int fap_scale[32] = { 1, 1, 2, 3, 5, 7, 8, 10, 12, 15, 18, 21, 25, 30, 35, 42, 
                      50, 60, 72, 87, 105, 128, 156, 191, 234, 288, 355, 439, 543, 
					  674, 836, 1039 };
