/********************************************************************/
/* 																	*/
/* This software module was originally developed and edited by		*/
/* Pat Mulroy (pmulroy@visual.bt.co.uk) BT Labs						*/
/* Gabriel Antunes Abrantes (Instituto Superior Tcnico)			*/	
/* Karl Olav Lillevold (Karl.Lillevold@nta.no) Telenor				*/
/* Robert Danielsen (Robert.Danielsen@nta.no) Telenor				*/
/* 																	*/
/* in the course of development of the MPEG-4 standard.	 			*/
/* This software module is an implementation of a part of one or 	*/
/* more MPEG-4 tools as specified by the	 						*/
/* MPEG-4 standard. ISO/IEC gives users of the MPEG-4 standard		*/
/* free license to this software module or modifications thereof 	*/
/* for use in hardware or software products claiming conformance 	*/
/* to the MPEG-4 standard. Those intending to use this software 	*/
/* module in hardware or software products are advised that its 	*/
/* use may infringe existing patents. The original developer of		*/
/* this software module and his company, the subsequent 			*/
/* editors and their companies, and ISO/IEC have no liability		*/
/* for use of this software module or modifications thereof in 		*/
/* an implementation. Copyright is not released for non				*/
/* MPEG-4 standard conforming products. BT Labs, ACTS-MoMuSys		*/
/* partners, and Telenor 											*/
/* retain full right to use the code for their own purposes, 		*/
/* assign or donate the code to a third party and to inhibit 		*/
/* third parties from using the code for non MPEG-4 standard		*/
/* conforming products. This copyright notice must be included 		*/
/* in all copies or derivative works. 								*/
/* Copyright (c)1997/98 											*/
/* 																	*/
/********************************************************************/




/***********************************HeaderBegin****************************
*   File : Arith_Coder.c                              
*
*   Description : Arithmetic Coder
*
*	The modules Ar_encode(), encoder_flush(), bit_in_psc_layer(), 
*   bit_opp_bits(), are described in the MPEG4 FCD-Visual text 
*	and were modified for this module by Gabriel Antunes Abrantes
*
*  These modules are part of
*  sac.c, part of tmn (TMN encoder) 
*  Copyright (C) 1996  Telenor R&D, Norway
*        Karl Olav Lillevold <Karl.Lillevold@nta.no>
*
*  This program(sac.c) is free software; you can redistribute it and/or modify
*  it under the terms of the GNU General Public License as published by
*  the Free Software Foundation; either version 2 of the License, or
*  (at your option) any later version.
*
*  This program(sac.c) is distributed in the hope that it will be useful,
*  but WITHOUT ANY WARRANTY; without even the implied warranty of
*  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
*  GNU General Public License for more details.
*
*  You should have received a copy of the GNU General Public License
*  along with this program; if not, write to the Free Software
*  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*
*  Karl Olav Lillevold               <Karl.Lillevold@nta.no>
*  Telenor Research and Development
*  P.O.Box 83                        tel.:   +47 63 84 84 00
*  N-2007 Kjeller, Norway            fax.:   +47 63 81 00 76
*
*  Robert Danielsen                  e-mail: Robert.Danielsen@nta.no
*  Telenor Research and Development  www:    http://www.nta.no/brukere/DVC/
*  P.O.Box 83                        tel.:   +47 63 84 84 00
*  N-2007 Kjeller, Norway            fax.:   +47 63 81 00 76
*
*
* SAC Encoder Module
* Algorithm as specified in H263 (Annex E)
*         (c) BT Labs 1995
*
* Author: Pat Mulroy <pmulroy@visual.bt.co.uk>
*
****************************************HeaderEnd**************************/



#include <stdio.h>
#include <math.h>
#include <stdlib.h>
#include <ctype.h>
#include <string.h>
#include "face.h"
#include "anim.h"
#include "coder.h"

#define   q1             16384
#define   q2             32768
#define   q3             49152
#define   top            65535
#define   MAX_FREQUENCY  16383

#define   ZEROMAX           22


static long   low=0, high=top, opposite_bits=0, length=0;

static int zerocount = 0;

void bit_in_psc_layer(int bit, FILE *fap_stream)
{
	if(zerocount==ZEROMAX) {
		PutBits(1,MARKER_BIT,fap_stream);
		zerocount=0;
	}
	
	PutBits(1, bit, fap_stream);
	
	if(!bit) zerocount++;
	else zerocount = 0;
}



void bit_opp_bits(int bit, FILE *fap_stream) 
{                                   
  bit_in_psc_layer(bit,fap_stream);

  while(opposite_bits > 0){
    bit_in_psc_layer(!bit,fap_stream);
    opposite_bits--;
  }

}



void AR_Encode(int index, int cumul_freq[ ], FILE *fap_stream)
{
	
	if (index<0) {
		sprintf(word,"\n Error ARcoding, symbol is negative");
		WindowPrintf();
		exit(-1);
	}
		
	length = high - low + 1;
	high = low - 1 + (length * cumul_freq[index]) / cumul_freq[0];
	low += (length * cumul_freq[index+1]) / cumul_freq[0];
	
	for ( ; ; ) {
		if (high < q2) {
			bit_opp_bits(0,fap_stream);
		}
		else if (low >= q2) {
			bit_opp_bits(1,fap_stream);	
			low -= q2; 
			high -= q2;
		}
		else if (low >= q1 && high < q3) {
			opposite_bits += 1;
			low -= q1; 
			high -= q1;
		}
		else break;
		
		low *= 2; 
		high = 2*high+1;
	}
	
}



void encoder_flush(FILE *fap_stream)
{
  opposite_bits++;

  if (low < q1) {
    bit_opp_bits(0,fap_stream);
  }
  else {
    bit_opp_bits(1,fap_stream);
  }
  low = 0; 
  high = top;
  zerocount = 0;
}




