/********************************************************************/
/* 																	*/
/* This software module was originally developed and edited by		*/
/* Pat Mulroy (pmulroy@visual.bt.co.uk) BT Labs						*/
/* Gabriel Antunes Abrantes (Instituto Superior Tcnico)			*/	
/* Karl Olav Lillevold (Karl.Lillevold@nta.no) Telenor				*/
/* Robert Danielsen (Robert.Danielsen@nta.no) Telenor				*/
/* 																	*/
/* in the course of development of the MPEG-4 standard.	 			*/
/* This software module is an implementation of a part of one or 	*/
/* more MPEG-4 tools as specified by the	 						*/
/* MPEG-4 standard. ISO/IEC gives users of the MPEG-4 standard		*/
/* free license to this software module or modifications thereof 	*/
/* for use in hardware or software products claiming conformance 	*/
/* to the MPEG-4 standard. Those intending to use this software 	*/
/* module in hardware or software products are advised that its 	*/
/* use may infringe existing patents. The original developer of		*/
/* this software module and his company, the subsequent 			*/
/* editors and their companies, and ISO/IEC have no liability		*/
/* for use of this software module or modifications thereof in 		*/
/* an implementation. Copyright is not released for non				*/
/* MPEG-4 standard conforming products. BT Labs, ACTS-MoMuSys		*/
/* partners, and Telenor 											*/
/* retain full right to use the code for their own purposes, 		*/
/* assign or donate the code to a third party and to inhibit 		*/
/* third parties from using the code for non MPEG-4 standard		*/
/* conforming products. This copyright notice must be included 		*/
/* in all copies or derivative works. 								*/
/* Copyright (c)1997/98 											*/
/* 																	*/
/********************************************************************/


/***********************************HeaderBegin****************************
*   File : Arith_Decoder.c                              
*
*   Description : Arithmetic Decoder
*
*	The modules aa_decode(), decoder_reset() and bit_out_psc_layer(), 
*   are described in the MPEG4 FCD-Visual text 
*	and were modified for this module by Gabriel Antunes Abrantes
*
****************************************HeaderEnd**************************/



#include <stdio.h>
#include <math.h>
#include <stdlib.h>
#include <ctype.h>
#include <string.h>
#include "face.h"
#include "anim.h"
#include "decoder.h"

#define   q1             16384
#define   q2             32768
#define   q3             49152
#define   top            65535
#define   MAX_FREQUENCY  16383
#define   ZEROMAX           22


static long low, high, code_value, Bit, length, sacindex, cum;


int bit_out_psc_layer(FILE *fp_stream)
{	
	if(zerocount == ZEROMAX) {
		if(!GetBits(1,fp_stream)) {	
			sprintf(word,"MARKER BIT error in arithmetic decoding\n");
			WindowPrintf();
			return OFF;
		}
		zerocount = 0;
	}
	
	Bit = GetBits(1,fp_stream);
	
	if(!Bit) zerocount++;
	else zerocount = 0;

	return ON;
}



int decoder_reset(FILE *fp_stream )
{
	int i;
	
	code_value = 0;
	low = 0;
	high = top;
	zerocount = 0;
	
	for (i = 1;   i <= 16;   i++) {
		if(!bit_out_psc_layer(fp_stream))
			return OFF; 
		code_value = 2 * code_value + Bit;
	}
	
	return ON;
}



int aa_decode(int cumul_freq[ ],FILE *fp_stream)
{
	
	length = high - low + 1;
	cum = (-1 + (code_value - low + 1) * cumul_freq[0]) / length;
	for (sacindex = 1; cumul_freq[sacindex] > cum; sacindex++);
	high = low - 1 + (length * cumul_freq[sacindex-1]) / cumul_freq[0];
	low += (length * cumul_freq[sacindex]) / cumul_freq[0];
	
	for ( ; ; ) {  
		if (high < q2) ;
		else if (low >= q2) {
			code_value -= q2; 
			low -= q2; 
			high -= q2;
		}
		
		else if (low >= q1 && high < q3) {
			code_value -= q1; 
			low -= q1; 
			high -= q1;
		}
		
		else {
			break;
		}
		
		low *= 2; 
		high = 2*high + 1;
		if(!bit_out_psc_layer(fp_stream))
			return DEC_ERROR; 
		code_value = 2*code_value + Bit;
		
	}
	
	return (sacindex-1);
}




void update(int symbol, int index,int *range_max, int *range_min,int **cumul_freq, int **freq) 
{
	int n;
	int *cumul_tmp, *freq_tmp;
	
	n = range_max[index]-range_min[index]+1;
	cumul_tmp = cumul_freq[index];
	freq_tmp = freq[index];
	
	if (cumul_tmp[0] == MAX_FREQUENCY) {
		int i, cum;
		cum = 0;
		for(i=n-1; i>=0; i--) {
			freq_tmp[i] = (freq_tmp[i]+1)/2;
			cum += freq_tmp[i];
			cumul_tmp[i] = cum;
		}
		cumul_tmp[n] = 0;
	}
	
	if(symbol < 0 || symbol >= n){
		printf("Error in symbol%d : %d ",index+1,symbol);
		exit(0);
	}

	freq_tmp[symbol] += 1;
	cumul_tmp[symbol] += 1;
	while(symbol>0)
		cumul_tmp[--symbol] += 1;
}




void update_viseme(int symbol, int index,int **Vis_cumul_freq, int **Vis_freq,int scale) 
{
  int n;
  int *cumul_tmp, *freq_tmp;
//##
	if( (index==0) || (index==1) )
		n = 14+14+1;
	else n = 2*quantize(63,scale)+1;
    
	cumul_tmp = Vis_cumul_freq[index];
    freq_tmp = Vis_freq[index];

  if (cumul_tmp[0] == MAX_FREQUENCY) {
    int i, cum;
    cum = 0;

    for(i=n-1; i>=0; i--) {
      freq_tmp[i] = (freq_tmp[i]+1)/2;
      cum += freq_tmp[i];
      cumul_tmp[i] = cum;
    }
    cumul_tmp[n] = 0;
  }

  freq_tmp[symbol] += 1;
  cumul_tmp[symbol] += 1;

  while(symbol>0) {
    symbol -= 1;
    cumul_tmp[symbol] += 1;
  }

}



void update_expression(int symbol, int index,int **Exp_cumul_freq, int **Exp_freq,int scale) 
{
  int n;
  int *cumul_tmp, *freq_tmp;

	if( (index==0) || (index==2) )
		n = 6+6+1;
	else n = 2*quantize(63,scale)+1;
    
	cumul_tmp = Exp_cumul_freq[index];
    freq_tmp = Exp_freq[index];

  if (cumul_tmp[0] == MAX_FREQUENCY) {
    int i, cum;
    cum = 0;

    for(i=n-1; i>=0; i--) {
      freq_tmp[i] = (freq_tmp[i]+1)/2;
      cum += freq_tmp[i];
      cumul_tmp[i] = cum;
    }
    cumul_tmp[n] = 0;
  }

  freq_tmp[symbol] += 1;
  cumul_tmp[symbol] += 1;
 
  while(symbol>0) {
    symbol -= 1;
    cumul_tmp[symbol] += 1;
  }

}





void init_arith(int *Irange_max,int *Irange_min, 
				int **Icumul_freq, int *range_max, 
				int *range_min, int **cumul_freq, int **freq, 
				int	**Exp_freq,int **Exp_cumul_freq,int **IExp_cumul_freq,
				int	**Vis_freq,int **Vis_cumul_freq,int **IVis_cumul_freq,
				int mask[NO_FAPS+1],int scale)
{
	int i, j, n;
	

	if(mask[0]) {
		// visemes values fields 0&1
		for(i=0;i<2;i++) {
			
			free(IVis_cumul_freq[i]);
//##			
			n = 14+1;
			IVis_cumul_freq[i] = (int *) calloc(n+1,sizeof(int));
			
			for(j=0; j<n; j++) 
				IVis_cumul_freq[i][j] = n-j;
			
			IVis_cumul_freq[i][n] = 0;			
		}
		
		for(i=0;i<2;i++) {
			
			free(Vis_freq[i]);
			free(Vis_cumul_freq[i]);
//##						
			n = 2*14+1;
			Vis_freq[i] = (int *) calloc(n,sizeof(int));
			Vis_cumul_freq[i] = (int *) calloc(n+1,sizeof(int));
			
			for(j=0; j<n; j++) {
				Vis_freq[i][j] = 1;
				Vis_cumul_freq[i][j] = n-j;
			}
			
			Vis_cumul_freq[i][n] = 0;
		}
		
		
		// viseme blend field
		
		free(IVis_cumul_freq[2]);
		
		n = quantize(63,scale)+1;
		IVis_cumul_freq[2] = (int *) calloc(n+1,sizeof(int));
		
		for(j=0; j<n; j++) 
			IVis_cumul_freq[2][j] = n-j;
		
		IVis_cumul_freq[2][n] = 0;
		
		
		free(Vis_freq[2]);
		free(Vis_cumul_freq[2]);
		
		n = 2*quantize(63,scale)+1;
		Vis_freq[2] = (int *) calloc(n,sizeof(int));
		Vis_cumul_freq[2] = (int *) calloc(n+1,sizeof(int));
		
		for(j=0; j<n; j++) {
			Vis_freq[2][j] = 1;
			Vis_cumul_freq[2][j] = n-j;
		}
		
		Vis_cumul_freq[2][n] = 0;

	}


	if(mask[1]) {
		// expression values fields 0&2
		for(i=0;i<3;i+=2) {
			
			free(IExp_cumul_freq[i]);
			
			n = 6+1;
			IExp_cumul_freq[i] = (int *) calloc(n+1,sizeof(int));
			
			for(j=0; j<n; j++) 
				IExp_cumul_freq[i][j] = n-j;
						
			IExp_cumul_freq[i][n] = 0;			
		}
		
		for(i=0;i<3;i+=2) {
			
			free(Exp_freq[i]);
			free(Exp_cumul_freq[i]);
			
			n = 2*6+1;
			Exp_freq[i] = (int *) calloc(n,sizeof(int));
			Exp_cumul_freq[i] = (int *) calloc(n+1,sizeof(int));
			
			for(j=0; j<n; j++) {
				Exp_freq[i][j] = 1;
				Exp_cumul_freq[i][j] = n-j;
			}
			
			Exp_cumul_freq[i][n] = 0;
		}


		// expressions intensities fields 1&3
		for(i=1;i<4;i+=2) {

			free(IExp_cumul_freq[i]);
			
			n = quantize(63,scale)+1;
			IExp_cumul_freq[i] = (int *) calloc(n+1,sizeof(int));
			
			for(j=0; j<n; j++) 
				IExp_cumul_freq[i][j] = n-j;

			IExp_cumul_freq[i][n] = 0;
		}

		for(i=1;i<4;i+=2) {

			free(Exp_freq[i]);
			free(Exp_cumul_freq[i]);

			n = 2*quantize(63,scale)+1;
			Exp_freq[i] = (int *) calloc(n,sizeof(int));
			Exp_cumul_freq[i] = (int *) calloc(n+1,sizeof(int));
			
			for(j=0; j<n; j++) {
				Exp_freq[i][j] = 1;
				Exp_cumul_freq[i][j] = n-j;
			}
			
			Exp_cumul_freq[i][n] = 0;
		}
	}


	for(i=2; i<NO_FAPS; i++) {
		if(mask[i]) {

			free(Icumul_freq[i]);
			
			n = Irange_max[i]-Irange_min[i]+1;

			Icumul_freq[i] = (int *) calloc(n+1,sizeof(int));
			
			for(j=0; j<n; j++)
				Icumul_freq[i][j] = n-j;
			
			Icumul_freq[i][n] = 0;
		}
	}
    
	for(i=2; i<NO_FAPS; i++) {
		if(mask[i]) {

			free(freq[i]);
			free(cumul_freq[i]);
			
			n = range_max[i]-range_min[i]+1;
			freq[i] = (int *) calloc(n,sizeof(int));
			cumul_freq[i] = (int *) calloc(n+1,sizeof(int));
			
			for(j=0; j<n; j++) {
				freq[i][j] = 1;
				cumul_freq[i][j] = n-j;
			}
			
			cumul_freq[i][n] = 0;
		}
	}
}


void AdjustBits(FILE *fp_stream) 
{
	int bitin;
	
	bitin = 8-bitsin;
	
	
	if(bitin<=5) {
		fseek(fp_stream, -3L, SEEK_CUR);
		bitsin = 0;
		GetBits(bitin+2,fp_stream);
	}
	else {
		fseek(fp_stream, -2L, SEEK_CUR);
		bitsin = 0;
		GetBits(bitin-6,fp_stream);
	}
	
}
