/********************************************************************/
/* 																	*/
/* This software module was originally developed and edited by		*/
/* Gabriel Antunes Abrantes (Instituto Superior Tecnico)			*/	
/* 																	*/
/* in the course of development of the MPEG-4 (Visual) standard		*/
/* (ISO/IEC 14496-2).	 											*/
/* This software module is an implementation of a part of one or 	*/
/* more MPEG-4 tools as specified by the	 						*/
/* MPEG-4 standard. ISO/IEC gives users of the MPEG-4 standard		*/
/* free license to this software module or modifications thereof 	*/
/* for use in hardware or software products claiming conformance 	*/
/* to the MPEG-4 standard. Those intending to use this software 	*/
/* module in hardware or software products are advised that its 	*/
/* use may infringe existing patents. The original developer of		*/
/* this software module and his company, the subsequent 			*/
/* editors and their companies, and ISO/IEC have no liability		*/
/* for use of this software module or modifications thereof in 		*/
/* an implementation. Copyright is not released for non				*/
/* MPEG-4 standard conforming products. ACTS-MoMuSys partners	 	*/
/* retain full right to use the code for their own purposes, 		*/
/* assign or donate the code to a third party and to inhibit 		*/
/* third parties from using the code for non MPEG-4 standard		*/
/* conforming products. This copyright notice must be included 		*/
/* in all copies or derivative works. 								*/
/* Copyright (c)1997/98												*/
/* 																	*/
/********************************************************************/




/***********************************HeaderBegin****************************
*   File : IST_ArithDec.c                              
*   
*   Author  : Gabriel Antunes Abrantes(gabriel.abrantes@lx.it.pt)
*   Created : 25-July-1997 
*
*   Description : Frame-based Face Object Decoder
*
*   Modified:
*   Version 3.2 / March-1998
*   Updated for MPEG Tokyo Meeting
*
****************************************HeaderEnd**************************/




#include <stdio.h>
#include <math.h>
#include <stdlib.h>
#include <ctype.h>
#include <string.h>
#include <time.h> 
#include "face.h"
#include "anim.h"
#include "decoder.h"




/* arithmetic coding */


#define ABORT                 2


#define DEBUG      0



int fapx[NO_FAPS+1];
int fapx_mask[NO_FAPS+1];

extern int FapQuantSteps[NO_FAPS];
extern int GroupMaskLimits[11];


int mask_type[GROUP_MASK_SIZE];





void ResetIntra(int *quant_steps,int fap_quant)
{
	int counter;
	
	for(counter=0;counter<NO_FAPS;counter++) {
		fapx[counter] = 0;
		quant_steps[counter] = fap_quant*FapQuantSteps[counter];
	}
}


int CheckGroupBegin(int pos)
{
	int counter;

	for(counter=0;counter<11;counter++) {
		if(GroupMaskLimits[counter]==pos) return ON;
	}
	
	return OFF;
}


int DeCode_New_Minmax( int is_I_new_max,int is_I_new_min,int is_P_new_max,
					   int is_P_new_min,int *i_new_max,int *i_new_min,int *p_new_max,
					   int *p_new_min,int *Irange_max,int  *Irange_min,int *range_max,
					   int *range_min,FILE *fp_stream)
{
	int counter,aux;
	
	if(is_I_new_max) {
		for(counter=2;counter<NO_FAPS;counter++) {  

			if(CheckGroupBegin(counter)) aux=0;

			if(!(aux&3)) {
				if(!GetBits(1,fp_stream)) {
					sprintf(word,"WRONG MARKERBIT in CodeNewMinMax\n");
					WindowPrintf();
					return OFF;
				}
			}
						
			if(fapx_mask[counter]) {
				i_new_max[counter] = GetBits(5,fp_stream);
				Irange_max[counter] = RANGE_DELTA*i_new_max[counter];
			}
			
			aux++;
		}
	}	
	
	if(is_I_new_min) {
		for(counter=2;counter<NO_FAPS;counter++) {  
			
			if(CheckGroupBegin(counter)) aux=0;

			if(!(aux&3)) {
				if(!GetBits(1,fp_stream)) {
					sprintf(word,"WRONG MARKERBIT in CodeNewMinMax\n");
					WindowPrintf();
					return OFF;
				}
			}

			if(fapx_mask[counter]) {
				i_new_min[counter] = GetBits(5,fp_stream);
				Irange_min[counter] = -RANGE_DELTA*i_new_min[counter];
			}

			aux++;
		}
	}
	
	if(is_P_new_max) {
		for(counter=2;counter<NO_FAPS;counter++) {  

			if(CheckGroupBegin(counter)) aux=0;

			if(!(aux&3)) {
				if(!GetBits(1,fp_stream)) {
					sprintf(word,"WRONG MARKERBIT in CodeNewMinMax\n");
					WindowPrintf();
					return OFF;
				}
			}

			if(fapx_mask[counter]) {
				p_new_max[counter] = GetBits(5,fp_stream);
				range_max[counter] = RANGE_DELTA*p_new_max[counter];
			}

			aux++;
		}
	}
				
	if(is_P_new_min) {
		for(counter=2;counter<NO_FAPS;counter++) {  

			if(CheckGroupBegin(counter)) aux=0;

			if(!(aux&3)) {
				if(!GetBits(1,fp_stream)) {
					sprintf(word,"WRONG MARKERBIT in CodeNewMinMax\n");
					WindowPrintf();
					return OFF;
				}
			}

			if(fapx_mask[counter]) {
				p_new_min[counter] = GetBits(5,fp_stream);
				range_min[counter] = -RANGE_DELTA*p_new_min[counter];
			}

			aux++;
		}
	}

	return ON;
}





int Decode_Viseme(int is_intra, int **Vis_freq,int **Vis_cumul_freq,int **IVis_cumul_freq,
					int scale,FILE *fp_stream)
{
	static int vis_prev[3];
	int vis_diff;

	if(!decoder_reset(fp_stream))
		return OFF;	

	if(is_intra) {
		
		fap_vis[0] = aa_decode(IVis_cumul_freq[0],fp_stream);
		vis_prev[0] = fap_vis[0];
		fap_vis[1] = aa_decode(IVis_cumul_freq[1],fp_stream);
		vis_prev[1] = fap_vis[1];
		fap_vis[2] = aa_decode(IVis_cumul_freq[2],fp_stream)*scale;
		vis_prev[2] = fap_vis[2];

	}
	else {

		vis_diff = aa_decode(Vis_cumul_freq[0],fp_stream);
		update_viseme(vis_diff,0,Vis_cumul_freq,Vis_freq,scale);
		vis_diff-=14;
		fap_vis[0]+=vis_diff;
		vis_prev[0] = fap_vis[0];

		vis_diff = aa_decode(Vis_cumul_freq[1],fp_stream);
		update_viseme(vis_diff,1,Vis_cumul_freq,Vis_freq,scale);
		vis_diff-=14;
		fap_vis[1]+=vis_diff;
		vis_prev[1] = fap_vis[2];

		vis_diff = aa_decode(Vis_cumul_freq[2],fp_stream);
		update_viseme(vis_diff,2,Vis_cumul_freq,Vis_freq,scale);
		vis_diff-=quantize(63,scale);
		vis_diff*=scale;
		fap_vis[2]+=vis_diff;
		vis_prev[2] = fap_vis[3];

	}
	
	AdjustBits(fp_stream);

	fap_vis[3] = GetBits(1,fp_stream);


	return ON;
}





int Decode_Expression(int is_intra, int **Exp_freq,int **Exp_cumul_freq,int **IExp_cumul_freq,
					  int scale,FILE *fp_stream)
{
	static int expr_prev[4];
	int expr_diff;

	if(!decoder_reset(fp_stream))
		return OFF;	

	if(is_intra) {
		
		fap_expr[0] = aa_decode(IExp_cumul_freq[0],fp_stream);
		expr_prev[0] = fap_expr[0];
		fap_expr[1] = aa_decode(IExp_cumul_freq[1],fp_stream);
		expr_prev[1] = fap_expr[1]*scale;
		fap_expr[2] = aa_decode(IExp_cumul_freq[2],fp_stream);
		expr_prev[2] = fap_expr[2];
		fap_expr[3] = aa_decode(IExp_cumul_freq[3],fp_stream);
		expr_prev[3] = fap_expr[3]*scale;

	}
	else {

		expr_diff = aa_decode(Exp_cumul_freq[0],fp_stream);
		update_expression(expr_diff,0,Exp_cumul_freq,Exp_freq,scale);
		expr_diff-=6;
		fap_expr[0]+=expr_diff;
		expr_prev[0] = fap_expr[0];

		expr_diff = aa_decode(Exp_cumul_freq[1],fp_stream);
		update_expression(expr_diff,1,Exp_cumul_freq,Exp_freq,scale);
		expr_diff-=quantize(63,scale);
		expr_diff*=scale;
		fap_expr[1]+=expr_diff;
		expr_prev[1] = fap_expr[1];

		expr_diff = aa_decode(Exp_cumul_freq[2],fp_stream);
		update_expression(expr_diff,2,Exp_cumul_freq,Exp_freq,scale);
		expr_diff-=6;
		fap_expr[2]+=expr_diff;
		expr_prev[2] = fap_expr[2];

		expr_diff = aa_decode(Exp_cumul_freq[3],fp_stream);
		update_expression(expr_diff,3,Exp_cumul_freq,Exp_freq,scale);
		expr_diff-=quantize(63,scale);
		expr_diff*=scale;
		fap_expr[3]+=expr_diff;
		expr_prev[3] = fap_expr[3];

	}
	
	AdjustBits(fp_stream);

	fap_expr[4] = GetBits(1,fp_stream);
	fap_expr[5] = GetBits(1,fp_stream);


	return ON;
}




int DeCode_IFap(int *fapQuantSteps,
				int *Irange_max, int *Irange_min, int **Icumul_freq, 
				int *range_max, int *range_min, 
				int **cumul_freq, int **freq,int **Exp_freq,int **Exp_cumul_freq,
				int **IExp_cumul_freq,int **Vis_freq,int **Vis_cumul_freq,
				int **IVis_cumul_freq, int scale, FILE *fp_stream)
{
	int counter;
	int fap_quant,dec_fap;
		
	if(fapx_mask[0]){
		if(!Decode_Viseme(ON,Vis_freq,Vis_cumul_freq,IVis_cumul_freq,scale,fp_stream))
			return OFF;	
	}

	if(fapx_mask[1]) {
		if(!Decode_Expression(ON,Exp_freq,Exp_cumul_freq,IExp_cumul_freq,scale,fp_stream))
			return OFF;	
	}

	
	if(!decoder_reset(fp_stream))
		return OFF;
		
	for(counter=2;counter<NO_FAPS;counter++) {
		if(fapx_mask[counter]) {
			fap_quant = aa_decode(Icumul_freq[counter],fp_stream);
			if(fap_quant == DEC_ERROR)
				return OFF;
			else fap_quant+=Irange_min[counter];
			dec_fap =  fapQuantSteps[counter]*fap_quant;
			fapx[counter] = dec_fap;
			
		}
	}
	AdjustBits(fp_stream);
	
	return ON;
}






int DeCode_PFap(int *fapQuantSteps,
				int *Irange_max, int *Irange_min, int **Icumul_freq, 
				int *range_max, int *range_min, 
				int **cumul_freq, int **freq, int **Exp_freq,int **Exp_cumul_freq,
				int **IExp_cumul_freq,int **Vis_freq,int **Vis_cumul_freq,
				int **IVis_cumul_freq,int scale,FILE *fp_stream)
{
	int counter;
	int fap_quant,dec_fap;

	if(fapx_mask[0]) {
		if(!Decode_Viseme(OFF,Vis_freq,Vis_cumul_freq,IVis_cumul_freq,scale,fp_stream))
			return OFF;	
	}
	if(fapx_mask[1]) {
		if(!Decode_Expression(OFF,Exp_freq,Exp_cumul_freq,IExp_cumul_freq,scale,fp_stream))
			return OFF;	
	}

	if(!decoder_reset(fp_stream))
		return OFF;
	
	for(counter=2;counter<NO_FAPS;counter++) {
		if(fapx_mask[counter]) {
			fap_quant = aa_decode(cumul_freq[counter],fp_stream);
			if(fap_quant == DEC_ERROR)
				return OFF;
			update(fap_quant,counter,range_max,range_min,cumul_freq,freq );
			fap_quant += range_min[counter];
			dec_fap =  fapQuantSteps[counter]*fap_quant + fapx[counter];
			fapx[counter] = dec_fap;
		}
	}
	AdjustBits(fp_stream);
	
	return ON;
}
