/********************************************************************/
/* 																	*/
/* This software module was originally developed and edited by		*/
/* Gabriel Antunes Abrantes (Instituto Superior Tecnico)			*/	
/* 																	*/
/* in the course of development of the MPEG-4 (Visual) standard		*/
/* (ISO/IEC 14496-2).	 											*/
/* This software module is an implementation of a part of one or 	*/
/* more MPEG-4 tools as specified by the	 						*/
/* MPEG-4 standard. ISO/IEC gives users of the MPEG-4 standard		*/
/* free license to this software module or modifications thereof 	*/
/* for use in hardware or software products claiming conformance 	*/
/* to the MPEG-4 standard. Those intending to use this software 	*/
/* module in hardware or software products are advised that its 	*/
/* use may infringe existing patents. The original developer of		*/
/* this software module and his company, the subsequent 			*/
/* editors and their companies, and ISO/IEC have no liability		*/
/* for use of this software module or modifications thereof in 		*/
/* an implementation. Copyright is not released for non				*/
/* MPEG-4 standard conforming products. ACTS-MoMuSys partners	 	*/
/* retain full right to use the code for their own purposes, 		*/
/* assign or donate the code to a third party and to inhibit 		*/
/* third parties from using the code for non MPEG-4 standard		*/
/* conforming products. This copyright notice must be included 		*/
/* in all copies or derivative works. 								*/
/* Copyright (c)1997/98												*/
/* 																	*/
/********************************************************************/


/***********************************HeaderBegin****************************
*   File : IST_ArithDec.c                              
*   
*   Author  : Gabriel Antunes Abrantes(gabriel.abrantes@lx.it.pt)
*   Created : 15-October-1997 
*
*   Description : DCT Face Object Decoder
*
*   Modified:
*   Version 2.1 / March-1998
*   Updated for MPEG Tokyo Meeting
*
****************************************HeaderEnd**************************/




#include <stdio.h>
#include <math.h>
#include <stdlib.h>
#include <ctype.h>
#include <string.h>
#include <time.h> 
#include "face.h"
#include "anim.h"
#include "decoder.h"



#define DC_MAX			32767
#define DC_MIN			-32767


/* face object syntax */


#define RUN 0
#define DC  1
#define AC  2

#define BUFFER_SIZE 256

#define DEBUG    0

extern float DCTQuantSteps[NO_FAPS];

extern int fap_scale[32];


int   **dc_table,**ac_table,**run_table;	





/******************************************************************


 - Memory allocation


******************************************************************/

float **FloatMatrixAllocation(int vs,int hs)
{
  float **new_matrix;
  int i;
  
  new_matrix=(float **) calloc(vs,sizeof(float*));
  if ( (new_matrix==NULL) ) {
	  sprintf(word,"Out of Memory \n");
	  WindowPrintf();
	  exit(1);
  }
  
  for(i=0;i<vs;i++) {
	  new_matrix[i]=(float *) calloc(hs,sizeof(float));
	  if ( (new_matrix[i]==NULL)  ){
		  sprintf(word,"Out of Memory \n");
		  WindowPrintf();
		  exit(1);
	  }
  }
  
  return new_matrix;
}



void MatrixDealoc(float **matrix,int vs)
{
  int i;
  
  if(matrix==NULL) return;

  for(i=0;i<vs;i++) 
    free(matrix[i]);
  
  free(matrix);
}




int **IntMatrixAllocation(int vs,int hs)
{
  int **new_matrix;
  int i;
  
  new_matrix=(int **) calloc(vs,sizeof(int*));
  if ( (new_matrix==NULL) ) {
		sprintf(word,"Out of Memory \n");
		WindowPrintf();
		exit(1);
  }
  
  for(i=0;i<vs;i++) {
    new_matrix[i]=(int *) calloc(hs,sizeof(int));
    if ( (new_matrix[i]==NULL)  ){
		sprintf(word,"Out of Memory \n");
		WindowPrintf();
		exit(1);
    }
  }
  
  return new_matrix;
}




void IntMatrixDealoc(int **matrix,int vs)
{
  int i;

  if(matrix==NULL) return;
  
  for(i=0;i<vs;i++) 
    free(matrix[i]);
  
  free(matrix);
}




/***********************************************************************


 Get Short of stream - 

***********************************************************************/


int ShortGetBits(FILE *fp_stream) 
{
	int counter;
	short value = 0;	
	
	for(counter=0; counter<16; counter++) {
        value <<= 1;
        value += GetBits(1,fp_stream);
	}
	
	
	return (int)value;
}



/***********************************************************************


  DCT Coder 


***********************************************************************/



void MakeIDCTMatrix(float **idct)
{
	float idct_coef;
	int   line,colunm;
	
	for(line=0; line<SEGMENT_LENGTH; line++) {
		
		if(line==0) 
			idct_coef = (float)(1.0/sqrt(2));
		else idct_coef = 1;
		
		for(colunm=0; colunm<SEGMENT_LENGTH; colunm++)
			idct[colunm][line] = (float)(sqrt(2.0/SEGMENT_LENGTH))*idct_coef*
							(float)(cos(PI*(float)(2*colunm+1)*line/2.0/SEGMENT_LENGTH));
		
	}

}



void Obtain_IDCT(float *faps, float *faps_coeff, float **dct)
{
	int line,colunm;
	
	for (line=0; line<SEGMENT_LENGTH; line++) {
		faps_coeff[line]=0;
		for (colunm=0; colunm<SEGMENT_LENGTH; colunm++)
			faps_coeff[line] += dct[line][colunm]*faps[colunm];
	}
}





/***********************************************************************


  Huffman-Tables and decoding


***********************************************************************/




int CheckHuffmanTables()
{
	FILE *fp_table;
	int  i,a;

	char location[BUFFER_SIZE];
	char *comline = GetCommandLine();
	
	i = 1;
	a = 0;
	while(comline[i]!='"')
		location[a++] = comline[i++];
	location[a] = '\0';
	
	while(location[a]!='\\')
		location[a--] = '\0';
	location[a] = '\0';


	sprintf(word,"%s/run.table",location);
	if( (fp_table  = fopen(word,"rb"))==NULL ) {
		sprintf(word,"\n File - run.table - does not exist. \n");
		WindowPrintf();
		return OFF;
	}
	fclose(fp_table);

	sprintf(word,"%s/dc.table",location);
	if( (fp_table  = fopen(word,"rb"))==NULL ) {
		sprintf(word,"\n File - dc.table - does not exist. \n");
		WindowPrintf();
		return OFF;
	}
	fclose(fp_table);

	sprintf(word,"%s/ac.table",location);
	if( (fp_table  = fopen(word,"rb"))==NULL ) {
		sprintf(word,"\n File - ac.table - does not exist. \n");
		WindowPrintf();
		return OFF;
	}
	fclose(fp_table);

	return ON;
}



void ReadHuffmanTables()
{
	FILE *fp_table;
	char word[BUFFER_SIZE];
	int counter,bitcounter,i,a;


	char location[BUFFER_SIZE];
	char *comline = GetCommandLine();
	
	i = 1;
	a = 0;
	while(comline[i]!='"')
		location[a++] = comline[i++];
	location[a] = '\0';
	
	while(location[a]!='\\')
		location[a--] = '\0';
	location[a] = '\0';

	
	
	/* huffman tables */

	run_table = IntMatrixAllocation(16,10);
	dc_table  = IntMatrixAllocation(512,18);
	ac_table  = IntMatrixAllocation(512,18);

	sprintf(word,"%s/run.table",location);
	fp_table  = fopen(word,"rb");

	for(counter=0;counter<16;counter++) {
		fscanf(fp_table,"%d",&run_table[counter][0]);
		
		for(bitcounter=1;bitcounter<=run_table[counter][0];bitcounter++) 
			fscanf(fp_table,"%d",&run_table[counter][bitcounter]);
		fgets(word,BUFFER_SIZE,fp_table);
	}

	fclose(fp_table);


	sprintf(word,"%s/dc.table",location);
	fp_table  = fopen(word,"rb");

	for(counter=0;counter<512;counter++) {
		fscanf(fp_table,"%d",&dc_table[counter][0]);
		
		for(bitcounter=1;bitcounter<=dc_table[counter][0];bitcounter++) 
			fscanf(fp_table,"%d",&dc_table[counter][bitcounter]);
		fgets(word,BUFFER_SIZE,fp_table);
	}

	fclose(fp_table);


	sprintf(word,"%s/ac.table",location);
	fp_table  = fopen(word,"rb");

	for(counter=0;counter<512;counter++) {
		fscanf(fp_table,"%d",&ac_table[counter][0]);
		
		for(bitcounter=1;bitcounter<=ac_table[counter][0];bitcounter++) 
			fscanf(fp_table,"%d",&ac_table[counter][bitcounter]);
		fgets(word,BUFFER_SIZE,fp_table);
	}

	
	fclose(fp_table);
}



int GetHuffmanSymbol(int **table,int type,FILE *fap_stream)
{
	int symbol,bit;
	int size,no_bits,counter;
	int found = OFF;
	int *check;

	if(!type) size = 16;
	else size = 512;

	check = (int *)calloc(size,sizeof(int));
	
	no_bits = 1;

	while(!found) {

		bit = GetBits(1,fap_stream);
		
		for(counter=0;counter<size;counter++) {
			if(!check[counter]) {

				if( (table[counter][0]==no_bits) && (table[counter][no_bits]==bit) ) {
					symbol = counter;
					found=ON;
					break;
				}
				else if( table[counter][no_bits]!=bit )
					check[counter]=ON;
			}
		}
		no_bits++;
	}

	free(check);
	return symbol;
}




/***********************************************************************


  Encoder


***********************************************************************/





void Dequantize(int dc_Q,float *dc_pred,float quant_step)
{
	*dc_pred = (float)dc_Q*quant_step;
}


					
void DC_I_Decode(int *dc_Q,FILE *fap_stream)
{
	*dc_Q = ShortGetBits(fap_stream);

	if(*dc_Q == DC_MIN-1)
		*dc_Q = GetBits(32,fap_stream);
}




void AC_Decode(float *faps_coeff, float quant_step, int **run_table,
			   int **ac_table,FILE *fap_stream)
{
	int *coeff_Q,counter;
	int run_length,run_counter,symbol;

	coeff_Q = (int *)calloc(SEGMENT_LENGTH,sizeof(int));


	run_length = 1;	
	run_counter = 0;

	while(run_length ) {
		 
		if( run_counter+1 >= SEGMENT_LENGTH )
			break;
		
		run_length = GetHuffmanSymbol(run_table,RUN,fap_stream);
		
		if(run_length!=0) {
			run_counter+=run_length;			
			symbol = GetHuffmanSymbol(ac_table,AC,fap_stream);
			if(symbol==256)
				DC_I_Decode(&coeff_Q[run_counter],fap_stream);
			else coeff_Q[run_counter] = symbol-256;
		}
	}

	
	for(counter=1;counter<SEGMENT_LENGTH;counter++)
		Dequantize(coeff_Q[counter],&faps_coeff[counter],quant_step);	

	free(coeff_Q);
}


 

					
void Predict(float dc,float dc_pred,float *dc_diff)
{
	*dc_diff = dc - dc_pred;
}
					 

					

void Reconstruct(float dc_diff,float *dc_pred )
{
	*dc_pred +=dc_diff;
} 



void DC_P_Decode(int *dc_Q,int **dc_table,FILE *fap_stream)
{

	*dc_Q = GetHuffmanSymbol(dc_table,DC,fap_stream);

	if(*dc_Q == 0)
		DC_I_Decode(dc_Q,fap_stream);
	else
		*dc_Q-=256;

}



int last_viseme[3];


void DeCode_I_Viseme_Segment(FILE *fap_stream, int **fap_viseme)
{
	int counter;
	
	fap_viseme[0][0] = GetBits(5,fap_stream);
	fap_viseme[0][1] = GetBits(5,fap_stream);
	fap_viseme[0][2] = GetBits(7,fap_stream);
	fap_viseme[0][3] = GetBits(1,fap_stream);


	for(counter=1;counter<SEGMENT_LENGTH;counter++) {
		fap_viseme[counter][0] = GetBits(5,fap_stream)+fap_viseme[counter-1][0]-14;
		fap_viseme[counter][1] = GetBits(5,fap_stream)+fap_viseme[counter-1][1]-14;
		fap_viseme[counter][2] = GetBits(7,fap_stream)+fap_viseme[counter-1][2]-63;
		fap_viseme[counter][3] = GetBits(1,fap_stream);
	}

	for(counter=0;counter<3;counter++)
		last_viseme[counter] = fap_viseme[15][counter];
}



void DeCode_P_Viseme_Segment(FILE *fap_stream, int **fap_viseme)
{
	int counter;
	
	fap_viseme[0][0] = GetBits(5,fap_stream) + last_viseme[0] - 14;
	fap_viseme[0][1] = GetBits(5,fap_stream) + last_viseme[1] - 14;
	fap_viseme[0][2] = GetBits(7,fap_stream) + last_viseme[2] - 63;
	fap_viseme[0][3] = GetBits(1,fap_stream);


	for(counter=1;counter<SEGMENT_LENGTH;counter++) {
		fap_viseme[counter][0] = GetBits(5,fap_stream)+fap_viseme[counter-1][0]-14;
		fap_viseme[counter][1] = GetBits(5,fap_stream)+fap_viseme[counter-1][1]-14;
		fap_viseme[counter][2] = GetBits(7,fap_stream)+fap_viseme[counter-1][2]-63;
		fap_viseme[counter][3] = GetBits(1,fap_stream);
	}

	for(counter=0;counter<3;counter++)
		last_viseme[counter] = fap_viseme[15][counter];
}



int last_expression[4];


void DeCode_I_Expression_Segment(FILE *fap_stream, int **fap_expression)
{
	int counter;
	
	fap_expression[0][0] = GetBits(4,fap_stream);
	fap_expression[0][2] = GetBits(4,fap_stream);
	fap_expression[0][1] = GetBits(7,fap_stream);
	fap_expression[0][3] = GetBits(7,fap_stream);
	fap_expression[0][4] = GetBits(1,fap_stream);
	fap_expression[0][5] = GetBits(1,fap_stream);


	for(counter=1;counter<SEGMENT_LENGTH;counter++) {
		fap_expression[counter][0] = GetBits(4,fap_stream)+fap_expression[counter-1][0]-6;
		fap_expression[counter][2] = GetBits(4,fap_stream)+fap_expression[counter-1][2]-6;
		fap_expression[counter][1] = GetBits(7,fap_stream)+fap_expression[counter-1][1]-63;
		fap_expression[counter][3] = GetBits(7,fap_stream)+fap_expression[counter-1][3]-63;
		fap_expression[counter][4] = GetBits(1,fap_stream);
		fap_expression[counter][5] = GetBits(1,fap_stream);
	}

	for(counter=0;counter<4;counter++)
		last_expression[counter] = fap_expression[15][counter];
}



void DeCode_P_Expression_Segment(FILE *fap_stream, int **fap_expression)
{
	int counter;
	
	fap_expression[0][0] = GetBits(4,fap_stream) + last_expression[0] - 6;
	fap_expression[0][2] = GetBits(4,fap_stream) + last_expression[2] - 6;
	fap_expression[0][1] = GetBits(7,fap_stream) + last_expression[1] - 63;
	fap_expression[0][3] = GetBits(7,fap_stream) + last_expression[3] - 63;
	fap_expression[0][4] = GetBits(1,fap_stream);
	fap_expression[0][5] = GetBits(1,fap_stream);


	for(counter=1;counter<SEGMENT_LENGTH;counter++) {
		fap_expression[counter][0] = GetBits(4,fap_stream)+fap_expression[counter-1][0]-6;
		fap_expression[counter][2] = GetBits(4,fap_stream)+fap_expression[counter-1][2]-6;
		fap_expression[counter][1] = GetBits(7,fap_stream)+fap_expression[counter-1][1]-63;
		fap_expression[counter][3] = GetBits(7,fap_stream)+fap_expression[counter-1][3]-63;
		fap_expression[counter][4] = GetBits(1,fap_stream);
		fap_expression[counter][5] = GetBits(1,fap_stream);
	}

	for(counter=0;counter<4;counter++)
		last_expression[counter] = fap_expression[15][counter];
}




void DeCode_ISegment(int *fap_mask, float **faps, int	**fap_expression,int **fap_viseme,
					 float *fapQuantSteps,float *dc_pred,float **idct, int **run_table,
					 int **ac_table,FILE *fap_stream)
{
	int counter;
	int group_number;

	float **faps_coeff;
	int   dc_Q;

	faps_coeff = FloatMatrixAllocation(NO_FAPS,SEGMENT_LENGTH);
	
	
	for(group_number=0;group_number<10;group_number++) {
		
		if( group_number+1 == 1)  {
			if(fap_mask[1]) 
				DeCode_I_Expression_Segment(fap_stream,fap_expression);
			if(fap_mask[0]) 
				DeCode_I_Viseme_Segment(fap_stream,fap_viseme);
		}
		
		else {
			
			for(counter=GroupMaskLimits[group_number];
				counter<GroupMaskLimits[group_number+1];counter++) 

				if(fap_mask[counter]) {

					DC_I_Decode(&dc_Q,fap_stream);
					Dequantize(dc_Q,&faps_coeff[counter][0],fapQuantSteps[counter]/3);
					AC_Decode(faps_coeff[counter],fapQuantSteps[counter],
								run_table,ac_table,fap_stream);
					
					Obtain_IDCT(faps_coeff[counter],faps[counter],idct);
				}
		}
	}

	for(counter = 0; counter<NO_FAPS; counter++)
		dc_pred[counter] = faps_coeff[counter][0];

	MatrixDealoc(faps_coeff,NO_FAPS);
}




void DeCode_PSegment(int *fap_mask,float **faps,int	**fap_expression,int **fap_viseme,
					 float *fapQuantSteps, float *dc_pred,float **idct,int **run_table,
					 int **dc_table,int **ac_table,FILE *fap_stream)
{
	int counter;
	int group_number;

	float **faps_coeff;
	int   dc_Q;
	float *dc_diff;

	faps_coeff = FloatMatrixAllocation(NO_FAPS,SEGMENT_LENGTH);
	dc_diff    = (float *)calloc(NO_FAPS,sizeof(float));
	

	for(group_number=0;group_number<10;group_number++) {
		
		if( group_number+1 == 1 ) {
			if(fap_mask[1]) 
				DeCode_P_Expression_Segment(fap_stream,fap_expression);
			if(fap_mask[0]) 
				DeCode_P_Viseme_Segment(fap_stream,fap_viseme);
		}		
		else {
			
			for(counter=GroupMaskLimits[group_number];
				counter<GroupMaskLimits[group_number+1];counter++) 

				if(fap_mask[counter]) {

					DC_P_Decode(&dc_Q,dc_table,fap_stream);	

					Dequantize(dc_Q,&dc_diff[counter],fapQuantSteps[counter]/3);
					
					Reconstruct(dc_diff[counter],&dc_pred[counter]);
					
					faps_coeff[counter][0] = dc_pred[counter];

					AC_Decode(faps_coeff[counter],fapQuantSteps[counter],run_table,ac_table,
								fap_stream);
					
					Obtain_IDCT(faps_coeff[counter],faps[counter],idct);
				}
		}
		
	}

	MatrixDealoc(faps_coeff,NO_FAPS);
	free(dc_diff);
}







void ResetDCTIntra(float **faps,int **fap_expression,float *dctquant_steps,int fap_quant_index)
{
	int line,colunm,counter;

	for(line=0;line<NO_FAPS;line++) {
		for(colunm=0;colunm<SEGMENT_LENGTH;colunm++)
			faps[line][colunm]=0;
	}

	for(line=0;line<SEGMENT_LENGTH;line++) {
		for(colunm=0;colunm<6;colunm++)
			fap_expression[line][colunm]=0;
	}

	for(counter=0;counter<NO_FAPS;counter++) 
		dctquant_steps[counter] = fap_scale[fap_quant_index]*DCTQuantSteps[counter];

}




int ShowAllDCTFrames(float **group_faps,int **dct_expression,int **dct_viseme,
					 int *dct_fap_mask,int frame_number)
{
	int    counter,i;

	counter=0;
	while(counter<SEGMENT_LENGTH) {
		
		for(i=0;i<NO_FAPS;i++)
			fapx[i] = group_faps[i][counter];
		for(i=0;i<6;i++)
			fap_expr[i] = dct_expression[counter][i];
		for(i=0;i<4;i++)
			fap_expr[i] = dct_viseme[counter][i];
		
		if(DisplayDecodedFrame(frame_number+counter)!=ON) return OFF;
		counter++;
	}
	
	return ON;
}



