/********************************************************************/
/* 																	*/
/* This software module was originally developed and edited by		*/
/* Gabriel Antunes Abrantes (Instituto Superior Tecnico)			*/	
/* 																	*/
/* in the course of development of the MPEG-4 (Visual) standard		*/
/* (ISO/IEC 14496-2).	 											*/
/* This software module is an implementation of a part of one or 	*/
/* more MPEG-4 tools as specified by the	 						*/
/* MPEG-4 standard. ISO/IEC gives users of the MPEG-4 standard		*/
/* free license to this software module or modifications thereof 	*/
/* for use in hardware or software products claiming conformance 	*/
/* to the MPEG-4 standard. Those intending to use this software 	*/
/* module in hardware or software products are advised that its 	*/
/* use may infringe existing patents. The original developer of		*/
/* this software module and his company, the subsequent 			*/
/* editors and their companies, and ISO/IEC have no liability		*/
/* for use of this software module or modifications thereof in 		*/
/* an implementation. Copyright is not released for non				*/
/* MPEG-4 standard conforming products. ACTS-MoMuSys partners	 	*/
/* retain full right to use the code for their own purposes, 		*/
/* assign or donate the code to a third party and to inhibit 		*/
/* third parties from using the code for non MPEG-4 standard		*/
/* conforming products. This copyright notice must be included 		*/
/* in all copies or derivative works. 								*/
/* Copyright (c)1997/98												*/
/* 																	*/
/********************************************************************/




/***********************************HeaderBegin****************************
*   File : IST_Editor.c                             
*   
*   Author :  Gabriel Antunes Abrantes(gabriel.abrantes@lx.it.pt)
*   Created : January-1998 
*
*   Description : FAP files editor
*
*   Version 1.1
*
****************************************HeaderEnd**************************/



#include <stdio.h>	 
#include <windows.h>
#include <commctrl.h>
#include <windowsx.h>
#include "face.h"
#include "editor.h"
#include "anim.h"
#include "face_res.h"
#include "coder.h"



int fapeditfirst = OFF;
int editfap = OFF;
int editcancel = OFF;

int frame = 0;
int setfap = 0;
int setfap_id = 1;
int setexpr = 0;
int setexprint = 0;

int testing = OFF;
int setting = OFF;


FAPeditor fapeditor;

void Handle_FAPDLG_Command(HWND,WPARAM, LPARAM);
void Handle_CoderDLG_Command(HWND,WPARAM, LPARAM);

FILE *fp_editfap;



void FreeFAPeditor() 
{
	int i;

	for(i=0;i<fapeditor.framenumber;i++) {
		free(fapeditor.mask[i]);
		free(fapeditor.faps[i]);
	}
	free(fapeditor.mask);
	free(fapeditor.faps);
	
}



void ResetModel()
{
	int counter,pos;
  
	/* reset points,normals & faps */

	for(counter=0;counter<NO_POINTS;counter++)
		for(pos=X;pos<=Z;pos++)	{
			pts[counter][pos] = points_orig[counter][pos];
			norms[counter][pos] = 0;
		}
}




void ShowCurrentEditFace()
{
	int i;
	
	for(i=3;i<NO_FAPS;i++)
		faps[i] = fapeditor.faps[frame][i-1];
		
	ResetModel();
	FrameAnimation();
	ObtainVertsNormals();
	SendPaintMsg();
	
}



void ApplyExpressionToEditFAP(HWND hDlg)
{
	int   counter;
	float fapval,intensity;

	for(counter=2;counter<NO_FAPS;counter++) {
		fapval    = (float)ISTexpressions[setexpr][counter+1];
		intensity = ((float)setexprint)/63.0;
		fapeditor.faps[frame][counter] = (int)(fapval*intensity);
	}

	SetDlgItemInt(hDlg,ID_FAPVALEDIT,fapeditor.faps[frame][setfap_id-1],TRUE);
	ShowCurrentEditFace();
}



int TestEditFAPValues(HWND hDlg)
{
	int test[5];

	testing = ON;

	setfap     = GetDlgItemInt(hDlg,ID_FAPVALEDIT,&test[0],TRUE);
	setfap_id  = GetDlgItemInt(hDlg,ID_FAPEDIT,&test[1],FALSE);
	frame      = GetDlgItemInt(hDlg,ID_FAPFRAMENO,&test[2],FALSE);
	setexpr    = GetDlgItemInt(hDlg,ID_EXPR_SEL,&test[3],TRUE);
	setexprint = GetDlgItemInt(hDlg,ID_EXPR_INT,&test[4],FALSE);
				
	if( test[0]!=1){ 
		sprintf(word,"\n Invalid value for FAP!");
		WindowPrintf();
		SetDlgItemInt(hDlg,ID_FAPVALEDIT,0,FALSE);
		testing = OFF;
		return OFF;
	}
	
	if( test[1]!=1 || setfap_id<1 || setfap_id>NO_FAPS ){ 
		sprintf(word,"\n Invalid FAP!");
		WindowPrintf();
		SetDlgItemInt(hDlg,ID_FAPEDIT,1,FALSE);
		testing = OFF;
		return OFF;
	}
	
	if( test[2]!=1 || frame<0 || frame>fapeditor.framenumber ){ 
		sprintf(word,"\n Invalid framenumber: 0 to %d!",fapeditor.framenumber-1);
		WindowPrintf();
		SetDlgItemInt(hDlg,ID_FAPFRAMENO,0,FALSE);
		testing = OFF;
		return OFF;
	}

	if( test[3]!=1 || setexpr<0 || setexpr>6 ){ 
		sprintf(word,"\n Invalid Expression: 1 to 6!");
		WindowPrintf();
		SetDlgItemInt(hDlg,ID_EXPR_SEL,0,FALSE);
		testing = OFF;
		return OFF;
	}

	if( test[4]!=1 || setexprint<0 || setexprint>63 ){ 
		sprintf(word,"\n Invalid expression intensity: 0 to 63!");
		WindowPrintf();
		SetDlgItemInt(hDlg,ID_EXPR_INT,0,FALSE);
		testing = OFF;
		return OFF;
	}

	testing = OFF;
	return ON;
}







int ReadEditFapHeader(char *fap_file)
{
	float version;
	
	if(fp_editfap!=NULL) {
		fclose(fp_editfap);
		fp_editfap = NULL;
	}

	sprintf(word,"%s",fap_file);
	if( (fp_editfap  = fopen(word,"rb"))==NULL ) 
		return OFF;
	
	TestforTrash(fp_editfap);    
	fscanf(fp_editfap,"%f",&version);
	
	if(version<2.0){
		sprintf(word,"Only FAPfiles version 2.+ are supported, not %f\n",version);
		WindowPrintf();
		fclose(fp_editfap);
		return OFF;
	}
	
	PassBlank(fp_editfap);
	ReadWord(fp_editfap,word);
	fscanf(fp_editfap,"%f",&fapeditor.framerate);
	fscanf(fp_editfap,"%d",&fapeditor.framenumber);

	KillLine(fp_editfap);
	
	return ON;
}



void ReadEditFrames()
{
	int counter,i;
	int frame_number;

	fapeditor.mask = (int **)calloc(fapeditor.framenumber,sizeof(int *));
	fapeditor.faps = (int **)calloc(fapeditor.framenumber,sizeof(int *));
	
	for(i=0;i<fapeditor.framenumber;i++) {
		
		fapeditor.mask[i] = (int *)calloc(NO_FAPS,sizeof(int));
		fapeditor.faps[i] = (int *)calloc(NO_FAPS,sizeof(int));
		
		TestforTrash(fp_editfap);
		for(counter=0;counter<NO_FAPS;counter++) 
			fscanf(fp_editfap,"%d",&fapeditor.mask[i][counter]);
		KillLine(fp_editfap);
		
		TestforTrash(fp_editfap);    
		
		fscanf(fp_editfap,"%d",&frame_number);
		
		if(fapeditor.mask[i][0]) {
			for(counter=0;counter<4;counter++)
				GetInt(fp_editfap);	
		}
		
		if(fapeditor.mask[i][1]) {
			for(counter=0;counter<6;counter++)
				GetInt(fp_editfap);	
		}
		
		for(counter=2;counter<NO_FAPS;counter++) {
			if(fapeditor.mask[i][counter])
				fscanf(fp_editfap,"%d",&fapeditor.faps[i][counter]);
		}
		KillLine(fp_editfap);
	}
		
	fclose(fp_editfap);
}





BOOL CALLBACK EditFAP_DlgProc(HWND hDlg , UINT Message, WPARAM wParam, LPARAM lParam)
{
	int test,i;
	static int getfile;

	switch(Message) {


	case WM_INITDIALOG:
				
		sprintf(word,"1.0");
		SetDlgItemText(hDlg,ID_FAPRATE,word);

		SetDlgItemInt(hDlg,ID_FAPFRAME,1,FALSE);

		Button_SetCheck(GetDlgItem(hDlg,ID_FAPCHECK),OFF);

		editfap = OFF;
		getfile = OFF;
		return TRUE;
		break;

	case WM_COMMAND:
		
		if(LOWORD(wParam) == ID_FAPCHECK) {
			if(Button_GetCheck(GetDlgItem(hDlg,ID_FAPCHECK)))
				getfile = ON;
			else 
				getfile = OFF;
			return TRUE;
		}

		
		if(LOWORD(wParam) == IDOK) {
			
			GetDlgItemText(hDlg,ID_FAPRATE,word,BUFFER_SIZE);
			sscanf(word,"%f",&fapeditor.framerate);
			if(fapeditor.framerate<1.0) { 
				sprintf(word,"\n Invalid frame rate!");
				WindowPrintf();
				return TRUE;
			}			
			
			fapeditor.framenumber = GetDlgItemInt(hDlg,ID_FAPFRAME,&test,FALSE);
			if(!test) {
				sprintf(word,"\n Invalid number of frames!");
				WindowPrintf();
				return TRUE;
			}
			


			if(getfile) {
					
				if(!OpenFAPForEdit(hDlg)) {
					EndDialog(hDlg,LOWORD(wParam));
					editfap = OFF;
					editcancel = ON;
					return TRUE;	
				}

			}
			else {
				fapeditor.mask = (int **)calloc(fapeditor.framenumber,sizeof(int *));
				fapeditor.faps = (int **)calloc(fapeditor.framenumber,sizeof(int *));
				for(i=0;i<fapeditor.framenumber;i++) {
					fapeditor.mask[i] = (int *)calloc(NO_FAPS,sizeof(int));
					fapeditor.faps[i] = (int *)calloc(NO_FAPS,sizeof(int));
				}
			}
			EndDialog(hDlg,LOWORD(wParam));
			editfap=ON;			
			return TRUE;
		}

		if(LOWORD(wParam) == IDCANCEL) {
			EndDialog(hDlg,LOWORD(wParam));
			editfap = OFF;
			editcancel = ON;
			return TRUE;
		}

		break;
	}
	return FALSE;
}


BOOL CALLBACK FAP_DlgProc(HWND hDlg , UINT Message, WPARAM wParam, LPARAM lParam)
{
	switch(Message) {

	case WM_INITDIALOG:
		
		fapeditfirst = ON;
		SetDlgItemInt(hDlg,ID_FAPFRAMENO,0,FALSE);
		SetDlgItemInt(hDlg,ID_FAPEDIT,1,FALSE);
		SetDlgItemInt(hDlg,ID_FAPVALEDIT,0,FALSE);
		SetDlgItemInt(hDlg,ID_EXPR_SEL,0,FALSE);
		SetDlgItemInt(hDlg,ID_EXPR_INT,0,FALSE);

		setfap = 0;
		fapeditfirst=OFF;

		return TRUE;
		break;

	case WM_COMMAND:

		Handle_FAPDLG_Command(hDlg,wParam,lParam);
		return TRUE;
		break;
	}
	return FALSE;
}



void Handle_FAPDLG_Command(HWND hDlg,WPARAM wParam, LPARAM lParam)
{
	if( fapeditfirst==ON || testing==ON ) return;
	
	if(!TestEditFAPValues(hDlg)) return;
	
	switch(LOWORD(wParam)) {

	
	case ID_FAPVALEDIT:
	
		fapeditor.faps[frame][setfap_id-1] = setfap;
		ShowCurrentEditFace();
		break;	


	case ID_FAPFRAMENO:

		ShowCurrentEditFace();
		break;


	case ID_FAPEDIT:
		SetDlgItemInt(hDlg,ID_FAPVALEDIT,fapeditor.faps[frame][setfap_id-1],TRUE);
		break;
		

	case ID_FAP_FRAMEPLUS:
		
		if(frame+1<fapeditor.framenumber) {
			fapeditor.faps[frame][setfap_id-1] = setfap;
			frame++;
			SetDlgItemInt(hDlg,ID_FAPFRAMENO,frame,FALSE);
			SetDlgItemInt(hDlg,ID_FAPVALEDIT,fapeditor.faps[frame][setfap_id-1],TRUE);
			ShowCurrentEditFace();
		}
		break;
		
	case ID_FAP_FRAMEMINUS:
		
		if(frame-1>=0) {
			fapeditor.faps[frame][setfap_id-1] = setfap;
			frame--;
			SetDlgItemInt(hDlg,ID_FAPFRAMENO,frame,FALSE);
			SetDlgItemInt(hDlg,ID_FAPVALEDIT,fapeditor.faps[frame][setfap_id-1],TRUE);
			ShowCurrentEditFace();
		}
		break;
		
	case ID_FAP_PLUS:
		
		if(setfap_id+1<NO_FAPS) {
			setfap_id++;
			SetDlgItemInt(hDlg,ID_FAPEDIT,setfap_id,FALSE);
			SetDlgItemInt(hDlg,ID_FAPVALEDIT,fapeditor.faps[frame][setfap_id-1],TRUE);
		}
		break;
		
	case ID_FAP_MINUS:
		
		if(setfap_id-1>0) {
			setfap_id--;
			SetDlgItemInt(hDlg,ID_FAPEDIT,setfap_id,FALSE);
			SetDlgItemInt(hDlg,ID_FAPVALEDIT,fapeditor.faps[frame][setfap_id-1],TRUE);
		}
		break;
		
	case ID_FAPVAL_PLUS:
		
		setfap++;
		SetDlgItemInt(hDlg,ID_FAPVALEDIT,setfap,TRUE);
		fapeditor.faps[frame][setfap_id-1] = setfap;
		ShowCurrentEditFace();
		break;	
		
		
	case ID_FAPVAL_MINUS:
		
		setfap--;
		SetDlgItemInt(hDlg,ID_FAPVALEDIT,setfap,TRUE);
		fapeditor.faps[frame][setfap_id-1] = setfap;
		ShowCurrentEditFace();
		break;


	case ID_EXPR_SEL:
		
		ApplyExpressionToEditFAP(hDlg);
		break;
		

	case ID_EXPR_INT:

		ApplyExpressionToEditFAP(hDlg);
		break;
		

	case ID_EXPR_SEL_PLUS:
		
		if(setexpr+1<=6) {
			setexpr++;
			SetDlgItemInt(hDlg,ID_EXPR_SEL,setexpr,FALSE);
			ApplyExpressionToEditFAP(hDlg);
		}
		break;


	case ID_EXPR_SEL_MINUS:
		
		if(setexpr-1>=0) {
			setexpr--;
			SetDlgItemInt(hDlg,ID_EXPR_SEL,setexpr,FALSE);
			ApplyExpressionToEditFAP(hDlg);
		}
		break;

		
	case ID_EXPR_INT_PLUS:
		
		if(setexprint+1<=63) {
			setexprint++;
			SetDlgItemInt(hDlg,ID_EXPR_INT,setexprint,FALSE);
			ApplyExpressionToEditFAP(hDlg);
		}
		break;


	case ID_EXPR_INT_MINUS:
		
		if(setexprint-1>=0) {
			setexprint--;
			SetDlgItemInt(hDlg,ID_EXPR_INT,setexprint,FALSE);
			ApplyExpressionToEditFAP(hDlg);
		}
		break;

		
	case IDOK: 

		EndDialog(hDlg,LOWORD(wParam));
		editcancel = OFF;
		break;
	

	case IDCANCEL:
	
		FreeFAPeditor();
		EndDialog(hDlg,LOWORD(wParam));
		editcancel = ON;
		break;


	default:
		break;

	}

	return;
}





int coder_setting ;
int codereditfirst,codertest;

int coding_type,fap_quant,IPrange;


void SetInitValues()
{
	coder_setting = OFF;
	codertest = OFF;

	fap_quant  = 1;
	coding_type = ARITH_TYPE;
	IPrange     = 10;
}


BOOL CALLBACK Coder_DlgProc(HWND hDlg , UINT Message, WPARAM wParam, LPARAM lParam)
{
	switch(Message) {

	case WM_INITDIALOG:	

		codereditfirst = ON;

		SetInitValues();

		SetDlgItemInt(hDlg,ID_QSTEP,fap_quant,FALSE);
		SetDlgItemInt(hDlg,ID_IPRANGE,IPrange,FALSE);

		Button_SetCheck(GetDlgItem(hDlg,ID_FRAMEBASED),ON);
		Button_SetCheck(GetDlgItem(hDlg,ID_DCTBASED),OFF);

		codereditfirst = OFF;

		return TRUE;
		break;

	case WM_COMMAND:

		Handle_CoderDLG_Command(hDlg,wParam,lParam);
		return TRUE;
		break;
	}
	return FALSE;
}




void Handle_CoderDLG_Command(HWND hDlg,WPARAM wParam, LPARAM lParam)
{
	int test,temp,id = LOWORD(wParam);

	if(codereditfirst || codertest) return;

	switch(id) {
		
	case ID_FRAMEBASED:
		codertest = ON;
		if(Button_GetCheck(GetDlgItem(hDlg,ID_FRAMEBASED)))
			coding_type = ARITH_TYPE;
		codertest = OFF;
		break;

	case ID_DCTBASED:
		codertest = ON;
		if(Button_GetCheck(GetDlgItem(hDlg,ID_DCTBASED)))
			coding_type = DCT_TYPE;
		codertest = OFF;
		break;

	case ID_QSTEP:
		codertest = ON;
		temp = GetDlgItemInt(hDlg,ID_QSTEP,&test,FALSE);
		if( (temp<1 || temp>31) && test) 
			SetDlgItemInt(hDlg,ID_QSTEP,fap_quant,FALSE);
		else if(test) 
			fap_quant = temp;
		codertest = OFF;
		break;


	case ID_IPRANGE:
		codertest = ON;
		temp = GetDlgItemInt(hDlg,ID_IPRANGE,&test,FALSE);
		if( temp<1  && test) 
			SetDlgItemInt(hDlg,ID_IPRANGE,IPrange,FALSE);
		else if(test) 
			IPrange = temp;
		codertest = OFF;
		break;

	case IDOK:
		coder_setting = ON;
		EndDialog(hDlg,LOWORD(wParam));
		break;

	case IDCANCEL:
		coder_setting = OFF;
		EndDialog(hDlg,LOWORD(wParam));
		break;

	default:
		break;
	}

	return;
}






int SaveFAPorStreamFile(HWND hWnd,char *filename, char *extension)
{
	int counter,i;
	FILE *fp;
	HCURSOR		 hCursor;

	if( (fp = fopen(filename,"wb")) == NULL)
		return OFF;

	if(!strcmp(extension,"fap")) {
		fprintf(fp,"2.0 %s %.2f %d\n",filename,fapeditor.framerate,fapeditor.framenumber);
		
		for(counter=0;counter<fapeditor.framenumber;counter++) {
			
			for(i=0;i<NO_FAPS;i++) {
				if(fapeditor.faps[counter][i])
					fapeditor.mask[counter][i]=ON;
				fprintf(fp,"%d ",fapeditor.mask[counter][i]);
			}
			fprintf(fp,"\n");
			
			fprintf(fp,"%d ",counter);
			for(i=0;i<NO_FAPS;i++)
				if(fapeditor.mask[counter][i])
					fprintf(fp,"%d ",fapeditor.faps[counter][i]);
				fprintf(fp,"\n");
		}
		
		fprintf(fp,"\n");
	}
	else if(!strcmp(extension,"stream")) {

		DialogBox(hInst,"CODER_BOX",hWnd,Coder_DlgProc);

		sprintf(word,"Coding FAPsequence, please wait...");
		ShowText(word);
		hCursor = SetCursor(LoadCursor (NULL, IDC_WAIT));

		if(coder_setting)
			CodeFapFile(fp);

		SetCursor (hCursor);
		
	}


	fclose(fp);
	FreeFAPeditor();

	return ON;
}




