/********************************************************************/
/* 																	*/
/* This software module was originally developed and edited by		*/
/* Gabriel Antunes Abrantes (Instituto Superior Tecnico)			*/	
/* 																	*/
/* in the course of development of the MPEG-4 (Visual) standard		*/
/* (ISO/IEC 14496-2).	 											*/
/* This software module is an implementation of a part of one or 	*/
/* more MPEG-4 tools as specified by the	 						*/
/* MPEG-4 standard. ISO/IEC gives users of the MPEG-4 standard		*/
/* free license to this software module or modifications thereof 	*/
/* for use in hardware or software products claiming conformance 	*/
/* to the MPEG-4 standard. Those intending to use this software 	*/
/* module in hardware or software products are advised that its 	*/
/* use may infringe existing patents. The original developer of		*/
/* this software module and his company, the subsequent 			*/
/* editors and their companies, and ISO/IEC have no liability		*/
/* for use of this software module or modifications thereof in 		*/
/* an implementation. Copyright is not released for non				*/
/* MPEG-4 standard conforming products. ACTS-MoMuSys partners	 	*/
/* retain full right to use the code for their own purposes, 		*/
/* assign or donate the code to a third party and to inhibit 		*/
/* third parties from using the code for non MPEG-4 standard		*/
/* conforming products. This copyright notice must be included 		*/
/* in all copies or derivative works. 								*/
/* Copyright (c)1997/98												*/
/* 																	*/
/********************************************************************/



/***********************************HeaderBegin****************************
*   File : ISTOGLface.c                             
*   
*   Author  : Gabriel Antunes Abrantes(gabriel.abrantes@lx.it.pt)
*   Created : September-1997 
*
*   Description : module for 3D model displaying (in OpenGL)
*
*   Version 1.0
*
****************************************HeaderEnd**************************/




#include "face.h"
#include "anim.h"


#define NO_COLOURS      11
#define NEUTRAL          9

#define SIZE_FACTOR   4.5
#define ORTHO_FACTOR  2.4
#define NEARFAR       20.0

GLfloat InitLight[5][4] = { { 0.15f, 0.15f, 0.15f, 1.0f },			//ambient
							{ 0.7f , 0.7f , 0.7f , 1.0f },			//diffuse			
							{ 0.7f , 0.7f , 0.7f , 1.0f },			//specular
							{ 0.35f, 0.35f, 0.35f, 1.0f }  };		//lmodel_amb
							 			
GLfloat scale_factor;
GLfloat defscale;

GLfloat ModelShininess[NO_COLOURS] = {	100.0f, 64.0f, 100.0f, 60.0f, 90.0f, 60.0f, 60.0f, 
										128.0f, 70.0f, 100.0f, 0.0f};

GLfloat ModelColours[3*NO_COLOURS][4] = {
	{0.5f,0.3f,0.0f,1.0f},	{0.2f,0.2f,0.2f,1.0f},	{0.5f,0.5f,0.5f,1.0f}, //hair,eyebrows
	{0.7f,0.2f,0.4f,1.0f},	{1.0f,0.5f,0.4f,1.0f},	{1.0f,0.7f,0.7f,1.0f}, //tongue
	{1.0f,0.9f,0.8f,1.0f},	{1.0f,0.7f,0.6f,1.0f},	{0.5f,0.5f,0.5f,1.0f}, //skin
	{0.8f,0.2f,0.3f,1.0f},	{1.0f,0.5f,0.4f,1.0f},	{1.0f,0.7f,0.7f,1.0f}, //mouth
	{0.4f,0.4f,0.8f,1.0f},	{0.4f,0.4f,0.8f,1.0f},	{0.7f,0.7f,0.9f,1.0f}, //fringe
	{0.0f,0.0f,0.0f,1.0f},	{0.0f,0.0f,0.0f,1.0f},	{0.5f,0.5f,0.5f,1.0f}, //pupil 
	{0.5f,0.7f,0.4f,1.0f},	{0.4f,0.4f,0.8f,1.0f},	{0.5f,0.5f,0.8f,1.0f}, //iris
	{0.8f,0.8f,0.8f,1.0f},	{1.0f,1.0f,0.9f,1.0f},	{1.0f,1.0f,1.0f,1.0f}, //eyewhite
	{0.8f,0.8f,0.8f,1.0f},	{0.9f,0.9f,0.9f,1.0f},	{0.8f,0.8f,0.8f,1.0f}, //teeth
	{0.8f,0.8f,0.8f,1.0f},	{0.8f,0.8f,0.8f,1.0f},	{0.8f,0.8f,0.8f,1.0f}, //neutral
	{0.0f,0.0f,0.0f,1.0f},	{0.0f,0.0f,0.0f,1.0f},	{0.0f,0.0f,0.0f,1.0f}, //black
};

 

int CreatePixelFormat(HDC hdc)
{	
	int pxl_format;

	static PIXELFORMATDESCRIPTOR pfd = {
		sizeof(PIXELFORMATDESCRIPTOR),
		1, PFD_DRAW_TO_WINDOW | PFD_SUPPORT_OPENGL | PFD_DOUBLEBUFFER,		
		PFD_TYPE_RGBA, 24, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 32, 0, 0, 
		PFD_MAIN_PLANE, 0, 0, 0, 0 		
	};
        
	pxl_format = ChoosePixelFormat(hdc, &pfd);

	if (!pxl_format) {
		MessageBox(NULL, "Error in ChoosePixelFormat ", "Face", MB_OK);
		return OFF;
	}

	if (SetPixelFormat(hdc, pxl_format, &pfd) == FALSE)	{
		MessageBox(NULL, "Error in SetPixelFormat ", "Face", MB_OK);
		return OFF;
	}

	return ON;
}



void OGL_Initialize(HWND hWnd)
{
	RECT hwindow;
	GLfloat scale;

	glLightfv(GL_LIGHT1, GL_AMBIENT, InitLight[0]);
	glLightfv(GL_LIGHT1, GL_DIFFUSE, InitLight[1]);
	glLightfv(GL_LIGHT1, GL_SPECULAR, InitLight[2]);
	glLightModelfv(GL_LIGHT_MODEL_AMBIENT, InitLight[3]);

	glEnable(GL_LIGHTING);
	glEnable(GL_LIGHT1);
	glEnable(GL_DEPTH_TEST);

	glClearColor(0.0f, 0.3f, 0.5f, 1.0f);	//background

	GetClientRect(hWnd, &hwindow);

	glMatrixMode(GL_PROJECTION);
	glLoadIdentity();

	if(hwindow.right>(GLfloat)hwindow.bottom)
		scale = (GLfloat)hwindow.right/(GLfloat)hwindow.bottom;
	else 
		scale = (GLfloat)hwindow.bottom/(GLfloat)hwindow.right;

	glOrtho(-ORTHO_FACTOR*scale,ORTHO_FACTOR*scale, -ORTHO_FACTOR, ORTHO_FACTOR, -NEARFAR, 
			NEARFAR);

	glViewport(hwindow.left, hwindow.top, hwindow.right, hwindow.bottom);

	glClearDepth( 1.0 );
}



 
void GetModelSize()
{
	float max_val,min_val,size;
	int   counter,pos;

	max_val=0;
	min_val=0;
	
	for(counter=0; counter< NO_POINTS ; counter++) {
		for(pos=X; pos<=Z; pos++) {
			if(points_orig[counter][pos]<min_val)		min_val = points_orig[counter][pos];
			else if(points_orig[counter][pos]>max_val)	max_val = points_orig[counter][pos];
		}
	}	

	size = max_val - min_val;

	scale_factor = SIZE_FACTOR/size;
	defscale = scale_factor;
}




void ResizeViewPort(HWND hWnd, HWND sWnd)
{
	RECT hwindow,swindow;
	
	GetClientRect(hWnd, &hwindow);	
	GetClientRect(sWnd, &swindow);

	glViewport(hwindow.left, swindow.bottom, hwindow.right, hwindow.bottom-swindow.bottom);
}




void BuildFaceModel()
{
	int i,j;

	glEnable(GL_NORMALIZE);

	if(model==GRID) 
		glDisable(GL_CULL_FACE);
	else {
		glEnable(GL_CULL_FACE);
		glCullFace(GL_BACK);
	}


	if(!texture){
		glMaterialfv( GL_FRONT, GL_AMBIENT,   ModelColours[3*9]);
		glMaterialfv( GL_FRONT, GL_DIFFUSE,   ModelColours[3*9+1]);
		glMaterialfv( GL_FRONT, GL_SPECULAR,  ModelColours[3*9+2]);
		glMaterialf ( GL_FRONT, GL_SHININESS, ModelShininess[9]);
	}		

	for(i=0 ; i<NO_VERTX; i++) {


		if(texture) {
			if( (vertx[i][3]!=vertx[i-1][3]) || (i==0) ) {
				glMaterialfv( GL_FRONT, GL_AMBIENT,   ModelColours[3*vertx[i][3]]);
				glMaterialfv( GL_FRONT, GL_DIFFUSE,   ModelColours[3*vertx[i][3]+1]);
				glMaterialfv( GL_FRONT, GL_SPECULAR,  ModelColours[3*vertx[i][3]+2]);
				glMaterialf ( GL_FRONT, GL_SHININESS, ModelShininess[vertx[i][3]]);
			}
		}		

		glBegin(GL_POLYGON);
		for(j=0;j<3;j++) {
			glNormal3fv(norms[vertx[i][j]]);
			glVertex3fv(pts[vertx[i][j]]);
		}
		glEnd();

	}


}



void Paint_OGL_FaceModel(void)
{
	HDC hDC;

	glClear( GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT );

	if(model==GRID) 
		glPolygonMode(GL_FRONT_AND_BACK,GL_LINE);
	else 
		glPolygonMode(GL_FRONT_AND_BACK,GL_FILL);

	glShadeModel(GL_SMOOTH);
	glMatrixMode(GL_MODELVIEW);
	glLoadIdentity();

	glPushMatrix();

	glScalef(scale_factor,scale_factor,scale_factor);
		
	BuildFaceModel();
 	
	glPopMatrix();
	
	hDC = wglGetCurrentDC();
	SwapBuffers(hDC);
}





 