/********************************************************************/
/* 																	*/
/* This software module was originally developed and edited by		*/
/* Gabriel Antunes Abrantes (Instituto Superior Tcnico)			*/	
/* 																	*/
/* in the course of development of the MPEG-4 (Visual) standard		*/
/* (ISO/IEC 14496-2).	 											*/
/* This software module is an implementation of a part of one or 	*/
/* more MPEG-4 tools as specified by the	 						*/
/* MPEG-4 standard. ISO/IEC gives users of the MPEG-4 standard		*/
/* free license to this software module or modifications thereof 	*/
/* for use in hardware or software products claiming conformance 	*/
/* to the MPEG-4 standard. Those intending to use this software 	*/
/* module in hardware or software products are advised that its 	*/
/* use may infringe existing patents. The original developer of		*/
/* this software module and his company, the subsequent 			*/
/* editors and their companies, and ISO/IEC have no liability		*/
/* for use of this software module or modifications thereof in 		*/
/* an implementation. Copyright is not released for non				*/
/* MPEG-4 standard conforming products. ACTS-MoMuSys partners	 	*/
/* retain full right to use the code for their own purposes, 		*/
/* assign or donate the code to a third party and to inhibit 		*/
/* third parties from using the code for non MPEG-4 standard		*/
/* conforming products. This copyright notice must be included 		*/
/* in all copies or derivative works. 								*/
/* Copyright (c)1997/98 											*/
/* 																	*/
/********************************************************************/




/***********************************HeaderBegin****************************
*   File : ISTanim_file.c                             
*   
*   Author  : Gabriel Antunes Abrantes(gabriel.abrantes@lx.it.pt)
*   Created : September-1997 
*
*   Description : module for handling of FAP files and FAP bitstreams
*   
*   Version 1.0 
*
****************************************HeaderEnd**************************/




#include <stdio.h>
#include <math.h>
#include <stdlib.h>
#include <ctype.h>
#include <string.h>
#include <time.h> 
#include "face.h"
#include "anim.h"
#include "decoder.h"

#define DEBUG 0


#define BUFFER_SIZE 256

#define NO_FACE_REGIONS  10

#define NO_PAIRS 24
#define INNER_LIPS 1
#define OUTER_LIPS 7

int LeftRightPairs[NO_PAIRS] = { 6,8,10,12,19,21,23,25,27,29,31,33,35,37,39,41,
							     53,55,57,59,61,63,65,67};

int LRP_GROUP[10] = { 0,4,10,14,16,16,16,20,22,24 };


float  points_orig[NO_POINTS][3]; 
float  pts[NO_POINTS][3]; 
int    vertx[NO_VERTX][4];
float  norms[NO_POINTS][3];

float  faps[NO_FAPS+1];
int    fap_mask[NO_FAPS+1];
int	   fap_expr[6];
int	   fap_vis[4];
int    ISTexpressions[7][NO_FAPS+1];




static FILE *fp_fap = NULL;
int    Nframe;
float  fps;
long   sequence_start;

int    no_frame_display;



int Pass(FILE *fp) 
{
	int c;
	
	while(c = getc(fp),(!isalnum(c)) && (c!=EOF) );
	
	return ungetc(c,fp);
}


int PassToNumber(FILE *fp) 
{
	int c;
	
	while(c = getc(fp),(!isdigit(c)) && (c!=EOF) && (c!='-') );
	
	return ungetc(c,fp);
}


int GetInt(FILE *fp)
{
	int get;

	PassToNumber(fp);
	fscanf(fp,"%d", &get);
	return get;
}




int ReadWord(FILE *fp,char *s)
{
  int c;
  
  while(c = getc(fp),!isspace(c)) 
    *s++ = c;
  *s='\0';
  
  return ungetc(c,fp);
}



int PassBlank(FILE *fp) 
{
  int c;
  
  while(c = getc(fp),isspace(c));
  
  return ungetc(c,fp);
}


 
void KillLine(FILE *fp)
{ 
  int c;

  while((c = getc(fp)) != EOF && c != '\n')
    continue;
}




void TestforTrash(FILE *fp)
{
  while(ungetc(getc(fp),fp) == '#')
	  KillLine(fp);
}



int GetPoints(void)
{
	FILE *fp_pts;
	int  points_number,no,a;
	char trash[BUFFER_SIZE];
	
	
	char location[BUFFER_SIZE];
	char *comline = GetCommandLine();
	
	int i = 1;
	a = 0;

	while(comline[i]!='"')
		location[a++] = comline[i++];
	location[a] = '\0';
	
	while(location[a]!='\\')
		location[a--] = '\0';
	location[a] = '\0';
	
	
	sprintf(word,"%s/ISTfacepoints",location);
	if( (fp_pts=fopen(word,"rb")) == NULL)
		return OFF;
		
	fscanf(fp_pts,"%d",&points_number);
	
	fgets(trash,BUFFER_SIZE,fp_pts);
	
	no=0; 
	while(no!=points_number) {
		fscanf(fp_pts,"%f",&points_orig[no][0]);
		fscanf(fp_pts,"%f",&points_orig[no][1]);
		fscanf(fp_pts,"%f",&points_orig[no][2]);
		fgets(trash,BUFFER_SIZE,fp_pts);
		a = points_orig[no][0]-points_orig[no][1];
		no++;
	}
	
	fclose (fp_pts);
	
	return ON;
}



void ResetExpressionVisemesTable()
{
	int i,j,k;

	for(i=0;i<384;i++)
		for(j=0;j<384;j++)
			expressions_out[i][j] = NULL;			

	for(i=0;i<15;i++)
		for(j=0;j<15;j++)
			for(k=0;k<64;k++)
				visemes_out[i][j][k] = NULL;

	for(i=0;i<15;i++)
		for(j=0;j<=NO_FAPS;j++)
			vis_table[i][j] = 0;
}



void FreeExpressionVisemesTable()
{
	int i,j,k;

	for(i=0;i<384;i++)
		for(j=0;j<384;j++)
			if(expressions_out[i][j] != NULL)
				free(expressions_out[i][j]);			

	for(i=0;i<15;i++)
		for(j=0;j<15;j++)
			for(k=0;k<64;k++)
				if(visemes_out[i][j][k] != NULL)
					free(visemes_out[i][j][k]);			

}



int GetVertex()
{
	FILE *fp_vertx;
	int  no,n1,counter,colour,no_verts,c;
	char trash[BUFFER_SIZE];
	
	char location[BUFFER_SIZE];
	char *comline = GetCommandLine();
	
	int i = 1,a = 0;

	while(comline[i]!='"')
		location[a++] = comline[i++];
	location[a] = '\0';
	
	while(location[a]!='\\')
		location[a--] = '\0';
	location[a] = '\0';
	
	sprintf(word,"%s/ISTfacevertx",location);
	if( (fp_vertx=fopen(word,"rb")) == NULL)
		return OFF;
	
	
	ResetExpressionVisemesTable();
	
	n1 = 0; 
	
	for(counter = 0; counter < NO_FACE_REGIONS ; counter++) {
		
		TestforTrash(fp_vertx);
		
		fscanf(fp_vertx,"%d %d ",&colour,&no_verts);
		
		for(no=0;no<no_verts;no++) {
			
			fscanf(fp_vertx,"%d",&vertx[n1][V1]);
			while(c = getc(fp_vertx),isspace(c) || ispunct(c));
			ungetc(c,fp_vertx);
			
			fscanf(fp_vertx,"%d",&vertx[n1][V2]);
			while(c = getc(fp_vertx),isspace(c) || ispunct(c));
			ungetc(c,fp_vertx);
			
			fscanf(fp_vertx,"%d",&vertx[n1][V3]);
			fgets(trash,BUFFER_SIZE,fp_vertx);
			vertx[n1][COLOUR] = colour;
			
			n1++;
		}
		
	}
	
	fclose (fp_vertx);
	
	return ON;
	
}


void GetFapUnits(void)
{
  irisd = (points_orig[213][Y]-points_orig[191][Y])/1024;

  es    = (points_orig[553][X]-points_orig[681][X])/1024;
  ens   = (points_orig[553][Y]-points_orig[86][Y])/1024;
  mns   = (points_orig[86][Y]-points_orig[60][Y])/1024;
  mw    = (points_orig[69][X]-points_orig[345][X])/1024;

  au    = 1e-5f;
}




void GetExpressions()
{
	int  counter,exp,i,a;
	FILE *fp_exp;
	char trash[BUFFER_SIZE];
	
	
	char location[BUFFER_SIZE];
	char *comline = GetCommandLine();
	
	i = 1;
	a = 0;

	while(comline[i]!='"')
		location[a++] = comline[i++];
	location[a] = '\0';
	
	while(location[a]!='\\')
		location[a--] = '\0';
	location[a] = '\0';
	
	
	sprintf(word,"%s/IST_expressions",location);
	if( (fp_exp  = fopen(word,"r"))==NULL ) {
		sprintf(word,"\n IST-expressions file IST_expressions not available\n");
		WindowPrintf();
		return;
	}
	
	for(counter=0;counter<=NO_FAPS;counter++)
		ISTexpressions[0][counter] = 0;
	
	
	for(exp=1;exp<=6;exp++) {
		
		TestforTrash(fp_exp);
		
		/*read mask */
		
		for(counter=1;counter<=NO_FAPS;counter++) 
			ISTexpressions[exp][counter] = GetInt(fp_exp);
		
		/* read faps */
		
		for(counter=1;counter<=NO_FAPS;counter++) {
			if(ISTexpressions[exp][counter]==ON)
				ISTexpressions[exp][counter] = GetInt(fp_exp);
		}
		
		fgets(trash,BUFFER_SIZE,fp_exp);
	}
	
	fclose (fp_exp);
	
}



void Reset(int no_points, int first)
{
	int counter,pos;
  
	/* reset points,normals & faps */

	for(counter=0;counter<no_points;counter++)
		for(pos=X;pos<=Z;pos++)	{
			pts[counter][pos] = points_orig[counter][pos];
			norms[counter][pos] = 0;
		}

	if(first)
		for(counter=0;counter<=NO_FAPS;counter++) 
			faps[counter] = 0;
  
}




int ReadFapHeader(char *fap_file)
{
	float version;
	
	CloseOpenFiles();
	
	no_frame_display = 0;
	ShowText("");
	
	sprintf(word,"%s",fap_file);
	if( (fp_fap  = fopen(word,"rb"))==NULL ) 
		return OFF;
	
	TestforTrash(fp_fap);    
	fscanf(fp_fap,"%f",&version);
	
	if(version<2.0){
		sprintf(word,"Only FAPfiles version 2.+ are supported, not %f\n",version);
		WindowPrintf();
		fclose(fp_fap);
		return OFF;
	}
	
	PassBlank(fp_fap);
	ReadWord(fp_fap,word);
	fscanf(fp_fap,"%f",&fps);
	fscanf(fp_fap,"%d",&Nframe);

	KillLine(fp_fap);

	sequence_start = ftell(fp_fap);
	
	Reset(NO_POINTS,ON);
	
	FreeExpressionVisemesTable();
	ResetExpressionVisemesTable();

	return ON;
}



int ReadFapFrame()
{
	int counter;
	int frame_number;
	
	TestforTrash(fp_fap);
	for(counter=1;counter<=NO_FAPS;counter++) 
		fscanf(fp_fap,"%d",&fap_mask[counter]);
	KillLine(fp_fap);
    
	TestforTrash(fp_fap);    
	
	fscanf(fp_fap,"%d",&frame_number);
	
	sprintf(word,"Frame : %d",frame_number);
	ShowText(word);

	if(fap_mask[1]) {
		for(counter=0;counter<4;counter++)
			GetInt(fp_fap);	
	}
	
	if(fap_mask[2]) {
		for(counter=0;counter<6;counter++)
			fap_expr[counter] = GetInt(fp_fap);	
	}

	for(counter=3;counter<=NO_FAPS;counter++)
		if(fap_mask[counter]==ON)
			fscanf(fp_fap,"%f",&faps[counter]);
		
	KillLine(fp_fap);
		
	FAP_Interpolation();
	Expression();
	Visemes();

	return frame_number;
}






void NormalizeVector(float *vec)
{
	float s;
	int   i;

	s = sqrt(sqr(vec[0]) + sqr(vec[1]) + sqr(vec[2]) );

	for(i=0;i<3;i++)
		vec[i]/=s;
}



void ObtainNormal(float *vector1, float *vector2, float *vector3, float *norm)
{
	norm[X] = vector1[Y]*(vector2[Z] - vector3[Z]) + vector2[Y]*(vector3[Z] - vector1[Z])
		+ vector3[Y]*(vector1[Z] - vector2[Z]);
	
	norm[Y] = vector1[X]*(vector3[Z] - vector2[Z]) + vector2[X]*(vector1[Z] - vector3[Z])
		+ vector3[X]*(vector2[Z] - vector1[Z]);
	
	norm[Z] = vector1[X]*(vector2[Y] - vector3[Y]) + vector2[X]*(vector3[Y] - vector1[Y])
		+ vector3[X]*(vector1[Y] - vector2[Y]);

	NormalizeVector(norm);
}




void ObtainVertsNormals()
{
	int   counter,i,j;
	float normal[3];

	for(counter=0;counter<NO_VERTX;counter++) {
		ObtainNormal(pts[vertx[counter][0]],pts[vertx[counter][1]],pts[vertx[counter][2]],normal);

	for(i=0;i<3;i++)
		for(j=0;j<3;j++) 
			norms[vertx[counter][i]][j] += normal[j];
	}

}




void FAP_Interpolation(void)
{
	int counter,i;

	if(file_type==FAP) {

		/* left-right interpolation */
		for(counter=1;counter<GROUP_MASK_SIZE;counter++) {
			for(i=LRP_GROUP[counter-1];i<LRP_GROUP[counter];i++) {
				
				if(!fap_mask[LeftRightPairs[i]])// &&  ((int)faps[LeftRightPairs[i]]==0))
					faps[LeftRightPairs[i]] = faps[LeftRightPairs[i]+1];
				
				if(!fap_mask[LeftRightPairs[i]+1])// && ((int)faps[LeftRightPairs[i]+1]==0))
					faps[LeftRightPairs[i]+1] = faps[LeftRightPairs[i]];
			}
		}

		/* lips interpolation */
		
		for(counter=51;counter<=60;counter++) {
			if( (fap_mask[counter] == OFF) && ((int)faps[counter-47]!=0) )
				faps[counter] = faps[counter-47];
		}

		for(counter=4;counter<=13;counter++) {
			if( (fap_mask[counter] == OFF) && ((int)faps[counter+47]!=0) )
				faps[counter] = faps[counter+47];
		}

		if(fap_mask[3]) faps[44] = -1*faps[3];		

	}
	
	if(file_type==STREAM) {

		for(counter=1;counter<GROUP_MASK_SIZE;counter++) 
			if(mask_type[counter] == FAP_MASK_TYPE2) {

				/* left-right interpolation */
				
				for(i=LRP_GROUP[counter-1];i<LRP_GROUP[counter];i++) {
					
					if((fap_mask[LeftRightPairs[i]] == OFF) &&  ((int)faps[LeftRightPairs[i]]==0))
						faps[LeftRightPairs[i]]=faps[LeftRightPairs[i]+1];
					
					if((fap_mask[LeftRightPairs[i]+1] == OFF) && ((int)faps[LeftRightPairs[i]+1]==0))
						faps[LeftRightPairs[i]+1]=faps[LeftRightPairs[i]];
				}

				/* lips interpolation */
			
				if(counter==INNER_LIPS)
					for(i=4;i<=13;i++) 
						if( (fap_mask[i] == OFF) && ((int)faps[i+47]!=0) )
							 faps[i] = faps[i+47];

				if(counter==OUTER_LIPS)
					for(i=51;i<=60;i++) 
						if( (fap_mask[i] == OFF) && ((int)faps[i-47]!=0) )
							 faps[i] = faps[i-47];
			}
	}
}



 
void AnimateOneFrame(int  frame_counter)
{
    int  frame_number;

	if( (no_frame_display==0) && (file_type==FAP))
		Reset(NO_POINTS,ON);
	else Reset(NO_POINTS,OFF);

	if(file_type==FAP) {
		if( (frame_number=ReadFapFrame()) != frame_counter) {
			sprintf(word,"Different frames: %d (expected) vs %d\n",frame_counter,frame_number);
			WindowPrintf();
		}
	}

	FrameAnimation();

	ObtainVertsNormals();
}





int TestConditions()
{
	if(no_frame_display>=Nframe) {
		sprintf(word,"No more frames, sequence at end!\n");
		WindowPrintf();
		return OFF;
	}

	return ON;
}





int ShowNeutralFace()
{
	Reset(NO_POINTS,OFF);
	ObtainVertsNormals();

	SendPaintMsg();

	sprintf(word,"Neutral Face");
	ShowText(word);

	return ON;
}



int Show1Frame()
{
	if(file_type==FAP) 
		if(!TestConditions())
			return OFF;

	AnimateOneFrame(no_frame_display);
	no_frame_display++;

	SendPaintMsg();

	return ON;
}




int PlaySequence()
{
	if(!TestConditions()) return ON;

	while(no_frame_display < Nframe) {
		if(!Show1Frame())
			return OFF;
		if(StopSequence())
			return ON;
	}

	return ON;
}




int PlayRealTime(void)
{
	time_t current_time;	 
	int    skipped_frames;		 
	float  frame_duration;


	if(!TestConditions()) return ON;

	skipped_frames = 0;
	frame_duration   = (float)(CLOCKS_PER_SEC/fps);
	current_time   = clock();

	while(no_frame_display < Nframe) {
		
		while(current_time>clock());
		
		if(Show1Frame()==0)
			return 0;

		current_time+=frame_duration;
	
		while(current_time<clock()) {
		
			if(no_frame_display>=Nframe-1)
				break;					
			
			ReadFapFrame();

			no_frame_display++;
			skipped_frames++;
			current_time+=frame_duration;
		}

		if(StopSequence())
			return ON;
	}

	sprintf(word,"Sequence has %d frames at %d fps, %d were skipped, average frame_rate: %.2f",
			Nframe,(int)fps,skipped_frames,(Nframe-skipped_frames)*fps/Nframe);
	ShowText(word);

	return 1;
}




void RewindSequence()
{	
	ShowNeutralFace();

	no_frame_display = 0;

	fseek(fp_fap,sequence_start,SEEK_SET);

	return;
}



void CloseOpenFiles()
{
	if(fp_fap!=NULL) {
		fclose(fp_fap);
		fp_fap=NULL;
	}

	if(fp_stream!=NULL) {
		fclose(fp_stream);
		fp_stream=NULL;
	}
}



