/********************************************************************/
/* 																	*/
/* This software module was originally developed and edited by		*/
/* Gabriel Antunes Abrantes (Instituto Superior Tecnico)			*/	
/* 																	*/
/* in the course of development of the MPEG-4 (Visual) standard		*/
/* (ISO/IEC 14496-2).	 											*/
/* This software module is an implementation of a part of one or 	*/
/* more MPEG-4 tools as specified by the	 						*/
/* MPEG-4 standard. ISO/IEC gives users of the MPEG-4 standard		*/
/* free license to this software module or modifications thereof 	*/
/* for use in hardware or software products claiming conformance 	*/
/* to the MPEG-4 standard. Those intending to use this software 	*/
/* module in hardware or software products are advised that its 	*/
/* use may infringe existing patents. The original developer of		*/
/* this software module and his company, the subsequent 			*/
/* editors and their companies, and ISO/IEC have no liability		*/
/* for use of this software module or modifications thereof in 		*/
/* an implementation. Copyright is not released for non				*/
/* MPEG-4 standard conforming products. ACTS-MoMuSys partners	 	*/
/* retain full right to use the code for their own purposes, 		*/
/* assign or donate the code to a third party and to inhibit 		*/
/* third parties from using the code for non MPEG-4 standard		*/
/* conforming products. This copyright notice must be included 		*/
/* in all copies or derivative works. 								*/
/* Copyright (c)1997/98												*/
/* 																	*/
/********************************************************************/


/***********************************HeaderBegin****************************
*   File : IST_coder.c                              
*   
*   Author  : Gabriel Antunes Abrantes
*   Created : 23-July-1997 
*
*   Description : Face Object Coder
*
*   Modified:
*   Version 4.1 / February-1998
*   Updated for  MPEG-4 Tokyo Reunion, includes frame based 
*						and DCT based coding modes
*
****************************************HeaderEnd**************************/



#include <stdio.h>
#include <math.h>
#include <stdlib.h>
#include <ctype.h>
#include <string.h>
#include "face.h"
#include "coder.h"
#include "anim.h"
#include "decoder.h"
#include "editor.h"

/* arithmetic coding */

#define   IrangeMin		  -900
#define   IrangeMax		   900
#define   rangeMin		  -900
#define   rangeMax		   900

#define   RANGE_DELTA       60


int FapQuantSteps[NO_FAPS] ={ 1, 1, 4, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 1, 1, 1, 1, 1,
							  1, 1, 1, 1, 128, 128, 128, 128, 1, 1, 1, 1, 2, 2, 
							  2, 2, 2, 2, 1, 1, 2, 2, 2, 2, 1, 1, 1, 1, 512, 128,
							  128, 128, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 1, 1, 1, 1, 
							  1, 1, 1, 1 };



int GroupMaskLimits[11] =  { 0,2,18,30,38,42,47,50,60,64,68 };




float DCTQuantSteps[NO_FAPS]  = { 1, 1, 7.5, 7.5, 7.5, 7.5, 7.5, 7.5, 7.5, 7.5,
								  7.5, 7.5, 7.5,  15, 15, 15, 15, 5, 10,10,
								  10, 10, 425, 425, 425, 425, 5, 5, 5, 5,
								  7.5, 7.5, 7.5, 7.5, 7.5, 7.5, 7.5, 7.5, 20, 20,
								  20, 20, 10, 10, 10, 10, 255, 170, 255, 255,
								  7.5, 7.5, 7.5, 7.5, 7.5, 7.5, 7.5, 7.5, 7.5, 7.5,
								  15, 15, 15, 15, 10, 10, 10, 10 };


int fap_scale[32] = { 1, 1, 2, 3, 5, 7, 8, 10, 12, 15, 18, 21, 25, 30, 35, 42, 
                      50, 60, 72, 87, 105, 128, 156, 191, 234, 288, 355, 439, 543, 
					  674, 836, 1039 };






/***********************************************************************

   Put bits in stream

***********************************************************************/

static int bitsout,bitcount=0;
static int framebits;

unsigned char coderbyte;

void PutBits (int nobits, int value, FILE *fap_stream)
{
	int counter;
	unsigned int mask;
	int bit;
	
	bitcount+=nobits;
	framebits+=nobits;
	
	mask = (int)pow(2,nobits-1); 
	
	for (counter=0; counter<nobits; counter++) {
		
		bitsout--;
		coderbyte <<= 1;
		
		bit = mask&value;
		bit>>=(nobits-1-counter);
		
		coderbyte|=bit;
		
		mask >>= 1; 
		
		if (bitsout==0)  {
			putc(coderbyte,fap_stream);
			bitsout = 8;
		}
		
	}
}


void ShortPutBits (short symbol, FILE *fap_stream)
{
	int counter;
	unsigned int mask,value;
	int bit;
	
	bitcount+=16;
	framebits+=16;
	
	value = (unsigned int)symbol;
	
	mask = (int)pow(2,15); 
	
	for (counter=0; counter<16; counter++) {
		
		bitsout--;
		coderbyte <<= 1;
		
		bit = mask&value;
		bit>>=(15-counter);
		
		coderbyte|=bit;
		
		mask >>= 1; 
		
		if (bitsout==0)  {
			putc(coderbyte,fap_stream);
			bitsout = 8;
		}
		
	}
}






/***********************************************************************


  DCT Coder 


***********************************************************************/



void MakeDCTMatrix(float **dct)
{
	float dct_coef;
	int   line,colunm;
	
	for(line=0; line<SEGMENT_LENGTH; line++) {
		
		if(line==0) 
			dct_coef = (float)(1.0/sqrt(2));
		else dct_coef = 1;
		
		for(colunm=0; colunm<SEGMENT_LENGTH; colunm++)
			dct[line][colunm] = (float)(sqrt(2.0/SEGMENT_LENGTH))*dct_coef*
							(float)(cos(PI*(float)(2*colunm+1)*line/2.0/SEGMENT_LENGTH));
		
	}

}



void ResetDCT(float **faps)
{
	int line,colunm;


	for(line=0;line<NO_FAPS;line++) {
		for(colunm=0;colunm<SEGMENT_LENGTH;colunm++)
			faps[line][colunm]=0;
	}
}




/***********************************************************************

  
	Common parts of Encoder


***********************************************************************/



void  Next_Start_Code(FILE *fap_stream)
{

  if(bitsout!=8){
    PutBits(1,0,fap_stream);
    while(bitsout!=8)
      PutBits(1,1,fap_stream);
  }
  else 
    PutBits(8,127,fap_stream);
}




void Put_Fba_Paramset_Mask(int fpm,FILE *fap_stream)
{
  if(fpm==OFF)
    PutBits(2,0,fap_stream);
  else if(fpm==FAP1)
    PutBits(2,1,fap_stream);
  else if(fpm==BAP2)
    PutBits(2,2,fap_stream);
  else if(fpm==FBAP3)
    PutBits(2,3,fap_stream);
}



void  Code_Frame_Rate(int frame_rate,int seconds,int frequency_offset,
		      FILE *fap_stream)
{
  
  /* frame_rate */

  if(frame_rate>=0 && frame_rate<256)
    PutBits(8,frame_rate,fap_stream);
  else {
    sprintf(word,"\n Error: Frame_rate range from 0 to 255 \n");
	WindowPrintf();
    exit(-1);
  }

  /* seconds */

  seconds = 0;
  if(seconds>=0 && seconds<16)
    PutBits(4,seconds,fap_stream);
  else {
    sprintf(word,"\n Error: Seconds(framerate) range from 0 to 15 \n");
	WindowPrintf();
    exit(-1);
  }

  /* frequency_offset */

  frequency_offset = OFF;
  PutBits(1,frequency_offset,fap_stream);
}





void  Code_Time_Code(FILE *fap_stream,int time_code_hours,
		     int time_code_minutes,int marker_bit,
		     int time_code_seconds)
{
  
  if(time_code_hours>=0 && time_code_hours<24)
    PutBits(5,time_code_hours,fap_stream);
  else {
    sprintf(word,"\n Error: Hours(timecode) range from 0 to 23 \n");
	WindowPrintf();
    exit(-1);
  }

  
  if(time_code_minutes>=0 && time_code_minutes<60)
  PutBits(6,time_code_minutes,fap_stream);
  else {
    sprintf(word,"\n Error: Minutes(timecode) range from 0 to 59 \n");
	WindowPrintf();
    exit(-1);
  }

  
  PutBits(1,marker_bit,fap_stream);

  
  if(time_code_seconds>=0 && time_code_seconds<60)
  PutBits(6,time_code_seconds,fap_stream);
  else {
    sprintf(word,"\n Error: Seconds(timecode) range from 0 to 59 \n");
	WindowPrintf();
    exit(-1);
  }

}




void Code_Skip_Frames(int no_frames_skip,FILE *fap_stream)
{
  int nfs;
  
  if(no_frames_skip>254) {
    sprintf(word,"\n Maximum allowable frames skip is 254.\n");
	WindowPrintf();
    exit(-1);
  }

  if(no_frames_skip > 14) {
    PutBits(4,15,fap_stream);
    nfs = no_frames_skip - 15;
  }
  else {
    nfs = no_frames_skip;
    PutBits(4,nfs,fap_stream);
    return;
  }

  if(nfs<15) {
      PutBits(4,nfs,fap_stream);
      return;
  }
  
  do {
    PutBits(4,15,fap_stream);
    nfs-=15;
  }
  while (nfs > 14);
  
  nfs+=16;
  PutBits(4,nfs,fap_stream);
}






void Temporal_Header(int is_intra,int is_frame_rate,int frame_rate,int fr_seconds,int fr_offset,
					 int is_time_code,int tc_hours,int tc_minutes,int tc_mbit,
					 int tc_seconds,int skip_frames,int no_frames_skip,FILE *fap_stream)
{
		if(is_intra) {
			PutBits(1,is_frame_rate,fap_stream);
			if (is_frame_rate) 
				Code_Frame_Rate(frame_rate,fr_seconds,fr_offset,fap_stream);
			
			PutBits(1,is_time_code,fap_stream);
			if (is_time_code) 
				Code_Time_Code(fap_stream,tc_hours,tc_minutes,tc_mbit,tc_seconds);
		}
			
		PutBits(1,skip_frames,fap_stream);
		if (skip_frames) 
			Code_Skip_Frames(no_frames_skip,fap_stream);

}



void FBA_Object_Plane_Header(int is_intra,int fba_paramset_mask,int is_frame_rate,
							 int frame_rate,int fr_seconds,int fr_offset,int is_time_code,
							 int tc_hours,int tc_minutes,int tc_mbit,int tc_seconds,
							 int skip_frames,int no_frames_skip,FILE *fap_stream)
{
	if(is_intra) {
		Next_Start_Code(fap_stream);
		PutBits(32,FACE_OBJECT_PLANE_START_CODE,fap_stream);	
	}
	
	PutBits(1,is_intra,fap_stream);
	Put_Fba_Paramset_Mask(fba_paramset_mask,fap_stream);
	
	//temporal header 
	Temporal_Header(is_intra,is_frame_rate,	frame_rate,fr_seconds,fr_offset,
		is_time_code,tc_hours,tc_minutes,tc_mbit,tc_seconds,skip_frames,
		no_frames_skip,fap_stream);
}





void Code_Fap_Group_Mask(int group_number,int *fap_mask,FILE *fap_stream)
{
  int mask;

  for(mask=GroupMaskLimits[group_number];mask<GroupMaskLimits[group_number+1];mask++)
    PutBits(1,fap_mask[mask],fap_stream);
}




/**************************************************************************

  
	Arithmetic Coding Parts


**************************************************************************/


void ResetCoderIntra(int *fap_pred)
{
  int counter;
  
  for(counter=0;counter<NO_FAPS;counter++) 
    fap_pred[counter]=0;	
	  
}



int TestNewFapMask(int frame_counter,int *fap_mask,int *old_fap_mask) 
{
  int counter,test;

  if(frame_counter==0) {

    for(counter=0;counter<NO_FAPS;counter++)
      old_fap_mask[counter]=fap_mask[counter];
    
      return ON;    
  }
  
  else {

    test=OFF;
        
    for(counter=0;counter<NO_FAPS;counter++)
      if(fap_mask[counter])
	if(old_fap_mask[counter]!=fap_mask[counter])
	  test=ON;
            
    for(counter=0;counter<NO_FAPS;counter++)
      old_fap_mask[counter]=fap_mask[counter];
  
  }

  return test;
  
}



void Code_New_Minmax(int is_I_new_max,int is_I_new_min,int is_P_new_max,
					 int is_P_new_min,int *fap_mask,int *i_new_max, 
					 int  *i_new_min,int *p_new_max,int *p_new_min,
					 FILE *fap_stream)
{
	int counter,aux;
	
	if(is_I_new_max)
		for(counter=2;counter<NO_FAPS;counter++) {

			if(CheckGroupBegin(counter)) aux=0;

			if(!(aux&3)) 
				PutBits(1,MARKER_BIT,fap_stream);

			if(fap_mask[counter])
				PutBits(5,i_new_max[counter],fap_stream);
			
			aux++;
		}


	if(is_I_new_min)
		for(counter=2;counter<NO_FAPS;counter++){

			if(CheckGroupBegin(counter)) aux=0;

			if(!(aux&3)) 
				PutBits(1,MARKER_BIT,fap_stream);

			if(fap_mask[counter])
				PutBits(5,i_new_min[counter],fap_stream);
			
			aux++;
		}
			
	if(is_P_new_max)
		for(counter=2;counter<NO_FAPS;counter++) {

			if(CheckGroupBegin(counter)) aux=0;

			if(!(aux&3)) 
				PutBits(1,MARKER_BIT,fap_stream);

			if(fap_mask[counter])
				PutBits(5,p_new_max[counter],fap_stream);
			
			aux++;
		}
			
	if(is_P_new_min)
		for(counter=2;counter<NO_FAPS;counter++) {

			if(CheckGroupBegin(counter)) aux=0;

			if(!(aux&3)) 
				PutBits(1,MARKER_BIT,fap_stream);

			if(fap_mask[counter])
				PutBits(5,p_new_min[counter],fap_stream);
			
			aux++;
		}
}



int quantize(int fap, int quantStep) 
{
	
	if(!quantStep)
		return fap;
	else  if( abs(fap)<=quantStep/2 )
		return 0;
	else 
		return (fap + sign(fap)*quantStep/2 ) / quantStep;
	
}



void Code_IFap(int *fap_mask,int *fap_pred, float *fap,
			   int *fapQuantSteps,
			   int *Irange_max, int *Irange_min, int **Icumul_freq, 
			   int **Ifreq, int *range_max, int *range_min, int **cumul_freq, int **freq,
			   int fapq,FILE *fap_stream )
{
	int counter;
	int fap_quant;
	int group_number;
	
	for(group_number=1;group_number<GROUP_MASK_SIZE;group_number++) {

		for(counter=GroupMaskLimits[group_number];counter<
				GroupMaskLimits[group_number+1];counter++) 
				if(fap_mask[counter]) {
					
					fap_quant = quantize((int)fap[counter],fapQuantSteps[counter]) - 
						Irange_min[counter];
										
					AR_Encode(fap_quant,Icumul_freq[counter],fap_stream);
					
					fap_pred[counter] = (fap_quant + Irange_min[counter]) *
						fapQuantSteps[counter];
									
				}
	}
}





void Code_PFap(int *fap_mask,int *fap_pred, float *fap,
			   int *fapQuantSteps,
			   int *Irange_max, int *Irange_min, int **Icumul_freq, 
			   int **Ifreq,int *range_max, int *range_min, int **cumul_freq,int **freq,
			   int fapq,FILE *fap_stream)
{
	int   counter;
	int   fap_quant;
	int   fap_diff;
	int   group_number; 
		
	for(group_number=1;group_number<GROUP_MASK_SIZE;group_number++) {	
				
			for(counter=GroupMaskLimits[group_number];counter<
				GroupMaskLimits[group_number+1];counter++) 
				if(fap_mask[counter]) {
					
					fap_diff = (int)fap[counter] - fap_pred[counter];
					
					fap_quant = quantize(fap_diff,fapQuantSteps[counter]) - range_min[counter];
										
					AR_Encode(fap_quant, cumul_freq[counter],fap_stream);
										
					update(fap_quant,counter,range_max,range_min,cumul_freq,freq);

					fap_diff = (fap_quant + range_min[counter])*fapQuantSteps[counter];
					
					fap_pred[counter]+=fap_diff;
					
				}	
	}
}

	



/***********************************************************************

  DCT Encoder

***********************************************************************/




void Obtain_DCT(float *faps, float *faps_coeff, float **dct)
{
	int line,colunm;
	
	for (line=0; line<SEGMENT_LENGTH; line++) {
		faps_coeff[line]=0;
		for (colunm=0; colunm<SEGMENT_LENGTH; colunm++)
			faps_coeff[line] += dct[line][colunm]*faps[colunm];
	}
}




void DCTQuantize(float dc, int *dc_Q,float quant_step)
{
	if(dc>0.0)
		*dc_Q = (int)((dc/quant_step)+0.5);
	else
		*dc_Q = (int)((dc/quant_step)-0.5);
}




void DC_I_Encode(int dc_Q,FILE *fap_stream)
{
	if( (dc_Q > DC_MAX) || (dc_Q < DC_MIN) ) {

		ShortPutBits((short)(DC_MIN-1),fap_stream);
		PutBits(32,dc_Q,fap_stream);
	}
	else 
		ShortPutBits((short)dc_Q,fap_stream);

}
					


void HuffmanCoding(int **table,int symbol,FILE *fap_stream)
{
	int counter,bitcounter;
	
	bitcounter = table[symbol][0];
	
	for(counter=1;counter<=bitcounter;counter++)
		PutBits(1,table[symbol][counter],fap_stream);


}



void AC_Encode(float *faps_coeff, float quant_step, int **run_table,
			   int **ac_table,FILE *fap_stream)
{
	int counter,*coeff_Q;
	int run_length;

	coeff_Q = (int *)calloc(SEGMENT_LENGTH,sizeof(int));

	for(counter=1;counter<SEGMENT_LENGTH;counter++)
		DCTQuantize(faps_coeff[counter],&coeff_Q[counter],quant_step);	

	run_length = 1;
	

	for(counter=1;counter<SEGMENT_LENGTH;counter++) {
	
		if(coeff_Q[counter] != 0) {

			HuffmanCoding(run_table,run_length,fap_stream);
			
			if( (coeff_Q[counter]>255) || (coeff_Q[counter]<-256) ){
				HuffmanCoding(ac_table,256,fap_stream);
				DC_I_Encode(coeff_Q[counter],fap_stream);			
			}
			else 
				HuffmanCoding(ac_table,coeff_Q[counter]+256,fap_stream);
			
			run_length = 1;
		}
	
		else run_length++;

	}

	if(run_length != 1)
		HuffmanCoding(run_table,0,fap_stream);

	free(coeff_Q);

}


 
					

					
void DC_P_Encode(int dc_Q,int **dc_table,FILE *fap_stream)
{
	if( (dc_Q>255) || (dc_Q<-255) ){
		HuffmanCoding(dc_table,0,fap_stream);
		DC_I_Encode(dc_Q,fap_stream);
	}
	else 
		HuffmanCoding(dc_table,dc_Q+256,fap_stream);

}
					 



void Code_ISegment( int *fap_mask, float **faps,
				   float *fapQuantSteps,float *dc_pred,float **dct, int **run_table,
				   int **ac_table,FILE *fap_stream)
{
	int counter;
	int group_number;
	float step;

	float **faps_coeff;
	int   dc_Q;

	faps_coeff = FloatMatrixAllocation(NO_FAPS,SEGMENT_LENGTH);
	
	for(group_number=1;group_number<10;group_number++) {
		
		for(counter=GroupMaskLimits[group_number];
		counter<GroupMaskLimits[group_number+1];counter++) 
			
			if(fap_mask[counter]) {
				step = fapQuantSteps[counter];
				
				Obtain_DCT(faps[counter],faps_coeff[counter],dct);
				DCTQuantize(faps_coeff[counter][0],&dc_Q,step/3.0f);
				Dequantize(dc_Q,&dc_pred[counter],step/3.0f);
				DC_I_Encode(dc_Q,fap_stream);
				AC_Encode(faps_coeff[counter],step,run_table,ac_table,fap_stream);
				
			}
		
	}

	
	MatrixDealoc(faps_coeff,NO_FAPS);
}



void Code_PSegment( int *fap_mask, float **faps,
					float *fapQuantSteps,float *dc_pred,float **dct, int **run_table, 
					int **dc_table, int **ac_table,FILE *fap_stream)
{
	int counter;
	int group_number;
	float step;

	float **faps_coeff;
	int   dc_Q;
	float *dc_diff;

	faps_coeff = FloatMatrixAllocation(NO_FAPS,SEGMENT_LENGTH);
	dc_diff    = (float *)calloc(NO_FAPS,sizeof(float));
	

	for(group_number=1;group_number<10;group_number++) {
			
			for(counter=GroupMaskLimits[group_number];
				counter<GroupMaskLimits[group_number+1];counter++) 

				if(fap_mask[counter]) {

					step = fapQuantSteps[counter];
					
					Obtain_DCT(faps[counter],faps_coeff[counter],dct);
			
					Predict(faps_coeff[counter][0],dc_pred[counter],&dc_diff[counter]);
					
					DCTQuantize(dc_diff[counter],&dc_Q,step/3.0f);
					
					DC_P_Encode(dc_Q,dc_table,fap_stream);
					
					Dequantize(dc_Q,&dc_diff[counter],step/3.0f);
					
					Reconstruct(dc_diff[counter],&dc_pred[counter]);
					
					AC_Encode(faps_coeff[counter],step,run_table,ac_table,fap_stream);
										
				}
		
	}

	MatrixDealoc(faps_coeff,NO_FAPS);
	free(dc_diff);
}





/*****************************************************************************



  Face Object()


*****************************************************************************/




void InitializeBitstream(FILE *fp_stream)
{
	/* initialize bitcounter */
	
	bitcount = 0;
	bitsout = 8;

	/* face object class */
	
	PutBits(32,FACE_OBJECT_START_CODE,fp_stream);

}




void EndBitstream(FILE *fp_stream)
{
	Next_Start_Code(fp_stream);
	PutBits(32,FACE_OBJECT_END_CODE,fp_stream);
}





void Face_Object_Plane(int face_object_coding_type,int is_intra,int frame_counter,
					   int is_frame_rate,int frame_rate,int fr_seconds,int fr_offset,
					   int is_time_code,int tc_hours,int tc_minutes,int tc_mbit,int tc_seconds,
					   int skip_frames,int no_frames_skip,
					   int fap_quant,int *IPfap_mask,int *old_fap_mask,int *mask_type,
					  
					   int *quant_steps,int *fap_pred, float *fap,
					   int *Irange_max, int *Irange_min, int **Icumul_freq,int **Ifreq,
					   int *range_max, int *range_min, int **cumul_freq,int **freq,
					   int *i_new_max,int *i_new_min,int *p_new_max,int *p_new_min,
					   int	**Exp_freq,int **Exp_cumul_freq,int **IExp_cumul_freq,
					   int	**Vis_freq,int **Vis_cumul_freq,int **IVis_cumul_freq,
					   
					   float **dctfaps,float *dctquant_steps, 
					   float *dc_pred,float **dct, int **run_table,int **ac_table,int **dc_table,
					   
					   FILE *fap_str)
{
	int   counter,group_number,fap_mask_type;
	int   fba_paramset_mask = FAP1;
	int   is_I_new_max,is_I_new_min,is_P_new_max,is_P_new_min;
	
	framebits = 0;
		
	//fba_object_plane_header 
	FBA_Object_Plane_Header(is_intra,fba_paramset_mask,is_frame_rate,frame_rate,fr_seconds,
							fr_offset,is_time_code,tc_hours,tc_minutes,tc_mbit,tc_seconds,
							skip_frames,no_frames_skip,fap_str);
	
	
	if( fba_paramset_mask == FAP1) {
		
		if(is_intra) {
			
			PutBits(5,fap_quant,fap_str);
			
			for(group_number=0;group_number<GROUP_MASK_SIZE;group_number++) {
				
				PutBits(1,MARKER_BIT,fap_str);
				fap_mask_type = mask_type[group_number];
				PutBits(2,fap_mask_type,fap_str);
				if(fap_mask_type == FAP_MASK_TYPE1 || fap_mask_type == FAP_MASK_TYPE2)
					Code_Fap_Group_Mask(group_number,IPfap_mask,fap_str);
			}
			
			
			PutBits(2,face_object_coding_type,fap_str);
			
			/* arithmetic coder */
			if(face_object_coding_type == ARITH_TYPE) {
				
				for(counter=0;counter<NO_FAPS;counter++) 
					quant_steps[counter] = FapQuantSteps[counter]*fap_quant;
				
				ResetCoderIntra(fap_pred);
				
				for(counter=2;counter<NO_FAPS;counter++) {
					Irange_max[counter] = i_new_max[counter]*RANGE_DELTA/fap_quant;
					Irange_min[counter] = -i_new_min[counter]*RANGE_DELTA/fap_quant;
					range_max[counter]  = p_new_max[counter]*RANGE_DELTA/fap_quant;
					range_min[counter]  = -p_new_min[counter]*RANGE_DELTA/fap_quant;
				}		
				
				if( TestNewFapMask(frame_counter,IPfap_mask,old_fap_mask) ) {
					is_I_new_max=1;
					is_I_new_min=1;
					is_P_new_max=1;
					is_P_new_min=1;
				}
				
				PutBits(1,is_I_new_max,fap_str);
				PutBits(1,is_I_new_min,fap_str);
				PutBits(1,is_P_new_max,fap_str);
				PutBits(1,is_P_new_min,fap_str);
				
				init_arith(Irange_max,Irange_min,Icumul_freq,range_max,range_min,cumul_freq,
					freq,Exp_freq,Exp_cumul_freq,IExp_cumul_freq,
					Vis_freq,Vis_cumul_freq,IVis_cumul_freq,IPfap_mask,fap_quant);
				
				Code_New_Minmax(is_I_new_max,is_I_new_min,is_P_new_max,
					is_P_new_min,IPfap_mask,i_new_max,i_new_min,
					p_new_max,p_new_min,fap_str);
				
				Code_IFap(IPfap_mask,fap_pred,fap,quant_steps,Irange_max,Irange_min,
					Icumul_freq,Ifreq,range_max,range_min,cumul_freq,freq,fap_quant,fap_str);
				
				encoder_flush(fap_str);
			}
			else if(face_object_coding_type == DCT_TYPE) {
				
				for(counter=0;counter<NO_FAPS;counter++)
					dctquant_steps[counter] = fap_scale[fap_quant]*DCTQuantSteps[counter];
				
				Code_ISegment(IPfap_mask,dctfaps,dctquant_steps,dc_pred,dct,
					run_table,ac_table,fap_str);
				
			}
			
		}
		else {
			//P frame/segment
			if(face_object_coding_type == ARITH_TYPE) {
				Code_PFap(IPfap_mask,fap_pred,fap,quant_steps,
					Irange_max,Irange_min,Icumul_freq,Ifreq,range_max,
					range_min,cumul_freq,freq,
					fap_quant,fap_str);
				encoder_flush(fap_str);
			}
			else if(face_object_coding_type == DCT_TYPE) {
				Code_PSegment(IPfap_mask,dctfaps,dctquant_steps,dc_pred,dct,
					run_table,dc_table,ac_table,fap_str);
			}
		}
		
	}
	
	if(face_object_coding_type == DCT_TYPE) 
		ResetDCT(dctfaps);	
}




void PutInFaps(int *fap,float **faps,int frame)
{
	int counter;

	for(counter=0;counter<NO_FAPS;counter++)
		faps[counter][frame] = fap[counter];
}





void MakeMasks(int *IPfap_mask,int *mask_type, int frame_counter,int length)
{
	int k,j,uplimit;

	for(k=0;k<NO_FAPS;k++)
		IPfap_mask[k] = OFF;
	
	for(k=0;k<GROUP_MASK_SIZE;k++)
		mask_type[k] = FAP_MASK_TYPE0;

	uplimit = frame_counter + IPrange*length;
	if(uplimit>fapeditor.framenumber) uplimit = fapeditor.framenumber-1;

	
	for(j=frame_counter;j<uplimit;j++) {
		for(k=2;k<NO_FAPS;k++)
			if(fapeditor.mask[j][k]) IPfap_mask[k] = ON;
	}
				
	for(j=1;j<11;j++) {
		for(k=GroupMaskLimits[j];k<GroupMaskLimits[j+1];k++)
			if(IPfap_mask[k]) mask_type[j] = FAP_MASK_TYPE2;
	}

	if(mask_type[1]) mask_type[7] = FAP_MASK_TYPE2;
	if(mask_type[7]) mask_type[1] = FAP_MASK_TYPE2;
	
}



void GetARranges(int *Irange_min,int *Irange_max,int *range_min,int *range_max,
				 int *i_new_min,int *i_new_max,int *p_new_min,int *p_new_max)
{
	int   counter,frame;
	float range_pos,range_neg;

	for(counter=2; counter<NO_FAPS; counter++) {
		Irange_min[counter] = IrangeMin;
		i_new_min[counter]  = IrangeMin/RANGE_DELTA;
		Irange_max[counter] = IrangeMax;
		i_new_max[counter]  = IrangeMax/RANGE_DELTA;
		range_min[counter]  = rangeMin;
		p_new_min[counter]  = rangeMin/RANGE_DELTA;
		range_max[counter]  = rangeMax;
		p_new_max[counter]  = rangeMax/RANGE_DELTA;
	}

	for(counter=2; counter<NO_FAPS; counter++) {

		range_pos = 0;
		range_neg = 0;

		for(frame=0;frame<fapeditor.framenumber;frame++) {
			if(fapeditor.faps[frame][counter]>range_pos) 
				range_pos = fapeditor.faps[frame][counter];
			if(fapeditor.faps[frame][counter]<range_neg) 
				range_neg = fapeditor.faps[frame][counter];
		}

		range_pos/=FapQuantSteps[counter];
		range_neg/=FapQuantSteps[counter];

		Irange_max[counter] =  (int)((range_pos/RANGE_DELTA)+1)*RANGE_DELTA;
		i_new_max[counter]  =  (int)((range_pos/RANGE_DELTA)+1);
		Irange_min[counter] =  (int)((range_neg/RANGE_DELTA)-1)*RANGE_DELTA;
		i_new_min[counter]  = -(int)((range_neg/RANGE_DELTA)-1);

		range_pos = 0;
		range_neg = 0;

		for(frame=1;frame<fapeditor.framenumber;frame++) {
			if(fapeditor.faps[frame][counter]-fapeditor.faps[frame-1][counter]
				>range_pos) 
				range_pos = fapeditor.faps[frame][counter]-
				fapeditor.faps[frame-1][counter];
			if(fapeditor.faps[frame][counter]-fapeditor.faps[frame-1][counter]
				<range_neg) 
				range_neg = fapeditor.faps[frame][counter]-
				fapeditor.faps[frame-1][counter];
		}
		
		range_pos/=FapQuantSteps[counter];
		range_neg/=FapQuantSteps[counter];

		range_max[counter] =  (int)((range_pos/RANGE_DELTA)+1)*RANGE_DELTA;
		p_new_max[counter] =  (int)((range_pos/RANGE_DELTA)+1);
		range_min[counter] =  (int)((range_neg/RANGE_DELTA)-1)*RANGE_DELTA;
		p_new_min[counter] = -(int)((range_neg/RANGE_DELTA)-1);

	}

}



void CodeFapFile(FILE *fp_str)
{
	int   frame_counter,counter,ip_counter,i;

	int   *IPfap_mask,old_fap_mask[NO_FAPS],*mask_type;
    
	int   frame_rate,fr_seconds,fr_offset;
	int   tc_hours,tc_minutes,tc_mbit,tc_seconds;
	int   is_intra,is_time_code,is_frame_rate;
	int   skip_frames,no_frames_skip;

	int   *quant_steps;
	int   *fap_pred;
	float *code_fap;
	int	  *i_new_max,*i_new_min,*p_new_max,*p_new_min;
	int   *Irange_max, *Irange_min,*range_max, *range_min;
	int   **freq,**cumul_freq,**Ifreq,**Icumul_freq;
	int	  **Exp_freq,**Exp_cumul_freq,**IExp_cumul_freq;		
	int	  **Vis_freq,**Vis_cumul_freq,**IVis_cumul_freq;		

	float **dct;
	float **dctfaps;
	float *dc_pred,*dctquant_steps;


	// common
	IPfap_mask  = (int *)calloc(NO_FAPS,sizeof(int));
	mask_type = (int *)calloc(10,sizeof(int));

	if(coding_type == ARITH_TYPE) {
		
		code_fap    = (float *)calloc(NO_FAPS,sizeof(float));
		fap_pred    = (int *)calloc(NO_FAPS,sizeof(int));
		quant_steps = (int *)calloc(NO_FAPS,sizeof(int));
		Icumul_freq = (int **)calloc(NO_FAPS,sizeof(int *));
		Ifreq		= (int **)calloc(NO_FAPS,sizeof(int *));
		freq		= (int **)calloc(NO_FAPS,sizeof(int *));
		cumul_freq	= (int **)calloc(NO_FAPS,sizeof(int *));
		
		IExp_cumul_freq = (int **)calloc(4,sizeof(int *));
		Exp_freq		= (int **)calloc(4,sizeof(int *));
		Exp_cumul_freq	= (int **)calloc(4,sizeof(int *));


		IVis_cumul_freq = (int **)calloc(3,sizeof(int *));
		Vis_freq		= (int **)calloc(3,sizeof(int *));
		Vis_cumul_freq	= (int **)calloc(3,sizeof(int *));

		/* initialize arithmetic coding range */
				
		i_new_max = (int *)calloc(NO_FAPS,sizeof(int));
		i_new_min = (int *)calloc(NO_FAPS,sizeof(int));
		p_new_max = (int *)calloc(NO_FAPS,sizeof(int));
		p_new_min = (int *)calloc(NO_FAPS,sizeof(int));
		
		Irange_max = (int *)calloc(NO_FAPS,sizeof(int));
		Irange_min = (int *)calloc(NO_FAPS,sizeof(int));
		range_max  = (int *)calloc(NO_FAPS,sizeof(int));
		range_min  = (int *)calloc(NO_FAPS,sizeof(int));
		
		GetARranges(Irange_min,Irange_max,range_min,range_max,i_new_min,i_new_max,
					p_new_min,p_new_max);
	}	
	else if(coding_type == DCT_TYPE) {

		/* dct faps*/
		dctquant_steps = (float *)calloc(NO_FAPS,sizeof(float));
		dctfaps  = FloatMatrixAllocation(NO_FAPS,SEGMENT_LENGTH);
		dc_pred    = (float *)calloc(NO_FAPS,sizeof(float));
		
		/* make dct matrix */
		dct  = FloatMatrixAllocation(SEGMENT_LENGTH,SEGMENT_LENGTH);
		MakeDCTMatrix(dct);
	}

	InitializeBitstream(fp_str);

	is_frame_rate	= ON;
	frame_rate		= fapeditor.framerate;
	fr_seconds		= 0;
	fr_offset		= OFF;
	is_time_code	= OFF;
	skip_frames		= OFF;


	if(coding_type == ARITH_TYPE) {

		if(IPrange>fapeditor.framenumber) IPrange = fapeditor.framenumber-1;

		ip_counter = IPrange-1;
		for(frame_counter=0;frame_counter<fapeditor.framenumber;frame_counter++) {

			ip_counter++;
			if(ip_counter==IPrange) {
				is_intra = ON;
				ip_counter = 0;
			}
			else is_intra = OFF;

			if(is_intra) 
				MakeMasks(IPfap_mask,mask_type,frame_counter,1);

			for(i=0;i<NO_FAPS;i++)
				code_fap[i] = fapeditor.faps[frame_counter][i];

			Face_Object_Plane(	coding_type,is_intra,frame_counter,
								is_frame_rate,frame_rate,fr_seconds,fr_offset,
								is_time_code,tc_hours,tc_minutes,tc_mbit,tc_seconds,
								skip_frames,no_frames_skip,
								fap_quant,IPfap_mask,old_fap_mask,mask_type,
								quant_steps,fap_pred,code_fap,
								Irange_max,Irange_min,Icumul_freq,Ifreq,
								range_max,range_min,cumul_freq,freq,
								i_new_max,i_new_min,p_new_max,p_new_min,
								Exp_freq,Exp_cumul_freq,IExp_cumul_freq,
								Vis_freq,Vis_cumul_freq,IVis_cumul_freq,
								NULL,NULL,
								NULL,NULL,NULL,NULL,NULL,
								fp_str);



		}
	}
	else if(coding_type == DCT_TYPE) {

		ip_counter = IPrange-1;
		i = 0;

		for(frame_counter=0;frame_counter<fapeditor.framenumber;frame_counter++) {

			PutInFaps(fapeditor.faps[frame_counter],dctfaps,i);
			i++;

			if(i==SEGMENT_LENGTH || frame_counter==fapeditor.framenumber-1) {

				ip_counter++;
				if(ip_counter==IPrange) {
					is_intra = ON;
					ip_counter = 0;
				}
				else is_intra = OFF;

				if(is_intra) 
					MakeMasks(IPfap_mask,mask_type,frame_counter,SEGMENT_LENGTH);

				Face_Object_Plane(	coding_type,is_intra,frame_counter,
									is_frame_rate,frame_rate,fr_seconds,fr_offset,
									is_time_code,tc_hours,tc_minutes,tc_mbit,tc_seconds,
									skip_frames,no_frames_skip,
									fap_quant,IPfap_mask,NULL,mask_type,
									NULL,NULL,NULL,
									NULL,NULL,NULL,NULL,
									NULL,NULL,NULL,NULL,
									NULL,NULL,NULL,NULL,
									NULL,NULL,NULL,
									NULL,NULL,NULL,
									dctfaps,dctquant_steps, 
									dc_pred,dct,run_table,ac_table,dc_table,
									fp_str);
				i=0;
			}

		}
	}

	EndBitstream(fp_str);

	
	/* memory free */

	free(IPfap_mask);
	free(mask_type);

	if(coding_type == ARITH_TYPE) {
		free(code_fap);
		free(fap_pred);
		free(quant_steps);
		free(i_new_max);
		free(i_new_min);
		free(p_new_max);
		free(p_new_min);
		free(Irange_max);
		free(Irange_min);
		free(range_max);
		free(range_min);
		for(counter=2; counter<NO_FAPS; counter++) {
			free(Ifreq[counter]);
			free(Icumul_freq[counter]);
			free(freq[counter]);
			free(cumul_freq[counter]);
		}
		free(Ifreq);
		free(Icumul_freq);
		free(freq);
		free(cumul_freq);
		for(counter=0; counter<4; counter++) {
			free(IExp_cumul_freq[counter]);
			free(Exp_freq[counter]);
			free(Exp_cumul_freq[counter]);
		}
		free(IExp_cumul_freq);
		free(Exp_freq);
		free(Exp_cumul_freq);

		for(counter=0; counter<3; counter++) {
			free(IVis_cumul_freq[counter]);
			free(Vis_freq[counter]);
			free(Vis_cumul_freq[counter]);
		}
		
		free(IVis_cumul_freq);
		free(Vis_freq);
		free(Vis_cumul_freq);

	}
	else if(coding_type == DCT_TYPE) {

		MatrixDealoc(dct,SEGMENT_LENGTH);
		MatrixDealoc(dctfaps,NO_FAPS);
		free(dctquant_steps);
		free(dc_pred);
	}

}

