/********************************************************************/
/* 																	*/
/* This software module was originally developed and edited by		*/
/* Gabriel Antunes Abrantes (Instituto Superior Tecnico)			*/	
/* 																	*/
/* in the course of development of the MPEG-4 (Visual) standard		*/
/* (ISO/IEC 14496-2).	 											*/
/* This software module is an implementation of a part of one or 	*/
/* more MPEG-4 tools as specified by the	 						*/
/* MPEG-4 standard. ISO/IEC gives users of the MPEG-4 standard		*/
/* free license to this software module or modifications thereof 	*/
/* for use in hardware or software products claiming conformance 	*/
/* to the MPEG-4 standard. Those intending to use this software 	*/
/* module in hardware or software products are advised that its 	*/
/* use may infringe existing patents. The original developer of		*/
/* this software module and his company, the subsequent 			*/
/* editors and their companies, and ISO/IEC have no liability		*/
/* for use of this software module or modifications thereof in 		*/
/* an implementation. Copyright is not released for non				*/
/* MPEG-4 standard conforming products. ACTS-MoMuSys partners	 	*/
/* retain full right to use the code for their own purposes, 		*/
/* assign or donate the code to a third party and to inhibit 		*/
/* third parties from using the code for non MPEG-4 standard		*/
/* conforming products. This copyright notice must be included 		*/
/* in all copies or derivative works. 								*/
/* Copyright (c)1997/98												*/
/* 																	*/
/********************************************************************/




/***********************************HeaderBegin****************************
*   File : ISTdecoder.c                              
*   
*   Author  : Gabriel Antunes Abrantes(gabriel.abrantes@lx.it.pt)
*   Created : 25-July-1997 
*
*   Description : Face Object Decoder
*
*   Modified:
*   Version 4.0 / March-1998
*   Updated for MPEG Tokyo Meeting
*
****************************************HeaderEnd**************************/




#include <stdio.h>
#include <math.h>
#include <stdlib.h>
#include <ctype.h>
#include <string.h>
#include <time.h> 
#include "face.h"
#include "anim.h"
#include "decoder.h"


/* arithmetic coding */

#define   IrangeMin		  -900
#define   IrangeMax		   900
#define   rangeMin		  -900
#define   rangeMax		   900


#define ABORT   2

FILE *fp_stream = NULL;

int zerocount = 0;
int end = OFF;


/***********************************************************************

  Get bits of stream - 
   	
***********************************************************************/

int bitsin;


int GetBits(int no_bits,FILE *fp_stream) 
{
	int	counter;
	int	value = 0;
	int	bit;
	static unsigned char byte;
	
	for(counter=0; counter<no_bits; counter++) {
        
		if(bitsin==0) {
			byte = getc(fp_stream);
			bitsin = 8;
		}

		bitsin -= 1;
		bit = (byte&0200);
		bit >>=7;
		byte <<=1;
		
		value <<= 1;
        value |= bit;
	}
	
	return value;
}



void NextStartCode(FILE *fp_stream) 
{
	/*GetBits(1,fp_stream);*/
	bitsin = 0;
}






/***********************************************************************

 
  Common parts of Bit-stream Decoder
  
   
***********************************************************************/



int Get_Face_Object_Start_Code(FILE *fp_stream)
{
	int dec_sym;
	
	dec_sym = GetBits(32,fp_stream);
	if(dec_sym != FACE_OBJECT_START_CODE) {
		sprintf(word,"\n Wrong FBA-Sequence start code\n (try option rewind)");
		WindowPrintf();
		return OFF;
	}  
	return ON;
}


int Check_Face_Paramset_Mask(int face_paramset_mask)
{
	if(face_paramset_mask != FAP1) {
		sprintf(word,"\n Only FA-frames decoding implemented \n");
		WindowPrintf();
		return OFF;
	}  
	return ON;
}





void  DeCode_Frame_Rate(float *frm_rate,FILE *fp_stream)
{
	int frame_rate,seconds,frequency_offset;
	
	frame_rate = GetBits(8,fp_stream);
	
	seconds = GetBits(4,fp_stream);
	
	frequency_offset = GetBits(1,fp_stream);    
	
	*frm_rate = (float)1000/(1000+frequency_offset)*
		(float)(frame_rate+(float)seconds/16);
}


void  DeCode_Time_Code(FILE *fp_stream)
{
	int time_code_hours,time_code_minutes,marker_bit,time_code_seconds;
	
	time_code_hours   = GetBits(5,fp_stream);
	
	time_code_minutes = GetBits(6,fp_stream);
	
	marker_bit  = GetBits(1,fp_stream);
	
	time_code_seconds = GetBits(6,fp_stream);
		
}




int DeCode_Skip_Frames(int *no_frames_skip,FILE *fp_stream)
{
	int nfs,value;
	
	nfs = GetBits(4,fp_stream);
	
	if(nfs<15) {
		*no_frames_skip = nfs;
		return ON;
	}
	
	value = nfs;
	
	do {
		value = GetBits(4,fp_stream);
		nfs+=value;
	}
	while (value == 15);
	
	if(nfs>254) {
		sprintf(word,"\n Maximum allowable frames skip is 254\n");
		WindowPrintf();
		return OFF;
	}
	
	*no_frames_skip = nfs;
	return ON;
}


void Decode_FAP_Mask( FILE *fp_stream)
{
	int counter,fap_mask_type,mask;
    
	for(counter=0;counter<NO_FAPS;counter++)
		fapx_mask[counter] = OFF;  

	for(counter=0;counter<GROUP_MASK_SIZE;counter++) {
		
		GetBits(1,fp_stream);
		
		fap_mask_type = GetBits(2,fp_stream);
		
		mask_type[counter] = fap_mask_type;
		
		switch(fap_mask_type) {
			
		case FAP_MASK_TYPE0:
			break;
			
		case FAP_MASK_TYPE1:
			for(mask=GroupMaskLimits[counter];mask<GroupMaskLimits[counter+1];mask++) 
				fapx_mask[mask] = GetBits(1,fp_stream);
			break;
			
		case FAP_MASK_TYPE2:
			for(mask=GroupMaskLimits[counter];mask<GroupMaskLimits[counter+1];mask++) 
				fapx_mask[mask] = GetBits(1,fp_stream);
			break;
			
		case FAP_MASK_TYPE3:
			for(mask=GroupMaskLimits[counter];mask<GroupMaskLimits[counter+1];mask++)
				fapx_mask[mask]=ON;
			break;
		}
	}
}



int Check_Face_Object_End(FILE *fp_stream)
{
	int  b0,b1,b2,b3;
	long position = ftell(fp_stream);

	if(!bitsin) getc(fp_stream);

	b0 = getc(fp_stream);
	b1 = getc(fp_stream);
	b2 = getc(fp_stream);
	b3 = getc(fp_stream);
	
	if( b0==0 && b1==0 && b2==1 && b3==3 ) {
		fseek(fp_stream,position,SEEK_SET);
		return OFF;
	}
	else if( b0==0 && b1==0 && (b2==1 || b2==0) ) return ON;
	else fseek(fp_stream,position,SEEK_SET);
		
	return OFF;
}



int Check_Face_Object_Plane_Start_Code(FILE *fp_stream)
{
	int   b0,b1,b2,b3;
	long  position = ftell(fp_stream);

	if(!bitsin) getc(fp_stream);

	b0 = getc(fp_stream);
	b1 = getc(fp_stream);
	b2 = getc(fp_stream);
	b3 = getc(fp_stream);
	
	if( b0==0 && b1==0 && b2==1 && b3==3 ) bitsin = 0;
	else fseek(fp_stream,position,SEEK_SET);

	return ON;
}






/***********************************************************************

 
  Bit-stream Decoder
  
   
***********************************************************************/

								   
								   
int Test_Decoder_Conditions()
{
	if(fp_stream==NULL){
		sprintf(word,"No BitStream-file is open!\n" );
		WindowPrintf();
		return OFF;
	}
	
	if(end){
		sprintf(word,"Sequence at end.\n" );
		WindowPrintf();
		return OFF;
	}
	
	return ON;
}



int DisplayDecodedFrame(int frame_number)
{
	int  counter;

	for(counter=1;counter<=NO_FAPS;counter++) {
		faps[counter]     = fapx[counter-1];
		fap_mask[counter] = fapx_mask[counter-1];
	}

	FAP_Interpolation();

	sprintf(word,"Frame : %d",frame_number);
	ShowText(word);
	
	Show1Frame();
	
	if(StopSequence()) return ABORT;
	
	return ON;
}



int OpenStreamFile(char *stream_file)
{
	CloseOpenFiles();
	ShowText("");
	
	sprintf(word,"%s",stream_file);
	if( (fp_stream  = fopen(word,"rb"))==NULL )
		return OFF;
	
	end = OFF;
	sequence_start = ftell(fp_stream);
	zerocount = 0;
	
	FreeExpressionVisemesTable();
	ResetExpressionVisemesTable();

	return ON;
}





void RewindBitStream()
{
	end = OFF;

	if(fp_stream==NULL){
		sprintf(word,"No BitStream-file is open!\n" );
		WindowPrintf();
		return ;
	}
 
	ShowNeutralFace();

 	fseek(fp_stream,sequence_start,SEEK_SET);
	
	return;
}



void FillHighLevelFapTables(int cod_type,float **group_faps,int **dct_expression)
{
	int counter,i;

	if(cod_type==ARITH_TYPE) {
		for(counter=1;counter<=NO_FAPS;counter++) {
			faps[counter]     = fapx[counter-1];
			fap_mask[counter] = fapx_mask[counter-1];
		}

		FAP_Interpolation();
		Expression();
		Visemes();
	}

	else if(cod_type==DCT_TYPE) {
		counter=0;
		while(counter<SEGMENT_LENGTH) {
			
			for(i=0;i<NO_FAPS;i++)
				fapx[i] = group_faps[i][counter];
			for(i=0;i<6;i++)
				fap_expr[i] = dct_expression[counter][i];

			for(i=1;i<=NO_FAPS;i++) {
				faps[i]     = fapx[i-1];
				fap_mask[i] = fapx_mask[i-1];
			}
		
			FAP_Interpolation();
			Expression();
			Visemes();

			counter++;
		}
	}
}



static int face_object_coding_type;

int Decode_Face_Object_Plane(int *no_frames_skip,int *quant_steps,
							 int *is_rate,
							 int *i_new_max,int *i_new_min,int *p_new_max,
							 int *p_new_min,int *Irange_max,int *Irange_min,
							 int *range_max,int *range_min,int **freq,
							 int **cumul_freq,int **Icumul_freq,
							 int **Exp_freq,int **Exp_cumul_freq,int **IExp_cumul_freq,
							 int **Vis_freq,int **Vis_cumul_freq,int **IVis_cumul_freq,
							 float **dctfaps,int **dctfap_expression,int **dctfap_viseme,
							 float *dctquant_steps,float *dc_pred,float **idct,
							 int **run_table,int **dc_table,int **ac_table)
{	
	int   counter;
	int   face_paramset_mask;
	int   is_time_code,is_frame_rate;
	int   skip_frames;
	int   is_I_new_max,is_I_new_min,is_P_new_max,is_P_new_min;
	int   is_intra;

	static int fap_quant;

	if( Check_Face_Object_Plane_Start_Code(fp_stream) ) {
		
		is_intra = GetBits(1,fp_stream);

		face_paramset_mask = GetBits(2,fp_stream);
		
		if(!Check_Face_Paramset_Mask(face_paramset_mask))
			return DEC_ERROR;

		if(is_intra) {
			is_frame_rate = GetBits(1,fp_stream);
			*is_rate = is_frame_rate;
			if (is_frame_rate) 
				DeCode_Frame_Rate(&fps,fp_stream);
			
			is_time_code = GetBits(1,fp_stream);
			if (is_time_code) 
				DeCode_Time_Code(fp_stream);
		}

		skip_frames = GetBits(1,fp_stream);
		if (skip_frames) 
			if(!DeCode_Skip_Frames(no_frames_skip,fp_stream))
				return DEC_ERROR;
	
	
		if( face_paramset_mask == FAP1) {
		
			if(is_intra) {

				fap_quant = GetBits(5,fp_stream);

				Decode_FAP_Mask(fp_stream);

				face_object_coding_type = GetBits(2,fp_stream);
				
				if(face_object_coding_type == ARITH_TYPE) {
	
					ResetIntra(quant_steps,fap_quant);
					
					is_I_new_max = GetBits(1,fp_stream);
					is_I_new_min = GetBits(1,fp_stream);
					is_P_new_max = GetBits(1,fp_stream);
					is_P_new_min = GetBits(1,fp_stream);
					
					if(!DeCode_New_Minmax(is_I_new_max,is_I_new_min,is_P_new_max,is_P_new_min,
						i_new_max,i_new_min, p_new_max,p_new_min,
						Irange_max,Irange_min,range_max,range_min,fp_stream))
						return DEC_ERROR;
					
					
					if(is_I_new_max || is_I_new_min || is_P_new_max || is_P_new_min) {
						for(counter=0; counter<NO_FAPS; counter++) {
							if (fapx_mask[counter]) {
								if(is_I_new_max) Irange_max[counter] = 
									RANGE_DELTA*i_new_max[counter]/fap_quant;
								if(is_I_new_min) Irange_min[counter] = 
									-RANGE_DELTA*i_new_min[counter]/fap_quant;
								if(is_P_new_max) range_max[counter]  = 
									RANGE_DELTA*p_new_max[counter]/fap_quant;
								if(is_P_new_min) range_min[counter]  = 
									-RANGE_DELTA*p_new_min[counter]/fap_quant;
							}
						}
					}

				init_arith(Irange_max,Irange_min,Icumul_freq,range_max,range_min,cumul_freq,
							freq,Exp_freq,Exp_cumul_freq,IExp_cumul_freq,
							Vis_freq,Vis_cumul_freq,IVis_cumul_freq,fapx_mask,fap_quant);
				
				if(!DeCode_IFap(quant_steps,Irange_max,Irange_min,Icumul_freq,range_max,
								range_min,cumul_freq,freq,Exp_freq,Exp_cumul_freq,
								IExp_cumul_freq,Vis_freq,Vis_cumul_freq,
								IVis_cumul_freq,fap_quant,fp_stream))
					return DEC_ERROR;
				
				}
				else if(face_object_coding_type == DCT_TYPE) {

					ResetDCTIntra(dctfaps,dctfap_expression,dctquant_steps,fap_quant);

					DeCode_ISegment(fapx_mask,dctfaps,dctfap_expression,dctfap_viseme,
									dctquant_steps,dc_pred,idct,run_table,ac_table,fp_stream);
					
				}
				else {
					sprintf(word,"\n Error in FAPbitstream: Unknown coding type.\n");
					WindowPrintf();
					return DEC_ERROR;
				}  
	
			} /* Intra-frame decoding */

			else {
				if(face_object_coding_type == ARITH_TYPE) {

					if(!DeCode_PFap(quant_steps,Irange_max,Irange_min,Icumul_freq,range_max,
						range_min,cumul_freq,freq,Exp_freq,Exp_cumul_freq,IExp_cumul_freq,
						Vis_freq,Vis_cumul_freq,IVis_cumul_freq,fap_quant,fp_stream))
						return DEC_ERROR;
				}
				else if(face_object_coding_type == DCT_TYPE) {

					DeCode_PSegment(fapx_mask,dctfaps,dctfap_expression,dctfap_viseme,
									dctquant_steps,dc_pred,idct,run_table,dc_table,ac_table,
									fp_stream);
				}
			} /* Inter-frame decoding */

		} // fba paramset mask

	}

	FillHighLevelFapTables(face_object_coding_type,dctfaps,dctfap_expression);

    return Check_Face_Object_End(fp_stream);
}





int Decode_Face_Object(int mode)
{
	int  counter,i,frame_number;
	int  *quant_steps;
	
	int  no_frames_skip = 0;
	int  end_code;
	
	int  is_frame_rate;
	
	int  *i_new_max,*i_new_min,*p_new_max,*p_new_min;
	int  *Irange_max, *Irange_min,*range_max, *range_min;
	int  **freq,**cumul_freq,**Icumul_freq;
	int	 **Exp_freq,**Exp_cumul_freq,**IExp_cumul_freq;		
	int	 **Vis_freq,**Vis_cumul_freq,**IVis_cumul_freq;		

	float *dctquant_steps;

	float **idct;
	float **dctfaps;
	float *dc_pred;

	int	  **dctfap_expression,**dctfap_viseme;
	int   first = ON;

	static float frame_duration;
	static time_t current_time;	 
	int    skipped_frames = 0;

	if(!Test_Decoder_Conditions())	
		return OFF;

	/* initialize bitcounter */
	bitsin = 0;

	if(!Get_Face_Object_Start_Code(fp_stream))
		return ON;
	
	/* fap masks */
	quant_steps = (int *)calloc(NO_FAPS,sizeof(int));
		
	/* initialize arithmetic coding */
	Icumul_freq = (int **)calloc(NO_FAPS,sizeof(int *));
	freq		= (int **)calloc(NO_FAPS,sizeof(int *));
	cumul_freq	= (int **)calloc(NO_FAPS,sizeof(int *));
	IExp_cumul_freq = (int **)calloc(4,sizeof(int *));
	Exp_freq		= (int **)calloc(4,sizeof(int *));
	Exp_cumul_freq	= (int **)calloc(4,sizeof(int *));
	IVis_cumul_freq = (int **)calloc(3,sizeof(int *));
	Vis_freq		= (int **)calloc(3,sizeof(int *));
	Vis_cumul_freq	= (int **)calloc(3,sizeof(int *));

	i_new_max = (int *)calloc(NO_FAPS,sizeof(int));
	i_new_min = (int *)calloc(NO_FAPS,sizeof(int));
	p_new_max = (int *)calloc(NO_FAPS,sizeof(int));
	p_new_min = (int *)calloc(NO_FAPS,sizeof(int));
	
	Irange_max = (int *)calloc(NO_FAPS,sizeof(int));
	Irange_min = (int *)calloc(NO_FAPS,sizeof(int));
	range_max  = (int *)calloc(NO_FAPS,sizeof(int));
	range_min  = (int *)calloc(NO_FAPS,sizeof(int));
	
	
	for(counter=0; counter<NO_FAPS; counter++) {
		Irange_min[counter] = IrangeMin;
		Irange_max[counter] = IrangeMax;
		range_min[counter]  = rangeMin;
		range_max[counter]  = rangeMax;
	}
	
	
	/* DCT FAPs */
	dctquant_steps = (float *)calloc(NO_FAPS,sizeof(float));
	dctfaps  = FloatMatrixAllocation(NO_FAPS,SEGMENT_LENGTH);
	dctfap_expression = IntMatrixAllocation(SEGMENT_LENGTH,6);
	dctfap_viseme     = IntMatrixAllocation(SEGMENT_LENGTH,4);

	/* DCT faps*/
	dc_pred    = (float *)calloc(NO_FAPS,sizeof(float));

	/* make DCT matrix */
	idct  = FloatMatrixAllocation(SEGMENT_LENGTH,SEGMENT_LENGTH);
	MakeIDCTMatrix(idct);

	
	/* decode face object class */
	frame_number = 0;
	end_code = OFF;	
	fps = 1.0f;
	/* decode face object plane class */


	switch(mode) {

	case ALL_FRAMES:

		do {                
			end_code = Decode_Face_Object_Plane(&no_frames_skip,
				quant_steps,&is_frame_rate,i_new_max,i_new_min, p_new_max,p_new_min,
				Irange_max,Irange_min,range_max,range_min,freq,cumul_freq,
				Icumul_freq,Exp_freq,Exp_cumul_freq,IExp_cumul_freq,
				Vis_freq,Vis_cumul_freq,IVis_cumul_freq,
				dctfaps,dctfap_expression,dctfap_viseme,dctquant_steps,dc_pred,idct,run_table,
				dc_table,ac_table);
			
			if(end_code == DEC_ERROR) break;
			
			
			if(face_object_coding_type == ARITH_TYPE) {
				if(DisplayDecodedFrame(frame_number) == ABORT) break;
				frame_number++;
			}
			else if(face_object_coding_type == DCT_TYPE) {
				if(!ShowAllDCTFrames(dctfaps,dctfap_expression,dctfap_viseme,
					fapx_mask,frame_number)) break;
				no_frames_skip=0;
				frame_number+=16;
			}
			
		} /* FA frame decoding */
		while(!end_code); /* FBA Sequence decoding */
		
		break;

	case REAL_SEQUENCE:

		current_time = clock();
		
		do {                
			end_code = Decode_Face_Object_Plane(&no_frames_skip,
				quant_steps,&is_frame_rate,i_new_max,i_new_min, p_new_max,p_new_min,
				Irange_max,Irange_min,range_max,range_min,freq,cumul_freq,
				Icumul_freq,Exp_freq,Exp_cumul_freq,IExp_cumul_freq,
				Vis_freq,Vis_cumul_freq,IVis_cumul_freq,
				dctfaps,dctfap_expression,dctfap_viseme,dctquant_steps,dc_pred,idct,run_table,
				dc_table,ac_table);
			
			if(end_code == DEC_ERROR) break;
			
			
			if(face_object_coding_type == ARITH_TYPE) {
	
				if(is_frame_rate)
					frame_duration = (float)(CLOCKS_PER_SEC/fps);

				if(first) {
					if(DisplayDecodedFrame(frame_number)!=ON) end_code = ON;
					frame_number++;
					current_time+=frame_duration;
					first = OFF;
				}
				else if(no_frames_skip>0) {
					frame_number+=no_frames_skip;
					current_time+=(no_frames_skip*frame_duration);
					while(current_time>clock());
					no_frames_skip=0;
					break;
				}
				else {
									
					if(current_time>clock()) {
						if(DisplayDecodedFrame(frame_number)!=ON) end_code = ON;
						while(current_time>clock());
					}
					else skipped_frames++;

					frame_number++;
					current_time+=frame_duration;
				}


			}
			else if(face_object_coding_type == DCT_TYPE) {
				
				counter=0;
				if(is_frame_rate)
					frame_duration = (float)(CLOCKS_PER_SEC/fps);

				if(first) {
					
					for(i=0;i<NO_FAPS;i++) 
						fapx[i] = dctfaps[i][counter];
					for(i=0;i<6;i++)
						fap_expr[i] = dctfap_expression[counter][i];
					for(i=0;i<4;i++)
						fap_vis[i] = dctfap_viseme[counter][i];
					
					if(DisplayDecodedFrame(frame_number)!=ON) end_code = ON;
					counter++;
					frame_number++;
					current_time+=frame_duration;
					first = OFF;
				}
				
				while(counter<SEGMENT_LENGTH) {
					
					if(current_time>clock()) {

						for(i=0;i<NO_FAPS;i++)
							fapx[i] = dctfaps[i][counter];
						for(i=0;i<6;i++)
							fap_expr[i] = dctfap_expression[counter][i];
						for(i=0;i<4;i++)
							fap_vis[i] = dctfap_viseme[counter][i];
						
						if(DisplayDecodedFrame(frame_number)!=ON) end_code = ON;
						while(current_time>clock());
					}
					else skipped_frames++;
					
					frame_number++;
					counter++;
					current_time+=frame_duration;
				}
				
				if(no_frames_skip>0) {
					current_time+=(no_frames_skip*frame_duration);
					while(current_time>clock());
				}
				
			}
		} /* FA frame decoding */
		while(!end_code); /* FBA Sequence decoding */

		sprintf(word,"%d frames showed at %.2f fps, %d were skipped, original frame_rate: %.2f fps",
			frame_number,(frame_number-skipped_frames)*fps/frame_number,skipped_frames,fps);
		ShowText(word);

		break;
	}


	// arith

	free(quant_steps);	
	free(i_new_max);
	free(i_new_min);
	free(p_new_max);
	free(p_new_min);
	free(Irange_max);
	free(Irange_min);
	free(range_max);
	free(range_min);
	
	for(counter=0; counter<NO_FAPS; counter++) {
		free(Icumul_freq[counter]);
		free(freq[counter]);
		free(cumul_freq[counter]);
	}
	
	free(Icumul_freq);
	free(freq);
	free(cumul_freq);
	
	for(counter=0; counter<4; counter++) {
		free(IExp_cumul_freq[counter]);
		free(Exp_freq[counter]);
		free(Exp_cumul_freq[counter]);
	}

	for(counter=0; counter<3; counter++) {
		free(IVis_cumul_freq[counter]);
		free(Vis_freq[counter]);
		free(Vis_cumul_freq[counter]);
	}
	
	free(IExp_cumul_freq);
	free(Exp_freq);
	free(Exp_cumul_freq);
		

	//dct
	MatrixDealoc(idct,SEGMENT_LENGTH);
	MatrixDealoc(dctfaps,NO_FAPS);
	IntMatrixDealoc(dctfap_expression,SEGMENT_LENGTH);
	IntMatrixDealoc(dctfap_viseme,SEGMENT_LENGTH);
	
	free(dctquant_steps);
	free(dc_pred);

	return ON;
}

