/********************************************************************/
/* 																	*/
/* This software module was originally developed and edited by		*/
/* Gabriel Antunes Abrantes (Instituto Superior Tecnico)			*/	
/* 																	*/
/* in the course of development of the MPEG-4 (Visual) standard		*/
/* (ISO/IEC 14496-2).	 											*/
/* This software module is an implementation of a part of one or 	*/
/* more MPEG-4 tools as specified by the	 						*/
/* MPEG-4 standard. ISO/IEC gives users of the MPEG-4 standard		*/
/* free license to this software module or modifications thereof 	*/
/* for use in hardware or software products claiming conformance 	*/
/* to the MPEG-4 standard. Those intending to use this software 	*/
/* module in hardware or software products are advised that its 	*/
/* use may infringe existing patents. The original developer of		*/
/* this software module and his company, the subsequent 			*/
/* editors and their companies, and ISO/IEC have no liability		*/
/* for use of this software module or modifications thereof in 		*/
/* an implementation. Copyright is not released for non				*/
/* MPEG-4 standard conforming products. ACTS-MoMuSys partners	 	*/
/* retain full right to use the code for their own purposes, 		*/
/* assign or donate the code to a third party and to inhibit 		*/
/* third parties from using the code for non MPEG-4 standard		*/
/* conforming products. This copyright notice must be included 		*/
/* in all copies or derivative works. 								*/
/* Copyright (c)1997/98												*/
/* 																	*/
/********************************************************************/




/***********************************HeaderBegin****************************
*   File : ISTface.c                             
*   
*   Author :  Gabriel Antunes Abrantes(gabriel.abrantes@lx.it.pt)
*   Created : September-1997 
*
*   Description : main module of facial animation system
*
*   Version 1.0
*
****************************************HeaderEnd**************************/




#include <stdio.h>	 
#include <windows.h>
#include <windowsx.h>
#include <commctrl.h>
#include <time.h>
#include "face_res.h"
#include "face.h"
#include "anim.h"
#include "decoder.h"
#include "editor.h"

#define ANIM_ALL	10
#define ANIM_REAL	11
#define DEC_ALL		12
#define DEC_REAL	13


HINSTANCE hInst;
static HWND MainWnd, StatWnd;
static CHAR szAppName[] = "Face";


char	word[BUFFER_SIZE];
int		stop = ON;
int		model;
int		texture = ON;
DWORD	file_type;


BOOL Register(); 
HWND WINAPI CreateWnd(int);
void Handle_Create_MSG(HWND);
void Handle_Setup_MSG(HWND);
void Handle_Size_MSG (HWND,WPARAM,LPARAM);
void Handle_Command_MSG (HWND, WPARAM, LPARAM);
void Handle_Paint_MSG(HWND);
void Handle_Close_MSG(HWND);
void Handle_Destroy_MSG(HWND);
void Handle_IDOPEN(HWND);
void Handle_IDEDITFAP(HWND);
void Handle_IDPlay(HWND,int);

LRESULT CALLBACK FaceWndProc (HWND, UINT, WPARAM, LPARAM);
BOOL CALLBACK AboutDlgProc(HWND,UINT,WPARAM,LPARAM);
BOOL CALLBACK IntroDlgProc(HWND,UINT,WPARAM,LPARAM);

void CheckModelMenuItems(HWND);
int	 SaveEditFAPs(HWND);

int  CheckExtension(char *);
void GetAndCheckArgs(HWND); 

int  fap_file = OFF;
int  stream_file = OFF;
char fap_name[BUFFER_SIZE];
char stream_name[BUFFER_SIZE];


int WINAPI WinMain (HINSTANCE hInstance, HINSTANCE hPrevInstance, LPSTR lpCmdLine, int nCmdShow)
{
	MSG Msg;

	hInst = hInstance; 

        
	if (!hPrevInstance)
		if(!Register())
			return 0;

	MainWnd = CreateWnd(nCmdShow);
	if (!MainWnd)
		return FALSE;

	
	while (GetMessage (&Msg, NULL, 0, 0)) {
		TranslateMessage (&Msg);
		DispatchMessage (&Msg);
	}

	
	return Msg.wParam;
}



BOOL Register()
{
	WNDCLASS WndClass;
	
	WndClass.style         = CS_HREDRAW | CS_VREDRAW;
	WndClass.lpfnWndProc   = (WNDPROC)FaceWndProc;
	WndClass.cbClsExtra    = 0;
	WndClass.cbWndExtra    = 0;
	WndClass.hInstance	   = hInst;
	WndClass.hIcon		   = LoadIcon (hInst,"ISTFACE");
	WndClass.hCursor       = LoadCursor (NULL,IDC_ARROW);
	WndClass.hbrBackground = (HBRUSH)(COLOR_WINDOW+1);
	WndClass.lpszMenuName  = szAppName;
	WndClass.lpszClassName = szAppName;
	
	return (RegisterClass (&WndClass) != 0);
}






HWND WINAPI CreateWnd(int nCmdShow)
{
	HWND hWindow;

 	hWindow = CreateWindow (szAppName,"FACE",
							WS_OVERLAPPEDWINDOW | WS_CLIPSIBLINGS | WS_CLIPCHILDREN,
							200,100,700,550,NULL,NULL,hInst,NULL);

	if (!hWindow)
		return FALSE;

	SendMessage(hWindow, WM_SETUP, 0, 0L);

	ShowWindow(hWindow, nCmdShow);
	UpdateWindow(hWindow);

	DialogBox(hInst,"Intro",hWindow,IntroDlgProc);

	GetAndCheckArgs(hWindow);

	return hWindow;
}





LRESULT CALLBACK FaceWndProc (HWND hWnd, UINT Message, WPARAM wParam, LPARAM lParam)
{
	switch (Message) {
		
		case WM_CREATE:

			Handle_Create_MSG(hWnd);
			break;

		case WM_SETUP:

			Handle_Setup_MSG(hWnd);
			break;
				
		case WM_SIZE:
			
			Handle_Size_MSG(hWnd, wParam, lParam);
			break;
		
		case WM_COMMAND:

			Handle_Command_MSG (hWnd, wParam, lParam);
			break;
		
		case WM_PAINT:

			Handle_Paint_MSG(hWnd);
			break;

		case WM_CLOSE:
			
			Handle_Close_MSG(hWnd);
			break;
		
		case WM_DESTROY:
			
			Handle_Destroy_MSG(hWnd);
			break;
		
		default:
			
			return DefWindowProc (hWnd, Message, wParam, lParam);
			break;
	}

	return ON;
}




void Handle_Create_MSG(HWND hWnd)
{
	HDC hDC;
	HGLRC hRC;

	hDC = GetDC(hWnd);
	CreatePixelFormat(hDC);
	
	hRC = wglCreateContext(hDC);
	wglMakeCurrent(hDC,hRC);
	
	return;	
}			  




void Handle_Setup_MSG(HWND hWnd)
{
	int error = OFF;

	StatWnd = CreateStatusWindow(WS_CHILD|WS_VISIBLE,"",hWnd,StatBarID);
	OGL_Initialize(hWnd);
	
	if(GetPoints()) 
		GetFapUnits();
	else {
		sprintf(word,"Error loading PointsFile ('ISTfacepoints') \n");
		MessageBox(hWnd,word,szAppName,MB_OK);
		error = ON;
	}
	
	if(!error) {
		if(!GetVertex()) {
			sprintf(word,"Error loading VertxFile ('ISTfacevertx') \n");
			MessageBox(hWnd,word,szAppName,MB_OK);
			error = ON;
		}
		else {
			GetExpressions();
			GetModelSize();
			model   = POLYGON;
			texture = ON;
			if(CheckHuffmanTables())
				ReadHuffmanTables();
			else 
				error = ON;			
		}
	}

	if(error)
		SendMessage(hWnd, WM_CLOSE, 0, 0L);
	else 
		ShowNeutralFace();

	return;
}



void Handle_Size_MSG (HWND hWnd, WPARAM wParam, LPARAM lParam)
{
	SendMessage(StatWnd,WM_SIZE, wParam, lParam);
	ResizeViewPort(hWnd,StatWnd);
	return;
}



void Handle_Command_MSG (HWND hWnd, WPARAM wParam, LPARAM lParam)
{	

	switch (LOWORD(wParam)) {
		
	case ID_OPEN:
		
		Handle_IDOPEN(hWnd);
		break;
		
	case ID_EDITFAP:
		
		Handle_IDEDITFAP(hWnd);			
		break;
		
	case ID_1FRAME:
		
		Show1Frame();				
		break;
		
	case ID_ALLFRAMES:
		
		Handle_IDPlay(hWnd,ANIM_ALL);
		break;
		
	case ID_PLAY:
		
		Handle_IDPlay(hWnd,ANIM_REAL);
		break;
		
	case ID_DECODE_ALL:
		
		Handle_IDPlay(hWnd,DEC_ALL);
		break;
		
	case ID_DECODE_REAL:
		
		Handle_IDPlay(hWnd,DEC_REAL);
		break;
		
	case ID_STOP:
		
		stop = ON;
		break;
		
	case ID_REWIND:
		
		if(file_type==FAP)
			RewindSequence();
		if(file_type==STREAM)
			RewindBitStream();
		break;
		
	case ID_GRID:
		
		model = !model;
		SendPaintMsg();
		CheckModelMenuItems(hWnd);
		break;
		
	case ID_TEXTURE:
		
		texture = !texture;
		SendPaintMsg();
		CheckModelMenuItems(hWnd);
		break;
		
	case ID_ZOOMIN:
		
		scale_factor*=ZOOM_IN;
		SendPaintMsg();
		break;
		
	case ID_ZOOMOUT:
		
		scale_factor*=ZOOM_OUT;
		SendPaintMsg();
		break;
		
	case ID_ZOOMDEF:
		
		scale_factor = defscale;
		SendPaintMsg();
		break;
		
	case ID_ABOUT:
		
		DialogBox(hInst,"ABOUT_BOX",hWnd,AboutDlgProc);
		break;
		
	case ID_EXIT:
		
		PostMessage (hWnd, WM_CLOSE, 0, 0L);
		break;
		
	default:
		break;
	 }
	 
	 return;
}



void Handle_Paint_MSG(HWND hWnd)
{
	PAINTSTRUCT	ps;
	
	BeginPaint(hWnd, &ps);
	Paint_OGL_FaceModel();
	EndPaint(hWnd, &ps);
}



void Handle_Close_MSG(HWND hWnd)
{
	if(!stop) return;
	
	DestroyWindow (hWnd);

	return;
}





void Handle_Destroy_MSG(HWND hWnd)
{
	HDC hDC;
	HGLRC hRC;
	
	CloseOpenFiles();
	FreeExpressionVisemesTable();

	IntMatrixDealoc(run_table,16);
	IntMatrixDealoc(dc_table,512);
	IntMatrixDealoc(ac_table,512);

	
	hRC = wglGetCurrentContext();
	hDC = wglGetCurrentDC();
	
	wglMakeCurrent(NULL, NULL);
	
	if (hRC) wglDeleteContext(hRC);
	if (hDC) ReleaseDC(hWnd, hDC);
	
	PostQuitMessage (0);
}



int CheckExtension(char *extension)
{
	if(!strcmp(extension,"fap"))
		return FAP;
	else if(!strcmp(extension,"stream"))
		return STREAM;
	else if(!strcmp(extension,"eam"))
		return STREAM;
	else return 100;
}





void Handle_IDOPEN(HWND hWnd)
{
	OPENFILENAME ofn;
	HMENU        new_menu;

	char         fname[BUFFER_SIZE]="";

	ofn.lStructSize      	= sizeof(OPENFILENAME);
	ofn.hwndOwner	      	= hWnd;
	ofn.hInstance	      	= hInst;
	ofn.lpstrFilter       	= "All Files (*.*)\0*.*\0FAPbitstream files (*.stream)\0*.stream\0Facial Animation Parameter Files (*.fap)\0*.fap\0";
	ofn.lpstrCustomFilter	= NULL;
	ofn.nMaxCustFilter    	= 0;
	ofn.nFilterIndex      	= file_type;
	ofn.lpstrFile	      	= fname;
	ofn.nMaxFile	      	= sizeof(fname);
	ofn.lpstrFileTitle    	= NULL;
	ofn.nMaxFileTitle     	= 0;
	ofn.lpstrInitialDir   	= NULL;
	ofn.lpstrTitle	      	= "Open";
	ofn.Flags	      		= OFN_EXPLORER | OFN_FILEMUSTEXIST | OFN_HIDEREADONLY;
	ofn.nFileOffset       	= 0;
	ofn.nFileExtension    	= 0;
	ofn.lpstrDefExt       	= "stream";
	ofn.lCustData	      	= (DWORD)NULL;
	ofn.lpfnHook	      	= NULL;
	ofn.lpTemplateName    	= NULL;
	
	
	
	if ( GetOpenFileName(&ofn) ) {
				
		file_type = CheckExtension(ofn.lpstrFile+ofn.nFileExtension);

		switch(file_type) {
			
		case FAP:
			
			if( ReadFapHeader(ofn.lpstrFile) ) {
				fap_file = ON;				
				wsprintf(word,"FACE - %s",ofn.lpstrFile+ofn.nFileOffset);
				SetWindowText(hWnd,word);
				new_menu = LoadMenu(hInst,"Face_Anim");
				SetMenu(hWnd,new_menu);	
				model = POLYGON;
				CheckModelMenuItems(hWnd);
				ShowNeutralFace();
			}	
			else {
				sprintf(word,"\n Error in FAPfile %s\n",ofn.lpstrFile);
				WindowPrintf();
				fap_file = OFF;
			}
			
			break;
			
		case STREAM:
			
			if( OpenStreamFile(ofn.lpstrFile) ) {
				stream_file = ON;
					
				wsprintf(word,"FACE - %s",ofn.lpstrFile+ofn.nFileOffset);
				SetWindowText(hWnd,word);
				new_menu = LoadMenu(hInst,"Face_Dec");
				SetMenu(hWnd,new_menu);
				model = POLYGON;
				CheckModelMenuItems(hWnd);
				ShowNeutralFace();
			}	
			else {
				sprintf(word,"\n Error in STREAMfile %s\n",ofn.lpstrFile);
				WindowPrintf();
				stream_file = OFF;
			}
			
			break;
			

		default:
			sprintf(word,"\n %s - Unknown file type!\n",ofn.lpstrFile+ofn.nFileOffset);
			WindowPrintf();
			break;			
		}
	}
	
}





void Handle_IDEDITFAP(HWND hWnd)
{	
	HMENU new_menu;
    
	new_menu = LoadMenu(hInst,"Face_Model");
	SetMenu(hWnd,new_menu);	
	CheckModelMenuItems(hWnd);
	
	DialogBox(hInst,"FAP_INFO",hWnd,EditFAP_DlgProc);

	if(editfap) {
		ShowNeutralFace();
		DialogBox(hInst,"FAP_BOX",hWnd,FAP_DlgProc);
	}

	if(!editcancel) {
		while(!SaveEditFAPs(hWnd));
	}

}




void Handle_IDPlay(HWND hWnd,int play_type)
{
	HMENU old_menu,new_menu;
    char *menu_name;

	if( (play_type == DEC_REAL) || (play_type == DEC_ALL) )
		menu_name = "Dec_Stop";
	else
		menu_name = "Anim_Stop";

	stop = OFF;
	old_menu = GetMenu(hWnd);		 
	new_menu = LoadMenu(hInst,menu_name);  
	SetMenu(hWnd,new_menu);	
	CheckModelMenuItems(hWnd);
	
	switch(play_type) {
	
	case ANIM_ALL:
		
		PlaySequence();
		break;

	case ANIM_REAL:
		
		PlayRealTime();
		break;
	
	case DEC_ALL:

		Decode_Face_Object(ALL_FRAMES); 
		break;
	
	case DEC_REAL:

		Decode_Face_Object(REAL_SEQUENCE); 
		break;
	}

	stop = ON;
	SetMenu(hWnd,old_menu);
	CheckModelMenuItems(hWnd);
	DestroyMenu(new_menu);
}






int OpenFAPForEdit(HWND hWnd)
{
	OPENFILENAME ofn;
	char         fname[BUFFER_SIZE]="";
	int			 ftype,file_ok;

	ofn.lStructSize      	= sizeof(OPENFILENAME);
	ofn.hwndOwner	      	= hWnd;
	ofn.hInstance	      	= hInst;
	ofn.lpstrFilter       	= "All Files (*.*)\0*.*\0Facial Animation Parameter Files (*.fap)\0";
	ofn.lpstrCustomFilter	= NULL;
	ofn.nMaxCustFilter    	= 0;
	ofn.nFilterIndex      	= file_type;
	ofn.lpstrFile	      	= fname;
	ofn.nMaxFile	      	= sizeof(fname);
	ofn.lpstrFileTitle    	= NULL;
	ofn.nMaxFileTitle     	= 0;
	ofn.lpstrInitialDir   	= NULL;
	ofn.lpstrTitle	      	= "Open";
	ofn.Flags	      		= OFN_EXPLORER | OFN_FILEMUSTEXIST | OFN_HIDEREADONLY;
	ofn.nFileOffset       	= 0;
	ofn.nFileExtension    	= 0;
	ofn.lpstrDefExt       	= "fap";
	ofn.lCustData	      	= (DWORD)NULL;
	ofn.lpfnHook	      	= NULL;
	ofn.lpTemplateName    	= NULL;
	
	file_ok = OFF;
	
	
	if ( GetOpenFileName(&ofn) ) {
				
		ftype = CheckExtension(ofn.lpstrFile+ofn.nFileExtension);

		switch(ftype) {
			
		case FAP:
			
			if( ReadEditFapHeader(ofn.lpstrFile) ) {
				ReadEditFrames();
				file_ok = ON;
			}	
			else {
				sprintf(word,"\n Error in FAPfile %s\n",ofn.lpstrFile);
				WindowPrintf();
				file_ok = OFF;
			}
			
			break;
			
		default:
			sprintf(word,"\n %s - Unknown file type!\n",ofn.lpstrFile+ofn.nFileOffset);
			WindowPrintf();
			file_ok = OFF;
			break;			
		}
	}
	
	return file_ok;
}





int SaveEditFAPs(HWND hWnd)
{
	OPENFILENAME ofn;
	char         fname[BUFFER_SIZE]="";
	FILE         *fp;

	ofn.lStructSize      	= sizeof(OPENFILENAME);
	ofn.hwndOwner	      	= hWnd;
	ofn.hInstance	      	= hInst;
	ofn.lpstrFilter       	= "FAPbitstream (*.stream)\0*.stream\0Facial Animation Parameter Files (*.fap)\0*.fap\0";
	ofn.lpstrCustomFilter	= NULL;
	ofn.nMaxCustFilter    	= 0;
	ofn.nFilterIndex      	= file_type;
	ofn.lpstrFile	      	= fname;
	ofn.nMaxFile	      	= sizeof(fname);
	ofn.lpstrFileTitle    	= NULL;
	ofn.nMaxFileTitle     	= 0;
	ofn.lpstrInitialDir   	= NULL;
	ofn.lpstrTitle	      	= "Save";
	ofn.Flags	      		= OFN_EXPLORER | OFN_HIDEREADONLY;
	ofn.nFileOffset       	= 0;
	ofn.nFileExtension    	= 0;
	ofn.lpstrDefExt       	= "stream";
	ofn.lCustData	      	= (DWORD)NULL;
	ofn.lpfnHook	      	= NULL;
	ofn.lpTemplateName    	= NULL;
	

	
	if ( GetSaveFileName(&ofn) ) {
		
		if( (fp = fopen(ofn.lpstrFile,"r")) != NULL) {
			fclose(fp);
			sprintf(word,"%s\n\nThis file already exists.\nOverwrite existing file?",ofn.lpstrFile);
			if(MessageBox(hWnd,word,szAppName,MB_YESNO)==IDNO)
				return OFF;
		}
		
		if(!SaveFAPorStreamFile(hWnd,ofn.lpstrFile+ofn.nFileOffset,ofn.lpstrFile+ofn.nFileExtension)) {
			sprintf(word,"\n Error for file %s\n",ofn.lpstrFile);
			WindowPrintf();
		}

	}

	ShowNeutralFace();
	return ON;
}







BOOL CALLBACK AboutDlgProc(HWND hDlg , UINT Message, WPARAM wParam, LPARAM lParam)
{
	switch(Message) {

	case WM_INITDIALOG:
		return TRUE;

	case WM_COMMAND:
		if(LOWORD(wParam) == IDOK || LOWORD(wParam) == IDCANCEL) {
			EndDialog(hDlg,LOWORD(wParam));
			return TRUE;
		}
		break;
	}
	return FALSE;
}



BOOL CALLBACK IntroDlgProc(HWND hDlg , UINT Message, WPARAM wParam, LPARAM lParam)
{
	switch(Message) {

	case WM_INITDIALOG:
		return TRUE;
		break;

	case WM_COMMAND:
		if(LOWORD(wParam) == IDOK) {
			EndDialog(hDlg,LOWORD(wParam));
			return TRUE;
		}
		break;	
	}

	return FALSE;
}




void WindowPrintf()
{
	MessageBox(MainWnd,word,szAppName,MB_OK);
}



void ShowText(char *text)
{
	SetWindowText(StatWnd,text);
}



void SendPaintMsg()
{
	PostMessage (MainWnd, WM_PAINT, 0, 0L);
	UpdateWindow(MainWnd);
}



int StopSequence() 
{
	MSG msg;

	while (PeekMessage(&msg, NULL, 0, 0, PM_REMOVE)) {
		TranslateMessage(&msg);
		DispatchMessage(&msg);
	}
	return stop;
}



void CheckModelMenuItems(HWND hWnd)
{
	HMENU menu;
	
	menu = GetMenu(hWnd);
	if(!model)
		CheckMenuItem(menu,ID_GRID,MF_BYCOMMAND | MF_CHECKED);
	else 
		CheckMenuItem(menu,ID_GRID,MF_BYCOMMAND | MF_UNCHECKED);

	if(texture)
		CheckMenuItem(menu,ID_TEXTURE,MF_BYCOMMAND | MF_CHECKED);
	else 
		CheckMenuItem(menu,ID_TEXTURE,MF_BYCOMMAND | MF_UNCHECKED);
}




void GetAndCheckArgs(HWND hWnd) 
{
	char *comline,args[BUFFER_SIZE],extension[3];
	int  i,a,ext;
	HBITMAP hbm = NULL;
	HMENU   new_menu;

	comline = GetCommandLine();
	i = 1;
	while(comline[i]!='"') i++;
	i++;
	while(isspace(comline[i])) i++;

	a=0;
	while(comline[i]!='\0') 
		args[a++] = comline[i++];
	args[a] = '\0';

	a = strlen(args);
	
	extension[0] = args[a-3];
	extension[1] = args[a-2];
	extension[2] = args[a-1];
	extension[3] = '\0';

	ext = CheckExtension(extension);
	if(ext==100) {
		extension[0] = args[a-1];
		extension[1] = '\0';
		ext = CheckExtension(extension);
	}

	switch(ext) {
			
		case STREAM:

			OpenStreamFile(args);

			stream_file = ON;
			file_type=STREAM;
			wsprintf(stream_name,"%s",args);

			wsprintf(word,"FACE - %s",args);
			SetWindowText(hWnd,word);
			new_menu = LoadMenu(hInst,"Face_Dec");
			SetMenu(hWnd,new_menu);

			PostMessage (hWnd, WM_COMMAND, ID_DECODE_REAL, 0L);
			break;

		case FAP:
			
			ReadFapHeader(args);

			fap_file = ON;
			wsprintf(fap_name,"%s",args);
			file_type=FAP;

			wsprintf(word,"FACE - %s",args);
			SetWindowText(hWnd,word);
			new_menu = LoadMenu(hInst,"Face_Anim");
			SetMenu(hWnd,new_menu);	

			PostMessage (hWnd, WM_COMMAND, ID_PLAY, 0L);
			break;

		default:
			break;
	}
}


