/**********************************************************************/
/*                                                                    */
/* This software module was originally developed by                   */
/*                                                                    */
/* A. Tanju Erdem (Eastman Kodak Company)                             */
/* Candemir Toklu (University of Rochester)                           */
/*                                                                    */
/* and edited by                                                      */
/*                                                                    */
/* Peter van Beek (University of Rochester)                           */
/*                                                                    */
/* in the course of development of MPEG-4 Visual (ISO/IEC 14496-2).   */
/* This software module is an implementation of a part of one or      */
/* more MPEG-4 Visual (ISO/IEC 14496-2) tools as specified by the     */
/* MPEG-4 Visual (ISO/IEC 14496-2). ISO/IEC gives users of MPEG-4     */
/* Visual (ISO/IEC 14496-2) free license to this software module or   */
/* modifications thereof for use in hardware or software products     */
/* claiming conformance to MPEG-4 Visual (ISO/IEC 14496-2). Those     */
/* intending to use this software module in hardware or software      */
/* products are advised that its use may infringe existing patents.   */
/* The original developer of this software module and his/her         */
/* company, the subsequent editors and their companies, and ISO/IEC   */
/* have no liability for use of this software module or modifications */ 
/* thereof in an implementation. Copyright is not released for non    */
/* MPEG-4 Visual (ISO/IEC 14496-2) conforming products.               */
/* Eastman Kodak Company and the University of Rochester retain full  */
/* right to use the code for their own purposes, assign or donate the */
/* code to a third party and to inhibit third parties from using the  */
/* code for non MPEG-4 Visual (ISO/IEC 14496-2) conforming products.  */
/* This copyright notice must be included in all copies or derivative */
/* works.                                                             */
/*                                                                    */
/* Copyright (c) 1994-1997                                            */
/*                                                                    */
/**********************************************************************/

/*+*********************************************************************
   NAME         general.h
   AUTHOR       A. Tanju Erdem - Eastman Kodak Company and Candemir Toklu - 
                University of Rochester.
   DATE         September 1994
   UPDATE       Peter Van Beek, Tue Aug  4 11:54:53 1998
   DESCRIPTION  Some general definitions. 

*********************************************************************-*/

#ifndef GENERAL_H_DEF
#define GENERAL_H_DEF

#include <stdio.h>
#include <stdlib.h>
#ifndef WIN32
#include <stream.h>
#include <unistd.h>
#endif
#include <iostream.h>
#include <fstream.h>
#include <math.h>
#include <time.h>
#include "pointClass.h"


//-----------------------------------------------------------------------------
// various defines

#ifndef NULL
#define NULL	0
#endif // NULL

#ifndef TRUE
#define TRUE	1
#define FALSE	0
#endif // TRUE

#ifndef OK
#define OK		1
#define NOT_OK	0
#endif // OK

#ifndef BYTE
#define BYTE	unsigned char
#endif // BYTE

#ifndef O_BINARY
#define O_BINARY	0
#define O_TEXT		0
#endif // O_BINARY

#ifndef uint
#define uint unsigned int
#endif // uint

#ifndef uchar
#define uchar unsigned char
#endif // uchar


//-----------------------------------------------------------------------------
// various macros

#define MAX(a,b) ((a)<(b)?(b):(a))		// Define the maximum function
#define MIN(a,b)  ((a)>(b)?(b):(a))		// Define the minimum function
#define SGN(a) ((a)<0?-1:1)				// Define the sgn function 
#define SQR(a) ((a)*(a))				// Define the square function
#define ABS(a) (a<0?-(a):(a))			// Redefine abs to be type insensitive

// error messages
#define doError(s)			{ fprintf(stderr, "Error:\t%s", s); }
#define doWarning(s)		{ fprintf(stderr, "Warning:\t%s", s); }

// variable argument debugging macro
#if defined(DEBUGGING) || defined(_DEBUG)
// if debugging is on during compile-time then print message
#define DBUG(vargs)			((void) printf vargs)
#else
// else replace debug macro's with blank spaces
#define DBUG(vargs)
#endif // DEBUGGING


//-----------------------------------------------------------------------------
// inline functions

//------------------------------------------------------------------------
inline int Sign( double a)
{
  return (a == 0)? 0: ( ( a > 0)? 1: -1);
}

//------------------------------------------------------------------------
inline int SignE( double a)
{
  return (fabs(a)<EPSILON)? 0: ( ( a > 0)? 1: -1);
}

//------------------------------------------------------------------------
inline int Sign1at0( double a)
{
  return (( a >= 0)? 1: -1);
}

//------------------------------------------------------------------------
inline int Mod( int i, int n)
{
   return (((i%=n)>=0)?i:i+n);
}

//------------------------------------------------------------------------
inline Point2d point( double a, double b)
{
  return Point2d( a, b);
}

//------------------------------------------------------------------------
inline double pt2dMagSq( Point2d& p)
{
  return p*p;
}

//------------------------------------------------------------------------
inline double pt2dMag( Point2d& p)
{
  return sqrt(p*p);
}

//------------------------------------------------------------------------
inline Point2d operator*( const Point2d& p, double a)
{
  return Point2d( p.x * a, p.y * a);
}

//------------------------------------------------------------------------
inline Point2d operator*( double a, const Point2d& p)
{
  return Point2d( p.x * a, p.y * a);
}

//------------------------------------------------------------------------
inline Point2d operator/( const Point2d& p, double a)
{
  return Point2d( p.x / a, p.y / a);
}

//------------------------------------------------------------------------
inline ostream& operator<<( ostream& os, const Point2d& p)
{
  os << "(" << p.x << ", " << p.y << ")";
  return os;
}

//------------------------------------------------------------------------
inline istream& operator>>( istream& is, Point2d& p)
{
  is >> p.x >> p.y;
  return is;
}


#endif // GENERAL_H_DEF
