/**********************************************************************/
/*                                                                    */
/* This software module was originally developed by                   */
/*                                                                    */
/* A. Tanju Erdem (Eastman Kodak Company)                             */
/* Candemir Toklu (University of Rochester)                           */
/* Peter van Beek (University of Rochester)                           */
/*                                                                    */
/* in the course of development of MPEG-4 Visual (ISO/IEC 14496-2).   */
/* This software module is an implementation of a part of one or      */
/* more MPEG-4 Visual (ISO/IEC 14496-2) tools as specified by the     */
/* MPEG-4 Visual (ISO/IEC 14496-2). ISO/IEC gives users of MPEG-4     */
/* Visual (ISO/IEC 14496-2) free license to this software module or   */
/* modifications thereof for use in hardware or software products     */
/* claiming conformance to MPEG-4 Visual (ISO/IEC 14496-2). Those     */
/* intending to use this software module in hardware or software      */
/* products are advised that its use may infringe existing patents.   */
/* The original developer of this software module and his/her         */
/* company, the subsequent editors and their companies, and ISO/IEC   */
/* have no liability for use of this software module or modifications */ 
/* thereof in an implementation. Copyright is not released for non    */
/* MPEG-4 Visual (ISO/IEC 14496-2) conforming products.               */
/* Eastman Kodak Company and the University of Rochester retain full  */
/* right to use the code for their own purposes, assign or donate the */
/* code to a third party and to inhibit third parties from using the  */
/* code for non MPEG-4 Visual (ISO/IEC 14496-2) conforming products.  */
/* This copyright notice must be included in all copies or derivative */
/* works.                                                             */
/*                                                                    */
/* Copyright (c) 1994-1998                                            */
/*                                                                    */
/**********************************************************************/

/*+*********************************************************************
   NAME         linkClass.h
   AUTHOR       A. Tanju Erdem - Eastman Kodak Company and Candemir Toklu - 
                University of Rochester.
   DATE         September 1994
   UPDATE       14 August 1996 by Candemir Toklu
                17 February 1998 by Peter Van Beek
                Added 'motion' and 'mark' fields.
                Added 'setPoint()' method.
                17 August 1998 by Peter Van Beek
                Simplified.
   DESCRIPTION  The Pt2dLink class defines a Point2d-link that is part
                of a point list. Only class Pt2dList functions can
                create and destroy Pt2dLink objects. Otherwise one
                should only declare pointers to Pt2dLink objects. Its
                private members are: links to the next and previous
                points in a linked list and the location of the point
                itself. One can also store a motion vector for this point
                and a marker.
   SYNOPSIS     Include files : 
                #include <iostream.h>
                #include <fstream.h>
                #include "pointClass.h"
*********************************************************************-*/

#include <iostream.h>
#include <fstream.h>
#include "pointClass.h"


#ifndef POINT_LINK_CLASS
#define POINT_LINK_CLASS

//------------------------------------------------------------------------

// A link is a point with pointers to the previous and next neighboring points.
// Only class Pt2dList functions can create and destroy Pt2dLink objects.
// Otherwise one should only declare pointers to Pt2dLink objects.
#ifndef WIN32
class Pt2dLink {
#else
class __declspec(dllexport) Pt2dLink {
#endif
	friend class Pt2dList;
	friend class Tri2dTess;
  public:
	Pt2dLink();
	Pt2dLink( const Point2d&);
//	  ~Pt2dLink();			// Default destructor
	inline Point2d getPoint();
	inline void setPoint(const Point2d&);
	Point2d motion;	   // Motion vector of the point, PvB
	char mark;		   // Marker of this point, PvB
  private:
	Point2d here;  // Coordinates of the point
	Pt2dLink *next, *prev;	// Pointers to the next and previous point-links in a list
};

Point2d Pt2dLink::getPoint()
{
	return here;
}

void Pt2dLink::setPoint(const Point2d& p)
{
	here = p;
}

#endif
