/**********************************************************************/
/*                                                                    */
/* This software module was originally developed by                   */
/*                                                                    */
/* A. Tanju Erdem (Eastman Kodak Company)                             */
/* Candemir Toklu (University of Rochester)                           */
/*                                                                    */
/* and edited by                                                      */
/*                                                                    */
/* Peter van Beek (University of Rochester)                           */
/*                                                                    */
/* in the course of development of MPEG-4 Visual (ISO/IEC 14496-2).   */
/* This software module is an implementation of a part of one or      */
/* more MPEG-4 Visual (ISO/IEC 14496-2) tools as specified by the     */
/* MPEG-4 Visual (ISO/IEC 14496-2). ISO/IEC gives users of MPEG-4     */
/* Visual (ISO/IEC 14496-2) free license to this software module or   */
/* modifications thereof for use in hardware or software products     */
/* claiming conformance to MPEG-4 Visual (ISO/IEC 14496-2). Those     */
/* intending to use this software module in hardware or software      */
/* products are advised that its use may infringe existing patents.   */
/* The original developer of this software module and his/her         */
/* company, the subsequent editors and their companies, and ISO/IEC   */
/* have no liability for use of this software module or modifications */ 
/* thereof in an implementation. Copyright is not released for non    */
/* MPEG-4 Visual (ISO/IEC 14496-2) conforming products.               */
/* Eastman Kodak Company and the University of Rochester retain full  */
/* right to use the code for their own purposes, assign or donate the */
/* code to a third party and to inhibit third parties from using the  */
/* code for non MPEG-4 Visual (ISO/IEC 14496-2) conforming products.  */
/* This copyright notice must be included in all copies or derivative */
/* works.                                                             */
/*                                                                    */
/* Copyright (c) 1994-1998                                            */
/*                                                                    */
/**********************************************************************/

/*+*********************************************************************
   NAME         pointClass.cpp
   AUTHOR       A. Tanju Erdem - Eastman Kodak Company and Candemir Toklu - 
                University of Rochester.
   DATE         September 1994
   UPDATE       14 August 1996 by Candemir Toklu
                - general
                Jun 12 1998 by Peter van Beek
                - inlined constructors and deleted some methods
   DESCRIPTION  Member functions for Point2d class are defined in this
                file.
   SYNOPSIS     Include files: 
                #include "general.h"
                #include "pointClass.h"
*********************************************************************-*/

#include "general.h"
#include "pointClass.h"
#include "polyClass.h"


// Norm of a vector point
Point2d Point2d::unit()
{
	return (*this) / (double) (*this).norm();
}

// Distance-squared to a point
double Point2d::distanceSqrTo( Point2d p)
{
	Point2d temp = *this - p;
	return temp.normSqr();
}

// Distance to a point
double Point2d::distanceTo( Point2d p)
{
	return sqrt( distanceSqrTo(p));
}

// Is the point inside a given - convex - polygon?
int Point2d::isInside( Poly2d pg)
{
	int i;
	int size = pg.getSize();

	if( size == 0) return 0;

	// Obtain the vectors from the point to the vertices of the polygon.
	for( i = 0; i < size; i++) {
		pg[i] -= *this;
	}

	// If the point is inside the polygon, then angles between the 
	// consecutive vectors will either all be >= 180 or <= 180. We 
	// determine whether an angle is less or greater than or equal 
	// to 180 by using the cross-product operation.
	int newSign;
	int initialSign = 0;
	for( i = 0; i < size; i++) {
		if( ( newSign = SignE(pg[i] ^ pg[i+1])) !=0) {
			if( initialSign == 0) initialSign = newSign;
			if( newSign != initialSign) break;
		}	  
	}
	// if i == size then the point is inside the polygon 
	return ( i == size);
}


// end of file
