/**********************************************************************/
/*                                                                    */
/* This software module was originally developed by                   */
/*                                                                    */
/* Peter van Beek (University of Rochester)                           */
/*                                                                    */
/* in the course of development of MPEG-4 Visual (ISO/IEC 14496-2).   */
/* This software module is an implementation of a part of one or      */
/* more MPEG-4 Visual (ISO/IEC 14496-2) tools as specified by the     */
/* MPEG-4 Visual (ISO/IEC 14496-2). ISO/IEC gives users of MPEG-4     */
/* Visual (ISO/IEC 14496-2) free license to this software module or   */
/* modifications thereof for use in hardware or software products     */
/* claiming conformance to MPEG-4 Visual (ISO/IEC 14496-2). Those     */
/* intending to use this software module in hardware or software      */
/* products are advised that its use may infringe existing patents.   */
/* The original developer of this software module and his/her         */
/* company, the subsequent editors and their companies, and ISO/IEC   */
/* have no liability for use of this software module or modifications */ 
/* thereof in an implementation. Copyright is not released for non    */
/* MPEG-4 Visual (ISO/IEC 14496-2) conforming products.               */
/* The University of Rochester retains full right to use the code for */
/* their own purposes, assign or donate the code to a third party and */
/* to inhibit third parties from using the code for non MPEG-4 Visual */
/* (ISO/IEC 14496-2) conforming products.                             */
/* This copyright notice must be included in all copies or derivative */
/* works.                                                             */
/*                                                                    */
/* Copyright (c) 1997-1998                                            */
/*                                                                    */
/**********************************************************************/

/*+*********************************************************************
   NAME         queueClass.h
   AUTHOR       Peter Van Beek, University of Rochester
   DATE         Tue Aug 11 10:18:13 1998
   DESCRIPTION  Queue of triangle elements implemented by singly linked
                list. Queue elements are handled on a first-in-first-out 
                basis. Each element contains a pointer to a triangle
                as well as an edge number.
   SYNOPSIS     Include files:
                triRefClass.h
   
***************************************************************************-*/

#include <iostream.h>
#include "triRefClass.h"

#ifndef TRI_QUEUE_CLASS
#define TRI_QUEUE_CLASS

//------------------------------------------------------------------------

// Queue element class; contains pointer to triangle and one of its edges 
// and a pointer to the next element
class TriangleQueueElem {
friend class TriangleQueue;
  public:
	// pointer to a triangle
	Tri2dRef *tri;
	// edge number
	int edge;
  private:
	// pointer to next element
	TriangleQueueElem *next;
};


// Triplet Queue class
// Queue is implemented with singly linked list
class TriangleQueue {
  public:
	// normal constructor
	TriangleQueue ();
	// copy constructor
	TriangleQueue (const TriangleQueue&);
	// assignment operator
	TriangleQueue& operator= (const TriangleQueue&);
	// destructor
	~TriangleQueue ();
	// append new element at the end of the queue
	TriangleQueue& append (Tri2dRef *, int);
	// remove element from front of the queue and return it by reference
	TriangleQueue& remove (TriangleQueueElem&);
	// remove element from front of the queue and return its data
	TriangleQueue& remove(Tri2dRef *&tri, int &edge);
	// get data from the front element of the queue
	TriangleQueue& front(Tri2dRef *&tri, int &edge);
	// is the queue empty?
	int empty ();
	// print elements of the queue
	void print ();

  private:
	// pointers to first and last elements in linked list
	TriangleQueueElem *first, *last;
};


#endif
