/**********************************************************************/
/*                                                                    */
/* This software module was originally developed by                   */
/*                                                                    */
/* Peter van Beek (University of Rochester)                           */
/*                                                                    */
/* in the course of development of MPEG-4 Visual (ISO/IEC 14496-2).   */
/* This software module is an implementation of a part of one or      */
/* more MPEG-4 Visual (ISO/IEC 14496-2) tools as specified by the     */
/* MPEG-4 Visual (ISO/IEC 14496-2). ISO/IEC gives users of MPEG-4     */
/* Visual (ISO/IEC 14496-2) free license to this software module or   */
/* modifications thereof for use in hardware or software products     */
/* claiming conformance to MPEG-4 Visual (ISO/IEC 14496-2). Those     */
/* intending to use this software module in hardware or software      */
/* products are advised that its use may infringe existing patents.   */
/* The original developer of this software module and his/her         */
/* company, the subsequent editors and their companies, and ISO/IEC   */
/* have no liability for use of this software module or modifications */ 
/* thereof in an implementation. Copyright is not released for non    */
/* MPEG-4 Visual (ISO/IEC 14496-2) conforming products.               */
/* The University of Rochester retains full right to use the code for */
/* their own purposes, assign or donate the code to a third party and */
/* to inhibit third parties from using the code for non MPEG-4 Visual */
/* (ISO/IEC 14496-2) conforming products.                             */
/* This copyright notice must be included in all copies or derivative */
/* works.                                                             */
/*                                                                    */
/* Copyright (c) 1996-1998                                            */
/*                                                                    */
/**********************************************************************/

/*+***************************************************************************
   NAME         imageFunc.h
   AUTHOR       Peter van Beek, University of Rochester
   DATE         Wed Apr 17 11:01:13 1996 
   UPDATE       Peter Van Beek, Wed Nov  4 18:03:34 1998
   DESCRIPTION  Header file for basic image functions.
***************************************************************************-*/

#ifndef IM_FUNC_DEF
#define IM_FUNC_DEF

#include "imageClass.h"



// Type of data or component number
#define T_Y     1               // luminance
#define T_U     2               // chrominance 1
#define T_V     3               // chrominance 2
#define T_A     1               // alpha/segmentation
#define T_S     1               // single component
#define T_R     1               // red
#define T_G     2               // green
#define T_B     3               // blue

// Size of file-header
#define OFFSET_HEADER   0

// Morphological padding
#define BIGNUMBER   10000
#define DT_A        1
#define DT_B        2

// Grayscale alpha threshold
#define ALPHA_TH    1.0


// Function declarations
extern int read_im(image &im, char * fname, int frame);
extern int read_yuv(image &y, image &u, image &v, char * fname, int frame);
extern int write_im(image &im, char * fname, int frame);
extern int write_yuv(image &y, image &u, image &v, char * fname, int frame);

extern void im_draw_clip_line(image &im, int b, int x1, int y1, int x2, int y2, float col);

extern int im_padding(image &imi, image &mask, image &imo, int conn, int maxdist);

extern int im_extract_alpha (image &imalpha, int index, int mode);
extern double im_comp_psnr (image &yCurr, image &aCurr, image &yOrig, image &aOrig, double &totSSD, long &totPix);


#endif IM_FUNC_DEF


/* End of file */
