/**********************************************************************/
/*                                                                    */
/* This software module was originally developed by                   */
/*                                                                    */
/* Candemir Toklu (University of Rochester)                           */
/* A. Tanju Erdem (Eastman Kodak Company)                             */
/*                                                                    */
/* and edited by                                                      */
/*                                                                    */
/* Peter van Beek (University of Rochester)                           */
/*                                                                    */
/* in the course of development of MPEG-4 Visual (ISO/IEC 14496-2).   */
/* This software module is an implementation of a part of one or      */
/* more MPEG-4 Visual (ISO/IEC 14496-2) tools as specified by the     */
/* MPEG-4 Visual (ISO/IEC 14496-2). ISO/IEC gives users of MPEG-4     */
/* Visual (ISO/IEC 14496-2) free license to this software module or   */
/* modifications thereof for use in hardware or software products     */
/* claiming conformance to MPEG-4 Visual (ISO/IEC 14496-2). Those     */
/* intending to use this software module in hardware or software      */
/* products are advised that its use may infringe existing patents.   */
/* The original developer of this software module and his/her         */
/* company, the subsequent editors and their companies, and ISO/IEC   */
/* have no liability for use of this software module or modifications */ 
/* thereof in an implementation. Copyright is not released for non    */
/* MPEG-4 Visual (ISO/IEC 14496-2) conforming products.               */
/* Eastman Kodak Company and the University of Rochester retain full  */
/* right to use the code for their own purposes, assign or donate the */
/* code to a third party and to inhibit third parties from using the  */
/* code for non MPEG-4 Visual (ISO/IEC 14496-2) conforming products.  */
/* This copyright notice must be included in all copies or derivative */
/* works.                                                             */
/*                                                                    */
/* Copyright (c) 1994-1998                                            */
/*                                                                    */
/**********************************************************************/

/*+*********************************************************************
   NAME         matrixClass.cpp
   AUTHOR       Candemir Toklu - University of Rochester and 
                A. Tanju Erdem - Eastman Kodak Company. 
   DATE         September 1994
   UPDATE       14 August 1996 by Candemir Toklu
   DESCRIPTION  Member functions for Matrix class are defined in this
                file.

   SYNOPSIS     Include files : 
                                
                #include "matrixClass.h"
                #include "vectorClass.h"
                               
*********************************************************************-*/

#include "matrixClass.h"
#include "vectorClass.h"

//----------------------------------------------------------------------------

Matrix::Matrix()
{
	matrix = NULL;
	row = 0;
	column = 0;
}

Matrix::Matrix( int r, int c)
{
	int i;

	if( r <= 0 || c <= 0) {
		cout << "\nWarning: Row and column size must be positive! "
			 << "\nDefined an empty matrix." << endl;
		matrix = NULL;
		row = 0;
		column = 0;
	}
	else {
		row = r;
		column = c;
		matrix = (double**) new double*[r];
		for( i = 0; i < r; i++) {
			matrix[i] = (double*) new double[c];
		}
		for( i = 0; i < r; i++) {
			for( int j = 0; j < c; j++)
				{
					matrix[i][j] = 0.0;
				}
		}
	}
}

Matrix::Matrix( const Matrix& m)
{
	matrix = NULL;
	*this = m;
}

Matrix::Matrix( double f)
{
	matrix = (double**) new long[1];
	matrix[0] = (double*) new double[1];
	matrix[0][0] = f;
	row = column = 1;
}

Matrix::~Matrix()
{
	if( matrix) {
		for( int i = 0; i < row; i++) {
			delete[] matrix[i];
		}
		delete[] matrix;
	}
}

Matrix& Matrix::empty()
{
	if( matrix) {
		for( int i = 0; i < row; i++) {
			delete[] matrix[i];
		}
		delete[] matrix;
	}
	matrix = NULL;
	row = 0;
	column = 0;
	return *this;
}

Matrix& Matrix::setSize( int r, int c)
{
	int i;

	//this->~Matrix();
	empty();
	if( r <= 0 || c <= 0) {
		cout << "\nWarning: Row and column size must be positive! "
			 << "\nDefined an empty matrix." << endl;
		matrix = NULL;
		row = 0;
		column = 0;
	}
	else {
		row = r;
		column = c;
		matrix = (double**) new double*[r];
		for( i = 0; i < r; i++) {
			matrix[i] = (double*) new double[c];
		}
		for( i = 0; i < r; i++) {
			for( int j = 0; j < c; j++) {
				matrix[i][j] = 0.0;
			}
		}
	}
	return *this;
}

Matrix& Matrix::operator=( const Matrix& m)
{
	int i;

	if(matrix) {			
		// First, delete the old matrix if it is not empty
		for( i = 0; i < row; i++) {
			delete[] matrix[i];
		}
		delete[] matrix;	
	}

	if(m.matrix) {						  
		// Construct the new matrix
		row = m.row;
		column = m.column;

		matrix = (double**) new long[row];
		for( i = 0; i < row; i++) {
			matrix[i] = (double*) new double[column];
		}
		for( i = 0; i < row; i++) {
			for( int j = 0; j < column; j++) {
				matrix[i][j] = m.matrix[i][j];
			}
		}
	}
	else {
		matrix = NULL;
		row = 0;
		column = 0;
	}
	return *this;
}

Matrix& Matrix::operator=( double a)
{
	if(! matrix)  {
		for( int i = 0; i < row; i++) {
			for( int j = 0; j < column; j++) {
				matrix[i][j] = a;
			}
		}
	}
	return *this;
}

double& Matrix::operator()( int i, int j)
{
	if( ! matrix || i < 0 || i >= row || j < 0 || j >= column) {
		cout << "\nError: The indeces are out of range or the matrix is empty." 
			 << endl;
		exit(1);
	}
	else {
		return matrix[i][j];
	}
}

double& Matrix::operator[]( int i)
{
	if( i < 0 || i >= row*column) {
		cout << "\nError: The index is out of range." << endl;
		exit(1);
	}
	else {
		return matrix[i/column][i%column];
	}
}

Matrix& Matrix::operator*=( const Matrix& m)
{
	if( ! matrix || ! m.matrix) {
		if( matrix) {
			for( int i = 0; i < row; i++) {
				delete[] matrix[i];
			}
			delete[] matrix;
		}
		matrix = NULL;
		row = 0;
		column = 0;
	}
	else {
		if( column == m.row) {

			Matrix t(row, m.column);
			for( int i = 0; i < row; i++) {
				for( int j = 0; j < m.column; j++) {
					for( int k = 0; k < column; k++) {
						t.matrix[i][j] += matrix[i][k] * m.matrix[k][j];
					}
				}
			}
			*this = t;
		}
		else {
			cout << "\nWarning: The matrices are not compatible for multiplication."
				 << "\nThe first matrix is left unchanged" << endl;
		}
	}

	return *this;
}

Matrix& Matrix::operator+=( const Matrix& m)
{
	if( column == m.column && row == m.row) {
		for( int i = 0; i < row; i++) {
			for( int j = 0; j < column; j++) {
				matrix[i][j] += m.matrix[i][j];
			}
		}
	}
	else {
		cout << "\nWarning: The matrices are not compatible for addition."
			 << "\nThe first matrix is left unchanged" << endl;
	}

	return *this;
}

Matrix& Matrix::operator-=( const Matrix& m)
{
	if( column == m.column && row == m.row) {
		for( int i = 0; i < row; i++) {
			for( int j = 0; j < column; j++) {
				matrix[i][j] -= m.matrix[i][j];
			}
		}
	}
	else {
		cout << "\nWarning: The matrices are not compatible for addition."
			 << "\nThe first matrix is left unchanged" << endl;
	}

	return *this;
}

//------------------------------------------------------------------------
// Compute the adjoint of a matrix
//------------------------------------------------------------------------


Matrix& Matrix::adjoint()  // For 3x3 matrices only!
{
	if( row == column && row == 3) {
		Matrix t(3,3);

		for( int i = 0; i < 3; i++) {
			for( int j = 0; j < 3; j++) {
				t.matrix[j][i] = matrix[(i+1)%3][(j+1)%3] * 
					matrix[(i+2)%3][(j+2)%3]
					- matrix[(i+1)%3][(j+2)%3] * 
					matrix[(i+2)%3][(j+1)%3];
			}
		}

		*this = t;
	}
	else {
		cout << "\nWarning: Adjoint operation currently implemented only"
			 << "for 3x3 matrices." << endl;
	}

	return *this;	 
}

Matrix& Matrix::ludcmp( int* indx, double* d)
{
  
	int		i,imax,j,k,n;
	double	 big,dum,sum,temp;
	
	n=row;
	Vector vv(n);
	*d=1.0;
	for( i=0;i<n;i++) {
		big=0.0;
		for( j=0;j<n;j++)
			if ((temp=fabs(matrix[i][j])) > big) big=temp;
		if (big==0.0) cout << "\nSingular Matrix in routine LUDCMP" << endl;
		vv[i]=1.0/big;
	}
	for( j=0;j<n;j++) {
		for( i=0;i<j;i++) {
			sum=matrix[i][j];
			for( k=0;k<i;k++)
				sum -= matrix[i][k]*matrix[k][j];
			matrix[i][j]=sum;
		}
		big=0.0;
		for( i=j;i<n;i++) {
			sum=matrix[i][j];
			for( k=0;k<j;k++)
				sum -= matrix[i][k]*matrix[k][j];	 
			matrix[i][j]=sum;
			if( ( dum=vv[i]*fabs(sum)) >= big) {
				big=dum;
				imax=i;
			}
		}
		if( j != imax) {
			for( k=0;k<n;k++) {
				dum=matrix[imax][k];
				matrix[imax][k]=matrix[j][k];
				matrix[j][k]=dum;
			}
			*d = -(*d);
			vv[imax]=vv[j];
		}
		indx[j]=imax;
		if( matrix[j][j] == 0.0) matrix[j][j]=TINY;
		if( j != n) {
			dum=1.0/(matrix[j][j]);
			for( i=j+1;i<n;i++) matrix[i][j] *= dum;
		}	 
	}  // for j
	return	*this;
} // ludcmp

Matrix& Matrix::lubksb(int* indx, Vector& b)
{
	int	   n,i,ii=-1,ip,j;
	double	sum;
	n=row;
	for( i=0;i<n;i++) {
		ip=indx[i];
		sum=b[ip];
		b[ip]=b[i];
		if (ii>=0)
			for( j=ii;j<=i-1;j++) sum -= matrix[i][j]*b[j];
		else if (sum) ii=i;
		b[i]=sum;
	}
	for( i=n-1;i>=0;i--) {
		sum=b[i];
		for( j=i+1;j<n;j++) sum -= matrix[i][j]*b[j];
		b[i]=sum/matrix[i][i];
	}
	return *this;
}

Matrix& Matrix::inverse()
{
	if( column!=row) {
		cout << "\nError: The matrix is not a square matrix."
			 << "\nOperation Failed, exiting" << endl;
		exit(0);
	}
	else {
		int   *index,n,i,j;
		double d;
		Matrix	 temp;
	 
		temp=*this;
		n=row;
		index= (int *) new int[n];
		Vector col(n);
		temp.ludcmp(index,&d);
		for( j=0;j<n;j++) { 
			for( i=0;i<n;i++) col[i]=0.0;
			col[j]=1.0;
			temp.lubksb(index,col);
			for( i=0;i<n;i++) matrix[i][j]=col[i];
		}
		delete[] index;
	}
	return *this;	 
}

// Solve linear system; note that b is changed
void Matrix::linearSysSol(Vector& b)
{
	if( b.getN()==row) {	 
		int *index,n;
		double d;
		n=row;
		index= (int *) new int[n];
		ludcmp(index,&d);
		lubksb(index,b);
		delete[] index; 
	}
	else {
		cout << "\nError:Vector and Matrix Dimensions' mismatch"
			 << "\nExiting!" << endl;
		exit(0);
	}	 
}

Matrix& Matrix::transpose()
{
	if( !matrix) {
		cout << "\nWarning: Null matrix. Transpose is also null." << endl;
	}
	else { 
		Matrix MT( column, row);
		for( int i = 0; i < row; i++) {
			for( int j = 0; j < column; j++) {
				MT.matrix[j][i] = matrix[i][j];
			}
		}
		(*this) = MT;
	}
	return *this;
}

// Least squares routines; note that b is changed
// b is nx1, matrix is nxm
void Matrix::leastSquares(Vector& b)   
{
	if( b.getN()==row) {
	
		Matrix mt;
		Vector bt(column);
	
		mt=(*this);
		mt.transpose();			// mt is mxn
		mt*=(*this) ;			 // mt is mxm
		for (int i=0; i<column; i++) {
			bt[i]=0.0;
			for( int j=0; j<row; j++)
				{
					bt[i]+=matrix[j][i]*b[j];  //multiply with the transpose;
				} 
		}
		b=bt;
		mt.linearSysSol(b);
	}
	else {
		cout << "\nError:Vector and Matrix Dimensions' mismatch"
			 << "\nExiting!" << endl;
		exit(0);
	}					
}

Vector Matrix::vectorRow( int r)  // Extract a row into a vector
{
	Vector v(column);
	for( int i = 0; i < column; i++)
		v[i] = matrix[r][i];
	return v;
}

Vector Matrix::vectorCol( int c)  // Extract a column into a vector
{
	Vector v(row);
	for( int i = 0; i < row; i++)
		v[i] = matrix[i][c];
	return v;
}

void Matrix::print()
{
	for( int i = 0; i < row; i++) {
		cout << "\n";
		for( int j = 0; j < column; j++) {
			cout << "	 " << matrix[i][j];
		}
		cout << endl;
	}
}

int Matrix::read(ifstream& fp)
{
	int tmpRow, tmpColumn;
	int i;

	fp >> tmpRow >> tmpColumn;
	if( tmpRow < 1 || tmpColumn < 1) return 0;
	if( tmpRow != row || tmpColumn != column) {
		// Delete the old matrix
		for( i = 0; i < row; i++) {
			delete[] matrix[i];
		}
		delete[] matrix;
  
		// Construct the new matrix
		row = tmpRow;
		column = tmpColumn;
		matrix = (double**) new long[row];
		for( i = 0; i < row; i++) {
			matrix[i] = (double*) new double[column];
		}
	}

	for( i = 0; i < row; i++) {
		for( int j = 0; j < column; j++) {
			if(!fp) return 0;
			fp >> matrix[i][j];
		}
	}

	return 1;
}

int Matrix::write(ofstream& fp)
{
	if(!fp) return 0;
	fp << row << " " << column << "\n";
	for( int i = 0; i < row; i++) {
		for( int j = 0; j < column - 1; j++) {
			if(!fp) return 0;
			fp.width(10);
			fp << matrix[i][j] << " ";
		}
		if(!fp) return 0;
		fp.width(10);
		fp << matrix[i][ column - 1] << endl;
	}
	return 1;
}

// end of file
