/**********************************************************************/
/*                                                                    */
/* This software module was originally developed by                   */
/*                                                                    */
/* Candemir Toklu (University of Rochester)                           */
/* A. Tanju Erdem (Eastman Kodak Company)                             */
/*                                                                    */
/* and edited by                                                      */
/*                                                                    */
/* Peter van Beek (University of Rochester)                           */
/*                                                                    */
/* in the course of development of MPEG-4 Visual (ISO/IEC 14496-2).   */
/* This software module is an implementation of a part of one or      */
/* more MPEG-4 Visual (ISO/IEC 14496-2) tools as specified by the     */
/* MPEG-4 Visual (ISO/IEC 14496-2). ISO/IEC gives users of MPEG-4     */
/* Visual (ISO/IEC 14496-2) free license to this software module or   */
/* modifications thereof for use in hardware or software products     */
/* claiming conformance to MPEG-4 Visual (ISO/IEC 14496-2). Those     */
/* intending to use this software module in hardware or software      */
/* products are advised that its use may infringe existing patents.   */
/* The original developer of this software module and his/her         */
/* company, the subsequent editors and their companies, and ISO/IEC   */
/* have no liability for use of this software module or modifications */ 
/* thereof in an implementation. Copyright is not released for non    */
/* MPEG-4 Visual (ISO/IEC 14496-2) conforming products.               */
/* Eastman Kodak Company and the University of Rochester retain full  */
/* right to use the code for their own purposes, assign or donate the */
/* code to a third party and to inhibit third parties from using the  */
/* code for non MPEG-4 Visual (ISO/IEC 14496-2) conforming products.  */
/* This copyright notice must be included in all copies or derivative */
/* works.                                                             */
/*                                                                    */
/* Copyright (c) 1994-1998                                            */
/*                                                                    */
/**********************************************************************/

/*+*********************************************************************
   NAME         matrixClass.h
   AUTHOR       Candemir Toklu - University of Rochester and 
                A. Tanju Erdem - Eastman Kodak Company.
   DATE         September 1994
   UPDATE       14 August 1996 by Candemir Toklu
   DESCRIPTION  The Matrix class defines a matrix. Size of the matrix
                can be changed dynamically.

   SYNOPSIS     Include files : 
                                
                #include <stdlib.h>
                #include <iostream.h>
                #include <fstream.h>
                #include <math.h>
                #include <time.h>

*********************************************************************-*/

#include <stdlib.h>
#include <iostream.h>
#include <fstream.h>
#include <math.h>
#include <time.h>

class Vector;

#ifndef MATRIX_CLASS
#define MATRIX_CLASS

#define	 TINY 1.0e-20

///--------------------------------------------------------------------------
// Matrix class.
class Matrix {
  public:
	Matrix();  // 0x0 matrix
	Matrix( int, int);	// rxc matrix. All enties initialized to zero.
	Matrix( const Matrix&);
	Matrix( double);
	~Matrix();
	Matrix& empty();
	Matrix& setSize( int, int);
	Matrix& operator=( const Matrix&);
	Matrix& operator=( double);
	double& operator()( int, int);
	double& operator[]( int);
	Matrix& operator*=( const Matrix&);
	Matrix& operator+=( const Matrix&);
	Matrix& operator-=( const Matrix&);
	inline int getRow();
	inline int getColumn();
	Matrix& adjoint();	// For 3x3 matrices only!
	Matrix& ludcmp( int*, double*);
	Matrix& lubksb( int*, Vector&);
	Matrix& inverse();
	void linearSysSol( Vector&);
	Matrix& transpose();
	void leastSquares( Vector&);
	Vector vectorRow( int);	 // Extract a row into a vector
	Vector vectorCol( int);	 // Extract a column into a vector
	void print();
	int read( ifstream&);
	int write( ofstream&);
  private:
	double** matrix;
	int row;
	int column;
};

int Matrix::getRow()
{
  return row;
}

int Matrix::getColumn()
{
  return column;
}

#endif
