/**********************************************************************/
/*                                                                    */
/* This software module was originally developed by                   */
/*                                                                    */
/* A. Tanju Erdem (Eastman Kodak Company)                             */
/* Candemir Toklu (University of Rochester)                           */
/*                                                                    */
/* and edited by                                                      */
/*                                                                    */
/* Peter van Beek (University of Rochester)                           */
/*                                                                    */
/* in the course of development of MPEG-4 Visual (ISO/IEC 14496-2).   */
/* This software module is an implementation of a part of one or      */
/* more MPEG-4 Visual (ISO/IEC 14496-2) tools as specified by the     */
/* MPEG-4 Visual (ISO/IEC 14496-2). ISO/IEC gives users of MPEG-4     */
/* Visual (ISO/IEC 14496-2) free license to this software module or   */
/* modifications thereof for use in hardware or software products     */
/* claiming conformance to MPEG-4 Visual (ISO/IEC 14496-2). Those     */
/* intending to use this software module in hardware or software      */
/* products are advised that its use may infringe existing patents.   */
/* The original developer of this software module and his/her         */
/* company, the subsequent editors and their companies, and ISO/IEC   */
/* have no liability for use of this software module or modifications */ 
/* thereof in an implementation. Copyright is not released for non    */
/* MPEG-4 Visual (ISO/IEC 14496-2) conforming products.               */
/* Eastman Kodak Company and the University of Rochester retain full  */
/* right to use the code for their own purposes, assign or donate the */
/* code to a third party and to inhibit third parties from using the  */
/* code for non MPEG-4 Visual (ISO/IEC 14496-2) conforming products.  */
/* This copyright notice must be included in all copies or derivative */
/* works.                                                             */
/*                                                                    */
/* Copyright (c) 1994-1998                                            */
/*                                                                    */
/**********************************************************************/

/*+*********************************************************************
   NAME         parClass.h
   AUTHOR       A. Tanju Erdem - Eastman Kodak Company and Candemir Toklu - 
                University of Rochester.
   DATE         September 1994
   UPDATE       14 August 1996 by Candemir Toklu
   DESCRIPTION  The ParAffine class defines the affine motion model 
                parameters. 

   SYNOPSIS     Include files : 
                                
                #include <iostream.h>
                #include <fstream.h>
                #include <math.h>
                #include <time.h>
                #include "pointClass.h"
                #include "polyClass.h"

*********************************************************************-*/

#include <iostream.h>
#include <fstream.h>
#include <math.h>
#include <time.h>
#include "pointClass.h"
#include "polyClass.h"

class Matrix;
class Vector;

#ifndef TRANSFORMATION_CLASS
#define TRANSFORMATION_CLASS

// The following are used to indicate the direction and pixel-assignment 
// method for spatial transformations 
enum mapDirection { MFORWARD, MBACKWARD};
enum mapMethod { TRANSL, TRRTZM, AFFINE, PERS, BILIN};
enum interpMethod { NEAREST, BILINEAR, AREAWISE};
enum movingMethod { PROJFROM3D, MOVECORNERS};
enum projMethod { ORTHOGRAPHIC, PERSPECTIVE};

///--------------------------------------------------------------------------

//--------------------------------------------------------------------------
// Affine transformation model parameters
class ParAffine{
	friend class Point2d;
	friend class ParPers;
	friend class ParBilin;
  public:
	ParAffine();
	ParAffine( double, double, 
			   double, double,
			   double, double);
	ParAffine( const ParAffine&);
	ParAffine& operator=( const ParAffine&);
	void toMatrix( Matrix&);
	void fromMatrix( Matrix&);
	ParAffine& getParUtoT( Tri2d);
	ParAffine& cascade( ParAffine&, ParAffine&);
	ParAffine& getParTtoUInverse( Tri2d);
	ParAffine& getParTtoUDirect( Tri2d);
	Point2d tform( Point2d&);
	Poly2d tform( Poly2d&);
	int getPars( Poly2d, Poly2d);
	void print();
	int read( istream& fp);
	int write( ostream& fp);
	mapMethod type();
	void printType();
	int sizePar();
  private:
	double a11, a12,
		   a21, a22,
		   a31, a32;
};

#endif
