/**********************************************************************/
/*                                                                    */
/* This software module was originally developed by                   */
/*                                                                    */
/* A. Tanju Erdem (Eastman Kodak Company)                             */
/* Candemir Toklu (University of Rochester)                           */
/*                                                                    */
/* and edited by                                                      */
/*                                                                    */
/* Peter van Beek (University of Rochester)                           */
/*                                                                    */
/* in the course of development of MPEG-4 Visual (ISO/IEC 14496-2).   */
/* This software module is an implementation of a part of one or      */
/* more MPEG-4 Visual (ISO/IEC 14496-2) tools as specified by the     */
/* MPEG-4 Visual (ISO/IEC 14496-2). ISO/IEC gives users of MPEG-4     */
/* Visual (ISO/IEC 14496-2) free license to this software module or   */
/* modifications thereof for use in hardware or software products     */
/* claiming conformance to MPEG-4 Visual (ISO/IEC 14496-2). Those     */
/* intending to use this software module in hardware or software      */
/* products are advised that its use may infringe existing patents.   */
/* The original developer of this software module and his/her         */
/* company, the subsequent editors and their companies, and ISO/IEC   */
/* have no liability for use of this software module or modifications */ 
/* thereof in an implementation. Copyright is not released for non    */
/* MPEG-4 Visual (ISO/IEC 14496-2) conforming products.               */
/* Eastman Kodak Company and the University of Rochester retain full  */
/* right to use the code for their own purposes, assign or donate the */
/* code to a third party and to inhibit third parties from using the  */
/* code for non MPEG-4 Visual (ISO/IEC 14496-2) conforming products.  */
/* This copyright notice must be included in all copies or derivative */
/* works.                                                             */
/*                                                                    */
/* Copyright (c) 1994-1998                                            */
/*                                                                    */
/**********************************************************************/
/*+*********************************************************************
   NAME         vect2dClass.h
   AUTHOR       A. Tanju Erdem - Eastman Kodak Company and Candemir Toklu - 
                University of Rochester.
   DATE         September 1994
   UPDATE       14 August 1996 by Candemir Toklu
   DESCRIPTION  The vect2dClass defines a 2-D vector. 
                Its public members are two Point2d's. This class is useful
                for checking intersections among line segments.

   SYNOPSIS     Include files : 
                                
                #include <iostream.h>
                #include <fstream.h>
                #include <math.h>
                #include <time.h>
                #include "pointClass.h"

*********************************************************************-*/
#include <iostream.h>
#include <fstream.h>
#include <math.h>
#include <time.h>
#include "pointClass.h"

#ifndef VECTOR2D_CLASS
#define VECTOR2D_CLASS

//------------------------------------------------------------------------

// Vect2d is the set of points that lie on the line from Point tail to
// Point head, including the Point tail but excluding the point head.
class Vect2d {
  public:
	Point2d tail;
	Point2d head;
	inline Vect2d();
	inline Vect2d( const Vect2d&);
	inline Vect2d( const Point2d&, const Point2d&);
	Vect2d( const Point2d&);
	inline ~Vect2d();
	inline Vect2d& operator=( const Vect2d&);
	inline int operator==( const Vect2d&);
	inline int operator!=( const Vect2d&);
	void print();
};

//------------------------------------------------------------------------
// Constructors for class Vect2d

Vect2d::Vect2d()  // Initialization
{;}

Vect2d::Vect2d( const Vect2d& v)  // Initialization
{
	tail = v.tail;
	head = v.head;
}

Vect2d::Vect2d( const Point2d& t, const Point2d& h)	 // Initialization
{
	tail = t;
	head = h;
}

Vect2d::~Vect2d()  // Destructor
{;}


//------------------------------------------------------------------------
// Operators for class Vect2d

Vect2d& Vect2d::operator=( const Vect2d& v)
{
	tail = v.tail;
	head = v.head;
	return *this;
}

int Vect2d::operator==( const Vect2d& v)
{
	return( tail == v.tail && head == v.head);
}

int Vect2d::operator!=( const Vect2d& v)
{
	return( tail != v.tail || head != v.head);
}


#endif
