/**********************************************************************/
/*                                                                    */
/* This software module was originally developed by                   */
/*                                                                    */
/* Candemir Toklu (University of Rochester)                           */
/* A. Tanju Erdem (Eastman Kodak Company)                             */
/*                                                                    */
/* and edited by                                                      */
/*                                                                    */
/* Peter van Beek (University of Rochester)                           */
/*                                                                    */
/* in the course of development of MPEG-4 Visual (ISO/IEC 14496-2).   */
/* This software module is an implementation of a part of one or      */
/* more MPEG-4 Visual (ISO/IEC 14496-2) tools as specified by the     */
/* MPEG-4 Visual (ISO/IEC 14496-2). ISO/IEC gives users of MPEG-4     */
/* Visual (ISO/IEC 14496-2) free license to this software module or   */
/* modifications thereof for use in hardware or software products     */
/* claiming conformance to MPEG-4 Visual (ISO/IEC 14496-2). Those     */
/* intending to use this software module in hardware or software      */
/* products are advised that its use may infringe existing patents.   */
/* The original developer of this software module and his/her         */
/* company, the subsequent editors and their companies, and ISO/IEC   */
/* have no liability for use of this software module or modifications */ 
/* thereof in an implementation. Copyright is not released for non    */
/* MPEG-4 Visual (ISO/IEC 14496-2) conforming products.               */
/* Eastman Kodak Company and the University of Rochester retain full  */
/* right to use the code for their own purposes, assign or donate the */
/* code to a third party and to inhibit third parties from using the  */
/* code for non MPEG-4 Visual (ISO/IEC 14496-2) conforming products.  */
/* This copyright notice must be included in all copies or derivative */
/* works.                                                             */
/*                                                                    */
/* Copyright (c) 1994-1998                                            */
/*                                                                    */
/**********************************************************************/

/*+*********************************************************************
   NAME         vectorClass.h
   AUTHOR       Candemir Toklu - University of Rochester and 
                A. Tanju Erdem - Eastman Kodak Company.
   DATE         September 1994
   UPDATE       14 August 1996 by Candemir Toklu
   DESCRIPTION  The Vector class defines a 1-D vector. Size of the vector
                can be changed dynamically.

   SYNOPSIS     Include files : 
                                
                #include <stdlib.h>
                #include <iostream.h>
                #include <fstream.h>
                #include <math.h>
                #include <time.h>

*********************************************************************-*/

#include <stdlib.h>
#include <iostream.h>
#include <fstream.h>
#include <math.h>
#include <time.h>


class Matrix;

#ifndef MYVECTOR_CLASS
#define MYVECTOR_CLASS

//-------------------------------------------------------------------------
// Vector class.

class Vector {
  public:
	Vector();  // 0x0 vector
	Vector( int c);	 // cx1 vector. All enties initialized to zero.
	Vector( const Vector&);
	Vector( double);
	~Vector();
	Vector& empty();
	Vector& setSize( int);
	Vector& operator=( const Vector&);
	Vector& operator=( double);
	Vector& operator*=( double);
	Vector& operator/=( double);
	double& operator()( int);
	double& operator[]( int);
	Vector& operator+=( const Vector&);
	Vector& operator-=( const Vector&);
	int setN( int);
	inline int getN();
	Vector& append( double);
	Vector& remove( int);
	int where( double);
	Vector& leftMult( Matrix&);
	Vector& rightMult( Matrix&);
	Matrix matrixRow();	 // Make a row matrix from a vector
	Matrix matrixCol();	 // Make a column matrix from a vector
	void print();
	int read( ifstream&);
	int write( ofstream&);
  private:
	double* column;
	int n;
};

int Vector::getN()
{
	return n ;
}

#endif
