/**********************************************************************/
/*                                                                    */
/* This software module was originally developed by                   */
/*                                                                    */
/* A. Tanju Erdem (Eastman Kodak Company)                             */
/* Candemir Toklu (University of Rochester)                           */
/*                                                                    */
/* and edited by                                                      */
/*                                                                    */
/* Peter van Beek (University of Rochester)                           */
/*                                                                    */
/* in the course of development of MPEG-4 Visual (ISO/IEC 14496-2).   */
/* This software module is an implementation of a part of one or      */
/* more MPEG-4 Visual (ISO/IEC 14496-2) tools as specified by the     */
/* MPEG-4 Visual (ISO/IEC 14496-2). ISO/IEC gives users of MPEG-4     */
/* Visual (ISO/IEC 14496-2) free license to this software module or   */
/* modifications thereof for use in hardware or software products     */
/* claiming conformance to MPEG-4 Visual (ISO/IEC 14496-2). Those     */
/* intending to use this software module in hardware or software      */
/* products are advised that its use may infringe existing patents.   */
/* The original developer of this software module and his/her         */
/* company, the subsequent editors and their companies, and ISO/IEC   */
/* have no liability for use of this software module or modifications */ 
/* thereof in an implementation. Copyright is not released for non    */
/* MPEG-4 Visual (ISO/IEC 14496-2) conforming products.               */
/* Eastman Kodak Company and the University of Rochester retain full  */
/* right to use the code for their own purposes, assign or donate the */
/* code to a third party and to inhibit third parties from using the  */
/* code for non MPEG-4 Visual (ISO/IEC 14496-2) conforming products.  */
/* This copyright notice must be included in all copies or derivative */
/* works.                                                             */
/*                                                                    */
/* Copyright (c) 1994-1998                                            */
/*                                                                    */
/**********************************************************************/

/*+*********************************************************************
   NAME         warpingProto.h
   AUTHOR       A. Tanju Erdem - Eastman Kodak Company, Candemir Toklu - 
                University of Rochester 
   DATE         September 1994
   UPDATE       14 August 1996 by Candemir Toklu
                Peter Van Beek, Mon Nov 17 15:28:39 1997
   DESCRIPTION  Here prototypes of functions for intensity warping, error
                computation, several polygon utilities are listed. In order
                to access to polygon and mesh tools including this file
                is enough since all the include files for those tools
                are included here.

   SYNOPSIS     Include files : 
                                
                #include <iostream.h>
                #include <fstream.h>
                #include <math.h>
                #include <time.h>
                #include "imageClass.h"
                #include "pointClass.h"
                #include "linkClass.h"
                #include "listClass.h"
                #include "polyClass.h"
                #include "vect2dClass.h"
                #include "parClass.h"
                #include "polyRefClass.h"
                #include "polyTessClass.h"
                #include "vectorClass.h"
                #include "matrixClass.h"
                #include "general.h"

*********************************************************************-*/

#ifndef WARPINGPROTO_H_DEF
#define WARPINGPROTO_H_DEF


#include <iostream.h>
#include <fstream.h>
#include <math.h>
#include <time.h>
#include "general.h"
#include "imageClass.h"
#include "pointClass.h"
#include "polyClass.h"
#include "linkClass.h"
#include "listClass.h"
#include "vect2dClass.h"
#include "parClass.h"
#include "vectorClass.h"
#include "matrixClass.h"
#include "triRefClass.h"
#include "triTessClass.h"


//-----------------------------------------------------------------------------
// Inline functions

//------------------------------------------------------------------------
inline Vect2d vector2d( const Point2d& t, const Point2d& h)
{
	return Vect2d( t, h);
}

//-----------------------------------------------------------------------------
// polygonFunc.cpp
Quad2d squareAroundPoint( Point2d&, double);
Poly2d cellAroundPixel( int, int);
void smallestRect( Poly2d&, int&, int&, int&, int&, int, int);
int intersection( Vect2d, Vect2d, Point2d&);
Poly2d intersection(Poly2d, Poly2d, int);
int intersectHoriz( double, Vect2d, Point2d&, Point2d&);
int getEndPoints( double, Poly2d&, Point2d&, Point2d&);
int getEndPoints( double, int, int, Poly2d&, Point2d&, Point2d&);


//-----------------------------------------------------------------------------
// mappingFunc.cpp
int mapTessImage(image&, image&, Tri2dTess&, Tri2dTess&, interpMethod);
int mapRegion( ParAffine, image&, image&, Poly2d, Poly2d, int, mapDirection, 
		   interpMethod, double, double);
int mapRegion( ParAffine, image&, image&, Poly2d, int, mapDirection, 
		   interpMethod, double, double);
int decide( int&, int&, Poly2d&, int, int);
int decide( int&, int&, int&, int&, Poly2d&, int);
void interpNearestN( ParAffine, image&, image&, int, int, int, int, double,
			 double);
void interpBilinear( ParAffine, image&, image&, int, int, int, int, double, 
			 double);
void interpAreawise( ParAffine, image&, image&, int, int, int, int, Poly2d&, 
			 double, double);



#endif WARPINGPROTO_H_DEF
