// =====================================================
//
// File:   dct.cc
// Class:  DCT
// 
// Author: Jrgen Ahlberg <ahlberg@isy.liu.se>
//         Dept. of Electrical Engineering
//         Linkping University
//         SE-581 83 Linkping, SWEDEN
// Date:   1998-01-12
//
// This program is distributed in the hope that it will
// be useful, but WITHOUT ANY WARRANTY.
//
// =====================================================


#include "dct.hh"


// ==========================================
//
// Initialization
//
// ==========================================

DCT::DCT(int sz) {
  float a;
  int i,j;
  size = sz;
  
  dct_matrix = new float[size*size];
  
  for(i = 0; i < size; i++) {
    if(i == 0) {
      a = (float)(1.0/sqrt(2));
    } else {
      a = 1.0;
    }
    for(j = 0; j < size; j++)
      dct_matrix[i*size+j] = float(sqrt(2.0/size)) * a *
	float(cos(3.1415926*float(2*j+1)*float(i)/2.0/float(size)));
  }
}


// ==========================================
//
// encode (transform)
//
// ==========================================

void DCT::
encode(float *input, int modulo, float *output) {
  int i,j;
  float norm = sqrt(size);
  for (i = 0; i < size; i++){
    output[i] = 0;
    for (j = 0; j < size; j++)
      output[i] += dct_matrix[i*size+j] * input[j*modulo];
    output[i] /= norm;
  }
}

// ==========================================

void DCT::
encode(float *input, int modulo) {
  float *output = new float[size];
  encode(input, modulo, output);
  for (int i = 0; i < size; i++)
    input[i*modulo] = output[i];
  delete output;
}


// ==========================================
//
// decode (inverse transform)
//
// ==========================================

void DCT::
decode(float *input, int modulo, float *output) {
  int i,j;
  float norm = sqrt(size);
  for (i = 0; i < size; i++){
    output[i] = 0;
    for (j = 0; j < size; j++)
      output[i] += dct_matrix[j*size+i] * input[j*modulo];
    output[i] *= norm;
  }
}

// ==========================================

void DCT::
decode(float *input, int modulo) {
  float *output = new float[size];
  decode(input, modulo, output);
  for (int i = 0; i < size; i++)
    input[i*modulo] = output[i];
  delete output;
}
