// =====================================================
//
// File:   fit.cc
// Class:  FIT (several classes)
//         Routines for interpolation.
// 
// Author: Jrgen Ahlberg <ahlberg@isy.liu.se>
//         Dept. of Electrical Engineering
//         Linkping University
//         SE-581 83 Linkping, SWEDEN
// Date:   1998-01-12
//
// This program is distributed in the hope that it will
// be useful, but WITHOUT ANY WARRANTY.
//
// =====================================================


#include "fit.hh"


// =====================================================
//
// interpolate
//
// =====================================================

FAPs& FIT::
interpolate(FAPs& faps) {

  int nodei, parenti, linki, uplinki, fapi, fap, ip_count;
  FIT_Node node, parent;
  FIT_Link link;
  FIT_Function function;
  Mask ip_mask, fullmask;
  FAPs *newfaps;

  // Create new FAPs

  fullmask.resize(NFAPS);
  fullmask.set_all();
  newfaps = new FAPs(fullmask, faps.getInfo());
  newfaps->empty();
  newfaps->insert(faps);
  ip_mask.copy(faps.getFAPmask());

  // Algorithm according to document M2599

  ip_count = 1;
  while(ip_count) {
    ip_count = 0;
    for(nodei = 0; nodei < nodelist.nnodes; nodei++) {
      node = nodelist.nodes[nodei];
      for(uplinki = 0; uplinki < node.nuplinks; uplinki++) {
	linki = node.uplinks[uplinki];
	link = linklist.links[linki];
	if(nodei != link.toNode) {
	  cerr << "Internal error in FIT::interpolate()!\n";
	  exit(0);
	}
	parenti = link.fromNode;
	parent = nodelist.nodes[parenti];
	for(fapi = 0; fapi < node.nfaps; fapi++) {
	  fap = node.faps[fapi];
	  if(!ip_mask.bit(fap-1)) {
	    function = link.fcns[fapi];
	    if(function.fapsavailable(parent, ip_mask)) {
	      interpolate(fap, function, parent, *newfaps);
	      ip_mask.set(fap-1);
	      ip_count = 1;
	    }
	  }
	}
      }
    }
  }
  
  // Remove FAPs that never got interpolated

  newfaps->remove_params(ip_mask);

  return *newfaps;
}

// =====================================================

void FIT::
interpolate(int fap, FIT_Function& function, 
	    FIT_Node& parent, FAPs& faps) {
  for(int frame=0;frame<faps.nframes();frame++) {
    faps.set(frame,fap-1,function.evaluate(parent,faps,frame));
  }
}


// =====================================================
//
// fapsavailable
//
// =====================================================

int FIT_Function::
fapsavailable(FIT_Node& parent, Mask& ip_mask) {
  if(numerator.fapsavailable(parent,ip_mask)) {
    return denominator.fapsavailable(parent,ip_mask);
  } else {
    return false;
  }
}

// =====================================================

int FIT_Polynomial::
fapsavailable(FIT_Node& parent, Mask& ip_mask) {
  for(int termi=0;termi<nterms;termi++)
    if(!terms[termi].fapsavailable(parent,ip_mask))
      return false;
  return true;
}

// =====================================================

int FIT_Term::
fapsavailable(FIT_Node& parent, Mask& ip_mask) {
  for(int fapi=0;fapi<nfaps;fapi++)
    if(!faps[fapi].fapsavailable(parent,ip_mask))
      return false;
  return true;
}

// =====================================================

int FIT_FAP::
fapsavailable(FIT_Node& parent, Mask& ip_mask) {
  return ip_mask.bit(parent.faps[index]-1);
}


