// =====================================================
//
// File:   fit.hh
// Class:  FIT (Several classes)
// Uses:   FAPs
// 
// Author: Jrgen Ahlberg <ahlberg@isy.liu.se>
//         Dept. of Electrical Engineering
//         Linkping University
//         SE-581 83 Linkping, SWEDEN
// Date:   1998-01-12
//
// This program is distributed in the hope that it will
// be useful, but WITHOUT ANY WARRANTY.
//
// =====================================================


#ifndef FIT_HH
#define FIT_HH

#include <iostream.h>
#include "../sequence/faps.hh"


// =====================================================
//
// FIT Nodes
//
// =====================================================

class FIT_LinkList;

class FIT_Node {
public:
  int nfaps, *faps;
  int nuplinks, *uplinks;

  void connect(FIT_LinkList&, int);

  friend istream& operator >> (istream&, FIT_Node&);
  friend ostream& operator << (ostream&, FIT_Node&);
};

// =====================================================

class FIT_NodeList {
public:  
  int nnodes;
  FIT_Node *nodes;

  friend istream& operator >> (istream&, FIT_NodeList&);
  friend ostream& operator << (ostream&, FIT_NodeList&);
};


// =====================================================
//
// FIT Polynomials
//
// =====================================================

class FIT_FAP {
public:
  int index, numerator, denominator;

  float evaluate(FIT_Node&, FAPs&, int);
  int fapsavailable(FIT_Node&, Mask&);

  friend istream& operator >> (istream&, FIT_FAP&);
  friend ostream& operator << (ostream&, FIT_FAP&);
};

// =====================================================

class FIT_Term {
public:
  int numerator, nfaps;
  FIT_FAP *faps;
  
  float evaluate(FIT_Node&, FAPs&, int);
  int fapsavailable(FIT_Node&, Mask&);

  friend istream& operator >> (istream&, FIT_Term&);
  friend ostream& operator << (ostream&, FIT_Term&);
};

// =====================================================

class FIT_Polynomial {
public:
  int nterms, denominator;
  FIT_Term *terms;
  
  float evaluate(FIT_Node&, FAPs&, int);
  int fapsavailable(FIT_Node&, Mask&);

  friend istream& operator >> (istream&, FIT_Polynomial&);
  friend ostream& operator << (ostream&, FIT_Polynomial&);
};

// =====================================================

class FIT_Function {
public:
  FIT_Polynomial numerator, denominator;
  
  float evaluate(FIT_Node&, FAPs&, int);
  int fapsavailable(FIT_Node&, Mask&);

  friend istream& operator >> (istream&, FIT_Function&);
  friend ostream& operator << (ostream&, FIT_Function&);
};


// =====================================================
//
// FIT Links
//
// =====================================================

class FIT_Link {
public:
  int fromNode, toNode, priority, nfcns;
  FIT_Function *fcns;
  
  void read(FIT_NodeList&, istream&);
  friend ostream& operator << (ostream&, FIT_Link&) ;
};

// =====================================================

class FIT_LinkList {
public:
  int nlinks;
  FIT_Link *links;
  
  void read(FIT_NodeList&, istream&);
  friend ostream& operator << (ostream&, FIT_LinkList&);
};


// =====================================================
//
// FIT Superclass
//
// =====================================================

class FIT {
public:
  FIT_NodeList nodelist;
  FIT_LinkList linklist;

  void connect(void);
  FAPs& interpolate(FAPs&);
  void interpolate(int, FIT_Function&, FIT_Node&, FAPs&);

  friend istream& operator >> (istream&, FIT&);
  friend ostream& operator << (ostream&, FIT&);
};


#endif
