// =====================================================
//
// File:   fit_eval.cc
// Class:  FIT (several classes)
//         Routines for evaluation of FOT-functions.
// 
// Author: Jrgen Ahlberg <ahlberg@isy.liu.se>
//         Dept. of Electrical Engineering
//         Linkping University
//         SE-581 83 Linkping, SWEDEN
// Date:   1998-01-12
//
// This program is distributed in the hope that it will
// be useful, but WITHOUT ANY WARRANTY.
//
// =====================================================


#include "fit.hh"


// =====================================================

float FIT_Function::
evaluate(FIT_Node& parent, FAPs& faps, int frame) {
  float n, d;
  //cerr << "Num.poly: ";
  n = numerator.evaluate(parent,faps,frame);
  //cerr << "Den.poly: ";
  d = denominator.evaluate(parent,faps,frame);
  if(d==0.0) d=1.0;
  //cerr << "Function: "<<n<<"/"<<d<<" = "<<n/d<<"\n";
  return n/d;
}

// =====================================================

float FIT_Polynomial::
evaluate(FIT_Node& parent, FAPs& faps, int frame) {
  if(nterms==0) {
    //cerr << "nil\n";
    return 0.0;
  } else {
    float tmpf = 0.0;
    //cerr<<"(1/"<<denominator<<")*(";
    for(int termi=0;termi<nterms;termi++) {
      tmpf += terms[termi].evaluate(parent,faps,frame);
      //if(termi != nterms-1) cerr<<" + ";
    }
    //cerr<<")\n";
    return tmpf/float(denominator);
  }
}

// =====================================================

float FIT_Term::
evaluate(FIT_Node& parent, FAPs& Faps, int frame) {
  float tmpf = 0.0;
  //if(numerator>0) cerr<<numerator<<"*";
  //else cerr<<"("<<numerator<<")*";
  for(int fapi=0;fapi<nfaps;fapi++) {
    tmpf += faps[fapi].evaluate(parent,Faps,frame);
    //if(fapi!=nfaps-1) cerr<<"*"; 
  }
  return tmpf*float(numerator);
}

// =====================================================

float FIT_FAP::
evaluate(FIT_Node& parent, FAPs& faps, int frame) {
  float fap, exponent;
  fap = faps.get(frame, faps.getindex(parent.faps[index]));
  exponent = float(numerator)/float(denominator);
  //cerr<<fap;
  //if(!(numerator==1 && denominator==1))
  //if(denominator==1) cerr<<"^"<<numerator;
  //else cerr<<"^("<<numerator<<"/"<<denominator;
  return float(pow(fap, exponent));
}


