// =====================================================
//
// File:   fit_io.cc
// Class:  FIT (several classes)
//         Input/output-routines.
// 
// Author: Jrgen Ahlberg <ahlberg@isy.liu.se>
//         Dept. of Electrical Engineering
//         Linkping University
//         SE-581 83 Linkping, SWEDEN
// Date:   1998-01-12
//
// This program is distributed in the hope that it will
// be useful, but WITHOUT ANY WARRANTY.
//
// =====================================================


#include "fit.hh"


// =====================================================
//
// connect
// Make connection between linklist and nodelist
// (Handy in the interpoaltion algorithm).
//
// =====================================================

void FIT::
connect(void) {
  for(int link=0;link<linklist.nlinks;link++)
    nodelist.nodes[linklist.links[link].toNode].connect(linklist,link);
}

// =====================================================

void FIT_Node::
connect(FIT_LinkList& linklist, int link) {

  int *newuplinks, oldlink, i;
  newuplinks = new int[nuplinks+1];
  oldlink=0;
  for(i=0;i<nuplinks+1;i++) {
    if(oldlink < nuplinks) {
      if(linklist.links[link].priority <
	 linklist.links[uplinks[oldlink]].priority) {
	newuplinks[i]=link;
      } else {
	newuplinks[i]=uplinks[oldlink++];
      }
    } else {
      newuplinks[i]=link;
    }
  }
  if(uplinks) delete uplinks;
  uplinks = newuplinks;
  nuplinks++;
}


// =====================================================
//
// FIT Nodes I/O
//
// =====================================================


istream& operator >> (istream& is, FIT_Node& n) {
  is >> n.nfaps;
  n.faps = new int[n.nfaps];
  for(int i=0;i<n.nfaps;i++) is >> n.faps[i];
}
ostream& operator << (ostream& os, FIT_Node& n) {
  os << n.nfaps << " ";
  for(int i=0;i<n.nfaps;i++)os<<n.faps[i]<<" ";
}

// =====================================================

istream& operator >> (istream& is, FIT_NodeList& nl) {
    is >> nl.nnodes;
    nl.nodes = new FIT_Node[nl.nnodes];
    for(int i=0;i<nl.nnodes;i++) is>>nl.nodes[i];
  }
ostream& operator << (ostream& os, FIT_NodeList& nl) {
  os << nl.nnodes << "\n\n";
  for(int i=0;i<nl.nnodes;i++)os<<nl.nodes[i]<<"\n";
}


// =====================================================
//
// FIT Polynomials I/O
//
// =====================================================


istream& operator >> (istream& is, FIT_FAP& f) {
  is >> f.index >> f.numerator >> f.denominator;
}
ostream& operator << (ostream& os, FIT_FAP& f) {
  os << f.index <<" "<< f.numerator <<" "<< f.denominator;
}

// =====================================================

istream& operator >> (istream& is, FIT_Term& t) {
  int sgn;
  is >> sgn >> t.numerator;
  if(sgn) t.numerator = -t.numerator;
  is >> t.nfaps;
  t.faps = new FIT_FAP[t.nfaps];
  for(int i=0;i<t.nfaps;i++) {
    is >> t.faps[i];
  }
}
ostream& operator << (ostream& os, FIT_Term& t) {
  if(t.numerator>0)os<<"0 ";else {os<<"1 ";t.numerator=-t.numerator;}
  os<<t.numerator<<" "<<t.nfaps<<"  ";
  for(int i=0;i<t.nfaps;i++)os<<t.faps[i]<<"  ";
}

// =====================================================

istream& operator >> (istream& is, FIT_Polynomial& p) {
  is >> p.nterms;
  if(p.nterms > 0) {
    is >> p.denominator;
    p.terms = new FIT_Term[p.nterms];
    for(int i=0;i<p.nterms;i++) {
      is >> p.terms[i];
    }
  }
}
ostream& operator << (ostream& os, FIT_Polynomial& p) {
  os<<p.nterms<<"  ";
  if(p.nterms>0) {
    os<<p.denominator<<"  ";
    for(int i=0;i<p.nterms;i++)os<<p.terms[i]<<" ";
  }
}

// =====================================================

istream& operator >> (istream& is, FIT_Function& f) {
  is >> f.numerator >> f.denominator;
}
ostream& operator << (ostream& os, FIT_Function& f) {
  os << f.numerator <<"\n"<< f.denominator;
}
  

// =====================================================
//
// FIT Links I/O
//
// =====================================================


void FIT_Link::
read(FIT_NodeList& nodelist, istream& is) {
  is >> fromNode >> toNode >> priority;
  nfcns = nodelist.nodes[toNode].nfaps;
  fcns = new FIT_Function[nfcns];
  for(int i=0;i<nfcns;i++) {
    is >> fcns[i];
  }
}
ostream& operator << (ostream& os, FIT_Link& l) {
    os<<l.fromNode<<" "<<l.toNode<<" "<<l.priority<<"\n";
    for(int i=0;i<l.nfcns;i++)os<<l.fcns[i]<<"\n";
}
  
// =====================================================

void FIT_LinkList::
read(FIT_NodeList& nodelist, istream& is) {
  is >> nlinks;
  links = new FIT_Link[nlinks];
  for(int i=0;i<nlinks;i++) links[i].read(nodelist, is);
}
ostream& operator << (ostream& os, FIT_LinkList& ll) {
  os << ll.nlinks << "\n\n";
  for(int i=0;i<ll.nlinks;i++)os<<ll.links[i]<<"\n";
}


// =====================================================
//
// FIT Superclass I/O
//
// =====================================================


istream& operator >> (istream& is, FIT& f) {
  cin >> f.nodelist;
  f.linklist.read(f.nodelist,cin);
  f.connect();
}
ostream& operator << (ostream& os, FIT& f) {
  os << f.nodelist << "\n"  << f.linklist;
}
