// =====================================================
//
// File:   matrix_io.cc
// Class:  Matrix
//         Routines for I/O.
// 
// Author: Jrgen Ahlberg <ahlberg@isy.liu.se>
//         Dept. of Electrical Engineering
//         Linkping University
//         SE-581 83 Linkping, SWEDEN
// Date:   1998-01-12
//
// This program is distributed in the hope that it will
// be useful, but WITHOUT ANY WARRANTY.
//
// =====================================================


#include "matrix.hh"


// =====================================================
//
// read_ascii
//
// =====================================================

int Matrix::
read_ascii(char* filename) {

  FILE *fp;

  if((fp = fopen(filename, "r")) == 0) {
    fprintf(stderr, "Class Matrix: File %s cannot be opened!\n", filename);
    exit(0);
  }
  read_ascii(fp);
  fclose(fp);
}

// =====================================================

int Matrix::
read_ascii(FILE* fp) {

  float tmpf;
  int i;

  if(nrows == 0 || ncols == 0) {
    fprintf(stderr, "Class Matrix: Unknown matrix size! (nrows == %d, ncols == %d)\n",nrows,ncols);
    exit(0);
  }
  if(!data) {
    fprintf(stderr, "Class Matrix: No space allocated for data!\n");
    exit(0);
  }

  for(i = 0; i < nrows*ncols; i++) {
    fscanf(fp, "%f", &tmpf);
    data[i] = tmpf;
  }
}

// =====================================================

float Matrix::
read_element_ascii(FILE* fp, int r, int c) {
  float tmpf;
  int tmpi;
  fscanf(fp,"%f ",&tmpf);
  return set(r,c,tmpf);
}

// =====================================================

int Matrix::
readln_ascii(FILE* fp, int r) {
  for(int c=0;c<ncols;c++)
    read_element_ascii(fp, r, c);
}


// =====================================================
//
// write_ascii
//
// =====================================================

void Matrix::
writerow_ascii(FILE *fp, int row) {
  for(int col = 0; col < ncols; col++)
    fprintf(fp, "%f ", data[row*ncols + col]);
  fprintf(fp,"\n");
}

// =====================================================

void Matrix::
write_ascii(FILE *fp) {
  for(int i=0;i<nrows;i++)
    writerow_ascii(fp,i);
}

// =====================================================

void Matrix::
writerow_int_ascii(FILE *fp, int row) {
  int tmpi;
  for(int col = 0; col < ncols; col++) {
    tmpi = (int)round(data[row*ncols + col]);
    fprintf(fp, "%d ", tmpi);
  }
  fprintf(fp,"\n");
}

// =====================================================

void Matrix::
write_int_ascii(FILE *fp) {
  for(int i=0;i<nrows;i++)
    writerow_int_ascii(fp,i);
}


// =====================================================
//
// Report
//
// =====================================================

void Matrix::
report(void) {
  printf("Matrix %dx%d\n",nrows,ncols);
}

