// =====================================================
//
// File:   mask.hh
// Class:  Mask
// Uses:   ibstream, obstream
// 
// Author: Jrgen Ahlberg <ahlberg@isy.liu.se>
//         Dept. of Electrical Engineering
//         Linkping University
//         SE-581 83 Linkping, SWEDEN
// Date:   1998-01-12
//
// This program is distributed in the hope that it will
// be useful, but WITHOUT ANY WARRANTY.
//
// =====================================================


#ifndef MASK_HH
#define MASK_HH

#include "../codec.hh"
#include "../stream/ibstream.hh"
#include "../stream/obstream.hh"

#include <stdio.h>
#include <stdlib.h>


// =====================================================


class Mask {

private:

  int _size;
  int _sum;
  byte *_data;

public:

  // Constructors

  Mask(void) {_size=0;_sum=-1;_data=0;}
  Mask(int s) {_size=0;_sum=-1;_data=new byte[s];}

  // Primitives

  inline Mask& resize(int s) {return renew(s, new byte[s]);}
  inline Mask& renew(int s, byte *d) {_size=s; if(_data) delete _data;_data=d; _sum=-1; return *this;}
  inline int size(void) {return(_size);}
  inline int sum(void) {return _sum==-1?calcsum():_sum;}
  inline byte bit(int i) {return(_data[i]);}
  inline byte set(int i, byte b) {return(_data[i]=b);}
  inline int set(int i, int b) {return (int)set(i,(byte)b);}
  inline byte set(int i) {return set(i,1);}
  inline void set_all(void) {for(int i=0;i<_size;i++)set(i);}
  inline byte reset(int i) {return set(i,0);}
  inline int length(void) {return _size;}

  // Copy

  Mask& copy(Mask&);

  // Input & output

  int read_ascii(char*);
  int read_ascii(FILE*);
  int write_ascii(char*);
  int write_ascii(FILE*);

  int read_bitstream(ibstream&);
  int write_bitstream(obstream&);

private:

  int calcsum(void);

};

#endif
