// =====================================================
//
// File:   sequence.hh
// Class:  Sequence
// Uses:   Matrix, Mask, FileInfo, DCT
// 
// Author: Jrgen Ahlberg <ahlberg@isy.liu.se>
//         Dept. of Electrical Engineering
//         Linkping University
//         SE-581 83 Linkping, SWEDEN
// Date:   1998-01-12
//
// This program is distributed in the hope that it will
// be useful, but WITHOUT ANY WARRANTY.
//
// =====================================================


#ifndef SEQUENCE_HH
#define SEQUENCE_HH

#include "../codec.hh"
#include "../matrix/matrix.hh"
#include "../sequence/mask.hh"
#include "../sequence/fileinfo.hh"
#include "../dct/dct.hh"


// =====================================================


class Sequence : public Matrix {

protected:

  // Attributes

  Mask mask;
  FileInfo fileinfo;
  byte quant;
  byte coding_method;

public:

  // Constructors

  Sequence(void);
  Sequence(int, int);
  Sequence(int, Mask&);
  Sequence(Mask&, FileInfo&);

  // Public primitives

  inline Sequence& resize(int r, int c) {
    Matrix::resize(r, c); return *this;
  }

  inline Mask& getMask(void) {return mask;}
  inline Mask& setMask(Mask& src) {return mask.copy(src);}

  inline FileInfo& setInfo(FileInfo& src) {return fileinfo.copy(src);}
  inline FileInfo& getInfo(void) {return fileinfo;}

  inline float version(void) {return fileinfo.version;}
  inline char* basename(void) {return fileinfo.basename;}
  inline int framerate(void) {return fileinfo.framerate;}
  inline int nframes(void) {return fileinfo.nframes;}

  int getindex(int);

  void remove_params(Mask&);
  void insert(Sequence&);

  // Quantization

  float inline qstep(void) {return QP/float(quant);}

  // Coding

  void encode(byte, byte);
  void decode(void);
private:
  float* construct_quantvector(float*, float);
  void encode_predictive(float*, float*, int);
  void decode_predictive(float*, float*, int);
  void encode_dct(float*);
  void decode_dct(float*);
public:
  
  // Input

  void nextValidLine(FILE*);

  // Output

  void report(void);

};

#endif



