
/*------------------------------------------------------------------------------*/
/*										*/
/*										*/
/*  This software module was originally developed by                            */
/*        Prem Kalra, MIRALab, University of Geneva                             */
/*  and edited by                                                               */
/*        Igor Pandzic, MIRALab, University of Geneva                           */
/*        Marc Escher, MIRALab, University of Geneva                            */
/*        Pierre Beylot, MIRALab, University of Geneva                          */
/*  in the course of development of the MPEG-4 Video (ISO/IEC 14496-2).	        */
/*  This software module is an implementation of a part of one or more	        */
/*  MPEG-4 Video (ISO/IEC 14496-2) tools as specified by the MPEG-4 Video       */
/*  (ISO/IEC 14496-2). ISO/IEC gives users of the                               */
/*  MPEG-4 Video (ISO/IEC 14496-2) free license to this software module or	*/
/*  modifications thereof for use in hardware or software products claiming 	*/
/*  conformance to the MPEG-4 Video (ISO/IEC 14496-2). Those intending to use	*/
/*  this software module in hardware or software products are advised that	*/
/*  its use may infringe existing patents. The original developer of this	*/
/*  software module and his/her company, the subsequent editors and their	*/
/*  companies, and ISO/IEC have no liability for use of this software		*/
/*  module or modifications thereof. Copyright is not released for non		*/
/*  MPEG-4 Video (ISO/IEC 14496-2) conforming products.                         */
/*  MIRALab, University of Geneva retains full right to use the code for its    */
/*  own purpose, assign or donate the code to a third party and to inhibit      */
/*  third parties from using the code for non MPEG-4 Video (ISO/IEC 14496-2)    */
/*  conforming products. This copyright notice must be included in all copies   */
/*  or derivative works. Copyright (c) (1997).			      		*/
/*										*/
/*										*/
/*------------------------------------------------------------------------------*/


#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include <gl.h>
#include <device.h>
#include <math.h>
#include <time.h>
#include <sys/time.h>
#include "SurfMan.h"
#include "matrix_h.h"

#define MH3D_EPSILON        1e-6


#include <signal.h>
/*
----------------------------  Lights and camera  --------------------------
*/
/*
**  Static camera position and perspective projection.
*/


static float CAM_EYE_X = 0.0;	
static float CAM_EYE_Y = 5.0;
static float CAM_EYE_Z = 80.0;
static float CAM_INT_X = 0.0;
static float CAM_INT_Y = 0.0;
static float CAM_INT_Z = 0.0;
static float CAM_TWIST = 0;
static float CAM_FOV   = 450;
static float CAM_DEPTH = 500.0;

int WINX = 500;
int WINY = 400;
/*
**  Global viewing matrix and its inverse matrix.
*/
MATRIX_H3D V_matrix, V_inv_matrix;

/*
**  GL light properties.
*/
#define LIGHT_VIEWER_SIZE	14
float light_prop_viewer[LIGHT_VIEWER_SIZE] = {
    POSITION,	0.0, 0.0, 0.0, 0.0,
    LCOLOR,	1.0, 1.0, 1.0,
    AMBIENT,	0.1, 0.0, 0.0,
    LMNULL
};

#define LIGHT_SPHERE_SIZE	10
float light_prop_sphere[LIGHT_SPHERE_SIZE] = {
    LCOLOR,	0.8, 0.8, 0.8,
    POSITION,	0.0, 0.0, 0.1, 0.0,
    LMNULL
};

VECTOR_H3D light_direction = {
    50.0, 50.0, 140.0, 1.0,
};

void
MH3D_gl_lookat(vx,vy,vz,px,py,pz,twist,matrix)
Coord vx,vy,vz,px,py,pz;
Angle twist;
MATRIX_H3D matrix;
{
    MATRIX_H3D applic;
    float pvx,pvy,pvz;
    float sin_roty,cos_roty,sin_rotx,cos_rotx,rxz,rxyz,twist_rad;

    twist_rad = (float) (-twist)/ 1800.0 * M_PI;
    pvx = (float) ((px-vx) * (px-vx));
    pvy = (float) ((py-vy) * (py-vy));
    pvz = (float) ((pz-vz) * (pz-vz));
    rxz = fsqrt(pvx+pvz);
    rxyz = fsqrt(pvx+pvy+pvz);

    MH3D_INIT(matrix,
		1.0, 0.0, 0.0, 0.0,
		0.0, 1.0, 0.0, 0.0,
		0.0, 0.0, 1.0, 0.0,
		(float) - vx, (float) - vy, (float) - vz, 1.0);

    if (rxz >= MH3D_EPSILON) {
	sin_roty = (float) (px-vx)/ rxz;
	cos_roty = (float) (vz-pz)/ rxz;
	MH3D_INIT(applic,
		cos_roty, 0.0, -sin_roty, 0.0,
		0.0, 1.0, 0.0, 0.0,
		sin_roty, 0.0, cos_roty, 0.0,
		0.0, 0.0, 0.0, 1.0);
	MH3D_mul(matrix,matrix,applic);
    }

    if (rxyz >= MH3D_EPSILON) {
	sin_rotx = (float) (vy-py)/ rxyz;
	cos_rotx = rxz/ rxyz;
	MH3D_INIT(applic,
		1.0, 0.0, 0.0, 0.0,
		0.0, cos_rotx, sin_rotx, 0.0,
		0.0, -sin_rotx, cos_rotx, 0.0,
		0.0, 0.0, 0.0, 1.0);
	MH3D_mul(matrix,matrix,applic);
    }

    MH3D_INIT(applic,
	    (float) cos(twist_rad), (float) sin(twist_rad), 0.0, 0.0,
	    (float) -sin(twist_rad), (float) cos(twist_rad), 0.0, 0.0,
	    0.0, 0.0, 1.0, 0.0,
	    0.0, 0.0, 0.0, 1.0);
    MH3D_mul(matrix,matrix,applic);
}

void
MH3D_gl_perspective(fov,aspect,near,far,matrix)
Angle fov;
float aspect;
Coord near,far;
MATRIX_H3D matrix;
{
    float cot;

    cot = 1.0/ ftan( (float) fov/3600.0 * M_PI);
    MH3D_INIT(matrix,
	    cot/aspect, 0.0, 0.0, 0.0,
	    0.0, cot, 0.0, 0.0,
	    0.0, 0.0, -(far + near)/(far - near), -1.0,
	    0.0, 0.0, -(2.0 * far * near)/(far - near), 0.0);
}

void
MH3D_gl_scale(Sx, Sy, Sz, matrix)
float Sx, Sy, Sz;
MATRIX_H3D matrix;
{
    MH3D_INIT(matrix,
		 Sx, 0.0, 0.0, 0.0,
		0.0,  Sy, 0.0, 0.0,
		0.0, 0.0,  Sz, 0.0,
		0.0, 0.0, 0.0, 1.0);
}

void
MH3D_gl_rot(theta, axis, matrix)
float theta;
char axis;
MATRIX_H3D matrix;
{
    float sin_theta, cos_theta;

    theta *= M_PI / 180.0;
    sin_theta = fsin(theta);
    cos_theta = fcos(theta);
    switch (axis) {
	case 'x':
	case 'X':
	    MH3D_INIT(matrix,
		    1.0, 0.0, 0.0, 0.0,
		    0.0, cos_theta, sin_theta, 0.0,
		    0.0, -sin_theta, cos_theta, 0.0,
		    0.0, 0.0, 0.0, 1.0);
	    break;

	case 'y':
	case 'Y':
	    MH3D_INIT(matrix,
		    cos_theta, 0.0, -sin_theta, 0.0,
		    0.0, 1.0, 0.0, 0.0,
		    sin_theta, 0.0, cos_theta, 0.0,
		    0.0, 0.0, 0.0, 1.0);
	    break;

	case 'z':
	case 'Z':
	    MH3D_INIT(matrix,
		    cos_theta, sin_theta, 0.0, 0.0,
		    -sin_theta, cos_theta, 0.0, 0.0,
		    0.0, 0.0, 1.0, 0.0,
		    0.0, 0.0, 0.0, 1.0);
	    break;

	default: 
	    MH3D_gl_scale(1.0, 1.0, 1.0, matrix);
	    break;
    }
}

SM_material_gl_lmbind(mat_index)
int mat_index;
/*
**  First, if this material has changed or has not been lmdef (gldef flag), it
**  does it.  Second, it lmbinds it as the current gl material.
*/
{
    SM_Mat_ptr the_prop;

    the_prop = &the_materials.mat_list[mat_index];
    if (!the_prop->gldef) {
	lmdef(DEFMATERIAL, mat_index + 1, SM_MAT_LIST_LENGTH, the_prop->gl_properties);
	the_prop->gldef = YES;
	the_materials.glbind = SM_INDEX_ERROR;
    }

    /*
    **	Issues an lmbind command if mat_index has changed.
    */
    if (the_materials.glbind != mat_index) {
	lmbind(MATERIAL, mat_index + 1);
	the_materials.glbind = mat_index;
    }
}



void SM_surface_quick_draw(the_surface)
SM_Surface_ptr the_surface;
/*
**  Optimisation for all shading:  To avoid costly material binding, we loop first on
**  materials to draw all triangles with the same material.
*/
{
    SM_Vertex_ptr vert_list;
    static VECTOR_3D center, normal, normal1, normal2, normal3;
    int tri_index;
    BOOL transparent_mat;
    SM_Mat_ptr the_material;
    int current_material_index,previous_material_index;

    /*
    **	Make sure that the materials will be "cpack"ed or "lmbind"ed at least
    **	once if needed during this redraw.
    */
    the_materials.glbind = -2;
    the_materials.glcpack = -2;

    vert_list = the_surface->vert_list;

  
            previous_material_index = -5;

	    SM_FOR_EACH_TRIANGLE(the_surface)
                current_material_index = triangle_ptr->material_index;
                the_material = &the_materials.mat_list[current_material_index];
                if (current_material_index != previous_material_index) { 
		 SM_material_gl_lmbind(current_material_index);
                 previous_material_index = current_material_index;
                }
		if ( (the_material->gl_properties[SM_ALPHA + 1] >= 1.0)  &&
			SM_vertex_normal_sm_get(the_surface, 
					 triangle_ptr->vertex1,
					 triangle_ptr->normal1,
					 normal1) &&
			SM_vertex_normal_sm_get(the_surface, 
					 triangle_ptr->vertex2,	    
					 triangle_ptr->normal2,
					 normal2) &&
			SM_vertex_normal_sm_get(the_surface, 
					 triangle_ptr->vertex3, 
					 triangle_ptr->normal3,
					 normal3)) {
			bgnpolygon();
			n3f(normal1);
			v3f(vert_list[triangle_ptr->vertex1].point);
			n3f(normal2);
			v3f(vert_list[triangle_ptr->vertex2].point);
			n3f(normal3);
			v3f(vert_list[triangle_ptr->vertex3].point);
			endpolygon();
		 }
	    SM_END_FOR_EACH


}


/*
----------------------------------  Viewers  --------------------------------
*/
#define MAX_VIEWERS 4

typedef struct {
    BOOL struct_used, freeze, references[2];

    /*	Viewing and drawing. */
    long window_id;
    Screencoord the_viewport[4];
    long window_X_org, window_Y_org;
    unsigned long Drawing_bits[3];
    int current_disp_prim;

    /*
    **	Current transformation matrices for this window.
    **	Transform sequence is MNVP where,
    **	M - Model matrix: relates the object reletive to node coordinates.
    **	N - Node matrix: moved with spaceball.  Relates center of rotation
    **			 and orientation in camera coordinates.
    **	VP - Viewing and projection matrices.
    */
    MATRIX_H3D M_matrix, N_matrix, MN_matrix, MN_inv_matrix, 
	       VP_matrix, VP_inv_matrix, 
	       MNVP_matrix, MNVP_inv_matrix;

    /*	Toolkit objects.  */
    long theWindow;
} Viewer, * Viewer_ptr;

Viewer viewers[MAX_VIEWERS];

#define SM_FOR_EACH_VIEWER \
    { \
	Viewer_ptr viewer_ptr; \
	register int viewer_index; \
	for (	viewer_index = 0, \
		viewer_ptr = viewers; \
		viewer_index < MAX_VIEWERS; \
		viewer_index++, \
		viewer_ptr++	) \
	    if (viewer_ptr->struct_used) { 


/*
**  A global variable that always points to the current viewer.
*/
Viewer_ptr current_viewer;

/* long window_background_color = 0xffff3333; */
 long window_background_color = 0xff000000; 


long line_color = 0xffff7777;

static BOOL is_shade = YES;

/*
----------------------------------  Global variables  ---------------------------------
*/
#define CLOCK_TICKS_SEC 5

#define LONGAXE 25.0
#define AXIS_Z_COLOR 0xff55bbcc
#define AXIS_X_COLOR 0xffbbcc55
#define AXIS_Y_COLOR 0xffcc55bb

/*
*   origine vector and identity matrix.
*/
VECTOR_3D origine = { 0.0, 0.0, 0.0};
MATRIX_H3D I_matrix;


BOOL lm_bind;
/*
-------------------------------  OBJECT DISPLAY  ---------------------------------
*/

#define GRID_XMIN   -400.0
#define GRID_XMAX   400.0
#define GRID_ZMIN   -CAM_DEPTH + 5.0
#define GRID_ZMAX   CAM_DEPTH / 2.0
#define GRID_Y	    -50.0
#define GRID_PREC   10.0
#define SHADOW_PLANE_Y		GRID_Y
#define SHADOW_PLANE_COLOR	0xff907070
#define SHADOW_PRIM_COLOR	0xff111111
#define PICKING_REGION_COLOR	0x994285ff
#define SB_STATUS_LINE_COLOR	0xff8888ff


/*--------------------------------------------------*/

/* donne le temps actuel en secondes */
double donne_temps()
{
    struct timeval temps_s;         /*structure de saisie du temps */
    struct timezone temp_zone;
    double temp_sec,temp_usec;

    BSDgettimeofday(&temps_s,&temp_zone);

    temp_sec=(double)(temps_s.tv_sec);
    temp_usec=(double)(temps_s.tv_usec);

    return(temp_sec+(temp_usec/(1e+6)));
}

/*--------------------------------------------------*/

draw_ref_axis(shadow)
BOOL shadow;
/*
**  Draws the node axis.
*/
{
    VECTOR_3D axe;

    if (shadow) cpack(SHADOW_PRIM_COLOR);
    else {
	cpack(AXIS_X_COLOR);
	linewidth(3);
    }
    bgnline(); v3f(origine); 
    V3D_COPY(axe,origine); axe[X_INDEX] = LONGAXE; v3f(axe); endline();

    if (!shadow) cpack(AXIS_Y_COLOR);
    bgnline(); v3f(origine); 
    V3D_COPY(axe,origine); axe[Y_INDEX] = LONGAXE; v3f(axe); endline();

    if (!shadow) cpack(AXIS_Z_COLOR);
    bgnline(); v3f(origine); 
    V3D_COPY(axe,origine); axe[Z_INDEX] = LONGAXE; v3f(axe); endline();

    linewidth(1);
}


draw_ref_grid()
/*
**  Draws a grid at the same height as the bottom shadowing plane.
*/
{
    float x, z;
    VECTOR_3D v;

    cpack(0xff444450);
    for (x = GRID_XMIN; x <= GRID_XMAX; x += (GRID_XMAX - GRID_XMIN) / GRID_PREC) {
	bgnline();
	V3D_INIT(v, x, GRID_Y, GRID_ZMIN);
	v3f(v);
	v[Z_INDEX] = GRID_ZMAX;
	v3f(v);
	endline();
    }
    for (z = GRID_ZMIN; z <= GRID_ZMAX; z += (GRID_ZMAX - GRID_ZMIN) / GRID_PREC) {
	bgnline();
	V3D_INIT(v, GRID_XMIN, GRID_Y, z);
	v3f(v);
	v[X_INDEX] = GRID_XMAX;
	v3f(v);
	endline();
    }
}

/*
----------------------------------   VIEWERS   --------------------------------
*/

viewer_initialise()
/*
**  Initialises the data structures for the viewers.
*/
{
    int i;
    for (i = 0; i < MAX_VIEWERS; i++) viewers[i].struct_used = NO;
}


viewer_object_projection_set(viewer_ptr)
Viewer_ptr viewer_ptr;
/*
**  Loads GE with object projection and viewing matrices for "viewer_ptr".
*/
{
    viewport( viewer_ptr->the_viewport[0], 
	      viewer_ptr->the_viewport[1], 
	      viewer_ptr->the_viewport[2], 
	      viewer_ptr->the_viewport[3] );
    perspective( CAM_FOV,
		 (viewer_ptr->the_viewport[1] -
		 viewer_ptr->the_viewport[0]) /
		(float) (viewer_ptr->the_viewport[3] -
		 viewer_ptr->the_viewport[2]), 1.0, CAM_DEPTH );

    loadmatrix(I_matrix);
    lookat( CAM_EYE_X, CAM_EYE_Y, CAM_EYE_Z, CAM_INT_X, CAM_INT_Y, CAM_INT_Z, CAM_TWIST);
    lmbind(LIGHT0, 1);
}


viewer_text_projection_set(viewer_ptr)
Viewer_ptr viewer_ptr;
/*
**  Loads GE with text projection matrices for "viewer_ptr".
*/
{
    viewport( viewer_ptr->the_viewport[0], 
	      viewer_ptr->the_viewport[1], 
	      viewer_ptr->the_viewport[2], 
	      viewer_ptr->the_viewport[3] );
    ortho2(   viewer_ptr->the_viewport[0], 
	      viewer_ptr->the_viewport[1], 
	      viewer_ptr->the_viewport[2], 
	      viewer_ptr->the_viewport[3] );
    loadmatrix(I_matrix);
}


viewer_redraw(viewer_ptr,the_surface)
Viewer_ptr viewer_ptr;
SM_Surface_ptr the_surface;
/*
**  Redraws the "viewer_ptr".  Takes care of shadows and references when
**  they are wanted.
*/
{
    VECTOR_3D v;
    int x, y;
    /*
    extern char *sbStatus();
    */
    VECTOR_3D point;

    VECTOR_3D p1,p2,p3;
    int v1,v2,v3;

/*
    winset(viewer_ptr->window_id);
*/       
    czclear(window_background_color,0);

    viewer_object_projection_set(viewer_ptr);
  
   /*
    **	Draws the object normally with its references guides.
    */

    SM_surface_quick_draw(the_surface);
    swapbuffers();
}



viewer_compute_matrices(viewer_ptr)
Viewer_ptr viewer_ptr;
{
    MH3D_invert(viewer_ptr->VP_inv_matrix, 
		MH3D_determinant(viewer_ptr->VP_matrix), 
		viewer_ptr->VP_matrix);   
    MH3D_mul(viewer_ptr->MN_matrix, viewer_ptr->M_matrix, viewer_ptr->N_matrix);
    MH3D_invert(viewer_ptr->MN_inv_matrix, 
		MH3D_determinant(viewer_ptr->MN_matrix),
		viewer_ptr->MN_matrix);
    MH3D_mul(viewer_ptr->MNVP_matrix, viewer_ptr->MN_matrix, viewer_ptr->VP_matrix);
    MH3D_invert(viewer_ptr->MNVP_inv_matrix,
		MH3D_determinant(viewer_ptr->MNVP_matrix),
		viewer_ptr->MNVP_matrix);
}


viewer_moved_reshaped(viewer_ptr)
Viewer_ptr viewer_ptr;
/*
**  Updates the internal matrices, window size and viewport stuff, when a viewer is
**  moved or reshaped.
*/
{
    MATRIX_H3D P_matrix;
    long win_size_x, win_size_y;

    winset(viewer_ptr->window_id);
    getorigin( &viewer_ptr->window_X_org, &viewer_ptr->window_Y_org);

    getsize( &win_size_x, &win_size_y );
    viewer_ptr->the_viewport[0] = 0;
    viewer_ptr->the_viewport[1] = win_size_x;
    viewer_ptr->the_viewport[2] = 0;
    viewer_ptr->the_viewport[3] = win_size_y;

    MH3D_gl_perspective( CAM_FOV, 
			 (viewer_ptr->the_viewport[1] -
			 viewer_ptr->the_viewport[0]) /
		 (float) (viewer_ptr->the_viewport[3] -
			 viewer_ptr->the_viewport[2]),
			 1.0, CAM_DEPTH, P_matrix);
    MH3D_mul(viewer_ptr->VP_matrix, V_matrix, P_matrix);
    viewer_compute_matrices(viewer_ptr);
}


BOOL
viewer_open(the_surface)
SM_Surface_ptr the_surface;
/*
**  Adds a new viewer.
**  Returns NO if there is no room for another.
*/
{
    Viewer_ptr viewer_ptr;
    register int viewer_index, i;

    MH3D_IDENTITY(I_matrix);
    MH3D_invert(V_inv_matrix, MH3D_determinant(V_matrix), V_matrix);

    /*
    **	Viewers stuff.
    */
    current_viewer = NULL;
    viewer_initialise();

    for (   viewer_index = 0, 
	    viewer_ptr = viewers; 
	    viewer_index < MAX_VIEWERS; 
	    viewer_index++, 
	    viewer_ptr++  ) 

	if (!viewer_ptr->struct_used) {
	    viewer_ptr->struct_used = YES;
	    SM_matrix_get(the_surface, viewer_ptr->M_matrix);
	    MH3D_gl_rot(-45.0, 'y', viewer_ptr->N_matrix);

	    MH3D_IDENTITY(viewer_ptr->N_matrix);
	    viewer_compute_matrices(viewer_ptr);

	    viewer_ptr->freeze = NO;

	    for (i = 0; i < 2; i++)
		viewer_ptr->references[i] = NO;
/*
	    viewer_ptr->references[0] = YES;
*/
/*
	    prefsize(XMAXPAL, YMAXPAL);
*/
	    foreground();
	    prefsize(WINX, WINY);
	    noborder();
	    viewer_ptr->theWindow = winopen("window");
	    /*
	    **	GL and others.
	    */
	    viewer_ptr->window_id = viewer_ptr->theWindow;
	    
	    RGBmode();
	    zbuffer(TRUE);
	    lsetdepth(0x7fffff,0);
	    zfunction(ZF_GEQUAL);
	    doublebuffer();
	    gconfig();
	    viewer_moved_reshaped(viewer_ptr);

	    /*
	    **	Lights.
	    */
	    light_initialize();
	
	    viewer_ptr->Drawing_bits[0] = SM_PICKED;
	    viewer_ptr->Drawing_bits[1] = SM_SHADED | SM_PICKED;
	    viewer_ptr->Drawing_bits[2] = SM_PICKED;
	    viewer_ptr->current_disp_prim = 0;
	    current_viewer = viewer_ptr;
	    return(YES);
    	}

    winset(viewer_ptr->window_id);
    viewer_object_projection_set(viewer_ptr); 

    return(NO);
}

viewer_close(viewer_ptr)
Viewer_ptr viewer_ptr;
/*
**  Closes (kills) the viewer window.
*/
{
    winclose(viewer_ptr->window_id);
    viewer_ptr->struct_used = NO;
}





/*
------------------------------------  LIGHTS  ------------------------------------
*/

light_initialize()
/*
**  Defines and binds the software's light.
*/
{
    VECTOR_3D position, shadow_P, shadow_N, shadow_L;

    V3D_INIT(shadow_P, 0.0, SHADOW_PLANE_Y, 0.0);
    V3D_INIT(shadow_N, 0.0, 1.0, 0.0);
    V3D_NORMALIZE(shadow_L, light_direction);
    mmode(MVIEWING);
    loadmatrix(I_matrix);
    lookat( CAM_EYE_X, CAM_EYE_Y, CAM_EYE_Z, CAM_INT_X, CAM_INT_Y, CAM_INT_Z, CAM_TWIST);
    V3D_FROM_VH3D(position, light_direction);
    light_prop_viewer[1] = position[X_INDEX];
    light_prop_viewer[2] = position[Y_INDEX];
    light_prop_viewer[3] = position[Z_INDEX];
    light_prop_viewer[4] = 0.0;
    lmdef(DEFLIGHT, 1, LIGHT_VIEWER_SIZE, light_prop_viewer);
    lmdef(DEFLIGHT, 2, LIGHT_SPHERE_SIZE, light_prop_sphere);
    lmbind(LIGHT0, 1);
    lmdef(DEFLMODEL, 2, 0, NULL);
    lmbind(LMODEL, 2);
}



viewer_redraw_all(the_surface)
SM_Surface_ptr the_surface;
/*
**  Redraws all the object in their viewers if they are not frozen.
*/
{
    SM_FOR_EACH_VIEWER
	viewer_redraw(viewer_ptr,the_surface);
    SM_END_FOR_EACH
}

