

/*------------------------------------------------------------------------------*/
/*										*/
/*										*/
/*  This software module was originally developed by                            */
/*        Eric Petajan, Lucent Technologies Inc.                                */
/*  and edited by                                                               */
/*        Igor Pandzic, MIRALab, University of Geneva                           */
/*  in the course of development of the MPEG-4 Video (ISO/IEC 14496-2).	        */
/*  This software module is an implementation of a part of one or more	        */
/*  MPEG-4 Video (ISO/IEC 14496-2) tools as specified by the MPEG-4 Video       */
/*  (ISO/IEC 14496-2). ISO/IEC gives users of the                               */
/*  MPEG-4 Video (ISO/IEC 14496-2) free license to this software module or	*/
/*  modifications thereof for use in hardware or software products claiming 	*/
/*  conformance to the MPEG-4 Video (ISO/IEC 14496-2). Those intending to use	*/
/*  this software module in hardware or software products are advised that	*/
/*  its use may infringe existing patents. The original developer of this	*/
/*  software module and his/her company, the subsequent editors and their	*/
/*  companies, and ISO/IEC have no liability for use of this software		*/
/*  module or modifications thereof. Copyright is not released for non		*/
/*  MPEG-4 Video (ISO/IEC 14496-2) conforming products.                         */
/*  Lucent Technologies Inc. retains full right to use the code for its         */
/*  own purpose, assign or donate the code to a third party and to inhibit      */
/*  third parties from using the code for non MPEG-4 Video (ISO/IEC 14496-2)    */
/*  conforming products. This copyright notice must be included in all copies   */
/*  or derivative works. Copyright (c) (1997).			      		*/
/*										*/
/*										*/
/*------------------------------------------------------------------------------*/

#include <stdlib.h>
#include <stdio.h>
#include <time.h>
#include <sys/time.h>
#include <sys/types.h>

#include "SurfMan.h"

typedef struct {
  int **fapmask;/*[MAXFRAMES][69] framenumber,fapnum, note: fapmask[n][0] is not used*/
  int **faps;/*[MAXFRAMES][69] 0th is framenum, not a fap*/
  float **lfaps;/*[MAXFRAMES][69] faps in local face scale*/
  int framecount,framerate;
  char *basename;/*[256]*/
  float version;
} FAPtype;

int skipped_frames = 0, total_frames = 0;


leftrightinterpolation(FAPtype *FAPs, int frameind, int startfapnum, int endfapnum) {
  int fapnum,i;
  int **fapmask;
  int **faps;

  fapmask = FAPs->fapmask;
  faps = FAPs->faps;

  for(fapnum=startfapnum;fapnum <= endfapnum;fapnum+=2)
    if(fapmask[frameind][fapnum] && (!fapmask[frameind][fapnum+1])) {
        faps[frameind][fapnum+1] = faps[frameind][fapnum];
        fapmask[frameind][fapnum+1] = 1;
    }
    else {
      if(fapmask[frameind][fapnum+1] && (!fapmask[frameind][fapnum])) {
          faps[frameind][fapnum] = faps[frameind][fapnum+1];
          fapmask[frameind][fapnum] = 1;
      }
    }
}


readfap(char *fapfilename, FAPtype *FAPs) 
{
  FILE *fapfile;
  char cbuf[1000];
  int fapnum,frameind,i;
  int **fapmask;
  int **faps;
  
  fapmask = FAPs->fapmask;
  faps = FAPs->faps;
  if(!(fapfile = fopen(fapfilename,"r"))) printf("ERROR: can't read %s\n",fapfilename);
  while(1) 
    {/*read header line*/
      if(fscanf(fapfile,"%s",cbuf));
      else {
	printf("ERROR: bad fapfile: %s\n",cbuf);
	exit(2);
      }
      if(cbuf[0] == '#') while(getc(fapfile) != '\n');
      else {
	sscanf(cbuf,"%f ",&(FAPs->version));
	fscanf(fapfile,"%s %d %d",FAPs->basename,&(FAPs->framerate),&(FAPs->framecount));
	if(fabs(FAPs->version - 2.1) > 0.001)
	  printf("WARNING: Expecting FAP file version 2.1 not %f\n",FAPs->version);
	while(getc(fapfile) != '\n');
	break;
      }
    }
  for(frameind=0;frameind<FAPs->framecount;frameind++) 
    {
      while(1) {
	if(fscanf(fapfile,"%s",cbuf)!= EOF);
	else {
	  printf("ERROR: bad fapfile: %s\n",cbuf);
	  exit(2);
	}
	if(cbuf[0] == '#') while(getc(fapfile) != '\n');
	else {
	  sscanf(cbuf,"%d",&fapmask[frameind][1]);
	  for(fapnum=2;fapnum<69;fapnum++) fscanf(fapfile,"%d",&fapmask[frameind][fapnum]);
	  while(getc(fapfile) != '\n');
	  break;
	}
      }
      while(1) {
	if(fscanf(fapfile,"%s",cbuf)!= EOF);
	else {
	  printf("ERROR: bad fapfile: %s\n",cbuf);
	  exit(2);
	}
	if(cbuf[0] == '#') while(getc(fapfile) != '\n');
	else {
	  sscanf(cbuf,"%d",&faps[frameind][0]);
	  for(fapnum=1;fapnum<69;fapnum++) if(fapmask[frameind][fapnum]) fscanf(fapfile,"%d",&faps[frameind][fapnum]);
	  while(getc(fapfile) != '\n');
	  break;
	}
      }
      leftrightinterpolation(FAPs,frameind, 6, 12);/*inner lips*/
      leftrightinterpolation(FAPs,frameind, 53, 61);/*outer lips,nose*/
      leftrightinterpolation(FAPs,frameind, 19, 41);/*eyelids,eyebrows,eyeballs,pupils,cheeks*/
      leftrightinterpolation(FAPs,frameind, 65, 67);/*ears*/
    }
}

applyfaps(int fr, FAPtype *FAPs)
{
  int faps[68],fap_mask[68];
  int i;
  struct timeval tod;
  static unsigned int pts0 = 0;
  unsigned int pts;
  static unsigned int t0 = 0;
  unsigned int t;
  int dt;

  static SM_Surface_ptr the_surface = 0, the_face;
  static SM_Region_List_ptr the_region_list;

  if(!the_surface)
    {
      init_face(&the_surface,&the_face,&the_region_list);
    }

  pts = (fr * 1000.0f)/FAPs->framerate;
  if(t0 == 0)
    {
      pts0 = pts;
      gettimeofday(&tod, (struct timezone *)0);
      t0 = tod.tv_sec * 1000 + tod.tv_usec/1000;
    }

  gettimeofday(&tod, (struct timezone *)0);
  t = tod.tv_sec * 1000 + tod.tv_usec/1000;

  dt = (pts - pts0) - (t-t0);

  total_frames++;

  if(dt < 0) 
    {
      skipped_frames++;
      return;
    }

  for(i=0;i<68;i++)
    {      
      fap_mask[i] = (int)FAPs->fapmask[fr][i+1];
      if(fap_mask[i])
	faps[i] = FAPs->faps[fr][i+1];
      else faps[i] = 0;
	
/*
      fprintf(stderr,"%d: %d, %d\n",i,fap_mask[i],faps[i]);
*/
    }

  run_faps(fap_mask,faps,the_surface,the_face,the_region_list);  

  /* If too fast, wait for PTS */
  do
    {
      gettimeofday(&tod, (struct timezone *)0);
      t = tod.tv_sec * 1000 + tod.tv_usec/1000;
      
      dt = (pts - pts0) - (t-t0);
    }
  while(dt > 0);
}

main(int argc, char *argv[]) 
{
  char *UserName,cbuf[256],cdummy[256];
  FILE *fapfile;
  FAPtype FAPs;
  int i,j, c, dummy,maxframes; 
  float version; 
  
  if(argc != 2) 
    {
      printf("Usage: miraface <fap filename>\n");
      exit(2);
    }
  if(!(fapfile = fopen(argv[1],"r"))) printf("ERROR: can't read %s\n",argv[1]);
  while(1) {/*just read header line for mallocs below*/
    if(fscanf(fapfile,"%s",cbuf));
    else {
      printf("ERROR: bad fapfile: %s\n",cbuf);
      exit(2);
    }
    if(cbuf[0] == '#') while(getc(fapfile) != '\n');
    else {
      sscanf(cbuf,"%f ",&version);
      fscanf(fapfile,"%s %d %d",cdummy,&dummy,&maxframes);
      if(fabs(version - 2.1) > 0.001)
	printf("WARNING: Expecting FAP file version 2.1 not %f\n",version);
      while(getc(fapfile) != '\n');
      break;
    }
  }
  fclose(fapfile);

  if(!(FAPs.fapmask = (int **)malloc(maxframes*sizeof(int *)))) {
    printf("ERROR: malloc failed on FAPs.fapmask\n");
    exit(2);
  }
  for(i=0;i<maxframes;i++)
    if(!(FAPs.fapmask[i] = (int *)malloc(69*sizeof(int )))) {
      printf("ERROR: malloc failed on FAPs.fapmask\n");
      exit(2);
    }
  if(!(FAPs.faps = (int **)malloc(maxframes*sizeof(int *)))) {
    printf("ERROR: malloc failed on FAPs.faps\n");
    exit(2);
  }
  for(i=0;i<maxframes;i++)
    if(!(FAPs.faps[i] = (int *)malloc(69*sizeof(int )))) {
      printf("ERROR: malloc failed on FAPs.faps\n");
      exit(2);
    }
  if(!(FAPs.lfaps = (float **)malloc(maxframes*sizeof(float *)))) {
    printf("ERROR: malloc failed on FAPs.lfaps\n");
    exit(2);
  }
  for(i=0;i<maxframes;i++)
    if(!(FAPs.lfaps[i] = (float *)malloc(69*sizeof(float )))) {
      printf("ERROR: malloc failed on FAPs.lfaps\n");
      exit(2);
    }
  if(!(FAPs.basename = (char *)malloc(256*sizeof(char )))) {
    printf("ERROR: malloc failed on FAPs.basename\n");
    exit(2);
  }

  readfap(argv[1],&FAPs);
  
  /* SYNCHRONIZATION WITH AUDIO */
  printf("STARTING ANIMATION\n");
  fflush(stdout);

  for(i=0;i<maxframes;i++)
    {
      applyfaps(i,&FAPs);
    }
  
  fprintf(stderr,"Real time performance: %d skipped frames out of total %d frames\n",skipped_frames,total_frames);
}

