/*************************************************************************

This software module was originally developed by

        Wei Wu (weiwu@stallion.risc.rockwell.com),  Rockwell Science Center

in the course of development of the MPEG-4 Video (ISO/IEC 14496-2).
This software module is an implementation of a part of one or more MPEG-4
Video tools as specified by the MPEG-4 Video. ISO/IEC gives users of the
MPEG-4 Video free license to this software module or modifications thereof
for use in hardware or software products claiming conformance to the MPEG-4
Video. Those intending to use this software module in hardware or software
products are advised that its use may infringe existing patents. The original
developer of this software module and his/her company, the subsequent editors
and their companies, and ISO/IEC have no liability for use of this software
module or modifications thereof in an implementation. Copyright is not
released for non MPEG-4 Video conforming products. Rockwell Science Center
retain full right to use the code for his/her own purpose, assign
or donate  the code to a third party and to inhibit third parties from using the
code  for non <MPEG standard> conforming products. This copyright notice must be

included in all copies or derivative works. Copyright (c) 1998.

*************************************************************************/


#include <math.h>
#include "sim.h"

#define SHORT_MIN	-32768
#define SHORT_MAX	32767
#define PI		3.14159265358979323846

static float DCTQuantSteps[NFAP] = { 1, 1, 7.5, 7.5, 7.5, 7.5, 7.5, 7.5, 7.5, 7.5,
                                     7.5, 7.5, 7.5, 15, 15, 15, 15, 5, 10,10,
                                     10, 10, 425, 425, 425, 425, 5, 5, 5, 5,
                                     7.5, 7.5, 7.5, 7.5, 7.5, 7.5, 7.5, 7.5, 20, 20,
                                     20, 20, 10, 10, 10, 10, 255, 170, 255, 255,
                                     7.5, 7.5, 7.5, 7.5, 7.5, 7.5, 7.5, 7.5, 7.5, 7.5,
                                     15, 15, 15, 15, 10, 10, 10, 10 };
static int fap_scale[32] = { 1, 1, 2, 3, 5, 7, 8, 10, 12, 15, 18, 21, 25, 30, 35, 42,
                      50, 60, 72, 87, 105, 128, 156, 191, 234, 288, 355, 439, 543,
                                          674, 836, 1039 };

static float bap_DCTQuantSteps[NUM_BAPS] = {64,64,64,128,128,128,128,256,256,128,128,256,256,
128,128,256,256,256,256,256,256,256,256,128,128,128,128,128,128,128,128,64,64,64,64,256,256,
64,64,256,256,128,128,128,128,256,256,128,128,128,256,256,256,128,128,128,256,256,256,128,128,128,256,256,256,256,256,256,256,256,256,
128,128,128,256,256,256,256,256,256,256,256,256,256,256,256,128,128,128,256,256,256,256,256,256,256,256,256,
128,128,128,256,256,256,256,256,256,128,128,128,256,256,256,128,128,128,256,256,256,128,128,128,512,512,128,128,128,128,256,256,
128,128,128,128,256,256,128,128,128,128,256,256,128,128,128,128,256,256,128,128,128,128,256,256,128,128,128,128,256,256,128,128,128,128,256,256,
128,128,128,128,128,128,128,128,256,256,128,128,128,128,64,64,64,64,64,64};

static int bap_scale[32] = { 1, 1, 2, 3, 5, 7, 8, 10, 12, 15, 18, 21, 25, 30, 35, 42,
                      50, 60, 72, 87, 105, 128, 156, 191, 234, 288, 355, 439, 543,
                                          674, 836, 1039 };

void
init_dctQP() {
  int i;
  int quant = fop.fo.quant;

  for (i=0; i<NFAP; i++) {
	fop.fo.dctQP[i] = fap_scale[quant]*DCTQuantSteps[i];
  }
}

void
init_htables() {
  int i, j, n, m;
  FILE *ifp;

  ifp = fopen("dc.table", "r");
  fscanf(ifp, "%d", &n);
  fop.fo.dc_table = (int**) malloc(n*sizeof(int*));
  for (i=0; i<n; i++) {
	fscanf(ifp, "%d", &m);
	fop.fo.dc_table[i] = (int*) malloc((m+1)*sizeof(int));
	fop.fo.dc_table[i][0] = m;
	for (j=1; j<=m; j++) {
		fscanf(ifp, "%d", &fop.fo.dc_table[i][j]);
	}
  }
  fclose(ifp);

  ifp = fopen("runlength.table", "r");
  fscanf(ifp, "%d", &n);
  fop.fo.runlength_table = (int**) malloc(n*sizeof(int*));
  for (i=0; i<n; i++) {
	fscanf(ifp, "%d", &m);
	fop.fo.runlength_table[i] = (int*) malloc((m+1)*sizeof(int));
	fop.fo.runlength_table[i][0] = m;
	for (j=1; j<=m; j++) {
		fscanf(ifp, "%d", &fop.fo.runlength_table[i][j]);
	}
  }
  fclose(ifp);

  ifp = fopen("ac.table", "r");
  fscanf(ifp, "%d", &n);
  fop.fo.ac_table = (int**) malloc(n*sizeof(int*));
  for (i=0; i<n; i++) {
	fscanf(ifp, "%d", &m);
	fop.fo.ac_table[i] = (int*) malloc((m+1)*sizeof(int));
	fop.fo.ac_table[i][0] = m;
	for (j=1; j<=m; j++) {
		fscanf(ifp, "%d", &fop.fo.ac_table[i][j]);
	}
  }
  fclose(ifp);

  ifp = fopen("v_sel.table", "r");
  fscanf(ifp, "%d", &n);
  fop.fo.v_sel_table = (int**) malloc(n*sizeof(int*));
  for (i=0; i<n; i++) {
	fscanf(ifp, "%d", &m);
	fop.fo.v_sel_table[i] = (int*) malloc((m+1)*sizeof(int));
	fop.fo.v_sel_table[i][0] = m;
	for (j=1; j<=m; j++) {
		fscanf(ifp, "%d", &fop.fo.v_sel_table[i][j]);
	}
  }
  fclose(ifp);

  ifp = fopen("v_blend.table", "r");
  fscanf(ifp, "%d", &n);
  fop.fo.v_blend_table = (int**) malloc(n*sizeof(int*));
  for (i=0; i<n; i++) {
	fscanf(ifp, "%d", &m);
	fop.fo.v_blend_table[i] = (int*) malloc((m+1)*sizeof(int));
	fop.fo.v_blend_table[i][0] = m;
	for (j=1; j<=m; j++) {
		fscanf(ifp, "%d", &fop.fo.v_blend_table[i][j]);
	}
  }
  fclose(ifp);

  ifp = fopen("e_sel.table", "r");
  fscanf(ifp, "%d", &n);
  fop.fo.e_sel_table = (int**) malloc(n*sizeof(int*));
  for (i=0; i<n; i++) {
	fscanf(ifp, "%d", &m);
	fop.fo.e_sel_table[i] = (int*) malloc((m+1)*sizeof(int));
	fop.fo.e_sel_table[i][0] = m;
	for (j=1; j<=m; j++) {
		fscanf(ifp, "%d", &fop.fo.e_sel_table[i][j]);
	}
  }
  fclose(ifp);

  ifp = fopen("e_intensity.table", "r");
  fscanf(ifp, "%d", &n);
  fop.fo.e_intensity_table = (int**) malloc(n*sizeof(int*));
  for (i=0; i<n; i++) {
	fscanf(ifp, "%d", &m);
	fop.fo.e_intensity_table[i] = (int*) malloc((m+1)*sizeof(int));
	fop.fo.e_intensity_table[i][0] = m;
	for (j=1; j<=m; j++) {
		fscanf(ifp, "%d", &fop.fo.e_intensity_table[i][j]);
	}
  }
  fclose(ifp);
}

void
init_dct() {
  int i,j;
  float c;
 
  for(i=0; i<SEG_LENGTH; i++) {
    if(i==0) c = (float)(1.0/sqrt(2));
    else c = 1;
    for(j=0; j<SEG_LENGTH; j++) {
	fop.fo.dctMatrix[i][j] = c * (float)(sqrt(2.0/SEG_LENGTH))
				* (float)(cos(PI*(float)(2*j+1)*i/2.0/SEG_LENGTH));
    }
  }

  for(i=0; i<SEG_LENGTH; i++)
    for(j=0; j<SEG_LENGTH; j++)
      	fop.fo.idctMatrix[i][j] = fop.fo.dctMatrix[j][i];

}

void
dctTrans(float *src, float *dest) {
  int i, j;

  for (i=0; i<SEG_LENGTH; i++) {
    dest[i] = 0;
    for (j=0; j<SEG_LENGTH; j++) {
	dest[i] += fop.fo.dctMatrix[i][j]*src[j];
    }
  }
}

void
idctTrans(float *src, float *dest) {
  int i, j;

  for (i=0; i<SEG_LENGTH; i++) {
    dest[i] = 0;
    for (j=0; j<SEG_LENGTH; j++) {
	dest[i] += fop.fo.idctMatrix[i][j]*src[j];
    }
  }
}

void
dctquant(float *src, int *dest, float dctQP) {/*edp*/
  int i;

  if (src[0] > 0) {
	dest[0] = (int) (src[0]/(dctQP/3.0) + 0.5);
  } else {
	dest[0] = (int) (src[0]/(dctQP/3.0) - 0.5);
  }

  for (i=1; i<SEG_LENGTH; i++) {
	if (src[i] > 0) {
		dest[i] = (int) (src[i]/dctQP + 0.5);
	} else {
		dest[i] = (int) (src[i]/dctQP - 0.5);
	}
  }
}

void
dctdequant(int *src, float *dest,float dctQP) {/*edp*/
  int i;

  dest[0] = src[0] * (dctQP/3.0);

  for (i=1; i<SEG_LENGTH; i++) {
	dest[i] = src[i] * dctQP;
  }
}

void
sendIDC(int symbol) {
  if ((symbol > SHORT_MAX) || (symbol <= SHORT_MIN)) {
	putbits(16, SHORT_MAX+1);
	putbits(32, symbol);
	fop.cur_frame_bit_cnt += 48;
  } else {
	if (symbol < 0) {
		symbol += 65536;
	}
	putbits(16, symbol);
	fop.cur_frame_bit_cnt += 16;
  }
}

void
sendPDC(int symbol) {
  if ((symbol>255) || (symbol<-255)) {
	huffman_code(0, fop.fo.dc_table);
	sendIDC(symbol);
  } else {
	huffman_code(symbol+256, fop.fo.dc_table);
  }
}

void
sendAC(int *coef) {
  int i;
  int runlength;

  runlength = 1;
  for (i=1; i<SEG_LENGTH; i++) {
	if (coef[i]) {
		huffman_code(runlength, fop.fo.runlength_table);
		if (coef[i]>255 || coef[i]<-256) {
			huffman_code(256, fop.fo.ac_table);
			sendIDC(coef[i]);
		} else {
			huffman_code(coef[i]+256, fop.fo.ac_table);
		}
		runlength = 1;
	} else {
		runlength++;
	}
  }

  if(runlength != 1) {
	huffman_code(0, fop.fo.runlength_table);
  }
}

void
huffman_code(int symbol, int **table) {
  int i, n;
  
  n = table[symbol][0];
  for (i=1; i<=n; i++) {
	putbits(1, table[symbol][i]);
  }
  fop.cur_frame_bit_cnt += n;
}

void 
read1Segment(int segLength, FILE *fapfile) {
  int i, j;
  int nframe;
  int fapvalue;
  int fap_mask[SEG_LENGTH][NFAP];
  bool_t get_line(FILE *);

  fop.is_intra = 0;

  for (i=0; i<segLength; i++) {
	get_line(fapfile);
	for (j=0; j<NFAP; j++) {
	  if(fscanf(fapfile, "%d", &fap_mask[i][j]) == EOF) {
	    printf("ERROR: premature EOF on fapfile\n");
	    exit(2);
	  }
	}
	if(fscanf(fapfile, "%d", &nframe) == EOF) {
	    printf("ERROR: premature EOF on fapfile\n");
	    exit(2);
	}
	if (fap_mask[i][0]) {
		if(fscanf(fapfile, "%d%d%d%d", &fop.fo.visemes[0][i], &fop.fo.visemes[1][i], &fop.fo.visemes[2][i], &fop.fo.visemes[3][i]) == EOF) {
		  printf("ERROR: premature EOF on fapfile\n");
		  exit(2);
		}
	}
	if (fap_mask[i][1]) {
		if(fscanf(fapfile, "%d%d%d%d%d%d", &fop.fo.expressions[0][i], 
			&fop.fo.expressions[1][i], &fop.fo.expressions[2][i], 
			&fop.fo.expressions[3][i], &fop.fo.expressions[4][i],
			  &fop.fo.expressions[5][i]) == EOF) {
		    printf("ERROR: premature EOF on fapfile\n");
		    exit(2);
		 }
	}
	for (j=2; j<NFAP; j++) {
		if (fap_mask[i][j]) {
		  if(fscanf(fapfile, "%d", &fapvalue) == EOF) {
		    printf("ERROR: premature EOF on fapfile\n");
		    exit(2);
		  }
		  fop.fo.faps[j][i] = (float) fapvalue;
		}
	}
  }

  for (i=0; i<NFAP; i++) {
	if (fop.fo.fap_mask[i] != fap_mask[0][i]) {
		fop.is_intra = 1;
		fop.fo.fap_mask[i] = fap_mask[0][i];
	}
  }

  for (i=1; i<segLength; i++) {
	for (j=0; j<NFAP; j++) {
		if (fap_mask[i][j] != fap_mask[i-1][j]) {
			fop.is_intra = 1;
			fop.fo.fap_mask[j] |= fap_mask[i][j];
		}
	}
  }

  for (i=0; i<segLength; i++) {
	for (j=0; j<NFAP; j++) {
		if (fop.fo.fap_mask[j] && !fap_mask[i][j]) {
			fop.fo.faps[j][i] = 0;
		}
	}
  }

  for (i=segLength; i<SEG_LENGTH; i++) {
	if (fop.fo.fap_mask[0]) {
		fop.fo.visemes[0][i] = fop.fo.visemes[1][i] = 0;
		fop.fo.visemes[2][i] = fop.fo.visemes[3][i] = 0;
	}
	if (fop.fo.fap_mask[1]) {
		fop.fo.expressions[0][i] = fop.fo.expressions[1][i] = 0;
		fop.fo.expressions[2][i] = fop.fo.expressions[3][i] = 0;
		fop.fo.expressions[4][i] = fop.fo.expressions[5][i] = 0;
	}
	for (j=2; j<NFAP; j++) {
		if (fop.fo.fap_mask[j]) {
			fop.fo.faps[j][i] = 0;
		}
	}
  }

  for (i=0; i<N_FAP_GROUP; i++) {
	fop.fo.fap_mask_type[i] = 0;
	for (j=fop.fo.mask_index[i]; j<fop.fo.mask_index[i+1]; j++) {
		if (fop.fo.fap_mask[j]) {
			fop.fo.fap_mask_type[i] = 2;
			break;
		}
	}
  }

  for(i=0; i<NFAP; i++) {/*check if mask has changed and store new mask -edp*/
    if(fop.fo.prev_fap_mask[i] != fop.fo.fap_mask[i]) fop.is_intra = 1;
    fop.fo.prev_fap_mask[i] = fop.fo.fap_mask[i];
  }

}

void 
write1Segment(int curFrame, FILE *fapfile) {
  int i, j;

  for (i=0; i<SEG_LENGTH; i++) {
    for (j=0; j<NFAP; j++) {
	fprintf(fapfile, "%d ", fop.fo.fap_mask[j]);
    }
    fprintf(fapfile, "\n");
    fprintf(fapfile, "%d ", curFrame);
    if(fop.fo.fap_mask[0]) {
	fprintf(fapfile, "%d %d %d %d ", fop.fo.visemes[0][i],
		fop.fo.visemes[1][i], fop.fo.visemes[2][i],
		fop.fo.visemes[3][i]);
    }
    if(fop.fo.fap_mask[1]) {
	fprintf(fapfile, "%d %d %d %d %d %d ", fop.fo.expressions[0][i],
		fop.fo.expressions[1][i], fop.fo.expressions[2][i],
		fop.fo.expressions[3][i], fop.fo.expressions[4][i],
		fop.fo.expressions[5][i]);
    }
    for (j=2; j<NFAP; j++) {
	if(fop.fo.fap_mask[j]) {
		fprintf(fapfile, "%d ", (int) fop.fo.faps[j][i]);
	}
    }
    fprintf(fapfile, "\n");
    curFrame++;
  }
}
/*baps*/
void
init_bap_dctQP() {
  int i;
  int quant = fop.bo.quant;

  for (i=0; i<NUM_BAPS; i++) {
	fop.bo.dctQP[i] = bap_scale[quant]*bap_DCTQuantSteps[i];
  }
}

void
init_bap_htables() {
  int i, j, n, m;
  FILE *ifp;

  ifp = fopen("dc.table", "r");
  fscanf(ifp, "%d", &n);
  fop.bo.dc_table = (int**) malloc(n*sizeof(int*));
  for (i=0; i<n; i++) {
	fscanf(ifp, "%d", &m);
	fop.bo.dc_table[i] = (int*) malloc((m+1)*sizeof(int));
	fop.bo.dc_table[i][0] = m;
	for (j=1; j<=m; j++) {
		fscanf(ifp, "%d", &fop.bo.dc_table[i][j]);
	}
  }
  fclose(ifp);

  ifp = fopen("runlength.table", "r");
  fscanf(ifp, "%d", &n);
  fop.bo.runlength_table = (int**) malloc(n*sizeof(int*));
  for (i=0; i<n; i++) {
	fscanf(ifp, "%d", &m);
	fop.bo.runlength_table[i] = (int*) malloc((m+1)*sizeof(int));
	fop.bo.runlength_table[i][0] = m;
	for (j=1; j<=m; j++) {
		fscanf(ifp, "%d", &fop.bo.runlength_table[i][j]);
	}
  }
  fclose(ifp);

  ifp = fopen("ac.table", "r");
  fscanf(ifp, "%d", &n);
  fop.bo.ac_table = (int**) malloc(n*sizeof(int*));
  for (i=0; i<n; i++) {
	fscanf(ifp, "%d", &m);
	fop.bo.ac_table[i] = (int*) malloc((m+1)*sizeof(int));
	fop.bo.ac_table[i][0] = m;
	for (j=1; j<=m; j++) {
		fscanf(ifp, "%d", &fop.bo.ac_table[i][j]);
	}
  }
  fclose(ifp);

}

void
init_bap_dct() {
  int i,j;
  float c;
 
  for(i=0; i<SEG_LENGTH; i++) {
    if(i==0) c = (float)(1.0/sqrt(2));
    else c = 1;
    for(j=0; j<SEG_LENGTH; j++) {
	fop.bo.dctMatrix[i][j] = c * (float)(sqrt(2.0/SEG_LENGTH))
				* (float)(cos(PI*(float)(2*j+1)*i/2.0/SEG_LENGTH));
    }
  }

  for(i=0; i<SEG_LENGTH; i++)
    for(j=0; j<SEG_LENGTH; j++)
      	fop.bo.idctMatrix[i][j] = fop.bo.dctMatrix[j][i];

}

void
bap_dctTrans(float *src, float *dest) {
  int i, j;

  for (i=0; i<SEG_LENGTH; i++) {
    dest[i] = 0;
    for (j=0; j<SEG_LENGTH; j++) {
	dest[i] += fop.bo.dctMatrix[i][j]*src[j];
    }
  }
}

void
bap_idctTrans(float *src, float *dest) {
  int i, j;

  for (i=0; i<SEG_LENGTH; i++) {
    dest[i] = 0;
    for (j=0; j<SEG_LENGTH; j++) {
	dest[i] += fop.bo.idctMatrix[i][j]*src[j];
    }
  }
}

void
bap_dctquant(float *src, int *dest,float dctQP) {
  int i;

  if (src[0] > 0) {
	dest[0] = (int) (src[0]/(dctQP/3.0) + 0.5);
  } else {
	dest[0] = (int) (src[0]/(dctQP/3.0) - 0.5);
  }

  for (i=1; i<SEG_LENGTH; i++) {
	if (src[i] > 0) {
		dest[i] = (int) (src[i]/dctQP + 0.5);
	} else {
		dest[i] = (int) (src[i]/dctQP - 0.5);
	}
  }
}

void
bap_dctdequant(int *src, float *dest,float dctQP) {
  int i;

  dest[0] = src[0] * (dctQP/3.0);

  for (i=1; i<SEG_LENGTH; i++) {
	dest[i] = src[i] * dctQP;
  }
}

void
bap_sendPDC(int symbol) {
  if ((symbol>255) || (symbol<-255)) {
	huffman_code(0, fop.bo.dc_table);
	sendIDC(symbol);
  } else {
	huffman_code(symbol+256, fop.bo.dc_table);
  }
}

void
bap_sendAC(int *coef) {
  int i;
  int runlength;

  runlength = 1;
  for (i=1; i<SEG_LENGTH; i++) {
	if (coef[i]) {
		huffman_code(runlength, fop.bo.runlength_table);
		if (coef[i]>255 || coef[i]<-256) {
			huffman_code(256, fop.bo.ac_table);
			sendIDC(coef[i]);
		} else {
			huffman_code(coef[i]+256, fop.bo.ac_table);
		}
		runlength = 1;
	} else {
		runlength++;
	}
  }

  if(runlength != 1) {
	huffman_code(0, fop.bo.runlength_table);
  }
}


void 
read1bapSegment(int segLength, FILE *bapfile) {
  int i, j;
  int nframe;
  int bapvalue;
  int bap_mask[SEG_LENGTH][NUM_BAPS];
  bool_t get_line(FILE *);

  fop.is_intra = 0;

  for (i=0; i<segLength; i++) {
	get_line(bapfile);
	for (j=0; j<NUM_BAPS; j++) {
	  if(fscanf(bapfile, "%d", &bap_mask[i][j]) == EOF) {
	    printf("ERROR: premature EOF on bapfile\n");
	    exit(2);
	  }
	}
	if(fscanf(bapfile, "%d", &nframe) == EOF) {
	    printf("ERROR: premature EOF on bapfile\n");
	    exit(2);
	  }
	for (j=0; j<NUM_BAPS; j++) {
		if (bap_mask[i][j]) {
		  if(fscanf(bapfile, "%d", &bapvalue) == EOF) {
		    printf("ERROR: premature EOF on bapfile\n");
		    exit(2);
		  }
		  fop.bo.baps[j][i] = (float) bapvalue;
		}
	}
  }

  for (i=0; i<NUM_BAPS; i++) {
	if (fop.bo.bap_mask[i] != bap_mask[0][i]) {
		fop.is_intra = 1;
		fop.bo.bap_mask[i] = bap_mask[0][i];
	}
  }

  for (i=1; i<segLength; i++) {
	for (j=0; j<NUM_BAPS; j++) {
		if (bap_mask[i][j] != bap_mask[i-1][j]) {
			fop.is_intra = 1;
			fop.bo.bap_mask[j] |= bap_mask[i][j];
		}
	}
  }

  for (i=0; i<segLength; i++) {
	for (j=0; j<NUM_BAPS; j++) {
		if (fop.bo.bap_mask[j] && !bap_mask[i][j]) {
			fop.bo.baps[j][i] = 0;
		}
	}
  }

  for (i=segLength; i<SEG_LENGTH; i++) {
	for (j=0; j<NUM_BAPS; j++) {
		if (fop.bo.bap_mask[j]) {
			fop.bo.baps[j][i] = 0;
		}
	}
  }

  for (i=0; i<BAP_NUM_GROUPS; i++) {
	fop.bo.bap_mask_type[i] = 0;
	for (j=0; j<fop.bo.baps_in_group[i]; j++) {
		if (fop.bo.bap_mask[fop.bo.bap_mask_index[i][j]]) {
			fop.bo.bap_mask_type[i] = 1;/*send baps individually-edp*/
			break;
		}
	}
  }

  for (i=0; i<NUM_BAPS; i++) {/*check if mask has changed and store new mask -edp*/
    if(fop.bo.prev_bap_mask[i] != fop.bo.bap_mask[i]) fop.is_intra = 1;
    fop.bo.prev_bap_mask[i] = fop.bo.bap_mask[i];
  }

}

void 
write1bapSegment(int curFrame, FILE *bapfile) {
  int i, j;

  for (i=0; i<SEG_LENGTH; i++) {
    for (j=0; j<NUM_BAPS; j++) {
	fprintf(bapfile, "%d ", fop.bo.bap_mask[j]);
    }
    fprintf(bapfile, "\n");
    fprintf(bapfile, "%d ", curFrame);
    for (j=0; j<NUM_BAPS; j++) {
	if(fop.bo.bap_mask[j]) {
		fprintf(bapfile, "%d ", (int) fop.bo.baps[j][i]);
		printf("%d.%d ",j, (int) fop.bo.baps[j][i]);
	}
    }
    fprintf(bapfile, "\n");
    printf("\n");
    curFrame++;
  }
}















