/*************************************************************************

This software module developed by 

	Wei Wu (weiwu@stallion.risc.rockwell.com),  Rockwell Science Center 
        Tolga K. Capin (capin@lig.di.epfl.ch), EPFL-LIG,
        Joaquim Esmerado (jle@lig.di.epfl.ch), EPFL-LIG.
	Kayvon Pirestani (kayvon@cs.cmu.edu), EPFL DI-LIG

in the course of development of the MPEG-4 Video (ISO/IEC 14496-2). 
This software module is an implementation of a part of one or more MPEG-4 Video
tools as specified by the MPEG-4 Video. ISO/IEC gives users of the MPEG-4 Video
free license to this software module or modifications thereof for use in 
hardware or software products claiming conformance to the MPEG-4 Video. Those 
intending to use this software module in hardware or software products are advised 
that its use may infringe existing patents. The original developer of this software 
module and his/her company, the subsequent editors and their companies, and ISO/IEC 
have no liability for use of this software module or modifications thereof in an 
implementation. Copyright is not released for non MPEG-4 Video conforming 
products. Rockwell Science Center and EPFL-LIG retain full right to use the code for
his/her own  purpose, assign or donate the code to a third party and to inhibit third
parties from  using the code for non <MPEG standard> conforming products. This copyright
notice  must be included in all copies or derivative works. Copyright (c) 1997.
 
Module Name:	fba_object_plane.c

Abstract:	Code face/body animation frame

Update:		April-1997
		July-1997
		August-1997
		Nov-1997 

Remarks:	see RCS log at end of encode.c

*************************************************************************/

#include "common.h"
#include "sim.h"

static int QP[NFAP] = {1, 1, 4, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 1, 1, 1, 1, 1,
	        1, 1, 1, 1, 128, 128, 128, 128, 8, 8, 1, 1, 2, 2, 
		2, 2, 2, 2, 1, 1, 2, 2, 2, 2, 1, 1, 1, 1, 512, 170,
		170, 170, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 1, 1, 1, 1, 
		1, 1, 1, 1};

static int BQP[NUM_BAPS] = {64,64,64,128,128,128,128,256,256,128,128,256,256,
128,128,256,256,256,256,256,256,256,256,128,128,128,128,128,128,128,128,64,64,64,64,256,256,
64,64,256,256,128,128,128,128,256,256,128,128,128,256,256,256,128,128,128,256,256,256,128,128,128,256,256,256,256,256,256,256,256,256,
128,128,128,256,256,256,256,256,256,256,256,256,256,256,256,128,128,128,256,256,256,256,256,256,256,256,256,
128,128,128,256,256,256,256,256,256,128,128,128,256,256,256,128,128,128,256,256,256,128,128,128,512,512,128,128,128,128,256,256,
128,128,128,128,256,256,128,128,128,128,256,256,128,128,128,128,256,256,128,128,128,128,256,256,128,128,128,128,256,256,128,128,128,128,256,256,
128,128,128,128,128,128,128,128,256,256,128,128,128,128,64,64,64,64,64,64};

static int bap_pred_scale[32] = {0,1,2,3,5,7,9,11,14,17,20,23,27,31,35,39,43,47,52,57,62,67,72,77,82,88,94,100,106,113,120,127};


void code_viseme(int is_intra);
void code_expression(int is_intra);
void code_bap_group_mask(int n);
void code_fap_group_mask(int n);

void fba_object_plane_header() 
{
  if (fop.is_startcode) { /*changed from is_intra -edp*/
    fop.cur_frame_bit_cnt = next_start_code();
    fop.cur_frame_bit_cnt += putbits(32, FBA_OBJECT_PLANE_START_CODE);
    printf("FBA_OBJECT_PLANE_START_CODE %d\n",FBA_OBJECT_PLANE_START_CODE);
  }
  fop.cur_frame_bit_cnt += putbits( 1, fop.is_intra);
  printf("fop.is_intra %d\n",fop.is_intra);
  fop.cur_frame_bit_cnt += putbits(2, fop.fba_paramset);
  printf("is_fba_paramset(2) %d\n",fop.fba_paramset);

  /* Temporal header */
  if (fop.is_intra) {
    if (fop.fba_paramset & FAP_PARAMSET_MASK) {

      fop.cur_frame_bit_cnt += putbits(1, fop.fo.is_frame_rate);
      printf("is_frame_rate %d\n",fop.fo.is_frame_rate);
      if(fop.fo.is_frame_rate) {
	code_frame_rate();
      }
      fop.cur_frame_bit_cnt += putbits(1, fop.fo.is_time_code);
      printf("is_time_code %d\n", fop.fo.is_time_code);
      if(fop.fo.is_time_code) {
	code_time_code();
      }
    }
  }/*v17*/
  if (fop.fba_paramset & FAP_PARAMSET_MASK) {/*v17*/
    fop.cur_frame_bit_cnt += putbits(1, fop.fo.skip_frames);
    printf("fop.fo.skip_frames %d\n",fop.fo.skip_frames);
    if(fop.fo.skip_frames) {
      code_skip_frames();
    }
  }
    
  if (fop.is_intra) {
    if ((fop.fba_paramset & BAP_PARAMSET_MASK) && !(fop.fba_paramset & FAP_PARAMSET_MASK)) {
      fop.cur_frame_bit_cnt += putbits(1, fop.bo.is_frame_rate);
      printf("bo.is_frame_rate %d\n",fop.bo.is_frame_rate);
      if(fop.bo.is_frame_rate) {
	bap_code_frame_rate();
      }
      
      fop.cur_frame_bit_cnt += putbits(1, fop.bo.is_time_code);
      printf("bo.is_time_code %d\n",fop.bo.is_time_code);
      if(fop.bo.is_time_code) {
	bap_code_time_code();
      }
    }
  }      /*v17*/
  if ((fop.fba_paramset & BAP_PARAMSET_MASK) && !(fop.fba_paramset & FAP_PARAMSET_MASK)) {/*v17*/
    fop.cur_frame_bit_cnt += putbits(1, fop.bo.skip_frames);
    printf("bo.skip_frames %d\n",fop.bo.skip_frames);
    if(fop.bo.skip_frames) {
      bap_code_skip_frames();
    }
  }
}


void fba_object_plane_data() 
{
  int i,j;
  static int fr_no=0;
  int coding_type = fop.fo.coding_type; /* 6-25-98 */
  static int no_frames=0;
  if(coding_type == PREDICTIVE_CODING_TYPE) printf("Frame %d\n", no_frames++);
  else {
    printf("Frames %d thru %d\n",no_frames,no_frames+15);
    no_frames+=16;
  }

    fr_no++;

  if (fop.fba_paramset & FAP_PARAMSET_MASK) {

    if(fop.is_intra) {

      fop.cur_frame_bit_cnt += putbits(5, fop.fo.quant);
      printf("fop.fo.quant(5) = %d\n",fop.fo.quant);

      for(i=0; i<N_FAP_GROUP; i++) {
	fop.cur_frame_bit_cnt += putbits(1, MARKER_BIT);
	fop.cur_frame_bit_cnt += putbits(2, fop.fo.fap_mask_type[i]);
	if(fop.fo.fap_mask_type[i]==1 || fop.fo.fap_mask_type[i]==2) {
	  code_fap_group_mask(i);
	}
      }

      fop.cur_frame_bit_cnt += putbits(1, fop.fo.suggested_gender);
      fop.cur_frame_bit_cnt += putbits(1, coding_type); /* 3-3-98 *//*changed 2 to 1 -edp*/
      if (coding_type == PREDICTIVE_CODING_TYPE) {
	if(fop.fo.havesetminmax) for(i=0;i<NFAP;i++) if(fop.fo.fap_mask[i] && !fop.fo.minmaxset[i]) {/*check that minmax sent for each fap*/
	  fop.fo.is_i_new_max = 1;fop.fo.is_i_new_min = 1;fop.fo.is_p_new_max = 1;fop.fo.is_p_new_min = 1;
	  break;
	}
	fop.cur_frame_bit_cnt += putbits(1, fop.fo.is_i_new_max);
	fop.cur_frame_bit_cnt += putbits(1, fop.fo.is_i_new_min);
	fop.cur_frame_bit_cnt += putbits(1, fop.fo.is_p_new_max);
	fop.cur_frame_bit_cnt += putbits(1, fop.fo.is_p_new_min);
      	code_new_minmax();
	code_ifap();
      }
      else {
	code_i_segment();
      }
    }
    else {
      if (coding_type == PREDICTIVE_CODING_TYPE) {
	code_pfap();
      } else {
	code_p_segment();
      }
    }
  }

  if (fop.fba_paramset & BAP_PARAMSET_MASK) {

    if(fop.is_intra) {

      fop.cur_frame_bit_cnt += putbits(5, fop.bo.quant);


      for(i=0; i<BAP_NUM_GROUPS; i++) {
	fop.cur_frame_bit_cnt += putbits(1, MARKER_BIT);
	fop.cur_frame_bit_cnt += putbits(2, fop.bo.bap_mask_type[i]);
	if(fop.bo.bap_mask_type[i]==1) {
	  code_bap_group_mask(i);
	}
      }

      fop.cur_frame_bit_cnt += putbits(1, fop.fo.suggested_gender);
      fop.cur_frame_bit_cnt += putbits(1, coding_type); /* 3-3-98 *//*changed 2 to 1 -edp*/

      if (coding_type == PREDICTIVE_CODING_TYPE) {
	if(fop.bo.havesetminmax) for(i=0;i<BAP_NUM_GROUPS;i++) for(j=0;j<fop.bo.baps_in_group[i]; j++) if(fop.bo.bap_mask[fop.bo.bap_mask_index[i][j]] && !fop.bo.minmaxset[fop.bo.bap_mask_index[i][j]]) {/*check that minmax sent for each bap*/
	  fop.bo.is_i_new_max = 1;fop.bo.is_i_new_min = 1;fop.bo.is_p_new_max = 1;fop.bo.is_p_new_min = 1;
	  break;
	}
	fop.cur_frame_bit_cnt += putbits(1, fop.bo.is_i_new_max);
	fop.cur_frame_bit_cnt += putbits(1, fop.bo.is_i_new_min);
	fop.cur_frame_bit_cnt += putbits(1, fop.bo.is_p_new_max);
	fop.cur_frame_bit_cnt += putbits(1, fop.bo.is_p_new_min);

	/* has_extendable_baps = 0 for the moment 
	fop.cur_frame_bit_cnt += putbits(1, 0);   */ 

	bap_code_new_minmax();
	code_ibap();
      }
      else {
	bap_code_i_segment();
      }
    } else {
      if (coding_type == PREDICTIVE_CODING_TYPE) {
	code_pbap();
      } else {
	bap_code_p_segment();
      }
    }
  }
}

void
code_frame_rate() 
{
  fop.cur_frame_bit_cnt += putbits(8, fop.fo.frame_rate);
  printf("fop.fo.frame_rate(8) %d\n",fop.fo.frame_rate);
  fop.cur_frame_bit_cnt += putbits(4, fop.fo.fr_seconds);
  printf("fop.fo.fr_seconds(4) %d\n",fop.fo.fr_seconds);
  fop.cur_frame_bit_cnt += putbits(1, fop.fo.fr_frequency_offset);
  printf("fop.fo.fr_frequency_offset %d\n",fop.fo.fr_frequency_offset);
}


void
code_time_code() 
{
  fop.cur_frame_bit_cnt += putbits(5, fop.fo.time_code_hours);
  fop.cur_frame_bit_cnt += putbits(6, fop.fo.time_code_minutes);
  fop.cur_frame_bit_cnt += putbits(1, fop.marker_bit);
  fop.cur_frame_bit_cnt += putbits(6, fop.fo.time_code_seconds);
}


void
code_skip_frames() 
{
  int ivalue;

  do {
    if(fop.fo.number_of_frames_to_skip<15) {
      ivalue = fop.fo.number_of_frames_to_skip;
    } else {
      ivalue = 15;
      fop.fo.number_of_frames_to_skip -= 15;
    }
    fop.cur_frame_bit_cnt += putbits(4, ivalue);
  } while (ivalue==15);
}


void
bap_code_frame_rate() 
{
  fop.cur_frame_bit_cnt += putbits(8, fop.bo.frame_rate);
  fop.cur_frame_bit_cnt += putbits(4, fop.bo.fr_seconds);
  fop.cur_frame_bit_cnt += putbits(1, fop.bo.fr_frequency_offset);
}


void
bap_code_time_code() 
{
  fop.cur_frame_bit_cnt += putbits(5, fop.bo.time_code_hours);
  fop.cur_frame_bit_cnt += putbits(6, fop.bo.time_code_minutes);
  fop.cur_frame_bit_cnt += putbits(1, fop.marker_bit);
  fop.cur_frame_bit_cnt += putbits(6, fop.bo.time_code_seconds);
}


void
bap_code_skip_frames() 
{
  int ivalue;

  do {
    if(fop.bo.number_of_frames_to_skip<15) {
      ivalue = fop.bo.number_of_frames_to_skip;
    } else {
      ivalue = 15;
      fop.bo.number_of_frames_to_skip -= 15;
    }
    fop.cur_frame_bit_cnt += putbits(4, ivalue);
  } while (ivalue==15);
}

void 
code_fap_group_mask(int n) 
{
  int i;
  for(i=fop.fo.mask_index[n]; i<fop.fo.mask_index[n+1]; i++) 
    fop.cur_frame_bit_cnt += putbits(1, fop.fo.fap_mask[i]);
}

void 
code_bap_group_mask(int n) 
{
  int i;
  int temp_index;

  for(i=0;i<fop.bo.baps_in_group[n]; i++) {
    temp_index = fop.bo.bap_mask_index[n][i];
    fop.cur_frame_bit_cnt += putbits(1, fop.bo.bap_mask[temp_index]);
  }
}

void bap_code_new_minmax()
{
  int i, j;
  int temp_index;

  if(fop.bo.is_i_new_max) {
    fop.bo.havesetminmax = 1;
    for (i=0; i<BAP_NUM_GROUPS; i++) {
        for(j=0;j<fop.bo.baps_in_group[i]; j++) {
	  temp_index = fop.bo.bap_mask_index[i][j];
	  if(!(j&3)) 
	    fop.cur_frame_bit_cnt += putbits(1, MARKER_BIT);
	  if(fop.bo.bap_mask[temp_index]) {
	    fop.cur_frame_bit_cnt += putbits(5, fop.bo.i_new_max[temp_index]);
	    fop.bo.minmaxset[temp_index] = 1;/* just set here. assume that I/P min max all set for a given bap -edp*/
	  }
	}
    }
  }

  if(fop.bo.is_i_new_min) {
    for (i=0; i<BAP_NUM_GROUPS; i++) {
        for(j=0;j<fop.bo.baps_in_group[i]; j++) {
	  temp_index = fop.bo.bap_mask_index[i][j];
	  if(!(j&3)) 
	    fop.cur_frame_bit_cnt += putbits(1, MARKER_BIT);
	  if(fop.bo.bap_mask[temp_index]) 
	    fop.cur_frame_bit_cnt += putbits(5, fop.bo.i_new_min[temp_index]);
	}
    }
  }

  if(fop.bo.is_p_new_max) {
    for (i=0; i<BAP_NUM_GROUPS; i++) {
        for(j=0;j<fop.bo.baps_in_group[i]; j++) {
	  temp_index = fop.bo.bap_mask_index[i][j];
	  if(!(j&3)) 
	    fop.cur_frame_bit_cnt += putbits(1, MARKER_BIT);
	  if(fop.bo.bap_mask[temp_index]) 
	    fop.cur_frame_bit_cnt += putbits(5, fop.bo.p_new_max[temp_index]);
	}
    }
  }

  if(fop.bo.is_p_new_min) {
    for (i=0; i<BAP_NUM_GROUPS; i++) {
        for(j=0;j<fop.bo.baps_in_group[i]; j++) {
	  temp_index = fop.bo.bap_mask_index[i][j];
	  if(!(j&3)) 
	    fop.cur_frame_bit_cnt += putbits(1, MARKER_BIT);
	  if(fop.bo.bap_mask[temp_index]) 
	    fop.cur_frame_bit_cnt += putbits(5, fop.bo.p_new_min[temp_index]);
	}
    }
  }

}

void code_new_minmax() 
{
  int i, j;

  if(fop.fo.is_i_new_max) {
    fop.fo.havesetminmax = 1;
    for(i=2, j=0; i<NFAP; i++, j++) {
      if(i==18 || i==30 || i==38 || i== 42 ||
	 i==47 || i==50 || i==60 || i== 64) {
	j=0;
      }
      if(!(j&3)) 
	fop.cur_frame_bit_cnt += putbits(1, MARKER_BIT);
      
      if(fop.fo.fap_mask[i]) {
	fop.cur_frame_bit_cnt += putbits(5, fop.fo.i_new_max[i]);
	fop.fo.minmaxset[i] = 1;/* just set here. assume that I/P min max all set for a given bap -edp*/
      }
    }
  }
  if(fop.fo.is_i_new_min) {
    for(i=2, j=0; i<NFAP; i++, j++) {
      if(i==18 || i==30 || i==38 || i== 42 ||
	 i==47 || i==50 || i==60 || i== 64) {
	j=0;
      }
      if(!(j&3)) 
	fop.cur_frame_bit_cnt += putbits(1, MARKER_BIT);
      
      if(fop.fo.fap_mask[i]) 
	fop.cur_frame_bit_cnt += putbits(5, fop.fo.i_new_min[i]);
    }
  }
  if(fop.fo.is_p_new_max) {
    for(i=2, j=0; i<NFAP; i++, j++) {
      if(i==18 || i==30 || i==38 || i== 42 ||
	 i==47 || i==50 || i==60 || i== 64) {
	j=0;
      }
      if(!(j&3)) {
	fop.cur_frame_bit_cnt += putbits(1, MARKER_BIT);
      }
      if(fop.fo.fap_mask[i]) {
	fop.cur_frame_bit_cnt += putbits(5, fop.fo.p_new_max[i]);
      }
    }
  }
  if(fop.fo.is_p_new_min) {
    for(i=2, j=0; i<NFAP; i++, j++) {
      if(i==18 || i==30 || i==38 || i== 42 ||
	 i==47 || i==50 || i==60 || i== 64) {
	j=0;
      }
      if(!(j&3))
	fop.cur_frame_bit_cnt += putbits(1, MARKER_BIT);
      
      if(fop.fo.fap_mask[i])
	fop.cur_frame_bit_cnt += putbits(5, fop.fo.p_new_min[i]);
    }
  }
}


void 
code_ibap() 
{
  int i, j, k, qstep;
  int symbol;
  int nbits;

  for (k=0; k<BAP_NUM_GROUPS; k++) {
    for(j=0;j<fop.bo.baps_in_group[k]; j++) {
	  i = fop.bo.bap_mask_index[k][j];
	  if(fop.bo.bap_mask[i]) { /* code the ith bap */
	    qstep = BQP[i]*bap_pred_scale[fop.bo.quant];
	    fop.bo.bap_Q[i] = quant(fop.bo.bap_orig[i], qstep);
	    symbol = fop.bo.bap_Q[i] - fop.bo.Irange_min[i];
	    bap_check_range(symbol, i, 0);
	    nbits = AR_Encode(symbol, fop.bo.Icumul_freq[i]);
	    fop.cur_frame_bit_cnt += nbits;
	    bap_update(symbol, i);
	    fop.bo.bap_prev[i] = dequant(fop.bo.bap_Q[i], qstep);
	    fprintf(fop.bo.bap_out_fp, "%d ", fop.bo.bap_prev[i]);
	  }
    }
  }
  nbits = encoder_flush();
  fop.cur_frame_bit_cnt += nbits;
  fprintf(fop.bo.bap_out_fp, "\n");
  
}

void
code_pbap() {

  int i, j, k, qstep;
  int symbol;
  int nbits;

  
  /* v17
     fop.cur_frame_bit_cnt += putbits(1, fop.bo.skip_frames);
  if(fop.bo.skip_frames) 
    bap_code_skip_frames();
    */

  for (k=0; k<BAP_NUM_GROUPS; k++) {
    for(j=0;j<fop.bo.baps_in_group[k]; j++) {
	  i = fop.bo.bap_mask_index[k][j];
	  if(fop.bo.bap_mask[i]) { /* code the ith bap */
	    qstep = BQP[i]*bap_pred_scale[fop.bo.quant];
	    fop.bo.bap_diff[i] = pred(fop.bo.bap_orig[i], fop.bo.bap_prev[i]);
	    fop.bo.bap_Q[i] = quant(fop.bo.bap_diff[i], qstep);
	    symbol = fop.bo.bap_Q[i] - fop.bo.range_min[i];
	    bap_check_range(symbol, i, 1);
	    nbits = AR_Encode(symbol, fop.bo.cumul_freq[i]);
	    fop.cur_frame_bit_cnt += nbits;
	    bap_update(symbol, i);
	    fop.bo.bap_diff[i] = dequant(fop.bo.bap_Q[i], qstep);
	    fop.bo.bap_prev[i] = recon(fop.bo.bap_diff[i], fop.bo.bap_prev[i]);
	    fprintf(fop.bo.bap_out_fp, "%d ", fop.bo.bap_prev[i]);
	  }
    }
  }
  nbits = encoder_flush();
  fop.cur_frame_bit_cnt += nbits;
  fprintf(fop.bo.bap_out_fp, "\n");
  
}

void 
code_ifap() 
{
  int i, qstep;
  int symbol;
  int nbits;

  if(fop.fo.fap_mask[0]) 
    code_viseme(1);
  
  if(fop.fo.fap_mask[1]) 
    code_expression(1);

  for(i=2; i<NFAP; i++) {
    if(fop.fo.fap_mask[i]) { /* code the ith fap */
      qstep = QP[i]*fop.fo.quant;
      fop.fo.fap_Q[i] = quant(fop.fo.fap_orig[i], qstep);
      symbol = fop.fo.fap_Q[i] - fop.fo.Irange_min[i];
      check_range(symbol, i, 0);
      nbits = AR_Encode(symbol, fop.fo.Icumul_freq[i]);
      fop.cur_frame_bit_cnt += nbits;
      update(symbol, i);
      fop.fo.fap_prev[i] = dequant(fop.fo.fap_Q[i], qstep);
      fprintf(fop.fo.fap_out_fp, "%d ", fop.fo.fap_prev[i]);
    }
  }
  nbits = encoder_flush();
  fop.cur_frame_bit_cnt += nbits;
  fprintf(fop.fo.fap_out_fp, "\n");
  
}

void
code_pfap() {

  int i, qstep;
  int symbol;
  int nbits;

  /*  v17
      fop.cur_frame_bit_cnt += putbits(1, fop.fo.skip_frames);
  if(fop.fo.skip_frames) 
    code_skip_frames();
    */
  if(fop.fo.fap_mask[0]) 
    code_viseme(0);
  
  if(fop.fo.fap_mask[1]) 
    code_expression(0);

  for(i=2; i<NFAP; i++) {
    if(fop.fo.fap_mask[i]) { /* code the ith fap */
      qstep = QP[i]*fop.fo.quant;
      fop.fo.fap_diff[i] = pred(fop.fo.fap_orig[i], fop.fo.fap_prev[i]);
      fop.fo.fap_Q[i] = quant(fop.fo.fap_diff[i], qstep);
      symbol = fop.fo.fap_Q[i] - fop.fo.range_min[i];
      check_range(symbol, i, 1);
      nbits = AR_Encode(symbol, fop.fo.cumul_freq[i]);
      fop.cur_frame_bit_cnt += nbits;
      update(symbol, i);
      fop.fo.fap_diff[i] = dequant(fop.fo.fap_Q[i], qstep);
      fop.fo.fap_prev[i] = recon(fop.fo.fap_diff[i], fop.fo.fap_prev[i]);
      fprintf(fop.fo.fap_out_fp, "%d ", fop.fo.fap_prev[i]);
    }
  }
  nbits = encoder_flush();
  fop.cur_frame_bit_cnt += nbits;
  fprintf(fop.fo.fap_out_fp, "\n");

}

void
code_viseme(int is_intra) {
  int nbits;
  int symbol;

  if(is_intra) {
    symbol = fop.fo.ve_orig[0];
    nbits = AR_Encode(symbol, fop.fo.Icumul_ve[0]);
    update_viseme_expression(symbol, 0);
    fop.fo.ve_prev[0] = fop.fo.ve_orig[0];

    symbol = fop.fo.ve_orig[1];
    nbits += AR_Encode(symbol, fop.fo.Icumul_ve[1]);
    update_viseme_expression(symbol, 1);
    fop.fo.ve_prev[1] = fop.fo.ve_orig[1];

    fop.fo.viseme_blend_Q = quant(fop.fo.ve_orig[2], fop.fo.quant);
    symbol = fop.fo.viseme_blend_Q;
    nbits += AR_Encode(symbol, fop.fo.Icumul_ve[2]);
    update_viseme_expression(symbol, 2);
    fop.fo.ve_prev[2] = dequant(fop.fo.viseme_blend_Q, fop.fo.quant);

    nbits += encoder_flush();
    nbits += putbits(1, fop.fo.viseme_def);
    fop.cur_frame_bit_cnt += nbits;
    fprintf(fop.fo.fap_out_fp, "%d %d %d %d ", fop.fo.ve_prev[0], fop.fo.ve_prev[1],
	    fop.fo.ve_prev[2], fop.fo.viseme_def);
  } else {
    symbol = fop.fo.ve_orig[0]-fop.fo.ve_prev[0];
    symbol -= fop.fo.range_ve_min[0];
    nbits = AR_Encode(symbol, fop.fo.cumul_ve[0]);
    update_viseme_expression(symbol, 0);
    fop.fo.ve_prev[0] = fop.fo.ve_orig[0];

    symbol = fop.fo.ve_orig[1]-fop.fo.ve_prev[1];
    symbol -= fop.fo.range_ve_min[1];
    nbits += AR_Encode(symbol, fop.fo.cumul_ve[1]);
    update_viseme_expression(symbol, 1);
    fop.fo.ve_prev[1] = fop.fo.ve_orig[1];

    fop.fo.ve_diff[2] = pred(fop.fo.ve_orig[2], fop.fo.ve_prev[2]);
    fop.fo.viseme_blend_Q = quant(fop.fo.ve_diff[2], fop.fo.quant);
    symbol = fop.fo.viseme_blend_Q - fop.fo.range_ve_min[2];
    nbits += AR_Encode(symbol, fop.fo.cumul_ve[2]);
    update_viseme_expression(symbol, 2);
    fop.fo.ve_diff[2] = dequant(fop.fo.viseme_blend_Q, fop.fo.quant);
    fop.fo.ve_prev[2] = recon(fop.fo.ve_diff[2], fop.fo.ve_prev[2]);

    nbits += encoder_flush();
    nbits += putbits(1, fop.fo.viseme_def);
    fop.cur_frame_bit_cnt += nbits;
    fprintf(fop.fo.fap_out_fp, "%d %d %d %d ", fop.fo.ve_prev[0], fop.fo.ve_prev[1],
	    fop.fo.ve_prev[2], fop.fo.viseme_def);
  }
}


void
code_expression(int is_intra) 
{
  int nbits;
  int symbol;

  if(is_intra) {
    symbol = fop.fo.ve_orig[3];
    nbits = AR_Encode(symbol, fop.fo.Icumul_ve[3]);
    update_viseme_expression(symbol, 3);
    fop.fo.ve_prev[3] = fop.fo.ve_orig[3];

    fop.fo.expr_intensity1_Q = quant(fop.fo.ve_orig[4], fop.fo.quant);
    symbol = fop.fo.expr_intensity1_Q;
    nbits += AR_Encode(symbol, fop.fo.Icumul_ve[4]);
    update_viseme_expression(symbol, 4);
    fop.fo.ve_prev[4] = dequant(fop.fo.expr_intensity1_Q, fop.fo.quant);

    symbol = fop.fo.ve_orig[5];
    nbits += AR_Encode(symbol, fop.fo.Icumul_ve[5]);
    update_viseme_expression(symbol, 5);
    fop.fo.ve_prev[5] = fop.fo.ve_orig[5];

    fop.fo.expr_intensity2_Q = quant(fop.fo.ve_orig[6], fop.fo.quant);
    symbol = fop.fo.expr_intensity2_Q;
    nbits += AR_Encode(symbol, fop.fo.Icumul_ve[6]);
    update_viseme_expression(symbol, 6);
    fop.fo.ve_prev[6] = dequant(fop.fo.expr_intensity2_Q, fop.fo.quant);

    nbits += encoder_flush();
    nbits += putbits(1, fop.fo.init_face);
    nbits += putbits(1, fop.fo.expression_def);
    fop.cur_frame_bit_cnt += nbits;
    fprintf(fop.fo.fap_out_fp, "%d %d %d %d %d %d ", fop.fo.ve_prev[3], fop.fo.ve_prev[4],
	    fop.fo.ve_prev[5], fop.fo.ve_prev[6], fop.fo.init_face, 
	    fop.fo.expression_def);
  } else {
    symbol = fop.fo.ve_orig[3]-fop.fo.ve_prev[3];
    symbol -= fop.fo.range_ve_min[3];
    nbits = AR_Encode(symbol, fop.fo.cumul_ve[3]);
    update_viseme_expression(symbol, 3);
    fop.fo.ve_prev[3] = fop.fo.ve_orig[3];

    fop.fo.ve_diff[4] = pred(fop.fo.ve_orig[4], fop.fo.ve_prev[4]);
    fop.fo.expr_intensity1_Q = quant(fop.fo.ve_diff[4], fop.fo.quant);
    symbol = fop.fo.expr_intensity1_Q - fop.fo.range_ve_min[4];
    nbits += AR_Encode(symbol, fop.fo.cumul_ve[4]);
    update_viseme_expression(symbol, 4);
    fop.fo.ve_diff[4] = dequant(fop.fo.expr_intensity1_Q, fop.fo.quant);
    fop.fo.ve_prev[4] = recon(fop.fo.ve_diff[4], fop.fo.ve_prev[4]);

    symbol = fop.fo.ve_orig[5]-fop.fo.ve_prev[5];
    symbol -= fop.fo.range_ve_min[5];
    nbits += AR_Encode(symbol, fop.fo.cumul_ve[5]);
    update_viseme_expression(symbol, 5);
    fop.fo.ve_prev[5] = fop.fo.ve_orig[5];

    fop.fo.ve_diff[6] = pred(fop.fo.ve_orig[6], fop.fo.ve_prev[6]);
    fop.fo.expr_intensity2_Q = quant(fop.fo.ve_diff[6], fop.fo.quant);
    symbol = fop.fo.expr_intensity2_Q - fop.fo.range_ve_min[6];
    nbits += AR_Encode(symbol, fop.fo.cumul_ve[6]);
    update_viseme_expression(symbol, 6);
    fop.fo.ve_diff[6] = dequant(fop.fo.expr_intensity2_Q, fop.fo.quant);
    fop.fo.ve_prev[6] = recon(fop.fo.ve_diff[6], fop.fo.ve_prev[6]);

    nbits += encoder_flush();
    nbits += putbits(1, fop.fo.init_face);
    nbits += putbits(1, fop.fo.expression_def);
    fop.cur_frame_bit_cnt += nbits;
    fprintf(fop.fo.fap_out_fp, "%d %d %d %d %d %d ", fop.fo.ve_prev[3], 
	    fop.fo.ve_prev[4], fop.fo.ve_prev[5], fop.fo.ve_prev[6], 
	    fop.fo.init_face, fop.fo.expression_def);
  }

}

void 
check_range(int symbol, int i, int inter) 
{
  if(inter) {
    symbol += fop.fo.range_min[i];
    if(symbol < fop.fo.range_min[i] || symbol > fop.fo.range_max[i]) {
      fprintf(stderr, "Error: AR coding out of range[1]:\n");
      fprintf(stderr, "\tParameter #%d: %d\n", i, symbol);
      fprintf(stderr, "\tRange: (%d, %d)\n", 
	      fop.fo.range_min[i], fop.fo.range_max[i]);
      fprintf(stderr, "\tNeed to reset %dth inter parameter range!\n", i);
      exit(-1);
    }
  } else {
    symbol += fop.fo.Irange_min[i];
    if(symbol < fop.fo.Irange_min[i] || symbol > fop.fo.Irange_max[i]) {
      fprintf(stderr, "Error: AR coding out of range[2]:\n");
      fprintf(stderr, "\tParameter #%d: %d\n", i, symbol);
      fprintf(stderr, "\tRange: (%d, %d)\n", 
	      fop.fo.Irange_min[i], fop.fo.Irange_max[i]);
      fprintf(stderr, "\tNeed to reset %dth intra parameter range!\n", i);
      exit(-1);
    }
  }
}

void 
bap_check_range(int symbol, int i, int inter) 
{
  if(inter) {
    symbol += fop.bo.range_min[i];
    if(symbol < fop.bo.range_min[i] || symbol > fop.bo.range_max[i]) {
      fprintf(stderr, "Error: AR coding out of range[3]:\n");
      fprintf(stderr, "\tParameter #%d: %d\n", i, symbol);
      fprintf(stderr, "\tRange: (%d, %d)\n", 
	      fop.bo.range_min[i], fop.bo.range_max[i]);
      fprintf(stderr, "\tNeed to reset %dth inter parameter range!\n", i);
      exit(-1);
    }
  } else {
    symbol += fop.bo.Irange_min[i];
    if(symbol < fop.bo.Irange_min[i] || symbol > fop.bo.Irange_max[i]) {
      fprintf(stderr, "Error: AR coding out of range[4]:\n");
      fprintf(stderr, "\tParameter #%d: %d\n", i, symbol);
      fprintf(stderr, "\tRange: (%d, %d)\n", 
	      fop.bo.Irange_min[i], fop.bo.Irange_max[i]);
      fprintf(stderr, "\tNeed to reset %dth intra parameter range!\n", i);
      exit(-1);
    }
  }
}

int 
pred(int fap_curr, int fap_prev) {
  int fap_diff;

  fap_diff = fap_curr - fap_prev;
  return fap_diff;
}

int
recon(int fap_diff, int fap_prev) {
  int fap_rec;

  fap_rec = fap_diff + fap_prev;
  return fap_rec;
}
