/*************************************************************************

This software module was originally developed by 
	Wei Wu (weiwu@stallion.risc.rockwell.com),  Rockwell Science Center 
        Tolga K. Capin (capin@lig.di.epfl.ch), EPFL-LIG,
        Joaquim Esmerado (jle@lig.di.epfl.ch), EPFL-LIG.

in the course of development of the MPEG-4 Video (ISO/IEC 14496-2). 
This software module is an implementation of a part of one or more MPEG-4 Video
tools as specified by the MPEG-4 Video. ISO/IEC gives users of the MPEG-4 Video
free license to this software module or modifications thereof for use in 
hardware or software products claiming conformance to the MPEG-4 Video. Those 
intending to use this software module in hardware or software products are advised 
that its use may infringe existing patents. The original developer of this software 
module and his/her company, the subsequent editors and their companies, and ISO/IEC 
have no liability for use of this software module or modifications thereof in an 
implementation. Copyright is not released for non MPEG-4 Video conforming 
products. Rockwell Science Center and EPFL-LIG retain full right to use the
code for his/her own  purpose, assign or donate the code to a third party and
to inhibit third parties from  using the code for non <MPEG standard> conforming
products. This copyright notice  must be included in all copies or derivative works.
Copyright (c) 1997.

Module Name:	sim.h

Abstract:	head file for FAP coding

Remarks:

*************************************************************************/

#ifndef __SIM_H_
#define __SIM_H_

#include <stdio.h>
#include <stdlib.h>
#include <limits.h>
#include "common.h"

/* XXX change this : do something cool, look in
 *     /usr/include/assert.h and change the logging  XXX
 */
#define DEBUG         1
#define LP            DEBUG, 1, stdout, __FILE__
#define INVALID_INT   INT_MAX

#define RANGE_DELTA	60
#define DEFAULTMIN -1000
#define DEFAULTMAX 1000
#define sign(x) 	((x) < 0 ? -1 : 1)

#define FDP_PARAMSET_MASK	0
#define	FAP_PARAMSET_MASK	1
#define BAP_PARAMSET_MASK	2
#define BDP_PARAMSET_MASK	3

#define FBA_OBJECT_START_CODE		442 /*0x100 + 0xBA*/
#define FBA_OBJECT_PLANE_START_CODE	443 /*0x100 + 0xBB*/
#define FBA_OBJECT_END_CODE		433 /*0x100 + 0xB1 visual object sequence end code*/

#define PREDICTIVE_CODING_TYPE		0
#define DCT_CODE_TYPE			1

#define MARKER_BIT                      1

#define NFAP                            68
#define N_FAP_GROUP                     10

#define SEG_LENGTH			16

#define MAX_BAPS          30        /* Max no of BAPs that belong to any group */
#define BAP_NUM_GROUPS    24        /* Total no of BAP groups - changed from 19 to 24 for extended baps - edp*/
#define NUM_BAPS          296       /* Total no of BAPs - changed to 296 from 186 for extended baps*/


/*
** Macros to choose one of the 8 types 
*/

#define B_GG  (fop.bo.bap_mask_base[is_global_type] &&   \
	       fop.bo.bap_mask_base[group_mask_type])
#define B_G   (fop.bo.bap_mask_base[is_global_type] &&   \
	       !fop.bo.bap_mask_base[group_mask_type])
#define B_JG  (!fop.bo.bap_mask_base[is_global_type] &&  \
	       fop.bo.bap_mask_base[group_mask_type] &&  \
	       fop.bo.bap_mask_base[spine_type])
#define B_JGC (!fop.bo.bap_mask_base[is_global_type] &&  \
	       fop.bo.bap_mask_base[group_mask_type] &&  \
	       !fop.bo.bap_mask_base[spine_type])
#define B_J   (!fop.bo.bap_mask_base[is_global_type] &&  \
	       !fop.bo.bap_mask_base[group_mask_type] && \
	       fop.bo.bap_mask_base[spine_type])
#define B_JC  (!fop.bo.bap_mask_base[is_global_type] &&  \
	       !fop.bo.bap_mask_base[group_mask_type] && \
	       !fop.bo.bap_mask_base[spine_type])
#define B_HG  (fop.bo.bap_mask_base[hand_type] &&        \
	       fop.bo.bap_mask_base[hand_mask_type])
#define B_H   (fop.bo.bap_mask_base[hand_type] &&        \
	       !fop.bo.bap_mask_base[hand_mask_type])
	       

/* index labels for BAP mask */
typedef enum { is_global_type, hand_type, highlevel_type, group_mask_type, 
	       hand_mask_type, spine_type, unused_1, unused_2 } bap_mask_index;


/*
** 
** FAP structure 
**
*/

typedef struct 
{
  int frame_num;
  int quant;
  bool_t is_time_code;
  bool_t is_frame_rate;
  bool_t skip_frames;
  int frame_rate;
  int fr_seconds;
  int fr_frequency_offset;
  int number_of_frames_to_skip;
  int time_code_hours;
  int time_code_minutes;
  int time_code_seconds;

  bool_t is_i_new_max;
  bool_t is_i_new_min;
  bool_t is_p_new_max;
  bool_t is_p_new_min;
  bool_t is_fap_quant;
  bool_t new_mask_flag;
  bool_t havesetminmax;
  int ve_orig[7];
  int ve_prev[7]; 
  int ve_diff[7];
  int viseme_def; 
  int init_face;
  int expression_def;
  int viseme_blend_Q; 
  int expr_intensity1_Q, expr_intensity2_Q;
  int *freq_ve[7];
  int *cumul_ve[7];
  int *Ifreq_ve[7];
  int *Icumul_ve[7];
  int Irange_ve_max[7];
  int Irange_ve_min[7];
  int range_ve_max[7];
  int range_ve_min[7];
  int *fap_mask, *fap_mask2, *prev_fap_mask;
  int fap_mask_type[N_FAP_GROUP];
  int range_max[NFAP], range_min[NFAP];
  int Irange_max[NFAP], Irange_min[NFAP];
  int *freq[NFAP], *cumul_freq[NFAP];
  int *Ifreq[NFAP], *Icumul_freq[NFAP];
  int i_new_max[NFAP];
  int i_new_min[NFAP];
  int p_new_max[NFAP];
  int p_new_min[NFAP];
  int minmaxset[NFAP];
  int fap_orig[NFAP], fap_Q[NFAP];
  int fap_prev[NFAP], fap_diff[NFAP];
  FILE *fap_out_fp;
  int mask_index[11];
  int suggested_gender;
  int coding_type;
/* DCT part */
  float *dctQP;
  float ** faps;
  int **visemes;
  int **expressions;
  float *dcPred; 
  int **dc_table;
  int **ac_table;
  int **runlength_table;
  int **v_sel_table;
  int **v_blend_table;
  int **e_sel_table;
  int **e_intensity_table;
  float **dctMatrix;
  float **idctMatrix;

} face_object_t;


/*
** 
** BAP Structure 
**
*/

typedef struct 
{
  int quant;
  int frame_num;
  bool_t is_time_code;
  bool_t is_frame_rate;
  bool_t skip_frames;
  int frame_rate;
  int fr_seconds;
  int fr_frequency_offset;
  int number_of_frames_to_skip;
  int time_code_hours;
  int time_code_minutes;
  int time_code_seconds;

  bool_t is_bap_quant;
  bool_t new_mask_flag;

  bool_t is_i_new_max;
  bool_t is_i_new_min;
  bool_t is_p_new_max;
  bool_t is_p_new_min;
  bool_t havesetminmax;

  int i_new_max[NUM_BAPS];
  int i_new_min[NUM_BAPS];
  int p_new_max[NUM_BAPS];
  int p_new_min[NUM_BAPS];
  int minmaxset[NUM_BAPS];
  int bap_orig[NUM_BAPS], bap_Q[NUM_BAPS];
  int bap_prev[NUM_BAPS], bap_diff[NUM_BAPS];

  int *bap_mask, *bap_mask2, *prev_bap_mask;
  int bap_mask_type[BAP_NUM_GROUPS];
  int range_max[NUM_BAPS], range_min[NUM_BAPS];
  int Irange_max[NUM_BAPS], Irange_min[NUM_BAPS];
  int *freq[NUM_BAPS], *cumul_freq[/*NUM_BAPS*/50000];
  int *Ifreq[NUM_BAPS], *Icumul_freq[/*NUM_BAPS*/50000];

  FILE *bap_out_fp;

  int bap_mask_index[BAP_NUM_GROUPS][MAX_BAPS];
  int baps_in_group[BAP_NUM_GROUPS];
  int suggested_gender;
  int coding_type;
  /* DCT part */
  float *dctQP;
  float ** baps;
  float *dcPred; 
  int **dc_table;
  int **ac_table;
  int **runlength_table;
  float **dctMatrix;
  float **idctMatrix;

} body_object_t;



/*
**
** FBA_OBJECT_PLANE structure 
**
*/

typedef struct 
{
  bool_t is_startcode;/*-edp*/
  int fba_paramset;
  bool_t is_intra;
  bool_t marker_bit;
  int cur_frame_bit_cnt;
  FILE *stream_fp;
  face_object_t fo;
  body_object_t bo;
} fba_object_plane_t;




/*
**
** global variables 
**
*/

extern fba_object_plane_t fop;


/* 
**
** function prototypes
**
*/


/* sac.c: function prototypes */
int AR_Encode(int, int*);
int decode_a_symbol(int*);
int bit_opp_bits(int);
int encoder_flush(void);
void decoder_reset(void); 
int bit_in_psc_layer(int);
void bit_out_psc_layer(void);
void update(int, int);
void update_viseme_expression(int, int);
void init_arith(void);

/* bapsac.c: function prototypes */
void bap_update(int, int);
void bap_init_arith(void);
int BAP_AR_Encode(int, int*);

/* putbits.c: function prototypes */
void initbits(void);
int putbits(int, int);
int getbits(int);
int getbit(void);
int next_start_code(void);

/* fba_object_plane.c: function prototypes */
void check_range(int symbol, int i, int inter);
void bap_check_range(int symbol, int i, int inter);
void fba_object_plane_header(void);
void fba_object_plane_data(void);
void decode_fba(void);
void code_frame_rate(void);
void decode_frame_rate(void);
void code_time_code(void);
void decode_time_code(void);
void code_skip_frames(void);
void decode_skip_frames(void);
void code_fap_group_mask(int);
void decode_fap_group_mask(int);
void code_new_minmax(void);
void decode_new_minmax(void);
void code_ifap(void);
void decode_ifap(void);
void code_pfap(void);
void decode_pfap(void);
int quant(int, int);
int dequant(int, int);
int pred(int, int);
int recon(int, int);
void code_i_segment(void); /* 3-3-98 */
void decode_i_segment(void); /* 3-3-98 */
void code_p_segment(void); /* 3-3-98 */
void decode_p_segment(void); /* 3-3-98 */

/* DCT related functions */
void read1Segment(int size, FILE *fapfile);
void read1bapSegment(int size, FILE *fapfile);
void write1Segment(int curFrame, FILE *fapfile);
void write1bapSegment(int curFrame, FILE *fapfile);
void init_dctQP();
void init_htables();
void init_dct();
void code_visemes_segment(int);
void code_expressions_segment(int);
void huffman_code(int symbol, int **table);
void dctTrans(float*, float*);
void idctTrans(float*, float*);
void dctquant(float*, int*,float);
void dctdequant(int*, float*,float);
void sendIDC(int);
void sendPDC(int);
void sendAC(int*);

void bap_code_frame_rate(void);
void bap_code_time_code(void);
void bap_code_skip_frames(void);
void bap_code_group_mask(int);
void code_ibap(void);
void code_pbap(void);
void bap_code_new_minmax(void);

void init_bap_dctQP();
void init_bap_htables();
void init_bap_dct();
void bap_dctTrans(float*, float*);
void bap_dctquant(float*, int*,float);
void bap_sendIDC(int);
void bap_sendAC(int*);
void bap_dctdequant(int*, float*,float);
void bap_idctTrans(float*, float*);
void bap_sendPDC(int);

#endif












