/******************************************************************************

This software module was originally developed by 
	Wei Wu (weiwu@stallion.risc.rockwell.com),  Rockwell Science Center 
in the course of development of the MPEG-4 Video (ISO/IEC 14496-2). 
This software module is an implementation of a part of one or more MPEG-4 Video
tools as specified by the MPEG-4 Video. ISO/IEC gives users of the MPEG-4 Video
free license to this software module or modifications thereof for use in 
hardware or software products claiming conformance to the MPEG-4 Video. Those 
intending to use this software module in hardware or software products are advised 
that its use may infringe existing patents. The original developer of this software 
module and his/her company, the subsequent editors and their companies, and ISO/IEC 
have no liability for use of this software module or modifications thereof in an 
implementation. Copyright is not released for non MPEG-4 Video conforming 
products. Rockwell Science Center retains full right to use the code for his/her own 
purpose, assign or donate the code to a third party and to inhibit third parties from 
using the code for non <MPEG standard> conforming products. This copyright notice 
must be included in all copies or derivative works. Copyright (c) 1997.
 
******************************************************************************/

#include <stdio.h>
#include <stdlib.h>

#define MAX32			4294967296.0
#define MAX16			65536
#define VDO_START_CODE		0x1BF
#define VDOP_START_CODE		0x1BD
#define MARKER			1
#define RWIN_WIDTH		512
#define RWIN_HEIGHT		512

FILE *streamfile;

main(int argc, char *argv[])
{
  int i;
  int nframes=20;
  int FOV;
  float vx, vy, vz, ax, ay, az;
  int ivalue;
  char prefix[100], ofname[100];
  FILE *ifp;
  void initbits();
  void putbits(int, int);
  int alignbits();
  unsigned int norm(float);

  if(argc!=5) {
	fprintf(stderr, "Usage: %s inpath_file nframes FOV outbit_file\n", argv[0]);
	exit(-1);
  }

  ifp = fopen(argv[1], "r");
  if(ifp==NULL) {
	fprintf(stderr, "Unable to open path file\n"); 
	exit(-1);
  }
  nframes = atoi(argv[2]);
  FOV = atoi(argv[3]);
  streamfile = fopen(argv[4], "wb");

  initbits();

  putbits(32, VDO_START_CODE);
  putbits(16, FOV);
  putbits(1, MARKER);
  putbits(16, RWIN_WIDTH);
  putbits(1, MARKER);
  putbits(16, RWIN_HEIGHT);
  putbits(1, MARKER);
  alignbits();
  for(i=0; i<nframes; i++) {
  	putbits(32, VDOP_START_CODE);
  	fscanf(ifp, "%f%f%f%f%f%f", &vx, &vy, &vz, &ax, &ay, &az);
	ivalue = norm(vx);
	putbits(16, ivalue&0xFFFF);
	putbits(MARKER, MARKER);
	putbits(16, ivalue>>16);
	putbits(MARKER, MARKER);

	ivalue = norm(vy);
	putbits(16, ivalue&0xFFFF);
	putbits(MARKER, MARKER);
	putbits(16, ivalue>>16);
	putbits(MARKER, MARKER);

	ivalue = norm(vz);
	putbits(16, ivalue&0xFFFF);
	putbits(MARKER, MARKER);
	putbits(16, ivalue>>16);
	putbits(MARKER, MARKER);

	ivalue = norm(ax);
	putbits(16, ivalue&0xFFFF);
	putbits(MARKER, MARKER);
	putbits(16, ivalue>>16);
	putbits(MARKER, MARKER);

	ivalue = norm(ay);
	putbits(16, ivalue&0xFFFF);
	putbits(MARKER, MARKER);
	putbits(16, ivalue>>16);
	putbits(MARKER, MARKER);

	ivalue = norm(az);
	putbits(16, ivalue&0xFFFF);
	putbits(MARKER, MARKER);
	putbits(16, ivalue>>16);
	putbits(MARKER, MARKER);

	alignbits();
  }

  fclose(ifp);
  fclose(streamfile);
}

	
#define VMIN	-100.0
#define VMAX	100.0

unsigned int
norm(float fvalue)
{

double dtmp;

dtmp = ((fvalue-VMIN)/(VMAX-VMIN))*(MAX32-1);

return (unsigned int) dtmp;
}


#if 0
void
putbits (int n, int val)
{
  int i;
  unsigned int mask;
 
  mask = 1 << (n-1); 
 
  for (i=0; i<n; i++) {
 
    	if (val & mask)
		fprintf(ofp, "%c", '1');
	else
		fprintf(ofp, "%c", '0');
 
    	mask >>= 1; /* select next bit */
  }
}
#endif
