/*****************************************************************************

This software module was originally developed by 
	Karl Olav Lillevold (Karl.Lillevold@nta.no), Telenor
	Robert Danielsen (Robert.Danielsen@nta.no) Telenor
	Wei Wu (weiwu@stallion.risc.rockwell.com),  Rockwell Science Center 
in the course of development of the MPEG-4 Video (ISO/IEC 14496-2). 
This software module is an implementation of a part of one or more MPEG-4 Video
tools as specified by the MPEG-4 Video. ISO/IEC gives users of the MPEG-4 Video
free license to this software module or modifications thereof for use in 
hardware or software products claiming conformance to the MPEG-4 Video. Those 
intending to use this software module in hardware or software products are advised 
that its use may infringe existing patents. The original developer of this software 
module and his/her company, the subsequent editors and their companies, and ISO/IEC 
have no liability for use of this software module or modifications thereof in an 
implementation. Copyright is not released for non MPEG-4 Video conforming 
products. Telenor and Rockwell Science Center retains full right to use the code
for his/her own purpose, assign or donate the code to a third party and to 
inhibit third parties from using the code for non <MPEG standard> conforming 
products. This copyright notice must be included in all copies or derivative 
works. Copyright (c) 1997.
 
*****************************************************************************/


#include <stdio.h>
#include <stdlib.h>

extern FILE *streamfile; /* the only global var we need here */

/* private data */
static unsigned char outbfr;
static int outcnt;
static int bytecnt;

/* initialize buffer, call once before first putbits or alignbits */
void
initbits()
{
  outcnt = 8;
  bytecnt = 0;
}

/* write rightmost n (0<=n<=32) bits of val to outfile */

void
putbits (int n, int val)
{
  int i;
  unsigned int mask;
  char bitstring[32];

/*
  if (trace) {
    if (n > 0) {
      BitPrint(n,val,bitstring);
      fprintf(tf,bitstring);
    }
  }
*/

  mask = 1 << (n-1); /* selects first (leftmost) bit */

  for (i=0; i<n; i++) {
    outbfr <<= 1;

    if (val & mask)
      outbfr|= 1;

    mask >>= 1; /* select next bit */
    outcnt--;

    if (outcnt==0) /* 8 bit buffer full */ {
      putc(outbfr,streamfile);
      outcnt = 8;
      bytecnt++;
    }
  }
}


/* zero bit stuffing to next byte boundary (5.2.3, 6.2.1) */

int
alignbits ()
{
  int ret_value;
  
  if (outcnt!=8) {
    ret_value = outcnt;	/* outcnt is reset in call to putbits () */
    putbits (outcnt, 0);
    return ret_value;
  }
  else
    return 0;
}

int
next_start_code ()
{
  int mask;
  int ret;
  
  if (outcnt==1) {
    putbits(1, 0);
    return 1;
  } else if(outcnt==8) {
    return 0;
  } else {
    mask = (1<<(outcnt-1))-1;
    ret = outcnt;
    putbits (outcnt, mask);
    return ret;
  }
}

/* return total number of generated bits */
int
bitcount()
{
  return 8*bytecnt + (8-outcnt);
}

/* convert to binary number */
void 
BitPrint(int length, int val, char *bit)
{
  int m;
  
  m = length;
  bit[0] = '"';
  while (m--) {
    bit[length-m] = (val & (1<<m)) ? '1' : '0';
  }
  bit[length+1] = '"';
  bit[length+2] = '\n';
  bit[length+3] = '\0';
  return;
}

