/*************************************************************************

This software module was originally developed by 

	Ming-Chieh Lee (mingcl@microsoft.com), Microsoft Corporation
	Wei-ge Chen (wchen@microsoft.com), Microsoft Corporation
	Bruce Lin (blin@microsoft.com), Microsoft Corporation
	Chuang Gu (chuanggu@microsoft.com), Microsoft Corporation
	(date: March, 1996)

in the course of development of the MPEG-4 Video (ISO/IEC 14496-2). 
This software module is an implementation of a part of one or more MPEG-4 Video tools 
as specified by the MPEG-4 Video. 
ISO/IEC gives users of the MPEG-4 Video free license to this software module or modifications 
thereof for use in hardware or software products claiming conformance to the MPEG-4 Video. 
Those intending to use this software module in hardware or software products are advised that its use may infringe existing patents. 
The original developer of this software module and his/her company, 
the subsequent editors and their companies, 
and ISO/IEC have no liability for use of this software module or modifications thereof in an implementation. 
Copyright is not released for non MPEG-4 Video conforming products. 
Microsoft retains full right to use the code for his/her own purpose, 
assign or donate the code to a third party and to inhibit third parties from using the code for non <MPEG standard> conforming products. 
This copyright notice must be included in all copies or derivative works. 

Copyright (c) 1996, 1997.

Module Name:

	decoder.cpp

Abstract:

	caller for decoder

Revision History:

*************************************************************************/

#include <stdio.h>
#include <stdlib.h>
#include <iostream.h> // VDS
#include <fstream.h>

#ifdef __PC_COMPILER_
#include <windows.h>
#include <mmsystem.h>
#endif // __PC_COMPILER_

#include "typeapi.h"
#include "mode.h"
#include "vopses.h"
#include "decoder/vopsedec.h"
#ifndef __GLOBAL_VAR_
#define __GLOBAL_VAR_
#endif

#include "global.h"

#include "vd_grid.h" // VDS
#include "vd_vector.h"

#ifdef __MFC_
#ifdef _DEBUG
#undef THIS_FILE
static char BASED_CODE THIS_FILE[] = __FILE__;
#endif

#define new DEBUG_NEW				   
#endif // __MFC_

#define HELP_MESSAGE "Usage: DECODER bitstream_file [enhn_layer_bitstream_file] output_file width height [path_file mesh_file]" // VDS


Void main (Int argc, Char* argv[])
{
	Bool bScalability = FALSE;
	Bool bVdscalability = FALSE; // VDS
	if (argc < 5) {       
		printf ("%s\n", HELP_MESSAGE);
		return;
	}
	else if (argc == 6) // VDS
		bScalability = TRUE;
	else if (argc == 7) // VDS
		bVdscalability = TRUE;
	assert((bScalability&bVdscalability)==0);

	CRct rctDisplay (0, 0, atoi (argv [3 + bScalability]), atoi (argv [4 + bScalability]));
	FILE *pfBits;
	if ((pfBits = fopen (argv [1], "rb")) == NULL ){
		printf ("Bitstream File Not Found\n");
		exit (1);
	}
	fclose (pfBits);
	if (bScalability == TRUE)	{
		if ((pfBits = fopen (argv [2], "rb")) == NULL )	{
			printf ("Bitstream File for Enhancement Layer Not Found\n");
			exit (1);
		}
	}
	fclose (pfBits);

	FILE *pf; // VDS
	CGrid *pgrid;
	ifstream *pfPath;
	if (bVdscalability == TRUE) {
		if ((pf = fopen (argv [5], "r")) == NULL )	{
			printf ("Path File for View Dependent Object Not Found\n");
			exit (1);
		}
		fclose(pf);
		pfPath = new ifstream(argv[5], ios::in);
		if ((pf = fopen (argv [6], "r")) == NULL )	{
			printf ("Mesh File for View Dependent Object Not Found\n");
			exit (1);
		}
		fclose(pf);
		pgrid = new CGrid(argv[6]);
	}


#define BASE_LAYER 0
#define ENHN_LAYER 1

	CVideoObjectDecoder* pvodec [2];
	CRct rctDisplayBackup = rctDisplay; // for recovery from sprite decoding
	pvodec [BASE_LAYER] = new CVideoObjectDecoder (argv [1], rctDisplay.width, rctDisplay.height(), pgrid); // VDS
	if (pvodec [BASE_LAYER] -> fSptUsage () == 1) {
			//pvodec [BASE_LAYER] -> setRctDisp (rctDisplayBackup);
			// decode the initial sprite
			pvodec [BASE_LAYER] -> decode ();
			if (pvodec [BASE_LAYER]->volmd ().fAUsage != RECTANGLE) {
				// change m_pvopcRefQ1 to m_pvopcSptQ for warping
				pvodec [BASE_LAYER] -> swapRefQ1toSpt ();
				// restore m_pvopcCurrQ size to the normal one
				pvodec [BASE_LAYER] ->  changeSizeofCurrQ (rctDisplayBackup);
			}
	}

	if (bScalability)
		pvodec [ENHN_LAYER] = new CVideoObjectDecoder (argv [2], rctDisplay.width, rctDisplay.height());
	FILE* pfReconYUV [2];
	Char pchTmp [100];
	sprintf (pchTmp, "%s.yuv", argv [2 + bScalability]);
	pfReconYUV [BASE_LAYER] = fopen (pchTmp, "wb");
	assert (pfReconYUV [BASE_LAYER] != NULL);
	if (bScalability)	{
		sprintf (pchTmp, "%s_e.yuv", argv [2 + bScalability]);
		pfReconYUV [ENHN_LAYER]= fopen (pchTmp, "wb");
		assert (pfReconYUV [ENHN_LAYER] != NULL);
	}
	sprintf (pchTmp, "%s.seg", argv [2 + bScalability]);
	FILE* pfReconSeg = NULL;
	if (pvodec [BASE_LAYER]-> volmd ().fAUsage != RECTANGLE)	{
		pfReconSeg = fopen (pchTmp, "wb");
		assert (pfReconSeg != NULL);
	}
	Int iEof = 1;
#ifdef __PC_COMPILER_
	Int tickBegin = ::GetTickCount ();
#endif // __PC_COMPILER_
	Int nFrames = 0;
	while (iEof != EOF) {
		vector ViewPos, AimPos; // VDS
		if(bVdscalability) {
			float fxview, fyview, fzview, fxaim, fyaim, fzaim;
			(*pfPath) >> fxview >> fyview >> fzview;
			(*pfPath) >> fxaim >> fyaim >> fzaim;
			ViewPos = vector(fxview, fyview, fzview);
			AimPos = vector(fxaim, fyaim, fzaim);
		}
			
		for (Int iLayer = BASE_LAYER; iLayer <= bScalability * ENHN_LAYER; iLayer++)	{
			const CVOPU8YUVBA* pvopcBaseQuant = (iLayer == ENHN_LAYER) ? 
													pvodec [BASE_LAYER]->pvopcRefQLater () : (CVOPU8YUVBA*) NULL;
		    if(bVdscalability==FALSE) { // VDS
			iEof = (pvodec [BASE_LAYER]->fSptUsage () == 1) ? pvodec [BASE_LAYER] -> decodeSpt () : pvodec [iLayer] -> decode ();
		    } else { // VDS
			assert(pvodec [BASE_LAYER]->fSptUsage () == 0);
			iEof = pvodec [iLayer] -> decode (ViewPos, AimPos);

		    }
//							   pvodec [iLayer] -> decode (pvopfBaseQuant);
//			iEof =  pvodec [iLayer] -> decode (pvopcBaseQuant);
			if (iEof != EOF && iEof!=FALSE) {
				const CVOPU8YUVBA* pvopcQuant = 
//					(uiSprite == 1) ? pvodec [BASE_LAYER]->pvopfQuantCurr () -> pvopYUV () : 
								pvodec [iLayer]->pvopcReconCurr ();
//				pvopFilteredQuant -> where (rctDisplay);
//				pvopFilteredQuant -> multiplyBiAlpha ();

				pvopcQuant->getPlane (Y_PLANE)->dump (pfReconYUV [iLayer], rctDisplay);
				pvopcQuant->getPlane (U_PLANE)->dump (pfReconYUV [iLayer], rctDisplay / 2);
				pvopcQuant->getPlane (V_PLANE)->dump (pfReconYUV [iLayer], rctDisplay / 2);
				if (pvodec [BASE_LAYER]->volmd ().fAUsage != RECTANGLE)
					pvopcQuant->getPlane (BY_PLANE)->dump (pfReconSeg, rctDisplay);
			}
		}
#ifdef __PC_COMPILER_
		nFrames++;
#endif // __PC_COMPILER_
	}
#ifdef __PC_COMPILER_
	nFrames--;
	Int tickAfter = ::GetTickCount ();
	printf ("Total time: %d\n", tickAfter - tickBegin);
	Double dAverage = (Double) (tickAfter - tickBegin) / (Double) nFrames;
	printf ("Total frame %d\tAverage time: %.6lf\n", nFrames, dAverage);
	printf ("FPS %.6lf\n", 1000.0 / dAverage);
#endif // __PC_COMPILER_

	fclose (pfReconYUV [BASE_LAYER]);
	if (pvodec [BASE_LAYER]->volmd ().fAUsage != RECTANGLE)
		fclose (pfReconSeg);
	delete pvodec [BASE_LAYER];
	if (bScalability)	{
		fclose (pfReconYUV [ENHN_LAYER]);
		delete pvodec [ENHN_LAYER];
	}
	delete pfPath;

}
