/*************************************************************************

This software module was originally developed by 

	Ming-Chieh Lee (mingcl@microsoft.com), Microsoft Corporation
	Wei-ge Chen (wchen@microsoft.com), Microsoft Corporation
	Bruce Lin (blin@microsoft.com), Microsoft Corporation
	Chuang Gu (chuanggu@microsoft.com), Microsoft Corporation
	(date: March, 1996)

in the course of development of the MPEG-4 Video (ISO/IEC 14496-2). 
This software module is an implementation of a part of one or more MPEG-4 Video tools 
as specified by the MPEG-4 Video. 
ISO/IEC gives users of the MPEG-4 Video free license to this software module or modifications 
thereof for use in hardware or software products claiming conformance to the MPEG-4 Video. 
Those intending to use this software module in hardware or software products are advised that its use may infringe existing patents. 
The original developer of this software module and his/her company, 
the subsequent editors and their companies, 
and ISO/IEC have no liability for use of this software module or modifications thereof in an implementation. 
Copyright is not released for non MPEG-4 Video conforming products. 
Microsoft retains full right to use the code for his/her own purpose, 
assign or donate the code to a third party and to inhibit third parties from using the code for non <MPEG standard> conforming products. 
This copyright notice must be included in all copies or derivative works. 

Copyright (c) 1996, 1997.

Module Name:

	encoder.cpp

Abstract:

	caller for encoder

Revision History:

*************************************************************************/

#include <stdio.h>
#include <stdlib.h>

#ifdef __PC_COMPILER_
#include <windows.h>
#include <mmsystem.h>
#endif // __PC_COMPILER_

#include "typeapi.h"
#include "codehead.h"
#include "mode.h"
#include "fstream.h"
#include "encoder/sesenc.h"

#ifndef __GLOBAL_VAR_
#define __GLOBAL_VAR_
#endif
#include "global.h"

#ifdef __MFC_
#ifdef _DEBUG
#undef THIS_FILE
static char BASED_CODE THIS_FILE[] = __FILE__;
#endif

#define new DEBUG_NEW				   
#endif // __MFC_

Int rgiDefaultIntraQMatrixAlpha [BLOCK_SQUARE_SIZE] = {
	16, 16, 16, 16, 16, 16, 16, 16,
	16, 16, 16, 16, 16, 16, 16, 16,
	16, 16, 16, 16, 16, 16, 16, 16,
	16, 16, 16, 16, 16, 16, 16, 16,
	16, 16, 16, 16, 16, 16, 16, 16,
	16, 16, 16, 16, 16, 16, 16, 16,
	16, 16, 16, 16, 16, 16, 16, 16,
	16, 16, 16, 16, 16, 16, 16, 16
};
Int rgiDefaultInterQMatrixAlpha [BLOCK_SQUARE_SIZE] = {
	16, 16, 16, 16, 16, 16, 16, 16,
	16, 16, 16, 16, 16, 16, 16, 16,
	16, 16, 16, 16, 16, 16, 16, 16,
	16, 16, 16, 16, 16, 16, 16, 16,
	16, 16, 16, 16, 16, 16, 16, 16,
	16, 16, 16, 16, 16, 16, 16, 16,
	16, 16, 16, 16, 16, 16, 16, 16,
	16, 16, 16, 16, 16, 16, 16, 16
};

Void nextValidLine (FILE *pfPara, UInt* pnLine);
Int bPrint = 0;
Void main (Int argc, Char* argv[])
{
	UInt nLine = 1;
	UInt* pnLine = &nLine;
	FILE *pfPara;

	if (argc != 2)	{
		printf ("Usage: %s Parameter_File\n", argv [0]);
		exit (1);
	}

	if ((pfPara = fopen (argv[1], "r")) == NULL ){
		printf (" Parameter File Not Found \n");
		exit (1);
	}

	// verify version number
	UInt iVersion;
	nextValidLine (pfPara, pnLine);
	if ( fscanf (pfPara, "%u", &iVersion) != 1 )	{
		fprintf(stderr, "wrong parameter file format on line %d\n", *pnLine);
		assert (FALSE);
	}
	if (iVersion != 804)	{
		fprintf(stderr, "wrong parameter file version for VM8.0.4\n");
		assert (FALSE); // please update version number when you
						// make changes in the following
	}

	// VDS: view-dependent scalability enable
	Bool bvds;
	nextValidLine (pfPara, pnLine);
	if (fscanf (pfPara, "%d", &bvds) != 1)  {
		fprintf(stderr, "wrong parameter file format on line %d\n", *pnLine);
		assert (FALSE);
	}
	assert(bvds==1 || bvds==0);

	// frame size code
	UInt uiFrmWidth, uiFrmHeight;		
	nextValidLine (pfPara, pnLine);
	if (fscanf (pfPara, "%u", &uiFrmWidth) != 1)	{
		fprintf(stderr, "wrong parameter file format on line %d\n", *pnLine);
		assert (FALSE);
	}
	nextValidLine (pfPara, pnLine);
	if (fscanf (pfPara, "%u", &uiFrmHeight) != 1)	{
		fprintf(stderr, "wrong parameter file format on line %d\n", *pnLine);
		assert (FALSE);
	}

	// first and last frame number
	UInt firstFrm, lastFrm;
	nextValidLine (pfPara, pnLine);
	if ( fscanf (pfPara, "%u", &firstFrm) != 1) {
		fprintf(stderr, "wrong parameter file format on line %d\n", *pnLine);
		assert (FALSE);
	}
	nextValidLine (pfPara, pnLine);
	if ( fscanf (pfPara, "%u", &lastFrm) != 1) {
		fprintf(stderr, "wrong parameter file format on line %d\n", *pnLine);
		assert (FALSE);
	}
	assert (lastFrm >= firstFrm);

	// object indexes
	UInt firstVO, lastVO;
	nextValidLine (pfPara, pnLine);
	if ( fscanf (pfPara, "%u", &firstVO) != 1)	{
		fprintf(stderr, "wrong parameter file format on line %d\n", *pnLine);
		assert (FALSE);
	}
	nextValidLine (pfPara, pnLine);
	if ( fscanf (pfPara, "%u", &lastVO) != 1)	{
		fprintf(stderr, "wrong parameter file format on line %d\n", *pnLine);
		assert (FALSE);
	}
	assert (lastVO >= firstVO);
	UInt nVO = lastVO - firstVO + 1;

	UInt iObj;
	// spatial scalability indicators
	nextValidLine (pfPara, pnLine);
	Bool* rgbSpatialScalability = new Bool [nVO];
	Bool bAnyScalability = FALSE;
	for (iObj = 0; iObj < nVO; iObj++)	{
		if (fscanf (pfPara, "%d", &rgbSpatialScalability [iObj]) != 1)	{
			fprintf(stderr, "wrong parameter file format on line %d\n", *pnLine);
			exit (1);
		}
		if (rgbSpatialScalability[iObj] == 1)
			bAnyScalability = TRUE;
		else
			assert (rgbSpatialScalability[iObj] == 0);
	}

	// enhancement layer upsample factor
	nextValidLine (pfPara, pnLine);
	Bool* rgiEnhnLayerUpsampleFactor = new Int [nVO];
	for (iObj = 0; iObj < nVO; iObj++)	{
		if (fscanf (pfPara, "%d", &rgiEnhnLayerUpsampleFactor [iObj]) != 1)	{
			fprintf(stderr, "wrong parameter file format on line %d\n", *pnLine);
			exit (1);
		}
		if (rgbSpatialScalability [iObj] == TRUE)
			assert (rgiEnhnLayerUpsampleFactor [iObj] == 1 || rgiEnhnLayerUpsampleFactor [iObj] == 2);	//only two cases are supported at this time
	}

#define BASE_LAYER 0
#define ENHN_LAYER 1

	// rate control flag
	nextValidLine (pfPara, pnLine);
	Bool* rgbRateControl [2];
	rgbRateControl [BASE_LAYER] = new Bool [nVO];
	for (iObj = 0; iObj < nVO; iObj++)	{
		if (fscanf (pfPara, "%d", &rgbRateControl [BASE_LAYER] [iObj] ) != 1)	{
			fprintf(stderr, "wrong parameter file format on line %d\n", *pnLine);
			exit (1);
		}
		assert (rgbRateControl [BASE_LAYER] [iObj]  == 0 || 
				rgbRateControl [BASE_LAYER] [iObj]  == 1);
	}
	if (bAnyScalability)	{
		rgbRateControl [ENHN_LAYER] = new Bool [nVO];	
		for (iObj = 0; iObj < nVO; iObj++)	{
			if (fscanf (pfPara, "%d", &rgbRateControl [ENHN_LAYER] [iObj]) != 1)	{
				fprintf(stderr, "wrong parameter file format on line %d\n", *pnLine);
				exit (1);
			}
			assert (rgbRateControl [ENHN_LAYER] [iObj]  == 0 || 
					rgbRateControl [ENHN_LAYER] [iObj]  == 1);
		}
	}

	// bit budget for each object.
	nextValidLine (pfPara, pnLine);
	UInt* rguiBitsBudget [2];
	rguiBitsBudget [BASE_LAYER] = new UInt [nVO];
	for (iObj = 0; iObj < nVO; iObj++)	{
		if (fscanf (pfPara, "%d", &rguiBitsBudget [BASE_LAYER] [iObj]) != 1)	{
			fprintf(stderr, "wrong parameter file format on line %d\n", *pnLine);
			exit (1);
		}
	}
	if (bAnyScalability)	{
		rguiBitsBudget [ENHN_LAYER] = new UInt [nVO];	
		for (iObj = 0; iObj < nVO; iObj++)	{
			if (fscanf (pfPara, "%d", &rguiBitsBudget [ENHN_LAYER] [iObj]) != 1)	{
				fprintf(stderr, "wrong parameter file format on line %d\n", *pnLine);
				exit (1);
			}
			assert (rguiBitsBudget [ENHN_LAYER] [iObj]  > 0);
		}
	}


	// alpha usage for each object.  0: rectangle, 1: binary, 2: 8-bit, 3: shape only
	nextValidLine (pfPara, pnLine);
	AlphaUsage* rgfAlphaUsage = new AlphaUsage [nVO];
	Bool* rgbShapeOnly = new Bool [nVO];
	for (iObj = 0; iObj < nVO; iObj++)	{
		UInt uiAlpha;
		if (fscanf (pfPara, "%d", &uiAlpha) != 1)	{
			fprintf(stderr, "wrong parameter file format on line %d\n", *pnLine);
			exit (1);
		}
		else {
			assert (uiAlpha == 0 || uiAlpha == 1 || uiAlpha == 2 || uiAlpha == 3);
			if(uiAlpha<3)
			{
				rgfAlphaUsage [iObj] = (AlphaUsage) uiAlpha;
				rgbShapeOnly [iObj] = FALSE;
			}
			else
			{
				rgfAlphaUsage [iObj] = ONE_BIT;
				rgbShapeOnly [iObj] = TRUE;
			}
		}
	}

	// binary shape rounding para
	nextValidLine (pfPara, pnLine);
	Int* rgiBinaryAlphaTH = new Int [nVO];
	for (iObj = 0; iObj < nVO; iObj++)	{
		if (fscanf (pfPara, "%d", &rgiBinaryAlphaTH [iObj]) != 1)	{
			fprintf	(stderr, "wrong parameter file format on line %d\n", *pnLine);
			exit (1);
		}
		assert (rgiBinaryAlphaTH [iObj] >= 0);
	}

	// binary shape size conversion flag
	nextValidLine (pfPara, pnLine);
	Int* rgbNoCrChange = new Bool [nVO];
	for (iObj = 0; iObj < nVO; iObj++)	{
		if (fscanf (pfPara, "%d", &rgbNoCrChange [iObj]) != 1)	{
			fprintf	(stderr, "wrong parameter file format on line %d\n", *pnLine);
			exit (1);
		}
		assert (rgbNoCrChange [iObj] == TRUE || rgbNoCrChange [iObj] == FALSE); //boolean value
		if (rgiBinaryAlphaTH [iObj] == 0)
			assert (rgbNoCrChange [iObj] == TRUE); //MB-level size conversion of shape is off in lossless mode\n");
	}

	// coding mode for each object
	// Separate motion-texture coding
	nextValidLine (pfPara, pnLine);
	Bool* rgbSeprMotionTexture [2];
	rgbSeprMotionTexture [BASE_LAYER] = new Bool [nVO];
	for (iObj = 0; iObj < nVO; iObj++)	{
		if (fscanf (pfPara, "%d", &(rgbSeprMotionTexture [BASE_LAYER] [iObj])) != 1)	{
			fprintf (stderr, "wrong parameter file format on line %d\n", *pnLine);
			exit (1);
		}
		assert (rgbSeprMotionTexture [BASE_LAYER] [iObj] == 0 || 
				rgbSeprMotionTexture [BASE_LAYER] [iObj] == 1);
	}
	if (bAnyScalability)	{
		rgbSeprMotionTexture [ENHN_LAYER] = new Bool [nVO];	
		for (iObj = 0; iObj < nVO; iObj++)	{
			if (fscanf (pfPara, "%d", &rgbSeprMotionTexture [ENHN_LAYER] [iObj]) != 1)	{
				fprintf(stderr, "wrong parameter file format on line %d\n", *pnLine);
				exit (1);
			}
			assert (rgbSeprMotionTexture [ENHN_LAYER] [iObj]  == 0 || 
					rgbSeprMotionTexture [ENHN_LAYER] [iObj]  == 1);
		}
	}

	// error resilient coding disable
	nextValidLine (pfPara, pnLine);
	Bool* rgbErrorResilientDisable [2];
	rgbErrorResilientDisable [BASE_LAYER] = new Bool [nVO];
	Bool* rgbDataPartitioning [2];
	rgbDataPartitioning [BASE_LAYER] = new Bool [nVO];
	Bool* rgbReversibleVlc [2];
	rgbReversibleVlc [BASE_LAYER] = new Bool [nVO];
	Int iErrorResilientFlags;
	for (iObj = 0; iObj < nVO; iObj++)	{
		if (fscanf (pfPara, "%d", &iErrorResilientFlags) != 1)	{
			fprintf(stderr, "wrong parameter file format on line %d\n", *pnLine);
			exit (1);
		}
		rgbErrorResilientDisable [BASE_LAYER] [iObj] = iErrorResilientFlags & 0x1;
		rgbDataPartitioning [BASE_LAYER] [iObj]		 = iErrorResilientFlags & 0x2;
		rgbReversibleVlc [BASE_LAYER] [iObj]		 = iErrorResilientFlags & 0x4;
	}
	if (bAnyScalability)	{
		rgbErrorResilientDisable  [ENHN_LAYER] = new Bool [nVO];	
		for (iObj = 0; iObj < nVO; iObj++)	{
			if (fscanf (pfPara, "%d", &rgbErrorResilientDisable [ENHN_LAYER] [iObj]) != 1)	{
				fprintf(stderr, "wrong parameter file format on line %d\n", *pnLine);
				exit (1);
			}
			assert (rgbErrorResilientDisable [ENHN_LAYER] [iObj] == TRUE); //error resilient coding not supported as for now
		}
	}

	// Bit threshold for video packet spacing control
	nextValidLine (pfPara, pnLine);
	Bool* rgbVPBitTh [2];
	rgbVPBitTh [BASE_LAYER] = new Bool [nVO];
	for (iObj = 0; iObj < nVO; iObj++)	{
		if (fscanf (pfPara, "%d", &rgbVPBitTh [BASE_LAYER] [iObj]) != 1)	{
			fprintf(stderr, "wrong parameter file format on line %d\n", *pnLine);
			exit (1);
		}
		if (!rgbErrorResilientDisable [BASE_LAYER] [iObj])
			assert (rgbVPBitTh [BASE_LAYER] [iObj] > 0);
	}
	if (bAnyScalability)	{
		rgbVPBitTh  [ENHN_LAYER] = new Bool [nVO];	
		for (iObj = 0; iObj < nVO; iObj++)	{
			if (fscanf (pfPara, "%d", &rgbVPBitTh [ENHN_LAYER] [iObj]) != 1)	{
				fprintf(stderr, "wrong parameter file format on line %d\n", *pnLine);
				exit (1);
			}
			assert (rgbVPBitTh [ENHN_LAYER] [iObj] >= 0);
		}
	}

	// Interlaced coding
	nextValidLine (pfPara, pnLine);
	Bool* rgbInterlacedCoding [2];
	rgbInterlacedCoding [BASE_LAYER] = new Bool [nVO];
	for (iObj = 0; iObj < nVO; iObj++)	{
		if (fscanf (pfPara, "%d", &rgbInterlacedCoding [BASE_LAYER] [iObj]) != 1)	{
			fprintf(stderr, "wrong parameter file format on line %d\n", *pnLine);
			exit (1);
		}
		assert (rgbInterlacedCoding [BASE_LAYER] [iObj] == 0 || rgbInterlacedCoding [BASE_LAYER] [iObj] == 1);
	}
	if (bAnyScalability)	{
		rgbInterlacedCoding [ENHN_LAYER] = new Bool [nVO];	
		for (iObj = 0; iObj < nVO; iObj++)	{
			if (fscanf (pfPara, "%d", &rgbInterlacedCoding [ENHN_LAYER] [iObj]) != 1)	{
				fprintf(stderr, "wrong parameter file format on line %d\n", *pnLine);
				exit (1);
			}
			assert (rgbInterlacedCoding [BASE_LAYER] [iObj] == 0 || rgbInterlacedCoding [BASE_LAYER] [iObj] == 1);
		}
	}
	
	// quantizer selection: 0 -- H.263, 1 -- MPEG
	nextValidLine (pfPara, pnLine);
	Quantizer* rgfQuant [2];
	rgfQuant [BASE_LAYER] = new Quantizer [nVO]; 
	for (iObj = 0; iObj < nVO; iObj++)	{
		if (fscanf (pfPara, "%d", &(rgfQuant [BASE_LAYER] [iObj])) != 1)	{
			fprintf(stderr, "wrong parameter file format on line %d\n", *pnLine);
			exit (1);
		}
		assert (rgfQuant [BASE_LAYER] [iObj] ==0 || rgfQuant [BASE_LAYER] [iObj] == 1);
	}
	if (bAnyScalability)	{
		rgfQuant [ENHN_LAYER] = new Quantizer [nVO]; 
		for (iObj = 0; iObj < nVO; iObj++)	{
			if (fscanf (pfPara, "%d", &(rgfQuant [ENHN_LAYER] [iObj])) != 1)	{
				fprintf(stderr, "wrong parameter file format on line %d\n", *pnLine);
				exit (1);
			}
			assert (rgfQuant [BASE_LAYER] [iObj] ==0 || rgfQuant [ENHN_LAYER] [iObj] == 1);
		}
	}

	// load non-default intra Q-Matrix, 0 -- FALSE, 1 -- TRUE
	nextValidLine (pfPara, pnLine);
	Bool* rgbLoadIntraMatrix [2]; 
	rgbLoadIntraMatrix [BASE_LAYER] = new Bool [nVO];
	Int** rgppiIntraQuantizerMatrix [2];
	rgppiIntraQuantizerMatrix [BASE_LAYER] = new Int* [nVO];
	for (iObj = 0; iObj < nVO; iObj++)	{
		rgppiIntraQuantizerMatrix [BASE_LAYER] [iObj] = new Int [BLOCK_SQUARE_SIZE];
		if (fscanf (pfPara, "%d", &rgbLoadIntraMatrix [BASE_LAYER] [iObj]) != 1)	{
			fprintf(stderr, "wrong parameter file format on line %d\n", *pnLine);
			exit (1);
		}
		assert (rgbLoadIntraMatrix [BASE_LAYER] [iObj] ==0 || 
				rgbLoadIntraMatrix [BASE_LAYER] [iObj] == 1);
		if (rgbLoadIntraMatrix [BASE_LAYER] [iObj]) {
			UInt i = 0;
			do {
				if (fscanf (pfPara, "%d", &rgppiIntraQuantizerMatrix [BASE_LAYER] [iObj] [i]) != 1)	{
					fprintf(stderr, "wrong parameter file format on line %d\n", *pnLine);
					exit (1);
				}
			} while (rgppiIntraQuantizerMatrix [BASE_LAYER] [iObj] [i] != 0 && ++i < BLOCK_SQUARE_SIZE);
		}
		else
			memcpy (rgppiIntraQuantizerMatrix [BASE_LAYER] [iObj], rgiDefaultIntraQMatrix, BLOCK_SQUARE_SIZE * sizeof (Int));
	}
	if (bAnyScalability)	{
		rgbLoadIntraMatrix [ENHN_LAYER] = new Bool [nVO];
		rgppiIntraQuantizerMatrix [ENHN_LAYER] = new Int* [nVO];
		for (iObj = 0; iObj < nVO; iObj++)	{
			rgppiIntraQuantizerMatrix [ENHN_LAYER] [iObj] = new Int [BLOCK_SQUARE_SIZE];
			if (fscanf (pfPara, "%d", &rgbLoadIntraMatrix [ENHN_LAYER] [iObj]) != 1)	{
				fprintf(stderr, "wrong parameter file format on line %d\n", *pnLine);
				exit (1);
			}
			assert (rgbLoadIntraMatrix [ENHN_LAYER] [iObj] ==0 || 
					rgbLoadIntraMatrix [ENHN_LAYER] [iObj] == 1);
			if (rgbLoadIntraMatrix [ENHN_LAYER] [iObj]) {
				for (UInt i = 0; i < BLOCK_SQUARE_SIZE; i++) {
					if (fscanf (pfPara, "%d", &rgppiIntraQuantizerMatrix [ENHN_LAYER] [iObj] [i]) != 1)	{
						fprintf(stderr, "wrong parameter file format on line %d\n", *pnLine);
						exit (1);
					}
				}
			}
			else
				memcpy (rgppiIntraQuantizerMatrix [ENHN_LAYER] [iObj], rgiDefaultIntraQMatrix, BLOCK_SQUARE_SIZE * sizeof (Int));
		}
	}

	// load non-default inter Q-Matrix, 0 -- FALSE, 1 -- TRUE
	nextValidLine (pfPara, pnLine);
	Bool* rgbLoadInterMatrix [2]; 
	rgbLoadInterMatrix [BASE_LAYER] = new Bool [nVO];
	Int** rgppiInterQuantizerMatrix [2];
	rgppiInterQuantizerMatrix [BASE_LAYER] = new Int* [nVO];
	for (iObj = 0; iObj < nVO; iObj++)	{
		rgppiInterQuantizerMatrix [BASE_LAYER] [iObj] = new Int [BLOCK_SQUARE_SIZE];
		if (fscanf (pfPara, "%d", &rgbLoadInterMatrix [BASE_LAYER] [iObj]) != 1)	{
			fprintf(stderr, "wrong parameter file format on line %d\n", *pnLine);
			exit (1);
		}
		assert (rgbLoadInterMatrix [BASE_LAYER] [iObj] ==0 || 
				rgbLoadInterMatrix [BASE_LAYER] [iObj] == 1);
		if (rgbLoadInterMatrix [BASE_LAYER] [iObj]) {
			UInt i = 0;
			do {
				if (fscanf (pfPara, "%d", &rgppiInterQuantizerMatrix [BASE_LAYER] [iObj] [i]) != 1)	{
					fprintf(stderr, "wrong parameter file format on line %d\n", *pnLine);
					exit (1);
				}
			} while (rgppiInterQuantizerMatrix [BASE_LAYER] [iObj] [i] != 0 && ++i < BLOCK_SQUARE_SIZE);
		}
		else
			memcpy (rgppiInterQuantizerMatrix [BASE_LAYER] [iObj], rgiDefaultInterQMatrix, BLOCK_SQUARE_SIZE * sizeof (Int));
	}
	if (bAnyScalability)	{
		rgbLoadInterMatrix [ENHN_LAYER] = new Bool [nVO];
		rgppiInterQuantizerMatrix [ENHN_LAYER] = new Int* [nVO];
		for (iObj = 0; iObj < nVO; iObj++)	{
			rgppiInterQuantizerMatrix [ENHN_LAYER] [iObj] = new Int [BLOCK_SQUARE_SIZE];
			if (fscanf (pfPara, "%d", &rgbLoadInterMatrix [ENHN_LAYER] [iObj]) != 1)	{
				fprintf(stderr, "wrong parameter file format on line %d\n", *pnLine);
				exit (1);
			}
			assert (rgbLoadInterMatrix [ENHN_LAYER] [iObj] ==0 || 
					rgbLoadInterMatrix [ENHN_LAYER] [iObj] == 1);
			if (rgbLoadInterMatrix [ENHN_LAYER] [iObj]) {
				for (UInt i = 0; i < BLOCK_SQUARE_SIZE; i++) {
					if (fscanf (pfPara, "%d", &rgppiInterQuantizerMatrix [ENHN_LAYER] [iObj] [i]) != 1)	{
						fprintf(stderr, "wrong parameter file format on line %d\n", *pnLine);
						exit (1);
					}
				}
			}
			else
				memcpy (rgppiInterQuantizerMatrix [ENHN_LAYER] [iObj], rgiDefaultInterQMatrix, BLOCK_SQUARE_SIZE * sizeof (Int));
		}
	}

	// Intra-DC/AC prediction disable
	nextValidLine (pfPara, pnLine);
	Bool* rgbIntraPredictionDisable [2];
	rgbIntraPredictionDisable [BASE_LAYER] = new Bool [nVO]; 
	for (iObj = 0; iObj < nVO; iObj++)	{
		if (fscanf (pfPara, "%d", &(rgbIntraPredictionDisable [BASE_LAYER] [iObj])) != 1)	{
			fprintf(stderr, "wrong parameter file format on line %d\n", *pnLine);
			exit (1);
		}
		assert (rgbIntraPredictionDisable [BASE_LAYER] [iObj] ==0 || 
				rgbIntraPredictionDisable [BASE_LAYER] [iObj] == 1);
	}
	if (bAnyScalability)	{
		rgbIntraPredictionDisable [ENHN_LAYER] = new Bool [nVO];
		for (iObj = 0; iObj < nVO; iObj++)	{
			if (fscanf (pfPara, "%d", &(rgbIntraPredictionDisable [ENHN_LAYER] [iObj])) != 1)	{
				fprintf(stderr, "wrong parameter file format on line %d\n", *pnLine);
				exit (1);
			}
			assert (rgbIntraPredictionDisable [ENHN_LAYER] [iObj] ==0 || 
					rgbIntraPredictionDisable [ENHN_LAYER] [iObj] == 1);
		}
	}

	// threhold to code Intra-DC as AC: only meanful when prediction is on
	nextValidLine (pfPara, pnLine);
	Bool* rgiIntraDCSwitchingThr [2];
	rgiIntraDCSwitchingThr [BASE_LAYER] = new Int [nVO]; 
	for (iObj = 0; iObj < nVO; iObj++)	{
		if (fscanf (pfPara, "%d", &(rgiIntraDCSwitchingThr [BASE_LAYER] [iObj])) != 1)	{
			fprintf(stderr, "wrong parameter file format on line %d\n", *pnLine);
			exit (1);
		}
		assert (rgiIntraDCSwitchingThr [BASE_LAYER] [iObj] >= 0 && 
				rgiIntraDCSwitchingThr [BASE_LAYER] [iObj] <= 7);
	}
	if (bAnyScalability)	{
		rgiIntraDCSwitchingThr [ENHN_LAYER] = new Int [nVO]; 
		for (iObj = 0; iObj < nVO; iObj++)	{
			if (fscanf (pfPara, "%d", &(rgiIntraDCSwitchingThr [ENHN_LAYER] [iObj])) != 1)	{
				fprintf(stderr, "wrong parameter file format on line %d\n", *pnLine);
				exit (1);
			}
			assert (rgiIntraDCSwitchingThr [ENHN_LAYER] [iObj] >= 0 && 
					rgiIntraDCSwitchingThr [ENHN_LAYER] [iObj] <= 7);
		}
	}

	// I-VO quantization stepsize
	nextValidLine (pfPara, pnLine);
	Int* rgiIStep [2]; 
	rgiIStep [BASE_LAYER] = new Int [nVO]; 
	for (iObj = 0; iObj < nVO; iObj++)	{
		if (fscanf (pfPara, "%d", &(rgiIStep [BASE_LAYER] [iObj])) != 1)	{
			fprintf(stderr, "wrong parameter file format on line %d\n", *pnLine);
			exit (1);
		}
		assert (rgiIStep [BASE_LAYER] [iObj] > 0 && rgiIStep [BASE_LAYER] [iObj] < 32);
	}
	if (bAnyScalability)	{
		rgiIStep [ENHN_LAYER] = new Int [nVO]; 
		for (iObj = 0; iObj < nVO; iObj++)	{
			if (fscanf (pfPara, "%d", &(rgiIStep [ENHN_LAYER] [iObj])) != 1)	{
				fprintf(stderr, "wrong parameter file format on line %d\n", *pnLine);
				exit (1);
			}
			assert (rgiIStep [ENHN_LAYER] [iObj] > 0 && rgiIStep [ENHN_LAYER] [iObj] < 32);
		}
	}

	// P-VO quantization stepsize
	nextValidLine (pfPara, pnLine);
	Int* rgiPStep [2];
	rgiPStep [BASE_LAYER] = new Int [nVO]; 
	for (iObj = 0; iObj < nVO; iObj++)	{
		if (fscanf (pfPara, "%d", &(rgiPStep [BASE_LAYER] [iObj])) != 1)	{
			fprintf(stderr, "wrong parameter file format on line %d\n", *pnLine);
			exit (1);
		}
		assert (rgiPStep [BASE_LAYER] [iObj] > 0 && rgiPStep [BASE_LAYER] [iObj] < 32);
	}
	if (bAnyScalability)	{
		rgiPStep [ENHN_LAYER] = new Int [nVO]; 
		for (iObj = 0; iObj < nVO; iObj++)	{
			if (fscanf (pfPara, "%d", &(rgiPStep [ENHN_LAYER] [iObj])) != 1)	{
				fprintf(stderr, "wrong parameter file format on line %d\n", *pnLine);
				exit (1);
			}
			assert (rgiPStep [ENHN_LAYER] [iObj] > 0 && rgiPStep [ENHN_LAYER] [iObj] < 32);
		}
	}

	// load non-default gray alpha intra Q-Matrix, 0 -- FALSE, 1 -- TRUE
	nextValidLine (pfPara, pnLine);
	Bool* rgbLoadIntraMatrixAlpha [2];
	rgbLoadIntraMatrixAlpha [BASE_LAYER] = new Bool [nVO]; 	
	Int** rgppiIntraQuantizerMatrixAlpha [2];
	rgppiIntraQuantizerMatrixAlpha [BASE_LAYER] = new Int* [nVO];
	for (iObj = 0; iObj < nVO; iObj++)	{
		rgppiIntraQuantizerMatrixAlpha [BASE_LAYER] [iObj] = new Int [BLOCK_SQUARE_SIZE];
		if (fscanf (pfPara, "%d", &rgbLoadIntraMatrixAlpha [BASE_LAYER] [iObj]) != 1)	{
			fprintf(stderr, "wrong parameter file format on line %d\n", *pnLine);
			exit (1);
		}
		assert (rgbLoadIntraMatrixAlpha [BASE_LAYER] [iObj] ==0 || 
				rgbLoadIntraMatrixAlpha [BASE_LAYER] [iObj] == 1);
		if (rgbLoadIntraMatrixAlpha [BASE_LAYER] [iObj]) {
			for (UInt i = 0; i < BLOCK_SQUARE_SIZE; i++) {
				if (fscanf (pfPara, "%d", &rgppiIntraQuantizerMatrixAlpha [BASE_LAYER] [iObj] [i]) != 1)	{
					fprintf(stderr, "wrong parameter file format on line %d\n", *pnLine);
					exit (1);
				}
			}
		}
		else
			memcpy (rgppiIntraQuantizerMatrixAlpha [BASE_LAYER] [iObj], rgiDefaultIntraQMatrixAlpha, BLOCK_SQUARE_SIZE * sizeof (Int));
	}
	if (bAnyScalability)	{
		rgbLoadIntraMatrixAlpha [ENHN_LAYER] = new Bool [nVO]; 	
		rgppiIntraQuantizerMatrixAlpha [ENHN_LAYER] = new Int* [nVO];
		for (iObj = 0; iObj < nVO; iObj++)	{
			rgppiIntraQuantizerMatrixAlpha [ENHN_LAYER] [iObj] = new Int [BLOCK_SQUARE_SIZE];
			if (fscanf (pfPara, "%d", &rgbLoadIntraMatrixAlpha [ENHN_LAYER] [iObj]) != 1)	{
				fprintf(stderr, "wrong parameter file format on line %d\n", *pnLine);
				exit (1);
			}
			assert (rgbLoadIntraMatrixAlpha [ENHN_LAYER] [iObj] ==0 || 
					rgbLoadIntraMatrixAlpha [ENHN_LAYER] [iObj] == 1);
			if (rgbLoadIntraMatrixAlpha [ENHN_LAYER] [iObj]) {
				for (UInt i = 0; i < BLOCK_SQUARE_SIZE; i++) {
					if (fscanf (pfPara, "%d", &rgppiIntraQuantizerMatrixAlpha [ENHN_LAYER] [iObj] [i]) != 1)	{
						fprintf(stderr, "wrong parameter file format on line %d\n", *pnLine);
						exit (1);
					}
				}
			}
			else
				memcpy (rgppiIntraQuantizerMatrixAlpha [ENHN_LAYER] [iObj], rgiDefaultIntraQMatrixAlpha, BLOCK_SQUARE_SIZE * sizeof (Int));
		}
	}

	// load non-default  gray alpha inter Q-Matrix, 0 -- FALSE, 1 -- TRUE
	nextValidLine (pfPara, pnLine);
	Bool* rgbLoadInterMatrixAlpha [2];
	rgbLoadInterMatrixAlpha [BASE_LAYER] = new Bool [nVO]; 
	Int** rgppiInterQuantizerMatrixAlpha [2];
	rgppiInterQuantizerMatrixAlpha [BASE_LAYER] = new Int* [nVO];
	for (iObj = 0; iObj < nVO; iObj++)	{
		rgppiInterQuantizerMatrixAlpha [BASE_LAYER] [iObj] = new Int [BLOCK_SQUARE_SIZE];
		if (fscanf (pfPara, "%d", &rgbLoadInterMatrixAlpha [BASE_LAYER] [iObj]) != 1)	{
			fprintf(stderr, "wrong parameter file format on line %d\n", *pnLine);
			exit (1);
		}
		assert (rgbLoadInterMatrixAlpha [BASE_LAYER] [iObj] ==0 || 
				rgbLoadInterMatrixAlpha [BASE_LAYER] [iObj] == 1);
		if (rgbLoadInterMatrixAlpha [BASE_LAYER] [iObj]) {
			for (UInt i = 0; i < BLOCK_SQUARE_SIZE; i++) {
				if (fscanf (pfPara, "%d", &rgppiInterQuantizerMatrixAlpha [BASE_LAYER] [iObj] [i]) != 1)	{
					fprintf(stderr, "wrong parameter file format on line %d\n", *pnLine);
					exit (1);
				}
			}
		}
		else
			memcpy (rgppiInterQuantizerMatrixAlpha [BASE_LAYER] [iObj], rgiDefaultInterQMatrixAlpha, BLOCK_SQUARE_SIZE * sizeof (Int));
	}
	if (bAnyScalability)	{
		rgbLoadInterMatrixAlpha [ENHN_LAYER] = new Bool [nVO]; 
		rgppiInterQuantizerMatrixAlpha [ENHN_LAYER] = new Int* [nVO];
		for (iObj = 0; iObj < nVO; iObj++)	{
			rgppiInterQuantizerMatrixAlpha [ENHN_LAYER] [iObj] = new Int [BLOCK_SQUARE_SIZE];
			if (fscanf (pfPara, "%d", &rgbLoadInterMatrixAlpha [ENHN_LAYER] [iObj]) != 1)	{
				fprintf(stderr, "wrong parameter file format on line %d\n", *pnLine);
				exit (1);
			}
			assert (rgbLoadInterMatrixAlpha [ENHN_LAYER] [iObj] ==0 || 
					rgbLoadInterMatrixAlpha [ENHN_LAYER] [iObj] == 1);
			if (rgbLoadInterMatrixAlpha [ENHN_LAYER] [iObj]) {
				for (UInt i = 0; i < BLOCK_SQUARE_SIZE; i++) {
					if (fscanf (pfPara, "%d", &rgppiInterQuantizerMatrixAlpha [ENHN_LAYER] [iObj] [i]) != 1)	{
						fprintf(stderr, "wrong parameter file format on line %d\n", *pnLine);
						exit (1);
					}
				}
			}
			else
				memcpy (rgppiInterQuantizerMatrixAlpha [ENHN_LAYER] [iObj], rgiDefaultInterQMatrixAlpha, BLOCK_SQUARE_SIZE * sizeof (Int));
		}
	}

	// I-VO quantization stepsize for Alpha
	nextValidLine (pfPara, pnLine);
	Int* rgiIStepAlpha [2];
	rgiIStepAlpha [BASE_LAYER] = new Int [nVO]; 
	for (iObj = 0; iObj < nVO; iObj++)	{
		if (fscanf (pfPara, "%d", &(rgiIStepAlpha [BASE_LAYER] [iObj])) != 1)	{
			fprintf(stderr, "wrong parameter file format on line %d\n", *pnLine);
			exit (1);
		}
		assert (rgiIStepAlpha [BASE_LAYER] [iObj] > 0 && rgiIStep [BASE_LAYER] [iObj] < 32);
	}
	if (bAnyScalability)	{
		rgiIStepAlpha [ENHN_LAYER] = new Int [nVO]; 
		for (iObj = 0; iObj < nVO; iObj++)	{
			if (fscanf (pfPara, "%d", &(rgiIStepAlpha [ENHN_LAYER] [iObj])) != 1)	{
				fprintf(stderr, "wrong parameter file format on line %d\n", *pnLine);
				exit (1);
			}
			assert (rgiIStepAlpha [ENHN_LAYER] [iObj] > 0 && rgiIStep [ENHN_LAYER] [iObj] < 32);
		}
	}

	// P-VO quantization stepsize for Alpha
	nextValidLine (pfPara, pnLine);
	Int* rgiPStepAlpha [2];
	rgiPStepAlpha [BASE_LAYER] = new Int [nVO]; 
	for (iObj = 0; iObj < nVO; iObj++)	{
		if (fscanf (pfPara, "%d", &(rgiPStepAlpha [BASE_LAYER] [iObj])) != 1)	{
			fprintf(stderr, "wrong parameter file format on line %d\n", *pnLine);
			exit (1);
		}
		assert (rgiPStepAlpha [BASE_LAYER] [iObj] > 0 && rgiPStepAlpha [BASE_LAYER] [iObj] < 64);
	}
	if (bAnyScalability)	{
		rgiPStepAlpha [ENHN_LAYER] = new Int [nVO]; 
		for (iObj = 0; iObj < nVO; iObj++)	{
			if (fscanf (pfPara, "%d", &(rgiPStepAlpha [ENHN_LAYER] [iObj])) != 1)	{
				fprintf(stderr, "wrong parameter file format on line %d\n", *pnLine);
				exit (1);
			}
			assert (rgiPStepAlpha [ENHN_LAYER] [iObj] > 0 && rgiPStepAlpha [ENHN_LAYER] [iObj] < 64);
		}
	}

	// disable_gray_quant_update
	nextValidLine (pfPara, pnLine);
	Bool* rgbNoGrayQuantUpdate [2];
	rgbNoGrayQuantUpdate [BASE_LAYER] = new Bool [nVO]; 
	for (iObj = 0; iObj < nVO; iObj++)	{
		if (fscanf (pfPara, "%d", &(rgbNoGrayQuantUpdate [BASE_LAYER] [iObj])) != 1)	{
			fprintf(stderr, "wrong parameter file format on line %d\n", *pnLine);
			exit (1);
		}
	}
	if (bAnyScalability)	{
		rgbNoGrayQuantUpdate [ENHN_LAYER] = new Bool [nVO]; 
		for (iObj = 0; iObj < nVO; iObj++)	{
			if (fscanf (pfPara, "%d", &(rgbNoGrayQuantUpdate [ENHN_LAYER] [iObj])) != 1)	{
				fprintf(stderr, "wrong parameter file format on line %d\n", *pnLine);
				exit (1);
			}
		}
	}

	// number of P-VOP's between 2 I-VOP's; if there are B-VOPs, the no. of encode frames will multiply
	nextValidLine (pfPara, pnLine);
	Int* rgiNumPbetweenIVOP = new Int [nVO];
	for (iObj = 0; iObj < nVO; iObj++)	{
		if (fscanf (pfPara, "%d", &rgiNumPbetweenIVOP [iObj]) != 1)	{
			fprintf(stderr, "wrong parameter file format on line %d\n", *pnLine);
			exit (1);
		}
		if (rgiNumPbetweenIVOP [iObj] < 0)
			rgiNumPbetweenIVOP [iObj] = lastFrm - firstFrm + 1;		//only 1 I at the beginning
	}

	// number of B-VOP's between 2 P-VOP's
	nextValidLine (pfPara, pnLine);
	Int* rgiNumBbetweenPVOP = new Int [nVO];
	for (iObj = 0; iObj < nVO; iObj++)	{
		if (fscanf (pfPara, "%d", &rgiNumBbetweenPVOP [iObj]) != 1)	{
			fprintf(stderr, "wrong parameter file format on line %d\n", *pnLine);
			exit (1);
		}
		assert (rgiNumBbetweenPVOP [iObj] >= 0);
	}

	// code for quantization stepsize for B-VOP
	nextValidLine (pfPara, pnLine);
	Int* rgiStepBCode [2];
	rgiStepBCode [BASE_LAYER] = new Int [nVO]; 
	for (iObj = 0; iObj < nVO; iObj++)	{
		if (fscanf (pfPara, "%d", &(rgiStepBCode [BASE_LAYER] [iObj])) != 1)	{
			fprintf(stderr, "wrong parameter file format on line %d\n", *pnLine);
			exit (1);
		}
		assert (rgiStepBCode [BASE_LAYER] [iObj] >= 0 && rgiStepBCode [BASE_LAYER] [iObj] <= 3);
	}
	if (bAnyScalability)	{
		rgiStepBCode [ENHN_LAYER] = new Int [nVO]; 
		for (iObj = 0; iObj < nVO; iObj++)	{
			if (fscanf (pfPara, "%d", &(rgiStepBCode [ENHN_LAYER] [iObj])) != 1)	{
				fprintf(stderr, "wrong parameter file format on line %d\n", *pnLine);
				exit (1);
			}
			assert (rgiStepBCode [ENHN_LAYER] [iObj] >= 0 && 
					rgiStepBCode [ENHN_LAYER] [iObj] <= 3);
		}
	}

	// deblocking filter disable
	nextValidLine (pfPara, pnLine);
	Bool* rgbDeblockFilterDisable = new Bool [nVO];
	for (iObj = 0; iObj < nVO; iObj++)	{
		if (fscanf (pfPara, "%d", &rgbDeblockFilterDisable [iObj]) != 1)	{
			fprintf(stderr, "wrong parameter file format on line %d\n", *pnLine);
			exit (1);
		}
		if (rgbDeblockFilterDisable [iObj] != TRUE)	{
			assert (FALSE);
			fprintf(stderr, "Deblocking filter is not supported in this version\n");
			exit (1);
		}
	}

	// arithmetic coding mode
	nextValidLine (pfPara, pnLine);
	EntropyCodeType* rgfEncType = new EntropyCodeType [nVO];
	for (iObj = 0; iObj < nVO; iObj++)	{
		UInt uiEncType;
		if (fscanf (pfPara, "%d", &uiEncType) != 1)	{
			fprintf(stderr, "wrong parameter file format on line %d\n", *pnLine);
			exit (1);
		}
		else {
			assert (uiEncType == 0 || uiEncType == 1);
			rgfEncType [iObj] = (uiEncType == 0) ? huffman : arithmetic;
		}
	}

	// Temporal sampling rate
	nextValidLine (pfPara, pnLine);
	Int* rgiTSRate = new Int [nVO];
	for (iObj = 0; iObj < nVO; iObj++)	{
		if (fscanf (pfPara, "%d", &rgiTSRate [iObj]) != 1)	{
			fprintf(stderr, "wrong parameter file format on line %d\n", *pnLine);
			exit (1);
		}
		assert (rgiTSRate [iObj] >= 1);
	}

	// maximum displacement
	nextValidLine (pfPara, pnLine);
	UInt* rguiSearchRange [2];
	rguiSearchRange [BASE_LAYER] = new UInt [nVO];
	for (iObj = 0; iObj < nVO; iObj++) {
		if (fscanf (pfPara, "%d", &rguiSearchRange [BASE_LAYER] [iObj]) != 1)	{
			fprintf(stderr, "wrong parameter file format on line %d\n", *pnLine);
			exit (1);
		}
		assert (rguiSearchRange [BASE_LAYER] [iObj] >= 1);
	}
	if (bAnyScalability)	{
		rguiSearchRange [ENHN_LAYER] = new UInt [nVO];
		for (iObj = 0; iObj < nVO; iObj++) {
			if (fscanf (pfPara, "%d", &rguiSearchRange [ENHN_LAYER] [iObj]) != 1)	{
				fprintf(stderr, "wrong parameter file format on line %d\n", *pnLine);
				exit (1);
			}
			assert (rguiSearchRange [ENHN_LAYER] [iObj] >= 1);
		}
	}

	// use original or reconstructed previous VO for ME
	nextValidLine (pfPara, pnLine);
	Bool* rgbOriginalME [2];
	rgbOriginalME [BASE_LAYER] = new Bool [nVO];
	for (iObj = 0; iObj < nVO; iObj++)	{
		if (fscanf (pfPara, "%d", &rgbOriginalME [BASE_LAYER] [iObj]) != 1)	{
			fprintf(stderr, "wrong parameter file format on line %d\n", *pnLine);
			exit (1);
		}
		assert (rgbOriginalME [BASE_LAYER] [iObj] == 0 || rgbOriginalME [BASE_LAYER] [iObj] == 1);
	}
	if (bAnyScalability)	{
		rgbOriginalME [ENHN_LAYER] = new Bool [nVO];
		for (iObj = 0; iObj < nVO; iObj++)	{
			if (fscanf (pfPara, "%d", &rgbOriginalME [ENHN_LAYER] [iObj]) != 1)	{
				fprintf(stderr, "wrong parameter file format on line %d\n", *pnLine);
				exit (1);
			}
			assert (rgbOriginalME [ENHN_LAYER] [iObj] == 0 || rgbOriginalME [ENHN_LAYER] [iObj] == 1);
		}
	}

	// disable advance prediction (obmc and [8x8, not yet])?
	nextValidLine (pfPara, pnLine);
	Bool* rgbAdvPredDisable [2];
	rgbAdvPredDisable [BASE_LAYER] = new Bool [nVO];
	for (iObj = 0; iObj < nVO; iObj++)	{
		if (fscanf (pfPara, "%d", &rgbAdvPredDisable [BASE_LAYER] [iObj]) != 1)	{
			fprintf(stderr, "wrong parameter file format on line %d\n", *pnLine);
			exit (1);
		}
		assert (rgbAdvPredDisable [BASE_LAYER] [iObj] == 0 || rgbAdvPredDisable [BASE_LAYER] [iObj] == 1);
	}
	if (bAnyScalability)	{
		rgbAdvPredDisable [ENHN_LAYER] = new Bool [nVO];
		for (iObj = 0; iObj < nVO; iObj++)	{
			if (fscanf (pfPara, "%d", &rgbAdvPredDisable [ENHN_LAYER] [iObj]) != 1)	{
				fprintf(stderr, "wrong parameter file format on line %d\n", *pnLine);
				exit (1);
			}
			assert (rgbAdvPredDisable [ENHN_LAYER] [iObj] == 0 || rgbAdvPredDisable [ENHN_LAYER] [iObj] == 1);
		}
	}

	
	// file information		
	Char pchPrefix [100];
	nextValidLine (pfPara, pnLine);
	fscanf (pfPara, "%s", pchPrefix);

	// original file directory
	Char pchBmpDir [100];
	nextValidLine (pfPara, pnLine);
	fscanf (pfPara, "%100s", pchBmpDir);

	// chrominance format
	nextValidLine (pfPara, pnLine);
	ChromType* rgfChrType = new ChromType [nVO];
	for (iObj = 0; iObj < nVO; iObj++)	{
		UInt uiChrType;
		if (fscanf (pfPara, "%d", &uiChrType) != 1)	{
			fprintf(stderr, "wrong parameter file format on line %d\n", *pnLine);
			exit (1);
		}
		else {
			assert (uiChrType == 0 || uiChrType == 1 || uiChrType == 2);
			rgfChrType [iObj] = (ChromType) uiChrType;
		}
	}

	//VDS: back-channel bitstream file
	Char pchBCStrFile[100];
	nextValidLine (pfPara, pnLine);
	fscanf (pfPara, "%100s", pchBCStrFile);

	//VDS: mesh data file
	Char pchMeshFile[100];
	nextValidLine (pfPara, pnLine);
	fscanf (pfPara, "%100s", pchMeshFile);
	
	// output file directory
	Char pchOutBmpDir [100];
	nextValidLine (pfPara, pnLine);
	fscanf (pfPara, "%100s", pchOutBmpDir);

	// output bitstream file
	Char pchOutStrFile[100];
	nextValidLine (pfPara, pnLine);
	fscanf (pfPara, "%100s", pchOutStrFile);

	// statistics dumping options
	nextValidLine (pfPara, pnLine);
	Bool* rgbDumpMB = new Bool [nVO];
	for (iObj = 0; iObj < nVO; iObj++)	{
		if (fscanf (pfPara, "%d", &rgbDumpMB [iObj]) != 1)	{
			fprintf(stderr, "wrong parameter file format on line %d\n", *pnLine);
			exit (1);
		}
		assert (rgbDumpMB [iObj] == 0 || rgbDumpMB [iObj] == 1);
	}

	// trace file options
	nextValidLine (pfPara, pnLine);
	Bool* rgbTrace = new Bool [nVO];
	for (iObj = 0; iObj < nVO; iObj++)	{
		if (fscanf (pfPara, "%d", &rgbTrace [iObj]) != 1)	{
			fprintf(stderr, "wrong parameter file format on line %d\n", *pnLine);
			exit (1);
		}
		assert (rgbTrace [iObj] == 0 || rgbTrace [iObj] == 1);
	}

	// sprite info
	// sprite usage
	nextValidLine (pfPara, pnLine);
	UInt* rguiSpriteUsage = new UInt [nVO]; 
	for (iObj = 0; iObj < nVO; iObj++)	{
		if (fscanf (pfPara, "%d", &rguiSpriteUsage [iObj]) != 1)	{
			fprintf(stderr, "wrong parameter file format on line %d\n", *pnLine);
			exit (1);
		}
		assert ((rguiSpriteUsage [iObj] == 0) || (rguiSpriteUsage [iObj] == 1)); // only support static sprite at the moment
		if (rgbSpatialScalability [iObj] == TRUE)
			assert (rguiSpriteUsage [iObj] == 0);
	}

	// warping accuracy
	nextValidLine (pfPara, pnLine);
	UInt* rguiWarpingAccuracy = new UInt [nVO]; 
	for (iObj = 0; iObj < nVO; iObj++)	{
		if (fscanf (pfPara, "%d", &rguiWarpingAccuracy [iObj]) != 1)	{
			fprintf(stderr, "wrong parameter file format on line %d\n", *pnLine);
			exit (1);
		}
		if (rguiSpriteUsage [iObj] != 0)
			assert (rguiWarpingAccuracy [iObj] == 0 || rguiWarpingAccuracy [iObj] == 1 || rguiWarpingAccuracy [iObj] == 2 || rguiWarpingAccuracy [iObj] == 3);
	}

	// number of points
	nextValidLine (pfPara, pnLine);
	Int* rgiNumPnts = new Int [nVO]; 
	for (iObj = 0; iObj < nVO; iObj++)	{
		if (fscanf (pfPara, "%d", &rgiNumPnts [iObj]) != 1)	{
			fprintf(stderr, "wrong parameter file format on line %d\n", *pnLine);
			exit (1);
		}
		if (rgbSpatialScalability [iObj] == TRUE)
			assert (rgiNumPnts [iObj] == -1);
		if (rguiSpriteUsage [iObj] == 1) 
			assert (rgiNumPnts [iObj] == 0 ||
					rgiNumPnts [iObj] == 1 ||
					rgiNumPnts [iObj] == 2 ||
					rgiNumPnts [iObj] == 3 ||
					rgiNumPnts [iObj] == 4);
	}

	// sprite directory
	Char pchSptDir [100];
	nextValidLine (pfPara, pnLine);
	fscanf (pfPara, "%100s", pchSptDir);

	// point directory
	Char pchSptPntDir [100];
	nextValidLine (pfPara, pnLine);
	fscanf (pfPara, "%100s", pchSptPntDir);

	CSessionEncoder* penc = new CSessionEncoder (
		// general info
		bvds, // VDS: view-dependent scalability enable
		uiFrmWidth, // frame width
		uiFrmHeight, // frame height
		firstFrm, // first frame number
		lastFrm, // last frame number
		firstVO, // first VO index
		lastVO, // last VO index
		rgbSpatialScalability, // spatial scalability indicator
		rgiEnhnLayerUpsampleFactor, //enhancement upsample factor

		rgbRateControl,
		rguiBitsBudget,
		rgfAlphaUsage, // alpha usage for each VO.
		rgbShapeOnly,
		rgiBinaryAlphaTH,
		rgbNoCrChange, 

		// motion estimation part for each VO
		rguiSearchRange,
		rgbOriginalME, // flag indicating whether use the original previous VO for ME
		rgbAdvPredDisable,

		// systax mode
		rgbSeprMotionTexture, // separate motion-texture coding
		rgbErrorResilientDisable, //error resilient disable
		rgbVPBitTh,	// Bit threshold for video packet spacing control
		rgbDataPartitioning,  //data partitioning
		rgbReversibleVlc, //reversible VLC

		// for texture coding
		rgfQuant,
		rgbLoadIntraMatrix,
		rgppiIntraQuantizerMatrix,
		rgbLoadInterMatrix,
		rgppiInterQuantizerMatrix,
		rgbIntraPredictionDisable,	//disable intra dc/ac prediction
		rgiIntraDCSwitchingThr,		//threshold to code dc as ac when pred. is on
		rgiIStep,					// I-VOP quantization stepsize
		rgiPStep,					// P-VOP quantization stepsize
		rgbLoadIntraMatrixAlpha,
		rgppiIntraQuantizerMatrixAlpha,
		rgbLoadInterMatrixAlpha,
		rgppiInterQuantizerMatrixAlpha,
		rgiIStepAlpha,				// I-VOP gray alpha quantization stepsize
		rgiPStepAlpha,				// P-VOP gray alpha quantization stepsize
		rgbNoGrayQuantUpdate,		//disable gray quant update within VOP
		rgiStepBCode,				// code for quantization stepsize for B-VOP
		rgiNumBbetweenPVOP,			// no of B-VOPs between P-VOPs
		rgiNumPbetweenIVOP,			// no of P-VOPs between I-VOPs
		rgbDeblockFilterDisable,	//disable deblocing filter
		rgfEncType,					// Arithmetic code/huffman usage,

		
		// file information
		pchPrefix, // prefix name of the movie
		pchBmpDir, // bmp file directory location
		rgfChrType,
		pchBCStrFile, // VDS: back-channel bitstream file
		pchMeshFile, // VDS: mesh data file
		pchOutBmpDir, // quantized frame file directory
		pchOutStrFile, // output bitstream file
		rgiTSRate,
		rgbDumpMB,
		rgbTrace,
		rguiSpriteUsage,
		rguiWarpingAccuracy,
		rgiNumPnts,
		pchSptDir,
		pchSptPntDir
	);
#ifdef __PC_COMPILER_
	Int tickBegin = ::GetTickCount ();
#endif // __PC_COMPILER_
	penc -> encode ();
#ifdef __PC_COMPILER_
	Int tickAfter = ::GetTickCount ();
	Int nFrames = 1 + (lastFrm - firstFrm) / rgiTSRate [0];
	printf ("Total time: %d\n", tickAfter - tickBegin);
	Double dAverage = (Double) (tickAfter - tickBegin) / (Double) nFrames;
	printf ("Total frame %d\tAverage time: %.6lf\n", nFrames, dAverage);
	printf ("FPS %.6lf\n", 1000.0 / dAverage);
#endif // __PC_COMPILER_
	delete penc;

	for (Int iLayer = BASE_LAYER; iLayer <= bAnyScalability * ENHN_LAYER; iLayer++)	{
		delete [] rgbRateControl [iLayer];
		delete [] rguiBitsBudget [iLayer];
		delete [] rgbSeprMotionTexture [iLayer];		
		delete [] rgbErrorResilientDisable [iLayer];


		// for texture coding
		delete [] rgfQuant [iLayer];
		delete [] rgbIntraPredictionDisable [iLayer];
		delete [] rgiIntraDCSwitchingThr [iLayer];
		delete [] rgiIStep [iLayer];				// error signal quantization stepsize
		delete [] rgiPStep [iLayer];				// error signal quantization stepsize
		delete [] rgiIStepAlpha [iLayer];		// error signal quantization stepsize
		delete [] rgiPStepAlpha [iLayer];		// error signal quantization stepsize
		delete [] rgbNoGrayQuantUpdate [iLayer]; //discouple change of gray quant with tex. quant
		delete [] rgiStepBCode [iLayer];			// error signal quantization stepsize
		delete [] rgbLoadIntraMatrix [iLayer];
		delete [] rgbLoadInterMatrix [iLayer];
		delete [] rgbLoadIntraMatrixAlpha [iLayer];
		delete [] rgbLoadInterMatrixAlpha [iLayer];
		for (iObj = 0; iObj < nVO; iObj++) {
			delete [] rgppiIntraQuantizerMatrix  [iLayer] [iObj];
			delete [] rgppiInterQuantizerMatrix  [iLayer] [iObj];
			delete [] rgppiIntraQuantizerMatrixAlpha  [iLayer] [iObj];
			delete [] rgppiInterQuantizerMatrixAlpha  [iLayer] [iObj];
		}
		delete [] rgppiIntraQuantizerMatrix [iLayer];
		delete [] rgppiInterQuantizerMatrix [iLayer];
		delete [] rgppiIntraQuantizerMatrixAlpha [iLayer];
		delete [] rgppiInterQuantizerMatrixAlpha [iLayer];
		// for motion esti.
		delete [] rgbOriginalME [iLayer];
		delete [] rgbAdvPredDisable [iLayer];
		delete [] rguiSearchRange [iLayer];
	}

	delete [] rgbSpatialScalability;
	delete [] rgiEnhnLayerUpsampleFactor;
	// for mask coding, should fill in later on
	delete [] rgfAlphaUsage; // alpha usage for each VO.  0: binary, 1: 8-bit
	delete [] rgbShapeOnly;
	delete [] rgiBinaryAlphaTH;
	delete [] rgbNoCrChange;

	delete [] rgbDeblockFilterDisable;	
	delete [] rgfEncType;			// Arithmetic code usage
	delete [] rgfChrType;
	delete [] rgiNumBbetweenPVOP;			// no of B-VOPs between P-VOPs
	delete [] rgiNumPbetweenIVOP;			// no of P-VOPs between I-VOPs
	delete [] rgiTSRate;

	// sprite
	delete [] rguiSpriteUsage;
	delete [] rguiWarpingAccuracy;
	delete [] rgiNumPnts;
	delete [] rgbDumpMB;
	delete [] rgbTrace;
}


Void nextValidLine (FILE *pfPara, UInt* pnLine)	{

	U8 chBoL = 0;
	Char pchPlaceHolder[200];
	fgets (pchPlaceHolder, 200, pfPara); // get the next line 
	(*pnLine)++;
	while (feof (pfPara) == 0 && (chBoL = fgetc(pfPara)) == '%') {
		// skip a line
		fgets (pchPlaceHolder, 200, pfPara); // get the next line 
		(*pnLine)++;
	}
	ungetc (chBoL, pfPara);				

}

