/*************************************************************************

This software module was originally developed by 

	Ming-Chieh Lee (mingcl@microsoft.com), Microsoft Corporation
	Wei-ge Chen (wchen@microsoft.com), Microsoft Corporation
	Bruce Lin (blin@microsoft.com), Microsoft Corporation
	Chuang Gu (chuanggu@microsoft.com), Microsoft Corporation
	Simon Winder (swinder@microsoft.com), Microsoft Corporation

in the course of development of the MPEG-4 Video (ISO/IEC 14496-2). 
This software module is an implementation of a part of one or more MPEG-4 Video tools 
as specified by the MPEG-4 Video. 
ISO/IEC gives users of the MPEG-4 Video free license to this software module or modifications 
thereof for use in hardware or software products claiming conformance to the MPEG-4 Video. 
Those intending to use this software module in hardware or software products are advised that its use may infringe existing patents. 
The original developer of this software module and his/her company, 
the subsequent editors and their companies, 
and ISO/IEC have no liability for use of this software module or modifications thereof in an implementation. 
Copyright is not released for non MPEG-4 Video conforming products. 
Microsoft retains full right to use the code for his/her own purpose, 
assign or donate the code to a third party and to inhibit third parties from using the code for non <MPEG standard> conforming products. 
This copyright notice must be included in all copies or derivative works. 

Copyright (c) 1996, 1997.


Module Name:

	wavelet.h

Abstract:

	wavelet coder header file

Revision History:

*************************************************************************/

#define BUFFER_SIZE				(256*1024)   // number of bytes in bitstream o/p buffer

#define TOL_START_CODE 0x10
#define NUMBITS_TOL_START_CODE  28
#define NUMBITS_TOL_ID          4
#define NUMBITS_TOL_SHAPE       2
#define NUMBITS_TOL_SIZE        16
#define NUMBITS_TOL_DECOMP      8
#define NUMBITS_TOL_DCMEAN      8
#define NUMBITS_TOL_DCQUANT     8
#define NUMBITS_TOL_DCLENGTH    16
#define NUMBITS_TOL_SPA_SCALE   5
#define NUMBITS_TOL_SNR_SCALE   5
#define NUMBITS_TOL_QUANT_TYPE  2
#define NUMBITS_TOL_ACQUANT     8
#define NUMBITS_TOL_ACBILEVELQ  16
#define NUMBITS_TOL_ACLENGTH	24
#define NUMBITS_AC_SNRLENGTH    15
#define NUMBITS_AC_NUMSYMS      7
#define NUMBITS_AC_DCBANDBITS   7

#define DEBUG_OPS		0
#define DEBUG_PARAMS	0
#define DEBUG_BITS		0
#define DEBUG_DECODE    1

typedef enum { WVT_LL, WVT_LH, WVT_HL, WVT_HH } FilterBank;
typedef enum { DAUBE_9_3, DAUBE_9_7 } FilterType;
typedef enum { SymExtA,SymExtB,SymExtAB,SymExtBA } SymExt;
typedef enum { AC_FIXED,AC_ADAPTIVE } ACType;
typedef enum { QT_SINGLE=0,QT_MULTIPLE=1,QT_BILEVEL=2 } QuantType;

typedef struct {
	Float fSymAxis;
	Float fDelay;
	SymExt seSym;
	Float *pfTaps;
	Int iLength;
} WvtFilter;

Class CWvtArithCoderModel;
Class CWvtArithCoder;
Class CWvtCoderChannel;

typedef struct {
	CWvtArithCoder *pacCoder;
	CWvtArithCoderModel *pacModelType;
	CWvtArithCoderModel *pacModelValz;
	CWvtArithCoderModel *pacModelValnz;
} WvtArithCoderSet;

typedef struct {
	CWvtArithCoder **ppacCoder;
	CWvtArithCoderModel *pacModel;
	Int iCurrCoder;
	Int iNumCoders;
} WvtBiLevelACSet;

typedef struct {
	Int iObjectID;
	Bool bLengthMarker;
	Bool bSpatialMarker;
	QuantType qtQuantType;
	CWvtCoderChannel *pCoderChanLuma;
	CWvtCoderChannel *pCoderChanChroma;
	CWvtCoderChannel *pCoderChan;
} WvtCoderParams;

typedef struct {
	QuantType qtQuantType;
	Bool bSNRLengthMarker;
	Int iSpatialLevels;
	Int iSNRLevels;
	Int iReqSpatial;
	Int iReqSNR;
	Int iQuant;
	Int *piMultiQuant;
} WvtDecodeInfo;

#include "bitstrm.hpp"
#include "wvtchannel.hpp"
#include "wvtarith.hpp"
#include "wvtstream.hpp"
#include "wvtdecomp.hpp"
#include "wvtqtree.hpp"
#include "wvtxfobj.hpp"
#include "wvttxtcdr.hpp"
#include "wvtfilter.hpp"
#include "wvt1dproc.hpp"

// quantiser with no deadband
#define MidRiseQuant(V,Q) ((Int)(((V)>=0)?floor((double)(V)/(Q)+0.5):\
	ceil((double)(V)/(Q)-0.5)))
#define InvMidRiseQuant(V,Q) ((Int)((V)*(Q)))

// quantiser with deadband = 2*Q
#define MidRiseQuantDB(V,Q) ((Int)(((V)>=0)?floor((double)(V)/(Q)):\
    ceil((double)(V)/(Q))))
#define InvMidRiseQuantDB(V,Q) \
	((Int)( ((V)==0) ? 0 : (((V)>0)?((V)+0.5)*(Q):((V)-0.5)*(Q)) ))

#define DCQuant(V,Q) MidRiseQuantDB(V,Q)
#define InvDCQuant(V,Q) InvMidRiseQuantDB(V,Q)
#define ACQuant(V,Q) MidRiseQuantDB(V,Q)
#define InvACQuant(V,Q) InvMidRiseQuantDB(V,Q)

#define ACReorder(V) (((V)<0)?(((-(V))<<1)-1):((V)<<1))
#define InvACReorder(V) (((V)&1)?(-(((V)+1)>>1)):((V)>>1))

