/*************************************************************************

This software module was originally developed by 

	Ming-Chieh Lee (mingcl@microsoft.com), Microsoft Corporation
	Wei-ge Chen (wchen@microsoft.com), Microsoft Corporation
	Bruce Lin (blin@microsoft.com), Microsoft Corporation
	Chuang Gu (chuanggu@microsoft.com), Microsoft Corporation
	Simon Winder (swinder@microsoft.com), Microsoft Corporation

in the course of development of the MPEG-4 Video (ISO/IEC 14496-2). 
This software module is an implementation of a part of one or more MPEG-4 Video tools 
as specified by the MPEG-4 Video. 
ISO/IEC gives users of the MPEG-4 Video free license to this software module or modifications 
thereof for use in hardware or software products claiming conformance to the MPEG-4 Video. 
Those intending to use this software module in hardware or software products are advised that its use may infringe existing patents. 
The original developer of this software module and his/her company, 
the subsequent editors and their companies, 
and ISO/IEC have no liability for use of this software module or modifications thereof in an implementation. 
Copyright is not released for non MPEG-4 Video conforming products. 
Microsoft retains full right to use the code for his/her own purpose, 
assign or donate the code to a third party and to inhibit third parties from using the code for non <MPEG standard> conforming products. 
This copyright notice must be included in all copies or derivative works. 

Copyright (c) 1996, 1997.


Module Name:

	wvt1dproc.cpp

Abstract:

	process 1d data objects, convolution.


Revision History:

*************************************************************************/

#include <stdio.h>
#include <stdlib.h>
#include <math.h>
#include <string.h>
#include "typeapi.h"
#include "wavelet.h"

#ifdef __MFC_
#ifdef _DEBUG
#undef THIS_FILE
static char BASED_CODE THIS_FILE[] = __FILE__;
#endif

#define new DEBUG_NEW				   
#endif // __MFC_

CWvt1dFloatArray::CWvt1dFloatArray(Int iLength)
{
	m_pfData = new Float [iLength];
	m_iLength = iLength;
}

CWvt1dFloatArray::CWvt1dFloatArray(Int iLength,SymExt type)
{
	switch (type)
	{
	case SymExtA:
		m_iLength = iLength<<1;
		break;
	case SymExtB:
		m_iLength = (iLength<<1)-2;
		break;
	case SymExtAB:
		m_iLength = (iLength<<1)-1;
		break;
	case SymExtBA:
	default:
		m_iLength = (iLength<<1)-1;
		break;
	}
	m_pfData = new Float [m_iLength];
}


CWvt1dFloatArray::~CWvt1dFloatArray()
{
	delete m_pfData;
}

void CWvt1dFloatArray::load(const Float *pfSrc)
{
	Int i;

	for(i=0;i<m_iLength;i++)
		m_pfData[i]=pfSrc[i];
}

void CWvt1dFloatArray::load(const Float *pfSrc,Int iLength)
{
	Int i;

	for(i=0;i<iLength;i++)
		m_pfData[i]=pfSrc[i];
}

void CWvt1dFloatArray::store(Float *pfDst)
{
	Int i;

	for(i=0;i<m_iLength;i++)
		pfDst[i]=m_pfData[i];
}

void CWvt1dFloatArray::store(Float *pfDst,Int iLength)
{
	Int i;

	for(i=0;i<iLength;i++)
		pfDst[i]=m_pfData[i];
}

void CWvt1dFloatArray::symExtend_A(Int iLength)
{
	Int i;

	for(i=0;i<iLength;i++)
		m_pfData[iLength+i]=m_pfData[iLength-i-1];
}

void CWvt1dFloatArray::symExtend_B(Int iLength)
{
	Int i;
	for(i=0;i<iLength-2;i++)
		m_pfData[iLength+i]=m_pfData[iLength-i-2];
}

void CWvt1dFloatArray::symExtend_BA(Int iLength)
{
	Int i;
	for(i=0;i<iLength-1;i++)
		m_pfData[iLength+i]=m_pfData[iLength-i-1];
}

void CWvt1dFloatArray::symExtend_AB(Int iLength)
{
	Int i;
	for(i=0;i<iLength-1;i++)
		m_pfData[iLength+i]=m_pfData[iLength-i-2];
}


void CWvt1dFloatArray::symExtend(Int iLength,SymExt type)
{
	switch (type)
	{
	case SymExtA:
		symExtend_A(iLength);
		break;
	case SymExtB:
		symExtend_B(iLength);
		break;
	case SymExtAB:
		symExtend_AB(iLength);
		break;
	case SymExtBA:
	default:
		symExtend_BA(iLength);
		break;
	}
}

void CWvt1dFloatArray::filterDownConvolve(Float *pfDst,Int iCount,Int iStep,
										  WvtFilter *pwf)
{
	Int iFLength = pwf->iLength;
	Float *pfFilter = pwf->pfTaps;
	Float fSum;
	Int iOffset = (Int)ceil((double)(pwf->fSymAxis+pwf->fDelay));

	iOffset=(iOffset+m_iLength)%m_iLength;

	Int n = iOffset;
	Int i,j,k;
	for(i=0;i<iCount;i++)
	{
		fSum=0.0f;
		for(j=0,k=n;j<iFLength;j++,k--)
		{
			if(k<0)
				k+=m_iLength;
			fSum+=m_pfData[k]*pfFilter[j];
		}
		pfDst[i]=fSum;
		n+=iStep;
		if(n>=m_iLength)
			n-=m_iLength;
	}
}

void CWvt1dFloatArray::filterUpConvolve(Float *pfDst,Int iCount,Int iStep,
									  WvtFilter *pwf)
{
	Int iFLength = pwf->iLength;
	Float *pfFilter = pwf->pfTaps;
	Float fSum;
	Int iOffset = (Int)ceil((double)(pwf->fSymAxis+pwf->fDelay));

	iOffset=(iOffset+m_iLength*iStep)%(m_iLength*iStep);
	
	Int n = iOffset;
	Int m = n/iStep;
	Int d = n-m*iStep;
	Int i,j,k;
	for(i=0;i<iCount;i++)
	{
		fSum=0.0f;
		for(j=d,k=m;j<iFLength;j+=iStep,k--)
		{
			if(k<0)
				k+=m_iLength;
			fSum+=m_pfData[k]*pfFilter[j];
		}
		pfDst[i]=fSum;
		d++;
		if(d==iStep)
		{
			d=0;
			m++;
			if(m==m_iLength)
				m-=m_iLength;
		}
	}
}


void CWvt1dFloatArray::txtDump()
{
	Int i;

	for(i=0;i<m_iLength;i++)
		printf("%.5g\t",m_pfData[i]);
	printf("\n");

}

