/*************************************************************************

This software module was originally developed by 

	Ming-Chieh Lee (mingcl@microsoft.com), Microsoft Corporation
	Wei-ge Chen (wchen@microsoft.com), Microsoft Corporation
	Bruce Lin (blin@microsoft.com), Microsoft Corporation
	Chuang Gu (chuanggu@microsoft.com), Microsoft Corporation
	Simon Winder (swinder@microsoft.com), Microsoft Corporation

in the course of development of the MPEG-4 Video (ISO/IEC 14496-2). 
This software module is an implementation of a part of one or more MPEG-4 Video tools 
as specified by the MPEG-4 Video. 
ISO/IEC gives users of the MPEG-4 Video free license to this software module or modifications 
thereof for use in hardware or software products claiming conformance to the MPEG-4 Video. 
Those intending to use this software module in hardware or software products are advised that its use may infringe existing patents. 
The original developer of this software module and his/her company, 
the subsequent editors and their companies, 
and ISO/IEC have no liability for use of this software module or modifications thereof in an implementation. 
Copyright is not released for non MPEG-4 Video conforming products. 
Microsoft retains full right to use the code for his/her own purpose, 
assign or donate the code to a third party and to inhibit third parties from using the code for non <MPEG standard> conforming products. 
This copyright notice must be included in all copies or derivative works. 

Copyright (c) 1996, 1997.


Module Name:

	wvtarith.cpp

Abstract:

	arithmetic codec for wavelets

Revision History:

*************************************************************************/

#include <stdio.h>
#include <stdlib.h>
#include <math.h>
#include <string.h>
#include "typeapi.h"
#include "wavelet.h"


#ifdef __MFC_
#ifdef _DEBUG
#undef THIS_FILE
static char BASED_CODE THIS_FILE[] = __FILE__;
#endif

#define new DEBUG_NEW				   
#endif // __MFC_


#define AC_BITS       (16)
#define AC_TOP        ((1<<AC_BITS)-1)
#define AC_FIRST_QTR  ((AC_TOP>>2)+1)
#define AC_HALF	      (AC_FIRST_QTR<<1)
#define AC_THIRD_QTR  (AC_FIRST_QTR*3)
#define AC_MAX_FREQ   (16383)

#define AC_BUFFERSIZE (1024)

// Class CWvtArithCoder

CWvtArithCoder::CWvtArithCoder()
{
	m_pucBuffer=NULL;
	m_bArrayAlloc=FALSE;
}

CWvtArithCoder::~CWvtArithCoder()
{
	if(m_pucBuffer!=NULL)
		delete m_pucBuffer;
	if(m_bArrayAlloc==TRUE)
		delete m_pucDecodeBuffer;
}

// ENCODER

void CWvtArithCoder::encodeStart()
{
	m_iBitCount=0;
	m_iBitsToFollow=0;
	m_iLow=0;
	m_iHigh=AC_TOP;
	if(m_pucBuffer!=NULL)
		delete m_pucBuffer;
	m_pucBuffer = new U8 [AC_BUFFERSIZE];
	m_iBufSize=AC_BUFFERSIZE;
	m_iBufLoc=0;
	m_iBitsToGo=8;
}

Int CWvtArithCoder::encodeSymbol(CWvtArithCoderModel &acModel,Int iSymbol)
{
	assert(iSymbol>=0 && iSymbol<acModel.getNSyms());

	//printf("(%d)",iSymbol);
	Int iPrevCount = m_iBitCount;
	Int iRange = m_iHigh-m_iLow+1;
	Int iMaxCumFreq=acModel.getMaxCumFreq();

	m_iHigh = m_iLow + (iRange*acModel.getSymCumFreq(iSymbol))/iMaxCumFreq - 1;
	m_iLow += (iRange*acModel.getSymCumFreq(iSymbol+1))/iMaxCumFreq;

	while(1)
	{
		if(m_iHigh<AC_HALF)
			outputBitFollower(0);
		else if(m_iLow>=AC_HALF)
		{
			outputBitFollower(1);
			m_iLow-=AC_HALF;
			m_iHigh-=AC_HALF;
		}
		else if(m_iLow>=AC_FIRST_QTR && m_iHigh<AC_THIRD_QTR)
		{
			m_iBitsToFollow++;
			m_iLow-=AC_FIRST_QTR;
			m_iHigh-=AC_FIRST_QTR;
		}
		else
			break;
		m_iLow<<=1;
		m_iHigh=(m_iHigh<<1)+1;
	}

	acModel.updateModel(iSymbol);

	return m_iBitCount-iPrevCount;
}

Int CWvtArithCoder::encodeVLCValue(Int iValue,Int iLength)
{
	Int iTemp,iBits = 0;
	Int iMask = (1<<(iLength))-1;

	for(iTemp=iValue;iTemp>iMask;iTemp>>=iLength)
	{
		outputBit(1);
		outputBits(iTemp&iMask,iLength);
		iBits+=iLength+1;
	}
	outputBit(0);
	outputBits(iTemp&iMask,iLength);
	iBits+=iLength+1;

	return iBits;
}

Int CWvtArithCoder::encodeEnd()
{
	Int iPrevCount = m_iBitCount;

	m_iBitsToFollow++;
	if(m_iLow<AC_FIRST_QTR)
		outputBitFollower(0);
	else
		outputBitFollower(1);

	return m_iBitCount-iPrevCount;
}

Int CWvtArithCoder::encodeWriteStream(COutBitStream &bs)
{
	bs.putBits((Char *)m_pucBuffer,m_iBitCount);
	return m_iBitCount;
}

// DECODER

void CWvtArithCoder::decodeGrabStream(CInBitStream &bs,Int iBitCount)
{
	assert(iBitCount>0);
	m_pbsInStream=NULL;
	m_pucDecodeBuffer = new U8 [(iBitCount+7)>>3];
	m_iBitCount=iBitCount;
	m_bArrayAlloc=TRUE;

	// now load bits
	Int iStreamLoc = 0;
	Int i = iBitCount;
	for(;i>=8;i-=8)
		m_pucDecodeBuffer[iStreamLoc++]=bs.getBits(8);
	if(i>0)
		m_pucDecodeBuffer[iStreamLoc]=bs.getBits(i);
}

void CWvtArithCoder::decodeStart(CInBitStream &bs,Int iBitCount)
{
	m_pucDecodeBuffer=NULL;
	m_pbsInStream=&bs;
	m_iBitCount=iBitCount;
	m_bInit=TRUE;
}

void CWvtArithCoder::decodeStart(U8 *pucBuffer,Int iBitCount)
{
	m_pbsInStream=NULL;
	m_pucDecodeBuffer=pucBuffer;
	m_iBitCount=iBitCount;
	m_iBufLoc=0;
	if(iBitCount<8)
		m_iBitsToGo=iBitCount;
	else
		m_iBitsToGo=8;
	m_bInit=TRUE;
}

void CWvtArithCoder::decodeStart()
{
	m_iBufLoc=0;
	if(m_iBitCount<8)
		m_iBitsToGo=m_iBitCount;
	else
		m_iBitsToGo=8;
	m_bInit=TRUE;
}

Int CWvtArithCoder::decodeSymbol(CWvtArithCoderModel &acModel)
{
	if(m_bInit==TRUE)
	{
		m_iLow=0;
		m_iHigh=AC_TOP;
		if(m_iBitCount>=AC_BITS)
			m_iValue=inputBits(AC_BITS);
		else
		{
			Int iTemp = m_iBitCount;
			m_iValue=inputBits(iTemp)<<(AC_BITS-iTemp);
		}
		m_bInit=FALSE;
	}

	Int iSymbol=0;
	Int iRange = m_iHigh-m_iLow+1;
	Int iMaxCumFreq = acModel.getMaxCumFreq();
	Int iCum = ((m_iValue-m_iLow+1)*iMaxCumFreq-1)/iRange;

	while(acModel.getSymCumFreq(iSymbol+1)>iCum)
		iSymbol++;
	assert(iSymbol>=0 && iSymbol<acModel.getNSyms());
	
	m_iHigh = m_iLow + (iRange*acModel.getSymCumFreq(iSymbol))/iMaxCumFreq - 1;
	m_iLow += (iRange*acModel.getSymCumFreq(iSymbol+1))/iMaxCumFreq;

	while(1)
	{
		if(m_iHigh<AC_HALF)
			;
		else if(m_iLow>=AC_HALF)
		{
			m_iValue -= AC_HALF;
			m_iLow -= AC_HALF;
			m_iHigh -= AC_HALF;
		}
		else if(m_iLow>=AC_FIRST_QTR && m_iHigh<AC_THIRD_QTR)
		{
			m_iValue -= AC_FIRST_QTR;
			m_iLow -= AC_FIRST_QTR;
			m_iHigh -= AC_FIRST_QTR;
		}
		else
			break;
		m_iLow<<=1;
		m_iHigh = (m_iHigh<<1)+1;
		m_iValue = (m_iValue<<1)+inputBits(1);
	}

	acModel.updateModel(iSymbol);
//printf("(%d)",m_iBitCount);
	return iSymbol;
}

void CWvtArithCoder::decodeEnd()
{
}

Int CWvtArithCoder::decodeVLCValue(Int iLength)
{
	Int iExtend,iShift = 0;
	Int iVal = 0;

	do {
		iExtend = inputBits(1);
		iVal |= (inputBits(iLength)<<iShift);
		iShift+=iLength;
	} while(iExtend==1);

	return iVal;
}

// bit handling

Int CWvtArithCoder::inputBits(Int iNBits)
{
	Int i,iResult=0;

	if(iNBits>m_iBitCount)
		iNBits=m_iBitCount;
	
	if(m_pbsInStream!=NULL)
	{
		if(iNBits>0)
		{
			m_iBitCount-=iNBits;
			iResult=m_pbsInStream->getBits(iNBits);
		}
		else
			iResult=0;
	}
	else
		for(i=0;i<iNBits;i++)
			iResult=(iResult<<1)+inputBufBit();
	
	return iResult;
}

Int CWvtArithCoder::inputBufBit()
{
	Int iBit;

	m_iBitsToGo--;
	m_iBitCount--;
	iBit=(m_pucDecodeBuffer[m_iBufLoc]>>m_iBitsToGo)&1;
	if(m_iBitsToGo==0)
	{
		if(m_iBitCount<8)
			m_iBitsToGo=m_iBitCount;
		else
			m_iBitsToGo=8;
		m_iBufLoc++;
	}
	//printf("%d",iBit);
	return iBit;
}

void CWvtArithCoder::outputBit(Int iBit)
{
	//printf("%d",iBit);
	// put bit in buffer
	m_pucBuffer[m_iBufLoc]=(m_pucBuffer[m_iBufLoc]<<1)+(iBit!=0);
	m_iBitsToGo--;
	m_iBitCount++;
	if(m_iBitsToGo==0)
	{
		m_iBitsToGo=8;
		m_iBufLoc++;
		if(m_iBufLoc==m_iBufSize)
		{
			// extend the encode buffer
			U8 *pucTmp = new U8 [m_iBufSize+AC_BUFFERSIZE];
			Int i;
			for(i=0;i<m_iBufSize;i++)
				pucTmp[i]=m_pucBuffer[i];
			delete m_pucBuffer;
			m_pucBuffer=pucTmp;
			m_iBufSize+=AC_BUFFERSIZE;
		}
	}
}

// not used very often, so not exactly optimised
void CWvtArithCoder::outputBits(Int iBits,Int iCount)
{
	for(;iCount>0;iCount--)
		outputBit(iBits&(1<<(iCount-1)));
}

void CWvtArithCoder::outputBitFollower(Int iBit)
{
	outputBit(iBit);
	for(;m_iBitsToFollow>0;m_iBitsToFollow--)
		outputBit(!iBit);
}


// Class CWvtArithCoderModel

CWvtArithCoderModel::CWvtArithCoderModel(ACType type,Int iNSyms,Int *piFreq)
	: m_type(type), m_iNSyms(iNSyms)
{
	m_piFreq = new Int [iNSyms+1];
	m_piCumFreq = new Int [iNSyms+1];

	loadModel(piFreq);
}

CWvtArithCoderModel::~CWvtArithCoderModel()
{
	delete m_piFreq;
	delete m_piCumFreq;
}

void CWvtArithCoderModel::loadModel(Int *piFreq)
{
	Int i;

	m_piCumFreq[m_iNSyms]=0;
	if(piFreq==NULL || m_type==AC_ADAPTIVE)
		for(i=m_iNSyms-1;i>=0;i--)
		{
			m_piFreq[i]=1;
			m_piCumFreq[i]=m_iNSyms-i;
		}
	else
		for(i=m_iNSyms-1;i>=0;i--)
		{
			m_piFreq[i]=piFreq[i];
			m_piCumFreq[i]=m_piCumFreq[i+1]+m_piFreq[i];
		}
	assert(m_piCumFreq[0]<=AC_MAX_FREQ);
}

void CWvtArithCoderModel::saveModel(Int *piFreq)
{
	Int i;

	for(i=0;i<m_iNSyms;i++)
		piFreq[i]=m_piFreq[i];
}

void CWvtArithCoderModel::updateModel(Int iSymbol)
{
	Int i;

	assert(iSymbol<m_iNSyms);

	if(m_type==AC_ADAPTIVE)
	{
		if(m_piCumFreq[0]==AC_MAX_FREQ)
		{
			for(i=m_iNSyms-1;i>=0;i--)
			{
				m_piFreq[i]=(m_piFreq[i]+1)>>1;
				m_piCumFreq[i]=m_piCumFreq[i+1]+m_piFreq[i];
			}
		}
		m_piFreq[iSymbol]++;
		for(i=iSymbol;i>=0;i--)
			m_piCumFreq[i]++;
	}
}
