/*************************************************************************

This software module was originally developed by 

	Ming-Chieh Lee (mingcl@microsoft.com), Microsoft Corporation
	Wei-ge Chen (wchen@microsoft.com), Microsoft Corporation
	Bruce Lin (blin@microsoft.com), Microsoft Corporation
	Chuang Gu (chuanggu@microsoft.com), Microsoft Corporation
	Simon Winder (swinder@microsoft.com), Microsoft Corporation

in the course of development of the MPEG-4 Video (ISO/IEC 14496-2). 
This software module is an implementation of a part of one or more MPEG-4 Video tools 
as specified by the MPEG-4 Video. 
ISO/IEC gives users of the MPEG-4 Video free license to this software module or modifications 
thereof for use in hardware or software products claiming conformance to the MPEG-4 Video. 
Those intending to use this software module in hardware or software products are advised that its use may infringe existing patents. 
The original developer of this software module and his/her company, 
the subsequent editors and their companies, 
and ISO/IEC have no liability for use of this software module or modifications thereof in an implementation. 
Copyright is not released for non MPEG-4 Video conforming products. 
Microsoft retains full right to use the code for his/her own purpose, 
assign or donate the code to a third party and to inhibit third parties from using the code for non <MPEG standard> conforming products. 
This copyright notice must be included in all copies or derivative works. 

Copyright (c) 1996, 1997.


Module Name:

	wvtarith.hpp

Abstract:

	wavelet arithmetic coder

Revision History:

*************************************************************************/

#ifndef __WVTARITH_HPP_
#define __WVTARITH_HPP_


Class CWvtArithCoderModel
{
public:
	CWvtArithCoderModel(ACType,Int,Int *piFreq = NULL);
	~CWvtArithCoderModel();

	Int getMaxCumFreq() { return m_piCumFreq[0]; }
	Int getSymCumFreq(Int iSymbol) { return m_piCumFreq[iSymbol]; }
	Int getNSyms() {return m_iNSyms; }
	void loadModel(Int *piFreq = NULL);
	void saveModel(Int *);
	void updateModel(Int);

private:
	Int *m_piFreq;
	Int *m_piCumFreq;
	Int m_iNSyms;
	ACType m_type;
};


Class CWvtArithCoder
{
public:
	CWvtArithCoder();
	~CWvtArithCoder();

	Int getBitCount() { return m_iBitCount; }
	U8 *getEncoderBuffer() { return m_pucBuffer; }
	// encoder
	void encodeStart();
	Int encodeSymbol(CWvtArithCoderModel &,Int);
	Int encodeVLCValue(Int,Int);
	Int encodeEnd();
	Int encodeWriteStream(COutBitStream &);
	// decoder
	void decodeGrabStream(CInBitStream &,Int);
	void decodeStart(CInBitStream &,Int);  // decode from bitstream
	void decodeStart(U8 *,Int);            // decode from array
	void decodeStart();					   // decode from grabbed stream
	Int decodeSymbol(CWvtArithCoderModel &);
	Int decodeVLCValue(Int);
	void decodeEnd();

private:
	Int inputBits(Int);
	Int inputBufBit();
	void outputBit(Int);	
	void outputBits(Int,Int);
	void outputBitFollower(Int);

	Int m_iBitCount;
	Int m_iBitsToFollow;
	Int m_iLow;
	Int m_iHigh;
	Int m_iValue;
	Bool m_bInit;
	U8 *m_pucBuffer;
	U8 *m_pucDecodeBuffer;
	Int m_iBitsToGo;
	Int m_iBufLoc;
	Int m_iBufSize;
	CInBitStream *m_pbsInStream;
	Bool m_bArrayAlloc;
};

#endif __WVTARITH_HPP_
