/*************************************************************************

This software module was originally developed by 

	Ming-Chieh Lee (mingcl@microsoft.com), Microsoft Corporation
	Wei-ge Chen (wchen@microsoft.com), Microsoft Corporation
	Bruce Lin (blin@microsoft.com), Microsoft Corporation
	Chuang Gu (chuanggu@microsoft.com), Microsoft Corporation
	Simon Winder (swinder@microsoft.com), Microsoft Corporation

in the course of development of the MPEG-4 Video (ISO/IEC 14496-2). 
This software module is an implementation of a part of one or more MPEG-4 Video tools 
as specified by the MPEG-4 Video. 
ISO/IEC gives users of the MPEG-4 Video free license to this software module or modifications 
thereof for use in hardware or software products claiming conformance to the MPEG-4 Video. 
Those intending to use this software module in hardware or software products are advised that its use may infringe existing patents. 
The original developer of this software module and his/her company, 
the subsequent editors and their companies, 
and ISO/IEC have no liability for use of this software module or modifications thereof in an implementation. 
Copyright is not released for non MPEG-4 Video conforming products. 
Microsoft retains full right to use the code for his/her own purpose, 
assign or donate the code to a third party and to inhibit third parties from using the code for non <MPEG standard> conforming products. 
This copyright notice must be included in all copies or derivative works. 

Copyright (c) 1996, 1997.


Module Name:

	wvtchannel.cpp

Abstract:

	convenient data structure

Revision History:

*************************************************************************/

#include <stdio.h>
#include <stdlib.h>
#include <math.h>
#include <string.h>
#include "typeapi.h"
#include "wavelet.h"

#ifdef __MFC_
#ifdef _DEBUG
#undef THIS_FILE
static char BASED_CODE THIS_FILE[] = __FILE__;
#endif

#define new DEBUG_NEW				   
#endif // __MFC_

CWvtCoderChannel::CWvtCoderChannel()
{
	m_pSpatialList=NULL;
	m_iSpatialSave = -1;
	m_iSNRSave = -1;
	m_iNumQuants = 0;
}

CWvtCoderChannel::~CWvtCoderChannel()
{
	Int i;

	if(m_pSpatialList!=NULL)
	{
		for(i=0;i<m_iSpatialLevels;i++)
			if(m_pSpatialList[i].pSNRList!=NULL)
				delete m_pSpatialList[i].pSNRList;			
		delete m_pSpatialList;
	}
}

void CWvtCoderChannel::createSpatialSet(Int iSpatialLevels)
{
	Int i;
	
	if(m_pSpatialList!=NULL)
	{
		for(i=0;i<m_iSpatialLevels;i++)
			if(m_pSpatialList[i].pSNRList!=NULL)
				delete m_pSpatialList[i].pSNRList;
		delete m_pSpatialList;
	}
	m_iNumQuants = 0;
	m_iSpatialLevels = iSpatialLevels;
	m_pSpatialList = new WvtSpatialParams [iSpatialLevels];
	for(i=0;i<iSpatialLevels;i++)
		m_pSpatialList[i].pSNRList = NULL;
}

void CWvtCoderChannel::createSNRSet(Int iSpatialLevel,Int iSNRLevels)
{
	if(m_pSpatialList[iSpatialLevel].pSNRList!=NULL)
	{
		m_iNumQuants -= m_pSpatialList[iSpatialLevel].iSNRLevels;
		delete m_pSpatialList[iSpatialLevel].pSNRList;
	}

	m_pSpatialList[iSpatialLevel].pSNRList = new WvtSNRParams [iSNRLevels];
	m_pSpatialList[iSpatialLevel].iSNRLevels = iSNRLevels;
	m_iNumQuants += iSNRLevels;
}

void CWvtCoderChannel::setSNRParams(Int iSpatialLevel,Int iSNRLevel,Int iACQuant,
								 Int iBitBudget)
{
	m_pSpatialList[iSpatialLevel].pSNRList[iSNRLevel].iACQuant = iACQuant;
	m_pSpatialList[iSpatialLevel].pSNRList[iSNRLevel].iBitBudget = iBitBudget;
}

void CWvtCoderChannel::setSave(Int iSpatialLevel,Int iSNRLevel)
{
	m_iSpatialSave = iSpatialLevel;
	m_iSNRSave = iSNRLevel;
}

Bool CWvtCoderChannel::checkSave(Int iSpatialLevel,Int iSNRLevel)
{
	if(iSpatialLevel==m_iSpatialSave && iSNRLevel==m_iSNRSave)
		return TRUE;
	else
		return FALSE;
}
