/*************************************************************************

This software module was originally developed by 

	Ming-Chieh Lee (mingcl@microsoft.com), Microsoft Corporation
	Wei-ge Chen (wchen@microsoft.com), Microsoft Corporation
	Bruce Lin (blin@microsoft.com), Microsoft Corporation
	Chuang Gu (chuanggu@microsoft.com), Microsoft Corporation
	Simon Winder (swinder@microsoft.com), Microsoft Corporation

in the course of development of the MPEG-4 Video (ISO/IEC 14496-2). 
This software module is an implementation of a part of one or more MPEG-4 Video tools 
as specified by the MPEG-4 Video. 
ISO/IEC gives users of the MPEG-4 Video free license to this software module or modifications 
thereof for use in hardware or software products claiming conformance to the MPEG-4 Video. 
Those intending to use this software module in hardware or software products are advised that its use may infringe existing patents. 
The original developer of this software module and his/her company, 
the subsequent editors and their companies, 
and ISO/IEC have no liability for use of this software module or modifications thereof in an implementation. 
Copyright is not released for non MPEG-4 Video conforming products. 
Microsoft retains full right to use the code for his/her own purpose, 
assign or donate the code to a third party and to inhibit third parties from using the code for non <MPEG standard> conforming products. 
This copyright notice must be included in all copies or derivative works. 

Copyright (c) 1996, 1997.


Module Name:

	wvtfilter.cpp

Abstract:

	class for wavelet filtering

Revision History:

*************************************************************************/

#include <stdio.h>
#include <stdlib.h>
#include <math.h>
#include <string.h>
#include "typeapi.h"
#include "wavelet.h"

#ifdef __MFC_
#ifdef _DEBUG
#undef THIS_FILE
static char BASED_CODE THIS_FILE[] = __FILE__;
#endif

#define new DEBUG_NEW				   
#endif // __MFC_

CWvtFilterUnit::CWvtFilterUnit(FilterType type)
{
	m_pwfHighPassAna = new WvtFilter;
	m_pwfLowPassAna = new WvtFilter;
	m_pwfHighPassSyn = new WvtFilter;
	m_pwfLowPassSyn = new WvtFilter;

	switch(type)
	{
	case DAUBE_9_3:
		setDaube_9_3();
		break;
	case DAUBE_9_7:
		setDaube_9_7();
		break;
	default:
		break;
	}
}

CWvtFilterUnit::~CWvtFilterUnit()
{
	delete m_pwfHighPassAna;
	delete m_pwfLowPassAna;
	delete m_pwfHighPassSyn;
	delete m_pwfLowPassSyn;
}

void CWvtFilterUnit::setDaube_9_3()
{
#define D93LEN 9
	const Double h0[D93LEN] = { 0.03314563036812, -0.06629126073624,
		-0.17677669529665, 0.41984465132952, 0.99436891104360, 0.41984465132952,
		-0.17677669529665, -0.06629126073624, 0.03314563036812 };
    const Double h1[D93LEN] = {0.35355339059327, -0.70710678118655, 0.35355339059327,
		0.0, 0.0, 0.0, 0.0, 0.0, 0.0 };
    const Double g0[D93LEN] = {0.35355339059327, 0.70710678118655, 0.35355339059327,
		0.0, 0.0, 0.0, 0.0, 0.0, 0.0 };
    const Double g1[D93LEN] = { -0.03314563036812, -0.06629126073624,
		0.17677669529665, 0.41984465132952, -0.99436891104360, 0.41984465132952,
		0.17677669529665, -0.06629126073624, -0.03314563036812 };
	Double fBias = 1.0f;
	static Float sh0[D93LEN],sh1[D93LEN],sg0[D93LEN],sg1[D93LEN];

	m_pwfLowPassAna->pfTaps = sh0;
	m_pwfLowPassAna->iLength = 9;
	m_pwfLowPassAna->fSymAxis = 4.0f;
	m_pwfLowPassAna->fDelay = 0.0f;
	m_pwfLowPassAna->seSym = SymExtB;

	m_pwfHighPassAna->pfTaps = sh1;
	m_pwfHighPassAna->iLength = 3;
	m_pwfHighPassAna->fSymAxis = 1.0f;
	m_pwfHighPassAna->fDelay = 1.0f;
	m_pwfHighPassAna->seSym = SymExtB;

	m_pwfLowPassSyn->pfTaps = sg0;
	m_pwfLowPassSyn->iLength = 3;
	m_pwfLowPassSyn->fSymAxis = 1.0f;
	m_pwfLowPassSyn->fDelay = 0.0f;
	m_pwfLowPassSyn->seSym = SymExtBA;

	m_pwfHighPassSyn->pfTaps = sg1;
	m_pwfHighPassSyn->iLength = 9;
	m_pwfHighPassSyn->fSymAxis = 4.0f;
	m_pwfHighPassSyn->fDelay = -1.0f;
	m_pwfHighPassSyn->seSym = SymExtAB;

	Int i;
	Double fS=0.0f;
	for(i=0;i<D93LEN;i++)
		fS+=h0[i]*h0[i];
	fS=sqrt(fS)/fBias;
	for(i=0;i<D93LEN;i++)
	{
		sh0[i]=(Float)(h0[i]/fS);
		sh1[i]=(Float)(h1[i]*fS);
		sg0[i]=(Float)(g0[i]*fS);
		sg1[i]=(Float)(g1[i]/fS);
	}
#undef D93LEN
}

void CWvtFilterUnit::setDaube_9_7()
{
#define D97LEN 9
	const Double h0[D97LEN]= {
		0.03782879857992,
		-0.02384929751586,
		-0.11062402748951,
		0.37740268810913,
		0.85269865321930,
		0.37740268810913,
		-0.11062402748951,
		-0.02384929751586,
		0.03782879857992
	};
    const Double h1[D97LEN] = {
		-0.06453905013246,
		0.04068975261660,
		0.41809244072573,
		-0.78848487220618,
		0.41809244072573,
		0.04068975261660,
		-0.06453905013246,
		0.0, 0.0
	};
	const Double g0[D97LEN] = {
		-0.06453905013246,
		-0.04068975261660,
		0.41809244072573,
		0.78848487220618,
		0.41809244072573,
		-0.04068975261660,
		-0.06453905013246,
		0.0, 0.0
	};
    const Double g1[D97LEN] = {
		-0.03782879857992,
		-0.02384929751586,
		0.11062402748951,
		0.37740268810913,
		-0.85269865321930,
		0.37740268810913,
		0.11062402748951,
		-0.02384929751586,
		-0.03782879857992,
	};
	Double fBias = 1.0f;
	static Float sh0[D97LEN],sh1[D97LEN],sg0[D97LEN],sg1[D97LEN];

	m_pwfLowPassAna->pfTaps = sh0;
	m_pwfLowPassAna->iLength = 9;
	m_pwfLowPassAna->fSymAxis = 4.0f;
	m_pwfLowPassAna->fDelay = 0.0f;
	m_pwfLowPassAna->seSym = SymExtB;

	m_pwfHighPassAna->pfTaps = sh1;
	m_pwfHighPassAna->iLength = 7;
	m_pwfHighPassAna->fSymAxis = 3.0f;
	m_pwfHighPassAna->fDelay = 1.0f;
	m_pwfHighPassAna->seSym = SymExtB;

	m_pwfLowPassSyn->pfTaps = sg0;
	m_pwfLowPassSyn->iLength = 7;
	m_pwfLowPassSyn->fSymAxis = 3.0f;
	m_pwfLowPassSyn->fDelay = 0.0f;
	m_pwfLowPassSyn->seSym = SymExtBA;

	m_pwfHighPassSyn->pfTaps = sg1;
	m_pwfHighPassSyn->iLength = 9;
	m_pwfHighPassSyn->fSymAxis = 4.0f;
	m_pwfHighPassSyn->fDelay = -1.0f;
	m_pwfHighPassSyn->seSym = SymExtAB;

	Int i;
	Double fS=0.0f;
	for(i=0;i<D97LEN;i++)
		fS+=h0[i]*h0[i];
	fS=sqrt(fS)/fBias;
	for(i=0;i<D97LEN;i++)
	{
		sh0[i]=(Float)(h0[i]/fS);
		sh1[i]=(Float)(h1[i]*fS);
		sg0[i]=(Float)(g0[i]*fS);
		sg1[i]=(Float)(g1[i]/fS);
	}
#undef D97LEN
}

void CWvtFilterUnit::forwardTransform(CFloatImage *pfiSrc,CWvtDecompTree *pDst)
{
	Int iWidth = pfiSrc->where().width;
	Int iHeight = pfiSrc->where().height();

	assert((iWidth>>1) == pDst->where().width &&
		(iHeight>>1) == pDst->where().height());

	CFloatImage fiTmp(CRct(0,0,iWidth>>1,iHeight));

	filterDownByRow(pfiSrc,&fiTmp,m_pwfLowPassAna);
	filterDownByCol(&fiTmp,pDst->data(WVT_LH),m_pwfHighPassAna);
	filterDownByCol(&fiTmp,pDst->data(WVT_LL),m_pwfLowPassAna);
	filterDownByRow(pfiSrc,&fiTmp,m_pwfHighPassAna);
	filterDownByCol(&fiTmp,pDst->data(WVT_HH),m_pwfHighPassAna);
	filterDownByCol(&fiTmp,pDst->data(WVT_HL),m_pwfLowPassAna);
}

void CWvtFilterUnit::inverseTransform(CWvtDecompTree *pSrc,CFloatImage *pfiDst)
{
	Int iWidth = pfiDst->where().width;
	Int iHeight = pfiDst->where().height();

	assert((iWidth>>1) == pSrc->where().width &&
		(iHeight>>1) == pSrc->where().height());

	CFloatImage fiTmp(CRct(0,0,iWidth>>1,iHeight));
	CFloatImage fiSum(CRct(0,0,iWidth,iHeight));

	filterUpByCol(pSrc->data(WVT_LL),&fiTmp,m_pwfLowPassSyn);
	filterUpByRow(&fiTmp,pfiDst,m_pwfLowPassSyn);

	filterUpByCol(pSrc->data(WVT_LH),&fiTmp,m_pwfHighPassSyn);
	filterUpByRow(&fiTmp,&fiSum,m_pwfLowPassSyn);
	*pfiDst += fiSum;

	filterUpByCol(pSrc->data(WVT_HL),&fiTmp,m_pwfLowPassSyn);
	filterUpByRow(&fiTmp,&fiSum,m_pwfHighPassSyn);
	*pfiDst += fiSum;

	filterUpByCol(pSrc->data(WVT_HH),&fiTmp,m_pwfHighPassSyn);
	filterUpByRow(&fiTmp,&fiSum,m_pwfHighPassSyn);
	*pfiDst += fiSum;
}

void CWvtFilterUnit::filterUpByRow(CFloatImage *pfiSrc,CFloatImage *pfiDst,
								   WvtFilter *pwf)
{
	Int iY;
	Int iWidth = pfiSrc->where().width;
	Int iHeight = pfiSrc->where().height();
	Int iDestWidth = iWidth<<1;
	
	CWvt1dFloatArray f1dRow(iWidth,pwf->seSym);

	const PixelF *ppxlfSrc = pfiSrc->pixels();
	PixelF *ppxlfDst = (PixelF *)pfiDst->pixels();
	for(iY=0;iY<iHeight;iY++,ppxlfSrc+=iWidth,ppxlfDst+=iDestWidth)
	{
		f1dRow.load(ppxlfSrc,iWidth);
		f1dRow.symExtend(iWidth,pwf->seSym);
		f1dRow.filterUpConvolve((Float *)ppxlfDst,iDestWidth,2,pwf);
	}
}

void CWvtFilterUnit::filterUpByCol(CFloatImage *pfiSrc,CFloatImage *pfiDst,
								   WvtFilter *pwf)
{
	Int iX,iY;
	Int iWidth = pfiSrc->where().width;
	Int iHeight = pfiSrc->where().height();
	Int iDestHeight = iHeight<<1;

	CWvt1dFloatArray f1dCol(iHeight,pwf->seSym);
	CWvt1dFloatArray f1dColOut(iDestHeight);

	Float *pfCol = f1dCol.data();
	Float *pfColOut = f1dColOut.data();
	const PixelF *ppxlfSrc;
	PixelF *ppxlfDst;

	for(iX=0;iX<iWidth;iX++)
	{
		ppxlfSrc = pfiSrc->pixels()+iX;
		for(iY=0;iY<iHeight;iY++,ppxlfSrc+=iWidth)
			pfCol[iY]=(Float)*ppxlfSrc;

		f1dCol.symExtend(iHeight,pwf->seSym);
		f1dCol.filterUpConvolve(pfColOut,iDestHeight,2,pwf);

		ppxlfDst = (PixelF *)pfiDst->pixels()+iX;
		for(iY=0;iY<iDestHeight;iY++,ppxlfDst+=iWidth)
			*ppxlfDst = (PixelF)pfColOut[iY];
	}
}

void CWvtFilterUnit::filterDownByRow(CFloatImage *pfiSrc,CFloatImage *pfiDst,
									 WvtFilter *pwf)
{
	Int iY;
	Int iWidth = pfiSrc->where().width;
	Int iHeight = pfiSrc->where().height();
	Int iDestWidth = iWidth>>1;

	CWvt1dFloatArray f1dRow(iWidth,pwf->seSym);

	const PixelF *ppxlfSrc = pfiSrc->pixels();
	PixelF *ppxlfDst = (PixelF *)pfiDst->pixels();
	for(iY=0;iY<iHeight;iY++,ppxlfSrc+=iWidth,ppxlfDst+=iDestWidth)
	{
		f1dRow.load((Float *)ppxlfSrc,iWidth);
		f1dRow.symExtend(iWidth,pwf->seSym);
		f1dRow.filterDownConvolve((Float *)ppxlfDst,iDestWidth,2,pwf);
	}
}

void CWvtFilterUnit::filterDownByCol(CFloatImage *pfiSrc,CFloatImage *pfiDst,
									 WvtFilter *pwf)
{
	Int iY,iX;
	Int iWidth = pfiSrc->where().width;
	Int iHeight = pfiSrc->where().height();
	Int iDestHeight = iHeight>>1;
	
	CWvt1dFloatArray f1dCol(iHeight,pwf->seSym);
	CWvt1dFloatArray f1dColOut(iDestHeight);

	Float *pfCol = f1dCol.data();
	Float *pfColOut = f1dColOut.data();
	const PixelF *ppxlfSrc;
	PixelF *ppxlfDst;

	for(iX=0;iX<iWidth;iX++)
	{
		ppxlfSrc = pfiSrc->pixels()+iX;
		for(iY=0;iY<iHeight;iY++,ppxlfSrc+=iWidth)
			pfCol[iY]=(Float)*ppxlfSrc;

		f1dCol.symExtend(iHeight,pwf->seSym);

		f1dCol.filterDownConvolve(pfColOut,iDestHeight,2,pwf);
		
		ppxlfDst = (PixelF *)pfiDst->pixels()+iX;
		for(iY=0;iY<iDestHeight;iY++,ppxlfDst+=iWidth)
			*ppxlfDst = (PixelF)pfColOut[iY];

	}
}

