/*************************************************************************

This software module was originally developed by 

	Ming-Chieh Lee (mingcl@microsoft.com), Microsoft Corporation
	Wei-ge Chen (wchen@microsoft.com), Microsoft Corporation
	Bruce Lin (blin@microsoft.com), Microsoft Corporation
	Chuang Gu (chuanggu@microsoft.com), Microsoft Corporation
	Simon Winder (swinder@microsoft.com), Microsoft Corporation

in the course of development of the MPEG-4 Video (ISO/IEC 14496-2). 
This software module is an implementation of a part of one or more MPEG-4 Video tools 
as specified by the MPEG-4 Video. 
ISO/IEC gives users of the MPEG-4 Video free license to this software module or modifications 
thereof for use in hardware or software products claiming conformance to the MPEG-4 Video. 
Those intending to use this software module in hardware or software products are advised that its use may infringe existing patents. 
The original developer of this software module and his/her company, 
the subsequent editors and their companies, 
and ISO/IEC have no liability for use of this software module or modifications thereof in an implementation. 
Copyright is not released for non MPEG-4 Video conforming products. 
Microsoft retains full right to use the code for his/her own purpose, 
assign or donate the code to a third party and to inhibit third parties from using the code for non <MPEG standard> conforming products. 
This copyright notice must be included in all copies or derivative works. 

Copyright (c) 1996, 1997.


Module Name:

	wvtqtree.hpp

Abstract:

	quantisation and coding tree

Revision History:

*************************************************************************/

#ifndef __WVTQTREE_HPP_
#define __WVTQTREE_HPP_

Class CWvtQuantTree
{
public:
	CWvtQuantTree(CWvtDecompTree *);
	~CWvtQuantTree();

	CWvtQuantTree *getChild() { return m_pChild; }
	CWvtQuantTree *getParent() { return m_pParent; }
	CWvtQuantTree *getRoot();
	Int getWidth() { return m_pwvtDecomp->where().width; }
	Int getHeight() { return m_pwvtDecomp->where().height(); }
	Int getArea() { return m_pwvtDecomp->where().area(); }
	CIntImage *getOriginal() { return m_piiOrigLL; }
	void saveOriginal();
	void codeSingleQ(CWvtCoderChannel *,Int,Int,CWvtCodedStream *,CIntImage *);
	void codeMultipleQ(CWvtCoderChannel *,Int,CWvtCodedStream *,CIntImage *);
	void codeBiLevelQ(CWvtCoderChannel *,Bool,Bool,CWvtCodedStream *,CIntImage *);
	void decodeSingleQ(CInBitStream &,Int,Int,CIntImage *);
	void decodeMultipleQ(CInBitStream &,Int,Int,CIntImage *,Bool);
	void decodeBiLevelQ(CWvtCodedStream *,CIntImage *,Bool,Int,Int);

private:
	void load(FilterBank);
	void quant(Int,FilterBank);
	void invQuant(Int,FilterBank,Bool,Bool);
	void makeSNRLayer(Int,Bool,Bool);
	void restoreDecompTree(FilterBank);
	void makeZeroTree(FilterBank,Bool,Int *,Int *);
	Int loadGetMax();
	void threshBiLevel(FilterBank,Int,Bool);
	void makeZeroTreeBiLevel(FilterBank);
	Int codeZeroTreeVert(WvtArithCoderSet *,FilterBank,Int,Int);
	Int codeZeroTreeHoriz(WvtArithCoderSet *);
	Int codeZeroTreeNode(WvtArithCoderSet *,Int,Int);
	Int codeZeroTreeBandBiLevel(FilterBank,Int,WvtBiLevelACSet *);
	Int codeZeroTreeBiLevel(Int,WvtBiLevelACSet *);
	Int codeZeroTreeRefineBandBiLevel(FilterBank,Int,WvtBiLevelACSet *);
	Int codeZeroTreeRefineBiLevel(Int,WvtBiLevelACSet *);
	void decodeZeroTreeVert(WvtArithCoderSet *,FilterBank,Int,Int,Bool,Int);
	void decodeZeroTreeHoriz(WvtArithCoderSet *,Int);
	void decodeZeroTreeNode(WvtArithCoderSet *,Int,U8 *,PixelI *,U8 *,Int);
	void decodeZeroTreeBiLevel(Int *,WvtBiLevelACSet *);
	void decodeZeroTreeRefineBiLevel(Int *,WvtBiLevelACSet *);
	void decodeZeroTreeBandBiLevel(FilterBank band,Int *,WvtBiLevelACSet *);
	void decodeZeroTreeRefineBandBiLevel(FilterBank,Int *,WvtBiLevelACSet *);
	void copyQuantToWvt(FilterBank);
	void zeroQuant(FilterBank);
	void initMap(FilterBank);
	void copyResidualToWvt(FilterBank);

	CIntImage *m_piiOrigLL;
	CIntImage *m_piiQuant[4];
	CIntImage *m_piiResidual[4];
	CWvtDecompTree *m_pwvtDecomp;
	U8 *m_pucMap[4];
	CWvtQuantTree *m_pParent;
	CWvtQuantTree *m_pChild;
	QuantType m_iQuantType;
};

#endif // __WVTQTREE_HPP_