/*************************************************************************

This software module was originally developed by 

	Ming-Chieh Lee (mingcl@microsoft.com), Microsoft Corporation
	Wei-ge Chen (wchen@microsoft.com), Microsoft Corporation
	Bruce Lin (blin@microsoft.com), Microsoft Corporation
	Chuang Gu (chuanggu@microsoft.com), Microsoft Corporation
	Simon Winder (swinder@microsoft.com), Microsoft Corporation

in the course of development of the MPEG-4 Video (ISO/IEC 14496-2). 
This software module is an implementation of a part of one or more MPEG-4 Video tools 
as specified by the MPEG-4 Video. 
ISO/IEC gives users of the MPEG-4 Video free license to this software module or modifications 
thereof for use in hardware or software products claiming conformance to the MPEG-4 Video. 
Those intending to use this software module in hardware or software products are advised that its use may infringe existing patents. 
The original developer of this software module and his/her company, 
the subsequent editors and their companies, 
and ISO/IEC have no liability for use of this software module or modifications thereof in an implementation. 
Copyright is not released for non MPEG-4 Video conforming products. 
Microsoft retains full right to use the code for his/her own purpose, 
assign or donate the code to a third party and to inhibit third parties from using the code for non <MPEG standard> conforming products. 
This copyright notice must be included in all copies or derivative works. 

Copyright (c) 1996, 1997.


Module Name:

	wvtstream.cpp

Abstract:

	bitstream holding class for bi-level quantisation

Revision History:

*************************************************************************/


#include <stdio.h>
#include <stdlib.h>
#include <math.h>
#include <string.h>
#include "typeapi.h"
#include "wavelet.h"


#ifdef __MFC_
#ifdef _DEBUG
#undef THIS_FILE
static char BASED_CODE THIS_FILE[] = __FILE__;
#endif

#define new DEBUG_NEW				   
#endif // __MFC_

CWvtCodedStream::CWvtCodedStream()
{
	m_pacCodedDC=NULL;
	m_iSpatialLevels=0;
	m_pSpatialStream=NULL;
}

CWvtCodedStream::~CWvtCodedStream()
{
	if(m_pacCodedDC!=NULL)
		delete m_pacCodedDC;
	if(m_pSpatialStream!=NULL)
	{
		Int i;
		WvtSNRStream *pStream,*pTemp;

		for(i=0;i<m_iSpatialLevels;i++)
		{
			pStream=m_pSpatialStream[i].pSNRStream;
			while(pStream!=NULL)
			{
				pTemp=pStream->next;
				delete pStream;
				pStream=pTemp;
			}
		}
		delete m_pSpatialStream;
	}
}

void CWvtCodedStream::setSpatialLevels(Int iSpatialLevels)
{
	assert(m_iSpatialLevels==NULL);
	m_iSpatialLevels=iSpatialLevels;
	m_pSpatialStream = new WvtSpatialStream [iSpatialLevels];
	
	Int i;
	for(i=0;i<iSpatialLevels;i++)
	{
		m_pSpatialStream[i].iSNRLevels=0;
		m_pSpatialStream[i].pSNRStream=NULL;
	}
}

void CWvtCodedStream::attachACStream(Int iSpatialLevel,CWvtArithCoder *pacCoder,Int iQuant)
{
	assert(iSpatialLevel<m_iSpatialLevels);

	WvtSNRStream *pStream = m_pSpatialStream[iSpatialLevel].pSNRStream;
	if(pStream==NULL)
		pStream = m_pSpatialStream[iSpatialLevel].pSNRStream = new WvtSNRStream;
	else
	{
		while(pStream->next!=NULL)
			pStream = pStream->next;
		pStream->next = new WvtSNRStream;
		pStream = pStream->next;
	}
	pStream->next=NULL;
	pStream->pacCodedAC = pacCoder;
	pStream->iQuant = iQuant;
	m_pSpatialStream[iSpatialLevel].iSNRLevels++;
}

CWvtArithCoder *CWvtCodedStream::getACStream(Int iSpatialLevel,Int iSNRLevel)
{
	assert(iSpatialLevel<m_iSpatialLevels);

	WvtSNRStream *pStream = m_pSpatialStream[iSpatialLevel].pSNRStream;
	for(;pStream!=NULL && iSNRLevel>0;pStream=pStream->next)
		iSNRLevel--;
	
	assert(pStream!=NULL);

	return pStream->pacCodedAC;
}

Int CWvtCodedStream::getACQuant(Int iSpatialLevel,Int iSNRLevel)
{
	assert(iSpatialLevel<m_iSpatialLevels);

	WvtSNRStream *pStream = m_pSpatialStream[iSpatialLevel].pSNRStream;
	for(;pStream!=NULL && iSNRLevel>0;pStream=pStream->next)
		iSNRLevel--;
	
	assert(pStream!=NULL);

	return pStream->iQuant;
}
