/*************************************************************************

This software module was originally developed by 

	Ming-Chieh Lee (mingcl@microsoft.com), Microsoft Corporation
	Wei-ge Chen (wchen@microsoft.com), Microsoft Corporation
	Bruce Lin (blin@microsoft.com), Microsoft Corporation
	Chuang Gu (chuanggu@microsoft.com), Microsoft Corporation
	Simon Winder (swinder@microsoft.com), Microsoft Corporation

in the course of development of the MPEG-4 Video (ISO/IEC 14496-2). 
This software module is an implementation of a part of one or more MPEG-4 Video tools 
as specified by the MPEG-4 Video. 
ISO/IEC gives users of the MPEG-4 Video free license to this software module or modifications 
thereof for use in hardware or software products claiming conformance to the MPEG-4 Video. 
Those intending to use this software module in hardware or software products are advised that its use may infringe existing patents. 
The original developer of this software module and his/her company, 
the subsequent editors and their companies, 
and ISO/IEC have no liability for use of this software module or modifications thereof in an implementation. 
Copyright is not released for non MPEG-4 Video conforming products. 
Microsoft retains full right to use the code for his/her own purpose, 
assign or donate the code to a third party and to inhibit third parties from using the code for non <MPEG standard> conforming products. 
This copyright notice must be included in all copies or derivative works. 

Copyright (c) 1996, 1997.


Module Name:

	wvttxtcdr.cpp

Abstract:

	high level texture layer coding class

Revision History:

*************************************************************************/

#include <stdio.h>
#include <stdlib.h>
#include <math.h>
#include <string.h>
#include "typeapi.h"
#include "wavelet.h"


#ifdef __MFC_
#ifdef _DEBUG
#undef THIS_FILE
static char BASED_CODE THIS_FILE[] = __FILE__;
#endif

#define new DEBUG_NEW				   
#endif // __MFC_

CWvtTextureCoder::CWvtTextureCoder(CRct &yrct,Int iDecompLevels,FilterType type)
	: m_rct(yrct),m_iDecompLevels(iDecompLevels)
{
	Int iWidth = m_rct.width;
	Int iHeight = m_rct.height();
	m_pwvtxfoY = new CWvtTransformObject(iWidth,iHeight,iDecompLevels+1,
		type);
	m_pwvtxfoU = new CWvtTransformObject(iWidth>>1,iHeight>>1,iDecompLevels,
		type);
	m_pwvtxfoV = new CWvtTransformObject(iWidth>>1,iHeight>>1,iDecompLevels,
		type);
	resetStats();
}

CWvtTextureCoder::~CWvtTextureCoder()
{
	delete m_pwvtxfoY;
	delete m_pwvtxfoU;
	delete m_pwvtxfoV;
}

void CWvtTextureCoder::encode(WvtCoderParams *pCoderParams,COutBitStream &bs,
							  CVideoObjectPlane &vop,FILE *pfsSave)
{
	Int iWidth = m_rct.width;
	Int iHeight = m_rct.height();
	CWvtCoderChannel *pCoderChanLuma = pCoderParams->pCoderChanLuma;
	CWvtCoderChannel *pCoderChanChroma = pCoderParams->pCoderChanChroma;

	// vop dimensions must be constant and match initialised values
	assert(vop.where().width==iWidth && vop.where().height()==iHeight);
	
	// get data from vop and put in transform objects
	vop.getDownSampledPlane(*m_pwvtxfoY->image(),0,1,1);
	vop.getDownSampledPlane(*m_pwvtxfoU->image(),1,2,2);
	vop.getDownSampledPlane(*m_pwvtxfoV->image(),2,2,2);

	// extract mean
#if(DEBUG_OPS)
	printf("debug: Stripping out DC coefficient\n");
#endif
	m_pwvtxfoY->removeDC();
	m_pwvtxfoU->removeDC();
	m_pwvtxfoV->removeDC();

	// transform
#if(DEBUG_OPS)
	printf("debug: Wavelet transform\n");
#endif
	m_pwvtxfoY->forwardTransform();
	m_pwvtxfoU->forwardTransform();
	m_pwvtxfoV->forwardTransform();

	// save scaled low pass images for SNR comparison
	m_pwvtxfoY->saveOriginal();
	m_pwvtxfoU->saveOriginal();
	m_pwvtxfoV->saveOriginal();

	// dc quant
#if(DEBUG_OPS)
	printf("debug: DC quantisation\n");
#endif
	m_pwvtxfoY->quantDC(pCoderChanLuma->getDCQuant());
	m_pwvtxfoU->quantDC(pCoderChanChroma->getDCQuant());
	m_pwvtxfoV->quantDC(pCoderChanChroma->getDCQuant());

	// make coded bitstreams
#if(DEBUG_OPS)
	printf("debug: DC arithmetic coding\n");
#endif
	m_pwvtxfoY->codeDC();
	m_pwvtxfoU->codeDC();
	m_pwvtxfoV->codeDC();

#if(DEBUG_OPS)
	printf("debug: AC quantisation and coding\n");
#endif
	pCoderParams->pCoderChan = pCoderParams->pCoderChanLuma;
	CIntImage *piiSaveY = m_pwvtxfoY->codeAC(pCoderParams);
	pCoderParams->pCoderChan = pCoderParams->pCoderChanChroma;
	CIntImage *piiSaveU = m_pwvtxfoU->codeAC(pCoderParams);
	CIntImage *piiSaveV = m_pwvtxfoV->codeAC(pCoderParams);
	
#if(DEBUG_OPS)
	printf("debug: Computing stats and saving\n");
#endif
	CIntImage *piiCompareY;
	CIntImage *piiCompareU;
	CIntImage *piiCompareV;
	Float fPsnrY = 0.0f;
	Float fPsnrU = 0.0f;
	Float fPsnrV = 0.0f;

	if(piiSaveY!=NULL)
	{
		piiCompareY = m_pwvtxfoY->getOriginal(pCoderChanLuma->getSpatialSave());

		m_pwvtxfoY->insertDC(piiSaveY);
		m_pwvtxfoY->insertDC(piiCompareY);

		fPsnrY = (Float)piiCompareY->snr(*piiSaveY);
		printf("  PSNR Y = %g dB\n",fPsnrY);

		piiSaveY->dump(pfsSave);
		delete piiSaveY;
	}
	if(piiSaveU!=NULL)
	{
		piiCompareU = m_pwvtxfoU->getOriginal(pCoderChanChroma->getSpatialSave());

		m_pwvtxfoU->insertDC(piiSaveU);
		m_pwvtxfoU->insertDC(piiCompareU);

		fPsnrU = (Float)piiCompareU->snr(*piiSaveU);
		printf("  PSNR U = %g dB\n",fPsnrU);

		piiSaveU->dump(pfsSave);
		delete piiSaveU;
	}
	if(piiSaveV!=NULL)
	{
		piiCompareV = m_pwvtxfoV->getOriginal(pCoderChanChroma->getSpatialSave());

		m_pwvtxfoV->insertDC(piiSaveV);
		m_pwvtxfoV->insertDC(piiCompareV);

		fPsnrV = (Float)piiCompareV->snr(*piiSaveV);
		printf("  PSNR V = %g dB\n",fPsnrV);

		piiSaveV->dump(pfsSave);
		delete piiSaveV;
	}

#if(DEBUG_OPS)
	printf("debug: Writing bitstream\n");
#endif
	Int iBits=0;
	bs.putBits(TOL_START_CODE,NUMBITS_TOL_START_CODE,"Start Code");
#if(DEBUG_BITS)
	printf("  Start code\t\t\t= %d\t(%d bits)\n",TOL_START_CODE,
		NUMBITS_TOL_START_CODE);
#endif

	bs.putBits(pCoderParams->iObjectID,NUMBITS_TOL_ID,"ID");
#if(DEBUG_BITS)
	printf("  Object ID\t\t\t= %d\t(%d bits)\n",pCoderParams->iObjectID,
		NUMBITS_TOL_ID);
#endif
	
	bs.putBits((Int)0,NUMBITS_TOL_SHAPE,"Shape");
#if(DEBUG_BITS)
	printf("  Shape code\t\t\t= %d\t(%d bits)\n",0,NUMBITS_TOL_SHAPE);
#endif
	
	bs.putBits(m_rct.width,NUMBITS_TOL_SIZE,"Width");
#if(DEBUG_BITS)
	printf("  Width\t\t\t\t= %d\t(%d bits)\n",m_rct.width,NUMBITS_TOL_SIZE);
#endif

	bs.putBits(m_rct.height(),NUMBITS_TOL_SIZE,"Height");
#if(DEBUG_BITS)
	printf("  Height\t\t\t= %d\t(%d bits)\n",m_rct.height(),NUMBITS_TOL_SIZE);
#endif
	
	bs.putBits(m_iDecompLevels,NUMBITS_TOL_DECOMP,"Decomp Levels");
#if(DEBUG_BITS)
	printf("  Decomp levels\t\t\t= %d\t(%d bits)\n",m_iDecompLevels,
		NUMBITS_TOL_DECOMP);
#endif
	iBits += NUMBITS_TOL_START_CODE+NUMBITS_TOL_ID+NUMBITS_TOL_SHAPE+NUMBITS_TOL_SIZE+
		NUMBITS_TOL_SIZE+NUMBITS_TOL_DECOMP;
	
#if(DEBUG_BITS)
	printf("  Y: ");
#endif
	iBits += m_pwvtxfoY->dumpDCMean(bs);
#if(DEBUG_BITS)
	printf("  U: ");
#endif
	iBits += m_pwvtxfoU->dumpDCMean(bs);
#if(DEBUG_BITS)
	printf("  V: ");
#endif
	iBits += m_pwvtxfoV->dumpDCMean(bs);
	
	bs.putBits(pCoderChanLuma->getDCQuant(),NUMBITS_TOL_DCQUANT,"Luma DC Quant");
#if(DEBUG_BITS)
	printf("  Luma DC quant\t\t\t= %d\t(%d bits)\n",
		pCoderChanLuma->getDCQuant(),NUMBITS_TOL_DCQUANT);
#endif

	bs.putBits(pCoderChanChroma->getDCQuant(),NUMBITS_TOL_DCQUANT,"Chroma DC Quant");
#if(DEBUG_BITS)
	printf("  Chroma DC quant\t\t= %d\t(%d bits)\n",
		pCoderChanLuma->getDCQuant(),NUMBITS_TOL_DCQUANT);
#endif
	iBits += NUMBITS_TOL_DCQUANT+NUMBITS_TOL_DCQUANT;

	// dc coding
	pCoderParams->pCoderChan = pCoderParams->pCoderChanLuma;
#if(DEBUG_BITS)
	printf("  Y: ");
#endif
	iBits += m_pwvtxfoY->dumpDCCoding(pCoderParams,bs);
	pCoderParams->pCoderChan = pCoderParams->pCoderChanChroma;
#if(DEBUG_BITS)
	printf("  U: ");
#endif
	iBits += m_pwvtxfoU->dumpDCCoding(pCoderParams,bs);
#if(DEBUG_BITS)
	printf("  V: ");
#endif
	iBits += m_pwvtxfoV->dumpDCCoding(pCoderParams,bs);

	Int iSLevels = pCoderChanLuma->getSpatialLevels();
	if(pCoderParams->qtQuantType==QT_BILEVEL)
	{
		if(pCoderParams->bSpatialMarker==TRUE)
			iSLevels = m_iDecompLevels;
		else
			iSLevels = 1;
	}

	bs.putBits(iSLevels,NUMBITS_TOL_SPA_SCALE,"Spatial Scale Levels");
#if(DEBUG_BITS)
	printf("  Spatial scale levels\t\t= %d\t(%d bits)\n",
		iSLevels,NUMBITS_TOL_SPA_SCALE);
#endif

	bs.putBits((Int)pCoderParams->qtQuantType,NUMBITS_TOL_QUANT_TYPE,
		"Quant Type");
#if(DEBUG_BITS)
	printf("  Quant type\t\t\t= %d\t(%d bits)\n",(Int)pCoderParams->qtQuantType,
		NUMBITS_TOL_QUANT_TYPE);
#endif

	bs.putBits((pCoderParams->bLengthMarker==TRUE)?1:0,1);
#if(DEBUG_BITS)
	printf("  Snr length marker\t\t= %d\t(1 bits)\n",
		(pCoderParams->bLengthMarker==TRUE)?1:0);
#endif

	iBits += NUMBITS_TOL_SPA_SCALE+NUMBITS_TOL_QUANT_TYPE+1;

	// ac coding
	pCoderParams->pCoderChan = pCoderParams->pCoderChanLuma;
#if(DEBUG_BITS)
	printf("  Y: ");
#endif
	iBits += m_pwvtxfoY->dumpACCoding(pCoderParams,bs,TRUE,TRUE);
	pCoderParams->pCoderChan = pCoderParams->pCoderChanChroma;
#if(DEBUG_BITS)
	printf("  U: ");
#endif
	iBits += m_pwvtxfoU->dumpACCoding(pCoderParams,bs,FALSE,TRUE);
#if(DEBUG_BITS)
	printf("  V: ");
#endif
	iBits += m_pwvtxfoV->dumpACCoding(pCoderParams,bs,FALSE,FALSE);

	printf("\n  Total length = %d bits\n",iBits);

	updateStats(fPsnrY,fPsnrU,fPsnrV,iBits);
}

CVideoObjectPlane *CWvtTextureCoder::decode(CInBitStream &bs,
											Int iSpatialLayer,Int iSNRLayer)
{
#if(DEBUG_BITS)
	printf("Y: ");
#endif
	m_pwvtxfoY->readDCMean(bs);
#if(DEBUG_BITS)
	printf("U: ");
#endif
	m_pwvtxfoU->readDCMean(bs);
#if(DEBUG_BITS)
	printf("V: ");
#endif
	m_pwvtxfoV->readDCMean(bs);

	Int iQuantLuma = bs.getBits(NUMBITS_TOL_DCQUANT);
	Int iQuantChroma = bs.getBits(NUMBITS_TOL_DCQUANT);
#if(DEBUG_BITS)
	printf("Luma quant\t\t= %d\n",iQuantLuma);
	printf("Chroma quant\t\t= %d\n",iQuantLuma);
#endif

#if(DEBUG_BITS)
	printf("Y: ");
#endif
	m_pwvtxfoY->readDCCoding(bs);
#if(DEBUG_BITS)
	printf("U: ");
#endif
	m_pwvtxfoU->readDCCoding(bs);
#if(DEBUG_BITS)
	printf("V: ");
#endif
	m_pwvtxfoV->readDCCoding(bs);

	m_pwvtxfoY->invquantDC(iQuantLuma);
	m_pwvtxfoU->invquantDC(iQuantChroma);
	m_pwvtxfoV->invquantDC(iQuantChroma);

	WvtDecodeInfo wvtDecode;
	wvtDecode.iSpatialLevels = bs.getBits(NUMBITS_TOL_SPA_SCALE);
	wvtDecode.qtQuantType = (QuantType)bs.getBits(NUMBITS_TOL_QUANT_TYPE);
	wvtDecode.bSNRLengthMarker = (Bool)bs.getBits(1);
	wvtDecode.piMultiQuant = NULL;
	wvtDecode.iReqSpatial = iSpatialLayer;
	wvtDecode.iReqSNR = iSNRLayer;
#if(DEBUG_BITS)
	printf("Spatial levels\t\t= %d\n",wvtDecode.iSpatialLevels);
	printf("Quant type\t\t= %d\n",(Int)wvtDecode.qtQuantType);
	printf("SNR length enable\t= %d\n",(Int)wvtDecode.bSNRLengthMarker);
#endif

#if(DEBUG_BITS || DEBUG_DECODE)
	printf("Y: ");
#endif
	CIntImage *piiSaveY = m_pwvtxfoY->readACCoding(&wvtDecode,bs,TRUE,TRUE);
#if(DEBUG_BITS || DEBUG_DECODE)
	printf("U: ");
#endif
	CIntImage *piiSaveU = m_pwvtxfoU->readACCoding(&wvtDecode,bs,FALSE,TRUE);
#if(DEBUG_BITS || DEBUG_DECODE)
	printf("V: ");
#endif
	CIntImage *piiSaveV = m_pwvtxfoV->readACCoding(&wvtDecode,bs,FALSE,FALSE);

	m_pwvtxfoY->insertDC(piiSaveY);
	m_pwvtxfoU->insertDC(piiSaveU);
	m_pwvtxfoV->insertDC(piiSaveV);

	CVideoObjectPlane *pvop = new CVideoObjectPlane(piiSaveY->where());

	// transfer save images to pvop
	pvop->setUpSampledPlane(*piiSaveY,0,1,1);
	pvop->setUpSampledPlane(*piiSaveU,1,2,2);
	pvop->setUpSampledPlane(*piiSaveV,2,2,2);

	delete piiSaveY;
	delete piiSaveU;
	delete piiSaveV;

	if(wvtDecode.piMultiQuant!=NULL)
		delete wvtDecode.piMultiQuant;

	return pvop;
}

void CWvtTextureCoder::resetStats()
{
	m_fPsnrY=0.0f;
	m_fPsnrU=0.0f;
	m_fPsnrV=0.0f;
	m_iBitCount=0;
}

void CWvtTextureCoder::dumpStats()
{
	printf("\nCumulative statistics:\n");
	printf("  PSNR Y = %g dB\n",m_fPsnrY);
	printf("  PSNR U = %g dB\n",m_fPsnrU);
	printf("  PSNR V = %g dB\n",m_fPsnrV);
	printf("\n  Total length = %d bits\n",m_iBitCount);
}

void CWvtTextureCoder::updateStats(Float fPsnrY,Float fPsnrU,Float fPsnrV,
								   Int iBitCount)
{
	m_fPsnrY += fPsnrY;
	m_fPsnrU += fPsnrU;
	m_fPsnrV += fPsnrV;
	m_iBitCount += iBitCount;
}
