/*************************************************************************

This software module was originally developed by 

	Ming-Chieh Lee (mingcl@microsoft.com), Microsoft Corporation
	Wei-ge Chen (wchen@microsoft.com), Microsoft Corporation
	Bruce Lin (blin@microsoft.com), Microsoft Corporation
	Chuang Gu (chuanggu@microsoft.com), Microsoft Corporation
	Simon Winder (swinder@microsoft.com), Microsoft Corporation

in the course of development of the MPEG-4 Video (ISO/IEC 14496-2). 
This software module is an implementation of a part of one or more MPEG-4 Video tools 
as specified by the MPEG-4 Video. 
ISO/IEC gives users of the MPEG-4 Video free license to this software module or modifications 
thereof for use in hardware or software products claiming conformance to the MPEG-4 Video. 
Those intending to use this software module in hardware or software products are advised that its use may infringe existing patents. 
The original developer of this software module and his/her company, 
the subsequent editors and their companies, 
and ISO/IEC have no liability for use of this software module or modifications thereof in an implementation. 
Copyright is not released for non MPEG-4 Video conforming products. 
Microsoft retains full right to use the code for his/her own purpose, 
assign or donate the code to a third party and to inhibit third parties from using the code for non <MPEG standard> conforming products. 
This copyright notice must be included in all copies or derivative works. 

Copyright (c) 1996, 1997.


Module Name:

	wvtxfobj.cpp

Abstract:

	coding class, one for each component in YUV

Revision History:

*************************************************************************/

#include <stdio.h>
#include <stdlib.h>
#include <math.h>
#include <string.h>
#include "typeapi.h"
#include "wavelet.h"

#ifdef __MFC_
#ifdef _DEBUG
#undef THIS_FILE
static char BASED_CODE THIS_FILE[] = __FILE__;
#endif

#define new DEBUG_NEW				   
#endif // __MFC_

CWvtTransformObject::CWvtTransformObject(Int iWidth,Int iHeight,Int iDecompLevels,
										 FilterType ftype)
	: m_iDecompLevels(iDecompLevels)
{
	m_piiImage = new CIntImage(CRct(0,0,iWidth,iHeight));
	m_pwvtDecomp = new CWvtDecompTree(iWidth,iHeight,iDecompLevels,ftype);
	m_pwvtQuant = new CWvtQuantTree(m_pwvtDecomp);
	m_piiQDC = new CIntImage(m_pwvtDecomp->dcBand()->where());
	m_pwvtCodedStream = NULL;
}

CWvtTransformObject::~CWvtTransformObject()
{
	delete m_piiImage;
	delete m_pwvtQuant;
	delete m_pwvtDecomp;
	delete m_piiQDC;
	if(m_pwvtCodedStream!=NULL)
		delete m_pwvtCodedStream;
}

void CWvtTransformObject::removeDC()
{
	PixelI *ppxliBase = (PixelI *)image()->pixels();
	Int iArea = image()->where().area();
	PixelI *ppxli;
	Int i,iSum;

	iSum=0;
	for(ppxli=ppxliBase,i=0;i<iArea;i++)
		iSum+=(*ppxli++);
	iSum=(Int)((Double)iSum/iArea+0.5);
	for(ppxli=ppxliBase,i=0;i<iArea;i++,ppxli++)
		*ppxli-=iSum;
	m_iDCVal=iSum;
#if(DEBUG_OPS)
	printf("  DC mean = %d\n",iSum);
#endif
}

void CWvtTransformObject::insertDC(CIntImage *pii)
{
	PixelI *ppxliBase = (PixelI *)pii->pixels();
	Int iArea = pii->where().area();
	PixelI *ppxli;
	Int i,iVal;

	for(ppxli=ppxliBase,i=0;i<iArea;i++,ppxli++)
	{
		iVal=*ppxli+m_iDCVal;
		*ppxli=checkrange(iVal,0,255);
	}
}

void CWvtTransformObject::forwardTransform()
{
	m_pwvtDecomp->forwardTransform(m_piiImage);

	//m_pwvtDecomp->roundNearestInt(WVT_LL);
	//m_pwvtDecomp->roundNearestInt(WVT_LH);
	//m_pwvtDecomp->roundNearestInt(WVT_HL);
	//m_pwvtDecomp->roundNearestInt(WVT_HH);
}

void CWvtTransformObject::inverseTransform(CIntImage *piiDst,Int iScaleLevel)
{
	CWvtDecompTree *pwvtTemp = m_pwvtDecomp;
	assert(iScaleLevel>=0 && iScaleLevel<m_iDecompLevels);

	if(piiDst==NULL)
		piiDst=m_piiImage;
	for(;iScaleLevel>0;iScaleLevel--)
		pwvtTemp=pwvtTemp->getChild();
	pwvtTemp->inverseTransform(piiDst);
}

void CWvtTransformObject::saveOriginal()
{
	m_pwvtQuant->saveOriginal();
}

CIntImage *CWvtTransformObject::getOriginal(Int iSpatialLayer)
{
	Int i;

	if(iSpatialLayer==0)
		return m_piiImage;
	CWvtQuantTree *pwvtQuant = m_pwvtQuant;
	for(i=iSpatialLayer-1;i>0 && pwvtQuant!=NULL;i--)
		pwvtQuant=pwvtQuant->getChild();
	assert(pwvtQuant!=NULL);
	return pwvtQuant->getOriginal();
}

void CWvtTransformObject::quantDC(Int iQ)
{
	CFloatImage *pfiDCBand = m_pwvtDecomp->dcBand();
	PixelF *ppxlfDCBand = (PixelF *)pfiDCBand->pixels();
	PixelI *ppxliQCoefs = (PixelI *)m_piiQDC->pixels();
	Int iA,iB,iC,iPred,iQuant;
	Float fTemp;
	Int iWidth = pfiDCBand->where().width;
	Int iHeight = pfiDCBand->where().height();
	Int iX,iY,iMin,iMax;

	fTemp = ppxlfDCBand[0];
	ppxliQCoefs[0] = iQuant = DCQuant(fTemp,iQ);
	ppxlfDCBand[0] = (Float)(iA = InvDCQuant(iQuant,iQ));
	iMin = iMax = iQuant;

	for(iX=1;iX<iWidth;iX++)
	{
		fTemp = ppxlfDCBand[iX] - iA;
		ppxliQCoefs[iX] = iQuant = DCQuant(fTemp,iQ);
		ppxlfDCBand[iX] = (Float)(iA = InvDCQuant(iQuant,iQ) + iA);
		if(iQuant<iMin)
			iMin=iQuant;
		else if(iQuant>iMax)
			iMax=iQuant;
	}
	ppxlfDCBand+=iWidth;
	ppxliQCoefs+=iWidth;
	for(iY=1;iY<iHeight;iY++,ppxlfDCBand+=iWidth,ppxliQCoefs+=iWidth)
	{
		iB = (Int)ppxlfDCBand[-iWidth];
		fTemp = ppxlfDCBand[0] - iB;
		ppxliQCoefs[0] = iQuant = DCQuant(fTemp,iQ);
		ppxlfDCBand[0] = (Float)(iA = InvDCQuant(iQuant,iQ) + iB);
		if(iQuant<iMin)
			iMin=iQuant;
		else if(iQuant>iMax)
			iMax=iQuant;

		for(iX=1;iX<iWidth;iX++)
		{
			iC = (Int)ppxlfDCBand[iX-iWidth];
			if(abs(iA-iB)<abs(iA-iC))
				iPred=iC;
			else
				iPred=iA;
			fTemp = ppxlfDCBand[iX] - iPred;
			ppxliQCoefs[iX] = iQuant = DCQuant(fTemp,iQ);
			ppxlfDCBand[iX] = (Float)(iA = InvDCQuant(iQuant,iQ) + iPred);
			iB=iC;
			if(iQuant<iMin)
				iMin=iQuant;
			else if(iQuant>iMax)
				iMax=iQuant;
		}
	}
	m_iBandOffset=iMin;
	m_iBandRange=iMax-iMin;
#if(DEBUG_OPS)
	printf("  DC min value = %d, DC range (# symbols) = %d\n",
		m_iBandOffset,m_iBandRange+1);
#endif
}

void CWvtTransformObject::codeDC()
{
	Int i;
	CWvtArithCoderModel acModel(AC_ADAPTIVE,m_iBandRange+1);

	CWvtArithCoder *pacCodedDC = new CWvtArithCoder();
	pacCodedDC->encodeStart();

	Int iArea=m_piiQDC->where().area();
	const PixelI *ppxliQCoefs = m_piiQDC->pixels();
	for(i=0;i<iArea;i++,ppxliQCoefs++)
		pacCodedDC->encodeSymbol(acModel,*ppxliQCoefs - m_iBandOffset);
	pacCodedDC->encodeEnd();

#if(DEBUG_OPS)
	printf("  Total symbols = %d, Total bits = %d\n",iArea,pacCodedDC->getBitCount());
#endif

	if(m_pwvtCodedStream!=NULL)
		delete m_pwvtCodedStream;
	m_pwvtCodedStream = new CWvtCodedStream;
	m_pwvtCodedStream->attachDCStream(pacCodedDC);
}

CIntImage *CWvtTransformObject::codeAC(WvtCoderParams *pCoderParams)
{
	Int i;
	CIntImage *piiSaveImage = NULL;
	CWvtCoderChannel *pCoderChan = pCoderParams->pCoderChan;
	Int iSpatialLevels = pCoderChan->getSpatialLevels();

	assert(m_pwvtCodedStream!=NULL);
	switch(pCoderParams->qtQuantType)
	{
	case QT_SINGLE:
		m_pwvtCodedStream->setSpatialLevels(iSpatialLevels);
		for(i=0;i<iSpatialLevels;i++)
		{
			CWvtQuantTree *pwvtQuant = m_pwvtQuant;
			Int iSkip;
			for(iSkip=i;iSkip>0;iSkip--)
				pwvtQuant=pwvtQuant->getChild();

			if(pCoderChan->getSpatialSave()==i)
				piiSaveImage = new CIntImage(CRct(0,0,pwvtQuant->getWidth()<<1,
					pwvtQuant->getHeight()<<1));
			pwvtQuant->codeSingleQ(pCoderChan,i,0,m_pwvtCodedStream,piiSaveImage);
		}
		break;
	case QT_MULTIPLE:
		m_pwvtCodedStream->setSpatialLevels(iSpatialLevels);
		for(i=0;i<iSpatialLevels;i++)
		{
			CWvtQuantTree *pwvtQuant = m_pwvtQuant;
			Int iSkip;
			for(iSkip=i;iSkip>0;iSkip--)
				pwvtQuant=pwvtQuant->getChild();

			if(pCoderChan->getSpatialSave()==i)
				piiSaveImage = new CIntImage(CRct(0,0,pwvtQuant->getWidth()<<1,
					pwvtQuant->getHeight()<<1));

			pwvtQuant->codeMultipleQ(pCoderChan,i,m_pwvtCodedStream,piiSaveImage);
		}
		break;
	case QT_BILEVEL:
		// encoder cant save different snr/spatial image qualities
		if(pCoderChan->checkSave(0,0))
			piiSaveImage = new CIntImage(CRct(0,0,m_pwvtQuant->getWidth()<<1,
				m_pwvtQuant->getHeight()<<1));

		// if more than one spatial layer is required, code them all separately
		// if length markers are enabled, code all snr layers separately
		m_pwvtQuant->codeBiLevelQ(pCoderChan,pCoderParams->bSpatialMarker,
			pCoderParams->bLengthMarker,m_pwvtCodedStream,piiSaveImage);
		break;
	default:
		assert(FALSE);
		break;
	}
	return piiSaveImage;
}

Int CWvtTransformObject::dumpDCMean(COutBitStream &bs)
{
	bs.putBits(m_iDCVal,NUMBITS_TOL_DCMEAN,"DC Mean");
#if(DEBUG_BITS)
	printf("DC mean\t\t\t= %d\t(%d bits)\n",m_iDCVal,NUMBITS_TOL_DCMEAN);
#endif
	return NUMBITS_TOL_DCMEAN;
}

Int CWvtTransformObject::dumpDCCoding(WvtCoderParams *pCoderParams,COutBitStream &bs)
{
	Int iBits,iVLC;
	CWvtArithCoder *pacCodedDC = m_pwvtCodedStream->getDCStream();
	assert(pacCodedDC!=NULL);

	bs.putBits(pacCodedDC->getBitCount(),NUMBITS_TOL_DCLENGTH,
		"DC Bitstream Length");
#if(DEBUG_BITS)
	printf("DC bitstream length\t= %d\t(%d bits)\n",pacCodedDC->getBitCount(),
		NUMBITS_TOL_DCLENGTH);
#endif
	iBits=NUMBITS_TOL_DCLENGTH;

	// band offset is always negative, but we code as a positive value
	iBits += iVLC = dumpVLCValue(bs,-m_iBandOffset,NUMBITS_AC_DCBANDBITS,
		"Band Offset");
#if(DEBUG_BITS)
	printf("     DC band offset\t\t= %d\t(%d bits)\n",-m_iBandOffset,iVLC);
#endif
	iBits += iVLC = dumpVLCValue(bs,m_iBandRange,NUMBITS_AC_DCBANDBITS,
		"Band Range");
#if(DEBUG_BITS)
	printf("     DC band range\t\t= %d\t(%d bits)\n",m_iBandRange,iVLC);
#endif
	iBits += pacCodedDC->encodeWriteStream(bs);
#if(DEBUG_BITS)
	printf("     DC coded bitstream\t\t= [...]\t(%d bits)\n",
		pacCodedDC->getBitCount());
#endif

	return iBits;
}

Int CWvtTransformObject::dumpACCoding(WvtCoderParams *pCoderParams,COutBitStream &bs,
									  Bool bDumpSNRFlag,Bool bDumpQuantFlag)
{
	Int i,j,iBits,iTotal=0,iVLC;
	CWvtCoderChannel *pCoderChan = pCoderParams->pCoderChan;
	CWvtArithCoder *pacCodedAC;

	switch(pCoderParams->qtQuantType)
	{
	case QT_SINGLE:
#if(DEBUG_BITS)
		printf("Single quantiser\n");
#endif
		for(i=0;i<pCoderChan->getSpatialLevels();i++)
		{
#if(DEBUG_BITS)
			printf("     Spatial level %d:\n",i);
#endif
			iBits = 0;
			pacCodedAC = m_pwvtCodedStream->getACStream(i,0);

			bs.putBits(pacCodedAC->getBitCount(),NUMBITS_TOL_ACLENGTH,
				"AC Bitstream Length");
#if(DEBUG_BITS)
			printf("     AC bitstream length\t= %d\t(%d bits)\n",
				pacCodedAC->getBitCount(),NUMBITS_TOL_ACLENGTH);
#endif
			iBits += NUMBITS_TOL_ACLENGTH;

			if(bDumpQuantFlag==TRUE)
			{
				bs.putBits(pCoderChan->getSNRACQuant(i,0),NUMBITS_TOL_ACQUANT,
					"AC Quant");
#if(DEBUG_BITS)
				printf("     AC quant\t\t\t= %d\t(%d bits)\n",
					pCoderChan->getSNRACQuant(i,0),NUMBITS_TOL_ACQUANT);
#endif
				iBits += NUMBITS_TOL_ACQUANT;
			}

			iBits += pacCodedAC->encodeWriteStream(bs);
#if(DEBUG_BITS)
			printf("     AC coded bitstream\t\t= [...]\t(%d bits)\n",
				pacCodedAC->getBitCount());
#endif

			iTotal += iBits;
		}
		break;
	case QT_MULTIPLE:
#if(DEBUG_BITS)
		printf("Multiple quantiser\n");
#endif
		for(i=0;i<pCoderChan->getSpatialLevels();i++)
		{
			iBits=0;
			// count all bits inside the snr loop
			for(j=0;j<pCoderChan->getSNRLevels(i);j++)
			{
				Int iBitsSnr = m_pwvtCodedStream->getACStream(i,j)->getBitCount();
				if(pCoderParams->bLengthMarker==TRUE)
					iBitsSnr += countVLCBits(iBitsSnr,NUMBITS_AC_SNRLENGTH);

				iBitsSnr += 1;
				if(bDumpQuantFlag==TRUE)
					iBitsSnr += NUMBITS_TOL_ACQUANT;
				iBits += iBitsSnr;
			}
#if(DEBUG_BITS)
			printf("     Spatial level %d:\n",i);
#endif
			bs.putBits(iBits,NUMBITS_TOL_ACLENGTH,"AC Bitstream Length");
#if(DEBUG_BITS)
			printf("     AC bitstream length\t= %d\t(%d bits)\n",
				iBits,NUMBITS_TOL_ACLENGTH);
#endif
			iBits += NUMBITS_TOL_ACLENGTH;

			if(bDumpSNRFlag==TRUE)
			{
				bs.putBits(pCoderChan->getSNRLevels(i),NUMBITS_TOL_SNR_SCALE,
					"TOL SNR Scalability Levels");
#if(DEBUG_BITS)
				printf("     Snr scalability levels\t= %d\t(%d bits)\n",
					pCoderChan->getSNRLevels(i),NUMBITS_TOL_SNR_SCALE);
#endif
				iBits += NUMBITS_TOL_SNR_SCALE;
			}

			for(j=0;j<pCoderChan->getSNRLevels(i);j++)
			{
#if(DEBUG_BITS)
				printf("     Snr level %d:\n",j);
#endif
				if(bDumpQuantFlag==TRUE)
				{
					bs.putBits(pCoderChan->getSNRACQuant(i,j),NUMBITS_TOL_ACQUANT,
						"AC Quant");
#if(DEBUG_BITS)
					printf("       AC quant\t\t\t= %d\t(%d bits)\n",
						pCoderChan->getSNRACQuant(i,j),NUMBITS_TOL_ACQUANT);
#endif
				}

				bs.putBits((Int)0,1,"SNR All Zero");
#if(DEBUG_BITS)
				printf("       Snr all zero flag\t= %d\t(%d bits)\n",0,1);
#endif

				if(pCoderParams->bLengthMarker==TRUE)
				{
					iVLC = dumpVLCValue(bs,m_pwvtCodedStream->getACStream(i,j)->getBitCount(),
								 NUMBITS_AC_SNRLENGTH,"SNR Bitstream Length");
#if(DEBUG_BITS)
					printf("       Snr bitstream length\t= %d\t(%d bits)\n",
						m_pwvtCodedStream->getACStream(i,j)->getBitCount(),iVLC);
#endif
				}
				m_pwvtCodedStream->getACStream(i,j)->encodeWriteStream(bs);
#if(DEBUG_BITS)
				printf("       AC coded bitstream\t= [...]\t(%d bits)\n",
					m_pwvtCodedStream->getACStream(i,j)->getBitCount());
#endif
			}

			iTotal += iBits;
		}
		break;
	case QT_BILEVEL:
#if(DEBUG_BITS)
		printf("Bi-level quantiser\n");
#endif
		for(i=0;i<m_pwvtCodedStream->getSpatialLevels();i++)
		{
			iBits = 0;

			for(j=0;j<m_pwvtCodedStream->getSNRLevels(i);j++)
			{
				Int iBitsSnr = m_pwvtCodedStream->getACStream(i,j)->getBitCount();
				if(pCoderParams->bLengthMarker==TRUE)
					iBitsSnr += countVLCBits(iBitsSnr,NUMBITS_AC_SNRLENGTH);
				
				iBits += iBitsSnr;
			}

#if(DEBUG_BITS)
			printf("     Spatial level %d:\n",i);
#endif

			bs.putBits(iBits,NUMBITS_TOL_ACLENGTH,"AC Bitstream Length");
#if(DEBUG_BITS)
			printf("     AC bitstream length\t= %d\t(%d bits)\n",iBits,
				NUMBITS_TOL_ACLENGTH);
#endif

			bs.putBits(m_pwvtCodedStream->getQuant(),NUMBITS_TOL_ACBILEVELQ,
				"Bi-Level Quantiser");
#if(DEBUG_BITS)
			printf("     AC quant\t\t\t= %d\t(%d bits)\n",
				m_pwvtCodedStream->getQuant(),NUMBITS_TOL_ACBILEVELQ);
#endif
			iBits += NUMBITS_TOL_ACLENGTH + NUMBITS_TOL_ACBILEVELQ;

			for(j=0;j<m_pwvtCodedStream->getSNRLevels(i);j++)
			{
#if(DEBUG_BITS)
				printf("     Snr level %d:\n",j);
#endif

				if(pCoderParams->bLengthMarker==TRUE)
				{
					iVLC = dumpVLCValue(bs,m_pwvtCodedStream->getACStream(i,j)->getBitCount(),
								 NUMBITS_AC_SNRLENGTH,"SNR Bitstream Length");
#if(DEBUG_BITS)
					printf("       Snr bitsream length\t= %d\t(%d bits)\n",
						m_pwvtCodedStream->getACStream(i,j)->getBitCount(),iVLC);
#endif
				}
				m_pwvtCodedStream->getACStream(i,j)->encodeWriteStream(bs);
#if(DEBUG_BITS)
				printf("       AC coded bitstream\t= [...]\t(%d bits)\n",
					m_pwvtCodedStream->getACStream(i,j)->getBitCount());
#endif
			}

			iTotal += iBits;
		}
		break;
	default:
		assert(FALSE);
		break;
	}

	return iTotal;
}

Int CWvtTransformObject::dumpVLCValue(COutBitStream &bs,Int iValue,Int iLength,
								 Char *pchSymbol)
{
	Int iTemp,iBits = 0;
	Int iMask = (1<<(iLength))-1;

	for(iTemp=iValue;iTemp>iMask;iTemp>>=iLength)
	{
		bs.putBits(1,1,pchSymbol);
		bs.putBits(iTemp&iMask,iLength,pchSymbol);
		iBits+=iLength+1;
	}
	bs.putBits((Int)0,1,pchSymbol);
	bs.putBits(iTemp&iMask,iLength,pchSymbol);
	iBits+=iLength+1;

	return iBits;
}

void CWvtTransformObject::readDCMean(CInBitStream &bs)
{
	m_iDCVal = bs.getBits(NUMBITS_TOL_DCMEAN);
#if(DEBUG_BITS)
	printf("DC mean\t\t= %d\n",m_iDCVal);
#endif
}

void CWvtTransformObject::readDCCoding(CInBitStream &bs)
{
	Int iLength = bs.getBits(NUMBITS_TOL_DCLENGTH);
#if(DEBUG_BITS)
	printf("DC length\t\t= %d\n",iLength);
#endif

	m_iBandOffset = -readVLCValue(bs,NUMBITS_AC_DCBANDBITS);
	m_iBandRange = readVLCValue(bs,NUMBITS_AC_DCBANDBITS);
#if(DEBUG_BITS)
	printf("   DC band offset\t= %d\n",m_iBandOffset);
	printf("   DC band range\t= %d\n",m_iBandRange);
#endif
	CWvtArithCoderModel acModel(AC_ADAPTIVE,m_iBandRange+1);
	CWvtArithCoder acCodedDC;

	// decode using arithmetic coder
	acCodedDC.decodeStart(bs,iLength);

	Int iArea = m_piiQDC->where().area();
	Int i;
	PixelI *ppxliQCoefs = (PixelI *)m_piiQDC->pixels();
	for(i=0;i<iArea;i++,ppxliQCoefs++)
		*ppxliQCoefs = acCodedDC.decodeSymbol(acModel) + m_iBandOffset;

	acCodedDC.decodeEnd();
}

void CWvtTransformObject::invquantDC(Int iQ)
{
	CFloatImage *pfiDCBand = m_pwvtDecomp->dcBand();
	PixelF *ppxlfDCBand = (PixelF *)pfiDCBand->pixels();
	PixelI *ppxliQCoefs = (PixelI *)m_piiQDC->pixels();
	Int iWidth = pfiDCBand->where().width;
	Int iHeight = pfiDCBand->where().height();
	Int iQuant,iPred,iA,iB,iC;
	Int iX,iY;

	// top left
	iQuant = ppxliQCoefs[0];
	ppxlfDCBand[0] = (Float)(iA = InvDCQuant(iQuant,iQ));
	
	// top row
	for(iX=1;iX<iWidth;iX++)
	{
		iQuant = ppxliQCoefs[iX];
		ppxlfDCBand[iX] = (Float)(iA = InvDCQuant(iQuant,iQ) + iA);
	}
	ppxlfDCBand+=iWidth;
	ppxliQCoefs+=iWidth;
	for(iY=1;iY<iHeight;iY++,ppxlfDCBand+=iWidth,ppxliQCoefs+=iWidth)
	{
		// left edge
		iB = (Int)ppxlfDCBand[-iWidth];
		iQuant = ppxliQCoefs[0];
		ppxlfDCBand[0] = (Float)(iA = InvDCQuant(iQuant,iQ) + iB);

		// centre
		for(iX=1;iX<iWidth;iX++)
		{
			iC = (Int)ppxlfDCBand[iX-iWidth];
			if(abs(iA-iB)<abs(iA-iC))
				iPred=iC;
			else
				iPred=iA;
			iQuant = ppxliQCoefs[iX];
			ppxlfDCBand[iX] = (Float)(iA = InvDCQuant(iQuant,iQ) + iPred);
			iB=iC;
		}
	}
}

CIntImage *CWvtTransformObject::readACCoding(WvtDecodeInfo *pwvtDecode,
									   CInBitStream &bs,
									   Bool bReadSNRFlag,Bool bReadQuantFlag)
{
	Int i;
	CWvtQuantTree *pwvtQuantTree = m_pwvtQuant;
	CIntImage *piiSave = NULL;

	switch(pwvtDecode->qtQuantType)
	{
	case QT_SINGLE:
#if(DEBUG_BITS)
		printf("Single quantiser\n");
#endif
		if(pwvtDecode->iReqSpatial>=pwvtDecode->iSpatialLevels)
			pwvtDecode->iReqSpatial = pwvtDecode->iSpatialLevels-1;
		for(i=0;i<pwvtDecode->iSpatialLevels;i++)
		{
#if(DEBUG_BITS)
			printf("   Spatial layer: %d\n",i);
#endif
			
			Int iLength = bs.getBits(NUMBITS_TOL_ACLENGTH);
#if(DEBUG_BITS)
			printf("   Bitstream length\t= %d\n",iLength);
#endif

			if(bReadQuantFlag==TRUE)
				pwvtDecode->iQuant = bs.getBits(NUMBITS_TOL_ACQUANT);
#if(DEBUG_BITS)
			printf("   AC quant\t\t= %d\n",pwvtDecode->iQuant);
#endif

			assert(pwvtQuantTree!=NULL);
			if(i==pwvtDecode->iReqSpatial)
			{
				piiSave = new CIntImage(CRct(0,0,pwvtQuantTree->getWidth()<<1,
					pwvtQuantTree->getHeight()<<1));
				pwvtQuantTree->decodeSingleQ(bs,iLength,pwvtDecode->iQuant,piiSave);
			}
			else
				//skip this bitstream
				skipBits(bs,iLength);

			pwvtQuantTree = pwvtQuantTree->getChild();  // go down a spatial level
		}
		break;

	case QT_MULTIPLE:
#if(DEBUG_BITS)
		printf("Multiple quantiser\n");
#endif
		if(pwvtDecode->iReqSpatial>=pwvtDecode->iSpatialLevels)
			pwvtDecode->iReqSpatial = pwvtDecode->iSpatialLevels-1;

		for(i=0;i<pwvtDecode->iSpatialLevels;i++)
		{
#if(DEBUG_BITS)
			printf("   Spatial layer: %d\n",i);
#endif

			Int iLength = bs.getBits(NUMBITS_TOL_ACLENGTH);
#if(DEBUG_BITS)
			printf("   Bitstream length\t= %d\n",iLength);
#endif

			if(bReadSNRFlag==TRUE)
			{
				pwvtDecode->iSNRLevels = bs.getBits(NUMBITS_TOL_SNR_SCALE);
#if(DEBUG_BITS)
				printf("   SNR levels\t\t= %d\n",pwvtDecode->iSNRLevels);
#endif
			}

			if(bReadQuantFlag==TRUE)
			{
				if(pwvtDecode->piMultiQuant!=NULL)
					delete pwvtDecode->piMultiQuant;
				pwvtDecode->piMultiQuant = new Int [pwvtDecode->iSNRLevels];
			}
			
			assert(pwvtQuantTree!=NULL);

			Int iReqSNRCount = pwvtDecode->iSNRLevels - pwvtDecode->iReqSNR;
			if(iReqSNRCount>pwvtDecode->iSNRLevels)
				iReqSNRCount = pwvtDecode->iSNRLevels;
			if(iReqSNRCount<1)
				iReqSNRCount = 1;
			
			Int j;
			Bool bInitFlag = TRUE;
#if(DEBUG_DECODE && !DEBUG_BITS)
			if(i==pwvtDecode->iReqSpatial)
				printf("Decoding: ");
#endif
			for(j=0;j<pwvtDecode->iSNRLevels;j++)
			{
#if(DEBUG_BITS)
				printf("   SNR layer: %d\n",j);
#endif
				if(bReadQuantFlag==TRUE)
					pwvtDecode->iQuant = pwvtDecode->piMultiQuant[j] =
						bs.getBits(NUMBITS_TOL_ACQUANT);
				else
					pwvtDecode->iQuant = pwvtDecode->piMultiQuant[j];
#if(DEBUG_BITS)
				printf("      AC Quant\t\t= %d\n",pwvtDecode->iQuant);
#endif
				
				Int iSNRLength = 0;
				Int iSNRAllZero = bs.getBits(1);
#if(DEBUG_BITS)
				printf("      SNR all zero flag\t= %d\n",iSNRAllZero);
#endif

				iReqSNRCount--;
				if(iSNRAllZero==0)
				{
					if(pwvtDecode->bSNRLengthMarker==TRUE)
						iSNRLength = readVLCValue(bs,NUMBITS_AC_SNRLENGTH);
					else
					{
						// This decoder does not support disabling of SNR bitstream
						// length information for multiple quantiser mode.
						// This is because the bitstream length info is required
						// to control the termination of the arithmetic decoder.
						// Please enable SNR length markers at the encoder.
						assert(FALSE);
					}
				}
				
				if(iReqSNRCount>=0 && i==pwvtDecode->iReqSpatial)
				{
#if(DEBUG_DECODE && !DEBUG_BITS)
					printf("[%d]",j);
#endif
					// decode this layer
					if(iReqSNRCount==0)
						// last time, so save image
						piiSave = new CIntImage(CRct(0,0,
								pwvtQuantTree->getWidth()<<1,
								pwvtQuantTree->getHeight()<<1));
					pwvtQuantTree->decodeMultipleQ(bs,iSNRLength,pwvtDecode->iQuant,
												   piiSave,bInitFlag);
					bInitFlag=FALSE;
				}
				else
				{
					// skip this layer
#if(DEBUG_BITS)
					printf("   Skipped!\n");
#endif
					skipBits(bs,iSNRLength);
				}
			}
			pwvtQuantTree = pwvtQuantTree->getChild();  // go down a spatial level
		}
#if(DEBUG_DECODE && !DEBUG_BITS)
		printf("\n");
#endif
		break;

	case QT_BILEVEL:
		Int iLevels = pwvtDecode->iSpatialLevels;
		Int iLevelsInternal = m_iDecompLevels;
		if(iLevels>1)
		{
			iLevels = m_iDecompLevels;
			iLevelsInternal = 1;
		}

		for(i=0;i<pwvtDecode->iReqSpatial;i++)
			pwvtQuantTree = pwvtQuantTree->getChild(); // move towards root

		CWvtCodedStream csStream;
		csStream.setSpatialLevels(iLevels);
		Int iSNRCount;
#if(DEBUG_BITS)
		printf("Bi-level quantiser\n");
#endif
		for(i=0;i<iLevels;i++)
		{
#if(DEBUG_BITS)
			printf("   Spatial layer: %d\n",i);
#endif
	
			// length of spatial steam
			Int iLength = bs.getBits(NUMBITS_TOL_ACLENGTH);
#if(DEBUG_BITS)
			printf("   Bitstream length\t= %d\n",iLength);
#endif
			
			// quant is local for bi-level coding
			Int iQuant = bs.getBits(NUMBITS_TOL_ACBILEVELQ);
#if(DEBUG_BITS)
			printf("   AC quant\t\t= %d\n",iQuant);
#endif
			iSNRCount = 0;

			// read snr streams
			while(iLength>0)
			{
				CWvtArithCoder *pacCoder = new CWvtArithCoder();
				if(pwvtDecode->bSNRLengthMarker==FALSE)
				{
					pacCoder->decodeGrabStream(bs,iLength);
					iLength=0;
					iSNRCount++;
				}
				else
				{
					// read snr length
					Int iSNRLength = readVLCValue(bs,NUMBITS_AC_SNRLENGTH);
#if(DEBUG_BITS)
					printf("   SNR layer %d length\t= %d\n",iSNRCount,iSNRLength);
#endif
					iLength -= countVLCBits(iSNRLength,NUMBITS_AC_SNRLENGTH);

					// get bitstream
					pacCoder->decodeGrabStream(bs,iSNRLength);
					iLength -= iSNRLength;
					iSNRCount++;
				}
				csStream.attachACStream(i,pacCoder,iQuant);
			}
		}

		piiSave = new CIntImage(CRct(0,0,pwvtQuantTree->getWidth()<<1,
							      pwvtQuantTree->getHeight()<<1));

		Int iSNRSelect = iSNRCount - pwvtDecode->iReqSNR - 1;
		if(iSNRSelect<0)
			iSNRSelect=0;
		if(iSNRSelect>=iSNRCount)
			iSNRSelect=iSNRCount-1;

		pwvtQuantTree->decodeBiLevelQ(&csStream,piiSave,pwvtDecode->bSNRLengthMarker,
			iSNRSelect,iLevelsInternal);
		break;
	}
	
	assert(piiSave!=NULL);

	return piiSave;
}

Int CWvtTransformObject::readVLCValue(CInBitStream &bs,Int iLength)
{
	Int iExtend,iShift = 0;
	Int iVal = 0;

	do {
		iExtend = bs.getBits(1);
		iVal |= (bs.getBits(iLength)<<iShift);
		iShift+=iLength;
	} while(iExtend==1);

	return iVal;
}

Int CWvtTransformObject::countVLCBits(Int iValue,Int iLength)
{
	Int iBits = iLength+1;

	while(iValue>=(1<<iLength))
	{
		iBits += iLength+1;
		iValue>>=iLength;
	}

	return iBits;
}


void CWvtTransformObject::skipBits(CInBitStream &bs,Int iLength)
{
	while(iLength>=24)
	{
		bs.getBits(24);
		iLength-=24;
	}
	if(iLength>0)
		bs.getBits(iLength);
}


