/*************************************************************************

This software module was originally developed by 

	Wei-ge Chen (wchen@microsoft.com), Microsoft Corporation
	Ming-Chieh Lee (mingcl@microsoft.com), Microsoft Corporation
	(date: July, 1997)

in the course of development of the MPEG-4 Video (ISO/IEC 14496-2). 
This software module is an implementation of a part of one or more MPEG-4 Video tools 
as specified by the MPEG-4 Video. 
ISO/IEC gives users of the MPEG-4 Video free license to this software module or modifications 
thereof for use in hardware or software products claiming conformance to the MPEG-4 Video. 
Those intending to use this software module in hardware or software products are advised that its use may infringe existing patents. 
The original developer of this software module and his/her company, 
the subsequent editors and their companies, 
and ISO/IEC have no liability for use of this software module or modifications thereof in an implementation. 
Copyright is not released for non MPEG-4 Video conforming products. 
Microsoft retains full right to use the code for his/her own purpose, 
assign or donate the code to a third party and to inhibit third parties from using the code for non <MPEG standard> conforming products. 
This copyright notice must be included in all copies or derivative works. 

Copyright (c) 1996, 1997.

Module Name:

	mvDec.cpp

Abstract:

	motion vector decoder

Revision History:

*************************************************************************/

#include "typeapi.h"
#include "codehead.h"
#include "mode.h"
#include "global.h"
#include "entropy/bitstrm.h"
#include "entropy/entropy.h"
#include "entropy/huffman.h"
#include "vopses.h"
#include "vopsedec.h"

#ifdef __MFC_
#ifdef _DEBUG
#undef THIS_FILE
static char BASED_CODE THIS_FILE[] = __FILE__;
#endif

#define new DEBUG_NEW				   
#endif // __MFC_


Void CVideoObjectDecoder::decodeMV (const CMBMode* pmbmd, CMotionVector* pmv, Bool bLeftBndry, Bool bRightBndry, Bool bTopBndry)
{
	if (pmbmd->m_bSkip || pmbmd->m_dctMd == INTRA || pmbmd->m_dctMd == INTRAQ)	{
		memset (pmv, 0, 5 * sizeof (CMotionVector));
		return;
	}
	CVector vctDiff, vctDecode, vctPred;
//	if (m_volmd.volType == ENHN_LAYER && m_vopmd.vopPredType == PVOP)	{
//		*pmv = *(pmv + 1) = *(pmv + 2) = *(pmv + 3) = *(pmv + 4) = mvZero;
//		return;
//	}
	if (pmbmd->m_bhas4MVForward)	{
		for (Int iBlk = Y_BLOCK1; iBlk <= Y_BLOCK4; iBlk++)	{
			if (bLeftBndry || bRightBndry || bTopBndry)
				find8x8MVpredAtBoundary (vctPred, pmv, bLeftBndry, bRightBndry, bTopBndry, (BlockNum) iBlk);
			else 
				find8x8MVpredInterior (vctPred, pmv, (BlockNum) iBlk);
			getDiffMV (vctDiff, m_vopmd.mvInfoForward);
			vctDecode = vctDiff + vctPred;																//fit in range
			fitMvInRange (vctDecode, m_vopmd.mvInfoForward);
			pmv [iBlk] = CMotionVector (vctDecode);		//convert to full pel now
		}
	}
	else {			//1 mv
		find16x16MVpred (vctPred, pmv, bLeftBndry, bRightBndry, bTopBndry);
		getDiffMV (vctDiff, m_vopmd.mvInfoForward);
		vctDecode = vctDiff + vctPred;																//fit in range
		fitMvInRange (vctDecode, m_vopmd.mvInfoForward);
		*pmv++ = CMotionVector (vctDecode);			//convert to full pel now
		for (Int i = 0; i < 4; i++) {
			*pmv = *(pmv - 1);
			pmv++;
		}
	}
}

Void CVideoObjectDecoder::decodeMVofBVOP (CMotionVector* pmvForward, CMotionVector* pmvBackward, CMBMode* pmbmd, 
										  Int iMBX, Int iMBY, const CMotionVector* pmvRef, const CMBMode* pmbmdRef)
{
	CVector vctDiff;
	CVector vctPred;
	CVector vctDecode;
	UInt nBits = 0;
	if (pmbmd->m_mbType == FORWARD || pmbmd->m_mbType == INTERPOLATE)	{
		assert (pmbmd->m_bSkip != TRUE);
		assert (pmbmd->m_bhas4MVForward != TRUE);
//		findMVpredictorOfBVOP (vctPred, pmvForward, pmbmd, iMBX);
		vctPred = m_vctForwardMvPredBVOP;
		getDiffMV (vctDiff, m_vopmd.mvInfoForward);
		vctDecode = vctDiff + vctPred;											//fit in range
		fitMvInRange (vctDecode, m_vopmd.mvInfoForward);
		m_vctForwardMvPredBVOP = vctDecode;
		*pmvForward++ = CMotionVector (vctDecode);										//convert to full pel now
		for (Int i = 0; i < 4; i++) {
			*pmvForward = *(pmvForward - 1);
			pmvForward++;
		}
	}
	if ((pmbmd->m_mbType == BACKWARD || pmbmd->m_mbType == INTERPOLATE) && m_volmd.volType != ENHN_LAYER)	{		
		assert (pmbmd->m_bSkip != TRUE);
		assert (pmbmd->m_bhas4MVBackward != TRUE);
//		findMVpredictorOfBVOP (vctPred, pmvBackward, pmbmd, iMBX);
		vctPred = m_vctBackwardMvPredBVOP;
		getDiffMV (vctDiff, m_vopmd.mvInfoBackward);
		vctDecode = vctDiff + vctPred;							//fit in range
		fitMvInRange (vctDecode, m_vopmd.mvInfoBackward);
		m_vctBackwardMvPredBVOP = vctDecode;
		*pmvBackward++ = CMotionVector (vctDecode);						//convert to full pel now
		for (Int i = 0; i < 4; i++) {
			*pmvBackward = *(pmvBackward - 1);
			pmvBackward++;
		}
	}		
	if (pmbmd->m_mbType == DIRECT)	{	
		assert(pmvRef!=NULL);
		assert (pmbmd->m_bhas4MVForward != TRUE);
		if (pmbmd->m_bSkip)
		//	memset (pmvForward, 0, 5 * sizeof (CMotionVector));	//set forward mv to zero
			vctDiff.x = vctDiff.y = 0;
		else  {
		//	getDiffMV (vctDiff, m_vopmd.mvInfoForward);
			Long lSymbol = m_pentrdecSet->m_pentrdecMV->decodeSymbol ();
			vctDiff.x  = deScaleMV (lSymbol - 32, 0, 1);
			lSymbol = m_pentrdecSet->m_pentrdecMV->decodeSymbol ();
			vctDiff.y  = deScaleMV (lSymbol - 32, 0, 1);
		}
		computeDirectForwardMV (vctDiff, pmvForward, pmvRef, pmbmdRef);	//compute forward mv from diff
		pmbmd->m_bhas4MVBackward = pmbmd->m_bhas4MVForward = pmbmdRef->m_bhas4MVForward;			//reset 4mv mode
		if (pmbmd->m_bhas4MVBackward)	{
			for (Int i = 0; i < 4; i++) {
				pmvForward++;
				pmvBackward++;
				pmvRef++;
				backwardMVFromForwardMV (*pmvBackward, *pmvForward, *pmvRef,
										 vctDiff); //compute back mv from forward mv and ref mv for direct mode
			}
		}
		else
			backwardMVFromForwardMV (*pmvBackward, *pmvForward, *pmvRef,
									 vctDiff); //compute back mv from forward mv and ref mv for direct mode
	}
}

Void CVideoObjectDecoder::computeDirectForwardMV (CVector vctDiff, CMotionVector* pmv, const CMotionVector* pmvRef, const CMBMode* pmbmdRef)
{
	//assuming colocated mb exists
	Int iPartInterval = m_t - m_tPastRef;
	Int iFullInterval = m_tFutureRef - m_tPastRef;
	if (pmbmdRef->m_bhas4MVForward == FALSE)	{
		CVector vctRefScaled = pmvRef->trueMVHalfPel () * iPartInterval;
		vctRefScaled.x /= iFullInterval;						//truncation as per vm
		vctRefScaled.y /= iFullInterval;						//truncation as per vm
		CVector vctDecode = vctDiff + vctRefScaled;
//		fitMvInRange (vctDecode, m_vopmd.mvInfoForward);
		*pmv++ = CMotionVector (vctDecode);							//convert to full pel now
		for (Int i = 0; i < 4; i++) {
			*pmv = *(pmv - 1);
			pmv++;
		}
	}
	else {
		for (Int i = 0; i < 4; i++) {
			pmv++;
			pmvRef++;
			CVector vctRefScaled = pmvRef->trueMVHalfPel () * iPartInterval;
			vctRefScaled.x /= iFullInterval;						//truncation as per vm
			vctRefScaled.y /= iFullInterval;						//truncation as per vm
			CVector vctDecode = vctDiff + vctRefScaled;
//			fitMvInRange (vctDecode, m_vopmd.mvInfoForward);
			*pmv = CMotionVector (vctDecode);							//convert to full pel now
		}
	}
}

Void CVideoObjectDecoder::decodeMVWithShape (const CMBMode* pmbmd, CoordI iMBX, CoordI iMBY, CMotionVector* pmv)
{
	if (pmbmd->m_bSkip || pmbmd->m_dctMd == INTRA || pmbmd->m_dctMd == INTRAQ) {
		memset (pmv, 0, 5 * sizeof (CMotionVector));
		return;
	}
	CVector vctDiff, vctDecode, vctPred;

//	if (m_volmd.volType == ENHN_LAYER && m_vopmd.vopPredType == PVOP)	{
//		*pmv == *(pmv + 1) = *(pmv + 2) = *(pmv + 3) = *(pmv + 4) = CMotionVector ();
//		return;
//	}
	if (pmbmd->m_bhas4MVForward)	{
		for (Int iBlk = Y_BLOCK1; iBlk <= Y_BLOCK4; iBlk++)	{
			if (pmbmd->m_rgTranspStatus [iBlk] == ALL)	
				pmv [iBlk] = CMotionVector (NOT_MV, NOT_MV);		
			else	{
				findMVpredGeneric (vctPred, pmv, pmbmd, iBlk, iMBX, iMBY);
				getDiffMV (vctDiff, m_vopmd.mvInfoForward);
				vctDecode = vctDiff + vctPred;																//fit in range
				fitMvInRange (vctDecode, m_vopmd.mvInfoForward);
				pmv [iBlk] = CMotionVector (vctDecode);
			}
		}
	}
	else {			//1 mv
		assert (pmbmd->m_rgTranspStatus [0] != ALL);
		findMVpredGeneric (vctPred, pmv, pmbmd, ALL_Y_BLOCKS, iMBX, iMBY);
		getDiffMV (vctDiff, m_vopmd.mvInfoForward);
		vctDecode = vctDiff + vctPred;																//fit in range
		fitMvInRange (vctDecode, m_vopmd.mvInfoForward);
		*pmv++ = CMotionVector (vctDecode);
		for (Int i = 0; i < 4; i++) {
			*pmv = *(pmv - 1);
			pmv++;
		}
	}
}

Void CVideoObjectDecoder::getDiffMV (CVector& vctDiffMV, MVInfo mvinfo)		//get half pel
{
	Int iResidual;
	Long lSymbol = m_pentrdecSet->m_pentrdecMV->decodeSymbol ();
	Int iVLC = lSymbol - 32;
	if (iVLC != 0)	
		iResidual = m_pbitstrmIn->getBits (mvinfo.uiFCode - 1);
	else
		iResidual = 0;
	vctDiffMV.x  = deScaleMV (iVLC, iResidual, mvinfo.uiScaleFactor);

	lSymbol = m_pentrdecSet->m_pentrdecMV->decodeSymbol ();
	iVLC = lSymbol - 32;
	if (iVLC != 0)
		iResidual = m_pbitstrmIn->getBits (mvinfo.uiFCode - 1);
	else
		iResidual = 0;
	vctDiffMV.y  = deScaleMV (iVLC, iResidual, mvinfo.uiScaleFactor);
}

Int CVideoObjectDecoder::deScaleMV (Int iVLC, Int iResidual, Int iScaleFactor)
{
	if (iVLC == 0 && iResidual == 0)
		return 0;
	else if (iScaleFactor == 1)
		return (iVLC);
	else {
		Int iAbsDiffMVcomponent = abs (iVLC) * iScaleFactor + iResidual - iScaleFactor + 1;	//changed a'c to enc
		return (sign (iVLC) * iAbsDiffMVcomponent);
	}
}

Void CVideoObjectDecoder::fitMvInRange (CVector& vctSrc, MVInfo mvinfo)
{
	Int iMvRange = mvinfo.uiRange;	//in half pel unit
	if (vctSrc.x < -1 * iMvRange)					//* 2 to get length of [-range, range]
		vctSrc.x += 2 * iMvRange;				
	else if (vctSrc.x >= iMvRange)
		vctSrc.x  -= 2 * iMvRange;

	if (vctSrc.y < -1 * iMvRange)
		vctSrc.y += 2 * iMvRange;
	else if (vctSrc.y >= iMvRange)
		vctSrc.y  -= 2 * iMvRange;
}

