/*************************************************************************

This software module was originally developed by 

	Chuang Gu (chuanggu@microsoft.com), Microsoft Corporation
	(date: Augest, 1997)

in the course of development of the MPEG-4 Video (ISO/IEC 14496-2). 
This software module is an implementation of a part of one or more MPEG-4 Video tools 
as specified by the MPEG-4 Video. 
ISO/IEC gives users of the MPEG-4 Video free license to this software module or modifications 
thereof for use in hardware or software products claiming conformance to the MPEG-4 Video. 
Those intending to use this software module in hardware or software products are advised that its use may infringe existing patents. 
The original developer of this software module and his/her company, 
the subsequent editors and their companies, 
and ISO/IEC have no liability for use of this software module or modifications thereof in an implementation. 
Copyright is not released for non MPEG-4 Video conforming products. 
Microsoft retains full right to use the code for his/her own purpose, 
assign or donate the code to a third party and to inhibit third parties from using the code for non <MPEG standard> conforming products. 
This copyright notice must be included in all copies or derivative works. 

Copyright (c) 1996, 1997.

Module Name:

	sptdec.cpp

Abstract:

	Sprite decoder.

Revision History:

*************************************************************************/
#include <stdio.h>
#include <fstream.h>
#include <math.h>

#include "typeapi.h"
#include "codehead.h"
#include "entropy/bitstrm.h"
#include "entropy/entropy.h"
#include "entropy/huffman.h"
#include "global.h"
#include "mode.h"
#include "vopses.h"
#include "vopsedec.h"

#ifdef __MFC_
#ifdef _DEBUG
#undef THIS_FILE
static char BASED_CODE THIS_FILE[] = __FILE__;
#endif

#define new DEBUG_NEW				   
#endif // __MFC_

#ifdef __PC_COMPILER_
#define IOSBINARY ios::binary
#else
#define IOSBINARY ios::bin
#endif // __PC_COMPILER_

Int CVideoObjectDecoder::decodeSpt ()
{
	if(findStartCode() == EOF)
		return EOF;

	decodeVOPHead ();
	cout << "\t" << "Time..." << m_t << "\n";
	cout.flush ();
	assert (m_vopmd.vopPredType == SPRITE);
	if (m_iNumOfPnts > 0) {
		decodeWarpPoints ();
		CRct rctWarp = (m_volmd.fAUsage != RECTANGLE)? m_rctCurrVOPY : CRct (0, 0, m_ivolWidth, m_ivolHeight);
		CPerspective2D perspYA (m_iNumOfPnts, m_rgstSrcQ, m_rgstDstQ, m_uiWarpingAccuracy);
		warpYA (perspYA, rctWarp, m_uiWarpingAccuracy);
		CSiteD rgstSrcQUV [4], rgstDstQUV [4];
		for (Int i = 0; i < m_iNumOfPnts; i++) {
			rgstSrcQUV [i] = (m_rgstSrcQ [i] - CSiteD (0.5, 0.5)) / 2;
			rgstDstQUV [i] = (m_rgstDstQ [i] - CSiteD (0.5, 0.5)) / 2;
		}
		CPerspective2D perspUV (m_iNumOfPnts, rgstSrcQUV, rgstDstQUV, m_uiWarpingAccuracy);
		warpUV (perspUV, rctWarp / 2, m_uiWarpingAccuracy);
	}
	return TRUE;
}

Void CVideoObjectDecoder::decodeWarpPoints ()
{
	assert (m_iNumOfPnts > 0);

	// Dest corner points MV decoding
	Int rgiU [4], rgiV [4], rgiDU [4], rgiDV [4];
	Int rgiWrpPnt0Del [2];
	CInBitStream* pibstrmWrpPt0 = m_pentrdecSet->m_pentrdecWrpPnt -> bitstream ();
	Int j;
	for (j = 0; j < m_iNumOfPnts; j++) { 
		for (UInt iXorY = 0; iXorY < 2; iXorY++)	{
			Long lSz = m_pentrdecSet->m_pentrdecWrpPnt->decodeSymbol();
			Int iSign = pibstrmWrpPt0->peekBits (1);	//get the sign
			if (iSign == 1)	
				rgiWrpPnt0Del[iXorY] = pibstrmWrpPt0->getBits (lSz);
			else	{
				Int iAbsWrpPnt0Del = ~(pibstrmWrpPt0->getBits (lSz));	//get the magnitude
				Int iMask = ~(0xFFFFFFFF << lSz);	//mask out the irrelavent bits
				rgiWrpPnt0Del[iXorY] = -1 * (iAbsWrpPnt0Del & iMask); //masking and signing
			}
			assert (rgiWrpPnt0Del[iXorY] >= -2047 && rgiWrpPnt0Del[iXorY] <= 2047);
		}
		rgiDU [j] = rgiWrpPnt0Del[0];
		rgiDV [j] = rgiWrpPnt0Del[1];
	}

	// create Src VOP bounding box
	switch (m_iNumOfPnts) {
	case 1: 
		m_rgstSrcQ [0] = CSiteD (m_rctCurrVOPY.left, m_rctCurrVOPY.top);
		break;
	case 2: 
		m_rgstSrcQ [0] = CSiteD (m_rctCurrVOPY.left, m_rctCurrVOPY.top);
		m_rgstSrcQ [1] = CSiteD (m_rctCurrVOPY.right, m_rctCurrVOPY.top);
		break;
	case 3:
		m_rgstSrcQ [0] = CSiteD (m_rctCurrVOPY.left, m_rctCurrVOPY.top);
		m_rgstSrcQ [1] = CSiteD (m_rctCurrVOPY.right, m_rctCurrVOPY.top);
		m_rgstSrcQ [2] = CSiteD (m_rctCurrVOPY.left, m_rctCurrVOPY.bottom);
		break;
	case 4:
		m_rgstSrcQ [0] = CSiteD (m_rctCurrVOPY.left, m_rctCurrVOPY.top);
		m_rgstSrcQ [1] = CSiteD (m_rctCurrVOPY.right, m_rctCurrVOPY.top);
		m_rgstSrcQ [2] = CSiteD (m_rctCurrVOPY.left, m_rctCurrVOPY.bottom);
		m_rgstSrcQ [3] = CSiteD (m_rctCurrVOPY.right, m_rctCurrVOPY.bottom);
		break;
	}

	// Dest corner points reconstruction 
	rgiU [0] = rgiDU [0];									rgiV [0] = rgiDV [0];
	rgiU [1] = rgiDU [1] + rgiU [0];						rgiV [1] = rgiDV [1] + rgiV [0];
	rgiU [2] = rgiDU [2] + rgiU [0];						rgiV [2] = rgiDV [2] + rgiV [0];
	rgiU [3] = rgiDU [3] + rgiU [2] + rgiU [1] - rgiU [0];	rgiV [3] = rgiDV [3] + rgiV [2] + rgiV [1] - rgiV [0];
	for (j = 0; j < m_iNumOfPnts; j++) {
		m_rgstDstQ [j].x = (rgiU [j] + 2 * m_rgstSrcQ [j].x) / 2.0;
		m_rgstDstQ [j].y = (rgiV [j] + 2 * m_rgstSrcQ [j].y) / 2.0;
	}
}

/*
Void CVideoObjectDecoder::computeMVInfo ()
{
	m_volmd.mvInfo.uiScaleFactor = 1 << (m_volmd.mvInfo.uiFCode - 1);
	m_volmd.mvInfo.uiRange = 
		(m_volmd.mvInfo.uiFCode == 1) ? 32 :
		(m_volmd.mvInfo.uiFCode == 2) ? 64 :
		128;
}

CVector2D CVideoObjectDecoder::getDiffMV ()
{
	CVector2D vctDiffMV;
	UInt uiResidual;
	Long lSymbol;
	Int iVLC, iSign;

	lSymbol = m_pbitstrmIn -> peekBits (NUMBITS_SPRITE_MV_ESC);
	if (lSymbol == SPRITE_MV_ESC)	{
		lSymbol = m_pbitstrmIn -> getBits (NUMBITS_SPRITE_MV_ESC);
		iSign = m_pbitstrmIn -> getBits (1);
		vctDiffMV.x = m_pbitstrmIn -> getBits (NUMBITS_SPRITE_MV_ESC - 1);
		if (iSign == 0)
			vctDiffMV.x *= -1;
	}
	else {
		lSymbol = m_pentrdecSet->m_pentrdecMV->decodeSymbol ();
		Int iVLC = lSymbol - 32;
		if (iVLC != 0)	
			uiResidual = m_pbitstrmIn->getBits (m_volmd.mvInfo.uiFCode - 1);
		else
			uiResidual = 0;
		vctDiffMV.x  = (Double) deScaleMV (iVLC, uiResidual, m_volmd.mvInfo.uiScaleFactor);
	}

	lSymbol = m_pbitstrmIn -> peekBits (NUMBITS_SPRITE_MV_ESC);
	if (lSymbol == SPRITE_MV_ESC)	{
		lSymbol = m_pbitstrmIn -> getBits (NUMBITS_SPRITE_MV_ESC);
		iSign = m_pbitstrmIn -> getBits (1);
		vctDiffMV.y = m_pbitstrmIn -> getBits (NUMBITS_SPRITE_MV_ESC - 1);
		if (iSign == 0)
			vctDiffMV.y *= -1;
	}
	else {
		lSymbol = m_pentrdecSet->m_pentrdecMV->decodeSymbol ();
		iVLC = lSymbol - 32;
		if (iVLC != 0)
			uiResidual = m_pbitstrmIn->getBits (m_volmd.mvInfo.uiFCode - 1);
		else
			uiResidual = 0;
		vctDiffMV.y  = (Double) deScaleMV (iVLC, uiResidual, m_volmd.mvInfo.uiScaleFactor);
	}

	vctDiffMV = vctDiffMV / 2.0;
	return vctDiffMV;
}

Int CVideoObjectDecoder::deScaleMV (  const Int iVLC, 
								   const UInt uiResidual, 
								   const UInt uiScaleFactor)
{
	if (iVLC == 0 && uiResidual == 0)
		return 0;
	else if (uiScaleFactor == 1)
		return (iVLC);
	else {
		UInt uiAbsDiffMVcomponent = abs (iVLC) * uiScaleFactor + uiResidual - uiScaleFactor + 1;	//changed a'c to enc
		return (sign (iVLC) * uiAbsDiffMVcomponent);
	}
}
*/
