/*************************************************************************

This software module was originally developed by 

	Ming-Chieh Lee (mingcl@microsoft.com), Microsoft Corporation
	Wei-ge Chen (wchen@microsoft.com), Microsoft Corporation
	Bruce Lin (blin@microsoft.com), Microsoft Corporation
	Chuang Gu (chuanggu@microsoft.com), Microsoft Corporation
	Simon Winder (swinder@microsoft.com), Microsoft Corporation
	(date: March, 1996)

and edited by  
 
        Wei Wu (weiwu@stallion.risc.rockwell.com), Rockwell Science Center 
        Homer Chen (homer@jupiter.risc.rockwell.com), Rockwell Science Center 

in the course of development of the MPEG-4 Video (ISO/IEC 14496-2). 
This software module is an implementation of a part of one or more MPEG-4 Video tools 
as specified by the MPEG-4 Video. 
ISO/IEC gives users of the MPEG-4 Video free license to this software module or modifications 
thereof for use in hardware or software products claiming conformance to the MPEG-4 Video. 
Those intending to use this software module in hardware or software products are advised that its use may infringe existing patents. 
The original developer of this software module and his/her company, 
the subsequent editors and their companies, 
and ISO/IEC have no liability for use of this software module or modifications thereof in an implementation. 
Copyright is not released for non MPEG-4 Video conforming products. 
Microsoft retains full right to use the code for his/her own purpose, 
assign or donate the code to a third party and to inhibit third parties from using the code for non <MPEG standard> conforming products. 
This copyright notice must be included in all copies or derivative works. 

Copyright (c) 1996, 1997.

Module Name:

	vopmbDec.cpp

Abstract:

	Decoder for VOP composed of MB's

Revision History:

*************************************************************************/
#include <stdio.h>
#include <math.h>
#include <stdlib.h>

#include "typeapi.h"
#include "mode.h"
#include "codehead.h"
#include "entropy/bitstrm.h"
#include "entropy/entropy.h"
#include "entropy/huffman.h"
#include "global.h"
#include "vopses.h"
#include "vopsedec.h"
#include "vd_grid.h" // VDS
#include "vd_vector.h" // VDS
#include "vd_projector.h" // VDS

#ifdef __MFC_
#ifdef _DEBUG
#undef THIS_FILE
static char BASED_CODE THIS_FILE[] = __FILE__;
#endif
#define new DEBUG_NEW				   
#endif // __MFC_

// VDS
#define FOV             30
#define RWIN_WIDTH      512
#define RWIN_HEIGHT     512

Void CVideoObjectDecoder::decodeVOP ()	
{
	if (m_volmd.fAUsage != RECTANGLE) {
		if (m_vopmd.vopPredType == PVOP)
			decodePVOP_WithShape ();
		else if (m_vopmd.vopPredType == IVOP)
			decodeIVOP_WithShape ();
		else
			decodeBVOP_WithShape ();
	}
	else {
		if (m_vopmd.vopPredType == PVOP)
			decodePVOP ();
		else if (m_vopmd.vopPredType == IVOP)
			decodeIVOP ();
		else
			decodeBVOP ();
	}
}

Void CVideoObjectDecoder::decodeIVOP_WithShape ()	
{
	//in case the IVOP is used as an ref for direct mode
	memset (m_rgmv, 0, m_iNumMB * 5 * sizeof (CMotionVector));

	Int iMBX, iMBY;
	CMBMode* pmbmd = m_rgmbmd;

	PixelC* ppxlcRefY  = (PixelC*) m_pvopcRefQ1->pixelsY () + m_iStartInRefToCurrRctY;
	PixelC* ppxlcRefU  = (PixelC*) m_pvopcRefQ1->pixelsU () + m_iStartInRefToCurrRctUV;
	PixelC* ppxlcRefV  = (PixelC*) m_pvopcRefQ1->pixelsV () + m_iStartInRefToCurrRctUV;
	PixelC* ppxlcRefBY = (PixelC*) m_pvopcRefQ1->pixelsBY () + m_iStartInRefToCurrRctY;
	PixelC* ppxlcRefA  = (PixelC*) m_pvopcRefQ1->pixelsA () + m_iStartInRefToCurrRctY;
	PixelC* ppxlcRefBUV = (PixelC*) m_pvopcRefQ1->pixelsBUV () + m_iStartInRefToCurrRctUV;

	Int iCurrentQP  = m_vopmd.intStepI;	
	Int iCurrentQPA = m_vopmd.intStepIAlpha;	
	for (iMBY = 0; iMBY < m_iNumMBY; iMBY++) {
		PixelC* ppxlcRefMBY  = ppxlcRefY;
		PixelC* ppxlcRefMBU  = ppxlcRefU;
		PixelC* ppxlcRefMBV  = ppxlcRefV;
		PixelC* ppxlcRefMBBY = ppxlcRefBY;
		PixelC* ppxlcRefMBBUV = ppxlcRefBUV;
		PixelC* ppxlcRefMBA  = ppxlcRefA;
		for (iMBX = 0; iMBX < m_iNumMBX; iMBX++) {
			decodeIntraShape (pmbmd, iMBX, iMBY, m_ppxlcCurrMBBY, ppxlcRefMBBY);
			downSampleBY (m_ppxlcCurrMBBY, m_ppxlcCurrMBBUV); // downsample original BY now for LPE padding (using original shape)
			if(m_volmd.bShapeOnly==FALSE)
			{
				pmbmd->m_bPadded=FALSE;
				if (pmbmd->m_rgTranspStatus [0] != ALL) {
					decodeMBTextureHeadOfIVOP (pmbmd, iCurrentQP);
					decodeTextureIntraMB (pmbmd, iMBX, iMBY, ppxlcRefMBY, ppxlcRefMBU, ppxlcRefMBV);
					if (m_volmd.fAUsage == EIGHT_BIT) {
						decodeMBAlphaHeadOfIVOP (pmbmd, iCurrentQPA);
						decodeAlphaIntraMB (pmbmd, ppxlcRefMBA);
					}

					// MC padding
					if (pmbmd -> m_rgTranspStatus [0] == PARTIAL)
						mcPadCurrMB (ppxlcRefMBY, ppxlcRefMBU, ppxlcRefMBV, ppxlcRefMBA);
					padNeighborTranspMBs (
						iMBX, iMBY,
						pmbmd,
						ppxlcRefMBY, ppxlcRefMBU, ppxlcRefMBV
					);
				}
				else {
					padCurrAndTopTranspMBFromNeighbor (
						iMBX, iMBY,
						pmbmd,
						ppxlcRefMBY, ppxlcRefMBU, ppxlcRefMBV
					);
				}
			}

			ppxlcRefMBA += MB_SIZE;
			ppxlcRefMBBY += MB_SIZE;
			ppxlcRefMBBUV += BLOCK_SIZE;
			pmbmd++;
			ppxlcRefMBY += MB_SIZE;
			ppxlcRefMBU += BLOCK_SIZE;
			ppxlcRefMBV += BLOCK_SIZE;
		}
		MacroBlockMemory** ppmbmTemp = m_rgpmbmAbove;
		m_rgpmbmAbove = m_rgpmbmCurr;
		m_rgpmbmCurr  = ppmbmTemp;
		ppxlcRefY += m_iFrameWidthYxMBSize;
		ppxlcRefU += m_iFrameWidthUVxBlkSize;
		ppxlcRefV += m_iFrameWidthUVxBlkSize;
		ppxlcRefBY += m_iFrameWidthYxMBSize;
		ppxlcRefA += m_iFrameWidthYxMBSize;

		ppxlcRefBUV += m_iFrameWidthUVxBlkSize;
	}
}

Void CVideoObjectDecoder::decodeIVOP ()	
{
	//in case the IVOP is used as an ref for direct mode
	memset (m_rgmv, 0, m_iNumMB * 5 * sizeof (CMotionVector));

	Int iMBX, iMBY;
	CMBMode* pmbmd = m_rgmbmd;
	pmbmd->m_stepSize = m_vopmd.intStepI;
	PixelC* ppxlcRefY = (PixelC*) m_pvopcRefQ1->pixelsY () + m_iStartInRefToCurrRctY;
	PixelC* ppxlcRefU = (PixelC*) m_pvopcRefQ1->pixelsU () + m_iStartInRefToCurrRctUV;
	PixelC* ppxlcRefV = (PixelC*) m_pvopcRefQ1->pixelsV () + m_iStartInRefToCurrRctUV;

	Int iCurrentQP  = m_vopmd.intStepI;		
	Int	iVideoPacketNumber = 0; 			//	added for error resilience mode by Toshiba
	for (iMBY = 0; iMBY < m_iNumMBY; iMBY++) {
		PixelC* ppxlcRefMBY = ppxlcRefY;
		PixelC* ppxlcRefMBU = ppxlcRefU;
		PixelC* ppxlcRefMBV = ppxlcRefV;
		for (iMBX = 0; iMBX < m_iNumMBX; iMBX++) {			
			if	(m_volmd.bErrorResilientDisable != TRUE)	{	//added for error resilience mode by Toshiba
				if	(checkResyncMarker())	{
					decodeVideoPacketHeader(iCurrentQP);
					iVideoPacketNumber++;
				}
			}
			pmbmd->m_iVideoPacketNumber = iVideoPacketNumber;	//mv out of if by wchen to set even when errR is off; always used in mbdec 
			decodeMBTextureHeadOfIVOP (pmbmd, iCurrentQP);
			decodeTextureIntraMB (pmbmd, iMBX, iMBY, ppxlcRefMBY, ppxlcRefMBU, ppxlcRefMBV);
			pmbmd++;
			ppxlcRefMBY += MB_SIZE;
			ppxlcRefMBU += BLOCK_SIZE;
			ppxlcRefMBV += BLOCK_SIZE;
		}
		MacroBlockMemory** ppmbmTemp = m_rgpmbmAbove;
		m_rgpmbmAbove = m_rgpmbmCurr;
		m_rgpmbmCurr  = ppmbmTemp;
		ppxlcRefY += m_iFrameWidthYxMBSize;
		ppxlcRefU += m_iFrameWidthUVxBlkSize;
		ppxlcRefV += m_iFrameWidthUVxBlkSize;
	}
}

// VDS: overload function
Void CVideoObjectDecoder::decodeIVOP (vector ViewPos, vector AimPos)	
{
	//in case the IVOP is used as an ref for direct mode
	memset (m_rgmv, 0, m_iNumMB * 5 * sizeof (CMotionVector));

	Int iMBX, iMBY;
	CMBMode* pmbmd = m_rgmbmd;
	pmbmd->m_stepSize = m_vopmd.intStepI;
	PixelC* ppxlcRefY = (PixelC*) m_pvopcRefQ1->pixelsY () + m_iStartInRefToCurrRctY;
	PixelC* ppxlcRefU = (PixelC*) m_pvopcRefQ1->pixelsU () + m_iStartInRefToCurrRctUV;
	PixelC* ppxlcRefV = (PixelC*) m_pvopcRefQ1->pixelsV () + m_iStartInRefToCurrRctUV;

	Int iCurrentQP  = m_vopmd.intStepI;		
	Int	iVideoPacketNumber = 0; 			//	added for error resilience mode by Toshiba
	// VDS
	Int iCellX, iCellY;
	double x_over_y = (double) RWIN_WIDTH / (double) RWIN_HEIGHT;
	vector vup = vector(0, 0, 1);
	vector v00, v01, v10, v11;
	Bool bIsVisible, bIsUVisible;
	double alpha, beta, dist, Rr;
/*
	double uv_alpha, uv_beta, uv_dist;
*/
	projector proj(ViewPos, AimPos);
	CRct mask_rct(0, 0, 8, 8);
	CU8Image *mask[6];
	CGrid *pgrid = m_pgrid;
	for (Int i=0; i<6; i++) {
		mask[i] = new CU8Image(mask_rct, 0);
	}
	Rr = (double) RWIN_WIDTH / (2*tan(FOV*3.141592653589/360.0)) / allmaskY->where().width;
	for (iMBY = 0, iCellY = 0; iMBY < m_iNumMBY; iMBY++, iCellY+=2) { // VDS
		PixelC* ppxlcRefMBY = ppxlcRefY;
		PixelC* ppxlcRefMBU = ppxlcRefU;
		PixelC* ppxlcRefMBV = ppxlcRefV;
		for (iMBX = 0, iCellX = 0; iMBX < m_iNumMBX; iMBX++, iCellX+=2) { // VDS
			if	(m_volmd.bErrorResilientDisable != TRUE)	{	//added for error resilience mode by Toshiba
				if	(checkResyncMarker())	{
					decodeVideoPacketHeader(iCurrentQP);
					iVideoPacketNumber++;
				}
			}

			bIsUVisible = FALSE;
			// Y_BLOCK1
			v00 = pgrid->getVertex(iCellY, iCellX);
			v01 = pgrid->getVertex(iCellY, iCellX+1);
			v10 = pgrid->getVertex(iCellY+1, iCellX);
			v11 = pgrid->getVertex(iCellY+1, iCellX+1);
			bIsVisible = proj.in_fov(v00, v01, v10, v11, FOV, x_over_y, vup);
			mask[0]->pixels(0);
			if(bIsVisible) {
				proj.quad(v00, v01, v10, v11, 1);
				alpha = proj.alpha_val();
				beta = proj.beta_val();
				dist = proj.cal_dist(v00, v01, v10, v11);
				shrink(alpha, beta, dist, Rr, iCellX, iCellY,
					allmaskY, mask[0]);
				bIsUVisible = TRUE;
/*
				uv_alpha = alpha;
				uv_beta = beta;
*/
			} 
			// Y_BLOCK2
			iCellX++;
			v00 = pgrid->getVertex(iCellY, iCellX);
			v01 = pgrid->getVertex(iCellY, iCellX+1);
			v10 = pgrid->getVertex(iCellY+1, iCellX);
			v11 = pgrid->getVertex(iCellY+1, iCellX+1);
			bIsVisible = proj.in_fov(v00, v01, v10, v11, FOV, x_over_y, vup);
			mask[1]->pixels(0);
			if(bIsVisible) {
				proj.quad(v00, v01, v10, v11, 1);
				alpha = proj.alpha_val();
				beta = proj.beta_val();
				dist = proj.cal_dist(v00, v01, v10, v11);
				shrink(alpha, beta, dist, Rr, iCellX, iCellY,
					allmaskY, mask[1]);
				bIsUVisible = TRUE;
/*
				uv_alpha = max(uv_alpha, alpha);
				uv_beta = max(uv_beta, beta);
*/
			} 
			iCellX--;
			// Y_BLOCK3
			iCellY++;
			v00 = pgrid->getVertex(iCellY, iCellX);
			v01 = pgrid->getVertex(iCellY, iCellX+1);
			v10 = pgrid->getVertex(iCellY+1, iCellX);
			v11 = pgrid->getVertex(iCellY+1, iCellX+1);
			bIsVisible = proj.in_fov(v00, v01, v10, v11, FOV, x_over_y, vup);
			mask[2]->pixels(0);
			if(bIsVisible) {
				proj.quad(v00, v01, v10, v11, 1);
				alpha = proj.alpha_val();
				beta = proj.beta_val();
				dist = proj.cal_dist(v00, v01, v10, v11);
				shrink(alpha, beta, dist, Rr, iCellX, iCellY,
					allmaskY, mask[2]);
				bIsUVisible = TRUE;
/*
				uv_alpha = max(uv_alpha, alpha);
				uv_beta = max(uv_beta, beta);
*/
			} 
			// Y_BLOCK4
			iCellX++;
			v00 = pgrid->getVertex(iCellY, iCellX);
			v01 = pgrid->getVertex(iCellY, iCellX+1);
			v10 = pgrid->getVertex(iCellY+1, iCellX);
			v11 = pgrid->getVertex(iCellY+1, iCellX+1);
			bIsVisible = proj.in_fov(v00, v01, v10, v11, FOV, x_over_y, vup);
			mask[3]->pixels(0);
			if(bIsVisible) {
				proj.quad(v00, v01, v10, v11, 1);
				alpha = proj.alpha_val();
				beta = proj.beta_val();
				dist = proj.cal_dist(v00, v01, v10, v11);
				shrink(alpha, beta, dist, Rr, iCellX, iCellY,
					allmaskY, mask[3]);
				bIsUVisible = TRUE;
/*
				uv_alpha = max(uv_alpha, alpha);
				uv_beta = max(uv_beta, beta);
*/
			} 
			iCellX--; 
			iCellY--;

			// UV_BLOCK
			mask[4]->pixels(0);
			mask[5]->pixels(0);
			if(bIsUVisible) {
/*
				v00 = pgrid->getVertex(iCellY, iCellX);
				v01 = pgrid->getVertex(iCellY, iCellX+2);
				v10 = pgrid->getVertex(iCellY+2, iCellX);
				v11 = pgrid->getVertex(iCellY+2, iCellX+2);
				uv_dist = proj.cal_dist(v00, v01, v10, v11);
				shrink(uv_alpha, uv_beta, uv_dist, iMBX, iMBY,
					allmaskUV, mask[4]);
*/
				shrink_UV(iMBX, iMBY, mask, allmaskUV);
			}

			pmbmd->m_iVideoPacketNumber = iVideoPacketNumber;	//mv out of if by wchen to set even when errR is off; always used in mbdec 
			decodeMBTextureHeadOfIVOP (pmbmd, iCurrentQP);
			decodeTextureIntraMB (pmbmd, iMBX, iMBY, ppxlcRefMBY, ppxlcRefMBU, ppxlcRefMBV, mask);
			pmbmd++;
			ppxlcRefMBY += MB_SIZE;
			ppxlcRefMBU += BLOCK_SIZE;
			ppxlcRefMBV += BLOCK_SIZE;
		}
		MacroBlockMemory** ppmbmTemp = m_rgpmbmAbove;
		m_rgpmbmAbove = m_rgpmbmCurr;
		m_rgpmbmCurr  = ppmbmTemp;
		ppxlcRefY += m_iFrameWidthYxMBSize;
		ppxlcRefU += m_iFrameWidthUVxBlkSize;
		ppxlcRefV += m_iFrameWidthUVxBlkSize;
	}
}

Void CVideoObjectDecoder::decodePVOP_WithShape ()
{
	Int iMBX, iMBY;
	CoordI y = m_rctCurrVOPY.top; 
	CMBMode* pmbmd = m_rgmbmd;
	CMotionVector* pmv = m_rgmv;
	CMotionVector* pmvBY = m_rgmvBY;
	PixelC* ppxlcCurrQY  = (PixelC*) m_pvopcRefQ1->pixelsY () + m_iStartInRefToCurrRctY;
	PixelC* ppxlcCurrQU  = (PixelC*) m_pvopcRefQ1->pixelsU () + m_iStartInRefToCurrRctUV;
	PixelC* ppxlcCurrQV  = (PixelC*) m_pvopcRefQ1->pixelsV () + m_iStartInRefToCurrRctUV;
	PixelC* ppxlcCurrQBY = (PixelC*) m_pvopcRefQ1->pixelsBY () + m_iStartInRefToCurrRctY;
	PixelC* ppxlcCurrQA  = (PixelC*) m_pvopcRefQ1->pixelsA () + m_iStartInRefToCurrRctY;
	
	Int iCurrentQP  = m_vopmd.intStep;	
	Int iCurrentQPA = m_vopmd.intStepAlpha;	
	for (iMBY = 0; iMBY < m_iNumMBY; iMBY++, y += MB_SIZE) {
		PixelC* ppxlcCurrQMBY  = ppxlcCurrQY;
		PixelC* ppxlcCurrQMBU  = ppxlcCurrQU;
		PixelC* ppxlcCurrQMBV  = ppxlcCurrQV;
		PixelC* ppxlcCurrQMBBY = ppxlcCurrQBY;
		PixelC* ppxlcCurrQMBA  = ppxlcCurrQA;
		ShapeMode shpmdColocatedMB = m_rgmbmdRef [min (iMBY, m_iNumMBYRef - 1) * m_iNumMBXRef].m_shpmd;
		decodeInterShape (m_pvopcRefQ0, pmbmd, 0, iMBY, m_rctCurrVOPY.left, y, pmv, pmvBY, m_ppxlcCurrMBBY, ppxlcCurrQMBBY, shpmdColocatedMB);
		downSampleBY (m_ppxlcCurrMBBY, m_ppxlcCurrMBBUV);
		if (pmbmd->m_rgTranspStatus [0] != ALL && m_volmd.bShapeOnly==FALSE) {
			decodeMBTextureHeadOfPVOP (pmbmd, iCurrentQP);
			decodeMVWithShape (pmbmd, 0, iMBY, pmv);
		}
		CoordI x = m_rctCurrVOPY.left;
		for (iMBX = 0; iMBX < m_iNumMBX; iMBX++, x += MB_SIZE) {
			pmbmd->m_bPadded = FALSE;
			if (pmbmd->m_rgTranspStatus [0] != ALL && m_volmd.bShapeOnly==FALSE) {
				if ((pmbmd->m_dctMd == INTRA || pmbmd->m_dctMd == INTRAQ) && !pmbmd->m_bSkip) {
					decodeTextureIntraMB (pmbmd, iMBX, iMBY, ppxlcCurrQMBY, ppxlcCurrQMBU, ppxlcCurrQMBV);
					if (m_volmd.fAUsage == EIGHT_BIT) {
						decodeMBAlphaHeadOfPVOP (pmbmd, iCurrentQPA);
						decodeAlphaIntraMB (pmbmd, ppxlcCurrQMBA);
					}
				}
				else {
					if (!pmbmd->m_bSkip)
						decodeTextureInterMB (pmbmd);
					if (m_volmd.fAUsage == EIGHT_BIT) {
						decodeMBAlphaHeadOfPVOP (pmbmd, iCurrentQPA);
						decodeAlphaInterMB (pmbmd);
					}
				}
			}
			// decode shape, overhead, and MV for the right MB
			if (iMBX != m_iNumMBX - 1) {
				CMBMode* pmbmdRight = pmbmd + 1;
				shpmdColocatedMB = m_rgmbmdRef [
					min (max (0, iMBX + 1), m_iNumMBXRef - 1) + 
 					min (max (0, iMBY), m_iNumMBYRef - 1) * m_iNumMBXRef
				].m_shpmd;
				decodeInterShape (
					m_pvopcRefQ0,
					pmbmdRight, 
					iMBX + 1, iMBY, 
					x + MB_SIZE, y, 
					pmv + 5, pmvBY + 1, 
					m_ppxlcRightMBBY, ppxlcCurrQMBBY + MB_SIZE, 
					shpmdColocatedMB
				);
				downSampleBY (m_ppxlcRightMBBY, m_ppxlcRightMBBUV);
				if (pmbmdRight->m_rgTranspStatus [0] != ALL && m_volmd.bShapeOnly==FALSE) {
					decodeMBTextureHeadOfPVOP (pmbmdRight, iCurrentQP);
					decodeMVWithShape (pmbmdRight, iMBX + 1, iMBY, pmv + 5);
				}
			}

			if(m_volmd.bShapeOnly==FALSE) {
				if (pmbmd->m_rgTranspStatus [0] != ALL) {
					if (pmbmd->m_dctMd == INTER || pmbmd->m_dctMd == INTERQ)	{
			 			motionCompMB (
							m_ppxlcPredMBY, m_pvopcRefQ0->pixelsY (),
							pmv, pmbmd, 
							iMBX, iMBY, 
							x, y,
							pmbmd->m_bSkip,
							&m_rctRefVOPY0
						);
						if (!pmbmd->m_bSkip) {
							CoordI iXRefUV, iYRefUV;
							mvLookupUVWithShape (pmbmd, pmv, iXRefUV, iYRefUV);
							motionCompUV (m_ppxlcPredMBU, m_ppxlcPredMBV, m_pvopcRefQ0,
								x, y, iXRefUV, iYRefUV, m_vopmd.iRoundingControl,&m_rctRefVOPY0);
							addErrorAndPredToCurrQ (ppxlcCurrQMBY, ppxlcCurrQMBU, ppxlcCurrQMBV);
						}
						else {
							if (m_volmd.bAdvPredDisable)
								copyFromRefToCurrQ (m_pvopcRefQ0, x, y, ppxlcCurrQMBY, ppxlcCurrQMBU, ppxlcCurrQMBV, &m_rctRefVOPY0);
							else
								copyFromPredForYAndRefForCToCurrQ (x, y, ppxlcCurrQMBY, ppxlcCurrQMBU, ppxlcCurrQMBV, &m_rctRefVOPY0);
						}
					}

					if (m_volmd.fAUsage == EIGHT_BIT) {
			 			motionCompMB (
							m_ppxlcPredMBA, m_pvopcRefQ0->pixelsA (),
							pmv, pmbmd, 
							iMBX, iMBY, 
							x, y,
							pmbmd->m_bSkipA, &m_rctRefVOPY0
						);
						if (!pmbmd->m_bSkipA)
							addAlphaErrorAndPredToCurrQ (ppxlcCurrQMBA);
						else {
							if (m_volmd.bAdvPredDisable)
								copyAlphaFromRefToCurrQ (x, y, ppxlcCurrQMBA);			// MC padding
							else
								copyAlphaFromPredToCurrQ (x, y, ppxlcCurrQMBA);			// MC padding
						}
					}

					if (pmbmd->m_rgTranspStatus [0] == PARTIAL)
						mcPadCurrMB (ppxlcCurrQMBY, ppxlcCurrQMBU, ppxlcCurrQMBV, ppxlcCurrQMBA);
					padNeighborTranspMBs (
						iMBX, iMBY,
						pmbmd,
						ppxlcCurrQMBY, ppxlcCurrQMBU, ppxlcCurrQMBV
					);
				}
				else {
					padCurrAndTopTranspMBFromNeighbor (
						iMBX, iMBY,
						pmbmd,
						ppxlcCurrQMBY, ppxlcCurrQMBU, ppxlcCurrQMBV
					);
				}
			}
			if (iMBX != m_iNumMBX - 1)
				swapCurrAndRightMBForShape ();

			if(pmbmd->m_bhas4MVForward)
				padMotionVectors(pmbmd,pmv);

			pmbmd++;
			pmv += 5;
			pmvBY++;
			ppxlcCurrQMBY += MB_SIZE;
			ppxlcCurrQMBU += BLOCK_SIZE;
			ppxlcCurrQMBV += BLOCK_SIZE;
			ppxlcCurrQMBBY += MB_SIZE;
			ppxlcCurrQMBA += MB_SIZE;
		}
		MacroBlockMemory** ppmbmTemp = m_rgpmbmAbove;
		m_rgpmbmAbove = m_rgpmbmCurr;
		m_rgpmbmCurr  = ppmbmTemp;

		ppxlcCurrQY  += m_iFrameWidthYxMBSize;
		ppxlcCurrQU  += m_iFrameWidthUVxBlkSize;
		ppxlcCurrQV  += m_iFrameWidthUVxBlkSize;
		ppxlcCurrQBY += m_iFrameWidthYxMBSize;
		ppxlcCurrQA  += m_iFrameWidthYxMBSize;
	}
}

Void CVideoObjectDecoder::decodePVOP ()
{
	Int iMBX, iMBY;
	CoordI y = 0; 
	CMBMode* pmbmd = m_rgmbmd;
	CMotionVector* pmv = m_rgmv;
	PixelC* ppxlcCurrQY = (PixelC*) m_pvopcRefQ1->pixelsY () + m_iStartInRefToCurrRctY;
	PixelC* ppxlcCurrQU = (PixelC*) m_pvopcRefQ1->pixelsU () + m_iStartInRefToCurrRctUV;
	PixelC* ppxlcCurrQV = (PixelC*) m_pvopcRefQ1->pixelsV () + m_iStartInRefToCurrRctUV;

	Int iCurrentQP  = m_vopmd.intStep;
	Int	iVideoPacketNumber = 0;
	Bool bLeftBndry;
	Bool bRightBndry;
	Bool bTopBndry;
	for (iMBY = 0; iMBY < m_iNumMBY; iMBY++, y += MB_SIZE) {
		PixelC* ppxlcCurrQMBY = ppxlcCurrQY;
		PixelC* ppxlcCurrQMBU = ppxlcCurrQU;
		PixelC* ppxlcCurrQMBV = ppxlcCurrQV;
		//	Added for error resilience mode By Toshiba
		if	((iMBY != 0) && (m_volmd.bErrorResilientDisable != TRUE))	{
			if	(checkResyncMarker()) {
				decodeVideoPacketHeader(iCurrentQP);
				iVideoPacketNumber++;
			}
		}
		pmbmd->m_iVideoPacketNumber = iVideoPacketNumber;
		if (m_volmd.bErrorResilientDisable != TRUE)	{
			if (iMBY==0)	{
				bRightBndry = TRUE;
				bTopBndry = TRUE;
			}
			else	{
				bRightBndry = !((pmbmd - m_iNumMBX + 1) -> m_iVideoPacketNumber == pmbmd->m_iVideoPacketNumber);
				bTopBndry = !((pmbmd - m_iNumMBX) -> m_iVideoPacketNumber == pmbmd->m_iVideoPacketNumber);
			}
		}
		else	{
			bRightBndry = FALSE;
			bTopBndry = (iMBY == 0);
		}
		decodeMBTextureHeadOfPVOP (pmbmd, iCurrentQP);
		//changed for error resilience mode By Toshiba
		//decodeMV (pmbmd, pmv, TRUE, FALSE, iMBY == 0);
		decodeMV (pmbmd, pmv, TRUE, bRightBndry, bTopBndry);
		CoordI x = 0;
		for (iMBX = 0; iMBX < m_iNumMBX; iMBX++, x += MB_SIZE) {
			if (pmbmd->m_dctMd == INTRA || pmbmd->m_dctMd == INTRAQ)
				decodeTextureIntraMB (pmbmd, iMBX, iMBY, ppxlcCurrQMBY, ppxlcCurrQMBU, ppxlcCurrQMBV);
			else {
				if (!pmbmd->m_bSkip)
					decodeTextureInterMB (pmbmd);
			}
			if (iMBX != m_iNumMBX - 1) {				
				CMBMode* pmbmdRight = pmbmd + 1;
				if(m_volmd.bErrorResilientDisable != TRUE){		//added for error resilience mode By Toshiba
					if (checkResyncMarker())	{
						decodeVideoPacketHeader(iCurrentQP);
						iVideoPacketNumber++;
					}
					if (iMBY == 0)	{
						bLeftBndry = !(pmbmd -> m_iVideoPacketNumber == pmbmdRight -> m_iVideoPacketNumber);
						bRightBndry = TRUE;
						bTopBndry = TRUE;
					}
					else	{
						bLeftBndry =  !(pmbmd -> m_iVideoPacketNumber == pmbmdRight -> m_iVideoPacketNumber);
						if ((iMBX + 1) == m_iNumMBX - 1)
							bRightBndry = TRUE;
						else
							bRightBndry = !((pmbmdRight - m_iNumMBX + 1) -> m_iVideoPacketNumber == pmbmdRight->m_iVideoPacketNumber);
						bTopBndry = !((pmbmdRight - m_iNumMBX) -> m_iVideoPacketNumber == pmbmdRight->m_iVideoPacketNumber);
					}
				}
				else	{
					bLeftBndry = FALSE;
					bRightBndry = ((iMBX + 1) == m_iNumMBX - 1);
					bTopBndry = (iMBY == 0);
				}
				pmbmdRight->m_iVideoPacketNumber = iVideoPacketNumber;
				decodeMBTextureHeadOfPVOP (pmbmdRight, iCurrentQP);
				//changed for error resilience
				//decodeMV (pmbmdRight, pmv + 5, FALSE, (iMBX + 1) == m_iNumMBX - 1, iMBY == 0);
				decodeMV (pmbmdRight, pmv + 5, bLeftBndry, bRightBndry, bTopBndry);
			}
			if (pmbmd->m_dctMd == INTER || pmbmd->m_dctMd == INTERQ) {
				motionCompMB (
					m_ppxlcPredMBY, m_pvopcRefQ0->pixelsY (),
					pmv, pmbmd, 
					iMBX, iMBY, 
					x, y,
					pmbmd->m_bSkip,
					&m_rctRefVOPY0
				);
				if (!pmbmd->m_bSkip) {
					CoordI iXRefUV, iYRefUV;
					mvLookupUV (pmbmd, pmv, iXRefUV, iYRefUV);
					motionCompUV (m_ppxlcPredMBU, m_ppxlcPredMBV, m_pvopcRefQ0,
						x, y, iXRefUV, iYRefUV, m_vopmd.iRoundingControl,&m_rctRefVOPY0);
					addErrorAndPredToCurrQ (ppxlcCurrQMBY, ppxlcCurrQMBU, ppxlcCurrQMBV);
				}
				else {
					if (m_volmd.bAdvPredDisable)
						copyFromRefToCurrQ (m_pvopcRefQ0, x, y, ppxlcCurrQMBY, ppxlcCurrQMBU, ppxlcCurrQMBV, NULL);
					else
						copyFromPredForYAndRefForCToCurrQ (x, y, ppxlcCurrQMBY, ppxlcCurrQMBU, ppxlcCurrQMBV, NULL);
				}
			}
			pmbmd++;
			pmv += 5;
			ppxlcCurrQMBY += MB_SIZE;
			ppxlcCurrQMBU += BLOCK_SIZE;
			ppxlcCurrQMBV += BLOCK_SIZE;
		}
		MacroBlockMemory** ppmbmTemp = m_rgpmbmAbove;
		m_rgpmbmAbove = m_rgpmbmCurr;
		m_rgpmbmCurr  = ppmbmTemp;
		ppxlcCurrQY += m_iFrameWidthYxMBSize;
		ppxlcCurrQU += m_iFrameWidthUVxBlkSize;
		ppxlcCurrQV += m_iFrameWidthUVxBlkSize;
	}
}

Void CVideoObjectDecoder::decodeBVOP ()
{
	Int iMBX, iMBY;
	CoordI y = 0; 
	CMBMode* pmbmd = m_rgmbmd;
	CMBMode* pmbmdRef = m_rgmbmdRef;
	CMotionVector* pmv = m_rgmv;
	CMotionVector* pmvRef = m_rgmvRef;
	CMotionVector* pmvBackward = m_rgmvBackward;
	PixelC* ppxlcCurrQY = (PixelC*) m_pvopcCurrQ->pixelsY () + m_iStartInRefToCurrRctY;
	PixelC* ppxlcCurrQU = (PixelC*) m_pvopcCurrQ->pixelsU () + m_iStartInRefToCurrRctUV;
	PixelC* ppxlcCurrQV = (PixelC*) m_pvopcCurrQ->pixelsV () + m_iStartInRefToCurrRctUV;

	Int iCurrentQP  = m_vopmd.intStepB;		
	for (iMBY = 0; iMBY < m_iNumMBY; iMBY++, y += MB_SIZE) {
		PixelC* ppxlcCurrQMBY = ppxlcCurrQY;
		PixelC* ppxlcCurrQMBU = ppxlcCurrQU;
		PixelC* ppxlcCurrQMBV = ppxlcCurrQV;
		CoordI x = 0; 
		m_vctForwardMvPredBVOP.x  = m_vctForwardMvPredBVOP.y  = 0;
		m_vctBackwardMvPredBVOP.x = m_vctBackwardMvPredBVOP.y = 0;
		for (iMBX = 0; iMBX < m_iNumMBX; iMBX++, x += MB_SIZE) {
			pmbmd->m_bColocatedMBSkip = pmbmdRef->m_bSkip;
			if (pmbmd->m_bColocatedMBSkip) { // don't need to send any bit for this mode
				copyFromRefToCurrQ (
					m_pvopcRefQ0,
					x, y, 
					ppxlcCurrQMBY, ppxlcCurrQMBU, ppxlcCurrQMBV, NULL
				);
				pmbmd->m_bSkip = TRUE;
				memset (pmv, 0, 5 * sizeof (CMotionVector));
				memset (pmvBackward, 0, 5 * sizeof (CMotionVector));
				pmbmd->m_mbType = DIRECT; // can be set to FORWARD mode since the result is the same
			}
			else {
				decodeMBTextureHeadOfBVOP (pmbmd, iCurrentQP);
				decodeMVofBVOP (pmv, pmvBackward, pmbmd, iMBX, iMBY, pmvRef, pmbmdRef);
				if (!pmbmd->m_bSkip) {
					decodeTextureInterMB (pmbmd);
					motionCompAndAddErrorMB_BVOP (
						pmv, pmvBackward,
						pmbmd, 
						iMBX, iMBY, 
						x, y,
						ppxlcCurrQMBY, ppxlcCurrQMBU, ppxlcCurrQMBV,
						&m_rctRefVOPY0,&m_rctRefVOPY1);
				}
				else 
					motionCompSkipMB_BVOP (pmbmd, pmv, pmvBackward,
						x, y, ppxlcCurrQMBY, ppxlcCurrQMBU, ppxlcCurrQMBV, &m_rctRefVOPY0, &m_rctRefVOPY1);
			}
			pmbmd++;
			pmbmdRef++;
			pmv += 5;
			pmvRef += 5;
			pmvBackward += 5;
			ppxlcCurrQMBY += MB_SIZE;
			ppxlcCurrQMBU += BLOCK_SIZE;
			ppxlcCurrQMBV += BLOCK_SIZE;
		}
		ppxlcCurrQY += m_iFrameWidthYxMBSize;
		ppxlcCurrQU += m_iFrameWidthUVxBlkSize;
		ppxlcCurrQV += m_iFrameWidthUVxBlkSize;
	}
}

Void CVideoObjectDecoder::decodeBVOP_WithShape ()
{
	Int iMBX, iMBY;
	CoordI y = m_rctCurrVOPY.top; 
	CMBMode* pmbmd = m_rgmbmd;
	CMotionVector* pmv = m_rgmv;
	CMotionVector* pmvBackward = m_rgmvBackward;
	CMotionVector* pmvBY = m_rgmvBY;
	const CMotionVector* pmvRef;
	const CMBMode* pmbmdRef;
	Bool bColocatedMBExist;
	PixelC* ppxlcCurrQY = (PixelC*) m_pvopcCurrQ->pixelsY () + m_iStartInRefToCurrRctY;
	PixelC* ppxlcCurrQU = (PixelC*) m_pvopcCurrQ->pixelsU () + m_iStartInRefToCurrRctUV;
	PixelC* ppxlcCurrQV = (PixelC*) m_pvopcCurrQ->pixelsV () + m_iStartInRefToCurrRctUV;
	PixelC* ppxlcCurrQBY = (PixelC*) m_pvopcCurrQ->pixelsBY () + m_iStartInRefToCurrRctY;
	PixelC* ppxlcCurrQA  = (PixelC*) m_pvopcCurrQ->pixelsA () + m_iStartInRefToCurrRctY;

	Int iCurrentQP  = m_vopmd.intStepB;		
	Int iCurrentQPA = m_vopmd.intStepBAlpha;	
	for (iMBY = 0; iMBY < m_iNumMBY; iMBY++, y += MB_SIZE) {
		PixelC* ppxlcCurrQMBY = ppxlcCurrQY;
		PixelC* ppxlcCurrQMBU = ppxlcCurrQU;
		PixelC* ppxlcCurrQMBV = ppxlcCurrQV;
		PixelC* ppxlcCurrQMBBY = ppxlcCurrQBY;
		PixelC* ppxlcCurrQMBA  = ppxlcCurrQA;
		CoordI x = m_rctCurrVOPY.left; 
		m_vctForwardMvPredBVOP.x  = m_vctForwardMvPredBVOP.y  = 0;
		m_vctBackwardMvPredBVOP.x = m_vctBackwardMvPredBVOP.y = 0;
		for (iMBX = 0; iMBX < m_iNumMBX; iMBX++, x += MB_SIZE) {
			findColocatedMB (iMBX, iMBY, bColocatedMBExist, pmbmdRef, pmvRef);
			pmbmd->m_bColocatedMBSkip = (bColocatedMBExist && pmbmdRef->m_bSkip);
			//ShapeMode shpmdColocatedMB = (bColocatedMBExist) ? pmbmdRef->m_shpmd : ALL_TRANSP;
			ShapeMode shpmdColocatedMB = m_rgshpmd [min (max (0, iMBX), m_iRefShpNumMBX - 1)
				+ min (max (0, iMBY), m_iRefShpNumMBY - 1) * m_iRefShpNumMBX];
			decodeInterShape (m_pvopcRefQ0,pmbmd, iMBX, iMBY, x, y, pmv, pmvBY, m_ppxlcCurrMBBY, ppxlcCurrQMBBY, shpmdColocatedMB);
			downSampleBY (m_ppxlcCurrMBBY, m_ppxlcCurrMBBUV);
			if (pmbmd->m_bColocatedMBSkip) { // don't need to send any bit for this mode
				copyFromRefToCurrQ (
					m_pvopcRefQ0,
					x, y, 
					ppxlcCurrQMBY, ppxlcCurrQMBU, ppxlcCurrQMBV,
					&m_rctRefVOPY0
				);
				pmbmd->m_bSkip = TRUE;
				memset (pmv, 0, 5 * sizeof (CMotionVector));
				memset (pmvBackward, 0, 5 * sizeof (CMotionVector));
				memset (pmvBY, 0, sizeof (CMotionVector));
				pmbmd->m_mbType = DIRECT; // can be set to FORWARD mode since the result is the same
			}
			else {
				if (pmbmd->m_rgTranspStatus [0] != ALL) {
					decodeMBTextureHeadOfBVOP (pmbmd, iCurrentQP);
					decodeMVofBVOP (pmv, pmvBackward, pmbmd, iMBX, iMBY, pmvRef, pmbmdRef);
					if (!pmbmd->m_bSkip) {
						decodeTextureInterMB (pmbmd);
						motionCompAndAddErrorMB_BVOP (
							pmv, pmvBackward,
							pmbmd, 
							iMBX, iMBY, 
							x, y,
							ppxlcCurrQMBY, ppxlcCurrQMBU, ppxlcCurrQMBV,
							&m_rctRefVOPY0, &m_rctRefVOPY1
						);
					}
					else {
//						memset (pmv, 0, 5 * sizeof (CMotionVector));
//						memset (pmvBackward, 0, 5 * sizeof (CMotionVector));
//						copyFromRefToCurrQ_BVOP (pmbmd, x, y, ppxlcCurrQMBY, ppxlcCurrQMBU, ppxlcCurrQMBV);
						assert(bColocatedMBExist);
						motionCompSkipMB_BVOP (pmbmd, pmv, pmvBackward, x, y,
							ppxlcCurrQMBY, ppxlcCurrQMBU, ppxlcCurrQMBV, &m_rctRefVOPY0, &m_rctRefVOPY1);
					}
				}
			}


			pmbmd++;
			pmv += 5;
			pmvBY++;
			pmvBackward += 5;
			ppxlcCurrQMBY += MB_SIZE;
			ppxlcCurrQMBU += BLOCK_SIZE;
			ppxlcCurrQMBV += BLOCK_SIZE;
			ppxlcCurrQMBBY += MB_SIZE;
		}
		ppxlcCurrQY += m_iFrameWidthYxMBSize;
		ppxlcCurrQU += m_iFrameWidthUVxBlkSize;
		ppxlcCurrQV += m_iFrameWidthUVxBlkSize;
		ppxlcCurrQBY += m_iFrameWidthYxMBSize;
	}
}

// added for error resilience mode By Toshiba
Bool CVideoObjectDecoder::checkResyncMarker()
{
	Int nBitsPeeked;
	Int iStuffedBits = m_pbitstrmIn->peekBitsTillByteAlign (nBitsPeeked);
	assert (nBitsPeeked > 0 && nBitsPeeked <= 8);
	if (iStuffedBits == ((1 << (nBitsPeeked - 1)) - 1))
		return (m_pbitstrmIn->peekBitsFromByteAlign (NUMBITS_VP_RESYNC_MARKER) 
				== RESYNC_MARKER);
	return FALSE;
}

Int	CVideoObjectDecoder::decodeVideoPacketHeader(Int& iCurrentQP)
{
	m_pbitstrmIn -> flush(8);
	UInt uiResyncMarker = m_pbitstrmIn -> getBits (NUMBITS_VP_RESYNC_MARKER);

	Int	NumOfMB = m_ivolWidth/16 * m_ivolHeight/16;
	assert (NumOfMB>0);
	Int LengthOfMBNumber = (Int)(log(NumOfMB-1)/log(2)) + 1;
	UInt uiMBNumber = m_pbitstrmIn -> getBits (LengthOfMBNumber);

	Int stepDecoded = m_pbitstrmIn -> getBits (NUMBITS_VP_QUANTIZER);
	iCurrentQP = stepDecoded; 

	UInt uiHEC = m_pbitstrmIn -> getBits (NUMBITS_VP_HEC);
	if (uiHEC){

		// Time reference and VOP_pred_type
		Int iModuloInc = 0;
		while (m_pbitstrmIn -> getBits (1) != 0)
			iModuloInc++;
		Time tCurrSec = iModuloInc + (m_vopmd.vopPredType != BVOP ? m_tOldModuloBaseDecd : m_tOldModuloBaseDisp);
		Time tVopIncr = m_pbitstrmIn -> getBits (m_iNumBitsTimeIncr);
		assert(m_t == tCurrSec * m_volmd.iClockRate + tVopIncr);				//in terms of clock ticks

		VOPpredType vopPredType = (VOPpredType) m_pbitstrmIn -> getBits (NUMBITS_VP_PRED_TYPE);
		assert(m_vopmd.vopPredType == vopPredType);

	}
	return	TRUE;
}
