/*************************************************************************

This software module was originally developed by 

	Ming-Chieh Lee (mingcl@microsoft.com), Microsoft Corporation
	Wei-ge Chen (wchen@microsoft.com), Microsoft Corporation
	Bruce Lin (blin@microsoft.com), Microsoft Corporation
	Chuang Gu (chuanggu@microsoft.com), Microsoft Corporation
	(date: March, 1996)

and edited by  
 
        Wei Wu (weiwu@stallion.risc.rockwell.com), Rockwell Science Center 
        Homer Chen (homer@jupiter.risc.rockwell.com), Rockwell Science Center 

in the course of development of the MPEG-4 Video (ISO/IEC 14496-2). 
This software module is an implementation of a part of one or more MPEG-4 Video tools 
as specified by the MPEG-4 Video. 
ISO/IEC gives users of the MPEG-4 Video free license to this software module or modifications 
thereof for use in hardware or software products claiming conformance to the MPEG-4 Video. 
Those intending to use this software module in hardware or software products are advised that its use may infringe existing patents. 
The original developer of this software module and his/her company, 
the subsequent editors and their companies, 
and ISO/IEC have no liability for use of this software module or modifications thereof in an implementation. 
Copyright is not released for non MPEG-4 Video conforming products. 
Microsoft retains full right to use the code for his/her own purpose, 
assign or donate the code to a third party and to inhibit third parties from using the code for non <MPEG standard> conforming products. 
This copyright notice must be included in all copies or derivative works. 

Copyright (c) 1996, 1997.

Module Name:

	vopSeDec.cpp

Abstract:

	Decoder for one Video Object.

Revision History:

*************************************************************************/
#include <stdio.h>
#include <fstream.h>
#include <math.h>

#include "typeapi.h"
#include "codehead.h"
#include "entropy/bitstrm.h"
#include "entropy/entropy.h"
#include "entropy/huffman.h"
#include "global.h"
#include "mode.h"
#include "vopses.h"
#include "vopsedec.h"
#include "vd_grid.h" // VDS
#include "vd_vector.h" // VDS

#ifdef __MFC_
#ifdef _DEBUG
#undef THIS_FILE
static char BASED_CODE THIS_FILE[] = __FILE__;
#endif

#define new DEBUG_NEW				   
#endif // __MFC_

#ifdef __PC_COMPILER_
#define IOSBINARY ios::binary
#else
#define IOSBINARY ios::bin
#endif // __PC_COMPILER_

CVideoObjectDecoder::~CVideoObjectDecoder ()
{
	delete m_pistrm;
	delete m_pbitstrmIn;
	delete m_pentrdecSet;
	delete m_pvopcRightMB;
	if(m_pgrid!=NULL) { // VDS
		free(m_pgrid);
	}
}

CVideoObjectDecoder::CVideoObjectDecoder (
	const Char* pchStrFile,
	Int iDisplayWidth, Int iDisplayHeight,
	CGrid *pgrid
) : CVideoObject ()
{
	m_pistrm = new ifstream (pchStrFile, IOSBINARY | ios::in);
	m_pbitstrmIn = new CInBitStream (*m_pistrm);
	m_pentrdecSet = new CEntropyDecoderSet (*m_pbitstrmIn);

	m_t = m_tPastRef = m_tFutureRef = 0;
	m_iBCount = 0;

	decodeVOHead ();
	printf ("VOP %d...\n", m_uiVOId);
	decodeVOLHead ();

	if (m_volmd.fAUsage == RECTANGLE) {
		if (iDisplayWidth != m_ivolWidth || iDisplayHeight != m_ivolHeight)
			exit (1);
	}

	m_pgrid = pgrid; // VDS
	if(pgrid!=NULL) {
		Int i, j, k;
		Int nMBX = m_ivolWidth/BLOCK_SIZE;
		Int nMBY = m_ivolHeight/BLOCK_SIZE;
		CRct allmask_rctY(0, 0, m_ivolWidth, m_ivolHeight);
		CRct allmask_rctUV(0, 0, m_ivolWidth/2, m_ivolHeight/2);
		allmaskY = new CU8Image(allmask_rctY, 0);
		allmaskUV = new CU8Image(allmask_rctUV, 0);
		m_vdsCoefQ = (Int****) malloc(nMBY*sizeof(Int***));
		for(j = 0; j < nMBY; j++) {
		    m_vdsCoefQ[j] = (Int***) malloc(nMBX*sizeof(Int**));
		    for(i = 0; i < nMBX; i++) {
			m_vdsCoefQ[j][i] = (Int**) malloc(6*sizeof(Int*));
			for(k = 0; k < 6; k++) {
			    m_vdsCoefQ[j][i][k] = (Int*) malloc(BLOCK_SQUARE_SIZE*sizeof(Int));
			    memset(m_vdsCoefQ[j][i][k], 0, BLOCK_SQUARE_SIZE*sizeof(Int));
			} 
		    }
		}
	}
	
	if (m_uiSprite == 1) { // change iDisplay size in order to get the first sprite piece
		iDisplayWidth = (Int) m_rctSpt.width;
		iDisplayHeight = (Int) m_rctSpt.height ();
	}

	Int iMod = iDisplayWidth % MB_SIZE;
	Int iDisplayWidthRound = (iMod > 0) ? iDisplayWidth + MB_SIZE - iMod : iDisplayWidth;
	iMod = iDisplayHeight % MB_SIZE;
	Int iDisplayHeightRound = (iMod > 0) ? iDisplayHeight + MB_SIZE - iMod : iDisplayHeight;
	m_rctRefFrameY = CRct (
		-EXPANDY_REF_FRAME, -EXPANDY_REF_FRAME, 
		EXPANDY_REF_FRAME + iDisplayWidthRound, EXPANDY_REF_FRAME + iDisplayHeightRound
	);
	m_rctRefFrameUV = m_rctRefFrameY.downSampleBy2 ();
	allocateVOLMembers (iDisplayWidth, iDisplayHeight);

	if (m_volmd.fAUsage == RECTANGLE) {
		m_rctCurrVOPY = CRct (0, 0, iDisplayWidthRound, iDisplayHeightRound);
		m_rctCurrVOPUV = m_rctCurrVOPY.downSampleBy2 ();

		m_rctRefVOPY0 = m_rctCurrVOPY;
		m_rctRefVOPY0.expand (EXPANDY_REFVOP);
		m_rctRefVOPUV0 = m_rctRefVOPY0.downSampleBy2 ();
		
		m_rctRefVOPY1 = m_rctRefVOPY0;
		m_rctRefVOPUV1 = m_rctRefVOPUV0;

		computeVOLConstMembers (); // these VOP members are the same for all frames
	}
	// buffer for shape decoding
	m_pvopcRightMB = new CVOPU8YUVBA (m_volmd.fAUsage, CRct (0, 0, MB_SIZE, MB_SIZE));
	m_ppxlcRightMBBY = (PixelC*) m_pvopcRightMB->pixelsBY ();
	m_ppxlcRightMBBUV = (PixelC*) m_pvopcRightMB->pixelsBUV ();
}

Int CVideoObjectDecoder::decode (vector ViewPos, vector AimPos) // VDS
{
	if (findStartCode () == EOF)
		return EOF;

	Bool bCoded = decodeVOPHead (); // set the bounding box here

	if(bCoded==FALSE)
	{
		cout << "\t" << "Time..." << m_t << "\tNot coded.\n";
		cout.flush ();
		return FALSE;
	}

	// set time stamps
	if(m_vopmd.vopPredType==IVOP || m_vopmd.vopPredType==PVOP)
	{
		m_tPastRef = m_tFutureRef;
		m_tFutureRef = m_t;
		m_iBCount = 0;
	}

	// count B-VOPs
	if(m_vopmd.vopPredType==BVOP)
		m_iBCount++;

	cout << "\t" << "Time..." << m_t;
	switch(m_vopmd.vopPredType)
	{
	case IVOP:
		cout << "\tIVOP";
		break;
	case PVOP:
		cout << "\tPVOP (reference: t=" << m_tPastRef <<")";
		break;
	case BVOP:
		cout << "\tBVOP (past ref: t=" << m_tPastRef
			<< ", future ref: t=" << m_tFutureRef <<")";
		break;
	default:
		break;
	}
	cout << "\n";
	cout.flush ();

	if (!m_bGOV_Reset)
		updateAllRefVOPs ();				// update all reconstructed VOP'sm_pvopcRefQ1->vdlDump ("c:\\refq1.vdl");
	m_bGOV_Reset = FALSE;					// GOV header decoding can reset it.
	if (m_volmd.fAUsage != RECTANGLE)
		resetBYPlane ();
	
	if (m_volmd.fAUsage != RECTANGLE) {
		setRefStartingPointers ();
		computeVOPMembers ();
	}

	if(m_pgrid==NULL) { // VDS
		decodeVOP ();
	} else { // VDS: overload function
		decodeIVOP(ViewPos, AimPos);
	}

	// store the direct mode data
	if (m_vopmd.vopPredType != BVOP) {
		if(m_volmd.fAUsage != RECTANGLE)
			saveShapeMode();
		CMBMode* pmbmdTmp = m_rgmbmd;
		m_rgmbmd = m_rgmbmdRef;
		m_rgmbmdRef = pmbmdTmp;
		CMotionVector* pmvTmp = m_rgmv;
		m_rgmv = m_rgmvRef;
		m_rgmvRef = pmvTmp;
	}
	
	if (m_volmd.fAUsage != RECTANGLE) {
		if (m_vopmd.vopPredType != BVOP) {
			m_iNumMBRef = m_iNumMB;
			m_iNumMBXRef = m_iNumMBX;
			m_iNumMBYRef = m_iNumMBY;
			m_iOffsetForPadY = m_rctRefFrameY.offset (m_rctCurrVOPY.left, m_rctCurrVOPY.top);
			m_iOffsetForPadUV = m_rctRefFrameUV.offset (m_rctCurrVOPUV.left, m_rctCurrVOPUV.top);
			m_rctPrevNoExpandY = m_rctCurrVOPY;
			m_rctPrevNoExpandUV = m_rctCurrVOPUV;
			
			m_rctRefVOPY1 = m_rctCurrVOPY;
			m_rctRefVOPY1.expand (EXPANDY_REFVOP);
			m_rctRefVOPUV1 = m_rctCurrVOPUV;
			m_rctRefVOPUV1.expand (EXPANDUV_REFVOP);
			m_pvopcRefQ1->setBoundRct (m_rctRefVOPY1);
		}
		//give a comment that this is ac/dc pred stuff
		Int nBlk = (m_volmd.fAUsage == EIGHT_BIT) ? 10 : 6;
		delete [] m_rgblkmCurrMB;
		for (Int iMB = 0; iMB < m_iNumMBX; iMB++)	{
			for (Int iBlk = 0; iBlk < nBlk; iBlk++)	{
				delete [] (m_rgpmbmAbove [iMB]->rgblkm) [iBlk];
				delete [] (m_rgpmbmCurr  [iMB]->rgblkm) [iBlk];
			}
			delete [] m_rgpmbmAbove [iMB]->rgblkm;
			delete m_rgpmbmAbove [iMB];
			delete [] m_rgpmbmCurr [iMB]->rgblkm;
			delete m_rgpmbmCurr [iMB];
		}
		delete [] m_rgpmbmAbove;
		delete [] m_rgpmbmCurr;
	}
	// update the direct mode data if it is a P-VOP
	return TRUE;
}

Int CVideoObjectDecoder::findStartCode()
{
	// ensure byte alignment
	m_pbitstrmIn->flush ();

	while(m_pbitstrmIn->peekBits(NUMBITS_START_CODE_PREFIX)!=START_CODE_PREFIX)
	{
		m_pbitstrmIn->getBits(8);
		if(m_pbitstrmIn->eof()==EOF)
			return EOF;
	}
	m_pbitstrmIn->getBits(NUMBITS_START_CODE_PREFIX);
	return 0;
}

Void CVideoObjectDecoder::decodeVOHead ()
{
	findStartCode();
	UInt uiVoStartCode = m_pbitstrmIn -> getBits (NUMBITS_VO_START_CODE);
	assert(uiVoStartCode == VO_START_CODE);

	m_uiVOId = m_pbitstrmIn -> getBits (NUMBITS_VO_ID);
}

Void CVideoObjectDecoder::decodeVOLHead ()
{
	findStartCode();
	UInt uiVolStartCode = m_pbitstrmIn -> getBits (NUMBITS_VOL_START_CODE);
	assert(uiVolStartCode == VOL_START_CODE);

	UInt uiVOLId = m_pbitstrmIn -> getBits (NUMBITS_VOL_ID);
	UInt uiAUsage = m_pbitstrmIn -> getBits (NUMBITS_VOL_SHAPE); 
	m_volmd.iClockRate = m_pbitstrmIn -> getBits (NUMBITS_TIME_RESOLUTION);
	Int iClockRate = m_volmd.iClockRate;
	assert (iClockRate >= 1 && iClockRate < 65536);
	for (m_iNumBitsTimeIncr = 1; m_iNumBitsTimeIncr <= 15; m_iNumBitsTimeIncr++)	{	
		if (iClockRate == 1)			
			break;
		iClockRate = (iClockRate >> 1);
	}

#ifdef __VERIFICATION_MODEL_
	if(uiAUsage==3)  
#else
	if(uiAUsage==2)  // shape-only mode
#endif // __VERIFICATION_MODEL_
	{
		m_volmd.bShapeOnly=TRUE;
		m_volmd.fAUsage=ONE_BIT;

		// default to some values - probably not needed
		m_volmd.bAdvPredDisable = 0;
		m_volmd.fQuantizer = Q_H263; 
		m_volmd.bErrorResilientDisable = 1;
		m_volmd.bIntraPredictionDisable = 0;
		m_volmd.bSeprMotionTexture = 0;
		m_volmd.volType = BASE_LAYER;
		m_volmd.bDeblockFilterDisable = TRUE;
	}
	else
	{
		m_volmd.bShapeOnly=FALSE;
#ifndef __VERIFICATION_MODEL_
		if(uiAUsage==3)   // if WD, allow grey level alpha but map to 3, not 2.
			uiAUsage=2;
#endif // __VERIFICATION_MODEL_
		m_volmd.fAUsage = (AlphaUsage) uiAUsage;
		if (m_volmd.fAUsage == RECTANGLE) {
			Int iMarker1 = m_pbitstrmIn -> getBits (MARKER_BIT);
			m_ivolWidth = m_pbitstrmIn -> getBits (NUMBITS_VOP_WIDTH);
			Int iMarker  = m_pbitstrmIn -> getBits (MARKER_BIT);
			m_ivolHeight = m_pbitstrmIn -> getBits (NUMBITS_VOP_HEIGHT);
			if (iMarker != 1)	
				errorInBitstream ("Bit 1 not expected");
		}		
	}

	if(m_volmd.bShapeOnly==TRUE)
		return;

#ifdef __VERIFICATION_MODEL_
	Int iMarker = m_pbitstrmIn -> getBits (MARKER_BIT);
	if (iMarker != 1)	
		errorInBitstream ("Bit 1 not expected");
	m_pbitstrmIn -> getBits (1);				//VOL_NOT_8_BIT_VIDEO
#endif //__VERIFICATION_MODEL_

	m_volmd.bAdvPredDisable = m_pbitstrmIn -> getBits (1);  //VOL_obmc_Disable

#ifdef __VERIFICATION_MODEL_
	if (m_volmd.fAUsage != RECTANGLE)
		m_pbitstrmIn->getBits (4);						//VOL_Shape_Effect, no effect
#endif //__VERIFICATION_MODEL_

	// decode sprite info
	m_uiSprite = m_pbitstrmIn -> getBits (NUMBITS_SPRITE_USAGE);
	if (m_uiSprite == 1) { // sprite information
		Int isprite_hdim = m_pbitstrmIn -> getBits (NUMBITS_SPRITE_HDIM);
		Int iMarker = m_pbitstrmIn -> getBits (MARKER_BIT);
		assert (iMarker == 1);
		Int isprite_vdim = m_pbitstrmIn -> getBits (NUMBITS_SPRITE_VDIM);
		iMarker = m_pbitstrmIn -> getBits (MARKER_BIT);
		assert (iMarker == 1);
		Int isprite_left_edge = (m_pbitstrmIn -> getBits (1) == 0) ? m_pbitstrmIn -> getBits (NUMBITS_SPRITE_LEFT_EDGE - 1) : ((Int)m_pbitstrmIn -> getBits (NUMBITS_SPRITE_LEFT_EDGE - 1) - (1 << 12));
		iMarker = m_pbitstrmIn -> getBits (MARKER_BIT);
		assert (iMarker == 1);
		Int isprite_top_edge = (m_pbitstrmIn -> getBits (1) == 0) ? m_pbitstrmIn -> getBits (NUMBITS_SPRITE_TOP_EDGE - 1) : ((Int)m_pbitstrmIn -> getBits (NUMBITS_SPRITE_LEFT_EDGE - 1) - (1 << 12));
		iMarker = m_pbitstrmIn -> getBits (MARKER_BIT);
		assert (iMarker == 1);
		m_rctSpt.left = isprite_left_edge;
		m_rctSpt.right = isprite_left_edge + isprite_hdim;
		m_rctSpt.top = isprite_top_edge;
		m_rctSpt.bottom = isprite_top_edge + isprite_vdim;
		m_rctSpt.width = isprite_hdim;
		if (m_volmd.fAUsage == RECTANGLE) { //rct of initial sprite
			Int isprite_hdim = m_pbitstrmIn -> getBits (NUMBITS_SPRITE_HDIM);
			iMarker = m_pbitstrmIn -> getBits (MARKER_BIT);
			assert (iMarker == 1);
			Int isprite_vdim = m_pbitstrmIn -> getBits (NUMBITS_SPRITE_VDIM);
			iMarker = m_pbitstrmIn -> getBits (MARKER_BIT);
			assert (iMarker == 1);
			Int isprite_left_edge = m_pbitstrmIn -> getBits (NUMBITS_SPRITE_LEFT_EDGE );
			iMarker = m_pbitstrmIn -> getBits (MARKER_BIT);
			assert (iMarker == 1);
			Int isprite_top_edge = m_pbitstrmIn -> getBits (NUMBITS_SPRITE_TOP_EDGE);
			m_rctSpt.left += isprite_left_edge;
			m_rctSpt.right = isprite_left_edge + isprite_hdim;
			m_rctSpt.top += isprite_top_edge;
			m_rctSpt.bottom = isprite_top_edge + isprite_vdim;
			m_rctSpt.width = isprite_hdim;
		}
		m_iNumOfPnts = m_pbitstrmIn -> getBits (NUMBITS_NUM_SPRITE_POINTS);
		m_rgstSrcQ = new CSiteD [m_iNumOfPnts];
		m_rgstDstQ = new CSiteD [m_iNumOfPnts];
		m_uiWarpingAccuracy = m_pbitstrmIn -> getBits (NUMBITS_WARPING_ACCURACY);
		Bool bLightChange = m_pbitstrmIn -> getBits (1);
	}

#ifdef __VERIFICATION_MODEL_
	if (m_volmd.fAUsage != RECTANGLE)	{
		if (m_pbitstrmIn->getBits (1) != 1)
			errorInBitstream ("SADCT not supportted. Bit 1 not expected");
	}
#endif  //__VERIFICATION_MODEL_

	m_volmd.fQuantizer = (Quantizer) m_pbitstrmIn -> getBits (1); 
	if (m_volmd.fQuantizer == Q_MPEG) {
		m_volmd.bLoadIntraMatrix = m_pbitstrmIn -> getBits (1);
		if (m_volmd.bLoadIntraMatrix) {
			UInt i = 0;
			Int iElem;
			do {
				iElem = m_pbitstrmIn -> getBits (NUMBITS_QMATRIX);
				m_volmd.rgiIntraQuantizerMatrix [grgiStandardZigzag[i]] = iElem;
			} while (iElem != 0 && ++i < BLOCK_SQUARE_SIZE);
			for (UInt j = i; j < BLOCK_SQUARE_SIZE; j++) {
				m_volmd.rgiIntraQuantizerMatrix [j] = m_volmd.rgiIntraQuantizerMatrix [grgiStandardZigzag[i - 1]];
			}
		}
		else {
			memcpy (m_volmd.rgiIntraQuantizerMatrix, rgiDefaultIntraQMatrix, BLOCK_SQUARE_SIZE * sizeof (Int));
		}
		m_volmd.bLoadInterMatrix = m_pbitstrmIn -> getBits (1);
		if (m_volmd.bLoadInterMatrix) {
			UInt i = 0;
			Int iElem;
			do {
				iElem = m_pbitstrmIn -> getBits (NUMBITS_QMATRIX);
				m_volmd.rgiInterQuantizerMatrix [grgiStandardZigzag[i]] = iElem;
			} while (iElem != 0 && ++i < BLOCK_SQUARE_SIZE);
			for (UInt j = i; j < BLOCK_SQUARE_SIZE; j++) {
				m_volmd.rgiInterQuantizerMatrix [j] = m_volmd.rgiInterQuantizerMatrix [grgiStandardZigzag[i - 1]];
			}
		}
		else {
			memcpy (m_volmd.rgiInterQuantizerMatrix, rgiDefaultInterQMatrix, BLOCK_SQUARE_SIZE * sizeof (Int));
		}
		if (m_volmd.fAUsage == EIGHT_BIT)	{
			m_volmd.bLoadIntraMatrixAlpha = m_pbitstrmIn -> getBits (1);
			if (m_volmd.bLoadIntraMatrixAlpha) {
				for (UInt i = 0; i < BLOCK_SQUARE_SIZE; i++)	{
					Int iElem = m_pbitstrmIn -> getBits (NUMBITS_QMATRIX);
					if (iElem != 0)
						m_volmd.rgiIntraQuantizerMatrixAlpha [grgiStandardZigzag[i]] = iElem;
					else
						m_volmd.rgiIntraQuantizerMatrixAlpha [i] = m_volmd.rgiIntraQuantizerMatrixAlpha [grgiStandardZigzag[i - 1]];
				}
			}
			else {
				for (UInt i = 0; i < BLOCK_SQUARE_SIZE; i++)
					m_volmd.rgiIntraQuantizerMatrixAlpha [i] = 16;
			}
			m_volmd.bLoadInterMatrixAlpha = m_pbitstrmIn -> getBits (1);
			if (m_volmd.bLoadInterMatrixAlpha) {
				for (UInt i = 0; i < BLOCK_SQUARE_SIZE; i++)	{
					Int iElem = m_pbitstrmIn -> getBits (NUMBITS_QMATRIX);
					if (iElem != 0)
						m_volmd.rgiInterQuantizerMatrixAlpha [grgiStandardZigzag[i]] = iElem;
					else
						m_volmd.rgiInterQuantizerMatrixAlpha [i] = m_volmd.rgiInterQuantizerMatrixAlpha [grgiStandardZigzag[i - 1]];
				}
			}
			else {
				for (UInt i = 0; i < BLOCK_SQUARE_SIZE; i++)
					m_volmd.rgiInterQuantizerMatrixAlpha [i] = 16;
			}
		}
	}
	m_volmd.bErrorResilientDisable = m_pbitstrmIn -> getBits (1);
	m_volmd.bIntraPredictionDisable = m_pbitstrmIn -> getBits (1);
#ifdef __VERIFICATION_MODEL_
	m_volmd.bSeprMotionTexture = m_pbitstrmIn -> getBits (NUMBITS_SEP_MOTION_TEXTURE);
#endif  //__VERIFICATION_MODEL_
	m_volmd.volType = (m_pbitstrmIn -> getBits (1) == 0) ? BASE_LAYER : ENHN_LAYER;
	if (m_volmd.volType == ENHN_LAYER)	{
		m_pbitstrmIn->getBits (1);
		m_pbitstrmIn->getBits (1);
		m_volmd.iEnhnUpFactor = m_pbitstrmIn->getBits (5);			//The vm is not very clear about this.
		m_pbitstrmIn->getBits (5);
		m_volmd.iEnhnUpFactor = m_pbitstrmIn->getBits (5);			//Should be the same for H and V
		m_pbitstrmIn->getBits (5);
		m_pbitstrmIn->getBits (1);									//Always 1
	}
//	computeMVInfo ();
	m_volmd.bDeblockFilterDisable = TRUE;							//no deblocking filter
}

Bool CVideoObjectDecoder::decodeVOPHead ()
{
	// Start code
	UInt uiVopStartCode = m_pbitstrmIn -> getBits (NUMBITS_VOP_START_CODE);
	assert(uiVopStartCode == VOP_START_CODE);

	m_vopmd.vopPredType = (VOPpredType) m_pbitstrmIn -> getBits (NUMBITS_VOP_PRED_TYPE);
	// Time reference and VOP_pred_type
	Int iModuloInc = 0;
	while (m_pbitstrmIn -> getBits (1) != 0)
		iModuloInc++;
	Time tCurrSec = iModuloInc + (m_vopmd.vopPredType != BVOP ? m_tModuloBaseDecd : m_tModuloBaseDisp);
	Time tVopIncr = m_pbitstrmIn -> getBits (m_iNumBitsTimeIncr);
	//m_t = (SOURCE_FRAME_RATE*(tCurrSec * m_volmd.iClockRate + tVopIncr))/m_volmd.iClockRate; // in frames
	m_t = (tCurrSec * m_volmd.iClockRate + tVopIncr); // in clocks
	if (m_volmd.bErrorResilientDisable != TRUE)		{
		m_tOldModuloBaseDecd = m_tModuloBaseDecd;
		m_tOldModuloBaseDisp = m_tModuloBaseDisp;
	}
	if (m_vopmd.vopPredType != BVOP)
	{
		m_tModuloBaseDisp = m_tModuloBaseDecd;		//update most recently displayed time base
		m_tModuloBaseDecd = tCurrSec;
	}

	if (m_pbitstrmIn->getBits (1) == 0)		//vop_coded == false
		return FALSE;

#ifdef __VERIFICATION_MODEL_
 	if(m_volmd.bShapeOnly==FALSE)
		m_vopmd.iIntraDcSwitchThr = m_pbitstrmIn->getBits (3);
#endif //__VERIFICATION_MODEL_

	if (m_vopmd.vopPredType == PVOP && m_volmd.bShapeOnly==FALSE)
		m_vopmd.iRoundingControl = m_pbitstrmIn->getBits (1); //"VOP_Rounding_Type"
	else
		m_vopmd.iRoundingControl = 0;

	if (m_volmd.fAUsage != RECTANGLE) {
		Int width = m_pbitstrmIn -> getBits (NUMBITS_VOP_WIDTH);
// spt VOP		assert (width % MB_SIZE == 0); // has to be multiples of MB_SIZE
		Int marker;
		marker = m_pbitstrmIn -> getBits (1); // marker bit
		assert(marker==1);
		Int height = m_pbitstrmIn -> getBits (NUMBITS_VOP_HEIGHT);
// spt VOP		assert (height % MB_SIZE == 0); // has to be multiples of MB_SIZE
		marker = m_pbitstrmIn -> getBits (1); // marker bit
		assert(marker==1);
		Int iSign = (m_pbitstrmIn -> getBits (1) == 1)? -1 : 1;
		Int left = iSign * m_pbitstrmIn -> getBits (NUMBITS_VOP_HORIZONTAL_SPA_REF - 1);
		marker = m_pbitstrmIn -> getBits (1); // marker bit
		assert(marker==1);
		iSign = (m_pbitstrmIn -> getBits (1) == 1)? -1 : 1;
		Int top = iSign * m_pbitstrmIn -> getBits (NUMBITS_VOP_VERTICAL_SPA_REF - 1);
		
		m_rctCurrVOPY = CRct (left, top, left + width, top + height);
		m_rctCurrVOPUV = m_rctCurrVOPY.downSampleBy2 ();

#ifdef __VERIFICATION_MODEL_
		Int iVopCr = 0;
		if(m_volmd.bShapeOnly==FALSE)
			iVopCr = m_pbitstrmIn -> getBits (1); //VOP_CR, don't support vop level size conversion
		
		if (iVopCr != 0)	{
			fprintf (stderr, "VOP level shape size conversion and SA-DCT not supported");
			assert (FALSE);
			exit (1);
		}
#endif //__VERIFICATION_MODEL_
		m_volmd.bNoCrChange = m_pbitstrmIn -> getBits (1);	//VOP_CR_Change_Disable
	}

	if(m_volmd.bShapeOnly==TRUE)
	{
		m_vopmd.intStep = 10; // probably not needed 
		m_vopmd.intStepI = 10;
		m_vopmd.intStepB = 10;
		return TRUE;
	}

#ifndef __VERIFICATION_MODEL_
	m_vopmd.iIntraDcSwitchThr = m_pbitstrmIn->getBits (3);
#endif //__VERIFICATION_MODEL_

	if (m_pbitstrmIn -> getBits (1) == TRUE)				//VOL_Interlaced, don't support interlace 
		errorInBitstream ("Interlace not supportted. Bit 1 not expected");

	if (m_vopmd.vopPredType == BVOP) {
		m_vopmd.intDBQuant = m_pbitstrmIn -> getBits (NUMBITS_VOP_DBQUANT);
//m_vopmd.intDBQuant = m_pbitstrmIn -> getBits (5);

		if (m_vopmd.intDBQuant == 0)
			m_vopmd.intStepB = min ((m_vopmd.intStep * 5) >> 2, 31);
		else if (m_vopmd.intDBQuant == 1)
			m_vopmd.intStepB = min ((m_vopmd.intStep * 6) >> 2, 31);
		else if (m_vopmd.intDBQuant == 2)
			m_vopmd.intStepB = min ((m_vopmd.intStep * 7) >> 2, 31);
		else if (m_vopmd.intDBQuant == 3)
			m_vopmd.intStepB = min ((m_vopmd.intStep * 8) >> 2, 31);
		else
			assert (FALSE);
		if (m_volmd.fAUsage == EIGHT_BIT)	{
			if (m_vopmd.intDBQuant == 0)
				m_vopmd.intStepBAlpha = min ((m_vopmd.intStepAlpha * 5) >> 2, 63);
			else if (m_vopmd.intDBQuant == 1)
				m_vopmd.intStepBAlpha = min ((m_vopmd.intStepAlpha * 6) >> 2, 63);
			else if (m_vopmd.intDBQuant == 2)
				m_vopmd.intStepBAlpha = min ((m_vopmd.intStepAlpha * 7) >> 2, 63);
			else if (m_vopmd.intDBQuant == 3)
				m_vopmd.intStepBAlpha = min ((m_vopmd.intStepAlpha * 8) >> 2, 63);
			else
				assert (FALSE);
		}
	}
	else {
		Int stepDecoded = m_pbitstrmIn -> getBits (NUMBITS_VOP_QUANTIZER);
		if (m_vopmd.vopPredType == PVOP) 
			m_vopmd.intStep = stepDecoded; 
		else	{
			m_vopmd.intStepI = stepDecoded;
			m_vopmd.intStep = m_vopmd.intStepI; 
		}
		if (m_volmd.fAUsage == EIGHT_BIT)	{
			stepDecoded = m_pbitstrmIn -> getBits (NUMBITS_VOP_ALPHA_QUANTIZER);
			if (m_vopmd.vopPredType == PVOP) 
				m_vopmd.intStepAlpha = stepDecoded; 
			else {
				m_vopmd.intStepIAlpha = stepDecoded;
				m_vopmd.intStepAlpha = m_vopmd.intStepIAlpha; 
			}
		}
	}
	if (m_vopmd.vopPredType != IVOP) {
		m_vopmd.mvInfoForward.uiFCode = m_pbitstrmIn -> getBits (NUMBITS_VOP_FCODE);
		m_vopmd.mvInfoForward.uiScaleFactor = 1 << (m_vopmd.mvInfoForward.uiFCode - 1);
		m_vopmd.mvInfoForward.uiRange = (m_vopmd.mvInfoForward.uiFCode == 1) ? 32 :
										(m_vopmd.mvInfoForward.uiFCode == 2) ? 64 :
										(m_vopmd.mvInfoForward.uiFCode == 3) ? 128 : 256;
	}
	if (m_vopmd.vopPredType == BVOP) {
		m_vopmd.mvInfoBackward.uiFCode = m_pbitstrmIn -> getBits (NUMBITS_VOP_FCODE);
		m_vopmd.mvInfoBackward.uiScaleFactor = 1 << (m_vopmd.mvInfoBackward.uiFCode - 1);
		m_vopmd.mvInfoBackward.uiRange = (m_vopmd.mvInfoBackward.uiFCode == 1) ? 32 :
										(m_vopmd.mvInfoBackward.uiFCode == 2) ? 64 :
										(m_vopmd.mvInfoBackward.uiFCode == 3) ? 128 : 256;
	}
	return TRUE;
}

Void CVideoObjectDecoder::errorInBitstream (Char* rgchErorrMsg)
{
	fprintf (stderr, "%s at %d\n", rgchErorrMsg, m_pbitstrmIn->getCounter ());
	assert (FALSE);
	exit (1);
}

/*
Void CVideoObjectDecoder::decodeWarpPoints ()
{
	assert (m_iNumOfPnts > 0);

	// Dest corner points MV decoding
	Int rgiU [4], rgiV [4], rgiDU [4], rgiDV [4];
	Int rgiWrpPnt0Del [2];
	CInBitStream* pibstrmWrpPt0 = m_pentrdecSet->m_pentrdecWrpPnt -> bitstream ();
	Int j;
	for (j = 0; j < m_iNumOfPnts; j++) { 
		for (UInt iXorY = 0; iXorY < 2; iXorY++)	{
			Long lSz = m_pentrdecSet->m_pentrdecWrpPnt->decodeSymbol();
			Int iSign = pibstrmWrpPt0->peekBits (1);	//get the sign
			if (iSign == 1)	
				rgiWrpPnt0Del[iXorY] = pibstrmWrpPt0->getBits (lSz);
			else	{
				Int iAbsWrpPnt0Del = ~(pibstrmWrpPt0->getBits (lSz));	//get the magnitude
				Int iMask = ~(0xFFFFFFFF << lSz);	//mask out the irrelavent bits
				rgiWrpPnt0Del[iXorY] = -1 * (iAbsWrpPnt0Del & iMask); //masking and signing
			}
			assert (rgiWrpPnt0Del[iXorY] >= -2047 && rgiWrpPnt0Del[iXorY] <= 2047);
		}
		rgiDU [j] = rgiWrpPnt0Del[0];
		rgiDV [j] = rgiWrpPnt0Del[1];
	}

	// Src corner points decoding
	for (j = 0; j < m_iNumOfPnts; j++) { 
		for (UInt iXorY = 0; iXorY < 2; iXorY++)	{
			Long lSz = m_pentrdecSet->m_pentrdecWrpPnt->decodeSymbol();
			Int iSign = pibstrmWrpPt0->peekBits (1);	//get the sign
			if (iSign == 1)	
				rgiWrpPnt0Del[iXorY] = pibstrmWrpPt0->getBits (lSz);
			else	{
				Int iAbsWrpPnt0Del = ~(pibstrmWrpPt0->getBits (lSz));	//get the magnitude
				Int iMask = ~(0xFFFFFFFF << lSz);	//mask out the irrelavent bits
				rgiWrpPnt0Del[iXorY] = -1 * (iAbsWrpPnt0Del & iMask); //masking and signing
			}
			assert (rgiWrpPnt0Del[iXorY] >= -2047 && rgiWrpPnt0Del[iXorY] <= 2047);
		}
		m_rgstSrcQ [j].x = rgiWrpPnt0Del[0];
		m_rgstSrcQ [j].y = rgiWrpPnt0Del[1];
	}

	// Dest corner points reconstruction 
	rgiU [0] = rgiDU [0];									rgiV [0] = rgiDV [0];
	rgiU [1] = rgiDU [1] + rgiU [0];						rgiV [1] = rgiDV [1] + rgiV [0];
	rgiU [2] = rgiDU [2] + rgiU [0];						rgiV [2] = rgiDV [2] + rgiV [0];
	rgiU [3] = rgiDU [3] + rgiU [2] + rgiU [1] - rgiU [0];	rgiV [3] = rgiDV [3] + rgiV [2] + rgiV [1] - rgiV [0];
	for (j = 0; j < m_iNumOfPnts; j++) {
		m_rgstDstQ [j].x = (rgiU [j] + 2 * (m_rgstSrcQ [j].x - m_rgstSrcQ [0].x)) / 2.0;
		m_rgstDstQ [j].y = (rgiV [j] + 2 * (m_rgstSrcQ [j].y - m_rgstSrcQ [0].y)) / 2.0;
	}
}

Void CVideoObjectDecoder::computeMVInfo ()
{
	m_volmd.mvInfo.uiScaleFactor = 1 << (m_volmd.mvInfo.uiFCode - 1);
	m_volmd.mvInfo.uiRange = 
		(m_volmd.mvInfo.uiFCode == 1) ? 32 :
		(m_volmd.mvInfo.uiFCode == 2) ? 64 :
		128;
}

CVector2D CVideoObjectDecoder::getDiffMV ()
{
	CVector2D vctDiffMV;
	UInt uiResidual;
	Long lSymbol;
	Int iVLC, iSign;

	lSymbol = m_pbitstrmIn -> peekBits (NUMBITS_SPRITE_MV_ESC);
	if (lSymbol == SPRITE_MV_ESC)	{
		lSymbol = m_pbitstrmIn -> getBits (NUMBITS_SPRITE_MV_ESC);
		iSign = m_pbitstrmIn -> getBits (1);
		vctDiffMV.x = m_pbitstrmIn -> getBits (NUMBITS_SPRITE_MV_ESC - 1);
		if (iSign == 0)
			vctDiffMV.x *= -1;
	}
	else {
		lSymbol = m_pentrdecSet->m_pentrdecMV->decodeSymbol ();
		Int iVLC = lSymbol - 32;
		if (iVLC != 0)	
			uiResidual = m_pbitstrmIn->getBits (m_volmd.mvInfo.uiFCode - 1);
		else
			uiResidual = 0;
		vctDiffMV.x  = (Double) deScaleMV (iVLC, uiResidual, m_volmd.mvInfo.uiScaleFactor);
	}

	lSymbol = m_pbitstrmIn -> peekBits (NUMBITS_SPRITE_MV_ESC);
	if (lSymbol == SPRITE_MV_ESC)	{
		lSymbol = m_pbitstrmIn -> getBits (NUMBITS_SPRITE_MV_ESC);
		iSign = m_pbitstrmIn -> getBits (1);
		vctDiffMV.y = m_pbitstrmIn -> getBits (NUMBITS_SPRITE_MV_ESC - 1);
		if (iSign == 0)
			vctDiffMV.y *= -1;
	}
	else {
		lSymbol = m_pentrdecSet->m_pentrdecMV->decodeSymbol ();
		iVLC = lSymbol - 32;
		if (iVLC != 0)
			uiResidual = m_pbitstrmIn->getBits (m_volmd.mvInfo.uiFCode - 1);
		else
			uiResidual = 0;
		vctDiffMV.y  = (Double) deScaleMV (iVLC, uiResidual, m_volmd.mvInfo.uiScaleFactor);
	}

	vctDiffMV = vctDiffMV / 2.0;
	return vctDiffMV;
}

Int CVideoObjectDecoder::deScaleMV (  const Int iVLC, 
								   const UInt uiResidual, 
								   const UInt uiScaleFactor)
{
	if (iVLC == 0 && uiResidual == 0)
		return 0;
	else if (uiScaleFactor == 1)
		return (iVLC);
	else {
		UInt uiAbsDiffMVcomponent = abs (iVLC) * uiScaleFactor + uiResidual - uiScaleFactor + 1;	//changed a'c to enc
		return (sign (iVLC) * uiAbsDiffMVcomponent);
	}
}
*/
