/*************************************************************************

This software module was originally developed by 

	Ming-Chieh Lee (mingcl@microsoft.com), Microsoft Corporation
	Wei-ge Chen (wchen@microsoft.com), Microsoft Corporation
	Bruce Lin (blin@microsoft.com), Microsoft Corporation
	Chuang Gu (chuanggu@microsoft.com), Microsoft Corporation
	(date: March, 1996)

and edited by  
 
        Wei Wu (weiwu@stallion.risc.rockwell.com), Rockwell Science Center 
        Homer Chen (homer@jupiter.risc.rockwell.com), Rockwell Science Center 

in the course of development of the MPEG-4 Video (ISO/IEC 14496-2). 
This software module is an implementation of a part of one or more MPEG-4 Video tools 
as specified by the MPEG-4 Video. 
ISO/IEC gives users of the MPEG-4 Video free license to this software module or modifications 
thereof for use in hardware or software products claiming conformance to the MPEG-4 Video. 
Those intending to use this software module in hardware or software products are advised that its use may infringe existing patents. 
The original developer of this software module and his/her company, 
the subsequent editors and their companies, 
and ISO/IEC have no liability for use of this software module or modifications thereof in an implementation. 
Copyright is not released for non MPEG-4 Video conforming products. 
Microsoft retains full right to use the code for his/her own purpose, 
assign or donate the code to a third party and to inhibit third parties from using the code for non <MPEG standard> conforming products. 
This copyright notice must be included in all copies or derivative works. 

Copyright (c) 1996, 1997.

Module Name:

	vopSeDec.hpp

Abstract:

	Decoder for one Video Object.

Revision History:

	09-30-97: merge error resilient changes from Toshiba by wchen@microsoft.com

*************************************************************************/
#include "vd_vector.h" // VDS

#ifndef __VOPSEDEC_HPP_ 
#define __VOPSEDEC_HPP_

class CVideoObject;
class ifstream;
class CInBitStream;
class CEntropyDecoderSet;
class CEntropyDecoder;
class CGrid; // VDS
class CU8Image; // VDS

Class CVideoObjectDecoder : public CVideoObject
{
public:
	// Constructors
	~CVideoObjectDecoder ();
	CVideoObjectDecoder () {};
	CVideoObjectDecoder ( 
		const Char* pchStrFile, // bitstream file
		Int iDisplayWidth, Int iDisplayHeight,
		CGrid *pgrid=NULL
	);

	// Operations
	Int decode (vector ViewPos=vector(), vector AimPos=vector()); // VDS
	Int decodeSpt ();

///////////////// implementation /////////////////

protected:
	CGrid *m_pgrid; // VDS
	CU8Image *allmaskY, *allmaskUV; // VDS
	Int****m_vdsCoefQ; // VDS
	
	own ifstream* m_pistrm;
	own CInBitStream* m_pbitstrmIn;		  //bitstream
	own CEntropyDecoderSet* m_pentrdecSet;//huffman decoder set
										  //used for video packet header decoding by Toshiba
										  //they are basically cached values of not update after the header decoding
	Time m_tOldModuloBaseDisp;			  //of the most recently displayed I/Pvop
	Time m_tOldModuloBaseDecd;			  //of the most recently decoded I/Pvop
	// buffer data
	CVOPU8YUVBA* m_pvopcRightMB;
	PixelC *m_ppxlcRightMBBY, *m_ppxlcRightMBBUV;

	// error handling
	Void errorInBitstream (Char* rgchErorrMsg);

	Int findStartCode();

	// VO and VOL routines
	Void decodeVOHead ();
	Void decodeVOLHead ();

	// VOP routines
	Bool decodeVOPHead ();
	Void decodeVOP ();
	Void decodeIVOP ();
	Void decodeIVOP (vector ViewPos, vector AimPos); // VDS: overload function
	Void decodePVOP ();
	Void decodeBVOP ();
	Void decodeIVOP_WithShape ();
	Void decodePVOP_WithShape ();
	Void decodeBVOP_WithShape ();
//	Void computeMVInfo ();

	// MB routines	: texture and overhead
	// I-VOP
	Void decodeMBTextureHeadOfIVOP (CMBMode* pmbmd, Int& iCurrQP);
	Void decodeTextureIntraMB (CMBMode* pmbmd, CoordI iMBX, CoordI iMBY, 
			PixelC* ppxlcCurrFrmQY, PixelC* ppxlcCurrFrmQU, PixelC* ppxlcCurrFrmQV, CU8Image**mask=NULL); // VDS
	Void decodeMBAlphaHeadOfIVOP (CMBMode* pmbmd, Int& iCurrQP);
	Void decodeAlphaIntraMB (const CMBMode* pmbmd, PixelC* ppxlcRefMBA);	
	
	// P-VOP
	Void decodeMBTextureHeadOfPVOP (CMBMode* pmbmd, Int& iCurrQP);
	Void decodeMBAlphaHeadOfPVOP (CMBMode* pmbmd, Int& iCurrQP);
	Void decodeTextureInterMB (CMBMode* pmbmd);
	Void decodeAlphaInterMB (const CMBMode* pmbmdpmbmd);
	
	// B-VOP
	Void decodeMBTextureHeadOfBVOP (CMBMode* pmbmd, Int& iCurrQP);
	Void decodeMBAlphaHeadOfBVOP (CMBMode* pmbmd, Int& iCurrQP);
	
	Void setCBPYandC (CMBMode* pmbmd, Int iCBPC, Int iCBPY, Int cNonTrnspBlk);

	// motion compensation
	Void motionCompTexture (
		const CMotionVector* pmv, const CMBMode* pmbmd, 
		Int imbX, Int imbY,
		CoordI x, CoordI y
	);
	Void motionCompAlpha (
		const CMotionVector* pmv, const CMBMode* pmbmd, 
		Int imbX, Int imbY,
		CoordI x, CoordI y
	);
	// B-VOP
	Void motionCompAndAddErrorMB_BVOP (
		const CMotionVector* pmvForward, const CMotionVector* pmvBackward,
		const CMBMode* pmbmd, 
		Int iMBX, Int iMBY, 
		CoordI x, CoordI y,
		PixelC* ppxlcCurrQMBY, PixelC* ppxlcCurrQMBU, PixelC* ppxlcCurrQMBV,
		CRct *prctMVLimitForward,CRct *prctMVLimitBackward
	); 
	Void copyFromRefToCurrQ_BVOP (
		const CMBMode* pmbmd,
		CoordI x, CoordI y, 
		PixelC* ppxlcCurrQMBY, PixelC* ppxlcCurrQMBU, PixelC* ppxlcCurrQMBV,
		CRct *prctMVLimitForward, CRct *prctMVLimitBackward
	);
	Void motionCompSkipMB_BVOP (
		const CMBMode* pmbmd, const CMotionVector* pmvForward, const CMotionVector* pmvBackward,
		CoordI x, CoordI y, 
		PixelC* ppxlcCurrQMBY, PixelC* ppxlcCurrQMBU, PixelC* ppxlcCurrQMBV,
		CRct *prctMVLimitForward,CRct *prctMVLimitBackward
	);


	Void copyFromPredForYAndRefForCToCurrQ (
		CoordI x, CoordI y, 
		PixelC* ppxlcCurrQMBY, PixelC* ppxlcCurrQMBU, PixelC* ppxlcCurrQMBV,
		CRct *prctMVLimit
	); // for obmc mode
	Void copyAlphaFromPredToCurrQ (
		CoordI x, CoordI y, 
		PixelC* ppxlcCurrQMBA
	);

	// error signal for B-VOP MB
	Void averagePredAndAddErrorToCurrQ (
		PixelC* ppxlcCurrQMBY, PixelC* ppxlcCurrQMBU, PixelC* ppxlcCurrQMBV
	);
	Void averagePredAndAssignToCurrQ (
		PixelC* ppxlcCurrQMBY, PixelC* ppxlcCurrQMBU, PixelC* ppxlcCurrQMBV
	);

	// motion vector
	Void decodeMV (const CMBMode* pmbmd, CMotionVector* pmv, Bool bLeftBndry, Bool bRightBndry, Bool bTopBndry);
	Void decodeMVWithShape (const CMBMode* pmbmd, CoordI iMBX, CoordI iMBY, CMotionVector* pmv);
	Void getDiffMV (CVector& vctDiffMV, MVInfo mvinfo);		//get half pel
	Int  deScaleMV (Int iVLC, Int iResidual, Int iScaleFactor);
	Void fitMvInRange (CVector& vctSrc, MVInfo mvinfo);
	Void decodeMVofBVOP (CMotionVector* pmvForward, CMotionVector* pmvBackward, CMBMode* pmbmd, 
						 Int iMBX, Int iMBY, const CMotionVector* pmvRef, const CMBMode* pmbmdRef);
	Void computeDirectForwardMV (CVector vctDiff, CMotionVector* pmv, const CMotionVector* pmvRef, const CMBMode* pmbmdRef);

	// shape decoding functions
	Void swapCurrAndRightMBForShape ();
	Void decodeIntraShape (CMBMode* pmbmd, Int iMBX, Int iMBY, PixelC* ppxlcCurrMBBY, PixelC* ppxlcCurrMBBYFrm);
	Void decodeInterShape (CVOPU8YUVBA* pvopcRefQ,CMBMode* pmbmd, CoordI iMBX, CoordI iMBY, CoordI iX, CoordI iY, CMotionVector* pmv, CMotionVector* pmvBY, 
						   PixelC* ppxlcMBBY, PixelC* ppxlcMBBYFrm, const ShapeMode& shpmdColocatedMB);
	Int  shpMdTableIndex (const CMBMode* pmbmd, Int iMBX, Int iMBY);
	Void decodeIntraCaeBAB (PixelC* ppxlcCurrMBBY, PixelC* ppxlcCurrMBBYFrm);
	Void decodeIntraCAEH ();
	Void decodeIntraCAEV ();
	Void decodeInterCAEH (const PixelC *);
	Void decodeInterCAEV (const PixelC *);
	Void decodeMVDS (CMotionVector& mvDiff);

	// Block decoding functions
	Void decodeIntraBlockTexture (PixelC* rgchBlkDst,
								 Int iWidthDst,
								 Int iQP, 
								 Int iDcScaler,
								 Bool bIsYBlock,
								 MacroBlockMemory* pmbmCurr,
								 CMBMode* pmbmd,
 								 const BlockMemory blkmPred,
								 Int iQPPred,
								 CU8Image *blkmask=NULL, // VDS
								 Int* vdsCoefQ=NULL); // VDS
	Void decideIntraPred (const BlockMemory& blkmRet, 
						   CMBMode* pmbmdCurr,
						   Int&	iQPpred,
						   BlockNum blkn,									   
						   const MacroBlockMemory* pmbmLeft, 
  						   const MacroBlockMemory* pmbmTop, 
						   const MacroBlockMemory* pmbmLeftTop,
						   const MacroBlockMemory* pmbmCurr,
						   const CMBMode* pmbmdLeft,
						   const CMBMode* pmbmdTop,
						   const CMBMode* pmbmdLeftTop);
	Void decodeTextureInterBlock (Int* rgiBlkCurrQ, Int iWidthCurrQ, Int iQP);
	Void decodeIntraTCOEF (Int* rgiCoefQ, Int iCoefStart, Int* rgiZigzag);
	Void decodeInterTCOEF (Int* rgiCoefQ, Int iCoefStart, Int* rgiZigzag);
	Int decodeIntraDCmpeg (Bool bIsYBlk);					//decode the intra-dc: mpeg2 method
	Void decodeInterVLCtableIndex (Int iIndex, Int&  iLevel, Int&  iRun, Bool& bIsLastRun);  // return islastrun, run and level
	Void decodeIntraVLCtableIndex (Int iIndex, Int&  iLevel, Int&  iRun, Bool& bIsLastRun);  // return islastrun, run and level
	typedef Void (CVideoObjectDecoder::*DECODE_TABLE_INDEX)(Int iIndex, Bool& bIsLastRun, Int&  iRun, Int&  iLevel);	//func ptr code escp. decoding
	Void decodeEscape (Int& iLevel, Int& iRun, Int& bIsLastRun, const Int* rgiLMAX, const Int* rgiRMAX, 
					   CEntropyDecoder* pentrdec, DECODE_TABLE_INDEX decodeVLCtableIndex);
	Void inverseDCACPred (const CMBMode* pmbmd, Int iBlkIdx, Int* rgiCoefQ, Int iQP, const BlockMemory blkmPred, Int iQpPred);

	// sprite decoding
	Void decodeWarpPoints ();

	// added for error resilience mode By Toshiba
	Int	checkResyncMarker();
	Bool	decodeVideoPacketHeader(Int& iCurrentQP);
};

#endif // __VOPSEDEC_HPP_
