/*************************************************************************

This software module was originally developed by 

	Yoshihiro Kikuchi (TOSHIBA CORPORATION)
	Takeshi Nagai (TOSHIBA CORPORATION)

    and edited by:

	Toshiaki Watanabe (TOSHIBA CORPORATION)

  in the course of development of the <MPEG-4 Video(ISO/IEC 14496-2)>. This
  software module is an implementation of a part of one or more <MPEG-4 Video
  (ISO/IEC 14496-2)> tools as specified by the <MPEG-4 Video(ISO/IEC 14496-2)
  >. ISO/IEC gives users of the <MPEG-4 Video(ISO/IEC 14496-2)> free license
  to this software module or modifications thereof for use in hardware or
  software products claiming conformance to the <MPEG-4 Video(ISO/IEC 14496-2
  )>. Those intending to use this software module in hardware or software
  products are advised that its use may infringe existing patents. The
  original developer of this software module and his/her company, the
  subsequent editors and their companies, and ISO/IEC have no liability for
  use of this software module or modifications thereof in an implementation.
  Copyright is not released for non <MPEG-4 Video(ISO/IEC 14496-2)>
  conforming products. TOSHIBA CORPORATION retains full right to use the code
  for his/her own purpose, assign or donate the code to a third party and to
  inhibit third parties from using the code for non <MPEG-4 Video(ISO/IEC
  14496-2)> conforming products. This copyright notice must be included in
  all copies or derivative works.
  Copyright (c)1997.

*************************************************************************/

#include <stdio.h>
#include <math.h>
#include <stdlib.h>
#include <iostream.h>

#include "typeapi.h"
#include "codehead.h"
#include "entropy/bitstrm.h"
#include "entropy/entropy.h"
#include "entropy/huffman.h"
#include "mode.h"
#include "global.h"
#include "vopses.h"
#include "vopseenc.h"
#include "errenc.h"

#ifdef __MFC_
#ifdef _DEBUG
#undef THIS_FILE
static char BASED_CODE THIS_FILE[] = __FILE__;
#endif

#define new DEBUG_NEW				   
#endif // __MFC_

// Boundary decision
Bool CVideoObjectEncoder::bVPNoLeft(Int iMBnum, Int iMBX)	{return (iMBnum <= iVPMBnum || iMBX == 0);}
Bool CVideoObjectEncoder::bVPNoRightTop(Int iMBnum, Int iMBX)	{return (iMBnum - m_iNumMBX + 1 < iVPMBnum || iMBX == m_iNumMBX - 1);}
Bool CVideoObjectEncoder::bVPNoTop(Int iMBnum)	{return(iMBnum - m_iNumMBX < iVPMBnum);}
Bool CVideoObjectEncoder::bVPNoLeftTop(Int iMBnum, Int iMBX)	{return(iMBnum - m_iNumMBX - 1 < iVPMBnum || iMBX == 0);}

Void CVideoObjectEncoder::VideoPacketResetVOP(
// Wchen-9-29-97: These are no longer needed since they are member variables now
//	UInt nBitsModuloBase,
//	Int iVopTimeIncr
)
{
	iVPCounter = m_pbitstrmOut -> getCounter();		// reset coding bits counter
	assert(m_iNumMBX>0 && m_iNumMBY>0);
	numBitsVPMBnum = clog2 (m_iNumMBX * m_iNumMBY);	// number of bits for macroblock_number
	numVideoPacket = 1;								// reset number of video packet in a VOP
	iVPMBnum = 0;									// start MB in the 1st VP
// Wchen-9-29-97: These are no longer needed since they are member variables now
	// Time reference
//	numBitsModuloBase = nBitsModuloBase;
//	VopTimeIncr = iVopTimeIncr;
}


Void CVideoObjectEncoder::codeVideoPacketHeader (
	Int iMBX, Int iMBY, Int QuantScale
)
{
	Int iCounter = m_pbitstrmOut -> getCounter();
	if( iCounter - iVPCounter > m_volmd.bVPBitTh ) {
#ifdef __TRACE_AND_STATS_
		m_pbitstrmOut->trace ("Video_Packet_Header\n");
#endif // __TRACE_AND_STATS_
		iVPMBnum = iMBY * m_iNumMBX + iMBX;
		assert(iVPMBnum > 0 && iVPMBnum < m_iNumMBX * m_iNumMBY);
		assert(QuantScale > 0 && QuantScale < 32);

		iVPCounter = iCounter;

		m_pbitstrmOut->flush ();
		m_pbitstrmOut -> putBits (RESYNC_MARKER, NUMBITS_VP_RESYNC_MARKER, "resync_marker");
		m_pbitstrmOut -> putBits (iVPMBnum, numBitsVPMBnum, "VPH_macroblock_number");
		m_pbitstrmOut -> putBits (QuantScale, NUMBITS_VP_QUANTIZER, "VPH_quant_scale");

		Bool	HEC = ((++numVideoPacket) == 2);	// use HEC in 2nd VP
		m_pbitstrmOut -> putBits (HEC, NUMBITS_VP_HEC, "VPH_header_extension_code");
		if( HEC == TRUE ) {
// Wchen-9-29-97: using m_nBitsModuloBase instead
			m_pbitstrmOut -> putBits ((Int) 0xFFFFFFFE, (UInt) m_nBitsModuloBase + 1, "VPH_Modulo_Time_Base");
// Wchen-9-29-97: using m_iVopTimeIncr instead
			m_pbitstrmOut -> putBits (m_iVopTimeIncr, m_iNumBitsTimeIncr, "VPH_Time_Incr");
			m_pbitstrmOut -> putBits (m_vopmd.vopPredType, NUMBITS_VP_PRED_TYPE, "VPH_Pred_Type");
		}
	}
}


UInt CVideoObjectEncoder::encodeMVVP (const CMotionVector* pmv, Bool bHas4MV, Int iMBX, Int iMBY)
{
	Int iMBnum = iMBY * m_iNumMBX + iMBX;
	assert(iMBnum >= iVPMBnum);
	return encodeMV (pmv, bHas4MV, bVPNoLeft(iMBnum, iMBX), bVPNoRightTop(iMBnum, iMBX), bVPNoTop(iMBnum));
}
