/*************************************************************************

This software module was originally developed by 

	Ming-Chieh Lee (mingcl@microsoft.com), Microsoft Corporation
	Wei-ge Chen (wchen@microsoft.com), Microsoft Corporation
	(date: June, 1997)

and edited by  
 
        Wei Wu (weiwu@stallion.risc.rockwell.com), Rockwell Science Center 
        Homer Chen (homer@jupiter.risc.rockwell.com), Rockwell Science Center

in the course of development of the MPEG-4 Video (ISO/IEC 14496-2). 
This software module is an implementation of a part of one or more MPEG-4 Video tools 
as specified by the MPEG-4 Video. 
ISO/IEC gives users of the MPEG-4 Video free license to this software module or modifications 
thereof for use in hardware or software products claiming conformance to the MPEG-4 Video. 
Those intending to use this software module in hardware or software products are advised that its use may infringe existing patents. 
The original developer of this software module and his/her company, 
the subsequent editors and their companies, 
and ISO/IEC have no liability for use of this software module or modifications thereof in an implementation. 
Copyright is not released for non MPEG-4 Video conforming products. 
Microsoft retains full right to use the code for his/her own purpose, 
assign or donate the code to a third party and to inhibit third parties from using the code for non <MPEG standard> conforming products. 
This copyright notice must be included in all copies or derivative works. 

Copyright (c) 1996, 1997.

Module Name:

	MBenc.cpp

Abstract:

	MacroBlock encoder

Revision History:

NOTE:
	
	m_pvopfCurrQ holds the original data until it is texture quantized

*************************************************************************/

#include <stdlib.h>
#include <math.h>
#include <iostream.h>
#include "typeapi.h"
#include "codehead.h"
#include "mode.h"
#include "global.h"
#include "entropy/bitstrm.h"
#include "entropy/entropy.h"
#include "entropy/huffman.h"
#include "vopses.h"
#include "vopseenc.h"

#ifdef __MFC_
#ifdef _DEBUG
#undef THIS_FILE
static char BASED_CODE THIS_FILE[] = __FILE__;
#endif

#define new DEBUG_NEW				   
#endif // __MFC_

/*Void CVideoObjectEncoder::encodePVOPMBWithShape (
	PixelC* ppxlcRefMBY, PixelC* ppxlcRefMBU, PixelC* ppxlcRefMBV,
	PixelC* ppxlcRefMBA, PixelC* ppxlcRefBY,
	CMBMode* pmbmd, const CMotionVector* pmv, CMotionVector* pmvBY,
	ShapeMode shpmdColocatedMB,
	Int imbX, Int imbY,
	CoordI x, CoordI y, Int& iQPPrev)
{
	encodePVOPMBJustShape(ppxlcRefBY,pmbmd,shpmdColocatedMB,pmv,pmvBY,x,y,imbX,imbY);
	dumpCachedShapeBits();
	encodePVOPMBTextureWithShape(ppxlcRefMBY,ppxlcRefMBU,ppxlcRefMBV,ppxlcRefMBA,pmbmd,
		pmv,imbX,imbY,x,y,iQPPrev);
}*/

Void CVideoObjectEncoder::encodePVOPMBJustShape(
	PixelC* ppxlcRefBY, CMBMode* pmbmd, ShapeMode shpmdColocatedMB,
		const CMotionVector* pmv, CMotionVector* pmvBY,
		CoordI x, CoordI y, Int imbX, Int imbY)
{
	m_statsMB.nBitsShape += codeInterShape (
		ppxlcRefBY, m_pvopcRefQ0, pmbmd, shpmdColocatedMB,
		pmv, pmvBY, x, y, imbX, imbY
	);

	// change pmbmd to inter if all transparent
	decideTransparencyStatus (pmbmd, m_ppxlcCurrMBBY);
}

Void CVideoObjectEncoder::dumpCachedShapeBits()
{
#ifdef __TRACE_AND_STATS_
	m_pbitstrmOut->trace("INSERTING PRE-ENCODED MB SHAPE STREAM HERE\n");
	m_pbitstrmOut->trace(m_pbitstrmOut->getCounter(),"Location Before");
#endif // __TRACE_AND_STATS_
	m_pbitstrmOut->putBitStream(*m_pbitstrmShapeMBOut);
#ifdef __TRACE_AND_STATS_
	m_pbitstrmOut->trace(m_pbitstrmOut->getCounter(),"Location After");
#endif // __TRACE_AND_STATS_
	m_pbitstrmShapeMBOut->flush();
	m_pbitstrmShapeMBOut->resetAll();
}

Void CVideoObjectEncoder::encodePVOPMBTextureWithShape(
	PixelC* ppxlcRefMBY, PixelC* ppxlcRefMBU, PixelC* ppxlcRefMBV,
	PixelC* ppxlcRefMBA, CMBMode* pmbmd, const CMotionVector* pmv,
	Int imbX, Int imbY,	CoordI x, CoordI y, Int& iQPPrev)
{
	if(m_volmd.bShapeOnly==TRUE)
		return;

	downSampleBY (m_ppxlcCurrMBBY, m_ppxlcCurrMBBUV);

	if (pmbmd -> m_dctMd == INTRA || pmbmd -> m_dctMd == INTRAQ) {
		if(pmbmd -> m_rgTranspStatus [0] == PARTIAL)
			LPEPadding (pmbmd);
		pmbmd->m_stepSize = iQPPrev + pmbmd->m_intStepDelta;	
		iQPPrev = pmbmd->m_stepSize;
		assert (pmbmd -> m_rgTranspStatus [0] != ALL);
		quantizeTextureIntraMB (imbX, imbY, pmbmd, ppxlcRefMBY, ppxlcRefMBU, ppxlcRefMBV);
		codeMBTextureHeadOfPVOP (pmbmd);
		sendDCTCoefOfIntraMBTexture (pmbmd);
		if (m_volmd.fAUsage == EIGHT_BIT) {
			quantizeAlphaIntraMB (pmbmd, ppxlcRefMBA);
			codeMBAlphaHeadOfPVOP (pmbmd);
			sendDCTCoefOfIntraMBAlpha (pmbmd);
		}
	}
	else {
		if (pmbmd -> m_rgTranspStatus [0] != ALL) {
			pmbmd->m_stepSize = iQPPrev + pmbmd->m_intStepDelta;	
			iQPPrev = pmbmd->m_stepSize;
			if (pmbmd -> m_rgTranspStatus [0] == PARTIAL) {
				CoordI xRefUV, yRefUV;
				mvLookupUVWithShape (pmbmd, pmv, xRefUV, yRefUV);
				motionCompUV (m_ppxlcPredMBU, m_ppxlcPredMBV, m_pvopcRefQ0, x, y,
					xRefUV, yRefUV, m_vopmd.iRoundingControl,&m_rctRefVOPY0);
				motionCompMBYEnc (pmv, pmbmd, imbX, imbY, x, y, &m_rctRefVOPY0);
				computeTextureErrorWithShape ();
			}
			else {
				CoordI xRefUV, yRefUV;
				mvLookupUV (pmbmd, pmv, xRefUV, yRefUV);
				motionCompUV (m_ppxlcPredMBU, m_ppxlcPredMBV, m_pvopcRefQ0, x, y,
					xRefUV, yRefUV, m_vopmd.iRoundingControl, &m_rctRefVOPY0);
				motionCompMBYEnc (pmv, pmbmd, imbX, imbY, x, y, &m_rctRefVOPY0);
				computeTextureError ();
			}
			Bool bSkip = pmbmd->m_bhas4MVForward ? (pmv [1].isZero () && pmv [2].isZero () && pmv [3].isZero () && pmv [4].isZero ())
												 : pmv->isZero ();
			quantizeTextureInterMB (pmbmd, pmv, bSkip); // decide COD here
			codeMBTextureHeadOfPVOP (pmbmd);
			if (!pmbmd -> m_bSkip) {
				m_statsMB.nBitsMV += encodeMVWithShape (pmv, pmbmd, imbX, imbY);
				sendDCTCoefOfInterMBTexture (pmbmd);
				addErrorAndPredToCurrQ (ppxlcRefMBY, ppxlcRefMBU, ppxlcRefMBV);
			}
			else
				assignPredToCurrQ (ppxlcRefMBY, ppxlcRefMBU, ppxlcRefMBV);
			if (m_volmd.fAUsage == EIGHT_BIT) {
				motionCompMB (
					m_ppxlcPredMBA, m_pvopcRefQ0->pixelsA (), 
					pmv, pmbmd, 
					imbX, imbY, 
					x, y, FALSE,
					&m_rctRefVOPY0
				);
				if (pmbmd->m_rgTranspStatus [0] == PARTIAL)
					computeAlphaErrorWithShape ();
				else
					computeAlphaError ();
				quantizeAlphaInterMB (pmbmd);
				codeMBAlphaHeadOfPVOP (pmbmd);
				if (!pmbmd -> m_bSkipA) {
					sendDCTCoefOfInterMBAlpha (pmbmd);
					addAlphaErrorAndPredToCurrQ (ppxlcRefMBA);
				}
				else
					assignAlphaPredToCurrQ (ppxlcRefMBA);
			}
		}
	}
}

Void CVideoObjectEncoder::encodePVOPMB (
	PixelC* ppxlcRefMBY, PixelC* ppxlcRefMBU, PixelC* ppxlcRefMBV,
	CMBMode* pmbmd, const CMotionVector* pmv,
	Int iMBX, Int iMBY,
	CoordI x, CoordI y
)
{
	if (pmbmd->m_dctMd == INTRA || pmbmd->m_dctMd == INTRAQ) {
		pmbmd->m_bSkip = FALSE;			//in case use by direct mode in the future
		quantizeTextureIntraMB (iMBX, iMBY, pmbmd, ppxlcRefMBY, ppxlcRefMBU, ppxlcRefMBV);
		codeMBTextureHeadOfPVOP (pmbmd);
		sendDCTCoefOfIntraMBTexture (pmbmd);
	}
	else {
		CoordI xRefUV, yRefUV;
		mvLookupUV (pmbmd, pmv, xRefUV, yRefUV);
		motionCompUV (m_ppxlcPredMBU, m_ppxlcPredMBV, m_pvopcRefQ0, x, y,
			xRefUV, yRefUV, m_vopmd.iRoundingControl,&m_rctRefVOPY0);
		motionCompMBYEnc (pmv, pmbmd, iMBX, iMBY, x, y, &m_rctRefVOPY0);
		computeTextureError ();
		Bool bSkip = pmbmd->m_bhas4MVForward ? (pmv [1].isZero () && pmv [2].isZero () && pmv [3].isZero () && pmv [4].isZero ())
											 : pmv->isZero ();
		quantizeTextureInterMB (pmbmd, pmv, bSkip); // decide COD here
		codeMBTextureHeadOfPVOP (pmbmd);
		if (!pmbmd -> m_bSkip) {
			if (m_volmd.bErrorResilientDisable == TRUE)
				m_statsMB.nBitsMV += encodeMV (pmv, pmbmd->m_bhas4MVForward, iMBX == 0, iMBX == m_iNumMBX - 1, iMBY == 0);
			else
				m_statsMB.nBitsMV += encodeMVVP (pmv, pmbmd->m_bhas4MVForward, iMBX, iMBY);
			sendDCTCoefOfInterMBTexture (pmbmd);
			addErrorAndPredToCurrQ (ppxlcRefMBY, ppxlcRefMBU, ppxlcRefMBV);
		}
		else {
			assignPredToCurrQ (ppxlcRefMBY, ppxlcRefMBU, ppxlcRefMBV);
		}
	}
}


Void CVideoObjectEncoder::encodeBVOPMB (
	PixelC* ppxlcCurrQMBY, PixelC* ppxlcCurrQMBU, PixelC* ppxlcCurrQMBV,
	CMBMode* pmbmd, const CMotionVector* pmv, const CMotionVector* pmvBackward,
	const CMBMode* pmbmdRef, const CMotionVector* pmvRef,
	Int iMBX, Int iMBY,
	CoordI x, CoordI y
)
{
	motionCompAndDiff_BVOP_MB (pmv, pmvBackward, pmbmd, x, y, &m_rctRefVOPY0, &m_rctRefVOPY1);
	Bool bSkip;
	if (pmbmd->m_mbType == DIRECT)	{
		CVector vctRefScaled;
		Int iPartInterval = m_t - m_tPastRef;
		vctRefScaled = pmvRef->trueMVHalfPel () * iPartInterval;
		Int iFullInterval = m_tFutureRef - m_tPastRef;
		vctRefScaled.x /= iFullInterval;		//truncation as per vm
		vctRefScaled.y /= iFullInterval;		//truncation as per vm
		m_vctDirectDeltaMV = pmv->m_vctTrueHalfPel - vctRefScaled;	//mvdb not necessaryly 0 due to truncation of half pel 
		bSkip = (m_vctDirectDeltaMV.x ==0 && m_vctDirectDeltaMV.y == 0);
	}
	else 
		bSkip = FALSE;
	quantizeTextureInterMB (pmbmd, pmv, bSkip); // decide COD here; skip not allowed in non-direct mode
	codeMBTextureHeadOfBVOP (pmbmd);
	if (!pmbmd->m_bSkip) {
		m_statsMB.nBitsMV += encodeMVofBVOP (pmv, pmvBackward, pmbmd, iMBX, iMBY, pmvRef, pmbmdRef);
		sendDCTCoefOfInterMBTexture (pmbmd);
		addErrorAndPredToCurrQ (ppxlcCurrQMBY, ppxlcCurrQMBU, ppxlcCurrQMBV);
	}
	else
		assignPredToCurrQ (ppxlcCurrQMBY, ppxlcCurrQMBU, ppxlcCurrQMBV);
}

Void CVideoObjectEncoder::encodeBVOPMB_WithShape (
	PixelC* ppxlcCurrQMBY, PixelC* ppxlcCurrQMBU, PixelC* ppxlcCurrQMBV, PixelC* ppxlcCurrQMBA, PixelC* ppxlcCurrQBY,
	CMBMode* pmbmd, const CMotionVector* pmv, const CMotionVector* pmvBackward, 
	CMotionVector* pmvBY, ShapeMode shpmdColocatedMB,
	const CMBMode* pmbmdRef, const CMotionVector* pmvRef,
	Int iMBX, Int iMBY,
	CoordI x, CoordI y
)
{
	//in case mb-level rate control is implemented, these two lines should be elliminated 
	pmbmd->m_stepSize = m_vopmd.intStepB;
	pmbmd->m_stepSizeAlpha = m_vopmd.intStepBAlpha;

	m_statsMB.nBitsShape += codeInterShape (
		ppxlcCurrQBY, 
		m_pvopcRefQ0,
		pmbmd, shpmdColocatedMB,
		pmv, pmvBY, 
		x, y, 
		iMBX, iMBY
	);
	downSampleBY (m_ppxlcCurrMBBY, m_ppxlcCurrMBBUV);
	decideTransparencyStatus (pmbmd, m_ppxlcCurrMBBY); // change pmbmd to inter if all transparent
	if (pmbmd->m_rgTranspStatus [0] != ALL) {
		if (pmbmd->m_rgTranspStatus [0] == PARTIAL)
			motionCompAndDiff_BVOP_MB_WithShape (pmv, pmvBackward, pmbmd, x, y, &m_rctRefVOPY0, &m_rctRefVOPY1);
		else
			motionCompAndDiff_BVOP_MB (pmv, pmvBackward, pmbmd, x, y, &m_rctRefVOPY0, &m_rctRefVOPY1);
		Bool bSkip;
		if (pmbmd->m_mbType == DIRECT)	{
			CVector vctRefScaled;
			Int iPartInterval = m_t - m_tPastRef;
			vctRefScaled = pmvRef->trueMVHalfPel () * iPartInterval;
			Int iFullInterval = m_tFutureRef - m_tPastRef;
			vctRefScaled.x /= iFullInterval;		//truncation as per vm
			vctRefScaled.y /= iFullInterval;		//truncation as per vm
			m_vctDirectDeltaMV = pmv->m_vctTrueHalfPel - vctRefScaled;	//mvdb not necessaryly 0 due to truncation of half pel 
			bSkip = (m_vctDirectDeltaMV.x ==0 && m_vctDirectDeltaMV.y == 0);
		}
		else 
			bSkip = FALSE;
		quantizeTextureInterMB (pmbmd, pmv, bSkip); // decide COD here; skip not allowed in non-direct mode
		codeMBTextureHeadOfBVOP (pmbmd);
		if (!pmbmd -> m_bSkip) {
			m_statsMB.nBitsMV += encodeMVofBVOP (pmv, pmvBackward, pmbmd, iMBX, iMBY, pmvRef, pmbmdRef);	// need to see the MB type to decide what MV to be sent
			sendDCTCoefOfInterMBTexture (pmbmd);
			addErrorAndPredToCurrQ (ppxlcCurrQMBY, ppxlcCurrQMBU, ppxlcCurrQMBV);
		}
		else
			assignPredToCurrQ (ppxlcCurrQMBY, ppxlcCurrQMBU, ppxlcCurrQMBV);
		if (m_volmd.fAUsage == EIGHT_BIT) {
			motionCompMB (
				m_ppxlcPredMBA, m_pvopcRefQ0->pixelsA (), 
				pmv, pmbmd, 
				iMBX, iMBY, 
				x, y, FALSE, &m_rctRefVOPY0
			);
			if (pmbmd->m_rgTranspStatus [0] == PARTIAL)
				computeAlphaErrorWithShape ();
			else
				computeAlphaError ();
			quantizeAlphaInterMB (pmbmd);
			codeMBAlphaHeadOfBVOP (pmbmd);
			if (!pmbmd->m_bSkipA) {
				sendDCTCoefOfInterMBAlpha (pmbmd);
				addAlphaErrorAndPredToCurrQ (ppxlcCurrQMBA);
			}
			else
				assignAlphaPredToCurrQ (ppxlcCurrQMBA);
		}
	}
}

Void CVideoObjectEncoder::codeMBTextureHeadOfIVOP (const CMBMode* pmbmd)
{
	UInt CBPC = (pmbmd->getCodedBlockPattern (U_BLOCK) << 1)
				| pmbmd->getCodedBlockPattern (V_BLOCK);	
														//per defintion of H.263's CBPC 
	assert (CBPC >= 0 && CBPC <= 3);
	Int CBPY = 0;
	UInt cNonTrnspBlk = 0, iBlk;
	for (iBlk = (UInt) Y_BLOCK1; iBlk <= (UInt) Y_BLOCK4; iBlk++) {
		if (pmbmd->m_rgTranspStatus [iBlk] != ALL)	
			cNonTrnspBlk++;
	}
	UInt iBitPos = 1;
	for (iBlk = (UInt) Y_BLOCK1; iBlk <= (UInt) Y_BLOCK4; iBlk++)	{
		if (pmbmd->m_rgTranspStatus [iBlk] != ALL)	{
			CBPY |= pmbmd->getCodedBlockPattern ((BlockNum) iBlk) << (cNonTrnspBlk - iBitPos);
			iBitPos++;
		}
	}
	assert (CBPY >= 0 && CBPY <= 15);								//per defintion of H.263's CBPY 

	Int iSymbol = 4 * pmbmd->m_dctMd + CBPC;
	assert (iSymbol >= 0 && iSymbol <= 7);			//send MCBPC
#ifdef __TRACE_AND_STATS_
	m_pbitstrmOut->trace (iSymbol, "MB_MCBPC");
#endif // __TRACE_AND_STATS_
	m_statsMB.nBitsMCBPC += m_pentrencSet->m_pentrencMCBPCintra->encodeSymbol (iSymbol, "MB_MCBPC");
	if (!m_volmd.bIntraPredictionDisable)
		m_pbitstrmOut->putBits (pmbmd->m_bACPrediction, 1, "MB_ACPRED");
	m_statsMB.nBitsIntraPred++;
#ifdef __TRACE_AND_STATS_
	m_pbitstrmOut->trace (cNonTrnspBlk, "MB_NumNonTranspBlks");
	m_pbitstrmOut->trace (CBPY, "MB_CBPY (P-style)");
#endif // __TRACE_AND_STATS_
	switch (cNonTrnspBlk) {
	case 1:
		m_pbitstrmOut->putBits (CBPY, 1, "MB_CBPY");
		m_statsMB.nBitsCBPY++;
		break;
	case 2:
		m_statsMB.nBitsCBPY += m_pentrencSet->m_pentrencCBPY2->encodeSymbol (CBPY, "MB_CBPY");
		break;
	case 3:
		m_statsMB.nBitsCBPY += m_pentrencSet->m_pentrencCBPY3->encodeSymbol (CBPY, "MB_CBPY");
		break;
	case 4:
		m_statsMB.nBitsCBPY += m_pentrencSet->m_pentrencCBPY->encodeSymbol (CBPY, "MB_CBPY");
		break;
	default:
		assert (FALSE);
	}
	m_statsMB.nIntraMB++;
	if (pmbmd->m_dctMd == INTRAQ)	{
		Int DQUANT = pmbmd->m_intStepDelta;			//send DQUANT
		assert (DQUANT >= -2 && DQUANT <= 2);
		if (DQUANT != 0)	{	
			if (sign (DQUANT) == 1)
				m_pbitstrmOut->putBits (DQUANT + 1, 2, "MB_DQUANT");
			else
				m_pbitstrmOut->putBits (-1 - DQUANT, 2, "MB_DQUANT");
			m_statsMB.nBitsDQUANT += 2;
		}
	}
}

Void CVideoObjectEncoder::codeMBTextureHeadOfPVOP (const CMBMode* pmbmd)
{
	UInt CBPC = (pmbmd->getCodedBlockPattern (U_BLOCK) << 1)
				| pmbmd->getCodedBlockPattern (V_BLOCK);	
														//per defintion of H.263's CBPC 
	assert (CBPC >= 0 && CBPC <= 3);
	Int CBPY = 0;
	UInt cNonTrnspBlk = 0, iBlk;
	for (iBlk = (UInt) Y_BLOCK1; iBlk <= (UInt) Y_BLOCK4; iBlk++) {
		if (pmbmd->m_rgTranspStatus [iBlk] != ALL)	
			cNonTrnspBlk++;
	}
	UInt iBitPos = 1;
	for (iBlk = (UInt) Y_BLOCK1; iBlk <= (UInt) Y_BLOCK4; iBlk++)	{
		if (pmbmd->m_rgTranspStatus [iBlk] != ALL)	{
			CBPY |= pmbmd->getCodedBlockPattern ((BlockNum) iBlk) << (cNonTrnspBlk - iBitPos);
			iBitPos++;
		}
	}
	assert (CBPY >= 0 && CBPY <= 15);								//per defintion of H.263's CBPY 
	m_pbitstrmOut->putBits (pmbmd->m_bSkip, 1, "MB_Skip");
	m_statsMB.nBitsCOD++;
	if (pmbmd->m_bSkip)
		m_statsMB.nSkipMB++;
	else {
		Int iMBtype;								//per H.263's MBtype
		if (pmbmd->m_dctMd == INTRA || pmbmd->m_dctMd == INTRAQ)
			iMBtype = pmbmd->m_dctMd + 3;
		else
			iMBtype = (pmbmd -> m_dctMd - 2) | pmbmd -> m_bhas4MVForward << 1;
		assert (iMBtype >= 0 && iMBtype <= 4);
#ifdef __TRACE_AND_STATS_
		m_pbitstrmOut->trace (iMBtype, "MB_MBtype");
		m_pbitstrmOut->trace (CBPC, "MB_CBPC");
#endif // __TRACE_AND_STATS_
		m_statsMB.nBitsMCBPC += m_pentrencSet->m_pentrencMCBPCinter->encodeSymbol (iMBtype * 4 + CBPC, "MCBPC");
#ifndef MS_BACKWARD_COMPATIBLE
		if (pmbmd->m_dctMd == INTRA || pmbmd->m_dctMd == INTRAQ)	{
			if (!m_volmd.bIntraPredictionDisable)
				m_pbitstrmOut->putBits (pmbmd->m_bACPrediction, 1, "MB_ACPRED");
			m_statsMB.nBitsIntraPred++;
#ifdef __TRACE_AND_STATS_
			m_pbitstrmOut->trace (cNonTrnspBlk, "MB_NumNonTranspBlks");
			m_pbitstrmOut->trace (CBPY, "MB_CBPY (I-style)");
#endif // __TRACE_AND_STATS_
			switch (cNonTrnspBlk) {
			case 1:
				m_pbitstrmOut->putBits (CBPY, 1, "MB_CBPY");
				m_statsMB.nBitsCBPY++;
				break;
			case 2:
				m_statsMB.nBitsCBPY += m_pentrencSet->m_pentrencCBPY2->encodeSymbol (CBPY, "MB_CBPY");
				break;
			case 3:
				m_statsMB.nBitsCBPY += m_pentrencSet->m_pentrencCBPY3->encodeSymbol (CBPY, "MB_CBPY");
				break;
			case 4:
				m_statsMB.nBitsCBPY += m_pentrencSet->m_pentrencCBPY->encodeSymbol (CBPY, "MB_CBPY");
				break;
			default:
				assert (FALSE);
			}
			m_statsMB.nIntraMB++;
		}
		else {
#ifdef __TRACE_AND_STATS_
			m_pbitstrmOut->trace (cNonTrnspBlk, "MB_NumNonTranspBlks");
			m_pbitstrmOut->trace (CBPY, "MB_CBPY (P-style)");
#endif // __TRACE_AND_STATS_
			switch (cNonTrnspBlk) {
			case 1:
				m_pbitstrmOut->putBits (CBPY, 1, "MB_CBPY");
				m_statsMB.nBitsCBPY++;
				break;
			case 2:
				m_statsMB.nBitsCBPY += m_pentrencSet->m_pentrencCBPY2->encodeSymbol (3 - CBPY, "MB_CBPY");
				break;
			case 3:
				m_statsMB.nBitsCBPY += m_pentrencSet->m_pentrencCBPY3->encodeSymbol (7 - CBPY, "MB_CBPY");
				break;
			case 4:
				m_statsMB.nBitsCBPY += m_pentrencSet->m_pentrencCBPY->encodeSymbol (15 - CBPY, "MB_CBPY");
				break;
			default:
				assert (FALSE);
			}
			if(pmbmd ->m_bhas4MVForward)
				m_statsMB.nInter4VMB++;
			else
				m_statsMB.nInterMB++;
		}
#else	//MS_BACKWARD_COMPATIBLE
		if (pmbmd->m_dctMd == INTRA || pmbmd->m_dctMd == INTRAQ)	{
			m_pbitstrmOut->putBits (pmbmd->m_bACPrediction, 1, "MB_ACPRED");
			m_statsMB.nBitsIntraPred++;
		}
#ifdef __TRACE_AND_STATS_
		m_pbitstrmOut->trace (cNonTrnspBlk, "MB_NumNonTranspBlks");
		m_pbitstrmOut->trace (CBPY, "MB_CBPY (P-style)");
#endif // __TRACE_AND_STATS_
		switch (cNonTrnspBlk) {
		case 1:
			m_pbitstrmOut->putBits (CBPY, 1, "MB_CBPY");
			m_statsMB.nBitsCBPY++;
			break;
		case 2:
			m_statsMB.nBitsCBPY += m_pentrencSet->m_pentrencCBPY2->encodeSymbol (3 - CBPY, "MB_CBPY");
			break;
		case 3:
			m_statsMB.nBitsCBPY += m_pentrencSet->m_pentrencCBPY3->encodeSymbol (7 - CBPY, "MB_CBPY");
			break;
		case 4:
			m_statsMB.nBitsCBPY += m_pentrencSet->m_pentrencCBPY->encodeSymbol (15 - CBPY, "MB_CBPY");
			break;
		default:
			assert (FALSE);
		}
#endif  //MS_BACKWARD_COMPATIBLE
		if (pmbmd->m_dctMd == INTERQ || pmbmd->m_dctMd == INTRAQ) {
			Int DQUANT = pmbmd->m_intStepDelta;			//send DQUANT
			assert (DQUANT >= -2 && DQUANT <= 2);
			if (DQUANT != 0) {	
				if (sign (DQUANT) == 1)
					m_pbitstrmOut->putBits (DQUANT + 1, 2, "MB_DQUANT");
				else
					m_pbitstrmOut->putBits (-1 - DQUANT, 2, "MB_DQUANT");
				m_statsMB.nBitsDQUANT += 2;
			}
		}				 
	}
}

Void CVideoObjectEncoder::codeMBTextureHeadOfBVOP (const CMBMode* pmbmd)
{
	U8 uchCBPB;
	if (pmbmd->m_bSkip) {
		assert ((m_volmd.volType == BASE_LAYER && pmbmd->m_mbType == DIRECT) || 
				(m_volmd.volType == ENHN_LAYER && pmbmd->m_mbType == FORWARD));
		if (m_volmd.volType == BASE_LAYER)	{
//new code; will not work for arbitrarilly shaped
			assert (pmbmd -> m_rgTranspStatus [0] != ALL);
			assert (m_volmd.volType == BASE_LAYER && pmbmd->m_mbType == DIRECT);								
			if (pmbmd->m_bColocatedMBSkip)
				return;
//old code
//				if (m_bColocatedMBSkip != NULL)	 {
//					if (pmbmdColocatedRef1 -> m_bSkip &&
//						pmbmdColocatedRef1 -> m_rgTranspStatus [0] != ALL)	{
//						return;
//					}
//				}
		}
		m_pbitstrmOut->putBits (0, 1, "MB_MODB");
		m_statsMB.nBitsMODB++;
		m_statsMB.nSkipMB++;
	}
	else {
		Int iCBPC = (pmbmd->getCodedBlockPattern (U_BLOCK) << 1) | 
					 pmbmd->getCodedBlockPattern (V_BLOCK);			//per defintion of H.263's CBPC 														
		assert (iCBPC >= 0 && iCBPC <= 3);
		Int iCBPY = 0;
		Int iNumNonTrnspBlk = 0, iBlk;
		for (iBlk = Y_BLOCK1; iBlk <= Y_BLOCK4; iBlk++) 
			iNumNonTrnspBlk += (pmbmd->m_rgTranspStatus [iBlk] != ALL);
		Int iBitPos = 1;
		for (iBlk = Y_BLOCK1; iBlk <= Y_BLOCK4; iBlk++)	{
			if (pmbmd->m_rgTranspStatus [iBlk] != ALL)	{
				iCBPY |= pmbmd->getCodedBlockPattern ((BlockNum) iBlk) << (iNumNonTrnspBlk - iBitPos);
				iBitPos++;
			}
		}
		assert (iCBPY >= 0 && iCBPY <= 15);								//per defintion of H.263's CBPY 
		uchCBPB = (iCBPY << 2 | iCBPC) & 0x3F;
		Int iMODB = 0;

		if (uchCBPB == 0)	{
			iMODB = 2;
			m_pbitstrmOut->putBits (iMODB, 2, "MB_MODB");
			m_statsMB.nBitsMODB += 2;
			if (m_volmd.volType == BASE_LAYER)
				m_statsMB.nBitsMBTYPE += m_pentrencSet->m_pentrencMbTypeBVOP->encodeSymbol (pmbmd->m_mbType, "MB_TYPE");
			else {
				Int iMbtypeBits = (pmbmd->m_mbType == FORWARD) ? 1 : 2;
				Int iMbtypeValue = (pmbmd->m_mbType == INTERPOLATE) ? 0 : 1;
				m_statsMB.nBitsMBTYPE += iMbtypeBits;
				m_pbitstrmOut->putBits (iMbtypeValue, iMbtypeBits, "MB_TYPE");
			}
		}
		else	{
			iMODB = 3;
			m_pbitstrmOut->putBits (iMODB, 2,"MB_MODB");
			m_statsMB.nBitsMODB += 2;
			//m_stat.nBitsMBTYPE += m_pentrencMbTypeBVOP->encodeSymbol (pmbmd->m_mbType, "MB_TYPE");
			if (m_volmd.volType == BASE_LAYER)
				m_statsMB.nBitsMBTYPE += m_pentrencSet->m_pentrencMbTypeBVOP->encodeSymbol (pmbmd->m_mbType, "MB_TYPE");
			else {
				UInt iMbtypeBits = (pmbmd->m_mbType == FORWARD) ? 1 : 2;
				Int iMbtypeValue = (pmbmd->m_mbType == INTERPOLATE) ? 0 : 1;
				m_statsMB.nBitsMBTYPE += iMbtypeBits;
				m_pbitstrmOut->putBits (iMbtypeValue, iMbtypeBits, "MB_TYPE");
			}
			m_pbitstrmOut -> putBits (uchCBPB, iNumNonTrnspBlk + 2, "MB_CBPB");
			m_statsMB.nBitsCBPB += iNumNonTrnspBlk + 2;
		}	
		switch (pmbmd->m_mbType)	{
		case FORWARD:
			m_statsMB.nForwardMB++;
			break;
		case BACKWARD:
			m_statsMB.nBackwardMB++;
			break;
		case DIRECT:
			m_statsMB.nDirectMB++;
			break;
		case INTERPOLATE:
			m_statsMB.nInterpolateMB++;
			break;
		default:
			assert (FALSE);
		}
	}
	if (pmbmd->m_mbType != DIRECT)	{
		if (uchCBPB != 0)		{	//no need to send when MODB = 10
			Int DQUANT = pmbmd->m_intStepDelta;			//send DQUANT
			assert (DQUANT == -2 || DQUANT == 2 || DQUANT == 0);
			if (DQUANT == 0)	{
				m_pbitstrmOut->putBits ((Int) 0, (UInt) 1, "MB_DQUANT");
				m_statsMB.nBitsDQUANT += 1;
			}
			else {
				if (DQUANT == 2)	
					m_pbitstrmOut->putBits (3, 2, "MB_DQUANT");
				else 
					m_pbitstrmOut->putBits (2, 2, "MB_DQUANT");
				m_statsMB.nBitsDQUANT += 2;
			}
		}
	}
}


Void CVideoObjectEncoder::copyToCurrBuffWithShape (
	const PixelC* ppxlcCurrY, const PixelC* ppxlcCurrU, const PixelC* ppxlcCurrV,
	const PixelC* ppxlcCurrBY, const PixelC* ppxlcCurrA,
	Int iWidthY, Int iWidthUV
)
{
	PixelC* ppxlcCurrMBY = m_ppxlcCurrMBY;
	PixelC* ppxlcCurrMBU = m_ppxlcCurrMBU;
	PixelC* ppxlcCurrMBV = m_ppxlcCurrMBV;
	PixelC* ppxlcCurrMBBY = m_ppxlcCurrMBBY;
	Int ic;
	for (ic = 0; ic < BLOCK_SIZE; ic++) {
		memcpy (ppxlcCurrMBY, ppxlcCurrY, MB_SIZE);
		memcpy (ppxlcCurrMBU, ppxlcCurrU, BLOCK_SIZE);
		memcpy (ppxlcCurrMBV, ppxlcCurrV, BLOCK_SIZE);
		memcpy (ppxlcCurrMBBY, ppxlcCurrBY, MB_SIZE);
		ppxlcCurrMBY += MB_SIZE; ppxlcCurrY += iWidthY;
		ppxlcCurrMBU += BLOCK_SIZE; ppxlcCurrU += iWidthUV;
		ppxlcCurrMBV += BLOCK_SIZE;	ppxlcCurrV += iWidthUV;
		ppxlcCurrMBBY += MB_SIZE; ppxlcCurrBY += iWidthY;
		
		memcpy (ppxlcCurrMBY, ppxlcCurrY, MB_SIZE); // two rows for Y
		memcpy (ppxlcCurrMBBY, ppxlcCurrBY, MB_SIZE);
		ppxlcCurrMBY += MB_SIZE; ppxlcCurrY += iWidthY;
		ppxlcCurrMBBY += MB_SIZE; ppxlcCurrBY += iWidthY;
	}
	if (m_volmd.fAUsage == EIGHT_BIT) {
		PixelC* ppxlcCurrMBA = m_ppxlcCurrMBA;
		for (ic = 0; ic < MB_SIZE; ic++) {
			memcpy (ppxlcCurrMBA, ppxlcCurrA, MB_SIZE);
			ppxlcCurrMBA += MB_SIZE; 
			ppxlcCurrA += iWidthY;
		}
	}
}

Void CVideoObjectEncoder::copyToCurrBuffJustShape(const PixelC* ppxlcCurrBY,
												  Int iWidthY)
{
	PixelC* ppxlcCurrMBBY = m_ppxlcCurrMBBY;
	Int ic;
	for (ic = 0; ic < BLOCK_SIZE; ic++) {
		memcpy (ppxlcCurrMBBY, ppxlcCurrBY, MB_SIZE);
		ppxlcCurrMBBY += MB_SIZE; ppxlcCurrBY += iWidthY;
		memcpy (ppxlcCurrMBBY, ppxlcCurrBY, MB_SIZE);
		ppxlcCurrMBBY += MB_SIZE; ppxlcCurrBY += iWidthY;
	}
}

Void CVideoObjectEncoder::copyToCurrBuff (
	const PixelC* ppxlcCurrY, const PixelC* ppxlcCurrU, const PixelC* ppxlcCurrV,
	Int iWidthY, Int iWidthUV
)
{
	PixelC* ppxlcCurrMBY = m_ppxlcCurrMBY;
	PixelC* ppxlcCurrMBU = m_ppxlcCurrMBU;
	PixelC* ppxlcCurrMBV = m_ppxlcCurrMBV;
	Int ic;
	for (ic = 0; ic < BLOCK_SIZE; ic++) {
		memcpy (ppxlcCurrMBY, ppxlcCurrY, MB_SIZE);
		memcpy (ppxlcCurrMBU, ppxlcCurrU, BLOCK_SIZE);
		memcpy (ppxlcCurrMBV, ppxlcCurrV, BLOCK_SIZE);
		ppxlcCurrMBY += MB_SIZE; ppxlcCurrY += iWidthY;
		ppxlcCurrMBU += BLOCK_SIZE; ppxlcCurrU += iWidthUV;
		ppxlcCurrMBV += BLOCK_SIZE;	ppxlcCurrV += iWidthUV;
		
		memcpy (ppxlcCurrMBY, ppxlcCurrY, MB_SIZE); // two rows for Y
		ppxlcCurrMBY += MB_SIZE; ppxlcCurrY += iWidthY;
	}
}

// compute error signal

Void CVideoObjectEncoder::computeTextureError ()
{
	CoordI ix;
	// Y
	for (ix = 0; ix < MB_SQUARE_SIZE; ix++)
		m_ppxliErrorMBY [ix] = m_ppxlcCurrMBY [ix] - m_ppxlcPredMBY [ix];

	// UV
	for (ix = 0; ix < BLOCK_SQUARE_SIZE; ix++) {
		m_ppxliErrorMBU [ix] = m_ppxlcCurrMBU [ix] - m_ppxlcPredMBU [ix];
		m_ppxliErrorMBV [ix] = m_ppxlcCurrMBV [ix] - m_ppxlcPredMBV [ix];
	}
}

Void CVideoObjectEncoder::computeTextureErrorWithShape ()
{
	CoordI ix;
	// Y
	for (ix = 0; ix < MB_SQUARE_SIZE; ix++) {
		if (m_ppxlcCurrMBBY [ix] == transpValue)
			m_ppxliErrorMBY [ix] = 0; // zero padding
		else
			m_ppxliErrorMBY [ix] = m_ppxlcCurrMBY [ix] - m_ppxlcPredMBY [ix];
	}

	// UV
	for (ix = 0; ix < BLOCK_SQUARE_SIZE; ix++) {
		if (m_ppxlcCurrMBBUV [ix] == transpValue)
			m_ppxliErrorMBU [ix] = m_ppxliErrorMBV [ix] = 0;
		else {
			m_ppxliErrorMBU [ix] = m_ppxlcCurrMBU [ix] - m_ppxlcPredMBU [ix];
			m_ppxliErrorMBV [ix] = m_ppxlcCurrMBV [ix] - m_ppxlcPredMBV [ix];
		}
	}
}

Void CVideoObjectEncoder::computeAlphaError ()
{
	CoordI ix;
	for (ix = 0; ix < MB_SQUARE_SIZE; ix++)
		m_ppxliErrorMBA [ix] = m_ppxlcCurrMBA [ix] - m_ppxlcPredMBA [ix];
}

Void CVideoObjectEncoder::computeAlphaErrorWithShape ()
{
	CoordI ix;
	for (ix = 0; ix < MB_SQUARE_SIZE; ix++) {
		if (m_ppxlcCurrMBBY [ix] == transpValue)
			m_ppxliErrorMBA [ix] = 0; // zero padding
		else
			m_ppxliErrorMBA [ix] = m_ppxlcCurrMBA [ix] - m_ppxlcPredMBA [ix];
	}
}

Void CVideoObjectEncoder::quantizeTextureIntraMB (	
	Int imbX, Int imbY,
	CMBMode* pmbmd, 
	PixelC* ppxlcCurrQMBY, PixelC* ppxlcCurrQMBU, PixelC* ppxlcCurrQMBV,
	CU8Image** mask // VDS
)
{
	assert (pmbmd != NULL);
	assert (pmbmd -> m_dctMd == INTRA || pmbmd->m_dctMd == INTRAQ);

	assert (pmbmd -> m_rgTranspStatus [0] != ALL);
	if (pmbmd -> m_stepSize < 1)
		pmbmd -> m_stepSize = 1;
	else if (pmbmd -> m_stepSize > 31)
		pmbmd -> m_stepSize = 31;
	Int iQP = pmbmd->m_stepSize;
	for (Int i = 0; i <= 31; i++)	{
		if (iQP <= 4)	{
			m_rgiDcScalerY [i] = 8;
			m_rgiDcScalerC [i] = 8;
		}
		else if (iQP >= 5 && iQP <= 8)	{
			m_rgiDcScalerY [i] = 2 * iQP;
			m_rgiDcScalerC [i] = (iQP + 13) / 2;
		}
		else if (iQP >= 9 && iQP <= 24)	{
			m_rgiDcScalerY [i] = iQP + 8;
			m_rgiDcScalerC [i] = (iQP + 13) / 2;
		}
		else	{
			m_rgiDcScalerY [i] = 2 * iQP - 16;
			m_rgiDcScalerC [i] = iQP - 6;
		}
	}
	Int iCoefToStart;
	if (pmbmd -> m_stepSize >= grgiDCSwitchingThreshold [m_vopmd.iIntraDcSwitchThr])	{
		pmbmd->m_bCodeDcAsAc = TRUE;
		iCoefToStart = 0;
	}
	else {
		pmbmd->m_bCodeDcAsAc = FALSE;
		iCoefToStart = 1;
	}

	//for intra pred
	MacroBlockMemory* pmbmLeft = NULL;
	MacroBlockMemory* pmbmTop = NULL;
	MacroBlockMemory* pmbmLeftTop = NULL;
	CMBMode* pmbmdLeft = NULL;
	CMBMode* pmbmdTop = NULL;
	CMBMode* pmbmdLeftTop = NULL;
	if (m_volmd.bIntraPredictionDisable != TRUE)	{												 
		if ( m_volmd.bErrorResilientDisable == TRUE ) {
			Int imbTop	= imbY - 1;
			if (imbTop >= 0)	{
				pmbmTop  = m_rgpmbmAbove [imbX];
				pmbmdTop = pmbmd - m_iNumMBX;
			}
			if (imbX > 0)	{
				pmbmLeft  = m_rgpmbmCurr [imbX - 1];
				pmbmdLeft = pmbmd -  1; 
			}
			if (imbTop >= 0 && imbX > 0)	{
				pmbmLeftTop  = m_rgpmbmAbove [imbX - 1];
				pmbmdLeftTop = pmbmd - m_iNumMBX - 1; 
			}
		} 
		else {
			Int iMBnum = imbY * m_iNumMBX + imbX;
			if (!bVPNoTop(iMBnum))	{
				pmbmTop  = m_rgpmbmAbove [imbX];
				pmbmdTop = pmbmd - m_iNumMBX;
			}
			if (!bVPNoLeft(iMBnum, imbX))	{
				pmbmLeft  = m_rgpmbmCurr [imbX - 1];
				pmbmdLeft = pmbmd -  1;
			}
			if (!bVPNoLeftTop(iMBnum, imbX))	{
				pmbmLeftTop  = m_rgpmbmAbove [imbX - 1];
				pmbmdLeftTop = pmbmd - m_iNumMBX - 1;
			}
		}
	}

	PixelC* rgchBlkDst = NULL;
	PixelC* rgchBlkSrc = NULL;
	Int iWidthDst, iWidthSrc;
	Int iDcScaler;
	Int* rgiCoefQ;
	Int iSumErr = 0; //sum of error to determin intra ac prediction
	Int iBlk;
	for (iBlk = (Int) Y_BLOCK1; iBlk <= V_BLOCK; iBlk++) { // + 1 is because of the indexing
		if (iBlk < (Int) U_BLOCK) {
			if (pmbmd -> m_rgTranspStatus [iBlk] == ALL) 
				continue;
			switch (iBlk) 
			{
			case (Y_BLOCK1): 
				rgchBlkDst = ppxlcCurrQMBY;
				rgchBlkSrc = m_ppxlcCurrMBY;
				break;
			case (Y_BLOCK2): 
				rgchBlkDst = ppxlcCurrQMBY + BLOCK_SIZE;
				rgchBlkSrc = m_ppxlcCurrMBY + BLOCK_SIZE;
				break;
			case (Y_BLOCK3): 
				rgchBlkDst = ppxlcCurrQMBY + m_iFrameWidthYxBlkSize;
				rgchBlkSrc = m_ppxlcCurrMBY + MB_SIZE * BLOCK_SIZE;
				break;
			case (Y_BLOCK4): 
				rgchBlkDst = ppxlcCurrQMBY + m_iFrameWidthYxBlkSize + BLOCK_SIZE;
				rgchBlkSrc = m_ppxlcCurrMBY + MB_SIZE * BLOCK_SIZE + BLOCK_SIZE;
				break;
			}
			iWidthDst = m_iFrameWidthY;
			iWidthSrc = MB_SIZE;
			iDcScaler = m_rgiDcScalerY [iQP];
		}
		else {
			iWidthDst = m_iFrameWidthUV;
			iWidthSrc = BLOCK_SIZE;
			rgchBlkDst = (iBlk == U_BLOCK) ? ppxlcCurrQMBU: ppxlcCurrQMBV;
			rgchBlkSrc = (iBlk == U_BLOCK) ? m_ppxlcCurrMBU: m_ppxlcCurrMBV;
			iDcScaler = m_rgiDcScalerC [iQP];
		}
		assert (iDcScaler > 0 && iDcScaler <= 46);
		rgiCoefQ = m_rgpiCoefQ [iBlk - 1];
		CU8Image *blkmask; // VDS:
		if(mask==NULL) {
			blkmask = NULL;
		} else {
			blkmask = mask[iBlk-1];
		}
		iSumErr += quantizeIntraBlockTexture (
			rgchBlkSrc,
			iWidthSrc,
			rgchBlkDst,
			iWidthDst,
			rgiCoefQ, 
			iQP, 
			iDcScaler,
			iBlk,	//from here til last
			pmbmLeft, 
			pmbmTop, 
			pmbmLeftTop, 
			m_rgpmbmCurr [imbX],
			pmbmdLeft, 
			pmbmdTop, 
			pmbmdLeftTop, 
			pmbmd,
			blkmask
		); //all for intra-pred
		pmbmd->m_bACPrediction = (m_volmd.bIntraPredictionDisable == FALSE && iSumErr >= 0);
	}

	for (iBlk = (UInt) Y_BLOCK1; iBlk <= V_BLOCK; iBlk++) { // + 1 is because of the indexing
		if (pmbmd->m_rgTranspStatus [iBlk] == ALL) {
			pmbmd->setCodedBlockPattern ((BlockNum) iBlk, FALSE);
			continue;
		}
		rgiCoefQ = m_rgpiCoefQ [iBlk - 1];
		if (m_volmd.bIntraPredictionDisable != TRUE)
			intraPred ((BlockNum) iBlk, pmbmd, rgiCoefQ, iQP, m_rgblkmCurrMB [iBlk - 1], m_rgiQPpred [iBlk - 1]);
		Bool bCoded = FALSE;
		UInt i;
		for (i = iCoefToStart; i < BLOCK_SQUARE_SIZE; i++) {
			if (rgiCoefQ [i] != 0)	{
				bCoded = TRUE;
				break;
			}
		}

		pmbmd->setCodedBlockPattern ((BlockNum) iBlk, bCoded);
	}
}

Void CVideoObjectEncoder::quantizeTextureInterMB (CMBMode* pmbmd, 
												  const CMotionVector* pmv, 
												  Bool bSkip)	//bSkip: tested mv is zero
{
	assert (pmbmd != NULL);
	assert (pmbmd -> m_dctMd == INTER || pmbmd -> m_dctMd == INTERQ);

	assert (pmbmd->m_rgTranspStatus [0] != ALL);
	if (pmbmd -> m_stepSize < 1)
		pmbmd -> m_stepSize = 1;
	else if (pmbmd -> m_stepSize > 31)
		pmbmd -> m_stepSize = 31;
	Int iQP = pmbmd->m_stepSize;


//	Bool bSkip = pmbmd->m_bhas4MVForward ? (bSkipAllowed && pmv [1].isZero () && pmv [2].isZero () && pmv [3].isZero () && pmv [4].isZero ())
//										 : bSkipAllowed && pmv->isZero ();
	Int* rgiBlkCurrQ = m_ppxliErrorMBY;
	Int* rgiCoefQ;
	Int iWidthCurrQ;
	for (UInt iBlk = (UInt) Y_BLOCK1; iBlk <= V_BLOCK; iBlk++) { 
		if (iBlk < (UInt) U_BLOCK) {
			if (pmbmd -> m_rgTranspStatus [iBlk] == ALL) 
				continue;
			switch (iBlk) 
			{
			case (Y_BLOCK1): 
				rgiBlkCurrQ = m_ppxliErrorMBY;
				break;
			case (Y_BLOCK2): 
				rgiBlkCurrQ = m_ppxliErrorMBY + BLOCK_SIZE;
				break;
			case (Y_BLOCK3): 
				rgiBlkCurrQ = m_ppxliErrorMBY + MB_SIZE * BLOCK_SIZE;
				break;
			case (Y_BLOCK4): 
				rgiBlkCurrQ = m_ppxliErrorMBY + MB_SIZE * BLOCK_SIZE + BLOCK_SIZE;
				break;
			}
			iWidthCurrQ = MB_SIZE;
		}
		else {
			iWidthCurrQ = BLOCK_SIZE;
			rgiBlkCurrQ = (iBlk == U_BLOCK) ? m_ppxliErrorMBU: m_ppxliErrorMBV;
		}

		rgiCoefQ = m_rgpiCoefQ [iBlk - 1];
		quantizeTextureInterBlock (rgiBlkCurrQ, iWidthCurrQ, rgiCoefQ, iQP); //all for intra-pred
		Bool bCoded = FALSE;
		UInt i;
		for (i = 0; i < BLOCK_SQUARE_SIZE; i++) {
			if (rgiCoefQ [i] != 0)	{
				bCoded = TRUE;
				break;
			}
		}
		bSkip = bSkip & !bCoded;
		pmbmd->setCodedBlockPattern ((BlockNum) iBlk, bCoded);
	}	
	pmbmd->m_bSkip = bSkip; 
}

Void CVideoObjectEncoder::sendDCTCoefOfInterMBTexture (const CMBMode* pmbmd) 
{
	assert (pmbmd != NULL);
	assert (pmbmd -> m_dctMd == INTER || pmbmd -> m_dctMd == INTERQ);
	assert (pmbmd -> m_rgTranspStatus [0] != ALL);

	UInt nBits, iBlk = 0;
	for (iBlk = Y_BLOCK1; iBlk <= V_BLOCK; iBlk++) {
#ifdef __TRACE_AND_STATS_
		m_pbitstrmOut->trace (iBlk, "BLK_NO");
#endif // __TRACE_AND_STATS_
		if (iBlk < U_BLOCK)
			if (pmbmd -> m_rgTranspStatus [iBlk] == ALL) continue;
		if (pmbmd->getCodedBlockPattern ((BlockNum) iBlk))	{
			Int* rgiCoefQ = m_rgpiCoefQ [iBlk - 1];
#ifdef __TRACE_AND_STATS_
			m_pbitstrmOut->trace (rgiCoefQ, BLOCK_SQUARE_SIZE, "BLK_QUANTIZED_COEF");
#endif // __TRACE_AND_STATS_
			nBits = sendTCOEFInter (rgiCoefQ, 0, grgiStandardZigzag);
			switch (iBlk) {
			case U_BLOCK: 
				m_statsMB.nBitsCr += nBits;
				break;
			case V_BLOCK: 
				m_statsMB.nBitsCb += nBits;
				break;
			default:
				m_statsMB.nBitsY += nBits;
			}
		}
	}	
}

Void CVideoObjectEncoder::sendDCTCoefOfIntraMBTexture (const CMBMode* pmbmd) 
{
	assert (pmbmd != NULL);
	assert (pmbmd -> m_dctMd == INTRA || pmbmd -> m_dctMd == INTRAQ);
	assert (pmbmd->m_rgTranspStatus [0] != ALL);

	UInt iBlk = 0;
	for (iBlk = Y_BLOCK1; iBlk <= V_BLOCK; iBlk++) {
		UInt nBits = 0;
#ifdef __TRACE_AND_STATS_
		m_pbitstrmOut->trace (iBlk, "BLK_NO");
#endif // __TRACE_AND_STATS_
		if (iBlk < U_BLOCK)
			if (pmbmd -> m_rgTranspStatus [iBlk] == ALL) continue;
		Int* rgiCoefQ = m_rgpiCoefQ [iBlk - 1];
#ifdef __TRACE_AND_STATS_
		m_pbitstrmOut->trace (rgiCoefQ, BLOCK_SQUARE_SIZE, "BLK_QUANTIZED_COEF");
#endif // __TRACE_AND_STATS_
		Int iCoefStart = 0;
		if (pmbmd->m_bCodeDcAsAc != TRUE)	{
			iCoefStart = 1;
			nBits = sendIntraDC (rgiCoefQ, (BlockNum) iBlk);
		}
		if (pmbmd->getCodedBlockPattern ((BlockNum) iBlk))	{
			Int* rgiZigzag = grgiStandardZigzag;
			if (pmbmd->m_bACPrediction)	
				rgiZigzag = (pmbmd->m_preddir [iBlk - 1] == HORIZONTAL) ? grgiVerticalZigzag : grgiHorizontalZigzag;
			nBits += sendTCOEFIntra (rgiCoefQ, iCoefStart, rgiZigzag);
		}
		switch (iBlk) {
		case U_BLOCK: 
			m_statsMB.nBitsCr += nBits;
			break;
		case V_BLOCK: 
			m_statsMB.nBitsCb += nBits;
			break;
		default:
			m_statsMB.nBitsY += nBits;
		}
	}	
}

UInt CVideoObjectEncoder::sumAbsCurrMB ()
{
	PixelC* ppxlcCurrMBY = m_ppxlcCurrMBY;
	UInt uisumAbs = 0;
	Int ic;
	for (ic = 0; ic < MB_SQUARE_SIZE; ic++) {
		uisumAbs += ppxlcCurrMBY [ic];
	}
	return uisumAbs;
}

Void CVideoObjectEncoder::quantizeAlphaIntraMB (CMBMode* pmbmd, PixelC* ppxlcRefMBA) {}
Void CVideoObjectEncoder::quantizeAlphaInterMB (CMBMode* pmbmd) {}
Void CVideoObjectEncoder::codeMBAlphaHeadOfIVOP (const CMBMode* pmbmd) {}
Void CVideoObjectEncoder::codeMBAlphaHeadOfPVOP (const CMBMode* pmbmd) {}
Void CVideoObjectEncoder::codeMBAlphaHeadOfBVOP (const CMBMode* pmbmd) {}
Void CVideoObjectEncoder::sendDCTCoefOfIntraMBAlpha (const CMBMode* pmbmd) {}
Void CVideoObjectEncoder::sendDCTCoefOfInterMBAlpha (const CMBMode* pmbmd) {}
