/*************************************************************************

This software module was originally developed by 

	Ming-Chieh Lee (mingcl@microsoft.com), Microsoft Corporation
	Wei-ge Chen (wchen@microsoft.com), Microsoft Corporation
	Bruce Lin (blin@microsoft.com), Microsoft Corporation
	(date: March, 1996)

in the course of development of the MPEG-4 Video (ISO/IEC 14496-2). 
This software module is an implementation of a part of one or more MPEG-4 Video tools 
as specified by the MPEG-4 Video. 
ISO/IEC gives users of the MPEG-4 Video free license to this software module or modifications 
thereof for use in hardware or software products claiming conformance to the MPEG-4 Video. 
Those intending to use this software module in hardware or software products are advised that its use may infringe existing patents. 
The original developer of this software module and his/her company, 
the subsequent editors and their companies, 
and ISO/IEC have no liability for use of this software module or modifications thereof in an implementation. 
Copyright is not released for non MPEG-4 Video conforming products. 
Microsoft retains full right to use the code for his/her own purpose, 
assign or donate the code to a third party and to inhibit third parties from using the code for non <MPEG standard> conforming products. 
This copyright notice must be included in all copies or derivative works. 

Copyright (c) 1996, 1997.


Module Name:

	motEst.cpp

Abstract:

	Motion estimation routines.

Revision History:

*************************************************************************/

#include <stdio.h>
#include <math.h>
#include <stdlib.h>
#include <iostream.h>

#include "typeapi.h"
#include "codehead.h"
#include "global.h"
#include "entropy/bitstrm.h"
#include "entropy/entropy.h"
#include "entropy/huffman.h"
#include "mode.h"
#include "vopses.h"
#include "vopseenc.h"

#ifdef __MFC_
#ifdef _DEBUG
#undef THIS_FILE
static char BASED_CODE THIS_FILE[] = __FILE__;
#endif

#define new DEBUG_NEW				   
#endif // __MFC_

#define FAVORZERO		129
#define FAVOR_DIRECT	129
#define FAVOR_INTER		512
#define FAVOR_16x16		129

inline Int minimum (Int a, Int b, Int c, Int d)
{
	if (a <= b && a <= c && a <= d)
		return a;
	else if (b <= a && b <= c && b <= d)
		return b;
	else if (c <= a && c <= b && c <= d)
		return c;
	else
		return d;
}

Void CVideoObjectEncoder::motionEstPVOP ()
{
	m_iMAD = 0;
	CoordI y = 0; 
	CMBMode* pmbmd = m_rgmbmd;
	CMotionVector* pmv = m_rgmv;
	const PixelC* ppxlcOrigY = m_pvopcOrig->pixelsBoundY ();
	const PixelC* ppxlcRefY = (m_volmd.bOriginalForME ? m_pvopcRefOrig0 : m_pvopcRefQ0)->pixelsY () + m_iStartInRefToCurrRctY;
	Int iMBX, iMBY;
	for (iMBY = 0; iMBY < m_iNumMBY; iMBY++, y += MB_SIZE) {
		const PixelC* ppxlcOrigMBY = ppxlcOrigY;
		const PixelC* ppxlcRefMBY = ppxlcRefY;
		CoordI x = 0;
		for (iMBX = 0; iMBX < m_iNumMBX; iMBX++, x += MB_SIZE)	{
#ifdef __TRACE_AND_STATS_
			m_pbitstrmOut->trace (CSite (iMBX, iMBY), "MB_X_Y");
#endif // __TRACE_AND_STATS_
			copyToCurrBuffY (ppxlcOrigMBY);
			m_iMAD += motionEstMB_PVOP (x, y, pmv, pmbmd, ppxlcRefMBY);
			pmbmd++;
#ifdef __TRACE_AND_STATS_
			m_pbitstrmOut->trace (CSite (iMBX, iMBY), "MB_X_Y");
			m_pbitstrmOut->trace (pmv [0], "MV16");
			m_pbitstrmOut->trace (pmv [1], "MV8");
			m_pbitstrmOut->trace (pmv [2], "MV8");
			m_pbitstrmOut->trace (pmv [3], "MV8");
			m_pbitstrmOut->trace (pmv [4], "MV8");
#endif // __TRACE_AND_STATS_
			pmv += 5;
			ppxlcOrigMBY += MB_SIZE;
			ppxlcRefMBY += MB_SIZE;
		}
		ppxlcOrigY += m_iFrameWidthYxMBSize;
		ppxlcRefY += m_iFrameWidthYxMBSize;
	}
}

Void CVideoObjectEncoder::motionEstPVOP_WithShape ()
{
	m_iMAD = 0;
	CoordI y = m_rctCurrVOPY.top; 
	CMBMode* pmbmd = m_rgmbmd;
	CMotionVector* pmv = m_rgmv;
	const PixelC* ppxlcOrigY = m_pvopcOrig->pixelsBoundY ();
	const PixelC* ppxlcOrigBY = m_pvopcOrig->pixelsBoundBY ();
	const PixelC* ppxlcRefY = (m_volmd.bOriginalForME ? m_pvopcRefOrig0 : m_pvopcRefQ0)->pixelsY () + m_iStartInRefToCurrRctY;
	Int iMBX, iMBY;
	for (iMBY = 0; iMBY < m_iNumMBY; iMBY++, y += MB_SIZE) {
		const PixelC* ppxlcOrigMBY = ppxlcOrigY;
		const PixelC* ppxlcOrigMBBY = ppxlcOrigBY;
		const PixelC* ppxlcRefMBY = ppxlcRefY;
		CoordI x = m_rctCurrVOPY.left;
		for (iMBX = 0; iMBX < m_iNumMBX; iMBX++, x += MB_SIZE)	{
#ifdef __TRACE_AND_STATS_
			if(m_volmd.bShapeOnly==FALSE)
				m_pbitstrmOut->trace (CSite (iMBX, iMBY), "MB_X_Y");
#endif // __TRACE_AND_STATS_
			copyToCurrBuffWithShapeY (ppxlcOrigMBY, ppxlcOrigMBBY);
			decideTransparencyStatus (pmbmd, m_ppxlcCurrMBBY);
			if(m_volmd.bShapeOnly==FALSE)
			{
				if (pmbmd->m_rgTranspStatus [0] == NONE)
					m_iMAD += motionEstMB_PVOP (x, y, pmv, pmbmd, ppxlcRefMBY);
				else if (pmbmd->m_rgTranspStatus [0] == PARTIAL)
					m_iMAD += motionEstMB_PVOP_WithShape (x, y, pmv, pmbmd, ppxlcRefMBY);
#ifdef __TRACE_AND_STATS_
				m_pbitstrmOut->trace (CSite (iMBX, iMBY), "MB_X_Y");
				m_pbitstrmOut->trace (pmv [0], "MV16");
				m_pbitstrmOut->trace (pmv [1], "MV8");
				m_pbitstrmOut->trace (pmv [2], "MV8");
				m_pbitstrmOut->trace (pmv [3], "MV8");
				m_pbitstrmOut->trace (pmv [4], "MV8");
#endif // __TRACE_AND_STATS_
			}
			pmbmd++;
			pmv += 5;
			ppxlcOrigMBY += MB_SIZE;
			ppxlcOrigMBBY += MB_SIZE;
			ppxlcRefMBY += MB_SIZE;
		}
		ppxlcOrigY += m_iFrameWidthYxMBSize;
		ppxlcOrigBY += m_iFrameWidthYxMBSize;
		ppxlcRefY += m_iFrameWidthYxMBSize;
	}
}

Void CVideoObjectEncoder::motionEstBVOP ()
{
	m_iMAD = 0;
	CoordI y = 0; // frame-based, always start from 0
	CMBMode* pmbmd = m_rgmbmd;
	const CMBMode* pmbmdRef = m_rgmbmdRef;
	CMotionVector* pmv = m_rgmv;
	const CMotionVector* pmvRef = m_rgmvRef;		//mv in ref frame (for direct mode)
	CMotionVector* pmvBackward = m_rgmvBackward;
	const PixelC* ppxlcOrigY = m_pvopcOrig->pixelsBoundY ();
	const PixelC* ppxlcRef0Y = m_pvopcRefQ0->pixelsY () + m_iStartInRefToCurrRctY;
	const PixelC* ppxlcRef1Y = m_pvopcRefQ1->pixelsY () + m_iStartInRefToCurrRctY;
	Int iMBX, iMBY;
	for (iMBY = 0; iMBY < m_iNumMBY; iMBY++, y += MB_SIZE) {
		const PixelC* ppxlcOrigMBY = ppxlcOrigY;
		const PixelC* ppxlcRef0MBY = ppxlcRef0Y;
		const PixelC* ppxlcRef1MBY = ppxlcRef1Y;
		CoordI x = 0; // frame-based, always start from 0
		for (iMBX = 0; iMBX < m_iNumMBX; iMBX++, x += MB_SIZE) {
			pmbmd->m_dctMd = INTER; // B-VOP is always INTER
			pmbmd->m_bhas4MVForward = pmbmd->m_bhas4MVBackward = FALSE;  //out encoder doesn't support 4 mv in bvop (not even direct mode)
			pmbmd->m_bColocatedMBSkip = pmbmdRef->m_bSkip;
			if (pmbmd->m_bColocatedMBSkip) {
				pmbmd->m_bSkip = TRUE;
				memset (pmv, 0, 5 * sizeof (CMotionVector));
				pmbmd->m_mbType = DIRECT; // can be set to FORWARD mode since the result is the same
			}
			else {
				copyToCurrBuffY (ppxlcOrigMBY);
				m_iMAD += motionEstMB_BVOP (
					x, y, 
					pmv, pmvBackward,
					pmbmd, 
					pmbmdRef, pmvRef,
					ppxlcRef0MBY, ppxlcRef1MBY
				);
			}
			pmbmd++;
			pmv += 5;
			pmvBackward += 5;
			pmvRef += 5;
			pmbmdRef++;
			ppxlcOrigMBY += MB_SIZE;
			ppxlcRef0MBY += MB_SIZE;
			ppxlcRef1MBY += MB_SIZE;
		}
		ppxlcOrigY += m_iFrameWidthYxMBSize;
		ppxlcRef0Y += m_iFrameWidthYxMBSize;
		ppxlcRef1Y += m_iFrameWidthYxMBSize;
	}
}

Void CVideoObjectEncoder::motionEstBVOP_WithShape ()
{
	m_iMAD = 0;
	CoordI y = m_rctCurrVOPY.top; 
	CMBMode* pmbmd = m_rgmbmd;
	CMotionVector* pmv = m_rgmv;
	CMotionVector* pmvBackward = m_rgmvBackward;
	const CMBMode* pmbmdRef;
	const CMotionVector* pmvRef;
	Bool bColocatedMBExist;
	const PixelC* ppxlcOrigY = m_pvopcOrig->pixelsBoundY ();
	const PixelC* ppxlcOrigBY = m_pvopcOrig->pixelsBoundBY ();
	const PixelC* ppxlcRef0Y = m_pvopcRefOrig0->pixelsY () + m_iStartInRefToCurrRctY;
	const PixelC* ppxlcRef1Y = m_pvopcRefOrig1->pixelsY () + m_iStartInRefToCurrRctY;
	Int iMBX, iMBY;
	for (iMBY = 0; iMBY < m_iNumMBY; iMBY++, y += MB_SIZE) {
		const PixelC* ppxlcOrigMBY = ppxlcOrigY;
		const PixelC* ppxlcOrigMBBY = ppxlcOrigBY;
		const PixelC* ppxlcRef0MBY = ppxlcRef0Y;
		const PixelC* ppxlcRef1MBY = ppxlcRef1Y;
		CoordI x = m_rctCurrVOPY.left;
		for (iMBX = 0; iMBX < m_iNumMBX; iMBX++, x += MB_SIZE) {
			pmbmd->m_dctMd = INTER; // B-VOP is always INTER
			pmbmd->m_bhas4MVForward = pmbmd->m_bhas4MVBackward = FALSE;
			copyToCurrBuffWithShapeY (ppxlcOrigMBY, ppxlcOrigMBBY);
			decideTransparencyStatus (pmbmd, m_ppxlcCurrMBBY);
			if (pmbmd->m_rgTranspStatus [0] != ALL) {
				findColocatedMB (iMBX, iMBY, bColocatedMBExist, pmbmdRef, pmvRef);
				pmbmd->m_bColocatedMBSkip = (bColocatedMBExist && pmbmdRef->m_bSkip);
				if (pmbmd->m_bColocatedMBSkip) {
					pmbmd->m_bSkip = TRUE;
					memset (pmv, 0, 5 * sizeof (CMotionVector));
					m_statsMB.nDirectMB++;
					pmbmd->m_mbType = FORWARD;
				}
				else {
					if (pmbmd->m_rgTranspStatus [0] == NONE) {
						m_iMAD += motionEstMB_BVOP (
							x, y, 
							pmv, pmvBackward,
							pmbmd, 
							pmbmdRef, pmvRef,
							ppxlcRef0MBY, ppxlcRef1MBY,
							bColocatedMBExist
						);
					}
					else if (pmbmd->m_rgTranspStatus [0] == PARTIAL) {
						m_iMAD += motionEstMB_BVOP_WithShape (
							x, y, 
							pmv, pmvBackward,
							pmbmd, 
							pmbmdRef, pmvRef,
							ppxlcRef0MBY, ppxlcRef1MBY,
							bColocatedMBExist
						);
					}
				}
			}
			ppxlcOrigMBBY += MB_SIZE;
			pmbmd++;
			pmv += 5;
			pmvBackward += 5;
			ppxlcOrigMBY += MB_SIZE;
			ppxlcRef0MBY += MB_SIZE;
			ppxlcRef1MBY += MB_SIZE;
		}
		ppxlcOrigY += m_iFrameWidthYxMBSize;
		ppxlcOrigBY += m_iFrameWidthYxMBSize;
		ppxlcRef0Y += m_iFrameWidthYxMBSize;
		ppxlcRef1Y += m_iFrameWidthYxMBSize;
	}
}

Void CVideoObjectEncoder::copyToCurrBuffWithShapeY (const PixelC* ppxlcCurrY, const PixelC* ppxlcCurrBY)
{
	PixelC* ppxlcCurrMBY = m_ppxlcCurrMBY;
	PixelC* ppxlcCurrMBBY = m_ppxlcCurrMBBY;
	Int ic;
	for (ic = 0; ic < MB_SIZE; ic++) {
		memcpy (ppxlcCurrMBY, ppxlcCurrY, MB_SIZE);
		memcpy (ppxlcCurrMBBY, ppxlcCurrBY, MB_SIZE);
		ppxlcCurrMBY += MB_SIZE; ppxlcCurrY += m_iFrameWidthY;
		ppxlcCurrMBBY += MB_SIZE; ppxlcCurrBY += m_iFrameWidthY;
	}
}

Void CVideoObjectEncoder::copyToCurrBuffY (const PixelC* ppxlcCurrY)
{
	PixelC* ppxlcCurrMBY = m_ppxlcCurrMBY;
	Int ic;
	for (ic = 0; ic < MB_SIZE; ic++) {
		memcpy (ppxlcCurrMBY, ppxlcCurrY, MB_SIZE);
		ppxlcCurrMBY += MB_SIZE; ppxlcCurrY += m_iFrameWidthY;
	}
}

Int CVideoObjectEncoder::motionEstMB_PVOP (
	CoordI x, CoordI y, 
	CMotionVector* pmv, CMBMode* pmbmd, 
	const PixelC* ppxlcRefMBY
)
{
	Int iInitSAD = sad16x16At0 (ppxlcRefMBY);
	Int iSADInter = blkmatch16 (pmv, x, y, x, y, iInitSAD, ppxlcRefMBY, m_vopmd.iSearchRangeForward);
	Int iSumDev = sumDev ();
#ifdef __TRACE_AND_STATS_
	m_pbitstrmOut->trace (iSADInter, "MB_SAD16");
#endif // __TRACE_AND_STATS_

	CoordI blkX, blkY;
	Int iSAD8 = 0;
	CMotionVector* pmv8 = pmv + 1;
	blkX = x + BLOCK_SIZE;
	blkY = y + BLOCK_SIZE;
	const PixelC* ppxlcCodedBlkY = m_ppxlcCurrMBY;
	
	// 8x8
	iSAD8 += blockmatch8 (ppxlcCodedBlkY, pmv8, x, y, pmv, m_vopmd.iSearchRangeForward);
	pmv8++;
	ppxlcCodedBlkY += BLOCK_SIZE;
#ifdef __TRACE_AND_STATS_
	m_pbitstrmOut->trace (iSAD8, "MB_SAD8");
#endif // __TRACE_AND_STATS_

	iSAD8 += blockmatch8 (ppxlcCodedBlkY, pmv8, blkX, y, pmv, m_vopmd.iSearchRangeForward);
	pmv8++;
	ppxlcCodedBlkY += BLOCK_SIZE * MB_SIZE - BLOCK_SIZE;
#ifdef __TRACE_AND_STATS_
	m_pbitstrmOut->trace (iSAD8, "MB_SAD8");
#endif // __TRACE_AND_STATS_

	iSAD8 += blockmatch8 (ppxlcCodedBlkY, pmv8, x, blkY, pmv, m_vopmd.iSearchRangeForward);
	pmv8++;
	ppxlcCodedBlkY += BLOCK_SIZE;
#ifdef __TRACE_AND_STATS_
	m_pbitstrmOut->trace (iSAD8, "MB_SAD8");
#endif // __TRACE_AND_STATS_

	iSAD8 += blockmatch8 (ppxlcCodedBlkY, pmv8, blkX, blkY, pmv, m_vopmd.iSearchRangeForward);
#ifdef __TRACE_AND_STATS_
	m_pbitstrmOut->trace (iSAD8, "MB_SAD8");
#endif // __TRACE_AND_STATS_
	
	if (iSADInter - FAVOR_16x16 <= iSAD8) {
		pmbmd -> m_bhas4MVForward = FALSE;
		pmv -> computeTrueMV (); // compute here instead of blkmatch to save computation
		for (UInt i = 1; i < 5; i++) // didn't increment the last pmv
			pmv [i] = *pmv;
	}
	else {
		pmv [1].computeTrueMV ();
		pmv [2].computeTrueMV ();
		pmv [3].computeTrueMV ();
		pmv [4].computeTrueMV ();
		pmbmd -> m_bhas4MVForward = TRUE;
		iSADInter = iSAD8;
	}
	if (iSumDev < (iSADInter - FAVOR_INTER)) {
		pmbmd -> m_bSkip = FALSE;
		pmbmd -> m_dctMd = INTRA;
		memset (pmv, 0, 5 * sizeof (CMotionVector));
		return ((m_bRateControl) ? sumAbsCurrMB () : 0);
	}
	else {
		pmbmd -> m_dctMd = INTER;
		if (pmbmd->m_bhas4MVForward == FALSE && pmv->m_vctTrueHalfPel.x == 0 && pmv->m_vctTrueHalfPel.y == 0)
			return (iSADInter + FAVORZERO);
		else 
			return iSADInter;
	}
}

Int CVideoObjectEncoder::motionEstMB_PVOP_WithShape (
	CoordI x, CoordI y, 
	CMotionVector* pmv, CMBMode* pmbmd, 
	const PixelC* ppxlcRefMBY
)
{
	assert (pmbmd->m_rgTranspStatus [0] == PARTIAL);
	Int iInitSAD = sad16x16At0WithShape (ppxlcRefMBY, pmbmd);
	Int iSADInter = blkmatch16WithShape (pmv, x, y, x, y, iInitSAD, ppxlcRefMBY, m_vopmd.iSearchRangeForward);
	Int iSumDev = sumDevWithShape (pmbmd -> m_rgNumNonTranspPixels [0]);
#ifdef __TRACE_AND_STATS_
	m_pbitstrmOut->trace (iSADInter, "MB_SAD16");
#endif // __TRACE_AND_STATS_

	CoordI blkX, blkY;
	Int iSAD8 = 0;
	CMotionVector* pmv8 = pmv + 1;
	blkX = x + BLOCK_SIZE;
	blkY = y + BLOCK_SIZE;
	const PixelC* ppxlcCodedBlkY = m_ppxlcCurrMBY;
	const PixelC* ppxlcCodedBlkBY = m_ppxlcCurrMBBY;

	// 8 x 8
	iSAD8 += 
		(pmbmd->m_rgTranspStatus [1] == PARTIAL) ? blockmatch8WithShape (ppxlcCodedBlkY, ppxlcCodedBlkBY, pmv8, x, y, pmv, m_vopmd.iSearchRangeForward) :
		(pmbmd->m_rgTranspStatus [1] == NONE) ? blockmatch8 (ppxlcCodedBlkY, pmv8, x, y, pmv, m_vopmd.iSearchRangeForward) : 0;
	pmv8++;
	ppxlcCodedBlkY += BLOCK_SIZE;
	ppxlcCodedBlkBY += BLOCK_SIZE;
#ifdef __TRACE_AND_STATS_
	m_pbitstrmOut->trace (iSAD8, "MB_SAD8");
#endif // __TRACE_AND_STATS_

	iSAD8 += 
		(pmbmd->m_rgTranspStatus [2] == PARTIAL) ? blockmatch8WithShape (ppxlcCodedBlkY, ppxlcCodedBlkBY, pmv8, blkX, y, pmv, m_vopmd.iSearchRangeForward) :
		(pmbmd->m_rgTranspStatus [2] == NONE)	? blockmatch8 (ppxlcCodedBlkY, pmv8, blkX, y, pmv, m_vopmd.iSearchRangeForward) : 0;
	pmv8++;
	ppxlcCodedBlkY += BLOCK_SIZE * MB_SIZE - BLOCK_SIZE;
	ppxlcCodedBlkBY += BLOCK_SIZE * MB_SIZE - BLOCK_SIZE;
#ifdef __TRACE_AND_STATS_
	m_pbitstrmOut->trace (iSAD8, "MB_SAD8");
#endif // __TRACE_AND_STATS_
	
	iSAD8 += 
		(pmbmd->m_rgTranspStatus [3] == PARTIAL) ? blockmatch8WithShape (ppxlcCodedBlkY, ppxlcCodedBlkBY, pmv8, x, blkY, pmv, m_vopmd.iSearchRangeForward) :
		(pmbmd->m_rgTranspStatus [3] == NONE)	? blockmatch8 (ppxlcCodedBlkY, pmv8, x, blkY, pmv, m_vopmd.iSearchRangeForward) : 0;
	pmv8++;
	ppxlcCodedBlkY += BLOCK_SIZE;
	ppxlcCodedBlkBY += BLOCK_SIZE;
#ifdef __TRACE_AND_STATS_
	m_pbitstrmOut->trace (iSAD8, "MB_SAD8");
#endif // __TRACE_AND_STATS_
	
	iSAD8 += 
		(pmbmd->m_rgTranspStatus [4] == PARTIAL) ? blockmatch8WithShape (ppxlcCodedBlkY, ppxlcCodedBlkBY, pmv8, blkX, blkY, pmv, m_vopmd.iSearchRangeForward) :
		(pmbmd->m_rgTranspStatus [4] == NONE)	? blockmatch8 (ppxlcCodedBlkY, pmv8, blkX, blkY, pmv, m_vopmd.iSearchRangeForward) : 0;
#ifdef __TRACE_AND_STATS_
	m_pbitstrmOut->trace (iSAD8, "MB_SAD8");
#endif // __TRACE_AND_STATS_

	if ((iSADInter - ((pmbmd->m_rgNumNonTranspPixels [0] >> 1) + 1)) <= iSAD8) {
		pmbmd -> m_bhas4MVForward = FALSE;
		pmv -> computeTrueMV (); // compute here instead of blkmatch to save computation
		for (UInt i = 1; i < 5; i++) // didn't increment the last pmv
			pmv [i] = *pmv;
	}
	else {
		pmv [1].computeTrueMV ();
		pmv [2].computeTrueMV ();
		pmv [3].computeTrueMV ();
		pmv [4].computeTrueMV ();
		pmbmd -> m_bhas4MVForward = TRUE;
		iSADInter = iSAD8;
	}
	if (iSumDev < (iSADInter - (pmbmd->m_rgNumNonTranspPixels [0] << 1))) {
		pmbmd -> m_bSkip = FALSE;
		pmbmd -> m_dctMd = INTRA;
		memset (pmv, 0, 5 * sizeof (CMotionVector));
		return ((m_bRateControl) ? sumAbsCurrMB () : 0);
	}
	else {
		pmbmd -> m_dctMd = INTER;
		return iSADInter;
	}
}

CMotionVector rgmvDirectBack [5];
CMotionVector rgmvDirectForward [5];

Int CVideoObjectEncoder::motionEstMB_BVOP (
	CoordI x, CoordI y, 
	CMotionVector* pmvForward, CMotionVector* pmvBackward,
	CMBMode* pmbmd,
	const CMBMode* pmbmdRef, const CMotionVector* pmvRef,
	const PixelC* ppxlcRef0MBY, const PixelC* ppxlcRef1MBY,
	Bool bColocatedMBExist
)
{
	Int iInitSAD = sad16x16At0 (ppxlcRef0MBY) + FAVORZERO;
	Int iSADForward = blkmatch16 (pmvForward, x, y, x, y, iInitSAD, ppxlcRef0MBY, m_vopmd.iSearchRangeForward);
	pmvForward->computeTrueMV ();

	Int iSADDirect = 1000000000;
#ifndef OLD_DIRECT_MOTION_ESTI
	CVector vctRefScaled;

	if (bColocatedMBExist && pmbmdRef->m_bhas4MVForward == FALSE) {
		Int iPartInterval = m_t - m_tPastRef;		//initialize at MVDB = 0
		vctRefScaled = pmvRef->trueMVHalfPel () * iPartInterval;
		Int iFullInterval = m_tFutureRef - m_tPastRef;
		vctRefScaled.x /= iFullInterval;				//truncation as per vm
		vctRefScaled.y /= iFullInterval;				//truncation as per vm
		//set up initial forward vector; 
		rgmvDirectForward->iMVX = vctRefScaled.x / 2;
		rgmvDirectForward->iMVY = vctRefScaled.y / 2;
		rgmvDirectForward->iHalfX = 0;
		rgmvDirectForward->iHalfY = 0;
		rgmvDirectForward->computeTrueMV ();
		//set up initial backward vector
		m_vctDirectDeltaMV = rgmvDirectForward->m_vctTrueHalfPel - vctRefScaled;	//mvdb not necessaryly 0 due to truncation of half pel 
		backwardMVFromForwardMV (rgmvDirectBack [0], rgmvDirectForward [0], *pmvRef, m_vctDirectDeltaMV);
		//compute initial sad
		iSADDirect = interpolateAndDiffY (rgmvDirectForward, rgmvDirectBack, x, y,
			&m_rctRefVOPY0, &m_rctRefVOPY1) - FAVORZERO;
		//set up inital position in ref frame
		Int iXInit = x + rgmvDirectForward->iMVX;
		Int iYInit = y + rgmvDirectForward->iMVY;
		const PixelC* ppxlcInitRefMBY = m_pvopcRefQ0->pixelsY () + m_rctRefFrameY.offset (iXInit, iYInit);
		//compute forward mv and sad; to be continue in iSADMin==iSADDirect
		iSADDirect = blkmatch16 (rgmvDirectForward, iXInit, iYInit, x, y, iSADDirect, ppxlcInitRefMBY, 15)	//use 15; leave 1 for half pel
					 - FAVOR_DIRECT;
//		iSADDirect = blkmatch16 (rgmvDirectForward, iXInit, iYInit, x, y, iSADDirect, ppxlcInitRefMBY, 2)	//use 15; leave 1 for half pel
//					 - FAVOR_DIRECT;
		rgmvDirectForward->computeTrueMV ();
		m_vctDirectDeltaMV = rgmvDirectForward->m_vctTrueHalfPel - vctRefScaled;
		backwardMVFromForwardMV (rgmvDirectBack [0], rgmvDirectForward [0], *pmvRef, 
								 m_vctDirectDeltaMV);
		iSADDirect = interpolateAndDiffY (rgmvDirectForward, rgmvDirectBack, x, y,
										  &m_rctRefVOPY0, &m_rctRefVOPY1);
		if (m_vctDirectDeltaMV.x == 0 && m_vctDirectDeltaMV.y == 0)
			iSADDirect -= FAVORZERO;
	}
#else //OLD_DIRECT_MOTION_ESTI
	if (bColocatedMBExist && pmbmdRef->m_bhas4MVForward == FALSE) {
		m_vctDirectDeltaMV = pmvForward [0].m_vctTrueHalfPel - vctRefScaled;
		backwardMVFromForwardMV (pmvForward [0], rgmvDirectBack [0], *pmvRef, 
								 m_vctDirectDeltaMV);
		iSADDirect = interpolateAndDiffY (
			pmvForward, rgmvDirectBack, 
			x, y
		) - FAVOR_DIRECT;
	}
#endif //OLD_DIRECT_MOTION_ESTI

	iInitSAD = sad16x16At0 (ppxlcRef1MBY) + FAVORZERO;
	Int iSADBackward = blkmatch16 (pmvBackward, x, y, x, y, iInitSAD, ppxlcRef1MBY, m_vopmd.iSearchRangeBackward);
	pmvBackward->computeTrueMV ();
	Int iSADInterpolate = interpolateAndDiffY (
		pmvForward, pmvBackward,
		x, y, &m_rctRefVOPY0, &m_rctRefVOPY1
	);

	Int iSADMin = minimum (iSADDirect, iSADForward, iSADBackward, iSADInterpolate);
	Int iBlk;
	if (iSADMin == iSADDirect) {
		pmbmd->m_mbType = DIRECT;
		//compute backward mv
#ifndef OLD_DIRECT_MOTION_ESTI
		pmvForward [0]  = rgmvDirectForward  [0];
#endif //OLD_DIRECT_MOTION_ESTI
		pmvBackward [0] = rgmvDirectBack [0];
		for (iBlk = 1; iBlk <= 4; iBlk++) {
			pmvForward [iBlk] = pmvForward [0];
			pmvBackward [iBlk] = pmvBackward [0];
		}
	}
	else if (iSADMin == iSADForward) {
		pmbmd->m_mbType = FORWARD;
		for (iBlk = 1; iBlk <= 4; iBlk++)
			pmvForward [iBlk] = pmvForward [0];
	}
	else if (iSADMin == iSADBackward) {
		pmbmd->m_mbType = BACKWARD;
		for (iBlk = 1; iBlk <= 4; iBlk++)
			pmvBackward [iBlk] = pmvBackward [0];
	}
	else {
		pmbmd->m_mbType = INTERPOLATE;
		for (iBlk = 1; iBlk <= 4; iBlk++) {
			pmvForward [iBlk] = pmvForward [0];
			pmvBackward [iBlk] = pmvBackward [0];
		}
	}
	return iSADMin;
}

Int CVideoObjectEncoder::motionEstMB_BVOP_WithShape (
	CoordI x, CoordI y, 
	CMotionVector* pmvForward, CMotionVector* pmvBackward,
	CMBMode* pmbmd, 
	const CMBMode* pmbmdRef, const CMotionVector* pmvRef,
	const PixelC* ppxlcRef0MBY, const PixelC* ppxlcRef1MBY,
	Bool bColocatedMBExist
)
{
	assert (pmbmd->m_rgTranspStatus [0] == PARTIAL);

	Int iInitSAD = sad16x16At0WithShape (ppxlcRef0MBY, pmbmd);	//shouldn't favor 0 mv inside this function
	Int iSADForward = blkmatch16WithShape (pmvForward, x, y, x, y, iInitSAD, ppxlcRef0MBY, m_vopmd.iSearchRangeForward);
	pmvForward->computeTrueMV ();
	Int iSADDirect = 1000000000;
#ifndef OLD_DIRECT_MOTION_ESTI
	CVector vctRefScaled;
	if (bColocatedMBExist && pmbmdRef->m_bhas4MVForward == FALSE) {
		Int iPartInterval = m_t - m_tPastRef;		//initialize at MVDB = 0
		vctRefScaled = pmvRef->trueMVHalfPel () * iPartInterval;
		Int iFullInterval = m_tFutureRef - m_tPastRef;
		vctRefScaled.x /= iFullInterval;						//truncation as per vm
		vctRefScaled.y /= iFullInterval;						//truncation as per vm
		//set up initial forward vector; 
		rgmvDirectForward->iMVX = vctRefScaled.x / 2;
		rgmvDirectForward->iMVY = vctRefScaled.y / 2;
		rgmvDirectForward->iHalfX = 0;
		rgmvDirectForward->iHalfY = 0;
		rgmvDirectForward->computeTrueMV ();
		//set up initial backward vector
		m_vctDirectDeltaMV = rgmvDirectForward->m_vctTrueHalfPel - vctRefScaled;	//mvdb not necessaryly 0 due to truncation of half pel 
		backwardMVFromForwardMV (rgmvDirectBack [0], rgmvDirectForward [0], *pmvRef, m_vctDirectDeltaMV);
		//compute initial sad
		iSADDirect = interpolateAndDiffY_WithShape (rgmvDirectForward, rgmvDirectBack, x, y,
			&m_rctRefVOPY0, &m_rctRefVOPY1) - FAVORZERO;;
		//set up inital position in ref frame
		Int iXInit = x + rgmvDirectForward->iMVX;
		Int iYInit = y + rgmvDirectForward->iMVY;
		const PixelC* ppxlcInitRefMBY = m_pvopcRefQ0->pixelsY () + m_rctRefFrameY.offset (iXInit, iYInit);
		//compute forward mv and sad; to be continue in iSADMin==iSADDirect
		iSADDirect = blkmatch16WithShape  (rgmvDirectForward, iXInit, iYInit, x, y, iSADDirect, ppxlcInitRefMBY, 15)
					 - FAVOR_DIRECT;
		rgmvDirectForward->computeTrueMV ();
		m_vctDirectDeltaMV = rgmvDirectForward->m_vctTrueHalfPel - vctRefScaled;
		backwardMVFromForwardMV (rgmvDirectBack [0], rgmvDirectForward [0], *pmvRef, 
								 m_vctDirectDeltaMV);
		iSADDirect = interpolateAndDiffY (rgmvDirectForward, rgmvDirectBack, x, y,
										  &m_rctRefVOPY0, &m_rctRefVOPY1);
		if (m_vctDirectDeltaMV.x == 0 && m_vctDirectDeltaMV.y == 0)
			iSADDirect -= FAVORZERO;
	}
#else //OLD_DIRECT_MOTION_ESTI
	if (bColocatedMBExist && pmbmdRef->m_bhas4MVForward == FALSE) {
		m_vctDirectDeltaMV = pmvForward [0].m_vctTrueHalfPel - vctRefScaled;
		backwardMVFromForwardMV (pmvForward [0], rgmvDirectBack [0], *pmvRef, 
								 m_vctDirectDeltaMV);
		iSADDirect = interpolateAndDiffY_WithShape (
			pmvForward, rgmvDirectBack,
			x, y
		) - ((pmbmd->m_rgNumNonTranspPixels [0] >> 1) + 1);
	}
#endif //OLD_DIRECT_MOTION_ESTI

	iInitSAD = sad16x16At0WithShape (ppxlcRef1MBY, pmbmd);//shouldn't favor 0 mv inside this function
	Int iSADBackward = blkmatch16WithShape (pmvBackward, x, y, x, y, iInitSAD, ppxlcRef1MBY, m_vopmd.iSearchRangeBackward);
	pmvBackward->computeTrueMV ();
	Int iSADInterpolate = interpolateAndDiffY_WithShape (
		pmvForward, pmvBackward,
		x, y, &m_rctRefVOPY0, &m_rctRefVOPY1
	);

	Int iSADMin = minimum (iSADDirect, iSADForward, iSADBackward, iSADInterpolate);
	Int iBlk;
	if (iSADMin == iSADDirect) {
		pmbmd->m_mbType = DIRECT;
		//compute backward mv
#ifndef OLD_DIRECT_MOTION_ESTI
		pmvForward [0] = rgmvDirectForward [0];
#endif //OLD_DIRECT_MOTION_ESTI
		pmvBackward [0] = rgmvDirectBack [0];
		for (iBlk = 1; iBlk <= 4; iBlk++) {
			pmvForward [iBlk] = pmvForward [0];
			pmvBackward [iBlk] = pmvBackward [0];
		}
	}
	else if (iSADMin == iSADForward) {
		pmbmd->m_mbType = FORWARD;
		for (iBlk = 1; iBlk <= 4; iBlk++)
			pmvForward [iBlk] = pmvForward [0];
	}
	else if (iSADMin == iSADBackward) {
		pmbmd->m_mbType = BACKWARD;
		for (iBlk = 1; iBlk <= 4; iBlk++)
			pmvBackward [iBlk] = pmvBackward [0];
	}
	else {
		pmbmd->m_mbType = INTERPOLATE;
		for (iBlk = 1; iBlk <= 4; iBlk++) {
			pmvForward [iBlk] = pmvForward [0];
			pmvBackward [iBlk] = pmvBackward [0];
		}
	}
	return iSADMin;
}

Int CVideoObjectEncoder::blkmatch16 (
	CMotionVector* pmv, 
	CoordI iXRef, CoordI iYRef,
	CoordI iXCurr, CoordI iYCurr,
	Int iMinSAD,
	const PixelC* ppxlcRefMBY,
	Int iSearchRange
)
{
	Int mbDiff; // start with big
	Int iXDest = iXRef, iYDest = iYRef;
	CoordI x = iXRef, y = iYRef, ix, iy;
	Int uiPosInLoop, iLoop;
	// Spiral Search for the rest
	for (iLoop = 1; iLoop <= iSearchRange; iLoop++) {
		x++;
		y++;
		ppxlcRefMBY += m_iFrameWidthY + 1;
		for (uiPosInLoop = 0; uiPosInLoop < (iLoop << 3); uiPosInLoop++) { // inside each spiral loop 
			if (
				x >= m_rctRefVOPY0.left && y >= m_rctRefVOPY0.top && 
				x <= m_rctRefVOPY0.right - MB_SIZE && y <= m_rctRefVOPY0.bottom - MB_SIZE
			) {
				const PixelC* ppxlcTmpC = m_ppxlcCurrMBY;
				const PixelC* ppxlcRefMB = ppxlcRefMBY;
				mbDiff = 0;
				for (iy = 0; iy < MB_SIZE; iy++) {
					for (ix = 0; ix < MB_SIZE; ix++)
						mbDiff += abs (ppxlcTmpC [ix] - ppxlcRefMB [ix]);
#ifndef MS_BACKWARD_COMPATIBLE
					if (mbDiff >= iMinSAD)
#else
					if (mbDiff > iMinSAD)
#endif
						goto NEXT_POSITION; // skip the current position
					ppxlcRefMB += m_iFrameWidthY;
					ppxlcTmpC += MB_SIZE;
				}
#ifdef MS_BACKWARD_COMPATIBLE
if ((mbDiff < iMinSAD) || (abs (x - iXMB) + abs (y - iYMB) <= abs (iXDest - iXMB) + abs (iYDest - iYMB) && x > iXDest && y > iYDest)) {
#endif // MS_BACKWARD_COMPATIBLE
				iMinSAD = mbDiff;
				iXDest = x;
				iYDest = y;
#ifdef MS_BACKWARD_COMPATIBLE
}
#endif // MS_BACKWARD_COMPATIBLE
			}
NEXT_POSITION:	
			if (uiPosInLoop < (iLoop << 1)) {
				ppxlcRefMBY--;
				x--;
			}
			else if (uiPosInLoop < (iLoop << 2)) {
				ppxlcRefMBY -= m_iFrameWidthY;
				y--;
			}
			else if (uiPosInLoop < (iLoop * 6)) {
				ppxlcRefMBY++;					  
				x++;
			}
			else {
				ppxlcRefMBY += m_iFrameWidthY;
				y++;
			}
		}
	}
	*pmv = CMotionVector (iXDest - iXCurr, iYDest - iYCurr);
	
	// half pel search
	I8 destHalfX = 0, destHalfY = 0;
	Int lowX = (iXDest == m_rctRefVOPY0.left) ? 0 : -1;
	Int lowY = (iYDest == m_rctRefVOPY0.top) ? 0 : -1;
	Int highX = (iXDest == m_rctRefVOPY0.right - MB_SIZE + 1) ? 0 : 1;
	Int highY = (iYDest == m_rctRefVOPY0.bottom - MB_SIZE + 1) ? 0 : 1;
	const PixelC* ppxlcRefZoom = m_puciRefQZoom0->pixels ((iXDest << 1) + lowX, (iYDest << 1) + lowY);
	for (Int incY = lowY; incY <= highY; incY++) {
		for (Int incX = lowX; incX <= highX; incX++) {
			if (!(incX == 0 && incY == 0)) {
				const PixelC* ppxlcRefZoomMB = ppxlcRefZoom;
				const PixelC* ppxlcTmpC = m_ppxlcCurrMBY;
				mbDiff = 0;
				for (iy = 0; iy < MB_SIZE; iy++) {
					for (ix = 0; ix < MB_SIZE; ix++)
						mbDiff += abs (ppxlcTmpC [ix] - ppxlcRefZoomMB [2 * ix]);
					if (mbDiff >= iMinSAD)
						goto NEXT_HALF_POSITION;
					ppxlcRefZoomMB += m_iFrameWidthZoomY * 2;
					ppxlcTmpC += MB_SIZE;
				}
				iMinSAD = mbDiff;
				destHalfX = incX;
				destHalfY = incY;
			}
NEXT_HALF_POSITION:
			ppxlcRefZoom++;
		}
		ppxlcRefZoom += m_iFrameWidthZoomY - 3;
	}
	pmv -> iHalfX = destHalfX;
	pmv -> iHalfY = destHalfY;
	return iMinSAD;
}

Int CVideoObjectEncoder::blkmatch16WithShape (
	CMotionVector* pmv, 
	CoordI iXRef, CoordI iYRef,
	CoordI iXCurr, CoordI iYCurr,
	Int iMinSAD,
	const PixelC* ppxlcRefMBY,
	Int iSearchRange
)
{
	Int mbDiff; // start with big
	Int iXDest = iXRef, iYDest = iYRef;
	CoordI x = iXRef, y = iYRef, ix, iy;
	Int uiPosInLoop, iLoop;
	// Spiral Search for the rest
	for (iLoop = 1; iLoop <= iSearchRange; iLoop++) {
		x++;
		y++;
		ppxlcRefMBY += m_iFrameWidthY + 1;
		for (uiPosInLoop = 0; uiPosInLoop < (iLoop << 3); uiPosInLoop++) { // inside each spiral loop 
			if (
				x >= m_rctRefVOPY0.left && y >= m_rctRefVOPY0.top && 
				x <= m_rctRefVOPY0.right - MB_SIZE && y <= m_rctRefVOPY0.bottom - MB_SIZE
			) {
				const PixelC* ppxlcTmpC = m_ppxlcCurrMBY;
				const PixelC* ppxlcTmpCBY = m_ppxlcCurrMBBY;
				const PixelC* ppxlcRefMB = ppxlcRefMBY;
				mbDiff = 0;
				for (iy = 0; iy < MB_SIZE; iy++) {
					for (ix = 0; ix < MB_SIZE; ix++) {
						if (ppxlcTmpCBY [ix] != transpValue)
							mbDiff += abs (ppxlcTmpC [ix] - ppxlcRefMB [ix]);
					}
#ifndef MS_BACKWARD_COMPATIBLE
					if (mbDiff >= iMinSAD)
#else
					if (mbDiff > iMinSAD)
#endif
						goto NEXT_POSITION; // skip the current position
					ppxlcRefMB += m_iFrameWidthY;
					ppxlcTmpC += MB_SIZE;
					ppxlcTmpCBY += MB_SIZE;
				}
#ifdef MS_BACKWARD_COMPATIBLE
if ((mbDiff < iMinSAD) || (abs (x - iXMB) + abs (y - iYMB) <= abs (iXDest - iXMB) + abs (iYDest - iYMB) && x > iXDest && y > iYDest)) {
#endif // MS_BACKWARD_COMPATIBLE
				iMinSAD = mbDiff;
				iXDest = x;
				iYDest = y;
#ifdef MS_BACKWARD_COMPATIBLE
}
#endif // MS_BACKWARD_COMPATIBLE
			}
NEXT_POSITION:	
			if (uiPosInLoop < (iLoop << 1)) {
				ppxlcRefMBY--;
				x--;
			}
			else if (uiPosInLoop < (iLoop << 2)) {
				ppxlcRefMBY -= m_iFrameWidthY;
				y--;
			}
			else if (uiPosInLoop < (iLoop * 6)) {
				ppxlcRefMBY++;					  
				x++;
			}
			else {
				ppxlcRefMBY += m_iFrameWidthY;
				y++;
			}
		}
	}
	*pmv = CMotionVector (iXDest - iXCurr, iYDest - iYCurr);

	// check for case when zero MV due to MB outside the extended BB.
	if(iXDest==iXCurr && iYDest==iYCurr
		&& (iXDest < m_rctRefVOPY0.left || iYDest < m_rctRefVOPY0.top
			|| iXDest > m_rctRefVOPY0.right - MB_SIZE
			|| iYDest > m_rctRefVOPY0.bottom - MB_SIZE))
	{
		pmv -> iHalfX = 0;
		pmv -> iHalfY = 0;
		return iMinSAD;  
	}

	// half pel search
	I8 destHalfX = 0, destHalfY = 0;
	Int lowY, highY, lowX, highX;
	lowX = (iXDest == m_rctRefVOPY0.left) ? 0 : -1;
	lowY = (iYDest == m_rctRefVOPY0.top) ? 0 : -1;
	highX = (iXDest == m_rctRefVOPY0.right - MB_SIZE + 1) ? 0 : 1;
	highY = (iYDest == m_rctRefVOPY0.bottom - MB_SIZE + 1) ? 0 : 1;
	const PixelC* ppxlcRefZoom = m_puciRefQZoom0->pixels ((iXDest << 1) + lowX, (iYDest << 1) + lowY);
	for (Int incY = lowY; incY <= highY; incY++) {
		for (Int incX = lowX; incX <= highX; incX++) {
			if (!(incX == 0 && incY == 0)) {
				const PixelC* ppxlcRefZoomMB = ppxlcRefZoom;
				const PixelC* ppxlcTmpC = m_ppxlcCurrMBY;
				const PixelC* ppxlcTmpCBY = m_ppxlcCurrMBBY;
				mbDiff = 0;
				for (iy = 0; iy < MB_SIZE; iy++) {
					for (ix = 0; ix < MB_SIZE; ix++) {
						if (ppxlcTmpCBY [ix] != transpValue) 
							mbDiff += abs (ppxlcTmpC [ix] - ppxlcRefZoomMB [2 * ix]);
					}
					if (mbDiff >= iMinSAD)
						goto NEXT_HALF_POSITION;
					ppxlcRefZoomMB += m_iFrameWidthZoomY * 2;
					ppxlcTmpC += MB_SIZE;
					ppxlcTmpCBY += MB_SIZE;
				}
				iMinSAD = mbDiff;
				destHalfX = incX;
				destHalfY = incY;
			}
NEXT_HALF_POSITION:
			ppxlcRefZoom++;
		}
		ppxlcRefZoom += m_iFrameWidthZoomY - 3;
	}
	pmv -> iHalfX = destHalfX;
	pmv -> iHalfY = destHalfY;
	return iMinSAD;
}

Int CVideoObjectEncoder::blockmatch8 (
	const PixelC* ppxlcCodedBlkY, 
	CMotionVector* pmv8, 
	CoordI iXBlk, CoordI iYBlk,
	const CMotionVector* pmvPred,
	Int iSearchRange
)
{
	Int mbDiff; // start with big
	CoordI x = iXBlk + pmvPred->iMVX, y = iYBlk + pmvPred->iMVY, ix, iy;
	CoordI left = -1 * min (abs (pmvPred->iMVX + iSearchRange), ADD_DISP);
	CoordI top = -1 * min (abs (pmvPred->iMVY + iSearchRange), ADD_DISP);
	CoordI right = min (abs (pmvPred->iMVX - iSearchRange), ADD_DISP);
	CoordI bottom = min (abs (pmvPred->iMVY - iSearchRange), ADD_DISP);

	CoordI iOrigX = x, iOrigY = y;
	CoordI iXDest = x, iYDest = y;
	Int uiPosInLoop, iLoop;
	const PixelC* ppxlcRefBlk = (m_volmd.bOriginalForME ? m_pvopcRefOrig0 : m_pvopcRefQ0)->getPlane (Y_PLANE)->pixels (x, y);
	Int iMinSAD = sad8x8At0 (ppxlcCodedBlkY, ppxlcRefBlk);
	// Spiral Search for the rest
	Int iix = 0, iiy = 0;
	for (iLoop = 1; iLoop <= ADD_DISP; iLoop++) {
		x++;
		y++;
		iix++; iiy++;
		ppxlcRefBlk += m_iFrameWidthY + 1;
		for (uiPosInLoop = 0; uiPosInLoop < (iLoop << 3); uiPosInLoop++) { // inside each spiral loop 
			if (
				iix >= left && iix <= right &&
				iiy >= top && iiy <= bottom &&
				x >= m_rctRefVOPY0.left && y >= m_rctRefVOPY0.top && 
				x <= m_rctRefVOPY0.right - MB_SIZE && y <= m_rctRefVOPY0.bottom - MB_SIZE
			) {
				const PixelC* ppxlcTmpC = ppxlcCodedBlkY;
				const PixelC* ppxlcRefMB = ppxlcRefBlk;
				mbDiff = 0;
				for (iy = 0; iy < BLOCK_SIZE; iy++) {
					for (ix = 0; ix < BLOCK_SIZE; ix++)
						mbDiff += abs (ppxlcTmpC [ix] - ppxlcRefMB [ix]);
//					if (mbDiff >= iMinSAD)
					if (mbDiff > iMinSAD)
						goto NEXT_POSITION; // skip the current position
					ppxlcRefMB += m_iFrameWidthY;
					ppxlcTmpC += MB_SIZE;
				}
#ifdef MS_BACKWARD_COMPATIBLE
if ((mbDiff < iMinSAD) || abs (x - iOrigX) + abs (y - iOrigY) <= abs (iXDest - iOrigX) + abs (iYDest - iOrigY)) {
#endif // MS_BACKWARD_COMPATIBLE
				iMinSAD = mbDiff;
				iXDest = x;
				iYDest = y;
#ifdef MS_BACKWARD_COMPATIBLE
}
#endif // MS_BACKWARD_COMPATIBLE
			}
NEXT_POSITION:	
			if (uiPosInLoop < (iLoop << 1)) {
				ppxlcRefBlk--;
				x--;
				iix--;
			}
			else if (uiPosInLoop < (iLoop << 2)) {
				ppxlcRefBlk -= m_iFrameWidthY;
				y--;
				iiy--;
			}
			else if (uiPosInLoop < (iLoop * 6)) {
				ppxlcRefBlk++;					  
				x++;
				iix++;
			}
			else {
				ppxlcRefBlk += m_iFrameWidthY;
				y++;
				iiy++;
			}
		}
	}
	*pmv8 = CMotionVector (iXDest - iXBlk, iYDest - iYBlk);
	
	// half pel search
#ifdef MS_BACKWARD_COMPATIBLE
iMinSAD = 10000000;
#endif // MS_BACKWARD_COMPATIBLE
	I8 destHalfX = 0, destHalfY = 0;
	Int lowY, highY, lowX, highX;
	lowX = (iXDest == m_rctRefVOPY0.left) ? 0 : -1;
	lowY = (iYDest == m_rctRefVOPY0.top) ? 0 : -1;
	highX = (iXDest == m_rctRefVOPY0.right - MB_SIZE + 1) ? 0 : 1;
	highY = (iYDest == m_rctRefVOPY0.bottom - MB_SIZE + 1) ? 0 : 1;
	const PixelC* ppxlcRefZoom = m_puciRefQZoom0->pixels ((iXDest << 1) + lowX, (iYDest << 1) + lowY);
	for (Int incY = lowY; incY <= highY; incY++) {
		for (Int incX = lowX; incX <= highX; incX++) {
#ifndef MS_BACKWARD_COMPATIBLE
			if (!(incX == 0 && incY == 0)) {
#endif // MS_BACKWARD_COMPATIBLE
				const PixelC* ppxlcRefZoomBlk = ppxlcRefZoom;
				const PixelC* ppxlcTmpC = ppxlcCodedBlkY;
				mbDiff = 0;
				for (iy = 0; iy < BLOCK_SIZE; iy++) {
					for (ix = 0; ix < BLOCK_SIZE; ix++)
						mbDiff += abs (ppxlcTmpC [ix] - ppxlcRefZoomBlk [2 * ix]);
#ifndef MS_BACKWARD_COMPATIBLE
					if (mbDiff >= iMinSAD)
						goto NEXT_HALF_POSITION;
#endif // MS_BACKWARD_COMPATIBLE
					ppxlcRefZoomBlk += m_iFrameWidthZoomY * 2;
					ppxlcTmpC += MB_SIZE;
				}
#ifdef MS_BACKWARD_COMPATIBLE
if (mbDiff < iMinSAD) {
#endif // MS_BACKWARD_COMPATIBLE
				iMinSAD = mbDiff;
				destHalfX = incX;
				destHalfY = incY;
#ifdef MS_BACKWARD_COMPATIBLE
}
#else
			}
#endif MS_BACKWARD_COMPATIBLE
NEXT_HALF_POSITION:
			ppxlcRefZoom++;
		}
		ppxlcRefZoom += m_iFrameWidthZoomY - 3;
	}
	pmv8 -> iHalfX = destHalfX;
	pmv8 -> iHalfY = destHalfY;
	return iMinSAD;
}

Int CVideoObjectEncoder::blockmatch8WithShape (
	const PixelC* ppxlcCodedBlkY, 
	const PixelC* ppxlcCodedBlkBY, 
	CMotionVector* pmv8, 
	CoordI iXBlk, CoordI iYBlk,
	const CMotionVector* pmvPred,
	Int iSearchRange
)
{
	Int mbDiff; // start with big
	CoordI x = iXBlk + pmvPred->iMVX, y = iYBlk + pmvPred->iMVY, ix, iy;
	CoordI left = -1 * min (abs (pmvPred->iMVX + iSearchRange), ADD_DISP);
	CoordI top = -1 * min (abs (pmvPred->iMVY + iSearchRange), ADD_DISP);
	CoordI right = min (abs (pmvPred->iMVX - iSearchRange), ADD_DISP);
	CoordI bottom = min (abs (pmvPred->iMVY - iSearchRange), ADD_DISP);

	CoordI iOrigX = x, iOrigY = y;
	CoordI iXDest = x, iYDest = y;
	Int uiPosInLoop, iLoop;
	const PixelC* ppxlcRefBlk = (m_volmd.bOriginalForME ? m_pvopcRefOrig0 : m_pvopcRefQ0)->getPlane (Y_PLANE)->pixels (x, y);
	Int iMinSAD = sad8x8At0WithShape (ppxlcCodedBlkY, ppxlcCodedBlkBY, ppxlcRefBlk);
	// Spiral Search for the rest
	Int iix = 0, iiy = 0;	
	for (iLoop = 1; iLoop <= ADD_DISP; iLoop++) {
		x++;
		y++;
		iix++;
		iiy++;
		ppxlcRefBlk += m_iFrameWidthY + 1;
		for (uiPosInLoop = 0; uiPosInLoop < (iLoop << 3); uiPosInLoop++) { // inside each spiral loop 
			if (
				iix >= left && iix <= right &&
				iiy >= top && iiy <= bottom &&
				x >= m_rctRefVOPY0.left && y >= m_rctRefVOPY0.top && 
				x <= m_rctRefVOPY0.right - MB_SIZE && y <= m_rctRefVOPY0.bottom - MB_SIZE
			) {
				const PixelC* ppxlcTmpC = ppxlcCodedBlkY;
				const PixelC* ppxlcTmpCBY = ppxlcCodedBlkBY;
				const PixelC* ppxlcRefMB = ppxlcRefBlk;
				mbDiff = 0;
				for (iy = 0; iy < BLOCK_SIZE; iy++) {
					for (ix = 0; ix < BLOCK_SIZE; ix++) {
						if (ppxlcTmpCBY [ix] != transpValue)
							mbDiff += abs (ppxlcTmpC [ix] - ppxlcRefMB [ix]);
					}
//					if (mbDiff >= iMinSAD)
					if (mbDiff > iMinSAD)
						goto NEXT_POSITION; // skip the current position
					ppxlcRefMB += m_iFrameWidthY;
					ppxlcTmpC += MB_SIZE;
					ppxlcTmpCBY += MB_SIZE;
				}
#ifdef MS_BACKWARD_COMPATIBLE
if ((mbDiff < iMinSAD) || abs (x - iOrigX) + abs (y - iOrigY) <= abs (iXDest - iOrigX) + abs (iYDest - iOrigY)) {
#endif // MS_BACKWARD_COMPATIBLE
				iMinSAD = mbDiff;
				iXDest = x;
				iYDest = y;
#ifdef MS_BACKWARD_COMPATIBLE
}
#endif // MS_BACKWARD_COMPATIBLE
			}
NEXT_POSITION:	
			if (uiPosInLoop < (iLoop << 1)) {
				ppxlcRefBlk--;
				x--;
				iix--;
			}
			else if (uiPosInLoop < (iLoop << 2)) {
				ppxlcRefBlk -= m_iFrameWidthY;
				y--;
				iiy--;
			}
			else if (uiPosInLoop < (iLoop * 6)) {
				ppxlcRefBlk++;					  
				x++;
				iix++;
			}
			else {
				ppxlcRefBlk += m_iFrameWidthY;
				y++;
				iiy++;
			}
		}
	}
	*pmv8 = CMotionVector (iXDest - iXBlk, iYDest - iYBlk);
	
	// check for case when zero MV due to MB outside the extended BB.
	if(iXDest==iXBlk && iYDest==iYBlk
		&& (iXDest < m_rctRefVOPY0.left || iYDest < m_rctRefVOPY0.top
			|| iXDest > m_rctRefVOPY0.right - MB_SIZE
			|| iYDest > m_rctRefVOPY0.bottom - MB_SIZE))

	{
		pmv8 -> iHalfX = 0;
		pmv8 -> iHalfY = 0;
		return iMinSAD;  
	}

	// half pel search
#ifdef MS_BACKWARD_COMPATIBLE
iMinSAD = 10000000;
#endif // MS_BACKWARD_COMPATIBLE
	I8 destHalfX = 0, destHalfY = 0;
	Int lowY, highY, lowX, highX;
	lowX = (iXDest == m_rctRefVOPY0.left) ? 0 : -1;
	lowY = (iYDest == m_rctRefVOPY0.top) ? 0 : -1;
	highX = (iXDest == m_rctRefVOPY0.right - MB_SIZE + 1) ? 0 : 1;
	highY = (iYDest == m_rctRefVOPY0.bottom - MB_SIZE + 1) ? 0 : 1;
	const PixelC* ppxlcRefZoom = m_puciRefQZoom0->pixels ((iXDest << 1) + lowX, (iYDest << 1) + lowY);
	for (Int incY = lowY; incY <= highY; incY++) {
		const PixelC* ppxlcRefZoomRow = ppxlcRefZoom;
		for (Int incX = lowX; incX <= highX; incX++) {
#ifndef MS_BACKWARD_COMPATIBLE
			if (!(incX == 0 && incY == 0)) {
#endif // MS_BACKWARD_COMPATIBLE
				const PixelC* ppxlcRefZoomBlk = ppxlcRefZoomRow;
				const PixelC* ppxlcTmpC = ppxlcCodedBlkY;
				const PixelC* ppxlcTmpCBY = ppxlcCodedBlkBY;
				mbDiff = 0;
				for (iy = 0; iy < BLOCK_SIZE; iy++) {
					for (ix = 0; ix < BLOCK_SIZE; ix++) {
						if (ppxlcTmpCBY [ix] != transpValue)
							mbDiff += abs (ppxlcTmpC [ix] - ppxlcRefZoomBlk [2 * ix]);
					}
#ifndef MS_BACKWARD_COMPATIBLE
					if (mbDiff >= iMinSAD)
						goto NEXT_HALF_POSITION;
#endif // MS_BACKWARD_COMPATIBLE
					ppxlcRefZoomBlk += m_iFrameWidthZoomY * 2;
					ppxlcTmpC += MB_SIZE;
					ppxlcTmpCBY += MB_SIZE;
				}
#ifdef MS_BACKWARD_COMPATIBLE
if (mbDiff < iMinSAD) {
#endif // MS_BACKWARD_COMPATIBLE
				iMinSAD = mbDiff;
				destHalfX = incX;
				destHalfY = incY;
#ifdef MS_BACKWARD_COMPATIBLE
}
#else
			}
#endif
NEXT_HALF_POSITION:
			ppxlcRefZoomRow++;
		}
		ppxlcRefZoom += m_iFrameWidthZoomY;
	}
	pmv8 -> iHalfX = destHalfX;
	pmv8 -> iHalfY = destHalfY;
	return iMinSAD;
}

Int CVideoObjectEncoder::sumDev () const // compute sum of deviation of an MB
{
	const PixelC* ppxlcY = m_ppxlcCurrMBY;
	Int iMean = 0;
	UInt iy;
	for (iy = 0; iy < MB_SQUARE_SIZE; iy++)
		iMean += ppxlcY [iy];
	iMean /= MB_SQUARE_SIZE;
	Int iDevRet = 0;
	for (iy = 0; iy < MB_SQUARE_SIZE; iy++)
		iDevRet += abs (m_ppxlcCurrMBY [iy] - iMean);
	return iDevRet;
}


Int CVideoObjectEncoder::sumDevWithShape (UInt uiNumTranspPels) const // compute sum of deviation of an MB
{
	const PixelC* ppxlcY = m_ppxlcCurrMBY;
	const PixelC* ppxlcBY = m_ppxlcCurrMBBY;
	Int iMean = 0;
	CoordI iy;
	for (iy = 0; iy < MB_SQUARE_SIZE; iy++) {
		if (ppxlcBY [iy] != transpValue)
			iMean += ppxlcY [iy];
	}
	iMean /= uiNumTranspPels;
	Int iDevRet = 0;
	for (iy = 0; iy < MB_SQUARE_SIZE; iy++) {
		if (m_ppxlcCurrMBBY [iy] != transpValue)
			iDevRet += abs (m_ppxlcCurrMBY [iy] - iMean);
	}
	return iDevRet;
}

Int CVideoObjectEncoder::sad16x16At0 (const PixelC* ppxlcRefY) const
{
	Int iInitSAD = 0;
	CoordI ix, iy;
	const PixelC* ppxlcCurrY = m_ppxlcCurrMBY;
	for (iy = 0; iy < MB_SIZE; iy++) {
		for (ix = 0; ix < MB_SIZE; ix++)
			iInitSAD += abs (ppxlcCurrY [ix] - ppxlcRefY [ix]);
		ppxlcCurrY += MB_SIZE;
		ppxlcRefY += m_iFrameWidthY;
	}
	iInitSAD -= FAVORZERO;
	return iInitSAD;
}

Int CVideoObjectEncoder::sad16x16At0WithShape (const PixelC* ppxlcRefY, const CMBMode* pmbmd) const
{
	Int iInitSAD = 0;
	CoordI ix, iy;
	const PixelC* ppxlcCurrY = m_ppxlcCurrMBY;
	const PixelC* ppxlcCurrBY = m_ppxlcCurrMBBY;
	for (iy = 0; iy < MB_SIZE; iy++) {
		for (ix = 0; ix < MB_SIZE; ix++) {
			if (ppxlcCurrBY [ix] != transpValue)
				iInitSAD += abs (ppxlcCurrY [ix] - ppxlcRefY [ix]);
		}
		ppxlcCurrBY += MB_SIZE;
		ppxlcCurrY += MB_SIZE;
		ppxlcRefY += m_iFrameWidthY;
	}
	iInitSAD -= (pmbmd->m_rgNumNonTranspPixels [0] >> 1) + 1;
	return iInitSAD;
}

Int CVideoObjectEncoder::sad8x8At0 (const PixelC* ppxlcCurrY, const PixelC* ppxlcRefY) const
{
	Int iInitSAD = 0;
	CoordI ix, iy;
	for (iy = 0; iy < BLOCK_SIZE; iy++) {
		for (ix = 0; ix < BLOCK_SIZE; ix++)
			iInitSAD += abs (ppxlcCurrY [ix] - ppxlcRefY [ix]);
		ppxlcCurrY += MB_SIZE;
		ppxlcRefY += m_iFrameWidthY;
	}
	return iInitSAD;
}

Int CVideoObjectEncoder::sad8x8At0WithShape (const PixelC* ppxlcCurrY, const PixelC* ppxlcCurrBY, const PixelC* ppxlcRefY) const
{
	Int iInitSAD = 0;
	CoordI ix, iy;
	for (iy = 0; iy < BLOCK_SIZE; iy++) {
		for (ix = 0; ix < BLOCK_SIZE; ix++) {
			if (ppxlcCurrBY [ix] != transpValue)
				iInitSAD += abs (ppxlcCurrY [ix] - ppxlcRefY [ix]);
		}
		ppxlcCurrBY += MB_SIZE;
		ppxlcCurrY += MB_SIZE;
		ppxlcRefY += m_iFrameWidthY;
	}
	return iInitSAD;
}
