#include <stdio.h>
#include <fstream.h>
#include <math.h>
#include <stdlib.h>

#include "typeapi.h"
#include "codehead.h"
#include "global.h"
#include "entropy/bitstrm.h"
#include "entropy/entropy.h"
#include "entropy/huffman.h"
#include "mode.h"
#include "vopses.h"
#include "vopseenc.h"

#ifdef __MFC_
#ifdef _DEBUG
#undef THIS_FILE
static char BASED_CODE THIS_FILE[] = __FILE__;
#endif

#define new DEBUG_NEW				   
#endif // __MFC_

UInt CVideoObjectEncoder::encodeMV (const CMotionVector* pmv, Bool bHas4MV, Bool bLeftBndry, Bool bRightBndry, Bool bTopBndry)
{
	UInt nBits = 0;
	CVector vctDiff;
	CVector vctPred;

	if (bHas4MV)	{	//4mv
		if (bLeftBndry || bRightBndry || bTopBndry)	{
			for (UInt iBlk = Y_BLOCK1; iBlk <= Y_BLOCK4; iBlk++)	{
				find8x8MVpredAtBoundary (vctPred, pmv, bLeftBndry, bRightBndry, bTopBndry, (BlockNum) iBlk);
				vctDiff = (pmv + iBlk)->trueMVHalfPel () - vctPred;
#ifdef __TRACE_AND_STATS_
				m_pbitstrmOut->trace ((pmv + iBlk)->trueMVHalfPel (), "MB_MV_Curr");
				m_pbitstrmOut->trace (vctPred, "MB_MV_Pred");
				m_pbitstrmOut->trace (vctDiff, "MB_MV_Diff");
#endif // __TRACE_AND_STATS_
				nBits += sendDiffMV (vctDiff);
			}
		}
		else {
			for (UInt iBlk = Y_BLOCK1; iBlk <= Y_BLOCK4; iBlk++)	{
				find8x8MVpredInterior (vctPred, pmv, (BlockNum) iBlk);
				vctDiff = (pmv + iBlk) -> trueMVHalfPel () - vctPred;
#ifdef __TRACE_AND_STATS_
				m_pbitstrmOut->trace ((pmv + iBlk)->trueMVHalfPel (), "MB_MV_Curr");
				m_pbitstrmOut->trace (vctPred, "MB_MV_Pred");
				m_pbitstrmOut->trace (vctDiff, "MB_MV_Diff");
#endif // __TRACE_AND_STATS_
				nBits += sendDiffMV (vctDiff);
			}
		}
	}
	else {		//1mv
		find16x16MVpred (vctPred, pmv, bLeftBndry, bRightBndry, bTopBndry);
		vctDiff = pmv->trueMVHalfPel () - vctPred;
#ifdef __TRACE_AND_STATS_
		m_pbitstrmOut->trace (pmv->trueMVHalfPel (), "MB_MV_Curr");
		m_pbitstrmOut->trace (vctPred, "MB_MV_Pred");
		m_pbitstrmOut->trace (vctDiff, "MB_MV_Diff");
#endif // __TRACE_AND_STATS_
		nBits += sendDiffMV (vctDiff);
	}
	return nBits;
}

UInt CVideoObjectEncoder::encodeMVWithShape (const CMotionVector* pmv, const CMBMode* pmbmd, Int iXMB, Int iYMB)
{
	UInt nBits = 0;
	CVector vctDiff;
	CVector vctPred;

	if (pmbmd->m_bhas4MVForward)	{	//4mv
		for (UInt iBlk = Y_BLOCK1; iBlk <= Y_BLOCK4; iBlk++)	{
			if (pmbmd->m_rgTranspStatus[iBlk] != ALL)	{
				findMVpredGeneric (vctPred, pmv, pmbmd, iBlk, iXMB, iYMB);
				vctDiff = (pmv + iBlk)->trueMVHalfPel () - vctPred;
				assert (vctDiff.x >= -256 && vctDiff.x <= 256);
				assert (vctDiff.y >= -256 && vctDiff.y <= 256);
				nBits += sendDiffMV (vctDiff);
			}
		}
	}
	else {		//1mv
		findMVpredGeneric (vctPred, pmv, pmbmd, ALL_Y_BLOCKS, iXMB, iYMB);
		vctDiff = pmv->trueMVHalfPel () - vctPred;
		assert (vctDiff.x >= -256 && vctDiff.x <= 256);
		assert (vctDiff.y >= -256 && vctDiff.y <= 256);
		nBits += sendDiffMV (vctDiff);
	}
	return nBits;
}

Void CVideoObjectEncoder::scaleMV (Int& iVLC, UInt& uiResidual, Int iDiffMVcomponent)
{
	assert (iDiffMVcomponent >= (-3 * (Int) m_vopmd.mvInfoForward.uiRange));
	assert (iDiffMVcomponent < 3 * (Int) m_vopmd.mvInfoForward.uiRange);
	if (iDiffMVcomponent < (-1 * (Int) m_vopmd.mvInfoForward.uiRange))
		iDiffMVcomponent += 2 * m_vopmd.mvInfoForward.uiRange;
	else if (iDiffMVcomponent > (Int) (m_vopmd.mvInfoForward.uiRange - 1))
		iDiffMVcomponent -= 2 * m_vopmd.mvInfoForward.uiRange;

	if (iDiffMVcomponent == 0)	{									//nothing to do
		iVLC = 0;
		uiResidual = 0;
	}
	else	{
		if (m_vopmd.mvInfoForward.uiScaleFactor == 1)	{									//simple no-scale case
			iVLC = iDiffMVcomponent;
			uiResidual = 0;
		}
		else {														//stupid scaling
			UInt uiAbsDiffMVcomponent = abs (iDiffMVcomponent);
			uiResidual = (uiAbsDiffMVcomponent - 1) % m_vopmd.mvInfoForward.uiScaleFactor;																	
			UInt uiVLCmagnitude = (uiAbsDiffMVcomponent 
								- uiResidual + (m_vopmd.mvInfoForward.uiScaleFactor - 1)) 
								/ m_vopmd.mvInfoForward.uiScaleFactor;	// absorb ++ into here (m_volmd.mvInfo.uiScaleFactor - 1)
			iVLC = uiVLCmagnitude * sign (iDiffMVcomponent);
		}
	}
}

UInt CVideoObjectEncoder::sendDiffMV (const CVector& vctDiffMVHalfPel)
{
	Int iVLC;
	UInt uiResidual;
	UInt nBits = 0;

	// send the bits
	scaleMV (iVLC, uiResidual, vctDiffMVHalfPel.x);

	Long lsymbol = iVLC + 32;
	nBits += m_pentrencSet -> m_pentrencMV->encodeSymbol (lsymbol, "MB_MV_Magnitude");
	if (iVLC != 0) {
		m_pbitstrmOut->putBits ((Char) uiResidual, m_vopmd.mvInfoForward.uiFCode - 1, "MB_MV_Residual");
		nBits += m_vopmd.mvInfoForward.uiFCode - 1;
	}
	else
		assert (uiResidual == 0);

	scaleMV (iVLC, uiResidual, vctDiffMVHalfPel.y);
	lsymbol = iVLC + 32;
	nBits += m_pentrencSet -> m_pentrencMV->encodeSymbol (lsymbol, "MB_MV_Magnitude");
	if (iVLC != 0) {
		m_pbitstrmOut->putBits ((Char) uiResidual, m_vopmd.mvInfoForward.uiFCode - 1, "MB_MV_Residual");
		nBits += m_vopmd.mvInfoForward.uiFCode - 1;
	}
	else
		assert (uiResidual == 0);
	return nBits;
}

UInt CVideoObjectEncoder::encodeMVofBVOP (const CMotionVector* pmvForward, 
										  const CMotionVector* pmvBackward, 
										  const CMBMode*	   pmbmd, 
										  Int	iMBX,	Int iMBY,
										  const CMotionVector* pmvRef, 
										  const CMBMode*	   pmbmdRef
										  )  // encode motion vectors for b-vop
{
	assert (pmbmd->m_rgTranspStatus [0] != ALL);
	assert (pmbmd->m_dctMd == INTER || pmbmd->m_dctMd == INTERQ);
	assert (pmbmd->m_bSkip == FALSE);

	CVector vctDiff;
	CVector vctPred;
	UInt nBits = 0;
	if (pmbmd->m_mbType == FORWARD || pmbmd->m_mbType == INTERPOLATE)	{
		assert (pmbmd->m_bhas4MVForward != TRUE);
//		findMVpredictorOfBVOP (vctPred, pmvForward, pmbmd, iMBX);
		vctPred = m_vctForwardMvPredBVOP;
		vctDiff = pmvForward->m_vctTrueHalfPel - vctPred;
		assert (vctDiff.x < 256 && vctDiff.x > -256 && vctDiff.y < 256 && vctDiff.y > -256);
#ifdef __TRACE_AND_STATS_
		m_pbitstrmOut->trace (*pmvForward, "MB_F_MV_Curr");
		m_pbitstrmOut->trace (vctPred, "MB_F_MV_Pred");
		m_pbitstrmOut->trace (vctDiff, "MB_F_MV_Diff");
#endif // __TRACE_AND_STATS_
		nBits += sendDiffMV (vctDiff);
		m_vctForwardMvPredBVOP = pmvForward->m_vctTrueHalfPel;
	}
	if ((pmbmd->m_mbType == BACKWARD || pmbmd->m_mbType == INTERPOLATE) && m_volmd.volType != ENHN_LAYER)	{		
		assert (pmbmd->m_bhas4MVBackward != TRUE);
		vctPred = m_vctBackwardMvPredBVOP;
//		findMVpredictorOfBVOP (vctPred, pmvBackward, pmbmd, iMBX);
		vctDiff = pmvBackward->m_vctTrueHalfPel - vctPred;
		assert (vctDiff.x < 256 && vctDiff.x > -256 && vctDiff.y < 256 && vctDiff.y > -256);
#ifdef __TRACE_AND_STATS_
		m_pbitstrmOut->trace (*pmvBackward, "MB_B_MV_Curr");
		m_pbitstrmOut->trace (vctPred, "MB_B_MV_Pred");
		m_pbitstrmOut->trace (vctDiff, "MB_B_MV_Diff");
#endif // __TRACE_AND_STATS_
		nBits += sendDiffMV (vctDiff);
		m_vctBackwardMvPredBVOP = pmvBackward->m_vctTrueHalfPel;
	}		
	if (pmbmd->m_mbType == DIRECT)	{					
		assert (pmbmd->m_bhas4MVForward != TRUE);
		//computeDirectDeltaMV (vctDiff, pmvForward, pmvRef, pmbmdRef);
#ifdef __TRACE_AND_STATS_
		m_pbitstrmOut->trace (m_vctDirectDeltaMV, "MB_D_MV_Diff");
#endif // __TRACE_AND_STATS_
//		nBits += sendDiffMV (m_vctDirectDeltaMV);
		Long lsymbol = m_vctDirectDeltaMV.x + 32;
		nBits += m_pentrencSet -> m_pentrencMV->encodeSymbol (lsymbol, "MB_MVDB_Magnitude");
		lsymbol = m_vctDirectDeltaMV.y + 32;
		nBits += m_pentrencSet -> m_pentrencMV->encodeSymbol (lsymbol, "MB_MVDB_Magnitude");
	}
	assert (nBits <= 224);										//interpolate mode:longest VLC + residue (4MVs)
	return nBits;
}
