/*************************************************************************

This software module was originally developed by 

	Ming-Chieh Lee (mingcl@microsoft.com), Microsoft Corporation
	Wei-ge Chen (wchen@microsoft.com), Microsoft Corporation
	Bruce Lin (blin@microsoft.com), Microsoft Corporation
	Chuang Gu (chuanggu@microsoft.com), Microsoft Corporation
	(date: March, 1996)

and edited by  
 
        Wei Wu (weiwu@stallion.risc.rockwell.com), Rockwell Science Center 
        Homer Chen (homer@jupiter.risc.rockwell.com), Rockwell Science Center

in the course of development of the MPEG-4 Video (ISO/IEC 14496-2). 
This software module is an implementation of a part of one or more MPEG-4 Video tools 
as specified by the MPEG-4 Video. 
ISO/IEC gives users of the MPEG-4 Video free license to this software module or modifications 
thereof for use in hardware or software products claiming conformance to the MPEG-4 Video. 
Those intending to use this software module in hardware or software products are advised that its use may infringe existing patents. 
The original developer of this software module and his/her company, 
the subsequent editors and their companies, 
and ISO/IEC have no liability for use of this software module or modifications thereof in an implementation. 
Copyright is not released for non MPEG-4 Video conforming products. 
Microsoft retains full right to use the code for his/her own purpose, 
assign or donate the code to a third party and to inhibit third parties from using the code for non <MPEG standard> conforming products. 
This copyright notice must be included in all copies or derivative works. 

Copyright (c) 1996, 1997.


Module Name:

	sesEnc.cpp

Abstract:

	Encoder for one video session.

Revision History:
	Sept. 30, 1997: Error resilient tools added by Toshiba

*************************************************************************/

#include "stdlib.h"
#include "stdio.h"

#include "fstream.h"
#include "iostream.h"

#include "typeapi.h"
#include "codehead.h"
#include "global.h"
#include "entropy/bitstrm.h"
#include "entropy/entropy.h"
#include "entropy/huffman.h"

#include "mode.h"
#include "sesenc.h"

#include "vd_grid.h"	// VDS:
#include "vd_vector.h"	// VDS:

#include "vopses.h"
#include "vopseenc.h"

#ifdef __MFC_
#ifdef _DEBUG
#undef THIS_FILE
static char BASED_CODE THIS_FILE[] = __FILE__;
#endif

#define new DEBUG_NEW				   
#endif // __MFC_

#ifdef __PC_COMPILER_					//OS specific stuff
#define SUB_CMPFILE "%s\\%2.2d\\%s.cmp"
#define SUB_TRCFILE "%s\\%2.2d\\%s.trc"
#define SUB_YUVFILE	"%s\\%2.2d\\%s.yuv"
#define SUB_SEGFILE	"%s\\%2.2d\\%s.seg"
#define ENHN_SUB_CMPFILE "%s\\%2.2d\\%s_e.cmp"
#define ENHN_SUB_TRCFILE "%s\\%2.2d\\%s_e.trc"
#define ENHN_SUB_YUVFILE "%s\\%2.2d\\%s_e.yuv"
#define ENHN_SUB_SEGFILE "%s\\%2.2d\\%s_e.seg"
#define SUB_VDLFILE "%s\\%2.2d\\%s.spt"	//for sprite i/o
#define SUB_PNTFILE "%s\\%2.2d\\%s.pnt"	//for sprite i/o
#define ROOT_YUVFILE "%s\\%s.yuv"
#define ROOT_SEGFILE "%s\\%s.seg"
#define IOS_BINARY ios::binary
#define MKDIR "mkdir %s\\%2.2d"
#else
#define SUB_CMPFILE "%s/%2.2d/%s.cmp"
#define SUB_TRCFILE "%s/%2.2d/%s.trc"
#define SUB_YUVFILE	"%s/%2.2d/%s.yuv"
#define SUB_SEGFILE	"%s/%2.2d/%s.seg"
#define ENHN_SUB_CMPFILE "%s/%2.2d/%s_e.cmp"
#define ENHN_SUB_TRCFILE "%s/%2.2d/%s_e.trc"
#define ENHN_SUB_YUVFILE "%s/%2.2d/%s_e.yuv"
#define ENHN_SUB_SEGFILE "%s/%2.2d/%s_e.seg"
#define SUB_VDLFILE "%s/%2.2d/%s.vdl"	//for sprite i/o
#define SUB_PNTFILE "%s/%2.2d/%s.pnt"	//for sprite i/o
#define ROOT_YUVFILE "%s/%s.yuv"
#define ROOT_SEGFILE "%s/%s.seg"
#define IOS_BINARY ios::bin
#define MKDIR "mkdir %s/%2.2d"
#endif

#define BASE_LAYER 0
#define ENHN_LAYER 1

Bool fileExistence (const Char* pchFile) 
{
	Bool ret = 1;
	FILE* pf = fopen (pchFile, "r");
	if (pf == NULL)
		ret = 0;
	else
		fclose (pf);
	return ret;
}

Void CSessionEncoder::createReconDir (UInt idx) const
{
	Char pchTmp [100];
	sprintf (pchTmp, SUB_YUVFILE, m_pchReconYUVDir, idx, m_pchPrefix);
	FILE* pf = fopen (pchTmp, "wb");
	if (pf == NULL) {
		sprintf (pchTmp, MKDIR, m_pchReconYUVDir, idx);
		system (pchTmp);																	
	}
	else
		fclose (pf);
}

Void CSessionEncoder::createCmpDir (UInt idx) const
{
	Char pchTmp [100];
	sprintf (pchTmp, SUB_CMPFILE, m_pchOutStrFiles, idx, m_pchPrefix);
	FILE* pf = fopen (pchTmp, "wb");
	if (pf == NULL) {
		sprintf (pchTmp, MKDIR, m_pchOutStrFiles, idx);
		system (pchTmp);																	
	}
	else
		fclose (pf);
}

CSessionEncoder::~CSessionEncoder ()
{
	delete [] m_rgvolmd [BASE_LAYER];
	delete [] m_rgvolmd [ENHN_LAYER];
	delete [] m_rgvopmd [BASE_LAYER];
	delete [] m_rgvopmd [ENHN_LAYER];

	// sprite 
	for (Int iVO = 0; iVO < m_iNumVO; iVO++) {
		delete [] m_ppstSrc [iVO];
		for (Int ifr = 0; ifr < m_iNumFrame; ifr++)
			delete [] m_pppstDst [iVO] [ifr];
		delete [] m_pppstDst [iVO];
	}
	delete [] m_ppstSrc;
	delete [] m_pppstDst;
}


CSessionEncoder::CSessionEncoder (
		// general info
		Bool bvds, // VDS: view-dependent scalability enable
		UInt uiFrmWidth, // frame width
		UInt uiFrmHeight, // frame height
		Int iFirstFrm, // first frame number
		Int iLastFrm, // last frame number
		Int iFirstVO, // first VOP index
		Int iLastVO, // last VOP index
		const Bool* rgbSpatialScalability, // spatial scalability indicator
		const Int*	rgiEnhnLayerUpsampleFactor, // upsample the baselayer by this factor
		Bool* rgbRateControl [], // rate control on/off
		UInt* rguiBudget [], // for rate control
		// for shape coding
		const AlphaUsage* rgfAlphaUsage, // alpha usage for each VOP.  0: binary, 1: 8-bit
		const Bool* rgbShapeOnly,		 // disable texture motion coding
		const Int* rgiBinaryAlphaTH,	 // threhold is shape size conversion
		const Bool* rgbNoCrChange,		 // no change of cr from mb to mb

		// motion estimation part for each VOP
		UInt** rguiSearchRange,  // motion search range
		Bool** rgbOriginalForME, // flag indicating whether use the original previous VOP for ME
		Bool** rgbAdvPredDisable,//no advanced MC (currenly = obmc, later = obmc + 8x8)

		// major syntax mode
		Bool** rgbSeprMotionTexture, // separate motion-texture coding
		Bool** rgbErrorResilientDisable, //error resilient disable
		Int**	rgbVPBitTh,			// Bit threshold for video packet spacing control
		Bool** rgbDataPartitioning,  //data partitioning
		Bool** rgbReversibleVlc, //reversible VLC

		// for texture coding
		Quantizer** rgfQuant, // quantizer selection, either H.263 or MPEG
		Bool** rgbLoadIntraMatrix, // load user-defined intra Q-Matrix
		Int*** rgppiIntraQuantizerMatrix, // Intra Q-Matrix
		Bool** rgbLoadInterMatrix, // load user-defined inter Q-Matrix
		Int*** rgppiInterQuantizerMatrix, // Inter Q-Matrix
		Bool** rgbIntraPredictionDisable, // intra-DC disable		
		Int** rgiIntraDCSwitchingThr,		//threshold to code dc as ac when pred. is on
		Int** rgiStepI, // I-VOP quantization stepsize
		Int** rgiStep, // P-VOP quantization stepsize
		Bool**	rgbLoadIntraMatrixAlpha,
		Int*** rgppiIntraQuantizerMatrixAlpha,
		Bool**	rgbLoadInterMatrixAlpha,
		Int*** rgppiInterQuantizerMatrixAlpha,
		Int** rgiStepIAlpha, // I-VOP quantization stepsize for Alpha
		Int** rgiStepAlpha, // P-VOP quantization stepsize for Alpha
		Int** rgbNoAlphaQuantUpdate, // discouple gray quant update with tex. quant
		Int** rgiStepBCode, // code for quantization stepsize for B-VOP
		const Int* rgiNumOfBbetweenPVOP,		// no of B-VOPs between P-VOPs
		const Int* rgiNumOfPbetweenIVOP,		// no of P-VOPs between I-VOPs
		const Bool* rgbDeblockFilterDisable, //deblocing filter disable
		const EntropyCodeType* rgfEncType, // Arithmetic code usage
		
		// file information
		const Char* pchPrefix, // prefix name of the movie
		const Char* pchBmpFiles, // bmp file directory location
		const ChromType* rgfChrType, // input chrominance type. 0 - 4:4:4, 1 - 4:2:2, 0 - 4:2:0
		const Char* pchBCStrFile, // VDS: back-channel bitstream file
		const Char* pchMeshFile, // VDS: mesh data file
		const Char* pchOutBmpFiles, // quantized frame file directory
		const Char* pchOutStrFiles, // output bitstream file
		const Int* rgiTemporalRate, // temporal subsampling rate
		
		// statistics dumping options
		const Bool* rgbDumpMB,
		const Bool* rgbTrace,

		// for sprite info
		const UInt* rguiSpriteUsage, // sprite usage
		const UInt* rguiWarpingAccuracy, // warping accuracy
		const Int* rgNumOfPnts, // number of points for sprite, 0 for stationary and -1 for no sprite
		const Char* pchSptDir, // sprite directory
		const Char* pchSptPntDir // sprite point file
) :
	m_iFirstFrame (iFirstFrm), m_iLastFrame (iLastFrm),
	m_iFirstVO (iFirstVO), m_iLastVO (iLastVO),
	m_rgbSpatialScalability (rgbSpatialScalability),
	m_rgbRateControl (rgbRateControl), 
	m_rguiBudget (rguiBudget),
	m_pchPrefix (pchPrefix), m_pchBmpFiles (pchBmpFiles),
	m_pchBCStrFile (pchBCStrFile), m_pchMeshFile (pchMeshFile), // VDS:
	m_pchReconYUVDir (pchOutBmpFiles), m_rgfChrType (rgfChrType),
	m_pchOutStrFiles (pchOutStrFiles),
	m_rguiSpriteUsage (rguiSpriteUsage), m_rguiWarpingAccuracy (rguiWarpingAccuracy), 
	m_rgNumOfPnts (rgNumOfPnts), m_pchSptDir (pchSptDir), 
	m_pchSptPntDir (pchSptPntDir)
{
	// data preparation
	m_iNumFrame = m_iLastFrame - m_iFirstFrame + 1;
	assert (m_iNumFrame > 0);
	m_rctOrg = CRct (0, 0, uiFrmWidth, uiFrmHeight);
	assert (uiFrmWidth <= 8192 && uiFrmHeight <= 8192); //2^13 maximum size
	m_iNumVO = m_iLastVO - m_iFirstVO + 1;
	assert (m_iNumVO > 0);
	m_rgvolmd [BASE_LAYER] = new VOLMode [m_iNumVO];
	m_rgvolmd [ENHN_LAYER] = new VOLMode [m_iNumVO];
	m_rgvopmd [BASE_LAYER] = new VOPMode [m_iNumVO];
	m_rgvopmd [ENHN_LAYER] = new VOPMode [m_iNumVO];
	Int iVO;
	for (iVO = 0; iVO < m_iNumVO; iVO++) {
		// set VOL and VOP mode values
		for (Int iLayer = BASE_LAYER; iLayer <= m_rgbSpatialScalability [iVO] * ENHN_LAYER; iLayer++)	{
			m_rgvolmd [iLayer][iVO].volType			= (VOLtype) iLayer;
			m_rgvolmd [iLayer][iVO].iEnhnUpFactor	= (iLayer == BASE_LAYER) ? 1 : rgiEnhnLayerUpsampleFactor [iVO];
			m_rgvolmd [iLayer][iVO].bvds	= bvds; // VDS:
			m_rgvolmd [iLayer][iVO].fAUsage			= rgfAlphaUsage [iVO];
			m_rgvolmd [iLayer][iVO].bShapeOnly		= rgbShapeOnly [iVO];
			m_rgvolmd [iLayer][iVO].iBinaryAlphaTH	= rgiBinaryAlphaTH [iVO] * 16;  //magic no. from the vm
			m_rgvolmd [iLayer][iVO].bNoCrChange		= rgbNoCrChange [iVO];
			m_rgvolmd [iLayer][iVO].bOriginalForME	= rgbOriginalForME [iLayer][iVO];
			m_rgvolmd [iLayer][iVO].bAdvPredDisable = rgbAdvPredDisable [iLayer][iVO];
			m_rgvolmd [iLayer][iVO].bSeprMotionTexture 
													= rgbSeprMotionTexture [iLayer][iVO];
			m_rgvolmd [iLayer][iVO].bErrorResilientDisable 
													= rgbErrorResilientDisable [iLayer][iVO];
			m_rgvolmd [iLayer][iVO].bVPBitTh		= rgbVPBitTh[iLayer][iVO];
			m_rgvolmd [iLayer][iVO].bDataPartitioning = rgbDataPartitioning [iLayer][iVO];
			m_rgvolmd [iLayer][iVO].bReversibleVlc  = rgbReversibleVlc [iLayer][iVO];

			m_rgvolmd [iLayer][iVO].fQuantizer		= rgfQuant [iLayer][iVO];
			m_rgvolmd [iLayer][iVO].bIntraPredictionDisable 
													= rgbIntraPredictionDisable [iLayer][iVO];
			m_rgvolmd [iLayer][iVO].bNoGrayQuantUpdate 
													= rgbNoAlphaQuantUpdate [iLayer][iVO];

			m_rgvopmd [iLayer][iVO].iSearchRangeForward	= rguiSearchRange [iLayer][iVO];
			m_rgvopmd [iLayer][iVO].iSearchRangeBackward= rguiSearchRange [iLayer][iVO];
			m_rgvopmd [iLayer][iVO].intStepI		= rgiStepI [iLayer][iVO];
			m_rgvopmd [iLayer][iVO].intStep			= rgiStep [iLayer][iVO];
			m_rgvopmd [iLayer][iVO].intStepIAlpha	= rgiStepIAlpha [iLayer][iVO];
			m_rgvopmd [iLayer][iVO].intStepAlpha	= rgiStepAlpha [iLayer][iVO];
			m_rgvopmd [iLayer][iVO].intDBQuant		= rgiStepBCode [iLayer][iVO];
			m_rgvopmd [iLayer][iVO].iIntraDcSwitchThr 
													= rgiIntraDCSwitchingThr [iLayer][iVO];
			m_rgvolmd [iLayer][iVO].bLoadIntraMatrix= rgbLoadIntraMatrix [iLayer][iVO];
			m_rgvolmd [iLayer][iVO].bLoadInterMatrix= rgbLoadInterMatrix [iLayer][iVO];
			m_rgvolmd [iLayer][iVO].bLoadIntraMatrixAlpha 
													= rgbLoadIntraMatrixAlpha [iLayer][iVO];
			m_rgvolmd [iLayer][iVO].bLoadInterMatrixAlpha 
													= rgbLoadInterMatrixAlpha [iLayer][iVO];
			memcpy (m_rgvolmd [iLayer][iVO].rgiIntraQuantizerMatrix,		rgppiIntraQuantizerMatrix [iLayer][iVO], BLOCK_SQUARE_SIZE * sizeof (Int));
			memcpy (m_rgvolmd [iLayer][iVO].rgiInterQuantizerMatrix,		rgppiInterQuantizerMatrix [iLayer][iVO], BLOCK_SQUARE_SIZE * sizeof (Int));
			memcpy (m_rgvolmd [iLayer][iVO].rgiIntraQuantizerMatrixAlpha,	rgppiIntraQuantizerMatrixAlpha [iLayer][iVO], BLOCK_SQUARE_SIZE * sizeof (Int));
			memcpy (m_rgvolmd [iLayer][iVO].rgiInterQuantizerMatrixAlpha,	rgppiInterQuantizerMatrixAlpha [iLayer][iVO], BLOCK_SQUARE_SIZE * sizeof (Int));

			m_rgvolmd [iLayer][iVO].bDeblockFilterDisable 
													= rgbDeblockFilterDisable [iVO];
			m_rgvolmd [iLayer][iVO].fEntropyType	= rgfEncType [iVO];
			m_rgvolmd [iLayer][iVO].iBbetweenP	= rgiNumOfBbetweenPVOP [iVO];
			m_rgvolmd [iLayer][iVO].iPbetweenI	= rgiNumOfPbetweenIVOP [iVO];
//			m_rgvolmd [iLayer][iVO].iPeriodOfIVOP	= (rgiNumOfBbetweenPVOP [iVO] + 1) * (rgiNumOfPbetweenIVOP [iVO] + 1);			//in temporally sumsampled domain
//			m_rgvolmd [iLayer][iVO].iPeriodOfPVOP	= rgiNumOfBbetweenPVOP [iVO] + 1;			//in temporally sumsampled domain
			m_rgvolmd [iLayer][iVO].iTemporalRate	= rgiTemporalRate [iVO];
			Int iClockRate = SOURCE_FRAME_RATE / m_rgvolmd [iLayer][iVO].iTemporalRate;
			if(iClockRate * m_rgvolmd [iLayer][iVO].iTemporalRate != SOURCE_FRAME_RATE)
				iClockRate = SOURCE_FRAME_RATE;
			m_rgvolmd [iLayer][iVO].iClockRate		= iClockRate;
			m_rgvolmd [iLayer][iVO].bDumpMB			= rgbDumpMB [iVO];
			m_rgvolmd [iLayer][iVO].bTrace			= rgbTrace [iVO];
		}
	}
	static Char pchYUV [100], pchSeg [100];
	// check whether there are data for separate layers
	sprintf (pchYUV, ROOT_YUVFILE, m_pchBmpFiles, m_pchPrefix);
	sprintf (pchSeg, ROOT_SEGFILE, m_pchBmpFiles, m_pchPrefix);
	m_bTexturePerVOP = !fileExistence (pchYUV);
	m_bAlphaPerVOP = !fileExistence (pchSeg);

	// sprite info
	m_ppstSrc = new CSiteD* [m_iNumVO];
	m_pppstDst = new CSiteD** [m_iNumVO];
	for (iVO = 0; iVO < m_iNumVO; iVO++) {
		m_ppstSrc [iVO] = (m_rgNumOfPnts [iVO] > 0) ? new CSiteD [m_rgNumOfPnts [iVO]] : NULL;
		m_pppstDst [iVO] = new CSiteD* [m_iNumFrame];
		for (Int ifr = 0; ifr < m_iNumFrame; ifr++) {
			m_pppstDst [iVO] [ifr] = 
				(m_rgNumOfPnts [iVO] > 0) ? new CSiteD [m_rgNumOfPnts [iVO]] : 
				NULL;
		}
	}
	for (iVO = m_iFirstVO; iVO <= m_iLastVO; iVO++)
		if (m_rgNumOfPnts [iVO - m_iFirstVO] > 0)
				readPntFile (iVO);
}	

Void CSessionEncoder::encode ()
{
	FILE* pfYuvSrc = NULL;
	FILE* pfSegSrc = NULL;
	FILE* rgpfReconYUV [2];
	FILE* rgpfReconSeg [2];
	Int iVO;
	UInt iVOrelative = 0;
	for (iVO = m_iFirstVO; iVO <= (Int) m_iLastVO; iVO++, iVOrelative++) {
		ofstream* rgpostrm [2];
		ofstream* rgpostrmTrace [2];
		PixelC pxlcObjColor;
		VOLMode volmd = m_rgvolmd [BASE_LAYER] [iVOrelative];
		Bool bvds = volmd.bvds; // VDS:
		getInputFiles (	pfYuvSrc, pfSegSrc, 
				rgpfReconYUV, rgpfReconSeg, 
						rgpostrm, rgpostrmTrace, pxlcObjColor, iVO, volmd);

/*
 *CVOPIntYUVBA* pvopfSpt = NULL;
 *CRct rctSprite;
 *if (m_rguiSpriteUsage [iVOidx] == 1) {
 *pvopfSpt = loadSpt (iobj);
 *	rctSprite = pvopfSpt -> getPlane (Y_PLANE) -> where ();
 *}
 */
		CRct rctOrg;
		CVideoObjectEncoder* rgpvoenc [2];
		if (m_rguiSpriteUsage [iVO - m_iFirstVO] == 1) {
			// change m_rctOrg to sprite size
			rctOrg = m_rctOrg;
			m_rctOrg = findSptRct (iVO);
		}

		initVOEncoder (rgpvoenc, iVO, rgpostrmTrace);
		rgpostrm [BASE_LAYER]->write (rgpvoenc [BASE_LAYER]->pOutStream ()->str (),			//VO and VOL header
									  rgpvoenc [BASE_LAYER]->pOutStream ()->pcount ());

		if (m_rguiSpriteUsage [iVO - m_iFirstVO] == 1) {
			// load sprite data into m_pvopcOrig
			loadSpt (iVO, rgpvoenc [BASE_LAYER] -> m_pvopcOrig);
			// encode the initial sprite
			rgpvoenc [BASE_LAYER] -> encode (TRUE, -1, IVOP);
			if (volmd.fAUsage != RECTANGLE) {
				// change m_pvopcRefQ1 to m_pvopcSptQ for warping
				rgpvoenc [BASE_LAYER] -> swapRefQ1toSpt ();
				// restore m_pvopcCurrQ size to the normal one
				m_rctOrg = rctOrg;
				rgpvoenc [BASE_LAYER] -> changeSizeofCurrQ (rctOrg);
			}
			rgpostrm [BASE_LAYER]->write (rgpvoenc [BASE_LAYER]->pOutStream ()->str (), //write sprite unit
										  rgpvoenc [BASE_LAYER]->pOutStream ()->pcount ());
		}
		assert (!m_rctOrg.empty ());
//		assert (m_rctOrg <= rgpvoenc [BASE_LAYER]->m_pvopcRefOrig0->whereY ());

		/* Simon's algorithm for generation of IPB sequences with arbitrary shape
			(transparency allows for possible skipped frames) 
			P prediction across skips is not allowed */

		Int iRefFrame;
		Int iDT = volmd.iTemporalRate;
		Int iRefInterval = volmd.iBbetweenP + 1;
		Int iPCount;
		Bool bObjectExists;
		for(iRefFrame = m_iFirstFrame; iRefFrame <= m_iLastFrame; iRefFrame += iDT)
		{
			// encode initial I frame or non-coded sequence
			if(rgpvoenc [BASE_LAYER] -> skipTest(iRefFrame,IVOP)) // rate control
				continue;
			bObjectExists = loadDataSpriteCheck (iVOrelative,iRefFrame, pfYuvSrc, pfSegSrc, pxlcObjColor, rgpvoenc [BASE_LAYER]->m_pvopcOrig, volmd);
			if(bvds) { // VDS: using overload function
				encodeVideoObject(bObjectExists, iRefFrame, IVOP,
				iVO,iVOrelative,pfYuvSrc,pfSegSrc,
				m_pchBCStrFile, m_pchMeshFile, // VDS:
				rgpfReconYUV,rgpfReconSeg,
				pxlcObjColor,rgpvoenc,volmd,rgpostrm);
			} else {
				encodeVideoObject(bObjectExists, iRefFrame, IVOP,
				iVO,iVOrelative,pfYuvSrc,pfSegSrc,rgpfReconYUV,rgpfReconSeg,
				pxlcObjColor,rgpvoenc,volmd,rgpostrm);
			}
			
			// go to next frame if this was not coded
			if(!bObjectExists)
				continue; 

			iPCount = volmd.iPbetweenI;
			
			Int iWaitInterval = 0;

			while(TRUE)
			{
				// search for next reference frame
				Int iSearchFrame;
				for(iSearchFrame = iRefFrame + iDT * iRefInterval + iWaitInterval;
						iSearchFrame > iRefFrame; iSearchFrame -= iDT)
					if(iSearchFrame <= m_iLastFrame)
						if(loadDataSpriteCheck(iVOrelative,iSearchFrame, pfYuvSrc, pfSegSrc, pxlcObjColor, rgpvoenc [BASE_LAYER]->m_pvopcOrig, volmd))
							break;  // found a suitable reference frame

				if(iSearchFrame==iRefFrame)
					break;  // ignore non coded until a new I frame

				if(rgpvoenc [BASE_LAYER] -> skipTest(iSearchFrame,iPCount ? PVOP : IVOP)) // rate control
				{
					// too early! need to wait a frame
					iWaitInterval+=iDT;
					continue;
				}
				iWaitInterval = 0;

				// encode the next reference frame
				if(iPCount==0)
				{
					// encode IVOP
					encodeVideoObject(TRUE, iSearchFrame, IVOP,
						iVO,iVOrelative,pfYuvSrc,pfSegSrc,rgpfReconYUV,rgpfReconSeg,
						pxlcObjColor,rgpvoenc,volmd,rgpostrm);
					iPCount = volmd.iPbetweenI;
				}
				else
				{
					// encoder PVOP
					encodeVideoObject(TRUE, iSearchFrame, PVOP,
						iVO,iVOrelative,pfYuvSrc,pfSegSrc,rgpfReconYUV,rgpfReconSeg,
						pxlcObjColor,rgpvoenc,volmd,rgpostrm);
					if(iPCount>0)  // needed to handle iPCount = -1
						iPCount--;
				}

				// encode B frames if needed
				Int iBFrame;
				for(iBFrame = iRefFrame + iDT; iBFrame < iSearchFrame; iBFrame += iDT)
				{
					if(rgpvoenc [BASE_LAYER] -> skipTest(iBFrame,BVOP))
						continue;
					bObjectExists = loadDataSpriteCheck(iVOrelative,iBFrame, pfYuvSrc, pfSegSrc, pxlcObjColor, rgpvoenc [BASE_LAYER]->m_pvopcOrig, volmd);
					encodeVideoObject(bObjectExists, iBFrame, BVOP,
						iVO,iVOrelative,pfYuvSrc,pfSegSrc,rgpfReconYUV,rgpfReconSeg,
						pxlcObjColor,rgpvoenc,volmd,rgpostrm);
				}

				// move onwards
				iRefFrame = iSearchFrame;
			}
		}

#ifdef MS_BACKWARD_COMPATIBLE
		//to write out the left over byte in non-alignment case
		((COutBitStream* ) rgpvoenc [BASE_LAYER]->pOutStream ())->reset (); 
		((COutBitStream* ) rgpvoenc [BASE_LAYER]->pOutStream ())->flush ();
		rgpostrm [BASE_LAYER]->write (rgpvoenc [BASE_LAYER]->pOutStream ()->str (),
									  rgpvoenc [BASE_LAYER]->pOutStream ()->pcount ());
#endif
		cout << "\nBASE VOL " << iVO << "\n";
		CStatistics sts = rgpvoenc [BASE_LAYER] -> statVOL ();
#ifdef __TRACE_AND_STATS_
		sts.print (TRUE); // dumping statistics
#endif // __TRACE_AND_STATS_
		delete rgpvoenc [BASE_LAYER];
		fclose (rgpfReconYUV [BASE_LAYER]);
		if (m_rgvolmd [BASE_LAYER] [iVOrelative].fAUsage != RECTANGLE)	{
			fclose (rgpfReconSeg [BASE_LAYER]);
		}
		delete rgpostrm [BASE_LAYER];
		delete rgpostrmTrace [BASE_LAYER];

		if (m_rgbSpatialScalability [iVO] == TRUE)	{
			cout << "\nEHNANCED VOL " << iVO << "\n";
			sts = rgpvoenc [ENHN_LAYER] -> statVOL ();
			sts.print (TRUE); // dumping statistics
			delete rgpvoenc [ENHN_LAYER];
			fclose (rgpfReconYUV [ENHN_LAYER]);
			if (m_rgvolmd [BASE_LAYER] [iVOrelative].fAUsage != RECTANGLE)	{
				fclose (rgpfReconSeg [ENHN_LAYER]);
			}
			delete rgpostrm [ENHN_LAYER];
			delete rgpostrmTrace [ENHN_LAYER];
		}
	}
}

Bool CSessionEncoder::loadDataSpriteCheck(UInt iVOrelative,UInt iFrame, FILE* pfYuvSrc, FILE* pfSegSrc, PixelC pxlcObjColor, CVOPU8YUVBA* pvopcDst, const VOLMode& volmd)
{
	Bool bObjectExists = TRUE;
	if(m_rguiSpriteUsage [iVOrelative] == 0)
		bObjectExists = loadData(iFrame,pfYuvSrc,pfSegSrc, pxlcObjColor, pvopcDst, volmd);
	return bObjectExists;
}

Void CSessionEncoder::encodeVideoObject(Bool bObjectExists,
										Int iFrame,
										VOPpredType predType,
										Int iVO,Int iVOrelative,
										FILE* pfYuvSrc,
										FILE* pfSegSrc,
										FILE* rgpfReconYUV[],
										FILE* rgpfReconSeg[],
										PixelC pxlcObjColor,
										CVideoObjectEncoder** rgpvoenc,
										const VOLMode& volmd,
										ofstream* rgpostrm[])
{
	Int iLayer;
	for (iLayer = BASE_LAYER; iLayer <= m_rgbSpatialScalability [iVOrelative] * ENHN_LAYER; iLayer++)	{
		if (m_rguiSpriteUsage [iVOrelative] == 0) {
			// no sprite used
			if(iLayer!=BASE_LAYER)
			{
				if(rgpvoenc [iLayer] -> skipTest((Time)iFrame,predType))  // rate control
					continue;
				bObjectExists = loadData (iFrame, pfYuvSrc, pfSegSrc, pxlcObjColor, rgpvoenc [BASE_LAYER]->m_pvopcOrig, volmd);
			}
			const CVOPU8YUVBA* pvopcBaseQuant = (iLayer == ENHN_LAYER) ? rgpvoenc [BASE_LAYER]->pvopcQuantCurr () : NULL;
			rgpvoenc [iLayer] -> encode (bObjectExists, (Time) (iFrame - m_iFirstFrame), predType); //, pvopcBaseQuant
		}
		else {
			CRct rctWarp = (m_rgNumOfPnts [iVO - m_iFirstVO] > 0)? findBoundBoxInAlpha (iFrame, iVO) : m_rctOrg;
			rgpvoenc [BASE_LAYER]->encodeSptTrajectory (iFrame, m_pppstDst [iVO - m_iFirstVO] [iFrame - m_iFirstFrame], rctWarp);
			bObjectExists = TRUE;
		}
		//dump the output
		if (bObjectExists)	{
			dumpData (rgpfReconYUV [iLayer], rgpfReconSeg [iLayer], 
				rgpvoenc [iLayer]->pvopcReconCurr (), volmd);
		}
		rgpostrm [BASE_LAYER]->write (rgpvoenc [iLayer]->pOutStream ()->str (),
			rgpvoenc [iLayer]->pOutStream ()->pcount ());
	}
}

#define VDO_START_CODE		0x1BF
#define VDOP_START_CODE		0x1BD
// VDS: overload function
Void CSessionEncoder::encodeVideoObject(Bool bObjectExists,
										Int iFrame,
										VOPpredType predType,
										Int iVO,Int iVOrelative,
										FILE* pfYuvSrc,
										FILE* pfSegSrc,
										const Char* pchBCStrFile,
										const Char* pchMeshFile,
										FILE* rgpfReconYUV[],
										FILE* rgpfReconSeg[],
										PixelC pxlcObjColor,
										CVideoObjectEncoder** rgpvoenc,
										const VOLMode& volmd,
										ofstream* rgpostrm[])
{
	assert(volmd.bvds);
	Int vdo_start_code, vdop_start_code;
	Int FOV;
	Int rwin_width, rwin_height;
	Int marker;
	ifstream *pbcstrm;
	CInBitStream *pbcbitstrmIn;
	CGrid grid(pchMeshFile);
	Float denorm(UInt, UInt);

	pbcstrm = new ifstream (m_pchBCStrFile, IOS_BINARY | ios::in);
	pbcbitstrmIn = new CInBitStream (*pbcstrm);

	vdo_start_code = pbcbitstrmIn->getBits(32);
	assert(vdo_start_code==VDO_START_CODE);
	FOV = pbcbitstrmIn->getBits(16);
	marker = pbcbitstrmIn->getBits(1);
	assert(marker);
	rwin_width = pbcbitstrmIn->getBits(16);
	marker = pbcbitstrmIn->getBits(1);
	assert(marker);
	rwin_height = pbcbitstrmIn->getBits(16);
	marker = pbcbitstrmIn->getBits(1);
	assert(marker);

	Int iLayer;
	for (iLayer = BASE_LAYER; iLayer <= m_rgbSpatialScalability [iVOrelative] * ENHN_LAYER; iLayer++)	{
		assert(m_rguiSpriteUsage [iVOrelative] == 0); // no sprite used
		if(iLayer!=BASE_LAYER)
		{
			if(rgpvoenc [iLayer] -> skipTest((Time)iFrame,predType))  // rate control
				continue;
			bObjectExists = loadData (iFrame, pfYuvSrc, pfSegSrc, pxlcObjColor, rgpvoenc [BASE_LAYER]->m_pvopcOrig, volmd);
		}
		const CVOPU8YUVBA* pvopcBaseQuant = (iLayer == ENHN_LAYER) ? rgpvoenc [BASE_LAYER]->pvopcQuantCurr () : NULL;
		Int nframe=0;
		CRct allmask_rctY(0, 0, m_rctOrg.width, m_rctOrg.height ());
		CRct allmask_rctUV(0, 0, m_rctOrg.width/2, m_rctOrg.height ()/2);
		CU8Image *allmaskY = new CU8Image(allmask_rctY, 0);
		CU8Image *allmaskUV = new CU8Image(allmask_rctUV, 0);
		while(nextbits_bytealigned(pbcbitstrmIn) == VDOP_START_CODE) {
			UInt xpos1, xpos2, ypos1, ypos2, zpos1, zpos2;
			UInt xaim1, xaim2, yaim1, yaim2, zaim1, zaim2;
			Float fxpos, fypos, fzpos;
			Float fxaim, fyaim, fzaim;
			vector ViewPos, AimPos;

			xpos1 = pbcbitstrmIn->getBits(16);
			marker = pbcbitstrmIn->getBits(1);
			assert(marker);
			xpos2 = pbcbitstrmIn->getBits(16);
			marker = pbcbitstrmIn->getBits(1);
			assert(marker);
			fxpos = denorm(xpos1, xpos2);

			ypos1 = pbcbitstrmIn->getBits(16);
			marker = pbcbitstrmIn->getBits(1);
			assert(marker);
			ypos2 = pbcbitstrmIn->getBits(16);
			marker = pbcbitstrmIn->getBits(1);
			assert(marker);
			fypos = denorm(ypos1, ypos2);

			zpos1 = pbcbitstrmIn->getBits(16);
			marker = pbcbitstrmIn->getBits(1);
			assert(marker);
			zpos2 = pbcbitstrmIn->getBits(16);
			marker = pbcbitstrmIn->getBits(1);
			assert(marker);
			fzpos = denorm(zpos1, zpos2);

			xaim1 = pbcbitstrmIn->getBits(16);
			marker = pbcbitstrmIn->getBits(1);
			assert(marker);
			xaim2 = pbcbitstrmIn->getBits(16);
			marker = pbcbitstrmIn->getBits(1);
			assert(marker);
			fxaim = denorm(xaim1, xaim2);

			yaim1 = pbcbitstrmIn->getBits(16);
			marker = pbcbitstrmIn->getBits(1);
			assert(marker);
			yaim2 = pbcbitstrmIn->getBits(16);
			marker = pbcbitstrmIn->getBits(1);
			assert(marker);
			fyaim = denorm(yaim1, yaim2);

			zaim1 = pbcbitstrmIn->getBits(16);
			marker = pbcbitstrmIn->getBits(1);
			assert(marker);
			zaim2 = pbcbitstrmIn->getBits(16);
			marker = pbcbitstrmIn->getBits(1);
			assert(marker);
			fzaim = denorm(zaim1, zaim2);

			ViewPos = vector(fxpos, fypos, fzpos);
			AimPos = vector(fxaim, fyaim, fzaim);

			rgpvoenc [iLayer] -> encode (
				bObjectExists, (Time) nframe, predType, 
				&grid, ViewPos, AimPos, FOV, rwin_width, 
				rwin_height, allmaskY, allmaskUV); //, pvopcBaseQuant
			nframe += volmd.iTemporalRate;
			//dump the output
			dumpData (rgpfReconYUV [iLayer], rgpfReconSeg [iLayer], 
				rgpvoenc [iLayer]->pvopcReconCurr (), volmd);
			rgpostrm [BASE_LAYER]->write (rgpvoenc [iLayer]->pOutStream ()->str (),
				rgpvoenc [iLayer]->pOutStream ()->pcount ());
		}
	}
}

#define VMIN    -100.0
#define VMAX    100.0
#define MAX32	4294967296.0
 
float denorm(UInt uivalue1, UInt uivalue2)
{
	UInt uivalue = (uivalue2<<16)+uivalue1;
	Float fvalue = uivalue*(VMAX-VMIN)/(MAX32-1) + VMIN;

	return fvalue;
}

Int CSessionEncoder::nextbits_bytealigned(CInBitStream *pbcbitstrmIn)
{
	// ensure byte alignment
	pbcbitstrmIn->flush();

	while(pbcbitstrmIn->peekBits(32)!=VDOP_START_CODE)
        {
                pbcbitstrmIn->getBits(8);
                if(pbcbitstrmIn->eof()==EOF)
                        return EOF;
        }
        Int ret = pbcbitstrmIn->getBits(32);
        return ret;
}
	


Void CSessionEncoder::readPntFile (UInt iobj)
{
	Char pchtmp [100];
	sprintf (pchtmp, SUB_PNTFILE, m_pchSptPntDir, iobj, m_pchPrefix);
	FILE* pfPnt = fopen (pchtmp, "r");
	Int numPnt;
	Int iVOidx = iobj - m_iFirstVO;
	fscanf (pfPnt, "%d", &numPnt);
	assert (numPnt == m_rgNumOfPnts [iVOidx]);
	Int ifrF;
	Double dblX, dblY;
	for (Int ip = 0; ip < numPnt; ip++)
		fscanf (pfPnt, "%lf%lf", &m_ppstSrc [iVOidx] [ip].x, &m_ppstSrc [iVOidx] [ip].y); // not used
	for (Int ifr = 0; ifr <= m_iLastFrame; ifr++) {
		fscanf (pfPnt, "%d", &ifrF);
		if (ifrF >= m_iFirstFrame && ifrF <= m_iLastFrame) {
			for (Int ip = 0; ip < numPnt; ip++)	{
				fscanf (pfPnt, "%lf%lf", &dblX, &dblY);
				Int iFrmIndex = ifrF - m_iFirstFrame;
				if (iFrmIndex % (m_rgvolmd [BASE_LAYER] [iVOidx].iTemporalRate) == 0)	{
					m_pppstDst [iVOidx] [iFrmIndex] [ip].x = dblX;
					m_pppstDst [iVOidx] [iFrmIndex] [ip].y = dblY;
				}
			}
		} else {
			for (Int ip = 0; ip < numPnt; ip++)	{
				fscanf (pfPnt, "%lf%lf", &dblX, &dblY);
			}
		}
	}
}

Void CSessionEncoder::loadSpt (UInt iobj, CVOPU8YUVBA* pvopcDst)
{
	Char pchSpt [100];
	sprintf (pchSpt, SUB_VDLFILE, m_pchSptDir, iobj, m_pchPrefix);

	FILE* pf = fopen (pchSpt, "rb");
	// read overhead
	Int c0 = getc (pf);
	Int c1 = getc (pf);
	Int c2 = getc (pf);
	assert (c0 == 'S' && (c1 == 'P' || c2 == 'T') );
	CRct rctSpt;
	Bool bAUsage;
	fread (&rctSpt.left, sizeof (CoordI), 1, pf);
	fread (&rctSpt.top, sizeof (CoordI), 1, pf);
	fread (&rctSpt.right, sizeof (CoordI), 1, pf);
	fread (&rctSpt.bottom, sizeof (CoordI), 1, pf);
	fread (&bAUsage, sizeof (Int), 1, pf);
	assert (bAUsage != EIGHT_BIT); // sprite with Alpha channel is not supported at this moment
	Int iYDataHeight = m_rctOrg.height ();
	Int iUVDataHeight = m_rctOrg.height () / 2;
	Int iYFrmWidth = pvopcDst->whereY ().width;
	Int iUvFrmWidth = pvopcDst->whereUV ().width;
	Int nSkipYPixel = iYFrmWidth * EXPANDY_REF_FRAME + EXPANDY_REF_FRAME;
	Int nSkipUvPixel = iUvFrmWidth * EXPANDUV_REF_FRAME + EXPANDUV_REF_FRAME;
	PixelC* ppxlcY = (PixelC*) pvopcDst->pixelsY () + nSkipYPixel;
	PixelC* ppxlcU = (PixelC*) pvopcDst->pixelsU () + nSkipUvPixel;
	PixelC* ppxlcV = (PixelC*) pvopcDst->pixelsV () + nSkipUvPixel;
	
	CoordI y;
	for (y = 0; y < iYDataHeight; y++) {
		Int size = (Int) fread (ppxlcY, sizeof (U8), m_rctOrg.width, pf);
		if (size == 0)
			fprintf (stderr, "Unexpected end of file\n");
		ppxlcY += iYFrmWidth;
	}

	for (y = 0; y < iUVDataHeight; y++) {
		Int size = (Int) fread (ppxlcU, sizeof (U8), m_rctOrg.width / 2, pf);
		if (size == 0)
			fprintf (stderr, "Unexpected end of file\n");
		ppxlcU += iUvFrmWidth;
	}

	for (y = 0; y < iUVDataHeight; y++) {
		Int size = (Int) fread (ppxlcV, sizeof (U8), m_rctOrg.width / 2, pf);
		if (size == 0)
			fprintf (stderr, "Unexpected end of file\n");
		ppxlcV += iUvFrmWidth;
	}

	PixelC* ppxlcBY = (PixelC*) pvopcDst->pixelsBY () + nSkipYPixel;
	Int iObjectExist = 0;
	if ((bAUsage == ONE_BIT) && (pvopcDst -> fAUsage () == ONE_BIT)) { // load Alpha
		//binary
		for (CoordI y = 0; y < iYDataHeight; y++) {
			Int size = (Int) fread (ppxlcBY, sizeof (U8), m_rctOrg.width, pf);
			if (size == 0)
				fprintf (stderr, "Unexpected end of file\n");
			ppxlcBY += iYFrmWidth;
		}
		((CU8Image*) pvopcDst->getPlane (BUV_PLANE))->decimateBinaryShapeFrom (*(pvopcDst->getPlane (BY_PLANE)));
	}
}

Void CSessionEncoder::getInputFiles (FILE*& pfYuvSrc, FILE*& pfAlpSrc, 
									 FILE* rgpfReconYUV [], FILE* rgpfReconSeg [], 
									 ofstream* rgpostrm [], ofstream* rgpostrmTrace [],
									 PixelC& pxlcObjColor, Int iobj, const VOLMode& volmd)
{
	static Char pchYUV [100], pchSeg [100], pchCmp [100], pchTrace [100];

	//output files
	createCmpDir (iobj);
	sprintf (pchCmp, SUB_CMPFILE, m_pchOutStrFiles, iobj, m_pchPrefix);
	sprintf (pchTrace, SUB_TRCFILE, m_pchOutStrFiles, iobj, m_pchPrefix);
	rgpostrm [BASE_LAYER] = new ofstream (pchCmp, IOS_BINARY | ios::out);	
	rgpostrmTrace [BASE_LAYER] = NULL;
	if (volmd.bTrace)
		rgpostrmTrace [BASE_LAYER] = new ofstream (pchTrace);

	// prepare for the reconstructed files
	createReconDir (iobj); // create a directory for the reconstructed YUV in case it doesn't exist
	sprintf (pchYUV, SUB_YUVFILE, m_pchReconYUVDir, iobj, m_pchPrefix);
	sprintf (pchSeg, SUB_SEGFILE, m_pchReconYUVDir, iobj, m_pchPrefix);

	rgpfReconYUV [BASE_LAYER] = fopen (pchYUV, "wb"); // reconstructed YUV file
	assert (rgpfReconYUV [BASE_LAYER] != NULL);
	if (volmd.fAUsage != RECTANGLE)	{
		rgpfReconSeg [BASE_LAYER] = fopen (pchSeg, "wb"); // reconstructed seg file
		assert (rgpfReconSeg [BASE_LAYER] != NULL);
	}

	if (m_rgbSpatialScalability [iobj] == TRUE)	{
		sprintf (pchCmp, ENHN_SUB_CMPFILE, m_pchOutStrFiles, iobj, m_pchPrefix);
		sprintf (pchTrace, ENHN_SUB_TRCFILE, m_pchOutStrFiles, iobj, m_pchPrefix);
		rgpostrm [ENHN_LAYER] = new ofstream (pchCmp, IOS_BINARY | ios::out);
		rgpostrmTrace [ENHN_LAYER] = NULL;
		if (volmd.bTrace)
			rgpostrmTrace [ENHN_LAYER] = new ofstream (pchTrace);

		sprintf (pchYUV, ENHN_SUB_YUVFILE, m_pchReconYUVDir, iobj, m_pchPrefix);
		sprintf (pchSeg, ENHN_SUB_SEGFILE, m_pchReconYUVDir, iobj, m_pchPrefix);
		rgpfReconYUV [ENHN_LAYER] = fopen (pchYUV, "wb");		// reconstructed YUV file
		if (volmd.fAUsage != RECTANGLE)	{
			rgpfReconSeg [ENHN_LAYER] = fopen (pchSeg, "wb"); // reconstructed seg file
			assert (FALSE);									// no spatial scalability for shape
		}
	}
	if (!m_bTexturePerVOP) 
		sprintf (pchYUV, ROOT_YUVFILE, m_pchBmpFiles, m_pchPrefix);
	else 
		sprintf (pchYUV, SUB_YUVFILE, m_pchBmpFiles, iobj, m_pchPrefix);
	pfYuvSrc = fopen (pchYUV, "rb");
	if (pfYuvSrc == NULL)	{
		fprintf (stderr, "can't open %s\n", pchYUV);
		exit (1);
	}
	if (volmd.fAUsage != RECTANGLE) { // load Alpha
		if (!m_bAlphaPerVOP) {
			pxlcObjColor = iobj;
			sprintf (pchSeg, ROOT_SEGFILE, m_pchBmpFiles, m_pchPrefix);
		}
		else	{
			pxlcObjColor = opaqueValue;		  // out data notation
			sprintf (pchSeg, SUB_SEGFILE, m_pchBmpFiles, iobj, m_pchPrefix);
		}
		pfAlpSrc = fopen (pchSeg, "rb");
		if (pfAlpSrc == NULL)	{
			fprintf (stderr, "can't open %s\n", pchSeg);
			exit (1);
		}
	}
}

Void CSessionEncoder::initVOEncoder (CVideoObjectEncoder** rgpvoenc, Int iobj, ofstream* rgpostrmTrace [])
{
	Int iVOidx = iobj - m_iFirstVO;
	if (m_rgbSpatialScalability [iVOidx] == TRUE)	{
		Int iBaseWidth = (m_rgvolmd [BASE_LAYER] [iVOidx].iEnhnUpFactor == 2) ? m_rctOrg.width / 2 : m_rctOrg.width;
		Int iBaseHeight = (m_rgvolmd [BASE_LAYER] [iVOidx].iEnhnUpFactor == 2) ? m_rctOrg.height () / 2 : m_rctOrg.height ();
		rgpvoenc [BASE_LAYER] = new CVideoObjectEncoder (
														iobj, 
														m_rgvolmd [BASE_LAYER] [iVOidx], 
														m_rgvopmd [BASE_LAYER] [iVOidx], 
														m_iFirstFrame,
														m_iLastFrame,
														iBaseWidth,
														iBaseHeight,
														m_rgbRateControl [BASE_LAYER] [iVOidx],
														m_rguiBudget [BASE_LAYER] [iVOidx],
														rgpostrmTrace [BASE_LAYER],
														m_rguiWarpingAccuracy [iVOidx],
														m_rgNumOfPnts [iVOidx],
														m_rctOrg
														);
		rgpvoenc [ENHN_LAYER] = new CVideoObjectEncoder (
														iobj, 
														m_rgvolmd [ENHN_LAYER] [iVOidx],
														m_rgvopmd [ENHN_LAYER] [iVOidx], 
														m_iFirstFrame,
														m_iLastFrame,
														m_rctOrg.width,
														m_rctOrg.height (),
														m_rgbRateControl [ENHN_LAYER] [iVOidx],
														m_rguiBudget [ENHN_LAYER] [iVOidx],
														rgpostrmTrace [ENHN_LAYER],
														0,
														-1,
														m_rctOrg
														//NULL
													   );
	}
	else	{
		rgpvoenc [BASE_LAYER] = new CVideoObjectEncoder (
														iobj, 
														m_rgvolmd [BASE_LAYER] [iVOidx], 
														m_rgvopmd [BASE_LAYER] [iVOidx], 
														m_iFirstFrame,
														m_iLastFrame,
														m_rctOrg.width,
														m_rctOrg.height (),
														m_rgbRateControl [BASE_LAYER] [iVOidx],
														m_rguiBudget [BASE_LAYER] [iVOidx],
														rgpostrmTrace [BASE_LAYER],
														m_rguiWarpingAccuracy [iVOidx],
														m_rgNumOfPnts [iVOidx],
														m_rctOrg
														);
	}
}

Bool CSessionEncoder::loadData (UInt iFrame, FILE* pfYuvSrc, FILE* pfSegSrc, PixelC pxlcObjColor, CVOPU8YUVBA* pvopcDst, const VOLMode& volmd)
{
	Int iLeadingPixels = iFrame * m_rctOrg.area ();
	fseek (pfYuvSrc, iLeadingPixels + iLeadingPixels / 2, SEEK_SET);	//4:2:0
	if (volmd.fAUsage != RECTANGLE)
		fseek (pfSegSrc, iLeadingPixels, SEEK_SET);	

	static Int iYDataHeight = m_rctOrg.height ();
	static Int iUVDataHeight = m_rctOrg.height () / 2;
	static Int iYFrmWidth = pvopcDst->whereY ().width;
	static Int iUvFrmWidth = pvopcDst->whereUV ().width;
	static Int nSkipYPixel = iYFrmWidth * EXPANDY_REF_FRAME + EXPANDY_REF_FRAME;
	static Int nSkipUvPixel = iUvFrmWidth * EXPANDUV_REF_FRAME + EXPANDUV_REF_FRAME;
	PixelC* ppxlcY = (PixelC*) pvopcDst->pixelsY () + nSkipYPixel;
	PixelC* ppxlcU = (PixelC*) pvopcDst->pixelsU () + nSkipUvPixel;
	PixelC* ppxlcV = (PixelC*) pvopcDst->pixelsV () + nSkipUvPixel;
	
	CoordI y;
	for (y = 0; y < iYDataHeight; y++) {
		Int size = (Int) fread (ppxlcY, sizeof (U8), m_rctOrg.width, pfYuvSrc);
		if (size == 0)
			fprintf (stderr, "Unexpected end of file\n");
		ppxlcY += iYFrmWidth;
	}

	for (y = 0; y < iUVDataHeight; y++) {
		Int size = (Int) fread (ppxlcU, sizeof (U8), m_rctOrg.width / 2, pfYuvSrc);
		if (size == 0)
			fprintf (stderr, "Unexpected end of file\n");
		ppxlcU += iUvFrmWidth;
	}

	for (y = 0; y < iUVDataHeight; y++) {
		Int size = (Int) fread (ppxlcV, sizeof (U8), m_rctOrg.width / 2, pfYuvSrc);
		if (size == 0)
			fprintf (stderr, "Unexpected end of file\n");
		ppxlcV += iUvFrmWidth;
	}

	PixelC* ppxlcBY = (PixelC*) pvopcDst->pixelsBY () + nSkipYPixel;
	Int iObjectExist = 0;
	if (volmd.fAUsage == ONE_BIT) { // load Alpha
		//binary
		for (CoordI y = 0; y < iYDataHeight; y++) {
			for (CoordI x = 0; x < m_rctOrg.width; x++) {
				PixelC pxlcCurr = getc (pfSegSrc);
				ppxlcBY [x] = (pxlcCurr == pxlcObjColor) ? opaqueValue : transpValue;
				iObjectExist += ppxlcBY [x];
			}
			ppxlcBY += iYFrmWidth;
		}
		((CU8Image*) pvopcDst->getPlane (BUV_PLANE))->decimateBinaryShapeFrom (*(pvopcDst->getPlane (BY_PLANE)));
	}
	else if (volmd.fAUsage == EIGHT_BIT) { // load Alpha
		//gray
		PixelC* ppxlcA = (PixelC*) pvopcDst->pixelsA () + nSkipYPixel;
		for (CoordI y = 0; y < iYDataHeight; y++) {
			for (CoordI x = 0; x < m_rctOrg.width; x++) {
				PixelC pxlcCurr = getc (pfSegSrc);
				ppxlcA [x]  = (pxlcCurr >= GRAY_ALPHA_THRESHOLD) ? pxlcCurr : transpValue;
				ppxlcBY [x] = (pxlcCurr >= GRAY_ALPHA_THRESHOLD) ? opaqueValue : transpValue;
				iObjectExist += ppxlcBY [x];
			}
			ppxlcBY += iYFrmWidth;
			ppxlcA  += iYFrmWidth;
		}
		((CU8Image*) pvopcDst->getPlane (BUV_PLANE))->decimateBinaryShapeFrom (*(pvopcDst->getPlane (BY_PLANE)));
	}
	else  //rectangle
		iObjectExist = 1;
	return (iObjectExist != 0);

/* Scalability stuff
if (iLayer == BASE_LAYER && m_rgbSpatialScalability [iobj] == TRUE && 
	m_rgvolmd [BASE_LAYER][iVOidx].iEnhnUpFactor == 2)	{				//high res image read in
						CVOPU8YUVBA* pvopcEnhn = new CVOPU8YUVBA (*pvopTexture, m_rgvolmd [BASE_LAYER] [iVOidx].fAUsage);	
						pvopcCurr = pvopcEnhn->downsampleForSpatialScalability ();						//get low res image
	delete pvopcEnhn;
}
*/
}

CRct CSessionEncoder::findBoundBoxInAlpha (UInt ifr, UInt iobj)
{
	CIntImage* pfiARet = NULL;
	Char pchSeg [100];
	if (m_rgvolmd [BASE_LAYER] [iobj].fAUsage != RECTANGLE) { // load Alpha
		if (!m_bAlphaPerVOP) {
			sprintf (pchSeg, ROOT_SEGFILE, m_pchBmpFiles, m_pchPrefix);
			pfiARet = alphaFromCompFile (pchSeg, ifr, iobj, m_rctOrg);
		}
		else {
			sprintf (pchSeg, SUB_SEGFILE, m_pchBmpFiles, iobj, m_pchPrefix);
			pfiARet = new CIntImage (pchSeg, ifr, m_rctOrg);
			pfiARet -> threshold (64);
		}
		CRct rctBoundingBox = pfiARet -> whereVisible ();
		if (rctBoundingBox.left % 2 != 0)
			rctBoundingBox.left--;
		if (rctBoundingBox.top % 2 != 0)
			rctBoundingBox.top--;
		if (rctBoundingBox.right % 2 != 0)
			rctBoundingBox.right++;
		if (rctBoundingBox.bottom % 2 != 0)
			rctBoundingBox.bottom++;
		rctBoundingBox.width = rctBoundingBox.right - rctBoundingBox.left;
		delete pfiARet;
		return rctBoundingBox;
	}
	else
		return m_rctOrg;
}

CRct CSessionEncoder::findSptRct (UInt iobj)
{
	CRct rctSpt;
	Char pchSpt [100];
	sprintf (pchSpt, SUB_VDLFILE, m_pchSptDir, iobj, m_pchPrefix);
	FILE* pf = fopen (pchSpt, "rb");
	// read overhead
	Int c0 = getc (pf);
	Int c1 = getc (pf);
	Int c2 = getc (pf);
	assert (c0 == 'S' && (c1 == 'P' || c2 == 'T') );
	fread (&rctSpt.left, sizeof (CoordI), 1, pf);
	fread (&rctSpt.top, sizeof (CoordI), 1, pf);
	fread (&rctSpt.right, sizeof (CoordI), 1, pf);
	fread (&rctSpt.bottom, sizeof (CoordI), 1, pf);
	rctSpt.width = rctSpt.right - rctSpt.left;
	fclose (pf);
	return rctSpt;
}

Void CSessionEncoder::dumpData (FILE* pfYuvDst, FILE* pfSegDst, const CVOPU8YUVBA* pvopcSrc, const VOLMode& volmd)
{
	static Int iYDataHeight = m_rctOrg.height ();
	static Int iUVDataHeight = m_rctOrg.height () / 2;
	static Int iYFrmWidth = pvopcSrc->whereY ().width;
	static Int iUvFrmWidth = pvopcSrc->whereUV ().width;
	static Int nSkipYPixel = iYFrmWidth * EXPANDY_REF_FRAME + EXPANDY_REF_FRAME;
	static Int nSkipUvPixel = iUvFrmWidth * EXPANDUV_REF_FRAME + EXPANDUV_REF_FRAME;
	const PixelC* ppxlcY = pvopcSrc->pixelsY () + nSkipYPixel;
	const PixelC* ppxlcU = pvopcSrc->pixelsU () + nSkipUvPixel;
	const PixelC* ppxlcV = pvopcSrc->pixelsV () + nSkipUvPixel;
	
	CoordI y;
	for (y = 0; y < iYDataHeight; y++) {
		Int size = (Int) fwrite (ppxlcY, sizeof (U8), m_rctOrg.width, pfYuvDst);
		if (size == 0)
			fprintf (stderr, "Can't write to file\n");
		ppxlcY += iYFrmWidth;
	}

	for (y = 0; y < iUVDataHeight; y++) {
		Int size = (Int) fwrite (ppxlcU, sizeof (U8), m_rctOrg.width / 2, pfYuvDst);
		if (size == 0)
			fprintf (stderr, "Can't write to file\n");
		ppxlcU += iUvFrmWidth;
	}

	for (y = 0; y < iUVDataHeight; y++) {
		Int size = (Int) fwrite (ppxlcV, sizeof (U8), m_rctOrg.width / 2, pfYuvDst);
		if (size == 0)
			fprintf (stderr, "Can't write to file\n");
		ppxlcV += iUvFrmWidth;
	}

	if (volmd.fAUsage != RECTANGLE)	{
		const PixelC* ppxlcA = (volmd.fAUsage == ONE_BIT) ? pvopcSrc->pixelsBY () + nSkipYPixel
														  : pvopcSrc->pixelsA () + nSkipYPixel;
		for (CoordI y = 0; y < iYDataHeight; y++) {
			Int size = (Int) fwrite (ppxlcA, sizeof (U8), m_rctOrg.width, pfSegDst);
			if (size == 0)
				fprintf (stderr, "Can't write to file\n");
			ppxlcA += iYFrmWidth;
		}
	}
/*
 *if (m_rgbSpatialScalability [iVOidx] == TRUE && iLayer == BASE_LAYER	&&
 *	m_rgvolmd [BASE_LAYER][iVOidx].iEnhnUpFactor == 2)
 *	pvopFilteredQuant -> where (m_rctOrg / 2);
 *else
 *	pvopFilteredQuant -> where (m_rctOrg);
 */
}
