/*************************************************************************

This software module was originally developed by 

	Ming-Chieh Lee (mingcl@microsoft.com), Microsoft Corporation
	Wei-ge Chen (wchen@microsoft.com), Microsoft Corporation
	Bruce Lin (blin@microsoft.com), Microsoft Corporation
	Chuang Gu (chuanggu@microsoft.com), Microsoft Corporation
	(date: March, 1996)

and edited by  
 
        Wei Wu (weiwu@stallion.risc.rockwell.com), Rockwell Science Center 
        Homer Chen (homer@jupiter.risc.rockwell.com), Rockwell Science Center

in the course of development of the MPEG-4 Video (ISO/IEC 14496-2). 
This software module is an implementation of a part of one or more MPEG-4 Video tools 
as specified by the MPEG-4 Video. 
ISO/IEC gives users of the MPEG-4 Video free license to this software module or modifications 
thereof for use in hardware or software products claiming conformance to the MPEG-4 Video. 
Those intending to use this software module in hardware or software products are advised that its use may infringe existing patents. 
The original developer of this software module and his/her company, 
the subsequent editors and their companies, 
and ISO/IEC have no liability for use of this software module or modifications thereof in an implementation. 
Copyright is not released for non MPEG-4 Video conforming products. 
Microsoft retains full right to use the code for his/her own purpose, 
assign or donate the code to a third party and to inhibit third parties from using the code for non <MPEG standard> conforming products. 
This copyright notice must be included in all copies or derivative works. 

Copyright (c) 1996, 1997.

Module Name:

	sesEnc.hpp

Abstract:

	Encoder for one image session.

Revision History:
	Sept. 30, 1997: Error resilient tools added by Toshiba

*************************************************************************/

#ifndef __SESENC_HPP_ 
#define __SESENC_HPP_

class COutBitStream;
class CInBitStream; // VDS:
class CEntropyEncoder;
class CEntropyEncoderSet;
class CVideoObjectEncoder;

Class CSessionEncoder
{
public:
	// Constructors
	~CSessionEncoder ();
	CSessionEncoder (
		// general info
		Bool bvds,	// VDS: view-dependent scalability enable
		UInt uiFrmWidth,			// frame width
		UInt uiFrmHeight,			// frame height
		Int iFirstFrm,			// first frame number
		Int iLastFrm,				// last frame number
		Int uiFirstVO,				// first VOP index
		Int iLastVO,				// last VOP index
		const Bool* rgbSpatialScalability, // spatial scalability indicator
		const Int*	iEnhnLayerUpsampleFactor, // upsample the baselayer by this factor
		Bool* rgbRateControl [],	// rate control on/off
		UInt* rguiBudget [],		// for rate control
		// for shape coding
		const AlphaUsage* rgfAlphaUsage,// alpha usage for each VOP.  0: binary, 1: 8-bit
		const Bool* rgbShapeOnly,		// shape only mode
		const Int* rgiBinaryAlphaTH,
		const Bool* rgbNoCrChange,
		// motion estimation part for each VOP
		UInt** rguiSearchRange,			// motion search range
		Bool** rgbOriginalForME,		// flag indicating whether use the original previous VOP for ME
		Bool** rgbAdvPredDisable,		// no advanced MC (currenly = obmc, later = obmc + 8x8)
		// major syntax mode
		Bool** rgbSeprMotionTexture,	// separate motion-texture coding
		Bool** rgbErrorResilientDisable,// error resilient disable
		Int**	rgbVPBitTh,				// Bit threshold for video packet spacing control
		Bool**	bDataPartitioning,		// data partitioning
		Bool**	bReversibleVlc,			// reversible VLC

		// for texture coding
		Quantizer** rgfQuant, // quantizer selection, either H.263 or MPEG
		Bool** rgbLoadIntraMatrix, // load user-defined intra Q-Matrix
		Int*** rgppiIntraQuantizerMatrix, // Intra Q-Matrix
		Bool** rgbLoadInterMatrix, // load user-defined inter Q-Matrix
		Int*** rgppiInterQuantizerMatrix, // Inter Q-Matrix
		Bool** rgbIntraPredictionDisable, // intra-DC disable		
		Int** rgiIntraDCSwitchingThr,		//threshold to code dc as ac when pred. is on
		Int** rgiStepI, // I-VOP quantization stepsize
		Int** rgiStep, // P-VOP quantization stepsize
		Bool**	rgbLoadIntraMatrixAlpha,
		Int*** rgppiIntraQuantizerMatrixAlpha,
		Bool**	rgbLoadInterMatrixAlpha,
		Int*** rgppiInterQuantizerMatrixAlpha,
		Int** rgiStepIAlpha, // I-VOP quantization stepsize for Alpha
		Int** rgiStepAlpha, // P-VOP quantization stepsize for Alpha
		Int** rgbNoAlphaQuantUpdate, // discouple gray quant update with tex. quant
		Int** rgiStepBCode, // code for quantization stepsize for B-VOP
		const Int* rgiNumOfBbetweenPVOP,		// no of B-VOPs between P-VOPs
		const Int* rgiNumOfPbetweenIVOP,		// no of P-VOPs between I-VOPs
		const Bool* rgbDeblockFilterDisable, //deblocing filter disable
		const EntropyCodeType* rgfEncType, // Arithmetic code usage
		
		// file information
		const Char* pchPrefix, // prefix name of the movie
		const Char* pchBmpFiles, // bmp file directory location
		const ChromType* rgfChrType, // input chrominance type. 0 - 4:4:4, 1 - 4:2:2, 0 - 4:2:0
		const Char* pchBCStrFile, // VDS: back-channel bitstream file
		const Char* pchMeshFile, // VDS: mesh data file
		const Char* pchOutBmpFiles, // quantized frame file directory
		const Char* pchOutStrFiles, // output bitstream file
		const Int* rguiTemporalRate, // temporal subsampling rate
		
		// statistics dumping options
		const Bool* rgbDumpMB,
		const Bool* rgbTrace,

		// for sprite info
		const UInt* rguiSpriteUsage, // sprite usage
		const UInt* rguiWarpingAccuracy, // warping accuracy
		const Int* rgNumOfPnts, // number of points for sprite, 0 for stationary and -1 for no sprite
		const Char* pchSptDir, // sprite directory
		const Char* pchSptPntDir // sprite point file
	);

	// Attributes
	UInt compressedSize () const {return m_uCmpSize;} // in bits

	Int nextbits_bytealigned(CInBitStream *pbcbitstrmIn); // VDS:

	// Operations
	Void encode ();

///////////////// implementation /////////////////

protected:
	CRct m_rctOrg;
	Int m_iFirstFrame;				// first frame number
	Int m_iLastFrame;					// last frame number
	Int m_iNumFrame;					// number of frames
	Int m_iFirstVO;					// first VO index
	Int m_iLastVO;					// last VO index
	Int m_iNumVO;					//  number of VO's

	const Bool* m_rgbSpatialScalability;	//spatial scalability on/off
	Bool** m_rgbRateControl;		// rate control on/off
	UInt** m_rguiBudget;			// for rate control
	own VOLMode* m_rgvolmd [2];		// VOL modes
	own VOPMode* m_rgvopmd [2];		// VOP modes
	
	// file information
	const Char* m_pchPrefix; // prefix name of the movie
	const Char* m_pchBmpFiles; // bmp file directory location
	const ChromType* m_rgfChrType; // input chrominance type. 0 - 4:4:4, 1 - 4:2:2, 0 - 4:2:0
	const Char* m_pchBCStrFile; // VDS: back-channel bitstream file
	const Char* m_pchMeshFile; // VDS: mesh data file
	const Char* m_pchReconYUVDir; // quantized frame file directory
	const Char* m_pchOutStrFiles; // output bitstream file
	UInt m_uCmpSize; // in bits
	Bool m_bTexturePerVOP; // Bool for whether there are texture data for each VOP
	Bool m_bAlphaPerVOP; // Bool for whether there are alpha data for each VOP

	// for sprite info
	const Char* m_pchSptDir; // sprite directory
	const Char* m_pchSptPntDir; // sprite point file
	const UInt* m_rguiSpriteUsage; // sprite usage
	const UInt* m_rguiWarpingAccuracy; // warping accuracy
	const Int* m_rgNumOfPnts; // number of points for each VOP
	own CSiteD** m_ppstSrc; // source sites, [numVOP] [numPnt]
	own CSiteD*** m_pppstDst; // destination sites, [numVOP] [numFrm] [numPnt]

	Void getInputFiles (FILE*& pfYuvSrc, FILE*& pfAlpSrc, 
			FILE* rgpfReconYUV [], FILE* rgpfReconSeg [], 
		 	ofstream* rgpostrm [], ofstream* rgpostrmTrace [],
		 	PixelC& pxlcObjColor, Int iobj, const VOLMode& volmd);
	Void initVOEncoder (CVideoObjectEncoder** rgpvoenc, Int iobj, ofstream* rgpostrmTrace []);
	Bool loadDataSpriteCheck(UInt iVOrelative,UInt iFrame, FILE* pfYuvSrc, FILE* pfSegSrc,
					PixelC pxlcObjColor, CVOPU8YUVBA* pvopcDst, const VOLMode& volmd);
	Bool loadData (UInt iFrame, FILE* pfiYuvSrc, FILE* pfiAlpSrc,
				   PixelC pxlcObjColor, CVOPU8YUVBA* pvopcDst, const VOLMode& volmd);
	Void dumpData (FILE* pfYuvDst, FILE* pfSegDst, const CVOPU8YUVBA* pvopcSrc, const VOLMode& volmd);
	Void createReconDir (UInt idx) const;
	Void createCmpDir (UInt idx) const;
	Void encodeVideoObject(
		Bool bObjectExists,
		Int iFrame,
		VOPpredType predType,
		Int iVO,Int iVOrelative,
		FILE* pfYuvSrc,
		FILE* pfSegSrc,
		FILE* rgpfReconYUV[],
		FILE* rgpfReconSeg[],
		PixelC pxlcObjColor,
		CVideoObjectEncoder** rgpvoenc,
		const VOLMode& volmd,
		ofstream* rgpostrm[]
	);
	Void encodeVideoObject( // VDS: overload function for vds
		Bool bObjectExists,
		Int iFrame,
		VOPpredType predType,
		Int iVO,Int iVOrelative,
		FILE* pfYuvSrc,
		FILE* pfSegSrc,
		const Char* pchBCStrFile, // VDS:
		const Char* pchMeshFile, // VDS:
		FILE* rgpfReconYUV[],
		FILE* rgpfReconSeg[],
		PixelC pxlcObjColor,
		CVideoObjectEncoder** rgpvoenc,
		const VOLMode& volmd,
		ofstream* rgpostrm[]
	);

	// sprite
	Void readPntFile (UInt iobj);
	Void loadSpt (UInt iobj, CVOPU8YUVBA* pvopcDst);
	CRct findBoundBoxInAlpha (UInt ifr, UInt iobj);
	CRct findSptRct (UInt iobj);
};

#endif // __SESENC_HPP_
