/*************************************************************************

This software module was originally developed by 

	Ming-Chieh Lee (mingcl@microsoft.com), Microsoft Corporation
	Bruce Lin (blin@microsoft.com), Microsoft Corporation
	(date: June, 1997)

in the course of development of the MPEG-4 Video (ISO/IEC 14496-2). 
This software module is an implementation of a part of one or more MPEG-4 Video tools 
as specified by the MPEG-4 Video. 
ISO/IEC gives users of the MPEG-4 Video free license to this software module or modifications 
thereof for use in hardware or software products claiming conformance to the MPEG-4 Video. 
Those intending to use this software module in hardware or software products are advised that its use may infringe existing patents. 
The original developer of this software module and his/her company, 
the subsequent editors and their companies, 
and ISO/IEC have no liability for use of this software module or modifications thereof in an implementation. 
Copyright is not released for non MPEG-4 Video conforming products. 
Microsoft retains full right to use the code for his/her own purpose, 
assign or donate the code to a third party and to inhibit third parties from using the code for non <MPEG standard> conforming products. 
This copyright notice must be included in all copies or derivative works. 

Copyright (c) 1996, 1997.

Module Name:

	mc.cpp

Abstract:

	Motion compensation routines (common for encoder and decoder).

Revision History:

*************************************************************************/

#include <stdio.h>

#include "typeapi.h"
#include "codehead.h"
#include "mode.h"
#include "vopses.h"
#include "global.h"

#ifdef __MFC_
#ifdef _DEBUG
#undef THIS_FILE
static char BASED_CODE THIS_FILE[] = __FILE__;
#endif

#define new DEBUG_NEW				   
#endif // __MFC_

Void CVideoObject::limitMVRangeToExtendedBBFullPel (CoordI &x,CoordI &y,CRct *prct,Int iBlkSize)
{
	if(prct==NULL)
		return;

	if(x < prct->left)
		x=prct->left;
	else if(x > (prct->right-iBlkSize))
		x=(prct->right-iBlkSize);
	if(y < prct->top)
		y=prct->top;
	else if(y > (prct->bottom-iBlkSize))
		y=(prct->bottom-iBlkSize);
}

Void CVideoObject::limitMVRangeToExtendedBBHalfPel (CoordI &x,CoordI &y,CRct *prct,Int iBlkSize)
{
	if(prct==NULL)
		return;

	if(x < prct->left*2)
		x=prct->left*2;
	else if(x > (prct->right-iBlkSize)*2)
		x=(prct->right-iBlkSize)*2;
	if(y < prct->top*2)
		y=prct->top*2;
	else if(y > (prct->bottom-iBlkSize)*2)
		y=(prct->bottom-iBlkSize)*2;
}

Void CVideoObject::motionCompMB (
	PixelC* ppxlcPredMB,
	const PixelC* ppxlcRefLeftTop,
	const CMotionVector* pmv, const CMBMode* pmbmd, 
	Int imbX, Int imbY,
	CoordI x, CoordI y,
	Bool bSkipNonOBMC,
	CRct *prctMVLimit
)
{
	if (!m_volmd.bAdvPredDisable) {
		motionCompOverLap (
			ppxlcPredMB, ppxlcRefLeftTop,
			pmv, pmbmd,
			imbX, imbY,
			x, y,
			prctMVLimit
		);
	}
	else {
		if (bSkipNonOBMC)
			return;
		if (!pmbmd -> m_bhas4MVForward)
			motionComp (
				ppxlcPredMB, ppxlcRefLeftTop,
				MB_SIZE, 
				x * 2 + pmv->trueMVHalfPel ().x, 
				y * 2 + pmv->trueMVHalfPel ().y ,
				prctMVLimit
			);
		else {
			const CMotionVector* pmv8 = pmv;
			pmv8++;
			CoordI blkX = x + BLOCK_SIZE;
			CoordI blkY = y + BLOCK_SIZE;
			if (pmbmd->m_rgTranspStatus [Y_BLOCK1] != ALL)
				motionComp (
					ppxlcPredMB, ppxlcRefLeftTop,
					BLOCK_SIZE, 
					x * 2 + pmv8->trueMVHalfPel ().x, 
					y * 2 + pmv8->trueMVHalfPel ().y,
					prctMVLimit
				);
			pmv8++;
			if (pmbmd->m_rgTranspStatus [Y_BLOCK2] != ALL)
				motionComp (
					ppxlcPredMB + OFFSET_BLK1, ppxlcRefLeftTop,
					BLOCK_SIZE, 
					blkX * 2 + pmv8->trueMVHalfPel ().x, 
					y * 2 + pmv8->trueMVHalfPel ().y,
					prctMVLimit
				);
			pmv8++;
			if (pmbmd->m_rgTranspStatus [Y_BLOCK3] != ALL)
				motionComp (
					ppxlcPredMB + OFFSET_BLK2, ppxlcRefLeftTop,
					BLOCK_SIZE, 
					x * 2 + pmv8->trueMVHalfPel ().x, 
					blkY * 2 + pmv8->trueMVHalfPel ().y,
					prctMVLimit
				);
			pmv8++;
			if (pmbmd->m_rgTranspStatus [Y_BLOCK4] != ALL)
				motionComp (
					ppxlcPredMB + OFFSET_BLK3, ppxlcRefLeftTop,
					BLOCK_SIZE, 
					blkX * 2 + pmv8->trueMVHalfPel ().x, 
					blkY * 2 + pmv8->trueMVHalfPel ().y,
					prctMVLimit
				);
		}
	}
}

Void CVideoObject::motionComp (
	PixelC* ppxlcPred, // can be either Y or A
	const PixelC* ppxlcRefLeftTop, // point to left-top of the frame
	Int iSize, // either MB or BLOCK size
	CoordI xRef, CoordI yRef, // x + mvX, in half pel unit
	CRct *prctMVLimit // extended bounding box
)
{
	CoordI ix, iy;

	limitMVRangeToExtendedBBHalfPel(xRef,yRef,prctMVLimit,iSize);

	const PixelC* ppxlcRef = 
		ppxlcRefLeftTop + 
		((yRef >> 1) + EXPANDY_REF_FRAME) * m_iFrameWidthY + (xRef >> 1) + EXPANDY_REF_FRAME;

	if (!(yRef & 1)) {
		if (!(xRef & 1)) {  //!bXSubPxl && !bYSubPxl
			for (iy = 0; iy < iSize; iy++) {
				memcpy (ppxlcPred, ppxlcRef, iSize);
				ppxlcRef += m_iFrameWidthY;
				ppxlcPred += MB_SIZE;
			}
		}
		else {  //bXSubPxl && !bYSubPxl
			for (iy = 0; iy < iSize; iy++){
				for (ix = 0; ix < iSize; ix++)
					ppxlcPred [ix] = (ppxlcRef [ix] + ppxlcRef [ix + 1] + 1 - m_vopmd.iRoundingControl) >> 1;
				ppxlcRef += m_iFrameWidthY;
				ppxlcPred += MB_SIZE;
			}
		}
	}
	else {
		const PixelC* ppxlcRefBot;
		if (!(xRef & 1)) {  //!bXSubPxl&& bYSubPxl
			for (iy = 0; iy < iSize; iy++) {
				ppxlcRefBot = ppxlcRef + m_iFrameWidthY;		//UPln -> pixels (xInt,yInt+1);
				for (ix = 0; ix < iSize; ix++) 
					ppxlcPred [ix] = (ppxlcRef [ix] + ppxlcRefBot [ix] + 1 - m_vopmd.iRoundingControl) >> 1;
				ppxlcRef = ppxlcRefBot;
				ppxlcPred += MB_SIZE;
			}
		}
		else { // bXSubPxl && bYSubPxl
			for (iy = 0; iy < iSize; iy++) {
				ppxlcRefBot = ppxlcRef + m_iFrameWidthY;		//UPln -> pixels (xInt,yInt+1);
				for (ix = 0; ix < iSize; ix++){
					ppxlcPred [ix] = (
						ppxlcRef [ix + 1] + ppxlcRef [ix] +
						ppxlcRefBot [ix + 1] + ppxlcRefBot [ix] + 2 - m_vopmd.iRoundingControl
					) >> 2;

				}
				ppxlcRef = ppxlcRefBot;
				ppxlcPred += MB_SIZE;
			}
		}
	}
}

Void CVideoObject::motionCompUV (
	PixelC* ppxlcPredMBU, PixelC* ppxlcPredMBV,
	const CVOPU8YUVBA* pvopcRef,
	CoordI x, CoordI y, 
	CoordI xRefUV, CoordI yRefUV,
	Int iRoundingControl,
	CRct *prctMVLimit
)
{
	UInt ix, iy;
	CoordI iTmpX = x + xRefUV;
	CoordI iTmpY = y + yRefUV;
	limitMVRangeToExtendedBBFullPel (iTmpX,iTmpY,prctMVLimit,MB_SIZE);
	xRefUV = iTmpX - x;
	yRefUV = iTmpY - y;

	Int iPxLoc = (((y + yRefUV) >> 1) + EXPANDUV_REF_FRAME) * m_iFrameWidthUV + ((x + xRefUV) >> 1) + EXPANDUV_REF_FRAME;
	const PixelC* ppxlcPrevU = pvopcRef->pixelsU () + iPxLoc;
	const PixelC* ppxlcPrevV = pvopcRef->pixelsV () + iPxLoc;
	if (!(yRefUV & 1)) {
		if (!(xRefUV & 1)) {  //!bXSubPxl && !bYSubPxl
			for (iy = 0; iy < BLOCK_SIZE; iy++) {
				memcpy (ppxlcPredMBU, ppxlcPrevU, BLOCK_SIZE);
				memcpy (ppxlcPredMBV, ppxlcPrevV, BLOCK_SIZE);
				ppxlcPrevU += m_iFrameWidthUV;
				ppxlcPrevV += m_iFrameWidthUV;
				ppxlcPredMBU += BLOCK_SIZE;
				ppxlcPredMBV += BLOCK_SIZE;
			}
		}
		else {  //bXSubPxl && !bYSubPxl
			for (iy = 0; iy < BLOCK_SIZE; iy++) {
				for (ix = 0; ix < BLOCK_SIZE; ix++) {
					ppxlcPredMBU [ix] = (ppxlcPrevU [ix + 1] + ppxlcPrevU [ix] + 1 - iRoundingControl) >> 1;
					ppxlcPredMBV [ix] = (ppxlcPrevV [ix + 1] + ppxlcPrevV [ix] + 1 - iRoundingControl) >> 1;
				}
				ppxlcPrevU += m_iFrameWidthUV;
				ppxlcPrevV += m_iFrameWidthUV;
				ppxlcPredMBU += BLOCK_SIZE;
				ppxlcPredMBV += BLOCK_SIZE;
			}
		}
	}
	else {
		const PixelC* ppxlcPrevUBot; 
		const PixelC* ppxlcPrevVBot; 
		if (!(xRefUV & 1)) {  //!bXSubPxl&& bYSubPxl
			for (iy = 0; iy < BLOCK_SIZE; iy++) {
				ppxlcPrevUBot = ppxlcPrevU + m_iFrameWidthUV;            //UPln -> pixels (xInt,yInt+1);
				ppxlcPrevVBot = ppxlcPrevV + m_iFrameWidthUV;            //VPln -> pixels (xInt,yInt+1);
				for (ix = 0; ix < BLOCK_SIZE; ix++) {
//					ppxlcPredMBU [ix] = (ppxlcPrevU [ix] + ppxlcPrevUBot [ix] + 1) >> 1;
//					ppxlcPredMBV [ix] = (ppxlcPrevV [ix] + ppxlcPrevVBot [ix] + 1) >> 1;
					ppxlcPredMBU [ix] = (ppxlcPrevU [ix] + ppxlcPrevUBot [ix] + 1 - iRoundingControl) >> 1;
					ppxlcPredMBV [ix] = (ppxlcPrevV [ix] + ppxlcPrevVBot [ix] + 1 - iRoundingControl) >> 1;
				}
				ppxlcPredMBU += BLOCK_SIZE;
				ppxlcPredMBV += BLOCK_SIZE;
				ppxlcPrevU = ppxlcPrevUBot; 
				ppxlcPrevV = ppxlcPrevVBot; 
			}
		}
		else { // bXSubPxl && bYSubPxl
			for (iy = 0; iy < BLOCK_SIZE; iy++){
				ppxlcPrevUBot = ppxlcPrevU + m_iFrameWidthUV; //UPln -> pixels (xInt,yInt+1);
				ppxlcPrevVBot = ppxlcPrevV + m_iFrameWidthUV; //VPln -> pixels (xInt,yInt+1);
				for (ix = 0; ix < BLOCK_SIZE; ix++){
/*
					ppxlcPredMBU [ix] = (
						ppxlcPrevU [ix + 1] + ppxlcPrevU [ix] + 
						ppxlcPrevUBot [ix + 1] + ppxlcPrevUBot [ix] + 2
					) >> 2;
					ppxlcPredMBV [ix] = (
						ppxlcPrevV [ix + 1] + ppxlcPrevV [ix] + 
						ppxlcPrevVBot [ix + 1] + ppxlcPrevVBot [ix] + 2
					) >> 2;
*/
					ppxlcPredMBU [ix] = (
						ppxlcPrevU [ix + 1] + ppxlcPrevU [ix] + 
						ppxlcPrevUBot [ix + 1] + ppxlcPrevUBot [ix] + 2 - iRoundingControl
					) >> 2;
					ppxlcPredMBV [ix] = (
						ppxlcPrevV [ix + 1] + ppxlcPrevV [ix] + 
						ppxlcPrevVBot [ix + 1] + ppxlcPrevVBot [ix] + 2 - iRoundingControl
					) >> 2;
				}
				ppxlcPredMBU += BLOCK_SIZE;
				ppxlcPredMBV += BLOCK_SIZE;
				ppxlcPrevU = ppxlcPrevUBot; 
				ppxlcPrevV = ppxlcPrevVBot; 
			}
		}
	}
}

UInt gOvrlpPredY [64];

Void CVideoObject::motionCompOverLap (
	PixelC* ppxlcPredMB, 
	const PixelC* ppxlcRefLeftTop,
	const CMotionVector* pmv, // motion vector
	const CMBMode* pmbmd, // macroblk mode	
	Int imbx, // current macroblk index
	Int imby, // current macroblk index
	CoordI x, // current coordinate system
	CoordI y, // current coordinate system
	CRct *prctMVLimit
)
{
	// Overlap Motion Comp use motion vector of current blk and motion vectors of neighboring blks.
	const CMotionVector *pmvC,*pmvT,*pmvB,*pmvR,*pmvL; // MVs of Cur, Top, Bot, Right and Left Blocks.
	const CMotionVector *pmvCurrMb,*pmvTopMb,*pmvRightMb,*pmvLeftMb; // MVs of Cur, Top, Right and Left MacroBlocks.
	const CMBMode *pmbmdTopMb, *pmbmdRightMb, *pmbmdLeftMb; // MVs of Cur, Top, Right and Left MacroBlocks.
	Bool bIntraT, bIntraR, bIntraL; // flags of 4MV for Cur, Top, Right and Left MacroBlocks.
	Bool bLeftBndry, bRightBndry, bTopBndry;
	bLeftBndry = (imbx == 0);
	bRightBndry = (imbx == m_iNumMBX - 1);
	bTopBndry = (imby == 0);

	pmvCurrMb = pmv;
	// assign the neighboring blk's MVs to pmv[TBRLC]
	if (!bTopBndry) {
		pmbmdTopMb = pmbmd - m_iNumMBX; 
		bIntraT = (pmbmdTopMb->m_dctMd == INTRA || pmbmdTopMb->m_dctMd == INTRAQ);
		pmvTopMb = pmv - m_iNumOfTotalMVPerRow;
	}
	if (!bLeftBndry) {
		pmbmdLeftMb = pmbmd - 1;
		bIntraL = (pmbmdLeftMb->m_dctMd == INTRA || pmbmdLeftMb->m_dctMd == INTRAQ);
		pmvLeftMb = pmv - 5;
	}
	if (!bRightBndry) {
		pmbmdRightMb = pmbmd + 1;
		bIntraR = (pmbmdRightMb->m_dctMd == INTRA || pmbmdRightMb->m_dctMd == INTRAQ);
		pmvRightMb = pmv + 5;
	}

	UInt i;
	// assign the neighboring blk's MVs to pmv[TBRLC] 
	for (i = 1; i < 5; i++) {
		if (pmbmd->m_rgTranspStatus [i] == ALL)
			continue;
		pmvC = pmvCurrMb + i;
		switch (i) {
			case 1:
				if (pmbmd->m_rgTranspStatus [3] == ALL)
					pmvB = pmvCurrMb + 1;
				else
					pmvB = pmvCurrMb + 3;
				
				if (pmbmd->m_rgTranspStatus [2] == ALL)
					pmvR = pmvCurrMb + 1; 	
				else
					pmvR = pmvCurrMb + 2; 	

				if (bTopBndry || bIntraT || pmbmdTopMb->m_rgTranspStatus [3] == ALL)
					pmvT = pmvCurrMb + 1;
				else
					pmvT = pmvTopMb + 3;

				if (bLeftBndry || bIntraL || pmbmdLeftMb->m_rgTranspStatus [2] == ALL)
					pmvL = pmvCurrMb + 1;
				else
					pmvL = pmvLeftMb + 2;
				break;
			case 2:
				if (pmbmd->m_rgTranspStatus [4] == ALL)
					pmvB = pmvCurrMb + 2;
				else
					pmvB = pmvCurrMb + 4;

				if (pmbmd->m_rgTranspStatus [1] == ALL)
					pmvL = pmvCurrMb + 2;
				else
					pmvL = pmvCurrMb + 1;

				if (bTopBndry || bIntraT || pmbmdTopMb->m_rgTranspStatus [4] == ALL)
					pmvT = pmvCurrMb + 2;
				else
					pmvT = pmvTopMb + 4;      	

				if (bRightBndry || bIntraR || pmbmdRightMb->m_rgTranspStatus [1] == ALL)
					pmvR = pmvCurrMb + 2;
				else
					pmvR = pmvRightMb + 1;
				break;
			case 3:
				if (pmbmd->m_rgTranspStatus [1] == ALL)
					pmvT = pmvCurrMb + 3;  
				else
					pmvT = pmvCurrMb + 1;
				
				pmvB = pmvCurrMb + 3; // use the current mv

				if (pmbmd->m_rgTranspStatus [4] == ALL)
					pmvR = pmvCurrMb + 3;
				else
					pmvR = pmvCurrMb + 4;

				if (bLeftBndry || bIntraL || pmbmdLeftMb->m_rgTranspStatus [4] == ALL)
					pmvL = pmvCurrMb + 3;
				else
					pmvL = pmvLeftMb + 4;
				break;
			case 4:
				if (pmbmd->m_rgTranspStatus [2] == ALL)
					pmvT = pmvCurrMb + 4;    	
				else
					pmvT = pmvCurrMb + 2;
				
				pmvB = pmvCurrMb + 4;

				if (pmbmd->m_rgTranspStatus [3] == ALL)
					pmvL = pmvCurrMb + 4;  	
				else
					pmvL = pmvCurrMb + 3;
				
				if (bRightBndry || bIntraR || pmbmdRightMb->m_rgTranspStatus [3] == ALL)
					pmvR = pmvCurrMb + 4;
				else
					pmvR = pmvRightMb + 3;
				break;
			default:
				assert (FALSE);
			}
		// Compute the top left corner's x,y coordinates of current block.
		UInt dxc = (((i - 1) & 1) << 3);
		UInt dyc = (((i - 1) & 2) << 2);
		UInt nxcY = x + dxc;
		UInt nycY = y + dyc;
		// Compute the corresponding positions on Ref frm, using 5 MVs.
		CoordI xRefC = ((nxcY + pmvC -> iMVX) << 1) + pmvC -> iHalfX, yRefC = ((nycY + pmvC -> iMVY) << 1) + pmvC -> iHalfY;
		CoordI xRefT = ((nxcY + pmvT -> iMVX) << 1) + pmvT -> iHalfX, yRefT = ((nycY + pmvT -> iMVY) << 1) + pmvT -> iHalfY;
		CoordI xRefB = ((nxcY + pmvB -> iMVX) << 1) + pmvB -> iHalfX, yRefB = ((nycY + pmvB -> iMVY) << 1) + pmvB -> iHalfY;
		CoordI xRefR = ((nxcY + pmvR -> iMVX) << 1) + pmvR -> iHalfX, yRefR = ((nycY + pmvR -> iMVY) << 1) + pmvR -> iHalfY;
		CoordI xRefL = ((nxcY + pmvL -> iMVX) << 1) + pmvL -> iHalfX, yRefL = ((nycY + pmvL -> iMVY) << 1) + pmvL -> iHalfY;

		limitMVRangeToExtendedBBHalfPel (xRefC,yRefC,prctMVLimit,BLOCK_SIZE);
		limitMVRangeToExtendedBBHalfPel (xRefT,yRefT,prctMVLimit,BLOCK_SIZE);
		limitMVRangeToExtendedBBHalfPel (xRefB,yRefB,prctMVLimit,BLOCK_SIZE);
		limitMVRangeToExtendedBBHalfPel (xRefR,yRefR,prctMVLimit,BLOCK_SIZE);
		limitMVRangeToExtendedBBHalfPel (xRefL,yRefL,prctMVLimit,BLOCK_SIZE);

		Bool bXSubPxlC = (xRefC & 1), bYSubPxlC = (yRefC & 1),
			 bXSubPxlT = (xRefT & 1), bYSubPxlT = (yRefT & 1),
			 bXSubPxlB = (xRefB & 1), bYSubPxlB = (yRefB & 1),
			 bXSubPxlR = (xRefR & 1), bYSubPxlR = (yRefR & 1),
			 bXSubPxlL = (xRefL & 1), bYSubPxlL = (yRefL & 1);
		
		// 5 starting pos. in Zoomed Ref. Frames
		const PixelC* ppxliPrevYC = ppxlcRefLeftTop + ((yRefC >> 1) + EXPANDY_REF_FRAME) * m_iFrameWidthY + (xRefC >> 1) + EXPANDY_REF_FRAME;
		const PixelC* ppxliPrevYT = ppxlcRefLeftTop + ((yRefT >> 1) + EXPANDY_REF_FRAME) * m_iFrameWidthY + (xRefT >> 1) + EXPANDY_REF_FRAME;
		const PixelC* ppxliPrevYB = ppxlcRefLeftTop + ((yRefB >> 1) + EXPANDY_REF_FRAME) * m_iFrameWidthY + (xRefB >> 1) + EXPANDY_REF_FRAME;
		const PixelC* ppxliPrevYR = ppxlcRefLeftTop + ((yRefR >> 1) + EXPANDY_REF_FRAME) * m_iFrameWidthY + (xRefR >> 1) + EXPANDY_REF_FRAME;
		const PixelC* ppxliPrevYL = ppxlcRefLeftTop + ((yRefL >> 1) + EXPANDY_REF_FRAME) * m_iFrameWidthY + (xRefL >> 1) + EXPANDY_REF_FRAME;
		UInt *pWghtC, *pWghtT, *pWghtB, *pWghtR, *pWghtL;
		pWghtC = (UInt*) gWghtC;
		pWghtT = (UInt*) gWghtT;
		pWghtB = (UInt*) gWghtB + 32;
		pWghtR = (UInt*) gWghtR;
		pWghtL = (UInt*) gWghtL;
		UInt* uiOvrlpPredY = gOvrlpPredY;

		if (bXSubPxlC && bYSubPxlC)
			bilnrMCVH (uiOvrlpPredY, ppxliPrevYC, pWghtC, 0, 8, 0, 8, FALSE);
		else if (bXSubPxlC && !bYSubPxlC)
			bilnrMCH (uiOvrlpPredY, ppxliPrevYC, pWghtC, 0, 8, 0, 8, FALSE);
		else if (!bXSubPxlC && bYSubPxlC)
			bilnrMCV (uiOvrlpPredY, ppxliPrevYC, pWghtC, 0, 8, 0, 8, FALSE);
		else // (!bXSubPxlC && !bYSubPxlC)
			bilnrMC (uiOvrlpPredY, ppxliPrevYC, pWghtC, 0, 8, 0, 8, FALSE);

		if (bXSubPxlT && bYSubPxlT)
			bilnrMCVH (uiOvrlpPredY, ppxliPrevYT, pWghtT, 0, 8, 0, 4, TRUE);
		else if (bXSubPxlT && !bYSubPxlT)
			bilnrMCH (uiOvrlpPredY, ppxliPrevYT, pWghtT, 0, 8, 0, 4, TRUE);
		else if (!bXSubPxlT && bYSubPxlT)
			bilnrMCV (uiOvrlpPredY, ppxliPrevYT, pWghtT, 0, 8, 0, 4, TRUE);
		else // (!bXSubPxlT && !bYSubPxlT)
			bilnrMC (uiOvrlpPredY, ppxliPrevYT, pWghtT, 0, 8, 0, 4, TRUE);

		if (bXSubPxlB && bYSubPxlB)
			bilnrMCVH (&uiOvrlpPredY [32], ppxliPrevYB + (m_iFrameWidthY << 2), pWghtB, 0, 8, 4, 8, TRUE);
		else if (bXSubPxlB && !bYSubPxlB)
			bilnrMCH (&uiOvrlpPredY [32], ppxliPrevYB + (m_iFrameWidthY << 2), pWghtB, 0, 8, 4, 8, TRUE);
		else if (!bXSubPxlB && bYSubPxlB)
			bilnrMCV (&uiOvrlpPredY [32], ppxliPrevYB + (m_iFrameWidthY << 2), pWghtB, 0, 8, 4, 8, TRUE);
		else // (!bXSubPxlB && !bYSubPxlB)
			bilnrMC (&uiOvrlpPredY [32], ppxliPrevYB + (m_iFrameWidthY << 2), pWghtB, 0, 8, 4, 8, TRUE);

		if (bXSubPxlR && bYSubPxlR)
			bilnrMCVH (uiOvrlpPredY, ppxliPrevYR, pWghtR, 4, 8, 0, 8, TRUE);
		else if (bXSubPxlR && !bYSubPxlR)
			bilnrMCH (uiOvrlpPredY, ppxliPrevYR, pWghtR, 4, 8, 0, 8, TRUE);
		else if (!bXSubPxlR && bYSubPxlR)
			bilnrMCV (uiOvrlpPredY, ppxliPrevYR, pWghtR, 4, 8, 0, 8, TRUE);
		else // (!bXSubPxlR && !bYSubPxlR)
			bilnrMC (uiOvrlpPredY, ppxliPrevYR, pWghtR, 4, 8, 0, 8, TRUE);

		if (bXSubPxlL && bYSubPxlL)
			bilnrMCVH (uiOvrlpPredY, ppxliPrevYL, pWghtL, 0, 4, 0, 8, TRUE);
		else if (bXSubPxlL && !bYSubPxlL)
			bilnrMCH (uiOvrlpPredY, ppxliPrevYL, pWghtL, 0, 4, 0, 8, TRUE);
		else if (!bXSubPxlL && bYSubPxlL)
			bilnrMCV (uiOvrlpPredY, ppxliPrevYL, pWghtL, 0, 4, 0, 8, TRUE);
		else // (!bXSubPxlL && !bYSubPxlL)
			bilnrMC (uiOvrlpPredY, ppxliPrevYL, pWghtL, 0, 4, 0, 8, TRUE);

		PixelC* ppxliPredY = ppxlcPredMB + dxc + dyc * MB_SIZE;  // Starting of Pred. Frame
		for (UInt iy = 0; iy < BLOCK_SIZE; iy++){
			for (UInt ix = 0; ix < BLOCK_SIZE; ix++)
				*ppxliPredY++= (*uiOvrlpPredY++ + 4) >> 3;
			ppxliPredY += BLOCK_SIZE;
		}
	}
}

Void CVideoObject::bilnrMCVH (UInt* PredY, const PixelC* ppxliPrevYC, UInt* pMWght, UInt xlow, UInt xhigh, UInt ylow, UInt yhigh, Bool bAdd)
{
	const PixelC* ppxliPrevYCBot = ppxliPrevYC + m_iFrameWidthY;
	if (bAdd)
		for (UInt iy = ylow; iy < yhigh; iy++) {
			for (UInt ix = xlow; ix < xhigh; ix++)
//				PredY [ix] += ((UInt) (ppxliPrevYC [ix] + ppxliPrevYC [ix + 1] + ppxliPrevYCBot [ix] + ppxliPrevYCBot[ix+1] + 2) >> 2) * pMWght [ix];
				PredY [ix] += ((UInt) (ppxliPrevYC [ix] + ppxliPrevYC [ix + 1] + ppxliPrevYCBot [ix] + ppxliPrevYCBot[ix+1] + 2 - m_vopmd.iRoundingControl) >> 2) * pMWght [ix];
			ppxliPrevYC += m_iFrameWidthY;
			ppxliPrevYCBot += m_iFrameWidthY;
			PredY += BLOCK_SIZE;
			pMWght += BLOCK_SIZE;
		}
	else
		for (UInt iy = ylow; iy < yhigh; iy++) {
			for (UInt ix = xlow; ix < xhigh; ix++)
//				PredY [ix] = ((UInt) (ppxliPrevYC [ix] + ppxliPrevYC [ix + 1] + ppxliPrevYCBot [ix] + ppxliPrevYCBot[ix+1] + 2) >> 2) * pMWght [ix];
				PredY [ix] = ((UInt) (ppxliPrevYC [ix] + ppxliPrevYC [ix + 1] + ppxliPrevYCBot [ix] + ppxliPrevYCBot[ix+1] + 2 - m_vopmd.iRoundingControl) >> 2) * pMWght [ix];
			ppxliPrevYC += m_iFrameWidthY;
			ppxliPrevYCBot += m_iFrameWidthY;
			PredY += BLOCK_SIZE;
			pMWght += BLOCK_SIZE;
		}
}

Void CVideoObject::bilnrMCV (UInt* PredY, const PixelC* ppxliPrevYC, UInt* pMWght, UInt xlow, UInt xhigh, UInt ylow, UInt yhigh, Bool bAdd)
{
	const PixelC* ppxliPrevYCBot = ppxliPrevYC + m_iFrameWidthY;
	if (bAdd)
		for (UInt iy = ylow; iy < yhigh; iy++) {
			for (UInt ix = xlow; ix < xhigh; ix++)
//				PredY [ix] += ((UInt) (ppxliPrevYC [ix] + ppxliPrevYCBot [ix] + 1) >> 1) * pMWght [ix];
 				PredY [ix] += ((UInt) (ppxliPrevYC [ix] + ppxliPrevYCBot [ix] + 1 - m_vopmd.iRoundingControl) >> 1) * pMWght [ix];
			ppxliPrevYC += m_iFrameWidthY;
			ppxliPrevYCBot += m_iFrameWidthY;
			PredY += BLOCK_SIZE;
			pMWght += BLOCK_SIZE;
		}
	else
		for (UInt iy = ylow; iy < yhigh; iy++) {
			for (UInt ix = xlow; ix < xhigh; ix++)
//				PredY [ix] = ((UInt) (ppxliPrevYC [ix] + ppxliPrevYCBot [ix] + 1) >> 1) * pMWght [ix];
				PredY [ix] = ((UInt) (ppxliPrevYC [ix] + ppxliPrevYCBot [ix] + 1 - m_vopmd.iRoundingControl) >> 1) * pMWght [ix];
			ppxliPrevYC += m_iFrameWidthY;
			ppxliPrevYCBot += m_iFrameWidthY;
			PredY += BLOCK_SIZE;
			pMWght += BLOCK_SIZE;
		}
}

Void CVideoObject::bilnrMCH (UInt* PredY, const PixelC* ppxliPrevYC, UInt* pMWght, UInt xlow, UInt xhigh, UInt ylow, UInt yhigh, Bool bAdd)
{
	//PixelC* ppxliPrevYCBot = ppxliPrevYC + m_iFrameWidthY;
	if (bAdd)
		for (UInt iy = ylow; iy < yhigh; iy++) {
			for (UInt ix = xlow; ix < xhigh; ix++)
//				PredY [ix] += ((UInt) (ppxliPrevYC [ix] + ppxliPrevYC [ix + 1] + 1) >> 1) * pMWght [ix];
				PredY [ix] += ((UInt) (ppxliPrevYC [ix] + ppxliPrevYC [ix + 1] + 1 - m_vopmd.iRoundingControl) >> 1) * pMWght [ix];
			ppxliPrevYC += m_iFrameWidthY;
			PredY += BLOCK_SIZE;
			pMWght += BLOCK_SIZE;
		}
	else
		for (UInt iy = ylow; iy < yhigh; iy++) {
			for (UInt ix = xlow; ix < xhigh; ix++)
//				PredY [ix] = ((UInt) (ppxliPrevYC [ix] + ppxliPrevYC [ix + 1] + 1) >> 1) * pMWght [ix];
				PredY [ix] = ((UInt) (ppxliPrevYC [ix] + ppxliPrevYC [ix + 1] + 1 - m_vopmd.iRoundingControl) >> 1) * pMWght [ix];
			ppxliPrevYC += m_iFrameWidthY;
			PredY += BLOCK_SIZE;
			pMWght += BLOCK_SIZE;
		}
}

Void CVideoObject::bilnrMC (UInt* PredY, const PixelC* ppxliPrevYC, UInt* pMWght, UInt xlow, UInt xhigh, UInt ylow, UInt yhigh, Bool bAdd)
{
	//PixelC* ppxliPrevYCBot = ppxliPrevYC + m_iFrameWidthY;
	if (bAdd)
		for (UInt iy = ylow; iy < yhigh; iy++) {
			for (UInt ix = xlow; ix < xhigh; ix++)
				PredY [ix] += (UInt) ppxliPrevYC [ix] * pMWght[ix];
			ppxliPrevYC += m_iFrameWidthY;
			PredY += BLOCK_SIZE;
			pMWght += BLOCK_SIZE;
		}
	else
		for (UInt iy = ylow; iy < yhigh; iy++) {
			for (UInt ix = xlow; ix < xhigh; ix++)
				PredY [ix] = (UInt) ppxliPrevYC [ix] * pMWght [ix];
			ppxliPrevYC += m_iFrameWidthY;
			PredY += BLOCK_SIZE;
			pMWght += BLOCK_SIZE;
		}
}

Void CVideoObject::motionCompBY (
	PixelC* ppxlcPred, // can be either Y or A
	const PixelC* ppxlcRefLeftTop,
	CoordI iXRef, CoordI iYRef // x + mvX in full pel unit
)
{
	CoordI iY;
	const PixelC* ppxlcRef = ppxlcRefLeftTop + 
							(iYRef + EXPANDY_REF_FRAME) * m_iFrameWidthY + iXRef + EXPANDY_REF_FRAME;
	Int iLeftBound = max(0,m_rctRefVOPY0.left - iXRef);
	Int iRightBound = max(0,m_rctRefVOPY0.right - iXRef);
	Int iTopBound = max(0,m_rctRefVOPY0.top - iYRef);
	Int iBottomBound = max(0,m_rctRefVOPY0.bottom - iYRef);
	iLeftBound = min(MC_BAB_SIZE,iLeftBound);
	iRightBound = min(MC_BAB_SIZE,iRightBound);
	iTopBound = min(MC_BAB_SIZE,iTopBound);
	iBottomBound = min(MC_BAB_SIZE,iBottomBound);

	Int iHeightMax = iBottomBound-iTopBound;
	Int iWidthMax  = iRightBound-iLeftBound;

	if (iHeightMax == MC_BAB_SIZE && iWidthMax == MC_BAB_SIZE)	{
		for (iY = 0; iY < MC_BAB_SIZE; iY++) {
			memcpy (ppxlcPred, ppxlcRef, MC_BAB_SIZE);
			ppxlcRef  += m_iFrameWidthY;
			ppxlcPred += MC_BAB_SIZE;
		}
	} else if(iWidthMax == 0 || iHeightMax==0)
		for (iY = 0; iY < MC_BAB_SIZE; iY++) {
			memset (ppxlcPred, 0, MC_BAB_SIZE);
			ppxlcPred += MC_BAB_SIZE;
		}
	else {
		for (iY = 0; iY < MC_BAB_SIZE; iY++) {
			if(iY<iTopBound || iY>=iBottomBound)
				memset (ppxlcPred, 0, MC_BAB_SIZE);  // clear row
			else
			{
				if(iLeftBound>0)
					memset (ppxlcPred, 0, iLeftBound); // clear left hand span
				if(iRightBound<MC_BAB_SIZE)
					memset (ppxlcPred+iRightBound, 0, MC_BAB_SIZE-iRightBound); // right span
				memcpy (ppxlcPred+iLeftBound, ppxlcRef+iLeftBound, iWidthMax); // copy middle region
			}

			ppxlcRef += m_iFrameWidthY;
			ppxlcPred += MC_BAB_SIZE;
		}
	}
}

Void CVideoObject::copyFromRefToCurrQ (
	const CVOPU8YUVBA* pvopcRef, // reference VOP
	CoordI x, CoordI y, 
	PixelC* ppxlcCurrQMBY, PixelC* ppxlcCurrQMBU, PixelC* ppxlcCurrQMBV,
	CRct *prctMVLimit
)
{
	// needs limiting to reference area bounding box
	limitMVRangeToExtendedBBFullPel(x,y,prctMVLimit,MB_SIZE);
	
	Int iOffsetY = (y + EXPANDY_REF_FRAME) * m_iFrameWidthY + x + EXPANDY_REF_FRAME;
	Int iOffsetUV = (y / 2 + EXPANDUV_REF_FRAME) * m_iFrameWidthUV + x / 2 + EXPANDUV_REF_FRAME;
	const PixelC* ppxlcRefMBY = pvopcRef->pixelsY () + iOffsetY;
	const PixelC* ppxlcRefMBU = pvopcRef->pixelsU () + iOffsetUV;
	const PixelC* ppxlcRefMBV = pvopcRef->pixelsV () + iOffsetUV;

	CoordI iY;
	for (iY = 0; iY < BLOCK_SIZE; iY++) {
		memcpy (ppxlcCurrQMBY, ppxlcRefMBY, MB_SIZE);
		memcpy (ppxlcCurrQMBU, ppxlcRefMBU, BLOCK_SIZE);
		memcpy (ppxlcCurrQMBV, ppxlcRefMBV, BLOCK_SIZE);

		ppxlcCurrQMBY += m_iFrameWidthY; ppxlcRefMBY += m_iFrameWidthY;
		ppxlcCurrQMBU += m_iFrameWidthUV; ppxlcRefMBU += m_iFrameWidthUV;
		ppxlcCurrQMBV += m_iFrameWidthUV; ppxlcRefMBV += m_iFrameWidthUV;

		memcpy (ppxlcCurrQMBY, ppxlcRefMBY, MB_SIZE);
		ppxlcCurrQMBY += m_iFrameWidthY; ppxlcRefMBY += m_iFrameWidthY;
	}
}

Void CVideoObject::copyFromRefToCurrQ_WithShape (
	const CVOPU8YUVBA* pvopcRef, // reference VOP
	CoordI x, CoordI y, 
	PixelC* ppxlcCurrQMBY, PixelC* ppxlcCurrQMBU, PixelC* ppxlcCurrQMBV, PixelC* ppxlcCurrQMBBY
)
{
	Int iOffsetY = (y + EXPANDY_REF_FRAME) * m_iFrameWidthY + x + EXPANDY_REF_FRAME;
	Int iOffsetUV = (y / 2 + EXPANDUV_REF_FRAME) * m_iFrameWidthUV + x / 2 + EXPANDUV_REF_FRAME;
	const PixelC* ppxlcRefMBY = pvopcRef->pixelsY () + iOffsetY;
	const PixelC* ppxlcRefMBBY = pvopcRef->pixelsBY () + iOffsetY;
	const PixelC* ppxlcRefMBU = pvopcRef->pixelsU () + iOffsetUV;
	const PixelC* ppxlcRefMBV = pvopcRef->pixelsV () + iOffsetUV;

	CoordI iY;
	for (iY = 0; iY < BLOCK_SIZE; iY++) {
		memcpy (ppxlcCurrQMBY, ppxlcRefMBY, MB_SIZE);
		memcpy (ppxlcCurrQMBBY, ppxlcRefMBBY, MB_SIZE);
		memcpy (ppxlcCurrQMBU, ppxlcRefMBU, BLOCK_SIZE);
		memcpy (ppxlcCurrQMBV, ppxlcRefMBV, BLOCK_SIZE);

		ppxlcCurrQMBY += m_iFrameWidthY; ppxlcRefMBY += m_iFrameWidthY;
		ppxlcCurrQMBBY += m_iFrameWidthY; ppxlcRefMBBY += m_iFrameWidthY;
		ppxlcCurrQMBU += m_iFrameWidthUV; ppxlcRefMBU += m_iFrameWidthUV;
		ppxlcCurrQMBV += m_iFrameWidthUV; ppxlcRefMBV += m_iFrameWidthUV;

		memcpy (ppxlcCurrQMBY, ppxlcRefMBY, MB_SIZE);
		ppxlcCurrQMBY += m_iFrameWidthY; ppxlcRefMBY += m_iFrameWidthY;
		memcpy (ppxlcCurrQMBBY, ppxlcRefMBBY, MB_SIZE);
		ppxlcCurrQMBBY += m_iFrameWidthY; ppxlcRefMBBY += m_iFrameWidthY;
	}
}

Void CVideoObject::copyAlphaFromRefToCurrQ (
	CoordI x, CoordI y, 
	PixelC* ppxlcCurrQMBA
)
{
	Int iOffsetY = (y + EXPANDY_REF_FRAME) * m_iFrameWidthY + x + EXPANDY_REF_FRAME;
	const PixelC* ppxlcRefMBA = m_pvopcRefQ0->pixelsA () + iOffsetY;

	CoordI iY;
	for (iY = 0; iY < MB_SIZE; iY++) {
		memcpy (ppxlcCurrQMBA, ppxlcRefMBA, MB_SIZE);
		ppxlcCurrQMBA += m_iFrameWidthY; 
		ppxlcRefMBA += m_iFrameWidthY;
	}
}

