/*************************************************************************

This software module was originally developed by 

	Ming-Chieh Lee (mingcl@microsoft.com), Microsoft Corporation
	Wei-ge Chen (wchen@microsoft.com), Microsoft Corporation
	Bruce Lin (blin@microsoft.com), Microsoft Corporation
	Chuang Gu (chuanggu@microsoft.com), Microsoft Corporation
	(date: March, 1996)

in the course of development of the MPEG-4 Video (ISO/IEC 14496-2). 
This software module is an implementation of a part of one or more MPEG-4 Video tools 
as specified by the MPEG-4 Video. 
ISO/IEC gives users of the MPEG-4 Video free license to this software module or modifications 
thereof for use in hardware or software products claiming conformance to the MPEG-4 Video. 
Those intending to use this software module in hardware or software products are advised that its use may infringe existing patents. 
The original developer of this software module and his/her company, 
the subsequent editors and their companies, 
and ISO/IEC have no liability for use of this software module or modifications thereof in an implementation. 
Copyright is not released for non MPEG-4 Video conforming products. 
Microsoft retains full right to use the code for his/her own purpose, 
assign or donate the code to a third party and to inhibit third parties from using the code for non <MPEG standard> conforming products. 
This copyright notice must be included in all copies or derivative works. 

Copyright (c) 1996, 1997.

Module Name:

	mode.h

Abstract:

	basic coding modes for VO, VOL, VOP, MB	and RD

Revision History:
	Sept. 29, 1997: add Video Packet, data partition, RVLC by Toshiba

*************************************************************************/

#ifndef __MODE_H_
#define __MODE_H_
Class CMBMode;
Class CDirectModeData;
Class CStatistics;
typedef enum {BASE_LAYER, ENHN_LAYER} VOLtype; // will be generlized later
typedef enum {INTRA, INTRAQ, INTER, INTERQ} DCTMode; // define pixel component
typedef enum {DIRECT, INTERPOLATE, BACKWARD, FORWARD} MBType; // define MB type
typedef enum {UNKNOWN_DIR, HORIZONTAL, VERTICAL, DIAGONAL} Direction;	
typedef enum {ALL_TRANSP, ALL_OPAQUE, INTRA_CAE, INTER_CAE_MVDZ, INTER_CAE_MVDNZ, MVDZ_NOUPDT, MVDNZ_NOUPDT, UNKNOWN} ShapeMode;
typedef enum {UNTRANSMITTED, TRANSMITTED, UPDATED, FINISHED} MBSptMode; // MB sprite mode
typedef Direction IntraPredDirection;								//for readability
typedef Direction CAEScanDirection;								//for readability

#ifndef NOT_IN_TABLE
#define NOT_IN_TABLE -1
#endif
#ifndef	TCOEF_ESCAPE
#define TCOEF_ESCAPE 102							// see table.13/H.263
#endif

// VM 5.1 Rate Control
#define RC_START_RATE_CONTROL	1
#define RC_MAX_SLIDING_WINDOW	20
#define RC_PAST_PERCENT  		0.05
#define RC_SAFETY_MARGIN 		0.10
#define RC_SKIP_MARGIN 			80
#define RC_MAX_Q_INCREASE		0.25
#define RC_MAX_QUANT 			31
#define RC_MIN_QUANT 			1

typedef struct MVInfo					 // for motion vector coding
{
	UInt uiRange;						 // search range
	UInt uiFCode;						 // f-code 
	UInt uiScaleFactor;					 // scale factor
} MVInfo;


typedef struct VOLMode					 // VideoObjectLayer Mode
{
	// type of VOL
	VOLtype		volType;				 // what type of VOL
	Int			iEnhnUpFactor;			 // upsample the baselayer by this factor. 1 if base layer

	// VDS: view-dependent scalability enable
	Bool bvds;

	// time info
	Int			iClockRate;				 //rate of clock used to count vops in Hz

	// shape info
	AlphaUsage	fAUsage;				 //binary or gray level alpha; or no alpha (rectangle VO)
	Bool		bShapeOnly;				 // code only the shape
	Int			iBinaryAlphaTH;			 //binary shaperounding parameter
	Bool		bNoCrChange;			 //nobinary shape size conversion
	
	// motion info
	Bool	bOriginalForME;			// flag indicating whether use the original previous VOP for ME
	UInt	uiWarpingAccuracy;		// indicates the quantization accuracy of motion vector for sprite warping
	Bool	bAdvPredDisable;		// No OBMC, (8x8 in the future).

	Bool	bSeprMotionTexture;		// TRUE = seperate motion and texture coding
	Bool	bErrorResilientDisable;	// error resilient coding is disabled or not
									// affects mv coding; disable intra prediction; only for single VOP case
	Bool	bVPBitTh;				// Bit threshold for video packet spacing control
	Bool	bDataPartitioning;		// data partitioning
	Bool	bReversibleVlc;			// reversible VLC

	// texture coding info
	Quantizer fQuantizer;			// either H.263 or MPEG
	Bool bIntraPredictionDisable;	// inter AC/DC prediction
	Bool bLoadIntraMatrix;			// flag indicating whether to load intra Q-matrix
	Int rgiIntraQuantizerMatrix [BLOCK_SQUARE_SIZE]; // Intra Q-Matrix
	Bool bLoadInterMatrix;			// flag indicating whether to load inter Q-matrix
	Int rgiInterQuantizerMatrix [BLOCK_SQUARE_SIZE]; // Inter Q-Matrix
	Bool bLoadIntraMatrixAlpha;			// flag indicating whether to load intra Q-matrix
	Int rgiIntraQuantizerMatrixAlpha [BLOCK_SQUARE_SIZE];
	Bool bLoadInterMatrixAlpha;			// flag indicating whether to load inter Q-matrix
	Int	rgiInterQuantizerMatrixAlpha [BLOCK_SQUARE_SIZE];
	Bool bDeblockFilterDisable;		// apply deblocking filter or not.
	Bool bNoGrayQuantUpdate;		// decouple gray quant and dquant
	EntropyCodeType fEntropyType;	// Entropy code type
	
	// frame rate info
	Int iTemporalRate;				//no. of input frames between two encoded VOP's assuming 30Hz input
	Int iPbetweenI;
	Int iBbetweenP;
	//Int iPeriodOfIVOP;				//in temporally sumsampled domain
	//Int iPeriodOfPVOP;				//in temporally sumsampled domain
	
	// scalability info
	Int iEnhnType;					//enhancement type

	// statistics dumping options
	Bool bDumpMB;			// dump statitstics at MB level
	Bool bTrace;			// dumping trace file
} VOLMode;

typedef struct VOPMode // VideoObjectPlane Mode
{
	// user specify, per VOP
	Int intStepI;	// I-VOP stepsize for DCT
	Int intStep;	// P-VOP stepsize for DCT
	Int intStepB;	// stepsize for B-VOP
	Int intStepIAlpha;	// I-VOP stepsize for DCT
	Int intStepAlpha;	// P-VOP stepsize for DCT
	Int intStepBAlpha;	// stepsize for B-VOP
	Int intStepDiff;	// stepsize difference for updating for DCT (DQUANT)
	Int intDBQuant;
	VOPpredType vopPredType; //whether IVOP, PVOP, BVOP, or Sprite
	Int iIntraDcSwitchThr;	 //threshold to code intraDC as with AC VLCs
	Int iRoundingControl;	 //rounding control

	// motion search info
	MVInfo	mvInfoForward;					// motion search info
	MVInfo	mvInfoBackward;					// motion search info
	Int	iSearchRangeForward;				// maximum search range for motion estimation
	Int	iSearchRangeBackward;				// maximum search range for motion estimation

	// for scalability
	Int iRefSelectCode;
	Int iLoadForShape; // load_forward_shape
	Int iLoadBakShape; // load_backward_shape

} VOPMode;



Class CMBMode // MacroBlock Mode
{
public:
	// Constructors
	~CMBMode ();
	CMBMode (); 
	CMBMode (const CMBMode& md);

	// Operations
	Void setCodedBlockPattern (BlockNum blkn, Bool bisCoded) 
								{m_rgbCodedBlockPattern[(UInt) blkn - 1] = bisCoded;}
	Void setCodedBlockPattern (const Bool* rgbblockNum);
	Void setMinError (BlockNum blkn, Int iminError)
								{m_rgfltMinErrors[(UInt) blkn - 1] = (Float) iminError;}
	Void setMinError (const Float* pfltminError);
	Void operator = (const CMBMode& md);

	// Attributes
	Bool getCodedBlockPattern (BlockNum blkn) const 
								{return m_rgbCodedBlockPattern[(UInt) blkn - 1];};
	Bool* getCodedBlockPattern () const {return m_rgbCodedBlockPattern;}
	Float getMinError (BlockNum blkn) const
									{return m_rgfltMinErrors[(UInt) blkn - 1];};
	Float* getMinError () const {return m_rgfltMinErrors;}

	// Some extra data member
	TransparentStatus m_rgTranspStatus [11]; 
		// indicating the transparency status of the MB, either ALL, PARTIAL, or NONE transparent
		// 0: 16x16,  1-4: 8x8
	Int m_rgNumNonTranspPixels [11]; // number of non-transparent pixels
	Bool m_bPadded;				// to see whether this all-transparent has been padded
	Bool m_bSkip;				// is the Macroblock skiped. = COD in H.263
	MBType m_mbType;			// macroblock type, DIRECT, FORWARD, BACKWARD, or INTERPOLATE
	DCTMode m_dctMd;			// is the Macroblock inter- or intra- coded
	ShapeMode m_shpmd;			//different context for the first MMR code
	Int m_intStepDelta;			// change of quantization stepsize = DQUANT in h.263
	Bool m_bhas4MVForward;		//whether the MB has four motion vectors (for forward vectors)
	Bool m_bhas4MVBackward;		//whether the MB has four motion vectors (for backward vectors)
	Bool m_bPerspectiveForward; //whether the MB uses forward perspective motion
	Bool m_bPerspectiveBackward;//whether the MB uses backward perspective motion
	Int m_stepSize;				//qp for texture
	Int m_stepSizeAlpha;		//qp for alpha
	Bool m_bSkipA;				// 0: not skipped, 1: all opaque but not colocated MB, 2: use colocated
	IntraPredDirection m_preddir [10]; // horizonal or vertical
	Bool m_bACPrediction;		// use AC prediction or not
	Bool m_bInterShapeCoding;	//use predicted binary shape
	Bool m_bIntraOrSkip;		//temporary flag used only in separate s-m-t mode; (NO_MV = 0; before NO_DCT, could be either skip or intra)
	Bool m_bCodeDcAsAc;			//code Intra DC with Ac VLC
	Bool m_bColocatedMBSkip;	// for B-VOP, colocated MB skipped or not
	Int  m_iVideoPacketNumber;	//	Video Packet Number; added by Toshiba

private:
	Bool* m_rgbCodedBlockPattern; //for each block, 1 = some non-DC components are coded
	Float* m_rgfltMinErrors; //mininal prediction errors for each luminance block
};


Class CDirectModeData // to handle data for direct mode in B-VOP
{
public:
	// Constructor
	~CDirectModeData ();
	CDirectModeData ();

	// Attributes
	UInt numMB () const {return m_uiNumMB;}
	UInt numMBX () const {return m_uiNumMBX;}
	UInt numMBY () const {return m_uiNumMBY;}
	Bool inBound (UInt iMbIdx) const; // check whether the index is inbound
	Bool inBound (UInt idX, UInt idY) const; // check whether the index is inbound
	CMBMode** mbMode () const {return m_ppmbmd;}
	const CMBMode* mbMode (UInt iMbIdx) const 
		{assert (inBound (iMbIdx)); return m_ppmbmd [iMbIdx];}
	const CMBMode* mbMode (UInt idX, UInt idY) const 
		{assert (inBound (idX, idY)); return m_ppmbmd [idX + idY * m_uiNumMBX];}
	CMotionVector** mv () const {return m_prgmv;}
	const CMotionVector* mv (UInt iMbIdx) const 
		{assert (inBound (iMbIdx)); return m_prgmv [iMbIdx];}
	const CMotionVector* mv (UInt idX, UInt idY) const 
		{assert (inBound (idX, idY)); return m_prgmv [idX + idY * m_uiNumMBX];}

	// Operations
	Void reassign (UInt numMBX, UInt numMBY);
	Void assign (UInt imb, const CMBMode& mbmd, const CMotionVector* rgmv);

///////////////// implementation /////////////////

private:
	own CMBMode** m_ppmbmd;
	own CMotionVector** m_prgmv; // [m_uiNumMB][5]
	UInt m_uiNumMB, m_uiNumMBX, m_uiNumMBY;

	Void destroyMem ();

};

Class CStatistics
{
public:

	//constructor 
	CStatistics ();
	~CStatistics () {};

	//resultant
	Void print (Bool bVOPPrint = FALSE);
	CStatistics& operator = (const CStatistics& statSrc);

	//Operation
	Void operator += (const CStatistics& statSrc);
	Void reset ();
	UInt total ();
	UInt head ();

	//data members
	UInt nBitsHead;
	UInt nBitsY;
	UInt nBitsCr;
	UInt nBitsCb;
	UInt nBitsA;
	UInt nBitsShapeMode;
	UInt nBitsCOD;
	UInt nBitsCBPY;
	UInt nBitsMCBPC;
	UInt nBitsDQUANT;
	UInt nBitsMODB;
	UInt nBitsCBPB;
	UInt nBitsMBTYPE;
	UInt nBitsIntraPred;	//intra ac/dc switch
	UInt nBitsNoDCT;		//no. of DCT in sepr m-s-t mode
	UInt nBitsCODA;
	UInt nBitsCBPA;
	UInt nBitsMODBA;
	UInt nBitsCBPBA;
	UInt nBitsStuffing;
	UInt nSkipMB;
	UInt nInterMB;
	UInt nInter4VMB;
	UInt nIntraMB;
	UInt nDirectMB;
	UInt nForwardMB;
	UInt nBackwardMB;
	UInt nInterpolateMB;
	UInt nBitsMV;
	UInt nBitsShape;
	UInt nBitsTotal;
	Double dSNRY;
	Double dSNRU;
	Double dSNRV;
	Double dSNRA;

private:
	UInt nBitsTexture;
};

Class CRCMode
{
public:

	//constructor 
	CRCMode () {};
	~CRCMode () {};

	//resultant
	UInt updateQuanStepsize ();	// Target bits and quantization level calculation
	Bool skipNextFrame () const {return m_skipNextFrame;}
	Bool firstFrame () const {return m_bfirstFrame;}
	UInt noCodedFrame () const {return m_Nc;}  // return the coded P frames

	//Operation
	Void resetSkipMode () {m_skipNextFrame = FALSE; m_bfirstFrame = TRUE;}
	Void resetFirstFrame () {m_bfirstFrame = FALSE;}
	Bool skipThisFrame ();
	Void reset (UInt uiFirstFrame, UInt uiLastFrame, UInt uiTemporalRate, 
				UInt uiBufferSize, Double mad, UInt uiBitsFirstFrame);
	Void setMad (Double mad) {m_Ep = m_Ec; m_Ec = mad;}
	Void setQc (UInt QStep) {m_Qc = QStep;}
	Void updateRCModel (UInt uiBitsTotalCurr, UInt uiBitsHeadCurr);		// Update RD model

private:

	Void RCModelEstimator (UInt nWindowSize);	// Rate Control: RD model estimator

	Double m_X1;// 1st order coefficient
	Double m_X2;// 2nd order coefficient
	UInt m_Rs;	// bit rate for sequence. e.g. 24000 bits/sec	
	UInt m_Rf;	// bits used for the first frame, e.g. 10000 bits
	UInt m_Rc;	// bits used for the current frame after encoding
	UInt m_Rp;	// bits to be removed from the buffer per picture
	Double m_Ts;// number of seconds for the sequence, e.g. 10 sec
	Double m_Ec;// mean absolute difference for the current frame after motion compensation
	Double m_Ep;// mean absolute difference for the previous frame after motion compensation
	UInt m_Qc;	// quantization level used for the current frame
	UInt m_Qp;	// quantization level used for the previous frame
	UInt m_Nr;	// number of P frames remaining for encoding
	UInt m_Nc;	// number of P frames coded
	UInt m_Ns;	// distance between encoded frames
	Int m_Rr;	// number of bits remaining for encoding this sequence 
	UInt m_T;	// target bit to be used for the current frame
	UInt m_S;	// number of bits used for encoding the previous frame
	UInt m_Hc;	// header and motion vector bits used in the current frame
	UInt m_Hp;	// header and motion vector bits used in the previous frame
	UInt m_Bs;	// buffer size 
	Int m_B;	// current buffer level
	Bool m_skipNextFrame;						// TRUE if buffer is full
	Bool m_bfirstFrame;							// TRUE if this is the first frame
	UInt m_rgQp[RC_MAX_SLIDING_WINDOW];			// quantization levels for the past frames
	Double m_rgRp[RC_MAX_SLIDING_WINDOW];		// scaled encoding complexity used for the past frames;
	Bool m_rgRejected[RC_MAX_SLIDING_WINDOW];	// outliers
};

#endif	//__MODE_H

