/******************************************************************************

This software module was originally developed by
        Wei Wu (weiwu@stallion.risc.rockwell.com),  Rockwell Science Center
        Homer Chen (homer@jupiter.risc.rockwell.com), Rockwell Science Center
in the course of development of the MPEG-4 Video (ISO/IEC 14496-2).
This software module is an implementation of a part of one or more MPEG-4 Video
tools as specified by the MPEG-4 Video. ISO/IEC gives users of the MPEG-4 Video
free license to this software module or modifications thereof for use in
hardware or software products claiming conformance to the MPEG-4 Video. Those
intending to use this software module in hardware or software products are advised
that its use may infringe existing patents. The original developer of this software
module and his/her company, the subsequent editors and their companies, and ISO/IEC
have no liability for use of this software module or modifications thereof in an
implementation. Copyright is not released for non MPEG-4 Video conforming
products. Rockwell Science Center retains full right to use the code for his/her own
purpose, assign or donate the code to a third party and to inhibit third parties from
using the code for non <MPEG standard> conforming products. This copyright notice
must be included in all copies or derivative works. Copyright (c) 1997.

******************************************************************************/

#include <stdio.h>

#include "typeapi.h"
#include "codehead.h"
#include "mode.h"
#include "dct.h"
#include "cae.h"
#include "vopses.h"

#ifdef __MFC_
#ifdef _DEBUG
#undef THIS_FILE
static char BASED_CODE THIS_FILE[] = __FILE__;
#endif  

#define new DEBUG_NEW
#endif // __MFC_

// VDS
Void CVideoObject::shrink(double alpha, double beta, double dist, double Rr,
                        int nx, int ny, CU8Image *allmask, CU8Image *mask)
{
        long int ix, iy; 
        long int xoffset = nx*8;
        long int yoffset = ny*8;
        double thrX, thrY;

        thrX = beta*8/dist*Rr;
        thrY = alpha*8/dist*Rr;

        if(thrX>8) thrX = 8;
        if(thrY>8) thrY = 8;

        for (iy=0; iy<thrY; iy++) {
                for (ix=0; ix<thrX; ix++) {
                        if (allmask->pixel(ix+xoffset, iy+yoffset)==0) {
                                allmask->pixel(ix+xoffset, iy+yoffset, (PixelC) 255);
                                mask->pixel(ix, iy, (PixelC) 255);
                        }
                }
        }
}

// VDS
Void CVideoObject::shrink_UV(int nx, int ny, CU8Image **mask, CU8Image *allmask)
{
	long int ix, iy;
	long int xoffset = nx*8;
	long int yoffset = ny*8;

	for (iy=0; iy<8; iy++) {
		for(ix=0; ix<8; ix++) {
			if(allmask->pixel(ix+xoffset, iy+yoffset)==0 && 
			   (mask[0]->pixel(ix, iy)!=0 ||
			    mask[1]->pixel(ix, iy)!=0 ||
			    mask[2]->pixel(ix, iy)!=0 ||
			    mask[3]->pixel(ix, iy)!=0)) {
				allmask->pixel(ix+xoffset, iy+yoffset, (PixelC) 255);
				mask[4]->pixel(ix, iy, (PixelC) 255);
				mask[5]->pixel(ix, iy, (PixelC) 255);
			}
		}
	}
}
