/*************************************************************************

This software module was originally developed by 

	Chuang Gu (chuanggu@microsoft.com), Microsoft Corporation
	(date: Auguest, 1997)

in the course of development of the MPEG-4 Video (ISO/IEC 14496-2). 
This software module is an implementation of a part of one or more MPEG-4 Video tools 
as specified by the MPEG-4 Video. 
ISO/IEC gives users of the MPEG-4 Video free license to this software module or modifications 
thereof for use in hardware or software products claiming conformance to the MPEG-4 Video. 
Those intending to use this software module in hardware or software products are advised that its use may infringe existing patents. 
The original developer of this software module and his/her company, 
the subsequent editors and their companies, 
and ISO/IEC have no liability for use of this software module or modifications thereof in an implementation. 
Copyright is not released for non MPEG-4 Video conforming products. 
Microsoft retains full right to use the code for his/her own purpose, 
assign or donate the code to a third party and to inhibit third parties from using the code for non <MPEG standard> conforming products. 
This copyright notice must be included in all copies or derivative works. 

Copyright (c) 1996, 1997.


Module Name:

	spt.cpp

Abstract:

	Functions for sprite warping 

Revision History:

*************************************************************************/

#include <stdio.h>
#include <math.h>
#include "typeapi.h"
#include "mode.h"
#include "vopses.h"

#ifdef __MFC_
#ifdef _DEBUG
#undef THIS_FILE
static char BASED_CODE THIS_FILE[] = __FILE__;
#endif

#define new DEBUG_NEW				   
#endif // __MFC_

Void CVideoObject::warpYA (const CPerspective2D& persp, const CRct& rctWarpedBound, UInt accuracy) 
// warp m_pvopcSptQ's Y and A components to m_pvopcCurrQs
{
	assert (m_pvopcCurrQ -> whereY ().includes (rctWarpedBound));
	const CU8Image* puciCurrY = m_pvopcCurrQ -> getPlane (Y_PLANE);
	const CU8Image* puciCurrA = (m_pvopcSptQ -> fAUsage () == EIGHT_BIT)? m_pvopcCurrQ -> getPlane (A_PLANE) : m_pvopcCurrQ -> getPlane (BY_PLANE);
	const CU8Image* puciSptY = m_pvopcSptQ -> getPlane (Y_PLANE);
	const CU8Image* puciSptA = (m_pvopcSptQ -> fAUsage () == EIGHT_BIT)? m_pvopcSptQ -> getPlane (A_PLANE) : m_pvopcSptQ -> getPlane (BY_PLANE);
	const CRct rctSptY = m_pvopcSptQ -> whereY ();
	const UInt offset = m_pvopcCurrQ -> whereY ().width - rctWarpedBound.width;
	UInt accuracy1 = accuracy + 1;
	PixelC* ppxlcCurrQY = (PixelC*) puciCurrY -> pixels ();
	PixelC* ppxlcCurrQA = (PixelC*) puciCurrA -> pixels ();
	memset (ppxlcCurrQY, 0, puciCurrY -> where ().area ());
	memset (ppxlcCurrQA, 0, puciCurrA -> where ().area ());
	ppxlcCurrQY = (PixelC*) puciCurrY -> pixels (rctWarpedBound.left, rctWarpedBound.top);
	ppxlcCurrQA = (PixelC*) puciCurrA -> pixels (rctWarpedBound.left, rctWarpedBound.top);
	for (CoordI y = rctWarpedBound.top; y < rctWarpedBound.bottom; y++) {
		for (CoordI x = rctWarpedBound.left; x < rctWarpedBound.right; x++) {
			CSite src = persp * (CSite (x, y)); 
			CoordD dx = (CoordD)src.x / (1 << accuracy1);
			CoordD dy = (CoordD)src.y / (1 << accuracy1);
			CoordI fx = (CoordI) floor (dx);
			CoordI fy = (CoordI) floor (dy); 
			CoordI cx = (CoordI) ceil (dx);
			CoordI cy = (CoordI) ceil (dy);
			if (rctSptY.includes (fx, fy) && rctSptY.includes (fx, cy) && rctSptY.includes (cx, fy) && rctSptY.includes (cx, cy)) {
				PixelC pxlcWarpedA = puciSptA -> pixel (src, accuracy);
				if (pxlcWarpedA >= 128) {
					*ppxlcCurrQA = OPAQUE;
					*ppxlcCurrQY = puciSptY -> pixel (src, accuracy);
				}
			}
			ppxlcCurrQY++;	
			ppxlcCurrQA++;
		}
		ppxlcCurrQY += offset;
		ppxlcCurrQA += offset;
	}
}		 

Void CVideoObject::warpUV (const CPerspective2D& persp, const CRct& rctWarpedBound, UInt accuracy) 
// warp m_pvopcSptQ's U and V components to m_pvopcCurrQs
{
	assert (m_pvopcCurrQ -> whereUV ().includes (rctWarpedBound));
	const CU8Image* puciCurrU = m_pvopcCurrQ -> getPlane (U_PLANE);
	const CU8Image* puciCurrV = m_pvopcCurrQ -> getPlane (V_PLANE);
	const CU8Image* puciCurrBY = m_pvopcCurrQ -> getPlane (BY_PLANE);
	const CU8Image* puciSptU = m_pvopcSptQ -> getPlane (U_PLANE);
	const CU8Image* puciSptV = m_pvopcSptQ -> getPlane (V_PLANE);
	const CRct rctSptUV = m_pvopcSptQ -> whereUV ();
	const UInt offsetUV = m_pvopcCurrQ -> whereUV ().width - rctWarpedBound.width;
	const UInt offsetBY = 2 * (m_pvopcCurrQ -> whereY ().width -  rctWarpedBound.width);
	UInt accuracy1 = accuracy + 1;
	PixelC* ppxlcCurrQU = (PixelC*) puciCurrU -> pixels ();
	PixelC* ppxlcCurrQV = (PixelC*) puciCurrV -> pixels ();
	memset (ppxlcCurrQU, 0, puciCurrU -> where ().area ());
	memset (ppxlcCurrQV, 0, puciCurrV -> where ().area ());
	ppxlcCurrQU = (PixelC*) puciCurrU -> pixels (rctWarpedBound.left, rctWarpedBound.top);
	ppxlcCurrQV = (PixelC*) puciCurrV -> pixels (rctWarpedBound.left, rctWarpedBound.top);
	PixelC* ppxlcCurrQBY = (PixelC*) puciCurrBY -> pixels (2 * rctWarpedBound.left, 2 * rctWarpedBound.top);
	PixelC* ppxlcCurrQBYNextRow = ppxlcCurrQBY + puciCurrBY -> where ().width;
	for (CoordI y = rctWarpedBound.top; y < rctWarpedBound.bottom; y++) {
		for (CoordI x = rctWarpedBound.left; x < rctWarpedBound.right; x++) {
			CSite src = persp * (CSite (x, y)); 
			CoordD dx = (CoordD)src.x / (1 << accuracy1);
			CoordD dy = (CoordD)src.y / (1 << accuracy1);
			CoordI fx = (CoordI) floor (dx);
			CoordI fy = (CoordI) floor (dy); 
			CoordI cx = (CoordI) ceil (dx);
			CoordI cy = (CoordI) ceil (dy);
			if (rctSptUV.includes (fx, fy) && rctSptUV.includes (fx, cy) && rctSptUV.includes (cx, fy) && rctSptUV.includes (cx, cy)) {
				if (*ppxlcCurrQBY | *(ppxlcCurrQBY + 1) | *ppxlcCurrQBYNextRow | *(ppxlcCurrQBYNextRow+1)) {
					*ppxlcCurrQU = puciSptU -> pixel (src, accuracy);
					*ppxlcCurrQV = puciSptV -> pixel (src, accuracy);
				}
			}
			ppxlcCurrQBY += 2;
			ppxlcCurrQBYNextRow += 2;
			ppxlcCurrQU++;	
			ppxlcCurrQV++;
		}
		ppxlcCurrQBY += offsetBY;
		ppxlcCurrQBYNextRow += offsetBY;
		ppxlcCurrQU += offsetUV;
		ppxlcCurrQV += offsetUV;
	}
}		 
