/*************************************************************************

This software module was originally developed by 

	Ming-Chieh Lee (mingcl@microsoft.com), Microsoft Corporation
	Wei-ge Chen (wchen@microsoft.com), Microsoft Corporation
	Bruce Lin (blin@microsoft.com), Microsoft Corporation
	Chuang Gu (chuanggu@microsoft.com), Microsoft Corporation
	(date: March, 1996)

in the course of development of the MPEG-4 Video (ISO/IEC 14496-2). 
This software module is an implementation of a part of one or more MPEG-4 Video tools 
as specified by the MPEG-4 Video. 
ISO/IEC gives users of the MPEG-4 Video free license to this software module or modifications 
thereof for use in hardware or software products claiming conformance to the MPEG-4 Video. 
Those intending to use this software module in hardware or software products are advised that its use may infringe existing patents. 
The original developer of this software module and his/her company, 
the subsequent editors and their companies, 
and ISO/IEC have no liability for use of this software module or modifications thereof in an implementation. 
Copyright is not released for non MPEG-4 Video conforming products. 
Microsoft retains full right to use the code for his/her own purpose, 
assign or donate the code to a third party and to inhibit third parties from using the code for non <MPEG standard> conforming products. 
This copyright notice must be included in all copies or derivative works. 

Copyright (c) 1996, 1997.

Module Name:

	vopSes.cpp

Abstract:

	Base class for the encoder for one VOP session.

Revision History:

*************************************************************************/

#include <stdio.h>

#include "typeapi.h"
#include "codehead.h"
#include "mode.h"
#include "dct.h"
#include "cae.h"
#include "vopses.h"

#ifdef __MFC_
#ifdef _DEBUG
#undef THIS_FILE
static char BASED_CODE THIS_FILE[] = __FILE__;
#endif

#define new DEBUG_NEW				   
#endif // __MFC_

CVideoObject::~CVideoObject ()
{
	delete m_pvopcCurrQ;
	delete m_pvopcRefQ0;
	delete m_pvopcRefQ1;

	delete m_pvopcCurrMB;
	delete m_pvopcPredMB;
	delete m_pvopiErrorMB;
	delete [] m_rgmbmd;
	delete [] m_rgmv;
	delete [] m_rgmvBackward;
	delete [] m_rgmvBY;

	delete [] m_rgmvRef;
	delete m_pidct;

	Int nBlk = (m_volmd.fAUsage == EIGHT_BIT) ? 10 : 6;
	Int iBlk;
	for (iBlk = 0; iBlk < nBlk; iBlk++)
		delete [] m_rgpiCoefQ [iBlk];
	delete [] m_rgpiCoefQ;
//	delete m_pvopfCurrFilteredQ;
//	delete m_pdrtmdRef1;

	delete [] m_rgiQPpred;

	if (m_volmd.fAUsage == RECTANGLE) {
		Int nBlk = (m_volmd.fAUsage == EIGHT_BIT) ? 10 : 6;
		delete [] m_rgblkmCurrMB;
		for (Int iMB = 0; iMB < m_iNumMBX; iMB++)	{
			for (iBlk = 0; iBlk < nBlk; iBlk++)	{
				delete [] (m_rgpmbmAbove [iMB]->rgblkm) [iBlk];
				delete [] (m_rgpmbmCurr  [iMB]->rgblkm) [iBlk];
			}
			delete [] m_rgpmbmAbove [iMB]->rgblkm;
			delete m_rgpmbmAbove [iMB];
			delete [] m_rgpmbmCurr [iMB]->rgblkm;
			delete m_rgpmbmCurr [iMB];
		}
		delete [] m_rgpmbmAbove;
		delete [] m_rgpmbmCurr;
	}

	// sprite
	if (m_uiSprite == 1) {
		delete [] m_rgstSrcQ;
		delete [] m_rgstDstQ;
		delete m_pvopcSptQ;
	}
	delete m_pbEmptyRowArray;

	m_rgiClipTab -= 400;
	delete [] m_rgiClipTab;
	m_rgiClipTab = NULL;

	// shape
	if(m_rgshpmd!=NULL)
		delete m_rgshpmd;
	delete m_puciPredBAB;
	delete m_parcodec;
	delete [] m_ppxlcCurrMBBYDown4;
	delete [] m_ppxlcCurrMBBYDown2;
	delete [] m_ppxlcReconCurrBAB;
	delete [] m_ppxlcPredBABDown2;
	delete [] m_ppxlcPredBABDown4;
}

CVideoObject::CVideoObject () :
	m_t (0), m_iBCount (0), m_tPastRef(0), m_tFutureRef(0),
	m_rgmbmd (NULL), m_rgmv (NULL), m_rgmvBackward (NULL), m_rgmvBY (NULL),
	m_rgmbmdRef (NULL), m_rgmvRef (NULL),
	m_tModuloBaseDecd (0), m_tModuloBaseDisp (0),
	m_pvopcCurrQ (NULL), 
//	m_pvopfCurrFilteredQ (NULL),
	m_pvopcRefQ0 (NULL), m_pvopcRefQ1 (NULL), 
	m_puciPredBAB (NULL), m_ppxlcReconCurrBAB (NULL), m_parcodec (NULL),
	m_ppxlcCurrMBBYDown4 (NULL),
	m_ppxlcCurrMBBYDown2 (NULL),
	m_ppxlcPredBABDown4 (NULL),
	m_ppxlcPredBABDown2 (NULL),
	m_pvopcSptQ (NULL), m_uiSprite (0), m_rgstSrcQ (NULL), m_rgstDstQ (NULL),
	m_iOffsetForPadY (0), m_iOffsetForPadUV (0),
	m_bGOV_Reset (TRUE),
	m_rgshpmd(NULL)
//	m_iNumOfPnts (-1), m_uiSprite (0),
//	m_rgstSrcQ (NULL), m_rgstDstQ (NULL), m_pvopfSptQ (NULL)
{
	m_pbEmptyRowArray = new Bool [MB_SIZE];

	m_rgiClipTab = new U8 [1024];
	m_rgiClipTab += 400;
	Int i;
	for (i = -400; i < 624; i++)
		m_rgiClipTab [i] = (i < 0) ? 0 : (i > 255) ? 255 : i;

}

Void CVideoObject::allocateVOLMembers (Int iSessionWidth, Int iSessionHeight)
{
	m_pvopcCurrMB = new CVOPU8YUVBA (m_volmd.fAUsage, CRct (0, 0, MB_SIZE, MB_SIZE));
	m_ppxlcCurrMBY = (PixelC*) m_pvopcCurrMB->pixelsY ();
	m_ppxlcCurrMBU = (PixelC*) m_pvopcCurrMB->pixelsU ();
	m_ppxlcCurrMBV = (PixelC*) m_pvopcCurrMB->pixelsV ();
	m_ppxlcCurrMBBY = (PixelC*) m_pvopcCurrMB->pixelsBY ();
	m_ppxlcCurrMBBUV = (PixelC*) m_pvopcCurrMB->pixelsBUV ();
	m_ppxlcCurrMBA = (PixelC*) m_pvopcCurrMB->pixelsA ();

	m_pvopcPredMB = new CVOPU8YUVBA (m_volmd.fAUsage, CRct (0, 0, MB_SIZE, MB_SIZE));
	m_ppxlcPredMBY = (PixelC*) m_pvopcPredMB->pixelsY ();
	m_ppxlcPredMBU = (PixelC*) m_pvopcPredMB->pixelsU ();
	m_ppxlcPredMBV = (PixelC*) m_pvopcPredMB->pixelsV ();
	m_ppxlcPredMBA = (PixelC*) m_pvopcPredMB->pixelsA ();
	
	// B-VOP MB buffer data
	m_pvopcPredMBBack = new CVOPU8YUVBA (m_volmd.fAUsage, CRct (0, 0, MB_SIZE, MB_SIZE));
	m_ppxlcPredMBBackY = (PixelC*) m_pvopcPredMBBack->pixelsY ();
	m_ppxlcPredMBBackU = (PixelC*) m_pvopcPredMBBack->pixelsU ();
	m_ppxlcPredMBBackV = (PixelC*) m_pvopcPredMBBack->pixelsV ();
	m_ppxlcPredMBBackA = (PixelC*) m_pvopcPredMBBack->pixelsA ();

	m_pvopiErrorMB = new CVOPIntYUVBA (m_volmd.fAUsage, CRct (0, 0, MB_SIZE, MB_SIZE));
	m_ppxliErrorMBY = (PixelI*) m_pvopiErrorMB->getPlane (Y_PLANE)->pixels ();
	m_ppxliErrorMBU = (PixelI*) m_pvopiErrorMB->getPlane (U_PLANE)->pixels ();
	m_ppxliErrorMBV = (PixelI*) m_pvopiErrorMB->getPlane (V_PLANE)->pixels ();
	m_ppxliErrorMBA = (PixelI*) m_pvopiErrorMB->getPlane (A_PLANE)->pixels ();

	m_pvopcCurrQ = new CVOPU8YUVBA (m_volmd.fAUsage, m_rctRefFrameY);
	assert (m_pvopcCurrQ != NULL);
	m_pvopcRefQ0 = new CVOPU8YUVBA (m_volmd.fAUsage, m_rctRefFrameY);
	assert (m_pvopcRefQ0 != NULL);
	m_pvopcRefQ1 = new CVOPU8YUVBA (m_volmd.fAUsage, m_rctRefFrameY);
	assert (m_pvopcRefQ1 != NULL);

	m_iFrameWidthY = m_pvopcRefQ0->whereY ().width;
	m_iFrameWidthUV = m_pvopcRefQ0->whereUV ().width;
	m_iFrameWidthYxMBSize = MB_SIZE * m_pvopcRefQ0->whereY ().width; 
	m_iFrameWidthYxBlkSize = BLOCK_SIZE * m_pvopcRefQ0->whereY ().width; 
	m_iFrameWidthUVxBlkSize = BLOCK_SIZE * m_pvopcRefQ0->whereUV ().width;

	// MB data	
	Int nBlk = (m_volmd.fAUsage == EIGHT_BIT) ? 10 : 6;
	m_rgpiCoefQ = new Int* [nBlk];
	m_rgiQPpred = new Int [nBlk];
	Int iBlk;
	for (iBlk = 0; iBlk < nBlk; iBlk++)
		m_rgpiCoefQ [iBlk] = new Int [BLOCK_SQUARE_SIZE];
	m_pidct = new CInvBlockDCT;

	// motion vectors and MBMode
	Int iNumMBX = iSessionWidth / MB_SIZE; 
	if (iSessionWidth  % MB_SIZE != 0)				//round up
		iNumMBX++;
	Int iNumMBY = iSessionHeight / MB_SIZE;
	if (iSessionHeight % MB_SIZE != 0)				//deal with frational MB
		iNumMBY++;
	Int iNumMB = iNumMBX * iNumMBY;

	m_rgmbmd = new CMBMode [iNumMB];
	m_rgmv = new CMotionVector [iNumMB * 5];
	m_rgmvBackward = new CMotionVector [iNumMB * 5];
	m_rgmvBY = new CMotionVector [iNumMB]; // for shape
	m_rgmvRef = new CMotionVector [iNumMB * 5];
	m_rgmbmdRef = new CMBMode [iNumMB];

	// shape data
	if (m_volmd.fAUsage != RECTANGLE) {
		m_puciPredBAB = new CU8Image (CRct (0, 0, MC_BAB_SIZE, MC_BAB_SIZE));
		m_ppxlcReconCurrBAB = new PixelC [BAB_SIZE * BAB_SIZE];
		m_parcodec = new ArCodec;
		m_ppxlcCurrMBBYDown4 = new PixelC [8 * 8];
		m_ppxlcCurrMBBYDown2 = new PixelC [12 * 12];
		m_ppxlcPredBABDown4 = new PixelC [6*6];
		m_ppxlcPredBABDown2 = new PixelC [10*10];
	}
}

Void CVideoObject::computeVOLConstMembers ()
{
	m_iOffsetForPadY = m_rctRefFrameY.offset (m_rctCurrVOPY.left, m_rctCurrVOPY.top);
	m_iOffsetForPadUV = m_rctRefFrameUV.offset (m_rctCurrVOPUV.left, m_rctCurrVOPUV.top);
	m_rctPrevNoExpandY = m_rctCurrVOPY;
	m_rctPrevNoExpandUV = m_rctCurrVOPUV;
	m_iVOPWidthY = m_rctCurrVOPY.width;
	m_iVOPWidthUV = m_rctCurrVOPUV.width;
	m_iNumMBX = m_iNumMBXRef = m_iVOPWidthY / MB_SIZE; 
	m_iNumMBY = m_iNumMBYRef = m_rctCurrVOPY.height () / MB_SIZE;
	m_iNumMB = m_iNumMBRef = m_iNumMBX * m_iNumMBY;
	m_iNumOfTotalMVPerRow = 5 * m_iNumMBX;
	setRefStartingPointers ();
	m_pvopcCurrQ->setBoundRct (m_rctCurrVOPY);
	m_pvopcRefQ0->setBoundRct (m_rctRefVOPY0);
	m_pvopcRefQ1->setBoundRct (m_rctRefVOPY1);

	Int iMB, iBlk;
	Int nBlk = (m_volmd.fAUsage == EIGHT_BIT) ? 10 : 6;
	m_rgblkmCurrMB = new BlockMemory [nBlk];

	m_rgpmbmAbove = new MacroBlockMemory* [m_iNumMBX];
	m_rgpmbmCurr  = new MacroBlockMemory* [m_iNumMBX];
	for (iMB = 0; iMB < m_iNumMBX; iMB++)	{
		m_rgpmbmAbove [iMB] = new MacroBlockMemory;
		m_rgpmbmAbove [iMB]->rgblkm = new BlockMemory [nBlk];
		m_rgpmbmCurr  [iMB] = new MacroBlockMemory;
		m_rgpmbmCurr  [iMB]->rgblkm = new BlockMemory [nBlk];
		for (iBlk = 0; iBlk < nBlk; iBlk++)	{
			(m_rgpmbmAbove [iMB]->rgblkm) [iBlk] = new Int [(BLOCK_SIZE << 1) - 1];
			(m_rgpmbmCurr  [iMB]->rgblkm) [iBlk] = new Int [(BLOCK_SIZE << 1) - 1];
		}
	}
}

Void CVideoObject::computeVOPMembers ()
{
	m_iVOPWidthY = m_rctCurrVOPY.width;
	m_iVOPWidthUV = m_rctCurrVOPUV.width;
	m_iNumMBX = m_iVOPWidthY / MB_SIZE; 
	m_iNumMBY = m_rctCurrVOPY.height () / MB_SIZE;
	m_iNumMB = m_iNumMBX * m_iNumMBY;
	m_iNumOfTotalMVPerRow = 5 * m_iNumMBX;

	Int iMB, iBlk;
	Int nBlk = (m_volmd.fAUsage == EIGHT_BIT) ? 10 : 6;
	m_rgblkmCurrMB = new BlockMemory [nBlk];

	m_rgpmbmAbove = new MacroBlockMemory* [m_iNumMBX];
	m_rgpmbmCurr  = new MacroBlockMemory* [m_iNumMBX];
	for (iMB = 0; iMB < m_iNumMBX; iMB++)	{
		m_rgpmbmAbove [iMB] = new MacroBlockMemory;
		m_rgpmbmAbove [iMB]->rgblkm = new BlockMemory [nBlk];
		m_rgpmbmCurr  [iMB] = new MacroBlockMemory;
		m_rgpmbmCurr  [iMB]->rgblkm = new BlockMemory [nBlk];
		for (iBlk = 0; iBlk < nBlk; iBlk++)	{
			// BLOCK_SIZE*2-1 is 15 Ints for dc/ac prediction of coefficients
			(m_rgpmbmAbove [iMB]->rgblkm) [iBlk] = new Int [(BLOCK_SIZE << 1) - 1];
			(m_rgpmbmCurr  [iMB]->rgblkm) [iBlk] = new Int [(BLOCK_SIZE << 1) - 1];
		}
	}
}

#define BINARIZE_THRESHOLD	128

Void CVideoObject::repeatPadYOrA (PixelC* ppxlcOldLeft, CVOPU8YUVBA* pvopcRef)
{
	PixelC* ppxlcLeftTop = ppxlcOldLeft - EXPANDY_REFVOP * m_iFrameWidthY - EXPANDY_REFVOP;
		// point to the left top of the expanded bounding box (+- EXPANDY_REFVOP)
	const PixelC* ppxlcOldRight = ppxlcOldLeft + m_rctPrevNoExpandY.width - 1;
		// point to the right-top of the bounding box
	const PixelC* ppxlcOldTopLn = ppxlcOldLeft - EXPANDY_REFVOP;
		// point to the (left, top) - EXPANDY_REFVOP
	PixelC* ppxlcNewLeft = (PixelC*) ppxlcOldTopLn;
	PixelC* ppxlcNewRight = (PixelC*) ppxlcOldRight + 1;
	
	CoordI y;
	for (y = 0; y < m_rctPrevNoExpandY.height (); y++) { // x-direction interpolation
#ifndef MS_BACKWARD_COMPATIBLE
		memset (ppxlcNewLeft, *ppxlcOldLeft, EXPANDY_REFVOP);
		memset (ppxlcNewRight, *ppxlcOldRight, EXPANDY_REFVOP);
#else
		memset (ppxlcNewLeft, 0, EXPANDY_REFVOP);
		memset (ppxlcNewRight, 0, EXPANDY_REFVOP);
#endif
		
		ppxlcNewLeft += pvopcRef->whereY ().width;		
		ppxlcNewRight += pvopcRef->whereY ().width;
		ppxlcOldLeft += pvopcRef->whereY ().width;
		ppxlcOldRight += pvopcRef->whereY ().width;
	}
	
	const PixelC* ppxlcOldBotLn = ppxlcNewLeft - pvopcRef->whereY ().width;
	for (y = 0; y < EXPANDY_REFVOP; y++) {
#ifndef MS_BACKWARD_COMPATIBLE
//		memcpy (ppxlcLeftTop, ppxlcOldTopLn, pvopcRef->whereBoundY ().width + 2 * EXPANDY_REFVOP);
//		memcpy (ppxlcNewLeft, ppxlcOldBotLn, pvopcRef->whereBoundY ().width + 2 * EXPANDY_REFVOP);
//temporary fix; we don't understand the correct meaning of whereBoundY
		memcpy (ppxlcLeftTop, ppxlcOldTopLn, m_iFrameWidthY - EXPANDY_REFVOP);
		memcpy (ppxlcNewLeft, ppxlcOldBotLn, m_iFrameWidthY - EXPANDY_REFVOP);
#else
		memset (ppxlcLeftTop, 0, pvopcRef->whereBoundY ().width + 2 * EXPANDY_REFVOP);
		memset (ppxlcNewLeft, 0, pvopcRef->whereBoundY ().width + 2 * EXPANDY_REFVOP);
#endif
		ppxlcNewLeft += pvopcRef->whereY ().width;
		ppxlcLeftTop += pvopcRef->whereY ().width;
	}
}

Void CVideoObject::repeatPadUV (CVOPU8YUVBA* pvopcRef)
{
	const PixelC* ppxlcOldLeftU = pvopcRef->pixelsU () + m_iOffsetForPadUV; // point to the left-top of bounding box
	const PixelC* ppxlcOldLeftV = pvopcRef->pixelsV () + m_iOffsetForPadUV; // point to the left-top of bounding box
	PixelC* ppxlcLeftTopU = (PixelC*) ppxlcOldLeftU - EXPANDUV_REFVOP * m_iFrameWidthUV - EXPANDUV_REFVOP;
	PixelC* ppxlcLeftTopV = (PixelC*) ppxlcOldLeftV - EXPANDUV_REFVOP * m_iFrameWidthUV - EXPANDUV_REFVOP;
		// point to the left top of the expanded bounding box (+- EXPANDY_REFVOP)
	const PixelC* ppxlcOldRightU = ppxlcOldLeftU + m_rctPrevNoExpandUV.width - 1;
	const PixelC* ppxlcOldRightV = ppxlcOldLeftV + m_rctPrevNoExpandUV.width - 1;
		// point to the right-top of the bounding box
	const PixelC* ppxlcOldTopLnU = ppxlcOldLeftU - EXPANDUV_REFVOP;
	const PixelC* ppxlcOldTopLnV = ppxlcOldLeftV - EXPANDUV_REFVOP;
		// point to the (left, top) - EXPANDY_REFVOP
	PixelC* ppxlcNewLeftU = (PixelC*) ppxlcOldTopLnU;
	PixelC* ppxlcNewLeftV = (PixelC*) ppxlcOldTopLnV;
	PixelC* ppxlcNewRightU = (PixelC*) ppxlcOldRightU + 1;
	PixelC* ppxlcNewRightV = (PixelC*) ppxlcOldRightV + 1;
	
	CoordI y;
	for (y = 0; y < m_rctPrevNoExpandUV.height (); y++) { // x-direction interpolation
#ifndef MS_BACKWARD_COMPATIBLE
		memset (ppxlcNewLeftU, *ppxlcOldLeftU, EXPANDUV_REFVOP);
		memset (ppxlcNewLeftV, *ppxlcOldLeftV, EXPANDUV_REFVOP);
		memset (ppxlcNewRightU, *ppxlcOldRightU, EXPANDUV_REFVOP);
		memset (ppxlcNewRightV, *ppxlcOldRightV, EXPANDUV_REFVOP);
#else
		memset (ppxlcNewLeftU, 0, EXPANDUV_REFVOP);
		memset (ppxlcNewLeftV, 0, EXPANDUV_REFVOP);
		memset (ppxlcNewRightU, 0, EXPANDUV_REFVOP);
		memset (ppxlcNewRightV, 0, EXPANDUV_REFVOP);
#endif
		
		ppxlcNewLeftU += pvopcRef->whereUV ().width;		
		ppxlcNewLeftV += pvopcRef->whereUV ().width;		
		ppxlcNewRightU += pvopcRef->whereUV ().width;
		ppxlcNewRightV += pvopcRef->whereUV ().width;
		ppxlcOldLeftU += pvopcRef->whereUV ().width;
		ppxlcOldLeftV += pvopcRef->whereUV ().width;
		ppxlcOldRightU += pvopcRef->whereUV ().width;
		ppxlcOldRightV += pvopcRef->whereUV ().width;
	}
	
	const PixelC* ppxlcOldBotLnU = ppxlcNewLeftU - pvopcRef->whereUV ().width;
	const PixelC* ppxlcOldBotLnV = ppxlcNewLeftV - pvopcRef->whereUV ().width;
	for (y = 0; y < EXPANDUV_REFVOP; y++) {
#ifndef MS_BACKWARD_COMPATIBLE
/*
		memcpy (ppxlcLeftTopU, ppxlcOldTopLnU, pvopcRef->whereBoundUV ().width + 2 * EXPANDUV_REFVOP);
		memcpy (ppxlcLeftTopV, ppxlcOldTopLnV, pvopcRef->whereBoundUV ().width + 2 * EXPANDUV_REFVOP);
		memcpy (ppxlcNewLeftU, ppxlcOldBotLnU, pvopcRef->whereBoundUV ().width + 2 * EXPANDUV_REFVOP);
		memcpy (ppxlcNewLeftV, ppxlcOldBotLnV, pvopcRef->whereBoundUV ().width + 2 * EXPANDUV_REFVOP);
*/
//same fix as padding for Y
		memcpy (ppxlcLeftTopU, ppxlcOldTopLnU, m_iFrameWidthUV - EXPANDUV_REFVOP);
		memcpy (ppxlcLeftTopV, ppxlcOldTopLnV, m_iFrameWidthUV -  EXPANDUV_REFVOP);
		memcpy (ppxlcNewLeftU, ppxlcOldBotLnU, m_iFrameWidthUV -  EXPANDUV_REFVOP);
		memcpy (ppxlcNewLeftV, ppxlcOldBotLnV, m_iFrameWidthUV -  EXPANDUV_REFVOP);
#else
		memset (ppxlcLeftTopU, 0, pvopcRef->whereBoundUV ().width + 2 * EXPANDUV_REFVOP);
		memset (ppxlcLeftTopV, 0, pvopcRef->whereBoundUV ().width + 2 * EXPANDUV_REFVOP);
		memset (ppxlcNewLeftU, 0, pvopcRef->whereBoundUV ().width + 2 * EXPANDUV_REFVOP);
		memset (ppxlcNewLeftV, 0, pvopcRef->whereBoundUV ().width + 2 * EXPANDUV_REFVOP);
#endif
		ppxlcNewLeftU += pvopcRef->whereUV ().width;
		ppxlcNewLeftV += pvopcRef->whereUV ().width;
		ppxlcLeftTopU += pvopcRef->whereUV ().width;
		ppxlcLeftTopV += pvopcRef->whereUV ().width;
	}
}

Void CVideoObject::saveShapeMode()
{
	// called after reference frame encode/decode
	if(m_rgshpmd == NULL)
	{
		// first time
		m_iRefShpNumMBX = m_iNumMBX;
		m_iRefShpNumMBY = m_iNumMBY;
		m_rgshpmd = new ShapeMode [m_iNumMB];
	}
	else
	{
		// update if changed
		if(m_iRefShpNumMBX!=m_iNumMBXRef || m_iRefShpNumMBY != m_iNumMBYRef)
		{
			delete [] m_rgshpmd;
			m_rgshpmd = new ShapeMode [m_iNumMBRef];
			m_iRefShpNumMBX = m_iNumMBXRef;
			m_iRefShpNumMBY = m_iNumMBYRef;
		}
		// copy shape mode from previous ref to save
		Int i;
		for (i=0; i<m_iNumMBRef; i++ )
			m_rgshpmd[i] = m_rgmbmdRef[i].m_shpmd;
	}
}

Void CVideoObject::resetBYPlane ()
{
	if (m_vopmd.vopPredType == PVOP || m_vopmd.vopPredType == IVOP) {
		PixelC* ppxlcBY = (PixelC*) m_pvopcRefQ1->pixelsBY ();
		//PixelC* ppxlcY = (PixelC*) m_pvopcRefQ1->pixelsY ();
		//PixelC* ppxlcU = (PixelC*) m_pvopcRefQ1->pixelsU ();
		//PixelC* ppxlcV = (PixelC*) m_pvopcRefQ1->pixelsV ();
		memset (ppxlcBY, 0, m_pvopcRefQ1->whereY().area());
		//memset(ppxlcY,128, m_pvopcRefQ1->whereY().area());
		//memset(ppxlcU,128, m_pvopcRefQ1->whereUV().area());
		//memset(ppxlcV,128, m_pvopcRefQ1->whereUV().area());
	}
	else {
		PixelC* ppxlcBY = (PixelC*) m_pvopcCurrQ->pixelsBY ();
		memset (ppxlcBY, 0, m_pvopcCurrQ->whereY().area());
	}
}

Void CVideoObject::updateAllRefVOPs ()
// perform this after VOP prediction type decided and before encoding
{
	if (m_vopmd.vopPredType != BVOP) {
		m_rctRefVOPY0 = m_rctRefVOPY1;
		swapVOPU8Pointers (m_pvopcRefQ0, m_pvopcRefQ1);
		
		repeatPadYOrA ((PixelC*) m_pvopcRefQ0->pixelsY () + m_iOffsetForPadY, m_pvopcRefQ0);
		repeatPadUV (m_pvopcRefQ0);
			
		//reset by in RefQ1 so that no left-over from last frame
		if (m_volmd.fAUsage != RECTANGLE)	{
			if (m_volmd.fAUsage == EIGHT_BIT)
				repeatPadYOrA ((PixelC*) m_pvopcRefQ0->pixelsA () + m_iOffsetForPadY, m_pvopcRefQ0);
		}

	}
	else if (m_iBCount == 1) { // first B-VOP
		repeatPadYOrA ((PixelC*) m_pvopcRefQ1->pixelsY () + m_iOffsetForPadY, m_pvopcRefQ1);
		repeatPadUV (m_pvopcRefQ1);
		if (m_volmd.fAUsage != RECTANGLE)	{
			if (m_volmd.fAUsage == EIGHT_BIT)
				repeatPadYOrA ((PixelC*) m_pvopcRefQ1->pixelsA () + m_iOffsetForPadY, m_pvopcRefQ1);
		}
	}
}

Void CVideoObject::swapVOPU8Pointers (CVOPU8YUVBA*& pvopc0, CVOPU8YUVBA*& pvopc1)
{
	CVOPU8YUVBA* pvopcTmp = pvopc0;
	pvopc0 = pvopc1;
	pvopc1 = pvopcTmp;
}

Void CVideoObject::swapVOPIntPointers (CVOPIntYUVBA*& pvopi0, CVOPIntYUVBA*& pvopi1)
{
	CVOPIntYUVBA* pvopiTmp = pvopi0;
	pvopi0 = pvopi1;
	pvopi1 = pvopiTmp;
}

Void CVideoObject::setRefStartingPointers ()
{
	m_iStartInRefToCurrRctY = m_rctRefFrameY.offset (m_rctCurrVOPY.left, m_rctCurrVOPY.top);
	m_iStartInRefToCurrRctUV = m_rctRefFrameUV.offset (m_rctCurrVOPUV.left, m_rctCurrVOPUV.top);
}

Void CVideoObject::swapRefQ1toSpt ()
{
	m_pvopcSptQ = m_pvopcRefQ1;
	m_pvopcSptQ -> shift (m_rctSpt.left, m_rctSpt.top); 
	m_pvopcRefQ1 = NULL;
}

Void CVideoObject::changeSizeofCurrQ (CRct rctOrg)
{
	delete m_pvopcCurrQ;
	rctOrg.expand (EXPANDY_REF_FRAME);
	m_pvopcCurrQ = new CVOPU8YUVBA (m_pvopcSptQ -> fAUsage (), rctOrg);
}
