/*************************************************************************

This software module was originally developed by 

	Ming-Chieh Lee (mingcl@microsoft.com), Microsoft Corporation
	Wei-ge Chen (wchen@microsoft.com), Microsoft Corporation
	Bruce Lin (blin@microsoft.com), Microsoft Corporation
	Chuang Gu (chuanggu@microsoft.com), Microsoft Corporation
	(date: March, 1996)

in the course of development of the MPEG-4 Video (ISO/IEC 14496-2). 
This software module is an implementation of a part of one or more MPEG-4 Video tools 
as specified by the MPEG-4 Video. 
ISO/IEC gives users of the MPEG-4 Video free license to this software module or modifications 
thereof for use in hardware or software products claiming conformance to the MPEG-4 Video. 
Those intending to use this software module in hardware or software products are advised that its use may infringe existing patents. 
The original developer of this software module and his/her company, 
the subsequent editors and their companies, 
and ISO/IEC have no liability for use of this software module or modifications thereof in an implementation. 
Copyright is not released for non MPEG-4 Video conforming products. 
Microsoft retains full right to use the code for his/her own purpose, 
assign or donate the code to a third party and to inhibit third parties from using the code for non <MPEG standard> conforming products. 
This copyright notice must be included in all copies or derivative works. 

Copyright (c) 1996, 1997.

Module Name:

	vopSes.hpp

Abstract:

	Base class for the encoder for one Video Object.

Revision History:

*************************************************************************/

#ifndef __VOPSES_HPP_ 
#define __VOPSES_HPP_

#define BlockMemory Int*
typedef struct MacroBlockMemory {
	BlockMemory* rgblkm;
} MacroBlockMemory;

Class ArCodec;

Class CInvBlockDCT;

Class CVideoObject
{
public:
	// Constructors
	virtual ~CVideoObject ();
	CVideoObject ();

	// Attributes
	const CVOPU8YUVBA* pvopcQuantCurr () const {return m_pvopcCurrQ;}
//	const CVOPU8YUVBA* pvopfFilteredQuantCurr () const {return m_pvopfCurrFilteredQ;}
	const CVOPU8YUVBA* pvopcRefQPrev () const {return m_pvopcRefQ0;}
	const CVOPU8YUVBA* pvopcRefQLater () const {return m_pvopcRefQ1;}
	const CVOPU8YUVBA* pvopcReconCurr () const {return (m_vopmd.vopPredType == BVOP || (m_vopmd.vopPredType == SPRITE && m_volmd.fAUsage != RECTANGLE)) ? m_pvopcCurrQ : m_pvopcRefQ1;}	//reconstructed current frame
	const VOLMode& volmd () const {return m_volmd;}
	const VOPMode& vopmd () const {return m_vopmd;}
	UInt fSptUsage () const {return m_uiSprite;}
//	Int iNumOfPnts () const {return m_iNumOfPnts;}
//	const CVOPU8YUVBA* pvopfSptQ () const {return m_pvopfSptQ;}

	// Operations
	virtual Void encode () {}
	virtual Int decode () {return 0;}

	Void shrink(double alpha, double beta, double dist, double Rr, int nx, int ny,
                        CU8Image *allmask, CU8Image *mask); // VDS
	Void shrink_UV(int nx, int ny, CU8Image **mask, CU8Image *allmask); // VDS

	Void swapRefQ1toSpt ();
	Void changeSizeofCurrQ (CRct rctOrg);
	Void setRctDisp (CRct rctDisp) {m_rctSptDisp = rctDisp;}

///////////////// implementation /////////////////

protected:
	// general routines
	Void swapVOPU8Pointers (CVOPU8YUVBA*& pvopc0, CVOPU8YUVBA*& pvopc1);
	Void swapVOPIntPointers (CVOPIntYUVBA*& pvopi0, CVOPIntYUVBA*& pvopi1);

	// VOL routines
	Void allocateVOLMembers (Int iSessionWidth, Int iSessionHeight);
	Void computeVOLConstMembers ();

	// Sprite routines
	Void warpYA (const CPerspective2D& persp, const CRct& rctWarpedBound, UInt accuracy);
	Void warpUV (const CPerspective2D& persp, const CRct& rctWarpedBound, UInt accuracy);

	// VOP routines
	Void computeVOPMembers ();
	Void setRefStartingPointers ();
	Void updateAllRefVOPs ();
	Void repeatPadYOrA (PixelC* ppxlcOldLeft, CVOPU8YUVBA* pvopcRef);
	Void repeatPadUV (CVOPU8YUVBA* pvopcRef);
	Void resetBYPlane ();

	// MB routines
	Void decideTransparencyStatus (CMBMode* pmbmd, const PixelC* ppxlcMBBY);
	Void decideMBTransparencyStatus (CMBMode* pmbmd);
	Void downSampleBY (const PixelC* ppxlcMBBY, PixelC* ppxlcMBBUV); // downsample original BY now for LPE padding (using original shape)
	
	// error signal processing
	Void addErrorAndPredToCurrQ (PixelC* ppxlcRefMBY, PixelC* ppxlcRefMBU, PixelC* ppxlcRefMBV);
	Void addAlphaErrorAndPredToCurrQ (PixelC* ppxlcRefMBA);

	// shape coding functions
	Void saveShapeMode();
	own Int* computeShapeSubBlkIndex (Int iSubBlkSize, Int iSrcSize);
	Int getContextUS (PixelC a, PixelC b, PixelC c, PixelC d, PixelC e, PixelC f, PixelC g, PixelC h);
	PixelC getRefValue (const PixelC* ppxlcRow, Int x_adr, Int y_adr, Int h_size, Int v_size);
	Void adaptiveUpSampleShape (const PixelC* rgpxlcSrc, PixelC* rgpxlcDst, Int h_size, Int v_size);
	Void downSampleShapeMCPred(const PixelC* ppxlcSrc,PixelC* ppxlcDst,Int iRate);
	Void upSampleShape (PixelC* ppxlcBYFrm,const PixelC* rgpxlcSrc, PixelC* rgpxlcDst);
	Void copyLeftTopBorderFromVOP (PixelC* ppxlcSrc, PixelC* ppxlcDst);
	Void subsampleLeftTopBorderFromVOP (PixelC* ppxlcSrc, PixelC* ppxlcDst);
	Void makeRightBottomBorder (PixelC* ppxlcSrc, Int iWidth);
	Int contextIntra (const PixelC* ppxlcSrc);
	Int contextIntraTranspose (const PixelC* ppxlcSrc);
	Int contextInter (const PixelC* ppxlcSrc, const PixelC* ppxlcPred);
	Int contextInterTranspose (const PixelC* ppxlcSrc, const PixelC* ppxlcPred);
	Void copyReconShapeToMbAndRef (PixelC* ppxlcDstMB, PixelC* ppxlcRefFrm, PixelC pxlcSrc);
	Void copyReconShapeToMbAndRef (PixelC* ppxlcDstMB, PixelC* ppxlcRefFrm, const PixelC* ppxlcSrc, Int iSrcWidth, Int iBorder);
	Void copyReconShapeUVToRef (PixelC* ppxlcRefFrm, const PixelC* ppxlcSrc);
	CMotionVector findShapeMVP (const CMotionVector* pmv, const CMotionVector* pmvBY, 
								const CMBMode* pmbmd, Int iMBX, Int iMBY) const;
	Int m_iInverseCR;

	// B-VOP stuff
	Void findColocatedMB (Int iMBX, Int iMBY, Bool& bColocatedMBExist, const CMBMode*& pmbmdRef, const CMotionVector*& pmvRef);
	
	// motion compensation
	Void limitMVRangeToExtendedBBHalfPel (CoordI &x,CoordI &y,CRct *prct,Int iBlkSize);
	Void limitMVRangeToExtendedBBFullPel (CoordI &x,CoordI &y,CRct *prct,Int iBlkSize);
	Void motionCompMB (
		PixelC* ppxlcPredMB,
		const PixelC* ppxlcRefLeftTop,
		const CMotionVector* pmv, const CMBMode* pmbmd, 
		Int imbX, Int imbY,
		CoordI x, CoordI y,
		Bool bSkipNonOBMC,
		CRct *prctMVLimit
	);
	Void motionComp (
		PixelC* ppxlcPred, // can be either Y or A
		const PixelC* ppxlcRefLeftTop,
		Int iSize, // either MB or BLOCK size
		CoordI xRef, CoordI yRef, // x + mvX, in half pel unit
		CRct *prctMVLimit
	);
	Void motionCompUV (
		PixelC* ppxlcPredMBU, PixelC* ppxlcPredMBV,
		const CVOPU8YUVBA* pvopcRef,
		CoordI x, CoordI y, 
		CoordI xRefUV, CoordI yRefUV, Int iRoundingControl,
		CRct *prctMVLimit
	);
	Void motionCompOverLap (
		PixelC* ppxlcPredMB, 
		const PixelC* ppxlcRefLeftTop,
		const CMotionVector* pmv, // motion vector
		const CMBMode* pmbmd, // macroblk mode	
		Int imbx, // current macroblk index
		Int imby, // current macroblk index
		CoordI x, // current coordinate system
		CoordI y, // current coordinate system
		CRct *prctMVLimit
	);
	Void motionCompBY (
		PixelC* ppxlcPred,
		const PixelC* ppxlcRefLeftTop,
		CoordI iXRef, CoordI iYRef
	);

	// B-VOP stuff
	Void motionCompInterp (
		const CMotionVector* pmvForward, const CMotionVector* pmvBackward, 
		CoordI x, CoordI y
	);

	Void bilnrMCVH (UInt* PredY, const PixelC* ppxliPrevYC, UInt* pMWght, UInt xlow, UInt xhigh, UInt ylow, UInt yhigh, Bool bAdd);
	Void bilnrMCV (UInt* PredY, const PixelC* ppxliPrevYC, UInt* pMWght, UInt xlow, UInt xhigh, UInt ylow, UInt yhigh, Bool bAdd);
	Void bilnrMCH (UInt* PredY, const PixelC* ppxliPrevYC, UInt* pMWght, UInt xlow, UInt xhigh, UInt ylow, UInt yhigh, Bool bAdd);
	Void bilnrMC (UInt* PredY, const PixelC* ppxliPrevYC, UInt* pMWght, UInt xlow, UInt xhigh, UInt ylow, UInt yhigh, Bool bAdd);
	Void assignPredToCurrQ (PixelC* ppxlcRefMBY, PixelC* ppxlcRefMBU, PixelC* ppxlcRefMBV);
	Void assignAlphaPredToCurrQ (PixelC* ppxlcRefMBA);

	Void copyFromRefToCurrQ ( // zero motion case
		const CVOPU8YUVBA* pvopcRef, // reference VOP
		CoordI x, CoordI y, 
		PixelC* ppxlcCurrQMBY, PixelC* ppxlcCurrQMBU, PixelC* ppxlcCurrQMBV,
		CRct *prctMVLimit
	); // for non-obmc mode
	Void copyFromRefToCurrQ_WithShape ( // zero motion case
		const CVOPU8YUVBA* pvopcRef, // reference VOP
		CoordI x, CoordI y, 
		PixelC* ppxlcCurrQMBY, PixelC* ppxlcCurrQMBU, PixelC* ppxlcCurrQMBV, PixelC* ppxlcCurrQMBBY
	); // for non-obmc mode
	Void copyAlphaFromRefToCurrQ (
		CoordI x, CoordI y, 
		PixelC* ppxlcCurrQMBA
	);

	// MC padding 
	Void mcPadCurrMB (
		PixelC* ppxlcRefMBY, 
		PixelC* ppxlcRefMBU, PixelC* ppxlcRefMBV,
		PixelC* ppxlcRefMBA 
	);
	Void mcPadCurr (
		PixelC *ppxlcTextureBase, // (uiStride X ???)
		const PixelC *ppxlcAlphaBase, // uiBlkSize X uiBlkSize
		UInt uiBlkSize, UInt uiStride
	);
	Void padNeighborTranspMBs (
		CoordI xb, CoordI yb,
		CMBMode* pmbmd,
		PixelC* ppxlcRefMBY, PixelC* ppxlcRefMBU, PixelC* ppxlcRefMBV
	);
	Void padCurrAndTopTranspMBFromNeighbor (
		CoordI xb, CoordI yb,
		CMBMode* pmbmd,
		PixelC* ppxlcRefMBY, PixelC* ppxlcRefMBU, PixelC* ppxlcRefMBV
	);
	Void mcPadLeftMB (PixelC* ppxlcMBY, PixelC* ppxlcMBU, PixelC* ppxlcMBV);
	Void mcPadTopMB (PixelC* ppxlcMBY, PixelC* ppxlcMBU, PixelC* ppxlcMBV);
	Void mcPadCurrMBFromLeft (PixelC* ppxlcMBY, PixelC* ppxlcMBU, PixelC* ppxlcMBV);
	Void mcPadCurrMBFromTop (PixelC* ppxlcMBY, PixelC* ppxlcMBU, PixelC* ppxlcMBV);
	Void mcSetTopMBGray (PixelC* ppxlcMBY, PixelC* ppxlcMBU, PixelC* ppxlcMBV);
	Void mcSetCurrMBGray (PixelC* ppxlcMBY, PixelC* ppxlcMBU, PixelC* ppxlcMBV);
	Void mcSetLeftMBGray (PixelC* ppxlcMBY, PixelC* ppxlcMBU, PixelC* ppxlcMBV);

	// motion vector
	Void find16x16MVpred (CVector& vctPred, const CMotionVector* pmv, Bool bLeftBndry, Bool bRightBndry, Bool bTopBndry) const;
	Void find8x8MVpredAtBoundary (CVector& vctPred, const CMotionVector* pmv, Bool bLeftBndry, Bool bRightBndry, Bool bTopBndry, BlockNum blknCurr) const;
	Void find8x8MVpredInterior (CVector& vctPred, const CMotionVector* pmv, BlockNum blknCurr) const;
	Void findMVpredGeneric (CVector& vctPred, const CMotionVector* pmv, const CMBMode* pmbmd, Int iBlk, Int iXMB, Int iYMB) const;
	Void findMVpredictorOfBVOP (CVector& vctPred, const CMotionVector* pmv, const CMBMode* pmbmd, Int iMBX) const;  		    //for B-VOP only
	Void mvLookupUV (const CMBMode* pmbmd, const CMotionVector* pmv, CoordI& xRefUV, CoordI& yRefUV);
	Void mvLookupUVWithShape (const CMBMode* pmbmd, const CMotionVector* pmv, CoordI& xRefUV, CoordI& yRefUV);
	Bool validBlock (const CMBMode* pmbmd, BlockNum blkn) const;
	Void computeDirectDeltaMV (CVector& vctDiff, const CMotionVector* pmv, const CMotionVector* pmvRef, const CMBMode* pmbmdRef);	//for B-VOP only
	CVector averageOfRefMV (const CMotionVector* pmvRef, const CMBMode* pmbmdRef);													//for B-VOP only
	Void padMotionVectors(const CMBMode *pmbmd,CMotionVector *pmv);
	//Void backwardMVFromForwardMV (const CMotionVector* pmvForward, CMotionVector* pmvDirectBack, const CMotionVector* pmvRef);
	Void backwardMVFromForwardMV (								//compute back mv from forward mv and ref mv for direct mode
					CMotionVector& mvBackward, const CMotionVector& mvForward, const CMotionVector& mvRef,
					CVector vctDirectDeltaMV);


	// block level
	Void inverseQuantizeIntraDc (Int* rgiCoefQ, Int iDcScaler);
	Void inverseQuantizeDCTcoefH263 (Int* rgiCoefQ, Int iStart, Int iQP);
	Void inverseQuantizeIntraDCTcoefMPEG (Int* rgiCoefQ, Int iStart, Int iQP);
	Void inverseQuantizeInterDCTcoefMPEG (Int* rgiCoefQ, Int iStart, Int iQP);
	const BlockMemory findPredictorBlock (
		BlockNum iBlk, 
		IntraPredDirection predDir,
		const MacroBlockMemory* pmbmLeft, 
		const MacroBlockMemory* pmbmTop,
		const MacroBlockMemory* pmbmLeftTop,
		const MacroBlockMemory* pmbmCurr,
		const CMBMode* pmbmdLeft, 
		const CMBMode* pmbmdTop,
		const CMBMode* pmbmdLeftTop,
		const CMBMode* pmbmdCurr,
		Int& iQPpred
	);
	Int decideIntraPredDir (
		Int* rgiCoefQ,
		BlockNum blkn,
		const BlockMemory& blkmRet, 
		const MacroBlockMemory* pmbmLeft, 
		const MacroBlockMemory* pmbmTop, 
		const MacroBlockMemory* pmbmLeftTop,
		const MacroBlockMemory* pmbmCurr,
		const CMBMode* pmbmdLeft,
		const CMBMode* pmbmdTop,
		const CMBMode* pmbmdLeftTop,
		CMBMode* pmbmdCurr,
		Int&	iQPpred,
		Bool bDecideDCOnly = FALSE
	);


	Time m_t; // current time
	Time m_tPastRef; // time of reference frame in past (for P/B)
	Time m_tFutureRef; // time of reference frame in future (for B)
	Int m_iBCount; // counts 1,2,3 with B frames, used to pad ref vop once.

	Int	 m_iNumBitsTimeIncr;
	//Time m_tBCounter; // b-vop counter, 1 (1st), 2 (2nd), ... 
	//Time m_tDistanceBetwIPVOP; //no. of frms betw. I/PVOP = PeriodOfPVOP except for at irregular period (end of sequence)
	//Time m_tPrevIorPVOPCounter; //frm. no of previous encoded I/PVOP
	Time m_tModuloBaseDisp;							//of the most recently displayed I/Pvop
	Time m_tModuloBaseDecd;							//of the most recently decoded I/Pvop
	Bool m_bGOV_Reset;								//reset at the begening of GOV; no dependency on prevous frames

	// MB buffer data
	CVOPU8YUVBA* m_pvopcPredMB;
	PixelC *m_ppxlcPredMBY, *m_ppxlcPredMBU, *m_ppxlcPredMBV, *m_ppxlcPredMBA;
	CVOPIntYUVBA* m_pvopiErrorMB;
	PixelI *m_ppxliErrorMBY, *m_ppxliErrorMBU, *m_ppxliErrorMBV, *m_ppxliErrorMBA;

	// B-VOP MB buffer
	CVOPU8YUVBA* m_pvopcPredMBBack; // backward buffer data
	PixelC *m_ppxlcPredMBBackY, *m_ppxlcPredMBBackU, *m_ppxlcPredMBBackV, *m_ppxlcPredMBBackA;

	// MB shape data
	ArCodec* m_parcodec;			//arithmatic coder
	CU8Image* m_puciPredBAB;		//motion compensated binary shape
	PixelC *m_ppxlcPredBABDown2, *m_ppxlcPredBABDown4;
	PixelC* m_ppxlcReconCurrBAB;
	Int m_iWidthCurrBAB;
	PixelC* m_rgpxlcCaeSymbol;
	PixelC *m_ppxlcCurrMBBYDown4, *m_ppxlcCurrMBBYDown2;

	ShapeMode* m_rgshpmd;  // for saving reference shape mode
	Int m_iRefShpNumMBX;
	Int m_iRefShpNumMBY;

	CVOPU8YUVBA* m_pvopcCurrMB;
	PixelC *m_ppxlcCurrMBY, *m_ppxlcCurrMBU, *m_ppxlcCurrMBV, *m_ppxlcCurrMBBY, *m_ppxlcCurrMBBUV, *m_ppxlcCurrMBA;
	
	// for MC-padding
	Bool* m_pbEmptyRowArray;
	Bool* m_pbEmptyColArray;

	// VOP mode 
	VOLMode	m_volmd; // vol mode
	VOPMode	m_vopmd; // vop mode
	UInt m_uiVOId; // VOP ID

	// sprite info
	UInt m_uiSprite; // whether this is a sprite VOP: 1 - yes; other - no
	UInt m_uiWarpingAccuracy; // accuracy for sprite warping
	Int m_iNumOfPnts; // for sprite warping
	own CSiteD* m_rgstSrcQ; // quantized src sts for sprite warping
	own CSiteD* m_rgstDstQ; // quantized dst sts for sprite warping
	own CVOPU8YUVBA* m_pvopcSptQ; // Loaded first as original sprite and then become quantized sprite. 
	CRct m_rctSpt; // rct of m_pvopcSptQ
	CRct m_rctSptDisp; //rct to display sprite (decoder side)
	own MBSptMode* m_rgsptmd; // sprite mode 0: untransmitted; 1: transmitted; 2: updated; 3: done

	// internal data
	own CVOPU8YUVBA* m_pvopcRefQ0; // original reference VOP in a previous time
	own CVOPU8YUVBA* m_pvopcRefQ1; // original reference VOP in a later time
	own CVOPU8YUVBA* m_pvopcCurrQ; // original reference VOP in a later time
	Int m_iStartInRefToCurrRctY, m_iStartInRefToCurrRctUV;

	// motion data
	own CDirectModeData* m_pdrtmdRef1; // for direct mode
	CVector	m_vctForwardMvPredBVOP;		//motion vector predictors for B-VOP
	CVector	m_vctBackwardMvPredBVOP;


	// some fixed variables (VOL)
	CRct m_rctRefFrameY, m_rctRefFrameUV;
	Int m_iFrameWidthYxMBSize, m_iFrameWidthYxBlkSize, m_iFrameWidthUVxBlkSize;
	Int m_iFrameWidthY, m_iFrameWidthUV;
	Int m_ivolWidth, m_ivolHeight;

	// VOP variables
	CRct m_rctCurrVOPY, m_rctCurrVOPUV;
	CRct m_rctRefVOPY0, m_rctRefVOPUV0;
	CRct m_rctRefVOPY1, m_rctRefVOPUV1;
	Int m_iOffsetForPadY, m_iOffsetForPadUV;
	CRct m_rctPrevNoExpandY, m_rctPrevNoExpandUV;

	Int m_iVOPWidthY, m_iVOPWidthUV;
	Int m_iNumMB, m_iNumMBX, m_iNumMBY;
	Int m_iNumOfTotalMVPerRow;

	CMBMode* m_rgmbmd; // VOP size.  need to renew every VOP
	CMotionVector* m_rgmv; // VOP size.  need to renew every VOP
	CMotionVector* m_rgmvBackward; // VOP size.  need to renew every VOP
	CMotionVector* m_rgmvBY; //Motion vectors for BY plane
	
	CMBMode* m_rgmbmdRef;
	CMotionVector* m_rgmvRef; // VOP size.  need to renew every VOP
	Int m_iNumMBRef, m_iNumMBXRef, m_iNumMBYRef;

	// clipping tables
	U8* m_rgiClipTab; // clapping the reconstructed pixels

	// MB data
	Int** m_rgpiCoefQ;
	MacroBlockMemory** m_rgpmbmAbove;
	MacroBlockMemory** m_rgpmbmCurr;
	BlockMemory* m_rgblkmCurrMB;	//predictor blocks for each current block
	Int* m_rgiQPpred;				//QP from previous block for intra ac prediction
	Int m_rgiDcScalerY [32];		//intra dc quantization scaler; qp dependent	
	Int m_rgiDcScalerC [32];		//[0] elem never used

	// block data
	Int m_rgiDCTcoef [BLOCK_SQUARE_SIZE];
	CInvBlockDCT* m_pidct;
};

#endif // __VOPSES_HPP_
