/* FILE: "tab2.c" */
/*
    ITU-T G.723.1 Floating Point Speech Coder ANSI C Source Code.  Version 5.1F

    Original fixed-point code copyright (c) 1995,
    AudioCodes, DSP Group, France Telecom, Universite de Sherbrooke.
    All rights reserved.

    Floating-point code copyright (c) 1995,
    Intel Corporation and France Telecom (CNET).
    All rights reserved.
*/

#include "typedef2.h"
#include "cst2.h"
#include "tab2.h"

#define F  FLOAT


/*
**
** File:        tab2.c
**
** Description: Tables used for G.723.1 encoding and decoding
**
** Tables:      HammingWindowTable[180]
**
**              LPC Computation and Filtering
**
**                  BinomialWindowTable[10]
**                  BandExpTable[10]
**
**              LSP calculation and quantization
**
**                  CosineTable[512]
**                  LspDcTable[10]
**                  BandInfoTable[3][2], int
**                  Band0Tb8[256*3]
**                  Band1Tb8[256*3]
**                  Band2Tb8[256*4]
**                  BandQntTable[3], FLOAT*
**
**              Perceptual Filtering and Post Filtering
**
**
**                  PerFiltZeroTable[10]
**                  PerFiltPoleTable[10]
**                  PostFiltZeroTable[10]
**                  PostFiltPoleTable[10]
**                  LpfConstTable[2] (pitch postfilter)
**
**              ACELP or MP-MLQ
**
**                  Nb_puls[4] (MP-MLQ), int
**                  FcbkGainTable[24] (ACELP and MP-MLQ)
**                  MaxPosTable[4]  (MP-MLQ), Word32
**                  CombinatorialTable[6][30] (MP-MLQ), Word32
**                  epsi170[170] (ACELP), int
**                  gain170[170] (ACELP)
**
**              Pitch Prediction
**
**                  AcbkGainTable085[85*20]
**                  AcbkGainTable170[170*20]
**                  AcbkGainTablePtr[2], FLOAT*
**
**              Taming procedure
**
**                  tabgain170[170]
**                  tabgain85[85]
**
**              Comfort Noise Generation
**
**                  fact[4]
**                  bseg[3]
**                  base[3]
**
**              All tables are FLOAT unless separately denoted
*/

/*
**  HammingWindowTable:
**
**  Hamming Window coefficients.
**
*/

FLOAT   HammingWindowTable[LpcFrame] = {
  (F)0.080000,
  (F)0.080283,
  (F)0.081133,
  (F)0.082548,
  (F)0.084527,
  (F)0.087067,
  (F)0.090164,
  (F)0.093816,
  (F)0.098018,
  (F)0.102764,
  (F)0.108049,
  (F)0.113866,
  (F)0.120208,
  (F)0.127067,
  (F)0.134435,
  (F)0.142303,
  (F)0.150660,
  (F)0.159498,
  (F)0.168804,
  (F)0.178567,
  (F)0.188775,
  (F)0.199417,
  (F)0.210478,
  (F)0.221944,
  (F)0.233803,
  (F)0.246039,
  (F)0.258637,
  (F)0.271582,
  (F)0.284857,
  (F)0.298447,
  (F)0.312335,
  (F)0.326502,
  (F)0.340933,
  (F)0.355609,
  (F)0.370513,
  (F)0.385625,
  (F)0.400927,
  (F)0.416401,
  (F)0.432026,
  (F)0.447785,
  (F)0.463658,
  (F)0.479624,
  (F)0.495665,
  (F)0.511761,
  (F)0.527891,
  (F)0.544037,
  (F)0.560177,
  (F)0.576292,
  (F)0.592363,
  (F)0.608369,
  (F)0.624291,
  (F)0.640109,
  (F)0.655804,
  (F)0.671356,
  (F)0.686747,
  (F)0.701956,
  (F)0.716966,
  (F)0.731758,
  (F)0.746314,
  (F)0.760616,
  (F)0.774645,
  (F)0.788386,
  (F)0.801821,
  (F)0.814933,
  (F)0.827706,
  (F)0.840125,
  (F)0.852174,
  (F)0.863839,
  (F)0.875104,
  (F)0.885957,
  (F)0.896384,
  (F)0.906371,
  (F)0.915907,
  (F)0.924980,
  (F)0.933579,
  (F)0.941693,
  (F)0.949312,
  (F)0.956426,
  (F)0.963028,
  (F)0.969109,
  (F)0.974660,
  (F)0.979677,
  (F)0.984151,
  (F)0.988079,
  (F)0.991454,
  (F)0.994273,
  (F)0.996533,
  (F)0.998230,
  (F)0.999363,
  (F)0.999929,
  (F)0.999929,
  (F)0.999363,
  (F)0.998230,
  (F)0.996533,
  (F)0.994273,
  (F)0.991454,
  (F)0.988079,
  (F)0.984151,
  (F)0.979677,
  (F)0.974660,
  (F)0.969109,
  (F)0.963028,
  (F)0.956427,
  (F)0.949312,
  (F)0.941693,
  (F)0.933579,
  (F)0.924980,
  (F)0.915907,
  (F)0.906371,
  (F)0.896384,
  (F)0.885957,
  (F)0.875104,
  (F)0.863839,
  (F)0.852174,
  (F)0.840125,
  (F)0.827706,
  (F)0.814933,
  (F)0.801821,
  (F)0.788386,
  (F)0.774645,
  (F)0.760616,
  (F)0.746314,
  (F)0.731758,
  (F)0.716966,
  (F)0.701956,
  (F)0.686747,
  (F)0.671356,
  (F)0.655804,
  (F)0.640110,
  (F)0.624291,
  (F)0.608369,
  (F)0.592363,
  (F)0.576292,
  (F)0.560177,
  (F)0.544037,
  (F)0.527891,
  (F)0.511761,
  (F)0.495665,
  (F)0.479624,
  (F)0.463658,
  (F)0.447785,
  (F)0.432027,
  (F)0.416401,
  (F)0.400927,
  (F)0.385625,
  (F)0.370513,
  (F)0.355609,
  (F)0.340933,
  (F)0.326502,
  (F)0.312335,
  (F)0.298447,
  (F)0.284857,
  (F)0.271582,
  (F)0.258637,
  (F)0.246039,
  (F)0.233803,
  (F)0.221945,
  (F)0.210478,
  (F)0.199417,
  (F)0.188775,
  (F)0.178567,
  (F)0.168804,
  (F)0.159498,
  (F)0.150660,
  (F)0.142303,
  (F)0.134435,
  (F)0.127067,
  (F)0.120208,
  (F)0.113866,
  (F)0.108049,
  (F)0.102764,
  (F)0.098018,
  (F)0.093816,
  (F)0.090164,
  (F)0.087067,
  (F)0.084527,
  (F)0.082548,
  (F)0.081133,
  (F)0.080283,
  (F)0.080000
 } ;


/*
**  BinomialWindowTable:
**
**  Purpose:
**     Binomial Window coefficients used to weight the autocorrelation before
**     Levinson-Durbin in the LPC coefficient calculation.
**
**
*/

FLOAT BinomialWindowTable[LpcOrder] = {
  (F)0.999443,
  (F)0.997775,
  (F)0.995002,
  (F)0.991132,
  (F)0.986178,
  (F)0.987,
  (F)0.973088,
  (F)0.964995,
  (F)0.955905,
  (F)0.945847
  } ;

/*
**  BandExpTable:
**
**  Purpose:
**      Do bandwidth expansion on the LPC coefficients by scaling the
**      poles of the LPC synthesis filter by a factor of 0.994

**
**  Table Structure:
**      Table values correspond to (0.994) to the power of x,
**      where x = [0,..,10].
**
*/

FLOAT BandExpTable[LpcOrder] = {
  (F)0.994 ,
  (F)0.988036 ,
  (F)0.9821078 ,
  (F)0.9762151 ,
  (F)0.9703578 ,
  (F)0.9645357 ,
  (F)0.9587485 ,
  (F)0.9529960 ,
  (F)0.9472780 ,
  (F)0.9415944 ,
};



/*
**  CosineTable:
**
**  Purpose:
**      Used to evaluate polynomial for LSP-LPC conversion
**
**  Table Structure:
**      Contains one period of a cosine wave.
**
*/

FLOAT CosineTable[CosineTableSize] = {
   (F)1 ,
   (F)0.999939 ,
   (F)0.999695 ,
   (F)0.999329 ,
   (F)0.998779 ,
   (F)0.998108 ,
   (F)0.997314 ,
   (F)0.996338 ,
   (F)0.995178 ,
   (F)0.993896 ,
   (F)0.992493 ,
   (F)0.990906 ,
   (F)0.989197 ,
   (F)0.987305 ,
   (F)0.985291 ,
   (F)0.983093 ,
   (F)0.980774 ,
   (F)0.978333 ,
   (F)0.975708 ,
   (F)0.972961 ,
   (F)0.970032 ,
   (F)0.96698 ,
   (F)0.963806 ,
   (F)0.960449 ,
   (F)0.95697 ,
   (F)0.953308 ,
   (F)0.949524 ,
   (F)0.945618 ,
   (F)0.941528 ,
   (F)0.937317 ,
   (F)0.932983 ,
   (F)0.928528 ,
   (F)0.923889 ,
   (F)0.919128 ,
   (F)0.914185 ,
   (F)0.90918 ,
   (F)0.903992 ,
   (F)0.898682 ,
   (F)0.89325 ,
   (F)0.887634 ,
   (F)0.881897 ,
   (F)0.876099 ,
   (F)0.870117 ,
   (F)0.863953 ,
   (F)0.857727 ,
   (F)0.851379 ,
   (F)0.844849 ,
   (F)0.838196 ,
   (F)0.831482 ,
   (F)0.824585 ,
   (F)0.817566 ,
   (F)0.810486 ,
   (F)0.803223 ,
   (F)0.795837 ,
   (F)0.78833 ,
   (F)0.780762 ,
   (F)0.77301 ,
   (F)0.765198 ,
   (F)0.757202 ,
   (F)0.749146 ,
   (F)0.740967 ,
   (F)0.732666 ,
   (F)0.724243 ,
   (F)0.715759 ,
   (F)0.707092 ,
   (F)0.698364 ,
   (F)0.689514 ,
   (F)0.680603 ,
   (F)0.67157 ,
   (F)0.662415 ,
   (F)0.653198 ,
   (F)0.64386 ,
   (F)0.634399 ,
   (F)0.624878 ,
   (F)0.615234 ,
   (F)0.60553 ,
   (F)0.595703 ,
   (F)0.585815 ,
   (F)0.575806 ,
   (F)0.565735 ,
   (F)0.555542 ,
   (F)0.545349 ,
   (F)0.534973 ,
   (F)0.524597 ,
   (F)0.514099 ,
   (F)0.50354 ,
   (F)0.49292 ,
   (F)0.482178 ,
   (F)0.471375 ,
   (F)0.46051 ,
   (F)0.449585 ,
   (F)0.438599 ,
   (F)0.427551 ,
   (F)0.416443 ,
   (F)0.405212 ,
   (F)0.393982 ,
   (F)0.38269 ,
   (F)0.371338 ,
   (F)0.359924 ,
   (F)0.348389 ,
   (F)0.336914 ,
   (F)0.325317 ,
   (F)0.31366 ,
   (F)0.302002 ,
   (F)0.290283 ,
   (F)0.278503 ,
   (F)0.266724 ,
   (F)0.254883 ,
   (F)0.242981 ,
   (F)0.231079 ,
   (F)0.219116 ,
   (F)0.207092 ,
   (F)0.195068 ,
   (F)0.183044 ,
   (F)0.170959 ,
   (F)0.158875 ,
   (F)0.146729 ,
   (F)0.134583 ,
   (F)0.122437 ,
   (F)0.110229 ,
   (F)0.0980225 ,
   (F)0.0858154 ,
   (F)0.0735474 ,
   (F)0.0613403 ,
   (F)0.0490723 ,
   (F)0.0368042 ,
   (F)0.0245361 ,
   (F)0.0122681 ,
   (F)0 ,
   (F)-0.0122681 ,
   (F)-0.0245361 ,
   (F)-0.0368042 ,
   (F)-0.0490723 ,
   (F)-0.0613403 ,
   (F)-0.0735474 ,
   (F)-0.0858154 ,
   (F)-0.0980225 ,
   (F)-0.110229 ,
   (F)-0.122437 ,
   (F)-0.134583 ,
   (F)-0.146729 ,
   (F)-0.158875 ,
   (F)-0.170959 ,
   (F)-0.183044 ,
   (F)-0.195068 ,
   (F)-0.207092 ,
   (F)-0.219116 ,
   (F)-0.231079 ,
   (F)-0.242981 ,
   (F)-0.254883 ,
   (F)-0.266724 ,
   (F)-0.278503 ,
   (F)-0.290283 ,
   (F)-0.302002 ,
   (F)-0.31366 ,
   (F)-0.325317 ,
   (F)-0.336914 ,
   (F)-0.348389 ,
   (F)-0.359924 ,
   (F)-0.371338 ,
   (F)-0.38269 ,
   (F)-0.393982 ,
   (F)-0.405212 ,
   (F)-0.416443 ,
   (F)-0.427551 ,
   (F)-0.438599 ,
   (F)-0.449585 ,
   (F)-0.46051 ,
   (F)-0.471375 ,
   (F)-0.482178 ,
   (F)-0.49292 ,
   (F)-0.50354 ,
   (F)-0.514099 ,
   (F)-0.524597 ,
   (F)-0.534973 ,
   (F)-0.545349 ,
   (F)-0.555542 ,
   (F)-0.565735 ,
   (F)-0.575806 ,
   (F)-0.585815 ,
   (F)-0.595703 ,
   (F)-0.60553 ,
   (F)-0.615234 ,
   (F)-0.624878 ,
   (F)-0.634399 ,
   (F)-0.64386 ,
   (F)-0.653198 ,
   (F)-0.662415 ,
   (F)-0.67157 ,
   (F)-0.680603 ,
   (F)-0.689514 ,
   (F)-0.698364 ,
   (F)-0.707092 ,
   (F)-0.715759 ,
   (F)-0.724243 ,
   (F)-0.732666 ,
   (F)-0.740967 ,
   (F)-0.749146 ,
   (F)-0.757202 ,
   (F)-0.765198 ,
   (F)-0.77301 ,
   (F)-0.780762 ,
   (F)-0.78833 ,
   (F)-0.795837 ,
   (F)-0.803223 ,
   (F)-0.810486 ,
   (F)-0.817566 ,
   (F)-0.824585 ,
   (F)-0.831482 ,
   (F)-0.838196 ,
   (F)-0.844849 ,
   (F)-0.851379 ,
   (F)-0.857727 ,
   (F)-0.863953 ,
   (F)-0.870117 ,
   (F)-0.876099 ,
   (F)-0.881897 ,
   (F)-0.887634 ,
   (F)-0.89325 ,
   (F)-0.898682 ,
   (F)-0.903992 ,
   (F)-0.90918 ,
   (F)-0.914185 ,
   (F)-0.919128 ,
   (F)-0.923889 ,
   (F)-0.928528 ,
   (F)-0.932983 ,
   (F)-0.937317 ,
   (F)-0.941528 ,
   (F)-0.945618 ,
   (F)-0.949524 ,
   (F)-0.953308 ,
   (F)-0.95697 ,
   (F)-0.960449 ,
   (F)-0.963806 ,
   (F)-0.96698 ,
   (F)-0.970032 ,
   (F)-0.972961 ,
   (F)-0.975708 ,
   (F)-0.978333 ,
   (F)-0.980774 ,
   (F)-0.983093 ,
   (F)-0.985291 ,
   (F)-0.987305 ,
   (F)-0.989197 ,
   (F)-0.990906 ,
   (F)-0.992493 ,
   (F)-0.993896 ,
   (F)-0.995178 ,
   (F)-0.996338 ,
   (F)-0.997314 ,
   (F)-0.998108 ,
   (F)-0.998779 ,
   (F)-0.999329 ,
   (F)-0.999695 ,
   (F)-0.999939 ,
   (F)-1 ,
   (F)-0.999939 ,
   (F)-0.999695 ,
   (F)-0.999329 ,
   (F)-0.998779 ,
   (F)-0.998108 ,
   (F)-0.997314 ,
   (F)-0.996338 ,
   (F)-0.995178 ,
   (F)-0.993896 ,
   (F)-0.992493 ,
   (F)-0.990906 ,
   (F)-0.989197 ,
   (F)-0.987305 ,
   (F)-0.985291 ,
   (F)-0.983093 ,
   (F)-0.980774 ,
   (F)-0.978333 ,
   (F)-0.975708 ,
   (F)-0.972961 ,
   (F)-0.970032 ,
   (F)-0.96698 ,
   (F)-0.963806 ,
   (F)-0.960449 ,
   (F)-0.95697 ,
   (F)-0.953308 ,
   (F)-0.949524 ,
   (F)-0.945618 ,
   (F)-0.941528 ,
   (F)-0.937317 ,
   (F)-0.932983 ,
   (F)-0.928528 ,
   (F)-0.923889 ,
   (F)-0.919128 ,
   (F)-0.914185 ,
   (F)-0.90918 ,
   (F)-0.903992 ,
   (F)-0.898682 ,
   (F)-0.89325 ,
   (F)-0.887634 ,
   (F)-0.881897 ,
   (F)-0.876099 ,
   (F)-0.870117 ,
   (F)-0.863953 ,
   (F)-0.857727 ,
   (F)-0.851379 ,
   (F)-0.844849 ,
   (F)-0.838196 ,
   (F)-0.831482 ,
   (F)-0.824585 ,
   (F)-0.817566 ,
   (F)-0.810486 ,
   (F)-0.803223 ,
   (F)-0.795837 ,
   (F)-0.78833 ,
   (F)-0.780762 ,
   (F)-0.77301 ,
   (F)-0.765198 ,
   (F)-0.757202 ,
   (F)-0.749146 ,
   (F)-0.740967 ,
   (F)-0.732666 ,
   (F)-0.724243 ,
   (F)-0.715759 ,
   (F)-0.707092 ,
   (F)-0.698364 ,
   (F)-0.689514 ,
   (F)-0.680603 ,
   (F)-0.67157 ,
   (F)-0.662415 ,
   (F)-0.653198 ,
   (F)-0.64386 ,
   (F)-0.634399 ,
   (F)-0.624878 ,
   (F)-0.615234 ,
   (F)-0.60553 ,
   (F)-0.595703 ,
   (F)-0.585815 ,
   (F)-0.575806 ,
   (F)-0.565735 ,
   (F)-0.555542 ,
   (F)-0.545349 ,
   (F)-0.534973 ,
   (F)-0.524597 ,
   (F)-0.514099 ,
   (F)-0.50354 ,
   (F)-0.49292 ,
   (F)-0.482178 ,
   (F)-0.471375 ,
   (F)-0.46051 ,
   (F)-0.449585 ,
   (F)-0.438599 ,
   (F)-0.427551 ,
   (F)-0.416443 ,
   (F)-0.405212 ,
   (F)-0.393982 ,
   (F)-0.38269 ,
   (F)-0.371338 ,
   (F)-0.359924 ,
   (F)-0.348389 ,
   (F)-0.336914 ,
   (F)-0.325317 ,
   (F)-0.31366 ,
   (F)-0.302002 ,
   (F)-0.290283 ,
   (F)-0.278503 ,
   (F)-0.266724 ,
   (F)-0.254883 ,
   (F)-0.242981 ,
   (F)-0.231079 ,
   (F)-0.219116 ,
   (F)-0.207092 ,
   (F)-0.195068 ,
   (F)-0.183044 ,
   (F)-0.170959 ,
   (F)-0.158875 ,
   (F)-0.146729 ,
   (F)-0.134583 ,
   (F)-0.122437 ,
   (F)-0.110229 ,
   (F)-0.0980225 ,
   (F)-0.0858154 ,
   (F)-0.0735474 ,
   (F)-0.0613403 ,
   (F)-0.0490723 ,
   (F)-0.0368042 ,
   (F)-0.0245361 ,
   (F)-0.0122681 ,
   (F)0 ,
   (F)0.0122681 ,
   (F)0.0245361 ,
   (F)0.0368042 ,
   (F)0.0490723 ,
   (F)0.0613403 ,
   (F)0.0735474 ,
   (F)0.0858154 ,
   (F)0.0980225 ,
   (F)0.110229 ,
   (F)0.122437 ,
   (F)0.134583 ,
   (F)0.146729 ,
   (F)0.158875 ,
   (F)0.170959 ,
   (F)0.183044 ,
   (F)0.195068 ,
   (F)0.207092 ,
   (F)0.219116 ,
   (F)0.231079 ,
   (F)0.242981 ,
   (F)0.254883 ,
   (F)0.266724 ,
   (F)0.278503 ,
   (F)0.290283 ,
   (F)0.302002 ,
   (F)0.31366 ,
   (F)0.325317 ,
   (F)0.336914 ,
   (F)0.348389 ,
   (F)0.359924 ,
   (F)0.371338 ,
   (F)0.38269 ,
   (F)0.393982 ,
   (F)0.405212 ,
   (F)0.416443 ,
   (F)0.427551 ,
   (F)0.438599 ,
   (F)0.449585 ,
   (F)0.46051 ,
   (F)0.471375 ,
   (F)0.482178 ,
   (F)0.49292 ,
   (F)0.50354 ,
   (F)0.514099 ,
   (F)0.524597 ,
   (F)0.534973 ,
   (F)0.545349 ,
   (F)0.555542 ,
   (F)0.565735 ,
   (F)0.575806 ,
   (F)0.585815 ,
   (F)0.595703 ,
   (F)0.60553 ,
   (F)0.615234 ,
   (F)0.624878 ,
   (F)0.634399 ,
   (F)0.64386 ,
   (F)0.653198 ,
   (F)0.662415 ,
   (F)0.67157 ,
   (F)0.680603 ,
   (F)0.689514 ,
   (F)0.698364 ,
   (F)0.707092 ,
   (F)0.715759 ,
   (F)0.724243 ,
   (F)0.732666 ,
   (F)0.740967 ,
   (F)0.749146 ,
   (F)0.757202 ,
   (F)0.765198 ,
   (F)0.77301 ,
   (F)0.780762 ,
   (F)0.78833 ,
   (F)0.795837 ,
   (F)0.803223 ,
   (F)0.810486 ,
   (F)0.817566 ,
   (F)0.824585 ,
   (F)0.831482 ,
   (F)0.838196 ,
   (F)0.844849 ,
   (F)0.851379 ,
   (F)0.857727 ,
   (F)0.863953 ,
   (F)0.870117 ,
   (F)0.876099 ,
   (F)0.881897 ,
   (F)0.887634 ,
   (F)0.89325 ,
   (F)0.898682 ,
   (F)0.903992 ,
   (F)0.90918 ,
   (F)0.914185 ,
   (F)0.919128 ,
   (F)0.923889 ,
   (F)0.928528 ,
   (F)0.932983 ,
   (F)0.937317 ,
   (F)0.941528 ,
   (F)0.945618 ,
   (F)0.949524 ,
   (F)0.953308 ,
   (F)0.95697 ,
   (F)0.960449 ,
   (F)0.963806 ,
   (F)0.96698 ,
   (F)0.970032 ,
   (F)0.972961 ,
   (F)0.975708 ,
   (F)0.978333 ,
   (F)0.980774 ,
   (F)0.983093 ,
   (F)0.985291 ,
   (F)0.987305 ,
   (F)0.989197 ,
   (F)0.990906 ,
   (F)0.992493 ,
   (F)0.993896 ,
   (F)0.995178 ,
   (F)0.996338 ,
   (F)0.997314 ,
   (F)0.998108 ,
   (F)0.998779 ,
   (F)0.999329 ,
   (F)0.999695 ,
   (F)0.999939
};

/*
**  LspDcTable: LSP long term Dc component
**
*/

FLOAT LspDcTable[LpcOrder] = {
   (F)24.4609,
   (F)36.8828,
   (F)60.0781,
   (F)84.4219,
   (F)108.375,
   (F)128.867,
   (F)154.312,
   (F)173.906,
   (F)199.094,
   (F)216.547,
};

int BandInfoTable[LspQntBands][2] = {
/*
**  BandInfoTable:
**
**  Purpose:
**      Used to index in arrays. Called by LSP sub vector
**      quantization routines.
**
**  Table Structure:
**      The unquantized LSP vector, quantized LSP vector, and residual
**      LSP error vector are each divided into three subvectors.
**              The table format is as follows:
**
**      1st pair corresponds to subvector 0
**      2nd pair corresponds to subvector 1
**      3rd pair corresponds to subvector 2
**
**      The 1st slot in each pair is used to index the location of
**      the vector in a 10-element array.  For example, for subvector
**      1, dimensions [0,..,2] get automatically mapped to array
**      location [3,..,5], and for subvector 3, dimensions
**      [0,..,3] automatically get mapped to array location [6,..,9].
**
**      The 2nd slot in each pair corresponds to the dimension of
**      the subvector
*/

   { 0,3},
   { 3,3},
   { 6,4}
   };

/*
**  Band0Tb8:
**
**  Purpose:
**      Vector Quantize the first 3-vector of the 10 LSP parameters.
**
**  Table Structure:
**      8 bit, 256 entry table.
*/

FLOAT Band0Tb8[LspCbSize*3] = {
    (F)0,  (F)0,  (F)0,
    (F)-2.10938,  (F)-10.7188,  (F)-8.0625,
    (F)-4.22656,  (F)-12.8906,  (F)-10.7969,
    (F)-5.64844,  (F)-15.7109,  (F)-17.2891,
    (F)-7.35156,  (F)-8.76562,  (F)-15.1719,
    (F)-6.09375,  (F)-8.94531,  (F)-19.1719,
    (F)-6.90625,  (F)-10.2266,  (F)-10.7266,
    (F)-8.21094,  (F)-11.8984,  (F)-13.7969,
    (F)-8.46094,  (F)-12.6719,  (F)-17.9688,
    (F)-6.07031,  (F)-10.7578,  (F)-16.7734,
    (F)-7.30469,  (F)-11.4609,  (F)-21.5859,
    (F)-6.26562,  (F)-10.3672,  (F)-27.1172,
    (F)-7.30469,  (F)-15.3047,  (F)-31.2422,
    (F)-1.875,  (F)-0.695312,  (F)1.73438,
    (F)-5.16406,  (F)-2.00781,  (F)-1.25,
    (F)-7.76562,  (F)-3.64062,  (F)-3.27344,
    (F)-1.46875,  (F)-1.28125,  (F)-2.17188,
    (F)-2.67188,  (F)-4,  (F)-3.24219,
    (F)-4.74219,  (F)-3.99219,  (F)-6.22656,
    (F) 0.125,  (F)0.148438,  (F)-5.59375,
    (F) 2.92188,  (F)3.32031,  (F)-7.59375,
    (F)-2.70312,  (F)1.91406,  (F)-2.20312,
    (F)-2.07031,  (F)3.95312,  (F)-5.89062,
    (F)-4.84375,  (F)-1.14844,  (F)15.2734,
    (F)-5.79688,  (F)-6.71875,  (F)20.2891,
    (F)-1.17188,  (F)-2.75,  (F)21.125,
    (F) 2.38281,  (F)6.875,  (F)15.2656,
    (F) 0.960938,  (F)5.71094,  (F)21.6094,
    (F)-2.71875,  (F)5.97656,  (F)25.9922,
    (F) 4.82812,  (F)1.72656,  (F)25.4531,
    (F)-1.39062,  (F)-0.367188,  (F)32.9609,
    (F) 3.07031,  (F)10.1875,  (F)30.6,
    (F) 5.45312,  (F)13.2969,  (F)37.5078,
    (F) 0.492188,  (F)-4.5625,  (F)9.66,
    (F)-1.67969,  (F)-5.71875,  (F)13.3125,
    (F) 1.34375,  (F)-2.61719,  (F)14.9141,
    (F)-0.625,  (F)1.6875,  (F)14.0391,
    (F) 2.75781,  (F)0.992188,  (F)17.2266,
    (F)-9.4375,  (F)1.46875,  (F)0.0859375,
    (F)-4.00781,  (F)-0.585938,  (F)-5.33594,
    (F)-7.66,  (F)1.73438,  (F)-5.04688,
    (F)-4.8125,  (F)-6.58594,  (F)-3.03125,
    (F)-7.42188,  (F)-8.69531,  (F)-2.80469,
    (F)-11.1797,  (F)-4.86719,  (F)-5.50781,
    (F)-10.9219,  (F)-8.30469,  (F)-1.39062,
    (F)-0.351562,  (F)-3.66,  (F)0.273438,
    (F)-0.0703125,  (F)-5.13281,  (F)-1.6875,
    (F) 0.992188,  (F)-8.42188,  (F)0.742188,
    (F)-7.42188,  (F)-9.03125,  (F)4.5625,
    (F)-11.5625,  (F)-11.6719,  (F)3.50781,
    (F)-0.9375,  (F)-5.50781,  (F)4.03125,
    (F)-2.875,  (F)-7.50781,  (F)5.67969,
    (F)-2.95312,  (F)-4.10938,  (F)7.66,
    (F)-6.19531,  (F)-4.79688,  (F)5.28125,
    (F)-6.25781,  (F)-5.89844,  (F)10.0547,
    (F)-11.5312,  (F)-2.65625,  (F)12.7812,
    (F)-3.94531,  (F)-9.79688,  (F)12.0547,
    (F)-9.71094,  (F)-12.6719,  (F)11.9688,
    (F)-6.0625,  (F)-11.5391,  (F)-5.11719,
    (F)-8.99219,  (F)-10.125,  (F)-6.42969,
    (F)-9.00781,  (F)-13.0625,  (F)-8.78125,
    (F)-10.0859,  (F)-15.6484,  (F)-13.2969,
    (F)-4.85938,  (F)-10.0234,  (F)0.445312,
    (F)-3.67969,  (F)-12.5859,  (F)3.97656,
    (F)-8.28125,  (F)-12.2656,  (F)-1.08594,
    (F)-6.82031,  (F)-16.8438,  (F)-4.1875,
    (F)-13.4062,  (F)-15.7891,  (F)-2.84375,
    (F)-16.7969,  (F)-25.1406,  (F)-10.0859,
    (F)-9.75,  (F)-15.1953,  (F)-22.6875,
    (F)-9.49219,  (F)-20.5703,  (F)-22.3047,
    (F) 1.30469,  (F)-1.90625,  (F)0.65625,
    (F) 2.72656,  (F)-3.21875,  (F)-1.69531,
    (F)-0.3125,  (F)-2.75,  (F)4.9375,
    (F) 1.77344,  (F)-4.13281,  (F)3.16406,
    (F) 0.53125,  (F)-2.99219,  (F)-3.46094,
    (F) 1.30469,  (F)-4.35938,  (F)-5.51562,
    (F)-2.14844,  (F)-6.67188,  (F)-0.109375,
    (F)-2.74219,  (F)-8.50781,  (F)-3.50781,
    (F) 2.66406,  (F)-0.5625,  (F)-2.25781,
    (F) 4.71094,  (F)-0.828125,  (F)-3.70312,
    (F) 2.51562,  (F)-1.71094,  (F)-5.07031,
    (F) 1.39844,  (F)-2.47656,  (F)-7.79688,
    (F) 3.51562,  (F)-2.27344,  (F)-7.78125,
    (F) 4.33594,  (F)1.52344,  (F)-4.16,
    (F) 6.125,  (F)2.125,  (F)-6.49219,
    (F)-1.15625,  (F)-3,  (F)-6.63281,
    (F) 0.640625,  (F)-4.1875,  (F)-10.6016,
    (F) 1.85938,  (F)-1.34375,  (F)-10.5781,
    (F) 3.29688,  (F)-2.09375,  (F)-14.3828,
    (F) 2.32031,  (F)-5.75781,  (F)-16.2422,
    (F)-0.867188,  (F)-6.25781,  (F)-4.67188,
    (F) 0.0078125,  (F)-5.21875,  (F)-7.6875,
    (F)-1.02344,  (F)-6.39062,  (F)-10.1484,
    (F)-2.57031,  (F)-4.07031,  (F)-10.2344,
    (F)-1.17969,  (F)-6.07812,  (F)-14.3281,
    (F)-0.726562,  (F)-2.75,  (F)-13.6406,
    (F)-4.4375,  (F)-5,  (F)-14.2266,
    (F)-3.97656,  (F)-7.35156,  (F)-17.0547,
    (F) 3.625,  (F)-6.36719,  (F)-9.76562,
    (F) 0.617188,  (F)-8.85156,  (F)-12.4766,
    (F)-1.4375,  (F)-10.5703,  (F)-16.5859,
    (F)-1.53125,  (F)-3.20312,  (F)-18.9609,
    (F)-1.5,  (F)-6.50781,  (F)-21.9531,
    (F)-2.02344,  (F)-10.7969,  (F)-23.7891,
    (F)-1.69531,  (F)0.03125,  (F)-9.10938,
    (F)-6.25,  (F)-2.53906,  (F)-9.52344,
    (F)-2.83594,  (F)-6.48438,  (F)-7.62,
    (F)-5.16406,  (F)-8.85938,  (F)-7.5,
    (F)-3.62,  (F)-7.65625,  (F)-11.7266,
    (F)-4.89844,  (F)-9.05469,  (F)-13.4531,
    (F)-7.05469,  (F)-6.47656,  (F)-6.67969,
    (F)-5.35156,  (F)-6.47656,  (F)-10.2578,
    (F)-8.32031,  (F)-7.49219,  (F)-10.9766,
    (F) 3.44531,  (F)0.195312,  (F)-6.61719,
    (F) 5.11719,  (F)-0.210938,  (F)-9.22656,
    (F) 9.05469,  (F)-0.859375,  (F)-5.50781,
    (F) 6.6875,  (F)1.97656,  (F)-13.0547,
    (F) 3.24219,  (F)3.15625,  (F)-0.0078125,
    (F) 2.51562,  (F)7.05469,  (F)-3.10938,
    (F) 5.23438,  (F)3.89844,  (F)-2.28125,
    (F) 6.27344,  (F)4.61719,  (F)-4.76562,
    (F) 8.9375,  (F)4.61719,  (F)-6.35938,
    (F) 5.66,  (F)1.42969,  (F)3.07031,
    (F) 6.69531,  (F)2.97656,  (F)0.828125,
    (F) 4.75781,  (F)0.484375,  (F)-0.210938,
    (F) 6.1875,  (F)1.54688,  (F)-2.53906,
    (F) 5.74219,  (F)6.28906,  (F)0.6875,
    (F) 8.92188,  (F)6.34375,  (F)0.609375,
    (F) 8.03125,  (F)2.85938,  (F)-2.28125,
    (F) 10.2266,  (F)5.80469,  (F)-1.85156,
    (F) 12.6172,  (F)4.66,  (F)-0.617188,
    (F) 7.89062,  (F)4.99219,  (F)-1.89844,
    (F) 7.80469,  (F)7.53125,  (F)-2.42969,
    (F) 11.7188,  (F)8.88281,  (F)-4.80469,
    (F) 7.71875,  (F)2.78906,  (F)5.04688,
    (F) 9.58594,  (F)5.21094,  (F)5.33594,
    (F) 9.09375,  (F)12.2266,  (F)6.98438,
    (F) 10.875,  (F)15.7422,  (F)3.72656,
    (F) 8.89062,  (F)4.16406,  (F)1.95312,
    (F) 11.2266,  (F)7,  (F)3.05469,
    (F) 13.7891,  (F)8.73438,  (F)0.773438,
    (F) 8.6875,  (F)8.51562,  (F)6.26562,
    (F) 12.4688,  (F)6.60938,  (F)8.85938,
    (F) 7.32031,  (F)9.07031,  (F)2.17969,
    (F) 13.4297,  (F)9.79688,  (F)5.33594,
    (F) 10.4531,  (F)8.48438,  (F)0.273438,
    (F) 11.0859,  (F)10.3438,  (F)3.34375,
    (F) 11.1562,  (F)11.9062,  (F)0.3125,
    (F) 16.4688,  (F)12.4531,  (F)0.695312,
    (F) 7.92969,  (F)4.25,  (F)9.54688,
    (F) 8.75781,  (F)7.22656,  (F)9.86719,
    (F) 8.04688,  (F)10.2969,  (F)11.6016,
    (F) 10.1172,  (F)6.16406,  (F)14.1953,
    (F) 10.3359,  (F)9.9375,  (F)14.9141,
    (F) 13.4688,  (F)9.66406,  (F)14.0859,
    (F) 14.0391,  (F)13.1953,  (F)6.70312,
    (F) 16.7891,  (F)10.6797,  (F)10.1641,
    (F) 17.9844,  (F)14.5859,  (F)5.94531,
    (F) 22.3672,  (F)18.3672,  (F)8.22656,
    (F) 0.40625,  (F)1.27344,  (F)-0.59375,
    (F) 1.79688,  (F)2.41406,  (F)-3.84375,
    (F)-0.554688,  (F)4.83594,  (F)0.304688,
    (F)-1.70312,  (F)6.6875,  (F)3.89844,
    (F)-5.10938,  (F)5.75,  (F)-1.61719,
    (F)-4.17969,  (F)9.83594,  (F)1.21094,
    (F)-3.75,  (F)11.5312,  (F)5.02344,
    (F) 2.04688,  (F)8.44531,  (F)0.796875,
    (F) 2.41406,  (F)12.4375,  (F)-1.42188,
    (F) 4.89844,  (F)12.7266,  (F)4.17188,
    (F) 2.63281,  (F)5.02344,  (F)3.5625,
    (F) 5.92188,  (F)5.23438,  (F)5.57031,
    (F) 1.57812,  (F)8.79688,  (F)5.14062,
    (F) 4.78125,  (F)8.83594,  (F)5.20312,
    (F) 5.35938,  (F)9.55469,  (F)8.875,
    (F)-1.02344,  (F)2.94531,  (F)4.16,
    (F) 0.328125,  (F)5.53125,  (F)7.08594,
    (F) 0.679688,  (F)11.625,  (F)8.08594,
    (F) 3.375,  (F)16.5391,  (F)7.0625,
    (F) 1.07031,  (F)7.66406,  (F)10.4062,
    (F)-3.49219,  (F)7.92188,  (F)8.875,
    (F)-6.55469,  (F)14.0078,  (F)9.73438,
    (F)-4.36719,  (F)2.32031,  (F)1.54688,
    (F)-6.64062,  (F)5.35156,  (F)3.48438,
    (F)-9.94531,  (F)4.9375,  (F)6.45312,
    (F)-3.13281,  (F)-4.25,  (F)1.35156,
    (F)-5.88281,  (F)-6.19531,  (F)1.125,
    (F)-3.40625,  (F)-0.0703125,  (F)6.03125,
    (F)-0.898438,  (F)-1.89844,  (F)10.2344,
    (F)-5.23438,  (F)-2.16,  (F)2.92188,
    (F)-8.02344,  (F)-0.162,  (F)4.99219,
    (F)-6.92969,  (F)-0.632812,  (F)8.88281,
    (F)-9.97656,  (F)-3.55469,  (F)1.23438,
    (F)-11.0234,  (F)-5.625,  (F)5.75,
    (F) 1.34375,  (F)0.6875,  (F)3.14844,
    (F) 3.62,  (F)1.99219,  (F)5.90625,
    (F)-3.90625,  (F)4.07812,  (F)7.10938,
    (F)-7.48438,  (F)5.14844,  (F)10.8438,
    (F)-3.08594,  (F)2.35156,  (F)10.5,
    (F)-2.78125,  (F)6,  (F)14.1641,
    (F)-4.78906,  (F)6.57031,  (F)18.8984,
    (F) 3.47656,  (F)-0.953125,  (F)1.96875,
    (F) 4.91406,  (F)-0.679688,  (F)5.64844,
    (F) 2.21094,  (F)-1.97656,  (F)6.79688,
    (F) 3.5625,  (F)-0.90625,  (F)10.7891,
    (F) 5.91406,  (F)1.40625,  (F)8.27344,
    (F) 4.15625,  (F)3.1875,  (F)11.7891,
    (F) 7.39844,  (F)2.25,  (F)14.1094,
    (F) 10.3516,  (F)7.76562,  (F)19.7188,
    (F) 6.96875,  (F)9.52344,  (F)23.6172,
    (F) 10.9141,  (F)12.4688,  (F)26.6094,
    (F) 8.92969,  (F)12.125,  (F)19.8906,
    (F) 14.4531,  (F)11.1953,  (F)21.1719,
    (F)-0.078125,  (F)1.04688,  (F)7.82812,
    (F) 1.20312,  (F)3.89844,  (F)10.3359,
    (F) 3.96875,  (F)6.1875,  (F)8.72656,
    (F) 3.97656,  (F)10.4688,  (F)12.625,
    (F) 5.95312,  (F)6.73438,  (F)12.5625,
    (F) 6.14844,  (F)5.78125,  (F)18.125,
    (F) 6.20312,  (F)13.4922,  (F)10.0234,
    (F) 3.63281,  (F)16.4688,  (F)12.9688,
    (F)-0.9375,  (F)11.3359,  (F)12.6016,
    (F)-3.62,  (F)15.75,  (F)16.9453,
    (F) 6.96094,  (F)9.57031,  (F)16.6,
    (F) 3.5625,  (F)11.5625,  (F)17.0703,
    (F) 11.6641,  (F)10.0234,  (F)9.44531,
    (F) 10.9141,  (F)12.7812,  (F)11.8594,
    (F) 13.875,  (F)13.5781,  (F)12.125,
    (F) 12.2812,  (F)13.2656,  (F)16.7266,
    (F) 10.8516,  (F)16.6094,  (F)9.92969,
    (F) 15.3047,  (F)18.8516,  (F)8.74219,
    (F) 10.6641,  (F)22.5938,  (F)11.7578,
    (F) 17.2344,  (F)15.3984,  (F)12.6797,
    (F) 16.2188,  (F)15.2344,  (F)17.8125,
    (F) 13.4141,  (F)17.8984,  (F)14.5859,
    (F) 18.4844,  (F)19.6484,  (F)15.2578,
    (F) 22.3828,  (F)22.1719,  (F)19.7031,
    (F) 19.8047,  (F)27.0703,  (F)15.7109,
    (F) 26.4141,  (F)32.2422,  (F)20.6094,
    (F) 6.53125,  (F)20.8359,  (F)17.8828,
    (F) 13.7578,  (F)21.6641,  (F)18.2578,
    (F) 11.0547,  (F)25.9766,  (F)22.7422,
    (F) 18.3906,  (F)24.5156,  (F)24.4219,
    (F) 20.7734,  (F)32.75,  (F)31.3281,
    (F) 8.1875,  (F)13.9531,  (F)14.2031,
    (F) 9.70312,  (F)16.4922,  (F)17.5,
    (F) 11.8125,  (F)16.2422,  (F)21.7188,
    (F) 12.2891,  (F)19.4609,  (F)24.5156,
    (F) 17.4219,  (F)18.5703,  (F)21.7344,
    (F) 3.25,  (F)13.8516,  (F)21.125,
    (F) 5.66406,  (F)18.25,  (F)25.7578,
    (F) 9.78125,  (F)18.5391,  (F)31.0781,
    (F) 16.3594,  (F)17.7188,  (F)27.875,
    (F) 15.7109,  (F)21.1875,  (F)35.375,
    (F) 10.4766,  (F)27.3984,  (F)30.2812,
    (F) 9.5,  (F)30.6172,  (F)38.4531,
    (F) 13.2266,  (F)37.4453,  (F)46.9688,
};

/*
**  Band1Tb8:
**
**  Purpose:
**      Vector Quantize the second 3-vector of the 10 LSP parameters.
**
**  Table Structure:
**      8 bit, 256 entry table.
*/

FLOAT Band1Tb8[LspCbSize*3] = {
    (F)0,  (F)0,  (F)0,
    (F)-16.5156,  (F)-10.1719,  (F)0.59375,
    (F)-20.7188,  (F)-9.98438,  (F)-10.6875,
    (F)-22.2422,  (F)-6.46875,  (F)-2.72656,
    (F)-29.7812,  (F)-17.1094,  (F)-2.72656,
    (F)-30.8281,  (F)-2.84375,  (F)-3.50781,
    (F)-21.2891,  (F)-35.0938,  (F)-28.1797,
    (F)-27.3047,  (F)-37.2188,  (F)-13.625,
    (F)-0.398438,  (F)-5.90625,  (F)0.65625,
    (F)-1.19531,  (F)-9.30469,  (F)3.9375,
    (F) 0.84375,  (F)-11.0781,  (F)9.11719,
    (F)-6.52344,  (F)-7,  (F)3.04688,
    (F)-4.44531,  (F)-13.2969,  (F)0.679688,
    (F)-8.99219,  (F)-14.2031,  (F)7.28906,
    (F)-14.2656,  (F)-19.8984,  (F)3.21094,
    (F)-14.3906,  (F)-14.2031,  (F)11.3359,
    (F)-19.0469,  (F)-12.5859,  (F)6.16,
    (F)-21.4609,  (F)-19.3516,  (F)10.2422,
    (F)-7.34375,  (F)9.78125,  (F)3.72656,
    (F)-12.7266,  (F)13.1875,  (F)4.70312,
    (F)-9.39062,  (F)4.82031,  (F)2.1875,
    (F)-13.5703,  (F)3.07031,  (F)4.53125,
    (F)-11.9375,  (F)8.41406,  (F)9.36719,
    (F)-16.9141,  (F)-1.25781,  (F)11,
    (F)-19.5625,  (F)-8.49219,  (F)18.5234,
    (F)-27.6,  (F)-1.36719,  (F)10.8984,
    (F)-10.9141,  (F)-0.765625,  (F)-6.58594,
    (F)-17.5938,  (F)-1.38281,  (F)-8.97656,
    (F)-11.6328,  (F)-5.67188,  (F)-10.0234,
    (F)-12.1719,  (F)-2.07031,  (F)-13.625,
    (F)-14.5859,  (F)-6.41406,  (F)-14.8203,
    (F)-16.1094,  (F)-11.8438,  (F)-18.2812,
    (F)-20.2734,  (F)-8.92188,  (F)-22.3516,
    (F) 1.32812,  (F)0.359375,  (F)-6.39844,
    (F)-1.50781,  (F)-1.59375,  (F)-8.99219,
    (F) 2.54688,  (F)-1.53125,  (F)-11.9688,
    (F) 6.09375,  (F)2.57031,  (F)-6.375,
    (F) 1.57031,  (F)2.88281,  (F)-9.71094,
    (F) 5.07812,  (F)-1.63281,  (F)-8.28125,
    (F) 8.9375,  (F)-0.117188,  (F)-9.5,
    (F) 9.39844,  (F)-2.02344,  (F)-14.5859,
    (F)-6.95312,  (F)-4.40625,  (F)-11.1719,
    (F)-4.98438,  (F)-6.65625,  (F)-15.0078,
    (F) 1.38281,  (F)-5.77344,  (F)-10.6094,
    (F)-2.03906,  (F)-4.10938,  (F)-13.6,
    (F) 1.60938,  (F)-3.17969,  (F)-17.6172,
    (F) 2.64062,  (F)-4.10938,  (F)-6.42188,
    (F) 3.28906,  (F)-8.55469,  (F)-7.88281,
    (F) 5.97656,  (F)-4.74219,  (F)-11,
    (F) 6.44531,  (F)-10.1172,  (F)-15.6562,
    (F) 2.78906,  (F)-7.07031,  (F)-14.1797,
    (F)-0.453125,  (F)-9.75,  (F)-12.4062,
    (F)-4.65625,  (F)-11.2188,  (F)-15.9844,
    (F)-0.570312,  (F)-9.05469,  (F)-16.5312,
    (F)-0.898438,  (F)-10.7969,  (F)-20.1641,
    (F)-1.25,  (F)-13.4609,  (F)-15.25,
    (F)-0.046875,  (F)-17.1562,  (F)-23.0781,
    (F)-5.07031,  (F)-13.3203,  (F)-20.3359,
    (F)-4.82031,  (F)-11.3516,  (F)-25.6406,
    (F)-7.41406,  (F)-15.7734,  (F)-24.2344,
    (F)-6.34375,  (F)12.0625,  (F)15.1328,
    (F)-14.4844,  (F)4.48438,  (F)15.625,
    (F)-11.4297,  (F)8.90625,  (F)20.6953,
    (F)-20.9609,  (F)13.6562,  (F)11.3438,
    (F)-19.4219,  (F)17.5078,  (F)19.7109,
    (F) 6.11719,  (F)14.9219,  (F)11.2109,
    (F) 4.53906,  (F)20.9531,  (F)10.75,
    (F) 1.84375,  (F)17.1641,  (F)14.7266,
    (F)-3.53906,  (F)22.9922,  (F)16.0703,
    (F)-5.32812,  (F)17.6,  (F)20.0391,
    (F)-10.4844,  (F)25.0078,  (F)26,
    (F)-2.25,  (F)-1.4375,  (F)2.04688,
    (F) 0.945312,  (F)-1.16406,  (F)-1.42969,
    (F) 5.92188,  (F)-3.21875,  (F)1.60938,
    (F) 8.10938,  (F)-1.59375,  (F)6.66406,
    (F) 12.3203,  (F)-3.57031,  (F)5.46875,
    (F) 7.32031,  (F)-5,  (F)-4.42969,
    (F) 11.7812,  (F)-4.125,  (F)-8,
    (F)-1.75781,  (F)-4.11719,  (F)-3.33594,
    (F)-4.40625,  (F)-8.55469,  (F)-2.59375,
    (F)-5.79688,  (F)-2.75781,  (F)-1.45312,
    (F)-10.0625,  (F)-3.58594,  (F)0.65625,
    (F)-14.4766,  (F)-3.78125,  (F)-2.14062,
    (F)-12.1406,  (F)-5.71094,  (F)6.44531,
    (F)-18.9453,  (F)-1.82812,  (F)2.98438,
    (F)-13.4531,  (F)2.28906,  (F)-2.11719,
    (F)-19.6484,  (F)3.32031,  (F)-4.40625,
    (F)-20.3047,  (F)6.39062,  (F)3.625,
    (F)-2.79688,  (F)0.921875,  (F)-2.92969,
    (F)-4.78906,  (F)1.54688,  (F)-6.82812,
    (F)-5.39062,  (F)5.33594,  (F)-2.53125,
    (F)-10.5625,  (F)9.02344,  (F)-1.3125,
    (F)-8.53906,  (F)1.00781,  (F)-2.53125,
    (F)-9.25,  (F)4.77344,  (F)-6.70312,
    (F) 3.38281,  (F)3.62,  (F)-2.90625,
    (F)-0.9375,  (F)3.79688,  (F)-4.95312,
    (F) 1.82812,  (F)6.64844,  (F)-4.92969,
    (F) 4.70312,  (F)1,  (F)0.359375,
    (F) 8.58594,  (F)3.20312,  (F)1.24219,
    (F) 5.58594,  (F)-1.13281,  (F)-3.3125,
    (F) 9.35938,  (F)-0.664062,  (F)-4.63281,
    (F) 10.8594,  (F)2.86719,  (F)-2.79688,
    (F) 13.1484,  (F)2.82812,  (F)-7.53125,
    (F) 13.3672,  (F)4.85938,  (F)0.351562,
    (F) 15.8828,  (F)6.50781,  (F)-2.99219,
    (F) 22.5781,  (F)4.28906,  (F)-3.95312,
    (F) 0.0546875,  (F)3.13281,  (F)0.40625,
    (F) 0.5625,  (F)6.33594,  (F)3.24219,
    (F) 4.42188,  (F)5.21875,  (F)0.320312,
    (F) 3.64844,  (F)9.51562,  (F)1.62,
    (F) 0.53125,  (F)7.47656,  (F)-1.46094,
    (F)-0.195312,  (F)12.8828,  (F)-0.804688,
    (F)-5.16406,  (F)2.03125,  (F)1.67188,
    (F)-7.22656,  (F)-0.734375,  (F)4.78125,
    (F)-2.50781,  (F)-3.29688,  (F)7.53906,
    (F)-6.15625,  (F)-5.25,  (F)13.9297,
    (F) 3.125,  (F)-5.25781,  (F)6.08594,
    (F) 5.78906,  (F)-4.64844,  (F)12.7734,
    (F)-1.25781,  (F)2.39844,  (F)5.13281,
    (F)-2.98438,  (F)6.53125,  (F)6.80469,
    (F)-6.35938,  (F)3.125,  (F)9.55469,
    (F) 2.84375,  (F)4.73438,  (F)9.74219,
    (F) 0.445312,  (F)0.585938,  (F)12.2734,
    (F) 1.17969,  (F)3.67969,  (F)17.8672,
    (F)-0.632812,  (F)7.97656,  (F)11.7344,
    (F) 1.77344,  (F)11.4844,  (F)8.57031,
    (F) 5.14062,  (F)9.96094,  (F)12.9141,
    (F) 5.1875,  (F)11.5469,  (F)18.5703,
    (F) 2.05469,  (F)-0.992188,  (F)3.46875,
    (F) 2.0625,  (F)0.695312,  (F)7.57031,
    (F) 6.20312,  (F)1.33594,  (F)4.5,
    (F) 6.41406,  (F)1.45312,  (F)9.57812,
    (F) 3.15625,  (F)3.60938,  (F)4.03906,
    (F) 2.64844,  (F)7.17188,  (F)6.20312,
    (F) 10,  (F)11.1172,  (F)1.53125,
    (F) 11.3516,  (F)15.7734,  (F)2.85156,
    (F) 12.6172,  (F)11.5703,  (F)5.25,
    (F) 18.7031,  (F)13.3438,  (F)3.96875,
    (F) 6.29688,  (F)9.67188,  (F)4.47656,
    (F) 5.57031,  (F)9.04688,  (F)8.42188,
    (F) 10.0391,  (F)11.2188,  (F)9.625,
    (F) 13.9844,  (F)9.28125,  (F)8.91406,
    (F) 5.97656,  (F)5.02344,  (F)6.75,
    (F) 8.0625,  (F)6.22656,  (F)9.99219,
    (F) 7.03125,  (F)4.39844,  (F)14.2734,
    (F) 11.8281,  (F)5.25781,  (F)18.0625,
    (F) 12.0625,  (F)8.82031,  (F)25.3125,
    (F) 11.4766,  (F)8.20312,  (F)12.4531,
    (F) 15.1953,  (F)10.2969,  (F)15.5312,
    (F) 18.7266,  (F)15.8281,  (F)16.0938,
    (F) 27.6406,  (F)16.0703,  (F)20.4688,
    (F) 9.75781,  (F)-0.921875,  (F)0.578125,
    (F) 13.4922,  (F)1.51562,  (F)3.28906,
    (F) 16.2344,  (F)-0.390625,  (F)-3.61719,
    (F) 7.57812,  (F)5.375,  (F)-3.375,
    (F) 8.97656,  (F)7.4375,  (F)-0.859375,
    (F) 9.79688,  (F)9.96094,  (F)-5.08594,
    (F) 10.8281,  (F)7.25781,  (F)3.13281,
    (F) 15.3125,  (F)9.11719,  (F)1.8125,
    (F) 3.17969,  (F)-5.875,  (F)-1.89844,
    (F) 6.71094,  (F)-8.73438,  (F)1.34375,
    (F)-1.77344,  (F)-6.71875,  (F)-7.75,
    (F)-6.21875,  (F)-9.17969,  (F)-10.7812,
    (F) 0.0625,  (F)-10.6,  (F)-3.03125,
    (F) 2.75781,  (F)-13.9141,  (F)-8.16,
    (F)-5.71875,  (F)-3.16,  (F)-6.30469,
    (F)-6.66406,  (F)-0.21875,  (F)-10.4844,
    (F)-9.66,  (F)-9.42969,  (F)-15.3047,
    (F)-7.92969,  (F)-8.78906,  (F)-19.8672,
    (F)-11.3438,  (F)-13.9922,  (F)-21.2891,
    (F)-14.7734,  (F)-18.875,  (F)-25.5391,
    (F)-7.17188,  (F)-12.7266,  (F)-6.11719,
    (F)-4.53125,  (F)-16.8359,  (F)-5.45312,
    (F)-8.57031,  (F)-18.4688,  (F)-0.75,
    (F)-10.8359,  (F)-11.8203,  (F)0.0546875,
    (F)-12.4062,  (F)-16.2188,  (F)-5.1875,
    (F)-11.5078,  (F)-21.4062,  (F)-6.125,
    (F)-18.5781,  (F)-24.6016,  (F)-0.4375,
    (F)-22.3125,  (F)-16.3438,  (F)-1.32031,
    (F)-26.4922,  (F)-28.9688,  (F)2.46875,
    (F)-9.1875,  (F)-6.95312,  (F)-4.79688,
    (F)-15.1875,  (F)-8.28906,  (F)-6.25,
    (F)-2.33594,  (F)-11.8516,  (F)-7.8125,
    (F)-5,  (F)-14.4531,  (F)-11.9219,
    (F)-11.3594,  (F)-12,  (F)-9.63281,
    (F)-14.7656,  (F)-15.2734,  (F)-13.7188,
    (F)-8.48438,  (F)-15.0078,  (F)-16.5781,
    (F)-5.85938,  (F)-18.1641,  (F)-17.6562,
    (F)-10.3516,  (F)-18.8516,  (F)-20.8828,
    (F)-8.70312,  (F)-19.8594,  (F)-27.0234,
    (F)-10.4766,  (F)-22.6641,  (F)-30.9609,
    (F)-9.0625,  (F)-17.3906,  (F)-10.8828,
    (F)-7.82031,  (F)-21.6562,  (F)-12.2891,
    (F)-12.4531,  (F)-20.6328,  (F)-15.4531,
    (F)-11.9844,  (F)-23.7969,  (F)-20.5,
    (F)-17.375,  (F)-17.1562,  (F)-5.27344,
    (F)-21.9297,  (F)-23.8594,  (F)-8.60938,
    (F)-15.6875,  (F)-22.1875,  (F)-9.26562,
    (F)-15.4688,  (F)-26.0312,  (F)-13.2422,
    (F)-13.3984,  (F)-27.8281,  (F)-3.94531,
    (F)-19.7422,  (F)-31.25,  (F)-14.7422,
    (F)-18.2266,  (F)-21.3594,  (F)-17.9375,
    (F)-26.875,  (F)-18.7578,  (F)-25.0859,
    (F)-15.6875,  (F)-27.5625,  (F)-26.0703,
    (F)-17.5547,  (F)-25.7109,  (F)-35.2344,
    (F)-3.71094,  (F)7.41406,  (F)1.21094,
    (F)-1.16406,  (F)10.6641,  (F)4.25781,
    (F)-5.91406,  (F)12.8438,  (F)8.46094,
    (F)-1.69531,  (F)16.0391,  (F)10.5703,
    (F)-11.1953,  (F)17.9766,  (F)11.4219,
    (F) 3.86719,  (F)12.9766,  (F)4.13281,
    (F) 0.078125,  (F)15.9141,  (F)5.78125,
    (F) 16.2656,  (F)14.8281,  (F)7.64062,
    (F) 22.1172,  (F)17.9219,  (F)7.11719,
    (F) 6.57812,  (F)6.19531,  (F)3.28125,
    (F) 9.55469,  (F)7.99219,  (F)6.74219,
    (F) 9.66406,  (F)3.52344,  (F)6.09375,
    (F) 13.625,  (F)5.53125,  (F)6.42188,
    (F) 11.9766,  (F)2.21875,  (F)10.8125,
    (F) 16.6797,  (F)4.75781,  (F)12.6,
    (F) 18.0078,  (F)4.89062,  (F)4.21875,
    (F) 18.5,  (F)9.27344,  (F)7.46094,
    (F) 20.2031,  (F)9.80469,  (F)-0.0546875,
    (F) 24.3438,  (F)8.83594,  (F)5.67188,
    (F) 26.8047,  (F)13.5156,  (F)3.34375,
    (F) 21.3594,  (F)12.875,  (F)10.2109,
    (F) 23.3438,  (F)9.61719,  (F)15.7031,
    (F) 27.5234,  (F)15.8125,  (F)11.625,
    (F) 8.07812,  (F)12.9453,  (F)6.80469,
    (F) 9.42188,  (F)16.8984,  (F)8.09375,
    (F) 14.1172,  (F)18.5312,  (F)9.63281,
    (F) 14.125,  (F)13.8203,  (F)11.6641,
    (F) 12.2891,  (F)18.2188,  (F)13.8984,
    (F) 9.5,  (F)12.5703,  (F)14.5781,
    (F) 11.5625,  (F)14.8281,  (F)19.6328,
    (F) 3.63281,  (F)21.1562,  (F)21.6875,
    (F) 6.02344,  (F)28.4219,  (F)26.0781,
    (F) 14.6016,  (F)20.3047,  (F)20.4922,
    (F) 22.0703,  (F)21.4453,  (F)19.2812,
    (F) 20.6094,  (F)19.0547,  (F)12.3828,
    (F) 16.3594,  (F)23.2031,  (F)10.2188,
    (F) 15.7969,  (F)23.8828,  (F)15.6172,
    (F) 26.7812,  (F)22.75,  (F)14.1875,
    (F) 35.4375,  (F)23.2344,  (F)16.6328,
    (F) 8.17188,  (F)20.0234,  (F)16.2969,
    (F) 10.6484,  (F)27.8281,  (F)18.1094,
    (F) 19.6172,  (F)14.7734,  (F)23.3125,
    (F) 14.5781,  (F)18.6,  (F)31.1406,
    (F) 25.5625,  (F)22.8438,  (F)28.7656,
    (F) 24.5781,  (F)27.8438,  (F)17.75,
    (F) 28.0625,  (F)31,  (F)22.0469,
    (F) 18.9922,  (F)26.3203,  (F)23.9766,
    (F) 15.0859,  (F)36.7891,  (F)24.1406,
    (F) 20.5391,  (F)32.9688,  (F)31.1406,
    (F) 36.2422,  (F)31.6875,  (F)28.625,
    (F) 31.5234,  (F)41.6719,  (F)38.375,
};


/*
**  Band2Tb8:
**
**  Purpose:
**      Vector Quantize the last 4-vector of the 10 LSP parameters.
**
**  Table Structure:
**      8 bit, 256 entry table.
*/

FLOAT Band2Tb8[LspCbSize*4] = {
    (F) 0,  (F)0,  (F)0,  (F)0,
    (F) 4.69531,  (F)4,  (F)-4.23438,  (F)2.60938,
    (F) 3.34375,  (F)8.49219,  (F)-3.78125,  (F)-1.03125,
    (F) 5.09375,  (F)4.85938,  (F)-3.05469,  (F)-4.46875,
    (F) 2.95312,  (F)6.24219,  (F)1.16,  (F)-6.71875,
    (F) 8.125,  (F)3.19531,  (F)0.875,  (F)-4.32812,
    (F) 8.77344,  (F)5.23438,  (F)-0.585938,  (F)-6.61719,
    (F) 11.1016,  (F)3.85938,  (F)-2.46094,  (F)-8.55469,
    (F) 6.14844,  (F)7.82031,  (F)0.890625,  (F)-3.59375,
    (F) 7.71875,  (F)13.0625,  (F)1.6875,  (F)-5.32031,
    (F) 7.86719,  (F)9.69531,  (F)-1.03125,  (F)-9.74219,
    (F) 8.38281,  (F)3.11719,  (F)1.45312,  (F)-0.0390625,
    (F) 9.85938,  (F)1.50781,  (F)-5.42188,  (F)-1.00781,
    (F) 2.53906,  (F)1.53125,  (F)0.398438,  (F)-5.00781,
    (F) 6.72656,  (F)-0.460938,  (F)2.73438,  (F)-3.57812,
    (F) 9.85156,  (F)4.42969,  (F)4.57812,  (F)-2.70312,
    (F) 11.9688,  (F)6.91406,  (F)1.64062,  (F)-4.03906,
    (F) 15.8359,  (F)7.32031,  (F)0.882812,  (F)-6.1875,
    (F) 10.8047,  (F)8.3125,  (F)2.60938,  (F)0.296875,
    (F) 15.3438,  (F)11.4688,  (F)3.58594,  (F)1.03906,
    (F) 16.1094,  (F)9.26562,  (F)-0.765625,  (F)-0.945312,
    (F) 20.1328,  (F)11.2891,  (F)3.95312,  (F)-2.91406,
    (F) 18.0469,  (F)13.1406,  (F)-0.625,  (F)-7.5,
    (F) 22.4688,  (F)15.1484,  (F)5.97656,  (F)1.07812,
    (F) 27.9766,  (F)18.4375,  (F)5.07031,  (F)-3.23438,
    (F) 1.71094,  (F)1.375,  (F)-3.10938,  (F)-2.41406,
    (F) 3.39062,  (F)-0.609375,  (F)-3.39844,  (F)-6.875,
    (F)-2.6875,  (F)2.35156,  (F)2.07031,  (F)-4.3125,
    (F)-7.14844,  (F)3.67188,  (F)5.13281,  (F)-2.96875,
    (F) 3.27344,  (F)-3.375,  (F)-1.27344,  (F)-3.53906,
    (F) 2.74219,  (F)-7.44531,  (F)0.0625,  (F)-4.39062,
    (F) 6.16406,  (F)-0.335938,  (F)0.15625,  (F)-7.48438,
    (F) 2.35938,  (F)-4.64062,  (F)-2.75,  (F)-9.05469,
    (F) 8.125,  (F)0.84375,  (F)-5.21875,  (F)-7.21875,
    (F) 10.4141,  (F)1.64062,  (F)-9.50781,  (F)-12.9922,
    (F) 3.77344,  (F)4.66,  (F)-2.73438,  (F)-8.90625,
    (F) 7.83594,  (F)6.4375,  (F)-6.26562,  (F)-9.25,
    (F) 5.82031,  (F)0.453125,  (F)-4.66,  (F)-11.2734,
    (F) 2.70312,  (F)1.92969,  (F)-7.14844,  (F)-13.1484,
    (F) 2.10938,  (F)6.21875,  (F)-5.625,  (F)-15.9609,
    (F) 9.4375,  (F)5.64062,  (F)-1.73438,  (F)-1.50781,
    (F) 11.6094,  (F)9.21875,  (F)-3.21875,  (F)-5.25,
    (F) 13.4531,  (F)1.39844,  (F)-0.539062,  (F)-4.07031,
    (F) 15.9922,  (F)6.71875,  (F)-5.20312,  (F)-11.6,
    (F)-1.14062,  (F)1.73438,  (F)-2.19531,  (F)-6.28906,
    (F)-1.47656,  (F)0.703125,  (F)-0.890625,  (F)-10.2109,
    (F)-1.1875,  (F)8.48438,  (F)-1.88281,  (F)-5.96875,
    (F)-3.42969,  (F)5.72656,  (F)-4.69531,  (F)-10.1719,
    (F)-6.50781,  (F)-1.30469,  (F)-2.74219,  (F)-4.69531,
    (F)-6.6875,  (F)-3.29688,  (F)-3.21094,  (F)-8.27344,
    (F)-5.83594,  (F)-2.77344,  (F)-4.54688,  (F)-12.8438,
    (F)-6.53906,  (F)1.64062,  (F)-7.15625,  (F)-8.9375,
    (F)-14.0625,  (F)0.25,  (F)-6.85938,  (F)-13.1797,
    (F)-0.375,  (F)-0.179688,  (F)-8.95312,  (F)0.40625,
    (F)-2.73438,  (F)-3.19531,  (F)-12.9375,  (F)-2.84375,
    (F) 2.07031,  (F)-5.6875,  (F)-6.70312,  (F)-4.50781,
    (F) 3.57812,  (F)-1.92969,  (F)-8.91406,  (F)-7.78906,
    (F) 5.39844,  (F)-3.17969,  (F)-15.5312,  (F)-9.07031,
    (F)-0.515625,  (F)-0.8125,  (F)-5.50781,  (F)-9.75781,
    (F)-3.36719,  (F)-0.726562,  (F)-9.30469,  (F)-14.4062,
    (F) 1.58594,  (F)-5.71875,  (F)-7.8125,  (F)-13.2266,
    (F) 0.078125,  (F)-6.5,  (F)-14.4219,  (F)-14.2109,
    (F) 3.85156,  (F)-1,  (F)-11.2188,  (F)-13.8125,
    (F) 3.8125,  (F)-2.42969,  (F)-13.5156,  (F)-19.8438,
    (F)-5.16,  (F)-4.15625,  (F)-8.98438,  (F)-9.15625,
    (F)-8.48438,  (F)-2.25781,  (F)-13.3281,  (F)-11.9766,
    (F)-5.46094,  (F)-9.41406,  (F)-9.5,  (F)-13.7969,
    (F)-8.0625,  (F)-11.5703,  (F)-16.2031,  (F)-11.8984,
    (F)-5.63281,  (F)-9.53125,  (F)-17.7891,  (F)-20.3125,
    (F) 0.09375,  (F)-4.21094,  (F)-11.5938,  (F)-8.83594,
    (F)-0.3125,  (F)-7.11719,  (F)-16.4531,  (F)-3.44531,
    (F)-3.67969,  (F)-3.78125,  (F)-17.7109,  (F)-12.1016,
    (F)-1.16,  (F)-7.71875,  (F)-23.4844,  (F)-13.4453,
    (F)-12.0703,  (F)-16.4219,  (F)-4.55469,  (F)2.67188,
    (F)-10.8047,  (F)-21.6562,  (F)-3.62,  (F)-0.101562,
    (F)-16.5469,  (F)-20.2266,  (F)-9.41406,  (F)0.5625,
    (F)-16.7734,  (F)-25.2422,  (F)-7.53906,  (F)3.04688,
    (F)-23.0391,  (F)-25.7812,  (F)-4.85156,  (F)4.97656,
    (F)-30.5234,  (F)-32.3281,  (F)-6.75781,  (F)6.27344,
    (F)-10.0547,  (F)-6.66,  (F)-2.92969,  (F)-4.28125,
    (F)-11.0625,  (F)-9.13281,  (F)-3.80469,  (F)-9.97656,
    (F)-10.9375,  (F)-13.2031,  (F)-8.02344,  (F)-3.26562,
    (F)-15.7656,  (F)-14.9141,  (F)-9.28125,  (F)-9.84375,
    (F)-11.0781,  (F)-17.3594,  (F)-15.8516,  (F)-1,
    (F)-16.1484,  (F)-23.4219,  (F)-21.0391,  (F)-2.42188,
    (F)-7.42188,  (F)-8.03125,  (F)-12.6,  (F)1.44531,
    (F)-12.625,  (F)-7.14844,  (F)-17.2266,  (F)-4.28906,
    (F) 0.148438,  (F)-6.41406,  (F)-8.94531,  (F)2.75,
    (F) 1.4375,  (F)-9.17969,  (F)-10.5938,  (F)-4.89844,
    (F)-4.27344,  (F)-8.5,  (F)-12.9766,  (F)-7.11719,
    (F)-1.6875,  (F)-11.7344,  (F)-17.1641,  (F)-7.40625,
    (F)-6.21094,  (F)-10.2031,  (F)-18.5469,  (F)-3.52344,
    (F)-7.21875,  (F)-14.7578,  (F)-21.8438,  (F)-5.3125,
    (F)-4.6875,  (F)-12.6094,  (F)-28.1953,  (F)-6.91406,
    (F)-18.6875,  (F)-19.75,  (F)2.49219,  (F)2.36719,
    (F)-22.7188,  (F)-16.3672,  (F)-2.42188,  (F)4.47656,
    (F)-27.0312,  (F)-16.7266,  (F)0.382812,  (F)-0.882812,
    (F)-17.4297,  (F)-3.5,  (F)5.27344,  (F)-1.14062,
    (F)-21.9141,  (F)-4.15625,  (F)9.61719,  (F)3.74219,
    (F)-20.9688,  (F)-3.79688,  (F)-1.5625,  (F)4.77344,
    (F)-27.5391,  (F)-7.58594,  (F)-1.54688,  (F)5.5,
    (F)-28.9609,  (F)1.35156,  (F)2.72656,  (F)1.98438,
    (F)-36.9844,  (F)-11.3047,  (F)-0.265625,  (F)6.875,
    (F) 6.07031,  (F)-4,  (F)0.890625,  (F)-0.078125,
    (F) 9.76562,  (F)-0.515625,  (F)3.45312,  (F)-0.0390625,
    (F) 4.71875,  (F)4.78906,  (F)3.53125,  (F)-2.75,
    (F) 9.5625,  (F)6.07031,  (F)5.27344,  (F)-7.92188,
    (F)-10.7188,  (F)-0.617188,  (F)-9.4375,  (F)-1.85938,
    (F)-18.6641,  (F)-0.132812,  (F)-9.03906,  (F)-6.39062,
    (F)-11.75,  (F)-5.25781,  (F)-8.85156,  (F)-8.28125,
    (F)-15.5,  (F)-6.24219,  (F)-15.6641,  (F)-15.4141,
    (F)-15.9141,  (F)-6.23438,  (F)-8.34375,  (F)-0.820312,
    (F)-24.9219,  (F)-7.02344,  (F)-14.1953,  (F)-1.51562,
    (F)-1.21875,  (F)-6.92188,  (F)3.07812,  (F)-2.48438,
    (F)-2.62,  (F)-10.0234,  (F)4.30469,  (F)1.57812,
    (F)-4.1875,  (F)-13.5078,  (F)7.10938,  (F)2.58594,
    (F)-6.61719,  (F)-8.66406,  (F)6.21094,  (F)-1.27344,
    (F)-9.14844,  (F)-8.8125,  (F)5.58594,  (F)4.05469,
    (F)-8.4375,  (F)-10.3047,  (F)13.1641,  (F)5.21875,
    (F)-7.8125,  (F)-15.0078,  (F)0.75,  (F)1.64844,
    (F)-11.6172,  (F)-16.7812,  (F)6.49219,  (F)1.35938,
    (F)-8.89844,  (F)-2.92188,  (F)3.23438,  (F)-0.03125,
    (F)-11.8516,  (F)-10.8047,  (F)3.09375,  (F)-2.75,
    (F)-7.90625,  (F)3.42969,  (F)-0.460938,  (F)-7.55469,
    (F)-14.1562,  (F)5.51562,  (F)-3.4375,  (F)-8.04688,
    (F)-15.3984,  (F)-2.57031,  (F)-0.265625,  (F)-6.46094,
    (F)-19.3125,  (F)-12.4062,  (F)-1.17969,  (F)-4.73438,
    (F)-16.8828,  (F)2.92188,  (F)-2.19531,  (F)0.59375,
    (F)-23.5312,  (F)1.80469,  (F)-0.117188,  (F)-5.39062,
    (F) 8.625,  (F)4.42188,  (F)5.63281,  (F)1.63281,
    (F) 13.1641,  (F)4.40625,  (F)2.99219,  (F)0.765625,
    (F) 14.8281,  (F)5.85938,  (F)6.1875,  (F)-0.757812,
    (F) 4.34375,  (F)-0.5,  (F)4.38281,  (F)-0.726562,
    (F) 6.84375,  (F)1.26562,  (F)7.13281,  (F)-0.171875,
    (F) 7.50781,  (F)5.27344,  (F)10.125,  (F)1.09375,
    (F) 5.90625,  (F)-3.09375,  (F)6.64844,  (F)4.25,
    (F) 2.8125,  (F)-2.36719,  (F)10.4766,  (F)3.09375,
    (F) 6.85938,  (F)-0.171875,  (F)11.4375,  (F)6.74219,
    (F)-2.41406,  (F)-2.13281,  (F)5.62,  (F)-1.00781,
    (F)-5.35938,  (F)-0.640625,  (F)6.57812,  (F)3.54688,
    (F)-0.0390625,  (F)-0.367188,  (F)8.35156,  (F)7.79688,
    (F)-0.734375,  (F)7.55469,  (F)9.97656,  (F)2.32812,
    (F)-3.82031,  (F)3.00781,  (F)11.5078,  (F)5.82812,
    (F)-2.88281,  (F)-5.66,  (F)10.4141,  (F)1.89062,
    (F) 2.19531,  (F)-7.75781,  (F)13.4844,  (F)7.21875,
    (F) 3.625,  (F)4.69531,  (F)12.3047,  (F)10.75,
    (F)-1.95312,  (F)1.60938,  (F)18.2734,  (F)9.17969,
    (F)-3.42188,  (F)2.94531,  (F)-4.66406,  (F)-2.22656,
    (F)-7.96875,  (F)6.14844,  (F)-6.17188,  (F)-2.24219,
    (F)-3.57812,  (F)-3.20312,  (F)1.67969,  (F)2.30469,
    (F)-4.66,  (F)-6.71875,  (F)-0.945312,  (F)6.22656,
    (F)-9.17969,  (F)0.953125,  (F)-3.41406,  (F)3.64062,
    (F)-11.5625,  (F)-0.945312,  (F)2.86719,  (F)7.21875,
    (F) 1.82812,  (F)2.52344,  (F)6.62,  (F)-4.33594,
    (F) 1.13281,  (F)0.234375,  (F)7.78125,  (F)0.203125,
    (F) 0.515625,  (F)6.63281,  (F)0.726562,  (F)-1.13281,
    (F)-0.914062,  (F)9.85156,  (F)3.70312,  (F)-3.11719,
    (F)-11.6797,  (F)8.21094,  (F)1.70312,  (F)-3.95312,
    (F)-10.8594,  (F)5.42188,  (F)7.76562,  (F)0.6875,
    (F) 4.8125,  (F)0.0546875,  (F)0.609375,  (F)2.375,
    (F) 8.28125,  (F)0.40625,  (F)-0.484375,  (F)6.52344,
    (F) 6.50781,  (F)3.54688,  (F)5.07031,  (F)10.6172,
    (F)-6.62,  (F)3.625,  (F)0.367188,  (F)0.726562,
    (F)-4.48438,  (F)9.36719,  (F)-0.304688,  (F)2.96094,
    (F) 0.890625,  (F)-0.765625,  (F)3.8125,  (F)3.78906,
    (F) 5.67969,  (F)1.90625,  (F)4.73438,  (F)5.4375,
    (F)-0.59375,  (F)3.55469,  (F)5.24219,  (F)4.26562,
    (F)-4.41406,  (F)-0.162,  (F)1.13281,  (F)6.39844,
    (F)-2.9375,  (F)4.44531,  (F)3.5,  (F)8.8125,
    (F) 1.70312,  (F)0.953125,  (F)2.07031,  (F)9.11719,
    (F) 1.79688,  (F)5.76562,  (F)7.28125,  (F)7.83594,
    (F) 1.07812,  (F)3.72656,  (F)0.28125,  (F)3.51562,
    (F) 3.15625,  (F)6.14844,  (F)-0.570312,  (F)7.8125,
    (F) 3.88281,  (F)9.83594,  (F)3.02344,  (F)9.61719,
    (F) 0.132812,  (F)1.61719,  (F)1.52344,  (F)-0.617188,
    (F) 4.39062,  (F)2.79688,  (F)0.414062,  (F)-1.23438,
    (F) 3.85156,  (F)3.02344,  (F)3.73438,  (F)1.47656,
    (F) 5.29688,  (F)6.49219,  (F)5,  (F)4.35938,
    (F)-1.53906,  (F)4.08594,  (F)4.78906,  (F)0.445312,
    (F) 3.35156,  (F)6.98438,  (F)6.00781,  (F)0.867188,
    (F) 0.523438,  (F)9.17188,  (F)4.4375,  (F)3.99219,
    (F) 9.70312,  (F)6.4375,  (F)1.96094,  (F)6.5625,
    (F) 11.0859,  (F)8.39062,  (F)6.75,  (F)3.75781,
    (F) 7.21875,  (F)11.5156,  (F)5.22656,  (F)5.65625,
    (F) 12.0234,  (F)14.6797,  (F)5.10938,  (F)12.4219,
    (F) 3.47656,  (F)2.63281,  (F)8.67969,  (F)4.22656,
    (F) 3.6875,  (F)11.1016,  (F)9.875,  (F)8.54688,
    (F) 6.20312,  (F)5.74219,  (F)8.61719,  (F)5.21875,
    (F) 8.24219,  (F)6.74219,  (F)9.3125,  (F)7.96875,
    (F) 6.07812,  (F)8.63281,  (F)6.29688,  (F)14.0469,
    (F) 8.21875,  (F)11.9297,  (F)12.3984,  (F)16.8047,
    (F) 6.88281,  (F)12.125,  (F)9.88281,  (F)3.05469,
    (F) 5.67188,  (F)6.8125,  (F)14.1562,  (F)4.69531,
    (F) 11.4766,  (F)2.1875,  (F)7.875,  (F)4.8125,
    (F) 10.9609,  (F)4.50781,  (F)14.0859,  (F)9.71875,
    (F) 12.8906,  (F)10.2656,  (F)8.96875,  (F)8.375,
    (F) 10.1328,  (F)13.0391,  (F)14.9297,  (F)8.62,
    (F) 16.3516,  (F)8.15625,  (F)16.5234,  (F)9.28906,
    (F) 12.8438,  (F)15.3203,  (F)20.2109,  (F)11.8125,
    (F) 0.195312,  (F)-2.46094,  (F)-0.0703125,  (F)-0.828125,
    (F) 2.26562,  (F)-2.64844,  (F)3.34375,  (F)-3.46875,
    (F)-0.53125,  (F)-6.11719,  (F)5.74219,  (F)6.03125,
    (F) 1.91406,  (F)-4.33594,  (F)3.65625,  (F)0.367188,
    (F) 2.60938,  (F)-6.99219,  (F)6.35938,  (F)1.14062,
    (F) 1.83594,  (F)2.875,  (F)-7.53125,  (F)-7.49219,
    (F)-1.58594,  (F)2.46094,  (F)-12.2344,  (F)-9.50781,
    (F) 6.25781,  (F)0.132812,  (F)-2.15625,  (F)-2.76562,
    (F) 6.98438,  (F)-3.86719,  (F)-6.16406,  (F)-4.96094,
    (F) 5.59375,  (F)2.27344,  (F)-9.28906,  (F)-2.78906,
    (F) 4.375,  (F)-2.03125,  (F)-5.72656,  (F)-0.625,
    (F) 5.30469,  (F)-3.96875,  (F)-11.1641,  (F)1.64844,
    (F)-0.398438,  (F)-0.484375,  (F)-3.34375,  (F)4.35156,
    (F) 2.51562,  (F)-4.98438,  (F)-1.64844,  (F)4.79688,
    (F)-6.85938,  (F)-8.25781,  (F)-0.65625,  (F)-0.554688,
    (F)-3.03125,  (F)-11.0547,  (F)-1.30469,  (F)-2.48438,
    (F)-5.89062,  (F)-12.2969,  (F)1.67188,  (F)-4.21094,
    (F)-11.0859,  (F)-15.6562,  (F)-0.71875,  (F)-6.14844,
    (F)-0.367188,  (F)-6.6875,  (F)-2.71094,  (F)-1.99219,
    (F) 0.179688,  (F)-9.46094,  (F)-1.35156,  (F)2.5,
    (F)-5.14062,  (F)-3.80469,  (F)-6.97656,  (F)2.75781,
    (F)-6.11719,  (F)-12.3984,  (F)-4.5625,  (F)3.96094,
    (F)-11.0938,  (F)-6.71094,  (F)-2.95312,  (F)3.44531,
    (F)-16.3672,  (F)-11.6484,  (F)-1.07031,  (F)3.42969,
    (F)-2.50781,  (F)-11.3281,  (F)-10.0625,  (F)-0.09375,
    (F)-2.80469,  (F)-16.5078,  (F)-4.32031,  (F)-0.0625,
    (F)-6.49219,  (F)-14.9844,  (F)-12.1953,  (F)0.25,
    (F)-7.92188,  (F)-19.4297,  (F)-10.6172,  (F)-7.33594,
    (F)-3.71094,  (F)-2.42969,  (F)-1.32031,  (F)-1.84375,
    (F)-7.08594,  (F)-3.32812,  (F)2.15625,  (F)-4.77344,
    (F)-0.75,  (F)-3.125,  (F)0.390625,  (F)-5.54688,
    (F)-3.32812,  (F)-7.98438,  (F)-0.078125,  (F)-7.69531,
    (F)-1.53906,  (F)-2.62,  (F)-5.8125,  (F)-4.49219,
    (F)-4.77344,  (F)-7.26562,  (F)-6.02344,  (F)-3.07812,
    (F)-2.08594,  (F)-6.0625,  (F)-4.78125,  (F)-7.33594,
    (F)-2,  (F)-10.5156,  (F)-6.26562,  (F)-8.76562,
    (F)-6.21875,  (F)-12.2656,  (F)-6.44531,  (F)-5.89062,
    (F) 5.5625,  (F)6.84375,  (F)1.16,  (F)1.77344,
    (F) 7.66406,  (F)11.7891,  (F)0.664062,  (F)0.96875,
    (F) 11.4219,  (F)9.59375,  (F)7.64844,  (F)-0.304688,
    (F) 13.5469,  (F)7.80469,  (F)11.5703,  (F)3.4375,
    (F) 17.9141,  (F)8.71875,  (F)6.00781,  (F)3.4375,
    (F) 19.5625,  (F)11.5625,  (F)9.69531,  (F)2.78125,
    (F) 19.3281,  (F)14.9141,  (F)12.1719,  (F)6.32812,
    (F) 7.16406,  (F)8.85938,  (F)4.74219,  (F)-1.04688,
    (F) 3.97656,  (F)14.1328,  (F)6.16,  (F)-0.960938,
    (F) 13.375,  (F)11.7656,  (F)4.36719,  (F)-3.30469,
    (F) 15.9141,  (F)18.1016,  (F)5.67188,  (F)-1.21094,
    (F) 23.6797,  (F)20.9062,  (F)9.39844,  (F)2.58594,
    (F) 28.625,  (F)25.5781,  (F)13.8125,  (F)4.14844,
    (F) 12.5781,  (F)14.3672,  (F)6.77344,  (F)1.42969,
    (F) 13.8594,  (F)15.4062,  (F)12.6,  (F)0.757812,
    (F) 14.2344,  (F)16.8594,  (F)10.6,  (F)5.14844,
    (F) 17.3594,  (F)21.5469,  (F)14.2031,  (F)7.03125,
    (F) 25.3984,  (F)16.5938,  (F)13.4609,  (F)7.78125,
    (F) 28.3828,  (F)18.25,  (F)18.8125,  (F)11.3516,
    (F) 22.8359,  (F)27.4766,  (F)20.0547,  (F)10.2969,
};

/*
**  BandQntTable:
**
**  Purpose:
**      collects the three subvector tables.
*/

FLOAT *BandQntTable[LspQntBands] = {
   Band0Tb8 ,
   Band1Tb8 ,
   Band2Tb8 ,
   };

/*
**  PerFiltZeroTable:
**
**  Purpose:
**     Creates the FIR part of the formant perceptual weighting filter.
**     Corresponds to gamma1 in section 2.8.
**
**  Table Structure:
**     (0.9) to the x power, where x = [1,..,10]
*/

FLOAT PerFiltZeroTable[LpcOrder] = {
   (F)0.9 ,
   (F)0.81 ,
   (F)0.729 ,
   (F)0.6561 ,
   (F)0.59049 ,
   (F)0.531441 ,
   (F)0.4781969 ,
   (F)0.43046721 ,
   (F)0.38742049 ,
   (F)0.34867844 ,
};

/*
**  PerFiltPoleTable:
**
**  Purpose:
**     Creates the IIR part of the formant perceptual weighting filter.
**     Corresponds to gamma2 in section 2.8.
**
**  Table Structure:
**     (0.5) to the x power, where x = [1,..,10]
*/

FLOAT PerFiltPoleTable[LpcOrder] = {
   (F)0.5 ,
   (F)0.25 ,
   (F)0.125 ,
   (F)0.0625 ,
   (F)0.03125 ,
   (F)0.625 ,
   (F)0.0078125 ,
   (F)0.00390625 ,
   (F)0.001953125 ,
   (F)0.0009765625 ,
};

/*
**  PostFiltZeroTable:
**
**  Purpose:
**     Creates the FIR part of the formant postfilter.  Corresponds to
**     lambda1 in section 3.8.
**
**  Table Structure:
**     (0.65) to the x power, where x = [1,..,10]
*/

FLOAT   PostFiltZeroTable[LpcOrder] = {
   (F)0.65 ,
   (F)0.4225 ,
   (F)0.274625 ,
   (F)0.17850625 ,
   (F)0.11602906 ,
   (F)0.075418891 ,
   (F)0.049022279 ,
   (F)0.031864483 ,
   (F)0.020711913 ,
   (F)0.013462743 ,
   };

/*
**  PostFiltPoleTable:
**
**  Purpose:
**     Creates the IIR part of the formant postfilter.  Corresponds to
**     lambda2 in section 3.8.
**
**  Table Structure:
**     (0.75) to the x power, where x = [1,..,10]
*/

FLOAT   PostFiltPoleTable[LpcOrder] = {
   (F)0.75 ,
   (F)0.5625 ,
   (F)0.421875 ,
   (F)0.31640625 ,
   (F)0.23730469 ,
   (F)0.177978752 ,
   (F)0.13348389 ,
   (F)0.10011292 ,
   (F)0.075084686 ,
   (F)0.056313515 ,
   };

/*
**  Nb_puls:
**
**  Purpose:
**      Indexing
**
**  Table Structure:
**      Table values are the number of non-zero pulses in the high-rate
**      excitation (MP-MLQ), indexed by subframe number (0,..,3).
*/

int Nb_puls[4] = { 6,5,6,5};

/*
**  FcbkGainTable:
**
**  Purpose:
**      Logarithmic scalar quantizer in 24 steps of 3.2 dB each
**
**  Table Structure:
**      Contains x where 20*log10(x) = 3.2*i,  i = 1,..,24
*/

FLOAT FcbkGainTable[NumOfGainLev] = {
   (F)2 ,
   (F)4 ,
   (F)6 ,
   (F)8 ,
   (F)12 ,
   (F)18 ,
   (F)26 ,
   (F)36 ,
   (F)52 ,
   (F)76 ,
   (F)110 ,
   (F)160 ,
   (F)230 ,
   (F)332 ,
   (F)480 ,
   (F)696 ,
   (F)1004 ,
   (F)1452 ,
   (F)2100 ,
   (F)3034 ,
   (F)4386 ,
   (F)6340 ,
   (F)9164 ,
   (F)13246 ,
};

/*
**  MaxPosTable:
**
**  Purpose:
**      size of the high rate fixed excitation codebooks (MP-MLQ)
**
**  Table Structure:
**      Table values are the number of codewords in the high rate fixed
**      excitation codebook (MP-MLQ), indexed by subframe number (0,..,3).
**      MaxPosTable[i] is the number of combinations of i elements among 30
**      non-zero pulses in the high-rate
*/

Word32   MaxPosTable[4] = {
 0x00090f6fL,
 0x00022caaL,
 0x00090f6fL,
 0x00022caaL,
};

/*
**  CombinatorialTable:
**
**  Purpose:
**      used for the coding and the decoding of the pulses positions
**      for the high-rate fixed excitation codebook
**
**  Table Structure:
**      CombinatorialTable[i][j] is the number of combinations of
**      (MaxPulseNum-1-i) elements among (SubFrLen-2-j)
*/

Word32   CombinatorialTable[MaxPulseNum][SubFrLen/Sgrid] = {
 { 118755L,
  98280L,
  80730L,
  65780L,
  53130L,
  42504L,
  33649L,
  26334,
  20349,
  15504,
  11628,
   8568,
   6188,
   4368,
   3003,
   2002,
   1287,
    792,
    462,
    252,
    126,
     56,
     21,
      6,
      1,
      0,
      0,
      0,
      0,
      0 },

 { 23751,
  20475,
  17550,
  14950,
  12650,
  10626,
   8855,
   7315,
   5985,
   4845,
   3876,
   3060,
   2380,
   1820,
   1365,
   1001,
    715,
    495,
    330,
    210,
    126,
     70,
     35,
     15,
      5,
      1,
      0,
      0,
      0,
      0 },

  { 3654,
   3276,
   2925,
   2600,
   2300,
   2024,
   1771,
   1540,
   1330,
   1140,
    969,
    816,
    680,
    560,
    455,
    364,
    286,
    220,
    165,
    120,
     84,
     56,
     35,
     20,
     10,
      4,
      1,
      0,
      0,
      0 },

   {  406,
    378,
    351,
    325,
    300,
    276,
    253,
    231,
    210,
    190,
    171,
    153,
    136,
    120,
    105,
     91,
     78,
     66,
     55,
     45,
     36,
     28,
     21,
     15,
     10,
      6,
      3,
      1,
      0,
      0 },

   {   29,
     28,
     27,
     26,
     25,
     24,
     23,
     22,
     21,
     20,
     19,
     18,
     17,
     16,
     15,
     14,
     13,
     12,
     11,
     10,
      9,
      8,
      7,
      6,
      5,
      4,
      3,
      2,
      1,
      0  },

   {    1,
      1,
      1,
      1,
      1,
      1,
      1,
      1,
      1,
      1,
      1,
      1,
      1,
      1,
      1,
      1,
      1,
      1,
      1,
      1,
      1,
      1,
      1,
      1,
      1,
      1,
      1,
      1,
      1,
      1 }
   };

/*
**  AcbkGainTable085:
**
**  Purpose:
**      Used to calculate the error expression in pitch prediction
**      optimization (also described as an adaptive codebook approach)
**
**      Table Structure:
**      The table is structured as 85 20-element vectors.  These
**      vectors are structured as precalculated values in the error
**      expression for the pitch predictor.  Gi is the gain value
**      multiplying the signal delayed by a pitch period (+/- offset).
**      In equation 41.2, Gi would be equivalent to bij.
**
**      1st 5 elements:   G1  G2  G3  G4  G5 in Q14
**      2nd 5 elements:  -G1^2  -G2^2  -G3^2  -G4^2  -G5^2
**      Next 10 elements: These are the off-diagonal elements
**           -G1*G2  -G1*G3  -G2*G3  -G1*G4  -G2*G4
**           -G3*G4  -G1*G5  -G2*G5  -G3*G5  -G4*G5
*/

 FLOAT   AcbkGainTable085[85*20] = {
 (F)0.000000, (F)0.000000, (F)0.000000, (F)0.000000, (F)0.000000, (F)0.000000,
(F)0.000000, (F)0.000000, (F)0.000000, (F)0.000000, (F)0.000000, (F)0.000000,
(F)0.000000, (F)0.000000, (F)0.000000, (F)0.000000, (F)0.000000, (F)0.000000,
(F)0.000000, (F)0.000000,
(F)0.048828, (F)0.091309, (F)0.010193, (F)-0.625, (F)-0.020630, (F)-0.002384,
(F)-0.008337, (F)-0.000104, (F)-0.000244, (F)-0.000426, (F)-0.004458, (F)-0.000498,
(F)-0.000931, (F)0.000763, (F)0.001427, (F)0.009, (F)0.001007, (F)0.001884,
(F)0.000210, (F)-0.000322,
(F)-0.028198, (F)-0.041870, (F)0.030090, (F)0.157166, (F)0.018982, (F)-0.000795,
(F)-0.001753, (F)-0.000905, (F)-0.024701, (F)-0.000360, (F)-0.001181, (F)0.000848,
(F)0.001260, (F)0.004432, (F)0.006581, (F)-0.004729, (F)0.000535, (F)0.000795,
(F)-0.000571, (F)-0.002983,
(F)0.090515, (F)0.008789, (F)0.047852, (F)0.056641, (F)0.075867, (F)-0.008193,
(F)-0.000077, (F)-0.002290, (F)-0.003208, (F)-0.005756, (F)-0.000796, (F)-0.004331,
(F)-0.000421, (F)-0.005127, (F)-0.000498, (F)-0.002710, (F)-0.006867, (F)-0.000667,
(F)-0.003630, (F)-0.004297,
(F)-0.004700, (F)0.016785, (F)0.214966, (F)0.064453, (F)-0.076538, (F)-0.000022,
(F)-0.000282, (F)-0.046210, (F)-0.004154, (F)-0.005858, (F)0.000079, (F)0.001010,
(F)-0.003608, (F)0.000303, (F)-0.001082, (F)-0.013855, (F)-0.000360, (F)0.001285,
(F)0.016453, (F)0.004933,
(F)-0.007629, (F)-0.002441, (F)-0.016113, (F)0.023254, (F)0.306824, (F)-0.000058,
(F)-0.000006, (F)-0.000260, (F)-0.000541, (F)-0.094141, (F)-0.000019, (F)-0.000123,
(F)-0.000039, (F)0.000177, (F)0.000057, (F)0.000375, (F)0.002341, (F)0.000749,
(F)0.004944, (F)-0.007135,
(F)0.008423, (F)0.020264, (F)0.135193, (F)0.157104, (F)0.081726, (F)-0.000071,
(F)-0.000411, (F)-0.018277, (F)-0.024682, (F)-0.006679, (F)-0.000171, (F)-0.001139,
(F)-0.002740, (F)-0.001323, (F)-0.003184, (F)-0.021239, (F)-0.000688, (F)-0.001656,
(F)-0.011049, (F)-0.012840,
(F)0.224915, (F)0.175964, (F)-0.054138, (F)0.052856, (F)-0.100037, (F)-0.050587,
(F)-0.030963, (F)-0.002931, (F)-0.002794, (F)-0.010007, (F)-0.039577, (F)0.012176,
(F)0.009526, (F)-0.011888, (F)-0.009301, (F)0.002862, (F)0.022500, (F)0.017603,
(F)-0.005416, (F)0.005288,
(F)0.085205, (F)0.130981, (F)0.136414, (F)0.021057, (F)0.057495, (F)-0.007260,
(F)-0.017156, (F)-0.018609, (F)-0.000443, (F)-0.003306, (F)-0.011160, (F)-0.011623,
(F)-0.017868, (F)-0.001794, (F)-0.002758, (F)-0.002872, (F)-0.004899, (F)-0.007531,
(F)-0.007843, (F)-0.001211,
(F)0.000793, (F)0.273132, (F)0.216614, (F)-0.049072, (F)-0.039978, (F)-0.000001,
(F)-0.074601, (F)-0.046922, (F)-0.002408, (F)-0.098, (F)-0.000217, (F)-0.000172,
(F)-0.059164, (F)0.000039, (F)0.013403, (F)0.010630, (F)0.000032, (F)0.010919,
(F)0.008660, (F)-0.001962,
(F)-0.044189, (F)0.503, (F)0.014771, (F)0.369202, (F)0.150269, (F)-0.001953,
(F)-0.000240, (F)-0.000218, (F)-0.136310, (F)-0.022581, (F)0.000685, (F)0.000653,
(F)-0.000229, (F)0.016315, (F)-0.005724, (F)-0.005453, (F)0.006640, (F)-0.002330,
(F)-0.002220, (F)-0.055479,
(F)0.038208, (F)-0.104553, (F)0.373596, (F)0.278381, (F)-0.064758, (F)-0.001460,
(F)-0.010931, (F)-0.139574, (F)-0.077496, (F)-0.004194, (F)0.003995, (F)-0.014274,
(F)0.039061, (F)-0.010636, (F)0.029106, (F)-0.104002, (F)0.002474, (F)-0.006771,
(F)0.024193, (F)0.018028,
(F)-0.021362, (F)0.084900, (F)0.487366, (F)0.031189, (F)-0.024719, (F)-0.000456,
(F)-0.007208, (F)-0.237525, (F)-0.000973, (F)-0.000611, (F)0.001814, (F)0.010411,
(F)-0.041377, (F)0.000666, (F)-0.002648, (F)-0.200, (F)-0.000528, (F)0.002099,
(F)0.012047, (F)0.000771,
(F)0.191895, (F)-0.032288, (F)0.037109, (F)0.154419, (F)0.236694, (F)-0.036824,
(F)-0.001042, (F)-0.001377, (F)-0.023845, (F)-0.056024, (F)0.006196, (F)-0.007121,
(F)0.001198, (F)-0.029632, (F)0.004986, (F)-0.005730, (F)-0.045420, (F)0.007642,
(F)-0.008784, (F)-0.036550,
(F)0.158020, (F)-0.042053, (F)0.185852, (F)0.341980, (F)-0.024658, (F)-0.024970,
(F)-0.001768, (F)-0.034541, (F)-0.116950, (F)-0.000608, (F)0.006645, (F)-0.029368,
(F)0.007816, (F)-0.054040, (F)0.014381, (F)-0.063558, (F)0.003896, (F)-0.001037,
(F)0.004583, (F)0.008433,
(F)0.189636, (F)0.031311, (F)0.083862, (F)-0.219360, (F)-0.300415, (F)-0.035962,
(F)-0.000980, (F)-0.007033, (F)-0.048119, (F)-0.090249, (F)-0.005938, (F)-0.903,
(F)-0.002626, (F)0.041599, (F)0.006868, (F)0.018396, (F)0.056970, (F)0.009406,
(F)0.025193, (F)-0.065899,
(F)0.150635, (F)0.366821, (F)0.067566, (F)-0.023804, (F)0.119690, (F)-0.022691,
(F)-0.134558, (F)-0.004565, (F)-0.000567, (F)-0.014326, (F)-0.055256, (F)-0.010178,
(F)-0.024785, (F)0.003586, (F)0.008732, (F)0.001608, (F)-0.018029, (F)-0.043905,
(F)-0.008087, (F)0.002849,
(F)0.291321, (F)0.168823, (F)0.144531, (F)0.296753, (F)-0.246826, (F)-0.084868,
(F)-0.028501, (F)-0.020889, (F)-0.088062, (F)-0.060923, (F)-0.049182, (F)-0.042105,
(F)-0.024400, (F)-0.086450, (F)-0.050099, (F)-0.042890, (F)0.071906, (F)0.041670,
(F)0.035674, (F)0.073246,
(F)0.101624, (F)-0.114685, (F)0.088074, (F)0.103821, (F)0.522583, (F)-0.010327,
(F)-0.013153, (F)-0.007757, (F)-0.010779, (F)-0.273093, (F)0.011655, (F)-0.008950,
(F)0.010101, (F)-0.010551, (F)0.011907, (F)-0.009144, (F)-0.053107, (F)0.059932,
(F)-0.046026, (F)-0.054255,
(F)0.165771, (F)0.549988, (F)-0.061462, (F)0.046082, (F)-0.108948, (F)-0.027480,
(F)-0.302487, (F)-0.003778, (F)-0.002124, (F)-0.011870, (F)-0.091172, (F)0.010189,
(F)0.033804, (F)-0.007639, (F)-0.025344, (F)0.002832, (F)0.018060, (F)0.059920,
(F)-0.006696, (F)0.005020,
(F)-0.166443, (F)0.460693, (F)0.078430, (F)0.057251, (F)0.208740, (F)-0.027703,
(F)-0.212238, (F)-0.006151, (F)-0.003278, (F)-0.043572, (F)0.076679, (F)0.013054,
(F)-0.036132, (F)0.009529, (F)-0.026375, (F)-0.004490, (F)0.034743, (F)-0.096165,
(F)-0.016372, (F)-0.011951,
(F)0.224426, (F)0.053833, (F)0.247192, (F)0.073364, (F)0.141785, (F)-0.050367,
(F)-0.002898, (F)-0.061104, (F)-0.005382, (F)-0.020103, (F)-0.012082, (F)-0.055476,
(F)-0.013307, (F)-0.016465, (F)-0.003949, (F)-0.018135, (F)-0.031820, (F)-0.007633,
(F)-0.035048, (F)-0.010402,
(F)0.124878, (F)-0.045959, (F)0.007446, (F)0.616577, (F)0.036804, (F)-0.594,
(F)-0.002112, (F)-0.000055, (F)-0.380167, (F)-0.001355, (F)0.005739, (F)-0.000930,
(F)0.000342, (F)-0.076997, (F)0.028338, (F)-0.004591, (F)-0.004596, (F)0.001692,
(F)-0.000274, (F)-0.022693,
(F)0.027405, (F)-0.110779, (F)0.652466, (F)0.236206, (F)-0.032166, (F)-0.000751,
(F)-0.012272, (F)-0.425712, (F)-0.055793, (F)-0.001035, (F)0.003036, (F)-0.017881,
(F)0.072279, (F)-0.006473, (F)0.026167, (F)-0.154116, (F)0.000881, (F)-0.003563,
(F)0.020987, (F)0.007598,
(F)-0.057434, (F)0.143555, (F)0.308167, (F)0.222778, (F)0.160950, (F)-0.003299,
(F)-0.020608, (F)-0.094967, (F)-0.049630, (F)-0.025905, (F)0.008245, (F)0.017699,
(F)-0.044239, (F)0.012795, (F)-0.031981, (F)-0.068653, (F)0.009244, (F)-0.023105,
(F)-0.049599, (F)-0.035856,
(F)-0.114685, (F)0.048584, (F)0.190247, (F)0.584045, (F)-0.024658, (F)-0.013153,
(F)-0.002360, (F)-0.036194, (F)-0.341109, (F)-0.000608, (F)0.005572, (F)0.021818,
(F)-0.009243, (F)0.066981, (F)-0.028375, (F)-0.111113, (F)-0.002828, (F)0.001198,
(F)0.004691, (F)0.014402,
(F)-0.104675, (F)0.124695, (F)-0.127930, (F)0.607056, (F)0.244202, (F)-0.010957,
(F)-0.549, (F)-0.016366, (F)-0.368517, (F)-0.059634, (F)0.013052, (F)-0.013391,
(F)0.952, (F)0.063544, (F)-0.075697, (F)0.077660, (F)0.025562, (F)-0.030451,
(F)0.031241, (F)-0.148244,
(F)-0.352295, (F)-0.037720, (F)-0.239319, (F)0.017334, (F)-0.192566, (F)-0.124112,
(F)-0.001423, (F)-0.057274, (F)-0.000300, (F)-0.037082, (F)-0.013288, (F)-0.084311,
(F)-0.009027, (F)0.006107, (F)0.000654, (F)0.004148, (F)-0.067840, (F)-0.007264,
(F)-0.046085, (F)0.003338,
(F)0.023315, (F)0.289795, (F)0.488464, (F)-0.125977, (F)0.134155, (F)-0.000544,
(F)-0.083981, (F)-0.238597, (F)-0.870, (F)-0.017998, (F)-0.006757, (F)-0.011389,
(F)-0.141554, (F)0.002937, (F)0.036507, (F)0.061535, (F)-0.003128, (F)-0.038878,
(F)-0.065530, (F)0.016900,
(F)-0.052917, (F)0.184021, (F)0.727905, (F)-0.102234, (F)0.057800, (F)-0.002800,
(F)-0.033864, (F)-0.529846, (F)-0.010452, (F)-0.003341, (F)0.009738, (F)0.038519,
(F)-0.133950, (F)-0.005410, (F)0.018813, (F)0.074417, (F)0.003059, (F)-0.010636,
(F)-0.042073, (F)0.005909,
(F)-0.301086, (F)-0.026367, (F)0.412415, (F)0.230164, (F)0.082520, (F)-0.090653,
(F)-0.000695, (F)-0.170086, (F)-0.052975, (F)-0.006809, (F)-0.007939, (F)0.124172,
(F)0.010874, (F)0.069299, (F)0.006069, (F)-0.094923, (F)0.024846, (F)0.002176,
(F)-0.034032, (F)-0.018993,
(F)0.009277, (F)0.593628, (F)0.258240, (F)-0.117676, (F)0.090942, (F)-0.000086,
(F)-0.352394, (F)-0.066688, (F)-0.013848, (F)-0.008271, (F)-0.005507, (F)-0.002396,
(F)-0.153298, (F)0.001092, (F)0.069856, (F)0.030389, (F)-0.000844, (F)-0.053986,
(F)-0.023485, (F)0.010702,
(F)0.153320, (F)0.011780, (F)0.551331, (F)0.327209, (F)-0.192139, (F)-0.023507,
(F)-0.000139, (F)-0.303965, (F)-0.107066, (F)-0.036917, (F)-0.001806, (F)-0.084530,
(F)-0.006495, (F)-0.050168, (F)-0.003854, (F)-0.180401, (F)0.029459, (F)0.002263,
(F)0.105932, (F)0.062870,
(F)0.516602, (F)0.173584, (F)0.000732, (F)0.033508, (F)0.130127, (F)-0.266877,
(F)-0.030131, (F)-0.000001, (F)-0.001123, (F)-0.016933, (F)-0.089674, (F)-0.000378,
(F)-0.000127, (F)-0.017310, (F)-0.005817, (F)-0.000025, (F)-0.067224, (F)-0.022588,
(F)-0.000095, (F)-0.004360,
(F)0.130676, (F)-0.158813, (F)0.437378, (F)0.552124, (F)-0.082397, (F)-0.017076,
(F)-0.025222, (F)-0.191299, (F)-0.304841, (F)-0.006789, (F)0.020753, (F)-0.057155,
(F)0.069462, (F)-0.072150, (F)0.087685, (F)-0.241487, (F)0.010767, (F)-0.013086,
(F)0.036039, (F)0.045494,
(F)-0.014893, (F)0.016968, (F)0.812195, (F)0.074890, (F)-0.009827, (F)-0.000222,
(F)-0.000288, (F)-0.659660, (F)-0.005609, (F)-0.000097, (F)0.000253, (F)0.012096,
(F)-0.013781, (F)0.001115, (F)-0.001271, (F)-0.060825, (F)-0.000146, (F)0.000167,
(F)0.007981, (F)0.000736,
(F)-0.118835, (F)0.526, (F)0.382629, (F)0.055969, (F)-0.156189, (F)-0.014122,
(F)-0.251528, (F)-0.146405, (F)-0.003133, (F)-0.024395, (F)0.059599, (F)0.045470,
(F)-0.191899, (F)0.006651, (F)-0.028070, (F)-0.021415, (F)-0.018561, (F)0.078333,
(F)0.059762, (F)0.008742,
(F)-0.110474, (F)0.125793, (F)-0.169739, (F)0.369629, (F)0.613892, (F)-0.012204,
(F)-0.824, (F)-0.028811, (F)-0.136626, (F)-0.376863, (F)0.013897, (F)-0.018752,
(F)0.021352, (F)0.040834, (F)-0.046497, (F)0.062740, (F)0.067819, (F)-0.077224,
(F)0.104201, (F)-0.226912,
(F)0.477722, (F)0.559143, (F)-0.208069, (F)0.150940, (F)-0.092163, (F)-0.228218,
(F)-0.312641, (F)-0.043293, (F)-0.022783, (F)-0.008494, (F)-0.267115, (F)0.099399,
(F)0.116340, (F)-0.072107, (F)-0.084397, (F)0.031406, (F)0.044028, (F)0.051532,
(F)-0.019176, (F)0.013911,
(F)-0.087280, (F)0.507385, (F)0.582336, (F)-0.180359, (F)0.099243, (F)-0.007618,
(F)-0.257440, (F)-0.339116, (F)-0.032529, (F)-0.009849, (F)0.044285, (F)0.050826,
(F)-0.295469, (F)-0.742, (F)0.091511, (F)0.105030, (F)0.008662, (F)-0.050355,
(F)-0.057793, (F)0.017899,
(F)0.172058, (F)-0.259216, (F)0.329041, (F)0.529358, (F)0.168213, (F)-0.029604,
(F)-0.067193, (F)-0.108268, (F)-0.280220, (F)-0.028296, (F)0.044600, (F)-0.056614,
(F)0.085293, (F)-0.091080, (F)0.137218, (F)-0.174180, (F)-0.028942, (F)0.043604,
(F)-0.055349, (F)-0.089045,
(F)0.008606, (F)0.888184, (F)0.042114, (F)0.686, (F)-0.006836, (F)-0.000074,
(F)-0.788870, (F)-0.001774, (F)-0.000246, (F)-0.000047, (F)-0.007644, (F)-0.000362,
(F)-0.037405, (F)-0.000135, (F)-0.013932, (F)-0.000661, (F)0.000059, (F)0.006072,
(F)0.000288, (F)0.000107,
(F)0.728638, (F)-0.113464, (F)0.110229, (F)-0.058716, (F)-0.053955, (F)-0.530913,
(F)-0.012874, (F)-0.012151, (F)-0.003448, (F)-0.002911, (F)0.082674, (F)-0.080317,
(F)0.012507, (F)0.042783, (F)-0.006662, (F)0.006472, (F)0.039314, (F)-0.006122,
(F)0.005947, (F)-0.003168,
(F)0.227539, (F)0.120972, (F)0.138184, (F)0.279785, (F)0.191101, (F)-0.051774,
(F)-0.014634, (F)-0.019095, (F)-0.078280, (F)-0.036520, (F)-0.027526, (F)-0.031442,
(F)-0.016716, (F)-0.063662, (F)-0.033846, (F)-0.038662, (F)-0.043483, (F)-0.023118,
(F)-0.026407, (F)-0.053467,
(F)-0.072083, (F)0.046753, (F)0.043945, (F)0.872986, (F)-0.013184, (F)-0.005196,
(F)-0.002186, (F)-0.001931, (F)-0.762104, (F)-0.000174, (F)0.003370, (F)0.003168,
(F)-0.002055, (F)0.062927, (F)-0.040815, (F)-0.038364, (F)-0.000950, (F)0.000616,
(F)0.000579, (F)0.011509,
(F)-0.267639, (F)0.294556, (F)0.617188, (F)0.095764, (F)0.206787, (F)-0.071631,
(F)-0.086763, (F)-0.380920, (F)-0.009171, (F)-0.042761, (F)0.078835, (F)0.165184,
(F)-0.181796, (F)0.025630, (F)-0.028208, (F)-0.059104, (F)0.055344, (F)-0.060910,
(F)-0.127626, (F)-0.019803,
(F)-0.027344, (F)0.117920, (F)0.951599, (F)-0.112427, (F)0.040100, (F)-0.000748,
(F)-0.013905, (F)-0.905541, (F)-0.012640, (F)-0.001608, (F)0.003224, (F)0.026020,
(F)-0.112212, (F)-0.003074, (F)0.013257, (F)0.106985, (F)0.001096, (F)-0.004729,
(F)-0.038159, (F)0.004508,
(F)-0.252747, (F)0.112549, (F)0.241821, (F)0.336731, (F)0.379639, (F)-0.063881,
(F)-0.012667, (F)-0.058478, (F)-0.113388, (F)-0.144126, (F)0.028446, (F)0.061120,
(F)-0.027217, (F)0.085108, (F)-0.037899, (F)-0.081429, (F)0.095952, (F)-0.042728,
(F)-0.091805, (F)-0.127836,
(F)0.044495, (F)-0.140320, (F)0.900574, (F)0.223206, (F)-0.058105, (F)-0.001980,
(F)-0.019690, (F)-0.811033, (F)-0.049821, (F)-0.003376, (F)0.006243, (F)-0.040071,
(F)0.126368, (F)-0.009931, (F)0.031320, (F)-0.201013, (F)0.002585, (F)-0.008153,
(F)0.052328, (F)0.012969,
(F)-0.084534, (F)0.354065, (F)0.814148, (F)-0.191467, (F)0.082031, (F)-0.007146,
(F)-0.125362, (F)-0.662837, (F)-0.036660, (F)-0.006729, (F)0.029930, (F)0.068823,
(F)-0.288261, (F)-0.016185, (F)0.067792, (F)0.155883, (F)0.006934, (F)-0.029044,
(F)-0.066786, (F)0.706,
(F)0.287476, (F)0.589661, (F)0.065491, (F)-0.150574, (F)0.199829, (F)-0.082642,
(F)-0.347700, (F)-0.004289, (F)-0.022672, (F)-0.039932, (F)-0.169513, (F)-0.018827,
(F)-0.038617, (F)0.043286, (F)0.088787, (F)0.009861, (F)-0.057446, (F)-0.117831,
(F)-0.013087, (F)0.030089,
(F)0.112488, (F)-0.221191, (F)0.758179, (F)0.421021, (F)-0.095154, (F)-0.012654,
(F)-0.048926, (F)-0.574835, (F)-0.177258, (F)-0.009054, (F)0.024881, (F)-0.085286,
(F)0.167703, (F)-0.047360, (F)0.093126, (F)-0.319209, (F)0.010704, (F)-0.021047,
(F)0.072144, (F)0.040062,
(F)0.087463, (F)0.146118, (F)0.153, (F)0.084229, (F)0.535889, (F)-0.007650,
(F)-0.021351, (F)-0.023414, (F)-0.007094, (F)-0.287177, (F)-0.012780, (F)-0.013383,
(F)-0.022358, (F)-0.007367, (F)-0.012307, (F)-0.012888, (F)-0.046871, (F)-0.078303,
(F)-0.081999, (F)-0.045137,
(F)-0.104492, (F)0.788269, (F)0.359070, (F)-0.134094, (F)0.060486, (F)-0.010919,
(F)-0.621368, (F)-0.128931, (F)-0.017981, (F)-0.003659, (F)0.082368, (F)0.037520,
(F)-0.283044, (F)-0.014012, (F)0.105702, (F)0.048149, (F)0.006320, (F)-0.047679,
(F)-0.021719, (F)0.008111,
(F)0.919922, (F)0.131714, (F)-0.075989, (F)0.055542, (F)-0.030273, (F)-0.846256,
(F)-0.017349, (F)-0.005774, (F)-0.003085, (F)-0.000916, (F)-0.121166, (F)0.069904,
(F)0.010009, (F)-0.051094, (F)-0.007316, (F)0.004221, (F)0.027849, (F)0.003987,
(F)-0.002300, (F)0.001681,
(F)0.284119, (F)0.446716, (F)0.300049, (F)0.035767, (F)-0.206360, (F)-0.080723,
(F)-0.199555, (F)-0.090029, (F)-0.001279, (F)-0.042584, (F)-0.126920, (F)-0.085249,
(F)-0.134037, (F)-0.010162, (F)-0.978, (F)-0.010732, (F)0.058631, (F)0.092184,
(F)0.061918, (F)0.007381,
(F)0.072693, (F)-0.130615, (F)0.312500, (F)0.823853, (F)-0.086548, (F)-0.005284,
(F)-0.017060, (F)-0.097656, (F)-0.678733, (F)-0.007491, (F)0.009495, (F)-0.022717,
(F)0.040817, (F)-0.059888, (F)0.107608, (F)-0.257454, (F)0.006291, (F)-0.011304,
(F)0.027046, (F)0.071303,
(F)-0.093384, (F)-0.049866, (F)0.499939, (F)0.587891, (F)-0.088623, (F)-0.008721,
(F)-0.002487, (F)-0.249939, (F)-0.345615, (F)-0.007854, (F)-0.004657, (F)0.046686,
(F)0.024930, (F)0.054899, (F)0.029316, (F)-0.293909, (F)-0.008276, (F)-0.004419,
(F)0.044306, (F)0.052101,
(F)-0.197510, (F)0.037048, (F)0.103516, (F)-0.128540, (F)0.700989, (F)-0.039010,
(F)-0.001373, (F)-0.010715, (F)-0.016523, (F)-0.491385, (F)0.007317, (F)0.020445,
(F)-0.003835, (F)-0.025388, (F)0.004762, (F)0.013306, (F)0.138452, (F)-0.025970,
(F)-0.072563, (F)0.090105,
(F)0.195496, (F)-0.116150, (F)-0.051086, (F)0.590759, (F)0.430725, (F)-0.038219,
(F)-0.013491, (F)-0.002610, (F)-0.348997, (F)-0.185524, (F)0.022707, (F)0.009987,
(F)-0.005934, (F)-0.115491, (F)0.068617, (F)0.030180, (F)-0.084205, (F)0.050029,
(F)0.022004, (F)-0.254455,
(F)0.013000, (F)-0.039978, (F)1.038513, (F)0.037842, (F)-0.023438, (F)-0.000169,
(F)-0.098, (F)-1.078510, (F)-0.001432, (F)-0.000549, (F)0.000520, (F)-0.013501,
(F)0.041518, (F)-0.000492, (F)0.013, (F)-0.039299, (F)0.000305, (F)-0.000937,
(F)0.024340, (F)0.000887,
(F)0.310547, (F)0.002136, (F)-0.203796, (F)0.227417, (F)0.518066, (F)-0.096439,
(F)-0.000005, (F)-0.041533, (F)-0.051718, (F)-0.268393, (F)-0.000663, (F)0.063288,
(F)0.000435, (F)-0.070624, (F)-0.000486, (F)0.046347, (F)-0.160884, (F)-0.001107,
(F)0.105580, (F)-0.117817,
(F)-0.060303, (F)0.088745, (F)-0.164062, (F)0.917908, (F)0.163696, (F)-0.003636,
(F)-0.007876, (F)-0.026917, (F)-0.842555, (F)-0.026796, (F)0.005352, (F)-0.009893,
(F)0.014560, (F)0.055352, (F)-0.081460, (F)0.150594, (F)0.009871, (F)-0.014527,
(F)0.026856, (F)-0.150258,
(F)-0.296936, (F)-0.173462, (F)-0.003235, (F)0.335388, (F)0.398132, (F)-0.088171,
(F)-0.030089, (F)-0.000010, (F)-0.112485, (F)-0.158509, (F)-0.051507, (F)-0.000961,
(F)-0.000561, (F)0.099589, (F)0.058177, (F)0.001085, (F)0.118220, (F)0.069061,
(F)0.001288, (F)-0.133529,
(F)0.187744, (F)0.919739, (F)-0.177856, (F)0.110474, (F)-0.059265, (F)-0.035248,
(F)-0.845919, (F)-0.031633, (F)-0.012204, (F)-0.003512, (F)-0.172676, (F)0.033392,
(F)0.163581, (F)-0.020741, (F)-0.101607, (F)0.019648, (F)0.011127, (F)0.054508,
(F)-0.010541, (F)0.006547,
(F)0.080017, (F)0.326843, (F)0.674683, (F)0.140320, (F)-0.223022, (F)-0.006403,
(F)-0.106827, (F)-0.455197, (F)-0.019690, (F)-0.049739, (F)-0.026153, (F)-0.053986,
(F)-0.220515, (F)-0.011228, (F)-0.045863, (F)-0.094671, (F)0.017846, (F)0.072893,
(F)0.150469, (F)0.031294,
(F)0.246277, (F)0.342957, (F)0.281860, (F)0.110779, (F)0.116699, (F)-0.060652,
(F)-0.117619, (F)-0.079445, (F)-0.012272, (F)-0.013619, (F)-0.084462, (F)-0.069416,
(F)-0.096666, (F)-0.027282, (F)-0.037992, (F)-0.031224, (F)-0.028740, (F)-0.040023,
(F)-0.032893, (F)-0.012928,
(F)-0.130066, (F)0.168091, (F)-0.274719, (F)0.786072, (F)0.453613, (F)-0.016917,
(F)-0.028255, (F)-0.075471, (F)-0.617909, (F)-0.205765, (F)0.021863, (F)-0.035732,
(F)0.046178, (F)0.102241, (F)-0.132131, (F)0.215949, (F)0.059000, (F)-0.076248,
(F)0.124616, (F)-0.356573,
(F)0.342896, (F)-0.031433, (F)0.196472, (F)-0.253235, (F)0.296448, (F)-0.117577,
(F)-0.000988, (F)-0.038601, (F)-0.064128, (F)-0.087881, (F)0.010778, (F)-0.067369,
(F)0.006176, (F)0.086833, (F)-0.007960, (F)0.049754, (F)-0.101651, (F)0.009318,
(F)-0.058244, (F)0.075071,
(F)0.112183, (F)-0.104309, (F)0.069519, (F)1.029480, (F)0.043701, (F)-0.012585,
(F)-0.010880, (F)-0.004833, (F)-1.059829, (F)-0.001910, (F)0.011702, (F)-0.007799,
(F)0.007251, (F)-0.115490, (F)0.107384, (F)-0.071568, (F)-0.004903, (F)0.004558,
(F)-0.003038, (F)-0.044989,
(F)0.120728, (F)-0.234680, (F)0.613403, (F)0.723511, (F)-0.152100, (F)-0.014575,
(F)-0.055075, (F)-0.376264, (F)-0.523468, (F)-0.023134, (F)0.028332, (F)-0.074055,
(F)0.143954, (F)-0.087348, (F)0.169794, (F)-0.443804, (F)0.018363, (F)-0.035695,
(F)0.093298, (F)0.110046,
(F)-0.158691, (F)0.686401, (F)0.689026, (F)-0.222595, (F)0.089294, (F)-0.025183,
(F)-0.471147, (F)-0.474757, (F)-0.049549, (F)-0.007973, (F)0.108926, (F)0.109342,
(F)-0.472948, (F)-0.035324, (F)0.152790, (F)0.153374, (F)0.014170, (F)-0.061292,
(F)-0.061526, (F)0.019877,
(F)0.210083, (F)0.055359, (F)0.166321, (F)0.547485, (F)0.222046, (F)-0.044135,
(F)-0.003065, (F)-0.027663, (F)-0.299740, (F)-0.049304, (F)-0.011630, (F)-0.034941,
(F)-0.009207, (F)-0.115017, (F)-0.030308, (F)-0.091058, (F)-0.046648, (F)-0.012292,
(F)-0.036931, (F)-0.121567,
(F)-0.007690, (F)1.067993, (F)0.212463, (F)-0.122253, (F)0.066528, (F)-0.000059,
(F)-1.140609, (F)-0.045141, (F)-0.014946, (F)-0.004426, (F)0.008213, (F)0.001634,
(F)-0.226909, (F)-0.000940, (F)0.130566, (F)0.025974, (F)0.000512, (F)-0.071052,
(F)-0.014135, (F)0.008133,
(F)-0.110718, (F)0.063965, (F)-0.132080, (F)0.248413, (F)0.993164, (F)-0.012258,
(F)-0.004092, (F)-0.017445, (F)-0.061709, (F)-0.986375, (F)0.007082, (F)-0.014624,
(F)0.008448, (F)0.027504, (F)-0.890, (F)0.032810, (F)0.109961, (F)-0.063528,
(F)0.131177, (F)-0.246715,
(F)0.570923, (F)0.767822, (F)-0.114929, (F)0.058716, (F)-0.037659, (F)-0.325953,
(F)-0.589551, (F)-0.013209, (F)-0.003448, (F)-0.001418, (F)-0.438367, (F)0.065616,
(F)0.088245, (F)-0.033522, (F)-0.045083, (F)0.006748, (F)0.021500, (F)0.028915,
(F)-0.004328, (F)0.002211,
(F)0.104248, (F)0.256287, (F)0.450867, (F)0.386658, (F)0.061218, (F)-0.010868,
(F)-0.065683, (F)-0.203281, (F)-0.149504, (F)-0.003748, (F)-0.026717, (F)-0.047002,
(F)-0.115551, (F)-0.040308, (F)-0.099095, (F)-0.174331, (F)-0.006382, (F)-0.689,
(F)-0.027601, (F)-0.023671,
(F)-0.166, (F)0.343201, (F)1.056519, (F)-0.143616, (F)0.036133, (F)-0.025650,
(F)-0.117787, (F)-1.116231, (F)-0.020625, (F)-0.001306, (F)0.054966, (F)0.169208,
(F)-0.362598, (F)-0.023001, (F)0.049289, (F)0.151733, (F)0.005787, (F)-0.012401,
(F)-0.038175, (F)0.005189,
(F)0.073914, (F)-0.051880, (F)0.072815, (F)-0.117554, (F)0.976074, (F)-0.005463,
(F)-0.002692, (F)-0.005302, (F)-0.013819, (F)-0.952721, (F)0.003835, (F)-0.005382,
(F)0.003778, (F)0.008689, (F)-0.006099, (F)0.008560, (F)-0.072145, (F)0.050639,
(F)-0.071073, (F)0.114741,
(F)0.243103, (F)-0.125366, (F)0.702271, (F)-0.001160, (F)0.221069, (F)-0.059099,
(F)-0.717, (F)-0.493184, (F)-0.000001, (F)-0.048872, (F)0.030477, (F)-0.170724,
(F)0.088041, (F)0.000282, (F)-0.000145, (F)0.000814, (F)-0.053743, (F)0.027715,
(F)-0.155250, (F)0.000256,
(F)-0.047974, (F)-0.082642, (F)0.982178, (F)0.442261, (F)-0.101624, (F)-0.002301,
(F)-0.006830, (F)-0.964673, (F)-0.195595, (F)-0.010327, (F)-0.003965, (F)0.047119,
(F)0.081169, (F)0.021217, (F)0.036549, (F)-0.434379, (F)-0.004875, (F)-0.008398,
(F)0.099812, (F)0.044944,
(F)-0.263428, (F)-0.031189, (F)0.407349, (F)0.711975, (F)0.282776, (F)-0.069394,
(F)-0.000973, (F)-0.165933, (F)-0.506909, (F)-0.079962, (F)-0.008216, (F)0.107307,
(F)0.012705, (F)0.187554, (F)0.022206, (F)-0.290022, (F)0.074491, (F)0.008819,
(F)-0.115188, (F)-0.201329,
(F)0.482910, (F)0.278137, (F)0.118225, (F)0.115051, (F)0.429504, (F)-0.233202,
(F)-0.077360, (F)-0.013977, (F)-0.013237, (F)-0.184474, (F)-0.134315, (F)-0.057092,
(F)-0.032883, (F)-0.055559, (F)-0.032000, (F)-0.013602, (F)-0.207412, (F)-0.119461,
(F)-0.050778, (F)-0.049415,
(F)0.209595, (F)0.181091, (F)0.358093, (F)0.500244, (F)0.535034, (F)-0.043930,
(F)-0.032794, (F)-0.128231, (F)-0.250244, (F)-0.286262, (F)-0.037956, (F)-0.075054,
(F)-0.064848, (F)-0.104849, (F)-0.090590, (F)-0.179134, (F)-0.112140, (F)-0.096890,
(F)-0.191592, (F)-0.267648,
(F)0.358948, (F)0.560059, (F)0.495544, (F)0.240112, (F)0.204773, (F)-0.128843,
(F)-0.313666, (F)-0.245564, (F)-0.057654, (F)-0.041932, (F)-0.201032, (F)-0.177875,
(F)-0.277534, (F)-0.086188, (F)-0.134477, (F)-0.118986, (F)-0.073503, (F)-0.114685,
(F)-0.101474, (F)-0.049169 };


/*
**  AcbkGainTable170:
**
**  Purpose:
**      Used to calculate the error expression in pitch prediction
**      optimization (also expressed as an adaptive codebook approach)
**
**  Table Structure:
**      The table is structured as 170 20-element vectors.  These
**      vectors are structured as precalculated values in the error
**      expression for the pitch
**      predictor.  Gi is the gain value multiplying the signal
**      delayed by a pitch period (+/- offset).  In equation 41.2,
**      Gi would be equivalent to bij.
**
**      1st 5 elements:   G1  G2  G3  G4  G5 in Q14
**      2nd 5 elements:  -G1^2  -G2^2  -G3^2  -G4^2  -G5^2
**      Next 10 elements: These are the off-diagonal elements
**           -G1*G2  -G1*G3  -G2*G3  -G1*G4  -G2*G4
**           -G3*G4  -G1*G5  -G2*G5  -G3*G5  -G4*G5
*/

 FLOAT   AcbkGainTable170[170*20] = {
 (F)0.000000, (F)0.000000, (F)0.000000, (F)0.000000, (F)0.000000, (F)0.000000,
(F)0.000000, (F)0.000000, (F)0.000000, (F)0.000000, (F)0.000000, (F)0.000000,
(F)0.000000, (F)0.000000, (F)0.000000, (F)0.000000, (F)0.000000, (F)0.000000,
(F)0.000000, (F)0.000000,
(F)0.047363, (F)0.012939, (F)0.043640, (F)0.040894, (F)0.049377, (F)-0.002243,
(F)-0.000167, (F)-0.001904, (F)-0.001672, (F)-0.002438, (F)-0.000613, (F)-0.002067,
(F)-0.000565, (F)-0.001937, (F)-0.000529, (F)-0.001785, (F)-0.002339, (F)-0.000639,
(F)-0.002155, (F)-0.002019,
(F)0.079102, (F)0.080322, (F)-0.010254, (F)-0.019531, (F)-0.049744, (F)-0.006257,
(F)-0.006452, (F)-0.000105, (F)-0.000381, (F)-0.002474, (F)-0.006354, (F)0.000811,
(F)0.000824, (F)0.045, (F)0.001569, (F)-0.000200, (F)0.003935, (F)0.003996,
(F)-0.000510, (F)-0.000972,
(F)-0.035950, (F)0.041504, (F)0.151245, (F)0.018799, (F)-0.036377, (F)-0.001292,
(F)-0.001723, (F)-0.022875, (F)-0.000353, (F)-0.001323, (F)0.001492, (F)0.005437,
(F)-0.006277, (F)0.000676, (F)-0.000780, (F)-0.002843, (F)-0.001308, (F)0.010,
(F)0.005502, (F)0.000684,
(F)-0.044861, (F)-0.029724, (F)-0.000305, (F)0.179932, (F)0.028564, (F)-0.002012,
(F)-0.000884, (F)0.000000, (F)-0.032375, (F)-0.000816, (F)-0.001333, (F)-0.000014,
(F)-0.000009, (F)0.008072, (F)0.005348, (F)0.000055, (F)0.001281, (F)0.000849,
(F)0.000009, (F)-0.005140,
(F)0.063599, (F)0.105591, (F)0.065186, (F)0.020325, (F)0.038208, (F)-0.004045,
(F)-0.011149, (F)-0.004249, (F)-0.000413, (F)-0.001460, (F)-0.006715, (F)-0.004146,
(F)-0.006883, (F)-0.001293, (F)-0.002146, (F)-0.001325, (F)-0.002430, (F)-0.004034,
(F)-0.002491, (F)-0.000777,
(F)0.029663, (F)-0.046936, (F)0.248657, (F)0.172424, (F)-0.067566, (F)-0.000880,
(F)-0.002203, (F)-0.061830, (F)-0.029730, (F)-0.004565, (F)0.001392, (F)-0.007376,
(F)0.011671, (F)-0.005115, (F)0.008093, (F)-0.042875, (F)0.002004, (F)-0.003171,
(F)0.016801, (F)0.011650,
(F)0.092834, (F)-0.046814, (F)-0.007568, (F)0.263672, (F)0.062622, (F)-0.008618,
(F)-0.002192, (F)-0.000057, (F)-0.069523, (F)-0.003922, (F)0.004346, (F)0.000703,
(F)-0.000354, (F)-0.024478, (F)0.012344, (F)0.001996, (F)-0.005813, (F)0.002932,
(F)0.000474, (F)-0.016512,
(F)0.148010, (F)0.077332, (F)0.209900, (F)-0.005554, (F)-0.071167, (F)-0.021907,
(F)-0.005980, (F)-0.044058, (F)-0.000031, (F)-0.005065, (F)-0.011446, (F)-0.031067,
(F)-0.016232, (F)0.000822, (F)0.000430, (F)0.001166, (F)0.010533, (F)0.005503,
(F)0.014938, (F)-0.000395,
(F)-0.076355, (F)0.059509, (F)0.010559, (F)0.246521, (F)0.122375, (F)-0.005830,
(F)-0.003541, (F)-0.000111, (F)-0.060773, (F)-0.014976, (F)0.004544, (F)0.000806,
(F)-0.000628, (F)0.018823, (F)-0.014670, (F)-0.002603, (F)0.009344, (F)-0.007282,
(F)-0.001292, (F)-0.030168,
(F)0.111084, (F)0.038574, (F)0.080688, (F)0.125854, (F)0.062927, (F)-0.012340,
(F)-0.001488, (F)-0.006511, (F)-0.839, (F)-0.003960, (F)-0.004285, (F)-0.008963,
(F)-0.003112, (F)-0.013980, (F)-0.004855, (F)-0.015, (F)-0.006990, (F)-0.002427,
(F)-0.005078, (F)-0.007920,
(F)-0.027283, (F)0.299316, (F)0.014893, (F)-0.019226, (F)-0.124390, (F)-0.000744,
(F)-0.089590, (F)-0.000222, (F)-0.000370, (F)-0.473, (F)0.008166, (F)0.000406,
(F)-0.004458, (F)-0.000525, (F)0.005755, (F)0.000286, (F)-0.003394, (F)0.037232,
(F)0.001852, (F)-0.002392,
(F)-0.012695, (F)-0.067261, (F)0.028259, (F)-0.027344, (F)0.345032, (F)-0.000161,
(F)-0.004524, (F)-0.000799, (F)-0.000748, (F)-0.119047, (F)-0.000854, (F)0.000359,
(F)0.001901, (F)-0.000347, (F)-0.001839, (F)0.000773, (F)0.004380, (F)0.023207,
(F)-0.009750, (F)0.009434,
(F)0.289246, (F)0.158691, (F)-0.113770, (F)0.052246, (F)-0.094849, (F)-0.083663,
(F)-0.025183, (F)-0.012944, (F)-0.002730, (F)-0.008996, (F)-0.045901, (F)0.032907,
(F)0.018054, (F)-0.112, (F)-0.008291, (F)0.005944, (F)0.027435, (F)0.015052,
(F)-0.010791, (F)0.004955,
(F)0.115601, (F)0.215637, (F)0.002136, (F)-0.087, (F)0.130920, (F)-0.013363,
(F)-0.046499, (F)-0.000005, (F)-0.000003, (F)-0.017140, (F)-0.024928, (F)-0.000247,
(F)-0.000461, (F)0.000183, (F)0.000342, (F)0.000003, (F)-0.134, (F)-0.028231,
(F)-0.000280, (F)0.000208,
(F)-0.164734, (F)0.112366, (F)0.248535, (F)0.149109, (F)0.096558, (F)-0.027137,
(F)-0.012626, (F)-0.061770, (F)-0.022233, (F)-0.009323, (F)0.018510, (F)0.040942,
(F)-0.027927, (F)0.024563, (F)-0.016755, (F)-0.037059, (F)0.906, (F)-0.010850,
(F)-0.023998, (F)-0.014398,
(F)-0.051514, (F)0.204956, (F)0.372681, (F)-0.064636, (F)-0.032776, (F)-0.002654,
(F)-0.042007, (F)-0.138891, (F)-0.004178, (F)-0.001074, (F)0.010558, (F)0.019198,
(F)-0.076383, (F)-0.003330, (F)0.013248, (F)0.024089, (F)-0.001688, (F)0.006718,
(F)0.012215, (F)-0.002119,
(F)0.076355, (F)0.062012, (F)0.184326, (F)0.134888, (F)0.088196, (F)-0.005830,
(F)-0.003845, (F)-0.033976, (F)-0.018195, (F)-0.007778, (F)-0.004735, (F)-0.014074,
(F)-0.011430, (F)-0.010299, (F)-0.008365, (F)-0.024863, (F)-0.006734, (F)-0.005469,
(F)-0.016257, (F)-0.011897,
(F)0.128113, (F)0.138977, (F)0.246460, (F)0.215637, (F)-0.175171, (F)-0.016413,
(F)-0.019315, (F)-0.060743, (F)-0.046499, (F)-0.030685, (F)-0.017805, (F)-0.031575,
(F)-0.034252, (F)-0.027626, (F)-0.029969, (F)-0.053146, (F)0.022442, (F)0.024345,
(F)0.043173, (F)0.037773,
(F)0.024414, (F)-0.067993, (F)0.519775, (F)0.147827, (F)-0.068176, (F)-0.000596,
(F)-0.004623, (F)-0.270166, (F)-0.021853, (F)-0.004648, (F)0.001660, (F)-0.012690,
(F)0.035341, (F)-0.003609, (F)0.010051, (F)-0.076837, (F)0.001664, (F)-0.004636,
(F)0.035436, (F)0.010078,
(F)-0.268433, (F)-0.122742, (F)0.238770, (F)-0.146912, (F)0.137817, (F)-0.072056,
(F)-0.066, (F)-0.057011, (F)-0.021583, (F)-0.018994, (F)-0.032948, (F)0.064094,
(F)0.029307, (F)-0.039436, (F)-0.018032, (F)0.035078, (F)0.036995, (F)0.016916,
(F)-0.032907, (F)0.020247,
(F)0.107849, (F)-0.088318, (F)0.258789, (F)0.375977, (F)-0.046204, (F)-0.011631,
(F)-0.007800, (F)-0.066972, (F)-0.141358, (F)-0.002135, (F)0.009525, (F)-0.027910,
(F)0.022856, (F)-0.040549, (F)0.033205, (F)-0.097299, (F)0.004983, (F)-0.004081,
(F)0.011957, (F)0.017371,
(F)0.097290, (F)-0.089294, (F)0.063171, (F)0.423462, (F)0.117188, (F)-0.009465,
(F)-0.007973, (F)-0.003991, (F)-0.179320, (F)-0.013733, (F)0.008687, (F)-0.006146,
(F)0.005641, (F)-0.041199, (F)0.037813, (F)-0.026751, (F)-0.011401, (F)0.010464,
(F)-0.007403, (F)-0.049624,
(F)-0.034302, (F)0.287842, (F)0.249329, (F)-0.036072, (F)0.147766, (F)-0.001177,
(F)-0.082853, (F)-0.062165, (F)-0.001301, (F)-0.021835, (F)0.009873, (F)0.008552,
(F)-0.071767, (F)-0.001237, (F)0.010383, (F)0.008994, (F)0.005069, (F)-0.042533,
(F)-0.036842, (F)0.005330,
(F)0.094727, (F)0.535767, (F)-0.057068, (F)0.021606, (F)-0.086914, (F)-0.008973,
(F)-0.287046, (F)-0.003257, (F)-0.000467, (F)-0.007554, (F)-0.050751, (F)0.005406,
(F)0.030575, (F)-0.002047, (F)-0.011576, (F)0.001233, (F)0.008233, (F)0.046566,
(F)-0.004960, (F)0.001878,
(F)0.294861, (F)-0.270447, (F)0.002502, (F)-0.090271, (F)-0.181641, (F)-0.086943,
(F)-0.073141, (F)-0.000006, (F)-0.008149, (F)-0.032993, (F)0.079744, (F)-0.000738,
(F)0.000677, (F)0.026617, (F)-0.024413, (F)0.000226, (F)0.053559, (F)-0.049124,
(F)0.000455, (F)-0.016397,
(F)0.127563, (F)0.072754, (F)0.061401, (F)0.100403, (F)0.296204, (F)-0.016272,
(F)-0.005293, (F)-0.003770, (F)-0.010081, (F)-0.087737, (F)-0.009281, (F)-0.007833,
(F)-0.004467, (F)-0.012808, (F)-0.007305, (F)-0.006165, (F)-0.037785, (F)-0.021550,
(F)-0.018187, (F)-0.029740,
(F)0.145630, (F)0.348633, (F)0.087036, (F)-0.005737, (F)0.082397, (F)-0.021208,
(F)-0.121545, (F)-0.007575, (F)-0.000033, (F)-0.006789, (F)-0.050771, (F)-0.012675,
(F)-0.030344, (F)0.000836, (F)0.002000, (F)0.000499, (F)-0.012000, (F)-0.028726,
(F)-0.007172, (F)0.000473,
(F)-0.082336, (F)0.066589, (F)0.101257, (F)0.542664, (F)0.019104, (F)-0.006779,
(F)-0.004434, (F)-0.010253, (F)-0.294484, (F)-0.000365, (F)0.005483, (F)0.008337,
(F)-0.006743, (F)0.044681, (F)-0.036136, (F)-0.054949, (F)0.073, (F)-0.001272,
(F)-0.001934, (F)-0.010367,
(F)0.609131, (F)0.055908, (F)-0.023254, (F)-0.049316, (F)0.005371, (F)-0.371040,
(F)-0.003126, (F)-0.000541, (F)-0.002432, (F)-0.000029, (F)-0.034055, (F)0.014165,
(F)0.001300, (F)0.030040, (F)0.002757, (F)-0.001147, (F)-0.003272, (F)-0.000300,
(F)0.000125, (F)0.000265,
(F)0.142700, (F)-0.062927, (F)-0.137, (F)0.239746, (F)0.395752, (F)-0.020363,
(F)-0.003960, (F)-0.000229, (F)-0.057478, (F)-0.156620, (F)0.008980, (F)0.002160,
(F)-0.000953, (F)-0.034212, (F)0.087, (F)0.003629, (F)-0.056474, (F)0.024904,
(F)0.005990, (F)-0.094880,
(F)0.376343, (F)-0.075684, (F)-0.020569, (F)0.224121, (F)-0.077942, (F)-0.141634,
(F)-0.005728, (F)-0.000423, (F)-0.050230, (F)-0.006075, (F)0.028483, (F)0.007741,
(F)-0.057, (F)-0.084346, (F)0.016962, (F)0.004610, (F)0.029333, (F)-0.005899,
(F)-0.001603, (F)0.017468,
(F)0.267151, (F)-0.008057, (F)-0.157959, (F)0.103821, (F)0.296936, (F)-0.071370,
(F)-0.000065, (F)-0.024951, (F)-0.010779, (F)-0.088171, (F)0.002152, (F)0.042199,
(F)-0.001273, (F)-0.027736, (F)0.000836, (F)0.016399, (F)-0.079327, (F)0.002392,
(F)0.046904, (F)-0.030828,
(F)0.158325, (F)0.201111, (F)0.240723, (F)0.090393, (F)0.041626, (F)-0.025067,
(F)-0.040446, (F)-0.057947, (F)-0.008171, (F)-0.001733, (F)-0.031841, (F)-0.038112,
(F)-0.048412, (F)-0.014311, (F)-0.018179, (F)-0.021760, (F)-0.006590, (F)-0.008371,
(F)-0.010020, (F)-0.003763,
(F)0.253357, (F)0.038086, (F)0.049744, (F)0.273743, (F)0.136047, (F)-0.064190,
(F)-0.001451, (F)-0.002474, (F)-0.074935, (F)-0.018509, (F)-0.009649, (F)-0.012603,
(F)-0.001895, (F)-0.069355, (F)-0.010426, (F)-0.013617, (F)-0.034469, (F)-0.005181,
(F)-0.006767, (F)-0.037242,
(F)-0.218201, (F)-0.022034, (F)0.300537, (F)0.170349, (F)0.286743, (F)-0.047612,
(F)-0.000485, (F)-0.090323, (F)-0.029019, (F)-0.082222, (F)-0.004808, (F)0.065577,
(F)0.006622, (F)0.037170, (F)0.003753, (F)-0.051196, (F)0.062568, (F)0.006318,
(F)-0.086177, (F)-0.048846,
(F)-0.153687, (F)0.231384, (F)0.526306, (F)0.019226, (F)0.150452, (F)-0.023620,
(F)-0.053539, (F)-0.276998, (F)-0.000370, (F)-0.022636, (F)0.035561, (F)0.080886,
(F)-0.121779, (F)0.002955, (F)-0.004449, (F)-0.010119, (F)0.023122, (F)-0.034812,
(F)-0.079184, (F)-0.002893,
(F)-0.042175, (F)0.182434, (F)0.607727, (F)-0.025696, (F)-0.069702, (F)-0.001779,
(F)-0.033282, (F)-0.369332, (F)-0.000660, (F)-0.004858, (F)0.007694, (F)0.025631,
(F)-0.110870, (F)-0.001084, (F)0.004688, (F)0.616, (F)-0.002940, (F)0.012716,
(F)0.042360, (F)-0.001791,
(F)-0.117065, (F)0.006348, (F)-0.194702, (F)-0.208130, (F)-0.270996, (F)-0.013704,
(F)-0.000040, (F)-0.037909, (F)-0.043318, (F)-0.073439, (F)0.000743, (F)-0.022793,
(F)0.001236, (F)-0.024365, (F)0.001321, (F)-0.040523, (F)-0.031724, (F)0.001720,
(F)-0.052764, (F)-0.056402,
(F)-0.205017, (F)-0.101807, (F)0.113159, (F)0.423523, (F)0.087280, (F)-0.042032,
(F)-0.010365, (F)-0.012805, (F)-0.179372, (F)-0.007618, (F)-0.020872, (F)0.023200,
(F)0.011520, (F)0.086829, (F)0.043117, (F)-0.047926, (F)0.017894, (F)0.008886,
(F)-0.009877, (F)-0.036965,
(F)0.002563, (F)0.592407, (F)0.193115, (F)-0.058105, (F)0.055359, (F)-0.000007,
(F)-0.350946, (F)-0.037293, (F)-0.003376, (F)-0.003065, (F)-0.019, (F)-0.000495,
(F)-0.114403, (F)0.000149, (F)0.034422, (F)0.011221, (F)-0.000142, (F)-0.032795,
(F)-0.010691, (F)0.003217,
(F)0.098083, (F)0.047913, (F)0.174683, (F)0.264099, (F)0.201843, (F)-0.009620,
(F)-0.002296, (F)-0.030514, (F)-0.069748, (F)-0.040741, (F)-0.004699, (F)-0.017133,
(F)-0.008369, (F)-0.025904, (F)-0.012654, (F)-0.046134, (F)-0.019797, (F)-0.009671,
(F)-0.035259, (F)-0.053307,
(F)0.415100, (F)0.208496, (F)0.135925, (F)0.102661, (F)-0.196350, (F)-0.172308,
(F)-0.043471, (F)-0.018476, (F)-0.010539, (F)-0.038553, (F)-0.086547, (F)-0.056423,
(F)-0.028340, (F)-0.042615, (F)-0.021404, (F)-0.013954, (F)0.081505, (F)0.040938,
(F)0.026689, (F)0.028,
(F)0.211121, (F)0.237915, (F)0.022217, (F)0.478577, (F)-0.161133, (F)-0.044572,
(F)-0.056604, (F)-0.000494, (F)-0.229036, (F)-0.025964, (F)-0.050229, (F)-0.004690,
(F)-0.005286, (F)-0.101037, (F)-0.113861, (F)-0.010632, (F)0.034018, (F)0.038336,
(F)0.003580, (F)0.077114,
(F)-0.360229, (F)-0.211060, (F)-0.150452, (F)-0.115112, (F)0.264526, (F)-0.129765,
(F)-0.044546, (F)-0.022636, (F)-0.013251, (F)-0.069974, (F)-0.076030, (F)-0.054197,
(F)-0.031754, (F)-0.041467, (F)-0.024296, (F)-0.017319, (F)0.095290, (F)0.055831,
(F)0.039798, (F)0.030450,
(F)-0.226440, (F)0.069153, (F)0.479065, (F)0.210144, (F)-0.013123, (F)-0.051275,
(F)-0.004782, (F)-0.229503, (F)-0.044161, (F)-0.000172, (F)0.659, (F)0.108480,
(F)-0.033129, (F)0.047585, (F)-0.014532, (F)-0.100673, (F)-0.002971, (F)0.000907,
(F)0.006287, (F)0.002758,
(F)0.252197, (F)0.172607, (F)0.000122, (F)-0.257080, (F)-0.251221, (F)-0.063603,
(F)-0.029793, (F)0.000000, (F)-0.066090, (F)-0.063112, (F)-0.043531, (F)-0.000031,
(F)-0.000021, (F)0.064835, (F)0.044374, (F)0.000031, (F)0.063357, (F)0.043363,
(F)0.000031, (F)-0.064584,
(F)0.358582, (F)0.518555, (F)-0.109619, (F)0.083984, (F)-0.109009, (F)-0.128581,
(F)-0.268899, (F)-0.012016, (F)-0.007053, (F)-0.011883, (F)-0.185944, (F)0.039307,
(F)0.056844, (F)-0.030115, (F)-0.043550, (F)0.009206, (F)0.039089, (F)0.056527,
(F)-0.011949, (F)0.009155,
(F)0.191833, (F)0.212585, (F)0.216431, (F)-0.048401, (F)0.273987, (F)-0.036800,
(F)-0.045193, (F)-0.046842, (F)-0.002343, (F)-0.075069, (F)-0.040781, (F)-0.041519,
(F)-0.046010, (F)0.009285, (F)0.010289, (F)0.010475, (F)-0.052560, (F)-0.058246,
(F)-0.059299, (F)0.013261,
(F)0.173645, (F)0.486145, (F)0.225525, (F)-0.331543, (F)0.242798, (F)-0.033,
(F)-0.236337, (F)-0.050861, (F)-0.109921, (F)-0.058951, (F)-0.084417, (F)-0.039161,
(F)-0.109638, (F)0.057571, (F)0.161178, (F)0.074771, (F)-0.042161, (F)-0.118035,
(F)-0.054757, (F)0.080498,
(F)0.093872, (F)0.009155, (F)0.435730, (F)0.125061, (F)0.189026, (F)-0.008812,
(F)-0.000084, (F)-0.189861, (F)-0.640, (F)-0.035731, (F)-0.000859, (F)-0.040903,
(F)-0.003989, (F)-0.011740, (F)-0.001145, (F)-0.054493, (F)-0.017744, (F)-0.001731,
(F)-0.082364, (F)-0.023640,
(F)0.037720, (F)0.458984, (F)0.288635, (F)-0.014526, (F)-0.205872, (F)-0.001423,
(F)-0.210667, (F)-0.083310, (F)-0.000211, (F)-0.042383, (F)-0.017313, (F)-0.010887,
(F)-0.132479, (F)0.000548, (F)0.006667, (F)0.004193, (F)0.007765, (F)0.094492,
(F)0.059422, (F)-0.002991,
(F)0.180969, (F)-0.183899, (F)0.482910, (F)0.431885, (F)-0.121887, (F)-0.032750,
(F)-0.033819, (F)-0.233202, (F)-0.186524, (F)-0.014856, (F)0.033280, (F)-0.087392,
(F)0.088807, (F)-0.078158, (F)0.079423, (F)-0.208562, (F)0.022058, (F)-0.022415,
(F)0.058861, (F)0.052641,
(F)0.149109, (F)-0.056702, (F)-0.067932, (F)0.590637, (F)0.252563, (F)-0.022233,
(F)-0.003215, (F)-0.004615, (F)-0.348852, (F)-0.063788, (F)0.008455, (F)0.010129,
(F)-0.003852, (F)-0.088069, (F)0.033490, (F)0.040123, (F)-0.037659, (F)0.014321,
(F)0.017157, (F)-0.149173,
(F)0.119995, (F)0.201965, (F)0.615540, (F)-0.207458, (F)0.128479, (F)-0.014399,
(F)-0.040790, (F)-0.378889, (F)-0.043039, (F)-0.016507, (F)-0.024235, (F)-0.073862,
(F)-0.124318, (F)0.024894, (F)0.041899, (F)0.127699, (F)-0.417, (F)-0.025948,
(F)-0.079084, (F)0.026654,
(F)-0.006836, (F)-0.083496, (F)0.541260, (F)0.294312, (F)0.125000, (F)-0.000047,
(F)-0.006972, (F)-0.292962, (F)-0.086619, (F)-0.625, (F)-0.000571, (F)0.003700,
(F)0.045193, (F)0.002012, (F)0.024574, (F)-0.159299, (F)0.000854, (F)0.010437,
(F)-0.067657, (F)-0.036789,
(F)-0.158508, (F)0.026733, (F)-0.112244, (F)0.380188, (F)0.443481, (F)-0.025125,
(F)-0.000715, (F)-0.012599, (F)-0.144543, (F)-0.196676, (F)0.004237, (F)-0.017792,
(F)0.003001, (F)0.060263, (F)-0.010164, (F)0.042674, (F)0.070295, (F)-0.011856,
(F)0.049778, (F)-0.168606,
(F)0.210815, (F)0.033752, (F)0.557068, (F)0.301880, (F)-0.341553, (F)-0.044443,
(F)-0.001139, (F)-0.310325, (F)-0.091131, (F)-0.116658, (F)-0.007116, (F)-0.117439,
(F)-0.018802, (F)-0.063641, (F)-0.010189, (F)-0.168168, (F)0.072005, (F)0.011528,
(F)0.190268, (F)0.103108,
(F)-0.032471, (F)-0.040466, (F)0.748413, (F)0.152283, (F)-0.061279, (F)-0.001054,
(F)-0.001638, (F)-0.560122, (F)-0.023190, (F)-0.003755, (F)-0.001314, (F)0.024301,
(F)0.030286, (F)0.004945, (F)0.006162, (F)-0.113970, (F)-0.001990, (F)-0.002480,
(F)0.045862, (F)0.009332,
(F)-0.128174, (F)0.080383, (F)-0.092102, (F)0.697327, (F)0.182922, (F)-0.016429,
(F)-0.006461, (F)-0.008483, (F)-0.486264, (F)-0.033461, (F)0.010303, (F)-0.011805,
(F)0.007403, (F)0.089379, (F)-0.056053, (F)0.064225, (F)0.023446, (F)-0.014704,
(F)0.016848, (F)-0.127557,
(F)0.072815, (F)-0.164612, (F)0.687195, (F)0.327942, (F)-0.046570, (F)-0.005302,
(F)-0.027097, (F)-0.472237, (F)-0.107546, (F)-0.002169, (F)0.011986, (F)-0.050038,
(F)0.113120, (F)-0.023879, (F)0.053983, (F)-0.225360, (F)0.003391, (F)-0.007666,
(F)0.032003, (F)0.272,
(F)0.146790, (F)-0.028748, (F)0.676208, (F)0.012390, (F)0.047729, (F)-0.021547,
(F)-0.000826, (F)-0.457258, (F)-0.004, (F)-0.002278, (F)0.004220, (F)-0.099260,
(F)0.019439, (F)-0.001819, (F)0.000356, (F)-0.008378, (F)-0.007006, (F)0.001372,
(F)-0.032275, (F)-0.000591,
(F)-0.114197, (F)0.346985, (F)0.689087, (F)-0.162476, (F)0.082581, (F)-0.013041,
(F)-0.120398, (F)-0.474841, (F)-0.026398, (F)-0.006820, (F)0.039625, (F)0.078692,
(F)-0.239103, (F)-0.018554, (F)0.056377, (F)0.111960, (F)0.009430, (F)-0.028654,
(F)-0.056905, (F)0.013417,
(F)-0.229980, (F)0.361755, (F)-0.188782, (F)0.307678, (F)0.318115, (F)-0.052891,
(F)-0.130867, (F)-0.035639, (F)-0.094666, (F)-0.101197, (F)0.083197, (F)-0.043416,
(F)0.068293, (F)0.070760, (F)-0.111304, (F)0.058084, (F)0.073160, (F)-0.115080,
(F)0.060054, (F)-0.097877,
(F)-0.155396, (F)0.579956, (F)0.364319, (F)-0.148926, (F)0.117676, (F)-0.024148,
(F)-0.336349, (F)-0.132728, (F)-0.022179, (F)-0.013848, (F)0.090123, (F)0.056614,
(F)-0.211289, (F)-0.023142, (F)0.086370, (F)0.054256, (F)0.018286, (F)-0.068247,
(F)-0.042872, (F)0.017525,
(F)-0.302917, (F)0.217773, (F)0.280273, (F)0.336182, (F)0.210632, (F)-0.091759,
(F)-0.047425, (F)-0.078553, (F)-0.113018, (F)-0.044366, (F)0.065967, (F)0.084900,
(F)-0.061036, (F)0.101835, (F)-0.073211, (F)-0.094223, (F)0.063804, (F)-0.045870,
(F)-0.059035, (F)-0.070811,
(F)0.106506, (F)-0.157837, (F)0.243103, (F)0.665222, (F)-0.094666, (F)-0.011344,
(F)-0.024912, (F)-0.059099, (F)-0.442521, (F)-0.008962, (F)0.016811, (F)-0.025892,
(F)0.038371, (F)-0.070850, (F)0.104997, (F)-0.161718, (F)0.010082, (F)-0.014942,
(F)0.023013, (F)0.062974,
(F)0.052917, (F)-0.027832, (F)-0.044373, (F)0.205627, (F)0.721558, (F)-0.002800,
(F)-0.000775, (F)-0.001969, (F)-0.042283, (F)-0.520645, (F)0.001473, (F)0.002348,
(F)-0.001235, (F)-0.010881, (F)0.005723, (F)0.009124, (F)-0.038183, (F)0.020082,
(F)0.032017, (F)-0.148372,
(F)0.459900, (F)0.147827, (F)0.117554, (F)0.085754, (F)0.097595, (F)-0.211508,
(F)-0.021853, (F)-0.013819, (F)-0.007354, (F)-0.009525, (F)-0.067986, (F)-0.054063,
(F)-0.017378, (F)-0.039438, (F)-0.012677, (F)-0.010081, (F)-0.044884, (F)-0.014427,
(F)-0.011473, (F)-0.008369,
(F)0.063538, (F)-0.044861, (F)-0.008667, (F)0.806213, (F)0.092468, (F)-0.004037,
(F)-0.002012, (F)-0.000075, (F)-0.649980, (F)-0.008550, (F)0.002850, (F)0.000551,
(F)-0.000389, (F)-0.051225, (F)0.036167, (F)0.006987, (F)-0.005875, (F)0.004148,
(F)0.000801, (F)-0.074549,
(F)0.485229, (F)0.411682, (F)-0.088135, (F)-0.061523, (F)0.166077, (F)-0.235448,
(F)-0.169482, (F)-0.007768, (F)-0.003785, (F)-0.027581, (F)-0.199760, (F)0.042766,
(F)0.036284, (F)0.029853, (F)0.025328, (F)-0.005422, (F)-0.080585, (F)-0.068371,
(F)0.014637, (F)0.010218,
(F)-0.290161, (F)-0.084167, (F)0.276001, (F)-0.035278, (F)-0.349915, (F)-0.084193,
(F)-0.007084, (F)-0.076177, (F)-0.001245, (F)-0.122440, (F)-0.024422, (F)0.080085,
(F)0.023230, (F)-0.010236, (F)-0.002969, (F)0.009737, (F)-0.132, (F)-0.029451,
(F)0.096577, (F)-0.012344,
(F)0.196533, (F)0.364868, (F)0.361511, (F)0.148315, (F)-0.164124, (F)-0.038625,
(F)-0.133129, (F)-0.130690, (F)-0.021997, (F)-0.026937, (F)-0.071709, (F)-0.071049,
(F)-0.131904, (F)-0.029149, (F)-0.054116, (F)-0.053618, (F)0.032256, (F)0.059883,
(F)0.059333, (F)0.024342,
(F)0.023010, (F)0.080078, (F)0.853149, (F)-0.089722, (F)0.041321, (F)-0.000529,
(F)-0.006413, (F)-0.727864, (F)-0.008050, (F)-0.001707, (F)-0.001843, (F)-0.019631,
(F)-0.068319, (F)0.002065, (F)0.007185, (F)0.076546, (F)-0.000951, (F)-0.003309,
(F)-0.035253, (F)0.003707,
(F)-0.209656, (F)0.413208, (F)0.019165, (F)-0.054016, (F)0.347046, (F)-0.043956,
(F)-0.170741, (F)-0.000367, (F)-0.002918, (F)-0.120441, (F)0.086631, (F)0.004018,
(F)-0.007919, (F)-0.011325, (F)0.022320, (F)0.001035, (F)0.072760, (F)-0.143402,
(F)-0.006651, (F)0.018746,
(F)0.193542, (F)0.535522, (F)-0.137939, (F)0.163513, (F)0.187317, (F)-0.037459,
(F)-0.286784, (F)-0.019027, (F)-0.026737, (F)-0.035088, (F)-0.103646, (F)0.026697,
(F)0.073870, (F)-0.031647, (F)-0.087565, (F)0.022555, (F)-0.036254, (F)-0.100312,
(F)0.025838, (F)-0.030629,
(F)0.339172, (F)0.092102, (F)0.330017, (F)-0.007629, (F)-0.450806, (F)-0.115038,
(F)-0.008483, (F)-0.108911, (F)-0.000058, (F)-0.203226, (F)-0.031238, (F)-0.111933,
(F)-0.030395, (F)0.002588, (F)0.000703, (F)0.002518, (F)0.152901, (F)0.041520,
(F)0.148774, (F)-0.003439,
(F)-0.173218, (F)0.158386, (F)0.807373, (F)0.016541, (F)0.109436, (F)-0.030004,
(F)-0.025086, (F)-0.651851, (F)-0.000274, (F)-0.011976, (F)0.027435, (F)0.139851,
(F)-0.127877, (F)0.002865, (F)-0.002620, (F)-0.013354, (F)0.018956, (F)-0.017333,
(F)-0.088356, (F)-0.001810,
(F)0.040527, (F)-0.016968, (F)0.913452, (F)0.058044, (F)-0.043396, (F)-0.001642,
(F)-0.000288, (F)-0.834395, (F)-0.003369, (F)-0.001883, (F)0.000688, (F)-0.037020,
(F)0.499, (F)-0.002352, (F)0.000985, (F)-0.053021, (F)0.001759, (F)-0.000736,
(F)0.039640, (F)0.002519,
(F)0.049316, (F)0.108032, (F)0.528442, (F)0.357849, (F)-0.090698, (F)-0.002432,
(F)-0.011671, (F)-0.279251, (F)-0.128056, (F)-0.008226, (F)-0.005328, (F)-0.026061,
(F)-0.057089, (F)-0.017648, (F)-0.038659, (F)-0.189103, (F)0.004473, (F)0.009798,
(F)0.047929, (F)0.032456,
(F)0.186951, (F)0.093933, (F)0.122070, (F)-0.033081, (F)0.584473, (F)-0.034951,
(F)-0.008823, (F)-0.014901, (F)-0.001094, (F)-0.341608, (F)-0.017561, (F)-0.022821,
(F)-0.011466, (F)0.006185, (F)0.003107, (F)0.004038, (F)-0.109268, (F)-0.054901,
(F)-0.071347, (F)0.019335,
(F)0.031372, (F)0.859558, (F)-0.019714, (F)0.054688, (F)-0.054382, (F)-0.000984,
(F)-0.738840, (F)-0.000389, (F)-0.002991, (F)-0.002957, (F)-0.026966, (F)0.000618,
(F)0.016946, (F)-0.001716, (F)-0.047007, (F)0.001078, (F)0.001706, (F)0.046745,
(F)-0.001072, (F)0.002974,
(F)0.054443, (F)0.009644, (F)0.319641, (F)0.674866, (F)-0.097839, (F)-0.002964,
(F)-0.000093, (F)-0.102170, (F)-0.455444, (F)-0.009573, (F)-0.000525, (F)-0.017402,
(F)-0.003082, (F)-0.036742, (F)-0.006508, (F)-0.215715, (F)0.005327, (F)0.000944,
(F)0.031273, (F)0.066028,
(F)-0.098389, (F)0.156494, (F)-0.262207, (F)0.764526, (F)0.348511, (F)-0.009680,
(F)-0.024490, (F)-0.068753, (F)-0.584501, (F)-0.121460, (F)0.397, (F)-0.025798,
(F)0.041034, (F)0.075221, (F)-0.119644, (F)0.200464, (F)0.034290, (F)-0.054540,
(F)0.091382, (F)-0.266446,
(F)0.298401, (F)-0.419800, (F)0.037354, (F)0.372864, (F)0.290100, (F)-0.089043,
(F)-0.176232, (F)-0.001395, (F)-0.139027, (F)-0.084158, (F)0.125269, (F)-0.011146,
(F)0.681, (F)-0.111263, (F)0.156528, (F)-0.013928, (F)-0.086566, (F)0.121784,
(F)-0.010836, (F)-0.108168,
(F)0.186340, (F)-0.030884, (F)0.146667, (F)0.587463, (F)0.080688, (F)-0.034723,
(F)-0.000954, (F)-0.021511, (F)-0.345113, (F)-0.006511, (F)0.005755, (F)-0.027330,
(F)0.004530, (F)-0.109468, (F)0.018143, (F)-0.086162, (F)-0.036, (F)0.002492,
(F)-0.011834, (F)-0.047402,
(F)-0.046021, (F)0.787598, (F)0.273438, (F)-0.128967, (F)0.089783, (F)-0.002118,
(F)-0.620310, (F)-0.074768, (F)-0.016633, (F)-0.008061, (F)0.036246, (F)0.012584,
(F)-0.215359, (F)-0.005935, (F)0.174, (F)0.035264, (F)0.004132, (F)-0.070713,
(F)-0.024550, (F)0.011579,
(F)0.259705, (F)0.090088, (F)0.308533, (F)0.146057, (F)0.177734, (F)-0.067446,
(F)-0.008116, (F)-0.095192, (F)-0.021333, (F)-0.031590, (F)-0.023396, (F)-0.080127,
(F)-0.027795, (F)-0.037932, (F)-0.013158, (F)-0.045063, (F)-0.046158, (F)-0.016012,
(F)-0.054837, (F)-0.025959,
(F)-0.083679, (F)0.032654, (F)0.086487, (F)0.891357, (F)-0.060852, (F)-0.007002,
(F)-0.001066, (F)-0.007480, (F)-0.794518, (F)-0.003703, (F)0.002732, (F)0.007237,
(F)-0.002824, (F)0.074588, (F)-0.029106, (F)-0.077091, (F)-0.005092, (F)0.001987,
(F)0.005263, (F)0.054241,
(F)0.327637, (F)-0.106445, (F)0.249207, (F)-0.075439, (F)0.229065, (F)-0.107346,
(F)-0.011331, (F)-0.062104, (F)-0.005691, (F)-0.052471, (F)0.034875, (F)-0.081649,
(F)0.026527, (F)0.024717, (F)-0.008030, (F)0.018800, (F)-0.075050, (F)0.024383,
(F)-0.057084, (F)0.017281,
(F)0.096924, (F)-0.189026, (F)0.879822, (F)0.225769, (F)-0.071472, (F)-0.009394,
(F)-0.035731, (F)-0.774086, (F)-0.050972, (F)-0.005108, (F)0.018321, (F)-0.085276,
(F)0.166309, (F)-0.021882, (F)0.042676, (F)-0.198637, (F)0.006927, (F)-0.013510,
(F)0.062883, (F)0.016136,
(F)0.012939, (F)0.487610, (F)0.578064, (F)-0.204102, (F)0.122620, (F)-0.000167,
(F)-0.237763, (F)-0.334158, (F)-0.041657, (F)-0.036, (F)-0.006309, (F)-0.007480,
(F)-0.281870, (F)0.002641, (F)0.099522, (F)0.117984, (F)-0.087, (F)-0.059791,
(F)-0.070882, (F)0.025027,
(F)0.052246, (F)0.140, (F)-0.038269, (F)0.422974, (F)0.422913, (F)-0.002730,
(F)-0.019604, (F)-0.001465, (F)-0.178907, (F)-0.178855, (F)-0.007315, (F)0.001999,
(F)0.005358, (F)-0.022099, (F)-0.059223, (F)0.016187, (F)-0.022096, (F)-0.059214,
(F)0.016184, (F)-0.178881,
(F)0.143860, (F)-0.255554, (F)0.437134, (F)0.468933, (F)0.205750, (F)-0.020696,
(F)-0.065308, (F)-0.191086, (F)-0.219898, (F)-0.042333, (F)0.036764, (F)-0.062886,
(F)0.111711, (F)-0.067461, (F)0.119838, (F)-0.204987, (F)-0.029599, (F)0.052580,
(F)-0.089940, (F)-0.096483,
(F)0.021362, (F)0.279846, (F)0.857727, (F)-0.233093, (F)0.074341, (F)-0.000456,
(F)-0.078314, (F)-0.735696, (F)-0.054332, (F)-0.005527, (F)-0.005978, (F)-0.018323,
(F)-0.240032, (F)0.004979, (F)0.065230, (F)0.199930, (F)-0.088, (F)-0.020804,
(F)-0.063764, (F)0.017328,
(F)-0.198242, (F)0.215515, (F)0.028992, (F)0.130432, (F)0.714783, (F)-0.039300,
(F)-0.046447, (F)-0.000841, (F)-0.017013, (F)-0.510914, (F)0.042724, (F)0.005747,
(F)-0.006248, (F)0.025857, (F)-0.028110, (F)-0.003781, (F)0.141700, (F)-0.154046,
(F)-0.020723, (F)-0.093231,
(F)-0.184143, (F)0.235107, (F)0.105286, (F)0.515137, (F)0.320862, (F)-0.033909,
(F)-0.055275, (F)-0.011085, (F)-0.265366, (F)-0.102952, (F)0.043293, (F)0.019388,
(F)-0.024753, (F)0.094859, (F)-0.121112, (F)-0.054237, (F)0.059084, (F)-0.075437,
(F)-0.033782, (F)-0.165288,
(F)-0.055481, (F)0.195068, (F)0.946777, (F)-0.154297, (F)0.065063, (F)-0.003078,
(F)-0.038052, (F)-0.896387, (F)-0.023808, (F)-0.004233, (F)0.010823, (F)0.052528,
(F)-0.184686, (F)-0.008561, (F)0.030098, (F)0.146085, (F)0.003610, (F)-0.012692,
(F)-0.061601, (F)0.010039,
(F)0.058533, (F)-0.205688, (F)0.586975, (F)0.582581, (F)-0.107483, (F)-0.003426,
(F)-0.042308, (F)-0.344540, (F)-0.339400, (F)-0.011553, (F)0.012040, (F)-0.034357,
(F)0.120734, (F)-0.034100, (F)0.119830, (F)-0.341960, (F)0.006291, (F)-0.022108,
(F)0.063090, (F)0.062617,
(F)0.462341, (F)0.719971, (F)-0.295715, (F)0.198486, (F)-0.103943, (F)-0.213759,
(F)-0.518358, (F)-0.087448, (F)-0.039397, (F)-0.010804, (F)-0.332872, (F)0.136721,
(F)0.212906, (F)-0.091768, (F)-0.142904, (F)0.058695, (F)0.048057, (F)0.074836,
(F)-0.030738, (F)0.020631,
(F)0.391174, (F)0.105103, (F)0.107666, (F)-0.255798, (F)0.367126, (F)-0.153017,
(F)-0.011047, (F)-0.011592, (F)-0.065433, (F)-0.134782, (F)-0.041113, (F)-0.042116,
(F)-0.011316, (F)0.100062, (F)0.026885, (F)0.027541, (F)-0.143610, (F)-0.038586,
(F)-0.039527, (F)0.093910,
(F)-0.147705, (F)0.742249, (F)0.307495, (F)0.078491, (F)-0.128052, (F)-0.021817,
(F)-0.550933, (F)-0.094553, (F)-0.006161, (F)-0.016397, (F)0.109634, (F)0.045419,
(F)-0.228238, (F)0.011594, (F)-0.058260, (F)-0.024136, (F)-0.018914, (F)0.095046,
(F)0.039375, (F)0.010051,
(F)-0.136230, (F)-0.081848, (F)0.442261, (F)0.578003, (F)-0.120667, (F)-0.018559,
(F)-0.006699, (F)-0.195595, (F)-0.334087, (F)-0.014560, (F)-0.011150, (F)0.060249,
(F)0.036198, (F)0.078742, (F)0.047308, (F)-0.255628, (F)-0.016438, (F)-0.009876,
(F)0.053366, (F)0.069746,
(F)0.062866, (F)0.429321, (F)0.258240, (F)0.094666, (F)0.187805, (F)-0.003952,
(F)-0.184317, (F)-0.066688, (F)-0.008962, (F)-0.035271, (F)-0.026990, (F)-0.016235,
(F)-0.110868, (F)-0.005951, (F)-0.040642, (F)-0.024446, (F)-0.011807, (F)-0.080629,
(F)-0.048499, (F)-0.017779,
(F)0.312561, (F)0.722351, (F)-0.029114, (F)-0.106750, (F)0.140259, (F)-0.097694,
(F)-0.521791, (F)-0.000848, (F)-0.011396, (F)-0.019673, (F)-0.225779, (F)0.009100,
(F)0.021030, (F)0.033366, (F)0.077111, (F)-0.003108, (F)-0.043839, (F)-0.101316,
(F)0.004083, (F)0.014973,
(F)-0.203125, (F)0.233582, (F)0.361389, (F)0.608887, (F)-0.063782, (F)-0.041260,
(F)-0.054560, (F)-0.130602, (F)-0.370743, (F)-0.004068, (F)0.047446, (F)0.073407,
(F)-0.084414, (F)0.123680, (F)-0.142225, (F)-0.220045, (F)-0.012956, (F)0.014898,
(F)0.023050, (F)0.038836,
(F)0.232727, (F)0.348999, (F)-0.284790, (F)-0.208252, (F)0.346313, (F)-0.054162,
(F)-0.121800, (F)-0.081105, (F)-0.043369, (F)-0.119933, (F)-0.081222, (F)0.066278,
(F)0.099391, (F)0.048466, (F)0.072680, (F)-0.059308, (F)-0.080597, (F)-0.120863,
(F)0.098627, (F)0.072120,
(F)0.082275, (F)-0.224121, (F)0.802856, (F)0.423462, (F)-0.103149, (F)-0.006769,
(F)-0.050230, (F)-0.644578, (F)-0.179320, (F)-0.010640, (F)0.018440, (F)-0.066055,
(F)0.179937, (F)-0.034840, (F)0.094907, (F)-0.339979, (F)0.008487, (F)-0.023118,
(F)0.082814, (F)0.043680,
(F)0.131287, (F)-0.218872, (F)0.424133, (F)0.740417, (F)-0.074707, (F)-0.017236,
(F)-0.047905, (F)-0.179889, (F)-0.548218, (F)-0.005581, (F)0.028735, (F)-0.055683,
(F)0.092831, (F)-0.097207, (F)0.162057, (F)-0.314036, (F)0.009808, (F)-0.016351,
(F)0.031686, (F)0.055314,
(F)-0.176331, (F)0.588806, (F)0.627197, (F)-0.166504, (F)0.060730, (F)-0.031092,
(F)-0.346693, (F)-0.393376, (F)-0.027724, (F)-0.003688, (F)0.103825, (F)0.110594,
(F)-0.369298, (F)-0.029360, (F)0.098039, (F)0.104431, (F)0.010709, (F)-0.035758,
(F)-0.038090, (F)0.010112,
(F)0.377014, (F)0.133301, (F)0.033875, (F)0.121155, (F)0.402161, (F)-0.142140,
(F)-0.017769, (F)-0.001147, (F)-0.014678, (F)-0.161733, (F)-0.050256, (F)-0.012771,
(F)-0.004515, (F)-0.045677, (F)-0.016150, (F)-0.004104, (F)-0.151620, (F)-0.053608,
(F)-0.013623, (F)-0.048724,
(F)-0.030029, (F)0.023865, (F)1.047729, (F)-0.041565, (F)0.014648, (F)-0.000902,
(F)-0.000570, (F)-1.097737, (F)-0.001728, (F)-0.000215, (F)0.000717, (F)0.031463,
(F)-0.025004, (F)-0.001248, (F)0.000992, (F)0.043549, (F)0.000440, (F)-0.000350,
(F)-0.348, (F)0.000609,
(F)0.770752, (F)0.332520, (F)-0.160522, (F)0.183777, (F)-0.164490, (F)-0.594059,
(F)-0.110569, (F)-0.025767, (F)-0.033774, (F)-0.027057, (F)-0.256290, (F)0.123723,
(F)0.053377, (F)-0.141646, (F)-0.061109, (F)0.029500, (F)0.126781, (F)0.054696,
(F)-0.026404, (F)0.030229,
(F)-0.221436, (F)-0.032593, (F)0.210388, (F)0.427368, (F)0.412048, (F)-0.049034,
(F)-0.001062, (F)-0.044263, (F)-0.182644, (F)-0.169784, (F)-0.007217, (F)0.046587,
(F)0.006857, (F)0.094635, (F)0.013929, (F)-0.089913, (F)0.091242, (F)0.013430,
(F)-0.086690, (F)-0.176096,
(F)-0.319763, (F)0.360168, (F)0.494812, (F)0.222778, (F)0.295776, (F)-0.102248,
(F)-0.129721, (F)-0.244839, (F)-0.049630, (F)-0.087484, (F)0.115169, (F)0.158223,
(F)-0.178216, (F)0.071236, (F)-0.080238, (F)-0.110233, (F)0.094578, (F)-0.106529,
(F)-0.146354, (F)-0.065893,
(F)0.375793, (F)0.407593, (F)0.250183, (F)-0.065552, (F)0.143311, (F)-0.141221,
(F)-0.166132, (F)-0.062592, (F)-0.004297, (F)-0.020538, (F)-0.153171, (F)-0.094017,
(F)-0.101973, (F)0.024634, (F)0.026718, (F)0.016400, (F)-0.053855, (F)-0.058412,
(F)-0.035854, (F)0.009394,
(F)0.065857, (F)0.982605, (F)0.014771, (F)-0.053772, (F)0.103149, (F)-0.004337,
(F)-0.965513, (F)-0.000218, (F)-0.002891, (F)-0.010640, (F)-0.064711, (F)-0.000973,
(F)-0.014514, (F)0.003541, (F)0.052837, (F)0.000794, (F)-0.006793, (F)-0.101355,
(F)-0.024, (F)0.005547,
(F)-0.016968, (F)0.013855, (F)-0.063416, (F)0.099854, (F)1.034241, (F)-0.000288,
(F)-0.000192, (F)-0.004022, (F)-0.009971, (F)-1.069654, (F)0.000235, (F)-0.001076,
(F)0.000879, (F)0.001694, (F)-0.001383, (F)0.006332, (F)0.017549, (F)-0.014329,
(F)0.065587, (F)-0.103273,
(F)0.963013, (F)-0.088745, (F)0.035034, (F)-0.036804, (F)-0.006531, (F)-0.927393,
(F)-0.007876, (F)-0.001227, (F)-0.001355, (F)-0.000043, (F)0.085463, (F)-0.033738,
(F)0.003109, (F)0.035443, (F)-0.003266, (F)0.001289, (F)0.006289, (F)-0.000580,
(F)0.000229, (F)-0.000240,
(F)0.146851, (F)-0.051025, (F)0.807129, (F)0.487854, (F)-0.260376, (F)-0.021565,
(F)-0.002604, (F)-0.651457, (F)-0.238002, (F)-0.067796, (F)0.007493, (F)-0.118527,
(F)0.041184, (F)-0.071642, (F)0.024893, (F)-0.393761, (F)0.038236, (F)-0.013286,
(F)0.217, (F)0.127025,
(F)0.194336, (F)0.876953, (F)-0.203613, (F)0.137634, (F)-0.109924, (F)-0.037766,
(F)-0.769047, (F)-0.041458, (F)-0.018943, (F)-0.012083, (F)-0.170424, (F)0.039569,
(F)0.178559, (F)-0.026747, (F)-0.120699, (F)0.028024, (F)0.021362, (F)0.096398,
(F)-0.022382, (F)0.129,
(F)-0.103638, (F)1.042236, (F)0.152832, (F)-0.032898, (F)-0.039429, (F)-0.010741,
(F)-1.086257, (F)-0.023358, (F)-0.001082, (F)-0.055, (F)0.108015, (F)0.015839,
(F)-0.159287, (F)-0.003409, (F)0.034287, (F)0.005028, (F)-0.004086, (F)0.041094,
(F)0.006026, (F)-0.001297,
(F)0.144958, (F)-0.048645, (F)-0.164551, (F)0.883972, (F)0.328552, (F)-0.021013,
(F)-0.002366, (F)-0.027077, (F)-0.781407, (F)-0.107947, (F)0.007052, (F)0.023853,
(F)-0.008005, (F)-0.128139, (F)0.043001, (F)0.145458, (F)-0.047626, (F)0.982,
(F)0.054064, (F)-0.290431,
(F)-0.164246, (F)0.155884, (F)-0.275879, (F)0.584290, (F)0.650330, (F)-0.026977,
(F)-0.024300, (F)-0.076109, (F)-0.341394, (F)-0.422929, (F)0.025603, (F)-0.045312,
(F)0.043005, (F)0.095967, (F)-0.091081, (F)0.161193, (F)0.106814, (F)-0.101376,
(F)0.179412, (F)-0.379981,
(F)-0.264404, (F)-0.268494, (F)-0.101135, (F)0.297852, (F)0.450134, (F)-0.069910,
(F)-0.072089, (F)-0.010228, (F)-0.088716, (F)-0.202621, (F)-0.070991, (F)-0.026741,
(F)-0.027154, (F)0.078753, (F)0.079971, (F)0.030123, (F)0.119017, (F)0.120858,
(F)0.045524, (F)-0.134073,
(F)-0.213501, (F)0.451965, (F)0.607605, (F)0.164368, (F)-0.181335, (F)-0.045583,
(F)-0.204273, (F)-0.369184, (F)-0.027017, (F)-0.032883, (F)0.096495, (F)0.129724,
(F)-0.274616, (F)0.035093, (F)-0.074288, (F)-0.099871, (F)-0.038715, (F)0.081957,
(F)0.110180, (F)0.029806,
(F)0.409973, (F)-0.231445, (F)0.472351, (F)0.170776, (F)0.212952, (F)-0.168078,
(F)-0.053567, (F)-0.223116, (F)-0.029165, (F)-0.045348, (F)0.094886, (F)-0.193651,
(F)0.109323, (F)-0.070014, (F)0.039525, (F)-0.080666, (F)-0.087304, (F)0.049287,
(F)-0.100588, (F)-0.036367,
(F)0.321228, (F)0.596313, (F)0.211365, (F)0.065247, (F)-0.242371, (F)-0.103187,
(F)-0.355590, (F)-0.044675, (F)-0.004257, (F)-0.058744, (F)-0.191553, (F)-0.067896,
(F)-0.126040, (F)-0.020959, (F)-0.038907, (F)-0.013791, (F)0.077856, (F)0.144529,
(F)0.051229, (F)0.814,
(F)0.075867, (F)-0.160767, (F)1.023682, (F)0.114197, (F)0.020264, (F)-0.005756,
(F)-0.025846, (F)-1.047924, (F)-0.013041, (F)-0.000411, (F)0.012197, (F)-0.077663,
(F)0.164574, (F)-0.008664, (F)0.018359, (F)-0.116901, (F)-0.037, (F)0.003258,
(F)-0.020744, (F)-0.002314,
(F)0.164246, (F)0.173096, (F)0.189514, (F)0.348572, (F)0.294006, (F)-0.026977,
(F)-0.029962, (F)-0.035916, (F)-0.121502, (F)-0.086440, (F)-0.028430, (F)-0.031127,
(F)-0.032804, (F)-0.057251, (F)-0.060336, (F)-0.066059, (F)-0.048289, (F)-0.050891,
(F)-0.055718, (F)-0.102482,
(F)0.266479, (F)0.179688, (F)0.440063, (F)0.221375, (F)0.070862, (F)-0.071011,
(F)-0.032288, (F)-0.193656, (F)-0.049007, (F)-0.005021, (F)-0.047883, (F)-0.117268,
(F)-0.079074, (F)-0.058992, (F)-0.039778, (F)-0.097419, (F)-0.018883, (F)-0.012733,
(F)-0.031184, (F)-0.687,
(F)0.847656, (F)0.241150, (F)-0.040955, (F)-0.140686, (F)0.204712, (F)-0.718521,
(F)-0.058153, (F)-0.001677, (F)-0.019793, (F)-0.041907, (F)-0.204412, (F)0.034715,
(F)0.009876, (F)0.119253, (F)0.033926, (F)-0.005762, (F)-0.173525, (F)-0.049366,
(F)0.008384, (F)0.028800,
(F)0.430481, (F)0.356873, (F)0.178772, (F)0.511353, (F)-0.292603, (F)-0.185314,
(F)-0.127358, (F)-0.031959, (F)-0.261481, (F)-0.085616, (F)-0.153627, (F)-0.076958,
(F)-0.063799, (F)-0.220128, (F)-0.182488, (F)-0.091416, (F)0.125960, (F)0.104422,
(F)0.052309, (F)0.149623,
(F)-0.133728, (F)0.783569, (F)0.560425, (F)-0.238953, (F)0.098694, (F)-0.017883,
(F)-0.613981, (F)-0.314076, (F)-0.057098, (F)-0.009740, (F)0.104785, (F)0.074945,
(F)-0.439132, (F)-0.031955, (F)0.187236, (F)0.133915, (F)0.013198, (F)-0.077333,
(F)-0.055310, (F)0.023583,
(F)-0.297913, (F)0.800415, (F)0.351990, (F)-0.026550, (F)0.253601, (F)-0.088752,
(F)-0.640664, (F)-0.123897, (F)-0.000705, (F)-0.064314, (F)0.238454, (F)0.104862,
(F)-0.281738, (F)-0.007910, (F)0.021251, (F)0.009345, (F)0.075551, (F)-0.202986,
(F)-0.089265, (F)0.006733,
(F)0.478699, (F)-0.076294, (F)0.012817, (F)0.433716, (F)-0.317566, (F)-0.229152,
(F)-0.005821, (F)-0.000164, (F)-0.188109, (F)-0.100848, (F)0.036522, (F)-0.006136,
(F)0.000978, (F)-0.207619, (F)0.033090, (F)-0.005559, (F)0.152018, (F)-0.024228,
(F)0.004070, (F)0.137733,
(F)-0.150330, (F)0.498535, (F)0.888123, (F)-0.238525, (F)0.111572, (F)-0.022599,
(F)-0.248537, (F)-0.788762, (F)-0.056894, (F)-0.012448, (F)0.074945, (F)0.133511,
(F)-0.442760, (F)-0.035857, (F)0.118913, (F)0.211840, (F)0.016773, (F)-0.055623,
(F)-0.099090, (F)0.026613,
(F)-0.096375, (F)0.021179, (F)-0.016602, (F)-0.166931, (F)0.978455, (F)-0.009288,
(F)-0.000449, (F)-0.000276, (F)-0.027866, (F)-0.957373, (F)0.002041, (F)-0.001600,
(F)0.000352, (F)-0.016088, (F)0.003535, (F)-0.002771, (F)0.094298, (F)-0.020723,
(F)0.016244, (F)0.163335,
(F)-0.386169, (F)0.081055, (F)0.310852, (F)-0.309998, (F)0.464905, (F)-0.149127,
(F)-0.006570, (F)-0.096629, (F)-0.096098, (F)-0.216136, (F)0.031301, (F)0.120042,
(F)-0.025196, (F)-0.119712, (F)0.025127, (F)0.096363, (F)0.179532, (F)-0.037683,
(F)-0.144517, (F)0.144119,
(F)0.200562, (F)-0.275208, (F)0.690063, (F)0.672913, (F)-0.160095, (F)-0.040225,
(F)-0.075739, (F)-0.476188, (F)-0.452811, (F)-0.025630, (F)0.055196, (F)-0.138400,
(F)0.189911, (F)-0.134960, (F)0.185191, (F)-0.464352, (F)0.032109, (F)-0.044059,
(F)0.110476, (F)0.107730,
(F)0.339783, (F)1.089661, (F)-0.229126, (F)0.071167, (F)-0.031677, (F)-0.115452,
(F)-1.187360, (F)-0.052499, (F)-0.005065, (F)-0.001003, (F)-0.370248, (F)0.077853,
(F)0.249670, (F)-0.024181, (F)-0.077548, (F)0.016306, (F)0.010763, (F)0.034517,
(F)-0.007258, (F)0.002254,
(F)-0.130371, (F)-0.025879, (F)0.933350, (F)0.311768, (F)-0.100586, (F)-0.016997,
(F)-0.000670, (F)-0.871141, (F)-0.097199, (F)-0.010118, (F)-0.003374, (F)0.121682,
(F)0.024154, (F)0.040645, (F)0.008068, (F)-0.290988, (F)-0.013113, (F)-0.002603,
(F)0.093882, (F)0.031359,
(F)0.138367, (F)-0.201233, (F)0.155579, (F)1.010925, (F)-0.048279, (F)-0.019145,
(F)-0.040495, (F)-0.024205, (F)-1.021970, (F)-0.002331, (F)0.027844, (F)-0.021527,
(F)0.031308, (F)-0.139878, (F)0.203431, (F)-0.157278, (F)0.006680, (F)-0.009715,
(F)0.007511, (F)0.048806,
(F)0.223083, (F)0.115906, (F)-0.205322, (F)0.383240, (F)0.763550, (F)-0.049766,
(F)-0.013434, (F)-0.042157, (F)-0.146873, (F)-0.583008, (F)-0.025857, (F)0.045804,
(F)0.023798, (F)-0.085494, (F)-0.044420, (F)0.078688, (F)-0.170335, (F)-0.088500,
(F)0.156774, (F)-0.292623,
(F)0.491455, (F)0.139465, (F)-0.377991, (F)0.313599, (F)0.366394, (F)-0.241528,
(F)-0.019451, (F)-0.142877, (F)-0.098344, (F)-0.134245, (F)-0.068541, (F)0.185765,
(F)0.052717, (F)-0.154120, (F)-0.043736, (F)0.118537, (F)-0.180066, (F)-0.051099,
(F)0.138494, (F)-0.114901,
(F)-0.023560, (F)-0.139832, (F)0.468689, (F)0.939148, (F)-0.166199, (F)-0.000555,
(F)-0.019553, (F)-0.219669, (F)-0.881999, (F)-0.027622, (F)-0.003294, (F)0.011042,
(F)0.065537, (F)0.022126, (F)0.131323, (F)-0.440168, (F)-0.003916, (F)-0.023240,
(F)0.077896, (F)0.156085,
(F)0.169495, (F)-0.071594, (F)0.243164, (F)0.798279, (F)0.153076, (F)-0.028728,
(F)-0.005126, (F)-0.059129, (F)-0.637249, (F)-0.023432, (F)0.012135, (F)-0.041215,
(F)0.017409, (F)-0.135304, (F)0.057152, (F)-0.194113, (F)-0.025946, (F)0.010959,
(F)-0.037223, (F)-0.122197,
(F)-0.114685, (F)0.108093, (F)-0.163818, (F)1.019592, (F)0.111877, (F)-0.013153,
(F)-0.011684, (F)-0.026836, (F)-1.039568, (F)-0.012517, (F)0.012397, (F)-0.018788,
(F)0.017708, (F)0.116932, (F)-0.110211, (F)0.167028, (F)0.012831, (F)-0.012093,
(F)0.018328, (F)-0.114069,
(F)0.064209, (F)0.274353, (F)0.947205, (F)0.089539, (F)-0.246094, (F)-0.004123,
(F)-0.075270, (F)-0.897197, (F)-0.008017, (F)-0.060562, (F)-0.017616, (F)-0.060819,
(F)-0.259868, (F)-0.005749, (F)-0.024565, (F)-0.084811, (F)0.801, (F)0.067517,
(F)0.233101, (F)0.022035,
(F)0.183716, (F)0.155273, (F)0.425354, (F)0.461365, (F)0.121826, (F)-0.033752,
(F)-0.024110, (F)-0.180926, (F)-0.212857, (F)-0.014842, (F)-0.028526, (F)-0.078144,
(F)-0.066046, (F)-0.084760, (F)-0.071638, (F)-0.196243, (F)-0.022381, (F)-0.018916,
(F)-0.051819, (F)-0.056206,
(F)-0.317932, (F)-0.113953, (F)0.531799, (F)0.631775, (F)0.094543, (F)-0.101081,
(F)-0.012985, (F)-0.282811, (F)-0.399140, (F)-0.008938, (F)-0.036229, (F)0.169076,
(F)0.060600, (F)0.200862, (F)0.071992, (F)-0.335977, (F)0.030058, (F)0.010773,
(F)-0.050278, (F)-0.059730,
(F)-0.263306, (F)0.519592, (F)0.055725, (F)0.460632, (F)-0.166199, (F)-0.069330,
(F)-0.269976, (F)-0.003105, (F)-0.212182, (F)-0.027622, (F)0.136812, (F)0.014673,
(F)-0.028954, (F)0.121287, (F)-0.239341, (F)-0.025669, (F)-0.043761, (F)0.086356,
(F)0.009261, (F)0.076557,
(F)0.789124, (F)-0.074402, (F)0.149414, (F)0.107239, (F)0.384705, (F)-0.622716,
(F)-0.005536, (F)-0.022325, (F)-0.011500, (F)-0.147998, (F)0.058712, (F)-0.117906,
(F)0.011117, (F)-0.084625, (F)0.007979, (F)-0.016023, (F)-0.303579, (F)0.028623,
(F)-0.057480, (F)-0.041255,
(F)0.394592, (F)0.252197, (F)0.193298, (F)0.192871, (F)0.347717, (F)-0.155703,
(F)-0.063603, (F)-0.037364, (F)-0.037199, (F)-0.120907, (F)-0.099515, (F)-0.076274,
(F)-0.048749, (F)-0.076105, (F)-0.048642, (F)-0.037282, (F)-0.137207, (F)-0.087693,
(F)-0.067213, (F)-0.067065,
(F)-0.221924, (F)0.296570, (F)1.154419, (F)-0.081482, (F)0.049438, (F)-0.049250,
(F)-0.087954, (F)-1.332683, (F)-0.006639, (F)-0.002444, (F)0.065816, (F)0.256193,
(F)-0.342366, (F)-0.018083, (F)0.024165, (F)0.094064, (F)0.010972, (F)-0.014662,
(F)-0.057073, (F)0.004028,
(F)0.539795, (F)0.479980, (F)0.039917, (F)-0.247986, (F)-0.346680, (F)-0.291379,
(F)-0.230381, (F)-0.093, (F)-0.061497, (F)-0.120187, (F)-0.259091, (F)-0.021547,
(F)-0.019159, (F)0.133861, (F)0.119028, (F)0.009899, (F)0.187136, (F)0.166399,
(F)0.013838, (F)-0.085972,
(F)0.113464, (F)0.160767, (F)0.563232, (F)0.060791, (F)0.577637, (F)-0.012874,
(F)-0.025846, (F)-0.317231, (F)-0.003696, (F)-0.333664, (F)-0.018241, (F)-0.063907,
(F)-0.090549, (F)-0.006898, (F)-0.009773, (F)-0.034239, (F)-0.065541, (F)-0.092865,
(F)-0.325344, (F)-0.035115,
(F)0.179993, (F)0.748291, (F)0.628052, (F)-0.030334, (F)-0.240662, (F)-0.032397,
(F)-0.559939, (F)-0.394449, (F)-0.000920, (F)-0.057918, (F)-0.134687, (F)-0.113045,
(F)-0.469965, (F)0.005460, (F)0.022699, (F)0.019052, (F)0.043317, (F)0.180085,
(F)0.151148, (F)-0.007300,
(F)-0.177185, (F)0.094727, (F)0.907410, (F)0.608521, (F)-0.030273, (F)-0.031395,
(F)-0.008973, (F)-0.823392, (F)-0.370297, (F)-0.000916, (F)0.016784, (F)0.160779,
(F)-0.085956, (F)0.107821, (F)-0.057643, (F)-0.552177, (F)-0.005364, (F)0.002868,
(F)0.027470, (F)0.018422,
(F)0.197937, (F)0.502563, (F)0.463013, (F)0.205872, (F)0.131287, (F)-0.039179,
(F)-0.252570, (F)-0.214381, (F)-0.042383, (F)-0.017236, (F)-0.099476, (F)-0.091647,
(F)-0.232693, (F)-0.040750, (F)-0.103464, (F)-0.095321, (F)-0.025986, (F)-0.065980,
(F)-0.060787, (F)-0.027028,
(F)-0.587830, (F)0.003662, (F)0.213684, (F)0.327087, (F)0.619507, (F)-0.345544,
(F)-0.000013, (F)-0.045661, (F)-0.106986, (F)-0.383789, (F)0.002153, (F)0.125610,
(F)-0.000783, (F)0.192272, (F)-0.001198, (F)-0.069893, (F)0.364164, (F)-0.002269,
(F)-0.132379, (F)-0.202633,
(F)-0.419556, (F)-0.155823, (F)-0.332397, (F)-0.133972, (F)-0.478577, (F)-0.176027,
(F)-0.024281, (F)-0.110488, (F)-0.017949, (F)-0.229036, (F)-0.065376, (F)-0.139459,
(F)-0.051795, (F)-0.056209, (F)-0.020876, (F)-0.044532, (F)-0.200790, (F)-0.074573,
(F)-0.159078, (F)-0.064116,
(F)-0.208313, (F)-0.069641, (F)0.282593, (F)0.830811, (F)0.481995, (F)-0.043394,
(F)-0.004850, (F)-0.079859, (F)-0.690246, (F)-0.232319, (F)-0.014507, (F)0.058868,
(F)0.019680, (F)0.173069, (F)0.057859, (F)-0.234781, (F)0.100406, (F)0.033567,
(F)-0.136208, (F)-0.400446,
(F)0.269165, (F)-0.346558, (F)0.488647, (F)-0.243652, (F)0.232605, (F)-0.072450,
(F)-0.120102, (F)-0.238776, (F)-0.059366, (F)-0.054105, (F)0.093281, (F)-0.131527,
(F)0.169344, (F)0.065583, (F)-0.084440, (F)0.119060, (F)-0.062609, (F)0.080611,
(F)-0.113662, (F)0.056675,
(F)0.345398, (F)0.704041, (F)0.134460, (F)-0.027588, (F)0.437561, (F)-0.119300,
(F)-0.495673, (F)-0.018080, (F)-0.000761, (F)-0.191460, (F)-0.243174, (F)-0.046442,
(F)-0.094666, (F)0.009529, (F)0.019423, (F)0.003709, (F)-0.151133, (F)-0.308061,
(F)-0.058835, (F)0.012071,
(F)0.177856, (F)-0.177856, (F)0.212708, (F)-0.242004, (F)0.834656, (F)-0.031633,
(F)-0.031633, (F)-0.045244, (F)-0.058566, (F)-0.696650, (F)0.031633, (F)-0.037831,
(F)0.037831, (F)0.043042, (F)-0.043042, (F)0.051476, (F)-0.148449, (F)0.148449,
(F)-0.177538, (F)0.201990,
(F)0.478699, (F)0.116394, (F)0.062378, (F)0.542114, (F)0.486572, (F)-0.229152,
(F)-0.013548, (F)-0.003891, (F)-0.293888, (F)-0.236753, (F)-0.055718, (F)-0.029860,
(F)-0.007260, (F)-0.259509, (F)-0.063099, (F)-0.033816, (F)-0.232922, (F)-0.056634,
(F)-0.030351, (F)-0.263778,
(F)0.842712, (F)0.581726, (F)0.087646, (F)0.098389, (F)0.108582, (F)-0.710164,
(F)-0.338405, (F)-0.007682, (F)-0.009680, (F)-0.011790, (F)-0.490228, (F)-0.073861,
(F)-0.050986, (F)-0.082913, (F)-0.057235, (F)-0.008623, (F)-0.091503, (F)-0.063165,
(F)-0.009517, (F)-0.010683,
(F)0.831299, (F)-0.325745, (F)0.332642, (F)-0.138123, (F)0.105164, (F)-0.691058,
(F)-0.106110, (F)-0.110650, (F)-0.019078, (F)-0.011059, (F)0.270791, (F)-0.276525,
(F)0.108356, (F)0.114821, (F)-0.044993, (F)0.045945, (F)-0.087422, (F)0.034256,
(F)-0.034982, (F)0.014525,
(F)0.323303, (F)0.511108, (F)0.541504, (F)0.469604, (F)0.399048, (F)-0.104525,
(F)-0.261232, (F)-0.293226, (F)-0.220528, (F)-0.159239, (F)-0.165243, (F)-0.175070,
(F)-0.276767, (F)-0.151825, (F)-0.240019, (F)-0.254293, (F)-0.129013, (F)-0.203957,
(F)-0.216086, (F)-0.187395 };


/*
**  AcbkGainTablePtr:
**
**  Purpose:
**      Contains pointers to 85 and 170 element codebooks
**
*/

FLOAT *AcbkGainTablePtr[2] = {
   AcbkGainTable085,
   AcbkGainTable170,
   };

/*
**  LpfConstTable:
**
**  Purpose:
**      Sets the postfilter gain weighting factor.
**
**  Table Structure:
**      (0.1875 , 0.25) in Q15 ( i.e. scale by 32768 ).

*/

FLOAT LpfConstTable[2] = {
 (F)0.1875,
   (F)0.25,
   } ;

/*
**  epsi170:
**
**  Purpose:
**      At the lower bitrate, the pitch contribution is added
**      into the output vector of ACELP when the pitch period is
**      less than 60 samples (one subframe). The pitch contribution
**      is estimated by a 1 tap long term predictor (LTP) instead of
**      the 5 tap LTP used in pitch prediction. The values in epsi170
**      effectively choose one of the 5 vectors used in the 5 tap LTP.
**
**  Table Structure:
**      The table is indexed by the index into the AcbkGainTable170
**      table (determined in Find_Acbk). The value, 60, is used only
**      as a place holder. They correspond to zeros in the gain170
**      table, which is the coefficient of the 1 tap long term
**      preditor (LTP).  The rest of the values range from -2 to +2.
**      These values pick one vector among the 5 vectors centered one
**      pitch period behind the current vector. (In contrast, the 5
**      tap LTP uses all five vectors centered one pitch period back.)
*/

int epsi170[170] = {
    60,      0,     60,      0,      1,      0,      0,      1,
    -1,      1,      0,     -1,      2,     -2,      0,      1,
     0,      0,     -1,      0,      0,      0,      1,      0,
    -1,     -1,      1,     -1,      1,     -2,      1,     -1,
    60,      0,      0,      1,      1,      0,      1,      1,
    -1,      1,     -1,     -1,     -1,      0,     60,     -2,
     0,      1,      0,     -1,      1,      1,      0,      1,
     2,     -1,      0,      1,      0,      0,      0,     60,
    -1,      1,      1,      2,     -1,      1,     -1,     60,
    -1,      0,      1,      0,     -1,      0,      0,      0,
     1,     -1,      0,      1,     -1,      0,     -1,      0,
     1,     60,      0,      0,      1,      1,      0,      2,
     1,      0,      1,     -2,      0,     -1,      1,      0,
    -1,      1,      0,      0,      1,      0,      0,      0,
    60,      1,      1,     -1,     -1,      2,     -2,      0,
    -1,     -1,     60,      2,     60,      0,      0,     60,
     0,      0,      0,     -2,     60,     -1,      0,     60,
     0,      2,      0,      0,     -1,      0,      1,      2,
    60,      1,      1,      1,      0,      0,     60,      0,
    60,      0,      0,     60,     60,     -1,      0,      0,
    60,     60,      1,      0,     60,      2,     60,     -2,
    -2,      0 };

/*
**  gain170:
**
**  Purpose:
**      At the lower bitrate, the pitch contribution is added into
**      the output vector of ACELP when the pitch period is less than
**      60 samples (one subframe). The pitch contribution
**      is estimated by a 1 tap long term predictor (LTP) instead of
**      the 5 tap LTP used in pitch prediction. This table holds the
**      coefficient for the one tap filter.
**
**  Table Structure:
**      The table is indexed by the index into the AcbkGainTable170
**      table (determined in Find_Acbk). The values are in Q12.
*/

 FLOAT gain170[170] = {
(F)0.000000,
(F)0.075958,
(F)0.000000,
(F)0.159210,
(F)0.188324,
(F)0.120636,
(F)0.316284,
(F)0.285553,
(F)0.269867,
(F)0.286743,
(F)0.176819,
(F)0.330078,
(F)0.354187,
(F)0.368408,
(F)0.262939,
(F)0.371399,
(F)0.440674,
(F)0.237640,
(F)0.418732,
(F)0.555573,
(F)0.441620,
(F)0.482391,
(F)0.466156,
(F)0.411377,
(F)0.560760,
(F)0.000000,
(F)0.341614,
(F)0.397034,
(F)0.574829,
(F)0.621155,
(F)0.495392,
(F)0.000000,
(F)0.000000,
(F)0.370178,
(F)0.408478,
(F)0.544250,
(F)0.637054,
(F)0.671265,
(F)0.000000,
(F)0.557770,
(F)0.650665,
(F)0.419250,
(F)0.582703,
(F)0.622162,
(F)0.000000,
(F)0.641113,
(F)0.000000,
(F)0.696014,
(F)0.485229,
(F)0.000000,
(F)0.548981,
(F)0.679291,
(F)0.744476,
(F)0.706116,
(F)0.730835,
(F)0.701813,
(F)0.689819,
(F)0.791687,
(F)0.800964,
(F)0.772736,
(F)0.806366,
(F)0.734283,
(F)0.831512,
(F)0.000000,
(F)0.780396,
(F)0.729919,
(F)0.791321,
(F)0.805908,
(F)0.581329,
(F)0.834869,
(F)0.748840,
(F)0.000000,
(F)0.765778,
(F)0.871246,
(F)0.000000,
(F)0.695587,
(F)0.783051,
(F)0.896515,
(F)0.923950,
(F)0.799500,
(F)0.687286,
(F)0.882538,
(F)0.835693,
(F)0.920471,
(F)0.000000,
(F)0.739655,
(F)0.917419,
(F)0.664154,
(F)0.943512,
(F)0.000000,
(F)0.954712,
(F)0.898224,
(F)0.818512,
(F)0.874481,
(F)0.956024,
(F)0.939911,
(F)0.957916,
(F)0.986389,
(F)0.969238,
(F)0.971375,
(F)0.822021,
(F)0.951538,
(F)0.942230,
(F)0.850616,
(F)0.928619,
(F)0.947571,
(F)0.000000,
(F)0.973877,
(F)0.964691,
(F)0.963287,
(F)0.779938,
(F)0.956940,
(F)0.000000,
(F)0.901367,
(F)0.884552,
(F)0.823486,
(F)0.963959,
(F)0.941040,
(F)0.950104,
(F)0.922943,
(F)0.938171,
(F)0.979309,
(F)0.000000,
(F)0.943054,
(F)0.000000,
(F)0.916351,
(F)0.886292,
(F)0.000000,
(F)0.969116,
(F)0.749298,
(F)0.782349,
(F)0.930817,
(F)0.000000,
(F)0.903839,
(F)0.935333,
(F)0.000000,
(F)0.893372,
(F)0.928192,
(F)0.838623,
(F)0.952545,
(F)0.858978,
(F)0.933838,
(F)0.965942,
(F)0.887085,
(F)0.000000,
(F)0.871918,
(F)0.876770,
(F)0.958923,
(F)0.862335,
(F)0.756744,
(F)0.000000,
(F)0.000000,
(F)0.000000,
(F)0.674591,
(F)0.849976,
(F)0.000000,
(F)0.000000,
(F)0.738953,
(F)0.752014,
(F)0.711212,
(F)0.000000,
(F)0.000000,
(F)0.698334,
(F)0.659210,
(F)0.000000,
(F)0.782776,
(F)0.000000,
(F)0.953766,
(F)0.767334,
(F)0.549255 };


/*
**  tabgain170:
**
**  Purpose:
**      In the taming procedure at the encoder the 170 5-taps LT filters
**      are modelled as 1-tap filter. This table gives the gain
**      value attributed to each filter as a worst case gain
**
**  Table Structure:
**      The table is indexed by the index into the AcbkGainTable170
**      table (determined in Find_Acbk). The values are in Q13.
*/

FLOAT tabgain170[170] = {
(F)0.125000,
(F)0.194214,
(F)0.204834,
(F)0.230835,
(F)0.258789,
(F)0.292847,
(F)0.362061,
(F)0.372192,
(F)0.388794,
(F)0.404907,
(F)0.419067,
(F)0.430054,
(F)0.455200,
(F)0.461304,
(F)0.462524,
(F)0.520264,
(F)0.543213,
(F)0.545532,
(F)0.575317,
(F)0.603516,
(F)0.604248,
(F)0.607910,
(F)0.611572,
(F)0.614258,
(F)0.646851,
(F)0.657837,
(F)0.657837,
(F)0.657837,
(F)0.689209,
(F)0.695923,
(F)0.699829,
(F)0.703735,
(F)0.732056,
(F)0.732056,
(F)0.750732,
(F)0.758179,
(F)0.773438,
(F)0.776367,
(F)0.783081,
(F)0.783081,
(F)0.784912,
(F)0.786133,
(F)0.788696,
(F)0.788696,
(F)0.794922,
(F)0.805786,
(F)0.828491,
(F)0.838867,
(F)0.846069,
(F)0.851074,
(F)0.852539,
(F)0.861328,
(F)0.861328,
(F)0.867310,
(F)0.868774,
(F)0.869507,
(F)0.871094,
(F)0.874146,
(F)0.874878,
(F)0.876465,
(F)0.876465,
(F)0.886475,
(F)0.892090,
(F)0.895264,
(F)0.895264,
(F)0.901733,
(F)0.904175,
(F)0.907471,
(F)0.908325,
(F)0.908325,
(F)0.913208,
(F)0.919922,
(F)0.923340,
(F)0.926758,
(F)0.938110,
(F)0.941650,
(F)0.947021,
(F)0.948730,
(F)0.951416,
(F)0.952393,
(F)0.954224,
(F)0.956909,
(F)0.960571,
(F)0.962524,
(F)0.965210,
(F)0.969971,
(F)0.975586,
(F)0.981323,
(F)0.984253,
(F)0.987183,
(F)0.987183,
(F)0.996948,
(F)0.999023,
(F)1.000977,
(F)1.000977,
(F)1.005981,
(F)1.009033,
(F)1.011108,
(F)1.014160,
(F)1.014160,
(F)1.017334,
(F)1.022461,
(F)1.025635,
(F)1.033081,
(F)1.039551,
(F)1.040527,
(F)1.041626,
(F)1.054810,
(F)1.058228,
(F)1.061523,
(F)1.068359,
(F)1.069458,
(F)1.074097,
(F)1.077637,
(F)1.091675,
(F)1.112305,
(F)1.113525,
(F)1.114868,
(F)1.117310,
(F)1.133789,
(F)1.135132,
(F)1.144165,
(F)1.145508,
(F)1.145508,
(F)1.148071,
(F)1.149414,
(F)1.156128,
(F)1.160034,
(F)1.166870,
(F)1.169556,
(F)1.179199,
(F)1.179199,
(F)1.186279,
(F)1.196167,
(F)1.199097,
(F)1.201904,
(F)1.201904,
(F)1.212158,
(F)1.225464,
(F)1.240723,
(F)1.248413,
(F)1.256226,
(F)1.273926,
(F)1.282104,
(F)1.283691,
(F)1.285400,
(F)1.293701,
(F)1.295288,
(F)1.296997,
(F)1.347656,
(F)1.351318,
(F)1.354980,
(F)1.356812,
(F)1.381226,
(F)1.383179,
(F)1.440918,
(F)1.477051,
(F)1.496948,
(F)1.496948,
(F)1.501465,
(F)1.505981,
(F)1.517456,
(F)1.562500,
(F)1.582275,
(F)1.594849,
(F)1.647461,
(F)1.686401,
(F)1.721191,
(F)1.733154,
(F)2.247192 };

/*
**  tabgain85:
**
**  Purpose:
**      In the taming procedure at the encoder the 85 5-taps LT filters
**      are modelled as 1-tap filter. This table gives the gain
**      value attributed to each filter as a worst case gain
**
**  Table Structure:
**      The table is indexed by the index into the AcbkGainTable085
**      table (determined in Find_Acbk). The values are in Q13.
*/


FLOAT tabgain85[85] = {
(F)0.125000,
(F)0.159668,
(F)0.232666,
(F)0.279663,
(F)0.306519,
(F)0.333984,
(F)0.402588,
(F)0.425903,
(F)0.431030,
(F)0.469238,
(F)0.532227,
(F)0.554321,
(F)0.571777,
(F)0.587524,
(F)0.618774,
(F)0.674805,
(F)0.680786,
(F)0.697388,
(F)0.723022,
(F)0.727295,
(F)0.727295,
(F)0.740234,
(F)0.748535,
(F)0.772827,
(F)0.777588,
(F)0.796753,
(F)0.797485,
(F)0.802612,
(F)0.809692,
(F)0.814331,
(F)0.833984,
(F)0.833984,
(F)0.851074,
(F)0.854004,
(F)0.878784,
(F)0.879517,
(F)0.905029,
(F)0.919067,
(F)0.920776,
(F)0.920776,
(F)0.938965,
(F)0.947021,
(F)0.956909,
(F)0.956909,
(F)0.960571,
(F)0.975586,
(F)0.976563,
(F)0.978516,
(F)0.983276,
(F)0.986206,
(F)0.991089,
(F)1.001953,
(F)1.007080,
(F)1.009033,
(F)1.012085,
(F)1.013184,
(F)1.016235,
(F)1.025635,
(F)1.030884,
(F)1.050415,
(F)1.052612,
(F)1.060425,
(F)1.068359,
(F)1.078735,
(F)1.079956,
(F)1.095337,
(F)1.098877,
(F)1.100098,
(F)1.121094,
(F)1.171021,
(F)1.180664,
(F)1.190430,
(F)1.201904,
(F)1.218018,
(F)1.237671,
(F)1.245361,
(F)1.259399,
(F)1.278809,
(F)1.293701,
(F)1.293701,
(F)1.305542,
(F)1.394653,
(F)1.424561,
(F)1.785767,
(F)1.862183 } ;

/*
**  fact   :
**  bseg   :
**  base   :
**
**  Purpose:
**      Quantization of the SID gain procedure
**      table fact :
**      for n = 0 :  used to quantize the excitation energy estimation
**                   in Comp_Info()
**                   fact(0) = 1 / (2 * SubFrLen)
**
**      for n >= 1 : used for average energy calculation
**                   fact(n) = fact_mul**2 * 32768 / n * Frame
**      table bseg : for each segment of the pseudo-log quantizer,
**                     gives the energy level corresponding to
**                     the first value of the next segment
**      table base : first code of each segment
**
*/
FLOAT   fact[4] = {(F)0.008333, (F)61.6815, (F)30.84075, (F)20.6145};
FLOAT   bseg[3] = {(F)1024., (F)9216., (F)115617.};
FLOAT   base[3] = {(F)0.0, (F)32.0, (F)96.0};
